@@ -98,6 +98,18 @@ check_tool Automake "$AUTOMAKE" "$ACLOCAL" 1 13 ftp://ftp.gnu.org/pub/gnu/automa
 check_tool Libtool "$LIBTOOLIZE" "" 2 4 ftp://ftp.gnu.org/pub/gnu/libtool/
 check_tool Gettext "$GETTEXT" "$GETTEXTIZE" 0 12 ftp://ftp.gnu.org/pub/gnu/gettext/
 
+if test -n "$ACLOCAL"; then
+  ac_dir="$($ACLOCAL $ACLOCAL_FLAGS --print-ac-dir)"
+  if test -s "$ac_dir/introspection.m4"; then
+    echo "GObject-introspection-devel: OK"
+  else
+    echo "ERROR: Cannot find introspection.m4 in $ac_dir."
+    echo "       Install gobject-introspection, including development files."
+    DIE=1
+  fi
+else
+  echo "GObject-introspection-devel: who knows"
+fi
 if test "$DIE" = 1; then
   exit 1
 fi
@@ -108,7 +120,7 @@ case $CC in
 esac
 
 # We don't want gettextize to mess with our files, but we want its config.rpath
-if test -f config.rpath; then
+if test -f build-aux/config.rpath; then
   # Nothing to do
   :
 else
@@ -119,7 +131,8 @@ else
   if test ! -n "$gettext_datadir"; then
     gettext_datadir="$gettext_dir"
   fi
-  if cp "$gettext_datadir/config.rpath" config.rpath; then
+  echo "autogen.sh: installing './config.rpath'"
+  if rm -f build-aux/config.rpath && ln -s "$gettext_datadir/config.rpath" build-aux/; then
     # OK
     :
   else
