@@ -31,6 +31,7 @@ def add_std(tostd, ucname):
     tostd.add('GWY_' + ucname + '_CLASS')
     tostd.add('GWY_' + ucname + '_GET_CLASS')
     tostd.add('GWY_TYPE_' + ucname)
+    tostd.add('Gwy' + ucname)
 
 sections = []
 section = None
@@ -55,6 +56,9 @@ for line in lines:
                 m = re.search(r'^gwy_\w+_get_type$', symbol)
                 if m:
                     tostd.add(symbol)
+                m = re.search(r'^Gwy(\w+)Private$', symbol)
+                if m:
+                    tostd.add(symbol)
 
             if debug:
                 sys.stderr.write(esection['FILE'] + ' ' + tostd + '\n')
