@@ -1,6 +1,6 @@
 /*
- *  $Id: about.c 27036 2024-12-11 15:03:03Z yeti-dn $
- *  Copyright (C) 2003-2022 David Necas (Yeti), Petr Klapetek.
+ *  $Id: about.c 29101 2026-01-07 18:01:38Z yeti-dn $
+ *  Copyright (C) 2003-2026 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -17,21 +17,27 @@
 
 #include "config.h"
 #include <string.h>
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#ifdef HAVE_GTKGLEXT
-#include <gtk/gtkgl.h>
-#endif
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwyversion.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymoduleloader.h>
-#include <app/app.h>
-#include "gwyddion.h"
-#include "authors.h"
-#include "release.h"
+
+#include "libgwyddion/macros.h"
+#include "libgwyddion/version.h"
+#include "libgwyui/icons.h"
+#include "libgwyapp/module-loader.h"
+#include "libgwyapp/app.h"
+#include "libgwyapp/sanity.h"
+
+#include "gwyddion/gwyddion.h"
+#include "gwyddion/authors.h"
+#include "gwyddion/release.h"
+#include "gwyfeatures.h"
 
 #define NA _("not available\n")
 
+enum {
+    GWY_ICON_SIZE_ABOUT = 60
+};
+
 static void           about_close              (void);
 static void           fill_credits             (GtkTextBuffer *buffer);
 static void           fill_features            (GtkTextBuffer *buffer);
@@ -49,7 +55,6 @@ gwy_app_about(void)
     GtkTextIter iter;
     GString *str;
     const gchar *verextra = "";
-    gint size;
 
     if (about) {
         gtk_window_present(GTK_WINDOW(about));
@@ -58,39 +63,38 @@ gwy_app_about(void)
     str = g_string_new(NULL);
     g_string_printf(str, _("About %s"), g_get_application_name());
     about = gtk_dialog_new_with_buttons(str->str, GTK_WINDOW(gwy_app_main_window_get()),
-                                        GTK_DIALOG_NO_SEPARATOR | GTK_DIALOG_DESTROY_WITH_PARENT,
-                                        GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
-                                        NULL);
+                                        GTK_DIALOG_DESTROY_WITH_PARENT, NULL, NULL);
+    gwy_add_close_button_to_dialog(GTK_DIALOG(about));
     gtk_dialog_set_default_response(GTK_DIALOG(about), GTK_RESPONSE_CLOSE);
     gtk_container_set_border_width(GTK_CONTAINER(about), 6);
     gtk_window_set_transient_for(GTK_WINDOW(about), GTK_WINDOW(gwy_app_main_window_get()));
     gtk_window_set_position(GTK_WINDOW(about), GTK_WIN_POS_CENTER);
     gwy_app_add_main_accel_group(GTK_WINDOW(about));
 
-    vbox = GTK_DIALOG(about)->vbox;
+    vbox = gtk_dialog_get_content_area(GTK_DIALOG(about));
     gtk_box_set_spacing(GTK_BOX(vbox), 8);
 
-    hbox = gtk_hbox_new(FALSE, 6);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 6);
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 
-    size = gtk_icon_size_from_name(GWY_ICON_SIZE_ABOUT);
-    widget = gtk_image_new_from_stock(GWY_STOCK_GWYDDION, size);
+    widget = gtk_image_new_from_icon_name(GWY_ICON_GWYDDION, GTK_ICON_SIZE_DIALOG);
+    gtk_image_set_pixel_size(GTK_IMAGE(widget), GWY_ICON_SIZE_ABOUT);
 
     gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 0);
-    gtk_misc_set_alignment(GTK_MISC(widget), 0.5, 0.0);
+    gtk_widget_set_valign(widget, GTK_ALIGN_START);
 
-    vbox = gtk_vbox_new(FALSE, 0);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
     gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);
 
     widget = gtk_label_new(NULL);
-    gtk_misc_set_alignment(GTK_MISC(widget), 0.0, 0.5);
-    gtk_misc_set_padding(GTK_MISC(widget), 2, 4);
+    gtk_label_set_xalign(GTK_LABEL(widget), 0.0);
+    gwy_set_widget_padding(widget, 2, 2, 4, 4);
     gtk_box_pack_start(GTK_BOX(vbox), widget, FALSE, FALSE, 0);
-    /* If we have unset release date but simple (non-date-extended) version
-     * string then we are not compiling from a public tarball. */
+    /* If we have unset release date but simple (non-date-extended) version string then we are not compiling from
+     * a public tarball. */
     if (RELEASEDATE == 0 && strlen(GWY_VERSION_STRING) < 9)
         verextra = "+SVN";
-    g_string_printf(str, "<span size='x-large' weight='bold'>%s %s%s</span>\n",
+    g_string_printf(str, "<span size='x-large' weight='bold'>%s %s%s (unstable branch)</span>\n",
                     g_get_application_name(), GWY_VERSION_STRING, verextra);
     construct_datetime_info(str);
     g_string_append(str, _("An SPM data visualization and analysis tool."));
@@ -100,8 +104,8 @@ gwy_app_about(void)
     gtk_box_pack_start(GTK_BOX(vbox), widget, FALSE, FALSE, 0);
 
     widget = gtk_label_new(NULL);
-    gtk_misc_set_alignment(GTK_MISC(widget), 0.0, 0.5);
-    gtk_misc_set_padding(GTK_MISC(widget), 2, 4);
+    gtk_label_set_xalign(GTK_LABEL(widget), 0.0);
+    gwy_set_widget_padding(widget, 2, 2, 4, 4);
     gtk_box_pack_start(GTK_BOX(vbox), widget, FALSE, FALSE, 0);
     g_string_printf(str, "%s <i>%s</i>", _("Report bugs to:"), PACKAGE_BUGREPORT);
     gtk_label_set_markup(GTK_LABEL(widget), str->str);
@@ -109,7 +113,7 @@ gwy_app_about(void)
 
     notebook = gtk_notebook_new();
     gtk_notebook_set_show_border(GTK_NOTEBOOK(notebook), FALSE);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(about)->vbox), notebook, TRUE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(about))), notebook, TRUE, TRUE, 0);
 
     /* Credits */
     buff = make_buffer_with_tags();
@@ -174,7 +178,10 @@ add_credits_block(GtkTextBuffer *buffer,
         gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, body, -1, "i", NULL);
     else
         gtk_text_buffer_insert(buffer, &iter, body, -1);
-    gtk_text_buffer_insert(buffer, &iter, "\n", 1);
+    if (g_str_has_suffix(body, "\n"))
+        gtk_text_buffer_insert(buffer, &iter, "\n", 1);
+    else
+        gtk_text_buffer_insert(buffer, &iter, "\n\n", 2);
 }
 
 static void
@@ -218,10 +225,19 @@ make_scrolled_text_window(GtkTextBuffer *buffer)
     return scwin;
 }
 
+static void
+add_file_format_block(GtkTextBuffer *buffer, const gchar *what, gboolean enabled)
+{
+    gchar *s = g_strdup_printf(_("%s file formats"), what);
+    add_credits_block(buffer, s, enabled ? _("enabled") : _("disabled"), !enabled);
+    g_free(s);
+}
+
 static void
 fill_features(GtkTextBuffer *buffer)
 {
     G_GNUC_UNUSED static const GwyEnum openmp_vers[] = {
+        { "6.0", 202411 },
         { "5.2", 202111 },
         { "5.1", 202011 },
         { "5.0", 201811 },
@@ -239,24 +255,6 @@ fill_features(GtkTextBuffer *buffer)
     const GwyModuleInfo *modinfo;
     G_GNUC_UNUSED gchar *s;
     G_GNUC_UNUSED const gchar *cs;
-    gboolean b;
-
-
-#ifdef HAVE_GTKGLEXT
-    /* XXX: Variables from DLLs work strangely in Win32.  We probably get uninitialised memory when reading them.
-     * Show the compiled in version, not run-time, so we show at least something sane... */
-    s = g_strdup_printf("GtkGLExt %d.%d.%d\n",
-#ifdef G_OS_WIN32
-                        GTKGLEXT_MAJOR_VERSION, GTKGLEXT_MINOR_VERSION, GTKGLEXT_MICRO_VERSION
-#else
-                        gtkglext_major_version, gtkglext_minor_version, gtkglext_micro_version
-#endif
-                       );
-    add_credits_block(buffer, _("OpenGL 3D View"), s, FALSE);
-    g_free(s);
-#else
-    add_credits_block(buffer, _("OpenGL 3D View"), NA, TRUE);
-#endif
 
 #ifdef _OPENMP
     cs = gwy_enum_to_string(_OPENMP, openmp_vers, G_N_ELEMENTS(openmp_vers));
@@ -270,20 +268,25 @@ fill_features(GtkTextBuffer *buffer)
     add_credits_block(buffer, _("OpenMP parallelization"), NA, TRUE);
 #endif
 
-    b = FALSE;
-#if (REMOTE_BACKEND == REMOTE_NONE)
-    cs = NA;
-    b = TRUE;
-#elif (REMOTE_BACKEND == REMOTE_X11)
-    cs = _("X11 protocol\n");
-#elif (REMOTE_BACKEND == REMOTE_WIN32)
-    cs = _("Win32 protocol\n");
-#elif (REMOTE_BACKEND == REMOTE_UNIQUE)
-    cs = _("LibUnique\n");
-#else
-#error "An unknown remote control backend."
-#endif
-    add_credits_block(buffer, _("Remote Control"), cs, b);
+    add_credits_block(buffer, _("16bit image formats"),
+                      GWY_APP_FEATURE_have_cxx ? _("enabled") : _("disabled"),
+                      !GWY_APP_FEATURE_have_cxx);
+
+    add_file_format_block(buffer, "Bzip2", GWY_APP_FEATURE_enable_bzip2);
+    add_file_format_block(buffer, "CFITSIO", GWY_APP_FEATURE_enable_cfitsio);
+    add_file_format_block(buffer, "HDF5", GWY_APP_FEATURE_enable_hdf5);
+    add_file_format_block(buffer, "JSON-glib", GWY_APP_FEATURE_enable_json_glib);
+    add_file_format_block(buffer, "LibXML2", GWY_APP_FEATURE_enable_libxml2);
+    add_file_format_block(buffer, "OpenEXR", GWY_APP_FEATURE_enable_exr);
+    add_file_format_block(buffer, "WebP", GWY_APP_FEATURE_enable_webp);
+
+    if (GWY_APP_FEATURE_found_zip) {
+        s = g_strdup_printf(_("%s file formats (%s)"), "ZIP", GWY_APP_FEATURE_zip_library);
+        add_credits_block(buffer, s, _("enabled"), FALSE);
+        g_free(s);
+    }
+    else
+        add_file_format_block(buffer, "ZIP", FALSE);
 
     if ((modinfo = gwy_module_lookup("pygwy"))) {
         s = g_strdup_printf("pygwy %s\n", modinfo->version);
@@ -292,6 +295,14 @@ fill_features(GtkTextBuffer *buffer)
     }
     else
         add_credits_block(buffer, _("Python Scripting Interface"), NA, TRUE);
+
+    s = g_strconcat(GWY_APP_FEATURE_enable_introspection ? "+" : "-", "introspection", " ",
+                    GWY_APP_FEATURE_enable_module_bundling ? "+" : "-", "module-bundling", " ",
+                    GWY_APP_FEATURE_enable_library_bloat ? "+" : "-", "library-bloat",
+                    NULL);
+
+    add_credits_block(buffer, _("Build settings"), s, FALSE);
+
 }
 
 /* The __DATE__ macro is pretty well specified to be a silly American date. Unlike date & time parsing functions,
