@@ -1,4 +1,7 @@
+#pragma once
 /* This is a GENERATED file */
+#ifndef __GWYDDION_AUTHORS_H__
+#define __GWYDDION_AUTHORS_H__
 static const gchar developers[] =
   "David Nečas (Yeti)\n"
   "Petr Klapetek\n"
@@ -58,3 +61,4 @@ static const gchar translators[] =
   "Sabrina Meindlhumer\n"
   "Ikuo Obataya\n"
 ;
+#endif
