@@ -1,5 +1,5 @@
 /*
- *  $Id: gwyddion.h 22302 2019-07-21 09:06:45Z yeti-dn $
+ *  $Id: gwyddion.h 27908 2025-05-05 14:26:24Z yeti-dn $
  *  Copyright (C) 2003-2019 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -24,43 +24,15 @@
 
 #include <gtk/gtkwidget.h>
 #include <libgwyddion/gwycontainer.h>
-#include <libgwyddion/gwyenum.h>
+#include <libgwyddion/enum.h>
 
 G_BEGIN_DECLS
 
 #define GWY_TOOLBOX_WM_ROLE "gwyddion-toolbox"
 
-#define REMOTE_NONE   0
-#define REMOTE_X11    1
-#define REMOTE_WIN32  2
-#define REMOTE_UNIQUE 3
-
-typedef struct _GwyRemote GwyRemote;
-
-typedef enum {
-    GWY_APP_REMOTE_DEFAULT,
-    GWY_APP_REMOTE_NONE,
-    GWY_APP_REMOTE_NEW,
-    GWY_APP_REMOTE_EXISTING,
-    GWY_APP_REMOTE_QUERY
-} GwyAppRemoteType;
-
-void       gwy_remote_setup             (GtkWidget *toolbox);
-void       gwy_remote_finalize          (GtkWidget *toolbox);
-void       gwy_remote_do                (GwyAppRemoteType type,
-                                         int argc,
-                                         char **argv);
-GwyRemote* gwy_remote_get               (void);
-void       gwy_remote_free              (GwyRemote *remote);
-gboolean   gwy_remote_open_files        (GwyRemote *remote,
-                                         int argc,
-                                         char **argv);
-void       gwy_remote_print             (GwyRemote *remote);
-
 GtkWidget* gwy_app_toolbox_window_create    (void);
 GtkWidget* gwy_app_show_data_browser        (void);
 void       gwy_app_about                    (void);
-void       gwy_app_tip_of_the_day           (void);
 void       gwy_app_splash_start             (gboolean visible);
 void       gwy_app_splash_finish            (void);
 void       gwy_app_splash_set_message       (const gchar *message);
