@@ -1,5 +1,5 @@
 /*
- *  $Id: splash.c 22154 2019-06-09 16:11:39Z yeti-dn $
+ *  $Id: splash.c 27936 2025-05-09 09:14:17Z yeti-dn $
  *  Copyright (C) 2004-2016 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -22,11 +22,14 @@
 #include "config.h"
 #include <string.h>
 #include <stdlib.h>
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwyddion/gwyversion.h>
-#include <libgwyddion/gwycontainer.h>
+
+#include "libgwyddion/macros.h"
+#include "libgwyddion/utils.h"
+#include "libgwyddion/version.h"
+#include "libgwyddion/gwycontainer.h"
+#include "libgwyapp/sanity.h"
 
 typedef struct {
     gchar c;
@@ -39,7 +42,6 @@ static void splash_map(void);
 static gboolean in_splash = FALSE;
 static GtkWidget *window = NULL;
 static GtkWidget *label = NULL;
-gchar *message_prefix = NULL;
 
 /* {{{ Digit images */
 static const guchar digit_0_png[] = {
@@ -1169,11 +1171,9 @@ compose_splash(const gchar *version, const gchar *extraversion)
     GdkPixbuf *base;
     GError *err = NULL;
     guint i;
-    gchar *p, *filename;
+    gchar *filename;
 
-    p = gwy_find_self_dir("pixmaps");
-    filename = g_build_filename(p, "splash.png", NULL);
-    g_free(p);
+    filename = gwy_find_self_path("data", "pixmaps", "splash.png", NULL);
 
     base = gdk_pixbuf_new_from_file(filename, &err);
     if (!base) {
@@ -1259,8 +1259,6 @@ gwy_app_splash_start(gboolean visible)
     window = gtk_window_new(GTK_WINDOW_POPUP);
     p = g_strconcat(_("Starting "), g_get_application_name(), NULL);
     gtk_window_set_title(GTK_WINDOW(window), p);
-    gtk_window_set_wmclass(GTK_WINDOW(window), "splash",
-                           g_get_application_name());
     gtk_window_set_resizable(GTK_WINDOW(window), FALSE);
     gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
     gtk_window_set_decorated(GTK_WINDOW(window), FALSE);
@@ -1277,7 +1275,7 @@ gwy_app_splash_start(gboolean visible)
     gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_ETCHED_OUT);
     gtk_container_add(GTK_CONTAINER(window), frame);
 
-    vbox = gtk_vbox_new(FALSE, 0);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
     gtk_container_add(GTK_CONTAINER(frame), vbox);
     gtk_container_set_border_width(GTK_CONTAINER(vbox), 2);
 
@@ -1296,17 +1294,16 @@ gwy_app_splash_start(gboolean visible)
         g_object_unref(pixbuf);
     }
     else
-        image = gtk_image_new_from_stock(GTK_STOCK_MISSING_IMAGE,
-                                         GTK_ICON_SIZE_DIALOG);
+        image = gtk_image_new_from_icon_name("missing-image", GTK_ICON_SIZE_DIALOG);
     gtk_box_pack_start(GTK_BOX(vbox), image, FALSE, FALSE, 0);
 
     label = gtk_label_new(p);
-    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
-    gtk_misc_set_padding(GTK_MISC(label), 6, 4);
+    gtk_label_set_xalign(GTK_LABEL(label), 0.0);
+    gwy_set_widget_padding(label, 6, 6, 4, 4);
     gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
     g_free(p);
 
-    lab = gtk_hseparator_new();
+    lab = gtk_separator_new(GTK_ORIENTATION_HORIZONTAL);
     gtk_box_pack_start(GTK_BOX(vbox), lab, FALSE, FALSE, 0);
 
     p = g_strconcat("<small>", PACKAGE_NAME,
@@ -1314,8 +1311,8 @@ gwy_app_splash_start(gboolean visible)
                     NULL);
     lab = gtk_label_new(NULL);
     gtk_label_set_markup(GTK_LABEL(lab), p);
-    gtk_misc_set_alignment(GTK_MISC(lab), 0.0, 0.5);
-    gtk_misc_set_padding(GTK_MISC(lab), 5, 3);
+    gtk_label_set_xalign(GTK_LABEL(lab), 0.0);
+    gwy_set_widget_padding(lab, 5, 5, 3, 3);
     gtk_box_pack_start(GTK_BOX(vbox), lab, FALSE, FALSE, 0);
     g_free(p);
 
@@ -1333,25 +1330,9 @@ gwy_app_splash_finish(void)
         return;
 
     gtk_widget_destroy(window);
-    g_free(message_prefix);
 
     window = NULL;
     label = NULL;
-    message_prefix = NULL;
-
-    while (gtk_events_pending())
-        gtk_main_iteration();
-}
-
-void
-gwy_app_splash_set_message_prefix(const gchar *prefix)
-{
-    g_return_if_fail(in_splash);
-    if (!window)
-        return;
-
-    g_free(message_prefix);
-    message_prefix = g_strdup(prefix);
 
     while (gtk_events_pending())
         gtk_main_iteration();
@@ -1367,13 +1348,7 @@ gwy_app_splash_set_message(const gchar *message)
     while (gtk_events_pending())
         gtk_main_iteration();
 
-    if (message_prefix) {
-        gchar *s = g_strconcat(message_prefix, message, NULL);
-        gtk_label_set_markup(GTK_LABEL(label), s);
-        g_free(s);
-    }
-    else
-        gtk_label_set_markup(GTK_LABEL(label), message);
+    gtk_label_set_markup(GTK_LABEL(label), message);
 
     while (gtk_events_pending())
         gtk_main_iteration();
