@@ -1,5 +1,5 @@
 /*
- *  $Id: toolbox-spec.c 24455 2021-11-03 13:29:04Z yeti-dn $
+ *  $Id: toolbox-spec.c 28772 2025-11-03 18:02:12Z yeti-dn $
  *  Copyright (C) 2016-2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -21,13 +21,13 @@
 #include <stdio.h>
 #include <errno.h>
 #include <glib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwymodule/gwymodule.h>
-#include <app/gwyapp.h>
-#include <app/gwyapp.h>
+#include <glib/gi18n.h>
 
-#include "gwyddion.h"
-#include "toolbox.h"
+#include "libgwyddion/macros.h"
+#include "libgwyapp/gwyapp.h"
+
+#include "gwyddion/gwyddion.h"
+#include "gwyddion/toolbox.h"
 
 #define TOOLBOX_UI_FILE_NAME "toolbox.xml"
 #define TOOLBOX_INDENT "  "
@@ -107,7 +107,7 @@ toolbox_ui_start_element(G_GNUC_UNUSED GMarkupParseContext *context,
         for (i = 0; (attname = attribute_names[i]); i++) {
             attval = attribute_values[i];
             if (gwy_strequal(attname, "id")) {
-                if (gwy_strisident(attval, NULL, NULL))
+                if (gwy_ascii_strisident(attval, NULL, NULL))
                     id = attval;
                 else
                     g_warning("Ignoring non-identifier id=\"%s\".", attval);
@@ -152,7 +152,7 @@ toolbox_ui_start_element(G_GNUC_UNUSED GMarkupParseContext *context,
         GwyToolboxGroupSpec *gspec;
         const gchar *function = NULL, *icon = NULL;
         GwyAppActionType tmptype, type = GWY_APP_ACTION_TYPE_NONE;
-        GwyRunType tmpmode, mode = 0;
+        GwyRunModeFlags tmpmode, mode = 0;
         GwyToolboxItemSpec ispec;
 
         if (!gwy_strequal(path->str, "/toolbox/group/item") || !group->len) {
@@ -166,7 +166,7 @@ toolbox_ui_start_element(G_GNUC_UNUSED GMarkupParseContext *context,
             attval = attribute_values[i];
             if (gwy_strequal(attname, "type") && (tmptype = gwy_toolbox_find_action_type(attval)) != -1)
                 type = tmptype;
-            else if (gwy_strequal(attname, "run") && (tmpmode = gwy_toolbox_find_mode(attval)) != (GwyRunType)-1)
+            else if (gwy_strequal(attname, "run") && (tmpmode = gwy_toolbox_find_mode(attval)) != (GwyRunModeFlags)-1)
                 mode = tmpmode;
             else if (gwy_strequal(attname, "function"))
                 function = attval;
@@ -189,7 +189,7 @@ toolbox_ui_start_element(G_GNUC_UNUSED GMarkupParseContext *context,
             }
             spec->seen_tool_placeholder = TRUE;
         }
-        if (type == GWY_APP_ACTION_TYPE_TOOL && function && !gwy_strisident(function, "_-", NULL)) {
+        if (type == GWY_APP_ACTION_TYPE_TOOL && function && !gwy_ascii_strisident(function, "_-", NULL)) {
             g_warning("Ignoring tool item with invalid function=\"%s\"", function);
             return;
         }
@@ -423,15 +423,13 @@ gwy_parse_toolbox_ui(gboolean ignore_user)
 {
     GwyToolboxSpec *spec;
     GError *error = NULL;
-    gchar *p, *q, *ui;
+    gchar *p, *ui;
     gsize ui_len;
 
     p = g_build_filename(gwy_get_user_dir(), "ui", TOOLBOX_UI_FILE_NAME, NULL);
     if (ignore_user || !g_file_get_contents(p, &ui, &ui_len, NULL)) {
         g_free(p);
-        q = gwy_find_self_dir("data");
-        p = g_build_filename(q, "ui", TOOLBOX_UI_FILE_NAME, NULL);
-        g_free(q);
+        p = gwy_find_self_path("data", "ui", TOOLBOX_UI_FILE_NAME, NULL);
         if (!g_file_get_contents(p, &ui, &ui_len, NULL)) {
             g_critical("Cannot find toolbox user interface %s", p);
             return NULL;
@@ -569,12 +567,12 @@ gwy_toolbox_find_action_type(const gchar *name)
 }
 
 const gchar*
-gwy_toolbox_mode_name(GwyRunType mode)
+gwy_toolbox_mode_name(GwyRunModeFlags mode)
 {
     return gwy_enum_to_string(mode, modes, G_N_ELEMENTS(modes));
 }
 
-GwyRunType
+GwyRunModeFlags
 gwy_toolbox_find_mode(const gchar *name)
 {
     return gwy_string_to_enum(name, modes, G_N_ELEMENTS(modes));
@@ -594,7 +592,7 @@ gwy_toolbox_action_nice_name(GwyAppActionType type, const gchar *name)
         if (name) {
             GType gtype = g_type_from_name(name);
             GwyToolClass *tool_class = g_type_class_peek(gtype);
-            return gettext(gwy_tool_class_get_title(tool_class));
+            return _(gwy_tool_class_get_title(tool_class));
         }
         return _("remaining tools");
     }
@@ -609,6 +607,8 @@ gwy_toolbox_action_nice_name(GwyAppActionType type, const gchar *name)
         menupath = gwy_process_func_get_menu_path(name);
     else if (type == GWY_APP_ACTION_TYPE_GRAPH)
         menupath = gwy_graph_func_get_menu_path(name);
+    else if (type == GWY_APP_ACTION_TYPE_SYNTH)
+        menupath = gwy_synth_func_get_menu_path(name);
     else if (type == GWY_APP_ACTION_TYPE_VOLUME)
         menupath = gwy_volume_func_get_menu_path(name);
     else if (type == GWY_APP_ACTION_TYPE_XYZ)
@@ -618,7 +618,7 @@ gwy_toolbox_action_nice_name(GwyAppActionType type, const gchar *name)
 
     if (menupath) {
         const gchar *p;
-        gchar *s = g_strdup(gettext(menupath));
+        gchar *s = g_strdup(_(menupath));
 
         if (!label)
             label = g_string_new(NULL);
@@ -635,30 +635,32 @@ gwy_toolbox_action_nice_name(GwyAppActionType type, const gchar *name)
 }
 
 const gchar*
-gwy_toolbox_action_stock_id(GwyAppActionType type, const gchar *name)
+gwy_toolbox_action_icon_name(GwyAppActionType type, const gchar *name)
 {
     if (!name)
         return NULL;
     if (type == GWY_APP_ACTION_TYPE_PROC)
-        return gwy_process_func_get_stock_id(name);
+        return gwy_process_func_get_icon_name(name);
     if (type == GWY_APP_ACTION_TYPE_GRAPH)
-        return gwy_graph_func_get_stock_id(name);
+        return gwy_graph_func_get_icon_name(name);
+    if (type == GWY_APP_ACTION_TYPE_SYNTH)
+        return gwy_synth_func_get_icon_name(name);
     if (type == GWY_APP_ACTION_TYPE_VOLUME)
-        return gwy_volume_func_get_stock_id(name);
+        return gwy_volume_func_get_icon_name(name);
     if (type == GWY_APP_ACTION_TYPE_XYZ)
-        return gwy_xyz_func_get_stock_id(name);
+        return gwy_xyz_func_get_icon_name(name);
     if (type == GWY_APP_ACTION_TYPE_CMAP)
-        return gwy_curve_map_func_get_stock_id(name);
+        return gwy_curve_map_func_get_icon_name(name);
     if (type == GWY_APP_ACTION_TYPE_TOOL) {
         GType gtype = g_type_from_name(name);
         GwyToolClass *tool_class = g_type_class_peek(gtype);
-        return gwy_tool_class_get_stock_id(tool_class);
+        return gwy_tool_class_get_icon_name(tool_class);
     }
     if (type == GWY_APP_ACTION_TYPE_BUILTIN) {
         const GwyToolboxBuiltinSpec* spec;
 
         if ((spec = gwy_toolbox_find_builtin_spec(name)))
-            return spec->stock_id;
+            return spec->icon_name;
     }
     return NULL;
 }
@@ -672,6 +674,8 @@ gwy_toolbox_action_detail(GwyAppActionType type, const gchar *name)
         return gwy_process_func_get_tooltip(name);
     if (type == GWY_APP_ACTION_TYPE_GRAPH)
         return gwy_graph_func_get_tooltip(name);
+    if (type == GWY_APP_ACTION_TYPE_SYNTH)
+        return gwy_synth_func_get_tooltip(name);
     if (type == GWY_APP_ACTION_TYPE_VOLUME)
         return gwy_volume_func_get_tooltip(name);
     if (type == GWY_APP_ACTION_TYPE_XYZ)
@@ -692,13 +696,15 @@ gwy_toolbox_action_detail(GwyAppActionType type, const gchar *name)
     return NULL;
 }
 
-GwyRunType
+GwyRunModeFlags
 gwy_toolbox_action_run_modes(GwyAppActionType type, const gchar *name)
 {
     if (!name)
         return 0;
     if (type == GWY_APP_ACTION_TYPE_PROC)
         return gwy_process_func_get_run_types(name);
+    if (type == GWY_APP_ACTION_TYPE_SYNTH)
+        return gwy_synth_func_get_run_modes(name);
     if (type == GWY_APP_ACTION_TYPE_VOLUME)
         return gwy_volume_func_get_run_types(name);
     if (type == GWY_APP_ACTION_TYPE_XYZ)
