@@ -1,58 +1,56 @@
 /*
- *  $Id: gwyfdcurvepreset.c 26535 2024-08-16 13:53:38Z yeti-dn $
+ *  $Id: gwyfdcurvepreset.c 28903 2025-11-24 15:49:51Z yeti-dn $
  *  Copyright (C) 2000-2003 Martin Siler.
  *  Copyright (C) 2004 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
+ *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
+ *  later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwydebugobjects.h>
-#include <libgwyddion/gwyfdcurvepreset.h>
-#include "gwyddioninternal.h"
+#include "config.h"
+#include <glib/gi18n-lib.h>
 
-static GwyFDCurvePreset*
-gwy_fd_curve_preset_new_static(const GwyNLFitPresetBuiltin *data);
+#include "libgwyddion/math.h"
+#include "libgwyddion/gwyfdcurvepreset.h"
+
+#include "libgwyddion/internal.h"
+
+static GwyFDCurvePreset* gwy_fd_curve_preset_new_static(const GwyNLFitPresetBuiltin *data);
+static void              resource_setup_builtins       (GwyInventory *inventory);
 
 G_DEFINE_TYPE(GwyFDCurvePreset, gwy_fd_curve_preset, GWY_TYPE_NLFIT_PRESET)
 
-static GwySIUnit*
-select_unit(GwySIUnit *siunit_x, guint param, const gchar **units, guint nparams)
+static GwyUnit*
+select_unit(GwyUnit *siunit_x, guint param, const gchar **units, guint nparams)
 {
     if (param == 0)
-        return gwy_si_unit_duplicate(siunit_x);
+        return gwy_unit_copy(siunit_x);
     if (param < nparams)
-        return gwy_si_unit_new(units[param]);
+        return gwy_unit_new(units[param]);
 
     g_assert_not_reached();
-    return gwy_si_unit_new(NULL);
+    return gwy_unit_new(NULL);
 }
 
 /******************* sszanette ********************************/
 static gdouble
 sszanette_func(gdouble x,
-               G_GNUC_UNUSED gint n_param,
                const gdouble *b,
                G_GNUC_UNUSED gpointer user_data,
                gboolean *fres)
 {
     /*xc, R, H */
     *fres = TRUE;
-    return b[1] -b[3]/6*(b[2]*b[2]*b[2] * (b[2]+2*(x-b[0])))/((x-b[0])*(x-b[0]) * pow(((x-b[0]) + b[2]), 3));
+    return b[1] - b[3]/6*(b[2]*b[2]*b[2] * (b[2]+2*(x-b[0])))/((x-b[0])*(x-b[0]) * pow(((x-b[0]) + b[2]), 3));
 }
 
 static void
@@ -83,7 +81,6 @@ sszanette_guess(gint n_dat,
 /******************* pyrzanette ********************************/
 static gdouble
 pyrzanette_func(gdouble x,
-                G_GNUC_UNUSED gint n_param,
                 const gdouble *b,
                 G_GNUC_UNUSED gpointer user_data,
                 gboolean *fres)
@@ -122,7 +119,6 @@ pyrzanette_guess(gint n_dat,
 /******************* tpyrzanette ********************************/
 static gdouble
 tpyrzanette_func(gdouble x,
-                 G_GNUC_UNUSED gint n_param,
                  const gdouble *b,
                  G_GNUC_UNUSED gpointer user_data,
                  gboolean *fres)
@@ -164,7 +160,6 @@ tpyrzanette_guess(gint n_dat,
 /******************* sphcapella ********************************/
 static gdouble
 sphcapella_func(gdouble x,
-                G_GNUC_UNUSED gint n_param,
                 const gdouble *b,
                 G_GNUC_UNUSED gpointer user_data,
                 gboolean *fres)
@@ -203,7 +198,6 @@ sphcapella_guess(gint n_dat,
 /******************* sphsphcapella ********************************/
 static gdouble
 sphsphcapella_func(gdouble x,
-                   G_GNUC_UNUSED gint n_param,
                    const gdouble *b,
                    G_GNUC_UNUSED gpointer user_data,
                    gboolean *fres)
@@ -243,7 +237,6 @@ sphsphcapella_guess(gint n_dat,
 /******************* conecapella ********************************/
 static gdouble
 conecapella_func(gdouble x,
-                 G_GNUC_UNUSED gint n_param,
                  const gdouble *b,
                  G_GNUC_UNUSED gpointer user_data,
                  gboolean *fres)
@@ -282,7 +275,6 @@ conecapella_guess(gint n_dat,
 /******************* cylindercapella ********************************/
 static gdouble
 cylindercapella_func(gdouble x,
-                     G_GNUC_UNUSED gint n_param,
                      const gdouble *b,
                      G_GNUC_UNUSED gpointer user_data,
                      gboolean *fres)
@@ -321,7 +313,6 @@ cylindercapella_guess(gint n_dat,
 /******************* paraboloidcapella ********************************/
 static gdouble
 parcapella_func(gdouble x,
-                G_GNUC_UNUSED gint n_param,
                 const gdouble *b,
                 G_GNUC_UNUSED gpointer user_data,
                 gboolean *fres)
@@ -362,7 +353,6 @@ parcapella_guess(gint n_dat,
 /******************* hertz spherical ********************************/
 static gdouble
 hertzsph_func(gdouble x,
-              G_GNUC_UNUSED gint n_param,
               const gdouble *param,
               G_GNUC_UNUSED gpointer user_data,
               gboolean *fres)
@@ -402,11 +392,11 @@ hertzsph_guess(gint n_dat,
     *fres = TRUE;
 }
 
-static GwySIUnit*
+static GwyUnit*
 hertzsph_get_units(G_GNUC_UNUSED GwyNLFitPreset *preset,
                    guint param,
-                   GwySIUnit *siunit_x,
-                   G_GNUC_UNUSED GwySIUnit *siunit_y)
+                   GwyUnit *siunit_x,
+                   G_GNUC_UNUSED GwyUnit *siunit_y)
 {
     static const gchar *units[] = { NULL, "m", "Pa", NULL };
     return select_unit(siunit_x, param, units, G_N_ELEMENTS(units));
@@ -415,7 +405,6 @@ hertzsph_get_units(G_GNUC_UNUSED GwyNLFitPreset *preset,
 /******************* hertz spherical with fixed thin film correction********************************/
 static gdouble
 hertzsphhfix_func(gdouble x,
-                  G_GNUC_UNUSED gint n_param,
                   const gdouble *param,
                   G_GNUC_UNUSED gpointer user_data,
                   gboolean *fres)
@@ -465,11 +454,11 @@ hertzsphhfix_guess(gint n_dat,
     *fres = TRUE;
 }
 
-static GwySIUnit*
+static GwyUnit*
 hertzsphfilm_get_units(G_GNUC_UNUSED GwyNLFitPreset *preset,
                        guint param,
-                       GwySIUnit *siunit_x,
-                       G_GNUC_UNUSED GwySIUnit *siunit_y)
+                       GwyUnit *siunit_x,
+                       G_GNUC_UNUSED GwyUnit *siunit_y)
 {
     static const gchar *units[] = { NULL, "m", "Pa", NULL, "m" };
     return select_unit(siunit_x, param, units, G_N_ELEMENTS(units));
@@ -478,7 +467,6 @@ hertzsphfilm_get_units(G_GNUC_UNUSED GwyNLFitPreset *preset,
 /******************* hertz spherical with free thin film correction********************************/
 static gdouble
 hertzsphhfree_func(gdouble x,
-                   G_GNUC_UNUSED gint n_param,
                    const gdouble *param,
                    G_GNUC_UNUSED gpointer user_data,
                    gboolean *fres)
@@ -531,7 +519,6 @@ hertzsphhfree_guess(gint n_dat,
 /******************* DMT  ********************************/
 static gdouble
 dmt_func(gdouble x,
-         G_GNUC_UNUSED gint n_param,
          const gdouble *param,
          G_GNUC_UNUSED gpointer user_data,
          gboolean *fres)
@@ -571,23 +558,22 @@ dmt_guess(gint n_dat,
     *fres = TRUE;
 }
 
-static GwySIUnit*
+static GwyUnit*
 dmt_get_units(G_GNUC_UNUSED GwyNLFitPreset *preset,
               guint param,
-              GwySIUnit *siunit_x,
-              GwySIUnit *siunit_y)
+              GwyUnit *siunit_x,
+              GwyUnit *siunit_y)
 {
     static const gchar *units[] = { NULL, NULL, "m", "Pa", NULL };
 
     if (param == 1)
-        return gwy_si_unit_duplicate(siunit_y);
+        return gwy_unit_copy(siunit_y);
     return select_unit(siunit_x, param, units, G_N_ELEMENTS(units));
 }
 
 /******************* Sneddon conical ********************************/
 static gdouble
 sneddon_func(gdouble x,
-             G_GNUC_UNUSED gint n_param,
              const gdouble *param,
              G_GNUC_UNUSED gpointer user_data,
              gboolean *fres)
@@ -626,11 +612,11 @@ sneddon_guess(gint n_dat,
     *fres = TRUE;
 }
 
-static GwySIUnit*
+static GwyUnit*
 sneddon_get_units(G_GNUC_UNUSED GwyNLFitPreset *preset,
                   guint param,
-                  GwySIUnit *siunit_x,
-                  G_GNUC_UNUSED GwySIUnit *siunit_y)
+                  GwyUnit *siunit_x,
+                  G_GNUC_UNUSED GwyUnit *siunit_y)
 {
     static const gchar *units[] = { NULL, "rad", "Pa", NULL };
     return select_unit(siunit_x, param, units, G_N_ELEMENTS(units));
@@ -639,16 +625,15 @@ sneddon_get_units(G_GNUC_UNUSED GwyNLFitPreset *preset,
 /******************* Sneddon conical with free film correction ********************************/
 static gdouble
 sneddonhfree_func(gdouble x,
-             G_GNUC_UNUSED gint n_param,
-             const gdouble *param,
-             G_GNUC_UNUSED gpointer user_data,
-             gboolean *fres)
+                  const gdouble *param,
+                  G_GNUC_UNUSED gpointer user_data,
+                  gboolean *fres)
 {
     /*xc, alpha, E, nu, h*/
     double xr = (param[0] - x); //this should be deformation and should be already entered only positive
     double zeta = 0.388;
-    double fc = 1 + zeta*2*tan(param[1])*xr/(M_PI*M_PI*param[4]) 
-              + 16*zeta*zeta*tan(param[1])*tan(param[1])*xr*xr/(param[4]*param[4]);
+    double fc = (1 + zeta*2*tan(param[1])*xr/(M_PI*M_PI*param[4]) 
+                 + 16*zeta*zeta*tan(param[1])*tan(param[1])*xr*xr/(param[4]*param[4]));
 
     *fres = TRUE;
     if (xr > 0)
@@ -659,10 +644,10 @@ sneddonhfree_func(gdouble x,
 
 static void
 sneddonhfree_guess(gint n_dat,
-              const gdouble *x,
-              const gdouble *y,
-              gdouble *param,
-              gboolean *fres)
+                   const gdouble *x,
+                   const gdouble *y,
+                   gdouble *param,
+                   gboolean *fres)
 {
     gint i;
     gdouble ymin = G_MAXDOUBLE;
@@ -683,11 +668,11 @@ sneddonhfree_guess(gint n_dat,
     *fres = TRUE;
 }
 
-static GwySIUnit*
+static GwyUnit*
 sneddonhfree_get_units(G_GNUC_UNUSED GwyNLFitPreset *preset,
                        guint param,
-                       GwySIUnit *siunit_x,
-                       G_GNUC_UNUSED GwySIUnit *siunit_y)
+                       GwyUnit *siunit_x,
+                       G_GNUC_UNUSED GwyUnit *siunit_y)
 {
     static const gchar *units[] = { NULL, "rad", "Pa", NULL, "m" };
     return select_unit(siunit_x, param, units, G_N_ELEMENTS(units));
@@ -696,10 +681,9 @@ sneddonhfree_get_units(G_GNUC_UNUSED GwyNLFitPreset *preset,
 /******************* Sneddon conical with fixed film correction ********************************/
 static gdouble
 sneddonhfix_func(gdouble x,
-             G_GNUC_UNUSED gint n_param,
-             const gdouble *param,
-             G_GNUC_UNUSED gpointer user_data,
-             gboolean *fres)
+                 const gdouble *param,
+                 G_GNUC_UNUSED gpointer user_data,
+                 gboolean *fres)
 {
     /*xc, alpha, E, nu, h*/
     double xr = (param[0] - x); //this should be deformation and should be already entered only positive
@@ -716,10 +700,10 @@ sneddonhfix_func(gdouble x,
 
 static void
 sneddonhfix_guess(gint n_dat,
-              const gdouble *x,
-              const gdouble *y,
-              gdouble *param,
-              gboolean *fres)
+                  const gdouble *x,
+                  const gdouble *y,
+                  gdouble *param,
+                  gboolean *fres)
 {
     gint i;
     gdouble ymin = G_MAXDOUBLE;
@@ -740,11 +724,11 @@ sneddonhfix_guess(gint n_dat,
     *fres = TRUE;
 }
 
-static GwySIUnit*
+static GwyUnit*
 sneddonhfix_get_units(G_GNUC_UNUSED GwyNLFitPreset *preset,
                       guint param,
-                      GwySIUnit *siunit_x,
-                      G_GNUC_UNUSED GwySIUnit *siunit_y)
+                      GwyUnit *siunit_x,
+                      G_GNUC_UNUSED GwyUnit *siunit_y)
 {
     static const gchar *units[] = { NULL, "rad", "Pa", NULL, "m" };
     return select_unit(siunit_x, param, units, G_N_ELEMENTS(units));
@@ -753,7 +737,6 @@ sneddonhfix_get_units(G_GNUC_UNUSED GwyNLFitPreset *preset,
 /******************* sphtiptap ********************************/
 static gdouble
 sphtiptap_func(gdouble x,
-               G_GNUC_UNUSED gint n_param,
                const gdouble *b,
                G_GNUC_UNUSED gpointer user_data,
                gboolean *fres)
@@ -793,7 +776,6 @@ sphtiptap_guess(gint n_dat,
 /******************* sphxu ********************************/
 static gdouble
 sphxu_func(gdouble x,
-           G_GNUC_UNUSED gint n_param,
            const gdouble *b,
            G_GNUC_UNUSED gpointer user_data,
            gboolean *fres)
@@ -832,7 +814,6 @@ sphxu_guess(gint n_dat,
 /******************* sphcappakarinen ********************************/
 static gdouble
 sphcappakarinen_func(gdouble x,
-                     G_GNUC_UNUSED gint n_param,
                      const gdouble *b,
                      G_GNUC_UNUSED gpointer user_data,
                      gboolean *fres)
@@ -861,7 +842,6 @@ sphcappakarinen_guess(gint n_dat,
 /******************* spheastman ********************************/
 static gdouble
 sphcapeastman_func(gdouble x,
-                   G_GNUC_UNUSED gint n_param,
                    const gdouble *b,
                    G_GNUC_UNUSED gpointer user_data,
                    gboolean *fres)
@@ -892,7 +872,6 @@ sphcapeastman_guess(gint n_dat,
 /******************* sphcapheinz ********************************/
 static gdouble
 sphcapheinz_func(gdouble x,
-                 G_GNUC_UNUSED gint n_param,
                  const gdouble *b,
                  G_GNUC_UNUSED gpointer user_data,
                  gboolean *fres)
@@ -921,7 +900,6 @@ sphcapheinz_guess(gint n_dat,
 /******************* sphesheinz ********************************/
 static gdouble
 sphesheinz_func(gdouble x,
-                G_GNUC_UNUSED gint n_param,
                 const gdouble *b,
                 G_GNUC_UNUSED gpointer user_data,
                 gboolean *fres)
@@ -950,7 +928,6 @@ sphesheinz_guess(gint n_dat,
 /******************* hsphhertz ********************************/
 static gdouble
 hsphhertz_func(gdouble x,
-               G_GNUC_UNUSED gint n_param,
                const gdouble *b,
                G_GNUC_UNUSED gpointer user_data,
                gboolean *fres)
@@ -979,7 +956,6 @@ hsphhertz_guess(gint n_dat,
 /******************* hertz paraboloid ********************************/
 static gdouble
 hertzpar_func(gdouble x,
-              G_GNUC_UNUSED gint n_param,
               const gdouble *b,
               G_GNUC_UNUSED gpointer user_data,
               gboolean *fres)
@@ -1018,7 +994,6 @@ hertzpar_guess(gint n_dat,
 /******************* electrostatic sphere capella ********************************/
 /*static gdouble
 esphcapella_func(gdouble x,
-                G_GNUC_UNUSED gint n_param,
                 const gdouble *b,
                 G_GNUC_UNUSED gpointer user_data,
                 gboolean *fres)
@@ -1029,10 +1004,10 @@ esphcapella_func(gdouble x,
 
 static void
 esphcapella_guess(gint n_dat,
-                 const gdouble *x,
-                 const gdouble *y,
-                 gdouble *param,
-                 gboolean *fres)
+                  const gdouble *x,
+                  const gdouble *y,
+                  gdouble *param,
+                  gboolean *fres)
 {
     gint i;
     gdouble xmin = x[0], xmax = x[n_dat - 1];
@@ -1562,6 +1537,7 @@ gwy_fd_curve_preset_class_init(GwyFDCurvePresetClass *klass)
     res_class->name = "fdcurvepresets";
     res_class->inventory = gwy_inventory_new(&res_class->item_type);
     gwy_inventory_forget_order(res_class->inventory);
+    res_class->setup_builtins = resource_setup_builtins;
 }
 
 static void
@@ -1572,35 +1548,24 @@ gwy_fd_curve_preset_init(G_GNUC_UNUSED GwyFDCurvePreset *preset)
 static GwyFDCurvePreset*
 gwy_fd_curve_preset_new_static(const GwyNLFitPresetBuiltin *data)
 {
-    GwyNLFitPreset *preset;
-
-    preset = g_object_new(GWY_TYPE_FD_CURVE_PRESET, "is-const", TRUE, NULL);
+    GwyNLFitPreset *preset = g_object_new(GWY_TYPE_FD_CURVE_PRESET,
+                                          "name", data->name,
+                                          "const", TRUE,
+                                          NULL);
     preset->builtin = data;
-    g_string_assign(GWY_RESOURCE(preset)->name, data->name);
 
     return (GwyFDCurvePreset*)preset;
 }
 
-void
-_gwy_fd_curve_preset_class_setup_presets(void)
+static void
+resource_setup_builtins(GwyInventory *inventory)
 {
-    GwyResourceClass *klass;
-    GwyFDCurvePreset *preset;
-    guint i;
-
-    /* Force class instantiation, this function is called before it's first
-     * referenced. */
-    klass = g_type_class_ref(GWY_TYPE_FD_CURVE_PRESET);
-
-    for (i = 0; i < G_N_ELEMENTS(fitting_presets); i++) {
-        preset = gwy_fd_curve_preset_new_static(fitting_presets + i);
-        gwy_inventory_insert_item(klass->inventory, preset);
+    for (guint i = 0; i < G_N_ELEMENTS(fitting_presets); i++) {
+        GwyFDCurvePreset *preset = gwy_fd_curve_preset_new_static(fitting_presets + i);
+        gwy_inventory_insert_item(inventory, preset);
         g_object_unref(preset);
     }
-    gwy_inventory_restore_order(klass->inventory);
-
-    /* The presets added a reference so we can safely unref it again */
-    g_type_class_unref(klass);
+    gwy_inventory_restore_order(inventory);
 }
 
 /**
@@ -1608,9 +1573,7 @@ _gwy_fd_curve_preset_class_setup_presets(void)
  *
  * Gets inventory with all the FD curve presets.
  *
- * Returns: FD curve preset inventory.
- *
- * Since: 2.7
+ * Returns: (transfer none): FD curve preset inventory.
  **/
 GwyInventory*
 gwy_fd_curve_presets(void)
@@ -1619,17 +1582,14 @@ gwy_fd_curve_presets(void)
     GWY_RESOURCE_CLASS(g_type_class_peek(GWY_TYPE_FD_CURVE_PRESET))->inventory;
 }
 
-/************************** Documentation ****************************/
-
 /**
- * SECTION:gwyfdcurvepreset
+ * SECTION: gwyfdcurvepreset
  * @title: GwyFDCurvePreset
  * @short_description: Force-distance curve fitting presets
  * @see_also: #GwyNLFitPreset
  *
- * Force-distance curve fitting presets are a particular subtype of non-linear
- * fitting presets.  They have their own class and inventory, but they are
- * functionally identical to #GwyNLFitPreset<!-- -->s.
+ * Force-distance curve fitting presets are a particular subtype of non-linear fitting presets.  They have their own
+ * class and inventory, but they are functionally identical to #GwyNLFitPreset<!-- -->s.
  **/
 
-/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
+/* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
