@@ -1,5 +1,5 @@
 /*
- *  $Id: gwynlfit.c 25916 2023-10-23 17:28:17Z yeti-dn $
+ *  $Id: gwynlfit.c 28903 2025-11-24 15:49:51Z yeti-dn $
  *  Copyright (C) 2000-2003 Martin Siler.
  *  Copyright (C) 2004-2023 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
@@ -18,11 +18,14 @@
 
 #include "config.h"
 #include <string.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwythreads.h>
-#include <libgwyddion/gwynlfit.h>
-#include "libgwyddion/gwyomp.h"
+#include <glib/gi18n-lib.h>
+
+#include "libgwyddion/macros.h"
+#include "libgwyddion/math.h"
+#include "libgwyddion/threads.h"
+#include "libgwyddion/gwynlfit.h"
+
+#include "libgwyddion/omp.h"
 
 /* Side step constant for numerical differentiation in gwy_math_nlfit_derive()
  */
@@ -56,32 +59,30 @@ typedef struct {
     GwySetMessageFunc set_message;
     GwyNLFitIdxFunc func_idx;
     GwyNLFitIdxDiffFunc diff_idx;
-    gboolean approx_geometric;  /* Ignored. */
     gint nparam;
 } GwyNLFitterReal;
 
-static void                gwy_math_nlfit_init    (GwyNLFitterReal *rnlfit);
-static gdouble             gwy_math_nlfit_fit_real(GwyNLFitterReal *rnlfit,
-                                                   guint ndata,
-                                                   const gdouble *x,
-                                                   const gdouble *y,
-                                                   const gdouble *weight,
-                                                   guint nparam,
-                                                   gdouble *param,
-                                                   const gboolean *fixed_param,
-                                                   const gint *link_map,
-                                                   gpointer user_data);
-static gdouble             gwy_math_nlfit_residua (GwyNLFitFunc func,
-                                                   GwyNLFitIdxFunc func_idx,
-                                                   guint ndata,
-                                                   const gdouble *x,
-                                                   const gdouble *y,
-                                                   const gdouble *weight,
-                                                   guint nparam,
-                                                   const gdouble *param,
-                                                   gpointer user_data,
-                                                   gdouble *resid,
-                                                   gboolean *success);
+static void    gwy_math_nlfit_init    (GwyNLFitterReal *rnlfit);
+static gdouble gwy_math_nlfit_fit_real(GwyNLFitterReal *rnlfit,
+                                       guint ndata,
+                                       const gdouble *x,
+                                       const gdouble *y,
+                                       const gdouble *weight,
+                                       guint nparam,
+                                       gdouble *param,
+                                       const gboolean *fixed_param,
+                                       const gint *link_map,
+                                       gpointer user_data);
+static gdouble gwy_math_nlfit_residua (GwyNLFitFunc func,
+                                       GwyNLFitIdxFunc func_idx,
+                                       guint ndata,
+                                       const gdouble *x,
+                                       const gdouble *y,
+                                       const gdouble *weight,
+                                       const gdouble *param,
+                                       gpointer user_data,
+                                       gdouble *resid,
+                                       gboolean *success);
 
 #if 0
 GType
@@ -101,15 +102,15 @@ gwy_math_nlfit_get_type(void)
 
 /**
  * gwy_math_nlfit_new:
- * @func: The fitted function.
- * @diff: The derivative of fitted function.
+ * @func: (scope call): The fitted function.
+ * @diff: (scope call) (nullable): The derivative of fitted function.
  *
  * Creates a new Marquardt-Levenberg nonlinear fitter for function with a real-valued independent variable.
  *
  * See gwy_math_nlfit_new_idx() for more complex scenarios.
  *
  * You can use gwy_math_nlfit_diff() computing the derivative numerically, when you do not know the derivatives
- * explicitely.  Since 2.46 passing %NULL as @diff has the same effect, i.e. the fitter will automatically use
+ * explicitely. Passing %NULL as @diff has the same effect, i.e. the fitter will automatically use
  * numerical differentiation.
  *
  * Returns: The newly created fitter.
@@ -122,15 +123,15 @@ gwy_math_nlfit_new(GwyNLFitFunc func, GwyNLFitDerFunc diff)
     rnlfit = g_new0(GwyNLFitterReal, 1);
     gwy_math_nlfit_init(rnlfit);
     rnlfit->fmarq = func;
-    rnlfit->dmarq = diff ? diff : gwy_math_nlfit_diff;
+    rnlfit->dmarq = diff;
 
     return (GwyNLFitter*)rnlfit;
 }
 
 /**
  * gwy_math_nlfit_new_idx:
- * @func: The fitted function.
- * @diff: The derivative of fitted function.
+ * @func: (scope call): The fitted function.
+ * @diff: (scope call) (nullable): The derivative of fitted function.
  *
  * Creates a new Marquardt-Levenberg nonlinear fitter for opaque indexed data.
  *
@@ -143,8 +144,6 @@ gwy_math_nlfit_new(GwyNLFitFunc func, GwyNLFitDerFunc diff)
  * function that performs the weighting (it can utilise the gwy_math_nlfit_diff_idx() helper).
  *
  * Returns: The newly created fitter.
- *
- * Since: 2.46
  **/
 GwyNLFitter*
 gwy_math_nlfit_new_idx(GwyNLFitIdxFunc func, GwyNLFitIdxDiffFunc diff)
@@ -196,8 +195,6 @@ gwy_math_nlfit_free(GwyNLFitter *nlfit)
  * This function is mostly usefil for language bindings.
  *
  * Returns: The newly created fitter.
- *
- * Since: 2.47
  **/
 GwyNLFitter*
 gwy_math_nlfit_copy(GwyNLFitter *nlfit)
@@ -248,9 +245,8 @@ gwy_math_nlfit_fit(GwyNLFitter *nlfit,
  * @ndata: The number of data points in @x, @y, @weight.
  * @x: Array of independent variable values.
  * @y: Array of dependent variable values.
- * @weight: Array of weights associated to each data point (usually equal to 
- *          inverse squares errors).  Can be %NULL, unit weight is then used
- *          for all data.
+ * @weight: Array of weights associated to each data point (usually equal to inverse squares errors).  Can be %NULL,
+ *          unit weight is then used for all data.
  * @nparam: The nuber of parameters.
  * @param: Array of parameters (of size @nparam).  Note the parameters must be initialized to reasonably near values.
  * @fixed_param: Which parameters should be treated as fixed (set corresponding element to %TRUE for them).  May be
@@ -295,8 +291,6 @@ gwy_math_nlfit_fit_full(GwyNLFitter *nlfit,
  * Performs a nonlinear fit of function on opaque indexed data.
  *
  * Returns: The final residual sum, a negative number in the case of failure.
- *
- * Since: 2.46
  **/
 gdouble
 gwy_math_nlfit_fit_idx(GwyNLFitter *nlfit,
@@ -329,8 +323,6 @@ gwy_math_nlfit_fit_idx(GwyNLFitter *nlfit,
  * ignored and master's property controls whether all are fixed or all variable.
  *
  * Returns: The final residual sum, a negative number in the case of failure.
- *
- * Since: 2.46
  **/
 gdouble
 gwy_math_nlfit_fit_idx_full(GwyNLFitter *nlfit,
@@ -395,7 +387,7 @@ gwy_math_nlfit_fit_real(GwyNLFitterReal *rnlfit,
     if (ndata < nparam)
         return -1.0;
 
-    g_return_val_if_fail((x && y && func && diff) || func_idx, -1.0);
+    g_return_val_if_fail((x && y && func) || func_idx, -1.0);
     g_return_val_if_fail(!func_idx || (!x && !y && !weight && !func), -1.0);
 
     if (set_message)
@@ -431,7 +423,7 @@ gwy_math_nlfit_fit_real(GwyNLFitterReal *rnlfit,
 
     resid = g_new(gdouble, ndata);
     param_best = g_memdup(param, nparam*sizeof(gdouble));
-    sumr_new = gwy_math_nlfit_residua(func, func_idx, ndata, x, y, w, nparam, param, user_data, resid, &rnlfit->eval);
+    sumr_new = gwy_math_nlfit_residua(func, func_idx, ndata, x, y, w, param, user_data, resid, &rnlfit->eval);
     gwy_debug("initial sumr %.16g", sumr_new);
     if (!rnlfit->eval) {
         g_warning("Initial residua evaluation failed");
@@ -527,12 +519,18 @@ gwy_math_nlfit_fit_real(GwyNLFitterReal *rnlfit,
                 gwy_clear(tv, n_var_param);
 
                 for (i = ifrom; i < ito; i++) {
-                    if (diff_idx)
-                        diff_idx(i, param, fixed, func_idx, user_data, der, &eeval);
-                    else if (diff)
-                        diff(x[i], nparam, param, fixed, func, user_data, der, &eeval);
-                    else
-                        gwy_math_nlfit_diff_idx(i, nparam, param, fixed, func_idx, user_data, der, &eeval);
+                    if (func_idx) {
+                        if (diff_idx)
+                            diff_idx(i, param, fixed, func_idx, user_data, der, &eeval);
+                        else
+                            gwy_math_nlfit_diff_idx(i, nparam, param, fixed, func_idx, user_data, der, &eeval);
+                    }
+                    else {
+                        if (diff)
+                            diff(x[i], param, fixed, func, user_data, der, &eeval);
+                        else
+                            gwy_math_nlfit_diff(x[i], nparam, param, fixed, func, user_data, der, &eeval);
+                    }
 
                     if (!eeval)
                         break;
@@ -646,8 +644,7 @@ gwy_math_nlfit_fit_real(GwyNLFitterReal *rnlfit,
             break;
 
         /* See what the new residua is. */
-        sumr_new = gwy_math_nlfit_residua(func, func_idx, ndata, x, y, w, nparam, param, user_data, resid,
-                                          &rnlfit->eval);
+        sumr_new = gwy_math_nlfit_residua(func, func_idx, ndata, x, y, w, param, user_data, resid, &rnlfit->eval);
         /* Catch failed evaluation even if it's not reported. */
         if (gwy_isinf(sumr_new) || gwy_isnan(sumr_new)) {
             gwy_debug("Inf/NaN encountered");
@@ -796,48 +793,19 @@ fail:
     return sumr_best;
 }
 
-/**
- * gwy_math_nlfit_derive:
- * @x: The value to compute the derivative at.
- * @nparam: The nuber of parameters.
- * @param: Array of parameters (of size @nparam).
- * @fixed_param: Which parameters should be treated as fixed (corresponding entries are set to %TRUE).
- * @func: The fitted function.
- * @user_data: User data to be passed to @func.
- * @der: Array where the put the result to.
- * @success: Set to %TRUE if succeeds, %FALSE on failure.
- *
- * Numerically computes the partial derivatives of a function.
- *
- * This is a legacy name for function gwy_math_nlfit_diff().
- **/
-void
-gwy_math_nlfit_derive(gdouble x,
-                      gint nparam,
-                      const gdouble *param,
-                      const gboolean *fixed_param,
-                      GwyNLFitFunc func,
-                      gpointer user_data,
-                      gdouble *diff,
-                      gboolean *success)
-{
-    return gwy_math_nlfit_diff(x, nparam, param, fixed_param, func, user_data, diff, success);
-}
-
 /**
  * gwy_math_nlfit_diff:
  * @x: The value to compute the derivative at.
  * @nparam: The nuber of parameters.
- * @param: Array of parameters (of size @nparam).
- * @fixed_param: Which parameters should be treated as fixed (corresponding entries are set to %TRUE).
- * @func: The fitted function.
+ * @param: (array length=nparam): Array of parameters (of size @nparam).
+ * @fixed_param: (array length=nparam):
+ *               Which parameters should be treated as fixed (corresponding entries are set to %TRUE).
+ * @func: (scope call): The fitted function.
  * @user_data: User data to be passed to @func.
  * @der: Array where the put the result to.
  * @success: Set to %TRUE if succeeds, %FALSE on failure.
  *
  * Numerically computes the partial derivatives of a function.
- *
- * Since: 2.46
  **/
 void
 gwy_math_nlfit_diff(gdouble x,
@@ -849,11 +817,10 @@ gwy_math_nlfit_diff(gdouble x,
                     gdouble *der,
                     gboolean *success)
 {
-    gdouble *param_tmp;
     gdouble hj, left, right;
     gint j;
 
-    param_tmp = g_newa(gdouble, nparam);
+    gdouble param_tmp[nparam];
     gwy_assign(param_tmp, param, nparam);
 
     for (j = 0; j < nparam; j++) {
@@ -867,12 +834,12 @@ gwy_math_nlfit_diff(gdouble x,
             hj = FitSqrtMachEps;
 
         param_tmp[j] -= hj;
-        left = func(x, nparam, param_tmp, user_data, success);
+        left = func(x, param_tmp, user_data, success);
         if (!*success)
             return;
 
         param_tmp[j] += 2 * hj;
-        right = func(x, nparam, param_tmp, user_data, success);
+        right = func(x, param_tmp, user_data, success);
         if (!*success)
             return;
 
@@ -885,9 +852,10 @@ gwy_math_nlfit_diff(gdouble x,
  * gwy_math_nlfit_diff_idx:
  * @i: Data index from the set {0, 1, 2, ..., ndata-1}.
  * @nparam: The nuber of parameters.
- * @param: Array of parameters (of size @nparam).
- * @fixed_param: Which parameters should be treated as fixed (corresponding entries are set to %TRUE).
- * @func: The fitted function.
+ * @param: (array length=nparam): Array of parameters (of size @nparam).
+ * @fixed_param: (array length=nparam):
+ *               Which parameters should be treated as fixed (corresponding entries are set to %TRUE).
+ * @func: (scope call): The fitted function.
  * @user_data: User data to be passed to @func.
  * @der: Array where the put the result to.
  * @success: Set to %TRUE if succeeds, %FALSE on failure.
@@ -900,8 +868,6 @@ gwy_math_nlfit_diff(gdouble x,
  *
  * You can employ this function in your own #GwyNLFitIdxDiffFunc function if you need to modify the derivatives
  * somehow, for instance to apply weighting.
- *
- * Since: 2.46
  **/
 void
 gwy_math_nlfit_diff_idx(guint i,
@@ -913,11 +879,10 @@ gwy_math_nlfit_diff_idx(guint i,
                         gdouble *der,
                         gboolean *success)
 {
-    gdouble *param_tmp;
     gdouble hj, left, right;
     gint j;
 
-    param_tmp = g_newa(gdouble, nparam);
+    gdouble param_tmp[nparam];
     gwy_assign(param_tmp, param, nparam);
 
     for (j = 0; j < nparam; j++) {
@@ -945,7 +910,7 @@ gwy_math_nlfit_diff_idx(guint i,
     }
 }
 
-/* NB: we apply weighting to to resid[]. */
+/* NB: we apply weighting to resid[]. */
 static gdouble
 gwy_math_nlfit_residua(GwyNLFitFunc func,
                        GwyNLFitIdxFunc func_idx,
@@ -953,7 +918,6 @@ gwy_math_nlfit_residua(GwyNLFitFunc func,
                        const gdouble *x,
                        const gdouble *y,
                        const gdouble *weight,
-                       guint nparam,
                        const gdouble *param,
                        gpointer user_data,
                        gdouble *resid,
@@ -991,7 +955,7 @@ gwy_math_nlfit_residua(GwyNLFitFunc func,
 #ifdef _OPENMP
 #pragma omp parallel if(gwy_threads_are_enabled()) default(none) \
             reduction(+:s) reduction(&&:ok) \
-            shared(func,x,y,ndata,resid,weight,nparam,param,user_data)
+            shared(func,x,y,ndata,resid,weight,param,user_data)
 #endif
         {
             guint ifrom = gwy_omp_chunk_start(ndata);
@@ -1001,7 +965,7 @@ gwy_math_nlfit_residua(GwyNLFitFunc func,
             for (i = ifrom; i < ito; i++) {
                 gboolean ook;
 
-                resid[i] = func(x[i], nparam, param, user_data, &ook) - y[i];
+                resid[i] = func(x[i], param, user_data, &ook) - y[i];
                 if (weight)
                     resid[i] *= weight[i];
                 s += resid[i] * resid[i];
@@ -1046,46 +1010,6 @@ gwy_math_nlfit_set_max_iterations(GwyNLFitter *nlfit,
     nlfit->maxiter = maxiter;
 }
 
-/**
- * gwy_math_nlfit_get_approx_geometric:
- * @nlfit: A Marquardt-Levenberg nonlinear fitter.
- *
- * Reports if a non-linear fitter performs approximately orthogonal fitting.
- *
- * This function does not do anything useful.
- *
- * Returns: %TRUE if the fitting is approximately orthogonal; %FALSE for ordinary least squares.
- *
- * Since: 2.47
- **/
-gboolean
-gwy_math_nlfit_get_approx_geometric(GwyNLFitter *nlfit)
-{
-    GwyNLFitterReal *rnlfit = (GwyNLFitterReal*)nlfit;
-
-    return rnlfit->approx_geometric;
-}
-
-/**
- * gwy_math_nlfit_set_approx_geometric:
- * @nlfit: A Marquardt-Levenberg nonlinear fitter.
- * @setting: %TRUE to approximately orthogonal (geometrical) fitting.
- *
- * Sets a non-linear fitter to perform ordinary or approximately orthogonal fitting.
- *
- * This function does not do anything useful.  Since 2.56 it is no-op.
- *
- * Since: 2.47
- **/
-void
-gwy_math_nlfit_set_approx_geometric(GwyNLFitter *nlfit,
-                                    gboolean setting)
-{
-    GwyNLFitterReal *rnlfit = (GwyNLFitterReal*)nlfit;
-
-    rnlfit->approx_geometric = setting;
-}
-
 /**
  * gwy_math_nlfit_succeeded:
  * @nlfit: A Marquardt-Levenberg nonlinear fitter.
@@ -1096,8 +1020,6 @@ gwy_math_nlfit_set_approx_geometric(GwyNLFitter *nlfit,
  * gwy_math_nlfit_fit() or gwy_math_nlfit_fit_full().  This function allows to test it later.
  *
  * Returns: %TRUE if the last fitting suceeded, %FALSE if it failed.
- *
- * Since: 2.7
  **/
 gboolean
 gwy_math_nlfit_succeeded(GwyNLFitter *nlfit)
@@ -1157,8 +1079,6 @@ gwy_math_nlfit_get_dispersion(GwyNLFitter *nlfit)
  * arguments for gwy_math_nlfit_get_correlations().
  *
  * Returns: The number of parameters for the last fit.  If the last fit was not successful zero is returned.
- *
- * Since: 2.47
  **/
 gint
 gwy_math_nlfit_get_nparam(GwyNLFitter *nlfit)
@@ -1175,8 +1095,6 @@ gwy_math_nlfit_get_nparam(GwyNLFitter *nlfit)
  * Gets the state of a nonlinear fitter.
  *
  * Returns: %TRUE if the last fit succeeded, %FALSE if it failed.
- *
- * Since: 2.47
  **/
 gboolean
 gwy_math_nlfit_get_eval(GwyNLFitter *nlfit)
@@ -1194,8 +1112,6 @@ gwy_math_nlfit_get_eval(GwyNLFitter *nlfit)
  *
  * Returns: The covariance matrix, as lower triangular matrix, owned by the fitter.  The return value will be %NULL if
  *          the last fit did not succeed.
- *
- * Since: 2.47
  **/
 const gdouble*
 gwy_math_nlfit_get_covar(GwyNLFitter *nlfit)
@@ -1238,12 +1154,10 @@ gwy_math_nlfit_get_correlations(GwyNLFitter *nlfit, gint par1, gint par2)
 /**
  * gwy_math_nlfit_set_callbacks:
  * @nlfit: A Marquardt-Levenberg nonlinear fitter.
- * @set_fraction: Function that sets fraction to output (or %NULL).
- * @set_message: Function that sets message to output (or %NULL).
+ * @set_fraction: (scope call) (nullable): Function that sets fraction to output (or %NULL).
+ * @set_message: (scope call) (nullable): Function that sets message to output (or %NULL).
  *
  * Sets callbacks reporting a non-linear least squares fitter progress.
- *
- * Since: 2.46
  **/
 void
 gwy_math_nlfit_set_callbacks(GwyNLFitter *fitter,
@@ -1270,8 +1184,6 @@ gwy_math_nlfit_set_callbacks(GwyNLFitter *fitter,
  * just pass an array of size @n because the number of free parameters is what this function calculates.
  *
  * Returns: The number of free parameters.  This is the number of ids filled in @id_map.
- *
- * Since: 2.50
  **/
 gint
 gwy_math_nlfit_map_free_params(const gboolean *fixed_param,
@@ -1311,9 +1223,9 @@ gwy_math_nlfit_map_free_params(const gboolean *fixed_param,
  * when there are some fixed or linked parameters.  Arbitrary additional (non-fitting) parameters can be passed to the
  * fited function in @user_data.
  *
- * After a successfull fit additional fit information can be obtained with gwy_math_nlfit_get_dispersion(),
+ * After a successful fit additional fit information can be obtained with gwy_math_nlfit_get_dispersion(),
  * gwy_math_nlfit_get_correlations(), gwy_math_nlfit_get_sigma(). Note these functions may be used only after
- * a successfull fit. When a fitter is no longer needed, it should be freed with gwy_math_nlfit_free().
+ * a successful fit. When a fitter is no longer needed, it should be freed with gwy_math_nlfit_free().
  *
  * Several common functions are also available as fitting presets that can be fitted with gwy_nlfit_preset_fit().  See
  * #GwyNLFitPreset for details.
@@ -1364,8 +1276,6 @@ gwy_math_nlfit_map_free_params(const gboolean *fixed_param,
  * estimated error of the @i-th data point.  Not multiplying by anything correspond to using the default unit weights.
  *
  * Returns: Difference between the function value and data in the @i-th data point.
- *
- * Since: 2.46
  **/
 
 /**
@@ -1384,8 +1294,6 @@ gwy_math_nlfit_map_free_params(const gboolean *fixed_param,
  *
  * The function must take care of weighting.  The derivatives should be multiplied by the inverse of the (unsquared)
  * estimated error of the @i-th data point.  Not multiplying by anything correspond to using the default unit weights.
- *
- * Since: 2.46
  **/
 
 /**
@@ -1422,8 +1330,6 @@ gwy_math_nlfit_map_free_params(const gboolean *fixed_param,
  *                          matters and it is always made non-negative.
  *
  * Type of fitting parameter properties.
- *
- * Since: 2.47
  **/
 
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
