@@ -1,66 +1,36 @@
 /*
- *  $Id: gwynlfitpreset.c 26809 2024-11-07 10:12:24Z yeti-dn $
+ *  $Id: gwynlfitpreset.c 29061 2026-01-02 15:01:53Z yeti-dn $
  *  Copyright (C) 2000-2003 Martin Siler.
  *  Copyright (C) 2004-2020 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
+ *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
+ *  later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "config.h"
 #include <string.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwydebugobjects.h>
-#include <libgwyddion/gwynlfitpreset.h>
-#include "gwyddioninternal.h"
+#include <glib/gi18n-lib.h>
 
-static GwyNLFitPreset*
-gwy_nlfit_preset_new_static(const GwyNLFitPresetBuiltin *data);
+#include "libgwyddion/macros.h"
+#include "libgwyddion/math.h"
+#include "libgwyddion/gwynlfitpreset.h"
+
+#include "libgwyddion/internal.h"
+
+static GwyNLFitPreset* create_nlfit_preset_builtin(const GwyNLFitPresetBuiltin *data);
+static void            resource_setup_builtins    (GwyInventory *inventory);
 
 G_DEFINE_TYPE(GwyNLFitPreset, gwy_nlfit_preset, GWY_TYPE_RESOURCE)
 
-G_GNUC_UNUSED
-static gdouble
-_gwy_gamma(gdouble x)
-{
-    static const gdouble p[] = {
-        676.5203681218851,
-        -1259.1392167224028,
-        771.32342877765313,
-        -176.61502916214059,
-        12.507343278686905,
-        -0.13857109526572012,
-        9.9843695780195716e-6,
-        1.5056327351493116e-7
-    };
-    gdouble xi = 0.99999999999980993, t;
-    guint i;
-
-    if (x < 0.5)
-        return G_PI/(sin(G_PI*x)*_gwy_gamma(1.0 - x));
-
-    x -= 1.0;
-    for (i = 0; i < G_N_ELEMENTS(p); i++)
-        xi += p[i]/(x + i + 1);
-
-    t = x + G_N_ELEMENTS(p) - 0.5;
-    return sqrt(2.0*G_PI) * pow(t, x + 0.5) * exp(-t) * xi;
-}
-
 static void
 generic_decay_guess(gint n_dat,
                     const gdouble *x,
@@ -90,7 +60,6 @@ generic_decay_guess(gint n_dat,
 /*********************** gaussian *****************************/
 static gdouble
 gauss_func(gdouble x,
-           G_GNUC_UNUSED gint n_param,
            const gdouble *b,
            G_GNUC_UNUSED gpointer user_data,
            gboolean *fres)
@@ -137,7 +106,7 @@ gauss_guess(gint n_dat,
     nv = G_N_ELEMENTS(vec);
     for (i = 0; i < nv; i++)
         vec[i] = y[i*(n_dat - 1)/(nv - 1)];
-    cm = gwy_math_median(nv, vec);
+    cm = gwy_math_median(vec, nv);
 
     if (cm - y[imin] < y[imax] - cm) {
         param[1] = y[imin];
@@ -167,7 +136,6 @@ gauss_guess(gint n_dat,
 /******************** gaussian PSDF ***************************/
 static gdouble
 gauss_psdf_func(gdouble x,
-                G_GNUC_UNUSED gint n_param,
                 const gdouble *b,
                 G_GNUC_UNUSED gpointer user_data,
                 gboolean *fres)
@@ -201,24 +169,24 @@ gauss_psdf_guess(gint n_dat,
         *fres = FALSE;
 }
 
-static GwySIUnit*
+static GwyUnit*
 psdf_get_units(G_GNUC_UNUSED GwyNLFitPreset *preset,
                guint param,
-               GwySIUnit *siunit_x,
-               GwySIUnit *siunit_y)
+               GwyUnit *siunit_x,
+               GwyUnit *siunit_y)
 {
-    GwySIUnit *result;
+    GwyUnit *result;
 
     switch (param) {
         case 0:
-        result = gwy_si_unit_multiply(siunit_x, siunit_y, NULL);
-        if (!gwy_si_unit_nth_root(result, 2, result))
-            gwy_si_unit_set_from_string(result, NULL);
+        result = gwy_unit_multiply(siunit_x, siunit_y, NULL);
+        if (!gwy_unit_nth_root(result, 2, result))
+            gwy_unit_clear(result);
         return result;
         break;
 
         case 1:
-        return gwy_si_unit_power(siunit_x, -1, NULL);
+        return gwy_unit_power(siunit_x, -1, NULL);
         break;
 
         default:
@@ -230,15 +198,14 @@ psdf_get_units(G_GNUC_UNUSED GwyNLFitPreset *preset,
 /******************** two-gaussian PSDF ***************************/
 static gdouble
 gauss2_psdf_func(gdouble x,
-                 G_GNUC_UNUSED gint n_param,
                  const gdouble *b,
                  G_GNUC_UNUSED gpointer user_data,
                  gboolean *fres)
 {
     gdouble f1, f2;
     gboolean fres1, fres2;
-    f1 = gauss_psdf_func(x, 2, b, NULL, &fres1);
-    f2 = gauss_psdf_func(x, 2, b + 2, NULL, &fres2);
+    f1 = gauss_psdf_func(x, b, NULL, &fres1);
+    f2 = gauss_psdf_func(x, b + 2, NULL, &fres2);
     *fres = fres1 && fres2;
     return f1 + f2;
 }
@@ -266,11 +233,11 @@ gauss2_psdf_guess(gint n_dat,
     param[1] /= GWY_SQRT3;
 }
 
-static GwySIUnit*
+static GwyUnit*
 psdf2_get_units(GwyNLFitPreset *preset,
                 guint param,
-                GwySIUnit *siunit_x,
-                GwySIUnit *siunit_y)
+                GwyUnit *siunit_x,
+                GwyUnit *siunit_y)
 {
     return psdf_get_units(preset, param % 2, siunit_x, siunit_y);
 }
@@ -278,7 +245,6 @@ psdf2_get_units(GwyNLFitPreset *preset,
 /******************* gaussian HHCF ********************************/
 static gdouble
 gauss_hhcf_func(gdouble x,
-                G_GNUC_UNUSED gint n_param,
                 const gdouble *b,
                 G_GNUC_UNUSED gpointer user_data,
                 gboolean *fres)
@@ -314,24 +280,24 @@ gauss_hhcf_guess(gint n_dat,
     *fres = TRUE;
 }
 
-static GwySIUnit*
+static GwyUnit*
 cf_get_units(G_GNUC_UNUSED GwyNLFitPreset *preset,
              guint param,
-             GwySIUnit *siunit_x,
-             GwySIUnit *siunit_y)
+             GwyUnit *siunit_x,
+             GwyUnit *siunit_y)
 {
-    GwySIUnit *result;
+    GwyUnit *result;
 
     switch (param) {
         case 0:
-        result = gwy_si_unit_duplicate(siunit_y);
-        if (!gwy_si_unit_nth_root(result, 2, result))
-            gwy_si_unit_set_from_string(result, NULL);
+        result = gwy_unit_copy(siunit_y);
+        if (!gwy_unit_nth_root(result, 2, result))
+            gwy_unit_clear(result);
         return result;
         break;
 
         case 1:
-        return gwy_si_unit_duplicate(siunit_x);
+        return gwy_unit_copy(siunit_x);
         break;
 
         default:
@@ -343,7 +309,6 @@ cf_get_units(G_GNUC_UNUSED GwyNLFitPreset *preset,
 /****************    gaussian ACF  *****************************************/
 static gdouble
 gauss_acf_func(gdouble x,
-               G_GNUC_UNUSED gint n_param,
                const gdouble *b,
                G_GNUC_UNUSED gpointer user_data,
                gboolean *fres)
@@ -375,7 +340,6 @@ gauss_acf_guess(gint n_dat,
 /******************** gaussian IPSDF ***************************/
 static gdouble
 gauss_rpsdf_func(gdouble x,
-                 G_GNUC_UNUSED gint n_param,
                  const gdouble *b,
                  G_GNUC_UNUSED gpointer user_data,
                  gboolean *fres)
@@ -447,7 +411,6 @@ gauss_rpsdf_guess(gint n_dat,
 /**************** exponential ************************************/
 static gdouble
 exp_func(gdouble x,
-         G_GNUC_UNUSED gint n_param,
          const gdouble *b,
          G_GNUC_UNUSED gpointer user_data,
          gboolean *fres)
@@ -506,7 +469,6 @@ exp_guess(gint n_dat,
 /**************** exponential PSDF **************************/
 static gdouble
 exp_psdf_func(gdouble x,
-              G_GNUC_UNUSED gint n_param,
               const gdouble *b,
               G_GNUC_UNUSED gpointer user_data,
               gboolean *fres)
@@ -544,7 +506,6 @@ exp_psdf_guess(gint n_dat,
 /***************** exponential HHCF ********************************/
 static gdouble
 exp_hhcf_func(gdouble x,
-              G_GNUC_UNUSED gint n_param,
               const gdouble *b,
               G_GNUC_UNUSED gpointer user_data,
               gboolean *fres)
@@ -582,7 +543,6 @@ exp_hhcf_guess(gint n_dat,
 /*************** exponential ACF ************************************/
 static gdouble
 exp_acf_func(gdouble x,
-             G_GNUC_UNUSED gint n_param,
              const gdouble *b,
              G_GNUC_UNUSED gpointer user_data,
              gboolean *fres)
@@ -614,7 +574,6 @@ exp_acf_guess(gint n_dat,
 /******************** exponential IPSDF ***************************/
 static gdouble
 exp_rpsdf_func(gdouble x,
-               G_GNUC_UNUSED gint n_param,
                const gdouble *b,
                G_GNUC_UNUSED gpointer user_data,
                gboolean *fres)
@@ -680,7 +639,6 @@ exp_rpsdf_guess(gint n_dat,
  * But we cannot normalise to σ² because C and σ are terribly correlated. */
 static gdouble
 kcorr_psdf_func(gdouble x,
-                G_GNUC_UNUSED gint n_param,
                 const gdouble *b,
                 G_GNUC_UNUSED gpointer user_data,
                 gboolean *fres)
@@ -708,14 +666,14 @@ kcorr_psdf_guess(gint n_dat,
     param[2] = 0.2;
 }
 
-static GwySIUnit*
+static GwyUnit*
 kcorr_psdf_get_units(GwyNLFitPreset *preset,
                      guint param,
-                     GwySIUnit *siunit_x,
-                     GwySIUnit *siunit_y)
+                     GwyUnit *siunit_x,
+                     GwyUnit *siunit_y)
 {
     if (param == 2)
-        return gwy_si_unit_new(NULL);
+        return gwy_unit_new(NULL);
 
     return psdf_get_units(preset, param, siunit_x, siunit_y);
 }
@@ -724,7 +682,6 @@ kcorr_psdf_get_units(GwyNLFitPreset *preset,
 
 static gdouble
 powerexp_acf_func(gdouble x,
-                  G_GNUC_UNUSED gint n_param,
                   const gdouble *b,
                   G_GNUC_UNUSED gpointer user_data,
                   gboolean *fres)
@@ -754,14 +711,14 @@ powerexp_acf_guess(gint n_dat,
     param[2] = 1.5;
 }
 
-static GwySIUnit*
+static GwyUnit*
 powerexp_acf_get_units(GwyNLFitPreset *preset,
                        guint param,
-                       GwySIUnit *siunit_x,
-                       GwySIUnit *siunit_y)
+                       GwyUnit *siunit_x,
+                       GwyUnit *siunit_y)
 {
     if (param == 2)
-        return gwy_si_unit_new(NULL);
+        return gwy_unit_new(NULL);
 
     return cf_get_units(preset, param, siunit_x, siunit_y);
 }
@@ -769,7 +726,6 @@ powerexp_acf_get_units(GwyNLFitPreset *preset,
 /**************   polynomial 0th order ********************************/
 static gdouble
 poly_0_func(G_GNUC_UNUSED gdouble x,
-            G_GNUC_UNUSED gint n_param,
             const gdouble *b,
             G_GNUC_UNUSED gpointer user_data,
             gboolean *fres)
@@ -786,17 +742,16 @@ poly_0_guess(gint n_dat,
              gdouble *param,
              gboolean *fres)
 {
-    gwy_math_fit_polynom(n_dat, x, y, 0, param);
+    gwy_math_fit_poly(x, y, n_dat, 0, param);
     *fres = TRUE;
 }
 
 /*************** polynomial 1st order ********************************/
 static gdouble
 poly_1_func(gdouble x,
-           G_GNUC_UNUSED gint n_param,
-           const gdouble *b,
-           G_GNUC_UNUSED gpointer user_data,
-           gboolean *fres)
+            const gdouble *b,
+            G_GNUC_UNUSED gpointer user_data,
+            gboolean *fres)
 {
     *fres = TRUE;
 
@@ -810,14 +765,13 @@ poly_1_guess(gint n_dat,
              gdouble *param,
              gboolean *fres)
 {
-    gwy_math_fit_polynom(n_dat, x, y, 1, param);
+    gwy_math_fit_poly(x, y, n_dat, 1, param);
     *fres = TRUE;
 }
 
 /************* polynomial 2nd order **********************************/
 static gdouble
 poly_2_func(gdouble x,
-            G_GNUC_UNUSED gint n_param,
             const gdouble *b,
             G_GNUC_UNUSED gpointer user_data,
             gboolean *fres)
@@ -834,14 +788,13 @@ poly_2_guess(gint n_dat,
              gdouble *param,
              gboolean *fres)
 {
-    gwy_math_fit_polynom(n_dat, x, y, 2, param);
+    gwy_math_fit_poly(x, y, n_dat, 2, param);
     *fres = TRUE;
 }
 
 /************** polynomial 3rd order *****************************/
 static gdouble
 poly_3_func(gdouble x,
-            G_GNUC_UNUSED gint n_param,
             const gdouble *b,
             G_GNUC_UNUSED gpointer user_data,
             gboolean *fres)
@@ -858,14 +811,13 @@ poly_3_guess(gint n_dat,
              gdouble *param,
              gboolean *fres)
 {
-    gwy_math_fit_polynom(n_dat, x, y, 3, param);
+    gwy_math_fit_poly(x, y, n_dat, 3, param);
     *fres = TRUE;
 }
 
 /************** polynomial 4th order *****************************/
 static gdouble
 poly_4_func(gdouble x,
-            G_GNUC_UNUSED gint n_param,
             const gdouble *b,
             G_GNUC_UNUSED gpointer user_data,
             gboolean *fres)
@@ -882,14 +834,13 @@ poly_4_guess(gint n_dat,
              gdouble *param,
              gboolean *fres)
 {
-    gwy_math_fit_polynom(n_dat, x, y, 4, param);
+    gwy_math_fit_poly(x, y, n_dat, 4, param);
     *fres = TRUE;
 }
 
 /************** polynomial 4th order *****************************/
 static gdouble
 poly_5_func(gdouble x,
-            G_GNUC_UNUSED gint n_param,
             const gdouble *b,
             G_GNUC_UNUSED gpointer user_data,
             gboolean *fres)
@@ -906,14 +857,13 @@ poly_5_guess(gint n_dat,
              gdouble *param,
              gboolean *fres)
 {
-    gwy_math_fit_polynom(n_dat, x, y, 5, param);
+    gwy_math_fit_poly(x, y, n_dat, 5, param);
     *fres = TRUE;
 }
 
 /******************* square signal ********************************/
 static gdouble
 square_func(gdouble x,
-            G_GNUC_UNUSED gint n_param,
             const gdouble *b,
             G_GNUC_UNUSED gpointer user_data,
             gboolean *fres)
@@ -968,7 +918,6 @@ square_guess(gint n_dat,
 /******************* parabolic step ********************************/
 static gdouble
 parstep_func(gdouble x,
-             G_GNUC_UNUSED gint n_param,
              const gdouble *b,
              G_GNUC_UNUSED gpointer user_data,
              gboolean *fres)
@@ -1055,7 +1004,6 @@ parstep_guess(gint n_dat,
 /******************* smooth step ********************************/
 static gdouble
 smstep_func(gdouble x,
-            G_GNUC_UNUSED gint n_param,
             const gdouble *b,
             G_GNUC_UNUSED gpointer user_data,
             gboolean *fres)
@@ -1134,7 +1082,7 @@ smstep_guess(gint n_dat,
         param[3] = (xymax > xymin) ? ymax - ymin : ymin - ymax;
     /* Using zero here prevents L-M from making progress. */
     param[4] = 0.05*param[3]/(xmax - xmin);
-    param[5] = 0.4*param[4];
+    param[5] = param[4]/(xmax - xmin);
 
     *fres = TRUE;
 }
@@ -1142,7 +1090,6 @@ smstep_guess(gint n_dat,
 /******************* erf bent step ********************************/
 static gdouble
 bentstep_func(gdouble x,
-              G_GNUC_UNUSED gint n_param,
               const gdouble *b,
               G_GNUC_UNUSED gpointer user_data,
               gboolean *fres)
@@ -1165,7 +1112,6 @@ bentstep_func(gdouble x,
 /******************* tanh bent step ********************************/
 static gdouble
 boltzmannstep_func(gdouble x,
-                   G_GNUC_UNUSED gint n_param,
                    const gdouble *b,
                    G_GNUC_UNUSED gpointer user_data,
                    gboolean *fres)
@@ -1188,7 +1134,6 @@ boltzmannstep_func(gdouble x,
 /******************* power function ********************************/
 static gdouble
 power_func(gdouble x,
-           G_GNUC_UNUSED gint n_param,
            const gdouble *b,
            G_GNUC_UNUSED gpointer user_data,
            gboolean *fres)
@@ -1253,24 +1198,24 @@ power_guess(gint n_dat,
     *fres = TRUE;
 }
 
-static GwySIUnit*
+static GwyUnit*
 power_get_units(G_GNUC_UNUSED GwyNLFitPreset *preset,
                 guint param,
-                G_GNUC_UNUSED GwySIUnit *siunit_x,
-                GwySIUnit *siunit_y)
+                G_GNUC_UNUSED GwyUnit *siunit_x,
+                GwyUnit *siunit_y)
 {
     switch (param) {
         case 0:
-        return gwy_si_unit_duplicate(siunit_y);
+        return gwy_unit_copy(siunit_y);
         break;
 
         case 1:
         /* XXX: Ugly, arbitrary real power */
-        return gwy_si_unit_new(NULL);
+        return gwy_unit_new(NULL);
         break;
 
         case 2:
-        return gwy_si_unit_new(NULL);
+        return gwy_unit_new(NULL);
         break;
 
         default:
@@ -1282,7 +1227,6 @@ power_get_units(G_GNUC_UNUSED GwyNLFitPreset *preset,
 /******************* lorentzian ********************************/
 static gdouble
 lorentz_func(gdouble x,
-             G_GNUC_UNUSED gint n_param,
              const gdouble *b,
              G_GNUC_UNUSED gpointer user_data,
              gboolean *fres)
@@ -1322,7 +1266,7 @@ lorentz_guess(gint n_dat,
     nv = G_N_ELEMENTS(vec);
     for (i = 0; i < nv; i++)
         vec[i] = y[i*(n_dat - 1)/(nv - 1)];
-    cm = gwy_math_median(nv, vec);
+    cm = gwy_math_median(vec, nv);
 
     if (cm - y[imin] < y[imax] - cm) {
         param[0] = y[imin];
@@ -1349,7 +1293,6 @@ lorentz_guess(gint n_dat,
 /******************* sinc ********************************/
 static gdouble
 sinc_func(gdouble x,
-          G_GNUC_UNUSED gint n_param,
           const gdouble *b,
           G_GNUC_UNUSED gpointer user_data,
           gboolean *fres)
@@ -1391,7 +1334,6 @@ sinc_guess(gint n_dat,
 
 static gdouble
 sine_func(gdouble x,
-          G_GNUC_UNUSED gint n_param,
           const gdouble *param,
           G_GNUC_UNUSED gpointer user_data,
           gboolean *fres)
@@ -1405,7 +1347,7 @@ sine_guess(gint n_dat,
            G_GNUC_UNUSED const gdouble *x,
            const gdouble *y,
            gdouble *param,
-           gboolean *fres)       
+           gboolean *fres)
 {
     gint i;
     gdouble min, max, avg;
@@ -1451,23 +1393,23 @@ weights_linear_decrease(gint n_dat,
 
 /************************** generic scalers ****************************/
 
-static GwySIUnit*
+static GwyUnit*
 builtin_get_unit(GwyNLFitPreset *preset,
-                 guint param,
-                 GwySIUnit *siunit_x,
-                 GwySIUnit *siunit_y)
+                 guint i,
+                 GwyUnit *siunit_x,
+                 GwyUnit *siunit_y)
 {
     const GwyNLFitParam *p;
 
     g_return_val_if_fail(preset->builtin, NULL);
-    g_return_val_if_fail(param < preset->builtin->nparams, NULL);
+    g_return_val_if_fail(i < preset->builtin->nparams, NULL);
 
     if (preset->builtin->get_unit)
-        return preset->builtin->get_unit(preset, param, siunit_x, siunit_y);
+        return preset->builtin->get_unit(preset, i, siunit_x, siunit_y);
 
     /* Generic unit construction when there are no peculiarities */
-    p = preset->builtin->param + param;
-    return gwy_si_unit_power_multiply(siunit_x, p->power_x,
+    p = preset->builtin->param + i;
+    return gwy_unit_power_multiply(siunit_x, p->power_x,
                                       siunit_y, p->power_y,
                                       NULL);
 }
@@ -1975,6 +1917,7 @@ gwy_nlfit_preset_class_init(GwyNLFitPresetClass *klass)
     res_class->name = "nlfitpresets";
     res_class->inventory = gwy_inventory_new(&res_class->item_type);
     gwy_inventory_forget_order(res_class->inventory);
+    res_class->setup_builtins = resource_setup_builtins;
 }
 
 static void
@@ -1983,37 +1926,26 @@ gwy_nlfit_preset_init(G_GNUC_UNUSED GwyNLFitPreset *preset)
 }
 
 static GwyNLFitPreset*
-gwy_nlfit_preset_new_static(const GwyNLFitPresetBuiltin *data)
+create_nlfit_preset_builtin(const GwyNLFitPresetBuiltin *data)
 {
-    GwyNLFitPreset *preset;
-
-    preset = g_object_new(GWY_TYPE_NLFIT_PRESET, "is-const", TRUE, NULL);
+    GwyNLFitPreset *preset = g_object_new(GWY_TYPE_NLFIT_PRESET,
+                                          "name", data->name,
+                                          "const", TRUE,
+                                          NULL);
     preset->builtin = data;
-    g_string_assign(GWY_RESOURCE(preset)->name, data->name);
 
     return preset;
 }
 
-void
-_gwy_nlfit_preset_class_setup_presets(void)
+static void
+resource_setup_builtins(GwyInventory *res_inventory)
 {
-    GwyResourceClass *klass;
-    GwyNLFitPreset *preset;
-    guint i;
-
-    /* Force class instantiation, this function is called before it's first
-     * referenced. */
-    klass = g_type_class_ref(GWY_TYPE_NLFIT_PRESET);
-
-    for (i = 0; i < G_N_ELEMENTS(fitting_presets); i++) {
-        preset = gwy_nlfit_preset_new_static(fitting_presets + i);
-        gwy_inventory_insert_item(klass->inventory, preset);
+    for (guint i = 0; i < G_N_ELEMENTS(fitting_presets); i++) {
+        GwyNLFitPreset *preset = create_nlfit_preset_builtin(fitting_presets + i);
+        gwy_inventory_insert_item(res_inventory, preset);
         g_object_unref(preset);
     }
-    gwy_inventory_restore_order(klass->inventory);
-
-    /* The presets added a reference so we can safely unref it again */
-    g_type_class_unref(klass);
+    gwy_inventory_restore_order(res_inventory);
 }
 
 /**
@@ -2034,8 +1966,7 @@ gwy_nlfit_preset_get_value(GwyNLFitPreset *preset,
                            gboolean *fres)
 {
     /* FIXME: builtin */
-    return preset->builtin->function(x, preset->builtin->nparams, params,
-                                     NULL, fres);
+    return preset->builtin->function(x, params, NULL, fres);
 }
 
 /**
@@ -2057,24 +1988,24 @@ gwy_nlfit_preset_get_formula(GwyNLFitPreset* preset)
 /**
  * gwy_nlfit_preset_get_param_name:
  * @preset: A NL fitter function preset.
- * @param: A parameter number.
+ * @i: Parameter index.
  *
  * Gets the name of a fitting parameter of a fitter preset.
  *
  * The name may contain Pango markup.
  *
- * Returns: The name of parameter @param.
+ * Returns: The name of @i-th parameter.
  **/
 const gchar*
 gwy_nlfit_preset_get_param_name(GwyNLFitPreset* preset,
-                                gint param)
+                                gint i)
 {
     const GwyNLFitParam *par;
 
     g_return_val_if_fail(GWY_IS_NLFIT_PRESET(preset), "");
     /* FIXME: builtin */
-    g_return_val_if_fail(param >= 0 && param < preset->builtin->nparams, NULL);
-    par = preset->builtin->param + param;
+    g_return_val_if_fail(i >= 0 && i < preset->builtin->nparams, NULL);
+    par = preset->builtin->param + i;
 
     return par->name;
 }
@@ -2082,29 +2013,73 @@ gwy_nlfit_preset_get_param_name(GwyNLFitPreset* preset,
 /**
  * gwy_nlfit_preset_get_param_units:
  * @preset: A NL fitter function preset.
- * @param: A parameter number.
+ * @i: Parameter index.
  * @siunit_x: SI unit of abscissa.
  * @siunit_y: SI unit of ordinate.
  *
  * Derives the SI unit of a fitting parameter from the units of abscissa and
  * ordinate.
  *
- * Returns: A newly created #GwySIUnit with the units of the parameter @param.
- *          If the units of @param are not representable as #GwySIUnit,
+ * Returns: A newly created #GwyUnit with the units of the @i-th parameter.
+ *          If the units are not representable as #GwyUnit,
  *          the result is unitless (i.e. it will be presented as a mere
  *          number).
- *
- * Since: 2.5
  **/
-GwySIUnit*
+GwyUnit*
 gwy_nlfit_preset_get_param_units(GwyNLFitPreset *preset,
-                                 gint param,
-                                 GwySIUnit *siunit_x,
-                                 GwySIUnit *siunit_y)
+                                 gint i,
+                                 GwyUnit *siunit_x,
+                                 GwyUnit *siunit_y)
 {
     g_return_val_if_fail(GWY_IS_NLFIT_PRESET(preset), NULL);
     /* FIXME: builtin */
-    return builtin_get_unit(preset, param, siunit_x, siunit_y);
+    return builtin_get_unit(preset, i, siunit_x, siunit_y);
+}
+
+/**
+ * gwy_nlfit_preset_get_param_power_x:
+ * @preset: A NL fitter function preset.
+ * @i: Parameter index.
+ *
+ * Gets the power of abscissa units in a fitting parameter.
+ *
+ * Usually it is easier to let gwy_nlfit_preset_get_param_units() derive the parameter units.
+ *
+ * Returns: The power of abscissa in the parameter.
+ **/
+gint
+gwy_nlfit_preset_get_param_power_x(GwyNLFitPreset *preset,
+                                   gint i)
+{
+    g_return_val_if_fail(GWY_IS_NLFIT_PRESET(preset), 0);
+
+    /* FIXME: builtin */
+    g_return_val_if_fail(i >= 0 && i < preset->builtin->nparams, 0);
+    const GwyNLFitParam *par = preset->builtin->param + i;
+    return par->power_x;
+}
+
+/**
+ * gwy_nlfit_preset_get_param_power_y:
+ * @preset: A NL fitter function preset.
+ * @i: Parameter index.
+ *
+ * Gets the power of ordinate units in a fitting parameter.
+ *
+ * Usually it is easier to let gwy_nlfit_preset_get_param_units() derive the parameter units.
+ *
+ * Returns: The power of ordinate in the parameter.
+ **/
+gint
+gwy_nlfit_preset_get_param_power_y(GwyNLFitPreset *preset,
+                                   gint i)
+{
+    g_return_val_if_fail(GWY_IS_NLFIT_PRESET(preset), 0);
+
+    /* FIXME: builtin */
+    g_return_val_if_fail(i >= 0 && i < preset->builtin->nparams, 0);
+    const GwyNLFitParam *par = preset->builtin->param + i;
+    return par->power_y;
 }
 
 /**
@@ -2168,18 +2143,13 @@ gwy_nlfit_preset_guess(GwyNLFitPreset *preset,
  * If you do not need to modify the fitter settings you can use
  * gwy_nlfit_preset_fit() directly with %NULL fitter.
  *
- * Returns: A newly created fitter for @preset.
- *
- * Since: 2.47
+ * Returns: (transfer none): A newly created fitter for @preset.
  **/
 GwyNLFitter*
 gwy_nlfit_preset_create_fitter(GwyNLFitPreset *preset)
 {
     g_return_val_if_fail(GWY_IS_NLFIT_PRESET(preset), NULL);
-    return gwy_math_nlfit_new(preset->builtin->function,
-                              preset->builtin->derive
-                              ? preset->builtin->derive
-                              : gwy_math_nlfit_derive);
+    return gwy_math_nlfit_new(preset->builtin->function, preset->builtin->diff);
 }
 
 /**
@@ -2220,18 +2190,15 @@ gwy_nlfit_preset_fit(GwyNLFitPreset *preset,
 
     g_return_val_if_fail(GWY_IS_NLFIT_PRESET(preset), NULL);
     /* FIXME: builtin */
-    /*use numerical derivation if necessary*/
+    /* Use numeric differentiation if necessary. */
     if (fitter) {
         /* XXX */
         g_return_val_if_fail(fitter->fmarq == preset->builtin->function, NULL);
     }
     else
-        fitter = gwy_math_nlfit_new(preset->builtin->function,
-                                    preset->builtin->derive
-                                    ? preset->builtin->derive
-                                    : gwy_math_nlfit_derive);
+        fitter = gwy_math_nlfit_new(preset->builtin->function, preset->builtin->diff);
 
-    /*load default weights for given function type*/
+    /* Load default weights for given function type. */
     if (preset->builtin->set_default_weights) {
         weight = g_new(gdouble, n_dat);
         preset->builtin->set_default_weights(n_dat, x, y, weight);
@@ -2258,37 +2225,31 @@ gwy_nlfit_preset_fit(GwyNLFitPreset *preset,
  *
  * Gets inventory with all the NLFit presets.
  *
- * Returns: NLFit preset inventory.
+ * Returns: (transfer none): NLFit preset inventory.
  **/
 GwyInventory*
 gwy_nlfit_presets(void)
 {
-    return
-        GWY_RESOURCE_CLASS(g_type_class_peek(GWY_TYPE_NLFIT_PRESET))->inventory;
+    return GWY_RESOURCE_CLASS(g_type_class_peek(GWY_TYPE_NLFIT_PRESET))->inventory;
 }
 
-/************************** Documentation ****************************/
-
 /**
- * SECTION:gwynlfitpreset
+ * SECTION: gwynlfitpreset
  * @title: GwyNLFitPreset
  * @short_description: NL fitter preset functions
  * @see_also: #GwyNLFitter
  *
- * <link linkend="GwyNLFitter">Non-linear fitter</link> presets are predefined
- * fitting functions, with guessing, weighting, etc. to ease fitting of
- * common functions.  Most of those currently implemented are related to
- * one-dimensional statistical characteristics of randomly rough surfaces,
- * however, several general-purpose fitting functions are also available.
+ * <link linkend="GwyNLFitter">Non-linear fitter</link> presets are predefined fitting functions, with guessing,
+ * weighting, etc. to ease fitting of common functions.  Most of those currently implemented are related to
+ * one-dimensional statistical characteristics of randomly rough surfaces, however, several general-purpose fitting
+ * functions are also available.
  *
- * The presets are identified by name and can be obtained from the
- * corresponding #GwyInventory, see gwy_nlfit_presets().  All their properties
- * can be queried with methods such as gwy_nlfit_preset_get_nparams() or
- * gwy_nlfit_preset_get_formula().  A generic method to derive units of
- * fitting parameters from units of fitted data is also available:
- * gwy_nlfit_preset_get_param_units().
+ * The presets are identified by name and can be obtained from the corresponding #GwyInventory, see
+ * gwy_nlfit_presets().  All their properties can be queried with methods such as gwy_nlfit_preset_get_nparams() or
+ * gwy_nlfit_preset_get_formula().  A generic method to derive units of fitting parameters from units of fitted data
+ * is also available: gwy_nlfit_preset_get_param_units().
  *
- * As of version 2.7 the defined functions include:
+ * The defined functions include:
  * <simplelist type='vert'>
  * <member><literal>"Gaussian"</literal></member>
  * <member><literal>"Gaussian (PSDF)"</literal></member>
@@ -2308,27 +2269,11 @@ gwy_nlfit_presets(void)
  * <member><literal>"Power"</literal></member>
  * <member><literal>"Lorentzian"</literal></member>
  * <member><literal>"Sinc"</literal></member>
- * </simplelist>
- *
- * The following presets are available since version 2.40:
- * <simplelist type='vert'>
  * <member><literal>"Parabolic step"</literal></member>
  * <member><literal>"Two Gaussians (PSDF)"</literal></member>
- * </simplelist>
- *
- * The following presets are available since version 2.41:
- * <simplelist type='vert'>
  * <member><literal>"Smooth bent step"</literal></member>
  * <member><literal>"Smooth slanted step"</literal></member>
- * </simplelist>
- *
- * The following presets are available since version 2.48:
- * <simplelist type='vert'>
  * <member><literal>"K-correlated (PSDF)"</literal></member>
- * </simplelist>
- *
- * The following presets are available since version 2.49:
- * <simplelist type='vert'>
  * <member><literal>"Boltzmann bent step"</literal></member>
  * </simplelist>
  *
@@ -2345,9 +2290,7 @@ gwy_nlfit_presets(void)
  * errors = g_new(gdouble, gwy_nlfit_preset_get_nparams(preset));
  * /&ast; Use a priori parameter value information, or: &ast;/
  * gwy_nlfit_preset_guess(preset, ndata, xdata, ydata, params, &ok);
- * fitter = gwy_nlfit_preset_fit(preset, NULL,
- *                               ndata, xdata, ydata, params,
- *                               params, errors, NULL);
+ * fitter = gwy_nlfit_preset_fit(preset, NULL, ndata, xdata, ydata, params, params, errors, NULL);
  *
  * /&ast; Examine fitting status, parameters and errors... &ast;/
  *
@@ -2357,4 +2300,4 @@ gwy_nlfit_presets(void)
  * ]|
  **/
 
-/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
+/* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
