@@ -1,5 +1,5 @@
 /*
- *  $Id: cmap_align.c 26949 2024-12-03 10:42:17Z yeti-dn $
+ *  $Id: cmap_align.c 28905 2025-11-24 15:51:18Z yeti-dn $
  *  Copyright (C) 2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,22 +16,11 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwythreads.h>
-#include <libprocess/lawn.h>
-#include <libprocess/stats.h>
-#include <libprocess/correct.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libgwydgets/gwydataview.h>
-#include <libgwydgets/gwygraph.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-cmap.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
-#include "libgwyddion/gwyomp.h"
+#include <gwy.h>
+#include "libgwyddion/omp.h"
 
 #define RUN_MODES (GWY_RUN_INTERACTIVE)
 
@@ -67,8 +56,8 @@ typedef struct {
     GwyParams *params;
     GwyLawn *lawn;
     GwyLawn *result;
-    GwyDataField *field;
-    GwyDataField *mask;
+    GwyField *field;
+    GwyField *mask;
     /* Cached input data properties. */
     gint nsegments;
 } ModuleArgs;
@@ -78,7 +67,6 @@ typedef struct {
     GtkWidget *dialog;
     GwyParamTable *table;
     GwyParamTable *table_output;
-    GwyContainer *data;
     GwySelection *selection;
     GwySelection *graph_selection;
     GwyGraphModel *gmodel;
@@ -86,11 +74,11 @@ typedef struct {
 
 static gboolean         module_register         (void);
 static GwyParamDef*     define_module_params    (void);
-static void             align                   (GwyContainer *data,
-                                                 GwyRunType runtype);
+static void             module_main             (GwyFile *data,
+                                                 GwyRunModeFlags mode);
 static void             execute                 (ModuleArgs *args);
 static GwyDialogOutcome run_gui                 (ModuleArgs *args,
-                                                 GwyContainer *data,
+                                                 GwyFile *data,
                                                  gint id);
 static void             param_changed           (ModuleGUI *gui,
                                                  gint id);
@@ -133,11 +121,11 @@ static gboolean
 module_register(void)
 {
     gwy_curve_map_func_register("cmap_align",
-                                (GwyCurveMapFunc)&align,
+                                module_main,
                                 N_("/_Align..."),
                                 NULL,
                                 RUN_MODES,
-                                GWY_MENU_FLAG_CURVE_MAP,
+                                GWY_MENU_FLAG_CMAP,
                                 N_("Align curves and extract topography"));
 
     return TRUE;
@@ -178,38 +166,31 @@ define_module_params(void)
 }
 
 static void
-align(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
     GwyLawn *lawn = NULL;
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     AlignOutput output;
-    GwyDataField *field;
+    GwyField *field;
     gint id, newid;
-    const guchar *gradient;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    g_return_if_fail(g_type_from_name("GwyLayerPoint"));
+    g_return_if_fail(mode & RUN_MODES);
 
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_LAWN, &lawn,
-                                     GWY_APP_LAWN_ID, &id,
-                                     0);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_LAWN, &lawn,
+                                 GWY_APP_LAWN_ID, &id,
+                                 0);
     g_return_if_fail(GWY_IS_LAWN(lawn));
     args.lawn = lawn;
     args.nsegments = gwy_lawn_get_n_segments(lawn);
     args.params = gwy_params_new_from_settings(define_module_params());
     sanitise_params(&args);
 
-    args.field = gwy_data_field_new(gwy_lawn_get_xres(lawn), gwy_lawn_get_yres(lawn),
-                                     gwy_lawn_get_xreal(lawn), gwy_lawn_get_yreal(lawn), TRUE);
-    gwy_data_field_set_xoffset(args.field, gwy_lawn_get_xoffset(lawn));
-    gwy_data_field_set_yoffset(args.field, gwy_lawn_get_yoffset(lawn));
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(args.field), gwy_lawn_get_si_unit_xy(lawn));
-    args.mask = gwy_data_field_new_alike(args.field, TRUE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(args.mask), NULL);
+    args.field = gwy_lawn_new_field_alike(lawn, -1, TRUE);
+    args.mask = gwy_field_new_alike(args.field, TRUE);
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, id);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -220,69 +201,62 @@ align(GwyContainer *data, GwyRunType runtype)
 
     output = gwy_params_get_flags(args.params, PARAM_OUTPUT);
     if (output & (1 << OUTPUT_PREVIEW)) {
-        field = gwy_container_get_object(data, gwy_app_get_lawn_preview_key_for_id(id));
-        gwy_data_field_assign(field, args.field);
-        gwy_data_field_data_changed(field);
+        field = gwy_container_get_object(GWY_CONTAINER(data), gwy_file_key_cmap_picture(id));
+        gwy_field_assign(field, args.field);
+        gwy_field_data_changed(field);
     }
     if (output & (1 << OUTPUT_TOPOGRAPHY)) {
-        newid = gwy_app_data_browser_add_data_field(args.field, data, TRUE);
-        gwy_app_set_data_field_title(data, newid, _("Topography"));
-        if (gwy_data_field_get_max(args.mask) > 0.0)
-            gwy_container_set_object(data, gwy_app_get_mask_key_for_id(newid), args.mask);
+        newid = gwy_file_add_image(data, args.field);
+        gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+        gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _("Topography"), TRUE);
+        if (gwy_field_get_max(args.mask) > 0.0)
+            gwy_file_pass_image_mask(data, gwy_file_key_image_mask(newid), args.mask);
+        else
+            g_clear_object(&args.mask);
 
-        if (gwy_container_gis_string(data, gwy_app_get_lawn_palette_key_for_id(id), &gradient)) {
-            gwy_container_set_const_string(data,
-                                           gwy_app_get_data_palette_key_for_id(newid), gradient);
-        }
+        gwy_file_sync_items(data, GWY_FILE_CMAP, id,
+                            data, GWY_FILE_IMAGE, newid,
+                            GWY_FILE_ITEM_PALETTE, FALSE);
 
-        gwy_app_channel_log_add(data, -1, newid, "cmap::cmap_align", NULL);
+        gwy_log_add_full(data, GWY_FILE_IMAGE, -1, newid, "cmap::cmap_align", NULL);
     }
     if (output & (1 << OUTPUT_ALIGN)) {
         gwy_lawn_data_changed(lawn);
-        gwy_app_curve_map_log_add_curve_map(data, id, id);
+        gwy_log_add(data, GWY_FILE_CMAP, id, id);
     }
 
 end:
-    GWY_OBJECT_UNREF(args.result);
+    g_clear_object(&args.result);
     g_object_unref(args.mask);
     g_object_unref(args.field);
     g_object_unref(args.params);
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
-    GtkWidget *hbox, *graph, *dataview, *align, *area;
+    GtkWidget *hbox, *graph, *dataview, *area;
     GwyParamTable *table;
     GwyDialog *dialog;
     ModuleGUI gui;
-    GwyDataField *field;
+    GwyField *field;
     GwyDialogOutcome outcome;
     GwyGraphCurveModel *gcmodel;
-    const guchar *gradient;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
-    gui.data = gwy_container_new();
-    field = gwy_container_get_object(data, gwy_app_get_lawn_preview_key_for_id(id));
-    gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(0), field);
-    if (gwy_container_gis_string(data, gwy_app_get_lawn_palette_key_for_id(id), &gradient))
-        gwy_container_set_const_string(gui.data, gwy_app_get_data_palette_key_for_id(0), gradient);
 
     gui.dialog = gwy_dialog_new(_("Align Map Curves"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    hbox = gwy_hbox_new(0);
-    gwy_dialog_add_content(GWY_DIALOG(gui.dialog), hbox, TRUE, TRUE, 0);
-
-    align = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-    gtk_box_pack_start(GTK_BOX(hbox), align, FALSE, FALSE, 0);
-
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
-    gtk_container_add(GTK_CONTAINER(align), dataview);
-    gui.selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(dataview), 0, "Point", 1, TRUE);
+    field = gwy_container_get_object(GWY_CONTAINER(data), gwy_file_key_cmap_picture(id));
+    dataview = gwy_create_preview(field, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_CMAP, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_REAL_SQUARE);
+    gui.selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(dataview), GWY_TYPE_LAYER_POINT, 1, TRUE);
     set_selection(&gui);
+    hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), FALSE);
 
     gui.gmodel = gwy_graph_model_new();
     gcmodel = gwy_graph_curve_model_new();
@@ -302,7 +276,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gtk_widget_set_size_request(graph, PREVIEW_SIZE, PREVIEW_SIZE);
     gtk_box_pack_start(GTK_BOX(hbox), graph, TRUE, TRUE, 0);
 
-    hbox = gwy_hbox_new(20);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 20);
     gwy_dialog_add_content(GWY_DIALOG(gui.dialog), hbox, TRUE, TRUE, 4);
 
     table = gui.table = gwy_param_table_new(args->params);
@@ -329,7 +303,6 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     outcome = gwy_dialog_run(dialog);
 
     g_object_unref(gui.gmodel);
-    g_object_unref(gui.data);
 
     return outcome;
 }
@@ -413,15 +386,15 @@ execute(ModuleArgs *args)
     AlignMethod method = gwy_params_get_enum(params, PARAM_METHOD);
     AlignOutput output = gwy_params_get_flags(params, PARAM_OUTPUT);
     GwyLawn *lawn = args->lawn;
-    GwyDataField *field = args->field, *mask = args->mask;
+    GwyField *field = args->field, *mask = args->mask;
     gint xres = gwy_lawn_get_xres(lawn), yres = gwy_lawn_get_yres(lawn);
     gdouble *data, *mdata;
 
     /* First locate the point in all data.  This allows interpolating the failed ones. */
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_z(field), gwy_lawn_get_si_unit_curve(lawn, abscissa));
-    gwy_data_field_clear(mask);
-    data = gwy_data_field_get_data(field);
-    mdata = gwy_data_field_get_data(mask);
+    gwy_unit_assign(gwy_field_get_unit_z(field), gwy_lawn_get_unit_curve(lawn, abscissa));
+    gwy_field_clear(mask);
+    data = gwy_field_get_data(field);
+    mdata = gwy_field_get_data(mask);
 
 #ifdef _OPENMP
 #pragma omp parallel if(gwy_threads_are_enabled()) default(none) \
@@ -443,15 +416,15 @@ execute(ModuleArgs *args)
         }
     }
 
-    if (gwy_data_field_get_max(mask) > 0.0)
-        gwy_data_field_laplace_solve(field, mask, -1, 1.0);
+    if (gwy_field_get_max(mask) > 0.0)
+        gwy_field_laplace_solve(field, mask, -1, 1.0);
 
     if (output & (1 << OUTPUT_ALIGN)) {
         gint ndata, k, m, col, row;
         gdouble *zcd;
         gdouble x;
 
-        data = gwy_data_field_get_data(field);
+        data = gwy_field_get_data(field);
         for (k = 0; k < xres*yres; k++) {
             col = k % xres;
             row = k/xres;
@@ -644,17 +617,17 @@ update_graph_model_props(GwyGraphModel *gmodel, ModuleArgs *args)
     GwyParams *params = args->params;
     gint abscissa = gwy_params_get_int(params, PARAM_ABSCISSA);
     gint ordinate = gwy_params_get_int(params, PARAM_ORDINATE);
-    GwySIUnit *xunit, *yunit;
+    GwyUnit *xunit, *yunit;
     const gchar *xlabel, *ylabel;
 
-    xunit = gwy_lawn_get_si_unit_curve(lawn, abscissa);
+    xunit = gwy_lawn_get_unit_curve(lawn, abscissa);
     xlabel = gwy_lawn_get_curve_label(lawn, abscissa);
-    yunit = gwy_lawn_get_si_unit_curve(lawn, ordinate);
+    yunit = gwy_lawn_get_unit_curve(lawn, ordinate);
     ylabel = gwy_lawn_get_curve_label(lawn, ordinate);
 
     g_object_set(gmodel,
-                 "si-unit-x", xunit,
-                 "si-unit-y", yunit,
+                 "unit-x", xunit,
+                 "unit-y", yunit,
                  "axis-label-bottom", xlabel ? xlabel : _("Untitled"),
                  "axis-label-left", ylabel ? ylabel : _("Untitled"),
                  "label-visible", FALSE,
