@@ -1,5 +1,5 @@
 /*
- *  $Id: cmap_crop.c 25639 2023-09-07 12:23:42Z yeti-dn $
+ *  $Id: cmap_crop.c 28860 2025-11-14 12:59:38Z yeti-dn $
  *  Copyright (C) 2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,16 +16,11 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/lawn.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-cmap.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "../process/preview.h"
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
@@ -42,7 +37,7 @@ typedef struct {
     GwyParams *params;
     GwyLawn *lawn;
     GwyLawn *result;
-    GwyDataField *preview;
+    GwyField *preview;
     /* Cached values for dimensions display. */
     gint xres;
     gint yres;
@@ -52,17 +47,16 @@ typedef struct {
     ModuleArgs *args;
     GtkWidget *dialog;
     GwyParamTable *table;
-    GwyContainer *data;
     GtkWidget *view;
     gint last_active;
 } ModuleGUI;
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             crop                (GwyContainer *data,
-                                             GwyRunType runtype);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static GwyDialogOutcome run_gui             (ModuleArgs *args,
-                                             GwyContainer *data,
+                                             GwyFile *data,
                                              gint id);
 static void             execute             (ModuleArgs *args);
 static void             param_changed       (ModuleGUI *gui,
@@ -86,11 +80,11 @@ static gboolean
 module_register(void)
 {
     gwy_curve_map_func_register("cmap_crop",
-                                (GwyCurveMapFunc)&crop,
+                                module_main,
                                 N_("/_Basic Operations/_Crop"),
-                                GWY_STOCK_CROP,
+                                GWY_ICON_CROP,
                                 RUN_MODES,
-                                GWY_MENU_FLAG_CURVE_MAP,
+                                GWY_MENU_FLAG_CMAP,
                                 N_("Crop data"));
 
     return TRUE;
@@ -115,7 +109,7 @@ define_module_params(void)
 }
 
 static void
-crop(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GQuark quarks[2];
     GwyParams *params;
@@ -123,22 +117,22 @@ crop(GwyContainer *data, GwyRunType runtype)
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     gint id, newid;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_LAWN, &args.lawn,
-                                     GWY_APP_LAWN_KEY, quarks,
-                                     GWY_APP_LAWN_ID, &id,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_LAWN, &args.lawn,
+                                 GWY_APP_LAWN_KEY, quarks,
+                                 GWY_APP_LAWN_ID, &id,
+                                 0);
     g_return_if_fail(args.lawn);
 
     args.xres = gwy_lawn_get_xres(args.lawn);
     args.yres = gwy_lawn_get_yres(args.lawn);
     args.params = params = gwy_params_new_from_settings(define_module_params());
-    args.preview = gwy_container_get_object(data, gwy_app_get_lawn_preview_key_for_id(id));
+    args.preview = gwy_container_get_object(GWY_CONTAINER(data), gwy_file_key_cmap_picture(id));
     g_object_ref(args.preview);
     sanitise_params(&args);
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, id);
         gwy_params_save_to_settings(params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -147,52 +141,44 @@ crop(GwyContainer *data, GwyRunType runtype)
 
     execute(&args);
 
-    newid = gwy_app_data_browser_add_lawn(args.result, args.preview, data, TRUE);
+    newid = gwy_file_add_cmap(data, args.result, args.preview);
+
+    gwy_file_set_visible(data, GWY_FILE_CMAP, newid, TRUE);
     g_object_unref(args.preview);
-    gwy_app_sync_curve_map_items(data, data, id, newid, FALSE,
-                                 GWY_DATA_ITEM_GRADIENT,
-                                 GWY_DATA_ITEM_REAL_SQUARE,
-                                 0);
-    gwy_app_set_lawn_title(data, newid, _("Detail"));
-    gwy_app_curve_map_log_add_curve_map(data, id, newid);
+    gwy_file_sync_items(data, GWY_FILE_CMAP, id, data, GWY_FILE_CMAP, newid,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_REAL_SQUARE, FALSE);
+    gwy_file_set_title(data, GWY_FILE_CMAP, newid, _("Detail"), TRUE);
+    gwy_log_add(data, GWY_FILE_CMAP, id, newid);
 
 end:
     g_object_unref(params);
-    GWY_OBJECT_UNREF(args.result);
+    g_clear_object(&args.result);
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     static const gint ids[] = { PARAM_X, PARAM_Y, PARAM_WIDTH, PARAM_HEIGHT };
     GwyDialogOutcome outcome;
-    GwyDataField *previewfield;
+    GwyField *field;
     GtkWidget *hbox;
     GwyDialog *dialog;
     GwyParamTable *table;
     ModuleGUI gui;
-    const gchar *gradient;
-    gboolean realsquare;
     guint i;
     gint parid;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
-    gui.data = gwy_container_new();
-
-    previewfield = gwy_data_field_duplicate(args->preview);
-    gwy_container_pass_object(gui.data, gwy_app_get_data_key_for_id(0), previewfield);
-    /* XXX: Manual cross-sync between Lawn and Field. */
-    if (gwy_container_gis_string(data, gwy_app_get_lawn_palette_key_for_id(id), (const guchar**)&gradient))
-        gwy_container_set_const_string(gui.data, gwy_app_get_data_palette_key_for_id(0), gradient);
-    if (gwy_container_gis_boolean(data, gwy_app_get_lawn_real_square_key_for_id(id), &realsquare))
-        gwy_container_set_boolean(gui.data, gwy_app_get_data_real_square_key_for_id(0), realsquare);
 
     gui.dialog = gwy_dialog_new(_("Crop"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    gui.view = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
+    field = gwy_field_copy(args->preview);
+    gui.view = gwy_create_preview(field, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(gui.view), data, GWY_FILE_CMAP, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_REAL_SQUARE);
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(gui.view), FALSE);
 
     table = gui.table = gwy_param_table_new(args->params);
@@ -221,7 +207,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 
     outcome = gwy_dialog_run(dialog);
 
-    g_object_unref(gui.data);
+    g_object_unref(field);
 
     return outcome;
 }
@@ -262,7 +248,7 @@ execute(ModuleArgs *args)
 {
     GwyParams *params = args->params;
     GwyLawn *lawn = args->lawn, *result;
-    GwyDataField *previewfield;
+    GwyField *previewfield;
     gint x = gwy_params_get_int(params, PARAM_X);
     gint y = gwy_params_get_int(params, PARAM_Y);
     gint width = gwy_params_get_int(params, PARAM_WIDTH);
@@ -270,10 +256,10 @@ execute(ModuleArgs *args)
     gboolean keep_offsets = gwy_params_get_boolean(params, PARAM_KEEP_OFFSETS);
 
     args->result = result = gwy_lawn_new_part(lawn, x, y, width, height, keep_offsets);
-    previewfield = gwy_data_field_area_extract(args->preview, x, y, width, height);
+    previewfield = gwy_field_area_extract(args->preview, x, y, width, height);
     if (keep_offsets) {
-        gwy_data_field_set_xoffset(previewfield, gwy_lawn_get_dx(lawn)*x + gwy_lawn_get_xoffset(lawn));
-        gwy_data_field_set_yoffset(previewfield, gwy_lawn_get_dy(lawn)*y + gwy_lawn_get_yoffset(lawn));
+        gwy_field_set_xoffset(previewfield, gwy_lawn_get_dx(lawn)*x + gwy_lawn_get_xoffset(lawn));
+        gwy_field_set_yoffset(previewfield, gwy_lawn_get_dy(lawn)*y + gwy_lawn_get_yoffset(lawn));
     }
 
     /* NB: This changes whether we hold a reference on args->preview (from no to yes). */
@@ -286,17 +272,17 @@ preview(gpointer user_data)
     ModuleGUI *gui = (ModuleGUI*)user_data;
     ModuleArgs *args = gui->args;
     GwyParams *params = args->params;
-    GwyDataField *field = args->preview;
-    GwyDataField *result = gwy_container_get_object(gui->data, gwy_app_get_data_key_for_id(0));
+    GwyField *field = args->preview;
+    GwyField *result = gwy_data_view_get_field(GWY_DATA_VIEW(gui->view));
     gint x = gwy_params_get_int(params, PARAM_X);
     gint y = gwy_params_get_int(params, PARAM_Y);
     gint width = gwy_params_get_int(params, PARAM_WIDTH);
     gint height = gwy_params_get_int(params, PARAM_HEIGHT);
 
     /* Crop just the preview image.  Do not bother with lawn here. */
-    gwy_data_field_resample(result, width, height, GWY_INTERPOLATION_NONE);
-    gwy_data_field_area_copy(field, result, x, y, width, height, 0, 0);
-    gwy_data_field_data_changed(result);
+    gwy_field_resize(result, width, height);
+    gwy_field_area_copy(field, result, x, y, width, height, 0, 0);
+    gwy_field_data_changed(result);
     gwy_set_data_preview_size(GWY_DATA_VIEW(gui->view), PREVIEW_SIZE);
 }
 
