@@ -1,5 +1,5 @@
 /*
- *  $Id: cmap_cutter.c 28036 2025-05-28 20:21:11Z klapetek $
+ *  $Id: cmap_cutter.c 28905 2025-11-24 15:51:18Z yeti-dn $
  *  Copyright (C) 2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,21 +16,11 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwynlfit.h>
-#include <libgwyddion/gwythreads.h>
-#include <libprocess/lawn.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libgwydgets/gwydataview.h>
-#include <libgwydgets/gwygraph.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-cmap.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
-#include "libgwyddion/gwyomp.h"
+#include <gwy.h>
+#include "libgwyddion/omp.h"
 
 #define RUN_MODES (GWY_RUN_INTERACTIVE)
 
@@ -97,7 +87,6 @@ typedef struct {
     GwyParamTable *table_mode;
     GSList *keep_curves;
     GtkWidget *keep_curves_label;
-    GwyContainer *data;
     GwySelection *selection;
     GwyGraphModel *gmodel;
     GwyEnum *curve_enum;
@@ -105,12 +94,12 @@ typedef struct {
 
 static gboolean         module_register         (void);
 static GwyParamDef*     define_module_params    (void);
-static void             cmap_cutter             (GwyContainer *data,
-                                                 GwyRunType runtype);
+static void             module_main             (GwyFile *data,
+                                                 GwyRunModeFlags mode);
 static gboolean         execute                 (ModuleArgs *args,
                                                  GtkWindow *wait_window);
 static GwyDialogOutcome run_gui                 (ModuleArgs *args,
-                                                 GwyContainer *data,
+                                                 GwyFile *data,
                                                  gint id);
 static GtkWidget*       create_keep_curves      (ModuleGUI *gui);
 static void             param_changed           (ModuleGUI *gui,
@@ -162,11 +151,11 @@ static gboolean
 module_register(void)
 {
     gwy_curve_map_func_register("cmap_cutter",
-                                (GwyCurveMapFunc)&cmap_cutter,
+                                module_main,
                                 N_("/_Cut to Segments..."),
                                 NULL,
                                 RUN_MODES,
-                                GWY_MENU_FLAG_CURVE_MAP,
+                                GWY_MENU_FLAG_CMAP,
                                 N_("Cut curves to segments"));
 
     return TRUE;
@@ -206,7 +195,7 @@ define_module_params(void)
 }
 
 static void
-cmap_cutter(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
     GwyLawn *lawn = NULL;
@@ -214,40 +203,40 @@ cmap_cutter(GwyContainer *data, GwyRunType runtype)
     gint i, oldid, newid;
     CutterOutput output;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    g_return_if_fail(g_type_from_name("GwyLayerPoint"));
+    g_return_if_fail(mode & RUN_MODES);
 
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_LAWN, &lawn,
-                                     GWY_APP_LAWN_ID, &oldid,
-                                     0);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_LAWN, &lawn,
+                                 GWY_APP_LAWN_ID, &oldid,
+                                 0);
     g_return_if_fail(GWY_IS_LAWN(lawn));
     args.lawn = lawn;
     args.params = gwy_params_new_from_settings(define_module_params());
     sanitise_params(&args);
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, oldid);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
             goto end;
     }
-    if (execute(&args, gwy_app_find_window_for_curve_map(data, oldid))) {
+    if (execute(&args, gwy_data_browser_get_window_for_data(data, GWY_FILE_CMAP, oldid))) {
         output = gwy_params_get_enum(args.params, PARAM_OUTPUT);
         if (output == OUTPUT_EXTRACT) {
             for (i = 0; i < args.nresults; i++) {
                 if (!(lawn = args.result[i]))
                     continue;
 
-                newid = gwy_app_data_browser_add_lawn(lawn, NULL, data, TRUE);
-                gwy_container_set_const_string(data, gwy_app_get_lawn_title_key_for_id(newid), args.segnames[i].name);
+                newid = gwy_file_add_cmap(data, lawn, NULL);
+                gwy_file_set_visible(data, GWY_FILE_CMAP, newid, TRUE);
+                gwy_file_set_title(data, GWY_FILE_CMAP, newid, args.segnames[i].name, FALSE);
             }
             for (i = 0; i < args.nresults; i++)
-                GWY_OBJECT_UNREF(args.result[i]);
+                g_clear_object(args.result + i);
             g_free(args.result);
         }
         else if (output == OUTPUT_MARK) {
-            gwy_app_curve_map_log_add_curve_map(data, oldid, oldid);
+            gwy_log_add(data, GWY_FILE_CMAP, oldid, oldid);
         }
     }
 
@@ -256,46 +245,38 @@ end:
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
-    GtkWidget *hbox, *graph, *dataview, *align, *kclist;
+    GtkWidget *hbox, *graph, *dataview, *kclist;
     GwyParamTable *table;
     GwyDialog *dialog;
     ModuleGUI gui;
-    GwyDataField *field;
+    GwyField *field;
     GwyDialogOutcome outcome;
-    const guchar *gradient;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
-    gui.data = gwy_container_new();
+
     gui.gmodel = gwy_graph_model_new();
-    field = gwy_container_get_object(data, gwy_app_get_lawn_preview_key_for_id(id));
-    gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(0), field);
-    if (gwy_container_gis_string(data, gwy_app_get_lawn_palette_key_for_id(id), &gradient))
-        gwy_container_set_const_string(gui.data, gwy_app_get_data_palette_key_for_id(0), gradient);
 
     gui.dialog = gwy_dialog_new(_("Cut to Segments"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    hbox = gwy_hbox_new(0);
-    gwy_dialog_add_content(GWY_DIALOG(gui.dialog), hbox, TRUE, TRUE, 0);
-
-    align = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-    gtk_box_pack_start(GTK_BOX(hbox), align, FALSE, FALSE, 0);
-
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
-    gtk_container_add(GTK_CONTAINER(align), dataview);
-    gui.selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(dataview), 0, "Point", 1, TRUE);
+    field = gwy_container_get_object(GWY_CONTAINER(data), gwy_file_key_cmap_picture(id));
+    dataview = gwy_create_preview(field, FALSE, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_CMAP, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_REAL_SQUARE);
+    gui.selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(dataview), GWY_TYPE_LAYER_POINT, 1, TRUE);
     set_selection(&gui);
+    hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), FALSE);
 
     graph = gwy_graph_new(gui.gmodel);
     gwy_graph_enable_user_input(GWY_GRAPH(graph), FALSE);
     gtk_widget_set_size_request(graph, PREVIEW_SIZE, PREVIEW_SIZE);
     gtk_box_pack_start(GTK_BOX(hbox), graph, TRUE, TRUE, 0);
 
-    hbox = gwy_hbox_new(20);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 20);
     gwy_dialog_add_content(GWY_DIALOG(gui.dialog), hbox, TRUE, TRUE, 4);
 
     table = gui.table = gwy_param_table_new(args->params);
@@ -322,7 +303,6 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     outcome = gwy_dialog_run(dialog);
 
     g_object_unref(gui.gmodel);
-    g_object_unref(gui.data);
     g_free(gui.curve_enum);
 
     return outcome;
@@ -331,7 +311,6 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 static GtkWidget*
 create_keep_curves(ModuleGUI *gui)
 {
-    GtkTable *table;
     GwyLawn *lawn = gui->args->lawn;
     gint keep_curves = gwy_params_get_int(gui->args->params, PARAM_KEEP_CURVES);
     const gchar *name;
@@ -347,16 +326,16 @@ create_keep_curves(ModuleGUI *gui)
     gui->keep_curves = gwy_check_boxes_create(gui->curve_enum, ncurves, G_CALLBACK(keep_curves_changed), gui,
                                               keep_curves);
 
-    table = GTK_TABLE(gtk_table_new(ncurves+1, 2, FALSE));
-    gtk_container_set_border_width(GTK_CONTAINER(table), 4);
-    gtk_table_set_row_spacings(table, 2);
-    gtk_table_set_col_spacings(table, 6);
+    GtkGrid *grid = GTK_GRID(gtk_grid_new());
+    gtk_container_set_border_width(GTK_CONTAINER(grid), 4);
+    gtk_grid_set_row_spacing(grid, 2);
+    gtk_grid_set_column_spacing(grid, 6);
     gui->keep_curves_label = gtk_label_new(_("Keep curves:"));
-    gtk_misc_set_alignment(GTK_MISC(gui->keep_curves_label), 0.0, 0.5);
-    gtk_table_attach(table, gui->keep_curves_label, 0, 2, 0, 1, 0, 0, 0, 0);
-    gwy_check_boxes_attach_to_table(gui->keep_curves, table, 2, 1);
+    gtk_label_set_xalign(GTK_LABEL(gui->keep_curves_label), 0.0);
+    gtk_grid_attach(grid, gui->keep_curves_label, 0, 0, 2, 1);
+    gwy_check_boxes_attach_to_grid(gui->keep_curves, grid, 2, 1);
 
-    return GTK_WIDGET(table);
+    return GTK_WIDGET(grid);
 }
 
 static void
@@ -480,10 +459,10 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
     result = gwy_lawn_new(xres, yres, gwy_lawn_get_xreal(lawn), gwy_lawn_get_yreal(lawn), nc, 0);
     gwy_lawn_set_xoffset(result, gwy_lawn_get_xoffset(lawn));
     gwy_lawn_set_yoffset(result, gwy_lawn_get_yoffset(lawn));
-    gwy_si_unit_assign(gwy_lawn_get_si_unit_xy(result), gwy_lawn_get_si_unit_xy(lawn));
+    gwy_unit_assign(gwy_lawn_get_unit_xy(result), gwy_lawn_get_unit_xy(lawn));
     for (m = n = 0; m < ncurves; m++) {
         if (keep_curves & (1 << m)) {
-            gwy_si_unit_assign(gwy_lawn_get_si_unit_curve(result, n), gwy_lawn_get_si_unit_curve(lawn, m));
+            gwy_unit_assign(gwy_lawn_get_unit_curve(result, n), gwy_lawn_get_unit_curve(lawn, m));
             gwy_lawn_set_curve_label(result, n, gwy_lawn_get_curve_label(lawn, m));
             n++;
         }
@@ -657,13 +636,13 @@ update_graph_model_props(ModuleGUI *gui)
     GwyParams *params = args->params;
     gint curve = gwy_params_get_int(params, PARAM_CURVE);
     GwyGraphModel *gmodel = gui->gmodel;
-    GwySIUnit *yunit;
+    GwyUnit *yunit;
     const gchar *ylabel;
 
-    yunit = gwy_lawn_get_si_unit_curve(lawn, curve);
+    yunit = gwy_lawn_get_unit_curve(lawn, curve);
     ylabel = gwy_lawn_get_curve_label(lawn, curve);
     g_object_set(gmodel,
-                 "si-unit-y", yunit,
+                 "unit-y", yunit,
                  "axis-label-bottom", _("sample"),
                  "axis-label-left", ylabel ? ylabel : _("Untitled"),
                  NULL);
@@ -948,7 +927,8 @@ zcut_simple(const gdouble *data, gint ifrom, gint ito, gboolean invert)
     b = (data[ito-1] - data[ifrom])/(ito - ifrom);
     for (i = ifrom; i < ito; i++) {
         y = data[i] - b*i;
-        if (invert) y = -y;
+        if (invert)
+            y = -y;
         if (y < m) {
             ibest = i;
             m = y;
