@@ -1,5 +1,5 @@
 /*
- *  $Id: cmap_extractcurve.c 26949 2024-12-03 10:42:17Z yeti-dn $
+ *  $Id: cmap_extractcurve.c 28808 2025-11-05 18:26:20Z yeti-dn $
  *  Copyright (C) 2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -17,17 +17,9 @@
 
 #include "config.h"
 #include <string.h>
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/lawn.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libgwydgets/gwydataview.h>
-#include <libgwydgets/gwygraph.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-cmap.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
+#include <gwy.h>
 
 #define RUN_MODES (GWY_RUN_INTERACTIVE)
 
@@ -64,7 +56,6 @@ typedef struct {
     ModuleArgs *args;
     GtkWidget *dialog;
     GwyParamTable *table;
-    GwyContainer *data;
     GwySelection *selection;
     GtkWidget *coordlist;
     gint current_point;
@@ -72,12 +63,12 @@ typedef struct {
 
 static gboolean         module_register            (void);
 static GwyParamDef*     define_module_params       (void);
-static void             extract_curve              (GwyContainer *data,
-                                                    GwyRunType runtype);
+static void             module_main                (GwyFile *data,
+                                                    GwyRunModeFlags mode);
 static void             execute                    (ModuleArgs *args,
                                                     GwySelection *selection);
 static GwyDialogOutcome run_gui                    (ModuleArgs *args,
-                                                    GwyContainer *data,
+                                                    GwyFile *data,
                                                     gint id);
 static GtkWidget*       create_coordlist           (ModuleGUI *gui);
 static void             param_changed              (ModuleGUI *gui,
@@ -116,11 +107,11 @@ static gboolean
 module_register(void)
 {
     gwy_curve_map_func_register("cmap_extractcurve",
-                                (GwyCurveMapFunc)&extract_curve,
+                                module_main,
                                 N_("/_Extract Curves..."),
                                 NULL,
                                 RUN_MODES,
-                                GWY_MENU_FLAG_CURVE_MAP,
+                                GWY_MENU_FLAG_CMAP,
                                 N_("Extract curves"));
 
     return TRUE;
@@ -150,7 +141,7 @@ define_module_params(void)
 }
 
 static void
-extract_curve(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
     GwyLawn *lawn = NULL;
@@ -158,12 +149,11 @@ extract_curve(GwyContainer *data, GwyRunType runtype)
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     gint id;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    g_return_if_fail(g_type_from_name("GwyLayerPoint"));
+    g_return_if_fail(mode & RUN_MODES);
 
-    gwy_app_data_browser_get_current(GWY_APP_LAWN, &lawn,
-                                     GWY_APP_LAWN_ID, &id,
-                                     0);
+    gwy_data_browser_get_current(GWY_APP_LAWN, &lawn,
+                                 GWY_APP_LAWN_ID, &id,
+                                 0);
     g_return_if_fail(GWY_IS_LAWN(lawn));
     args.lawn = lawn;
     args.nsegments = gwy_lawn_get_n_segments(lawn);
@@ -172,7 +162,7 @@ extract_curve(GwyContainer *data, GwyRunType runtype)
     sanitise_params(&args);
     update_graph_model_props(&args);
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, id);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -190,39 +180,30 @@ end:
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
-    GtkWidget *hbox, *graph, *dataview, *align, *coords;
+    GtkWidget *hbox, *graph, *dataview, *coords;
     GwyParamTable *table;
     GwyDialog *dialog;
     ModuleGUI gui;
-    GwyDataField *field;
+    GwyField *field;
     GwyDialogOutcome outcome;
-    const guchar *gradient;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
     gui.current_point = 0;
-    gui.data = gwy_container_new();
-    field = gwy_container_get_object(data, gwy_app_get_lawn_preview_key_for_id(id));
-    gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(0), field);
-    if (gwy_container_gis_string(data, gwy_app_get_lawn_palette_key_for_id(id), &gradient))
-        gwy_container_set_const_string(gui.data, gwy_app_get_data_palette_key_for_id(0), gradient);
 
     gui.dialog = gwy_dialog_new(_("Extract Map Curves"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_CLEAR, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    hbox = gwy_hbox_new(0);
-    gwy_dialog_add_content(GWY_DIALOG(gui.dialog), hbox, TRUE, TRUE, 0);
-
-    align = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-    gtk_box_pack_start(GTK_BOX(hbox), align, FALSE, FALSE, 0);
-
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
-    gtk_container_add(GTK_CONTAINER(align), dataview);
-    gui.selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(dataview), 0, "Point", 1, TRUE);
+    field = gwy_container_get_object(GWY_CONTAINER(data), gwy_file_key_cmap_picture(id));
+    dataview = gwy_create_preview(field, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_CMAP, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_REAL_SQUARE);
+    gui.selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(dataview), GWY_TYPE_LAYER_POINT, 1, TRUE);
     set_selection(&gui);
+    hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), FALSE);
 
     g_object_set(args->result, "label-visible", FALSE, NULL);
 
@@ -231,7 +212,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gtk_widget_set_size_request(graph, PREVIEW_SIZE, PREVIEW_SIZE);
     gtk_box_pack_start(GTK_BOX(hbox), graph, TRUE, TRUE, 0);
 
-    hbox = gwy_hbox_new(20);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 20);
     gwy_dialog_add_content(GWY_DIALOG(gui.dialog), hbox, TRUE, TRUE, 4);
 
     table = gui.table = gwy_param_table_new(args->params);
@@ -259,7 +240,6 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     outcome = gwy_dialog_run(dialog);
 
     g_object_set(args->result, "label-visible", TRUE, NULL);
-    g_object_unref(gui.data);
 
     return outcome;
 }
@@ -548,23 +528,23 @@ update_graph_model_props(ModuleArgs *args)
     gboolean abscissa_enabled = gwy_params_get_boolean(params, PARAM_ENABLE_ABSCISSA);
     gint abscissa = gwy_params_get_int(params, PARAM_ABSCISSA);
     gint ordinate = gwy_params_get_int(params, PARAM_ORDINATE);
-    GwySIUnit *xunit, *yunit;
+    GwyUnit *xunit, *yunit;
     const gchar *xlabel, *ylabel;
 
     if (abscissa_enabled) {
-        xunit = g_object_ref(gwy_lawn_get_si_unit_curve(lawn, abscissa));
+        xunit = g_object_ref(gwy_lawn_get_unit_curve(lawn, abscissa));
         xlabel = gwy_lawn_get_curve_label(lawn, abscissa);
     }
     else {
-        xunit = gwy_si_unit_new(NULL);
+        xunit = gwy_unit_new(NULL);
         xlabel = _("sample");
     }
-    yunit = gwy_lawn_get_si_unit_curve(lawn, ordinate);
+    yunit = gwy_lawn_get_unit_curve(lawn, ordinate);
     ylabel = gwy_lawn_get_curve_label(lawn, ordinate);
 
     g_object_set(gmodel,
-                 "si-unit-x", xunit,
-                 "si-unit-y", yunit,
+                 "unit-x", xunit,
+                 "unit-y", yunit,
                  "axis-label-bottom", xlabel ? xlabel : _("Untitled"),
                  "axis-label-left", ylabel ? ylabel : _("Untitled"),
                  NULL);
