@@ -1,5 +1,5 @@
 /*
- *  $Id: cmap_fztofd.c 28887 2025-11-20 11:22:08Z klapetek $
+ *  $Id: cmap_fztofd.c 28905 2025-11-24 15:51:18Z yeti-dn $
  *  Copyright (C) 2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,22 +16,11 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwythreads.h>
-#include <libprocess/lawn.h>
-#include <libprocess/stats.h>
-#include <libprocess/correct.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libgwydgets/gwydataview.h>
-#include <libgwydgets/gwygraph.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-cmap.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
-#include "libgwyddion/gwyomp.h"
+#include <gwy.h>
+#include "libgwyddion/omp.h"
 
 #define RUN_MODES (GWY_RUN_INTERACTIVE)
 
@@ -61,7 +50,7 @@ typedef struct {
     GwyParams *params;
     GwyLawn *lawn;
     GwyLawn *result;
-    GwyDataField *field;
+    GwyField *field;
     gboolean use_deflsens;
     gboolean use_stiffness;
 } ModuleArgs;
@@ -71,45 +60,45 @@ typedef struct {
     GtkWidget *dialog;
     GwyParamTable *table;
     GwyParamTable *table_output;
-    GwyContainer *data;
     GwySelection *selection;
     GwyGraphModel *gmodel;
 } ModuleGUI;
 
-static gboolean         module_register         (void);
-static GwyParamDef*     define_module_params    (void);
-static void             fztofd                  (GwyContainer *data,
-                                                 GwyRunType runtype);
-static void             execute                 (ModuleArgs *args);
-static GwyDialogOutcome run_gui                 (ModuleArgs *args,
-                                                 GwyContainer *data,
-                                                 gint id);
-static void             param_changed           (ModuleGUI *gui,
-                                                 gint id);
-static void             preview                 (gpointer user_data);
-static void             set_selection           (ModuleGUI *gui);
-static void             point_selection_changed (ModuleGUI *gui,
-                                                 gint id,
-                                                 GwySelection *selection);
-static void             extract_one_curve       (GwyLawn *lawn,
-                                                 GwyGraphCurveModel *gcmodel,
-                                                 gint col,
-                                                 gint row,
-                                                 GwyParams *params);
-static void             convert_one_curve       (GwyGraphCurveModel *gcmodel,
-                                                 GwyParams *params,
-                                                 gboolean use_deflsens,
-                                                 gboolean use_stiffness);
-static void             do_fz_to_fd             (const gdouble *xdata,
-                                                 const gdouble *ydata,
-                                                 gdouble *nxdata,
-                                                 gdouble *nydata, gint ndata,
-                                                 GwyFZInputType input_type,
-                                                 gdouble stiffness,
-                                                 gdouble tilt,
-                                                 gdouble deflsens,
-                                                 gboolean use_deflsens,
-                                                 gboolean use_stiffness);
+static gboolean         module_register        (void);
+static GwyParamDef*     define_module_params   (void);
+static void             module_main            (GwyFile *data,
+                                                GwyRunModeFlags mode);
+static void             execute                (ModuleArgs *args);
+static GwyDialogOutcome run_gui                (ModuleArgs *args,
+                                                GwyFile *data,
+                                                gint id);
+static void             param_changed          (ModuleGUI *gui,
+                                                gint id);
+static void             preview                (gpointer user_data);
+static void             set_selection          (ModuleGUI *gui);
+static void             point_selection_changed(ModuleGUI *gui,
+                                                gint id,
+                                                GwySelection *selection);
+static void             extract_one_curve      (GwyLawn *lawn,
+                                                GwyGraphCurveModel *gcmodel,
+                                                gint col,
+                                                gint row,
+                                                GwyParams *params);
+static void             convert_one_curve      (GwyGraphCurveModel *gcmodel,
+                                                GwyParams *params,
+                                                gboolean use_deflsens,
+                                                gboolean use_stiffness);
+static void             do_fz_to_fd            (const gdouble *xdata,
+                                                const gdouble *ydata,
+                                                gdouble *nxdata,
+                                                gdouble *nydata,
+                                                gint ndata,
+                                                GwyFZInputType input_type,
+                                                gdouble stiffness,
+                                                gdouble tilt,
+                                                gdouble deflsens,
+                                                gboolean use_deflsens,
+                                                gboolean use_stiffness);
 
 static void             update_graph_model_props(GwyGraphModel *gmodel,
                                                  ModuleArgs *args);
@@ -131,11 +120,11 @@ static gboolean
 module_register(void)
 {
     gwy_curve_map_func_register("cmap_fztofd",
-                                (GwyCurveMapFunc)&fztofd,
+                                module_main,
                                 N_("/_FZ Curves to FD..."),
                                 NULL,
                                 RUN_MODES,
-                                GWY_MENU_FLAG_CURVE_MAP,
+                                GWY_MENU_FLAG_CMAP,
                                 N_("Convert Force-Z Piezo to Force-Distance"));
 
     return TRUE;
@@ -144,11 +133,9 @@ module_register(void)
 static GwyParamDef*
 define_module_params(void)
 {
-
-
     static const GwyEnum input_types[] =  {
-        { N_("Height"),             GWY_FZ_HEIGHT,   },
-        { N_("Piezo extension"),    GWY_FZ_ZPIEZO,  },
+        { N_("Height"),          GWY_FZ_HEIGHT, },
+        { N_("Piezo extension"), GWY_FZ_ZPIEZO, },
     };
 
     static GwyParamDef *paramdef = NULL;
@@ -163,8 +150,10 @@ define_module_params(void)
     gwy_param_def_add_int(paramdef, PARAM_XPOS, "xpos", NULL, -1, G_MAXINT, -1);
     gwy_param_def_add_int(paramdef, PARAM_YPOS, "ypos", NULL, -1, G_MAXINT, -1);
     gwy_param_def_add_gwyenum(paramdef, PARAM_INPUT_TYPE, "input_type", _("_Z input"),
-                               input_types, G_N_ELEMENTS(input_types), GWY_FZ_HEIGHT);
+                              input_types, G_N_ELEMENTS(input_types), GWY_FZ_HEIGHT);
     gwy_param_def_add_double(paramdef, PARAM_STIFFNESS, "stiffness", _("Cantilever _stiffness"), 0.001, 1000, 20.0);
+    /* FIXME: This is wrong. Raw parameters should be always in radians. Set a conversion factor with
+     * gwy_param_table_slider_set_scaling(). */
     gwy_param_def_add_double(paramdef, PARAM_TILT, "tilt", _("Cantilever _tilt"), 0, 20, 0);
     gwy_param_def_add_double(paramdef, PARAM_DEFLSENS, "deflsens", _("_Deflection sensitivity"), 0.001, 1000, 100);
 
@@ -172,21 +161,19 @@ define_module_params(void)
 }
 
 static void
-fztofd(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
     GwyLawn *lawn = NULL;
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     gint id;
-    gchar newtitle[100];
 
-    g_return_if_fail(runtype & RUN_MODES);
-    g_return_if_fail(g_type_from_name("GwyLayerPoint"));
+    g_return_if_fail(mode & RUN_MODES);
 
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_LAWN, &lawn,
-                                     GWY_APP_LAWN_ID, &id,
-                                     0);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_LAWN, &lawn,
+                                 GWY_APP_LAWN_ID, &id,
+                                 0);
     g_return_if_fail(GWY_IS_LAWN(lawn));
     args.lawn = lawn;
     args.params = gwy_params_new_from_settings(define_module_params());
@@ -194,13 +181,9 @@ fztofd(GwyContainer *data, GwyRunType runtype)
     args.use_stiffness = FALSE;
     sanitise_params(&args);
 
-    args.field = gwy_data_field_new(gwy_lawn_get_xres(lawn), gwy_lawn_get_yres(lawn),
-                                     gwy_lawn_get_xreal(lawn), gwy_lawn_get_yreal(lawn), TRUE);
-    gwy_data_field_set_xoffset(args.field, gwy_lawn_get_xoffset(lawn));
-    gwy_data_field_set_yoffset(args.field, gwy_lawn_get_yoffset(lawn));
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(args.field), gwy_lawn_get_si_unit_xy(lawn));
+    args.field = gwy_lawn_new_field_alike(lawn, -1, TRUE);
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, id);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -209,51 +192,39 @@ fztofd(GwyContainer *data, GwyRunType runtype)
     if (outcome != GWY_DIALOG_HAVE_RESULT)
         execute(&args);
 
-    g_snprintf(newtitle, sizeof(newtitle), "%s: FD", gwy_app_get_lawn_title(data, id));
-    gwy_app_set_lawn_title(data, id, newtitle);
-
-    gwy_app_curve_map_log_add(data, -1, id, "cmap::cmap_fztofd", NULL);
+    /* TODO: Add log, maybe more. */
 
 end:
-    GWY_OBJECT_UNREF(args.result);
+    g_clear_object(&args.result);
     g_object_unref(args.field);
     g_object_unref(args.params);
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
-    GtkWidget *hbox, *graph, *dataview, *align;
+    GtkWidget *hbox, *graph, *dataview;
     GwyParamTable *table;
     GwyDialog *dialog;
     ModuleGUI gui;
-    GwyDataField *field;
+    GwyField *field;
     GwyDialogOutcome outcome;
     GwyGraphCurveModel *gcmodel;
-    const guchar *gradient;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
-    gui.data = gwy_container_new();
-    field = gwy_container_get_object(data, gwy_app_get_lawn_preview_key_for_id(id));
-    gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(0), field);
-    if (gwy_container_gis_string(data, gwy_app_get_lawn_palette_key_for_id(id), &gradient))
-        gwy_container_set_const_string(gui.data, gwy_app_get_data_palette_key_for_id(0), gradient);
 
     gui.dialog = gwy_dialog_new(_("Convert FZ to FD Curve"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    hbox = gwy_hbox_new(0);
-    gwy_dialog_add_content(GWY_DIALOG(gui.dialog), hbox, TRUE, TRUE, 0);
-
-    align = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-    gtk_box_pack_start(GTK_BOX(hbox), align, FALSE, FALSE, 0);
-
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
-    gtk_container_add(GTK_CONTAINER(align), dataview);
-    gui.selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(dataview), 0, "Point", 1, TRUE);
+    field = gwy_container_get_object(GWY_CONTAINER(data), gwy_file_key_cmap_picture(id));
+    dataview = gwy_create_preview(field, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_CMAP, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_REAL_SQUARE);
+    gui.selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(dataview), GWY_TYPE_LAYER_POINT, 1, TRUE);
     set_selection(&gui);
+    hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), FALSE);
 
     gui.gmodel = gwy_graph_model_new();
     gcmodel = gwy_graph_curve_model_new();
@@ -270,7 +241,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gtk_box_pack_start(GTK_BOX(hbox), graph, TRUE, TRUE, 0);
 
 
-    hbox = gwy_hbox_new(20);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 20);
     gwy_dialog_add_content(GWY_DIALOG(gui.dialog), hbox, TRUE, TRUE, 4);
 
     table = gui.table = gwy_param_table_new(args->params);
@@ -295,7 +266,6 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     outcome = gwy_dialog_run(dialog);
 
     g_object_unref(gui.gmodel);
-    g_object_unref(gui.data);
 
     return outcome;
 }
@@ -347,12 +317,12 @@ preview(gpointer user_data)
     gint row = gwy_params_get_int(params, PARAM_YPOS);
     gint ordinate = gwy_params_get_int(params, PARAM_ORDINATE);
 
-    if (gwy_si_unit_equal_string(gwy_lawn_get_si_unit_curve(args->lawn, ordinate), "V"))
+    if (gwy_unit_equal_string(gwy_lawn_get_unit_curve(args->lawn, ordinate), "V"))
         args->use_deflsens = TRUE;
     else
         args->use_deflsens = FALSE;
 
-    if (gwy_si_unit_equal_string(gwy_lawn_get_si_unit_curve(args->lawn, ordinate), "m"))
+    if (gwy_unit_equal_string(gwy_lawn_get_unit_curve(args->lawn, ordinate), "m"))
         args->use_stiffness = TRUE;
      else
         args->use_stiffness = FALSE;
@@ -362,7 +332,6 @@ preview(gpointer user_data)
     extract_one_curve(args->lawn, gwy_graph_model_get_curve(gui->gmodel, 0), col, row, params);
     convert_one_curve(gwy_graph_model_get_curve(gui->gmodel, 0), params, args->use_deflsens, args->use_stiffness);
     update_graph_model_props(gui->gmodel, args);
-
 }
 
 static void
@@ -373,7 +342,7 @@ execute(ModuleArgs *args)
     gint ordinate = gwy_params_get_int(params, PARAM_ORDINATE);
     GwyFZInputType input_type = gwy_params_get_enum(params, PARAM_INPUT_TYPE);
     gdouble stiffness = gwy_params_get_double(params, PARAM_STIFFNESS);
-    gdouble tilt = gwy_params_get_double(params, PARAM_TILT)*M_PI/180;
+    gdouble tilt = gwy_deg2rad(gwy_params_get_double(params, PARAM_TILT));
     gdouble deflsens = gwy_params_get_double(params, PARAM_DEFLSENS)*1e-9;
     gboolean use_deflsens = args->use_deflsens;
     gboolean use_stiffness = args->use_stiffness;
@@ -401,8 +370,7 @@ execute(ModuleArgs *args)
     }
 
     if (use_deflsens || use_stiffness)
-       gwy_lawn_set_si_unit_curve(args->lawn, ordinate, gwy_si_unit_new("N"));
-
+       gwy_unit_set_from_string(gwy_lawn_get_unit_curve(args->lawn, ordinate), "N");
 }
 
 static void
@@ -422,16 +390,16 @@ extract_one_curve(GwyLawn *lawn, GwyGraphCurveModel *gcmodel,
 
 static void
 do_fz_to_fd(const gdouble *xdata, const gdouble *ydata, gdouble *nxdata, gdouble *nydata, gint ndata,
-            GwyFZInputType input_type, gdouble stiffness, gdouble tilt, gdouble deflsens, 
+            GwyFZInputType input_type, gdouble stiffness, gdouble tilt, gdouble deflsens,
             gboolean use_deflsens, gboolean use_stiffness)
 {
     gint i;
     gdouble tiltcorr =  1.0/(cos(tilt)*cos(tilt));
     gdouble vtof = 1.0;
 
-    if (use_deflsens) 
+    if (use_deflsens)
         vtof = deflsens*stiffness;
-    else if (use_stiffness) 
+    else if (use_stiffness)
         vtof = stiffness;
 
     if (input_type == GWY_FZ_HEIGHT) {  //peak on the left side
@@ -458,7 +426,7 @@ convert_one_curve(GwyGraphCurveModel *gcmodel, GwyParams *params, gboolean use_d
 {
     GwyFZInputType input_type = gwy_params_get_enum(params, PARAM_INPUT_TYPE);
     gdouble stiffness = gwy_params_get_double(params, PARAM_STIFFNESS);
-    gdouble tilt = gwy_params_get_double(params, PARAM_TILT)*M_PI/180;
+    gdouble tilt = gwy_deg2rad(gwy_params_get_double(params, PARAM_TILT));
     gdouble deflsens = gwy_params_get_double(params, PARAM_DEFLSENS)*1e-9;
 
     const gdouble *xdata = gwy_graph_curve_model_get_xdata(gcmodel);
@@ -470,7 +438,7 @@ convert_one_curve(GwyGraphCurveModel *gcmodel, GwyParams *params, gboolean use_d
     nxdata = g_new(gdouble, ndata);
     nydata = g_new(gdouble, ndata);
 
-    do_fz_to_fd(xdata, ydata, nxdata, nydata, ndata, input_type, stiffness, tilt, deflsens, 
+    do_fz_to_fd(xdata, ydata, nxdata, nydata, ndata, input_type, stiffness, tilt, deflsens,
                 use_deflsens, use_stiffness);
 
     gwy_graph_curve_model_set_data(gcmodel, nxdata, nydata, ndata);
@@ -486,24 +454,24 @@ update_graph_model_props(GwyGraphModel *gmodel, ModuleArgs *args)
     GwyParams *params = args->params;
     gint abscissa = gwy_params_get_int(params, PARAM_ABSCISSA);
     gint ordinate = gwy_params_get_int(params, PARAM_ORDINATE);
-    GwySIUnit *xunit, *yunit;
+    GwyUnit *xunit, *yunit;
     const gchar *xlabel, *ylabel;
 
-    xunit = gwy_lawn_get_si_unit_curve(lawn, abscissa);
+    xunit = gwy_lawn_get_unit_curve(lawn, abscissa);
     xlabel = gwy_lawn_get_curve_label(lawn, abscissa);
 
     if (args->use_deflsens || args->use_stiffness) {
-        yunit = gwy_si_unit_new("N");
+        yunit = gwy_unit_new("N");
         ylabel = g_strdup("Force");
     }
     else {
-        yunit = gwy_lawn_get_si_unit_curve(lawn, ordinate);
+        yunit = gwy_lawn_get_unit_curve(lawn, ordinate);
         ylabel = gwy_lawn_get_curve_label(lawn, ordinate);
     }
 
     g_object_set(gmodel,
-                 "si-unit-x", xunit,
-                 "si-unit-y", yunit,
+                 "unit-x", xunit,
+                 "unit-y", yunit,
                  "axis-label-bottom", xlabel ? xlabel : _("Untitled"),
                  "axis-label-left", ylabel ? ylabel : _("Untitled"),
                  NULL);
