@@ -1,5 +1,5 @@
 /*
- *  $Id: cmap_linestat.c 26949 2024-12-03 10:42:17Z yeti-dn $
+ *  $Id: cmap_linestat.c 29023 2025-12-19 13:06:16Z yeti-dn $
  *  Copyright (C) 2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,22 +16,11 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwythreads.h>
-#include <libprocess/lawn.h>
-#include <libprocess/correct.h>
-#include <libprocess/stats.h>
-#include <libprocess/linestats.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libgwydgets/gwydataview.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-cmap.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
-#include "libgwyddion/gwyomp.h"
+#include <gwy.h>
+#include "libgwyddion/omp.h"
 
 #define RUN_MODES (GWY_RUN_INTERACTIVE)
 
@@ -61,7 +50,7 @@ typedef enum {
     NOUTPUTS
 } LineStatOutput;
 
-typedef gdouble (*LineStatFunc)(GwyDataLine *dataline);
+typedef gdouble (*LineStatFunc)(GwyLine *line);
 
 typedef struct {
     const gchar *name;
@@ -72,8 +61,8 @@ typedef struct {
 typedef struct {
     GwyParams *params;
     GwyLawn *lawn;
-    GwyDataField *result;
-    GwyDataField *mask;
+    GwyField *result;
+    GwyField *mask;
     /* Cached input data properties. */
     gint nsegments;
 } ModuleArgs;
@@ -83,19 +72,18 @@ typedef struct {
     GtkWidget *dialog;
     GwyParamTable *table_quantity;
     GwyParamTable *table_options;
-    GwyContainer *data;
     GwySelection *image_selection;
     GwyGraphModel *gmodel;
-    GwySIValueFormat *vf;
+    GwyValueFormat *vf;
 } ModuleGUI;
 
 static gboolean                    module_register        (void);
 static GwyParamDef*                define_module_params   (void);
-static void                        line_stat              (GwyContainer *data,
-                                                           GwyRunType runtype);
+static void                        module_main            (GwyFile *data,
+                                                           GwyRunModeFlags mode);
 static void                        execute                (ModuleArgs *args);
 static GwyDialogOutcome            run_gui                (ModuleArgs *args,
-                                                           GwyContainer *data,
+                                                           GwyFile *data,
                                                            gint id);
 static void                        param_changed          (ModuleGUI *gui,
                                                            gint id);
@@ -106,11 +94,11 @@ static void                        point_selection_changed(ModuleGUI *gui,
                                                            GwySelection *selection);
 static void                        update_graph_curve     (ModuleGUI *gui);
 static void                        update_current_value   (ModuleGUI *gui);
-static gdouble                     get_data_line_range    (GwyDataLine *dataline);
-static gdouble                     get_data_line_Rz       (GwyDataLine *dataline);
-static gdouble                     get_data_line_Rt       (GwyDataLine *dataline);
-static gint                        extract_data_line      (GwyLawn *lawn,
-                                                           GwyDataLine *target,
+static gdouble                     get_line_range         (GwyLine *line);
+static gdouble                     get_line_Rz            (GwyLine *line);
+static gdouble                     get_line_Rt            (GwyLine *line);
+static gint                        extract_line           (GwyLawn *lawn,
+                                                           GwyLine *target,
                                                            gint col,
                                                            gint row,
                                                            gint curveno,
@@ -120,26 +108,26 @@ static void                        sanitise_params        (ModuleArgs *args);
 
 /* XXX: This is more or less identical to tools/linestat.c. */
 static const LineStatQuantityInfo quantities[] =  {
-    { N_("Mean"),            GWY_LINE_STAT_MEAN,     gwy_data_line_get_avg,      },
-    { N_("Median"),          GWY_LINE_STAT_MEDIAN,   gwy_data_line_get_median,   },
-    { N_("Minimum"),         GWY_LINE_STAT_MINIMUM,  gwy_data_line_get_min,      },
-    { N_("Maximum"),         GWY_LINE_STAT_MAXIMUM,  gwy_data_line_get_max,      },
+    { N_("Mean"),            GWY_LINE_STAT_MEAN,     gwy_line_avg,      },
+    { N_("Median"),          GWY_LINE_STAT_MEDIAN,   gwy_line_median,   },
+    { N_("Minimum"),         GWY_LINE_STAT_MINIMUM,  gwy_line_min,      },
+    { N_("Maximum"),         GWY_LINE_STAT_MAXIMUM,  gwy_line_max,      },
     /* Positions require a secondary curve for abscissa.  With those available we can also consider the other
      * quantities which require an abscissa.
-    { N_("Min. position"),   GWY_LINE_STAT_MINPOS,   gwy_data_line_min_pos_i,    },
-    { N_("Max. position"),   GWY_LINE_STAT_MAXPOS,   gwy_data_line_max_pos_i,    },
-    { N_("Slope"),              GWY_LINE_STAT_SLOPE,     get_data_line_slope,         },
-    { N_("tan β<sub>0</sub>"),  GWY_LINE_STAT_TAN_BETA0, gwy_data_line_get_tan_beta0, },
-    { N_("Variation"),          GWY_LINE_STAT_VARIATION, gwy_data_line_get_variation, },
-    { N_("Developed length"),   GWY_LINE_STAT_LENGTH,    gwy_data_line_get_length,    },
+    { N_("Min. position"),   GWY_LINE_STAT_MINPOS,   gwy_line_min_pos_i,    },
+    { N_("Max. position"),   GWY_LINE_STAT_MAXPOS,   gwy_line_max_pos_i,    },
+    { N_("Slope"),              GWY_LINE_STAT_SLOPE,     get_line_slope,         },
+    { N_("tan β<sub>0</sub>"),  GWY_LINE_STAT_TAN_BETA0, gwy_line_tan_beta0, },
+    { N_("Variation"),          GWY_LINE_STAT_VARIATION, gwy_line_variation, },
+    { N_("Developed length"),   GWY_LINE_STAT_LENGTH,    gwy_line_length,    },
     */
-    { N_("Range"),           GWY_LINE_STAT_RANGE,    get_data_line_range,        },
-    { N_("Ra"),              GWY_LINE_STAT_RA,       gwy_data_line_get_ra,       },
-    { N_("Rq (RMS)"),        GWY_LINE_STAT_RMS,      gwy_data_line_get_rms,      },
-    { N_("Rz"),              GWY_LINE_STAT_RZ,       get_data_line_Rz,           },
-    { N_("Rt"),              GWY_LINE_STAT_RT,       get_data_line_Rt,           },
-    { N_("Skew"),            GWY_LINE_STAT_SKEW,     gwy_data_line_get_skew,     },
-    { N_("Excess kurtosis"), GWY_LINE_STAT_KURTOSIS, gwy_data_line_get_kurtosis, },
+    { N_("Range"),           GWY_LINE_STAT_RANGE,    get_line_range,        },
+    { N_("Ra"),              GWY_LINE_STAT_RA,       gwy_line_ra,       },
+    { N_("Rq (RMS)"),        GWY_LINE_STAT_RMS,      gwy_line_rms,      },
+    { N_("Rz"),              GWY_LINE_STAT_RZ,       get_line_Rz,           },
+    { N_("Rt"),              GWY_LINE_STAT_RT,       get_line_Rt,           },
+    { N_("Skew"),            GWY_LINE_STAT_SKEW,     gwy_line_skew,     },
+    { N_("Excess kurtosis"), GWY_LINE_STAT_KURTOSIS, gwy_line_kurtosis, },
 };
 
 static GwyModuleInfo module_info = {
@@ -158,11 +146,11 @@ static gboolean
 module_register(void)
 {
     gwy_curve_map_func_register("cmap_linestat",
-                                (GwyCurveMapFunc)&line_stat,
+                                module_main,
                                 N_("/_Summarize Curves..."),
                                 NULL,
                                 RUN_MODES,
-                                GWY_MENU_FLAG_CURVE_MAP,
+                                GWY_MENU_FLAG_CMAP,
                                 N_("Summarize curves"));
 
     return TRUE;
@@ -201,7 +189,7 @@ define_module_params(void)
 }
 
 static void
-line_stat(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
     GwyLawn *lawn = NULL;
@@ -211,26 +199,20 @@ line_stat(GwyContainer *data, GwyRunType runtype)
     const gchar *title;
     gint oldid, newid;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    g_return_if_fail(g_type_from_name("GwyLayerPoint"));
+    g_return_if_fail(mode & RUN_MODES);
 
-    gwy_app_data_browser_get_current(GWY_APP_LAWN, &lawn,
-                                     GWY_APP_LAWN_ID, &oldid,
-                                     0);
+    gwy_data_browser_get_current(GWY_APP_LAWN, &lawn,
+                                 GWY_APP_LAWN_ID, &oldid,
+                                 0);
     g_return_if_fail(GWY_IS_LAWN(lawn));
     args.lawn = lawn;
     args.nsegments = gwy_lawn_get_n_segments(lawn);
     args.params = gwy_params_new_from_settings(define_module_params());
-    args.result = gwy_data_field_new(gwy_lawn_get_xres(lawn), gwy_lawn_get_yres(lawn),
-                                     gwy_lawn_get_xreal(lawn), gwy_lawn_get_yreal(lawn), TRUE);
-    gwy_data_field_set_xoffset(args.result, gwy_lawn_get_xoffset(lawn));
-    gwy_data_field_set_yoffset(args.result, gwy_lawn_get_yoffset(lawn));
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(args.result), gwy_lawn_get_si_unit_xy(lawn));
-    args.mask = gwy_data_field_new_alike(args.result, TRUE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(args.mask), NULL);
+    args.result = gwy_lawn_new_field_alike(lawn, -1, TRUE);
+    args.mask = gwy_field_new_alike(args.result, TRUE);
     sanitise_params(&args);
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, oldid);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -242,15 +224,18 @@ line_stat(GwyContainer *data, GwyRunType runtype)
     output_type = gwy_params_get_enum(args.params, PARAM_OUTPUT_TYPE);
     quantity = gwy_params_get_enum(args.params, PARAM_QUANTITY);
     if (output_type == OUTPUT_IMAGE) {
-        newid = gwy_app_data_browser_add_data_field(args.result, data, TRUE);
-        title = gwy_sgettext(find_quantity(quantity)->name);
-        gwy_app_set_data_field_title(data, newid, title);
-        if (gwy_data_field_get_max(args.mask) > 0.0)
-            gwy_container_set_object(data, gwy_app_get_mask_key_for_id(newid), args.mask);
-        gwy_app_channel_log_add(data, -1, newid, "cmap::cmap_linestat", NULL);
+        newid = gwy_file_add_image(data, args.result);
+        gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+        title = gwy_C(find_quantity(quantity)->name);
+        gwy_file_set_title(data, GWY_FILE_IMAGE, newid, title, TRUE);
+        if (gwy_field_get_max(args.mask) > 0.0)
+            gwy_file_pass_image_mask(data, newid, args.mask);
+        else
+            g_clear_object(&args.mask);
+        gwy_log_add_full(data, GWY_FILE_IMAGE, -1, newid, "cmap::cmap_linestat", NULL);
     }
     else if (output_type == OUTPUT_PREVIEW)
-        gwy_container_set_object(data, gwy_app_get_lawn_preview_key_for_id(oldid), args.result);
+        gwy_container_set_object(GWY_CONTAINER(data), gwy_file_key_cmap_picture(oldid), args.result);
     else {
         g_assert_not_reached();
     }
@@ -262,37 +247,28 @@ end:
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
-    GtkWidget *hbox, *graph, *dataview, *align;
+    GtkWidget *hbox, *graph, *dataview;
     GwyParamTable *table;
     GwyDialog *dialog;
     ModuleGUI gui;
-    GwyDataField *field = args->result;
+    GwyField *field = args->result;
     GwyGraphCurveModel *gcmodel;
     GwyDialogOutcome outcome;
-    const guchar *gradient;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
-    gui.data = gwy_container_new();
-    gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(0), field);
-    if (gwy_container_gis_string(data, gwy_app_get_lawn_palette_key_for_id(id), &gradient))
-        gwy_container_set_const_string(gui.data, gwy_app_get_data_palette_key_for_id(0), gradient);
 
     gui.dialog = gwy_dialog_new(_("Summarize Map Curves"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_UPDATE, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    hbox = gwy_hbox_new(0);
-    gwy_dialog_add_content(GWY_DIALOG(gui.dialog), hbox, TRUE, TRUE, 0);
-
-    align = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-    gtk_box_pack_start(GTK_BOX(hbox), align, FALSE, FALSE, 0);
-
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
-    gtk_container_add(GTK_CONTAINER(align), dataview);
-    gui.image_selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(dataview), 0, "Point", 1, TRUE);
+    dataview = gwy_create_preview(field, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_CMAP, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_REAL_SQUARE);
+    gui.image_selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(dataview), GWY_TYPE_LAYER_POINT, 1, TRUE);
+    hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), FALSE);
 
     gui.gmodel = gwy_graph_model_new();
     g_object_set(gui.gmodel,
@@ -310,7 +286,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gtk_widget_set_size_request(graph, PREVIEW_SIZE, PREVIEW_SIZE);
     gtk_box_pack_start(GTK_BOX(hbox), graph, TRUE, TRUE, 0);
 
-    hbox = gwy_hbox_new(20);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 20);
     gwy_dialog_add_content(GWY_DIALOG(gui.dialog), hbox, TRUE, TRUE, 4);
 
     table = gui.table_quantity = gwy_param_table_new(args->params);
@@ -341,9 +317,8 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 
     outcome = gwy_dialog_run(dialog);
 
-    g_object_unref(gui.data);
     g_object_unref(gui.gmodel);
-    gwy_si_unit_value_format_free(gui.vf);
+    gwy_value_format_free(gui.vf);
 
     return outcome;
 }
@@ -397,7 +372,7 @@ update_graph_curve(ModuleGUI *gui)
     ModuleArgs *args = gui->args;
     GwyParams *params = args->params;
     GwyLawn *lawn = args->lawn;
-    GwyDataLine *line = gwy_data_line_new(1, 1.0, FALSE);
+    GwyLine *line = gwy_line_new(1, 1.0, FALSE);
     gint col = gwy_params_get_int(params, PARAM_XPOS);
     gint row = gwy_params_get_int(params, PARAM_YPOS);
     gint curveno = gwy_params_get_int(params, PARAM_CURVE);
@@ -406,14 +381,14 @@ update_graph_curve(ModuleGUI *gui)
     GwyGraphCurveModel *gcmodel = gwy_graph_model_get_curve(gui->gmodel, 0);
     const gchar *label;
 
-    if (extract_data_line(lawn, line, col, row, curveno, segment))
-        gwy_graph_curve_model_set_data_from_dataline(gcmodel, line, 0, 0);
+    if (extract_line(lawn, line, col, row, curveno, segment))
+        gwy_graph_curve_model_set_data_from_line(gcmodel, line, 0, 0);
     else
         gwy_graph_curve_model_set_data(gcmodel, NULL, NULL, 0);
     g_object_unref(line);
     label = gwy_lawn_get_curve_label(lawn, curveno);
     g_object_set(gui->gmodel,
-                 "si-unit-y", gwy_lawn_get_si_unit_curve(lawn, curveno),
+                 "unit-y", gwy_lawn_get_unit_curve(lawn, curveno),
                  "axis-label-left", label ? label : _("Untitled"),
                  NULL);
 
@@ -426,15 +401,15 @@ update_current_value(ModuleGUI *gui)
     ModuleArgs *args = gui->args;
     gint col = gwy_params_get_int(args->params, PARAM_XPOS);
     gint row = gwy_params_get_int(args->params, PARAM_YPOS);
-    GwySIValueFormat *vf;
-    GwySIUnit *unit;
+    GwyValueFormat *vf;
+    GwyUnit *unit;
     gdouble v, m;
     gchar *s;
 
-    v = gwy_data_field_get_val(args->result, col, row);
-    m = gwy_data_field_get_val(args->mask, col, row);
-    unit = gwy_data_field_get_si_unit_z(args->result);
-    vf = gui->vf = gwy_si_unit_get_format_with_digits(unit, GWY_SI_UNIT_FORMAT_VFMARKUP, v, 3, gui->vf);
+    v = gwy_field_get_val(args->result, col, row);
+    m = gwy_field_get_val(args->mask, col, row);
+    unit = gwy_field_get_unit_z(args->result);
+    vf = gui->vf = gwy_unit_get_format_with_digits(unit, GWY_UNIT_FORMAT_VFMARKUP, v, 3, gui->vf);
     s = g_strdup_printf("%.*f%s%s", vf->precision, v/vf->magnitude, *vf->units ? " " : "", vf->units);
     gwy_param_table_info_set_valuestr(gui->table_quantity, LABEL_VALUE, s);
     g_free(s);
@@ -448,33 +423,33 @@ preview(gpointer user_data)
     ModuleArgs *args = gui->args;
 
     execute(args);
-    gwy_data_field_data_changed(args->result);
-    gwy_data_field_data_changed(args->mask);
+    gwy_field_data_changed(args->result);
+    gwy_field_data_changed(args->mask);
     update_graph_curve(gui);
     gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
 }
 
 static gdouble
-get_data_line_range(GwyDataLine *dataline)
+get_line_range(GwyLine *line)
 {
     gdouble min, max;
 
-    gwy_data_line_get_min_max(dataline, &min, &max);
+    gwy_line_min_max(line, &min, &max);
     return max - min;
 }
 
 static gdouble
-get_data_line_Rt(GwyDataLine *dataline)
+get_line_Rt(GwyLine *line)
 {
-    gwy_data_line_add(dataline, -gwy_data_line_get_avg(dataline));
-    return gwy_data_line_get_xtm(dataline, 1, 1);
+    gwy_line_add(line, -gwy_line_avg(line));
+    return gwy_line_xtm(line, 1, 1);
 }
 
 static gdouble
-get_data_line_Rz(GwyDataLine *dataline)
+get_line_Rz(GwyLine *line)
 {
-    gwy_data_line_add(dataline, -gwy_data_line_get_avg(dataline));
-    return gwy_data_line_get_xtm(dataline, 5, 1);
+    gwy_line_add(line, -gwy_line_avg(line));
+    return gwy_line_xtm(line, 5, 1);
 }
 
 static void
@@ -486,31 +461,31 @@ execute(ModuleArgs *args)
     gboolean segment_enabled = args->nsegments ? gwy_params_get_boolean(params, PARAM_ENABLE_SEGMENT) : FALSE;
     gint segment = segment_enabled ? gwy_params_get_int(params, PARAM_SEGMENT) : -1;
     GwyLawn *lawn = args->lawn;
-    GwyDataField *field = args->result, *mask = args->mask;
-    GwySIUnit *imgunit, *valunit;
+    GwyField *field = args->result, *mask = args->mask;
+    GwyUnit *imgunit, *valunit;
     gint xres = gwy_lawn_get_xres(lawn), yres = gwy_lawn_get_yres(lawn);
     LineStatFunc lsfunc = NULL;
     gdouble *data, *mdata;
 
-    /* Quantities we handle (somewhat inefficiently) by using DataLine statistics. */
+    /* Quantities we handle (somewhat inefficiently) by using Line statistics. */
     lsfunc = find_quantity(quantity)->func;
 
-    gwy_data_field_clear(mask);
-    data = gwy_data_field_get_data(field);
-    mdata = gwy_data_field_get_data(mask);
+    gwy_field_clear(mask);
+    data = gwy_field_get_data(field);
+    mdata = gwy_field_get_data(mask);
 
 #ifdef _OPENMP
 #pragma omp parallel if(gwy_threads_are_enabled()) default(none) \
             shared(lawn,xres,yres,curveno,segment,data,mdata,lsfunc)
 #endif
     {
-        GwyDataLine *dline = gwy_data_line_new(1, 1.0, FALSE);
+        GwyLine *dline = gwy_line_new(1, 1.0, FALSE);
         guint kfrom = gwy_omp_chunk_start(xres*yres);
         guint kto = gwy_omp_chunk_end(xres*yres);
         guint k;
 
         for (k = kfrom; k < kto; k++) {
-            if (extract_data_line(lawn, dline, k % xres, k/xres, curveno, segment))
+            if (extract_line(lawn, dline, k % xres, k/xres, curveno, segment))
                 data[k] = lsfunc(dline);
             else
                 mdata[k] = 1.0;
@@ -519,21 +494,21 @@ execute(ModuleArgs *args)
         g_object_unref(dline);
     }
 
-    imgunit = gwy_data_field_get_si_unit_z(field);
-    valunit = gwy_lawn_get_si_unit_curve(lawn, curveno);
+    imgunit = gwy_field_get_unit_z(field);
+    valunit = gwy_lawn_get_unit_curve(lawn, curveno);
 
     if ((quantity == GWY_LINE_STAT_SKEW) || (quantity == GWY_LINE_STAT_KURTOSIS))
-        gwy_si_unit_set_from_string(imgunit, NULL);
+        gwy_unit_clear(imgunit);
     else
-        gwy_si_unit_assign(imgunit, valunit);
+        gwy_unit_assign(imgunit, valunit);
 
-    if (gwy_data_field_get_max(mask) > 0.0)
-        gwy_data_field_laplace_solve(field, mask, -1, 1.0);
+    if (gwy_field_get_max(mask) > 0.0)
+        gwy_field_laplace_solve(field, mask, -1, 1.0);
 }
 
 static gint
-extract_data_line(GwyLawn *lawn, GwyDataLine *target,
-                  gint col, gint row, gint curveno, gint segment)
+extract_line(GwyLawn *lawn, GwyLine *target,
+             gint col, gint row, gint curveno, gint segment)
 {
     gint pos = 0, len;
     const gdouble *cdata;
@@ -552,10 +527,10 @@ extract_data_line(GwyLawn *lawn, GwyDataLine *target,
             return 0;
     }
 
-    gwy_data_line_resample(target, len, GWY_INTERPOLATION_NONE);
-    ldata = gwy_data_line_get_data(target);
+    gwy_line_resize(target, len);
+    ldata = gwy_line_get_data(target);
     gwy_assign(ldata, cdata + pos, len);
-    gwy_data_line_set_real(target, len);
+    gwy_line_set_real(target, len);
 
     return len;
 }
