@@ -1,5 +1,5 @@
 /*
- *  $Id: cmap_lockin.c 26949 2024-12-03 10:42:17Z yeti-dn $
+ *  $Id: cmap_lockin.c 29061 2026-01-02 15:01:53Z yeti-dn $
  *  Copyright (C) 2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,24 +16,11 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwythreads.h>
-#include <libprocess/lawn.h>
-#include <libprocess/stats.h>
-#include <libprocess/correct.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libgwydgets/gwydataview.h>
-#include <libgwydgets/gwygraph.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-cmap.h>
-#include <libgwyddion/gwyddion.h>
-#include <libprocess/gwyprocess.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
-#include "libgwyddion/gwyomp.h"
+#include <gwy.h>
+#include "libgwyddion/omp.h"
 
 #define RUN_MODES (GWY_RUN_INTERACTIVE)
 
@@ -42,21 +29,20 @@ enum {
 };
 
 typedef enum {
-    OUTPUT_QUANTITY   = 0,
-    OUTPUT_PREVIEW    = 1,
+    OUTPUT_QUANTITY = 0,
+    OUTPUT_PREVIEW  = 1,
 } LockinOutput;
 
 typedef enum {
-    SHOW_CARRIER   = 0,
-    SHOW_QUANTITY  = 1,
+    SHOW_CARRIER  = 0,
+    SHOW_QUANTITY = 1,
 } LockinShow;
 
 typedef enum {
-    PREVIEW_POLYNOMIAL  = 0,
-    PREVIEW_FIT       = 1,
+    PREVIEW_POLYNOMIAL = 0,
+    PREVIEW_FIT        = 1,
 } LockinPreview;
 
-
 enum {
     PARAM_RANGE_FROM,
     PARAM_RANGE_TO,
@@ -75,9 +61,9 @@ enum {
 typedef struct {
     GwyParams *params;
     GwyLawn *lawn;
-    GwyDataField *amplitude;
-    GwyDataField *phase;
-    GwyDataField *mask;
+    GwyField *amplitude;
+    GwyField *phase;
+    GwyField *mask;
     gdouble xmin;
     gdouble xmax;
     gint nsegments;
@@ -88,63 +74,66 @@ typedef struct {
     GtkWidget *dialog;
     GwyParamTable *table;
     GwyParamTable *table_output;
-    GwyContainer *data;
     GwySelection *selection;
     GwySelection *graph_selection;
     GwyGraphModel *gmodel;
 } ModuleGUI;
 
-static gboolean         module_register         (void);
-static GwyParamDef*     define_module_params    (void);
-static void             lockin                  (GwyContainer *data,
-                                                 GwyRunType runtype);
-static void             execute                 (ModuleArgs *args,
-                                                 GtkWindow *window);
-static GwyDialogOutcome run_gui                 (ModuleArgs *args,
-                                                 GwyContainer *data,
-                                                 gint id);
-static void             param_changed           (ModuleGUI *gui,
-                                                 gint id);
-static void             preview                 (gpointer user_data);
-static void             graph_selected          (GwySelection* selection,
-                                                 gint i,
-                                                 ModuleGUI *gui);
-static void             set_selection           (ModuleGUI *gui);
-static void             point_selection_changed (ModuleGUI *gui,
-                                                 gint id,
-                                                 GwySelection *selection);
-static void             extract_one_curve       (GwyLawn *lawn,
-                                                 GwyGraphCurveModel *gcmodel,
-                                                 gint col,
-                                                 gint row,
-                                                 GwyParams *params);
-static void             do_polylevel            (const gdouble *xdata,
-                                                 const gdouble *ydata,
-                                                 gint ndata,
-                                                 gdouble **nxdata,
-                                                 gdouble **nydata,
-                                                 gint *nndata,
-                                                 const gint *segments,
-                                                 gint segment,
-                                                 gboolean segment_enabled,
-                                                 gdouble from,
-                                                 gdouble to,
-                                                 gint degree,
-                                                 gboolean subtract,
-                                                 gdouble *retparam);
-static gboolean         process_point           (GwyLawn *lawn,
-                                                 gint nsegments,
-                                                 const gint *segments,
-                                                 gint col,
-                                                 gint row,
-                                                 GwyParams *params,
-                                                 gdouble *amplitude,
-                                                 gdouble *phase,
-                                                 gdouble *frequency,
-                                                 gdouble *carrier_phase);
-static void             update_graph_model_props(GwyGraphModel *gmodel,
-                                                 ModuleArgs *args);
-static void             sanitise_params         (ModuleArgs *args);
+static gboolean         module_register           (void);
+static GwyParamDef*     define_module_params      (void);
+static void             module_main               (GwyFile *data,
+                                                   GwyRunModeFlags mode);
+static void             add_mask_and_copy_gradient(GwyFile *data,
+                                                   GwyField *mask,
+                                                   gint id,
+                                                   gint newid);
+static void             execute                   (ModuleArgs *args,
+                                                   GtkWindow *window);
+static GwyDialogOutcome run_gui                   (ModuleArgs *args,
+                                                   GwyFile *data,
+                                                   gint id);
+static void             param_changed             (ModuleGUI *gui,
+                                                   gint id);
+static void             preview                   (gpointer user_data);
+static void             graph_selected            (GwySelection* selection,
+                                                   gint i,
+                                                   ModuleGUI *gui);
+static void             set_selection             (ModuleGUI *gui);
+static void             point_selection_changed   (ModuleGUI *gui,
+                                                   gint id,
+                                                   GwySelection *selection);
+static void             extract_one_curve         (GwyLawn *lawn,
+                                                   GwyGraphCurveModel *gcmodel,
+                                                   gint col,
+                                                   gint row,
+                                                   GwyParams *params);
+static void             do_polylevel              (const gdouble *xdata,
+                                                   const gdouble *ydata,
+                                                   gint ndata,
+                                                   gdouble **nxdata,
+                                                   gdouble **nydata,
+                                                   gint *nndata,
+                                                   const gint *segments,
+                                                   gint segment,
+                                                   gboolean segment_enabled,
+                                                   gdouble from,
+                                                   gdouble to,
+                                                   gint degree,
+                                                   gboolean subtract,
+                                                   gdouble *retparam);
+static gboolean         process_point             (GwyLawn *lawn,
+                                                   gint nsegments,
+                                                   const gint *segments,
+                                                   gint col,
+                                                   gint row,
+                                                   GwyParams *params,
+                                                   gdouble *amplitude,
+                                                   gdouble *phase,
+                                                   gdouble *frequency,
+                                                   gdouble *carrier_phase);
+static void             update_graph_model_props  (GwyGraphModel *gmodel,
+                                                   ModuleArgs *args);
+static void             sanitise_params           (ModuleArgs *args);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
@@ -162,11 +151,11 @@ static gboolean
 module_register(void)
 {
     gwy_curve_map_func_register("cmap_lockin",
-                                (GwyCurveMapFunc)&lockin,
+                                module_main,
                                 N_("/_Demodulation..."),
                                 NULL,
                                 RUN_MODES,
-                                GWY_MENU_FLAG_CURVE_MAP,
+                                GWY_MENU_FLAG_CMAP,
                                 N_("Detect oscillations in a curve"));
 
     return TRUE;
@@ -216,24 +205,21 @@ define_module_params(void)
 }
 
 static void
-lockin(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
     GwyLawn *lawn = NULL;
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     gint id, newid;
     LockinOutput output;
-    GwyDataField *field;
-    const guchar *gradient;
+    GwyField *field;
 
+    g_return_if_fail(mode & RUN_MODES);
 
-    g_return_if_fail(runtype & RUN_MODES);
-    g_return_if_fail(g_type_from_name("GwyLayerPoint"));
-
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_LAWN, &lawn,
-                                     GWY_APP_LAWN_ID, &id,
-                                     0);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_LAWN, &lawn,
+                                 GWY_APP_LAWN_ID, &id,
+                                 0);
     g_return_if_fail(GWY_IS_LAWN(lawn));
 
     args.lawn = lawn;
@@ -244,64 +230,42 @@ lockin(GwyContainer *data, GwyRunType runtype)
 
     sanitise_params(&args);
 
-    args.amplitude = gwy_data_field_new(gwy_lawn_get_xres(lawn), gwy_lawn_get_yres(lawn),
-                                        gwy_lawn_get_xreal(lawn), gwy_lawn_get_yreal(lawn), TRUE);
-    gwy_data_field_set_xoffset(args.amplitude, gwy_lawn_get_xoffset(lawn));
-    gwy_data_field_set_yoffset(args.amplitude, gwy_lawn_get_yoffset(lawn));
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(args.amplitude), gwy_lawn_get_si_unit_xy(lawn));
+    args.amplitude = gwy_lawn_new_field_alike(lawn, -1, TRUE);
+    gwy_unit_assign(gwy_field_get_unit_xy(args.amplitude), gwy_lawn_get_unit_xy(lawn));
 
-    args.phase = gwy_data_field_new(gwy_lawn_get_xres(lawn), gwy_lawn_get_yres(lawn),
-                                    gwy_lawn_get_xreal(lawn), gwy_lawn_get_yreal(lawn), TRUE);
-    gwy_data_field_set_xoffset(args.phase, gwy_lawn_get_xoffset(lawn));
-    gwy_data_field_set_yoffset(args.phase, gwy_lawn_get_yoffset(lawn));
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(args.phase), gwy_lawn_get_si_unit_xy(lawn));
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_z(args.phase), gwy_si_unit_new("rad"));
+    args.phase = gwy_lawn_new_field_alike(lawn, -1, TRUE);
+    gwy_unit_assign(gwy_field_get_unit_z(args.phase), gwy_unit_new("rad"));
 
+    args.mask = gwy_lawn_new_field_alike(lawn, -1, TRUE);
 
-    args.mask = gwy_data_field_new(gwy_lawn_get_xres(lawn), gwy_lawn_get_yres(lawn),
-                                   gwy_lawn_get_xreal(lawn), gwy_lawn_get_yreal(lawn), TRUE);
-
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, id);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
             goto end;
     }
     if (outcome != GWY_DIALOG_HAVE_RESULT)
-        execute(&args, gwy_app_find_window_for_curve_map(data, id));
+        execute(&args, gwy_data_browser_get_window_for_data(data, GWY_FILE_CMAP, id));
 
 
     output = gwy_params_get_flags(args.params, PARAM_OUTPUT);
     if (output & (1 << OUTPUT_PREVIEW)) {
-        field = gwy_container_get_object(data, gwy_app_get_lawn_preview_key_for_id(id));
-        gwy_data_field_assign(field, args.amplitude);
-        gwy_data_field_data_changed(field);
+        field = gwy_container_get_object(GWY_CONTAINER(data), gwy_file_key_cmap_picture(id));
+        gwy_field_assign(field, args.amplitude);
+        gwy_field_data_changed(field);
     }
     if (output & (1 << OUTPUT_QUANTITY)) {
-        newid = gwy_app_data_browser_add_data_field(args.amplitude, data, TRUE);
-        gwy_app_set_data_field_title(data, newid, _("Amplitude"));
-        if (gwy_data_field_get_max(args.mask) > 0.0) {
-            gwy_container_set_object(data, gwy_app_get_mask_key_for_id(newid), args.mask);
-        }
+        newid = gwy_file_add_image(data, args.amplitude);
+        gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+        gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _("Amplitude"), TRUE);
+        add_mask_and_copy_gradient(data, args.mask, id, newid);
+        gwy_log_add_full(data, GWY_FILE_IMAGE, -1, newid, "cmap::cmap_lockin", NULL);
 
-        if (gwy_container_gis_string(data, gwy_app_get_lawn_palette_key_for_id(id), &gradient)) {
-            gwy_container_set_const_string(data,
-                                           gwy_app_get_data_palette_key_for_id(newid), gradient);
-        }
-
-        gwy_app_channel_log_add(data, -1, newid, "cmap::cmap_lockin", NULL);
-
-        newid = gwy_app_data_browser_add_data_field(args.phase, data, TRUE);
-        gwy_app_set_data_field_title(data, newid, _("Phase"));
-        if (gwy_data_field_get_max(args.mask) > 0.0)
-            gwy_container_set_object(data, gwy_app_get_mask_key_for_id(newid), args.mask);
-
-        if (gwy_container_gis_string(data, gwy_app_get_lawn_palette_key_for_id(id), &gradient)) {
-            gwy_container_set_const_string(data,
-                                           gwy_app_get_data_palette_key_for_id(newid), gradient);
-        }
-
-        gwy_app_channel_log_add(data, -1, newid, "cmap::cmap_lockin", NULL);
+        newid = gwy_file_add_image(data, args.phase);
+        gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+        gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _("Phase"), TRUE);
+        add_mask_and_copy_gradient(data, args.mask, id, newid);
+        gwy_log_add_full(data, GWY_FILE_IMAGE, -1, newid, "cmap::cmap_lockin", NULL);
     }
 
 end:
@@ -311,40 +275,42 @@ end:
     g_object_unref(args.params);
 }
 
-static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+static void
+add_mask_and_copy_gradient(GwyFile *data, GwyField *mask, gint id, gint newid)
 {
-    GtkWidget *hbox, *graph, *dataview, *align, *area;
+    if (gwy_field_get_max(mask) > 0.0)
+        gwy_file_pass_image_mask(data, newid, gwy_field_copy(mask));
+
+    gwy_file_sync_items(data, GWY_FILE_CMAP, id,
+                        data, GWY_FILE_IMAGE, newid,
+                        GWY_FILE_ITEM_PALETTE, FALSE);
+}
+
+static GwyDialogOutcome
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
+{
+    GtkWidget *hbox, *graph, *dataview, *area;
     GwyParamTable *table;
     GwyDialog *dialog;
     ModuleGUI gui;
-    GwyDataField *field;
+    GwyField *field;
     GwyDialogOutcome outcome;
     GwyGraphCurveModel *gcmodel;
-    const guchar *gradient;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
-    gui.data = gwy_container_new();
-    field = gwy_container_get_object(data, gwy_app_get_lawn_preview_key_for_id(id));
-    gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(0), field);
-    if (gwy_container_gis_string(data, gwy_app_get_lawn_palette_key_for_id(id), &gradient))
-        gwy_container_set_const_string(gui.data, gwy_app_get_data_palette_key_for_id(0), gradient);
 
     gui.dialog = gwy_dialog_new(_("Demodulation"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    hbox = gwy_hbox_new(0);
-    gwy_dialog_add_content(GWY_DIALOG(gui.dialog), hbox, TRUE, TRUE, 0);
-
-    align = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-    gtk_box_pack_start(GTK_BOX(hbox), align, FALSE, FALSE, 0);
-
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
-    gtk_container_add(GTK_CONTAINER(align), dataview);
-    gui.selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(dataview), 0, "Point", 1, TRUE);
+    field = gwy_container_get_object(GWY_CONTAINER(data), gwy_file_key_cmap_picture(id));
+    dataview = gwy_create_preview(field, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_CMAP, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_REAL_SQUARE);
+    gui.selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(dataview), GWY_TYPE_LAYER_POINT, 1, TRUE);
     set_selection(&gui);
+    hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), FALSE);
 
     gui.gmodel = gwy_graph_model_new();
     gcmodel = gwy_graph_curve_model_new();
@@ -365,8 +331,6 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_graph_model_add_curve(gui.gmodel, gcmodel);
     g_object_unref(gcmodel);
 
-
-
     graph = gwy_graph_new(gui.gmodel);
     area = gwy_graph_get_area(GWY_GRAPH(graph));
     gwy_graph_enable_user_input(GWY_GRAPH(graph), FALSE);
@@ -376,8 +340,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gtk_widget_set_size_request(graph, PREVIEW_SIZE, PREVIEW_SIZE);
     gtk_box_pack_start(GTK_BOX(hbox), graph, TRUE, TRUE, 0);
 
-
-    hbox = gwy_hbox_new(20);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 20);
     gwy_dialog_add_content(GWY_DIALOG(gui.dialog), hbox, TRUE, TRUE, 4);
 
     table = gui.table = gwy_param_table_new(args->params);
@@ -414,7 +377,6 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     outcome = gwy_dialog_run(dialog);
 
     g_object_unref(gui.gmodel);
-    g_object_unref(gui.data);
 
     return outcome;
 }
@@ -567,11 +529,11 @@ execute(ModuleArgs *args, GtkWindow *window)
     gint k, col, row;
     gboolean fitok;
 
-    gwy_data_field_fill(args->mask, 0);
+    gwy_field_fill(args->mask, 0);
 
-    adata = gwy_data_field_get_data(args->amplitude);
-    pdata = gwy_data_field_get_data(args->phase);
-    mdata = gwy_data_field_get_data(args->mask);
+    adata = gwy_field_get_data(args->amplitude);
+    pdata = gwy_field_get_data(args->phase);
+    mdata = gwy_field_get_data(args->mask);
 
     gwy_app_wait_start(window, _("Fitting..."));
 
@@ -589,17 +551,17 @@ execute(ModuleArgs *args, GtkWindow *window)
         adata[k] = fabs(amplitude);
         mdata[k] = !fitok;
     }
-    gwy_data_field_data_changed(args->amplitude);
-    gwy_data_field_data_changed(args->phase);
-    gwy_data_field_data_changed(args->mask);
+    gwy_field_data_changed(args->amplitude);
+    gwy_field_data_changed(args->phase);
+    gwy_field_data_changed(args->mask);
 
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_z(args->amplitude), gwy_lawn_get_si_unit_curve(args->lawn, quantity));
+    gwy_unit_assign(gwy_field_get_unit_z(args->amplitude), gwy_lawn_get_unit_curve(args->lawn, quantity));
 
     gwy_app_wait_finish();
 
-    if (gwy_data_field_get_max(args->mask) > 0.0) {
-        gwy_data_field_laplace_solve(args->amplitude, args->mask, -1, 1.0);
-        gwy_data_field_laplace_solve(args->phase, args->mask, -1, 1.0);
+    if (gwy_field_get_max(args->mask) > 0.0) {
+        gwy_field_laplace_solve(args->amplitude, args->mask, -1, 1.0);
+        gwy_field_laplace_solve(args->phase, args->mask, -1, 1.0);
     }
 }
 
@@ -689,7 +651,7 @@ do_polylevel(const gdouble *xdata, const gdouble *ydata, gint ndata,
     for (i = 1; i < 6; i++) {
         param[i] = 0;
     }
-    param = gwy_math_fit_polynom(n, xf, yf, degree, param);
+    param = gwy_math_fit_poly(xf, yf, n, degree, param);
 
     *nxdata = g_new(gdouble, n);
     *nydata = g_new(gdouble, n);
@@ -725,13 +687,12 @@ do_polylevel(const gdouble *xdata, const gdouble *ydata, gint ndata,
 
 static gdouble
 func_sine(gdouble x,
-          G_GNUC_UNUSED gint n_param,
           const gdouble *param,
           G_GNUC_UNUSED gpointer user_data,
           gboolean *fres)
 {
     *fres = TRUE;
-    return param[0]*sin(param[1]*x+param[2])+param[3];
+    return param[0]*sin(param[1]*x + param[2]) + param[3];
 }
 
 static gdouble
@@ -811,7 +772,7 @@ process_point(GwyLawn *lawn, gint nsegments, const gint *segments, gint col, gin
     *frequency = guess(ycl, nndata, &guess_amplitude, &guess_phase, &offset);
 
     //TODO: move it outside of the loop
-    fitter = gwy_math_nlfit_new((GwyNLFitFunc)func_sine, gwy_math_nlfit_diff);
+    fitter = gwy_math_nlfit_new(func_sine, NULL);
 
     gwy_clear(fix, 4);
     param[0] = guess_amplitude;
@@ -862,19 +823,19 @@ update_graph_model_props(GwyGraphModel *gmodel, ModuleArgs *args)
     gint carrier = gwy_params_get_int(params, PARAM_CARRIER);
     gint quantity = gwy_params_get_int(params, PARAM_QUANTITY);
     gint show = gwy_params_get_enum(params, PARAM_SHOW);
-    GwySIUnit *yunit;
+    GwyUnit *yunit;
     const gchar *ylabel;
 
     if (show == SHOW_CARRIER) {
-       yunit = gwy_lawn_get_si_unit_curve(lawn, carrier);
+       yunit = gwy_lawn_get_unit_curve(lawn, carrier);
        ylabel = gwy_lawn_get_curve_label(lawn, carrier);
     } else {
-       yunit = gwy_lawn_get_si_unit_curve(lawn, quantity);
+       yunit = gwy_lawn_get_unit_curve(lawn, quantity);
        ylabel = gwy_lawn_get_curve_label(lawn, quantity);
     }
 
     g_object_set(gmodel,
-                 "si-unit-y", yunit,
+                 "unit-y", yunit,
                  "axis-label-bottom", _("sample"),
                  "axis-label-left", ylabel ? ylabel : _("Untitled"),
                  NULL);
