@@ -1,5 +1,5 @@
 /*
- *  $Id: cmap_polylevel.c 29034 2025-12-22 18:51:03Z yeti-dn $
+ *  $Id: cmap_polylevel.c 29061 2026-01-02 15:01:53Z yeti-dn $
  *  Copyright (C) 2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,28 +16,16 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwythreads.h>
-#include <libprocess/lawn.h>
-#include <libprocess/stats.h>
-#include <libprocess/correct.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libgwydgets/gwydataview.h>
-#include <libgwydgets/gwygraph.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-cmap.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
-#include "libgwyddion/gwyomp.h"
+#include <gwy.h>
+#include "libgwyddion/omp.h"
 
 #define RUN_MODES (GWY_RUN_INTERACTIVE)
 
 enum {
     PREVIEW_SIZE = 360,
-    MAXFITPARAMS = 6,
 };
 
 enum {
@@ -50,17 +38,14 @@ enum {
     PARAM_XPOS,
     PARAM_YPOS,
     PARAM_DEGREE,
-    PARAM_MEAN_CURVE,
-    PARAM_APPLY_TO_ALL,
 };
 
 typedef struct {
     GwyParams *params;
     GwyLawn *lawn;
     GwyLawn *result;
-    GwyDataField *field;
+    GwyField *field;
     gint nsegments;
-    gdouble *meanfitparams;
 } ModuleArgs;
 
 typedef struct {
@@ -68,59 +53,51 @@ typedef struct {
     GtkWidget *dialog;
     GwyParamTable *table;
     GwyParamTable *table_output;
-    GwyContainer *data;
     GwySelection *selection;
     GwySelection *graph_selection;
     GwyGraphModel *gmodel;
-    gboolean meanfitdone;
 } ModuleGUI;
 
-static gboolean         module_register         (void);
-static GwyParamDef*     define_module_params    (void);
-static void             polylevel               (GwyContainer *data,
-                                                 GwyRunType runtype);
-static void             execute                 (ModuleArgs *args);
-static GwyDialogOutcome run_gui                 (ModuleArgs *args,
-                                                 GwyContainer *data,
-                                                 gint id);
-static void             param_changed           (ModuleGUI *gui,
-                                                 gint id);
-static void             preview                 (gpointer user_data);
-static void             set_selection           (ModuleGUI *gui);
-static void             point_selection_changed (ModuleGUI *gui,
-                                                 gint id,
-                                                 GwySelection *selection);
-static void             extract_one_curve       (GwyLawn *lawn,
-                                                 GwyGraphCurveModel *gcmodel,
-                                                 gint col,
-                                                 gint row,
-                                                 GwyParams *params);
-static void             convert_one_curve       (GwyGraphCurveModel *gcmodel,
-                                                 GwyParams *params,
-                                                 gdouble *fitparams,
-                                                 gint nsegments,
-                                                 const gint *segments);
-static void             do_polylevel            (const gdouble *xdata,
-                                                 const gdouble *ydata,
-                                                 gdouble *nydata,
-                                                 gint ndata,
-                                                 const gint *segments,
-                                                 gint segment,
-                                                 gboolean segment_enabled,
-                                                 gboolean apply_to_all,
-                                                 gdouble from,
-                                                 gdouble to,
-                                                 gint degree,
-                                                 gboolean subtract,
-                                                 gdouble *retparam);
-static void             graph_selected          (GwySelection* selection,
-                                                 gint i,
-                                                 ModuleGUI *gui);
+static gboolean         module_register        (void);
+static GwyParamDef*     define_module_params   (void);
+static void             module_main            (GwyFile *data,
+                                                GwyRunModeFlags mode);
+static void             execute                (ModuleArgs *args);
+static GwyDialogOutcome run_gui                (ModuleArgs *args,
+                                                GwyFile *data,
+                                                gint id);
+static void             param_changed          (ModuleGUI *gui,
+                                                gint id);
+static void             preview                (gpointer user_data);
+static void             set_selection          (ModuleGUI *gui);
+static void             point_selection_changed(ModuleGUI *gui,
+                                                gint id,
+                                                GwySelection *selection);
+static void             extract_one_curve      (GwyLawn *lawn,
+                                                GwyGraphCurveModel *gcmodel,
+                                                gint col,
+                                                gint row,
+                                                GwyParams *params);
+static void             convert_one_curve      (GwyGraphCurveModel *gcmodel,
+                                                GwyParams *params,
+                                                gdouble *fitparams,
+                                                gint nsegments,
+                                                const gint *segments);
+static void             do_polylevel           (const gdouble *xdata,
+                                                const gdouble *ydata,
+                                                gdouble *nydata,
+                                                gint ndata,
+                                                const gint *segments,
+                                                gint segment,
+                                                gboolean segment_enabled,
+                                                gdouble from,
+                                                gdouble to,
+                                                gint degree,
+                                                gboolean subtract,
+                                                gdouble *retparam);
+
 static void             update_graph_model_props(GwyGraphModel *gmodel,
                                                  ModuleArgs *args);
-static gdouble          polynomial              (gdouble *param,
-                                                 gint degree,
-                                                 gdouble x);
 static void             sanitise_params         (ModuleArgs *args);
 
 static GwyModuleInfo module_info = {
@@ -139,11 +116,11 @@ static gboolean
 module_register(void)
 {
     gwy_curve_map_func_register("cmap_polylevel",
-                                (GwyCurveMapFunc)&polylevel,
+                                module_main,
                                 N_("/Remove _Polynomial Background..."),
                                 NULL,
                                 RUN_MODES,
-                                GWY_MENU_FLAG_CURVE_MAP,
+                                GWY_MENU_FLAG_CMAP,
                                 N_("Remove polynomial background from all curves"));
 
     return TRUE;
@@ -164,48 +141,39 @@ define_module_params(void)
     gwy_param_def_add_int(paramdef, PARAM_XPOS, "xpos", NULL, -1, G_MAXINT, -1);
     gwy_param_def_add_int(paramdef, PARAM_YPOS, "ypos", NULL, -1, G_MAXINT, -1);
     /* Keep calling it order for compatibility. */
-    gwy_param_def_add_int(paramdef, PARAM_DEGREE, "order", _("_Degree"), 0, MAXFITPARAMS-1, 2);
+    gwy_param_def_add_int(paramdef, PARAM_DEGREE, "order", _("_Degree"), 0, 5, 2);
     gwy_param_def_add_double(paramdef, PARAM_RANGE_FROM, "from", _("_From"), 0.0, 1.0, 0.0);
     gwy_param_def_add_double(paramdef, PARAM_RANGE_TO, "to", _("_To"), 0.0, 1.0, 1.0);
     gwy_param_def_add_lawn_segment(paramdef, PARAM_SEGMENT, "segment", NULL);
     gwy_param_def_add_boolean(paramdef, PARAM_ENABLE_SEGMENT, "enable_segment", NULL, FALSE);
-    /* XXX: ’polynome’ is not a word. */
-    gwy_param_def_add_boolean(paramdef, PARAM_MEAN_CURVE, "mean_curve", _("Use _mean polynome"), FALSE);
-    gwy_param_def_add_boolean(paramdef, PARAM_APPLY_TO_ALL, "split", _("Apply to _all segments"), FALSE);
 
     return paramdef;
 }
 
 static void
-polylevel(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
     GwyLawn *lawn = NULL;
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     gint id;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    g_return_if_fail(g_type_from_name("GwyLayerPoint"));
+    g_return_if_fail(mode & RUN_MODES);
 
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_LAWN, &lawn,
-                                     GWY_APP_LAWN_ID, &id,
-                                     0);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_LAWN, &lawn,
+                                 GWY_APP_LAWN_ID, &id,
+                                 0);
     g_return_if_fail(GWY_IS_LAWN(lawn));
 
     args.lawn = lawn;
     args.nsegments = gwy_lawn_get_n_segments(lawn);
     args.params = gwy_params_new_from_settings(define_module_params());
-    args.meanfitparams = g_new(gdouble, MAXFITPARAMS);
     sanitise_params(&args);
 
-    args.field = gwy_data_field_new(gwy_lawn_get_xres(lawn), gwy_lawn_get_yres(lawn),
-                                    gwy_lawn_get_xreal(lawn), gwy_lawn_get_yreal(lawn), TRUE);
-    gwy_data_field_set_xoffset(args.field, gwy_lawn_get_xoffset(lawn));
-    gwy_data_field_set_yoffset(args.field, gwy_lawn_get_yoffset(lawn));
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(args.field), gwy_lawn_get_si_unit_xy(lawn));
+    args.field = gwy_lawn_new_field_alike(lawn, -1, TRUE);
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, id);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -214,51 +182,39 @@ polylevel(GwyContainer *data, GwyRunType runtype)
     if (outcome != GWY_DIALOG_HAVE_RESULT)
         execute(&args);
 
-    gwy_app_curve_map_log_add(data, -1, id, "cmap::cmap_polylevel", NULL);
+    gwy_log_add(data, GWY_FILE_CMAP, id, id);
 
 end:
-    GWY_OBJECT_UNREF(args.result);
+    g_clear_object(&args.result);
     g_object_unref(args.field);
     g_object_unref(args.params);
-    g_free(args.meanfitparams);
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
-    GtkWidget *hbox, *graph, *dataview, *align, *area;
+    GtkWidget *hbox, *graph, *dataview, *area;
     GwyParamTable *table;
     GwyDialog *dialog;
     ModuleGUI gui;
-    GwyDataField *field;
+    GwyField *field;
     GwyDialogOutcome outcome;
     GwyGraphCurveModel *gcmodel;
-    const guchar *gradient;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
-    gui.data = gwy_container_new();
-    gui.meanfitdone = FALSE;
- 
-    field = gwy_container_get_object(data, gwy_app_get_lawn_preview_key_for_id(id));
-    gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(0), field);
-    if (gwy_container_gis_string(data, gwy_app_get_lawn_palette_key_for_id(id), &gradient))
-        gwy_container_set_const_string(gui.data, gwy_app_get_data_palette_key_for_id(0), gradient);
 
     gui.dialog = gwy_dialog_new(_("Remove Polynomial Background"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    hbox = gwy_hbox_new(0);
-    gwy_dialog_add_content(GWY_DIALOG(gui.dialog), hbox, TRUE, TRUE, 0);
-
-    align = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-    gtk_box_pack_start(GTK_BOX(hbox), align, FALSE, FALSE, 0);
-
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
-    gtk_container_add(GTK_CONTAINER(align), dataview);
-    gui.selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(dataview), 0, "Point", 1, TRUE);
+    field = gwy_container_get_object(GWY_CONTAINER(data), gwy_file_key_cmap_picture(id));
+    dataview = gwy_create_preview(field, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_CMAP, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_REAL_SQUARE);
+    gui.selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(dataview), GWY_TYPE_LAYER_POINT, 1, TRUE);
     set_selection(&gui);
+    hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), FALSE);
 
     gui.gmodel = gwy_graph_model_new();
     gcmodel = gwy_graph_curve_model_new();
@@ -270,7 +226,6 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_graph_model_add_curve(gui.gmodel, gcmodel);
     g_object_unref(gcmodel);
 
-
     gcmodel = gwy_graph_curve_model_new();
     g_object_set(gcmodel,
                  "mode", GWY_GRAPH_CURVE_LINE,
@@ -284,12 +239,13 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     area = gwy_graph_get_area(GWY_GRAPH(graph));
     gwy_graph_enable_user_input(GWY_GRAPH(graph), FALSE);
     gwy_graph_area_set_status(GWY_GRAPH_AREA(area), GWY_GRAPH_STATUS_XSEL);
-    gwy_graph_area_set_selection_editable(GWY_GRAPH_AREA(area), TRUE);
+    gwy_graph_area_set_selection_editable(GWY_GRAPH_AREA(area), FALSE);
     gui.graph_selection = gwy_graph_area_get_selection(GWY_GRAPH_AREA(area), GWY_GRAPH_STATUS_XSEL);
     gtk_widget_set_size_request(graph, PREVIEW_SIZE, PREVIEW_SIZE);
     gtk_box_pack_start(GTK_BOX(hbox), graph, TRUE, TRUE, 0);
 
-    hbox = gwy_hbox_new(20);
+
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 20);
     gwy_dialog_add_content(GWY_DIALOG(gui.dialog), hbox, TRUE, TRUE, 4);
 
     table = gui.table = gwy_param_table_new(args->params);
@@ -306,41 +262,24 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
         gwy_param_table_append_lawn_segment(table, PARAM_SEGMENT, args->lawn);
         gwy_param_table_add_enabler(table, PARAM_ENABLE_SEGMENT, PARAM_SEGMENT);
     }
-    gwy_dialog_add_param_table(dialog, table);
-    gtk_box_pack_start(GTK_BOX(hbox), gwy_param_table_widget(table), FALSE, FALSE, 0);
 
-    table = gui.table_output = gwy_param_table_new(args->params);
-    gwy_param_table_append_checkbox(table, PARAM_MEAN_CURVE);
-    if (args->nsegments) {
-        gwy_param_table_append_checkbox(table, PARAM_APPLY_TO_ALL);
-    }
     gwy_dialog_add_param_table(dialog, table);
     gtk_box_pack_start(GTK_BOX(hbox), gwy_param_table_widget(table), FALSE, FALSE, 0);
 
     g_signal_connect_swapped(gui.table, "param-changed", G_CALLBACK(param_changed), &gui);
-    g_signal_connect_swapped(gui.table_output, "param-changed", G_CALLBACK(param_changed), &gui);
     g_signal_connect_swapped(gui.selection, "changed", G_CALLBACK(point_selection_changed), &gui);
-    g_signal_connect(gui.graph_selection, "changed", G_CALLBACK(graph_selected), &gui);    
     gwy_dialog_set_preview_func(dialog, GWY_PREVIEW_IMMEDIATE, preview, &gui, NULL);
 
     outcome = gwy_dialog_run(dialog);
 
     g_object_unref(gui.gmodel);
-    g_object_unref(gui.data);
+
     return outcome;
 }
 
 static void
-param_changed(ModuleGUI *gui, gint id)
+param_changed(ModuleGUI *gui, G_GNUC_UNUSED gint id)
 {
-    ModuleArgs *args = gui->args;
-    GwyParams *params = args->params;
-
-    if (id < 0 || id == PARAM_ENABLE_SEGMENT) {
-        gwy_param_table_set_sensitive(gui->table_output, PARAM_APPLY_TO_ALL,
-                                      gwy_params_get_boolean(params, PARAM_ENABLE_SEGMENT));
-    }
-
     gwy_dialog_invalidate(GWY_DIALOG(gui->dialog));
 }
 
@@ -375,95 +314,28 @@ point_selection_changed(ModuleGUI *gui, gint id, GwySelection *selection)
     gwy_param_table_param_changed(gui->table, PARAM_YPOS);
 }
 
-static void
-graph_selected(GwySelection* selection, gint i, ModuleGUI *gui)
-{
-    gdouble range[2];
-    gdouble xfrom, xto, cxfrom, cxto;
-    gboolean have_range = TRUE;
-    GwyGraphCurveModel *gc;
-    
-    g_return_if_fail(i <= 0);
-    
-    if (gwy_selection_get_data(selection, NULL) <= 0)
-        have_range = FALSE;
-    else {
-        gwy_selection_get_object(selection, 0, range);
-        if (range[0] == range[1])
-            have_range = FALSE;
-    }
-    if (have_range) {
-        gc = gwy_graph_model_get_curve(gui->gmodel, 0);
-        gwy_graph_curve_model_get_x_range(gc, &cxfrom, &cxto);
-
-        xfrom = MIN(range[0], range[1]);
-        xto = MAX(range[0], range[1]);
-
-        xfrom = MAX(xfrom, cxfrom);
-        xto = MIN(xto, cxto);
-    
-        gwy_param_table_set_double(gui->table, PARAM_RANGE_FROM, (xfrom - cxfrom)/(cxto - cxfrom));
-        gwy_param_table_set_double(gui->table, PARAM_RANGE_TO, (xto - cxfrom)/(cxto - cxfrom));
-    }
-}
-
 static void
 preview(gpointer user_data)
 {
     ModuleGUI *gui = (ModuleGUI*)user_data;
     ModuleArgs *args = gui->args;
-    GwyLawn *lawn = args->lawn;
     GwyParams *params = args->params;
     gint col = gwy_params_get_int(params, PARAM_XPOS);
     gint row = gwy_params_get_int(params, PARAM_YPOS);
     gdouble from = gwy_params_get_double(params, PARAM_RANGE_FROM);
     gdouble to = gwy_params_get_double(params, PARAM_RANGE_TO);
-    gboolean mc = gwy_params_get_boolean(params, PARAM_MEAN_CURVE);
-    gint degree = gwy_params_get_int(params, PARAM_DEGREE);
     gdouble sel[2];
     GwyGraphCurveModel *gc;
     gdouble xfrom, xto;
     gdouble *xfit, *yfit, *fitparams, x;
-    gint i, k, scol, srow, nfit = 100;
-    gint xres = gwy_lawn_get_xres(lawn), yres = gwy_lawn_get_yres(lawn);
+    gint i, nfit = 100;
 
     fitparams = g_new(gdouble, 6);
+
     gc = gwy_graph_model_get_curve(gui->gmodel, 0);
-
-    if (mc) {
-        if (!gui->meanfitdone) {
-            for (i = 0; i < MAXFITPARAMS; i++)
-                args->meanfitparams[i] = 0;
-
-            for (k = 0; k < xres*yres; k++) {
-                scol = k % xres;
-                srow = k/xres;
-
-                extract_one_curve(lawn, gc, scol, srow, params);
-
-                convert_one_curve(gc, params, fitparams, args->nsegments,
-                                  gwy_lawn_get_segments(lawn, scol, srow, NULL));
-
-                for (i = 0; i < MAXFITPARAMS; i++)
-                    args->meanfitparams[i] += fitparams[i];
-            }
-            for (i = 0; i < MAXFITPARAMS; i++) {
-                args->meanfitparams[i] /= (xres*yres);
-                fitparams[i] = args->meanfitparams[i];
-            }
-            gui->meanfitdone = TRUE;
-        }
-
-        for (i = 0; i < MAXFITPARAMS; i++)
-            fitparams[i] = args->meanfitparams[i];
-
-        extract_one_curve(lawn, gc, col, row, params);
-    }
-    else {
-        extract_one_curve(lawn, gc, col, row, params);
-        convert_one_curve(gc, params, fitparams, args->nsegments,
-                          gwy_lawn_get_segments(lawn, col, row, NULL));
-    }
+    extract_one_curve(args->lawn, gc, col, row, params);
+    convert_one_curve(gc, params, fitparams,  args->nsegments,
+                      gwy_lawn_get_segments(args->lawn, col, row, NULL));
     update_graph_model_props(gui->gmodel, args);
 
     gwy_graph_curve_model_get_x_range(gc, &xfrom, &xto);
@@ -476,7 +348,8 @@ preview(gpointer user_data)
     yfit = g_new(gdouble, nfit);
     for (i = 0; i < nfit; i++) {
         x = xfit[i];
-        yfit[i] = polynomial(fitparams, degree, x);
+        yfit[i] = fitparams[0] + x*(fitparams[1] + x*(fitparams[2] + x*(fitparams[3]
+                                                                        + x*(fitparams[4] + x*fitparams[5]))));
     }
     gc = gwy_graph_model_get_curve(gui->gmodel, 1);
     gwy_graph_curve_model_set_data(gc, xfit, yfit, nfit);
@@ -497,9 +370,7 @@ execute(ModuleArgs *args)
     gdouble to = gwy_params_get_double(params, PARAM_RANGE_TO);
     gint degree = gwy_params_get_int(params, PARAM_DEGREE);
     gboolean segment_enabled = args->nsegments ? gwy_params_get_boolean(params, PARAM_ENABLE_SEGMENT) : FALSE;
-    gboolean apply_to_all = gwy_params_get_boolean(params, PARAM_APPLY_TO_ALL);
     gint segment = segment_enabled ? gwy_params_get_int(params, PARAM_SEGMENT) : -1;
-    gboolean mc = gwy_params_get_boolean(params, PARAM_MEAN_CURVE);
 
     GwyLawn *lawn = args->lawn;
     gint xres = gwy_lawn_get_xres(lawn), yres = gwy_lawn_get_yres(lawn);
@@ -508,7 +379,7 @@ execute(ModuleArgs *args)
 
     const gdouble *cdx, *cdy;
     gdouble *ncdy;
-    gint ndata, i, k, col, row;
+    gint ndata, k, col, row;
 
     gint nsegments = gwy_lawn_get_n_segments(lawn);
     gint *segments = g_new(gint, 2*nsegments);
@@ -524,14 +395,7 @@ execute(ModuleArgs *args)
         g_array_set_size(newcdata, 2*ndata);
         ncdy = &g_array_index(newcdata, gdouble, ndata);
 
-        if (mc) {
-            for (i = 0; i < ndata; i++) {
-               ncdy[i] = cdy[i] - polynomial(args->meanfitparams, degree, cdx[i]);
-            }
-        } else {
-            do_polylevel(cdx, cdy, ncdy, ndata, segments, segment, segment_enabled, apply_to_all, 
-                         from, to, degree, TRUE, NULL);
-        }
+        do_polylevel(cdx, cdy, ncdy, ndata, segments, segment, segment_enabled, from, to, degree, TRUE, NULL);
 
         gwy_lawn_set_curve_data(lawn, col, row, ordinate, ncdy);
     }
@@ -552,21 +416,6 @@ extract_one_curve(GwyLawn *lawn, GwyGraphCurveModel *gcmodel,
     gwy_graph_curve_model_set_data(gcmodel, xdata, ydata, ndata);
 }
 
-static gdouble
-polynomial(gdouble *param, gint degree, gdouble x)
-{
-    gdouble val;
-    gint i;
-    val = param[0];
-    
-    val = param[degree];
-    for (i = degree; i >= 1; i--) {
-        val = val*x + param[i-1];
-    }
-    return val;
-}
-
-
 //subtract means that the data will be changed, which is good for final operation
 //fitparams are used to get the fit results and plot it with original data, which is good for GUI
 // FIXME: Almost duplicate with cmap_lockin.c
@@ -574,14 +423,14 @@ static void
 do_polylevel(const gdouble *xdata, const gdouble *ydata,
              gdouble *nydata, gint ndata,
              const gint *segments,
-             gint segment, gboolean segment_enabled, gboolean apply_to_all,
+             gint segment, gboolean segment_enabled,
              gdouble from, gdouble to,
              gint degree, gboolean subtract, gdouble *fitparams)
 {
     gint i, j, n;
     gdouble startval, endval, xmin, xmax, ymin, ymax;
     gdouble *xf, *yf;
-    gdouble param[MAXFITPARAMS], x;
+    gdouble param[6], x;
     gint segment_from, segment_to;
 
     /* FIXME: This function must be able to fail gracefully and report it to the caller – the fitting can definitely
@@ -629,24 +478,17 @@ do_polylevel(const gdouble *xdata, const gdouble *ydata,
         }
     }
 
-    gwy_math_fit_polynom(n, xf, yf, degree, param);
+    gwy_math_fit_poly(xf, yf, n, degree, param);
 
     if (subtract && nydata) {
         for (i = 0; i < ndata; i++) {
             x = xdata[i];
-            if (!segment_enabled || apply_to_all) 
-                nydata[i] = ydata[i] - polynomial(param, degree, x);
-            else {
-                if (i >= segment_from && i < segment_to)
-                    nydata[i] = ydata[i] - polynomial(param, degree, x);
-                else
-                    nydata[i] = ydata[i];
-            }
+            nydata[i] = ydata[i] - (param[0] + x*(param[1] + x*(param[2] + x*(param[3] + x*(param[4] + x*param[5])))));
         }
     }
 
     if (fitparams)
-        gwy_assign(fitparams, param, MAXFITPARAMS);
+        gwy_assign(fitparams, param, 6);
 
     g_free(xf);
     g_free(yf);
@@ -660,15 +502,13 @@ convert_one_curve(GwyGraphCurveModel *gcmodel, GwyParams *params, gdouble *fitpa
     gdouble to = gwy_params_get_double(params, PARAM_RANGE_TO);
     gint degree = gwy_params_get_int(params, PARAM_DEGREE);
     gboolean segment_enabled = nsegments ? gwy_params_get_boolean(params, PARAM_ENABLE_SEGMENT) : FALSE;
-    gboolean apply_to_all = gwy_params_get_boolean(params, PARAM_APPLY_TO_ALL);
     gint segment = segment_enabled ? gwy_params_get_int(params, PARAM_SEGMENT) : -1;
 
     const gdouble *xdata = gwy_graph_curve_model_get_xdata(gcmodel);
     const gdouble *ydata = gwy_graph_curve_model_get_ydata(gcmodel);
     gint ndata = gwy_graph_curve_model_get_ndata(gcmodel);
 
-    do_polylevel(xdata, ydata, NULL, ndata, segments, segment, segment_enabled, apply_to_all,
-                 from, to, degree, FALSE, fitparams);
+    do_polylevel(xdata, ydata, NULL, ndata, segments, segment, segment_enabled, from, to, degree, FALSE, fitparams);
 }
 
 static void
@@ -678,17 +518,17 @@ update_graph_model_props(GwyGraphModel *gmodel, ModuleArgs *args)
     GwyParams *params = args->params;
     gint abscissa = gwy_params_get_int(params, PARAM_ABSCISSA);
     gint ordinate = gwy_params_get_int(params, PARAM_ORDINATE);
-    GwySIUnit *xunit, *yunit;
+    GwyUnit *xunit, *yunit;
     const gchar *xlabel, *ylabel;
 
-    xunit = gwy_lawn_get_si_unit_curve(lawn, abscissa);
+    xunit = gwy_lawn_get_unit_curve(lawn, abscissa);
     xlabel = gwy_lawn_get_curve_label(lawn, abscissa);
-    yunit = gwy_lawn_get_si_unit_curve(lawn, ordinate);
+    yunit = gwy_lawn_get_unit_curve(lawn, ordinate);
     ylabel = gwy_lawn_get_curve_label(lawn, ordinate);
 
     g_object_set(gmodel,
-                 "si-unit-x", xunit,
-                 "si-unit-y", yunit,
+                 "unit-x", xunit,
+                 "unit-y", yunit,
                  "axis-label-bottom", xlabel ? xlabel : _("Untitled"),
                  "axis-label-left", ylabel ? ylabel : _("Untitled"),
                  NULL);
