@@ -1,5 +1,5 @@
 /*
- *  $Id: cmap_possearch.c 28939 2025-12-02 15:47:48Z klapetek $
+ *  $Id: cmap_possearch.c 28905 2025-11-24 15:51:18Z yeti-dn $
  *  Copyright (C) 2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,22 +16,11 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwythreads.h>
-#include <libprocess/lawn.h>
-#include <libprocess/stats.h>
-#include <libprocess/correct.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libgwydgets/gwydataview.h>
-#include <libgwydgets/gwygraph.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-cmap.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
-#include "libgwyddion/gwyomp.h"
+#include <gwy.h>
+#include "libgwyddion/omp.h"
 
 #define RUN_MODES (GWY_RUN_INTERACTIVE)
 
@@ -70,9 +59,9 @@ enum {
 typedef struct {
     GwyParams *params;
     GwyLawn *lawn;
-    GwyDataField *field;
-    GwyDataField *mask;
-    GwySIValueFormat *vf;
+    GwyField *field;
+    GwyField *mask;
+    GwyValueFormat *vf;
     /* Cached input data properties. */
     gint nsegments;
 } ModuleArgs;
@@ -82,46 +71,49 @@ typedef struct {
     GtkWidget *dialog;
     GwyParamTable *table;
     GwyParamTable *table_output;
-    GwyContainer *data;
     GwySelection *selection;
     GwySelection *graph_selection;
     GwyGraphModel *gmodel;
 } ModuleGUI;
 
-static gboolean         module_register         (void);
-static GwyParamDef*     define_module_params    (void);
-static void             possearch               (GwyContainer *data,
-                                                 GwyRunType runtype);
-static void             execute                 (ModuleArgs *args);
-static GwyDialogOutcome run_gui                 (ModuleArgs *args,
-                                                 GwyContainer *data,
-                                                 gint id);
-static void             param_changed           (ModuleGUI *gui,
-                                                 gint id);
-static void             preview                 (gpointer user_data);
-static void             set_selection           (ModuleGUI *gui);
-static void             point_selection_changed (ModuleGUI *gui,
-                                                 gint id,
-                                                 GwySelection *selection);
-static gboolean         locate_in_one_curve     (GwyLawn *lawn,
-                                                 gint col,
-                                                 gint row,
-                                                 gint pick_curve,
-                                                 gint search_curve,
-                                                 gint segment,
-                                                 PosSearchMethod method,
-                                                 PosSearchDir dir,
-                                                 gdouble threshold,
-                                                 gdouble *result);
-static void             extract_one_curve       (GwyLawn *lawn,
-                                                 GwyGraphCurveModel *gcmodel,
-                                                 gint col,
-                                                 gint row,
-                                                 gint segment,
-                                                 GwyParams *params);
-static void             update_graph_model_props(GwyGraphModel *gmodel,
-                                                 ModuleArgs *args);
-static void             sanitise_params         (ModuleArgs *args);
+static gboolean         module_register           (void);
+static GwyParamDef*     define_module_params      (void);
+static void             module_main               (GwyFile *data,
+                                                   GwyRunModeFlags mode);
+static void             execute                   (ModuleArgs *args);
+static GwyDialogOutcome run_gui                   (ModuleArgs *args,
+                                                   GwyFile *data,
+                                                   gint id);
+static void             param_changed             (ModuleGUI *gui,
+                                                   gint id);
+static void             preview                   (gpointer user_data);
+static void             set_selection             (ModuleGUI *gui);
+static void             point_selection_changed   (ModuleGUI *gui,
+                                                   gint id,
+                                                   GwySelection *selection);
+static gboolean         locate_in_one_curve       (GwyLawn *lawn,
+                                                   gint col,
+                                                   gint row,
+                                                   gint pick_curve,
+                                                   gint search_curve,
+                                                   gint segment,
+                                                   PosSearchMethod method,
+                                                   PosSearchDir dir,
+                                                   gdouble threshold,
+                                                   gdouble *result);
+static void             extract_one_curve         (GwyLawn *lawn,
+                                                   GwyGraphCurveModel *gcmodel,
+                                                   gint col,
+                                                   gint row,
+                                                   gint segment,
+                                                   GwyParams *params);
+static void             update_graph_model_props  (GwyGraphModel *gmodel,
+                                                   ModuleArgs *args);
+static void             add_mask_and_copy_gradient(GwyFile *data,
+                                                   GwyField *mask,
+                                                   gint id,
+                                                   gint newid);
+static void             sanitise_params           (ModuleArgs *args);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
@@ -139,11 +131,11 @@ static gboolean
 module_register(void)
 {
     gwy_curve_map_func_register("cmap_possearch",
-                                (GwyCurveMapFunc)&possearch,
+                                module_main,
                                 N_("/_Position Search..."),
                                 NULL,
                                 RUN_MODES,
-                                GWY_MENU_FLAG_CURVE_MAP,
+                                GWY_MENU_FLAG_CMAP,
                                 N_("Search for a threshold"));
 
     return TRUE;
@@ -160,7 +152,7 @@ define_module_params(void)
         { N_("Left to right"),    DIR_LR,    },
         { N_("Right to left"),    DIR_RL,    },
     };
-    static const GwyEnum outputs[] = {
+     static const GwyEnum outputs[] = {
         { N_("Extract quantity"),   (1 << OUTPUT_QUANTITY),   },
         { N_("Set preview"),        (1 << OUTPUT_PREVIEW),    },
     };
@@ -188,40 +180,33 @@ define_module_params(void)
 }
 
 static void
-possearch(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
     GwyLawn *lawn = NULL;
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     PosSearchOutput output;
-    GwyDataField *field;
+    GwyField *field;
     gint id, newid;
-    const guchar *gradient;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    g_return_if_fail(g_type_from_name("GwyLayerPoint"));
+    g_return_if_fail(mode & RUN_MODES);
 
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_LAWN, &lawn,
-                                     GWY_APP_LAWN_ID, &id,
-                                     0);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_LAWN, &lawn,
+                                 GWY_APP_LAWN_ID, &id,
+                                 0);
     g_return_if_fail(GWY_IS_LAWN(lawn));
     args.lawn = lawn;
     args.nsegments = gwy_lawn_get_n_segments(lawn);
     args.params = gwy_params_new_from_settings(define_module_params());
     sanitise_params(&args);
 
-    args.vf = gwy_lawn_get_value_format_curve(lawn, 0, GWY_SI_UNIT_FORMAT_VFMARKUP, NULL);
+    args.vf = gwy_lawn_get_value_format_curve(lawn, 0, GWY_UNIT_FORMAT_VFMARKUP, NULL);
 
-    args.field = gwy_data_field_new(gwy_lawn_get_xres(lawn), gwy_lawn_get_yres(lawn),
-                                    gwy_lawn_get_xreal(lawn), gwy_lawn_get_yreal(lawn), TRUE);
-    gwy_data_field_set_xoffset(args.field, gwy_lawn_get_xoffset(lawn));
-    gwy_data_field_set_yoffset(args.field, gwy_lawn_get_yoffset(lawn));
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(args.field), gwy_lawn_get_si_unit_xy(lawn));
-    args.mask = gwy_data_field_new_alike(args.field, TRUE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(args.mask), NULL);
+    args.field = gwy_lawn_new_field_alike(lawn, -1, TRUE);
+    args.mask = gwy_field_new_alike(args.field, TRUE);
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, id);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -232,22 +217,16 @@ possearch(GwyContainer *data, GwyRunType runtype)
 
     output = gwy_params_get_flags(args.params, PARAM_OUTPUT);
     if (output & (1 << OUTPUT_PREVIEW)) {
-        field = gwy_container_get_object(data, gwy_app_get_lawn_preview_key_for_id(id));
-        gwy_data_field_assign(field, args.field);
-        gwy_data_field_data_changed(field);
+        field = gwy_container_get_object(GWY_CONTAINER(data), gwy_file_key_cmap_picture(id));
+        gwy_field_assign(field, args.field);
+        gwy_field_data_changed(field);
     }
     if (output & (1 << OUTPUT_QUANTITY)) {
-        newid = gwy_app_data_browser_add_data_field(args.field, data, TRUE);
-        gwy_app_set_data_field_title(data, newid, _("Position"));
-        if (gwy_data_field_get_max(args.mask) > 0.0)
-            gwy_container_set_object(data, gwy_app_get_mask_key_for_id(newid), args.mask);
-
-        if (gwy_container_gis_string(data, gwy_app_get_lawn_palette_key_for_id(id), &gradient)) {
-            gwy_container_set_const_string(data,
-                                           gwy_app_get_data_palette_key_for_id(newid), gradient);
-        }
-
-        gwy_app_channel_log_add(data, -1, newid, "cmap::cmap_possearch", NULL);
+        newid = gwy_file_add_image(data, args.field);
+        gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+        gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _("Position"), TRUE);
+        add_mask_and_copy_gradient(data, args.mask, id, newid);
+        gwy_log_add_full(data, GWY_FILE_IMAGE, -1, newid, "cmap::cmap_possearch", NULL);
     }
 
 end:
@@ -256,40 +235,40 @@ end:
     g_object_unref(args.params);
 }
 
-static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+static void
+add_mask_and_copy_gradient(GwyFile *data, GwyField *mask, gint id, gint newid)
 {
-    GtkWidget *hbox, *graph, *dataview, *align, *area;
+    if (gwy_field_get_max(mask) > 0.0)
+        gwy_file_pass_image_mask(data, newid, gwy_field_copy(mask));
+
+    gwy_file_sync_items(data, GWY_FILE_CMAP, id,
+                        data, GWY_FILE_IMAGE, newid,
+                        GWY_FILE_ITEM_PALETTE, FALSE);
+}
+
+static GwyDialogOutcome
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
+{
+    GtkWidget *hbox, *graph, *dataview, *area;
     GwyParamTable *table;
     GwyDialog *dialog;
     ModuleGUI gui;
-    GwyDataField *field = args->field;
     GwyDialogOutcome outcome;
     GwyGraphCurveModel *gcmodel;
-    const guchar *gradient;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
-    gui.data = gwy_container_new();
-    //field = gwy_container_get_object(data, gwy_app_get_lawn_preview_key_for_id(id));
-    gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(0), field);
-    if (gwy_container_gis_string(data, gwy_app_get_lawn_palette_key_for_id(id), &gradient))
-        gwy_container_set_const_string(gui.data, gwy_app_get_data_palette_key_for_id(0), gradient);
 
     gui.dialog = gwy_dialog_new(_("Search for threshold"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    hbox = gwy_hbox_new(0);
-    gwy_dialog_add_content(GWY_DIALOG(gui.dialog), hbox, TRUE, TRUE, 0);
-
-    align = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-    gtk_box_pack_start(GTK_BOX(hbox), align, FALSE, FALSE, 0);
-
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
-    gtk_container_add(GTK_CONTAINER(align), dataview);
-    gui.selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(dataview), 0, "Point", 1, TRUE);
+    dataview = gwy_create_preview(args->field, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_CMAP, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_REAL_SQUARE);
+    gui.selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(dataview), GWY_TYPE_LAYER_POINT, 1, TRUE);
     set_selection(&gui);
+    hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), FALSE);
 
     gui.gmodel = gwy_graph_model_new();
     gcmodel = gwy_graph_curve_model_new();
@@ -309,7 +288,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gtk_widget_set_size_request(graph, PREVIEW_SIZE, PREVIEW_SIZE);
     gtk_box_pack_start(GTK_BOX(hbox), graph, TRUE, TRUE, 0);
 
-    hbox = gwy_hbox_new(20);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 20);
     gwy_dialog_add_content(GWY_DIALOG(gui.dialog), hbox, TRUE, TRUE, 4);
 
     table = gui.table = gwy_param_table_new(args->params);
@@ -343,7 +322,6 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     outcome = gwy_dialog_run(dialog);
 
     g_object_unref(gui.gmodel);
-    g_object_unref(gui.data);
 
     return outcome;
 }
@@ -366,7 +344,7 @@ param_changed(ModuleGUI *gui, gint id)
         gint yres = gwy_lawn_get_yres(args->lawn);
         gdouble min, max;
 
-        args->vf = gwy_lawn_get_value_format_curve(args->lawn, curve, GWY_SI_UNIT_FORMAT_VFMARKUP, args->vf);
+        args->vf = gwy_lawn_get_value_format_curve(args->lawn, curve, GWY_UNIT_FORMAT_VFMARKUP, args->vf);
 
         min = G_MAXDOUBLE;
         max = -G_MAXDOUBLE;
@@ -383,9 +361,8 @@ param_changed(ModuleGUI *gui, gint id)
         gwy_param_table_slider_restrict_range(gui->table, PARAM_THRESHOLD,
                                               min/args->vf->magnitude, max/args->vf->magnitude);
         gwy_param_table_set_unitstr(gui->table, PARAM_THRESHOLD, args->vf->units);
-
     }
-     if (id != PARAM_OUTPUT)
+    if (id != PARAM_OUTPUT)
         gwy_dialog_invalidate(GWY_DIALOG(gui->dialog));
 }
 
@@ -438,7 +415,7 @@ preview(gpointer user_data)
     gdouble x;
 
     execute(args);
-    gwy_data_field_data_changed(args->field);
+    gwy_field_data_changed(args->field);
 
 
     extract_one_curve(args->lawn, gwy_graph_model_get_curve(gui->gmodel, 0), col, row, segment, params);
@@ -462,15 +439,15 @@ execute(ModuleArgs *args)
     PosSearchDir dir = gwy_params_get_enum(params, PARAM_DIR);
     gdouble threshold = gwy_params_get_double(params, PARAM_THRESHOLD)*args->vf->magnitude;
     GwyLawn *lawn = args->lawn;
-    GwyDataField *field = args->field, *mask = args->mask;
+    GwyField *field = args->field, *mask = args->mask;
     gint xres = gwy_lawn_get_xres(lawn), yres = gwy_lawn_get_yres(lawn);
     gdouble *data, *mdata;
 
     /* First locate the point in all data.  This allows interpolating the failed ones. */
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_z(field), gwy_lawn_get_si_unit_curve(lawn, pick_curve));
-    gwy_data_field_clear(mask);
-    data = gwy_data_field_get_data(field);
-    mdata = gwy_data_field_get_data(mask);
+    gwy_unit_assign(gwy_field_get_unit_z(field), gwy_lawn_get_unit_curve(lawn, pick_curve));
+    gwy_field_clear(mask);
+    data = gwy_field_get_data(field);
+    mdata = gwy_field_get_data(mask);
 
     /*
 #ifdef _OPENMP
@@ -493,8 +470,8 @@ execute(ModuleArgs *args)
         }
     }
 
-    if (gwy_data_field_get_max(mask) > 0.0)
-        gwy_data_field_laplace_solve(field, mask, -1, 1.0);
+    if (gwy_field_get_max(mask) > 0.0)
+        gwy_field_laplace_solve(field, mask, -1, 1.0);
 }
 
 static gboolean
@@ -632,15 +609,15 @@ update_graph_model_props(GwyGraphModel *gmodel, ModuleArgs *args)
     GwyLawn *lawn = args->lawn;
     GwyParams *params = args->params;
     gint search_curve = gwy_params_get_int(params, PARAM_SEARCH_CURVE);
-    GwySIUnit *yunit;
+    GwyUnit *yunit;
     const gchar *xlabel, *ylabel;
 
     xlabel = g_strdup("sample");
-    yunit = gwy_lawn_get_si_unit_curve(lawn, search_curve);
+    yunit = gwy_lawn_get_unit_curve(lawn, search_curve);
     ylabel = gwy_lawn_get_curve_label(lawn, search_curve);
 
     g_object_set(gmodel,
-                 "si-unit-y", yunit,
+                 "unit-y", yunit,
                  "axis-label-bottom", xlabel ? xlabel : _("Untitled"),
                  "axis-label-left", ylabel ? ylabel : _("Untitled"),
                  "label-visible", FALSE,
