@@ -1,5 +1,5 @@
 /*
- *  $Id: cmap_simplemech.c 28321 2025-08-02 10:49:18Z klapetek $
+ *  $Id: cmap_simplemech.c 28905 2025-11-24 15:51:18Z yeti-dn $
  *  Copyright (C) 2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,23 +16,11 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwythreads.h>
-#include <libprocess/stats.h>
-#include <libprocess/correct.h>
-#include <libprocess/lawn.h>
-#include <libprocess/gwyprocess.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libgwydgets/gwydataview.h>
-#include <libgwydgets/gwygraph.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-cmap.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
-#include "libgwyddion/gwyomp.h"
+#include <gwy.h>
+#include "libgwyddion/omp.h"
 
 #define RUN_MODES (GWY_RUN_INTERACTIVE)
 
@@ -81,9 +69,9 @@ typedef struct {
 typedef struct {
     GwyParams *params;
     GwyLawn *lawn;
-    GwyDataField *result[NOUTPUTS];
-    GwyDataField *preview;
-    GwyDataField *mask;
+    GwyField *result[NOUTPUTS];
+    GwyField *preview;
+    GwyField *mask;
     /* Cached input data properties. */
     gint nsegments;
 } ModuleArgs;
@@ -91,64 +79,68 @@ typedef struct {
 typedef struct {
     ModuleArgs *args;
     GtkWidget *dialog;
+    GtkWidget *view;
     GwyParamTable *table;
     GwyParamTable *table_fit;
     GwyParamTable *table_output;
-    GwyContainer *data;
     GwyGraphModel *gmodel;
     GwySelection *selection;
     GwyResults *results;
     const gchar **result_ids;
 } ModuleGUI;
 
-static gboolean         module_register         (void);
-static GwyParamDef*     define_module_params    (void);
-static void             cmap_simplemech         (GwyContainer *data,
-                                                 GwyRunType runtype);
-static gboolean         execute                 (ModuleArgs *args,
-                                                 GtkWindow *wait_window);
-static GwyDialogOutcome run_gui                 (ModuleArgs *args,
-                                                 GwyContainer *data,
-                                                 gint id);
-static GwyResults*      create_results          (void);
-static void             dialog_response         (ModuleGUI *gui,
-                                                 gint response);
-static void             param_changed           (ModuleGUI *gui,
-                                                 gint id);
-static void             preview                 (gpointer user_data);
-static void             set_selection           (ModuleGUI *gui);
-static void             point_selection_changed (ModuleGUI *gui,
-                                                 gint id,
-                                                 GwySelection *selection);
-static gboolean         fit_one_curve           (GwyLawn *lawn,
-                                                 gint col,
-                                                 gint row,
-                                                 gint abscissa,
-                                                 gint ordinate,
-                                                 gint segment_approach,
-                                                 gint segment_retract,
-                                                 gdouble baseline_range,
-                                                 gdouble fit_upper,
-                                                 gdouble fit_lower,
-                                                 gdouble radius,
-                                                 gdouble nu,
-                                                 gdouble *result,
-                                                 gdouble *xp,
-                                                 gdouble *yp,
-                                                 gdouble *xb,
-                                                 gdouble *yb,
-                                                 gdouble *xf,
-                                                 gdouble *yf,
-                                                 gint nf);
-static void             extract_one_curve       (GwyLawn *lawn,
-                                                 GwyGraphCurveModel *gcmodel,
-                                                 gint col,
-                                                 gint row,
-                                                 gint segment,
-                                                 gint abscissa,
-                                                 gint ordinate);
-static void             update_graph_model_props(ModuleGUI *gui);
-static void             sanitise_params         (ModuleArgs *args);
+static gboolean         module_register           (void);
+static GwyParamDef*     define_module_params      (void);
+static void             module_main               (GwyFile *data,
+                                                   GwyRunModeFlags mode);
+static gboolean         execute                   (ModuleArgs *args,
+                                                   GtkWindow *wait_window);
+static GwyDialogOutcome run_gui                   (ModuleArgs *args,
+                                                   GwyFile *data,
+                                                   gint id);
+static GwyResults*      create_results            (void);
+static void             dialog_response           (ModuleGUI *gui,
+                                                   gint response);
+static void             param_changed             (ModuleGUI *gui,
+                                                   gint id);
+static void             preview                   (gpointer user_data);
+static void             set_selection             (ModuleGUI *gui);
+static void             point_selection_changed   (ModuleGUI *gui,
+                                                   gint id,
+                                                   GwySelection *selection);
+static gboolean         fit_one_curve             (GwyLawn *lawn,
+                                                   gint col,
+                                                   gint row,
+                                                   gint abscissa,
+                                                   gint ordinate,
+                                                   gint segment_approach,
+                                                   gint segment_retract,
+                                                   gdouble baseline_range,
+                                                   gdouble fit_upper,
+                                                   gdouble fit_lower,
+                                                   gdouble radius,
+                                                   gdouble nu,
+                                                   gdouble *result,
+                                                   gdouble *xp,
+                                                   gdouble *yp,
+                                                   gdouble *xb,
+                                                   gdouble *yb,
+                                                   gdouble *xf,
+                                                   gdouble *yf,
+                                                   gint nf);
+static void             extract_one_curve         (GwyLawn *lawn,
+                                                   GwyGraphCurveModel *gcmodel,
+                                                   gint col,
+                                                   gint row,
+                                                   gint segment,
+                                                   gint abscissa,
+                                                   gint ordinate);
+static void             update_graph_model_props  (ModuleGUI *gui);
+static void             add_mask_and_copy_gradient(GwyFile *data,
+                                                   GwyField *mask,
+                                                   gint id,
+                                                   gint newid);
+static void             sanitise_params           (ModuleArgs *args);
 
 /* NB: Items are directly indexed by NanomechOutput values.  Fix GUI order in gwy_enum_fill_from_struct() if needed. */
 static const NanomechOutputInfo output_info[] = {
@@ -176,11 +168,11 @@ static gboolean
 module_register(void)
 {
     gwy_curve_map_func_register("cmap_simplemech",
-                                (GwyCurveMapFunc)&cmap_simplemech,
+                                module_main,
                                 N_("/_Nanomechanical Fit..."),
                                 NULL,
                                 RUN_MODES,
-                                GWY_MENU_FLAG_CURVE_MAP,
+                                GWY_MENU_FLAG_CMAP,
                                 N_("Evaluate DMT modulus, adhesion, deformation and dissipation"));
 
     return TRUE;
@@ -220,7 +212,7 @@ define_module_params(void)
     gwy_param_def_add_double(paramdef, PARAM_NU, "nu", _("_Poisson's ratio"), 0, 1, 0.25);
     gwy_param_def_add_gwyflags(paramdef, PARAM_OUTPUT, "output", _("Output images"),
                                outputs, NOUTPUTS, (1 << OUTPUT_DMT_MODULUS));
-    gwy_param_def_add_gwyenum(paramdef, PARAM_DISPLAY, NULL, gwy_sgettext("verb|Display"),
+    gwy_param_def_add_gwyenum(paramdef, PARAM_DISPLAY, NULL, C_("verb", "Display"),
                               displays, NOUTPUTS+1, DISPLAY_ORIGINAL);
     gwy_param_def_add_int(paramdef, PARAM_XPOS, "xpos", NULL, -1, G_MAXINT, -1);
     gwy_param_def_add_int(paramdef, PARAM_YPOS, "ypos", NULL, -1, G_MAXINT, -1);
@@ -228,24 +220,21 @@ define_module_params(void)
 }
 
 static void
-cmap_simplemech(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
     GwyLawn *lawn = NULL;
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
-    GwyDataField *newmask;
     guint i, output;
     GtkWidget *dialog;
     gint oldid, newid;
-    const guchar *gradient;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    g_return_if_fail(g_type_from_name("GwyLayerPoint"));
+    g_return_if_fail(mode & RUN_MODES);
 
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_LAWN, &lawn,
-                                     GWY_APP_LAWN_ID, &oldid,
-                                     0);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_LAWN, &lawn,
+                                 GWY_APP_LAWN_ID, &oldid,
+                                 0);
     g_return_if_fail(GWY_IS_LAWN(lawn));
     args.lawn = lawn;
     args.nsegments = gwy_lawn_get_n_segments(lawn);
@@ -253,8 +242,8 @@ cmap_simplemech(GwyContainer *data, GwyRunType runtype)
     /* There are many other nonsensical inputs, but they will just produce garbage.  With unsegmented curves we cannot
      * proceed at all. */
     if (!args.nsegments) {
-        if (gwy_app_data_browser_get_gui_enabled() || gwy_app_wait_get_enabled()) {
-            dialog = gtk_message_dialog_new(gwy_app_find_window_for_curve_map(data, oldid),
+        if (gwy_data_browser_get_gui_enabled() || gwy_app_wait_get_enabled()) {
+            dialog = gtk_message_dialog_new(gwy_data_browser_get_window_for_data(data, GWY_FILE_CMAP, oldid),
                                             GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
                                             _("%s: Curves have to be segmented."), _("Nanomechanical Fit"));
             gtk_dialog_run(GTK_DIALOG(dialog));
@@ -264,46 +253,32 @@ cmap_simplemech(GwyContainer *data, GwyRunType runtype)
     }
 
     args.params = gwy_params_new_from_settings(define_module_params());
-    args.preview = gwy_container_get_object(data, gwy_app_get_lawn_preview_key_for_id(oldid));
+    args.preview = gwy_container_get_object(GWY_CONTAINER(data), gwy_file_key_cmap_picture(oldid));
     sanitise_params(&args);
 
-    for (i = 0; i < NOUTPUTS; i++) {
-        args.result[i] = gwy_data_field_new(gwy_lawn_get_xres(lawn), gwy_lawn_get_yres(lawn),
-                                            gwy_lawn_get_xreal(lawn), gwy_lawn_get_yreal(lawn),
-                                            TRUE);
-        gwy_data_field_set_xoffset(args.result[i], gwy_lawn_get_xoffset(lawn));
-        gwy_data_field_set_yoffset(args.result[i], gwy_lawn_get_yoffset(lawn));
-        gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(args.result[i]), gwy_lawn_get_si_unit_xy(lawn));
-    }
-    args.mask = gwy_data_field_new_alike(args.result[0], TRUE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(args.mask), NULL);
+    for (i = 0; i < NOUTPUTS; i++)
+        args.result[i] = gwy_lawn_new_field_alike(lawn, -1, TRUE);
+    args.mask = gwy_field_new_alike(args.result[0], TRUE);
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, oldid);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
             goto end;
     }
     if (outcome != GWY_DIALOG_HAVE_RESULT) {
-        if (!execute(&args, gwy_app_find_window_for_curve_map(data, oldid)))
+        if (!execute(&args, gwy_data_browser_get_window_for_data(data, GWY_FILE_CMAP, oldid)))
             goto end;
     }
     output = gwy_params_get_flags(args.params, PARAM_OUTPUT);
     for (i = 0; i < NOUTPUTS; i++) {
         if (!(output & (1 << i)))
             continue;
-        newid = gwy_app_data_browser_add_data_field(args.result[i], data, TRUE);
-        gwy_container_set_const_string(data, gwy_app_get_data_title_key_for_id(newid), _(output_info[i].label));
-        if (gwy_data_field_get_max(args.mask) > 0.0) {
-            newmask = gwy_data_field_duplicate(args.mask);
-            gwy_container_pass_object(data, gwy_app_get_mask_key_for_id(newid), newmask);
-        }
-        if (gwy_container_gis_string(data, gwy_app_get_lawn_palette_key_for_id(oldid), &gradient)) {
-            gwy_container_set_const_string(data,
-                                           gwy_app_get_data_palette_key_for_id(newid), gradient);
-        }
-
-        gwy_app_channel_log_add(data, -1, newid, "cmap::cmap_linestat", NULL);
+        newid = gwy_file_add_image(data, args.result[i]);
+        gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+        gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _(output_info[i].label), FALSE);
+        add_mask_and_copy_gradient(data, args.mask, oldid, newid);
+        gwy_log_add_full(data, GWY_FILE_IMAGE, -1, newid, "cmap::cmap_linestat", NULL);
     }
 
 end:
@@ -312,47 +287,51 @@ end:
     g_object_unref(args.params);
 }
 
-static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+static void
+add_mask_and_copy_gradient(GwyFile *data, GwyField *mask, gint id, gint newid)
 {
-    GtkWidget *hbox, *graph, *dataview, *align;
+    if (gwy_field_get_max(mask) > 0.0)
+        gwy_file_pass_image_mask(data, newid, gwy_field_copy(mask));
+
+    gwy_file_sync_items(data, GWY_FILE_CMAP, id,
+                        data, GWY_FILE_IMAGE, newid,
+                        GWY_FILE_ITEM_PALETTE, FALSE);
+}
+
+static GwyDialogOutcome
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
+{
+    GtkWidget *hbox, *graph, *dataview;
     GwyParamTable *table;
     GwyDialog *dialog;
     ModuleGUI gui;
-    GwyDataField *field;
+    GwyField *field;
     GwyDialogOutcome outcome;
     GwyGraphCurveModel *gcmodel;
-    const guchar *gradient;
     guint i;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
-    gui.data = gwy_container_new();
     gui.gmodel = gwy_graph_model_new();
     gui.results = create_results();
     gui.result_ids = g_new(const gchar*, NOUTPUTS);
     for (i = 0; i < NOUTPUTS; i++)
         gui.result_ids[i] = output_info[i].name;
-    field = gwy_container_get_object(data, gwy_app_get_lawn_preview_key_for_id(id));
-    gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(0), field);
-    if (gwy_container_gis_string(data, gwy_app_get_lawn_palette_key_for_id(id), &gradient))
-        gwy_container_set_const_string(gui.data, gwy_app_get_data_palette_key_for_id(0), gradient);
 
     gui.dialog = gwy_dialog_new(_("Nanomechanical Fit"));
     dialog = GWY_DIALOG(gui.dialog);
-    gtk_dialog_add_button(GTK_DIALOG(dialog), gwy_sgettext("verb|_Fit"), RESPONSE_FIT);
+    gtk_dialog_add_button(GTK_DIALOG(dialog), C_("verb", "_Fit"), RESPONSE_FIT);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    hbox = gwy_hbox_new(0);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
     gwy_dialog_add_content(GWY_DIALOG(gui.dialog), hbox, TRUE, TRUE, 0);
-
-    align = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-    gtk_box_pack_start(GTK_BOX(hbox), align, FALSE, FALSE, 0);
-
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
-    gtk_container_add(GTK_CONTAINER(align), dataview);
-    gui.selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(dataview), 0, "Point", 1, TRUE);
+    field = gwy_container_get_object(GWY_CONTAINER(data), gwy_file_key_cmap_picture(id));
+    dataview = gui.view = gwy_create_preview(field, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_CMAP, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_REAL_SQUARE);
+    gui.selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(dataview), GWY_TYPE_LAYER_POINT, 1, TRUE);
     set_selection(&gui);
+    hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), FALSE);
 
     gcmodel = gwy_graph_curve_model_new();
     g_object_set(gcmodel,
@@ -401,14 +380,12 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_graph_model_add_curve(gui.gmodel, gcmodel);
     g_object_unref(gcmodel);
 
-
-
     graph = gwy_graph_new(gui.gmodel);
     gwy_graph_enable_user_input(GWY_GRAPH(graph), FALSE);
     gtk_widget_set_size_request(graph, PREVIEW_SIZE, PREVIEW_SIZE);
     gtk_box_pack_start(GTK_BOX(hbox), graph, TRUE, TRUE, 0);
 
-    hbox = gwy_hbox_new(20);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 20);
     gwy_dialog_add_content(GWY_DIALOG(gui.dialog), hbox, TRUE, TRUE, 4);
 
     table = gui.table = gwy_param_table_new(args->params);
@@ -455,7 +432,6 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     outcome = gwy_dialog_run(dialog);
 
     g_object_unref(gui.gmodel);
-    g_object_unref(gui.data);
     g_object_unref(gui.results);
     g_free(gui.result_ids);
 
@@ -490,7 +466,7 @@ dialog_response(ModuleGUI *gui, gint response)
     if (response == RESPONSE_FIT) {
         if (execute(gui->args, GTK_WINDOW(gui->dialog)))
             gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
-        gwy_data_field_data_changed(gwy_container_get_object(gui->data, gwy_app_get_data_key_for_id(0)));
+        gwy_field_data_changed(gwy_data_view_get_field(GWY_DATA_VIEW(gui->view)));
     }
 }
 
@@ -503,9 +479,9 @@ param_changed(ModuleGUI *gui, gint id)
     if (id < 0 || id == PARAM_DISPLAY) {
         gint i = gwy_params_get_enum(params, PARAM_DISPLAY);
         if (i < NOUTPUTS)
-            gwy_container_set_object(gui->data, gwy_app_get_data_key_for_id(0), args->result[i]);
+            gwy_data_view_set_field(GWY_DATA_VIEW(gui->view), args->result[i]);
         else
-            gwy_container_set_object(gui->data, gwy_app_get_data_key_for_id(0), args->preview);
+            gwy_data_view_set_field(GWY_DATA_VIEW(gui->view), args->preview);
     }
     if (id < 0 || id == PARAM_OUTPUT) {
         guint output = gwy_params_get_flags(params, PARAM_OUTPUT);
@@ -565,7 +541,6 @@ preview(gpointer user_data)
     gdouble radius = gwy_params_get_double(params, PARAM_RADIUS);
     gdouble nu = gwy_params_get_double(params, PARAM_NU);
 
-
     GwyGraphCurveModel *gcmodel_approach = gwy_graph_model_get_curve(gui->gmodel, 0);
     GwyGraphCurveModel *gcmodel_retract = gwy_graph_model_get_curve(gui->gmodel, 1);
     GwyGraphCurveModel *gcmodel_points = gwy_graph_model_get_curve(gui->gmodel, 2);
@@ -581,8 +556,8 @@ preview(gpointer user_data)
     extract_one_curve(args->lawn, gcmodel_retract, col, row, segment_retract, abscissa, ordinate);
     update_graph_model_props(gui);
 
-    gwy_results_set_unit(results, "x", gwy_lawn_get_si_unit_curve(args->lawn, abscissa));
-    gwy_results_set_unit(results, "y", gwy_lawn_get_si_unit_curve(args->lawn, ordinate));
+    gwy_results_set_unit(results, "x", gwy_lawn_get_unit_curve(args->lawn, abscissa));
+    gwy_results_set_unit(results, "y", gwy_lawn_get_unit_curve(args->lawn, ordinate));
 
     xp = g_new(gdouble, 3);
     yp = g_new(gdouble, 3);
@@ -617,7 +592,6 @@ preview(gpointer user_data)
 //param[0]: F_ad, param[1]: E (modulus), param[2]: R, param[3]: xshift, param[4] nu
 static gdouble
 func_dmt(gdouble x,
-         G_GNUC_UNUSED gint n_param,
          const gdouble *param,
          G_GNUC_UNUSED gpointer user_data,
          gboolean *fres)
@@ -626,12 +600,11 @@ func_dmt(gdouble x,
     double xr = param[0] - x;
     *fres = TRUE;
     if (xr > 0)
-        return 4.0*param[3]/3.0/(1-param[4]*param[4])*sqrt(param[2]*xr*xr*xr) + param[1];
+        return 4.0*param[3]/3.0/(1 - param[4]*param[4])*sqrt(param[2]*xr*xr*xr) + param[1];
     else
         return param[1];
 }
 
-
 //xp,yp: important points to show to user, can be NULL to avoid filling it. Fixed size: 3 points
 //xb,yb: baseline fit to show to user, can be NULL to avoid filling it. Fixed size: 2 points
 //xf,yf: DMT fit to show to user, can be NULL to avoid filling it. Size can be set and must be allocated by user.
@@ -695,8 +668,10 @@ evaluate_curve(const gdouble *xdata, const gdouble *ydata,
             nbaseline++;
         }
     }
-    if (nbaseline>0) baseline /= nbaseline;
-    else baseline = yadata[nadata-1];
+    if (nbaseline > 0)
+        baseline /= nbaseline;
+    else
+        baseline = yadata[nadata-1];
 
     //find zero force - point where we reach baseline force on approach curve when going from peak value
     xzero = xadata[nadata-1]; //safe defaults
@@ -738,18 +713,18 @@ evaluate_curve(const gdouble *xdata, const gdouble *ydata,
     }
 
     //find dmt fit limits as points where we reach the points on retract curve when going from peak value
-    xupper = rfrom;//xpeak;     //safe inits
+    xupper = xpeak;     //safe inits
     xlower = xadhesion;
     iupper = 0;
     ilower = iadhesion;
     upperval = adhesion + fit_upper*(peak-adhesion);
     lowerval = adhesion + fit_lower*(peak-adhesion);
 
-    for (i = (nrdata - 1); i > 1; i--) {
-//        if (yrdata[i] >= upperval && yrdata[i + 1] < upperval) {
-//            xupper = xrdata[i];
-//            iupper = i;
-//        }
+    for (i = nrdata - 1; i > 1; i--) {
+        if (yrdata[i] >= upperval && yrdata[i + 1] < upperval) {
+            xupper = xrdata[i];
+            iupper = i;
+        }
         if (yrdata[i] >= lowerval && yrdata[i+1] < lowerval) {
             xlower = xrdata[i];
             ilower = i;
@@ -758,9 +733,9 @@ evaluate_curve(const gdouble *xdata, const gdouble *ydata,
 
     fit_done = FALSE;
     modulus = 5e7;
-    if ((ilower - iupper) > 4) {
+    if (ilower - iupper > 4) {
        //fit dmt
-       fitter = gwy_math_nlfit_new((GwyNLFitFunc)func_dmt, gwy_math_nlfit_diff);
+       fitter = gwy_math_nlfit_new(func_dmt, NULL);
 
        param[0] = xadhesion; //adhesion comes from direct measurement
        fix[0] = 0;
@@ -777,19 +752,18 @@ evaluate_curve(const gdouble *xdata, const gdouble *ydata,
        param[4] = nu;
        fix[4] = 1;
 
-
        if (gwy_math_nlfit_fit_full(fitter, ilower-iupper, xrdata + iupper, yrdata + iupper, NULL,
                                    5, param, fix, NULL, NULL) >= 0) {
           modulus = param[3];
           fit_done = TRUE;
 
-      //    printf("param: %g %g %g %g\n", param[0], param[1], param[2], param[3], param[4]);
+          // printf("param: %g %g %g %g\n", param[0], param[1], param[2], param[3], param[4]);
 
           if (xf && yf && nf) {
               gwy_math_linspace(xf, nf, xupper, (xlower - xupper)/nf);
               for (i = 0; i < nf; i++) {
-                  yf[i] = func_dmt(xf[i], 5, param, NULL, &fres);
-//                  printf("%g %g\n", xf[i], yf[i]);
+                  yf[i] = func_dmt(xf[i], param, NULL, &fres);
+                  //  printf("%g %g\n", xf[i], yf[i]);
               }
           }
        } else
@@ -890,7 +864,7 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
     gdouble radius = gwy_params_get_double(params, PARAM_RADIUS);
     gdouble nu = gwy_params_get_double(params, PARAM_NU);
 
-    GwySIUnit *unit, *xunit, *yunit;
+    GwyUnit *unit, *xunit, *yunit;
     gboolean cancelled = FALSE;
     gdouble *rdata[NOUTPUTS], *mdata;
     guint i;
@@ -898,21 +872,21 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
     /* Find segment cut points. */
     gwy_app_wait_start(wait_window, _("Fitting in progress..."));
 
-    xunit = gwy_lawn_get_si_unit_curve(lawn, abscissa);
-    yunit = gwy_lawn_get_si_unit_curve(lawn, ordinate);
+    xunit = gwy_lawn_get_unit_curve(lawn, abscissa);
+    yunit = gwy_lawn_get_unit_curve(lawn, ordinate);
     for (i = 0; i < NOUTPUTS; i++) {
-        unit = gwy_data_field_get_si_unit_z(args->result[i]);
+        unit = gwy_field_get_unit_z(args->result[i]);
         if (output_info[i].power_u)
-            gwy_si_unit_set_from_string(unit, "Pa");
+            gwy_unit_set_from_string(unit, "Pa");
         else if (output_info[i].power_v)
-            gwy_si_unit_set_from_string(unit, "eV");
+            gwy_unit_set_from_string(unit, "eV");
         else
-            gwy_si_unit_power_multiply(xunit, output_info[i].power_x, yunit, output_info[i].power_y, unit);
-        gwy_data_field_clear(args->result[i]);
-        rdata[i] = gwy_data_field_get_data(args->result[i]);
+            gwy_unit_power_multiply(xunit, output_info[i].power_x, yunit, output_info[i].power_y, unit);
+        gwy_field_clear(args->result[i]);
+        rdata[i] = gwy_field_get_data(args->result[i]);
     }
-    gwy_data_field_clear(args->mask);
-    mdata = gwy_data_field_get_data(args->mask);
+    gwy_field_clear(args->mask);
+    mdata = gwy_field_get_data(args->mask);
 
 /*#ifdef _OPENMP
 #pragma omp parallel if(gwy_threads_are_enabled()) default(none) \
@@ -946,13 +920,13 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
 
     if (cancelled) {
         for (i = 0; i < NOUTPUTS; i++)
-            gwy_data_field_clear(args->result[i]);
+            gwy_field_clear(args->result[i]);
         return FALSE;
     }
 
     for (i = 0; i < NOUTPUTS; i++) {
-        if (gwy_data_field_get_max(args->mask) > 0.0)
-            gwy_data_field_laplace_solve(args->result[i], args->mask, -1, 1.0);
+        if (gwy_field_get_max(args->mask) > 0.0)
+            gwy_field_laplace_solve(args->result[i], args->mask, -1, 1.0);
     }
     return TRUE;
 }
@@ -988,16 +962,16 @@ update_graph_model_props(ModuleGUI *gui)
     gint abscissa = gwy_params_get_int(params, PARAM_ABSCISSA);
     gint ordinate = gwy_params_get_int(params, PARAM_ORDINATE);
     GwyGraphModel *gmodel = gui->gmodel;
-    GwySIUnit *xunit, *yunit;
+    GwyUnit *xunit, *yunit;
     const gchar *xlabel, *ylabel;
 
-    xunit = gwy_lawn_get_si_unit_curve(lawn, abscissa);
+    xunit = gwy_lawn_get_unit_curve(lawn, abscissa);
     xlabel = gwy_lawn_get_curve_label(lawn, abscissa);
-    yunit = gwy_lawn_get_si_unit_curve(lawn, ordinate);
+    yunit = gwy_lawn_get_unit_curve(lawn, ordinate);
     ylabel = gwy_lawn_get_curve_label(lawn, ordinate);
     g_object_set(gmodel,
-                 "si-unit-x", xunit,
-                 "si-unit-y", yunit,
+                 "unit-x", xunit,
+                 "unit-y", yunit,
                  "axis-label-bottom", xlabel ? xlabel : _("Untitled"),
                  "axis-label-left", ylabel ? ylabel : _("Untitled"),
                  NULL);
