@@ -1,5 +1,5 @@
 /*
- *  $Id: cmap_sinebg.c 28888 2025-11-20 11:37:15Z klapetek $
+ *  $Id: cmap_sinebg.c 28905 2025-11-24 15:51:18Z yeti-dn $
  *  Copyright (C) 2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,23 +16,11 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwythreads.h>
-#include <libprocess/lawn.h>
-#include <libprocess/stats.h>
-#include <libprocess/correct.h>
-#include <libprocess/gwyprocess.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libgwydgets/gwydataview.h>
-#include <libgwydgets/gwygraph.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-cmap.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
-#include "libgwyddion/gwyomp.h"
+#include <gwy.h>
+#include "libgwyddion/omp.h"
 
 #define RUN_MODES (GWY_RUN_INTERACTIVE)
 
@@ -55,7 +43,7 @@ typedef struct {
     GwyParams *params;
     GwyLawn *lawn;
     GwyLawn *result;
-    GwyDataField *field;
+    GwyField *field;
     gint nsegments;
 } ModuleArgs;
 
@@ -64,57 +52,53 @@ typedef struct {
     GtkWidget *dialog;
     GwyParamTable *table;
     GwyParamTable *table_output;
-    GwyContainer *data;
     GwySelection *selection;
     GwySelection *graph_selection;
     GwyGraphModel *gmodel;
 } ModuleGUI;
 
-static gboolean         module_register         (void);
-static GwyParamDef*     define_module_params    (void);
-static void             sinebg                  (GwyContainer *data,
-                                                 GwyRunType runtype);
-static void             execute                 (ModuleArgs *args,
-                                                 GtkWindow *window);
-static GwyDialogOutcome run_gui                 (ModuleArgs *args,
-                                                 GwyContainer *data,
-                                                 gint id);
-static void             param_changed           (ModuleGUI *gui,
-                                                 gint id);
-static void             preview                 (gpointer user_data);
-static void             set_selection           (ModuleGUI *gui);
-static void             point_selection_changed (ModuleGUI *gui,
-                                                 gint id,
-                                                 GwySelection *selection);
-static void             extract_one_curve       (GwyLawn *lawn,
-                                                 GwyGraphCurveModel *gcmodel,
-                                                 gint col,
-                                                 gint row,
-                                                 GwyParams *params);
-static void             convert_one_curve       (GwyGraphCurveModel *gcmodel,
-                                                 GwyParams *params,
-                                                 gdouble *fitparams,
-                                                 gint nsegments,
-                                                 const gint *segments);
-static void             do_sinebg               (const gdouble *xdata,
-                                                 const gdouble *ydata,
-                                                 gdouble *nydata,
-                                                 gint ndata,
-                                                 const gint *segments,
-                                                 gint segment,
-                                                 gboolean segment_enabled,
-                                                 gdouble from,
-                                                 gdouble to,
-                                                 gboolean subtract,
-                                                 gdouble *retparam);
-static gdouble          func_sine               (gdouble x,
-                                                 gint n_param,
-                                                 const gdouble *param,
-                                                 G_GNUC_UNUSED gpointer user_data,
-                                                 gboolean *fres);
-static void             graph_selected          (GwySelection* selection,
-                                                 gint i,
-                                                 ModuleGUI *gui);
+static gboolean         module_register        (void);
+static GwyParamDef*     define_module_params   (void);
+static void             module_main            (GwyFile *data,
+                                                GwyRunModeFlags mode);
+static void             execute                (ModuleArgs *args,
+                                                GtkWindow *window);
+static GwyDialogOutcome run_gui                (ModuleArgs *args,
+                                                GwyFile *data,
+                                                gint id);
+static void             param_changed          (ModuleGUI *gui,
+                                                gint id);
+static void             preview                (gpointer user_data);
+static void             set_selection          (ModuleGUI *gui);
+static void             point_selection_changed(ModuleGUI *gui,
+                                                gint id,
+                                                GwySelection *selection);
+static void             extract_one_curve      (GwyLawn *lawn,
+                                                GwyGraphCurveModel *gcmodel,
+                                                gint col,
+                                                gint row,
+                                                GwyParams *params);
+static void             convert_one_curve      (GwyGraphCurveModel *gcmodel,
+                                                GwyParams *params,
+                                                gdouble *fitparams,
+                                                gint nsegments,
+                                                const gint *segments);
+static void             do_sinebg              (const gdouble *xdata,
+                                                const gdouble *ydata,
+                                                gdouble *nydata,
+                                                gint ndata,
+                                                const gint *segments,
+                                                gint segment,
+                                                gboolean segment_enabled,
+                                                gdouble from,
+                                                gdouble to,
+                                                gboolean subtract,
+                                                gdouble *retparam);
+static gdouble          func_sine              (gdouble x,
+                                                const gdouble *param,
+                                                G_GNUC_UNUSED gpointer user_data,
+                                                gboolean *fres);
+
 static void             update_graph_model_props(GwyGraphModel *gmodel,
                                                  ModuleArgs *args);
 static void             sanitise_params         (ModuleArgs *args);
@@ -135,11 +119,11 @@ static gboolean
 module_register(void)
 {
     gwy_curve_map_func_register("cmap_sinebg",
-                                (GwyCurveMapFunc)&sinebg,
+                                module_main,
                                 N_("/Remove _Sine Background..."),
                                 NULL,
                                 RUN_MODES,
-                                GWY_MENU_FLAG_CURVE_MAP,
+                                GWY_MENU_FLAG_CMAP,
                                 N_("Remove sine background from all curves"));
 
     return TRUE;
@@ -168,20 +152,19 @@ define_module_params(void)
 }
 
 static void
-sinebg(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
     GwyLawn *lawn = NULL;
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     gint id;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    g_return_if_fail(g_type_from_name("GwyLayerPoint"));
+    g_return_if_fail(mode & RUN_MODES);
 
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_LAWN, &lawn,
-                                     GWY_APP_LAWN_ID, &id,
-                                     0);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_LAWN, &lawn,
+                                 GWY_APP_LAWN_ID, &id,
+                                 0);
     g_return_if_fail(GWY_IS_LAWN(lawn));
 
     args.lawn = lawn;
@@ -189,64 +172,51 @@ sinebg(GwyContainer *data, GwyRunType runtype)
     args.params = gwy_params_new_from_settings(define_module_params());
     sanitise_params(&args);
 
-    args.field = gwy_data_field_new(gwy_lawn_get_xres(lawn), gwy_lawn_get_yres(lawn),
-                                    gwy_lawn_get_xreal(lawn), gwy_lawn_get_yreal(lawn), TRUE);
-    gwy_data_field_set_xoffset(args.field, gwy_lawn_get_xoffset(lawn));
-    gwy_data_field_set_yoffset(args.field, gwy_lawn_get_yoffset(lawn));
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(args.field), gwy_lawn_get_si_unit_xy(lawn));
+    args.field = gwy_lawn_new_field_alike(lawn, -1, TRUE);
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, id);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
             goto end;
     }
     if (outcome != GWY_DIALOG_HAVE_RESULT) {
-        execute(&args, gwy_app_find_window_for_curve_map(data, id));
+        execute(&args, gwy_data_browser_get_window_for_data(data, GWY_FILE_CMAP, id));
     }
 
-    gwy_app_curve_map_log_add(data, -1, id, "cmap::cmap_sinebg", NULL);
+    /* TODO: Add log, maybe more. */
 
 end:
-    GWY_OBJECT_UNREF(args.result);
+    g_clear_object(&args.result);
     g_object_unref(args.field);
     g_object_unref(args.params);
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
-    GtkWidget *hbox, *graph, *dataview, *align, *area;
+    GtkWidget *hbox, *graph, *dataview, *area;
     GwyParamTable *table;
     GwyDialog *dialog;
     ModuleGUI gui;
-    GwyDataField *field;
+    GwyField *field;
     GwyDialogOutcome outcome;
     GwyGraphCurveModel *gcmodel;
-    const guchar *gradient;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
-    gui.data = gwy_container_new();
-    field = gwy_container_get_object(data, gwy_app_get_lawn_preview_key_for_id(id));
-    gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(0), field);
-    if (gwy_container_gis_string(data, gwy_app_get_lawn_palette_key_for_id(id), &gradient))
-        gwy_container_set_const_string(gui.data, gwy_app_get_data_palette_key_for_id(0), gradient);
 
     gui.dialog = gwy_dialog_new(_("Remove Sine Background"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    hbox = gwy_hbox_new(0);
-    gwy_dialog_add_content(GWY_DIALOG(gui.dialog), hbox, TRUE, TRUE, 0);
-
-    align = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-    gtk_box_pack_start(GTK_BOX(hbox), align, FALSE, FALSE, 0);
-
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
-    gtk_container_add(GTK_CONTAINER(align), dataview);
-    gui.selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(dataview), 0, "Point", 1, TRUE);
+    field = gwy_container_get_object(GWY_CONTAINER(data), gwy_file_key_cmap_picture(id));
+    dataview = gwy_create_preview(field, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_CMAP, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_REAL_SQUARE);
+    gui.selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(dataview), GWY_TYPE_LAYER_POINT, 1, TRUE);
     set_selection(&gui);
+    hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), FALSE);
 
     gui.gmodel = gwy_graph_model_new();
     gcmodel = gwy_graph_curve_model_new();
@@ -258,7 +228,6 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_graph_model_add_curve(gui.gmodel, gcmodel);
     g_object_unref(gcmodel);
 
-
     gcmodel = gwy_graph_curve_model_new();
     g_object_set(gcmodel,
                  "mode", GWY_GRAPH_CURVE_LINE,
@@ -268,19 +237,16 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_graph_model_add_curve(gui.gmodel, gcmodel);
     g_object_unref(gcmodel);
 
-
-
     graph = gwy_graph_new(gui.gmodel);
     area = gwy_graph_get_area(GWY_GRAPH(graph));
     gwy_graph_enable_user_input(GWY_GRAPH(graph), FALSE);
     gwy_graph_area_set_status(GWY_GRAPH_AREA(area), GWY_GRAPH_STATUS_XSEL);
-    gwy_graph_area_set_selection_editable(GWY_GRAPH_AREA(area), TRUE);
+    gwy_graph_area_set_selection_editable(GWY_GRAPH_AREA(area), FALSE);
     gui.graph_selection = gwy_graph_area_get_selection(GWY_GRAPH_AREA(area), GWY_GRAPH_STATUS_XSEL);
     gtk_widget_set_size_request(graph, PREVIEW_SIZE, PREVIEW_SIZE);
     gtk_box_pack_start(GTK_BOX(hbox), graph, TRUE, TRUE, 0);
 
-
-    hbox = gwy_hbox_new(20);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 20);
     gwy_dialog_add_content(GWY_DIALOG(gui.dialog), hbox, TRUE, TRUE, 4);
 
     table = gui.table = gwy_param_table_new(args->params);
@@ -302,13 +268,11 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 
     g_signal_connect_swapped(gui.table, "param-changed", G_CALLBACK(param_changed), &gui);
     g_signal_connect_swapped(gui.selection, "changed", G_CALLBACK(point_selection_changed), &gui);
-    g_signal_connect(gui.graph_selection, "changed", G_CALLBACK(graph_selected), &gui);
     gwy_dialog_set_preview_func(dialog, GWY_PREVIEW_IMMEDIATE, preview, &gui, NULL);
 
     outcome = gwy_dialog_run(dialog);
 
     g_object_unref(gui.gmodel);
-    g_object_unref(gui.data);
 
     return outcome;
 }
@@ -350,38 +314,6 @@ point_selection_changed(ModuleGUI *gui, gint id, GwySelection *selection)
     gwy_param_table_param_changed(gui->table, PARAM_YPOS);
 }
 
-static void
-graph_selected(GwySelection* selection, gint i, ModuleGUI *gui)
-{   
-    gdouble range[2];
-    gdouble xfrom, xto, cxfrom, cxto;
-    gboolean have_range = TRUE;
-    GwyGraphCurveModel *gc;
-
-    g_return_if_fail(i <= 0);
-    
-    if (gwy_selection_get_data(selection, NULL) <= 0)
-        have_range = FALSE;
-    else {
-        gwy_selection_get_object(selection, 0, range);
-        if (range[0] == range[1])
-            have_range = FALSE;
-    }
-    if (have_range) {
-        gc = gwy_graph_model_get_curve(gui->gmodel, 0);
-        gwy_graph_curve_model_get_x_range(gc, &cxfrom, &cxto);
-
-        xfrom = MIN(range[0], range[1]); 
-        xto = MAX(range[0], range[1]);
-
-        xfrom = MAX(xfrom, cxfrom);
-        xto = MIN(xto, cxto);
-    
-        gwy_param_table_set_double(gui->table, PARAM_RANGE_FROM, (xfrom - cxfrom)/(cxto - cxfrom));
-        gwy_param_table_set_double(gui->table, PARAM_RANGE_TO, (xto - cxfrom)/(cxto - cxfrom));
-    }
-}
-
 static void
 preview(gpointer user_data)
 {
@@ -416,7 +348,7 @@ preview(gpointer user_data)
     xfit = gwy_math_linspace(NULL, nfit, xfrom, (xto - xfrom)/nfit);
     yfit = g_new(gdouble, nfit);
     for (i = 0; i < nfit; i++)
-        yfit[i] = func_sine(xfit[i], 4, fitparams, NULL, &fres);
+        yfit[i] = func_sine(xfit[i], fitparams, NULL, &fres);
 
     gc = gwy_graph_model_get_curve(gui->gmodel, 1);
     gwy_graph_curve_model_set_data(gc, xfit, yfit, nfit);
@@ -493,7 +425,6 @@ extract_one_curve(GwyLawn *lawn, GwyGraphCurveModel *gcmodel,
 
 static gdouble
 func_sine(gdouble x,
-          G_GNUC_UNUSED gint n_param,
           const gdouble *param,
           G_GNUC_UNUSED gpointer user_data,
           gboolean *fres)
@@ -570,7 +501,7 @@ do_sinebg(const gdouble *xdata, const gdouble *ydata,
     // XXX: Please allocate the fitter outside the function.
     // XXX: Also, the original interface is weird, with functions getting never-used arguments like n_param.
     // Usually gwy_math_nlfit_new_idx() is better.
-    fitter = gwy_math_nlfit_new((GwyNLFitFunc)func_sine, gwy_math_nlfit_diff);
+    fitter = gwy_math_nlfit_new(func_sine, NULL);
 
     param[0] = (ymax-ymin)/2.0; //amplitude
     param[1] = 2*M_PI/(xmax-xmin); //frequency
@@ -581,7 +512,7 @@ do_sinebg(const gdouble *xdata, const gdouble *ydata,
 
     if (subtract && nydata) {
         for (i = 0; i < ndata; i++) {
-            nydata[i] = ydata[i] - func_sine(xdata[j], 4, param, NULL, &fres);
+            nydata[i] = ydata[i] - func_sine(xdata[j], param, NULL, &fres);
         }
     }
 
@@ -616,17 +547,17 @@ update_graph_model_props(GwyGraphModel *gmodel, ModuleArgs *args)
     GwyParams *params = args->params;
     gint abscissa = gwy_params_get_int(params, PARAM_ABSCISSA);
     gint ordinate = gwy_params_get_int(params, PARAM_ORDINATE);
-    GwySIUnit *xunit, *yunit;
+    GwyUnit *xunit, *yunit;
     const gchar *xlabel, *ylabel;
 
-    xunit = gwy_lawn_get_si_unit_curve(lawn, abscissa);
+    xunit = gwy_lawn_get_unit_curve(lawn, abscissa);
     xlabel = gwy_lawn_get_curve_label(lawn, abscissa);
-    yunit = gwy_lawn_get_si_unit_curve(lawn, ordinate);
+    yunit = gwy_lawn_get_unit_curve(lawn, ordinate);
     ylabel = gwy_lawn_get_curve_label(lawn, ordinate);
 
     g_object_set(gmodel,
-                 "si-unit-x", xunit,
-                 "si-unit-y", yunit,
+                 "unit-x", xunit,
+                 "unit-y", yunit,
                  "axis-label-bottom", xlabel ? xlabel : _("Untitled"),
                  "axis-label-left", ylabel ? ylabel : _("Untitled"),
                  NULL);
