@@ -1,5 +1,5 @@
 /*
- *  $Id: cmap_volumize.c 29036 2025-12-22 19:19:15Z yeti-dn $
+ *  $Id: cmap_fit.c 25639 2023-09-07 12:23:42Z yeti-dn $
  *  Copyright (C) 2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -17,43 +17,17 @@
 
 #include "config.h"
 #include <string.h>
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwythreads.h>
-#include <libgwyddion/gwyfdcurvepreset.h>
-#include <libprocess/lawn.h>
-#include <libprocess/stats.h>
-#include <libprocess/correct.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libgwydgets/gwydataview.h>
-#include <libgwydgets/gwygraph.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-cmap.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
-#include "libgwyddion/gwyomp.h"
+#include <gwy.h>
+#include "libgwyddion/omp.h"
 
 #define RUN_MODES (GWY_RUN_INTERACTIVE)
 
-typedef enum {
-    BB_OK = 0,
-    BB_ERROR_NO_DATA,
-    BB_ERROR_SIZE,
-} BrickabilityError;
-
-typedef enum {
-    CUT_DIR_BEGINNING = 0,
-    CUT_DIR_END = 1
-} CutDir;
-
 enum {
     PARAM_ORDINATE,
     PARAM_SEGMENT,
     PARAM_ENABLE_SEGMENT,
-    PARAM_CUT,
-    PARAM_CUT_N,
-    PARAM_CUT_DIR,
     INFO_RESULT,
 };
 
@@ -61,7 +35,6 @@ typedef struct {
     GwyParams *params;
     GwyLawn *lawn;
     gint nsegments;
-    GwyLawn *result;
 } ModuleArgs;
 
 typedef struct {
@@ -70,14 +43,14 @@ typedef struct {
     GwyParamTable *table;
 } ModuleGUI;
 
-static gboolean         module_register         (void);
-static GwyParamDef*     define_module_params    (void);
-static void             volumize                (GwyContainer *data,
-                                                 GwyRunType runtype);
-static GwyBrick*        execute                 (ModuleArgs *args);
-static GwyDialogOutcome run_gui                 (ModuleArgs *args);
-static void             param_changed           (ModuleGUI *gui,
-                                                 gint id);
+static gboolean         module_register     (void);
+static GwyParamDef*     define_module_params(void);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
+static GwyBrick*        execute             (ModuleArgs *args);
+static GwyDialogOutcome run_gui             (ModuleArgs *args);
+static void             param_changed       (ModuleGUI *gui,
+                                             gint id);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
@@ -95,11 +68,11 @@ static gboolean
 module_register(void)
 {
     gwy_curve_map_func_register("cmap_volumize",
-                                (GwyCurveMapFunc)&volumize,
+                                module_main,
                                 N_("/_Volumize Curves..."),
                                 NULL,
                                 RUN_MODES,
-                                GWY_MENU_FLAG_CURVE_MAP,
+                                GWY_MENU_FLAG_CMAP,
                                 N_("Volumize curves"));
 
     return TRUE;
@@ -109,10 +82,6 @@ static GwyParamDef*
 define_module_params(void)
 {
     static GwyParamDef *paramdef = NULL;
-    static const GwyEnum directions[] = {
-       { N_("Beginning"), CUT_DIR_BEGINNING,  },
-       { N_("End"),       CUT_DIR_END,        },
-    };
 
     if (paramdef)
         return paramdef;
@@ -120,19 +89,15 @@ define_module_params(void)
     paramdef = gwy_param_def_new();
     gwy_param_def_set_function_name(paramdef, gwy_curve_map_func_current());
 
-    gwy_param_def_add_lawn_curve(paramdef, PARAM_ORDINATE, "ordinate", _("_Ordinate"));
+    gwy_param_def_add_lawn_curve(paramdef, PARAM_ORDINATE, "ordinate", _("Ordinate"));
     gwy_param_def_add_lawn_segment(paramdef, PARAM_SEGMENT, "segment", NULL);
     gwy_param_def_add_boolean(paramdef, PARAM_ENABLE_SEGMENT, "enable_segment", NULL, FALSE);
-    gwy_param_def_add_boolean(paramdef, PARAM_CUT, "do_cut", _("_Cut curves"), TRUE);
-    gwy_param_def_add_int(paramdef, PARAM_CUT_N, "cut_n", _("_Size"), 1, 100000, 1);
-    gwy_param_def_add_gwyenum(paramdef, PARAM_CUT_DIR, "cut_dir", _("_Cut data from"),
-                              directions, G_N_ELEMENTS(directions), CUT_DIR_BEGINNING);
 
     return paramdef;
 }
 
 static void
-volumize(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
     GwyLawn *lawn = NULL;
@@ -140,24 +105,21 @@ volumize(GwyContainer *data, GwyRunType runtype)
     gint id, newid, ordinate, segment;
     gboolean segment_enabled;
     GwyBrick *brick;
-    gchar title[100];
     const gchar *segmentlabel;
-    const guchar *gradient;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    g_return_if_fail(g_type_from_name("GwyLayerPoint"));
+    g_return_if_fail(mode & RUN_MODES);
 
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_LAWN, &lawn,
-                                     GWY_APP_LAWN_ID, &id,
-                                     0);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_LAWN, &lawn,
+                                 GWY_APP_LAWN_ID, &id,
+                                 0);
     g_return_if_fail(GWY_IS_LAWN(lawn));
 
     args.lawn = lawn;
     args.nsegments = gwy_lawn_get_n_segments(lawn);
     args.params = gwy_params_new_from_settings(define_module_params());
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -165,63 +127,62 @@ volumize(GwyContainer *data, GwyRunType runtype)
     }
     if (outcome != GWY_DIALOG_HAVE_RESULT) {
         brick = execute(&args);
-
         if (brick) {
-
-            newid = gwy_app_data_browser_add_brick(brick, NULL, data, TRUE);
+            newid = gwy_file_add_volume(data, brick, NULL);
+            gwy_file_set_visible(data, GWY_FILE_VOLUME, newid, TRUE);
 
             segment_enabled = args.nsegments ? gwy_params_get_boolean(args.params, PARAM_ENABLE_SEGMENT) : FALSE;
             segment = segment_enabled ? gwy_params_get_int(args.params, PARAM_SEGMENT) : -1;
             ordinate = gwy_params_get_int(args.params, PARAM_ORDINATE);
 
+            //printf("se %d  s %d o %d\n", segment_enabled, segment, ordinate);
+
+            gchar *title;
             if (segment_enabled) {
                 segmentlabel = gwy_lawn_get_segment_label(lawn, segment);
                 if (segmentlabel)
-                    snprintf(title, sizeof(title), "%s: %s", gwy_lawn_get_curve_label(lawn, ordinate), segmentlabel);
-                else
-                    snprintf(title, sizeof(title), _("%s: (Segment %d)"), gwy_lawn_get_curve_label(lawn, ordinate),
-                             segment + 1);
+                    title = g_strdup_printf("%s: %s", gwy_lawn_get_curve_label(lawn, ordinate), segmentlabel);
+                else {
+                    title = g_strdup_printf(_("%s: (Segment %d)"),
+                                            gwy_lawn_get_curve_label(lawn, ordinate), segment + 1);
+                }
             }
             else
-                snprintf(title, sizeof(title), "%s", gwy_lawn_get_curve_label(lawn, ordinate));
+                title = g_strdup(gwy_lawn_get_curve_label(lawn, ordinate));
 
-            gwy_app_set_brick_title(data, newid, title);
-
-            if (gwy_container_gis_string(data, gwy_app_get_lawn_palette_key_for_id(id), &gradient))
-            gwy_container_set_const_string(data, gwy_app_get_brick_palette_key_for_id(newid), gradient);
+            gwy_file_set_title(data, GWY_FILE_VOLUME, newid, title, TRUE);
+            g_free(title);
+            gwy_file_sync_items(data, GWY_FILE_CMAP, id,
+                                data, GWY_FILE_VOLUME, newid,
+                                GWY_FILE_ITEM_PALETTE, FALSE);
 
             g_object_unref(brick);
         }
     }
 
 end:
-    //GWY_OBJECT_UNREF(args.result); //do this better?
     g_object_unref(args.params);
 }
 
 static GwyDialogOutcome
 run_gui(ModuleArgs *args)
 {
-    GtkWidget *hbox, *align;
+    GtkWidget *hbox;
     GwyParamTable *table;
     GwyDialog *dialog;
     ModuleGUI gui;
-    GwyDialogOutcome outcome;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
 
     gui.dialog = gwy_dialog_new(_("Volumize Curves"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    hbox = gwy_hbox_new(0);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
     gwy_dialog_add_content(GWY_DIALOG(gui.dialog), hbox, TRUE, TRUE, 0);
 
-    align = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-    gtk_box_pack_start(GTK_BOX(hbox), align, FALSE, FALSE, 0);
-
-    hbox = gwy_hbox_new(20);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 20);
     gwy_dialog_add_content(GWY_DIALOG(gui.dialog), hbox, TRUE, TRUE, 4);
 
     table = gui.table = gwy_param_table_new(args->params);
@@ -230,10 +191,6 @@ run_gui(ModuleArgs *args)
         gwy_param_table_append_lawn_segment(table, PARAM_SEGMENT, args->lawn);
         gwy_param_table_add_enabler(table, PARAM_ENABLE_SEGMENT, PARAM_SEGMENT);
     }
-    gwy_param_table_append_checkbox(table, PARAM_CUT);
-    gwy_param_table_append_slider(table, PARAM_CUT_N);
-    gwy_param_table_append_radio_row(table, PARAM_CUT_DIR);
-
     gwy_param_table_append_info(table, INFO_RESULT, "");
 
     gwy_dialog_add_param_table(dialog, table);
@@ -241,20 +198,17 @@ run_gui(ModuleArgs *args)
 
     g_signal_connect_swapped(gui.table, "param-changed", G_CALLBACK(param_changed), &gui);
 
-    outcome = gwy_dialog_run(dialog);
-
-    return outcome;
+    return gwy_dialog_run(dialog);
 }
 
-static BrickabilityError
+static gboolean
 check_brickability(GwyLawn *lawn, gboolean segment_enabled, gint segment,
                    gint *bxres, gint *byres, gint *bzres)
 {
     gint xres = gwy_lawn_get_xres(lawn);
     gint yres = gwy_lawn_get_yres(lawn);
-    gint from, end, col, row, ndata0, ndata, mindata;
+    gint from, end, col, row, ndata0, ndata;
     const gint *segments;
-    gboolean errsize = FALSE;
 
     *bxres = xres;
     *byres = yres;
@@ -268,35 +222,24 @@ check_brickability(GwyLawn *lawn, gboolean segment_enabled, gint segment,
         ndata0 = end - from;
     }
     if (ndata0 <= 0)
-        return BB_ERROR_NO_DATA;
+        return FALSE;
 
-    mindata = ndata0;
-    for (col = 0; col<xres; col++) {
-        for (row = 0; row<yres; row++) {
+    for (col = 0; col < xres; col++) {
+        for (row = 0; row < yres; row++) {
             ndata = gwy_lawn_get_curve_length(lawn, col, row);
             if (segment >= 0) {
                 segments = gwy_lawn_get_segments(lawn, col, row, NULL);
                 from = segments[2*segment];
                 end = segments[2*segment + 1];
                 ndata = end - from;
-           }
-           if (ndata != ndata0)
-                errsize = TRUE;
-
-            if (mindata > ndata)
-                mindata = ndata;
+            }
+            if (ndata != ndata0)
+                return FALSE;
         }
     }
-    if (errsize) {
-        *bzres = mindata;
-        if (mindata == 0)
-            return BB_ERROR_NO_DATA;
-        else
-        return BB_ERROR_SIZE;
-    }
-
     *bzres = ndata0;
-    return BB_OK;
+
+    return TRUE;
 }
 
 static void
@@ -304,32 +247,15 @@ param_changed(ModuleGUI *gui, G_GNUC_UNUSED gint id)
 {
     GwyParams *params = gui->args->params;
     gint xres, yres, zres;
-    gchar result[100];
-    BrickabilityError err;
-    gboolean cut;
+    gchar result[50];
     gboolean segment_enabled = gui->args->nsegments ? gwy_params_get_boolean(params, PARAM_ENABLE_SEGMENT) : FALSE;
     gint segment = segment_enabled ? gwy_params_get_int(params, PARAM_SEGMENT) : -1;
 
-    err = check_brickability(gui->args->lawn, segment_enabled, segment, &xres, &yres, &zres);
-    gwy_param_table_slider_restrict_range(gui->table, PARAM_CUT_N, 1, zres);
-
-    if (err == BB_ERROR_NO_DATA) {
-        gwy_param_table_info_set_valuestr(gui->table, INFO_RESULT, _("No data to convert."));
-        gtk_dialog_set_response_sensitive(GTK_DIALOG(gui->dialog), GTK_RESPONSE_OK, FALSE);
-    } else if (err == BB_ERROR_SIZE) {
-        g_snprintf(result, sizeof(result), _("Z mismatch, loadable volume size: %d × %d × %d"), xres, yres, zres);
+    if (!check_brickability(gui->args->lawn, segment_enabled, segment, &xres, &yres, &zres))
+        gwy_param_table_info_set_valuestr(gui->table, INFO_RESULT, _("Cannot convert to volume data"));
+    else {
+        g_snprintf(result, sizeof(result), _("Volume data size: %d × %d × %d"), xres, yres, zres);
         gwy_param_table_info_set_valuestr(gui->table, INFO_RESULT, result);
-        gtk_dialog_set_response_sensitive(GTK_DIALOG(gui->dialog), GTK_RESPONSE_OK, TRUE);
-    } else {
-        g_snprintf(result, sizeof(result), _("Volume size: %d × %d × %d"), xres, yres, zres);
-        gwy_param_table_info_set_valuestr(gui->table, INFO_RESULT, result);
-        gtk_dialog_set_response_sensitive(GTK_DIALOG(gui->dialog), GTK_RESPONSE_OK, TRUE);
-    }
-
-    if (id < 0 || id == PARAM_CUT) {
-        cut = gwy_params_get_boolean(params, PARAM_CUT);
-        gwy_param_table_set_sensitive(gui->table, PARAM_CUT_N, cut);
-        gwy_param_table_set_sensitive(gui->table, PARAM_CUT_DIR, cut);
     }
 
     gwy_dialog_invalidate(GWY_DIALOG(gui->dialog));
@@ -342,10 +268,6 @@ execute(ModuleArgs *args)
     gint ordinate = gwy_params_get_int(params, PARAM_ORDINATE);
     gboolean segment_enabled = args->nsegments ? gwy_params_get_boolean(params, PARAM_ENABLE_SEGMENT) : FALSE;
     gint segment = segment_enabled ? gwy_params_get_int(params, PARAM_SEGMENT) : -1;
-    gboolean cut = gwy_params_get_boolean(params, PARAM_CUT);
-    gint ncut = gwy_params_get_int(params, PARAM_CUT_N);
-    CutDir cutdir = gwy_params_get_enum(params, PARAM_CUT_DIR);
-
     GwyLawn *lawn = args->lawn;
     GwyBrick *brick;
     gint col, row, lev, xres, yres, zres, from;
@@ -353,36 +275,28 @@ execute(ModuleArgs *args)
     const gdouble *ydata;
     gdouble *bdata;
 
-    if (check_brickability(args->lawn, segment_enabled, segment, &xres, &yres, &zres) == BB_ERROR_NO_DATA)
+    if (!check_brickability(args->lawn, segment_enabled, segment, &xres, &yres, &zres))
         return NULL;
 
-    if (cut)
-        zres = ncut;
-
     brick = gwy_brick_new(xres, yres, zres, gwy_lawn_get_xreal(lawn), gwy_lawn_get_yreal(lawn), zres, FALSE);
     bdata = gwy_brick_get_data(brick);
 
-    for (col = 0; col<xres; col++) {
-        for (row = 0; row<yres; row++) {
+    for (col = 0; col < xres; col++) {
+        for (row = 0; row < yres; row++) {
             ydata = gwy_lawn_get_curve_data_const(lawn, col, row, ordinate, NULL);
             from = 0;
             if (segment_enabled && segment >= 0) {
                 segments = gwy_lawn_get_segments(lawn, col, row, NULL);
                 from = segments[2*segment];
-                if (cut && cutdir == CUT_DIR_END)
-                    from += (segments[2*segment + 1] - from) - zres;
-            } else {
-               if (cut && cutdir == CUT_DIR_END)
-                   from += gwy_lawn_get_curve_length(lawn, col, row) - zres;
             }
             for (lev = 0; lev < zres; lev++)
                 bdata[col + xres*row + xres*yres*lev] = ydata[from + lev];
         }
     }
 
-    gwy_brick_set_si_unit_x(brick, gwy_lawn_get_si_unit_xy(lawn));
-    gwy_brick_set_si_unit_y(brick, gwy_lawn_get_si_unit_xy(lawn));
-    gwy_brick_set_si_unit_w(brick, gwy_lawn_get_si_unit_curve(lawn, ordinate));
+    gwy_unit_assign(gwy_brick_get_unit_x(brick), gwy_lawn_get_unit_xy(lawn));
+    gwy_unit_assign(gwy_brick_get_unit_y(brick), gwy_lawn_get_unit_xy(lawn));
+    gwy_unit_assign(gwy_brick_get_unit_w(brick), gwy_lawn_get_unit_curve(lawn, ordinate));
 
     return brick;
 }
