@@ -1,6 +1,6 @@
 /*
- *  $Id: accurexii-txt.c 26163 2024-02-06 16:56:55Z yeti-dn $
- *  Copyright (C) 2014 David Necas (Yeti).
+ *  $Id: accurexii-txt.c 28789 2025-11-04 17:14:03Z yeti-dn $
+ *  Copyright (C) 2014-2025 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
  *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
@@ -34,15 +34,11 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -53,22 +49,22 @@
 #define DATA_MAGIC "[Data Section]"
 #define EXTENSION ".txt"
 
-static gboolean      module_register(void);
-static gint          acii_detect    (const GwyFileDetectInfo *fileinfo,
-                                     gboolean only_name);
-static GwyContainer* acii_load      (const gchar *filename,
-                                     GwyRunType mode,
-                                     GError **error);
-static void          add_meta       (gpointer hkey,
-                                     gpointer hvalue,
-                                     gpointer user_data);
+static gboolean module_register(void);
+static gint     detect_file    (const GwyFileDetectInfo *fileinfo,
+                                gboolean only_name);
+static GwyFile* load_file      (const gchar *filename,
+                                GwyRunModeFlags mode,
+                                GError **error);
+static void     add_meta       (gpointer hkey,
+                                gpointer hvalue,
+                                gpointer user_data);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Accurex II text files."),
     "Yeti <yeti@gwyddion.net>",
-    "1.1",
+    "2.0",
     "David Nečas (Yeti)",
     "2014",
 };
@@ -80,17 +76,13 @@ module_register(void)
 {
     gwy_file_func_register("accurexii-txt",
                            N_("Accurex II text files (.txt)"),
-                           (GwyFileDetectFunc)&acii_detect,
-                           (GwyFileLoadFunc)&acii_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-acii_detect(const GwyFileDetectInfo *fileinfo,
-               gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     if (only_name)
         return g_str_has_suffix(fileinfo->name_lowercase, EXTENSION) ? 10 : 0;
@@ -104,14 +96,15 @@ acii_detect(const GwyFileDetectInfo *fileinfo,
     return 90;
 }
 
-static GwyContainer*
-acii_load(const gchar *filename,
-          G_GNUC_UNUSED GwyRunType mode,
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
           GError **error)
 {
-    GwyContainer *container = NULL, *meta;
-    GwyDataField *dfield = NULL;
-    GwySIUnit *xunit, *yunit, *zunit;
+    GwyFile *file = NULL;
+    GwyContainer *meta;
+    GwyField *dfield = NULL;
+    GwyUnit *xunit, *yunit, *zunit;
     gchar *end, *p, *value, *header = NULL, *buffer = NULL;
     gchar **dims;
     GwyTextHeaderParser parser;
@@ -146,7 +139,7 @@ acii_load(const gchar *filename,
         goto fail;
     }
 
-    gwy_clear(&parser, 1);
+    gwy_clear1(parser);
     parser.key_value_separator = "=";
     hash = gwy_text_header_parse(header, &parser, NULL, NULL);
 
@@ -171,21 +164,21 @@ acii_load(const gchar *filename,
         goto fail;
     }
     xreal = g_ascii_strtod(dims[0], &end);
-    xunit = gwy_si_unit_new_parse(end, &power10x);
+    xunit = gwy_unit_new_parse(end, &power10x);
     yreal = g_ascii_strtod(dims[1], &end);
-    yunit = gwy_si_unit_new_parse(end, &power10y);
+    yunit = gwy_unit_new_parse(end, &power10y);
     g_strfreev(dims);
     sanitise_real_size(&xreal, "x size");
     sanitise_real_size(&yreal, "y size");
-    if (!gwy_si_unit_equal(xunit, yunit)) {
+    if (!gwy_unit_equal(xunit, yunit)) {
         g_warning("X and Y units differ, using X");
     }
 
-    xreal *= pow10(power10x);
-    yreal *= pow10(power10y);
+    xreal *= gwy_exp10(power10x);
+    yreal *= gwy_exp10(power10y);
 
-    dfield = gwy_data_field_new(xres, yres, xreal, yreal, FALSE);
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(dfield), xunit);
+    dfield = gwy_field_new(xres, yres, xreal, yreal, FALSE);
+    gwy_unit_assign(gwy_field_get_unit_xy(dfield), xunit);
 
     g_object_unref(xunit);
     g_object_unref(yunit);
@@ -197,31 +190,28 @@ acii_load(const gchar *filename,
     q = 1.0;
     if (g_str_has_prefix(p, "Z-unit:")) {
         value = gwy_str_next_line(&p);
-        zunit = gwy_si_unit_new_parse(value + strlen("Z unit:"), &power10z);
-        gwy_si_unit_assign(gwy_data_field_get_si_unit_z(dfield), zunit);
+        zunit = gwy_unit_new_parse(value + strlen("Z unit:"), &power10z);
+        gwy_unit_assign(gwy_field_get_unit_z(dfield), zunit);
         g_object_unref(zunit);
-        q *= pow10(power10z);
+        q *= gwy_exp10(power10z);
     }
 
-    if (!gwy_parse_doubles(p, gwy_data_field_get_data(dfield), GWY_PARSE_DOUBLES_FREE_FORM, &yres, &xres, NULL, &err)) {
+    if (!gwy_parse_doubles(p, gwy_field_get_data(dfield), GWY_PARSE_DOUBLES_FREE_FORM, &yres, &xres, NULL, &err)) {
         err_PARSE_DOUBLES(error, &err);
-        GWY_OBJECT_UNREF(dfield);
+        g_clear_object(&dfield);
         goto fail;
     }
-    gwy_data_field_multiply(dfield, q);
+    gwy_field_multiply(dfield, q);
 
-    container = gwy_container_new();
-
-    gwy_container_pass_object(container, gwy_app_get_data_key_for_id(0), dfield);
+    file = gwy_file_new_in_construction();
 
+    gwy_file_pass_image(file, 0, dfield);
     if ((value = g_hash_table_lookup(hash, "Data Type")))
-        gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(0), value);
-
+        gwy_file_set_title(file, GWY_FILE_IMAGE, 0, value, FALSE);
     meta = gwy_container_new();
     g_hash_table_foreach(hash, add_meta, meta);
-    gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(0), meta);
-
-    gwy_file_channel_import_log_add(container, 0, NULL, filename);
+    gwy_file_pass_meta(file, GWY_FILE_IMAGE, 0, meta);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
 
 fail:
     g_free(header);
@@ -229,7 +219,7 @@ fail:
     if (hash)
         g_hash_table_destroy(hash);
 
-    return container;
+    return file;
 }
 
 static void
