@@ -1,6 +1,6 @@
 /*
- *  $Id: afmw-spec.c 25675 2023-09-18 05:41:59Z yeti-dn $
- *  Copyright (C) 2018 David Necas (Yeti).
+ *  $Id: afmw-spec.c 28803 2025-11-05 11:58:19Z yeti-dn $
+ *  Copyright (C) 2018-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -37,15 +37,10 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
 #include <string.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/datafield.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/data-browser.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -54,7 +49,7 @@
 
 typedef struct {
     gchar *name;       /* Extend Z-Sense, Retract T-B, ... */
-    GwySIUnit *unit;   /* unit */
+    GwyUnit *unit;   /* unit */
     gdouble magnitude; /* factor for data conversion */
 } AFMWColumn;
 
@@ -79,10 +74,10 @@ typedef struct {
 } AFMWSpectraSet;
 
 static gboolean        module_register          (void);
-static gint            afmw_detect              (const GwyFileDetectInfo *fileinfo,
+static gint            detect_file              (const GwyFileDetectInfo *fileinfo,
                                                  gboolean only_name);
-static GwyContainer*   afmw_load                (const gchar *filename,
-                                                 GwyRunType mode,
+static GwyFile*        load_file                (const gchar *filename,
+                                                 GwyRunModeFlags mode,
                                                  GError **error);
 static gboolean        check_compatibility      (const AFMWSingleFile *afmwfile,
                                                  const AFMWSingleFile *template);
@@ -98,7 +93,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports AFM Workshop spectrum files."),
     "Yeti <yeti@gwyddion.net>",
-    "1.1",
+    "2.0",
     "David Nečas (Yeti)",
     "2018",
 };
@@ -110,10 +105,7 @@ module_register(void)
 {
     gwy_file_func_register("afmw_spec",
                            N_("AFM Workshop spectrum files (.csv)"),
-                           (GwyFileDetectFunc)&afmw_detect,
-                           (GwyFileLoadFunc)&afmw_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
@@ -130,8 +122,7 @@ find_field_in_head(const GwyFileDetectInfo *fileinfo,
 }
 
 static gint
-afmw_detect(const GwyFileDetectInfo *fileinfo,
-            gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     const guchar *p;
 
@@ -158,19 +149,18 @@ afmw_detect(const GwyFileDetectInfo *fileinfo,
     return 0;
 }
 
-static GwyContainer*
-afmw_load(const gchar *filename,
-          G_GNUC_UNUSED GwyRunType mode,
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
           GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     gchar **filenames;
     AFMWSpectraSet specset;
     AFMWSingleFile *afmwfile;
-    GQuark quark;
     guint i, j;
 
-    gwy_clear(&specset, 1);
+    gwy_clear1(specset);
     filenames = find_all_file_names(filename);
     /* When we cannot enumerate files, just create a list containing the single file name we were given explicitly. */
     if (!filenames) {
@@ -198,28 +188,28 @@ afmw_load(const gchar *filename,
         if (!specset.spectra[i])
             continue;
 
-        if (!container)
-            container = gwy_container_new();
-        quark = gwy_app_get_spectra_key_for_id(j);
-        gwy_container_set_object(container, quark, specset.spectra[i]);
+        if (!file)
+            file = gwy_file_new_in_construction();
+        gwy_file_set_spectra(file, j, specset.spectra[i]);
+        gwy_log_add_import(file, GWY_FILE_SPECTRA, j, NULL, filename);
         j++;
     }
 
-    if (!container)
+    if (!file)
         err_NO_DATA(error);
 
 fail:
     if (specset.template) {
         if (specset.spectra) {
             for (i = 0; i < specset.template->ncolumns; i++)
-                GWY_OBJECT_UNREF(specset.spectra[i]);
+                g_clear_object(specset.spectra + i);
             g_free(specset.spectra);
         }
         afmw_single_file_free(specset.template);
     }
     g_strfreev(filenames);
 
-    return container;
+    return file;
 }
 
 static gboolean
@@ -234,7 +224,7 @@ check_compatibility(const AFMWSingleFile *afmwfile,
     for (i = 0; i < template->ncolumns; i++) {
         datcolumn = afmwfile->columns + i;
         tmplcolumn = template->columns + i;
-        if (!gwy_strequal(datcolumn->name, tmplcolumn->name) || !gwy_si_unit_equal(datcolumn->unit, tmplcolumn->unit))
+        if (!gwy_strequal(datcolumn->name, tmplcolumn->name) || !gwy_unit_equal(datcolumn->unit, tmplcolumn->unit))
             return FALSE;
     }
     return TRUE;
@@ -264,15 +254,15 @@ parse_column_header(gchar *colname, AFMWColumn *column)
             unit = p+1;
             strip_space_back(p, colname);
             *q = '\0';
-            column->unit = gwy_si_unit_new_parse(unit, &power10);
-            column->magnitude = pow10(power10);
+            column->unit = gwy_unit_new_parse(unit, &power10);
+            column->magnitude = gwy_exp10(power10);
         }
         else {
             g_warning("Column header %s has only opening (.", colname);
         }
     }
     if (!column->unit)
-        column->unit = gwy_si_unit_new(NULL);
+        column->unit = gwy_unit_new(NULL);
 }
 
 static void
@@ -283,7 +273,7 @@ add_curves_to_spectra_set(AFMWSpectraSet *specset,
     gdouble real, off;
     gboolean reversed;
     gdouble *data = afmwfile->data, *d;
-    GwyDataLine *dline;
+    GwyLine *dline;
     GwySpectra *spec;
     const AFMWColumn *abscissa = NULL, *ordinate;
 
@@ -314,11 +304,11 @@ add_curves_to_spectra_set(AFMWSpectraSet *specset,
             continue;
         }
 
-        dline = gwy_data_line_new(nrows, real, FALSE);
-        gwy_data_line_set_offset(dline, off);
-        gwy_si_unit_assign(gwy_data_line_get_si_unit_x(dline), abscissa->unit);
-        gwy_si_unit_assign(gwy_data_line_get_si_unit_y(dline), ordinate->unit);
-        d = gwy_data_line_get_data(dline);
+        dline = gwy_line_new(nrows, real, FALSE);
+        gwy_line_set_offset(dline, off);
+        gwy_unit_assign(gwy_line_get_unit_x(dline), abscissa->unit);
+        gwy_unit_assign(gwy_line_get_unit_y(dline), ordinate->unit);
+        d = gwy_line_get_data(dline);
         if (reversed) {
             for (j = 0; j < nrows; j++)
                 d[nrows-1 - j] = data[i + j*ncolumns];
@@ -330,7 +320,7 @@ add_curves_to_spectra_set(AFMWSpectraSet *specset,
 
         if (!(spec = specset->spectra[i])) {
             spec = specset->spectra[i] = gwy_spectra_new();
-            gwy_si_unit_set_from_string(gwy_spectra_get_si_unit_xy(spec), "m");
+            gwy_unit_set_from_string(gwy_spectra_get_unit_xy(spec), "m");
             gwy_spectra_set_title(spec, ordinate->name);
             gwy_spectra_set_spectrum_x_label(spec, abscissa->name);
             gwy_spectra_set_spectrum_y_label(spec, ordinate->name);
@@ -379,8 +369,8 @@ read_one_afmw_file(const gchar *filename, GError **error)
         sep++;
         if (g_str_has_prefix(line, "X, ") || g_str_has_prefix(line, "Y, ")) {
             gint power10;
-            GwySIUnit *unit = gwy_si_unit_new_parse(line + 3, &power10);
-            gdouble v = pow10(power10) * g_ascii_strtod(sep, NULL);
+            GwyUnit *unit = gwy_unit_new_parse(line + 3, &power10);
+            gdouble v = gwy_exp10(power10) * g_ascii_strtod(sep, NULL);
 
             if (line[0] == 'Y') {
                 afmwfile->y = v;
@@ -438,7 +428,7 @@ fail:
     if (afmwfile->columns) {
         for (i = 0; i < afmwfile->ncolumns; i++) {
             g_free(afmwfile->columns[i].name);
-            GWY_OBJECT_UNREF(afmwfile->columns[i].unit);
+            g_clear_object(&afmwfile->columns[i].unit);
         }
     }
     g_free(afmwfile);
@@ -453,7 +443,7 @@ afmw_single_file_free(AFMWSingleFile *afmwfile)
 
     for (i = 0; i < afmwfile->ncolumns; i++) {
         g_free(afmwfile->columns[i].name);
-        GWY_OBJECT_UNREF(afmwfile->columns[i].unit);
+        g_clear_object(&afmwfile->columns[i].unit);
     }
     g_free(afmwfile->columns);
     g_free(afmwfile->data);
