@@ -1,6 +1,6 @@
 /*
- *  $Id: aistfile.c 26163 2024-02-06 16:56:55Z yeti-dn $
- *  Copyright (C) 2009-2021 David Necas (Yeti).
+ *  $Id: aistfile.c 28805 2025-11-05 12:28:20Z yeti-dn $
+ *  Copyright (C) 2009-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -41,18 +41,13 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/datafield.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/data-browser.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -79,44 +74,44 @@ typedef struct {
 } AistCurve;
 
 typedef struct {
-    GwyContainer *container;
+    GwyFile *file;
     const gchar *filename;
     gint channel_id;
     gint graph_id;
 } AistContext;
 
-static gboolean      module_register   (void);
-static gint          aist_detect       (const GwyFileDetectInfo *fileinfo,
-                                        gboolean only_name);
-static GwyContainer* aist_load         (const gchar *filename,
-                                        GwyRunType mode,
-                                        GError **error);
-static gboolean      read_qt_bool      (const guchar **p,
-                                        gsize *size,
-                                        gboolean *value);
-static gboolean      read_qt_byte      (const guchar **p,
-                                        gsize *size,
-                                        guint *value);
-static gboolean      read_qt_int       (const guchar **p,
-                                        gsize *size,
-                                        gint *value);
-static gboolean      read_qt_double    (const guchar **p,
-                                        gsize *size,
-                                        gdouble *value);
-static gboolean      read_qt_string    (const guchar **p,
-                                        gsize *size,
-                                        gchar **value);
-static gboolean      read_qt_byte_array(const guchar **p,
-                                        gsize *size,
-                                        guint *len,
-                                        const guchar **value);
+static gboolean module_register   (void);
+static gint     detect_file       (const GwyFileDetectInfo *fileinfo,
+                                   gboolean only_name);
+static GwyFile* load_file         (const gchar *filename,
+                                   GwyRunModeFlags mode,
+                                   GError **error);
+static gboolean read_qt_bool      (const guchar **p,
+                                   gsize *size,
+                                   gboolean *value);
+static gboolean read_qt_byte      (const guchar **p,
+                                   gsize *size,
+                                   guint *value);
+static gboolean read_qt_int       (const guchar **p,
+                                   gsize *size,
+                                   gint *value);
+static gboolean read_qt_double    (const guchar **p,
+                                   gsize *size,
+                                   gdouble *value);
+static gboolean read_qt_string    (const guchar **p,
+                                   gsize *size,
+                                   gchar **value);
+static gboolean read_qt_byte_array(const guchar **p,
+                                   gsize *size,
+                                   guint *len,
+                                   const guchar **value);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports AIST-NT data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.5.3",
+    "0.6",
     "David Nečas (Yeti)",
     "2009",
 };
@@ -128,17 +123,13 @@ module_register(void)
 {
     gwy_file_func_register("aistfile",
                            N_("AIST-NT files (.aist)"),
-                           (GwyFileDetectFunc)&aist_detect,
-                           (GwyFileLoadFunc)&aist_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-aist_detect(const GwyFileDetectInfo *fileinfo,
-            gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     gint score = 0;
     const guchar *buf = fileinfo->head;
@@ -217,11 +208,11 @@ read_aist_common(const guchar **p, gsize *size,
 
 /* Lateral units are a weird thing but they contain the actual units in bracket somewhere.   If there are no brackes,
  * try to interpret the entire string as units -- I've been told this is the right thing.  */
-static GwySIUnit*
+static GwyUnit*
 extract_units(const gchar *label,
               gdouble *q)
 {
-    GwySIUnit *unit;
+    GwyUnit *unit;
     gint power10;
     const gchar *s;
     gchar *t;
@@ -230,27 +221,27 @@ extract_units(const gchar *label,
         t = g_strdup(s + 1);
         if (strchr(t, ']'))
             *strchr(t, ']') = '\0';
-        unit = gwy_si_unit_new_parse(t, &power10);
+        unit = gwy_unit_new_parse(t, &power10);
         g_free(t);
     }
     else
-        unit = gwy_si_unit_new_parse(label, &power10);
+        unit = gwy_unit_new_parse(label, &power10);
 
-    *q = pow10(power10);
+    *q = gwy_exp10(power10);
     return unit;
 }
 
-static GwyDataField*
-make_mask_field(GwyDataField *dfield,
+static GwyField*
+make_mask_field(GwyField *dfield,
                 const guchar *data)
 {
-    GwyDataField *mfield;
+    GwyField *mfield;
     guint xres, yres, i, j, rowstride;
     const guchar *datarow;
     gdouble *d;
 
-    xres = gwy_data_field_get_xres(dfield);
-    yres = gwy_data_field_get_yres(dfield);
+    xres = gwy_field_get_xres(dfield);
+    yres = gwy_field_get_yres(dfield);
     rowstride = xres;
 
     /* Do not create the mask if it is all zero. */
@@ -261,8 +252,8 @@ make_mask_field(GwyDataField *dfield,
     if (i == rowstride*yres)
         return NULL;
 
-    mfield = gwy_data_field_new_alike(dfield, FALSE);
-    d = gwy_data_field_get_data(mfield);
+    mfield = gwy_field_new_alike(dfield, FALSE);
+    d = gwy_field_get_data(mfield);
 
     for (i = 0; i < yres; i++) {
         datarow = data + (yres-1 - i)*rowstride;
@@ -278,8 +269,8 @@ static gboolean
 read_aist_raster(const guchar **p, gsize *size, AistContext *context)
 {
     AistRaster raster;
-    GwyDataField *dfield, *mfield;
-    GwySIUnit *xyunit, *zunit;
+    GwyField *dfield, *mfield;
+    GwyUnit *xyunit, *zunit;
     gboolean ok = FALSE;
     guint i, n, len;
     const guchar *data;
@@ -288,7 +279,7 @@ read_aist_raster(const guchar **p, gsize *size, AistContext *context)
     gdouble *d;
     gdouble qxy, qz;
 
-    gwy_clear(&raster, 1);
+    gwy_clear1(raster);
 
     gwy_debug("reading common");
     if (!read_aist_common(p, size, &raster.common))
@@ -314,28 +305,28 @@ read_aist_raster(const guchar **p, gsize *size, AistContext *context)
 
     xyunit = extract_units(raster.xunits, &qxy);
     zunit = extract_units(raster.zunits, &qz);
-    dfield = gwy_data_field_new(raster.xres, raster.yres,
-                                qxy*fabs(raster.right - raster.left), qxy*fabs(raster.bottom - raster.top),
-                                FALSE);
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(dfield), xyunit);
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_z(dfield), zunit);
+    dfield = gwy_field_new(raster.xres, raster.yres,
+                           qxy*fabs(raster.right - raster.left), qxy*fabs(raster.bottom - raster.top),
+                           FALSE);
+    gwy_unit_assign(gwy_field_get_unit_xy(dfield), xyunit);
+    gwy_unit_assign(gwy_field_get_unit_z(dfield), zunit);
     /* Apparently this is not generally wanted.
-    gwy_data_field_set_xoffset(dfield, qxy*MIN(raster.left, raster.right));
-    gwy_data_field_set_yoffset(dfield, qxy*MIN(raster.top, raster.bottom));
+    gwy_field_set_xoffset(dfield, qxy*MIN(raster.left, raster.right));
+    gwy_field_set_yoffset(dfield, qxy*MIN(raster.top, raster.bottom));
     */
 
     g_object_unref(xyunit);
     g_object_unref(zunit);
 
-    d = gwy_data_field_get_data(dfield);
+    d = gwy_field_get_data(dfield);
     for (i = 0; i < raster.yres; i++) {
         gwy_convert_raw_data(data + i*raster.xres*sizeof(gdouble), raster.xres, 1,
                              GWY_RAW_DATA_DOUBLE, GWY_BYTE_ORDER_LITTLE_ENDIAN,
                              d + (raster.yres-1 - i)*raster.xres, qz, 0.0);
     }
 
-    gwy_container_pass_object(context->container, gwy_app_get_data_key_for_id(context->channel_id), dfield);
-    gwy_file_channel_import_log_add(context->container, context->channel_id, NULL, context->filename);
+    gwy_file_pass_image(context->file, context->channel_id, dfield);
+    gwy_log_add_import(context->file, GWY_FILE_IMAGE, context->channel_id, NULL, context->filename);
 
     cn = raster.common.name;
     while (cn[0] == '_')
@@ -344,7 +335,7 @@ read_aist_raster(const guchar **p, gsize *size, AistContext *context)
         s = g_strchomp(g_strndup(cn, s - cn));
     else
         s = g_strdup(cn);
-    gwy_container_set_string(context->container, gwy_app_get_data_title_key_for_id(context->channel_id), s);
+    gwy_file_pass_title(context->file, GWY_FILE_IMAGE, context->channel_id, s);
 
     /* At this moment we consider the loading successful. */
     ok = TRUE;
@@ -356,7 +347,7 @@ read_aist_raster(const guchar **p, gsize *size, AistContext *context)
             mfield = make_mask_field(dfield, data);
             if (mfield) {
                 gwy_app_channel_remove_bad_data(dfield, mfield);
-                gwy_container_pass_object(context->container, gwy_app_get_mask_key_for_id(context->channel_id), mfield);
+                gwy_file_pass_image_mask(context->file, context->channel_id, mfield);
             }
         }
 
@@ -380,7 +371,7 @@ read_aist_curve(const guchar **p, gsize *size, AistContext *context)
     AistCurve curve;
     GwyGraphModel *gmodel;
     GwyGraphCurveModel *gcmodel;
-    GwySIUnit *xunit, *yunit;
+    GwyUnit *xunit, *yunit;
     gboolean ok = FALSE;
     guint len, viewlen, i;
     const guchar *data, *viewdata;
@@ -388,9 +379,8 @@ read_aist_curve(const guchar **p, gsize *size, AistContext *context)
     gdouble *xdatacal, *ydatacal;
     gdouble *must_free = NULL;
     gdouble qx, qy;
-    GQuark quark;
 
-    gwy_clear(&curve, 1);
+    gwy_clear1(curve);
 
     gwy_debug("reading common");
     if (!read_aist_common(p, size, &curve.common))
@@ -449,14 +439,14 @@ read_aist_curve(const guchar **p, gsize *size, AistContext *context)
     g_object_unref(gcmodel);
     g_object_set(gmodel,
                  "title", curve.common.name,
-                 "si-unit-x", xunit,
-                 "si-unit-y", yunit,
+                 "unit-x", xunit,
+                 "unit-y", yunit,
                  NULL);
     g_object_unref(xunit);
     g_object_unref(yunit);
 
-    quark = gwy_app_get_graph_key_for_id(context->graph_id+1);
-    gwy_container_pass_object(context->container, quark, gmodel);
+    gwy_file_pass_graph(context->file, context->graph_id, gmodel);
+    gwy_log_add_import(context->file, GWY_FILE_GRAPH, context->graph_id, NULL, context->filename);
 
     context->graph_id++;
 
@@ -543,9 +533,9 @@ fail:
     return ok;
 }
 
-static GwyContainer*
-aist_load(const gchar *filename,
-          G_GNUC_UNUSED GwyRunType mode,
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
           GError **error)
 {
     AistContext context;
@@ -562,7 +552,7 @@ aist_load(const gchar *filename,
     p = buffer;
     remaining = size;
 
-    context.container = gwy_container_new();
+    context.file = gwy_file_new_in_construction();
     context.filename = filename;
     context.channel_id = 0;
     context.graph_id = 0;
@@ -571,12 +561,12 @@ aist_load(const gchar *filename,
     gwy_file_abandon_contents(buffer, size, NULL);
 
     if ((context.channel_id == 0) && (context.graph_id == 0)) {
-        g_object_unref(context.container);
-        context.container = NULL;
+        g_object_unref(context.file);
+        context.file = NULL;
         err_NO_DATA(error);
     }
 
-    return context.container;
+    return context.file;
 }
 
 static gboolean
