@@ -1,6 +1,7 @@
 /*
- *  $Id: alicona.c 26135 2024-01-15 12:54:27Z yeti-dn $
- *  Copyright (C) 2011-2019 David Necas (Yeti).
+ *  $Id: alicona.c 28789 2025-11-04 17:14:03Z yeti-dn $
+ *  Copyright (C) 2011-2025 David Necas (Yeti).
+ *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
  *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
@@ -42,14 +43,10 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -95,34 +92,34 @@ typedef struct {
 } Al3DFile;
 
 static gboolean       module_register   (void);
-static gint           al3d_detect       (const GwyFileDetectInfo *fileinfo,
+static gint           detect_file       (const GwyFileDetectInfo *fileinfo,
                                          gboolean only_name);
-static GwyContainer*  al3d_load         (const gchar *filename,
-                                         GwyRunType mode,
+static GwyFile*       load_file         (const gchar *filename,
+                                         GwyRunModeFlags mode,
                                          GError **error);
-static void           set_title         (GwyContainer *container,
+static void           set_title         (GwyFile *file,
                                          guint id,
                                          const gchar *name,
                                          gint component);
-static void           add_meta          (GwyContainer *container,
+static void           add_meta          (GwyFile *file,
                                          guint id,
                                          const Al3DFile *afile);
 static gchar*         texture_lo_ptr    (const Al3DTag *tag);
 static gchar*         texture_ptr       (const Al3DTag *tag);
-static GwyDataField*  read_depth_image  (const Al3DFile *afile,
+static GwyField*      read_depth_image  (const Al3DFile *afile,
                                          const guchar *buffer,
-                                         GwyDataField **badmask);
-static GwyDataField*  read_hi_lo_texture(const Al3DFile *afile,
+                                         GwyField **badmask);
+static GwyField*      read_hi_lo_texture(const Al3DFile *afile,
                                          const Al3DTag *hitag,
                                          const Al3DTag *lotag,
                                          const guchar *buffer,
                                          GError **error);
-static GwyDataField*  read_texture      (const Al3DFile *afile,
+static GwyField*      read_texture      (const Al3DFile *afile,
                                          const Al3DTag *tag,
                                          guint planeno,
                                          const guchar *buffer,
                                          GError **error);
-static gboolean       al3d_load_header  (Al3DFile *afile,
+static gboolean       file_load_header  (Al3DFile *afile,
                                          const guchar *buffer,
                                          gsize size,
                                          GError **error);
@@ -143,7 +140,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports Alicona Imaging AL3D files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.4",
+    "0.5",
     "David Nečas (Yeti)",
     "2011",
 };
@@ -155,17 +152,13 @@ module_register(void)
 {
     gwy_file_func_register("alicona",
                            N_("Alicona Imaging AL3D files (.al3d)"),
-                           (GwyFileDetectFunc)&al3d_detect,
-                           (GwyFileLoadFunc)&al3d_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-al3d_detect(const GwyFileDetectInfo *fileinfo,
-           gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     if (only_name)
         return g_str_has_suffix(fileinfo->name_lowercase, EXTENSION) ? 10 : 0;
@@ -176,13 +169,13 @@ al3d_detect(const GwyFileDetectInfo *fileinfo,
     return 0;
 }
 
-static GwyContainer*
-al3d_load(const gchar *filename,
-          G_GNUC_UNUSED GwyRunType mode,
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
           GError **error)
 {
-    GwyContainer *container = NULL;
-    GwyDataField *field = NULL;
+    GwyFile *file = NULL;
+    GwyField *field = NULL;
     Al3DFile afile;
     guchar *buffer = NULL;
     guint firstfreepos;
@@ -195,7 +188,7 @@ al3d_load(const gchar *filename,
         goto fail;
     }
 
-    if (!al3d_load_header(&afile, buffer, size, error))
+    if (!file_load_header(&afile, buffer, size, error))
         goto fail;
 
     if (!read_uint_tag(&afile, "Cols", &afile.xres, error) || !read_uint_tag(&afile, "Rows", &afile.yres, error))
@@ -256,21 +249,21 @@ al3d_load(const gchar *filename,
         goto fail;
     }
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
 
     if (afile.depthoffset) {
-        GwyDataField *badmask = NULL;
+        GwyField *badmask = NULL;
 
         field = read_depth_image(&afile, buffer, &badmask);
-        gwy_container_set_object(container, gwy_app_get_data_key_for_id(id), field);
+        gwy_file_set_image(file, id, field);
         if (badmask) {
             gwy_app_channel_remove_bad_data(field, badmask);
-            gwy_container_pass_object(container, gwy_app_get_mask_key_for_id(id), badmask);
+            gwy_file_pass_image_mask(file, id, badmask);
         }
         g_object_unref(field);
-        set_title(container, id, "Depth", -1);
-        add_meta(container, id, &afile);
-        gwy_file_channel_import_log_add(container, id, NULL, filename);
+        set_title(file, id, "Depth", -1);
+        add_meta(file, id, &afile);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, id, NULL, filename);
         id++;
     }
 
@@ -291,11 +284,11 @@ al3d_load(const gchar *filename,
                 g_free(name);
                 continue;
             }
-            gwy_container_pass_object(container, gwy_app_get_data_key_for_id(id), field);
-            set_title(container, id, name, -1);
-            add_meta(container, id, &afile);
+            gwy_file_pass_image(file, id, field);
+            set_title(file, id, name, -1);
+            add_meta(file, id, &afile);
             g_free(name);
-            gwy_file_channel_import_log_add(container, id, NULL, filename);
+            gwy_log_add_import(file, GWY_FILE_IMAGE, id, NULL, filename);
             id++;
         }
         else if ((name = texture_ptr(tag))) {
@@ -312,10 +305,10 @@ al3d_load(const gchar *filename,
                     g_clear_error(&err);
                     continue;
                 }
-                gwy_container_pass_object(container, gwy_app_get_data_key_for_id(id), field);
-                set_title(container, id, name, nplanes > 1 ? j : -1);
-                add_meta(container, id, &afile);
-                gwy_file_channel_import_log_add(container, id, NULL, filename);
+                gwy_file_pass_image(file, id, field);
+                set_title(file, id, name, nplanes > 1 ? j : -1);
+                add_meta(file, id, &afile);
+                gwy_log_add_import(file, GWY_FILE_IMAGE, id, NULL, filename);
                 id++;
             }
             g_strfreev(planes);
@@ -325,23 +318,22 @@ al3d_load(const gchar *filename,
 
 fail:
     gwy_file_abandon_contents(buffer, size, NULL);
-    if (container && !gwy_container_get_n_items(container)) {
-        g_object_unref(container);
-        container = NULL;
+    if (file && !gwy_container_get_n_items(GWY_CONTAINER(file))) {
+        g_object_unref(file);
+        file = NULL;
     }
 
-    return container;
+    return file;
 }
 
 static void
-set_title(GwyContainer *container,
+set_title(GwyFile *file,
           guint id,
           const gchar *name,
           gint component)
 {
     const gchar *gradient = NULL;
     gchar *title;
-    GQuark quark;
 
     if (component == -1)
         title = g_strdup(name);
@@ -360,21 +352,17 @@ set_title(GwyContainer *container,
     else
         title = g_strdup_printf("%s (%u)", name, component);
 
-    quark = gwy_app_get_data_title_key_for_id(id);
-    gwy_container_set_string(container, quark, title);
-    if (gradient) {
-        quark = gwy_app_get_data_palette_key_for_id(id);
-        gwy_container_set_const_string(container, quark, gradient);
-    }
+    gwy_file_pass_title(file, GWY_FILE_IMAGE, id, title);
+    if (gradient)
+        gwy_file_set_palette(file, GWY_FILE_IMAGE, id, gradient);
 }
 
 static void
-add_meta(GwyContainer *container,
+add_meta(GwyFile *file,
          guint id,
          const Al3DFile *afile)
 {
     GwyContainer *meta = gwy_container_new();
-    gchar key[32];
     guint i;
 
     gwy_container_set_const_string_by_name(meta, afile->version->key, afile->version->value);
@@ -398,8 +386,7 @@ add_meta(GwyContainer *container,
             gwy_container_set_string_by_name(meta, "Comment", p);
     }
 
-    g_snprintf(key, sizeof(key), "/%u/meta", id);
-    gwy_container_pass_object_by_name(container, key, meta);
+    gwy_file_pass_meta(file, GWY_FILE_IMAGE, id, meta);
 }
 
 static gchar*
@@ -427,12 +414,12 @@ texture_ptr(const Al3DTag *tag)
     return gwy_strreplace(tag->key, "Ptr", "", 1);
 }
 
-static GwyDataField*
+static GwyField*
 read_depth_image(const Al3DFile *afile,
                  const guchar *buffer,
-                 GwyDataField **badmask)
+                 GwyField **badmask)
 {
-    GwyDataField *field, *mask = NULL;
+    GwyField *field, *mask = NULL;
     guint rowstride = (afile->xres*sizeof(gfloat) + 7)/8*8;
     gdouble *d, *m = NULL;
     guint i;
@@ -447,11 +434,11 @@ read_depth_image(const Al3DFile *afile,
     if (!invalid_is_nan)
         invalid_eps = 1.5e-7 * invalid_value;
 
-    field = gwy_data_field_new(xres, yres, afile->dx*xres, afile->dy*yres, FALSE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(field), "m");
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(field), "m");
+    field = gwy_field_new(xres, yres, afile->dx*xres, afile->dy*yres, FALSE);
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(field), "m");
+    gwy_unit_set_from_string(gwy_field_get_unit_z(field), "m");
 
-    d = gwy_data_field_get_data(field);
+    d = gwy_field_get_data(field);
     for (i = 0; i < yres; i++) {
         gwy_convert_raw_data(buffer + afile->depthoffset + i*rowstride, xres, 1,
                              GWY_RAW_DATA_FLOAT, GWY_BYTE_ORDER_LITTLE_ENDIAN, d + i*xres, 1.0, 0.0);
@@ -459,9 +446,9 @@ read_depth_image(const Al3DFile *afile,
     for (i = 0; i < xres*yres; i++) {
         if (gwy_isnan(d[i]) || (!invalid_is_nan && fabs(d[i] - invalid_value) < invalid_eps)) {
             if (!mask) {
-                mask = gwy_data_field_new_alike(field, FALSE);
-                gwy_data_field_fill(mask, 1.0);
-                m = gwy_data_field_get_data(mask);
+                mask = gwy_field_new_alike(field, FALSE);
+                gwy_field_fill(mask, 1.0);
+                m = gwy_field_get_data(mask);
             }
             d[i] = 0.0;
             m[i] = 0.0;
@@ -487,14 +474,14 @@ check_plane_no(const Al3DFile *afile,
     return FALSE;
 }
 
-static GwyDataField*
+static GwyField*
 read_hi_lo_texture(const Al3DFile *afile,
                    const Al3DTag *hitag,
                    const Al3DTag *lotag,
                    const guchar *buffer,
                    GError **error)
 {
-    GwyDataField *field;
+    GwyField *field;
     guint xres = afile->xres, yres = afile->yres;
     guint rowstride = (afile->xres + 7)/8*8;
     guint hiplaneno = (guint)atoi(hitag->value);
@@ -507,10 +494,10 @@ read_hi_lo_texture(const Al3DFile *afile,
         || !check_plane_no(afile, lotag->key, loplaneno, error))
         return NULL;
 
-    field = gwy_data_field_new(xres, yres, afile->dx*xres, afile->dy*yres, FALSE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(field), "m");
+    field = gwy_field_new(xres, yres, afile->dx*xres, afile->dy*yres, FALSE);
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(field), "m");
 
-    d = gwy_data_field_get_data(field);
+    d = gwy_field_get_data(field);
     for (i = 0; i < yres; i++) {
         const guchar *phi = (buffer + afile->textureoffset + hiplaneno*planesize + i*rowstride);
         const guchar *plo = (buffer + afile->textureoffset + loplaneno*planesize + i*rowstride);
@@ -522,14 +509,14 @@ read_hi_lo_texture(const Al3DFile *afile,
     return field;
 }
 
-static GwyDataField*
+static GwyField*
 read_texture(const Al3DFile *afile,
              const Al3DTag *tag,
              guint planeno,
              const guchar *buffer,
              GError **error)
 {
-    GwyDataField *field;
+    GwyField *field;
     guint xres = afile->xres, yres = afile->yres;
     guint rowstride = (afile->xres + 7)/8*8;
     guint planesize = yres*rowstride;
@@ -539,10 +526,10 @@ read_texture(const Al3DFile *afile,
     if (!check_plane_no(afile, tag->key, planeno, error))
         return NULL;
 
-    field = gwy_data_field_new(xres, yres, afile->dx*xres, afile->dy*yres, FALSE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(field), "m");
+    field = gwy_field_new(xres, yres, afile->dx*xres, afile->dy*yres, FALSE);
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(field), "m");
 
-    d = gwy_data_field_get_data(field);
+    d = gwy_field_get_data(field);
     for (i = 0; i < yres; i++) {
         const guchar *p = (buffer + afile->textureoffset + planeno*planesize + i*rowstride);
 
@@ -604,7 +591,7 @@ check_tag(const Al3DTag *tag,
 }
 
 static gboolean
-al3d_load_header(Al3DFile *afile,
+file_load_header(Al3DFile *afile,
                  const guchar *buffer,
                  gsize size,
                  GError **error)
