@@ -1,6 +1,6 @@
 /*
- *  $Id: ambfile.c 26038 2023-11-16 15:55:19Z yeti-dn $
- *  Copyright (C) 2005 David Necas (Yeti), Petr Klapetek.
+ *  $Id: ambfile.c 28789 2025-11-04 17:14:03Z yeti-dn $
+ *  Copyright (C) 2005-2025 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -43,14 +43,9 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/datafield.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 
 #include "err.h"
 #include "get.h"
@@ -65,19 +60,19 @@ enum {
     PARAM_OFFSET = 0x31,
 };
 
-static gboolean      module_register(void);
-static gint          amb_detect     (const GwyFileDetectInfo *fileinfo,
-                                     gboolean only_name);
-static GwyContainer* amb_load       (const gchar *filename,
-                                     GwyRunType mode,
-                                     GError **error);
+static gboolean module_register(void);
+static gint     detect_file    (const GwyFileDetectInfo *fileinfo,
+                                gboolean only_name);
+static GwyFile* load_file      (const gchar *filename,
+                                GwyRunModeFlags mode,
+                                GError **error);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Ambios AMB data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.3",
+    "0.4",
     "David Nečas (Yeti)",
     "2011",
 };
@@ -89,18 +84,14 @@ module_register(void)
 {
     gwy_file_func_register("ambfile",
                            N_("Ambios amb files (.amb)"),
-                           (GwyFileDetectFunc)&amb_detect,
-                           (GwyFileLoadFunc)&amb_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
     gwy_file_func_set_is_unfinished("ambfile", TRUE);
 
     return TRUE;
 }
 
 static gint
-amb_detect(const GwyFileDetectInfo *fileinfo,
-           gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     if (only_name)
         return g_str_has_suffix(fileinfo->name_lowercase, EXTENSION) ? 20 : 0;
@@ -113,12 +104,12 @@ amb_detect(const GwyFileDetectInfo *fileinfo,
     return 0;
 }
 
-static GwyContainer*
-amb_load(const gchar *filename,
-         G_GNUC_UNUSED GwyRunType mode,
-         GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     guchar *buffer = NULL;
     const guchar *p;
     gsize size = 0;
@@ -126,7 +117,7 @@ amb_load(const gchar *filename,
     guint xres, yres;
     gdouble xreal;
     G_GNUC_UNUSED gdouble q;
-    GwyDataField *dfield;
+    GwyField *dfield;
 
     if (!gwy_file_get_contents(filename, &buffer, &size, &err)) {
         err_GET_FILE_CONTENTS(error, &err);
@@ -154,23 +145,23 @@ amb_load(const gchar *filename,
     q = gwy_get_gfloat_le(&p);
     gwy_debug("xreal: %g, q: %g", xreal, q);
 
-    dfield = gwy_data_field_new(xres, yres, xreal, xreal, FALSE);
+    dfield = gwy_field_new(xres, yres, xreal, xreal, FALSE);
     gwy_convert_raw_data(buffer + HEADER_SIZE, xres*yres, 1, GWY_RAW_DATA_FLOAT, GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                         gwy_data_field_get_data(dfield), 1e-3, 0.0);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), "m");
+                         gwy_field_get_data(dfield), 1e-3, 0.0);
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
+    gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), "m");
 
-    container = gwy_container_new();
-    gwy_container_pass_object(container, gwy_app_get_data_key_for_id(0), dfield);
-    gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(0), "Topography");
-    gwy_app_channel_check_nonsquare(container, 0);
+    file = gwy_file_new_in_construction();
+    gwy_file_pass_image(file, 0, dfield);
+    gwy_file_set_title(file, GWY_FILE_IMAGE, 0, "Topography", FALSE);
+    gwy_check_nonsquare_image(file, 0);
 
-    gwy_file_channel_import_log_add(container, 0, NULL, filename);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
 
 fail:
     gwy_file_abandon_contents(buffer, size, NULL);
 
-    return container;
+    return file;
 }
 
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
