@@ -1,8 +1,11 @@
 /*
- *  $Id: anasys_xml.c 26494 2024-08-09 15:04:32Z yeti-dn $
+ *  $Id: anasys_xml.c 28848 2025-11-10 18:54:08Z yeti-dn $
  *  Copyright (C) 2018 Jeffrey J. Schwartz.
  *  E-mail: schwartz@physics.ucla.edu
  *
+ *  Copyright (C) 2025 David Necas (Yeti).
+ *  E-mail: yeti@gwyddion.net.
+ *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
  *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
  *  later version.
@@ -50,16 +53,11 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdio.h>
 #include <string.h>
 #include <glib/gstdio.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/stats.h>
-#include <libprocess/spectra.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "err.h"
 #include "get.h"
 
@@ -80,31 +78,30 @@
 #define strequal(a, b) xmlStrEqual((a), (const xmlChar*)(b))
 #define getprop(elem, name) xmlGetProp((elem), (const xmlChar*)(name))
 
-static gboolean      module_register(void);
-static gint          anasys_detect  (const GwyFileDetectInfo *fileinfo,
-                                     gboolean only_name);
-static GwyContainer* anasys_load    (const gchar *filename,
-                                     GwyRunType mode,
-                                     GError **error);
-static guint32       readHeightMaps (GwyContainer *container,
-                                     xmlDoc *doc,
-                                     const xmlNode *curNode,
-                                     const gchar *filename,
-                                     GError **error);
-static gboolean      readSpectra    (GwyContainer *container,
-                                     xmlDoc *doc,
-                                     const xmlNode *curNode);
-
-const gdouble PI_over_180          = G_PI / 180.0;
+static gboolean module_register(void);
+static gint     detect_file    (const GwyFileDetectInfo *fileinfo,
+                                gboolean only_name);
+static GwyFile* load_file      (const gchar *filename,
+                                GwyRunModeFlags mode,
+                                GError **error);
+static guint32  readHeightMaps (GwyFile *file,
+                                xmlDoc *doc,
+                                const xmlNode *curNode,
+                                const gchar *filename,
+                                GError **error);
+static gboolean readSpectra    (GwyFile *file,
+                                xmlDoc *doc,
+                                const xmlNode *curNode,
+                                const gchar *filename);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Analysis Studio XML (.axz & .axd) files."),
     "Jeffrey J. Schwartz <schwartz@physics.ucla.edu>",
-    "0.8",
+    "0.9",
     "Jeffrey J. Schwartz",
-    "September 2018",
+    "2018",
 };
 
 GWY_MODULE_QUERY(module_info)
@@ -114,30 +111,26 @@ module_register(void)
 {
     gwy_file_func_register("anasys_xml",
                            N_("Analysis Studio XML (.axz, .axd)"),
-                           (GwyFileDetectFunc)&anasys_detect,
-                           (GwyFileLoadFunc)&anasys_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
     return TRUE;
 }
 
 static gint
-anasys_detect(const GwyFileDetectInfo *fileinfo,
-              gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     if (only_name) {
         return (g_str_has_suffix(fileinfo->name_lowercase, EXTENSION)
                 || g_str_has_suffix(fileinfo->name_lowercase, EXTENSION2)) ? 20 : 0;
     }
     /* AXD, plain text XML data files */
-    if (fileinfo->buffer_len > MIN_SIZE &&
-        g_str_has_suffix(fileinfo->name_lowercase, EXTENSION)) {
+    if (fileinfo->buffer_len > MIN_SIZE
+        && g_str_has_suffix(fileinfo->name_lowercase, EXTENSION)) {
         if (gwy_memmem(fileinfo->head+350, 100, MAGIC, MAGIC_SIZE) != NULL)
             return 100;
     }
     /* AXZ, gzip-compressed XML data files */
-    if (fileinfo->buffer_len > MIN_SIZE2 &&
-        g_str_has_suffix(fileinfo->name_lowercase, EXTENSION2)) {
+    if (fileinfo->buffer_len > MIN_SIZE2
+        && g_str_has_suffix(fileinfo->name_lowercase, EXTENSION2)) {
         if (gwy_memmem(fileinfo->head, 10, MAGIC2, MAGIC2_SIZE) != NULL) {
             return 50;
         }
@@ -145,20 +138,20 @@ anasys_detect(const GwyFileDetectInfo *fileinfo,
     return 0;
 }
 
-static GwyContainer*
-anasys_load(const gchar *filename,
-            G_GNUC_UNUSED GwyRunType mode, GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode, GError **error)
 {
     guint32 valid_images = 0;
-    GwyContainer *container = gwy_container_new();
-    xmlDoc *doc = xmlReadFile(filename, NULL, XML_PARSE_NOERROR);
+    GwyFile *file = gwy_file_new_in_construction();
+    xmlDoc *doc = xmlReadFile(filename, NULL, XML_PARSE_NOERROR | XML_PARSE_NOWARNING);
     xmlNode *curNode, *rootElement = xmlDocGetRootElement(doc);
     xmlChar *ptDocType = NULL;
     xmlChar *ptVersion = NULL;
 
     if (rootElement != NULL) {
-        if (rootElement->type == XML_ELEMENT_NODE &&
-            strequal(rootElement->name, "Document")) {
+        if (rootElement->type == XML_ELEMENT_NODE
+            && strequal(rootElement->name, "Document")) {
             ptDocType = getprop(rootElement, "DocType");
             ptVersion = getprop(rootElement, "Version");
             if (strequal(ptDocType, "IR") - strequal(ptVersion, "1.0")) {
@@ -173,24 +166,24 @@ anasys_load(const gchar *filename,
         if (curNode->type != XML_ELEMENT_NODE)
             continue;
         if (strequal(curNode->name, "HeightMaps"))
-            valid_images = readHeightMaps(container, doc, curNode, filename, error);
+            valid_images = readHeightMaps(file, doc, curNode, filename, error);
         else if (strequal(curNode->name, "RenderedSpectra")) {
-            if (!readSpectra(container, doc, curNode))
+            if (!readSpectra(file, doc, curNode, filename))
                 valid_images = 0;
         }
     }
     xmlFreeDoc(doc);
     xmlCleanupParser();
     if (valid_images == 0) {
-        g_object_unref(container);
+        g_object_unref(file);
         err_NO_DATA(error);
         return NULL;
     }
-    return container;
+    return file;
 }
 
 static guint32
-readHeightMaps(GwyContainer *container, xmlDoc *doc, const xmlNode *curNode,
+readHeightMaps(GwyFile *file, xmlDoc *doc, const xmlNode *curNode,
                const gchar *filename, GError **error)
 {
     guint32 imageNum = 0;
@@ -215,9 +208,9 @@ readHeightMaps(GwyContainer *container, xmlDoc *doc, const xmlNode *curNode,
     gchar **endptr;
     guchar *decodedData;
     guchar *base64DataString;
-    GwyDataField *dfield;
-    GwyDataField *dfield_rotate;
-    GwyDataField *dfield_temp;
+    GwyField *dfield;
+    GwyField *dfield_rotate;
+    GwyField *dfield_temp;
     GwyContainer *meta;
     xmlChar *key, *xmlPropValue1, *xmlPropValue2;
     xmlNode *childNode, *posNode, *sizeNode, *resNode, *subNode, *tempNode, *tagNode;
@@ -251,7 +244,7 @@ readHeightMaps(GwyContainer *container, xmlDoc *doc, const xmlNode *curNode,
         xmlPropValue2 = NULL;
 
         xmlPropValue1 = getprop(childNode, "DataChannel");
-        meta = gwy_container_new();
+        meta = gwy_container_new_in_construction();
         gwy_container_set_const_string_by_name(meta, "DataChannel", (const guchar*)xmlPropValue1);
         xmlFree(xmlPropValue1);
 
@@ -266,7 +259,7 @@ readHeightMaps(GwyContainer *container, xmlDoc *doc, const xmlNode *curNode,
                      posNode = posNode->next) {
                     if (posNode->type != XML_ELEMENT_NODE)
                         continue;
-                    key = xmlNodeListGetString(doc, posNode->xmlChildrenNode, 1);
+                    key = xmlNodeListGetString(doc, posNode->children, 1);
                     if (strequal(posNode->name, "X"))
                         pos_x = g_ascii_strtod((const gchar*)key, endptr);
                     else if (strequal(posNode->name, "Y"))
@@ -283,7 +276,7 @@ readHeightMaps(GwyContainer *container, xmlDoc *doc, const xmlNode *curNode,
                      sizeNode = sizeNode->next) {
                     if (sizeNode->type != XML_ELEMENT_NODE)
                         continue;
-                    key = xmlNodeListGetString(doc, sizeNode->xmlChildrenNode, 1);
+                    key = xmlNodeListGetString(doc, sizeNode->children, 1);
                     if (strequal(sizeNode->name, "X"))
                         range_x = g_ascii_strtod((const gchar*)key, endptr);
                     else if (strequal(sizeNode->name, "Y"))
@@ -300,7 +293,7 @@ readHeightMaps(GwyContainer *container, xmlDoc *doc, const xmlNode *curNode,
                      resNode = resNode->next) {
                     if (resNode->type != XML_ELEMENT_NODE)
                         continue;
-                    key = xmlNodeListGetString(doc, resNode->xmlChildrenNode, 1);
+                    key = xmlNodeListGetString(doc, resNode->children, 1);
                     if (strequal(resNode->name, "X"))
                         resolution_x = (gint32)atoi((char *)key);
                     else if (strequal(resNode->name, "Y"))
@@ -313,13 +306,13 @@ readHeightMaps(GwyContainer *container, xmlDoc *doc, const xmlNode *curNode,
             }
             else if (strequal(tempNode->name, "Units")) {
                 key = xmlNodeListGetString(doc,
-                                           tempNode->xmlChildrenNode, 1);
+                                           tempNode->children, 1);
                 zUnit = g_strdup((gchar*)key);
                 gwy_container_set_const_string_by_name(meta, "Units", (guchar *)zUnit);
                 xmlFree(key);
             }
             else if (strequal(tempNode->name, "UnitPrefix")) {
-                key = xmlNodeListGetString(doc, tempNode->xmlChildrenNode, 1);
+                key = xmlNodeListGetString(doc, tempNode->children, 1);
                 if (strequal(key, "f"))
                     zUnitMultiplier = 1.0e-15;
                 else if (strequal(key, "p"))
@@ -364,13 +357,13 @@ readHeightMaps(GwyContainer *container, xmlDoc *doc, const xmlNode *curNode,
                 }
             }
             else if (strequal(tempNode->name, "SampleBase64")) {
-                key = xmlNodeListGetString(doc, tempNode->xmlChildrenNode, 1);
+                key = xmlNodeListGetString(doc, tempNode->children, 1);
                 base64DataString = (guchar*)g_strdup((gchar*)key);
                 xmlFree(key);
             }
             else {
                 if (xmlChildElementCount(tempNode) == 0) {
-                    key = xmlNodeListGetString(doc, tempNode->xmlChildrenNode, 1);
+                    key = xmlNodeListGetString(doc, tempNode->children, 1);
                     gwy_container_set_const_string_by_name(meta, (const gchar*)tempNode->name, (guchar*)key);
                     xmlFree(key);
                 }
@@ -380,7 +373,7 @@ readHeightMaps(GwyContainer *container, xmlDoc *doc, const xmlNode *curNode,
                          subNode = subNode->next) {
                         if (subNode->type != XML_ELEMENT_NODE)
                             continue;
-                        key = xmlNodeListGetString(doc, subNode->xmlChildrenNode, 1);
+                        key = xmlNodeListGetString(doc, subNode->children, 1);
                         tempStr = g_strdup_printf("%s_%s", tempNode->name, subNode->name);
                         gwy_container_set_const_string_by_name(meta, tempStr, (guchar*)key);
                         g_free(tempStr);
@@ -404,12 +397,12 @@ readHeightMaps(GwyContainer *container, xmlDoc *doc, const xmlNode *curNode,
             continue;
         }
 
-        dfield = gwy_data_field_new(resolution_x, resolution_y,
-                                    range_x*1.0e-6, range_y*1.0e-6, FALSE);
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), zUnit);
+        dfield = gwy_field_new(resolution_x, resolution_y,
+                               range_x*1.0e-6, range_y*1.0e-6, FALSE);
+        gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
+        gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), zUnit);
 
-        data = gwy_data_field_get_data(dfield);
+        data = gwy_field_get_data(dfield);
 
         decodedData = g_base64_decode((const gchar*)base64DataString, &decoded_size);
         if (err_SIZE_MISMATCH(error, sizeof(gfloat)*num_px, decoded_size, TRUE)) {
@@ -423,33 +416,33 @@ readHeightMaps(GwyContainer *container, xmlDoc *doc, const xmlNode *curNode,
                              data, zUnitMultiplier, 0.0);
 
         if (scan_angle == 0.0) {
-            gwy_data_field_invert(dfield, TRUE, FALSE, FALSE);
+            gwy_field_flip(dfield, FALSE, TRUE);
             width = range_x;
             height = range_y;
         }
         else if (scan_angle == 180.0) {
-            gwy_data_field_invert(dfield, FALSE, TRUE, FALSE);
+            gwy_field_flip(dfield, TRUE, FALSE);
             width = range_x;
             height = range_y;
         }
         else if (scan_angle == 90.0) {
             dfield_temp = dfield;
-            dfield = gwy_data_field_new_rotated_90(dfield, FALSE);
+            dfield = gwy_field_new_rotated_90(dfield, FALSE);
             g_object_unref(dfield_temp);
-            gwy_data_field_invert(dfield, TRUE, FALSE, FALSE);
+            gwy_field_flip(dfield, FALSE, TRUE);
             width = range_y;
             height = range_x;
         }
         else if (scan_angle == -90.0) {
             dfield_temp = dfield;
-            dfield = gwy_data_field_new_rotated_90(dfield, TRUE);
+            dfield = gwy_field_new_rotated_90(dfield, TRUE);
             g_object_unref(dfield_temp);
-            gwy_data_field_invert(dfield, TRUE, FALSE, FALSE);
+            gwy_field_flip(dfield, FALSE, TRUE);
             width = range_y;
             height = range_x;
         }
         else {
-            const gdouble rot_angle = PI_over_180 * scan_angle;
+            const gdouble rot_angle = gwy_deg2rad(scan_angle);
             /* Estimate the number of pixels in the image rotation with GWY_ROTATE_RESIZE_EXPAND will produce. */
             gdouble casa = fabs(cos(rot_angle)*sin(rot_angle));
             gdouble Lx = range_x, Ly = range_y;
@@ -461,63 +454,60 @@ readHeightMaps(GwyContainer *container, xmlDoc *doc, const xmlNode *curNode,
             gdouble reduction = sqrt(2048*2048/estim_npixels);
 
             if (reduction < 1.0) {
-                GwyDataField *reduced_field;
+                GwyField *reduced_field;
                 gint reduced_xres = GWY_ROUND(reduction*resolution_x);
                 gint reduced_yres = GWY_ROUND(reduction*resolution_y);
                 reduced_xres = MAX(reduced_xres, 2);
                 reduced_yres = MAX(reduced_yres, 2);
-                reduced_field = gwy_data_field_new_resampled(dfield, reduced_xres, reduced_yres,
-                                                             GWY_INTERPOLATION_BSPLINE);
-                dfield_rotate = gwy_data_field_new_rotated(reduced_field, NULL, rot_angle,
-                                                           GWY_INTERPOLATION_BSPLINE, GWY_ROTATE_RESIZE_EXPAND);
+                reduced_field = gwy_field_new_resampled(dfield, reduced_xres, reduced_yres,
+                                                        GWY_INTERPOLATION_BSPLINE);
+                dfield_rotate = gwy_field_new_rotated(reduced_field, NULL, rot_angle,
+                                                      GWY_INTERPOLATION_BSPLINE, GWY_ROTATE_RESIZE_EXPAND);
                 g_object_unref(reduced_field);
             }
             else {
-                dfield_rotate = gwy_data_field_new_rotated(dfield, NULL, rot_angle,
-                                                           GWY_INTERPOLATION_BSPLINE, GWY_ROTATE_RESIZE_EXPAND);
+                dfield_rotate = gwy_field_new_rotated(dfield, NULL, rot_angle,
+                                                      GWY_INTERPOLATION_BSPLINE, GWY_ROTATE_RESIZE_EXPAND);
             }
-            gwy_data_field_invert(dfield_rotate, TRUE, FALSE, FALSE);
-            width = gwy_data_field_get_xreal(dfield_rotate);
-            height = gwy_data_field_get_yreal(dfield_rotate);
+            gwy_field_flip(dfield_rotate, FALSE, TRUE);
+            width = gwy_field_get_xreal(dfield_rotate);
+            height = gwy_field_get_yreal(dfield_rotate);
             oblique_angle = TRUE;
         }
 
         if (oblique_angle) {
-            gwy_data_field_set_xoffset(dfield, 1.0);
-            gwy_data_field_set_yoffset(dfield, 1.0);
-            gwy_data_field_set_xoffset(dfield_rotate, pos_x*1.0e-6 - 0.5*width);
-            gwy_data_field_set_yoffset(dfield_rotate, pos_y*1.0e-6 - 0.5*height);
+            gwy_field_set_xoffset(dfield, 1.0);
+            gwy_field_set_yoffset(dfield, 1.0);
+            gwy_field_set_xoffset(dfield_rotate, pos_x*1.0e-6 - 0.5*width);
+            gwy_field_set_yoffset(dfield_rotate, pos_y*1.0e-6 - 0.5*height);
         }
         else {
-            gwy_data_field_set_xoffset(dfield, (pos_x - 0.5*width)*1.0e-6);
-            gwy_data_field_set_yoffset(dfield, (pos_y - 0.5*height)*1.0e-6);
+            gwy_field_set_xoffset(dfield, (pos_x - 0.5*width)*1.0e-6);
+            gwy_field_set_yoffset(dfield, (pos_y - 0.5*height)*1.0e-6);
         }
 
-        gwy_container_pass_object(container, gwy_app_get_data_key_for_id(imageNum), dfield);
-        gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(imageNum), meta);
+        gwy_file_pass_image(file, imageNum, dfield);
+        gwy_file_pass_meta(file, GWY_FILE_IMAGE, imageNum, meta);
 
         if (oblique_angle) {
-            gwy_container_pass_object(container, gwy_app_get_data_key_for_id(100000 + imageNum), dfield_rotate);
-            gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(100000 + imageNum),
-                                      gwy_container_duplicate(meta));
+            gwy_file_pass_image(file, 100000 + imageNum, dfield_rotate);
+            gwy_file_pass_meta(file, GWY_FILE_IMAGE, 100000 + imageNum, gwy_container_copy(meta));
             xmlPropValue1 = getprop(childNode, "Label");
-            gwy_container_set_string(container, gwy_app_get_data_title_key_for_id(100000 + imageNum),
-                                     g_strdup_printf("%s (Rotated)", xmlPropValue1));
+            gwy_file_pass_title(file, GWY_FILE_IMAGE, 100000 + imageNum,
+                                g_strdup_printf("%s (Rotated)", xmlPropValue1));
             xmlFree(xmlPropValue1);
             xmlPropValue1 = getprop(childNode, "Label");
-            gwy_container_set_string(container, gwy_app_get_data_title_key_for_id(imageNum),
-                                     g_strdup_printf("%s (Offset)", xmlPropValue1));
+            gwy_file_pass_title(file, GWY_FILE_IMAGE, imageNum, g_strdup_printf("%s (Offset)", xmlPropValue1));
             xmlFree(xmlPropValue1);
         }
         else {
             xmlPropValue1 = getprop(childNode, "Label");
-            gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(imageNum),
-                                           (const guchar*)xmlPropValue1);
+            gwy_file_set_title(file, GWY_FILE_IMAGE, imageNum, (const guchar*)xmlPropValue1, FALSE);
             xmlFree(xmlPropValue1);
         }
-        gwy_app_channel_check_nonsquare(container, imageNum);
-        gwy_file_channel_import_log_add(container, imageNum, NULL, filename);
-        ++valid_images;
+        gwy_check_nonsquare_image(file, imageNum);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, imageNum, NULL, filename);
+        valid_images++;
 
         g_free(zUnit);
         g_free(decodedData);
@@ -528,8 +518,9 @@ readHeightMaps(GwyContainer *container, xmlDoc *doc, const xmlNode *curNode,
 }
 
 static gboolean
-readSpectra(GwyContainer *container, xmlDoc *doc,
-            const xmlNode *curNode)
+readSpectra(GwyFile *file, xmlDoc *doc,
+            const xmlNode *curNode,
+            const gchar *filename)
 {
     guint32 specID = 0;
 
@@ -553,12 +544,12 @@ readSpectra(GwyContainer *container, xmlDoc *doc,
     gchar *channelName = NULL;
     gchar **endptr;
     gdouble *ydata;
-    GwyDataLine *dataline;
-    GwyDataLine *copy_dataline;
+    GwyLine *line;
+    GwyLine *copy_line;
     GwySpectra *spectra;
 
     GwySpectra *spectra_all = gwy_spectra_new();
-    gwy_si_unit_set_from_string(gwy_spectra_get_si_unit_xy(spectra_all), "m");
+    gwy_unit_set_from_string(gwy_spectra_get_unit_xy(spectra_all), "m");
     gwy_spectra_set_spectrum_x_label(spectra_all,
                                      "Wavenumber (cm<sup>-1</sup>)");
     gwy_spectra_set_title(spectra_all, "All Spectra (Polarization): DataChannel");
@@ -583,27 +574,27 @@ readSpectra(GwyContainer *container, xmlDoc *doc,
             if (subNode->type != XML_ELEMENT_NODE)
                 continue;
             if (strequal(subNode->name, "Label")) {
-                key = xmlNodeListGetString(doc, subNode->xmlChildrenNode, 1);
+                key = xmlNodeListGetString(doc, subNode->children, 1);
                 label = g_strdup((gchar*)key);
                 xmlFree(key);
             }
             else if (strequal(subNode->name, "DataPoints")) {
-                key = xmlNodeListGetString(doc, subNode->xmlChildrenNode, 1);
+                key = xmlNodeListGetString(doc, subNode->children, 1);
                 numDataPoints = (guint32)atoi((char*)key);
                 xmlFree(key);
             }
             else if (strequal(subNode->name, "StartWavenumber")) {
-                key = xmlNodeListGetString(doc, subNode->xmlChildrenNode, 1);
+                key = xmlNodeListGetString(doc, subNode->children, 1);
                 startWavenum = g_ascii_strtod((const gchar*)key, endptr);
                 xmlFree(key);
             }
             else if (strequal(subNode->name, "EndWavenumber")) {
-                key = xmlNodeListGetString(doc, subNode->xmlChildrenNode, 1);
+                key = xmlNodeListGetString(doc, subNode->children, 1);
                 endWavenum = g_ascii_strtod((const gchar*)key, endptr);
                 xmlFree(key);
             }
             else if (strequal(subNode->name, "Polarization")) {
-                key = xmlNodeListGetString(doc, subNode->xmlChildrenNode, 1);
+                key = xmlNodeListGetString(doc, subNode->children, 1);
                 polarization = g_strdup((gchar*)key);
                 xmlFree(key);
             }
@@ -611,7 +602,7 @@ readSpectra(GwyContainer *container, xmlDoc *doc,
                 for (locNode = subNode->children; locNode; locNode = locNode->next) {
                     if (locNode->type != XML_ELEMENT_NODE)
                         continue;
-                    key = xmlNodeListGetString(doc, locNode->xmlChildrenNode, 1);
+                    key = xmlNodeListGetString(doc, locNode->children, 1);
                     if (strequal(locNode->name, "X"))
                         location_x = g_ascii_strtod((const gchar*)key, endptr);
                     else if (strequal(locNode->name, "Y"))
@@ -623,7 +614,7 @@ readSpectra(GwyContainer *container, xmlDoc *doc,
                 ++specID;
                 base64SpecString = NULL;
                 spectra = gwy_spectra_new();
-                gwy_si_unit_set_from_string(gwy_spectra_get_si_unit_xy(spectra), "m");
+                gwy_unit_set_from_string(gwy_spectra_get_unit_xy(spectra), "m");
                 gwy_spectra_set_spectrum_x_label(spectra, "Wavenumber (cm<sup>-1</sup>)");
 
                 xmlPropValue1 = getprop(subNode, "DataChannel");
@@ -636,7 +627,7 @@ readSpectra(GwyContainer *container, xmlDoc *doc,
                     if (dcNode->type != XML_ELEMENT_NODE)
                         continue;
                     if (strequal(dcNode->name, "SampleBase64")) {
-                        key = xmlNodeListGetString(doc, dcNode->xmlChildrenNode, 1);
+                        key = xmlNodeListGetString(doc, dcNode->children, 1);
                         base64SpecString = (guchar*)g_strdup((gchar*)key);
                         xmlFree(key);
                         break;
@@ -665,30 +656,33 @@ readSpectra(GwyContainer *container, xmlDoc *doc,
                     g_free(base64SpecString);
                     continue;
                 }
-                dataline = gwy_data_line_new(numDataPoints,
-                                             (endWavenum-startWavenum)*(1.0+(1.0/((gdouble)numDataPoints-1.0))),
-                                             TRUE);
-                gwy_data_line_set_offset(dataline, startWavenum);
-                ydata = gwy_data_line_get_data(dataline);
+                line = gwy_line_new(numDataPoints,
+                                    (endWavenum-startWavenum)*(1.0+(1.0/((gdouble)numDataPoints-1.0))),
+                                    TRUE);
+                gwy_line_set_offset(line, startWavenum);
+                ydata = gwy_line_get_data(line);
                 gwy_convert_raw_data(decodedData, numDataPoints, 1, GWY_RAW_DATA_FLOAT, GWY_BYTE_ORDER_LITTLE_ENDIAN,
                                      ydata, 1.0, 0.0);
                 g_free(decodedData);
 
-                copy_dataline = gwy_data_line_duplicate(dataline);
-                gwy_spectra_add_spectrum(spectra, dataline, location_x*1.0e-6, location_y*1.0e-6);
-                gwy_spectra_add_spectrum(spectra_all, copy_dataline, location_x*1.0e-6, location_y*1.0e-6);
+                copy_line = gwy_line_copy(line);
+                gwy_spectra_add_spectrum(spectra, line, location_x*1.0e-6, location_y*1.0e-6);
+                gwy_spectra_add_spectrum(spectra_all, copy_line, location_x*1.0e-6, location_y*1.0e-6);
 
-                gwy_container_pass_object(container, gwy_app_get_spectra_key_for_id(specID), spectra);
-                g_object_unref(dataline);
-                g_object_unref(copy_dataline);
+                gwy_file_pass_spectra(file, specID, spectra);
+                gwy_log_add_import(file, GWY_FILE_SPECTRA, specID, NULL, filename);
+                g_object_unref(line);
+                g_object_unref(copy_line);
                 g_free(base64SpecString);
             }
         }
         g_free(label);
         g_free(polarization);
     }
-    if (specID > 0)
-        gwy_container_set_object_by_name(container, "/sps/0", spectra_all);
+    if (specID > 0) {
+        gwy_file_set_spectra(file, 0, spectra_all);
+        gwy_log_add_import(file, GWY_FILE_SPECTRA, 0, NULL, filename);
+    }
 
     g_object_unref(spectra_all);
 
