@@ -1,6 +1,6 @@
 /*
- *  $Id: andorsif.c 26873 2024-11-19 12:52:14Z yeti-dn $
- *  Copyright (C) 2023 David Necas (Yeti).
+ *  $Id: andorsif.c 28789 2025-11-04 17:14:03Z yeti-dn $
+ *  Copyright (C) 2023-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -15,6 +15,8 @@
  *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
+/* XXX: A Python module reading these files exists. So maybe there is a chance… */
+
 /**
  * [FILE-MAGIC-USERGUIDE]
  * Anfor SIF
@@ -37,16 +39,11 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/stats.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -283,41 +280,41 @@ typedef struct {
     AndorSIFImage image;
 } AndorSIFChannel;
 
-static gboolean      module_register           (void);
-static gint          sif_detect                (const GwyFileDetectInfo *fileinfo,
-                                                gboolean only_name);
-static GwyContainer* sif_load                  (const gchar *filename,
-                                                GwyRunType mode,
-                                                GError **error);
-static gchar*        sif_read_insta_image      (AndorSIFInstaImage *image,
-                                                gchar *p);
-static gchar*        sif_read_user_text        (AndorSIFUserText *user_text,
-                                                gchar *p);
-static gchar*        sif_read_shutter          (AndorSIFShutter *shutter,
-                                                gchar *p);
-static gchar*        sif_read_shamrock_save    (AndorSIFShamrockSave *save,
-                                                gchar *p);
-static gchar*        sif_read_spectrograph_save(AndorSIFSpectrographSave *save,
-                                                gchar *p);
-static gchar*        sif_read_unknown_record1  (AndorSIFUnknownRecord1 *unknown,
-                                                gchar *p);
-static gchar*        sif_read_unknown_record2  (AndorSIFUnknownRecord2 *unknown,
-                                                gchar *p);
-static gchar*        sif_read_calib_image      (AndorSIFCalibImage *image,
-                                                gchar *p);
-static gchar*        sif_read_image            (AndorSIFImage *image,
-                                                gchar *p);
-static void          sif_channel_free          (AndorSIFChannel *datasource);
-static gboolean      version_at_least          (gint version,
-                                                gint major,
-                                                gint minor);
+static gboolean module_register           (void);
+static gint     detect_file               (const GwyFileDetectInfo *fileinfo,
+                                           gboolean only_name);
+static GwyFile* load_file                 (const gchar *filename,
+                                           GwyRunModeFlags mode,
+                                           GError **error);
+static gchar*   sif_read_insta_image      (AndorSIFInstaImage *image,
+                                           gchar *p);
+static gchar*   sif_read_user_text        (AndorSIFUserText *user_text,
+                                           gchar *p);
+static gchar*   sif_read_shutter          (AndorSIFShutter *shutter,
+                                           gchar *p);
+static gchar*   sif_read_shamrock_save    (AndorSIFShamrockSave *save,
+                                           gchar *p);
+static gchar*   sif_read_spectrograph_save(AndorSIFSpectrographSave *save,
+                                           gchar *p);
+static gchar*   sif_read_unknown_record1  (AndorSIFUnknownRecord1 *unknown,
+                                           gchar *p);
+static gchar*   sif_read_unknown_record2  (AndorSIFUnknownRecord2 *unknown,
+                                           gchar *p);
+static gchar*   sif_read_calib_image      (AndorSIFCalibImage *image,
+                                           gchar *p);
+static gchar*   sif_read_image            (AndorSIFImage *image,
+                                           gchar *p);
+static void     sif_channel_free          (AndorSIFChannel *datasource);
+static gboolean version_at_least          (gint version,
+                                           gint major,
+                                           gint minor);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Andor SIF data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.2",
+    "0.3",
     "David Nečas (Yeti)",
     "2023",
 };
@@ -329,18 +326,14 @@ module_register(void)
 {
     gwy_file_func_register("andorsif",
                            N_("Andor SIF files (.sif)"),
-                           (GwyFileDetectFunc)&sif_detect,
-                           (GwyFileLoadFunc)&sif_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
     gwy_file_func_set_is_unfinished("andorsif", TRUE);
 
     return TRUE;
 }
 
 static gint
-sif_detect(const GwyFileDetectInfo *fileinfo,
-           gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     const gchar *p;
 
@@ -361,19 +354,19 @@ sif_detect(const GwyFileDetectInfo *fileinfo,
     return 95;
 }
 
-static GwyContainer*
-sif_load(const gchar *filename,
-         G_GNUC_UNUSED GwyRunType mode,
-         GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
     static const gchar *channels[] = {
         "Signal", "Reference", "Background", "Live", "Source",
     };
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     AndorSIFChannel datasource;
     gchar *p, *buffer = NULL;
     gsize size = 0, expected_size, available_size;
-    GwyDataField *field;
+    GwyField *field;
     GError *err = NULL;
     gboolean is_present, ok = FALSE;
     guint cno;
@@ -385,7 +378,7 @@ sif_load(const gchar *filename,
         return NULL;
     }
 
-    gwy_clear(&datasource, 1);
+    gwy_clear1(datasource);
     if (!(p = strchr(buffer, '\n'))) {
         err_FILE_TYPE(error, "Andor SIF");
         goto end;
@@ -399,7 +392,7 @@ sif_load(const gchar *filename,
     }
     p += 6;
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     for (cno = 0; cno < G_N_ELEMENTS(channels); cno++) {
         if (p - buffer > size-2 || (p[0] != '1' && p[0] != '0') || p[1] != '\n') {
             err_FILE_TYPE(error, "Andor SIF");
@@ -477,11 +470,11 @@ sif_load(const gchar *filename,
             if (err_SIZE_MISMATCH(error, expected_size, available_size, FALSE))
                 goto end;
 
-            field = gwy_data_field_new(xres, yres, xres, yres, FALSE);
+            field = gwy_field_new(xres, yres, xres, yres, FALSE);
             gwy_convert_raw_data((guchar*)p, xres*yres, 1, GWY_RAW_DATA_FLOAT, GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                                 gwy_data_field_get_data(field), 1.0, 0.0);
-            gwy_file_channel_import_log_add(container, nimages, NULL, filename);
-            gwy_container_pass_object(container, gwy_app_get_data_key_for_id(nimages), field);
+                                 gwy_field_get_data(field), 1.0, 0.0);
+            gwy_log_add_import(file, GWY_FILE_IMAGE, nimages, NULL, filename);
+            gwy_file_pass_image(file, nimages, field);
 
             nimages++;
             p += expected_size;
@@ -497,11 +490,11 @@ sif_load(const gchar *filename,
 
 end:
     if (!ok)
-        GWY_OBJECT_UNREF(container);
+        g_clear_object(&file);
     sif_channel_free(&datasource);
     g_free(buffer);
 
-    return container;
+    return file;
 }
 
 static inline gchar*
