@@ -1,6 +1,6 @@
 /*
- *  $Id: anfatec.c 26135 2024-01-15 12:54:27Z yeti-dn $
- *  Copyright (C) 2010-2022 David Necas (Yeti)
+ *  $Id: anfatec.c 28911 2025-11-24 18:27:42Z yeti-dn $
+ *  Copyright (C) 2010-2025 David Necas (Yeti)
  *  E-mail: yeti@gwyddion.net
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -42,15 +42,10 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
 #include <string.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <libprocess/datafield.h>
-#include <libprocess/spectra.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -69,44 +64,44 @@ typedef struct {
     GwyBrick *brick;
 } AnfatecFVMatrix;
 
-static gboolean         module_register           (void);
-static gint             anfatec_detect            (const GwyFileDetectInfo *fileinfo,
-                                                   gboolean only_name);
-static gchar*           anfatec_find_parameterfile(const gchar *filename);
-static GwyContainer*    anfatec_load              (const gchar *filename,
-                                                   GwyRunType mode,
-                                                   GError **error);
-static GwyDataField*    anfatec_load_image        (GHashTable *hash,
-                                                   gint id,
-                                                   const gchar *dirname);
-static GwyLawn*         anfatec_load_curvemap     (GHashTable *hash,
-                                                   GHashTable *extra_meta,
-                                                   gint id,
-                                                   const gchar *dirname);
-static gboolean         anfatec_try_to_find_data  (const gchar *dirname_glib,
-                                                   const gchar *basename_sys,
-                                                   gchar **buffer,
-                                                   gsize *size);
-static AnfatecFVMatrix* read_fv_matrix_file       (gchar *buffer,
-                                                   gint header_cols,
-                                                   gint header_rows,
-                                                   GError **error);
-static void             anfatec_fv_matrix_free    (AnfatecFVMatrix *fvm);
-static gdouble          lawn_reduce_avg           (gint ncurves,
-                                                   gint curvelength,
-                                                   const gdouble *curvedata,
-                                                   gpointer user_data);
-static GwyContainer*    get_meta                  (GHashTable *hash);
-static void             add_meta                  (gpointer hkey,
-                                                   gpointer hvalue,
-                                                   gpointer user_data);
+static gboolean         module_register    (void);
+static gint             detect_file        (const GwyFileDetectInfo *fileinfo,
+                                            gboolean only_name);
+static GwyFile*         load_file          (const gchar *filename,
+                                            GwyRunModeFlags mode,
+                                            GError **error);
+static gchar*           find_parameterfile (const gchar *filename);
+static GwyField*        load_image         (GHashTable *hash,
+                                            gint id,
+                                            const gchar *dirname);
+static GwyLawn*         load_curvemap      (GHashTable *hash,
+                                            GHashTable *extra_meta,
+                                            gint id,
+                                            const gchar *dirname);
+static gboolean         try_to_find_data   (const gchar *dirname_glib,
+                                            const gchar *basename_sys,
+                                            gchar **buffer,
+                                            gsize *size);
+static AnfatecFVMatrix* read_fv_matrix_file(gchar *buffer,
+                                            gint header_cols,
+                                            gint header_rows,
+                                            GError **error);
+static void             fv_matrix_free     (AnfatecFVMatrix *fvm);
+static gdouble          lawn_reduce_avg    (gint ncurves,
+                                            gint curvelength,
+                                            const gdouble *curvedata,
+                                            gpointer user_data);
+static GwyContainer*    get_meta           (GHashTable *hash);
+static void             add_meta           (gpointer hkey,
+                                            gpointer hvalue,
+                                            gpointer user_data);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Anfatec data files (two-part .txt + .int)."),
     "Yeti <yeti@gwyddion.net>",
-    "0.6",
+    "0.7",
     "David Nečas (Yeti)",
     "2010",
 };
@@ -118,17 +113,13 @@ module_register(void)
 {
     gwy_file_func_register("anfatec",
                            N_("Anfatec files (.par + .int)"),
-                           (GwyFileDetectFunc)&anfatec_detect,
-                           (GwyFileLoadFunc)&anfatec_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-anfatec_detect(const GwyFileDetectInfo *fileinfo,
-               gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     FILE *fh;
     gchar *parameterfile;
@@ -142,7 +133,7 @@ anfatec_detect(const GwyFileDetectInfo *fileinfo,
     if (strstr(fileinfo->head, MAGIC))
         return 90;
 
-    if (!(parameterfile = anfatec_find_parameterfile(fileinfo->name)))
+    if (!(parameterfile = find_parameterfile(fileinfo->name)))
         return 0;
 
     fh = gwy_fopen(parameterfile, "r");
@@ -165,7 +156,7 @@ anfatec_detect(const GwyFileDetectInfo *fileinfo,
 }
 
 static gchar*
-anfatec_find_parameterfile(const gchar *filename)
+find_parameterfile(const gchar *filename)
 {
     gchar *paramfile;
     /* 4 is the length of .int, we start with removal of that. */
@@ -215,12 +206,13 @@ anfatec_find_parameterfile(const gchar *filename)
     return NULL;
 }
 
-static GwyContainer*
-anfatec_load(const gchar *filename,
-             GwyRunType mode,
-             GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *container = NULL, *meta = NULL;
+    GwyFile *file = NULL;
+    GwyContainer *meta = NULL;
     GHashTable *hash = NULL, *hash2 = NULL;
     gchar *line, *value, *key, *dirname = NULL, *text = NULL, *p;
     GError *err = NULL;
@@ -233,12 +225,12 @@ anfatec_load(const gchar *filename,
     }
 
     if (!gwy_memmem(text, MIN(size, GWY_FILE_DETECT_BUFFER_SIZE), MAGIC, MAGIC_SIZE)) {
-        gchar *paramfile = anfatec_find_parameterfile(filename);
+        gchar *paramfile = find_parameterfile(filename);
 
         /* If we are given data but find a suitable parameter file, recurse. */
         if (paramfile) {
             if (!gwy_strequal(paramfile, filename))
-                container = anfatec_load(paramfile, mode, error);
+                file = load_file(paramfile, mode, error);
             else {
                 g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_IO,
                             _("The parameter file cannot be loaded."));
@@ -250,7 +242,7 @@ anfatec_load(const gchar *filename,
                         _("Cannot find the corresponding parameter file."));
         }
         g_free(text);
-        return container;
+        return file;
     }
 
     /* Cannot use GwyTextHeaderParser due to unlabelled sections. */
@@ -315,12 +307,12 @@ anfatec_load(const gchar *filename,
     if (!require_keys(hash, error, "xPixel", "yPixel", "XScanRange", "YScanRange", NULL))
         goto fail;
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     meta = get_meta(hash);
     dirname = g_path_get_dirname(filename);
     maxid = id;
     for (id = 0; id <= maxid; id++) {
-        GwyDataField *dfield;
+        GwyField *dfield;
         GwyLawn *lawn;
         GwyContainer *metacopy;
         const gchar *dataname, *title;
@@ -336,50 +328,49 @@ anfatec_load(const gchar *filename,
 
         g_hash_table_remove_all(hash2);
 
-        if ((lawn = anfatec_load_curvemap(hash, hash2, id, dirname))) {
+        if ((lawn = load_curvemap(hash, hash2, id, dirname))) {
             ncurves = gwy_lawn_get_n_curves(lawn);
-            gwy_container_pass_object(container, gwy_app_get_lawn_key_for_id(id), lawn);
+            gwy_file_pass_cmap(file, id, lawn);
 
             if (meta) {
-                metacopy = gwy_container_duplicate(meta);
+                metacopy = gwy_container_copy(meta);
                 g_hash_table_foreach(hash2, add_meta, metacopy);
-                gwy_container_pass_object(container, gwy_app_get_lawn_meta_key_for_id(id), metacopy);
+                gwy_file_pass_meta(file, GWY_FILE_CMAP, id, metacopy);
             }
             if (title) {
-                gwy_container_set_const_string(container, gwy_app_get_lawn_title_key_for_id(id), title);
+                gwy_file_set_title(file, GWY_FILE_CMAP, id, title, FALSE);
                 gwy_lawn_set_curve_label(lawn, ncurves-1, title);
             }
 
-            dfield = gwy_data_field_new(gwy_lawn_get_xres(lawn), gwy_lawn_get_yres(lawn),
-                                        gwy_lawn_get_xreal(lawn), gwy_lawn_get_yreal(lawn),
-                                        FALSE);
+            dfield = gwy_field_new(gwy_lawn_get_xres(lawn), gwy_lawn_get_yres(lawn),
+                                   gwy_lawn_get_xreal(lawn), gwy_lawn_get_yreal(lawn),
+                                   FALSE);
             gwy_lawn_reduce_to_plane(lawn, dfield, lawn_reduce_avg, NULL);
-            gwy_si_unit_assign(gwy_data_field_get_si_unit_z(dfield), gwy_lawn_get_si_unit_curve(lawn, ncurves-1));
-            gwy_container_pass_object(container, gwy_app_get_lawn_preview_key_for_id(id), dfield);
+            gwy_unit_assign(gwy_field_get_unit_z(dfield), gwy_lawn_get_unit_curve(lawn, ncurves-1));
+            gwy_container_pass_object(GWY_CONTAINER(file), gwy_file_key_cmap_picture(id), dfield);
 
-            gwy_file_curve_map_import_log_add(container, id, NULL, dataname);
+            gwy_log_add_import(file, GWY_FILE_CMAP, id, NULL, dataname);
         }
-        else if ((dfield = anfatec_load_image(hash, id, dirname))) {
-            gwy_container_pass_object(container, gwy_app_get_data_key_for_id(id), dfield);
-            if (meta) {
-                gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(id),
-                                          gwy_container_duplicate(meta));
-            }
-            if (title)
-                gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(id), title);
-            else
-                gwy_app_channel_title_fall_back(container, id);
+        else if ((dfield = load_image(hash, id, dirname))) {
+            gwy_file_pass_image(file, id, dfield);
+            if (meta)
+                gwy_file_pass_meta(file, GWY_FILE_IMAGE, id, gwy_container_copy(meta));
 
-            gwy_file_channel_import_log_add(container, id, NULL, dataname);
+            if (title)
+                gwy_file_set_title(file, GWY_FILE_IMAGE, id, title, FALSE);
+            else
+                gwy_image_title_fall_back(file, id);
+
+            gwy_log_add_import(file, GWY_FILE_IMAGE, id, NULL, dataname);
         }
     }
 
-    if (!gwy_container_get_n_items(container)) {
-        GWY_OBJECT_UNREF(container);
+    if (!gwy_container_get_n_items(GWY_CONTAINER(file))) {
+        g_clear_object(&file);
         err_NO_DATA(error);
     }
 fail:
-    GWY_OBJECT_UNREF(meta);
+    g_clear_object(&meta);
     g_free(dirname);
     g_free(text);
     if (hash)
@@ -387,16 +378,16 @@ fail:
     if (hash2)
         g_hash_table_destroy(hash2);
 
-    return container;
+    return file;
 }
 
-static GwyDataField*
-anfatec_load_image(GHashTable *hash,
-                   gint id,
-                   const gchar *dirname)
+static GwyField*
+load_image(GHashTable *hash,
+           gint id,
+           const gchar *dirname)
 {
-    GwyDataField *dfield = NULL;
-    GwySIUnit *unitx = NULL, *unity = NULL, *unitz = NULL;
+    GwyField *dfield = NULL;
+    GwyUnit *unitx = NULL, *unity = NULL, *unitz = NULL;
     const gchar *filename;
     gint xres, yres, power10x, power10y, power10z;
     gdouble xreal, yreal, q, offset;
@@ -428,7 +419,7 @@ anfatec_load_image(GHashTable *hash,
     }
     gwy_debug("filename: %s", filename);
 
-    if (!anfatec_try_to_find_data(dirname, filename, &buffer, &size)) {
+    if (!try_to_find_data(dirname, filename, &buffer, &size)) {
         g_warning("Cannot open %s.", filename);
         goto fail;
     }
@@ -442,50 +433,50 @@ anfatec_load_image(GHashTable *hash,
     yreal = g_ascii_strtod(g_hash_table_lookup(hash, "YScanRange"), NULL);
     sanitise_real_size(&yreal, "y size");
 
-    unitx = gwy_si_unit_new_parse(g_hash_table_lookup(hash, "XPhysUnit"), &power10x);
-    unity = gwy_si_unit_new_parse(g_hash_table_lookup(hash, "YPhysUnit"), &power10y);
-    if (!gwy_si_unit_equal(unitx, unity))
+    unitx = gwy_unit_new_parse(g_hash_table_lookup(hash, "XPhysUnit"), &power10x);
+    unity = gwy_unit_new_parse(g_hash_table_lookup(hash, "YPhysUnit"), &power10y);
+    if (!gwy_unit_equal(unitx, unity))
         g_warning("X and Y units differ, using X");
 
     key = g_strdup_printf("%d::PhysUnit", id);
-    unitz = gwy_si_unit_new_parse(g_hash_table_lookup(hash, key), &power10z);
+    unitz = gwy_unit_new_parse(g_hash_table_lookup(hash, key), &power10z);
     g_free(key);
 
-    dfield = gwy_data_field_new(xres, yres, xreal*pow10(power10x), yreal*pow10(power10y), FALSE);
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(dfield), unitx);
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_z(dfield), unitz);
+    dfield = gwy_field_new(xres, yres, xreal*gwy_exp10(power10x), yreal*gwy_exp10(power10y), FALSE);
+    gwy_unit_assign(gwy_field_get_unit_xy(dfield), unitx);
+    gwy_unit_assign(gwy_field_get_unit_z(dfield), unitz);
     if ((value = g_hash_table_lookup(hash, "xCenter"))) {
-        offset = (g_ascii_strtod(value, NULL) - 0.5*xreal)*pow10(power10x);
-        gwy_data_field_set_xoffset(dfield, offset);
+        offset = (g_ascii_strtod(value, NULL) - 0.5*xreal)*gwy_exp10(power10x);
+        gwy_field_set_xoffset(dfield, offset);
     }
     if ((value = g_hash_table_lookup(hash, "yCenter"))) {
-        offset = (g_ascii_strtod(value, NULL) - 0.5*yreal)*pow10(power10y);
-        gwy_data_field_set_yoffset(dfield, offset);
+        offset = (g_ascii_strtod(value, NULL) - 0.5*yreal)*gwy_exp10(power10y);
+        gwy_field_set_yoffset(dfield, offset);
     }
 
-    q = pow10(power10z);
+    q = gwy_exp10(power10z);
     key = g_strdup_printf("%d::Scale", id);
     if ((value = g_hash_table_lookup(hash, key)))
         q *= g_ascii_strtod(value, NULL);
     g_free(key);
 
     gwy_convert_raw_data(buffer, xres*yres, 1, GWY_RAW_DATA_SINT32, GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                         gwy_data_field_get_data(dfield), q, 0.0);
+                         gwy_field_get_data(dfield), q, 0.0);
 
 fail:
     g_free(buffer);
-    GWY_OBJECT_UNREF(unitx);
-    GWY_OBJECT_UNREF(unity);
-    GWY_OBJECT_UNREF(unitz);
+    g_clear_object(&unitx);
+    g_clear_object(&unity);
+    g_clear_object(&unitz);
 
     return dfield;
 }
 
 static GwyLawn*
-anfatec_load_curvemap(GHashTable *hash,
-                      GHashTable *extra_meta,
-                      gint id,
-                      const gchar *dirname)
+load_curvemap(GHashTable *hash,
+              GHashTable *extra_meta,
+              gint id,
+              const gchar *dirname)
 {
     AnfatecFVMatrix *fvm = NULL;
     GwyLawn *lawn = NULL;
@@ -516,7 +507,7 @@ anfatec_load_curvemap(GHashTable *hash,
     header_cols = atoi(value);
     gwy_debug("HeaderCols: %d", header_cols);
 
-    if (!anfatec_try_to_find_data(dirname, filename, &buffer, &size)) {
+    if (!try_to_find_data(dirname, filename, &buffer, &size)) {
         g_warning("Cannot open %s.", filename);
         goto fail;
     }
@@ -547,15 +538,15 @@ anfatec_load_curvemap(GHashTable *hash,
     lawn = gwy_lawn_new(xres, yres, gwy_brick_get_xreal(fvm->brick), gwy_brick_get_yreal(fvm->brick), ncurves+1, 0);
     gwy_lawn_set_xoffset(lawn, gwy_brick_get_xoffset(fvm->brick));
     gwy_lawn_set_yoffset(lawn, gwy_brick_get_yoffset(fvm->brick));
-    gwy_si_unit_set_from_string(gwy_lawn_get_si_unit_xy(lawn), "m");
+    gwy_unit_set_from_string(gwy_lawn_get_unit_xy(lawn), "m");
     b = gwy_brick_get_data(fvm->brick);
 
     q = g_new(gdouble, ntotalcurves);
     for (i = 0; i < ntotalcurves; i++) {
         if ((j = abscissa_map[i]) >= 0) {
             gwy_lawn_set_curve_label(lawn, j, fvm->absnames[i]);
-            gwy_si_unit_set_from_string_parse(gwy_lawn_get_si_unit_curve(lawn, j), fvm->absunits[i], &k);
-            q[i] = pow10(k);
+            k = gwy_unit_set_from_string(gwy_lawn_get_unit_curve(lawn, j), fvm->absunits[i]);
+            q[i] = gwy_exp10(k);
         }
     }
 
@@ -576,8 +567,8 @@ anfatec_load_curvemap(GHashTable *hash,
         g_free(name);
 
         if (unit) {
-            gwy_si_unit_set_from_string_parse(gwy_lawn_get_si_unit_curve(lawn, ncurves), unit, &k);
-            qdata = pow10(k);
+            k = gwy_unit_set_from_string(gwy_lawn_get_unit_curve(lawn, ncurves), unit);
+            qdata = gwy_exp10(k);
         }
     }
     else {
@@ -606,7 +597,7 @@ fail:
     g_free(buffer);
     g_free(datablock);
     g_free(abscissa_map);
-    anfatec_fv_matrix_free(fvm);
+    fv_matrix_free(fvm);
 
     return lawn;
 }
@@ -614,10 +605,10 @@ fail:
 /* We get the directory name in GLib encoding and the basename in system encoding.  Which ensures lots of fun for the
  * long winter evenings.  */
 static gboolean
-anfatec_try_to_find_data(const gchar *dirname_glib,
-                         const gchar *basename_sys,
-                         gchar **buffer,
-                         gsize *size)
+try_to_find_data(const gchar *dirname_glib,
+                 const gchar *basename_sys,
+                 gchar **buffer,
+                 gsize *size)
 {
     static const gchar *encodings[] = {
         "UTF-16", "CP1252", "CP1251", "CP1250", "CP1253", "CP1254", "CP1255", "CP1256", "CP1257", "CP1258",
@@ -870,7 +861,7 @@ read_fv_matrix_file(gchar *buffer,
 
 fail:
     if (!ok) {
-        anfatec_fv_matrix_free(fvm);
+        fv_matrix_free(fvm);
         fvm = NULL;
     }
     g_free(xyindex);
@@ -884,7 +875,7 @@ fail:
 }
 
 static void
-anfatec_fv_matrix_free(AnfatecFVMatrix *fvm)
+fv_matrix_free(AnfatecFVMatrix *fvm)
 {
     if (!fvm)
         return;
@@ -892,7 +883,7 @@ anfatec_fv_matrix_free(AnfatecFVMatrix *fvm)
     g_strfreev(fvm->absunits);
     g_free(fvm->absdata);
     g_free(fvm->is_changing);
-    GWY_OBJECT_UNREF(fvm->brick);
+    g_clear_object(&fvm->brick);
     g_free(fvm);
 }
 
