@@ -1,6 +1,6 @@
 /*
- *  $Id: apedaxfile.c 25789 2023-10-06 14:44:53Z yeti-dn $
- *  Copyright (C) 2005 David Necas (Yeti), Petr Klapetek.
+ *  $Id: apedaxfile.c 28805 2025-11-05 12:28:20Z yeti-dn $
+ *  Copyright (C) 2005-2025 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  APDT and DAX file format Importer Module
@@ -72,13 +72,8 @@
 /* Includes */
 
 #include "config.h"
-#include <libgwyddion/gwyddion.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils-file.h>
+#include <glib/gi18n-lib.h>
+#include <gwy.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
@@ -145,11 +140,11 @@ typedef enum {
 
 /* SPM Modes Labels */
 static const GwyEnum spm_modes_display_names[] = {
-    { "SNOM",                  SPM_MODE_SNOM },
-    { "AFM Non-contact",       SPM_MODE_AFM_NONCONTACT },
-    { "AFM Contact",           SPM_MODE_AFM_CONTACT },
-    { "STM",                   SPM_MODE_STM },
-    { "Phase detection AFM",   SPM_MODE_PHASE_DETECT_AFM },
+    { "SNOM",                SPM_MODE_SNOM,             },
+    { "AFM Non-contact",     SPM_MODE_AFM_NONCONTACT,   },
+    { "AFM Contact",         SPM_MODE_AFM_CONTACT,      },
+    { "STM",                 SPM_MODE_STM,              },
+    { "Phase detection AFM", SPM_MODE_PHASE_DETECT_AFM, },
 };
 
 static const GwyEnum spm_modes_names[] = {
@@ -195,14 +190,14 @@ static const APEXmlField dax_afm_c[] = {
 };
 
 static const APEXmlField dax_afm_nc[] = {
-    { "TipOscFreq", "Tip Oscillation Frequency" },
-    { "VPmt1", "BIAS DC Voltage" }
+    { "TipOscFreq", "Tip Oscillation Frequency", },
+    { "VPmt1",      "BIAS DC Voltage",           },
 };
 
 static const APEXmlField dax_snom[] = {
-    { "TipOscFreq", "Tip Oscillation Frequency" },
-    { "VPmt1", "PMT 1 Voltage" },
-    { "VPmt2", "PMT 2 Voltage" }
+    { "TipOscFreq", "Tip Oscillation Frequency", },
+    { "VPmt1",      "PMT 1 Voltage",             },
+    { "VPmt2",      "PMT 2 Voltage",             },
 };
 
 static const APEXmlField dax_stm[] = {
@@ -210,45 +205,42 @@ static const APEXmlField dax_stm[] = {
 
 /* Prototypes */
 
-static gboolean      module_register              (void);
-static gint          apedax_detect                (const GwyFileDetectInfo *fileinfo,
-                                                   gboolean only_name);
-static gboolean      apedax_is_plausible_file     (const guchar *filehead,
-                                                   gsize len);
-static GwyContainer* apedax_load                  (const gchar *filename,
-                                                   GwyRunType mode,
-                                                   GError **error);
-static GwyContainer* apedax_get_meta              (guchar *scan_xml_content,
-                                                   gsize content_size,
-                                                   gboolean apdt_file);
-static gchar*        apedax_get_xml_node_as_string(xmlDocPtr doc,
-                                                   const gchar *node_xpath);
-static GwyDataField* apedax_get_data_field        (GwyZipFile uFile,
-                                                   const gchar *chFileName,
-                                                   const APEScanSize *scan_size,
-                                                   const gchar *zUnit,
-                                                   gdouble scale,
-                                                   GError **error);
-static gboolean      apedax_get_channels_data     (GwyZipFile uFile,
-                                                   guchar *scan_xml_content,
-                                                   gsize content_size,
-                                                   const gchar *filename,
-                                                   GwyContainer *container,
-                                                   GwyContainer *meta,
-                                                   const APEScanSize *scan_size,
-                                                   GError **error);
-
-static gboolean     apedax_is_xml_valid            (guchar *scan_xml_content,
-                                                    gsize content_size);
-
-static gboolean     apedax_is_xml_node_present     (xmlDocPtr doc,
-                                                    const gchar *node_xbase_path,
-                                                    const gchar *node_xpath,
-                                                    gboolean unique);
-
-static gboolean     apedax_get_scan_size           (xmlDocPtr doc,
-                                                    APEScanSize *scan_size,
-                                                    GError **error);
+static gboolean      module_register       (void);
+static gint          detect_file           (const GwyFileDetectInfo *fileinfo,
+                                            gboolean only_name);
+static gboolean      is_plausible_file     (const guchar *filehead,
+                                            gsize len);
+static GwyFile*      load_file             (const gchar *filename,
+                                            GwyRunModeFlags mode,
+                                            GError **error);
+static GwyContainer* get_meta              (guchar *scan_xml_content,
+                                            gsize content_size,
+                                            gboolean apdt_file);
+static gchar*        get_xml_node_as_string(xmlDocPtr doc,
+                                            const gchar *node_xpath);
+static GwyField*     get_field             (GwyZipFile uFile,
+                                            const gchar *chFileName,
+                                            const APEScanSize *scan_size,
+                                            const gchar *zUnit,
+                                            gdouble scale,
+                                            GError **error);
+static gboolean      get_channels_data     (GwyZipFile uFile,
+                                            guchar *scan_xml_content,
+                                            gsize content_size,
+                                            const gchar *filename,
+                                            GwyFile *file,
+                                            GwyContainer *meta,
+                                            const APEScanSize *scan_size,
+                                            GError **error);
+static gboolean      is_xml_valid          (guchar *scan_xml_content,
+                                            gsize content_size);
+static gboolean      is_xml_node_present   (xmlDocPtr doc,
+                                            const gchar *node_xbase_path,
+                                            const gchar *node_xpath,
+                                            gboolean unique);
+static gboolean      get_scan_size         (xmlDocPtr doc,
+                                            APEScanSize *scan_size,
+                                            GError **error);
 
 /* Informations about the module */
 
@@ -259,7 +251,7 @@ static GwyModuleInfo module_info = {
     "Andrea Cervesato <infos@aperesearch.com>, "
         "Gianfranco Gallizia <infos@aperesearch.com>, "
         "Samo Ziberna <infos@aperesearch.com>",
-    "0.8.1",
+    "0.9",
     "A.P.E. Research srl",
     "2015"
 };
@@ -271,10 +263,7 @@ module_register(void)
 {
     gwy_file_func_register("apedaxfile",
                            N_("A.P.E. Research DAX Files (.dax) and APDT File (.apdt)"),
-                           (GwyFileDetectFunc)&apedax_detect,
-                           (GwyFileLoadFunc)&apedax_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
@@ -282,8 +271,7 @@ module_register(void)
 /* Detect function */
 
 static gint
-apedax_detect(const GwyFileDetectInfo *fileinfo,
-              gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     gint score = 0;
     GwyZipFile uFile;
@@ -291,10 +279,8 @@ apedax_detect(const GwyFileDetectInfo *fileinfo,
     gsize size = 0;
     GError **error = NULL;
 
-    score += (g_str_has_suffix(fileinfo->name_lowercase, EXTENSION)
-              ? 10 : 0);
-    score += (g_str_has_suffix(fileinfo->name_lowercase, APDT_EXTENSION)
-              ? 10 : 0);
+    score += (g_str_has_suffix(fileinfo->name_lowercase, EXTENSION) ? 10 : 0);
+    score += (g_str_has_suffix(fileinfo->name_lowercase, APDT_EXTENSION) ? 10 : 0);
 
     if (only_name)
         return score;
@@ -307,21 +293,19 @@ apedax_detect(const GwyFileDetectInfo *fileinfo,
 
     /* Weed out surely non-DAX files before calling possibly expensive ZIP
      * functions. */
-    if (!apedax_is_plausible_file(fileinfo->head, fileinfo->buffer_len))
+    if (!is_plausible_file(fileinfo->head, fileinfo->buffer_len))
         return 0;
 
     gwy_debug("Opening the file with MiniZIP/libzip");
     uFile = gwyzip_open(fileinfo->name, NULL);
 
-    if (uFile == NULL) {
+    if (!uFile)
         return 0;
-    }
 
     if (gwyzip_locate_file(uFile, XML_FILE_NAME, 0, NULL)) {
-
         buffer = gwyzip_get_file_content(uFile, &size, error);
 
-        if (buffer != NULL) {
+        if (buffer) {
             score += 30;
             g_free(buffer);
         }
@@ -340,7 +324,7 @@ apedax_detect(const GwyFileDetectInfo *fileinfo,
 
 /* Try to find the name of an expected file somewhere. */
 static gboolean
-apedax_is_plausible_file(const guchar *filehead, gsize len)
+is_plausible_file(const guchar *filehead, gsize len)
 {
     static const gchar *filenames[] = {
         ".bin", "scan.xml",
@@ -357,12 +341,12 @@ apedax_is_plausible_file(const guchar *filehead, gsize len)
 
 /* Load function */
 
-static GwyContainer*
-apedax_load(const gchar *filename,
-            G_GNUC_UNUSED GwyRunType mode,
-            GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     GwyContainer *meta = NULL;
     GwyZipFile uFile;
     guchar *buffer = NULL;
@@ -386,7 +370,7 @@ apedax_load(const gchar *filename,
     gwy_debug("Opening the file with MiniZIP/libzip");
     uFile = gwyzip_open(filename, error);
 
-    if (uFile == NULL)
+    if (!uFile)
         return NULL;
 
     gwy_debug("Locating the XML file");
@@ -396,33 +380,32 @@ apedax_load(const gchar *filename,
 
     buffer = gwyzip_get_file_content(uFile, &content_size, error);
 
-    if (buffer == NULL)
+    if (!buffer)
         goto fail;
 
-    if (!apedax_is_xml_valid(buffer, content_size))
+    if (!is_xml_valid(buffer, content_size))
         goto fail;
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
 
-    meta = apedax_get_meta(buffer, content_size, apdt_flag);
+    meta = get_meta(buffer, content_size, apdt_flag);
 
-    if (meta == NULL) {
+    if (!meta) {
         gwy_debug("Metadata Container is NULL");
         goto fail;
     }
 
     doc = xmlReadMemory(buffer, content_size, XML_FILE_NAME, NULL, 0);
-    if (doc == NULL)
+    if (!doc)
         goto fail;
 
-    scan_size_result = apedax_get_scan_size(doc, &scan_size, error);
+    scan_size_result = get_scan_size(doc, &scan_size, error);
 
     if (!scan_size_result)
         goto fail;
 
-    channels_data_result = apedax_get_channels_data(uFile, buffer, content_size,
-                                                    filename, container, meta,
-                                                    &scan_size, error);
+    channels_data_result = get_channels_data(uFile, buffer, content_size, filename, file, meta,
+                                             &scan_size, error);
     if (!channels_data_result)
         goto fail;
 
@@ -436,29 +419,25 @@ fail:
         err_FILE_TYPE(error, FILE_TYPE);
     }
     gwy_debug("Cleaning up after a fail");
-    if (container != NULL) {
-        GWY_OBJECT_UNREF(container);
-    }
+    g_clear_object(&file);
 
 cleanup:
     /* Clean */
     xmlFreeDoc(doc);
     g_free(buffer);
-    if (meta != NULL) {
-        g_object_unref(meta);
-    }
-    if (uFile != NULL) {
+    g_clear_object(&meta);
+    if (uFile) {
         gwyzip_close(uFile);
     }
 
-    return container;
+    return file;
 }
 
 /* Gets the metadata from the XML file */
 static GwyContainer*
-apedax_get_meta(guchar *scan_xml_content,
-                gsize content_size,
-                gboolean apdt_file)
+get_meta(guchar *scan_xml_content,
+         gsize content_size,
+         gboolean apdt_file)
 {
     GwyContainer* meta = NULL;
     xmlDocPtr doc = NULL;
@@ -481,29 +460,26 @@ apedax_get_meta(guchar *scan_xml_content,
 
     const gchar *spm_mode_disp = NULL;
 
-    if (scan_xml_content == NULL || content_size == 0)
+    if (!scan_xml_content || content_size == 0)
         return NULL;
 
-    meta = gwy_container_new();
+    meta = gwy_container_new_in_construction();
 
     gwy_debug("Parsing the scan XML file");
     doc = xmlReadMemory(scan_xml_content, content_size, XML_FILE_NAME, NULL, 0);
 
-    if (doc == NULL) {
+    if (!doc)
         goto fail;
-     }
 
-    gwy_debug("Populating metadata container");
+    gwy_debug("Populating metadata file");
 
     context = xmlXPathNewContext(doc);
-    if (context == NULL) {
+    if (!context)
         goto fail;
-    }
-    path_object = xmlXPathEvalExpression(header_string, context);
 
-    if (path_object == NULL) {
+    path_object = xmlXPathEvalExpression(header_string, context);
+    if (!path_object)
         goto fail;
-    }
 
     /* There must be at least one header node */
     node_set = path_object->nodesetval;
@@ -520,7 +496,7 @@ apedax_get_meta(guchar *scan_xml_content,
                 if (cur_child->type == XML_ELEMENT_NODE) {
                     key_child = (gchar*)cur_child->name;
                     value = xmlNodeGetContent(cur_child);
-                    if (value != NULL) {
+                    if (value) {
                         concat_key = g_strconcat(key, " - ", key_child, NULL);
                         gwy_container_set_const_string_by_name(meta, concat_key, (gchar*)value);
                         g_free(concat_key);
@@ -532,7 +508,7 @@ apedax_get_meta(guchar *scan_xml_content,
             }
             if (!have_child) {
                 value = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
-                if (value != NULL) {
+                if (value) {
                     gwy_container_set_const_string_by_name(meta, key, (gchar*)value);
                     xmlFree(value);
                 }
@@ -541,7 +517,6 @@ apedax_get_meta(guchar *scan_xml_content,
         cur = cur->next;
     }
 
-
     if (apdt_file) {
         gwy_debug("Selected APDT file fields");
     }
@@ -592,22 +567,20 @@ apedax_get_meta(guchar *scan_xml_content,
         }
     }
 
-    gwy_debug("Returning metadata container");
+    gwy_debug("Returning metadata file");
     goto cleanup;
 
 fail:
     gwy_debug("Cleaning up after a fail");
-    GWY_OBJECT_UNREF(meta);
+    g_clear_object(&meta);
 cleanup:
-    if (path_object != NULL) {
+    if (path_object)
         xmlXPathFreeObject(path_object);
-    }
-    if (context != NULL) {
+    if (context)
         xmlXPathFreeContext(context);
-    }
-    if (doc != NULL) {
+    if (doc)
         xmlFreeDoc(doc);
-    }
+
     return meta;
 }
 
@@ -617,7 +590,7 @@ cleanup:
  * Return the value of the xml node or NULL if not found
  */
 static gchar*
-apedax_get_xml_node_as_string(xmlDocPtr doc, const gchar *node_xpath)
+get_xml_node_as_string(xmlDocPtr doc, const gchar *node_xpath)
 {
     gchar *node_string = NULL;
     xmlChar *node_xstring = NULL;
@@ -627,16 +600,13 @@ apedax_get_xml_node_as_string(xmlDocPtr doc, const gchar *node_xpath)
 
     context = xmlXPathNewContext(doc);
 
-    if (context == NULL) {
+    if (!context)
         return NULL;
-    }
 
     path_object = xmlXPathEvalExpression((const xmlChar*)node_xpath, context);
 
-    if (path_object != NULL) {
-
+    if (path_object) {
         if (!xmlXPathNodeSetIsEmpty(path_object->nodesetval)) {
-
             node_set = path_object->nodesetval;
 
             if (node_set->nodeNr == 1) {
@@ -646,24 +616,23 @@ apedax_get_xml_node_as_string(xmlDocPtr doc, const gchar *node_xpath)
             }
         }
         xmlXPathFreeObject(path_object);
-
     }
     xmlXPathFreeContext(context);
 
     return node_string;
 }
 
-static GwyDataField*
-apedax_get_data_field(GwyZipFile uFile,
-                      const gchar *channel_filename,
-                      const APEScanSize *scan_size,
-                      const gchar *zUnit,
-                      gdouble scale,
-                      GError **error)
+static GwyField*
+get_field(GwyZipFile uFile,
+          const gchar *channel_filename,
+          const APEScanSize *scan_size,
+          const gchar *zUnit,
+          gdouble scale,
+          GError **error)
 {
-    GwyDataField *dfield = NULL;
-    GwySIUnit *xyUnit;
-    GwySIUnit *zSIUnit;
+    GwyField *dfield = NULL;
+    GwyUnit *xyUnit;
+    GwyUnit *zSIUnit;
     gdouble *data;
     guchar *buffer;
     gsize content_size, expected_size;
@@ -679,17 +648,17 @@ apedax_get_data_field(GwyZipFile uFile,
         return NULL;
     }
 
-    dfield = gwy_data_field_new(scan_size->XRes, scan_size->YRes,
-                                scan_size->XReal, scan_size->YReal,
-                                FALSE);
+    dfield = gwy_field_new(scan_size->XRes, scan_size->YRes,
+                           scan_size->XReal, scan_size->YReal,
+                           FALSE);
 
-    data = gwy_data_field_get_data(dfield);
+    data = gwy_field_get_data(dfield);
 
-    xyUnit = gwy_data_field_get_si_unit_xy(dfield);
-    gwy_si_unit_set_from_string(xyUnit, XY_UNIT_STRING);
+    xyUnit = gwy_field_get_unit_xy(dfield);
+    gwy_unit_set_from_string(xyUnit, XY_UNIT_STRING);
 
-    zSIUnit = gwy_data_field_get_si_unit_z(dfield);
-    gwy_si_unit_set_from_string(zSIUnit, zUnit);
+    zSIUnit = gwy_field_get_unit_z(dfield);
+    gwy_unit_set_from_string(zSIUnit, zUnit);
 
     gwy_debug("Reading RAW data");
 
@@ -702,14 +671,14 @@ apedax_get_data_field(GwyZipFile uFile,
 }
 
 static gboolean
-apedax_get_channels_data(GwyZipFile uFile,
-                         guchar *scan_xml_content,
-                         gsize content_size,
-                         const gchar *filename,
-                         GwyContainer *container,
-                         GwyContainer *meta,
-                         const APEScanSize *scan_size,
-                         GError **error)
+get_channels_data(GwyZipFile uFile,
+                  guchar *scan_xml_content,
+                  gsize content_size,
+                  const gchar *filename,
+                  GwyFile *file,
+                  GwyContainer *meta,
+                  const APEScanSize *scan_size,
+                  GError **error)
 {
     gboolean outcome = FALSE;
     xmlDocPtr doc = NULL;
@@ -718,39 +687,31 @@ apedax_get_channels_data(GwyZipFile uFile,
     xmlXPathObjectPtr path_object = NULL;
     xmlNodeSetPtr node_set;
     xmlChar *buffer = NULL;
-    gchar key[256];
     gint i;
     gint power10 = 0;
     gdouble scale_factor = 1.0;
-    GwySIUnit *zUnit;
+    GwyUnit *zUnit;
     gchar *label = NULL;
     gchar *zUnit_string = NULL;
     gchar *bin_filename = NULL;
-    GwyDataField *dfield;
+    GwyField *dfield;
     GwyContainer *channel_meta;
 
-    if (scan_xml_content == NULL || content_size == 0)
+    if (!scan_xml_content || content_size == 0)
         return FALSE;
 
-    gwy_clear(key, sizeof(key));
-
     doc = xmlReadMemory(scan_xml_content, content_size, XML_FILE_NAME, NULL, 0);
-
-    if (doc == NULL) {
+    if (!doc)
         goto fail;
-    }
 
     context = xmlXPathNewContext(doc);
-
-    if (context == NULL) {
+    if (!context)
         goto fail;
-    }
 
     path_object = xmlXPathEvalExpression(channel_string, context);
-
-    if (path_object == NULL) {
+    if (!path_object)
         goto fail;
-    }
+
     /* There must be at least one channel */
     if (xmlXPathNodeSetIsEmpty(path_object->nodesetval)) {
         err_NO_DATA(error);
@@ -758,7 +719,6 @@ apedax_get_channels_data(GwyZipFile uFile,
     }
 
     node_set = path_object->nodesetval;
-
     if (node_set->nodeNr < 0) {
         goto fail;
     }
@@ -771,11 +731,11 @@ apedax_get_channels_data(GwyZipFile uFile,
         bin_filename = NULL;
 
         cur = node_set->nodeTab[i]->xmlChildrenNode;
-        /* Create a meta container for this channel */
-        channel_meta = gwy_container_duplicate(meta);
+        /* Create a meta file for this channel */
+        channel_meta = gwy_container_copy(meta);
         while (cur) {
             buffer = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
-            if (buffer != NULL) {
+            if (buffer) {
                 const gchar *curname = cur->name;
 
                 /* Label */
@@ -789,7 +749,7 @@ apedax_get_channels_data(GwyZipFile uFile,
                 /* Unit */
                 else if (gwy_strequal(curname, XML_CHANNEL_DATA_UNIT_NAME)) {
                     zUnit_string = g_strdup((gchar*)buffer);
-                    zUnit = gwy_si_unit_new_parse(zUnit_string, &power10);
+                    zUnit = gwy_unit_new_parse(zUnit_string, &power10);
                     g_object_unref(zUnit);
                 }
                 /* Binary file name */
@@ -805,16 +765,16 @@ apedax_get_channels_data(GwyZipFile uFile,
             cur = cur->next;
         }
 
-        if (bin_filename != NULL && zUnit_string != NULL && label != NULL) {
-            g_snprintf(key, sizeof(key), "/%d/data/title", i);
-            gwy_container_set_string_by_name(container, key, label);
-            scale_factor *= pow10(power10);
+        if (bin_filename && zUnit_string && label) {
+            gwy_file_pass_title(file, GWY_FILE_IMAGE, i, label);
+            scale_factor *= gwy_exp10(power10);
 
-            dfield = apedax_get_data_field(uFile, bin_filename, scan_size, zUnit_string, scale_factor, error);
+            dfield = get_field(uFile, bin_filename, scan_size, zUnit_string, scale_factor, error);
             if (dfield) {
-                gwy_container_pass_object(container, gwy_app_get_data_key_for_id(i), dfield);
-                gwy_container_set_object(container, gwy_app_get_data_meta_key_for_id(i), channel_meta);
-                gwy_file_channel_import_log_add(container, i, NULL, filename);
+                gwy_file_pass_image(file, i, dfield);
+                gwy_file_pass_meta(file, GWY_FILE_IMAGE, i, channel_meta);
+                gwy_log_add_import(file, GWY_FILE_IMAGE, i, NULL, filename);
+                channel_meta = NULL;
                 outcome = TRUE; /* We have at least one channel */
             }
 
@@ -823,7 +783,7 @@ apedax_get_channels_data(GwyZipFile uFile,
         else {
             g_warning("Missing one or more chanel field(s). Ignoring chanel.");
         }
-        g_object_unref(channel_meta);
+        g_clear_object(&channel_meta);
         g_free(zUnit_string);
     }
 
@@ -836,20 +796,17 @@ apedax_get_channels_data(GwyZipFile uFile,
 
 fail:
     /* Something went wrong!! */
-    if (error == NULL) {
+    if (!error)
         err_FILE_TYPE(error, FILE_TYPE);
-    }
 
 cleanup:
-    if (path_object != NULL) {
+    if (path_object)
         xmlXPathFreeObject(path_object);
-    }
-    if (context != NULL) {
+    if (context)
         xmlXPathFreeContext(context);
-    }
-    if (doc != NULL) {
+    if (doc)
         xmlFreeDoc(doc);
-    }
+
     return outcome;
 }
 
@@ -860,32 +817,32 @@ cleanup:
  * false otherwise.
  */
 static gboolean
-apedax_get_scan_size(xmlDocPtr doc, APEScanSize *scan_size, GError **error)
+get_scan_size(xmlDocPtr doc, APEScanSize *scan_size, GError **error)
 {
     xmlChar *buffer = NULL;
 
     /* Number of columns (XRes) */
-    buffer = apedax_get_xml_node_as_string(doc, XML_PATH_SCAN_SIZE_XRES);
+    buffer = get_xml_node_as_string(doc, XML_PATH_SCAN_SIZE_XRES);
 
-    if (buffer == NULL) {
+    if (!buffer) {
         return FALSE;
     }
     scan_size->XRes = (gint)g_ascii_strtod(buffer, NULL);
     g_free(buffer);
 
     /* Number of rows (YRes) */
-    buffer = apedax_get_xml_node_as_string(doc, XML_PATH_SCAN_SIZE_YRES);
+    buffer = get_xml_node_as_string(doc, XML_PATH_SCAN_SIZE_YRES);
 
-    if (buffer == NULL) {
+    if (!buffer) {
         return FALSE;
     }
     scan_size->YRes = (gint)g_ascii_strtod(buffer, NULL);
     g_free(buffer);
 
     /* Width in nanometers */
-    buffer = apedax_get_xml_node_as_string(doc, XML_PATH_SCAN_SIZE_X);
+    buffer = get_xml_node_as_string(doc, XML_PATH_SCAN_SIZE_X);
 
-    if (buffer == NULL) {
+    if (!buffer) {
         return FALSE;
     }
     scan_size->XReal = g_ascii_strtod(buffer, NULL);
@@ -894,9 +851,9 @@ apedax_get_scan_size(xmlDocPtr doc, APEScanSize *scan_size, GError **error)
 
 
     /* Height in nanometers */
-    buffer = apedax_get_xml_node_as_string(doc, XML_PATH_SCAN_SIZE_Y);
+    buffer = get_xml_node_as_string(doc, XML_PATH_SCAN_SIZE_Y);
 
-    if (buffer == NULL) {
+    if (!buffer) {
         return FALSE;
     }
     scan_size->YReal = g_ascii_strtod(buffer, NULL);
@@ -927,7 +884,7 @@ apedax_get_scan_size(xmlDocPtr doc, APEScanSize *scan_size, GError **error)
 }
 
 static gboolean
-apedax_is_xml_valid(guchar *scan_xml_content, gsize content_size)
+is_xml_valid(guchar *scan_xml_content, gsize content_size)
 {
     xmlDocPtr doc;
 
@@ -937,14 +894,14 @@ apedax_is_xml_valid(guchar *scan_xml_content, gsize content_size)
 
     doc = xmlReadMemory(scan_xml_content, content_size, XML_FILE_NAME, NULL, 0);
 
-    if (doc == NULL) {
+    if (!doc) {
         return FALSE;
     }
 
     /* Check if is present all the minimum Scan data (names in scan[]) */
     num_elements = G_N_ELEMENTS(scan);
     for (i = 0; i < num_elements; i++) {
-        if (!apedax_is_xml_node_present(doc, scan_string, scan[i], TRUE)) {
+        if (!is_xml_node_present(doc, scan_string, scan[i], TRUE)) {
             xmlFreeDoc(doc);
             return FALSE;
         }
@@ -953,7 +910,7 @@ apedax_is_xml_valid(guchar *scan_xml_content, gsize content_size)
     /* Check if is present all the minimum header data (names in header[]) */
     num_elements = G_N_ELEMENTS(header);
     for (i = 0; i < num_elements; i++) {
-        if (!apedax_is_xml_node_present(doc, header_string, header[i], TRUE)) {
+        if (!is_xml_node_present(doc, header_string, header[i], TRUE)) {
             xmlFreeDoc(doc);
             return FALSE;
         }
@@ -963,7 +920,7 @@ apedax_is_xml_valid(guchar *scan_xml_content, gsize content_size)
      */
     num_elements = G_N_ELEMENTS(scanSizeArray);
     for (i = 0; i < num_elements; i++) {
-        if (!apedax_is_xml_node_present(doc, scanSize_string, scanSizeArray[i], TRUE)) {
+        if (!is_xml_node_present(doc, scanSize_string, scanSizeArray[i], TRUE)) {
             xmlFreeDoc(doc);
             return FALSE;
         }
@@ -974,7 +931,7 @@ apedax_is_xml_valid(guchar *scan_xml_content, gsize content_size)
     channel_ok = TRUE;
     num_elements = G_N_ELEMENTS(channel);
     for (i = 0; i < num_elements; i++) {
-        if (!apedax_is_xml_node_present(doc, channel_string, channel[i], FALSE)) {
+        if (!is_xml_node_present(doc, channel_string, channel[i], FALSE)) {
             channel_ok = FALSE;
             break;
         }
@@ -985,10 +942,10 @@ apedax_is_xml_valid(guchar *scan_xml_content, gsize content_size)
 }
 
 static gboolean
-apedax_is_xml_node_present(xmlDocPtr doc,
-                           const gchar *node_xbase_path,
-                           const gchar *node_xpath,
-                           gboolean unique)
+is_xml_node_present(xmlDocPtr doc,
+                    const gchar *node_xbase_path,
+                    const gchar *node_xpath,
+                    gboolean unique)
 {
     xmlXPathContextPtr context;
     xmlXPathObjectPtr path_object;
@@ -998,19 +955,19 @@ apedax_is_xml_node_present(xmlDocPtr doc,
 
     context = xmlXPathNewContext(doc);
 
-    if (context == NULL) {
+    if (!context) {
         return result;
     }
 
     node_absolute_path = g_strconcat(node_xbase_path, "/", node_xpath, NULL);
 
-    if (node_absolute_path == NULL) {
+    if (!node_absolute_path) {
         gwy_debug("Memory exception");
         return FALSE;
     }
     path_object = xmlXPathEvalExpression((const xmlChar*)node_absolute_path, context);
 
-    if (path_object != NULL) {
+    if (path_object) {
         if (!xmlXPathNodeSetIsEmpty(path_object->nodesetval)) {
             node_set = path_object->nodesetval;
             if (node_set->nodeNr >= 1) {
