@@ -1,6 +1,6 @@
 /*
- *  $Id: apefile.c 25689 2023-09-20 15:55:12Z yeti-dn $
- *  Copyright (C) 2005 David Necas (Yeti), Petr Klapetek.
+ *  $Id: apefile.c 28789 2025-11-04 17:14:03Z yeti-dn $
+ *  Copyright (C) 2005-2025 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  Date conversion code copied from Wine, see below.
@@ -58,13 +58,8 @@
 #include <string.h>
 #include <time.h>
 #include "config.h"
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/datafield.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils-file.h>
+#include <glib/gi18n-lib.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -156,19 +151,19 @@ typedef struct {
     gint xy_hv_status;
     gint z_hv_status;
     /* The data */
-    GwyDataField **data;
+    GwyField **data;
 } APEFile;
 
-static gboolean      module_register     (void);
-static gint          apefile_detect      (const GwyFileDetectInfo *fileinfo,
-                                          gboolean only_name);
-static GwyContainer* apefile_load        (const gchar *filename,
-                                          GwyRunType mode,
-                                          GError **error);
-static void          fill_data_fields    (APEFile *apefile,
-                                          const guchar *buffer);
-static GwyContainer* apefile_get_metadata(APEFile *apefile);
-static gchar*        format_vt_date      (gdouble vt_date);
+static gboolean      module_register(void);
+static gint          detect_file    (const GwyFileDetectInfo *fileinfo,
+                                     gboolean only_name);
+static GwyFile*      load_file      (const gchar *filename,
+                                     GwyRunModeFlags mode,
+                                     GError **error);
+static void          fill_fields    (APEFile *apefile,
+                                     const guchar *buffer);
+static GwyContainer* get_metadata   (APEFile *apefile);
+static gchar*        format_vt_date (gdouble vt_date);
 
 static const GwyEnum spm_modes[] = {
     { "SNOM",                  SPM_MODE_SNOM },
@@ -191,7 +186,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports APE (Applied Physics and Engineering) data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.13",
+    "0.14",
     "David Nečas (Yeti) & Petr Klapetek",
     "2005",
 };
@@ -203,17 +198,13 @@ module_register(void)
 {
     gwy_file_func_register("apefile",
                            N_("APE files (.dat)"),
-                           (GwyFileDetectFunc)&apefile_detect,
-                           (GwyFileLoadFunc)&apefile_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-apefile_detect(const GwyFileDetectInfo *fileinfo,
-               gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     const guchar *buffer;
     gint score = 0;
@@ -240,18 +231,19 @@ apefile_detect(const GwyFileDetectInfo *fileinfo,
     return score;
 }
 
-static GwyContainer*
-apefile_load(const gchar *filename,
-             G_GNUC_UNUSED GwyRunType mode,
-             GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
     APEFile apefile;
-    GwyContainer *meta, *container = NULL;
+    GwyFile *file = NULL;
+    GwyContainer *meta;
     guchar *buffer = NULL;
     const guchar *p;
     gsize size = 0;
     GError *err = NULL;
-    GwyDataField *dfield = NULL;
+    GwyField *dfield = NULL;
     guint b, i, n;
 
     if (!gwy_file_get_contents(filename, &buffer, &size, &err)) {
@@ -259,7 +251,7 @@ apefile_load(const gchar *filename,
         return NULL;
     }
     p = buffer;
-    gwy_clear(&apefile, 1);
+    gwy_clear1(apefile);
     apefile.version = *(p++);
     if (size < 1294) {
         err_TOO_SHORT(error);
@@ -366,12 +358,12 @@ apefile_load(const gchar *filename,
         gwy_file_abandon_contents(buffer, size, NULL);
         return NULL;
     }
-    fill_data_fields(&apefile, p);
+    fill_fields(&apefile, p);
     gwy_file_abandon_contents(buffer, size, NULL);
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     /* All metadata seems to be per-file (global) */
-    meta = apefile_get_metadata(&apefile);
+    meta = get_metadata(&apefile);
     for (b = apefile.channels, n = 0, i = 0; b; b = b >> 1, i++) {
         const gchar *title;
 
@@ -381,7 +373,7 @@ apefile_load(const gchar *filename,
             break;
 
         dfield = apefile.data[n];
-        gwy_container_set_object(container, gwy_app_get_data_key_for_id(n), dfield);
+        gwy_file_set_image(file, n, dfield);
         g_object_unref(apefile.data[n]);
 
         /*
@@ -438,12 +430,9 @@ apefile_load(const gchar *filename,
                                         NULL);
         }
         if (title && *title)
-            gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(n), title);
-
-        gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(n), gwy_container_duplicate(meta));
-
-        gwy_file_channel_import_log_add(container, n, NULL, filename);
-
+            gwy_file_set_title(file, GWY_FILE_IMAGE, n, title, FALSE);
+        gwy_file_pass_meta(file, GWY_FILE_IMAGE, n, gwy_container_copy(meta));
+        gwy_log_add_import(file, GWY_FILE_IMAGE, n, NULL, filename);
         n++;
     }
 
@@ -451,21 +440,21 @@ apefile_load(const gchar *filename,
     g_free(apefile.data);
     g_free(apefile.remark);
 
-    return container;
+    return file;
 }
 
 static void
-fill_data_fields(APEFile *apefile,
-                 const guchar *buffer)
+fill_fields(APEFile *apefile,
+            const guchar *buffer)
 {
-    GwyDataField *dfield;
-    GwySIUnit *unit;
+    GwyField *dfield;
+    GwyUnit *unit;
     gdouble *data;
     const gchar *zunit;
     guint b, n, i, k;
     gdouble q;
 
-    apefile->data = g_new0(GwyDataField*, apefile->ndata);
+    apefile->data = g_new0(GwyField*, apefile->ndata);
     for (b = apefile->channels, n = 0, k = 0; b; b = b >> 1, k++) {
         if (!(b & 1))
             continue;
@@ -473,11 +462,11 @@ fill_data_fields(APEFile *apefile,
             break;
 
         gwy_debug("reading field %u (bit 0x%08x)", n, (1 << k));
-        dfield = gwy_data_field_new(apefile->res, apefile->res, apefile->xreal, apefile->yreal, FALSE);
-        unit = gwy_data_field_get_si_unit_xy(dfield);
-        gwy_si_unit_set_from_string(unit, "m");
+        dfield = gwy_field_new(apefile->res, apefile->res, apefile->xreal, apefile->yreal, FALSE);
+        unit = gwy_field_get_unit_xy(dfield);
+        gwy_unit_set_from_string(unit, "m");
 
-        data = gwy_data_field_get_data(dfield);
+        data = gwy_field_get_data(dfield);
         buffer += (apefile->res + 1)*sizeof(float);
         switch (k) {
             case APE_HEIGHT:
@@ -500,8 +489,8 @@ fill_data_fields(APEFile *apefile,
             break;
         }
 
-        unit = gwy_data_field_get_si_unit_z(dfield);
-        gwy_si_unit_set_from_string(unit, zunit);
+        unit = gwy_field_get_unit_z(dfield);
+        gwy_unit_set_from_string(unit, zunit);
 
         for (i = 0; i < apefile->res; i++) {
             /* There is always one ignored record, do not ask me why... */
@@ -518,12 +507,9 @@ fill_data_fields(APEFile *apefile,
     gwy_container_set_string_by_name(meta, key, g_strdup_printf(fmt, apefile->field))
 
 static GwyContainer*
-apefile_get_metadata(APEFile *apefile)
+get_metadata(APEFile *apefile)
 {
-    GwyContainer *meta;
-    gchar *p;
-
-    meta = gwy_container_new();
+    GwyContainer *meta = gwy_container_new_in_construction();
 
     gwy_container_set_string_by_name(meta, "Version", g_strdup_printf("%u.%u", apefile->version, apefile->subversion));
 
@@ -556,6 +542,7 @@ apefile_get_metadata(APEFile *apefile)
     HASH_STORE("Raster delay", "%.6f s", raster_delay);
     HASH_STORE("Tip distance", "%g nm", tip_dist);
 
+    gchar *p;
     if (apefile->remark && *apefile->remark && (p = gwy_convert_to_utf8(apefile->remark, -1, "ISO-8859-1")))
         gwy_container_set_string_by_name(meta, "Comment", p);
     gwy_container_set_const_string_by_name(meta, "SPM mode",
@@ -572,20 +559,16 @@ apefile_get_metadata(APEFile *apefile)
 /*
  * Copyright (C) the Wine project
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General
+ * Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option)
+ * any later version.
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
+ * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
+ * more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA
+ * You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
 typedef guint WORD;
@@ -811,7 +794,7 @@ format_vt_date(gdouble vt_date)
     if (!gwy_VarUdateFromDate(vt_date, &udate))
         return NULL;
 
-    gwy_clear(&tm, 1);
+    gwy_clear1(tm);
     gwy_debug("Date: %d-%d-%d %d:%d:%d",
               udate.st.wYear, udate.st.wMonth, udate.st.wDay, udate.st.wHour, udate.st.wMinute, udate.st.wSecond);
     tm.tm_year = udate.st.wYear - 1900;
