@@ -1,8 +1,11 @@
 /*
- *  $Id: ardf.c 26519 2024-08-15 18:04:20Z yeti-dn $
- *  Copyright (C) Vojtech Klapetek, Matt Poss
+ *  $Id: ardf.c 28848 2025-11-10 18:54:08Z yeti-dn $
+ *  Copyright (C) 2023 Vojtech Klapetek, Matt Poss
  *  E-mail: vojvojta@gmail.com
  *
+ *  Copyright (C) 2025 David Necas (Yeti).
+ *  E-mail: yeti@gwyddion.net.
+ *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
  *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
  *  later version.
@@ -35,18 +38,12 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdio.h>
 #include <stdbool.h>
 #include <glib/gstdio.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/stats.h>
-#include <libprocess/brick.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/wait.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 
 #include "err.h"
 #include "get.h"
@@ -158,10 +155,10 @@ typedef struct {
 } ARDF_TEXT;
 
 static gboolean      module_register    (void);
-static gint          ardf_detect        (const GwyFileDetectInfo *fileinfo,
+static gint          detect_file        (const GwyFileDetectInfo *fileinfo,
                                          gboolean only_name);
-static GwyContainer* ardf_load          (const gchar *filename,
-                                         GwyRunType mode,
+static GwyFile*      load_file          (const gchar *filename,
+                                         GwyRunModeFlags mode,
                                          GError **error);
 static void          err_HEADER_MISMATCH(GError **error,
                                          const gchar *found,
@@ -217,13 +214,12 @@ static void          free_ARDF_TEXT     (ARDF_TEXT *txt);
 static void          free_ARDF_VNAM     (ARDF_VNAM *vnam);
 static void          free_ARDF_VDAT     (ARDF_VDAT *vdat);
 
-
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Asylum Research ARDF files."),
     "Vojtech Klapetek <vojvojta@gmail.com>",
-    "0.1",
+    "0.2",
     "Vojtech Klapetek, Matt Poss",
     "2023",
 };
@@ -235,17 +231,13 @@ module_register(void)
 {
     gwy_file_func_register("ardf",
                            N_("ARDF file (.ARDF)"),
-                           (GwyFileDetectFunc)&ardf_detect,
-                           (GwyFileLoadFunc)&ardf_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-ardf_detect(const GwyFileDetectInfo *fileinfo,
-            gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     if (only_name)
         return g_str_has_suffix(fileinfo->name, EXTENSION) ? 50 : 0;
@@ -257,9 +249,9 @@ ardf_detect(const GwyFileDetectInfo *fileinfo,
     return 0;
 }
 
-static GwyContainer*
-ardf_load(const gchar *filename,
-          G_GNUC_UNUSED GwyRunType mode,
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
           GError **error)
 {
     //ARDFFile ardf;
@@ -274,10 +266,10 @@ ardf_load(const gchar *filename,
     ARDF_TOC *idx = NULL;
     ARDF_DEF *imdef = NULL;
     ARDF_DEF *vdef = NULL;
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     GwyContainer *instrument_data = NULL;
     GwyContainer *meta = NULL;
-    GwyDataField *dfield = NULL;
+    GwyField *dfield = NULL;
     GwyBrick **volm_channels = NULL;
     guchar *buffer, *b, *p;
     const guchar *cb;
@@ -385,7 +377,7 @@ ardf_load(const gchar *filename,
     txt = NULL;
     gwy_debug("instrument notes (kind of) read\n\n");
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
 
     gwy_debug("image count: %i\n", ftoc->len_imag);
     for (i = 0; i < ftoc->len_imag; i++) {
@@ -432,43 +424,40 @@ ardf_load(const gchar *filename,
                     //err_INVALID(error, "IMAG note");
                     goto fail;
                 }
-                gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(i), parse_metadata(raw_note));
+                gwy_file_pass_meta(file, GWY_FILE_IMAGE, i, parse_metadata(raw_note));
                 free_ARDF_TEXT(raw_note);
                 raw_note = NULL;
             }
         }
         if (numb_imag_text == 0) {
             // if no note is present, adds instrument data instead
-            gwy_container_set_object(container, gwy_app_get_data_meta_key_for_id(i), instrument_data);
+            gwy_file_pass_meta(file, GWY_FILE_IMAGE, i, gwy_container_copy(instrument_data));
         }
 
         gwy_debug("size %i %i\n", idat->x_size, idat->y_size);
 
-        dfield = gwy_data_field_new(idat->x_size,
-                                    idat->y_size,
-                                    lat_size,
-                                    lat_size,
-                                    FALSE);
+        dfield = gwy_field_new(idat->x_size, idat->y_size,
+                               lat_size, lat_size,
+                               FALSE);
         gwy_convert_raw_data(idat->data, idat->x_size*idat->y_size,
                              1, GWY_RAW_DATA_FLOAT,
                              GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                             gwy_data_field_get_data(dfield),
+                             gwy_field_get_data(dfield),
                              1, 0);
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
-        if (strstr(imdef->image_title, "Force") != NULL)
-            gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), "N");
-        else if (strstr(imdef->image_title, "Adhesion") != NULL)
-            gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), "N");
-        else if (strstr(imdef->image_title, "Young") != NULL)
-            gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), "Pa");
-        else if (strstr(imdef->image_title, "Voltage") != NULL)
-            gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), "V");
-        else
-            gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), "m");
+        gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
 
-        gwy_container_pass_object(container, gwy_app_get_data_key_for_id(i), dfield);
-        gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(i), imdef->image_title);
-        gwy_file_channel_import_log_add(container, i, NULL, filename);
+        const gchar *unitstr = "m";
+        if (strstr(imdef->image_title, "Force") || strstr(imdef->image_title, "Adhesion"))
+            unitstr = "N";
+        else if (strstr(imdef->image_title, "Young"))
+            unitstr = "Pa";
+        else if (strstr(imdef->image_title, "Voltage"))
+            unitstr = "V";
+        gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), unitstr);
+
+        gwy_file_pass_image(file, i, dfield);
+        gwy_file_set_title(file, GWY_FILE_IMAGE, i, imdef->image_title, FALSE);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, i, NULL, filename);
 
         GWY_FREE(imdef);
         free_ARDF_TOC(imtoc);
@@ -675,10 +664,10 @@ ardf_load(const gchar *filename,
             volm_channels[j] = gwy_brick_new(numb_points, numb_lines, data_len,
                                              lat_size, lat_size, 1,
                                              TRUE);
-            gwy_si_unit_set_from_string(gwy_brick_get_si_unit_x(volm_channels[j]), "m");
-            gwy_si_unit_set_from_string(gwy_brick_get_si_unit_y(volm_channels[j]), "m");
-            gwy_si_unit_set_from_string(gwy_brick_get_si_unit_z(volm_channels[j]), "m");
-            gwy_si_unit_set_from_string(gwy_brick_get_si_unit_w(volm_channels[j]), "m");
+            gwy_unit_set_from_string(gwy_brick_get_unit_x(volm_channels[j]), "m");
+            gwy_unit_set_from_string(gwy_brick_get_unit_y(volm_channels[j]), "m");
+            gwy_unit_set_from_string(gwy_brick_get_unit_z(volm_channels[j]), "m");
+            gwy_unit_set_from_string(gwy_brick_get_unit_w(volm_channels[j]), "m");
         }
 
         for (r = 0; r < numb_lines; r++) {
@@ -769,11 +758,11 @@ ardf_load(const gchar *filename,
         }
 
         for (j = 0; j < numb_channels; j++) {
-            gwy_brick_invert(volm_channels[j], FALSE, FALSE, FALSE, TRUE);
-            gwy_container_pass_object(container, gwy_app_get_brick_key_for_id(c_volm), volm_channels[j]);
-            gwy_container_set_const_string(container, gwy_app_get_brick_title_key_for_id(c_volm), vchn[j]);
-            gwy_file_volume_import_log_add(container, j, NULL, filename);
-            c_volm ++;
+            gwy_brick_invert_value(volm_channels[j]);
+            gwy_file_pass_volume(file, c_volm, volm_channels[j]);
+            gwy_file_set_title(file, GWY_FILE_VOLUME, c_volm, vchn[j], FALSE);
+            gwy_log_add_import(file, GWY_FILE_VOLUME, j, NULL, filename);
+            c_volm++;
         }
 
         //g_object_unref(dfield); //causes errors
@@ -798,7 +787,7 @@ fail:  //FIXME: some fails lead to free() error
     if (wait_dialogue_shown)
         gwy_app_wait_finish();
     g_free(apoint);
-    GWY_OBJECT_UNREF(instrument_data);
+    g_clear_object(&instrument_data);
     free_ARDF_TOC(ftoc);
     free_ARDF_TOC(ttoc);
     free_ARDF_TOC(imtoc);
@@ -807,21 +796,21 @@ fail:  //FIXME: some fails lead to free() error
     free_ARDF_TOC(volm);
     free_ARDF_TOC(volm_ttoc);
     free_ARDF_TOC(idx);
-    GWY_OBJECT_UNREF(meta);
+    g_clear_object(&meta);
     g_free(imdef);
     g_free(vdef);
-    if (channel_name != NULL)
+    if (channel_name)
         g_string_free(channel_name, TRUE);
     g_free(volm_channel);
-    //if (unit != NULL) g_object_unref(unit);
-    //if (dfield != NULL) g_object_unref(dfield);
-    if (volm_channels != NULL) {
+    //if (unit) g_object_unref(unit);
+    //if (dfield) g_object_unref(dfield);
+    if (volm_channels) {
         for (i = 0; i < numb_channels; i++) {
             g_free(volm_channels[i]);
         }
         g_free(volm_channels);
     }
-    if (vchn != NULL) {
+    if (vchn) {
         for (i = 0; i < c_vchn; i++) {
             g_free(vchn[i]);
         }
@@ -833,7 +822,7 @@ fail:  //FIXME: some fails lead to free() error
     free_ARDF_TEXT(txt);
     free_ARDF_TEXT(raw_note);
     gwy_file_abandon_contents(buffer, size, NULL);
-    return container;
+    return file;
 }
 
 static inline void
@@ -987,37 +976,29 @@ read_ARDF_TOC(guchar **pp, guchar *start, gsize size, gint32 address, guchar poi
             b += 24;
         else
             b += (toc->size_entry - 16);
-        if (memcmp(TYPE_IMAG, bpoint->type_pnt, 4) == 0) {
-            c_imag += 1;
-        }
-        else if (memcmp(TYPE_VOLM, bpoint->type_pnt, 4) == 0) {
-            c_volm += 1;
-        }
-        else if (memcmp(TYPE_NEXT, bpoint->type_pnt, 4) == 0) {
-            c_next += 1;
-        }
-        else if (memcmp(TYPE_NSET, bpoint->type_pnt, 4) == 0) {
-            c_nset += 1;
-        }
-        else if (memcmp(TYPE_THMB, bpoint->type_pnt, 4) == 0) {
-            c_thmb += 1;
-        }
-        else if (memcmp(TYPE_TOFF, bpoint->type_pnt, 4) == 0) {
-            c_toff += 1;
-        }
-        else if (memcmp(TYPE_IDAT, bpoint->type_pnt, 4) == 0) {
-            c_idat += 1;
-        }
-        else if (memcmp(TYPE_VOFF, bpoint->type_pnt, 4) == 0) {
-            c_voff += 1;
-        }
+        if (memcmp(TYPE_IMAG, bpoint->type_pnt, 4) == 0)
+            c_imag++;
+        else if (memcmp(TYPE_VOLM, bpoint->type_pnt, 4) == 0)
+            c_volm++;
+        else if (memcmp(TYPE_NEXT, bpoint->type_pnt, 4) == 0)
+            c_next++;
+        else if (memcmp(TYPE_NSET, bpoint->type_pnt, 4) == 0)
+            c_nset++;
+        else if (memcmp(TYPE_THMB, bpoint->type_pnt, 4) == 0)
+            c_thmb++;
+        else if (memcmp(TYPE_TOFF, bpoint->type_pnt, 4) == 0)
+            c_toff++;
+        else if (memcmp(TYPE_IDAT, bpoint->type_pnt, 4) == 0)
+            c_idat++;
+        else if (memcmp(TYPE_VOFF, bpoint->type_pnt, 4) == 0)
+            c_voff++;
         else if (memcmp(TYPE_XXXX, bpoint->type_pnt, 4) == 0) {
             if (memcmp(TYPE_IBOX, apoint->type_pnt, 4) == 0) {
-                c_idat += 1;  //uses same list as IDAT
+                c_idat++;  //uses same list as IDAT
                 gwy_debug("|");
             }
             else if (memcmp(TYPE_VTOC, apoint->type_pnt, 4) == 0) {
-                c_voff += 1; //uses same lists as VOFF
+                c_voff++; //uses same lists as VOFF
             }
             else
                 done = TRUE;
@@ -1550,7 +1531,7 @@ parse_metadata(ARDF_TEXT *raw)
     gboolean value_wanted = FALSE;
     guint32 i, key_i, val_i;
 
-    meta = gwy_container_new();
+    meta = gwy_container_new_in_construction();
 
     key_i = 0;
     val_i = 0;
@@ -1598,7 +1579,7 @@ parse_metadata(ARDF_TEXT *raw)
 static void
 free_ARDF_TOC(ARDF_TOC *toc)
 {
-    if (toc != NULL) {
+    if (toc) {
         g_free(toc->pnt_imag);
         g_free(toc->pnt_volm);
         g_free(toc->pnt_next);
@@ -1617,7 +1598,7 @@ free_ARDF_TOC(ARDF_TOC *toc)
 static void
 free_ARDF_TEXT(ARDF_TEXT *txt)
 {
-    if (txt != NULL) {
+    if (txt) {
         g_free(txt->text);
         g_free(txt);
     }
@@ -1626,7 +1607,7 @@ free_ARDF_TEXT(ARDF_TEXT *txt)
 static void
 free_ARDF_VNAM(ARDF_VNAM *vnam)
 {
-    if (vnam != NULL) {
+    if (vnam) {
          g_free(vnam->name);
          g_free(vnam);
     }
@@ -1635,7 +1616,7 @@ free_ARDF_VNAM(ARDF_VNAM *vnam)
 static void
 free_ARDF_VDAT(ARDF_VDAT *vdat)
 {
-    if (vdat != NULL) {
+    if (vdat) {
         g_free(vdat->data);
         g_free(vdat);
     }
