@@ -1,6 +1,6 @@
 /*
- *  $Id: asciiexport.c 25905 2023-10-22 16:32:36Z yeti-dn $
- *  Copyright (C) 2003-2023 David Necas (Yeti).
+ *  $Id: asciiexport.c 28911 2025-11-24 18:27:42Z yeti-dn $
+ *  Copyright (C) 2003-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -28,17 +28,14 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 #include <glib/gstdio.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <libprocess/datafield.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -64,19 +61,19 @@ typedef struct {
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static gint             asciiexport_detect  (const GwyFileDetectInfo *fileinfo,
+static gint             detect_file         (const GwyFileDetectInfo *fileinfo,
                                              gboolean only_name);
-static GwyContainer*    asciiexport_load    (const gchar *filename,
-                                             GwyRunType mode,
+static GwyFile*         load_file           (const gchar *filename,
+                                             GwyRunModeFlags mode,
                                              GError **error);
-static gboolean         asciiexport_export  (GwyContainer *data,
+static gboolean         export_file         (GwyFile *data,
                                              const gchar *filename,
-                                             GwyRunType mode,
+                                             GwyRunModeFlags mode,
                                              GError **error);
 static GwyDialogOutcome run_gui             (ModuleArgs *args);
 static void             param_changed       (ModuleGUI *gui,
                                              gint id);
-static gboolean         export_one_channel  (GwyContainer *data,
+static gboolean         export_one_channel  (GwyFile *data,
                                              gint id,
                                              ModuleArgs *args,
                                              FILE *fh);
@@ -86,7 +83,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Exports data as simple ASCII matrix and reads back this format."),
     "Yeti <yeti@gwyddion.net>",
-    "3.0",
+    "4.0",
     "David Nečas (Yeti)",
     "2004",
 };
@@ -98,10 +95,7 @@ module_register(void)
 {
     gwy_file_func_register("asciiexport",
                            N_("ASCII data matrix (.txt)"),
-                           (GwyFileDetectFunc)&asciiexport_detect,
-                           (GwyFileLoadFunc)&asciiexport_load,
-                           NULL,
-                           (GwyFileSaveFunc)&asciiexport_export);
+                           detect_file, load_file, NULL, &export_file);
 
     return TRUE;
 }
@@ -200,8 +194,7 @@ parse_header(const gchar **s,
 }
 
 static gint
-asciiexport_detect(const GwyFileDetectInfo *fileinfo,
-                   gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     const gchar *channel, *width, *height, *zunit, *p;
 
@@ -215,17 +208,17 @@ asciiexport_detect(const GwyFileDetectInfo *fileinfo,
     return 50;
 }
 
-static GwyContainer*
-asciiexport_load(const gchar *filename,
-                 G_GNUC_UNUSED GwyRunType mode,
-                 GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     gchar *buffer, *s, *end;
     gsize size;
     const gchar *channel, *width, *height, *zunit, *p;
     gboolean found_header = FALSE, moved;
-    GwyDataField *field;
+    GwyField *field;
     GError *err = NULL;
     gint xres, yres, i, id, power10;
     gdouble r;
@@ -267,46 +260,46 @@ asciiexport_load(const gchar *filename,
         }
 
         if (err_DIMENSION(error, xres) || err_DIMENSION(error, yres)) {
-            GWY_OBJECT_UNREF(container);
+            g_clear_object(&file);
             break;
         }
 
-        field = gwy_data_field_new(xres, yres, 1.0, 1.0, FALSE);
-        if (!gwy_parse_doubles(p, gwy_data_field_get_data(field), 0, &yres, &xres, NULL, error)) {
-            GWY_OBJECT_UNREF(field);
-            GWY_OBJECT_UNREF(container);
+        field = gwy_field_new(xres, yres, 1.0, 1.0, FALSE);
+        if (!gwy_parse_doubles(p, gwy_field_get_data(field), 0, &yres, &xres, NULL, error)) {
+            g_clear_object(&field);
+            g_clear_object(&file);
             break;
         }
 
-        if (!container)
-            container = gwy_container_new();
+        if (!file)
+            file = gwy_file_new_in_construction();
 
         /* Z unit. */
         s = g_strndup(zunit, to_end_of_line(zunit) - zunit);
-        gwy_si_unit_set_from_string_parse(gwy_data_field_get_si_unit_z(field), s, &power10);
+        power10 = gwy_unit_set_from_string(gwy_field_get_unit_z(field), s);
         g_free(s);
-        gwy_data_field_multiply(field, pow10(power10));
+        gwy_field_multiply(field, gwy_exp10(power10));
 
         /* X and Y */
         r = g_strtod(height, &end);
         sanitise_real_size(&r, "y size");
         s = g_strndup(end, to_end_of_line(end) - end);
-        gwy_si_unit_set_from_string_parse(gwy_data_field_get_si_unit_xy(field), s, &power10);
-        gwy_data_field_set_yreal(field, r*pow10(power10));
+        power10 = gwy_unit_set_from_string(gwy_field_get_unit_xy(field), s);
+        gwy_field_set_yreal(field, r*gwy_exp10(power10));
         g_free(s);
 
         r = g_strtod(width, &end);
         sanitise_real_size(&r, "x size");
         s = g_strndup(end, to_end_of_line(end) - end);
-        gwy_si_unit_set_from_string_parse(gwy_data_field_get_si_unit_xy(field), s, &power10);
-        gwy_data_field_set_xreal(field, r*pow10(power10));
+        power10 = gwy_unit_set_from_string(gwy_field_get_unit_xy(field), s);
+        gwy_field_set_xreal(field, r*gwy_exp10(power10));
         g_free(s);
 
-        gwy_container_pass_object(container, gwy_app_get_data_key_for_id(id), field);
+        gwy_file_pass_image(file, id, field);
 
         /* Title. */
         s = g_strstrip(g_strndup(channel, to_end_of_line(channel) - channel));
-        gwy_container_set_string(container, gwy_app_get_data_title_key_for_id(id), s);
+        gwy_file_pass_title(file, GWY_FILE_IMAGE, id, s);
 
         id++;
         p += i;
@@ -318,14 +311,14 @@ asciiexport_load(const gchar *filename,
 
     g_free(buffer);
 
-    return container;
+    return file;
 }
 
 static gboolean
-asciiexport_export(GwyContainer *data,
-                   const gchar *filename,
-                   GwyRunType mode,
-                   GError **error)
+export_file(GwyFile *data,
+            const gchar *filename,
+            GwyRunModeFlags mode,
+            GError **error)
 {
     ModuleArgs args;
     gint i, id, *ids;
@@ -333,7 +326,7 @@ asciiexport_export(GwyContainer *data,
     FILE *fh = NULL;
     gboolean ok = FALSE;
 
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD_ID, &id, 0);
+    gwy_data_browser_get_current(GWY_APP_FIELD_ID, &id, 0);
     if (id < 0) {
         err_NO_CHANNEL_EXPORT(error);
         return FALSE;
@@ -355,7 +348,7 @@ asciiexport_export(GwyContainer *data,
     }
 
     if (gwy_params_get_boolean(args.params, PARAM_CONCAT_ALL)) {
-        ids = gwy_app_data_browser_get_data_ids(data);
+        ids = gwy_file_get_ids(data, GWY_FILE_IMAGE);
         for (i = 0; ids[i] >= 0; i++) {
             if (!export_one_channel(data, ids[i], &args, fh) || gwy_fprintf(fh, "\n") < 0) {
                 err_WRITE(error);
@@ -427,9 +420,9 @@ param_changed(ModuleGUI *gui, gint id)
 }
 
 static gboolean
-export_one_channel(GwyContainer *data, gint id, ModuleArgs *args, FILE *fh)
+export_one_channel(GwyFile *data, gint id, ModuleArgs *args, FILE *fh)
 {
-    GwyDataField *field = gwy_container_get_object(data, gwy_app_get_data_key_for_id(id));
+    GwyField *field = gwy_file_get_image(data, id);
     gboolean decimal_dot = gwy_params_get_boolean(args->params, PARAM_DECIMAL_DOT);
     gint precision = gwy_params_get_int(args->params, PARAM_PRECISION);
     gboolean add_comment = gwy_params_get_boolean(args->params, PARAM_ADD_COMMENT);
@@ -438,24 +431,22 @@ export_one_channel(GwyContainer *data, gint id, ModuleArgs *args, FILE *fh)
     const gdouble *d;
     GString *str;
 
-    g_return_val_if_fail(GWY_IS_DATA_FIELD(field), FALSE);
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
-    d = gwy_data_field_get_data_const(field);
+    g_return_val_if_fail(GWY_IS_FIELD(field), FALSE);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
+    d = gwy_field_get_data_const(field);
 
     str = g_string_new(NULL);
     if (add_comment) {
-        gdouble xreal = gwy_data_field_get_xreal(field), yreal = gwy_data_field_get_yreal(field);
-        GwySIUnit *units;
-        GwySIValueFormat *vf = NULL;
+        gdouble xreal = gwy_field_get_xreal(field), yreal = gwy_field_get_yreal(field);
+        GwyValueFormat *vf = NULL;
         gdouble v;
-        gchar *s;
 
-        s = gwy_app_get_data_field_title(data, id);
+        gchar *s = gwy_file_get_display_title(data, GWY_FILE_IMAGE, id);
         g_string_printf(str, "# %s %s\n", english_comment ? "Channel:" : _("Channel:"), s);
         g_free(s);
 
-        vf = gwy_data_field_get_value_format_xy(field, GWY_SI_UNIT_FORMAT_VFMARKUP, vf);
+        vf = gwy_field_get_value_format_xy(field, GWY_UNIT_FORMAT_VFMARKUP, vf);
 
         g_string_append_printf(str, "# %s ", english_comment ? "Width:" : _("Width:"));
         v = xreal/vf->magnitude;
@@ -467,14 +458,14 @@ export_one_channel(GwyContainer *data, gint id, ModuleArgs *args, FILE *fh)
         gwy_append_doubles_to_gstring(str, &v, 1, precision, "", decimal_dot);
         g_string_append_printf(str, " %s\n", vf->units);
 
-        units = gwy_data_field_get_si_unit_z(field);
-        s = gwy_si_unit_get_string(units, GWY_SI_UNIT_FORMAT_VFMARKUP);
+        GwyUnit *units = gwy_field_get_unit_z(field);
+        s = gwy_unit_get_string(units, GWY_UNIT_FORMAT_VFMARKUP);
         g_string_append_printf(str, "# %s %s\n", english_comment ? "Value units:" : _("Value units:"), s);
         g_free(s);
 
         fputs(str->str, fh);
 
-        gwy_si_unit_value_format_free(vf);
+        gwy_value_format_free(vf);
     }
 
     for (i = 0; i < yres; i++) {
