@@ -1,6 +1,6 @@
 /*
- *  $Id: asdfile.c 26510 2024-08-15 17:12:23Z yeti-dn $
- *  Copyright (C) 2024 David Necas (Yeti).
+ *  $Id: asdfile.c 28772 2025-11-03 18:02:12Z yeti-dn $
+ *  Copyright (C) 2024-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -34,13 +34,8 @@
  **/
 
 #include "config.h"
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/stats.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils-file.h>
+#include <glib/gi18n-lib.h>
+#include <gwy.h>
 
 #include "get.h"
 #include "err.h"
@@ -147,15 +142,16 @@ typedef struct {
 } ASDFrame;
 
 static gboolean      module_register     (void);
-static gint          asd_detect          (const GwyFileDetectInfo *fileinfo,
+static gint          detect_file         (const GwyFileDetectInfo *fileinfo,
                                           gboolean only_name);
-static GwyContainer* asd_load            (const gchar *filename,
-                                          GwyRunType mode,
+static GwyFile*      load_file           (const gchar *filename,
+                                          GwyRunModeFlags mode,
                                           GError **error);
 static GwyContainer* get_meta            (ASDHeader *header);
-static void          create_graphs       (GwyContainer *container,
+static void          create_graphs       (GwyFile *file,
                                           GArray *frames,
-                                          gint id);
+                                          gint id,
+                                          const gchar *filename);
 static gboolean      read_header         (ASDHeader *header,
                                           const guchar **p,
                                           gsize size,
@@ -175,7 +171,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports ASD high-speed AFM data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.1",
+    "0.2",
     "David Nečas (Yeti)",
     "2024",
 };
@@ -187,18 +183,14 @@ module_register(void)
 {
     gwy_file_func_register("asdfile",
                            N_("ASD high-speed AFM files (.asd)"),
-                           (GwyFileDetectFunc)&asd_detect,
-                           (GwyFileLoadFunc)&asd_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
     gwy_file_func_set_is_unfinished("asdfile", TRUE);
 
     return TRUE;
 }
 
 static gint
-asd_detect(const GwyFileDetectInfo *fileinfo,
-           gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     gsize minsize, header_size, fh_size, oplen = 0, commoff = 0, commlen = 0;
     guint version, ch1, ch2;
@@ -268,12 +260,12 @@ asd_detect(const GwyFileDetectInfo *fileinfo,
     return 60;
 }
 
-static GwyContainer*
-asd_load(const gchar *filename,
-         G_GNUC_UNUSED GwyRunType mode,
-         GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     guchar *buffer = NULL;
     const guchar *p;
     const gchar *title;
@@ -290,7 +282,7 @@ asd_load(const gchar *filename,
         err_GET_FILE_CONTENTS(error, &err);
         return NULL;
     }
-    gwy_clear(&header, 1);
+    gwy_clear1(header);
 
     if (size <= sizeof(guint32)) {
         err_TOO_SHORT(error);
@@ -319,7 +311,7 @@ asd_load(const gchar *filename,
         goto fail;
 
     /* A bit silly, but the files have exactly two channels. */
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     for (id = 0; id < 2; id++) {
         ASDDataType datatype = (id == 0 ? header.data_type_ch1 : header.data_type_ch2);
 
@@ -329,8 +321,8 @@ asd_load(const gchar *filename,
         qsens = value_scaling_factor(&header, datatype);
         converter_parameters(&header, &qconv, &z0conv);
         brick = gwy_brick_new(xres, yres, nframes, header.xreal, header.yreal, 1.0, FALSE);
-        gwy_si_unit_set_from_string(gwy_brick_get_si_unit_x(brick), "m");
-        gwy_si_unit_set_from_string(gwy_brick_get_si_unit_y(brick), "m");
+        gwy_unit_set_from_string(gwy_brick_get_unit_x(brick), "m");
+        gwy_unit_set_from_string(gwy_brick_get_unit_y(brick), "m");
         d = gwy_brick_get_data(brick);
         frames = g_array_new(FALSE, FALSE, sizeof(ASDFrame));
         for (i = 0; i < nframes; i++) {
@@ -353,8 +345,8 @@ asd_load(const gchar *filename,
             continue;
         }
 
-        gwy_container_pass_object(container, gwy_app_get_brick_key_for_id(id), brick);
-        gwy_container_pass_object(container, gwy_app_get_brick_meta_key_for_id(id), get_meta(&header));
+        gwy_file_pass_volume(file, id, brick);
+        gwy_file_pass_meta(file, GWY_FILE_VOLUME, id, get_meta(&header));
 
         title = gwy_enuml_to_string(datatype,
                                     "Topography", ASD_TOPOGRAPHY,
@@ -362,25 +354,25 @@ asd_load(const gchar *filename,
                                     "Error", ASD_ERROR,
                                     NULL);
         if (*title)
-            gwy_container_set_const_string(container, gwy_app_get_brick_title_key_for_id(id), title);
+            gwy_file_set_title(file, GWY_FILE_VOLUME, id, title, FALSE);
 
-        create_graphs(container, frames, id);
+        create_graphs(file, frames, id, filename);
         g_array_free(frames, TRUE);
 
-        gwy_file_volume_import_log_add(container, id, NULL, filename);
+        gwy_log_add_import(file, GWY_FILE_VOLUME, id, NULL, filename);
     }
 
 fail:
     gwy_file_abandon_contents(buffer, size, NULL);
     asd_header_free(&header);
 
-    return container;
+    return file;
 }
 
 static GwyContainer*
 get_meta(ASDHeader *header)
 {
-    GwyContainer *meta = gwy_container_new();
+    GwyContainer *meta = gwy_container_new_in_construction();
 
     gwy_container_set_string_by_name(meta, "Date and time",
                                      g_strdup_printf("%04u-%02u-%02u %02u:%02u:%02u",
@@ -410,14 +402,14 @@ get_meta(ASDHeader *header)
 }
 
 static void
-create_graphs(GwyContainer *container, GArray *frames, gint id)
+create_graphs(GwyFile *file, GArray *frames, gint id, const gchar *filename)
 {
     GwyGraphModel *gmodel;
     GwyGraphCurveModel *gcmodel;
     ASDFrame *frame;
     guint i, n = frames->len;
     gdouble *storage = g_new(gdouble, 3*n), *idata = storage, *xdata = storage + n, *ydata = storage + 2*n;
-    GwySIUnit *yunit = gwy_si_unit_new("m");
+    GwyUnit *yunit = gwy_unit_new("m");
 
     gwy_math_linspace(idata, frames->len, 0, 1);
     for (i = 0; i < frames->len; i++) {
@@ -431,7 +423,7 @@ create_graphs(GwyContainer *container, GArray *frames, gint id)
                  "title", "Offsets",
                  "axis-label-left", "Offset",
                  "axis-label-bottom", "Frame number",
-                 "si-unit-y", yunit,
+                 "unit-y", yunit,
                  NULL);
 
     gcmodel = gwy_graph_curve_model_new();
@@ -454,7 +446,8 @@ create_graphs(GwyContainer *container, GArray *frames, gint id)
     gwy_graph_model_add_curve(gmodel, gcmodel);
     g_object_unref(gcmodel);
 
-    gwy_container_pass_object(container, gwy_app_get_graph_key_for_id(id+1), gmodel);
+    gwy_file_pass_graph(file, id, gmodel);
+    gwy_log_add_import(file, GWY_FILE_GRAPH, id, NULL, filename);
     g_object_unref(yunit);
     g_free(storage);
 }
