@@ -1,6 +1,7 @@
 /*
- *  $Id: attocube.c 26163 2024-02-06 16:56:55Z yeti-dn $
- *  Copyright (C) 2009 David Necas (Yeti).
+ *  $Id: attocube.c 28911 2025-11-24 18:27:42Z yeti-dn $
+ *  Copyright (C) 2009-2025 David Necas (Yeti).
+ *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
  *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
@@ -47,15 +48,10 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/stats.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -67,12 +63,12 @@
 #define DATA_MAGIC "# Start of Data:"
 #define EXTENSION ".asc"
 
-static gboolean      module_register(void);
-static gint          asc_detect     (const GwyFileDetectInfo *fileinfo,
-                                     gboolean only_name);
-static GwyContainer* asc_load       (const gchar *filename,
-                                     GwyRunType mode,
-                                     GError **error);
+static gboolean module_register(void);
+static gint     detect_file    (const GwyFileDetectInfo *fileinfo,
+                                gboolean only_name);
+static GwyFile* load_file      (const gchar *filename,
+                                GwyRunModeFlags mode,
+                                GError **error);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
@@ -91,17 +87,13 @@ module_register(void)
 {
     gwy_file_func_register("attocube",
                            N_("Attocube ASCII files (.asc)"),
-                           (GwyFileDetectFunc)&asc_detect,
-                           (GwyFileLoadFunc)&asc_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-asc_detect(const GwyFileDetectInfo *fileinfo,
-               gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     if (only_name)
         return g_str_has_suffix(fileinfo->name_lowercase, EXTENSION) ? 10 : 0;
@@ -112,13 +104,14 @@ asc_detect(const GwyFileDetectInfo *fileinfo,
     return 100;
 }
 
-static GwyContainer*
-asc_load(const gchar *filename,
-         G_GNUC_UNUSED GwyRunType mode,
-         GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *container = NULL, *meta;
-    GwyDataField *dfield = NULL;
+    GwyFile *file = NULL;
+    GwyContainer *meta;
+    GwyField *dfield = NULL;
     gchar *line, *p, *value, *header, *buffer = NULL;
     GwyTextHeaderParser parser;
     GHashTable *hash = NULL;
@@ -155,7 +148,7 @@ asc_load(const gchar *filename,
     *p = '\0';
     p += strlen(DATA_MAGIC);
 
-    gwy_clear(&parser, 1);
+    gwy_clear1(parser);
     parser.line_prefix = "#";
     parser.key_value_separator = ":";
     hash = gwy_text_header_parse(header, &parser, NULL, NULL);
@@ -173,52 +166,51 @@ asc_load(const gchar *filename,
     yreal = g_ascii_strtod(g_hash_table_lookup(hash, "y-length"), NULL);
     sanitise_real_size(&yreal, "y-length");
 
-    dfield = gwy_data_field_new(xres, yres, xreal, yreal, FALSE);
+    dfield = gwy_field_new(xres, yres, xreal, yreal, FALSE);
 
     if ((value = g_hash_table_lookup(hash, "x-unit"))) {
         if ((line = g_hash_table_lookup(hash, "y-unit")) && !gwy_strequal(line, value)) {
             g_warning("X and Y units differ, using X");
         }
 
-        gwy_si_unit_set_from_string_parse(gwy_data_field_get_si_unit_xy(dfield), value, &power10);
-        q = pow10(power10);
+        power10 = gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), value);
+        q = gwy_exp10(power10);
         xreal *= q;
         yreal *= q;
-        gwy_data_field_set_xreal(dfield, xreal);
-        gwy_data_field_set_yreal(dfield, yreal);
+        gwy_field_set_xreal(dfield, xreal);
+        gwy_field_set_yreal(dfield, yreal);
     }
     else
         q = 1.0;
 
     if ((value = g_hash_table_lookup(hash, "x-offset")))
-        gwy_data_field_set_xoffset(dfield, q*g_ascii_strtod(value, NULL));
+        gwy_field_set_xoffset(dfield, q*g_ascii_strtod(value, NULL));
     if ((value = g_hash_table_lookup(hash, "y-offset")))
-        gwy_data_field_set_yoffset(dfield, q*g_ascii_strtod(value, NULL));
+        gwy_field_set_yoffset(dfield, q*g_ascii_strtod(value, NULL));
 
     if ((value = g_hash_table_lookup(hash, "z-unit"))) {
-        gwy_si_unit_set_from_string_parse(gwy_data_field_get_si_unit_z(dfield), value, &power10);
-        q = pow10(power10);
+        power10 = gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), value);
+        q = gwy_exp10(power10);
     }
     else
         q = 1.0;
 
-    if (!(gwy_parse_doubles(p, gwy_data_field_get_data(dfield), GWY_PARSE_DOUBLES_FREE_FORM,
+    if (!(gwy_parse_doubles(p, gwy_field_get_data(dfield), GWY_PARSE_DOUBLES_FREE_FORM,
                             &yres, &xres, NULL, &err))) {
         err_PARSE_DOUBLES(error, &err);
-        GWY_OBJECT_UNREF(dfield);
+        g_clear_object(&dfield);
         goto fail;
     }
-    gwy_data_field_multiply(dfield, q);
+    gwy_field_multiply(dfield, q);
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
 
-    gwy_container_pass_object(container, gwy_app_get_data_key_for_id(0), dfield);
+    gwy_file_pass_image(file, 0, dfield);
 
     if ((value = g_hash_table_lookup(hash, "display")))
-        gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(0), value);
+        gwy_file_set_title(file, GWY_FILE_IMAGE, 0, value, FALSE);
 
-    meta = gwy_container_new();
-    gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(0), meta);
+    meta = gwy_container_new_in_construction();
 
     value = g_strdup_printf("%04u-%02u-%02u %02u:%02u:%02u",
                             year, month, day, hour, minute, second);
@@ -233,13 +225,14 @@ asc_load(const gchar *filename,
     if ((value = g_hash_table_lookup(hash, "y-offset")))
         gwy_container_set_const_string_by_name(meta, "Y Offset", value);
 
-    gwy_file_channel_import_log_add(container, 0, NULL, filename);
+    gwy_file_pass_meta(file, GWY_FILE_IMAGE, 0, meta);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
 
 fail:
     g_free(buffer);
     g_hash_table_destroy(hash);
 
-    return container;
+    return file;
 }
 
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
