@@ -1,6 +1,6 @@
 /*
- *  $Id: bcrfile.c 25669 2023-09-14 14:49:06Z yeti-dn $
- *  Copyright (C) 2005 David Necas (Yeti), Petr Klapetek.
+ *  $Id: bcrfile.c 28911 2025-11-24 18:27:42Z yeti-dn $
+ *  Copyright (C) 2005-2025 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -56,19 +56,14 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
 #include <string.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/stats.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -99,34 +94,34 @@
 
 #define HEADER_SIZE_MAGIC "h\000e\000a\000d\000e\000r\000s\000i\000z\000e\000"
 
-static gboolean      module_register     (void);
-static gint          bcrfile_detect      (const GwyFileDetectInfo *fileinfo,
-                                          gboolean only_name);
-static GwyContainer* bcrfile_load        (const gchar *filename,
-                                          GwyRunType mode,
-                                          GError **error);
-static void          find_header_size    (const guchar *buffer,
-                                          gsize *header_size);
-static GwyDataField* file_load_real      (const guchar *buffer,
-                                          gsize size,
-                                          gsize header_size,
-                                          GHashTable *meta,
-                                          GwyDataField **voidmask,
-                                          GError **error);
-static GwyDataField* read_data_field     (const guchar *buffer,
-                                          gint xres,
-                                          gint yres,
-                                          GwyRawDataType rawtype,
-                                          GwyByteOrder byteorder,
-                                          GwyDataField **voidmask);
-static GwyContainer* bcrfile_get_metadata(GHashTable *bcrmeta);
+static gboolean      module_register (void);
+static gint          detect_file     (const GwyFileDetectInfo *fileinfo,
+                                      gboolean only_name);
+static GwyFile*      load_file       (const gchar *filename,
+                                      GwyRunModeFlags mode,
+                                      GError **error);
+static void          find_header_size(const guchar *buffer,
+                                      gsize *header_size);
+static GwyField*     file_load_real  (const guchar *buffer,
+                                      gsize size,
+                                      gsize header_size,
+                                      GHashTable *meta,
+                                      GwyField **voidmask,
+                                      GError **error);
+static GwyField*     read_field      (const guchar *buffer,
+                                      gint xres,
+                                      gint yres,
+                                      GwyRawDataType rawtype,
+                                      GwyByteOrder byteorder,
+                                      GwyField **voidmask);
+static GwyContainer* get_metadata    (GHashTable *bcrmeta);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Image Metrology BCR data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.18",
+    "0.19",
     "David Nečas (Yeti) & Petr Klapetek",
     "2005",
 };
@@ -138,17 +133,13 @@ module_register(void)
 {
     gwy_file_func_register("bcrfile",
                            N_("BCR files (.bcr, .bcrf)"),
-                           (GwyFileDetectFunc)&bcrfile_detect,
-                           (GwyFileLoadFunc)&bcrfile_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-bcrfile_detect(const GwyFileDetectInfo *fileinfo,
-               gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     gint score = 0;
 
@@ -169,17 +160,18 @@ bcrfile_detect(const GwyFileDetectInfo *fileinfo,
     return score;
 }
 
-static GwyContainer*
-bcrfile_load(const gchar *filename,
-             G_GNUC_UNUSED GwyRunType mode,
-             GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *meta, *container = NULL;
+    GwyFile *file = NULL;
+    GwyContainer *meta;
     guchar *buffer = NULL;
     gchar *header;
     gsize header_size, size = 0;
     GError *err = NULL;
-    GwyDataField *dfield = NULL, *voidmask = NULL;
+    GwyField *dfield = NULL, *voidmask = NULL;
     GwyTextHeaderParser parser;
     GHashTable *bcrmeta = NULL;
     gboolean utf16 = FALSE;
@@ -208,7 +200,6 @@ bcrfile_load(const gchar *filename,
             gwy_file_abandon_contents(buffer, size, NULL);
             return NULL;
         }
-
     }
     else {
         header_size = HEADER_SIZE;
@@ -216,7 +207,7 @@ bcrfile_load(const gchar *filename,
         header[header_size-1] = '\0';
     }
 
-    gwy_clear(&parser, 1);
+    gwy_clear1(parser);
     parser.comment_prefix = "#\n%";
     parser.key_value_separator = "=";
     bcrmeta = gwy_text_header_parse(header, &parser, NULL, NULL);
@@ -224,26 +215,24 @@ bcrfile_load(const gchar *filename,
     dfield = file_load_real(buffer + header_size, size, header_size, bcrmeta, &voidmask, error);
     gwy_file_abandon_contents(buffer, size, NULL);
     if (dfield) {
-        container = gwy_container_new();
-        gwy_container_pass_object(container, gwy_app_get_data_key_for_id(0), dfield);
+        file = gwy_file_new_in_construction();
+        gwy_file_pass_image(file, 0, dfield);
 
-        /*********************************************************************/
-        /*gwy_app_set_data_field_title(container, 0, g_strdup(g_strrstr(filename, "/") + 1));*/
-        /***********************************************************************/
-        gwy_app_channel_title_fall_back(container, 0);
+        //gwy_app_set_field_title(file, 0, g_strdup(g_strrstr(filename, "/") + 1));
+        gwy_image_title_fall_back(file, 0);
         if (voidmask)
-            gwy_container_pass_object(container, gwy_app_get_mask_key_for_id(0), voidmask);
+            gwy_file_pass_image_mask(file, 0, voidmask);
 
-        meta = bcrfile_get_metadata(bcrmeta);
+        meta = get_metadata(bcrmeta);
         if (meta)
-            gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(0), meta);
+            gwy_file_pass_meta(file, GWY_FILE_IMAGE, 0, meta);
 
-        gwy_file_channel_import_log_add(container, 0, NULL, filename);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
     }
     g_hash_table_destroy(bcrmeta);
     g_free(header);
 
-    return container;
+    return file;
 }
 
 /* Newer files contain header size embedded in the header.  Change header_size if we find it. */
@@ -279,16 +268,16 @@ find_header_size(const guchar *buffer,
     }
 }
 
-static GwyDataField*
+static GwyField*
 file_load_real(const guchar *buffer,
                gsize size,
                gsize header_size,
                GHashTable *meta,
-               GwyDataField **voidmask,
+               GwyField **voidmask,
                GError **error)
 {
-    GwyDataField *dfield;
-    GwySIUnit *siunit2 = NULL;
+    GwyField *dfield;
+    GwyUnit *siunit2 = NULL;
     gboolean voidpixels = FALSE;
     GwyByteOrder byteorder = GWY_BYTE_ORDER_LITTLE_ENDIAN;
     GwyRawDataType rawtype;
@@ -335,56 +324,56 @@ file_load_real(const guchar *buffer,
 
     /* Do not trust `voidpixels' and always read data assuming the rules for
      * bad pixels. */
-    dfield = read_data_field(buffer, xres, yres, rawtype, byteorder, voidmask);
+    dfield = read_field(buffer, xres, yres, rawtype, byteorder, voidmask);
 
     if ((s = g_hash_table_lookup(meta, "xlength")) && (q = g_ascii_strtod(s, NULL)) > 0) {
         if (!(s = g_hash_table_lookup(meta, "xunit")))
             s = "nm";
 
-        gwy_si_unit_set_from_string_parse(gwy_data_field_get_si_unit_xy(dfield), s, &power10);
-        q *= pow10(power10);
-        gwy_data_field_set_xreal(dfield, q);
+        power10 = gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), s);
+        q *= gwy_exp10(power10);
+        gwy_field_set_xreal(dfield, q);
     }
 
     if ((s = g_hash_table_lookup(meta, "ylength")) && (q = g_ascii_strtod(s, NULL)) > 0) {
         if (!(s = g_hash_table_lookup(meta, "yunit")))
             s = "nm";
 
-        siunit2 = gwy_si_unit_new_parse(s, &power10);
-        q *= pow10(power10);
-        if (!gwy_si_unit_equal(gwy_data_field_get_si_unit_xy(dfield), siunit2))
+        siunit2 = gwy_unit_new_parse(s, &power10);
+        q *= gwy_exp10(power10);
+        if (!gwy_unit_equal(gwy_field_get_unit_xy(dfield), siunit2))
             g_warning("Incompatible x and y units");
         g_object_unref(siunit2);
-        gwy_data_field_set_yreal(dfield, q);
+        gwy_field_set_yreal(dfield, q);
     }
 
     if (!(s = g_hash_table_lookup(meta, "zunit")))
         s = "nm";
-    gwy_si_unit_set_from_string_parse(gwy_data_field_get_si_unit_z(dfield), s, &power10);
-    q = pow10(power10);
+    power10 = gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), s);
+    q = gwy_exp10(power10);
 
     if (rawtype == GWY_RAW_DATA_SINT16) {
         if ((s = g_hash_table_lookup(meta, "bit2nm")) && (qq = g_ascii_strtod(s, NULL)) > 0)
-            gwy_data_field_multiply(dfield, q*qq);
+            gwy_field_multiply(dfield, q*qq);
     }
     else if (rawtype == GWY_RAW_DATA_FLOAT)
-        gwy_data_field_multiply(dfield, q);
+        gwy_field_multiply(dfield, q);
 
     if ((s = g_hash_table_lookup(meta, "zmin")) && (qq = g_ascii_strtod(s, NULL)) > 0)
-        gwy_data_field_add(dfield, q*qq - gwy_data_field_get_min(dfield));
+        gwy_field_add(dfield, q*qq - gwy_field_get_min(dfield));
 
     if (voidpixels) {
-        gwy_data_field_set_xreal(*voidmask, gwy_data_field_get_xreal(dfield));
-        gwy_data_field_set_yreal(*voidmask, gwy_data_field_get_yreal(dfield));
-        gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(*voidmask), gwy_data_field_get_si_unit_xy(dfield));
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(*voidmask), NULL);
+        gwy_field_set_xreal(*voidmask, gwy_field_get_xreal(dfield));
+        gwy_field_set_yreal(*voidmask, gwy_field_get_yreal(dfield));
+        gwy_unit_assign(gwy_field_get_unit_xy(*voidmask), gwy_field_get_unit_xy(dfield));
+        gwy_unit_clear(gwy_field_get_unit_z(*voidmask));
     }
 
     return dfield;
 }
 
 static GwyContainer*
-bcrfile_get_metadata(GHashTable *bcrmeta)
+get_metadata(GHashTable *bcrmeta)
 {
     const struct {
         const gchar *id;
@@ -400,13 +389,11 @@ bcrfile_get_metadata(GHashTable *bcrmeta)
         { "starttime",   NULL,      "Scan time"         },
         /* FIXME: I've seen other stuff, but don't know interpretation */
     };
-    GwyContainer *meta;
-    gchar *value;
-    guint i;
 
-    meta = gwy_container_new();
+    GwyContainer *meta = gwy_container_new_in_construction();
 
-    for (i = 0; i < G_N_ELEMENTS(metakeys); i++) {
+    for (guint i = 0; i < G_N_ELEMENTS(metakeys); i++) {
+        gchar *value;
         if (!(value = g_hash_table_lookup(bcrmeta, metakeys[i].id)))
             continue;
 
@@ -416,32 +403,30 @@ bcrfile_get_metadata(GHashTable *bcrmeta)
             gwy_container_set_const_string_by_name(meta, metakeys[i].key, value);
     }
 
-    if (!gwy_container_get_n_items(meta)) {
-        g_object_unref(meta);
-        meta = NULL;
-    }
+    if (!gwy_container_get_n_items(meta))
+        g_clear_object(&meta);
 
     return meta;
 }
 
-static GwyDataField*
-read_data_field(const guchar *buffer,
-                gint xres,
-                gint yres,
-                GwyRawDataType rawtype,
-                GwyByteOrder byteorder,
-                GwyDataField **voidmask)
+static GwyField*
+read_field(const guchar *buffer,
+           gint xres,
+           gint yres,
+           GwyRawDataType rawtype,
+           GwyByteOrder byteorder,
+           GwyField **voidmask)
 {
-    GwyDataField *dfield;
+    GwyField *dfield;
     gdouble *data, *voids;
     guint i, n = xres*yres;
 
-    dfield = gwy_data_field_new(xres, yres, 1e-6, 1e-6, FALSE);
-    *voidmask = gwy_data_field_new(xres, yres, 1e-6, 1e-6, FALSE);
-    gwy_data_field_fill(*voidmask, 1.0);
+    dfield = gwy_field_new(xres, yres, 1e-6, 1e-6, FALSE);
+    *voidmask = gwy_field_new(xres, yres, 1e-6, 1e-6, FALSE);
+    gwy_field_fill(*voidmask, 1.0);
 
-    data = gwy_data_field_get_data(dfield);
-    voids = gwy_data_field_get_data(*voidmask);
+    data = gwy_field_get_data(dfield);
+    voids = gwy_field_get_data(*voidmask);
     gwy_convert_raw_data(buffer, n, 1, rawtype, byteorder, data, 1.0, 0.0);
     if (rawtype == GWY_RAW_DATA_SINT16) {
         for (i = 0; i < n; i++) {
@@ -457,7 +442,7 @@ read_data_field(const guchar *buffer,
     }
 
     if (!gwy_app_channel_remove_bad_data(dfield, *voidmask))
-        GWY_OBJECT_UNREF(*voidmask);
+        g_clear_object(voidmask);
 
     return dfield;
 }
