@@ -1,6 +1,6 @@
 /*
- *  $Id: burleigh_bii.c 26155 2024-02-06 12:02:49Z yeti-dn $
- *  Copyright (C) 2008 David Necas (Yeti), Petr Klapetek, Jan Horak.
+ *  $Id: burleigh_bii.c 28848 2025-11-10 18:54:08Z yeti-dn $
+ *  Copyright (C) 2008-2025 David Necas (Yeti), Petr Klapetek, Jan Horak.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net, xhorak@gmail.com.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -36,15 +36,9 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/datafield.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils-file.h>
-#include <libprocess/stats.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -62,19 +56,19 @@ enum {
     FILE_IMG_Z_SCALE_POS = 0xCC,
 };
 
-static gboolean      module_register    (void);
-static gint          burleigh_bii_detect(const GwyFileDetectInfo *fileinfo,
-                                         gboolean only_name);
-static GwyContainer* burleigh_bii_load  (const gchar *filename,
-                                         GwyRunType mode,
-                                         GError **error);
+static gboolean module_register(void);
+static gint     detect_file    (const GwyFileDetectInfo *fileinfo,
+                                gboolean only_name);
+static GwyFile* load_file      (const gchar *filename,
+                                GwyRunModeFlags mode,
+                                GError **error);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Burleigh BII binary data files."),
     "Jan Hořák <xhorak@gmail.com>, Yeti <yeti@gwyddion.net>",
-    "0.3",
+    "0.4",
     "Jan Hořák & David Nečas (Yeti) & Petr Klapetek",
     "2008",
 };
@@ -86,17 +80,13 @@ module_register(void)
 {
     gwy_file_func_register("burleigh_bii",
                            N_("Burleigh Image Studio files (.bii)"),
-                           (GwyFileDetectFunc)&burleigh_bii_detect,
-                           (GwyFileLoadFunc)&burleigh_bii_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-burleigh_bii_detect(const GwyFileDetectInfo *fileinfo,
-                    gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     if (only_name)
         return g_str_has_suffix(fileinfo->name_lowercase, EXTENSION) ? 10 : 0;
@@ -114,10 +104,10 @@ burleigh_bii_detect(const GwyFileDetectInfo *fileinfo,
     return 30;
 }
 
-static GwyContainer*
-burleigh_bii_load(const gchar *filename,
-                  G_GNUC_UNUSED GwyRunType mode,
-                  GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
     guchar *buffer = NULL;
     const guchar *p;
@@ -125,8 +115,8 @@ burleigh_bii_load(const gchar *filename,
     GError *err = NULL;
     gint width, height, data_end_offset;
     gdouble x_scale, y_scale, z_scale;
-    GwyDataField *dfield;
-    GwyContainer *container;
+    GwyField *dfield;
+    GwyFile *file;
 
     if (!gwy_file_get_contents(filename, &buffer, &size, &err)) {
         err_GET_FILE_CONTENTS(error, &err);
@@ -137,11 +127,11 @@ burleigh_bii_load(const gchar *filename,
         gwy_file_abandon_contents(buffer, size, NULL);
         return NULL;
     }
-    /* get datafield dimensions from file */
+    /* get field dimensions from file */
     p = buffer + FILE_IMG_SIZE_POS;
     width = gwy_get_gint32_le(&p);
     height = gwy_get_gint32_le(&p);
-    // get file offset where datafield ends
+    // get file offset where field ends
     data_end_offset = width * height * 2 + FILE_IMG_DATA_START_POS;
     // get scaling factors of axes
     if (err_SIZE_MISMATCH(error, data_end_offset + FILE_IMG_Z_SCALE_POS + 8, size, FALSE)) {
@@ -159,25 +149,25 @@ burleigh_bii_load(const gchar *filename,
     //FIXME: all axes always in nm?
     gwy_debug("dfield: w=%d h=%d, scale: x=%g y=%g z=%g",
               width, height, x_scale, y_scale, z_scale);
-    // create new datafield
-    dfield = gwy_data_field_new(width, height, x_scale*Nanometre, y_scale*Nanometre, FALSE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), "m");
+    // create new field
+    dfield = gwy_field_new(width, height, x_scale*Nanometre, y_scale*Nanometre, FALSE);
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
+    gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), "m");
 
-    // set pointer to 16bit signed integers to start position of data definition in file and fill datafield
+    // set pointer to 16bit signed integers to start position of data definition in file and fill field
     gwy_convert_raw_data(buffer + FILE_IMG_DATA_START_POS, width*height, 1,
                          GWY_RAW_DATA_SINT16, GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                         gwy_data_field_get_data(dfield), z_scale*Nanometre, 0.0);
-    gwy_data_field_invert(dfield, TRUE, FALSE, FALSE);
+                         gwy_field_get_data(dfield), z_scale*Nanometre, 0.0);
+    gwy_field_flip(dfield, FALSE, TRUE);
     gwy_file_abandon_contents(buffer, size, NULL);
 
-    // fill container
-    container = gwy_container_new();
-    gwy_container_pass_object(container, gwy_app_get_data_key_for_id(0), dfield);
-    gwy_app_channel_title_fall_back(container, 0);
-    gwy_file_channel_import_log_add(container, 0, NULL, filename);
+    // fill file
+    file = gwy_file_new_in_construction();
+    gwy_file_pass_image(file, 0, dfield);
+    gwy_image_title_fall_back(file, 0);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
 
-    return container;
+    return file;
 }
 
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
