@@ -1,6 +1,6 @@
 /*
- *  $Id: burleigh_exp.c 26163 2024-02-06 16:56:55Z yeti-dn $
- *  Copyright (C) 2007 David Necas (Yeti).
+ *  $Id: burleigh_exp.c 28789 2025-11-04 17:14:03Z yeti-dn $
+ *  Copyright (C) 2007-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -33,15 +33,10 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
 #include <string.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/datafield.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -60,30 +55,30 @@ typedef struct {
     gdouble yscale;
     gdouble zscale;
     gdouble zres;
-    GwySIUnit *xyunits;
-    GwySIUnit *zunits;
+    GwyUnit *xyunits;
+    GwyUnit *zunits;
     gboolean binary;
     guint length;
     guint bpp;
 } BurleighExpHeader;
 
-static gboolean      module_register         (void);
-static gint          burleigh_exp_detect     (const GwyFileDetectInfo *fileinfo,
-                                              gboolean only_name);
-static GwyContainer* burleigh_exp_load       (const gchar *filename,
-                                              GwyRunType mode,
-                                              GError **error);
-static gboolean      burleigh_exp_read_header(BurleighExpHeader *header,
-                                              gchar *buf,
-                                              GError **error);
-static void          free_header             (BurleighExpHeader *header);
+static gboolean module_register(void);
+static gint     detect_file    (const GwyFileDetectInfo *fileinfo,
+                                gboolean only_name);
+static GwyFile* load_file      (const gchar *filename,
+                                GwyRunModeFlags mode,
+                                GError **error);
+static gboolean read_header    (BurleighExpHeader *header,
+                                gchar *buf,
+                                GError **error);
+static void     free_header    (BurleighExpHeader *header);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Burleigh text/bin exported images."),
     "Yeti <yeti@gwyddion.net>",
-    "0.2",
+    "0.3",
     "David Nečas (Yeti)",
     "2007",
 };
@@ -95,17 +90,13 @@ module_register(void)
 {
     gwy_file_func_register("burleigh_exp",
                            N_("Burleigh exported data (.txt, .bin)"),
-                           (GwyFileDetectFunc)&burleigh_exp_detect,
-                           (GwyFileLoadFunc)&burleigh_exp_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-burleigh_exp_detect(const GwyFileDetectInfo *fileinfo,
-                    gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     if (only_name)
         return 0;
@@ -119,17 +110,17 @@ burleigh_exp_detect(const GwyFileDetectInfo *fileinfo,
     return 0;
 }
 
-static GwyContainer*
-burleigh_exp_load(const gchar *filename,
-                  G_GNUC_UNUSED GwyRunType mode,
-                  GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     gchar *buffer = NULL;
     BurleighExpHeader header;
     gsize size = 0;
     GError *err = NULL;
-    GwyDataField *dfield;
+    GwyField *dfield;
     gdouble *data;
     guint i, n;
 
@@ -143,7 +134,7 @@ burleigh_exp_load(const gchar *filename,
         return NULL;
     }
 
-    if (!burleigh_exp_read_header(&header, buffer, error))
+    if (!read_header(&header, buffer, error))
         goto fail;
 
     n = header.xres * header.yres;
@@ -156,8 +147,8 @@ burleigh_exp_load(const gchar *filename,
             goto fail;
     }
 
-    dfield = gwy_data_field_new(header.xres, header.yres, header.xscale, header.yscale, FALSE);
-    data = gwy_data_field_get_data(dfield);
+    dfield = gwy_field_new(header.xres, header.yres, header.xscale, header.yscale, FALSE);
+    data = gwy_field_get_data(dfield);
 
     if (header.binary) {
         gwy_convert_raw_data(buffer + header.length, n, 1, GWY_RAW_DATA_SINT16, GWY_BYTE_ORDER_LITTLE_ENDIAN,
@@ -170,36 +161,36 @@ burleigh_exp_load(const gchar *filename,
             data[i] = strtol(p, &p, 10);
     }
 
-    gwy_data_field_multiply(dfield, header.zscale/32768.0);
+    gwy_field_multiply(dfield, header.zscale/32768.0);
 
     /* Units references released in free_header() */
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(dfield), header.xyunits);
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_z(dfield), header.zunits);
+    gwy_unit_assign(gwy_field_get_unit_xy(dfield), header.xyunits);
+    gwy_unit_assign(gwy_field_get_unit_z(dfield), header.zunits);
 
-    container = gwy_container_new();
-    gwy_container_pass_object(container, gwy_app_get_data_key_for_id(0), dfield);
-    gwy_app_channel_title_fall_back(container, 0);
-    gwy_file_channel_import_log_add(container, 0, NULL, filename);
+    file = gwy_file_new_in_construction();
+    gwy_file_pass_image(file, 0, dfield);
+    gwy_image_title_fall_back(file, 0);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
 
 fail:
     free_header(&header);
     g_free(buffer);
 
-    return container;
+    return file;
 }
 
 static void
 free_header(BurleighExpHeader *header)
 {
-    GWY_OBJECT_UNREF(header->xyunits);
-    GWY_OBJECT_UNREF(header->zunits);
+    g_clear_object(&header->xyunits);
+    g_clear_object(&header->zunits);
 }
 
 static gboolean
 parse_scale(gchar **p,
             const gchar *name,
             double *value,
-            GwySIUnit **units,
+            GwyUnit **units,
             GError **error)
 {
     gint power10;
@@ -227,8 +218,8 @@ parse_scale(gchar **p,
     }
 
     *value = g_ascii_strtod(vp, &vp);
-    *units = gwy_si_unit_new_parse(vp, &power10);
-    *value *= pow10(power10);
+    *units = gwy_unit_new_parse(vp, &power10);
+    *value *= gwy_exp10(power10);
     sanitise_real_size(value, name);
 
     return TRUE;
@@ -271,11 +262,11 @@ parse_dim(gchar **p,
 }
 
 static gboolean
-burleigh_exp_read_header(BurleighExpHeader *header,
-                         gchar *buf,
-                         GError **error)
+read_header(BurleighExpHeader *header,
+            gchar *buf,
+            GError **error)
 {
-    GwySIUnit *yunits = NULL, *dummy = NULL;
+    GwyUnit *yunits = NULL, *dummy = NULL;
     gchar *p, *line;
 
     gwy_clear(header, 1);
