@@ -1,8 +1,8 @@
 /*
- *  $Id: createc.c 27905 2025-05-05 11:07:15Z yeti-dn $
+ *  $Id: createc.c 28789 2025-11-04 17:14:03Z yeti-dn $
  *  Copyright (C) 2004 Rok Zitko.
  *  E-mail: rok.zitko@ijs.si.
- *  Copyright (C) 2009,2012 David Necas (Yeti).
+ *  Copyright (C) 2009-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  Based on nanoscope.c, Copyright (C) 2004 David Necas (Yeti), Petr Klapetek.
@@ -52,17 +52,11 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
 
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/datafield.h>
-#include <libprocess/filters.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 #include "gwyzlib.h"
 #include "err.h"
 
@@ -94,28 +88,28 @@ typedef enum {
     CHANNEL_AUX1       = 1u << 11,
 } CreatecChannelFlags;
 
-static gboolean       module_register       (void);
-static gint           createc_detect        (const GwyFileDetectInfo *fileinfo,
-                                             gboolean only_name);
-static GwyContainer*  createc_load          (const gchar *filename,
-                                             GwyRunType mode,
-                                             GError **error);
-static CreatecVersion createc_get_version   (const gchar *buffer,
-                                             gsize size);
-static GwyDataField*  hash_to_data_field    (GHashTable *hash,
-                                             gint version,
-                                             guint channelbit,
-                                             const gchar *buffer,
-                                             gsize size,
-                                             gsize *offset,
-                                             GError **error);
-static gsize          first_channel_offset  (gint version);
-static guint          channel_bpp           (gint version);
-static void           read_binary_data      (gint n,
-                                             gdouble *data,
-                                             const gchar *buffer,
-                                             gint bpp);
-static GwyContainer*  createc_get_metadata  (GHashTable *hash);
+static gboolean       module_register     (void);
+static gint           detect_file         (const GwyFileDetectInfo *fileinfo,
+                                           gboolean only_name);
+static GwyFile*       load_file           (const gchar *filename,
+                                           GwyRunModeFlags mode,
+                                           GError **error);
+static CreatecVersion createc_get_version (const gchar *buffer,
+                                           gsize size);
+static GwyField*      hash_to_field       (GHashTable *hash,
+                                           gint version,
+                                           guint channelbit,
+                                           const gchar *buffer,
+                                           gsize size,
+                                           gsize *offset,
+                                           GError **error);
+static gsize          first_channel_offset(gint version);
+static guint          channel_bpp         (gint version);
+static void           read_binary_data    (gint n,
+                                           gdouble *data,
+                                           const gchar *buffer,
+                                           gint bpp);
+static GwyContainer*  get_metadata        (GHashTable *hash);
 
 static const GwyEnum versions[] = {
     { "[Parameter]", CREATEC_1,  },
@@ -128,29 +122,25 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports Createc data files."),
     "Rok Zitko <rok.zitko@ijs.si>",
-    "1.3",
+    "2.0",
     "Rok Zitko, David Nečas (Yeti)",
     "2004",
 };
 
-GWY_MODULE_QUERY(module_info)
+GWY_MODULE_QUERY2(module_info, createc)
 
 static gboolean
 module_register(void)
 {
     gwy_file_func_register("createc",
                            N_("Createc files (.dat)"),
-                           (GwyFileDetectFunc)&createc_detect,
-                           (GwyFileLoadFunc)&createc_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-createc_detect(const GwyFileDetectInfo *fileinfo,
-               gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     if (only_name)
         return g_str_has_suffix(fileinfo->name_lowercase, ".dat") ? 10 : 0;
@@ -163,7 +153,7 @@ createc_detect(const GwyFileDetectInfo *fileinfo,
 
 #define createc_atof(x) g_ascii_strtod(x, NULL)
 
-/* Macros to extract integer/double variables in hash_to_data_field() */
+/* Macros to extract integer/double variables in hash_to_field() */
 /* Any missing keyword/value pair is fatal, so we return a NULL pointer. */
 #define HASH_GET(key, var, typeconv, err) \
     if (!(s = g_hash_table_lookup(hash, key))) { \
@@ -192,10 +182,10 @@ createc_detect(const GwyFileDetectInfo *fileinfo,
 #define HASH_DOUBLE2(key1, key2, var, err) HASH_GET2(key1, key2, var, createc_atof, err)
 #define HASH_STRING2(key1, key2, var, err) HASH_GET2(key1, key2, var, /* */, err)
 
-static GwyContainer*
-createc_load(const gchar *filename,
-             G_GNUC_UNUSED GwyRunType mode,
-             GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
     static const GwyEnum channel_names[] = {
         { "Topography", CHANNEL_TOPOGRAPHY, },
@@ -213,7 +203,8 @@ createc_load(const gchar *filename,
         /* Fallback, usually right. */
         { "Voltage",    0,                  },
     };
-    GwyContainer *meta, *container = NULL;
+    GwyFile *file = NULL;
+    GwyContainer *meta;
     guchar *buffer = NULL, *zbuffer = NULL, *imagedata;
     gchar *p, *head;
     gsize size = 0, datasize = 0, offset;
@@ -222,7 +213,7 @@ createc_load(const gchar *filename,
     const gchar *s; /* for HASH_GET macros */
     GwyTextHeaderParser parser;
     GHashTable *hash = NULL;
-    GwyDataField *dfield;
+    GwyField *dfield;
     CreatecVersion version;
     const gchar *title;
 
@@ -243,7 +234,7 @@ createc_load(const gchar *filename,
         ;
 
     /* Lots of lines contain just an equal sign, make them comments. */
-    gwy_clear(&parser, 1);
+    gwy_clear1(parser);
     parser.comment_prefix = "=";
     parser.key_value_separator = "=";
     hash = gwy_text_header_parse(p, &parser, NULL, NULL);
@@ -287,24 +278,24 @@ createc_load(const gchar *filename,
         if (!channelselect || !channelbit)
             channelbit = 1;
 
-        dfield = hash_to_data_field(hash, version, channelbit, imagedata, datasize, &offset, error);
+        dfield = hash_to_field(hash, version, channelbit, imagedata, datasize, &offset, error);
         if (!dfield)
             break;
 
-        if (!container)
-            container = gwy_container_new();
+        if (!file)
+            file = gwy_file_new_in_construction();
 
-        gwy_container_pass_object(container, gwy_app_get_data_key_for_id(id), dfield);
+        gwy_file_pass_image(file, id, dfield);
         title = gwy_enum_to_string(channelbit, channel_names, G_N_ELEMENTS(channel_names));
         if (strlen(title))
-            gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(id), title);
+            gwy_file_set_title(file, GWY_FILE_IMAGE, id, title, FALSE);
         else
-            gwy_app_channel_title_fall_back(container, id);
+            gwy_image_title_fall_back(file, id);
 
-        if ((meta = createc_get_metadata(hash)))
-            gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(id), meta);
+        if ((meta = get_metadata(hash)))
+            gwy_file_pass_meta(file, GWY_FILE_IMAGE, id, meta);
 
-        gwy_file_channel_import_log_add(container, id, NULL, filename);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, id, NULL, filename);
 
         channelbit <<= 1;
     }
@@ -317,7 +308,7 @@ fail:
         g_hash_table_destroy(hash);
     gwy_file_abandon_contents(buffer, size, NULL);
 
-    return container;
+    return file;
 }
 
 static CreatecVersion
@@ -361,14 +352,14 @@ channel_bpp(gint version)
     g_return_val_if_reached(4);
 }
 
-static GwyDataField*
-hash_to_data_field(GHashTable *hash,
-                   gint version,
-                   guint channelbit,
-                   const gchar *buffer,
-                   gsize size,
-                   gsize *offset,
-                   GError **error)
+static GwyField*
+hash_to_field(GHashTable *hash,
+              gint version,
+              guint channelbit,
+              const gchar *buffer,
+              gsize size,
+              gsize *offset,
+              GError **error)
 {
     static const GwyEnum channel_units[] = {
         /* FIXME: I am just putting "V" everywhere I am not sure what the instrument actually does. */
@@ -388,7 +379,7 @@ hash_to_data_field(GHashTable *hash,
         { "V",       0,                  },
     };
 
-    GwyDataField *dfield = NULL;
+    GwyField *dfield = NULL;
     const gchar *zunitstr;
     const gchar *s; /* for HASH_GET macros */
     guint xres, yres, bpp;
@@ -435,15 +426,15 @@ hash_to_data_field(GHashTable *hash,
     q *= 20.0/gwy_powi(2.0, dacbits) * ti2; /* voltage per dac */
     q *= Angstrom * td; /* piezoconstant [A/V] */
 
-    dfield = gwy_data_field_new(xres, yres, xreal, yreal, FALSE);
-    data = gwy_data_field_get_data(dfield);
+    dfield = gwy_field_new(xres, yres, xreal, yreal, FALSE);
+    data = gwy_field_get_data(dfield);
     read_binary_data(xres*yres, data, buffer + *offset, bpp);
     *offset += bpp*xres*yres;
-    gwy_data_field_multiply(dfield, q);
+    gwy_field_multiply(dfield, q);
 
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
     zunitstr = gwy_enum_to_string(channelbit, channel_units, G_N_ELEMENTS(channel_units));
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), strlen(zunitstr) ? zunitstr : "V");
+    gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), strlen(zunitstr) ? zunitstr : "V");
 
 fail:
     g_free(imagedata);
@@ -460,7 +451,7 @@ fail:
     }
 
 static GwyContainer*
-createc_get_metadata(GHashTable *hash)
+get_metadata(GHashTable *hash)
 {
     /* Relocation-less storage */
     static const gchar tobestored[] =
@@ -723,7 +714,7 @@ createc_get_metadata(GHashTable *hash)
     const gchar *ctr;
     gchar *val;
 
-    meta = gwy_container_new();
+    meta = gwy_container_new_in_construction();
 
     for (ctr = tobestored; *ctr; ctr += strlen(ctr) + 1)
         HASH_STORE(ctr);
