@@ -1,6 +1,6 @@
 /*
- *  $Id: dektakvca.c 26873 2024-11-19 12:52:14Z yeti-dn $
- *  Copyright (C) 2017-2018 David Necas (Yeti).
+ *  $Id: dektakvca.c 28789 2025-11-04 17:14:03Z yeti-dn $
+ *  Copyright (C) 2017-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -39,15 +39,10 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/datafield.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/data-browser.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 
 #include "err.h"
 #include "get.h"
@@ -154,22 +149,23 @@ typedef struct {
 } DektakItem;
 
 static gboolean          module_register (void);
-static gint              dektakvca_detect(const GwyFileDetectInfo *fileinfo,
+static gint              detect_file     (const GwyFileDetectInfo *fileinfo,
                                           gboolean only_name);
-static GwyContainer*     dektakvca_load  (const gchar *filename,
-                                          GwyRunType mode,
+static GwyFile*          load_file       (const gchar *filename,
+                                          GwyRunModeFlags mode,
                                           GError **error);
 static gboolean          find_1d_data    (GHashTable *hash,
-                                          GwyContainer *container,
+                                          GwyFile *file,
                                           GString *str,
+                                          const gchar *filename,
                                           GError **error);
 static gboolean          find_2d_data    (GHashTable *hash,
-                                          GwyContainer *container,
+                                          GwyFile *file,
                                           GString *str,
                                           const gchar *filename,
                                           GError **error);
 static GwyContainer*     create_meta     (GHashTable *hash);
-static GwySIUnit*        find_quantity   (GHashTable *hash,
+static GwyUnit*          find_quantity   (GHashTable *hash,
                                           const gchar *key,
                                           GString *str,
                                           gdouble *value,
@@ -191,7 +187,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports Dektak OPDx data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.3",
+    "0.4",
     "David Nečas (Yeti)",
     "2017",
 };
@@ -203,17 +199,13 @@ module_register(void)
 {
     gwy_file_func_register("dektakvca",
                            N_("Dektak OPDx data files (.OPDx)"),
-                           (GwyFileDetectFunc)&dektakvca_detect,
-                           (GwyFileLoadFunc)&dektakvca_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-dektakvca_detect(const GwyFileDetectInfo *fileinfo,
-                 gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     const gchar *head = fileinfo->head;
 
@@ -226,12 +218,12 @@ dektakvca_detect(const GwyFileDetectInfo *fileinfo,
     return 100;
 }
 
-static GwyContainer*
-dektakvca_load(const gchar *filename,
-               G_GNUC_UNUSED GwyRunType mode,
-               GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     guchar *buffer = NULL;
     gsize size = 0;
     GError *err = NULL;
@@ -263,41 +255,40 @@ dektakvca_load(const gchar *filename,
 
     /* Many things have two values, one in measurement settings and the other in the data.  The value in data seems to
      * be the actual one, the value in settings is nominal? */
-    container = gwy_container_new();
-    if (!find_1d_data(hash, container, str, error)) {
-        GWY_OBJECT_UNREF(container);
+    file = gwy_file_new_in_construction();
+    if (!find_1d_data(hash, file, str, filename, error)) {
+        g_clear_object(&file);
         goto fail;
     }
 
-    if (!find_2d_data(hash, container, str, filename, error)) {
-        GWY_OBJECT_UNREF(container);
+    if (!find_2d_data(hash, file, str, filename, error)) {
+        g_clear_object(&file);
         goto fail;
     }
 
-    if (!gwy_container_get_n_items(container)) {
+    if (!gwy_container_get_n_items(GWY_CONTAINER(file))) {
         err_NO_DATA(error);
-        GWY_OBJECT_UNREF(container);
+        g_clear_object(&file);
     }
 
-
 fail:
     g_hash_table_destroy(hash);
     g_string_free(str, TRUE);
     gwy_file_abandon_contents(buffer, size, NULL);
 
-    return container;
+    return file;
 }
 
 /* Only returns FALSE when @error is set.  When it does not seem there are 1D data it returns TRUE; must check if
- * @container has anyhthing in it later. */
+ * @file has anyhthing in it later. */
 static gboolean
-find_1d_data(GHashTable *hash, GwyContainer *container, GString *str,
+find_1d_data(GHashTable *hash, GwyFile *file, GString *str, const gchar *filename,
              GError **error)
 {
     GwyGraphModel *gmodel;
     GwyGraphCurveModel *gcmodel;
     const DektakItem *item, *arrayitem;
-    GwySIUnit *xunit = NULL, *yunit = NULL;
+    GwyUnit *xunit = NULL, *yunit = NULL;
     const guchar *rawydata, *rawxdata = NULL;
     gdouble *xdata, *ydata;
     guint res;
@@ -365,17 +356,18 @@ find_1d_data(GHashTable *hash, GwyContainer *container, GString *str,
     g_free(ydata);
 
     g_object_set(gmodel,
-                 "si-unit-x", xunit,
-                 "si-unit-y", yunit,
+                 "unit-x", xunit,
+                 "unit-y", yunit,
                  "title", str->str,
                  NULL);
-    gwy_container_pass_object(container, gwy_app_get_graph_key_for_id(0), gmodel);
+    gwy_file_pass_graph(file, 0, gmodel);
+    gwy_log_add_import(file, GWY_FILE_GRAPH, 0, NULL, filename);
 
     ok = TRUE;
 
 fail:
-    GWY_OBJECT_UNREF(xunit);
-    GWY_OBJECT_UNREF(yunit);
+    g_clear_object(&xunit);
+    g_clear_object(&yunit);
 
     return ok;
 }
@@ -402,19 +394,18 @@ find_2d_data_matrix(gpointer key, gpointer value, gpointer user_data)
 }
 
 static gboolean
-find_2d_data(GHashTable *hash, GwyContainer *container, GString *str, const gchar *filename,
+find_2d_data(GHashTable *hash, GwyFile *file, GString *str, const gchar *filename,
              GError **error)
 {
-    GwySIUnit *xunit = NULL, *yunit = NULL, *zunit = NULL;
+    GwyUnit *xunit = NULL, *yunit = NULL, *zunit = NULL;
     GwyContainer *meta;
     GPtrArray *channels;
-    GwyDataField *dfield, *mask;
+    GwyField *dfield, *mask;
     const DektakItem *item;
     gboolean ok = FALSE;
     const guchar *rawdata;
-    guint i, len, xres, yres;
+    guint len, xres, yres;
     gdouble xreal, yreal, q = 1.0;
-    GString *s;
 
     channels = g_ptr_array_new();
     g_hash_table_foreach(hash, find_2d_data_matrix, channels);
@@ -423,8 +414,8 @@ find_2d_data(GHashTable *hash, GwyContainer *container, GString *str, const gcha
         return TRUE;
     }
 
-    s = g_string_new(NULL);
-    for (i = 0; i < channels->len; i++) {
+    GString *s = g_string_new(NULL);
+    for (guint i = 0; i < channels->len; i++) {
         g_string_assign(str, "/2D_Data/");
         g_string_append(str, (gchar*)g_ptr_array_index(channels, i));
         len = str->len;
@@ -458,42 +449,41 @@ find_2d_data(GHashTable *hash, GwyContainer *container, GString *str, const gcha
             goto fail;
         gwy_debug("q %g", q);
 
-        if (!gwy_si_unit_equal(xunit, yunit)) {
+        if (!gwy_unit_equal(xunit, yunit)) {
             g_warning("Different x and y units are not representable, ignoring y.");
         }
 
-        dfield = gwy_data_field_new(xres, yres, xreal, yreal, FALSE);
-        gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(dfield), xunit);
-        gwy_si_unit_assign(gwy_data_field_get_si_unit_z(dfield), zunit);
+        dfield = gwy_field_new(xres, yres, xreal, yreal, FALSE);
+        gwy_unit_assign(gwy_field_get_unit_xy(dfield), xunit);
+        gwy_unit_assign(gwy_field_get_unit_z(dfield), zunit);
 
         gwy_convert_raw_data(rawdata, xres*yres, 1, GWY_RAW_DATA_FLOAT, GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                             gwy_data_field_get_data(dfield), q, 0.0);
+                             gwy_field_get_data(dfield), q, 0.0);
 
-        gwy_container_set_object(container, gwy_app_get_data_key_for_id(i), dfield);
-        gwy_file_channel_import_log_add(container, i, NULL, filename);
+        gwy_file_set_image(file, i, dfield);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, i, NULL, filename);
         if ((mask = gwy_app_channel_mask_of_nans(dfield, TRUE)))
-            gwy_container_pass_object(container, gwy_app_get_mask_key_for_id(i), mask);
+            gwy_file_pass_image_mask(file, i, mask);
         g_object_unref(dfield);
 
-        gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(i),
-                                       g_ptr_array_index(channels, i));
-        gwy_app_channel_check_nonsquare(container, i);
+        gwy_file_set_title(file, GWY_FILE_IMAGE, i, g_ptr_array_index(channels, i), FALSE);
+        gwy_check_nonsquare_image(file, i);
 
-        GWY_OBJECT_UNREF(xunit);
-        GWY_OBJECT_UNREF(yunit);
-        GWY_OBJECT_UNREF(zunit);
+        g_clear_object(&xunit);
+        g_clear_object(&yunit);
+        g_clear_object(&zunit);
 
         if ((meta = create_meta(hash)))
-            gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(i), meta);
+            gwy_file_pass_meta(file, GWY_FILE_IMAGE, i, meta);
     }
     ok = TRUE;
 
 fail:
-    GWY_OBJECT_UNREF(xunit);
-    GWY_OBJECT_UNREF(yunit);
-    GWY_OBJECT_UNREF(zunit);
+    g_clear_object(&xunit);
+    g_clear_object(&yunit);
+    g_clear_object(&zunit);
     g_string_free(s, TRUE);
-    for (i = 0; i < channels->len; i++)
+    for (guint i = 0; i < channels->len; i++)
         g_free(g_ptr_array_index(channels, i));
     g_ptr_array_free(channels, TRUE);
     return ok;
@@ -559,14 +549,14 @@ create_meta_item(gpointer key, gpointer value, gpointer user_data)
     }
 
     metakey = gwy_strreplace(name + 10, "/", "::", (gsize)-1);
-    gwy_container_set_string_by_name(meta, metakey, (const guchar*)metavalue);
+    gwy_container_set_string_by_name(meta, metakey, metavalue);
     g_free(metakey);
 }
 
 static GwyContainer*
 create_meta(GHashTable *hash)
 {
-    GwyContainer *meta = gwy_container_new();
+    GwyContainer *meta = gwy_container_new_in_construction();
 
     g_hash_table_foreach(hash, create_meta_item, meta);
     if (gwy_container_get_n_items(meta))
@@ -576,20 +566,20 @@ create_meta(GHashTable *hash)
     return NULL;
 }
 
-static GwySIUnit*
+static GwyUnit*
 find_quantity(GHashTable *hash, const gchar *key, GString *str,
               gdouble *value, GError **error)
 {
     const DektakItem *item;
-    GwySIUnit *unit;
+    GwyUnit *unit;
     gint power10;
 
     if (!(item = find_item(hash, key, DEKTAK_QUANTITY, TRUE, error)))
         return NULL;
     g_string_truncate(str, 0);
     g_string_append_len(str, item->data.qun.symbol.p, item->data.qun.symbol.len);
-    unit = gwy_si_unit_new_parse(str->str, &power10);
-    *value = item->data.qun.value * pow10(power10);
+    unit = gwy_unit_new_parse(str->str, &power10);
+    *value = item->data.qun.value * gwy_exp10(power10);
 
     return unit;
 }
@@ -811,7 +801,7 @@ read_item(const DektakBuf *buf, guint32 *pos, GHashTable *hash, GString *path,
     if (!read_with_check(buf, pos, 1, &typeid))
         goto fail;
 
-    gwy_clear(&item, 1);
+    gwy_clear1(item);
     item.typeid = typeid;
 
     /* Simple types. */
@@ -862,7 +852,7 @@ read_item(const DektakBuf *buf, guint32 *pos, GHashTable *hash, GString *path,
         *pos = buf->len;
     }
     /* Container types.  Cannot tell any difference between these two.  Raw data purpose seems to be wrapping actual
-     * raw data in something container-like. */
+     * raw data in something file-like. */
     else if (typeid == DEKTAK_CONTAINER || typeid == DEKTAK_RAW_DATA || typeid == DEKTAK_RAW_DATA_2D) {
         if (!read_structured(buf, pos, &content))
             goto fail;
@@ -961,7 +951,7 @@ read_item(const DektakBuf *buf, guint32 *pos, GHashTable *hash, GString *path,
     }
 
     gwy_debug("%s (typeid %02x)", path->str, typeid);
-    g_hash_table_insert(hash, g_strndup(path->str, path->len), g_slice_dup(DektakItem, &item));
+    g_hash_table_insert(hash, g_strndup(path->str, path->len), g_memdup2(&item, sizeof(DektakItem)));
     g_string_truncate(path, orig_path_len);
     return TRUE;
 
@@ -978,7 +968,7 @@ dektak_item_free(gpointer p)
     /* Currently the only data type with heap-allocated data. */
     if (item->typeid == DEKTAK_STRING_LIST)
         g_array_free(item->data.strlist, TRUE);
-    g_slice_free(DektakItem, item);
+    g_free(item);
 }
 
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
