@@ -1,6 +1,6 @@
 /*
- *  $Id: dimensionfile.c 26038 2023-11-16 15:55:19Z yeti-dn $
- *  Copyright (C) 2016 David Necas (Yeti).
+ *  $Id: dimensionfile.c 28789 2025-11-04 17:14:03Z yeti-dn $
+ *  Copyright (C) 2016-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -23,14 +23,9 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/datafield.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/data-browser.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 
 #include "err.h"
 #include "get.h"
@@ -56,21 +51,21 @@ enum {
     XYREAL_OFFSET = 0x09df,
 };
 
-static gboolean      module_register(void);
-static gint          dimfile_detect (const GwyFileDetectInfo *fileinfo,
-                                     gboolean only_name);
-static GwyContainer* dimfile_load   (const gchar *filename,
-                                     GwyRunType mode,
-                                     GError **error);
-static gchar**       find_images    (const guchar *buffer,
-                                     guint size);
+static gboolean module_register(void);
+static gint     detect_file    (const GwyFileDetectInfo *fileinfo,
+                                gboolean only_name);
+static GwyFile* load_file      (const gchar *filename,
+                                GwyRunModeFlags mode,
+                                GError **error);
+static gchar**  find_images    (const guchar *buffer,
+                                guint size);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports old Veeco Dimension 3100D files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.1",
+    "0.2",
     "David Nečas (Yeti)",
     "2016",
 };
@@ -82,18 +77,14 @@ module_register(void)
 {
     gwy_file_func_register("dimensionfile",
                            N_("Dimension 3100D files (.001, .002, ...)"),
-                           (GwyFileDetectFunc)&dimfile_detect,
-                           (GwyFileLoadFunc)&dimfile_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
     gwy_file_func_set_is_unfinished("dimensionfile", TRUE);
 
     return TRUE;
 }
 
 static gint
-dimfile_detect(const GwyFileDetectInfo *fileinfo,
-               gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     const gchar *head = fileinfo->head;
 
@@ -117,17 +108,17 @@ dimfile_detect(const GwyFileDetectInfo *fileinfo,
     return 70;
 }
 
-static GwyContainer*
-dimfile_load(const gchar *filename,
-             G_GNUC_UNUSED GwyRunType mode,
-             GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     guchar *buffer = NULL;
     gsize size = 0;
     GError *err = NULL;
-    GwyDataField *dfield = NULL;
-    GwySIUnit *xyunit = NULL;
+    GwyField *dfield = NULL;
+    GwyUnit *xyunit = NULL;
     guint xres, yres, nimages, i, imagesize;
     gdouble xreal, yreal;
     gchar **images = NULL;
@@ -170,13 +161,13 @@ dimfile_load(const gchar *filename,
             break;
     }
     if (i < 16) {
-        xyunit = gwy_si_unit_new_parse(p, &power10);
-        xreal *= pow10(power10);
-        yreal *= pow10(power10);
+        xyunit = gwy_unit_new_parse(p, &power10);
+        xreal *= gwy_exp10(power10);
+        yreal *= gwy_exp10(power10);
     }
     else {
         g_warning("Real size not followed by a unit.  Assuming nm.");
-        xyunit = gwy_si_unit_new("m");
+        xyunit = gwy_unit_new("m");
         xreal *= 1e-9;
         yreal *= 1e-9;
     }
@@ -188,24 +179,24 @@ dimfile_load(const gchar *filename,
     if (err_SIZE_MISMATCH(error, HEADER_SIZE + nimages*imagesize, size, TRUE))
         goto fail;
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     for (i = 0; i < nimages; i++) {
-        dfield = gwy_data_field_new(xres, yres, xreal, yreal, FALSE);
-        gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(dfield), xyunit);
+        dfield = gwy_field_new(xres, yres, xreal, yreal, FALSE);
+        gwy_unit_assign(gwy_field_get_unit_xy(dfield), xyunit);
         gwy_convert_raw_data(buffer + HEADER_SIZE + i*imagesize, xres*yres, 1, GWY_RAW_DATA_SINT16, G_LITTLE_ENDIAN,
-                             gwy_data_field_get_data(dfield), 1.0, 0.0);
-        gwy_container_pass_object(container, gwy_app_get_data_key_for_id(i), dfield);
-        gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(i), images[i]);
-        gwy_app_channel_check_nonsquare(container, i);
-        gwy_file_channel_import_log_add(container, i, NULL, filename);
+                             gwy_field_get_data(dfield), 1.0, 0.0);
+        gwy_file_pass_image(file, i, dfield);
+        gwy_file_set_title(file, GWY_FILE_IMAGE, i, images[i], FALSE);
+        gwy_check_nonsquare_image(file, i);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, i, NULL, filename);
     }
 
 fail:
     g_strfreev(images);
-    GWY_OBJECT_UNREF(xyunit);
+    g_clear_object(&xyunit);
     gwy_file_abandon_contents(buffer, size, NULL);
 
-    return container;
+    return file;
 }
 
 static gchar**
