@@ -1,6 +1,6 @@
 /*
- *  $Id: dm3file.c 28157 2025-06-25 16:02:44Z yeti-dn $
- *  Copyright (C) 2012-2021 David Necas (Yeti).
+ *  $Id: dm3file.c 28789 2025-11-04 17:14:03Z yeti-dn $
+ *  Copyright (C) 2012-2025 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
  *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
@@ -47,15 +47,11 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
 #include <stdarg.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -190,18 +186,18 @@ typedef struct {
 } DM3MetaData;
 
 static gboolean      module_register       (void);
-static gint          dm3_detect            (const GwyFileDetectInfo *fileinfo,
+static gint          dm3file_detect        (const GwyFileDetectInfo *fileinfo,
                                             gboolean only_name);
-static gint          dm4_detect            (const GwyFileDetectInfo *fileinfo,
+static gint          dm4file_detect        (const GwyFileDetectInfo *fileinfo,
                                             gboolean only_name);
-static GwyContainer* dm3_load              (const gchar *filename,
-                                            GwyRunType mode,
+static GwyFile*      dm3file_load          (const gchar *filename,
+                                            GwyRunModeFlags mode,
                                             GError **error);
-static GwyContainer* dm4_load              (const gchar *filename,
-                                            GwyRunType mode,
+static GwyFile*      dm4file_load          (const gchar *filename,
+                                            GwyRunModeFlags mode,
                                             GError **error);
 static DM3ImgResult  dm3_read_image        (DM3File *dm3file,
-                                            GwyContainer *container,
+                                            GwyFile *file,
                                             GwyContainer *meta,
                                             guint i,
                                             guint *id,
@@ -282,7 +278,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Reads Digital Micrograph DM3 and DM4 files."),
     "Yeti <yeti@gwyddion.net>",
-    "2.5",
+    "3.0",
     "David Nečas (Yeti)",
     "2012",
 };
@@ -294,23 +290,16 @@ module_register(void)
 {
     gwy_file_func_register("dm3file",
                            N_("Digital Micrograph DM3 TEM data (.dm3)"),
-                           (GwyFileDetectFunc)&dm3_detect,
-                           (GwyFileLoadFunc)&dm3_load,
-                           NULL,
-                           NULL);
+                           dm3file_detect, dm3file_load, NULL, NULL);
     gwy_file_func_register("dm4file",
                            N_("Digital Micrograph DM4 TEM data (.dm4)"),
-                           (GwyFileDetectFunc)&dm4_detect,
-                           (GwyFileLoadFunc)&dm4_load,
-                           NULL,
-                           NULL);
+                           dm4file_detect, dm4file_load, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-dm3_detect(const GwyFileDetectInfo *fileinfo,
-           gboolean only_name)
+dm3file_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     const guchar *p = fileinfo->head;
     guint version, size, ordering, is_sorted, is_open;
@@ -342,8 +331,7 @@ dm3_detect(const GwyFileDetectInfo *fileinfo,
 }
 
 static gint
-dm4_detect(const GwyFileDetectInfo *fileinfo,
-           gboolean only_name)
+dm4file_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     const guchar *p = fileinfo->head;
     guint version, ordering, is_sorted, is_open;
@@ -375,12 +363,13 @@ dm4_detect(const GwyFileDetectInfo *fileinfo,
     return 100;
 }
 
-static GwyContainer*
-dm3_load(const gchar *filename,
-         G_GNUC_UNUSED GwyRunType mode,
-         GError **error)
+static GwyFile*
+dm3file_load(const gchar *filename,
+             G_GNUC_UNUSED GwyRunModeFlags mode,
+             GError **error)
 {
-    GwyContainer *container = NULL, *meta = NULL;
+    GwyFile *file = NULL;
+    GwyContainer *meta = NULL;
     DM3File dm3file;
     DM3ImgResult status;
     guchar *buffer = NULL;
@@ -393,7 +382,7 @@ dm3_load(const gchar *filename,
         err_GET_FILE_CONTENTS(error, &err);
         return NULL;
     }
-    gwy_clear(&dm3file, 1);
+    gwy_clear1(dm3file);
     p = buffer;
     remaining = size;
 
@@ -410,18 +399,18 @@ dm3_load(const gchar *filename,
     dm3_build_hash(dm3file.hash, &dm3file.root_entry);
     meta = dm3_create_meta(&dm3file);
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     do {
-        status = dm3_read_image(&dm3file, container, meta, i, &id, error);
+        status = dm3_read_image(&dm3file, file, meta, i, &id, error);
         i++;
     } while (status == DM3_IMG_OK || status == DM3_IMG_SKIP);
 
     if (status == DM3_IMG_NOT_FOUND && !id) {
-        GWY_OBJECT_UNREF(container);
+        g_clear_object(&file);
         err_NO_DATA(error);
     }
     else if (status == DM3_IMG_ERROR) {
-        GWY_OBJECT_UNREF(container);
+        g_clear_object(&file);
     }
 
 fail:
@@ -431,17 +420,18 @@ fail:
         dm3file.hash = NULL;
     }
     gwy_file_abandon_contents(buffer, size, NULL);
-    GWY_OBJECT_UNREF(meta);
+    g_clear_object(&meta);
 
-    return container;
+    return file;
 }
 
-static GwyContainer*
-dm4_load(const gchar *filename,
-         G_GNUC_UNUSED GwyRunType mode,
-         GError **error)
+static GwyFile*
+dm4file_load(const gchar *filename,
+             G_GNUC_UNUSED GwyRunModeFlags mode,
+             GError **error)
 {
-    GwyContainer *container = NULL, *meta = NULL;
+    GwyFile *file = NULL;
+    GwyContainer *meta = NULL;
     DM3File dm4file;
     DM3ImgResult status;
     guchar *buffer = NULL;
@@ -454,7 +444,7 @@ dm4_load(const gchar *filename,
         err_GET_FILE_CONTENTS(error, &err);
         return NULL;
     }
-    gwy_clear(&dm4file, 1);
+    gwy_clear1(dm4file);
     p = buffer;
     remaining = size;
 
@@ -471,18 +461,18 @@ dm4_load(const gchar *filename,
     dm3_build_hash(dm4file.hash, &dm4file.root_entry);
     meta = dm3_create_meta(&dm4file);
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     do {
-        status = dm3_read_image(&dm4file, container, meta, i, &id, error);
+        status = dm3_read_image(&dm4file, file, meta, i, &id, error);
         i++;
     } while (status == DM3_IMG_OK || status == DM3_IMG_SKIP);
 
     if (status == DM3_IMG_NOT_FOUND && !id) {
-        GWY_OBJECT_UNREF(container);
+        g_clear_object(&file);
         err_NO_DATA(error);
     }
     else if (status == DM3_IMG_ERROR) {
-        GWY_OBJECT_UNREF(container);
+        g_clear_object(&file);
     }
 
 fail:
@@ -492,14 +482,14 @@ fail:
         dm4file.hash = NULL;
     }
     gwy_file_abandon_contents(buffer, size, NULL);
-    GWY_OBJECT_UNREF(meta);
+    g_clear_object(&meta);
 
-    return container;
+    return file;
 }
 
 static DM3ImgResult
 dm3_read_image(DM3File *dm3file,
-               GwyContainer *container,
+               GwyFile *file,
                GwyContainer *meta,
                guint i,
                guint *id,
@@ -518,12 +508,12 @@ dm3_read_image(DM3File *dm3file,
     GwyRawDataType rawdatatype;
     gdouble xreal, yreal, xoff, yoff;
     guint xres, yres, pixeldepth, itemsize;
-    gchar *xunit = NULL, *yunit = NULL, *title = NULL, *key;
-    GwySIUnit *unit;
+    gchar *xunit = NULL, *yunit = NULL, *title = NULL;
+    GwyUnit *unit;
     gint power10;
     guint nfields = 1, stride = 1, j;
     gboolean is_rgb = FALSE;
-    GwyDataField *field = NULL;
+    GwyField *field = NULL;
 
     if (!dm3_get_uint(dm3file, &xres, res_fmt, i, 0)
         || !dm3_get_uint(dm3file, &yres, res_fmt, i, 1)
@@ -593,23 +583,23 @@ dm3_read_image(DM3File *dm3file,
     if (!gwy_strequal(yunit, xunit))
         g_warning("X and Y units differ, using X");
 
-    unit = gwy_si_unit_new_parse(yunit, &power10);
-    yreal *= pow10(power10);
-    yoff *= pow10(power10);
+    unit = gwy_unit_new_parse(yunit, &power10);
+    yreal *= gwy_exp10(power10);
+    yoff *= gwy_exp10(power10);
     g_object_unref(unit);
 
-    unit = gwy_si_unit_new_parse(xunit, &power10);
-    xreal *= pow10(power10);
-    xoff *= pow10(power10);
+    unit = gwy_unit_new_parse(xunit, &power10);
+    xreal *= gwy_exp10(power10);
+    xoff *= gwy_exp10(power10);
 
     for (j = 0; j < nfields; j++) {
-        field = gwy_data_field_new(xres, yres, xreal*xres, yreal*yres, FALSE);
-        gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(field), unit);
+        field = gwy_field_new(xres, yres, xreal*xres, yreal*yres, FALSE);
+        gwy_unit_assign(gwy_field_get_unit_xy(field), unit);
 
         gwy_convert_raw_data((const guchar*)type->data + itemsize*j, xres*yres, stride, rawdatatype, byteorder,
-                             gwy_data_field_get_data(field), 1.0, 0.0);
+                             gwy_field_get_data(field), 1.0, 0.0);
 
-        gwy_container_pass_object(container, gwy_app_get_data_key_for_id(*id), field);
+        gwy_file_pass_image(file, *id, field);
 
         title = NULL;
         if (dm3_get_string(dm3file, &title, name_fmt, i)) {
@@ -623,18 +613,16 @@ dm3_read_image(DM3File *dm3file,
             title = g_strdup_printf("%s [%s]", title, rgb_channels[j]);
 
         if (title) {
-            key = g_strdup_printf("/%u/data/title", *id);
-            gwy_container_set_string_by_name(container, key, title);
-            g_free(key);
+            gwy_file_pass_title(file, GWY_FILE_IMAGE, *id, title);
             title = NULL;
         }
 
         if (meta) {
-            GwyContainer *mymeta = gwy_container_duplicate(meta);
-            gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(*id), mymeta);
+            GwyContainer *mymeta = gwy_container_copy(meta);
+            gwy_file_pass_meta(file, GWY_FILE_IMAGE, *id, mymeta);
         }
 
-        gwy_file_channel_import_log_add(container, *id, NULL, dm3file->filename);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, *id, NULL, dm3file->filename);
 
         (*id)++;
     }
@@ -747,7 +735,7 @@ dm3_create_meta(DM3File *dm3file)
 {
     DM3MetaData md = { NULL, NULL, NULL };
 
-    md.meta = gwy_container_new();
+    md.meta = gwy_container_new_in_construction();
     md.value = g_string_new(NULL);
     md.dm3file = dm3file;
     g_hash_table_foreach(dm3file->hash, create_meta, &md);
