@@ -1,6 +1,6 @@
 /*
- *  $Id: dmefile.c 25677 2023-09-18 15:38:20Z yeti-dn $
- *  Copyright (C) 2006 David Necas (Yeti), Petr Klapetek.
+ *  $Id: dmefile.c 28848 2025-11-10 18:54:08Z yeti-dn $
+ *  Copyright (C) 2006-2025 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -42,14 +42,9 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/datafield.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 
 #include "err.h"
 #include "get.h"
@@ -113,21 +108,21 @@ typedef struct {
     gdouble height_scale_factor;
 } DMEFile;
 
-static gboolean      module_register(void);
-static gint          dme_detect     (const GwyFileDetectInfo *fileinfo,
-                                     gboolean only_name);
-static GwyContainer* dme_load       (const gchar *filename,
-                                     GwyRunType mode,
-                                     GError **error);
-static void          dme_read_header(const guchar *p,
-                                     DMEFile *dmefile);
+static gboolean module_register(void);
+static gint     detect_file    (const GwyFileDetectInfo *fileinfo,
+                                gboolean only_name);
+static GwyFile* load_file      (const gchar *filename,
+                                GwyRunModeFlags mode,
+                                GError **error);
+static void     dme_read_header(const guchar *p,
+                                DMEFile *dmefile);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Danish Micro Engineering (DME) data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.5.1",
+    "0.6",
     "David Nečas (Yeti) & Petr Klapetek",
     "2006",
 };
@@ -139,17 +134,13 @@ module_register(void)
 {
     gwy_file_func_register("dmefile",
                            N_("DME files (.img)"),
-                           (GwyFileDetectFunc)&dme_detect,
-                           (GwyFileLoadFunc)&dme_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-dme_detect(const GwyFileDetectInfo *fileinfo,
-           gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     if (only_name)
         return g_str_has_suffix(fileinfo->name_lowercase, EXTENSION) ? 15 : 0;
@@ -160,16 +151,16 @@ dme_detect(const GwyFileDetectInfo *fileinfo,
     return 0;
 }
 
-static GwyContainer*
-dme_load(const gchar *filename,
-         G_GNUC_UNUSED GwyRunType mode,
-         GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     guchar *buffer = NULL;
     gsize size = 0;
     GError *err = NULL;
-    GwyDataField *dfield = NULL;
+    GwyField *dfield = NULL;
     guint i, t;
     DMEFile dmefile;
     gdouble *data;
@@ -218,8 +209,8 @@ dme_load(const gchar *filename,
     dmefile.yreal *= dmefile.y_calibration;
     sanitise_real_size(&dmefile.yreal, "y size");
 
-    dfield = gwy_data_field_new(dmefile.xres, dmefile.yres, Angstrom*dmefile.xreal, Angstrom*dmefile.yreal, FALSE);
-    data = gwy_data_field_get_data(dfield);
+    dfield = gwy_field_new(dmefile.xres, dmefile.yres, Angstrom*dmefile.xreal, Angstrom*dmefile.yreal, FALSE);
+    data = gwy_field_get_data(dfield);
     p = buffer + dmefile.header_size + dmefile.xres*dmefile.yres*sizeof(gint16);
     for (i = 0; i < dmefile.yres; i++) {
         t = gwy_get_gint16_le(&p);
@@ -228,24 +219,24 @@ dme_load(const gchar *filename,
                              GWY_RAW_DATA_SINT16, GWY_BYTE_ORDER_LITTLE_ENDIAN,
                              data + i*dmefile.xres, q, 0.0);
     }
-    gwy_data_field_invert(dfield, FALSE, TRUE, FALSE);
+    gwy_field_flip(dfield, TRUE, FALSE);
 
     gwy_file_abandon_contents(buffer, size, NULL);
 
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), "m");
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
+    gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), "m");
 
-    container = gwy_container_new();
-    gwy_container_pass_object(container, gwy_app_get_data_key_for_id(0), dfield);
+    file = gwy_file_new_in_construction();
+    gwy_file_pass_image(file, 0, dfield);
 
     if (dmefile.title[0])
-        gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(0), dmefile.title);
+        gwy_file_set_title(file, GWY_FILE_IMAGE, 0, dmefile.title, FALSE);
     else
-        gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(0), "Topography");
+        gwy_file_set_title(file, GWY_FILE_IMAGE, 0, "Topography", FALSE);
 
-    gwy_file_channel_import_log_add(container, 0, NULL, filename);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
 
-    return container;
+    return file;
 }
 
 static void
