@@ -1,6 +1,6 @@
 /*
- *  $Id: ecsfile.c 26503 2024-08-14 09:12:18Z yeti-dn $
- *  Copyright (C) 2006 David Necas (Yeti), Petr Klapetek, Markus Pristovsek
+ *  $Id: ecsfile.c 28848 2025-11-10 18:54:08Z yeti-dn $
+ *  Copyright (C) 2006-2025 David Necas (Yeti), Petr Klapetek, Markus Pristovsek
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net,
  *  prissi@gift.physik.tu-berlin.de.
  *
@@ -34,14 +34,9 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/datafield.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 
 #include "err.h"
 #include "get.h"
@@ -64,23 +59,23 @@ enum {
     ECS_SCAN_SIZE  = 0x2ec
 };
 
-static gboolean      module_register(void);
-static gint          ecs_detect     (const GwyFileDetectInfo *fileinfo,
-                                     gboolean only_name);
-static GwyContainer* ecs_load       (const gchar *filename,
-                                     GwyRunType mode,
-                                     GError **error);
-static gboolean      get_scan_size  (const gchar *s,
-                                     gdouble *xreal,
-                                     gdouble *q,
-                                     guchar *c);
+static gboolean module_register(void);
+static gint     detect_file    (const GwyFileDetectInfo *fileinfo,
+                                gboolean only_name);
+static GwyFile* load_file      (const gchar *filename,
+                                GwyRunModeFlags mode,
+                                GError **error);
+static gboolean get_scan_size  (const gchar *s,
+                                gdouble *xreal,
+                                gdouble *q,
+                                guchar *c);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports ECS IMG files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.7",
+    "0.8",
     "David Nečas (Yeti) & Petr Klapetek & Markus Pristovsek",
     "2006",
 };
@@ -92,17 +87,13 @@ module_register(void)
 {
     gwy_file_func_register("ecsfile",
                            N_("ECS files (.img)"),
-                           (GwyFileDetectFunc)&ecs_detect,
-                           (GwyFileLoadFunc)&ecs_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-ecs_detect(const GwyFileDetectInfo *fileinfo,
-           gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     guint xres, yres;
     const guchar *p;
@@ -126,16 +117,17 @@ ecs_detect(const GwyFileDetectInfo *fileinfo,
     return 100;
 }
 
-static GwyContainer*
-ecs_load(const gchar *filename,
-         G_GNUC_UNUSED GwyRunType mode,
-         GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *meta, *container = NULL;
+    GwyFile *file = NULL;
+    GwyContainer *meta;
     guchar *buffer = NULL;
     gsize size = 0;
     GError *err = NULL;
-    GwyDataField *dfield = NULL;
+    GwyField *dfield = NULL;
     gchar *s = NULL, *s2 = NULL;
     const guchar *p;
     guint xres, yres;
@@ -194,26 +186,26 @@ ecs_load(const gchar *filename,
 
     /* This does not make much sense when xres != yres, but it is what
      * Snomputz does. */
-    dfield = gwy_data_field_new(xres, yres, xreal, xreal, FALSE);
+    dfield = gwy_field_new(xres, yres, xreal, xreal, FALSE);
     gwy_convert_raw_data(buffer + HEADER_SIZE, xres*yres, 1, GWY_RAW_DATA_SINT16, GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                         gwy_data_field_get_data(dfield), q, 0.0);
-    gwy_data_field_invert(dfield, TRUE, FALSE, FALSE);
+                         gwy_field_get_data(dfield), q, 0.0);
+    gwy_field_flip(dfield, FALSE, TRUE);
 
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), "m");
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
+    gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), "m");
 
-    container = gwy_container_new();
-    gwy_container_set_object(container, gwy_app_get_data_key_for_id(0), dfield);
+    file = gwy_file_new_in_construction();
+    gwy_file_set_image(file, 0, dfield);
 
     /* Channel title */
     p = buffer + ECS_CHANNEL;
     s = get_PASCAL_STRING(&p, HEADER_SIZE - ECS_CHANNEL);
     if (!s || !*s)
         s = g_strdup("Topography");
-    gwy_container_set_string(container, gwy_app_get_data_title_key_for_id(0), s);
+    gwy_file_pass_title(file, GWY_FILE_IMAGE, 0, s);
     s = NULL;
 
-    meta = gwy_container_new();
+    meta = gwy_container_new_in_construction();
 
     /* Date & time */
     p = buffer + ECS_DATE;
@@ -239,18 +231,19 @@ ecs_load(const gchar *filename,
     }
 
     if (gwy_container_get_n_items(meta))
-        gwy_container_set_object(container, gwy_app_get_data_meta_key_for_id(0), meta);
-    g_object_unref(meta);
+        gwy_file_pass_meta(file, GWY_FILE_IMAGE, 0, meta);
+    else
+        g_object_unref(meta);
 
-    gwy_file_channel_import_log_add(container, 0, NULL, filename);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
 
 fail:
     g_free(s);
     g_free(s2);
-    GWY_OBJECT_UNREF(dfield);
+    g_clear_object(&dfield);
     gwy_file_abandon_contents(buffer, size, NULL);
 
-    return container;
+    return file;
 }
 
 static gboolean
