@@ -1,5 +1,5 @@
 /*
- *  $Id: err.h 26168 2024-02-08 10:18:02Z yeti-dn $
+ *  $Id: err.h 27856 2025-04-18 10:21:13Z yeti-dn $
  *  Copyright (C) 2006 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -19,8 +19,8 @@
 
 #include <errno.h>
 #include <glib.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwymodule/gwymodule-file.h>
+#include <libgwyddion/math.h>
+#include <libgwyapp/module-file.h>
 
 /* I/O Errors */
 static inline void
