@@ -1,6 +1,6 @@
 /*
- *  $Id: ezdfile.c 25689 2023-09-20 15:55:12Z yeti-dn $
- *  Copyright (C) 2005 David Necas (Yeti), Petr Klapetek.
+ *  $Id: ezdfile.c 28911 2025-11-24 18:27:42Z yeti-dn $
+ *  Copyright (C) 2005-2025 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -45,15 +45,10 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <libprocess/datafield.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 
 #include "err.h"
 #include "get.h"
@@ -99,39 +94,39 @@ typedef struct {
     GError *firsterror;
 } EZDSection;
 
-static gboolean      module_register       (void);
-static gint          ezdfile_detect        (const GwyFileDetectInfo *fileinfo,
-                                            gboolean only_name);
-static GwyContainer* ezdfile_load          (const gchar *filename,
-                                            GwyRunType mode,
-                                            GError **error);
-static guint         find_data_start       (const guchar *buffer,
-                                            gsize size);
-static void          ezdfile_free          (GPtrArray *ezdfile);
-static void          ezdsection_free       (EZDSection *section);
-static GwyDataField* read_data_field       (EZDSection *section,
-                                            GError **error);
-static gboolean      file_read_header      (GPtrArray *ezdfile,
-                                            gchar *buffer,
-                                            GError **error);
-static guint         find_data_offsets     (const gchar *buffer,
-                                            gsize size,
-                                            GPtrArray *ezdfile,
-                                            GError **error);
-static GwyContainer* ezdfile_get_metadata  (GPtrArray *ezdfile,
-                                            gint idx);
-static void          fix_scales            (EZDSection *section,
-                                            gint idx,
-                                            GwyContainer *container);
-static void          check_section_ranges  (gpointer data,
-                                            gpointer user_data);
+static gboolean      module_register     (void);
+static gint          detect_file         (const GwyFileDetectInfo *fileinfo,
+                                          gboolean only_name);
+static GwyFile*      load_file           (const gchar *filename,
+                                          GwyRunModeFlags mode,
+                                          GError **error);
+static guint         find_data_start     (const guchar *buffer,
+                                          gsize size);
+static void          ezdfile_free        (GPtrArray *ezdfile);
+static void          ezdsection_free     (EZDSection *section);
+static GwyField*     read_field          (EZDSection *section,
+                                          GError **error);
+static gboolean      file_read_header    (GPtrArray *ezdfile,
+                                          gchar *buffer,
+                                          GError **error);
+static guint         find_data_offsets   (const gchar *buffer,
+                                          gsize size,
+                                          GPtrArray *ezdfile,
+                                          GError **error);
+static GwyContainer* ezdfile_get_metadata(GPtrArray *ezdfile,
+                                          gint idx);
+static void          fix_scales          (EZDSection *section,
+                                          gint idx,
+                                          GwyFile *file);
+static void          check_section_ranges(gpointer data,
+                                          gpointer user_data);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Nanosurf EZD and NID data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.9",
+    "0.10",
     "David Nečas (Yeti) & Petr Klapetek",
     "2005",
 };
@@ -143,17 +138,13 @@ module_register(void)
 {
     gwy_file_func_register("ezdfile",
                            N_("Nanosurf files (.ezd, .nid)"),
-                           (GwyFileDetectFunc)&ezdfile_detect,
-                           (GwyFileLoadFunc)&ezdfile_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-ezdfile_detect(const GwyFileDetectInfo *fileinfo,
-               gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     gint score = 0;
 
@@ -168,17 +159,18 @@ ezdfile_detect(const GwyFileDetectInfo *fileinfo,
     return score;
 }
 
-static GwyContainer*
-ezdfile_load(const gchar *filename,
-             G_GNUC_UNUSED GwyRunType mode,
-             GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *meta, *container = NULL;
+    GwyFile *file = NULL;
+    GwyContainer *meta;
     guchar *buffer = NULL;
     gsize size = 0;
     GError *err = NULL;
     EZDSection *section = NULL;
-    GwyDataField *dfield = NULL;
+    GwyField *dfield = NULL;
     GPtrArray *ezdfile;
     guint header_size, n;
     gint i;
@@ -212,7 +204,7 @@ ezdfile_load(const gchar *filename,
     /* fix for ranges equal to zero */
     g_ptr_array_foreach(ezdfile, &check_section_ranges, NULL);
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     i = 0;
     for (n = 0; n < ezdfile->len; n++) {
         section = (EZDSection*)g_ptr_array_index(ezdfile, n);
@@ -220,22 +212,22 @@ ezdfile_load(const gchar *filename,
             continue;
         if (err_DIMENSION(NULL, section->xres) || err_DIMENSION(NULL, section->yres))
             continue;
-        if (!(dfield = read_data_field(section, NULL)))
+        if (!(dfield = read_field(section, NULL)))
             continue;
-        gwy_container_pass_object(container, gwy_app_get_data_key_for_id(i), dfield);
-        fix_scales(section, i, container);
+        gwy_file_pass_image(file, i, dfield);
+        fix_scales(section, i, file);
 
         meta = ezdfile_get_metadata(ezdfile, n);
         if (meta)
-            gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(i), meta);
+            gwy_file_pass_meta(file, GWY_FILE_IMAGE, i, meta);
 
-        gwy_file_channel_import_log_add(container, i, NULL, filename);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, i, NULL, filename);
         i++;
     }
     gwy_file_abandon_contents(buffer, size, NULL);
     ezdfile_free(ezdfile);
 
-    return container;
+    return file;
 }
 
 static guint
@@ -490,28 +482,24 @@ find_data_offsets(const gchar *buffer,
 static void
 fix_scales(EZDSection *section,
            gint idx,
-           GwyContainer *container)
+           GwyFile *file)
 {
-    GwyDataField *dfield;
-    GwySIUnit *siunit;
+    GwyField *dfield = gwy_file_get_image(file, idx);
     gint power10;
-    gdouble r;
-
-    dfield = GWY_DATA_FIELD(gwy_container_get_object(container, gwy_app_get_data_key_for_id(idx)));
 
     /* Fix value scale */
-    gwy_si_unit_set_from_string_parse(gwy_data_field_get_si_unit_z(dfield), section->zrange.unit, &power10);
-    r = pow10(power10);
-    gwy_data_field_multiply(dfield, r*section->zrange.range);
-    gwy_data_field_add(dfield, r*section->zrange.min);
+    power10 = gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), section->zrange.unit);
+    gdouble r = gwy_exp10(power10);
+    gwy_field_multiply(dfield, r*section->zrange.range);
+    gwy_field_add(dfield, r*section->zrange.min);
 
     /* Fix lateral scale */
-    gwy_si_unit_set_from_string_parse(gwy_data_field_get_si_unit_xy(dfield), section->xrange.unit, &power10);
-    gwy_data_field_set_xreal(dfield, pow10(power10)*section->xrange.range);
+    power10 = gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), section->xrange.unit);
+    gwy_field_set_xreal(dfield, gwy_exp10(power10)*section->xrange.range);
 
-    siunit = gwy_si_unit_new_parse(section->yrange.unit, &power10);
-    gwy_data_field_set_yreal(dfield, pow10(power10)*section->yrange.range);
-    g_object_unref(siunit);
+    GwyUnit *unit = gwy_unit_new_parse(section->yrange.unit, &power10);
+    gwy_field_set_yreal(dfield, gwy_exp10(power10)*section->yrange.range);
+    g_object_unref(unit);
 
     /* Some metadata */
     if (section->zrange.name) {
@@ -530,15 +518,12 @@ fix_scales(EZDSection *section,
             s = "";
             break;
         }
-        gwy_container_set_string(container, gwy_app_get_data_title_key_for_id(idx),
-                                 g_strdup_printf("%s%s", section->zrange.name, s));
+        gwy_file_pass_title(file, GWY_FILE_IMAGE, idx, g_strdup_printf("%s%s", section->zrange.name, s));
     }
 }
 
 static void
-store_meta(gpointer key,
-           gpointer value,
-           gpointer user_data)
+store_meta(gpointer key, gpointer value, gpointer user_data)
 {
     GwyContainer *meta = (GwyContainer*)user_data;
     gchar *cval;
@@ -556,7 +541,7 @@ ezdfile_get_metadata(GPtrArray *ezdfile,
     GwyContainer *meta;
     guint i;
 
-    meta = gwy_container_new();
+    meta = gwy_container_new_in_construction();
 
     for (i = 0; i < ezdfile->len; i++) {
         section = (EZDSection*)g_ptr_array_index(ezdfile, i);
@@ -572,12 +557,12 @@ ezdfile_get_metadata(GPtrArray *ezdfile,
     return meta;
 }
 
-static GwyDataField*
-read_data_field(EZDSection *section,
-                GError **error)
+static GwyField*
+read_field(EZDSection *section,
+           GError **error)
 {
     GwyRawDataType rawtype;
-    GwyDataField *field;
+    GwyField *field;
     gdouble q, z0;
 
     if (section->bitdepth == 8)
@@ -592,12 +577,12 @@ read_data_field(EZDSection *section,
     }
 
     g_assert(section->data);
-    field = gwy_data_field_new(section->xres, section->yres, 1.0, 1.0, FALSE);
+    field = gwy_field_new(section->xres, section->yres, 1.0, 1.0, FALSE);
     q = gwy_powi(2.0, section->bitdepth);
     z0 = q/2.0;
     gwy_convert_raw_data(section->data, section->xres*section->yres, 1, rawtype, GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                         gwy_data_field_get_data(field), 1.0/q, z0/q);
-    gwy_data_field_invert(field, TRUE, FALSE, FALSE);
+                         gwy_field_get_data(field), 1.0/q, z0/q);
+    gwy_field_flip(field, FALSE, TRUE);
 
     return field;
 }
