@@ -1,6 +1,6 @@
 /*
- *  $Id: femtoscan.c 26873 2024-11-19 12:52:14Z yeti-dn $
- *  Copyright (C) 2014 David Necas (Yeti).
+ *  $Id: femtoscan.c 28848 2025-11-10 18:54:08Z yeti-dn $
+ *  Copyright (C) 2014-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -38,14 +38,10 @@
  * Read
  **/
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/datafield.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -54,29 +50,29 @@
 
 #define DATA_LENGTH "\\Data length:"
 
-static gboolean      module_register   (void);
-static gint          femtoscan_detect  (const GwyFileDetectInfo *fileinfo,
-                                        gboolean only_name);
-static GwyContainer* femtoscan_load    (const gchar *filename,
-                                        GwyRunType mode,
-                                        GError **error);
-static GwyDataField* hash_to_data_field(GHashTable *hash,
-                                        gchar *buffer,
-                                        gsize size,
-                                        GError **error);
-static GwyContainer* get_metadata      (GHashTable *hash,
-                                        GList *globals);
-static gsize         header_length     (const guchar *buffer,
-                                        gsize size);
-static GList*        parse_header      (gchar *buffer,
-                                        GError **error);
+static gboolean      module_register(void);
+static gint          detect_file    (const GwyFileDetectInfo *fileinfo,
+                                     gboolean only_name);
+static GwyFile*      load_file      (const gchar *filename,
+                                     GwyRunModeFlags mode,
+                                     GError **error);
+static GwyField*     hash_to_field  (GHashTable *hash,
+                                     gchar *buffer,
+                                     gsize size,
+                                     GError **error);
+static GwyContainer* get_metadata   (GHashTable *hash,
+                                     GList *globals);
+static gsize         header_length  (const guchar *buffer,
+                                     gsize size);
+static GList*        parse_header   (gchar *buffer,
+                                     GError **error);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports FemtoScan SPM data files."),
     "Yeti <yeti@gwyddion.net>",
-    "1.0",
+    "2.0",
     "David Nečas (Yeti)",
     "2014",
 };
@@ -88,17 +84,13 @@ module_register(void)
 {
     gwy_file_func_register("femtoscan",
                            N_("FemtoScan SPM files"),
-                           (GwyFileDetectFunc)&femtoscan_detect,
-                           (GwyFileLoadFunc)&femtoscan_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-femtoscan_detect(const GwyFileDetectInfo *fileinfo,
-                 gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     gint score = 0;
 
@@ -111,12 +103,12 @@ femtoscan_detect(const GwyFileDetectInfo *fileinfo,
     return score;
 }
 
-static GwyContainer*
-femtoscan_load(const gchar *filename,
-               G_GNUC_UNUSED GwyRunType mode,
-               GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     GError *err = NULL;
     guchar *buffer = NULL;
     G_GNUC_UNUSED const gchar *self;
@@ -139,11 +131,11 @@ femtoscan_load(const gchar *filename,
     header[headerlen] = '\0';
     all_sections = parse_header(header, error);
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
 
     for (l = all_sections; l; l = g_list_next(l)) {
         GHashTable *hash = (GHashTable*)l->data;
-        GwyDataField *dfield;
+        GwyField *dfield;
 
         self = g_hash_table_lookup(hash, "#self");
         gwy_debug("processing section %s", self);
@@ -156,22 +148,20 @@ femtoscan_load(const gchar *filename,
         }
         images = g_list_append(images, hash);
 
-        dfield = hash_to_data_field(hash, buffer, size, err ? NULL : &err);
+        dfield = hash_to_field(hash, buffer, size, err ? NULL : &err);
         if (!dfield)
             continue;
 
-        gwy_container_pass_object(container, gwy_app_get_data_key_for_id(id), dfield);
-        gwy_file_channel_import_log_add(container, id, NULL, filename);
-        gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(id),
-                                       (gchar*)g_hash_table_lookup(hash, "Image data"));
+        gwy_file_pass_image(file, id, dfield);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, id, NULL, filename);
+        gwy_file_set_title(file, GWY_FILE_IMAGE, id, (gchar*)g_hash_table_lookup(hash, "Image data"), FALSE);
 
         id++;
     }
 
     id = 0;
     for (l = images; l; l = g_list_next(l)) {
-        gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(id),
-                                  get_metadata((GHashTable*)l->data, globals));
+        gwy_file_pass_meta(file, GWY_FILE_IMAGE, id, get_metadata((GHashTable*)l->data, globals));
         id++;
     }
 
@@ -184,14 +174,14 @@ femtoscan_load(const gchar *filename,
     gwy_file_abandon_contents(buffer, size, NULL);
 
     if (!id) {
-        GWY_OBJECT_UNREF(container);
+        g_clear_object(&file);
         if (err)
             g_propagate_error(error, err);
         else
             err_NO_DATA(error);
     }
 
-    return container;
+    return file;
 }
 
 static void
@@ -232,7 +222,7 @@ get_metadata(GHashTable *hash,
     const gchar *self;
     GList *l;
 
-    meta = gwy_container_new();
+    meta = gwy_container_new_in_construction();
     self = g_hash_table_lookup(hash, "#self");
     g_object_set_data(G_OBJECT(meta), "prefix", (gpointer)self);
     g_hash_table_foreach(hash, add_metadata, meta);
@@ -249,14 +239,14 @@ get_metadata(GHashTable *hash,
     return meta;
 }
 
-static GwyDataField*
-hash_to_data_field(GHashTable *hash,
-                   gchar *buffer,
-                   gsize size,
-                   GError **error)
+static GwyField*
+hash_to_field(GHashTable *hash,
+              gchar *buffer,
+              gsize size,
+              GError **error)
 {
-    GwyDataField *dfield;
-    GwySIUnit *unitz, *unitx, *unity;
+    GwyField *dfield;
+    GwyUnit *unitz, *unitx, *unity;
     gchar *val, *end;
     gint xres, yres, power10, bpp = 2;
     gsize offset, datalen;
@@ -287,8 +277,8 @@ hash_to_data_field(GHashTable *hash,
         return NULL;
     }
     val = end;
-    unitx = gwy_si_unit_new_parse(val, &power10);
-    xreal *= pow10(power10);
+    unitx = gwy_unit_new_parse(val, &power10);
+    xreal *= gwy_exp10(power10);
     gwy_debug("xreal = %g", xreal);
 
     val = g_hash_table_lookup(hash, "Scan size Y");
@@ -299,14 +289,14 @@ hash_to_data_field(GHashTable *hash,
         return NULL;
     }
     val = end;
-    unity = gwy_si_unit_new_parse(val, &power10);
-    yreal *= pow10(power10);
+    unity = gwy_unit_new_parse(val, &power10);
+    yreal *= gwy_exp10(power10);
     gwy_debug("yreal = %g", yreal);
 
-    if (!gwy_si_unit_equal(unitx, unity)) {
+    if (!gwy_unit_equal(unitx, unity)) {
         g_warning("X and Y units differ, using X.");
     }
-    GWY_OBJECT_UNREF(unity);
+    g_clear_object(&unity);
 
     val = g_hash_table_lookup(hash, "Z scale");
     zscale = g_ascii_strtod(val, &end);
@@ -316,19 +306,19 @@ hash_to_data_field(GHashTable *hash,
         return NULL;
     }
     val = end;
-    unitz = gwy_si_unit_new_parse(val, &power10);
-    zscale *= pow10(power10);
+    unitz = gwy_unit_new_parse(val, &power10);
+    zscale *= gwy_exp10(power10);
     gwy_debug("zscale = %g", zscale);
 
-    dfield = gwy_data_field_new(xres, yres, xreal, yreal, FALSE);
-    data = gwy_data_field_get_data(dfield);
+    dfield = gwy_field_new(xres, yres, xreal, yreal, FALSE);
+    data = gwy_field_get_data(dfield);
     gwy_convert_raw_data(buffer + offset, xres*yres, 1, GWY_RAW_DATA_SINT16, GWY_BYTE_ORDER_LITTLE_ENDIAN,
                          data, zscale/65536.0, 0.0);
-    gwy_data_field_invert(dfield, TRUE, FALSE, FALSE);
+    gwy_field_flip(dfield, FALSE, TRUE);
 
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(dfield), unitx);
+    gwy_unit_assign(gwy_field_get_unit_xy(dfield), unitx);
     g_object_unref(unitx);
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_z(dfield), unitz);
+    gwy_unit_assign(gwy_field_get_unit_z(dfield), unitz);
     g_object_unref(unitz);
 
     return dfield;
