@@ -1,6 +1,6 @@
 /*
- *  $Id: fitsfile.c 26873 2024-11-19 12:52:14Z yeti-dn $
- *  Copyright (C) 2014 David Necas (Yeti).
+ *  $Id: fitsfile.c 28911 2025-11-24 18:27:42Z yeti-dn $
+ *  Copyright (C) 2014-2025 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
  *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
@@ -27,17 +27,12 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 
 #include <string.h>
 #include <stdlib.h>
 #include <fitsio.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/stats.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/data-browser.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -59,24 +54,24 @@ enum {
     FITS_BLOCK_LEN = 80
 };
 
-static gboolean      module_register    (void);
-static gint          fits_detect        (const GwyFileDetectInfo *fileinfo,
-                                         gboolean only_name);
-static GwyContainer* fits_load          (const gchar *filename,
-                                         GwyRunType mode,
-                                         GError **error);
-static gboolean      get_real_and_offset(fitsfile *fptr,
-                                         gint i,
-                                         guint res,
-                                         gdouble *real,
-                                         gdouble *off);
+static gboolean module_register    (void);
+static gint     detect_file        (const GwyFileDetectInfo *fileinfo,
+                                    gboolean only_name);
+static GwyFile* load_file          (const gchar *filename,
+                                    GwyRunModeFlags mode,
+                                    GError **error);
+static gboolean get_real_and_offset(fitsfile *fptr,
+                                    gint i,
+                                    guint res,
+                                    gdouble *real,
+                                    gdouble *off);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Reads Flexible Image Transport System (FITS) files."),
     "Yeti <yeti@gwyddion.net>",
-    "1.1",
+    "2.0",
     "David Nečas (Yeti)",
     "2014",
 };
@@ -88,17 +83,13 @@ module_register(void)
 {
     gwy_file_func_register("fitsfile",
                            N_("Flexible Image Transport System FITS (.fits)"),
-                           (GwyFileDetectFunc)&fits_detect,
-                           (GwyFileLoadFunc)&fits_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-fits_detect(const GwyFileDetectInfo *fileinfo,
-            gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     if (only_name) {
         if (g_str_has_suffix(fileinfo->name_lowercase, EXTENSION1))
@@ -130,14 +121,14 @@ err_FITS(GError **error, gint status)
                 buf);
 }
 
-static GwyContainer*
-fits_load(const gchar *filename,
-          G_GNUC_UNUSED GwyRunType mode,
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
           GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     fitsfile *fptr = NULL;
-    GwyDataField *field = NULL, *mask;
+    GwyField *field = NULL, *mask;
     gint status = 0;   /* Must be initialised to zero! */
     gint hdutype, naxis, anynull, nkeys, k;
     glong res[3];    /* First index is the fast looping one. */
@@ -189,16 +180,17 @@ fits_load(const gchar *filename,
     if (err_DIMENSION(error, res[0]) || err_DIMENSION(error, res[1]))
         goto fail;
 
-    field = gwy_data_field_new(res[0], res[1], res[0], res[1], FALSE);
+    field = gwy_field_new(res[0], res[1], res[0], res[1], FALSE);
     invalid = g_new(gchar, res[0]*res[1]);
-    if (fits_read_imgnull(fptr, TDOUBLE, 1, res[0]*res[1], field->data, invalid, &anynull, &status)) {
+    if (fits_read_imgnull(fptr, TDOUBLE, 1, res[0]*res[1], gwy_field_get_data(field), invalid,
+                          &anynull, &status)) {
         err_FITS(error, status);
         goto fail;
     }
 
-    container = gwy_container_new();
-    gwy_container_set_object(container, gwy_app_get_data_key_for_id(0), field);
-    gwy_file_channel_import_log_add(container, 0, NULL, filename);
+    file = gwy_file_new_in_construction();
+    gwy_file_set_image(file, 0, field);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
 
     /* Failures here are non-fatal.  We already have an image. */
     if (fits_get_hdrspace(fptr, &nkeys, NULL, &status)) {
@@ -210,9 +202,9 @@ fits_load(const gchar *filename,
         gint power10;
 
         gwy_debug("BUINT = <%s>", strvalue);
-        gwy_si_unit_set_from_string_parse(gwy_data_field_get_si_unit_z(field), strvalue, &power10);
+        power10 = gwy_unit_set_from_string(gwy_field_get_unit_z(field), strvalue);
         if (power10)
-            gwy_data_field_multiply(field, pow10(power10));
+            gwy_field_multiply(field, gwy_exp10(power10));
     }
     status = 0;
 
@@ -220,36 +212,38 @@ fits_load(const gchar *filename,
         if (real < 0.0) {
             off += real;
             real = -real;
-            gwy_data_field_invert(field, FALSE, TRUE, FALSE);
+            gwy_field_flip(field, TRUE, FALSE);
         }
-        gwy_data_field_set_xreal(field, real);
-        gwy_data_field_set_xoffset(field, off);
+        gwy_field_set_xreal(field, real);
+        gwy_field_set_xoffset(field, off);
     }
 
     if (get_real_and_offset(fptr, 2, res[1], &real, &off)) {
         if (real < 0.0) {
             off += real;
             real = -real;
-            gwy_data_field_invert(field, TRUE, FALSE, FALSE);
+            gwy_field_flip(field, FALSE, TRUE);
         }
-        gwy_data_field_set_yreal(field, real);
-        gwy_data_field_set_yoffset(field, off);
+        gwy_field_set_yreal(field, real);
+        gwy_field_set_yoffset(field, off);
     }
 
     /* Create a mask of invalid data. */
-    for (k = 0; k < field->xres*field->yres; k++) {
+    gdouble *d = gwy_field_get_data(field);
+    guint n = field->xres * field->yres;
+    for (k = 0; k < n; k++) {
         if (invalid[k])
-            field->data[k] = NAN;
+            d[k] = NAN;
     }
     if ((mask = gwy_app_channel_mask_of_nans(field, TRUE)))
-        gwy_container_pass_object(container, gwy_app_get_mask_key_for_id(0), mask);
+        gwy_file_pass_image_mask(file, 0, mask);
 
 fail:
     fits_close_file(fptr, &status);
-    GWY_OBJECT_UNREF(field);
+    g_clear_object(&field);
     g_free(invalid);
 
-    return container;
+    return file;
 }
 
 static gboolean
