@@ -1,6 +1,6 @@
 /*
- *  $Id: formats3d.c 25690 2023-09-21 08:05:30Z yeti-dn $
- *  Copyright (C) 2009-2020 David Necas (Yeti).
+ *  $Id: formats3d.c 28822 2025-11-06 16:01:09Z yeti-dn $
+ *  Copyright (C) 2009-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -63,16 +63,11 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <math.h>
 #include <string.h>
 #include <glib/gstdio.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <libprocess/stats.h>
-#include <libgwydgets/gwyradiobuttons.h>
-#include <libgwydgets/gwydgetutils.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -128,16 +123,16 @@ typedef struct {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
+    GwyField *field;
     /* Cached input data properties. */
     ScalingInfo scinfo;
-    const guchar *title;
+    gchar *title;
 } ExportArgs;
 
 typedef struct {
     GwyParams *params;
     GwySurface *surface;
-    GwyDataField *image;
+    GwyField *image;
 } ImportArgs;
 
 typedef gboolean (*Export3DFunc)(FILE *fh,
@@ -165,31 +160,28 @@ typedef struct {
     const gchar *name;
     const gchar *title;
     const gchar *description;
-    GwyFileDetectFunc detect;
-    Export3DFunc export_;
-    Import3DFunc import;
+    GwyFileDetectFunc detect_file;
+    Export3DFunc export_file;
+    Import3DFunc import_file;
     gboolean has_triang_none;
     gboolean is_binary;
 } Format3D;
 
 static gboolean         module_register       (void);
 static gint             detect3d_vtk          (const GwyFileDetectInfo *fileinfo,
-                                               gboolean only_name,
-                                               const gchar *name);
+                                               gboolean only_name);
 static gboolean         export3d_vtk          (FILE *fh,
                                                GArray *vertices,
                                                GArray *triangles,
                                                ExportArgs *args);
 static gint             detect3d_ply          (const GwyFileDetectInfo *fileinfo,
-                                               gboolean only_name,
-                                               const gchar *name);
+                                               gboolean only_name);
 static gboolean         export3d_ply          (FILE *fh,
                                                GArray *vertices,
                                                GArray *triangles,
                                                ExportArgs *args);
 static gint             detect3d_obj          (const GwyFileDetectInfo *fileinfo,
-                                               gboolean only_name,
-                                               const gchar *name);
+                                               gboolean only_name);
 static gboolean         export3d_obj          (FILE *fh,
                                                GArray *vertices,
                                                GArray *triangles,
@@ -198,15 +190,13 @@ static GwySurface*      import3d_obj          (guchar *buffer,
                                                gsize size,
                                                GError **error);
 static gint             detect3d_off          (const GwyFileDetectInfo *fileinfo,
-                                               gboolean only_name,
-                                               const gchar *name);
+                                               gboolean only_name);
 static gboolean         export3d_off          (FILE *fh,
                                                GArray *vertices,
                                                GArray *triangles,
                                                ExportArgs *args);
 static gint             detect3d_stl          (const GwyFileDetectInfo *fileinfo,
-                                               gboolean only_name,
-                                               const gchar *name);
+                                               gboolean only_name);
 static gboolean         export3d_stl          (FILE *fh,
                                                GArray *vertices,
                                                GArray *triangles,
@@ -215,27 +205,24 @@ static GwySurface*      import3d_stl          (guchar *buffer,
                                                gsize size,
                                                GError **error);
 static gint             detect3d_xyz          (const GwyFileDetectInfo *fileinfo,
-                                               gboolean only_name,
-                                               const gchar *name);
+                                               gboolean only_name);
 static GwySurface*      import3d_xyz          (guchar *buffer,
                                                gsize size,
                                                GError **error);
-static gboolean         formats3d_export      (GwyContainer *data,
+static gboolean         formats3d_export      (GwyFile *data,
                                                const gchar *filename,
-                                               GwyRunType runtype,
-                                               GError **error,
-                                               const gchar *name);
+                                               GwyRunModeFlags mode,
+                                               GError **error);
 static void             export_sanitise_params(ExportArgs *args,
                                                const Format3D *fmt);
 static GwyDialogOutcome export_run_gui        (const Format3D *fmt,
                                                ExportArgs *args);
 static void             export_param_changed  (ExportGUI *gui,
                                                gint id);
-static GwyContainer*    formats3d_import      (const gchar *filename,
-                                               GwyRunType runtype,
-                                               GError **error,
-                                               const gchar *name);
-static GwyDataField*    check_regular_grid    (GwySurface *surface);
+static GwyFile*         formats3d_import      (const gchar *filename,
+                                               GwyRunModeFlags mode,
+                                               GError **error);
+static GwyField*        check_regular_grid    (GwySurface *surface);
 static GwyDialogOutcome import_run_gui        (const Format3D *fmt,
                                                ImportArgs *args);
 static void             import_param_changed  (ImportGUI *gui,
@@ -286,7 +273,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Exports images as miscellaneous 3D data formats and imports XYZ points from 3D formats."),
     "Yeti <yeti@gwyddion.net>",
-    "2.1",
+    "3.0",
     "David Nečas (Yeti)",
     "2020",
 };
@@ -301,10 +288,10 @@ module_register(void)
     for (i = 0; i < G_N_ELEMENTS(formats3d); i++) {
         const Format3D *fmt = formats3d + i;
         gwy_file_func_register(fmt->name, _(fmt->description),
-                               fmt->detect,
-                               fmt->import ? formats3d_import : NULL,
+                               fmt->detect_file,
+                               fmt->import_file ? formats3d_import : NULL,
                                NULL,
-                               fmt->export_ ? formats3d_export : NULL);
+                               fmt->export_file ? formats3d_export : NULL);
     }
     gwy_file_func_set_is_detectable("rawxyz", FALSE);
 
@@ -366,18 +353,14 @@ define_import_params(void)
 }
 
 static gint
-detect3d_vtk(const GwyFileDetectInfo *fileinfo,
-             gboolean only_name,
-             G_GNUC_UNUSED const gchar *name)
+detect3d_vtk(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     g_return_val_if_fail(only_name, 0);
     return g_str_has_suffix(fileinfo->name_lowercase, ".vtk") ? 30 : 0;
 }
 
 static gint
-detect3d_ply(const GwyFileDetectInfo *fileinfo,
-             gboolean only_name,
-             G_GNUC_UNUSED const gchar *name)
+detect3d_ply(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     g_return_val_if_fail(only_name, 0);
     return g_str_has_suffix(fileinfo->name_lowercase, ".ply") ? 30 : 0;
@@ -405,9 +388,7 @@ try_to_match_keyword(const gchar *s, const gchar *k, guint maxlen)
 }
 
 static gint
-detect3d_obj(const GwyFileDetectInfo *fileinfo,
-             gboolean only_name,
-             G_GNUC_UNUSED const gchar *name)
+detect3d_obj(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     enum { kw_min = 98, kw_max = 118 };
     static const gchar keywords[] =
@@ -526,18 +507,14 @@ decide:
 }
 
 static gint
-detect3d_off(const GwyFileDetectInfo *fileinfo,
-             gboolean only_name,
-             G_GNUC_UNUSED const gchar *name)
+detect3d_off(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     g_return_val_if_fail(only_name, 0);
     return g_str_has_suffix(fileinfo->name_lowercase, ".off") ? 30 : 0;
 }
 
 static gint
-detect3d_stl(const GwyFileDetectInfo *fileinfo,
-             gboolean only_name,
-             G_GNUC_UNUSED const gchar *name)
+detect3d_stl(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     guint i, j, ntri, ngood = 0;
     const guchar *p;
@@ -583,9 +560,7 @@ detect3d_stl(const GwyFileDetectInfo *fileinfo,
 }
 
 static gint
-detect3d_xyz(const GwyFileDetectInfo *fileinfo,
-             gboolean only_name,
-             G_GNUC_UNUSED const gchar *name)
+detect3d_xyz(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     const gchar *s;
     gchar *end;
@@ -652,16 +627,16 @@ find_format(const gchar *name)
  ********************************************************************************************************************/
 
 static void
-make_scaling_info(GwyDataField *dfield, ScalingInfo *scinfo)
+make_scaling_info(GwyField *dfield, ScalingInfo *scinfo)
 {
     gdouble min, max, dx, dy, a;
     gint xres, yres;
 
-    xres = gwy_data_field_get_xres(dfield);
-    yres = gwy_data_field_get_yres(dfield);
-    dx = gwy_data_field_get_dx(dfield);
-    dy = gwy_data_field_get_dy(dfield);
-    gwy_data_field_get_min_max(dfield, &min, &max);
+    xres = gwy_field_get_xres(dfield);
+    yres = gwy_field_get_yres(dfield);
+    dx = gwy_field_get_dx(dfield);
+    dy = gwy_field_get_dy(dfield);
+    gwy_field_get_min_max(dfield, &min, &max);
 
     scinfo->zscale_1_1 = 1.0/sqrt(dx*dy);
     scinfo->dx = dx*scinfo->zscale_1_1;
@@ -703,15 +678,15 @@ make_triangulation(ExportArgs *args,
     ZScaleType zscale_type = gwy_params_get_enum(args->params, PARAM_ZSCALE_TYPE);
     TriangulationType triang_type = gwy_params_get_enum(args->params, PARAM_TRIANG_TYPE);
     const ScalingInfo *scinfo = &args->scinfo;
-    GwyDataField *field = args->field;
+    GwyField *field = args->field;
     gint xres, yres, i, j, k, stride;
     gdouble z, min, dx = scinfo->dx, dy = scinfo->dy, qz = scinfo->zscale_1_1;
     const gdouble *d;
     GwyXYZ xyz;
 
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
-    min = gwy_data_field_get_min(field);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
+    min = gwy_field_get_min(field);
 
     if (zscale_type == ZSCALE_USER)
         qz *= gwy_params_get_double(args->params, PARAM_ZSCALE);
@@ -720,7 +695,7 @@ make_triangulation(ExportArgs *args,
 
     g_array_set_size(vertices, 0);
     g_array_set_size(triangles, 0);
-    d = gwy_data_field_get_data_const(field);
+    d = gwy_field_get_data_const(field);
 
     if (triang_type == TRIANGULATION_MIDPOINT) {
         stride = 2*xres - 1;
@@ -867,11 +842,10 @@ fix_triangulation(GArray *vertices, GArray *triangles,
 }
 
 static gboolean
-formats3d_export(G_GNUC_UNUSED GwyContainer *data,
+formats3d_export(G_GNUC_UNUSED GwyFile *data,
                  const gchar *filename,
-                 GwyRunType runtype,
-                 GError **error,
-                 const gchar *name)
+                 GwyRunModeFlags mode,
+                 GError **error)
 {
     const Format3D *fmt;
     ExportArgs args;
@@ -882,14 +856,14 @@ formats3d_export(G_GNUC_UNUSED GwyContainer *data,
     gint id;
     FILE *fh;
 
-    fmt = find_format(name);
+    fmt = find_format(gwy_file_func_current());
     g_return_val_if_fail(fmt, FALSE);
-    g_return_val_if_fail(fmt->export_, FALSE);
+    g_return_val_if_fail(fmt->export_file, FALSE);
 
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
 
     if (!args.field) {
         err_NO_CHANNEL_EXPORT(error);
@@ -900,7 +874,7 @@ formats3d_export(G_GNUC_UNUSED GwyContainer *data,
     args.params = params = gwy_params_new_from_settings(define_export_params());
     export_sanitise_params(&args, fmt);
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = export_run_gui(fmt, &args);
         gwy_params_save_to_settings(params);
         if (outcome == GWY_DIALOG_CANCEL) {
@@ -909,9 +883,7 @@ formats3d_export(G_GNUC_UNUSED GwyContainer *data,
         }
     }
 
-    if (!gwy_container_gis_string(data, gwy_app_get_data_title_key_for_id(id), &args.title))
-        args.title = _("Untitled");
-
+    args.title = gwy_file_get_display_title(data, GWY_FILE_IMAGE, id);
     if (!(fh = gwy_fopen(filename, fmt->is_binary ? "wb" : "w"))) {
         err_OPEN_WRITE(error);
         goto end;
@@ -924,7 +896,7 @@ formats3d_export(G_GNUC_UNUSED GwyContainer *data,
                       gwy_params_get_boolean(params, PARAM_SWAP_XY), gwy_params_get_boolean(params, PARAM_FLIP_Z));
 
     tried_to_write_file = TRUE;
-    if (!(ok = fmt->export_(fh, vertices, triangles, &args)))
+    if (!(ok = fmt->export_file(fh, vertices, triangles, &args)))
         err_WRITE(error);
     fclose(fh);
 
@@ -932,6 +904,7 @@ formats3d_export(G_GNUC_UNUSED GwyContainer *data,
     g_array_free(triangles, TRUE);
 
 end:
+    g_free(args.title);
     g_object_unref(args.params);
     if (!ok && tried_to_write_file)
         g_unlink(filename);
@@ -985,8 +958,8 @@ export3d_vtk(FILE *fh, GArray *vertices, GArray *triangles, ExportArgs *args)
     TriangulationType triang_type = gwy_params_get_enum(args->params, PARAM_TRIANG_TYPE);
     guint xres, yres, nvert, ntri;
 
-    xres = gwy_data_field_get_xres(args->field);
-    yres = gwy_data_field_get_yres(args->field);
+    xres = gwy_field_get_xres(args->field);
+    yres = gwy_field_get_yres(args->field);
     nvert = vertices->len;
     ntri = triangles->len;
 
@@ -1157,7 +1130,7 @@ export_run_gui(const Format3D *fmt, ExportArgs *args)
     gchar *title = NULL;
     ExportGUI gui;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
 
     title = g_strdup_printf(_("Export %s"), fmt->title);
@@ -1224,19 +1197,18 @@ export_sanitise_params(ExportArgs *args, const Format3D *fmt)
  *
  ********************************************************************************************************************/
 
-static GwyContainer*
+static GwyFile*
 formats3d_import(const gchar *filename,
-                 GwyRunType runtype,
-                 GError **error,
-                 const gchar *name)
+                 GwyRunModeFlags mode,
+                 GError **error)
 {
     const Format3D *fmt;
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     ImportArgs args;
     GwyDialogOutcome outcome;
-    GwyDataField *image;
+    GwyField *image;
     GwySurface *surface;
-    GwySIUnit *xyunit = NULL, *zunit = NULL;
+    GwyUnit *xyunit = NULL, *zunit = NULL;
     gint power10xy, power10z;
     gdouble q;
     gchar *buffer = NULL;
@@ -1245,17 +1217,17 @@ formats3d_import(const gchar *filename,
     GwyXYZ *data;
     guint n, k;
 
-    fmt = find_format(name);
+    fmt = find_format(gwy_file_func_current());
     g_return_val_if_fail(fmt, NULL);
-    g_return_val_if_fail(fmt->import, NULL);
+    g_return_val_if_fail(fmt->import_file, NULL);
 
     if (!g_file_get_contents(filename, &buffer, &size, &err)) {
         err_GET_FILE_CONTENTS(error, &err);
         return NULL;
     }
 
-    gwy_clear(&args, 1);
-    args.surface = surface = fmt->import(buffer, size, error);
+    gwy_clear1(args);
+    args.surface = surface = fmt->import_file(buffer, size, error);
     g_free(buffer);
     if (!surface)
         return NULL;
@@ -1267,7 +1239,7 @@ formats3d_import(const gchar *filename,
 
     args.image = image = check_regular_grid(surface);
     args.params = gwy_params_new_from_settings(define_import_params());
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = import_run_gui(fmt, &args);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL) {
@@ -1276,33 +1248,33 @@ formats3d_import(const gchar *filename,
         }
     }
 
-    container = gwy_container_new();
-    xyunit = gwy_si_unit_new_parse(gwy_params_get_string(args.params, PARAM_XY_UNITS), &power10xy);
-    zunit = gwy_si_unit_new_parse(gwy_params_get_string(args.params, PARAM_Z_UNITS), &power10z);
+    file = gwy_file_new_in_construction();
+    xyunit = gwy_unit_new_parse(gwy_params_get_string(args.params, PARAM_XY_UNITS), &power10xy);
+    zunit = gwy_unit_new_parse(gwy_params_get_string(args.params, PARAM_Z_UNITS), &power10z);
 
     if (image && gwy_params_get_boolean(args.params, PARAM_DO_RASTERISE)) {
-        gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(image), xyunit);
+        gwy_unit_assign(gwy_field_get_unit_xy(image), xyunit);
         if (power10xy) {
-            q = pow10(power10xy);
-            gwy_data_field_set_xreal(image, q*gwy_data_field_get_xreal(image));
-            gwy_data_field_set_yreal(image, q*gwy_data_field_get_yreal(image));
-            gwy_data_field_set_xoffset(image, q*gwy_data_field_get_xoffset(image));
-            gwy_data_field_set_yoffset(image, q*gwy_data_field_get_yoffset(image));
+            q = gwy_exp10(power10xy);
+            gwy_field_set_xreal(image, q*gwy_field_get_xreal(image));
+            gwy_field_set_yreal(image, q*gwy_field_get_yreal(image));
+            gwy_field_set_xoffset(image, q*gwy_field_get_xoffset(image));
+            gwy_field_set_yoffset(image, q*gwy_field_get_yoffset(image));
         }
 
-        gwy_si_unit_assign(gwy_data_field_get_si_unit_z(image), zunit);
+        gwy_unit_assign(gwy_field_get_unit_z(image), zunit);
         if (power10z)
-            gwy_data_field_multiply(image, pow10(power10z));
+            gwy_field_multiply(image, gwy_exp10(power10z));
 
-        gwy_container_set_object(container, gwy_app_get_data_key_for_id(0), image);
-        gwy_app_channel_title_fall_back(container, 0);
-        gwy_file_channel_import_log_add(container, 0, NULL, filename);
+        gwy_file_set_image(file, 0, image);
+        gwy_image_title_fall_back(file, 0);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
     }
     else {
         data = gwy_surface_get_data(surface);
-        gwy_si_unit_assign(gwy_surface_get_si_unit_xy(surface), xyunit);
+        gwy_unit_assign(gwy_surface_get_unit_xy(surface), xyunit);
         if (power10xy) {
-            q = pow10(power10xy);
+            q = gwy_exp10(power10xy);
             for (k = 0; k < n; k++) {
                 data[k].x *= q;
                 data[k].y *= q;
@@ -1310,52 +1282,53 @@ formats3d_import(const gchar *filename,
             gwy_surface_invalidate(surface);
         }
 
-        gwy_si_unit_assign(gwy_surface_get_si_unit_z(surface), zunit);
+        gwy_unit_assign(gwy_surface_get_unit_z(surface), zunit);
         if (power10z) {
-            q = pow10(power10z);
+            q = gwy_exp10(power10z);
             for (k = 0; k < n; k++)
                 data[k].z *= q;
             gwy_surface_invalidate(surface);
         }
 
-        gwy_container_set_object(container, gwy_app_get_surface_key_for_id(0), surface);
-        gwy_app_xyz_title_fall_back(container, 0);
-        gwy_file_xyz_import_log_add(container, 0, NULL, filename);
+        gwy_file_set_xyz(file, 0, surface);
+        gwy_xyz_title_fall_back(file, 0);
+        gwy_log_add_import(file, GWY_FILE_XYZ, 0, NULL, filename);
     }
 
 fail:
     g_object_unref(args.params);
-    GWY_OBJECT_UNREF(args.surface);
-    GWY_OBJECT_UNREF(args.image);
-    GWY_OBJECT_UNREF(xyunit);
-    GWY_OBJECT_UNREF(zunit);
+    g_clear_object(&args.surface);
+    g_clear_object(&args.image);
+    g_clear_object(&xyunit);
+    g_clear_object(&zunit);
 
-    return container;
+    return file;
 }
 
 /* Create a data field directly if the XY positions form a complete regular grid.  */
-static GwyDataField*
+static GwyField*
 check_regular_grid(GwySurface *surface)
 {
     GwyXY xymin, dxy;
     guint n, xres, yres, k;
-    GwyDataField *dfield;
+    GwyField *dfield;
     gdouble *data;
     guint *map;
 
     n = surface->n;
-    if (!(map = gwy_check_regular_2d_grid((const gdouble*)surface->data, 3, n, -1.0, &xres, &yres, &xymin, &dxy)))
+    const GwyXYZ *xyz = gwy_surface_get_data_const(surface);
+    if (!(map = gwy_check_regular_2d_grid((const gdouble*)xyz, 3, n, -1.0, &xres, &yres, &xymin, &dxy)))
         return NULL;
 
-    dfield = gwy_data_field_new(xres, yres, xres*dxy.x, yres*dxy.y, FALSE);
-    data = gwy_data_field_get_data(dfield);
+    dfield = gwy_field_new(xres, yres, xres*dxy.x, yres*dxy.y, FALSE);
+    data = gwy_field_get_data(dfield);
     for (k = 0; k < n; k++)
-        data[k] = surface->data[map[k]].z;
+        data[k] = xyz[map[k]].z;
     g_free(map);
 
-    gwy_data_field_set_xoffset(dfield, xymin.x);
-    gwy_data_field_set_yoffset(dfield, xymin.y);
-    gwy_surface_copy_units_to_data_field(surface, dfield);
+    gwy_field_set_xoffset(dfield, xymin.x);
+    gwy_field_set_yoffset(dfield, xymin.y);
+    gwy_surface_copy_units_to_field(surface, dfield);
 
     return dfield;
 }
@@ -1568,7 +1541,7 @@ import_run_gui(const Format3D *fmt, ImportArgs *args)
     gchar *title = NULL, *s;
     ImportGUI gui;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
 
     title = g_strdup_printf(_("Import %s"), fmt->title);
@@ -1626,20 +1599,20 @@ static void
 update_range_lables(GwyParamTable *table, gint id,
                     gdouble min, gdouble max, const gchar *unitstring)
 {
-    GwySIValueFormat *vf;
-    GwySIUnit *siunit;
+    GwyValueFormat *vf;
+    GwyUnit *siunit;
     gint power10;
     gchar *s;
 
-    siunit = gwy_si_unit_new_parse(unitstring, &power10);
-    min *= pow10(power10);
-    max *= pow10(power10);
-    vf = gwy_si_unit_get_format_with_digits(siunit, GWY_SI_UNIT_FORMAT_VFMARKUP, fmax(fabs(min), fabs(max)), 3, NULL);
+    siunit = gwy_unit_new_parse(unitstring, &power10);
+    min *= gwy_exp10(power10);
+    max *= gwy_exp10(power10);
+    vf = gwy_unit_get_format_with_digits(siunit, GWY_UNIT_FORMAT_VFMARKUP, fmax(fabs(min), fabs(max)), 3, NULL);
     s = g_strdup_printf("%.*f – %.*f", vf->precision, min/vf->magnitude, vf->precision, max/vf->magnitude);
     gwy_param_table_info_set_valuestr(table, id, s);
     g_free(s);
     gwy_param_table_set_unitstr(table, id, vf->units);
-    gwy_si_unit_value_format_free(vf);
+    gwy_value_format_free(vf);
     g_object_unref(siunit);
 }
 
