@@ -1,6 +1,6 @@
 /*
- *  $Id: gdeffile.c 26519 2024-08-15 18:04:20Z yeti-dn $
- *  Copyright (C) 2010-2020 David Necas (Yeti).
+ *  $Id: gdeffile.c 28789 2025-11-04 17:14:03Z yeti-dn $
+ *  Copyright (C) 2010-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -39,11 +39,9 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <time.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/datafield.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 
 #include "err.h"
 #include "get.h"
@@ -112,12 +110,12 @@ struct _GDEFControlBlock {
     GDEFControlBlock *next;
 };
 
-static gboolean      module_register(void);
-static gint          gdef_detect    (const GwyFileDetectInfo *fileinfo,
-                                     gboolean only_name);
-static GwyContainer* gdef_load      (const gchar *filename,
-                                     GwyRunType mode,
-                                     GError **error);
+static gboolean module_register(void);
+static gint     detect_file    (const GwyFileDetectInfo *fileinfo,
+                                gboolean only_name);
+static GwyFile* load_file      (const gchar *filename,
+                                GwyRunModeFlags mode,
+                                GError **error);
 
 static const gsize type_sizes[VAR_NVARS] = {
     4, 4, 8, 2, 4, 1, 0, 0, 1
@@ -128,7 +126,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports DME GDEF data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.4",
+    "0.5",
     "David Nečas (Yeti)",
     "2010",
 };
@@ -140,15 +138,12 @@ module_register(void)
 {
     gwy_file_func_register("gdeffile",
                            N_("DME GDEF files (.gdf)"),
-                           (GwyFileDetectFunc)&gdef_detect,
-                           (GwyFileLoadFunc)&gdef_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
     return TRUE;
 }
 
 static gint
-gdef_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     gint score = 0;
 
@@ -454,13 +449,13 @@ gdef_get_string_var(GDEFVariable *var)
     g_return_val_if_reached(NULL);
 }
 
-static GwyDataField*
+static GwyField*
 gdef_read_channel(GDEFControlBlock *block)
 {
     GDEFVariable *xres_var, *yres_var, *ymis_var, *xreal_var, *yreal_var, *zunit_var, *data_var, *value_var;
     GDEFControlBlock *datablock;
-    GwyDataField *dfield;
-    GwySIUnit *siunit;
+    GwyField *dfield;
+    GwyUnit *siunit;
     gdouble *data;
     const guchar *rawdata;
     const gchar *unit;
@@ -505,10 +500,10 @@ gdef_read_channel(GDEFControlBlock *block)
     sanitise_real_size(&xreal, "x size");
     sanitise_real_size(&yreal, "y size");
 
-    dfield = gwy_data_field_new(xres, yres - ymis, xreal, yreal, FALSE);
-    siunit = gwy_data_field_get_si_unit_xy(dfield);
-    gwy_si_unit_set_from_string(siunit, "m");
-    siunit = gwy_data_field_get_si_unit_z(dfield);
+    dfield = gwy_field_new(xres, yres - ymis, xreal, yreal, FALSE);
+    siunit = gwy_field_get_unit_xy(dfield);
+    gwy_unit_set_from_string(siunit, "m");
+    siunit = gwy_field_get_unit_z(dfield);
     unit = gwy_enuml_to_string(gdef_get_int_var(zunit_var),
                                "m", 0,
                                "V", 1,
@@ -518,9 +513,9 @@ gdef_read_channel(GDEFControlBlock *block)
                                "", 5,
                                "1/m", 6,
                                NULL);
-    gwy_si_unit_set_from_string(siunit, unit);
+    gwy_unit_set_from_string(siunit, unit);
 
-    data = gwy_data_field_get_data(dfield);
+    data = gwy_field_get_data(dfield);
     rawdata = value_var->data;
     /* flip the data vertically */
     for (i = 0; i < yres - ymis; i++) {
@@ -556,7 +551,7 @@ gdef_read_channel_meta(GDEFControlBlock *block)
     gchar *s;
     guint i;
 
-    meta = gwy_container_new();
+    meta = gwy_container_new_in_construction();
     for (i = 0; i < block->n_variables; i++) {
         GDEFVariable *var = block->variables + i;
 
@@ -587,12 +582,13 @@ gdef_read_channel_meta(GDEFControlBlock *block)
     return NULL;
 }
 
-static GwyContainer*
-gdef_load(const gchar *filename,
-          G_GNUC_UNUSED GwyRunType mode,
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
           GError **error)
 {
-    GwyContainer *meta, *container = NULL;
+    GwyFile *file = NULL;
+    GwyContainer *meta;
     guchar *buffer = NULL;
     const guchar *p;
     gsize size = 0;
@@ -600,8 +596,7 @@ gdef_load(const gchar *filename,
     GDEFHeader header;
     GDEFControlBlock *block, *blocks = NULL;
     GDEFVariable *var;
-    GQuark quark;
-    gchar *key, *title;
+    const gchar *title;
     guint i;
 
     if (!gwy_file_get_contents(filename, &buffer, &size, &err)) {
@@ -618,7 +613,7 @@ gdef_load(const gchar *filename,
 
     i = 0;
     for (block = blocks; block; block = block->next) {
-        GwyDataField *dfield;
+        GwyField *dfield;
 #ifdef DEBUG
         guint j;
 
@@ -630,11 +625,10 @@ gdef_load(const gchar *filename,
 #endif
 
         if ((dfield = gdef_read_channel(block))) {
-            if (!container)
-                container = gwy_container_new();
+            if (!file)
+                file = gwy_file_new_in_construction();
 
-            quark = gwy_app_get_data_key_for_id(i);
-            gwy_container_pass_object(container, quark, dfield);
+            gwy_file_pass_image(file, i, dfield);
 
             var = gdef_find_variable(block, "SourceChannel", VAR_INTEGRAL, FALSE);
             if (var) {
@@ -657,30 +651,26 @@ gdef_load(const gchar *filename,
                                             "Z Sensor", 15,
                                             "Unknown", 16,
                                             NULL);
-                key = g_strdup_printf("/%d/data/title", i);
-                gwy_container_set_const_string_by_name(container, key, title);
-                g_free(key);
+                gwy_file_set_title(file, GWY_FILE_IMAGE, i, title, FALSE);
 
                 /* XXX: That's what I've been told is necessary to obtain values in degrees. */
                 if (gdef_get_int_var(var) == 12) {
-                    gwy_data_field_multiply(dfield, 18.0);
+                    gwy_field_multiply(dfield, 18.0);
                 }
             }
             else {
-                gwy_app_channel_title_fall_back(container, i);
+                gwy_image_title_fall_back(file, i);
             }
 
-            if ((meta = gdef_read_channel_meta(block))) {
-                key = g_strdup_printf("/%d/meta", i);
-                gwy_container_pass_object_by_name(container, key, meta);
-                g_free(key);
-            }
-            gwy_file_channel_import_log_add(container, i, NULL, filename);
+            if ((meta = gdef_read_channel_meta(block)))
+                gwy_file_pass_meta(file, GWY_FILE_IMAGE, i, meta);
+
+            gwy_log_add_import(file, GWY_FILE_IMAGE, i, NULL, filename);
             i++;
         }
     }
 
-    if (!container)
+    if (!file)
         err_NO_DATA(error);
 
 fail:
@@ -688,7 +678,7 @@ fail:
     g_free(header.description);
     gdef_free_control_block_list(blocks);
 
-    return container;
+    return file;
 }
 
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
