@@ -1,5 +1,5 @@
 /*
- *  $Id: gwyfile.c 25719 2023-09-26 13:09:32Z yeti-dn $
+ *  $Id: gwyfile.c 28789 2025-11-04 17:14:03Z yeti-dn $
  *  Copyright (C) 2003-2006 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -34,17 +34,18 @@
  * # see http://gwyddion.net/
  * 0 string GWYOGwyContainer\0 Gwyddion SPM data version 1
  * 0 string GWYPGwyContainer\0 Gwyddion SPM data version 2
- * 0 string GWYQGwyContainer\0 Gwyddion SPM data version 3
+ * 0 string GWYQGwyFile\0 Gwyddion SPM data version 3
  **/
 
 /**
  * [FILE-MAGIC-USERGUIDE]
  * Gwyddion native data
  * .gwy
- * Read Save SPS Volume
+ * Read Save SPS Volume XYZ
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <errno.h>
 #include <stdlib.h>
@@ -52,11 +53,7 @@
 #include <unistd.h>
 #endif
 #include <glib/gstdio.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/datafield.h>
-#include <libdraw/gwyselection.h>
-#include <libgwymodule/gwymodule-file.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -77,21 +74,21 @@ typedef struct {
     GString *str;   /* scratch space */
 } CompressIdData;
 
-static gboolean      module_register         (void);
-static gint          gwyfile_detect          (const GwyFileDetectInfo *fileinfo,
-                                              gboolean only_name);
-static GwyContainer* gwyfile_load            (const gchar *filename,
-                                              GwyRunType mode,
-                                              GError **error);
-static gboolean      gwyfile_save            (GwyContainer *data,
-                                              const gchar *filename,
-                                              GwyRunType mode,
-                                              GError **error);
-static void          gwyfile_pack_metadata   (GwyContainer *data);
-static void          gwyfile_remove_old_data (GObject *object);
-static GObject*      gwy_container_deserialize_old (const guchar *buffer,
-                                                    gsize size,
-                                                    gsize *position);
+static gboolean      module_register              (void);
+static gint          gwyfile_detect               (const GwyFileDetectInfo *fileinfo,
+                                                   gboolean only_name);
+static GwyContainer* gwyfile_load                 (const gchar *filename,
+                                                   GwyRunModeFlags mode,
+                                                   GError **error);
+static gboolean      gwyfile_save                 (GwyContainer *data,
+                                                   const gchar *filename,
+                                                   GwyRunModeFlags mode,
+                                                   GError **error);
+static void          gwyfile_pack_metadata        (GwyContainer *data);
+static void          gwyfile_remove_old_data      (GObject *object);
+static GObject*      gwy_container_deserialize_old(const guchar *buffer,
+                                                   gsize size,
+                                                   gsize *position);
 
 
 static GwyModuleInfo module_info = {
@@ -111,9 +108,9 @@ module_register(void)
 {
     gwy_file_func_register("gwyfile",
                            N_("Gwyddion native format (.gwy)"),
-                           (GwyFileDetectFunc)&gwyfile_detect,
-                           (GwyFileLoadFunc)&gwyfile_load,
-                           (GwyFileSaveFunc)&gwyfile_save,
+                           &gwyfile_detect,
+                           &gwyfile_load,
+                           &gwyfile_save,
                            NULL);
 
     return TRUE;
@@ -137,7 +134,7 @@ gwyfile_detect(const GwyFileDetectInfo *fileinfo,
 
 static GwyContainer*
 gwyfile_load(const gchar *filename,
-             G_GNUC_UNUSED GwyRunType mode,
+             G_GNUC_UNUSED GwyRunModeFlags mode,
              GError **error)
 {
     GwyContainer *container;
@@ -161,10 +158,12 @@ gwyfile_load(const gchar *filename,
         object = gwy_container_deserialize_old(buffer + MAGIC_SIZE, size - MAGIC_SIZE, &pos);
         gwyfile_remove_old_data(object);
     }
-    else
-        object = gwy_serializable_deserialize(buffer + MAGIC_SIZE, size - MAGIC_SIZE, &pos);
+    else {
+        /* TODO: Do something with the error list. */
+        object = gwy_deserialize_memory(buffer + MAGIC_SIZE, size - MAGIC_SIZE, &pos, NULL);
+    }
 
-    gwy_file_abandon_contents(buffer, size, &err);
+    gwy_file_abandon_contents(buffer, size, NULL);
     if (!object) {
         g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
                     _("Data deserialization failed."));
@@ -189,21 +188,18 @@ gwyfile_load(const gchar *filename,
 static gboolean
 gwyfile_save(GwyContainer *data,
              const gchar *filename,
-             G_GNUC_UNUSED GwyRunType mode,
+             G_GNUC_UNUSED GwyRunModeFlags mode,
              GError **error)
 {
-    GByteArray *buffer;
-    gchar *filename_orig_utf8, *filename_utf8;
-    FILE *fh;
-    gboolean restore_filename, ok = TRUE;
+    gboolean ok = TRUE;
 
     /* Ensure the saved file contains its own name under "/filename" */
-    restore_filename = TRUE;
-    filename_orig_utf8 = NULL;
-    gwy_container_gis_string_by_name(data, "/filename", (const guchar**)&filename_orig_utf8);
-    filename_orig_utf8 = g_strdup(filename_orig_utf8);
+    gboolean restore_filename = TRUE;
+    const gchar *s = NULL;
+    gwy_container_gis_string_by_name(data, "/filename", &s);
+    gchar *filename_orig_utf8 = g_strdup(s);
 
-    filename_utf8 = g_filename_to_utf8(filename, -1, NULL, NULL, NULL);
+    gchar *filename_utf8 = g_filename_to_utf8(filename, -1, NULL, NULL, NULL);
     if (!filename_utf8)
         gwy_container_remove_by_name(data, "/filename");
     else if (filename_orig_utf8 && gwy_strequal(filename_orig_utf8, filename_utf8)) {
@@ -214,28 +210,24 @@ gwyfile_save(GwyContainer *data,
         filename_utf8 = NULL;
     }
 
-    /* Serialize first.  If we get OOM and hard-abort here, at least keep any existing file intact. */
-    buffer = gwy_serializable_serialize(G_OBJECT(data), NULL);
-
-    /* Now actually write the file. */
-    if (!(fh = gwy_fopen(filename, "wb"))) {
-        err_OPEN_WRITE(error);
+    GError *err = NULL;
+    GFile *gfile = g_file_new_for_path(filename);
+    GFileOutputStream *stream;
+    if (!(stream = g_file_replace(gfile, NULL, FALSE, 0, NULL, &err))) {
+        err_OPEN_WRITE_GERROR(error, &err);
         ok = FALSE;
     }
-    else {
-        if (fwrite(MAGIC2, 1, MAGIC_SIZE, fh) != MAGIC_SIZE
-            || fwrite(buffer->data, 1, buffer->len, fh) != buffer->len) {
-            err_WRITE(error);
+    if (ok) {
+        g_output_stream_write(G_OUTPUT_STREAM(stream), MAGIC2, MAGIC_SIZE, NULL, NULL);
+        if (!gwy_serialize_gio(GWY_SERIALIZABLE(data), G_OUTPUT_STREAM(stream), &err)) {
+            g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_IO,
+                        _("Cannot write to file: %s."), err->message);
+            g_clear_error(&err);
             ok = FALSE;
-            g_unlink(filename);
-        }
-        if (fclose(fh)) {
-            err_WRITE(error);
-            ok = FALSE;
-            g_unlink(filename);
         }
     }
-    g_byte_array_free(buffer, TRUE);
+    g_clear_object(&stream);
+    g_object_unref(gfile);
 
     /* Restore filename if save failed */
     if (!ok && restore_filename) {
@@ -272,17 +264,15 @@ gwyfile_gather_one_meta(GQuark quark,
 }
 
 static void
-gwyfile_gather_meta(gpointer key, gpointer value, gpointer user_data)
+gwyfile_gather_meta(GQuark quark, GValue *gvalue, gpointer user_data)
 {
-    gwyfile_gather_one_meta(GPOINTER_TO_UINT(key),
-                            (GValue*)value, (GwyContainer*)user_data, "/meta", sizeof("/meta")-1);
+    gwyfile_gather_one_meta(quark, gvalue, (GwyContainer*)user_data, "/meta", sizeof("/meta")-1);
 }
 
 static void
-gwyfile_gather_0_data_meta(gpointer key, gpointer value, gpointer user_data)
+gwyfile_gather_0_data_meta(GQuark quark, GValue *gvalue, gpointer user_data)
 {
-    gwyfile_gather_one_meta(GPOINTER_TO_UINT(key),
-                            (GValue*)value, (GwyContainer*)user_data, "/0/data/meta", sizeof("/0/data/meta")-1);
+    gwyfile_gather_one_meta(quark, gvalue, (GwyContainer*)user_data, "/0/data/meta", sizeof("/0/data/meta")-1);
 }
 
 /**
@@ -316,13 +306,10 @@ static GwySelection*
 gwyfile_gather_old_rect_selection(GwyContainer *data)
 {
     GwySelection *sel;
-    GType type;
     gboolean selected;
     gdouble xy[4];
 
-    type = g_type_from_name("GwySelectionRectangle");
-    if (!type
-        || !gwy_container_gis_boolean_by_name(data, "/0/select/rect/selected", &selected)
+    if (!gwy_container_gis_boolean_by_name(data, "/0/select/rect/selected", &selected)
         || !selected)
         return NULL;
 
@@ -332,7 +319,7 @@ gwyfile_gather_old_rect_selection(GwyContainer *data)
         || !gwy_container_gis_double_by_name(data, "/0/select/rect/y1", &xy[3]))
         return NULL;
 
-    sel = GWY_SELECTION(g_object_new(type, "max-objects", 1, NULL));
+    sel = GWY_SELECTION(g_object_new(GWY_TYPE_SELECTION_RECTANGLE, "max-objects", 1, NULL));
     gwy_selection_set_object(sel, 0, xy);
 
     return sel;
@@ -342,20 +329,18 @@ static GwySelection*
 gwyfile_gather_old_point_selection(GwyContainer *data)
 {
     GwySelection *sel;
-    GType type;
     gint i, nselected;
     gdouble xy[2];
     gchar key[40];
 
-    type = g_type_from_name("GwySelectionPoint");
-    if (!type || !gwy_container_gis_int32_by_name(data, "/0/select/points/nselected", &nselected))
+    if (!gwy_container_gis_int32_by_name(data, "/0/select/points/nselected", &nselected))
         return NULL;
 
     nselected = CLAMP(nselected, 0, 16);
     if (!nselected)
         return NULL;
 
-    sel = GWY_SELECTION(g_object_new(type, "max-objects", nselected, NULL));
+    sel = GWY_SELECTION(g_object_new(GWY_TYPE_SELECTION_POINT, "max-objects", nselected, NULL));
     for (i = 0; i < nselected; i++) {
         g_snprintf(key, sizeof(key), "/0/select/points/%d/x", i);
         if (!gwy_container_gis_double_by_name(data, key, &xy[0]))
@@ -368,7 +353,7 @@ gwyfile_gather_old_point_selection(GwyContainer *data)
     }
 
     if (!i)
-        GWY_OBJECT_UNREF(sel);
+        g_clear_object(&sel);
 
     return sel;
 }
@@ -377,20 +362,18 @@ static GwySelection*
 gwyfile_gather_old_line_selection(GwyContainer *data)
 {
     GwySelection *sel;
-    GType type;
     gint i, nselected;
     gdouble xy[4];
     gchar key[40];
 
-    type = g_type_from_name("GwySelectionLine");
-    if (!type || !gwy_container_gis_int32_by_name(data, "/0/select/lines/nselected", &nselected))
+    if (!gwy_container_gis_int32_by_name(data, "/0/select/lines/nselected", &nselected))
         return NULL;
 
     nselected = CLAMP(nselected, 0, 16);
     if (!nselected)
         return NULL;
 
-    sel = GWY_SELECTION(g_object_new(type, "max-objects", nselected, NULL));
+    sel = GWY_SELECTION(g_object_new(GWY_TYPE_SELECTION_LINE, "max-objects", nselected, NULL));
     for (i = 0; i < nselected; i++) {
         g_snprintf(key, sizeof(key), "/0/select/lines/%d/x0", i);
         if (!gwy_container_gis_double_by_name(data, key, &xy[0]))
@@ -409,7 +392,7 @@ gwyfile_gather_old_line_selection(GwyContainer *data)
     }
 
     if (!i)
-        GWY_OBJECT_UNREF(sel);
+        g_clear_object(&sel);
 
     return sel;
 }
@@ -453,9 +436,9 @@ gwyfile_remove_old_data(GObject *object)
 
 /** Low-level deserialization functions for 1.x file import {{{ **/
 static inline gboolean
-gwy_serialize_unpack_boolean(const guchar *buffer,
-                             gsize size,
-                             gsize *position)
+unpack_boolean(const guchar *buffer,
+               gsize size,
+               gsize *position)
 {
     gboolean value;
 
@@ -471,9 +454,9 @@ gwy_serialize_unpack_boolean(const guchar *buffer,
 }
 
 static inline guchar
-gwy_serialize_unpack_char(const guchar *buffer,
-                          gsize size,
-                          gsize *position)
+unpack_char(const guchar *buffer,
+            gsize size,
+            gsize *position)
 {
     guchar value;
 
@@ -489,9 +472,9 @@ gwy_serialize_unpack_char(const guchar *buffer,
 }
 
 static inline gint32
-gwy_serialize_unpack_int32(const guchar *buffer,
-                           gsize size,
-                           gsize *position)
+unpack_int32(const guchar *buffer,
+             gsize size,
+             gsize *position)
 {
     gint32 value;
 
@@ -508,9 +491,9 @@ gwy_serialize_unpack_int32(const guchar *buffer,
 }
 
 static inline gint64
-gwy_serialize_unpack_int64(const guchar *buffer,
-                           gsize size,
-                           gsize *position)
+unpack_int64(const guchar *buffer,
+             gsize size,
+             gsize *position)
 {
     gint64 value;
 
@@ -527,9 +510,9 @@ gwy_serialize_unpack_int64(const guchar *buffer,
 }
 
 static inline gdouble
-gwy_serialize_unpack_double(const guchar *buffer,
-                            gsize size,
-                            gsize *position)
+unpack_double(const guchar *buffer,
+              gsize size,
+              gsize *position)
 {
     gdouble value;
 
@@ -549,9 +532,9 @@ gwy_serialize_unpack_double(const guchar *buffer,
 }
 
 static inline guchar*
-gwy_serialize_unpack_string(const guchar *buffer,
-                            gsize size,
-                            gsize *position)
+unpack_string(const guchar *buffer,
+              gsize size,
+              gsize *position)
 {
     guchar *value;
     const guchar *p;
@@ -569,67 +552,88 @@ gwy_serialize_unpack_string(const guchar *buffer,
     return value;
 }
 
+static inline gsize
+check_string(const guchar *buffer,
+             gsize size,
+             gsize position,
+             const guchar *compare_to)
+{
+    const guchar *p;
+
+    gwy_debug("<%s> buf = %p, size = %" G_GSIZE_FORMAT ", pos = %"
+              G_GSIZE_FORMAT,
+              compare_to ? compare_to : (const guchar*)"(null)",
+              buffer, size, position);
+    g_assert(buffer);
+    g_assert(size > 0);
+    g_return_val_if_fail(position < size, 0);
+    p = (guchar*)memchr(buffer + position, 0, size - position);
+    if (!p || (compare_to && !gwy_strequal(buffer + position, compare_to)))
+        return 0;
+
+    return (p - buffer) + 1 - position;
+}
+
 static GObject*
 gwy_container_deserialize_old(const guchar *buffer,
                               gsize size,
                               gsize *position)
 {
-    GwyContainer *container;
     gsize mysize, pos;
     const guchar *buf;
 
     gwy_debug("");
     g_return_val_if_fail(buffer, NULL);
 
-    pos = gwy_serialize_check_string(buffer, size, *position, g_type_name(GWY_TYPE_CONTAINER));
+    pos = check_string(buffer, size, *position, g_type_name(GWY_TYPE_CONTAINER));
     g_return_val_if_fail(pos, NULL);
     *position += pos;
-    mysize = gwy_serialize_unpack_int32(buffer, size, position);
+    mysize = unpack_int32(buffer, size, position);
     buf = buffer + *position;
     pos = 0;
 
-    container = (GwyContainer*)gwy_container_new();
-    container->in_construction = TRUE;
+    GwyContainer *container = gwy_container_new();
+    gwy_container_start_construction(container);
     while (pos < mysize) {
         GType type;
         guchar *name;
         GQuark key;
         GObject *object;
 
-        type = gwy_serialize_unpack_int32(buf, mysize, &pos);
-        name = gwy_serialize_unpack_string(buf, mysize, &pos);
+        type = unpack_int32(buf, mysize, &pos);
+        name = unpack_string(buf, mysize, &pos);
         gwy_debug("deserializing %s => %s", name, g_type_name(type));
         key = g_quark_from_string(name);
         g_free(name);
 
         switch (type) {
             case G_TYPE_OBJECT:
-            if ((object = gwy_serializable_deserialize(buf, mysize, &pos)))
+            if ((object = gwy_deserialize_memory(buf, mysize, &pos, NULL)))
                 gwy_container_pass_object(container, key, object);
             break;
 
             case G_TYPE_BOOLEAN:
-            gwy_container_set_boolean(container, key, gwy_serialize_unpack_boolean(buf, mysize, &pos));
+            gwy_container_set_boolean(container, key, unpack_boolean(buf, mysize, &pos));
             break;
 
             case G_TYPE_UCHAR:
-            gwy_container_set_uchar(container, key, gwy_serialize_unpack_char(buf, mysize, &pos));
+            gwy_container_set_uchar(container, key, unpack_char(buf, mysize, &pos));
             break;
 
             case G_TYPE_INT:
-            gwy_container_set_int32(container, key, gwy_serialize_unpack_int32(buf, mysize, &pos));
+            gwy_container_set_int32(container, key, unpack_int32(buf, mysize, &pos));
             break;
 
             case G_TYPE_INT64:
-            gwy_container_set_int64(container, key, gwy_serialize_unpack_int64(buf, mysize, &pos));
+            gwy_container_set_int64(container, key, unpack_int64(buf, mysize, &pos));
             break;
 
             case G_TYPE_DOUBLE:
-            gwy_container_set_double(container, key, gwy_serialize_unpack_double(buf, mysize, &pos));
+            gwy_container_set_double(container, key, unpack_double(buf, mysize, &pos));
             break;
 
             case G_TYPE_STRING:
-            gwy_container_set_string(container, key, gwy_serialize_unpack_string(buf, mysize, &pos));
+            gwy_container_set_string(container, key, unpack_string(buf, mysize, &pos));
             break;
 
             default:
@@ -638,7 +642,7 @@ gwy_container_deserialize_old(const guchar *buffer,
         }
     }
     *position += mysize;
-    container->in_construction = FALSE;
+    gwy_container_finish_construction(container);
 
     return (GObject*)container;
 }
