@@ -1,5 +1,5 @@
 /*
- *  $Id: gwytiff.c 28461 2025-08-27 14:57:13Z yeti-dn $
+ *  $Id: gwytiff.c 28263 2025-07-18 14:34:37Z yeti-dn $
  *  Copyright (C) 2007-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,10 +16,9 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <glib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwyutils.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 #include "get.h"
 #include "err.h"
 #include "gwytiff.h"
@@ -653,8 +652,8 @@ gwy_tiff_get_float_entry(const GwyTIFF *tiff,
         *retval = (gdouble)numerator/denominator;
     }
     else if (type == GWY_TIFF_SRATIONAL) {
-        guint numerator = tiff->get_gint32(&p);
-        guint denominator = tiff->get_gint32(&p);
+        gint numerator = tiff->get_gint32(&p);
+        gint denominator = tiff->get_gint32(&p);
         *retval = (gdouble)numerator/denominator;
     }
     else
@@ -1163,7 +1162,7 @@ gwy_tiff_get_image_reader(const GwyTIFF *tiff,
     guint i, v, spp;
     guint *bps;
 
-    gwy_clear(&reader, 1);
+    gwy_clear1(reader);
     reader.dirno = dirno;
     reader.which_unpacked = G_MAXUINT64;
 
