@@ -1,5 +1,5 @@
 /*
- *  $Id: gwytiff.h 26193 2024-02-22 12:12:42Z yeti-dn $
+ *  $Id: gwytiff.h 28789 2025-11-04 17:14:03Z yeti-dn $
  *  Copyright (C) 2007-2024 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -18,8 +18,8 @@
 #define __GWY_TIFF__ 1
 
 #include <glib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwymodule/gwymodule-file.h>
+#include <libgwyddion/macros.h>
+#include <libgwyapp/module-file.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -281,25 +281,25 @@ err_TIFF_REQUIRED_TAG(GError **error, GwyTIFFTag tag)
 /* Parameters version and byteorder are inout.  If they are non-zero, the file must match the specified value to be
  * accepted.  In any case, they are set to the true values on success (and it returns position in buffer after the
  * header, which should usually be treated just as NULL/non-NULL boolean flag). */
-G_GNUC_INTERNAL const guchar*       gwy_tiff_detect                 (const guchar *buffer,
-                                                                     gsize size,
-                                                                     GwyTIFFVersion *version,
-                                                                     GwyByteOrder *byteorder);
-G_GNUC_INTERNAL void                gwy_tiff_allow_compressed       (GwyTIFF *tiff,
-                                                                     gboolean setting);
-G_GNUC_INTERNAL GwyTIFF*            gwy_tiff_load                   (const gchar *filename,
-                                                                     GError **error);
-G_GNUC_INTERNAL void                gwy_tiff_free                   (GwyTIFF *tiff);
-G_GNUC_INTERNAL guint               gwy_tiff_get_n_dirs             (const GwyTIFF *tiff);
-G_GNUC_INTERNAL const GwyTIFFEntry* gwy_tiff_find_tag_in_dir        (const GArray *tags,
-                                                                     guint tag);
-G_GNUC_INTERNAL const GwyTIFFEntry* gwy_tiff_find_tag               (const GwyTIFF *tiff,
-                                                                     guint dirno,
-                                                                     guint tag);
-G_GNUC_INTERNAL GwyTIFFImageReader* gwy_tiff_get_image_reader       (const GwyTIFF *tiff,
-                                                                     guint dirno,
-                                                                     guint max_samples,
-                                                                     GError **error);
+G_GNUC_INTERNAL const guchar*       gwy_tiff_detect          (const guchar *buffer,
+                                                              gsize size,
+                                                              GwyTIFFVersion *version,
+                                                              GwyByteOrder *byteorder);
+G_GNUC_INTERNAL void                gwy_tiff_allow_compressed(GwyTIFF *tiff,
+                                                              gboolean setting);
+G_GNUC_INTERNAL GwyTIFF*            gwy_tiff_load            (const gchar *filename,
+                                                              GError **error);
+G_GNUC_INTERNAL void                gwy_tiff_free            (GwyTIFF *tiff);
+G_GNUC_INTERNAL guint               gwy_tiff_get_n_dirs      (const GwyTIFF *tiff);
+G_GNUC_INTERNAL const GwyTIFFEntry* gwy_tiff_find_tag_in_dir (const GArray *tags,
+                                                              guint tag);
+G_GNUC_INTERNAL const GwyTIFFEntry* gwy_tiff_find_tag        (const GwyTIFF *tiff,
+                                                              guint dirno,
+                                                              guint tag);
+G_GNUC_INTERNAL GwyTIFFImageReader* gwy_tiff_get_image_reader(const GwyTIFF *tiff,
+                                                              guint dirno,
+                                                              guint max_samples,
+                                                              GError **error);
 /* Idempotent, can use like this: reader = gwy_tiff_image_reader_free(reader); */
 G_GNUC_INTERNAL GwyTIFFImageReader* gwy_tiff_image_reader_free      (GwyTIFFImageReader *reader);
 G_GNUC_INTERNAL gboolean            gwy_tiff_get_bool_entry         (const GwyTIFF *tiff,
@@ -370,13 +370,13 @@ G_GNUC_INTERNAL gboolean            gwy_tiff_read_image_row_averaged(const GwyTI
                                                                      gdouble *dest);
 /* You should not need the following functions. But you occassionally might, e.g. when a tag contains some embedded
  * binary header. */
-G_GNUC_INTERNAL guint               gwy_tiff_data_type_size         (GwyTIFFDataType type);
-G_GNUC_INTERNAL const guchar*       gwy_tiff_entry_get_data_pointer (const GwyTIFF *tiff,
-                                                                     const GwyTIFFEntry *entry);
-G_GNUC_INTERNAL gboolean            gwy_tiff_data_fits              (const GwyTIFF *tiff,
-                                                                     guint64 offset,
-                                                                     guint64 item_size,
-                                                                     guint64 nitems);
+G_GNUC_INTERNAL guint         gwy_tiff_data_type_size        (GwyTIFFDataType type);
+G_GNUC_INTERNAL const guchar* gwy_tiff_entry_get_data_pointer(const GwyTIFF *tiff,
+                                                              const GwyTIFFEntry *entry);
+G_GNUC_INTERNAL gboolean      gwy_tiff_data_fits             (const GwyTIFF *tiff,
+                                                              guint64 offset,
+                                                              guint64 item_size,
+                                                              guint64 nitems);
 /* Only useful for modules which mess with the TIFF directory, like fix stange tag types, and want to revalidate the
  * resulting pile of rubbish afterwards.
  *
@@ -388,14 +388,14 @@ G_GNUC_INTERNAL gboolean            gwy_tiff_tags_valid             (const GwyTI
  * a common occurrence).
  *
  * NOT intended for normal use as everything is validated by gwy_tiff_load(). */
-G_GNUC_INTERNAL GArray*             gwy_tiff_scan_ifd               (const GwyTIFF *tiff,
-                                                                     guint64 offset,
-                                                                     const guchar **pafter,
-                                                                     GError **error);
-G_GNUC_INTERNAL gboolean            gwy_tiff_ifd_is_vaild           (const GwyTIFF *tiff,
-                                                                     const GArray *tags,
-                                                                     GError **error);
-G_GNUC_INTERNAL void                gwy_tiff_sort_tag_array         (GArray *tags);
+G_GNUC_INTERNAL GArray*  gwy_tiff_scan_ifd      (const GwyTIFF *tiff,
+                                                 guint64 offset,
+                                                 const guchar **pafter,
+                                                 GError **error);
+G_GNUC_INTERNAL gboolean gwy_tiff_ifd_is_vaild  (const GwyTIFF *tiff,
+                                                 const GArray *tags,
+                                                 GError **error);
+G_GNUC_INTERNAL void     gwy_tiff_sort_tag_array(GArray *tags);
 
 #ifdef __cplusplus
 }
