@@ -1,5 +1,5 @@
 /*
- *  $Id: gwyzlib.h 25989 2023-10-31 17:27:00Z yeti-dn $
+ *  $Id: gwyzlib.h 27489 2025-03-08 08:12:18Z yeti-dn $
  *  Copyright (C) 2015-2023 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -23,8 +23,7 @@
 /* FIXME: Use GZlibDecompressor? It may actually more complicated to wrap than zlib itself because GConverter has a
  * *complicated* state management. */
 
-/* This is the function we always define. It will just fail if zlib is not available. Check HAVE_ZLIB if zlib
- * compression is a mandatory part of the module.
+/* This is the function we always define.
  *
  * Pass NULL as uncbuffer if you want the function to allocate a new buffer. If data size is unknown (zero uncsize)
  * you must pass NULL uncbuffer. The function cannot decompress zero amount of data – why would you need to do that?
@@ -47,7 +46,6 @@ static guchar* gwyzlib_unpack_compressed_data(const guchar *cbuffer,
  *
  ****************************************************************************/
 
-#ifdef HAVE_ZLIB
 #include <zlib.h>
 
 G_GNUC_UNUSED
@@ -67,7 +65,7 @@ gwyzlib_unpack_compressed_data(const guchar *cbuffer,
 
     g_return_val_if_fail(output_size_known || !uncbuffer, NULL);
 
-    gwy_clear(&zbuf, 1);
+    gwy_clear1(zbuf);
     zbuf.next_in = (char*)cbuffer;
     zbuf.avail_in = *csize;
     zbuf.next_out = uncbuffer;
@@ -174,6 +172,4 @@ gwyzlib_unpack_compressed_data(G_GNUC_UNUSED const guchar *cbuffer,
 }
 #endif
 
-#endif
-
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
