@@ -1,6 +1,6 @@
 /*
- *  $Id: gxyzffile.c 26494 2024-08-09 15:04:32Z yeti-dn $
- *  Copyright (C) 2013-2022 David Necas (Yeti).
+ *  $Id: gxyzffile.c 28822 2025-11-06 16:01:09Z yeti-dn $
+ *  Copyright (C) 2013-2025 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
  *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
@@ -40,16 +40,11 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
 #include <glib/gstdio.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/surface.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -65,54 +60,54 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyAppPage pageno;
+    GwyDataKind data_kind;
 } ModuleArgs;
 
-static gboolean         module_register              (void);
-static GwyParamDef*     define_module_params         (void);
-static gint             gxyzf_detect                 (const GwyFileDetectInfo *fileinfo,
-                                                      gboolean only_name);
-static GwyContainer*    gxyzf_load                   (const gchar *filename,
-                                                      GwyRunType mode,
-                                                      GError **error);
-static gboolean         gxyzf_export                 (GwyContainer *container,
-                                                      const gchar *filename,
-                                                      GwyRunType mode,
-                                                      GError **error);
-static GwyDialogOutcome run_gui                      (ModuleArgs *args,
-                                                      const gchar *title);
-static gboolean         export_data_fields           (const gchar *filename,
-                                                      GwyContainer *container,
-                                                      gint id,
-                                                      const ModuleArgs *args,
-                                                      GError **error);
-static gint*            gather_compatible_data_fields(GwyContainer *data,
-                                                      GwyDataField *dfield,
-                                                      guint *nchannels);
-static gboolean         export_surfaces              (const gchar *filename,
-                                                      GwyContainer *container,
-                                                      gint id,
-                                                      const ModuleArgs *args,
-                                                      GError **error);
-static gint*            gather_compatible_surfaces   (GwyContainer *data,
-                                                      GwySurface *surface,
-                                                      guint *nchannels);
-static gboolean         write_header                 (FILE *fh,
-                                                      guint nchannels,
-                                                      guint npoints,
-                                                      gchar **titles,
-                                                      GwySIUnit *xyunit,
-                                                      GwySIUnit **zunits,
-                                                      gint xres,
-                                                      gint yres,
-                                                      GError **error);
+static gboolean         module_register           (void);
+static GwyParamDef*     define_module_params      (void);
+static gint             detect_file               (const GwyFileDetectInfo *fileinfo,
+                                                   gboolean only_name);
+static GwyFile*         load_file                 (const gchar *filename,
+                                                   GwyRunModeFlags mode,
+                                                   GError **error);
+static gboolean         export_file               (GwyFile *file,
+                                                   const gchar *filename,
+                                                   GwyRunModeFlags mode,
+                                                   GError **error);
+static GwyDialogOutcome run_gui                   (ModuleArgs *args,
+                                                   const gchar *title);
+static gboolean         export_fields             (const gchar *filename,
+                                                   GwyFile *file,
+                                                   gint id,
+                                                   const ModuleArgs *args,
+                                                   GError **error);
+static gint*            gather_compatible_fields  (GwyFile *data,
+                                                   GwyField *dfield,
+                                                   guint *nchannels);
+static gboolean         export_surfaces           (const gchar *filename,
+                                                   GwyFile *file,
+                                                   gint id,
+                                                   const ModuleArgs *args,
+                                                   GError **error);
+static gint*            gather_compatible_surfaces(GwyFile *data,
+                                                   GwySurface *surface,
+                                                   guint *nchannels);
+static gboolean         write_header              (FILE *fh,
+                                                   guint nchannels,
+                                                   guint npoints,
+                                                   gchar **titles,
+                                                   GwyUnit *xyunit,
+                                                   GwyUnit **zunits,
+                                                   gint xres,
+                                                   gint yres,
+                                                   GError **error);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Gwyddion XYZ field files."),
     "Yeti <yeti@gwyddion.net>",
-    "2.1",
+    "3.0",
     "David Nečas (Yeti)",
     "2013",
 };
@@ -124,10 +119,7 @@ module_register(void)
 {
     gwy_file_func_register("gxyzfile",
                            N_("GwyXYZ data files"),
-                           (GwyFileDetectFunc)&gxyzf_detect,
-                           (GwyFileLoadFunc)&gxyzf_load,
-                           NULL,
-                           (GwyFileSaveFunc)&gxyzf_export);
+                           detect_file, load_file, NULL, export_file);
 
     return TRUE;
 }
@@ -148,8 +140,7 @@ define_module_params(void)
 }
 
 static gint
-gxyzf_detect(const GwyFileDetectInfo *fileinfo,
-             gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     if (only_name)
         return g_str_has_suffix(fileinfo->name_lowercase, EXTENSION) ? 20 : 0;
@@ -174,12 +165,12 @@ append_double(gdouble *target, const gdouble v)
     *target = u.d;
 }
 
-static GwyContainer*
-gxyzf_load(const gchar *filename,
-           G_GNUC_UNUSED GwyRunType mode,
-           GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     GwyTextHeaderParser parser;
     GHashTable *hash = NULL;
     guchar *p, *value, *buffer = NULL, *header = NULL, *datap;
@@ -187,8 +178,8 @@ gxyzf_load(const gchar *filename,
     gdouble *points = NULL;
     gsize size;
     GError *err = NULL;
-    GwySIUnit **zunits = NULL;
-    GwySIUnit *xyunit = NULL, *zunit = NULL;
+    GwyUnit **zunits = NULL;
+    GwyUnit *xyunit = NULL, *zunit = NULL;
     guint nchan = 0, pointlen, pointsize, npoints, i, id;
 
     if (!g_file_get_contents(filename, (gchar**)&buffer, &size, &err)) {
@@ -210,7 +201,7 @@ gxyzf_load(const gchar *filename,
     header = g_strdup(p);
     datap += 8 - ((datap - buffer) % 8);
 
-    gwy_clear(&parser, 1);
+    gwy_clear1(parser);
     parser.key_value_separator = "=";
     if (!(hash = gwy_text_header_parse(header, &parser, NULL, NULL))) {
         g_propagate_error(error, err);
@@ -238,18 +229,18 @@ gxyzf_load(const gchar *filename,
     npoints = (size - (datap - buffer))/pointsize;
 
     value = g_hash_table_lookup(hash, "XYUnits");
-    xyunit = gwy_si_unit_new(value);
+    xyunit = gwy_unit_new(value);
 
     /* If there is ZUnits it applies to all channels. */
     if ((value = g_hash_table_lookup(hash, "ZUnits")))
-        zunit = gwy_si_unit_new(value);
+        zunit = gwy_unit_new(value);
     else {
-        zunits = g_new0(GwySIUnit*, nchan);
+        zunits = g_new0(GwyUnit*, nchan);
         for (id = 0; id < nchan; id++) {
-            gchar buf[16];
-            g_snprintf(buf, sizeof(buf), "ZUnits%u", id+1);
+            gchar *buf = g_strdup_printf("ZUnits%u", id+1);
             value = g_hash_table_lookup(hash, buf);
-            zunits[id] = gwy_si_unit_new(value);
+            g_free(buf);
+            zunits[id] = gwy_unit_new(value);
         }
     }
 
@@ -260,30 +251,30 @@ gxyzf_load(const gchar *filename,
         append_double(&xyzpoints[i].y, points[i*pointlen + 1]);
     }
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     for (id = 0; id < nchan; id++) {
         GwySurface *surface;
-        GwySIUnit *unit;
-        gchar buf[24];
+        GwyUnit *unit;
 
         for (i = 0; i < npoints; i++)
             append_double(&xyzpoints[i].z, points[i*pointlen + 2+id]);
 
         surface = gwy_surface_new_from_data(xyzpoints, npoints);
-        unit = gwy_surface_get_si_unit_z(surface);
+        unit = gwy_surface_get_unit_z(surface);
         if (zunit)
-            gwy_si_unit_assign(unit, zunit);
+            gwy_unit_assign(unit, zunit);
         else
-            gwy_si_unit_assign(unit, zunits[id]);
-        unit = gwy_surface_get_si_unit_xy(surface);
-        gwy_si_unit_assign(unit, xyunit);
+            gwy_unit_assign(unit, zunits[id]);
+        unit = gwy_surface_get_unit_xy(surface);
+        gwy_unit_assign(unit, xyunit);
 
-        gwy_container_pass_object(container, gwy_app_get_surface_key_for_id(id), surface);
+        gwy_file_pass_xyz(file, id, surface);
 
-        g_snprintf(buf, sizeof(buf), "Title%u", id+1);
+        gchar *buf = g_strdup_printf("Title%u", id+1);
         if ((value = g_hash_table_lookup(hash, buf)))
-            gwy_container_set_const_string(container, gwy_app_get_surface_title_key_for_id(id), value);
-        gwy_file_xyz_import_log_add(container, id, NULL, filename);
+            gwy_file_set_title(file, GWY_FILE_XYZ, id, value, FALSE);
+        g_free(buf);
+        gwy_log_add_import(file, GWY_FILE_XYZ, id, NULL, filename);
     }
 
 fail:
@@ -292,45 +283,45 @@ fail:
     g_free(buffer);
     if (hash)
         g_hash_table_destroy(hash);
-    GWY_OBJECT_UNREF(xyunit);
-    GWY_OBJECT_UNREF(zunit);
+    g_clear_object(&xyunit);
+    g_clear_object(&zunit);
     if (zunits) {
         for (i = 0; i < nchan; i++)
-            GWY_OBJECT_UNREF(zunits[i]);
+            g_clear_object(zunits + i);
         g_free(zunits);
     }
 
-    return container;
+    return file;
 }
 
 static gboolean
-gxyzf_export(G_GNUC_UNUSED GwyContainer *data,
-             const gchar *filename,
-             GwyRunType mode,
-             GError **error)
+export_file(G_GNUC_UNUSED GwyFile *data,
+            const gchar *filename,
+            GwyRunModeFlags mode,
+            GError **error)
 {
     ModuleArgs args;
-    GwyDataField *dfield;
+    GwyField *dfield;
     GwySurface *surface;
     gint fid, sid;
-    const guchar *title = NULL;
+    const gchar *title = NULL;
     GwyDialogOutcome outcome;
     gboolean ok = FALSE;
 
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &dfield,
-                                     GWY_APP_DATA_FIELD_ID, &fid,
-                                     GWY_APP_SURFACE, &surface,
-                                     GWY_APP_SURFACE_ID, &sid,
-                                     GWY_APP_PAGE, &args.pageno,
-                                     0);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &dfield,
+                                 GWY_APP_FIELD_ID, &fid,
+                                 GWY_APP_SURFACE, &surface,
+                                 GWY_APP_SURFACE_ID, &sid,
+                                 GWY_APP_DATA_KIND, &args.data_kind,
+                                 0);
 
     /* Ensure at most one is set.  We produce an error if no exportable data type is available or both types are
      * available but neither is active. When only one is available or one is active we assume that is what the user
      * wants to export. */
     if (dfield && surface) {
-        if (args.pageno != GWY_PAGE_CHANNELS)
+        if (args.data_kind != GWY_FILE_IMAGE)
             dfield = NULL;
-        if (args.pageno != GWY_PAGE_XYZS)
+        if (args.data_kind != GWY_FILE_XYZ)
             surface = NULL;
     }
     if (!dfield && !surface) {
@@ -339,16 +330,14 @@ gxyzf_export(G_GNUC_UNUSED GwyContainer *data,
     }
 
     if (dfield)
-        args.pageno = GWY_PAGE_CHANNELS;
+        args.data_kind = GWY_FILE_IMAGE;
     if (surface)
-        args.pageno = GWY_PAGE_XYZS;
+        args.data_kind = GWY_FILE_XYZ;
 
     args.params = gwy_params_new_from_settings(define_module_params());
-
-    if (dfield)
-        gwy_container_gis_string(data, gwy_app_get_data_title_key_for_id(fid), &title);
-    if (surface)
-        gwy_container_gis_string(data, gwy_app_get_surface_title_key_for_id(sid), &title);
+    title = gwy_file_get_title(data, args.data_kind, dfield ? fid : sid);
+    if (!title)
+        title = _("Untitled");
 
     if (mode == GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, title);
@@ -360,7 +349,7 @@ gxyzf_export(G_GNUC_UNUSED GwyContainer *data,
     }
 
     if (dfield)
-        ok = export_data_fields(filename, data, fid, &args, error);
+        ok = export_fields(filename, data, fid, &args, error);
     else if (surface)
         ok = export_surfaces(filename, data, sid, &args, error);
     else {
@@ -382,9 +371,9 @@ run_gui(ModuleArgs *args, const gchar *title)
     dialog = GWY_DIALOG(gwy_dialog_new(_("Export GXYZF")));
     gwy_dialog_add_buttons(dialog, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    if (args->pageno == GWY_PAGE_CHANNELS)
-        desc = _("Channel");
-    else if (args->pageno == GWY_PAGE_XYZS)
+    if (args->data_kind == GWY_FILE_IMAGE)
+        desc = _("Image");
+    else if (args->data_kind == GWY_FILE_XYZ)
         desc = _("XYZ data");
 
     table = gwy_param_table_new(args->params);
@@ -399,21 +388,21 @@ run_gui(ModuleArgs *args, const gchar *title)
 }
 
 static gboolean
-export_data_fields(const gchar *filename,
-                   GwyContainer *container,
-                   gint id,
-                   const ModuleArgs *args,
-                   GError **error)
+export_fields(const gchar *filename,
+              GwyFile *file,
+              gint id,
+              const ModuleArgs *args,
+              GError **error)
 {
     gdouble *ddbl = NULL;
     gint *ids = NULL;
     guint nchannels, ci, i, j, k, xres, yres;
     size_t npts;
-    GwyDataField *dfield, *other;
+    GwyField *dfield, *other;
     const gdouble **d;
     gdouble xreal, yreal, xoff, yoff;
     gchar **titles = NULL;
-    GwySIUnit *xyunit, **zunits = NULL;
+    GwyUnit *xyunit, **zunits = NULL;
     FILE *fh;
 
     if (!(fh = gwy_fopen(filename, "wb"))) {
@@ -421,19 +410,19 @@ export_data_fields(const gchar *filename,
         return FALSE;
     }
 
-    dfield = gwy_container_get_object(container, gwy_app_get_data_key_for_id(id));
+    dfield = gwy_file_get_image(file, id);
     g_return_val_if_fail(dfield, FALSE);
 
-    xres = gwy_data_field_get_xres(dfield);
-    yres = gwy_data_field_get_yres(dfield);
-    xreal = gwy_data_field_get_xreal(dfield);
-    yreal = gwy_data_field_get_yreal(dfield);
-    xoff = gwy_data_field_get_xoffset(dfield);
-    yoff = gwy_data_field_get_yoffset(dfield);
-    xyunit = gwy_data_field_get_si_unit_xy(dfield);
+    xres = gwy_field_get_xres(dfield);
+    yres = gwy_field_get_yres(dfield);
+    xreal = gwy_field_get_xreal(dfield);
+    yreal = gwy_field_get_yreal(dfield);
+    xoff = gwy_field_get_xoffset(dfield);
+    yoff = gwy_field_get_yoffset(dfield);
+    xyunit = gwy_field_get_unit_xy(dfield);
 
     if (gwy_params_get_boolean(args->params, PARAM_ALL_CHANNELS))
-        ids = gather_compatible_data_fields(container, dfield, &nchannels);
+        ids = gather_compatible_fields(file, dfield, &nchannels);
     else {
         nchannels = 1;
         ids = g_new(gint, 2);
@@ -442,14 +431,14 @@ export_data_fields(const gchar *filename,
     }
     g_return_val_if_fail(nchannels, FALSE);
 
-    zunits = g_new0(GwySIUnit*, nchannels + 1);
+    zunits = g_new0(GwyUnit*, nchannels + 1);
     titles = g_new0(gchar*, nchannels + 1);
     d = g_new0(const gdouble*, nchannels + 1);
     for (ci = 0; ci < nchannels; ci++) {
-        other = gwy_container_get_object(container, gwy_app_get_data_key_for_id(ids[ci]));
-        zunits[ci] = gwy_data_field_get_si_unit_z(other);
-        d[ci] = gwy_data_field_get_data_const(other);
-        titles[ci] = gwy_app_get_data_field_title(container, ids[ci]);
+        other = gwy_file_get_image(file, ids[ci]);
+        zunits[ci] = gwy_field_get_unit_z(other);
+        d[ci] = gwy_field_get_data_const(other);
+        titles[ci] = gwy_file_get_display_title(file, GWY_FILE_IMAGE, ids[ci]);
     }
 
     if (!write_header(fh, nchannels, xres*yres, titles, xyunit, zunits, xres, yres, error))
@@ -492,20 +481,16 @@ fail:
 }
 
 static gint*
-gather_compatible_data_fields(GwyContainer *container, GwyDataField *dfield,
-                              guint *nchannels)
+gather_compatible_fields(GwyFile *file, GwyField *dfield,
+                         guint *nchannels)
 {
-    GwyDataField *other;
-    gint *ids;
-    guint ci, n = 0;
+    gint *ids = gwy_file_get_ids(file, GWY_FILE_IMAGE);
+    guint n = 0;
 
-    ids = gwy_app_data_browser_get_data_ids(container);
-    for (ci = 0; ids[ci] > -1; ci++) {
-        other = gwy_container_get_object(container, gwy_app_get_data_key_for_id(ids[ci]));
-        if (gwy_data_field_check_compatibility(dfield, other,
-                                               GWY_DATA_COMPATIBILITY_RES
-                                               | GWY_DATA_COMPATIBILITY_REAL
-                                               | GWY_DATA_COMPATIBILITY_LATERAL))
+    for (guint ci = 0; ids[ci] > -1; ci++) {
+        GwyField *other = gwy_file_get_image(file, ids[ci]);
+        if (gwy_field_is_incompatible(dfield, other,
+                                      GWY_DATA_MISMATCH_RES | GWY_DATA_MISMATCH_REAL | GWY_DATA_MISMATCH_LATERAL))
             continue;
         ids[n] = ids[ci];
         n++;
@@ -518,7 +503,7 @@ gather_compatible_data_fields(GwyContainer *container, GwyDataField *dfield,
 
 static gboolean
 export_surfaces(const gchar *filename,
-                GwyContainer *container,
+                GwyFile *file,
                 gint id,
                 const ModuleArgs *args,
                 GError **error)
@@ -530,7 +515,7 @@ export_surfaces(const gchar *filename,
     GwySurface *surface, *other;
     const GwyXYZ **d;
     gchar **titles = NULL;
-    GwySIUnit *xyunit, **zunits = NULL;
+    GwyUnit *xyunit, **zunits = NULL;
     FILE *fh;
 
     if (!(fh = gwy_fopen(filename, "wb"))) {
@@ -538,14 +523,14 @@ export_surfaces(const gchar *filename,
         return FALSE;
     }
 
-    surface = gwy_container_get_object(container, gwy_app_get_surface_key_for_id(id));
+    surface = gwy_file_get_xyz(file, id);
     g_return_val_if_fail(surface, FALSE);
 
-    xyunit = gwy_surface_get_si_unit_xy(surface);
+    xyunit = gwy_surface_get_unit_xy(surface);
     n = gwy_surface_get_npoints(surface);
 
     if (gwy_params_get_boolean(args->params, PARAM_ALL_CHANNELS))
-        ids = gather_compatible_surfaces(container, surface, &nchannels);
+        ids = gather_compatible_surfaces(file, surface, &nchannels);
     else {
         nchannels = 1;
         ids = g_new(gint, 2);
@@ -554,14 +539,14 @@ export_surfaces(const gchar *filename,
     }
     g_return_val_if_fail(nchannels, FALSE);
 
-    zunits = g_new0(GwySIUnit*, nchannels + 1);
+    zunits = g_new0(GwyUnit*, nchannels + 1);
     titles = g_new0(gchar*, nchannels + 1);
     d = g_new0(const GwyXYZ*, nchannels + 1);
     for (ci = 0; ci < nchannels; ci++) {
-        other = gwy_container_get_object(container, gwy_app_get_surface_key_for_id(ids[ci]));
-        zunits[ci] = gwy_surface_get_si_unit_z(other);
+        other = gwy_file_get_xyz(file, ids[ci]);
+        zunits[ci] = gwy_surface_get_unit_z(other);
         d[ci] = gwy_surface_get_data_const(other);
-        titles[ci] = gwy_app_get_surface_title(container, ids[ci]);
+        titles[ci] = gwy_file_get_display_title(file, GWY_FILE_XYZ, ids[ci]);
     }
 
     if (!write_header(fh, nchannels, n, titles, xyunit, zunits, 0, 0, error))
@@ -604,16 +589,14 @@ fail:
 }
 
 static gint*
-gather_compatible_surfaces(GwyContainer *container, GwySurface *surface,
+gather_compatible_surfaces(GwyFile *file, GwySurface *surface,
                            guint *nchannels)
 {
-    GwySurface *other;
-    gint *ids;
-    guint ci, n = 0;
+    gint *ids = gwy_file_get_ids(file, GWY_FILE_XYZ);
+    guint n = 0;
 
-    ids = gwy_app_data_browser_get_xyz_ids(container);
-    for (ci = 0; ids[ci] > -1; ci++) {
-        other = gwy_container_get_object(container, gwy_app_get_surface_key_for_id(ids[ci]));
+    for (guint ci = 0; ids[ci] > -1; ci++) {
+        GwySurface *other = gwy_file_get_xyz(file, ids[ci]);
         if (!gwy_surface_xy_is_compatible(surface, other))
             continue;
 
@@ -628,7 +611,7 @@ gather_compatible_surfaces(GwyContainer *container, GwySurface *surface,
 
 static gboolean
 write_header(FILE *fh, guint nchannels, guint npoints,
-             gchar **titles, GwySIUnit *xyunit, GwySIUnit **zunits,
+             gchar **titles, GwyUnit *xyunit, GwyUnit **zunits,
              gint xres, gint yres,
              GError **error)
 {
@@ -641,15 +624,15 @@ write_header(FILE *fh, guint nchannels, guint npoints,
     g_string_append_printf(header, "NChannels = %u\n", nchannels);
     g_string_append_printf(header, "NPoints = %u\n", npoints);
 
-    if (!gwy_si_unit_equal_string(xyunit, NULL)) {
-        s = gwy_si_unit_get_string(xyunit, GWY_SI_UNIT_FORMAT_PLAIN);
+    if (!gwy_unit_equal_string(xyunit, NULL)) {
+        s = gwy_unit_get_string(xyunit, GWY_UNIT_FORMAT_PLAIN);
         g_string_append_printf(header, "XYUnits = %s\n", s);
         g_free(s);
     }
 
     for (i = 0; i < nchannels; i++) {
-        if (!gwy_si_unit_equal_string(zunits[i], NULL)) {
-            s = gwy_si_unit_get_string(zunits[i], GWY_SI_UNIT_FORMAT_PLAIN);
+        if (!gwy_unit_equal_string(zunits[i], NULL)) {
+            s = gwy_unit_get_string(zunits[i], GWY_UNIT_FORMAT_PLAIN);
             g_string_append_printf(header, "ZUnits%u = %s\n", i+1, s);
             g_free(s);
         }
