@@ -1,6 +1,6 @@
 /*
- *  $Id: hdf4file.c 26163 2024-02-06 16:56:55Z yeti-dn $
- *  Copyright (C) 2006 David Necas (Yeti), Petr Klapetek, Markus Pristovsek
+ *  $Id: hdf4file.c 28911 2025-11-24 18:27:42Z yeti-dn $
+ *  Copyright (C) 2006-2025 David Necas (Yeti), Petr Klapetek, Markus Pristovsek
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net,
  *  prissi@gift.physik.tu-berlin.de.
  *
@@ -69,14 +69,9 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/datafield.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 
 /* We do not have actually a szlib (normally a dependency of HDF5) configure check because the code does not work
  * anyway. */
@@ -294,10 +289,10 @@ typedef struct {
 } PsiHeader;
 
 static gboolean      module_register   (void);
-static gint          psi_detect        (const GwyFileDetectInfo *fileinfo,
+static gint          detect_file       (const GwyFileDetectInfo *fileinfo,
                                         gboolean only_name);
-static GwyContainer* psi_load          (const gchar *filename,
-                                        GwyRunType mode,
+static GwyFile*      load_file         (const gchar *filename,
+                                        GwyRunModeFlags mode,
                                         GError **error);
 static gboolean      psi_read_header   (const guchar *buffer,
                                         gsize size,
@@ -307,7 +302,6 @@ static GArray*       hdf4_read_tags    (const guchar *buffer,
                                         gsize size,
                                         GError **error);
 static GwyContainer* psi_get_metadadata(const PsiHeader *header);
-
 #ifdef DEBUG
 static guint        get_data_type_size     (HDF4TypeInfo id,
                                             GError **error);
@@ -321,7 +315,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports Hierarchical Data Format (HDF) files, version 4."),
     "Yeti <yeti@gwyddion.net>",
-    "0.8",
+    "0.9",
     "David Nečas (Yeti) & Petr Klapetek",
     "2006",
 };
@@ -333,17 +327,13 @@ module_register(void)
 {
     gwy_file_func_register("psifile",
                            N_("Psi HDF4 files (.hdf)"),
-                           (GwyFileDetectFunc)&psi_detect,
-                           (GwyFileLoadFunc)&psi_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-psi_detect(const GwyFileDetectInfo *fileinfo,
-           gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     gint score = 0;
 
@@ -375,18 +365,17 @@ psi_detect(const GwyFileDetectInfo *fileinfo,
             g_array_free(tags, TRUE);
         }
         gwy_file_abandon_contents(buffer, size, NULL);
-
     }
 
     return score;
 }
 
-static GwyContainer*
-psi_load(const gchar *filename,
-         G_GNUC_UNUSED GwyRunType mode,
-         GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     guchar *buffer = NULL;
     GError *err = NULL;
     PsiHeader *header = NULL;
@@ -458,7 +447,7 @@ psi_load(const gchar *filename,
         SZ_com_t params;
         gint status;
 
-        gwy_clear(&params, 1);
+        gwy_clear1(params);
         //params.bits_per_pixel = 16;
         //params.pixels_per_scanline = header->xres;
         params.pixels_per_block = 1;
@@ -488,31 +477,30 @@ psi_load(const gchar *filename,
         fail = err_SIZE_MISMATCH(error, expected_size, data_size, FALSE);
 
     if (!fail) {
-        GwyDataField *dfield;
+        GwyField *dfield;
         gint power10;
 
-        dfield = gwy_data_field_new(header->xres, header->yres,
-                                    header->xscale * Micrometer, header->yscale * Micrometer,
-                                    FALSE);
+        dfield = gwy_field_new(header->xres, header->yres,
+                               header->xscale * Micrometer, header->yscale * Micrometer,
+                               FALSE);
 
-        d = gwy_data_field_get_data(dfield);
+        d = gwy_field_get_data(dfield);
         gwy_convert_raw_data(data, header->xres*header->yres, 1,
                              GWY_RAW_DATA_SINT16, GWY_BYTE_ORDER_LITTLE_ENDIAN, d, 1.0, 0.0);
 
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
-        gwy_si_unit_set_from_string_parse(gwy_data_field_get_si_unit_z(dfield),
-                                          header->zgain_unit, &power10);
+        gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
+        power10 = gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), header->zgain_unit);
 
-        gwy_data_field_multiply(dfield, header->zgain*pow10(power10));
-        gwy_data_field_invert(dfield, TRUE, FALSE, FALSE);
+        gwy_field_multiply(dfield, header->zgain*gwy_exp10(power10));
+        gwy_field_flip(dfield, FALSE, TRUE);
 
-        container = gwy_container_new();
-        gwy_container_pass_object(container, gwy_app_get_data_key_for_id(0), dfield);
+        file = gwy_file_new_in_construction();
+        gwy_file_pass_image(file, 0, dfield);
         if (header->title[0])
-            gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(0), header->title);
-        gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(0), psi_get_metadadata(header));
+            gwy_file_set_title(file, GWY_FILE_IMAGE, 0, header->title, FALSE);
+        gwy_file_pass_meta(file, GWY_FILE_IMAGE, 0, psi_get_metadadata(header));
 
-        gwy_file_channel_import_log_add(container, 0, NULL, filename);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
     }
 
     gwy_file_abandon_contents(buffer, size, NULL);
@@ -522,10 +510,10 @@ psi_load(const gchar *filename,
     g_free(uncdata);
 #endif
 
-    if (!container && !fail)
+    if (!file && !fail)
         err_NO_DATA(error);
 
-    return container;
+    return file;
 }
 
 static gboolean
@@ -594,9 +582,7 @@ psi_read_header(const guchar *buffer,
 static GwyContainer*
 psi_get_metadadata(const PsiHeader *header)
 {
-    GwyContainer *meta;
-
-    meta = gwy_container_new();
+    GwyContainer *meta = gwy_container_new_in_construction();
 
     gwy_container_set_string_by_name(meta, "Set point",
                                      g_strdup_printf("%g %s", header->set_point, header->set_point_unit));
