@@ -1,5 +1,5 @@
 /*
- *  $Id: datxfile.c 28533 2025-09-10 14:07:50Z yeti-dn $
+ *  $Id: datxfile.c 28911 2025-11-24 18:27:42Z yeti-dn $
  *  Copyright (C) 2020-2025 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -37,11 +37,8 @@
  **/
 
 #include "config.h"
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/datafield.h>
-#include <libprocess/correct.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils-file.h>
+#include <glib/gi18n-lib.h>
+#include <gwy.h>
 
 #include "gwyhdf5.h"
 #include "hdf5file.h"
@@ -67,41 +64,38 @@ typedef struct {
     DatxMetadata *metadata;
 } DatxFile;
 
-static gint           datx_detect              (const GwyFileDetectInfo *fileinfo,
-                                                gboolean only_name);
-static GwyContainer*  datx_load                (const gchar *filename,
-                                                GwyRunType mode,
-                                                GError **error);
-static gboolean       datx_read_images         (const DatxFile *dfile,
-                                                hid_t file_id,
-                                                GwyContainer *container,
-                                                gboolean surfaces,
-                                                gint *id,
-                                                GError **error);
-static GwyDataField*  channel_mask_of_invalid  (GwyDataField *dfield,
-                                                gboolean removebad);
-static gboolean       datx_read_metadata       (DatxFile *dfile,
-                                                hid_t file_id,
-                                                GError **error);
-static DatxConverter* datx_get_converter_params(hid_t file_id,
-                                                const gchar *path,
-                                                const gchar *name);
-static void           datx_converter_free      (DatxConverter *converter);
-static void           datx_free                (DatxFile *dfile);
+static gint           detect_file            (const GwyFileDetectInfo *fileinfo,
+                                              gboolean only_name);
+static GwyFile*       load_file              (const gchar *filename,
+                                              GwyRunModeFlags mode,
+                                              GError **error);
+static gboolean       read_images            (const DatxFile *dfile,
+                                              hid_t file_id,
+                                              GwyFile *file,
+                                              gboolean surfaces,
+                                              gint *id,
+                                              GError **error);
+static GwyField*      channel_mask_of_invalid(GwyField *dfield,
+                                              gboolean removebad);
+static gboolean       read_metadata          (DatxFile *dfile,
+                                              hid_t file_id,
+                                              GError **error);
+static DatxConverter* get_converter_params   (hid_t file_id,
+                                              const gchar *path,
+                                              const gchar *name);
+static void           free_datx_converter    (DatxConverter *converter);
+static void           free_datxfile          (DatxFile *dfile);
 
 void
 gwyhdf5_register_datxfile(void)
 {
     gwy_file_func_register("datxfile",
                            N_("Zygo DATX HDF5 files (.datx)"),
-                           (GwyFileDetectFunc)&datx_detect,
-                           (GwyFileLoadFunc)&datx_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 }
 
 static gint
-datx_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     hid_t file_id;
     gint version, dims = 1, score = 0;
@@ -121,12 +115,12 @@ datx_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
     return score;
 }
 
-static GwyContainer*
-datx_load(const gchar *filename,
-          G_GNUC_UNUSED GwyRunType mode,
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
           GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     GwyHDF5File ghfile;
     DatxFile dfile;
     hid_t file_id;
@@ -139,51 +133,50 @@ datx_load(const gchar *filename,
         return NULL;
     }
     gwy_debug("file_id %d", (gint)file_id);
-    status = H5Oget_info(file_id, &infobuf);
+    status = H5Oget_info(file_id, &infobuf, H5O_INFO_BASIC);
     if (!gwyhdf5_check_status(status, file_id, NULL, "H5Oget_info", error))
         return NULL;
 
-    gwy_clear(&dfile, 1);
+    gwy_clear1(dfile);
     dfile.filename = filename;
-    gwyhdf5_init(&ghfile);
+    gwyhdf5_init(&ghfile, file_id, infobuf.token);
     ghfile.impl = &dfile;
-    g_array_append_val(ghfile.addr, infobuf.addr);
 
-    if (!datx_read_metadata(&dfile, file_id, error))
+    if (!read_metadata(&dfile, file_id, error))
         goto fail;
 
     status = H5Literate(file_id, H5_INDEX_NAME, H5_ITER_NATIVE, NULL, gwyhdf5_scan_file, &ghfile);
     if (!gwyhdf5_check_status(status, file_id, &ghfile, "H5Literate", error)) {
-        datx_free(&dfile);
+        free_datxfile(&dfile);
         return NULL;
     }
     /* Other file formats also do H5Aiterate2() but here we do not seem to get anything useful. */
 
-    container = gwy_container_new();
-    if (!datx_read_images(&dfile, file_id, container, TRUE, &id, error)
-        || !datx_read_images(&dfile, file_id, container, FALSE, &id, error))
-        GWY_OBJECT_UNREF(container);
-    else if (!gwy_container_get_n_items(container)) {
-        GWY_OBJECT_UNREF(container);
+    file = gwy_file_new_in_construction();
+    if (!read_images(&dfile, file_id, file, TRUE, &id, error)
+        || !read_images(&dfile, file_id, file, FALSE, &id, error))
+        g_clear_object(&file);
+    else if (!gwy_container_get_n_items(GWY_CONTAINER(file))) {
+        g_clear_object(&file);
         err_NO_DATA(error);
     }
 
 fail:
     status = gwyhdf5_fclose(file_id);
     gwy_debug("status %d", status);
-    datx_free(&dfile);
+    free_datxfile(&dfile);
     gwyhdf5_free(&ghfile);
 
-    return container;
+    return file;
 }
 
 static gboolean
-datx_read_images(const DatxFile *dfile, hid_t file_id, GwyContainer *container,
-                 gboolean surfaces,
-                 gint *id, GError **error)
+read_images(const DatxFile *dfile, hid_t file_id, GwyFile *file,
+            gboolean surfaces,
+            gint *id, GError **error)
 {
     gint dims[2] = { -1, -1 };
-    GwyDataField *field, *mask;
+    GwyField *field, *mask;
     const DatxMetadata *m;
     DatxConverter *converter;
     GString *title;
@@ -222,27 +215,27 @@ datx_read_images(const DatxFile *dfile, hid_t file_id, GwyContainer *container,
             goto fail;
 
         xscale = yscale = zscale = 1.0;
-        if ((converter = datx_get_converter_params(file_id, m->destination, "X Converter"))
+        if ((converter = get_converter_params(file_id, m->destination, "X Converter"))
             && converter->nparams > 1
             && gwy_strequal(converter->category, "LateralCat")
             && gwy_strequal(converter->base_unit, "Pixels"))
             xscale = converter->params[1];
         else
             g_warning("Cannot obtain X Converter");
-        datx_converter_free(converter);
+        free_datx_converter(converter);
         sanitise_real_size(&xscale, "x scale");
 
-        if ((converter = datx_get_converter_params(file_id, m->destination, "Y Converter"))
+        if ((converter = get_converter_params(file_id, m->destination, "Y Converter"))
             && converter->nparams > 1
             && gwy_strequal(converter->category, "LateralCat")
             && gwy_strequal(converter->base_unit, "Pixels"))
             yscale = converter->params[1];
         else
             g_warning("Cannot obtain Y Converter");
-        datx_converter_free(converter);
+        free_datx_converter(converter);
         sanitise_real_size(&yscale, "y scale");
 
-        converter = datx_get_converter_params(file_id, m->destination, "Z Converter");
+        converter = get_converter_params(file_id, m->destination, "Z Converter");
         if (surfaces) {
             if (converter && converter->nparams > 1 && gwy_strequal(converter->category, "HeightCat"))
                 zscale = converter->params[1];
@@ -256,8 +249,8 @@ datx_read_images(const DatxFile *dfile, hid_t file_id, GwyContainer *container,
              * For intensity there is basically no conversion so we just keep the values as they are. */
         }
 
-        field = gwy_data_field_new(dims[1], dims[0], xscale*dims[1], yscale*dims[0], FALSE);
-        status = H5Dread(dataset, H5T_NATIVE_DOUBLE, H5S_ALL, H5S_ALL, H5P_DEFAULT, gwy_data_field_get_data(field));
+        field = gwy_field_new(dims[1], dims[0], xscale*dims[1], yscale*dims[0], FALSE);
+        status = H5Dread(dataset, H5T_NATIVE_DOUBLE, H5S_ALL, H5S_ALL, H5P_DEFAULT, gwy_field_get_data(field));
         if (status < 0) {
             id = 0;
             g_object_unref(field);
@@ -268,23 +261,23 @@ datx_read_images(const DatxFile *dfile, hid_t file_id, GwyContainer *container,
         /* FIXME: The lateral unit should be probably the same as for Resolutions, e.g.
          * /Attributes/{UUID}/{ChannelName} Data Context/Lateral Resolution/Unit
          * But it probably cannot be anything else than metre anyway. */
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(field), "m");
+        gwy_unit_set_from_string(gwy_field_get_unit_xy(field), "m");
         if (surfaces && converter) {
-            gwy_si_unit_set_from_string_parse(gwy_data_field_get_si_unit_z(field), converter->base_unit, &power10);
-            zscale = pow10(power10);
+            power10 = gwy_unit_set_from_string(gwy_field_get_unit_z(field), converter->base_unit);
+            zscale = gwy_exp10(power10);
         }
 
         /* Create the mask first. Applying units can mess up the invalid value detection. */
         mask = channel_mask_of_invalid(field, TRUE);
-        gwy_data_field_multiply(field, zscale);
-        datx_converter_free(converter);
+        gwy_field_multiply(field, zscale);
+        free_datx_converter(converter);
 
         /* If we see image called "Surface", put it to id 0. */
-        gwy_container_pass_object(container, gwy_app_get_data_key_for_id(*id), field);
+        gwy_file_pass_image(file, *id, field);
         if (mask)
-            gwy_container_pass_object(container, gwy_app_get_mask_key_for_id(*id), mask);
-        gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(*id), title->str);
-        gwy_file_channel_import_log_add(container, *id, NULL, dfile->filename);
+            gwy_file_pass_image_mask(file, *id, mask);
+        gwy_file_set_title(file, GWY_FILE_IMAGE, *id, title->str, FALSE);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, *id, NULL, dfile->filename);
         (*id)++;
     }
 
@@ -299,20 +292,20 @@ fail:
 /* Like gwy_app_channel_mask_of_nans(), but the invalid data are not NaNs, they are large finite values.
  * FIXME: We should probably have ‘also mark suspiciously huge values’ as an option for gwy_app_channel_mask_of_nans()
  * but it is an API change, so do it in 3.x. */
-static GwyDataField*
-channel_mask_of_invalid(GwyDataField *dfield, gboolean removebad)
+static GwyField*
+channel_mask_of_invalid(GwyField *dfield, gboolean removebad)
 {
-    GwyDataField *mask = NULL;
+    GwyField *mask = NULL;
     guint k, n = dfield->xres*dfield->yres;
-    gdouble *d = gwy_data_field_get_data(dfield), *m = NULL;
+    gdouble *d = gwy_field_get_data(dfield), *m = NULL;
 
     for (k = 0; k < n; k++) {
         /* Probably anything larger than √G_MAXDOUBLE is suspicious, but this should be a sufficient margin. */
         if (gwy_isnan(d[k]) || gwy_isinf(d[k]) || fabs(d[k]) > 1e-9*G_MAXDOUBLE) {
             if (G_UNLIKELY(!mask)) {
-                mask = gwy_data_field_new_alike(dfield, TRUE);
-                m = gwy_data_field_get_data(mask);
-                gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(mask), NULL);
+                mask = gwy_field_new_alike(dfield, TRUE);
+                m = gwy_field_get_data(mask);
+                gwy_unit_clear(gwy_field_get_unit_z(mask));
             }
             m[k] = 1.0;
         }
@@ -321,7 +314,7 @@ channel_mask_of_invalid(GwyDataField *dfield, gboolean removebad)
     if (!mask || !removebad)
         return mask;
 
-    gwy_data_field_laplace_solve(dfield, mask, -1, 0.25);
+    gwy_field_laplace_solve(dfield, mask, -1, 0.25);
 
     return mask;
 }
@@ -346,7 +339,7 @@ channel_mask_of_invalid(GwyDataField *dfield, gboolean removebad)
  * There is also group "Processed Data: " with hardlinks to all these.
  */
 static gboolean
-datx_read_metadata(DatxFile *dfile, hid_t file_id, GError **error)
+read_metadata(DatxFile *dfile, hid_t file_id, GError **error)
 {
     static const gchar *member_names[3] = { "Source", "Link", "Destination" };
     hid_t metadata_id, member_type = -1, file_member_type = -1, str_type = -1, metadata_type = -1;
@@ -480,7 +473,7 @@ end:
 }
 
 static gchar*
-datx_get_compound_string(hid_t attr, gint i, const gchar *name)
+get_compound_string(hid_t attr, gint i, const gchar *name)
 {
     hid_t attr_type, member_type, str_type, file_member_type;
     gchar *v, *s;
@@ -513,7 +506,7 @@ datx_get_compound_string(hid_t attr, gint i, const gchar *name)
 }
 
 static DatxConverter*
-datx_get_converter_params(hid_t file_id, const gchar *path, const gchar *name)
+get_converter_params(hid_t file_id, const gchar *path, const gchar *name)
 {
     DatxConverter converter;
     hid_t attr, attr_type, space, member_type, item_type;
@@ -569,8 +562,8 @@ datx_get_converter_params(hid_t file_id, const gchar *path, const gchar *name)
     H5Dvlen_reclaim(member_type, space, H5P_DEFAULT, &v);
     H5Sclose(space);
 
-    converter.category = datx_get_compound_string(attr, icateogry, "Category");
-    converter.base_unit = datx_get_compound_string(attr, ibaseunit, "BaseUnit");
+    converter.category = get_compound_string(attr, icateogry, "Category");
+    converter.base_unit = get_compound_string(attr, ibaseunit, "BaseUnit");
     H5Aclose(attr);
 
     if (!converter.category || !converter.base_unit) {
@@ -584,7 +577,7 @@ datx_get_converter_params(hid_t file_id, const gchar *path, const gchar *name)
 }
 
 static void
-datx_converter_free(DatxConverter *converter)
+free_datx_converter(DatxConverter *converter)
 {
     if (!converter)
         return;
@@ -595,7 +588,7 @@ datx_converter_free(DatxConverter *converter)
 }
 
 static void
-datx_free(DatxFile *dfile)
+free_datxfile(DatxFile *dfile)
 {
     DatxMetadata *m;
     gint i;
