@@ -1,6 +1,6 @@
 /*
- *  $Id: epflfile.c 28533 2025-09-10 14:07:50Z yeti-dn $
- *  Copyright (C) 2020-2024 David Necas (Yeti), Petr Klapetek.
+ *  $Id: epflfile.c 28911 2025-11-24 18:27:42Z yeti-dn $
+ *  Copyright (C) 2020-2025 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -28,10 +28,8 @@
  **/
 
 #include "config.h"
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/datafield.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils-file.h>
+#include <glib/gi18n-lib.h>
+#include <gwy.h>
 
 #include "gwyhdf5.h"
 #include "hdf5file.h"
@@ -79,17 +77,17 @@ typedef struct {
     const gchar *filename;
 } EPFLFile;
 
-static gint          epfl_detect             (const GwyFileDetectInfo *fileinfo,
+static gint          detect_file             (const GwyFileDetectInfo *fileinfo,
                                               gboolean only_name);
-static GwyContainer* epfl_load               (const gchar *filename,
-                                              GwyRunType mode,
+static GwyFile*      load_file               (const gchar *filename,
+                                              GwyRunModeFlags mode,
                                               GError **error);
-static void          epfl_create_image       (hid_t file_id,
+static void          create_image            (hid_t file_id,
                                               const gchar *dataset_path,
                                               GwyHDF5File *ghfile,
-                                              GwyContainer *container,
+                                              GwyFile *file,
                                               gint *id);
-static GwyContainer* epfl_gather_meta        (hid_t file_id,
+static GwyContainer* gather_meta             (hid_t file_id,
                                               GwyHDF5File *ghfile,
                                               const gchar *basekey);
 static gboolean      read_usid_dimension_info(hid_t file_id,
@@ -102,15 +100,11 @@ gwyhdf5_register_epflfile(void)
 {
     gwy_file_func_register("epflfile",
                            N_("EPFL HDF5 files (.h5)"),
-                           (GwyFileDetectFunc)&epfl_detect,
-                           (GwyFileLoadFunc)&epfl_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 }
 
 static gint
-epfl_detect(const GwyFileDetectInfo *fileinfo,
-            gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     static const gchar *groups[] = {
         "electron microscopy measurements", "fluorescence measurements", "spm measurements",
@@ -146,12 +140,12 @@ epfl_detect(const GwyFileDetectInfo *fileinfo,
     return score;
 }
 
-static GwyContainer*
-epfl_load(const gchar *filename,
-          G_GNUC_UNUSED GwyRunType mode,
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
           GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     GwyHDF5File ghfile;
     EPFLFile efile;
     hid_t file_id;
@@ -165,12 +159,11 @@ epfl_load(const gchar *filename,
         return NULL;
     }
     gwy_debug("file_id %d", (gint)file_id);
-    status = H5Oget_info(file_id, &infobuf);
+    status = H5Oget_info(file_id, &infobuf, H5O_INFO_BASIC);
     if (!gwyhdf5_check_status(status, file_id, NULL, "H5Oget_info", error))
         return NULL;
 
-    gwyhdf5_init(&ghfile);
-    g_array_append_val(ghfile.addr, infobuf.addr);
+    gwyhdf5_init(&ghfile, file_id, infobuf.token);
     ghfile.impl = &efile;
     efile.filename = filename;
 
@@ -179,37 +172,37 @@ epfl_load(const gchar *filename,
         return NULL;
     /* Other file formats also do H5Aiterate2() but here we do not seem to get anything useful. */
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     id = 0;
     for (i = 0; i < ghfile.datasets->len; i++)
-        epfl_create_image(file_id, g_array_index(ghfile.datasets, gchar*, i), &ghfile, container, &id);
+        create_image(file_id, g_array_index(ghfile.datasets, gchar*, i), &ghfile, file, &id);
 
     status = gwyhdf5_fclose(file_id);
     gwy_debug("status %d", status);
 
     gwyhdf5_free(&ghfile);
 
-    if (!gwy_container_get_n_items(container)) {
-        GWY_OBJECT_UNREF(container);
+    if (!gwy_container_get_n_items(GWY_CONTAINER(file))) {
+        g_clear_object(&file);
         err_NO_DATA(error);
     }
 
-    return container;
+    return file;
 }
 
 static void
-epfl_create_image(hid_t file_id,
-                  const gchar *dataset_path,
-                  GwyHDF5File *ghfile,
-                  GwyContainer *container,
-                  gint *id)
+create_image(hid_t file_id,
+             const gchar *dataset_path,
+             GwyHDF5File *ghfile,
+             GwyFile *file,
+             gint *id)
 {
     GwyContainer *meta, *hash = ghfile->meta;
     EPFLFile *efile = (EPFLFile*)ghfile->impl;
     gint dims[2] = { -1, -1 };
     const gchar *name;
-    const guchar *s;
-    GwyDataField *field;
+    const gchar *s;
+    GwyField *field;
     GString *key;
     gchar *end;
     hid_t dataset;
@@ -238,7 +231,7 @@ epfl_create_image(hid_t file_id,
     g_string_append_c(key, '/');
     klen = key->len;
 
-    if (!(meta = epfl_gather_meta(file_id, ghfile, key->str))) {
+    if (!(meta = gather_meta(file_id, ghfile, key->str))) {
         g_string_free(key, TRUE);
         H5Dclose(dataset);
         return;
@@ -252,8 +245,8 @@ epfl_create_image(hid_t file_id,
         return;
     }
 
-    field = gwy_data_field_new(dims[1], dims[0], dims[1], dims[0], FALSE);
-    status = H5Dread(dataset, H5T_NATIVE_DOUBLE, H5S_ALL, H5S_ALL, H5P_DEFAULT, gwy_data_field_get_data(field));
+    field = gwy_field_new(dims[1], dims[0], dims[1], dims[0], FALSE);
+    status = H5Dread(dataset, H5T_NATIVE_DOUBLE, H5S_ALL, H5S_ALL, H5P_DEFAULT, gwy_field_get_data(field));
     gwy_debug("status %d", status);
     H5Dclose(dataset);
     if (status < 0) {
@@ -262,24 +255,24 @@ epfl_create_image(hid_t file_id,
         return;
     }
 
-    gwy_container_pass_object(container, gwy_app_get_data_key_for_id(*id), field);
-    gwy_file_channel_import_log_add(container, *id, NULL, efile->filename);
+    gwy_file_pass_image(file, *id, field);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, *id, NULL, efile->filename);
 
     /* The two labels are usually the same. */
     s = name;
     g_string_truncate(key, klen);
     g_string_append(key, "quantity");
     gwy_container_gis_string_by_name(hash, key->str, &s);
-    gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(*id), s);
+    gwy_file_set_title(file, GWY_FILE_IMAGE, *id, s, FALSE);
 
     /* Value units. */
     g_string_truncate(key, klen);
     g_string_append(key, "units");
     if (gwy_container_gis_string_by_name(hash, key->str, &s)) {
         gwy_debug("found units <%s>", s);
-        gwy_si_unit_set_from_string_parse(gwy_data_field_get_si_unit_z(field), s, &power10);
+        power10 = gwy_unit_set_from_string(gwy_field_get_unit_z(field), s);
         if (power10)
-            gwy_data_field_multiply(field, pow10(power10));
+            gwy_field_multiply(field, gwy_exp10(power10));
     }
     else {
         gwy_debug("did NOT find units at %s", key->str);
@@ -290,9 +283,9 @@ epfl_create_image(hid_t file_id,
         gwy_debug("found Width <%s>", s);
         u = g_ascii_strtod(s, &end);
         sanitise_real_size(&u, "x size");
-        gwy_si_unit_set_from_string_parse(gwy_data_field_get_si_unit_xy(field), end, &power10);
-        u *= pow10(power10);
-        gwy_data_field_set_xreal(field, u);
+        power10 = gwy_unit_set_from_string(gwy_field_get_unit_xy(field), end);
+        u *= gwy_exp10(power10);
+        gwy_field_set_xreal(field, u);
     }
     else {
         gwy_debug("did NOT find Width");
@@ -302,17 +295,18 @@ epfl_create_image(hid_t file_id,
         gwy_debug("found Height <%s>", s);
         u = g_ascii_strtod(s, &end);
         sanitise_real_size(&u, "y size");
-        gwy_si_unit_set_from_string_parse(gwy_data_field_get_si_unit_xy(field), end, &power10);
-        u *= pow10(power10);
-        gwy_data_field_set_yreal(field, u);
+        power10 = gwy_unit_set_from_string(gwy_field_get_unit_xy(field), end);
+        u *= gwy_exp10(power10);
+        gwy_field_set_yreal(field, u);
     }
     else {
         gwy_debug("did NOT find Height");
     }
 
     if (gwy_container_get_n_items(meta))
-        gwy_container_set_object(container, gwy_app_get_data_meta_key_for_id(*id), meta);
-    g_object_unref(meta);
+        gwy_file_pass_meta(file, GWY_FILE_IMAGE, *id, meta);
+    else
+        g_object_unref(meta);
 
     g_string_free(key, TRUE);
 
@@ -357,7 +351,7 @@ scan_meta_group(G_GNUC_UNUSED hid_t loc_id,
 }
 
 static GwyContainer*
-epfl_gather_meta(hid_t file_id, GwyHDF5File *ghfile, const gchar *basekey)
+gather_meta(hid_t file_id, GwyHDF5File *ghfile, const gchar *basekey)
 {
     /* There is a lot of rubbish. So use a whitelist. */
     static const gchar *dataset_attributes[] = {
@@ -368,8 +362,8 @@ epfl_gather_meta(hid_t file_id, GwyHDF5File *ghfile, const gchar *basekey)
     static const gchar *group_attributes[] = {
         "Width", "Height", "Date", "Mode", "Lines", "Scan Rate", "X Offset", "Y Offset",
     };
-    GwyContainer *meta = gwy_container_new();
-    const guchar *s;
+    GwyContainer *meta = gwy_container_new_in_construction();
+    const gchar *s;
     GroupMetaData gmd;
     GString *key;
     hid_t group;
@@ -435,7 +429,7 @@ read_usid_dimension_info(hid_t file_id,
     GwyContainer *hash = ghfile->meta;
     GString *key = g_string_new(NULL);
     gboolean ok = FALSE;
-    const guchar *s;
+    const gchar *s;
     hid_t dataset;
 
     /* Data. */
