@@ -1,6 +1,6 @@
 /*
- *  $Id: ergofile.c 28724 2025-10-23 13:43:19Z yeti-dn $
- *  Copyright (C) 2020-2024 David Necas (Yeti), Petr Klapetek.
+ *  $Id: ergofile.c 28848 2025-11-10 18:54:08Z yeti-dn $
+ *  Copyright (C) 2020-2025 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -32,10 +32,8 @@
  **/
 
 #include "config.h"
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/datafield.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils-file.h>
+#include <glib/gi18n-lib.h>
+#include <gwy.h>
 
 #include "gwyhdf5.h"
 #include "hdf5file.h"
@@ -43,8 +41,8 @@
 /* Asylym Research Ergo */
 typedef struct {
     gchar *name;
-    GwySIUnit *xyunit;
-    GwySIUnit *zunit;
+    GwyUnit *xyunit;
+    GwyUnit *zunit;
     gint xypower10;
     gint zpower10;
     gdouble realcoords[4];
@@ -57,40 +55,36 @@ typedef struct {
     gint version[3];
 } ErgoFile;
 
-static gint          ergo_detect       (const GwyFileDetectInfo *fileinfo,
-                                        gboolean only_name);
-static GwyContainer* ergo_load         (const gchar *filename,
-                                        GwyRunType mode,
-                                        GError **error);
-static void          ergo_attr_handler (GwyHDF5File *ghfile,
-                                        hid_t loc_id,
-                                        const char *attr_name);
-static GwyContainer* ergo_read_channels(hid_t file_id,
-                                        GwyHDF5File *ghfile,
-                                        GError **error);
-static GwyDataField* ergo_read_field   (hid_t file_id,
-                                        gint version_major,
-                                        guint r,
-                                        ErgoChannel *c,
-                                        gint frameid,
-                                        const gint *yxres,
-                                        GString *str,
-                                        GError **error);
+static gint      detect_file  (const GwyFileDetectInfo *fileinfo,
+                               gboolean only_name);
+static GwyFile*  load_file    (const gchar *filename,
+                               GwyRunModeFlags mode,
+                               GError **error);
+static void      attr_handler (GwyHDF5File *ghfile,
+                               hid_t loc_id,
+                               const char *attr_name);
+static GwyFile*  read_channels(hid_t file_id,
+                               GwyHDF5File *ghfile,
+                               GError **error);
+static GwyField* read_field   (hid_t file_id,
+                               gint version_major,
+                               guint r,
+                               ErgoChannel *c,
+                               gint frameid,
+                               const gint *yxres,
+                               GString *str,
+                               GError **error);
 
 void
 gwyhdf5_register_ergofile(void)
 {
     gwy_file_func_register("ergofile",
                            N_("Asylum Research Ergo HDF5 files (.h5,.aris)"),
-                           (GwyFileDetectFunc)&ergo_detect,
-                           (GwyFileLoadFunc)&ergo_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 }
 
 static gint
-ergo_detect(const GwyFileDetectInfo *fileinfo,
-            gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     hid_t file_id;
     gchar *format = NULL;
@@ -113,12 +107,13 @@ ergo_detect(const GwyFileDetectInfo *fileinfo,
     return score;
 }
 
-static GwyContainer*
-ergo_load(const gchar *filename,
-          G_GNUC_UNUSED GwyRunType mode,
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
           GError **error)
 {
-    GwyContainer *meta, *container = NULL;
+    GwyFile *file = NULL;
+    GwyContainer *meta;
     GwyHDF5File ghfile;
     ErgoFile efile;
     hid_t file_id;
@@ -133,19 +128,19 @@ ergo_load(const gchar *filename,
         return NULL;
     }
     gwy_debug("file_id %d", (gint)file_id);
-    status = H5Oget_info(file_id, &infobuf);
+    status = H5Oget_info(file_id, &infobuf, H5O_INFO_BASIC);
     if (!gwyhdf5_check_status(status, file_id, NULL, "H5Oget_info", error))
         return NULL;
 
-    gwy_clear(&efile, 1);
+    gwy_clear1(efile);
     if (!gwyhdf5_get_ints_attr(file_id, ".", "ARVersion", 1, &ver_dim, efile.version, NULL))
         g_warning("Cannot determine AR Version");
     else if (efile.version[0] != 1 && efile.version[0] != 2)
         g_warning("ARVersion = %d.%d.%d is not implemented", efile.version[0], efile.version[1], efile.version[2]);
 
-    gwyhdf5_init(&ghfile);
+    gwyhdf5_init(&ghfile, file_id, infobuf.token);
     ghfile.impl = &efile;
-    ghfile.attr_handler = ergo_attr_handler;
+    ghfile.attr_handler = attr_handler;
     gwyhdf5_alloc_lists(&ghfile, 1);
     if (efile.version[0] >= 2) {
         gwy_debug("assuming version >= 2");
@@ -156,7 +151,6 @@ ergo_load(const gchar *filename,
         ghfile.lists[0].idprefix = "/DataSet/Resolution ";
     }
     ghfile.lists[0].idwhat = H5O_TYPE_GROUP;
-    g_array_append_val(ghfile.addr, infobuf.addr);
 
     efile.filename = filename;
     efile.channels = g_array_new(FALSE, FALSE, sizeof(ErgoChannel));
@@ -183,7 +177,7 @@ ergo_load(const gchar *filename,
 
     if (gwyhdf5_get_int_attr(file_id, prefix, "NumFrames", &efile.nframes, error)) {
         gwy_debug("nframes %d", efile.nframes);
-        container = ergo_read_channels(file_id, &ghfile, error);
+        file = read_channels(file_id, &ghfile, error);
     }
 
     status = gwyhdf5_fclose(file_id);
@@ -192,24 +186,24 @@ ergo_load(const gchar *filename,
     for (i = 0; i < efile.channels->len; i++) {
         ErgoChannel *c = &g_array_index(efile.channels, ErgoChannel, i);
         g_free(c->name);
-        GWY_OBJECT_UNREF(c->xyunit);
-        GWY_OBJECT_UNREF(c->zunit);
+        g_clear_object(&c->xyunit);
+        g_clear_object(&c->zunit);
     }
     g_array_free(efile.channels, TRUE);
     gwyhdf5_free(&ghfile);
 
-    return container;
+    return file;
 }
 
-static GwyContainer*
-ergo_read_channels(hid_t file_id, GwyHDF5File *ghfile, GError **error)
+static GwyFile*
+read_channels(hid_t file_id, GwyHDF5File *ghfile, GError **error)
 {
     ErgoFile *efile = (ErgoFile*)ghfile->impl;
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     GArray *channels = efile->channels;
     GArray *resolutions = ghfile->lists[0].idlist;
     GString *str = ghfile->buf;
-    GwyDataField *dfield, *mask;
+    GwyField *dfield, *mask;
     GError *err = NULL;
     gint expected2[2] = { 2, 2 }, yxres[2];
     gchar *s, *s2[2];
@@ -232,7 +226,7 @@ ergo_read_channels(hid_t file_id, GwyHDF5File *ghfile, GError **error)
             if (!gwyhdf5_get_str_attr(file_id, str->str, "DataUnits", &s, error))
                 goto fail;
             gwy_debug("zunit of %s is %s", c->name, s);
-            c->zunit = gwy_si_unit_new_parse(s, &c->zpower10);
+            c->zunit = gwy_unit_new_parse(s, &c->zpower10);
             H5free_memory(s);
 
             if (!gwyhdf5_get_strs_attr(file_id, str->str, "DimUnits", 1, expected2, s2, error))
@@ -240,7 +234,7 @@ ergo_read_channels(hid_t file_id, GwyHDF5File *ghfile, GError **error)
             gwy_debug("xyunits of %s are %s and %s", c->name, s2[0], s2[1]);
             if (!gwy_strequal(s2[0], s2[1]))
                 g_warning("X and Y units differ, using X");
-            c->xyunit = gwy_si_unit_new_parse(s2[1], &c->xypower10);
+            c->xyunit = gwy_unit_new_parse(s2[1], &c->xypower10);
             H5free_memory(s2[0]);
             H5free_memory(s2[1]);
 
@@ -256,46 +250,45 @@ ergo_read_channels(hid_t file_id, GwyHDF5File *ghfile, GError **error)
             gwy_debug("resid %u res %dx%d", r, yxres[1], yxres[0]);
 
             for (frameid = 0; frameid < efile->nframes; frameid++) {
-                if (!(dfield = ergo_read_field(file_id, efile->version[0], r, c, frameid, yxres, str, &err))) {
+                if (!(dfield = read_field(file_id, efile->version[0], r, c, frameid, yxres, str, &err))) {
                     /* Sometimes this happens? Perhaps interrupted measurement? */
                     g_warning("Cannot read %s: %s", str->str, err->message);
                     g_clear_error(&err);
                     continue;
                 }
 
-                if (!container)
-                    container = gwy_container_new();
+                if (!file)
+                    file = gwy_file_new_in_construction();
 
                 mask = gwy_app_channel_mask_of_nans(dfield, TRUE);
-                gwy_container_pass_object(container, gwy_app_get_data_key_for_id(id), dfield);
-                gwy_file_channel_import_log_add(container, id, NULL, efile->filename);
+                gwy_file_pass_image(file, id, dfield);
+                gwy_log_add_import(file, GWY_FILE_IMAGE, id, NULL, efile->filename);
                 if (mask)
-                    gwy_container_pass_object(container, gwy_app_get_mask_key_for_id(id), mask);
-                gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(id), c->name);
-                gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(id),
-                                          gwy_container_duplicate(ghfile->meta));
+                    gwy_file_pass_image_mask(file, id, mask);
+                gwy_file_set_title(file, GWY_FILE_IMAGE, id, c->name, FALSE);
+                gwy_file_pass_meta(file, GWY_FILE_IMAGE, id, gwy_container_copy(ghfile->meta));
                 id++;
             }
         }
     }
 
-    if (container)
-        return container;
+    if (file)
+        return file;
 
     err_NO_DATA(error);
 
 fail:
-    GWY_OBJECT_UNREF(container);
+    g_clear_object(&file);
     return NULL;
 }
 
-static GwyDataField*
-ergo_read_field(hid_t file_id,
-                gint version_major,
-                guint r, ErgoChannel *c, gint frameid, const gint *yxres,
-                GString *str, GError **error)
+static GwyField*
+read_field(hid_t file_id,
+           gint version_major,
+           guint r, ErgoChannel *c, gint frameid, const gint *yxres,
+           GString *str, GError **error)
 {
-    GwyDataField *dfield;
+    GwyField *dfield;
     hid_t dataset;
     gdouble q, xreal, yreal, xoff, yoff;
     herr_t status;
@@ -310,7 +303,7 @@ ergo_read_field(hid_t file_id,
     if ((dataset = gwyhdf5_open_and_check_dataset(file_id, str->str, 2, (gint*)yxres, error)) < 0)
         return NULL;
 
-    q = pow10(c->xypower10);
+    q = gwy_exp10(c->xypower10);
 
     /* NB: In all dimensions y is first, then x. */
     xreal = c->realcoords[3] - c->realcoords[2];
@@ -321,21 +314,21 @@ ergo_read_field(hid_t file_id,
     sanitise_real_size(&yreal, "y size");
     yoff = MIN(c->realcoords[0], c->realcoords[1]);
 
-    dfield = gwy_data_field_new(yxres[1], yxres[0], q*xreal, q*yreal, FALSE);
-    gwy_data_field_set_xoffset(dfield, q*xoff);
-    gwy_data_field_set_yoffset(dfield, q*yoff);
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(dfield), c->xyunit);
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_z(dfield), c->zunit);
+    dfield = gwy_field_new(yxres[1], yxres[0], q*xreal, q*yreal, FALSE);
+    gwy_field_set_xoffset(dfield, q*xoff);
+    gwy_field_set_yoffset(dfield, q*yoff);
+    gwy_unit_assign(gwy_field_get_unit_xy(dfield), c->xyunit);
+    gwy_unit_assign(gwy_field_get_unit_z(dfield), c->zunit);
 
-    status = H5Dread(dataset, H5T_NATIVE_DOUBLE, H5S_ALL, H5S_ALL, H5P_DEFAULT, gwy_data_field_get_data(dfield));
+    status = H5Dread(dataset, H5T_NATIVE_DOUBLE, H5S_ALL, H5S_ALL, H5P_DEFAULT, gwy_field_get_data(dfield));
     H5Dclose(dataset);
-    gwy_data_field_invert(dfield, TRUE, FALSE, FALSE);
+    gwy_field_flip(dfield, FALSE, TRUE);
     if (c->zpower10)
-        gwy_data_field_multiply(dfield, pow10(c->zpower10));
+        gwy_field_multiply(dfield, gwy_exp10(c->zpower10));
 
     if (status < 0) {
         err_HDF5(error, "H5Dread", status);
-        GWY_OBJECT_UNREF(dfield);
+        g_clear_object(&dfield);
     }
     return dfield;
 }
@@ -346,7 +339,7 @@ append_channel_name(GArray *channels, const gchar *name)
     ErgoChannel c;
 
     gwy_debug("found channel %s", name);
-    gwy_clear(&c, 1);
+    gwy_clear1(c);
     c.name = g_strdup(name);
     g_strstrip(c.name);
     g_array_append_val(channels, c);
@@ -354,7 +347,7 @@ append_channel_name(GArray *channels, const gchar *name)
 
 /* XXX: Handle /DataSetInfo/ChannelNames which do not have unique paths and we just build them during the scan */
 static void
-ergo_attr_handler(GwyHDF5File *ghfile, hid_t loc_id, const char *attr_name)
+attr_handler(GwyHDF5File *ghfile, hid_t loc_id, const char *attr_name)
 {
     ErgoFile *efile = (ErgoFile*)ghfile->impl;
     H5T_class_t type_class;
