@@ -1,5 +1,5 @@
 /*
- *  $Id: fusionscope.c 28533 2025-09-10 14:07:50Z yeti-dn $
+ *  $Id: fusionscope.c 28911 2025-11-24 18:27:42Z yeti-dn $
  *  Copyright (C) 2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -28,11 +28,9 @@
  **/
 
 #include "config.h"
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/datafield.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils-file.h>
-#include <jansson.h>
+#include <glib/gi18n-lib.h>
+#include <gwy.h>
+#include <json-glib/json-glib.h>
 
 #include "gwyhdf5.h"
 #include "hdf5file.h"
@@ -67,8 +65,8 @@
  */
 
 typedef struct {
-    GwyDataField *field;
-    GwyDataField *mask;
+    GwyField *field;
+    GwyField *mask;
     GwyContainer *meta;
     gchar *title;
     gchar *id_prefix;
@@ -88,6 +86,7 @@ typedef struct {
     const gchar *frames;
     gchar *region_clean;
     gchar *frame_clean;
+    JsonParser *parser;
     GString *path;
     GString *buf;
     GHashTable *region_meta;
@@ -96,54 +95,50 @@ typedef struct {
     GArray *images;
 } FScopeFile;
 
-static gint          detect_file        (const GwyFileDetectInfo *fileinfo,
-                                         gboolean only_name);
-static GwyContainer* import_file        (const gchar *filename,
-                                         GwyRunType mode,
-                                         GError **error);
-static herr_t        region_scan        (hid_t loc_id,
-                                         const char *name,
-                                         const H5L_info_t *info,
-                                         void *user_data);
-static herr_t        frames_scan        (hid_t loc_id,
-                                         const char *name,
-                                         const H5L_info_t *info,
-                                         void *user_data);
-static herr_t        data_scan          (hid_t loc_id,
-                                         const char *name,
-                                         const H5L_info_t *info,
-                                         void *user_data);
-static herr_t        create_image       (FScopeFile *fsfile,
-                                         hid_t dataset,
-                                         const gint *dims);
-static gint          compare_images     (gconstpointer pa,
-                                         gconstpointer pb);
-static void          set_real_dimensions(GwyDataField *field,
-                                         FScopeFile *fsfile);
-static void          create_image_meta  (FScopeFile *fsfile,
-                                         FScopeImage *image);
-static gboolean      fill_hash_from_json(FScopeFile *fsfile,
-                                         hid_t loc_id,
-                                         const gchar *name,
-                                         GHashTable *hash);
-static void          fscope_file_init   (FScopeFile *fsfile,
-                                         const gchar *filename);
-static void          fscope_file_free   (FScopeFile *fsfile);
+static gint     detect_file        (const GwyFileDetectInfo *fileinfo,
+                                    gboolean only_name);
+static GwyFile* load_file          (const gchar *filename,
+                                    GwyRunModeFlags mode,
+                                    GError **error);
+static herr_t   region_scan        (hid_t loc_id,
+                                    const char *name,
+                                    const H5L_info_t *info,
+                                    void *user_data);
+static herr_t   frames_scan        (hid_t loc_id,
+                                    const char *name,
+                                    const H5L_info_t *info,
+                                    void *user_data);
+static herr_t   data_scan          (hid_t loc_id,
+                                    const char *name,
+                                    const H5L_info_t *info,
+                                    void *user_data);
+static herr_t   create_image       (FScopeFile *fsfile,
+                                    hid_t dataset,
+                                    const gint *dims);
+static gint     compare_images     (gconstpointer pa,
+                                    gconstpointer pb);
+static void     set_real_dimensions(GwyField *field,
+                                    FScopeFile *fsfile);
+static void     create_image_meta  (FScopeFile *fsfile,
+                                    FScopeImage *image);
+static gboolean fill_hash_from_json(FScopeFile *fsfile,
+                                    hid_t loc_id,
+                                    const gchar *name,
+                                    GHashTable *hash);
+static void     fscope_file_init   (FScopeFile *fsfile,
+                                    const gchar *filename);
+static void     fscope_file_free   (FScopeFile *fsfile);
 
 void
 gwyhdf5_register_fusionscope(void)
 {
     gwy_file_func_register("fusionscope",
                            N_("FusionScope HDF5 files (.fsexp)"),
-                           (GwyFileDetectFunc)&detect_file,
-                           (GwyFileLoadFunc)&import_file,
-                           NULL,
-                           NULL);
+                           &detect_file, &load_file, NULL, NULL);
 }
 
 static gint
-detect_file(const GwyFileDetectInfo *fileinfo,
-            gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     static const gchar *attributes[] = {
         "APPLICATION_VERSION", "EXPERIMENT_FILE_VERSION", "EXPERIMENT_INFO",
@@ -185,14 +180,14 @@ detect_file(const GwyFileDetectInfo *fileinfo,
     return score;
 }
 
-static GwyContainer*
-import_file(const gchar *filename,
-            G_GNUC_UNUSED GwyRunType mode,
-            GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
     FScopeFile fsfile;
     hid_t file_id, pipeline_data_group;
-    GwyContainer *data = NULL;
+    GwyFile *file = NULL;
     herr_t status;
     H5O_info_t infobuf;
     guint i;
@@ -202,7 +197,7 @@ import_file(const gchar *filename,
         return NULL;
     }
     gwy_debug("file_id %d", (gint)file_id);
-    status = H5Oget_info(file_id, &infobuf);
+    status = H5Oget_info(file_id, &infobuf, H5O_INFO_BASIC);
     if (!gwyhdf5_check_status(status, file_id, NULL, "H5Oget_info", error))
         return NULL;
 
@@ -212,6 +207,7 @@ import_file(const gchar *filename,
         goto end;
     }
 
+    fsfile.parser = json_parser_new_immutable();
     g_string_assign(fsfile.path, "experiement_info");
     fill_hash_from_json(&fsfile, file_id, "EXPERIMENT_INFO", fsfile.experiment_meta);
 
@@ -224,24 +220,27 @@ import_file(const gchar *filename,
         goto end;
     }
 
-    data = gwy_container_new();
+    file = gwy_file_new_in_construction();
     g_array_sort(fsfile.images, compare_images);
     for (i = 0; i < fsfile.images->len; i++) {
         FScopeImage *image = &g_array_index(fsfile.images, FScopeImage, i);
 
-        gwy_container_set_object(data, gwy_app_get_data_key_for_id(i), image->field);
+        gwy_file_set_image(file, i, image->field);
         if (image->mask)
-            gwy_container_set_object(data, gwy_app_get_mask_key_for_id(i), image->mask);
-        if (image->meta)
-            gwy_container_set_object(data, gwy_app_get_data_meta_key_for_id(i), image->meta);
-        gwy_container_set_string(data, gwy_app_get_data_title_key_for_id(i), image->title);
+            gwy_file_set_image_mask(file, i, image->mask);
+        if (image->meta) {
+            gwy_file_pass_meta(file, GWY_FILE_IMAGE, i, image->meta);
+            image->meta = NULL;
+        }
+        gwy_file_pass_title(file, GWY_FILE_IMAGE, i, image->title);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, i, NULL, filename);
         image->title = NULL;
     }
 
 end:
     fscope_file_free(&fsfile);
 
-    return data;
+    return file;
 }
 
 static herr_t
@@ -259,7 +258,7 @@ region_scan(hid_t loc_id,
 
     gwy_debug("  %s", name);
 
-    status = H5Oget_info_by_name(loc_id, name, &infobuf, H5P_DEFAULT);
+    status = H5Oget_info_by_name(loc_id, name, &infobuf, H5O_INFO_BASIC, H5P_DEFAULT);
     if (status < 0)
         return status;
 
@@ -315,7 +314,7 @@ frames_scan(hid_t loc_id,
 
     gwy_debug("    %s", name);
 
-    status = H5Oget_info_by_name(loc_id, name, &infobuf, H5P_DEFAULT);
+    status = H5Oget_info_by_name(loc_id, name, &infobuf, H5O_INFO_BASIC, H5P_DEFAULT);
     if (status < 0)
         return status;
 
@@ -380,7 +379,7 @@ data_scan(hid_t loc_id,
 
     gwy_debug("      %s", name);
 
-    status = H5Oget_info_by_name(loc_id, name, &infobuf, H5P_DEFAULT);
+    status = H5Oget_info_by_name(loc_id, name, &infobuf, H5O_INFO_BASIC, H5P_DEFAULT);
     if (status < 0)
         return status;
 
@@ -415,35 +414,35 @@ create_image(FScopeFile *fsfile, hid_t dataset, const gint *dims)
     FScopeImage image;
     gchar *channel_name = NULL;
     const gchar *chname = "Unknown";
-    GwyDataField *field;
-    GwySIUnit *unit;
+    GwyField *field;
+    GwyUnit *unit;
     herr_t status;
     const gchar *sval;
     gchar *lc_datatype;
     gint power10 = 0;
 
     gwy_clear(&image, 1);
-    field = gwy_data_field_new(dims[0], dims[1], 1.0, 1.0, FALSE);
-    status = H5Dread(dataset, H5T_NATIVE_DOUBLE, H5S_ALL, H5S_ALL, H5P_DEFAULT, gwy_data_field_get_data(field));
+    field = gwy_field_new(dims[0], dims[1], 1.0, 1.0, FALSE);
+    status = H5Dread(dataset, H5T_NATIVE_DOUBLE, H5S_ALL, H5S_ALL, H5P_DEFAULT, gwy_field_get_data(field));
     if (status < 0) {
         g_object_unref(field);
         return status;
     }
 
-    unit = gwy_data_field_get_si_unit_z(field);
+    unit = gwy_field_get_unit_z(field);
     if (fsfile->data_id >= 0) {
         /* NB: Always use raw data units, even for processed. They do generally not store units for processed data.
          * Sometimes they are there but… */
         g_string_printf(fsfile->buf, "frame_info::meta_data_raw_%u::datachannel_units", fsfile->data_id);
         if ((sval = g_hash_table_lookup(fsfile->frame_meta, fsfile->buf->str))) {
-            gwy_si_unit_set_from_string_parse(unit, sval, &power10);
+            power10 = gwy_unit_set_from_string(unit, sval);
             if (power10)
-                gwy_data_field_multiply(field, pow10(power10));
+                gwy_field_multiply(field, gwy_exp10(power10));
         }
     }
 
     /* There is regionUnits in REGION_INFO but it is always empty. */
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(field), "m");
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(field), "m");
     set_real_dimensions(field, fsfile);
 
     image.field = field;
@@ -471,7 +470,7 @@ create_image(FScopeFile *fsfile, hid_t dataset, const gint *dims)
 }
 
 static void
-set_real_dimensions(GwyDataField *field, FScopeFile *fsfile)
+set_real_dimensions(GwyField *field, FScopeFile *fsfile)
 {
     const gchar *sval;
     gdouble xstart = 0.0, xend = 1.0, ystart = 0.0, yend = 1.0;
@@ -515,10 +514,10 @@ set_real_dimensions(GwyDataField *field, FScopeFile *fsfile)
         xend = xreal;
         yend = yreal;
     }
-    gwy_data_field_set_xreal(field, (xend - xstart)*1e-6);
-    gwy_data_field_set_yreal(field, (yend - ystart)*1e-6);
-    gwy_data_field_set_xoffset(field, xstart*1e-6);
-    gwy_data_field_set_yoffset(field, ystart*1e-6);
+    gwy_field_set_xreal(field, (xend - xstart)*1e-6);
+    gwy_field_set_yreal(field, (yend - ystart)*1e-6);
+    gwy_field_set_xoffset(field, xstart*1e-6);
+    gwy_field_set_yoffset(field, ystart*1e-6);
 }
 
 static void
@@ -567,7 +566,7 @@ add_meta_with_data_id(gpointer hkey, gpointer hdata, gpointer user_data)
 static void
 create_image_meta(FScopeFile *fsfile, FScopeImage *image)
 {
-    image->meta = gwy_container_new();
+    image->meta = gwy_container_new_in_construction();
 
     /* TODO: There are also a couple top level attributes and attributes scattered in the HDF5 groups. Most do not
      * seem very useful, but some may be. */
@@ -617,6 +616,7 @@ fscope_file_free(FScopeFile *fsfile)
 {
     guint i;
 
+    g_clear_object(&fsfile->parser);
     g_string_free(fsfile->buf, TRUE);
     g_string_free(fsfile->path, TRUE);
     g_hash_table_destroy(fsfile->frame_meta);
@@ -636,18 +636,18 @@ fscope_file_free(FScopeFile *fsfile)
 
 /* Expand JSON metadata into individual fields. Silently ignore any errors. */
 static gboolean
-format_atomic_json_meta(json_t *item, GString *buf)
+format_atomic_json_meta(JsonNode *node, GString *str)
 {
-    if (json_is_string(item))
-        g_string_append(buf, json_string_value(item));
-    else if (json_is_true(item))
-        g_string_append(buf, "True");
-    else if (json_is_false(item))
-        g_string_append(buf, "False");
-    else if (json_is_real(item))
-        g_string_append_printf(buf, "%.9g", json_real_value(item));
-    else if (json_is_integer(item))
-        g_string_append_printf(buf, "%ld", (glong)json_integer_value(item));
+    GType type = json_node_get_value_type(node);
+
+    if (type == G_TYPE_STRING)
+        g_string_append(str, json_node_get_string(node));
+    else if (type == G_TYPE_INT64)
+        g_string_append_printf(str, "%" G_GINT64_FORMAT, json_node_get_int(node));
+    else if (type == G_TYPE_DOUBLE)
+        g_string_append_printf(str, "%.9g", json_node_get_double(node));
+    else if (type == G_TYPE_BOOLEAN)
+        g_string_assign(str, json_node_get_boolean(node) ? "True" : "False");
     else
         return FALSE;
 
@@ -655,52 +655,54 @@ format_atomic_json_meta(json_t *item, GString *buf)
 }
 
 static void
-expand_json_object(json_t *node, GHashTable *hash, GString *path, GString *buf)
+expand_json_object(JsonObject *object, GHashTable *hash, GString *path, GString *buf)
 {
-    const gchar *name;
-    json_t *member, *item;
-    guint i, pfxlen;
-
     g_string_append(path, "::");
-    pfxlen = path->len;
-    json_object_foreach(node, name, member) {
+    guint pfxlen = path->len;
+
+    JsonObjectIter iter;
+    json_object_iter_init(&iter, object);
+
+    const gchar *name;
+    JsonNode *node;
+    while (json_object_iter_next(&iter, &name, &node)) {
         g_string_append(path, name);
         g_string_truncate(buf, 0);
-        if (format_atomic_json_meta(member, buf)) {
-            gwy_debug("%s = %s", path->str, buf->str);
+        if (format_atomic_json_meta(node, buf)) {
             g_hash_table_replace(hash, g_strdup(path->str), g_strdup(buf->str));
         }
-        else if (json_is_array(member)) {
-            json_array_foreach(member, i, item) {
-                if (format_atomic_json_meta(item, buf))
+        else if (JSON_NODE_HOLDS_ARRAY(node)) {
+            JsonArray *array = json_node_get_array(node);
+            guint n = json_array_get_length(array);
+            for (guint i = 0; i < n; i++) {
+                JsonNode *item_node = json_array_get_element(array, i);
+                if (format_atomic_json_meta(item_node, buf))
                     g_string_append(buf, "; ");
             }
             if (buf->len >= 2)
                 g_string_truncate(buf, buf->len-2);
-            gwy_debug("%s = %s", path->str, buf->str);
             g_hash_table_replace(hash, g_strdup(path->str), g_strdup(buf->str));
         }
-        else if (json_is_object(member))
-            expand_json_object(member, hash, path, buf);
+        else if (JSON_NODE_HOLDS_OBJECT(node))
+            expand_json_object(json_node_get_object(node), hash, path, buf);
         else {
-            gwy_debug("unhandled meta %s of type %d", name, json_typeof(member));
+            gwy_debug("unhandled meta %s of type %d", name, JSON_NODE_TYPE(member));
         }
         g_string_truncate(path, pfxlen);
     }
 }
 
 static void
-expand_json_meta(GHashTable *hash, const gchar *str, GString *path, GString *buf)
+expand_json_meta(GHashTable *hash, JsonParser *parser, const gchar *str, GString *path, GString *buf)
 {
-    json_error_t jerror;
-    json_t *root;
-
-    if (!(root = json_loads(str, 0, &jerror)))
+    if (!json_parser_load_from_data(parser, str, -1, NULL))
         return;
-    if (json_is_object(root))
-        expand_json_object(root, hash, path, buf);
 
-    json_decref(root);
+    JsonNode *root_node = json_parser_get_root(parser);
+    if (!JSON_NODE_HOLDS_OBJECT(root_node))
+        return;
+
+    expand_json_object(json_node_get_object(root_node), hash, path, buf);
 }
 
 /* There are quoted JSON strings inside the JSON! Yay! */
@@ -726,7 +728,7 @@ fill_hash_from_json(FScopeFile *fsfile, hid_t loc_id, const gchar *name, GHashTa
     if (!gwyhdf5_get_str_attr(loc_id, ".", name, &strattr, NULL))
         return FALSE;
 
-    expand_json_meta(hash, strattr, fsfile->path, fsfile->buf);
+    expand_json_meta(hash, fsfile->parser, strattr, fsfile->path, fsfile->buf);
     H5free_memory(strattr);
 
     embedded_jsons = g_ptr_array_new();
@@ -740,7 +742,7 @@ fill_hash_from_json(FScopeFile *fsfile, hid_t loc_id, const gchar *name, GHashTa
         sep = g_strrstr_len(fsfile->path->str, fsfile->path->len, "::");
         if (sep)
             g_string_truncate(fsfile->path, sep - fsfile->path->str);
-        expand_json_meta(hash, jsonstr, fsfile->path, fsfile->buf);
+        expand_json_meta(hash, fsfile->parser, jsonstr, fsfile->path, fsfile->buf);
         g_hash_table_remove(hash, key);
     }
     g_ptr_array_free(embedded_jsons, TRUE);
