@@ -1,6 +1,6 @@
 /*
- *  $Id: gwyhdf5.c 28998 2025-12-15 13:20:20Z yeti-dn $
- *  Copyright (C) 2020-2024 David Necas (Yeti), Petr Klapetek.
+ *  $Id: gwyhdf5.c 29026 2025-12-19 14:03:38Z yeti-dn $
+ *  Copyright (C) 2020-2025 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -21,7 +21,8 @@
  **/
 
 #include "config.h"
-#include <libgwyddion/gwyutils.h>
+#include <glib/gi18n-lib.h>
+#include <gwy.h>
 #include "gwyhdf5.h"
 
 /*******************************************************************************************************************
@@ -52,15 +53,17 @@ gwyhdf5_quick_check(const GwyFileDetectInfo *fileinfo,
 }
 
 void
-gwyhdf5_init(GwyHDF5File *ghfile)
+gwyhdf5_init(GwyHDF5File *ghfile, hid_t file_id, H5O_token_t token)
 {
     gwy_clear(ghfile, 1);
+    ghfile->file_id = file_id;
     ghfile->meta = gwy_container_new();
     ghfile->path = g_string_new(NULL);
     ghfile->buf = g_string_new(NULL);
-    ghfile->addr = g_array_new(FALSE, FALSE, sizeof(haddr_t));
+    ghfile->tokens = g_array_new(FALSE, FALSE, sizeof(H5O_token_t));
     ghfile->datasets = g_array_new(FALSE, FALSE, sizeof(gchar*));
     ghfile->datascales = g_array_new(FALSE, FALSE, sizeof(gchar*));
+    g_array_append_val(ghfile->tokens, token);
 }
 
 void
@@ -88,7 +91,7 @@ gwyhdf5_free(GwyHDF5File *ghfile)
     for (i = 0; i < ghfile->datascales->len; i++)
         g_free(g_array_index(ghfile->datascales, gchar*, i));
     g_array_free(ghfile->datascales, TRUE);
-    g_array_free(ghfile->addr, TRUE);
+    g_array_free(ghfile->tokens, TRUE);
     if (ghfile->nlists) {
         for (i = 0; i < ghfile->nlists; i++)
             g_array_free(ghfile->lists[i].idlist, TRUE);
@@ -96,7 +99,7 @@ gwyhdf5_free(GwyHDF5File *ghfile)
     }
     g_string_free(ghfile->path, TRUE);
     g_string_free(ghfile->buf, TRUE);
-    GWY_OBJECT_UNREF(ghfile->meta);
+    g_clear_object(&ghfile->meta);
 }
 
 /* NB: loc_id is ‘parent’ location and name is particlar item within it. */
@@ -109,23 +112,24 @@ gwyhdf5_scan_file(hid_t loc_id,
     GwyHDF5File *ghfile = (GwyHDF5File*)user_data;
     herr_t status, return_val = 0;
     H5O_info_t infobuf;
-    GArray *addr = ghfile->addr;
+    GArray *tokens = ghfile->tokens;
     GString *path = ghfile->path;
     guint i, len = path->len;
-    hid_t dataset;
+    hid_t dataset, file_id = ghfile->file_id;
     gchar *s;
 
-    status = H5Oget_info_by_name(loc_id, name, &infobuf, H5P_DEFAULT);
+    status = H5Oget_info_by_name(loc_id, name, &infobuf, H5O_INFO_BASIC | H5O_INFO_NUM_ATTRS, H5P_DEFAULT);
     if (status < 0)
         return status;
 
     /* Detect loops. */
-    for (i = 0; i < addr->len; i++) {
-        if (g_array_index(addr, haddr_t, i) == infobuf.addr)
+    for (i = 0; i < tokens->len; i++) {
+        int cmpval;
+        if (H5Otoken_cmp(file_id, &infobuf.token, &g_array_index(tokens, H5O_token_t, i), &cmpval) < 0 || !cmpval)
             return -1;
     }
 
-    g_array_append_val(addr, infobuf.addr);
+    g_array_append_val(tokens, infobuf.token);
     g_string_append_c(path, '/');
     g_string_append(path, name);
     gwy_debug("path %s", path->str);
@@ -165,7 +169,7 @@ gwyhdf5_scan_file(hid_t loc_id,
     }
 
     g_string_truncate(path, len);
-    g_array_set_size(addr, addr->len-1);
+    g_array_set_size(tokens, tokens->len-1);
 
     return return_val;
 }
@@ -345,8 +349,6 @@ process_fixed_string_attribute(hid_t attr, hid_t attr_type, gint nitems,
     return status;
 }
 
-#if 0
-/* XXX: This all requires HDF5 1.12. Currently that's way too new for our conservative taste. */
 static herr_t
 process_reference_attribute(hid_t attr, gint nitems,
                             GString *outbuf)
@@ -381,67 +383,6 @@ process_reference_attribute(hid_t attr, gint nitems,
 
     return status;
 }
-#endif
-
-static herr_t
-process_reference_attribute(hid_t attr, gint nitems,
-                            GString *outbuf)
-{
-    hobj_ref_t objref;
-    hdset_reg_ref_t dsrref;
-    herr_t status;
-    ssize_t size = -1;
-    H5R_type_t reftype = H5R_BADTYPE;
-    gpointer refptr = NULL;
-    gchar buf[4];
-
-    if (nitems <= 0)
-        return -1;
-
-    if (nitems > 1) {
-        gwy_debug("Cannot handle reference arrays yet.");
-        return -1;
-    }
-
-    gwy_debug("reference");
-    gwy_clear(&objref, 1);
-    if ((status = H5Aread(attr, H5T_STD_REF_OBJ, &objref)) >= 0) {
-        /* It seems we cannot actually pass NULL for buf in the old API because there is an assertion HDAssert(name).
-         * Pass a small buffer we are going to ignore. If the reference is not to an object, we can get still get
-         * a good status above, but then we get zero size here. */
-        size = H5Rget_name(attr, H5R_OBJECT, &objref, buf, sizeof(buf));
-        gwy_debug("object reference size %ld", (glong)size);
-        if (size < 0)
-            return -1;
-        if (size > 0) {
-            reftype = H5R_OBJECT;
-            refptr = &objref;
-        }
-    }
-    if (size == 0 && (status = H5Aread(attr, H5T_STD_REF_DSETREG, &dsrref)) >= 0) {
-        /* FIXME: Don't actually know what to do with a dataset region reference. */
-        size = H5Rget_name(attr, H5R_DATASET_REGION, &dsrref, buf, sizeof(buf));
-        gwy_debug("dataset region reference size %ld", (glong)size);
-        if (size < 0)
-            return -1;
-        if (size > 0) {
-            reftype = H5R_DATASET_REGION;
-            refptr = &dsrref;
-        }
-    }
-
-    if (refptr && reftype != H5R_BADTYPE) {
-        g_assert(size > 0);
-        g_string_set_size(outbuf, size);
-        /* Not passing size+1 makes it miss the last character. We have the +1 space so pass it. But make sure
-         * the string is terminated afterwards. */
-        H5Rget_name(attr, reftype, refptr, outbuf->str, size+1);
-        g_string_truncate(outbuf, size);
-        gwy_debug("reference target <%s>", outbuf->str);
-    }
-
-    return status;
-}
 
 static herr_t
 process_compound_attribute(hid_t space,
@@ -923,8 +864,6 @@ gwyhdf5_get_simple_space_dims(hid_t space, gint expected_ndims, gint *dims,
     return TRUE;
 }
 
-/* NB: Pass 0 as expected_ndims and NULL for dims for SCALAR datasets (scalars are not one-element arrays; they are
- * zero-dimension point-like objects in HDF5). */
 hid_t
 gwyhdf5_open_and_check_dataset(hid_t file_id, const gchar *name,
                                gint expected_ndims, gint *dims,
@@ -958,40 +897,39 @@ gwyhdf5_open_and_check_dataset(hid_t file_id, const gchar *name,
 gchar*
 gwyhdf5_read_string_dataset(hid_t loc_id, const gchar *name, GError **error)
 {
-    hid_t dataset, dataset_type, str_type;
-    H5T_class_t type_class;
-    gint status, len = -1;
-    size_t type_size;
+    hid_t dataset;
+    gint len = -1;
     gchar *s = NULL;
-    gboolean is_vlenstr;
 
     if ((dataset = gwyhdf5_open_and_check_dataset(loc_id, name, 1, &len, error)) < 0)
         return NULL;
 
-    dataset_type = H5Dget_type(dataset);
-    type_class = H5Tget_class(dataset_type);
+    hid_t dataset_type = H5Dget_type(dataset);
+    H5T_class_t type_class = H5Tget_class(dataset_type);
     gwy_debug("len = %d, class = %d", len, type_class);
     if (type_class != H5T_STRING)
         goto end;
 
     /* We only read string datasets which are simple strings formed by single characters. No
      * array-of-variable-length-strings nonsense. */
-    is_vlenstr = H5Tis_variable_str(dataset_type);
+    gboolean is_vlenstr = H5Tis_variable_str(dataset_type);
     if (is_vlenstr) {
         err_UNSUPPORTED(error, "is_variable_str");
         goto end;
     }
-    type_size = H5Tget_size(dataset_type);
+    gsize type_size = H5Tget_size(dataset_type);
     if (type_size != 1) {
         err_UNSUPPORTED(error, "str_size");
         goto end;
     }
 
-    str_type = H5Tcopy(H5T_C_S1);
+    hid_t str_type = H5Tcopy(H5T_C_S1);
     H5Tset_size(str_type, 1);
     H5Tset_strpad(str_type, H5T_STR_NULLTERM);
     H5Tset_cset(str_type, H5Tget_cset(dataset_type));
     s = g_new0(gchar, len+1);
+
+    gint status;
     if ((status = H5Dread(dataset, str_type, H5S_ALL, H5S_ALL, H5P_DEFAULT, s)) < 0) {
         err_HDF5(error, "H5Dread", status);
         GWY_FREE(s);
@@ -1048,10 +986,10 @@ gwyhdf5_check_status(herr_t status, hid_t file_id, GwyHDF5File *ghfile,
 }
 
 static void
-transform_key(gpointer hkey, gpointer hvalue, gpointer user_data)
+transform_key(GQuark quark, GValue *gvalue, gpointer user_data)
 {
-    const gchar *oldkey = g_quark_to_string(GPOINTER_TO_UINT(hkey));
-    const gchar *value = g_value_get_string((GValue*)hvalue);
+    const gchar *oldkey = g_quark_to_string(quark);
+    const gchar *value = g_value_get_string(gvalue);
     GwyContainer *newmeta = (GwyContainer*)user_data;
     gchar *newkey = gwy_strreplace(oldkey, "/", "::", (gsize)-1);
 
