@@ -1,6 +1,6 @@
 /*
- *  $Id: hdf5file.c 29007 2025-12-15 16:09:18Z yeti-dn $
- *  Copyright (C) 2020-2024 David Necas (Yeti), Petr Klapetek.
+ *  $Id: hdf5file.c 29026 2025-12-19 14:03:38Z yeti-dn $
+ *  Copyright (C) 2020-2025 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -42,11 +42,8 @@
  * Therefore, this file must be compiled with -DH5_USE_18_API
  */
 #include "config.h"
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/datafield.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils-file.h>
+#include <glib/gi18n-lib.h>
+#include <gwy.h>
 
 #include "gwyhdf5.h"
 #include "hdf5file.h"
@@ -64,22 +61,22 @@ static gboolean module_register(void);
 
 
 /* Matlab MAT 7 (only detection differs). */
-static gint          mat7x_detect             (const GwyFileDetectInfo *fileinfo,
-                                               gboolean only_name);
+static gint     mat7x_detect(const GwyFileDetectInfo *fileinfo,
+                             gboolean only_name);
 
 /* Generic */
-static gint          ghdf5_detect             (const GwyFileDetectInfo *fileinfo,
-                                               gboolean only_name);
-static GwyContainer* ghdf5_load               (const gchar *filename,
-                                               GwyRunType mode,
-                                               GError **error);
+static gint     ghdf5_detect(const GwyFileDetectInfo *fileinfo,
+                             gboolean only_name);
+static GwyFile* ghdf5_load  (const gchar *filename,
+                             GwyRunModeFlags mode,
+                             GError **error);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports files based on Hierarchical Data Format (HDF), version 5."),
     "Yeti <yeti@gwyddion.net>",
-    "2.9",
+    "3.2",
     "David Nečas (Yeti) & Petr Klapetek",
     "2020",
 };
@@ -100,23 +97,17 @@ module_register(void)
 
     gwy_file_func_register("hdf5generic",
                            N_("Generic HDF5 files (.h5)"),
-                           (GwyFileDetectFunc)&ghdf5_detect,
-                           (GwyFileLoadFunc)&ghdf5_load,
-                           NULL,
-                           NULL);
+                           &ghdf5_detect, &ghdf5_load, NULL, NULL);
     /* Read MAT 7.x files using the generic HDF5 reader. None of the Matlab specific stuff is meaningful for us
      * anyway. */
     gwy_file_func_register("mat7xfile",
                            N_("Matlab 7.x HDF5 MAT files (.mat)"),
-                           (GwyFileDetectFunc)&mat7x_detect,
-                           (GwyFileLoadFunc)&ghdf5_load,
-                           NULL,
-                           NULL);
+                           &mat7x_detect, &ghdf5_load, NULL, NULL);
 
     gwyhdf5_register_datxfile();
     gwyhdf5_register_epflfile();
     gwyhdf5_register_ergofile();
-#ifdef HAVE_JANSSON
+#ifdef HAVE_JSON_GLIB
     gwyhdf5_register_fusionscope();
 #endif
     gwyhdf5_register_gxsm4();
@@ -134,8 +125,7 @@ module_register(void)
  *******************************************************************************************************************/
 
 static gint
-mat7x_detect(const GwyFileDetectInfo *fileinfo,
-             gboolean only_name)
+mat7x_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     GwyFileDetectInfo subinfo;
     hid_t file_id;
@@ -166,8 +156,7 @@ mat7x_detect(const GwyFileDetectInfo *fileinfo,
  *******************************************************************************************************************/
 
 static gint
-ghdf5_detect(const GwyFileDetectInfo *fileinfo,
-             gboolean only_name)
+ghdf5_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     hid_t file_id;
 
@@ -180,12 +169,12 @@ ghdf5_detect(const GwyFileDetectInfo *fileinfo,
     return 50;
 }
 
-static GwyContainer*
+static GwyFile*
 ghdf5_load(const gchar *filename,
-           G_GNUC_UNUSED GwyRunType mode,
+           G_GNUC_UNUSED GwyRunModeFlags mode,
            GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     GwyHDF5File ghfile;
     hid_t file_id, dataset;
     herr_t status;
@@ -198,12 +187,11 @@ ghdf5_load(const gchar *filename,
         return NULL;
     }
     gwy_debug("file_id %d", (gint)file_id);
-    status = H5Oget_info(file_id, &infobuf);
+    status = H5Oget_info(file_id, &infobuf, H5O_INFO_BASIC);
     if (!gwyhdf5_check_status(status, file_id, NULL, "H5Oget_info", error))
         return NULL;
 
-    gwyhdf5_init(&ghfile);
-    g_array_append_val(ghfile.addr, infobuf.addr);
+    gwyhdf5_init(&ghfile, file_id, infobuf.token);
     status = H5Literate(file_id, H5_INDEX_NAME, H5_ITER_NATIVE, NULL, gwyhdf5_scan_file, &ghfile);
     if (!gwyhdf5_check_status(status, file_id, &ghfile, "H5Literate", error))
         return NULL;
@@ -214,7 +202,7 @@ ghdf5_load(const gchar *filename,
     for (i = 0; i < ghfile.datasets->len; i++) {
         gchar *name = g_array_index(ghfile.datasets, gchar*, i);
         gint dims[2] = { -1, -1 };
-        GwyDataField *field, *mask;
+        GwyField *field, *mask;
 
         if ((dataset = gwyhdf5_open_and_check_dataset(file_id, name, 2, dims, NULL)) < 0)
             continue;
@@ -222,8 +210,8 @@ ghdf5_load(const gchar *filename,
             H5Dclose(dataset);
             continue;
         }
-        field = gwy_data_field_new(dims[1], dims[0], dims[1], dims[0], FALSE);
-        status = H5Dread(dataset, H5T_NATIVE_DOUBLE, H5S_ALL, H5S_ALL, H5P_DEFAULT, gwy_data_field_get_data(field));
+        field = gwy_field_new(dims[1], dims[0], dims[1], dims[0], FALSE);
+        status = H5Dread(dataset, H5T_NATIVE_DOUBLE, H5S_ALL, H5S_ALL, H5P_DEFAULT, gwy_field_get_data(field));
         gwy_debug("status %d", status);
         H5Dclose(dataset);
         if (status < 0) {
@@ -231,14 +219,14 @@ ghdf5_load(const gchar *filename,
             continue;
         }
 
-        if (!container)
-            container = gwy_container_new();
+        if (!file)
+            file = gwy_file_new_in_construction();
 
         mask = gwy_app_channel_mask_of_nans(field, TRUE);
-        gwy_container_pass_object(container, gwy_app_get_data_key_for_id(id), field);
+        gwy_file_pass_image(file, id, field);
         if (mask)
-            gwy_container_pass_object(container, gwy_app_get_mask_key_for_id(id), mask);
-        gwy_file_channel_import_log_add(container, id, NULL, filename);
+            gwy_file_pass_image_mask(file, id, mask);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, id, NULL, filename);
         id++;
     }
 
@@ -247,10 +235,10 @@ ghdf5_load(const gchar *filename,
 
     gwyhdf5_free(&ghfile);
 
-    if (!container)
+    if (!file)
         err_NO_DATA(error);
 
-    return container;
+    return file;
 }
 
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
