@@ -1,6 +1,7 @@
+#pragma once
 /*
- *  $Id: hdf5file.h 28957 2025-12-08 10:12:54Z yeti-dn $
- *  Copyright (C) 2024 David Necas (Yeti), Petr Klapetek.
+ *  $Id: hdf5file.h 29026 2025-12-19 14:03:38Z yeti-dn $
+ *  Copyright (C) 2024-2025 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -18,17 +19,19 @@
 #ifndef __GWY_HDF5FILE__
 #define __GWY_HDF5FILE__
 
+#include "config.h"
+
 /* This is a header for the module (as opposed to the HDF5 utils). It should only contain the individual registration
  * functions. */
 G_GNUC_INTERNAL void gwyhdf5_register_datxfile(void);
 G_GNUC_INTERNAL void gwyhdf5_register_epflfile(void);
 G_GNUC_INTERNAL void gwyhdf5_register_ergofile(void);
-#ifdef HAVE_JANSSON
+#ifdef HAVE_JSON_GLIB
 G_GNUC_INTERNAL void gwyhdf5_register_fusionscope(void);
 #endif
-G_GNUC_INTERNAL void gwyhdf5_register_gxsm4(void);
-G_GNUC_INTERNAL void gwyhdf5_register_nhffile(void);
-G_GNUC_INTERNAL void gwyhdf5_register_nsidfile(void);
+G_GNUC_INTERNAL void gwyhdf5_register_gxsm4     (void);
+G_GNUC_INTERNAL void gwyhdf5_register_nhffile   (void);
+G_GNUC_INTERNAL void gwyhdf5_register_nsidfile  (void);
 G_GNUC_INTERNAL void gwyhdf5_register_shilpsfile(void);
 
 #endif
