@@ -1,6 +1,6 @@
 /*
- *  $Id: nhffile.c 29007 2025-12-15 16:09:18Z yeti-dn $
- *  Copyright (C) 2024 David Necas (Yeti), Petr Klapetek.
+ *  $Id: nhffile.c 29006 2025-12-15 16:08:13Z yeti-dn $
+ *  Copyright (C) 2024-2025 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -44,12 +44,10 @@
  */
 
 #include "config.h"
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/datafield.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils-file.h>
-#ifdef HAVE_JANSSON
-#include <jansson.h>
+#include <glib/gi18n-lib.h>
+#include <gwy.h>
+#ifdef HAVE_JSON_GLIB
+#include <json-glib/json-glib.h>
 #endif
 
 #include "gwyhdf5.h"
@@ -104,7 +102,7 @@ typedef struct {
 
 typedef struct {
     gdouble *data;
-    GwySIUnit *unit;
+    GwyUnit *unit;
     gchar *name;
 } NHFCurveData;
 
@@ -113,7 +111,7 @@ typedef struct {
     GwyHDF5File *ghfile;
     GHashTable *targets;
     GString *path;
-    GwyContainer *container;
+    GwyFile *file;
     hid_t file_id;
     /* These are not global; they reflect the current state as we traverse the file. */
     NHFGroupType group_type;
@@ -132,7 +130,7 @@ typedef struct {
     gchar *segment_name;
     gchar *freewave_name;
     gchar *signal_data_pattern;
-    GwyDataLine *freewave;
+    GwyLine *freewave;
     GwyXY *xy;
     guint *coord_index;
     gboolean have_x;
@@ -144,55 +142,55 @@ typedef struct {
     GError *error;
 } NHFFile;
 
-static gint          nhf_detect               (const GwyFileDetectInfo *fileinfo,
-                                               gboolean only_name);
-static GwyContainer* nhf_load                 (const gchar *filename,
-                                               GwyRunType mode,
-                                               GError **error);
-static herr_t        handle_group             (hid_t loc_id,
-                                               const char *name,
-                                               const H5L_info_t *info,
-                                               void *op_data);
-static void          nhf_attr_handler         (GwyHDF5File *ghfile,
-                                               hid_t loc_id,
-                                               const char *attr_name);
-static NHFStatus     enter_grouping           (hid_t loc_id,
-                                               const gchar *name,
-                                               H5O_type_t expected_type,
-                                               const gchar *format,
-                                               guint namelen,
-                                               guint *idx,
-                                               GString *path,
-                                               guint *pfxlen,
-                                               GError **error);
-static void          leave_grouping           (hid_t obj_id,
-                                               GString *path,
-                                               guint pfxlen);
-static gboolean      create_image_data        (NHFFile *nhfile,
-                                               hid_t dataset_id);
-static gboolean      create_volume_data       (NHFFile *nhfile,
-                                               hid_t dataset_id);
-static void          create_free_wave_graph   (NHFFile *nhfile);
-static void          create_curve_map         (NHFFile *nhfile);
-static void          create_cmap_1x1_graph    (NHFFile *nhfile);
-static gboolean      get_image_pixel_res      (hid_t loc_id,
-                                               NHFFile *nhfile);
-static gboolean      get_image_real_range     (hid_t loc_id,
-                                               NHFFile *nhfile);
-static gboolean      prepare_calibration      (hid_t dataset_id,
-                                               GwySIUnit *unit,
-                                               gdouble *q,
-                                               gdouble *v0);
-static gchar*        extract_unit_symbol      (gchar *signal_unit);
-static void          add_segment_name_to_title(NHFFile *nhfile,
-                                               GQuark quark);
-static gboolean      dataset_is_placeholder   (hid_t parent_id,
-                                               const gchar *dataset_name,
-                                               gint *size);
-static void          set_gerror_hdf_internal  (NHFFile *nhfile,
-                                               const gchar *funcname,
-                                               herr_t status);
-static void          clean_up_json_meta       (GwyContainer *meta);
+static gint      detect_file              (const GwyFileDetectInfo *fileinfo,
+                                           gboolean only_name);
+static GwyFile*  load_file                (const gchar *filename,
+                                           GwyRunModeFlags mode,
+                                           GError **error);
+static herr_t    handle_group             (hid_t loc_id,
+                                           const char *name,
+                                           const H5L_info_t *info,
+                                           void *op_data);
+static void      nhf_attr_handler         (GwyHDF5File *ghfile,
+                                           hid_t loc_id,
+                                           const char *attr_name);
+static NHFStatus enter_grouping           (hid_t loc_id,
+                                           const gchar *name,
+                                           H5O_type_t expected_type,
+                                           const gchar *format,
+                                           guint namelen,
+                                           guint *idx,
+                                           GString *path,
+                                           guint *pfxlen,
+                                           GError **error);
+static void      leave_grouping           (hid_t obj_id,
+                                           GString *path,
+                                           guint pfxlen);
+static gboolean  create_image_data        (NHFFile *nhfile,
+                                           hid_t dataset_id);
+static gboolean  create_volume_data       (NHFFile *nhfile,
+                                           hid_t dataset_id);
+static void      create_free_wave_graph   (NHFFile *nhfile);
+static void      create_curve_map         (NHFFile *nhfile);
+static void      create_cmap_1x1_graph    (NHFFile *nhfile);
+static gboolean  get_image_pixel_res      (hid_t loc_id,
+                                           NHFFile *nhfile);
+static gboolean  get_image_real_range     (hid_t loc_id,
+                                           NHFFile *nhfile);
+static gboolean  prepare_calibration      (hid_t dataset_id,
+                                           GwyUnit *unit,
+                                           gdouble *q,
+                                           gdouble *v0);
+static gchar*    extract_unit_symbol      (gchar *signal_unit);
+static void      add_segment_name_to_title(NHFFile *nhfile,
+                                           GQuark quark);
+static gboolean  dataset_is_placeholder   (hid_t parent_id,
+                                           const gchar *dataset_name,
+                                           gint *size);
+static void      set_gerror_hdf_internal  (NHFFile *nhfile,
+                                           const gchar *funcname,
+                                           herr_t status);
+static void      clean_up_json_meta       (GwyContainer *meta);
 
 static const GwyEnum group_names[] = {
     { "calibration",      NHF_GROUP_CALIBRATION,      },
@@ -206,15 +204,15 @@ gwyhdf5_register_nhffile(void)
 {
     gwy_file_func_register("nhffile",
                            N_("Nanosurf NHF HDF5 files (.nhf)"),
-                           (GwyFileDetectFunc)&nhf_detect,
-                           (GwyFileLoadFunc)&nhf_load,
+                           detect_file,
+                           load_file,
                            NULL,
                            NULL);
 }
 
 static gint
-nhf_detect(const GwyFileDetectInfo *fileinfo,
-           gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo,
+            gboolean only_name)
 {
     hid_t file_id;
     gchar *created = NULL, *modified = NULL;
@@ -244,10 +242,10 @@ nhf_detect(const GwyFileDetectInfo *fileinfo,
     return score;
 }
 
-static GwyContainer*
-nhf_load(const gchar *filename,
-         G_GNUC_UNUSED GwyRunType mode,
-         GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
     GwyHDF5File ghfile;
     NHFFile nhfile;
@@ -255,7 +253,7 @@ nhf_load(const gchar *filename,
     H5O_info_t infobuf;
     hid_t file_id;
 
-    gwy_clear(&nhfile, 1);
+    gwy_clear1(nhfile);
     nhfile.filename = filename;
     if ((file_id = H5Fopen(filename, H5F_ACC_RDONLY, H5P_DEFAULT)) < 0) {
         err_HDF5(error, "H5Fopen", file_id);
@@ -264,18 +262,16 @@ nhf_load(const gchar *filename,
     nhfile.file_id = file_id;
     gwy_debug("file_id %d", (gint)file_id);
 
-    status = H5Oget_info(file_id, &infobuf);
+    status = H5Oget_info(file_id, &infobuf, H5O_INFO_BASIC);
     if (!gwyhdf5_check_status(status, file_id, NULL, "H5Oget_info", error))
         return NULL;
 
-    nhfile.graph_id = 1;
     nhfile.path = g_string_new(NULL);
     nhfile.targets = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
 
-    gwyhdf5_init(&ghfile);
+    gwyhdf5_init(&ghfile, file_id, infobuf.token);
     ghfile.impl = &nhfile;
     ghfile.attr_handler = nhf_attr_handler;
-    g_array_append_val(ghfile.addr, infobuf.addr);
     nhfile.ghfile = &ghfile;
 
     /* First we need to find all UUID targets because UUIDs can link to things we have not seen yet in a sequential
@@ -287,7 +283,7 @@ nhf_load(const gchar *filename,
         goto end;
     }
 
-    nhfile.container = gwy_container_new();
+    nhfile.file = gwy_file_new_in_construction();
     nhfile.group = nhfile.subgroup = nhfile.dataset = -1;
     if ((status = H5Literate(file_id, H5_INDEX_NAME, H5_ITER_NATIVE, NULL, handle_group, &nhfile)) < 0) {
         set_gerror_hdf_internal(&nhfile, "H5Literate", status);
@@ -299,7 +295,7 @@ nhf_load(const gchar *filename,
 
 end:
     if (nhfile.error) {
-        GWY_OBJECT_UNREF(nhfile.container);
+        g_clear_object(&nhfile.file);
         g_propagate_error(error, nhfile.error);
         nhfile.error = NULL;
     }
@@ -312,7 +308,7 @@ end:
     g_string_free(nhfile.path, TRUE);
     g_hash_table_destroy(nhfile.targets);
 
-    return nhfile.container;
+    return nhfile.file;
 }
 
 /* Returns -1 iff it sets @nhfile->error.
@@ -378,9 +374,9 @@ handle_volume_dataset(hid_t loc_id,
     NHFStatus status;
     guint pfxlen;
     gint xres = nhfile->xres, yres = nhfile->yres, size, ndata = xres * yres;
-    GwyDataLine *freewave = nhfile->freewave;
+    GwyLine *freewave = nhfile->freewave;
 
-    /* Just skip the data if there is no z freewave dataline. */
+    /* Just skip the data if there is no z freewave line. */
     if (!freewave)
         return NHF_NEUTRAL;
 
@@ -399,7 +395,7 @@ handle_volume_dataset(hid_t loc_id,
         return status;
 
     status = NHF_FAIL;
-    ndata *= gwy_data_line_get_res(freewave);
+    ndata *= gwy_line_get_res(freewave);
 
     if (ndata != size) {
         g_warning("Attribute dataset_size (%d) != xres*yres (%d). Ignoring and checking physical data size.",
@@ -426,7 +422,7 @@ handle_wnma_wave_dataset(hid_t loc_id,
 {
     NHFFile *nhfile = (NHFFile*)op_data;
     GString *path = nhfile->path;
-    GwyDataLine *freewave = nhfile->freewave;
+    GwyLine *freewave = nhfile->freewave;
     guint pfxlen;
     hid_t dataset_id = -1;
     gdouble q, v0;
@@ -453,19 +449,19 @@ handle_wnma_wave_dataset(hid_t loc_id,
     if ((dataset_id = gwyhdf5_open_and_check_dataset(loc_id, name, 1, &size, &nhfile->error)) < 0)
         goto end;
 
-    nhfile->freewave = freewave = gwy_data_line_new(size, size, TRUE);
-    d = gwy_data_line_get_data(freewave);
+    nhfile->freewave = freewave = gwy_line_new(size, size, TRUE);
+    d = gwy_line_get_data(freewave);
     if ((status = H5Dread(dataset_id, H5T_NATIVE_DOUBLE, H5S_ALL, H5S_ALL, H5P_DEFAULT, d)) < 0) {
         err_HDF5(&nhfile->error, "H5Dread", status);
-        GWY_OBJECT_UNREF(nhfile->freewave);
+        g_clear_object(&nhfile->freewave);
         goto end;
     }
 
     gwyhdf5_get_str_attr(dataset_id, ".", "signal_name", &nhfile->freewave_name, NULL);
 
-    prepare_calibration(dataset_id, gwy_data_line_get_si_unit_y(freewave), &q, &v0);
-    gwy_data_line_multiply(freewave, q);
-    gwy_data_line_add(freewave, v0);
+    prepare_calibration(dataset_id, gwy_line_get_unit_y(freewave), &q, &v0);
+    gwy_line_multiply(freewave, q);
+    gwy_line_add(freewave, v0);
 
     gwy_debug("found volume data free wave with %d points", size);
     /* NB: Once we return anything than NEUTRAL, HDF5 will stop iterating. */
@@ -644,13 +640,15 @@ handle_cmap_dataset(hid_t loc_id,
     H5free_memory(blocksize_id);
 
     status = NHF_NEUTRAL;
-    gwy_clear(&cd, 1);
+    gwy_clear1(cd);
     /* dataset_is_placeholder() reads "dataset_size" attribute which used to be the true total size (curve length
      * sum), but can apparently be either that or the allocated size (fullsize) */
     npts = nhfile->xres * nhfile->yres;
     ndata = npts*nhfile->maxlen;
     gwy_debug("xres %d, yres %d, size %d, maxlen %d", nhfile->xres, nhfile->yres, size, nhfile->maxlen);
     if (size != nhfile->curve_lengths[npts]) {
+        gwy_debug("dataset has the wrong size %d (expecting %d values)", size, ndata);
+        return NHF_NEUTRAL;
         gwy_debug("dataset has the wrong size %d (expecting true size %d values)", size, nhfile->curve_lengths[npts]);
         if (size != ndata) {
             gwy_debug("dataset has the wrong size also for maxlen (virtual %d values), giving up", ndata);
@@ -677,7 +675,7 @@ handle_cmap_dataset(hid_t loc_id,
     gwyhdf5_get_str_attr_g(dataset_id, ".", "signal_name", &cd.name, NULL);
     gwy_debug("signal name %s", cd.name);
 
-    cd.unit = gwy_si_unit_new(NULL);
+    cd.unit = gwy_unit_new(NULL);
     prepare_calibration(dataset_id, cd.unit, &q, &v0);
     for (i = 0; i < ndata; i++)
         cd.data[i] = q*cd.data[i] + v0;
@@ -807,7 +805,7 @@ end:
         for (i = 0; i < nhfile->curve_data->len; i++) {
             NHFCurveData *cd = &g_array_index(nhfile->curve_data, NHFCurveData, i);
             g_free(cd->name);
-            GWY_OBJECT_UNREF(cd->unit);
+            g_clear_object(&cd->unit);
             g_free(cd->data);
         }
         g_array_free(nhfile->curve_data, TRUE);
@@ -878,7 +876,7 @@ handle_wnma_group(hid_t grp_id, NHFFile *nhfile)
     create_free_wave_graph(nhfile);
 
 end:
-    GWY_OBJECT_UNREF(nhfile->freewave);
+    g_clear_object(&nhfile->freewave);
     GWY_FREE(nhfile->freewave_name);
 
     return status;
@@ -1113,7 +1111,7 @@ enter_grouping(hid_t loc_id, const gchar *name,
     H5O_info_t infobuf;
     herr_t status;
 
-    status = H5Oget_info_by_name(loc_id, name, &infobuf, H5P_DEFAULT);
+    status = H5Oget_info_by_name(loc_id, name, &infobuf, H5O_INFO_BASIC, H5P_DEFAULT);
     if (status < 0) {
         err_HDF5(error, "H5Oget_info_by_name", status);
         return NHF_FAIL;
@@ -1145,55 +1143,54 @@ leave_grouping(hid_t obj_id, GString *path, guint pfxlen)
 static gboolean
 create_image_data(NHFFile *nhfile, hid_t dataset_id)
 {
-    GwyContainer *container = nhfile->container, *meta = nhfile->ghfile->meta;
-    GwyDataField *field = NULL;
+    GwyFile *file = nhfile->file;
+    GwyContainer *meta = nhfile->ghfile->meta;
+    GwyField *field = NULL;
     gchar *signal_name = NULL;
     gint id, xres = nhfile->xres, yres = nhfile->yres;
     herr_t status;
     gdouble *d;
     gdouble q, v0;
-    GQuark quark;
     gboolean ok = FALSE;
 
-    field = gwy_data_field_new(xres, yres, nhfile->xreal, nhfile->yreal, FALSE);
-    d = gwy_data_field_get_data(field);
+    field = gwy_field_new(xres, yres, nhfile->xreal, nhfile->yreal, FALSE);
+    d = gwy_field_get_data(field);
     if ((status = H5Dread(dataset_id, H5T_NATIVE_DOUBLE, H5S_ALL, H5S_ALL, H5P_DEFAULT, d)) < 0) {
         err_HDF5(&nhfile->error, "H5Dread", status);
         goto end;
     }
 
     /* FIXME: signal_data_pattern??? */
-    gwy_data_field_invert(field, TRUE, FALSE, FALSE);
+    gwy_field_flip(field, FALSE, TRUE);
 
     gwyhdf5_get_str_attr(dataset_id, ".", "signal_name", &signal_name, NULL);
 
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(field), "m");
-    prepare_calibration(dataset_id, gwy_data_field_get_si_unit_z(field), &q, &v0);
-    gwy_data_field_multiply(field, q);
-    gwy_data_field_add(field, v0);
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(field), "m");
+    prepare_calibration(dataset_id, gwy_field_get_unit_z(field), &q, &v0);
+    gwy_field_multiply(field, q);
+    gwy_field_add(field, v0);
 
     id = nhfile->image_id;
-    gwy_container_set_object(container, gwy_app_get_data_key_for_id(id), field);
-    gwy_file_channel_import_log_add(container, id, NULL, nhfile->filename);
+    gwy_file_set_image(file, id, field);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, id, NULL, nhfile->filename);
 
-    quark = gwy_app_get_data_title_key_for_id(id);
     if (signal_name && *signal_name)
-        gwy_container_set_const_string(container, quark, signal_name);
+        gwy_file_set_title(file, GWY_FILE_IMAGE, id, signal_name, FALSE);
     else
-        gwy_app_channel_title_fall_back(container, id);
-    add_segment_name_to_title(nhfile, quark);
+        gwy_image_title_fall_back(file, id);
+    add_segment_name_to_title(nhfile, gwy_file_key_image_title(id));
 
     if (gwy_container_get_n_items(meta)) {
-        meta = gwy_container_duplicate(meta);
+        meta = gwy_container_copy(meta);
         clean_up_json_meta(meta);
-        gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(id), meta);
+        gwy_file_pass_meta(file, GWY_FILE_IMAGE, id, meta);
     }
 
     nhfile->image_id++;
     ok = TRUE;
 
 end:
-    GWY_OBJECT_UNREF(field);
+    g_clear_object(&field);
     if (signal_name)
         H5free_memory(signal_name);
 
@@ -1203,14 +1200,14 @@ end:
 static gboolean
 create_volume_data(NHFFile *nhfile, hid_t dataset_id)
 {
-    GwyContainer *container = nhfile->container, *meta = nhfile->ghfile->meta;
+    GwyFile *file = nhfile->file;
+    GwyContainer *meta = nhfile->ghfile->meta;
     GwyBrick *brick = NULL, *tmpbrick = NULL;
     gchar *signal_name = NULL;
-    gint id, xres = nhfile->xres, yres = nhfile->yres, zres = gwy_data_line_get_res(nhfile->freewave);
+    gint id, xres = nhfile->xres, yres = nhfile->yres, zres = gwy_line_get_res(nhfile->freewave);
     herr_t status;
     gdouble *d;
     gdouble q, v0;
-    GQuark quark;
     gboolean ok = FALSE;
 
     /* The data are organised by spectra (i.e. z is the innermost index). We have to transpose to storage by planes.
@@ -1224,7 +1221,7 @@ create_volume_data(NHFFile *nhfile, hid_t dataset_id)
 
     brick = gwy_brick_new(xres, yres, zres, nhfile->xreal, nhfile->yreal, zres, FALSE);
     gwy_brick_transpose(tmpbrick, brick, GWY_BRICK_TRANSPOSE_ZXY, FALSE, FALSE, FALSE);
-    GWY_OBJECT_UNREF(tmpbrick);
+    g_clear_object(&tmpbrick);
 
     gwyhdf5_get_str_attr(dataset_id, ".", "signal_name", &signal_name, NULL);
 
@@ -1232,33 +1229,32 @@ create_volume_data(NHFFile *nhfile, hid_t dataset_id)
     gwy_brick_set_xreal(brick, nhfile->xreal);
     gwy_brick_set_yreal(brick, nhfile->yreal);
     gwy_brick_set_zreal(brick, zres);
-    gwy_si_unit_set_from_string(gwy_brick_get_si_unit_x(brick), "m");
-    gwy_si_unit_set_from_string(gwy_brick_get_si_unit_y(brick), "m");
-    prepare_calibration(dataset_id, gwy_brick_get_si_unit_w(brick), &q, &v0);
+    gwy_unit_set_from_string(gwy_brick_get_unit_x(brick), "m");
+    gwy_unit_set_from_string(gwy_brick_get_unit_y(brick), "m");
+    prepare_calibration(dataset_id, gwy_brick_get_unit_w(brick), &q, &v0);
     gwy_brick_multiply(brick, q);
     gwy_brick_add(brick, v0);
 
     id = nhfile->volume_id;
-    gwy_container_set_object(container, gwy_app_get_brick_key_for_id(id), brick);
-    gwy_file_volume_import_log_add(container, id, NULL, nhfile->filename);
+    gwy_file_set_volume(file, id, brick);
+    gwy_log_add_import(file, GWY_FILE_VOLUME, id, NULL, nhfile->filename);
 
-    quark = gwy_app_get_brick_title_key_for_id(id);
     if (signal_name && *signal_name)
-        gwy_container_set_const_string(container, quark, signal_name);
-    add_segment_name_to_title(nhfile, quark);
+        gwy_file_set_title(file, GWY_FILE_VOLUME, id, signal_name, FALSE);
+    add_segment_name_to_title(nhfile, gwy_file_key_volume_title(id));
 
     if (gwy_container_get_n_items(meta)) {
-        meta = gwy_container_duplicate(meta);
+        meta = gwy_container_copy(meta);
         clean_up_json_meta(meta);
-        gwy_container_pass_object(container, gwy_app_get_brick_meta_key_for_id(id), meta);
+        gwy_file_pass_meta(file, GWY_FILE_VOLUME, id, meta);
     }
 
     nhfile->volume_id++;
     ok = TRUE;
 
 end:
-    GWY_OBJECT_UNREF(brick);
-    GWY_OBJECT_UNREF(tmpbrick);
+    g_clear_object(&brick);
+    g_clear_object(&tmpbrick);
     if (signal_name)
         H5free_memory(signal_name);
 
@@ -1278,11 +1274,11 @@ create_free_wave_graph(NHFFile *nhfile)
 
     gmodel = gwy_graph_model_new();
     gcmodel = gwy_graph_curve_model_new();
-    gwy_graph_curve_model_set_data_from_dataline(gcmodel, nhfile->freewave, 0, 0);
+    gwy_graph_curve_model_set_data_from_line(gcmodel, nhfile->freewave, 0, 0);
     gwy_graph_model_add_curve(gmodel, gcmodel);
     g_object_unref(gcmodel);
     g_object_set(gmodel,
-                 "si-unit-y", gwy_data_line_get_si_unit_y(nhfile->freewave),
+                 "unit-y", gwy_line_get_unit_y(nhfile->freewave),
                  "axis-label-bottom", _("Sample"),
                  NULL);
     if (nhfile->freewave_name) {
@@ -1295,15 +1291,17 @@ create_free_wave_graph(NHFFile *nhfile)
                  "mode", GWY_GRAPH_CURVE_LINE,
                  "description", _("Time series"),
                  NULL);
-    gwy_container_pass_object(nhfile->container, gwy_app_get_graph_key_for_id(nhfile->graph_id), gmodel);
+    gwy_file_pass_graph(nhfile->file, nhfile->graph_id, gmodel);
+    gwy_log_add_import(nhfile->file, GWY_FILE_GRAPH, nhfile->graph_id, NULL, nhfile->filename);
     nhfile->graph_id++;
 }
 
 static void
 create_curve_map(NHFFile *nhfile)
 {
+    GwyFile *file = nhfile->file;
+    GwyContainer *meta = nhfile->ghfile->meta;
     GwyLawn *lawn;
-    GwyContainer *container = nhfile->container, *meta = nhfile->ghfile->meta;
     gdouble *buf;
     guint m, start, len, ncurves, maxlen = nhfile->maxlen;
     gint id, i, k, xres = nhfile->xres, yres = nhfile->yres, npts = xres*yres;
@@ -1329,26 +1327,26 @@ create_curve_map(NHFFile *nhfile)
 
     g_free(buf);
 
-    gwy_si_unit_set_from_string(gwy_lawn_get_si_unit_xy(lawn), "m");
+    gwy_unit_set_from_string(gwy_lawn_get_unit_xy(lawn), "m");
     for (m = 0; m < ncurves; m++) {
         cd = &g_array_index(nhfile->curve_data, NHFCurveData, m);
         gwy_lawn_set_curve_label(lawn, m, cd->name);
-        gwy_si_unit_assign(gwy_lawn_get_si_unit_curve(lawn, m), cd->unit);
+        gwy_unit_assign(gwy_lawn_get_unit_curve(lawn, m), cd->unit);
     }
 
     id = nhfile->cmap_id;
-    gwy_container_pass_object(container, gwy_app_get_lawn_key_for_id(id), lawn);
-    gwy_file_curve_map_import_log_add(container, id, NULL, nhfile->filename);
+    gwy_file_pass_cmap(file, id, lawn);
+    gwy_log_add_import(file, GWY_FILE_CMAP, id, NULL, nhfile->filename);
 
     if (gwyhdf5_get_str_attr(nhfile->file_id, nhfile->path->str, "segment_name", &name, NULL)) {
-        gwy_container_set_const_string(container, gwy_app_get_lawn_title_key_for_id(id), name);
+        gwy_file_set_title(file, GWY_FILE_CMAP, id, name, FALSE);
         H5free_memory(name);
     }
 
     if (gwy_container_get_n_items(meta)) {
-        meta = gwy_container_duplicate(meta);
+        meta = gwy_container_copy(meta);
         clean_up_json_meta(meta);
-        gwy_container_pass_object(container, gwy_app_get_lawn_meta_key_for_id(id), meta);
+        gwy_file_pass_meta(file, GWY_FILE_CMAP, id, meta);
     }
 
     nhfile->cmap_id++;
@@ -1359,7 +1357,7 @@ create_cmap_1x1_graph(NHFFile *nhfile)
 {
     GwyGraphModel *gmodel;
     GwyGraphCurveModel *gcmodel;
-    GwyContainer *container = nhfile->container;
+    GwyFile *file = nhfile->file;
     gdouble *buf;
     guint m, ncurves, clen;
     gchar *name = NULL;
@@ -1381,7 +1379,7 @@ create_cmap_1x1_graph(NHFFile *nhfile)
                      "title", cd->name,
                      "axis-label-bottom", _("Sample"),
                      "axis-label-left", cd->name,
-                     "si-unit-y", cd->unit,
+                     "unit-y", cd->unit,
                      NULL);
         gcmodel = gwy_graph_curve_model_new();
         g_object_set(gcmodel, "mode", GWY_GRAPH_CURVE_LINE, NULL);
@@ -1390,7 +1388,9 @@ create_cmap_1x1_graph(NHFFile *nhfile)
         gwy_graph_curve_model_set_data(gcmodel, buf, cd->data, clen);
         gwy_graph_model_add_curve(gmodel, gcmodel);
         g_object_unref(gcmodel);
-        gwy_container_pass_object(container, gwy_app_get_graph_key_for_id(nhfile->graph_id++), gmodel);
+        gwy_file_pass_graph(file, nhfile->graph_id, gmodel);
+        gwy_log_add_import(file, GWY_FILE_GRAPH, nhfile->graph_id, NULL, nhfile->filename);
+        nhfile->graph_id++;
     }
 
     g_free(buf);
@@ -1433,7 +1433,7 @@ get_image_real_range(hid_t loc_id, NHFFile *nhfile)
 }
 
 static gboolean
-prepare_calibration(hid_t dataset_id, GwySIUnit *unit,
+prepare_calibration(hid_t dataset_id, GwyUnit *unit,
                     gdouble *q, gdouble *v0)
 {
     gdouble calib_min, calib_max, signal_minmax[2], r, p;
@@ -1447,7 +1447,7 @@ prepare_calibration(hid_t dataset_id, GwySIUnit *unit,
      * seem to have "signal_calibration_unit" so the condition below does not actually work. */
     if (!gwyhdf5_get_str_attr_g(dataset_id, ".", "signal_calibration_unit", &signal_unit, NULL)) {
         if (!gwyhdf5_get_str_attr_g(dataset_id, ".", "signal_unit", &signal_unit, NULL)) {
-            gwy_si_unit_set_from_string(unit, NULL);
+            gwy_unit_clear(unit);
             g_warning("Missing signal units.");
             return FALSE;
         }
@@ -1455,10 +1455,10 @@ prepare_calibration(hid_t dataset_id, GwySIUnit *unit,
     }
 
     signal_unit = extract_unit_symbol(signal_unit);
-    gwy_si_unit_set_from_string_parse(unit, signal_unit, &power10);
+    power10 = gwy_unit_set_from_string(unit, signal_unit);
     g_free(signal_unit);
 
-    *q = p = pow10(power10);
+    *q = p = gwy_exp10(power10);
     if (!have_calibration)
         return TRUE;
 
@@ -1511,9 +1511,9 @@ add_segment_name_to_title(NHFFile *nhfile, GQuark quark)
     if (!nhfile->segment_name)
         return;
 
-    s = gwy_container_get_string(nhfile->container, quark);
+    s = gwy_container_get_string(GWY_CONTAINER(nhfile->file), quark);
     t = g_strconcat(s, " [", nhfile->segment_name, "]", NULL);
-    gwy_container_set_string(nhfile->container, quark, t);
+    gwy_container_set_string(GWY_CONTAINER(nhfile->file), quark, t);
 }
 
 static gboolean
@@ -1535,21 +1535,21 @@ set_gerror_hdf_internal(NHFFile *nhfile, const gchar *funcname, herr_t status)
         err_HDF5(&nhfile->error, funcname, status);
 }
 
-#ifdef HAVE_JANSSON
+#ifdef HAVE_JSON_GLIB
 /* Expand JSON metadata into individual fields. Silently ignore any errors. */
 static gboolean
-format_atomic_json_meta(json_t *item, GString *buf)
+format_atomic_json_meta(JsonNode *node, GString *str)
 {
-    if (json_is_string(item))
-        g_string_append(buf, json_string_value(item));
-    else if (json_is_true(item))
-        g_string_append(buf, "True");
-    else if (json_is_false(item))
-        g_string_append(buf, "False");
-    else if (json_is_real(item))
-        g_string_append_printf(buf, "%g", json_real_value(item));
-    else if (json_is_integer(item))
-        g_string_append_printf(buf, "%ld", (glong)json_integer_value(item));
+    GType type = json_node_get_value_type(node);
+
+    if (type == G_TYPE_STRING)
+        g_string_append(str, json_node_get_string(node));
+    else if (type == G_TYPE_INT64)
+        g_string_append_printf(str, "%" G_GINT64_FORMAT, json_node_get_int(node));
+    else if (type == G_TYPE_DOUBLE)
+        g_string_append_printf(str, "%g", json_node_get_double(node));
+    else if (type == G_TYPE_BOOLEAN)
+        g_string_assign(str, json_node_get_boolean(node) ? "True" : "False");
     else
         return FALSE;
 
@@ -1557,46 +1557,61 @@ format_atomic_json_meta(json_t *item, GString *buf)
 }
 
 static void
-expand_json_meta(GwyContainer *meta, const gchar *str, GString *path, GString *buf)
+expand_json_meta(GwyContainer *meta, JsonParser *parser, const gchar *str, GString *path, GString *buf)
 {
-    const gchar *name, *propname;
-    json_error_t jerror;
-    json_t *root, *member, *item, *value, *unit;
-    guint i, pfxlen;
-
-    if (!(root = json_loads(str, 0, &jerror)))
+    if (!json_parser_load_from_data(parser, str, -1, NULL))
         return;
-    if (!json_is_object(root))
-        goto end;
 
+    JsonNode *root_node = json_parser_get_root(parser);
+    if (!JSON_NODE_HOLDS_OBJECT(root_node))
+        return;
+
+    JsonObject *root = json_node_get_object(root_node);
     g_string_append(path, "::");
-    pfxlen = path->len;
-    json_object_foreach(root, name, member) {
+    guint pfxlen = path->len;
+
+    JsonObjectIter iter;
+    json_object_iter_init(&iter, root);
+
+    const gchar *name;
+    JsonNode *node;
+    while (json_object_iter_next(&iter, &name, &node)) {
         g_string_append(path, name);
         g_string_truncate(buf, 0);
-        if (format_atomic_json_meta(member, buf))
+        if (format_atomic_json_meta(node, buf)) {
             gwy_container_set_const_string_by_name(meta, path->str, buf->str);
-        else if (json_is_array(member)) {
-            json_array_foreach(member, i, item) {
-                if (format_atomic_json_meta(item, buf))
+        }
+        else if (JSON_NODE_HOLDS_ARRAY(node)) {
+            JsonArray *array = json_node_get_array(node);
+            guint n = json_array_get_length(array);
+            for (guint i = 0; i < n; i++) {
+                JsonNode *item_node = json_array_get_element(array, i);
+                if (format_atomic_json_meta(item_node, buf))
                     g_string_append(buf, "; ");
             }
             if (buf->len >= 2)
                 g_string_truncate(buf, buf->len-2);
             gwy_container_set_const_string_by_name(meta, path->str, buf->str);
         }
-        else if (json_is_object(member) && gwy_strequal(name, "property")) {
+        else if (JSON_NODE_HOLDS_OBJECT(node) && gwy_strequal(name, "property")) {
             /* "property" is a nested dictionary with further nested members name:{value:x} or name{value:x,unit:y}.
              * Remove some of the silly nesting levels to make it look reasonable in the metadata. */
-            json_object_foreach(member, propname, item) {
-                if (!json_is_object(item))
+            JsonObject *property = json_node_get_object(node);
+            JsonObjectIter property_iter;
+            json_object_iter_init(&property_iter, property);
+
+            const gchar *property_name;
+            JsonNode *property_node;
+            while (json_object_iter_next(&property_iter, &property_name, &property_node)) {
+                if (!JSON_NODE_HOLDS_OBJECT(property_node))
                     continue;
-                value = json_object_get(item, "value");
-                unit = json_object_get(item, "unit");
+                JsonObject *property_item = json_node_get_object(property_node);
+                JsonNode *value = json_object_get_member(property_item, "value");
+                JsonNode *unit = json_object_get_member(property_item, "unit");
                 if (value) {
                     g_string_truncate(path, pfxlen);
                     g_string_truncate(buf, 0);
-                    g_string_append(path, propname);
+                    g_string_append(path, property_name);
                     format_atomic_json_meta(value, buf);
                     if (unit) {
                         g_string_append(buf, " ");
@@ -1607,13 +1622,10 @@ expand_json_meta(GwyContainer *meta, const gchar *str, GString *path, GString *b
             }
         }
         else {
-            gwy_debug("unhandled meta %s of type %d", name, json_typeof(member));
+            gwy_debug("unhandled meta %s of type %d", name, JSON_NODE_TYPE(member));
         }
         g_string_truncate(path, pfxlen);
     }
-
-end:
-    json_decref(root);
 }
 #endif
 
@@ -1622,12 +1634,13 @@ clean_up_json_meta(GwyContainer *meta)
 {
     GQuark *keys = gwy_container_keys(meta);
     guint i, n = gwy_container_get_n_items(meta);
-#ifdef HAVE_JANSSON
+#ifdef HAVE_JSON_GLIB
     GString *path = NULL, *buf = NULL;
+    JsonParser *parser = NULL;
 #endif
 
     for (i = 0; i < n; i++) {
-        const gchar *key, *value = gwy_container_get_string(meta, keys[i]);
+        const gchar *value = gwy_container_get_string(meta, keys[i]);
         guint len, x = 0, jschars = 0;
         gchar c;
 
@@ -1647,21 +1660,24 @@ clean_up_json_meta(GwyContainer *meta)
             continue;
 
         gwy_debug("handling JSON meta %.16s...", value);
-#ifdef HAVE_JANSSON
-        key = g_quark_to_string(keys[i]);
+#ifdef HAVE_JSON_GLIB
+        const gchar *key = g_quark_to_string(keys[i]);
         path = path ? g_string_assign(path, key) : g_string_new(key);
         if (!buf)
             buf = g_string_new(NULL);
+        if (!parser)
+            parser = json_parser_new_immutable();
 
         /* NB: It cannot not simply create a metadata item with the same name because we would remove it below.
          * We could to it in the opposite order if we duplicated the string… */
-        expand_json_meta(meta, value, path, buf);
+        expand_json_meta(meta, parser, value, path, buf);
 #endif
         gwy_container_remove(meta, keys[i]);
     }
 
     g_free(keys);
-#ifdef HAVE_JANSSON
+#ifdef HAVE_JSON_GLIB
+    g_clear_object(&parser);
     if (path)
         g_string_free(path, TRUE);
     if (buf)
