@@ -1,5 +1,5 @@
 /*
- *  $Id: nsidfile.c 28533 2025-09-10 14:07:50Z yeti-dn $
+ *  $Id: nsidfile.c 28911 2025-11-24 18:27:42Z yeti-dn $
  *  Copyright (C) 2024 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -28,11 +28,9 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/datafield.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 
 #include "gwyhdf5.h"
 #include "hdf5file.h"
@@ -61,32 +59,29 @@ typedef struct {
     GString *str;
 } CopyMetaData;
 
-static gint          nsid_detect              (const GwyFileDetectInfo *fileinfo,
-                                               gboolean only_name);
-static GwyContainer* nsid_load                (const gchar *filename,
-                                               GwyRunType mode,
-                                               GError **error);
-static void          read_image               (hid_t file_id,
-                                               const gchar *name,
-                                               GwyHDF5File *ghfile,
-                                               const gchar *filename,
-                                               GwyContainer *container,
-                                               gint *id);
-static gint          find_corresponding_scales(hid_t file_id,
-                                               hid_t dataset,
-                                               GwyHDF5File *ghfile,
-                                               gchar **data_scales,
-                                               guint ndims);
+static gint     detect_file              (const GwyFileDetectInfo *fileinfo,
+                                          gboolean only_name);
+static GwyFile* load_file                (const gchar *filename,
+                                          GwyRunModeFlags mode,
+                                          GError **error);
+static void     read_image               (hid_t file_id,
+                                          const gchar *name,
+                                          GwyHDF5File *ghfile,
+                                          const gchar *filename,
+                                          GwyFile *file,
+                                          gint *id);
+static gint     find_corresponding_scales(hid_t file_id,
+                                          hid_t dataset,
+                                          GwyHDF5File *ghfile,
+                                          gchar **data_scales,
+                                          guint ndims);
 
 void
 gwyhdf5_register_nsidfile(void)
 {
     gwy_file_func_register("nsidfile",
                            N_("N-Dimensional Spectroscopy and Imaging Data (NSID) HDF5 files (.h5)"),
-                           (GwyFileDetectFunc)&nsid_detect,
-                           (GwyFileLoadFunc)&nsid_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 }
 
 static void
@@ -127,7 +122,7 @@ detect_group_scan(hid_t loc_id,
 
     gwy_debug("%s", name);
 
-    status = H5Oget_info_by_name(loc_id, name, &infobuf, H5P_DEFAULT);
+    status = H5Oget_info_by_name(loc_id, name, &infobuf, H5O_INFO_BASIC, H5P_DEFAULT);
     if (status < 0)
         return status;
 
@@ -149,8 +144,7 @@ detect_group_scan(hid_t loc_id,
 }
 
 static gint
-nsid_detect(const GwyFileDetectInfo *fileinfo,
-            gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     static const guint other_flags[] = {
         FOUND_MACHINE_ID, FOUND_PLATFORM, FOUND_TIMESTAMP, FOUND_TITLE, FOUND_MODALITY, FOUND_DATA_TYPE,
@@ -193,12 +187,12 @@ nsid_detect(const GwyFileDetectInfo *fileinfo,
     return score;
 }
 
-static GwyContainer*
-nsid_load(const gchar *filename,
-          G_GNUC_UNUSED GwyRunType mode,
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
           GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     GwyHDF5File ghfile;
     hid_t file_id;
     herr_t status;
@@ -211,42 +205,41 @@ nsid_load(const gchar *filename,
         return NULL;
     }
     gwy_debug("file_id %d", (gint)file_id);
-    status = H5Oget_info(file_id, &infobuf);
+    status = H5Oget_info(file_id, &infobuf, H5O_INFO_BASIC);
     if (!gwyhdf5_check_status(status, file_id, NULL, "H5Oget_info", error))
         return NULL;
 
-    gwyhdf5_init(&ghfile);
-    g_array_append_val(ghfile.addr, infobuf.addr);
+    gwyhdf5_init(&ghfile, file_id, infobuf.token);
     status = H5Literate(file_id, H5_INDEX_NAME, H5_ITER_NATIVE, NULL, gwyhdf5_scan_file, &ghfile);
     if (!gwyhdf5_check_status(status, file_id, &ghfile, "H5Literate", error))
         return NULL;
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
 
     /* Read generic simple 2D data, i.e. images.
      * TODO: May want to read also other data types. */
     id = 0;
     for (i = 0; i < ghfile.datasets->len; i++)
-        read_image(file_id, g_array_index(ghfile.datasets, gchar*, i), &ghfile, filename, container, &id);
+        read_image(file_id, g_array_index(ghfile.datasets, gchar*, i), &ghfile, filename, file, &id);
 
     status = gwyhdf5_fclose(file_id);
     gwy_debug("status %d", status);
 
     gwyhdf5_free(&ghfile);
 
-    if (!gwy_container_get_n_items(container)) {
-        GWY_OBJECT_UNREF(container);
+    if (!gwy_container_get_n_items(GWY_CONTAINER(file))) {
+        g_clear_object(&file);
         err_NO_DATA(error);
     }
 
-    return container;
+    return file;
 }
 
 static void
-copy_meta(gpointer hkey, gpointer hvalue, gpointer user_data)
+copy_meta(GQuark quark, GValue *gvalue, gpointer user_data)
 {
-    const gchar *key = g_quark_to_string(GPOINTER_TO_UINT(hkey));
-    const gchar *value = g_value_get_string((GValue*)hvalue);
+    const gchar *key = g_quark_to_string(quark);
+    const gchar *value = g_value_get_string(gvalue);
     CopyMetaData *cmd = (CopyMetaData*)user_data;
     GString *str = cmd->str;
     gchar *s, *newkey = NULL;
@@ -288,12 +281,12 @@ copy_meta(gpointer hkey, gpointer hvalue, gpointer user_data)
 
 static void
 read_image(hid_t file_id, const gchar *path, GwyHDF5File *ghfile, const gchar *filename,
-           GwyContainer *container, gint *id)
+           GwyFile *file, gint *id)
 {
     enum { FIELD_NDIMS = 2 };
     gint dims[FIELD_NDIMS], scaledim[1];
     gchar *dim_scales[FIELD_NDIMS];
-    GwyDataField *field;
+    GwyField *field;
     CopyMetaData cmd;
     hid_t dataset;
     herr_t status;
@@ -317,8 +310,8 @@ read_image(hid_t file_id, const gchar *path, GwyHDF5File *ghfile, const gchar *f
 
     find_corresponding_scales(file_id, dataset, ghfile, dim_scales, FIELD_NDIMS);
 
-    field = gwy_data_field_new(dims[1], dims[0], dims[1], dims[0], FALSE);
-    status = H5Dread(dataset, H5T_NATIVE_DOUBLE, H5S_ALL, H5S_ALL, H5P_DEFAULT, gwy_data_field_get_data(field));
+    field = gwy_field_new(dims[1], dims[0], dims[1], dims[0], FALSE);
+    status = H5Dread(dataset, H5T_NATIVE_DOUBLE, H5S_ALL, H5S_ALL, H5P_DEFAULT, gwy_field_get_data(field));
     gwy_debug("status %d", status);
     H5Dclose(dataset);
     if (status < 0) {
@@ -327,9 +320,9 @@ read_image(hid_t file_id, const gchar *path, GwyHDF5File *ghfile, const gchar *f
     }
 
     if (gwyhdf5_get_str_attr_g(file_id, path, "units", &s, NULL)) {
-        gwy_si_unit_set_from_string_parse(gwy_data_field_get_si_unit_z(field), s, &power10);
+        power10 = gwy_unit_set_from_string(gwy_field_get_unit_z(field), s);
         if (power10)
-            gwy_data_field_multiply(field, pow10(power10));
+            gwy_field_multiply(field, gwy_exp10(power10));
         g_free(s);
     }
 
@@ -340,7 +333,7 @@ read_image(hid_t file_id, const gchar *path, GwyHDF5File *ghfile, const gchar *f
         power10 = 0;
         if (gwyhdf5_get_str_attr_g(file_id, dim_scales[j], "units", &s, NULL)) {
             /* This ultimately uses Y for units because it comes second. */
-            gwy_si_unit_set_from_string_parse(gwy_data_field_get_si_unit_xy(field), s, &power10);
+            power10 = gwy_unit_set_from_string(gwy_field_get_unit_xy(field), s);
             g_free(s);
         }
 
@@ -353,32 +346,33 @@ read_image(hid_t file_id, const gchar *path, GwyHDF5File *ghfile, const gchar *f
                     dx *= dims[j]/(dims[j] - 1.0);
 
                 if (j == 1)
-                    gwy_data_field_set_yreal(field, pow10(power10)*dx);
+                    gwy_field_set_yreal(field, gwy_exp10(power10)*dx);
                 else
-                    gwy_data_field_set_xreal(field, pow10(power10)*dx);
+                    gwy_field_set_xreal(field, gwy_exp10(power10)*dx);
             }
             g_free(scale_data);
             H5Dclose(dataset);
         }
     }
 
-    gwy_container_pass_object(container, gwy_app_get_data_key_for_id(*id), field);
-    gwy_file_channel_import_log_add(container, *id, NULL, filename);
+    gwy_file_pass_image(file, *id, field);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, *id, NULL, filename);
 
     if (gwyhdf5_get_str_attr_g(file_id, path, "title", &s, NULL)
         || gwyhdf5_get_str_attr_g(file_id, path, "quantity", &s, NULL)
         || gwyhdf5_get_str_attr_g(file_id, path, "main_data_name", &s, NULL))
-        gwy_container_set_string(container, gwy_app_get_data_title_key_for_id(*id), s);
+        gwy_file_pass_title(file, GWY_FILE_IMAGE, *id, s);
 
-    cmd.meta = gwy_container_new();
+    cmd.meta = gwy_container_new_in_construction();
     cmd.prefix = g_strdup(path);
     if ((s = strrchr(cmd.prefix, '/')))
         *s = '\0';
     cmd.str = g_string_new("/");
     gwy_container_foreach(ghfile->meta, NULL, copy_meta, &cmd);
     if (gwy_container_get_n_items(cmd.meta))
-        gwy_container_set_object(container, gwy_app_get_data_meta_key_for_id(*id), cmd.meta);
-    g_object_unref(cmd.meta);
+        gwy_file_pass_meta(file, GWY_FILE_IMAGE, *id, cmd.meta);
+    else
+        g_object_unref(cmd.meta);
     g_string_free(cmd.str, TRUE);
     g_free(cmd.prefix);
 
