@@ -1,6 +1,6 @@
 /*
- *  $Id: hitachi-afm.c 25687 2023-09-19 14:44:44Z yeti-dn $
- *  Copyright (C) 2005 David Necas (Yeti), Petr Klapetek, Markus Pristovsek
+ *  $Id: hitachi-afm.c 28848 2025-11-10 18:54:08Z yeti-dn $
+ *  Copyright (C) 2005-2025 David Necas (Yeti), Petr Klapetek, Markus Pristovsek
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *  prissi@gift.physik.tu-berlin.de.
  *
@@ -16,7 +16,7 @@
  *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-/* Note read_data_field() and read_data_field_old() read two completely different file formats, even if they were
+/* Note read_field() and read_field_old() read two completely different file formats, even if they were
  * probably both invented at Hitachi. */
 
 /**
@@ -46,14 +46,9 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <libprocess/datafield.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -81,28 +76,36 @@ enum {
     NS_OFFSET_OLD    = 0xc8,
 };
 
-static gboolean      module_register    (void);
-static gint          hitachi_detect     (const GwyFileDetectInfo *fileinfo,
-                                         gboolean only_name);
-static gint          hitachi_old_detect (const GwyFileDetectInfo *fileinfo,
-                                         gboolean only_name);
-static GwyContainer* hitachi_load       (const gchar *filename,
-                                         GwyRunType mode,
-                                         GError **error,
-                                         const gchar *name);
-static GwyDataField* read_data_field    (const guchar *buffer,
-                                         guint size,
-                                         GError **error);
-static GwyDataField* read_data_field_old(const guchar *buffer,
-                                         guint size,
-                                         GError **error);
+typedef GwyField* (*ReadImageFunc)(const guchar *buffer, gsize size, GError **error);
+
+static gboolean  module_register(void);
+static gint      detect_file    (const GwyFileDetectInfo *fileinfo,
+                                 gboolean only_name);
+static gint      file_detect_old(const GwyFileDetectInfo *fileinfo,
+                                 gboolean only_name);
+static GwyFile*  load_file      (const gchar *filename,
+                                 GwyRunModeFlags mode,
+                                 GError **error);
+static GwyFile*  file_load_old  (const gchar *filename,
+                                 GwyRunModeFlags mode,
+                                 GError **error);
+static GwyFile*  hitachi_load   (const gchar *filename,
+                                 ReadImageFunc read_image,
+                                 gsize header_size,
+                                 GError **error);
+static GwyField* read_field     (const guchar *buffer,
+                                 gsize size,
+                                 GError **error);
+static GwyField* read_field_old (const guchar *buffer,
+                                 gsize size,
+                                 GError **error);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Hitachi AFM files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.7",
+    "0.8",
     "David Nečas (Yeti) & Petr Klapetek & Markus Pristovsek",
     "2005",
 };
@@ -115,23 +118,16 @@ module_register(void)
     /* Register two functions to keep the disctinction in the app although the load function physically the same */
     gwy_file_func_register("hitachi-afm",
                            N_("Hitachi AFM files (.afm)"),
-                           (GwyFileDetectFunc)&hitachi_detect,
-                           (GwyFileLoadFunc)&hitachi_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
     gwy_file_func_register("hitachi-afm-old",
                            N_("Hitachi AFM files, old (.afm)"),
-                           (GwyFileDetectFunc)&hitachi_old_detect,
-                           (GwyFileLoadFunc)&hitachi_load,
-                           NULL,
-                           NULL);
+                           file_detect_old, file_load_old, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-hitachi_detect(const GwyFileDetectInfo *fileinfo,
-               gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     gint score = 0;
 
@@ -147,12 +143,9 @@ hitachi_detect(const GwyFileDetectInfo *fileinfo,
 }
 
 static gint
-hitachi_old_detect(const GwyFileDetectInfo *fileinfo,
-                   gboolean only_name)
+file_detect_old(const GwyFileDetectInfo *fileinfo,
+                gboolean only_name)
 {
-    guint xres, yres;
-    const guchar *p;
-
     if (only_name)
         return g_str_has_suffix(fileinfo->name_lowercase, EXTENSION) ? 10 : 0;
 
@@ -162,9 +155,9 @@ hitachi_old_detect(const GwyFileDetectInfo *fileinfo,
         || fileinfo->head[0] != 0 || fileinfo->head[1] != 1)
         return 0;
 
-    p = fileinfo->head + RES_OFFSET_OLD;
-    xres = gwy_get_guint16_le(&p);
-    yres = gwy_get_guint16_le(&p);
+    const guchar *p = fileinfo->head + RES_OFFSET_OLD;
+    guint xres = gwy_get_guint16_le(&p);
+    guint yres = gwy_get_guint16_le(&p);
 
     if (fileinfo->file_size == 2*xres*yres + HEADER_SIZE_OLD)
         return 100;
@@ -172,33 +165,31 @@ hitachi_old_detect(const GwyFileDetectInfo *fileinfo,
     return 0;
 }
 
-static GwyContainer*
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
+{
+    return hitachi_load(filename, read_field, HEADER_SIZE, error);
+}
+
+static GwyFile*
+file_load_old(const gchar *filename,
+              G_GNUC_UNUSED GwyRunModeFlags mode,
+              GError **error)
+{
+    return hitachi_load(filename, read_field_old, HEADER_SIZE_OLD, error);
+}
+
+static GwyFile*
 hitachi_load(const gchar *filename,
-             G_GNUC_UNUSED GwyRunType mode,
-             GError **error,
-             const gchar *name)
+             ReadImageFunc read_image,
+             gsize header_size,
+             GError **error)
 {
-    GwyContainer *container = NULL;
     guchar *buffer = NULL;
     gsize size = 0;
     GError *err = NULL;
-    GwyDataField *dfield = NULL;
-    GwyDataField *(*do_load)(const guchar*, guint, GError**);
-    guint header_size;
-
-    if (gwy_strequal(name, "hitachi-afm")) {
-        do_load = &read_data_field;
-        header_size = HEADER_SIZE;
-    }
-    else if (gwy_strequal(name, "hitachi-afm-old")) {
-        do_load = &read_data_field_old;
-        header_size = HEADER_SIZE_OLD;
-    }
-    else {
-        g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_UNIMPLEMENTED,
-                    _("Hitachi-AFM has not registered file type `%s'."), name);
-        return NULL;
-    }
 
     if (!gwy_file_get_contents(filename, &buffer, &size, &err)) {
         err_GET_FILE_CONTENTS(error, &err);
@@ -210,29 +201,29 @@ hitachi_load(const gchar *filename,
         return NULL;
     }
 
-    dfield = do_load(buffer, size, error);
+    GwyField *dfield = read_image(buffer, size, error);
     gwy_file_abandon_contents(buffer, size, NULL);
     if (!dfield)
         return NULL;
 
-    container = gwy_container_new();
-    gwy_container_pass_object(container, gwy_app_get_data_key_for_id(0), dfield);
-    gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(0), "Topography");
+    GwyFile *file = gwy_file_new_in_construction();
+    gwy_file_pass_image(file, 0, dfield);
+    gwy_file_set_title(file, GWY_FILE_IMAGE, 0, "Topography", FALSE);
 
-    gwy_app_channel_check_nonsquare(container, 0);
-    gwy_file_channel_import_log_add(container, 0, NULL, filename);
+    gwy_check_nonsquare_image(file, 0);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
 
-    return container;
+    return file;
 }
 
-static GwyDataField*
-read_data_field(const guchar *buffer,
-                guint size,
-                GError **error)
+static GwyField*
+read_field(const guchar *buffer,
+           gsize size,
+           GError **error)
 {
     gint xres, yres, n;
     gdouble xreal, yreal, q;
-    GwyDataField *dfield;
+    GwyField *dfield;
     const guchar *p;
 
     p = buffer + RES_OFFSET;
@@ -260,27 +251,27 @@ read_data_field(const guchar *buffer,
     q /= 2.0;
     q /= 65536.0;
 
-    dfield = gwy_data_field_new(xres, yres, xreal, yreal, FALSE);
+    dfield = gwy_field_new(xres, yres, xreal, yreal, FALSE);
     gwy_convert_raw_data(buffer + HEADER_SIZE, xres*yres, 1, GWY_RAW_DATA_UINT16, GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                         gwy_data_field_get_data(dfield), q, 0.0);
-    gwy_data_field_invert(dfield, TRUE, FALSE, FALSE);
+                         gwy_field_get_data(dfield), q, 0.0);
+    gwy_field_flip(dfield, FALSE, TRUE);
 
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), "m");
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
+    gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), "m");
 
     return dfield;
 }
 
-static GwyDataField*
-read_data_field_old(const guchar *buffer,
-                    guint size,
-                    GError **error)
+static GwyField*
+read_field_old(const guchar *buffer,
+               gsize size,
+               GError **error)
 {
     gint xres, yres, n, vx, vy;
     G_GNUC_UNUSED gint vz;
     gdouble xscale, yscale, zscale, xreal, yreal, q;
     G_GNUC_UNUSED gdouble xunit, yunit, zunit;
-    GwyDataField *dfield;
+    GwyField *dfield;
     const guchar *p;
 
     p = buffer + RES_OFFSET_OLD;
@@ -314,12 +305,12 @@ read_data_field_old(const guchar *buffer,
     sanitise_real_size(&xreal, "x size");
     sanitise_real_size(&yreal, "y size");
 
-    dfield = gwy_data_field_new(xres, yres, xreal, yreal, FALSE);
+    dfield = gwy_field_new(xres, yres, xreal, yreal, FALSE);
     gwy_convert_raw_data(buffer + HEADER_SIZE_OLD, xres*yres, 1, GWY_RAW_DATA_UINT16, GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                         gwy_data_field_get_data(dfield), q, 0.0);
+                         gwy_field_get_data(dfield), q, 0.0);
 
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), "m");
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
+    gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), "m");
 
     return dfield;
 }
