@@ -1,6 +1,6 @@
 /*
- *  $Id: hitachi-sem.c 25797 2023-10-06 15:38:40Z yeti-dn $
- *  Copyright (C) 2011 David Necas (Yeti).
+ *  $Id: hitachi-sem.c 28789 2025-11-04 17:14:03Z yeti-dn $
+ *  Copyright (C) 2011-2025 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
  *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
@@ -40,16 +40,11 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/stats.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -59,27 +54,27 @@
 #define MAGIC_SIZE (sizeof(MAGIC)-1)
 #define HEADER_EXTENSION ".txt"
 
-static gboolean      module_register       (void);
-static gint          hitachi_detect        (const GwyFileDetectInfo *fileinfo,
-                                            gboolean only_name);
-static GwyContainer* hitachi_load          (const gchar *filename,
-                                            GwyRunType mode,
-                                            GError **error);
-static GHashTable*   hitachi_load_header   (const gchar *filename,
-                                            gchar **header,
-                                            GError **error);
-static gchar*        hitachi_find_data_name(const gchar *header_name,
-                                            const gchar *image_name);
-static void          store_meta            (gpointer key,
-                                            gpointer value,
-                                            gpointer user_data);
+static gboolean    module_register       (void);
+static gint        detect_file           (const GwyFileDetectInfo *fileinfo,
+                                          gboolean only_name);
+static GwyFile*    load_file             (const gchar *filename,
+                                          GwyRunModeFlags mode,
+                                          GError **error);
+static GHashTable* hitachi_load_header   (const gchar *filename,
+                                          gchar **header,
+                                          GError **error);
+static gchar*      hitachi_find_data_name(const gchar *header_name,
+                                          const gchar *image_name);
+static void        store_meta            (gpointer key,
+                                          gpointer value,
+                                          gpointer user_data);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Hitachi S-3700 and S-4800 SEM files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.2",
+    "0.3",
     "David Nečas (Yeti)",
     "2011",
 };
@@ -91,17 +86,13 @@ module_register(void)
 {
     gwy_file_func_register("hitachi-sem",
                            N_("Hitachi SEM files (.txt + image)"),
-                           (GwyFileDetectFunc)&hitachi_detect,
-                           (GwyFileLoadFunc)&hitachi_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-hitachi_detect(const GwyFileDetectInfo *fileinfo,
-               gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     GHashTable *hash;
     gchar *header, *imagename, *fullname;
@@ -132,14 +123,15 @@ hitachi_detect(const GwyFileDetectInfo *fileinfo,
     return score;
 }
 
-static GwyContainer*
-hitachi_load(const gchar *filename,
-             G_GNUC_UNUSED GwyRunType mode,
-             GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *container = NULL, *meta;
+    GwyFile *file = NULL;
+    GwyContainer *meta;
     GdkPixbuf *pixbuf = NULL;
-    GwyDataField *dfield = NULL;
+    GwyField *dfield = NULL;
     gchar *value, *imagename = NULL, *header = NULL;
     guchar *pixels;
     GHashTable *hash = NULL;
@@ -190,10 +182,10 @@ hitachi_load(const gchar *filename,
     sanitise_real_size(&dx, "pixel size");
     dx *= Nanometre;
 
-    dfield = gwy_data_field_new(hxres, hyres, hxres*dx, hyres*dx, FALSE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
+    dfield = gwy_field_new(hxres, hyres, hxres*dx, hyres*dx, FALSE);
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
 
-    data = gwy_data_field_get_data(dfield);
+    data = gwy_field_get_data(dfield);
     pixels = gdk_pixbuf_get_pixels(pixbuf);
     rowstride = gdk_pixbuf_get_rowstride(pixbuf);
     nchannels = gdk_pixbuf_get_n_channels(pixbuf);
@@ -205,27 +197,27 @@ hitachi_load(const gchar *filename,
             drow[j] = (p[0] + p[1] + p[2])/765.0;
     }
 
-    container = gwy_container_new();
-    gwy_container_pass_object(container, gwy_app_get_data_key_for_id(0), dfield);
+    file = gwy_file_new_in_construction();
+    gwy_file_pass_image(file, 0, dfield);
 
     if ((value = g_hash_table_lookup(hash, "SampleName")) && *value)
-        gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(0), value);
+        gwy_file_set_title(file, GWY_FILE_IMAGE, 0, value, FALSE);
     else
-        gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(0), "SEM");
+        gwy_file_set_title(file, GWY_FILE_IMAGE, 0, "SEM", FALSE);
 
-    meta = gwy_container_new();
+    meta = gwy_container_new_in_construction();
     g_hash_table_foreach(hash, store_meta, meta);
-    gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(0), meta);
+    gwy_file_pass_meta(file, GWY_FILE_IMAGE, 0, meta);
 
-    gwy_file_channel_import_log_add(container, 0, NULL, filename);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
 
 fail:
-    GWY_OBJECT_UNREF(pixbuf);
+    g_clear_object(&pixbuf);
     g_free(imagename);
     g_free(header);
     g_hash_table_destroy(hash);
 
-    return container;
+    return file;
 }
 
 static GHashTable*
@@ -254,7 +246,7 @@ hitachi_load_header(const gchar *filename,
         return NULL;
     }
 
-    gwy_clear(&parser, 1);
+    gwy_clear1(parser);
     parser.key_value_separator = "=";
     gwy_debug("reading header");
     hash = gwy_text_header_parse(p, &parser, NULL, NULL);
@@ -306,9 +298,7 @@ hitachi_find_data_name(const gchar *header_name,
 }
 
 static void
-store_meta(gpointer key,
-           gpointer value,
-           gpointer user_data)
+store_meta(gpointer key, gpointer value, gpointer user_data)
 {
     GwyContainer *meta = (GwyContainer*)user_data;
 
