@@ -1,6 +1,6 @@
 /*
- *  $Id: igorfile.c 27984 2025-05-14 14:38:49Z yeti-dn $
- *  Copyright (C) 2009-2024 David Necas (Yeti).
+ *  $Id: igorfile.c 28911 2025-11-24 18:27:42Z yeti-dn $
+ *  Copyright (C) 2009-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -48,18 +48,12 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <glib/gstdio.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/stats.h>
-#include <libprocess/arithmetic.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 
 #include "err.h"
 #include "get.h"
@@ -221,15 +215,19 @@ typedef struct {
 } PXPRecordHeader;
 
 static gboolean      module_register         (void);
-static gint          igor_detect             (const GwyFileDetectInfo *fileinfo,
+static gint          ibwfile_detect          (const GwyFileDetectInfo *fileinfo,
                                               gboolean only_name);
-static GwyContainer* igor_load               (const gchar *filename,
-                                              GwyRunType mode,
+static GwyFile*      ibwfile_load            (const gchar *filename,
+                                              GwyRunModeFlags mode,
+                                              GError **error);
+static gboolean      ibwfile_export          (GwyFile *data,
+                                              const gchar *filename,
+                                              GwyRunModeFlags mode,
                                               GError **error);
 static gboolean      igor_load_single        (const guchar *buffer,
                                               gsize size,
                                               const gchar *filename,
-                                              GwyContainer *container,
+                                              GwyFile *file,
                                               gint idgraph,
                                               gint idfield,
                                               gint idbrick,
@@ -240,25 +238,21 @@ static gboolean      igor_load_single        (const guchar *buffer,
 static void          igor_read_images        (IgorFile *igorfile,
                                               const gchar *buffer,
                                               const gchar *filename,
-                                              GwyContainer *container,
+                                              GwyFile *file,
                                               gint idfrom,
                                               gint *nfields);
 static void          igor_read_volumes       (IgorFile *igorfile,
                                               const gchar *buffer,
                                               const gchar *filename,
-                                              GwyContainer *container,
+                                              GwyFile *file,
                                               gint idfrom,
                                               gint *nbricks);
 static void          igor_read_curves        (IgorFile *igorfile,
                                               const gchar *buffer,
                                               const gchar *filename,
-                                              GwyContainer *container,
+                                              GwyFile *file,
                                               gint idfrom,
                                               gint *nfields);
-static gboolean      igor_export             (GwyContainer *data,
-                                              const gchar *filename,
-                                              GwyRunType mode,
-                                              GError **error);
 static guint         igor_read_headers       (IgorFile *igorfile,
                                               const guchar *buffer,
                                               gsize size,
@@ -271,7 +265,7 @@ static guint         igor_checksum           (gconstpointer buffer,
                                               gsize size,
                                               gboolean lsb);
 static guint         igor_data_type_size     (IgorDataType type);
-static GwyDataField* igor_read_data_field    (const IgorFile *igorfile,
+static GwyField*     igor_read_field         (const IgorFile *igorfile,
                                               const guchar *buffer,
                                               guint i,
                                               const gchar *zunits,
@@ -281,7 +275,7 @@ static GwyBrick*     igor_read_brick         (const IgorFile *igorfile,
                                               guint i,
                                               const gchar *wunits,
                                               gboolean is_imaginary);
-static GwyDataLine*  igor_read_data_line     (const IgorFile *igorfile,
+static GwyLine*      igor_read_line          (const IgorFile *igorfile,
                                               const guchar *buffer,
                                               guint i,
                                               const gchar *yunits,
@@ -293,26 +287,25 @@ static GPtrArray*    read_channel_labels     (const gchar *p,
                                               guint l);
 static gchar*        canonicalize_title      (const gchar *title);
 static const gchar*  channel_title_to_units  (const gchar *title);
-static gint*         find_compatible_channels(GwyContainer *container,
-                                              GwyDataField *dfield,
+static gint*         find_compatible_channels(GwyFile *file,
+                                              GwyField *dfield,
                                               guint *n);
-
-static gint          pxp_detect            (const GwyFileDetectInfo *fileinfo,
-                                            gboolean only_name);
-static GwyContainer* pxp_load              (const gchar *filename,
-                                            GwyRunType mode,
-                                            GError **error);
-static gboolean      pxp_read_record_header(PXPRecordHeader *header,
-                                            GwyByteOrder byteorder,
-                                            const guchar **p,
-                                            gsize size);
+static gint          pxpfile_detect          (const GwyFileDetectInfo *fileinfo,
+                                              gboolean only_name);
+static GwyFile*      pxpfile_load            (const gchar *filename,
+                                              GwyRunModeFlags mode,
+                                              GError **error);
+static gboolean      pxp_read_record_header  (PXPRecordHeader *header,
+                                              GwyByteOrder byteorder,
+                                              const guchar **p,
+                                              gsize size);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Igor binary waves (.ibw)."),
     "Yeti <yeti@gwyddion.net>",
-    "0.15",
+    "0.16",
     "David Nečas (Yeti)",
     "2009",
 };
@@ -324,23 +317,16 @@ module_register(void)
 {
     gwy_file_func_register("igorfile",
                            N_("Igor binary waves (.ibw)"),
-                           (GwyFileDetectFunc)&igor_detect,
-                           (GwyFileLoadFunc)&igor_load,
-                           NULL,
-                           (GwyFileSaveFunc)&igor_export);
+                           ibwfile_detect, ibwfile_load, NULL, ibwfile_export);
     gwy_file_func_register("pxpfile",
                            N_("Igor packed experiment files (.pxp)"),
-                           (GwyFileDetectFunc)&pxp_detect,
-                           (GwyFileLoadFunc)&pxp_load,
-                           NULL,
-                           NULL);
+                           pxpfile_detect, pxpfile_load, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-igor_detect(const GwyFileDetectInfo *fileinfo,
-            gboolean only_name)
+ibwfile_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     if (only_name)
         return g_str_has_suffix(fileinfo->name_lowercase, IGOR_EXTENSION) ? 10 : 0;
@@ -356,8 +342,7 @@ igor_detect(const GwyFileDetectInfo *fileinfo,
 }
 
 static gint
-pxp_detect(const GwyFileDetectInfo *fileinfo,
-           gboolean only_name)
+pxpfile_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     PXPRecordHeader header;
     GwyByteOrder byteorder;
@@ -386,12 +371,12 @@ pxp_detect(const GwyFileDetectInfo *fileinfo,
     return 0;
 }
 
-static GwyContainer*
-igor_load(const gchar *filename,
-          G_GNUC_UNUSED GwyRunType mode,
-          GError **error)
+static GwyFile*
+ibwfile_load(const gchar *filename,
+             G_GNUC_UNUSED GwyRunModeFlags mode,
+             GError **error)
 {
-    GwyContainer *container;
+    GwyFile *file;
     GError *err = NULL;
     guchar *buffer = NULL;
     gsize size;
@@ -402,29 +387,29 @@ igor_load(const gchar *filename,
         return NULL;
     }
 
-    container = gwy_container_new();
-    if (!igor_load_single(buffer, size, filename, container, 0, 0, 0, &ngraphs, &nfields, &nbricks, error)) {
-        GWY_OBJECT_UNREF(container);
+    file = gwy_file_new_in_construction();
+    if (!igor_load_single(buffer, size, filename, file, 0, 0, 0, &ngraphs, &nfields, &nbricks, error)) {
+        g_clear_object(&file);
     }
     else if (!ngraphs && !nfields && !nbricks) {
         err_NO_DATA(error);
-        GWY_OBJECT_UNREF(container);
+        g_clear_object(&file);
     }
     gwy_file_abandon_contents(buffer, size, NULL);
 
     if (!nbricks) {
         gwy_debug("not volume data, just returning the single file content");
-        return container;
+        return file;
     }
 
     gwy_debug("volume data, trying to merge with other files");
     /* TODO */
-    return container;
+    return file;
 }
 
 static gboolean
 igor_load_single(const guchar *buffer, gsize size, const gchar *filename,
-                 GwyContainer *container,
+                 GwyFile *file,
                  gint idgraph, gint idfield, gint idbrick,
                  gint *ngraphs, gint *nfields, gint *nbricks,
                  GError **error)
@@ -442,7 +427,7 @@ igor_load_single(const guchar *buffer, gsize size, const gchar *filename,
 
     *ngraphs = *nfields = *nbricks = 0;
 
-    gwy_clear(&igorfile, 1);
+    gwy_clear1(igorfile);
     if (!igor_read_headers(&igorfile, buffer, size, FALSE, error))
         goto fail;
 
@@ -544,7 +529,7 @@ igor_load_single(const guchar *buffer, gsize size, const gchar *filename,
         if (igorfile.variant == IGOR_BASE)
             g_warning("Trying to parse the note for apparently base producer variant as Asylum Research note.");
         note = g_strndup((const gchar*)p, size);
-        gwy_clear(&parser, 1);
+        gwy_clear1(parser);
         parser.key_value_separator = ":";
         igorfile.meta = gwy_text_header_parse(note, &parser, NULL, NULL);
     }
@@ -596,13 +581,13 @@ igor_load_single(const guchar *buffer, gsize size, const gchar *filename,
     }
 
     /* XXX: We also have igorfile.asylum_shape.  But only sometimes so it is not clear how to use it. */
-    /* NB: The caller expects us to not put anything into @container unless we have actual valid data. */
+    /* NB: The caller expects us to not put anything into @file unless we have actual valid data. */
     if (igorfile.data_shape == IGOR_DATA_IMAGE)
-        igor_read_images(&igorfile, buffer, filename, container, idfield, nfields);
+        igor_read_images(&igorfile, buffer, filename, file, idfield, nfields);
     else if (igorfile.data_shape == IGOR_DATA_CURVE)
-        igor_read_curves(&igorfile, buffer, filename, container, idgraph, ngraphs);
+        igor_read_curves(&igorfile, buffer, filename, file, idgraph, ngraphs);
     else if (igorfile.data_shape == IGOR_DATA_VOLUME)
-        igor_read_volumes(&igorfile, buffer, filename, container, idbrick, nbricks);
+        igor_read_volumes(&igorfile, buffer, filename, file, idbrick, nbricks);
     else {
         g_assert_not_reached();
     }
@@ -629,10 +614,10 @@ fail:
 
 static void
 igor_read_images(IgorFile *igorfile, const gchar *buffer, const gchar *filename,
-                 GwyContainer *container, gint idfrom, gint *nfields)
+                 GwyFile *file, gint idfrom, gint *nfields)
 {
     IgorWaveHeader5 *wave5 = &igorfile->wave5;
-    GwyDataField *dfield = NULL, *maskfield = NULL;
+    GwyField *dfield = NULL, *maskfield = NULL;
     GwyContainer *meta = NULL;
     gint i, chid;
 
@@ -645,49 +630,40 @@ igor_read_images(IgorFile *igorfile, const gchar *buffer, const gchar *filename,
             zunits = chinfo->units;
             meta = igor_get_metadata(igorfile, i + 1);
         }
-        dfield = igor_read_data_field(igorfile, buffer, i, zunits, FALSE);
+        dfield = igor_read_field(igorfile, buffer, i, zunits, FALSE);
         maskfield = gwy_app_channel_mask_of_nans(dfield, TRUE);
-        gwy_container_pass_object(container, gwy_app_get_data_key_for_id(chid), dfield);
+        gwy_file_pass_image(file, chid, dfield);
         if (maskfield)
-            gwy_container_set_object(container, gwy_app_get_mask_key_for_id(chid), maskfield);
+            gwy_file_set_image_mask(file, chid, maskfield);
         if (meta)
-            gwy_container_set_object(container, gwy_app_get_data_meta_key_for_id(chid), meta);
+            gwy_file_pass_meta(file, GWY_FILE_IMAGE, chid, meta);
 
         if (title)
-            gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(chid), title);
-        gwy_app_channel_title_fall_back(container, chid);
+            gwy_file_set_title(file, GWY_FILE_IMAGE, chid, title, FALSE);
+        gwy_image_title_fall_back(file, chid);
 
         if (wave5->type & IGOR_COMPLEX) {
             chid++;
-            dfield = igor_read_data_field(igorfile, buffer, i, zunits, TRUE);
-            gwy_container_pass_object(container, gwy_app_get_data_key_for_id(chid), dfield);
-            if (meta) {
-                /* container still holds a reference */
-                g_object_unref(meta);
-                meta = gwy_container_duplicate(meta);
-                gwy_container_set_object(container, gwy_app_get_data_meta_key_for_id(chid), meta);
-            }
-            if (maskfield) {
-                /* container still holds a reference */
-                g_object_unref(maskfield);
-                maskfield = gwy_data_field_duplicate(maskfield);
-                gwy_container_set_object(container, gwy_app_get_mask_key_for_id(chid), maskfield);
-            }
+            dfield = igor_read_field(igorfile, buffer, i, zunits, TRUE);
+            gwy_file_pass_image(file, chid, dfield);
+            if (meta)
+                gwy_file_pass_meta(file, GWY_FILE_IMAGE, chid, gwy_container_copy(meta));
+            if (maskfield)
+                gwy_file_pass_image_mask(file, chid, gwy_field_copy(maskfield));
 
             if (title)
-                gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(chid), title);
-            gwy_app_channel_title_fall_back(container, chid);
-            gwy_file_channel_import_log_add(container, chid, NULL, filename);
+                gwy_file_set_title(file, GWY_FILE_IMAGE, chid, title, FALSE);
+            gwy_image_title_fall_back(file, chid);
+            gwy_log_add_import(file, GWY_FILE_IMAGE, chid, NULL, filename);
         }
-        GWY_OBJECT_UNREF(meta);
-        GWY_OBJECT_UNREF(maskfield);
+        g_clear_object(&maskfield);
     }
     *nfields = chid - idfrom;
 }
 
 static void
 igor_read_volumes(IgorFile *igorfile, const gchar *buffer, const gchar *filename,
-                  GwyContainer *container, gint idfrom, gint *nbricks)
+                  GwyFile *file, gint idfrom, gint *nbricks)
 {
     IgorWaveHeader5 *wave5 = &igorfile->wave5;
     GwyBrick *brick;
@@ -704,41 +680,36 @@ igor_read_volumes(IgorFile *igorfile, const gchar *buffer, const gchar *filename
             meta = igor_get_metadata(igorfile, i + 1);
         }
         brick = igor_read_brick(igorfile, buffer, i, wunits, FALSE);
-        gwy_container_pass_object(container, gwy_app_get_brick_key_for_id(chid), brick);
+        gwy_file_pass_volume(file, chid, brick);
         if (meta)
-            gwy_container_set_object(container, gwy_app_get_brick_meta_key_for_id(chid), meta);
+            gwy_file_pass_meta(file, GWY_FILE_VOLUME, chid, meta);
 
         if (title)
-            gwy_container_set_const_string(container, gwy_app_get_brick_title_key_for_id(chid), title);
-        //gwy_app_channel_title_fall_back(container, chid);
+            gwy_file_set_title(file, GWY_FILE_VOLUME, chid, title, FALSE);
+        //gwy_image_title_fall_back(file, chid);
 
         if (wave5->type & IGOR_COMPLEX) {
             chid++;
             brick = igor_read_brick(igorfile, buffer, i, wunits, TRUE);
-            gwy_container_pass_object(container, gwy_app_get_brick_key_for_id(chid), brick);
-            if (meta) {
-                /* container still holds a reference */
-                g_object_unref(meta);
-                meta = gwy_container_duplicate(meta);
-                gwy_container_set_object(container, gwy_app_get_brick_meta_key_for_id(chid), meta);
-            }
+            gwy_file_pass_volume(file, chid, brick);
+            if (meta)
+                gwy_file_pass_meta(file, GWY_FILE_VOLUME, chid, gwy_container_copy(meta));
 
             if (title)
-                gwy_container_set_const_string(container, gwy_app_get_brick_title_key_for_id(chid), title);
-            //gwy_app_channel_title_fall_back(container, chid);
-            gwy_file_volume_import_log_add(container, chid, NULL, filename);
+                gwy_file_set_title(file, GWY_FILE_VOLUME, chid, title, FALSE);
+            //gwy_image_title_fall_back(file, chid);
+            gwy_log_add_import(file, GWY_FILE_VOLUME, chid, NULL, filename);
         }
-        GWY_OBJECT_UNREF(meta);
     }
     *nbricks = chid - idfrom;
 }
 
 static void
-igor_read_curves(IgorFile *igorfile, const gchar *buffer, G_GNUC_UNUSED const gchar *filename,
-                 GwyContainer *container, gint idfrom, gint *ngraphs)
+igor_read_curves(IgorFile *igorfile, const gchar *buffer, const gchar *filename,
+                 GwyFile *file, gint idfrom, gint *ngraphs)
 {
     IgorWaveHeader5 *wave5 = &igorfile->wave5;
-    GwyDataLine *dline;
+    GwyLine *dline;
     GwyGraphModel *gmodel;
     GwyGraphCurveModel *gcmodel;
     gint i, chid;
@@ -749,12 +720,12 @@ igor_read_curves(IgorFile *igorfile, const gchar *buffer, G_GNUC_UNUSED const gc
 
         /* XXX: For images we use channel info gathered from the note.  For graphs the units seem to be either stored
          * normally in wave5 or who knows where. */
-        dline = igor_read_data_line(igorfile, buffer, i, NULL, FALSE);
+        dline = igor_read_line(igorfile, buffer, i, NULL, FALSE);
         gmodel = gwy_graph_model_new();
-        gwy_graph_model_set_units_from_data_line(gmodel, dline);
+        gwy_graph_model_set_units_from_line(gmodel, dline);
         g_object_set(gmodel, "title", title, NULL);
         gcmodel = gwy_graph_curve_model_new();
-        gwy_graph_curve_model_set_data_from_dataline(gcmodel, dline, 0, 0);
+        gwy_graph_curve_model_set_data_from_line(gcmodel, dline, 0, 0);
         g_object_unref(dline);
         g_object_set(gcmodel,
                      "mode", GWY_GRAPH_CURVE_LINE,
@@ -771,9 +742,9 @@ igor_read_curves(IgorFile *igorfile, const gchar *buffer, G_GNUC_UNUSED const gc
         g_object_unref(gcmodel);
 
         if (wave5->type & IGOR_COMPLEX) {
-            dline = igor_read_data_line(igorfile, buffer, i, NULL, TRUE);
+            dline = igor_read_line(igorfile, buffer, i, NULL, TRUE);
             gcmodel = gwy_graph_curve_model_new();
-            gwy_graph_curve_model_set_data_from_dataline(gcmodel, dline, 0, 0);
+            gwy_graph_curve_model_set_data_from_line(gcmodel, dline, 0, 0);
             g_object_unref(dline);
             s = g_strconcat(title, " ", "(Im)", NULL);
             g_object_set(gcmodel,
@@ -786,7 +757,8 @@ igor_read_curves(IgorFile *igorfile, const gchar *buffer, G_GNUC_UNUSED const gc
             g_object_unref(gcmodel);
         }
 
-        gwy_container_pass_object(container, gwy_app_get_graph_key_for_id(chid + 1), gmodel);
+        gwy_file_pass_graph(file, chid, gmodel);
+        gwy_log_add_import(file, GWY_FILE_GRAPH, chid, NULL, filename);
     }
 
     *ngraphs = chid - idfrom;
@@ -1054,18 +1026,18 @@ igor_data_type_size(IgorDataType type)
     return gwy_raw_data_size(rawtype)*((type & IGOR_COMPLEX) ? 2 : 1);
 }
 
-static GwyDataField*
-igor_read_data_field(const IgorFile *igorfile,
-                     const guchar *buffer,
-                     guint i,
-                     const gchar *zunits,
-                     gboolean is_imaginary)
+static GwyField*
+igor_read_field(const IgorFile *igorfile,
+                const guchar *buffer,
+                guint i,
+                const gchar *zunits,
+                gboolean is_imaginary)
 {
     const IgorWaveHeader5 *wave5;
     GwyRawDataType rawtype;
     GwyByteOrder byteorder;
     guint n, xres, yres;
-    GwyDataField *dfield;
+    GwyField *dfield;
     gdouble *data;
     const guchar *p;
     gint power10, stride;
@@ -1077,8 +1049,8 @@ igor_read_data_field(const IgorFile *igorfile,
     n = xres*yres;
     p = buffer + igorfile->headers_size + n*igorfile->type_size*i;
 
-    dfield = gwy_data_field_new(xres, yres, wave5->sfA[0]*xres, wave5->sfA[1]*yres, FALSE);
-    data = gwy_data_field_get_data(dfield);
+    dfield = gwy_field_new(xres, yres, wave5->sfA[0]*xres, wave5->sfA[1]*yres, FALSE);
+    data = gwy_field_get_data(dfield);
 
     g_return_val_if_fail(!is_imaginary || (wave5->type & IGOR_COMPLEX), dfield);
     if (is_imaginary) {
@@ -1093,16 +1065,15 @@ igor_read_data_field(const IgorFile *igorfile,
     byteorder = (igorfile->lsb ? GWY_BYTE_ORDER_LITTLE_ENDIAN : GWY_BYTE_ORDER_BIG_ENDIAN);
 
     /* TODO: Support extended units */
-    gwy_si_unit_set_from_string_parse(gwy_data_field_get_si_unit_xy(dfield), wave5->dim_units[0], &power10);
-    gwy_data_field_set_xreal(dfield, pow10(power10)*wave5->sfA[0]*xres);
-    gwy_data_field_set_yreal(dfield, pow10(power10)*wave5->sfA[1]*yres);
+    power10 = gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), wave5->dim_units[0]);
+    gwy_field_set_xreal(dfield, gwy_exp10(power10)*wave5->sfA[0]*xres);
+    gwy_field_set_yreal(dfield, gwy_exp10(power10)*wave5->sfA[1]*yres);
 
-    gwy_si_unit_set_from_string_parse(gwy_data_field_get_si_unit_z(dfield),
-                                      zunits ? zunits : wave5->data_units, &power10);
-    q = pow10(power10);
+    power10 = gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), zunits ? zunits : wave5->data_units);
+    q = gwy_exp10(power10);
 
     gwy_convert_raw_data(p, n, stride, rawtype, byteorder, data, q, 0.0);
-    gwy_data_field_invert(dfield, TRUE, FALSE, FALSE);
+    gwy_field_flip(dfield, FALSE, TRUE);
 
     return dfield;
 }
@@ -1149,35 +1120,34 @@ igor_read_brick(const IgorFile *igorfile,
     byteorder = (igorfile->lsb ? GWY_BYTE_ORDER_LITTLE_ENDIAN : GWY_BYTE_ORDER_BIG_ENDIAN);
 
     /* TODO: Support extended units */
-    gwy_si_unit_set_from_string_parse(gwy_brick_get_si_unit_x(brick), wave5->dim_units[0], &power10);
-    gwy_brick_set_xreal(brick, pow10(power10)*wave5->sfA[0]*xres);
-    gwy_si_unit_set_from_string_parse(gwy_brick_get_si_unit_y(brick), wave5->dim_units[1], &power10);
-    gwy_brick_set_yreal(brick, pow10(power10)*wave5->sfA[1]*yres);
-    gwy_si_unit_set_from_string_parse(gwy_brick_get_si_unit_z(brick), wave5->dim_units[2], &power10);
-    gwy_brick_set_zreal(brick, pow10(power10)*wave5->sfA[2]*zres);
+    power10 = gwy_unit_set_from_string(gwy_brick_get_unit_x(brick), wave5->dim_units[0]);
+    gwy_brick_set_xreal(brick, gwy_exp10(power10)*wave5->sfA[0]*xres);
+    power10 = gwy_unit_set_from_string(gwy_brick_get_unit_y(brick), wave5->dim_units[1]);
+    gwy_brick_set_yreal(brick, gwy_exp10(power10)*wave5->sfA[1]*yres);
+    power10 = gwy_unit_set_from_string(gwy_brick_get_unit_z(brick), wave5->dim_units[2]);
+    gwy_brick_set_zreal(brick, gwy_exp10(power10)*wave5->sfA[2]*zres);
 
-    gwy_si_unit_set_from_string_parse(gwy_brick_get_si_unit_w(brick),
-                                      wunits ? wunits : wave5->data_units, &power10);
-    q = pow10(power10);
+    power10 = gwy_unit_set_from_string(gwy_brick_get_unit_w(brick), wunits ? wunits : wave5->data_units);
+    q = gwy_exp10(power10);
 
     gwy_convert_raw_data(p, n, stride, rawtype, byteorder, data, q, 0.0);
-    //gwy_brick_invert(brick, TRUE, FALSE, FALSE);
+    //gwy_brick_flip(brick, TRUE, FALSE, FALSE);
 
     return brick;
 }
 
-static GwyDataLine*
-igor_read_data_line(const IgorFile *igorfile,
-                    const guchar *buffer,
-                    guint i,
-                    const gchar *yunits,
-                    gboolean is_imaginary)
+static GwyLine*
+igor_read_line(const IgorFile *igorfile,
+               const guchar *buffer,
+               guint i,
+               const gchar *yunits,
+               gboolean is_imaginary)
 {
     const IgorWaveHeader5 *wave5;
     GwyRawDataType rawtype;
     GwyByteOrder byteorder;
     guint n, res;
-    GwyDataLine *dline;
+    GwyLine *dline;
     gdouble *data;
     const guchar *p;
     gint power10, stride;
@@ -1188,8 +1158,8 @@ igor_read_data_line(const IgorFile *igorfile,
     n = res;
     p = buffer + igorfile->headers_size + n*igorfile->type_size*i;
 
-    dline = gwy_data_line_new(res, wave5->sfA[0]*res, FALSE);
-    data = gwy_data_line_get_data(dline);
+    dline = gwy_line_new(res, wave5->sfA[0]*res, FALSE);
+    data = gwy_line_get_data(dline);
 
     g_return_val_if_fail(!is_imaginary || (wave5->type & IGOR_COMPLEX), dline);
     if (is_imaginary) {
@@ -1204,15 +1174,14 @@ igor_read_data_line(const IgorFile *igorfile,
     byteorder = (igorfile->lsb ? GWY_BYTE_ORDER_LITTLE_ENDIAN : GWY_BYTE_ORDER_BIG_ENDIAN);
 
     /* TODO: Support extended units */
-    gwy_si_unit_set_from_string_parse(gwy_data_line_get_si_unit_x(dline), wave5->dim_units[0], &power10);
-    gwy_data_line_set_real(dline, pow10(power10)*wave5->sfA[0]*res);
+    power10 = gwy_unit_set_from_string(gwy_line_get_unit_x(dline), wave5->dim_units[0]);
+    gwy_line_set_real(dline, gwy_exp10(power10)*wave5->sfA[0]*res);
 
-    gwy_si_unit_set_from_string_parse(gwy_data_line_get_si_unit_y(dline),
-                                      yunits ? yunits : wave5->data_units, &power10);
-    q = pow10(power10);
+    power10 = gwy_unit_set_from_string(gwy_line_get_unit_y(dline), yunits ? yunits : wave5->data_units);
+    q = gwy_exp10(power10);
 
     gwy_convert_raw_data(p, n, stride, rawtype, byteorder, data, q, 0.0);
-    gwy_data_line_invert(dline, TRUE, FALSE);
+    gwy_line_flip(dline);
 
     return dline;
 }
@@ -1303,7 +1272,7 @@ gather_channel_meta(gpointer hkey,
                     gpointer user_data)
 {
     const gchar *key = (const gchar*)hkey;
-    const gchar *value = (const gchar*)hvalue;
+    gchar *value = (gchar*)hvalue;
     IgorFile *igorfile = (IgorFile*)user_data;
     guint i;
 
@@ -1341,7 +1310,7 @@ igor_get_metadata(IgorFile *igorfile,
     if (!igorfile->ignore_prefixes)
         igorfile->ignore_prefixes = ignore_prefixes;
 
-    igorfile->channelmeta = gwy_container_new();
+    igorfile->channelmeta = gwy_container_new_in_construction();
     g_hash_table_foreach(igorfile->meta, gather_channel_meta, igorfile);
 
     return igorfile->channelmeta;
@@ -1414,14 +1383,14 @@ append_string(GByteArray *content, const gchar *str, guint maxlen)
 }
 
 static gboolean
-igor_export(GwyContainer *data,
-            const gchar *filename,
-            G_GNUC_UNUSED GwyRunType mode,
-            GError **error)
+ibwfile_export(GwyFile *data,
+               const gchar *filename,
+               G_GNUC_UNUSED GwyRunModeFlags mode,
+               GError **error)
 {
     GByteArray *header = NULL;
-    GwyDataField *dfield;
-    GwySIUnit *xyunit, *zunit;
+    GwyField *dfield;
+    GwyUnit *xyunit, *zunit;
     const gdouble *d, *drow;
     gfloat *dfl = NULL, *frow;
     gint *ids = NULL;
@@ -1434,7 +1403,7 @@ igor_export(GwyContainer *data,
     gchar *title, *title_latin1, *unitstr;
     gchar bname[MAX_WAVE_NAME5+1];
 
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &dfield, 0);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &dfield, 0);
     if (!dfield) {
         err_NO_CHANNEL_EXPORT(error);
         return FALSE;
@@ -1449,12 +1418,12 @@ igor_export(GwyContainer *data,
         return FALSE;
     }
 
-    xres = gwy_data_field_get_xres(dfield);
-    yres = gwy_data_field_get_yres(dfield);
-    xreal = gwy_data_field_get_xreal(dfield);
-    yreal = gwy_data_field_get_yreal(dfield);
-    xyunit = gwy_data_field_get_si_unit_xy(dfield);
-    zunit = gwy_data_field_get_si_unit_z(dfield);
+    xres = gwy_field_get_xres(dfield);
+    yres = gwy_field_get_yres(dfield);
+    xreal = gwy_field_get_xreal(dfield);
+    yreal = gwy_field_get_yreal(dfield);
+    xyunit = gwy_field_get_unit_xy(dfield);
+    zunit = gwy_field_get_unit_z(dfield);
 
     n = xres*yres;
     npts = nchannels*n;
@@ -1514,11 +1483,11 @@ igor_export(GwyContainer *data,
     append_double(header, 0.0); /* sfB[3] */
 
     /* natural data units */
-    unitstr = gwy_si_unit_get_string(zunit, GWY_SI_UNIT_FORMAT_PLAIN);
+    unitstr = gwy_unit_get_string(zunit, GWY_UNIT_FORMAT_PLAIN);
     append_string(header, strlen(unitstr) == 1 ? unitstr : NULL, MAX_UNIT_CHARS);
 
     /* dimension units */
-    unitstr = gwy_si_unit_get_string(xyunit, GWY_SI_UNIT_FORMAT_PLAIN);
+    unitstr = gwy_unit_get_string(xyunit, GWY_UNIT_FORMAT_PLAIN);
     append_string(header, strlen(unitstr) == 1 ? unitstr : NULL, MAX_UNIT_CHARS);
     append_string(header, strlen(unitstr) == 1 ? unitstr : NULL, MAX_UNIT_CHARS);
     g_free(unitstr);
@@ -1547,8 +1516,8 @@ igor_export(GwyContainer *data,
 
     dfl = g_new(gfloat, n);
     for (k = 0; ids[k] >= 0; k++) {
-        dfield = gwy_container_get_object(data, gwy_app_get_data_key_for_id(ids[k]));
-        d = gwy_data_field_get_data_const(dfield);
+        dfield = gwy_file_get_image(data, ids[k]);
+        d = gwy_field_get_data_const(dfield);
         for (i = 0; i < yres; i++) {
             frow = dfl + i*xres;
             drow = d + (yres-1 - i)*xres;
@@ -1579,7 +1548,7 @@ igor_export(GwyContainer *data,
      * start from the next one. */
     append_string(header, "", MAX_WAVE_NAME5);
     for (k = 0; ids[k] >= 0; k++) {
-        title = gwy_app_get_data_field_title(data, ids[k]);
+        title = gwy_file_get_display_title(data, GWY_FILE_IMAGE, ids[k]);
         title_latin1 = g_convert(title, -1, "ISO-8859-1", "UTF-8", NULL, NULL, NULL);
         append_string(header, title_latin1, MAX_WAVE_NAME5);
         g_free(title_latin1);
@@ -1606,37 +1575,33 @@ fail:
 }
 
 static gint*
-find_compatible_channels(GwyContainer *container,
-                         GwyDataField *dfield,
-                         guint *n)
+find_compatible_channels(GwyFile *file,
+                         GwyField *dfield,
+                         guint *pn)
 {
-    GwyDataField *otherfield;
-    GQuark quark;
-    guint i, j;
-    gint *ids;
+    gint *ids = gwy_file_get_ids(file, GWY_FILE_IMAGE);
+    guint n = 0;
 
-    ids = gwy_app_data_browser_get_data_ids(container);
-    for (i = j = 0; ids[i] >= 0; i++) {
-        quark = gwy_app_get_data_key_for_id(ids[i]);
-        otherfield = gwy_container_get_object(container, quark);
-        if (gwy_data_field_check_compatibility(dfield, otherfield, GWY_DATA_COMPATIBILITY_ALL) != 0)
+    for (guint i = 0; ids[i] >= 0; i++) {
+        GwyField *otherfield = gwy_file_get_image(file, ids[i]);
+        if (gwy_field_is_incompatible(dfield, otherfield, GWY_DATA_MISMATCH_ALL) != 0)
             continue;
 
-        ids[j++] = ids[i];
+        ids[n++] = ids[i];
     }
-    g_assert(j > 0);
-    *n = j;
-    ids[j] = -1;
+    g_assert(n > 0);
+    *pn = n;
+    ids[n] = -1;
 
     return ids;
 }
 
-static GwyContainer*
-pxp_load(const gchar *filename,
-         G_GNUC_UNUSED GwyRunType mode,
-         GError **error)
+static GwyFile*
+pxpfile_load(const gchar *filename,
+             G_GNUC_UNUSED GwyRunModeFlags mode,
+             GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     PXPRecordHeader header;
     GwyByteOrder byteorder;
     GError *err = NULL;
@@ -1674,16 +1639,16 @@ pxp_load(const gchar *filename,
 
     /* Knowing the byte order, start again from scratch. */
     p = buffer;
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     while (p < buffer + size) {
         if (!pxp_read_record_header(&header, byteorder, &p, size - (p - buffer))) {
-            GWY_OBJECT_UNREF(container);
+            g_clear_object(&file);
             err_FILE_TYPE(error, "PXP");
             goto fail;
         }
         gwy_debug("record type %u, version %d, size %u", header.type, header.version, header.size);
         if (header.size > size - (p - buffer)) {
-            GWY_OBJECT_UNREF(container);
+            g_clear_object(&file);
             err_TRUNCATED_PART(error, "record");
             goto fail;
         }
@@ -1709,7 +1674,7 @@ pxp_load(const gchar *filename,
          * distinguish their versions and then read them differently – and I don't see those versions. So just
          * ignore them for now. */
         if (header.type == PXP_WAVE) {
-            if (igor_load_single(p, header.size, filename, container,
+            if (igor_load_single(p, header.size, filename, file,
                                  idgraph, idfield, idbrick, &ngraphs, &nbricks, &nfields, NULL)) {
                 idgraph += ngraphs;
                 idfield += nfields;
@@ -1721,13 +1686,13 @@ pxp_load(const gchar *filename,
     }
 
     if (!idgraph && !idfield && !idbrick) {
-        GWY_OBJECT_UNREF(container);
+        g_clear_object(&file);
         err_NO_DATA(error);
     }
 
 fail:
     gwy_file_abandon_contents(buffer, size, NULL);
-    return container;
+    return file;
 }
 
 
