@@ -1,6 +1,6 @@
 /*
- *  $Id: intelliwave.c 25688 2023-09-20 11:08:15Z yeti-dn $
- *  Copyright (C) 2016 David Necas (Yeti).
+ *  $Id: intelliwave.c 28789 2025-11-04 17:14:03Z yeti-dn $
+ *  Copyright (C) 2016-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -41,14 +41,9 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/datafield.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 
 #include "err.h"
 #include "get.h"
@@ -197,35 +192,35 @@ typedef struct {
     IntWaveFileInfo info;
 } IntWaveFile;
 
-static gboolean      module_register        (void);
-static gint          intw_detect            (const GwyFileDetectInfo *fileinfo,
-                                             gboolean only_name);
-static GwyContainer* intw_load              (const gchar *filename,
-                                             GwyRunType mode,
-                                             GError **error);
-static const guchar* intw_read_header       (const guchar *p,
-                                             IntWaveFile *intwfile);
-static const guchar* intw_read_info         (const guchar *p,
-                                             IntWaveFile *intwfile);
-static guint         check_data_block       (const IntWaveFile *intwfile,
-                                             guint i,
-                                             guint size,
-                                             guint minbpp);
-static void          read_data_field_opd    (GwyContainer *container,
-                                             gint *id,
-                                             const IntWaveFile *intwfile,
-                                             const guchar *p);
-static void          read_data_field_fringes(GwyContainer *container,
-                                             gint *id,
-                                             const IntWaveFile *intwfile,
-                                             const guchar *p);
+static gboolean      module_register   (void);
+static gint          detect_file       (const GwyFileDetectInfo *fileinfo,
+                                        gboolean only_name);
+static GwyFile*      load_file         (const gchar *filename,
+                                        GwyRunModeFlags mode,
+                                        GError **error);
+static const guchar* read_header       (const guchar *p,
+                                        IntWaveFile *intwfile);
+static const guchar* read_info         (const guchar *p,
+                                        IntWaveFile *intwfile);
+static guint         check_data_block  (const IntWaveFile *intwfile,
+                                        guint i,
+                                        guint size,
+                                        guint minbpp);
+static void          read_field_opd    (GwyFile *file,
+                                        gint *id,
+                                        const IntWaveFile *intwfile,
+                                        const guchar *p);
+static void          read_field_fringes(GwyFile *file,
+                                        gint *id,
+                                        const IntWaveFile *intwfile,
+                                        const guchar *p);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports IntelliWave interferometric ESD data."),
     "Yeti <yeti@gwyddion.net>",
-    "0.1",
+    "0.2",
     "David Nečas (Yeti)",
     "2016",
 };
@@ -237,17 +232,13 @@ module_register(void)
 {
     gwy_file_func_register("intelliwave",
                            N_("IntelliWave interferometric data (.esd)"),
-                           (GwyFileDetectFunc)&intw_detect,
-                           (GwyFileLoadFunc)&intw_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-intw_detect(const GwyFileDetectInfo *fileinfo,
-            gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     if (only_name)
         return g_str_has_suffix(fileinfo->name_lowercase, EXTENSION) ? 15 : 0;
@@ -258,12 +249,12 @@ intw_detect(const GwyFileDetectInfo *fileinfo,
     return 0;
 }
 
-static GwyContainer*
-intw_load(const gchar *filename,
-          G_GNUC_UNUSED GwyRunType mode,
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
           GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     guchar *buffer = NULL;
     const guchar *p;
     gsize size = 0;
@@ -285,9 +276,9 @@ intw_load(const gchar *filename,
 
     intwfile.filename = filename;
     p = buffer;
-    p = intw_read_header(p, &intwfile);
+    p = read_header(p, &intwfile);
     g_assert(p - buffer == HEADER_SIZE);
-    p = intw_read_info(p, &intwfile);
+    p = read_info(p, &intwfile);
     g_assert(p - buffer == HEADER_SIZE + INFO_SIZE);
 
     for (i = 0; i < N_IS_VALID; i++) {
@@ -309,27 +300,29 @@ intw_load(const gchar *filename,
         goto fail;
     }
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
 
     i = INTWAVE_DATA_OPD;
     if ((offset = check_data_block(&intwfile, i, size, sizeof(gfloat))))
-        read_data_field_opd(container, &id, &intwfile, buffer + offset);
+        read_field_opd(file, &id, &intwfile, buffer + offset);
 
     i = INTWAVE_DATA_FRINGE;
     if ((offset = check_data_block(&intwfile, i, size, N_INTERFEROGRAMS*sizeof(gint16))))
-        read_data_field_fringes(container, &id, &intwfile, buffer + offset);
+        read_field_fringes(file, &id, &intwfile, buffer + offset);
 
-    if (!gwy_container_get_n_items(container))
+    if (!gwy_container_get_n_items(GWY_CONTAINER(file))) {
+        g_clear_object(&file);
         err_NO_DATA(error);
+    }
 
 fail:
     gwy_file_abandon_contents(buffer, size, NULL);
-    return container;
+    return file;
 }
 
 static const guchar*
-intw_read_header(const guchar *p,
-                 IntWaveFile *intwfile)
+read_header(const guchar *p,
+            IntWaveFile *intwfile)
 {
     IntWaveFileHeader *header = &intwfile->header;
     guint i;
@@ -353,8 +346,8 @@ intw_read_header(const guchar *p,
 }
 
 static const guchar*
-intw_read_info(const guchar *p,
-               IntWaveFile *intwfile)
+read_info(const guchar *p,
+          IntWaveFile *intwfile)
 {
     IntWaveFileInfo *info = &intwfile->info;
     guint i;
@@ -459,23 +452,23 @@ check_data_block(const IntWaveFile *intwfile, guint i,
 }
 
 static void
-read_data_field_opd(GwyContainer *container, gint *id,
-                    const IntWaveFile *intwfile, const guchar *p)
+read_field_opd(GwyFile *file, gint *id,
+               const IntWaveFile *intwfile, const guchar *p)
 {
     guint xres = intwfile->info.nx;
     guint yres = intwfile->info.ny;
     guint n = intwfile->info.nz;
     guint k;
-    GwyDataField *dfield, *mask;
+    GwyField *dfield, *mask;
     gdouble *d, *m;
     gdouble q = 1e-6 * intwfile->info.wavelength, invalid = intwfile->info.invalid_value;
 
-    dfield = gwy_data_field_new(xres, yres, xres, yres, FALSE);
-    d = gwy_data_field_get_data(dfield);
+    dfield = gwy_field_new(xres, yres, xres, yres, FALSE);
+    d = gwy_field_get_data(dfield);
     gwy_convert_raw_data(p, n, 1, GWY_RAW_DATA_FLOAT, GWY_BYTE_ORDER_LITTLE_ENDIAN, d, 1.0, 0.0);
 
-    mask = gwy_data_field_new_alike(dfield, TRUE);
-    m = gwy_data_field_get_data(mask);
+    mask = gwy_field_new_alike(dfield, TRUE);
+    m = gwy_field_get_data(mask);
     for (k = 0; k < n; k++) {
         if (d[k] != invalid) {
             m[k] = 1.0;
@@ -483,54 +476,53 @@ read_data_field_opd(GwyContainer *container, gint *id,
         }
     }
     if (!gwy_app_channel_remove_bad_data(dfield, mask))
-        GWY_OBJECT_UNREF(mask);
+        g_clear_object(&mask);
 
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), "m");
-    gwy_container_pass_object(container, gwy_app_get_data_key_for_id(*id), dfield);
-    gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(*id), "OPD");
+    gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), "m");
+    gwy_file_pass_image(file, *id, dfield);
+    gwy_file_set_title(file, GWY_FILE_IMAGE, *id, "OPD", FALSE);
     if (mask)
-        gwy_container_pass_object(container, gwy_app_get_mask_key_for_id(*id), mask);
+        gwy_file_pass_image_mask(file, *id, mask);
 
-    gwy_file_channel_import_log_add(container, *id, NULL, intwfile->filename);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, *id, NULL, intwfile->filename);
     (*id)++;
 }
 
 static void
-read_data_field_fringes(GwyContainer *container,
-                        gint *id,
-                        const IntWaveFile *intwfile,
-                        const guchar *p)
+read_field_fringes(GwyFile *file,
+                   gint *id,
+                   const IntWaveFile *intwfile,
+                   const guchar *p)
 {
     guint xres = intwfile->info.nx;
     guint yres = intwfile->info.ny;
     guint n = intwfile->info.nz;
     guint i, k;
-    GwyDataField *dfield, *mask;
+    GwyField *dfield, *mask;
     gdouble *d, *m;
     gdouble invalid = intwfile->info.invalid_value;
 
     for (i = 0; i < N_INTERFEROGRAMS; i++) {
-        dfield = gwy_data_field_new(xres, yres, xres, yres, FALSE);
-        d = gwy_data_field_get_data(dfield);
+        dfield = gwy_field_new(xres, yres, xres, yres, FALSE);
+        d = gwy_field_get_data(dfield);
         gwy_convert_raw_data(p + i*n*sizeof(gint16), n, 1, GWY_RAW_DATA_SINT16, GWY_BYTE_ORDER_LITTLE_ENDIAN,
                              d, 1.0, 0.0);
 
-        mask = gwy_data_field_new_alike(dfield, TRUE);
-        m = gwy_data_field_get_data(mask);
+        mask = gwy_field_new_alike(dfield, TRUE);
+        m = gwy_field_get_data(mask);
         for (k = 0; k < n; k++) {
             if (d[k] != invalid)
                 m[k] = 1.0;
         }
         if (!gwy_app_channel_remove_bad_data(dfield, mask))
-            GWY_OBJECT_UNREF(mask);
+            g_clear_object(&mask);
 
-        gwy_container_pass_object(container, gwy_app_get_data_key_for_id(*id), dfield);
-        gwy_container_set_string(container, gwy_app_get_data_title_key_for_id(*id),
-                                 g_strdup_printf("Interferogram %u", i+1));
+        gwy_file_pass_image(file, *id, dfield);
+        gwy_file_pass_title(file, GWY_FILE_IMAGE, *id, g_strdup_printf("Interferogram %u", i+1));
         if (mask)
-            gwy_container_pass_object(container, gwy_app_get_mask_key_for_id(*id), mask);
+            gwy_file_pass_image_mask(file, *id, mask);
 
-        gwy_file_channel_import_log_add(container, *id, NULL, intwfile->filename);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, *id, NULL, intwfile->filename);
         (*id)++;
     }
 }
