@@ -1,5 +1,5 @@
 /*
- *  $Id: intematix.c 26163 2024-02-06 16:56:55Z yeti-dn $
+ *  $Id: intematix.c 28805 2025-11-05 12:28:20Z yeti-dn $
  *  Copyright (C) 2007 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -37,14 +37,9 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/datafield.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 
 #include "err.h"
 #include "gwytiff.h"
@@ -98,25 +93,25 @@ typedef struct {
     gchar *dataunit;
 } ISDFImage;
 
-static gboolean      module_register       (void);
-static gint          isdf_detect           (const GwyFileDetectInfo *fileinfo,
-                                            gboolean only_name);
-static GwyContainer* isdf_load             (const gchar *filename,
-                                            GwyRunType mode,
-                                            GError **error);
-static gboolean      isdf_image_fill_info  (ISDFImage *image,
-                                            const GwyTIFF *tiff,
-                                            GError **error);
-static GwyContainer* isdf_get_metadata     (const GwyTIFF *tiff,
-                                            const ISDFImage *image);
-static void          isdf_image_free       (ISDFImage *image);
+static gboolean      module_register     (void);
+static gint          detect_file         (const GwyFileDetectInfo *fileinfo,
+                                          gboolean only_name);
+static GwyFile*      load_file           (const gchar *filename,
+                                          GwyRunModeFlags mode,
+                                          GError **error);
+static gboolean      isdf_image_fill_info(ISDFImage *image,
+                                          const GwyTIFF *tiff,
+                                          GError **error);
+static GwyContainer* isdf_get_metadata   (const GwyTIFF *tiff,
+                                          const ISDFImage *image);
+static void          isdf_image_free     (ISDFImage *image);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     module_register,
     N_("Imports Intematix SDF data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.4",
+    "0.5",
     "David Nečas (Yeti) & Petr Klapetek",
     "2006",
 };
@@ -128,16 +123,13 @@ module_register(void)
 {
     gwy_file_func_register("intematix",
                            N_("Intematix SDF data files (.sdf)"),
-                           (GwyFileDetectFunc)&isdf_detect,
-                           (GwyFileLoadFunc)&isdf_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-isdf_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     GwyTIFF *tiff;
     gint score = 0;
@@ -152,8 +144,8 @@ isdf_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
         return 0;
 
     if ((tiff = gwy_tiff_load(fileinfo->name, NULL))
-         && gwy_tiff_get_sint0(tiff, ISDF_TIFFTAG_FILEID, &magic)
-         && magic == ISDF_MAGIC_NUMBER)
+        && gwy_tiff_get_sint0(tiff, ISDF_TIFFTAG_FILEID, &magic)
+        && magic == ISDF_MAGIC_NUMBER)
         score = 100;
 
     if (tiff)
@@ -162,26 +154,27 @@ isdf_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
     return score;
 }
 
-static GwyContainer*
-isdf_load(const gchar *filename,
-          G_GNUC_UNUSED GwyRunType mode,
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
           GError **error)
 {
-    GwyContainer *meta, *container = NULL;
-    GwyDataField *dfield;
-    GwySIUnit *siunitx, *siunity, *siunitz;
+    GwyFile *file = NULL;
+    GwyContainer *meta;
+    GwyField *dfield;
+    GwyUnit *siunitx, *siunity, *siunitz;
     GwyTIFF *tiff;
     ISDFImage image;
     GwyRawDataType rawtype;
     guint magic, t;
     gint power10x, power10y, power10z;
-    const guchar *p;
+    const gchar *p;
     gdouble q;
 
     if (!(tiff = gwy_tiff_load(filename, error)))
         return NULL;
 
-    gwy_clear(&image, 1);
+    gwy_clear1(image);
     if (!gwy_tiff_get_sint0(tiff, ISDF_TIFFTAG_FILEID, &magic) || magic != ISDF_MAGIC_NUMBER) {
         err_FILE_TYPE(error, "Intematix SDF");
         goto fail;
@@ -202,23 +195,23 @@ isdf_load(const gchar *filename,
     if (err_SIZE_MISMATCH(error, t*image.xres*image.yres, t*image.raw_data_len, TRUE))
         goto fail;
 
-    siunitx = gwy_si_unit_new_parse(image.xunit, &power10x);
-    siunity = gwy_si_unit_new_parse(image.yunit, &power10y);
-    siunitz = gwy_si_unit_new_parse(image.dataunit, &power10z);
-    if (!gwy_si_unit_equal(siunitx, siunity))
+    siunitx = gwy_unit_new_parse(image.xunit, &power10x);
+    siunity = gwy_unit_new_parse(image.yunit, &power10y);
+    siunitz = gwy_unit_new_parse(image.dataunit, &power10z);
+    if (!gwy_unit_equal(siunitx, siunity))
         g_warning("Different x and y units are not representable, ignoring y.");
 
     sanitise_real_size(&image.xreal, "x size");
     sanitise_real_size(&image.yreal, "y size");
-    dfield = gwy_data_field_new(image.xres, image.yres,
-                                image.xreal*pow10(power10x), image.yreal*pow10(power10y),
-                                FALSE);
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(dfield), siunitx);
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_z(dfield), siunitz);
+    dfield = gwy_field_new(image.xres, image.yres,
+                           image.xreal*gwy_exp10(power10x), image.yreal*gwy_exp10(power10y),
+                           FALSE);
+    gwy_unit_assign(gwy_field_get_unit_xy(dfield), siunitx);
+    gwy_unit_assign(gwy_field_get_unit_z(dfield), siunitz);
     g_object_unref(siunitx);
     g_object_unref(siunity);
     g_object_unref(siunitz);
-    q = pow10(power10z)/image.data_cnvt;
+    q = gwy_exp10(power10z)/image.data_cnvt;
 
     if (image.raw_data_type == GWY_TIFF_SLONG)
         rawtype = GWY_RAW_DATA_SINT32;
@@ -228,22 +221,22 @@ isdf_load(const gchar *filename,
         g_assert_not_reached();
     }
     gwy_convert_raw_data(image.raw_data, image.xres*image.yres, 1, rawtype, tiff->byte_order,
-                         gwy_data_field_get_data(dfield), q, 0.0);
+                         gwy_field_get_data(dfield), q, 0.0);
 
-    container = gwy_container_new();
-    gwy_container_pass_object(container, gwy_app_get_data_key_for_id(0), dfield);
+    file = gwy_file_new_in_construction();
+    gwy_file_pass_image(file, 0, dfield);
     meta = isdf_get_metadata(tiff, &image);
-    gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(0), meta);
+    gwy_file_pass_meta(file, GWY_FILE_IMAGE, 0, meta);
     if (gwy_container_gis_string_by_name(meta, "Data type", &p))
-        gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(0), p);
-    gwy_app_channel_title_fall_back(container, 0);
-    gwy_file_channel_import_log_add(container, 0, NULL, filename);
+        gwy_file_set_title(file, GWY_FILE_IMAGE, 0, p, FALSE);
+    gwy_image_title_fall_back(file, 0);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
 
 fail:
     isdf_image_free(&image);
     gwy_tiff_free(tiff);
 
-    return container;
+    return file;
 }
 
 static gboolean
@@ -318,7 +311,7 @@ isdf_get_metadata(const GwyTIFF *tiff,
     const gchar *s;
     gdouble v;
 
-    meta = gwy_container_new();
+    meta = gwy_container_new_in_construction();
 
     if ((s = gwy_enuml_to_string(image->file_type, "STM", 1, "AFM", 2, "EMP", 3, NULL)))
         gwy_container_set_const_string_by_name(meta, "File type", s);
