@@ -1,6 +1,6 @@
 /*
- *  $Id: iso28600.c 26510 2024-08-15 17:12:23Z yeti-dn $
- *  Copyright (C) 2011-2016 David Necas (Yeti).
+ *  $Id: iso28600.c 28822 2025-11-06 16:01:09Z yeti-dn $
+ *  Copyright (C) 2011-2025 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
  *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
@@ -41,15 +41,10 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/stats.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -141,27 +136,27 @@ typedef union {
     gdouble d;
     const gchar *s;
     struct { const gchar *str; gint value; } enumerated;
-    struct { GwySIUnit *unit; gint power10; } unit;
+    struct { GwyUnit *unit; gint power10; } unit;
     struct { const gchar **items; guint n; } text_list;
     struct { gint *items; guint n; } int_list;
     struct { gdouble *items; guint n; } real_list;
 } ISO28600FieldValue;
 
 static gboolean            module_register    (void);
-static gint                iso28600_detect    (const GwyFileDetectInfo *fileinfo,
+static gint                detect_file        (const GwyFileDetectInfo *fileinfo,
                                                gboolean only_name);
-static GwyContainer*       iso28600_load      (const gchar *filename,
-                                               GwyRunType mode,
+static GwyFile*            load_file          (const gchar *filename,
+                                               GwyRunModeFlags mode,
                                                GError **error);
-static gboolean            iso28600_export    (GwyContainer *data,
+static gboolean            export_file        (GwyFile *data,
                                                const gchar *filename,
-                                               GwyRunType mode,
+                                               GwyRunModeFlags mode,
                                                GError **error);
 static ISO28600FieldValue* load_header        (gchar **buffer,
                                                gchar **strings,
                                                GError **error);
 static void                free_header        (ISO28600FieldValue *header);
-static GwyContainer*       load_channels      (ISO28600FieldValue *header,
+static GwyFile*            load_channels      (ISO28600FieldValue *header,
                                                const gchar *filename,
                                                gchar **strings,
                                                gchar **p,
@@ -172,7 +167,7 @@ static GwyContainer*       load_channels      (ISO28600FieldValue *header,
                                                gdouble xreal,
                                                gdouble yreal,
                                                GError **error);
-static GwyContainer*       load_xyz_data      (ISO28600FieldValue *header,
+static GwyFile*            load_xyz_data      (ISO28600FieldValue *header,
                                                const gchar *filename,
                                                gchar **strings,
                                                gchar **p,
@@ -181,7 +176,8 @@ static GwyContainer*       load_xyz_data      (ISO28600FieldValue *header,
                                                gdouble qx,
                                                gdouble qy,
                                                GError **error);
-static GwyContainer*       load_spectra_graphs(ISO28600FieldValue *header,
+static GwyFile*            load_spectra_graphs(ISO28600FieldValue *header,
+                                               const gchar *filename,
                                                gchar **p,
                                                ISO28600SpectroscopyScanMode smode,
                                                guint nord,
@@ -190,7 +186,7 @@ static GwyContainer*       load_spectra_graphs(ISO28600FieldValue *header,
 static GwyContainer*       get_meta           (ISO28600FieldValue *header,
                                                gchar **strings,
                                                guint id);
-static gchar*              convert_unit       (GwySIUnit *unit);
+static gchar*              convert_unit       (GwyUnit *unit);
 static void                build_unit         (const gchar *str,
                                                ISO28600FieldValue *value);
 static void                build_enum         (const gchar *str,
@@ -355,7 +351,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports and exports ISO 28600:2011 SPM data transfer format."),
     "Yeti <yeti@gwyddion.net>",
-    "0.5",
+    "0.6",
     "David Nečas (Yeti)",
     "2011",
 };
@@ -369,17 +365,13 @@ module_register(void)
 
     gwy_file_func_register("iso28600",
                            N_("ISO 28600:2011 SPM data transfer files (.spm)"),
-                           (GwyFileDetectFunc)&iso28600_detect,
-                           (GwyFileLoadFunc)&iso28600_load,
-                           NULL,
-                           (GwyFileSaveFunc)&iso28600_export);
+                           detect_file, load_file, NULL, export_file);
 
     return TRUE;
 }
 
 static gint
-iso28600_detect(const GwyFileDetectInfo *fileinfo,
-                gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     if (only_name)
         return g_str_has_suffix(fileinfo->name_lowercase, EXTENSION) ? 10 : 0;
@@ -390,12 +382,12 @@ iso28600_detect(const GwyFileDetectInfo *fileinfo,
     return 100;
 }
 
-static GwyContainer*
-iso28600_load(const gchar *filename,
-              G_GNUC_UNUSED GwyRunType mode,
-              GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     ISO28600FieldValue *header = NULL;
     ISO28600ExperimentMode experiment;
     gchar *p, *buffer = NULL;
@@ -427,15 +419,15 @@ iso28600_load(const gchar *filename,
                 goto fail;
             }
         }
-        if (!gwy_si_unit_equal(header[25].unit.unit, header[26].unit.unit))
+        if (!gwy_unit_equal(header[25].unit.unit, header[26].unit.unit))
             g_warning("X and Y units differ, using X");
 
         if (header[16].enumerated.value == ISO28600_SCAN_IRREGULAR_MAPPING) {
             /* Ignore all area information because we have explicit X and Y coordinates of each point.  Just
              * precalculate the factors. */
-            xreal = pow10(header[25].unit.power10);
-            yreal = pow10(header[26].unit.power10);
-            container = load_xyz_data(header, filename, strings, &p, experiment, nchannels, xreal, yreal, error);
+            xreal = gwy_exp10(header[25].unit.power10);
+            yreal = gwy_exp10(header[26].unit.power10);
+            file = load_xyz_data(header, filename, strings, &p, experiment, nchannels, xreal, yreal, error);
         }
         else if (header[16].enumerated.value == ISO28600_SCAN_REGULAR_MAPPING) {
             xres = header[23].i;
@@ -447,10 +439,10 @@ iso28600_load(const gchar *filename,
             yreal = header[28].d;
             sanitise_real_size(&xreal, "x size");
             sanitise_real_size(&yreal, "y size");
-            xreal *= pow10(header[25].unit.power10);
-            yreal *= pow10(header[26].unit.power10);
-            container = load_channels(header, filename, strings, &p, experiment, nchannels, xres, yres, xreal, yreal,
-                                      error);
+            xreal *= gwy_exp10(header[25].unit.power10);
+            yreal *= gwy_exp10(header[26].unit.power10);
+            file = load_channels(header, filename, strings, &p, experiment, nchannels, xres, yres, xreal, yreal,
+                                 error);
         }
         else {
             g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
@@ -497,7 +489,7 @@ iso28600_load(const gchar *filename,
             goto fail;
         }
 
-        container = load_spectra_graphs(header, &p, smode, nord, npts, error);
+        file = load_spectra_graphs(header, filename, &p, smode, nord, npts, error);
     }
     else {
         err_NO_DATA(error);
@@ -512,10 +504,10 @@ fail:
     free_header(header);
     g_free(buffer);
 
-    return container;
+    return file;
 }
 
-static GwyContainer*
+static GwyFile*
 load_channels(ISO28600FieldValue *header,
               const gchar *filename,
               gchar **strings,
@@ -526,29 +518,28 @@ load_channels(ISO28600FieldValue *header,
               gdouble xreal, gdouble yreal,
               GError **error)
 {
-    GwyContainer *container = NULL;
-    GwyDataField *fields[MAX_CHANNELS];
+    GwyFile *file = NULL;
+    GwyField *fields[MAX_CHANNELS];
     gdouble *datas[MAX_CHANNELS];
     gdouble powers10[MAX_CHANNELS];
     gchar *line, *end;
-    GQuark quark;
     guint id, k;
 
     for (id = 0; id < nchannels; id++) {
-        GwySIUnit *unit;
+        GwyUnit *unit;
 
-        fields[id] = gwy_data_field_new(xres, yres, xreal, yreal, FALSE);
-        datas[id] = gwy_data_field_get_data(fields[id]);
-        unit = gwy_data_field_get_si_unit_xy(fields[id]);
-        gwy_si_unit_assign(unit, header[25].unit.unit);
-        unit = gwy_data_field_get_si_unit_z(fields[id]);
+        fields[id] = gwy_field_new(xres, yres, xreal, yreal, FALSE);
+        datas[id] = gwy_field_get_data(fields[id]);
+        unit = gwy_field_get_unit_xy(fields[id]);
+        gwy_unit_assign(unit, header[25].unit.unit);
+        unit = gwy_field_get_unit_z(fields[id]);
         if (experiment == ISO28600_EXPERIMENT_MAP_SC) {
-            gwy_si_unit_assign(unit, header[69].unit.unit);
-            powers10[id] = pow10(header[69].unit.power10);
+            gwy_unit_assign(unit, header[69].unit.unit);
+            powers10[id] = gwy_exp10(header[69].unit.power10);
         }
         else {
-            gwy_si_unit_assign(unit, header[95 + 3 * id].unit.unit);
-            powers10[id] = pow10(header[95 + 3*id].unit.power10);
+            gwy_unit_assign(unit, header[95 + 3 * id].unit.unit);
+            powers10[id] = gwy_exp10(header[95 + 3*id].unit.power10);
         }
     }
 
@@ -581,40 +572,35 @@ load_channels(ISO28600FieldValue *header,
         goto fail;
     }
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     for (id = 0; id < nchannels; id++) {
         GwyContainer *meta;
         const gchar *title;
 
-        quark = gwy_app_get_data_key_for_id(id);
-        gwy_container_set_object(container, quark, fields[id]);
+        gwy_file_set_image(file, id, fields[id]);
 
-        if ((meta = get_meta(header, strings, id))) {
-            quark = gwy_app_get_data_meta_key_for_id(id);
-            gwy_container_pass_object(container, quark, meta);
-        }
+        if ((meta = get_meta(header, strings, id)))
+            gwy_file_pass_meta(file, GWY_FILE_IMAGE, id, meta);
 
         if (experiment == ISO28600_EXPERIMENT_MAP_SC)
             title = header[68].s;
         else
             title = header[94 + 3*id].s;
 
-        if (*title) {
-            quark = gwy_app_get_data_title_key_for_id(id);
-            gwy_container_set_const_string(container, quark, title);
-        }
+        if (*title)
+            gwy_file_set_title(file, GWY_FILE_IMAGE, id, title, FALSE);
 
-        gwy_file_channel_import_log_add(container, id, NULL, filename);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, id, NULL, filename);
     }
 
 fail:
     for (id = 0; id < nchannels; id++)
         g_object_unref(fields[id]);
 
-    return container;
+    return file;
 }
 
-static GwyContainer*
+static GwyFile*
 load_xyz_data(ISO28600FieldValue *header,
               const gchar *filename,
               gchar **strings,
@@ -624,7 +610,7 @@ load_xyz_data(ISO28600FieldValue *header,
               gdouble qx, gdouble qy,
               GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     gdouble powers10[MAX_CHANNELS+2];
     GArray *alldata;
     gchar *line, *end;
@@ -634,9 +620,9 @@ load_xyz_data(ISO28600FieldValue *header,
     powers10[1] = qy;
     for (id = 0; id < nchannels; id++) {
         if (experiment == ISO28600_EXPERIMENT_MAP_SC)
-            powers10[id+2] = pow10(header[69].unit.power10);
+            powers10[id+2] = gwy_exp10(header[69].unit.power10);
         else
-            powers10[id+2] = pow10(header[95 + 3*id].unit.power10);
+            powers10[id+2] = gwy_exp10(header[95 + 3*id].unit.power10);
     }
 
     /* It is not clear how many rows with data there are because the specification is obviously nonsensical, using
@@ -667,14 +653,13 @@ load_xyz_data(ISO28600FieldValue *header,
         goto fail;
     }
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     k = alldata->len/(nchannels + 2);
     for (id = 0; id < nchannels; id++) {
-        GQuark quark;
         GwyContainer *meta;
         const gchar *title;
         GwySurface *surface;
-        GwySIUnit *unit;
+        GwyUnit *unit;
         GwyXYZ *data;
         guint i;
 
@@ -688,73 +673,68 @@ load_xyz_data(ISO28600FieldValue *header,
             pt.z = g_array_index(alldata, gdouble, (nchannels + 2)*i + 2 + id);
             data[i] = pt;
         }
-        unit = gwy_surface_get_si_unit_xy(surface);
-        gwy_si_unit_assign(unit, header[25].unit.unit);
-        unit = gwy_surface_get_si_unit_z(surface);
+        unit = gwy_surface_get_unit_xy(surface);
+        gwy_unit_assign(unit, header[25].unit.unit);
+        unit = gwy_surface_get_unit_z(surface);
         if (experiment == ISO28600_EXPERIMENT_MAP_SC) {
-            gwy_si_unit_assign(unit, header[69].unit.unit);
+            gwy_unit_assign(unit, header[69].unit.unit);
         }
         else {
-            gwy_si_unit_assign(unit, header[95 + 3 * id].unit.unit);
+            gwy_unit_assign(unit, header[95 + 3 * id].unit.unit);
         }
 
-        quark = gwy_app_get_data_key_for_id(id);
-        gwy_container_set_object(container, quark, surface);
-
-        if ((meta = get_meta(header, strings, id))) {
-            quark = gwy_app_get_surface_meta_key_for_id(id);
-            gwy_container_pass_object(container, quark, meta);
-        }
+        gwy_file_set_xyz(file, id, surface);
+        if ((meta = get_meta(header, strings, id)))
+            gwy_file_pass_meta(file, GWY_FILE_XYZ, id, meta);
 
         if (experiment == ISO28600_EXPERIMENT_MAP_SC)
             title = header[68].s;
         else
             title = header[94 + 3*id].s;
 
-        if (*title) {
-            quark = gwy_app_get_surface_title_key_for_id(id);
-            gwy_container_set_const_string(container, quark, title);
-        }
+        if (*title)
+            gwy_file_set_title(file, GWY_FILE_XYZ, id, title, FALSE);
 
-        gwy_file_xyz_import_log_add(container, id, NULL, filename);
+        gwy_log_add_import(file, GWY_FILE_XYZ, id, NULL, filename);
         g_object_unref(surface);
     }
 
 fail:
     g_array_free(alldata, TRUE);
 
-    return container;
+    return file;
 }
 
 /* XXX: The data format for spectroscopy is not well described so I estimate it is how I would do it. */
-static GwyContainer*
+static GwyFile*
 load_spectra_graphs(ISO28600FieldValue *header,
+                    const gchar *filename,
                     gchar **p,
                     ISO28600SpectroscopyScanMode smode,
                     guint nord,
                     guint npts,
                     GError **error)
 {
-    GwyContainer *container = NULL;
-    GwySIUnit **units;
+    GwyFile *file = NULL;
+    GwyUnit **units;
     gdouble *data, *powers10;
     gchar *line, *end;
     guint id, k, from;
 
     /* For an irregular scan, the ordinates are preceeded by abscissa values. At least, I suppose. */
-    units = g_new(GwySIUnit*, nord + 1);
+    units = g_new(GwyUnit*, nord + 1);
     powers10 = g_new(gdouble, nord + 1);
     units[0] = g_object_ref(header[75].unit.unit);
-    powers10[0] = pow10(header[75].unit.power10);
+    powers10[0] = gwy_exp10(header[75].unit.power10);
     from = (smode == ISO28600_SPECTROSCOPY_SCAN_REGULAR) ? 1 : 0;
     for (id = 0; id < nord; id++) {
         if (header[83].text_list.n) {
             gint power10;
-            units[id+1] = gwy_si_unit_new_parse(header[83].text_list.items[id], &power10);
-            powers10[id+1] = pow10(power10);
+            units[id+1] = gwy_unit_new_parse(header[83].text_list.items[id], &power10);
+            powers10[id+1] = gwy_exp10(power10);
         }
         else {
-            units[id+1] = gwy_si_unit_new(NULL);
+            units[id+1] = gwy_unit_new(NULL);
             powers10[id+1] = 1.0;
         }
     }
@@ -790,15 +770,15 @@ load_spectra_graphs(ISO28600FieldValue *header,
         goto fail;
     }
 
-    container = gwy_container_new();
-    for (id = 1; id <= nord; id++) {
+    file = gwy_file_new_in_construction();
+    for (id = 0; id < nord; id++) {
         GwyGraphModel *gmodel = gwy_graph_model_new();
         GwyGraphCurveModel *gcmodel = gwy_graph_curve_model_new();
         const gchar *label = "";
 
-        gwy_graph_curve_model_set_data(gcmodel, data, data + id*npts, npts);
+        gwy_graph_curve_model_set_data(gcmodel, data, data + (id + 1)*npts, npts);
         if (header[82].text_list.n)
-            label = header[82].text_list.items[id-1];
+            label = header[82].text_list.items[id];
 
         g_object_set(gcmodel,
                      "mode", GWY_GRAPH_CURVE_LINE,
@@ -808,14 +788,15 @@ load_spectra_graphs(ISO28600FieldValue *header,
         g_object_unref(gcmodel);
 
         g_object_set(gmodel,
-                     "si-unit-x", units[0],
-                     "si-unit-y", units[id],
+                     "unit-x", units[0],
+                     "unit-y", units[id+1],
                      "title", header[72].s,
                      "axis-label-left", label,
                      "axis-label-bottom", header[74].s,
                      NULL);
 
-        gwy_container_pass_object(container, gwy_app_get_graph_key_for_id(id), gmodel);
+        gwy_file_pass_graph(file, id, gmodel);
+        gwy_log_add_import(file, GWY_FILE_GRAPH, id, NULL, filename);
     }
 
 fail:
@@ -823,7 +804,7 @@ fail:
     g_free(units);
     g_free(powers10);
 
-    return container;
+    return file;
 }
 
 static GwyContainer*
@@ -843,7 +824,7 @@ get_meta(ISO28600FieldValue *header,
         { "deg", 60 }, { "deg", 61 },
     };
     gint year, month, day, hour, minute, second, offset;
-    GwyContainer *meta = gwy_container_new();
+    GwyContainer *meta = gwy_container_new_in_construction();
     guint k;
 
     for (k = 0; k < G_N_ELEMENTS(fields); k++) {
@@ -862,8 +843,10 @@ get_meta(ISO28600FieldValue *header,
     for (k = 0; k < G_N_ELEMENTS(fields_with_units)/2; k++) {
         guint i = fields_with_units[2*k], j = fields_with_units[2*k + 1];
 
-        if (header[i].d)
-            gwy_container_set_string_by_name(meta, header_fields[i].name, g_strconcat(strings[i], " ", strings[j], NULL));
+        if (header[i].d) {
+            gwy_container_set_string_by_name(meta, header_fields[i].name,
+                                             g_strconcat(strings[i], " ", strings[j], NULL));
+        }
     }
 
     for (k = 0; k < G_N_ELEMENTS(fields_without_units); k++) {
@@ -889,7 +872,7 @@ get_meta(ISO28600FieldValue *header,
                                     year, month, day, hour, minute, second, offset);
         else
             value = g_strdup_printf("%04u-%02u-%02u %02u:%02u:%02u",
-                                    year, month, day, hour, minute, second);
+                                                 year, month, day, hour, minute, second);
         gwy_container_set_string_by_name(meta, "Date", value);
     }
 
@@ -988,7 +971,7 @@ build_unit(const gchar *str,
     if (gwy_stramong(str, "d", "n", NULL))
         str = NULL;
 
-    value->unit.unit = gwy_si_unit_new_parse(str, &value->unit.power10);
+    value->unit.unit = gwy_unit_new_parse(str, &value->unit.power10);
 }
 
 static gchar**
@@ -1118,7 +1101,7 @@ free_header(ISO28600FieldValue *header)
     if (header) {
         for (i = 0; i < G_N_ELEMENTS(header_fields); i++) {
             if (header_fields[i].type == ISO28600_UNIT)
-                GWY_OBJECT_UNREF(header[i].unit.unit);
+                g_clear_object(&header[i].unit.unit);
             else if (header_fields[i].type == ISO28600_TEXT_LIST)
                 g_free(header[i].text_list.items);
             else if (header_fields[i].type == ISO28600_INTEGERS)
@@ -1133,10 +1116,10 @@ free_header(ISO28600FieldValue *header)
 }
 
 static gboolean
-iso28600_export(GwyContainer *container,
-                const gchar *filename,
-                G_GNUC_UNUSED GwyRunType mode,
-                GError **error)
+export_file(GwyFile *file,
+            const gchar *filename,
+            G_GNUC_UNUSED GwyRunModeFlags mode,
+            GError **error)
 {
     static const gchar header_template[] = /* {{{ */
     "ISO/TC 201 SPM data transfer format\n"
@@ -1274,16 +1257,16 @@ iso28600_export(GwyContainer *container,
 
     gchar xreal[32], yreal[32], xoff[32], yoff[32];
     gchar *unitxy = NULL, *unitz = NULL, *title = NULL;
-    GwyDataField *dfield;
+    GwyField *dfield;
     const gdouble *d;
     guint xres, yres, i, j, n;
     gint id;
     gboolean ok = FALSE;
     FILE *fh;
 
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &dfield,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &dfield,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     if (!dfield) {
         err_NO_CHANNEL_EXPORT(error);
         return FALSE;
@@ -1295,13 +1278,13 @@ iso28600_export(GwyContainer *container,
         return FALSE;
     }
 
-    d = gwy_data_field_get_data_const(dfield);
-    xres = gwy_data_field_get_xres(dfield);
-    yres = gwy_data_field_get_yres(dfield);
-    unitxy = convert_unit(gwy_data_field_get_si_unit_xy(dfield));
-    unitz = convert_unit(gwy_data_field_get_si_unit_z(dfield));
+    d = gwy_field_get_data_const(dfield);
+    xres = gwy_field_get_xres(dfield);
+    yres = gwy_field_get_yres(dfield);
+    unitxy = convert_unit(gwy_field_get_unit_xy(dfield));
+    unitz = convert_unit(gwy_field_get_unit_z(dfield));
 
-    title = gwy_app_get_data_field_title(container, id);
+    title = gwy_file_get_display_title(file, GWY_FILE_IMAGE, id);
     n = strlen(title);
     for (i = 0; i < n; i++) {
         if ((guchar)title[i] & 0x80)
@@ -1312,10 +1295,10 @@ iso28600_export(GwyContainer *container,
         title = g_strdup("Not representable in ASCII. Ask the committee to fix the standard to permit UTF-8.");
     }
 
-    g_ascii_formatd(xreal, sizeof(xreal), "%.8g", gwy_data_field_get_xreal(dfield));
-    g_ascii_formatd(yreal, sizeof(yreal), "%.8g", gwy_data_field_get_yreal(dfield));
-    g_ascii_formatd(xoff, sizeof(xoff), "%.8g", gwy_data_field_get_xoffset(dfield));
-    g_ascii_formatd(yoff, sizeof(yoff), "%.8g", gwy_data_field_get_yoffset(dfield));
+    g_ascii_formatd(xreal, sizeof(xreal), "%.8g", gwy_field_get_xreal(dfield));
+    g_ascii_formatd(yreal, sizeof(yreal), "%.8g", gwy_field_get_yreal(dfield));
+    g_ascii_formatd(xoff, sizeof(xoff), "%.8g", gwy_field_get_xoffset(dfield));
+    g_ascii_formatd(yoff, sizeof(yoff), "%.8g", gwy_field_get_yoffset(dfield));
 
     if (gwy_fprintf(fh, header_template,
                     xres, yres,
@@ -1356,9 +1339,9 @@ fail:
 }
 
 static gchar*
-convert_unit(GwySIUnit *unit)
+convert_unit(GwyUnit *unit)
 {
-    gchar *str = gwy_si_unit_get_string(unit, GWY_SI_UNIT_FORMAT_PLAIN);
+    gchar *str = gwy_unit_get_string(unit, GWY_UNIT_FORMAT_PLAIN);
     const gchar *convstr;
 
     /* Ignore non-base units altogether because we never produce them. */
