@@ -1,6 +1,6 @@
 /*
- *  $Id: jeol.c 26460 2024-08-04 06:29:59Z yeti-dn $
- *  Copyright (C) 2006 David Necas (Yeti), Petr Klapetek.
+ *  $Id: jeol.c 28789 2025-11-04 17:14:03Z yeti-dn $
+ *  Copyright (C) 2006-2025 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -43,15 +43,10 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdarg.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/stats.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 
 #include "err.h"
 #include "get.h"
@@ -336,17 +331,17 @@ typedef struct {
     /* JEOLLMCantileverParam special_measurement_param; */
 } JEOLImageHeader;
 
-static gboolean      module_register       (void);
-static gint          jeol_detect           (const GwyFileDetectInfo *fileinfo,
-                                            gboolean only_name);
-static GwyContainer* jeol_load             (const gchar *filename,
-                                            GwyRunType mode,
-                                            GError **error);
-static void          jeol_read_image_header(const guchar *p,
-                                            JEOLImageHeader *header);
-static GwyDataField* jeol_read_data_field  (const guchar *buffer,
-                                            const JEOLImageHeader *header);
-static GwyContainer* jeol_get_metadata     (const JEOLImageHeader *header);
+static gboolean      module_register  (void);
+static gint          detect_file      (const GwyFileDetectInfo *fileinfo,
+                                       gboolean only_name);
+static GwyFile*      load_file        (const gchar *filename,
+                                       GwyRunModeFlags mode,
+                                       GError **error);
+static void          read_image_header(const guchar *p,
+                                       JEOLImageHeader *header);
+static GwyField*     read_field       (const guchar *buffer,
+                                       const JEOLImageHeader *header);
+static GwyContainer* get_metadata     (const JEOLImageHeader *header);
 
 static const GwyEnum data_sources[] = {
     { "Topography",        JEOL_DATA_SOURCE_Z,                 },
@@ -421,7 +416,7 @@ static GwyModuleInfo module_info = {
     module_register,
     N_("Imports JEOL data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.9",
+    "0.10",
     "David Nečas (Yeti) & Petr Klapetek",
     "2006",
 };
@@ -433,16 +428,13 @@ module_register(void)
 {
     gwy_file_func_register("jeol",
                            N_("JEOL data files (.tif)"),
-                           (GwyFileDetectFunc)&jeol_detect,
-                           (GwyFileLoadFunc)&jeol_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-jeol_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     if (only_name)
         return 0;
@@ -456,19 +448,17 @@ jeol_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
     return 90;
 }
 
-/****************************************************************************/
-
-static GwyContainer*
-jeol_load(const gchar *filename,
-          G_GNUC_UNUSED GwyRunType mode,
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
           GError **error)
 {
     JEOLImageHeader image_header;
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     guchar *buffer = NULL;
     gsize expected_size, size = 0;
     GError *err = NULL;
-    GwyDataField *dfield = NULL;
+    GwyField *dfield = NULL;
     const gchar *title;
     gchar *s;
 
@@ -485,7 +475,7 @@ jeol_load(const gchar *filename,
         goto fail;
     }
 
-    jeol_read_image_header(buffer, &image_header);
+    read_image_header(buffer, &image_header);
     /* Elementrary sanity */
     if (image_header.bpp != 16) {
         err_BPP(error, image_header.bpp);
@@ -507,41 +497,41 @@ jeol_load(const gchar *filename,
     sanitise_real_size(&image_header.xreal, "x size");
     sanitise_real_size(&image_header.yreal, "y size");
 
-    dfield = jeol_read_data_field(buffer + JEOL_DATA_START, &image_header);
+    dfield = read_field(buffer + JEOL_DATA_START, &image_header);
     if (!dfield) {
         g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
                     _("The type of data is unknown.  Please report it to the developers."));
         goto fail;
     }
 
-    container = gwy_container_new();
-    gwy_container_pass_object(container, gwy_app_get_data_key_for_id(0), dfield);
+    file = gwy_file_new_in_construction();
+    gwy_file_pass_image(file, 0, dfield);
 
     /* Title */
     s = gwy_convert_to_utf8(image_header.internal_filename, -1, "ISO-8859-1");
     if (s)
         g_strstrip(s);
     if (s && *s)
-        gwy_container_set_string(container, gwy_app_get_data_title_key_for_id(0), s);
+        gwy_file_pass_title(file, GWY_FILE_IMAGE, 0, s);
     else {
         title = gwy_enum_to_string(image_header.data_source, data_sources, G_N_ELEMENTS(data_sources));
         if (title)
-            gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(0), title);
+            gwy_file_set_title(file, GWY_FILE_IMAGE, 0, title, FALSE);
     }
 
     /* Meta */
-    gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(0), jeol_get_metadata(&image_header));
+    gwy_file_pass_meta(file, GWY_FILE_IMAGE, 0, get_metadata(&image_header));
 
-    gwy_file_channel_import_log_add(container, 0, NULL, filename);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
 
 fail:
     gwy_file_abandon_contents(buffer, size, NULL);
-    return container;
+    return file;
 }
 
 static void
-jeol_read_image_header(const guchar *p,
-                       JEOLImageHeader *header)
+read_image_header(const guchar *p,
+                  JEOLImageHeader *header)
 {
     p += TIFF_HEADER_SIZE;
 
@@ -581,11 +571,11 @@ jeol_read_image_header(const guchar *p,
     header->save_date.weekday_sun = *(p++);
     header->measurement_time.hour = *(p++);
     header->measurement_time.minute = *(p++);
-    header->measurement_time.second= *(p++);
+    header->measurement_time.second = *(p++);
     header->measurement_time.second_100 = *(p++);
     header->save_time.hour = *(p++);
     header->save_time.minute = *(p++);
-    header->save_time.second= *(p++);
+    header->save_time.second = *(p++);
     header->save_time.second_100 = *(p++);
     p += 0x100;  /* lookup table */
     header->fft_offset = gwy_get_guint32_le(&p);
@@ -687,11 +677,11 @@ jeol_read_image_header(const guchar *p,
     /* stuff... */
 }
 
-static GwyDataField*
-jeol_read_data_field(const guchar *buffer,
-                     const JEOLImageHeader *header)
+static GwyField*
+read_field(const guchar *buffer,
+           const JEOLImageHeader *header)
 {
-    GwyDataField *dfield;
+    GwyField *dfield;
     const gchar *unitstr;
     gdouble q, z0;
 
@@ -727,13 +717,13 @@ jeol_read_data_field(const guchar *buffer,
         break;
     }
 
-    dfield = gwy_data_field_new(header->xres, header->yres, Nanometer*header->xreal, Nanometer*header->yreal, FALSE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), unitstr);
+    dfield = gwy_field_new(header->xres, header->yres, Nanometer*header->xreal, Nanometer*header->yreal, FALSE);
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
+    gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), unitstr);
 
     gwy_convert_raw_data(buffer, header->xres*header->yres, 1,
                          GWY_RAW_DATA_UINT16, GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                         gwy_data_field_get_data(dfield), q, z0);
+                         gwy_field_get_data(dfield), q, z0);
     return dfield;
 }
 
@@ -769,7 +759,7 @@ format_bit(GwyContainer *meta,
 }
 
 static GwyContainer*
-jeol_get_metadata(const JEOLImageHeader *header)
+get_metadata(const JEOLImageHeader *header)
 {
     const JEOLSPMParam *spm_param;
     const JEOLDate *date;
@@ -777,7 +767,7 @@ jeol_get_metadata(const JEOLImageHeader *header)
     GwyContainer *meta;
     const gchar *s;
 
-    meta = gwy_container_new();
+    meta = gwy_container_new_in_construction();
 
     spm_param = &header->spm_param;
     format_meta(meta, "WinSPM Version", "%.2f", header->winspm_version/100.0);
