@@ -1,5 +1,5 @@
 /*
- *  $Id: jeoltem.c 26005 2023-11-09 13:17:31Z yeti-dn $
+ *  $Id: jeoltem.c 28789 2025-11-04 17:14:03Z yeti-dn $
  *  Copyright (C) 2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -25,11 +25,9 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/stats.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 #include "err.h"
 #include "gwytiff.h"
 
@@ -99,29 +97,29 @@ typedef struct {
     gint camerano;
 } JEOLTEMFile;
 
-static gboolean            module_register        (void);
-static gint                jeoltem_detect         (const GwyFileDetectInfo *fileinfo,
-                                                   gboolean only_name);
-static GwyContainer*       jeoltem_load           (const gchar *filename,
-                                                   GwyRunType mode,
-                                                   GError **error);
-static GwyContainer*       jeoltem_load_data      (JEOLTEMFile *jtfile,
-                                                   GError **error);
-static gboolean            jeoltem_load_header    (JEOLTEMFile *jtfile,
-                                                   GError **error);
-static GwyContainer*       get_meta               (JEOLTEMFile *jtfile);
-static void                jeoltem_file_free      (JEOLTEMFile *jtfile);
-static void                jeoltem_read_other_tags(JEOLTEMTags *tags,
-                                                   GwyTIFF *tiff);
-static const GwyTIFFEntry* jeoltem_find_header    (GwyTIFF *tiff,
-                                                   GError **error);
+static gboolean            module_register(void);
+static gint                detect_file    (const GwyFileDetectInfo *fileinfo,
+                                           gboolean only_name);
+static GwyFile*            load_file      (const gchar *filename,
+                                           GwyRunModeFlags mode,
+                                           GError **error);
+static GwyFile*            load_data      (JEOLTEMFile *jtfile,
+                                           GError **error);
+static gboolean            load_header    (JEOLTEMFile *jtfile,
+                                           GError **error);
+static GwyContainer*       get_meta       (JEOLTEMFile *jtfile);
+static void                free_file      (JEOLTEMFile *jtfile);
+static void                read_other_tags(JEOLTEMTags *tags,
+                                           GwyTIFF *tiff);
+static const GwyTIFFEntry* find_header    (GwyTIFF *tiff,
+                                           GError **error);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     module_register,
     N_("Imports JEOL TEM images."),
     "Yeti <yeti@gwyddion.net>",
-    "0.1",
+    "0.2",
     "David Nečas (Yeti)",
     "2021",
 };
@@ -133,16 +131,13 @@ module_register(void)
 {
     gwy_file_func_register("jeol-tem",
                            N_("JEOL TIF TEM image (.tif)"),
-                           (GwyFileDetectFunc)&jeoltem_detect,
-                           (GwyFileLoadFunc)&jeoltem_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-jeoltem_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     GwyTIFF *tiff;
     const GwyTIFFEntry *entry;
@@ -164,7 +159,7 @@ jeoltem_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
         return 0;
 
     /* Check the beginning of the binary header. */
-    if (!(entry = jeoltem_find_header(tiff, NULL)) || entry->count < 26)
+    if (!(entry = find_header(tiff, NULL)) || entry->count < 26)
         goto end;
 
     t = entry->value;
@@ -184,29 +179,29 @@ end:
     return score;
 }
 
-static GwyContainer*
-jeoltem_load(const gchar *filename, G_GNUC_UNUSED GwyRunType mode, GError **error)
+static GwyFile*
+load_file(const gchar *filename, G_GNUC_UNUSED GwyRunModeFlags mode, GError **error)
 {
     JEOLTEMFile jtfile;
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
 
-    gwy_clear(&jtfile, 1);
+    gwy_clear1(jtfile);
 
     jtfile.tiff = gwy_tiff_load(filename, error);
     if (!jtfile.tiff)
         return NULL;
-    if (!jeoltem_load_header(&jtfile, error))
+    if (!load_header(&jtfile, error))
         goto end;
 
-    jeoltem_read_other_tags(&jtfile.tags, jtfile.tiff);
-    container = jeoltem_load_data(&jtfile, error);
-    if (container)
-        gwy_file_channel_import_log_add(container, 0, NULL, filename);
+    read_other_tags(&jtfile.tags, jtfile.tiff);
+    file = load_data(&jtfile, error);
+    if (file)
+        gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
 
 end:
-    jeoltem_file_free(&jtfile);
+    free_file(&jtfile);
 
-    return container;
+    return file;
 }
 
 static gchar*
@@ -373,7 +368,7 @@ read_variable_block(BlockContent *b, const guchar **p, const guchar *end, GError
 }
 
 static gboolean
-jeoltem_load_header(JEOLTEMFile *jtfile, GError **error)
+load_header(JEOLTEMFile *jtfile, GError **error)
 {
     GwyTIFF *tiff = jtfile->tiff;
     const GwyTIFFEntry *entry;
@@ -383,7 +378,7 @@ jeoltem_load_header(JEOLTEMFile *jtfile, GError **error)
     G_GNUC_UNUSED guint tagsize, four, zero;
     gboolean ok = FALSE;
 
-    if (!(entry = jeoltem_find_header(tiff, error)))
+    if (!(entry = find_header(tiff, error)))
         return FALSE;
 
     t = entry->value;
@@ -411,7 +406,7 @@ jeoltem_load_header(JEOLTEMFile *jtfile, GError **error)
     while (p < end) {
         BlockContent b;
 
-        gwy_clear(&b, 1);
+        gwy_clear1(b);
         if (end - p < 3) {
             err_TRUNCATED_HEADER(error);
             goto end;
@@ -478,11 +473,12 @@ end:
     return ok;
 }
 
-static GwyContainer*
-jeoltem_load_data(JEOLTEMFile *jtfile, GError **error)
+static GwyFile*
+load_data(JEOLTEMFile *jtfile, GError **error)
 {
-    GwyContainer *container, *meta;
-    GwyDataField *dfield;
+    GwyFile *file;
+    GwyContainer *meta;
+    GwyField *dfield;
     GwyTIFFImageReader *reader;
     gdouble xstep, ystep, q;
     gdouble *data;
@@ -499,26 +495,23 @@ jeoltem_load_data(JEOLTEMFile *jtfile, GError **error)
         xstep = ystep = 17.87e-6/jtfile->mag;
 
     q = 1.0/((1 << reader->bits_per_sample) - 1);
-    dfield = gwy_data_field_new(reader->width, reader->height, reader->width * xstep, reader->height * ystep, FALSE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
+    dfield = gwy_field_new(reader->width, reader->height, reader->width * xstep, reader->height * ystep, FALSE);
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
 
-    data = gwy_data_field_get_data(dfield);
+    data = gwy_field_get_data(dfield);
     for (i = 0; i < reader->height; i++)
         gwy_tiff_read_image_row_averaged(jtfile->tiff, reader, i, q, 0.0, data + i*reader->width);
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
 
-    gwy_container_pass_object_by_name(container, "/0/data", dfield);
-
-    gwy_container_set_const_string_by_name(container, "/0/data/title", "Intensity");
-
-    if ((meta = get_meta(jtfile))) {
-        gwy_container_pass_object_by_name(container, "/0/meta", meta);
-    }
+    gwy_file_pass_image(file, 0, dfield);
+    gwy_file_set_title(file, GWY_FILE_IMAGE, 0, "Intensity", FALSE);
+    if ((meta = get_meta(jtfile)))
+        gwy_file_pass_meta(file, GWY_FILE_IMAGE, 0, meta);
 
     gwy_tiff_image_reader_free(reader);
 
-    return container;
+    return file;
 }
 
 /* NB: This consumes string values in blocks.  We do not have to free them later, but we cannot access them later
@@ -526,7 +519,7 @@ jeoltem_load_data(JEOLTEMFile *jtfile, GError **error)
 static GwyContainer*
 get_meta(JEOLTEMFile *jtfile)
 {
-    GwyContainer *meta = gwy_container_new();
+    GwyContainer *meta = gwy_container_new_in_construction();
     GArray *blocks = jtfile->blocks;
     guint i, n = blocks->len;
 
@@ -558,7 +551,7 @@ get_meta(JEOLTEMFile *jtfile)
 }
 
 static void
-jeoltem_file_free(JEOLTEMFile *jtfile)
+free_file(JEOLTEMFile *jtfile)
 {
     GArray *blocks = jtfile->blocks;
     guint i;
@@ -580,7 +573,7 @@ jeoltem_file_free(JEOLTEMFile *jtfile)
 }
 
 static void
-jeoltem_read_other_tags(JEOLTEMTags *tags, GwyTIFF *tiff)
+read_other_tags(JEOLTEMTags *tags, GwyTIFF *tiff)
 {
     gwy_clear(tags, 1);
 
@@ -615,7 +608,7 @@ jeoltem_read_other_tags(JEOLTEMTags *tags, GwyTIFF *tiff)
 }
 
 static const GwyTIFFEntry*
-jeoltem_find_header(GwyTIFF *tiff, GError **error)
+find_header(GwyTIFF *tiff, GError **error)
 {
     const GwyTIFFEntry *entry;
 
