@@ -1,6 +1,6 @@
 /*
- *  $Id: jspmfile.c 25688 2023-09-20 11:08:15Z yeti-dn $
- *  Copyright (C) 2014 David Necas (Yeti).
+ *  $Id: jspmfile.c 28911 2025-11-24 18:27:42Z yeti-dn $
+ *  Copyright (C) 2014-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -15,8 +15,8 @@
  *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-/* This format seems similar in spirit to jeol.c, but newer, more complicated
- * and, above all, lacking proper documentation. */
+/* This format seems similar in spirit to jeol.c, but newer, more complicated and, above all, lacking proper
+ * documentation. */
 
 /**
  * [FILE-MAGIC-FREEDESKTOP]
@@ -49,15 +49,10 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdarg.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/stats.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 
 #include "err.h"
 #include "get.h"
@@ -150,41 +145,41 @@ typedef struct {
     gdouble piezo_c;
 } JSPMFile;
 
-static gboolean      module_register        (void);
-static gint          jspm_detect            (const GwyFileDetectInfo *fileinfo,
-                                             gboolean only_name);
-static GwyContainer* jspm_load              (const gchar *filename,
-                                             GwyRunType mode,
-                                             GError **error);
-static gboolean      meas_header_seems_ok   (const gchar *buffer);
-static gboolean      jspm_read_headers      (JSPMFile *jspmfile,
-                                             const guchar *buffer,
-                                             gsize size,
-                                             GError **error);
-static gboolean      read_image_header_block(JSPMFile *jspmfile,
-                                             const guchar *buffer,
-                                             gsize size,
-                                             GError **error);
-static gboolean      read_file_header_block (JSPMFile *jspmfile,
-                                             const guchar *buffer,
-                                             gsize size,
-                                             GError **error);
-static gboolean      read_piezo_header_block(JSPMFile *jspmfile,
-                                             const guchar *buffer,
-                                             gsize size,
-                                             GError **error);
-static void          jspm_add_data_field    (const JSPMFile *jspmfile,
-                                             const guchar *buffer,
-                                             GwyContainer *container);
-static void          jspm_add_meta          (JSPMFile *jspmfile,
-                                             GwyContainer *container);
+static gboolean module_register        (void);
+static gint     detect_file            (const GwyFileDetectInfo *fileinfo,
+                                        gboolean only_name);
+static GwyFile* load_file              (const gchar *filename,
+                                        GwyRunModeFlags mode,
+                                        GError **error);
+static gboolean meas_header_seems_ok   (const gchar *buffer);
+static gboolean read_headers           (JSPMFile *jspmfile,
+                                        const guchar *buffer,
+                                        gsize size,
+                                        GError **error);
+static gboolean read_image_header_block(JSPMFile *jspmfile,
+                                        const guchar *buffer,
+                                        gsize size,
+                                        GError **error);
+static gboolean read_file_header_block (JSPMFile *jspmfile,
+                                        const guchar *buffer,
+                                        gsize size,
+                                        GError **error);
+static gboolean read_piezo_header_block(JSPMFile *jspmfile,
+                                        const guchar *buffer,
+                                        gsize size,
+                                        GError **error);
+static void     add_field              (const JSPMFile *jspmfile,
+                                        const guchar *buffer,
+                                        GwyFile *file);
+static void     add_meta               (JSPMFile *jspmfile,
+                                        GwyFile *file);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     module_register,
     N_("Imports JEOL JSPM data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.2",
+    "0.3",
     "David Nečas (Yeti)",
     "2014",
 };
@@ -196,16 +191,13 @@ module_register(void)
 {
     gwy_file_func_register("jspmfile",
                            N_("JEOL JSPM data files (.tif)"),
-                           (GwyFileDetectFunc)&jspm_detect,
-                           (GwyFileLoadFunc)&jspm_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-jspm_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     if (only_name)
         return 0;
@@ -219,18 +211,18 @@ jspm_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
     return 100;
 }
 
-static GwyContainer*
-jspm_load(const gchar *filename,
-          G_GNUC_UNUSED GwyRunType mode,
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
           GError **error)
 {
     JSPMFile jspmfile;
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     guchar *buffer = NULL;
     gsize size = 0;
     GError *err = NULL;
 
-    gwy_clear(&jspmfile, 1);
+    gwy_clear1(jspmfile);
 
     if (!gwy_file_get_contents(filename, &buffer, &size, &err)) {
         err_GET_FILE_CONTENTS(error, &err);
@@ -245,20 +237,20 @@ jspm_load(const gchar *filename,
         goto fail;
     }
 
-    if (!jspm_read_headers(&jspmfile, buffer, size, error))
+    if (!read_headers(&jspmfile, buffer, size, error))
         goto fail;
 
-    container = gwy_container_new();
-    jspm_add_data_field(&jspmfile, buffer, container);
-    jspm_add_meta(&jspmfile, container);
-    gwy_file_channel_import_log_add(container, 0, NULL, filename);
+    file = gwy_file_new_in_construction();
+    add_field(&jspmfile, buffer, file);
+    add_meta(&jspmfile, file);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
 
 fail:
     g_free(jspmfile.comment);
     if (jspmfile.blocks)
         g_array_free(jspmfile.blocks, TRUE);
     gwy_file_abandon_contents(buffer, size, NULL);
-    return container;
+    return file;
 }
 
 static gboolean
@@ -278,10 +270,10 @@ err_JSPM_BLOCK(GError **error, guint i)
 /* We kind of know how to walk through the physical structure of the file headers.  We are at a loss as to what to do
  * with their content though. */
 static gboolean
-jspm_read_headers(JSPMFile *jspmfile,
-                  const guchar *buffer,
-                  gsize size,
-                  GError **error)
+read_headers(JSPMFile *jspmfile,
+             const guchar *buffer,
+             gsize size,
+             GError **error)
 {
     JSPMHeaderBlock block;
     const guchar *p = buffer;
@@ -440,7 +432,8 @@ read_file_header_block(JSPMFile *jspmfile,
 
 static gboolean
 read_piezo_header_block(JSPMFile *jspmfile,
-                        const guchar *buffer, G_GNUC_UNUSED gsize size,
+                        const guchar *buffer,
+                        G_GNUC_UNUSED gsize size,
                         GError **error)
 {
     enum {
@@ -475,10 +468,9 @@ read_piezo_header_block(JSPMFile *jspmfile,
 }
 
 static void
-jspm_add_data_field(const JSPMFile *jspmfile,
-                    const guchar *buffer,
-                    GwyContainer *container)
-
+add_field(const JSPMFile *jspmfile,
+          const guchar *buffer,
+          GwyFile *file)
 {
     static const GwyEnum signal_names[] = {
         { "Topography",       JSPM_SIGNAL_TOPOGRAPHY,   },
@@ -513,15 +505,15 @@ jspm_add_data_field(const JSPMFile *jspmfile,
         { "",    JSPM_UNIT_NONE,           },
     };
 
-    GwyDataField *dfield;
-    GwySIUnit *zunit;
+    GwyField *dfield;
+    GwyUnit *zunit;
     gdouble q = 1.0, z0 = 0.0;
     gint power10 = 0;
     const gchar *unitstr, *title;
 
-    dfield = gwy_data_field_new(jspmfile->xres, jspmfile->yres,
-                                Nanometer*jspmfile->xreal, Nanometer*jspmfile->yreal,
-                                FALSE);
+    dfield = gwy_field_new(jspmfile->xres, jspmfile->yres,
+                           Nanometer*jspmfile->xreal, Nanometer*jspmfile->yreal,
+                           FALSE);
 
     title = gwy_enum_to_string(jspmfile->signal_name, signal_names, G_N_ELEMENTS(signal_names));
     if (!title || !strlen(title))
@@ -529,8 +521,8 @@ jspm_add_data_field(const JSPMFile *jspmfile,
 
     unitstr = gwy_enum_to_string(jspmfile->unit, unit_types, G_N_ELEMENTS(unit_types));
 
-    zunit = gwy_data_field_get_si_unit_z(dfield);
-    gwy_si_unit_set_from_string_parse(zunit, unitstr, &power10);
+    zunit = gwy_field_get_unit_z(dfield);
+    power10 = gwy_unit_set_from_string(zunit, unitstr);
 
 #if 0
     /* This is what we apparently should be doing for topography but it does not work. */
@@ -560,16 +552,16 @@ jspm_add_data_field(const JSPMFile *jspmfile,
     }
     else {
         g_warning("Unknown data type %u.%u, importing as raw.", jspmfile->signal_name, jspmfile->unit);
-        q = 1.0/32767.0 * pow10(power10);
+        q = 1.0/32767.0 * gwy_exp10(power10);
     }
 
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
 
     gwy_convert_raw_data(buffer + jspmfile->data_offset, jspmfile->xres*jspmfile->yres, 1,
                          GWY_RAW_DATA_UINT32, GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                         gwy_data_field_get_data(dfield), q, z0);
-    gwy_container_pass_object(container, gwy_app_get_data_key_for_id(0), dfield);
-    gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(0), title);
+                         gwy_field_get_data(dfield), q, z0);
+    gwy_file_pass_image(file, 0, dfield);
+    gwy_file_set_title(file, GWY_FILE_IMAGE, 0, title, FALSE);
 }
 
 static void
@@ -588,15 +580,15 @@ format_meta(GwyContainer *meta,
 }
 
 static void
-jspm_add_meta(JSPMFile *jspmfile, GwyContainer *container)
+add_meta(JSPMFile *jspmfile, GwyFile *file)
 {
-    GwyContainer *meta = gwy_container_new();
+    GwyContainer *meta = gwy_container_new_in_construction();
 
     format_meta(meta, "WinSPM Version", "%.2f", jspmfile->winspm_version/100.0);
     if (jspmfile->comment)
         format_meta(meta, "Comment", "%s", jspmfile->comment);
 
-    gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(0), meta);
+    gwy_file_pass_meta(file, GWY_FILE_IMAGE, 0, meta);
 }
 
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
