@@ -1,6 +1,6 @@
 /*
- *  $Id: keyence.c 26873 2024-11-19 12:52:14Z yeti-dn $
- *  Copyright (C) 2015-2024 David Necas (Yeti).
+ *  $Id: keyence.c 28910 2025-11-24 18:11:09Z yeti-dn $
+ *  Copyright (C) 2015-2025 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
  *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
@@ -92,19 +92,14 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
 #include <string.h>
 #include <stdarg.h>
 #include <time.h>
 #include <stdio.h>
 #include <glib/gstdio.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/stats.h>
-#include <app/data-browser.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 #include "gwyzip.h"
 #include "get.h"
 #include "err.h"
@@ -349,13 +344,13 @@ static gint          keyence3_detect     (const GwyFileDetectInfo *fileinfo,
                                           gboolean only_name);
 static gint          keyence4_detect     (const GwyFileDetectInfo *fileinfo,
                                           gboolean only_name);
-static GwyContainer* keyence3_load       (const gchar *filename,
-                                          GwyRunType mode,
+static GwyFile*      keyence3_load       (const gchar *filename,
+                                          GwyRunModeFlags mode,
                                           GError **error);
-static GwyContainer* keyence4_load       (const gchar *filename,
-                                          GwyRunType mode,
+static GwyFile*      keyence4_load       (const gchar *filename,
+                                          GwyRunModeFlags mode,
                                           GError **error);
-static GwyContainer* keyence4_load_membuf(const guchar *buffer,
+static GwyFile*      keyence4_load_membuf(const guchar *buffer,
                                           gsize size,
                                           gint version,
                                           const gchar *filename,
@@ -385,16 +380,16 @@ static gboolean      read_line_meas      (Keyence4File *kfile,
                                           GError **error);
 static gboolean      read_character_strs (Keyence4File *kfile,
                                           GError **error);
-static GwyDataField* create_data_field   (const Keyence4FalseColorImage *image,
+static GwyField*     create_field        (const Keyence4FalseColorImage *image,
                                           const Keyence4MeasurementConditions *measconds,
                                           gboolean is_height);
-static GwyDataField* create_color_field  (const Keyence4TrueColorImage *image,
+static GwyField*     create_color_field  (const Keyence4TrueColorImage *image,
                                           const Keyence4MeasurementConditions *measconds,
                                           gint channelid);
 static GwyContainer* create_meta         (const Keyence4File *kfile);
-static void          add_data_field      (GwyContainer *data,
+static void          add_field           (GwyFile *file,
                                           gint *id,
-                                          GwyDataField *dfield,
+                                          GwyField *dfield,
                                           GwyContainer *meta,
                                           const gchar *title,
                                           const gchar *filename,
@@ -404,25 +399,25 @@ static void          add_data_field      (GwyContainer *data,
 #ifdef HAVE_GWYZIP
 static gint          keyence6_detect           (const GwyFileDetectInfo *fileinfo,
                                                 gboolean only_name);
-static GwyContainer* keyence6_load             (const gchar *filename,
-                                                GwyRunType mode,
+static GwyFile*      keyence6_load             (const gchar *filename,
+                                                GwyRunModeFlags mode,
                                                 GError **error);
 static gint          keyence7_detect           (const GwyFileDetectInfo *fileinfo,
                                                 gboolean only_name);
-static GwyContainer* keyence7_load             (const gchar *filename,
-                                                GwyRunType mode,
+static GwyFile*      keyence7_load             (const gchar *filename,
+                                                GwyRunModeFlags mode,
                                                 GError **error);
-static GwyContainer* keyence67_load_common     (const gchar *filename,
+static GwyFile*      keyence67_load_common     (const gchar *filename,
                                                 gint version,
                                                 GError **error);
-static void          add_vk6_hdr_images        (GwyContainer *data,
+static void          add_vk6_hdr_images        (GwyFile *data,
                                                 GwyZipFile zipfile,
                                                 const gchar *filename);
 static gboolean      read_vk6_hdr_images       (const guchar *buffer,
                                                 gsize size,
-                                                GwyDataField **fields,
+                                                GwyField **fields,
                                                 guint nf);
-static void          distribute_meta6          (GwyContainer *data,
+static void          distribute_meta6          (GwyFile *data,
                                                 GwyContainer *addmeta);
 static GwyContainer* read_vk6_measure_condition(GwyZipFile zipfile);
 static GwyContainer* parse_xml_metadata        (const gchar *buffer,
@@ -441,7 +436,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports Keyence VK3, VK4, VK6 and VK7 files."),
     "Yeti <yeti@gwyddion.net>",
-    "2.5",
+    "3.0",
     "David Nečas (Yeti)",
     "2015",
 };
@@ -453,29 +448,17 @@ module_register(void)
 {
     gwy_file_func_register("keyence3",
                            N_("Keyence VK3 data files (.vk3)"),
-                           (GwyFileDetectFunc)&keyence3_detect,
-                           (GwyFileLoadFunc)&keyence3_load,
-                           NULL,
-                           NULL);
+                           &keyence3_detect, &keyence3_load, NULL, NULL);
     gwy_file_func_register("keyence4",
                            N_("Keyence VK4 data files (.vk4)"),
-                           (GwyFileDetectFunc)&keyence4_detect,
-                           (GwyFileLoadFunc)&keyence4_load,
-                           NULL,
-                           NULL);
+                           &keyence4_detect, &keyence4_load, NULL, NULL);
 #ifdef HAVE_GWYZIP
     gwy_file_func_register("keyence6",
                            N_("Keyence VK6 data files (.vk6)"),
-                           (GwyFileDetectFunc)&keyence6_detect,
-                           (GwyFileLoadFunc)&keyence6_load,
-                           NULL,
-                           NULL);
+                           &keyence6_detect, &keyence6_load, NULL, NULL);
     gwy_file_func_register("keyence7",
                            N_("Keyence VK7 data files (.vk7)"),
-                           (GwyFileDetectFunc)&keyence7_detect,
-                           (GwyFileLoadFunc)&keyence7_load,
-                           NULL,
-                           NULL);
+                           &keyence7_detect, &keyence7_load, NULL, NULL);
 #endif
     return TRUE;
 }
@@ -542,12 +525,12 @@ keyence7_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 }
 #endif
 
-static GwyContainer*
+static GwyFile*
 keyence3_load(const gchar *filename,
-              G_GNUC_UNUSED GwyRunType mode,
+              G_GNUC_UNUSED GwyRunModeFlags mode,
               GError **error)
 {
-    GwyContainer *data;
+    GwyFile *data;
     GError *err = NULL;
     guchar *buffer;
     gsize size;
@@ -561,12 +544,12 @@ keyence3_load(const gchar *filename,
     return data;
 }
 
-static GwyContainer*
+static GwyFile*
 keyence4_load(const gchar *filename,
-              G_GNUC_UNUSED GwyRunType mode,
+              G_GNUC_UNUSED GwyRunModeFlags mode,
               GError **error)
 {
-    GwyContainer *data;
+    GwyFile *data;
     GError *err = NULL;
     guchar *buffer;
     gsize size;
@@ -580,7 +563,7 @@ keyence4_load(const gchar *filename,
     return data;
 }
 
-static GwyContainer*
+static GwyFile*
 keyence4_load_membuf(const guchar *buffer,
                      gsize size,
                      gint version,
@@ -588,8 +571,9 @@ keyence4_load_membuf(const guchar *buffer,
                      GError **error)
 {
     Keyence4File *kfile;
-    GwyDataField *dfield;
-    GwyContainer *data = NULL, *meta = NULL;
+    GwyField *dfield;
+    GwyFile *data = NULL;
+    GwyContainer *meta = NULL;
     const guchar *p;
     guint i, id;
 
@@ -614,35 +598,35 @@ keyence4_load_membuf(const guchar *buffer,
         goto fail;
     }
 
-    data = gwy_container_new();
+    data = gwy_file_new_in_construction();
     meta = create_meta(kfile);
     id = 0;
 
     for (i = 0; i < G_N_ELEMENTS(kfile->height); i++) {
         if (kfile->height[i].data) {
-            dfield = create_data_field(&kfile->height[i], &kfile->meas_conds, TRUE);
-            add_data_field(data, &id, dfield, meta, "Height", filename, i, NULL);
+            dfield = create_field(&kfile->height[i], &kfile->meas_conds, TRUE);
+            add_field(data, &id, dfield, meta, "Height", filename, i, NULL);
         }
     }
 
     for (i = 0; i < G_N_ELEMENTS(kfile->light); i++) {
         if (kfile->light[i].data) {
-            dfield = create_data_field(&kfile->light[i], &kfile->meas_conds, FALSE);
-            add_data_field(data, &id, dfield, meta, "Light", filename, i, NULL);
+            dfield = create_field(&kfile->light[i], &kfile->meas_conds, FALSE);
+            add_field(data, &id, dfield, meta, "Light", filename, i, NULL);
         }
     }
 
     if (kfile->color_peak.data) {
         for (i = 0; i < 3; i++) {
             dfield = create_color_field(&kfile->color_peak, &kfile->meas_conds, i);
-            add_data_field(data, &id, dfield, meta, peaknames[i], filename, -1, gradientnames[i]);
+            add_field(data, &id, dfield, meta, peaknames[i], filename, -1, gradientnames[i]);
         }
     }
 
     if (kfile->color_light.data) {
         for (i = 0; i < 3; i++) {
             dfield = create_color_field(&kfile->color_light, &kfile->meas_conds, i);
-            add_data_field(data, &id, dfield, meta, lightnames[i], filename, -1, gradientnames[i]);
+            add_field(data, &id, dfield, meta, lightnames[i], filename, -1, gradientnames[i]);
         }
     }
 
@@ -1155,22 +1139,22 @@ read_character_strs(Keyence4File *kfile,
     return TRUE;
 }
 
-static GwyDataField*
-create_data_field(const Keyence4FalseColorImage *image,
-                  const Keyence4MeasurementConditions *measconds,
-                  gboolean is_height)
+static GwyField*
+create_field(const Keyence4FalseColorImage *image,
+             const Keyence4MeasurementConditions *measconds,
+             gboolean is_height)
 {
     guint w = image->width, h = image->height;
     gdouble dx = measconds->x_length_per_pixel * Picometre;
     gdouble dy = measconds->y_length_per_pixel * Picometre;
     GwyRawDataType datatype = GWY_RAW_DATA_UINT8;
-    GwyDataField *dfield;
+    GwyField *dfield;
     gdouble *data;
     gdouble q;
 
     /* The -1 is from comparison with original software. It makes users happy they see the same image dimensions but
      * it can mean all lateral distances are actually off because we read them differently. */
-    dfield = gwy_data_field_new(w, h, dx*(w - 1.0), dy*(h - 1.0), FALSE);
+    dfield = gwy_field_new(w, h, dx*(w - 1.0), dy*(h - 1.0), FALSE);
     if (image->bit_depth == 16)
         datatype = GWY_RAW_DATA_UINT16;
     else if (image->bit_depth == 32)
@@ -1178,17 +1162,17 @@ create_data_field(const Keyence4FalseColorImage *image,
 
     q = (is_height ? measconds->z_length_per_digit * Picometre : gwy_powi(0.5, image->bit_depth));
 
-    data = gwy_data_field_get_data(dfield);
+    data = gwy_field_get_data(dfield);
     gwy_convert_raw_data(image->data, w*h, 1, datatype, GWY_BYTE_ORDER_LITTLE_ENDIAN, data, q, 0.0);
 
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
     if (is_height)
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), "m");
+        gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), "m");
 
     return dfield;
 }
 
-static GwyDataField*
+static GwyField*
 create_color_field(const Keyence4TrueColorImage *image,
                    const Keyence4MeasurementConditions *measconds,
                    gint channelid)
@@ -1196,17 +1180,17 @@ create_color_field(const Keyence4TrueColorImage *image,
     guint w = image->width, h = image->height;
     gdouble dx = measconds->x_length_per_pixel * Picometre;
     gdouble dy = measconds->y_length_per_pixel * Picometre;
-    GwyDataField *dfield;
+    GwyField *dfield;
     gdouble *data;
 
     /* The -1 is from comparison with original software. It makes users happy they see the same image dimensions but
      * it can mean all lateral distances are actually off because we read them differently. */
-    dfield = gwy_data_field_new(w, h, dx*(w - 1.0), dy*(h - 1.0), FALSE);
-    data = gwy_data_field_get_data(dfield);
+    dfield = gwy_field_new(w, h, dx*(w - 1.0), dy*(h - 1.0), FALSE);
+    data = gwy_field_get_data(dfield);
     gwy_convert_raw_data(image->data + channelid, w*h, 3, GWY_RAW_DATA_UINT8, GWY_BYTE_ORDER_LITTLE_ENDIAN,
                          data, 1.0/255.0, 0.0);
 
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
     return dfield;
 }
 
@@ -1235,7 +1219,7 @@ create_meta(const Keyence4File *kfile)
 {
     const Keyence4MeasurementConditions *measconds = &kfile->meas_conds;
     const Keyence4CharacterStrings *charstrs = &kfile->char_strs;
-    GwyContainer *meta = gwy_container_new();
+    GwyContainer *meta = gwy_container_new_in_construction();
     gchar buf[48];
 
     g_snprintf(buf, sizeof(buf), "%u.%u.%u.%u",
@@ -1310,57 +1294,54 @@ create_meta(const Keyence4File *kfile)
 }
 
 static void
-add_data_field(GwyContainer *data, gint *id,
-               GwyDataField *dfield, GwyContainer *meta,
-               const gchar *title, const gchar *filename,
-               gint i, const gchar *gradient)
+add_field(GwyFile *file, gint *id,
+          GwyField *dfield, GwyContainer *meta,
+          const gchar *title, const gchar *filename,
+          gint i, const gchar *gradient)
 {
-    GwyContainer *tmpmeta;
-
-    gwy_container_pass_object(data, gwy_app_get_data_key_for_id(*id), dfield);
-    gwy_file_channel_import_log_add(data, *id, NULL, filename);
+    gwy_file_pass_image(file, *id, dfield);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, *id, NULL, filename);
 
     if (i >= 0) {
         gchar *t = g_strdup_printf("%s %u", title, i);
-        gwy_container_set_string(data, gwy_app_get_data_title_key_for_id(*id), t);
+        gwy_file_pass_title(file, GWY_FILE_IMAGE, *id, t);
     }
     else
-        gwy_container_set_const_string(data, gwy_app_get_data_title_key_for_id(*id), title);
+        gwy_file_set_title(file, GWY_FILE_IMAGE, *id, title, FALSE);
 
-    if (meta) {
-        tmpmeta = gwy_container_duplicate(meta);
-        gwy_container_pass_object(data, gwy_app_get_data_meta_key_for_id(*id), tmpmeta);
-    }
+    if (meta)
+        gwy_file_pass_meta(file, GWY_FILE_IMAGE, *id, gwy_container_copy(meta));
 
     if (gradient)
-        gwy_container_set_const_string(data, gwy_app_get_data_palette_key_for_id(*id), gradient);
+        gwy_file_set_palette(file, GWY_FILE_IMAGE, *id, gradient);
 
     (*id)++;
 }
 
 #ifdef HAVE_GWYZIP
-static GwyContainer*
+static GwyFile*
 keyence6_load(const gchar *filename,
-              G_GNUC_UNUSED GwyRunType mode,
+              G_GNUC_UNUSED GwyRunModeFlags mode,
               GError **error)
 {
     return keyence67_load_common(filename, 6, error);
 }
 
-static GwyContainer*
+static GwyFile*
 keyence7_load(const gchar *filename,
-              G_GNUC_UNUSED GwyRunType mode,
+              G_GNUC_UNUSED GwyRunModeFlags mode,
               GError **error)
 {
     return keyence67_load_common(filename, 7, error);
 }
 
-static GwyContainer*
+static GwyFile*
 keyence67_load_common(const gchar *filename,
                       gint version,
                       GError **error)
 {
-    GwyContainer *data = NULL, *meta = NULL;
+    GwyFile *data = NULL;
+    GwyContainer *meta = NULL;
     GError *err = NULL;
     gchar *zipfilename = NULL;
     guchar *buffer = NULL, *vk4buf = NULL;
@@ -1427,7 +1408,7 @@ keyence67_load_common(const gchar *filename,
         distribute_meta6(data, meta);
 
 fail:
-    GWY_OBJECT_UNREF(meta);
+    g_clear_object(&meta);
     g_free(vk4buf);
     if (zipfile)
         gwyzip_close(zipfile);
@@ -1441,10 +1422,10 @@ fail:
 }
 
 static void
-add_vk6_hdr_images(GwyContainer *data, GwyZipFile zipfile, const gchar *filename)
+add_vk6_hdr_images(GwyFile *data, GwyZipFile zipfile, const gchar *filename)
 {
     GwyZipFile hdrzipfile;
-    GwyDataField *rgbfield[3] = { NULL, NULL, NULL }, *errfield = NULL, *field = NULL, *mask;
+    GwyField *rgbfield[3] = { NULL, NULL, NULL }, *errfield = NULL, *field = NULL, *mask;
     GwyContainer *meta = NULL;
     guchar *buffer;
     gchar *hdrfilename;
@@ -1479,18 +1460,18 @@ add_vk6_hdr_images(GwyContainer *data, GwyZipFile zipfile, const gchar *filename
         g_free(buffer);
 
         /* Don't create masks if the error field is empty. */
-        if (gwy_data_field_get_max(errfield) <= 0.0)
-            GWY_OBJECT_UNREF(errfield);
+        if (gwy_field_get_max(errfield) <= 0.0)
+            g_clear_object(&errfield);
     }
 
-    ids = gwy_app_data_browser_get_data_ids(data);
+    ids = gwy_file_get_ids(data, GWY_FILE_IMAGE);
     id = -1;
     for (i = 0; ids[i] >= 0; i++) {
         id = MAX(id, ids[i]);
         if (!field)
-            field = gwy_container_get_object(data, gwy_app_get_data_key_for_id(ids[i]));
+            field = gwy_file_get_image(data, ids[i]);
         if (!meta)
-            gwy_container_gis_object(data, gwy_app_get_data_meta_key_for_id(ids[i]), (GObject**)&meta);
+            gwy_file_get_meta(data, GWY_FILE_IMAGE, ids[i]);
     }
     id++;
     g_free(ids);
@@ -1499,28 +1480,28 @@ add_vk6_hdr_images(GwyContainer *data, GwyZipFile zipfile, const gchar *filename
         if (!rgbfield[i])
             continue;
         if (field) {
-            gwy_data_field_copy_units(field, rgbfield[i]);
-            gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(rgbfield[i]), NULL);
-            gwy_data_field_set_xreal(rgbfield[i], gwy_data_field_get_xreal(field));
-            gwy_data_field_set_yreal(rgbfield[i], gwy_data_field_get_xreal(field));
+            gwy_field_copy_units(field, rgbfield[i]);
+            gwy_unit_clear(gwy_field_get_unit_z(rgbfield[i]));
+            gwy_field_set_xreal(rgbfield[i], gwy_field_get_xreal(field));
+            gwy_field_set_yreal(rgbfield[i], gwy_field_get_xreal(field));
         }
         /* This unrefs rgbfield[i]. */
-        add_data_field(data, &id, rgbfield[i], meta, hdrnames[i], filename, -1, gradientnames[i]);
+        add_field(data, &id, rgbfield[i], meta, hdrnames[i], filename, -1, gradientnames[i]);
     }
 
-    ids = gwy_app_data_browser_get_data_ids(data);
+    ids = gwy_file_get_ids(data, GWY_FILE_IMAGE);
     for (i = 0; ids[i] >= 0; i++) {
-        if (errfield && !gwy_data_field_check_compatibility(field, errfield, GWY_DATA_COMPATIBILITY_RES)) {
-            mask = gwy_data_field_new_alike(field, FALSE);
-            gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(mask), NULL);
-            gwy_data_field_copy(errfield, mask, FALSE);
+        if (errfield && !gwy_field_is_incompatible(field, errfield, GWY_DATA_MISMATCH_RES)) {
+            mask = gwy_field_new_alike(field, FALSE);
+            gwy_unit_clear(gwy_field_get_unit_z(mask));
+            gwy_field_copy_data(errfield, mask);
             /* FIXME: Should we apply Laplace interpolation here, as usual? */
-            gwy_container_pass_object(data, gwy_app_get_mask_key_for_id(ids[i]), mask);
+            gwy_file_pass_image_mask(data, ids[i], mask);
         }
     }
     g_free(ids);
 
-    GWY_OBJECT_UNREF(errfield);
+    g_clear_object(&errfield);
 
     gwyzip_close(hdrzipfile);
     g_unlink(hdrfilename);
@@ -1529,7 +1510,7 @@ add_vk6_hdr_images(GwyContainer *data, GwyZipFile zipfile, const gchar *filename
 
 static gboolean
 read_vk6_hdr_images(const guchar *buffer, gsize size,
-                    GwyDataField **fields, guint nf)
+                    GwyField **fields, guint nf)
 {
     guint xres, yres, bpr, bps, rowstride, i;
     GwyRawDataType rawtype;
@@ -1570,30 +1551,26 @@ read_vk6_hdr_images(const guchar *buffer, gsize size,
     }
 
     for (i = 0; i < nf; i++) {
-        fields[i] = gwy_data_field_new(xres, yres, xres, yres, FALSE);
+        fields[i] = gwy_field_new(xres, yres, xres, yres, FALSE);
         gwy_convert_raw_data(p + i*bps, xres*yres, nf, rawtype, GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                             gwy_data_field_get_data(fields[i]), 1.0, 0.0);
+                             gwy_field_get_data(fields[i]), 1.0, 0.0);
     }
     gwy_debug("%u images read OK", nf);
     return TRUE;
 }
 
 static void
-distribute_meta6(GwyContainer *data, GwyContainer *addmeta)
+distribute_meta6(GwyFile *file, GwyContainer *addmeta)
 {
-    gint *ids = gwy_app_data_browser_get_data_ids(data);
-    gint i;
+    gint *ids = gwy_file_get_ids(file, GWY_FILE_IMAGE);
 
-    for (i = 0; ids[i] >= 0; i++) {
-        GQuark quark = gwy_app_get_data_meta_key_for_id(ids[i]);
+    for (gint i = 0; ids[i] >= 0; i++) {
         GwyContainer *meta;
 
-        if (gwy_container_gis_object(data, quark, &meta))
-            gwy_container_transfer(addmeta, meta, "", "", FALSE);
-        else {
-            meta = gwy_container_duplicate(addmeta);
-            gwy_container_pass_object(data, quark, meta);
-        }
+        if ((meta = gwy_file_get_meta(file, GWY_FILE_IMAGE, ids[i])))
+            gwy_container_transfer(addmeta, meta, "", "", FALSE, FALSE);
+        else
+            gwy_file_pass_meta(file, GWY_FILE_IMAGE, ids[i], gwy_container_copy(addmeta));
     }
     g_free(ids);
 }
@@ -1751,7 +1728,7 @@ parse_xml_metadata(const gchar *buffer, gsize size)
     GMarkupParseContext *context = NULL;
     Keyence6Meta vk6meta;
 
-    vk6meta.meta = gwy_container_new();
+    vk6meta.meta = gwy_container_new_in_construction();
     vk6meta.path = g_string_new(NULL);
     vk6meta.curr_element = g_string_new(NULL);
     vk6meta.compdepths = g_array_new(FALSE, FALSE, sizeof(gint));
@@ -1766,7 +1743,7 @@ parse_xml_metadata(const gchar *buffer, gsize size)
     g_string_free(vk6meta.curr_element, TRUE);
     g_array_free(vk6meta.compdepths, TRUE);
     if (!gwy_container_get_n_items(vk6meta.meta))
-        GWY_OBJECT_UNREF(vk6meta.meta);
+        g_clear_object(&vk6meta.meta);
 
     return vk6meta.meta;
 }
