@@ -1,6 +1,6 @@
 /*
- *  $Id: leica.c 26519 2024-08-15 18:04:20Z yeti-dn $
- *  Copyright (C) 2014 Daniil Bratashov (dn2010), David Necas (Yeti)..
+ *  $Id: leica.c 28848 2025-11-10 18:54:08Z yeti-dn $
+ *  Copyright (C) 2014-2025 Daniil Bratashov (dn2010), David Necas (Yeti).
  *
  *  E-mail: dn2010@gmail.com.
  *
@@ -46,19 +46,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/datafield.h>
-#include <libprocess/spectra.h>
-#include <libprocess/brick.h>
-#include <libgwydgets/gwygraphmodel.h>
-#include <libgwydgets/gwygraphbasics.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
 #include "err.h"
@@ -139,56 +130,56 @@ typedef struct {
     GArray *elements;
 } XMLParserData;
 
-static gboolean      module_register     (void);
-static gint          lif_detect          (const GwyFileDetectInfo *fileinfo,
-                                          gboolean only_name);
-static GwyContainer* lif_load            (const gchar *filename,
-                                          GwyRunType mode,
-                                          GError **error);
-static LIFFile*      parse_xml_header    (LIFHeader *header,
-                                          GError **error);
-static const gchar*  lut_name_to_palette (const gchar *lut);
-static gboolean      create_images       (LIFElement *element,
-                                          LIFMemBlock *memblock,
-                                          gint *channelno,
-                                          GwyContainer *container,
-                                          const gchar *filename,
-                                          GError **error);
-static gboolean      create_volumes      (LIFElement *element,
-                                          LIFMemBlock *memblock,
-                                          gint *channelno,
-                                          GwyContainer *container,
-                                          const gchar *filename,
-                                          GError **error);
-static void          lif_file_free       (LIFFile *file);
-static LIFMemBlock*  lif_read_memblock   (const guchar *buffer,
-                                          gsize *size,
-                                          gint version);
-static gboolean      lif_remove_memblock (gpointer key,
-                                          gpointer value,
-                                          gpointer user_data);
-static void          header_start_element(GMarkupParseContext *context,
-                                          const gchar *element_name,
-                                          const gchar **attribute_names,
-                                          const gchar **attribute_values,
-                                          gpointer user_data,
-                                          GError **error);
-static void          header_end_element  (GMarkupParseContext *context,
-                                          const gchar *element_name,
-                                          gpointer user_data,
-                                          GError **error);
-static void          header_parse_text   (GMarkupParseContext *context,
-                                          const gchar *text,
-                                          gsize text_len,
-                                          gpointer user_data,
-                                          GError **error);
+static gboolean     module_register     (void);
+static gint         detect_file         (const GwyFileDetectInfo *fileinfo,
+                                         gboolean only_name);
+static GwyFile*     load_file           (const gchar *filename,
+                                         GwyRunModeFlags mode,
+                                         GError **error);
+static LIFFile*     parse_xml_header    (LIFHeader *header,
+                                         GError **error);
+static const gchar* lut_name_to_palette (const gchar *lut);
+static gboolean     create_images       (LIFElement *element,
+                                         LIFMemBlock *memblock,
+                                         gint *channelno,
+                                         GwyFile *file,
+                                         const gchar *filename,
+                                         GError **error);
+static gboolean     create_volumes      (LIFElement *element,
+                                         LIFMemBlock *memblock,
+                                         gint *channelno,
+                                         GwyFile *file,
+                                         const gchar *filename,
+                                         GError **error);
+static void         lif_file_free       (LIFFile *file);
+static LIFMemBlock* lif_read_memblock   (const guchar *buffer,
+                                         gsize *size,
+                                         gint version);
+static gboolean     lif_remove_memblock (gpointer key,
+                                         gpointer value,
+                                         gpointer user_data);
+static void         header_start_element(GMarkupParseContext *context,
+                                         const gchar *element_name,
+                                         const gchar **attribute_names,
+                                         const gchar **attribute_values,
+                                         gpointer user_data,
+                                         GError **error);
+static void         header_end_element  (GMarkupParseContext *context,
+                                         const gchar *element_name,
+                                         gpointer user_data,
+                                         GError **error);
+static void         header_parse_text   (GMarkupParseContext *context,
+                                         const gchar *text,
+                                         gsize text_len,
+                                         gpointer user_data,
+                                         GError **error);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Leica CLSM image files (LIF)."),
     "Daniil Bratashov <dn2010@gmail.com>",
-    "0.6",
+    "0.7",
     "Daniil Bratashov (dn2010), David Necas (Yeti)",
     "2016",
 };
@@ -200,17 +191,13 @@ module_register(void)
 {
     gwy_file_func_register("leica",
                            N_("Leica LIF image files (.lif)"),
-                           (GwyFileDetectFunc)&lif_detect,
-                           (GwyFileLoadFunc)&lif_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-lif_detect(const GwyFileDetectInfo *fileinfo,
-           gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     gint score = 0;
 
@@ -224,15 +211,15 @@ lif_detect(const GwyFileDetectInfo *fileinfo,
     return score;
 }
 
-static GwyContainer*
-lif_load(const gchar *filename,
-         G_GNUC_UNUSED GwyRunType mode,
-         GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     LIFHeader *header = NULL;
     LIFMemBlock *memblock = NULL;
-    LIFFile *file = NULL;
+    LIFFile *liffile = NULL;
     LIFElement *element = NULL;
     LIFDimension *dimension = NULL;
     gsize size = 0, memblock_size = 0;
@@ -284,13 +271,13 @@ lif_load(const gchar *filename,
     // gwy_debug("%s", header->xmlheader);
 
     /* Parse XML header */
-    if (!(file = parse_xml_header(header, error)))
+    if (!(liffile = parse_xml_header(header, error)))
         goto fail;
 
     /* Reading memblocks */
-    file->memblocks = g_hash_table_new(g_str_hash, g_str_equal);
+    liffile->memblocks = g_hash_table_new(g_str_hash, g_str_equal);
     while (remaining > 0) {
-        memblock = lif_read_memblock(p, &memblock_size, file->version);
+        memblock = lif_read_memblock(p, &memblock_size, liffile->version);
         if (!memblock) {
             break;
         }
@@ -298,14 +285,14 @@ lif_load(const gchar *filename,
         if (remaining >= 0) {
             gwy_debug("remaining = %" G_GUINT64_FORMAT "", remaining);
             p += memblock_size;
-            g_hash_table_insert(file->memblocks, memblock->memid, memblock);
+            g_hash_table_insert(liffile->memblocks, memblock->memid, memblock);
         }
     }
 
     /* Create data objects. */
-    container = gwy_container_new();
-    for (i = 0; i < file->elements->len; i++) {
-        element = &g_array_index(file->elements, LIFElement, i);
+    file = gwy_file_new_in_construction();
+    for (i = 0; i < liffile->elements->len; i++) {
+        element = &g_array_index(liffile->elements, LIFElement, i);
 
         if ((element->dimensions == NULL) || (element->channels == NULL)) {
             gwy_debug("Empty element");
@@ -338,7 +325,7 @@ lif_load(const gchar *filename,
             }
         }
 
-        memblock = (LIFMemBlock *)g_hash_table_lookup(file->memblocks, element->memid);
+        memblock = (LIFMemBlock*)g_hash_table_lookup(liffile->memblocks, element->memid);
         if (!memblock) {
             gwy_debug("Failed to locate memblock with key %s", element->memid);
             continue;
@@ -346,12 +333,12 @@ lif_load(const gchar *filename,
 
         if (element->dimensions->len == 2) {
             /* Image */
-            if (!create_images(element, memblock, &imageno, container, filename, error))
+            if (!create_images(element, memblock, &imageno, file, filename, error))
                 goto fail;
         }
         else if (element->dimensions->len == 3 || (element->dimensions->len == 4 && res == 1)) {
             /* Volume */
-            if (!create_volumes(element, memblock, &volumeno, container, filename, error))
+            if (!create_volumes(element, memblock, &volumeno, file, filename, error))
                 goto fail;
         }
     }
@@ -360,33 +347,33 @@ lif_load(const gchar *filename,
 fail:
     /* freeing all stuff */
     g_free(buffer);
-    lif_file_free(file);
+    lif_file_free(liffile);
     if (header)
         g_free(header->xmlheader);
     g_free(header);
 
     if (!ok)
-        GWY_OBJECT_UNREF(container);
-    else if (!gwy_container_get_n_items(container)) {
+        g_clear_object(&file);
+    else if (!gwy_container_get_n_items(GWY_CONTAINER(file))) {
         err_NO_DATA(error);
-        GWY_OBJECT_UNREF(container);
+        g_clear_object(&file);
     }
 
-    return container;
+    return file;
 }
 
 static gboolean
 create_images(LIFElement *element, LIFMemBlock *memblock, gint *channelno,
-              GwyContainer *container, const gchar *filename,
+              GwyFile *file, const gchar *filename,
               GError **error)
 {
     gdouble zscale = 1.0;
-    GwySIUnit *siunitxy = NULL, *siunitz = NULL;
+    GwyUnit *siunitxy = NULL, *siunitz = NULL;
     gint power10xy = 0, power10z = 0;
     LIFDimension *dimension = NULL;
     LIFChannel *channel = NULL;
     const guchar *tp, *p = memblock->data;
-    GwyDataField *dfield = NULL;
+    GwyField *dfield = NULL;
     gdouble *data = NULL;
     const gchar *lutname;
     gint x, xres, xstep, y, yres, ystep, offset;
@@ -399,7 +386,7 @@ create_images(LIFElement *element, LIFMemBlock *memblock, gint *channelno,
         xreal = dimension->length;
         xoffset = dimension->origin;
         xstep = dimension->bytesinc;
-        siunitxy = gwy_si_unit_new_parse(dimension->unit, &power10xy);
+        siunitxy = gwy_unit_new_parse(dimension->unit, &power10xy);
 
         dimension = &g_array_index(element->dimensions, LIFDimension, 1);
         yres = dimension->res;
@@ -414,9 +401,9 @@ create_images(LIFElement *element, LIFMemBlock *memblock, gint *channelno,
 
         channel = &g_array_index(element->channels, LIFChannel, j);
         offset = channel->bytesinc;
-        siunitz = gwy_si_unit_new_parse(channel->unit, &power10z);
+        siunitz = gwy_unit_new_parse(channel->unit, &power10z);
 
-        zscale = pow10(power10z);
+        zscale = gwy_exp10(power10z);
         if (offset + (xres - 1)*xstep + (yres - 1)*ystep > memblock->memsize) {
             gwy_debug("Memblock too small");
             gwy_debug("%d %" G_GUINT64_FORMAT "",
@@ -426,13 +413,13 @@ create_images(LIFElement *element, LIFMemBlock *memblock, gint *channelno,
             return FALSE;
         }
 
-        dfield = gwy_data_field_new(xres, yres,
-                                    xreal*pow10(power10xy), yreal*pow10(power10xy),
-                                    TRUE);
-        gwy_data_field_set_xoffset(dfield, xoffset*pow10(power10xy));
-        gwy_data_field_set_yoffset(dfield, yoffset*pow10(power10xy));
+        dfield = gwy_field_new(xres, yres,
+                               xreal*gwy_exp10(power10xy), yreal*gwy_exp10(power10xy),
+                               TRUE);
+        gwy_field_set_xoffset(dfield, xoffset*gwy_exp10(power10xy));
+        gwy_field_set_yoffset(dfield, yoffset*gwy_exp10(power10xy));
 
-        data = gwy_data_field_get_data(dfield);
+        data = gwy_field_get_data(dfield);
         if (xstep == 1) {
             for (y = 0; y < yres; y++) {
                 for (x = 0; x < xres; x++) {
@@ -450,25 +437,23 @@ create_images(LIFElement *element, LIFMemBlock *memblock, gint *channelno,
         }
 
         if (siunitxy) {
-            gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(dfield), siunitxy);
+            gwy_unit_assign(gwy_field_get_unit_xy(dfield), siunitxy);
             g_object_unref(siunitxy);
         }
         if (siunitz) {
-            gwy_si_unit_assign(gwy_data_field_get_si_unit_z(dfield), siunitz);
+            gwy_unit_assign(gwy_field_get_unit_z(dfield), siunitz);
             g_object_unref(siunitz);
         }
 
-        gwy_container_pass_object(container, gwy_app_get_data_key_for_id(*channelno), dfield);
+        gwy_file_pass_image(file, *channelno, dfield);
         if (element->name)
-            gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(*channelno), element->name);
-        if (element->metadata) {
-            gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(*channelno),
-                                      gwy_container_duplicate(element->metadata));
-        }
+            gwy_file_set_title(file, GWY_FILE_IMAGE, *channelno, element->name, FALSE);
+        if (element->metadata)
+            gwy_file_pass_meta(file, GWY_FILE_IMAGE, *channelno, gwy_container_copy(element->metadata));
         if ((lutname = lut_name_to_palette(channel->lut)))
-            gwy_container_set_const_string(container, gwy_app_get_data_palette_key_for_id(*channelno), lutname);
+            gwy_file_set_palette(file, GWY_FILE_IMAGE, *channelno, lutname);
 
-        gwy_file_channel_import_log_add(container, *channelno, NULL, filename);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, *channelno, NULL, filename);
 
         (*channelno)++;
     }
@@ -478,11 +463,11 @@ create_images(LIFElement *element, LIFMemBlock *memblock, gint *channelno,
 
 static gboolean
 create_volumes(LIFElement *element, LIFMemBlock *memblock, gint *channelno,
-               GwyContainer *container, const gchar *filename,
+               GwyFile *file, const gchar *filename,
                GError **error)
 {
     gdouble wscale = 1.0;
-    GwySIUnit *siunitx = NULL, *siunity = NULL, *siunitz = NULL, *siunitw = NULL;
+    GwyUnit *siunitx = NULL, *siunity = NULL, *siunitz = NULL, *siunitw = NULL;
     gint power10x = 0, power10y = 0, power10z = 0, power10w = 0;
     LIFDimension *dimension = NULL;
     LIFChannel *channel = NULL;
@@ -502,7 +487,7 @@ create_volumes(LIFElement *element, LIFMemBlock *memblock, gint *channelno,
         gwy_debug("xreal = %g", xreal);
         xoffset = dimension->origin;
         xstep = dimension->bytesinc;
-        siunitx = gwy_si_unit_new_parse(dimension->unit, &power10x);
+        siunitx = gwy_unit_new_parse(dimension->unit, &power10x);
 
         dimension = &g_array_index(element->dimensions, LIFDimension, 1);
         yres = dimension->res;
@@ -510,7 +495,7 @@ create_volumes(LIFElement *element, LIFMemBlock *memblock, gint *channelno,
         gwy_debug("yreal = %g", yreal);
         yoffset = dimension->origin;
         ystep = dimension->bytesinc;
-        siunity = gwy_si_unit_new_parse(dimension->unit, &power10y);
+        siunity = gwy_unit_new_parse(dimension->unit, &power10y);
 
         if (element->dimensions->len == 3)
             dimension = &g_array_index(element->dimensions, LIFDimension, 2);
@@ -527,12 +512,12 @@ create_volumes(LIFElement *element, LIFMemBlock *memblock, gint *channelno,
             flipz = TRUE;
         }
         zstep = dimension->bytesinc;
-        siunitz = gwy_si_unit_new_parse(dimension->unit, &power10z);
+        siunitz = gwy_unit_new_parse(dimension->unit, &power10z);
 
         channel = &g_array_index(element->channels, LIFChannel, j);
         offset = channel->bytesinc;
-        siunitw = gwy_si_unit_new_parse(channel->unit, &power10w);
-        wscale = pow10(power10w);
+        siunitw = gwy_unit_new_parse(channel->unit, &power10w);
+        wscale = gwy_exp10(power10w);
 
         if (offset + (xres-1)*xstep + (yres-1)*ystep + (zres-1)*zstep > memblock->memsize) {
             gwy_debug("Memblock too small");
@@ -545,15 +530,15 @@ create_volumes(LIFElement *element, LIFMemBlock *memblock, gint *channelno,
             return FALSE;
         }
         brick = gwy_brick_new(xres, yres, zres,
-                              xreal*pow10(power10x), yreal*pow10(power10y), zreal*pow10(power10z),
+                              xreal*gwy_exp10(power10x), yreal*gwy_exp10(power10y), zreal*gwy_exp10(power10z),
                               TRUE);
-        gwy_brick_set_xoffset(brick, xoffset*pow10(power10x));
-        gwy_brick_set_yoffset(brick, yoffset*pow10(power10y));
-        gwy_brick_set_zoffset(brick, zoffset*pow10(power10z));
+        gwy_brick_set_xoffset(brick, xoffset*gwy_exp10(power10x));
+        gwy_brick_set_yoffset(brick, yoffset*gwy_exp10(power10y));
+        gwy_brick_set_zoffset(brick, zoffset*gwy_exp10(power10z));
 
         data = gwy_brick_get_data(brick);
 
-        /* FIXME: I would like to use gwy_convert_raw_data(), and perhaps gwy_brick_invert() if necessary.
+        /* FIXME: I would like to use gwy_convert_raw_data(), and perhaps gwy_brick_flip() if necessary.
          * But it might not be less messy than this at the end… */
         if (!flipz) {
             if (xstep == 1) {
@@ -599,34 +584,32 @@ create_volumes(LIFElement *element, LIFMemBlock *memblock, gint *channelno,
         }
 
         if (siunitx) {
-            gwy_si_unit_assign(gwy_brick_get_si_unit_x(brick), siunitx);
+            gwy_unit_assign(gwy_brick_get_unit_x(brick), siunitx);
             g_object_unref(siunitx);
         }
         if (siunity) {
-            gwy_si_unit_assign(gwy_brick_get_si_unit_y(brick), siunity);
+            gwy_unit_assign(gwy_brick_get_unit_y(brick), siunity);
             g_object_unref(siunity);
         }
         if (siunitz) {
-            gwy_si_unit_assign(gwy_brick_get_si_unit_z(brick), siunitz);
+            gwy_unit_assign(gwy_brick_get_unit_z(brick), siunitz);
             g_object_unref(siunitz);
         }
         if (siunitw) {
-            gwy_si_unit_assign(gwy_brick_get_si_unit_w(brick), siunitw);
+            gwy_unit_assign(gwy_brick_get_unit_w(brick), siunitw);
             g_object_unref(siunitw);
         }
 
-        gwy_container_pass_object(container, gwy_app_get_brick_key_for_id(*channelno), brick);
+        gwy_file_pass_volume(file, *channelno, brick);
 
         if (element->name)
-            gwy_container_set_const_string(container, gwy_app_get_brick_title_key_for_id(*channelno), element->name);
-        if (element->metadata) {
-            gwy_container_pass_object(container, gwy_app_get_brick_meta_key_for_id(*channelno),
-                                      gwy_container_duplicate(element->metadata));
-        }
+            gwy_file_set_title(file, GWY_FILE_VOLUME, *channelno, element->name, FALSE);
+        if (element->metadata)
+            gwy_file_pass_meta(file, GWY_FILE_VOLUME, *channelno, gwy_container_copy(element->metadata));
         if ((lutname = lut_name_to_palette(channel->lut)))
-            gwy_container_set_const_string(container, gwy_app_get_brick_palette_key_for_id(*channelno), lutname);
+            gwy_file_set_palette(file, GWY_FILE_VOLUME, *channelno, lutname);
 
-        gwy_file_volume_import_log_add(container, *channelno, NULL, filename);
+        gwy_log_add_import(file, GWY_FILE_VOLUME, *channelno, NULL, filename);
 
         (*channelno)++;
     }
@@ -684,7 +667,7 @@ lif_file_free(LIFFile *file)
 
             g_free(element->name);
             g_free(element->memid);
-            GWY_OBJECT_UNREF(element->metadata);
+            g_clear_object(&element->metadata);
         }
         g_array_free(file->elements, TRUE);
     }
@@ -833,7 +816,7 @@ header_start_element(G_GNUC_UNUSED GMarkupParseContext *context,
     else if (gwy_strequal(element_name, "Element")) {
         LIFElement elem;
 
-        gwy_clear(&elem, 1);
+        gwy_clear1(elem);
         while (*name_cursor) {
             if (gwy_strequal(*name_cursor, "Name"))
                 elem.name = g_strdup(*value_cursor);
@@ -869,7 +852,7 @@ header_start_element(G_GNUC_UNUSED GMarkupParseContext *context,
         if (!(element = get_last_element(data->elements, "ChannelDescription", error)))
             goto fail_xml;
 
-        gwy_clear(&channel, 1);
+        gwy_clear1(channel);
         while (*name_cursor) {
             if (gwy_strequal(*name_cursor, "Resolution"))
                 channel.res = atoi(*value_cursor);
@@ -899,7 +882,7 @@ header_start_element(G_GNUC_UNUSED GMarkupParseContext *context,
         if (!(element = get_last_element(data->elements, "DimensionDescription", error)))
             goto fail_xml;
 
-        gwy_clear(&dimension, 1);
+        gwy_clear1(dimension);
         while (*name_cursor) {
             if (gwy_strequal(*name_cursor, "DimID"))
                 dimension.dimid = atoi(*value_cursor);
@@ -927,7 +910,7 @@ header_start_element(G_GNUC_UNUSED GMarkupParseContext *context,
             goto fail_xml;
 
         if (!element->metadata)
-            element->metadata = gwy_container_new();
+            element->metadata = gwy_container_new_in_construction();
 
         while (*name_cursor) {
             gwy_container_set_const_string_by_name(element->metadata, *name_cursor, *value_cursor);
