@@ -1,6 +1,6 @@
 /*
- *  $Id: lextfile.c 28937 2025-12-02 15:09:01Z yeti-dn $
- *  Copyright (C) 2010-2019 David Necas (Yeti), Petr Klapetek.
+ *  $Id: lextfile.c 28938 2025-12-02 15:09:14Z yeti-dn $
+ *  Copyright (C) 2010-2025 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -35,11 +35,9 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/stats.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 #include "err.h"
 #include "gwytiff.h"
 
@@ -64,26 +62,26 @@ typedef struct {
     gdouble zcal;
 } LextFile;
 
-static gboolean      module_register   (void);
-static gint          lext_detect       (const GwyFileDetectInfo *fileinfo,
-                                        gboolean only_name);
-static GwyContainer* lext_load         (const gchar *filename,
-                                        GwyRunType mode,
-                                        GError **error);
-static GwyContainer* lext_load_tiff    (const GwyTIFF *tiff,
-                                        const gchar *filename,
-                                        GError **error);
-static const gchar*  guess_image0_title(const GwyTIFF *tiff);
-static void          add_info_from_exif(LextFile *lfile,
-                                        const GwyTIFF *tiff);
-static void          create_metadata   (LextFile *lfile);
+static gboolean     module_register   (void);
+static gint         detect_file       (const GwyFileDetectInfo *fileinfo,
+                                       gboolean only_name);
+static GwyFile*     load_file         (const gchar *filename,
+                                       GwyRunModeFlags mode,
+                                       GError **error);
+static GwyFile*     lext_load_tiff    (const GwyTIFF *tiff,
+                                       const gchar *filename,
+                                       GError **error);
+static const gchar* guess_image0_title(const GwyTIFF *tiff);
+static void         add_info_from_exif(LextFile *lfile,
+                                       const GwyTIFF *tiff);
+static void         create_metadata   (LextFile *lfile);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     module_register,
     N_("Imports LEXT data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.6.1",
+    "0.7.1",
     "David Nečas (Yeti) & Petr Klapetek",
     "2010",
 };
@@ -95,16 +93,13 @@ module_register(void)
 {
     gwy_file_func_register("lext",
                            N_("Olympus LEXT OLS4000 (.lext)"),
-                           (GwyFileDetectFunc)&lext_detect,
-                           (GwyFileLoadFunc)&lext_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-lext_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     GwyTIFF *tiff;
     gint score = 0;
@@ -132,22 +127,22 @@ lext_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
     return score;
 }
 
-static GwyContainer*
-lext_load(const gchar *filename,
-         G_GNUC_UNUSED GwyRunType mode,
-         GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
     GwyTIFF *tiff;
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
 
     tiff = gwy_tiff_load(filename, error);
     if (!tiff)
         return NULL;
 
-    container = lext_load_tiff(tiff, filename, error);
+    file = lext_load_tiff(tiff, filename, error);
     gwy_tiff_free(tiff);
 
-    return container;
+    return file;
 }
 
 static void
@@ -218,14 +213,14 @@ titlecase_channel_name(gchar *name)
     }
 }
 
-static GwyContainer*
+static GwyFile*
 lext_load_tiff(const GwyTIFF *tiff, const gchar *filename, GError **error)
 {
     const gchar *colour_channels[] = { "Red", "Green", "Blue" };
     const gchar *colour_channel_gradients[] = { "RGB-Red", "RGB-Green", "RGB-Blue" };
 
-    GwyContainer *container = NULL, *tmpmeta;
-    GwyDataField *dfield;
+    GwyFile *file = NULL;
+    GwyField *dfield;
     GwyTIFFImageReader *reader = NULL;
     GMarkupParser parser = { start_element, end_element, text, NULL, NULL };
     GHashTable *hash = NULL;
@@ -234,7 +229,7 @@ lext_load_tiff(const GwyTIFF *tiff, const gchar *filename, GError **error)
     gchar *comment = NULL, *title = NULL;
     const gchar *value, *image0title, *keytitle, *channeltitle, *unitstr;
     GError *err = NULL;
-    guint dir_num2 = 0, ndirs, ch, spp;
+    guint ch, spp;
     gint id = 0;
     GString *key;
 
@@ -263,13 +258,12 @@ lext_load_tiff(const GwyTIFF *tiff, const gchar *filename, GError **error)
     image0title = guess_image0_title(tiff);
     gwy_debug("guessed image0 type: %s", image0title ? image0title : "UNKNOWN");
 
-    /* Go through the directories twice. Extract "Height" during the first pass and everything else during the second
-     * one. */
-    ndirs = gwy_tiff_get_n_dirs(tiff);
-    for (dir_num2 = 0; dir_num2 < 2*ndirs; dir_num2++) {
-        gint dir_num = dir_num2 % ndirs;
+    /* Go through the TIFF directories twice. Extract "Height" during the first pass and everything else during the
+     * second one. */
+    guint ndirs = gwy_tiff_get_n_dirs(tiff);
+    for (guint dir_num2 = 0; dir_num2 < 2*ndirs; dir_num2++) {
+        guint dir_num = dir_num2 % ndirs;
         double xscale, yscale, zfactor;
-        GQuark quark;
         gdouble *data;
         gint i;
 
@@ -336,16 +330,16 @@ lext_load_tiff(const GwyTIFF *tiff, const gchar *filename, GError **error)
         }
         zfactor = g_ascii_strtod(value, NULL);
 
-        if (!container)
-            container = gwy_container_new();
+        if (!file)
+            file = gwy_file_new_in_construction();
 
         spp = reader->samples_per_pixel;
         for (ch = 0; ch < spp; ch++) {
-            dfield = gwy_data_field_new(reader->width, reader->height,
-                                        reader->width * xscale, reader->height * yscale,
-                                        FALSE);
+            dfield = gwy_field_new(reader->width, reader->height,
+                                   reader->width * xscale, reader->height * yscale,
+                                   FALSE);
             // units
-            gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
+            gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
 
             if (gwy_strequal(title, "Height")) {
                 unitstr = "m";
@@ -356,38 +350,31 @@ lext_load_tiff(const GwyTIFF *tiff, const gchar *filename, GError **error)
             else
                 unitstr = NULL;
 
-            gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), unitstr);
+            gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), unitstr);
 
-            data = gwy_data_field_get_data(dfield);
+            data = gwy_field_get_data(dfield);
             for (i = 0; i < reader->height; i++)
                 gwy_tiff_read_image_row(tiff, reader, ch, i, zfactor, 0.0, data + i*reader->width);
 
-            /* add read datafield to container */
-            quark = gwy_app_get_data_key_for_id(id);
-            gwy_container_pass_object(container, quark, dfield);
+            /* add read field to file */
+            gwy_file_pass_image(file, id, dfield);
 
-            quark = gwy_app_get_data_title_key_for_id(id);
             channeltitle = (spp == 3 ? colour_channels[ch] : title);
-            gwy_container_set_const_string(container, quark, channeltitle);
+            gwy_file_set_title(file, GWY_FILE_IMAGE, id, channeltitle, FALSE);
 
-            if (lfile.meta) {
-                tmpmeta = gwy_container_duplicate(lfile.meta);
-                quark = gwy_app_get_data_meta_key_for_id(id);
-                gwy_container_pass_object(container, quark, tmpmeta);
-            }
+            if (lfile.meta)
+                gwy_file_pass_meta(file, GWY_FILE_IMAGE, id, gwy_container_copy(lfile.meta));
 
-            if (spp == 3) {
-                g_string_printf(key, "/%u/base/palette", id);
-                gwy_container_set_const_string_by_name(container, key->str, colour_channel_gradients[ch]);
-            }
+            if (spp == 3)
+                gwy_file_set_palette(file, GWY_FILE_IMAGE, id, colour_channel_gradients[ch]);
 
-            gwy_file_channel_import_log_add(container, id, NULL, filename);
+            gwy_log_add_import(file, GWY_FILE_IMAGE, id, NULL, filename);
             id++;
         }
     }
 
 fail:
-    GWY_OBJECT_UNREF(lfile.meta);
+    g_clear_object(&lfile.meta);
     g_free(title);
     g_free(comment);
     if (reader) {
@@ -401,7 +388,7 @@ fail:
     if (context)
         g_markup_parse_context_free(context);
 
-    return container;
+    return file;
 }
 
 static const gchar*
@@ -545,9 +532,7 @@ fail:
 }
 
 static void
-add_metadata(gpointer hkey,
-             gpointer hvalue,
-             gpointer user_data)
+add_metadata(gpointer hkey, gpointer hvalue, gpointer user_data)
 {
     const gchar *key = (const gchar*)hkey, *value = (const gchar*)hvalue;
     LextFile *lfile = (LextFile*)user_data;
@@ -566,11 +551,11 @@ add_metadata(gpointer hkey,
 static void
 create_metadata(LextFile *lfile)
 {
-    lfile->meta = gwy_container_new();
+    lfile->meta = gwy_container_new_in_construction();
     g_hash_table_foreach(lfile->hash, add_metadata, lfile);
 
     if (!gwy_container_get_n_items(lfile->meta))
-        GWY_OBJECT_UNREF(lfile->meta);
+        g_clear_object(&lfile->meta);
 
     /* XXX: We could also extract date & time from EXIT TIFF tags and put them to the metadata... */
 }
