@@ -1,6 +1,6 @@
 /*
- *  $Id: magellan.c 26135 2024-01-15 12:54:27Z yeti-dn $
- *  Copyright (C) 2013 David Necas (Yeti).
+ *  $Id: magellan.c 28789 2025-11-04 17:14:03Z yeti-dn $
+ *  Copyright (C) 2013-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -28,11 +28,9 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/stats.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 #include "err.h"
 #include "gwytiff.h"
 
@@ -43,12 +41,12 @@ enum {
 };
 
 static gboolean      module_register(void);
-static gint          mgl_detect     (const GwyFileDetectInfo *fileinfo,
+static gint          detect_file    (const GwyFileDetectInfo *fileinfo,
                                      gboolean only_name);
-static GwyContainer* mgl_load       (const gchar *filename,
-                                     GwyRunType mode,
+static GwyFile*      load_file      (const gchar *filename,
+                                     GwyRunModeFlags mode,
                                      GError **error);
-static GwyContainer* mgl_load_tiff  (const GwyTIFF *tiff,
+static GwyFile*      mgl_load_tiff  (const GwyTIFF *tiff,
                                      const gchar *filename,
                                      GError **error);
 static GwyContainer* get_meta       (GHashTable *hash);
@@ -61,7 +59,7 @@ static GwyModuleInfo module_info = {
     module_register,
     N_("Imports FEI Magellan SEM images."),
     "Yeti <yeti@gwyddion.net>",
-    "1.2",
+    "2.0",
     "David Nečas (Yeti)",
     "2013",
 };
@@ -73,16 +71,13 @@ module_register(void)
 {
     gwy_file_func_register("magellan",
                            N_("FEI Magellan SEM image (.tif)"),
-                           (GwyFileDetectFunc)&mgl_detect,
-                           (GwyFileLoadFunc)&mgl_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-mgl_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     GwyTIFF *tiff;
     gint score = 0;
@@ -110,29 +105,30 @@ mgl_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
     return score;
 }
 
-static GwyContainer*
-mgl_load(const gchar *filename,
-         G_GNUC_UNUSED GwyRunType mode,
-         GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
     GwyTIFF *tiff;
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
 
     tiff = gwy_tiff_load(filename, error);
     if (!tiff)
         return NULL;
 
-    container = mgl_load_tiff(tiff, filename, error);
+    file = mgl_load_tiff(tiff, filename, error);
     gwy_tiff_free(tiff);
 
-    return container;
+    return file;
 }
 
-static GwyContainer*
+static GwyFile*
 mgl_load_tiff(const GwyTIFF *tiff, const gchar *filename, GError **error)
 {
-    GwyContainer *container = NULL, *meta;
-    GwyDataField *dfield;
+    GwyFile *file = NULL;
+    GwyContainer *meta;
+    GwyField *dfield;
     GwyTIFFImageReader *reader = NULL;
     GwyTextHeaderParser parser;
     GHashTable *hash;
@@ -153,7 +149,7 @@ mgl_load_tiff(const GwyTIFF *tiff, const gchar *filename, GError **error)
     }
 
     /* Read the comment header. */
-    gwy_clear(&parser, 1);
+    gwy_clear1(parser);
     parser.key_value_separator = "=";
     parser.section_template = "[\x1a]";
     parser.section_accessor = "::";
@@ -197,30 +193,29 @@ mgl_load_tiff(const GwyTIFF *tiff, const gchar *filename, GError **error)
         name = g_hash_table_lookup(hash, "Detectors::Name");
         mode = g_hash_table_lookup(hash, "Detectors::Mode");
 
-        dfield = gwy_data_field_new(reader->width, reader->height,
-                                    reader->width * xstep, reader->height * ystep,
-                                    FALSE);
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
+        dfield = gwy_field_new(reader->width, reader->height,
+                               reader->width * xstep, reader->height * ystep,
+                               FALSE);
+        gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
 
-        data = gwy_data_field_get_data(dfield);
+        data = gwy_field_get_data(dfield);
         for (i = 0; i < reader->height; i++)
             gwy_tiff_read_image_row_averaged(tiff, reader, i, q, 0.0, data + i*reader->width);
 
-        if (!container)
-            container = gwy_container_new();
+        if (!file)
+            file = gwy_file_new_in_construction();
 
-        gwy_container_pass_object(container, gwy_app_get_data_key_for_id(dir_num), dfield);
+        gwy_file_pass_image(file, dir_num, dfield);
         if (name && mode) {
-            gwy_container_set_string(container, gwy_app_get_data_title_key_for_id(dir_num),
-                                     g_strconcat(name, " ", mode, NULL));
+            gwy_file_pass_title(file, GWY_FILE_IMAGE, dir_num, g_strconcat(name, " ", mode, NULL));
         }
         if ((meta = get_meta(hash)))
-            gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(dir_num), meta);
+            gwy_file_pass_meta(file, GWY_FILE_IMAGE, dir_num, meta);
 
-        gwy_file_channel_import_log_add(container, dir_num, NULL, filename);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, dir_num, NULL, filename);
     }
 
-    if (!container)
+    if (!file)
         err_NO_DATA(error);
 
 fail:
@@ -233,13 +228,13 @@ fail:
         reader = NULL;
     }
 
-    return container;
+    return file;
 }
 
 static GwyContainer*
 get_meta(GHashTable *hash)
 {
-    GwyContainer *meta = gwy_container_new();
+    GwyContainer *meta = gwy_container_new_in_construction();
     g_hash_table_foreach(hash, add_meta, meta);
     if (gwy_container_get_n_items(meta))
         return meta;
