@@ -1,6 +1,6 @@
 /*
- *  $Id: matfile.c 26519 2024-08-15 18:04:20Z yeti-dn $
- *  Copyright (C) 2009,2023 David Necas (Yeti).
+ *  $Id: matfile.c 28848 2025-11-10 18:54:08Z yeti-dn $
+ *  Copyright (C) 2009-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -28,15 +28,10 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/datafield.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 #include "gwyzlib.h"
 #include "err.h"
 #include "get.h"
@@ -167,18 +162,18 @@ typedef struct _Mat5FileCursor {
     guint nbytes;
 } Mat5FileCursor;
 
-static gboolean      module_register   (void);
-static gint          mat5_detect       (const GwyFileDetectInfo *fileinfo,
-                                        gboolean only_name);
-static GwyContainer* mat5_load         (const gchar *filename,
-                                        GwyRunType mode,
-                                        GError **error);
-static GwyDataField* try_read_datafield(Mat5FileCursor *parent,
-                                        GString *name);
-static gboolean      zinflate_variable (Mat5FileCursor *cursor,
-                                        GError **error);
-static gboolean      raw_type_from_mat5(Mat5DataType dtype,
-                                        GwyRawDataType *rawtype);
+static gboolean  module_register   (void);
+static gint      detect_file       (const GwyFileDetectInfo *fileinfo,
+                                    gboolean only_name);
+static GwyFile*  load_file         (const gchar *filename,
+                                    GwyRunModeFlags mode,
+                                    GError **error);
+static GwyField* try_read_field    (Mat5FileCursor *parent,
+                                    GString *name);
+static gboolean  zinflate_variable (Mat5FileCursor *cursor,
+                                    GError **error);
+static gboolean  raw_type_from_mat5(Mat5DataType dtype,
+                                    GwyRawDataType *rawtype);
 
 /* Indexed by Mat5DataType */
 static const guint typesize[] = { 0, 1, 1, 2, 2, 4, 4, 4, 0, 8, 0, 0, 0, 8, 8, 0, 0, 1, 2, 4, };
@@ -188,29 +183,25 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports Matlab MAT files v5."),
     "Yeti <yeti@gwyddion.net>",
-    "0.2",
+    "0.3",
     "David Nečas (Yeti)",
     "2009",
 };
 
-GWY_MODULE_QUERY(module_info)
+GWY_MODULE_QUERY2(module_info, matfile)
 
 static gboolean
 module_register(void)
 {
     gwy_file_func_register("mat5file",
                            N_("Matlab MAT 5 file (.mat)"),
-                           (GwyFileDetectFunc)&mat5_detect,
-                           (GwyFileLoadFunc)&mat5_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-mat5_detect(const GwyFileDetectInfo *fileinfo,
-            gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     gint score = 0;
 
@@ -309,24 +300,24 @@ mat5_next_tag(Mat5FileCursor *cursor, GError **error)
     return TRUE;
 }
 
-static GwyContainer*
-mat5_load(const gchar *filename,
-          G_GNUC_UNUSED GwyRunType mode,
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
           GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     guchar *buffer = NULL;
     const guchar *p;
     gsize size = 0;
     gint G_GNUC_UNUSED tagno = 0, id = 0;
     GError *err = NULL;
-    GwyDataField *dfield = NULL;
+    GwyField *dfield = NULL;
     GString *name = NULL;
     Mat5FileContext fc;
     Mat5FileCursor cursor;
 
-    gwy_clear(&cursor, 1);
-    gwy_clear(&fc, 1);
+    gwy_clear1(cursor);
+    gwy_clear1(fc);
     cursor.context = &fc;
 
     if (!gwy_file_get_contents(filename, &buffer, &size, &err)) {
@@ -366,19 +357,19 @@ mat5_load(const gchar *filename,
 
         /* The only interesting case is MATRIX */
         if (cursor.data_type == MAT5_MATRIX) {
-            if ((dfield = try_read_datafield(&cursor, name))) {
-                if (!container)
-                    container = gwy_container_new();
+            if ((dfield = try_read_field(&cursor, name))) {
+                if (!file)
+                    file = gwy_file_new_in_construction();
 
-                gwy_container_pass_object(container, gwy_app_get_data_key_for_id(id), dfield);
-                gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(id), name->str);
-                gwy_file_channel_import_log_add(container, id, NULL, filename);
+                gwy_file_pass_image(file, id, dfield);
+                gwy_file_set_title(file, GWY_FILE_IMAGE, id, name->str, FALSE);
+                gwy_log_add_import(file, GWY_FILE_IMAGE, id, NULL, filename);
                 id++;
             }
         }
     }
 
-    if (!container)
+    if (!file)
         err_NO_DATA(error);
 
 fail:
@@ -386,12 +377,12 @@ fail:
     g_string_free(name, TRUE);
     g_byte_array_free(fc.zbuffer, TRUE);
 
-    return container;
+    return file;
 }
 
-static GwyDataField*
-try_read_datafield(Mat5FileCursor *parent,
-                   GString *name)
+static GwyField*
+try_read_field(Mat5FileCursor *parent,
+               GString *name)
 {
     Mat5FileContext *fc = parent->context;
     Mat5FileCursor cursor;
@@ -399,10 +390,10 @@ try_read_datafield(Mat5FileCursor *parent,
     G_GNUC_UNUSED Mat5ArrayFlags array_flags;
     GwyRawDataType rawtype;
     gint i, xres, yres;
-    GwyDataField *dfield, *buf;
+    GwyField *dfield, *buf;
     guint flags, rawsize;
 
-    gwy_clear(&cursor, 1);
+    gwy_clear1(cursor);
     cursor.context = parent->context;
     cursor.p = parent->zp;
     cursor.size = parent->nbytes;
@@ -468,12 +459,12 @@ try_read_datafield(Mat5FileCursor *parent,
     if (xres*yres > cursor.nbytes/rawsize)
         return NULL;
 
-    buf = gwy_data_field_new(yres, xres, yres, xres, FALSE);
-    dfield = gwy_data_field_new(xres, yres, xres, yres, FALSE);
+    buf = gwy_field_new(yres, xres, yres, xres, FALSE);
+    dfield = gwy_field_new(xres, yres, xres, yres, FALSE);
     gwy_convert_raw_data(cursor.zp, xres*yres, 1,
                          rawtype, cursor.context->msb ? GWY_BYTE_ORDER_BIG_ENDIAN : GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                         gwy_data_field_get_data(buf), 1.0, 0.0);
-    gwy_data_field_flip_xy(buf, dfield, FALSE);
+                         gwy_field_get_data(buf), 1.0, 0.0);
+    gwy_field_transpose(buf, dfield, FALSE);
     g_object_unref(buf);
     cursor.zp += xres*yres*rawsize;
 
