@@ -1,6 +1,6 @@
 /*
- *  $Id: metropro.c 25689 2023-09-20 15:55:12Z yeti-dn $
- *  Copyright (C) 2006,2014-2019 David Necas (Yeti), Petr Klapetek.
+ *  $Id: metropro.c 28789 2025-11-04 17:14:03Z yeti-dn $
+ *  Copyright (C) 2006-2025 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -45,13 +45,8 @@
 #include <string.h>
 #include <time.h>
 #include "config.h"
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/datafield.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils-file.h>
+#include <glib/gi18n-lib.h>
+#include <gwy.h>
 
 #include "get.h"
 #include "err.h"
@@ -296,19 +291,19 @@ typedef struct {
     /* Our stuff */
     guint infinity;
     const gchar *filename;
-    GwyDataField **intensity_data;
-    GwyDataField **intensity_mask;
-    GwyDataField *phase_data;
-    GwyDataField *phase_mask;
+    GwyField **intensity_data;
+    GwyField **intensity_mask;
+    GwyField *phase_data;
+    GwyField *phase_mask;
 } MProFile;
 
 static gboolean      module_register      (void);
-static gint          mprofile_detect      (const GwyFileDetectInfo *fileinfo,
+static gint          detect_file          (const GwyFileDetectInfo *fileinfo,
                                            gboolean only_name);
-static GwyContainer* mprofile_load        (const gchar *filename,
-                                           GwyRunType mode,
+static GwyFile*      load_file            (const gchar *filename,
+                                           GwyRunModeFlags mode,
                                            GError **error);
-static GwyContainer* mprofile_load_data   (MProFile *mprofile,
+static GwyFile*      mprofile_load_data   (MProFile *mprofile,
                                            const guchar *buffer,
                                            gsize size,
                                            GError **error);
@@ -316,7 +311,7 @@ static gboolean      mprofile_read_header (const guchar *buffer,
                                            gsize size,
                                            MProFile *mprofile,
                                            GError **error);
-static gint          fill_data_fields     (MProFile *mprofile,
+static gint          fill_fields          (MProFile *mprofile,
                                            const guchar *buffer);
 static GwyContainer* mprofile_get_metadata(MProFile *mprofile);
 
@@ -336,7 +331,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports binary MetroPro (Zygo) data files."),
     "Yeti <yeti@gwyddion.net>",
-    "1.3",
+    "2.0",
     "David Nečas (Yeti) & Petr Klapetek",
     "2006",
 };
@@ -348,17 +343,13 @@ module_register(void)
 {
     gwy_file_func_register("metropro",
                            N_("MetroPro files (.dat)"),
-                           (GwyFileDetectFunc)&mprofile_detect,
-                           (GwyFileLoadFunc)&mprofile_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-mprofile_detect(const GwyFileDetectInfo *fileinfo,
-                gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     gint score = 0;
     const guchar *p;
@@ -382,12 +373,12 @@ mprofile_detect(const GwyFileDetectInfo *fileinfo,
     return score;
 }
 
-static GwyContainer*
-mprofile_load(const gchar *filename,
-              G_GNUC_UNUSED GwyRunType mode,
-              GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     MProFile *mprofile;
     guchar *buffer = NULL;
     gsize size = 0;
@@ -406,29 +397,29 @@ mprofile_load(const gchar *filename,
     mprofile->filename = filename;
     mprofile->infinity = G_MAXUINT;
 
-    container = mprofile_load_data(mprofile, buffer, size, error);
+    file = mprofile_load_data(mprofile, buffer, size, error);
 
     for (n = 0; n < mprofile->ac_n_buckets; n++) {
-        GWY_OBJECT_UNREF(mprofile->intensity_data[n]);
-        GWY_OBJECT_UNREF(mprofile->intensity_mask[n]);
+        g_clear_object(mprofile->intensity_data + n);
+        g_clear_object(mprofile->intensity_mask + n);
     }
     g_free(mprofile->intensity_data);
     g_free(mprofile->intensity_mask);
-    GWY_OBJECT_UNREF(mprofile->phase_data);
-    GWY_OBJECT_UNREF(mprofile->phase_mask);
+    g_clear_object(&mprofile->phase_data);
+    g_clear_object(&mprofile->phase_mask);
     g_free(mprofile);
 
     gwy_file_abandon_contents(buffer, size, NULL);
 
-    return container;
+    return file;
 }
 
-static GwyContainer*
+static GwyFile*
 mprofile_load_data(MProFile *mprofile,
                    const guchar *buffer, gsize size, GError **error)
 {
-    GwyContainer *container = NULL;
-    GwyDataField *dfield = NULL, *vpmask = NULL;
+    GwyFile *file = NULL;
+    GwyField *dfield = NULL, *vpmask = NULL;
     gsize expected;
     const gchar *title;
     guint n, i, ac_npix, cn_npix;
@@ -449,13 +440,13 @@ mprofile_load_data(MProFile *mprofile,
         return NULL;
     }
 
-    n = fill_data_fields(mprofile, buffer);
+    n = fill_fields(mprofile, buffer);
     if (!n) {
         err_NO_DATA(error);
         return NULL;
     }
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     for (i = 0; i < n; i++) {
         if (i > 0) {
             dfield = mprofile->intensity_data[i-1];
@@ -467,15 +458,15 @@ mprofile_load_data(MProFile *mprofile,
             vpmask = mprofile->phase_mask;
             title = "Phase";
         }
-        gwy_container_set_object(container, gwy_app_get_data_key_for_id(i), dfield);
-        gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(i), title);
+        gwy_file_set_image(file, i, dfield);
+        gwy_file_set_title(file, GWY_FILE_IMAGE, i, title, FALSE);
         if (vpmask)
-            gwy_container_set_object(container, gwy_app_get_mask_key_for_id(i), vpmask);
-        gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(i), mprofile_get_metadata(mprofile));
-        gwy_file_channel_import_log_add(container, i, NULL, mprofile->filename);
+            gwy_file_set_image_mask(file, i, vpmask);
+        gwy_file_pass_meta(file, GWY_FILE_IMAGE, i, mprofile_get_metadata(mprofile));
+        gwy_log_add_import(file, GWY_FILE_IMAGE, i, NULL, mprofile->filename);
     }
 
-    return container;
+    return file;
 }
 
 static gboolean
@@ -778,19 +769,19 @@ mprofile_read_header(const guchar *buffer,
 }
 
 static void
-set_units(GwyDataField *dfield,
-            const MProFile *mprofile,
-            const gchar *zunit)
+set_units(GwyField *dfield,
+          const MProFile *mprofile,
+          const gchar *zunit)
 {
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), mprofile->lateral_res ? "m" : NULL);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), zunit);
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), mprofile->lateral_res ? "m" : NULL);
+    gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), zunit);
 }
 
 static gint
-fill_data_fields(MProFile *mprofile,
-                 const guchar *buffer)
+fill_fields(MProFile *mprofile,
+            const guchar *buffer)
 {
-    GwyDataField *dfield, *vpmask;
+    GwyField *dfield, *vpmask;
     gdouble *data, *mask;
     gdouble xreal, yreal, q;
     const guchar *p;
@@ -815,8 +806,8 @@ fill_data_fields(MProFile *mprofile,
     if (mprofile->ac_n_buckets) {
         const guint16 *d16;
 
-        mprofile->intensity_data = g_new(GwyDataField*, mprofile->ac_n_buckets);
-        mprofile->intensity_mask = g_new(GwyDataField*, mprofile->ac_n_buckets);
+        mprofile->intensity_data = g_new(GwyField*, mprofile->ac_n_buckets);
+        mprofile->intensity_mask = g_new(GwyField*, mprofile->ac_n_buckets);
 
         q = mprofile->sign ? -1.0 : 1.0;
 
@@ -832,12 +823,12 @@ fill_data_fields(MProFile *mprofile,
 
         for (id = 0; id < mprofile->ac_n_buckets; id++) {
             ndata++;
-            dfield = gwy_data_field_new(mprofile->ac_width, mprofile->ac_height, xreal, yreal, FALSE);
+            dfield = gwy_field_new(mprofile->ac_width, mprofile->ac_height, xreal, yreal, FALSE);
             set_units(dfield, mprofile, "");
-            vpmask = gwy_data_field_new_alike(dfield, FALSE);
-            gwy_data_field_fill(vpmask, 1.0);
-            data = gwy_data_field_get_data(dfield);
-            mask = gwy_data_field_get_data(vpmask);
+            vpmask = gwy_field_new_alike(dfield, FALSE);
+            gwy_field_fill(vpmask, 1.0);
+            data = gwy_field_get_data(dfield);
+            mask = gwy_field_get_data(vpmask);
             d16 = (const guint16*)p;
             for (i = 0; i < mprofile->ac_height; i++) {
                 for (j = 0; j < mprofile->ac_width; j++) {
@@ -853,7 +844,7 @@ fill_data_fields(MProFile *mprofile,
             }
 
             if (!gwy_app_channel_remove_bad_data(dfield, vpmask))
-                GWY_OBJECT_UNREF(vpmask);
+                g_clear_object(&vpmask);
 
             mprofile->intensity_data[id] = dfield;
             mprofile->intensity_mask[id] = vpmask;
@@ -895,11 +886,11 @@ fill_data_fields(MProFile *mprofile,
             xreal = mprofile->cn_width;
             yreal = mprofile->cn_height;
         }
-        dfield = gwy_data_field_new(mprofile->cn_width, mprofile->cn_height, xreal, yreal, FALSE);
-        vpmask = gwy_data_field_new_alike(dfield, FALSE);
-        gwy_data_field_fill(vpmask, 1.0);
-        data = gwy_data_field_get_data(dfield);
-        mask = gwy_data_field_get_data(vpmask);
+        dfield = gwy_field_new(mprofile->cn_width, mprofile->cn_height, xreal, yreal, FALSE);
+        vpmask = gwy_field_new_alike(dfield, FALSE);
+        gwy_field_fill(vpmask, 1.0);
+        data = gwy_field_get_data(dfield);
+        mask = gwy_field_get_data(vpmask);
         d32 = (const gint32*)p;
         for (i = 0; i < mprofile->cn_height; i++) {
             for (j = 0; j < mprofile->cn_width; j++) {
@@ -916,7 +907,7 @@ fill_data_fields(MProFile *mprofile,
 
         set_units(dfield, mprofile, "m");
         if (!gwy_app_channel_remove_bad_data(dfield, vpmask))
-            GWY_OBJECT_UNREF(vpmask);
+            g_clear_object(&vpmask);
 
         mprofile->phase_data = dfield;
         mprofile->phase_mask = vpmask;
@@ -946,7 +937,7 @@ fill_data_fields(MProFile *mprofile,
     } while (FALSE)
 
 static void
-store_meta_string(GwyContainer *container,
+store_meta_string(GwyContainer *meta,
                   const gchar *key,
                   gchar *field)
 {
@@ -955,7 +946,7 @@ store_meta_string(GwyContainer *container,
     g_strstrip(field);
     /* XXX: This gives inconsistent results on different systems. But it may be what the user expects. */
     if (field[0] && (p = g_locale_to_utf8(field, strlen(field), NULL, NULL, NULL)))
-        gwy_container_set_string_by_name(container, key, p);
+        gwy_container_set_string_by_name(meta, key, p);
 }
 
 /* Quite incomplete... */
@@ -991,7 +982,7 @@ mprofile_get_metadata(MProFile *mprofile)
     gchar *p;
     guint i;
 
-    meta = gwy_container_new();
+    meta = gwy_container_new_in_construction();
 
     /* Version */
     p = g_strdup_printf("%d.%d.%d", mprofile->swinfo_vers_maj, mprofile->swinfo_vers_min, mprofile->swinfo_vers_bug);
