@@ -1,6 +1,6 @@
 /*
- *  $Id: miffile.c 25689 2023-09-20 15:55:12Z yeti-dn $
- *  Copyright (C) 2010 David Necas (Yeti).
+ *  $Id: miffile.c 28789 2025-11-04 17:14:03Z yeti-dn $
+ *  Copyright (C) 2010-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -52,10 +52,8 @@
  **/
 
 #include "config.h"
-#include <libgwyddion/gwymath.h>
-#include <libprocess/datafield.h>
-#include <app/data-browser.h>
-#include <app/gwymoduleutils-file.h>
+#include <glib/gi18n-lib.h>
+#include <gwy.h>
 
 #include "err.h"
 #include "get.h"
@@ -178,17 +176,17 @@ typedef struct {
     MIFInfoItem images[INFO_N_IMAGES];
 } MIFFile;
 
-static gboolean      module_register    (void);
-static gint          mif_detect         (const GwyFileDetectInfo *fileinfo,
-                                         gboolean only_name);
-static GwyContainer* mif_load           (const gchar *filename,
-                                         GwyRunType mode,
-                                         GError **error);
-static GwyDataField* mif_read_data_field(const MIFImageHeader *image_header,
-                                         const MIFBlock *block,
-                                         const guchar *buffer,
-                                         gsize size,
-                                         GError **error);
+static gboolean  module_register(void);
+static gint      detect_file    (const GwyFileDetectInfo *fileinfo,
+                                 gboolean only_name);
+static GwyFile*  load_file      (const gchar *filename,
+                                 GwyRunModeFlags mode,
+                                 GError **error);
+static GwyField* read_field     (const MIFImageHeader *image_header,
+                                 const MIFBlock *block,
+                                 const guchar *buffer,
+                                 gsize size,
+                                 GError **error);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
@@ -207,15 +205,12 @@ module_register(void)
 {
     gwy_file_func_register("miffile",
                            N_("DME MIF files (.mif)"),
-                           (GwyFileDetectFunc)&mif_detect,
-                           (GwyFileLoadFunc)&mif_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
     return TRUE;
 }
 
 static gint
-mif_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     gint score = 0;
 
@@ -246,10 +241,10 @@ err_AXIS_INFO_TOO_SHORT(GError **error)
 }
 
 static gboolean
-mif_read_header(const guchar *buffer,
-                gsize size,
-                MIFHeader *header,
-                GError **error)
+read_header(const guchar *buffer,
+            gsize size,
+            MIFHeader *header,
+            GError **error)
 {
     const guchar *p = buffer;
 
@@ -300,9 +295,9 @@ mif_read_header(const guchar *buffer,
 }
 
 static void
-mif_read_block(MIFBlock *block,
-               G_GNUC_UNUSED const gchar *name,
-               const guchar **p)
+read_block(MIFBlock *block,
+           G_GNUC_UNUSED const gchar *name,
+           const guchar **p)
 {
     block->offset = gwy_get_guint32_le(p);
     block->size = gwy_get_guint32_le(p);
@@ -313,11 +308,11 @@ mif_read_block(MIFBlock *block,
 }
 
 static gboolean
-mif_read_image_items(MIFInfoItem *items,
-                     const guchar *buffer,
-                     G_GNUC_UNUSED gsize size,
-                     const MIFBlock *block,
-                     GError **error)
+read_image_items(MIFInfoItem *items,
+                 const guchar *buffer,
+                 G_GNUC_UNUSED gsize size,
+                 const MIFBlock *block,
+                 GError **error)
 {
     const guchar *p;
     guint i;
@@ -333,7 +328,7 @@ mif_read_image_items(MIFInfoItem *items,
 
     for (i = 0; i < INFO_N_IMAGES; i++) {
         p = buffer + block->offset + i*item_size;
-        mif_read_block(&items[i].image, "item", &p);
+        read_block(&items[i].image, "item", &p);
         items[i].image_type = *(p++);
         if (items[i].image.size || items[i].image_type) {
             gwy_debug("item #%u: type: %u, offset: %zu, size: %zu",
@@ -345,11 +340,11 @@ mif_read_image_items(MIFInfoItem *items,
 }
 
 static gboolean
-mif_read_axis_info(MIFAxisInfo *axis,
-                   const guchar **p,
-                   gsize size,
-                   guint file_version,
-                   GError **error)
+read_axis_info(MIFAxisInfo *axis,
+               const guchar **p,
+               gsize size,
+               guint file_version,
+               GError **error)
 {
     gwy_clear(axis, 1);
 
@@ -383,10 +378,10 @@ mif_read_axis_info(MIFAxisInfo *axis,
 
 G_GNUC_UNUSED
 static gboolean
-mif_read_id(MIFUniqueID *id,
-            const guchar **p,
-            gsize size,
-            GError **error)
+read_id(MIFUniqueID *id,
+        const guchar **p,
+        gsize size,
+        GError **error)
 {
     gwy_clear(id, 1);
 
@@ -407,11 +402,11 @@ mif_read_id(MIFUniqueID *id,
 }
 
 static gboolean
-mif_read_scan_setup(MIFScanSetup *setup,
-                    const guchar **p,
-                    gsize size,
-                    guint file_version,
-                    GError **error)
+read_scan_setup(MIFScanSetup *setup,
+                const guchar **p,
+                gsize size,
+                guint file_version,
+                GError **error)
 {
     if (file_version >= 0x103 && file_version <= 0x107) {
         if (size < SCAN_SETUP_SIZE_1_2)
@@ -451,11 +446,11 @@ mif_read_scan_setup(MIFScanSetup *setup,
 }
 
 static gboolean
-mif_read_image_configuration(MIFImageConfiguration *config,
-                             const guchar **p,
-                             gsize size,
-                             guint file_version,
-                             GError **error)
+read_image_configuration(MIFImageConfiguration *config,
+                         const guchar **p,
+                         gsize size,
+                         guint file_version,
+                         GError **error)
 {
     if (file_version >= 0x107 && file_version <= 0x109) {
         if (size < IMAGE_CONFIGURATION_SIZE_1_3)
@@ -494,11 +489,11 @@ mif_read_image_configuration(MIFImageConfiguration *config,
 }
 
 static gboolean
-mif_read_image_header(MIFImageHeader *header,
-                      const guchar **p,
-                      gsize size,
-                      guint file_version,
-                      GError **error)
+read_image_header(MIFImageHeader *header,
+                  const guchar **p,
+                  gsize size,
+                  guint file_version,
+                  GError **error)
 {
     const guchar *buffer = *p;
     guint i;
@@ -512,9 +507,9 @@ mif_read_image_header(MIFImageHeader *header,
     get_CHARARRAY(header->comment, p);
     gwy_debug("image <%.*s>", (guint)sizeof(header->title), header->title);
 
-    if (!mif_read_scan_setup(&header->setup, p, size - (*p - buffer), file_version, error))
+    if (!read_scan_setup(&header->setup, p, size - (*p - buffer), file_version, error))
         return FALSE;
-    if (!mif_read_image_configuration(&header->configuration, p, size - (*p - buffer), file_version, error))
+    if (!read_image_configuration(&header->configuration, p, size - (*p - buffer), file_version, error))
         return FALSE;
 
     if (file_version == 0x107) {
@@ -537,11 +532,11 @@ mif_read_image_header(MIFImageHeader *header,
         header->high_fraction = gwy_get_gfloat_le(p);
         gwy_debug("low_fraction: %g, high_fraction: %g", header->low_fraction, header->high_fraction);
 
-        if (!mif_read_axis_info(&header->z_axis, p, size - (*p - buffer), file_version, error))
+        if (!read_axis_info(&header->z_axis, p, size - (*p - buffer), file_version, error))
             return FALSE;
 
-        if (!mif_read_id(&header->id.primary, p, size - (*p - buffer), error)
-            || !mif_read_id(&header->id.secondary, p, size - (*p - buffer), error))
+        if (!read_id(&header->id.primary, p, size - (*p - buffer), error)
+            || !read_id(&header->id.secondary, p, size - (*p - buffer), error))
             return FALSE;
     }
     else {
@@ -551,17 +546,17 @@ mif_read_image_header(MIFImageHeader *header,
     return TRUE;
 }
 
-static GwyContainer*
-mif_load(const gchar *filename,
-         G_GNUC_UNUSED GwyRunType mode,
-         GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     guchar *buffer = NULL;
     gsize size = 0;
     GError *err = NULL;
     MIFFile mfile;
-    GwyDataField *dfield;
+    GwyField *dfield;
     guint i;
 
     if (!gwy_file_get_contents(filename, &buffer, &size, &err)) {
@@ -569,12 +564,12 @@ mif_load(const gchar *filename,
         return NULL;
     }
 
-    if (!mif_read_header(buffer, size, &mfile.header, error)) {
+    if (!read_header(buffer, size, &mfile.header, error)) {
         gwy_file_abandon_contents(buffer, size, NULL);
         return NULL;
     }
     /* TODO: Check file version */
-    if (!mif_read_image_items(mfile.images, buffer, size, &mfile.header.info, error)) {
+    if (!read_image_items(mfile.images, buffer, size, &mfile.header.info, error)) {
         gwy_file_abandon_contents(buffer, size, NULL);
         return NULL;
     }
@@ -586,7 +581,6 @@ mif_load(const gchar *filename,
         MIFBlock raster, macro_geometry, preview, image, curve, calc;
         G_GNUC_UNUSED guint ncalculations;
         const guchar *p = buffer + item->image.offset;
-        GQuark quark;
 
         if (!item->image.size)
             continue;
@@ -600,48 +594,47 @@ mif_load(const gchar *filename,
          * unaccounted-for space until the next block starts, 120 bytes for v1.7 files, after the image header which
          * is in fact still occupied by the image header. MIFBlock says 714 bytes but the true size is
          * 834 = 714 + 120. */
-        if (!mif_read_image_header(&image_header, &p, size - (p - buffer), mfile.header.file_version, error))
+        if (!read_image_header(&image_header, &p, size - (p - buffer), mfile.header.file_version, error))
             continue;
 
         if (p - buffer + 52 > size)
             continue;
 
-        mif_read_block(&raster, "raster", &p);
-        mif_read_block(&macro_geometry, "macro_geometry", &p);
-        mif_read_block(&preview, "preview", &p);
-        mif_read_block(&image, "image", &p);
-        mif_read_block(&curve, "curve", &p);
+        read_block(&raster, "raster", &p);
+        read_block(&macro_geometry, "macro_geometry", &p);
+        read_block(&preview, "preview", &p);
+        read_block(&image, "image", &p);
+        read_block(&curve, "curve", &p);
         ncalculations = gwy_get_guint32_le(&p);
-        mif_read_block(&calc, "calc", &p);
+        read_block(&calc, "calc", &p);
 
         gwy_debug("image header true size: %zu", (gsize)(p - (buffer + item->image.offset)));
 
-        dfield = mif_read_data_field(&image_header, &image, buffer, size, NULL);
+        dfield = read_field(&image_header, &image, buffer, size, NULL);
         if (!dfield)
             continue;
 
-        if (!container)
-            container = gwy_container_new();
+        if (!file)
+            file = gwy_file_new_in_construction();
 
-        quark = gwy_app_get_data_key_for_id(i);
-        gwy_container_pass_object(container, quark, dfield);
-        gwy_app_channel_title_fall_back(container, i);
-        gwy_file_channel_import_log_add(container, i, NULL, filename);
+        gwy_file_pass_image(file, i, dfield);
+        gwy_image_title_fall_back(file, i);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, i, NULL, filename);
     }
 
-    if (!container)
+    if (!file)
         err_NO_DATA(error);
 
     gwy_file_abandon_contents(buffer, size, NULL);
-    return container;
+    return file;
 }
 
-static GwyDataField*
-mif_read_data_field(const MIFImageHeader *image_header,
-                    const MIFBlock *block,
-                    const guchar *buffer,
-                    gsize size,
-                    GError **error)
+static GwyField*
+read_field(const MIFImageHeader *image_header,
+           const MIFBlock *block,
+           const guchar *buffer,
+           gsize size,
+           GError **error)
 {
     const MIFScanSetup *setup = &image_header->setup;
     const MIFImageConfiguration *configuration = &image_header->configuration;
@@ -652,7 +645,7 @@ mif_read_data_field(const MIFImageHeader *image_header,
     gdouble xoff = setup->xoff;
     gdouble yoff = setup->yoff;
     gdouble q = configuration->zcal/65536.0;
-    GwyDataField *dfield;
+    GwyField *dfield;
     gdouble *data, *linecorrdata = NULL;
     gsize datasize;
     gint i;
@@ -681,13 +674,13 @@ mif_read_data_field(const MIFImageHeader *image_header,
         }
     }
 
-    dfield = gwy_data_field_new(xres, yres, xreal, yreal, FALSE);
-    gwy_data_field_set_xoffset(dfield, xoff);
-    gwy_data_field_set_yoffset(dfield, yoff);
-    data = gwy_data_field_get_data(dfield);
+    dfield = gwy_field_new(xres, yres, xreal, yreal, FALSE);
+    gwy_field_set_xoffset(dfield, xoff);
+    gwy_field_set_yoffset(dfield, yoff);
+    data = gwy_field_get_data(dfield);
 
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), "m");
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
+    gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), "m");
 
     // XXX: Don't know why this range.  A user got it from DME.  Have no idea why scan_int_to_meter does not appear
     // here.
