@@ -1,9 +1,9 @@
 /*
- *  $Id: mifile.c 26510 2024-08-15 17:12:23Z yeti-dn $
+ *  $Id: mifile.c 28911 2025-11-24 18:27:42Z yeti-dn $
  *  Copyright (C) 2005 Chris Anderson, Molecular Imaging Corp.
  *  E-mail: sidewinder.asu@gmail.com
  *
- *  Copyright (C) 2006-2024 David Necas (Yeti), Petr Klapetek.
+ *  Copyright (C) 2006-2025 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -47,17 +47,10 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/datafield.h>
-#include <libgwydgets/gwygraphmodel.h>
-#include <libgwydgets/gwygraphbasics.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -137,60 +130,61 @@ typedef struct {
     const gchar *meta; /* NULL means the same as key */
 } MetaDataFormat;
 
-static gboolean      module_register       (void);
-static gint          mifile_detect         (const GwyFileDetectInfo *fileinfo,
-                                            gboolean only_name);
-static GwyContainer* mifile_load           (const gchar *filename,
-                                            GwyRunType mode,
-                                            GError **error);
-static guint         find_data_start       (const guchar *buffer,
-                                            gsize size,
-                                            MIDataType *data_type);
-static guint         image_file_read_header(MIFile *mifile,
-                                            gchar *buffer,
-                                            GError **error);
-static guint         spect_file_read_header(MISpectFile *mifile,
-                                            gchar *buffer,
-                                            GError **error);
-static GwyContainer* read_image_data       (MIFile *mifile,
-                                            const guchar *buffer,
-                                            MIDataType data_type,
-                                            gboolean ismulticol,
-                                            gsize header_size,
-                                            const gchar *filename,
-                                            GError **error);
-static GwyContainer* read_graph_data       (MISpectFile *mifile,
-                                            const guchar *buffer,
-                                            gsize size,
-                                            MIDataType data_type,
-                                            gboolean ismulticol,
-                                            gsize header_size,
-                                            GError **error);
-static GwyContainer* read_curve_map_data   (MISpectFile *mifile,
-                                            const guchar *buffer,
-                                            MIDataType data_type,
-                                            gsize header_size,
-                                            const gchar *filename,
-                                            GError **error);
-static void          image_file_free       (MIFile *mifile);
-static void          spect_file_free       (MISpectFile *mifile);
-static gboolean      mifile_get_double     (GHashTable *meta,
-                                            const gchar *key,
-                                            gdouble *value);
-static void          process_metadata      (MIFile *mifile,
-                                            guint id,
-                                            gboolean no_rescale,
-                                            GwyContainer *container);
-static gchar**       split_to_nparts       (const gchar *str,
-                                            const gchar *sep,
-                                            guint n);
+static gboolean module_register       (void);
+static gint     detect_file           (const GwyFileDetectInfo *fileinfo,
+                                       gboolean only_name);
+static GwyFile* load_file             (const gchar *filename,
+                                       GwyRunModeFlags mode,
+                                       GError **error);
+static guint    find_data_start       (const guchar *buffer,
+                                       gsize size,
+                                       MIDataType *data_type);
+static guint    image_file_read_header(MIFile *mifile,
+                                       gchar *buffer,
+                                       GError **error);
+static guint    spect_file_read_header(MISpectFile *mifile,
+                                       gchar *buffer,
+                                       GError **error);
+static GwyFile* read_image_data       (MIFile *mifile,
+                                       const guchar *buffer,
+                                       MIDataType data_type,
+                                       gboolean ismulticol,
+                                       gsize header_size,
+                                       const gchar *filename,
+                                       GError **error);
+static GwyFile* read_graph_data       (MISpectFile *mifile,
+                                       const guchar *buffer,
+                                       gsize size,
+                                       MIDataType data_type,
+                                       gboolean ismulticol,
+                                       gsize header_size,
+                                       const gchar *filename,
+                                       GError **error);
+static GwyFile* read_curve_map_data   (MISpectFile *mifile,
+                                       const guchar *buffer,
+                                       MIDataType data_type,
+                                       gsize header_size,
+                                       const gchar *filename,
+                                       GError **error);
+static void     image_file_free       (MIFile *mifile);
+static void     spect_file_free       (MISpectFile *mifile);
+static gboolean mifile_get_double     (GHashTable *meta,
+                                       const gchar *key,
+                                       gdouble *value);
+static void     process_metadata      (MIFile *mifile,
+                                       guint id,
+                                       gboolean no_rescale,
+                                       GwyFile *file);
+static gchar**  split_to_nparts       (const gchar *str,
+                                       const gchar *sep,
+                                       guint n);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Molecular Imaging MI data files."),
     "Chris Anderson <sidewinder.asu@gmail.com>",
-    "0.20",
+    "0.21",
     "Chris Anderson, Molecular Imaging Corp.",
     "2006",
 };
@@ -202,16 +196,13 @@ module_register(void)
 {
     gwy_file_func_register("mifile",
                            N_("PicoView Data Files (.mi)"),
-                           (GwyFileDetectFunc)&mifile_detect,
-                           (GwyFileLoadFunc)&mifile_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-mifile_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     gint score = 0;
 
@@ -226,14 +217,14 @@ mifile_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
     return score;
 }
 
-static GwyContainer*
-mifile_load(const gchar *filename,
-            G_GNUC_UNUSED GwyRunType mode,
-            GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
     MIFile *mifile_img = NULL;
     MISpectFile *mifile_spect = NULL;
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     guchar *buffer = NULL;
     gsize size = 0;
     GError *err = NULL;
@@ -326,37 +317,37 @@ mifile_load(const gchar *filename,
 
     /* Load the image data or spectroscopy data (graph/curve map). */
     if (isimage) {
-        container = read_image_data(mifile_img, buffer, data_type, ismulticol, header_size, filename, error);
+        file = read_image_data(mifile_img, buffer, data_type, ismulticol, header_size, filename, error);
         image_file_free(mifile_img);
     }
     else if (!mifile_spect->is_curve_map) {
-        container = read_graph_data(mifile_spect, buffer, size, data_type, ismulticol, header_size, error);
+        file = read_graph_data(mifile_spect, buffer, size, data_type, ismulticol, header_size, filename, error);
         spect_file_free(mifile_spect);
     }
     else {
         if (data_type == MI_ASCII) {
             err_UNSUPPORTED(error, "data");
-            GWY_OBJECT_UNREF(container);
+            g_clear_object(&file);
         }
         else {
-            container = read_curve_map_data(mifile_spect, buffer, data_type, header_size, filename, error);
+            file = read_curve_map_data(mifile_spect, buffer, data_type, header_size, filename, error);
             spect_file_free(mifile_spect);
         }
     }
 
     gwy_file_abandon_contents(buffer, size, NULL);
 
-    return container;
+    return file;
 }
 
-static GwyContainer*
+static GwyFile*
 read_image_data(MIFile *mifile, const guchar *buffer,
                 MIDataType data_type, gboolean ismulticol, gsize header_size, const gchar *filename,
                 GError **error)
 {
-    GwyContainer *container;
+    GwyFile *file;
     MIData *midata;
-    GwyDataField *dfield;
+    GwyField *dfield;
     gdouble *d, *v;
     const guchar *p = buffer + header_size;
     gchar *end;
@@ -364,7 +355,7 @@ read_image_data(MIFile *mifile, const guchar *buffer,
     guint i;
     gint xres = mifile->xres, yres = mifile->yres, k;
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
 
     /* Load image data. */
     if (ismulticol) {
@@ -376,29 +367,29 @@ read_image_data(MIFile *mifile, const guchar *buffer,
 
         k = xres*yres;
         if (!(v = gwy_parse_doubles(p, NULL, GWY_PARSE_DOUBLES_FREE_FORM, &mifile->n, &k, NULL, error))) {
-            g_object_unref(container);
+            g_object_unref(file);
             return NULL;
         }
 
         for (i = 0; i < mifile->n; i++) {
-            dfield = gwy_data_field_new(xres, yres, 1.0, 1.0, FALSE);
-            d = gwy_data_field_get_data(dfield);
+            dfield = gwy_field_new(xres, yres, 1.0, 1.0, FALSE);
+            d = gwy_field_get_data(dfield);
             for (k = 0; k < xres*yres; k++)
                 d[k] = v[k*mifile->n + i];
-            gwy_data_field_invert(dfield, TRUE, FALSE, FALSE);
-            gwy_container_pass_object(container, gwy_app_get_data_key_for_id(i), dfield);
-            process_metadata(mifile, i, TRUE, container);
-            gwy_file_channel_import_log_add(container, i, NULL, filename);
+            gwy_field_flip(dfield, FALSE, TRUE);
+            gwy_file_pass_image(file, i, dfield);
+            process_metadata(mifile, i, TRUE, file);
+            gwy_log_add_import(file, GWY_FILE_IMAGE, i, NULL, filename);
         }
 
         g_free(v);
 
-        return container;
+        return file;
     }
 
     for (i = 0; i < mifile->n; i++) {
-        dfield = gwy_data_field_new(xres, yres, 1.0, 1.0, FALSE);
-        d = gwy_data_field_get_data(dfield);
+        dfield = gwy_field_new(xres, yres, 1.0, 1.0, FALSE);
+        d = gwy_field_get_data(dfield);
         midata = mifile->buffers + i;
         midata->data = buffer + pos;
 
@@ -420,29 +411,29 @@ read_image_data(MIFile *mifile, const guchar *buffer,
         }
         else {
             if (!gwy_parse_doubles(p, d, GWY_PARSE_DOUBLES_FREE_FORM, &yres, &xres, &end, error)) {
-                g_object_unref(container);
+                g_object_unref(file);
                 g_object_unref(dfield);
                 return NULL;
             }
             p = end;
-            gwy_data_field_multiply(dfield, 1.0/23768.0);
-            gwy_data_field_invert(dfield, TRUE, FALSE, FALSE);
+            gwy_field_multiply(dfield, 1.0/23768.0);
+            gwy_field_flip(dfield, FALSE, TRUE);
         }
 
-        gwy_container_pass_object(container, gwy_app_get_data_key_for_id(i), dfield);
-        process_metadata(mifile, i, FALSE, container);
-        gwy_file_channel_import_log_add(container, i, NULL, filename);
+        gwy_file_pass_image(file, i, dfield);
+        process_metadata(mifile, i, FALSE, file);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, i, NULL, filename);
     }
 
-    return container;
+    return file;
 }
 
-static GwyContainer*
+static GwyFile*
 read_graph_data(MISpectFile *mifile, const guchar *buffer, gsize size,
-                MIDataType data_type, G_GNUC_UNUSED gboolean ismulticol, gsize header_size,
+                MIDataType data_type, G_GNUC_UNUSED gboolean ismulticol, gsize header_size, const gchar *filename,
                 GError **error)
 {
-    GwyContainer *container;
+    GwyFile *file;
     GwyGraphModel *gmodel;
     gdouble *xdata, *ydata;
     gsize pos;
@@ -452,7 +443,7 @@ read_graph_data(MISpectFile *mifile, const guchar *buffer, gsize size,
     gchar **lineparts;
 
     /* Load spectroscopy data. */
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     gwy_debug("data type=%d", data_type);
     gwy_debug("num_buffers %d", mifile->num_buffers);
 
@@ -469,17 +460,18 @@ read_graph_data(MISpectFile *mifile, const guchar *buffer, gsize size,
     gmodel = gwy_graph_model_new();
     g_object_set(gmodel, "title", _("Spectroscopy Graph"), NULL);
     if (mifile->num_buffers >= 2) {
-        GwySIUnit *xunit = gwy_si_unit_new(mifile->buffers[0].unit), *yunit = gwy_si_unit_new(mifile->buffers[1].unit);
+        GwyUnit *xunit = gwy_unit_new(mifile->buffers[0].unit), *yunit = gwy_unit_new(mifile->buffers[1].unit);
 
         g_object_set(gmodel,
-                     "si-unit-x", xunit, "si-unit-y", yunit,
+                     "unit-x", xunit, "unit-y", yunit,
                      "axis-label-bottom", mifile->buffers[0].label, "axis-label-left", mifile->buffers[1].label,
                      NULL);
         g_object_unref(xunit);
         g_object_unref(yunit);
     }
 
-    gwy_container_set_object(container, gwy_app_get_graph_key_for_id(1), gmodel);
+    gwy_file_set_graph(file, 0, gmodel);
+    gwy_log_add_import(file, GWY_FILE_GRAPH, 0, NULL, filename);
 
     for (j = 0; j < mifile->chunks->len; j++) {
         const MISpectChunk *chunk = &g_array_index(mifile->chunks, MISpectChunk, j);
@@ -492,7 +484,7 @@ read_graph_data(MISpectFile *mifile, const guchar *buffer, gsize size,
                 if (!(line = gwy_str_next_line(&p)) || !(lineparts = split_to_nparts(line, " \t\n\r", 3))) {
                     g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
                                 _("Cannot parse data values at line %d."), j+1);
-                    GWY_OBJECT_UNREF(container);
+                    g_clear_object(&file);
                     goto fail;
                 }
                 xdata[i] = g_ascii_strtod(lineparts[1], NULL);
@@ -531,15 +523,15 @@ fail:
     g_free(xdata);
     g_free(ydata);
 
-    return container;
+    return file;
 }
 
-static GwyContainer*
+static GwyFile*
 read_curve_map_data(MISpectFile *mifile, const guchar *buffer,
                     MIDataType data_type, gsize header_size, const gchar *filename,
                     GError **error)
 {
-    GwyContainer *container;
+    GwyFile *file;
     const MISpectChunk *chunk;
     //MISpectData *midata;
     GwyLawn *lawn;
@@ -567,26 +559,26 @@ read_curve_map_data(MISpectFile *mifile, const guchar *buffer,
     gwy_debug("nsegments %d, nabscissae %d", nsegments, nabscissa);
 
     /* TODO: Handle curve labels, units, segments, etc. */
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     lawn = gwy_lawn_new(xres, yres, mifile->xreal, mifile->yreal, 1 + nabscissa, nsegments);
-    gwy_si_unit_set_from_string(gwy_lawn_get_si_unit_xy(lawn), "m");
+    gwy_unit_set_from_string(gwy_lawn_get_unit_xy(lawn), "m");
     if (nabscissa >= 1) {
         gwy_lawn_set_curve_label(lawn, 0, "Time");
-        gwy_si_unit_set_from_string(gwy_lawn_get_si_unit_curve(lawn, 0), "s");
+        gwy_unit_set_from_string(gwy_lawn_get_unit_curve(lawn, 0), "s");
         if (nabscissa >= 2) {
             if (mifile->num_buffers >= 1) {
                 gwy_lawn_set_curve_label(lawn, 1, mifile->buffers[0].label);
-                gwy_si_unit_set_from_string(gwy_lawn_get_si_unit_curve(lawn, 1), mifile->buffers[0].unit);
+                gwy_unit_set_from_string(gwy_lawn_get_unit_curve(lawn, 1), mifile->buffers[0].unit);
             }
             else {
                 gwy_lawn_set_curve_label(lawn, 1, "Distance");
-                gwy_si_unit_set_from_string(gwy_lawn_get_si_unit_curve(lawn, 1), "m");
+                gwy_unit_set_from_string(gwy_lawn_get_unit_curve(lawn, 1), "m");
             }
         }
     }
     if (mifile->num_buffers >= 2) {
         gwy_lawn_set_curve_label(lawn, nabscissa, mifile->buffers[1].label);
-        gwy_si_unit_set_from_string(gwy_lawn_get_si_unit_curve(lawn, nabscissa), mifile->buffers[1].unit);
+        gwy_unit_set_from_string(gwy_lawn_get_unit_curve(lawn, nabscissa), mifile->buffers[1].unit);
     }
 
     for (k = 0; k < nsegments; k++) {
@@ -630,14 +622,14 @@ read_curve_map_data(MISpectFile *mifile, const guchar *buffer,
         }
     }
 
-    gwy_container_pass_object(container, gwy_app_get_lawn_key_for_id(i), lawn);
-    // TODO process_metadata(mifile, i, FALSE, container);
-    gwy_file_curve_map_import_log_add(container, 0, NULL, filename);
+    gwy_file_pass_cmap(file, i, lawn);
+    // TODO process_metadata(mifile, i, FALSE, file);
+    gwy_log_add_import(file, GWY_FILE_CMAP, 0, NULL, filename);
 
     g_free(d);
     g_free(segments);
 
-    return container;
+    return file;
 }
 
 static guint
@@ -792,7 +784,7 @@ spect_file_read_header(MISpectFile *mifile, gchar *buffer, GError **error)
         if (!strcmp(key, "chunk")) {
             MISpectChunk chunk;
 
-            gwy_clear(&chunk, 1);
+            gwy_clear1(chunk);
             parts = g_strsplit_set(value, " \t", 0);
             ncols = g_strv_length(parts);
             if (ncols > 1) {
@@ -895,7 +887,7 @@ static void
 process_metadata(MIFile *mifile,
                  guint id,
                  gboolean no_rescale,
-                 GwyContainer *container)
+                 GwyFile *file)
 {
     static const MetaDataFormat global_metadata[] = {
         { "version", "Version" },
@@ -936,40 +928,34 @@ process_metadata(MIFile *mifile,
         { "trace", NULL },
     };
 
-    MIData *data;
-    GwyContainer *meta;
-    GwyDataField *dfield;
-    const gchar *mode;
-    gchar *bufferUnit;
-    gint power10;
     gdouble bufferRange, q;
     gchar *p;
     const gchar *name;
     guint i;
     gdouble xLength, yLength;
 
-    dfield = GWY_DATA_FIELD(gwy_container_get_object(container, gwy_app_get_data_key_for_id(id)));
+    GwyField *dfield = gwy_file_get_image(file, id);
     /* Make "data" point to the selected buffer */
-    data = mifile->buffers + id;
+    MIData *data = mifile->buffers + id;
 
     /* Get the buffer mode */
-    mode = g_hash_table_lookup(data->meta, "bufferLabel");
+    const gchar *mode = g_hash_table_lookup(data->meta, "bufferLabel");
 
-    /* Set the container's title to whatever the buffer mode is */
-    gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(id), mode ? mode : "Unknown Channel");
+    /* Set the file's title to whatever the buffer mode is */
+    gwy_file_set_title(file, GWY_FILE_IMAGE, id, mode ? mode : "Unknown Channel", FALSE);
 
     /* Fix z-value scale */
-    power10 = 0;
-    bufferUnit = g_hash_table_lookup(data->meta, "bufferUnit");
+    gint power10 = 0;
+    gchar *bufferUnit = g_hash_table_lookup(data->meta, "bufferUnit");
     if (bufferUnit)
-        gwy_si_unit_set_from_string_parse(gwy_data_field_get_si_unit_z(dfield), bufferUnit, &power10);
-    q = pow10(power10);
+        power10 = gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), bufferUnit);
+    q = gwy_exp10(power10);
     if (!no_rescale && mifile_get_double(data->meta, "bufferRange", &bufferRange))
         q *= bufferRange;
-    gwy_data_field_multiply(dfield, q);
+    gwy_field_multiply(dfield, q);
 
     /* Fix x-y value scale */
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
 
     if (!mifile_get_double(mifile->meta, "xLength", &xLength)) {
         g_warning("Missing or invalid x length");
@@ -977,7 +963,7 @@ process_metadata(MIFile *mifile,
     }
     if (xLength <= 0.0)
         xLength = 1.0; /* Needed for 0-d calibration images */
-    gwy_data_field_set_xreal(dfield, xLength);
+    gwy_field_set_xreal(dfield, xLength);
 
     if (!mifile_get_double(mifile->meta, "yLength", &yLength)) {
         g_warning("Missing or invalid y length");
@@ -985,10 +971,10 @@ process_metadata(MIFile *mifile,
     }
     if (yLength <= 0.0)
         yLength = 1.0; /* Needed for 0-d calibration images */
-    gwy_data_field_set_yreal(dfield, yLength);
+    gwy_field_set_yreal(dfield, yLength);
 
     /* Store Metadata */
-    meta = gwy_container_new();
+    GwyContainer *meta = gwy_container_new_in_construction();
 
     /* Global */
     for (i = 0; i < G_N_ELEMENTS(global_metadata); i++) {
@@ -1030,8 +1016,9 @@ process_metadata(MIFile *mifile,
     */
 
     if (gwy_container_get_n_items(meta))
-        gwy_container_set_object(container, gwy_app_get_data_meta_key_for_id(id), meta);
-    g_object_unref(meta);
+        gwy_file_pass_meta(file, GWY_FILE_IMAGE, id, meta);
+    else
+        g_object_unref(meta);
 }
 
 static gchar**
