@@ -1,6 +1,6 @@
 /*
- *  $Id: mulfile.c 25636 2023-09-07 12:19:33Z yeti-dn $
- *  Copyright (C) 2011 David Necas (Yeti).
+ *  $Id: mulfile.c 28789 2025-11-04 17:14:03Z yeti-dn $
+ *  Copyright (C) 2011-2025 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
  *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
@@ -43,15 +43,10 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/stats.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 
 #include "get.h"
 #include "err.h"
@@ -129,33 +124,33 @@ typedef struct {
     gint spare_63;
 } MulImageLabel;
 
-static gboolean      module_register     (void);
-static gint          mul_detect          (const GwyFileDetectInfo *fileinfo,
-                                          gboolean only_name);
-static GwyContainer* mul_load            (const gchar *filename,
-                                          GwyRunType mode,
-                                          GError **error);
-static gint          mul_read_index      (const guchar *p,
-                                          gsize size,
-                                          MulIndexEntry *image_index);
-static gboolean      mul_read_image_label(const guchar *buffer,
-                                          gsize size,
-                                          const MulIndexEntry *entry,
-                                          MulImageLabel *label,
-                                          GError **error);
-static void          mul_read_image      (GwyContainer *container,
-                                          const guchar *buffer,
-                                          const MulIndexEntry *entry,
-                                          const MulImageLabel *label,
-                                          const gchar *filename);
-static GwyContainer* mul_get_meta        (const MulImageLabel *label);
+static gboolean      module_register (void);
+static gint          detect_file     (const GwyFileDetectInfo *fileinfo,
+                                      gboolean only_name);
+static GwyFile*      load_file       (const gchar *filename,
+                                      GwyRunModeFlags mode,
+                                      GError **error);
+static gint          read_index      (const guchar *p,
+                                      gsize size,
+                                      MulIndexEntry *image_index);
+static gboolean      read_image_label(const guchar *buffer,
+                                      gsize size,
+                                      const MulIndexEntry *entry,
+                                      MulImageLabel *label,
+                                      GError **error);
+static void          read_image      (GwyFile *file,
+                                      const guchar *buffer,
+                                      const MulIndexEntry *entry,
+                                      const MulImageLabel *label,
+                                      const gchar *filename);
+static GwyContainer* get_meta        (const MulImageLabel *label);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Aarhus MUL files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.2",
+    "0.3",
     "David Nečas (Yeti)",
     "2011",
 };
@@ -167,17 +162,13 @@ module_register(void)
 {
     gwy_file_func_register("mulfile",
                            N_("Aarhus MUL files (.mul)"),
-                           (GwyFileDetectFunc)&mul_detect,
-                           (GwyFileLoadFunc)&mul_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-mul_detect(const GwyFileDetectInfo *fileinfo,
-           gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     if (only_name)
         return g_str_has_suffix(fileinfo->name_lowercase, EXTENSION) ? 10 : 0;
@@ -190,12 +181,12 @@ mul_detect(const GwyFileDetectInfo *fileinfo,
     return 0;
 }
 
-static GwyContainer*
-mul_load(const gchar *filename,
-         G_GNUC_UNUSED GwyRunType mode,
-         GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     guchar *buffer = NULL;
     gsize size;
     GError *err = NULL;
@@ -212,37 +203,37 @@ mul_load(const gchar *filename,
         goto fail;
     }
 
-    n = mul_read_index(buffer, size, image_index);
+    n = read_index(buffer, size, image_index);
     if (!n) {
         err_NO_DATA(error);
         goto fail;
     }
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
 
     for (i = 0; i < n; i++) {
         MulImageLabel label;
-        if (!mul_read_image_label(buffer, size, image_index + i, &label, NULL))
+        if (!read_image_label(buffer, size, image_index + i, &label, NULL))
             continue;
 
-        mul_read_image(container, buffer, image_index + i, &label, filename);
+        read_image(file, buffer, image_index + i, &label, filename);
     }
 
-    if (!gwy_container_get_n_items(container)) {
+    if (!gwy_container_get_n_items(GWY_CONTAINER(file))) {
         err_NO_DATA(error);
-        GWY_OBJECT_UNREF(container);
+        g_clear_object(&file);
     }
 
 fail:
     gwy_file_abandon_contents(buffer, size, NULL);
 
-    return container;
+    return file;
 }
 
 static gint
-mul_read_index(const guchar *p,
-               gsize size,
-               MulIndexEntry *image_index)
+read_index(const guchar *p,
+           gsize size,
+           MulIndexEntry *image_index)
 {
     gboolean first_zero = TRUE;
     gint i, n = 0;
@@ -271,11 +262,11 @@ mul_read_index(const guchar *p,
 }
 
 static gboolean
-mul_read_image_label(const guchar *buffer,
-                     gsize size,
-                     const MulIndexEntry *entry,
-                     MulImageLabel *label,
-                     GError **error)
+read_image_label(const guchar *buffer,
+                 gsize size,
+                 const MulIndexEntry *entry,
+                 MulImageLabel *label,
+                 GError **error)
 {
     const guchar *p = buffer + entry->addr * MUL_BLOCK_SIZE;
     gint len;
@@ -380,26 +371,25 @@ mul_read_image_label(const guchar *buffer,
 }
 
 static void
-mul_read_image(GwyContainer *container,
-               const guchar *buffer,
-               const MulIndexEntry *entry,
-               const MulImageLabel *label,
-               const gchar *filename)
+read_image(GwyFile *file,
+           const guchar *buffer,
+           const MulIndexEntry *entry,
+           const MulImageLabel *label,
+           const gchar *filename)
 {
     const guchar *p = (buffer + (entry->addr + 1)* MUL_BLOCK_SIZE);
-    GwyDataField *field;
+    GwyField *field;
     GwyRawDataType rawtype;
-    GwyContainer *meta;
     gdouble *data;
     gdouble q_height, q_current, q_voltage, q = 1.0/32768;
     guint i;
-    guchar key[64], *title;
+    guchar *title;
 
     /* XXX: The specs say lenght unit is 0.1 Å but that does not seem right. */
-    field = gwy_data_field_new(label->xres, label->yres, Angstrom*label->xdim, Angstrom*label->ydim, FALSE);
+    field = gwy_field_new(label->xres, label->yres, Angstrom*label->xdim, Angstrom*label->ydim, FALSE);
     /*
-    gwy_data_field_set_xoffset(field, Angstrom*label->xoff);
-    gwy_data_field_set_yoffset(field, Angstrom*label->yoff);
+    gwy_field_set_xoffset(field, Angstrom*label->xoff);
+    gwy_field_set_yoffset(field, Angstrom*label->yoff);
     */
 
     //q_height = -0.5 * 0.136*Angstrom * label->zscale/200.0;
@@ -410,25 +400,25 @@ mul_read_image(GwyContainer *container,
 
     if (label->mode == MUL_MODE_HEIGHT || label->mode == MUL_MODE_DIFFERENT) {
         q = q_height;
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(field), "m");
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(field), "m");
+        gwy_unit_set_from_string(gwy_field_get_unit_xy(field), "m");
+        gwy_unit_set_from_string(gwy_field_get_unit_z(field), "m");
     }
     else if (label->mode == MUL_MODE_CURRENT) {
         q = q_current;
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(field), "m");
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(field), "A");
+        gwy_unit_set_from_string(gwy_field_get_unit_xy(field), "m");
+        gwy_unit_set_from_string(gwy_field_get_unit_z(field), "A");
     }
     else if (label->mode == MUL_MODE_V_X_I_Y) {
         gint xmin = label->spare_50, xmax = label->spare_51,
              ymin = label->spare_48, ymax = label->spare_49;
 
         q = q_height;
-        gwy_data_field_set_xreal(field, MAX(abs(xmax - xmin), 1)*q_voltage);
-        gwy_data_field_set_xoffset(field, xmin*q_voltage);
-        gwy_data_field_set_yreal(field, MAX(abs(ymax - ymin), 1)*q_current);
-        gwy_data_field_set_yoffset(field, ymin*q_voltage);
+        gwy_field_set_xreal(field, MAX(abs(xmax - xmin), 1)*q_voltage);
+        gwy_field_set_xoffset(field, xmin*q_voltage);
+        gwy_field_set_yreal(field, MAX(abs(ymax - ymin), 1)*q_current);
+        gwy_field_set_yoffset(field, ymin*q_voltage);
         /* No xy units, to reduce confusion as we cannot make the different. */
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(field), "m");
+        gwy_unit_set_from_string(gwy_field_get_unit_z(field), "m");
     }
     else if (label->mode == MUL_MODE_DI_DZ) {
         /* They say dZ is in spare54 but have I no idea how to use it. */
@@ -437,53 +427,48 @@ mul_read_image(GwyContainer *container,
         gint ymin = label->spare_48, ymax = label->spare_49;
 
         q = q_height;
-        gwy_data_field_set_yreal(field, MAX(abs(ymax - ymin), 1)*q_voltage);
-        gwy_data_field_set_yoffset(field, ymin*q_voltage);
+        gwy_field_set_yreal(field, MAX(abs(ymax - ymin), 1)*q_voltage);
+        gwy_field_set_yoffset(field, ymin*q_voltage);
         /* No xy units, to reduce confusion as we cannot make the different. */
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(field), "m");
+        gwy_unit_set_from_string(gwy_field_get_unit_z(field), "m");
     }
     else if (label->mode == MUL_MODE_I_Y) {
         gint ymin = label->spare_48, ymax = label->spare_49;
 
         q = q_height;
-        gwy_data_field_set_yreal(field, MAX(abs(ymax - ymin), 1)*q_current);
-        gwy_data_field_set_yoffset(field, ymin*q_current);
+        gwy_field_set_yreal(field, MAX(abs(ymax - ymin), 1)*q_current);
+        gwy_field_set_yoffset(field, ymin*q_current);
         /* No xy units, to reduce confusion as we cannot make the different. */
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(field), "m");
+        gwy_unit_set_from_string(gwy_field_get_unit_z(field), "m");
     }
     else if (label->mode == MUL_MODE_VOLTAGE) {
         q = q_voltage;
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(field), "m");
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(field), "V");
+        gwy_unit_set_from_string(gwy_field_get_unit_xy(field), "m");
+        gwy_unit_set_from_string(gwy_field_get_unit_z(field), "V");
     }
 
     rawtype = (label->mode == MUL_MODE_CURRENT ? GWY_RAW_DATA_UINT16 : GWY_RAW_DATA_SINT16);
-    data = gwy_data_field_get_data(field);
+    data = gwy_field_get_data(field);
     for (i = 0; i < label->yres; i++) {
         gwy_convert_raw_data(p + i*label->xres*gwy_raw_data_size(rawtype), label->xres, 1,
                              rawtype, GWY_BYTE_ORDER_LITTLE_ENDIAN, data + (label->yres-1 - i)*label->xres, q, 0.0);
     }
 
-    g_snprintf(key, sizeof(key), "/%d/data", label->id);
-    gwy_container_pass_object_by_name(container, key, field);
+    gwy_file_pass_image(file, label->id, field);
 
-    g_snprintf(key, sizeof(key), "/%d/data/title", label->id);
-    title = g_strdup_printf("%s, %s (%u)",
-                            label->sample, label->title, label->id);
-    gwy_container_set_string_by_name(container, key, title);
+    title = g_strdup_printf("%s, %s (%u)", label->sample, label->title, label->id);
+    gwy_file_pass_title(file, GWY_FILE_IMAGE, label->id, title);
 
-    g_snprintf(key, sizeof(key), "/%d/meta", label->id);
-    meta = mul_get_meta(label);
-    gwy_container_pass_object_by_name(container, key, meta);
+    gwy_file_pass_meta(file, GWY_FILE_IMAGE, label->id, get_meta(label));
 
-    gwy_app_channel_check_nonsquare(container, label->id);
-    gwy_file_channel_import_log_add(container, label->id, NULL, filename);
+    gwy_check_nonsquare_image(file, label->id);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, label->id, NULL, filename);
 }
 
 static GwyContainer*
-mul_get_meta(const MulImageLabel *label)
+get_meta(const MulImageLabel *label)
 {
-    GwyContainer *meta = gwy_container_new();
+    GwyContainer *meta = gwy_container_new_in_construction();
 
     gwy_container_set_string_by_name(meta, "Date",
                                      g_strdup_printf("%u-%02u-%02u %02u:%02u:%02u",
