@@ -1,6 +1,6 @@
 /*
- *  $Id: nanoeducator.c 26510 2024-08-15 17:12:23Z yeti-dn $
- *  Copyright (C) 2009 David Necas (Yeti).
+ *  $Id: nanoeducator.c 28848 2025-11-10 18:54:08Z yeti-dn $
+ *  Copyright (C) 2009-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -45,18 +45,10 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/datafield.h>
-#include <libprocess/spectra.h>
-#include <libgwydgets/gwygraphmodel.h>
-#include <libgwydgets/gwygraphbasics.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 
 #include "err.h"
 #include "get.h"
@@ -212,81 +204,81 @@ typedef struct {
     gchar comment8[NANOEDU_COMMENT_LENGTH+1];
 } NanoeduParameterHeader;
 
-static gboolean       module_register        (void);
-static gint           nanoedu_detect         (const GwyFileDetectInfo *fileinfo,
-                                              gboolean only_name);
-static GwyContainer*  nanoedu_load           (const gchar *filename,
-                                              GwyRunType mode,
-                                              GError **error);
-static gsize          nanoedu_read_header    (const guchar *buffer,
-                                              gsize size,
-                                              NanoeduFileHeader *header,
-                                              GError **error);
-static gsize          nanoedu_read_parameters(const guchar *buffer,
-                                              gsize size,
-                                              NanoeduParameterHeader *params,
-                                              GError **error);
-static GwyDataField*  nanoedu_read_data_field(const guchar *buffer,
-                                              gsize size,
-                                              gint xres,
-                                              gint yres,
-                                              gdouble xreal,
-                                              gdouble yreal,
-                                              const gchar *xyunits,
-                                              const char *zunits,
-                                              gdouble q,
-                                              GError **error);
-static GwyGraphModel* nanoedu_read_graph     (const guchar *buffer,
-                                              gsize size,
-                                              gint res,
-                                              gint ncurves,
-                                              gdouble real,
-                                              const gchar *xunits,
-                                              const char *yunits,
-                                              gdouble q,
-                                              GError **error);
-static GwySpectra*    nanoedu_read_fd_spectra(const guchar *pos_buffer,
-                                              gsize pos_size,
-                                              const guchar *data_buffer,
-                                              gsize data_size,
-                                              gint nspectra,
-                                              gint res,
-                                              gdouble xy_step,
-                                              gdouble xscale,
-                                              gdouble yscale,
-                                              GError **error);
-static GwySpectra*    nanoedu_read_iv_spectra(const guchar *pos_buffer,
-                                              gsize pos_size,
-                                              const guchar *data_buffer,
-                                              gsize data_size,
-                                              gint nspectra,
-                                              gint res,
-                                              gdouble xscale,
-                                              gdouble yscale,
-                                              gdouble vscale,
-                                              gdouble iscale,
-                                              GError **error);
-static GwySpectra*    nanoedu_read_iz_spectra(const guchar *pos_buffer,
-                                              gsize pos_size,
-                                              const guchar *data_buffer,
-                                              gsize data_size,
-                                              gint nspectra,
-                                              gint res,
-                                              gdouble xy_step,
-                                              gdouble xscale,
-                                              gdouble yscale,
-                                              gdouble iscale,
-                                              gint sp_type,
-                                              GError **error);
-static GwyContainer* nanoedu_read_meta       (const NanoeduFileHeader *header,
-                                              const NanoeduParameterHeader *param);
+static gboolean       module_register(void);
+static gint           detect_file    (const GwyFileDetectInfo *fileinfo,
+                                      gboolean only_name);
+static GwyFile*       load_file      (const gchar *filename,
+                                      GwyRunModeFlags mode,
+                                      GError **error);
+static gsize          read_header    (const guchar *buffer,
+                                      gsize size,
+                                      NanoeduFileHeader *header,
+                                      GError **error);
+static gsize          read_parameters(const guchar *buffer,
+                                      gsize size,
+                                      NanoeduParameterHeader *params,
+                                      GError **error);
+static GwyField*      read_field     (const guchar *buffer,
+                                      gsize size,
+                                      gint xres,
+                                      gint yres,
+                                      gdouble xreal,
+                                      gdouble yreal,
+                                      const gchar *xyunits,
+                                      const char *zunits,
+                                      gdouble q,
+                                      GError **error);
+static GwyGraphModel* read_graph     (const guchar *buffer,
+                                      gsize size,
+                                      gint res,
+                                      gint ncurves,
+                                      gdouble real,
+                                      const gchar *xunits,
+                                      const char *yunits,
+                                      gdouble q,
+                                      GError **error);
+static GwySpectra*    read_fd_spectra(const guchar *pos_buffer,
+                                      gsize pos_size,
+                                      const guchar *data_buffer,
+                                      gsize data_size,
+                                      gint nspectra,
+                                      gint res,
+                                      gdouble xy_step,
+                                      gdouble xscale,
+                                      gdouble yscale,
+                                      GError **error);
+static GwySpectra*    read_iv_spectra(const guchar *pos_buffer,
+                                      gsize pos_size,
+                                      const guchar *data_buffer,
+                                      gsize data_size,
+                                      gint nspectra,
+                                      gint res,
+                                      gdouble xscale,
+                                      gdouble yscale,
+                                      gdouble vscale,
+                                      gdouble iscale,
+                                      GError **error);
+static GwySpectra*    read_iz_spectra(const guchar *pos_buffer,
+                                      gsize pos_size,
+                                      const guchar *data_buffer,
+                                      gsize data_size,
+                                      gint nspectra,
+                                      gint res,
+                                      gdouble xy_step,
+                                      gdouble xscale,
+                                      gdouble yscale,
+                                      gdouble iscale,
+                                      gint sp_type,
+                                      GError **error);
+static GwyContainer*  read_meta      (const NanoeduFileHeader *header,
+                                      const NanoeduParameterHeader *param);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Nanoeducator data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.7",
+    "0.8",
     "David Nečas (Yeti)",
     "2009",
 };
@@ -298,17 +290,13 @@ module_register(void)
 {
     gwy_file_func_register("nanoeducator",
                            N_("Nanoeducator files (.mspm, .spm, .stm)"),
-                           (GwyFileDetectFunc)&nanoedu_detect,
-                           (GwyFileLoadFunc)&nanoedu_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-nanoedu_detect(const GwyFileDetectInfo *fileinfo,
-               gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     gint score = 0;
 
@@ -324,19 +312,20 @@ nanoedu_detect(const GwyFileDetectInfo *fileinfo,
     return score;
 }
 
-static GwyContainer*
-nanoedu_load(const gchar *filename,
-             G_GNUC_UNUSED GwyRunType mode,
-             GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
     NanoeduFileHeader header;
     NanoeduParameterHeader params;
-    GwyContainer *meta = NULL, *container = NULL;
+    GwyFile *file = NULL;
+    GwyContainer *meta = NULL;
     guchar *buffer = NULL;
     const guchar *p;
     gsize len, size = 0;
     GError *err = NULL;
-    GwyDataField *dfield = NULL;
+    GwyField *dfield = NULL;
     GwyGraphModel *gmodel = NULL;
     GwySpectra *spectra = NULL;
     gdouble scale, q, qx, qy;
@@ -349,7 +338,7 @@ nanoedu_load(const gchar *filename,
     }
     p = buffer;
 
-    if (!(len = nanoedu_read_header(p, size, &header, error)))
+    if (!(len = read_header(p, size, &header, error)))
         goto finish;
     p += len;
 
@@ -358,12 +347,12 @@ nanoedu_load(const gchar *filename,
         goto finish;
     }
 
-    if (!(len = nanoedu_read_parameters(p, size - (p - buffer), &params, error)))
+    if (!(len = read_parameters(p, size - (p - buffer), &params, error)))
         goto finish;
     p += len;
 
-    container = gwy_container_new();
-    meta = nanoedu_read_meta(&header, &params);
+    file = gwy_file_new_in_construction();
+    meta = read_meta(&header, &params);
 
     scale = Nanometer * params.xy_step;
     /* Version 12+ */
@@ -391,22 +380,22 @@ nanoedu_load(const gchar *filename,
                       params.topo_ny, header.topo_ny);
         }
 
-        dfield = nanoedu_read_data_field(buffer + header.topo_offset,
-                                         size - header.topo_offset,
-                                         header.topo_nx, header.topo_ny,
-                                         scale*header.topo_nx, scale*header.topo_ny,
-                                         "m", "m", q*Nanometer, error);
+        dfield = read_field(buffer + header.topo_offset,
+                            size - header.topo_offset,
+                            header.topo_nx, header.topo_ny,
+                            scale*header.topo_nx, scale*header.topo_ny,
+                            "m", "m", q*Nanometer, error);
         if (!dfield)
             goto finish;
 
         /* Setting data field offsets: */
-        gwy_data_field_set_xoffset(dfield, params.x_offset * Nanometer);
-        gwy_data_field_set_yoffset(dfield, -params.y_offset * Nanometer - scale * header.topo_ny);
+        gwy_field_set_xoffset(dfield, params.x_offset * Nanometer);
+        gwy_field_set_yoffset(dfield, -params.y_offset * Nanometer - scale * header.topo_ny);
 
-        gwy_container_pass_object(container, gwy_app_get_data_key_for_id(0), dfield);
-        gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(0), "Topography");
-        gwy_container_set_object(container, gwy_app_get_data_meta_key_for_id(0), meta);
-        gwy_file_channel_import_log_add(container, 0, NULL, filename);
+        gwy_file_pass_image(file, 0, dfield);
+        gwy_file_set_title(file, GWY_FILE_IMAGE, 0, "Topography", FALSE);
+        gwy_file_pass_meta(file, GWY_FILE_IMAGE, 0, gwy_container_copy(meta));
+        gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
         nobjects++;
     }
 
@@ -429,18 +418,18 @@ nanoedu_load(const gchar *filename,
         q *= Nanometer;
         units = "m";
 
-        gmodel = nanoedu_read_graph(buffer + header.topo_offset,
-                                    size - header.topo_offset,
-                                    header.topo_nx, header.topo_ny,
-                                    scale*header.topo_nx,
-                                    "m", units, q, error);
+        gmodel = read_graph(buffer + header.topo_offset,
+                            size - header.topo_offset,
+                            header.topo_nx, header.topo_ny,
+                            scale*header.topo_nx,
+                            "m", units, q, error);
         if (!gmodel)
             goto finish;
 
         g_object_set(gmodel,
                      "title", params.path_mode ? "Scanner Training (Y+)" : "Scanner Training (X+)",
                      NULL);
-        gwy_container_pass_object_by_name(container, "/0/graph/graph/1", gmodel);
+        gwy_file_pass_graph(file, 1, gmodel);
         nobjects++;
 
         /* This was already addsurf, so do not attempt to read it again. */
@@ -464,41 +453,42 @@ nanoedu_load(const gchar *filename,
         /* FIXME: This might be wrong, however, there does not seem to be any
          * other discriminating quantity in the headers. */
         if (params.probe_type == 0) {
-            spectra = nanoedu_read_fd_spectra(buffer + header.point_offset,
-                                              header.spec_offset - header.point_offset,
-                                              buffer + header.spec_offset,
-                                              size - header.spec_offset,
-                                              params.n_spectra_lines, params.n_spectrum_points,
-                                              Nanometer*q, Nanometer*qx, Nanometer*qy,
-                                              error);
+            spectra = read_fd_spectra(buffer + header.point_offset,
+                                      header.spec_offset - header.point_offset,
+                                      buffer + header.spec_offset,
+                                      size - header.spec_offset,
+                                      params.n_spectra_lines, params.n_spectrum_points,
+                                      Nanometer*q, Nanometer*qx, Nanometer*qy,
+                                      error);
         }
         else if ((params.spectroscopy_type == 1)
                  || (size - header.spec_offset == 4 * params.n_spectra_lines * params.n_spectrum_points)) {
-            spectra = nanoedu_read_iz_spectra(buffer + header.point_offset,
-                                              header.spec_offset - header.point_offset,
-                                              buffer + header.spec_offset,
-                                              size - header.spec_offset,
-                                              params.n_spectra_lines, params.n_spectrum_points,
-                                              Nanometer*q, Nanometer*qx, Nanometer*qy,
-                                              4.0*NanoAmpere/params.nA_D,
-                                              params.spectroscopy_type,
-                                              error);
+            spectra = read_iz_spectra(buffer + header.point_offset,
+                                      header.spec_offset - header.point_offset,
+                                      buffer + header.spec_offset,
+                                      size - header.spec_offset,
+                                      params.n_spectra_lines, params.n_spectrum_points,
+                                      Nanometer*q, Nanometer*qx, Nanometer*qy,
+                                      4.0*NanoAmpere/params.nA_D,
+                                      params.spectroscopy_type,
+                                      error);
         }
         else {
-            spectra = nanoedu_read_iv_spectra(buffer + header.point_offset,
-                                              header.spec_offset - header.point_offset,
-                                              buffer + header.spec_offset,
-                                              size - header.spec_offset,
-                                              params.n_spectra_lines, params.n_spectrum_points,
-                                              Nanometer*qx, Nanometer*qy, 1e-3*params.discr_z_mvolt,
-                                              4.0*NanoAmpere/params.nA_D,
-                                              error);
+            spectra = read_iv_spectra(buffer + header.point_offset,
+                                      header.spec_offset - header.point_offset,
+                                      buffer + header.spec_offset,
+                                      size - header.spec_offset,
+                                      params.n_spectra_lines, params.n_spectrum_points,
+                                      Nanometer*qx, Nanometer*qy, 1e-3*params.discr_z_mvolt,
+                                      4.0*NanoAmpere/params.nA_D,
+                                      error);
         }
 
         if (!spectra)
             goto finish;
 
-        gwy_container_pass_object_by_name(container, "/sps/0", spectra);
+        gwy_file_pass_spectra(file, 0, spectra);
+        gwy_log_add_import(file, GWY_FILE_SPECTRA, 0, NULL, filename);
         nobjects++;
     }
 
@@ -554,18 +544,18 @@ nanoedu_load(const gchar *filename,
             break;
         }
 
-        dfield = nanoedu_read_data_field(buffer + header.addsurf_offset, size - header.addsurf_offset,
-                                         header.addsurf_nx, header.addsurf_ny,
-                                         scale*header.addsurf_nx, scale*header.addsurf_ny,
-                                         "m", units, q, error);
+        dfield = read_field(buffer + header.addsurf_offset, size - header.addsurf_offset,
+                            header.addsurf_nx, header.addsurf_ny,
+                            scale*header.addsurf_nx, scale*header.addsurf_ny,
+                            "m", units, q, error);
         if (!dfield)
             goto finish;
 
         /* Setting data field offsets: */
-        gwy_data_field_set_xoffset(dfield, params.x_offset * Nanometer);
-        gwy_data_field_set_yoffset(dfield, -params.y_offset * Nanometer - scale*header.addsurf_ny);
+        gwy_field_set_xoffset(dfield, params.x_offset * Nanometer);
+        gwy_field_set_yoffset(dfield, -params.y_offset * Nanometer - scale*header.addsurf_ny);
 
-        gwy_container_pass_object(container, gwy_app_get_data_key_for_id(1), dfield);
+        gwy_file_pass_image(file, 1, dfield);
         title = gwy_enuml_to_string(params.aqui_add,
                                     "Topography", NANOEDU_TOPOGRAPHY,
                                     "Work Force",  NANOEDU_WORK_FORCE,
@@ -583,10 +573,10 @@ nanoedu_load(const gchar *filename,
                                     "Sens. Correction", NANOEDU_SENS_CORRECTION,
                                     NULL);
         if (title && *title)
-            gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(1), title);
+            gwy_file_set_title(file, GWY_FILE_IMAGE, 1, title, FALSE);
 
-        gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(1), gwy_container_duplicate(meta));
-        gwy_file_channel_import_log_add(container, 1, NULL, filename);
+        gwy_file_pass_meta(file, GWY_FILE_IMAGE, 1, gwy_container_copy(meta));
+        gwy_log_add_import(file, GWY_FILE_IMAGE, 1, NULL, filename);
         nobjects++;
     }
 
@@ -595,21 +585,21 @@ nanoedu_load(const gchar *filename,
 
 finish:
     gwy_file_abandon_contents(buffer, size, NULL);
-    GWY_OBJECT_UNREF(meta);
+    g_clear_object(&meta);
 
     if (!nobjects)
-        GWY_OBJECT_UNREF(container);
+        g_clear_object(&file);
     else
         g_clear_error(error);
 
-    return container;
+    return file;
 }
 
 static gsize
-nanoedu_read_header(const guchar *buffer,
-                    gsize size,
-                    NanoeduFileHeader *header,
-                    GError **error)
+read_header(const guchar *buffer,
+            gsize size,
+            NanoeduFileHeader *header,
+            GError **error)
 {
     if (size < NANOEDU_HEADER_SIZE) {
         err_TRUNCATED_HEADER(error);
@@ -664,10 +654,10 @@ nanoedu_read_header(const guchar *buffer,
 }
 
 static gsize
-nanoedu_read_parameters(const guchar *buffer,
-                        gsize size,
-                        NanoeduParameterHeader *params,
-                        GError **error)
+read_parameters(const guchar *buffer,
+                gsize size,
+                NanoeduParameterHeader *params,
+                GError **error)
 {
     if (size < NANOEDU_PARAMS_SIZE) {
         err_TRUNCATED_PART(error, "NanoeduParameterHeader");
@@ -799,16 +789,16 @@ nanoedu_read_parameters(const guchar *buffer,
     return NANOEDU_PARAMS_SIZE;
 }
 
-static GwyDataField*
-nanoedu_read_data_field(const guchar *buffer,
-                        gsize size,
-                        gint xres, gint yres,
-                        gdouble xreal, gdouble yreal,
-                        const gchar *xyunits, const char *zunits,
-                        gdouble q,
-                        GError **error)
+static GwyField*
+read_field(const guchar *buffer,
+           gsize size,
+           gint xres, gint yres,
+           gdouble xreal, gdouble yreal,
+           const gchar *xyunits, const char *zunits,
+           gdouble q,
+           GError **error)
 {
-    GwyDataField *dfield;
+    GwyField *dfield;
 
     if (err_SIZE_MISMATCH(error, 2*xres*yres, size, FALSE))
         return NULL;
@@ -816,29 +806,29 @@ nanoedu_read_data_field(const guchar *buffer,
     sanitise_real_size(&xreal, "x size");
     sanitise_real_size(&yreal, "y size");
 
-    dfield = gwy_data_field_new(xres, yres, xreal, yreal, FALSE);
+    dfield = gwy_field_new(xres, yres, xreal, yreal, FALSE);
     gwy_convert_raw_data(buffer, xres*yres, 1, GWY_RAW_DATA_SINT16, GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                         gwy_data_field_get_data(dfield), q, 0.0);
-    gwy_data_field_invert(dfield, TRUE, FALSE, FALSE);
+                         gwy_field_get_data(dfield), q, 0.0);
+    gwy_field_flip(dfield, FALSE, TRUE);
 
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), xyunits);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), zunits);
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), xyunits);
+    gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), zunits);
 
     return dfield;
 }
 
 static GwyGraphModel*
-nanoedu_read_graph(const guchar *buffer,
-                   gsize size,
-                   gint res, gint ncurves, gdouble real,
-                   const gchar *xunits, const char *yunits,
-                   gdouble q,
-                   GError **error)
+read_graph(const guchar *buffer,
+           gsize size,
+           gint res, gint ncurves, gdouble real,
+           const gchar *xunits, const char *yunits,
+           gdouble q,
+           GError **error)
 {
     gint i;
     GwyGraphModel *gmodel;
     GwyGraphCurveModel *gcmodel;
-    GwySIUnit *siunitx, *siunity;
+    GwyUnit *siunitx, *siunity;
     gdouble *xdata, *ydata;
     GString *str;
 
@@ -846,11 +836,11 @@ nanoedu_read_graph(const guchar *buffer,
         return NULL;
 
     sanitise_real_size(&real, "size");
-    siunitx = gwy_si_unit_new(xunits);
-    siunity = gwy_si_unit_new(yunits);
+    siunitx = gwy_unit_new(xunits);
+    siunity = gwy_unit_new(yunits);
     gmodel = g_object_new(GWY_TYPE_GRAPH_MODEL,
-                          "si-unit-x", siunitx,
-                          "si-unit-y", siunity,
+                          "unit-x", siunitx,
+                          "unit-y", siunity,
                           NULL);
     g_object_unref(siunitx);
     g_object_unref(siunity);
@@ -897,20 +887,20 @@ check_spectra_size(guint nspectra,
     return pointstep;
 }
 
-static GwyDataLine*
+static GwyLine*
 make_fd_spectrum(gint res, gdouble xy_step, const gint16 *d16, gboolean flip)
 {
-    GwyDataLine *dline;
+    GwyLine *dline;
     gdouble *data;
     gint j, amin;
     gdouble z0;
     gint16 v, a;
 
-    dline = gwy_data_line_new(res, xy_step*res, FALSE);
-    gwy_si_unit_set_from_string(gwy_data_line_get_si_unit_x(dline), "m");
+    dline = gwy_line_new(res, xy_step*res, FALSE);
+    gwy_unit_set_from_string(gwy_line_get_unit_x(dline), "m");
     /* y is unitless */
 
-    data = gwy_data_line_get_data(dline);
+    data = gwy_line_get_data(dline);
     amin = G_MAXINT;
     z0 = 1.0;
     /* XXX: The odd coordinates are abscissas.  We only use the zeroth for setting the offset.  If they are not
@@ -932,23 +922,23 @@ make_fd_spectrum(gint res, gdouble xy_step, const gint16 *d16, gboolean flip)
             z0 = data[j];
         }
     }
-    gwy_data_line_multiply(dline, 1.0/z0);
-    gwy_data_line_set_offset(dline, xy_step * GINT16_FROM_LE(d16[flip ? 2*(res-1) + 1 : 1]));
+    gwy_line_multiply(dline, 1.0/z0);
+    gwy_line_set_offset(dline, xy_step * GINT16_FROM_LE(d16[flip ? 2*(res-1) + 1 : 1]));
 
     return dline;
 }
 
 static GwySpectra*
-nanoedu_read_fd_spectra(const guchar *pos_buffer, gsize pos_size,
-                        const guchar *data_buffer, gsize data_size,
-                        gint nspectra, gint res,
-                        gdouble xy_step,
-                        gdouble xscale, gdouble yscale,
-                        GError **error)
+read_fd_spectra(const guchar *pos_buffer, gsize pos_size,
+                const guchar *data_buffer, gsize data_size,
+                gint nspectra, gint res,
+                gdouble xy_step,
+                gdouble xscale, gdouble yscale,
+                GError **error)
 {
     gint i, j, n, pointstep;
     GwySpectra *spectra;
-    GwyDataLine *dline;
+    GwyLine *dline;
     const gint16 *p16 = (const gint16*)pos_buffer;
     const gint16 *d16 = (const gint16*)data_buffer;
     gdouble x, y;
@@ -961,7 +951,7 @@ nanoedu_read_fd_spectra(const guchar *pos_buffer, gsize pos_size,
 
     sanitise_real_size(&xy_step, "step size");
     spectra = gwy_spectra_new();
-    gwy_si_unit_set_from_string(gwy_spectra_get_si_unit_xy(spectra), "m");
+    gwy_unit_set_from_string(gwy_spectra_get_unit_xy(spectra), "m");
     gwy_spectra_set_title(spectra, _("F-D spectra"));
     gwy_spectra_set_spectrum_x_label(spectra, "D");
     gwy_spectra_set_spectrum_y_label(spectra, "F");
@@ -991,38 +981,38 @@ nanoedu_read_fd_spectra(const guchar *pos_buffer, gsize pos_size,
     return spectra;
 }
 
-static GwyDataLine*
+static GwyLine*
 make_iv_spectrum(gint res, gdouble xy_step,
                  const gint16 *d16, gdouble q)
 {
-    GwyDataLine *dline;
+    GwyLine *dline;
     gint first = GINT16_FROM_LE(d16[0]), last = GINT16_FROM_LE(d16[2*(res - 1)]);
 
-    dline = gwy_data_line_new(res, xy_step * abs(last - first), FALSE);
-    gwy_si_unit_set_from_string(gwy_data_line_get_si_unit_x(dline), "V");
-    gwy_si_unit_set_from_string(gwy_data_line_get_si_unit_y(dline), "A");
+    dline = gwy_line_new(res, xy_step * abs(last - first), FALSE);
+    gwy_unit_set_from_string(gwy_line_get_unit_x(dline), "V");
+    gwy_unit_set_from_string(gwy_line_get_unit_y(dline), "A");
 
     /* XXX: The even coordinates are abscissas; the odd coordinates are values.  We only use the zeroth for setting
      * the offset.  If they are not equidistant, though luck... */
     gwy_convert_raw_data(d16+1, res, 2, GWY_RAW_DATA_SINT16, GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                         gwy_data_line_get_data(dline), q, 0.0);
+                         gwy_line_get_data(dline), q, 0.0);
 
-    gwy_data_line_set_offset(dline, xy_step*first);
+    gwy_line_set_offset(dline, xy_step*first);
 
     return dline;
 }
 
 static GwySpectra*
-nanoedu_read_iv_spectra(const guchar *pos_buffer, gsize pos_size,
-                        const guchar *data_buffer, gsize data_size,
-                        gint nspectra, gint res,
-                        gdouble xscale, gdouble yscale,
-                        gdouble vscale, gdouble iscale,
-                        GError **error)
+read_iv_spectra(const guchar *pos_buffer, gsize pos_size,
+                const guchar *data_buffer, gsize data_size,
+                gint nspectra, gint res,
+                gdouble xscale, gdouble yscale,
+                gdouble vscale, gdouble iscale,
+                GError **error)
 {
     gint i, j, n, pointstep;
     GwySpectra *spectra;
-    GwyDataLine *dline;
+    GwyLine *dline;
     const gint16 *p16 = (const gint16*)pos_buffer;
     const gint16 *d16 = (const gint16*)data_buffer;
     gdouble x, y;
@@ -1034,7 +1024,7 @@ nanoedu_read_iv_spectra(const guchar *pos_buffer, gsize pos_size,
         return NULL;
 
     spectra = gwy_spectra_new();
-    gwy_si_unit_set_from_string(gwy_spectra_get_si_unit_xy(spectra), "m");
+    gwy_unit_set_from_string(gwy_spectra_get_unit_xy(spectra), "m");
     gwy_spectra_set_title(spectra, _("I-V spectra"));
     gwy_spectra_set_spectrum_x_label(spectra, "V");
     gwy_spectra_set_spectrum_y_label(spectra, "I");
@@ -1058,40 +1048,40 @@ nanoedu_read_iv_spectra(const guchar *pos_buffer, gsize pos_size,
     return spectra;
 }
 
-static GwyDataLine*
+static GwyLine*
 make_iz_spectrum(gint res, gdouble xy_step,
                  const gint16 *d16, gdouble q)
 {
-    GwyDataLine *dline;
+    GwyLine *dline;
     gint first = GINT16_FROM_LE(d16[1]), last = GINT16_FROM_LE(d16[2*res - 1]);
     gdouble xstep = last - first;
 
-    dline = gwy_data_line_new(res, xy_step*xstep, FALSE);
-    gwy_si_unit_set_from_string(gwy_data_line_get_si_unit_x(dline), "m");
-    gwy_si_unit_set_from_string(gwy_data_line_get_si_unit_y(dline), "A");
+    dline = gwy_line_new(res, xy_step*xstep, FALSE);
+    gwy_unit_set_from_string(gwy_line_get_unit_x(dline), "m");
+    gwy_unit_set_from_string(gwy_line_get_unit_y(dline), "A");
 
     /* XXX: The odd coordinates are abscissas; the even coordinates are values.  We use the zeroth and last for
      * x offset and scale. If they are not equidistant, though luck... */
     gwy_convert_raw_data(d16, res, 2, GWY_RAW_DATA_SINT16, GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                         gwy_data_line_get_data(dline), q, 0.0);
-    gwy_data_line_set_offset(dline, xy_step*first);
+                         gwy_line_get_data(dline), q, 0.0);
+    gwy_line_set_offset(dline, xy_step*first);
 
     return dline;
 }
 
 static GwySpectra*
-nanoedu_read_iz_spectra(const guchar *pos_buffer, gsize pos_size,
-                        const guchar *data_buffer, gsize data_size,
-                        gint nspectra, gint res,
-                        gdouble xy_step,
-                        gdouble xscale, gdouble yscale,
-                        gdouble iscale,
-                        gint sp_type,
-                        GError **error)
+read_iz_spectra(const guchar *pos_buffer, gsize pos_size,
+                const guchar *data_buffer, gsize data_size,
+                gint nspectra, gint res,
+                gdouble xy_step,
+                gdouble xscale, gdouble yscale,
+                gdouble iscale,
+                gint sp_type,
+                GError **error)
 {
     gint i, j, n, pointstep;
     GwySpectra *spectra;
-    GwyDataLine *dline;
+    GwyLine *dline;
     const gint16 *p16 = (const gint16*)pos_buffer;
     const gint16 *d16 = (const gint16*)data_buffer;
     gdouble x, y;
@@ -1103,7 +1093,7 @@ nanoedu_read_iz_spectra(const guchar *pos_buffer, gsize pos_size,
         return NULL;
 
     spectra = gwy_spectra_new();
-    gwy_si_unit_set_from_string(gwy_spectra_get_si_unit_xy(spectra), "m");
+    gwy_unit_set_from_string(gwy_spectra_get_unit_xy(spectra), "m");
     gwy_spectra_set_title(spectra, _("I-Z spectra"));
     gwy_spectra_set_spectrum_x_label(spectra, "Z");
     gwy_spectra_set_spectrum_y_label(spectra, "I");
@@ -1137,12 +1127,12 @@ nanoedu_read_iz_spectra(const guchar *pos_buffer, gsize pos_size,
 }
 
 static GwyContainer*
-nanoedu_read_meta(const NanoeduFileHeader *header,
-                  const NanoeduParameterHeader *param)
+read_meta(const NanoeduFileHeader *header,
+          const NanoeduParameterHeader *param)
 {
     GwyContainer *meta;
 
-    meta = gwy_container_new();
+    meta = gwy_container_new_in_construction();
     gwy_container_set_string_by_name(meta, "Version",
                                      g_strdup_printf("%u", header->version));
     gwy_container_set_string_by_name(meta, "Date",
