@@ -1,6 +1,6 @@
 /*
- *  $Id: nanomagnetics.c 26504 2024-08-14 13:47:40Z yeti-dn $
- *  Copyright (C) 2012-2024 Sameer Grover, David Necas (Yeti).
+ *  $Id: nanomagnetics.c 28805 2025-11-05 12:28:20Z yeti-dn $
+ *  Copyright (C) 2012-2025 Sameer Grover, David Necas (Yeti).
  *  E-mail: sameer.grover.1@gmail.com, yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -53,13 +53,9 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/stats.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -330,75 +326,75 @@ typedef struct {
 
 /* END PARAMETER STRUCTS */
 
-static gboolean      module_register          (void);
-static gint          nmi_detect               (const GwyFileDetectInfo *fileinfo,
-                                               gboolean only_name);
-static GwyContainer* nmi_load                 (const gchar *filename,
-                                               GwyRunType mode,
-                                               GError **error);
-static gsize         read_footer              (NMIParameters *params,
-                                               const guchar **p,
-                                               gsize size);
-static void          populate_meta_data       (GwyContainer *meta,
-                                               NMIParameters *params,
-                                               NMIChannelParameters *chparams);
-static void          cleanup_global_parameters(NMIParameters *params);
-static void          cleanup_channels         (NMIChannelParameters *channels,
-                                               gint no_of_channels);
-static void          cleanup_data_fields      (GwyDataField **dfields,
-                                               gint no_of_channels);
-static void          get_ordinate_scale       (const NMIParameters *params,
-                                               const NMIChannelParameters *chparams,
-                                               gdouble *q,
-                                               gdouble *z0,
-                                               NMIUnit *unit);
-static gboolean      calc_real_ordinate       (const NMIParameters *params,
-                                               const NMIChannelParameters *chparams,
-                                               gdouble *z,
-                                               NMIUnit *unit);
-static gboolean      calc_real_ordinate_base  (const NMIParameters *params,
-                                               NMIChannel channel,
-                                               gint numAdds,
-                                               gint16 gain,
-                                               gdouble *z,
-                                               NMIUnit *unit);
-static gboolean      calc_real_ordinate_AFM   (const NMIParameters *params,
-                                               NMIChannel channel,
-                                               gint numAdds,
-                                               guint16 gain,
-                                               gdouble *z,
-                                               NMIUnit *unit);
-static gboolean      calc_real_ordinate_NCAFM (const NMIParameters *params,
-                                               NMIChannel channel,
-                                               gint numAdds,
-                                               guint16 gain,
-                                               gdouble *z,
-                                               NMIUnit *unit);
-static gboolean      calc_real_ordinate_SHPM  (const NMIParameters *params,
-                                               NMIChannel channel,
-                                               gint numAdds,
-                                               guint16 gain,
-                                               gdouble *z,
-                                               NMIUnit *unit);
-static gboolean      is_laser_mode            (NMISpmType spm_type,
-                                               NMILtMode lt_mode,
-                                               NMILaserMode laser_mode);
-static gboolean      is_hall_channel          (NMIChannel channel,
-                                               gboolean also_Vz4,
-                                               gboolean also_HallVoltage);
-static gdouble       convert_hall             (gdouble val,
-                                               const NMIParameters *params);
-static gdouble       convert_pll              (gdouble val,
-                                               const NMIParameters *params);
-static gchar*        get_string_LEB128        (const guchar **p,
-                                               gsize *size);
+static gboolean module_register          (void);
+static gint     detect_file              (const GwyFileDetectInfo *fileinfo,
+                                          gboolean only_name);
+static GwyFile* load_file                (const gchar *filename,
+                                          GwyRunModeFlags mode,
+                                          GError **error);
+static gsize    read_footer              (NMIParameters *params,
+                                          const guchar **p,
+                                          gsize size);
+static void     populate_meta_data       (GwyContainer *meta,
+                                          NMIParameters *params,
+                                          NMIChannelParameters *chparams);
+static void     cleanup_global_parameters(NMIParameters *params);
+static void     cleanup_channels         (NMIChannelParameters *channels,
+                                          gint no_of_channels);
+static void     cleanup_fields           (GwyField **dfields,
+                                          gint no_of_channels);
+static void     get_ordinate_scale       (const NMIParameters *params,
+                                          const NMIChannelParameters *chparams,
+                                          gdouble *q,
+                                          gdouble *z0,
+                                          NMIUnit *unit);
+static gboolean calc_real_ordinate       (const NMIParameters *params,
+                                          const NMIChannelParameters *chparams,
+                                          gdouble *z,
+                                          NMIUnit *unit);
+static gboolean calc_real_ordinate_base  (const NMIParameters *params,
+                                          NMIChannel channel,
+                                          gint numAdds,
+                                          gint16 gain,
+                                          gdouble *z,
+                                          NMIUnit *unit);
+static gboolean calc_real_ordinate_AFM   (const NMIParameters *params,
+                                          NMIChannel channel,
+                                          gint numAdds,
+                                          guint16 gain,
+                                          gdouble *z,
+                                          NMIUnit *unit);
+static gboolean calc_real_ordinate_NCAFM (const NMIParameters *params,
+                                          NMIChannel channel,
+                                          gint numAdds,
+                                          guint16 gain,
+                                          gdouble *z,
+                                          NMIUnit *unit);
+static gboolean calc_real_ordinate_SHPM  (const NMIParameters *params,
+                                          NMIChannel channel,
+                                          gint numAdds,
+                                          guint16 gain,
+                                          gdouble *z,
+                                          NMIUnit *unit);
+static gboolean is_laser_mode            (NMISpmType spm_type,
+                                          NMILtMode lt_mode,
+                                          NMILaserMode laser_mode);
+static gboolean is_hall_channel          (NMIChannel channel,
+                                          gboolean also_Vz4,
+                                          gboolean also_HallVoltage);
+static gdouble  convert_hall             (gdouble val,
+                                          const NMIParameters *params);
+static gdouble  convert_pll              (gdouble val,
+                                          const NMIParameters *params);
+static gchar*   get_string_LEB128        (const guchar **p,
+                                          gsize *size);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Nanomagnetics' NMI file format version 3 and 5"),
     "Sameer Grover <sameer.grover.1@gmail.com>",
-    "1.3",
+    "2.0",
     "Sameer Grover, Tata Institute of Fundamental Research",
     "2012",
 };
@@ -410,16 +406,13 @@ module_register(void)
 {
     gwy_file_func_register("nanomagnetics",
                            _("Nanomagnetics File (.nmi)"),
-                           (GwyFileDetectFunc)&nmi_detect,
-                           (GwyFileLoadFunc)&nmi_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-nmi_detect(const GwyFileDetectInfo * fileinfo, gboolean only_name)
+detect_file(const GwyFileDetectInfo * fileinfo, gboolean only_name)
 {
     if (only_name)
         return g_str_has_suffix(fileinfo->name_lowercase, EXTENSION) ? 20 : 0;
@@ -431,16 +424,18 @@ nmi_detect(const GwyFileDetectInfo * fileinfo, gboolean only_name)
     return 0;
 }
 
-/* THE MAIN FUNCTION */
-static GwyContainer *
-nmi_load(const gchar *filename, G_GNUC_UNUSED GwyRunType mode, GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
     guchar *buffer;
     gsize size, originalsize;
     GError *err = NULL;
     const guchar *p;
-    GwyContainer *container, *meta;
-    GwyDataField **dfields;
+    GwyFile *file;
+    GwyContainer *meta;
+    GwyField **dfields;
     gdouble *data;
     gint channelno, i;       /*loop counters */
     NMIParameters params;
@@ -453,7 +448,7 @@ nmi_load(const gchar *filename, G_GNUC_UNUSED GwyRunType mode, GError **error)
         return NULL;
     }
 
-    gwy_clear(&params, 1);
+    gwy_clear1(params);
     originalsize = size;
 
     if (size <= MAGIC_SIZE) {
@@ -462,8 +457,7 @@ nmi_load(const gchar *filename, G_GNUC_UNUSED GwyRunType mode, GError **error)
         return NULL;
     }
 
-    if (memcmp(buffer, MAGIC3, MAGIC_SIZE) != 0 
-        && memcmp(buffer, MAGIC5, MAGIC_SIZE) != 0) {
+    if (memcmp(buffer, MAGIC3, MAGIC_SIZE) != 0 && memcmp(buffer, MAGIC5, MAGIC_SIZE) != 0) {
         err_FILE_TYPE(error, "Nanomagnetics");
         gwy_file_abandon_contents(buffer, originalsize, NULL);
         return NULL;
@@ -624,7 +618,7 @@ nmi_load(const gchar *filename, G_GNUC_UNUSED GwyRunType mode, GError **error)
     params.FeedBackValue = gwy_get_gfloat_le(&p);
     size -= headersize;
 
-    dfields = g_new0(GwyDataField*, params.numImages);
+    dfields = g_new0(GwyField*, params.numImages);
     channel_params = g_new0(NMIChannelParameters, params.numImages);
 
     /* Channels start */
@@ -636,7 +630,7 @@ nmi_load(const gchar *filename, G_GNUC_UNUSED GwyRunType mode, GError **error)
             err_TOO_SHORT(error);
             gwy_file_abandon_contents(buffer, originalsize, NULL);
             cleanup_channels(channel_params, params.numImages);
-            cleanup_data_fields(dfields, params.numImages);
+            cleanup_fields(dfields, params.numImages);
             cleanup_global_parameters(&params);
             return NULL;
         }
@@ -645,7 +639,7 @@ nmi_load(const gchar *filename, G_GNUC_UNUSED GwyRunType mode, GError **error)
             err_TOO_SHORT(error);
             gwy_file_abandon_contents(buffer, originalsize, NULL);
             cleanup_channels(channel_params, params.numImages);
-            cleanup_data_fields(dfields, params.numImages);
+            cleanup_fields(dfields, params.numImages);
             cleanup_global_parameters(&params);
             return NULL;
         }
@@ -665,15 +659,15 @@ nmi_load(const gchar *filename, G_GNUC_UNUSED GwyRunType mode, GError **error)
         if (err_SIZE_MISMATCH(error, datasize + sizeof(gint32), size, FALSE)) {
             gwy_file_abandon_contents(buffer, originalsize, NULL);
             cleanup_channels(channel_params, params.numImages);
-            cleanup_data_fields(dfields, params.numImages);
+            cleanup_fields(dfields, params.numImages);
             cleanup_global_parameters(&params);
             return NULL;
         }
-        dfields[channelno] = gwy_data_field_new(chparams->Width, chparams->Height,
-                                                chparams->RealWidth * 1.0e-10, chparams->RealHeight * 1.0e-10,
-                                                FALSE);
+        dfields[channelno] = gwy_field_new(chparams->Width, chparams->Height,
+                                           chparams->RealWidth * 1.0e-10, chparams->RealHeight * 1.0e-10,
+                                           FALSE);
         gwy_convert_raw_data(p, chparams->Width*chparams->Height, 1, GWY_RAW_DATA_SINT32, GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                             gwy_data_field_get_data(dfields[channelno]), 1.0, 0.0);
+                             gwy_field_get_data(dfields[channelno]), 1.0, 0.0);
 
         p += datasize;
         chparams->NumAdds = gwy_get_gint32_le(&p);
@@ -687,46 +681,46 @@ nmi_load(const gchar *filename, G_GNUC_UNUSED GwyRunType mode, GError **error)
     size = read_footer(&params, &p, size);
 
     /* Read channels all over again, scale the values, set the units and metadata */
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     for (channelno = 0; channelno < params.numImages; channelno++) {
         NMIChannelParameters *chparams = channel_params + channelno;
-        GwyDataField *dfield = dfields[channelno];
+        GwyField *dfield = dfields[channelno];
         NMIUnit unit;
         gdouble q, z0;
 
         get_ordinate_scale(&params, chparams, &q, &z0, &unit);
-        data = gwy_data_field_get_data(dfield);
+        data = gwy_field_get_data(dfield);
         for (i = 0; i < chparams->Width * chparams->Height; i++)
             data[i] = q*data[i] + z0;
 
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield),
-                                    gwy_enuml_to_string(unit,
-                                                        "V", UNIT_Volt,
-                                                        "A", UNIT_Ampere,
-                                                        "Hz", UNIT_Hertz,
-                                                        "m", UNIT_metre,
-                                                        "G", UNIT_Gauss,
-                                                        "levels", UNIT_levels,
-                                                        "deg", UNIT_degree,
-                                                        NULL));
+        gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
+        gwy_unit_set_from_string(gwy_field_get_unit_z(dfield),
+                                 gwy_enuml_to_string(unit,
+                                                     "V", UNIT_Volt,
+                                                     "A", UNIT_Ampere,
+                                                     "Hz", UNIT_Hertz,
+                                                     "m", UNIT_metre,
+                                                     "G", UNIT_Gauss,
+                                                     "levels", UNIT_levels,
+                                                     "deg", UNIT_degree,
+                                                     NULL));
 
-        gwy_container_set_object(container, gwy_app_get_data_key_for_id(channelno), dfield);
-        gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(channelno), chparams->Name);
+        gwy_file_set_image(file, channelno, dfield);
+        gwy_file_set_title(file, GWY_FILE_IMAGE, channelno, chparams->Name, FALSE);
         /* set metadata */
-        meta = gwy_container_new();
+        meta = gwy_container_new_in_construction();
         populate_meta_data(meta, &params, chparams);
-        gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(channelno), meta);
-        gwy_file_channel_import_log_add(container, channelno, NULL, filename);
+        gwy_file_pass_meta(file, GWY_FILE_IMAGE, channelno, meta);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, channelno, NULL, filename);
     }
 
     cleanup_channels(channel_params, params.numImages);
-    cleanup_data_fields(dfields, params.numImages);
+    cleanup_fields(dfields, params.numImages);
     cleanup_global_parameters(&params);
 
     gwy_file_abandon_contents(buffer, originalsize, NULL);
 
-    return container;
+    return file;
 }
 
 static gboolean
@@ -1234,12 +1228,12 @@ cleanup_channels(NMIChannelParameters *channels, gint no_of_channels)
 }
 
 static void
-cleanup_data_fields(GwyDataField **dfields, gint no_of_channels)
+cleanup_fields(GwyField **dfields, gint no_of_channels)
 {
     gint i;
 
     for (i = 0; i < no_of_channels; i++)
-        GWY_OBJECT_UNREF(dfields[i]);
+        g_clear_object(dfields + i);
     g_free(dfields);
 }
 
