@@ -1,6 +1,6 @@
 /*
- *  $Id: nanonics.c 25680 2023-09-18 16:35:33Z yeti-dn $
- *  Copyright (C) 2009-2011 David Necas (Yeti).
+ *  $Id: nanonics.c 28911 2025-11-24 18:27:42Z yeti-dn $
+ *  Copyright (C) 2009-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -41,17 +41,10 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyexpr.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <libprocess/datafield.h>
-#include <libprocess/stats.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -80,28 +73,28 @@ typedef struct {
     GHashTable **pagemeta;
 } NanonicsFile;
 
-static gboolean      module_register          (void);
-static gint          nanonics_detect          (const GwyFileDetectInfo *fileinfo,
-                                               gboolean only_name);
-static GwyContainer* nanonics_load            (const gchar *filename,
-                                               GwyRunType mode,
-                                               GError **error);
-static GHashTable*   nanonics_read_header     (gchar *text,
-                                               const gchar *name,
-                                               GError **error);
-static void          nanonics_parse_comment   (GHashTable *hash,
-                                               const gchar *comment);
-static void          nanonics_read_data_fields(const NanonicsFile *nfile,
-                                               guint id,
-                                               const guchar *buffer,
-                                               GwyDataField **fields);
+static gboolean    module_register(void);
+static gint        detect_file    (const GwyFileDetectInfo *fileinfo,
+                                   gboolean only_name);
+static GwyFile*    load_file      (const gchar *filename,
+                                   GwyRunModeFlags mode,
+                                   GError **error);
+static GHashTable* read_header    (gchar *text,
+                                   const gchar *name,
+                                   GError **error);
+static void        parse_comment  (GHashTable *hash,
+                                   const gchar *comment);
+static void        read_fields    (const NanonicsFile *nfile,
+                                   guint id,
+                                   const guchar *buffer,
+                                   GwyField **fields);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Nanonics NAN data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.5",
+    "0.6",
     "David Nečas (Yeti)",
     "2009",
 };
@@ -113,17 +106,13 @@ module_register(void)
 {
     gwy_file_func_register("nanonics",
                            N_("Nanonics files (.nan)"),
-                           (GwyFileDetectFunc)&nanonics_detect,
-                           (GwyFileLoadFunc)&nanonics_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-nanonics_detect(const GwyFileDetectInfo *fileinfo,
-                gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     if (only_name)
         return g_str_has_suffix(fileinfo->name_lowercase, EXTENSION) ? 10 : 0;
@@ -134,14 +123,14 @@ nanonics_detect(const GwyFileDetectInfo *fileinfo,
     return 80;
 }
 
-static GwyContainer*
-nanonics_load(const gchar *filename,
-              G_GNUC_UNUSED GwyRunType mode,
-              GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
     enum { size_guess = 4096 };
 
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     guchar *buffer = NULL;
     gchar *s, *header = NULL;
     NanonicsFile nfile;
@@ -154,7 +143,7 @@ nanonics_load(const gchar *filename,
         return NULL;
     }
 
-    gwy_clear(&nfile, 1);
+    gwy_clear1(nfile);
     if (memcmp(buffer, MAGIC, MAGIC_SIZE) != 0) {
         err_FILE_TYPE(error, "Nanonics");
         goto fail;
@@ -186,7 +175,7 @@ nanonics_load(const gchar *filename,
         g_warning("Cannot convert header to UTF-8.");
     }
 
-    if (!(nfile.meta = nanonics_read_header(header, "Header", error)))
+    if (!(nfile.meta = read_header(header, "Header", error)))
         goto fail;
 
     g_free(header);
@@ -266,7 +255,7 @@ nanonics_load(const gchar *filename,
     for (i = 0; i < ndata; i++) {
         gwy_debug("reading page header %u", i);
         header = g_strndup(s + i*nfile.page_size, nfile.page_header_size);
-        if (!(nfile.pagemeta[i] = nanonics_read_header(header, "Channel Header", &err))) {
+        if (!(nfile.pagemeta[i] = read_header(header, "Channel Header", &err))) {
             if (i == 0) {
                 g_propagate_error(error, err);
                 goto fail;
@@ -280,32 +269,29 @@ nanonics_load(const gchar *filename,
         header = NULL;
     }
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     s += nfile.page_header_size;
     for (i = 0; i < ndata; i++) {
-        GwyDataField *fields[2];
+        GwyField *fields[2];
         guint j;
 
-        nanonics_read_data_fields(&nfile, i, s + i*nfile.page_size, fields);
+        read_fields(&nfile, i, s + i*nfile.page_size, fields);
         for (j = 0; j < 2; j++) {
-            gchar *key, *title;
-            GQuark quark;
+            gint id = 2*i + j;
+            gchar *title;
 
-            quark = gwy_app_get_data_key_for_id(2*i + j);
-            gwy_container_set_object(container, quark, fields[j]);
+            gwy_file_set_image(file, id, fields[j]);
             g_object_unref(fields[j]);
 
             if ((title = g_hash_table_lookup(nfile.pagemeta[i], "CHN"))) {
-                key = g_strconcat(g_quark_to_string(quark), "/title", NULL);
                 if (j) {
                     title = g_strconcat(title, " [Retrace]", NULL);
-                    gwy_container_set_string_by_name(container, key, title);
+                    gwy_file_pass_title(file, GWY_FILE_IMAGE, id, title);
                 }
                 else
-                    gwy_container_set_const_string_by_name(container, key, title);
-                g_free(key);
+                    gwy_file_set_title(file, GWY_FILE_IMAGE, id, title, FALSE);
             }
-            gwy_file_channel_import_log_add(container, 2*i + j, NULL, filename);
+            gwy_log_add_import(file, GWY_FILE_IMAGE, id, NULL, filename);
         }
     }
 
@@ -322,11 +308,11 @@ fail:
     }
     gwy_file_abandon_contents(buffer, size, NULL);
 
-    return container;
+    return file;
 }
 
 static GHashTable*
-nanonics_read_header(gchar *text, const gchar *name, GError **error)
+read_header(gchar *text, const gchar *name, GError **error)
 {
     GHashTable *hash;
     gchar *line, *p, *s, *val, *marker;
@@ -400,14 +386,13 @@ nanonics_read_header(gchar *text, const gchar *name, GError **error)
                 gwy_debug("<%s>=<%s>", s, val);
             }
         }
-
     }
 
     /* Should not happen if the comment is properly terminated. */
     if (comment)
         g_string_free(comment, TRUE);
     else if ((s = g_hash_table_lookup(hash, "comment")) && strchr(s, '\n'))
-        nanonics_parse_comment(hash, s);
+        parse_comment(hash, s);
 
     if (!line) {
         g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
@@ -426,8 +411,8 @@ nanonics_read_header(gchar *text, const gchar *name, GError **error)
 }
 
 static void
-nanonics_parse_comment(GHashTable *hash,
-                       const gchar *comment)
+parse_comment(GHashTable *hash,
+              const gchar *comment)
 {
     gchar *buffer = g_strdup(comment);
     gchar *p = buffer, *line, *value, **fields;
@@ -510,7 +495,7 @@ get_double_value(GHashTable *hash, const gchar *name, gdouble *value)
 }
 
 static gdouble
-nanonics_read_formula(const NanonicsFile *nfile, guint id)
+read_formula(const NanonicsFile *nfile, guint id)
 {
     const gchar *const var_names[] = { "x", "DZ", "div", "RDiv" };
 
@@ -555,10 +540,10 @@ end:
 }
 
 static void
-nanonics_read_data_fields(const NanonicsFile *nfile,
-                          guint id,
-                          const guchar *buffer,
-                          GwyDataField **fields)
+read_fields(const NanonicsFile *nfile,
+            guint id,
+            const guchar *buffer,
+            GwyField **fields)
 {
     const gint16 *d16 = (const gint16*)buffer;
     gint xres, yres, i, j, power10;
@@ -571,16 +556,16 @@ nanonics_read_data_fields(const NanonicsFile *nfile,
 
     xres = nfile->xres;
     yres = nfile->yres;
-    fields[0] = gwy_data_field_new(xres, yres, nfile->xreal, nfile->yreal, FALSE);
-    d = gwy_data_field_get_data(fields[0]);
+    fields[0] = gwy_field_new(xres, yres, nfile->xreal, nfile->yreal, FALSE);
+    d = gwy_field_get_data(fields[0]);
     for (i = 0; i < yres; i++) {
         for (j = 0; j < xres; j++) {
             gint16 v = d16[i*2*xres + j];
             *(d++) = GINT16_FROM_BE(v);
         }
     }
-    fields[1] = gwy_data_field_new_alike(fields[0], FALSE);
-    d = gwy_data_field_get_data(fields[1]);
+    fields[1] = gwy_field_new_alike(fields[0], FALSE);
+    d = gwy_field_get_data(fields[1]);
     for (i = 0; i < yres; i++) {
         for (j = 0; j < xres; j++) {
             gint16 v = d16[(i + 1)*2*xres - 1 - j];
@@ -590,10 +575,10 @@ nanonics_read_data_fields(const NanonicsFile *nfile,
 
     /* Use first channel min, max and formula to calibrate data */
     if (get_double_value(nfile->pagemeta[id], "CMX", &cmx) && get_double_value(nfile->pagemeta[id], "CMN", &cmn)) {
-        gwy_data_field_get_min_max(fields[0], &min, &max);
+        gwy_field_get_min_max(fields[0], &min, &max);
         have_minmax = TRUE;
     }
-    c = nanonics_read_formula(nfile, id);
+    c = read_formula(nfile, id);
 
     if ((s = g_hash_table_lookup(nfile->pagemeta[id], "CHU"))) {
         /* Fix some silly units */
@@ -604,15 +589,15 @@ nanonics_read_data_fields(const NanonicsFile *nfile,
     }
 
     for (j = 0; j < 2; j++) {
-        GwyDataField *dfield = fields[j];
+        GwyField *dfield = fields[j];
 
         if (have_minmax) {
-            gwy_data_field_multiply(dfield, (cmx - cmn)/(max - min));
-            gwy_data_field_add(dfield, (cmx + cmn)/2);
+            gwy_field_multiply(dfield, (cmx - cmn)/(max - min));
+            gwy_field_add(dfield, (cmx + cmn)/2);
         }
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
-        gwy_si_unit_set_from_string_parse(gwy_data_field_get_si_unit_z(dfield), s, &power10);
-        gwy_data_field_multiply(dfield, c*pow10(power10));
+        gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
+        power10 = gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), s);
+        gwy_field_multiply(dfield, c*gwy_exp10(power10));
     }
 }
 
