@@ -1,22 +1,18 @@
 /*
- *  $Id: nanonis-spec.c 25894 2023-10-22 05:25:33Z yeti-dn $
- *  Copyright (C) 2018 David Necas (Yeti).
+ *  $Id: nanonis-spec.c 28803 2025-11-05 11:58:19Z yeti-dn $
+ *  Copyright (C) 2018-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
+ *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
+ *  later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -44,15 +40,10 @@
 #define MAGIC1_SIZE (sizeof(MAGIC1)-1)
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
 #include <string.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/datafield.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/data-browser.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -78,10 +69,10 @@ typedef struct {
 } DATSpectraSet;
 
 static gboolean       module_register          (void);
-static gint           dat_detect               (const GwyFileDetectInfo *fileinfo,
+static gint           detect_file              (const GwyFileDetectInfo *fileinfo,
                                                 gboolean only_name);
-static GwyContainer*  dat_load                 (const gchar *filename,
-                                                GwyRunType mode,
+static GwyFile*       load_file                (const gchar *filename,
+                                                GwyRunModeFlags mode,
                                                 GError **error);
 static gboolean       check_compatibility      (const DATSingleFile *datfile,
                                                 const DATSingleFile *template);
@@ -97,7 +88,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports Nanonis DAT spectrum files."),
     "Yeti <yeti@gwyddion.net>",
-    "1.3",
+    "2.0",
     "David Nečas (Yeti)",
     "2018",
 };
@@ -109,10 +100,7 @@ module_register(void)
 {
     gwy_file_func_register("nanonis_spec",
                            N_("Nanonis spectrum files (.dat)"),
-                           (GwyFileDetectFunc)&dat_detect,
-                           (GwyFileLoadFunc)&dat_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
@@ -122,16 +110,14 @@ find_field_in_head(const GwyFileDetectInfo *fileinfo,
                    const guchar *p,
                    const gchar *s)
 {
-    p = gwy_memmem(p, fileinfo->buffer_len - (p - fileinfo->head),
-                   s, strlen(s));
+    p = gwy_memmem(p, fileinfo->buffer_len - (p - fileinfo->head), s, strlen(s));
     if (p && (p == fileinfo->head || *(p-1) == '\r' || *(p-1) == '\n'))
         return p;
     return NULL;
 }
 
 static gint
-dat_detect(const GwyFileDetectInfo *fileinfo,
-           gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     const guchar *p;
 
@@ -156,19 +142,18 @@ dat_detect(const GwyFileDetectInfo *fileinfo,
     return 0;
 }
 
-static GwyContainer*
-dat_load(const gchar *filename,
-         G_GNUC_UNUSED GwyRunType mode,
-         GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     gchar **filenames;
     DATSpectraSet specset;
     DATSingleFile *datfile;
-    GQuark quark;
     guint i, j;
 
-    gwy_clear(&specset, 1);
+    gwy_clear1(specset);
     filenames = find_all_file_names(filename);
     /* When we cannot enumerate files, just create a list containing the single
      * file name we were given explicitly. */
@@ -198,28 +183,28 @@ dat_load(const gchar *filename,
         if (!specset.spectra[i])
             continue;
 
-        if (!container)
-            container = gwy_container_new();
-        quark = gwy_app_get_spectra_key_for_id(j);
-        gwy_container_set_object(container, quark, specset.spectra[i]);
+        if (!file)
+            file = gwy_file_new_in_construction();
+        gwy_file_set_spectra(file, j, specset.spectra[i]);
+        gwy_log_add_import(file, GWY_FILE_SPECTRA, j, NULL, filename);
         j++;
     }
 
-    if (!container)
+    if (!file)
         err_NO_DATA(error);
 
 fail:
     if (specset.template) {
         if (specset.spectra) {
             for (i = 0; i < specset.template->ncolumns; i++)
-                GWY_OBJECT_UNREF(specset.spectra[i]);
+                g_clear_object(specset.spectra + i);
             g_free(specset.spectra);
         }
         dat_single_file_free(specset.template);
     }
     g_strfreev(filenames);
 
-    return container;
+    return file;
 }
 
 static gboolean
@@ -328,7 +313,7 @@ add_curves_to_spectra_set(DATSpectraSet *specset,
     gboolean reversed;
     gdouble *data = datfile->data, *d;
     gchar *xlabel, *ylabel;
-    GwyDataLine *dline;
+    GwyLine *dline;
     GwySpectra *spec;
     const DATColumn *abscissa = NULL, *ordinate;
 
@@ -361,14 +346,12 @@ add_curves_to_spectra_set(DATSpectraSet *specset,
         if (gwy_strequal(ordinate->name, abscissa->name))
             continue;
 
-        dline = gwy_data_line_new(nrows, real, FALSE);
-        gwy_data_line_set_offset(dline, off);
-        gwy_si_unit_set_from_string(gwy_data_line_get_si_unit_x(dline),
-                                    ordinate->unit);
+        dline = gwy_line_new(nrows, real, FALSE);
+        gwy_line_set_offset(dline, off);
+        gwy_unit_set_from_string(gwy_line_get_unit_x(dline), ordinate->unit);
         ylabel = make_axis_label(ordinate);
-        gwy_si_unit_set_from_string(gwy_data_line_get_si_unit_y(dline),
-                                    ordinate->unit);
-        d = gwy_data_line_get_data(dline);
+        gwy_unit_set_from_string(gwy_line_get_unit_y(dline), ordinate->unit);
+        d = gwy_line_get_data(dline);
         if (reversed) {
             for (j = 0; j < nrows; j++)
                 d[nrows-1 - j] = data[i + j*ncolumns];
@@ -380,7 +363,7 @@ add_curves_to_spectra_set(DATSpectraSet *specset,
 
         if (!(spec = specset->spectra[i])) {
             spec = specset->spectra[i] = gwy_spectra_new();
-            gwy_si_unit_set_from_string(gwy_spectra_get_si_unit_xy(spec), "m");
+            gwy_unit_set_from_string(gwy_spectra_get_unit_xy(spec), "m");
             gwy_spectra_set_title(spec, ylabel);
             gwy_spectra_set_spectrum_x_label(spec, xlabel);
             gwy_spectra_set_spectrum_y_label(spec, ylabel);
@@ -589,4 +572,4 @@ find_all_file_names(const gchar *filename)
     return (gchar**)g_ptr_array_free(fnames, FALSE);
 }
 
-/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
+/* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
