@@ -1,6 +1,6 @@
 /*
- *  $Id: nanonis.c 26135 2024-01-15 12:54:27Z yeti-dn $
- *  Copyright (C) 2006,2015-2019 David Necas (Yeti), Petr Klapetek.
+ *  $Id: nanonis.c 28848 2025-11-10 18:54:08Z yeti-dn $
+ *  Copyright (C) 2006,2015-2025 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -42,15 +42,10 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
 #include <string.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/datafield.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -99,10 +94,10 @@ typedef struct {
 
 static gboolean      module_register     (void);
 static GwyParamDef*  define_module_params(void);
-static gint          sxm_detect          (const GwyFileDetectInfo *fileinfo,
+static gint          detect_file         (const GwyFileDetectInfo *fileinfo,
                                           gboolean only_name);
-static GwyContainer* sxm_load            (const gchar *filename,
-                                          GwyRunType mode,
+static GwyFile*      load_file           (const gchar *filename,
+                                          GwyRunModeFlags mode,
                                           GError **error);
 static GwyContainer* sxm_build_meta      (const SXMFile *sxmfile,
                                           guint id);
@@ -112,7 +107,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports Nanonis SXM data files."),
     "Yeti <yeti@gwyddion.net>",
-    "1.4",
+    "2.0",
     "David Nečas (Yeti) & Petr Klapetek",
     "2006",
 };
@@ -131,10 +126,7 @@ module_register(void)
 {
     gwy_file_func_register("nanonis",
                            N_("Nanonis SXM files (.sxm)"),
-                           (GwyFileDetectFunc)&sxm_detect,
-                           (GwyFileLoadFunc)&sxm_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
@@ -155,8 +147,7 @@ define_module_params(void)
 }
 
 static gint
-sxm_detect(const GwyFileDetectInfo *fileinfo,
-           gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     gint score = 0;
 
@@ -170,8 +161,7 @@ sxm_detect(const GwyFileDetectInfo *fileinfo,
 }
 
 static gchar**
-split_line_in_place(gchar *line,
-                    gchar delim)
+split_line_in_place(gchar *line, gchar delim)
 {
     gchar **strs;
     guint i, n = 0;
@@ -393,28 +383,28 @@ sxm_read_tag(SXMFile *sxmfile,
 }
 
 static void
-read_data_field(GwyContainer *container,
-                gint *id,
-                const gchar *filename,
-                const SXMFile *sxmfile,
-                const SXMDataInfo *data_info,
-                SXMDirection dir,
-                GwyParams *params,
-                const guchar **p)
+read_field(GwyFile *file,
+           gint *id,
+           const gchar *filename,
+           const SXMFile *sxmfile,
+           const SXMDataInfo *data_info,
+           SXMDirection dir,
+           GwyParams *params,
+           const guchar **p)
 {
     gboolean preserve_coordinates = gwy_params_get_boolean(params, PARAM_PRESRRVE_COORDINATES);
     GwyContainer *meta;
-    GwyDataField *dfield, *mfield = NULL;
+    GwyField *dfield, *mfield = NULL;
     gdouble *data, *mdata;
     gint j, n;
     gchar *s;
     gboolean flip_vertically = FALSE, flip_horizontally = FALSE;
 
-    dfield = gwy_data_field_new(sxmfile->xres, sxmfile->yres, sxmfile->xreal, sxmfile->yreal, FALSE);
+    dfield = gwy_field_new(sxmfile->xres, sxmfile->yres, sxmfile->xreal, sxmfile->yreal, FALSE);
     /* This is correct for both preserved and non-preserved coordinates? */
-    gwy_data_field_set_xoffset(dfield, sxmfile->xoff - 0.5*sxmfile->xreal);
-    gwy_data_field_set_yoffset(dfield, sxmfile->yoff - 0.5*sxmfile->yreal);
-    data = gwy_data_field_get_data(dfield);
+    gwy_field_set_xoffset(dfield, sxmfile->xoff - 0.5*sxmfile->xreal);
+    gwy_field_set_yoffset(dfield, sxmfile->yoff - 0.5*sxmfile->yreal);
+    data = gwy_field_get_data(dfield);
 
     n = sxmfile->xres*sxmfile->yres;
     for (j = 0; j < n; j++) {
@@ -427,8 +417,8 @@ read_data_field(GwyContainer *container,
     }
 
     if (j < n) {
-        mfield = gwy_data_field_new_alike(dfield, TRUE);
-        mdata = gwy_data_field_get_data(mfield);
+        mfield = gwy_field_new_alike(dfield, TRUE);
+        mdata = gwy_field_get_data(mfield);
         while (j < n) {
             if (((*p)[0] & 0x7f) == 0x7f && (*p)[1] == 0xff) {
                 mdata[j] = -1.0;
@@ -438,35 +428,35 @@ read_data_field(GwyContainer *container,
                 data[j] = gwy_get_gfloat_be(p);
             j++;
         }
-        gwy_data_field_add(mfield, 1.0);
+        gwy_field_add(mfield, 1.0);
         gwy_app_channel_remove_bad_data(dfield, mfield);
     }
 
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), data_info->unit);
-    gwy_container_set_object(container, gwy_app_get_data_key_for_id(*id), dfield);
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
+    gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), data_info->unit);
+    gwy_file_set_image(file, *id, dfield);
 
     if (mfield) {
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(mfield), "m");
-        gwy_container_set_object(container, gwy_app_get_mask_key_for_id(*id), mfield);
+        gwy_unit_set_from_string(gwy_field_get_unit_xy(mfield), "m");
+        gwy_file_set_image_mask(file, *id, mfield);
     }
 
     if (!dir) {
-        gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(*id), data_info->name);
+        gwy_file_set_title(file, GWY_FILE_IMAGE, *id, data_info->name, FALSE);
     }
     else {
         gchar *title;
 
         title = g_strdup_printf("%s (%s)", data_info->name, dir == DIR_BACKWARD ? "Backward" : "Forward");
-        gwy_container_set_string(container, gwy_app_get_data_title_key_for_id(*id), title);
-        /* Don't free title, container eats it */
+        gwy_file_pass_title(file, GWY_FILE_IMAGE, *id, title);
+        /* Don't free title, file eats it */
     }
 
     if ((meta = sxm_build_meta(sxmfile, *id))) {
-        gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(*id), meta);
+        gwy_file_pass_meta(file, GWY_FILE_IMAGE, *id, meta);
     }
 
-    gwy_app_channel_check_nonsquare(container, *id);
+    gwy_check_nonsquare_image(file, *id);
 
     if (dir == DIR_BACKWARD)
         flip_horizontally = TRUE;
@@ -479,26 +469,26 @@ read_data_field(GwyContainer *container,
             flip_vertically = TRUE;
     }
 
-    gwy_data_field_invert(dfield, flip_vertically, flip_horizontally, FALSE);
+    gwy_field_flip(dfield, flip_horizontally, flip_vertically);
     g_object_unref(dfield);
 
     if (mfield) {
-        gwy_data_field_invert(mfield, flip_vertically, flip_horizontally, FALSE);
+        gwy_field_flip(mfield, flip_horizontally, flip_vertically);
         g_object_unref(mfield);
     }
 
-    gwy_file_channel_import_log_add(container, *id, NULL, filename);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, *id, NULL, filename);
 
     (*id)++;
 }
 
-static GwyContainer*
-sxm_load(const gchar *filename,
-         G_GNUC_UNUSED GwyRunType mode,
-         GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
     SXMFile sxmfile;
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     GwyParams *params;
     GPtrArray *header_lines;
     guchar *buffer = NULL;
@@ -536,7 +526,7 @@ sxm_load(const gchar *filename,
         return NULL;
     }
 
-    gwy_clear(&sxmfile, 1);
+    gwy_clear1(sxmfile);
     sxmfile.meta = g_hash_table_new_full(g_str_hash, g_str_equal, NULL, g_free);
 
     header = g_memdup(buffer, p - buffer + 1);
@@ -725,16 +715,16 @@ sxm_load(const gchar *filename,
     if (sxmfile.ok) {
         gint id = 0;
 
-        container = gwy_container_new();
+        file = gwy_file_new_in_construction();
         for (i = 0; i < sxmfile.ndata; i++) {
             guint d = sxmfile.data_info[i].direction;
 
             if (d == DIR_BOTH) {
-                read_data_field(container, &id, filename, &sxmfile, sxmfile.data_info + i, DIR_FORWARD, params, &p);
-                read_data_field(container, &id, filename, &sxmfile, sxmfile.data_info + i, DIR_BACKWARD, params, &p);
+                read_field(file, &id, filename, &sxmfile, sxmfile.data_info + i, DIR_FORWARD, params, &p);
+                read_field(file, &id, filename, &sxmfile, sxmfile.data_info + i, DIR_BACKWARD, params, &p);
             }
             else if (d == DIR_FORWARD || d == DIR_BACKWARD) {
-                read_data_field(container, &id, filename, &sxmfile, sxmfile.data_info + i, d, params, &p);
+                read_field(file, &id, filename, &sxmfile, sxmfile.data_info + i, d, params, &p);
             }
             else {
                 g_assert_not_reached();
@@ -752,7 +742,7 @@ sxm_load(const gchar *filename,
     gwy_params_save_to_settings(params);
     g_object_unref(params);
 
-    return container;
+    return file;
 }
 
 static inline gchar*
@@ -786,7 +776,7 @@ static GwyContainer*
 sxm_build_meta(const SXMFile *sxmfile,
                G_GNUC_UNUSED guint id)
 {
-    GwyContainer *meta = gwy_container_new();
+    GwyContainer *meta = gwy_container_new_in_construction();
     GHashTable *hash = sxmfile->meta;
     const gchar *value;
 
