@@ -1,6 +1,6 @@
 /*
- *  $Id: nanoobserver.c 26168 2024-02-08 10:18:02Z yeti-dn $
- *  Copyright (C) 2012-2017 David Necas (Yeti).
+ *  $Id: nanoobserver.c 28848 2025-11-10 18:54:08Z yeti-dn $
+ *  Copyright (C) 2012-2015 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
  *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
@@ -48,15 +48,11 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
 #include <limits.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 
 #include "err.h"
 #include "gwyzip.h"
@@ -135,70 +131,70 @@ typedef struct {
     const gchar *filename;
 } NAOFile;
 
-static gboolean      module_register             (void);
-static gint          nao_detect                  (const GwyFileDetectInfo *fileinfo,
-                                                  gboolean only_name);
-static gint          nao123_detect               (const GwyFileDetectInfo *fileinfo);
-static gint          nao133_detect               (const GwyFileDetectInfo *fileinfo);
-static GwyContainer* nao_load                    (const gchar *filename,
-                                                  GwyRunType mode,
-                                                  GError **error);
-static GwyContainer* nao123_load                 (GwyZipFile zipfile,
-                                                  NAOFile *naofile,
-                                                  GError **error);
-static GwyContainer* nao133_imaging_load         (GwyZipFile zipfile,
-                                                  NAOFile *naofile,
-                                                  GError **error);
-static GwyContainer* nao133_spectro_load         (GwyZipFile zipfile,
-                                                  NAOFile *naofile,
-                                                  GError **error);
-static void          add_meta                    (gpointer hkey,
-                                                  gpointer hvalue,
-                                                  gpointer user_data);
-static GwyDataField* nao_read_field              (GwyZipFile zipfile,
-                                                  NAOFile *naofile,
-                                                  guint id,
-                                                  GError **error);
-static gboolean      nao_parse_xml_header        (GwyZipFile zipfile,
-                                                  NAOFile *naofile,
-                                                  const gchar *filename,
-                                                  GMarkupParser *parser,
-                                                  GError **error);
-static gboolean      nao133_parse_spectrum       (GwyZipFile zipfile,
-                                                  NAOSpectrum *spectrum,
-                                                  GError **error);
-static void          create_channel              (NAOFile *naofile,
-                                                  GwyDataField *dfield,
-                                                  NAOStream *stream,
-                                                  guint channelno,
-                                                  GwyContainer *container);
-static void          create_spectra              (NAOFile *naofile,
-                                                  GwyContainer *container);
-static GwyDataLine*  create_dataline_for_spectrum(NAOSpectrumData *specdata,
-                                                  NAOSpectrum *spectrum);
-static void          add_dline_to_spectra        (GPtrArray *sps,
-                                                  GwyDataLine *dline,
-                                                  const gchar *xtitle,
-                                                  const gchar *name,
-                                                  const gchar *dir,
-                                                  gdouble x,
-                                                  gdouble y);
-static gboolean      find_size_and_resolution    (NAOFile *naofile,
-                                                  GError **error);
-static gboolean      find_spectrum_abscissa      (NAOSpectrum *spectrum,
-                                                  GError **error);
-static const gchar*  find_attribute              (const gchar **attribute_names,
-                                                  const gchar **attribute_values,
-                                                  const gchar *attrname);
-static void          nao_file_free               (NAOFile *naofile);
-static void          nao_free_spectrum           (NAOSpectrum *spectrum);
+static gboolean     module_register         (void);
+static gint         detect_file             (const GwyFileDetectInfo *fileinfo,
+                                             gboolean only_name);
+static gint         nao123_detect           (const GwyFileDetectInfo *fileinfo);
+static gint         nao133_detect           (const GwyFileDetectInfo *fileinfo);
+static GwyFile*     load_file               (const gchar *filename,
+                                             GwyRunModeFlags mode,
+                                             GError **error);
+static GwyFile*     nao123_load             (GwyZipFile zipfile,
+                                             NAOFile *naofile,
+                                             GError **error);
+static GwyFile*     nao133_imaging_load     (GwyZipFile zipfile,
+                                             NAOFile *naofile,
+                                             GError **error);
+static GwyFile*     nao133_spectro_load     (GwyZipFile zipfile,
+                                             NAOFile *naofile,
+                                             GError **error);
+static void         add_meta                (gpointer hkey,
+                                             gpointer hvalue,
+                                             gpointer user_data);
+static GwyField*    nao_read_field          (GwyZipFile zipfile,
+                                             NAOFile *naofile,
+                                             guint id,
+                                             GError **error);
+static gboolean     nao_parse_xml_header    (GwyZipFile zipfile,
+                                             NAOFile *naofile,
+                                             const gchar *filename,
+                                             GMarkupParser *parser,
+                                             GError **error);
+static gboolean     nao133_parse_spectrum   (GwyZipFile zipfile,
+                                             NAOSpectrum *spectrum,
+                                             GError **error);
+static void         create_channel          (NAOFile *naofile,
+                                             GwyField *dfield,
+                                             NAOStream *stream,
+                                             guint channelno,
+                                             GwyFile *file);
+static void         create_spectra          (NAOFile *naofile,
+                                             GwyFile *file);
+static GwyLine*     create_line_for_spectrum(NAOSpectrumData *specdata,
+                                             NAOSpectrum *spectrum);
+static void         add_dline_to_spectra    (GPtrArray *sps,
+                                             GwyLine *dline,
+                                             const gchar *xtitle,
+                                             const gchar *name,
+                                             const gchar *dir,
+                                             gdouble x,
+                                             gdouble y);
+static gboolean     find_size_and_resolution(NAOFile *naofile,
+                                             GError **error);
+static gboolean     find_spectrum_abscissa  (NAOSpectrum *spectrum,
+                                             GError **error);
+static const gchar* find_attribute          (const gchar **attribute_names,
+                                             const gchar **attribute_values,
+                                             const gchar *attrname);
+static void         nao_file_free           (NAOFile *naofile);
+static void         nao_free_spectrum       (NAOSpectrum *spectrum);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Reads Nano-Solution/NanoObserver .nao files."),
     "Yeti <yeti@gwyddion.net>",
-    "2.3",
+    "3.0",
     "David Nečas (Yeti)",
     "2012",
 };
@@ -210,17 +206,13 @@ module_register(void)
 {
     gwy_file_func_register("nanoobserver",
                            N_("Nano-Solution/NanoObserver data (.nao)"),
-                           (GwyFileDetectFunc)&nao_detect,
-                           (GwyFileLoadFunc)&nao_load,
-                           NULL,
-                           NULL);
+                           &detect_file, &load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-nao_detect(const GwyFileDetectInfo *fileinfo,
-           gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     gint score;
 
@@ -307,7 +299,7 @@ nao133_detect(const GwyFileDetectInfo *fileinfo)
     }
 
     if (gwy_memmem(fileinfo->head, fileinfo->buffer_len, MAGIC133_1, sizeof(MAGIC133_1)-1)
-         || gwy_memmem(fileinfo->tail, fileinfo->buffer_len, MAGIC133_1, sizeof(MAGIC133_1)-1)) {
+        || gwy_memmem(fileinfo->tail, fileinfo->buffer_len, MAGIC133_1, sizeof(MAGIC133_1)-1)) {
         gwy_debug("found magic1 %s", MAGIC133_1);
     }
     else if (gwy_memmem(fileinfo->head, fileinfo->buffer_len, MAGIC133_2, sizeof(MAGIC133_2)-1)
@@ -355,30 +347,30 @@ nao133_detect(const GwyFileDetectInfo *fileinfo)
     return score;
 }
 
-static GwyContainer*
-nao_load(const gchar *filename,
-         G_GNUC_UNUSED GwyRunType mode,
-         GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     NAOFile naofile;
     GwyZipFile zipfile;
 
     if (!(zipfile = gwyzip_open(filename, error)))
         return NULL;
 
-    gwy_clear(&naofile, 1);
+    gwy_clear1(naofile);
     naofile.filename = filename;
     if (gwyzip_locate_file(zipfile, "NAO_v133.txt", 1, NULL)) {
         if (gwyzip_locate_file(zipfile, "Data/Imaging.xml", 1, NULL))
-            container = nao133_imaging_load(zipfile, &naofile, error);
+            file = nao133_imaging_load(zipfile, &naofile, error);
         else if (gwyzip_locate_file(zipfile, "Data/Spectro.xml", 1, NULL))
-            container = nao133_spectro_load(zipfile, &naofile, error);
+            file = nao133_spectro_load(zipfile, &naofile, error);
         else
             err_FILE_TYPE(error, "Nano-Solution");
     }
     else if (gwyzip_locate_file(zipfile, "Scan/Measure.xml", 1, NULL)) {
-        container = nao123_load(zipfile, &naofile, error);
+        file = nao123_load(zipfile, &naofile, error);
     }
     else {
         err_FILE_TYPE(error, "Nano-Solution");
@@ -387,7 +379,7 @@ nao_load(const gchar *filename,
     gwyzip_close(zipfile);
     nao_file_free(&naofile);
 
-    return container;
+    return file;
 }
 
 static void
@@ -396,7 +388,7 @@ add_meta(gpointer hkey, gpointer hvalue, gpointer user_data)
     gwy_container_set_const_string_by_name(GWY_CONTAINER(user_data), (gchar *)hkey, (gchar *)hvalue);
 }
 
-static GwyDataField*
+static GwyField*
 nao_read_field(GwyZipFile zipfile, NAOFile *naofile, guint id, GError **error)
 {
     gsize size, expected_size;
@@ -404,7 +396,7 @@ nao_read_field(GwyZipFile zipfile, NAOFile *naofile, guint id, GError **error)
     guchar *buffer;
     const guchar *p;
     const gchar *units;
-    GwyDataField *field;
+    GwyField *field;
     gdouble *data;
 
     if (!(buffer = gwyzip_get_file_content(zipfile, &size, error)))
@@ -428,8 +420,8 @@ nao_read_field(GwyZipFile zipfile, NAOFile *naofile, guint id, GError **error)
         return NULL;
     }
 
-    field = gwy_data_field_new(width, nscanlines, naofile->xreal, naofile->yreal*nscanlines/naofile->yres, TRUE);
-    data = gwy_data_field_get_data(field);
+    field = gwy_field_new(width, nscanlines, naofile->xreal, naofile->yreal*nscanlines/naofile->yres, TRUE);
+    data = gwy_field_get_data(field);
 
     for (i = 0; i < nscanlines; i++) {
         guint lineno = gwy_get_guint32_le(&p);
@@ -444,8 +436,8 @@ nao_read_field(GwyZipFile zipfile, NAOFile *naofile, guint id, GError **error)
 
     /* Older versions of the format had human-readable units there but we disregard these. */
     units = g_array_index(naofile->streams, NAOStream, id).units;
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(field), units);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(field), "m");
+    gwy_unit_set_from_string(gwy_field_get_unit_z(field), units);
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(field), "m");
 
     return field;
 }
@@ -473,7 +465,7 @@ nao123_start_element(G_GNUC_UNUSED GMarkupParseContext *context,
             && (unit = find_attribute(attribute_names, attribute_values, "Unit"))) {
             NAOStream stream;
             gwy_debug("Adding stream %s [%s]", name, unit);
-            gwy_clear(&stream, 1);
+            gwy_clear1(stream);
             stream.name = g_strdup(name);
             stream.units = g_strdup(unit);
             g_array_append_val(naofile->streams, stream);
@@ -522,7 +514,7 @@ nao123_text(G_GNUC_UNUSED GMarkupParseContext *context,
     }
 }
 
-static GwyContainer*
+static GwyFile*
 nao123_load(GwyZipFile zipfile, NAOFile *naofile, GError **error)
 {
     GMarkupParser parser = {
@@ -532,7 +524,7 @@ nao123_load(GwyZipFile zipfile, NAOFile *naofile, GError **error)
         NULL,
         NULL,
     };
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     gchar *filename_curr;
     guint len, id, channelno = 0;
     gboolean status;
@@ -544,7 +536,7 @@ nao123_load(GwyZipFile zipfile, NAOFile *naofile, GError **error)
     if (!status)
         goto fail;
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     while (status && gwyzip_get_current_filename(zipfile, &filename_curr, NULL)) {
         if (g_str_has_prefix(filename_curr, "Scan/Data/")) {
             const gchar *dataname = filename_curr + strlen("Scan/Data/");
@@ -564,10 +556,10 @@ nao123_load(GwyZipFile zipfile, NAOFile *naofile, GError **error)
                     break;
             }
             if (stream->dir) {
-                GwyDataField *dfield = nao_read_field(zipfile, naofile, id, error);
+                GwyField *dfield = nao_read_field(zipfile, naofile, id, error);
                 if (!dfield)
                     goto fail;
-                create_channel(naofile, dfield, stream, channelno, container);
+                create_channel(naofile, dfield, stream, channelno, file);
                 channelno++;
             }
         }
@@ -576,13 +568,13 @@ nao123_load(GwyZipFile zipfile, NAOFile *naofile, GError **error)
     }
 
 fail:
-    if (!container || !gwy_container_get_n_items(container)) {
-        GWY_OBJECT_UNREF(container);
+    if (!file || !gwy_container_get_n_items(GWY_CONTAINER(file))) {
+        g_clear_object(&file);
         if (error && !*error)
             err_NO_DATA(error);
     }
 
-    return container;
+    return file;
 }
 
 static void
@@ -622,7 +614,7 @@ nao133_imaging_start_element(G_GNUC_UNUSED GMarkupParseContext *context,
         if (naofile->current_name && (filename = find_attribute(attribute_names, attribute_values, "NaoSubFile"))) {
             NAOStream stream;
 
-            gwy_clear(&stream, 1);
+            gwy_clear1(stream);
             stream.name = g_strdup(naofile->current_name);
             stream.filename = g_strdelimit(g_strdup(filename), "\\", '/');
             stream.dir = g_strdup(element_name);
@@ -661,7 +653,7 @@ nao133_imaging_start_element(G_GNUC_UNUSED GMarkupParseContext *context,
             && (y = find_attribute(attribute_names, attribute_values, "Y"))) {
             NAOSpectrum spectrum;
 
-            gwy_clear(&spectrum, 1);
+            gwy_clear1(spectrum);
             spectrum.filename = g_strdelimit(g_strdup(filename), "\\", '/');
             spectrum.x = atoi(x);
             spectrum.y = atoi(y);
@@ -707,7 +699,7 @@ nao133_imaging_text(G_GNUC_UNUSED GMarkupParseContext *context,
         GHashTable *hash;
         gchar *imaging_parameters = g_strndup(text, text_len);
 
-        gwy_clear(&parser, 1);
+        gwy_clear1(parser);
         parser.key_value_separator = "=";
         hash = gwy_text_header_parse(imaging_parameters, &parser, NULL, NULL);
         if (naofile->hash && hash) {
@@ -725,7 +717,7 @@ nao133_imaging_text(G_GNUC_UNUSED GMarkupParseContext *context,
 }
 
 /* NB: This can be imaging + spectroscopy, just not standalone spectra cuves. */
-static GwyContainer*
+static GwyFile*
 nao133_imaging_load(GwyZipFile zipfile, NAOFile *naofile, GError **error)
 {
     GMarkupParser parser = {
@@ -735,14 +727,14 @@ nao133_imaging_load(GwyZipFile zipfile, NAOFile *naofile, GError **error)
         NULL,
         NULL,
     };
-    GwyContainer *container = NULL;
-    GwyDataField *dfield;
+    GwyFile *file = NULL;
+    GwyField *dfield;
     guint id;
 
     if (!nao_parse_xml_header(zipfile, naofile, "Data/Imaging.xml", &parser, error))
         goto fail;
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     for (id = 0; id < naofile->streams->len; id++) {
         NAOStream *stream = &g_array_index(naofile->streams, NAOStream, id);
 
@@ -751,7 +743,7 @@ nao133_imaging_load(GwyZipFile zipfile, NAOFile *naofile, GError **error)
         if (!(dfield = nao_read_field(zipfile, naofile, id, error)))
             goto fail;
 
-        create_channel(naofile, dfield, stream, id, container);
+        create_channel(naofile, dfield, stream, id, file);
     }
 
     for (id = 0; id < naofile->spectra->len; id++) {
@@ -765,25 +757,25 @@ nao133_imaging_load(GwyZipFile zipfile, NAOFile *naofile, GError **error)
         if (!nao133_parse_spectrum(zipfile, spectrum, error))
             goto fail;
     }
-    create_spectra(naofile, container);
+    create_spectra(naofile, file);
 
-    if (!gwy_container_get_n_items(container)) {
+    if (!gwy_container_get_n_items(GWY_CONTAINER(file))) {
         err_NO_DATA(error);
         goto fail;
     }
-    return container;
+    return file;
 
 fail:
-    GWY_OBJECT_UNREF(container);
+    g_clear_object(&file);
     return NULL;
 }
 
 /* The pure spectra case.  We do not know how to recalculate pixel coordinates to real coordinates so we import the
  * spectra as graph curves. */
-static GwyContainer*
+static GwyFile*
 nao133_spectro_load(GwyZipFile zipfile, NAOFile *naofile, GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     const gchar *xtitle;
     NAOSpectrum spec;
     NAOSpectrum *spectrum;
@@ -793,7 +785,7 @@ nao133_spectro_load(GwyZipFile zipfile, NAOFile *naofile, GError **error)
     naofile->path = g_string_new(NULL);
     naofile->spectra = g_array_new(FALSE, FALSE, sizeof(NAOSpectrum));
 
-    gwy_clear(&spec, 1);
+    gwy_clear1(spec);
     spec.path = naofile->path;
     g_array_append_val(naofile->spectra, spec);
     spectrum = &g_array_index(naofile->spectra, NAOSpectrum, naofile->spectra->len-1);
@@ -801,19 +793,18 @@ nao133_spectro_load(GwyZipFile zipfile, NAOFile *naofile, GError **error)
     if (!nao133_parse_spectrum(zipfile, spectrum, error))
         goto fail;
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     xtitle = g_hash_table_lookup(spectrum->hash, "SweepSignal");
 
     for (j = 0; j < spectrum->specdata->len; j++) {
         NAOSpectrumData *specdata = &g_array_index(spectrum->specdata, NAOSpectrumData, j);
-        GwyDataLine *dline = create_dataline_for_spectrum(specdata, spectrum);
+        GwyLine *dline = create_line_for_spectrum(specdata, spectrum);
         GwyGraphModel *gmodel = gwy_graph_model_new();
         GwyGraphCurveModel *gcmodel = gwy_graph_curve_model_new();
         gchar *fullname = g_strconcat(specdata->name, " ", specdata->dir, NULL);
-        GQuark key;
 
-        gwy_graph_curve_model_set_data_from_dataline(gcmodel, dline, 0, 0);
-        gwy_graph_model_set_units_from_data_line(gmodel, dline);
+        gwy_graph_curve_model_set_data_from_line(gcmodel, dline, 0, 0);
+        gwy_graph_model_set_units_from_line(gmodel, dline);
         g_object_unref(dline);
         g_object_set(gcmodel,
                      "mode", GWY_GRAPH_CURVE_LINE,
@@ -829,12 +820,12 @@ nao133_spectro_load(GwyZipFile zipfile, NAOFile *naofile, GError **error)
         if (xtitle)
             g_object_set(gmodel, "axis-label-bottom", xtitle, NULL);
 
-        key = gwy_app_get_graph_key_for_id(j);
-        gwy_container_pass_object(container, key, gmodel);
+        gwy_file_pass_graph(file, j, gmodel);
+        gwy_log_add_import(file, GWY_FILE_GRAPH, j, NULL, naofile->filename);
     }
 
 fail:
-    return container;
+    return file;
 }
 
 static gboolean
@@ -875,7 +866,7 @@ nao_parse_xml_header(GwyZipFile zipfile, NAOFile *naofile,
     ok = TRUE;
 
     if (g_hash_table_size(naofile->hash)) {
-        naofile->meta = gwy_container_new();
+        naofile->meta = gwy_container_new_in_construction();
         g_hash_table_foreach(naofile->hash, &add_meta, naofile->meta);
     }
 
@@ -920,7 +911,7 @@ nao133_spectro_start_element(G_GNUC_UNUSED GMarkupParseContext *context,
             && (sizeused = find_attribute(attribute_names, attribute_values, "SizeUsed"))) {
             NAOSpectrumData specdata;
 
-            gwy_clear(&specdata, 1);
+            gwy_clear1(specdata);
             specdata.capacity = atoi(capacity);
             specdata.sizeused = atoi(sizeused);
             specdata.dir = g_strdup(name);
@@ -972,7 +963,7 @@ nao133_spectro_text(G_GNUC_UNUSED GMarkupParseContext *context,
         GHashTable *hash;
         gchar *spectro_parameters = g_strndup(text, text_len);
 
-        gwy_clear(&parser, 1);
+        gwy_clear1(parser);
         parser.key_value_separator = "=";
         hash = gwy_text_header_parse(spectro_parameters, &parser, NULL, NULL);
         if (spectrum->hash && hash) {
@@ -1064,37 +1055,37 @@ fail:
 }
 
 static void
-create_channel(NAOFile *naofile, GwyDataField *dfield,
-               NAOStream *stream, guint channelno, GwyContainer *container)
+create_channel(NAOFile *naofile, GwyField *dfield,
+               NAOStream *stream, guint channelno, GwyFile *file)
 {
     GwyContainer *meta;
     gchar *title;
 
-    gwy_container_pass_object(container, gwy_app_get_data_key_for_id(channelno), dfield);
+    gwy_file_pass_image(file, channelno, dfield);
 
     title = g_strconcat(stream->name, " ", stream->dir, NULL);
-    gwy_container_set_string(container, gwy_app_get_data_title_key_for_id(channelno), title);
+    gwy_file_pass_title(file, GWY_FILE_IMAGE, channelno, title);
 
     if (naofile->meta) {
-        meta = gwy_container_duplicate(naofile->meta);
-        gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(channelno), meta);
+        meta = gwy_container_copy(naofile->meta);
+        gwy_file_pass_meta(file, GWY_FILE_IMAGE, channelno, meta);
     }
-    gwy_file_channel_import_log_add(container, channelno, NULL, naofile->filename);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, channelno, NULL, naofile->filename);
 }
 
 /* We must organise spectra differently than in the file: split them by channel and direction but group different
  * positions to one GwySpectra object. */
 static void
-create_spectra(NAOFile *naofile, GwyContainer *container)
+create_spectra(NAOFile *naofile, GwyFile *file)
 {
-    GwyDataField *dfield = NULL;
+    GwyField *dfield = NULL;
     GPtrArray *sps;
     guint i, j, id;
 
     if (!naofile->spectra || !naofile->spectra->len)
         return;
 
-    if (!gwy_container_gis_object(container, gwy_app_get_data_key_for_id(0), (GObject**)&dfield)) {
+    if (!(dfield = gwy_file_get_image(file, 0))) {
         g_warning("Cannot convert spectra pixel coordinates to real because there is no image.");
     }
 
@@ -1108,14 +1099,14 @@ create_spectra(NAOFile *naofile, GwyContainer *container)
             continue;
 
         if (dfield) {
-            spectrum->x = gwy_data_field_jtor(dfield, spectrum->x + 0.5);
-            spectrum->y = gwy_data_field_itor(dfield, dfield->yres - 0.5 - spectrum->y);
+            spectrum->x = gwy_field_jtor(dfield, spectrum->x + 0.5);
+            spectrum->y = gwy_field_itor(dfield, dfield->yres - 0.5 - spectrum->y);
         }
 
         xtitle = g_hash_table_lookup(spectrum->hash, "SweepSignal");
         for (j = 0; j < spectrum->specdata->len; j++) {
             NAOSpectrumData *specdata = &g_array_index(spectrum->specdata, NAOSpectrumData, j);
-            GwyDataLine *dline = create_dataline_for_spectrum(specdata, spectrum);
+            GwyLine *dline = create_line_for_spectrum(specdata, spectrum);
             add_dline_to_spectra(sps, dline, xtitle, specdata->name, specdata->dir, spectrum->x, spectrum->y);
         }
     }
@@ -1123,42 +1114,43 @@ create_spectra(NAOFile *naofile, GwyContainer *container)
     for (id = 0; id < sps->len; id++) {
         GwySpectra *spectra = (GwySpectra*)g_ptr_array_index(sps, id);
 
-        gwy_container_pass_object(container, gwy_app_get_spectra_key_for_id(id), spectra);
+        gwy_file_pass_spectra(file, id, spectra);
+        gwy_log_add_import(file, GWY_FILE_SPECTRA, id, NULL, naofile->filename);
     }
     g_ptr_array_free(sps, TRUE);
 }
 
-static GwyDataLine*
-create_dataline_for_spectrum(NAOSpectrumData *specdata, NAOSpectrum *spectrum)
+static GwyLine*
+create_line_for_spectrum(NAOSpectrumData *specdata, NAOSpectrum *spectrum)
 {
-    GwyDataLine *dline;
+    GwyLine *dline;
     gdouble real = spectrum->sweep_to - spectrum->sweep_from;
 
     sanitise_real_size(&real, "spectrum sweep range");
-    dline = gwy_data_line_new(specdata->nvalues, fabs(real), FALSE);
-    gwy_assign(gwy_data_line_get_data(dline), specdata->values, specdata->nvalues);
+    dline = gwy_line_new(specdata->nvalues, fabs(real), FALSE);
+    gwy_assign(gwy_line_get_data(dline), specdata->values, specdata->nvalues);
 
     if (real > 0.0)
-        gwy_data_line_set_offset(dline, spectrum->sweep_from);
+        gwy_line_set_offset(dline, spectrum->sweep_from);
     else {
-        gwy_data_line_invert(dline, TRUE, FALSE);
-        gwy_data_line_set_offset(dline, spectrum->sweep_to);
+        gwy_line_flip(dline);
+        gwy_line_set_offset(dline, spectrum->sweep_to);
     }
 
-    gwy_si_unit_set_from_string(gwy_data_line_get_si_unit_x(dline), spectrum->sweep_unit);
-    gwy_si_unit_set_from_string(gwy_data_line_get_si_unit_y(dline), specdata->unit);
+    gwy_unit_set_from_string(gwy_line_get_unit_x(dline), spectrum->sweep_unit);
+    gwy_unit_set_from_string(gwy_line_get_unit_y(dline), specdata->unit);
 
     return dline;
 }
 
 static void
-add_dline_to_spectra(GPtrArray *sps, GwyDataLine *dline,
+add_dline_to_spectra(GPtrArray *sps, GwyLine *dline,
                      const gchar *xtitle, const gchar *name, const gchar *dir,
                      gdouble x, gdouble y)
 {
     gchar *fullname = g_strconcat(name, " ", dir, NULL);
     const gchar *specxlabel;
-    GwyDataLine *firstspec;
+    GwyLine *firstspec;
     GwySpectra *spectra = NULL;
     guint i;
 
@@ -1171,8 +1163,8 @@ add_dline_to_spectra(GPtrArray *sps, GwyDataLine *dline,
         if (gwy_strequal(gwy_spectra_get_title(spectra), fullname)
             && gwy_strequal(gwy_spectra_get_spectrum_y_label(spectra), name)
             && ((!specxlabel && !xtitle) || (specxlabel && xtitle && gwy_strequal(specxlabel, xtitle)))
-            && gwy_si_unit_equal(gwy_data_line_get_si_unit_x(dline), gwy_data_line_get_si_unit_x(firstspec))
-            && gwy_si_unit_equal(gwy_data_line_get_si_unit_y(dline), gwy_data_line_get_si_unit_y(firstspec)))
+            && gwy_unit_equal(gwy_line_get_unit_x(dline), gwy_line_get_unit_x(firstspec))
+            && gwy_unit_equal(gwy_line_get_unit_y(dline), gwy_line_get_unit_y(firstspec)))
             break;
         spectra = NULL;
     }
@@ -1183,7 +1175,7 @@ add_dline_to_spectra(GPtrArray *sps, GwyDataLine *dline,
         gwy_spectra_set_spectrum_y_label(spectra, name);
         if (xtitle)
             gwy_spectra_set_spectrum_x_label(spectra, xtitle);
-        gwy_si_unit_set_from_string(gwy_spectra_get_si_unit_xy(spectra), "m");
+        gwy_unit_set_from_string(gwy_spectra_get_unit_xy(spectra), "m");
         g_ptr_array_add(sps, spectra);
     }
 
@@ -1248,27 +1240,11 @@ find_spectrum_abscissa(NAOSpectrum *spectrum, GError **error)
     GHashTable *hash = spectrum->hash;
     const gchar *value;
 
-    /* Ensure the parameter hash table exists. */
-    if (!hash) {
-        err_MISSING_FIELD(error, "SweepFromValue");
+    if (!require_keys(hash, error, "SweepFromValue", "SweepToValue", NULL))
         return FALSE;
-    }
 
-    if ((value = g_hash_table_lookup(hash, "SweepFromValue"))) {
-        spectrum->sweep_from = g_ascii_strtod(value, NULL);
-    }
-    else {
-        err_MISSING_FIELD(error, "SweepFromValue");
-        return FALSE;
-    }
-
-    if ((value = g_hash_table_lookup(hash, "SweepToValue"))) {
-        spectrum->sweep_to = g_ascii_strtod(value, NULL);
-    }
-    else {
-        err_MISSING_FIELD(error, "SweepToValue");
-        return FALSE;
-    }
+    spectrum->sweep_from = g_ascii_strtod(g_hash_table_lookup(hash, "SweepFromValue"), NULL);
+    spectrum->sweep_to = g_ascii_strtod(g_hash_table_lookup(hash, "SweepToValue"), NULL);
     gwy_debug("sweep from %g to %g", spectrum->sweep_from, spectrum->sweep_to);
 
     if ((value = g_hash_table_lookup(hash, "SweepSignalUnitName"))
@@ -1332,7 +1308,7 @@ nao_file_free(NAOFile *naofile)
     }
     g_free(naofile->current_name);
     g_free(naofile->imaging_parameters);
-    GWY_OBJECT_UNREF(naofile->meta);
+    g_clear_object(&naofile->meta);
 }
 
 static void
