@@ -1,5 +1,5 @@
 /*
- *  $Id: nanoscan.c 26168 2024-02-08 10:18:02Z yeti-dn $
+ *  $Id: nanoscan.c 28911 2025-11-24 18:27:42Z yeti-dn $
  *  Copyright (C) 2009 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -34,16 +34,10 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/stats.h>
-#include <libprocess/grains.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -126,67 +120,69 @@ typedef struct {
     GArray *meta;
 } NanoScanFile;
 
-static gboolean      module_register     (void);
-static gint          nanoscan_detect     (const GwyFileDetectInfo *fileinfo,
-                                          gboolean only_name);
-static GwyContainer* nanoscan_load       (const gchar *filename,
-                                          GwyRunType mode,
-                                          GError **error);
-static void          nanoscan_free       (NanoScanFile *nfile);
-static void          add_channel         (GwyContainer *container,
-                                          NanoScanFile *nfile,
-                                          const gchar *filename,
-                                          NanoScanChannel *channel,
-                                          gint id);
-static void          add_graph           (GwyContainer *container,
-                                          NanoScanFile *nfile,
-                                          NanoScanChannel *channel,
-                                          gint id);
-static void          add_curve_model     (NanoScanFile *nfile,
-                                          NanoScanChannel *channel,
-                                          guint i,
-                                          GwyGraphModel *gmodel);
-static void          add_multigraph      (GwyContainer *container,
-                                          NanoScanFile *nfile,
-                                          NanoScanChannel *channel,
-                                          gint id);
-static void          add_multicurve_model(NanoScanFile *nfile,
-                                          NanoScanChannel *channel,
-                                          guint i,
-                                          GwyGraphModel *gmodel);
-static void          fix_metadata        (NanoScanFile *nfile);
-static void          add_metadata        (GwyContainer *container,
-                                          NanoScanFile *nfile,
-                                          gint id);
-static void          start_element       (GMarkupParseContext *context,
-                                          const gchar *element_name,
-                                          const gchar **attribute_names,
-                                          const gchar **attribute_values,
-                                          gpointer user_data,
-                                          GError **error);
-static void          end_element         (GMarkupParseContext *context,
-                                          const gchar *element_name,
-                                          gpointer user_data,
-                                          GError **error);
-static void          text                (GMarkupParseContext *context,
-                                          const gchar *text,
-                                          gsize text_len,
-                                          gpointer user_data,
-                                          GError **error);
-static void          add_meta            (NanoScanFile *nfile,
-                                          const gchar *name,
-                                          gchar *value);
-static gfloat*       read_channel_data   (const gchar *value,
-                                          guint npixels,
-                                          guint *actual_npixels,
-                                          GError **error);
+static gboolean module_register     (void);
+static gint     detect_file         (const GwyFileDetectInfo *fileinfo,
+                                     gboolean only_name);
+static GwyFile* load_file           (const gchar *filename,
+                                     GwyRunModeFlags mode,
+                                     GError **error);
+static void     nanoscan_free       (NanoScanFile *nfile);
+static void     add_channel         (GwyFile *file,
+                                     NanoScanFile *nfile,
+                                     const gchar *filename,
+                                     NanoScanChannel *channel,
+                                     gint id);
+static void     add_graph           (GwyFile *file,
+                                     NanoScanFile *nfile,
+                                     const gchar *filename,
+                                     NanoScanChannel *channel,
+                                     gint id);
+static void     add_curve_model     (NanoScanFile *nfile,
+                                     NanoScanChannel *channel,
+                                     guint i,
+                                     GwyGraphModel *gmodel);
+static void     add_multigraph      (GwyFile *file,
+                                     NanoScanFile *nfile,
+                                     const gchar *filename,
+                                     NanoScanChannel *channel,
+                                     gint id);
+static void     add_multicurve_model(NanoScanFile *nfile,
+                                     NanoScanChannel *channel,
+                                     guint i,
+                                     GwyGraphModel *gmodel);
+static void     fix_metadata        (NanoScanFile *nfile);
+static void     add_metadata        (GwyFile *file,
+                                     NanoScanFile *nfile,
+                                     gint id);
+static void     start_element       (GMarkupParseContext *context,
+                                     const gchar *element_name,
+                                     const gchar **attribute_names,
+                                     const gchar **attribute_values,
+                                     gpointer user_data,
+                                     GError **error);
+static void     end_element         (GMarkupParseContext *context,
+                                     const gchar *element_name,
+                                     gpointer user_data,
+                                     GError **error);
+static void     text                (GMarkupParseContext *context,
+                                     const gchar *text,
+                                     gsize text_len,
+                                     gpointer user_data,
+                                     GError **error);
+static void     add_meta            (NanoScanFile *nfile,
+                                     const gchar *name,
+                                     gchar *value);
+static gfloat*  read_channel_data   (const gchar *value,
+                                     guint npixels,
+                                     guint *actual_npixels,
+                                     GError **error);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports NanoScan XML files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.9",
+    "0.10",
     "David Nečas (Yeti)",
     "2009",
 };
@@ -198,10 +194,7 @@ module_register(void)
 {
     gwy_file_func_register("nanoscan",
                            N_("NanoScan XML files (.xml)"),
-                           (GwyFileDetectFunc)&nanoscan_detect,
-                           (GwyFileLoadFunc)&nanoscan_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
@@ -215,8 +208,8 @@ check_magic(const gchar *header)
 }
 
 static gint
-nanoscan_detect(const GwyFileDetectInfo *fileinfo,
-                gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo,
+            gboolean only_name)
 {
     if (only_name)
         return g_str_has_suffix(fileinfo->name_lowercase, EXTENSION) ? 10 : 0;
@@ -224,12 +217,12 @@ nanoscan_detect(const GwyFileDetectInfo *fileinfo,
     return check_magic(fileinfo->head) ? 100 : 0;
 }
 
-static GwyContainer*
-nanoscan_load(const gchar *filename,
-              G_GNUC_UNUSED GwyRunType mode,
-              GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     NanoScanFile nfile;
     gchar *head, *buffer = NULL;
     GMarkupParser parser = { start_element, end_element, text, NULL, NULL };
@@ -243,7 +236,7 @@ nanoscan_load(const gchar *filename,
         goto fail;
     }
 
-    gwy_clear(&nfile, 1);
+    gwy_clear1(nfile);
     head = g_strndup(buffer, 1024);
     if (!check_magic(head)) {
         err_FILE_TYPE(error, "NanoScan XML");
@@ -284,8 +277,8 @@ nanoscan_load(const gchar *filename,
             axis->display_scale = 1.0;
     }
 
-    /* Construct a GwyContainer */
-    container = gwy_container_new();
+    /* Construct a GwyFile */
+    file = gwy_file_new_in_construction();
     fix_metadata(&nfile);
     for (i = id = 0; i < nfile.channels->len; i++) {
         NanoScanChannel *channel = &g_array_index(nfile.channels, NanoScanChannel, i);
@@ -293,12 +286,12 @@ nanoscan_load(const gchar *filename,
             continue;
 
         if (nfile.yres == 1 && nfile.axes->len >= 1)
-            add_graph(container, &nfile, channel, id+1);
+            add_graph(file, &nfile, filename, channel, id);
         else if (nfile.axes->len >= 2)
-            add_multigraph(container, &nfile, channel, id);
+            add_multigraph(file, &nfile, filename, channel, id);
         else {
-            add_channel(container, &nfile, filename, channel, id);
-            add_metadata(container, &nfile, id);
+            add_channel(file, &nfile, filename, channel, id);
+            add_metadata(file, &nfile, id);
         }
 
         id++;
@@ -306,14 +299,14 @@ nanoscan_load(const gchar *filename,
 
     if (!id) {
         err_NO_DATA(error);
-        GWY_OBJECT_UNREF(container);
+        g_clear_object(&file);
     }
 
 fail:
     g_free(buffer);
     nanoscan_free(&nfile);
 
-    return container;
+    return file;
 }
 
 static void
@@ -354,14 +347,14 @@ nanoscan_free(NanoScanFile *nfile)
 }
 
 static void
-add_channel(GwyContainer *container,
+add_channel(GwyFile *file,
             NanoScanFile *nfile,
             const gchar *filename,
             NanoScanChannel *channel,
             gint id)
 {
-    GwyDataField *dfield, *mask = NULL;
-    GwySIUnit *unit;
+    GwyField *dfield, *mask = NULL;
+    GwyUnit *unit;
     gdouble *d;
     gchar *title;
     gdouble yreal, q = 1.0;
@@ -370,43 +363,43 @@ add_channel(GwyContainer *container,
     nrows = (channel->ndata + nfile->xres-1)/nfile->xres;
     gwy_debug("nrows %d, yres %d", nrows, nfile->yres);
     yreal = nfile->yreal*nrows/nfile->yres;
-    dfield = gwy_data_field_new(nfile->xres, nrows, nfile->xreal, yreal, FALSE);
+    dfield = gwy_field_new(nfile->xres, nrows, nfile->xreal, yreal, FALSE);
     if (nfile->xyunits) {
-        unit = gwy_data_field_get_si_unit_xy(dfield);
-        gwy_si_unit_set_from_string_parse(unit, nfile->xyunits, &power10);
-        gwy_data_field_set_xreal(dfield, pow10(power10)*nfile->xreal);
-        gwy_data_field_set_yreal(dfield, pow10(power10)*yreal);
+        unit = gwy_field_get_unit_xy(dfield);
+        power10 = gwy_unit_set_from_string(unit, nfile->xyunits);
+        gwy_field_set_xreal(dfield, gwy_exp10(power10)*nfile->xreal);
+        gwy_field_set_yreal(dfield, gwy_exp10(power10)*yreal);
     }
     if (channel->zunits) {
-        unit = gwy_data_field_get_si_unit_z(dfield);
-        gwy_si_unit_set_from_string_parse(unit, channel->zunits, &power10);
-        q = pow10(power10);
+        unit = gwy_field_get_unit_z(dfield);
+        power10 = gwy_unit_set_from_string(unit, channel->zunits);
+        q = gwy_exp10(power10);
     }
 
-    d = gwy_data_field_get_data(dfield);
+    d = gwy_field_get_data(dfield);
     for (i = 0; i < channel->ndata; i++)
         d[i] = q*channel->data[i];
-    gwy_data_field_invert(dfield, TRUE, FALSE, FALSE);
+    gwy_field_flip(dfield, FALSE, TRUE);
 
     /* Create a mask that is the union of invalid and missing values. */
     mask = gwy_app_channel_mask_of_nans(dfield, FALSE);
     if (channel->ndata % nfile->xres != 0) {
         if (!mask)
-            mask = gwy_data_field_new_alike(dfield, TRUE);
+            mask = gwy_field_new_alike(dfield, TRUE);
 
-        d = gwy_data_field_get_data(mask);
+        d = gwy_field_get_data(mask);
         for (i = channel->ndata; i < nfile->xres*nrows; i++)
             d[i] = 1.0;
     }
 
     if (mask) {
-        gwy_data_field_grains_invert(mask);
+        gwy_field_grains_invert(mask);
         gwy_app_channel_remove_bad_data(dfield, mask);
     }
 
-    gwy_container_pass_object(container, gwy_app_get_data_key_for_id(id), dfield);
+    gwy_file_pass_image(file, id, dfield);
     if (mask)
-        gwy_container_pass_object(container, gwy_app_get_mask_key_for_id(id), mask);
+        gwy_file_pass_image_mask(file, id, mask);
 
     if (channel->name) {
         if (channel->direction == SCAN_FORWARD)
@@ -417,15 +410,16 @@ add_channel(GwyContainer *container,
             title = channel->name;
             channel->name = NULL;
         }
-        gwy_container_set_string(container, gwy_app_get_data_title_key_for_id(id), title);
+        gwy_file_pass_title(file, GWY_FILE_IMAGE, id, title);
     }
-    gwy_file_channel_import_log_add(container, id, NULL, filename);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, id, NULL, filename);
     channel->already_added = TRUE;
 }
 
 static void
-add_graph(GwyContainer *container,
+add_graph(GwyFile *file,
           NanoScanFile *nfile,
+          const gchar *filename,
           NanoScanChannel *channel,
           gint id)
 {
@@ -449,7 +443,8 @@ add_graph(GwyContainer *container,
     if (axis0->name)
         g_object_set(gmodel, "axis-label-bottom", axis0->name, NULL);
 
-    gwy_container_pass_object(container, gwy_app_get_graph_key_for_id(id), gmodel);
+    gwy_file_pass_graph(file, id, gmodel);
+    gwy_log_add_import(file, GWY_FILE_GRAPH, id, NULL, filename);
 }
 
 static void
@@ -460,31 +455,31 @@ add_curve_model(NanoScanFile *nfile,
 {
     NanoScanAxis *axis0 = &g_array_index(nfile->axes, NanoScanAxis, 0);
     GwyGraphCurveModel *gcmodel;
-    GwyDataLine *dline;
-    GwySIUnit *unit;
+    GwyLine *dline;
+    GwyUnit *unit;
     gdouble real, q = 1.0;
     gdouble *d;
     gint power10, j;
 
     real = (axis0->stop - axis0->start)*channel->ndata/nfile->xres;
-    dline = gwy_data_line_new(channel->ndata, real, FALSE);
-    gwy_data_line_set_offset(dline, axis0->start);
+    dline = gwy_line_new(channel->ndata, real, FALSE);
+    gwy_line_set_offset(dline, axis0->start);
     if (axis0->units) {
-        unit = gwy_data_line_get_si_unit_x(dline);
-        gwy_si_unit_set_from_string_parse(unit, axis0->units, &power10);
-        gwy_data_line_set_real(dline, pow10(power10)*real);
-        gwy_data_line_set_offset(dline, pow10(power10)*axis0->start);
+        unit = gwy_line_get_unit_x(dline);
+        power10 = gwy_unit_set_from_string(unit, axis0->units);
+        gwy_line_set_real(dline, gwy_exp10(power10)*real);
+        gwy_line_set_offset(dline, gwy_exp10(power10)*axis0->start);
     }
     if (channel->zunits) {
-        unit = gwy_data_line_get_si_unit_y(dline);
-        gwy_si_unit_set_from_string_parse(unit, channel->zunits, &power10);
-        q = pow10(power10);
+        unit = gwy_line_get_unit_y(dline);
+        power10 = gwy_unit_set_from_string(unit, channel->zunits);
+        q = gwy_exp10(power10);
     }
-    d = gwy_data_line_get_data(dline);
+    d = gwy_line_get_data(dline);
     for (j = 0; j < channel->ndata; j++)
         d[j] = q*channel->data[j];
     gcmodel = gwy_graph_curve_model_new();
-    gwy_graph_curve_model_set_data_from_dataline(gcmodel, dline, 0, 0);
+    gwy_graph_curve_model_set_data_from_line(gcmodel, dline, 0, 0);
     g_object_set(gcmodel,
                  "mode", GWY_GRAPH_CURVE_LINE,
                  "color", gwy_graph_get_preset_color(i),
@@ -499,20 +494,20 @@ add_curve_model(NanoScanFile *nfile,
 
     gwy_graph_model_add_curve(gmodel, gcmodel);
     g_object_unref(gcmodel);
-    gwy_graph_model_set_units_from_data_line(gmodel, dline);
+    gwy_graph_model_set_units_from_line(gmodel, dline);
     g_object_unref(dline);
     channel->already_added = TRUE;
 }
 
 static void
-add_multigraph(GwyContainer *container,
+add_multigraph(GwyFile *file,
                NanoScanFile *nfile,
+               const gchar *filename,
                NanoScanChannel *channel,
                gint id)
 {
     NanoScanAxis *axis0 = &g_array_index(nfile->axes, NanoScanAxis, 0);
     GwyGraphModel *gmodel;
-    GQuark quark;
     guint i;
 
     gmodel = gwy_graph_model_new();
@@ -537,8 +532,8 @@ add_multigraph(GwyContainer *container,
     if (axis0->name)
         g_object_set(gmodel, "axis-label-bottom", axis0->name, NULL);
 
-    quark = gwy_app_get_graph_key_for_id(id);
-    gwy_container_pass_object(container, quark, gmodel);
+    gwy_file_pass_graph(file, id, gmodel);
+    gwy_log_add_import(file, GWY_FILE_GRAPH, id, NULL, filename);
     channel->already_added = TRUE;
 }
 
@@ -551,8 +546,8 @@ add_multicurve_model(NanoScanFile *nfile,
     NanoScanAxis *axis0 = &g_array_index(nfile->axes, NanoScanAxis, 0);
     NanoScanAxis *axis1 = &g_array_index(nfile->axes, NanoScanAxis, 1);
     GwyGraphCurveModel *gcmodel;
-    GwyDataLine *dline;
-    GwySIUnit *unit;
+    GwyLine *dline;
+    GwyUnit *unit;
     gdouble real, yval, q = 1.0;
     gdouble *d;
     gchar *descr;
@@ -565,24 +560,24 @@ add_multicurve_model(NanoScanFile *nfile,
 
     npts = MIN(channel->ndata - start, nfile->xres);
     real = (axis0->stop - axis0->start)*npts/nfile->xres;
-    dline = gwy_data_line_new(npts, real, TRUE);
-    gwy_data_line_set_offset(dline, axis0->start);
+    dline = gwy_line_new(npts, real, TRUE);
+    gwy_line_set_offset(dline, axis0->start);
     if (axis0->units) {
-        unit = gwy_data_line_get_si_unit_x(dline);
-        gwy_si_unit_set_from_string_parse(unit, axis0->units, &power10);
-        gwy_data_line_set_real(dline, pow10(power10)*real);
-        gwy_data_line_set_offset(dline, pow10(power10)*axis0->start);
+        unit = gwy_line_get_unit_x(dline);
+        power10 = gwy_unit_set_from_string(unit, axis0->units);
+        gwy_line_set_real(dline, gwy_exp10(power10)*real);
+        gwy_line_set_offset(dline, gwy_exp10(power10)*axis0->start);
     }
     if (channel->zunits) {
-        unit = gwy_data_line_get_si_unit_y(dline);
-        gwy_si_unit_set_from_string_parse(unit, channel->zunits, &power10);
-        q = pow10(power10);
+        unit = gwy_line_get_unit_y(dline);
+        power10 = gwy_unit_set_from_string(unit, channel->zunits);
+        q = gwy_exp10(power10);
     }
-    d = gwy_data_line_get_data(dline);
+    d = gwy_line_get_data(dline);
     for (j = 0; j < npts; j++)
         d[j] = q*channel->data[start + j];
     gcmodel = gwy_graph_curve_model_new();
-    gwy_graph_curve_model_set_data_from_dataline(gcmodel, dline, 0, 0);
+    gwy_graph_curve_model_set_data_from_line(gcmodel, dline, 0, 0);
     yval = i/(nfile->yres - 1.0)*(axis1->stop - axis1->start) + axis1->start;
     descr = g_strdup_printf("%s %g%s%s",
                             axis1->name ? axis1->name : "Y",
@@ -598,7 +593,7 @@ add_multicurve_model(NanoScanFile *nfile,
 
     gwy_graph_model_add_curve(gmodel, gcmodel);
     g_object_unref(gcmodel);
-    gwy_graph_model_set_units_from_data_line(gmodel, dline);
+    gwy_graph_model_set_units_from_line(gmodel, dline);
     g_object_unref(dline);
     channel->already_added = TRUE;
 }
@@ -634,7 +629,7 @@ fix_metadata(NanoScanFile *nfile)
 }
 
 static void
-add_metadata(GwyContainer *container,
+add_metadata(GwyFile *file,
              NanoScanFile *nfile,
              gint id)
 {
@@ -644,7 +639,7 @@ add_metadata(GwyContainer *container,
     if (!nfile->meta->len)
         return;
 
-    metadata = gwy_container_new();
+    metadata = gwy_container_new_in_construction();
     for (i = 0; i < nfile->meta->len; i++) {
         NanoScanMeta *meta = &g_array_index(nfile->meta, NanoScanMeta, i);
 
@@ -652,7 +647,7 @@ add_metadata(GwyContainer *container,
             gwy_container_set_const_string_by_name(metadata, meta->name, meta->value);
     }
 
-    gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(id), metadata);
+    gwy_file_pass_meta(file, GWY_FILE_IMAGE, id, metadata);
 }
 
 static void
@@ -679,7 +674,7 @@ start_element(G_GNUC_UNUSED GMarkupParseContext *context,
 
         /* Individual channels are inside <contents> tags, but there can be empty <contents> tags and stuff.  Just
          * create what we see and hope upstream cann sort it out. */
-        gwy_clear(&channel, 1);
+        gwy_clear1(channel);
         channel.direction = nfile->current_direction;
         g_array_append_val(nfile->channels, channel);
         gwy_debug("channel #%u", (guint)nfile->channels->len);
@@ -687,7 +682,7 @@ start_element(G_GNUC_UNUSED GMarkupParseContext *context,
     else if (gwy_strequal(nfile->path->str, AXIS_PREFIX)) {
         NanoScanAxis axis;
 
-        gwy_clear(&axis, 1);
+        gwy_clear1(axis);
         g_array_append_val(nfile->axes, axis);
         gwy_debug("axis #%u", (guint)nfile->axes->len);
     }
