@@ -1,6 +1,6 @@
 /*
- *  $Id: nanoscantech.c 26491 2024-08-09 14:31:55Z yeti-dn $
- *  Copyright (C) 2012-2019 David Necas (Yeti), Daniil Bratashov (dn2010).
+ *  $Id: nanoscantech.c 28803 2025-11-05 11:58:19Z yeti-dn $
+ *  Copyright (C) 2012-2025 David Necas (Yeti), Daniil Bratashov (dn2010).
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
  *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
@@ -45,15 +45,11 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
 #include <limits.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 
 #include "err.h"
 #include "gwyzip.h"
@@ -100,12 +96,12 @@ typedef struct {
 } NST4DHeader;
 
 static gboolean       module_register(void);
-static gint           nst_detect     (const GwyFileDetectInfo *fileinfo,
+static gint           detect_file    (const GwyFileDetectInfo *fileinfo,
                                       gboolean only_name);
-static GwyContainer*  nst_load       (const gchar *filename,
-                                      GwyRunType mode,
+static GwyFile*       load_file      (const gchar *filename,
+                                      GwyRunModeFlags mode,
                                       GError **error);
-static GwyDataField*  nst_read_3d    (const gchar *buffer,
+static GwyField*      nst_read_3d    (const gchar *buffer,
                                       gsize size,
                                       gboolean is_utf,
                                       GwyContainer **metadata,
@@ -124,7 +120,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports NanoScanTech .nstdat files."),
     "Daniil Bratashov (dn2010@gmail.com)",
-    "0.17",
+    "0.18",
     "David Nečas (Yeti), Daniil Bratashov (dn2010), Antony Kikaxa",
     "2012",
 };
@@ -136,17 +132,13 @@ module_register(void)
 {
     gwy_file_func_register("nanoscantech",
                            N_("NanoScanTech data (.nstdat)"),
-                           (GwyFileDetectFunc)&nst_detect,
-                           (GwyFileLoadFunc)&nst_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-nst_detect(const GwyFileDetectInfo *fileinfo,
-           gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     GwyZipFile zipfile;
 
@@ -177,13 +169,14 @@ nst_detect(const GwyFileDetectInfo *fileinfo,
     return 100;
 }
 
-static GwyContainer*
-nst_load(const gchar *filename,
-         G_GNUC_UNUSED GwyRunType mode,
-         GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *container = NULL, *metadata = NULL;
-    GwyDataField *dfield;
+    GwyFile *file = NULL;
+    GwyContainer *metadata = NULL;
+    GwyField *dfield;
     GwyGraphModel *gmodel;
     GwyBrick *brick;
     GwyZipFile zipfile;
@@ -200,7 +193,7 @@ nst_load(const gchar *filename,
     if (!status)
         goto fail;
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     while (status && gwyzip_get_current_filename(zipfile, &filename_curr, NULL)) {
         if (g_str_has_suffix(filename_curr, ".lsdlsd")) {
             gwy_debug("channel %d: %s", channelno, filename_curr);
@@ -222,37 +215,39 @@ nst_load(const gchar *filename,
                 titlestr = NULL;
                 dfield = nst_read_3d(p, size, is_utf, &metadata, &titlestr);
                 if (dfield) {
-                    gwy_container_pass_object(container, gwy_app_get_data_key_for_id(channelno), dfield);
+                    gwy_file_pass_image(file, channelno, dfield);
                     if (metadata)
-                        gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(channelno), metadata);
+                        gwy_file_pass_meta(file, GWY_FILE_IMAGE, channelno, metadata);
                     if (!titlestr)
                         title = g_strdup_printf("Channel %u", channelno);
                     else
                         title = g_strdup_printf("%s (%u)", titlestr, channelno);
-                    gwy_container_set_string(container, gwy_app_get_data_title_key_for_id(channelno), title);
-                    gwy_file_channel_import_log_add(container, channelno, NULL, filename);
+                    gwy_file_pass_title(file, GWY_FILE_IMAGE, channelno, title);
+                    gwy_log_add_import(file, GWY_FILE_IMAGE, channelno, NULL, filename);
                 }
                 GWY_FREE(titlestr);
             }
             else if (gwy_strequal(line, "2d")) {
                 gwy_debug("2d: %d", channelno);
                 gmodel = nst_read_2d(p, channelno, is_utf);
-                if (gmodel)
-                    gwy_container_pass_object(container, gwy_app_get_graph_key_for_id(channelno+1), gmodel);
+                if (gmodel) {
+                    gwy_file_pass_graph(file, channelno, gmodel);
+                    gwy_log_add_import(file, GWY_FILE_GRAPH, channelno, NULL, filename);
+                }
             }
             else if (gwy_strequal(line, "4d")) {
                 gwy_debug("4d: %u", channelno);
                 brick = nst_read_4d(p, size, is_utf, &metadata, &titlestr);
                 if (brick) {
-                    gwy_container_pass_object(container, gwy_app_get_brick_key_for_id(channelno), brick);
+                    gwy_file_pass_volume(file, channelno, brick);
                     if (!titlestr)
                         title = g_strdup_printf("Channel %u", channelno);
                     else
                         title = g_strdup_printf("%s (%u)", titlestr, channelno);
-                    gwy_container_set_string(container, gwy_app_get_brick_title_key_for_id(channelno), title);
+                    gwy_file_pass_title(file, GWY_FILE_VOLUME, channelno, title);
                     if (metadata)
-                        gwy_container_pass_object(container, gwy_app_get_brick_meta_key_for_id(channelno), metadata);
-                    gwy_file_volume_import_log_add(container, channelno, NULL, filename);
+                        gwy_file_pass_meta(file, GWY_FILE_VOLUME, channelno, metadata);
+                    gwy_log_add_import(file, GWY_FILE_VOLUME, channelno, NULL, filename);
                 }
                 GWY_FREE(titlestr);
             }
@@ -267,12 +262,12 @@ nst_load(const gchar *filename,
 fail:
     gwyzip_close(zipfile);
     if (!channelno) {
-        GWY_OBJECT_UNREF(container);
+        g_clear_object(&file);
         if (error && !*error)
             err_NO_DATA(error);
     }
 
-    return container;
+    return file;
 }
 
 static gchar**
@@ -294,13 +289,13 @@ decode_string(const gchar *s, gboolean is_utf)
     return is_utf ? g_strdup(s) : gwy_convert_to_utf8(s, -1, "cp1251");
 }
 
-static GwyDataField *
+static GwyField *
 nst_read_3d(const gchar *buffer, gsize size, gboolean is_utf,
             GwyContainer **metadata, gchar **title)
 {
-    GwyDataField *dfield = NULL;
+    GwyField *dfield = NULL;
     GwyContainer *meta = NULL;
-    GwySIUnit *siunitxy = NULL, *siunitz = NULL;
+    GwyUnit *siunitxy = NULL, *siunitz = NULL;
     gchar *p, *line, *unit, *key, *value;
     gchar **lineparts, **attributes;
     gint x, y, xmax = 0, ymax = 0, xres = 1, yres = 1, i, j;
@@ -315,7 +310,7 @@ nst_read_3d(const gchar *buffer, gsize size, gboolean is_utf,
 
     p = (gchar*)buffer;
     dataarray = g_array_new(FALSE, TRUE, sizeof(gdouble));
-    meta = gwy_container_new();
+    meta = gwy_container_new_in_construction();
     while ((line = gwy_str_next_line(&p))) {
         if (gwy_strequal(line, "[BeginOfItem]")) {
             if (is_binary) {
@@ -367,7 +362,7 @@ nst_read_3d(const gchar *buffer, gsize size, gboolean is_utf,
             if (!(lineparts = split_to_nparts(line, " ", 3)))
                 goto fail;
             unit = decode_string(lineparts[1], is_utf);
-            siunitxy = gwy_si_unit_new_parse(unit, &power10xy);
+            siunitxy = gwy_unit_new_parse(unit, &power10xy);
             g_free(unit);
             x = atoi(lineparts[2]);
             if (x != 0)
@@ -378,7 +373,7 @@ nst_read_3d(const gchar *buffer, gsize size, gboolean is_utf,
             if (!(lineparts = split_to_nparts(line, " ", 3)))
                 goto fail;
             unit = decode_string(lineparts[1], is_utf);
-            siunitz = gwy_si_unit_new_parse(unit, &power10z);
+            siunitz = gwy_unit_new_parse(unit, &power10z);
             g_free(unit);
             z = atoi(lineparts[2]);
             if (z != 0)
@@ -423,11 +418,11 @@ nst_read_3d(const gchar *buffer, gsize size, gboolean is_utf,
                 }
                 else if (g_str_has_prefix(key, "XYUnit")) {
                     if (!siunitxy)
-                        siunitxy = gwy_si_unit_new_parse(value, &power10xy);
+                        siunitxy = gwy_unit_new_parse(value, &power10xy);
                 }
                 else if (g_str_has_prefix(key, "ZUnit")) {
                     if (!siunitz)
-                        siunitz = gwy_si_unit_new_parse(value, &power10z);
+                        siunitz = gwy_unit_new_parse(value, &power10z);
                 }
                 else if (g_str_has_prefix(key, "XMin")) {
                     xoffset = g_ascii_strtod(value, NULL);
@@ -458,24 +453,24 @@ nst_read_3d(const gchar *buffer, gsize size, gboolean is_utf,
 
     sanitise_real_size(&xscale, "x scale");
     sanitise_real_size(&yscale, "y scale");
-    dfield = gwy_data_field_new(xres, yres, xscale*pow10(power10xy), yscale*pow10(power10xy), TRUE);
-    gwy_data_field_set_xoffset(dfield, xoffset*pow10(power10xy));
-    gwy_data_field_set_yoffset(dfield, yoffset*pow10(power10xy));
+    dfield = gwy_field_new(xres, yres, xscale*gwy_exp10(power10xy), yscale*gwy_exp10(power10xy), TRUE);
+    gwy_field_set_xoffset(dfield, xoffset*gwy_exp10(power10xy));
+    gwy_field_set_yoffset(dfield, yoffset*gwy_exp10(power10xy));
 
-    gwy_assign(gwy_data_field_get_data(dfield), dataarray->data, xres*yres);
+    gwy_assign(gwy_field_get_data(dfield), dataarray->data, xres*yres);
 
     if (siunitxy)
-        gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(dfield), siunitxy);
+        gwy_unit_assign(gwy_field_get_unit_xy(dfield), siunitxy);
     if (siunitz)
-        gwy_si_unit_assign(gwy_data_field_get_si_unit_z(dfield), siunitz);
+        gwy_unit_assign(gwy_field_get_unit_z(dfield), siunitz);
 
     *metadata = g_object_ref(meta);
 
 fail:
-    GWY_OBJECT_UNREF(meta);
+    g_clear_object(&meta);
     g_array_free(dataarray, TRUE);
-    GWY_OBJECT_UNREF(siunitz);
-    GWY_OBJECT_UNREF(siunitxy);
+    g_clear_object(&siunitz);
+    g_clear_object(&siunitxy);
 
     return dfield;
 }
@@ -485,7 +480,7 @@ nst_read_2d(const gchar *buffer, guint channel, gboolean is_utf)
 {
     GwyGraphCurveModel *spectra;
     GwyGraphModel *gmodel;
-    GwySIUnit *siunitx = NULL, *siunity = NULL;
+    GwyUnit *siunitx = NULL, *siunity = NULL;
     gchar *p, *line, *unit, *key, *value, *xlabel = NULL, *ylabel = NULL;
     gchar **lineparts, **attributes;
     gint linecur;
@@ -532,8 +527,8 @@ nst_read_2d(const gchar *buffer, guint channel, gboolean is_utf)
                 ydata = g_new(gdouble, numpoints);
 
                 for (i = 0; i < numpoints; i++) {
-                    xdata[i] = g_array_index(xarray, gdouble, i) * pow10(power10x);
-                    ydata[i] = g_array_index(yarray, gdouble, i) * pow10(power10y);
+                    xdata[i] = g_array_index(xarray, gdouble, i) * gwy_exp10(power10x);
+                    ydata[i] = g_array_index(yarray, gdouble, i) * gwy_exp10(power10y);
                 }
 
                 spectra = gwy_graph_curve_model_new();
@@ -566,7 +561,7 @@ nst_read_2d(const gchar *buffer, guint channel, gboolean is_utf)
             if (!(lineparts = split_to_nparts(line, " ", 3)))
                 goto fail;
             unit = decode_string(lineparts[1], is_utf);
-            siunitx = gwy_si_unit_new_parse(unit, &power10x);
+            siunitx = gwy_unit_new_parse(unit, &power10x);
             g_free(unit);
             x = atoi(lineparts[2]);
             if (x != 0)
@@ -577,7 +572,7 @@ nst_read_2d(const gchar *buffer, guint channel, gboolean is_utf)
             if (!(lineparts = split_to_nparts(line, " ", 3)))
                 goto fail;
             unit = decode_string(lineparts[1], is_utf);
-            siunity = gwy_si_unit_new_parse(unit, &power10y);
+            siunity = gwy_unit_new_parse(unit, &power10y);
             g_free(unit);
             y = atoi(lineparts[2]);
             if (y != 0)
@@ -609,11 +604,11 @@ nst_read_2d(const gchar *buffer, guint channel, gboolean is_utf)
                 }
                 else if (g_str_has_prefix(key, "XUnit")) {
                     if (!siunitx)
-                        siunitx = gwy_si_unit_new_parse(value, &power10x);
+                        siunitx = gwy_unit_new_parse(value, &power10x);
                 }
                 else if (g_str_has_prefix(key, "YUnit")) {
                     if (!siunity)
-                        siunity = gwy_si_unit_new_parse(value, &power10y);
+                        siunity = gwy_unit_new_parse(value, &power10y);
                 }
 
                 g_free(key);
@@ -634,10 +629,10 @@ nst_read_2d(const gchar *buffer, guint channel, gboolean is_utf)
     g_object_set(gmodel, "title", title, NULL);
 
     if (siunitx)
-        g_object_set(gmodel, "si-unit-x", siunitx, NULL);
+        g_object_set(gmodel, "unit-x", siunitx, NULL);
 
     if (siunity)
-        g_object_set(gmodel, "si-unit-y", siunity, NULL);
+        g_object_set(gmodel, "unit-y", siunity, NULL);
 
     if (xlabel)
         gwy_graph_model_set_axis_label(gmodel, GTK_POS_BOTTOM, xlabel);
@@ -649,11 +644,11 @@ fail:
     g_free(title);
     g_free(ylabel);
     g_free(xlabel);
-    GWY_OBJECT_UNREF(siunitx);
-    GWY_OBJECT_UNREF(siunity);
+    g_clear_object(&siunitx);
+    g_clear_object(&siunity);
 
     if (!ok)
-        GWY_OBJECT_UNREF(gmodel);
+        g_clear_object(&gmodel);
 
     return gmodel;
 }
@@ -664,7 +659,7 @@ nst_read_4d(const gchar *buffer, gsize datasize, gboolean is_utf,
 {
     GwyBrick *brick = NULL, *brick_cropped = NULL;
     GwyContainer *meta = NULL;
-    GwyDataLine *calibration = NULL;
+    GwyLine *calibration = NULL;
     NST4DHeader *header = NULL;
     guint xres, yres, zres, zcrop;
     gdouble xreal, yreal, zreal;
@@ -805,7 +800,7 @@ nst_read_4d(const gchar *buffer, gsize datasize, gboolean is_utf,
             break;
         }
         else if (g_str_has_prefix(line, "Attributes")) {
-            meta = gwy_container_new();
+            meta = gwy_container_new_in_construction();
             if (!(lineparts = split_to_nparts(line, " ", 2)))
                 goto exit;
             attributes = g_strsplit(lineparts[1], "*_*|^_^", 1024);
@@ -840,18 +835,18 @@ exit2:
         }
 
         data = NULL;
-        calibration = gwy_data_line_new(zres, zreal, TRUE);
-        data = gwy_data_line_get_data(calibration);
+        calibration = gwy_line_new(zres, zreal, TRUE);
+        data = gwy_line_get_data(calibration);
         for (i = 0; i < zres; i++)
             *(data++) = 1e-9*(header->centerwl + header->dispersion * header->pixelxsize * (i - header->centralpixel));
-        gwy_si_unit_set_from_string(gwy_data_line_get_si_unit_y(calibration), "m");
+        gwy_unit_set_from_string(gwy_line_get_unit_y(calibration), "m");
         gwy_brick_set_zcalibration(brick, calibration);
         g_object_unref(calibration);
 
-        gwy_si_unit_set_from_string(gwy_brick_get_si_unit_x(brick), "m");
-        gwy_si_unit_set_from_string(gwy_brick_get_si_unit_y(brick), "m");
-        gwy_si_unit_set_from_string(gwy_brick_get_si_unit_z(brick), "m");
-        gwy_si_unit_set_from_string(gwy_brick_get_si_unit_w(brick), "Counts");
+        gwy_unit_set_from_string(gwy_brick_get_unit_x(brick), "m");
+        gwy_unit_set_from_string(gwy_brick_get_unit_y(brick), "m");
+        gwy_unit_set_from_string(gwy_brick_get_unit_z(brick), "m");
+        gwy_unit_set_from_string(gwy_brick_get_unit_w(brick), "Counts");
     }
     *metadata = meta;
 
