@@ -1,6 +1,6 @@
 /*
- *  $Id: nanoscope-ii.c 25636 2023-09-07 12:19:33Z yeti-dn $
- *  Copyright (C) 2004 David Necas (Yeti), Petr Klapetek.
+ *  $Id: nanoscope-ii.c 28789 2025-11-04 17:14:03Z yeti-dn $
+ *  Copyright (C) 2004-2025 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -39,14 +39,10 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/stats.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -59,29 +55,29 @@ enum { HEADER_SIZE = 2048 };
 
 typedef struct {
     GHashTable *hash;
-    GwyDataField *data_field;
+    GwyField *field;
 } NanoscopeData;
 
-static gboolean        module_register       (void);
-static gint            nanoscope_detect      (const GwyFileDetectInfo *fileinfo,
-                                              gboolean only_name);
-static GwyContainer*   nanoscope_load        (const gchar *filename,
-                                              GwyRunType mode,
-                                              GError **error);
-static GwyDataField*   hash_to_data_field    (GHashTable *hash,
-                                              const guchar *buffer,
-                                              gsize size,
-                                              GError **error);
-static GHashTable*     read_hash             (gchar *buffer,
-                                              GError **error);
-static GwyContainer*   nanoscope_get_metadata(GHashTable *hash);
+static gboolean      module_register       (void);
+static gint          detect_file           (const GwyFileDetectInfo *fileinfo,
+                                            gboolean only_name);
+static GwyFile*      load_file             (const gchar *filename,
+                                            GwyRunModeFlags mode,
+                                            GError **error);
+static GwyField*     hash_to_field         (GHashTable *hash,
+                                            const guchar *buffer,
+                                            gsize size,
+                                            GError **error);
+static GHashTable*   read_hash             (gchar *buffer,
+                                            GError **error);
+static GwyContainer* nanoscope_get_metadata(GHashTable *hash);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Digital Instruments Nanoscope II data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.3",
+    "0.4",
     "David Nečas (Yeti) & Petr Klapetek",
     "2007",
 };
@@ -93,17 +89,13 @@ module_register(void)
 {
     gwy_file_func_register("nanoscope-ii",
                            N_("Nanoscope II files"),
-                           (GwyFileDetectFunc)&nanoscope_detect,
-                           (GwyFileLoadFunc)&nanoscope_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-nanoscope_detect(const GwyFileDetectInfo *fileinfo,
-                 gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     gint score = 0;
 
@@ -118,13 +110,14 @@ nanoscope_detect(const GwyFileDetectInfo *fileinfo,
     return score;
 }
 
-static GwyContainer*
-nanoscope_load(const gchar *filename,
-               G_GNUC_UNUSED GwyRunType mode,
-               GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *meta, *container = NULL;
-    GwyDataField *dfield;
+    GwyFile *file = NULL;
+    GwyContainer *meta;
+    GwyField *dfield;
     GError *err = NULL;
     guchar *buffer = NULL;
     gchar *header;
@@ -152,8 +145,8 @@ nanoscope_load(const gchar *filename,
         return NULL;
     }
 
-    dfield = hash_to_data_field(hash, buffer + HEADER_SIZE, size - HEADER_SIZE,
-                                &err);
+    dfield = hash_to_field(hash, buffer + HEADER_SIZE, size - HEADER_SIZE,
+                           &err);
     meta = nanoscope_get_metadata(hash);
     gwy_file_abandon_contents(buffer, size, NULL);
     g_hash_table_destroy(hash);
@@ -161,26 +154,24 @@ nanoscope_load(const gchar *filename,
     if (!dfield)
         return NULL;
 
-    container = gwy_container_new();
-    gwy_container_pass_object_by_name(container, "/0/data", dfield);
+    file = gwy_file_new_in_construction();
+    gwy_file_pass_image(file, 0, dfield);
+    if (meta)
+        gwy_file_pass_meta(file, GWY_FILE_IMAGE, 0, meta);
 
-    if (meta) {
-        gwy_container_pass_object_by_name(container, "/0/meta", meta);
-    }
+    gwy_image_title_fall_back(file, 0);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
 
-    gwy_app_channel_title_fall_back(container, 0);
-    gwy_file_channel_import_log_add(container, 0, NULL, filename);
-
-    return container;
+    return file;
 }
 
-static GwyDataField*
-hash_to_data_field(GHashTable *hash,
-                   const guchar *buffer,
-                   gsize size,
-                   GError **error)
+static GwyField*
+hash_to_field(GHashTable *hash,
+              const guchar *buffer,
+              gsize size,
+              GError **error)
 {
-    GwyDataField *dfield;
+    GwyField *dfield;
     gchar *val;
     gint xres, yres, i;
     gdouble xreal, yreal, q;
@@ -210,16 +201,16 @@ hash_to_data_field(GHashTable *hash,
         err_INVALID(error, "scan_sz");
         return NULL;
     }
-    dfield = gwy_data_field_new(xres, yres, xreal, yreal, FALSE);
-    data = gwy_data_field_get_data(dfield);
+    dfield = gwy_field_new(xres, yres, xreal, yreal, FALSE);
+    data = gwy_field_get_data(dfield);
 
     for (i = 0; i < yres; i++) {
         gwy_convert_raw_data(buffer + (yres-1 - i)*xres*sizeof(gint16), xres, 1,
                              GWY_RAW_DATA_SINT16, GWY_BYTE_ORDER_LITTLE_ENDIAN, data + i*xres, q, 0.0);
     }
 
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), "m");
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
+    gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), "m");
 
     return dfield;
 }
@@ -276,7 +267,7 @@ nanoscope_get_metadata(GHashTable *hash)
     const gchar *val;
     guint i;
 
-    meta = gwy_container_new();
+    meta = gwy_container_new_in_construction();
     for (i = 0; i < G_N_ELEMENTS(metadata); i++) {
         if ((val = g_hash_table_lookup(hash, metadata[i].key)))
             gwy_container_set_string_by_name(meta, metadata[i].name, g_strconcat(val, metadata[i].unit, NULL));
