@@ -1,6 +1,6 @@
 /*
- *  $Id: nmmxyz.c 26519 2024-08-15 18:04:20Z yeti-dn $
- *  Copyright (C) 2016-2023 David Necas (Yeti).
+ *  $Id: nmmxyz.c 28789 2025-11-04 17:14:03Z yeti-dn $
+ *  Copyright (C) 2016-2025 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
  *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
@@ -44,16 +44,11 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <libprocess/datafield.h>
-#include <libgwydgets/gwydgetutils.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 
 #include "get.h"
 #include "err.h"
@@ -138,10 +133,10 @@ typedef struct {
 } ModuleGUI;
 
 static gboolean         module_register             (void);
-static gint             nmmxyz_detect               (const GwyFileDetectInfo *fileinfo,
+static gint             detect_file                 (const GwyFileDetectInfo *fileinfo,
                                                      gboolean only_name);
-static GwyContainer*    nmmxyz_load                 (const gchar *filename,
-                                                     GwyRunType mode,
+static GwyFile*         load_file                   (const gchar *filename,
+                                                     GwyRunModeFlags mode,
                                                      GError **error);
 static GwyDialogOutcome run_gui                     (ModuleArgs *args);
 static void             param_changed               (ModuleGUI *gui,
@@ -157,7 +152,7 @@ static GwyXYZ*          create_points_with_xy       (GwyDoubleArray *data,
 static void             rotate_points_xy            (GwyXYZ *points,
                                                      guint ndata,
                                                      gdouble angle);
-static void             create_data_field           (GwyContainer *container,
+static void             create_field                (GwyFile *file,
                                                      GwyContainer *meta,
                                                      ModuleArgs *args,
                                                      GwyDoubleArray *data,
@@ -201,7 +196,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports Nano Measuring Machine profile files."),
     "Yeti <yeti@gwyddion.net>",
-    "2.1",
+    "3.0",
     "David Nečas (Yeti)",
     "2016",
 };
@@ -213,10 +208,7 @@ module_register(void)
 {
     gwy_file_func_register("nmmxyz",
                            N_("Nano Measuring Machine files (*.dsc)"),
-                           (GwyFileDetectFunc)&nmmxyz_detect,
-                           (GwyFileLoadFunc)&nmmxyz_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
     return TRUE;
 }
 
@@ -226,7 +218,7 @@ static GwyParamDef*
 define_module_params(GArray *dscs)
 {
     static const GwyEnum xymodes[] = {
-        { N_("Non-square pixels"),   NMM_XYMODE_NONSQUARE, },
+        { N_("Non-square pixels"),    NMM_XYMODE_NONSQUARE, },
         { N_("Specify X resolution"), NMM_XYMODE_USE_XRES,  },
         { N_("Specify Y resolution"), NMM_XYMODE_USE_YRES,  },
     };
@@ -282,7 +274,7 @@ define_module_params(GArray *dscs)
 }
 
 static gint
-nmmxyz_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     gint score = 0;
 
@@ -296,14 +288,15 @@ nmmxyz_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
     return score;
 }
 
-static GwyContainer*
-nmmxyz_load(const gchar *filename, GwyRunType mode, GError **error)
+static GwyFile*
+load_file(const gchar *filename, GwyRunModeFlags mode, GError **error)
 {
     const NMMXYZProfileDesc *dsc;
     NMMXYZInfo info;
     ModuleArgs args;
     GwyParams *params;
-    GwyContainer *container = NULL, *meta = NULL;
+    GwyFile *file = NULL;
+    GwyContainer *meta = NULL;
     GwyDialogOutcome outcome;
     GwyParamDef *pardef = NULL;
     GwyDoubleArray *data = NULL;
@@ -312,7 +305,7 @@ nmmxyz_load(const gchar *filename, GwyRunType mode, GError **error)
     gboolean align_scanlines, plot_density, waiting = FALSE;
     gint err_channel;
     gdouble f, angle = 0.5*G_PI;
-    const guchar *s;
+    const gchar *s;
     guint i, ntodo, skip_err_channel = G_MAXUINT;
 
     /* In principle we can load data non-interactively, but it is going to take several minutes which is not good for
@@ -322,8 +315,8 @@ nmmxyz_load(const gchar *filename, GwyRunType mode, GError **error)
                     _("Nano Measuring Machine data import must be run as interactive."));
         return NULL;
     }
-    gwy_clear(&args, 1);
-    gwy_clear(&info, 1);
+    gwy_clear1(args);
+    gwy_clear1(info);
 
     meta = parse_dsc_file(filename);
     info.filename = filename;
@@ -407,16 +400,16 @@ nmmxyz_load(const gchar *filename, GwyRunType mode, GError **error)
     }
 
     plot_density = gwy_params_get_boolean(params, PARAM_PLOT_DENSITY);
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     ntodo = args.nincluded-2;
     for (i = 2; i < info.blocksize; i++) {
         if (gwy_params_get_boolean(params, PARAM_INCLUDE_CHANNEL0 + i) && i != skip_err_channel) {
-            create_data_field(container, meta, &args, data, points, i, align_scanlines, plot_density && ntodo == 1);
+            create_field(file, meta, &args, data, points, i, align_scanlines, plot_density && ntodo == 1);
             ntodo--;
             f = 1.0 - (gdouble)ntodo/args.nincluded;
             f = CLAMP(f, 0.0, 1.0);
             if (!gwy_app_wait_set_fraction(f)) {
-                GWY_OBJECT_UNREF(container);
+                g_clear_object(&file);
                 err_CANCELLED(error);
                 goto fail;
             }
@@ -431,13 +424,13 @@ fail:
         g_free(g_object_get_data(G_OBJECT(pardef), "channel-enum"));
         g_object_unref(pardef);
     }
-    GWY_OBJECT_UNREF(meta);
+    g_clear_object(&meta);
     g_free(points);
     free_profile_descriptions(dscs, TRUE);
     if (data)
         gwy_double_array_free(data, TRUE);
 
-    return container;
+    return file;
 }
 
 static GwyDialogOutcome
@@ -450,14 +443,14 @@ run_gui(ModuleArgs *args)
     guint i;
     GString *str;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
 
     gui.dialog = gwy_dialog_new(_("Import NMM Profile Set"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    hbox = gwy_hbox_new(24);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 24);
     gwy_dialog_add_content(dialog, hbox, TRUE, TRUE, 0);
 
     table = gui.table_options = gwy_param_table_new(args->params);
@@ -642,14 +635,14 @@ rotate_points_xy(GwyXYZ *points, guint ndata, gdouble angle)
 }
 
 static void
-create_data_field(GwyContainer *container,
-                  GwyContainer *meta,
-                  ModuleArgs *args,
-                  GwyDoubleArray *data,
-                  GwyXYZ *points,
-                  guint i,
-                  gboolean null_offsets,
-                  gboolean plot_density)
+create_field(GwyFile *file,
+             GwyContainer *meta,
+             ModuleArgs *args,
+             GwyDoubleArray *data,
+             GwyXYZ *points,
+             guint i,
+             gboolean null_offsets,
+             gboolean plot_density)
 {
     GArray *dscs = args->dscs;
     const NMMXYZProfileDesc *dsc = &g_array_index(dscs, NMMXYZProfileDesc, i);
@@ -657,7 +650,7 @@ create_data_field(GwyContainer *container,
     gint xres = gwy_params_get_int(args->params, PARAM_XRES);
     gint yres = gwy_params_get_int(args->params, PARAM_YRES);
     gboolean physically_square = gwy_params_get_boolean(args->params, PARAM_PHYSICALLY_SQUARE);
-    GwyDataField *dfield, *density_map = NULL;
+    GwyField *dfield, *density_map = NULL;
     gsize k, ndata;
     guint nincluded;
     gint id;
@@ -687,14 +680,14 @@ create_data_field(GwyContainer *container,
      * = nprofiles.  We only do the x-axis extension for consistency; it is usually extremely small. */
     dx = (info->xmax - info->xmin)*info->nfiles/info->ndata;
     dy = (info->ymax - info->ymin)/info->nfiles;
-    dfield = gwy_data_field_new(xres, yres, info->xmax - info->xmin + dx, info->ymax - info->ymin + dy, FALSE);
+    dfield = gwy_field_new(xres, yres, info->xmax - info->xmin + dx, info->ymax - info->ymin + dy, FALSE);
 
-    gwy_data_field_set_xoffset(dfield, info->xmin - 0.5*dx);
-    gwy_data_field_set_yoffset(dfield, info->ymin - 0.5*dy);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
+    gwy_field_set_xoffset(dfield, info->xmin - 0.5*dx);
+    gwy_field_set_yoffset(dfield, info->ymin - 0.5*dy);
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
 
     if (plot_density)
-        density_map = gwy_data_field_new_alike(dfield, FALSE);
+        density_map = gwy_field_new_alike(dfield, FALSE);
 
     if (gwy_stramong(dsc->short_name, "Lz", "Az", "Az0", "Az1", "-Lz+Az", "XY vector", NULL))
         zunit = "m";
@@ -702,7 +695,7 @@ create_data_field(GwyContainer *container,
         zunit = "V";
         q = 10.0/65536.0;
     }
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), zunit);
+    gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), zunit);
 
     /* Map i (column in data fields) to index in actually read blocks (which are only nincluded).  Do it by directly
      * moving d to the first value of the requested channel. */
@@ -720,28 +713,28 @@ create_data_field(GwyContainer *container,
     }
 
     /* FIXME: This has gint ndata argument. So no matter how we handle the arrays it will fail for > 2GB points. */
-    gwy_data_field_average_xyz(dfield, density_map, points, ndata);
+    gwy_field_average_xyz(dfield, density_map, points, ndata);
     if (q != 1.0)
-        gwy_data_field_multiply(dfield, q);
+        gwy_field_multiply(dfield, q);
 
-    /* Do it now because gwy_data_field_average_xyz() needs the shifted origin. */
+    /* Do it now because gwy_field_average_xyz() needs the shifted origin. */
     if (null_offsets) {
-        gwy_data_field_set_xoffset(dfield, 0.0);
-        gwy_data_field_set_yoffset(dfield, 0.0);
+        gwy_field_set_xoffset(dfield, 0.0);
+        gwy_field_set_yoffset(dfield, 0.0);
     }
 
     id = i-2;
-    gwy_container_pass_object(container, gwy_app_get_data_key_for_id(id), dfield);
+    gwy_file_pass_image(file, id, dfield);
 
     if (physically_square)
-        gwy_container_set_boolean(container, gwy_app_get_data_real_square_key_for_id(id), TRUE);
+        gwy_container_set_boolean(GWY_CONTAINER(file), gwy_file_key_image_real_square(id), TRUE);
 
-    gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(id), dsc->long_name);
-    gwy_file_channel_import_log_add(container, id, NULL, info->filename);
+    gwy_file_set_title(file, GWY_FILE_IMAGE, id, dsc->long_name, FALSE);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, id, NULL, info->filename);
 
     if (meta) {
-        meta = gwy_container_duplicate(meta);
-        gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(id), meta);
+        meta = gwy_container_copy(meta);
+        gwy_file_pass_meta(file, GWY_FILE_IMAGE, id, meta);
     }
 
     if (!density_map)
@@ -749,14 +742,14 @@ create_data_field(GwyContainer *container,
 
     id++;
 
-    gwy_container_pass_object(container, gwy_app_get_data_key_for_id(id), density_map);
+    gwy_file_pass_image(file, id, density_map);
 
-    gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(id), _("Point density map"));
-    gwy_file_channel_import_log_add(container, id, NULL, info->filename);
+    gwy_file_set_title(file, GWY_FILE_IMAGE, id, _("Point density map"), FALSE);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, id, NULL, info->filename);
 
     if (meta) {
-        meta = gwy_container_duplicate(meta);
-        gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(id), meta);
+        meta = gwy_container_copy(meta);
+        gwy_file_pass_meta(file, GWY_FILE_IMAGE, id, meta);
     }
 }
 
@@ -1029,7 +1022,7 @@ read_profile_description(const gchar *filename, GArray *dscs)
         if (g_strv_length(pieces) != 5)
             goto fail;
 
-        gwy_clear(&dsc, 1);
+        gwy_clear1(dsc);
         dsc.id = atoi(pieces[0]);
         dsc.date = g_strdup(pieces[1]);
         dsc.short_name = g_strdup(pieces[2]);
@@ -1117,7 +1110,7 @@ parse_dsc_file(const gchar *filename)
         return NULL;
     }
 
-    meta = gwy_container_new();
+    meta = gwy_container_new_in_construction();
     str = g_string_new(NULL);
     p = buffer;
     while ((line = gwy_str_next_line(&p))) {
@@ -1154,7 +1147,7 @@ parse_dsc_file(const gchar *filename)
     g_string_free(str, TRUE);
 
     if (!gwy_container_get_n_items(meta))
-        GWY_OBJECT_UNREF(meta);
+        g_clear_object(&meta);
 
     return meta;
 }
@@ -1162,7 +1155,7 @@ parse_dsc_file(const gchar *filename)
 static GwyDoubleArray*
 gwy_double_array_new(gsize nprealloc)
 {
-    GwyDoubleArray *array = g_slice_new0(GwyDoubleArray);
+    GwyDoubleArray *array = g_new0(GwyDoubleArray, 1);
 
     nprealloc = MAX(nprealloc, 16UL);
     array->size = nprealloc;
@@ -1205,7 +1198,7 @@ gwy_double_array_free(GwyDoubleArray *array, gboolean free_data)
         g_free(array->data);
         retval = NULL;
     }
-    g_slice_free(GwyDoubleArray, array);
+    g_free(array);
 
     return retval;
 }
