@@ -1,6 +1,6 @@
 /*
- *  $Id: npyfile.c 26873 2024-11-19 12:52:14Z yeti-dn $
- *  Copyright (C) 2023 David Necas (Yeti).
+ *  $Id: npyfile.c 28822 2025-11-06 16:01:09Z yeti-dn $
+ *  Copyright (C) 2023-2025 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
  *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
@@ -38,16 +38,11 @@
  * employ it). So currently we just implement the simple non-pickled data format here and hope for the best. */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/datafield.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 #include "gwyzip.h"
 #include "get.h"
 #include "err.h"
@@ -80,39 +75,39 @@ typedef struct {
     gsize expected_size;
 } NpyInfo;
 
-static gboolean      module_register      (void);
-static gint          npy_detect           (const GwyFileDetectInfo *fileinfo,
-                                           gboolean only_name);
-static GwyContainer* npy_load             (const gchar *filename,
-                                           GwyRunType mode,
-                                           GError **error);
-static gboolean      npy_export           (GwyContainer *data,
-                                           const gchar *filename,
-                                           GwyRunType mode,
-                                           GError **error);
-static gboolean      load_npy_from_memory (const guchar *buffer,
-                                           gsize size,
-                                           GwyContainer *container,
-                                           gint id,
-                                           const gchar *title,
-                                           const gchar *filename,
-                                           GError **error);
-static gboolean      read_file_header     (NpyInfo *info,
-                                           const guchar **p,
-                                           gsize size,
-                                           GError **error);
-static gboolean      parse_header_dict    (NpyInfo *info,
-                                           GError **error);
-static gboolean      parse_numpy_data_type(const gchar *s,
-                                           GwyRawDataType *rawtype,
-                                           GwyByteOrder *byteorder);
+static gboolean module_register      (void);
+static gint     npy_detect           (const GwyFileDetectInfo *fileinfo,
+                                      gboolean only_name);
+static GwyFile* npy_load             (const gchar *filename,
+                                      GwyRunModeFlags mode,
+                                      GError **error);
+static gboolean npy_export           (GwyFile *data,
+                                      const gchar *filename,
+                                      GwyRunModeFlags mode,
+                                      GError **error);
+static gboolean load_npy_from_memory (const guchar *buffer,
+                                      gsize size,
+                                      GwyFile *file,
+                                      gint id,
+                                      const gchar *title,
+                                      const gchar *filename,
+                                      GError **error);
+static gboolean read_file_header     (NpyInfo *info,
+                                      const guchar **p,
+                                      gsize size,
+                                      GError **error);
+static gboolean parse_header_dict    (NpyInfo *info,
+                                      GError **error);
+static gboolean parse_numpy_data_type(const gchar *s,
+                                      GwyRawDataType *rawtype,
+                                      GwyByteOrder *byteorder);
 
 #ifdef HAVE_GWYZIP
-static gint          npz_detect           (const GwyFileDetectInfo *fileinfo,
-                                           gboolean only_name);
-static GwyContainer* npz_load             (const gchar *filename,
-                                           GwyRunType mode,
-                                           GError **error);
+static gint     npz_detect           (const GwyFileDetectInfo *fileinfo,
+                                      gboolean only_name);
+static GwyFile* npz_load             (const gchar *filename,
+                                      GwyRunModeFlags mode,
+                                      GError **error);
 #endif
 
 static GwyModuleInfo module_info = {
@@ -120,7 +115,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports and exports binary serialized numpy arrays (NPY, NPZ)."),
     "Yeti <yeti@gwyddion.net>",
-    "1.1",
+    "2.0",
     "David Nečas (Yeti)",
     "2023",
 };
@@ -132,25 +127,18 @@ module_register(void)
 {
     gwy_file_func_register("npyfile",
                            N_("Serialized numpy array (.npy)"),
-                           (GwyFileDetectFunc)&npy_detect,
-                           (GwyFileLoadFunc)&npy_load,
-                           NULL,
-                           (GwyFileSaveFunc)&npy_export);
+                           &npy_detect, &npy_load, NULL, &npy_export);
 #ifdef HAVE_GWYZIP
     gwy_file_func_register("npzfile",
                            N_("Multiple serialized numpy arrays (.npz)"),
-                           (GwyFileDetectFunc)&npz_detect,
-                           (GwyFileLoadFunc)&npz_load,
-                           NULL,
-                           NULL);
+                           &npz_detect, &npz_load, NULL, NULL);
 #endif
 
     return TRUE;
 }
 
 static gint
-npy_detect(const GwyFileDetectInfo *fileinfo,
-           gboolean only_name)
+npy_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     if (only_name)
         return g_str_has_suffix(fileinfo->name_lowercase, EXTENSION) ? 20 : 0;
@@ -165,8 +153,7 @@ npy_detect(const GwyFileDetectInfo *fileinfo,
 
 #ifdef HAVE_GWYZIP
 static gint
-npz_detect(const GwyFileDetectInfo *fileinfo,
-           gboolean only_name)
+npz_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     GwyZipFile zipfile;
     gchar *filename;
@@ -205,12 +192,12 @@ npz_detect(const GwyFileDetectInfo *fileinfo,
 }
 #endif
 
-static GwyContainer*
+static GwyFile*
 npy_load(const gchar *filename,
-          G_GNUC_UNUSED GwyRunType mode,
-          GError **error)
+         G_GNUC_UNUSED GwyRunModeFlags mode,
+         GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     guchar *buffer = NULL;
     GError *err = NULL;
     gsize size;
@@ -220,23 +207,23 @@ npy_load(const gchar *filename,
         return NULL;
     }
 
-    container = gwy_container_new();
-    if (!load_npy_from_memory(buffer, size, container, 0, NULL, filename, error))
-        GWY_OBJECT_UNREF(container);
+    file = gwy_file_new_in_construction();
+    if (!load_npy_from_memory(buffer, size, file, 0, NULL, filename, error))
+        g_clear_object(&file);
 
     gwy_file_abandon_contents(buffer, size, NULL);
 
-    return container;
+    return file;
 }
 
 #ifdef HAVE_GWYZIP
-static GwyContainer*
+static GwyFile*
 npz_load(const gchar *filename,
-         G_GNUC_UNUSED GwyRunType mode,
+         G_GNUC_UNUSED GwyRunModeFlags mode,
          GError **error)
 {
     GwyZipFile zipfile;
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     guchar *content;
     gchar *title, *dot;
     gsize size;
@@ -247,7 +234,7 @@ npz_load(const gchar *filename,
     if (!gwyzip_first_file(zipfile, error))
         goto fail;
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     do {
         if (!(content = gwyzip_get_file_content(zipfile, &size, error)))
             goto fail;
@@ -258,38 +245,38 @@ npz_load(const gchar *filename,
         gwy_debug("trying to load %s", title);
         if ((dot = strrchr(title, '.')))
             *dot = '\0';
-        if (load_npy_from_memory(content, size, container, id, *title ? title : NULL, filename, NULL))
+        if (load_npy_from_memory(content, size, file, id, *title ? title : NULL, filename, NULL))
             id++;
         g_free(content);
         g_free(title);
     } while (gwyzip_next_file(zipfile, NULL));
 
     if (!id) {
-        GWY_OBJECT_UNREF(container);
+        g_clear_object(&file);
         err_NO_DATA(error);
     }
 
 fail:
     gwyzip_close(zipfile);
 
-    return container;
+    return file;
 }
 #endif
 
 static gboolean
 load_npy_from_memory(const guchar *buffer, gsize size,
-                     GwyContainer *container, gint id, const gchar *title, const gchar *filename,
+                     GwyFile *file, gint id, const gchar *title, const gchar *filename,
                      GError **error)
 {
     NpyInfo info;
     GwyRawDataType rawtype;
     GwyByteOrder byteorder;
-    GwyDataField *field;
+    GwyField *field;
     const guchar *p;
     gint xres, yres, header_size;
     gboolean imported_data = FALSE;
 
-    gwy_clear(&info, 1);
+    gwy_clear1(info);
     p = buffer;
     if (!read_file_header(&info, &p, size, error))
         goto fail;
@@ -315,12 +302,12 @@ load_npy_from_memory(const guchar *buffer, gsize size,
             yres = info.dims[0];
         }
 
-        field = gwy_data_field_new(xres, yres, xres, yres, FALSE);
-        gwy_convert_raw_data(p, xres*yres, 1, rawtype, byteorder, gwy_data_field_get_data(field), 1.0, 0.0);
-        gwy_container_pass_object(container, gwy_app_get_data_key_for_id(id), field);
-        gwy_file_channel_import_log_add(container, id, NULL, filename);
+        field = gwy_field_new(xres, yres, xres, yres, FALSE);
+        gwy_convert_raw_data(p, xres*yres, 1, rawtype, byteorder, gwy_field_get_data(field), 1.0, 0.0);
+        gwy_file_pass_image(file, id, field);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, id, NULL, filename);
         if (title)
-            gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(id), title);
+            gwy_file_set_title(file, GWY_FILE_IMAGE, id, title, FALSE);
         imported_data = TRUE;
     }
 
@@ -556,12 +543,12 @@ fail:
 }
 
 static gboolean
-npy_export(G_GNUC_UNUSED GwyContainer *data,
+npy_export(G_GNUC_UNUSED GwyFile *data,
            const gchar *filename,
-           G_GNUC_UNUSED GwyRunType mode,
+           G_GNUC_UNUSED GwyRunModeFlags mode,
            GError **error)
 {
-    GwyDataField *field;
+    GwyField *field;
     guchar file_header[HEADER_SIZE_v1_0 + 1] = MAGIC "\x01\x00..";  /* Version 1.0 is good enough for us. */
     GString *dictstr;
     gint xres, yres, n;
@@ -569,16 +556,16 @@ npy_export(G_GNUC_UNUSED GwyContainer *data,
     guint header_len;
     FILE *fh;
 
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &field, 0);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &field, 0);
 
     if (!field) {
         err_NO_CHANNEL_EXPORT(error);
         return FALSE;
     }
 
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
-    d = gwy_data_field_get_data_const(field);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
+    d = gwy_field_get_data_const(field);
     n = xres*yres;
 
     dictstr = g_string_new(NULL);
