@@ -1,6 +1,6 @@
 /*
- *  $Id: nrrdfile.c 25989 2023-10-31 17:27:00Z yeti-dn $
- *  Copyright (C) 2011-2019 David Necas (Yeti).
+ *  $Id: nrrdfile.c 28822 2025-11-06 16:01:09Z yeti-dn $
+ *  Copyright (C) 2011-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -47,25 +47,17 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
 #include <stdarg.h>
-
-#ifdef HAVE_ZLIB
 #include <zlib.h>
-#endif
 
 #ifdef HAVE_BZIP2
 #include <bzlib.h>
 #endif
 
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <libprocess/datafield.h>
-#include <app/data-browser.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 
 #include "err.h"
 #include "get.h"
@@ -102,14 +94,14 @@ typedef enum {
 } NRRDEncoding;
 
 static gboolean      module_register     (void);
-static gint          nrrdfile_detect     (const GwyFileDetectInfo *fileinfo,
+static gint          detect_file         (const GwyFileDetectInfo *fileinfo,
                                           gboolean only_name);
-static GwyContainer* nrrdfile_load       (const gchar *filename,
-                                          GwyRunType mode,
+static GwyFile*      load_file           (const gchar *filename,
+                                          GwyRunModeFlags mode,
                                           GError **error);
-static gboolean      nrrdfile_export     (GwyContainer *data,
+static gboolean      export_file         (GwyFile *data,
                                           const gchar *filename,
-                                          GwyRunType mode,
+                                          GwyRunModeFlags mode,
                                           GError **error);
 static void          normalise_field_name(gchar *name);
 static void          unescape_field_value(gchar *value);
@@ -139,7 +131,7 @@ static gconstpointer get_raw_data_pointer(gconstpointer base,
                                           gssize byteskip,
                                           GSList **buffers_to_free,
                                           GError **error);
-static GwyDataField* read_raw_data_field (guint xres,
+static GwyField*     read_raw_field      (guint xres,
                                           guint yres,
                                           gint stride,
                                           guint rowstride,
@@ -175,7 +167,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports and exports nearly raw raster data (NRRD) files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.9",
+    "0.10",
     "David Nečas (Yeti)",
     "2011",
 };
@@ -187,17 +179,13 @@ module_register(void)
 {
     gwy_file_func_register("nrrdfile",
                            N_("Nearly raw raster data (NRRD) files (.nrrd)"),
-                           (GwyFileDetectFunc)&nrrdfile_detect,
-                           (GwyFileLoadFunc)&nrrdfile_load,
-                           NULL,
-                           (GwyFileSaveFunc)&nrrdfile_export);
+                           detect_file, load_file, NULL, export_file);
 
     return TRUE;
 }
 
 static gint
-nrrdfile_detect(const GwyFileDetectInfo *fileinfo,
-                gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     if (only_name)
         return g_str_has_suffix(fileinfo->name_lowercase, EXTENSION) ? 20 : 0;
@@ -219,14 +207,15 @@ nrrd_encoding_is_compressed(NRRDEncoding encoding)
     return encoding == NRRD_ENCODING_GZIP || encoding == NRRD_ENCODING_BZIP2;
 }
 
-static GwyContainer*
-nrrdfile_load(const gchar *filename,
-              G_GNUC_UNUSED GwyRunType mode,
-              GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *meta = NULL, *container = NULL;
+    GwyFile *file = NULL;
+    GwyContainer *meta = NULL;
     GHashTable *hash, *fields = NULL, *keyvalue = NULL;
-    GwyDataField *dfield = NULL;
+    GwyField *dfield = NULL;
     GSList *l, *buffers_to_free = NULL;
     guchar *buffer = NULL, *data_buffer = NULL, *header_end;
     const guchar *data_start;
@@ -451,39 +440,38 @@ nrrdfile_load(const gchar *filename,
                                             &buffers_to_free, error)))
         goto fail;
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     meta = nrrd_make_meta(keyvalue);
 
     if (chanaxis == G_MAXUINT) {
         GwyBrick *brick = read_raw_brick(xres, yres, zres, rawdatatype, byteorder, fields, data_start);
 
-        gwy_container_pass_object(container, gwy_app_get_brick_key_for_id(0), brick);
+        gwy_file_pass_volume(file, 0, brick);
 
         if ((value = g_hash_table_lookup(fields, "content")))
-            gwy_container_set_const_string(container, gwy_app_get_brick_title_key_for_id(0), value);
-        if (meta)
-            gwy_container_set_object(container, gwy_app_get_brick_meta_key_for_id(0), meta);
+            gwy_file_set_title(file, GWY_FILE_VOLUME, 0, value, FALSE);
+        if (meta) {
+            gwy_file_pass_meta(file, GWY_FILE_VOLUME, 0, meta);
+            meta = NULL;
+        }
     }
     else {
         for (i = 0; i < nchannels; i++) {
             const guchar *chandata = data_start + i*fieldstride*gwy_raw_data_size(rawdatatype);
 
-            dfield = read_raw_data_field(xres, yres, stride, rowstride, rawdatatype, byteorder, fields, chandata);
-            gwy_container_pass_object(container, gwy_app_get_data_key_for_id(i), dfield);
+            dfield = read_raw_field(xres, yres, stride, rowstride, rawdatatype, byteorder, fields, chandata);
+            gwy_file_pass_image(file, i, dfield);
 
             if ((value = g_hash_table_lookup(fields, "content")))
-                gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(i), value);
-            if (meta) {
-                GwyContainer *chanmeta = gwy_container_duplicate(meta);
-
-                gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(i), chanmeta);
-            }
-            gwy_file_channel_import_log_add(container, i, NULL, filename);
+                gwy_file_set_title(file, GWY_FILE_IMAGE, i, value, FALSE);
+            if (meta)
+                gwy_file_pass_meta(file, GWY_FILE_IMAGE, i, gwy_container_copy(meta));
+            gwy_log_add_import(file, GWY_FILE_IMAGE, i, NULL, filename);
         }
     }
 
 fail:
-    GWY_OBJECT_UNREF(meta);
+    g_clear_object(&meta);
     for (l = buffers_to_free; l; l = g_slist_next(l))
         g_free(l->data);
     g_slist_free(buffers_to_free);
@@ -493,7 +481,7 @@ fail:
     if (keyvalue)
         g_hash_table_destroy(keyvalue);
 
-    return container;
+    return file;
 }
 
 static void
@@ -696,7 +684,6 @@ load_detached_file(const gchar *datafile,
     }
 
     if (gzcompressed) {
-#ifdef HAVE_ZLIB
         gzFile cfile = gzopen(datafile, "rb");
         gsize prevsize, toread;
         gssize readbytes;
@@ -732,11 +719,6 @@ load_detached_file(const gchar *datafile,
         gzclose(cfile);
         *size = prevsize + readbytes;
         gwy_debug("decompressed size: %lu", (gulong)*size);
-#else
-        g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_SPECIFIC,
-                    /* TRANSLATORS: %s is replaced by a software library name like zlib. */
-                    _("Cannot decompress compressed data.  Gwyddion was built without %s support."), "zlib");
-#endif
     }
     else if (bz2compressed) {
 #ifdef HAVE_BZIP2
@@ -1159,15 +1141,15 @@ get_raw_data_pointer(gconstpointer base,
     return base;
 }
 
-static GwyDataField*
-read_raw_data_field(guint xres, guint yres,
-                    gint stride, guint rowstride,
-                    GwyRawDataType rawdatatype, GwyByteOrder byteorder,
-                    GHashTable *fields,
-                    const guchar *data)
+static GwyField*
+read_raw_field(guint xres, guint yres,
+               gint stride, guint rowstride,
+               GwyRawDataType rawdatatype, GwyByteOrder byteorder,
+               GHashTable *fields,
+               const guchar *data)
 {
-    GwyDataField *dfield;
-    GwySIUnit *siunitz = NULL, *siunitxy = NULL;
+    GwyField *dfield;
+    GwyUnit *siunitz = NULL, *siunitxy = NULL;
     gdouble dx = 1.0, dy = 1.0, q = 1.0, z0 = 0.0, xoff = 0.0, yoff = 0.0;
     gdouble *d;
     gint power10;
@@ -1202,9 +1184,9 @@ read_raw_data_field(guint xres, guint yres,
     if ((value = g_hash_table_lookup(fields, "sampleunits"))) {
         gchar *unitz;
         if (parse_string_vector(value, 1, &unitz)) {
-            siunitz = gwy_si_unit_new_parse(unitz, &power10);
-            q *= pow10(power10);
-            z0 *= pow10(power10);
+            siunitz = gwy_unit_new_parse(unitz, &power10);
+            q *= gwy_exp10(power10);
+            z0 *= gwy_exp10(power10);
             g_free(unitz);
         }
     }
@@ -1214,28 +1196,28 @@ read_raw_data_field(guint xres, guint yres,
         if (parse_string_vector(value, 2, &unitx, &unity)) {
             if (!gwy_strequal(unitx, unity))
                 g_warning("X and Y units differ, using X");
-            siunitxy = gwy_si_unit_new_parse(unitx, &power10);
-            dx *= pow10(power10);
-            dy *= pow10(power10);
+            siunitxy = gwy_unit_new_parse(unitx, &power10);
+            dx *= gwy_exp10(power10);
+            dy *= gwy_exp10(power10);
             g_free(unitx);
             g_free(unity);
         }
     }
 
     rowstride *= gwy_raw_data_size(rawdatatype);
-    dfield = gwy_data_field_new(xres, yres, xres*dx, yres*dy, FALSE);
-    gwy_data_field_set_xoffset(dfield, xoff);
-    gwy_data_field_set_yoffset(dfield, yoff);
-    d = gwy_data_field_get_data(dfield);
+    dfield = gwy_field_new(xres, yres, xres*dx, yres*dy, FALSE);
+    gwy_field_set_xoffset(dfield, xoff);
+    gwy_field_set_yoffset(dfield, yoff);
+    d = gwy_field_get_data(dfield);
     for (i = 0; i < yres; i++)
         gwy_convert_raw_data(data + i*rowstride, xres, stride, rawdatatype, byteorder, d + i*xres, q, z0);
 
     if (siunitxy) {
-        gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(dfield), siunitxy);
+        gwy_unit_assign(gwy_field_get_unit_xy(dfield), siunitxy);
         g_object_unref(siunitxy);
     }
     if (siunitz) {
-        gwy_si_unit_assign(gwy_data_field_get_si_unit_z(dfield), siunitz);
+        gwy_unit_assign(gwy_field_get_unit_z(dfield), siunitz);
         g_object_unref(siunitz);
     }
 
@@ -1249,7 +1231,7 @@ read_raw_brick(guint xres, guint yres, guint zres,
                const guchar *data)
 {
     GwyBrick *brick;
-    GwySIUnit *siunitz = NULL, *siunitx = NULL, *siunity = NULL, *siunitw = NULL;
+    GwyUnit *siunitz = NULL, *siunitx = NULL, *siunity = NULL, *siunitw = NULL;
     gdouble dx = 1.0, dy = 1.0, dz = 1.0, q = 1.0, w0 = 0.0, xoff = 0.0, yoff = 0.0, zoff = 0.0;
     gdouble *d;
     gint power10;
@@ -1284,9 +1266,9 @@ read_raw_brick(guint xres, guint yres, guint zres,
     if ((value = g_hash_table_lookup(fields, "sampleunits"))) {
         gchar *unitw;
         if (parse_string_vector(value, 1, &unitw)) {
-            siunitw = gwy_si_unit_new_parse(unitw, &power10);
-            q *= pow10(power10);
-            w0 *= pow10(power10);
+            siunitw = gwy_unit_new_parse(unitw, &power10);
+            q *= gwy_exp10(power10);
+            w0 *= gwy_exp10(power10);
             g_free(unitw);
         }
     }
@@ -1294,12 +1276,12 @@ read_raw_brick(guint xres, guint yres, guint zres,
     if ((value = g_hash_table_lookup(fields, "units"))) {
         gchar *unitx, *unity, *unitz;
         if (parse_string_vector(value, 3, &unitx, &unity, &unitz)) {
-            siunitx = gwy_si_unit_new_parse(unitx, &power10);
-            dx *= pow10(power10);
-            siunity = gwy_si_unit_new_parse(unity, &power10);
-            dy *= pow10(power10);
-            siunitz = gwy_si_unit_new_parse(unitz, &power10);
-            dz *= pow10(power10);
+            siunitx = gwy_unit_new_parse(unitx, &power10);
+            dx *= gwy_exp10(power10);
+            siunity = gwy_unit_new_parse(unity, &power10);
+            dy *= gwy_exp10(power10);
+            siunitz = gwy_unit_new_parse(unitz, &power10);
+            dz *= gwy_exp10(power10);
             g_free(unitx);
             g_free(unity);
             g_free(unitz);
@@ -1314,19 +1296,19 @@ read_raw_brick(guint xres, guint yres, guint zres,
     gwy_convert_raw_data(data, xres*yres*zres, 1, rawdatatype, byteorder, d, q, w0);
 
     if (siunitx) {
-        gwy_si_unit_assign(gwy_brick_get_si_unit_x(brick), siunitx);
+        gwy_unit_assign(gwy_brick_get_unit_x(brick), siunitx);
         g_object_unref(siunitx);
     }
     if (siunity) {
-        gwy_si_unit_assign(gwy_brick_get_si_unit_y(brick), siunity);
+        gwy_unit_assign(gwy_brick_get_unit_y(brick), siunity);
         g_object_unref(siunity);
     }
     if (siunitz) {
-        gwy_si_unit_assign(gwy_brick_get_si_unit_z(brick), siunitz);
+        gwy_unit_assign(gwy_brick_get_unit_z(brick), siunitz);
         g_object_unref(siunitz);
     }
     if (siunitw) {
-        gwy_si_unit_assign(gwy_brick_get_si_unit_w(brick), siunitw);
+        gwy_unit_assign(gwy_brick_get_unit_w(brick), siunitw);
         g_object_unref(siunitw);
     }
 
@@ -1346,7 +1328,7 @@ add_meta(gpointer key, gpointer value, gpointer user_data)
 static GwyContainer*
 nrrd_make_meta(GHashTable *keyvalue)
 {
-    GwyContainer *meta = gwy_container_new();
+    GwyContainer *meta = gwy_container_new_in_construction();
 
     g_hash_table_foreach(keyvalue, add_meta, meta);
     if (gwy_container_get_n_items(meta))
@@ -1538,10 +1520,10 @@ fail:
 }
 
 static gboolean
-nrrdfile_export(G_GNUC_UNUSED GwyContainer *data,
-                const gchar *filename,
-                G_GNUC_UNUSED GwyRunType mode,
-                GError **error)
+export_file(G_GNUC_UNUSED GwyFile *data,
+            const gchar *filename,
+            G_GNUC_UNUSED GwyRunModeFlags mode,
+            GError **error)
 {
     /* We specify lateral units so at least version 4 is necessary. */
     static const gchar header_format[] =
@@ -1558,7 +1540,7 @@ nrrdfile_export(G_GNUC_UNUSED GwyContainer *data,
         "kinds: space space\n"
         "\n";
 
-    GwyDataField *dfield;
+    GwyField *dfield;
     const gdouble *d;
     gdouble xreal, yreal, xoff, yoff;
     guint xres, yres, i;
@@ -1568,7 +1550,7 @@ nrrdfile_export(G_GNUC_UNUSED GwyContainer *data,
     gboolean ok = TRUE;
     FILE *fh;
 
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &dfield, 0);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &dfield, 0);
     if (!dfield) {
         err_NO_CHANNEL_EXPORT(error);
         return FALSE;
@@ -1580,15 +1562,15 @@ nrrdfile_export(G_GNUC_UNUSED GwyContainer *data,
         return FALSE;
     }
 
-    d = gwy_data_field_get_data_const(dfield);
-    xres = gwy_data_field_get_xres(dfield);
-    yres = gwy_data_field_get_yres(dfield);
-    xreal = gwy_data_field_get_xreal(dfield);
-    yreal = gwy_data_field_get_yreal(dfield);
-    xoff = gwy_data_field_get_xoffset(dfield);
-    yoff = gwy_data_field_get_yoffset(dfield);
-    unitxy = gwy_si_unit_get_string(gwy_data_field_get_si_unit_xy(dfield), GWY_SI_UNIT_FORMAT_PLAIN);
-    unitz = gwy_si_unit_get_string(gwy_data_field_get_si_unit_z(dfield), GWY_SI_UNIT_FORMAT_PLAIN);
+    d = gwy_field_get_data_const(dfield);
+    xres = gwy_field_get_xres(dfield);
+    yres = gwy_field_get_yres(dfield);
+    xreal = gwy_field_get_xreal(dfield);
+    yreal = gwy_field_get_yreal(dfield);
+    xoff = gwy_field_get_xoffset(dfield);
+    yoff = gwy_field_get_yoffset(dfield);
+    unitxy = gwy_unit_get_string(gwy_field_get_unit_xy(dfield), GWY_UNIT_FORMAT_PLAIN);
+    unitz = gwy_unit_get_string(gwy_field_get_unit_z(dfield), GWY_UNIT_FORMAT_PLAIN);
 
     g_ascii_formatd(buf[0], sizeof(buf[0]), "%.8g", xoff);
     g_ascii_formatd(buf[1], sizeof(buf[1]), "%.8g", yoff);
