@@ -1,6 +1,6 @@
 /*
- *  $Id: nxiifile.c 25734 2023-09-29 09:38:34Z yeti-dn $
- *  Copyright (C) 2014 David Necas (Yeti).
+ *  $Id: nxiifile.c 28789 2025-11-04 17:14:03Z yeti-dn $
+ *  Copyright (C) 2014-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -28,15 +28,9 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/datafield.h>
-#include <libprocess/stats.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 
 #include "err.h"
 #include "get.h"
@@ -96,10 +90,10 @@ typedef struct {
 } NXIIFile;
 
 static gboolean      module_register (void);
-static gint          nxii_detect     (const GwyFileDetectInfo *fileinfo,
+static gint          detect_file     (const GwyFileDetectInfo *fileinfo,
                                       gboolean only_name);
-static GwyContainer* nxii_load       (const gchar *filename,
-                                      GwyRunType mode,
+static GwyFile*      load_file       (const gchar *filename,
+                                      GwyRunModeFlags mode,
                                       GError **error);
 static GwyContainer* get_meta        (NXIIFile *nxiifile);
 static gboolean      read_nxii_header(const guchar *p,
@@ -115,7 +109,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports EM4SYS data files."),
     "Yeti <yeti@gwyddion.net>",
-    "1.0",
+    "2.0",
     "David Nečas (Yeti)",
     "2014",
 };
@@ -127,17 +121,13 @@ module_register(void)
 {
     gwy_file_func_register("nxiifile",
                            N_("EM4SYS NX II files (.bmp)"),
-                           (GwyFileDetectFunc)&nxii_detect,
-                           (GwyFileLoadFunc)&nxii_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-nxii_detect(const GwyFileDetectInfo *fileinfo,
-           gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     guint xres, yres, size;
 
@@ -160,22 +150,22 @@ nxii_detect(const GwyFileDetectInfo *fileinfo,
     return 0;
 }
 
-static GwyContainer*
-nxii_load(const gchar *filename,
-         G_GNUC_UNUSED GwyRunType mode,
-         GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     guchar *buffer = NULL;
     const guchar *p;
     char *s;
     NXIIFile nxiifile;
-    GwySIUnit *unitz;
+    GwyUnit *unitz;
     guint xres, yres, bmpfilesize, expected_size;
     gdouble min, max;
     gsize size = 0;
     GError *err = NULL;
-    GwyDataField *dfield;
+    GwyField *dfield;
 
     if (!gwy_file_get_contents(filename, &buffer, &size, &err)) {
         err_GET_FILE_CONTENTS(error, &err);
@@ -200,48 +190,48 @@ nxii_load(const gchar *filename,
     if (err_SIZE_MISMATCH(error, expected_size, size, TRUE))
         goto fail;
 
-    dfield = gwy_data_field_new(nxiifile.xres, nxiifile.yres, nxiifile.xreal*Micron, nxiifile.yreal*Micron, FALSE);
-    gwy_data_field_set_xoffset(dfield, nxiifile.xoff*Micron);
-    gwy_data_field_set_yoffset(dfield, nxiifile.yoff*Micron);
+    dfield = gwy_field_new(nxiifile.xres, nxiifile.yres, nxiifile.xreal*Micron, nxiifile.yreal*Micron, FALSE);
+    gwy_field_set_xoffset(dfield, nxiifile.xoff*Micron);
+    gwy_field_set_yoffset(dfield, nxiifile.yoff*Micron);
     gwy_convert_raw_data(buffer + bmpfilesize + HEADER_SIZE, nxiifile.xres*nxiifile.yres, 1,
                          GWY_RAW_DATA_UINT16, GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                         gwy_data_field_get_data(dfield), 1.0, 0.0);
-    gwy_data_field_get_min_max(dfield, &min, &max);
+                         gwy_field_get_data(dfield), 1.0, 0.0);
+    gwy_field_get_min_max(dfield, &min, &max);
     if (min >= max)
-        gwy_data_field_clear(dfield);
+        gwy_field_clear(dfield);
     else if (nxiifile.mode == NXII_MODE_FFM || nxiifile.mode == NXII_MODE_FMM)
-        gwy_data_field_multiply(dfield, nxiifile.zreal*Nano/(max - min));
+        gwy_field_multiply(dfield, nxiifile.zreal*Nano/(max - min));
     else if (nxiifile.mode == NXII_MODE_LFM) {
         /* LFM can contain negative values.  This should be the correct formula.  No warranty. */
         gdouble k3 = 32767.0 - min, k5 = max - min;
         gdouble zoff = -GWY_ROUND(65536.0*k3/k5)/65536.0*k5;
-        gwy_data_field_add(dfield, zoff);
-        gwy_data_field_multiply(dfield, nxiifile.zreal*Nano/k5);
+        gwy_field_add(dfield, zoff);
+        gwy_field_multiply(dfield, nxiifile.zreal*Nano/k5);
     }
     else
-        gwy_data_field_multiply(dfield, nxiifile.zreal*Micron/(max - min));
+        gwy_field_multiply(dfield, nxiifile.zreal*Micron/(max - min));
 
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
-    unitz = gwy_data_field_get_si_unit_z(dfield);
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
+    unitz = gwy_field_get_unit_z(dfield);
     if (nxiifile.mode == NXII_MODE_LFM || nxiifile.mode == NXII_MODE_FFM)
-        gwy_si_unit_set_from_string(unitz, "N");
+        gwy_unit_set_from_string(unitz, "N");
     else if (nxiifile.mode == NXII_MODE_FMM)
-        gwy_si_unit_set_from_string(unitz, "N/m");
+        gwy_unit_set_from_string(unitz, "N/m");
     else
-        gwy_si_unit_set_from_string(unitz, "m");
+        gwy_unit_set_from_string(unitz, "m");
 
-    container = gwy_container_new();
-    gwy_container_pass_object(container, gwy_app_get_data_key_for_id(0), dfield);
+    file = gwy_file_new_in_construction();
+    gwy_file_pass_image(file, 0, dfield);
 
     s = g_strstrip(g_strndup(nxiifile.head_mode, 6));
-    gwy_container_set_string(container, gwy_app_get_data_title_key_for_id(0), s);
-    gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(0), get_meta(&nxiifile));
-    gwy_file_channel_import_log_add(container, 0, NULL, filename);
+    gwy_file_pass_title(file, GWY_FILE_IMAGE, 0, s);
+    gwy_file_pass_meta(file, GWY_FILE_IMAGE, 0, get_meta(&nxiifile));
+    gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
 
 fail:
     gwy_file_abandon_contents(buffer, size, NULL);
 
-    return container;
+    return file;
 }
 
 #define add_meta_str(name, field) \
@@ -253,7 +243,7 @@ fail:
 static GwyContainer*
 get_meta(NXIIFile *nxiifile)
 {
-    GwyContainer *meta = gwy_container_new();
+    GwyContainer *meta = gwy_container_new_in_construction();
 
     add_meta_str("Version", file_version);
     add_meta_str("Head mode", head_mode);
