@@ -1,6 +1,6 @@
 /*
- *  $Id: oirfile.c 26753 2024-10-18 15:57:39Z yeti-dn $
- *  Copyright (C) 2019-2023 David Necas (Yeti).
+ *  $Id: oirfile.c 28789 2025-11-04 17:14:03Z yeti-dn $
+ *  Copyright (C) 2019-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -51,16 +51,11 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
 #include <errno.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/datafield.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/data-browser.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 #include "err.h"
 #include "get.h"
 #include "gwyzip.h"
@@ -180,67 +175,67 @@ typedef struct {
     OIRXMLParserData xmldata;
 } OIRFile;
 
-static gboolean      module_register         (void);
-static gint          oirfile_detect          (const GwyFileDetectInfo *fileinfo,
-                                              gboolean only_name);
-static GwyContainer* oirfile_load            (const gchar *filename,
-                                              GwyRunType mode,
-                                              GError **error);
+static gboolean module_register(void);
+static gint     oirfile_detect (const GwyFileDetectInfo *fileinfo,
+                                gboolean only_name);
+static GwyFile* oirfile_load   (const gchar *filename,
+                                GwyRunModeFlags mode,
+                                GError **error);
 #ifdef HAVE_GWYZIP
-static GwyContainer* poirfile_load           (const gchar *filename,
-                                              GwyRunType mode,
-                                              GError **error);
-static gint          poirfile_detect         (const GwyFileDetectInfo *fileinfo,
-                                              gboolean only_name);
+static GwyFile* poirfile_load           (const gchar *filename,
+                                         GwyRunModeFlags mode,
+                                         GError **error);
+static gint     poirfile_detect         (const GwyFileDetectInfo *fileinfo,
+                                         gboolean only_name);
 #endif
-static gint          oirfile_load_from_memory(GwyContainer *container,
-                                              gint channelno,
-                                              const guchar *buffer,
-                                              gsize size,
-                                              const gchar *filename,
-                                              GError **error);
-static void          oirfile_free            (OIRFile *oirfile);
-static gboolean      read_file_header        (const guchar **p,
-                                              const guchar *end,
-                                              OIRFileHeader *header,
-                                              GError **error);
-static gboolean      read_image_data_block   (const guchar **p,
-                                              const guchar *end,
-                                              OIRImageDataBlock *image,
-                                              const gchar *id,
-                                              GError **error);
-static gboolean      read_image_meta_data    (const guchar **p,
-                                              const guchar *end,
-                                              OIRImageXMLFragment *fragment,
-                                              GError **error);
-static gboolean      read_more_meta_data     (const guchar **p,
-                                              const guchar *end,
-                                              OIRMetaData *metadata,
-                                              GError **error);
-static void          parse_xml_to_hash       (OIRXMLFragment *xml,
-                                              OIRXMLParserData *data);
-static gboolean      chunk_size_and_type     (const guchar *p,
-                                              const guchar *end,
-                                              guint32 *chunksize,
-                                              OIRChunkType *chunktype,
-                                              GError **error);
-static gboolean      skip_thumbnail          (const guchar **p,
-                                              const guchar *end,
-                                              GError **error);
-static gboolean      create_datafield        (GwyContainer *data,
-                                              GHashTable *hash,
-                                              guint imgid,
-                                              gint i,
-                                              const OIRImageDataBlock *image,
-                                              const gchar *filename,
-                                              GError **error);
+static gint     oirfile_load_from_memory(GwyFile *file,
+                                         gint channelno,
+                                         const guchar *buffer,
+                                         gsize size,
+                                         const gchar *filename,
+                                         GError **error);
+static void     oirfile_free            (OIRFile *oirfile);
+static gboolean read_file_header        (const guchar **p,
+                                         const guchar *end,
+                                         OIRFileHeader *header,
+                                         GError **error);
+static gboolean read_image_data_block   (const guchar **p,
+                                         const guchar *end,
+                                         OIRImageDataBlock *image,
+                                         const gchar *id,
+                                         GError **error);
+static gboolean read_image_meta_data    (const guchar **p,
+                                         const guchar *end,
+                                         OIRImageXMLFragment *fragment,
+                                         GError **error);
+static gboolean read_more_meta_data     (const guchar **p,
+                                         const guchar *end,
+                                         OIRMetaData *metadata,
+                                         GError **error);
+static void     parse_xml_to_hash       (OIRXMLFragment *xml,
+                                         OIRXMLParserData *data);
+static gboolean chunk_size_and_type     (const guchar *p,
+                                         const guchar *end,
+                                         guint32 *chunksize,
+                                         OIRChunkType *chunktype,
+                                         GError **error);
+static gboolean skip_thumbnail          (const guchar **p,
+                                         const guchar *end,
+                                         GError **error);
+static gboolean create_field            (GwyFile *data,
+                                         GHashTable *hash,
+                                         guint imgid,
+                                         gint i,
+                                         const OIRImageDataBlock *image,
+                                         const gchar *filename,
+                                         GError **error);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Olympus OIR data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.8",
+    "0.9",
     "David Nečas (Yeti)",
     "2019",
 };
@@ -256,25 +251,18 @@ module_register(void)
 {
     gwy_file_func_register("oirfile",
                            N_("Olympus OIR data files (.oir)"),
-                           (GwyFileDetectFunc)&oirfile_detect,
-                           (GwyFileLoadFunc)&oirfile_load,
-                           NULL,
-                           NULL);
+                           &oirfile_detect, &oirfile_load, NULL, NULL);
 #ifdef HAVE_GWYZIP
     gwy_file_func_register("poirfile",
                            N_("Olympus packed OIR data files (.poir)"),
-                           (GwyFileDetectFunc)&poirfile_detect,
-                           (GwyFileLoadFunc)&poirfile_load,
-                           NULL,
-                           NULL);
+                           &poirfile_detect, &poirfile_load, NULL, NULL);
 #endif
 
     return TRUE;
 }
 
 static gint
-oirfile_detect(const GwyFileDetectInfo *fileinfo,
-               gboolean only_name)
+oirfile_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     const gchar *head = fileinfo->head;
 
@@ -288,29 +276,27 @@ oirfile_detect(const GwyFileDetectInfo *fileinfo,
     return 100;
 }
 
-static GwyContainer*
+static GwyFile*
 oirfile_load(const gchar *filename,
-             G_GNUC_UNUSED GwyRunType mode,
+             G_GNUC_UNUSED GwyRunModeFlags mode,
              GError **error)
 {
-    GwyContainer *container = NULL;
     guchar *buffer = NULL;
     GError *err = NULL;
     gsize size = 0;
-    gint ndata;
 
     if (!gwy_file_get_contents(filename, &buffer, &size, error)) {
         err_GET_FILE_CONTENTS(error, &err);
         return NULL;
     }
 
-    container = gwy_container_new();
-    ndata = oirfile_load_from_memory(container, 0, buffer, size, filename, error);
+    GwyFile *file = gwy_file_new_in_construction();
+    gint ndata = oirfile_load_from_memory(file, 0, buffer, size, filename, error);
     gwy_file_abandon_contents(buffer, size, NULL);
     if (!ndata)
-        GWY_OBJECT_UNREF(container);
+        g_clear_object(&file);
 
-    return container;
+    return file;
 }
 
 #ifdef HAVE_GWYZIP
@@ -318,6 +304,12 @@ static gint
 poirfile_detect(const GwyFileDetectInfo *fileinfo,
                 gboolean only_name)
 {
+    static const GwyEnum magic_strings[] = {
+        { "^3D_LSM",    sizeof("^3D_LSM")-1, },
+        { "_LSM3D^",    sizeof("^3D_LSM")-1, },
+        { "_COLOR3D^",  sizeof("^3D_LSM")-1, },
+        { "^XY_Camera", sizeof("^3D_LSM")-1, },
+    };
     const gchar *head = fileinfo->head;
     guint count = 0;
 
@@ -333,14 +325,10 @@ poirfile_detect(const GwyFileDetectInfo *fileinfo,
         && !gwy_memmem(fileinfo->tail, fileinfo->buffer_len, ".oir", 4))
         return 0;
 
-    count += (gwy_memmem(fileinfo->head, fileinfo->buffer_len, "^3D_LSM", 7)
-              || gwy_memmem(fileinfo->tail, fileinfo->buffer_len, "^3D_LSM", 7));
-    count += (gwy_memmem(fileinfo->head, fileinfo->buffer_len, "_LSM3D^", 7)
-              || gwy_memmem(fileinfo->tail, fileinfo->buffer_len, "_LSM3D^", 7));
-    count += (gwy_memmem(fileinfo->head, fileinfo->buffer_len, "_COLOR3D^", 9)
-              || gwy_memmem(fileinfo->tail, fileinfo->buffer_len, "_COLOR3D^", 9));
-    count += (gwy_memmem(fileinfo->head, fileinfo->buffer_len, "^XY_Camera", 10)
-              || gwy_memmem(fileinfo->head, fileinfo->buffer_len, "^XY_Camera", 10));
+    for (guint i = 0; i < G_N_ELEMENTS(magic_strings); i++) {
+        count += (gwy_memmem(fileinfo->head, fileinfo->buffer_len, magic_strings[i].name, magic_strings[i].value)
+                  || gwy_memmem(fileinfo->tail, fileinfo->buffer_len, magic_strings[i].name, magic_strings[i].value));
+    }
     gwy_debug("count %u", count);
     if (count >= 2)
         return 100;
@@ -351,12 +339,12 @@ poirfile_detect(const GwyFileDetectInfo *fileinfo,
     return 0;
 }
 
-static GwyContainer*
+static GwyFile*
 poirfile_load(const gchar *filename,
-              G_GNUC_UNUSED GwyRunType mode,
+              G_GNUC_UNUSED GwyRunModeFlags mode,
               GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     GwyZipFile zipfile;
     gchar *currname = NULL;
     guchar *buffer = NULL;
@@ -369,10 +357,10 @@ poirfile_load(const gchar *filename,
     if (!gwyzip_first_file(zipfile, error))
         goto fail;
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     do {
         if (!gwyzip_get_current_filename(zipfile, &currname, error)) {
-            GWY_OBJECT_UNREF(container);
+            g_clear_object(&file);
             goto fail;
         }
 
@@ -383,14 +371,14 @@ poirfile_load(const gchar *filename,
         }
 
         if (!(buffer = gwyzip_get_file_content(zipfile, &size, error))) {
-            GWY_OBJECT_UNREF(container);
+            g_clear_object(&file);
             goto fail;
         }
 
         prevndata = ndata;
-        ndata = oirfile_load_from_memory(container, ndata, buffer, size, filename, error);
+        ndata = oirfile_load_from_memory(file, ndata, buffer, size, filename, error);
         if (ndata <= prevndata) {
-            GWY_OBJECT_UNREF(container);
+            g_clear_object(&file);
             goto fail;
         }
 
@@ -403,12 +391,12 @@ fail:
     g_free(buffer);
     g_free(currname);
 
-    return container;
+    return file;
 }
 #endif
 
 static gint
-oirfile_load_from_memory(GwyContainer *container,
+oirfile_load_from_memory(GwyFile *file,
                          gint channelno,
                          const guchar *buffer,
                          gsize size,
@@ -427,7 +415,7 @@ oirfile_load_from_memory(GwyContainer *container,
 #endif
     p = buffer;
     end = buffer + size;
-    gwy_clear(&oirfile, 1);
+    gwy_clear1(oirfile);
 
     /* File header */
     if (!read_file_header(&p, end, &oirfile.header, error))
@@ -527,9 +515,9 @@ oirfile_load_from_memory(GwyContainer *container,
             parse_xml_to_hash(xml, &oirfile.xmldata);
     }
 
-    if (!create_datafield(container, oirfile.xmldata.hash, 0, channelno + 0, &oirfile.red, filename, error)
-        || !create_datafield(container, oirfile.xmldata.hash, 1, channelno + 1, &oirfile.green, filename, error)
-        || !create_datafield(container, oirfile.xmldata.hash, 2, channelno + 2, &oirfile.blue, filename, error))
+    if (!create_field(file, oirfile.xmldata.hash, 0, channelno + 0, &oirfile.red, filename, error)
+        || !create_field(file, oirfile.xmldata.hash, 1, channelno + 1, &oirfile.green, filename, error)
+        || !create_field(file, oirfile.xmldata.hash, 2, channelno + 2, &oirfile.blue, filename, error))
         goto fail;
 
     channelno += 3;
@@ -1137,7 +1125,7 @@ add_meta(gpointer hkey, gpointer hvalue, gpointer user_data)
 static GwyContainer*
 make_metadata(GHashTable *hash)
 {
-    GwyContainer *meta = gwy_container_new();
+    GwyContainer *meta = gwy_container_new_in_construction();
 
     g_hash_table_foreach(hash, add_meta, meta);
     if (gwy_container_get_n_items(meta))
@@ -1149,11 +1137,11 @@ make_metadata(GHashTable *hash)
 
 /* XXX: This works for nice images, not strange images. */
 static gboolean
-create_datafield(GwyContainer *data, GHashTable *hash,
-                 guint imgid, gint i,
-                 const OIRImageDataBlock *image,
-                 const gchar *filename,
-                 GError **error)
+create_field(GwyFile *file, GHashTable *hash,
+             guint imgid, gint i,
+             const OIRImageDataBlock *image,
+             const gchar *filename,
+             GError **error)
 {
     static const gchar frameprops[] = "frameProperties::imageDefinition";
     static const gchar imgpprops[] = "imageProperties::imageInfo::phase::group::channel";
@@ -1164,9 +1152,7 @@ create_datafield(GwyContainer *data, GHashTable *hash,
     gint xres, yres, depth, j;
     gdouble xreal, yreal, zscale = 0.0;
     GwyRawDataType rawdatatype;
-    GwyDataField *dfield;
-    GwyContainer *meta;
-    GQuark quark;
+    GwyField *dfield;
 
     if (!(s = get_meta_or_fail(hash, frameprops, "width", -1, error)))
         return FALSE;
@@ -1244,18 +1230,17 @@ create_datafield(GwyContainer *data, GHashTable *hash,
     if (!name || !gwy_strequal(name, "HEIGHT"))
         zscale = 0.0;
 
-    dfield = gwy_data_field_new(xres, yres, xreal, yreal, FALSE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
+    dfield = gwy_field_new(xres, yres, xreal, yreal, FALSE);
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
     if (zscale != 0.0)
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), "m");
+        gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), "m");
 
     gwy_convert_raw_data(image->data, xres*yres, 1, rawdatatype, GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                         gwy_data_field_get_data(dfield),
+                         gwy_field_get_data(dfield),
                          zscale != 0.0 ? zscale : 1.0, 0.0);
 
-    quark = gwy_app_get_data_key_for_id(i);
-    gwy_container_set_object(data, quark, dfield);
-    gwy_file_channel_import_log_add(data, i, NULL, filename);
+    gwy_file_set_image(file, i, dfield);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, i, NULL, filename);
     g_object_unref(dfield);
 
     if (device && gwy_stramong(device, "Camera", "CAMERA", NULL)) {
@@ -1276,26 +1261,22 @@ create_datafield(GwyContainer *data, GHashTable *hash,
         }
     }
 
-    quark = gwy_app_get_data_title_key_for_id(i);
     if (name) {
         if (device) {
             title = g_strconcat(device, " ", name, NULL);
-            gwy_container_set_string(data, quark, title);
+            gwy_file_pass_title(file, GWY_FILE_IMAGE, i, title);
         }
         else
-            gwy_container_set_const_string(data, quark, name);
+            gwy_file_set_title(file, GWY_FILE_IMAGE, i, name, FALSE);
     }
     else if (device)
-        gwy_container_set_const_string(data, quark, device);
+        gwy_file_set_title(file, GWY_FILE_IMAGE, i, device, FALSE);
 
     if (gradient) {
-        quark = gwy_app_get_data_palette_key_for_id(i);
-        gwy_container_set_const_string(data, quark, gradient);
+        gwy_file_set_palette(file, GWY_FILE_IMAGE, i, gradient);
     }
 
-    meta = make_metadata(hash);
-    quark = gwy_app_get_data_meta_key_for_id(i);
-    gwy_container_pass_object(data, quark, meta);
+    gwy_file_pass_meta(file, GWY_FILE_IMAGE, i, make_metadata(hash));
 
     return TRUE;
 }
