@@ -1,23 +1,19 @@
 /*
- *  $Id: oldmda.c 26168 2024-02-08 10:18:02Z yeti-dn $
- *  Copyright (C) 2010-2011 David Necas (Yeti), Petr Klapetek,
+ *  $Id: oldmda.c 28803 2025-11-05 11:58:19Z yeti-dn $
+ *  Copyright (C) 2010-2025 David Necas (Yeti), Petr Klapetek,
  *  Daniil Bratashov (dn2010)
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net, dn2010@gmail.com
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
+ *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
+ *  later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -49,14 +45,10 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/brick.h>
-#include <libprocess/stats.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -100,50 +92,50 @@ typedef struct {
 } MDAXMLParams;
 
 typedef struct {
-    guint         xres;
-    guint         yres;
-    guint         zres;
-    gdouble       xreal;
-    gdouble       yreal;
-    GArray       *xdata;
-    GwySIUnit    *siunitx;
-    GwySIUnit    *siunity;
-    GwySIUnit    *siunitz;
-    gint          power10x;
-    gint          power10y;
-    gint          power10z;
-    const gchar  *filename;
-    GwyContainer *data;
+    guint        xres;
+    guint        yres;
+    guint        zres;
+    gdouble      xreal;
+    gdouble      yreal;
+    GArray      *xdata;
+    GwyUnit     *siunitx;
+    GwyUnit     *siunity;
+    GwyUnit     *siunitz;
+    gint         power10x;
+    gint         power10y;
+    gint         power10z;
+    const gchar *filename;
+    GwyFile     *file;
 }OldMDAFile;
 
-static gboolean      module_register     (void);
-static gint          oldmda_detect       (const GwyFileDetectInfo *fileinfo,
-                                          gboolean only_name);
-static GwyContainer* oldmda_load         (const gchar *filename,
-                                          GwyRunType mode,
-                                          GError **error);
-static void          oldmda_free         (MDAXMLParams *par);
-static void          oldmda_read_params  (MDAXMLParams *par,
-                                          OldMDAFile *mdafile);
-static void          oldmda_read_data    (OldMDAFile *mdafile,
-                                          const gchar *buffer);
-static gchar*      oldmda_find_data_name (const gchar *headername,
-                                          const gchar *dataname);
-static void          start_element       (GMarkupParseContext *context,
-                                          const gchar *element_name,
-                                          const gchar **attribute_names,
-                                          const gchar **attribute_values,
-                                          gpointer user_data,
-                                          GError **error);
-static void          end_element         (GMarkupParseContext *context,
-                                          const gchar *element_name,
-                                          gpointer user_data,
-                                          GError **error);
-static void          parse_text          (GMarkupParseContext *context,
-                                          const gchar *text,
-                                          gsize text_len,
-                                          gpointer user_data,
-                                          GError **error);
+static gboolean module_register(void);
+static gint     detect_file    (const GwyFileDetectInfo *fileinfo,
+                                gboolean only_name);
+static GwyFile* load_file      (const gchar *filename,
+                                GwyRunModeFlags mode,
+                                GError **error);
+static void     free_params    (MDAXMLParams *par);
+static void     read_params    (MDAXMLParams *par,
+                                OldMDAFile *mdafile);
+static void     read_data      (OldMDAFile *mdafile,
+                                const gchar *buffer);
+static gchar*   find_data_name (const gchar *headername,
+                                const gchar *dataname);
+static void     start_element  (GMarkupParseContext *context,
+                                const gchar *element_name,
+                                const gchar **attribute_names,
+                                const gchar **attribute_values,
+                                gpointer user_data,
+                                GError **error);
+static void     end_element    (GMarkupParseContext *context,
+                                const gchar *element_name,
+                                gpointer user_data,
+                                GError **error);
+static void     parse_text     (GMarkupParseContext *context,
+                                const gchar *text,
+                                gsize text_len,
+                                gpointer user_data,
+                                GError **error);
 
 
 static GwyModuleInfo module_info = {
@@ -151,7 +143,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports old NTMDT MDA Spectra files."),
     "dn2010 <dn2010@gmail.com>",
-    "0.4",
+    "0.5",
     "David Nečas (Yeti) & Daniil Bratashov (dn2010)",
     "2012",
 };
@@ -163,10 +155,7 @@ module_register(void)
 {
     gwy_file_func_register("oldmda",
                            N_("NTMDT old MDA Spectra data (.sxml .dat)"),
-                           (GwyFileDetectFunc)&oldmda_detect,
-                           (GwyFileLoadFunc)&oldmda_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
@@ -174,13 +163,11 @@ module_register(void)
 static gboolean
 check_magic(const gchar *header)
 {
-    return (memcmp(header, MAGIC1, MAGIC1_SIZE) == 0)
-            && (strstr(header, MAGIC2) != NULL);
+    return (memcmp(header, MAGIC1, MAGIC1_SIZE) == 0) && (strstr(header, MAGIC2) != NULL);
 }
 
 static gint
-oldmda_detect(const GwyFileDetectInfo *fileinfo,
-              gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     if (only_name)
         return g_str_has_suffix(fileinfo->name_lowercase, EXTENSION) ? 10 : 0;
@@ -188,12 +175,12 @@ oldmda_detect(const GwyFileDetectInfo *fileinfo,
     return check_magic(fileinfo->head) ? 100 : 0;
 }
 
-static GwyContainer*
-oldmda_load(const gchar *filename,
-            G_GNUC_UNUSED GwyRunType mode,
-            GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     OldMDAFile mdafile;
     gchar *head, *buffer = NULL;
     gchar *buffer2 = NULL;
@@ -209,7 +196,7 @@ oldmda_load(const gchar *filename,
         goto fail;
     }
 
-    gwy_clear(&mdafile, 1);
+    gwy_clear1(mdafile);
     head = g_strndup(buffer, 1024);
     if (!check_magic(head)) {
         err_FILE_TYPE(error, "NTMDT old MDA");
@@ -222,8 +209,7 @@ oldmda_load(const gchar *filename,
     params.numaxes = 0;
     params.axes = g_array_new(FALSE, FALSE, sizeof(MDAAxis));
     params.flag = MDA_XML_NONE;
-    context = g_markup_parse_context_new(&parser, G_MARKUP_TREAT_CDATA_AS_TEXT,
-                                         &params, NULL);
+    context = g_markup_parse_context_new(&parser, G_MARKUP_TREAT_CDATA_AS_TEXT, &params, NULL);
     if (!g_markup_parse_context_parse(context, buffer, size, &err)
         || !g_markup_parse_context_end_parse(context, &err)) {
         err_XML(error, &err);
@@ -238,10 +224,10 @@ oldmda_load(const gchar *filename,
         goto fail;
     }
 
-    oldmda_read_params(&params, &mdafile);
+    read_params(&params, &mdafile);
 
     dname = g_strdelimit(params.dataname, "\\/", G_DIR_SEPARATOR);
-    if (!(dataname = oldmda_find_data_name(filename, dname))) {
+    if (!(dataname = find_data_name(filename, dname))) {
         g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
                     _("No corresponding data file was found for header file."));
         goto fail;
@@ -262,21 +248,21 @@ oldmda_load(const gchar *filename,
         goto fail;
     }
 
-    container = gwy_container_new();
-    mdafile.data = container;
+    file = gwy_file_new_in_construction();
+    mdafile.file = file;
     mdafile.filename = dataname;
-    oldmda_read_data(&mdafile, buffer2);
+    read_data(&mdafile, buffer2);
 
 fail:
     g_free(buffer2);
     g_free(buffer);
-    oldmda_free(&params);
+    free_params(&params);
 
-    return container;
+    return file;
 }
 
 static void
-oldmda_free(MDAXMLParams *par)
+free_params(MDAXMLParams *par)
 {
     guint i;
 
@@ -294,78 +280,76 @@ oldmda_free(MDAXMLParams *par)
 }
 
 static void
-oldmda_read_params(MDAXMLParams *par, OldMDAFile *mdafile)
+read_params(MDAXMLParams *par, OldMDAFile *mdafile)
 {
     MDAAxis axis;
     mdafile->zres = par->res;
     mdafile->xdata = par->xdata;
     axis = g_array_index(par->axes, MDAAxis, 0);
     if (axis.unitname)
-        mdafile->siunitz = gwy_si_unit_new(axis.unitname);
+        mdafile->siunitz = gwy_unit_new(axis.unitname);
     else
-        mdafile->siunitz = gwy_si_unit_new(NULL);
+        mdafile->siunitz = gwy_unit_new(NULL);
     axis = g_array_index(par->axes, MDAAxis, 1);
     if (axis.unitname)
-        mdafile->siunitx = gwy_si_unit_new_parse(axis.unitname,
-                                                 &mdafile->power10x);
+        mdafile->siunitx = gwy_unit_new_parse(axis.unitname, &mdafile->power10x);
     else
-        mdafile->siunitx = gwy_si_unit_new(NULL);
+        mdafile->siunitx = gwy_unit_new(NULL);
     mdafile->xres = axis.maxindex - axis.minindex + 1;
     if (mdafile->xres < 1)
         mdafile->xres = 1;
     if (axis.scale <= 0.0)
         axis.scale = 1.0;
-    mdafile->xreal = axis.scale * mdafile->xres * pow10(mdafile->power10x);
+    mdafile->xreal = axis.scale * mdafile->xres * gwy_exp10(mdafile->power10x);
     axis = g_array_index(par->axes, MDAAxis, 2);
     if (axis.unitname)
-        mdafile->siunity = gwy_si_unit_new_parse(axis.unitname,
-                                                 &mdafile->power10y);
+        mdafile->siunity = gwy_unit_new_parse(axis.unitname, &mdafile->power10y);
     else
-        mdafile->siunity = gwy_si_unit_new(NULL);
+        mdafile->siunity = gwy_unit_new(NULL);
     mdafile->yres = axis.maxindex - axis.minindex + 1;
     if (mdafile->yres < 1)
         mdafile->yres = 1;
     if (axis.scale <= 0.0)
         axis.scale = 1.0;
-    mdafile->yreal = axis.scale * mdafile->yres * pow10(mdafile->power10y);
+    mdafile->yreal = axis.scale * mdafile->yres * gwy_exp10(mdafile->power10y);
 }
 
 static void
-oldmda_read_data(OldMDAFile *mdafile, const gchar *buffer)
+read_data(OldMDAFile *mdafile, const gchar *buffer)
 {
     GwyBrick *brick;
-    GwyDataLine *cal;
+    GwyLine *cal;
     gdouble *data;
     gint i, j, k;
     const guchar *p;
+    gint xres = mdafile->xres, yres = mdafile->yres, zres = mdafile->zres;
 
     p = buffer;
-    brick = gwy_brick_new(mdafile->xres, mdafile->yres, mdafile->zres,
-                          mdafile->xreal, mdafile->yreal, mdafile->zres,
+    brick = gwy_brick_new(xres, yres, zres,
+                          mdafile->xreal, mdafile->yreal, zres,
                           TRUE);
     data = gwy_brick_get_data(brick);
 
-    for (k = 0; k < mdafile->zres; k++) {
+    for (k = 0; k < zres; k++) {
         p = buffer + k * 4;
-        for (i = 0; i < mdafile->yres; i++)
-            for (j = 0; j < mdafile->xres; j++) {
-                *(data + k * mdafile->xres * mdafile->yres + j
-                    + (mdafile->yres - i - 1) * mdafile->xres)
-                        = (gdouble)gwy_get_gint32_le(&p);
-                p += (mdafile->zres - 1) * 4;
+        for (i = 0; i < yres; i++) {
+            for (j = 0; j < xres; j++) {
+                data[k*xres*yres + j + (yres - i - 1)*xres] = gwy_get_gint32_le(&p);
+                p += (zres - 1) * 4;
             }
+        }
     }
 
-    gwy_brick_set_si_unit_x(brick, mdafile->siunitx);
-    gwy_brick_set_si_unit_y(brick, mdafile->siunity);
-    gwy_brick_set_si_unit_z(brick, mdafile->siunitz);
+    gwy_unit_assign(gwy_brick_get_unit_x(brick), mdafile->siunitx);
+    gwy_unit_assign(gwy_brick_get_unit_y(brick), mdafile->siunity);
+    gwy_unit_assign(gwy_brick_get_unit_z(brick), mdafile->siunitz);
 
-    cal = gwy_data_line_new(mdafile->zres, mdafile->zres, FALSE);
-    data = gwy_data_line_get_data(cal);
-    for (k = 0; k < mdafile->zres; k++) {
+    cal = gwy_line_new(zres, zres, FALSE);
+    data = gwy_line_get_data(cal);
+    for (k = 0; k < zres; k++) {
         *(data++) = g_array_index(mdafile->xdata, gdouble, k);
     }
-    gwy_data_line_set_si_unit_y(cal, mdafile->siunitz);
+    gwy_unit_assign(gwy_line_get_unit_y(cal), mdafile->siunitz);
     gwy_brick_set_zcalibration(brick, cal);
     g_object_unref(cal);
 
@@ -373,16 +357,13 @@ oldmda_read_data(OldMDAFile *mdafile, const gchar *buffer)
     g_object_unref(mdafile->siunity);
     g_object_unref(mdafile->siunitz);
 
-    gwy_container_set_object_by_name(mdafile->data, "/brick/0", brick);
-    gwy_container_set_const_string_by_name(mdafile->data, "/brick/0/title",
-                                           "MDA data");
-    g_object_unref(brick);
-
-    gwy_file_volume_import_log_add(mdafile->data, 0, NULL, mdafile->filename);
+    gwy_file_pass_volume(mdafile->file, 0, brick);
+    gwy_file_set_title(mdafile->file, GWY_FILE_VOLUME, 0, "MDA data", FALSE);
+    gwy_log_add_import(mdafile->file, GWY_FILE_VOLUME, 0, NULL, mdafile->filename);
 }
 
 static gchar*
-oldmda_find_data_name(const gchar *headername, const gchar *dataname)
+find_data_name(const gchar *headername, const gchar *dataname)
 {
     gchar *dirname = g_path_get_dirname(headername);
     gchar *data_name = g_path_get_basename(dataname);
@@ -462,7 +443,7 @@ start_element(G_GNUC_UNUSED GMarkupParseContext *context,
         }
         else if (gwy_strequal(element_name, "Calibration")) {
             params->numaxes++;
-            gwy_clear(&axis, 1);
+            gwy_clear1(axis);
 
             while (*name_cursor) {
                 if (gwy_strequal(*name_cursor, "UnitName")) {
@@ -562,4 +543,4 @@ parse_text(G_GNUC_UNUSED GMarkupParseContext *context,
     g_free(v);
 }
 
-/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
+/* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
