@@ -1,6 +1,6 @@
 /*
- *  $Id: ols.c 26158 2024-02-06 12:31:23Z yeti-dn $
- *  Copyright (C) 2006 David Necas (Yeti), Petr Klapetek.
+ *  $Id: ols.c 28911 2025-11-24 18:27:42Z yeti-dn $
+ *  Copyright (C) 2006-2025 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -35,32 +35,30 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/stats.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 #include "err.h"
 #include "gwytiff.h"
 
 #define MAGIC_COMMENT "System Name =         OLS"
 
-static gboolean      module_register (void);
-static gint          ols_detect      (const GwyFileDetectInfo *fileinfo,
-                                      gboolean only_name);
-static GwyContainer* ols_load        (const gchar *filename,
-                                      GwyRunType mode,
-                                      GError **error);
-static GwyContainer* ols_load_tiff   (const GwyTIFF *tiff,
-                                      const gchar *filename,
-                                      GError **error);
+static gboolean module_register(void);
+static gint     detect_file    (const GwyFileDetectInfo *fileinfo,
+                                gboolean only_name);
+static GwyFile* load_file      (const gchar *filename,
+                                GwyRunModeFlags mode,
+                                GError **error);
+static GwyFile* ols_load_tiff  (const GwyTIFF *tiff,
+                                const gchar *filename,
+                                GError **error);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     module_register,
     N_("Imports OLS data files."),
     "Jan Hořák <xhorak@gmail.com>, Yeti <yeti@gwyddion.net>",
-    "0.11",
+    "0.12",
     "David Nečas (Yeti) & Petr Klapetek",
     "2008",
 };
@@ -72,16 +70,13 @@ module_register(void)
 {
     gwy_file_func_register("ols",
                            N_("Olympus LEXT OLS3000 (.ols)"),
-                           (GwyFileDetectFunc)&ols_detect,
-                           (GwyFileLoadFunc)&ols_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-ols_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     GwyTIFF *tiff;
     gint score = 0;
@@ -109,33 +104,33 @@ ols_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
     return score;
 }
 
-static GwyContainer*
-ols_load(const gchar *filename,
-         G_GNUC_UNUSED GwyRunType mode,
-         GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
     GwyTIFF *tiff;
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
 
     tiff = gwy_tiff_load(filename, error);
     if (!tiff)
         return NULL;
 
-    container = ols_load_tiff(tiff, filename, error);
+    file = ols_load_tiff(tiff, filename, error);
     gwy_tiff_free(tiff);
 
-    return container;
+    return file;
 }
 
-static GwyContainer*
+static GwyFile*
 ols_load_tiff(const GwyTIFF *tiff, const gchar *filename, GError **error)
 {
     const gchar *colour_channels[] = { "Red", "Green", "Blue" };
     const gchar *colour_channel_gradients[] = { "RGB-Red", "RGB-Green", "RGB-Blue" };
 
-    GwyContainer *container = NULL;
-    GwyDataField *dfield;
-    GwySIUnit *siunit;
+    GwyFile *file = NULL;
+    GwyField *dfield;
+    GwyUnit *siunit;
     GwyTIFFImageReader *reader = NULL;
     GwyTextHeaderParser parser;
     GHashTable *hash;
@@ -156,7 +151,7 @@ ols_load_tiff(const GwyTIFF *tiff, const gchar *filename, GError **error)
     }
 
     /* Read the comment header. */
-    gwy_clear(&parser, 1);
+    gwy_clear1(parser);
     parser.key_value_separator = "=";
     parser.section_template = "[\x1a]";
     parser.endsection_template = "[\x1a End]";
@@ -190,32 +185,32 @@ ols_load_tiff(const GwyTIFF *tiff, const gchar *filename, GError **error)
 
         for (ch = 0; ch < spp; ch++) {
             factor = 1e-9;
-            dfield = gwy_data_field_new(reader->width, reader->height,
-                                        reader->width * xy_axis * factor, reader->height * xy_axis * factor,
-                                        FALSE);
+            dfield = gwy_field_new(reader->width, reader->height,
+                                   reader->width * xy_axis * factor, reader->height * xy_axis * factor,
+                                   FALSE);
             // units
-            gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
+            gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
 
-            siunit = gwy_data_field_get_si_unit_z(dfield);
+            siunit = gwy_field_get_unit_z(dfield);
             power10 = 0;
             if (spp == 1) {
                 if (dir_num == 1)
-                    gwy_si_unit_set_from_string_parse(siunit, "nm", &power10);
+                    power10 = gwy_unit_set_from_string(siunit, "nm");
                 else
                     power10 = -6;
             }
 
-            factor = z_axis * pow10(power10);
-            data = gwy_data_field_get_data(dfield);
+            factor = z_axis * gwy_exp10(power10);
+            data = gwy_field_get_data(dfield);
 
             for (i = 0; i < reader->height; i++)
                 gwy_tiff_read_image_row(tiff, reader, ch, i, factor, 0.0, data + i*reader->width);
 
-            /* add read datafield to container */
-            if (!container)
-                container = gwy_container_new();
+            /* add read field to file */
+            if (!file)
+                file = gwy_file_new_in_construction();
 
-            gwy_container_pass_object(container, gwy_app_get_data_key_for_id(id), dfield);
+            gwy_file_pass_image(file, id, dfield);
 
             title = NULL;
             if (spp == 1) {
@@ -227,13 +222,12 @@ ols_load_tiff(const GwyTIFF *tiff, const gchar *filename, GError **error)
             }
             else {
                 title = colour_channels[ch];
-                gwy_container_set_const_string(container, gwy_app_get_data_palette_key_for_id(id),
-                                               colour_channel_gradients[ch]);
+                gwy_file_set_palette(file, GWY_FILE_IMAGE, id, colour_channel_gradients[ch]);
             }
             if (title)
-                gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(id), title);
+                gwy_file_set_title(file, GWY_FILE_IMAGE, id, title, FALSE);
 
-            gwy_file_channel_import_log_add(container, id, NULL, filename);
+            gwy_log_add_import(file, GWY_FILE_IMAGE, id, NULL, filename);
 
             id++;
         }
@@ -247,10 +241,10 @@ ols_load_tiff(const GwyTIFF *tiff, const gchar *filename, GError **error)
         reader = NULL;
     }
 
-    if (!container)
+    if (!file)
         err_NO_DATA(error);
 
-    return container;
+    return file;
 }
 
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
