@@ -1,6 +1,6 @@
 /*
- *  $Id: ometiff.c 26168 2024-02-08 10:18:02Z yeti-dn $
- *  Copyright (C) 2013-2016 David Necas (Yeti).
+ *  $Id: ometiff.c 28789 2025-11-04 17:14:03Z yeti-dn $
+ *  Copyright (C) 2013-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -46,11 +46,9 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/stats.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 #include "err.h"
 #include "gwytiff.h"
 
@@ -111,45 +109,45 @@ typedef struct {
     GHashTable *meta;
 } OMEFile;
 
-static gboolean      module_register        (void);
-static gint          ome_detect             (const GwyFileDetectInfo *fileinfo,
-                                             gboolean only_name);
-static GwyContainer* ome_load               (const gchar *filename,
-                                             GwyRunType mode,
-                                             GError **error);
-static GwyContainer* ome_load_tiff          (const GwyTIFF *tiff,
-                                             const gchar *filename,
-                                             GError **error);
-static void          ome_file_free          (OMEFile *omefile);
-static void          start_element          (GMarkupParseContext *context,
-                                             const gchar *element_name,
-                                             const gchar **attribute_names,
-                                             const gchar **attribute_values,
-                                             gpointer user_data,
-                                             GError **error);
-static void          end_element            (GMarkupParseContext *context,
-                                             const gchar *element_name,
-                                             gpointer user_data,
-                                             GError **error);
-static void          text                   (GMarkupParseContext *context,
-                                             const gchar *value,
-                                             gsize value_len,
-                                             gpointer user_data,
-                                             GError **error);
-static gboolean      assign_tiff_directories(OMEFile *omefile,
-                                             GError **error);
-static void          next_ztc               (const OMEFile *omefile,
-                                             const OMEData *data,
-                                             guint *z,
-                                             guint *t,
-                                             guint *c);
+static gboolean module_register        (void);
+static gint     detect_file            (const GwyFileDetectInfo *fileinfo,
+                                        gboolean only_name);
+static GwyFile* load_file              (const gchar *filename,
+                                        GwyRunModeFlags mode,
+                                        GError **error);
+static GwyFile* ome_load_tiff          (const GwyTIFF *tiff,
+                                        const gchar *filename,
+                                        GError **error);
+static void     ome_file_free          (OMEFile *omefile);
+static void     start_element          (GMarkupParseContext *context,
+                                        const gchar *element_name,
+                                        const gchar **attribute_names,
+                                        const gchar **attribute_values,
+                                        gpointer user_data,
+                                        GError **error);
+static void     end_element            (GMarkupParseContext *context,
+                                        const gchar *element_name,
+                                        gpointer user_data,
+                                        GError **error);
+static void     text                   (GMarkupParseContext *context,
+                                        const gchar *value,
+                                        gsize value_len,
+                                        gpointer user_data,
+                                        GError **error);
+static gboolean assign_tiff_directories(OMEFile *omefile,
+                                        GError **error);
+static void     next_ztc               (const OMEFile *omefile,
+                                        const OMEData *data,
+                                        guint *z,
+                                        guint *t,
+                                        guint *c);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     module_register,
     N_("Imports OME-TIFF data files."),
     "Yeti <yeti@gwyddion.net>",
-    "1.1",
+    "2.0",
     "David Nečas (Yeti)",
     "2013",
 };
@@ -161,16 +159,13 @@ module_register(void)
 {
     gwy_file_func_register("ometiff",
                            N_("Open Microscopy OME-TIFF (.ome.tiff)"),
-                           (GwyFileDetectFunc)&ome_detect,
-                           (GwyFileLoadFunc)&ome_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-ome_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     GwyTIFF *tiff;
     gint score = 0;
@@ -197,32 +192,32 @@ ome_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
     return score;
 }
 
-static GwyContainer*
-ome_load(const gchar *filename,
-         G_GNUC_UNUSED GwyRunType mode,
-         GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
     GwyTIFF *tiff;
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
 
     tiff = gwy_tiff_load(filename, error);
     if (!tiff)
         return NULL;
 
-    container = ome_load_tiff(tiff, filename, error);
+    file = ome_load_tiff(tiff, filename, error);
     gwy_tiff_free(tiff);
 
-    return container;
+    return file;
 }
 
-static GwyContainer*
+static GwyFile*
 ome_load_tiff(const GwyTIFF *tiff, const gchar *filename, GError **error)
 {
     const gchar *colour_channels[] = { "Red", "Green", "Blue" };
     const gchar *colour_channel_gradients[] = { "RGB-Red", "RGB-Green", "RGB-Blue" };
 
-    GwyContainer *container = NULL;
-    GwyDataField *dfield;
+    GwyFile *file = NULL;
+    GwyField *dfield;
     GwyTIFFImageReader *reader = NULL;
     GMarkupParser parser = { start_element, end_element, text, NULL, NULL };
     GMarkupParseContext *context = NULL;
@@ -245,7 +240,7 @@ ome_load_tiff(const GwyTIFF *tiff, const gchar *filename, GError **error)
     }
 
     /* Read the comment header. */
-    gwy_clear(&omefile, 1);
+    gwy_clear1(omefile);
     omefile.ndirs = tiff->dirs->len;
     omefile.meta = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
     omefile.path = key = g_string_new(NULL);
@@ -281,7 +276,7 @@ ome_load_tiff(const GwyTIFF *tiff, const gchar *filename, GError **error)
         goto fail;
     }
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     id = 0;
 
     for (dir_num = 0; dir_num < omefile.ndirs; dir_num++) {
@@ -312,17 +307,17 @@ ome_load_tiff(const GwyTIFF *tiff, const gchar *filename, GError **error)
                 yreal = omefile.yreal;
             }
 
-            dfield = gwy_data_field_new(reader->width, reader->height, xreal, yreal, FALSE);
+            dfield = gwy_field_new(reader->width, reader->height, xreal, yreal, FALSE);
             /* units */
             if (omefile.xreal > 0.0 && omefile.yreal > 0.0)
-                gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
+                gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
 
-            d = gwy_data_field_get_data(dfield);
+            d = gwy_field_get_data(dfield);
             for (i = 0; i < reader->height; i++)
                 gwy_tiff_read_image_row(tiff, reader, ch, i, zfactor, 0.0, d + i*reader->width);
 
-            /* add read datafield to container */
-            gwy_container_pass_object(container, gwy_app_get_data_key_for_id(id), dfield);
+            /* add read field to file */
+            gwy_file_pass_image(file, id, dfield);
 
             /* It makes sense to use only grey channels in OME TIFF since they
              * have an explicit colour plane.  But if there is RGB... */
@@ -338,14 +333,12 @@ ome_load_tiff(const GwyTIFF *tiff, const gchar *filename, GError **error)
                 channeltitle = g_strdup_printf("Z%u T%u C%u",
                                                assignment->z, assignment->t, assignment->c);
             }
-            gwy_container_set_string(container, gwy_app_get_data_title_key_for_id(id), channeltitle);
+            gwy_file_pass_title(file, GWY_FILE_IMAGE, id, channeltitle);
 
-            if (spp == 3) {
-                gwy_container_set_const_string(container, gwy_app_get_data_palette_key_for_id(id),
-                                               colour_channel_gradients[ch]);
-            }
+            if (spp == 3)
+                gwy_file_set_palette(file, GWY_FILE_IMAGE, id, colour_channel_gradients[ch]);
 
-            gwy_file_channel_import_log_add(container, id, NULL, filename);
+            gwy_log_add_import(file, GWY_FILE_IMAGE, id, NULL, filename);
             id++;
         }
     }
@@ -363,7 +356,7 @@ fail:
     if (context)
         g_markup_parse_context_free(context);
 
-    return container;
+    return file;
 }
 
 static void
@@ -460,7 +453,7 @@ start_element(G_GNUC_UNUSED GMarkupParseContext *context,
         OMEData data;
         gboolean have_ifd = FALSE, have_planecount = FALSE;
 
-        gwy_clear(&data, 1);
+        gwy_clear1(data);
         for (i = 0; attribute_names[i]; i++) {
             const gchar *name = attribute_names[i], *val = attribute_values[i];
             if (gwy_strequal(name, "IFD")) {
