@@ -1,6 +1,6 @@
 /*
- *  $Id: omicron.c 26157 2024-02-06 12:30:33Z yeti-dn $
- *  Copyright (C) 2006-2011 David Necas (Yeti), Petr Klapetek, Markus Pristovsek
+ *  $Id: omicron.c 28911 2025-11-24 18:27:42Z yeti-dn $
+ *  Copyright (C) 2006-2025 David Necas (Yeti), Petr Klapetek, Markus Pristovsek
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net,
  *  prissi@gift.physik.tu-berlin.de.
  *
@@ -49,15 +49,10 @@
 /* TODO: metadata */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
 #include <string.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <libprocess/datafield.h>
-#include <libprocess/spectra.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -126,33 +121,33 @@ typedef struct {
     GPtrArray *spectro_channels;
 } OmicronFile;
 
-static gboolean      module_register            (void);
-static gint          omicron_detect             (const GwyFileDetectInfo *fileinfo,
-                                                 gboolean only_name);
-static GwyContainer* omicron_load               (const gchar *filename,
-                                                 GwyRunType mode,
-                                                 GError **error);
-static gboolean      omicron_read_header        (gchar *buffer,
-                                                 OmicronFile *ofile,
-                                                 GError **error);
-static gboolean      omicron_read_topo_header   (gchar **buffer,
-                                                 OmicronTopoChannel *channel,
-                                                 GError **error);
-static gboolean      omicron_read_spectro_header(gchar **buffer,
-                                                 OmicronSpectroChannel *channel,
-                                                 GError **error);
-static GwyDataField* omicron_read_data          (OmicronFile *ofile,
-                                                 OmicronTopoChannel *channel,
-                                                 gchar **datafilename,
-                                                 GError **error);
-static GwySpectra*   omicron_read_cs_data       (OmicronFile *ofile,
-                                                 OmicronSpectroChannel *channel,
-                                                 GError **error);
-static GwySpectra*   omicron_read_be_data       (OmicronFile *ofile,
-                                                 OmicronSpectroChannel *channel,
-                                                 GError **error);
-static GwyContainer* omicron_make_meta          (OmicronFile *ofile);
-static void          omicron_file_free          (OmicronFile *ofile);
+static gboolean      module_register    (void);
+static gint          detect_file        (const GwyFileDetectInfo *fileinfo,
+                                         gboolean only_name);
+static GwyFile*      load_file          (const gchar *filename,
+                                         GwyRunModeFlags mode,
+                                         GError **error);
+static gboolean      read_header        (gchar *buffer,
+                                         OmicronFile *ofile,
+                                         GError **error);
+static gboolean      read_topo_header   (gchar **buffer,
+                                         OmicronTopoChannel *channel,
+                                         GError **error);
+static gboolean      read_spectro_header(gchar **buffer,
+                                         OmicronSpectroChannel *channel,
+                                         GError **error);
+static GwyField*     read_data          (OmicronFile *ofile,
+                                         OmicronTopoChannel *channel,
+                                         gchar **datafilename,
+                                         GError **error);
+static GwySpectra*   read_cs_data       (OmicronFile *ofile,
+                                         OmicronSpectroChannel *channel,
+                                         GError **error);
+static GwySpectra*   read_be_data       (OmicronFile *ofile,
+                                         OmicronSpectroChannel *channel,
+                                         GError **error);
+static GwyContainer* make_meta          (OmicronFile *ofile);
+static void          free_omicronfile   (OmicronFile *ofile);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
@@ -171,17 +166,13 @@ module_register(void)
 {
     gwy_file_func_register("omicron",
                            N_("Omicron files (.par + data)"),
-                           (GwyFileDetectFunc)&omicron_detect,
-                           (GwyFileLoadFunc)&omicron_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-omicron_detect(const GwyFileDetectInfo *fileinfo,
-               gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     const gchar *p = fileinfo->head;
 
@@ -208,8 +199,8 @@ omicron_detect(const GwyFileDetectInfo *fileinfo,
 
 /* Check whether file name ends .STEM[0-9] */
 static gboolean
-omicron_has_extension(const gchar *filename,
-                      const gchar *stem)
+has_extension(const gchar *filename,
+              const gchar *stem)
 {
     guint flen, slen;
 
@@ -224,18 +215,18 @@ omicron_has_extension(const gchar *filename,
             && !g_ascii_strncasecmp(filename + (flen-1-slen), stem, slen));
 }
 
-static GwyContainer*
-omicron_load(const gchar *filename,
-             G_GNUC_UNUSED GwyRunType mode,
-             GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
     OmicronFile ofile;
-    GwyContainer *container = NULL, *meta;
+    GwyFile *file = NULL;
+    GwyContainer *meta;
     gchar *text = NULL;
     GError *err = NULL;
-    GwyDataField *dfield = NULL;
+    GwyField *dfield = NULL;
     GwySpectra *spectra = NULL;
-    gchar key[32];
     guint i;
 
     /* @text must not be destroyed while @ofile is still in used because all strings are only references there */
@@ -244,9 +235,9 @@ omicron_load(const gchar *filename,
         return NULL;
     }
 
-    gwy_clear(&ofile, 1);
+    gwy_clear1(ofile);
     ofile.filename = filename;
-    if (!omicron_read_header(text, &ofile, error))
+    if (!read_header(text, &ofile, error))
         goto fail;
 
     if (!ofile.topo_channels || !ofile.topo_channels->len) {
@@ -254,7 +245,7 @@ omicron_load(const gchar *filename,
         goto fail;
     }
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
 
     /* First Load the Topographic Data */
     for (i = 0; i < ofile.topo_channels->len; i++) {
@@ -262,34 +253,30 @@ omicron_load(const gchar *filename,
         gchar *datafilename = NULL;
 
         channel = g_ptr_array_index(ofile.topo_channels, i);
-        dfield = omicron_read_data(&ofile, channel, &datafilename, error);
+        dfield = read_data(&ofile, channel, &datafilename, error);
         if (!dfield) {
-            GWY_OBJECT_UNREF(container);
+            g_clear_object(&file);
             goto fail;
         }
 
-        g_snprintf(key, sizeof(key), "/%u/data", i);
-        gwy_container_pass_object_by_name(container, key, dfield);
+        gwy_file_pass_image(file, i, dfield);
 
         if (channel->name) {
             gchar *s;
 
-            g_snprintf(key, sizeof(key), "/%u/data/title", i);
             if (channel->scandir == SCAN_FORWARD)
                 s = g_strdup_printf("%s (Forward)", channel->name);
             else if (channel->scandir == SCAN_BACKWARD)
                 s = g_strdup_printf("%s (Backward)", channel->name);
             else
                 s = g_strdup(channel->name);
-            gwy_container_set_string_by_name(container, key, s);
+            gwy_file_pass_title(file, GWY_FILE_IMAGE, i, s);
         }
 
-        if ((meta = omicron_make_meta(&ofile))) {
-            g_snprintf(key, sizeof(key), "/%u/meta", i);
-            gwy_container_pass_object_by_name(container, key, meta);
-        }
+        if ((meta = make_meta(&ofile)))
+            gwy_file_pass_meta(file, GWY_FILE_IMAGE, i, meta);
 
-        gwy_file_channel_import_log_add(container, i, NULL, datafilename);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, i, NULL, datafilename);
         g_free(datafilename);
     }
 
@@ -313,13 +300,12 @@ omicron_load(const gchar *filename,
             OmicronSpectroChannel *channel;
 
             channel = g_ptr_array_index(ofile.spectro_channels, i);
-            if (omicron_has_extension(channel->filename, "cs")) {
+            if (has_extension(channel->filename, "cs")) {
                 gchar *t;
-                GQuark quark;
 
-                spectra = omicron_read_cs_data(&ofile, channel, error);
+                spectra = read_cs_data(&ofile, channel, error);
                 if (!spectra) {
-                    GWY_OBJECT_UNREF(container);
+                    g_clear_object(&file);
                     goto fail;
                 }
 
@@ -333,16 +319,15 @@ omicron_load(const gchar *filename,
                 t = g_strconcat(channel->chan, "-", channel->param, NULL);
                 gwy_spectra_set_title(spectra, t);
                 g_free(t);
-                quark = gwy_app_get_spectra_key_for_id(i);
-                gwy_container_pass_object(container, quark, spectra);
+                gwy_file_pass_spectra(file, i, spectra);
+                gwy_log_add_import(file, GWY_FILE_SPECTRA, i, NULL, filename);
             }
-            else if (omicron_has_extension(channel->filename, "sf") || omicron_has_extension(channel->filename, "sb")) {
+            else if (has_extension(channel->filename, "sf") || has_extension(channel->filename, "sb")) {
                 gchar *t;
-                GQuark quark;
 
-                spectra = omicron_read_be_data(&ofile, channel, error);
+                spectra = read_be_data(&ofile, channel, error);
                 if (!spectra) {
-                    GWY_OBJECT_UNREF(container);
+                    g_clear_object(&file);
                     goto fail;
                 }
 
@@ -356,8 +341,8 @@ omicron_load(const gchar *filename,
                 t = g_strconcat(channel->chan, "-", channel->param, NULL);
                 gwy_spectra_set_title(spectra, t);
                 g_free(t);
-                quark = gwy_app_get_spectra_key_for_id(i);
-                gwy_container_pass_object(container, quark, spectra);
+                gwy_file_pass_spectra(file, i, spectra);
+                gwy_log_add_import(file, GWY_FILE_SPECTRA, i, NULL, filename);
             }
             else {
                 g_warning("Cannot determine spectra type of %s", channel->filename);
@@ -366,10 +351,10 @@ omicron_load(const gchar *filename,
     }
 
 fail:
-    omicron_file_free(&ofile);
+    free_omicronfile(&ofile);
     g_free(text);
 
-    return container;
+    return file;
 }
 
 /* XXX: The return value sometimes means FALSE, sometimes NULL. */
@@ -382,9 +367,9 @@ fail:
     } while (FALSE)
 
 static gboolean
-omicron_read_header(gchar *buffer,
-                    OmicronFile *ofile,
-                    GError **error)
+read_header(gchar *buffer,
+            OmicronFile *ofile,
+            GError **error)
 {
     gchar *line, *val, *comment;
 
@@ -439,7 +424,7 @@ omicron_read_header(gchar *buffer,
             gwy_debug("Topographic Channel found (type %c)", val[0]);
             channel = g_new0(OmicronTopoChannel, 1);
             channel->type = val[0];
-            if (!omicron_read_topo_header(&buffer, channel, error)) {
+            if (!read_topo_header(&buffer, channel, error)) {
                 g_free(channel);
                 return FALSE;
             }
@@ -454,14 +439,13 @@ omicron_read_header(gchar *buffer,
 
             channel = g_new0(OmicronSpectroChannel, 1);
             channel->chan = val;
-            if (!omicron_read_spectro_header(&buffer, channel, error)) {
+            if (!read_spectro_header(&buffer, channel, error)) {
                 g_free(channel);
                 return FALSE;
             }
             if (!ofile->spectro_channels)
                 ofile->spectro_channels = g_ptr_array_new();
             g_ptr_array_add(ofile->spectro_channels, channel);
-
         }
         else {
             gwy_debug("<%s> = <%s>", line, val);
@@ -508,9 +492,9 @@ omicron_read_header(gchar *buffer,
     g_strstrip(line)
 
 static gboolean
-omicron_read_topo_header(gchar **buffer,
-                         OmicronTopoChannel *channel,
-                         GError **error)
+read_topo_header(gchar **buffer,
+                 OmicronTopoChannel *channel,
+                 GError **error)
 {
     gchar *line, *p;
 
@@ -562,9 +546,9 @@ omicron_read_topo_header(gchar **buffer,
 }
 
 static gboolean
-omicron_read_spectro_header(gchar **buffer,
-                            OmicronSpectroChannel *channel,
-                            GError **error)
+read_spectro_header(gchar **buffer,
+                    OmicronSpectroChannel *channel,
+                    GError **error)
 {
     gchar *line, *p;
 
@@ -660,9 +644,9 @@ omicron_read_spectro_header(gchar **buffer,
 /* In most Omicron files, the letter case is arbitrary.  Try miscellaneous
  * variations till we finally give up */
 static gchar*
-omicron_fix_file_name(const gchar *parname,
-                      const gchar *orig,
-                      GError **error)
+fix_file_name(const gchar *parname,
+              const gchar *orig,
+              GError **error)
 {
     gchar *filename, *dirname, *base;
     guint len, i;
@@ -711,14 +695,14 @@ omicron_fix_file_name(const gchar *parname,
     return NULL;
 }
 
-static GwyDataField*
-omicron_read_data(OmicronFile *ofile,
-                  OmicronTopoChannel *channel,
-                  gchar **datafilename,
-                  GError **error)
+static GwyField*
+read_data(OmicronFile *ofile,
+          OmicronTopoChannel *channel,
+          gchar **datafilename,
+          GError **error)
 {
     GError *err = NULL;
-    GwyDataField *dfield;
+    GwyField *dfield;
     gchar *filename;
     gdouble *data;
     guchar *buffer;
@@ -729,7 +713,7 @@ omicron_read_data(OmicronFile *ofile,
     gint power10 = 0;
 
     *datafilename = NULL;
-    filename = omicron_fix_file_name(ofile->filename, channel->filename, error);
+    filename = fix_file_name(ofile->filename, channel->filename, error);
     if (!filename)
         return NULL;
 
@@ -749,8 +733,8 @@ omicron_read_data(OmicronFile *ofile,
     }
 
     scale = (channel->max_phys - channel->min_phys)/(channel->max_raw - channel->min_raw);
-    dfield = gwy_data_field_new(ofile->xres, ofile->yres, ofile->xreal, ofile->yreal, FALSE);
-    data = gwy_data_field_get_data(dfield);
+    dfield = gwy_field_new(ofile->xres, ofile->yres, ofile->xreal, ofile->yreal, FALSE);
+    data = gwy_field_get_data(dfield);
     d = (const gint16*)buffer;
     for (i = 0; i < ofile->yres; i++) {
         for (j = 0; j < ofile->xres; j++)
@@ -759,22 +743,22 @@ omicron_read_data(OmicronFile *ofile,
     }
     gwy_file_abandon_contents(buffer, size, NULL);
 
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
-    gwy_si_unit_set_from_string_parse(gwy_data_field_get_si_unit_z(dfield), channel->units, &power10);
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
+    power10 = gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), channel->units);
     if (power10)
-        gwy_data_field_multiply(dfield, pow10(power10));
+        gwy_field_multiply(dfield, gwy_exp10(power10));
 
     return dfield;
 }
 
 static GwySpectra*
-omicron_read_cs_data(OmicronFile *ofile,
-                     OmicronSpectroChannel *channel,
-                     GError **error)
+read_cs_data(OmicronFile *ofile,
+             OmicronSpectroChannel *channel,
+             GError **error)
 {
     GError *err = NULL;
-    GwyDataLine *dline;
-    GwySIUnit *siunit = NULL;
+    GwyLine *dline;
+    GwyUnit *siunit = NULL;
     const gchar *xlabel = NULL, *ylabel = NULL;
     GwySpectra *spectra = NULL;
     GPtrArray *spectrum = NULL;
@@ -788,12 +772,12 @@ omicron_read_cs_data(OmicronFile *ofile,
     gint ncurves = 0;
     gchar* line;
 
-    filename = omicron_fix_file_name(ofile->filename, channel->filename, error);
+    filename = fix_file_name(ofile->filename, channel->filename, error);
     if (!filename)
         return NULL;
 
     gwy_debug("Succeeded with <%s>", filename);
-    if (!g_file_get_contents(filename, &buffer, NULL , &err)) {
+    if (!g_file_get_contents(filename, &buffer, NULL, &err)) {
         g_free(filename);
         err_GET_FILE_CONTENTS(error, &err);
         return NULL;
@@ -848,9 +832,9 @@ omicron_read_cs_data(OmicronFile *ofile,
                             /(channel->npoints - 1));
 
             /* Read spectroscopy points */
-            dline = gwy_data_line_new(channel->npoints, real, FALSE);
-            gwy_data_line_set_offset(dline, (channel->start));
-            data = gwy_data_line_get_data(dline);
+            dline = gwy_line_new(channel->npoints, real, FALSE);
+            gwy_line_set_offset(dline, (channel->start));
+            data = gwy_line_get_data(dline);
             j = 0;
             while ((line = gwy_str_next_line(&buffer))) {
                 gchar *val2;
@@ -869,19 +853,19 @@ omicron_read_cs_data(OmicronFile *ofile,
             /* Set Units for the parameter (x) axis */
             xlabel = channel->param;
             if ((channel->param[0] == 'V') || (channel->param[0] == 'E')) {
-                siunit = gwy_si_unit_new("V");
+                siunit = gwy_unit_new("V");
                 power10 = 0;
             }
             else if (channel->param[0] == 'I')
-                siunit = gwy_si_unit_new_parse("nA", &power10);
+                siunit = gwy_unit_new_parse("nA", &power10);
             else if (channel->param[0] == 'Z')
-                siunit = gwy_si_unit_new_parse("nm", &power10);
+                siunit = gwy_unit_new_parse("nm", &power10);
             else {
                 gwy_debug("Parameter unit not recognised");
             }
 
             if (siunit) {
-                gwy_si_unit_assign(gwy_data_line_get_si_unit_x(dline), siunit);
+                gwy_unit_assign(gwy_line_get_unit_x(dline), siunit);
                 g_object_unref(siunit);
             }
 
@@ -889,19 +873,19 @@ omicron_read_cs_data(OmicronFile *ofile,
                 gdouble offset = 0;
                 gdouble realsize = 0;
 
-                offset = gwy_data_line_get_offset(dline)*pow10(power10);
-                realsize = gwy_data_line_get_real(dline)*pow10(power10);
+                offset = gwy_line_get_offset(dline)*gwy_exp10(power10);
+                realsize = gwy_line_get_real(dline)*gwy_exp10(power10);
 
-                gwy_data_line_set_offset(dline, offset);
-                gwy_data_line_set_real(dline, realsize);
+                gwy_line_set_offset(dline, offset);
+                gwy_line_set_real(dline, realsize);
             }
 
             /* Set Units for the Value (y) Axis */
             ylabel = channel->name;
-            gwy_si_unit_set_from_string_parse(gwy_data_line_get_si_unit_y(dline), channel->units, &power10);
+            power10 = gwy_unit_set_from_string(gwy_line_get_unit_y(dline), channel->units);
 
             if (power10)
-                gwy_data_line_multiply(dline, pow10(power10));
+                gwy_line_multiply(dline, gwy_exp10(power10));
 
             if (!spectrum)
                 spectrum = g_ptr_array_sized_new(ncurves);
@@ -925,7 +909,7 @@ omicron_read_cs_data(OmicronFile *ofile,
         }
     }
     spectra = gwy_spectra_new();
-    gwy_si_unit_set_from_string(gwy_spectra_get_si_unit_xy(spectra), "m");
+    gwy_unit_set_from_string(gwy_spectra_get_unit_xy(spectra), "m");
 
     for (i = 0; i < ncurves; i++) {
         dline = g_ptr_array_index(spectrum, i);
@@ -947,15 +931,15 @@ omicron_read_cs_data(OmicronFile *ofile,
 
 /* Read grid spectra. */
 static GwySpectra*
-omicron_read_be_data(OmicronFile *ofile,
-                     OmicronSpectroChannel *channel,
-                     GError **error)
+read_be_data(OmicronFile *ofile,
+             OmicronSpectroChannel *channel,
+             GError **error)
 {
     gdouble xreal = channel->end - channel->start,
             q = (channel->max_phys - channel->min_phys)/(channel->max_raw - channel->min_raw),
             z0 = channel->min_phys - q*channel->min_raw;
     GError *err = NULL;
-    GwySIUnit *siunit = NULL;
+    GwyUnit *siunit = NULL;
     const gchar *xlabel = NULL, *ylabel = NULL;
     GwySpectra *spectra = NULL;
     gsize size;
@@ -976,7 +960,7 @@ omicron_read_be_data(OmicronFile *ofile,
     gystep = abs(atoi(value));
     gwy_debug("grid: %ux%u with steps %u,%u", gxres, gyres, gxstep, gystep);
 
-    filename = omicron_fix_file_name(ofile->filename, channel->filename, error);
+    filename = fix_file_name(ofile->filename, channel->filename, error);
     if (!filename)
         return NULL;
 
@@ -994,38 +978,35 @@ omicron_read_be_data(OmicronFile *ofile,
     }
 
     spectra = gwy_spectra_new();
-    gwy_si_unit_set_from_string(gwy_spectra_get_si_unit_xy(spectra), "m");
+    gwy_unit_set_from_string(gwy_spectra_get_unit_xy(spectra), "m");
 
     for (i = 0; i < gyres; i++) {
         gdouble y = ofile->yreal*gystep*(gyres-1 - i)/ofile->yres;
         for (j = 0; j < gxres; j++) {
             gdouble x = ofile->xreal*gxstep*j/ofile->xres;
-            GwyDataLine *dline = gwy_data_line_new(channel->npoints, xreal,
-                                                   FALSE);
+            GwyLine *dline = gwy_line_new(channel->npoints, xreal, FALSE);
 
-            gwy_data_line_set_offset(dline, channel->start);
+            gwy_line_set_offset(dline, channel->start);
             gwy_convert_raw_data(buffer + 2*(i*gxres + j), channel->npoints,
-                                 gxres*gyres,
-                                 GWY_RAW_DATA_SINT16,
-                                 GWY_BYTE_ORDER_BIG_ENDIAN,
-                                 gwy_data_line_get_data(dline), q, z0);
+                                 gxres*gyres, GWY_RAW_DATA_SINT16, GWY_BYTE_ORDER_BIG_ENDIAN,
+                                 gwy_line_get_data(dline), q, z0);
 
             /* Set Units for the parameter (x) axis */
             xlabel = channel->param;
             if ((channel->param[0] == 'V') || (channel->param[0] == 'E')) {
-                siunit = gwy_si_unit_new("V");
+                siunit = gwy_unit_new("V");
                 power10 = 0;
             }
             else if (channel->param[0] == 'I')
-                siunit = gwy_si_unit_new_parse("nA", &power10);
+                siunit = gwy_unit_new_parse("nA", &power10);
             else if (channel->param[0] == 'Z')
-                siunit = gwy_si_unit_new_parse("nm", &power10);
+                siunit = gwy_unit_new_parse("nm", &power10);
             else {
                 gwy_debug("Parameter unit not recognised");
             }
 
             if (siunit) {
-                gwy_si_unit_assign(gwy_data_line_get_si_unit_x(dline), siunit);
+                gwy_unit_assign(gwy_line_get_unit_x(dline), siunit);
                 g_object_unref(siunit);
             }
 
@@ -1033,18 +1014,18 @@ omicron_read_be_data(OmicronFile *ofile,
                 gdouble offset = 0;
                 gdouble realsize = 0;
 
-                offset = gwy_data_line_get_offset(dline)*pow10(power10);
-                realsize = gwy_data_line_get_real(dline)*pow10(power10);
+                offset = gwy_line_get_offset(dline)*gwy_exp10(power10);
+                realsize = gwy_line_get_real(dline)*gwy_exp10(power10);
 
-                gwy_data_line_set_offset(dline, offset);
-                gwy_data_line_set_real(dline, realsize);
+                gwy_line_set_offset(dline, offset);
+                gwy_line_set_real(dline, realsize);
             }
 
             /* Set Units for the Value (y) Axis */
             ylabel = channel->name;
-            gwy_si_unit_set_from_string_parse(gwy_data_line_get_si_unit_y(dline), channel->units, &power10);
+            power10 = gwy_unit_set_from_string(gwy_line_get_unit_y(dline), channel->units);
             if (power10)
-                gwy_data_line_multiply(dline, pow10(power10));
+                gwy_line_multiply(dline, gwy_exp10(power10));
 
             gwy_spectra_add_spectrum(spectra, dline, x, y);
             gwy_debug("[%u,%u] %g, %g", j, i, x, y);
@@ -1072,9 +1053,9 @@ add_meta(gpointer key, gpointer value, gpointer user_data)
 }
 
 static GwyContainer*
-omicron_make_meta(OmicronFile *ofile)
+make_meta(OmicronFile *ofile)
 {
-    GwyContainer *meta = gwy_container_new();
+    GwyContainer *meta = gwy_container_new_in_construction();
 
     g_hash_table_foreach(ofile->meta, add_meta, meta);
     if (gwy_container_get_n_items(meta))
@@ -1085,7 +1066,7 @@ omicron_make_meta(OmicronFile *ofile)
 }
 
 static void
-omicron_file_free(OmicronFile *ofile)
+free_omicronfile(OmicronFile *ofile)
 {
     guint i;
 
