@@ -1,6 +1,6 @@
 /*
- *  $Id: opdfile.c 26163 2024-02-06 16:56:55Z yeti-dn $
- *  Copyright (C) 2008-2023 David Necas (Yeti).
+ *  $Id: opdfile.c 28789 2025-11-04 17:14:03Z yeti-dn $
+ *  Copyright (C) 2008-2025 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
  *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
@@ -61,15 +61,10 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/stats.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -130,70 +125,70 @@ typedef struct {
     const guchar *data;
 } OPDBlock;
 
-static gboolean      module_register   (void);
-static gint          opd_detect        (const GwyFileDetectInfo *fileinfo,
-                                        gboolean only_name);
-static gint          opd_asc_detect    (const GwyFileDetectInfo *fileinfo,
-                                        gboolean only_name);
-static GwyContainer* opd_load          (const gchar *filename,
-                                        GwyRunType mode,
-                                        GError **error);
-static GwyContainer* opd_asc_load      (const gchar *filename,
-                                        GwyRunType mode,
-                                        GError **error);
-static void          get_block         (OPDBlock *block,
-                                        const guchar **p);
-static gboolean      get_float         (const OPDBlock *header,
-                                        guint nblocks,
-                                        const gchar *name,
-                                        gdouble *value,
-                                        GError **error);
-static gboolean      get_int16         (const OPDBlock *header,
-                                        guint nblocks,
-                                        const gchar *name,
-                                        gint *value,
-                                        GError **error);
-static GwyDataField* get_data_field    (const OPDBlock *datablock,
-                                        gdouble pixel_size,
-                                        gdouble aspect,
-                                        gdouble wavelength,
-                                        const gchar *zunits,
-                                        GwyDataField **maskfield,
-                                        GError **error);
-static void          store_asc_meta    (gpointer key,
-                                        gpointer value,
-                                        gpointer user_data);
-static GwyDataField* get_asc_data_field(gchar **p,
-                                        OPDArrayFormat format,
-                                        guint xres,
-                                        guint yres,
-                                        gdouble pixel_size,
-                                        gdouble aspect,
-                                        gdouble wavelength,
-                                        const gchar *zunits,
-                                        GwyDataField **maskfield);
-static GwyContainer* get_meta          (const OPDBlock *header,
-                                        guint nblocks);
-static gboolean      check_sizes       (const OPDBlock *header,
-                                        guint nblocks,
-                                        GError **error);
-static guint         find_block        (const OPDBlock *header,
-                                        guint nblocks,
-                                        const gchar *name);
-static const guchar* get_array_params  (const guchar *p,
-                                        guint *xres,
-                                        guint *yres,
-                                        OPDArrayType *type);
-static void          clone_meta        (GwyContainer *container,
-                                        GwyContainer *meta,
-                                        guint nchannels);
+static gboolean      module_register (void);
+static gint          opd_detect      (const GwyFileDetectInfo *fileinfo,
+                                      gboolean only_name);
+static gint          opd_asc_detect  (const GwyFileDetectInfo *fileinfo,
+                                      gboolean only_name);
+static GwyFile*      opd_load        (const gchar *filename,
+                                      GwyRunModeFlags mode,
+                                      GError **error);
+static GwyFile*      opd_asc_load    (const gchar *filename,
+                                      GwyRunModeFlags mode,
+                                      GError **error);
+static void          get_block       (OPDBlock *block,
+                                      const guchar **p);
+static gboolean      get_float       (const OPDBlock *header,
+                                      guint nblocks,
+                                      const gchar *name,
+                                      gdouble *value,
+                                      GError **error);
+static gboolean      get_int16       (const OPDBlock *header,
+                                      guint nblocks,
+                                      const gchar *name,
+                                      gint *value,
+                                      GError **error);
+static GwyField*     get_field       (const OPDBlock *datablock,
+                                      gdouble pixel_size,
+                                      gdouble aspect,
+                                      gdouble wavelength,
+                                      const gchar *zunits,
+                                      GwyField **maskfield,
+                                      GError **error);
+static void          store_asc_meta  (gpointer key,
+                                      gpointer value,
+                                      gpointer user_data);
+static GwyField*     get_asc_field   (gchar **p,
+                                      OPDArrayFormat format,
+                                      guint xres,
+                                      guint yres,
+                                      gdouble pixel_size,
+                                      gdouble aspect,
+                                      gdouble wavelength,
+                                      const gchar *zunits,
+                                      GwyField **maskfield);
+static GwyContainer* get_meta        (const OPDBlock *header,
+                                      guint nblocks);
+static gboolean      check_sizes     (const OPDBlock *header,
+                                      guint nblocks,
+                                      GError **error);
+static guint         find_block      (const OPDBlock *header,
+                                      guint nblocks,
+                                      const gchar *name);
+static const guchar* get_array_params(const guchar *p,
+                                      guint *xres,
+                                      guint *yres,
+                                      OPDArrayType *type);
+static void          clone_meta      (GwyFile *file,
+                                      GwyContainer *meta,
+                                      guint nchannels);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Wyko OPD and ASC files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.12",
+    "0.13",
     "David Nečas (Yeti)",
     "2008",
 };
@@ -205,17 +200,11 @@ module_register(void)
 {
     gwy_file_func_register("opdfile",
                            N_("Wyko OPD files (.opd)"),
-                           (GwyFileDetectFunc)&opd_detect,
-                           (GwyFileLoadFunc)&opd_load,
-                           NULL,
-                           NULL);
+                           &opd_detect, &opd_load, NULL, NULL);
 
     gwy_file_func_register("opdfile-asc",
                            N_("Wyko ASCII export files (.asc)"),
-                           (GwyFileDetectFunc)&opd_asc_detect,
-                           (GwyFileLoadFunc)&opd_asc_load,
-                           NULL,
-                           NULL);
+                           &opd_asc_detect, &opd_asc_load, NULL, NULL);
 
     return TRUE;
 }
@@ -223,8 +212,7 @@ module_register(void)
 /***** Native binary OPD file *********************************************/
 
 static gint
-opd_detect(const GwyFileDetectInfo *fileinfo,
-           gboolean only_name)
+opd_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     if (only_name)
         return g_str_has_suffix(fileinfo->name_lowercase, EXTENSION) ? 10 : 0;
@@ -235,17 +223,17 @@ opd_detect(const GwyFileDetectInfo *fileinfo,
     return 100;
 }
 
-static GwyContainer*
+static GwyFile*
 opd_load(const gchar *filename,
-         G_GNUC_UNUSED GwyRunType mode,
+         G_GNUC_UNUSED GwyRunModeFlags mode,
          GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     guchar *buffer = NULL;
     OPDBlock directory_block, *header = NULL;
     gsize size = 0;
     GError *err = NULL;
-    GwyDataField *dfield = NULL, *mfield = NULL;
+    GwyField *dfield = NULL, *mfield = NULL;
     const guchar *p;
     guint nblocks, offset, i, j, k;
     gdouble pixel_size, wavelength, aspect = 1.0;
@@ -323,13 +311,10 @@ opd_load(const gchar *filename,
     get_float(header, nblocks, "Aspect", &aspect, NULL);
     wavelength /= mult;
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
 
     /* Read the data */
     for (i = j = 0; i < nblocks; i++) {
-        gboolean intensity;
-        gchar *s;
-
         if (!gwy_stramong(header[i].name,
                           "OPD", "SAMPLE_DATA", "RAW_DATA", "RAW DATA", "Image", "Intensity", "SecArr_0", "Raw",
                           NULL))
@@ -341,34 +326,30 @@ opd_load(const gchar *filename,
         }
 
         dfield = mfield = NULL;
-        intensity = gwy_stramong(header[i].name, "Image", "Intensity", "SecArr_0", NULL);
-        if (intensity) {
-            if (!(dfield = get_data_field(header + i, pixel_size, aspect, 1.0, NULL, NULL, error)))
+        gboolean is_intensity = gwy_stramong(header[i].name, "Image", "Intensity", "SecArr_0", NULL);
+        if (is_intensity) {
+            if (!(dfield = get_field(header + i, pixel_size, aspect, 1.0, NULL, NULL, error)))
                 goto fail;
         }
         else {
-            if (!(dfield = get_data_field(header + i, pixel_size, aspect, wavelength, "m", &mfield, error)))
+            if (!(dfield = get_field(header + i, pixel_size, aspect, wavelength, "m", &mfield, error)))
                 goto fail;
         }
 
-        gwy_container_pass_object(container, gwy_app_get_data_key_for_id(j), dfield);
+        gwy_file_pass_image(file, j, dfield);
         if (mfield)
-            gwy_container_pass_object(container, gwy_app_get_mask_key_for_id(j), mfield);
+            gwy_file_pass_image_mask(file, j, mfield);
 
-        s = g_strdup_printf("%s/title", g_quark_to_string(gwy_app_get_data_key_for_id(j)));
-        if (intensity)
-            gwy_container_set_const_string_by_name(container, s, header[i].name);
+        if (is_intensity)
+            gwy_file_set_title(file, GWY_FILE_IMAGE, j, header[i].name, FALSE);
         else if ((k = find_block(header, nblocks, "Title")) != nblocks)
-            gwy_container_set_string_by_name(container, s, g_strndup(header[k].data, header[k].size));
+            gwy_file_pass_title(file, GWY_FILE_IMAGE, j, g_strndup(header[k].data, header[k].size));
         else
-            gwy_app_channel_title_fall_back(container, j);
-        g_free(s);
+            gwy_image_title_fall_back(file, j);
 
-        s = g_strdup_printf("%s/realsquare", g_quark_to_string(gwy_app_get_data_key_for_id(j)));
-        gwy_container_set_boolean_by_name(container, s, TRUE);
-        g_free(s);
+        gwy_container_set_boolean(GWY_CONTAINER(file), gwy_file_key_image_real_square(j), TRUE);
 
-        gwy_file_channel_import_log_add(container, j, NULL, filename);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, j, NULL, filename);
 
         j++;
     }
@@ -376,11 +357,11 @@ opd_load(const gchar *filename,
     if (j) {
         GwyContainer *meta = get_meta(header, nblocks);
 
-        clone_meta(container, meta, j);
+        clone_meta(file, meta, j);
         g_object_unref(meta);
     }
     else {
-        GWY_OBJECT_UNREF(container);
+        g_clear_object(&file);
         err_NO_DATA(error);
     }
 
@@ -388,7 +369,7 @@ fail:
     g_free(header);
     gwy_file_abandon_contents(buffer, size, NULL);
 
-    return container;
+    return file;
 }
 
 static void
@@ -466,15 +447,15 @@ get_int16(const OPDBlock *header,
     return TRUE;
 }
 
-static GwyDataField*
-get_data_field(const OPDBlock *datablock,
-               gdouble pixel_size, gdouble aspect, gdouble wavelength,
-               const gchar *zunits,
-               GwyDataField **maskfield,
-               GError **error)
+static GwyField*
+get_field(const OPDBlock *datablock,
+          gdouble pixel_size, gdouble aspect, gdouble wavelength,
+          const gchar *zunits,
+          GwyField **maskfield,
+          GError **error)
 {
     OPDArrayType datatype;
-    GwyDataField *dfield, *mfield;
+    GwyField *dfield, *mfield;
     gdouble *data, *mdata;
     guint xres, yres, mcount;
     const guchar *p;
@@ -484,16 +465,16 @@ get_data_field(const OPDBlock *datablock,
         *maskfield = NULL;
 
     p = get_array_params(datablock->data, &xres, &yres, &datatype);
-    dfield = gwy_data_field_new(xres, yres, xres*pixel_size, aspect*yres*pixel_size, FALSE);
+    dfield = gwy_field_new(xres, yres, xres*pixel_size, aspect*yres*pixel_size, FALSE);
 
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), zunits);
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
+    gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), zunits);
 
-    mfield = gwy_data_field_new_alike(dfield, FALSE);
-    gwy_data_field_fill(mfield, 1.0);
+    mfield = gwy_field_new_alike(dfield, FALSE);
+    gwy_field_fill(mfield, 1.0);
 
-    data = gwy_data_field_get_data(dfield);
-    mdata = gwy_data_field_get_data(mfield);
+    data = gwy_field_get_data(dfield);
+    mdata = gwy_field_get_data(mfield);
     for (i = 0; i < xres; i++) {
         if (datatype == OPD_ARRAY_FLOAT) {
             for (j = yres; j; j--) {
@@ -540,7 +521,7 @@ static GwyContainer*
 get_meta(const OPDBlock *header,
          guint nblocks)
 {
-    GwyContainer *meta = gwy_container_new();
+    GwyContainer *meta = gwy_container_new_in_construction();
     guint i, type;
     const guchar *p;
     gchar *s;
@@ -664,8 +645,7 @@ find_block(const OPDBlock *header,
 /***** ASCII data *********************************************************/
 
 static gint
-opd_asc_detect(const GwyFileDetectInfo *fileinfo,
-               gboolean only_name)
+opd_asc_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     if (only_name)
         return g_str_has_suffix(fileinfo->name_lowercase, EXTENSION_ASC) ? 10 : 0;
@@ -677,13 +657,13 @@ opd_asc_detect(const GwyFileDetectInfo *fileinfo,
 }
 
 /* FIXME: This is woefuly confusing spaghetti. */
-static GwyContainer*
+static GwyFile*
 opd_asc_load(const gchar *filename,
-             G_GNUC_UNUSED GwyRunType mode,
+             G_GNUC_UNUSED GwyRunModeFlags mode,
              GError **error)
 {
-    GwyContainer *container = NULL;
-    GwyDataField *dfield = NULL, *mfield = NULL;
+    GwyFile *file = NULL;
+    GwyField *dfield = NULL, *mfield = NULL;
     gchar *line, *p, *s, *buffer = NULL;
     GHashTable *hash = NULL;
     guint j, xres = 0, yres = 0;
@@ -715,7 +695,7 @@ opd_asc_load(const gchar *filename,
         goto fail;
     }
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     hash = g_hash_table_new(g_str_hash, g_str_equal);
     j = 0;
     while ((s = line = gwy_str_next_line(&p))) {
@@ -750,7 +730,6 @@ opd_asc_load(const gchar *filename,
                          "OPD", "SAMPLE_DATA", "RAW_DATA", "RAW DATA", "Image", "Intensity", "SecArr_0", "Raw",
                          NULL)) {
             gdouble pixel_size, wavelength, mult = 1.0, aspect = 1.0, zcalib;
-            gboolean intensity;
             gchar *k;
 
             if (!xres) {
@@ -779,41 +758,37 @@ opd_asc_load(const gchar *filename,
             if ((s = g_hash_table_lookup(hash, "Mult")))
                 mult = g_ascii_strtod(s, NULL);
 
-            intensity = gwy_stramong(line, "Image", "Intensity", "SecArr_0", NULL);
+            gboolean is_intensity = gwy_stramong(line, "Image", "Intensity", "SecArr_0", NULL);
 
             pixel_size *= Milimeter;
             zcalib = Nanometer*mult;
-            if (!intensity && !real_units)
+            if (!is_intensity && !real_units)
                 zcalib *= wavelength;
 
             dfield = mfield = NULL;
-            dfield = get_asc_data_field(&p, format, xres, yres, pixel_size, aspect, zcalib,
-                                        intensity ? "" : "m",
-                                        intensity ? NULL : &mfield);
+            dfield = get_asc_field(&p, format, xres, yres, pixel_size, aspect, zcalib,
+                                   is_intensity ? "" : "m",
+                                   is_intensity ? NULL : &mfield);
             if (!dfield) {
                 err_TRUNCATED_PART(error, line);
-                GWY_OBJECT_UNREF(container);
+                g_clear_object(&file);
                 goto fail;
             }
 
-            gwy_container_set_object(container, gwy_app_get_data_key_for_id(j), dfield);
+            gwy_file_set_image(file, j, dfield);
             if (mfield)
-                gwy_container_set_object(container, gwy_app_get_mask_key_for_id(j), mfield);
+                gwy_file_set_image_mask(file, j, mfield);
 
-            s = g_strdup_printf("%s/title", g_quark_to_string(gwy_app_get_data_key_for_id(j)));
-            if (intensity)
-                gwy_container_set_const_string_by_name(container, s, line);
+            if (is_intensity)
+                gwy_file_set_title(file, GWY_FILE_IMAGE, j, line, FALSE);
             else if ((k = g_hash_table_lookup(hash, "Title")))
-                gwy_container_set_const_string_by_name(container, s, k);
+                gwy_file_set_title(file, GWY_FILE_IMAGE, j, k, FALSE);
             else
-                gwy_app_channel_title_fall_back(container, j);
-            g_free(s);
+                gwy_image_title_fall_back(file, j);
 
-            s = g_strdup_printf("%s/realsquare", g_quark_to_string(gwy_app_get_data_key_for_id(j)));
-            gwy_container_set_boolean_by_name(container, s, TRUE);
-            g_free(s);
+            gwy_container_set_boolean(GWY_CONTAINER(file), gwy_file_key_image_real_square(j), TRUE);
 
-            gwy_file_channel_import_log_add(container, j, NULL, filename);
+            gwy_log_add_import(file, GWY_FILE_IMAGE, j, NULL, filename);
 
             j++;
             continue;
@@ -828,31 +803,29 @@ opd_asc_load(const gchar *filename,
     }
 
     if (j) {
-        GwyContainer *meta = gwy_container_new();
+        GwyContainer *meta = gwy_container_new_in_construction();
 
         g_hash_table_foreach(hash, store_asc_meta, meta);
-        clone_meta(container, meta, j);
+        clone_meta(file, meta, j);
         g_object_unref(meta);
     }
     else {
-        GWY_OBJECT_UNREF(container);
+        g_clear_object(&file);
         err_NO_DATA(error);
     }
 
 fail:
-    GWY_OBJECT_UNREF(dfield);
-    GWY_OBJECT_UNREF(mfield);
+    g_clear_object(&dfield);
+    g_clear_object(&mfield);
     g_free(buffer);
     if (hash)
         g_hash_table_destroy(hash);
 
-    return container;
+    return file;
 }
 
 static void
-store_asc_meta(gpointer key,
-               gpointer value,
-               gpointer user_data)
+store_asc_meta(gpointer key, gpointer value, gpointer user_data)
 {
     GwyContainer *meta = (GwyContainer*)user_data;
     gchar *cval;
@@ -862,14 +835,14 @@ store_asc_meta(gpointer key,
     gwy_container_set_string_by_name(meta, key, cval);
 }
 
-static GwyDataField*
-get_asc_data_field(gchar **p,
-                   OPDArrayFormat format, guint xres, guint yres,
-                   gdouble pixel_size, gdouble aspect, gdouble zcalib,
-                   const gchar *zunits,
-                   GwyDataField **maskfield)
+static GwyField*
+get_asc_field(gchar **p,
+              OPDArrayFormat format, guint xres, guint yres,
+              gdouble pixel_size, gdouble aspect, gdouble zcalib,
+              const gchar *zunits,
+              GwyField **maskfield)
 {
-    GwyDataField *dfield, *mfield;
+    GwyField *dfield, *mfield;
     gdouble *data, *mdata;
     const gchar *sprev;
     guint mcount, i, j;
@@ -878,16 +851,16 @@ get_asc_data_field(gchar **p,
     if (maskfield)
         *maskfield = NULL;
 
-    dfield = gwy_data_field_new(xres, yres, xres*pixel_size, aspect*yres*pixel_size, FALSE);
+    dfield = gwy_field_new(xres, yres, xres*pixel_size, aspect*yres*pixel_size, FALSE);
 
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), zunits);
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
+    gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), zunits);
 
-    mfield = gwy_data_field_new_alike(dfield, FALSE);
-    gwy_data_field_fill(mfield, 1.0);
+    mfield = gwy_field_new_alike(dfield, FALSE);
+    gwy_field_fill(mfield, 1.0);
 
-    data = gwy_data_field_get_data(dfield);
-    mdata = gwy_data_field_get_data(mfield);
+    data = gwy_field_get_data(dfield);
+    mdata = gwy_field_get_data(mfield);
 
     if (format == OPD_XYZ_PIXEL || format == OPD_XYZ_REAL) {
         gwy_debug("assuming XYZ format");
@@ -959,23 +932,17 @@ fail:
     return NULL;
 }
 
-/***** Common *************************************************************/
+/***** Common *********************************************************/
 
 static void
-clone_meta(GwyContainer *container, GwyContainer *meta, guint nchannels)
+clone_meta(GwyFile *file, GwyContainer *meta, guint nchannels)
 {
-    guint i;
-    gchar s[32];
-
     if (!gwy_container_get_n_items(meta))
         return;
 
     /* Simply store identical metadata for each channel */
-    for (i = 0; i < nchannels; i++) {
-        GwyContainer *m = gwy_container_duplicate(meta);
-        g_snprintf(s, sizeof(s), "/%u/meta", i);
-        gwy_container_pass_object_by_name(container, s, m);
-    }
+    for (guint i = 0; i < nchannels; i++)
+        gwy_file_pass_meta(file, GWY_FILE_IMAGE, i, gwy_container_copy(meta));
 }
 
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
