@@ -1,6 +1,6 @@
 /*
- *  $Id: opengps.c 26873 2024-11-19 12:52:14Z yeti-dn $
- *  Copyright (C) 2014 David Necas (Yeti).
+ *  $Id: opengps.c 28789 2025-11-04 17:14:03Z yeti-dn $
+ *  Copyright (C) 2014-2025 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
  *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
@@ -34,16 +34,11 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
 #include <limits.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/stats.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 
 #include "err.h"
 #include "gwyzip.h"
@@ -98,39 +93,39 @@ typedef struct {
     gboolean *valid;
 } X3PFile;
 
-static gboolean      module_register       (void);
-static gint          x3p_detect            (const GwyFileDetectInfo *fileinfo,
-                                            gboolean only_name);
-static GwyContainer* x3p_load              (const gchar *filename,
-                                            GwyRunType mode,
-                                            GError **error);
-static void          create_images         (const X3PFile *x3pfile,
-                                            GwyContainer *container);
-static void          create_profiles       (const X3PFile *x3pfile,
-                                            GwyContainer *container);
-static gboolean      x3p_parse_main        (GwyZipFile zipfile,
-                                            X3PFile *x3pfile,
-                                            GError **error);
-static gboolean      data_start            (X3PFile *x3pfile,
-                                            GError **error);
-static gboolean      read_binary_data      (X3PFile *x3pfile,
-                                            GwyZipFile zipfile,
-                                            GError **error);
-static GwyContainer* get_meta              (const X3PFile *x3pfile);
-static void          add_meta_record       (gpointer hkey,
-                                            gpointer hvalue,
-                                            gpointer user_data);
-static void          x3p_file_free         (X3PFile *x3pfile);
-static gboolean      x3p_file_get_data_type(const gchar *type,
-                                            GwyRawDataType *rawtype,
-                                            GError **error);
+static gboolean      module_register (void);
+static gint          detect_file     (const GwyFileDetectInfo *fileinfo,
+                                      gboolean only_name);
+static GwyFile*      load_file       (const gchar *filename,
+                                      GwyRunModeFlags mode,
+                                      GError **error);
+static void          create_images   (const X3PFile *x3pfile,
+                                      GwyFile *file);
+static void          create_profiles (const X3PFile *x3pfile,
+                                      GwyFile *file);
+static gboolean      parse_main      (GwyZipFile zipfile,
+                                      X3PFile *x3pfile,
+                                      GError **error);
+static gboolean      data_start      (X3PFile *x3pfile,
+                                      GError **error);
+static gboolean      read_binary_data(X3PFile *x3pfile,
+                                      GwyZipFile zipfile,
+                                      GError **error);
+static GwyContainer* get_meta        (const X3PFile *x3pfile);
+static void          add_meta_record (gpointer hkey,
+                                      gpointer hvalue,
+                                      gpointer user_data);
+static void          free_x3p_file   (X3PFile *x3pfile);
+static gboolean      get_data_type   (const gchar *type,
+                                      GwyRawDataType *rawtype,
+                                      GError **error);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Reads ISO 5436-2 OpenGPS .x3p files."),
     "Yeti <yeti@gwyddion.net>",
-    "1.5",
+    "2.0",
     "David Nečas (Yeti)",
     "2014",
 };
@@ -142,17 +137,13 @@ module_register(void)
 {
     gwy_file_func_register("opengps",
                            N_("ISO 5436-2 OpenGPS data (.x3p)"),
-                           (GwyFileDetectFunc)&x3p_detect,
-                           (GwyFileLoadFunc)&x3p_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-x3p_detect(const GwyFileDetectInfo *fileinfo,
-           gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     GwyZipFile zipfile;
     guchar *content;
@@ -188,21 +179,21 @@ x3p_detect(const GwyFileDetectInfo *fileinfo,
     return score;
 }
 
-static GwyContainer*
-x3p_load(const gchar *filename,
-         G_GNUC_UNUSED GwyRunType mode,
-         GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     X3PFile x3pfile;
     GwyZipFile zipfile;
 
     if (!(zipfile = gwyzip_open(filename, NULL)))
         return NULL;
 
-    gwy_clear(&x3pfile, 1);
+    gwy_clear1(x3pfile);
     x3pfile.filename = filename;
-    if (!x3p_parse_main(zipfile, &x3pfile, error))
+    if (!parse_main(zipfile, &x3pfile, error))
         goto fail;
 
     if (!x3pfile.ndata) {
@@ -215,11 +206,11 @@ x3p_load(const gchar *filename,
             goto fail;
     }
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     if (x3pfile.feature_type == X3P_FEATURE_SUR)
-        create_images(&x3pfile, container);
+        create_images(&x3pfile, file);
     else if (x3pfile.feature_type == X3P_FEATURE_PRF)
-        create_profiles(&x3pfile, container);
+        create_profiles(&x3pfile, file);
     else {
         g_assert_not_reached();
     }
@@ -227,72 +218,63 @@ x3p_load(const gchar *filename,
 fail:
     gwy_debug("calling gwyzip_close()");
     gwyzip_close(zipfile);
-    x3p_file_free(&x3pfile);
+    free_x3p_file(&x3pfile);
 
-    return container;
+    return file;
 }
 
 static void
-create_images(const X3PFile *x3pfile, GwyContainer *container)
+create_images(const X3PFile *x3pfile, GwyFile *file)
 {
     gint id;
 
     for (id = 0; id < x3pfile->zres; id++) {
-        GwyContainer *meta;
         guint n = x3pfile->xres*x3pfile->yres, k;
-        GwyDataField *dfield, *mask;
         const gboolean *valid = x3pfile->valid + id*n;
-        GQuark quark;
-        gchar buf[40];
-
-        dfield = gwy_data_field_new(x3pfile->xres, x3pfile->yres,
-                                    x3pfile->xres*x3pfile->dx, x3pfile->yres*x3pfile->dy,
-                                    FALSE);
-        gwy_assign(dfield->data, x3pfile->values + id*n, n);
+        GwyField *dfield = gwy_field_new(x3pfile->xres, x3pfile->yres,
+                                         x3pfile->xres*x3pfile->dx, x3pfile->yres*x3pfile->dy,
+                                         FALSE);
+        gdouble *d = gwy_field_get_data(dfield);
+        gwy_assign(d, x3pfile->values + id*n, n);
         for (k = 0; k < n; k++) {
             if (!valid[k])
-                dfield->data[k] = NAN;
+                d[k] = NAN;
         }
 
-        quark = gwy_app_get_data_key_for_id(id);
-        gwy_container_set_object(container, quark, dfield);
+        gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
+        gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), "m");
+        gwy_file_set_image(file, id, dfield);
+        gwy_image_title_fall_back(file, id);
+        gwy_check_nonsquare_image(file, id);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, id, NULL, x3pfile->filename);
 
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), "m");
-        gwy_app_channel_title_fall_back(container, id);
-        gwy_app_channel_check_nonsquare(container, id);
-        gwy_file_channel_import_log_add(container, id, NULL, x3pfile->filename);
-
-        if ((mask = gwy_app_channel_mask_of_nans(dfield, TRUE))) {
-            quark = gwy_app_get_mask_key_for_id(id);
-            gwy_container_pass_object(container, quark, mask);
-        }
+        GwyField *mask;
+        if ((mask = gwy_app_channel_mask_of_nans(dfield, TRUE)))
+            gwy_file_pass_image_mask(file, id, mask);
         g_object_unref(dfield);
 
-        if ((meta = get_meta(x3pfile))) {
-            g_snprintf(buf, sizeof(buf), "/%u/meta", id);
-            gwy_container_pass_object_by_name(container, buf, meta);
-        }
+        GwyContainer *meta;
+        if ((meta = get_meta(x3pfile)))
+            gwy_file_pass_meta(file, GWY_FILE_IMAGE, id, meta);
     }
 }
 
 static void
 create_profiles(const X3PFile *x3pfile,
-                GwyContainer *container)
+                GwyFile *file)
 {
     GwyGraphModel *gmodel;
-    GwySIUnit *siunitx, *siunity;
+    GwyUnit *siunitx, *siunity;
     GArray *validx, *validy;
-    GQuark quark;
     gint id;
 
     gmodel = gwy_graph_model_new();
-    siunitx = gwy_si_unit_new("m");
-    siunity = gwy_si_unit_new("m");
+    siunitx = gwy_unit_new("m");
+    siunity = gwy_unit_new("m");
     g_object_set(gmodel,
                  "title", "Profiles",
-                 "si-unit-x", siunitx,
-                 "si-unit-y", siunity,
+                 "unit-x", siunitx,
+                 "unit-y", siunity,
                  NULL);
     g_object_unref(siunity);
     g_object_unref(siunitx);
@@ -337,8 +319,8 @@ create_profiles(const X3PFile *x3pfile,
     g_array_free(validy, TRUE);
     g_array_free(validx, TRUE);
 
-    quark = gwy_app_get_graph_key_for_id(0);
-    gwy_container_pass_object(container, quark, gmodel);
+    gwy_file_pass_graph(file, 0, gmodel);
+    gwy_log_add_import(file, GWY_FILE_GRAPH, 0, NULL, x3pfile->filename);
 }
 
 static const gchar*
@@ -352,12 +334,12 @@ remove_namespace(const gchar *element_name)
 }
 
 static void
-x3p_start_element(G_GNUC_UNUSED GMarkupParseContext *context,
-                  const gchar *element_name,
-                  G_GNUC_UNUSED const gchar **attribute_names,
-                  G_GNUC_UNUSED const gchar **attribute_values,
-                  gpointer user_data,
-                  GError **error)
+start_element(G_GNUC_UNUSED GMarkupParseContext *context,
+              const gchar *element_name,
+              G_GNUC_UNUSED const gchar **attribute_names,
+              G_GNUC_UNUSED const gchar **attribute_values,
+              gpointer user_data,
+              GError **error)
 {
     X3PFile *x3pfile = (X3PFile*)user_data;
     gchar *path;
@@ -378,10 +360,10 @@ x3p_start_element(G_GNUC_UNUSED GMarkupParseContext *context,
 }
 
 static void
-x3p_end_element(G_GNUC_UNUSED GMarkupParseContext *context,
-                const gchar *element_name,
-                gpointer user_data,
-                GError **error)
+end_element(G_GNUC_UNUSED GMarkupParseContext *context,
+            const gchar *element_name,
+            gpointer user_data,
+            GError **error)
 {
     X3PFile *x3pfile = (X3PFile*)user_data;
     guint n, len = x3pfile->path->len;
@@ -395,7 +377,7 @@ x3p_end_element(G_GNUC_UNUSED GMarkupParseContext *context,
     gwy_debug("%s", path);
 
     /* Invalid data points are represented by empty <Datum>. But then
-     * x3p_text() is not called at all and we must handle that here. */
+     * text() is not called at all and we must handle that here. */
     if (gwy_strequal(path, "/ISO5436_2/Record3/DataList/Datum") && !x3pfile->seen_datum) {
         if (x3pfile->datapos >= x3pfile->ndata) {
             g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
@@ -412,18 +394,18 @@ x3p_end_element(G_GNUC_UNUSED GMarkupParseContext *context,
 }
 
 static void
-x3p_text(G_GNUC_UNUSED GMarkupParseContext *context,
-         const gchar *text,
-         G_GNUC_UNUSED gsize text_len,
-         gpointer user_data,
-         G_GNUC_UNUSED GError **error)
+text(G_GNUC_UNUSED GMarkupParseContext *context,
+     const gchar *value,
+     gsize value_len,
+     gpointer user_data,
+     G_GNUC_UNUSED GError **error)
 {
     X3PFile *x3pfile = (X3PFile*)user_data;
     const gchar *semicolon;
     gchar *path = x3pfile->path->str;
-    gchar *value;
+    gchar *hvalue;
 
-    if (!*text)
+    if (!*value)
         return;
 
     /* Data represented directly in XML. */
@@ -435,10 +417,10 @@ x3p_text(G_GNUC_UNUSED GMarkupParseContext *context,
         }
         /* There can be a ;-separated list.  And it can contain dummy values for the unused y-axis for profiles (poor
          * design).  Since we do not care about PCL, we only ever want the last value of the list, whatever it is. */
-        if ((semicolon = strlenrchr(text, ';', text_len)))
-            text = semicolon + 1;
+        if ((semicolon = strlenrchr(value, ';', value_len)))
+            value = semicolon + 1;
 
-        x3pfile->values[x3pfile->datapos] = x3pfile->dz*g_ascii_strtod(text, NULL) + x3pfile->zoff;
+        x3pfile->values[x3pfile->datapos] = x3pfile->dz*g_ascii_strtod(value, NULL) + x3pfile->zoff;
         x3pfile->valid[x3pfile->datapos] = TRUE;
         gwy_debug("valid Datum %g", x3pfile->values[x3pfile->datapos]);
         x3pfile->datapos++;
@@ -446,20 +428,20 @@ x3p_text(G_GNUC_UNUSED GMarkupParseContext *context,
         return;
     }
 
-    value = g_strdup(text);
-    g_strstrip(value);
-    g_hash_table_replace(x3pfile->hash, g_strdup(path), value);
+    hvalue = g_strdup(value);
+    g_strstrip(hvalue);
+    g_hash_table_replace(x3pfile->hash, g_strdup(path), hvalue);
 }
 
 static gboolean
-x3p_parse_main(GwyZipFile zipfile,
-               X3PFile *x3pfile,
-               GError **error)
+parse_main(GwyZipFile zipfile,
+           X3PFile *x3pfile,
+           GError **error)
 {
     GMarkupParser parser = {
-        &x3p_start_element,
-        &x3p_end_element,
-        &x3p_text,
+        &start_element,
+        &end_element,
+        &text,
         NULL,
         NULL,
     };
@@ -658,7 +640,7 @@ read_binary_data(X3PFile *x3pfile, GwyZipFile zipfile, GError **error)
         return FALSE;
     }
 
-    if (!x3p_file_get_data_type(s, &rawtype, error))
+    if (!get_data_type(s, &rawtype, error))
         return FALSE;
 
     if (!(bindata = gwyzip_get_file_content(zipfile, &size, error)))
@@ -699,7 +681,7 @@ read_binary_data(X3PFile *x3pfile, GwyZipFile zipfile, GError **error)
 static GwyContainer*
 get_meta(const X3PFile *x3pfile)
 {
-    GwyContainer *meta = gwy_container_new();
+    GwyContainer *meta = gwy_container_new_in_construction();
 
     g_hash_table_foreach(x3pfile->hash, add_meta_record, meta);
     if (gwy_container_get_n_items(meta))
@@ -737,11 +719,11 @@ add_meta_record(gpointer hkey, gpointer hvalue, gpointer user_data)
     key = strrchr(key, '/');
     g_return_if_fail(key);
     key++;
-    gwy_container_set_string_by_name(meta, key, (const guchar*)g_strdup(value));
+    gwy_container_set_string_by_name(meta, key, g_strdup(value));
 }
 
 static void
-x3p_file_free(X3PFile *x3pfile)
+free_x3p_file(X3PFile *x3pfile)
 {
     if (x3pfile->hash) {
         g_hash_table_destroy(x3pfile->hash);
@@ -758,9 +740,9 @@ x3p_file_free(X3PFile *x3pfile)
 }
 
 static gboolean
-x3p_file_get_data_type(const gchar *type,
-                       GwyRawDataType *rawtype,
-                       GError **error)
+get_data_type(const gchar *type,
+              GwyRawDataType *rawtype,
+              GError **error)
 {
     if (gwy_strequal(type, "I")) {
         *rawtype = GWY_RAW_DATA_SINT16;
