@@ -1,6 +1,6 @@
 /*
- *  $Id: phoenix.c 26873 2024-11-19 12:52:14Z yeti-dn $
- *  Copyright (C) 2018 David Necas (Yeti).
+ *  $Id: phoenix.c 28910 2025-11-24 18:11:09Z yeti-dn $
+ *  Copyright (C) 2018-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -23,15 +23,11 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
 #include <string.h>
 #include <glib/gstdio.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/stats.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -162,12 +158,12 @@ typedef struct {
 } PhoenixFile;
 
 static gboolean       module_register   (void);
-static gint           phoenix_detect    (const GwyFileDetectInfo *fileinfo,
+static gint           detect_file       (const GwyFileDetectInfo *fileinfo,
                                          gboolean only_name);
-static GwyContainer*  phoenix_load      (const gchar *filename,
-                                         GwyRunType mode,
+static GwyFile*       load_file         (const gchar *filename,
+                                         GwyRunModeFlags mode,
                                          GError **error);
-static void           set_channel_meta  (GwyContainer *data,
+static void           set_channel_meta  (GwyFile *data,
                                          gint id,
                                          const PhoenixFile *phfile,
                                          const PhoenixBinaryHeader *bheader,
@@ -189,7 +185,7 @@ static gboolean       read_binary_header(const PhoenixFile *phfile,
                                          PhoenixBinaryHeader *header,
                                          gsize offset,
                                          GError **error);
-static GwyDataField*  read_data_field   (const PhoenixFile *phfile,
+static GwyField*      read_field        (const PhoenixFile *phfile,
                                          const PhoenixBinaryHeader *header,
                                          gsize offset,
                                          GError **error);
@@ -197,7 +193,7 @@ static gboolean       gather_table_info (const PhoenixFile *phfile,
                                          PhoenixRecord *rec,
                                          PhoenixTableFileInfo *table,
                                          GError **error);
-static GwyDataField*  read_table_file   (const PhoenixTableFileInfo *table,
+static GwyField*      read_table_file   (const PhoenixTableFileInfo *table,
                                          PhoenixAFMHeader *header,
                                          const gchar *lblfilename,
                                          guint dataid,
@@ -214,7 +210,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports AFM data files from NASA Phoenix Mars mission."),
     "Yeti <yeti@gwyddion.net>",
-    "1.1",
+    "2.0",
     "David Nečas (Yeti)",
     "2018",
 };
@@ -226,17 +222,13 @@ module_register(void)
 {
     gwy_file_func_register("phoenix",
                            N_("AFM data from NASA Phoenix mission (.dat, .lbl + .tab)"),
-                           (GwyFileDetectFunc)&phoenix_detect,
-                           (GwyFileLoadFunc)&phoenix_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-phoenix_detect(const GwyFileDetectInfo *fileinfo,
-               gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     const gchar *p;
 
@@ -266,21 +258,21 @@ phoenix_detect(const GwyFileDetectInfo *fileinfo,
     return 80;
 }
 
-static GwyContainer*
-phoenix_load(const gchar *filename,
-             G_GNUC_UNUSED GwyRunType mode,
-             GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
     PhoenixFile phfile;
-    GwyContainer *data = NULL;
+    GwyFile *file = NULL;
     GArray *records;
-    GwyDataField *dfield;
+    GwyField *dfield;
     GError *err = NULL;
     gboolean ok = FALSE;
     gsize offset;
     guint i, j;
 
-    gwy_clear(&phfile, 1);
+    gwy_clear1(phfile);
     if (!g_file_get_contents(filename, &phfile.buffer, &phfile.size, &err)) {
         err_GET_FILE_CONTENTS(error, &err);
         return NULL;
@@ -290,7 +282,7 @@ phoenix_load(const gchar *filename,
     if (!parse_text_header(&phfile, error))
         goto fail;
 
-    data = gwy_container_new();
+    file = gwy_file_new_in_construction();
     if ((offset = phfile.data_offset)) {
         for (i = 0; i < phfile.ndata; i++) {
             PhoenixBinaryHeader header;
@@ -298,15 +290,15 @@ phoenix_load(const gchar *filename,
             if (!read_binary_header(&phfile, &header, offset, error))
                 goto fail;
             offset += BINARY_HEADER_SIZE;
-            if (!(dfield = read_data_field(&phfile, &header, offset, error)))
+            if (!(dfield = read_field(&phfile, &header, offset, error)))
                 goto fail;
             offset += header.data_length;
 
-            gwy_container_pass_object(data, gwy_app_get_data_key_for_id(i), dfield);
+            gwy_file_pass_image(file, i, dfield);
 
-            set_channel_meta(data, i, &phfile, &header, NULL);
+            set_channel_meta(file, i, &phfile, &header, NULL);
         }
-        if (gwy_container_get_n_items(data))
+        if (gwy_container_get_n_items(GWY_CONTAINER(file)))
             ok = TRUE;
         else
             err_NO_DATA(error);
@@ -344,11 +336,11 @@ phoenix_load(const gchar *filename,
             }
 
             /* Skip constant-value images.  They are present for some reason or another but carry no information. */
-            gwy_data_field_get_min_max(dfield, &min, &max);
+            gwy_field_get_min_max(dfield, &min, &max);
             if (max > min) {
-                gwy_container_pass_object(data, gwy_app_get_data_key_for_id(i), dfield);
-                gwy_file_channel_import_log_add(data, i, NULL, filename);
-                set_channel_meta(data, i, &phfile, NULL, &header);
+                gwy_file_pass_image(file, i, dfield);
+                gwy_log_add_import(file, GWY_FILE_IMAGE, i, NULL, filename);
+                set_channel_meta(file, i, &phfile, NULL, &header);
             }
             else {
                 gwy_debug("skipping image #%u filled with constant value %g", j, max);
@@ -361,7 +353,7 @@ phoenix_load(const gchar *filename,
             GWY_FREE(header.oimage_after);
         }
 
-        if (gwy_container_get_n_items(data))
+        if (gwy_container_get_n_items(GWY_CONTAINER(file)))
             ok = TRUE;
         else
             err_NO_DATA(error);
@@ -370,13 +362,13 @@ phoenix_load(const gchar *filename,
 fail:
     phoenix_file_free(&phfile);
     if (!ok)
-        GWY_OBJECT_UNREF(data);
+        g_clear_object(&file);
 
-    return data;
+    return file;
 }
 
 static void
-set_channel_meta(GwyContainer *data, gint id,
+set_channel_meta(GwyFile *file, gint id,
                  const PhoenixFile *phfile,
                  const PhoenixBinaryHeader *bheader,
                  const PhoenixAFMHeader *theader)
@@ -408,8 +400,8 @@ set_channel_meta(GwyContainer *data, gint id,
     else if (direction == PHOENIX_DIRECTION_BACKWARD)
         dir = "Backward";
 
-    gwy_container_set_string(data, gwy_app_get_data_title_key_for_id(id), g_strconcat(chnl, ", ", dir, NULL));
-    gwy_container_pass_object(data, gwy_app_get_data_meta_key_for_id(id), create_meta(phfile, bheader, theader));
+    gwy_file_pass_title(file, GWY_FILE_IMAGE, id, g_strconcat(chnl, ", ", dir, NULL));
+    gwy_file_pass_meta(file, GWY_FILE_IMAGE, id, create_meta(phfile, bheader, theader));
 }
 
 static inline gboolean
@@ -652,11 +644,11 @@ read_binary_header(const PhoenixFile *phfile, PhoenixBinaryHeader *header,
     return TRUE;
 }
 
-static GwyDataField*
-read_data_field(const PhoenixFile *phfile, const PhoenixBinaryHeader *header,
-                gsize offset, GError **error)
+static GwyField*
+read_field(const PhoenixFile *phfile, const PhoenixBinaryHeader *header,
+           gsize offset, GError **error)
 {
-    GwyDataField *dfield;
+    GwyField *dfield;
     const guchar *p;
     guint xres, yres, i;
     gdouble *d, *row;
@@ -676,14 +668,14 @@ read_data_field(const PhoenixFile *phfile, const PhoenixBinaryHeader *header,
     if (err_SIZE_MISMATCH(error, (xres + AFM_LINE_SIZE)*yres, header->data_length, TRUE))
         return FALSE;
 
-    dfield = gwy_data_field_new(xres, yres, xres, yres, FALSE);
+    dfield = gwy_field_new(xres, yres, xres, yres, FALSE);
     zoff = 0.0;
     q = 1.0;
     if (header->channel == PHOENIX_CHANNEL_HEIGHT) {
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), "m");
+        gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), "m");
     }
     else if (header->channel == PHOENIX_CHANNEL_ERROR) {
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), "V");
+        gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), "V");
         /* The range is centered on the setpoint, which needs to be recovered from the commands (not reported back). */
         q = 20.0/255.0;
         zoff = -0.05;
@@ -694,10 +686,10 @@ read_data_field(const PhoenixFile *phfile, const PhoenixBinaryHeader *header,
 
     /* The physical dimensions corresponding to the data grid are not specificied in the telemetry and must be
      * recovered from the command sequence.  So... */
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), NULL);
+    gwy_unit_clear(gwy_field_get_unit_xy(dfield));
 
     p = phfile->buffer + offset;
-    d = gwy_data_field_get_data(dfield);
+    d = gwy_field_get_data(dfield);
     for (i = 0; i < yres; i++) {
         afmline.direction = *(p++);
         afmline.channel_mask = *(p++);
@@ -825,7 +817,7 @@ str_next_value(gchar **p)
     return v;
 }
 
-static GwyDataField*
+static GwyField*
 read_table_file(const PhoenixTableFileInfo *table,
                 PhoenixAFMHeader *header,
                 const gchar *lblfilename,
@@ -833,7 +825,7 @@ read_table_file(const PhoenixTableFileInfo *table,
                 GError **error)
 {
     gchar *buffer, *filename, *dirname, *fullfnm, *p, *line, *q, *v;
-    GwyDataField *dfield = NULL;
+    GwyField *dfield = NULL;
     GError *err = NULL;
     gsize size;
     guint i, lineno;
@@ -889,10 +881,10 @@ read_table_file(const PhoenixTableFileInfo *table,
         if (lineno == dataid+1) {
             if (!parse_afm_d_header(line, header, error))
                 goto fail;
-            dfield = gwy_data_field_new(header->xres, header->yres,
-                                        1e-6*header->x_scan_range, 1e-6*header->y_scan_range,
-                                        FALSE);
-            d = gwy_data_field_get_data(dfield);
+            dfield = gwy_field_new(header->xres, header->yres,
+                                   1e-6*header->x_scan_range, 1e-6*header->y_scan_range,
+                                   FALSE);
+            d = gwy_field_get_data(dfield);
         }
         else if (lineno >= table->lineno
                  && lineno < table->lineno + header->yres) {
@@ -919,23 +911,23 @@ read_table_file(const PhoenixTableFileInfo *table,
         goto fail;
     }
 
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
     if (header->channel == PHOENIX_CHANNEL_HEIGHT) {
         /* Derivative is unitless and the microns mutually cancel. */
         if (!table->is_derivative) {
-            gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), "m");
-            gwy_data_field_multiply(dfield, 1e-6);
+            gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), "m");
+            gwy_field_multiply(dfield, 1e-6);
         }
     }
     else if (header->channel == PHOENIX_CHANNEL_ERROR)
         if (table->is_derivative) {
             /* These are presumably the derivative units and scale.  Who knows,
              * really.*/
-            gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), "V/m");
-            gwy_data_field_multiply(dfield, 1e6);
+            gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), "V/m");
+            gwy_field_multiply(dfield, 1e6);
         }
         else {
-            gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), "V");
+            gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), "V");
         }
     else
         g_warning("Unknown channel type %u.", header->channel);
@@ -945,7 +937,7 @@ read_table_file(const PhoenixTableFileInfo *table,
 
 fail:
     if (!ok)
-        GWY_OBJECT_UNREF(dfield);
+        g_clear_object(&dfield);
     g_free(fullfnm);
     g_free(buffer);
 
@@ -1036,15 +1028,11 @@ create_meta(const PhoenixFile *phfile,
             const PhoenixBinaryHeader *bheader,
             const PhoenixAFMHeader *theader)
 {
-    GwyContainer *meta;
-    GArray *records;
-    PhoenixRecord *rec;
-    guint i;
+    GwyContainer *meta = gwy_container_new_in_construction();
+    GArray *records = phfile->records;
 
-    meta = gwy_container_new();
-    records = phfile->records;
-    for (i = 0; i < records->len; i++) {
-        rec = &g_array_index(records, PhoenixRecord, i);
+    for (guint i = 0; i < records->len; i++) {
+        PhoenixRecord *rec = &g_array_index(records, PhoenixRecord, i);
         /* Skip objects and pointers. */
         if (rec->records || !rec->name || rec->name[0] == '^')
             continue;
