@@ -1,6 +1,6 @@
 /*
- *  $Id: pixmap.c 25636 2023-09-07 12:19:33Z yeti-dn $
- *  Copyright (C) 2004-2022 David Necas (Yeti).
+ *  $Id: pixmap.c 28860 2025-11-14 12:59:38Z yeti-dn $
+ *  Copyright (C) 2004-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -30,22 +30,15 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <errno.h>
 #include <math.h>
 #include <stdarg.h>
 #include <gdk/gdk.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwythreads.h>
-#include <libdraw/gwypixfield.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <libprocess/stats.h>
-#include <libgwydgets/gwydgets.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
+#include "libgwyapp/sanity.h"
 #include "err.h"
 #include "gwytiff.h"
 
@@ -100,7 +93,7 @@ typedef struct {
     GwyParamTable *table_lateral;
     GwyParamTable *table_values;
     GdkPixbuf *small_pixbuf;
-    GwyContainer *data;
+    GwyField *field;
 } ModuleGUI;
 
 typedef gboolean (*PixmapFilterFunc)(const GwyFileDetectInfo *fileinfo);
@@ -109,7 +102,7 @@ typedef gboolean (*PixmapFilterFunc)(const GwyFileDetectInfo *fileinfo);
 typedef struct {
     const gchar *name;
     const gchar *description;
-    PixmapFilterFunc filter_func;
+    GwyFileDetectFunc detect_file;
 } PixmapKnownFormat;
 
 /* Actually registered formats. */
@@ -117,26 +110,52 @@ typedef struct {
     const gchar *name;
     const gchar *description;
     const GdkPixbufFormat *pixbuf_format;
-    PixmapFilterFunc filter_func;
+    GwyFileDetectFunc detect_file;
 } PixmapFormatInfo;
 
 static gboolean          module_register       (void);
-static gint              pixmap_detect         (const GwyFileDetectInfo *fileinfo,
-                                                gboolean only_name,
-                                                const gchar *name);
-static GwyContainer*     pixmap_load           (const gchar *filename,
-                                                GwyRunType runtype,
-                                                GError **error,
-                                                const gchar *name);
+static gint              detect_png            (const GwyFileDetectInfo *fileinfo,
+                                                gboolean only_name);
+static gint              detect_jpeg           (const GwyFileDetectInfo *fileinfo,
+                                                gboolean only_name);
+static gint              detect_tiff           (const GwyFileDetectInfo *fileinfo,
+                                                gboolean only_name);
+static gint              detect_pnm            (const GwyFileDetectInfo *fileinfo,
+                                                gboolean only_name);
+static gint              detect_bmp            (const GwyFileDetectInfo *fileinfo,
+                                                gboolean only_name);
+static gint              detect_tga            (const GwyFileDetectInfo *fileinfo,
+                                                gboolean only_name);
+static gint              detect_gif            (const GwyFileDetectInfo *fileinfo,
+                                                gboolean only_name);
+static gint              detect_jpeg2000       (const GwyFileDetectInfo *fileinfo,
+                                                gboolean only_name);
+static gint              detect_jpegxl         (const GwyFileDetectInfo *fileinfo,
+                                                gboolean only_name);
+static gint              detect_pcx            (const GwyFileDetectInfo *fileinfo,
+                                                gboolean only_name);
+static gint              detect_xpm            (const GwyFileDetectInfo *fileinfo,
+                                                gboolean only_name);
+static gint              detect_ras            (const GwyFileDetectInfo *fileinfo,
+                                                gboolean only_name);
+static gint              detect_icns           (const GwyFileDetectInfo *fileinfo,
+                                                gboolean only_name);
+static gint              detect_webp           (const GwyFileDetectInfo *fileinfo,
+                                                gboolean only_name);
+static gint              detect_heif           (const GwyFileDetectInfo *fileinfo,
+                                                gboolean only_name);
+static GwyFile*          load_file             (const gchar *filename,
+                                                GwyRunModeFlags mode,
+                                                GError **error);
 static GdkPixbuf*        pixmap_load_pixbuf    (const gchar *filename,
                                                 const gchar *name,
                                                 GError **error);
-static void              pixmap_set_field      (GwyContainer *container,
+static void              pixmap_set_field      (GwyFile *file,
                                                 gint id,
                                                 ModuleArgs *args,
                                                 PixmapMapType maptype);
 static void              pixmap_pixbuf_to_field(GdkPixbuf *pixbuf,
-                                                GwyDataField *field,
+                                                GwyField *field,
                                                 PixmapMapType maptype,
                                                 gdouble hue_offset);
 static GwyDialogOutcome  run_gui               (ModuleArgs *args,
@@ -146,29 +165,13 @@ static void              param_changed         (ModuleGUI *gui,
 static void              preview               (gpointer user_data);
 static gboolean          mapping_type_filter   (const GwyEnum *enumval,
                                                 gpointer user_data);
-static void              pixmap_add_import_log (GwyContainer *data,
+static void              pixmap_add_import_log (GwyFile *data,
                                                 gint id,
                                                 const gchar *filetype,
                                                 const gchar *filename);
 static PixmapFormatInfo* find_format           (const gchar *name);
 static void              sanitise_params       (ModuleArgs *args);
 
-static gboolean pixmap_filter_png     (const GwyFileDetectInfo *fileinfo);
-static gboolean pixmap_filter_jpeg    (const GwyFileDetectInfo *fileinfo);
-static gboolean pixmap_filter_tiff    (const GwyFileDetectInfo *fileinfo);
-static gboolean pixmap_filter_pnm     (const GwyFileDetectInfo *fileinfo);
-static gboolean pixmap_filter_bmp     (const GwyFileDetectInfo *fileinfo);
-static gboolean pixmap_filter_tga     (const GwyFileDetectInfo *fileinfo);
-static gboolean pixmap_filter_gif     (const GwyFileDetectInfo *fileinfo);
-static gboolean pixmap_filter_jpeg2000(const GwyFileDetectInfo *fileinfo);
-static gboolean pixmap_filter_jpegxl  (const GwyFileDetectInfo *fileinfo);
-static gboolean pixmap_filter_pcx     (const GwyFileDetectInfo *fileinfo);
-static gboolean pixmap_filter_xpm     (const GwyFileDetectInfo *fileinfo);
-static gboolean pixmap_filter_ras     (const GwyFileDetectInfo *fileinfo);
-static gboolean pixmap_filter_icns    (const GwyFileDetectInfo *fileinfo);
-static gboolean pixmap_filter_webp    (const GwyFileDetectInfo *fileinfo);
-static gboolean pixmap_filter_heif    (const GwyFileDetectInfo *fileinfo);
-
 static const GwyEnum map_types[PIXMAP_MAP_NTYPES] = {
     { N_("All channels"), PIXMAP_MAP_ALL,   },
     { N_("Red"),          PIXMAP_MAP_RED,   },
@@ -185,25 +188,25 @@ static const GwyEnum map_types[PIXMAP_MAP_NTYPES] = {
 /* Use a whitelist of safe formats for which we have at least basic weed-out pre-detection function.  GdkPixbuf
  * loaders tend to accept any rubbish as their format and then crash completely surprised when it isn't. */
 static const PixmapKnownFormat known_formats[] = {
-    { "png",      N_("Portable Network Graphics (.png)"),          pixmap_filter_png,      },
-    { "jpeg",     N_("JPEG (.jpeg,.jpg)"),                         pixmap_filter_jpeg,     },
+    { "png",      N_("Portable Network Graphics (.png)"),          detect_png,      },
+    { "jpeg",     N_("JPEG (.jpeg,.jpg)"),                         detect_jpeg,     },
 #ifndef __WIN64
     /* The TIFF loader (supposedly GDI-based) crashes on Win64.  Unclear why.  TIFF is madness.  There is a fallback
      * GwyTIFF-based loader in hdrimage which will take over when we exclude the GdkPixbuf-based one.  */
-    { "tiff",     N_("TIFF (.tiff,.tif)"),                         pixmap_filter_tiff,     },
+    { "tiff",     N_("TIFF (.tiff,.tif)"),                         detect_tiff,     },
 #endif
-    { "pnm",      N_("Portable Pixmap (.ppm,.pnm)"),               pixmap_filter_pnm,      },
-    { "bmp",      N_("Windows or OS2 Bitmap (.bmp)"),              pixmap_filter_bmp,      },
-    { "tga",      N_("TARGA (.tga,.targa)"),                       pixmap_filter_tga,      },
-    { "gif",      N_("Graphics Interchange Format GIF (.gif)"),    pixmap_filter_gif,      },
-    { "jpeg2000", N_("JPEG 2000 (.jpx)"),                          pixmap_filter_jpeg2000, },
-    { "jxl",      N_("JPEG XL (.jxl)"),                            pixmap_filter_jpegxl,   },
-    { "pcx",      N_("PCX (.pcx)"),                                pixmap_filter_pcx,      },
-    { "xpm",      N_("X Pixmap (.xpm)"),                           pixmap_filter_xpm,      },
-    { "ras",      N_("Sun raster image (.ras)"),                   pixmap_filter_ras,      },
-    { "icns",     N_("Apple icon (.icns)"),                        pixmap_filter_icns,     },
-    { "webp",     N_("WebP (.webp)"),                              pixmap_filter_webp,     },
-    { "heif",     N_("High Efficiency Image File Format (.heif)"), pixmap_filter_heif,     },
+    { "pnm",      N_("Portable Pixmap (.ppm,.pnm)"),               detect_pnm,      },
+    { "bmp",      N_("Windows or OS2 Bitmap (.bmp)"),              detect_bmp,      },
+    { "tga",      N_("TARGA (.tga,.targa)"),                       detect_tga,      },
+    { "gif",      N_("Graphics Interchange Format GIF (.gif)"),    detect_gif,      },
+    { "jpeg2000", N_("JPEG 2000 (.jpx)"),                          detect_jpeg2000, },
+    { "jxl",      N_("JPEG XL (.jxl)"),                            detect_jpegxl,   },
+    { "pcx",      N_("PCX (.pcx)"),                                detect_pcx,      },
+    { "xpm",      N_("X Pixmap (.xpm)"),                           detect_xpm,      },
+    { "ras",      N_("Sun raster image (.ras)"),                   detect_ras,      },
+    { "icns",     N_("Apple icon (.icns)"),                        detect_icns,     },
+    { "webp",     N_("WebP (.webp)"),                              detect_webp,     },
+    { "heif",     N_("High Efficiency Image File Format (.heif)"), detect_heif,     },
 };
 
 /* List of PixmapFormatInfo for all formats.  Created in module_register() and never freed. */
@@ -215,12 +218,12 @@ static GwyModuleInfo module_info = {
     N_("Imports data from low-depth pixmap images (PNG, TIFF, JPEG, ...).  The set of available formats depends on "
        "available GDK pixbuf loaders."),
     "Yeti <yeti@gwyddion.net>",
-    "10.2",
+    "11.0",
     "David Nečas (Yeti)",
     "2004-2014",
 };
 
-GWY_MODULE_QUERY(module_info)
+GWY_MODULE_QUERY2(module_info, pixmap)
 
 static gboolean
 module_register(void)
@@ -257,10 +260,11 @@ module_register(void)
         format_info->pixbuf_format = pixbuf_format;
         /* Copy from static data. */
         format_info->description = known_format->description;
-        format_info->filter_func = known_format->filter_func;
+        format_info->detect_file = known_format->detect_file;
 
         gwy_debug("Found GdkPixbuf loader for new type: %s", fmtname);
-        gwy_file_func_register(format_info->name, format_info->description, &pixmap_detect, &pixmap_load, NULL, NULL);
+        gwy_file_func_register(format_info->name, format_info->description,
+                               format_info->detect_file, load_file, NULL, NULL);
         pixmap_formats = g_slist_append(pixmap_formats, format_info);
     }
 
@@ -295,57 +299,25 @@ define_module_params(void)
 }
 
 static gint
-pixmap_detect(const GwyFileDetectInfo *fileinfo,
-              gboolean only_name,
-              const gchar *name)
+pixbuf_detect_try_loader(const GwyFileDetectInfo *fileinfo, const gchar *loadername, gint score)
 {
-    GdkPixbufLoader *loader;
+#ifdef DEBUG
     GError *err = NULL;
-    PixmapFormatInfo *format_info;
-    const gchar *loadername;
-    gint score;
+    GError **perr = &err;
+#else
+    GError **perr = NULL;
+#endif
 
-    if (only_name)
-        return 0;
-
-    gwy_debug("Running detection for file type %s", name);
-
-    format_info = find_format(name);
-    g_return_val_if_fail(format_info, 0);
-
-    /* This is not really correct, but no one is going to import data from such a small valid image anyway. */
-    if (fileinfo->buffer_len < 32)
-        return 0;
-
-    /* GdkPixbuf does a terrible job regarding detection so we do some sanity check ourselves */
-    if (!format_info->filter_func(fileinfo))
-        return 0;
-
-    loadername = (gwy_strequal(name, "heif") ? "heif/avif" : name);
-    gwy_debug("Creating a loader for type %s", loadername);
-    loader = gdk_pixbuf_loader_new_with_type(loadername, NULL);
+    GdkPixbufLoader *loader = gdk_pixbuf_loader_new_with_type(loadername, NULL);
     gwy_debug("Loader for type %s: %p", loadername, loader);
     if (!loader)
         return 0;
 
-    /* The TIFF loaders (both libTIFF and GDI-based) seem to crash on broken TIFFs a way too often.  Do not try to
-     * feed anything to it just accept the file is a TIFF and hope some other loader of a TIFF-based format will claim
-     * it with a higher score. */
-    score = 70;
-    if (gwy_strequal(loadername, "tiff")) {
-        gwy_debug("Avoiding feeding data to TIFF loader, calling gdk_pixbuf_loader_close().");
-        gdk_pixbuf_loader_close(loader, NULL);
-        gwy_debug("Unreferencing the TIFF loader");
-        g_object_unref(loader);
-        gwy_debug("Returning score %d for TIFF", score - 10);
-        return score - 10;
-    }
-
-    /* For sane readers, try to feed the start of the file and see if it fails.
-     * Success rarely means anything though. */
-    if (!gdk_pixbuf_loader_write(loader, fileinfo->head, fileinfo->buffer_len, &err)) {
+    if (!gdk_pixbuf_loader_write(loader, fileinfo->head, fileinfo->buffer_len, perr)) {
+#ifdef DEBUG
         gwy_debug("%s", err->message);
         g_clear_error(&err);
+#endif
         score = 0;
     }
     gdk_pixbuf_loader_close(loader, NULL);
@@ -354,16 +326,15 @@ pixmap_detect(const GwyFileDetectInfo *fileinfo,
     return score;
 }
 
-static GwyContainer*
-pixmap_load(const gchar *filename,
-            GwyRunType runtype,
-            GError **error,
-            const gchar *name)
+static GwyFile*
+load_file(const gchar *filename,
+          GwyRunModeFlags mode,
+          GError **error)
 {
     static const PixmapMapType rgbtypes[] = { PIXMAP_MAP_RED, PIXMAP_MAP_GREEN, PIXMAP_MAP_BLUE, PIXMAP_MAP_ALPHA };
     static const PixmapMapType greytypes[] = { PIXMAP_MAP_GREY, PIXMAP_MAP_ALPHA };
 
-    GwyContainer *data = NULL;
+    GwyFile *data = NULL;
     GdkPixbuf *pixbuf;
     guchar *pixels, *p;
     gint bpp, i, j, width, height, rowstride, nimages;
@@ -373,8 +344,9 @@ pixmap_load(const gchar *filename,
     const PixmapMapType *imgtypes;
     GwyDialogOutcome outcome;
     ModuleArgs args;
+    const gchar *name = gwy_file_func_current();
 
-    gwy_clear(&args, 1);
+    gwy_clear1(args);
     if (!(pixbuf = pixmap_load_pixbuf(filename, name, error)))
         return NULL;
     args.pixbuf = pixbuf;
@@ -403,7 +375,7 @@ pixmap_load(const gchar *filename,
     args.params = params = gwy_params_new_from_settings(define_module_params());
     sanitise_params(&args);
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, name);
         gwy_params_save_to_settings(params);
         if (outcome == GWY_DIALOG_CANCEL) {
@@ -412,7 +384,7 @@ pixmap_load(const gchar *filename,
         }
     }
 
-    data = gwy_container_new();
+    data = gwy_file_new_in_construction();
     maptype = gwy_params_get_enum(params, PARAM_MAP_TYPE);
     if (maptype == PIXMAP_MAP_ALL) {
         if (args.flags & PIXMAP_HAS_COLOURS) {
@@ -519,34 +491,32 @@ pixmap_load_pixbuf(const gchar *filename,
 }
 
 static void
-pixmap_set_field(GwyContainer *container,
+pixmap_set_field(GwyFile *file,
                  gint id,
                  ModuleArgs *args,
                  PixmapMapType maptype)
 {
     GwyParams *params = args->params;
     GdkPixbuf *pixbuf = args->pixbuf;
-    GwyDataField *field;
+    GwyField *field;
     gint power10xy, power10z;
 
-    field = gwy_data_field_new(gdk_pixbuf_get_width(pixbuf), gdk_pixbuf_get_height(pixbuf), 1.0, 1.0, FALSE);
+    field = gwy_field_new(gdk_pixbuf_get_width(pixbuf), gdk_pixbuf_get_height(pixbuf), 1.0, 1.0, FALSE);
     pixmap_pixbuf_to_field(pixbuf, field, maptype, gwy_params_get_double(params, PARAM_HUE_OFFSET)/6.0);
 
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(field), gwy_params_get_unit(params, PARAM_XYUNIT, &power10xy));
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_z(field), gwy_params_get_unit(params, PARAM_ZUNIT, &power10z));
-    gwy_data_field_set_xreal(field, gwy_params_get_double(params, PARAM_XREAL)*pow10(power10xy));
-    gwy_data_field_set_yreal(field, gwy_params_get_double(params, PARAM_YREAL)*pow10(power10xy));
-    gwy_data_field_multiply(field, gwy_params_get_double(params, PARAM_ZREAL)*pow10(power10z));
+    gwy_unit_assign(gwy_field_get_unit_xy(field), gwy_params_get_unit(params, PARAM_XYUNIT, &power10xy));
+    gwy_unit_assign(gwy_field_get_unit_z(field), gwy_params_get_unit(params, PARAM_ZUNIT, &power10z));
+    gwy_field_set_xreal(field, gwy_params_get_double(params, PARAM_XREAL)*gwy_exp10(power10xy));
+    gwy_field_set_yreal(field, gwy_params_get_double(params, PARAM_YREAL)*gwy_exp10(power10xy));
+    gwy_field_multiply(field, gwy_params_get_double(params, PARAM_ZREAL)*gwy_exp10(power10z));
 
-    gwy_container_pass_object(container, gwy_app_get_data_key_for_id(id), field);
-    gwy_container_set_const_string(container,
-                                   gwy_app_get_data_title_key_for_id(id),
-                                   gwy_enum_to_string(maptype, map_types, PIXMAP_MAP_NTYPES));
+    gwy_file_pass_image(file, id, field);
+    gwy_file_set_title(file, GWY_FILE_IMAGE, id, gwy_enum_to_string(maptype, map_types, PIXMAP_MAP_NTYPES), FALSE);
 }
 
 static void
 pixmap_pixbuf_to_field(GdkPixbuf *pixbuf,
-                       GwyDataField *field,
+                       GwyField *field,
                        PixmapMapType maptype,
                        gdouble hue_offset)
 {
@@ -560,8 +530,8 @@ pixmap_pixbuf_to_field(GdkPixbuf *pixbuf,
     height = gdk_pixbuf_get_height(pixbuf);
     rowstride = gdk_pixbuf_get_rowstride(pixbuf);
     bpp = gdk_pixbuf_get_has_alpha(pixbuf) ? 4 : 3;
-    gwy_data_field_resample(field, width, height, GWY_INTERPOLATION_NONE);
-    val = gwy_data_field_get_data(field);
+    gwy_field_resize(field, width, height);
+    val = gwy_field_get_data(field);
 
 #ifdef _OPENMP
 #pragma omp parallel for if (gwy_threads_are_enabled()) default(none) \
@@ -654,16 +624,15 @@ run_gui(ModuleArgs *args, const gchar *name)
 {
     ModuleGUI gui;
     GwyParamTable *table, *infotable;
-    GtkWidget *align, *hbox, *view, *label;
+    GtkWidget *hbox, *view, *label;
     GwyDialog *dialog;
     GwyDialogOutcome outcome;
     GwyResults *results;
-    GwyDataField *field;
     gint xres, yres, sxres, syres;
     gchar *s, *title;
     gdouble zoom;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
 
     xres = gdk_pixbuf_get_width(args->pixbuf);
@@ -673,10 +642,7 @@ run_gui(ModuleArgs *args, const gchar *name)
     sxres = MAX(GWY_ROUND(zoom*xres), 1);
     syres = MAX(GWY_ROUND(zoom*yres), 1);
     gui.small_pixbuf = gdk_pixbuf_scale_simple(args->pixbuf, sxres, syres, GDK_INTERP_TILES);
-
-    gui.data = gwy_container_new();
-    field = gwy_data_field_new(sxres, syres, sxres, syres, TRUE);
-    gwy_container_pass_object(gui.data, gwy_app_get_data_key_for_id(0), field);
+    gui.field = gwy_field_new(sxres, syres, sxres, syres, TRUE);
 
     s = g_ascii_strup(name, -1);
     /* TRANSLATORS: Dialog title; %s is PNG, TIFF, ... */
@@ -687,11 +653,9 @@ run_gui(ModuleArgs *args, const gchar *name)
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    hbox = gwy_hbox_new(20);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 20);
     gwy_dialog_add_content(dialog, hbox, FALSE, FALSE, 0);
 
-    align = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-    gtk_box_pack_start(GTK_BOX(hbox), align, TRUE, TRUE, 0);
 
     results = gwy_results_new();
     gwy_results_add_value(results, "xres", N_("Horizontal size"),
@@ -711,15 +675,14 @@ run_gui(ModuleArgs *args, const gchar *name)
     /* TODO: If the file contains resultion/size in physical units, show it here. */
     gwy_param_table_results_fill(infotable, WIDGET_IMAGE_INFO);
     gwy_dialog_add_param_table(dialog, infotable);
-    gtk_container_add(GTK_CONTAINER(align), gwy_param_table_widget(infotable));
+    gtk_box_pack_start(GTK_BOX(hbox), gwy_param_table_widget(infotable), TRUE, TRUE, 0);
 
-    align = gtk_alignment_new(1.0, 0.0, 0.0, 0.0);
-    gtk_box_pack_start(GTK_BOX(hbox), align, TRUE, TRUE, 0);
+    view = gwy_create_preview(gui.field, NULL, PREVIEW_SIZE);
+    gtk_widget_set_halign(view, GTK_ALIGN_START);
+    gtk_widget_set_valign(view, GTK_ALIGN_START);
+    gtk_box_pack_start(GTK_BOX(hbox), view, TRUE, TRUE, 0);
 
-    view = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
-    gtk_container_add(GTK_CONTAINER(align), view);
-
-    hbox = gwy_hbox_new(20);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 20);
     gwy_dialog_add_content(dialog, hbox, TRUE, TRUE, 0);
 
     table = gui.table_lateral = gwy_param_table_new(args->params);
@@ -747,8 +710,8 @@ run_gui(ModuleArgs *args, const gchar *name)
 
     if (args->flags & PIXMAP_HAS_COLOURS) {
         label = gtk_label_new(_("Warning: Colorful images cannot be reliably mapped to meaningful values."));
-        gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
-        gtk_misc_set_padding(GTK_MISC(label), 4, 6);
+        gtk_label_set_xalign(GTK_LABEL(label), 0.0);
+        gwy_set_widget_padding(label, 4, 4, 6, 6);
         gwy_dialog_add_content(dialog, label, FALSE, FALSE, 0);
     }
 
@@ -760,7 +723,7 @@ run_gui(ModuleArgs *args, const gchar *name)
     outcome = gwy_dialog_run(dialog);
 
     g_object_unref(gui.small_pixbuf);
-    g_object_unref(gui.data);
+    g_object_unref(gui.field);
 
     return outcome;
 }
@@ -771,13 +734,12 @@ preview(gpointer user_data)
     ModuleGUI *gui = (ModuleGUI*)user_data;
     ModuleArgs *args = gui->args;
     GwyParams *params = args->params;
-    GwyDataField *field = gwy_container_get_object(gui->data, gwy_app_get_data_key_for_id(0));
     PixmapMapType maptype = gwy_params_get_enum(params, PARAM_MAP_TYPE);
 
     if (maptype == PIXMAP_MAP_ALL)
         maptype = (args->flags & PIXMAP_HAS_COLOURS) ? PIXMAP_MAP_RED : PIXMAP_MAP_GREY;
-    pixmap_pixbuf_to_field(gui->small_pixbuf, field, maptype, gwy_params_get_double(params, PARAM_HUE_OFFSET)/6.0);
-    gwy_data_field_data_changed(field);
+    pixmap_pixbuf_to_field(gui->small_pixbuf, gui->field, maptype, gwy_params_get_double(params, PARAM_HUE_OFFSET)/6.0);
+    gwy_field_data_changed(gui->field);
 }
 
 static void
@@ -790,9 +752,9 @@ param_changed(ModuleGUI *gui, gint id)
     gboolean xymeasureeq = gwy_params_get_boolean(params, PARAM_XYMEASUREEQ);
     gint xres = gdk_pixbuf_get_width(args->pixbuf);
     gint yres = gdk_pixbuf_get_height(args->pixbuf);
-    GwySIUnit *unit;
+    GwyUnit *unit;
     gint power10;
-    GwySIValueFormat *vf = NULL;
+    GwyValueFormat *vf = NULL;
 
     if (id < 0 || id == PARAM_MAP_TYPE)
         gwy_param_table_set_sensitive(gui->table_values, PARAM_HUE_OFFSET, maptype == PIXMAP_MAP_HUE);
@@ -824,18 +786,18 @@ param_changed(ModuleGUI *gui, gint id)
 
     if (id < 0 || id == PARAM_XYUNIT) {
         unit = gwy_params_get_unit(params, PARAM_XYUNIT, &power10);
-        vf = gwy_si_unit_get_format_for_power10(unit, GWY_SI_UNIT_FORMAT_VFMARKUP, power10, vf);
+        vf = gwy_unit_get_format_for_power10(unit, GWY_UNIT_FORMAT_VFMARKUP, power10, vf);
         gwy_param_table_set_unitstr(gui->table_lateral, PARAM_XREAL, vf->units);
         gwy_param_table_set_unitstr(gui->table_lateral, PARAM_YREAL, vf->units);
     }
 
     if (id < 0 || id == PARAM_ZUNIT) {
         unit = gwy_params_get_unit(params, PARAM_ZUNIT, &power10);
-        vf = gwy_si_unit_get_format_for_power10(unit, GWY_SI_UNIT_FORMAT_VFMARKUP, power10, vf);
+        vf = gwy_unit_get_format_for_power10(unit, GWY_UNIT_FORMAT_VFMARKUP, power10, vf);
         gwy_param_table_set_unitstr(gui->table_values, PARAM_ZREAL, vf->units);
     }
 
-    GWY_SI_VALUE_FORMAT_FREE(vf);
+    GWY_FREE_VALUE_FORMAT(vf);
 
     if (id < 0 || id == PARAM_MAP_TYPE || id == PARAM_HUE_OFFSET)
         gwy_dialog_invalidate(GWY_DIALOG(gui->dialog));
@@ -856,18 +818,17 @@ mapping_type_filter(const GwyEnum *enumval, gpointer user_data)
 }
 
 static void
-pixmap_add_import_log(GwyContainer *data,
+pixmap_add_import_log(GwyFile *file,
                       gint id,
                       const gchar *filetype,
                       const gchar *filename)
 {
-    GwyContainer *settings;
     GQuark quark;
     gchar *myfilename = NULL, *fskey, *qualname;
 
     g_return_if_fail(filename);
     g_return_if_fail(filetype);
-    g_return_if_fail(data);
+    g_return_if_fail(file);
 
     if (g_utf8_validate(filename, -1, NULL))
         myfilename = g_strdup(filename);
@@ -881,11 +842,11 @@ pixmap_add_import_log(GwyContainer *data,
     g_free(fskey);
 
     /* Eats myfilename. */
-    settings = gwy_app_settings_get();
+    GwyContainer *settings = gwy_app_settings_get();
     gwy_container_set_string(settings, quark, myfilename);
 
     qualname = g_strconcat("file::", filetype, NULL);
-    gwy_app_channel_log_add(data, -1, id, qualname, NULL);
+    gwy_log_add_full(file, GWY_FILE_IMAGE, -1, id, qualname, NULL);
     g_free(qualname);
 
     /* We know pixmap functions have no such setting as "filename". */
@@ -939,133 +900,178 @@ sanitise_params(ModuleArgs *args)
     g_assert_not_reached();
 }
 
-static gboolean
-pixmap_filter_png(const GwyFileDetectInfo *fileinfo)
+/* Gdk-pixbuf is crap at detecting file types. And creating loaders is a big overhead.
+ *
+ * Use quick filters to get rid of most files which are clearly not in the right format. Then try to feed the initial
+ * part of the file to a pixbuf loader. This is expensive but we only do it on a likely match. */
+static gint
+detect_png(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
-    return fileinfo->buffer_len >= 8 && !memcmp(fileinfo->head, "\x89PNG\r\n\x1a\n", 8);
+    if (only_name || fileinfo->buffer_len < 8 || memcmp(fileinfo->head, "\x89PNG\r\n\x1a\n", 8))
+        return 0;
+    return pixbuf_detect_try_loader(fileinfo, "png", 70);
 }
 
-static gboolean
-pixmap_filter_jpeg(const GwyFileDetectInfo *fileinfo)
+static gint
+detect_jpeg(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
-    return fileinfo->buffer_len >= 2 && !memcmp(fileinfo->head, "\xff\xd8", 2);
+    if (only_name || fileinfo->buffer_len < 4 || memcmp(fileinfo->head, "\xff\xd8\xff", 3))
+        return 0;
+    /* We do not know what the next thing is, but the marker values (the second byte after 0xff) come from a
+     * a relatively limited set. */
+    if (fileinfo->head[3] < 0xd0 && fileinfo->head[3] != 0xc0 && fileinfo->head[3] != 0xc2 && fileinfo->head[3] != 0xc4)
+        return 0;
+    if (fileinfo->head[3] > 0xf0 && fileinfo->head[3] != 0xfe)
+        return 0;
+    return pixbuf_detect_try_loader(fileinfo, "jpeg", 70);
 }
 
 /* Unused on Win64 */
 G_GNUC_UNUSED
-static gboolean
-pixmap_filter_tiff(const GwyFileDetectInfo *fileinfo)
+static gint
+detect_tiff(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
+    if (only_name)
+        return FALSE;
+
     /* The pixbuf loader is unlikely to load BigTIFFs any time soon. */
     GwyTIFFVersion version = GWY_TIFF_CLASSIC;
     gwy_debug("Checking TIFF header");
     if (!gwy_tiff_detect(fileinfo->head, fileinfo->buffer_len, &version, NULL))
-        return FALSE;
+        return 0;
     gwy_debug("TIFF header OK (type %.2s)", fileinfo->head);
-    return TRUE;
+    /* The TIFF loaders (both libTIFF and GDI-based) seem to crash on broken TIFFs a way too often.  Do not try to
+     * feed anything to it just accept the file is a TIFF and hope some other loader of a TIFF-based format will claim
+     * it with a higher score. */
+    gwy_debug("Avoiding feeding data to TIFF pixbuf loader because it is broken as hell.");
+    /* Please someone return a higher score… */
+    return 60;
 }
 
-static gboolean
-pixmap_filter_pnm(const GwyFileDetectInfo *fileinfo)
+static gint
+detect_pnm(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
-    return fileinfo->buffer_len >= 2 && fileinfo->head[0] == 'P' && g_ascii_isdigit(fileinfo->head[1]);
+    if (only_name || fileinfo->buffer_len < 3
+        || fileinfo->head[0] != 'P' || !g_ascii_isdigit(fileinfo->head[1]) || !g_ascii_isspace(fileinfo->head[2]))
+        return 0;
+    return pixbuf_detect_try_loader(fileinfo, "pnm", 60);
 }
 
-static gboolean
-pixmap_filter_bmp(const GwyFileDetectInfo *fileinfo)
+static gint
+detect_bmp(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
-    return fileinfo->buffer_len >= 2 && !memcmp(fileinfo->head, "BM", 2);
+    if (only_name || fileinfo->buffer_len < 2 || memcmp(fileinfo->head, "BM", 2))
+        return 0;
+    return pixbuf_detect_try_loader(fileinfo, "bmp", 60);
 }
 
-static gboolean
-pixmap_filter_tga(const GwyFileDetectInfo *fileinfo)
+static gint
+detect_tga(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
-    guint8 cmtype, dtype;
-
-    if (fileinfo->buffer_len < 2)
-        return FALSE;
-
-    cmtype = fileinfo->head[1];
-    dtype = fileinfo->head[2];
+    if (only_name || fileinfo->buffer_len < 3)
+        return 0;
 
+    guint8 cmtype = fileinfo->head[1], dtype = fileinfo->head[2];
     if (dtype == 1 || dtype == 9 || dtype == 32 || dtype == 33) {
         if (cmtype != 1)
-            return FALSE;
+            return 0;
     }
     else if (dtype == 2 || dtype == 3 || dtype == 10 || dtype == 11) {
         if (cmtype != 0)
-            return FALSE;
+            return 0;
     }
     else
-        return FALSE;
+        return 0;
 
-    return TRUE;
+    return pixbuf_detect_try_loader(fileinfo, "tga", 70);
 }
 
-static gboolean
-pixmap_filter_gif(const GwyFileDetectInfo *fileinfo)
+static gint
+detect_gif(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
-    return (fileinfo->buffer_len >= 4 && !memcmp(fileinfo->head, "GIF8", 4));
+    if (only_name || fileinfo->buffer_len < 4 || memcmp(fileinfo->head, "GIF8", 4))
+        return 0;
+    return pixbuf_detect_try_loader(fileinfo, "gif", 70);
 }
 
-static gboolean
-pixmap_filter_jpeg2000(const GwyFileDetectInfo *fileinfo)
+static gint
+detect_jpeg2000(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
-    return (fileinfo->buffer_len >= 23
-            && !memcmp(fileinfo->head,
-                       "\x00\x00\x00\x0C\x6A\x50\x20\x20\x0D\x0A\x87\x0A\x00\x00\x00\x14\x66\x74\x79\x70\x6A\x70\x32",
-                       23));
+    static const guchar magic[] =
+        "\x00\x00\x00\x0C\x6A\x50\x20\x20\x0D\x0A\x87\x0A\x00\x00\x00\x14\x66\x74\x79\x70\x6A\x70\x32";
+
+    if (only_name || fileinfo->buffer_len < 23 || memcmp(fileinfo->head, magic, 23))
+        return 0;
+    return pixbuf_detect_try_loader(fileinfo, "jpeg2000", 70);
 }
 
-static gboolean
-pixmap_filter_jpegxl(const GwyFileDetectInfo *fileinfo)
+static gint
+detect_jpegxl(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     /* The two-byte one is silly, but used for standalone images in the wild. */
-    return (fileinfo->buffer_len >= 12
-            && (!memcmp(fileinfo->head, "\x00\x00\x00\x0C\x4A\x58\x4C\x20\x0D\x0A\x87\x0A", 12)
-                || !memcmp(fileinfo->head, "\xFF\x0A", 2)));
+    static const guchar magic[] = "\x00\x00\x00\x0C\x4A\x58\x4C\x20\x0D\x0A\x87\x0A";
+    static const guchar magic2[] = "\xFF\x0A";
+
+    if (only_name || fileinfo->buffer_len < 12)
+        return 0;
+
+    if (!memcmp(fileinfo->head, magic, 12))
+        return pixbuf_detect_try_loader(fileinfo, "jxl", 70);
+    if (!memcmp(fileinfo->head, magic2, 2))
+        return pixbuf_detect_try_loader(fileinfo, "jxl", 60);
+    return 0;
 }
 
-static gboolean
-pixmap_filter_pcx(const GwyFileDetectInfo *fileinfo)
+static gint
+detect_pcx(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
-    return fileinfo->buffer_len >= 2 && fileinfo->head[0] == '\x0a' && fileinfo->head[1] <= 0x05;
+    if (only_name || fileinfo->buffer_len < 2 || fileinfo->head[0] != '\x0a' || fileinfo->head[1] > 0x05)
+        return 0;
+    return pixbuf_detect_try_loader(fileinfo, "pcx", 60);
 }
 
-static gboolean
-pixmap_filter_xpm(const GwyFileDetectInfo *fileinfo)
+static gint
+detect_xpm(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
-    return fileinfo->buffer_len >= 9 && !memcmp(fileinfo->head, "/* XPM */", 9);
+    if (only_name || fileinfo->buffer_len < 9 || memcmp(fileinfo->head, "/* XPM */", 9))
+        return 0;
+    return pixbuf_detect_try_loader(fileinfo, "xpm", 70);
 }
 
-static gboolean
-pixmap_filter_ras(const GwyFileDetectInfo *fileinfo)
+static gint
+detect_ras(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
-    return fileinfo->buffer_len >= 4 && !memcmp(fileinfo->head, "\x59\xa6\x6a\x95", 4);
+    if (only_name || fileinfo->buffer_len < 4 || memcmp(fileinfo->head, "\x59\xa6\x6a\x95", 4))
+        return 0;
+    return pixbuf_detect_try_loader(fileinfo, "ras", 70);
 }
 
-static gboolean
-pixmap_filter_icns(const GwyFileDetectInfo *fileinfo)
+static gint
+detect_icns(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
-    return fileinfo->buffer_len >= 4 && !memcmp(fileinfo->head, "icns", 4);
+    if (only_name || fileinfo->buffer_len < 4 || memcmp(fileinfo->head, "icns", 4))
+        return 0;
+    return pixbuf_detect_try_loader(fileinfo, "icns", 70);
 }
 
-static gboolean
-pixmap_filter_webp(const GwyFileDetectInfo *fileinfo)
+static gint
+detect_webp(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
-    return (fileinfo->buffer_len >= 15
-            && !memcmp(fileinfo->head, "RIFF", 4)
-            && !memcmp(fileinfo->head + 8, "WEBPVP8", 7));
+    if (only_name || fileinfo->buffer_len < 15
+        || memcmp(fileinfo->head, "RIFF", 4) || memcmp(fileinfo->head + 8, "WEBPVP8", 7))
+        return 0;
+    return pixbuf_detect_try_loader(fileinfo, "webp", 70);
 }
 
-static gboolean
-pixmap_filter_heif(const GwyFileDetectInfo *fileinfo)
+static gint
+detect_heif(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
-    return (fileinfo->buffer_len >= 12
-            && !memcmp(fileinfo->head + 4, "ftyp", 4)
-            && (!memcmp(fileinfo->head + 8, "mif1", 4)
-                || !memcmp(fileinfo->head + 8, "heic", 4)
-                || !memcmp(fileinfo->head + 8, "heix", 4)));
+    const guchar *h8 = fileinfo->head + 8;
+    if (only_name || fileinfo->buffer_len < 12 || memcmp(fileinfo->head + 4, "ftyp", 4)
+        || (memcmp(h8, "mif1", 4) && memcmp(h8, "heic", 4) && memcmp(h8, "heix", 4)))
+        return 0;
+    /* The pixbuf format name is heif/avif. What a silly name. */
+    return pixbuf_detect_try_loader(fileinfo, "heif/avif", 70);
 }
 
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
