@@ -1,6 +1,6 @@
 /*
- *  $Id: pltfile.c 26161 2024-02-06 14:03:25Z yeti-dn $
- *  Copyright (C) 2009 David Necas (Yeti).
+ *  $Id: pltfile.c 28789 2025-11-04 17:14:03Z yeti-dn $
+ *  Copyright (C) 2009-2025 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
  *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
@@ -44,15 +44,10 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/stats.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -62,19 +57,19 @@
 #define MAGIC4 "#Points :"
 #define EXTENSION ".plt"
 
-static gboolean      module_register(void);
-static gint          plt_detect     (const GwyFileDetectInfo *fileinfo,
-                                     gboolean only_name);
-static GwyContainer* plt_load       (const gchar *filename,
-                                     GwyRunType mode,
-                                     GError **error);
+static gboolean module_register(void);
+static gint     detect_file    (const GwyFileDetectInfo *fileinfo,
+                                gboolean only_name);
+static GwyFile* load_file      (const gchar *filename,
+                                GwyRunModeFlags mode,
+                                GError **error);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Nanosurf PLT files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.2",
+    "0.3",
     "David Nečas (Yeti)",
     "2009",
 };
@@ -86,17 +81,13 @@ module_register(void)
 {
     gwy_file_func_register("pltfile",
                            N_("Nanosurf PLT files (.plt)"),
-                           (GwyFileDetectFunc)&plt_detect,
-                           (GwyFileLoadFunc)&plt_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-plt_detect(const GwyFileDetectInfo *fileinfo,
-               gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     if (only_name)
         return g_str_has_suffix(fileinfo->name_lowercase, EXTENSION) ? 10 : 0;
@@ -110,13 +101,13 @@ plt_detect(const GwyFileDetectInfo *fileinfo,
     return 0;
 }
 
-static GwyContainer*
-plt_load(const gchar *filename,
-         G_GNUC_UNUSED GwyRunType mode,
-         GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *container = NULL;
-    GwyDataField *dfield = NULL;
+    GwyFile *file = NULL;
+    GwyField *dfield = NULL;
     GwyTextHeaderParser parser;
     gchar *xunit, *yunit, *zunit;
     gchar *p, *value, *buffer = NULL;
@@ -146,7 +137,7 @@ plt_load(const gchar *filename,
     *p = '\0';
     p++;
 
-    gwy_clear(&parser, 1);
+    gwy_clear1(parser);
     parser.line_prefix = "#";
     parser.key_value_separator = ":";
     hash = gwy_text_header_parse(buffer, &parser, NULL, NULL);
@@ -164,33 +155,33 @@ plt_load(const gchar *filename,
     sanitise_real_size(&yreal, "y size");
     zreal = g_ascii_strtod(g_hash_table_lookup(hash, "ZRange"), &zunit);
 
-    dfield = gwy_data_field_new(xres, yres, xreal, yreal, FALSE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), xunit);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), zunit);
-    if (!gwy_si_unit_equal_string(gwy_data_field_get_si_unit_xy(dfield), yunit))
+    dfield = gwy_field_new(xres, yres, xreal, yreal, FALSE);
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), xunit);
+    gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), zunit);
+    if (!gwy_unit_equal_string(gwy_field_get_unit_xy(dfield), yunit))
         g_warning("X and Y units differ, using X");
 
-    if (!gwy_parse_doubles(p, gwy_data_field_get_data(dfield), GWY_PARSE_DOUBLES_FREE_FORM, &yres, &xres, &p, &err)) {
+    if (!gwy_parse_doubles(p, gwy_field_get_data(dfield), GWY_PARSE_DOUBLES_FREE_FORM, &yres, &xres, &p, &err)) {
         err_PARSE_DOUBLES(error, &err);
         g_object_unref(dfield);
         goto fail;
     }
 
-    container = gwy_container_new();
-    gwy_container_pass_object(container, gwy_app_get_data_key_for_id(0), dfield);
+    file = gwy_file_new_in_construction();
+    gwy_file_pass_image(file, 0, dfield);
 
     if ((value = g_hash_table_lookup(hash, "Channel")))
-        gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(0), value);
+        gwy_file_set_title(file, GWY_FILE_IMAGE, 0, value, FALSE);
     else
-        gwy_app_channel_title_fall_back(container, 0);
+        gwy_image_title_fall_back(file, 0);
 
-    gwy_file_channel_import_log_add(container, 0, NULL, filename);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
 
 fail:
     g_free(buffer);
     g_hash_table_destroy(hash);
 
-    return container;
+    return file;
 }
 
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
