@@ -1,6 +1,6 @@
 /*
- *  $Id: pnifile.c 25719 2023-09-26 13:09:32Z yeti-dn $
- *  Copyright (C) 2006-2018 David Necas (Yeti), Petr Klapetek.
+ *  $Id: pnifile.c 28789 2025-11-04 17:14:03Z yeti-dn $
+ *  Copyright (C) 2006-2025 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -46,14 +46,9 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/datafield.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -101,19 +96,19 @@ typedef enum {
     VALUE_TYPE_MV = 4
 } PNIValueType;
 
-static gboolean      module_register(void);
-static gint          pni_detect     (const GwyFileDetectInfo *fileinfo,
-                                     gboolean only_name);
-static GwyContainer* pni_load       (const gchar *filename,
-                                     GwyRunType mode,
-                                     GError **error);
+static gboolean module_register(void);
+static gint     detect_file    (const GwyFileDetectInfo *fileinfo,
+                                gboolean only_name);
+static GwyFile* load_file      (const gchar *filename,
+                                GwyRunModeFlags mode,
+                                GError **error);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Pacific Nanotechnology PNI data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.8",
+    "0.9",
     "David Nečas (Yeti) & Petr Klapetek",
     "2006",
 };
@@ -125,17 +120,13 @@ module_register(void)
 {
     gwy_file_func_register("pnifile",
                            N_("PNI files (.pni)"),
-                           (GwyFileDetectFunc)&pni_detect,
-                           (GwyFileLoadFunc)&pni_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-pni_detect(const GwyFileDetectInfo *fileinfo,
-           gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     guint32 firstbyte, xres, yres;
     const guchar *p;
@@ -167,10 +158,10 @@ pni_detect(const GwyFileDetectInfo *fileinfo,
     return 0;
 }
 
-static GwyContainer*
-pni_load(const gchar *filename,
-         G_GNUC_UNUSED GwyRunType mode,
-         GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
     enum {
         /* Absolute in file (file header). */
@@ -203,11 +194,11 @@ pni_load(const gchar *filename,
         { "Z Actuator", DATA_TYPE2_ZACTUATOR, },
     };
 
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     guchar *buffer = NULL;
     gsize size = 0;
     GError *err = NULL;
-    GwyDataField *dfield = NULL;
+    GwyField *dfield = NULL;
     const guchar *p;
     gint xres, yres, version;
     PNIValueType value_type;
@@ -277,7 +268,7 @@ pni_load(const gchar *filename,
     else
         direction = DIRECTION_FORWARD;  /* XXX: Whatever. */
 
-    dfield = gwy_data_field_new(xres, yres, xreal, yreal, FALSE);
+    dfield = gwy_field_new(xres, yres, xreal, yreal, FALSE);
     if (value_type == VALUE_TYPE_NM) {
         unitstr = "m";
         zscale *= Nanometer;
@@ -289,14 +280,14 @@ pni_load(const gchar *filename,
     else {
         g_warning("Value type %d is unknown", value_type);
     }
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), unitstr);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
+    gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), unitstr);
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
     gwy_convert_raw_data(buffer + DATA_START, xres*yres, 1, GWY_RAW_DATA_SINT16, GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                         gwy_data_field_get_data(dfield), zscale, 0.0);
+                         gwy_field_get_data(dfield), zscale, 0.0);
     gwy_file_abandon_contents(buffer, size, NULL);
 
-    container = gwy_container_new();
-    gwy_container_pass_object(container, gwy_app_get_data_key_for_id(0), dfield);
+    file = gwy_file_new_in_construction();
+    gwy_file_pass_image(file, 0, dfield);
 
     if (version == 1)
         title = gwy_enum_to_string(data_type, titles1, G_N_ELEMENTS(titles1));
@@ -305,16 +296,16 @@ pni_load(const gchar *filename,
 
     if (title) {
         s = g_strdup_printf("%s (%s)", title, direction ? "Backward" : "Forward");
-        gwy_container_set_string(container, gwy_app_get_data_title_key_for_id(0), s);
+        gwy_file_pass_title(file, GWY_FILE_IMAGE, 0, s);
     }
     else
         g_warning("Data type %d is unknown", data_type);
 
     /* TODO: Put version to metadata? */
 
-    gwy_file_channel_import_log_add(container, 0, NULL, filename);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
 
-    return container;
+    return file;
 }
 
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
