@@ -1,6 +1,6 @@
 /*
- *  $Id: princetonspe.c 26342 2024-05-15 09:09:02Z yeti-dn $
- *  Copyright (C) 2015 David Necas (Yeti).
+ *  $Id: princetonspe.c 28789 2025-11-04 17:14:03Z yeti-dn $
+ *  Copyright (C) 2015-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -23,15 +23,10 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/datafield.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/data-browser.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -88,24 +83,24 @@ typedef struct {
     GHashTable *hash;
 } PSPEFile;
 
-static gboolean      module_register (void);
-static gint          pspe_detect     (const GwyFileDetectInfo *fileinfo,
-                                      gboolean only_name);
-static GwyContainer* pspe_load       (const gchar *filename,
-                                      GwyRunType mode,
-                                      GError **error);
-static gboolean      pspe_read_header(PSPEFile *pspefile,
-                                      const guchar *buffer);
-static gboolean      pspe_check_size (PSPEFile *pspefile,
-                                      GError **error);
-static void          parse_xml_footer(PSPEFile *pspefile);
+static gboolean module_register (void);
+static gint     detect_file     (const GwyFileDetectInfo *fileinfo,
+                                 gboolean only_name);
+static GwyFile* load_file       (const gchar *filename,
+                                 GwyRunModeFlags mode,
+                                 GError **error);
+static gboolean pspe_read_header(PSPEFile *pspefile,
+                                 const guchar *buffer);
+static gboolean pspe_check_size (PSPEFile *pspefile,
+                                 GError **error);
+static void     parse_xml_footer(PSPEFile *pspefile);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Princeton Instruments camera SPE files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.1",
+    "0.2",
     "David Nečas (Yeti)",
     "2015",
 };
@@ -117,17 +112,13 @@ module_register(void)
 {
     gwy_file_func_register("princetonspe",
                            N_("Princeton Instruments SPE files"),
-                           (GwyFileDetectFunc)&pspe_detect,
-                           (GwyFileLoadFunc)&pspe_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-pspe_detect(const GwyFileDetectInfo *fileinfo,
-            gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     PSPEFile pspefile;
     gint score = 0;
@@ -135,7 +126,7 @@ pspe_detect(const GwyFileDetectInfo *fileinfo,
     if (only_name)
         return g_str_has_suffix(fileinfo->name_lowercase, EXTENSION) ? 10 : 0;
 
-    gwy_clear(&pspefile, 1);
+    gwy_clear1(pspefile);
     pspefile.size = fileinfo->file_size;
     if (fileinfo->file_size > HEADER_SIZE
         && fileinfo->buffer_len >= NUMFRAMES_OFFSET + sizeof(guint32)
@@ -150,20 +141,19 @@ pspe_detect(const GwyFileDetectInfo *fileinfo,
     return score;
 }
 
-static GwyContainer*
-pspe_load(const gchar *filename,
-          G_GNUC_UNUSED GwyRunType mode,
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
           GError **error)
 {
     PSPEFile pspefile;
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     GError *err = NULL;
-    GwyDataField *dfield = NULL;
+    GwyField *dfield = NULL;
     gchar *value;
-    GQuark quark;
     guint i, typesize, imagelen, len, nframes, data_size;
 
-    gwy_clear(&pspefile, 1);
+    gwy_clear1(pspefile);
 
     if (!gwy_file_get_contents(filename, &pspefile.buffer, &pspefile.size, &err)) {
         err_GET_FILE_CONTENTS(error, &err);
@@ -208,17 +198,15 @@ pspe_load(const gchar *filename,
         imagelen = len;
     }
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     for (i = 0; i < nframes; i++) {
-        dfield = gwy_data_field_new(pspefile.xres, pspefile.yres, pspefile.xres, pspefile.yres, FALSE);
+        dfield = gwy_field_new(pspefile.xres, pspefile.yres, pspefile.xres, pspefile.yres, FALSE);
         gwy_convert_raw_data(pspefile.buffer + HEADER_SIZE + imagelen*i, pspefile.xres * pspefile.yres, 1,
                              pspefile.rawtype, GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                             gwy_data_field_get_data(dfield), 1.0, 0.0);
-        quark = gwy_app_get_data_key_for_id(i);
-        gwy_container_pass_object(container, quark, dfield);
-
-        gwy_app_set_data_field_title(container, i, "Intensity");
-        gwy_file_channel_import_log_add(container, i, NULL, filename);
+                             gwy_field_get_data(dfield), 1.0, 0.0);
+        gwy_file_pass_image(file, i, dfield);
+        gwy_file_set_title(file, GWY_FILE_IMAGE, i, "Intensity", nframes > 1);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, i, NULL, filename);
     }
 
 fail:
@@ -228,7 +216,7 @@ fail:
     if (pspefile.str)
         g_string_free(pspefile.str, TRUE);
 
-    return container;
+    return file;
 }
 
 static gboolean
