@@ -1,5 +1,5 @@
 /*
- *  $Id: psia.c 26163 2024-02-06 16:56:55Z yeti-dn $
+ *  $Id: psia.c 28848 2025-11-10 18:54:08Z yeti-dn $
  *  Copyright (C) 2006-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -28,14 +28,9 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/stats.h>
-#include <libprocess/spectra.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 
 #include "err.h"
 #include "gwytiff.h"
@@ -197,49 +192,37 @@ typedef struct {
     gint reserved[30];
 } PSIASpectroscopyHeader;
 
-static gboolean      module_register         (void);
-static gint          psia_detect             (const GwyFileDetectInfo *fileinfo,
-                                              gboolean only_name);
-static GwyContainer* psia_load               (const gchar *filename,
-                                              GwyRunType mode,
-                                              GError **error);
-static GwyContainer* psia_load_tiff          (GwyTIFF *tiff,
-                                              GError **error);
-static void          psia_free_image_header  (PSIAImageHeader *header);
-static void          psia_read_data_field    (GwyDataField *dfield,
-                                              const guchar *p,
-                                              PSIADataType data_type,
-                                              gdouble q,
-                                              gdouble z_scale,
-                                              gdouble z0);
-static guint         psia_read_image_header  (const guchar *p,
-                                              gsize size,
-                                              guint version,
-                                              PSIAImageHeader *header,
-                                              GError **error);
-#if 0
-static void          psia_free_spectro_header(PSIASpectroscopyHeader *header);
-static gboolean      psia_read_spectro_header(const guchar *p,
-                                              gsize size,
-                                              guint version,
-                                              PSIASpectroscopyHeader *header,
-                                              GError **error);
-static void          psia_read_spectra       (GwyContainer *container,
-                                              GwyTIFF *tiff,
-                                              PSIADataType data_type,
-                                              guint version);
-#endif
-static gchar*        psia_wchar_to_utf8      (const guchar **src,
-                                              guint len);
-static GwyContainer* psia_get_metadata       (PSIAImageHeader *header,
-                                              guint version);
+static gboolean      module_register  (void);
+static gint          detect_file      (const GwyFileDetectInfo *fileinfo,
+                                       gboolean only_name);
+static GwyFile*      load_file        (const gchar *filename,
+                                       GwyRunModeFlags mode,
+                                       GError **error);
+static GwyFile*      load_tiff        (GwyTIFF *tiff,
+                                       GError **error);
+static void          free_image_header(PSIAImageHeader *header);
+static void          read_field       (GwyField *dfield,
+                                       const guchar *p,
+                                       PSIADataType data_type,
+                                       gdouble q,
+                                       gdouble z_scale,
+                                       gdouble z0);
+static guint         read_image_header(const guchar *p,
+                                       gsize size,
+                                       guint version,
+                                       PSIAImageHeader *header,
+                                       GError **error);
+static gchar*        wchar_to_utf8    (const guchar **src,
+                                       guint len);
+static GwyContainer* get_metadata     (PSIAImageHeader *header,
+                                       guint version);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     module_register,
     N_("Imports Park Systems data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.9",
+    "0.10",
     "David Nečas (Yeti) & Petr Klapetek",
     "2006",
 };
@@ -251,16 +234,13 @@ module_register(void)
 {
     gwy_file_func_register("psia",
                            N_("Park Systems data files (.tiff, .tif)"),
-                           (GwyFileDetectFunc)&psia_detect,
-                           (GwyFileLoadFunc)&psia_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-psia_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     GwyTIFF *tiff;
     gint score = 0;
@@ -287,35 +267,35 @@ psia_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
     return score;
 }
 
-static GwyContainer*
-psia_load(const gchar *filename,
-          G_GNUC_UNUSED GwyRunType mode,
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
           GError **error)
 {
     GwyTIFF *tiff;
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
 
     tiff = gwy_tiff_load(filename, error);
     if (!tiff)
         return NULL;
 
-    container = psia_load_tiff(tiff, error);
-    if (container)
-        gwy_file_channel_import_log_add(container, 0, NULL, filename);
+    file = load_tiff(tiff, error);
+    if (file)
+        gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
     gwy_tiff_free(tiff);
 
-    return container;
+    return file;
 }
 
-static GwyContainer*
-psia_load_tiff(GwyTIFF *tiff, GError **error)
+static GwyFile*
+load_tiff(GwyTIFF *tiff, GError **error)
 {
     const GwyTIFFEntry *entry;
     PSIAImageHeader header;
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     GwyContainer *meta = NULL;
-    GwyDataField *dfield;
-    GwySIUnit *siunit;
+    GwyField *dfield;
+    GwyUnit *siunit;
     guint magic, version, bps, i;
     const guchar *p, *data;
     gchar *comment = NULL;
@@ -360,47 +340,47 @@ psia_load_tiff(GwyTIFF *tiff, GError **error)
     gwy_debug("[Header] count: %d", count);
 
     /* Parse header */
-    if (!(bps = psia_read_image_header(p, count, version, &header, error))) {
-        psia_free_image_header(&header);
+    if (!(bps = read_image_header(p, count, version, &header, error))) {
+        free_image_header(&header);
         return NULL;
     }
 
     if (err_SIZE_MISMATCH(error, bps*header.xres*header.yres, data_len, TRUE)) {
-        psia_free_image_header(&header);
+        free_image_header(&header);
         return NULL;
     }
 
     gwy_tiff_get_string0(tiff, PSIA_TIFFTAG_Comments, &comment);
 
-    dfield = gwy_data_field_new(header.xres, header.yres, header.xreal, header.yreal, FALSE);
+    dfield = gwy_field_new(header.xres, header.yres, header.xreal, header.yreal, FALSE);
 
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
 
     if (header.z_unit)
-        siunit = gwy_si_unit_new_parse(header.z_unit, &power10);
+        siunit = gwy_unit_new_parse(header.z_unit, &power10);
     else {
         g_warning("Z units are missing");
-        siunit = gwy_si_unit_new_parse("um", &power10);
+        siunit = gwy_unit_new_parse("um", &power10);
     }
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_z(dfield), siunit);
+    gwy_unit_assign(gwy_field_get_unit_z(dfield), siunit);
     g_object_unref(siunit);
 
     if (header.z_scale == 0.0)
         header.z_scale = 1.0;
     z0 = header.z_offset;
-    q = pow10(power10)*header.data_gain;
-    psia_read_data_field(dfield, data, header.data_type, q, header.z_scale, z0);
+    q = gwy_exp10(power10)*header.data_gain;
+    read_field(dfield, data, header.data_type, q, header.z_scale, z0);
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
 
-    gwy_data_field_invert(dfield, TRUE, FALSE, FALSE);
-    gwy_container_pass_object_by_name(container, "/0/data", dfield);
-    gwy_app_channel_check_nonsquare(container, 0);
+    gwy_field_flip(dfield, FALSE, TRUE);
+    gwy_file_pass_image(file, 0, dfield);
+    gwy_check_nonsquare_image(file, 0);
 
     if (header.source_name && *header.source_name)
-        gwy_container_set_const_string_by_name(container, "/0/data/title", header.source_name);
+        gwy_file_set_title(file, GWY_FILE_IMAGE, 0, header.source_name, FALSE);
 
-    meta = psia_get_metadata(&header, version);
+    meta = get_metadata(&header, version);
     if (comment && *comment) {
         /* FIXME: Charset conversion. But from what? */
         gwy_container_set_string_by_name(meta, "Comment", comment);
@@ -408,18 +388,17 @@ psia_load_tiff(GwyTIFF *tiff, GError **error)
     }
     g_free(comment);
     gwy_container_set_string_by_name(meta, "Version", g_strdup_printf("%08x", version));
+    gwy_file_pass_meta(file, GWY_FILE_IMAGE, 0, meta);
 
-    gwy_container_pass_object_by_name(container, "/0/meta", meta);
+    free_image_header(&header);
 
-    psia_free_image_header(&header);
+    /* read_spectra(file, tiff, header.data_type, version); */
 
-    /* psia_read_spectra(container, tiff, header.data_type, version); */
-
-    return container;
+    return file;
 }
 
 static void
-psia_free_image_header(PSIAImageHeader *header)
+free_image_header(PSIAImageHeader *header)
 {
     g_free(header->source_name);
     g_free(header->image_mode);
@@ -432,12 +411,12 @@ psia_free_image_header(PSIAImageHeader *header)
 }
 
 static void
-psia_read_data_field(GwyDataField *dfield,
-                     const guchar *p,
-                     PSIADataType data_type,
-                     gdouble q,
-                     gdouble z_scale,
-                     gdouble z0)
+read_field(GwyField *dfield,
+           const guchar *p,
+           PSIADataType data_type,
+           gdouble q,
+           gdouble z_scale,
+           gdouble z0)
 {
     GwyRawDataType rawdatatype;
     gint xres, yres;
@@ -451,18 +430,18 @@ psia_read_data_field(GwyDataField *dfield,
     else
         g_return_if_reached();
 
-    xres = gwy_data_field_get_xres(dfield);
-    yres = gwy_data_field_get_yres(dfield);
+    xres = gwy_field_get_xres(dfield);
+    yres = gwy_field_get_yres(dfield);
     gwy_convert_raw_data(p, xres*yres, 1, rawdatatype, GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                         gwy_data_field_get_data(dfield), q*z_scale, q*z0);
+                         gwy_field_get_data(dfield), q*z_scale, q*z0);
 }
 
 static guint
-psia_read_image_header(const guchar *p,
-                       gsize size,
-                       guint version,
-                       PSIAImageHeader *header,
-                       GError **error)
+read_image_header(const guchar *p,
+                  gsize size,
+                  guint version,
+                  PSIAImageHeader *header,
+                  GError **error)
 {
     guint bps = 0;
 
@@ -482,8 +461,8 @@ psia_read_image_header(const guchar *p,
         err_NO_DATA(error);
         return 0;
     }
-    header->source_name = psia_wchar_to_utf8(&p, 32);
-    header->image_mode = psia_wchar_to_utf8(&p, 8);
+    header->source_name = wchar_to_utf8(&p, 32);
+    header->image_mode = wchar_to_utf8(&p, 8);
     gwy_debug("source_name: <%s>, image_mode: <%s>", header->source_name, header->image_mode);
     header->lpf_strength = gwy_get_gdouble_le(&p);
     header->auto_flatten = gwy_get_guint32_le(&p);
@@ -514,7 +493,7 @@ psia_read_image_header(const guchar *p,
     gwy_debug("xoff: %g, yoff: %g", header->xoff, header->yoff);
     header->scan_rate = gwy_get_gdouble_le(&p);
     header->set_point = gwy_get_gdouble_le(&p);
-    header->set_point_unit = psia_wchar_to_utf8(&p, 8);
+    header->set_point_unit = wchar_to_utf8(&p, 8);
     if (!header->set_point_unit)
         header->set_point_unit = g_strdup("V");
     header->tip_bias = gwy_get_gdouble_le(&p);
@@ -523,7 +502,7 @@ psia_read_image_header(const guchar *p,
     header->z_scale = gwy_get_gdouble_le(&p);
     header->z_offset = gwy_get_gdouble_le(&p);
     gwy_debug("data_gain: %g, z_scale: %g", header->data_gain, header->z_scale);
-    header->z_unit = psia_wchar_to_utf8(&p, 8);
+    header->z_unit = wchar_to_utf8(&p, 8);
     gwy_debug("z_unit: <%s>", header->z_unit);
     header->data_min = gwy_get_gint32_le(&p);
     header->data_max = gwy_get_gint32_le(&p);
@@ -535,15 +514,15 @@ psia_read_image_header(const guchar *p,
     if (version == PSIA_VERSION2) {
         header->z_servo_gain = gwy_get_gdouble_le(&p);
         header->z_scanner_range = gwy_get_gdouble_le(&p);
-        header->xy_voltage_mode = psia_wchar_to_utf8(&p, 8);
-        header->z_voltage_mode = psia_wchar_to_utf8(&p, 8);
-        header->xy_servo_mode = psia_wchar_to_utf8(&p, 8);
+        header->xy_voltage_mode = wchar_to_utf8(&p, 8);
+        header->z_voltage_mode = wchar_to_utf8(&p, 8);
+        header->xy_servo_mode = wchar_to_utf8(&p, 8);
         header->data_type = gwy_get_guint32_le(&p);
         header->reserved1 = gwy_get_guint32_le(&p);
         header->reserved2 = gwy_get_guint32_le(&p);
         header->ncm_amplitude = gwy_get_gdouble_le(&p);
         header->ncm_frequency = gwy_get_gdouble_le(&p);
-        header->cantilever_name = psia_wchar_to_utf8(&p, 16);
+        header->cantilever_name = wchar_to_utf8(&p, 16);
     }
     else
         header->data_type = PSIA_DATA_INT16;
@@ -561,11 +540,11 @@ psia_read_image_header(const guchar *p,
 
 #if 0
 static gboolean
-psia_read_spectro_header(const guchar *p,
-                         gsize size,
-                         guint version,
-                         PSIASpectroscopyHeader *header,
-                         GError **error)
+read_spectro_header(const guchar *p,
+                    gsize size,
+                    guint version,
+                    PSIASpectroscopyHeader *header,
+                    GError **error)
 {
     guint i;
 
@@ -583,8 +562,8 @@ psia_read_spectro_header(const guchar *p,
     for (i = 0; i < PSIA_MAX_SPECTRO_CHANNEL; i++) {
         PSIASpectroscopyChannel *channel = header->channel + i;
 
-        channel->source_name = psia_wchar_to_utf8(&p, 32);
-        channel->w_unit = psia_wchar_to_utf8(&p, 8);
+        channel->source_name = wchar_to_utf8(&p, 32);
+        channel->w_unit = wchar_to_utf8(&p, 8);
         gwy_debug("spectro channel[%u] source_name: %s, w_unit: %s",
                   i, channel->source_name, channel->w_unit);
         channel->data_gain = gwy_get_gdouble_le(&p);
@@ -632,7 +611,7 @@ psia_read_spectro_header(const guchar *p,
     header->delayed_start_time = gwy_get_gfloat_le(&p);
     header->z_servo = gwy_get_guint32_le(&p);
     header->data_gain = gwy_get_gdouble_le(&p);
-    header->w_unit = psia_wchar_to_utf8(&p, 8);
+    header->w_unit = wchar_to_utf8(&p, 8);
     header->use_extended_header = gwy_get_guint32_le(&p);
     header->spec_type = gwy_get_gint32_le(&p);
     header->reset_level = gwy_get_gfloat_le(&p);
@@ -650,7 +629,7 @@ psia_read_spectro_header(const guchar *p,
 }
 
 static void
-psia_free_spectro_header(PSIASpectroscopyHeader *header)
+free_spectro_header(PSIASpectroscopyHeader *header)
 {
     guint i;
 
@@ -662,17 +641,17 @@ psia_free_spectro_header(PSIASpectroscopyHeader *header)
 }
 
 static void
-psia_read_spectra(GwyContainer *container,
-                  GwyTIFF *tiff,
-                  PSIADataType data_type,
-                  guint version)
+read_spectra(GwyFile *file,
+             GwyTIFF *tiff,
+             PSIADataType data_type,
+             guint version)
 {
     PSIASpectroscopyHeader specheader;
     const guchar *p;
     const GwyTIFFEntry *entry;
     guint res, bps, npoints, nsources, data_len, driving_source_index, ipt, i;
     GwySpectra **spectra = NULL;
-    GwyDataLine **lines = NULL;
+    GwyLine **lines = NULL;
     /* The API does not know non-fatal failures so just dump spectra loading
      * errors to stderr. */
     GError *error = NULL;
@@ -693,8 +672,8 @@ psia_read_spectra(GwyContainer *container,
 
     gwy_debug("Found SpectroscopyHeader");
     p = entry->value;
-    if (!psia_read_spectro_header(tiff->data + tiff->get_guint32(&p),
-                                  entry->count, version, &specheader, &error)) {
+    if (!read_spectro_header(tiff->data + tiff->get_guint32(&p),
+                             entry->count, version, &specheader, &error)) {
         g_warning("%s", error->message);
         g_clear_error(&error);
         return;
@@ -740,7 +719,7 @@ psia_read_spectra(GwyContainer *container,
     // XXX: This is wrong; each spectrum, as read here, actually should be
     // two curves, forward and back.
     spectra = g_new0(GwySpectra*, nsources);
-    lines = g_new0(GwyDataLine*, nsources);
+    lines = g_new0(GwyLine*, nsources);
     for (i = 0; i < nsources; i++) {
         const PSIASpectroscopyChannel *channel = specheader.channel + i;
 
@@ -748,20 +727,19 @@ psia_read_spectra(GwyContainer *container,
             continue;
 
         spectra[i] = gwy_spectra_new();
-        gwy_si_unit_set_from_string(gwy_spectra_get_si_unit_xy(spectra[i]),
-                                    "m");
+        gwy_unit_set_from_string(gwy_spectra_get_unit_xy(spectra[i]), "m");
         gwy_spectra_set_title(spectra[i], channel->source_name);
     }
 
     for (ipt = 0; ipt < npoints; ipt++) {
         gdouble off, real;
         for (i = 0; i < nsources; i++) {
-            lines[i] = gwy_data_line_new(res, 1.0, FALSE);
+            lines[i] = gwy_line_new(res, 1.0, FALSE);
             /* FIXME: What are the scaling factors? */
             gwy_convert_raw_data(p + bps*res*nsources*ipt + i*res*bps,
                                  res, 1, rawdatatype,
                                  GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                                 gwy_data_line_get_data(lines[i]), 1.0, 0.0);
+                                 gwy_line_get_data(lines[i]), 1.0, 0.0);
         }
         off = lines[driving_source_index]->data[0];
         real = lines[driving_source_index]->data[res-1] - off;
@@ -769,8 +747,8 @@ psia_read_spectra(GwyContainer *container,
             //const PSIASpectroscopyChannel *channel = specheader.channel + i;
             // TODO: Set units
             if (i != driving_source_index) {
-                gwy_data_line_set_real(lines[i], real);
-                gwy_data_line_set_offset(lines[i], off);
+                gwy_line_set_real(lines[i], real);
+                gwy_line_set_offset(lines[i], off);
                 gwy_spectra_add_spectrum(spectra[i], lines[i], 0.0, 0.0);
             }
             g_object_unref(lines[i]);
@@ -778,15 +756,10 @@ psia_read_spectra(GwyContainer *container,
     }
 
     for (i = 0; i < nsources; i++) {
-        gchar *strkey;
-
-        if (!spectra[i])
-            continue;
-
-        strkey = g_strdup_printf("/sps/%d", i);
-        gwy_container_set_object_by_name(container, strkey, spectra[i]);
-        g_free(strkey);
-        g_object_unref(spectra[i]);
+        if (spectra[i]) {
+            gwy_file_pass_spectra(file, i, spectra[i]);
+            gwy_log_add_import(file, GWY_FILE_SPECTRA, i, NULL, filename);
+        }
     }
     g_free(spectra);
     g_free(lines);
@@ -796,13 +769,12 @@ psia_read_spectra(GwyContainer *container,
         g_warning("%s", error->message);
         g_clear_error(&error);
     }
-    psia_free_spectro_header(&specheader);
+    free_spectro_header(&specheader);
 }
 #endif
 
 static gchar*
-psia_wchar_to_utf8(const guchar **src,
-                   guint len)
+wchar_to_utf8(const guchar **src, guint len)
 {
     gchar *s;
 
@@ -813,12 +785,9 @@ psia_wchar_to_utf8(const guchar **src,
 }
 
 static GwyContainer*
-psia_get_metadata(PSIAImageHeader *header,
-                  guint version)
+get_metadata(PSIAImageHeader *header, guint version)
 {
-    GwyContainer *meta;
-
-    meta = gwy_container_new();
+    GwyContainer *meta = gwy_container_new_in_construction();
 
     if (header->source_name && *header->source_name) {
         gwy_container_set_string_by_name(meta, "Source name", header->source_name);
