@@ -1,6 +1,6 @@
 /*
- *  $Id: psppt.c 25334 2023-05-18 12:45:54Z yeti-dn $
- *  Copyright (C) 2021 David Necas (Yeti).
+ *  $Id: psppt.c 28911 2025-11-24 18:27:42Z yeti-dn $
+ *  Copyright (C) 2021-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -41,17 +41,10 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/stats.h>
-#include <libprocess/spectra.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/data-browser.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/wait.h>
-#include <jansson.h>
+#include <gwy.h>
+#include <json-glib/json-glib.h>
 
 #include "get.h"
 #include "err.h"
@@ -91,20 +84,20 @@ typedef struct {
 } PSPPTFrame;
 
 typedef struct {
-    json_t *root;
     gint xres;
     gint yres;
     gchar *direction;
     gdouble xreal;
     gdouble yreal;
+    JsonObject *root;
 } PSPPTScanStart;
 
 typedef struct {
-    json_t *root;
+    JsonObject *root;
 } PSPPTScanStop;
 
 typedef struct {
-    json_t *root;
+    JsonObject *root;
 } PSPPTParam;
 
 typedef struct {
@@ -112,7 +105,7 @@ typedef struct {
     PSPPTScanStart scanstart;
     PSPPTScanStop scanstop;
     PSPPTParam param;
-    guint nframes;           /* Real number; can be smaller than header.frames. */
+    guint nframes;           /* Real count; can be smaller than header.frames. */
     PSPPTFrame *frames;
     gchar **ids;
     gchar **units;
@@ -124,24 +117,24 @@ typedef struct {
     GwyContainer *meta;
 } PSPPTFile;
 
-static gboolean      module_register       (void);
-static gint          psppt_detect          (const GwyFileDetectInfo *fileinfo,
-                                            gboolean only_name);
-static GwyContainer* psppt_load            (const gchar *filename,
-                                            GwyRunType mode,
-                                            GError **error);
-static gboolean      check_string_list_item(json_t *root,
-                                            const gchar *name,
-                                            gchar **values,
-                                            guint i,
-                                            GError **error);
+static gboolean module_register       (void);
+static gint     detect_file           (const GwyFileDetectInfo *fileinfo,
+                                       gboolean only_name);
+static GwyFile* load_file             (const gchar *filename,
+                                       GwyRunModeFlags mode,
+                                       GError **error);
+static gboolean check_string_list_item(JsonObject *root,
+                                       const gchar *name,
+                                       gchar **values,
+                                       guint i,
+                                       GError **error);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     module_register,
     N_("Imports Park Systems PS-PPT data files."),
     "Yeti <yeti@gwyddion.net>",
-    "1.1",
+    "3.0",
     "David Nečas (Yeti)",
     "2021",
 };
@@ -153,16 +146,13 @@ module_register(void)
 {
     gwy_file_func_register("psppt",
                            N_("Park Systems PS-PPT data files (.ps-ppt)"),
-                           (GwyFileDetectFunc)&psppt_detect,
-                           (GwyFileLoadFunc)&psppt_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-psppt_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     if (only_name)
         return g_str_has_suffix(fileinfo->name_lowercase, ".ps-ppt") ? 20 : 0;
@@ -284,89 +274,94 @@ psppt_read_frame_table(PSPPTFile *pfile, const guchar *buf, gsize size, gsize *p
 }
 
 static gboolean
-err_JSON_STRUCTURE(GError **error, const gchar *what, const gchar *type)
+err_JSON_STRUCTURE(GError **error, const gchar *what, GType type, const gchar *typename)
 {
+    if (!typename && type) {
+        if (type == G_TYPE_INT64)
+            typename = "integer";
+        else if (type == G_TYPE_DOUBLE)
+            typename = "number";
+        else if (type == G_TYPE_STRING)
+            typename = "string";
+        else if (type == G_TYPE_BOOLEAN)
+            typename = "boolean";
+        else if (type == JSON_TYPE_ARRAY)
+            typename = "array";
+        else if (type == JSON_TYPE_OBJECT)
+            typename = "object";
+        else
+            typename = "???";
+    }
+
     g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
-                _("Unexpected JSON structure: %s should be %s."), what, type);
+                _("Unexpected JSON structure: %s should be %s."), what, typename);
     return FALSE;
 }
 
 static gboolean
-get_json_with_type(json_t *object, json_t **member, const gchar *key, json_type type, GError **error)
+get_json_with_type(JsonObject *object, JsonNode **member, const gchar *key, GType type, GError **error)
 {
-    static const GwyEnum typenames[] = {
-        { "object",  JSON_OBJECT,  },
-        { "array",   JSON_ARRAY,   },
-        { "string",  JSON_STRING,  },
-        { "integer", JSON_INTEGER, },
-        { "real",    JSON_REAL,    },
-        { "boolean", JSON_TRUE,    },
-    };
-
-    *member = json_object_get(object, key);
+    *member = json_object_get_member(object, key);
     if (*member) {
-        if (type == JSON_TRUE) {
-            if (json_is_boolean(*member))
-                return TRUE;
-        }
-        else if (type == JSON_REAL) {
-            if (json_is_number(*member))
-                return TRUE;
-        }
-        else {
-            if (json_typeof(*member) == type)
-                return TRUE;
-        }
+        GType nodetype = json_node_get_value_type(*member);
+        if (nodetype == type)
+            return TRUE;
+        if (type == G_TYPE_DOUBLE && nodetype == G_TYPE_INT64)
+            return TRUE;
     }
 
-    return err_JSON_STRUCTURE(error, key, gwy_enum_to_string(type, typenames, G_N_ELEMENTS(typenames)));
+    return err_JSON_STRUCTURE(error, key, type, NULL);
 }
 
-static json_t*
-psppt_read_frame(PSPPTFrame *frame, const guchar *buf, GError **error)
+static JsonObject*
+psppt_read_frame(JsonParser *parser, PSPPTFrame *frame, const guchar *buf, GError **error)
 {
-    json_error_t jerror;
     const gchar *start = buf + frame->offset;
-    json_t *root;
+    GError *err = NULL;
 
-    root = json_loadb(start, frame->size, 0, &jerror);
-    if (!root) {
+    if (!json_parser_load_from_data(parser, start, frame->size, &err)) {
         g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
-                    _("JSON parsing error: %s"), jerror.text);
-        return NULL;
-    }
-    if (!json_is_object(root)) {
-        err_JSON_STRUCTURE(error, "root", "object");
-        json_decref(root);
+                    _("JSON parsing error: %s"), err->message);
+        g_clear_error(&err);
         return NULL;
     }
 
-    return root;
+    JsonNode *root_node = json_parser_get_root(parser);
+    if (!JSON_NODE_HOLDS_OBJECT(root_node)) {
+        err_JSON_STRUCTURE(error, "root", JSON_TYPE_OBJECT, NULL);
+        return NULL;
+    }
+
+    return json_node_get_object(root_node);
 }
 
 static gboolean
-handle_scan_start(PSPPTScanStart *scanstart, json_t *root, GError **error)
+handle_scan_start(PSPPTScanStart *scanstart, JsonObject *root, GError **error)
 {
-    json_t *type, *geometry, *direction, *pixel_width, *pixel_height, *width, *height;
+    JsonNode *type, *geometry_node, *direction, *pixel_width, *pixel_height, *width, *height;
 
-    if (!get_json_with_type(root, &type, "type", JSON_STRING, error)
-        || !get_json_with_type(root, &geometry, "geometry", JSON_OBJECT, error)
-        || !get_json_with_type(geometry, &direction, "direction", JSON_STRING, error)
-        || !get_json_with_type(geometry, &pixel_height, "pixelHeight", JSON_INTEGER, error)
-        || !get_json_with_type(geometry, &pixel_width, "pixelWidth", JSON_INTEGER, error)
-        || !get_json_with_type(geometry, &width, "width", JSON_REAL, error)
-        || !get_json_with_type(geometry, &height, "height", JSON_REAL, error))
+    if (!get_json_with_type(root, &type, "type", G_TYPE_STRING, error)
+        || !get_json_with_type(root, &geometry_node, "geometry", JSON_TYPE_OBJECT, error))
         return FALSE;
-    if (!gwy_strequal(json_string_value(type), "scan.start"))
-        return err_JSON_STRUCTURE(error, "scan.start.type", "scan.start");
 
-    json_incref(root);
-    scanstart->root = root;
-    scanstart->xres = json_integer_value(pixel_width);
-    scanstart->yres = json_integer_value(pixel_height);
-    scanstart->xreal = json_number_value(width) * 1e-6;
-    scanstart->yreal = json_number_value(height) * 1e-6;
-    scanstart->direction = g_strdup(json_string_value(direction));
+    JsonObject *geometry = json_node_get_object(geometry_node);
+    if (!get_json_with_type(geometry, &direction, "direction", G_TYPE_STRING, error)
+        || !get_json_with_type(geometry, &pixel_height, "pixelHeight", G_TYPE_INT64, error)
+        || !get_json_with_type(geometry, &pixel_width, "pixelWidth", G_TYPE_INT64, error)
+        || !get_json_with_type(geometry, &width, "width", G_TYPE_DOUBLE, error)
+        || !get_json_with_type(geometry, &height, "height", G_TYPE_DOUBLE, error))
+        return FALSE;
+
+    if (!gwy_strequal(json_node_get_string(type), "scan.start"))
+        return err_JSON_STRUCTURE(error, "scan.start.type", 0, "scan.start");
+
+    g_assert(!scanstart->root);
+    scanstart->root = json_object_ref(root);
+    scanstart->xres = json_node_get_int(pixel_width);
+    scanstart->yres = json_node_get_int(pixel_height);
+    scanstart->xreal = json_node_get_double(width) * 1e-6;
+    scanstart->yreal = json_node_get_double(height) * 1e-6;
+    scanstart->direction = g_strdup(json_node_get_string(direction));
     gwy_debug("xres = %u, yres = %u", scanstart->xres, scanstart->yres);
     gwy_debug("xreal = %g, yreal = %g", scanstart->xreal, scanstart->yreal);
     gwy_debug("direction = %s", scanstart->direction);
@@ -375,35 +370,34 @@ handle_scan_start(PSPPTScanStart *scanstart, json_t *root, GError **error)
 }
 
 static gboolean
-handle_scan_stop(PSPPTScanStop *scanstop, json_t *root, GError **error)
+handle_scan_stop(PSPPTScanStop *scanstop, JsonObject *root, GError **error)
 {
-    json_t *type;
+    JsonNode *type;
 
-    if (!get_json_with_type(root, &type, "type", JSON_STRING, error))
+    if (!get_json_with_type(root, &type, "type", G_TYPE_STRING, error))
         return FALSE;
-    if (!gwy_strequal(json_string_value(type), "scan.stop"))
-        return err_JSON_STRUCTURE(error, "scan.stop.type", "scan.stop");
+    if (!gwy_strequal(json_node_get_string(type), "scan.stop"))
+        return err_JSON_STRUCTURE(error, "scan.stop.type", 0, "scan.stop");
 
-    json_incref(root);
-    scanstop->root = root;
+    g_assert(!scanstop->root);
+    scanstop->root = json_object_ref(root);
 
     return TRUE;
 }
 
 static gboolean
-handle_param(PSPPTParam *param, json_t *root, GError **error)
+handle_param(PSPPTParam *param, JsonObject *root, GError **error)
 {
-    json_t *type;
+    JsonNode *type;
 
-    if (!get_json_with_type(root, &type, "type", JSON_STRING, error))
+    if (!get_json_with_type(root, &type, "type", G_TYPE_STRING, error))
         return FALSE;
-    if (!gwy_strequal(json_string_value(type), "ppt.param"))
-        return err_JSON_STRUCTURE(error, "ppt.param.type", "ppt.param");
+    if (!gwy_strequal(json_node_get_string(type), "ppt.param"))
+        return err_JSON_STRUCTURE(error, "ppt.param.type", 0, "ppt.param");
 
     /* We can keep the first or the last or any of them if parameter change…  Keeping the first is simplest. */
     if (!param->root) {
-        json_incref(root);
-        param->root = root;
+        param->root = json_object_ref(root);
     }
 
     return TRUE;
@@ -431,33 +425,39 @@ try_to_reorder(gchar **names, gint *order, gint n,
 }
 
 static gboolean
-handle_rtfd(PSPPTFile *pfile, json_t *root, GError **error)
+handle_rtfd(PSPPTFile *pfile, JsonObject *root, GError **error)
 {
-    json_t *type, *info, *numbers, *channels, *indices, *fast, *slow, *padding, *item;
-    guint n, i, ri, nnum, col, row, npts, base64len = 0;
+    JsonNode *type, *info_node, *numbers_node, *channels_node, *indices_node, *fast, *slow, *padding;
     gint power10;
     GString *str = pfile->str;
     GwyLawn *lawn;
-    gboolean G_GNUC_UNUSED pad, lawn_is_new = FALSE;
-    gdouble *d;
+    gboolean lawn_is_new = FALSE;
 
-    if (!get_json_with_type(root, &type, "type", JSON_STRING, error)
-        || !get_json_with_type(root, &info, "info", JSON_OBJECT, error)
-        || !get_json_with_type(root, &numbers, "numbers", JSON_ARRAY, error)
-        || !get_json_with_type(info, &channels, "channels", JSON_ARRAY, error)
-        || !get_json_with_type(info, &indices, "index", JSON_OBJECT, error)
-        || !get_json_with_type(info, &padding, "padding", JSON_TRUE, error)
-        || !get_json_with_type(indices, &fast, "fast", JSON_INTEGER, error)
-        || !get_json_with_type(indices, &slow, "slow", JSON_INTEGER, error))
+    if (!get_json_with_type(root, &type, "type", G_TYPE_STRING, error)
+        || !get_json_with_type(root, &info_node, "info", JSON_TYPE_OBJECT, error)
+        || !get_json_with_type(root, &numbers_node, "numbers", JSON_TYPE_ARRAY, error))
         return FALSE;
-    if (!gwy_strequal(json_string_value(type), "ppt.rtfd"))
-        return err_JSON_STRUCTURE(error, "ppt.rtfd.type", "ppt.rtfd");
 
-    n = json_array_size(channels);
-    nnum = json_array_size(numbers);
-    col = json_integer_value(fast);
-    row = json_integer_value(slow);
-    pad = json_boolean_value(padding);
+    JsonObject *info = json_node_get_object(info_node);
+    if (!get_json_with_type(info, &channels_node, "channels", JSON_TYPE_ARRAY, error)
+        || !get_json_with_type(info, &padding, "padding", G_TYPE_BOOLEAN, error)
+        || !get_json_with_type(info, &indices_node, "index", JSON_TYPE_OBJECT, error))
+        return FALSE;
+
+    JsonObject *indices = json_node_get_object(indices_node);
+    if (!get_json_with_type(indices, &fast, "fast", G_TYPE_INT64, error)
+        || !get_json_with_type(indices, &slow, "slow", G_TYPE_INT64, error))
+        return FALSE;
+    if (!gwy_strequal(json_node_get_string(type), "ppt.rtfd"))
+        return err_JSON_STRUCTURE(error, "ppt.rtfd.type", 0, "ppt.rtfd");
+
+    JsonArray *channels = json_node_get_array(channels_node);
+    JsonArray *numbers = json_node_get_array(numbers_node);
+    guint n = json_array_get_length(channels);
+    guint nnum = json_array_get_length(numbers);
+    guint col = json_node_get_int(fast);
+    guint row = json_node_get_int(slow);
+    gboolean G_GNUC_UNUSED pad = json_node_get_boolean(padding);
     gwy_debug("(%u,%u) nchannels = %u, nnumbers = %u, padding = %d", col, row, n, nnum, pad);
 
     /* The first time we encounter a spectrum set use it as a template.  All other sets must follow the same
@@ -472,7 +472,7 @@ handle_rtfd(PSPPTFile *pfile, json_t *root, GError **error)
         lawn_is_new = TRUE;
         gwy_debug("creating lawn xres=%d, yres=%d", ss->xres, ss->yres);
         lawn = pfile->lawn = gwy_lawn_new(ss->xres, ss->yres, ss->xreal, ss->yreal, n, 0);
-        gwy_si_unit_set_from_string(gwy_lawn_get_si_unit_xy(lawn), "m");
+        gwy_unit_set_from_string(gwy_lawn_get_unit_xy(lawn), "m");
         /* These duplicates lawn's properties, but can be used with check_string_list_item(). */
         pfile->units = g_new0(gchar*, n+1);
         pfile->ids = g_new0(gchar*, n+1);
@@ -480,58 +480,62 @@ handle_rtfd(PSPPTFile *pfile, json_t *root, GError **error)
         pfile->reorder = g_new(gint, n);
     }
 
-    for (i = 0; i < n; i++) {
-        item = json_array_get(channels, i);
-        if (!json_is_object(item))
-            return err_JSON_STRUCTURE(error, "channels.item", "object");
+    guint base64len = 0;
+    for (guint i = 0; i < n; i++) {
+        JsonNode *item = json_array_get_element(channels, i);
+        if (!JSON_NODE_HOLDS_OBJECT(item))
+            return err_JSON_STRUCTURE(error, "channels.item", JSON_TYPE_OBJECT, NULL);
 
-        if (!check_string_list_item(item, "id", pfile->ids, i, error))
+        JsonObject *object = json_node_get_object(item);
+        if (!check_string_list_item(object, "id", pfile->ids, i, error))
             return FALSE;
-        if (!check_string_list_item(item, "unit", pfile->units, i, error))
+        if (!check_string_list_item(object, "unit", pfile->units, i, error))
             return FALSE;
 
-        item = json_array_get(numbers, i);
-        if (!json_is_string(item))
-            return err_JSON_STRUCTURE(error, "numbers.item", "string");
+        item = json_array_get_element(numbers, i);
+        const gchar *base64data = json_node_get_string(item);
+        if (!base64data)
+            return err_JSON_STRUCTURE(error, "numbers.item", G_TYPE_STRING, NULL);
 
         if (!i)
-            base64len = json_string_length(item);
-        else if (json_string_length(item) != base64len)
+            base64len = strlen(base64data);
+        else if (strlen(base64data) != base64len)
             return err_INCONSISTENT_SPECTRA(error);
     }
 
     if (lawn_is_new) {
-        for (i = 0; i < n; i++)
+        for (guint i = 0; i < n; i++)
             pfile->reorder[i] = i;
         try_to_reorder(pfile->ids, pfile->reorder, n, "Force", 0);
         try_to_reorder(pfile->ids, pfile->reorder, n, "ZHeight", 0);
         try_to_reorder(pfile->ids, pfile->reorder, n, "Lfm", n-1);
-        for (i = 0; i < n; i++) {
-            ri = pfile->reorder[i];
+        for (guint i = 0; i < n; i++) {
+            guint ri = pfile->reorder[i];
             gwy_lawn_set_curve_label(lawn, ri, pfile->ids[i]);
-            gwy_si_unit_set_from_string_parse(gwy_lawn_get_si_unit_curve(lawn, ri), pfile->units[i], &power10);
-            pfile->power10[i] = pow10(power10);
+            power10 = gwy_unit_set_from_string(gwy_lawn_get_unit_curve(lawn, ri), pfile->units[i]);
+            pfile->power10[i] = gwy_exp10(power10);
         }
     }
 
     g_array_set_size(pfile->databuf, 0);
-    for (i = 0; i < n; i++) {
-        ri = pfile->reorder[i];
-        item = json_array_get(numbers, i);
-        g_string_assign(str, json_string_value(item));
+    for (guint i = 0; i < n; i++) {
+        guint ri = pfile->reorder[i];
+        JsonNode *item = json_array_get_element(numbers, i);
+        g_string_assign(str, json_node_get_string(item));
         g_base64_decode_inplace(str->str, &str->len);
-        npts = str->len/sizeof(gfloat);
+        guint npts = str->len/sizeof(gfloat);
         if (i) {
             if (npts != pfile->databuf->len/n)
                 return err_INCONSISTENT_SPECTRA(error);
         }
         else
             g_array_set_size(pfile->databuf, n*npts);
-        d = &g_array_index(pfile->databuf, gdouble, npts*pfile->reorder[i]);
+
+        gdouble *d = &g_array_index(pfile->databuf, gdouble, npts*ri);
         gwy_convert_raw_data(str->str, npts, 1, GWY_RAW_DATA_FLOAT, GWY_BYTE_ORDER_LITTLE_ENDIAN, d,
                              pfile->power10[i], 0.0);
     }
-    npts = pfile->databuf->len/n;
+    guint npts = pfile->databuf->len/n;
     gwy_debug("items per curve %u (%u items total)", npts, pfile->databuf->len);
     gwy_lawn_set_curves(lawn, col, row, npts, &g_array_index(pfile->databuf, gdouble, 0), NULL);
 
@@ -539,64 +543,77 @@ handle_rtfd(PSPPTFile *pfile, json_t *root, GError **error)
 }
 
 static gboolean
-check_string_list_item(json_t *root, const gchar *name, gchar **values, guint i, GError **error)
+check_string_list_item(JsonObject *root, const gchar *name, gchar **values, guint i, GError **error)
 {
-    json_t *item;
+    JsonNode *item;
 
-    if (!get_json_with_type(root, &item, name, JSON_STRING, error))
+    if (!get_json_with_type(root, &item, name, G_TYPE_STRING, error))
         return FALSE;
     if (!values[i])
-        values[i] = g_strdup(json_string_value(item));
-    else if (!gwy_strequal(json_string_value(item), values[i]))
+        values[i] = g_strdup(json_node_get_string(item));
+    else if (!gwy_strequal(json_node_get_string(item), values[i]))
         return err_INCONSISTENT_SPECTRA(error);
     return TRUE;
 }
 
-static void
-add_one_meta(GwyContainer *meta, json_t *object, GString *path)
+static gboolean
+format_atomic_json_meta(JsonNode *node, GwyContainer *meta, const gchar *path)
 {
-    guint len = path->len;
-    const gchar *key;
-    void *iter;
-    json_t *value;
+    GType type = json_node_get_value_type(node);
 
-    if (json_is_object(object)) {
-        g_string_append(path, "::");
-        for (iter = json_object_iter(object); iter; iter = json_object_iter_next(object, iter)) {
-            if ((value = json_object_iter_value(iter))) {
-                key = json_object_iter_key(iter);
-                if (gwy_strequal(key, "type"))
-                    continue;
-                g_string_append(path, key);
-                add_one_meta(meta, value, path);
-                g_string_truncate(path, len+2);
-            }
-        }
-        g_string_truncate(path, len);
-    }
-    else if (json_is_string(object))
-        gwy_container_set_const_string_by_name(meta, path->str, json_string_value(object));
-    else if (json_is_integer(object))
-        gwy_container_set_string_by_name(meta, path->str, g_strdup_printf("%ld", (glong)json_integer_value(object)));
-    else if (json_is_real(object))
-        gwy_container_set_string_by_name(meta, path->str, g_strdup_printf("%g", json_real_value(object)));
-    else if (json_is_true(object))
-        gwy_container_set_const_string_by_name(meta, path->str, "True");
-    else if (json_is_false(object))
-        gwy_container_set_const_string_by_name(meta, path->str, "False");
-    else {
-        g_warning("Unhandled metadata of type %d.", json_typeof(object));
-    }
+    if (type == G_TYPE_STRING)
+        gwy_container_set_const_string_by_name(meta, path, json_node_get_string(node));
+    else if (type == G_TYPE_INT64)
+        gwy_container_set_string_by_name(meta, path, g_strdup_printf("%" G_GINT64_FORMAT, json_node_get_int(node)));
+    else if (type == G_TYPE_DOUBLE)
+        gwy_container_set_string_by_name(meta, path, g_strdup_printf("%g", json_node_get_double(node)));
+    else if (type == G_TYPE_BOOLEAN)
+        gwy_container_set_const_string_by_name(meta, path, json_node_get_boolean(node) ? "True" : "False");
+    else
+        return FALSE;
+
+    return TRUE;
 }
 
-static GwyContainer*
-psppt_load(const gchar *filename,
-           G_GNUC_UNUSED GwyRunType mode,
-           GError **error)
+static void
+add_one_meta(GwyContainer *meta, JsonObject *object, GString *path)
 {
-    GwyContainer *container = NULL;
+    guint len = path->len;
+
+    g_string_append(path, "::");
+
+    JsonObjectIter iter;
+    json_object_iter_init(&iter, object);
+
+    const gchar *name;
+    JsonNode *node;
+    while (json_object_iter_next(&iter, &name, &node)) {
+        if (gwy_strequal(name, "type"))
+            continue;
+
+        g_string_append(path, name);
+
+        if (format_atomic_json_meta(node, meta, path->str)) {
+            /* pass */
+        }
+        else if (JSON_NODE_HOLDS_OBJECT(node))
+            add_one_meta(meta, json_node_get_object(node), path);
+        else {
+            g_warning("Unhandled metadata of type %s.", g_type_name(json_node_get_value_type(node)));
+        }
+        g_string_truncate(path, len+2);
+    }
+    g_string_truncate(path, len);
+}
+
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
+{
+    GwyFile *file = NULL;
     PSPPTFile pfile;
-    json_t *root = NULL;
+    JsonParser *parser = NULL;
     guchar *buffer = NULL;
     GError *err = NULL;
     gsize pos, size = 0;
@@ -608,7 +625,7 @@ psppt_load(const gchar *filename,
         return NULL;
     }
 
-    gwy_clear(&pfile, 1);
+    gwy_clear1(pfile);
     pfile.str = g_string_new(NULL);
     pfile.databuf = g_array_new(FALSE, FALSE, sizeof(gdouble));
 
@@ -628,6 +645,8 @@ psppt_load(const gchar *filename,
         err_CANCELLED(error);
         goto fail;
     }
+
+    parser = json_parser_new_immutable();
     for (i = 0; i < pfile.nframes; i++) {
         PSPPTFrame *frame = pfile.frames + i;
         gboolean ok;
@@ -636,7 +655,8 @@ psppt_load(const gchar *filename,
             err_CANCELLED(error);
             goto fail;
         }
-        if (!(root = psppt_read_frame(frame, buffer, error)))
+        JsonObject *root;
+        if (!(root = psppt_read_frame(parser, frame, buffer, error)))
             goto fail;
 
         /* The types have been checked so we can be a bit less paranoid here. */
@@ -650,46 +670,44 @@ psppt_load(const gchar *filename,
             ok = handle_rtfd(&pfile, root, error);
         if (!ok)
             goto fail;
-        json_decref(root);
-        root = NULL;
     }
 
-    pfile.meta = gwy_container_new();
+    pfile.meta = gwy_container_new_in_construction();
     g_string_assign(pfile.str, "Param");
     add_one_meta(pfile.meta, pfile.param.root, pfile.str);
     g_string_assign(pfile.str, "Scan");
     add_one_meta(pfile.meta, pfile.scanstart.root, pfile.str);
 
-    container = gwy_container_new();
-    gwy_container_set_object(container, gwy_app_get_lawn_key_for_id(0), pfile.lawn);
-    gwy_container_set_const_string(container, gwy_app_get_lawn_title_key_for_id(0), pfile.scanstart.direction);
-    gwy_container_set_object(container, gwy_app_get_lawn_meta_key_for_id(0), pfile.meta);
-    gwy_file_curve_map_import_log_add(container, 0, NULL, filename);
+    file = gwy_file_new_in_construction();
+    gwy_file_set_cmap(file, 0, pfile.lawn);
+    gwy_file_set_title(file, GWY_FILE_CMAP, 0, pfile.scanstart.direction, FALSE);
+    gwy_file_pass_meta(file, GWY_FILE_CMAP, 0, pfile.meta);
+    pfile.meta = NULL;
+    gwy_log_add_import(file, GWY_FILE_CMAP, 0, NULL, filename);
 
 fail:
     if (waiting)
         gwy_app_wait_finish();
-    if (root)
-        json_decref(root);
     if (pfile.scanstart.root)
-        json_decref(pfile.scanstart.root);
+        json_object_unref(pfile.scanstart.root);
     if (pfile.scanstop.root)
-        json_decref(pfile.scanstop.root);
+        json_object_unref(pfile.scanstop.root);
     if (pfile.param.root)
-        json_decref(pfile.param.root);
+        json_object_unref(pfile.param.root);
+    g_clear_object(&parser);
     g_free(pfile.scanstart.direction);
     g_free(pfile.power10);
     g_free(pfile.reorder);
     g_strfreev(pfile.ids);
     g_strfreev(pfile.units);
     g_free(pfile.frames);
-    GWY_OBJECT_UNREF(pfile.lawn);
-    GWY_OBJECT_UNREF(pfile.meta);
+    g_clear_object(&pfile.lawn);
+    g_clear_object(&pfile.meta);
     g_array_free(pfile.databuf, TRUE);
     g_string_free(pfile.str, TRUE);
     gwy_file_abandon_contents(buffer, size, NULL);
 
-    return container;
+    return file;
 }
 
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
