@@ -1,6 +1,6 @@
 /*
- *  $Id: pt3file.c 26510 2024-08-15 17:12:23Z yeti-dn $
- *  Copyright (C) 2009 David Necas (Yeti).
+ *  $Id: pt3file.c 28789 2025-11-04 17:14:03Z yeti-dn $
+ *  Copyright (C) 2009-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -42,13 +42,8 @@
 
 #include <string.h>
 #include "config.h"
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/datafield.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/data-browser.h>
-#include <app/gwymoduleutils-file.h>
+#include <glib/gi18n-lib.h>
+#include <gwy.h>
 
 #include "get.h"
 #include "err.h"
@@ -255,16 +250,16 @@ typedef struct {
 } LineTrigger;
 
 static gboolean       module_register           (void);
-static gint           pt3file_detect            (const GwyFileDetectInfo *fileinfo,
+static gint           detect_file               (const GwyFileDetectInfo *fileinfo,
                                                  gboolean only_name);
-static GwyContainer*  pt3file_load              (const gchar *filename,
-                                                 GwyRunType mode,
+static GwyFile*       load_file                 (const gchar *filename,
+                                                 GwyRunModeFlags mode,
                                                  GError **error);
-static gsize          pt3file_read_header       (const guchar *buffer,
+static gsize          read_header               (const guchar *buffer,
                                                  gsize size,
                                                  PicoHarpFile *pt3file,
                                                  GError **error);
-static const guchar*  pt3file_read_board        (PicoHarpBoard *board,
+static const guchar*  read_board                (PicoHarpBoard *board,
                                                  const guchar *p);
 static const guchar*  read_pie710_imaging_header(PicoHarpImagingHeader *header,
                                                  const guchar *p);
@@ -272,22 +267,22 @@ static const guchar*  read_kdt180_imaging_header(PicoHarpImagingHeader *header,
                                                  const guchar *p);
 static const guchar*  read_lsm_imaging_header   (PicoHarpImagingHeader *header,
                                                  const guchar *p);
-static LineTrigger*   pt3file_scan_line_triggers(const PicoHarpFile *pt3file,
+static LineTrigger*   scan_line_triggers        (const PicoHarpFile *pt3file,
                                                  const guchar *p,
                                                  GError **error);
-static GwyDataField*  pt3file_extract_counts    (const PicoHarpFile *pt3file,
+static GwyField*      extract_counts            (const PicoHarpFile *pt3file,
                                                  const LineTrigger *linetriggers,
                                                  const guchar *p);
-static GwyGraphModel* pt3file_extract_decay     (const PicoHarpFile *pt3file,
+static GwyGraphModel* extract_decay             (const PicoHarpFile *pt3file,
                                                  const guchar *p);
-static GwyContainer*  pt3file_get_metadata      (PicoHarpFile *pt3file);
+static GwyContainer*  get_metadata              (PicoHarpFile *pt3file);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports SymPhoTime data files, version 2.0."),
     "Yeti <yeti@gwyddion.net>",
-    "0.2",
+    "0.3",
     "David Nečas (Yeti)",
     "2009",
 };
@@ -299,17 +294,13 @@ module_register(void)
 {
     gwy_file_func_register("pt3file",
                            N_("PicoHarp files (.pt3)"),
-                           (GwyFileDetectFunc)&pt3file_detect,
-                           (GwyFileLoadFunc)&pt3file_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-pt3file_detect(const GwyFileDetectInfo *fileinfo,
-                gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     gint score = 0;
 
@@ -354,14 +345,13 @@ read_t3_record(PicoHarpT3Record *rec,
     return p;
 }
 
-static GwyContainer*
-pt3file_load(const gchar *filename,
-             G_GNUC_UNUSED GwyRunType mode,
-             GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
     PicoHarpFile pt3file;
-    GwyContainer *container = NULL;
-    GwyDataField *dfield = NULL;
+    GwyFile *file = NULL;
     guchar *buffer = NULL;
     const guchar *p;
     gsize header_len, size = 0;
@@ -373,8 +363,8 @@ pt3file_load(const gchar *filename,
         return NULL;
     }
 
-    gwy_clear(&pt3file, 1);
-    if (!(header_len = pt3file_read_header(buffer, size, &pt3file, error)))
+    gwy_clear1(pt3file);
+    if (!(header_len = read_header(buffer, size, &pt3file, error)))
         goto fail;
 
     if (err_SIZE_MISMATCH
@@ -396,30 +386,29 @@ pt3file_load(const gchar *filename,
 
     /* Scan the records and find the line triggers */
     p = buffer + header_len;
-    if (!(linetriggers = pt3file_scan_line_triggers(&pt3file, p, error)))
+    if (!(linetriggers = scan_line_triggers(&pt3file, p, error)))
         goto fail;
 
-    container = gwy_container_new();
-
-    dfield = pt3file_extract_counts(&pt3file, linetriggers, p);
-    gwy_container_pass_object(container, gwy_app_get_data_key_for_id(0), dfield);
-    gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(0), "Photon count");
-    gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(0), pt3file_get_metadata(&pt3file));
-    gwy_file_channel_import_log_add(container, 0, NULL, filename);
-    gwy_container_pass_object(container, gwy_app_get_graph_key_for_id(1), pt3file_extract_decay(&pt3file, p));
+    file = gwy_file_new_in_construction();
+    gwy_file_pass_image(file, 0, extract_counts(&pt3file, linetriggers, p));
+    gwy_file_set_title(file, GWY_FILE_IMAGE, 0, "Photon count", FALSE);
+    gwy_file_pass_meta(file, GWY_FILE_IMAGE, 0, get_metadata(&pt3file));
+    gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
+    gwy_file_pass_graph(file, 0, extract_decay(&pt3file, p));
+    gwy_log_add_import(file, GWY_FILE_GRAPH, 0, NULL, filename);
 
 fail:
     g_free(linetriggers);
     gwy_file_abandon_contents(buffer, size, NULL);
 
-    return container;
+    return file;
 }
 
 static gsize
-pt3file_read_header(const guchar *buffer,
-                    gsize size,
-                    PicoHarpFile *pt3file,
-                    GError **error)
+read_header(const guchar *buffer,
+            gsize size,
+            PicoHarpFile *pt3file,
+            GError **error)
 {
     const guchar* (*read_imaging_header)(PicoHarpImagingHeader *header,
                                          const guchar *p);
@@ -497,7 +486,7 @@ pt3file_read_header(const guchar *buffer,
     pt3file->repeat_time = gwy_get_guint32_le(&p);
     pt3file->repeat_wait_time = gwy_get_guint32_le(&p);
     get_CHARARRAY(pt3file->script_name, &p);
-    p = pt3file_read_board(&pt3file->board, p);
+    p = read_board(&pt3file->board, p);
     p += BOARD_SIZE*(pt3file->number_of_boards - 1);
     pt3file->ext_devices = gwy_get_guint32_le(&p);
     pt3file->reserved1 = gwy_get_guint32_le(&p);
@@ -578,8 +567,8 @@ pt3file_read_header(const guchar *buffer,
 }
 
 static const guchar*
-pt3file_read_board(PicoHarpBoard *board,
-                   const guchar *p)
+read_board(PicoHarpBoard *board,
+           const guchar *p)
 {
     guint i;
 
@@ -668,9 +657,9 @@ read_lsm_imaging_header(PicoHarpImagingHeader *header,
 }
 
 static LineTrigger*
-pt3file_scan_line_triggers(const PicoHarpFile *pt3file,
-                           const guchar *p,
-                           GError **error)
+scan_line_triggers(const PicoHarpFile *pt3file,
+                   const guchar *p,
+                   GError **error)
 {
     LineTrigger *linetriggers;
     guint64 globaltime, globalbase;
@@ -746,12 +735,12 @@ pt3file_scan_line_triggers(const PicoHarpFile *pt3file,
     return linetriggers;
 }
 
-static GwyDataField*
-pt3file_extract_counts(const PicoHarpFile *pt3file,
-                       const LineTrigger *linetriggers,
-                       const guchar *p)
+static GwyField*
+extract_counts(const PicoHarpFile *pt3file,
+               const LineTrigger *linetriggers,
+               const guchar *p)
 {
-    GwyDataField *dfield;
+    GwyField *dfield;
     guint xres, yres, i, lineno, pixno, n;
     guint64 globaltime, globalbase, start, stop;
     gdouble pix_resol;
@@ -770,9 +759,9 @@ pt3file_extract_counts(const PicoHarpFile *pt3file,
     sanitise_real_size(&pix_resol, "pixel size");
     pix_resol *= 1e-6;
 
-    dfield = gwy_data_field_new(xres, yres, pix_resol*xres, pix_resol*yres, TRUE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
-    d = gwy_data_field_get_data(dfield);
+    dfield = gwy_field_new(xres, yres, pix_resol*xres, pix_resol*yres, TRUE);
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
+    d = gwy_field_get_data(dfield);
 
     lineno = 0;
     start = linetriggers[lineno].start;
@@ -810,12 +799,12 @@ pt3file_extract_counts(const PicoHarpFile *pt3file,
 }
 
 static GwyGraphModel*
-pt3file_extract_decay(const PicoHarpFile *pt3file,
-                      const guchar *buf)
+extract_decay(const PicoHarpFile *pt3file,
+              const guchar *buf)
 {
     GwyGraphModel *gmodel;
     GwyGraphCurveModel *gcmodel;
-    GwySIUnit *siunit;
+    GwyUnit *siunit;
     gdouble *xdata, *ydata;
     guint maxtime, n, i;
     const guchar *p;
@@ -853,12 +842,12 @@ pt3file_extract_decay(const PicoHarpFile *pt3file,
     g_free(ydata);
     g_free(xdata);
 
-    siunit = gwy_si_unit_new("s");
+    siunit = gwy_unit_new("s");
 
     gmodel = gwy_graph_model_new();
     g_object_set(gmodel,
                  "title", "Fluorescence decay",
-                 "si-unit-x", siunit,
+                 "unit-x", siunit,
                  "axis-label-bottom", "time",
                  "axis-label-left", "count",
                  NULL);
@@ -876,11 +865,9 @@ pt3file_extract_decay(const PicoHarpFile *pt3file,
     gwy_container_set_string_by_name(meta, name, g_strdup_printf("%u", pt3file->field))
 
 static GwyContainer*
-pt3file_get_metadata(PicoHarpFile *pt3file)
+get_metadata(PicoHarpFile *pt3file)
 {
-    GwyContainer *meta;
-
-    meta = gwy_container_new();
+    GwyContainer *meta = gwy_container_new_in_construction();
 
     add_meta_str("Format Version", format_version);
     add_meta_str("Creator Name", creator_name);
