@@ -1,6 +1,6 @@
 /*
- *  $Id: quazarnpic.c 26873 2024-11-19 12:52:14Z yeti-dn $
- *  Copyright (C) 2021 David Necas (Yeti).
+ *  $Id: quazarnpic.c 28911 2025-11-24 18:27:42Z yeti-dn $
+ *  Copyright (C) 2021-2025 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
  *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
@@ -29,15 +29,10 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/stats.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -356,19 +351,19 @@ static const GwyEnum objtype_names[] = {
 /* }}} */
 #endif
 
-static gboolean      module_register(void);
-static gint          npic_detect    (const GwyFileDetectInfo *fileinfo,
-                                     gboolean only_name);
-static GwyContainer* npic_load      (const gchar *filename,
-                                     GwyRunType mode,
-                                     GError **error);
+static gboolean module_register(void);
+static gint     detect_file    (const GwyFileDetectInfo *fileinfo,
+                                gboolean only_name);
+static GwyFile* load_file      (const gchar *filename,
+                                GwyRunModeFlags mode,
+                                GError **error);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Quazar data files stored as Python pickles v4."),
     "Yeti <yeti@gwyddion.net>",
-    "0.3",
+    "0.4",
     "David Nečas (Yeti)",
     "2021",
 };
@@ -380,18 +375,14 @@ module_register(void)
 {
     gwy_file_func_register("quazarnpic",
                            N_("Quazar Python-pickled data (.npic)"),
-                           (GwyFileDetectFunc)&npic_detect,
-                           (GwyFileLoadFunc)&npic_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
     gwy_file_func_set_is_unfinished("quazarnpic", TRUE);
 
     return TRUE;
 }
 
 static gint
-npic_detect(const GwyFileDetectInfo *fileinfo,
-            gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     static const gchar *paramnames[] = {
         "StepSize", "ImageSize", "ImageSizeUnit", "NoOfChannels", "ChannelNames", "Instrument",
@@ -536,7 +527,7 @@ free_arg(PickleArg *arg)
 static PickleObject*
 new_object(PickleObjectType type, PickleArg *arg)
 {
-    PickleObject *obj = g_slice_new0(PickleObject);
+    PickleObject *obj = g_new0(PickleObject, 1);
     obj->refcount = 1;
     obj->type = type;
     if (arg) {
@@ -572,7 +563,7 @@ free_object(PickleObject *obj)
             free_object(g_ptr_array_index(a, i));
         g_ptr_array_free(a, TRUE);
     }
-    g_slice_free(PickleObject, obj);
+    g_free(obj);
 }
 
 static guint
@@ -720,7 +711,7 @@ read_one_object(const guchar **p, gsize size, GError **error)
     gwy_debug("starting to read a new object");
     stack = g_ptr_array_new();
     memoized = g_ptr_array_new();
-    gwy_clear(&arg, 1);
+    gwy_clear1(arg);
     while (TRUE) {
         const PickleOpcode *opcode;
         PickleObject *obj = NULL;
@@ -1039,10 +1030,10 @@ check_global(PickleObject *obj, const gchar *module, const gchar *type)
  *         BOOL(False)
  *         BYTES(len=262144)
  */
-static GwyDataField*
+static GwyField*
 extract_image(PickleObject *root)
 {
-    GwyDataField *field;
+    GwyField *field;
     PickleObject *obj, *obj2, *obj3;
     gint xres, yres;
     GwyByteOrder byteorder;
@@ -1114,17 +1105,16 @@ extract_image(PickleObject *root)
         return NULL;
 
     gwy_debug("creating data field");
-    field = gwy_data_field_new(xres, yres, xres, yres, FALSE);
-    gwy_convert_raw_data(obj->v.s, xres*yres, 1, datatype, byteorder, gwy_data_field_get_data(field), 1.0, 0.0);
+    field = gwy_field_new(xres, yres, xres, yres, FALSE);
+    gwy_convert_raw_data(obj->v.s, xres*yres, 1, datatype, byteorder, gwy_field_get_data(field), 1.0, 0.0);
 
     return field;
 }
 
 static void
-gather_units(gpointer hkey, G_GNUC_UNUSED gpointer hvalue, gpointer user_data)
+gather_units(GQuark quark, G_GNUC_UNUSED GValue *gvalue, gpointer user_data)
 {
     GPtrArray *a = (GPtrArray*)user_data;
-    GQuark quark = GPOINTER_TO_UINT(hkey);
     GwyContainer *meta = g_ptr_array_index(a, 0);
     GString *str = g_ptr_array_index(a, 1);
     const gchar *strkey = g_quark_to_string(quark);
@@ -1197,7 +1187,7 @@ extract_metadata(PickleObject *root, NpicFileInfo *info)
 
     a = root->v.a;
     len = a->len;
-    meta = gwy_container_new();
+    meta = gwy_container_new_in_construction();
     str = g_string_new(NULL);
     for (i = 0; i+1 < len; i += 2) {
         keyobj = g_ptr_array_index(a, i);
@@ -1285,21 +1275,22 @@ extract_metadata(PickleObject *root, NpicFileInfo *info)
     return meta;
 }
 
-static GwyContainer*
-npic_load(const gchar *filename,
-          G_GNUC_UNUSED GwyRunType mode,
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
           GError **error)
 {
     NpicFileInfo info;
-    GwyContainer *container = NULL, *tmpmeta, *meta = NULL;
+    GwyFile *file = NULL;
+    GwyContainer *meta = NULL;
     GPtrArray *fields = NULL;
-    GwyDataField *field;
+    GwyField *field;
     guchar *buffer = NULL;
     const guchar *p;
     const gchar *dir;
     gchar *s;
     PickleObject *obj, *metaobj = NULL;
-    GwySIUnit *unit, *volt;
+    GwyUnit *unit, *volt;
     GError *err = NULL;
     gsize size;
     gint power10;
@@ -1311,7 +1302,7 @@ npic_load(const gchar *filename,
         return NULL;
     }
 
-    gwy_clear(&info, 1);
+    gwy_clear1(info);
     info.xcal = info.ycal = info.zcal = info.xstep = info.ystep = 1.0;
     fields = g_ptr_array_new();
     info.channel_names = g_ptr_array_new();
@@ -1336,57 +1327,57 @@ npic_load(const gchar *filename,
         goto fail;
     }
 
-    container = gwy_container_new();
-    unit = gwy_si_unit_new(NULL);
-    volt = gwy_si_unit_new("V");
+    file = gwy_file_new_in_construction();
+    unit = gwy_unit_new(NULL);
+    volt = gwy_unit_new("V");
     for (i = 0; i < fields->len; i++) {
         field = g_ptr_array_index(fields, i);
 
-        gwy_si_unit_set_from_string_parse(unit, info.xcalunit, &power10);
-        gwy_si_unit_multiply(unit, volt, unit);
-        q = pow10(power10);
-        gwy_data_field_set_xreal(field, q*info.xstep * info.xcal/100.0 * gwy_data_field_get_xres(field));
-        gwy_data_field_set_yreal(field, q*info.ystep * info.ycal/100.0 * gwy_data_field_get_yres(field));
-        gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(field), unit);
-        gwy_container_set_object(container, gwy_app_get_data_key_for_id(i), field);
+        power10 = gwy_unit_set_from_string(unit, info.xcalunit);
+        gwy_unit_multiply(unit, volt, unit);
+        q = gwy_exp10(power10);
+        gwy_field_set_xreal(field, q*info.xstep * info.xcal/100.0 * gwy_field_get_xres(field));
+        gwy_field_set_yreal(field, q*info.ystep * info.ycal/100.0 * gwy_field_get_yres(field));
+        gwy_unit_assign(gwy_field_get_unit_xy(field), unit);
+        gwy_file_set_image(file, i, field);
 
-        gwy_si_unit_set_from_string_parse(unit, info.zcalunit, &power10);
-        gwy_si_unit_multiply(unit, volt, unit);
-        q = pow10(power10);
+        power10 = gwy_unit_set_from_string(unit, info.zcalunit);
+        gwy_unit_multiply(unit, volt, unit);
+        q = gwy_exp10(power10);
         /* The minus is strange and I cannot find any negative calibration factor or something, but it seems to be
          * the only way to match screenshots. */
-        gwy_data_field_multiply(field, -q*info.zcal/100.0);
-        gwy_si_unit_assign(gwy_data_field_get_si_unit_z(field), unit);
+        gwy_field_multiply(field, -q*info.zcal/100.0);
+        gwy_unit_assign(gwy_field_get_unit_z(field), unit);
 
-        gwy_container_set_object(container, gwy_app_get_data_key_for_id(i), field);
-        gwy_file_channel_import_log_add(container, i, NULL, filename);
+        gwy_file_set_image(file, i, field);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, i, NULL, filename);
 
         dir = (i%2 ? "[Backward]" : "[Forward]");
         s = NULL;
         if (i/2 < info.channel_names->len)
             s = g_strconcat(g_ptr_array_index(info.channel_names, i/2), " ", dir, NULL);
-        else if (gwy_app_channel_title_fall_back(container, i))
-            s = g_strconcat(gwy_container_get_string(container, gwy_app_get_data_title_key_for_id(i)), " ", dir, NULL);
-        if (s)
-            gwy_container_set_string(container, gwy_app_get_data_title_key_for_id(i), s);
+        else if (gwy_image_title_fall_back(file, i))
+            s = g_strconcat(gwy_file_get_title(file, GWY_FILE_IMAGE, i), " ", dir, NULL);
 
-        tmpmeta = gwy_container_duplicate(meta);
-        gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(i), tmpmeta);
+        if (s)
+            gwy_file_pass_title(file, GWY_FILE_IMAGE, i, s);
+
+        gwy_file_pass_meta(file, GWY_FILE_IMAGE, i, gwy_container_copy(meta));
     }
     g_object_unref(unit);
     g_object_unref(volt);
 
 fail:
     free_object(metaobj);
-    GWY_OBJECT_UNREF(meta);
+    g_clear_object(&meta);
     for (i = 0; i < fields->len; i++)
-        GWY_OBJECT_UNREF(g_ptr_array_index(fields, i));
+        g_clear_object(&g_ptr_array_index(fields, i));
     g_ptr_array_free(fields, TRUE);
     g_ptr_array_free(info.channel_names, TRUE);
 
     gwy_file_abandon_contents(buffer, size, NULL);
 
-    return container;
+    return file;
 }
 
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
