@@ -1,6 +1,6 @@
 /*
- *  $Id: rawfile.c 28628 2025-09-30 12:17:05Z yeti-dn $
- *  Copyright (C) 2003-2022 David Necas (Yeti), Petr Klapetek.
+ *  $Id: rawfile.c 29029 2025-12-19 14:55:48Z yeti-dn $
+ *  Copyright (C) 2003-2025 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -35,19 +35,13 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
 #include <glib/gstdio.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <libprocess/stats.h>
-#include <libgwydgets/gwyinventorystore.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "../process/preview.h"
 
 #include "err.h"
@@ -164,13 +158,12 @@ typedef struct {
 typedef struct {
     GwyParams *params;
     RawFileFile file;
-    GwyDataField *field;
-    GwyDataField *mask;
+    GwyField *field;
+    GwyField *mask;
 } ModuleArgs;
 
 typedef struct {
     ModuleArgs *args;
-    GwyContainer *data;
     GtkWidget *dialog;
     GwyParamTable *table_dims;
     GwyParamTable *table_format;
@@ -186,9 +179,10 @@ static gboolean         module_register         (void);
 static GwyParamDef*     define_module_params    (void);
 static GwyParamDef*     define_preset_params    (void);
 static void             add_preset_params       (GwyParamDef *paramdef);
-static gint             rawfile_detect          (void);
-static GwyContainer*    rawfile_load            (const gchar *filename,
-                                                 GwyRunType runtype,
+static gint             detect_file             (const GwyFileDetectInfo *fileinfo,
+                                                 gboolean only_name);
+static GwyFile*         load_file               (const gchar *filename,
+                                                 GwyRunModeFlags mode,
                                                  GError **error);
 static GwyDialogOutcome run_gui                 (ModuleArgs *args);
 static GtkWidget*       preview_box_new         (ModuleGUI *gui);
@@ -198,7 +192,7 @@ static GtkWidget*       preset_tab_new          (ModuleGUI *gui);
 static void             param_changed           (ModuleGUI *gui,
                                                  gint id);
 static void             preview                 (gpointer user_data);
-static GwyDataField*    read_data_field         (ModuleArgs *args,
+static GwyField*        read_field              (ModuleArgs *args,
                                                  GError **error);
 static void             preset_selected         (ModuleGUI *gui);
 static void             load_preset             (ModuleGUI *gui);
@@ -257,7 +251,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports raw data files, both ASCII and binary, according to user-specified format."),
     "Yeti <yeti@gwyddion.net>",
-    "3.3",
+    "4.0",
     "David Nečas (Yeti) & Petr Klapetek",
     "2003",
 };
@@ -275,10 +269,7 @@ module_register(void)
 
     gwy_file_func_register("rawfile",
                            N_("Raw data files"),
-                           (GwyFileDetectFunc)&rawfile_detect,
-                           (GwyFileLoadFunc)&rawfile_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
     /* We provide a detection function, but that's only to become the fallback loading method. */
     gwy_file_func_set_is_detectable("rawfile", FALSE);
 
@@ -358,15 +349,15 @@ add_preset_params(GwyParamDef *paramdef)
     gwy_param_def_add_gwyenum(paramdef, PARAM_FORMAT, "format", NULL, formats, G_N_ELEMENTS(formats), RAW_BINARY);
 
     /* Text. */
-    gwy_param_def_add_int(paramdef, PARAM_LINE_OFFSET, "lineoffset", _("Start from file _line"), 0, 1 << 28, 0);
-    gwy_param_def_add_int(paramdef, PARAM_SKIP_FIELDS, "skipfields", _("_Each image row skip"), 0, 1 << 28, 0);
+    gwy_param_def_add_int(paramdef, PARAM_LINE_OFFSET, "lineoffset", _("Start from _line"), 0, 1 << 28, 0);
+    gwy_param_def_add_int(paramdef, PARAM_SKIP_FIELDS, "skipfields", _("_Each row skip"), 0, 1 << 28, 0);
     gwy_param_def_add_boolean(paramdef, PARAM_ONE_PER_LINE, "oneperline", _("One _value per line"), FALSE);
     gwy_param_def_add_int(paramdef, PARAM_COLUMN, "column", _("E_xtract column"), 1, 1 << 10, 1);
     gwy_param_def_add_string(paramdef, PARAM_DELIMITER, "delimiter", _("_Other delimiter"),
                              GWY_PARAM_STRING_NULL_IS_EMPTY | GWY_PARAM_STRING_DO_NOT_STRIP, NULL, "");
     gwy_param_def_add_boolean(paramdef, PARAM_DECOMMA, "decomma", _("_Decimal separator is comma"), FALSE);
 
-    /* Binary */
+    /* Binary. */
     gwy_param_def_add_gwyenum(paramdef, PARAM_BUILTIN, "builtin", _("Data t_ype"),
                               builtins, G_N_ELEMENTS(builtins), RAW_UNSIGNED_BYTE);
     gwy_param_def_add_int(paramdef, PARAM_BYTESWAP, "byteswap", _("Byte s_wap pattern"), 0, 64, 0);
@@ -380,7 +371,8 @@ add_preset_params(GwyParamDef *paramdef)
 }
 
 static gint
-rawfile_detect(void)
+detect_file(G_GNUC_UNUSED const GwyFileDetectInfo *fileinfo,
+            G_GNUC_UNUSED gboolean only_name)
 {
     gboolean takeover = FALSE;
 
@@ -390,24 +382,24 @@ rawfile_detect(void)
     return takeover ? 1 : 0;
 }
 
-static GwyContainer*
-rawfile_load(const gchar *filename,
-             GwyRunType runtype,
-             GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          GwyRunModeFlags mode,
+          GError **error)
 {
     ModuleArgs args;
-    GwyContainer *data = NULL;
+    GwyFile *file = NULL;
     GwyDialogOutcome outcome;
     GError *err = NULL;
     gsize size = 0;
 
-    if (runtype != GWY_RUN_INTERACTIVE) {
+    if (mode !=GWY_RUN_INTERACTIVE) {
         g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_INTERACTIVE,
                     _("Raw data import must be run as interactive."));
         return FALSE;
     }
 
-    gwy_clear(&args, 1);
+    gwy_clear1(args);
     args.params = gwy_params_new_from_settings(define_module_params());
     sanitise_params(args.params, TRUE);
 
@@ -417,7 +409,7 @@ rawfile_load(const gchar *filename,
     }
     args.file.filename = filename;
     args.file.filesize = size;
-    args.field = gwy_data_field_new(PREVIEW_SMALL_SIZE, PREVIEW_SMALL_SIZE, 1.0, 1.0, TRUE);
+    args.field = gwy_field_new(PREVIEW_SMALL_SIZE, PREVIEW_SMALL_SIZE, 1.0, 1.0, TRUE);
 
     outcome = run_gui(&args);
 
@@ -427,18 +419,18 @@ rawfile_load(const gchar *filename,
         goto end;
     }
 
-    data = gwy_container_new();
-    gwy_container_set_object(data, gwy_app_get_data_key_for_id(0), args.field);
+    file = gwy_file_new_in_construction();
+    gwy_file_set_image(file, 0, args.field);
     if (args.mask)
-        gwy_container_pass_object(data, gwy_app_get_mask_key_for_id(0), args.mask);
-    gwy_file_channel_import_log_add(data, 0, NULL, filename);
+        gwy_file_pass_image_mask(file, 0, args.mask);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
 
 end:
-    GWY_OBJECT_UNREF(args.params);
-    GWY_OBJECT_UNREF(args.field);
+    g_clear_object(&args.params);
+    g_clear_object(&args.field);
     g_free(args.file.buffer);
 
-    return data;
+    return file;
 }
 
 static GwyDialogOutcome
@@ -449,17 +441,15 @@ run_gui(ModuleArgs *args)
     ModuleGUI gui;
     GwyDialogOutcome outcome;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
-    gui.data = gwy_container_new();
-    gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(0), args->field);
 
     gui.dialog = gwy_dialog_new(_("Read Raw File"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_UPDATE, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
     gtk_dialog_set_response_sensitive(GTK_DIALOG(dialog), GTK_RESPONSE_OK, FALSE);
 
-    hbox = gwy_hbox_new(8);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 8);
     gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
     gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), hbox, TRUE, TRUE, 0);
 
@@ -480,8 +470,6 @@ run_gui(ModuleArgs *args)
 
     outcome = gwy_dialog_run(dialog);
 
-    g_object_unref(gui.data);
-
     return outcome;
 }
 
@@ -490,17 +478,17 @@ preview_box_new(ModuleGUI *gui)
 {
     GtkWidget *label, *vbox;
 
-    vbox = gwy_vbox_new(2);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 2);
 
     label = gtk_label_new(_("Preview"));
-    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+    gtk_label_set_xalign(GTK_LABEL(label), 0.0);
     gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
 
-    gui->dataview = gwy_create_preview(gui->data, 0, PREVIEW_SMALL_SIZE, FALSE);
+    gui->dataview = gwy_create_preview(gui->args->field, NULL, PREVIEW_SMALL_SIZE);
     gtk_box_pack_start(GTK_BOX(vbox), gui->dataview, FALSE, FALSE, 0);
 
     gui->error = gtk_label_new(NULL);
-    gtk_misc_set_alignment(GTK_MISC(gui->error), 0.0, 0.5);
+    gtk_label_set_xalign(GTK_LABEL(gui->error), 0.0);
     gtk_label_set_line_wrap(GTK_LABEL(gui->error), TRUE);
     gtk_widget_set_size_request(gui->error, PREVIEW_SMALL_SIZE, -1);
     gtk_box_pack_start(GTK_BOX(vbox), gui->error, FALSE, FALSE, 0);
@@ -513,8 +501,8 @@ dimensions_tab_new(ModuleGUI *gui)
 {
     ModuleArgs *args = gui->args;
     GwyParamTable *table;
-    GwySIUnit *unit;
-    GwySIValueFormat *vf;
+    GwyUnit *unit;
+    GwyValueFormat *vf;
     gchar *s, *ss;
 
     table = gui->table_dims = gwy_param_table_new(args->params);
@@ -527,10 +515,10 @@ dimensions_tab_new(ModuleGUI *gui)
     g_free(ss);
 
     gwy_param_table_info_set_add_punctuation(table, INFO_FILE, FALSE);
-    unit = gwy_si_unit_new("B");
-    vf = gwy_si_unit_get_format(unit, GWY_SI_UNIT_FORMAT_VFMARKUP, args->file.filesize, NULL);
+    unit = gwy_unit_new("B");
+    vf = gwy_unit_get_format(unit, GWY_UNIT_FORMAT_VFMARKUP, args->file.filesize, NULL);
     s = g_strdup_printf("%.*f %s", vf->precision, args->file.filesize/vf->magnitude, vf->units);
-    gwy_si_unit_value_format_free(vf);
+    gwy_value_format_free(vf);
     gwy_param_table_info_set_valuestr(table, INFO_FILE, s);
     g_object_unref(unit);
     g_free(s);
@@ -679,10 +667,10 @@ preset_tab_new(ModuleGUI *gui)
         GCallback callback;
     }
     buttons[PRESET_NBUTTONS] = {
-        { N_("verb|_Load"),  G_CALLBACK(load_preset),   },
-        { N_("verb|_Store"), G_CALLBACK(save_preset),   },
-        { N_("_Rename"),     G_CALLBACK(rename_preset), },
-        { N_("_Delete"),     G_CALLBACK(delete_preset), },
+        { gwy_NC("verb", "_Load"),  G_CALLBACK(load_preset),   },
+        { gwy_NC("verb", "_Store"), G_CALLBACK(save_preset),   },
+        { N_("_Rename"),            G_CALLBACK(rename_preset), },
+        { N_("_Delete"),            G_CALLBACK(delete_preset), },
     };
 
     static const GwyEnum columns[] = {
@@ -699,13 +687,12 @@ preset_tab_new(ModuleGUI *gui)
     GtkWidget *vbox, *label, *button, *scroll, *hbox;
     guint i;
 
-    vbox = gwy_vbox_new(0);   /* to prevent notebook expanding tables */
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);   /* to prevent notebook expanding tables */
     gtk_container_set_border_width(GTK_CONTAINER(vbox), 6);
 
     store = gwy_inventory_store_new(gwy_resource_class_get_inventory(g_type_class_peek(preset_resource_type)));
     filtermodel = gwy_create_inventory_model_without_default(store);
     gui->presetlist = gtk_tree_view_new_with_model(filtermodel);
-    gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(gui->presetlist), TRUE);
     g_object_unref(filtermodel);
     g_object_unref(store);
 
@@ -722,17 +709,16 @@ preset_tab_new(ModuleGUI *gui)
     gtk_container_add(GTK_CONTAINER(scroll), gui->presetlist);
     gtk_box_pack_start(GTK_BOX(vbox), scroll, TRUE, TRUE, 0);
 
-    hbox = gtk_hbutton_box_new();
-    gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_START);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 
     for (i = 0; i < G_N_ELEMENTS(buttons); i++) {
-        button = gui->preset_buttons[i] = gtk_button_new_with_mnemonic(gwy_sgettext(buttons[i].label));
-        gtk_container_add(GTK_CONTAINER(hbox), button);
+        button = gui->preset_buttons[i] = gtk_button_new_with_mnemonic(gwy_C(buttons[i].label));
+        gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
         g_signal_connect_swapped(button, "clicked", buttons[i].callback, gui);
     }
 
-    hbox = gwy_hbox_new(6);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 6);
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 4);
 
     label = gtk_label_new_with_mnemonic(_("Preset _name:"));
@@ -744,9 +730,8 @@ preset_tab_new(ModuleGUI *gui)
     gtk_label_set_mnemonic_widget(GTK_LABEL(label), gui->presetname);
 
     gui->preset_error = gtk_label_new(NULL);
-    gtk_misc_set_alignment(GTK_MISC(gui->preset_error), 0.0, 0.5);
+    gtk_label_set_xalign(GTK_LABEL(gui->preset_error), 0.0);
     gtk_box_pack_start(GTK_BOX(vbox), gui->preset_error, FALSE, FALSE, 4);
-    set_widget_as_warning_message(gui->preset_error);
 
     tselect = gtk_tree_view_get_selection(GTK_TREE_VIEW(gui->presetlist));
     gtk_tree_selection_set_mode(tselect, GTK_SELECTION_SINGLE);
@@ -776,13 +761,12 @@ param_changed(ModuleGUI *gui, gint id)
     gint xres = gwy_params_get_int(params, PARAM_XRES);
     gint yres = gwy_params_get_int(params, PARAM_YRES);
     gint power10;
-    guint i;
 
     if (id < 0 || id == PARAM_FORMAT || id == PARAM_BUILTIN) {
         /* This makes too many thing sensitive. We made them insensitive below. */
-        for (i = 0; i < G_N_ELEMENTS(text_params); i++)
+        for (guint i = 0; i < G_N_ELEMENTS(text_params); i++)
             gwy_param_table_set_sensitive(gui->table_format, text_params[i], format == RAW_TEXT);
-        for (i = 0; i < G_N_ELEMENTS(binary_params); i++)
+        for (guint i = 0; i < G_N_ELEMENTS(binary_params); i++)
             gwy_param_table_set_sensitive(gui->table_format, binary_params[i], format == RAW_BINARY);
 
         if (builtin) {
@@ -852,24 +836,31 @@ param_changed(ModuleGUI *gui, gint id)
     }
 
     if (id < 0 || id == PARAM_XYUNIT) {
-        GwySIUnit *unit = gwy_params_get_unit(params, PARAM_XYUNIT, &power10);
-        GwySIValueFormat *vf = gwy_si_unit_get_format_for_power10(unit, GWY_SI_UNIT_FORMAT_VFMARKUP, power10, NULL);
+        GwyUnit *unit = gwy_params_get_unit(params, PARAM_XYUNIT, &power10);
+        GwyValueFormat *vf = gwy_unit_get_format_for_power10(unit, GWY_UNIT_FORMAT_VFMARKUP, power10, NULL);
 
         gwy_param_table_set_unitstr(gui->table_dims, PARAM_XREAL, vf->units);
         gwy_param_table_set_unitstr(gui->table_dims, PARAM_YREAL, vf->units);
-        gwy_si_unit_value_format_free(vf);
+        gwy_value_format_free(vf);
     }
     if (id < 0 || id == PARAM_ZUNIT) {
-        GwySIUnit *unit = gwy_params_get_unit(params, PARAM_ZUNIT, &power10);
-        GwySIValueFormat *vf = gwy_si_unit_get_format_for_power10(unit, GWY_SI_UNIT_FORMAT_VFMARKUP, power10, NULL);
+        GwyUnit *unit = gwy_params_get_unit(params, PARAM_ZUNIT, &power10);
+        GwyValueFormat *vf = gwy_unit_get_format_for_power10(unit, GWY_UNIT_FORMAT_VFMARKUP, power10, NULL);
 
         gwy_param_table_set_unitstr(gui->table_dims, PARAM_ZSCALE, vf->units);
-        gwy_si_unit_value_format_free(vf);
+        gwy_value_format_free(vf);
     }
 
-    if (id < 0 || id == PARAM_FORMAT || id == PARAM_XRES || id == PARAM_YRES
-        || id == PARAM_SKIP_FIELDS || id == PARAM_LINE_OFFSET || id == PARAM_DELIMITER || id == PARAM_DELIMITER_TYPE
-        || id == PARAM_BUILTIN || id == PARAM_SIZE || id == PARAM_SKIP || id == PARAM_ROWSKIP || id == PARAM_OFFSET)
+    gboolean clear_error = (id < 0);
+    for (guint i = 0; !clear_error && i < G_N_ELEMENTS(text_params); i++) {
+        if (id == text_params[i])
+            clear_error = TRUE;
+    }
+    for (guint i = 0; !clear_error && i < G_N_ELEMENTS(binary_params); i++) {
+        if (id == binary_params[i])
+            clear_error = TRUE;
+    }
+    if (clear_error)
         gtk_label_set_text(GTK_LABEL(gui->error), "");
 }
 
@@ -878,28 +869,28 @@ preview(gpointer user_data)
 {
     ModuleGUI *gui = (ModuleGUI*)user_data;
     ModuleArgs *args = gui->args;
-    GwyDataField *field;
+    GwyField *field;
     GError *error = NULL;
 
-    if ((field = read_data_field(args, &error))) {
+    if ((field = read_field(args, &error))) {
         /* Fix any NaNs in the data for preview and remember the mask for later. */
         g_clear_object(&args->mask);
         args->mask = gwy_app_channel_mask_of_nans(field, TRUE);
-        gwy_data_field_assign(args->field, field);
+        gwy_field_assign(args->field, field);
         g_object_unref(field);
         gtk_label_set_text(GTK_LABEL(gui->error), "");
         gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
         gtk_dialog_set_response_sensitive(GTK_DIALOG(gui->dialog), GTK_RESPONSE_OK, TRUE);
     }
     else {
-        gwy_data_field_resample(args->field, PREVIEW_SMALL_SIZE, PREVIEW_SMALL_SIZE, GWY_INTERPOLATION_NONE);
-        gwy_data_field_clear(args->field);
+        gwy_field_resize(args->field, PREVIEW_SMALL_SIZE, PREVIEW_SMALL_SIZE);
+        gwy_field_clear(args->field);
         gtk_label_set_markup(GTK_LABEL(gui->error), error->message);
         g_clear_error(&error);
         gwy_dialog_invalidate(GWY_DIALOG(gui->dialog));
         gtk_dialog_set_response_sensitive(GTK_DIALOG(gui->dialog), GTK_RESPONSE_OK, FALSE);
     }
-    gwy_data_field_data_changed(args->field);
+    gwy_field_data_changed(args->field);
     gwy_set_data_preview_size(GWY_DATA_VIEW(gui->dataview), PREVIEW_SMALL_SIZE);
 }
 
@@ -1056,11 +1047,12 @@ preset_validate_name(ModuleGUI *gui,
                      const gchar *name,
                      gboolean show_warning)
 {
-    gboolean ok;
-
-    ok = (name && *name && !strchr(name, '/') && !strchr(name, '\\'));
-    if (show_warning)
-        gtk_label_set_text(GTK_LABEL(gui->preset_error), ok ? "" : _("Invalid preset name."));
+    gboolean ok = (name && *name && !strchr(name, '/') && !strchr(name, '\\'));
+    if (show_warning) {
+        gwy_set_message_label(GTK_LABEL(gui->preset_error),
+                              ok ? NULL : _("Invalid preset name."),
+                              GTK_MESSAGE_WARNING, FALSE);
+    }
 
     return ok;
 }
@@ -1074,17 +1066,18 @@ missingval_is_number(const gchar *missingval)
     v = g_strtod(missingval, &end);
     if (end == missingval)
         return FALSE;
-    /* If people do not check the missing value checkbox at all, NaN literals generally work fine. If they do, only
-     * consider actual numbers to be numbers. If they say ‘NAN’, expect literal strings ‘NAN’. It behaves weird when
-     * we return TRUE for NaNs here. */
+
+    /* If people do not check the missing value checkbox at all, NaN literals generally work fine and we catch them.
+     * If they do, only consider actual numbers to be numbers. If they say ‘NAN’, expect literal strings ‘NAN’. It
+     * behaves weird when we return TRUE for NaNs here. */
     if (gwy_isinf(v) || gwy_isnan(v))
         return FALSE;
 
     return TRUE;
 }
 
-static GwyDataField*
-read_data_field(ModuleArgs *args, GError **error)
+static GwyField*
+read_field(ModuleArgs *args, GError **error)
 {
     RawFileFile *file = &args->file;
     GwyParams *params = args->params;
@@ -1103,9 +1096,9 @@ read_data_field(ModuleArgs *args, GError **error)
     gulong lineoffset = gwy_params_get_int(params, PARAM_LINE_OFFSET);
     gdouble xreal = gwy_params_get_double(params, PARAM_XREAL);
     gdouble yreal = gwy_params_get_double(params, PARAM_YREAL);
-    GwyDataField *field;
+    GwyField *field;
     GError *err = NULL;
-    GwySIUnit *siunit;
+    GwyUnit *siunit;
     gulong reqsize;
     gint power10;
     guint i, n;
@@ -1139,21 +1132,21 @@ read_data_field(ModuleArgs *args, GError **error)
         return NULL;
     }
 
-    field = gwy_data_field_new(xres, yres, 1.0, 1.0, TRUE);
+    field = gwy_field_new(xres, yres, 1.0, 1.0, TRUE);
 
     siunit = gwy_params_get_unit(params, PARAM_XYUNIT, &power10);
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(field), siunit);
-    gwy_data_field_set_xreal(field, pow10(power10)*xreal);
-    gwy_data_field_set_yreal(field, pow10(power10)*yreal);
+    gwy_unit_assign(gwy_field_get_unit_xy(field), siunit);
+    gwy_field_set_xreal(field, gwy_exp10(power10)*xreal);
+    gwy_field_set_yreal(field, gwy_exp10(power10)*yreal);
     gwy_debug("dims %g x %g [10^%d because of %s]",
               xreal, yreal, power10, gwy_params_get_string(params, PARAM_XYUNIT));
 
     siunit = gwy_params_get_unit(params, PARAM_ZUNIT, &power10);
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_z(field), siunit);
+    gwy_unit_assign(gwy_field_get_unit_z(field), siunit);
     gwy_debug("zscale %g [10^%d because of %s]",
               gwy_params_get_double(params, PARAM_ZSCALE), power10, gwy_params_get_string(params, PARAM_ZUNIT));
 
-    d = gwy_data_field_get_data(field);
+    d = gwy_field_get_data(field);
     if (format == RAW_BINARY) {
         if (builtin)
             rawfile_read_builtin(args, d);
@@ -1167,7 +1160,7 @@ read_data_field(ModuleArgs *args, GError **error)
                           "The contents of `%s' does not match format: %s."),
                         file->filename, err->message);
             g_clear_error(&err);
-            GWY_OBJECT_UNREF(field);
+            g_clear_object(&field);
         }
     }
 
@@ -1184,7 +1177,7 @@ read_data_field(ModuleArgs *args, GError **error)
         }
     }
     if (field)
-        gwy_data_field_multiply(field, pow10(power10)*gwy_params_get_double(params, PARAM_ZSCALE));
+        gwy_field_multiply(field, gwy_exp10(power10)*gwy_params_get_double(params, PARAM_ZSCALE));
 
     return field;
 }
@@ -1446,7 +1439,7 @@ rawfile_read_builtin(ModuleArgs *args, gdouble *data)
     skip = skip/8;
     rowskip = rowskip/8;
 
-    gwy_clear(&good_alignment, 1);
+    gwy_clear1(good_alignment);
     b = (guchar*)&good_alignment;
 
     for (i = yres; i; i--) {
@@ -1497,16 +1490,13 @@ skip_fields(const guchar *buffer, gint nfields,
             gint delimlen, gint cdelim, const gchar *delim,
             gint rowno, GError **error)
 {
-    const gchar *end;
-    gint i, j;
-
     if (!nfields)
         return buffer;
 
     if (!delimlen) {
         buffer += strspn(buffer, " \t\n\r");
-        for (i = 0; i < nfields; i++) {
-            j = strcspn(buffer, " \t\n\r");
+        for (gint i = 0; i < nfields; i++) {
+            gint j = strcspn(buffer, " \t\n\r");
             buffer += j;
             j = strspn(buffer, " \t\n\r");
             if (!j) {
@@ -1519,8 +1509,8 @@ skip_fields(const guchar *buffer, gint nfields,
         }
     }
     else if (delimlen == 1) {
-        for (i = 0; i < nfields; i++) {
-            end = strchr(buffer, cdelim);
+        for (gint i = 0; i < nfields; i++) {
+            const gchar *end = strchr(buffer, cdelim);
             if (!end) {
                 g_set_error(error, error_domain, RAWFILE_ERROR_TEXT_PARSING,
                             _("Expected `%c' to skip more fields in row %u, got `%.16s'"),
@@ -1531,8 +1521,8 @@ skip_fields(const guchar *buffer, gint nfields,
         }
     }
     else {
-        for (i = 0; i < nfields; i++) {
-            end = strstr(buffer, delim);
+        for (gint i = 0; i < nfields; i++) {
+            const gchar *end = strstr(buffer, delim);
             if (!end) {
                 g_set_error(error, error_domain, RAWFILE_ERROR_TEXT_PARSING,
                             _("Expected `%s' to skip more fields in row %u, got `%.16s'"),
@@ -1553,17 +1543,16 @@ read_fields(const guchar *buffer, gint nfields,
             gint rowno, GError **error)
 {
     gboolean single_value = !nfields;
-    const guchar *end;
-    gdouble x;
-    gint i, j;
+    gint j;
 
     nfields = MAX(nfields, 1);
     if (!delimlen) {
-        for (i = 0; i < nfields; i++) {
+        for (gint i = 0; i < nfields; i++) {
             /* FIXME: This is not good enough when extracting a single value. The strtod function can also accept
              * \n and \r as whitespace, messing up the ‘one per line’ paradigm. If the data sort of match what the
              * user says, it works fine. If they do not the results are mysterious. */
-            x = strtod_func(buffer, missingval, (char**)&end);
+            const guchar *end;
+            gdouble x = strtod_func(buffer, missingval, (char**)&end);
             if (end == buffer) {
                 g_set_error(error, error_domain, RAWFILE_ERROR_TEXT_PARSING,
                             _("Garbage `%.16s' in row %u, column %u"),
@@ -1575,8 +1564,9 @@ read_fields(const guchar *buffer, gint nfields,
         }
     }
     else if (delimlen == 1) {
-        for (i = 0; i < nfields; i++) {
-            x = strtod_func(buffer, missingval, (char**)&end);
+        for (gint i = 0; i < nfields; i++) {
+            const guchar *end;
+            gdouble x = strtod_func(buffer, missingval, (char**)&end);
             if (end == buffer) {
                 g_set_error(error, error_domain, RAWFILE_ERROR_TEXT_PARSING,
                             _("Garbage `%.16s' in row %u, column %u"),
@@ -1600,8 +1590,9 @@ read_fields(const guchar *buffer, gint nfields,
         }
     }
     else {
-        for (i = 0; i < nfields; i++) {
-            x = strtod_func(buffer, missingval, (char**)&end);
+        for (gint i = 0; i < nfields; i++) {
+            const guchar *end;
+            gdouble x = strtod_func(buffer, missingval, (char**)&end);
             if (end == buffer) {
                 g_set_error(error, error_domain, RAWFILE_ERROR_TEXT_PARSING,
                             _("Garbage `%.16s' in row %u, column %u"),
@@ -1676,6 +1667,7 @@ rawfile_read_ascii(ModuleArgs *args, gdouble *data, GError **error)
         cdelim = delim[0];
     }
 
+
     if (oneperline) {
         /* Extract one value from each file line. */
         for (n = 0; n < xres*yres; n++) {
@@ -1698,7 +1690,6 @@ rawfile_read_ascii(ModuleArgs *args, gdouble *data, GError **error)
             /* Skip fields. */
             if (!(buffer = skip_fields(buffer, skipfields, delimlen, cdelim, delim, n, error)))
                 return FALSE;
-
             /* Read data. */
             if (!(buffer = read_fields(buffer, xres, delimlen, cdelim, delim, strtod_func, missingval, data, n, error)))
                 return FALSE;
@@ -1855,16 +1846,16 @@ static void
 convert_legacy_exponent(GwyParams *params, gint id_exp, gint id_unit)
 {
     gint power10_exp, power10_unit;
-    GwySIValueFormat *vf;
-    GwySIUnit *unit;
+    GwyValueFormat *vf;
+    GwyUnit *unit;
 
     if (!(power10_exp = gwy_params_get_int(params, id_exp)))
         return;
 
     unit = gwy_params_get_unit(params, id_unit, &power10_unit);
-    vf = gwy_si_unit_get_format_for_power10(unit, GWY_SI_UNIT_FORMAT_PLAIN, power10_unit + power10_exp, NULL);
+    vf = gwy_unit_get_format_for_power10(unit, GWY_UNIT_FORMAT_PLAIN, power10_unit + power10_exp, NULL);
     gwy_params_set_unit(params, id_unit, vf->units);
-    gwy_si_unit_value_format_free(vf);
+    gwy_value_format_free(vf);
     gwy_params_set_int(params, id_exp, 0);
 }
 
