@@ -1,6 +1,6 @@
 /*
- *  $Id: rawgraph.c 26334 2024-05-10 08:29:25Z yeti-dn $
- *  Copyright (C) 2009-2022 David Necas (Yeti).
+ *  $Id: rawgraph.c 28772 2025-11-03 18:02:12Z yeti-dn $
+ *  Copyright (C) 2009-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -29,16 +29,10 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <libgwydgets/gwygraphmodel.h>
-#include <libgwydgets/gwygraphcurvemodel.h>
-#include <libgwydgets/gwydgettypes.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -67,10 +61,10 @@ typedef struct {
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static gint             rawgraph_detect     (const GwyFileDetectInfo *fileinfo,
+static gint             detect_file         (const GwyFileDetectInfo *fileinfo,
                                              gboolean only_name);
-static GwyContainer*    rawgraph_load       (const gchar *filename,
-                                             GwyRunType mode,
+static GwyFile*         load_file           (const gchar *filename,
+                                             GwyRunModeFlags mode,
                                              GError **error);
 static GwyDialogOutcome run_gui             (ModuleArgs *args);
 static void             param_changed       (ModuleGUI *gui,
@@ -86,7 +80,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports simple text files as graph curves."),
     "Yeti <yeti@gwyddion.net>",
-    "0.9",
+    "0.10",
     "David Nečas (Yeti)",
     "2009",
 };
@@ -98,10 +92,7 @@ module_register(void)
 {
     gwy_file_func_register("rawgraph",
                            N_("ASCII graph curve files"),
-                           (GwyFileDetectFunc)&rawgraph_detect,
-                           (GwyFileLoadFunc)&rawgraph_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
     /* We provide a detection function, but the loading method tries a bit harder, so let the user choose explicitly. */
     gwy_file_func_set_is_detectable("rawgraph", FALSE);
 
@@ -135,8 +126,7 @@ define_module_params(void)
 }
 
 static gint
-rawgraph_detect(const GwyFileDetectInfo *fileinfo,
-                gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     const gchar *s;
     gchar *end;
@@ -174,13 +164,13 @@ next_line:
     return 50;
 }
 
-static GwyContainer*
-rawgraph_load(const gchar *filename,
-              GwyRunType mode,
-              GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          GwyRunModeFlags mode,
+          GError **error)
 {
     GError *err = NULL;
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     GwyGraphCurveModel *gcmodel;
     GwyDialogOutcome outcome;
     ModuleArgs args;
@@ -197,7 +187,7 @@ rawgraph_load(const gchar *filename,
         return NULL;
     }
 
-    gwy_clear(&args, 1);
+    gwy_clear1(args);
     if (!rawgraph_parse(buffer, &args, error))
         goto fail;
 
@@ -216,16 +206,17 @@ rawgraph_load(const gchar *filename,
     }
 
     execute(&args);
-    container = gwy_container_new();
-    gwy_container_set_object(container, gwy_app_get_graph_key_for_id(1), args.gmodel);
+    file = gwy_file_new_in_construction();
+    gwy_file_set_graph(file, 0, args.gmodel);
+    gwy_log_add_import(file, GWY_FILE_GRAPH, 0, NULL, filename);
 
 fail:
     g_free(buffer);
     g_free(args.data);
-    GWY_OBJECT_UNREF(args.params);
-    GWY_OBJECT_UNREF(args.gmodel);
+    g_clear_object(&args.params);
+    g_clear_object(&args.gmodel);
 
-    return container;
+    return file;
 }
 
 static GwyDialogOutcome
@@ -243,7 +234,7 @@ run_gui(ModuleArgs *args)
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    hbox = gwy_hbox_new(20);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 20);
     gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
     gwy_dialog_add_content(dialog, hbox, TRUE, TRUE, 0);
 
@@ -301,18 +292,18 @@ execute(ModuleArgs *args)
 {
     GwyGraphModel *gmodel = args->gmodel;
     GwyGraphCurveModel *gcmodel = gwy_graph_model_get_curve(gmodel, 0);
-    GwySIUnit *unitx, *unity;
+    GwyUnit *unitx, *unity;
     gint i, ndata, power10x, power10y;
     gdouble qx, qy;
 
-    unitx = gwy_si_unit_new_parse(gwy_params_get_string(args->params, PARAM_X_UNITS), &power10x);
-    unity = gwy_si_unit_new_parse(gwy_params_get_string(args->params, PARAM_Y_UNITS), &power10y);
-    g_object_set(gmodel, "si-unit-x", unitx, "si-unit-y", unity, NULL);
+    unitx = gwy_unit_new_parse(gwy_params_get_string(args->params, PARAM_X_UNITS), &power10x);
+    unity = gwy_unit_new_parse(gwy_params_get_string(args->params, PARAM_Y_UNITS), &power10y);
+    g_object_set(gmodel, "unit-x", unitx, "unit-y", unity, NULL);
     g_object_unref(unitx);
     g_object_unref(unity);
 
-    qx = pow10(power10x);
-    qy = pow10(power10y);
+    qx = gwy_exp10(power10x);
+    qy = gwy_exp10(power10y);
 
     ndata = args->nrows;
     for (i = 0; i < ndata; i++) {
