@@ -1,6 +1,6 @@
 /*
- *  $Id: renishaw.c 26510 2024-08-15 17:12:23Z yeti-dn $
- *  Copyright (C) 2014 Daniil Bratashov (dn2010), David Necas (Yeti)..
+ *  $Id: renishaw.c 28803 2025-11-05 11:58:19Z yeti-dn $
+ *  Copyright (C) 2014-2025 Daniil Bratashov (dn2010), David Necas (Yeti)..
  *  Data structures and constants are copyright (c) 2011 Renishaw plc.
  *
  *  E-mail: dn2010@gmail.com.
@@ -47,19 +47,10 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/datafield.h>
-#include <libprocess/spectra.h>
-#include <libprocess/brick.h>
-#include <libgwydgets/gwygraphmodel.h>
-#include <libgwydgets/gwygraphbasics.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
 #include "err.h"
@@ -265,7 +256,7 @@ typedef enum {
     WDF_PSET_KEYS_VERSION    = 432
 } WdfPsetPredefinedKeys;
 
-static const GwyEnum wdf_predefined_keys[] = {
+static const GwyEnum predefined_keys[] = {
     { "Properties",          WDF_PSET_KEYS_PROPERTIES },
     { "Label",               WDF_PSET_KEYS_LABEL      },
     { "MapType",             WDF_PSET_KEYS_MAPTYPE    },
@@ -367,32 +358,32 @@ typedef struct {
     WdfMapData   **maps;
 } WdfFile;
 
-static gboolean      module_register       (void);
-static gint          wdf_detect            (const GwyFileDetectInfo *fileinfo,
-                                            gboolean only_name);
-static GwyContainer* wdf_load              (const gchar *filename,
-                                            GwyRunType mode,
-                                            GError **error);
-static gsize         wdf_read_header       (const guchar *buffer,
-                                            gsize size,
-                                            WdfHeader *header,
-                                            GError **error);
-static gsize         wdf_read_block_header (const guchar *buffer,
-                                            gsize size,
-                                            WdfBlock *header,
-                                            GError **error);
-static void          wdf_read_maparea_block(const guchar *buffer,
-                                            WdfMapArea *maparea);
-static GwyContainer* wdf_read_pset         (const guchar *buffer,
-                                            gsize size);
-static gchar*        wdf_read_datetime     (gint64 ticks);
+static gboolean      module_register   (void);
+static gint          detect_file       (const GwyFileDetectInfo *fileinfo,
+                                        gboolean only_name);
+static GwyFile*      load_file         (const gchar *filename,
+                                        GwyRunModeFlags mode,
+                                        GError **error);
+static gsize         read_header       (const guchar *buffer,
+                                        gsize size,
+                                        WdfHeader *header,
+                                        GError **error);
+static gsize         read_block_header (const guchar *buffer,
+                                        gsize size,
+                                        WdfBlock *header,
+                                        GError **error);
+static void          read_maparea_block(const guchar *buffer,
+                                        WdfMapArea *maparea);
+static GwyContainer* read_pset         (const guchar *buffer,
+                                        gsize size);
+static gchar*        read_datetime     (gint64 ticks);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Renishaw WiRE data files (WDF)."),
     "Daniil Bratashov <dn2010@gmail.com>",
-    "0.12",
+    "0.13",
     "Daniil Bratashov (dn2010), David Necas (Yeti), Renishaw plc.",
     "2014",
 };
@@ -404,17 +395,13 @@ module_register(void)
 {
     gwy_file_func_register("renishaw",
                            N_("Renishaw WiRE data files (.wdf)"),
-                           (GwyFileDetectFunc)&wdf_detect,
-                           (GwyFileLoadFunc)&wdf_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-wdf_detect(const GwyFileDetectInfo *fileinfo,
-               gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     gint score = 0;
 
@@ -429,12 +416,13 @@ wdf_detect(const GwyFileDetectInfo *fileinfo,
     return score;
 }
 
-static GwyContainer*
-wdf_load(const gchar *filename,
-             G_GNUC_UNUSED GwyRunType mode,
-             GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *container = NULL, *metadata = NULL, *pset_data;
+    GwyFile *file = NULL;
+    GwyContainer *metadata = NULL, *pset_data;
     gchar *buffer = NULL;
     gsize len, size = 0;
     GError *err = NULL;
@@ -442,13 +430,13 @@ wdf_load(const gchar *filename,
     WdfHeader fileheader;
     WdfBlock block;
     const guchar *p;
-    gchar *title = NULL, *key = NULL;
+    gchar *title = NULL;
     GwyBrick *brick;
-    GwyDataField *dfield;
-    GwyDataLine *cal;
+    GwyField *dfield;
+    GwyLine *cal;
     GwyGraphModel *gmodel;
     GwyGraphCurveModel *gcmodel;
-    GwySIUnit *siunitx = NULL, *siunity = NULL, *siunitz = NULL, *siunitw = NULL, *siunitsp = NULL, *siunitm = NULL;
+    GwyUnit *siunitx = NULL, *siunity = NULL, *siunitz = NULL, *siunitw = NULL, *siunitsp = NULL, *siunitm = NULL;
     gdouble *ydata, *xdata, *data, *zdata = NULL, *xdatasp, *ydatasp;
     gint i, j, k, l, lsize, z, m;
     gint xres, yres, zres, xstart, xend, xstep, ystart, yend, ystep;
@@ -463,13 +451,13 @@ wdf_load(const gchar *filename,
     GdkPixbuf *pixbuf = NULL;
     guchar *pixels, *pix_p;
     const gchar *unit;
-    const guchar *label;
-    const guchar *spectrum, *spectrumx, *specxunit, *speclabel;
+    const gchar *label;
+    const gchar *spectrum, *spectrumx, *specxunit, *speclabel;
     gchar **specvalues, **specp;
     gint numvalues, mapspectra = 0;
     gboolean hasspectrum = FALSE;
 
-    gwy_clear(&filedata, 1);
+    gwy_clear1(filedata);
 
     if (!g_file_get_contents(filename, &buffer, &size, &err)) {
         err_GET_FILE_CONTENTS(error, &err);
@@ -477,7 +465,7 @@ wdf_load(const gchar *filename,
     }
 
     p = buffer;
-    if ((len = wdf_read_header(p, size, &fileheader, error)) != WDF_HEADER_SIZE)
+    if ((len = read_header(p, size, &fileheader, error)) != WDF_HEADER_SIZE)
         goto fail;
     p += WDF_HEADER_SIZE;
     size -= WDF_HEADER_SIZE;
@@ -488,11 +476,11 @@ wdf_load(const gchar *filename,
               fileheader.nspectra);
 
     filedata.xlistunits = WDF_DATAUNITS_ARBITRARY;
-    metadata = gwy_container_new();
-    container = gwy_container_new();
+    metadata = gwy_container_new_in_construction();
+    file = gwy_file_new_in_construction();
 
     while (size > 0) {
-        if ((len = wdf_read_block_header(p, size, &block, error)) == 0)
+        if ((len = read_block_header(p, size, &block, error)) == 0)
             goto fail;
         if (block.id == WDF_BLOCKID_DATA) {
             filedata.datasize = block.size - WDF_BLOCK_HEADER_SIZE;
@@ -546,8 +534,7 @@ wdf_load(const gchar *filename,
                 p += fileheader.nspectra * sizeof(gdouble);
             }
             p -= WDF_BLOCK_HEADER_SIZE + sizeof(guint32)
-               + norigins * (2 * sizeof(guint32) + 16
-                               + fileheader.nspectra * sizeof(gdouble));
+                + norigins * (2 * sizeof(guint32) + 16 + fileheader.nspectra * sizeof(gdouble));
         }
         else if (block.id == WDF_BLOCKID_MAPAREA) {
             if (block.size != WDF_MAP_AREA_SIZE) {
@@ -555,7 +542,7 @@ wdf_load(const gchar *filename,
                 goto fail;
             }
             filedata.maparea = g_new(WdfMapArea, 1);
-            wdf_read_maparea_block(block.data, filedata.maparea);
+            read_maparea_block(block.data, filedata.maparea);
         }
         else if (block.id == WDF_BLOCKID_WHITELIGHT) {
             loader = gdk_pixbuf_loader_new();
@@ -563,8 +550,7 @@ wdf_load(const gchar *filename,
              * happen entirely inside libjpeg. We just inherit it and then anything we try to do with the returned
              * pixbuf is uninitialised memory use. */
             if (!gdk_pixbuf_loader_write(loader, block.data, block.size, &err)) {
-                g_set_error(error, GWY_MODULE_FILE_ERROR,
-                            GWY_MODULE_FILE_ERROR_DATA,
+                g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
                             _("Pixbuf loader refused data: %s."),
                             err->message);
                 g_clear_error(&err);
@@ -573,8 +559,7 @@ wdf_load(const gchar *filename,
             }
             gwy_debug("Closing the loader.");
             if (!gdk_pixbuf_loader_close(loader, &err)) {
-                g_set_error(error, GWY_MODULE_FILE_ERROR,
-                            GWY_MODULE_FILE_ERROR_DATA,
+                g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
                             _("Pixbuf loader refused data: %s."),
                             err->message);
                 g_clear_error(&err);
@@ -591,9 +576,9 @@ wdf_load(const gchar *filename,
             filedata.whitelight = pixbuf;
         }
         else if ((block.id == WDF_BLOCKID_CALIBRATION)
-              || (block.id == WDF_BLOCKID_INSTRUMENT)
-              || (block.id == WDF_BLOCKID_MEASUREMENT)
-              || (block.id == WDF_BLOCKID_WIREDATA)) {
+                 || (block.id == WDF_BLOCKID_INSTRUMENT)
+                 || (block.id == WDF_BLOCKID_MEASUREMENT)
+                 || (block.id == WDF_BLOCKID_WIREDATA)) {
             p += WDF_BLOCK_HEADER_SIZE;
             magic = gwy_get_guint32_le(&p);
             pset_size = gwy_get_guint32_le(&p);
@@ -602,8 +587,10 @@ wdf_load(const gchar *filename,
                 gwy_debug("bad magic in pset");
                 goto fail;
             }
-            pset_data = wdf_read_pset(p, pset_size);
+            pset_data = read_pset(p, pset_size);
             if (pset_data) {
+                gchar *key;
+
                 if (block.id == WDF_BLOCKID_CALIBRATION)
                     key = g_strdup_printf("/calibration");
                 else if (block.id == WDF_BLOCKID_INSTRUMENT)
@@ -614,7 +601,7 @@ wdf_load(const gchar *filename,
                     key = g_strdup_printf("/data");
                 else
                     key = g_strdup_printf("/");
-                gwy_container_transfer(pset_data, metadata, "/", key, TRUE);
+                gwy_container_transfer(pset_data, metadata, "/", key, FALSE, TRUE);
                 g_free(key);
                 g_object_unref(pset_data);
             }
@@ -631,7 +618,7 @@ wdf_load(const gchar *filename,
             }
             filedata.maps = g_realloc(filedata.maps, (filedata.nummaps + 1) * sizeof(gpointer));
             filedata.maps[filedata.nummaps] = g_new0(WdfMapData, 1);
-            filedata.maps[filedata.nummaps]->pset = wdf_read_pset(p, pset_size);
+            filedata.maps[filedata.nummaps]->pset = read_pset(p, pset_size);
             p += pset_size;
             filedata.maps[filedata.nummaps]->numpoints = gwy_get_guint64_le(&p);
             filedata.maps[filedata.nummaps]->data = (gfloat *)p;
@@ -645,8 +632,8 @@ wdf_load(const gchar *filename,
     xscale = 1.0;
     gwy_debug("x units = %d", xunits);
     unit = gwy_enum_to_string(xunits, wdf_units, 26);
-    siunitx = gwy_si_unit_new_parse(unit, &power10x);
-    xscale = pow10(power10x);
+    siunitx = gwy_unit_new_parse(unit, &power10x);
+    xscale = gwy_exp10(power10x);
     if (xscale == 0.0) {
         xscale = 1.0;
     }
@@ -654,8 +641,8 @@ wdf_load(const gchar *filename,
     yscale = 1.0;
     gwy_debug("y units = %d", yunits);
     unit = gwy_enum_to_string(yunits, wdf_units, 26);
-    siunity = gwy_si_unit_new_parse(unit, &power10y);
-    yscale = pow10(power10y);
+    siunity = gwy_unit_new_parse(unit, &power10y);
+    yscale = gwy_exp10(power10y);
     if (yscale == 0.0) {
         yscale = 1.0;
     }
@@ -663,8 +650,8 @@ wdf_load(const gchar *filename,
     wscale = 1.0;
     gwy_debug("w units = %d", fileheader.units);
     unit = gwy_enum_to_string(fileheader.units, wdf_units, 26);
-    siunitw = gwy_si_unit_new_parse(unit, &power10w);
-    wscale = pow10(power10w);
+    siunitw = gwy_unit_new_parse(unit, &power10w);
+    wscale = gwy_exp10(power10w);
     if (wscale == 0.0) {
         wscale = 1.0;
     }
@@ -672,8 +659,8 @@ wdf_load(const gchar *filename,
     zscale = 1.0;
     gwy_debug("z units = %d", filedata.xlistunits);
     unit = gwy_enum_to_string(filedata.xlistunits, wdf_units, 26);
-    siunitz = gwy_si_unit_new_parse(unit, &power10z);
-    zscale = pow10(power10z);
+    siunitz = gwy_unit_new_parse(unit, &power10z);
+    zscale = gwy_exp10(power10z);
     if (zscale == 0.0) {
         zscale = 1.0;
     }
@@ -697,8 +684,8 @@ wdf_load(const gchar *filename,
         title = g_strdup(fileheader.title);
         gmodel = g_object_new(GWY_TYPE_GRAPH_MODEL,
                               "title", title,
-                              "si-unit-x", siunitz,
-                              "si-unit-y", siunitw,
+                              "unit-x", siunitz,
+                              "unit-y", siunitw,
                               NULL);
         gcmodel = g_object_new(GWY_TYPE_GRAPH_CURVE_MODEL,
                                "description", title,
@@ -713,9 +700,7 @@ wdf_load(const gchar *filename,
         gwy_graph_model_add_curve(gmodel, gcmodel);
         g_object_unref(gcmodel);
         g_free(title);
-        gwy_container_set_object_by_name(container, "/0/graph/graph/1",
-                                         gmodel);
-        g_object_unref(gmodel);
+        gwy_file_pass_graph(file, 1, gmodel);
     }
     else if (!filedata.maparea) { /* zscan */
         zres = fileheader.npoints;
@@ -727,8 +712,8 @@ wdf_load(const gchar *filename,
         title = g_strdup(fileheader.title);
         gmodel = g_object_new(GWY_TYPE_GRAPH_MODEL,
                               "title", title,
-                              "si-unit-x", siunitz,
-                              "si-unit-y", siunitw,
+                              "unit-x", siunitz,
+                              "unit-y", siunitw,
                               NULL);
         g_free(title);
         xdata = g_new(gdouble, zres);
@@ -767,8 +752,7 @@ wdf_load(const gchar *filename,
         g_object_unref(siunitz);
 
         g_free(xdata);
-        gwy_container_set_object_by_name(container, "/0/graph/graph/1", gmodel);
-        g_object_unref(gmodel);
+        gwy_file_pass_graph(file, 1, gmodel);
 
         for (m = 0; m < filedata.nummaps; m++) { /* z maps */
             zres = filedata.maps[m]->numpoints;
@@ -780,17 +764,17 @@ wdf_load(const gchar *filename,
                 title = g_strdup_printf("Map %d", m + 1);
 
             siunitm = NULL;
-            siunitm = gwy_si_unit_new_parse(unit, &power10m);
-            mscale = pow10(power10m);
+            siunitm = gwy_unit_new_parse(unit, &power10m);
+            mscale = gwy_exp10(power10m);
             if (!siunitm) {
-                siunitm = gwy_si_unit_new_parse("", &power10m);
+                siunitm = gwy_unit_new_parse("", &power10m);
                 mscale = 1.0;
             }
 
             gmodel = g_object_new(GWY_TYPE_GRAPH_MODEL,
                                   "title", title,
-                                  "si-unit-x", siunitm,
-                                  "si-unit-y", siunitw,
+                                  "unit-x", siunitm,
+                                  "unit-y", siunitw,
                                   NULL);
             g_object_unref(siunitm);
             xdata = g_new(gdouble, zres);
@@ -819,9 +803,7 @@ wdf_load(const gchar *filename,
             gwy_graph_curve_model_enforce_order(gcmodel);
             gwy_graph_model_add_curve(gmodel, gcmodel);
             g_object_unref(gcmodel);
-            key = g_strdup_printf("/0/graph/graph/%d", m + 2);
-            gwy_container_set_object_by_name(container, key, gmodel);
-            g_object_unref(gmodel);
+            gwy_file_pass_graph(file, m+2, gmodel);
         }
     }
     else { /* some kind of scan */
@@ -846,8 +828,8 @@ wdf_load(const gchar *filename,
             yscale = 1.0;
             gwy_debug("y units = %d", yunits);
             unit = gwy_enum_to_string(zunits, wdf_units, 26);
-            siunity = gwy_si_unit_new_parse(unit, &power10y);
-            yscale = pow10(power10y);
+            siunity = gwy_unit_new_parse(unit, &power10y);
+            yscale = gwy_exp10(power10y);
             if (yscale == 0.0) {
                 yscale = 1.0;
             }
@@ -891,10 +873,10 @@ wdf_load(const gchar *filename,
         p = (guchar *)filedata.data;
         for (z = 0; z < filedata.maparea->length[2]; z++) {
             brick = gwy_brick_new(xres, yres, zres, xreal, yreal, zres, TRUE);
-            gwy_brick_set_si_unit_x(brick, siunitx);
-            gwy_brick_set_si_unit_y(brick, siunity);
-            gwy_brick_set_si_unit_z(brick, siunitz);
-            gwy_brick_set_si_unit_w(brick, siunitw);
+            gwy_unit_assign(gwy_brick_get_unit_x(brick), siunitx);
+            gwy_unit_assign(gwy_brick_get_unit_y(brick), siunity);
+            gwy_unit_assign(gwy_brick_get_unit_z(brick), siunitz);
+            gwy_unit_assign(gwy_brick_get_unit_w(brick), siunitw);
 
             /* read data */
             /* XXX: Perhaps try simple gwy_convert_raw_data() and then gwy_brick_transpose() to clean up this?
@@ -1005,11 +987,11 @@ wdf_load(const gchar *filename,
             }
 
             /* reading calibration */
-            cal = gwy_data_line_new(zres, zres, FALSE);
-            data = gwy_data_line_get_data(cal);
+            cal = gwy_line_new(zres, zres, FALSE);
+            data = gwy_line_get_data(cal);
             gwy_convert_raw_data(filedata.xlistdata, zres, 1, GWY_RAW_DATA_FLOAT, GWY_BYTE_ORDER_LITTLE_ENDIAN,
                                  data, zscale, 0.0);
-            gwy_data_line_set_si_unit_y(cal, siunitz);
+            gwy_unit_assign(gwy_line_get_unit_y(cal), siunitz);
 
             gwy_brick_set_zcalibration(brick, cal);
             g_object_unref(cal);
@@ -1019,7 +1001,7 @@ wdf_load(const gchar *filename,
             gwy_brick_set_yoffset(brick, filedata.maparea->location[1] * yscale);
 
             /* packing */
-            gwy_container_pass_object(container, gwy_app_get_brick_key_for_id(z), brick);
+            gwy_file_pass_volume(file, z, brick);
             if (filedata.maparea->length[2] == 1) {
                 title = g_strdup(fileheader.title);
             }
@@ -1034,11 +1016,10 @@ wdf_load(const gchar *filename,
                                         fileheader.title,
                                         z);
             }
-            gwy_container_set_string(container, gwy_app_get_brick_title_key_for_id(z), title);
-            gwy_container_pass_object(container, gwy_app_get_brick_meta_key_for_id(z),
-                                      gwy_container_duplicate(metadata));
+            gwy_file_pass_title(file, GWY_FILE_VOLUME, z, title);
+            gwy_file_pass_meta(file, GWY_FILE_VOLUME, z, gwy_container_copy(metadata));
 
-            gwy_file_volume_import_log_add(container, z, NULL, filename);
+            gwy_log_add_import(file, GWY_FILE_VOLUME, z, NULL, filename);
             if (filedata.maparea->flags & WDF_MAPAREA_XYLINE) {
                 /* FIXME: it is better to do it here and don't break
                  * all other logic and implement reading of XYLINE
@@ -1050,12 +1031,12 @@ wdf_load(const gchar *filename,
 
         /* Reading maps */
         for (m = 0; m < filedata.nummaps; m++) {
-            dfield = gwy_data_field_new(xres, yres, xreal, yreal, TRUE);
-            gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(dfield), siunitx);
-            gwy_data_field_set_xoffset(dfield, filedata.maparea->location[0] * xscale);
-            gwy_data_field_set_yoffset(dfield, filedata.maparea->location[1] * yscale);
+            dfield = gwy_field_new(xres, yres, xreal, yreal, TRUE);
+            gwy_unit_assign(gwy_field_get_unit_xy(dfield), siunitx);
+            gwy_field_set_xoffset(dfield, filedata.maparea->location[0] * xscale);
+            gwy_field_set_yoffset(dfield, filedata.maparea->location[1] * yscale);
 
-            data = gwy_data_field_get_data(dfield);
+            data = gwy_field_get_data(dfield);
             p = (gchar *)filedata.maps[m]->data;
 
             if (filedata.maparea->flags & WDF_MAPAREA_COLUMNMAJOR) {
@@ -1143,15 +1124,14 @@ wdf_load(const gchar *filename,
                     for (j = ystart; j != yend; j += ystep)
                         for (i = xstart; i != xend; i += xstep)
                             *(data + i + j * xres) = gwy_get_gfloat_le(&p);
-
                  }
             }
 
-            gwy_container_pass_object(container, gwy_app_get_data_key_for_id(m + 1), dfield);
+            gwy_file_pass_image(file, m + 1, dfield);
             pset_data = filedata.maps[m]->pset;
 
             if (gwy_container_gis_string_by_name(pset_data, "/Label", &label))
-                gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(m + 1), label);
+                gwy_file_set_title(file, GWY_FILE_IMAGE, m + 1, label, FALSE);
 
             hasspectrum = FALSE;
             ydata = g_new(gdouble, zres);
@@ -1174,11 +1154,11 @@ wdf_load(const gchar *filename,
             if (gwy_container_gis_string_by_name(pset_data, "/overlayXListUnits", &specxunit)) {
                 xunits = atoi(specxunit);
                 unit = gwy_enum_to_string(xunits, wdf_units, 26);
-                siunitsp = gwy_si_unit_new_parse(unit, &power10x);
-                xscale = pow10(power10x);
+                siunitsp = gwy_unit_new_parse(unit, &power10x);
+                xscale = gwy_exp10(power10x);
             }
             if (!siunitsp) {
-                siunitsp = gwy_si_unit_new_parse("", &power10x);
+                siunitsp = gwy_unit_new_parse("", &power10x);
                 xscale = 1.0;
             }
 
@@ -1210,7 +1190,7 @@ wdf_load(const gchar *filename,
             if (hasspectrum) {
                 gmodel = g_object_new(GWY_TYPE_GRAPH_MODEL,
                                       "title", title,
-                                      "si-unit-x", siunitsp,
+                                      "unit-x", siunitsp,
                                       NULL);
                 gcmodel = g_object_new(GWY_TYPE_GRAPH_CURVE_MODEL,
                                        "description", title,
@@ -1223,16 +1203,15 @@ wdf_load(const gchar *filename,
                 gwy_graph_curve_model_enforce_order(gcmodel);
                 gwy_graph_model_add_curve(gmodel, gcmodel);
                 g_object_unref(gcmodel);
-                gwy_container_pass_object(container, gwy_app_get_graph_key_for_id(mapspectra + 1), gmodel);
+                gwy_file_pass_graph(file, mapspectra + 1, gmodel);
             }
             g_object_unref(siunitsp);
             g_free(title);
             g_free(xdata);
             g_free(ydata);
 
-            gwy_container_set_object(container, gwy_app_get_data_meta_key_for_id(m + 1), pset_data);
-            gwy_file_channel_import_log_add(container, m + 1, NULL, filename);
-
+            gwy_file_pass_meta(file, GWY_FILE_IMAGE, m + 1, gwy_container_copy(pset_data));
+            gwy_log_add_import(file, GWY_FILE_IMAGE, m + 1, NULL, filename);
         }
         g_object_unref(siunitz);
     }
@@ -1246,8 +1225,8 @@ wdf_load(const gchar *filename,
         bpp = gdk_pixbuf_get_has_alpha(pixbuf) ? 4 : 3;
 
         // FIXME: Whitelight imported as greyscale
-        dfield = gwy_data_field_new(width, height, width, height, TRUE);
-        data = gwy_data_field_get_data(dfield);
+        dfield = gwy_field_new(width, height, width, height, TRUE);
+        data = gwy_field_get_data(dfield);
         for (i = 0; i < height; i++) {
             pix_p = pixels + i * rowstride;
             for (j = 0; j < width; j++) {
@@ -1255,47 +1234,40 @@ wdf_load(const gchar *filename,
                 guchar green = pix_p[bpp*j+1];
                 guchar blue = pix_p[bpp*j+2];
 
-                *(data + i * width + j) = (0.2126 * red
-                                         + 0.7152 * green
-                                         + 0.0722 * blue) / 255.0;
+                data[i * width + j] = (0.2126 * red + 0.7152 * green + 0.0722 * blue)/255.0;
             }
         }
-        key = g_strdup_printf("/0/data");
-        gwy_container_set_object_by_name(container, key, dfield);
-        g_free(key);
-        g_object_unref(dfield);
+        gwy_file_pass_image(file, 0, dfield);
         title = g_strdup_printf("%s (WhiteLight)", fileheader.title);
-        key = g_strdup_printf("/0/data/title");
-        gwy_container_set_string_by_name(container, key, title);
-        g_free(key);
-        gwy_file_channel_import_log_add(container, 0, NULL, filename);
+        gwy_file_pass_title(file, GWY_FILE_IMAGE, 0, title);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
 
-        GWY_OBJECT_UNREF(filedata.whitelight);
+        g_clear_object(&filedata.whitelight);
     }
 
     fail:
     for (i = 0; i < filedata.nummaps; i++) {
-        GWY_OBJECT_UNREF(filedata.maps[i]->pset);
+        g_clear_object(&filedata.maps[i]->pset);
         g_free(filedata.maps[i]);
     }
     g_free(filedata.maps);
     g_free(filedata.maparea);
 
-    GWY_OBJECT_UNREF(metadata);
-    GWY_OBJECT_UNREF(siunitx);
-    GWY_OBJECT_UNREF(siunity);
-    GWY_OBJECT_UNREF(siunitw);
+    g_clear_object(&metadata);
+    g_clear_object(&siunitx);
+    g_clear_object(&siunity);
+    g_clear_object(&siunitw);
     g_free(zdata);
     g_free(buffer);
 
-    return container;
+    return file;
 }
 
 static gsize
-wdf_read_header(const guchar *buffer,
-                gsize size,
-                WdfHeader *header,
-                GError **error)
+read_header(const guchar *buffer,
+            gsize size,
+            WdfHeader *header,
+            GError **error)
 {
     gint i;
 
@@ -1363,10 +1335,10 @@ wdf_read_header(const guchar *buffer,
 }
 
 static gsize
-wdf_read_block_header(const guchar *buffer,
-                      gsize size,
-                      WdfBlock *header,
-                      GError **error)
+read_block_header(const guchar *buffer,
+                  gsize size,
+                  WdfBlock *header,
+                  GError **error)
 {
     if (size < WDF_BLOCK_HEADER_SIZE) {
         err_TRUNCATED_PART(error, "WdfBlock header");
@@ -1392,8 +1364,7 @@ wdf_read_block_header(const guchar *buffer,
 }
 
 static void
-wdf_read_maparea_block(const guchar *buffer,
-                       WdfMapArea *maparea)
+read_maparea_block(const guchar *buffer, WdfMapArea *maparea)
 {
     gint i;
 
@@ -1423,8 +1394,7 @@ wdf_read_maparea_block(const guchar *buffer,
 }
 
 static void
-set_standard_keys(gpointer key,
-                  G_GNUC_UNUSED gpointer value, gpointer user_data)
+set_standard_keys(gpointer key, G_GNUC_UNUSED gpointer value, gpointer user_data)
 {
     WdfPsetData *pdata;
     gpointer keys_value;
@@ -1435,7 +1405,7 @@ set_standard_keys(gpointer key,
     keys_value = g_hash_table_lookup(pdata->keys, key);
 
     if (!keys_value) {
-        key_text = gwy_enum_to_string(GPOINTER_TO_INT(key), wdf_predefined_keys, 9);
+        key_text = gwy_enum_to_string(GPOINTER_TO_INT(key), predefined_keys, 9);
         gwy_debug("key_text=%s", key_text);
 
         if (gwy_strequal(key_text, "")) {
@@ -1463,25 +1433,23 @@ extract_metadata_from_values(gpointer key, gpointer value,
 }
 
 static GwyContainer*
-wdf_read_pset(const guchar *buffer,
-              gsize size)
+read_pset(const guchar *buffer, gsize size)
 {
-    gsize remaining, size2;
+    gsize size2;
     gchar c;
     gint i, j;
     gint64 i64;
     gdouble d;
     gchar *str, *newstr;
-    GHashTable *keys, *values;
     WdfPropertySet pset;
     WdfPsetData pdata;
-    GwyContainer *data = NULL, *nested = NULL;
 
-    remaining = size;
-    keys = g_hash_table_new_full(g_direct_hash, g_direct_equal, NULL, g_free);
-    values = g_hash_table_new_full(g_direct_hash, g_direct_equal, NULL, g_free);
+    gsize remaining = size;
+    GHashTable *keys = g_hash_table_new_full(g_direct_hash, g_direct_equal, NULL, g_free);
+    GHashTable *values = g_hash_table_new_full(g_direct_hash, g_direct_equal, NULL, g_free);
 
-    data = gwy_container_new();
+    GwyContainer *data = gwy_container_new_in_construction();
+    GwyContainer *nested = NULL;
 
     while (remaining > 0) {
         pset.type = *(buffer++);
@@ -1548,7 +1516,7 @@ wdf_read_pset(const guchar *buffer,
                 case WDF_PTYPE_TIME:
                     i64 = gwy_get_gint64_le(&buffer);
                     gwy_debug("t = %" G_GINT64_FORMAT "", i64);
-                    g_hash_table_replace(values, GINT_TO_POINTER(pset.key), wdf_read_datetime(i64));
+                    g_hash_table_replace(values, GINT_TO_POINTER(pset.key), read_datetime(i64));
                     remaining -= 8;
                 break;
                 case WDF_PTYPE_STRING:
@@ -1578,10 +1546,9 @@ wdf_read_pset(const guchar *buffer,
                     gwy_debug("buffer before = %p", buffer);
                     gwy_debug("p size=%d", pset.size);
                     nested = NULL;
-                    nested = wdf_read_pset(buffer, size2);
-                    if (nested) {
-                        gwy_container_transfer(nested, data, "/", "/", FALSE);
-                    }
+                    nested = read_pset(buffer, size2);
+                    if (nested)
+                        gwy_container_transfer(nested, data, "/", "/", FALSE, FALSE);
                     gwy_debug("buffer after = %p", buffer);
                     buffer += size2;
                     remaining -= size2;
@@ -1723,8 +1690,8 @@ wdf_read_pset(const guchar *buffer,
     return data;
 }
 
-static gchar *
-wdf_read_datetime(gint64 ticks)
+static gchar*
+read_datetime(gint64 ticks)
 {
     gint64 time;
     gint hour, min, sec;
