@@ -1,6 +1,6 @@
 /*
- *  $Id: rhk-sm3.c 25672 2023-09-15 09:13:54Z yeti-dn $
- *  Copyright (C) 2005,2008 David Necas (Yeti), Petr Klapetek.
+ *  $Id: rhk-sm3.c 28848 2025-11-10 18:54:08Z yeti-dn $
+ *  Copyright (C) 2005-2025 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -43,16 +43,9 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/datafield.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <libgwydgets/gwygraphmodel.h>
-#include <libgwydgets/gwygraphcurvemodel.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 
 #include "err.h"
 #include "get.h"
@@ -60,8 +53,8 @@
 /* Ugly Microsoft UTF-16...
  * It reads: `STiMage 004.NNN N', but we do not check the NNN N */
 static const guchar MAGIC[] = {
-  0x53, 0x00, 0x54, 0x00, 0x69, 0x00, 0x4d, 0x00, 0x61, 0x00, 0x67, 0x00,
-  0x65, 0x00, 0x20, 0x00, 0x30, 0x00, 0x30, 0x00, 0x34, 0x00, 0x2e, 0x00,
+    0x53, 0x00, 0x54, 0x00, 0x69, 0x00, 0x4d, 0x00, 0x61, 0x00, 0x67, 0x00,
+    0x65, 0x00, 0x20, 0x00, 0x30, 0x00, 0x30, 0x00, 0x34, 0x00, 0x2e, 0x00,
 };
 
 #define EXTENSION ".sm3"
@@ -217,20 +210,20 @@ typedef struct {
     RHKColorInformation color_info;
 } RHKPage;
 
-static gboolean      module_register       (void);
-static gint          rhk_sm3_detect        (const GwyFileDetectInfo *fileinfo,
-                                            gboolean only_name);
-static GwyContainer* rhk_sm3_load          (const gchar *filename,
-                                            GwyRunType mode,
-                                            GError **error);
-static GwyContainer* rhk_sm3_get_metadata  (RHKPage *rhkpage);
+static gboolean      module_register(void);
+static gint          detect_file    (const GwyFileDetectInfo *fileinfo,
+                                     gboolean only_name);
+static GwyFile*      load_file      (const gchar *filename,
+                                     GwyRunModeFlags mode,
+                                     GError **error);
+static GwyContainer* get_metadata   (RHKPage *rhkpage);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports RHK Technology SM3 data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.16",
+    "0.17",
     "David Nečas (Yeti) & Petr Klapetek",
     "2005",
 };
@@ -249,17 +242,13 @@ module_register(void)
 {
     gwy_file_func_register("rhk-sm3",
                            N_("RHK SM3 files (.sm3)"),
-                           (GwyFileDetectFunc)&rhk_sm3_detect,
-                           (GwyFileLoadFunc)&rhk_sm3_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-rhk_sm3_detect(const GwyFileDetectInfo *fileinfo,
-               gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     gint score = 0;
 
@@ -274,8 +263,7 @@ rhk_sm3_detect(const GwyFileDetectInfo *fileinfo,
 }
 
 static guchar*
-rhk_sm3_read_string(const guchar **buffer,
-                    gsize len)
+read_string(const guchar **buffer, gsize len)
 {
     gchar *s;
     guint n;
@@ -297,7 +285,7 @@ rhk_sm3_read_string(const guchar **buffer,
 }
 
 static void
-rhk_sm3_page_free(RHKPage *page)
+page_free(RHKPage *page)
 {
     guint i;
 
@@ -307,9 +295,9 @@ rhk_sm3_page_free(RHKPage *page)
 }
 
 static RHKPage*
-rhk_sm3_read_page(const guchar **buffer,
-                  gsize *len,
-                  GError **error)
+read_page(const guchar **buffer,
+          gsize *len,
+          GError **error)
 {
     RHKPage *page;
     const guchar *p = *buffer;
@@ -391,7 +379,7 @@ rhk_sm3_read_page(const guchar **buffer,
         gchar *s;
 
         gwy_debug("position %04x", (guint)(p - *buffer));
-        s = rhk_sm3_read_string(&p, *len - (p - *buffer));
+        s = read_string(&p, *len - (p - *buffer));
         if (!s) {
             g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
                         _("End of file reached in string #%u."), i);
@@ -433,28 +421,27 @@ rhk_sm3_read_page(const guchar **buffer,
     return page;
 
 fail:
-    rhk_sm3_page_free(page);
+    page_free(page);
     return NULL;
 }
 
-static GwyDataField*
-rhk_sm3_page_to_data_field(const RHKPage *page)
+static GwyField*
+page_to_field(const RHKPage *page)
 {
-    GwyDataField *dfield;
+    GwyField *dfield;
     const gchar *unit;
     gint xres, yres;
 
     xres = page->x_size;
     yres = page->y_size;
-    dfield = gwy_data_field_new(xres, yres,
-                                xres*fabs(page->x_scale), yres*fabs(page->y_scale),
-                                FALSE);
+    dfield = gwy_field_new(xres, yres,
+                           xres*fabs(page->x_scale), yres*fabs(page->y_scale),
+                           FALSE);
     gwy_convert_raw_data(page->page_data, xres*yres, 1, GWY_RAW_DATA_SINT32, GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                         gwy_data_field_get_data(dfield), page->z_scale, page->z_offset);
-    gwy_data_field_invert(dfield, FALSE, TRUE, FALSE);
+                         gwy_field_get_data(dfield), page->z_scale, page->z_offset);
+    gwy_field_flip(dfield, TRUE, FALSE);
 
-    if (page->strings[RHK_STRING_X_UNITS]
-        && page->strings[RHK_STRING_Y_UNITS]) {
+    if (page->strings[RHK_STRING_X_UNITS] && page->strings[RHK_STRING_Y_UNITS]) {
         if (!gwy_strequal(page->strings[RHK_STRING_X_UNITS], page->strings[RHK_STRING_Y_UNITS]))
             g_warning("X and Y units differ, using X");
         unit = page->strings[RHK_STRING_X_UNITS];
@@ -465,7 +452,7 @@ rhk_sm3_page_to_data_field(const RHKPage *page)
         unit = page->strings[RHK_STRING_Y_UNITS];
     else
         unit = "";
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), unit);
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), unit);
 
     if (page->strings[RHK_STRING_Z_UNITS])
         unit = page->strings[RHK_STRING_Z_UNITS];
@@ -474,16 +461,16 @@ rhk_sm3_page_to_data_field(const RHKPage *page)
     /* Fix some silly units */
     if (gwy_strequal(unit, "N/sec"))
         unit = "s^-1";
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), unit);
+    gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), unit);
 
     return dfield;
 }
 
 static GwyGraphModel*
-rhk_sm3_page_to_spectra(const RHKPage *page)
+page_to_spectra(const RHKPage *page)
 {
     GwyGraphModel *gmodel;
-    GwyDataLine *dline;
+    GwyLine *dline;
     const gchar *unit;
     gint res, ncurves, i, j;
     const guchar *p;
@@ -492,15 +479,15 @@ rhk_sm3_page_to_spectra(const RHKPage *page)
     res = page->x_size;
     ncurves = page->y_size;
     gmodel = gwy_graph_model_new();
-    dline = gwy_data_line_new(res, res*fabs(page->x_scale), FALSE);
-    data = gwy_data_line_get_data(dline);
+    dline = gwy_line_new(res, res*fabs(page->x_scale), FALSE);
+    data = gwy_line_get_data(dline);
     p = page->page_data;
 
     if (page->strings[RHK_STRING_X_UNITS])
         unit = page->strings[RHK_STRING_X_UNITS];
     else
         unit = NULL;
-    gwy_si_unit_set_from_string(gwy_data_line_get_si_unit_x(dline), unit);
+    gwy_unit_set_from_string(gwy_line_get_unit_x(dline), unit);
 
     if (page->strings[RHK_STRING_Z_UNITS])
         unit = page->strings[RHK_STRING_Z_UNITS];
@@ -509,7 +496,7 @@ rhk_sm3_page_to_spectra(const RHKPage *page)
     /* Fix some silly units */
     if (gwy_strequal(unit, "N/sec"))
         unit = "s^-1";
-    gwy_si_unit_set_from_string(gwy_data_line_get_si_unit_y(dline), unit);
+    gwy_unit_set_from_string(gwy_line_get_unit_y(dline), unit);
 
     for (i = 0; i < ncurves; i++) {
         GwyGraphCurveModel *gcmodel = gwy_graph_curve_model_new();
@@ -519,7 +506,7 @@ rhk_sm3_page_to_spectra(const RHKPage *page)
         for (j = 0; j < res; j++)
             data[j] = gwy_get_gint32_le(&p) * page->z_scale + page->z_offset;
 
-        gwy_graph_curve_model_set_data_from_dataline(gcmodel, dline, 0, 0);
+        gwy_graph_curve_model_set_data_from_line(gcmodel, dline, 0, 0);
         g_object_set(gcmodel,
                      "mode", GWY_GRAPH_CURVE_LINE,
                      "color", gwy_graph_get_preset_color(i),
@@ -535,19 +522,19 @@ rhk_sm3_page_to_spectra(const RHKPage *page)
     return gmodel;
 }
 
-static GwyContainer*
-rhk_sm3_load(const gchar *filename,
-             G_GNUC_UNUSED GwyRunType mode,
-             GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
     GPtrArray *rhkfile;
     RHKPage *rhkpage;
-    GwyContainer *meta, *container = NULL;
+    GwyFile *file = NULL;
+    GwyContainer *meta;
     guchar *buffer = NULL;
     gsize fullsize, size = 0;
     GError *err = NULL;
     const guchar *p;
-    GString *key;
     guint i, count;
 
     if (!gwy_file_get_contents(filename, &buffer, &size, &err)) {
@@ -566,14 +553,14 @@ rhk_sm3_load(const gchar *filename,
     p = buffer;
     count = 0;
     gwy_debug("position %04x", (guint)(p - buffer));
-    while ((rhkpage = rhk_sm3_read_page(&p, &size, &err))) {
+    while ((rhkpage = read_page(&p, &size, &err))) {
         gwy_debug("Page #%u read OK", count);
         count++;
         rhkpage->pageno = count;
         gwy_debug("position %04x", (guint)(p - buffer));
         if (rhkpage->type != RHK_TYPE_IMAGE && rhkpage->type != RHK_TYPE_LINE) {
             gwy_debug("Page is neither IMAGE nor LINE, skipping");
-            rhk_sm3_page_free(rhkpage);
+            page_free(rhkpage);
             continue;
         }
         g_ptr_array_add(rhkfile, rhkpage);
@@ -592,14 +579,12 @@ rhk_sm3_load(const gchar *filename,
     }
     g_clear_error(&err);
 
-    container = gwy_container_new();
-    key = g_string_new(NULL);
+    file = gwy_file_new_in_construction();
 
     /* Process fields and lines separately to get nicer data ids */
     /* So, first fields. */
     for (i = count = 0; i < rhkfile->len; i++) {
-        GwyDataField *dfield;
-        GQuark quark;
+        GwyField *dfield;
         const gchar *cs;
         gchar *s;
 
@@ -607,27 +592,23 @@ rhk_sm3_load(const gchar *filename,
         if (rhkpage->type != RHK_TYPE_IMAGE)
             continue;
 
-        dfield = rhk_sm3_page_to_data_field(rhkpage);
-        quark = gwy_app_get_data_key_for_id(count);
-        gwy_container_pass_object(container, quark, dfield);
+        dfield = page_to_field(rhkpage);
+        gwy_file_pass_image(file, count, dfield);
         p = rhkpage->strings[RHK_STRING_LABEL];
         cs = gwy_enum_to_string(rhkpage->scan_dir, scan_directions, G_N_ELEMENTS(scan_directions));
         if (p && *p) {
-            g_string_assign(key, g_quark_to_string(quark));
-            g_string_append(key, "/title");
             if (cs)
                 s = g_strdup_printf("%s [%s]", p, cs);
             else
                 s = g_strdup(p);
-            gwy_container_set_string_by_name(container, key->str, s);
+            gwy_file_pass_title(file, GWY_FILE_IMAGE, count, s);
         }
 
-        meta = rhk_sm3_get_metadata(rhkpage);
-        g_string_printf(key, "/%d/meta", count);
-        gwy_container_pass_object_by_name(container, key->str, meta);
+        meta = get_metadata(rhkpage);
+        gwy_file_pass_meta(file, GWY_FILE_IMAGE, count, meta);
 
-        gwy_app_channel_check_nonsquare(container, count);
-        gwy_file_channel_import_log_add(container, count, NULL, filename);
+        gwy_check_nonsquare_image(file, count);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, count, NULL, filename);
         count++;
     }
 
@@ -636,7 +617,6 @@ rhk_sm3_load(const gchar *filename,
      * GwySpectra.  Just add them as graphs. */
     for (i = count = 0; i < rhkfile->len; i++) {
         GwyGraphModel *spectra;
-        GQuark quark;
         gchar *s = NULL;
 
         rhkpage = g_ptr_array_index(rhkfile, i);
@@ -644,7 +624,7 @@ rhk_sm3_load(const gchar *filename,
             continue;
 
         p = rhkpage->strings[RHK_STRING_LABEL];
-        if (!p || !*p)
+        if (!p || !*p) {
             p = gwy_enuml_to_string(rhkpage->line_type,
                                     "Histogram", RHK_LINE_HISTOGRAM,
                                     "Cross section", RHK_LINE_CROSS_SECTION,
@@ -662,38 +642,36 @@ rhk_sm3_load(const gchar *filename,
                                     "Image cross section", RHK_LINE_IMAGE_CROSS_SECTION,
                                     "Image avegrage", RHK_LINE_IMAGE_AVERAGE,
                                     NULL);
+        }
         if (!p || !*p)
             p = s = g_strdup_printf(_("Unknown line %d"), count);
 
-        spectra = rhk_sm3_page_to_spectra(rhkpage);
+        spectra = page_to_spectra(rhkpage);
         g_object_set(spectra, "title", p, NULL);
-        quark = gwy_app_get_graph_key_for_id(count);
-        gwy_container_pass_object(container, quark, spectra);
+        gwy_file_pass_graph(file, count, spectra);
+        gwy_log_add_import(file, GWY_FILE_GRAPH, count, NULL, filename);
 
         g_free(s);
 
         count++;
     }
 
-    g_string_free(key, TRUE);
-
     gwy_file_abandon_contents(buffer, fullsize, NULL);
     for (i = 0; i < rhkfile->len; i++)
-        rhk_sm3_page_free(g_ptr_array_index(rhkfile, i));
+        page_free(g_ptr_array_index(rhkfile, i));
     g_ptr_array_free(rhkfile, TRUE);
 
-    return container;
+    return file;
 }
 
 static GwyContainer*
-rhk_sm3_get_metadata(RHKPage *rhkpage)
+get_metadata(RHKPage *rhkpage)
 {
-    GwyContainer *meta;
     const gchar *s;
     gchar *str;
     guint i;
 
-    meta = gwy_container_new();
+    GwyContainer *meta = gwy_container_new_in_construction();
 
     s = gwy_enuml_to_string(rhkpage->page_type,
                             "Topographic", RHK_PAGE_TOPOGRAPHIC,
