@@ -1,6 +1,6 @@
 /*
- *  $Id: rhk-sm4.c 26510 2024-08-15 17:12:23Z yeti-dn $
- *  Copyright (C) 2009-2021 David Necas (Yeti).
+ *  $Id: rhk-sm4.c 28848 2025-11-10 18:54:08Z yeti-dn $
+ *  Copyright (C) 2009-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -43,17 +43,10 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/datafield.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <libgwydgets/gwygraphmodel.h>
-#include <libgwydgets/gwygraphcurvemodel.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 #include "gwyzlib.h"
 #include "err.h"
 #include "get.h"
@@ -370,74 +363,74 @@ typedef struct {
     RHKPageIndex *page_indices;
 } RHKFile;
 
-static gboolean             module_register                (void);
-static gint                 rhk_sm4_detect                 (const GwyFileDetectInfo *fileinfo,
-                                                            gboolean only_name);
-static GwyContainer*        rhk_sm4_load                   (const gchar *filename,
-                                                            GwyRunType mode,
-                                                            GError **error);
-static gboolean             rhk_sm4_read_page_index_header (RHKPageIndexHeader *header,
-                                                            const RHKObject *obj,
-                                                            const guchar *buffer,
-                                                            gsize size,
-                                                            GError **error);
-static gboolean             rhk_sm4_read_page_index        (RHKPageIndex *header,
-                                                            const RHKObject *obj,
-                                                            const guchar *buffer,
-                                                            gsize size,
-                                                            GError **error);
-static gboolean             rhk_sm4_read_page_header       (RHKPage *page,
-                                                            const RHKObject *obj,
-                                                            RHKDataType data_type,
-                                                            const guchar *buffer,
-                                                            gsize size,
-                                                            GError **error);
-static gboolean             rhk_sm4_read_page_data         (RHKPage *page,
-                                                            const RHKObject *obj,
-                                                            const guchar *buffer,
-                                                            GError **error);
-static gboolean             rhk_sm4_read_string_data       (RHKPage *page,
-                                                            const RHKObject *obj,
-                                                            guint count,
-                                                            const guchar *buffer);
-static RHKSpecDriftHeader*  rhk_sm4_read_drift_header      (const RHKObject *obj,
-                                                            const guchar *buffer);
-static RHKPiezoSensitivity* rhk_sm4_read_piezo_sensitivity (const RHKObject *obj,
-                                                            const guchar *buffer);
-static RHKSpecInfo*         rhk_sm4_read_spec_info         (const RHKObject *obj,
-                                                            const guchar *buffer,
-                                                            gsize size,
-                                                            guint nspec);
-static RHKObject*           rhk_sm4_read_objects           (const guchar *buffer,
-                                                            const guchar *p,
-                                                            gsize size,
-                                                            guint count,
-                                                            RHKObjectType intype,
-                                                            GError **error);
-static RHKObject*           rhk_sm4_find_object            (RHKObject *objects,
-                                                            guint count,
-                                                            RHKObjectType type,
-                                                            RHKObjectType parenttype,
-                                                            GError **error);
-static const gchar*         rhk_sm4_describe_object        (RHKObjectType type);
-static void                 rhk_sm4_free                   (RHKFile *rhkfile);
-static GwyDataField*        rhk_sm4_page_to_data_field     (const RHKPage *page);
-static GwyGraphModel*       rhk_sm4_page_to_graph_model    (const RHKPage *page);
-static GwyContainer*        rhk_sm4_get_metadata           (const RHKPageIndex *pi,
-                                                            const RHKPage *page,
-                                                            GwyContainer *basemeta);
-static void                 rhk_sm4_add_ppl_pro_status_meta(const RHKPage *page,
-                                                            GwyContainer *meta);
-static GwyContainer*        rhk_sm4_read_prm               (const RHKObject *prmheader,
-                                                            const RHKObject *prm,
-                                                            const guchar *buffer);
+static gboolean             module_register        (void);
+static gint                 detect_file            (const GwyFileDetectInfo *fileinfo,
+                                                    gboolean only_name);
+static GwyFile*             load_file              (const gchar *filename,
+                                                    GwyRunModeFlags mode,
+                                                    GError **error);
+static gboolean             read_page_index_header (RHKPageIndexHeader *header,
+                                                    const RHKObject *obj,
+                                                    const guchar *buffer,
+                                                    gsize size,
+                                                    GError **error);
+static gboolean             read_page_index        (RHKPageIndex *header,
+                                                    const RHKObject *obj,
+                                                    const guchar *buffer,
+                                                    gsize size,
+                                                    GError **error);
+static gboolean             read_page_header       (RHKPage *page,
+                                                    const RHKObject *obj,
+                                                    RHKDataType data_type,
+                                                    const guchar *buffer,
+                                                    gsize size,
+                                                    GError **error);
+static gboolean             read_page_data         (RHKPage *page,
+                                                    const RHKObject *obj,
+                                                    const guchar *buffer,
+                                                    GError **error);
+static gboolean             read_string_data       (RHKPage *page,
+                                                    const RHKObject *obj,
+                                                    guint count,
+                                                    const guchar *buffer);
+static RHKSpecDriftHeader*  read_drift_header      (const RHKObject *obj,
+                                                    const guchar *buffer);
+static RHKPiezoSensitivity* read_piezo_sensitivity (const RHKObject *obj,
+                                                    const guchar *buffer);
+static RHKSpecInfo*         read_spec_info         (const RHKObject *obj,
+                                                    const guchar *buffer,
+                                                    gsize size,
+                                                    guint nspec);
+static RHKObject*           read_objects           (const guchar *buffer,
+                                                    const guchar *p,
+                                                    gsize size,
+                                                    guint count,
+                                                    RHKObjectType intype,
+                                                    GError **error);
+static RHKObject*           find_object            (RHKObject *objects,
+                                                    guint count,
+                                                    RHKObjectType type,
+                                                    RHKObjectType parenttype,
+                                                    GError **error);
+static const gchar*         describe_object        (RHKObjectType type);
+static void                 free_rhkfile           (RHKFile *rhkfile);
+static GwyField*            page_to_field          (const RHKPage *page);
+static GwyGraphModel*       page_to_graph_model    (const RHKPage *page);
+static GwyContainer*        get_metadata           (const RHKPageIndex *pi,
+                                                    const RHKPage *page,
+                                                    GwyContainer *basemeta);
+static void                 add_ppl_pro_status_meta(const RHKPage *page,
+                                                    GwyContainer *meta);
+static GwyContainer*        read_prm               (const RHKObject *prmheader,
+                                                    const RHKObject *prm,
+                                                    const guchar *buffer);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports RHK Technology SM4 data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.8",
+    "0.10",
     "David Nečas (Yeti)",
     "2009",
 };
@@ -449,24 +442,20 @@ static const GwyEnum scan_directions[] = {
     { "Down",  RHK_SCAN_DOWN,  },
 };
 
-GWY_MODULE_QUERY(module_info)
+GWY_MODULE_QUERY2(module_info, rhk_sm4)
 
 static gboolean
 module_register(void)
 {
     gwy_file_func_register("rhk-sm4",
                            N_("RHK SM4 files (.sm4)"),
-                           (GwyFileDetectFunc)&rhk_sm4_detect,
-                           (GwyFileLoadFunc)&rhk_sm4_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-rhk_sm4_detect(const GwyFileDetectInfo *fileinfo,
-               gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     gint score = 0;
 
@@ -480,14 +469,15 @@ rhk_sm4_detect(const GwyFileDetectInfo *fileinfo,
     return score;
 }
 
-static GwyContainer*
-rhk_sm4_load(const gchar *filename,
-             G_GNUC_UNUSED GwyRunType mode,
-             GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
     RHKFile rhkfile;
     RHKObject *obj, o;
-    GwyContainer *meta, *prmmeta = NULL, *container = NULL;
+    GwyFile *file = NULL;
+    GwyContainer *prmmeta = NULL;
     guchar *buffer = NULL;
     gsize size = 0;
     GError *err = NULL;
@@ -500,7 +490,7 @@ rhk_sm4_load(const gchar *filename,
         return NULL;
     }
 
-    gwy_clear(&rhkfile, 1);
+    gwy_clear1(rhkfile);
     if (size < HEADER_SIZE) {
         err_TOO_SHORT(error);
         goto fail;
@@ -519,42 +509,42 @@ rhk_sm4_load(const gchar *filename,
     rhkfile.reserved2 = gwy_get_guint32_le(&p);
 
     /* Header objects */
-    if (!(rhkfile.objects = rhk_sm4_read_objects(buffer, p, size, rhkfile.object_count, RHK_OBJECT_FILE_HEADER, error)))
+    if (!(rhkfile.objects = read_objects(buffer, p, size, rhkfile.object_count, RHK_OBJECT_FILE_HEADER, error)))
         goto fail;
 
     /* Find page index header */
-    if (!(obj = rhk_sm4_find_object(rhkfile.objects, rhkfile.object_count,
-                                    RHK_OBJECT_PAGE_INDEX_HEADER, RHK_OBJECT_FILE_HEADER, error))
-        || !rhk_sm4_read_page_index_header(&rhkfile.page_index_header, obj, buffer, size, error))
+    if (!(obj = find_object(rhkfile.objects, rhkfile.object_count,
+                            RHK_OBJECT_PAGE_INDEX_HEADER, RHK_OBJECT_FILE_HEADER, error))
+        || !read_page_index_header(&rhkfile.page_index_header, obj, buffer, size, error))
         goto fail;
 
     /* There, find the page index array.  That's a single object in the object
      * list but it contains a page_count-long sequence of page indices. */
     rhkfile.page_indices = g_new0(RHKPageIndex,
                                   rhkfile.page_index_header.page_count);
-    if (!(obj = rhk_sm4_find_object(rhkfile.page_index_header.objects, rhkfile.page_index_header.object_count,
-                                    RHK_OBJECT_PAGE_INDEX_ARRAY, RHK_OBJECT_PAGE_INDEX_HEADER, error)))
+    if (!(obj = find_object(rhkfile.page_index_header.objects, rhkfile.page_index_header.object_count,
+                            RHK_OBJECT_PAGE_INDEX_ARRAY, RHK_OBJECT_PAGE_INDEX_HEADER, error)))
         goto fail;
 
     o = *obj;
     for (i = 0; i < rhkfile.page_index_header.page_count; i++) {
-        if (!rhk_sm4_read_page_index(rhkfile.page_indices + i, &o, buffer, size, error))
+        if (!read_page_index(rhkfile.page_indices + i, &o, buffer, size, error))
             goto fail;
 
         /* Carefully move to the next page index */
         o.offset += o.size + OBJECT_SIZE*rhkfile.page_indices[i].object_count;
     }
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     key = g_string_new(NULL);
 
     /* PRM metadata */
-    if ((obj = rhk_sm4_find_object(rhkfile.objects, rhkfile.object_count,
-                                   RHK_OBJECT_PRM_HEADER, RHK_OBJECT_FILE_HEADER, NULL))) {
+    if ((obj = find_object(rhkfile.objects, rhkfile.object_count,
+                           RHK_OBJECT_PRM_HEADER, RHK_OBJECT_FILE_HEADER, NULL))) {
         RHKObject *obj2;
-        if ((obj2 = rhk_sm4_find_object(rhkfile.objects, rhkfile.object_count,
-                                        RHK_OBJECT_PRM, RHK_OBJECT_FILE_HEADER, NULL))) {
-            prmmeta = rhk_sm4_read_prm(obj, obj2, buffer);
+        if ((obj2 = find_object(rhkfile.objects, rhkfile.object_count,
+                                RHK_OBJECT_PRM, RHK_OBJECT_FILE_HEADER, NULL))) {
+            prmmeta = read_prm(obj, obj2, buffer);
         }
     }
 
@@ -564,61 +554,55 @@ rhk_sm4_load(const gchar *filename,
         RHKPage *page = &pi->page;
 
         /* Page must contain header */
-        if (!(obj = rhk_sm4_find_object(pi->objects, pi->object_count,
-                                        RHK_OBJECT_PAGE_HEADER, RHK_OBJECT_PAGE_INDEX, error))
-            || !rhk_sm4_read_page_header(page, obj, pi->data_type, buffer, size, error))
+        if (!(obj = find_object(pi->objects, pi->object_count,
+                                RHK_OBJECT_PAGE_HEADER, RHK_OBJECT_PAGE_INDEX, error))
+            || !read_page_header(page, obj, pi->data_type, buffer, size, error))
             goto fail;
 
         /* Page must contain data */
-        if (!(obj = rhk_sm4_find_object(pi->objects, pi->object_count,
-                                        RHK_OBJECT_PAGE_DATA, RHK_OBJECT_PAGE_INDEX, error))
-            || !rhk_sm4_read_page_data(page, obj, buffer, error))
+        if (!(obj = find_object(pi->objects, pi->object_count,
+                                RHK_OBJECT_PAGE_DATA, RHK_OBJECT_PAGE_INDEX, error))
+            || !read_page_data(page, obj, buffer, error))
             goto fail;
 
         /* Page may contain strings */
-        if (!(obj = rhk_sm4_find_object(page->objects, page->object_count,
-                                        RHK_OBJECT_STRING_DATA, RHK_OBJECT_PAGE_HEADER, NULL))
-            || !rhk_sm4_read_string_data(page, obj, pi->page.string_count, buffer)) {
+        if (!(obj = find_object(page->objects, page->object_count,
+                                RHK_OBJECT_STRING_DATA, RHK_OBJECT_PAGE_HEADER, NULL))
+            || !read_string_data(page, obj, pi->page.string_count, buffer)) {
             g_warning("Failed to read string data in page %u", i);
         }
         /* Page may contain piezo sensitivity */
-        if ((obj = rhk_sm4_find_object(page->objects, page->object_count,
-                                       RHK_OBJECT_PIEZO_SENSITIVITY, RHK_OBJECT_PAGE_HEADER, NULL)))
-            page->piezo_sensitivity = rhk_sm4_read_piezo_sensitivity(obj, buffer);
+        if ((obj = find_object(page->objects, page->object_count,
+                               RHK_OBJECT_PIEZO_SENSITIVITY, RHK_OBJECT_PAGE_HEADER, NULL)))
+            page->piezo_sensitivity = read_piezo_sensitivity(obj, buffer);
 
         /* Read the data */
         if (pi->data_type == RHK_DATA_IMAGE) {
-            GwyDataField *dfield = rhk_sm4_page_to_data_field(page);
-            GQuark quark = gwy_app_get_data_key_for_id(imageid);
+            GwyField *dfield = page_to_field(page);
             const gchar *scandir, *name;
             gchar *title;
 
-            gwy_container_pass_object(container, quark, dfield);
+            gwy_file_pass_image(file, imageid, dfield);
 
             if ((name = page->strings[RHK_STRING_LABEL])) {
                 scandir = gwy_enum_to_string(page->scan_dir, scan_directions, G_N_ELEMENTS(scan_directions));
-                g_string_assign(key, g_quark_to_string(quark));
-                g_string_append(key, "/title");
                 if (scandir && *scandir)
                     title = g_strdup_printf("%s [%s]", name, scandir);
                 else
                     title = g_strdup(name);
-                gwy_container_set_string_by_name(container, key->str, title);
+                gwy_file_pass_title(file, GWY_FILE_IMAGE, imageid, title);
             }
             /* XXX: Maybe we should preferably show topograhical images?
              * Suggested by Andrés Muñiz Piniella.
             if (page->page_type == RHK_PAGE_TOPOGRAPHIC) {
                 g_string_assign(key, g_quark_to_string(quark));
                 g_string_append(key, "/visible");
-                gwy_container_set_boolean_by_name(container, key->str, TRUE);
+                gwy_container_set_boolean_by_name(file, key->str, TRUE);
             }
             */
 
-            meta = rhk_sm4_get_metadata(pi, page, prmmeta);
-            g_string_printf(key, "/%u/meta", imageid);
-            gwy_container_pass_object_by_name(container, key->str, meta);
-
-            gwy_file_channel_import_log_add(container, imageid, NULL, filename);
+            gwy_file_pass_meta(file, GWY_FILE_IMAGE, imageid, get_metadata(pi, page, prmmeta));
+            gwy_log_add_import(file, GWY_FILE_IMAGE, imageid, NULL, filename);
 
             imageid++;
         }
@@ -629,15 +613,16 @@ rhk_sm4_load(const gchar *filename,
             gwy_debug("line_type %u", page->line_type);
             gwy_debug("page_sizes %u %u", page->x_size, page->y_size);
             /* Page may contain drift header */
-            if ((obj = rhk_sm4_find_object(page->objects, page->object_count,
-                                           RHK_OBJECT_SPEC_DRIFT_HEADER, RHK_OBJECT_PAGE_HEADER, NULL)))
-                page->drift_header = rhk_sm4_read_drift_header(obj, buffer);
-            if ((obj = rhk_sm4_find_object(page->objects, page->object_count,
-                                           RHK_OBJECT_SPEC_DRIFT_DATA, RHK_OBJECT_PAGE_HEADER, NULL)))
-                page->spec_info = rhk_sm4_read_spec_info(obj, buffer, size, page->y_size);
-            if ((gmodel = rhk_sm4_page_to_graph_model(page))) {
+            if ((obj = find_object(page->objects, page->object_count,
+                                   RHK_OBJECT_SPEC_DRIFT_HEADER, RHK_OBJECT_PAGE_HEADER, NULL)))
+                page->drift_header = read_drift_header(obj, buffer);
+            if ((obj = find_object(page->objects, page->object_count,
+                                   RHK_OBJECT_SPEC_DRIFT_DATA, RHK_OBJECT_PAGE_HEADER, NULL)))
+                page->spec_info = read_spec_info(obj, buffer, size, page->y_size);
+            if ((gmodel = page_to_graph_model(page))) {
+                gwy_file_pass_graph(file, graphid, gmodel);
+                gwy_log_add_import(file, GWY_FILE_GRAPH, graphid, NULL, filename);
                 graphid++;
-                gwy_container_pass_object(container, gwy_app_get_graph_key_for_id(graphid), gmodel);
             }
         }
     }
@@ -647,29 +632,29 @@ rhk_sm4_load(const gchar *filename,
 
 fail:
     gwy_file_abandon_contents(buffer, size, NULL);
-    rhk_sm4_free(&rhkfile);
-    GWY_OBJECT_UNREF(prmmeta);
+    free_rhkfile(&rhkfile);
+    g_clear_object(&prmmeta);
     if (!imageid && !graphid) {
-        GWY_OBJECT_UNREF(container);
+        g_clear_object(&file);
     }
     if (key)
         g_string_free(key, TRUE);
 
-    return container;
+    return file;
 }
 
 static inline void
 err_OBJECT_TRUNCATED(GError **error, RHKObjectType type)
 {
-    err_TRUNCATED_PART(error, rhk_sm4_describe_object(type));
+    err_TRUNCATED_PART(error, describe_object(type));
 }
 
 static gboolean
-rhk_sm4_read_page_index_header(RHKPageIndexHeader *header,
-                               const RHKObject *obj,
-                               const guchar *buffer,
-                               gsize size,
-                               GError **error)
+read_page_index_header(RHKPageIndexHeader *header,
+                       const RHKObject *obj,
+                       const guchar *buffer,
+                       gsize size,
+                       GError **error)
 {
     const guchar *p;
 
@@ -686,16 +671,15 @@ rhk_sm4_read_page_index_header(RHKPageIndexHeader *header,
     header->reserved1 = gwy_get_guint32_le(&p);
     header->reserved2 = gwy_get_guint32_le(&p);
 
-    if (!(header->objects = rhk_sm4_read_objects(buffer, p, size, header->object_count, RHK_OBJECT_PAGE_INDEX_HEADER,
-                                                 error)))
+    if (!(header->objects = read_objects(buffer, p, size, header->object_count, RHK_OBJECT_PAGE_INDEX_HEADER, error)))
         return FALSE;
 
     return TRUE;
 }
 
 static gboolean
-rhk_sm4_read_page_index(RHKPageIndex *header, const RHKObject *obj,
-                        const guchar *buffer, gsize size, GError **error)
+read_page_index(RHKPageIndex *header, const RHKObject *obj,
+                const guchar *buffer, gsize size, GError **error)
 {
     const guchar *p;
 
@@ -714,16 +698,15 @@ rhk_sm4_read_page_index(RHKPageIndex *header, const RHKObject *obj,
     gwy_debug("data_type: %u, source: %u, object_count: %u, minorv: %u",
               header->data_type, header->source, header->object_count, header->minor_version);
 
-    if (!(header->objects = rhk_sm4_read_objects(buffer, p, size, header->object_count, RHK_OBJECT_PAGE_INDEX_ARRAY,
-                                                 error)))
+    if (!(header->objects = read_objects(buffer, p, size, header->object_count, RHK_OBJECT_PAGE_INDEX_ARRAY, error)))
         return FALSE;
 
     return TRUE;
 }
 
 static gboolean
-rhk_sm4_read_page_header(RHKPage *page, const RHKObject *obj, RHKDataType data_type,
-                         const guchar *buffer, gsize size, GError **error)
+read_page_header(RHKPage *page, const RHKObject *obj, RHKDataType data_type,
+                 const guchar *buffer, gsize size, GError **error)
 {
     const guchar *p;
     guint i;
@@ -807,15 +790,15 @@ rhk_sm4_read_page_header(RHKPage *page, const RHKObject *obj, RHKDataType data_t
     for (i = 0; i < G_N_ELEMENTS(page->reserved); i++)
         page->reserved[i] = gwy_get_guint32_le(&p);
 
-    if (!(page->objects = rhk_sm4_read_objects(buffer, p, size, page->object_count, RHK_OBJECT_PAGE_HEADER, error)))
+    if (!(page->objects = read_objects(buffer, p, size, page->object_count, RHK_OBJECT_PAGE_HEADER, error)))
         return FALSE;
 
     return TRUE;
 }
 
 static gboolean
-rhk_sm4_read_page_data(RHKPage *page, const RHKObject *obj,
-                       const guchar *buffer, GError **error)
+read_page_data(RHKPage *page, const RHKObject *obj,
+               const guchar *buffer, GError **error)
 {
     gsize expected_size;
 
@@ -829,7 +812,7 @@ rhk_sm4_read_page_data(RHKPage *page, const RHKObject *obj,
 }
 
 static gchar*
-rhk_sm4_read_string(const guchar **p, const guchar *end)
+read_string(const guchar **p, const guchar *end)
 {
     gchar *s;
     guint len;
@@ -848,8 +831,8 @@ rhk_sm4_read_string(const guchar **p, const guchar *end)
 }
 
 static gboolean
-rhk_sm4_read_string_data(RHKPage *page, const RHKObject *obj,
-                         guint count, const guchar *buffer)
+read_string_data(RHKPage *page, const RHKObject *obj,
+                 guint count, const guchar *buffer)
 {
     const guchar *p = buffer + obj->offset, *end = p + obj->size;
     guint i;
@@ -857,7 +840,7 @@ rhk_sm4_read_string_data(RHKPage *page, const RHKObject *obj,
     gwy_debug("count: %u, known strings: %u", count, RHK_STRING_NSTRINGS);
     count = MIN(count, RHK_STRING_NSTRINGS);
     for (i = 0; i < count; i++) {
-        if (!(page->strings[i] = rhk_sm4_read_string(&p, end)))
+        if (!(page->strings[i] = read_string(&p, end)))
             return FALSE;
         gwy_debug("string[%u]: <%s>", i, page->strings[i]);
     }
@@ -866,7 +849,7 @@ rhk_sm4_read_string_data(RHKPage *page, const RHKObject *obj,
 }
 
 static RHKSpecDriftHeader*
-rhk_sm4_read_drift_header(const RHKObject *obj, const guchar *buffer)
+read_drift_header(const RHKObject *obj, const guchar *buffer)
 {
     RHKSpecDriftHeader *drift_header = NULL;
     const guchar *p = buffer + obj->offset, *end = p + obj->size;
@@ -882,7 +865,7 @@ rhk_sm4_read_drift_header(const RHKObject *obj, const guchar *buffer)
     gwy_debug("nstrings = %u", nstrings);
     drift_header->strings = g_new0(gchar*, nstrings+1);
     for (i = 0; i < nstrings; i++) {
-        if (!(drift_header->strings[i] = rhk_sm4_read_string(&p, end))) {
+        if (!(drift_header->strings[i] = read_string(&p, end))) {
             g_strfreev(drift_header->strings);
             g_free(drift_header);
             return NULL;
@@ -894,7 +877,7 @@ rhk_sm4_read_drift_header(const RHKObject *obj, const guchar *buffer)
 }
 
 static RHKPiezoSensitivity*
-rhk_sm4_read_piezo_sensitivity(const RHKObject *obj, const guchar *buffer)
+read_piezo_sensitivity(const RHKObject *obj, const guchar *buffer)
 {
     RHKPiezoSensitivity *piezo_sensitivity = NULL;
     const guchar *p = buffer + obj->offset, *end = p + obj->size;
@@ -923,7 +906,7 @@ rhk_sm4_read_piezo_sensitivity(const RHKObject *obj, const guchar *buffer)
      * enough to hold the doubles and string count.  Even if there seem to be actually some
      * strings.  Other objects report their size correctly.  Why? */
     for (i = 0; i < nstrings; i++) {
-        if (!(piezo_sensitivity->strings[i] = rhk_sm4_read_string(&p, end))) {
+        if (!(piezo_sensitivity->strings[i] = read_string(&p, end))) {
             while (i--)
                 g_free(piezo_sensitivity->strings[i]);
             g_free(piezo_sensitivity);
@@ -936,8 +919,8 @@ rhk_sm4_read_piezo_sensitivity(const RHKObject *obj, const guchar *buffer)
 }
 
 static RHKSpecInfo*
-rhk_sm4_read_spec_info(const RHKObject *obj,
-                       const guchar *buffer, gsize size, guint nspec)
+read_spec_info(const RHKObject *obj,
+               const guchar *buffer, gsize size, guint nspec)
 {
     enum { SPEC_INFO_SIZE = 28 };
 
@@ -968,15 +951,15 @@ rhk_sm4_read_spec_info(const RHKObject *obj,
 /* FIXME: Some of the objects read are of type 0 and size 0, but maybe that's right and they allow empty object slots
  */
 static RHKObject*
-rhk_sm4_read_objects(const guchar *buffer, const guchar *p, gsize size, guint count,
-                     RHKObjectType intype, GError **error)
+read_objects(const guchar *buffer, const guchar *p, gsize size, guint count,
+             RHKObjectType intype, GError **error)
 {
     RHKObject *objects, *obj;
     guint i;
 
     if ((p - buffer) + count*OBJECT_SIZE >= size) {
         g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA, _("Object list in %s is truncated."),
-                    rhk_sm4_describe_object(intype));
+                    describe_object(intype));
         return NULL;
     }
 
@@ -987,10 +970,10 @@ rhk_sm4_read_objects(const guchar *buffer, const guchar *p, gsize size, guint co
         obj->offset = gwy_get_guint32_le(&p);
         obj->size = gwy_get_guint32_le(&p);
         gwy_debug("object of type %u (%s) at %u, size %u",
-                  obj->type, rhk_sm4_describe_object(obj->type), obj->offset, obj->size);
+                  obj->type, describe_object(obj->type), obj->offset, obj->size);
         if ((gsize)obj->size + obj->offset > size) {
             g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA, _("Object of type %s is truncated."),
-                        rhk_sm4_describe_object(obj->type));
+                        describe_object(obj->type));
             g_free(objects);
             return NULL;
         }
@@ -1000,9 +983,9 @@ rhk_sm4_read_objects(const guchar *buffer, const guchar *p, gsize size, guint co
 }
 
 static RHKObject*
-rhk_sm4_find_object(RHKObject *objects, guint count,
-                    RHKObjectType type, RHKObjectType parenttype,
-                    GError **error)
+find_object(RHKObject *objects, guint count,
+            RHKObjectType type, RHKObjectType parenttype,
+            GError **error)
 {
     guint i;
 
@@ -1014,12 +997,12 @@ rhk_sm4_find_object(RHKObject *objects, guint count,
     }
 
     g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA, _("Cannot find object %s in %s."),
-                rhk_sm4_describe_object(type), rhk_sm4_describe_object(parenttype));
+                describe_object(type), describe_object(parenttype));
     return NULL;
 }
 
 static const gchar*
-rhk_sm4_describe_object(RHKObjectType type)
+describe_object(RHKObjectType type)
 {
     static const GwyEnum types[] = {
         { "Undefined",          RHK_OBJECT_UNDEFINED,            },
@@ -1069,7 +1052,7 @@ rhk_sm4_describe_object(RHKObjectType type)
 };
 
 static void
-rhk_sm4_free(RHKFile *rhkfile)
+free_rhkfile(RHKFile *rhkfile)
 {
     RHKPage *page;
     guint i, j;
@@ -1098,20 +1081,20 @@ rhk_sm4_free(RHKFile *rhkfile)
     }
 }
 
-static GwyDataField*
-rhk_sm4_page_to_data_field(const RHKPage *page)
+static GwyField*
+page_to_field(const RHKPage *page)
 {
-    GwyDataField *dfield;
+    GwyField *dfield;
     const gchar *unit;
     gint xres, yres;
 
     xres = page->x_size;
     yres = page->y_size;
-    dfield = gwy_data_field_new(xres, yres, xres*fabs(page->x_scale), yres*fabs(page->y_scale), FALSE);
+    dfield = gwy_field_new(xres, yres, xres*fabs(page->x_scale), yres*fabs(page->y_scale), FALSE);
     gwy_convert_raw_data(page->data, xres*yres, 1, GWY_RAW_DATA_SINT32, GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                         gwy_data_field_get_data(dfield), page->z_scale, page->z_offset);
+                         gwy_field_get_data(dfield), page->z_scale, page->z_offset);
     /* Correct flipping of up images */
-    gwy_data_field_invert(dfield, page->y_scale > 0.0, TRUE, FALSE);
+    gwy_field_flip(dfield, TRUE, page->y_scale > 0.0);
 
     /* XY units */
     if (page->strings[RHK_STRING_X_UNITS]
@@ -1127,7 +1110,7 @@ rhk_sm4_page_to_data_field(const RHKPage *page)
     else
         unit = NULL;
 
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), unit);
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), unit);
 
     /* Z units */
     if (page->strings[RHK_STRING_Z_UNITS])
@@ -1140,17 +1123,17 @@ rhk_sm4_page_to_data_field(const RHKPage *page)
     else if (unit && gwy_stramong(unit, "Vrms", "Vp", NULL))
         unit = "V";
 
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), unit);
+    gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), unit);
 
     return dfield;
 }
 
 static GwyGraphModel*
-rhk_sm4_page_to_graph_model(const RHKPage *page)
+page_to_graph_model(const RHKPage *page)
 {
     GwyGraphModel *gmodel;
     GwyGraphCurveModel *gcmodel;
-    GwySIUnit *siunit;
+    GwyUnit *siunit;
     const gint32 *pdata;
     const gchar *name;
     gint res, ncurves, i, j;
@@ -1184,12 +1167,12 @@ rhk_sm4_page_to_graph_model(const RHKPage *page)
     g_free(xdata);
 
     /* Units */
-    siunit = gwy_si_unit_new(page->strings[RHK_STRING_X_UNITS]);
-    g_object_set(gmodel, "si-unit-x", siunit, NULL);
+    siunit = gwy_unit_new(page->strings[RHK_STRING_X_UNITS]);
+    g_object_set(gmodel, "unit-x", siunit, NULL);
     g_object_unref(siunit);
 
-    siunit = gwy_si_unit_new(page->strings[RHK_STRING_Z_UNITS]);
-    g_object_set(gmodel, "si-unit-y", siunit, NULL);
+    siunit = gwy_unit_new(page->strings[RHK_STRING_Z_UNITS]);
+    g_object_set(gmodel, "unit-y", siunit, NULL);
     g_object_unref(siunit);
 
     if (name)
@@ -1199,7 +1182,7 @@ rhk_sm4_page_to_graph_model(const RHKPage *page)
 }
 
 static void
-rhk_sm4_meta_string(const RHKPage *page, RHKStringType stringid, const gchar *name, GwyContainer *meta)
+meta_string(const RHKPage *page, RHKStringType stringid, const gchar *name, GwyContainer *meta)
 {
     const gchar *s;
 
@@ -1236,7 +1219,7 @@ set_meta_int(GwyContainer *meta, GString *str, const gchar *prefix, const gchar
 }
 
 static GwyContainer*
-rhk_sm4_get_metadata(const RHKPageIndex *pi, const RHKPage *page, GwyContainer *basemeta)
+get_metadata(const RHKPageIndex *pi, const RHKPage *page, GwyContainer *basemeta)
 {
     static const gchar hex[] = "0123456789abcdef";
 
@@ -1247,9 +1230,9 @@ rhk_sm4_get_metadata(const RHKPageIndex *pi, const RHKPage *page, GwyContainer *
     guint i, w;
 
     if (basemeta)
-        meta = gwy_container_duplicate(basemeta);
+        meta = gwy_container_copy(basemeta);
     else
-        meta = gwy_container_new();
+        meta = gwy_container_new_in_construction();
 
     key = g_string_new(NULL);
     s = gwy_enuml_to_string(page->page_type,
@@ -1339,22 +1322,22 @@ rhk_sm4_get_metadata(const RHKPageIndex *pi, const RHKPage *page, GwyContainer *
         gwy_container_set_string_by_name(meta, "Date", str);
     }
 
-    rhk_sm4_meta_string(page, RHK_STRING_LABEL, "Label", meta);
-    rhk_sm4_meta_string(page, RHK_STRING_PATH, "Path", meta);
-    rhk_sm4_meta_string(page, RHK_STRING_SYSTEM_TEXT, "System comment", meta);
-    rhk_sm4_meta_string(page, RHK_STRING_SESSION_TEXT, "Session comment", meta);
-    rhk_sm4_meta_string(page, RHK_STRING_USER_TEXT, "User comment", meta);
-    rhk_sm4_meta_string(page, RHK_STRING_X_UNITS, "X units", meta);
-    rhk_sm4_meta_string(page, RHK_STRING_Y_UNITS, "Y units", meta);
-    rhk_sm4_meta_string(page, RHK_STRING_Z_UNITS, "Z units", meta);
-    rhk_sm4_meta_string(page, RHK_STRING_X_LABEL, "X label", meta);
-    rhk_sm4_meta_string(page, RHK_STRING_Y_LABEL, "Y label", meta);
-    rhk_sm4_meta_string(page, RHK_STRING_STATUS_CHANNEL_TEXT, "Status channel text", meta);
-    rhk_sm4_meta_string(page, RHK_STRING_COMPLETED_LINE_COUNT, "Completed line count", meta);
-    rhk_sm4_meta_string(page, RHK_STRING_OVERSAMPLING_COUNT, "Oversampling count", meta);
-    rhk_sm4_meta_string(page, RHK_STRING_SLICED_VOLTAGE, "Sliced voltage", meta);
+    meta_string(page, RHK_STRING_LABEL, "Label", meta);
+    meta_string(page, RHK_STRING_PATH, "Path", meta);
+    meta_string(page, RHK_STRING_SYSTEM_TEXT, "System comment", meta);
+    meta_string(page, RHK_STRING_SESSION_TEXT, "Session comment", meta);
+    meta_string(page, RHK_STRING_USER_TEXT, "User comment", meta);
+    meta_string(page, RHK_STRING_X_UNITS, "X units", meta);
+    meta_string(page, RHK_STRING_Y_UNITS, "Y units", meta);
+    meta_string(page, RHK_STRING_Z_UNITS, "Z units", meta);
+    meta_string(page, RHK_STRING_X_LABEL, "X label", meta);
+    meta_string(page, RHK_STRING_Y_LABEL, "Y label", meta);
+    meta_string(page, RHK_STRING_STATUS_CHANNEL_TEXT, "Status channel text", meta);
+    meta_string(page, RHK_STRING_COMPLETED_LINE_COUNT, "Completed line count", meta);
+    meta_string(page, RHK_STRING_OVERSAMPLING_COUNT, "Oversampling count", meta);
+    meta_string(page, RHK_STRING_SLICED_VOLTAGE, "Sliced voltage", meta);
 
-    rhk_sm4_add_ppl_pro_status_meta(page, meta);
+    add_ppl_pro_status_meta(page, meta);
 
     str = g_new(gchar, 33);
     for (i = 0; i < 16; i++) {
@@ -1379,7 +1362,7 @@ rhk_sm4_get_metadata(const RHKPageIndex *pi, const RHKPage *page, GwyContainer *
 }
 
 static void
-rhk_sm4_add_ppl_pro_status_meta(const RHKPage *page, GwyContainer *meta)
+add_ppl_pro_status_meta(const RHKPage *page, GwyContainer *meta)
 {
     GString *key;
     gchar **lines;
@@ -1428,7 +1411,7 @@ rhk_sm4_add_ppl_pro_status_meta(const RHKPage *page, GwyContainer *meta)
 }
 
 static GwyContainer*
-rhk_sm4_read_prm(const RHKObject *prmheader, const RHKObject *prm, const guchar *buffer)
+read_prm(const RHKObject *prmheader, const RHKObject *prm, const guchar *buffer)
 {
     GwyContainer *prmmeta = NULL;
     gchar *prmtext = NULL, *q, *line;
@@ -1486,7 +1469,7 @@ rhk_sm4_read_prm(const RHKObject *prmheader, const RHKObject *prm, const guchar
     metaregex = g_regex_new("^<[0-9]{4}>\\s+(.+?)\\s+::(.*)$", G_REGEX_OPTIMIZE, 0, NULL);
     g_assert(metaregex);
 
-    prmmeta = gwy_container_new();
+    prmmeta = gwy_container_new_in_construction();
     key = g_string_new(NULL);
     q = prmtext;
     while ((line = gwy_str_next_line(&q))) {
