@@ -1,6 +1,6 @@
 /*
- *  $Id: rhk-spm32.c 26510 2024-08-15 17:12:23Z yeti-dn $
- *  Copyright (C) 2005 David Necas (Yeti), Petr Klapetek.
+ *  $Id: rhk-spm32.c 28911 2025-11-24 18:27:42Z yeti-dn $
+ *  Copyright (C) 2005-2025 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -42,17 +42,11 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/stats.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
-#include <libprocess/spectra.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -62,6 +56,7 @@
 #define MAGIC_SIZE (sizeof(MAGIC) - 1)
 #define EXTENSION ".sm2"
 
+/* FIXME GTK3: change to actual graph metadata once we some. */
 #define GRAPH_PREFIX "/0/graph/graph"
 
 typedef enum {
@@ -150,29 +145,29 @@ typedef struct {
     const guchar *buffer;
 } RHKPage;
 
-static gboolean       module_register      (void);
-static gint           rhkspm32_detect      (const GwyFileDetectInfo *fileinfo,
-                                            gboolean only_name);
-static GwyContainer*  rhkspm32_load        (const gchar *filename,
-                                            GwyRunType mode,
-                                            GError **error);
-static gboolean       rhkspm32_read_header (RHKPage *rhkpage,
-                                            GError **error);
-static gboolean       rhkspm32_read_range  (const gchar *buffer,
-                                            const gchar *name,
-                                            RHKRange *range);
-static void           rhkspm32_free        (RHKPage *rhkpage);
-static GwyContainer*  rhkspm32_get_metadata(RHKPage *rhkpage);
-static GwyDataField*  rhkspm32_read_data   (RHKPage *rhkpage);
-static GwySpectra*    rhkspm32_read_spectra(RHKPage *rhkpage);
-static GwyGraphModel* spectra_to_graph     (GwySpectra *spectra);
+static gboolean       module_register (void);
+static gint           detect_file     (const GwyFileDetectInfo *fileinfo,
+                                       gboolean only_name);
+static GwyFile*       load_file       (const gchar *filename,
+                                       GwyRunModeFlags mode,
+                                       GError **error);
+static gboolean       read_header     (RHKPage *rhkpage,
+                                       GError **error);
+static gboolean       read_range      (const gchar *buffer,
+                                       const gchar *name,
+                                       RHKRange *range);
+static void           free_rhkpage    (RHKPage *rhkpage);
+static GwyContainer*  get_metadata    (RHKPage *rhkpage);
+static GwyField*      read_data       (RHKPage *rhkpage);
+static GwySpectra*    read_spectra    (RHKPage *rhkpage);
+static GwyGraphModel* spectra_to_graph(GwySpectra *spectra);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports RHK Technology SPM32 data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.12",
+    "0.13",
     "David Nečas (Yeti) & Petr Klapetek, mod by Niv Levy",
     "2007",
 };
@@ -191,17 +186,13 @@ module_register(void)
 {
     gwy_file_func_register("rhk-spm32",
                            N_("RHK SPM32 files (.sm2)"),
-                           (GwyFileDetectFunc)&rhkspm32_detect,
-                           (GwyFileLoadFunc)&rhkspm32_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-rhkspm32_detect(const GwyFileDetectInfo *fileinfo,
-                gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     gint score = 0;
 
@@ -215,18 +206,19 @@ rhkspm32_detect(const GwyFileDetectInfo *fileinfo,
     return score;
 }
 
-static GwyContainer*
-rhkspm32_load(const gchar *filename,
-              G_GNUC_UNUSED GwyRunType mode,
-              GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
     GArray *rhkfile;
     RHKPage *rhkpage;
-    GwyContainer *meta, *container = NULL;
+    GwyFile *file = NULL;
+    GwyContainer *meta;
     guchar *buffer = NULL;
     gsize size = 0;
     GError *err = NULL;
-    GwyDataField *dfield = NULL;
+    GwyField *dfield = NULL;
     gsize totalpos, pagesize;
     GString *key;
     guint i;
@@ -251,7 +243,7 @@ rhkspm32_load(const gchar *filename,
         rhkpage->buffer = buffer + totalpos;
         // niv - if the header seems invalid, skip all the next ones as well (and cancel the element addition to the
         // g_array)
-        if (!rhkspm32_read_header(rhkpage, &err)) {
+        if (!read_header(rhkpage, &err)) {
             g_warning("failed to read rhk header after %u", rhkfile->len);
             g_free(rhkpage->date);
             g_array_set_size(rhkfile, rhkfile->len - 1);
@@ -260,7 +252,7 @@ rhkspm32_load(const gchar *filename,
 
         pagesize = rhkpage->data_offset + rhkpage->item_size*rhkpage->xres*rhkpage->yres;
         if (size < totalpos + pagesize) {
-            rhkspm32_free(rhkpage);
+            free_rhkpage(rhkpage);
             g_array_set_size(rhkfile, rhkfile->len - 1);
             break;
         }
@@ -280,7 +272,7 @@ rhkspm32_load(const gchar *filename,
     }
     g_clear_error(&err);
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     key = g_string_new(NULL);
     for (i = 0; i < rhkfile->len; i++) {
         const gchar *cs;
@@ -289,57 +281,54 @@ rhkspm32_load(const gchar *filename,
         gwy_debug("rhk-spm32: processing page %d of %d\n", i+1, rhkfile->len);
         rhkpage = &g_array_index(rhkfile, RHKPage, i);
         if (rhkpage->type == RHK_TYPE_IMAGE) { // niv - just leaving this alone
-            dfield = rhkspm32_read_data(rhkpage);
-            gwy_container_pass_object(container, gwy_app_get_data_key_for_id(i), dfield);
+            dfield = read_data(rhkpage);
+            gwy_file_pass_image(file, i, dfield);
             cs = gwy_enum_to_string(rhkpage->scan, scan_directions, G_N_ELEMENTS(scan_directions));
             if (rhkpage->label) {
                 s = (cs ? g_strdup_printf("%s [%s]", rhkpage->label, cs) : g_strdup(rhkpage->label));
-                gwy_container_set_string(container, gwy_app_get_data_title_key_for_id(i), s);
+                gwy_file_pass_title(file, GWY_FILE_IMAGE, i, s);
             }
             else
-                gwy_app_channel_title_fall_back(container, i);
+                gwy_image_title_fall_back(file, i);
 
-            gwy_file_channel_import_log_add(container, i, NULL, filename);
+            gwy_log_add_import(file, GWY_FILE_IMAGE, i, NULL, filename);
         }
         else if (rhkpage->type == RHK_TYPE_LINE) { // niv - after omicron.c
-            GwySpectra* spectra;
+            GwySpectra* spectra = read_spectra(rhkpage);
             GwyGraphModel *gmodel;
 
-            spectra = rhkspm32_read_spectra(rhkpage);
             /* converting to graphs, as there is no point in leaving these as sps - no xy coordinates, so the spectro
              * tool is kinda clueless */
             gwy_debug("processing graph in page %d\n", i);
-            if ((gmodel = spectra_to_graph(spectra)))
-                gwy_container_pass_object(container, gwy_app_get_graph_key_for_id(i+1), gmodel);
+            if ((gmodel = spectra_to_graph(spectra))) {
+                gwy_file_pass_graph(file, i, gmodel);
+                gwy_log_add_import(file, GWY_FILE_GRAPH, i, NULL, filename);
+            }
             g_object_unref(spectra);
         }
         else
             continue;
 
         gwy_debug("rhk-spm32: finished parsing page %d \n", i);
-        meta = rhkspm32_get_metadata(rhkpage);
+        meta = get_metadata(rhkpage);
         if (rhkpage->type == RHK_TYPE_IMAGE)
-            gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(i), meta);
-        else if (rhkpage->type == RHK_TYPE_LINE) {
-            /* this doesn't really work, but at least the meta data stays with the graph, even if the metadata viewer
-             * won't show it*/
-            g_string_printf(key, "%s/%d/meta", GRAPH_PREFIX, i+1);
-            gwy_container_pass_object_by_name(container, key->str, meta);
-        }
+            gwy_file_pass_meta(file, GWY_FILE_IMAGE, i, meta);
+        else if (rhkpage->type == RHK_TYPE_LINE)
+            gwy_file_pass_meta(file, GWY_FILE_GRAPH, i, meta);
     }
     g_string_free(key, TRUE);
 
     gwy_file_abandon_contents(buffer, size, NULL);
     for (i = 0; i < rhkfile->len; i++)
-        rhkspm32_free(&g_array_index(rhkfile, RHKPage, i));
+        free_rhkpage(&g_array_index(rhkfile, RHKPage, i));
     g_array_free(rhkfile, TRUE);
 
-    return container;
+    return file;
 }
 
 static gboolean
-rhkspm32_read_header(RHKPage *rhkpage,
-                     GError **error)
+read_header(RHKPage *rhkpage,
+            GError **error)
 {
     const gchar *buffer;
     gchar *end;
@@ -383,9 +372,9 @@ rhkspm32_read_header(RHKPage *rhkpage,
     }
     rhkpage->item_size = gwy_raw_data_size(rhkpage->rawtype);
 
-    if (!rhkspm32_read_range(buffer + 0x40, "X", &rhkpage->x)
-        || !rhkspm32_read_range(buffer + 0x60, "Y", &rhkpage->y)
-        || !rhkspm32_read_range(buffer + 0x80, "Z", &rhkpage->z)) {
+    if (!read_range(buffer + 0x40, "X", &rhkpage->x)
+        || !read_range(buffer + 0x60, "Y", &rhkpage->y)
+        || !read_range(buffer + 0x80, "Z", &rhkpage->z)) {
         err_INVALID(error, _("data ranges"));
         g_free(rhkpage->date);
         return FALSE;
@@ -419,7 +408,7 @@ rhkspm32_read_header(RHKPage *rhkpage,
     else
         rhkpage->e_alpha = TRUE;
 
-    if (!rhkspm32_read_range(buffer + 0xc0, "IV", &rhkpage->iv)) {
+    if (!read_range(buffer + 0xc0, "IV", &rhkpage->iv)) {
         err_INVALID(error, "IV");
         g_free(rhkpage->date);
         return FALSE;
@@ -466,9 +455,9 @@ rhkspm32_read_header(RHKPage *rhkpage,
 }
 
 static gboolean
-rhkspm32_read_range(const gchar *buffer,
-                    const gchar *name,
-                    RHKRange *range)
+read_range(const gchar *buffer,
+           const gchar *name,
+           RHKRange *range)
 {
     gchar *end;
     guint pos;
@@ -494,7 +483,7 @@ rhkspm32_read_range(const gchar *buffer,
 }
 
 static void
-rhkspm32_free(RHKPage *rhkpage)
+free_rhkpage(RHKPage *rhkpage)
 {
     g_free(rhkpage->date);
     g_free(rhkpage->x.units);
@@ -506,13 +495,11 @@ rhkspm32_free(RHKPage *rhkpage)
 }
 
 static GwyContainer*
-rhkspm32_get_metadata(RHKPage *rhkpage)
+get_metadata(RHKPage *rhkpage)
 {
-    GwyContainer *meta;
+    GwyContainer *meta = gwy_container_new_in_construction();
     const gchar *s;
 
-    meta = gwy_container_new();
-
     gwy_container_set_string_by_name(meta, "Tunneling voltage", g_strdup_printf("%g mV", 1e3*rhkpage->iv.offset));
     gwy_container_set_string_by_name(meta, "Current", g_strdup_printf("%g nA", 1e9*rhkpage->iv.scale));
     if (rhkpage->id)
@@ -536,10 +523,10 @@ rhkspm32_get_metadata(RHKPage *rhkpage)
     return meta;
 }
 
-static GwyDataField*
-rhkspm32_read_data(RHKPage *rhkpage)
+static GwyField*
+read_data(RHKPage *rhkpage)
 {
-    GwyDataField *dfield;
+    GwyField *dfield;
     const gchar *s;
     gdouble q;
     gint power10;
@@ -548,36 +535,36 @@ rhkspm32_read_data(RHKPage *rhkpage)
     xres = rhkpage->xres;
     yres = rhkpage->yres;
     /* the scales are no longer gurunteed to be positive, so they must be "fixed" here (to enable spectra) */
-    dfield = gwy_data_field_new(xres, yres, xres*fabs(rhkpage->x.scale), yres*fabs(rhkpage->y.scale), FALSE);
+    dfield = gwy_field_new(xres, yres, xres*fabs(rhkpage->x.scale), yres*fabs(rhkpage->y.scale), FALSE);
 
-    gwy_si_unit_set_from_string_parse(gwy_data_field_get_si_unit_xy(dfield), rhkpage->x.units, &power10);
+    power10 = gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), rhkpage->x.units);
     if (power10) {
-        q = pow10(power10);
-        gwy_data_field_set_xreal(dfield, q*gwy_data_field_get_xreal(dfield));
-        gwy_data_field_set_yreal(dfield, q*gwy_data_field_get_yreal(dfield));
+        q = gwy_exp10(power10);
+        gwy_field_set_xreal(dfield, q*gwy_field_get_xreal(dfield));
+        gwy_field_set_yreal(dfield, q*gwy_field_get_yreal(dfield));
     }
 
     s = rhkpage->z.units;
     /* Fix some silly units */
     if (gwy_strequal(s, "N/sec"))
         s = "s^-1";
-    gwy_si_unit_set_from_string_parse(gwy_data_field_get_si_unit_z(dfield), s, &power10);
-    q = pow10(power10);
+    power10 = gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), s);
+    q = gwy_exp10(power10);
 
     gwy_convert_raw_data(rhkpage->buffer + rhkpage->data_offset, xres*yres, 1,
-                         rhkpage->rawtype, GWY_BYTE_ORDER_LITTLE_ENDIAN, gwy_data_field_get_data(dfield),
+                         rhkpage->rawtype, GWY_BYTE_ORDER_LITTLE_ENDIAN, gwy_field_get_data(dfield),
                          q*fabs(rhkpage->z.scale), q*rhkpage->z.offset);
-    gwy_data_field_invert(dfield, FALSE, TRUE, FALSE);
+    gwy_field_flip(dfield, TRUE, FALSE);
 
     return dfield;
 }
 
 static GwySpectra*
-rhkspm32_read_spectra(RHKPage *rhkpage)
+read_spectra(RHKPage *rhkpage)
 {
     guint i, rowstride;
-    GwySIUnit *siunit = NULL;
-    GwyDataLine *dline;
+    GwyUnit *siunit = NULL;
+    GwyLine *dline;
     GwySpectra *spectra = NULL;
     GPtrArray *spectrum = NULL;
     // i'm leaving this alone, though it probably doesn't make sense, and i should just create graphs straight away
@@ -590,12 +577,12 @@ rhkspm32_read_spectra(RHKPage *rhkpage)
     gwy_debug("rhk-spm32: %d spectra in this page\n", rhkpage->yres);
     rowstride = rhkpage->item_size * rhkpage->xres;
     for (i = 0; i < rhkpage->yres; i++) {
-        dline = gwy_data_line_new(rhkpage->xres, rhkpage->x.scale, FALSE);
-        gwy_data_line_set_offset(dline, rhkpage->x.offset);
+        dline = gwy_line_new(rhkpage->xres, rhkpage->x.scale, FALSE);
+        gwy_line_set_offset(dline, rhkpage->x.offset);
         gwy_convert_raw_data(rhkpage->buffer + rhkpage->data_offset + i*rowstride, rhkpage->xres, 1,
-                             rhkpage->rawtype, GWY_BYTE_ORDER_LITTLE_ENDIAN, gwy_data_line_get_data(dline),
+                             rhkpage->rawtype, GWY_BYTE_ORDER_LITTLE_ENDIAN, gwy_line_get_data(dline),
                              rhkpage->z.scale, rhkpage->z.offset);
-        gwy_si_unit_set_from_string(gwy_data_line_get_si_unit_x(dline), rhkpage->x.units);
+        gwy_unit_set_from_string(gwy_line_get_unit_x(dline), rhkpage->x.units);
 
         // the y units (and data) for a 1D graph are stored in Z in the rhk spm32 format!
         /* Fix "/\xfbHz" to "/Hz".
@@ -603,12 +590,12 @@ rhkspm32_read_spectra(RHKPage *rhkpage)
          * sqrt. */
         if (g_str_has_suffix(rhkpage->z.units, "/\xfbHz")) {
             gchar *s = gwy_strkill(g_strdup(rhkpage->z.units), "\xfb");
-            siunit = gwy_si_unit_new(s);
+            siunit = gwy_unit_new(s);
             g_free(s);
         }
         else
-            siunit = gwy_si_unit_new(rhkpage->z.units);
-        gwy_data_line_set_si_unit_y(dline, siunit);
+            siunit = gwy_unit_new(rhkpage->z.units);
+        gwy_unit_assign(gwy_line_get_unit_y(dline), siunit);
         g_object_unref(siunit);
 
         if (!spectrum)
@@ -641,29 +628,29 @@ spectra_to_graph(GwySpectra *spectra)
     GwyGraphCurveModel *cmodel;
     gchar *curve_title = NULL;
     guint j, k, n_spectra, n_points;
-    GwyDataLine *dline;
+    GwyLine *dline;
     gdouble *data, *xdata, *ydata, x_offset, x_realsize;
-    GwySIUnit *x_si_unit, *y_si_unit;
+    GwyUnit *x_unit, *y_unit;
 
     if (!(n_spectra = gwy_spectra_get_n_spectra(spectra))) {
         gwy_debug("rhk-spm32: no spectra in rhkpage - something is odd\n");
         return NULL;
     }
     dline = gwy_spectra_get_spectrum(spectra, 0);
-    n_points = gwy_data_line_get_res(dline);
-    x_si_unit = gwy_data_line_get_si_unit_x(dline);
-    y_si_unit = gwy_data_line_get_si_unit_y(dline);
-    x_offset = gwy_data_line_get_offset(dline);
-    x_realsize = gwy_data_line_get_real(dline);
+    n_points = gwy_line_get_res(dline);
+    x_unit = gwy_line_get_unit_x(dline);
+    y_unit = gwy_line_get_unit_y(dline);
+    x_offset = gwy_line_get_offset(dline);
+    x_realsize = gwy_line_get_real(dline);
     xdata = gwy_math_linspace(NULL, n_points, x_offset, x_realsize);
     ydata = g_new0(gdouble, n_points);
     gmodel = gwy_graph_model_new();
-    g_object_set(gmodel, "si-unit-x", x_si_unit, "si-unit-y", y_si_unit, NULL);
+    g_object_set(gmodel, "unit-x", x_unit, "unit-y", y_unit, NULL);
     graph_title = gwy_spectra_get_title(spectra);
     g_object_set(gmodel, "title", graph_title, NULL);
     for (k = 1; k <= n_spectra; k++) {
         dline = gwy_spectra_get_spectrum(spectra, k-1);
-        data = gwy_data_line_get_data(dline);
+        data = gwy_line_get_data(dline);
         for (j = 0; j < n_points; j++)
             ydata[j] = data[j];
         cmodel = gwy_graph_curve_model_new();
