@@ -1,8 +1,11 @@
 /*
- *  $Id: robotics.c 26153 2024-02-05 12:28:41Z yeti-dn $
+ *  $Id: robotics.c 28789 2025-11-04 17:14:03Z yeti-dn $
  *  Copyright (C) 2011 Miroslav Valtr (Mira).
  *  E-mail: miraval@seznam.cz.
  *
+ *  Copyright (C) 2025 David Necas (Yeti).
+ *  E-mail: yeti@gwyddion.net.
+ *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
  *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
  *  later version.
@@ -46,14 +49,10 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/datafield.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/data-browser.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -64,19 +63,19 @@
 #define EXTENSION_MCE ".mce"
 #define NUM_DFIELDS 14
 
-static gboolean      module_register      (void);
-static gint          robotics_detect      (const GwyFileDetectInfo *fileinfo,
-                                           gboolean only_name);
-static GwyContainer* robotics_load        (const gchar *filename,
-                                           G_GNUC_UNUSED GwyRunType mode,
-                                           GError **error);
+static gboolean module_register(void);
+static gint     detect_file    (const GwyFileDetectInfo *fileinfo,
+                                gboolean only_name);
+static GwyFile* load_file      (const gchar *filename,
+                                GwyRunModeFlags mode,
+                                GError **error);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Automation & Robotics Dual Lensmapper data files"),
     "Mira <miraval@seznam.cz>",
-    "0.2",
+    "0.3",
     "Miroslav Valtr",
     "2011",
 };
@@ -88,17 +87,13 @@ module_register(void)
 {
     gwy_file_func_register("robotics",
                            N_("Dual Lensmapper files"),
-                           (GwyFileDetectFunc)&robotics_detect,
-                           (GwyFileLoadFunc)&robotics_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-robotics_detect(const GwyFileDetectInfo *fileinfo,
-                gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     if (only_name) {
         if (g_str_has_suffix(fileinfo->name_lowercase, EXTENSION_MCR)
@@ -115,16 +110,16 @@ robotics_detect(const GwyFileDetectInfo *fileinfo,
     return 50;
 }
 
-static GwyContainer*
-robotics_load(const gchar *filename,
-              G_GNUC_UNUSED GwyRunType mode,
-              GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
     static const gchar *channel_titles[NUM_DFIELDS] = {
         "PosX", "PosY", "Dpt", "Sph", "Cyl", "Axis", "Valid",
         "NormX", "NormY", "NormZ", "PosZ", "MinCurvX", "MinCurvY", "MinCurvZ",
     };
-    GwyContainer *container = NULL, *meta;
+    GwyFile *file = NULL;
     GError *err = NULL;
     gsize size = 0;
     gchar *line, *p, *buffer = NULL, *comment = NULL;
@@ -132,7 +127,7 @@ robotics_load(const gchar *filename,
     guint version, origin, i, j, xres, yres;
     gint valid;
     gchar **split_line = NULL;
-    GwyDataField **dfield = NULL;
+    GwyField **dfield = NULL;
     gdouble xreal, yreal, xoffset, yoffset;
     gdouble **data = NULL;
 
@@ -199,14 +194,14 @@ robotics_load(const gchar *filename,
     yoffset = 0.001*g_ascii_strtod(split_line[1], NULL);
     valid = g_ascii_strtod(split_line[6], NULL);
 
-    dfield = g_new0(GwyDataField*, NUM_DFIELDS);
+    dfield = g_new0(GwyField*, NUM_DFIELDS);
     data = g_new0(gdouble*, NUM_DFIELDS);
     for (i = 0; i < NUM_DFIELDS; i++) {
-         dfield[i] = gwy_data_field_new(xres, yres, xreal, yreal, TRUE);
-         data[i] = gwy_data_field_get_data(dfield[i]);
-         gwy_data_field_set_xoffset(dfield[i], xoffset);
-         gwy_data_field_set_yoffset(dfield[i], yoffset);
-         gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield[i]), "m");
+         dfield[i] = gwy_field_new(xres, yres, xreal, yreal, TRUE);
+         data[i] = gwy_field_get_data(dfield[i]);
+         gwy_field_set_xoffset(dfield[i], xoffset);
+         gwy_field_set_yoffset(dfield[i], yoffset);
+         gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield[i]), "m");
     }
 
     if (valid) {
@@ -235,14 +230,14 @@ robotics_load(const gchar *filename,
         g_strfreev(split_line);
     }
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     for (i = 0; i < NUM_DFIELDS; i++) {
-        gwy_container_pass_object(container, gwy_app_get_data_key_for_id(i), dfield[i]);
-        gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(i), channel_titles[i]);
-        gwy_file_channel_import_log_add(container, i, NULL, filename);
+        gwy_file_pass_image(file, i, dfield[i]);
+        gwy_file_set_title(file, GWY_FILE_IMAGE, i, channel_titles[i], FALSE);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, i, NULL, filename);
     }
 
-    meta = gwy_container_new();
+    GwyContainer *meta = gwy_container_new_in_construction();
 
     //gwy_container_set_string_by_name(meta, "File version:", g_strdup_printf("%d", version));
     gwy_container_set_string_by_name(meta, "Comment:", comment);
@@ -255,15 +250,17 @@ robotics_load(const gchar *filename,
     gwy_container_set_string_by_name(meta, "Size (x,y in mm):",
                                      g_strdup_printf("%.3lf,%.3lf", xreal*1000.0, yreal*1000.0));
 
-    /* XXX: What about the other channels? */
-    gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(0), meta);
+    for (i = 0; i < NUM_DFIELDS; i++)
+        gwy_file_pass_meta(file, GWY_FILE_IMAGE, i, gwy_container_copy(meta));
+
+    g_clear_object(&meta);
 
 fail:
     g_free(buffer);
     g_free(dfield);
     g_free(data);
 
-    return container;
+    return file;
 }
 
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
