@@ -1,6 +1,6 @@
 /*
- *  $Id: s94file.c 26180 2024-02-15 16:06:36Z yeti-dn $
- *  Copyright (C) 2013 David Necas (Yeti).
+ *  $Id: s94file.c 28848 2025-11-10 18:54:08Z yeti-dn $
+ *  Copyright (C) 2013-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -32,16 +32,11 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/stats.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -84,23 +79,23 @@ typedef struct {
 } S94File;
 
 static gboolean      module_register(void);
-static gint          s94_detect     (const GwyFileDetectInfo *fileinfo,
+static gint          detect_file    (const GwyFileDetectInfo *fileinfo,
                                      gboolean only_name);
-static GwyContainer* s94_load       (const gchar *filename,
-                                     GwyRunType mode,
+static GwyFile*      load_file      (const gchar *filename,
+                                     GwyRunModeFlags mode,
                                      GError **error);
-static gboolean      s94_read_header(S94File *s94file,
+static gboolean      read_header    (S94File *s94file,
                                      const guchar *p,
                                      gsize size,
                                      GError **error);
-static GwyContainer* s94_get_meta   (const S94File *s94file);
+static GwyContainer* get_meta       (const S94File *s94file);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports S94 STM data files."),
     "Yeti <yeti@gwyddion.net>",
-    "1.1",
+    "2.0",
     "David Nečas (Yeti)",
     "2013",
 };
@@ -112,17 +107,13 @@ module_register(void)
 {
     gwy_file_func_register("s94file",
                            N_("S94 STM files (.s94)"),
-                           (GwyFileDetectFunc)&s94_detect,
-                           (GwyFileLoadFunc)&s94_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-s94_detect(const GwyFileDetectInfo *fileinfo,
-           gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     guint xres, yres, swap, zgain, section;
 
@@ -146,73 +137,70 @@ s94_detect(const GwyFileDetectInfo *fileinfo,
     return 0;
 }
 
-static GwyContainer*
-s94_load(const gchar *filename,
-         G_GNUC_UNUSED GwyRunType mode,
-         GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *container = NULL, *meta;
+    GwyFile *file = NULL;
     guchar *buffer = NULL;
     const gchar *title, *unit;
     gsize size = 0;
     GError *err = NULL;
     S94File s94file;
-    GwyDataField *dfield;
+    GwyField *dfield;
 
     if (!gwy_file_get_contents(filename, &buffer, &size, &err)) {
         err_GET_FILE_CONTENTS(error, &err);
         return NULL;
     }
-    if (!s94_read_header(&s94file, buffer, size, error))
+    if (!read_header(&s94file, buffer, size, error))
         goto fail;
 
-    dfield = gwy_data_field_new(s94file.xres, s94file.yres,
-                                s94file.xreal * Nanometre, s94file.yreal * Nanometre,
-                                FALSE);
+    dfield = gwy_field_new(s94file.xres, s94file.yres,
+                           s94file.xreal * Nanometre, s94file.yreal * Nanometre,
+                           FALSE);
     gwy_convert_raw_data(buffer + HEADER_LEN, s94file.xres*s94file.yres,
                          1, GWY_RAW_DATA_SINT16, GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                         dfield->data, s94file.q, 0.0);
-    gwy_data_field_set_xoffset(dfield, s94file.xoff * Nanometre);
-    gwy_data_field_set_yoffset(dfield, s94file.yoff * Nanometre);
+                         gwy_field_get_data(dfield), s94file.q, 0.0);
+    gwy_field_set_xoffset(dfield, s94file.xoff * Nanometre);
+    gwy_field_set_yoffset(dfield, s94file.yoff * Nanometre);
 
     if (s94file.swapxy) {
-        gwy_data_field_rotate(dfield, 90, GWY_INTERPOLATION_ROUND);
-        gwy_data_field_invert(dfield, FALSE, TRUE, FALSE);
+        gwy_field_rotate(dfield, 90, GWY_INTERPOLATION_ROUND);
+        gwy_field_flip(dfield, TRUE, FALSE);
     }
 
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
     unit = gwy_enuml_to_string(s94file.mode,
                                "m", S94_TOPOGRAPHY,
                                "A", S94_CURRENT,
                                NULL);
     /* Keep the data unitless otherwise. */
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), unit);
+    gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), unit);
 
-    container = gwy_container_new();
-    gwy_container_pass_object(container, gwy_app_get_data_key_for_id(0), dfield);
+    file = gwy_file_new_in_construction();
+    gwy_file_pass_image(file, 0, dfield);
 
     title = gwy_enuml_to_string(s94file.mode,
                                 "Topography", S94_TOPOGRAPHY,
                                 "Current", S94_CURRENT,
                                 NULL);
-    gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(0), title ? title : "Unknown");
-
-    meta = s94_get_meta(&s94file);
-    gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(0), meta);
-
-    gwy_file_channel_import_log_add(container, 0, NULL, filename);
+    gwy_file_set_title(file, GWY_FILE_IMAGE, 0, title ? title : "Unknown", FALSE);
+    gwy_file_pass_meta(file, GWY_FILE_IMAGE, 0, get_meta(&s94file));
+    gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
 
 fail:
     gwy_file_abandon_contents(buffer, size, NULL);
 
-    return container;
+    return file;
 }
 
 static gboolean
-s94_read_header(S94File *s94file,
-                const guchar *p,
-                gsize size,
-                GError **error)
+read_header(S94File *s94file,
+            const guchar *p,
+            gsize size,
+            GError **error)
 {
     if (size <= HEADER_LEN) {
         err_TOO_SHORT(error);
@@ -262,9 +250,9 @@ s94_read_header(S94File *s94file,
 }
 
 static GwyContainer*
-s94_get_meta(const S94File *s94file)
+get_meta(const S94File *s94file)
 {
-    GwyContainer *meta = gwy_container_new();
+    GwyContainer *meta = gwy_container_new_in_construction();
 
     gwy_container_set_string_by_name(meta, "Image number (original)", g_strdup_printf("%u", s94file->orig_imageno));
     gwy_container_set_string_by_name(meta, "Image mode", g_strdup_printf("%u", s94file->mode));
