@@ -1,6 +1,6 @@
 /*
- *  $Id: scanfile.c 26873 2024-11-19 12:52:14Z yeti-dn $
- *  Copyright (C) 2024 David Necas (Yeti).
+ *  $Id: scanfile.c 28789 2025-11-04 17:14:03Z yeti-dn $
+ *  Copyright (C) 2024-2025 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
  *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
@@ -34,15 +34,11 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 #include "err.h"
 
 #define SCAN_MAGIC "<scandocument version="
@@ -62,22 +58,22 @@ typedef struct {
     guchar *data;
 } SCANFile;
 
-static gboolean      module_register(void);
-static gint          scan_detect    (const GwyFileDetectInfo *fileinfo,
-                                     gboolean only_name);
-static GwyContainer* scan_load      (const gchar *filename,
-                                     GwyRunType mode,
-                                     GError **error);
-static gboolean      scan_parse_xml (SCANFile *scanfile,
-                                     const gchar *buffer,
-                                     GError **error);
+static gboolean module_register(void);
+static gint     detect_file    (const GwyFileDetectInfo *fileinfo,
+                                gboolean only_name);
+static GwyFile* load_file      (const gchar *filename,
+                                GwyRunModeFlags mode,
+                                GError **error);
+static gboolean parse_xml      (SCANFile *scanfile,
+                                const gchar *buffer,
+                                GError **error);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Reads Cyber technologies SCAN files."),
     "Yeti <yeti@gwyddion.net>",
-    "1.0",
+    "2.0",
     "David Nečas (Yeti)",
     "2024",
 };
@@ -89,17 +85,13 @@ module_register(void)
 {
     gwy_file_func_register("scanfile",
                            N_("Cyber technologies SCAN data (.scan)"),
-                           (GwyFileDetectFunc)&scan_detect,
-                           (GwyFileLoadFunc)&scan_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-scan_detect(const GwyFileDetectInfo *fileinfo,
-            gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     const gchar *tags[] = { "<scanfile", "<header", "<raster" };
     gint score = 0;
@@ -130,14 +122,14 @@ scan_detect(const GwyFileDetectInfo *fileinfo,
     return score;
 }
 
-static GwyContainer*
-scan_load(const gchar *filename,
-          G_GNUC_UNUSED GwyRunType mode,
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
           GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     SCANFile scanfile;
-    GwyDataField *field, *mask;
+    GwyField *field, *mask;
     GError *err = NULL;
     guchar *buffer = NULL;
     gsize size, expected_size;
@@ -150,12 +142,12 @@ scan_load(const gchar *filename,
         return NULL;
     }
 
-    gwy_clear(&scanfile, 1);
+    gwy_clear1(scanfile);
     scanfile.hash = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
     scanfile.path = g_string_new(NULL);
     scanfile.str = str = g_string_new(NULL);
 
-    if (!scan_parse_xml(&scanfile, buffer, error))
+    if (!parse_xml(&scanfile, buffer, error))
         goto fail;
 
     xres = atol(g_hash_table_lookup(scanfile.hash, RASTER_PREFIX "rows"));     /* sic */
@@ -168,18 +160,18 @@ scan_load(const gchar *filename,
 
     xreal = 1e-3*g_ascii_strtod(g_hash_table_lookup(scanfile.hash, RASTER_PREFIX "height"), NULL);
     yreal = 1e-3*g_ascii_strtod(g_hash_table_lookup(scanfile.hash, RASTER_PREFIX "width"), NULL);
-    field = gwy_data_field_new(xres, yres, xreal, yreal, FALSE);
+    field = gwy_field_new(xres, yres, xreal, yreal, FALSE);
     gwy_convert_raw_data(scanfile.data, xres*yres, 1, GWY_RAW_DATA_FLOAT, GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                         gwy_data_field_get_data(field), 1e-3, 0.0);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(field), "m");
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(field), "m");
+                         gwy_field_get_data(field), 1e-3, 0.0);
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(field), "m");
+    gwy_unit_set_from_string(gwy_field_get_unit_z(field), "m");
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     if ((mask = gwy_app_channel_mask_of_nans(field, TRUE)))
-        gwy_container_pass_object(container, gwy_app_get_mask_key_for_id(0), mask);
-    gwy_container_pass_object(container, gwy_app_get_data_key_for_id(0), field);
-    gwy_file_channel_import_log_add(container, 0, NULL, filename);
-    gwy_app_channel_title_fall_back(container, 0);
+        gwy_file_pass_image_mask(file, 0, mask);
+    gwy_file_pass_image(file, 0, field);
+    gwy_image_title_fall_back(file, 0);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
 
 fail:
     g_free(scanfile.data);
@@ -188,16 +180,16 @@ fail:
     g_string_free(scanfile.str, TRUE);
     gwy_file_abandon_contents(buffer, size, NULL);
 
-    return container;
+    return file;
 }
 
 static void
-scan_start_element(G_GNUC_UNUSED GMarkupParseContext *context,
-                   const gchar *element_name,
-                   G_GNUC_UNUSED const gchar **attribute_names,
-                   G_GNUC_UNUSED const gchar **attribute_values,
-                   gpointer user_data,
-                   G_GNUC_UNUSED GError **error)
+start_element(G_GNUC_UNUSED GMarkupParseContext *context,
+              const gchar *element_name,
+              G_GNUC_UNUSED const gchar **attribute_names,
+              G_GNUC_UNUSED const gchar **attribute_values,
+              gpointer user_data,
+              G_GNUC_UNUSED GError **error)
 {
     SCANFile *scanfile = (SCANFile*)user_data;
     GString *path = scanfile->path, *str = scanfile->str;
@@ -222,10 +214,10 @@ scan_start_element(G_GNUC_UNUSED GMarkupParseContext *context,
 }
 
 static void
-scan_end_element(G_GNUC_UNUSED GMarkupParseContext *context,
-                 const gchar *element_name,
-                 gpointer user_data,
-                 G_GNUC_UNUSED GError **error)
+end_element(G_GNUC_UNUSED GMarkupParseContext *context,
+            const gchar *element_name,
+            gpointer user_data,
+            G_GNUC_UNUSED GError **error)
 {
     SCANFile *scanfile = (SCANFile*)user_data;
     GString *path = scanfile->path;
@@ -241,17 +233,17 @@ scan_end_element(G_GNUC_UNUSED GMarkupParseContext *context,
 }
 
 static void
-scan_text(G_GNUC_UNUSED GMarkupParseContext *context,
-          const gchar *text,
-          G_GNUC_UNUSED gsize text_len,
-          gpointer user_data,
-          G_GNUC_UNUSED GError **error)
+text(G_GNUC_UNUSED GMarkupParseContext *context,
+     const gchar *value,
+     G_GNUC_UNUSED gsize value_len,
+     gpointer user_data,
+     G_GNUC_UNUSED GError **error)
 {
     SCANFile *scanfile = (SCANFile*)user_data;
     GString *path = scanfile->path, *str = scanfile->str;
 
     if (!gwy_strequal(path->str, "/scandocument/scanfile/datapoints")) {
-        g_string_assign(str, text);
+        g_string_assign(str, value);
         g_strstrip(str->str);
         if (*(str->str))
             g_warning("Text found unexpectedly in %s", path->str);
@@ -263,15 +255,15 @@ scan_text(G_GNUC_UNUSED GMarkupParseContext *context,
         g_warning("Multiple <datapoints>");
         return;
     }
-    scanfile->data = g_base64_decode(text, &scanfile->size);
+    scanfile->data = g_base64_decode(value, &scanfile->size);
     gwy_debug("found data (%lu bytes)", (gulong)scanfile->size);
 }
 
 static gboolean
-scan_parse_xml(SCANFile *scanfile, const gchar *buffer, GError **error)
+parse_xml(SCANFile *scanfile, const gchar *buffer, GError **error)
 {
     GMarkupParser parser = {
-        &scan_start_element, &scan_end_element, &scan_text, NULL, NULL,
+        &start_element, &end_element, &text, NULL, NULL,
     };
     GMarkupParseContext *context = NULL;
     gboolean ok = FALSE;
