@@ -1,6 +1,6 @@
 /*
- *  $Id: scnxfile.c 26873 2024-11-19 12:52:14Z yeti-dn $
- *  Copyright (C) 2024 David Necas (Yeti).
+ *  $Id: scnxfile.c 28789 2025-11-04 17:14:03Z yeti-dn $
+ *  Copyright (C) 2024-2025 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
  *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
@@ -31,17 +31,12 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <glib/gstdio.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/stats.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 
 #include "err.h"
 #include "gwyzip.h"
@@ -83,27 +78,27 @@ typedef struct {
     gint stop_here;
 } SCNXFile;
 
-static gboolean      module_register      (void);
-static gint          scnx_detect          (const GwyFileDetectInfo *fileinfo,
-                                           gboolean only_name);
-static GwyContainer* scnx_load            (const gchar *filename,
-                                           GwyRunType mode,
-                                           GError **error);
-static GHashTable*   scnx_parse_one_file  (GwyZipFile zipfile,
-                                           SCNXFile *scnxfile,
-                                           GError **error);
-static gboolean      scnx_read_binary_data(GwyZipFile zipfile,
-                                           const gchar *name,
-                                           SCNXData *datafile,
-                                           GError **error);
-static void          scnx_file_free       (SCNXFile *scnxfile);
+static gboolean    module_register (void);
+static gint        detect_file     (const GwyFileDetectInfo *fileinfo,
+                                    gboolean only_name);
+static GwyFile*    load_file       (const gchar *filename,
+                                    GwyRunModeFlags mode,
+                                    GError **error);
+static GHashTable* parse_one_file  (GwyZipFile zipfile,
+                                    SCNXFile *scnxfile,
+                                    GError **error);
+static gboolean    read_binary_data(GwyZipFile zipfile,
+                                    const gchar *name,
+                                    SCNXData *datafile,
+                                    GError **error);
+static void        free_scnxfile   (SCNXFile *scnxfile);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Reads Cyber technologies SCNX files."),
     "Yeti <yeti@gwyddion.net>",
-    "1.1",
+    "2.0",
     "David Nečas (Yeti)",
     "2024",
 };
@@ -115,17 +110,13 @@ module_register(void)
 {
     gwy_file_func_register("scnxfile",
                            N_("Cyber technologies SCNX data (.scnx)"),
-                           (GwyFileDetectFunc)&scnx_detect,
-                           (GwyFileLoadFunc)&scnx_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-scnx_detect(const GwyFileDetectInfo *fileinfo,
-            gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     GwyZipFile zipfile;
     gchar *content;
@@ -160,15 +151,15 @@ scnx_detect(const GwyFileDetectInfo *fileinfo,
     return score;
 }
 
-static GwyContainer*
-scnx_load(const gchar *filename,
-          G_GNUC_UNUSED GwyRunType mode,
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
           GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     SCNXFile scnxfile;
     GwyZipFile zipfile = NULL;
-    GwyDataField *field, *mask;
+    GwyField *field, *mask;
     GString *str;
     guint i;
 
@@ -179,7 +170,7 @@ scnx_load(const gchar *filename,
     if (!(zipfile = gwyzip_open(filename, NULL)))
         return NULL;
 
-    gwy_clear(&scnxfile, 1);
+    gwy_clear1(scnxfile);
 
     //scnxfile.hash = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
     scnxfile.datafiles = g_array_new(FALSE, FALSE, sizeof(SCNXData));
@@ -190,13 +181,13 @@ scnx_load(const gchar *filename,
     while (TRUE) {
         SCNXData datafile;
 
-        gwy_clear(&datafile, 1);
+        gwy_clear1(datafile);
         g_string_printf(str, "scan/data/scaninfo%d.xml", i);
         if (!gwyzip_locate_file(zipfile, str->str, FALSE, NULL))
             break;
 
         gwy_debug("found XML file: <%s>", str->str);
-        if (!(datafile.hash = scnx_parse_one_file(zipfile, &scnxfile, error))) {
+        if (!(datafile.hash = parse_one_file(zipfile, &scnxfile, error))) {
             g_hash_table_destroy(datafile.hash);
             goto fail;
         }
@@ -212,7 +203,7 @@ scnx_load(const gchar *filename,
         }
         gwy_debug("found binary file: <%s>", str->str);
 
-        if (!scnx_read_binary_data(zipfile, str->str, &datafile, error)) {
+        if (!read_binary_data(zipfile, str->str, &datafile, error)) {
             g_hash_table_destroy(datafile.hash);
             goto fail;
         }
@@ -225,7 +216,7 @@ scnx_load(const gchar *filename,
         goto fail;
     }
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     for (i = 0; i < scnxfile.datafiles->len; i++) {
         SCNXData *datafile = &g_array_index(scnxfile.datafiles, SCNXData, i);
         gint xres = datafile->xres, yres = datafile->yres;
@@ -236,34 +227,34 @@ scnx_load(const gchar *filename,
         dy = g_ascii_strtod(g_hash_table_lookup(datafile->hash, "/scaninfo/stepsize/Y"), NULL);
         sanitise_real_size(&dy, "y step");
 
-        field = gwy_data_field_new(xres, yres, dx*xres*1e-3, dy*yres*1e-3, FALSE);
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(field), "m");
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(field), "m");
+        field = gwy_field_new(xres, yres, dx*xres*1e-3, dy*yres*1e-3, FALSE);
+        gwy_unit_set_from_string(gwy_field_get_unit_xy(field), "m");
+        gwy_unit_set_from_string(gwy_field_get_unit_z(field), "m");
         gwy_convert_raw_data(datafile->data + SCNX_BIN_HEADER_SIZE, xres*yres, 1,
                              GWY_RAW_DATA_DOUBLE, GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                             gwy_data_field_get_data(field), 1.0, 0.0);
+                             gwy_field_get_data(field), 1.0, 0.0);
         if ((mask = gwy_app_channel_mask_of_nans(field, TRUE)))
-            gwy_container_pass_object(container, gwy_app_get_mask_key_for_id(i), mask);
-        gwy_container_pass_object(container, gwy_app_get_data_key_for_id(i), field);
-        gwy_file_channel_import_log_add(container, i, NULL, filename);
-        gwy_app_channel_title_fall_back(container, i);
+            gwy_file_pass_image_mask(file, i, mask);
+        gwy_file_pass_image(file, i, field);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, i, NULL, filename);
+        gwy_image_title_fall_back(file, i);
     }
 
 fail:
     if (zipfile)
         gwyzip_close(zipfile);
-    scnx_file_free(&scnxfile);
+    free_scnxfile(&scnxfile);
 
-    return container;
+    return file;
 }
 
 static void
-scnx_start_element(G_GNUC_UNUSED GMarkupParseContext *context,
-                   const gchar *element_name,
-                   G_GNUC_UNUSED const gchar **attribute_names,
-                   G_GNUC_UNUSED const gchar **attribute_values,
-                   gpointer user_data,
-                   G_GNUC_UNUSED GError **error)
+start_element(G_GNUC_UNUSED GMarkupParseContext *context,
+              const gchar *element_name,
+              G_GNUC_UNUSED const gchar **attribute_names,
+              G_GNUC_UNUSED const gchar **attribute_values,
+              gpointer user_data,
+              G_GNUC_UNUSED GError **error)
 {
     SCNXFile *scnxfile = (SCNXFile*)user_data;
     const gchar *component_name = element_name;
@@ -309,10 +300,10 @@ scnx_start_element(G_GNUC_UNUSED GMarkupParseContext *context,
 }
 
 static void
-scnx_end_element(G_GNUC_UNUSED GMarkupParseContext *context,
-                 const gchar *element_name,
-                 gpointer user_data,
-                 G_GNUC_UNUSED GError **error)
+end_element(G_GNUC_UNUSED GMarkupParseContext *context,
+            const gchar *element_name,
+            gpointer user_data,
+            G_GNUC_UNUSED GError **error)
 {
     SCNXFile *scnxfile = (SCNXFile*)user_data;
     GString *path = scnxfile->path;
@@ -342,20 +333,20 @@ scnx_end_element(G_GNUC_UNUSED GMarkupParseContext *context,
 }
 
 static void
-scnx_text(G_GNUC_UNUSED GMarkupParseContext *context,
-          const gchar *text,
-          G_GNUC_UNUSED gsize text_len,
-          gpointer user_data,
-          G_GNUC_UNUSED GError **error)
+text(G_GNUC_UNUSED GMarkupParseContext *context,
+     const gchar *value,
+     G_GNUC_UNUSED gsize value_len,
+     gpointer user_data,
+     G_GNUC_UNUSED GError **error)
 {
     SCNXFile *scnxfile = (SCNXFile*)user_data;
     GString *path = scnxfile->path, *str = scnxfile->str;
     gchar *key;
 
-    if (!*text)
+    if (!*value)
         return;
 
-    g_string_assign(str, text);
+    g_string_assign(str, value);
     g_strstrip(str->str);
     if (*(str->str))
         return;
@@ -366,10 +357,10 @@ scnx_text(G_GNUC_UNUSED GMarkupParseContext *context,
 }
 
 static GHashTable*
-scnx_parse_one_file(GwyZipFile zipfile, SCNXFile *scnxfile, GError **error)
+parse_one_file(GwyZipFile zipfile, SCNXFile *scnxfile, GError **error)
 {
     GMarkupParser parser = {
-        &scnx_start_element, &scnx_end_element, &scnx_text, NULL, NULL,
+        &start_element, &end_element, &text, NULL, NULL,
     };
     GMarkupParseContext *context = NULL;
     GHashTable *hash = NULL;
@@ -409,8 +400,8 @@ fail:
 }
 
 static gboolean
-scnx_read_binary_data(GwyZipFile zipfile, const gchar *name,
-                      SCNXData *datafile, GError **error)
+read_binary_data(GwyZipFile zipfile, const gchar *name,
+                 SCNXData *datafile, GError **error)
 {
     guchar *content = NULL;
     const guchar *p;
@@ -464,7 +455,7 @@ scnx_read_binary_data(GwyZipFile zipfile, const gchar *name,
 }
 
 static void
-scnx_file_free(SCNXFile *scnxfile)
+free_scnxfile(SCNXFile *scnxfile)
 {
     guint i;
 
