@@ -1,6 +1,6 @@
 /*
- *  $Id: sdfile.c 26162 2024-02-06 14:58:50Z yeti-dn $
- *  Copyright (C) 2005-2019 David Necas (Yeti), Petr Klapetek.
+ *  $Id: sdfile.c 28822 2025-11-06 16:01:09Z yeti-dn $
+ *  Copyright (C) 2005-2025 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -48,16 +48,11 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
 #include <time.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/grains.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 
 #include "err.h"
 #include "get.h"
@@ -109,51 +104,51 @@ typedef struct {
     gint expected_size;
 } SDFile;
 
-static gboolean      module_register        (void);
-static gint          sdfile_detect_bin      (const GwyFileDetectInfo *fileinfo,
-                                             gboolean only_name);
-static gint          sdfile_detect_text     (const GwyFileDetectInfo *fileinfo,
-                                             gboolean only_name);
-static gint          micromap_detect        (const GwyFileDetectInfo *fileinfo,
-                                             gboolean only_name);
-static GwyContainer* sdfile_load_bin        (const gchar *filename,
-                                             GwyRunType mode,
-                                             GError **error);
-static GwyContainer* sdfile_load_text       (const gchar *filename,
-                                             GwyRunType mode,
-                                             GError **error);
-static gboolean      sdfile_export_text     (GwyContainer *data,
-                                             const gchar *filename,
-                                             GwyRunType mode,
-                                             GError **error);
-static GwyContainer* micromap_load          (const gchar *filename,
-                                             GwyRunType mode,
-                                             GError **error);
-static gboolean      check_params           (const SDFile *sdfile,
-                                             guint len,
-                                             GError **error);
-static gboolean      sdfile_read_header_bin (const guchar **p,
-                                             gsize *len,
-                                             SDFile *sdfile,
-                                             GError **error);
-static gboolean      sdfile_read_header_text(gchar **buffer,
-                                             gsize *len,
-                                             SDFile *sdfile,
-                                             GError **error);
-static gchar*        sdfile_next_line       (gchar **buffer,
-                                             const gchar *key,
-                                             GError **error);
-static GwyDataField* sdfile_read_data_bin   (SDFile *sdfile,
-                                             GwyDataField **mfield);
-static GwyDataField* sdfile_read_data_text  (SDFile *sdfile,
-                                             GError **error);
+static gboolean  module_register   (void);
+static gint      sdfile_detect_bin (const GwyFileDetectInfo *fileinfo,
+                                    gboolean only_name);
+static gint      sdfile_detect_text(const GwyFileDetectInfo *fileinfo,
+                                    gboolean only_name);
+static gint      micromap_detect   (const GwyFileDetectInfo *fileinfo,
+                                    gboolean only_name);
+static GwyFile*  sdfile_load_bin   (const gchar *filename,
+                                    GwyRunModeFlags mode,
+                                    GError **error);
+static GwyFile*  sdfile_load_text  (const gchar *filename,
+                                    GwyRunModeFlags mode,
+                                    GError **error);
+static gboolean  sdfile_export_text(GwyFile *data,
+                                    const gchar *filename,
+                                    GwyRunModeFlags mode,
+                                    GError **error);
+static GwyFile*  micromap_load     (const gchar *filename,
+                                    GwyRunModeFlags mode,
+                                    GError **error);
+static gboolean  check_params      (const SDFile *sdfile,
+                                    guint len,
+                                    GError **error);
+static gboolean  read_header_bin   (const guchar **p,
+                                    gsize *len,
+                                    SDFile *sdfile,
+                                    GError **error);
+static gboolean  read_header_text  (gchar **buffer,
+                                    gsize *len,
+                                    SDFile *sdfile,
+                                    GError **error);
+static gchar*    sdfile_next_line  (gchar **buffer,
+                                    const gchar *key,
+                                    GError **error);
+static GwyField* read_data_bin     (SDFile *sdfile,
+                                    GwyField **mfield);
+static GwyField* read_data_text    (SDFile *sdfile,
+                                    GError **error);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Surfstand group SDF (Surface Data File) files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.14",
+    "0.15",
     "David Nečas (Yeti) & Petr Klapetek",
     "2005",
 };
@@ -167,29 +162,19 @@ module_register(void)
 {
     gwy_file_func_register("sdfile-bin",
                            N_("Surfstand SDF files, binary (.sdf)"),
-                           (GwyFileDetectFunc)&sdfile_detect_bin,
-                           (GwyFileLoadFunc)&sdfile_load_bin,
-                           NULL,
-                           NULL);
+                           &sdfile_detect_bin, &sdfile_load_bin, NULL, NULL);
     gwy_file_func_register("sdfile-txt",
                            N_("Surfstand SDF files, text (.sdf)"),
-                           (GwyFileDetectFunc)&sdfile_detect_text,
-                           (GwyFileLoadFunc)&sdfile_load_text,
-                           NULL,
-                           (GwyFileSaveFunc)&sdfile_export_text);
+                           &sdfile_detect_text, &sdfile_load_text, NULL, &sdfile_export_text);
     gwy_file_func_register("micromap",
                            N_("Micromap SDF files (.sdfa)"),
-                           (GwyFileDetectFunc)&micromap_detect,
-                           (GwyFileLoadFunc)&micromap_load,
-                           NULL,
-                           NULL);
+                           &micromap_detect, &micromap_load, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-sdfile_detect_bin(const GwyFileDetectInfo *fileinfo,
-                  gboolean only_name)
+sdfile_detect_bin(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     SDFile sdfile;
     const gchar *p;
@@ -202,7 +187,7 @@ sdfile_detect_bin(const GwyFileDetectInfo *fileinfo,
     len = fileinfo->buffer_len;
     if (len <= SDF_HEADER_SIZE_BIN || fileinfo->head[0] != 'b')
         return 0;
-    if (sdfile_read_header_bin((const guchar**)&p, &len, &sdfile, NULL)
+    if (read_header_bin((const guchar**)&p, &len, &sdfile, NULL)
         && SDF_HEADER_SIZE_BIN + sdfile.expected_size <= fileinfo->file_size
         && !sdfile.compression
         && !sdfile.check_type)
@@ -212,8 +197,7 @@ sdfile_detect_bin(const GwyFileDetectInfo *fileinfo,
 }
 
 static gint
-sdfile_detect_text(const GwyFileDetectInfo *fileinfo,
-                   gboolean only_name)
+sdfile_detect_text(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     SDFile sdfile;
     gchar *buffer, *p;
@@ -228,7 +212,7 @@ sdfile_detect_text(const GwyFileDetectInfo *fileinfo,
         return 0;
 
     buffer = p = g_memdup(fileinfo->head, len);
-    if (sdfile_read_header_text(&p, &len, &sdfile, NULL)
+    if (read_header_text(&p, &len, &sdfile, NULL)
         && sdfile.expected_size <= fileinfo->file_size
         && !sdfile.compression
         && !sdfile.check_type)
@@ -241,8 +225,7 @@ sdfile_detect_text(const GwyFileDetectInfo *fileinfo,
 
 /* Perform generic SDF detection, then check for Micromap specific stuff */
 static gint
-micromap_detect(const GwyFileDetectInfo *fileinfo,
-                gboolean only_name)
+micromap_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     SDFile sdfile;
     gchar *buffer, *p;
@@ -257,7 +240,7 @@ micromap_detect(const GwyFileDetectInfo *fileinfo,
         return 0;
 
     buffer = p = g_memdup(fileinfo->head, len);
-    if (sdfile_read_header_text(&p, &len, &sdfile, NULL)
+    if (read_header_text(&p, &len, &sdfile, NULL)
         && sdfile.expected_size <= fileinfo->file_size
         && !sdfile.compression
         && !sdfile.check_type
@@ -275,25 +258,25 @@ micromap_detect(const GwyFileDetectInfo *fileinfo,
 
 static void
 sdfile_set_units(SDFile *sdfile,
-                 GwyDataField *dfield)
+                 GwyField *dfield)
 {
-    gwy_data_field_multiply(dfield, sdfile->zscale);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), "m");
+    gwy_field_multiply(dfield, sdfile->zscale);
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
+    gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), "m");
 }
 
-static GwyContainer*
+static GwyFile*
 sdfile_load_bin(const gchar *filename,
-                G_GNUC_UNUSED GwyRunType mode,
+                G_GNUC_UNUSED GwyRunModeFlags mode,
                 GError **error)
 {
     SDFile sdfile;
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     guchar *buffer = NULL;
     const guchar *p;
     gsize len, size = 0;
     GError *err = NULL;
-    GwyDataField *dfield = NULL, *mfield = NULL;
+    GwyField *dfield = NULL, *mfield = NULL;
 
     if (!gwy_file_get_contents(filename, &buffer, &size, &err)) {
         err_GET_FILE_CONTENTS(error, &err);
@@ -302,9 +285,9 @@ sdfile_load_bin(const gchar *filename,
 
     len = size;
     p = buffer;
-    if (sdfile_read_header_bin(&p, &len, &sdfile, error)) {
+    if (read_header_bin(&p, &len, &sdfile, error)) {
         if (check_params(&sdfile, len, error))
-            dfield = sdfile_read_data_bin(&sdfile, &mfield);
+            dfield = read_data_bin(&sdfile, &mfield);
     }
 
     gwy_file_abandon_contents(buffer, size, NULL);
@@ -313,20 +296,18 @@ sdfile_load_bin(const gchar *filename,
 
     sdfile_set_units(&sdfile, dfield);
 
-    container = gwy_container_new();
-    gwy_container_pass_object(container, gwy_app_get_data_key_for_id(0), dfield);
+    file = gwy_file_new_in_construction();
+    gwy_file_pass_image(file, 0, dfield);
     if (mfield)
-        gwy_container_pass_object(container, gwy_app_get_mask_key_for_id(0), mfield);
-    gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(0), "Topography");
-    gwy_file_channel_import_log_add(container, 0, NULL, filename);
+        gwy_file_pass_image_mask(file, 0, mfield);
+    gwy_file_set_title(file, GWY_FILE_IMAGE, 0, "Topography", FALSE);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
 
-    return container;
+    return file;
 }
 
 static void
-store_meta(gpointer key,
-           gpointer value,
-           gpointer user_data)
+store_meta(gpointer key, gpointer value, gpointer user_data)
 {
     GwyContainer *meta = (GwyContainer*)user_data;
     gchar *cval;
@@ -336,17 +317,17 @@ store_meta(gpointer key,
     gwy_container_set_string_by_name(meta, key, cval);
 }
 
-static GwyContainer*
+static GwyFile*
 sdfile_load_text(const gchar *filename,
-                 G_GNUC_UNUSED GwyRunType mode,
+                 G_GNUC_UNUSED GwyRunModeFlags mode,
                  GError **error)
 {
     SDFile sdfile;
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     gchar *p, *buffer = NULL;
     gsize len, size = 0;
     GError *err = NULL;
-    GwyDataField *dfield = NULL;
+    GwyField *dfield = NULL;
 
     if (!g_file_get_contents(filename, &buffer, &size, &err)) {
         err_GET_FILE_CONTENTS(error, &err);
@@ -354,9 +335,9 @@ sdfile_load_text(const gchar *filename,
     }
     len = size;
     p = buffer;
-    if (sdfile_read_header_text(&p, &len, &sdfile, error)) {
+    if (read_header_text(&p, &len, &sdfile, error)) {
         if (check_params(&sdfile, len, error))
-            dfield = sdfile_read_data_text(&sdfile, error);
+            dfield = read_data_text(&sdfile, error);
     }
 
     if (!dfield) {
@@ -366,27 +347,27 @@ sdfile_load_text(const gchar *filename,
 
     sdfile_set_units(&sdfile, dfield);
 
-    container = gwy_container_new();
-    gwy_container_pass_object(container, gwy_app_get_data_key_for_id(0), dfield);
-    gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(0), "Topography");
+    file = gwy_file_new_in_construction();
+    gwy_file_pass_image(file, 0, dfield);
+    gwy_file_set_title(file, GWY_FILE_IMAGE, 0, "Topography", FALSE);
 
     if (sdfile.extras) {
-        GwyContainer *meta = gwy_container_new();
+        GwyContainer *meta = gwy_container_new_in_construction();
         g_hash_table_foreach(sdfile.extras, store_meta, meta);
-        gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(0), meta);
+        gwy_file_pass_meta(file, GWY_FILE_IMAGE, 0, meta);
         g_hash_table_destroy(sdfile.extras);
     }
     g_free(buffer);
 
-    gwy_file_channel_import_log_add(container, 0, NULL, filename);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
 
-    return container;
+    return file;
 }
 
 static gboolean
-sdfile_export_text(G_GNUC_UNUSED GwyContainer *data,
+sdfile_export_text(G_GNUC_UNUSED GwyFile *data,
                    const gchar *filename,
-                   G_GNUC_UNUSED GwyRunType mode,
+                   G_GNUC_UNUSED GwyRunModeFlags mode,
                    GError **error)
 {
     enum { SCALE = 65535 };
@@ -408,7 +389,7 @@ sdfile_export_text(G_GNUC_UNUSED GwyContainer *data,
         "NanPresent  = 0\n"
         "*\n";
 
-    GwyDataField *dfield;
+    GwyField *dfield;
     const gdouble *d;
     gint xres, yres, i;
     const struct tm *ttm;
@@ -416,7 +397,7 @@ sdfile_export_text(G_GNUC_UNUSED GwyContainer *data,
     time_t t;
     FILE *fh;
 
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &dfield, 0);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &dfield, 0);
     if (!dfield) {
         err_NO_CHANNEL_EXPORT(error);
         return FALSE;
@@ -427,9 +408,9 @@ sdfile_export_text(G_GNUC_UNUSED GwyContainer *data,
         return FALSE;
     }
 
-    d = gwy_data_field_get_data_const(dfield);
-    xres = gwy_data_field_get_xres(dfield);
-    yres = gwy_data_field_get_yres(dfield);
+    d = gwy_field_get_data_const(dfield);
+    xres = gwy_field_get_xres(dfield);
+    yres = gwy_field_get_yres(dfield);
 
     /* We do not keep date information, just use the current time */
     time(&t);
@@ -438,8 +419,8 @@ sdfile_export_text(G_GNUC_UNUSED GwyContainer *data,
                 ttm->tm_mday, ttm->tm_mon, ttm->tm_year, ttm->tm_hour, ttm->tm_min,
                 ttm->tm_mday, ttm->tm_mon, ttm->tm_year, ttm->tm_hour, ttm->tm_min,
                 xres, yres,
-                gwy_data_field_get_dx(dfield),
-                gwy_data_field_get_dy(dfield),
+                gwy_field_get_dx(dfield),
+                gwy_field_get_dy(dfield),
                 1.0,
                 SDF_FLOAT);
     for (i = 0; i < xres*yres; i++) {
@@ -452,17 +433,17 @@ sdfile_export_text(G_GNUC_UNUSED GwyContainer *data,
     return TRUE;
 }
 
-static GwyContainer*
+static GwyFile*
 micromap_load(const gchar *filename,
-              G_GNUC_UNUSED GwyRunType mode,
+              G_GNUC_UNUSED GwyRunModeFlags mode,
               GError **error)
 {
     SDFile sdfile;
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     gchar *p, *buffer = NULL;
     gsize len, size = 0;
     GError *err = NULL;
-    GwyDataField *dfield = NULL;
+    GwyField *dfield = NULL;
     gdouble objectivemag, tubemag, cameraxpixel, cameraypixel;
 
     if (!g_file_get_contents(filename, &buffer, &size, &err)) {
@@ -471,9 +452,9 @@ micromap_load(const gchar *filename,
     }
     len = size;
     p = buffer;
-    if (sdfile_read_header_text(&p, &len, &sdfile, error)) {
+    if (read_header_text(&p, &len, &sdfile, error)) {
         if (check_params(&sdfile, len, error))
-            dfield = sdfile_read_data_text(&sdfile, error);
+            dfield = read_data_text(&sdfile, error);
     }
     if (!dfield) {
         g_free(buffer);
@@ -496,13 +477,13 @@ micromap_load(const gchar *filename,
     cameraypixel = g_ascii_strtod(g_hash_table_lookup(sdfile.extras, "CAMERAYPIXEL"), NULL);
 
     sdfile_set_units(&sdfile, dfield);
-    gwy_data_field_set_xreal(dfield, Micrometer * sdfile.xres * objectivemag * tubemag * cameraxpixel);
-    gwy_data_field_set_yreal(dfield, Micrometer * sdfile.yres * objectivemag * tubemag * cameraypixel);
+    gwy_field_set_xreal(dfield, Micrometer * sdfile.xres * objectivemag * tubemag * cameraxpixel);
+    gwy_field_set_yreal(dfield, Micrometer * sdfile.yres * objectivemag * tubemag * cameraypixel);
 
-    container = gwy_container_new();
-    gwy_container_set_object(container, gwy_app_get_data_key_for_id(0), dfield);
-    gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(0), "Topography");
-    gwy_file_channel_import_log_add(container, 0, NULL, filename);
+    file = gwy_file_new_in_construction();
+    gwy_file_set_image(file, 0, dfield);
+    gwy_file_set_title(file, GWY_FILE_IMAGE, 0, "Topography", FALSE);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
 
 fail:
     g_object_unref(dfield);
@@ -510,7 +491,7 @@ fail:
     if (sdfile.extras)
         g_hash_table_destroy(sdfile.extras);
 
-    return container;
+    return file;
 }
 
 static gboolean
@@ -539,10 +520,10 @@ check_params(const SDFile *sdfile,
 }
 
 static gboolean
-sdfile_read_header_bin(const guchar **p,
-                       gsize *len,
-                       SDFile *sdfile,
-                       GError **error)
+read_header_bin(const guchar **p,
+                gsize *len,
+                SDFile *sdfile,
+                GError **error)
 {
     const guchar *buf = *p;
 
@@ -631,10 +612,10 @@ sdfile_read_header_bin(const guchar **p,
 
 /* NB: Buffer must be writable and nul-terminated, its initial part is overwritten */
 static gboolean
-sdfile_read_header_text(gchar **buffer,
-                        gsize *len,
-                        SDFile *sdfile,
-                        GError **error)
+read_header_text(gchar **buffer,
+                 gsize *len,
+                 SDFile *sdfile,
+                 GError **error)
 {
     gchar *val, *p;
 
@@ -728,8 +709,8 @@ sdfile_next_line(gchar **buffer,
     return value;
 }
 
-static GwyDataField*
-sdfile_read_data_bin(SDFile *sdfile, GwyDataField **mfield)
+static GwyField*
+read_data_bin(SDFile *sdfile, GwyField **mfield)
 {
     static const struct {
         SDFDataType sdftype;
@@ -747,37 +728,36 @@ sdfile_read_data_bin(SDFile *sdfile, GwyDataField **mfield)
         { SDF_DOUBLE, GWY_RAW_DATA_DOUBLE, NAN,         },
     };
 
-    GwyDataField *dfield, *mask = NULL;
+    GwyField *dfield, *mask = NULL;
     gdouble *data, *m;
     guint i, n, idt;
 
-    dfield = gwy_data_field_new(sdfile->xres, sdfile->yres,
-                                sdfile->xres * sdfile->xscale, sdfile->yres * sdfile->yscale,
-                                FALSE);
+    dfield = gwy_field_new(sdfile->xres, sdfile->yres,
+                           sdfile->xres * sdfile->xscale, sdfile->yres * sdfile->yscale,
+                           FALSE);
     for (idt = 0; idt < G_N_ELEMENTS(data_types); idt++) {
         if (sdfile->data_type == data_types[idt].sdftype)
             break;
     }
     g_return_val_if_fail(idt < G_N_ELEMENTS(data_types), dfield);
 
-    data = gwy_data_field_get_data(dfield);
+    data = gwy_field_get_data(dfield);
     n = sdfile->xres * sdfile->yres;
     /* We assume Intel byteorder, although the format does not specify any order.  But it was developed in PC
      * context... */
-    gwy_convert_raw_data(sdfile->data, n, 1, data_types[idt].gwytype,
-                         GWY_BYTE_ORDER_LITTLE_ENDIAN, data, 1.0, 0.0);
+    gwy_convert_raw_data(sdfile->data, n, 1, data_types[idt].gwytype, GWY_BYTE_ORDER_LITTLE_ENDIAN, data, 1.0, 0.0);
     if (!gwy_isnan(data_types[idt].bad_data)) {
         for (i = 0; i < n; i++) {
             if (data[i] == data_types[idt].bad_data) {
                 if (!mask) {
-                    mask = gwy_data_field_new_alike(dfield, TRUE);
-                    m = gwy_data_field_get_data(mask);
+                    mask = gwy_field_new_alike(dfield, TRUE);
+                    m = gwy_field_get_data(mask);
                 }
                 m[i] = 1.0;
             }
         }
         if (mask) {
-            gwy_data_field_grains_invert(mask);
+            gwy_field_grains_invert(mask);
             gwy_app_channel_remove_bad_data(dfield, mask);
         }
     }
@@ -789,21 +769,20 @@ sdfile_read_data_bin(SDFile *sdfile, GwyDataField **mfield)
     return dfield;
 }
 
-static GwyDataField*
-sdfile_read_data_text(SDFile *sdfile,
-                      GError **error)
+static GwyField*
+read_data_text(SDFile *sdfile, GError **error)
 {
     gint i, n;
-    GwyDataField *dfield;
+    GwyField *dfield;
     gdouble *data;
     GError *err = NULL;
     gchar *p, *end, *line;
 
-    dfield = gwy_data_field_new(sdfile->xres, sdfile->yres,
-                                sdfile->xres * sdfile->xscale,
-                                sdfile->yres * sdfile->yscale,
-                                FALSE);
-    data = gwy_data_field_get_data(dfield);
+    dfield = gwy_field_new(sdfile->xres, sdfile->yres,
+                           sdfile->xres * sdfile->xscale,
+                           sdfile->yres * sdfile->yscale,
+                           FALSE);
+    data = gwy_field_get_data(dfield);
     n = sdfile->xres * sdfile->yres;
     p = sdfile->data;
     switch (sdfile->data_type) {
