@@ -1,6 +1,6 @@
 /*
- *  $Id: seiko.c 26163 2024-02-06 16:56:55Z yeti-dn $
- *  Copyright (C) 2006 David Necas (Yeti), Markus Pristovsek.
+ *  $Id: seiko.c 28789 2025-11-04 17:14:03Z yeti-dn $
+ *  Copyright (C) 2006-2025 David Necas (Yeti), Markus Pristovsek.
  *  E-mail: yeti@gwyddion.net, prissi@gift.physik.tu-berlin.de.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -60,14 +60,9 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/datafield.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 
 #include "err.h"
 #include "get.h"
@@ -97,23 +92,23 @@ typedef enum {
     SEIKO_CURRENT    = 2,
 } SeikoDataType;
 
-static gboolean      module_register   (void);
-static gint          seiko_detect      (const GwyFileDetectInfo *fileinfo,
-                                        gboolean only_name);
-static GwyContainer* seiko_load        (const gchar *filename,
-                                        GwyRunType mode,
-                                        GError **error);
-static GwyDataField* read_data_field   (const guchar *buffer,
-                                        guint size,
-                                        SeikoDataType datatype,
-                                        GError **error);
+static gboolean  module_register(void);
+static gint      detect_file    (const GwyFileDetectInfo *fileinfo,
+                                 gboolean only_name);
+static GwyFile*  load_file      (const gchar *filename,
+                                 GwyRunModeFlags mode,
+                                 GError **error);
+static GwyField* read_field     (const guchar *buffer,
+                                 guint size,
+                                 SeikoDataType datatype,
+                                 GError **error);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Seiko XQB, XQD, XQT and XQP files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.13",
+    "0.14",
     "David Nečas (Yeti) & Markus Pristovsek",
     "2006",
 };
@@ -125,17 +120,13 @@ module_register(void)
 {
     gwy_file_func_register("seiko",
                            N_("Seiko files (.xqb, .xqd, .xqt, .xqp)"),
-                           (GwyFileDetectFunc)&seiko_detect,
-                           (GwyFileLoadFunc)&seiko_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-seiko_detect(const GwyFileDetectInfo *fileinfo,
-             gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     gint score = 0;
 
@@ -162,21 +153,21 @@ seiko_detect(const GwyFileDetectInfo *fileinfo,
     return score;
 }
 
-static GwyContainer*
-seiko_load(const gchar *filename,
-           G_GNUC_UNUSED GwyRunType mode,
-           GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
     enum {
         COMMENT_OFFSET = 0x480,
         COMMENT_SIZE = 0x80,
     };
 
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     guchar *buffer = NULL;
     gsize size = 0;
     GError *err = NULL;
-    GwyDataField *dfield = NULL;
+    GwyField *dfield = NULL;
     SeikoDataType datatype = SEIKO_TOPOGRAPHY;
     gchar *comment;
     const gchar *extension;
@@ -209,36 +200,36 @@ seiko_load(const gchar *filename,
            datatype = SEIKO_CURRENT;
     }
 
-    dfield = read_data_field(buffer, size, datatype, error);
+    dfield = read_field(buffer, size, datatype, error);
     if (!dfield) {
         gwy_file_abandon_contents(buffer, size, NULL);
         return NULL;
     }
 
-    container = gwy_container_new();
-    gwy_container_pass_object(container, gwy_app_get_data_key_for_id(0), dfield);
+    file = gwy_file_new_in_construction();
+    gwy_file_pass_image(file, 0, dfield);
     comment = g_strndup(buffer + COMMENT_OFFSET, COMMENT_SIZE);
     g_strstrip(comment);
     if (*comment)
-        gwy_container_set_string(container, gwy_app_get_data_title_key_for_id(0), comment);
+        gwy_file_pass_title(file, GWY_FILE_IMAGE, 0, comment);
     else {
         g_free(comment);
-        gwy_app_channel_title_fall_back(container, 0);
+        gwy_image_title_fall_back(file, 0);
     }
 
-    gwy_app_channel_check_nonsquare(container, 0);
-    gwy_file_channel_import_log_add(container, 0, NULL, filename);
+    gwy_check_nonsquare_image(file, 0);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
 
     gwy_file_abandon_contents(buffer, size, NULL);
 
-    return container;
+    return file;
 }
 
-static GwyDataField*
-read_data_field(const guchar *buffer,
-                guint size,
-                SeikoDataType datatype,
-                GError **error)
+static GwyField*
+read_field(const guchar *buffer,
+           guint size,
+           SeikoDataType datatype,
+           GError **error)
 {
     enum {
         VERSION_OFFSET   = 0x10,
@@ -255,7 +246,7 @@ read_data_field(const guchar *buffer,
     G_GNUC_UNUSED guint version;
     guint endfile, datastart, imgsize;
     gdouble xreal, yreal, q, z0;
-    GwyDataField *dfield;
+    GwyField *dfield;
     const guchar *p, *unitstr;
 
     p = buffer + VERSION_OFFSET;
@@ -311,11 +302,11 @@ read_data_field(const guchar *buffer,
     xreal *= xres;
     yreal *= yres;
 
-    dfield = gwy_data_field_new(xres, yres, xreal, yreal, FALSE);
+    dfield = gwy_field_new(xres, yres, xreal, yreal, FALSE);
     gwy_convert_raw_data(buffer + HEADER_SIZE, xres*yres, 1,
                          GWY_RAW_DATA_UINT16, G_LITTLE_ENDIAN,
-                         gwy_data_field_get_data(dfield), q, z0);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
+                         gwy_field_get_data(dfield), q, z0);
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
 
     if (datatype == SEIKO_PHASE)
         unitstr = "deg";
@@ -323,7 +314,7 @@ read_data_field(const guchar *buffer,
         unitstr = "A";
     else
         unitstr = "m";
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), unitstr);
+    gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), unitstr);
 
     return dfield;
 }
