@@ -1,6 +1,6 @@
 /*
- *  $Id: sensofar.c 26468 2024-08-04 15:09:13Z yeti-dn $
- *  Copyright (C) 2004-2017 David Necas (Yeti), Petr Klapetek, Jan Horak.
+ *  $Id: sensofar.c 28805 2025-11-05 12:28:20Z yeti-dn $
+ *  Copyright (C) 2004-2025 David Necas (Yeti), Petr Klapetek, Jan Horak.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net, xhorak@gmail.com.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -35,10 +35,8 @@
  **/
 
 #include "config.h"
-#include <libgwyddion/gwymath.h>
-#include <libprocess/stats.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <glib/gi18n-lib.h>
+#include <gwy.h>
 
 #include "get.h"
 #include "err.h"
@@ -311,54 +309,55 @@ typedef struct {
     SensofarConfigMesura measure_config;
 } SensofarDataDesc;
 
-static gboolean      module_register       (void);
-static gint          sensofar_detect       (const GwyFileDetectInfo *fileinfo,
-                                            gboolean only_name);
-static GwyContainer* sensofar_load         (const gchar *filename,
-                                            GwyRunType mode,
-                                            GError **error);
-static gboolean      read_calibration_block(const guchar **p,
-                                            gsize size,
-                                            SensofarCalibratEixos_Arxiu *axes_config,
-                                            GError **error);
-static gboolean      read_config_mesura    (const guchar **p,
-                                            gsize size,
-                                            SensofarConfigMesura *measure_config,
-                                            GError **error);
-static gboolean      read_float_data_field (SensofarDataDesc *data_desc,
-                                            guint nrgb,
-                                            const gchar *filename,
-                                            GwyContainer *container,
-                                            guint *channelno,
-                                            const guchar **p,
-                                            gsize size,
-                                            GError **error);
-static gboolean      read_rgb_data_field   (SensofarDataDesc *data_desc,
-                                            const gchar *filename,
-                                            GwyContainer *container,
-                                            guint *channelno,
-                                            const guchar **p,
-                                            gsize size,
-                                            GError **error);
-static gboolean      read_rgb_data         (SensofarDataDesc *data_desc,
-                                            guint xres,
-                                            guint yres,
-                                            const gchar *filename,
-                                            GwyContainer *container,
-                                            guint *channelno,
-                                            const guchar **p,
-                                            gsize size,
-                                            GError **error);
-static gboolean      read_profiles         (SensofarDataDesc *data_desc,
-                                            GwyContainer *container,
-                                            guint *channelno,
-                                            const guchar **p,
-                                            gsize size,
-                                            GError **error);
-static void          add_image_meta        (SensofarDataDesc *data_desc,
-                                            GwyContainer *data,
-                                            guint channelno);
-static gboolean      parses_as_date        (const gchar *str);
+static gboolean module_register       (void);
+static gint     detect_file           (const GwyFileDetectInfo *fileinfo,
+                                       gboolean only_name);
+static GwyFile* load_file             (const gchar *filename,
+                                       GwyRunModeFlags mode,
+                                       GError **error);
+static gboolean read_calibration_block(const guchar **p,
+                                       gsize size,
+                                       SensofarCalibratEixos_Arxiu *axes_config,
+                                       GError **error);
+static gboolean read_config_mesura    (const guchar **p,
+                                       gsize size,
+                                       SensofarConfigMesura *measure_config,
+                                       GError **error);
+static gboolean read_float_field      (SensofarDataDesc *data_desc,
+                                       guint nrgb,
+                                       const gchar *filename,
+                                       GwyFile *file,
+                                       guint *channelno,
+                                       const guchar **p,
+                                       gsize size,
+                                       GError **error);
+static gboolean read_rgb_field        (SensofarDataDesc *data_desc,
+                                       const gchar *filename,
+                                       GwyFile *file,
+                                       guint *channelno,
+                                       const guchar **p,
+                                       gsize size,
+                                       GError **error);
+static gboolean read_rgb_data         (SensofarDataDesc *data_desc,
+                                       guint xres,
+                                       guint yres,
+                                       const gchar *filename,
+                                       GwyFile *file,
+                                       guint *channelno,
+                                       const guchar **p,
+                                       gsize size,
+                                       GError **error);
+static gboolean read_profiles         (SensofarDataDesc *data_desc,
+                                       const gchar *filename,
+                                       GwyFile *file,
+                                       guint *channelno,
+                                       const guchar **p,
+                                       gsize size,
+                                       GError **error);
+static void     add_image_meta        (SensofarDataDesc *data_desc,
+                                       GwyFile *data,
+                                       guint channelno);
+static gboolean parses_as_date        (const gchar *str);
 
 static const GwyEnum versions[] = {
     { "2000",  FORMAT_VERSION_2000,  },
@@ -550,7 +549,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports Sensofar PLu file format, version 2000 or newer."),
     "Jan Hořák <xhorak@gmail.com>, Yeti <yeti@gwyddion.net>",
-    "1.1",
+    "2.0",
     "David Nečas (Yeti) & Jan Hořák",
     "2008",
 };
@@ -562,16 +561,12 @@ module_register(void)
 {
     gwy_file_func_register("sensofar",
                            N_("Sensofar PLu files (.plu, .apx)"),
-                           (GwyFileDetectFunc)&sensofar_detect,
-                           (GwyFileLoadFunc)&sensofar_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
     return TRUE;
 }
 
 static gint
-sensofar_detect(const GwyFileDetectInfo *fileinfo,
-                gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     if (only_name)
         return g_str_has_suffix(fileinfo->name_lowercase, ".plu") ? 20 : 0;
@@ -585,15 +580,15 @@ sensofar_detect(const GwyFileDetectInfo *fileinfo,
     return 0;
 }
 
-static GwyContainer*
-sensofar_load(const gchar *filename,
-              G_GNUC_UNUSED GwyRunType mode,
-              GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
     SensofarDataDesc data_desc;
     SensofarConfigMesura *measure_config;
     MeasurementType meas_type;
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     guchar *buffer = NULL;
     gsize size = 0;
     GError *err = NULL;
@@ -645,27 +640,27 @@ sensofar_load(const gchar *filename,
     meas_type = measure_config->type;
     imatge_is_grey = (measure_config->version == FORMAT_VERSION_2000 || measure_config->version == FORMAT_VERSION_2006);
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     channelno = 0;
 
     for (i = 0; i < measure_config->num_layers; i++) {
         if (meas_type == MES_TOPO) {
-            if (!read_float_data_field(&data_desc, measure_config->num_images, filename, container, &channelno,
-                                       &p, size - (p - buffer), error))
+            if (!read_float_field(&data_desc, measure_config->num_images, filename, file, &channelno,
+                                  &p, size - (p - buffer), error))
                 goto fail;
         }
         else if (imatge_is_grey && meas_type == MES_IMATGE) {
-            if (!read_float_data_field(&data_desc, 0, filename, container, &channelno, &p, size - (p - buffer), error))
+            if (!read_float_field(&data_desc, 0, filename, file, &channelno, &p, size - (p - buffer), error))
                 goto fail;
         }
         else if (!imatge_is_grey && meas_type == MES_IMATGE) {
-            if (!read_rgb_data_field(&data_desc, filename, container, &channelno, &p, size - (p - buffer), error))
+            if (!read_rgb_field(&data_desc, filename, file, &channelno, &p, size - (p - buffer), error))
                 goto fail;
         }
         else if (meas_type == MES_PERFIL
                  || meas_type == MES_GRUIX
                  || meas_type == MES_MULTIPERFIL) {
-            if (!read_profiles(&data_desc, container, &channelno, &p, size - (p - buffer), error))
+            if (!read_profiles(&data_desc, filename, file, &channelno, &p, size - (p - buffer), error))
                 goto fail;
         }
         else {
@@ -678,11 +673,11 @@ sensofar_load(const gchar *filename,
      * specs does not seem agree with reality though. */
     gwy_debug("remaining data length: %lu", (gulong)(size - (p - buffer)));
 
-    return container;
+    return file;
 
 fail:
     gwy_file_abandon_contents(buffer, size, NULL);
-    GWY_OBJECT_UNREF(container);
+    g_clear_object(&file);
     return NULL;
 }
 
@@ -767,14 +762,14 @@ read_config_mesura(const guchar **p, gsize size,
 }
 
 static gboolean
-read_float_data_field(SensofarDataDesc *data_desc, guint nrgb,
-                      const gchar *filename,
-                      GwyContainer *container, guint *channelno,
-                      const guchar **p, gsize size,
-                      GError **error)
+read_float_field(SensofarDataDesc *data_desc, guint nrgb,
+                 const gchar *filename,
+                 GwyFile *file, guint *channelno,
+                 const guchar **p, gsize size,
+                 GError **error)
 {
-    GwyDataField *dfield, *mfield;
-    guint xres, yres, i, j, mcount;
+    GwyField *dfield, *mfield;
+    guint xres, yres, i, mcount;
     gdouble *data, *mdata;
     const guchar *buf = *p;
 
@@ -788,49 +783,43 @@ read_float_data_field(SensofarDataDesc *data_desc, guint nrgb,
     sanitise_real_size(&data_desc->axes_config.mppx, "x size");
     sanitise_real_size(&data_desc->axes_config.mppy, "y size");
 
-    dfield = gwy_data_field_new(xres, yres,
-                                data_desc->axes_config.mppx * xres * Micrometer,
-                                data_desc->axes_config.mppy * yres * Micrometer,
-                                FALSE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
+    dfield = gwy_field_new(xres, yres,
+                           data_desc->axes_config.mppx * xres * Micrometer,
+                           data_desc->axes_config.mppy * yres * Micrometer,
+                           FALSE);
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
 
-    mfield = gwy_data_field_new_alike(dfield, FALSE);
-    gwy_data_field_fill(mfield, 1.0);
+    mfield = gwy_field_new_alike(dfield, FALSE);
 
     /* In older files we can get here with both TOPO and IMGTE types. */
     if (data_desc->measure_config.type == MES_TOPO)
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), "m");
+        gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), "m");
 
-    data = gwy_data_field_get_data(dfield);
-    mdata = gwy_data_field_get_data(mfield);
-    for (i = 0; i < yres; i++) {
-        for (j = 0; j < xres; j++) {
-            gdouble v = gwy_get_gfloat_le(p);
-            if (v == LOST_PIXELS)
-                mdata[i*xres + j] = 0.0;
-            else
-                data[i*xres + j] = v;
-        }
-    }
-    *p += 2*sizeof(gfloat);   /* Data min and data max. */
+    data = gwy_field_get_data(dfield);
+    mdata = gwy_field_get_data(mfield);
+    gwy_convert_raw_data(p, xres*yres, 1, GWY_RAW_DATA_FLOAT, GWY_BYTE_ORDER_LITTLE_ENDIAN, data, 1.0, 0.0);
+    for (i = 0; i < xres*yres; i++)
+        mdata[i] = (data[i] != LOST_PIXELS);
+
+    *p += (xres*yres + 2)*sizeof(gfloat);   /* Plus data min and data max. */
 
     if (data_desc->measure_config.type == MES_TOPO)
-        gwy_data_field_multiply(dfield, Micrometer);
+        gwy_field_multiply(dfield, Micrometer);
 
     gwy_debug("Offset: %g %g",
               data_desc->axes_config.x_0, data_desc->axes_config.y_0);
     //FIXME: offset later, support of offset determined by version?
-    //gwy_data_field_set_xoffset(d, pow10(power10)*data_desc.axes_config.x_0);
-    //gwy_data_field_set_yoffset(d, pow10(power10)*data_desc.axes_config.y_0);
+    //gwy_field_set_xoffset(d, gwy_exp10(power10)*data_desc.axes_config.x_0);
+    //gwy_field_set_yoffset(d, gwy_exp10(power10)*data_desc.axes_config.y_0);
 
     mcount = gwy_app_channel_remove_bad_data(dfield, mfield);
 
-    gwy_container_pass_object(container, gwy_app_get_data_key_for_id(*channelno), dfield);
+    gwy_file_pass_image(file, *channelno, dfield);
     if (mcount)
-        gwy_container_set_object(container, gwy_app_get_mask_key_for_id(*channelno), mfield);
-    gwy_app_channel_title_fall_back(container, *channelno);
-    add_image_meta(data_desc, container, *channelno);
-    gwy_file_channel_import_log_add(container, *channelno, NULL, filename);
+        gwy_file_set_image_mask(file, *channelno, mfield);
+    gwy_image_title_fall_back(file, *channelno);
+    add_image_meta(data_desc, file, *channelno);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, *channelno, NULL, filename);
     (*channelno)++;
 
     g_object_unref(mfield);
@@ -840,7 +829,7 @@ read_float_data_field(SensofarDataDesc *data_desc, guint nrgb,
 
     size -= (*p - buf);
     for (i = 0; i < nrgb; i++) {
-        if (!read_rgb_data(data_desc, xres, yres, filename, container, channelno, p, size, error))
+        if (!read_rgb_data(data_desc, xres, yres, filename, file, channelno, p, size, error))
             return FALSE;
         size -= 3*xres*yres;
     }
@@ -849,11 +838,11 @@ read_float_data_field(SensofarDataDesc *data_desc, guint nrgb,
 }
 
 static gboolean
-read_rgb_data_field(SensofarDataDesc *data_desc,
-                    const gchar *filename,
-                    GwyContainer *container, guint *channelno,
-                    const guchar **p, gsize size,
-                    GError **error)
+read_rgb_field(SensofarDataDesc *data_desc,
+               const gchar *filename,
+               GwyFile *file, guint *channelno,
+               const guchar **p, gsize size,
+               GError **error)
 {
     guint xres, yres;
 
@@ -865,18 +854,18 @@ read_rgb_data_field(SensofarDataDesc *data_desc,
     xres = gwy_get_guint32_le(p);
     gwy_debug("Data size: %dx%d", xres, yres);
 
-    return read_rgb_data(data_desc, xres, yres, filename, container, channelno, p, size - 2*sizeof(guint32), error);
+    return read_rgb_data(data_desc, xres, yres, filename, file, channelno, p, size - 2*sizeof(guint32), error);
 }
 
 static gboolean
 read_rgb_data(SensofarDataDesc *data_desc,
               guint xres, guint yres,
               const gchar *filename,
-              GwyContainer *container, guint *channelno,
+              GwyFile *file, guint *channelno,
               const guchar **p, gsize size,
               GError **error)
 {
-    GwyDataField *rfield, *gfield, *bfield;
+    GwyField *rfield, *gfield, *bfield;
     guint k;
     gdouble *rdata, *gdata, *bdata;
     gboolean is_grey;
@@ -888,18 +877,18 @@ read_rgb_data(SensofarDataDesc *data_desc,
     sanitise_real_size(&data_desc->axes_config.mppx, "x size");
     sanitise_real_size(&data_desc->axes_config.mppy, "y size");
 
-    rfield = gwy_data_field_new(xres, yres,
-                                data_desc->axes_config.mppx * xres * Micrometer,
-                                data_desc->axes_config.mppy * yres * Micrometer,
-                                FALSE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(rfield), "m");
+    rfield = gwy_field_new(xres, yres,
+                           data_desc->axes_config.mppx * xres * Micrometer,
+                           data_desc->axes_config.mppy * yres * Micrometer,
+                           FALSE);
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(rfield), "m");
 
-    gfield = gwy_data_field_new_alike(rfield, FALSE);
-    bfield = gwy_data_field_new_alike(rfield, FALSE);
+    gfield = gwy_field_new_alike(rfield, FALSE);
+    bfield = gwy_field_new_alike(rfield, FALSE);
 
-    rdata = gwy_data_field_get_data(rfield);
-    gdata = gwy_data_field_get_data(gfield);
-    bdata = gwy_data_field_get_data(bfield);
+    rdata = gwy_field_get_data(rfield);
+    gdata = gwy_field_get_data(gfield);
+    bdata = gwy_field_get_data(bfield);
     is_grey = TRUE;
 
     for (k = 0; k < xres*yres; k++) {
@@ -913,38 +902,38 @@ read_rgb_data(SensofarDataDesc *data_desc,
     gwy_debug("Offset: %g %g",
               data_desc->axes_config.x_0, data_desc->axes_config.y_0);
     //FIXME: offset later, support of offset determined by version?
-    //gwy_data_field_set_xoffset(d, pow10(power10)*data_desc.axes_config.x_0);
-    //gwy_data_field_set_yoffset(d, pow10(power10)*data_desc.axes_config.y_0);
+    //gwy_field_set_xoffset(d, gwy_exp10(power10)*data_desc.axes_config.x_0);
+    //gwy_field_set_yoffset(d, gwy_exp10(power10)*data_desc.axes_config.y_0);
 
     /* Do not create three images when they are all the same. */
     if (is_grey) {
-        gwy_container_set_object(container, gwy_app_get_data_key_for_id(*channelno), rfield);
-        gwy_container_set_const_string(container, gwy_app_get_data_palette_key_for_id(*channelno), "Gray");
-        gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(*channelno), "Gray");
-        add_image_meta(data_desc, container, *channelno);
-        gwy_file_channel_import_log_add(container, *channelno, NULL, filename);
+        gwy_file_set_image(file, *channelno, rfield);
+        gwy_file_set_palette(file, GWY_FILE_IMAGE, *channelno, "Gray");
+        gwy_file_set_title(file, GWY_FILE_IMAGE, *channelno, "Gray", FALSE);
+        add_image_meta(data_desc, file, *channelno);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, *channelno, NULL, filename);
         (*channelno)++;
     }
     else {
-        gwy_container_set_object(container, gwy_app_get_data_key_for_id(*channelno), rfield);
-        gwy_container_set_const_string(container, gwy_app_get_data_palette_key_for_id(*channelno), "RGB-Red");
-        gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(*channelno), "Red");
-        add_image_meta(data_desc, container, *channelno);
-        gwy_file_channel_import_log_add(container, *channelno, NULL, filename);
+        gwy_file_set_image(file, *channelno, rfield);
+        gwy_file_set_palette(file, GWY_FILE_IMAGE, *channelno, "RGB-Red");
+        gwy_file_set_title(file, GWY_FILE_IMAGE, *channelno, "Red", FALSE);
+        add_image_meta(data_desc, file, *channelno);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, *channelno, NULL, filename);
         (*channelno)++;
 
-        gwy_container_set_object(container, gwy_app_get_data_key_for_id(*channelno), gfield);
-        gwy_container_set_const_string(container, gwy_app_get_data_palette_key_for_id(*channelno), "RGB-Green");
-        gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(*channelno), "Green");
-        add_image_meta(data_desc, container, *channelno);
-        gwy_file_channel_import_log_add(container, *channelno, NULL, filename);
+        gwy_file_set_image(file, *channelno, gfield);
+        gwy_file_set_palette(file, GWY_FILE_IMAGE, *channelno, "RGB-Green");
+        gwy_file_set_title(file, GWY_FILE_IMAGE, *channelno, "Green", FALSE);
+        add_image_meta(data_desc, file, *channelno);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, *channelno, NULL, filename);
         (*channelno)++;
 
-        gwy_container_set_object(container, gwy_app_get_data_key_for_id(*channelno), bfield);
-        gwy_container_set_const_string(container, gwy_app_get_data_palette_key_for_id(*channelno), "RGB-Blue");
-        gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(*channelno), "Blue");
-        add_image_meta(data_desc, container, *channelno);
-        gwy_file_channel_import_log_add(container, *channelno, NULL, filename);
+        gwy_file_set_image(file, *channelno, bfield);
+        gwy_file_set_palette(file, GWY_FILE_IMAGE, *channelno, "RGB-Blue");
+        gwy_file_set_title(file, GWY_FILE_IMAGE, *channelno, "Blue", FALSE);
+        add_image_meta(data_desc, file, *channelno);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, *channelno, NULL, filename);
         (*channelno)++;
     }
     g_object_unref(bfield);
@@ -956,17 +945,17 @@ read_rgb_data(SensofarDataDesc *data_desc,
 
 static gboolean
 read_profiles(SensofarDataDesc *data_desc,
-              GwyContainer *container, guint *channelno,
+              const gchar *filename,
+              GwyFile *file, guint *channelno,
               const guchar **p, gsize size,
               GError **error)
 {
     GwyGraphModel *gmodel;
     GwyGraphCurveModel *gcmodel;
     guint xres, yres, i, j, n;
-    GwySIUnit *units = NULL;
+    GwyUnit *units = NULL;
     gdouble *xdata, *ydata;
     gchar *s;
-    GQuark quark;
     gdouble dx;
 
     /* The yres is present and currectly set to 1 for MES_PERFIL and MES_GRUIX so we can process single and multiple
@@ -987,12 +976,12 @@ read_profiles(SensofarDataDesc *data_desc,
     gmodel = gwy_graph_model_new();
     g_object_set(gmodel, "title", _("Profile"), NULL);
 
-    units = gwy_si_unit_new("m"); /* values are in µm only */
-    g_object_set(gmodel, "si-unit-x", units, NULL);
+    units = gwy_unit_new("m"); /* values are in µm only */
+    g_object_set(gmodel, "unit-x", units, NULL);
     g_object_unref(units);
 
-    units = gwy_si_unit_new("m"); /* values are in µm only */
-    g_object_set(gmodel, "si-unit-y", units, NULL);
+    units = gwy_unit_new("m"); /* values are in µm only */
+    g_object_set(gmodel, "unit-y", units, NULL);
     g_object_unref(units);
 
     for (i = 0; i < yres; i++) {
@@ -1035,8 +1024,8 @@ read_profiles(SensofarDataDesc *data_desc,
         return FALSE;
     }
 
-    quark = gwy_app_get_graph_key_for_id(*channelno);
-    gwy_container_pass_object(container, quark, gmodel);
+    gwy_file_pass_graph(file, *channelno, gmodel);
+    gwy_log_add_import(file, GWY_FILE_GRAPH, *channelno, NULL, filename);
 
     (*channelno)++;
 
@@ -1045,7 +1034,7 @@ read_profiles(SensofarDataDesc *data_desc,
 
 static void
 set_meta_enum(GwyContainer *meta, guint value, const gchar *name,
-                   const GwyEnum *table, guint n)
+              const GwyEnum *table, guint n)
 {
     const gchar *s;
 
@@ -1055,15 +1044,13 @@ set_meta_enum(GwyContainer *meta, guint value, const gchar *name,
 
 static void
 add_image_meta(SensofarDataDesc *data_desc,
-               GwyContainer *data, guint channelno)
+               GwyFile *file, guint channelno)
 {
     SensofarCalibratEixos_Arxiu *axes_config = &data_desc->axes_config;
     SensofarConfigMesura *measure_config = &data_desc->measure_config;
     SensofarFOVScanSettings *fov = &measure_config->settings.fov;
-    GwyContainer *meta;
-    GQuark quark;
+    GwyContainer *meta = gwy_container_new_in_construction();
 
-    meta = gwy_container_new();
     gwy_container_set_const_string_by_name(meta, "Date", data_desc->date.str);
     if (*(data_desc->user_comment)) {
         gwy_container_set_const_string_by_name(meta, "Comment", data_desc->user_comment);
@@ -1116,8 +1103,7 @@ add_image_meta(SensofarDataDesc *data_desc,
                       acq_methods_map, G_N_ELEMENTS(acq_methods_map));
     }
 
-    quark = gwy_app_get_data_meta_key_for_id(channelno);
-    gwy_container_pass_object(data, quark, meta);
+    gwy_file_pass_meta(file, GWY_FILE_IMAGE, channelno, meta);
 }
 
 /* File starts with date, try to parse it.
