@@ -1,6 +1,6 @@
 /*
- *  $Id: sensofarx.c 26168 2024-02-08 10:18:02Z yeti-dn $
- *  Copyright (C) 2015 David Necas (Yeti).
+ *  $Id: sensofarx.c 28789 2025-11-04 17:14:03Z yeti-dn $
+ *  Copyright (C) 2015-2025 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
  *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
@@ -31,15 +31,10 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/stats.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 
 #include "err.h"
 #include "gwyzip.h"
@@ -58,34 +53,34 @@ typedef struct {
     gboolean parsing_recipe;
 } PLUxFile;
 
-static gboolean      module_register            (void);
-static gint          sensofarx_detect           (const GwyFileDetectInfo *fileinfo,
-                                                 gboolean only_name);
-static gboolean      sensofarx_is_plausible_file(const guchar *filehead,
-                                                 gsize len);
-static GwyContainer* sensofarx_load             (const gchar *filename,
-                                                 GwyRunType mode,
-                                                 GError **error);
-static gboolean      sensofarx_parse_index      (GwyZipFile zipfile,
-                                                 PLUxFile *pluxfile,
-                                                 GError **error);
-static void          sensofarx_parse_recipe     (GwyZipFile zipfile,
-                                                 PLUxFile *pluxfile);
-static gboolean      read_binary_data           (const PLUxFile *pluxfile,
-                                                 GwyZipFile zipfile,
-                                                 const gchar *filename,
-                                                 GwyContainer *container,
-                                                 GError **error);
-static void          sensofarx_file_free        (PLUxFile *pluxfile);
-static GwyContainer* get_metadata               (const PLUxFile *pluxfile,
-                                                 guint id);
+static gboolean      module_register  (void);
+static gint          detect_file      (const GwyFileDetectInfo *fileinfo,
+                                       gboolean only_name);
+static gboolean      is_plausible_file(const guchar *filehead,
+                                       gsize len);
+static GwyFile*      load_file        (const gchar *filename,
+                                       GwyRunModeFlags mode,
+                                       GError **error);
+static gboolean      parse_index      (GwyZipFile zipfile,
+                                       PLUxFile *pluxfile,
+                                       GError **error);
+static void          parse_recipe     (GwyZipFile zipfile,
+                                       PLUxFile *pluxfile);
+static gboolean      read_binary_data (const PLUxFile *pluxfile,
+                                       GwyZipFile zipfile,
+                                       const gchar *filename,
+                                       GwyFile *file,
+                                       GError **error);
+static void          free_pluxfile    (PLUxFile *pluxfile);
+static GwyContainer* get_metadata     (const PLUxFile *pluxfile,
+                                       guint id);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Reads Sensofar PLUx files."),
     "Yeti <yeti@gwyddion.net>",
-    "1.1",
+    "2.0",
     "David Nečas (Yeti)",
     "2015",
 };
@@ -97,17 +92,12 @@ module_register(void)
 {
     gwy_file_func_register("sensofarx",
                            N_("Sensofar PLUx files (.plux)"),
-                           (GwyFileDetectFunc)&sensofarx_detect,
-                           (GwyFileLoadFunc)&sensofarx_load,
-                           NULL,
-                           NULL);
-
+                           detect_file, load_file, NULL, NULL);
     return TRUE;
 }
 
 static gint
-sensofarx_detect(const GwyFileDetectInfo *fileinfo,
-                 gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     GwyZipFile zipfile;
     guchar *content;
@@ -123,8 +113,8 @@ sensofarx_detect(const GwyFileDetectInfo *fileinfo,
 
     /* It should contain some of the expected file names.  Unfortunately, they seem to store the raw data first and
      * they are to huge to catch anything following.  So this may not be as reliable as we would like. */
-    if (!sensofarx_is_plausible_file(fileinfo->head, fileinfo->buffer_len)
-        && !sensofarx_is_plausible_file(fileinfo->tail, fileinfo->buffer_len))
+    if (!is_plausible_file(fileinfo->head, fileinfo->buffer_len)
+        && !is_plausible_file(fileinfo->tail, fileinfo->buffer_len))
         return 0;
 
     /* We have to realy look inside.  And since index.xml is a popular name for the main XML document within such
@@ -145,16 +135,15 @@ sensofarx_detect(const GwyFileDetectInfo *fileinfo,
 
 /* Try to find the name of an expected file somewhere. */
 static gboolean
-sensofarx_is_plausible_file(const guchar *filehead, gsize len)
+is_plausible_file(const guchar *filehead, gsize len)
 {
     static const gchar *filenames[] = {
         "LAYER_0.raw", "LAYER_0.stack.raw",
         "LAYER_1.raw", "LAYER_1.stack.raw",
         "index.xml", "recipe.txt", "display.txt",
     };
-    guint i;
 
-    for (i = 0; i < G_N_ELEMENTS(filenames); i++) {
+    for (guint i = 0; i < G_N_ELEMENTS(filenames); i++) {
         if (gwy_memmem(filehead, len, filenames[i], strlen(filenames[i])))
             return TRUE;
     }
@@ -162,20 +151,20 @@ sensofarx_is_plausible_file(const guchar *filehead, gsize len)
     return FALSE;
 }
 
-static GwyContainer*
-sensofarx_load(const gchar *filename,
-               G_GNUC_UNUSED GwyRunType mode,
-               GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     PLUxFile pluxfile;
     GwyZipFile zipfile;
 
     if (!(zipfile = gwyzip_open(filename, error)))
         return NULL;
 
-    gwy_clear(&pluxfile, 1);
-    if (!sensofarx_parse_index(zipfile, &pluxfile, error))
+    gwy_clear1(pluxfile);
+    if (!parse_index(zipfile, &pluxfile, error))
         goto fail;
 
     if (!pluxfile.layers->len) {
@@ -183,37 +172,36 @@ sensofarx_load(const gchar *filename,
         goto fail;
     }
 
-    sensofarx_parse_recipe(zipfile, &pluxfile);
+    parse_recipe(zipfile, &pluxfile);
 
-    container = gwy_container_new();
-    if (!read_binary_data(&pluxfile, zipfile, filename, container, error))
-        GWY_OBJECT_UNREF(container);
+    file = gwy_file_new_in_construction();
+    if (!read_binary_data(&pluxfile, zipfile, filename, file, error))
+        g_clear_object(&file);
 
 fail:
     gwyzip_close(zipfile);
-    sensofarx_file_free(&pluxfile);
+    free_pluxfile(&pluxfile);
 
-    return container;
+    return file;
 }
 
 static gboolean
 read_binary_data(const PLUxFile *pluxfile,
                  GwyZipFile zipfile,
                  const gchar *filename,
-                 GwyContainer *container,
+                 GwyFile *file,
                  GError **error)
 {
     GArray *layers = pluxfile->layers;
     GHashTable *hash = pluxfile->hash;
     GString *str = pluxfile->str;
-    GwyDataField *dfield, *mask;
+    GwyField *dfield, *mask;
     GwyContainer *meta;
     guchar *content;
-    gchar *datafilename, *title;
+    gchar *datafilename;
     gsize contentsize, expected_size;
     guint xres, yres;
     gdouble xreal, yreal;
-    GQuark quark;
     gint id;
     guint i;
 
@@ -257,31 +245,25 @@ read_binary_data(const PLUxFile *pluxfile,
             return FALSE;
         }
 
-        dfield = gwy_data_field_new(xres, yres, xres*xreal*1e-6, yres*yreal*1e-6, FALSE);
+        dfield = gwy_field_new(xres, yres, xres*xreal*1e-6, yres*yreal*1e-6, FALSE);
         gwy_convert_raw_data(content, xres*yres, 1, GWY_RAW_DATA_FLOAT, GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                             gwy_data_field_get_data(dfield), 1e-6, 0.0);
+                             gwy_field_get_data(dfield), 1e-6, 0.0);
         g_free(content);
 
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), "m");
+        gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
+        gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), "m");
 
-        gwy_container_set_object(container, gwy_app_get_data_key_for_id(id), dfield);
-        if ((mask = gwy_app_channel_mask_of_nans(dfield, TRUE))) {
-            quark = gwy_app_get_mask_key_for_id(id);
-            gwy_container_pass_object(container, quark, mask);
-        }
+        gwy_file_set_image(file, id, dfield);
+        if ((mask = gwy_app_channel_mask_of_nans(dfield, TRUE)))
+            gwy_file_pass_image_mask(file, id, mask);
         g_object_unref(dfield);
 
-        g_string_printf(str, "/%d/data/title", id);
-        title = g_strdup("Z");
-        gwy_container_set_string_by_name(container, str->str, title);
+        gwy_file_pass_title(file, GWY_FILE_IMAGE, id, g_strdup("Z"));
 
-        if ((meta = get_metadata(pluxfile, id))) {
-            g_string_printf(str, "/%d/meta", id);
-            gwy_container_pass_object_by_name(container, str->str, meta);
-        }
+        if ((meta = get_metadata(pluxfile, id)))
+            gwy_file_pass_meta(file, GWY_FILE_IMAGE, id, meta);
 
-        gwy_file_channel_import_log_add(container, id, NULL, filename);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, id, NULL, filename);
 
         id++;
     }
@@ -290,12 +272,12 @@ read_binary_data(const PLUxFile *pluxfile,
 }
 
 static void
-sensofarx_start_element(G_GNUC_UNUSED GMarkupParseContext *context,
-                        const gchar *element_name,
-                        G_GNUC_UNUSED const gchar **attribute_names,
-                        G_GNUC_UNUSED const gchar **attribute_values,
-                        gpointer user_data,
-                        G_GNUC_UNUSED GError **error)
+start_element(G_GNUC_UNUSED GMarkupParseContext *context,
+              const gchar *element_name,
+              G_GNUC_UNUSED const gchar **attribute_names,
+              G_GNUC_UNUSED const gchar **attribute_values,
+              gpointer user_data,
+              G_GNUC_UNUSED GError **error)
 {
     PLUxFile *pluxfile = (PLUxFile*)user_data;
     gchar *path, *end, *s;
@@ -320,10 +302,10 @@ sensofarx_start_element(G_GNUC_UNUSED GMarkupParseContext *context,
 }
 
 static void
-sensofarx_end_element(G_GNUC_UNUSED GMarkupParseContext *context,
-                      const gchar *element_name,
-                      gpointer user_data,
-                      G_GNUC_UNUSED GError **error)
+end_element(G_GNUC_UNUSED GMarkupParseContext *context,
+            const gchar *element_name,
+            gpointer user_data,
+            G_GNUC_UNUSED GError **error)
 {
     PLUxFile *pluxfile = (PLUxFile*)user_data;
     guint n, len = pluxfile->path->len;
@@ -339,21 +321,21 @@ sensofarx_end_element(G_GNUC_UNUSED GMarkupParseContext *context,
 }
 
 static void
-sensofarx_text(G_GNUC_UNUSED GMarkupParseContext *context,
-               const gchar *text,
-               G_GNUC_UNUSED gsize text_len,
-               gpointer user_data,
-               G_GNUC_UNUSED GError **error)
+text(G_GNUC_UNUSED GMarkupParseContext *context,
+     const gchar *value,
+     G_GNUC_UNUSED gsize value_len,
+     gpointer user_data,
+     G_GNUC_UNUSED GError **error)
 {
     PLUxFile *pluxfile = (PLUxFile*)user_data;
     GHashTable *hash;
     gchar *path = pluxfile->path->str;
     GString *str = pluxfile->str;
 
-    if (!*text)
+    if (!*value)
         return;
 
-    g_string_assign(str, text);
+    g_string_assign(str, value);
     g_strstrip(str->str);
     if (!*(str->str))
         return;
@@ -364,14 +346,14 @@ sensofarx_text(G_GNUC_UNUSED GMarkupParseContext *context,
 }
 
 static gboolean
-sensofarx_parse_index(GwyZipFile zipfile,
-                      PLUxFile *pluxfile,
-                      GError **error)
+parse_index(GwyZipFile zipfile,
+            PLUxFile *pluxfile,
+            GError **error)
 {
     GMarkupParser parser = {
-        &sensofarx_start_element,
-        &sensofarx_end_element,
-        &sensofarx_text,
+        &start_element,
+        &end_element,
+        &text,
         NULL,
         NULL,
     };
@@ -410,15 +392,10 @@ sensofarx_parse_index(GwyZipFile zipfile,
 }
 
 static void
-sensofarx_parse_recipe(GwyZipFile zipfile,
-                       PLUxFile *pluxfile)
+parse_recipe(GwyZipFile zipfile, PLUxFile *pluxfile)
 {
     GMarkupParser parser = {
-        &sensofarx_start_element,
-        &sensofarx_end_element,
-        &sensofarx_text,
-        NULL,
-        NULL,
+        &start_element, &end_element, &text, NULL, NULL,
     };
     GMarkupParseContext *context = NULL;
     guchar *content = NULL, *s;
@@ -449,7 +426,7 @@ sensofarx_parse_recipe(GwyZipFile zipfile,
 }
 
 static void
-sensofarx_file_free(PLUxFile *pluxfile)
+free_pluxfile(PLUxFile *pluxfile)
 {
     if (pluxfile->hash)
         g_hash_table_destroy(pluxfile->hash);
@@ -496,7 +473,7 @@ static GwyContainer*
 get_metadata(const PLUxFile *pluxfile, guint id)
 {
     GHashTable *hash = pluxfile->hash;
-    GwyContainer *meta = gwy_container_new();
+    GwyContainer *meta = gwy_container_new_in_construction();
     gchar *name, *value;
     gchar buf[40], c;
     guint n, i;
