@@ -1,6 +1,6 @@
 /*
- *  $Id: sensolytics.c 26519 2024-08-15 18:04:20Z yeti-dn $
- *  Copyright (C) 2009-2023 David Necas (Yeti).
+ *  $Id: sensolytics.c 28911 2025-11-24 18:27:42Z yeti-dn $
+ *  Copyright (C) 2009-2025 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
  *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
@@ -63,19 +63,10 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/stats.h>
-#include <libprocess/correct.h>
-#include <libprocess/grains.h>
-#include <libprocess/lawn.h>
-#include <libgwydgets/gwygraphmodel.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -95,13 +86,13 @@ typedef struct {
 
 typedef struct {
     gint *pixel_index;
-    GwyDataField *mask;
+    GwyField *mask;
     gint ndata;
     gboolean add_mask;
 } SensolyticsGridMap;
 
 typedef struct {
-    GwyDataField *dfield;
+    GwyField *dfield;
     GwyLawn *lawn;
     gdouble *data;
     gchar **strings;
@@ -115,10 +106,10 @@ typedef struct {
 } SensolyticsChannel;
 
 static gboolean       module_register         (void);
-static gint           sly_detect              (const GwyFileDetectInfo *fileinfo,
+static gint           detect_file             (const GwyFileDetectInfo *fileinfo,
                                                gboolean only_name);
-static GwyContainer*  sly_load                (const gchar *filename,
-                                               GwyRunType mode,
+static GwyFile*       load_file               (const gchar *filename,
+                                               GwyRunModeFlags mode,
                                                GError **error);
 static GHashTable*    read_header             (gchar **p,
                                                GArray *channels,
@@ -146,10 +137,10 @@ static void           create_lawns            (GArray *channels,
 static GwyGraphModel* create_graph_model      (SensolyticsChannel *abscissa,
                                                SensolyticsChannel *ordinate);
 static GwyContainer*  get_meta                (GHashTable *hash);
-static void           clone_field_meta        (GwyContainer *container,
+static void           clone_field_meta        (GwyFile *file,
                                                GwyContainer *meta,
                                                gint id);
-static void           clone_lawn_meta         (GwyContainer *container,
+static void           clone_lawn_meta         (GwyFile *file,
                                                GwyContainer *meta,
                                                GwyContainer *specmeta,
                                                gint id);
@@ -175,17 +166,13 @@ module_register(void)
 {
     gwy_file_func_register("sensolytics",
                            N_("Sensolytics text files (.dat)"),
-                           (GwyFileDetectFunc)&sly_detect,
-                           (GwyFileLoadFunc)&sly_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-sly_detect(const GwyFileDetectInfo *fileinfo,
-           gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     guint i, iver, len = fileinfo->buffer_len;
     const guchar *head = fileinfo->head;
@@ -229,12 +216,13 @@ lawn_reduce_avg(gint ncurves, gint curvelength, const gdouble *curvedata, gpoint
     return s/curvelength;
 }
 
-static GwyContainer*
-sly_load(const gchar *filename,
-         G_GNUC_UNUSED GwyRunType mode,
-         GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *container = NULL, *meta = NULL;
+    GwyFile *file = NULL;
+    GwyContainer *meta = NULL;
     gchar *buffer = NULL;
     GError *err = NULL;
     GHashTable *hash = NULL;
@@ -243,7 +231,7 @@ sly_load(const gchar *filename,
     SensolyticsChannel *channel, *channel2;
     SensolyticsGridMap map;
     GwyGraphModel *gmodel;
-    GwyDataField *mask, *preview;
+    GwyField *mask, *preview;
     Dimensions dimensions;
     guint i, abscissa, startfrom;
     gint j, id;
@@ -253,7 +241,7 @@ sly_load(const gchar *filename,
         return NULL;
     }
 
-    gwy_clear(&map, 1);
+    gwy_clear1(map);
     channels = g_array_new(FALSE, TRUE, sizeof(SensolyticsChannel));
     p = buffer;
     if (!(hash = read_header(&p, channels, error)))
@@ -277,23 +265,23 @@ sly_load(const gchar *filename,
         create_lawns(channels, &dimensions, &map, filename);
         meta = get_meta(hash);
 
-        container = gwy_container_new();
+        file = gwy_file_new_in_construction();
         for (i = 0; i < channels->len; i++) {
             channel = &g_array_index(channels, SensolyticsChannel, i);
             if (!channel->dfield)
                 continue;
-            gwy_container_set_object(container, gwy_app_get_data_key_for_id(id), channel->dfield);
-            gwy_app_channel_check_nonsquare(container, id);
+            gwy_file_set_image(file, id, channel->dfield);
+            gwy_check_nonsquare_image(file, id);
             if (map.add_mask) {
-                mask = gwy_data_field_duplicate(map.mask);
-                gwy_container_pass_object(container, gwy_app_get_mask_key_for_id(id), mask);
+                mask = gwy_field_copy(map.mask);
+                gwy_file_pass_image_mask(file, id, mask);
             }
             if (channel->name)
-                gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(id), channel->name);
+                gwy_file_set_title(file, GWY_FILE_IMAGE, id, channel->name, FALSE);
             else
-                gwy_app_channel_title_fall_back(container, id);
-            gwy_file_channel_import_log_add(container, id, NULL, filename);
-            clone_field_meta(container, meta, id);
+                gwy_image_title_fall_back(file, id);
+            gwy_log_add_import(file, GWY_FILE_IMAGE, id, NULL, filename);
+            clone_field_meta(file, meta, id);
             id++;
         }
 
@@ -302,16 +290,16 @@ sly_load(const gchar *filename,
             channel = &g_array_index(channels, SensolyticsChannel, i);
             if (!channel->lawn)
                 continue;
-            gwy_container_set_object(container, gwy_app_get_lawn_key_for_id(id), channel->lawn);
+            gwy_file_set_cmap(file, id, channel->lawn);
             if (channel->title)
-                gwy_container_set_const_string(container, gwy_app_get_lawn_title_key_for_id(id), channel->title);
-            preview = gwy_data_field_new(dimensions.xres, dimensions.yres, dimensions.xreal, dimensions.yreal, FALSE);
+                gwy_file_set_title(file, GWY_FILE_CMAP, id, channel->title, FALSE);
+            preview = gwy_field_new(dimensions.xres, dimensions.yres, dimensions.xreal, dimensions.yreal, FALSE);
             j = gwy_lawn_get_n_curves(channel->lawn) - 1;
             gwy_lawn_reduce_to_plane(channel->lawn, preview, lawn_reduce_avg, GINT_TO_POINTER(j));
-            gwy_si_unit_assign(gwy_data_field_get_si_unit_z(preview), gwy_lawn_get_si_unit_curve(channel->lawn, j));
-            gwy_container_pass_object(container, gwy_app_get_lawn_preview_key_for_id(id), preview);
-            gwy_file_curve_map_import_log_add(container, id, NULL, filename);
-            clone_lawn_meta(container, meta, channel->specmeta, id);
+            gwy_unit_assign(gwy_field_get_unit_z(preview), gwy_lawn_get_unit_curve(channel->lawn, j));
+            gwy_container_pass_object(GWY_CONTAINER(file), gwy_file_key_cmap_picture(id), preview);
+            gwy_log_add_import(file, GWY_FILE_CMAP, id, NULL, filename);
+            clone_lawn_meta(file, meta, channel->specmeta, id);
             id++;
         }
 
@@ -336,13 +324,14 @@ sly_load(const gchar *filename,
             goto end;
         }
 
-        container = gwy_container_new();
+        file = gwy_file_new_in_construction();
         for (i = startfrom; i < channels->len; i++) {
             channel2 = &g_array_index(channels, SensolyticsChannel, i);
             if (!channel2->data)
                 continue;
             gmodel = create_graph_model(channel, channel2);
-            gwy_container_pass_object(container, gwy_app_get_graph_key_for_id(id), gmodel);
+            gwy_file_pass_graph(file, id, gmodel);
+            gwy_log_add_import(file, GWY_FILE_GRAPH, id, NULL, filename);
             id++;
         }
     }
@@ -351,14 +340,14 @@ end:
     g_free(buffer);
     if (hash)
         g_hash_table_destroy(hash);
-    GWY_OBJECT_UNREF(map.mask);
+    g_clear_object(&map.mask);
     g_free(map.pixel_index);
     if (channels) {
         clear_channels(channels);
         g_array_free(channels, TRUE);
     }
 
-    return container;
+    return file;
 }
 
 static GHashTable*
@@ -490,7 +479,7 @@ parse_channels(GHashTable *hash, GArray *channels, gchar *line,
         }
         g_strstrip(p);
 
-        gwy_clear(&channel, 1);
+        gwy_clear1(channel);
         channel.name = p;
         channel.unit = unit;
         g_array_append_val(channels, channel);
@@ -519,7 +508,7 @@ parse_channels(GHashTable *hash, GArray *channels, gchar *line,
     g_array_set_size(channels, 6);
     str = g_string_new(NULL);
     for (i = 0; i < ncols; i++) {
-        gwy_clear(&channel, 1);
+        gwy_clear1(channel);
 
         g_string_printf(str, "Channel %d Unit", i+1);
         channel.unit = g_hash_table_lookup(hash, str->str);
@@ -536,7 +525,7 @@ static gdouble
 parse_number_with_unit(GHashTable *hash, const gchar *key, const gchar **unitstr)
 {
     const gchar *s;
-    GwySIUnit *siunit;
+    GwyUnit *siunit;
     gchar *end;
     gint power10;
     gdouble v;
@@ -550,8 +539,8 @@ parse_number_with_unit(GHashTable *hash, const gchar *key, const gchar **unitstr
     if (unitstr)
         *unitstr = end;
     if (*end) {
-        siunit = gwy_si_unit_new_parse(end, &power10);
-        v *= pow10(power10);
+        siunit = gwy_unit_new_parse(end, &power10);
+        v *= gwy_exp10(power10);
         g_object_unref(siunit);
     }
 
@@ -630,15 +619,15 @@ read_data(GArray *channels, gchar **p, GError **error)
     SensolyticsChannel *channel;
     gint j, G_GNUC_UNUSED lineno, power10, ncols = channels->len;
     gchar *line, *end;
-    GwySIUnit *unit;
+    GwyUnit *unit;
     gdouble *q;
 
-    unit = gwy_si_unit_new(NULL);
+    unit = gwy_unit_new(NULL);
     q = g_new(gdouble, ncols);
     for (j = 0; j < ncols; j++) {
         channel = &g_array_index(channels, SensolyticsChannel, j);
-        gwy_si_unit_set_from_string_parse(unit, channel->unit, &power10);
-        q[j] = pow10(power10);
+        power10 = gwy_unit_set_from_string(unit, channel->unit);
+        q[j] = gwy_exp10(power10);
     }
     g_object_unref(unit);
 
@@ -730,9 +719,9 @@ map_to_grid(GArray *channels, const Dimensions *dimensions,
         g_warning("Column 4 is not \"Y rel.\", mapping to pixels will be nonsense.");
 
     map->pixel_index = pixel_index = g_new(gint, ndata);
-    map->mask = gwy_data_field_new(dimensions->xres, dimensions->yres,
-                                   fabs(dimensions->xreal), fabs(dimensions->yreal), TRUE);
-    m = gwy_data_field_get_data(map->mask);
+    map->mask = gwy_field_new(dimensions->xres, dimensions->yres,
+                              fabs(dimensions->xreal), fabs(dimensions->yreal), TRUE);
+    m = gwy_field_get_data(map->mask);
     /* Figure out which data row corresponds to which pixel in the image.
      * NB: The increments and lengths can be negative. If the step is negative the relative coordinates seem to
      * indeed go into negative. */
@@ -752,46 +741,46 @@ static void
 create_fields(GArray *channels, const Dimensions *dimensions, SensolyticsGridMap *map)
 {
     SensolyticsChannel *channel;
-    GwyDataField *field;
+    GwyField *field;
     gint n, k, ndata = map->ndata;
     gdouble *d, *m;
     guint i, ncols = channels->len;
     gint *pixel_index = map->pixel_index;
 
     n = dimensions->xres * dimensions->yres;
-    m = gwy_data_field_get_data(map->mask);
+    m = gwy_field_get_data(map->mask);
     for (i = 5; i < ncols; i++) {
         channel = &g_array_index(channels, SensolyticsChannel, i);
         /* Skip string channels for now, we need to create Lawns from them later. */
         if (!channel->data)
             continue;
 
-        field = gwy_data_field_new(dimensions->xres, dimensions->yres,
-                                   fabs(dimensions->xreal), fabs(dimensions->yreal), TRUE);
+        field = gwy_field_new(dimensions->xres, dimensions->yres,
+                              fabs(dimensions->xreal), fabs(dimensions->yreal), TRUE);
         /* The unit may be given with prefix, but we have already taken care of it. Do not multiply it by some power
          * of 10 again. */
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(field), dimensions->xyunit);
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(field), channel->unit);
-        d = gwy_data_field_get_data(field);
+        gwy_unit_set_from_string(gwy_field_get_unit_xy(field), dimensions->xyunit);
+        gwy_unit_set_from_string(gwy_field_get_unit_z(field), channel->unit);
+        d = gwy_field_get_data(field);
         for (k = 0; k < ndata; k++)
             d[pixel_index[k]] += channel->data[k];
         for (k = 0; k < n; k++)
             d[k] = m[k] > 0.0 ? d[k]/m[k] : 0.0;
 
-        if (gwy_data_field_get_rms(field) == 0.0) {
+        if (gwy_field_get_rms(field) == 0.0) {
             g_object_unref(field);
             continue;
         }
 
-        gwy_data_field_invert(field, FALSE, TRUE, FALSE);
+        gwy_field_flip(field, TRUE, FALSE);
         channel->dfield = field;
     }
-    if (gwy_data_field_get_min(map->mask) < 1.0) {
-        gwy_data_field_grains_invert(map->mask);
+    if (gwy_field_get_min(map->mask) < 1.0) {
+        gwy_field_grains_invert(map->mask);
         for (i = 5; i < ncols; i++) {
             channel = &g_array_index(channels, SensolyticsChannel, i);
             if (channel->dfield)
-                gwy_data_field_laplace_solve(channel->dfield, map->mask, -1, 0.5);
+                gwy_field_laplace_solve(channel->dfield, map->mask, -1, 0.5);
         }
         map->add_mask = TRUE;
     }
@@ -905,7 +894,7 @@ create_lawns(GArray *channels, const Dimensions *dimensions, SensolyticsGridMap
                     channel->specmeta = get_meta(spechash);
                 lawn = gwy_lawn_new(dimensions->xres, dimensions->yres, dimensions->xreal, dimensions->yreal,
                                     ncurves, 0);
-                gwy_si_unit_set_from_string(gwy_lawn_get_si_unit_xy(lawn), dimensions->xyunit);
+                gwy_unit_set_from_string(gwy_lawn_get_unit_xy(lawn), dimensions->xyunit);
                 if ((s = g_hash_table_lookup(spechash, "Experiment type")))
                     gwy_assign_string(&channel->title, s);
 
@@ -915,7 +904,7 @@ create_lawns(GArray *channels, const Dimensions *dimensions, SensolyticsGridMap
                     if ((d = specchannel->data)) {
                         if (specchannel->name)
                             gwy_lawn_set_curve_label(lawn, ncurves, specchannel->name);
-                        gwy_si_unit_set_from_string(gwy_lawn_get_si_unit_curve(lawn, ncurves), specchannel->unit);
+                        gwy_unit_set_from_string(gwy_lawn_get_unit_curve(lawn, ncurves), specchannel->unit);
                         ncurves++;
                     }
                 }
@@ -923,7 +912,7 @@ create_lawns(GArray *channels, const Dimensions *dimensions, SensolyticsGridMap
             else {
                 if (ncurves != gwy_lawn_get_n_curves(lawn)) {
                     g_warning("Inconsistent content of curve data files.");
-                    GWY_OBJECT_UNREF(lawn);
+                    g_clear_object(&lawn);
                     /* Terminate the loop, but allow the end cleanup to run. */
                     k = channel->n;
                     goto end;
@@ -957,13 +946,13 @@ create_graph_model(SensolyticsChannel *abscissa, SensolyticsChannel *ordinate)
 {
     GwyGraphModel *gmodel = gwy_graph_model_new();
     GwyGraphCurveModel *cmodel;
-    GwySIUnit *xunit, *yunit;
+    GwyUnit *xunit, *yunit;
 
     g_return_val_if_fail(abscissa->n == ordinate->n, gmodel);
 
-    xunit = gwy_si_unit_new(abscissa->unit);
-    yunit = gwy_si_unit_new(ordinate->unit);
-    g_object_set(gmodel, "si-unit-x", xunit, "si-unit-y", yunit, NULL);
+    xunit = gwy_unit_new(abscissa->unit);
+    yunit = gwy_unit_new(ordinate->unit);
+    g_object_set(gmodel, "unit-x", xunit, "unit-y", yunit, NULL);
     g_object_unref(xunit);
     g_object_unref(yunit);
 
@@ -1010,7 +999,7 @@ add_meta(gpointer hkey, gpointer hvalue, gpointer user_data)
 static GwyContainer*
 get_meta(GHashTable *hash)
 {
-    GwyContainer *meta = gwy_container_new();
+    GwyContainer *meta = gwy_container_new_in_construction();
 
     g_hash_table_foreach(hash, add_meta, meta);
 
@@ -1018,27 +1007,21 @@ get_meta(GHashTable *hash)
 }
 
 static void
-clone_field_meta(GwyContainer *container, GwyContainer *meta, gint id)
+clone_field_meta(GwyFile *file, GwyContainer *meta, gint id)
 {
-    GwyContainer *metacopy;
-
-    if (!gwy_container_get_n_items(meta))
-        return;
-
-    metacopy = gwy_container_duplicate(meta);
-    gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(id), metacopy);
+    if (gwy_container_get_n_items(meta))
+        gwy_file_pass_meta(file, GWY_FILE_IMAGE, id, gwy_container_copy(meta));
 }
 
 static void
-clone_lawn_meta(GwyContainer *container, GwyContainer *meta, GwyContainer *specmeta, gint id)
+clone_lawn_meta(GwyFile *file, GwyContainer *meta, GwyContainer *specmeta, gint id)
 {
-    GwyContainer *metacopy;
-
-    metacopy = gwy_container_duplicate(meta);
-    gwy_container_transfer(specmeta, metacopy, NULL, NULL, FALSE);
+    GwyContainer *metacopy = gwy_container_copy(meta);
+    gwy_container_transfer(specmeta, metacopy, NULL, NULL, FALSE, FALSE);
     if (gwy_container_get_n_items(metacopy))
-        gwy_container_set_object(container, gwy_app_get_lawn_meta_key_for_id(id), metacopy);
-    g_object_unref(metacopy);
+        gwy_file_pass_meta(file, GWY_FILE_CMAP, id, metacopy);
+    else
+        g_object_unref(metacopy);
 }
 
 static gboolean
@@ -1070,9 +1053,9 @@ clear_channels(GArray *channels)
 
     for (i = 0; i < channels->len; i++) {
         SensolyticsChannel *channel = &g_array_index(channels, SensolyticsChannel, i);
-        GWY_OBJECT_UNREF(channel->dfield);
-        GWY_OBJECT_UNREF(channel->lawn);
-        GWY_OBJECT_UNREF(channel->specmeta);
+        g_clear_object(&channel->dfield);
+        g_clear_object(&channel->lawn);
+        g_clear_object(&channel->specmeta);
         GWY_FREE(channel->data);
         GWY_FREE(channel->strings);
         GWY_FREE(channel->title);
