@@ -1,6 +1,6 @@
 /*
- *  $Id: shimadzu.c 26159 2024-02-06 14:01:34Z yeti-dn $
- *  Copyright (C) 2007 David Necas (Yeti).
+ *  $Id: shimadzu.c 28911 2025-11-24 18:27:42Z yeti-dn $
+ *  Copyright (C) 2007-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -64,14 +64,10 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/datafield.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/data-browser.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -85,43 +81,43 @@ enum {
 #define MAGIC_ASCII "ASCII:"
 #define MAGIC_ASCII_SIZE (sizeof(MAGIC_ASCII)-1)
 
-static gboolean      module_register      (void);
-static gint          shimadzu_detect      (const GwyFileDetectInfo *fileinfo,
-                                           gboolean only_name);
-static GwyContainer* shimadzu_load        (const gchar *filename,
-                                           GwyRunType mode,
-                                           GError **error);
-static GwyDataField* read_binary_data     (const gchar *buffer,
-                                           gsize size,
-                                           GHashTable *hash,
-                                           GError **error);
-static GwyDataField* read_text_data       (gchar *buffer,
-                                           GHashTable *hash,
-                                           GError **error);
-static GHashTable*   read_hash            (gchar *buffer,
-                                           gint *text_data_start,
-                                           GError **error);
-static gboolean      get_scales           (GHashTable *hash,
-                                           gboolean is_text,
-                                           gint *xres,
-                                           gint *yres,
-                                           gdouble *xreal,
-                                           gdouble *yreal,
-                                           gdouble *xoff,
-                                           gdouble *yoff,
-                                           GwySIUnit *si_unit_xy,
-                                           gdouble *zscale,
-                                           gdouble *zoff,
-                                           GwySIUnit *si_unit_z,
-                                           GError **error);
-static GwyContainer* shimadzu_get_metadata(GHashTable *hash);
+static gboolean      module_register (void);
+static gint          detect_file     (const GwyFileDetectInfo *fileinfo,
+                                      gboolean only_name);
+static GwyFile*      load_file       (const gchar *filename,
+                                      GwyRunModeFlags mode,
+                                      GError **error);
+static GwyField*     read_binary_data(const gchar *buffer,
+                                      gsize size,
+                                      GHashTable *hash,
+                                      GError **error);
+static GwyField*     read_text_data  (gchar *buffer,
+                                      GHashTable *hash,
+                                      GError **error);
+static GHashTable*   read_hash       (gchar *buffer,
+                                      gint *text_data_start,
+                                      GError **error);
+static gboolean      get_scales      (GHashTable *hash,
+                                      gboolean is_text,
+                                      gint *xres,
+                                      gint *yres,
+                                      gdouble *xreal,
+                                      gdouble *yreal,
+                                      gdouble *xoff,
+                                      gdouble *yoff,
+                                      GwyUnit *unit_xy,
+                                      gdouble *zscale,
+                                      gdouble *zoff,
+                                      GwyUnit *unit_z,
+                                      GError **error);
+static GwyContainer* get_metadata    (GHashTable *hash);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Shimadzu SPM data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.7",
+    "0.9",
     "David Nečas (Yeti)",
     "2007",
 };
@@ -133,17 +129,13 @@ module_register(void)
 {
     gwy_file_func_register("shimadzu",
                            N_("Shimadzu files"),
-                           (GwyFileDetectFunc)&shimadzu_detect,
-                           (GwyFileLoadFunc)&shimadzu_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-shimadzu_detect(const GwyFileDetectInfo *fileinfo,
-                gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     gint score = 0;
 
@@ -166,13 +158,13 @@ shimadzu_detect(const GwyFileDetectInfo *fileinfo,
     return score;
 }
 
-static GwyContainer*
-shimadzu_load(const gchar *filename,
-              G_GNUC_UNUSED GwyRunType mode,
-              GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *container = NULL;
-    GwyDataField *dfield = NULL;
+    GwyFile *file = NULL;
+    GwyField *dfield = NULL;
     GError *err = NULL;
     gchar *buffer = NULL;
     GHashTable *hash;
@@ -216,27 +208,27 @@ shimadzu_load(const gchar *filename,
     if (ok) {
         const gchar *title;
 
-        container = gwy_container_new();
-        gwy_container_pass_object(container, gwy_app_get_data_key_for_id(0), dfield);
-        gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(0), shimadzu_get_metadata(hash));
+        file = gwy_file_new_in_construction();
+        gwy_file_pass_image(file, 0, dfield);
+        gwy_file_pass_meta(file, GWY_FILE_IMAGE, 0, get_metadata(hash));
 
         title = g_hash_table_lookup(hash, "Channel");
         if (title && *title)
-            gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(0), title);
+            gwy_file_set_title(file, GWY_FILE_IMAGE, 0, title, FALSE);
         else
-            gwy_app_channel_title_fall_back(container, 0);
+            gwy_image_title_fall_back(file, 0);
 
-        gwy_file_channel_import_log_add(container, 0, NULL, filename);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
     }
 
     g_free(head);
     g_free(buffer);
     g_hash_table_destroy(hash);
 
-    return container;
+    return file;
 }
 
-static GwyDataField*
+static GwyField*
 read_binary_data(const gchar *buffer,
                  gsize size,
                  GHashTable *hash,
@@ -246,8 +238,8 @@ read_binary_data(const gchar *buffer,
     gint xres, yres;
     guint expected;
     gdouble xreal, yreal, zscale, xoff, yoff, zoff;
-    GwySIUnit *unitxy, *unitz;
-    GwyDataField *dfield = NULL;
+    GwyUnit *unitxy, *unitz;
+    GwyField *dfield = NULL;
     const gchar *s;
 
     if (!(s = g_hash_table_lookup(hash, "DataType"))) {
@@ -264,8 +256,8 @@ read_binary_data(const gchar *buffer,
         return NULL;
     }
 
-    unitxy = gwy_si_unit_new(NULL);
-    unitz = gwy_si_unit_new(NULL);
+    unitxy = gwy_unit_new(NULL);
+    unitz = gwy_unit_new(NULL);
 
     if (!get_scales(hash, FALSE, &xres, &yres, &xreal, &yreal, &xoff, &yoff, unitxy, &zscale, &zoff, unitz, error))
         goto fail;
@@ -274,15 +266,15 @@ read_binary_data(const gchar *buffer,
     if (err_SIZE_MISMATCH(error, expected, size, FALSE))
         goto fail;
 
-    dfield = gwy_data_field_new(xres, yres, xreal, yreal, FALSE);
-    gwy_data_field_set_xoffset(dfield, xoff);
-    gwy_data_field_set_yoffset(dfield, yoff);
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(dfield), unitxy);
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_z(dfield), unitz);
+    dfield = gwy_field_new(xres, yres, xreal, yreal, FALSE);
+    gwy_field_set_xoffset(dfield, xoff);
+    gwy_field_set_yoffset(dfield, yoff);
+    gwy_unit_assign(gwy_field_get_unit_xy(dfield), unitxy);
+    gwy_unit_assign(gwy_field_get_unit_z(dfield), unitz);
     gwy_convert_raw_data(buffer + HEADER_SIZE, xres*yres, 1,
                          rawtype, GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                         gwy_data_field_get_data(dfield), zscale, zoff);
-    gwy_data_field_invert(dfield, TRUE, FALSE, FALSE);
+                         gwy_field_get_data(dfield), zscale, zoff);
+    gwy_field_flip(dfield, FALSE, TRUE);
 
 fail:
     g_object_unref(unitxy);
@@ -290,7 +282,7 @@ fail:
     return dfield;
 }
 
-static GwyDataField*
+static GwyField*
 read_text_data(gchar *buffer,
                GHashTable *hash,
                GError **error)
@@ -298,32 +290,32 @@ read_text_data(gchar *buffer,
     const gchar *p;
     gint xres, yres, power10;
     gdouble xreal, yreal, zscale, xoff, yoff, zoff;
-    GwySIUnit *unitxy, *unitz;
-    GwyDataField *dfield = NULL;
+    GwyUnit *unitxy, *unitz;
+    GwyField *dfield = NULL;
     GError *err = NULL;
 
-    unitxy = gwy_si_unit_new(NULL);
-    unitz = gwy_si_unit_new(NULL);
+    unitxy = gwy_unit_new(NULL);
+    unitz = gwy_unit_new(NULL);
 
     if (!get_scales(hash, TRUE, &xres, &yres, &xreal, &yreal, &xoff, &yoff, unitxy, &zscale, &zoff, unitz, error))
         goto fail;
 
-    dfield = gwy_data_field_new(xres, yres, xreal, yreal, FALSE);
-    gwy_data_field_set_xoffset(dfield, xoff);
-    gwy_data_field_set_yoffset(dfield, yoff);
+    dfield = gwy_field_new(xres, yres, xreal, yreal, FALSE);
+    gwy_field_set_xoffset(dfield, xoff);
+    gwy_field_set_yoffset(dfield, yoff);
 
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(dfield), unitxy);
+    gwy_unit_assign(gwy_field_get_unit_xy(dfield), unitxy);
     p = g_hash_table_lookup(hash, "DATA Unit");
-    gwy_si_unit_set_from_string_parse(gwy_data_field_get_si_unit_z(dfield), p, &power10);
-    zscale = pow10(power10);
+    power10 = gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), p);
+    zscale = gwy_exp10(power10);
 
     g_strdelimit(buffer, ",;", ' ');
-    if (!gwy_parse_doubles(buffer, gwy_data_field_get_data(dfield), 0, &yres, &xres, NULL, &err)) {
+    if (!gwy_parse_doubles(buffer, gwy_field_get_data(dfield), 0, &yres, &xres, NULL, &err)) {
         err_PARSE_DOUBLES(error, &err);
-        GWY_OBJECT_UNREF(dfield);
+        g_clear_object(&dfield);
         goto fail;
     }
-    gwy_data_field_multiply(dfield, zscale);
+    gwy_field_multiply(dfield, zscale);
 
 fail:
     g_object_unref(unitxy);
@@ -452,13 +444,13 @@ get_scales(GHashTable *hash,
            gint *xres, gint *yres,
            gdouble *xreal, gdouble *yreal,
            gdouble *xoff, gdouble *yoff,
-           GwySIUnit *si_unit_xy,
+           GwyUnit *unit_xy,
            gdouble *zscale,
            gdouble *zoff,
-           GwySIUnit *si_unit_z,
+           GwyUnit *unit_z,
            GError **error)
 {
-    GwySIUnit *unit;
+    GwyUnit *unit;
     gint power10, zp;
     gchar *p;
     gboolean has_unit = FALSE;
@@ -477,20 +469,20 @@ get_scales(GHashTable *hash,
     if (err_DIMENSION(error, *yres))
         return FALSE;
 
-    unit = gwy_si_unit_new(NULL);
+    unit = gwy_unit_new(NULL);
 
     p = g_hash_table_lookup(hash, "SCANNING PARAMS::SizeX");
     *xreal = g_ascii_strtod(p, &p);
     sanitise_real_size(xreal, "x size");
-    gwy_si_unit_set_from_string_parse(si_unit_xy, p, &power10);
-    *xreal *= pow10(power10);
+    power10 = gwy_unit_set_from_string(unit_xy, p);
+    *xreal *= gwy_exp10(power10);
 
     p = g_hash_table_lookup(hash, "SCANNING PARAMS::SizeY");
     *yreal = g_ascii_strtod(p, &p);
     sanitise_real_size(yreal, "y size");
-    gwy_si_unit_set_from_string_parse(unit, p, &power10);
-    *yreal *= pow10(power10);
-    if (!gwy_si_unit_equal(unit, si_unit_xy)) {
+    power10 = gwy_unit_set_from_string(unit, p);
+    *yreal *= gwy_exp10(power10);
+    if (!gwy_unit_equal(unit, unit_xy)) {
         g_warning("X and Y units differ, using X");
     }
 
@@ -502,16 +494,16 @@ get_scales(GHashTable *hash,
     p = g_hash_table_lookup(hash, "SCANNING PARAMS::SizeZ");
     *zscale = g_ascii_strtod(p, &p);
     *zoff = 0.0;
-    gwy_si_unit_set_from_string_parse(si_unit_z, p, &power10);
-    *zscale *= pow10(power10)/zp;
+    power10 = gwy_unit_set_from_string(unit_z, p);
+    *zscale *= gwy_exp10(power10)/zp;
     /* XXX: Version 4 can have UNIT section that takes precedence.  The Conv factor may not be enough.  Apparently,
      * binary phase data need subtracting 180 deg because data are unsinged.  Also, all this is ignored by the text
      * data reader which just reads DATA Unit. Bite me. */
     if ((p = g_hash_table_lookup(hash, "UNIT::Unit"))) {
         const gchar *s = g_hash_table_lookup(hash, "UNIT::Name");
         has_unit = TRUE;
-        gwy_si_unit_set_from_string_parse(si_unit_z, p, &power10);
-        *zscale *= pow10(power10);
+        power10 = gwy_unit_set_from_string(unit_z, p);
+        *zscale *= gwy_exp10(power10);
         if ((p = g_hash_table_lookup(hash, "UNIT::Conv")))
             *zscale *= g_ascii_strtod(p, NULL);
 
@@ -523,9 +515,9 @@ get_scales(GHashTable *hash,
     *xoff = 0.0;
     if ((p = g_hash_table_lookup(hash, "SCANNING PARAMS::OffsetX"))) {
         *xoff = g_ascii_strtod(p, &p);
-        gwy_si_unit_set_from_string_parse(unit, p, &power10);
-        if (gwy_si_unit_equal(unit, si_unit_xy))
-            *xoff *= pow10(power10);
+        power10 = gwy_unit_set_from_string(unit, p);
+        if (gwy_unit_equal(unit, unit_xy))
+            *xoff *= gwy_exp10(power10);
         else {
             g_warning("X offset units differ from X size units, ignoring.");
             *xoff = 0.0;
@@ -535,9 +527,9 @@ get_scales(GHashTable *hash,
     *yoff = 0.0;
     if ((p = g_hash_table_lookup(hash, "SCANNING PARAMS::OffsetY"))) {
         *yoff = g_ascii_strtod(p, &p);
-        gwy_si_unit_set_from_string_parse(unit, p, &power10);
-        if (gwy_si_unit_equal(unit, si_unit_xy))
-            *yoff *= pow10(power10);
+        power10 = gwy_unit_set_from_string(unit, p);
+        if (gwy_unit_equal(unit, unit_xy))
+            *yoff *= gwy_exp10(power10);
         else {
             g_warning("Y offset units differ from Y size units, ignoring.");
             *yoff = 0.0;
@@ -549,9 +541,9 @@ get_scales(GHashTable *hash,
     if (!has_unit) {
         if ((p = g_hash_table_lookup(hash, "SCANNING PARAMS::OffsetZ"))) {
             *zoff = g_ascii_strtod(p, &p);
-            gwy_si_unit_set_from_string_parse(unit, p, &power10);
-            if (gwy_si_unit_equal(unit, si_unit_z))
-                *zoff *= pow10(power10);
+            power10 = gwy_unit_set_from_string(unit, p);
+            if (gwy_unit_equal(unit, unit_z))
+                *zoff *= gwy_exp10(power10);
             else {
                 g_warning("Z offset units differ from Z size units, ignoring.");
                 *zoff = 0.0;
@@ -565,9 +557,7 @@ get_scales(GHashTable *hash,
 }
 
 static void
-add_metadata(gpointer hkey,
-             gpointer hvalue,
-             gpointer user_data)
+add_metadata(gpointer hkey, gpointer hvalue, gpointer user_data)
 {
     const gchar *key = (const gchar*)hkey, *value = (const gchar*)hvalue;
     GwyContainer *meta = (GwyContainer*)user_data;
@@ -577,9 +567,9 @@ add_metadata(gpointer hkey,
 }
 
 static GwyContainer*
-shimadzu_get_metadata(GHashTable *hash)
+get_metadata(GHashTable *hash)
 {
-    GwyContainer *meta = gwy_container_new();
+    GwyContainer *meta = gwy_container_new_in_construction();
     g_hash_table_foreach(hash, add_metadata, meta);
     return meta;
 }
