@@ -1,9 +1,12 @@
 /*
- *  $Id: sicmfile.c 26163 2024-02-06 16:56:55Z yeti-dn $
+ *  $Id: sicmfile.c 28848 2025-11-10 18:54:08Z yeti-dn $
  *  IonScope SICM file format importer
  *  Copyright (C) 2008-2010 Matthew Caldwell.
  *  E-mail: m.caldwell@ucl.ac.uk
  *
+ *  Copyright (C) 2025 David Necas (Yeti).
+ *  E-mail: yeti@gwyddion.net
+ *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
  *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
  *  later version.
@@ -33,22 +36,12 @@
  * Read
  **/
 
-/*--------------------------------------------------------------------------
-  Dependencies
-----------------------------------------------------------------------------*/
-
 #include "config.h"
-#include <libgwyddion/gwymath.h>
-#include <libprocess/stats.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils-file.h>
+#include <glib/gi18n-lib.h>
+#include <gwy.h>
 
 #include "err.h"
 
-/*--------------------------------------------------------------------------
-  Constants
-----------------------------------------------------------------------------*/
-
 #define EXTENSION ".img"
 
 enum
@@ -57,10 +50,6 @@ enum
     HEADER_SIZE = 830
 };
 
-/*--------------------------------------------------------------------------
-  Types
-----------------------------------------------------------------------------*/
-
 /* Structure to hold the file header data */
 typedef struct _SICMImage
 {
@@ -169,69 +158,51 @@ typedef struct _SICMImage
 }
 SICMImage;
 
-/*--------------------------------------------------------------------------
-  Prototypes
-----------------------------------------------------------------------------*/
-
-static gboolean      module_register(void);
-static gint          sicm_detect    (const GwyFileDetectInfo *fileinfo,
-                                     gboolean only_name);
-static GwyContainer* sicm_load      (const gchar *filename,
-                                     GwyRunType mode,
-                                     GError **error);
-static void          set_string_meta(GwyContainer *meta,
-                                     const gchar *name,
-                                     const gchar *latin1value);
-static void          set_double_meta(GwyContainer *meta,
-                                     const gchar *name,
-                                     const gchar *unit,
-                                     gdouble value);
-static void          set_int_meta   (GwyContainer *meta,
-                                     const gchar *name,
-                                     gint value);
-static void          set_yesno_meta (GwyContainer *meta,
-                                     const gchar *name,
-                                     gboolean value);
-
-/*--------------------------------------------------------------------------
-  Implementation
-----------------------------------------------------------------------------*/
+static gboolean module_register(void);
+static gint     detect_file    (const GwyFileDetectInfo *fileinfo,
+                                gboolean only_name);
+static GwyFile* load_file      (const gchar *filename,
+                                GwyRunModeFlags mode,
+                                GError **error);
+static void     set_string_meta(GwyContainer *meta,
+                                const gchar *name,
+                                const gchar *latin1value);
+static void     set_double_meta(GwyContainer *meta,
+                                const gchar *name,
+                                const gchar *unit,
+                                gdouble value);
+static void     set_int_meta   (GwyContainer *meta,
+                                const gchar *name,
+                                gint value);
+static void     set_yesno_meta (GwyContainer *meta,
+                                const gchar *name,
+                                gboolean value);
 
 /* module details */
-static GwyModuleInfo module_info =
-{
+static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports IonScope SICM data files."),
     "Matthew Caldwell <m.caldwell@ucl.ac.uk>",
-    "1.3",
+    "2.0",
     "Matthew Caldwell",
     "2007-2010",
 };
+
 GWY_MODULE_QUERY2(module_info, sicmfile)
 
-/*--------------------------------------------------------------------------*/
-
-/* announce ourselves to gwyddion */
 static gboolean
 module_register(void)
 {
     gwy_file_func_register("sicm",
                            N_("IonScope SICM files (.img)"),
-                           (GwyFileDetectFunc) &sicm_detect,
-                           (GwyFileLoadFunc) &sicm_load,
-                           NULL,
-                           NULL);
-
+                           detect_file, load_file, NULL, NULL);
     return TRUE;
 }
 
-/*--------------------------------------------------------------------------*/
-
 /* check a candidate file for likely SICMness */
 static gint
-sicm_detect(const GwyFileDetectInfo *fileinfo,
-            gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     /* name isn't much of a test since every second SPM format -- and various non-SPM ones too -- use a .IMG
      * extension, but we have nothing else to go on */
@@ -254,17 +225,16 @@ sicm_detect(const GwyFileDetectInfo *fileinfo,
     return 0;
 }
 
-/*--------------------------------------------------------------------------*/
-
 /* load file data. */
-static GwyContainer*
-sicm_load(const gchar *filename,
-          G_GNUC_UNUSED GwyRunType mode,
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
           GError **error)
 {
     SICMImage sicm;
-    GwyContainer *container = NULL, *meta = NULL;
-    GwyDataField *dfield;
+    GwyFile *file = NULL;
+    GwyContainer *meta = NULL;
+    GwyField *dfield;
     guchar *buffer;
     const guchar *p, *unit_name;
     GError *err = NULL;
@@ -416,9 +386,9 @@ sicm_load(const gchar *filename,
     sicm.posStr[40] = 0;
     p += 41;
 
-    dfield = gwy_data_field_new(sicm.xdim, sicm.ydim,
-                                sicm.scanSize * 1e-8, sicm.scanSize * 1e-8,
-                                FALSE);
+    dfield = gwy_field_new(sicm.xdim, sicm.ydim,
+                           sicm.scanSize * 1e-8, sicm.scanSize * 1e-8,
+                           FALSE);
 
     /* scale factor depends on the channel data type, which we get from the mode string */
     if (sicm.modeStr[0] == 'C') {
@@ -443,21 +413,20 @@ sicm_load(const gchar *filename,
 
     /* image data is stored bottom-up, so reverse order of rows */
     gwy_convert_raw_data(p, sicm.xdim*sicm.ydim, 1, GWY_RAW_DATA_SINT16, GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                         gwy_data_field_get_data(dfield), scaling, 0.0);
-    gwy_data_field_invert(dfield, TRUE, FALSE, FALSE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), unit_name);
+                         gwy_field_get_data(dfield), scaling, 0.0);
+    gwy_field_flip(dfield, FALSE, TRUE);
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
+    gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), unit_name);
 
-    /* build the data container */
-    container = gwy_container_new();
-    gwy_container_pass_object(container, gwy_app_get_data_key_for_id(0), dfield);
-    gwy_container_set_string(container, gwy_app_get_data_title_key_for_id(0),
-                             gwy_convert_to_utf8(sicm.modeStr, -1, "ISO-8859-1"));
+    /* build the data file */
+    file = gwy_file_new_in_construction();
+    gwy_file_pass_image(file, 0, dfield);
+    gwy_file_pass_title(file, GWY_FILE_IMAGE, 0, gwy_convert_to_utf8(sicm.modeStr, -1, "ISO-8859-1"));
 
     /* add pretty much all metadata from the header a bunch of this doesn't seem to be used meaningfully; we prefix
      * probably useless or placeholder fields with a tilde so they sort to the bottom of the browser -- these should
      * probably be omitted entirely when we're sure they're not needed */
-    meta = gwy_container_new();
+    meta = gwy_container_new_in_construction();
 
     set_string_meta(meta, "Title", sicm.title);
     set_string_meta(meta, "Comment", sicm.comment);
@@ -557,13 +526,12 @@ sicm_load(const gchar *filename,
                                                      sicm.resThresholds[6], sicm.resThresholds[7]));
     set_int_meta(meta, "* Resolution Levels Used", sicm.numResLevels);
 
-    gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(0), meta);
-
-    gwy_file_channel_import_log_add(container, 0, NULL, filename);
+    gwy_file_pass_meta(file, GWY_FILE_IMAGE, 0, meta);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
 
     gwy_file_abandon_contents(buffer, size, NULL);
 
-    return container;
+    return file;
 }
 
 /* Metadata helpers. */
