@@ -1,6 +1,6 @@
 /*
- *  $Id: sis.c 26462 2024-08-04 06:43:12Z yeti-dn $
- *  Copyright (C) 2004 David Necas (Yeti), Petr Klapetek.
+ *  $Id: sis.c 28789 2025-11-04 17:14:03Z yeti-dn $
+ *  Copyright (C) 2004-2025 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -41,14 +41,9 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/datafield.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 
 #include "err.h"
 #include "get.h"
@@ -141,7 +136,7 @@ typedef struct {
 
 typedef struct {
     GHashTable *hash;
-    GwyDataField *data_field;
+    GwyField *field;
 } SISData;
 
 static const SISProcessingStep processing_steps[] = {
@@ -254,31 +249,31 @@ static const SISParameter sis_parameters[] = {
     { 144, G_TYPE_STRING, "Name of data in channel 8", NULL },
 };
 
-static gboolean       module_register     (void);
-static gint           sis_detect          (const GwyFileDetectInfo *fileinfo,
-                                           gboolean only_name);
-static GwyContainer*  sis_load            (const gchar *filename,
-                                           GwyRunType mode,
-                                           GError **error);
-static GwyDataField*  extract_data        (SISFile *sisfile,
-                                           guint ch,
-                                           guint im,
-                                           GError **error);
-static void           add_metadata        (SISFile *sisfile,
-                                           gint id,
-                                           guint ch,
-                                           GwyContainer *data);
-static gboolean       sis_real_load       (const guchar *buffer,
-                                           guint size,
-                                           SISFile *sisfile,
-                                           GError **error);
+static gboolean  module_register(void);
+static gint      detect_file    (const GwyFileDetectInfo *fileinfo,
+                                 gboolean only_name);
+static GwyFile*  load_file      (const gchar *filename,
+                                 GwyRunModeFlags mode,
+                                 GError **error);
+static GwyField* extract_data   (SISFile *sisfile,
+                                 guint ch,
+                                 guint im,
+                                 GError **error);
+static void      add_metadata   (SISFile *sisfile,
+                                 gint id,
+                                 guint ch,
+                                 GwyFile *file);
+static gboolean  sis_real_load  (const guchar *buffer,
+                                 guint size,
+                                 SISFile *sisfile,
+                                 GError **error);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports SIS (Surface Imaging Systems) data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.18",
+    "0.19",
     "David Nečas (Yeti) & Petr Klapetek",
     "2004",
 };
@@ -290,17 +285,13 @@ module_register(void)
 {
     gwy_file_func_register("sis",
                            N_("SIS files (.sis)"),
-                           (GwyFileDetectFunc)&sis_detect,
-                           (GwyFileLoadFunc)&sis_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-sis_detect(const GwyFileDetectInfo *fileinfo,
-           gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     gint score = 0;
 
@@ -314,16 +305,16 @@ sis_detect(const GwyFileDetectInfo *fileinfo,
     return score;
 }
 
-static GwyContainer*
-sis_load(const gchar *filename,
-         G_GNUC_UNUSED GwyRunType mode,
-         GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
     guchar *buffer;
     gsize size;
     GError *err = NULL;
-    GwyDataField *dfield = NULL;
-    GwyContainer *data = NULL;
+    GwyField *dfield = NULL;
+    GwyFile *file = NULL;
     SISFile sisfile;
     SISImage *image;
     GString *key;
@@ -336,10 +327,10 @@ sis_load(const gchar *filename,
         return NULL;
     }
     n = 0;
-    gwy_clear(&sisfile, 1);
+    gwy_clear1(sisfile);
     sisfile.params = g_hash_table_new_full(g_direct_hash, g_direct_equal, NULL, g_free);
     if (sis_real_load(buffer, size, &sisfile, error)) {
-        data = gwy_container_new();
+        file = gwy_file_new_in_construction();
         key = g_string_new(NULL);
         for (i = 0; i < sisfile.nchannels; i++) {
             for (j = 0; j < sisfile.channels[i].nimages; j++) {
@@ -350,7 +341,7 @@ sis_load(const gchar *filename,
                 dfield = extract_data(&sisfile, i, j, &err);
                 if (dfield) {
                     g_clear_error(&err);
-                    gwy_container_pass_object(data, gwy_app_get_data_key_for_id(n), dfield);
+                    gwy_file_pass_image(file, n, dfield);
                     s = gwy_enuml_to_string(sisfile.channels[i].data_type,
                                             "Topography", 1,
                                             "Field Contrast", 2,
@@ -367,9 +358,9 @@ sis_load(const gchar *filename,
                                             "User", 13,
                                             NULL);
                     if (s)
-                        gwy_container_set_const_string(data, gwy_app_get_data_title_key_for_id(n), s);
-                    add_metadata(&sisfile, n, i, data);
-                    gwy_file_channel_import_log_add(data, n, NULL, filename);
+                        gwy_file_set_title(file, GWY_FILE_IMAGE, n, s, FALSE);
+                    add_metadata(&sisfile, n, i, file);
+                    gwy_log_add_import(file, GWY_FILE_IMAGE, n, NULL, filename);
                     n++;
                 }
             }
@@ -381,7 +372,7 @@ sis_load(const gchar *filename,
                 g_propagate_error(error, err);
             else
                 err_NO_DATA(error);
-            GWY_OBJECT_UNREF(data);
+            g_clear_object(&file);
         }
     }
 
@@ -391,10 +382,10 @@ sis_load(const gchar *filename,
         g_free(sisfile.channels[i].images);
     g_free(sisfile.channels);
 
-    return data;
+    return file;
 }
 
-static GwyDataField*
+static GwyField*
 extract_data(SISFile *sisfile,
              guint ch,
              guint im,
@@ -404,7 +395,7 @@ extract_data(SISFile *sisfile,
         0, GWY_RAW_DATA_UINT8, GWY_RAW_DATA_UINT16, 0, GWY_RAW_DATA_UINT32,
     };
 
-    GwyDataField *dfield;
+    GwyField *dfield;
     SISChannel *channel;
     SISImage *image;
     gdouble xreal, yreal, zreal;
@@ -430,16 +421,16 @@ extract_data(SISFile *sisfile,
     sanitise_real_size(&xreal, "x size");
     sanitise_real_size(&yreal, "y size");
 
-    dfield = gwy_data_field_new(image->width, image->height, xreal, yreal, FALSE);
+    dfield = gwy_field_new(image->width, image->height, xreal, yreal, FALSE);
 
-    d = gwy_data_field_get_data(dfield);
+    d = gwy_field_get_data(dfield);
     n = image->width * image->height;
     gwy_convert_raw_data(image->image_data, n, 1, rawtypes[image->bpp], GWY_BYTE_ORDER_LITTLE_ENDIAN,
                          d, zreal/(gwy_powi(256.0, image->bpp) - 1.0), 0.0);
 
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
     if (channel->data_type == SIS_DATA_TYPE_TOPOGRAPHY)
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), "m");
+        gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), "m");
 
     return dfield;
 }
@@ -448,7 +439,7 @@ static void
 add_metadata(SISFile *sisfile,
              gint id,
              guint ch,
-             GwyContainer *data)
+             GwyFile *file)
 {
     static const guint good_metadata[] = {
         0, 1, 9, 10, 12, 13, 14, 15, 16, 18, 20, 21, 22, 23, 24, 25, 26, 27,
@@ -456,10 +447,11 @@ add_metadata(SISFile *sisfile,
     GwyContainer *meta;
     SISChannel *channel;
     guint i, j;
-    guchar *value, *key;
+    guchar *value;
+    const gchar *s;
     gpointer *p;
 
-    meta = gwy_container_new();
+    meta = gwy_container_new_in_construction();
     channel = sisfile->channels + ch;
     for (i = 0; i < G_N_ELEMENTS(good_metadata); i++) {
         for (j = 0; j < G_N_ELEMENTS(sis_parameters); j++) {
@@ -494,52 +486,49 @@ add_metadata(SISFile *sisfile,
 
     /* Special metadata */
     if ((p = g_hash_table_lookup(sisfile->params, GUINT_TO_POINTER(28)))) {
-        value = g_strdup(gwy_enuml_to_string(*(guint*)p,
-                                             "Gray", 0,
-                                             "Sky", 1,
-                                             "Red", 2,
-                                             "Green", 3,
-                                             "Blue", 4,
-                                             "Rainbow", 5,
-                                             NULL));
-        if (value) {
-            key = g_strdup_printf("/%d/base/palette", id);
-            gwy_container_set_string_by_name(data, key, value);
-            g_free(key);
-        }
+        s = gwy_enuml_to_string(*(guint*)p,
+                                "Gray", 0,
+                                "Sky", 1,
+                                "Red", 2,
+                                "Green", 3,
+                                "Blue", 4,
+                                "Rainbow", 5,
+                                NULL);
+        if (s)
+            gwy_file_set_palette(file, GWY_FILE_IMAGE, id, s);
     }
 
     if ((p = g_hash_table_lookup(sisfile->params, GUINT_TO_POINTER(6)))) {
-        value = g_strdup(gwy_enuml_to_string(*(guint*)p,
-                                             "Contact", 1,
-                                             "Non contact", 2,
-                                             NULL));
-        if (value)
-            gwy_container_set_string_by_name(meta, "Aqusition type", value);
+        s = gwy_enuml_to_string(*(guint*)p,
+                                "Contact", 1,
+                                "Non contact", 2,
+                                NULL);
+        if (s)
+            gwy_container_set_const_string_by_name(meta, "Aqusition type", s);
     }
 
-    value = g_strdup(gwy_enuml_to_string(channel->signal_source,
-                                         "Feedback", 1,
-                                         "ZSensor", 2,
-                                         "Interferometer", 3,
-                                         "Field", 4,
-                                         "NC Amplitude", 5,
-                                         "NC Phase", 6,
-                                         "FM Frequency", 7,
-                                         "LOC amplitude", 8,
-                                         "LOC phase", 9,
-                                         "PM Channel 1", 10,
-                                         "PM Channel 2", 11,
-                                         "PM Feedback", 12,
-                                         "Capacity", 13,
-                                         "LOC Software Amplitude", 14,
-                                         "LOC Software Phase", 15,
-                                         "User", 16,
-                                         NULL));
-    if (value)
-        gwy_container_set_string_by_name(meta, "Signal source", value);
+    s = gwy_enuml_to_string(channel->signal_source,
+                            "Feedback", 1,
+                            "ZSensor", 2,
+                            "Interferometer", 3,
+                            "Field", 4,
+                            "NC Amplitude", 5,
+                            "NC Phase", 6,
+                            "FM Frequency", 7,
+                            "LOC amplitude", 8,
+                            "LOC phase", 9,
+                            "PM Channel 1", 10,
+                            "PM Channel 2", 11,
+                            "PM Feedback", 12,
+                            "Capacity", 13,
+                            "LOC Software Amplitude", 14,
+                            "LOC Software Phase", 15,
+                            "User", 16,
+                            NULL);
+    if (s)
+        gwy_container_set_const_string_by_name(meta, "Signal source", s);
 
-    gwy_container_pass_object(data, gwy_app_get_data_meta_key_for_id(id), meta);
+    gwy_file_pass_meta(file, GWY_FILE_IMAGE, id, meta);
 }
 
 /* FIXME: what a mess. And in reality, the files look different than the specs say anyway... */
