@@ -1,6 +1,6 @@
 /*
- *  $Id: spcfile.c 27922 2025-05-07 14:58:05Z yeti-dn $
- *  Copyright (C) 2017 Daniil Bratashov (dn2010), David Necas (Yeti)..
+ *  $Id: spcfile.c 28772 2025-11-03 18:02:12Z yeti-dn $
+ *  Copyright (C) 2017-2025 Daniil Bratashov (dn2010), David Necas (Yeti).
  *
  *  E-mail: dn2010@gwyddion.net
  *
@@ -36,15 +36,10 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwydgets/gwygraphmodel.h>
-#include <libgwydgets/gwygraphbasics.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 
 #include "err.h"
 #include "get.h"
@@ -156,7 +151,7 @@ static const GwyEnum spc_xunits[] = {
     { "deg",     SPC_XUNITS_DEGREES,     },
     { "°F",      SPC_XUNITS_DEGREES_F,   },
     { "°C",      SPC_XUNITS_DEGREES_C,   },
-    { "°K",      SPC_XUNITS_DEGREES_K,   },
+    { "K",       SPC_XUNITS_DEGREES_K,   },
     { "pt",      SPC_XUNITS_POINTS,      },
     { "ms",      SPC_XUNITS_MILLISECS,   },
     { "µs",      SPC_XUNITS_MICROSECS,   },
@@ -188,7 +183,7 @@ static const GwyEnum spc_yunits[] = {
     { "dB",     SPC_YUNITS_YDECIBEL,      },
     { "°F",     SPC_YUNITS_YDEGREEF,      },
     { "°C",     SPC_YUNITS_YDEGREEC,      },
-    { "°K",     SPC_YUNITS_YDEGREEK,      },
+    { "K",      SPC_YUNITS_YDEGREEK,      },
     { "",       SPC_YUNITS_YINDEXREFLECT, },
     { "",       SPC_YUNITS_YEXTINCTIONCF, },
     { "",       SPC_YUNITS_YREAL,         },
@@ -256,24 +251,24 @@ typedef struct {
     gchar*   reserved; // 4
 } SPCSubHeader;
 
-static gboolean      module_register     (void);
-static gint          spc_detect          (const GwyFileDetectInfo *fileinfo,
-                                          gboolean only_name);
-static GwyContainer* spc_load            (const gchar *filename,
-                                          GwyRunType mode,
-                                          GError **error);
-static void          spc_read_main_header(const guchar *buffer,
-                                          SPCMainHeader *header,
-                                          GError **error);
-static void          spc_read_subheader  (const guchar *buffer,
-                                          SPCSubHeader *header);
+static gboolean module_register (void);
+static gint     detect_file     (const GwyFileDetectInfo *fileinfo,
+                                 gboolean only_name);
+static GwyFile* load_file       (const gchar *filename,
+                                 GwyRunModeFlags mode,
+                                 GError **error);
+static void     read_main_header(const guchar *buffer,
+                                 SPCMainHeader *header,
+                                 GError **error);
+static void     read_subheader  (const guchar *buffer,
+                                 SPCSubHeader *header);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Thermo Fisher SPC files."),
     "Daniil Bratashov <dn2010@gwyddion.net>",
-    "0.4",
+    "0.5",
     "Daniil Bratashov (dn2010), David Necas (Yeti)",
     "2018",
 };
@@ -285,17 +280,13 @@ module_register(void)
 {
     gwy_file_func_register("spcfile",
                            N_("Thermo Fisher SPC files"),
-                           (GwyFileDetectFunc)&spc_detect,
-                           (GwyFileLoadFunc)&spc_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-spc_detect(const GwyFileDetectInfo *fileinfo,
-           gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     gint score = 0;
 
@@ -320,12 +311,12 @@ spc_detect(const GwyFileDetectInfo *fileinfo,
     return score;
 }
 
-static GwyContainer*
-spc_load(const gchar *filename,
-         G_GNUC_UNUSED GwyRunType mode,
-         GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     guchar *buffer = NULL;
     gsize size = 0;
     gint remaining = 0;
@@ -333,7 +324,7 @@ spc_load(const gchar *filename,
     const guchar *p;
     SPCMainHeader *header = NULL;
     SPCSubHeader *subheader;
-    GwySIUnit *siunitx, *siunity;
+    GwyUnit *siunitx, *siunity;
     const gchar *unit;
     gdouble *xdata, *ydata;
     gdouble scale, xscale, yscale;
@@ -356,7 +347,7 @@ spc_load(const gchar *filename,
     remaining = size;
 
     header = g_new0(SPCMainHeader, 1);
-    spc_read_main_header(p, header, &err);
+    read_main_header(p, header, &err);
     p += 512;
     remaining -= 512;
     npoints = header->point_number;
@@ -371,20 +362,20 @@ spc_load(const gchar *filename,
         goto fail;
     }
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     xscale = 1.0;
 
     gwy_debug("x units = %d", header->x_units);
     unit = gwy_enum_to_string(header->x_units, spc_xunits, 32);
-    siunitx = gwy_si_unit_new_parse(unit, &power10x);
-    xscale = pow10(power10x);
+    siunitx = gwy_unit_new_parse(unit, &power10x);
+    xscale = gwy_exp10(power10x);
     if (xscale == 0.0)
         xscale = 1.0;
 
     gwy_debug("y units = %d", header->y_units);
     unit = gwy_enum_to_string(header->y_units, spc_yunits, 28);
-    siunity = gwy_si_unit_new_parse(unit, &power10y);
-    yscale = pow10(power10y);
+    siunity = gwy_unit_new_parse(unit, &power10y);
+    yscale = gwy_exp10(power10y);
     if (yscale == 0.0)
         yscale = 1.0;
 
@@ -402,7 +393,7 @@ spc_load(const gchar *filename,
     }
 
     subheader = g_new0(SPCSubHeader, 1);
-    spc_read_subheader(p, subheader);
+    read_subheader(p, subheader);
     p += 32;
     remaining -= 32;
     zres = npoints;
@@ -438,8 +429,8 @@ spc_load(const gchar *filename,
     }
 
     gmodel = g_object_new(GWY_TYPE_GRAPH_MODEL,
-                          "si-unit-x", siunitx,
-                          "si-unit-y", siunity,
+                          "unit-x", siunitx,
+                          "unit-y", siunity,
                           NULL);
 
     if (header->custom_axis_labels) {
@@ -463,7 +454,8 @@ spc_load(const gchar *filename,
     gwy_graph_curve_model_enforce_order(gcmodel);
     gwy_graph_model_add_curve(gmodel, gcmodel);
     g_object_unref(gcmodel);
-    gwy_container_pass_object_by_name(container, "/0/graph/graph/1", gmodel);
+    gwy_file_pass_graph(file, 0, gmodel);
+    gwy_log_add_import(file, GWY_FILE_GRAPH, 0, NULL, filename);
     g_free(subheader);
 
     g_object_unref(siunitx);
@@ -472,13 +464,13 @@ fail:
     g_free(header);
     gwy_file_abandon_contents(buffer, size, NULL);
 
-    return container;
+    return file;
 }
 
 static void
-spc_read_main_header(const guchar *buffer,
-                     SPCMainHeader *header,
-                     GError **error)
+read_main_header(const guchar *buffer,
+                 SPCMainHeader *header,
+                 GError **error)
 {
     guchar flags;
     const guchar *p;
@@ -550,8 +542,7 @@ spc_read_main_header(const guchar *buffer,
 }
 
 static void
-spc_read_subheader(const guchar *buffer,
-                   SPCSubHeader *header)
+read_subheader(const guchar *buffer, SPCSubHeader *header)
 {
     guchar flags;
     const guchar *p;
