@@ -1,6 +1,6 @@
 /*
- *  $Id: spml.c 26512 2024-08-15 17:41:43Z yeti-dn $
- *  Copyright (C) 2006-2022 David Necas (Yeti), Petr Klapetek.
+ *  $Id: spml.c 28911 2025-11-24 18:27:42Z yeti-dn $
+ *  Copyright (C) 2006-2025 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -23,6 +23,7 @@
 /* XXX: The typical length of a XML declaration is about 40 bytes.  So while there can be more stuff before <SPML than
  * 60 bytes, we have to find a compromise between generality and efficiency.  It's the SPML guys' fight, they should
  * have created a more easily detectable format... */
+
 /**
  * [FILE-MAGIC-FREEDESKTOP]
  * <mime-type type="application/x-spml-spm">
@@ -53,16 +54,11 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <libprocess/datafield.h>
+#include <gwy.h>
 
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwyutils.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 
 #include <libxml/xmlreader.h>
 #include "gwyzlib.h"
@@ -101,23 +97,24 @@ typedef struct {
     GList *datachannels;
 } DataChannelGroup;
 
-static gboolean      module_register(void);
-static gint          spml_detect    (const GwyFileDetectInfo * fileinfo,
-                                     gboolean only_name);
-static GwyContainer* spml_load      (const gchar *filename);
-static int           get_axis       (const gchar *filename,
-                                     const gchar *datachannel_name,
-                                     GArray **axes,
-                                     GArray **units,
-                                     GArray **names);
+static gboolean module_register(void);
+static gint     detect_file    (const GwyFileDetectInfo * fileinfo,
+                                gboolean only_name);
+static GwyFile* load_file      (const gchar *filename,
+                                GwyRunModeFlags mode,
+                                GError **error);
+static int      get_axis       (const gchar *filename,
+                                const gchar *datachannel_name,
+                                GArray **axes,
+                                GArray **units,
+                                GArray **names);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
-    N_("Loads SPML (Scanning Probe Microscopy Markup Language) "
-       "data files."),
+    N_("Loads SPML (Scanning Probe Microscopy Markup Language) data files."),
     "Jan Hořák <xhorak@gmail.com>",
-    "0.2",
+    "0.3",
     "Jan Hořák",
     "2006",
 };
@@ -129,13 +126,34 @@ module_register(void)
 {
     gwy_file_func_register("spml",
                            N_("SPML files (.xml)"),
-                           (GwyFileDetectFunc) &spml_detect,
-                           (GwyFileLoadFunc) &spml_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
     return TRUE;
 }
 
+static gint
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
+{
+    gint score = 0;
+
+    gwy_debug("%s: %s", fileinfo->name_lowercase, (only_name) ? "only_name" : "content");
+    if (only_name) {
+        if (g_str_has_suffix(fileinfo->name_lowercase, ".xml")) {
+            score += 50;
+            gwy_debug("Score += 50");
+        }
+    }
+    else {
+        if (fileinfo->head != NULL) {
+            gwy_debug("head not null");
+            if (strstr(fileinfo->head, "<SPML") != NULL) {
+                gwy_debug("Score += 100");
+                score += 100;
+            }
+        }
+    }
+    return score;
+}
+
 static void
 free_datachannel_group(DataChannelGroup *dc_group)
 {
@@ -304,7 +322,7 @@ decode_data(double **data, const xmlChar * input, gint data_format,
     if (coding == ASCII) {
         data_count = max_input_len;
         *data = gwy_parse_doubles((const gchar*)input, NULL, GWY_PARSE_DOUBLES_FREE_FORM,
-                                   &data_count, &ncols, NULL, NULL);
+                                  &data_count, &ncols, NULL, NULL);
         return data_count > 0;
     }
 
@@ -522,7 +540,6 @@ get_data(gboolean read_data_only, const gchar *filename, const gchar *datachanne
                 out_len = decode_data(data, value, format, coding, byte_order, max_input_len);
                 if (!out_len || !*data) {
                     g_warning("No input data available for datachannel '%s'.", datachannel_name);
-
                 }
                 else if (read_data_only) {
                     *dimensions = g_new(int, 1);
@@ -682,7 +699,6 @@ get_axis_datapoints(const gchar *filename, xmlNodePtr axis_node)
     return NULL;
 }
 
-
 static int
 get_axis(const gchar *filename, const gchar *datachannel_name,
          GArray **axes, GArray **units, GArray **names)
@@ -728,7 +744,6 @@ get_axis(const gchar *filename, const gchar *datachannel_name,
             if (channelReadMethodName) {
                 /* channelReadMethodName found, leave searching */
                 break;
-
             }
             sub_node = get_next_node_ptr(sub_node, "DataChannel");
         }
@@ -872,18 +887,21 @@ get_dimensions(GArray * axes)
         gwy_debug("Start: %e End: %e Width: %e\n", start, end, width);
         */
     }
-    return out_array;
 
+    return out_array;
 }
 
-static GwyContainer *
-spml_load(const gchar *filename)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          /* Print a warning. The error is unused – and that is wrong! The file import obviously can fail and should
+          * return an error in such case. */
+          GError **error)
 {
-    GwyContainer *object = NULL;
     gdouble *gwy_data;
-    GwySIUnit *siunit;
+    GwyUnit *siunit;
     int i, j, *dimensions, scattered, channel_number = 0;
-    GwyDataField *dfield = NULL;
+    GwyField *dfield = NULL;
     char *channel_name, *unit;
     GArray *axes, *axes_units, *names, *real_width_height;
     double x_dimen, y_dimen, *data = NULL;
@@ -894,6 +912,8 @@ spml_load(const gchar *filename)
 
     gwy_debug("file = %s", filename);
 
+    GwyFile *file = gwy_file_new_in_construction();
+
     /* get list of datagroups and datachannels */
     l = list_of_datagroups = get_list_of_datachannels(filename);
     while (l) {
@@ -926,27 +946,27 @@ spml_load(const gchar *filename)
             y_dimen = g_array_index(real_width_height, double, 1);
 
             /* parse unit and return power of 10 according to unit */
-            siunit = gwy_si_unit_new_parse(g_array_index(axes_units, xmlChar*, 0), &x_power10);
+            siunit = gwy_unit_new_parse(g_array_index(axes_units, xmlChar*, 0), &x_power10);
             g_object_unref(siunit);
 
-            siunit = gwy_si_unit_new_parse(g_array_index(axes_units, xmlChar*, 1), &y_power10);
-            /* create and allocate datafield of given dimensions and given physical
+            siunit = gwy_unit_new_parse(g_array_index(axes_units, xmlChar*, 1), &y_power10);
+            /* create and allocate field of given dimensions and given physical
                dimensions */
             if (rotate == 90) {
-                dfield = gwy_data_field_new(dimensions[1], dimensions[0],
-                                            y_dimen * pow10(y_power10), x_dimen * pow10(x_power10),
-                                            FALSE);
+                dfield = gwy_field_new(dimensions[1], dimensions[0],
+                                       y_dimen * gwy_exp10(y_power10), x_dimen * gwy_exp10(x_power10),
+                                       FALSE);
             }
             else {
-                dfield = gwy_data_field_new(dimensions[0], dimensions[1],
-                                            x_dimen * pow10(x_power10), y_dimen * pow10(y_power10),
-                                            FALSE);
+                dfield = gwy_field_new(dimensions[0], dimensions[1],
+                                       x_dimen * gwy_exp10(x_power10), y_dimen * gwy_exp10(y_power10),
+                                       FALSE);
             }
-            gwy_debug("X real width: %f", x_dimen * pow10(x_power10));
+            gwy_debug("X real width: %f", x_dimen * gwy_exp10(x_power10));
             gwy_debug("X real_width_height: %f", x_dimen);
-            gwy_data = gwy_data_field_get_data(dfield);
+            gwy_data = gwy_field_get_data(dfield);
             /* copy raw array of doubles extracted from spml file to Gwyddion's
-               datafield
+               field
                rotate -90 degrees: */
             if (rotate == 90) {
                 for (i = 0; i < dimensions[0]; i++) {
@@ -955,39 +975,32 @@ spml_load(const gchar *filename)
                     }
                 }
             } else {
-                memcpy(gwy_data, data,
-                       dimensions[0] * dimensions[1] * sizeof(double));
+                memcpy(gwy_data, data, dimensions[0] * dimensions[1] * sizeof(double));
             }
             gwy_debug("Dimensions: %dx%d", dimensions[0], dimensions[1]);
-            gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(dfield), siunit);
+            gwy_unit_assign(gwy_field_get_unit_xy(dfield), siunit);
             g_object_unref(siunit);
 
             /* set unit for Z axis */
-            gwy_si_unit_set_from_string_parse(gwy_data_field_get_si_unit_z(dfield), unit, &z_power10);
-            gwy_data_field_multiply(dfield, pow10(z_power10));
+            z_power10 = gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), unit);
+            gwy_field_multiply(dfield, gwy_exp10(z_power10));
 
             /* set offset to match axes */
-            siunit = gwy_si_unit_new_parse(g_array_index(axes_units, xmlChar*, 0), &z_power10);
+            siunit = gwy_unit_new_parse(g_array_index(axes_units, xmlChar*, 0), &z_power10);
             g_object_unref(siunit);
-            gwy_data_field_set_xoffset(dfield,
-                                       pow10(z_power10)*g_array_index(g_array_index(axes, GArray*, 0), double, 0));
+            gwy_field_set_xoffset(dfield,
+                                  gwy_exp10(z_power10)*g_array_index(g_array_index(axes, GArray*, 0), double, 0));
 
-            siunit = gwy_si_unit_new_parse(g_array_index(axes_units, xmlChar*, 1), &z_power10);
+            siunit = gwy_unit_new_parse(g_array_index(axes_units, xmlChar*, 1), &z_power10);
             g_object_unref(siunit);
-            gwy_data_field_set_yoffset(dfield,
-                                       pow10(z_power10)*g_array_index(g_array_index(axes, GArray*, 1), double, 0));
+            gwy_field_set_yoffset(dfield,
+                                  gwy_exp10(z_power10)*g_array_index(g_array_index(axes, GArray*, 1), double, 0));
 
-            if (!object) {
-                /* create gwyddion container */
-                object = gwy_container_new();
-            }
             /* put datachannel into container */
-            gwy_container_pass_object(object, gwy_app_get_data_key_for_id(channel_number), dfield);
-
+            gwy_file_pass_image(file, channel_number, dfield);
             /* set name of datachannel to store in container */
-            gwy_container_set_const_string(object, gwy_app_get_data_title_key_for_id(channel_number), channel_name);
-
-            gwy_file_channel_import_log_add(object, channel_number, NULL, filename);
+            gwy_file_set_title(file, GWY_FILE_IMAGE, channel_number, channel_name, FALSE);
+            gwy_log_add_import(file, GWY_FILE_IMAGE, channel_number, NULL, filename);
             channel_number++;
 
 finish:
@@ -1009,31 +1022,7 @@ finish:
         free_datachannel_group(l->data);
     g_list_free(list_of_datagroups);
 
-    return (GwyContainer*)object;
-}
-
-static gint
-spml_detect(const GwyFileDetectInfo * fileinfo, gboolean only_name)
-{
-    gint score = 0;
-
-    gwy_debug("%s: %s", fileinfo->name_lowercase, (only_name) ? "only_name" : "content");
-    if (only_name) {
-        if (g_str_has_suffix(fileinfo->name_lowercase, ".xml")) {
-            score += 50;
-            gwy_debug("Score += 50");
-        }
-    }
-    else {
-        if (fileinfo->head != NULL) {
-            gwy_debug("head not null");
-            if (strstr(fileinfo->head, "<SPML") != NULL) {
-                gwy_debug("Score += 100");
-                score += 100;
-            }
-        }
-    }
-    return score;
+    return file;
 }
 
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
