@@ -1,6 +1,6 @@
 /*
- *  $Id: spmlab.c 25800 2023-10-06 16:05:45Z yeti-dn $
- *  Copyright (C) 2004-2016 David Necas (Yeti), Petr Klapetek.
+ *  $Id: spmlab.c 28789 2025-11-04 17:14:03Z yeti-dn $
+ *  Copyright (C) 2004-2025 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  Roughly based on code in Kasgira by MV <kasigra@seznam.cz>.
@@ -97,15 +97,9 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/datafield.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/gwyapp.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -129,9 +123,9 @@ typedef struct {
     gdouble qrate;
     gdouble layers_from;
     gdouble layers_to;
-    GwySIUnit *unitxy;
-    GwySIUnit *unitz;
-    GwySIUnit *unitrate;
+    GwyUnit *unitxy;
+    GwyUnit *unitz;
+    GwyUnit *unitrate;
     gchar *datatype_str;
     gchar *probetype_str;
     gchar *datamode_str;
@@ -142,36 +136,36 @@ typedef struct {
     gchar *scantype;
 } SPMLabFile;
 
-static gboolean      module_register    (void);
-static gint          spmlab_detect      (const GwyFileDetectInfo *fileinfo,
-                                         gboolean only_name);
-static GwyContainer* spmlab_load        (const gchar *filename,
-                                         GwyRunType mode,
-                                         GError **error);
-static gboolean      spmlab_read_header (SPMLabFile *slfile,
-                                         const guchar *buffer,
-                                         guint size,
-                                         GError **error);
-static void          read_data_field    (SPMLabFile *slfile,
-                                         const guchar *buffer,
-                                         GwyContainer *container,
-                                         guint i);
-static void          add_meta           (SPMLabFile *slfile,
-                                         GwyContainer *container,
-                                         guint i);
-static void          spmlab_file_free   (SPMLabFile *slfile);
-static const gchar*  datatype_to_string (gint type);
-static const gchar*  datamode_to_string (gint mode);
-static const gchar*  stagetype_to_string(gint type);
-static const gchar*  probetype_to_string(gint type);
-static const gchar*  direction_to_string(gint type);
+static gboolean     module_register    (void);
+static gint         detect_file        (const GwyFileDetectInfo *fileinfo,
+                                        gboolean only_name);
+static GwyFile*     load_file          (const gchar *filename,
+                                        GwyRunModeFlags mode,
+                                        GError **error);
+static gboolean     spmlab_read_header (SPMLabFile *slfile,
+                                        const guchar *buffer,
+                                        guint size,
+                                        GError **error);
+static void         read_field         (SPMLabFile *slfile,
+                                        const guchar *buffer,
+                                        GwyFile *file,
+                                        guint i);
+static void         add_meta           (SPMLabFile *slfile,
+                                        GwyFile *file,
+                                        guint i);
+static void         spmlab_file_free   (SPMLabFile *slfile);
+static const gchar* datatype_to_string (gint type);
+static const gchar* datamode_to_string (gint mode);
+static const gchar* stagetype_to_string(gint type);
+static const gchar* probetype_to_string(gint type);
+static const gchar* direction_to_string(gint type);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Thermicroscopes SpmLab R3 to R7 data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.12",
+    "0.14",
     "David Nečas (Yeti) & Petr Klapetek",
     "2005",
 };
@@ -183,17 +177,13 @@ module_register(void)
 {
     gwy_file_func_register("spmlab",
                            N_("Thermicroscopes SpmLab files"),
-                           (GwyFileDetectFunc)&spmlab_detect,
-                           (GwyFileLoadFunc)&spmlab_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-spmlab_detect(const GwyFileDetectInfo *fileinfo,
-              gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     gint score = 0;
 
@@ -224,12 +214,12 @@ spmlab_detect(const GwyFileDetectInfo *fileinfo,
     return score;
 }
 
-static GwyContainer*
-spmlab_load(const gchar *filename,
-            G_GNUC_UNUSED GwyRunType mode,
-            GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     guchar *buffer = NULL;
     gsize size = 0, datablocksize;
     GError *err = NULL;
@@ -240,7 +230,7 @@ spmlab_load(const gchar *filename,
         err_GET_FILE_CONTENTS(error, &err);
         return NULL;
     }
-    gwy_clear(&slfile, 1);
+    gwy_clear1(slfile);
     if (buffer[0] != '#' || buffer[1] != 'R') {
         err_FILE_TYPE(error, "Thermicroscopes SpmLab");
         goto fail;
@@ -273,16 +263,16 @@ spmlab_load(const gchar *filename,
         goto fail;
     }
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     for (i = 0; i < slfile.nlayers; i++) {
-        read_data_field(&slfile, buffer, container, i);
-        gwy_file_channel_import_log_add(container, i, NULL, filename);
+        read_field(&slfile, buffer, file, i);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, i, NULL, filename);
     }
 
 fail:
     spmlab_file_free(&slfile);
     gwy_file_abandon_contents(buffer, size, NULL);
-    return container;
+    return file;
 }
 
 static gdouble
@@ -409,27 +399,27 @@ spmlab_read_header(SPMLabFile *slfile,
 
     p = buffer + offsets[UNITSTR_IDX];
     s = g_strndup(p, UNIT_LEN);
-    slfile->unitz = gwy_si_unit_new_parse(s, &power10);
+    slfile->unitz = gwy_unit_new_parse(s, &power10);
     g_free(s);
-    slfile->q *= pow10(power10);
-    slfile->z0 *= pow10(power10);
+    slfile->q *= gwy_exp10(power10);
+    slfile->z0 *= gwy_exp10(power10);
 
     p += UNIT_LEN;
     s = g_strndup(p, UNIT_LEN);
-    slfile->unitxy = gwy_si_unit_new_parse(s, &power10);
+    slfile->unitxy = gwy_unit_new_parse(s, &power10);
     g_free(s);
-    slfile->xreal *= pow10(power10);
-    slfile->yreal *= pow10(power10);
-    slfile->xoff *= pow10(power10);
-    slfile->yoff *= pow10(power10);
+    slfile->xreal *= gwy_exp10(power10);
+    slfile->yreal *= gwy_exp10(power10);
+    slfile->xoff *= gwy_exp10(power10);
+    slfile->yoff *= gwy_exp10(power10);
     gwy_debug("xres = %d, yres = %d, xreal = %g, yreal = %g, q = %g, z0 = %g",
               slfile->xres, slfile->yres, slfile->xreal, slfile->yreal, slfile->q, slfile->z0);
 
     p += UNIT_LEN;
     s = g_strndup(p, UNIT_LEN);
-    slfile->unitrate = gwy_si_unit_new_parse(s, &power10);
+    slfile->unitrate = gwy_unit_new_parse(s, &power10);
     g_free(s);
-    slfile->qrate = pow10(power10);
+    slfile->qrate = gwy_exp10(power10);
 
     /* Optional stuff, i.e. this that either exists only in some version or
      * we only know how to read in certain versions. */
@@ -472,26 +462,26 @@ spmlab_read_header(SPMLabFile *slfile,
 }
 
 static void
-read_data_field(SPMLabFile *slfile, const guchar *buffer,
-                GwyContainer *container, guint i)
+read_field(SPMLabFile *slfile, const guchar *buffer,
+           GwyFile *file, guint i)
 {
     const guchar *p = buffer + slfile->dataoffset;
     guint xres = slfile->xres, yres = slfile->yres, nlayers = slfile->nlayers;
     gsize datablocksize;
-    GwyDataField *dfield;
+    GwyField *dfield;
 
     datablocksize = slfile->xres*slfile->yres*sizeof(guint16);
-    dfield = gwy_data_field_new(xres, yres, slfile->xreal, slfile->yreal, FALSE);
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(dfield), slfile->unitxy);
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_z(dfield), slfile->unitz);
+    dfield = gwy_field_new(xres, yres, slfile->xreal, slfile->yreal, FALSE);
+    gwy_unit_assign(gwy_field_get_unit_xy(dfield), slfile->unitxy);
+    gwy_unit_assign(gwy_field_get_unit_z(dfield), slfile->unitz);
     gwy_convert_raw_data(p + i*datablocksize, xres*yres, 1, GWY_RAW_DATA_UINT16, GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                         gwy_data_field_get_data(dfield), slfile->q, slfile->z0);
+                         gwy_field_get_data(dfield), slfile->q, slfile->z0);
 
-    gwy_container_pass_object(container, gwy_app_get_data_key_for_id(i), dfield);
+    gwy_file_pass_image(file, i, dfield);
 
     if (slfile->datatype_str) {
         if (nlayers < 2)
-            gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(i), slfile->datatype_str);
+            gwy_file_set_title(file, GWY_FILE_IMAGE, i, slfile->datatype_str, FALSE);
         else {
             gdouble from = slfile->layers_from, to = slfile->layers_to, z;
             gchar *title;
@@ -502,19 +492,19 @@ read_data_field(SPMLabFile *slfile, const guchar *buffer,
             }
             else
                 title = g_strdup_printf("%s %u", slfile->datatype_str, i+1);
-            gwy_container_set_string(container, gwy_app_get_data_title_key_for_id(i), (const guchar*)title);
+            gwy_file_pass_title(file, GWY_FILE_IMAGE, i, title);
         }
     }
     else
-        gwy_app_channel_title_fall_back(container, i);
+        gwy_image_title_fall_back(file, i);
 
-    add_meta(slfile, container, i);
+    add_meta(slfile, file, i);
 }
 
 static void
-add_meta(SPMLabFile *slfile, GwyContainer *container, guint i)
+add_meta(SPMLabFile *slfile, GwyFile *file, guint i)
 {
-    GwyContainer *meta = gwy_container_new();
+    GwyContainer *meta = gwy_container_new_in_construction();
     const gchar *s;
 
     if ((s = slfile->datetime) && *s)
@@ -544,16 +534,17 @@ add_meta(SPMLabFile *slfile, GwyContainer *container, guint i)
         gwy_container_set_const_string_by_name(meta, "Direction", s);
 
     if (gwy_container_get_n_items(meta))
-        gwy_container_set_object(container, gwy_app_get_data_meta_key_for_id(i), meta);
-    g_object_unref(meta);
+        gwy_file_pass_meta(file, GWY_FILE_IMAGE, i, meta);
+    else
+        g_object_unref(meta);
 }
 
 static void
 spmlab_file_free(SPMLabFile *slfile)
 {
-    GWY_OBJECT_UNREF(slfile->unitxy);
-    GWY_OBJECT_UNREF(slfile->unitz);
-    GWY_OBJECT_UNREF(slfile->unitrate);
+    g_clear_object(&slfile->unitxy);
+    g_clear_object(&slfile->unitz);
+    g_clear_object(&slfile->unitrate);
     g_free(slfile->release);
     g_free(slfile->datetime);
     g_free(slfile->description);
