@@ -1,6 +1,6 @@
 /*
- *  $Id: spmxfile.c 26873 2024-11-19 12:52:14Z yeti-dn $
- *  Copyright (C) 2014 David Necas (Yeti).
+ *  $Id: spmxfile.c 28789 2025-11-04 17:14:03Z yeti-dn $
+ *  Copyright (C) 2014-2025 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
  *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
@@ -27,15 +27,10 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/stats.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 
 #include "err.h"
 #include "gwyzip.h"
@@ -56,10 +51,10 @@ typedef struct {
     guint blocksize;
     guint dim;
     GPtrArray *subimages;
-    GwySIUnit *unitxy[2];
+    GwyUnit *unitxy[2];
     guint xyres[2];
     gdouble xyscale[2];
-    GwySIUnit *unitz;
+    GwyUnit *unitz;
     gdouble zoff, zscale;
 } SPMXStream;
 
@@ -72,27 +67,27 @@ typedef struct {
     GArray *streams;
 } SPMXFile;
 
-static gboolean      module_register      (void);
-static gint          spmx_detect          (const GwyFileDetectInfo *fileinfo,
-                                           gboolean only_name);
-static GwyContainer* spmx_load            (const gchar *filename,
-                                           GwyRunType mode,
-                                           GError **error);
-static gboolean      spmx_parse_main      (GwyZipFile zipfile,
-                                           SPMXFile *spmxfile,
-                                           GError **error);
-static gboolean      read_binary_data     (const SPMXFile *spmxfile,
-                                           GwyZipFile zipfile,
-                                           GwyContainer *container,
-                                           GError **error);
-static void          spmx_file_free       (SPMXFile *spmxfile);
+static gboolean module_register (void);
+static gint     detect_file     (const GwyFileDetectInfo *fileinfo,
+                                 gboolean only_name);
+static GwyFile* load_file       (const gchar *filename,
+                                 GwyRunModeFlags mode,
+                                 GError **error);
+static gboolean parse_main      (GwyZipFile zipfile,
+                                 SPMXFile *spmxfile,
+                                 GError **error);
+static gboolean read_binary_data(const SPMXFile *spmxfile,
+                                 GwyZipFile zipfile,
+                                 GwyFile *file,
+                                 GError **error);
+static void     free_spmxfile   (SPMXFile *spmxfile);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Reads ATC SPMxFormat files."),
     "Yeti <yeti@gwyddion.net>",
-    "1.5",
+    "2.0",
     "David Nečas (Yeti)",
     "2014",
 };
@@ -104,17 +99,13 @@ module_register(void)
 {
     gwy_file_func_register("spmxfile",
                            N_("ATC SPMxFormat data (.spm)"),
-                           (GwyFileDetectFunc)&spmx_detect,
-                           (GwyFileLoadFunc)&spmx_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-spmx_detect(const GwyFileDetectInfo *fileinfo,
-           gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     GwyZipFile zipfile;
     guchar *content;
@@ -151,21 +142,21 @@ spmx_detect(const GwyFileDetectInfo *fileinfo,
     return score;
 }
 
-static GwyContainer*
-spmx_load(const gchar *filename,
-         G_GNUC_UNUSED GwyRunType mode,
-         GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     SPMXFile spmxfile;
     GwyZipFile zipfile;
 
     if (!(zipfile = gwyzip_open(filename, error)))
         return NULL;
 
-    gwy_clear(&spmxfile, 1);
+    gwy_clear1(spmxfile);
     spmxfile.filename = filename;
-    if (!spmx_parse_main(zipfile, &spmxfile, error))
+    if (!parse_main(zipfile, &spmxfile, error))
         goto fail;
 
     if (!spmxfile.streams->len) {
@@ -173,25 +164,25 @@ spmx_load(const gchar *filename,
         goto fail;
     }
 
-    container = gwy_container_new();
-    if (!read_binary_data(&spmxfile, zipfile, container, error))
-        GWY_OBJECT_UNREF(container);
+    file = gwy_file_new_in_construction();
+    if (!read_binary_data(&spmxfile, zipfile, file, error))
+        g_clear_object(&file);
 
 fail:
     gwyzip_close(zipfile);
-    spmx_file_free(&spmxfile);
+    free_spmxfile(&spmxfile);
 
-    return container;
+    return file;
 }
 
 static gboolean
 read_binary_data(const SPMXFile *spmxfile,
                  GwyZipFile zipfile,
-                 GwyContainer *container,
+                 GwyFile *file,
                  GError **error)
 {
     GArray *streams = spmxfile->streams;
-    GwyDataField *dfield;
+    GwyField *dfield;
     guchar *content;
     gchar *title, *subimage;
     gsize contentsize, expected_size;
@@ -213,22 +204,22 @@ read_binary_data(const SPMXFile *spmxfile,
         }
 
         for (j = 0; j < stream->subimages->len; j++) {
-            dfield = gwy_data_field_new(stream->xyres[0], stream->xyres[1],
-                                        stream->xyscale[0]*stream->xyres[0], stream->xyscale[1]*stream->xyres[1],
-                                        FALSE);
+            dfield = gwy_field_new(stream->xyres[0], stream->xyres[1],
+                                   stream->xyscale[0]*stream->xyres[0], stream->xyscale[1]*stream->xyres[1],
+                                   FALSE);
             gwy_convert_raw_data(content + j*n*stream->blocksize, n, 1,
                                  GWY_RAW_DATA_SINT16, GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                                 gwy_data_field_get_data(dfield), stream->zscale, stream->zoff);
+                                 gwy_field_get_data(dfield), stream->zscale, stream->zoff);
 
-            gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(dfield), stream->unitxy[0]);
-            gwy_si_unit_assign(gwy_data_field_get_si_unit_z(dfield), stream->unitz);
+            gwy_unit_assign(gwy_field_get_unit_xy(dfield), stream->unitxy[0]);
+            gwy_unit_assign(gwy_field_get_unit_z(dfield), stream->unitz);
 
-            gwy_container_pass_object(container, gwy_app_get_data_key_for_id(id), dfield);
-            gwy_file_channel_import_log_add(container, id, NULL, spmxfile->filename);
+            gwy_file_pass_image(file, id, dfield);
+            gwy_log_add_import(file, GWY_FILE_IMAGE, id, NULL, spmxfile->filename);
 
             subimage = (gchar*)g_ptr_array_index(stream->subimages, j);
             title = g_strdup_printf("%s %s", stream->name, subimage);
-            gwy_container_set_string(container, gwy_app_get_data_title_key_for_id(id), title);
+            gwy_file_pass_title(file, GWY_FILE_IMAGE, id, title);
 
             id++;
         }
@@ -295,12 +286,12 @@ require_attributes(const gchar *path,
 }
 
 static void
-spmx_start_element(G_GNUC_UNUSED GMarkupParseContext *context,
-                   const gchar *element_name,
-                   const gchar **attribute_names,
-                   const gchar **attribute_values,
-                   gpointer user_data,
-                   GError **error)
+start_element(G_GNUC_UNUSED GMarkupParseContext *context,
+              const gchar *element_name,
+              const gchar **attribute_names,
+              const gchar **attribute_values,
+              gpointer user_data,
+              GError **error)
 {
     SPMXFile *spmxfile = (SPMXFile*)user_data;
     SPMXStream *stream = current_stream(spmxfile);
@@ -319,7 +310,7 @@ spmx_start_element(G_GNUC_UNUSED GMarkupParseContext *context,
         if (!require_attributes(path, attribute_names, attribute_values, error, "id", "name", "blocksize", NULL))
             return;
 
-        gwy_clear(&newstream, 1);
+        gwy_clear1(newstream);
         value = find_attribute(attribute_names, attribute_values, "id");
         newstream.id = g_strdup(value);
         value = find_attribute(attribute_names, attribute_values, "name");
@@ -342,19 +333,18 @@ spmx_start_element(G_GNUC_UNUSED GMarkupParseContext *context,
         if (stream->unitz)
             return;
 
-        if (!require_attributes(path, attribute_names, attribute_values, error,
-                                "scale", "units", NULL))
+        if (!require_attributes(path, attribute_names, attribute_values, error, "scale", "units", NULL))
             return;
 
         value = find_attribute(attribute_names, attribute_values, "scale");
         stream->zscale = g_ascii_strtod(value, NULL);
         value = find_attribute(attribute_names, attribute_values, "units");
-        stream->unitz = gwy_si_unit_new_parse(value, &power10);
-        stream->zscale *= pow10(power10);
+        stream->unitz = gwy_unit_new_parse(value, &power10);
+        stream->zscale *= gwy_exp10(power10);
 
         if ((value = find_attribute(attribute_names, attribute_values,
                                     "start")))
-            stream->zoff = g_ascii_strtod(value, NULL) * pow10(power10);
+            stream->zoff = g_ascii_strtod(value, NULL) * gwy_exp10(power10);
     }
     else if (gwy_strequal(path, "/spmx/stream/dimension")) {
         g_return_if_fail(stream);
@@ -385,8 +375,8 @@ spmx_start_element(G_GNUC_UNUSED GMarkupParseContext *context,
         value = find_attribute(attribute_names, attribute_values, "scale");
         stream->xyscale[stream->dim] = g_ascii_strtod(value, NULL);
         value = find_attribute(attribute_names, attribute_values, "units");
-        stream->unitxy[stream->dim] = gwy_si_unit_new_parse(value, &power10);
-        stream->xyscale[stream->dim]*= pow10(power10);
+        stream->unitxy[stream->dim] = gwy_unit_new_parse(value, &power10);
+        stream->xyscale[stream->dim] *= gwy_exp10(power10);
     }
     else if (gwy_strequal(path, "/spmx/stream/data")) {
         g_return_if_fail(stream);
@@ -405,10 +395,10 @@ spmx_start_element(G_GNUC_UNUSED GMarkupParseContext *context,
 }
 
 static void
-spmx_end_element(G_GNUC_UNUSED GMarkupParseContext *context,
-                 const gchar *element_name,
-                 gpointer user_data,
-                 G_GNUC_UNUSED GError **error)
+end_element(G_GNUC_UNUSED GMarkupParseContext *context,
+            const gchar *element_name,
+            gpointer user_data,
+            G_GNUC_UNUSED GError **error)
 {
     SPMXFile *spmxfile = (SPMXFile*)user_data;
     guint n, len = spmxfile->path->len;
@@ -429,11 +419,11 @@ spmx_end_element(G_GNUC_UNUSED GMarkupParseContext *context,
 }
 
 static void
-spmx_text(G_GNUC_UNUSED GMarkupParseContext *context,
-          const gchar *text,
-          G_GNUC_UNUSED gsize text_len,
-          gpointer user_data,
-          G_GNUC_UNUSED GError **error)
+text(G_GNUC_UNUSED GMarkupParseContext *context,
+     const gchar *value,
+     G_GNUC_UNUSED gsize value_len,
+     gpointer user_data,
+     G_GNUC_UNUSED GError **error)
 {
     SPMXFile *spmxfile = (SPMXFile*)user_data;
     SPMXStream *stream = current_stream(spmxfile);
@@ -441,7 +431,7 @@ spmx_text(G_GNUC_UNUSED GMarkupParseContext *context,
     gchar *varid = spmxfile->varid->str;
     GString *str = spmxfile->str;
 
-    if (!*text)
+    if (!*value)
         return;
 
     /* I thought this would somehow refer to <var>s, but many mentioned in <depends> are blatantly missing...  So just
@@ -449,7 +439,7 @@ spmx_text(G_GNUC_UNUSED GMarkupParseContext *context,
     if (gwy_strequal(path, "/spmx/stream/depends"))
         return;
 
-    g_string_assign(str, text);
+    g_string_assign(str, value);
     g_strstrip(str->str);
     if (!*(str->str))
         return;
@@ -469,16 +459,12 @@ spmx_text(G_GNUC_UNUSED GMarkupParseContext *context,
 }
 
 static gboolean
-spmx_parse_main(GwyZipFile zipfile,
-                SPMXFile *spmxfile,
-                GError **error)
+parse_main(GwyZipFile zipfile,
+           SPMXFile *spmxfile,
+           GError **error)
 {
     GMarkupParser parser = {
-        &spmx_start_element,
-        &spmx_end_element,
-        &spmx_text,
-        NULL,
-        NULL,
+        &start_element, &end_element, &text, NULL, NULL,
     };
     GMarkupParseContext *context = NULL;
     guchar *content = NULL, *s;
@@ -518,7 +504,7 @@ spmx_parse_main(GwyZipFile zipfile,
 }
 
 static void
-spmx_file_free(SPMXFile *spmxfile)
+free_spmxfile(SPMXFile *spmxfile)
 {
     if (spmxfile->hash)
         g_hash_table_destroy(spmxfile->hash);
@@ -537,9 +523,9 @@ spmx_file_free(SPMXFile *spmxfile)
             g_free(stream->id);
             g_free(stream->name);
             g_free(stream->filename);
-            GWY_OBJECT_UNREF(stream->unitxy[0]);
-            GWY_OBJECT_UNREF(stream->unitxy[1]);
-            GWY_OBJECT_UNREF(stream->unitz);
+            g_clear_object(stream->unitxy + 0);
+            g_clear_object(stream->unitxy + 1);
+            g_clear_object(&stream->unitz);
 
             for (j = 0; j < stream->subimages->len; j++)
                 g_free(g_ptr_array_index(stream->subimages, j));
