@@ -1,12 +1,12 @@
 /*
- *  $Id: stmprg.c 26163 2024-02-06 16:56:55Z yeti-dn $
+ *  $Id: stmprg.c 28789 2025-11-04 17:14:03Z yeti-dn $
  *  Copyright (C) 2004 Rok Zitko
  *  E-mail: rok.zitko@ijs.si
  *
  *  Information on the format was published in the manual
  *  for STMPRG, Copyright (C) 1989-1992 Omicron.
  *
- *  Based on nanoscope.c, Copyright (C) 2004 David Necas (Yeti), Petr Klapetek.
+ *  Copyright (C) 2004-2025 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -52,15 +52,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/datafield.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 
 #include "get.h"
 #include "err.h"
@@ -211,23 +206,23 @@ typedef struct {
     StmprgOtherControl other_control;
 } StmprgFile;
 
-static gboolean      module_register     (void);
-static gint          stmprg_detect       (const GwyFileDetectInfo *fileinfo,
-                                          gboolean only_name);
-static GwyContainer* stmprg_load         (const gchar *filename,
-                                          GwyRunType mode,
-                                          GError **error);
-static gboolean      read_binary_ubedata (gint n,
-                                          gdouble *data,
-                                          const guchar *buffer,
-                                          gint bpp);
+static gboolean module_register    (void);
+static gint     detect_file        (const GwyFileDetectInfo *fileinfo,
+                                    gboolean only_name);
+static GwyFile* load_file          (const gchar *filename,
+                                    GwyRunModeFlags mode,
+                                    GError **error);
+static gboolean read_binary_ubedata(gint n,
+                                    gdouble *data,
+                                    const guchar *buffer,
+                                    gint bpp);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Omicron STMPRG data files (tp ta)."),
     "Rok Zitko <rok.zitko@ijs.si>, Yeti <yeti@gwyddion.net>",
-    "0.12",
+    "0.13",
     "Rok Zitko & David Nečas (Yeti)",
     "2004",
 };
@@ -239,17 +234,13 @@ module_register(void)
 {
     gwy_file_func_register("stmprg",
                            N_("Omicron STMPRG files (tp ta)"),
-                           (GwyFileDetectFunc)&stmprg_detect,
-                           (GwyFileLoadFunc)&stmprg_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-stmprg_detect(const GwyFileDetectInfo *fileinfo,
-              gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     gint score = 0;
 
@@ -489,15 +480,15 @@ read_parameters(const guchar *buffer,
     return TRUE;
 }
 
-static GwyDataField*
-read_datafield(const guchar *buffer,
-               guint size,
-               const StmprgFile *stmprgfile,
-               GError **error)
+static GwyField*
+read_field(const guchar *buffer,
+           guint size,
+           const StmprgFile *stmprgfile,
+           GError **error)
 {
     gint xres, yres, bpp;
     gdouble xreal, yreal, q;
-    GwyDataField *dfield;
+    GwyField *dfield;
     gdouble *data;
     const gchar *unitstr;
 
@@ -513,17 +504,17 @@ read_datafield(const guchar *buffer,
     if (err_SIZE_MISMATCH(error, bpp*xres*yres, size, FALSE))
         return NULL;
 
-    dfield = gwy_data_field_new(xres, yres, xreal, yreal, FALSE);
-    data = gwy_data_field_get_data(dfield);
+    dfield = gwy_field_new(xres, yres, xreal, yreal, FALSE);
+    data = gwy_field_get_data(dfield);
     if (!read_binary_ubedata(xres * yres, data, buffer, bpp)) {
         err_BPP(error, bpp);
         g_object_unref(dfield);
         return NULL;
     }
 
-    gwy_data_field_multiply(dfield, q);
+    gwy_field_multiply(dfield, q);
 
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
 
     /* Assuming we are reading channel1... */
     switch (stmprgfile->control.channel1) {
@@ -552,7 +543,7 @@ read_datafield(const guchar *buffer,
         g_assert_not_reached();
         break;
     }
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), unitstr);
+    gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), unitstr);
 
     return dfield;
 }
@@ -569,13 +560,11 @@ read_datafield(const guchar *buffer,
 #define HASH_STORE_I(keystr, vali) HASH_STORE("%i", keystr, vali)
 
 static GwyContainer*
-stmprg_get_metadata(const StmprgFile *stmprgfile)
+get_metadata(const StmprgFile *stmprgfile)
 {
-    GwyContainer *meta;
+    GwyContainer *meta = gwy_container_new_in_construction();
     gchar *value;
 
-    meta = gwy_container_new();
-
     HASH_STORE_F("inc_x", mainfield.inc_x);
     HASH_STORE_F("inc_y", mainfield.inc_y);
     HASH_STORE_F("angle", mainfield.angle);
@@ -590,17 +579,17 @@ stmprg_get_metadata(const StmprgFile *stmprgfile)
     return meta;
 }
 
-static GwyContainer*
-stmprg_load(const gchar *filename,
-            G_GNUC_UNUSED GwyRunType mode,
-            GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *meta, *container = NULL;
+    GwyFile *file = NULL;
     StmprgFile stmprgfile;
     guchar *buffer = NULL;
     gsize size = 0;
     GError *err = NULL;
-    GwyDataField *dfield;
+    GwyField *dfield;
     char *filename_ta, *ptr;
     gboolean ok;
 
@@ -614,7 +603,7 @@ stmprg_load(const gchar *filename,
         return NULL;
     }
 
-    gwy_clear(&stmprgfile, 1);
+    gwy_clear1(stmprgfile);
     ok = read_parameters(buffer, size, &stmprgfile, error);
     gwy_file_abandon_contents(buffer, size, NULL);
     if (!ok)
@@ -644,21 +633,18 @@ stmprg_load(const gchar *filename,
     }
     g_free(filename_ta);
 
-    dfield = read_datafield(buffer, size, &stmprgfile, error);
+    dfield = read_field(buffer, size, &stmprgfile, error);
     if (dfield) {
-        container = gwy_container_new();
-        gwy_container_pass_object(container, gwy_app_get_data_key_for_id(0), dfield);
+        file = gwy_file_new_in_construction();
+        gwy_file_pass_image(file, 0, dfield);
         /* FIXME: with documentation, we could perhaps do better */
-        gwy_app_channel_title_fall_back(container, 0);
-
-        meta = stmprg_get_metadata(&stmprgfile);
-        gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(0), meta);
-
-        gwy_file_channel_import_log_add(container, 0, NULL, filename);
+        gwy_image_title_fall_back(file, 0);
+        gwy_file_pass_meta(file, GWY_FILE_IMAGE, 0, get_metadata(&stmprgfile));
+        gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
     }
     gwy_file_abandon_contents(buffer, size, NULL);
 
-    return container;
+    return file;
 }
 
 /*
