@@ -1,6 +1,6 @@
 /*
- *  $Id: stpfile.c 26344 2024-05-15 13:27:20Z yeti-dn $
- *  Copyright (C) 2005 David Necas (Yeti), Petr Klstptek.
+ *  $Id: stpfile.c 28860 2025-11-14 12:59:38Z yeti-dn $
+ *  Copyright (C) 2005-2025 David Necas (Yeti), Petr Klstptek.
  *  E-mail: yeti@gwyddion.net, klstptek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -47,16 +47,11 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
 #include <glib/gprintf.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <libprocess/datafield.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -92,22 +87,22 @@ typedef struct {
     const gchar *format;
 } MetaDataFormat;
 
-static gboolean      module_register (void);
-static gint          stpfile_detect  (const GwyFileDetectInfo *fileinfo,
-                                      gboolean only_name);
-static GwyContainer* stpfile_load    (const gchar *filename,
-                                      GwyRunType mode,
-                                      GError **error);
-static guint         find_data_start (const guchar *buffer,
-                                      gsize size);
-static void          stpfile_free    (STPFile *stpfile);
-static void          read_data_field (GwyDataField *dfield,
-                                      STPData *stpdata);
-static guint         file_read_header(STPFile *stpfile,
-                                      gchar *buffer);
-static void          process_metadata(STPFile *stpfile,
-                                      guint id,
-                                      GwyContainer *container);
+static gboolean module_register (void);
+static gint     detect_file     (const GwyFileDetectInfo *fileinfo,
+                                 gboolean only_name);
+static GwyFile* load_file       (const gchar *filename,
+                                 GwyRunModeFlags mode,
+                                 GError **error);
+static guint    find_data_start (const guchar *buffer,
+                                 gsize size);
+static void     free_stpfile    (STPFile *stpfile);
+static void     read_field      (GwyField *dfield,
+                                 STPData *stpdata);
+static guint    file_read_header(STPFile *stpfile,
+                                 gchar *buffer);
+static void     process_metadata(STPFile *stpfile,
+                                 guint id,
+                                 GwyFile *file);
 
 static const GwyEnum channels[] = {
     { N_("Topography"),                         1,  },
@@ -135,7 +130,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports Molecular Imaging STP data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.5.3",
+    "0.6",
     "David Nečas (Yeti), Petr Klapetek, Chris Anderson",
     "2006",
 };
@@ -147,17 +142,13 @@ module_register(void)
 {
     gwy_file_func_register("stpfile",
                            N_("STP files (.stp)"),
-                           (GwyFileDetectFunc)&stpfile_detect,
-                           (GwyFileLoadFunc)&stpfile_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-stpfile_detect(const GwyFileDetectInfo *fileinfo,
-               gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     gint score = 0;
 
@@ -170,17 +161,17 @@ stpfile_detect(const GwyFileDetectInfo *fileinfo,
     return score;
 }
 
-static GwyContainer*
-stpfile_load(const gchar *filename,
-             G_GNUC_UNUSED GwyRunType mode,
-             GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
     STPFile *stpfile;
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     guchar *buffer = NULL;
     gsize size = 0;
     GError *err = NULL;
-    GwyDataField *dfield = NULL;
+    GwyField *dfield = NULL;
     guint header_size;
     gchar *p;
     gboolean ok;
@@ -225,27 +216,27 @@ stpfile_load(const gchar *filename,
             stpfile->buffers[i].data = (const guint16*)(buffer + pos);
             pos += 2*stpfile->buffers[i].xres * stpfile->buffers[i].yres;
 
-            dfield = gwy_data_field_new(stpfile->buffers[i].xres, stpfile->buffers[i].yres, 1.0, 1.0, FALSE);
-            read_data_field(dfield, stpfile->buffers + i);
+            dfield = gwy_field_new(stpfile->buffers[i].xres, stpfile->buffers[i].yres, 1.0, 1.0, FALSE);
+            read_field(dfield, stpfile->buffers + i);
 
-            if (!container)
-                container = gwy_container_new();
+            if (!file)
+                file = gwy_file_new_in_construction();
 
-            gwy_container_pass_object(container, gwy_app_get_data_key_for_id(i), dfield);
-            process_metadata(stpfile, i, container);
-            gwy_file_channel_import_log_add(container, i, NULL, filename);
+            gwy_file_pass_image(file, i, dfield);
+            process_metadata(stpfile, i, file);
+            gwy_log_add_import(file, GWY_FILE_IMAGE, i, NULL, filename);
         }
     }
 
     gwy_file_abandon_contents(buffer, size, NULL);
-    stpfile_free(stpfile);
+    free_stpfile(stpfile);
 
-    if (!container) {
+    if (!file) {
         err_NO_DATA(error);
         return NULL;
     }
 
-    return container;
+    return file;
 }
 
 static guint
@@ -258,7 +249,7 @@ find_data_start(const guchar *buffer,
 }
 
 static void
-stpfile_free(STPFile *stpfile)
+free_stpfile(STPFile *stpfile)
 {
     guint i;
 
@@ -333,7 +324,7 @@ stpfile_get_double(GHashTable *meta,
 static void
 process_metadata(STPFile *stpfile,
                  guint id,
-                 GwyContainer *container)
+                 GwyFile *file)
 {
     static const MetaDataFormat global_metadata[] = {
         { "software",      "Software",                  "%s",                 },
@@ -352,29 +343,25 @@ process_metadata(STPFile *stpfile,
         { "tip_bias",     "Force setpoint (tip bias)",  "%s V",  },
         { "line_freq",    "Line frequency (requested)", "%s Hz", },
     };
-    STPData *data;
-    GwyContainer *meta;
-    GwyDataField *dfield;
-    GwySIUnit *siunit;
+    STPData *data = stpfile->buffers + id;
+    GwyField *dfield = gwy_file_get_image(file, id);
+    GwyUnit *siunit;
     gdouble q, r;
     gchar *p, *s;
     const gchar *title;
-    GString *str;
     gint power10;
     guint mode, i;
 
-    dfield = gwy_container_get_object(container, gwy_app_get_data_key_for_id(id));
-    data = stpfile->buffers + id;
     if ((p = g_hash_table_lookup(data->meta, "source_mode"))) {
         mode = atol(p);
         title = gwy_enum_to_string(mode, channels, G_N_ELEMENTS(channels));
         if (title)
-            gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(id), title);
+            gwy_file_set_title(file, GWY_FILE_IMAGE, id, title, FALSE);
     }
     else
         mode = 1;
 
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
 
     /* Fix value scale */
     switch (mode) {
@@ -383,13 +370,13 @@ process_metadata(STPFile *stpfile,
         case 9:
         case 10:
         case 24:
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), "m");
+        gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), "m");
         if (stpfile_get_double(stpfile->meta, "z_v_to_angs", &q) && stpfile_get_double(data->meta, "servo_range", &r)) {
-            gwy_data_field_multiply(dfield, q*r/32768*Angstrom);
+            gwy_field_multiply(dfield, q*r/32768*Angstrom);
             gwy_debug("z_v_to_angs = %g, servo_range = %g", q, r);
         }
         else
-            gwy_data_field_multiply(dfield, Angstrom/32768);
+            gwy_field_multiply(dfield, Angstrom/32768);
         break;
 
         /* External */
@@ -397,23 +384,23 @@ process_metadata(STPFile *stpfile,
         if (stpfile_get_double(data->meta, "ExSourceZFact", &q)) {
             s = g_hash_table_lookup(data->meta, "ExSourceZBip");
             if (s && gwy_strequal(s, "TRUE"))
-                gwy_data_field_add(dfield, -32768);
-            gwy_data_field_multiply(dfield, q);
+                gwy_field_add(dfield, -32768);
+            gwy_field_multiply(dfield, q);
 
             if ((s = g_hash_table_lookup(data->meta, "ExSourceZUnit"))) {
-                siunit = GWY_SI_UNIT(gwy_si_unit_new_parse(s, &power10));
-                gwy_data_field_set_si_unit_z(dfield, siunit);
+                siunit = GWY_UNIT(gwy_unit_new_parse(s, &power10));
+                gwy_unit_assign(gwy_field_get_unit_z(dfield), siunit);
                 g_object_unref(siunit);
-                gwy_data_field_multiply(dfield, pow10(power10));
+                gwy_field_multiply(dfield, gwy_exp10(power10));
             }
         }
         break;
 
         /* Current, Deflection, Vec, Iec, Friction, and Auxes */
         default:
-        gwy_data_field_add(dfield, -32768.0);
-        gwy_data_field_multiply(dfield, 10.0/32768.0);
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), "V");
+        gwy_field_add(dfield, -32768.0);
+        gwy_field_multiply(dfield, 10.0/32768.0);
+        gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), "V");
         break;
     }
 
@@ -422,17 +409,17 @@ process_metadata(STPFile *stpfile,
         g_warning("Missing or invalid x length");
         r = 1e4;
     }
-    gwy_data_field_set_xreal(dfield, r*Angstrom);
+    gwy_field_set_xreal(dfield, r*Angstrom);
 
     if (!stpfile_get_double(data->meta, "length_y", &r)) {
         g_warning("Missing or invalid y length");
         r = 1e4;
     }
-    gwy_data_field_set_yreal(dfield, r*Angstrom);
+    gwy_field_set_yreal(dfield, r*Angstrom);
 
     /* Metadata */
-    meta = gwy_container_new();
-    str = g_string_new(NULL);
+    GwyContainer *meta = gwy_container_new_in_construction();
+    GString *str = g_string_new(NULL);
 
     /* Global */
     for (i = 0; i < G_N_ELEMENTS(global_metadata); i++) {
@@ -469,25 +456,26 @@ process_metadata(STPFile *stpfile,
     }
 
     if (gwy_container_get_n_items(meta))
-        gwy_container_set_object(container, gwy_app_get_data_meta_key_for_id(id), meta);
-    g_object_unref(meta);
+        gwy_file_pass_meta(file, GWY_FILE_IMAGE, id, meta);
+    else
+        g_object_unref(meta);
 
     g_string_free(str, TRUE);
 }
 
 static void
-read_data_field(GwyDataField *dfield,
-                STPData *stpdata)
+read_field(GwyField *dfield,
+           STPData *stpdata)
 {
     gint xres, yres;
 
     xres = stpdata->xres;
     yres = stpdata->yres;
-    gwy_data_field_resample(dfield, xres, yres, GWY_INTERPOLATION_NONE);
+    gwy_field_resize(dfield, xres, yres);
     gwy_convert_raw_data(stpdata->data, xres*yres, 1,
                          GWY_RAW_DATA_UINT16, GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                         gwy_data_field_get_data(dfield), 1.0, 0.0);
-    gwy_data_field_invert(dfield, TRUE, FALSE, FALSE);
+                         gwy_field_get_data(dfield), 1.0, 0.0);
+    gwy_field_flip(dfield, FALSE, TRUE);
 }
 
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
