@@ -1,5 +1,5 @@
 /*
- *  $Id: surffile.c 26494 2024-08-09 15:04:32Z yeti-dn $
+ *  $Id: surffile.c 28848 2025-11-10 18:54:08Z yeti-dn $
  *  Copyright (C) 2005-2024 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -47,12 +47,9 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/stats.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include <glib/gstdio.h>
 #include <stdlib.h>
 #include "get.h"
@@ -203,9 +200,9 @@ typedef struct {
     gchar t_axis_name[13+1];
     gchar t_step_unit[13+1];
 
-    GwyDataField *dfield;
-    GwySIUnit *xyunit;
-    GwySIUnit *zunit;
+    GwyField *dfield;
+    GwyUnit *xyunit;
+    GwyUnit *zunit;
 } SurfFile;
 
 /* New structure surfwriter to write directly the structure with cast gint16 of * enums */
@@ -269,18 +266,22 @@ typedef struct {
 } SurfWriter;
 
 static gboolean      module_register        (void);
-static gint          surffile_detect        (const GwyFileDetectInfo *fileinfo,
+static gint          detect_file            (const GwyFileDetectInfo *fileinfo,
                                              gboolean only_name);
-static GwyContainer* surffile_load          (const gchar *filename,
-                                             GwyRunType mode,
+static GwyFile*      load_file              (const gchar *filename,
+                                             GwyRunModeFlags mode,
                                              GError **error);
-static gsize         surffile_load_one      (const guchar *buffer,
+static gboolean      export_file            (GwyFile *data,
+                                             const gchar *filename,
+                                             GwyRunModeFlags mode,
+                                             GError **error);
+static gsize         load_one_surffile      (const guchar *buffer,
                                              gsize size,
                                              const gchar *filename,
-                                             GwyContainer *container,
+                                             GwyFile *file,
                                              gint *cid,
                                              GError **error);
-static gboolean      fill_data_fields       (SurfFile *surffile,
+static gboolean      fill_fields            (SurfFile *surffile,
                                              const guchar *buffer,
                                              GError **error);
 static GwyContainer* surffile_get_metadata  (SurfFile *surffile);
@@ -288,10 +289,6 @@ static guchar*       decompress_chunked_data(const guchar *buffer,
                                              gsize size,
                                              gsize expected_size,
                                              GError **error);
-static gboolean      surffile_save          (GwyContainer *data,
-                                             const gchar *filename,
-                                             G_GNUC_UNUSED GwyRunType mode,
-                                             GError **error);
 
 static const GwyEnum acq_modes[] = {
     { "Unknown",                     SURF_ACQ_UNKNOWN,        },
@@ -331,7 +328,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports Surf data files."),
     "Petr Klapetek <klapetek@gwyddion.net>",
-    "0.14",
+    "0.15",
     "David Nečas (Yeti) & Petr Klapetek",
     "2006",
 };
@@ -343,17 +340,13 @@ module_register(void)
 {
     gwy_file_func_register("surffile",
                            N_("Surf files (.sur)"),
-                           (GwyFileDetectFunc)&surffile_detect,
-                           (GwyFileLoadFunc)&surffile_load,
-                           NULL,
-                           (GwyFileSaveFunc)&surffile_save);
+                           detect_file, load_file, NULL, export_file);
 
     return TRUE;
 }
 
 static gint
-surffile_detect(const GwyFileDetectInfo *fileinfo,
-                gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     gint score = 0;
 
@@ -387,12 +380,12 @@ read_and_fix_unit(gchar *ustr, const guchar **p)
         ustr[0] = '\xb5';
 }
 
-static GwyContainer*
-surffile_load(const gchar *filename,
-              G_GNUC_UNUSED GwyRunType mode,
-              GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     guchar *buffer = NULL;
     gsize read_bytes, rb = 1, size = 0;
     GError *err = NULL;
@@ -403,17 +396,17 @@ surffile_load(const gchar *filename,
         return NULL;
     }
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     read_bytes = 0;
     while (size - read_bytes > SURF_HEADER_SIZE) {
-        if (!(rb = surffile_load_one(buffer + read_bytes, size - read_bytes, filename, container, &cid, &err)))
+        if (!(rb = load_one_surffile(buffer + read_bytes, size - read_bytes, filename, file, &cid, &err)))
             break;
         read_bytes += rb;
     }
 
     if (!cid && !rb) {
         g_propagate_error(error, err);
-        GWY_OBJECT_UNREF(container);
+        g_clear_object(&file);
     }
     else if (!rb) {
         g_warning("File was loaded only partially.");
@@ -421,17 +414,17 @@ surffile_load(const gchar *filename,
     }
     else if (!cid) {
         err_NO_DATA(error);
-        GWY_OBJECT_UNREF(container);
+        g_clear_object(&file);
     }
 
     gwy_file_abandon_contents(buffer, size, NULL);
 
-    return container;
+    return file;
 }
 
 static gsize
-surffile_load_one(const guchar *buffer, gsize size, const gchar *filename,
-                  GwyContainer *container, gint *cid,
+load_one_surffile(const guchar *buffer, gsize size, const gchar *filename,
+                  GwyFile *file, gint *cid,
                   GError **error)
 {
     SurfFile surffile;
@@ -447,7 +440,7 @@ surffile_load_one(const guchar *buffer, gsize size, const gchar *filename,
         return 0;
     }
 
-    gwy_clear(&surffile, 1);
+    gwy_clear1(surffile);
     p = buffer;
 
     get_CHARARRAY_ADD0(surffile.signature, &p);
@@ -584,42 +577,46 @@ surffile_load_one(const guchar *buffer, gsize size, const gchar *filename,
     p = buffer + SURF_HEADER_SIZE + add;
 
     /*units*/
-    surffile.xyunit = gwy_si_unit_new_parse(surffile.dx_unit, &power10);
-    surffile.dx *= pow10(power10);
+    surffile.xyunit = gwy_unit_new_parse(surffile.dx_unit, &power10);
+    surffile.dx *= gwy_exp10(power10);
     g_object_unref(surffile.xyunit);
-    surffile.xyunit = gwy_si_unit_new_parse(surffile.dy_unit, &power10);
-    surffile.dy *= pow10(power10);
-    surffile.zunit = gwy_si_unit_new_parse(surffile.dz_unit, &power10);
+    surffile.xyunit = gwy_unit_new_parse(surffile.dy_unit, &power10);
+    surffile.dy *= gwy_exp10(power10);
+    surffile.zunit = gwy_unit_new_parse(surffile.dz_unit, &power10);
 
     if (is_compressed) {
         if (!(decomp_buf = decompress_chunked_data(p, surffile.compressed_size, data_size, error)))
             goto fail;
         p = decomp_buf;
     }
-    if (!fill_data_fields(&surffile, p, error))
+    if (!fill_fields(&surffile, p, error))
         goto fail;
 
     if (surffile.inversion == SURF_INV_Z)
-        gwy_data_field_invert(surffile.dfield, FALSE, FALSE, TRUE);
-    else if (surffile.inversion == SURF_FLIP_Z)
-        gwy_data_field_invert(surffile.dfield, FALSE, TRUE, TRUE);
-    else if (surffile.inversion == SURF_FLOP_Z)
-        gwy_data_field_invert(surffile.dfield, TRUE, FALSE, TRUE);
+        gwy_field_invert_value(surffile.dfield);
+    else if (surffile.inversion == SURF_FLIP_Z) {
+        gwy_field_flip(surffile.dfield, TRUE, FALSE);
+        gwy_field_invert_value(surffile.dfield);
+    }
+    else if (surffile.inversion == SURF_FLOP_Z) {
+        gwy_field_flip(surffile.dfield, FALSE, TRUE);
+        gwy_field_invert_value(surffile.dfield);
+    }
 
-    gwy_data_field_multiply(surffile.dfield, pow10(power10)*surffile.dz);
-    surffile.zoffset *= pow10(power10);
-    gwy_data_field_add(surffile.dfield, surffile.zoffset);
+    gwy_field_multiply(surffile.dfield, gwy_exp10(power10)*surffile.dz);
+    surffile.zoffset *= gwy_exp10(power10);
+    gwy_field_add(surffile.dfield, surffile.zoffset);
 
-    gwy_container_pass_object(container, gwy_app_get_data_key_for_id(*cid), surffile.dfield);
-    gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(*cid), surffile_get_metadata(&surffile));
+    gwy_file_pass_image(file, *cid, surffile.dfield);
+    gwy_file_pass_meta(file, GWY_FILE_IMAGE, *cid, surffile_get_metadata(&surffile));
     if (*surffile.object_name)
-        gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(*cid), surffile.object_name);
+        gwy_file_set_title(file, GWY_FILE_IMAGE, *cid, surffile.object_name, FALSE);
     else if (*surffile.zaxis)
-        gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(*cid), surffile.zaxis);
+        gwy_file_set_title(file, GWY_FILE_IMAGE, *cid, surffile.zaxis, FALSE);
     else
-        gwy_app_channel_title_fall_back(container, *cid);
-    gwy_app_channel_check_nonsquare(container, *cid);
-    gwy_file_channel_import_log_add(container, *cid, NULL, filename);
+        gwy_image_title_fall_back(file, *cid);
+    gwy_check_nonsquare_image(file, *cid);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, *cid, NULL, filename);
 
     (*cid)++;
 
@@ -627,26 +624,26 @@ surffile_load_one(const guchar *buffer, gsize size, const gchar *filename,
 
 fail:
     g_free(decomp_buf);
-    GWY_OBJECT_UNREF(surffile.xyunit);
-    GWY_OBJECT_UNREF(surffile.zunit);
+    g_clear_object(&surffile.xyunit);
+    g_clear_object(&surffile.zunit);
 
     return ok ? expected_size : 0;
 }
 
 static gboolean
-fill_data_fields(SurfFile *surffile,
-                 const guchar *buffer,
-                 GError **error)
+fill_fields(SurfFile *surffile,
+            const guchar *buffer,
+            GError **error)
 {
     GwyRawDataType datatype;
-    GwyDataField *field;
+    GwyField *field;
     gdouble *data;
     guint n;
 
     n = surffile->xres*surffile->yres;
-    surffile->dfield = field = gwy_data_field_new(surffile->xres, surffile->yres,
-                                                  surffile->xres*surffile->dx, surffile->yres*surffile->dy,
-                                                  FALSE);
+    surffile->dfield = field = gwy_field_new(surffile->xres, surffile->yres,
+                                             surffile->xres*surffile->dx, surffile->yres*surffile->dy,
+                                             FALSE);
 
     if (surffile->pointsize == 16)
         datatype = GWY_RAW_DATA_SINT16;
@@ -656,11 +653,11 @@ fill_data_fields(SurfFile *surffile,
         err_BPP(error, surffile->pointsize);
         return FALSE;
     }
-    data = gwy_data_field_get_data(field);
+    data = gwy_field_get_data(field);
     gwy_convert_raw_data(buffer, n, 1, datatype, GWY_BYTE_ORDER_LITTLE_ENDIAN, data, 1.0, 0.0);
 
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(field), surffile->xyunit);
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_z(field), surffile->zunit);
+    gwy_unit_assign(gwy_field_get_unit_xy(field), surffile->xyunit);
+    gwy_unit_assign(gwy_field_get_unit_z(field), surffile->zunit);
 
     return TRUE;
 }
@@ -671,11 +668,9 @@ fill_data_fields(SurfFile *surffile,
 static GwyContainer*
 surffile_get_metadata(SurfFile *surffile)
 {
-    GwyContainer *meta;
+    GwyContainer *meta = gwy_container_new_in_construction();
     char date[40];
 
-    meta = gwy_container_new();
-
     g_snprintf(date, sizeof(date), "%d. %d. %d",
                surffile->day, surffile->month, surffile->year);
 
@@ -766,10 +761,10 @@ fill_string_field(gchar *target, const gchar *s, guint size)
 }
 
 static gboolean
-surffile_save(G_GNUC_UNUSED GwyContainer *data,
-              const gchar *filename,
-              G_GNUC_UNUSED GwyRunType mode,
-              GError **error)
+export_file(G_GNUC_UNUSED GwyFile *data,
+            const gchar *filename,
+            G_GNUC_UNUSED GwyRunModeFlags mode,
+            GError **error)
 {
     FILE *fh;
     gboolean ok = TRUE;
@@ -778,9 +773,9 @@ surffile_save(G_GNUC_UNUSED GwyContainer *data,
     gdouble zmaxreal, zminreal;
     gdouble xreal, yreal;
     gchar *dxunittmp, *dyunittmp, *dzunittmp;
-    GwySIUnit *xysi, *zsi;
+    GwyUnit *xysi, *zsi;
     gint k = 0;
-    GwyDataField *dfield;
+    GwyField *dfield;
     const gdouble *points;
     gint32 *integer_values;
 
@@ -830,7 +825,7 @@ surffile_save(G_GNUC_UNUSED GwyContainer *data,
     surf.zoffset = 0.0;
     fill_string_field(surf.reservedzone, "", sizeof(surf.reservedzone));
 
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &dfield, 0);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &dfield, 0);
     if (!dfield) {
         err_NO_CHANNEL_EXPORT(error);
         return FALSE;
@@ -842,11 +837,11 @@ surffile_save(G_GNUC_UNUSED GwyContainer *data,
     }
 
     /*header values*/
-    xysi = gwy_data_field_get_si_unit_xy(dfield);
-    zsi = gwy_data_field_get_si_unit_z(dfield);
-    dxunittmp = gwy_si_unit_get_string(xysi, GWY_SI_UNIT_FORMAT_PLAIN);
-    dyunittmp = gwy_si_unit_get_string(xysi, GWY_SI_UNIT_FORMAT_PLAIN);
-    dzunittmp = gwy_si_unit_get_string(zsi, GWY_SI_UNIT_FORMAT_PLAIN);
+    xysi = gwy_field_get_unit_xy(dfield);
+    zsi = gwy_field_get_unit_z(dfield);
+    dxunittmp = gwy_unit_get_string(xysi, GWY_UNIT_FORMAT_PLAIN);
+    dyunittmp = gwy_unit_get_string(xysi, GWY_UNIT_FORMAT_PLAIN);
+    dzunittmp = gwy_unit_get_string(zsi, GWY_UNIT_FORMAT_PLAIN);
     fill_string_field(surf.dx_unit, dxunittmp, sizeof(surf.dx_unit));
     fill_string_field(surf.dy_unit, dyunittmp, sizeof(surf.dy_unit));
     fill_string_field(surf.dz_unit, dzunittmp, sizeof(surf.dz_unit));
@@ -855,13 +850,13 @@ surffile_save(G_GNUC_UNUSED GwyContainer *data,
     g_free(dzunittmp);
 
     /*extrema*/
-    zmaxreal = gwy_data_field_get_max(dfield);
-    zminreal = gwy_data_field_get_min(dfield);
-    surf.xres = gwy_data_field_get_xres(dfield);
-    surf.yres = gwy_data_field_get_yres(dfield);
+    zmaxreal = gwy_field_get_max(dfield);
+    zminreal = gwy_field_get_min(dfield);
+    surf.xres = gwy_field_get_xres(dfield);
+    surf.yres = gwy_field_get_yres(dfield);
     surf.nofpoints = surf.xres * surf.yres;
-    xreal = gwy_data_field_get_xreal(dfield);
-    yreal = gwy_data_field_get_yreal(dfield);
+    xreal = gwy_field_get_xreal(dfield);
+    yreal = gwy_field_get_yreal(dfield);
 
     /*units*/
     surf.dx = xreal/surf.xres;
@@ -874,7 +869,7 @@ surffile_save(G_GNUC_UNUSED GwyContainer *data,
 
     /*convert data into integer32*/
     integer_values = g_new(gint32, surf.nofpoints);
-    points = gwy_data_field_get_data_const(dfield);
+    points = gwy_field_get_data_const(dfield);
 
     if (zminreal != zmaxreal) {
         for (k = 0; k < surf.nofpoints; k++)
