@@ -1,5 +1,5 @@
 /*
- *  $Id: talos.c 28460 2025-08-27 14:55:54Z yeti-dn $
+ *  $Id: talos.c 28789 2025-11-04 17:14:03Z yeti-dn $
  *  Copyright (C) 2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -29,10 +29,8 @@
 
 #include "config.h"
 #include <stdlib.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/stats.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <glib/gi18n-lib.h>
+#include <gwy.h>
 #include "err.h"
 #include "gwytiff.h"
 
@@ -56,10 +54,10 @@ typedef struct {
 static gboolean      module_register(void);
 static gint          detect_file    (const GwyFileDetectInfo *fileinfo,
                                      gboolean only_name);
-static GwyContainer* load_file      (const gchar *filename,
-                                     GwyRunType mode,
+static GwyFile*      load_file      (const gchar *filename,
+                                     GwyRunModeFlags mode,
                                      GError **error);
-static GwyContainer* load_file_tiff (const GwyTIFF *tiff,
+static GwyFile*      load_file_tiff (const GwyTIFF *tiff,
                                      const gchar *filename,
                                      GError **error);
 static GwyContainer* get_meta       (const gchar *comment);
@@ -69,7 +67,7 @@ static GwyModuleInfo module_info = {
     module_register,
     N_("Imports ThermoFisher Talos TEM images."),
     "Yeti <yeti@gwyddion.net>",
-    "1.0",
+    "1.1",
     "David Nečas (Yeti)",
     "2025",
 };
@@ -81,10 +79,7 @@ module_register(void)
 {
     gwy_file_func_register("talos",
                            N_("ThermoFisher Talos TEM image (.tif)"),
-                           (GwyFileDetectFunc)&detect_file,
-                           (GwyFileLoadFunc)&load_file,
-                           NULL,
-                           NULL);
+                           &detect_file, &load_file, NULL, NULL);
 
     return TRUE;
 }
@@ -128,29 +123,30 @@ detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
     return score;
 }
 
-static GwyContainer*
+static GwyFile*
 load_file(const gchar *filename,
-         G_GNUC_UNUSED GwyRunType mode,
-         GError **error)
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
     GwyTIFF *tiff;
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
 
     tiff = gwy_tiff_load(filename, error);
     if (!tiff)
         return NULL;
 
-    container = load_file_tiff(tiff, filename, error);
+    file = load_file_tiff(tiff, filename, error);
     gwy_tiff_free(tiff);
 
-    return container;
+    return file;
 }
 
-static GwyContainer*
+static GwyFile*
 load_file_tiff(const GwyTIFF *tiff, const gchar *filename, GError **error)
 {
-    GwyContainer *container = NULL, *meta;
-    GwyDataField *dfield;
+    GwyFile *file = NULL;
+    GwyContainer *meta;
+    GwyField *dfield;
     GwyTIFFImageReader *reader = NULL;
     gint i;
     gchar *comment = NULL;
@@ -193,20 +189,20 @@ load_file_tiff(const GwyTIFF *tiff, const gchar *filename, GError **error)
     sanitise_real_size(&xreal, "x scale");
     sanitise_real_size(&yreal, "y sscale");
 
-    dfield = gwy_data_field_new(reader->width, reader->height, xreal, yreal, FALSE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
+    dfield = gwy_field_new(reader->width, reader->height, xreal, yreal, FALSE);
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
 
-    data = gwy_data_field_get_data(dfield);
+    data = gwy_field_get_data(dfield);
     q = 1.0/((1 << reader->bits_per_sample) - 1);
     for (i = 0; i < reader->height; i++)
         gwy_tiff_read_image_row_averaged(tiff, reader, i, q, 0.0, data + i*reader->width);
 
-    container = gwy_container_new();
-    gwy_container_pass_object(container, gwy_app_get_data_key_for_id(0), dfield);
-    gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(0), "TEM");
+    file = gwy_file_new_in_construction();
+    gwy_file_pass_image(file, 0, dfield);
+    gwy_file_set_title(file, GWY_FILE_IMAGE, 0, "TEM", FALSE);
     if ((meta = get_meta(comment)))
-        gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(0), meta);
-    gwy_file_channel_import_log_add(container, 0, NULL, filename);
+        gwy_file_pass_meta(file, GWY_FILE_IMAGE, 0, meta);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
 
 fail:
     g_free(comment);
@@ -215,7 +211,7 @@ fail:
         reader = NULL;
     }
 
-    return container;
+    return file;
 }
 
 static void
@@ -307,7 +303,7 @@ get_meta(const gchar *comment)
     mpd.label = g_string_new(NULL);
     mpd.value = g_string_new(NULL);
     mpd.unit = g_string_new(NULL);
-    mpd.meta = gwy_container_new();
+    mpd.meta = gwy_container_new_in_construction();
 
     context = g_markup_parse_context_new(&parser, G_MARKUP_TREAT_CDATA_AS_TEXT, &mpd, NULL);
     g_markup_parse_context_parse(context, comment, -1, NULL);
