@@ -1,6 +1,6 @@
 /*
- *  $Id: tescan.c 26345 2024-05-15 13:32:22Z yeti-dn $
- *  Copyright (C) 2013-2019 David Necas (Yeti).
+ *  $Id: tescan.c 28789 2025-11-04 17:14:03Z yeti-dn $
+ *  Copyright (C) 2013-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -39,11 +39,9 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/stats.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 #include "err.h"
 #include "gwytiff.h"
 
@@ -76,12 +74,17 @@ typedef struct {
 } BlockCopyInfo;
 
 static gboolean            module_register       (void);
-static gint                tsctif_detect         (const GwyFileDetectInfo *fileinfo,
+static gint                detect_tiffile        (const GwyFileDetectInfo *fileinfo,
                                                   gboolean only_name);
-static GwyContainer*       tsctif_load           (const gchar *filename,
-                                                  GwyRunType mode,
+static GwyFile*            load_tiffile          (const gchar *filename,
+                                                  GwyRunModeFlags mode,
                                                   GError **error);
-static GwyContainer*       tsctif_load_tiff      (GwyTIFF *tiff,
+static gint                detect_pngfile        (const GwyFileDetectInfo *fileinfo,
+                                                  gboolean only_name);
+static GwyFile*            load_pngfile          (const gchar *filename,
+                                                  GwyRunModeFlags mode,
+                                                  GError **error);
+static GwyFile*            tsctif_load_tiff      (GwyTIFF *tiff,
                                                   const GwyTIFFEntry *entry,
                                                   GError **error);
 static const GwyTIFFEntry* tsctif_find_header    (GwyTIFF *tiff,
@@ -89,11 +92,6 @@ static const GwyTIFFEntry* tsctif_find_header    (GwyTIFF *tiff,
 static GArray*             tsctif_get_blocks     (GwyTIFF *tiff,
                                                   const GwyTIFFEntry *entry,
                                                   GError **error);
-static gint                tschdr_detect         (const GwyFileDetectInfo *fileinfo,
-                                                  gboolean only_name);
-static GwyContainer*       tschdr_load           (const gchar *filename,
-                                                  GwyRunType mode,
-                                                  GError **error);
 static gboolean            tschdr_find_image_file(GString *str);
 static void                parse_text_fields     (GHashTable *globalhash,
                                                   const gchar *prefix,
@@ -105,7 +103,7 @@ static GwyContainer*       get_meta              (GHashTable *hash);
 static void                add_meta              (gpointer hkey,
                                                   gpointer hvalue,
                                                   gpointer user_data);
-static GwyDataField*       data_field_from_pixbuf(GdkPixbuf *pixbuf,
+static GwyField*           field_from_pixbuf     (GdkPixbuf *pixbuf,
                                                   GHashTable *hash);
 
 static GwyModuleInfo module_info = {
@@ -113,7 +111,7 @@ static GwyModuleInfo module_info = {
     module_register,
     N_("Imports Tescan SEM images."),
     "Yeti <yeti@gwyddion.net>",
-    "2.0",
+    "3.0",
     "David Nečas (Yeti)",
     "2013",
 };
@@ -125,22 +123,16 @@ module_register(void)
 {
     gwy_file_func_register("tescan-tif",
                            N_("Tescan TIF SEM image (.tif)"),
-                           (GwyFileDetectFunc)&tsctif_detect,
-                           (GwyFileLoadFunc)&tsctif_load,
-                           NULL,
-                           NULL);
+                           detect_tiffile, load_tiffile, NULL, NULL);
     gwy_file_func_register("tescan-png",
                            N_("Tescan two-part SEM image (.hdr + .png)"),
-                           (GwyFileDetectFunc)&tschdr_detect,
-                           (GwyFileLoadFunc)&tschdr_load,
-                           NULL,
-                           NULL);
+                           detect_pngfile, load_pngfile, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-tsctif_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
+detect_tiffile(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     GwyTIFF *tiff;
     guint score = 0;
@@ -164,14 +156,54 @@ tsctif_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
     return score;
 }
 
-static GwyContainer*
-tsctif_load(const gchar *filename,
-            G_GNUC_UNUSED GwyRunType mode,
-            GError **error)
+static gint
+detect_pngfile(const GwyFileDetectInfo *fileinfo, gboolean only_name)
+{
+    static const gchar fields[] = "AccFrames=Device=Magnification=PixelSizeX=PixelSizeY=UserName=";
+    guint score = 0;
+    const gchar *s, *end;
+    GString *str;
+
+    if (only_name)
+        return score;
+
+    /* We can't find the image file name if this is not satisfied. */
+    if (!g_str_has_suffix(fileinfo->name_lowercase, ".hdr"))
+        return 0;
+
+    if (strncmp(fileinfo->head, "[MAIN]", 6))
+        return score;
+
+    s = fields;
+    while ((end = strchr(s, '='))) {
+        end++;
+        if (gwy_memmem(fileinfo->head, fileinfo->buffer_len, s, end - s)) {
+            gwy_debug("Found %.*s", (gint)(end-s), s);
+            score++;
+        }
+        s = end;
+    }
+
+    if (score < 4)
+        return 0;
+
+    /* It might be a Tescan header file.  Look for the image file. */
+    str = g_string_new(fileinfo->name);
+    gwy_debug("Looking for image file for %s", fileinfo->name);
+    score = (tschdr_find_image_file(str) ? 100 : 0);
+    g_string_free(str, TRUE);
+
+    return score;
+}
+
+static GwyFile*
+load_tiffile(const gchar *filename,
+             G_GNUC_UNUSED GwyRunModeFlags mode,
+             GError **error)
 {
     GwyTIFF *tiff;
     const GwyTIFFEntry *entry;
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
 
     tiff = gwy_tiff_load(filename, error);
     if (!tiff)
@@ -183,20 +215,21 @@ tsctif_load(const gchar *filename,
         return NULL;
     }
 
-    container = tsctif_load_tiff(tiff, entry, error);
-    if (container)
-        gwy_file_channel_import_log_add(container, 0, NULL, filename);
+    file = tsctif_load_tiff(tiff, entry, error);
+    if (file)
+        gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
 
     gwy_tiff_free(tiff);
 
-    return container;
+    return file;
 }
 
-static GwyContainer*
+static GwyFile*
 tsctif_load_tiff(GwyTIFF *tiff, const GwyTIFFEntry *entry, GError **error)
 {
-    GwyContainer *container = NULL, *meta;
-    GwyDataField *dfield;
+    GwyFile *file = NULL;
+    GwyContainer *meta;
+    GwyField *dfield;
     GwyTIFFImageReader *reader = NULL;
     GHashTable *hash = NULL;
     GArray *blocks = NULL;
@@ -245,24 +278,24 @@ tsctif_load_tiff(GwyTIFF *tiff, const GwyTIFFEntry *entry, GError **error)
     if (!(reader = gwy_tiff_get_image_reader(tiff, 0, 1, error)))
         goto fail;
 
-    dfield = gwy_data_field_new(reader->width, reader->height, reader->width * xstep, reader->height * ystep, FALSE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
+    dfield = gwy_field_new(reader->width, reader->height, reader->width * xstep, reader->height * ystep, FALSE);
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
 
-    data = gwy_data_field_get_data(dfield);
+    data = gwy_field_get_data(dfield);
     for (i = 0; i < reader->height; i++) {
         gwy_tiff_read_image_row(tiff, reader, 0, i, 1.0/((1 << reader->bits_per_sample) - 1), 0.0,
                                 data + i*reader->width);
     }
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
 
-    gwy_container_pass_object(container, gwy_app_get_data_key_for_id(0), dfield);
+    gwy_file_pass_image(file, 0, dfield);
 
     /* FIXME: Just kidding. */
-    gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(0), "Intensity");
+    gwy_file_set_title(file, GWY_FILE_IMAGE, 0, "Intensity", FALSE);
 
     if ((meta = get_meta(hash)))
-        gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(0), meta);
+        gwy_file_pass_meta(file, GWY_FILE_IMAGE, 0, meta);
 
 fail:
     if (hash)
@@ -274,7 +307,7 @@ fail:
         reader = NULL;
     }
 
-    return container;
+    return file;
 }
 
 static const GwyTIFFEntry*
@@ -346,58 +379,19 @@ fail:
     return NULL;
 }
 
-static gint
-tschdr_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
+static GwyFile*
+load_pngfile(const gchar *filename,
+             G_GNUC_UNUSED GwyRunModeFlags mode,
+             GError **error)
 {
-    static const gchar fields[] = "AccFrames=Device=Magnification=PixelSizeX=PixelSizeY=UserName=";
-    guint score = 0;
-    const gchar *s, *end;
-    GString *str;
-
-    if (only_name)
-        return score;
-
-    /* We can't find the image file name if this is not satisfied. */
-    if (!g_str_has_suffix(fileinfo->name_lowercase, ".hdr"))
-        return 0;
-
-    if (strncmp(fileinfo->head, "[MAIN]", 6))
-        return score;
-
-    s = fields;
-    while ((end = strchr(s, '='))) {
-        end++;
-        if (gwy_memmem(fileinfo->head, fileinfo->buffer_len, s, end - s)) {
-            gwy_debug("Found %.*s", (gint)(end-s), s);
-            score++;
-        }
-        s = end;
-    }
-
-    if (score < 4)
-        return 0;
-
-    /* It might be a Tescan header file.  Look for the image file. */
-    str = g_string_new(fileinfo->name);
-    gwy_debug("Looking for image file for %s", fileinfo->name);
-    score = (tschdr_find_image_file(str) ? 100 : 0);
-    g_string_free(str, TRUE);
-
-    return score;
-}
-
-static GwyContainer*
-tschdr_load(const gchar *filename,
-            G_GNUC_UNUSED GwyRunType mode,
-            GError **error)
-{
-    GwyContainer *container = NULL, *meta;
+    GwyFile *file = NULL;
+    GwyContainer *meta;
     GHashTable *hash = NULL;
     gchar *header = NULL;
     GString *imagefilename = NULL;
     GdkPixbuf *pixbuf = NULL;
     GwyTextHeaderParser parser;
-    GwyDataField *dfield;
+    GwyField *dfield;
     GError *err = NULL;
     gsize size;
 
@@ -406,7 +400,7 @@ tschdr_load(const gchar *filename,
         return NULL;
     }
 
-    gwy_clear(&parser, 1);
+    gwy_clear1(parser);
     parser.key_value_separator = "=";
     parser.section_template = "[\x1a]";
     parser.section_accessor = "::";
@@ -428,19 +422,19 @@ tschdr_load(const gchar *filename,
         g_clear_error(&err);
         goto fail;
     }
-    dfield = data_field_from_pixbuf(pixbuf, hash);
+    dfield = field_from_pixbuf(pixbuf, hash);
     g_object_unref(pixbuf);
 
-    container = gwy_container_new();
-    gwy_container_pass_object(container, gwy_app_get_data_key_for_id(0), dfield);
+    file = gwy_file_new_in_construction();
+    gwy_file_pass_image(file, 0, dfield);
 
     /* FIXME: Just kidding. */
-    gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(0), "Intensity");
+    gwy_file_set_title(file, GWY_FILE_IMAGE, 0, "Intensity", FALSE);
 
     if ((meta = get_meta(hash)))
-        gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(0), meta);
+        gwy_file_pass_meta(file, GWY_FILE_IMAGE, 0, meta);
 
-    gwy_file_channel_import_log_add(container, 0, NULL, filename);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
 
 fail:
     if (hash)
@@ -449,7 +443,7 @@ fail:
         g_string_free(imagefilename, TRUE);
     g_free(header);
 
-    return container;
+    return file;
 }
 
 static gboolean
@@ -497,7 +491,7 @@ parse_text_fields(GHashTable *globalhash, const gchar *prefix,
     memcpy(data, block->data, block->size-2);
     data[block->size-2] = '\0';
 
-    gwy_clear(&parser, 1);
+    gwy_clear1(parser);
     parser.key_value_separator = "=";
     hash = gwy_text_header_parse(data, &parser, NULL, NULL);
 
@@ -520,7 +514,7 @@ copy_with_prefix(gpointer hkey, gpointer hvalue, gpointer user_data)
 static GwyContainer*
 get_meta(GHashTable *hash)
 {
-    GwyContainer *meta = gwy_container_new();
+    GwyContainer *meta = gwy_container_new_in_construction();
     g_hash_table_foreach(hash, add_meta, meta);
     if (gwy_container_get_n_items(meta))
         return meta;
@@ -541,10 +535,10 @@ add_meta(gpointer hkey, gpointer hvalue, gpointer user_data)
     gwy_container_set_const_string_by_name(meta, skey, value);
 }
 
-static GwyDataField*
-data_field_from_pixbuf(GdkPixbuf *pixbuf, GHashTable *hash)
+static GwyField*
+field_from_pixbuf(GdkPixbuf *pixbuf, GHashTable *hash)
 {
-    GwyDataField *dfield;
+    GwyField *dfield;
     gint width, height, rowstride, i, bpp;
     guchar *pixels;
     gdouble *val;
@@ -564,8 +558,8 @@ data_field_from_pixbuf(GdkPixbuf *pixbuf, GHashTable *hash)
     height = gdk_pixbuf_get_height(pixbuf);
     rowstride = gdk_pixbuf_get_rowstride(pixbuf);
     bpp = gdk_pixbuf_get_has_alpha(pixbuf) ? 4 : 3;
-    dfield = gwy_data_field_new(width, height, dx*width, dy*height, FALSE);
-    val = gwy_data_field_get_data(dfield);
+    dfield = gwy_field_new(width, height, dx*width, dy*height, FALSE);
+    val = gwy_field_get_data(dfield);
 
     for (i = 0; i < height; i++) {
         guchar *p = pixels + i*rowstride;
@@ -579,7 +573,7 @@ data_field_from_pixbuf(GdkPixbuf *pixbuf, GHashTable *hash)
         }
     }
 
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
 
     return dfield;
 }
