@@ -1,6 +1,6 @@
 /*
- *  $Id: tiaser.c 28573 2025-09-17 13:55:53Z yeti-dn $
- *  Copyright (C) 2012 David Necas (Yeti), Daniil Bratashov (dn2010).
+ *  $Id: tiaser.c 28803 2025-11-05 11:58:19Z yeti-dn $
+ *  Copyright (C) 2012-2025 David Necas (Yeti), Daniil Bratashov (dn2010).
  *  E-mail: yeti@gwyddion.net, dn2010@gmail.com.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -48,15 +48,10 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/stats.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -130,51 +125,54 @@ typedef struct {
 } TIATimePosTag;
 
 typedef struct {
-    gdouble     calibrationoffset;
-    gdouble     calibrationdelta;
-    gint32      calibrationelement;
-    TIADataType datatype;
-    gint32      arraylength;
-    gchar      *data;
+    gdouble       calibrationoffset;
+    gdouble       calibrationdelta;
+    gint32        calibrationelement;
+    TIADataType   datatype;
+    gint32        arraylength;
+    const guchar *data;
 } TIA1DData;
 
 typedef struct {
-    gdouble     calibrationoffsetx;
-    gdouble     calibrationdeltax;
-    gint32      calibrationelementx;
-    gdouble     calibrationoffsety;
-    gdouble     calibrationdeltay;
-    gint32      calibrationelementy;
-    TIADataType datatype;
-    gint32      arraylengthx;
-    gint32      arraylengthy;
-    gchar      *data;
+    gdouble       calibrationoffsetx;
+    gdouble       calibrationdeltax;
+    gint32        calibrationelementx;
+    gdouble       calibrationoffsety;
+    gdouble       calibrationdeltay;
+    gint32        calibrationelementy;
+    TIADataType   datatype;
+    guint32       arraylengthx;
+    guint32       arraylengthy;
+    const guchar *data;
 } TIA2DData;
 
-static gboolean      module_register       (void);
-static gint          tia_detect            (const GwyFileDetectInfo *fileinfo,
-                                            gboolean only_name);
-static GwyContainer* tia_load              (const gchar *filename,
-                                            GwyRunType mode,
-                                            GError **error);
-static void          tia_load_header       (const guchar *p,
-                                            TIAHeader *header);
-static gboolean      tia_check_header      (TIAHeader *header,
-                                            gsize size);
-static gboolean      tia_load_dimarray     (const guchar *p,
-                                            TIADimension *dimarray,
-                                            gsize size);
-static GwyDataField *tia_read_2d           (const guchar *p,
-                                            gsize size);
-static GwyDataLine  *tia_read_1d_dataline  (const guchar *p,
-                                            gsize size);
+static gboolean       module_register         (void);
+static gint           detect_file             (const GwyFileDetectInfo *fileinfo,
+                                               gboolean only_name);
+static GwyFile*       load_file               (const gchar *filename,
+                                               GwyRunModeFlags mode,
+                                               GError **error);
+static void           load_header             (const guchar *p,
+                                               TIAHeader *header);
+static gboolean       check_header            (TIAHeader *header,
+                                               gsize size);
+static gboolean       load_dimarray           (const guchar *p,
+                                               TIADimension *dimarray,
+                                               gsize size);
+static GwyField*      read_2d_image           (const guchar *p,
+                                               gsize size);
+static GwyLine*       read_1d_line            (const guchar *p,
+                                               gsize size);
+static GwyRawDataType tia_datatype_to_raw_type(TIADataType datatype,
+                                               gdouble *q,
+                                               gboolean *is_complex);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports FEI Tecnai imaging and analysis (former Emispec) files."),
     "dn2010 <dn2010@gmail.com>",
-    "0.5",
+    "0.6",
     "David Nečas (Yeti), Daniil Bratashov (dn2010)",
     "2012",
 };
@@ -186,34 +184,28 @@ module_register(void)
 {
     gwy_file_func_register("tiaser",
                            N_("FEI TIA (Emispec) data"),
-                           (GwyFileDetectFunc)&tia_detect,
-                           (GwyFileLoadFunc)&tia_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-tia_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     gint score = 0;
 
     if (only_name)
         return g_str_has_suffix(fileinfo->name_lowercase, EXTENSION) ? 20 : 0;
 
-    if (fileinfo->buffer_len > MAGIC_SIZE
-        && memcmp(fileinfo->head, MAGIC1, MAGIC_SIZE) == 0)
+    if (fileinfo->buffer_len > MAGIC_SIZE && memcmp(fileinfo->head, MAGIC1, MAGIC_SIZE) == 0)
         score = 100;
-
-    if (fileinfo->buffer_len > MAGIC_SIZE
-        && memcmp(fileinfo->head, MAGIC2, MAGIC_SIZE) == 0)
+    if (fileinfo->buffer_len > MAGIC_SIZE && memcmp(fileinfo->head, MAGIC2, MAGIC_SIZE) == 0)
         score = 100;
 
     return score;
 }
 
-static void tia_load_header(const guchar *p, TIAHeader *header)
+static void load_header(const guchar *p, TIAHeader *header)
 {
     header->byteorder           = gwy_get_guint16_le(&p);
     header->seriesid            = gwy_get_guint16_le(&p);
@@ -238,7 +230,8 @@ static void tia_load_header(const guchar *p, TIAHeader *header)
               header->numberdimensions);
 }
 
-static gboolean tia_check_header(TIAHeader *header, gsize size)
+static gboolean
+check_header(TIAHeader *header, gsize size)
 {
     if ((header->byteorder != TIA_ES_LE)
         || (header->seriesid != TIA_ES_MAGIC)
@@ -256,7 +249,7 @@ static gboolean tia_check_header(TIAHeader *header, gsize size)
 }
 
 static gboolean
-tia_load_dimarray(const guchar *p, TIADimension *dimarray, gsize size)
+load_dimarray(const guchar *p, TIADimension *dimarray, gsize size)
 {
     dimarray->numelements        = gwy_get_guint32_le(&p);
     dimarray->calibrationoffset  = gwy_get_gdouble_le(&p);
@@ -291,11 +284,12 @@ tia_load_dimarray(const guchar *p, TIADimension *dimarray, gsize size)
     return TRUE;
 }
 
-static GwyContainer*
-tia_load(const gchar *filename,
-         G_GNUC_UNUSED GwyRunType mode, GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     guchar *buffer;
     gsize size;
     GError *err = NULL;
@@ -303,9 +297,8 @@ tia_load(const gchar *filename,
     TIAHeader *header;
     TIADimension *dimension, dim;
     GArray *dimarray, *dataoffsets, *tagoffsets;
-    GwyDataField *dfield;
+    GwyField *dfield;
     gint i, offset, dimarraylength, dimarraysize = 0;
-    gchar *strkey;
 
     if (!gwy_file_get_contents(filename, &buffer, &size, &err)) {
         err_GET_FILE_CONTENTS(error, &err);
@@ -319,8 +312,8 @@ tia_load(const gchar *filename,
 
     header = g_new0(TIAHeader, 1);
     p = buffer;
-    tia_load_header(p, header);
-    if (!tia_check_header(header, size)) {
+    load_header(p, header);
+    if (!check_header(header, size)) {
         err_FILE_TYPE(error, "FEI TIA");
         goto fail2;
     }
@@ -330,8 +323,8 @@ tia_load(const gchar *filename,
     dimarray = g_array_sized_new(FALSE, TRUE, sizeof(TIADimension), dimarraylength);
     for (i = 0; i < dimarraylength; i++) {
         p += dimarraysize;
-        gwy_clear(&dim, 1);
-        if (!tia_load_dimarray(p, &dim, size - TIA_HEADER_SIZE - dimarraysize)) {
+        gwy_clear1(dim);
+        if (!load_dimarray(p, &dim, size - TIA_HEADER_SIZE - dimarraysize)) {
             err_FILE_TYPE(error, "FEI TIA");
             goto fail3;
         }
@@ -357,7 +350,7 @@ tia_load(const gchar *filename,
         }
     }
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     if (header->datatypeid == TIA_2D_DATA)
         for (i = 0; i < header->validnumberelements; i++) {
             offset = g_array_index(dataoffsets, gint32, i);
@@ -365,16 +358,11 @@ tia_load(const gchar *filename,
                 gwy_debug("Attempt to read after EOF");
             }
             else {
-                dfield = tia_read_2d(buffer + offset, size - offset);
+                dfield = read_2d_image(buffer + offset, size - offset);
                 if (dfield) {
-                    GQuark key = gwy_app_get_data_key_for_id(i);
-
-                    gwy_container_pass_object(container, key, dfield);
-
-                    strkey = g_strdup_printf("/%u/data/title", i);
-                    gwy_container_set_const_string_by_name(container, strkey, "TEM");
-                    g_free(strkey);
-                    gwy_file_channel_import_log_add(container, i, NULL, filename);
+                    gwy_file_pass_image(file, i, dfield);
+                    gwy_file_set_title(file, GWY_FILE_IMAGE, i, "TEM", FALSE);
+                    gwy_log_add_import(file, GWY_FILE_IMAGE, i, NULL, filename);
                 }
             }
         }
@@ -386,11 +374,11 @@ tia_load(const gchar *filename,
          * Disabled for now. */
 #if 0
         GwyBrick *brick;
-        GwyDataLine *dline;
-        GwySIUnit *siunit;
+        GwyLine *dline;
+        GwyUnit *siunit;
+        gdouble *value, *data;
         gint xres, yres, zres, xpos, ypos;
         gdouble xreal, xoffset, yreal, yoffset, zreal, zoffset;
-        gdouble *value, *data;
 
         if (dimarray->len != 2) {
             gwy_debug("Wrong dimensions number");
@@ -412,17 +400,16 @@ tia_load(const gchar *filename,
             gwy_debug("Attempt to read after EOF");
             goto tagoffsets_fail;
         }
-        dline = tia_read_1d_dataline(buffer + offset, size - offset);
+        dline = read_1d_line(buffer + offset, size - offset);
         if (!dline) {
             goto tagoffsets_fail;
         }
-        zres = gwy_data_line_get_res(dline);
-        zreal = gwy_data_line_get_real(dline);
-        zoffset = gwy_data_line_get_offset(dline);
+        zres = gwy_line_get_res(dline);
+        zreal = gwy_line_get_real(dline);
+        zoffset = gwy_line_get_offset(dline);
         g_object_unref(dline);
 
-        brick = gwy_brick_new(xres, yres, zres,
-                              xreal, yreal, zreal, TRUE);
+        brick = gwy_brick_new(xres, yres, zres, xreal, yreal, zreal, TRUE);
         if (!brick) {
             goto tagoffsets_fail;
         }
@@ -430,13 +417,13 @@ tia_load(const gchar *filename,
         gwy_brick_set_yoffset(brick, yoffset);
         gwy_brick_set_zoffset(brick, zoffset);
 
-        siunit = gwy_si_unit_new("m");
-        gwy_brick_set_si_unit_x(brick, siunit);
-        gwy_brick_set_si_unit_y(brick, siunit);
+        siunit = gwy_unit_new("m");
+        gwy_unit_assign(gwy_brick_get_unit_x(brick), siunit);
+        gwy_unit_assign(gwy_brick_get_unit_y(brick), siunit);
         g_object_unref(siunit);
-        siunit = gwy_si_unit_new(NULL);
-        gwy_brick_set_si_unit_z(brick, siunit);
-        gwy_brick_set_si_unit_w(brick, siunit);
+        siunit = gwy_unit_new(NULL);
+        gwy_unit_assign(gwy_brick_get_unit_z(brick), siunit);
+        gwy_unit_assign(gwy_brick_get_unit_w(brick), siunit);
         g_object_unref(siunit);
         data = gwy_brick_get_data(brick);
 
@@ -446,12 +433,12 @@ tia_load(const gchar *filename,
                 gwy_debug("Attempt to read after EOF");
                 goto tagoffsets_fail;
             }
-            dline = tia_read_1d_dataline(buffer + offset, size - offset);
+            dline = read_1d_line(buffer + offset, size - offset);
             if (dline) {
                 xpos = i % xres;
                 ypos = (gint)(i / xres);
-                zres = gwy_data_line_get_res(dline);
-                value = (gdouble *)gwy_data_line_get_data_const(dline);
+                zres = gwy_line_get_res(dline);
+                value = (gdouble*)gwy_line_get_data_const(dline);
                 for (k = 0; k < zres; k++)
                     *(data + k * xres * yres + ypos * xres + xpos) = *(value++);
             }
@@ -460,9 +447,9 @@ tia_load(const gchar *filename,
             g_object_unref(dline);
         }
         if (brick) {
-            gwy_container_pass_object_by_name(container, "/brick/0", brick);
-            gwy_container_set_const_string_by_name(container, "/brick/0/title", "TEM Spectroscopy");
-            gwy_file_volume_import_log_add(container, 0, NULL, filename);
+            gwy_file_pass_volume(file, 0, brick);
+            gwy_file_set_title(file, GWY_FILE_VOLUME, 0, "TEM Spectroscopy", FALSE);
+            gwy_log_add_import(file, GWY_FILE_VOLUME, 0, NULL, filename);
         }
 #endif
     }
@@ -482,253 +469,132 @@ fail2:
     g_free(header);
 fail:
     gwy_file_abandon_contents(buffer, size, NULL);
-    if (container && !gwy_container_get_n_items(container)) {
-        g_object_unref(container);
-        container = NULL;
-    }
+    if (file && !gwy_container_get_n_items(GWY_CONTAINER(file)))
+        g_clear_object(&file);
 
-    return container;
+    return file;
 }
 
-static GwyDataField* tia_read_2d(const guchar *p, gsize size)
+static GwyField*
+read_2d_image(const guchar *p, gsize size)
 {
-    GwyDataField *dfield = NULL;
-    TIA2DData *fielddata;
-    gint i, n;
-    gdouble *data;
-    gdouble xoffset, yoffset;
-    gint tia_datasizes[11] = { 0, 1, 1, 2, 2, 4, 4, 4, 8, 8, 16 };
+    TIA2DData info;
+    info.calibrationoffsetx  = gwy_get_gdouble_le(&p);
+    info.calibrationdeltax   = gwy_get_gdouble_le(&p);
+    info.calibrationelementx = gwy_get_guint32_le(&p);
+    info.calibrationoffsety  = gwy_get_gdouble_le(&p);
+    info.calibrationdeltay   = gwy_get_gdouble_le(&p);
+    info.calibrationelementy = gwy_get_guint32_le(&p);
+    info.datatype            = (TIADataType)gwy_get_guint16_le(&p);
+    info.arraylengthx        = gwy_get_guint32_le(&p);
+    info.arraylengthy        = gwy_get_guint32_le(&p);
+    info.data                = p;
 
-    fielddata = g_new0(TIA2DData, 1);
-    fielddata->calibrationoffsetx  = gwy_get_gdouble_le(&p);
-    fielddata->calibrationdeltax   = gwy_get_gdouble_le(&p);
-    fielddata->calibrationelementx = gwy_get_guint32_le(&p);
-    fielddata->calibrationoffsety  = gwy_get_gdouble_le(&p);
-    fielddata->calibrationdeltay   = gwy_get_gdouble_le(&p);
-    fielddata->calibrationelementy = gwy_get_guint32_le(&p);
-    fielddata->datatype            = (TIADataType)gwy_get_guint16_le(&p);
-    fielddata->arraylengthx        = gwy_get_guint32_le(&p);
-    fielddata->arraylengthy        = gwy_get_guint32_le(&p);
-    fielddata->data = (gchar *)p;
+    gboolean is_complex;
+    gdouble q;
+    GwyRawDataType rawtype = tia_datatype_to_raw_type(info.datatype, &q, &is_complex);
 
-    if ((fielddata->datatype < TIA_DATA_UINT8)
-     || (fielddata->datatype > TIA_DATA_DOUBLE)
-     || (size < 50 + fielddata->arraylengthx * fielddata->arraylengthy
-              * tia_datasizes[fielddata->datatype])) {
+    if (rawtype == (GwyRawDataType)G_MAXUINT || is_complex) {
         gwy_debug("Unsupported datatype");
-        goto fail_2d;
+        return NULL;
+    }
+
+    gsize itemsize = gwy_raw_data_size(rawtype) * (is_complex ? 2 : 1);
+    if (size < 50 || (size - 50)/info.arraylengthx/info.arraylengthy < itemsize) {
+        gwy_debug("data size mismatch");
+        return NULL;
     }
 
     gwy_debug("X: caloffset=%G caldelta=%G calelem=%i",
-              fielddata->calibrationoffsetx,
-              fielddata->calibrationdeltax,
-              fielddata->calibrationelementx);
+              info.calibrationoffsetx, info.calibrationdeltax, info.calibrationelementx);
     gwy_debug("Y: caloffset=%G caldelta=%G calelem=%i",
-              fielddata->calibrationoffsety,
-              fielddata->calibrationdeltay,
-              fielddata->calibrationelementy);
-    gwy_debug("nx=%i ny=%i type=%i", fielddata->arraylengthx,
-                                     fielddata->arraylengthy,
-                                     fielddata->datatype);
+              info.calibrationoffsety, info.calibrationdeltay, info.calibrationelementy);
+    gwy_debug("nx=%i ny=%i type=%i",
+              info.arraylengthx, info.arraylengthy, info.datatype);
 
-    dfield = gwy_data_field_new(fielddata->arraylengthx,
-                 fielddata->arraylengthy,
-                 fielddata->arraylengthx*fielddata->calibrationdeltax,
-                 fielddata->arraylengthy*fielddata->calibrationdeltay,
-                 TRUE);
-    xoffset = fielddata->calibrationoffsetx
-        - fielddata->calibrationdeltax * fielddata->calibrationelementx;
-    yoffset = fielddata->calibrationoffsety
-        - fielddata->calibrationdeltay * fielddata->calibrationelementy;
-    if (dfield) {
-        gwy_data_field_set_xoffset(dfield, xoffset);
-        gwy_data_field_set_yoffset(dfield, yoffset);
-        gwy_si_unit_set_from_string(
-                            gwy_data_field_get_si_unit_xy(dfield), "m");
-        data = gwy_data_field_get_data(dfield);
-        n = fielddata->arraylengthx * fielddata->arraylengthy;
-        switch (fielddata->datatype) {
-            case TIA_DATA_UINT8:
-            {
-                for (i = 0; i < n; i++)
-                    *(data++) = (*(p++)) / (gdouble)G_MAXUINT8;
-            }
-            break;
-            case TIA_DATA_UINT16:
-            {
-                const guint16 *tp = (const guint16 *)p;
+    GwyField *dfield = gwy_field_new(info.arraylengthx,
+                                     info.arraylengthy,
+                                     info.arraylengthx*info.calibrationdeltax,
+                                     info.arraylengthy*info.calibrationdeltay,
+                                     TRUE);
 
-                for (i = 0; i < n; i++)
-                    *(data++) = GUINT16_FROM_LE(*(tp++))
-                                                 / (gdouble)G_MAXUINT16;
-            }
-            break;
-            case TIA_DATA_UINT32:
-            {
-                const guint32 *tp = (const guint32 *)p;
+    gwy_field_set_xoffset(dfield, info.calibrationoffsetx - info.calibrationdeltax * info.calibrationelementx);
+    gwy_field_set_yoffset(dfield, info.calibrationoffsety - info.calibrationdeltay * info.calibrationelementy);
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
+    gsize n = info.arraylengthx * info.arraylengthy;
+    gwy_convert_raw_data(info.data, n, 1, rawtype, GWY_BYTE_ORDER_LITTLE_ENDIAN,
+                         gwy_field_get_data(dfield), q, 0.0);
 
-                for (i = 0; i < n; i++)
-                    *(data++) = GUINT32_FROM_LE(*(tp++))
-                                                 / (gdouble)G_MAXUINT32;
-            }
-            break;
-            case TIA_DATA_INT8:
-            {
-                const gchar *tp = (const gchar *)p;
-
-                for (i = 0; i < n; i++)
-                    *(data++) = (*(tp++)) / (gdouble)G_MAXINT8;
-            }
-            break;
-            case TIA_DATA_INT16:
-            {
-                const gint16 *tp = (const gint16 *)p;
-
-                for (i = 0; i < n; i++)
-                    *(data++) = GINT16_FROM_LE(*(tp++))
-                                                  / (gdouble)G_MAXINT16;
-            }
-            break;
-            case TIA_DATA_INT32:
-            {
-                const gint32 *tp = (const gint32 *)p;
-
-                for (i = 0; i < n; i++)
-                    *(data++) = GINT32_FROM_LE(*(tp++))
-                                                  / (gdouble)G_MAXINT32;
-            }
-            break;
-            case TIA_DATA_FLOAT:
-            {
-                for (i = 0; i < n; i++)
-                    *(data++) = gwy_get_gfloat_le(&p);
-            }
-            break;
-            case TIA_DATA_DOUBLE:
-            {
-                for (i = 0; i < n; i++)
-                    *(data++) = gwy_get_gdouble_le(&p);
-            }
-            break;
-            default:
-            g_assert_not_reached();
-            break;
-        }
-    }
-
-    fail_2d:
-    g_free(fielddata);
     return dfield;
 }
 
-static GwyDataLine *tia_read_1d_dataline(const guchar *p, gsize size)
+static GwyLine*
+read_1d_line(const guchar *p, gsize size)
 {
-    TIA1DData *spectradata;
-    GwyDataLine *dline = NULL;
-    gint i, n;
-    gdouble *data;
-    gint tia_datasizes[11] = { 0, 1, 1, 2, 2, 4, 4, 4, 8, 8, 16 };
+    TIA1DData info;
+    info.calibrationoffset  = gwy_get_gdouble_le(&p);
+    info.calibrationdelta   = gwy_get_gdouble_le(&p);
+    info.calibrationelement = gwy_get_guint32_le(&p);
+    info.datatype           = (TIADataType)gwy_get_guint16_le(&p);
+    info.arraylength        = gwy_get_guint32_le(&p);
+    info.data               = p;
 
-    spectradata = g_new0(TIA1DData, 1);
-    spectradata->calibrationoffset  = gwy_get_gdouble_le(&p);
-    spectradata->calibrationdelta   = gwy_get_gdouble_le(&p);
-    spectradata->calibrationelement = gwy_get_guint32_le(&p);
-    spectradata->datatype           = (TIADataType)gwy_get_guint16_le(&p);
-    spectradata->arraylength        = gwy_get_guint32_le(&p);
-    spectradata->data               = (gchar *)p;
+    gboolean is_complex;
+    gdouble q;
+    GwyRawDataType rawtype = tia_datatype_to_raw_type(info.datatype, &q, &is_complex);
 
-    if ((spectradata->datatype < TIA_DATA_UINT8)
-     || (spectradata->datatype > TIA_DATA_DOUBLE)
-     || (size < 50 + spectradata->arraylength
-                              * tia_datasizes[spectradata->datatype])) {
+    if (rawtype == (GwyRawDataType)G_MAXUINT || is_complex) {
         gwy_debug("Unsupported datatype");
-        goto fail_1d;
+        return NULL;
     }
 
-    dline = gwy_data_line_new(spectradata->arraylength,
-               spectradata->arraylength * spectradata->calibrationdelta,
-               TRUE);
-    if (!dline) {
-        gwy_debug("Failed to create dataline");
-        goto fail_1d;
+    gsize itemsize = gwy_raw_data_size(rawtype) * (is_complex ? 2 : 1);
+    if (size < 26 || (size - 26)/info.arraylength < itemsize) {
+        gwy_debug("data size mismatch");
+        return NULL;
     }
 
-    gwy_data_line_set_offset(dline, spectradata->calibrationoffset
-     - spectradata->calibrationdelta * spectradata->calibrationelement);
-    data = gwy_data_line_get_data(dline);
+    GwyLine *dline = gwy_line_new(info.arraylength,
+                                  info.arraylength * info.calibrationdelta,
+                                  TRUE);
 
-    n = spectradata->arraylength;
-    switch (spectradata->datatype) {
-        case TIA_DATA_UINT8:
-        {
-            for (i = 0; i < n; i++)
-                *(data++) = (*(p++)) / (gdouble)G_MAXUINT8;
-        }
-        break;
-        case TIA_DATA_UINT16:
-        {
-            const guint16 *tp = (const guint16 *)p;
+    gwy_line_set_offset(dline, info.calibrationoffset - info.calibrationdelta * info.calibrationelement);
+    gsize n = info.arraylength;
+    gwy_convert_raw_data(info.data, n, 1, rawtype, GWY_BYTE_ORDER_LITTLE_ENDIAN,
+                         gwy_line_get_data(dline), q, 0.0);
 
-            for (i = 0; i < n; i++)
-                *(data++) = GUINT16_FROM_LE(*(tp++))
-                                             / (gdouble)G_MAXUINT16;
-        }
-        break;
-        case TIA_DATA_UINT32:
-        {
-            const guint32 *tp = (const guint32 *)p;
-
-            for (i = 0; i < n; i++)
-                *(data++) = GUINT32_FROM_LE(*(tp++))
-                                             / (gdouble)G_MAXUINT32;
-        }
-        break;
-        case TIA_DATA_INT8:
-        {
-            const gchar *tp = (const gchar *)p;
-
-            for (i = 0; i < n; i++)
-                *(data++) = (*(tp++)) / (gdouble)G_MAXINT8;
-        }
-        break;
-        case TIA_DATA_INT16:
-        {
-            const gint16 *tp = (const gint16 *)p;
-
-            for (i = 0; i < n; i++)
-                *(data++) = GINT16_FROM_LE(*(tp++))
-                                              / (gdouble)G_MAXINT16;
-        }
-        break;
-        case TIA_DATA_INT32:
-        {
-            const gint32 *tp = (const gint32 *)p;
-
-            for (i = 0; i < n; i++)
-                *(data++) = GINT32_FROM_LE(*(tp++))
-                                              / (gdouble)G_MAXINT32;
-        }
-        break;
-        case TIA_DATA_FLOAT:
-        {
-            for (i = 0; i < n; i++)
-                *(data++) = gwy_get_gfloat_le(&p);
-        }
-        break;
-        case TIA_DATA_DOUBLE:
-        {
-            for (i = 0; i < n; i++)
-                *(data++) = gwy_get_gdouble_le(&p);
-        }
-        break;
-        default:
-        g_assert_not_reached();
-        break;
-    }
-
-    fail_1d:
-    g_free(spectradata);
     return dline;
 }
 
+static GwyRawDataType
+tia_datatype_to_raw_type(TIADataType datatype, gdouble *q, gboolean *is_complex)
+{
+    static const struct {
+        TIADataType tiatype;
+        GwyRawDataType rawtype;
+        gdouble q;
+        gboolean is_complex;
+    } conversion_table[] = {
+        { TIA_DATA_UINT8,          GWY_RAW_DATA_UINT8,  1.0/G_MAXUINT8,  FALSE, },
+        { TIA_DATA_UINT16,         GWY_RAW_DATA_UINT16, 1.0/G_MAXUINT16, FALSE, },
+        { TIA_DATA_UINT32,         GWY_RAW_DATA_UINT32, 1.0/G_MAXUINT32, FALSE, },
+        { TIA_DATA_INT8,           GWY_RAW_DATA_SINT8,  1.0/G_MAXINT8,   FALSE, },
+        { TIA_DATA_INT16,          GWY_RAW_DATA_SINT16, 1.0/G_MAXINT16,  FALSE, },
+        { TIA_DATA_INT32,          GWY_RAW_DATA_SINT32, 1.0/G_MAXINT32,  FALSE, },
+        { TIA_DATA_FLOAT,          GWY_RAW_DATA_FLOAT,  1.0,             FALSE, },
+        { TIA_DATA_DOUBLE,         GWY_RAW_DATA_DOUBLE, 1.0,             FALSE, },
+        { TIA_DATA_FLOAT_COMPLEX,  GWY_RAW_DATA_FLOAT,  1.0,             TRUE,  },
+        { TIA_DATA_DOUBLE_COMPLEX, GWY_RAW_DATA_DOUBLE, 1.0,             TRUE,  },
+    };
+
+    for (guint i = 0; i < G_N_ELEMENTS(conversion_table); i++) {
+        if (conversion_table[i].tiatype == datatype) {
+            *is_complex = conversion_table[i].is_complex;
+            *q = conversion_table[i].q;
+            return conversion_table[i].rawtype;
+        }
+    }
+    return (GwyRawDataType)G_MAXUINT;
+}
+
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
