@@ -1,6 +1,6 @@
 /*
- *  $Id: ttffile.c 25669 2023-09-14 14:49:06Z yeti-dn $
- *  Copyright (C) 2019-2021 David Necas (Yeti), Thomas Wagner.
+ *  $Id: ttffile.c 28910 2025-11-24 18:11:09Z yeti-dn $
+ *  Copyright (C) 2019-2025 David Necas (Yeti), Thomas Wagner.
  *  E-mail: yeti@gwyddion.net, hirschbeutel@gmail.com.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -35,11 +35,9 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/correct.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 #include "err.h"
 #include "gwytiff.h"
 
@@ -140,11 +138,11 @@ typedef struct {
 static gboolean      module_register        (void);
 static gint          ttf_detect             (const GwyFileDetectInfo *fileinfo,
                                              gboolean only_name);
-static GwyContainer* ttf_load               (const gchar *filename,
-                                             GwyRunType mode,
+static GwyFile*      ttf_load               (const gchar *filename,
+                                             GwyRunModeFlags mode,
                                              GError **error);
 static gboolean      ttf_load_image         (GwyTIFF *tiff,
-                                             GwyContainer *container,
+                                             GwyFile *file,
                                              gint dirno,
                                              GError **error);
 static guint         read_image_data        (const guchar *p,
@@ -162,16 +160,16 @@ static GwyContainer* ttf_get_meta           (GwyTIFF *tiff,
                                              gint dirno);
 static gint          ccsv_detect            (const GwyFileDetectInfo *fileinfo,
                                              gboolean only_name);
-static GwyContainer* ccsv_load              (const gchar *filename,
-                                             GwyRunType mode,
+static GwyFile*      ccsv_load              (const gchar *filename,
+                                             GwyRunModeFlags mode,
                                              GError **error);
 static gchar*        ccsv_read_header       (CorningCSVHeader *header,
                                              gchar *p,
                                              GError **error);
 static gboolean      ccsv_read_images       (const CorningCSVHeader *header,
                                              gchar *p,
-                                             GwyDataField **dfield,
-                                             GwyDataField **mask,
+                                             GwyField **dfield,
+                                             GwyField **mask,
                                              GError **error);
 
 static GwyModuleInfo module_info = {
@@ -179,7 +177,7 @@ static GwyModuleInfo module_info = {
     module_register,
     N_("Imports Corning Tropel UltraSort files."),
     "Yeti <yeti@gwyddion.net>, Thomas Wagner <hirschbeutel@gmail.com>",
-    "1.0",
+    "2.0",
     "David Nečas (Yeti), Thomas Wagner",
     "2019",
 };
@@ -191,16 +189,10 @@ module_register(void)
 {
     gwy_file_func_register("ttffile",
                            N_("Corning Tropel UltraSort data (.ttf)"),
-                           (GwyFileDetectFunc)&ttf_detect,
-                           (GwyFileLoadFunc)&ttf_load,
-                           NULL,
-                           NULL);
+                           &ttf_detect, &ttf_load, NULL, NULL);
     gwy_file_func_register("corningcsvfile",
                            N_("Corning Tropel UltraSort CSV export (.csv)"),
-                           (GwyFileDetectFunc)&ccsv_detect,
-                           (GwyFileLoadFunc)&ccsv_load,
-                           NULL,
-                           NULL);
+                           &ccsv_detect, &ccsv_load, NULL, NULL);
 
     return TRUE;
 }
@@ -225,17 +217,17 @@ ttf_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 
     /* Use GwyTIFF for detection to avoid problems with fragile libtiff. Progressively try finer tests. */
     if ((tiff = gwy_tiff_load(fileinfo->name, NULL))
-         && gwy_tiff_get_string0(tiff, GWY_TIFFTAG_MAKE, &make)
-         && gwy_tiff_get_string0(tiff, GWY_TIFFTAG_MODEL, &model)
-         && gwy_tiff_find_tag(tiff, 0, CORNING_TIFFTAG_FIRST)
-         && (entry = gwy_tiff_find_tag(tiff, 0, CORNING_TIFFTAG_INVDX))
-         && entry->type == (guint)CORNING_TIFF_DOUBLE
-         && (entry = gwy_tiff_find_tag(tiff, 0, CORNING_TIFFTAG_SENS))
-         && entry->type == (guint)CORNING_TIFF_DOUBLE
-         && (entry = gwy_tiff_find_tag(tiff, 0, CORNING_TIFFTAG_DATA))
-         && entry->type == (guint)GWY_TIFF_LONG
-         && gwy_tiff_get_uint0(tiff, CORNING_TIFFTAG_XRES, &xres)
-         && gwy_tiff_get_uint0(tiff, CORNING_TIFFTAG_YRES, &yres))
+        && gwy_tiff_get_string0(tiff, GWY_TIFFTAG_MAKE, &make)
+        && gwy_tiff_get_string0(tiff, GWY_TIFFTAG_MODEL, &model)
+        && gwy_tiff_find_tag(tiff, 0, CORNING_TIFFTAG_FIRST)
+        && (entry = gwy_tiff_find_tag(tiff, 0, CORNING_TIFFTAG_INVDX))
+        && entry->type == (guint)CORNING_TIFF_DOUBLE
+        && (entry = gwy_tiff_find_tag(tiff, 0, CORNING_TIFFTAG_SENS))
+        && entry->type == (guint)CORNING_TIFF_DOUBLE
+        && (entry = gwy_tiff_find_tag(tiff, 0, CORNING_TIFFTAG_DATA))
+        && entry->type == (guint)GWY_TIFF_LONG
+        && gwy_tiff_get_uint0(tiff, CORNING_TIFFTAG_XRES, &xres)
+        && gwy_tiff_get_uint0(tiff, CORNING_TIFFTAG_YRES, &yres))
         score = 100;
 
     /* We could check these for "UltraSort" and "Corning Tropel", but who knows how stable they are. */
@@ -247,13 +239,40 @@ ttf_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
     return score;
 }
 
-static GwyContainer*
+static gint
+ccsv_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
+{
+    /* They many not be all there, for instance of the last four we expect about two... */
+    const gchar *wanted_strings[] = {
+        "Time: ", "Size: ", "Zoom: ", "Units: ", "ZRes: ", "Outside: ", "Sensitivity: ", "Scale: ", "Mapformat: ",
+        "Tropel", "Corning", "UltraSort", "TMSPlot",
+    };
+    guint i, is_not_ccsv = 100;
+
+    if (only_name)
+        return 0;
+
+    if (strncmp(fileinfo->head, MAGIC, MAGIC_SIZE))
+        return 0;
+
+    for (i = 0; i < G_N_ELEMENTS(wanted_strings); i++) {
+        if (strstr(fileinfo->head, wanted_strings[i])) {
+            gwy_debug("found %s", wanted_strings[i]);
+            is_not_ccsv = 2*is_not_ccsv/3;
+        }
+    }
+    gwy_debug("is_not %d", is_not_ccsv);
+
+    return 100 - is_not_ccsv;
+}
+
+static GwyFile*
 ttf_load(const gchar *filename,
-         G_GNUC_UNUSED GwyRunType mode,
+         G_GNUC_UNUSED GwyRunModeFlags mode,
          GError **error)
 {
     GwyTIFF *tiff;
-    GwyContainer *container = NULL, *meta;
+    GwyFile *file = NULL;
     gint ndirs, i;
 
     tiff = gwy_tiff_load(filename, error);
@@ -267,26 +286,25 @@ ttf_load(const gchar *filename,
     if (!fix_corning_double_tags(tiff, error))
         goto fail;
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     for (i = 0; i < ndirs; i++) {
-        if (!ttf_load_image(tiff, container, i, error)) {
-            GWY_OBJECT_UNREF(container);
+        if (!ttf_load_image(tiff, file, i, error)) {
+            g_clear_object(&file);
             break;
         }
-        meta = ttf_get_meta(tiff, i);
-        gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(i), meta);
+        gwy_file_pass_meta(file, GWY_FILE_IMAGE, i, ttf_get_meta(tiff, i));
     }
 
 fail:
     gwy_tiff_free(tiff);
 
-    return container;
+    return file;
 }
 
 static gboolean
-ttf_load_image(GwyTIFF *tiff, GwyContainer *container, gint dirno, GError **error)
+ttf_load_image(GwyTIFF *tiff, GwyFile *file, gint dirno, GError **error)
 {
-    GwyDataField *field = NULL, *mask = NULL;
+    GwyField *field = NULL, *mask = NULL;
     const GwyTIFFEntry *entry;
     const guchar *p;
     gulong size, offset;
@@ -358,14 +376,14 @@ ttf_load_image(GwyTIFF *tiff, GwyContainer *container, gint dirno, GError **erro
         err_INVALID(error, "NaNCountNBits");
         return FALSE;
     }
-    field = gwy_data_field_new(xres, yres, 1e-3*xres/invdx, 1e-3*yres/invdx, FALSE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(field), "m");
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(field), "m");
+    field = gwy_field_new(xres, yres, 1e-3*xres/invdx, 1e-3*yres/invdx, FALSE);
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(field), "m");
+    gwy_unit_set_from_string(gwy_field_get_unit_z(field), "m");
 
-    mask = gwy_data_field_new_alike(field, TRUE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(mask), NULL);
+    mask = gwy_field_new_alike(field, TRUE);
+    gwy_unit_clear(gwy_field_get_unit_z(mask));
 
-    nread = read_image_data(p, size, gwy_data_field_get_data(field), gwy_data_field_get_data(mask),
+    nread = read_image_data(p, size, gwy_field_get_data(field), gwy_field_get_data(mask),
                             delta_nbits, data_nbits, nan_count_nbits, n, sens/scale * Micrometre);
     if (nread < n) {
         g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA, _("Data block is truncated."));
@@ -375,15 +393,15 @@ ttf_load_image(GwyTIFF *tiff, GwyContainer *container, gint dirno, GError **erro
     }
 
     if (!gwy_app_channel_remove_bad_data(field, mask))
-        GWY_OBJECT_UNREF(mask);
+        g_clear_object(&mask);
 
-    gwy_container_pass_object(container, gwy_app_get_data_key_for_id(dirno), field);
+    gwy_file_pass_image(file, dirno, field);
     if (mask) {
-        gwy_container_pass_object(container, gwy_app_get_mask_key_for_id(dirno), mask);
+        gwy_file_pass_image_mask(file, dirno, mask);
     }
 
     if (gwy_tiff_get_string(tiff, dirno, CORNING_TIFFTAG_MAPID, &title))
-        gwy_container_set_string(container, gwy_app_get_data_title_key_for_id(dirno), title);
+        gwy_file_pass_title(file, GWY_FILE_IMAGE, dirno, title);
 
     return TRUE;
 }
@@ -506,7 +524,6 @@ fix_corning_double_tags(GwyTIFF *tiff, GError **error)
             if ((guint)entry->type == CORNING_TIFF_DOUBLE)
                 entry->type = GWY_TIFF_DOUBLE;
         }
-
     }
 
     return gwy_tiff_tags_valid(tiff, error);
@@ -554,13 +571,12 @@ ttf_get_meta(GwyTIFF *tiff, gint dirno)
         { "Map id",             CORNING_TIFFTAG_MAPID,     },
     };
 
-    GwyContainer *meta;
     gchar *s;
     gdouble d;
     guint i, u;
     gchar buf[32];
 
-    meta = gwy_container_new();
+    GwyContainer *meta = gwy_container_new_in_construction();
 
     for (i = 0; i < G_N_ELEMENTS(dir0_tags); i++) {
         if (gwy_tiff_get_string0(tiff, dir0_tags[i].value, &s))
@@ -586,47 +602,19 @@ ttf_get_meta(GwyTIFF *tiff, gint dirno)
     return meta;
 }
 
-static gint
-ccsv_detect(const GwyFileDetectInfo *fileinfo,
-            gboolean only_name)
-{
-    /* They many not be all there, for instance of the last four we expect about two... */
-    const gchar *wanted_strings[] = {
-        "Time: ", "Size: ", "Zoom: ", "Units: ", "ZRes: ", "Outside: ", "Sensitivity: ", "Scale: ", "Mapformat: ",
-        "Tropel", "Corning", "UltraSort", "TMSPlot",
-    };
-    guint i, is_not_ccsv = 100;
-
-    if (only_name)
-        return 0;
-
-    if (strncmp(fileinfo->head, MAGIC, MAGIC_SIZE))
-        return 0;
-
-    for (i = 0; i < G_N_ELEMENTS(wanted_strings); i++) {
-        if (strstr(fileinfo->head, wanted_strings[i])) {
-            gwy_debug("found %s", wanted_strings[i]);
-            is_not_ccsv = 2*is_not_ccsv/3;
-        }
-    }
-    gwy_debug("is_not %d", is_not_ccsv);
-
-    return 100 - is_not_ccsv;
-}
-
-static GwyContainer*
+static GwyFile*
 ccsv_load(const gchar *filename,
-          G_GNUC_UNUSED GwyRunType mode,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
           GError **error)
 {
     CorningCSVHeader header;
-    GwyContainer *container = NULL;
-    GwyDataField *dfield = NULL, *mask = NULL;
+    GwyFile *file = NULL;
+    GwyField *dfield = NULL, *mask = NULL;
     GError *err = NULL;
     gchar *buffer, *p;
     gsize size;
 
-    gwy_clear(&header, 1);
+    gwy_clear1(header);
     if (!g_file_get_contents(filename, &buffer, &size, &err)) {
         err_GET_FILE_CONTENTS(error, &err);
         return NULL;
@@ -643,19 +631,19 @@ ccsv_load(const gchar *filename,
     if (!ccsv_read_images(&header, p, &dfield, &mask, error))
         goto fail;
 
-    container = gwy_container_new();
-    gwy_container_set_object(container, gwy_app_get_data_key_for_id(0), dfield);
-    gwy_container_set_object(container, gwy_app_get_mask_key_for_id(0), mask);
-    gwy_app_channel_title_fall_back(container, 0);
-    gwy_file_channel_import_log_add(container, 0, NULL, filename);
+    file = gwy_file_new_in_construction();
+    gwy_file_set_image(file, 0, dfield);
+    gwy_file_set_image_mask(file, 0, mask);
+    gwy_image_title_fall_back(file, 0);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
 
 fail:
-    GWY_OBJECT_UNREF(mask);
-    GWY_OBJECT_UNREF(dfield);
+    g_clear_object(&mask);
+    g_clear_object(&dfield);
     g_free(buffer);
     g_free(header.units);
 
-    return container;
+    return file;
 }
 
 #define free_regex(r) if (r) g_regex_unref(r); r = NULL
@@ -744,7 +732,7 @@ fail:
 static gboolean
 ccsv_read_images(const CorningCSVHeader *header,
                  gchar *p,
-                 GwyDataField **dfield, GwyDataField **mask,
+                 GwyField **dfield, GwyField **mask,
                  GError **error)
 {
     gdouble *d, *m;
@@ -754,15 +742,15 @@ ccsv_read_images(const CorningCSVHeader *header,
     xres = header->xres;
     yres = header->yres;
 
-    *dfield = gwy_data_field_new(xres, yres, header->xreal, header->yreal, TRUE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(*dfield), "m");
-    *mask = gwy_data_field_new_alike(*dfield, TRUE);
+    *dfield = gwy_field_new(xres, yres, header->xreal, header->yreal, TRUE);
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(*dfield), "m");
+    *mask = gwy_field_new_alike(*dfield, TRUE);
     if (!gwy_strequal(header->units, "Microns")) {
         g_warning("Units are not Microns, setting to metre anyway.");
     }
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(*dfield), "m");
-    d = gwy_data_field_get_data(*dfield);
-    m = gwy_data_field_get_data(*mask);
+    gwy_unit_set_from_string(gwy_field_get_unit_z(*dfield), "m");
+    d = gwy_field_get_data(*dfield);
+    m = gwy_field_get_data(*mask);
 
     while (*p == '\r' || *p == '\n')
         p++;
@@ -791,7 +779,7 @@ ccsv_read_images(const CorningCSVHeader *header,
         }
     }
 
-    gwy_data_field_laplace_solve(*dfield, *mask, -1, 1.0);
+    gwy_field_laplace_solve(*dfield, *mask, -1, 1.0);
 
     return TRUE;
 }
