@@ -1,6 +1,6 @@
 /*
- *  $Id: unisoku.c 26190 2024-02-22 10:47:47Z yeti-dn $
- *  Copyright (C) 2005 David Necas (Yeti), Petr Klapetek.
+ *  $Id: unisoku.c 28911 2025-11-24 18:27:42Z yeti-dn $
+ *  Copyright (C) 2005-2025 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -42,16 +42,11 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
 #include <stdarg.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/stats.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -116,38 +111,37 @@ typedef struct {
     gchar *ad_name;
 } UnisokuFile;
 
-static gboolean      module_register        (void);
-static gint          unisoku_detect         (const GwyFileDetectInfo *fileinfo,
-                                             gboolean only_name);
-static GwyContainer* unisoku_load           (const gchar *filename,
-                                             GwyRunType mode,
-                                             GError **error);
-static gboolean      unisoku_read_header    (gchar *buffer,
-                                             UnisokuFile *ufile,
-                                             GError **error);
-static gint          unisoku_sscanf         (const gchar *str,
-                                             const gchar *format,
-                                             ...);
-static GwyDataField* unisoku_read_data_field(const guchar *buffer,
-                                             gsize size,
-                                             UnisokuFile *ufile,
-                                             GError **error);
-static GwyContainer* unisoku_get_metadata   (UnisokuFile *ufile);
-static gchar*        unisoku_find_data_name (const gchar *header_name);
-static void          unisoku_file_free      (UnisokuFile *ufile);
+static gboolean       module_register         (void);
+static gint           detect_file             (const GwyFileDetectInfo *fileinfo,
+                                               gboolean only_name);
+static GwyFile*       load_file               (const gchar *filename,
+                                               GwyRunModeFlags mode,
+                                               GError **error);
+static gboolean       read_header             (gchar *buffer,
+                                               UnisokuFile *ufile,
+                                               GError **error);
+static gint           ascii_sscanf            (const gchar *str,
+                                               const gchar *format,
+                                               ...);
+static GwyField*      read_field              (const guchar *buffer,
+                                               gsize size,
+                                               UnisokuFile *ufile,
+                                               GError **error);
+static GwyContainer*  get_metadata            (UnisokuFile *ufile);
+static gchar*         find_data_name          (const gchar *header_name);
+static GwyRawDataType unisoku_type_to_raw_type(UnisokuDataType datatype);
+static void           free_unisokufile        (UnisokuFile *ufile);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Unisoku data files (two-part .hdr + .dat)."),
     "Yeti <yeti@gwyddion.net>",
-    "0.10",
+    "0.11",
     "David Nečas (Yeti) & Petr Klapetek",
     "2005",
 };
 
-static const guint type_sizes[] = { 0, 0, 1, 1, 2, 2, 0, 0, 4 };
-
 GWY_MODULE_QUERY2(module_info, unisoku)
 
 static gboolean
@@ -155,17 +149,13 @@ module_register(void)
 {
     gwy_file_func_register("unisoku",
                            N_("Unisoku files (.hdr + .dat)"),
-                           (GwyFileDetectFunc)&unisoku_detect,
-                           (GwyFileLoadFunc)&unisoku_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-unisoku_detect(const GwyFileDetectInfo *fileinfo,
-               gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     gint score = 0;
 
@@ -177,7 +167,7 @@ unisoku_detect(const GwyFileDetectInfo *fileinfo,
         && g_str_has_suffix(fileinfo->name_lowercase, EXTENSION_HEADER)) {
         gchar *data_name;
 
-        if ((data_name = unisoku_find_data_name(fileinfo->name))) {
+        if ((data_name = find_data_name(fileinfo->name))) {
             score = 100;
             g_free(data_name);
         }
@@ -186,18 +176,18 @@ unisoku_detect(const GwyFileDetectInfo *fileinfo,
     return score;
 }
 
-static GwyContainer*
-unisoku_load(const gchar *filename,
-             G_GNUC_UNUSED GwyRunType mode,
-             GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
     UnisokuFile ufile;
-    GwyContainer *meta, *container = NULL;
+    GwyFile *file = NULL;
     guchar *buffer = NULL;
     gchar *text = NULL;
     gsize i, size = 0;
     GError *err = NULL;
-    GwyDataField *dfield = NULL;
+    GwyField *dfield = NULL;
     gchar *data_name;
 
     if (!g_file_get_contents(filename, &text, &size, &err)) {
@@ -210,58 +200,54 @@ unisoku_load(const gchar *filename,
             text[i] = ' ';
     }
 
-    gwy_clear(&ufile, 1);
-    if (!unisoku_read_header(text, &ufile, error)) {
-        unisoku_file_free(&ufile);
+    gwy_clear1(ufile);
+    if (!read_header(text, &ufile, error)) {
+        free_unisokufile(&ufile);
         g_free(text);
         return NULL;
     }
     g_free(text);
 
-    if (ufile.data_type < UNISOKU_UINT8
-        || ufile.data_type > UNISOKU_FLOAT
-        || type_sizes[ufile.data_type] == 0) {
+    if (unisoku_type_to_raw_type(ufile.data_type) == (GwyRawDataType)G_MAXUINT) {
         err_DATA_TYPE(error, ufile.data_type);
-        unisoku_file_free(&ufile);
+        free_unisokufile(&ufile);
         return NULL;
     }
 
-    if (!(data_name = unisoku_find_data_name(filename))) {
+    if (!(data_name = find_data_name(filename))) {
         g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
                     _("No corresponding data file was found for header file."));
-        unisoku_file_free(&ufile);
+        free_unisokufile(&ufile);
         return NULL;
     }
 
     if (!gwy_file_get_contents(data_name, &buffer, &size, &err)) {
         err_GET_FILE_CONTENTS(error, &err);
-        unisoku_file_free(&ufile);
+        free_unisokufile(&ufile);
         g_free(data_name);
         return NULL;
     }
 
-    dfield = unisoku_read_data_field(buffer, size, &ufile, error);
+    dfield = read_field(buffer, size, &ufile, error);
     gwy_file_abandon_contents(buffer, size, NULL);
 
     if (!dfield) {
-        unisoku_file_free(&ufile);
+        free_unisokufile(&ufile);
         g_free(data_name);
         return NULL;
     }
 
-    container = gwy_container_new();
-    gwy_container_pass_object(container, gwy_app_get_data_key_for_id(0), dfield);
-    gwy_app_channel_title_fall_back(container, 0);
+    file = gwy_file_new_in_construction();
+    gwy_file_pass_image(file, 0, dfield);
+    gwy_image_title_fall_back(file, 0);
 
-    meta = unisoku_get_metadata(&ufile);
-    gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(0), meta);
+    gwy_file_pass_meta(file, GWY_FILE_IMAGE, 0, get_metadata(&ufile));
+    gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, data_name);
 
-    gwy_file_channel_import_log_add(container, 0, NULL, data_name);
-
-    unisoku_file_free(&ufile);
+    free_unisokufile(&ufile);
     g_free(data_name);
 
-    return container;
+    return file;
 }
 
 #define NEXT(buffer, line, err) \
@@ -275,9 +261,9 @@ unisoku_load(const gchar *filename,
     g_strstrip(line)
 
 static gboolean
-unisoku_read_header(gchar *buffer,
-                    UnisokuFile *ufile,
-                    GError **error)
+read_header(gchar *buffer,
+            UnisokuFile *ufile,
+            GError **error)
 {
     gchar *line;
     gint type1, type2;
@@ -290,7 +276,7 @@ unisoku_read_header(gchar *buffer,
     /* garbage */
 
     NEXT(buffer, line, error);
-    if (unisoku_sscanf(line, "i", &ufile->format_version) != 1) {
+    if (ascii_sscanf(line, "i", &ufile->format_version) != 1) {
         err_UNSUPPORTED(error, _("format version"));
         return FALSE;
     }
@@ -305,14 +291,14 @@ unisoku_read_header(gchar *buffer,
     ufile->remark = g_strdup(line);
 
     NEXT(buffer, line, error);
-    if (unisoku_sscanf(line, "ii", &ufile->ascii_flag, &type1) != 2) {
+    if (ascii_sscanf(line, "ii", &ufile->ascii_flag, &type1) != 2) {
         err_INVALID(error, _("format flags"));
         return FALSE;
     }
     ufile->data_type = type1;
 
     NEXT(buffer, line, error);
-    if (unisoku_sscanf(line, "ii", &ufile->xres, &ufile->yres) != 2) {
+    if (ascii_sscanf(line, "ii", &ufile->xres, &ufile->yres) != 2) {
         err_INVALID(error, _("resolution"));
         return FALSE;
     }
@@ -321,7 +307,7 @@ unisoku_read_header(gchar *buffer,
         return FALSE;
 
     NEXT(buffer, line, error);
-    if (unisoku_sscanf(line, "ii", &type1, &type2) != 2) {
+    if (ascii_sscanf(line, "ii", &type1, &type2) != 2) {
         /* FIXME */
         g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
                     "Missing or invalid some integers heaven knows what they mean but that should be here.");
@@ -335,8 +321,8 @@ unisoku_read_header(gchar *buffer,
 
     NEXT(buffer, line, error);
     /* The log flags seem to be missing occasionally.  Do not abort when it happens. */
-    if (unisoku_sscanf(line, "ddi",
-                       &ufile->start_x, &ufile->end_x, &ufile->log_flag_x) < 2) {
+    if (ascii_sscanf(line, "ddi",
+                     &ufile->start_x, &ufile->end_x, &ufile->log_flag_x) < 2) {
         err_INVALID(error, _("x scale parameters"));
         return FALSE;
     }
@@ -345,8 +331,8 @@ unisoku_read_header(gchar *buffer,
     ufile->unit_y = g_strdup(line);
 
     NEXT(buffer, line, error);
-    if (unisoku_sscanf(line, "ddii",
-                       &ufile->start_y, &ufile->end_y, &ufile->ineq_flag, &ufile->log_flag_y) < 3) {
+    if (ascii_sscanf(line, "ddii",
+                     &ufile->start_y, &ufile->end_y, &ufile->ineq_flag, &ufile->log_flag_y) < 3) {
         err_INVALID(error, _("y scale parameters"));
         return FALSE;
     }
@@ -355,15 +341,15 @@ unisoku_read_header(gchar *buffer,
     ufile->unit_z = g_strdup(line);
 
     NEXT(buffer, line, error);
-    if (unisoku_sscanf(line, "ddddi",
-                       &ufile->max_raw_z, &ufile->min_raw_z, &ufile->max_z, &ufile->min_z, &ufile->log_flag_z) < 4) {
+    if (ascii_sscanf(line, "ddddi",
+                     &ufile->max_raw_z, &ufile->min_raw_z, &ufile->max_z, &ufile->min_z, &ufile->log_flag_z) < 4) {
         err_INVALID(error, _("z scale parameters"));
         return FALSE;
     }
 
     NEXT(buffer, line, error);
-    if (unisoku_sscanf(line, "dddi",
-                       &ufile->stm_voltage, &ufile->stm_current, &ufile->scan_time, &ufile->accum) != 4) {
+    if (ascii_sscanf(line, "dddi",
+                     &ufile->stm_voltage, &ufile->stm_current, &ufile->scan_time, &ufile->accum) != 4) {
         err_INVALID(error, _("data type parameters"));
         return FALSE;
     }
@@ -386,9 +372,9 @@ unisoku_read_header(gchar *buffer,
 }
 
 static gint
-unisoku_sscanf(const gchar *str,
-               const gchar *format,
-               ...)
+ascii_sscanf(const gchar *str,
+             const gchar *format,
+             ...)
 {
     va_list ap;
     gchar *endptr;
@@ -426,35 +412,21 @@ unisoku_sscanf(const gchar *str,
     return count;
 }
 
-static GwyDataField*
-unisoku_read_data_field(const guchar *buffer,
-                        gsize size,
-                        UnisokuFile *ufile,
-                        GError **error)
+static GwyField*
+read_field(const guchar *buffer,
+           gsize size,
+           UnisokuFile *ufile,
+           GError **error)
 {
-    GwyRawDataType rawtype;
-    gint n, power10;
+    gint power10;
+    gsize n;
     const gchar *unit;
-    GwyDataField *dfield;
+    GwyField *dfield;
     gdouble q, pmin, pmax, rmin, rmax, xreal, yreal;
-
-    /* FIXME: what to do when ascii_flag is set? */
-    if (ufile->data_type == UNISOKU_UINT8)
-        rawtype = GWY_RAW_DATA_UINT8;
-    else if (ufile->data_type == UNISOKU_SINT8)
-        rawtype = GWY_RAW_DATA_SINT8;
-    else if (ufile->data_type == UNISOKU_UINT16)
-        rawtype = GWY_RAW_DATA_UINT16;
-    else if (ufile->data_type == UNISOKU_SINT16)
-        rawtype = GWY_RAW_DATA_SINT16;
-    else if (ufile->data_type == UNISOKU_FLOAT)
-        rawtype = GWY_RAW_DATA_FLOAT;
-    else {
-        g_return_val_if_reached(NULL);
-    }
+    GwyRawDataType rawtype = unisoku_type_to_raw_type(ufile->data_type);
 
     n = ufile->xres * ufile->yres;
-    if (err_SIZE_MISMATCH(error, n*type_sizes[ufile->data_type], size, FALSE))
+    if (err_SIZE_MISMATCH(error, n*gwy_raw_data_size(rawtype), size, FALSE))
         return NULL;
 
     /* Use negated positive conditions to catch NaNs */
@@ -463,35 +435,35 @@ unisoku_read_data_field(const guchar *buffer,
     yreal = ufile->end_y - ufile->start_y;
     sanitise_real_size(&yreal, "y size");
 
-    dfield = gwy_data_field_new(ufile->xres, ufile->yres, xreal, yreal, FALSE);
+    dfield = gwy_field_new(ufile->xres, ufile->yres, xreal, yreal, FALSE);
     gwy_convert_raw_data(buffer, n, 1, rawtype, GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                         gwy_data_field_get_data(dfield), 1.0, 0.0);
+                         gwy_field_get_data(dfield), 1.0, 0.0);
 
     unit = ufile->unit_x;
     if (!*unit)
         unit = "nm";
-    gwy_si_unit_set_from_string_parse(gwy_data_field_get_si_unit_xy(dfield), unit, &power10);
-    q = pow10(power10);
-    gwy_data_field_set_xreal(dfield, q*gwy_data_field_get_xreal(dfield));
-    gwy_data_field_set_yreal(dfield, q*gwy_data_field_get_yreal(dfield));
+    power10 = gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), unit);
+    q = gwy_exp10(power10);
+    gwy_field_set_xreal(dfield, q*gwy_field_get_xreal(dfield));
+    gwy_field_set_yreal(dfield, q*gwy_field_get_yreal(dfield));
 
     /* XXX: No fallback yet, just make z unitless */
-    gwy_si_unit_set_from_string_parse(gwy_data_field_get_si_unit_z(dfield), ufile->unit_z, &power10);
-    q = pow10(power10);
+    power10 = gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), ufile->unit_z);
+    q = gwy_exp10(power10);
     pmin = q*ufile->min_z;
     pmax = q*ufile->max_z;
     rmin = ufile->min_raw_z;
     rmax = ufile->max_raw_z;
-    gwy_data_field_multiply(dfield, (pmax - pmin)/(rmax - rmin));
-    gwy_data_field_add(dfield, (pmin*rmax - pmax*rmin)/(rmax - rmin));
+    gwy_field_multiply(dfield, (pmax - pmin)/(rmax - rmin));
+    gwy_field_add(dfield, (pmin*rmax - pmax*rmin)/(rmax - rmin));
 
     return dfield;
 }
 
 static GwyContainer*
-unisoku_get_metadata(UnisokuFile *ufile)
+get_metadata(UnisokuFile *ufile)
 {
-    GwyContainer *meta = gwy_container_new();
+    GwyContainer *meta = gwy_container_new_in_construction();
 
     gwy_container_set_string_by_name(meta, "Date", g_strconcat(ufile->date, " ", ufile->time, NULL));
     if (*ufile->remark)
@@ -505,23 +477,21 @@ unisoku_get_metadata(UnisokuFile *ufile)
 }
 
 static gchar*
-unisoku_find_data_name(const gchar *header_name)
+find_data_name(const gchar *header_name)
 {
     GString *data_name;
     gchar *retval;
     gboolean ok = FALSE;
 
     data_name = g_string_new(header_name);
-    g_string_truncate(data_name,
-                      data_name->len - (sizeof(EXTENSION_HEADER) - 1));
+    g_string_truncate(data_name, data_name->len - (sizeof(EXTENSION_HEADER) - 1));
     g_string_append(data_name, EXTENSION_DATA);
     if (g_file_test(data_name->str, G_FILE_TEST_IS_REGULAR))
         ok = TRUE;
     else {
         guint i;
         for (i = 0; i < sizeof(EXTENSION_DATA); i++)
-            data_name->str[data_name->len-1 - i]
-                = g_ascii_toupper(data_name->str[data_name->len-1 - i]);
+            data_name->str[data_name->len-1 - i] = g_ascii_toupper(data_name->str[data_name->len-1 - i]);
         if (g_file_test(data_name->str, G_FILE_TEST_IS_REGULAR))
             ok = TRUE;
     }
@@ -531,8 +501,24 @@ unisoku_find_data_name(const gchar *header_name)
     return ok ? retval : NULL;
 }
 
+static GwyRawDataType
+unisoku_type_to_raw_type(UnisokuDataType datatype)
+{
+    if (datatype == UNISOKU_UINT8)
+        return GWY_RAW_DATA_UINT8;
+    else if (datatype == UNISOKU_SINT8)
+        return GWY_RAW_DATA_SINT8;
+    else if (datatype == UNISOKU_UINT16)
+        return GWY_RAW_DATA_UINT16;
+    else if (datatype == UNISOKU_SINT16)
+        return GWY_RAW_DATA_SINT16;
+    else if (datatype == UNISOKU_FLOAT)
+        return GWY_RAW_DATA_FLOAT;
+    return (GwyRawDataType)G_MAXUINT;
+}
+
 static void
-unisoku_file_free(UnisokuFile *ufile)
+free_unisokufile(UnisokuFile *ufile)
 {
     g_free(ufile->date);
     g_free(ufile->time);
