@@ -1,8 +1,11 @@
 /*
- *  $Id: win_stm.c 26493 2024-08-09 15:01:17Z yeti-dn $
+ *  $Id: win_stm.c 28911 2025-11-24 18:27:42Z yeti-dn $
  *  Copyright (C) 2014 Jeffrey J. Schwartz.
  *  E-mail: schwartz@physics.ucla.edu
  *
+ *  Copyright (C) 2025 David Necas (Yeti).
+ *  E-mail: yeti@gwyddion.net
+ *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
  *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
  *  later version.
@@ -41,13 +44,9 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/stats.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 #include "err.h"
 #include "get.h"
 
@@ -73,12 +72,12 @@ typedef struct {
     gdouble Current;
 } WinSTM_File;
 
-static gboolean      module_register(void);
-static gint          winSTM_detect  (const GwyFileDetectInfo *fileinfo,
-                                     gboolean only_name);
-static GwyContainer* winSTM_load    (const gchar *filename,
-                                     GwyRunType mode,
-                                     GError **error);
+static gboolean module_register(void);
+static gint     detect_file    (const GwyFileDetectInfo *fileinfo,
+                                gboolean only_name);
+static GwyFile* load_file      (const gchar *filename,
+                                GwyRunModeFlags mode,
+                                GError **error);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
@@ -97,16 +96,12 @@ module_register(void)
 {
     gwy_file_func_register("win_stm",
                            N_("WinSTM files (.stm)"),
-                           (GwyFileDetectFunc)&winSTM_detect,
-                           (GwyFileLoadFunc)&winSTM_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
     return TRUE;
 }
 
 static gint
-winSTM_detect(const GwyFileDetectInfo *fileinfo,
-              gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     if (only_name)
         return g_str_has_suffix(fileinfo->name_lowercase, EXTENSION) ? 20 : 0;
@@ -115,14 +110,14 @@ winSTM_detect(const GwyFileDetectInfo *fileinfo,
     return 0;
 }
 
-static GwyContainer*
-winSTM_load(const gchar *filename,
-            G_GNUC_UNUSED GwyRunType mode,
-            GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
     WinSTM_File stm;
-    GwyContainer *container = NULL;
-    GwyDataField *dfield;
+    GwyFile *file = NULL;
+    GwyField *dfield;
     guchar *buffer;
     const guchar *p;
     GError *err = NULL;
@@ -157,11 +152,10 @@ winSTM_load(const gchar *filename,
         goto end;
     }
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     for (id = 0; id < stm.NChan; id++) {
         gchar UnitWord[64];
         gchar Title[100];
-        GwyContainer *meta;
         const gchar *zunit;
         guint datasize;
         gint power10;
@@ -176,6 +170,7 @@ winSTM_load(const gchar *filename,
             zunit = "Å";
         else if (gwy_strequal(zunit, "pAmps"))
             zunit = "pA";
+
         /* XXX: There was a logic basically treating anything else as ‘V’, but that should be fine as the parser
          * understands things like ‘Volts’ now. */
         stm.Bias = gwy_get_gdouble_le(&p);
@@ -185,33 +180,29 @@ winSTM_load(const gchar *filename,
         p += 48;
         datasize = sizeof(gint32) * stm.Xres * stm.Yres;
         if (err_SIZE_MISMATCH(error, datasize + (p - buffer), size, FALSE)) {
-            GWY_OBJECT_UNREF(container);
+            g_clear_object(&file);
             goto end;
         }
-        dfield = gwy_data_field_new(stm.Xres, stm.Yres, stm.RangeX, stm.RangeY, FALSE);
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
-        gwy_si_unit_set_from_string_parse(gwy_data_field_get_si_unit_z(dfield), zunit, &power10);
+        dfield = gwy_field_new(stm.Xres, stm.Yres, stm.RangeX, stm.RangeY, FALSE);
+        gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
+        power10 = gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), zunit);
         gwy_convert_raw_data(p, stm.Xres * stm.Yres, 1, GWY_RAW_DATA_SINT32, GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                             gwy_data_field_get_data(dfield), pow10(power10) * stm.Gain, 0.0);
+                             gwy_field_get_data(dfield), gwy_exp10(power10) * stm.Gain, 0.0);
         p += datasize;
 
-        gwy_container_pass_object(container, gwy_app_get_data_key_for_id(id), dfield);
-        gwy_container_set_string(container,
-                                 gwy_app_get_data_title_key_for_id(id),
-                                 gwy_convert_to_utf8(Title, -1, "ISO-8859-1"));
-        meta = gwy_container_new();
-        gwy_container_set_string_by_name(meta, "Bias",
-                                         (const guchar *)g_strdup_printf("%.3f V", stm.Bias));
-        gwy_container_set_string_by_name(meta, "Current Set Point",
-                                         (const guchar *)g_strdup_printf("%.3f pA", stm.Current));
-        gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(id), meta);
-        gwy_file_channel_import_log_add(container, id, NULL, filename);
+        gwy_file_pass_image(file, id, dfield);
+        gwy_file_pass_title(file, GWY_FILE_IMAGE, id, gwy_convert_to_utf8(Title, -1, "ISO-8859-1"));
+        GwyContainer *meta = gwy_container_new_in_construction();
+        gwy_container_set_string_by_name(meta, "Bias", g_strdup_printf("%.3f V", stm.Bias));
+        gwy_container_set_string_by_name(meta, "Current Set Point", g_strdup_printf("%.3f pA", stm.Current));
+        gwy_file_pass_meta(file, GWY_FILE_IMAGE, id, meta);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, id, NULL, filename);
     }
 
 end:
     gwy_file_abandon_contents(buffer, size, NULL);
 
-    return container;
+    return file;
 }
 
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
