@@ -1,6 +1,6 @@
 /*
- *  $Id: witec-asc.c 26163 2024-02-06 16:56:55Z yeti-dn $
- *  Copyright (C) 2011 David Necas (Yeti).
+ *  $Id: witec-asc.c 28789 2025-11-04 17:14:03Z yeti-dn $
+ *  Copyright (C) 2011-2025 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
  *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
@@ -40,16 +40,11 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/stats.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -57,19 +52,19 @@
 #define MAGIC_SIZE (sizeof(MAGIC)-1)
 #define EXTENSION ".dat"
 
-static gboolean      module_register(void);
-static gint          dat_detect     (const GwyFileDetectInfo *fileinfo,
-                                     gboolean only_name);
-static GwyContainer* dat_load       (const gchar *filename,
-                                     GwyRunType mode,
-                                     GError **error);
+static gboolean module_register(void);
+static gint     detect_file    (const GwyFileDetectInfo *fileinfo,
+                                gboolean only_name);
+static GwyFile* load_file      (const gchar *filename,
+                                GwyRunModeFlags mode,
+                                GError **error);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports WITec ASCII export files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.3",
+    "0.4",
     "David Nečas (Yeti)",
     "2011",
 };
@@ -81,16 +76,13 @@ module_register(void)
 {
     gwy_file_func_register("witec-asc",
                            N_("WITec ASCII files (.dat)"),
-                           (GwyFileDetectFunc)&dat_detect,
-                           (GwyFileLoadFunc)&dat_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-dat_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     if (only_name)
         return g_str_has_suffix(fileinfo->name_lowercase, EXTENSION) ? 10 : 0;
@@ -120,15 +112,15 @@ header_end(G_GNUC_UNUSED const GwyTextHeaderContext *context,
     *pp += length;
 }
 
-static GwyContainer*
-dat_load(const gchar *filename,
-         G_GNUC_UNUSED GwyRunType mode,
-         GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *container = NULL;
-    GwyDataField *dfield = NULL;
+    GwyFile *file = NULL;
+    GwyField *dfield = NULL;
     GwyTextHeaderParser parser;
-    GwySIUnit *xyunit = NULL, *zunit = NULL;
+    GwyUnit *xyunit = NULL, *zunit = NULL;
     gint power10xy, power10z;
     gchar *line, *p, *value, *title, *buffer = NULL;
     GHashTable *hash = NULL;
@@ -162,7 +154,7 @@ dat_load(const gchar *filename,
         goto fail;
     }
 
-    gwy_clear(&parser, 1);
+    gwy_clear1(parser);
     parser.key_value_separator = "=";
     parser.terminator = "[Data]";
     parser.error = &header_error;
@@ -180,22 +172,22 @@ dat_load(const gchar *filename,
     if (err_DIMENSION(error, xres) || err_DIMENSION(error, yres))
         goto fail;
 
-    xyunit = gwy_si_unit_new_parse(g_hash_table_lookup(hash, "ScanUnit"), &power10xy);
-    zunit = gwy_si_unit_new_parse(g_hash_table_lookup(hash, "DataUnit"), &power10z);
+    xyunit = gwy_unit_new_parse(g_hash_table_lookup(hash, "ScanUnit"), &power10xy);
+    zunit = gwy_unit_new_parse(g_hash_table_lookup(hash, "DataUnit"), &power10z);
     xreal = g_ascii_strtod(g_hash_table_lookup(hash, "ScanWidth"), NULL);
     yreal = g_ascii_strtod(g_hash_table_lookup(hash, "ScanHeight"), NULL);
     sanitise_real_size(&xreal, "x size");
     sanitise_real_size(&yreal, "y size");
-    xreal *= pow10(power10xy);
-    yreal *= pow10(power10xy);
+    xreal *= gwy_exp10(power10xy);
+    yreal *= gwy_exp10(power10xy);
 
-    dfield = gwy_data_field_new(xres, yres, xreal, yreal, FALSE);
+    dfield = gwy_field_new(xres, yres, xreal, yreal, FALSE);
 
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(dfield), xyunit);
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_z(dfield), zunit);
-    q = pow10(power10z);
+    gwy_unit_assign(gwy_field_get_unit_xy(dfield), xyunit);
+    gwy_unit_assign(gwy_field_get_unit_z(dfield), zunit);
+    q = gwy_exp10(power10z);
 
-    data = gwy_data_field_get_data(dfield);
+    data = gwy_field_get_data(dfield);
     value = p;
     for (i = 0; i < xres*yres; i++) {
         data[i] = q*g_ascii_strtod(value, &p);
@@ -212,9 +204,9 @@ dat_load(const gchar *filename,
         value = p;
     }
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
 
-    gwy_container_set_object(container, gwy_app_get_data_key_for_id(0), dfield);
+    gwy_file_set_image(file, 0, dfield);
 
     title = g_hash_table_lookup(hash, "ImageName");
     if (title) {
@@ -223,23 +215,23 @@ dat_load(const gchar *filename,
             title[len-1] = '\0';
             title++;
         }
-        gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(0), title);
+        gwy_file_set_title(file, GWY_FILE_IMAGE, 0, title, FALSE);
     }
     else
-        gwy_app_channel_title_fall_back(container, 0);
+        gwy_image_title_fall_back(file, 0);
 
-    gwy_app_channel_check_nonsquare(container, 0);
-    gwy_file_channel_import_log_add(container, 0, NULL, filename);
+    gwy_check_nonsquare_image(file, 0);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
 
 fail:
     g_free(buffer);
-    GWY_OBJECT_UNREF(dfield);
-    GWY_OBJECT_UNREF(xyunit);
-    GWY_OBJECT_UNREF(zunit);
+    g_clear_object(&dfield);
+    g_clear_object(&xyunit);
+    g_clear_object(&zunit);
     if (hash)
         g_hash_table_destroy(hash);
 
-    return container;
+    return file;
 }
 
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
