@@ -1,6 +1,6 @@
 /*
- *  $Id: witfile.c 26343 2024-05-15 09:23:24Z yeti-dn $
- *  Copyright (C) 2005 David Necas (Yeti), Petr Klapetek.
+ *  $Id: witfile.c 28789 2025-11-04 17:14:03Z yeti-dn $
+ *  Copyright (C) 2005-2025 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -32,14 +32,9 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/datafield.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 
 #include "err.h"
 #include "get.h"
@@ -169,42 +164,42 @@ typedef struct {
     WITecImageOptions image_options;
 } WITecFile;
 
-static gboolean      module_register          (void);
-static gint          witec_detect            (const GwyFileDetectInfo *fileinfo,
-                                              gboolean only_name);
-static GwyContainer* witec_load               (const gchar *filename,
-                                               GwyRunType mode,
-                                               GError **error);
-static gboolean      witec_read_file          (const guchar *buffer,
-                                               gsize len,
-                                               WITecFile *witfile,
-                                               GError **error);
-static GwyContainer* witec_get_metadata       (WITecFile *witfile,
-                                               gint i);
-static GwyDataField* witec_image_to_data_field(WITecFile *witfile,
-                                               guint i);
-static gboolean      witec_read_scale         (const guchar **p,
-                                               gsize *len,
-                                               WITecScale *scale);
-static gboolean      witec_read_header        (const guchar **p,
-                                               gsize *len,
-                                               WITecHeader *header);
-static gboolean      witec_read_footer        (const guchar **p,
-                                               gsize *len,
-                                               WITecFooter *footer);
-static gboolean      witec_read_range_options (const guchar **p,
-                                               gsize *len,
-                                               WITecRangeOptions *options);
-static gboolean      witec_read_image_options (const guchar **p,
-                                               gsize *len,
-                                               WITecImageOptions *options);
+static gboolean      module_register   (void);
+static gint          detect_file       (const GwyFileDetectInfo *fileinfo,
+                                        gboolean only_name);
+static GwyFile*      load_file         (const gchar *filename,
+                                        GwyRunModeFlags mode,
+                                        GError **error);
+static gboolean      read_file         (const guchar *buffer,
+                                        gsize len,
+                                        WITecFile *witfile,
+                                        GError **error);
+static GwyContainer* get_metadata      (WITecFile *witfile,
+                                        gint i);
+static GwyField*     image_to_field    (WITecFile *witfile,
+                                        guint i);
+static gboolean      read_scale        (const guchar **p,
+                                        gsize *len,
+                                        WITecScale *scale);
+static gboolean      read_header       (const guchar **p,
+                                        gsize *len,
+                                        WITecHeader *header);
+static gboolean      read_footer       (const guchar **p,
+                                        gsize *len,
+                                        WITecFooter *footer);
+static gboolean      read_range_options(const guchar **p,
+                                        gsize *len,
+                                        WITecRangeOptions *options);
+static gboolean      read_image_options(const guchar **p,
+                                        gsize *len,
+                                        WITecImageOptions *options);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports WITec WIT data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.7",
+    "0.8",
     "David Nečas (Yeti) & Petr Klapetek",
     "2005",
 };
@@ -216,17 +211,13 @@ module_register(void)
 {
     gwy_file_func_register("witfile",
                            N_("WITec files (.wit)"),
-                           (GwyFileDetectFunc)&witec_detect,
-                           (GwyFileLoadFunc)&witec_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-witec_detect(const GwyFileDetectInfo *fileinfo,
-             gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     gint score = 0;
 
@@ -242,7 +233,7 @@ witec_detect(const GwyFileDetectInfo *fileinfo,
         gsize len = fileinfo->file_size;
         gsize expected;
 
-        if (witec_read_header(&p, &len, &header)) {
+        if (read_header(&p, &len, &header)) {
             expected = (WITEC_SIZE_HEADER
                         + header.channels*(2*header.rows*header.pixels + WITEC_SIZE_SCALE)
                         + WITEC_SIZE_FOOTER
@@ -255,17 +246,17 @@ witec_detect(const GwyFileDetectInfo *fileinfo,
     return score;
 }
 
-static GwyContainer*
-witec_load(const gchar *filename,
-           G_GNUC_UNUSED GwyRunType mode,
-           GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
     WITecFile witfile;
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     guchar *buffer = NULL;
     gsize size = 0;
     GError *err = NULL;
-    GwyDataField *dfield = NULL;
+    GwyField *dfield = NULL;
     gchar *s;
     guint i;
 
@@ -274,34 +265,34 @@ witec_load(const gchar *filename,
         return NULL;
     }
 
-    gwy_clear(&witfile, 1);
-    if (!witec_read_file(buffer, size, &witfile, error)) {
+    gwy_clear1(witfile);
+    if (!read_file(buffer, size, &witfile, error)) {
         gwy_file_abandon_contents(buffer, size, NULL);
         return NULL;
     }
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     for (i = 0; i < witfile.header.channels; i++) {
-        dfield = witec_image_to_data_field(&witfile, i);
-        gwy_container_pass_object(container, gwy_app_get_data_key_for_id(i), dfield);
+        dfield = image_to_field(&witfile, i);
+        gwy_file_pass_image(file, i, dfield);
         s = g_strndup(witfile.scales[i].name, sizeof(witfile.scales[i].name));
-        gwy_container_set_string(container, gwy_app_get_data_title_key_for_id(i), s);
-        gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(i), witec_get_metadata(&witfile, i));
-        gwy_file_channel_import_log_add(container, i, NULL, filename);
+        gwy_file_pass_title(file, GWY_FILE_IMAGE, i, s);
+        gwy_file_pass_meta(file, GWY_FILE_IMAGE, i, get_metadata(&witfile, i));
+        gwy_log_add_import(file, GWY_FILE_IMAGE, i, NULL, filename);
     }
 
     gwy_file_abandon_contents(buffer, size, NULL);
     g_free(witfile.images);
     g_free(witfile.scales);
 
-    return container;
+    return file;
 }
 
 static gboolean
-witec_read_file(const guchar *buffer,
-                gsize len,
-                WITecFile *witfile,
-                GError **error)
+read_file(const guchar *buffer,
+          gsize len,
+          WITecFile *witfile,
+          GError **error)
 {
     const guchar *p = buffer;
     gint xres, yres;
@@ -312,7 +303,7 @@ witec_read_file(const guchar *buffer,
         return FALSE;
     }
 
-    witec_read_header(&p, &len, &witfile->header);
+    read_header(&p, &len, &witfile->header);
     xres = witfile->header.pixels;
     yres = witfile->header.rows;
     if (err_DIMENSION(error, xres) || err_DIMENSION(error, yres))
@@ -333,26 +324,25 @@ witec_read_file(const guchar *buffer,
         len -= 2*xres*yres;
     }
 
-    witec_read_footer(&p, &len, &witfile->footer);
+    read_footer(&p, &len, &witfile->footer);
     witfile->scales = g_new(WITecScale, ndata);
     for (i = 0; i < ndata; i++)
-        witec_read_scale(&p, &len, witfile->scales + i);
+        read_scale(&p, &len, witfile->scales + i);
 
-    witec_read_range_options(&p, &len, &witfile->range_options);
-    witec_read_image_options(&p, &len, &witfile->image_options);
+    read_range_options(&p, &len, &witfile->range_options);
+    read_image_options(&p, &len, &witfile->image_options);
 
     return TRUE;
 }
 
-static GwyDataField*
-witec_image_to_data_field(WITecFile *witfile,
-                          guint i)
+static GwyField*
+image_to_field(WITecFile *witfile, guint i)
 {
     const WITecRangeOptions *rangeopts = &witfile->range_options;
     const WITecImageOptions *imageopts = &witfile->image_options;
     gchar unit[9];
-    GwyDataField *dfield;
-    GwySIUnit *siunit;
+    GwyField *dfield;
+    GwyUnit *siunit;
     gint xres, yres, power10;
     gdouble q, z0;
 
@@ -366,43 +356,42 @@ witec_image_to_data_field(WITecFile *witfile,
         g_warning("rows (%d) != lines_per_image (%d). Someone has to tell me what it means.",
                   yres, imageopts->lines_per_image);
     }
-    dfield = gwy_data_field_new(xres, yres,
-                                witfile->footer.xscale.scale * imageopts->points_per_line,
-                                witfile->footer.yscale.scale * imageopts->lines_per_image,
-                                FALSE);
+    dfield = gwy_field_new(xres, yres,
+                           witfile->footer.xscale.scale * imageopts->points_per_line,
+                           witfile->footer.yscale.scale * imageopts->lines_per_image,
+                           FALSE);
     q = witfile->scales[i].scale;
     z0 = witfile->scales[i].offset;
     gwy_convert_raw_data(witfile->images[i], xres*yres, 1, GWY_RAW_DATA_SINT16, GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                         gwy_data_field_get_data(dfield), q, 32768*q + z0);
+                         gwy_field_get_data(dfield), q, 32768*q + z0);
 
     if (strncmp(rangeopts->unit_x, rangeopts->unit_y, sizeof(rangeopts->unit_x)) != 0) {
         g_warning("X and Y units differ, using X");
     }
     unit[sizeof(unit)-1] = 0;
     memcpy(unit, rangeopts->unit_x, sizeof(rangeopts->unit_x));
-    siunit = gwy_si_unit_new_parse(unit, &power10);
-    gwy_data_field_set_si_unit_xy(dfield, siunit);
+    siunit = gwy_unit_new_parse(unit, &power10);
+    gwy_unit_assign(gwy_field_get_unit_xy(dfield), siunit);
     g_object_unref(siunit);
-    q = pow10((gdouble)power10);
-    gwy_data_field_set_xreal(dfield, gwy_data_field_get_xreal(dfield)*q);
-    gwy_data_field_set_yreal(dfield, gwy_data_field_get_yreal(dfield)*q);
+    q = gwy_exp10((gdouble)power10);
+    gwy_field_set_xreal(dfield, gwy_field_get_xreal(dfield)*q);
+    gwy_field_set_yreal(dfield, gwy_field_get_yreal(dfield)*q);
 
     memcpy(unit, witfile->scales[i].measure, sizeof(witfile->scales[i].measure));
-    siunit = gwy_si_unit_new_parse(unit, &power10);
-    gwy_data_field_set_si_unit_z(dfield, siunit);
+    siunit = gwy_unit_new_parse(unit, &power10);
+    gwy_unit_assign(gwy_field_get_unit_z(dfield), siunit);
     g_object_unref(siunit);
-    q = pow10((gdouble)power10);
-    gwy_data_field_multiply(dfield, q);
+    q = gwy_exp10((gdouble)power10);
+    gwy_field_multiply(dfield, q);
 
     return dfield;
 }
 
 /* FIXME: There seems to be no local metadata */
 static GwyContainer*
-witec_get_metadata(WITecFile *witfile,
-                   G_GNUC_UNUSED gint i)
+get_metadata(WITecFile *witfile, G_GNUC_UNUSED gint i)
 {
-    GwyContainer *meta = gwy_container_new();
+    GwyContainer *meta = gwy_container_new_in_construction();
     gdouble v;
 
     if (witfile->footer.title[0])
@@ -428,9 +417,9 @@ witec_get_metadata(WITecFile *witfile,
 }
 
 static gboolean
-witec_read_scale(const guchar **p,
-                 gsize *len,
-                 WITecScale *scale)
+read_scale(const guchar **p,
+           gsize *len,
+           WITecScale *scale)
 {
     if (*len < WITEC_SIZE_SCALE) {
         g_warning("Scale truncated");
@@ -448,9 +437,9 @@ witec_read_scale(const guchar **p,
 }
 
 static gboolean
-witec_read_header(const guchar **p,
-                  gsize *len,
-                  WITecHeader *header)
+read_header(const guchar **p,
+            gsize *len,
+            WITecHeader *header)
 {
     if (*len < WITEC_SIZE_HEADER) {
         g_warning("Header truncated");
@@ -470,9 +459,9 @@ witec_read_header(const guchar **p,
 }
 
 static gboolean
-witec_read_footer(const guchar **p,
-                  gsize *len,
-                  WITecFooter *footer)
+read_footer(const guchar **p,
+            gsize *len,
+            WITecFooter *footer)
 {
     guint i;
 
@@ -499,8 +488,8 @@ witec_read_footer(const guchar **p,
     get_CHARARRAY0(footer->comments, p);
 
     /* The len argument is a fake here, data size was already checked */
-    witec_read_scale(p, len, &footer->xscale);
-    witec_read_scale(p, len, &footer->yscale);
+    read_scale(p, len, &footer->xscale);
+    read_scale(p, len, &footer->yscale);
 
     sanitise_real_size(&footer->xscale.scale, "x scale");
     sanitise_real_size(&footer->yscale.scale, "y scale");
@@ -510,9 +499,9 @@ witec_read_footer(const guchar **p,
 }
 
 static gboolean
-witec_read_range_options(const guchar **p,
-                         gsize *len,
-                         WITecRangeOptions *options)
+read_range_options(const guchar **p,
+                   gsize *len,
+                   WITecRangeOptions *options)
 {
     if (*len < WITEC_SIZE_RANGE_OPTIONS) {
         g_warning("Range options truncated");
@@ -547,9 +536,9 @@ witec_read_range_options(const guchar **p,
 }
 
 static gboolean
-witec_read_image_options(const guchar **p,
-                         gsize *len,
-                         WITecImageOptions *options)
+read_image_options(const guchar **p,
+                   gsize *len,
+                   WITecImageOptions *options)
 {
     guint i;
 
