@@ -1,6 +1,6 @@
 /*
- *  $Id: wsffile.c 26160 2024-02-06 14:02:40Z yeti-dn $
- *  Copyright (C) 2011-2019 David Necas (Yeti).
+ *  $Id: wsffile.c 28789 2025-11-04 17:14:03Z yeti-dn $
+ *  Copyright (C) 2011-2025 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
  *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
@@ -49,15 +49,10 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/stats.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -72,19 +67,19 @@
 #define EXTENSION ".wsf"
 
 static gboolean      module_register(void);
-static gint          wsf_detect     (const GwyFileDetectInfo *fileinfo,
+static gint          detect_file    (const GwyFileDetectInfo *fileinfo,
                                      gboolean only_name);
-static GwyContainer* wsf_load       (const gchar *filename,
-                                     GwyRunType mode,
+static GwyFile*      load_file      (const gchar *filename,
+                                     GwyRunModeFlags mode,
                                      GError **error);
-static GwyContainer* wsf_get_meta   (GHashTable *hash);
+static GwyContainer* get_meta       (GHashTable *hash);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports WSF ASCII files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.6",
+    "0.7",
     "David Nečas (Yeti)",
     "2011",
 };
@@ -96,17 +91,13 @@ module_register(void)
 {
     gwy_file_func_register("wsffile",
                            N_("WSF ASCII files (.wsf)"),
-                           (GwyFileDetectFunc)&wsf_detect,
-                           (GwyFileLoadFunc)&wsf_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-wsf_detect(const GwyFileDetectInfo *fileinfo,
-           gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     const guchar *p;
 
@@ -121,17 +112,18 @@ wsf_detect(const GwyFileDetectInfo *fileinfo,
     return 100;
 }
 
-static GwyContainer*
-wsf_load(const gchar *filename,
-         G_GNUC_UNUSED GwyRunType mode,
-         GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *container = NULL, *meta;
-    GwyDataField *dfield = NULL;
+    GwyFile *file = NULL;
+    GwyContainer *meta;
+    GwyField *dfield = NULL;
     gchar *line, *p, *header_end, *title, *header = NULL, *buffer = NULL;
     GwyTextHeaderParser parser;
     GHashTable *hash = NULL;
-    GwySIUnit *siunit;
+    GwyUnit *siunit;
     gsize size;
     GError *err = NULL;
     gdouble xreal, yreal, q;
@@ -162,7 +154,7 @@ wsf_load(const gchar *filename,
     }
     header = g_strndup(p, header_end - p);
 
-    gwy_clear(&parser, 1);
+    gwy_clear1(parser);
     parser.key_value_separator = ":";
     hash = gwy_text_header_parse(header, &parser, NULL, NULL);
 
@@ -181,14 +173,14 @@ wsf_load(const gchar *filename,
     xreal *= Micro;
     yreal *= Micro;
 
-    dfield = gwy_data_field_new(xres, yres, xreal, yreal, FALSE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
+    dfield = gwy_field_new(xres, yres, xreal, yreal, FALSE);
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
     title = g_hash_table_lookup(hash, "Display Type");
     zunit = g_hash_table_lookup(hash, "Units");
     q = 1.0;
     if (zunit) {
-        siunit = gwy_si_unit_new_parse(zunit, &power10);
-        q = pow10(power10);
+        siunit = gwy_unit_new_parse(zunit, &power10);
+        q = gwy_exp10(power10);
         g_object_unref(siunit);
     }
     else {
@@ -206,31 +198,31 @@ wsf_load(const gchar *filename,
     if (!zunit) {
         g_warning("Unknown type %s, cannot determine units.", title);
     }
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), zunit);
+    gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), zunit);
 
-    if (!gwy_parse_doubles(header_end, gwy_data_field_get_data(dfield), GWY_PARSE_DOUBLES_FREE_FORM,
+    if (!gwy_parse_doubles(header_end, gwy_field_get_data(dfield), GWY_PARSE_DOUBLES_FREE_FORM,
                            &yres, &xres, NULL, &err)) {
         err_PARSE_DOUBLES(error, &err);
         goto fail;
     }
-    gwy_data_field_multiply(dfield, q);
+    gwy_field_multiply(dfield, q);
 
-    container = gwy_container_new();
-    gwy_container_set_object(container, gwy_app_get_data_key_for_id(0), dfield);
-    gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(0), title);
-    if ((meta = wsf_get_meta(hash)))
-        gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(0), meta);
+    file = gwy_file_new_in_construction();
+    gwy_file_set_image(file, 0, dfield);
+    gwy_file_set_title(file, GWY_FILE_IMAGE, 0, title, FALSE);
+    if ((meta = get_meta(hash)))
+        gwy_file_pass_meta(file, GWY_FILE_IMAGE, 0, meta);
 
-    gwy_file_channel_import_log_add(container, 0, NULL, filename);
+    gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
 
 fail:
-    GWY_OBJECT_UNREF(dfield);
+    g_clear_object(&dfield);
     g_free(header);
     g_free(buffer);
     if (hash)
         g_hash_table_destroy(hash);
 
-    return container;
+    return file;
 }
 
 static void
@@ -272,9 +264,9 @@ add_meta(gpointer hkey, gpointer hvalue, gpointer user_data)
 }
 
 static GwyContainer*
-wsf_get_meta(GHashTable *hash)
+get_meta(GHashTable *hash)
 {
-    GwyContainer *meta = gwy_container_new();
+    GwyContainer *meta = gwy_container_new_in_construction();
     g_hash_table_foreach(hash, add_meta, meta);
     if (gwy_container_get_n_items(meta))
         return meta;
