@@ -1,6 +1,6 @@
 /*
- *  $Id: wsxmfile.c 26176 2024-02-13 14:00:37Z yeti-dn $
- *  Copyright (C) 2005-2021 David Necas (Yeti), Petr Klapetek.
+ *  $Id: wsxmfile.c 28848 2025-11-10 18:54:08Z yeti-dn $
+ *  Copyright (C) 2005-2025 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -56,20 +56,14 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 #include <glib/gstdio.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwyversion.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/stats.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -85,23 +79,23 @@ static gint           wsxmfile_detect       (const GwyFileDetectInfo *fileinfo,
                                              gboolean only_name);
 static gint           wsxmcurve_detect      (const GwyFileDetectInfo *fileinfo,
                                              gboolean only_name);
-static GwyContainer*  wsxmfile_load         (const gchar *filename,
-                                             GwyRunType mode,
+static GwyFile*       wsxmfile_load         (const gchar *filename,
+                                             GwyRunModeFlags mode,
                                              GError **error);
-static GwyContainer*  wsxmcurve_load        (const gchar *filename,
-                                             GwyRunType mode,
+static GwyFile*       wsxmcurve_load        (const gchar *filename,
+                                             GwyRunModeFlags mode,
                                              GError **error);
-static gboolean       wsxmfile_export_double(GwyContainer *data,
+static gboolean       wsxmfile_export_double(GwyFile *data,
                                              const gchar *filename,
-                                             GwyRunType mode,
+                                             GwyRunModeFlags mode,
                                              GError **error);
-static GwyDataField*  read_data_field       (const guchar *buffer,
+static GwyField*      read_field            (const guchar *buffer,
                                              gint xres,
                                              gint yres,
                                              GwyRawDataType type);
 static void           process_metadata      (GHashTable *wsxmmeta,
                                              GwyRawDataType rawtype,
-                                             GwyContainer *container);
+                                             GwyFile *file);
 static GwyGraphModel* read_graph_model      (const guchar *p,
                                              gint ncurves,
                                              gint npts,
@@ -117,7 +111,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports Nanotec WSxM data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.23",
+    "0.24",
     "David Nečas (Yeti) & Petr Klapetek",
     "2005",
 };
@@ -129,16 +123,10 @@ module_register(void)
 {
     gwy_file_func_register("wsxmfile",
                            N_("WSxM files (.tom, .top, .stp)"),
-                           (GwyFileDetectFunc)&wsxmfile_detect,
-                           (GwyFileLoadFunc)&wsxmfile_load,
-                           NULL,
-                           (GwyFileSaveFunc)&wsxmfile_export_double);
+                           &wsxmfile_detect, &wsxmfile_load, NULL, &wsxmfile_export_double);
     gwy_file_func_register("wsxmcurve",
                            N_("WSxM curve files (.cur)"),
-                           (GwyFileDetectFunc)&wsxmcurve_detect,
-                           (GwyFileLoadFunc)&wsxmcurve_load,
-                           NULL,
-                           NULL);
+                           &wsxmcurve_detect, &wsxmcurve_load, NULL, NULL);
 
     return TRUE;
 }
@@ -318,7 +306,7 @@ parse_header_common(guchar *buffer, gsize size,
         err_FILE_TYPE(error, filetype_name);
         return NULL;
     }
-    gwy_clear(&parser, 1);
+    gwy_clear1(parser);
     parser.key_value_separator = ":";
     parser.section_template = "[\x1a]";
     parser.section_accessor = "::";
@@ -328,17 +316,17 @@ parse_header_common(guchar *buffer, gsize size,
     return gwy_text_header_parse(p, &parser, NULL, NULL);
 }
 
-static GwyContainer*
+static GwyFile*
 wsxmfile_load(const gchar *filename,
-              G_GNUC_UNUSED GwyRunType mode,
+              G_GNUC_UNUSED GwyRunModeFlags mode,
               GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     guchar *buffer = NULL;
     const gchar *rest = NULL;
     gsize size = 0;
     GError *err = NULL;
-    GwyDataField *dfield = NULL;
+    GwyField *dfield = NULL;
     GHashTable *meta = NULL;
     GwyRawDataType type = GWY_RAW_DATA_SINT16;
     guint header_size;
@@ -382,15 +370,15 @@ wsxmfile_load(const gchar *filename,
     if (ok)
         ok = !err_SIZE_MISMATCH(error, 2*xres*yres, (guint)size - header_size, FALSE);
     if (ok)
-        dfield = read_data_field(buffer + header_size, xres, yres, type);
+        dfield = read_field(buffer + header_size, xres, yres, type);
     gwy_file_abandon_contents(buffer, size, NULL);
 
     if (dfield) {
-        container = gwy_container_new();
-        gwy_container_pass_object(container, gwy_app_get_data_key_for_id(0), dfield);
-        process_metadata(meta, type, container);
+        file = gwy_file_new_in_construction();
+        gwy_file_pass_image(file, 0, dfield);
+        process_metadata(meta, type, file);
 
-        gwy_file_channel_import_log_add(container, 0, NULL, filename);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
     }
     else {
         g_assert(!ok);
@@ -398,15 +386,15 @@ wsxmfile_load(const gchar *filename,
     g_hash_table_destroy(meta);
     g_free(header);
 
-    return container;
+    return file;
 }
 
-static GwyContainer*
+static GwyFile*
 wsxmcurve_load(const gchar *filename,
-               G_GNUC_UNUSED GwyRunType mode,
+               G_GNUC_UNUSED GwyRunModeFlags mode,
                GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     guchar *buffer = NULL;
     const gchar *xlabel = NULL, *ylabel = NULL, *xunitstr = NULL, *yunitstr = NULL;
     const gchar *which, *rest = NULL;
@@ -461,19 +449,18 @@ wsxmcurve_load(const gchar *filename,
     gwy_file_abandon_contents(buffer, size, NULL);
 
     if (gmodel) {
-        container = gwy_container_new();
-        gwy_container_pass_object(container, gwy_app_get_graph_key_for_id(1), gmodel);
+        file = gwy_file_new_in_construction();
+        gwy_file_pass_graph(file, 0, gmodel);
+        gwy_log_add_import(file, GWY_FILE_GRAPH, 0, NULL, filename);
     }
     g_hash_table_destroy(meta);
     g_free(header);
 
-    return container;
+    return file;
 }
 
 static void
-store_meta(gpointer key,
-           gpointer value,
-           gpointer user_data)
+store_meta(gpointer key, gpointer value, gpointer user_data)
 {
     GwyContainer *meta = (GwyContainer*)user_data;
 
@@ -509,24 +496,20 @@ get_double_value(GHashTable *hash, const gchar *name, gdouble *value, gchar **pe
 static void
 process_metadata(GHashTable *wsxmmeta,
                  GwyRawDataType rawtype,
-                 GwyContainer *container)
+                 GwyFile *file)
 {
     const gchar *nometa[] = {
         "General Info::Z Amplitude", "Control::X Amplitude", "Control::Y Amplitude",
         "General Info::Number of rows", "General Info::Number of columns",
     };
-    GwyDataField *dfield;
-    GwySIUnit *siunit;
-    GwyContainer *meta;
+    GwyField *dfield = gwy_file_get_image(file, 0);
+    GwyUnit *siunit;
     gdouble p, r;
     gchar *end;
     gint power10;
     guint i;
     gdouble min, max;
-    GQuark quark;
 
-    quark = gwy_app_get_data_key_for_id(0);
-    dfield = GWY_DATA_FIELD(gwy_container_get_object(container, quark));
 
     /* Fix value scale */
     if (get_double_value(wsxmmeta, "General Info::Z Amplitude", &r, &end)) {
@@ -535,72 +518,69 @@ process_metadata(GHashTable *wsxmmeta,
         while (g_ascii_isspace(*end))
             end++;
         if (gwy_strequal(end, "a.u."))
-            siunit = gwy_si_unit_new(NULL);
+            siunit = gwy_unit_new(NULL);
         else if (gwy_strequal(end, "Pi")) {
-            siunit = gwy_si_unit_new(NULL);
+            siunit = gwy_unit_new(NULL);
             r *= G_PI;
         }
         else {
-            siunit = gwy_si_unit_new_parse(end, &power10);
-            p = pow10(power10);
+            siunit = gwy_unit_new_parse(end, &power10);
+            p = gwy_exp10(power10);
         }
-        gwy_si_unit_assign(gwy_data_field_get_si_unit_z(dfield), siunit);
+        gwy_unit_assign(gwy_field_get_unit_z(dfield), siunit);
         g_object_unref(siunit);
 
         if (rawtype == GWY_RAW_DATA_FLOAT || rawtype == GWY_RAW_DATA_DOUBLE) {
-            gwy_data_field_multiply(dfield, p);
+            gwy_field_multiply(dfield, p);
         }
         else {
-            gwy_data_field_get_min_max(dfield, &min, &max);
-            gwy_data_field_multiply(dfield, r*p/(max - min));
+            gwy_field_get_min_max(dfield, &min, &max);
+            gwy_field_multiply(dfield, r*p/(max - min));
         }
 
-        gwy_app_channel_title_fall_back(container, 0);
+        gwy_image_title_fall_back(file, 0);
     }
     else
-        gwy_data_field_multiply(dfield, 1e-9);
+        gwy_field_multiply(dfield, 1e-9);
 
     /* Fix lateral scale */
     if (get_double_value(wsxmmeta, "Control::X Amplitude", &r, &end)) {
-        siunit = gwy_si_unit_new_parse(end, &power10);
-        gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(dfield), siunit);
+        siunit = gwy_unit_new_parse(end, &power10);
+        gwy_unit_assign(gwy_field_get_unit_xy(dfield), siunit);
         g_object_unref(siunit);
-        gwy_data_field_set_xreal(dfield, r*pow10(power10));
+        gwy_field_set_xreal(dfield, r*gwy_exp10(power10));
     }
 
     if (get_double_value(wsxmmeta, "Control::Y Amplitude", &r, &end)) {
-        siunit = gwy_si_unit_new_parse(end, &power10);
+        siunit = gwy_unit_new_parse(end, &power10);
         g_object_unref(siunit);
-        gwy_data_field_set_yreal(dfield, r*pow10(power10));
+        gwy_field_set_yreal(dfield, r*gwy_exp10(power10));
     }
     else
-        gwy_data_field_set_yreal(dfield, gwy_data_field_get_xreal(dfield));
+        gwy_field_set_yreal(dfield, gwy_field_get_xreal(dfield));
 
     /* And store everything else as metadata */
     for (i = 0; i < G_N_ELEMENTS(nometa); i++)
         g_hash_table_remove(wsxmmeta, nometa[i]);
 
-    meta = gwy_container_new();
+    GwyContainer *meta = gwy_container_new_in_construction();
     g_hash_table_foreach(wsxmmeta, store_meta, meta);
-    if (gwy_container_get_n_items(meta)) {
-        quark = gwy_app_get_data_meta_key_for_id(0);
-        gwy_container_set_object(container, quark, meta);
-    }
-    g_object_unref(meta);
+    if (gwy_container_get_n_items(meta))
+        gwy_file_pass_meta(file, GWY_FILE_IMAGE, 0, meta);
+    else
+        g_object_unref(meta);
 }
 
-static GwyDataField*
-read_data_field(const guchar *buffer,
-                gint xres, gint yres,
-                GwyRawDataType type)
+static GwyField*
+read_field(const guchar *buffer,
+           gint xres, gint yres,
+           GwyRawDataType type)
 {
-    GwyDataField *dfield;
-
-    dfield = gwy_data_field_new(xres, yres, 1e-6, 1e-6, FALSE);
+    GwyField *dfield = gwy_field_new(xres, yres, 1e-6, 1e-6, FALSE);
     /* The conversion is probably wrong for the SINT16 type. */
     gwy_convert_raw_data(buffer, xres*yres, 1, type, GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                         gwy_data_field_get_data(dfield), 1.0, 0.0);
-    gwy_data_field_invert(dfield, TRUE, TRUE, FALSE);
+                         gwy_field_get_data(dfield), 1.0, 0.0);
+    gwy_field_flip(dfield, TRUE, TRUE);
 
     return dfield;
 }
@@ -619,14 +599,14 @@ read_graph_model(const guchar *p,
     gchar *end, *s;
     guint i, j, k, n = ncurves*npts;
     gint pwr10;
-    GwySIUnit *xunit, *yunit;
+    GwyUnit *xunit, *yunit;
     GString *str;
     gdouble qx = 1.0, qy = 1.0;
 
-    xunit = gwy_si_unit_new_parse(xunitstr, &pwr10);
-    qx = pow10(pwr10);
-    yunit = gwy_si_unit_new_parse(yunitstr, &pwr10);
-    qy = pow10(pwr10);
+    xunit = gwy_unit_new_parse(xunitstr, &pwr10);
+    qx = gwy_exp10(pwr10);
+    yunit = gwy_unit_new_parse(yunitstr, &pwr10);
+    qy = gwy_exp10(pwr10);
 
     data = g_new(gdouble, 2*n);
     for (i = 0; i < npts; i++) {
@@ -666,8 +646,8 @@ read_graph_model(const guchar *p,
     g_string_append(str, " curve");
 
     g_object_set(gmodel,
-                 "si-unit-x", xunit,
-                 "si-unit-y", yunit,
+                 "unit-x", xunit,
+                 "unit-y", yunit,
                  "title", str->str,
                  NULL);
 
@@ -700,9 +680,9 @@ fail:
 }
 
 static gboolean
-wsxmfile_export_double(GwyContainer *data,
+wsxmfile_export_double(GwyFile *data,
                        const gchar *filename,
-                       G_GNUC_UNUSED GwyRunType mode,
+                       G_GNUC_UNUSED GwyRunModeFlags mode,
                        GError **error)
 {
     static const gchar header_template[] =
@@ -730,7 +710,7 @@ wsxmfile_export_double(GwyContainer *data,
         "\r\n"
         "[Header end]\r\n";
 
-    GwyDataField *dfield;
+    GwyField *dfield;
     const gdouble *d, *r;
     guint8 *bytes = NULL;
     gdouble *dbuf = NULL, *drow;
@@ -743,9 +723,9 @@ wsxmfile_export_double(GwyContainer *data,
     gboolean ok = FALSE;
     FILE *fh;
 
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &dfield,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &dfield,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     if (!dfield) {
         err_NO_CHANNEL_EXPORT(error);
         return FALSE;
@@ -756,17 +736,17 @@ wsxmfile_export_double(GwyContainer *data,
         return FALSE;
     }
 
-    d = gwy_data_field_get_data_const(dfield);
-    xres = gwy_data_field_get_xres(dfield);
-    yres = gwy_data_field_get_yres(dfield);
-    gwy_data_field_get_min_max(dfield, &min, &max);
+    d = gwy_field_get_data_const(dfield);
+    xres = gwy_field_get_xres(dfield);
+    yres = gwy_field_get_yres(dfield);
+    gwy_field_get_min_max(dfield, &min, &max);
 
-    xyunit = gwy_si_unit_get_string(gwy_data_field_get_si_unit_xy(dfield), GWY_SI_UNIT_FORMAT_PLAIN);
-    zunit = gwy_si_unit_get_string(gwy_data_field_get_si_unit_z(dfield), GWY_SI_UNIT_FORMAT_PLAIN);
-    title = gwy_app_get_data_field_title(data, id);
+    xyunit = gwy_unit_get_string(gwy_field_get_unit_xy(dfield), GWY_UNIT_FORMAT_PLAIN);
+    zunit = gwy_unit_get_string(gwy_field_get_unit_z(dfield), GWY_UNIT_FORMAT_PLAIN);
+    title = gwy_file_get_display_title(data, GWY_FILE_IMAGE, id);
     header = g_strdup_printf(header_template,
-                             gwy_data_field_get_xreal(dfield), xyunit,
-                             gwy_data_field_get_yreal(dfield), xyunit,
+                             gwy_field_get_xreal(dfield), xyunit,
+                             gwy_field_get_yreal(dfield), xyunit,
                              title, xres, yres,
                              max-min, zunit,
                              gwy_version_string());
