@@ -1,6 +1,6 @@
 /*
- *  $Id: xyzexport.c 25245 2023-02-23 16:16:20Z yeti-dn $
- *  Copyright (C) 2015-2022 David Necas (Yeti).
+ *  $Id: xyzexport.c 28822 2025-11-06 16:01:09Z yeti-dn $
+ *  Copyright (C) 2015-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -23,20 +23,14 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 #include <glib/gstdio.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <libprocess/datafield.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libgwydgets/gwyradiobuttons.h>
-#include <libgwydgets/gwydgetutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 
 #include "err.h"
 
@@ -53,33 +47,33 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyAppPage pageno;
+    GwyDataKind data_kind;
     gboolean have_mask;
 } ModuleArgs;
 
-static gboolean         module_register            (void);
-static gint             xyzexport_detect           (const GwyFileDetectInfo *fileinfo,
-                                                    gboolean only_name);
-static gboolean         xyzexport_export           (GwyContainer *data,
-                                                    const gchar *filename,
-                                                    GwyRunType mode,
-                                                    GError **error);
-static GwyDialogOutcome run_gui                    (ModuleArgs *args,
-                                                    const gchar *title);
-static gboolean         xyzexport_export_data_field(GwyDataField *dfield,
-                                                    GwyDataField *mfield,
-                                                    FILE *fh,
-                                                    ModuleArgs *args);
-static gboolean         xyzexport_export_surface   (GwySurface *surface,
-                                                    FILE *fh,
-                                                    ModuleArgs *args);
+static gboolean         module_register(void);
+static gint             detect_file    (const GwyFileDetectInfo *fileinfo,
+                                        gboolean only_name);
+static gboolean         export_file    (GwyFile *file,
+                                        const gchar *filename,
+                                        GwyRunModeFlags mode,
+                                        GError **error);
+static GwyDialogOutcome run_gui        (ModuleArgs *args,
+                                        const gchar *title);
+static gboolean         export_field   (GwyField *dfield,
+                                        GwyField *mfield,
+                                        FILE *fh,
+                                        ModuleArgs *args);
+static gboolean         export_surface (GwySurface *surface,
+                                        FILE *fh,
+                                        ModuleArgs *args);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Exports data as simple XYZ text file."),
     "Yeti <yeti@gwyddion.net>",
-    "3.1",
+    "4.0",
     "David Nečas (Yeti)",
     "2015",
 };
@@ -91,10 +85,7 @@ module_register(void)
 {
     gwy_file_func_register("xyzexport",
                            N_("XYZ text data (.xyz)"),
-                           (GwyFileDetectFunc)&xyzexport_detect,
-                           NULL,
-                           NULL,
-                           (GwyFileSaveFunc)&xyzexport_export);
+                           detect_file, NULL, NULL, export_file);
 
     return TRUE;
 }
@@ -119,42 +110,42 @@ define_module_params(void)
 }
 
 static gint
-xyzexport_detect(const GwyFileDetectInfo *fileinfo,
-                 G_GNUC_UNUSED gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo,
+            G_GNUC_UNUSED gboolean only_name)
 {
     return g_str_has_suffix(fileinfo->name_lowercase, EXTENSION) ? 20 : 0;
 }
 
 static gboolean
-xyzexport_export(G_GNUC_UNUSED GwyContainer *data,
-                 const gchar *filename,
-                 GwyRunType mode,
-                 GError **error)
+export_file(GwyFile *file,
+            const gchar *filename,
+            GwyRunModeFlags mode,
+            GError **error)
 {
     ModuleArgs args;
-    GwyDataField *dfield, *mfield;
+    GwyField *dfield, *mfield;
     GwySurface *surface;
     gint fid, sid;
-    const guchar *title = NULL;
+    const gchar *title = NULL;
     GwyDialogOutcome outcome;
     FILE *fh = NULL;
     gboolean ok = FALSE;
 
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &dfield,
-                                     GWY_APP_DATA_FIELD_ID, &fid,
-                                     GWY_APP_MASK_FIELD, &mfield,
-                                     GWY_APP_SURFACE, &surface,
-                                     GWY_APP_SURFACE_ID, &sid,
-                                     GWY_APP_PAGE, &args.pageno,
-                                     0);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &dfield,
+                                 GWY_APP_FIELD_ID, &fid,
+                                 GWY_APP_MASK_FIELD, &mfield,
+                                 GWY_APP_SURFACE, &surface,
+                                 GWY_APP_SURFACE_ID, &sid,
+                                 GWY_APP_DATA_KIND, &args.data_kind,
+                                 0);
 
     /* Ensure at most one is set.  We produce an error if no exportable data type is available or both types are
      * available but neither is active. When only one is available or one is active we assume that is what the user
      * wants to export. */
     if (dfield && surface) {
-        if (args.pageno != GWY_PAGE_CHANNELS)
+        if (args.data_kind != GWY_FILE_IMAGE)
             dfield = NULL;
-        if (args.pageno != GWY_PAGE_XYZS)
+        if (args.data_kind != GWY_FILE_XYZ)
             surface = NULL;
     }
     if (!dfield && !surface) {
@@ -163,20 +154,16 @@ xyzexport_export(G_GNUC_UNUSED GwyContainer *data,
     }
 
     if (dfield) {
-        args.pageno = GWY_PAGE_CHANNELS;
+        args.data_kind = GWY_FILE_IMAGE;
     }
     if (surface) {
-        args.pageno = GWY_PAGE_XYZS;
+        args.data_kind = GWY_FILE_XYZ;
         mfield = NULL;
     }
 
     args.params = gwy_params_new_from_settings(define_module_params());
 
-    if (dfield)
-        gwy_container_gis_string(data, gwy_app_get_data_title_key_for_id(fid), &title);
-    if (surface)
-        gwy_container_gis_string(data, gwy_app_get_surface_title_key_for_id(sid), &title);
-
+    title = gwy_file_get_title(file, args.data_kind, dfield ? fid : sid);
     if (!title)
         title = _("Untitled");
 
@@ -195,37 +182,37 @@ xyzexport_export(G_GNUC_UNUSED GwyContainer *data,
     }
 
     if (gwy_params_get_boolean(args.params, PARAM_ADD_COMMENT)) {
-        GwySIUnit *xyunit = NULL, *zunit = NULL;
+        GwyUnit *xyunit = NULL, *zunit = NULL;
         gchar *s;
 
-        if (args.pageno == GWY_PAGE_CHANNELS) {
+        if (args.data_kind == GWY_FILE_IMAGE) {
             gwy_fprintf(fh, "# %s %s\n", _("Channel:"), title);
-            xyunit = gwy_data_field_get_si_unit_xy(dfield);
-            zunit = gwy_data_field_get_si_unit_z(dfield);
+            xyunit = gwy_field_get_unit_xy(dfield);
+            zunit = gwy_field_get_unit_z(dfield);
         }
-        else if (args.pageno == GWY_PAGE_XYZS) {
+        else if (args.data_kind == GWY_FILE_XYZ) {
             gwy_fprintf(fh, "# %s %s\n", _("XYZ data:"), title);
-            xyunit = gwy_surface_get_si_unit_xy(surface);
-            zunit = gwy_surface_get_si_unit_z(surface);
+            xyunit = gwy_surface_get_unit_xy(surface);
+            zunit = gwy_surface_get_unit_z(surface);
         }
 
-        s = gwy_si_unit_get_string(xyunit, GWY_SI_UNIT_FORMAT_VFMARKUP);
+        s = gwy_unit_get_string(xyunit, GWY_UNIT_FORMAT_VFMARKUP);
         gwy_fprintf(fh, "# %s %s\n", _("Lateral units:"), s);
         g_free(s);
 
-        s = gwy_si_unit_get_string(zunit, GWY_SI_UNIT_FORMAT_VFMARKUP);
+        s = gwy_unit_get_string(zunit, GWY_UNIT_FORMAT_VFMARKUP);
         gwy_fprintf(fh, "# %s %s\n", _("Value units:"), s);
         g_free(s);
     }
 
-    if (args.pageno == GWY_PAGE_CHANNELS) {
-        if (!xyzexport_export_data_field(dfield, mfield, fh, &args)) {
+    if (args.data_kind == GWY_FILE_IMAGE) {
+        if (!export_field(dfield, mfield, fh, &args)) {
             err_WRITE(error);
             goto fail;
         }
     }
-    else if (args.pageno == GWY_PAGE_XYZS) {
-        if (!xyzexport_export_surface(surface, fh, &args)) {
+    else if (args.data_kind == GWY_FILE_XYZ) {
+        if (!export_surface(surface, fh, &args)) {
             err_WRITE(error);
             goto fail;
         }
@@ -256,9 +243,9 @@ run_gui(ModuleArgs *args, const gchar *title)
     dialog = GWY_DIALOG(gwy_dialog_new(_("Export XYZ")));
     gwy_dialog_add_buttons(dialog, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    if (args->pageno == GWY_PAGE_CHANNELS)
-        desc = _("Channel");
-    else if (args->pageno == GWY_PAGE_XYZS)
+    if (args->data_kind == GWY_FILE_IMAGE)
+        desc = _("Image");
+    else if (args->data_kind == GWY_FILE_XYZ)
         desc = _("XYZ data");
 
     table = gwy_param_table_new(args->params);
@@ -279,8 +266,8 @@ run_gui(ModuleArgs *args, const gchar *title)
 }
 
 static gboolean
-xyzexport_export_data_field(GwyDataField *dfield, GwyDataField *mfield,
-                            FILE *fh, ModuleArgs *args)
+export_field(GwyField *dfield, GwyField *mfield,
+             FILE *fh, ModuleArgs *args)
 {
     gboolean decimal_dot = gwy_params_get_boolean(args->params, PARAM_DECIMAL_DOT);
     gint precision = gwy_params_get_int(args->params, PARAM_PRECISION);
@@ -291,15 +278,15 @@ xyzexport_export_data_field(GwyDataField *dfield, GwyDataField *mfield,
     gdouble xyz[3];
     GString *str;
 
-    xres = gwy_data_field_get_xres(dfield);
-    yres = gwy_data_field_get_yres(dfield);
-    dx = gwy_data_field_get_dx(dfield);
-    dy = gwy_data_field_get_dy(dfield);
-    xoff = gwy_data_field_get_xoffset(dfield);
-    yoff = gwy_data_field_get_yoffset(dfield);
+    xres = gwy_field_get_xres(dfield);
+    yres = gwy_field_get_yres(dfield);
+    dx = gwy_field_get_dx(dfield);
+    dy = gwy_field_get_dy(dfield);
+    xoff = gwy_field_get_xoffset(dfield);
+    yoff = gwy_field_get_yoffset(dfield);
 
-    d = gwy_data_field_get_data_const(dfield);
-    m = mfield ? gwy_data_field_get_data_const(mfield) : NULL;
+    d = gwy_field_get_data_const(dfield);
+    m = mfield ? gwy_field_get_data_const(mfield) : NULL;
 
     str = g_string_new(NULL);
     for (i = 0; i < yres; i++) {
@@ -330,8 +317,8 @@ xyzexport_export_data_field(GwyDataField *dfield, GwyDataField *mfield,
 }
 
 static gboolean
-xyzexport_export_surface(GwySurface *surface,
-                         FILE *fh, ModuleArgs *args)
+export_surface(GwySurface *surface,
+               FILE *fh, ModuleArgs *args)
 {
     gboolean decimal_dot = gwy_params_get_boolean(args->params, PARAM_DECIMAL_DOT);
     gint precision = gwy_params_get_int(args->params, PARAM_PRECISION);
