@@ -1,6 +1,6 @@
 /*
- *  $Id: zeiss.c 26163 2024-02-06 16:56:55Z yeti-dn $
- *  Copyright (C) 2011 David Necas (Yeti).
+ *  $Id: zeiss.c 28789 2025-11-04 17:14:03Z yeti-dn $
+ *  Copyright (C) 2011-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -28,11 +28,9 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/stats.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 #include "err.h"
 #include "gwytiff.h"
 
@@ -43,25 +41,25 @@ enum {
     ZEISS_HEADER_TAG = 34118,
 };
 
-static gboolean      module_register(void);
-static gint          zeiss_detect   (const GwyFileDetectInfo *fileinfo,
-                                     gboolean only_name);
-static GwyContainer* zeiss_load     (const gchar *filename,
-                                     GwyRunType mode,
-                                     GError **error);
-static GwyContainer* zeiss_load_tiff(const GwyTIFF *tiff,
-                                     GError **error);
-static void          add_meta       (gpointer hkey,
-                                     gpointer hvalue,
-                                     gpointer user_data);
-static GHashTable*   parse_comment  (gchar *text);
+static gboolean    module_register(void);
+static gint        detect_file    (const GwyFileDetectInfo *fileinfo,
+                                   gboolean only_name);
+static GwyFile*    load_file      (const gchar *filename,
+                                   GwyRunModeFlags mode,
+                                   GError **error);
+static GwyFile*    zeiss_load_tiff(const GwyTIFF *tiff,
+                                   GError **error);
+static void        add_meta       (gpointer hkey,
+                                   gpointer hvalue,
+                                   gpointer user_data);
+static GHashTable* parse_comment  (gchar *text);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     module_register,
     N_("Imports Carl Zeiss SEM images."),
     "Yeti <yeti@gwyddion.net>",
-    "0.6",
+    "0.7",
     "David Nečas (Yeti)",
     "2011",
 };
@@ -73,16 +71,13 @@ module_register(void)
 {
     gwy_file_func_register("zeiss",
                            N_("Carl Zeiss SEM scans (.tif)"),
-                           (GwyFileDetectFunc)&zeiss_detect,
-                           (GwyFileLoadFunc)&zeiss_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-zeiss_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     GwyTIFF *tiff;
     gint score = 0;
@@ -113,33 +108,34 @@ zeiss_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
     return score;
 }
 
-static GwyContainer*
-zeiss_load(const gchar *filename,
-           G_GNUC_UNUSED GwyRunType mode,
-           GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
     GwyTIFF *tiff;
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
 
     tiff = gwy_tiff_load(filename, error);
     if (!tiff)
         return NULL;
 
-    container = zeiss_load_tiff(tiff, error);
-    if (container)
-        gwy_file_channel_import_log_add(container, 0, NULL, filename);
+    file = zeiss_load_tiff(tiff, error);
+    if (file)
+        gwy_log_add_import(file, GWY_FILE_IMAGE, 0, NULL, filename);
 
     gwy_tiff_free(tiff);
 
-    return container;
+    return file;
 }
 
-static GwyContainer*
+static GwyFile*
 zeiss_load_tiff(const GwyTIFF *tiff, GError **error)
 {
-    GwyContainer *container = NULL, *meta = NULL;
-    GwyDataField *dfield;
-    GwySIUnit *siunit;
+    GwyFile *file = NULL;
+    GwyContainer *meta = NULL;
+    GwyField *dfield;
+    GwyUnit *siunit;
     GwyTIFFImageReader *reader = NULL;
     GHashTable *hash = NULL;
     gint i, power10;
@@ -193,29 +189,30 @@ zeiss_load_tiff(const GwyTIFF *tiff, GError **error)
     if (!(reader = gwy_tiff_get_image_reader(tiff, 0, 3, error)))
         goto fail;
 
-    siunit = gwy_si_unit_new_parse(end, &power10);
-    factor = pow10(power10);
-    dfield = gwy_data_field_new(reader->width, reader->height,
-                                reader->width * factor * dx, reader->height * factor * dx,
-                                FALSE);
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(dfield), siunit);
+    siunit = gwy_unit_new_parse(end, &power10);
+    factor = gwy_exp10(power10);
+    dfield = gwy_field_new(reader->width, reader->height,
+                           reader->width * factor * dx, reader->height * factor * dx,
+                           FALSE);
+    gwy_unit_assign(gwy_field_get_unit_xy(dfield), siunit);
     g_object_unref(siunit);
 
-    data = gwy_data_field_get_data(dfield);
+    data = gwy_field_get_data(dfield);
     for (i = 0; i < reader->height; i++)
         gwy_tiff_read_image_row_averaged(tiff, reader, i, 1.0, 0.0, data + i*reader->width);
     gwy_tiff_image_reader_free(reader);
 
-    container = gwy_container_new();
-    gwy_container_pass_object(container, gwy_app_get_data_key_for_id(0), dfield);
-    gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(0), "Secondary electron count");
+    file = gwy_file_new_in_construction();
+    gwy_file_pass_image(file, 0, dfield);
+    gwy_file_set_title(file, GWY_FILE_IMAGE, 0, "Secondary electron count", FALSE);
 
     if (new_file) {
-        meta = gwy_container_new();
+        meta = gwy_container_new_in_construction();
         g_hash_table_foreach(hash, add_meta, meta);
         if (gwy_container_get_n_items(meta))
-            gwy_container_set_object(container, gwy_app_get_data_meta_key_for_id(0), meta);
-        g_object_unref(meta);
+            gwy_file_pass_meta(file, GWY_FILE_IMAGE, 0, meta);
+        else
+            g_object_unref(meta);
     }
 
 fail:
@@ -223,7 +220,7 @@ fail:
         g_hash_table_destroy(hash);
     g_free(comment);
 
-    return container;
+    return file;
 }
 
 static void
