@@ -1,22 +1,18 @@
 /*
- *  $Id: zeissczi.c 25135 2022-11-02 14:01:25Z yeti-dn $
+ *  $Id: zeissczi.c 28772 2025-11-03 18:02:12Z yeti-dn $
  *  Copyright (C) 2022 David Necas (Yeti), Daniil Bratashov (dn2010)
  *  E-mail: dn2010@gwyddion.net.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
+ *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
+ *  later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 /*
@@ -55,10 +51,9 @@
 
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymath.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 #include "err.h"
 
 #define MAGIC "ZISRAWFILE"
@@ -123,12 +118,12 @@ typedef struct {
     const guchar *attachments;
 } CziRawSubBlock;
 
-static gboolean           module_register       (void);
-static gint               czi_detect            (const GwyFileDetectInfo *fileinfo,
-                                                 gboolean only_name);
-static GwyContainer*      czi_load              (const gchar *filename,
-                                                 GwyRunType mode,
-                                                 GError **error);
+static gboolean module_register(void);
+static gint     detect_file    (const GwyFileDetectInfo *fileinfo,
+                                gboolean only_name);
+static GwyFile* load_file      (const gchar *filename,
+                                GwyRunModeFlags mode,
+                                GError **error);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
@@ -147,22 +142,18 @@ module_register(void)
 {
     gwy_file_func_register("zeissczi",
                            N_("Carl Zeiss CZI images (.czi)"),
-                           (GwyFileDetectFunc)&czi_detect,
-                           (GwyFileLoadFunc)&czi_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-czi_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     gint score = 0;
 
     if (only_name)
-        return (g_str_has_suffix(fileinfo->name_lowercase, EXTENSION))
-                ? 20 : 0;
+        return (g_str_has_suffix(fileinfo->name_lowercase, EXTENSION)) ? 20 : 0;
 
     if (fileinfo->buffer_len > MAGIC_SIZE
         && memcmp(fileinfo->head, MAGIC, MAGIC_SIZE) == 0)
@@ -171,15 +162,15 @@ czi_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
     return score;
 }
 
-static GwyContainer*
-czi_load(const gchar *filename,
-         G_GNUC_UNUSED GwyRunType mode,
-         GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-    GwyContainer *container = NULL;
+    GwyFile *file = NULL;
     gchar *buffer = NULL;
     gsize size = 0;
-    GError *err = NULL;    
+    GError *err = NULL;
     const guchar *p;
 
     if (!g_file_get_contents(filename, &buffer, &size, &err)) {
@@ -195,7 +186,9 @@ czi_load(const gchar *filename,
     p = buffer;
 
 fail:
-	g_free(buffer);
+    g_free(buffer);
 
-    return container;
+    return file;
 }
+
+/* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
