@@ -1,6 +1,6 @@
 /*
- *  $Id: zeisslsm.c 26012 2023-11-10 10:13:31Z yeti-dn $
- *  Copyright (C) 2017 David Necas (Yeti), Daniil Bratashov (dn2010).
+ *  $Id: zeisslsm.c 28803 2025-11-05 11:58:19Z yeti-dn $
+ *  Copyright (C) 2017-2025 David Necas (Yeti), Daniil Bratashov (dn2010).
  *  E-mail: dn2010@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -44,12 +44,9 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/stats.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 #include "err.h"
 #include "gwytiff.h"
 
@@ -334,35 +331,35 @@ typedef struct {
     gdouble laser_power;
 } LSMEntryRecording;
 
-static gboolean           module_register       (void);
-static gint               lsm_detect            (const GwyFileDetectInfo *fileinfo,
-                                                 gboolean only_name);
-static GwyContainer*      lsm_load              (const gchar *filename,
-                                                 GwyRunType mode,
-                                                 GError **error);
-static GwyContainer*      lsm_load_tiff         (const GwyTIFF *tiff,
-                                                 const gchar *filename,
-                                                 GError **error);
-static LSMHeaderTag*      lsm_read_header_tag   (const GwyTIFF *tiff,
-                                                 const GwyTIFFEntry *tag,
-                                                 GError **error);
-static LSMNamesColors*    lsm_read_names_colors (const GwyTIFF *tiff,
-                                                 guint32 offset,
-                                                 GError **error);
-static LSMEntryRecording* lsm_read_recording    (const GwyTIFF *tiff,
-                                                 GwyContainer *meta,
-                                                 guint32 offset,
-                                                 GError **error);
-static LSMEntry*          lsm_read_entry        (const GwyTIFF *tiff,
-                                                 guint32 offset,
-                                                 GError **error);
+static gboolean           module_register  (void);
+static gint               detect_file      (const GwyFileDetectInfo *fileinfo,
+                                            gboolean only_name);
+static GwyFile*           load_file        (const gchar *filename,
+                                            GwyRunModeFlags mode,
+                                            GError **error);
+static GwyFile*           load_tiff        (const GwyTIFF *tiff,
+                                            const gchar *filename,
+                                            GError **error);
+static LSMHeaderTag*      read_header_tag  (const GwyTIFF *tiff,
+                                            const GwyTIFFEntry *tag,
+                                            GError **error);
+static LSMNamesColors*    read_names_colors(const GwyTIFF *tiff,
+                                            guint32 offset,
+                                            GError **error);
+static LSMEntryRecording* read_recording   (const GwyTIFF *tiff,
+                                            GwyContainer *meta,
+                                            guint32 offset,
+                                            GError **error);
+static LSMEntry*          read_entry       (const GwyTIFF *tiff,
+                                            guint32 offset,
+                                            GError **error);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     module_register,
     N_("Imports Carl Zeiss CLSM images."),
     "Daniil Bratashov <dn2010@gwyddion.net>",
-    "0.4",
+    "0.5",
     "Daniil Bratashov (dn2010), David Nečas (Yeti)",
     "2017",
 };
@@ -374,32 +371,29 @@ module_register(void)
 {
     gwy_file_func_register("zeisslsm",
                            N_("Carl Zeiss CLSM images (.lsm)"),
-                           (GwyFileDetectFunc)&lsm_detect,
-                           (GwyFileLoadFunc)&lsm_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-lsm_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     GwyTIFF *tiff = NULL;
     gint score = 0;
     GwyTIFFVersion version = GWY_TIFF_CLASSIC;
     guint byteorder = G_LITTLE_ENDIAN;
-    const GwyTIFFEntry *lsm_tag = NULL;
+    const GwyTIFFEntry *tag = NULL;
 
     if (only_name)
-        return (g_str_has_suffix(fileinfo->name_lowercase, EXTENSION)) ? 20 : 0;
+        return g_str_has_suffix(fileinfo->name_lowercase, EXTENSION) ? 20 : 0;
 
     /* Weed out non-TIFFs */
     if (!gwy_tiff_detect(fileinfo->head, fileinfo->buffer_len, &version, &byteorder))
         return 0;
 
     if ((tiff = gwy_tiff_load(fileinfo->name, NULL))
-        && (lsm_tag = gwy_tiff_find_tag(tiff, 0, ZEISS_LSM_HEADER_TAG))) {
+        && (tag = gwy_tiff_find_tag(tiff, 0, ZEISS_LSM_HEADER_TAG))) {
         score = 100;
     }
 
@@ -409,24 +403,19 @@ lsm_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
     return score;
 }
 
-static GwyContainer*
-lsm_load(const gchar *filename,
-         G_GNUC_UNUSED GwyRunType mode,
-         GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
-
-    GwyTIFF *tiff;
-    GwyContainer *container = NULL;
-
-    tiff = gwy_tiff_load(filename, error);
+    GwyTIFF *tiff = gwy_tiff_load(filename, error);
     if (!tiff)
         return NULL;
 
-    container = lsm_load_tiff(tiff, filename, error);
-
+    GwyFile *file = load_tiff(tiff, filename, error);
     gwy_tiff_free(tiff);
 
-    return container;
+    return file;
 }
 
 static const gchar*
@@ -455,23 +444,24 @@ palette_name_for_photometric(const GwyTIFFImageReader *reader, guint i)
     return "Gray";
 }
 
-static GwyContainer*
-lsm_load_tiff(const GwyTIFF *tiff,
-              const gchar *filename,
-              GError **error)
+static GwyFile*
+load_tiff(const GwyTIFF *tiff,
+          const gchar *filename,
+          GError **error)
 {
-    GwyContainer *container = NULL, *meta = NULL;
-    GwyDataLine *dataline = NULL;
+    GwyFile *file = NULL;
+    GwyContainer *meta = NULL;
+    GwyLine *line = NULL;
     GwyGraphCurveModel *gcmodel;
     GwyGraphModel *gmodel = NULL;
-    GwyDataField *dfield = NULL;
+    GwyField *dfield = NULL;
     GwyBrick *brick;
-    GwySIUnit *siunit;
+    GwyUnit *siunit;
     gint i, j, k, l, volumes, ndirs, z, xres = 0, yres = 0, zres = 0;
     gdouble xreal = 1.0, yreal = 1.0, zreal = 1.0;
     gchar *name;
     const gchar *lutname;
-    const GwyTIFFEntry *lsm_tag;
+    const GwyTIFFEntry *tag;
     gdouble *data;
     GwyTIFFImageReader *reader = NULL;
     LSMHeaderTag *header_tag = NULL;
@@ -480,24 +470,24 @@ lsm_load_tiff(const GwyTIFF *tiff,
     gboolean is_image = FALSE, is_volume = FALSE, is_line = FALSE;
     GArray *bricks, *bricks_preview;
 
-    if (!(lsm_tag = gwy_tiff_find_tag(tiff, 0, ZEISS_LSM_HEADER_TAG))) {
+    if (!(tag = gwy_tiff_find_tag(tiff, 0, ZEISS_LSM_HEADER_TAG))) {
         err_FILE_TYPE(error, "Carl Zeiss LSM");
         goto fail;
     }
 
-    if (!(header_tag = lsm_read_header_tag(tiff, lsm_tag, error))) {
+    if (!(header_tag = read_header_tag(tiff, tag, error))) {
         err_FILE_TYPE(error, "Carl Zeiss LSM");
         goto fail;
     }
 
-    names_colors = lsm_read_names_colors(tiff, header_tag->offset_channel_colors_names, error);
-    meta = gwy_container_new();
-    recording = lsm_read_recording(tiff, meta, header_tag->offset_scan_information, error);
+    names_colors = read_names_colors(tiff, header_tag->offset_channel_colors_names, error);
+    meta = gwy_container_new_in_construction();
+    recording = read_recording(tiff, meta, header_tag->offset_scan_information, error);
 
     ndirs = gwy_tiff_get_n_dirs(tiff);
     gwy_debug("ndirs=%u", ndirs);
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     k = 0; /* number of images in resulting file */
     volumes = 0;
 
@@ -547,9 +537,9 @@ lsm_load_tiff(const GwyTIFF *tiff,
                 if (xres != header_tag->xres)
                     xreal = header_tag->xres * header_tag->x_voxel_size;
                 is_line = TRUE;
-                siunit = gwy_si_unit_new("m");
+                siunit = gwy_unit_new("m");
                 gmodel = g_object_new(GWY_TYPE_GRAPH_MODEL,
-                                      "si-unit-x", siunit,
+                                      "unit-x", siunit,
                                       NULL);
                 g_object_unref(siunit);
             break;
@@ -606,14 +596,14 @@ lsm_load_tiff(const GwyTIFF *tiff,
              * gwy_tiff_read_image_row() because it can then fail. Without compression a succesfully created reader
              * can never fail. */
             if (is_image || is_volume) {
-                dfield = gwy_data_field_new(xres, yres, xreal, yreal, TRUE);
-                data = gwy_data_field_get_data(dfield);
+                dfield = gwy_field_new(xres, yres, xreal, yreal, TRUE);
+                data = gwy_field_get_data(dfield);
                 for (l = 0; l < yres; l++)
                     gwy_tiff_read_image_row(tiff, reader, j, l, 1.0, 0.0, data + l*xres);
             }
             else if (is_line) {
-                dataline = gwy_data_line_new(xres, xreal, TRUE);
-                data = gwy_data_line_get_data(dataline);
+                line = gwy_line_new(xres, xreal, TRUE);
+                data = gwy_line_get_data(line);
                 /* FIXME: I do not see yres == 1 checked or enforced elsewhere. Perhaps we could do this in a cycle
                  * and create more curves? Just read the single image row for now to be on the safe side. */
                 gwy_tiff_read_image_row(tiff, reader, j, 0, 1.0, 0.0, data);
@@ -628,35 +618,33 @@ lsm_load_tiff(const GwyTIFF *tiff,
                                        "mode", GWY_GRAPH_CURVE_LINE,
                                        "color", gwy_graph_get_preset_color(k),
                                        NULL);
-                gwy_graph_curve_model_set_data_from_dataline(gcmodel, dataline, 0, 0);
+                gwy_graph_curve_model_set_data_from_line(gcmodel, line, 0, 0);
                 gwy_graph_model_add_curve(gmodel, gcmodel);
                 g_object_unref(gcmodel);
             }
 
             if (is_image) {
-                gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(dfield), "m");
-                gwy_container_pass_object(container, gwy_app_get_data_key_for_id(k), dfield);
+                gwy_unit_set_from_string(gwy_field_get_unit_xy(dfield), "m");
+                gwy_file_pass_image(file, k, dfield);
                 if (gwy_container_get_n_items(meta)) {
-                    gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(k),
-                                              gwy_container_duplicate(meta));
+                    gwy_file_pass_meta(file, GWY_FILE_IMAGE, k, gwy_container_copy(meta));
                 }
-                gwy_file_channel_import_log_add(container, k, NULL, filename);
+                gwy_log_add_import(file, GWY_FILE_IMAGE, k, NULL, filename);
 
                 if (names_colors && (i % 2 == 0)) {
                     name = (gchar*)g_ptr_array_index(names_colors->names, j);
-                    gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(k), name);
+                    gwy_file_set_title(file, GWY_FILE_IMAGE, k, name, FALSE);
 
                     lutname = "Gray";
                     if (j < names_colors->colors->len)
                         lutname = palette_name_for_colour_num(g_array_index(names_colors->colors, gint32, j));
-                    gwy_container_set_const_string(container, gwy_app_get_data_palette_key_for_id(k), lutname);
+                    gwy_file_set_palette(file, GWY_FILE_IMAGE, k, lutname);
                 }
                 else {
-                    gwy_container_set_string(container, gwy_app_get_data_title_key_for_id(k),
-                                             g_strdup_printf("LSM Image %u (channel %u)", i/2, j));
+                    gwy_file_pass_title(file, GWY_FILE_IMAGE, k, g_strdup_printf("LSM Image %u (channel %u)", i/2, j));
 
                     lutname = palette_name_for_photometric(reader, j);
-                    gwy_container_set_const_string(container, gwy_app_get_data_palette_key_for_id(k), lutname);
+                    gwy_file_set_palette(file, GWY_FILE_IMAGE, k, lutname);
                 } /* else */
             }
 
@@ -664,13 +652,13 @@ lsm_load_tiff(const GwyTIFF *tiff,
                 if (i % (zres*2) == 0 || i % (zres*2) == 1) {
                     brick = gwy_brick_new(xres, yres, zres, xreal, yreal, zreal, TRUE);
 
-                    gwy_si_unit_set_from_string(gwy_brick_get_si_unit_x(brick), "m");
-                    gwy_si_unit_set_from_string(gwy_brick_get_si_unit_y(brick), "m");
+                    gwy_unit_set_from_string(gwy_brick_get_unit_x(brick), "m");
+                    gwy_unit_set_from_string(gwy_brick_get_unit_y(brick), "m");
                     if (header_tag->scan_type == LSM_SCANTYPE_XYZ) {
-                        gwy_si_unit_set_from_string(gwy_brick_get_si_unit_x(brick), "m");
+                        gwy_unit_set_from_string(gwy_brick_get_unit_x(brick), "m");
                     }
                     else {
-                        gwy_si_unit_set_from_string(gwy_brick_get_si_unit_x(brick), "s");
+                        gwy_unit_set_from_string(gwy_brick_get_unit_x(brick), "s");
                     }
 
                     if (i % (zres * 2) == 0)
@@ -678,30 +666,28 @@ lsm_load_tiff(const GwyTIFF *tiff,
                     else
                         g_array_insert_val(bricks_preview, j, brick);
 
-                    gwy_container_pass_object(container, gwy_app_get_brick_key_for_id(volumes+1), brick);
+                    gwy_file_pass_volume(file, volumes+1, brick);
                     if (gwy_container_get_n_items(meta)) {
-                        gwy_container_pass_object(container, gwy_app_get_brick_meta_key_for_id(volumes+1),
-                                                  gwy_container_duplicate(meta));
+                        gwy_file_pass_meta(file, GWY_FILE_VOLUME, volumes+1, gwy_container_copy(meta));
                     }
-                    gwy_file_volume_import_log_add(container, volumes+1, NULL, filename);
+                    gwy_log_add_import(file, GWY_FILE_VOLUME, volumes+1, NULL, filename);
 
                     if (names_colors && (i % 2 == 0)) {
                         name = (gchar *)g_ptr_array_index(names_colors->names, j);
-                        gwy_container_set_const_string(container, gwy_app_get_brick_title_key_for_id(volumes+1), name);
+                        gwy_file_set_title(file, GWY_FILE_VOLUME, volumes+1, name, FALSE);
 
                         lutname = "Gray";
                         if (j < names_colors->colors->len)
                             lutname = palette_name_for_colour_num(g_array_index(names_colors->colors, gint32, j));
-                        gwy_container_set_const_string(container, gwy_app_get_brick_palette_key_for_id(volumes+1),
-                                                       lutname);
+                        gwy_file_set_palette(file, GWY_FILE_VOLUME, volumes+1,
+                                             lutname);
                     }
                     else {
-                        gwy_container_set_string(container, gwy_app_get_brick_title_key_for_id(volumes+1),
-                                                 g_strdup_printf("LSM Volume %d (channel %u)", i/2/zres, j));
+                        gwy_file_pass_title(file, GWY_FILE_VOLUME, volumes+1,
+                                            g_strdup_printf("LSM Volume %d (channel %u)", i/2/zres, j));
 
                         lutname = palette_name_for_photometric(reader, j);
-                        gwy_container_set_const_string(container, gwy_app_get_brick_palette_key_for_id(volumes+1),
-                                                       lutname);
+                        gwy_file_set_palette(file, GWY_FILE_VOLUME, volumes+1, lutname);
                     } /* else */
                     volumes++;
                 }
@@ -712,7 +698,7 @@ lsm_load_tiff(const GwyTIFF *tiff,
                 else
                     brick = g_array_index(bricks, GwyBrick*, j);
 
-                if (gwy_data_field_check_compatibility_with_brick_xy(dfield, brick, GWY_DATA_COMPATIBILITY_RES)) {
+                if (gwy_field_is_incompatible_with_brick_xy(dfield, brick, GWY_DATA_MISMATCH_RES)) {
                     g_object_unref(dfield);
                     /* TODO: A more meaningful error. */
                     err_FILE_TYPE(error, "Carl Zeiss LSM");
@@ -731,11 +717,11 @@ lsm_load_tiff(const GwyTIFF *tiff,
     g_array_free(bricks_preview, TRUE);
 
     if (is_line)
-        gwy_container_pass_object(container, gwy_app_get_graph_key_for_id(1), gmodel);
+        gwy_file_pass_graph(file, 0, gmodel);
 
 fail:
     reader = gwy_tiff_image_reader_free(reader);
-    GWY_OBJECT_UNREF(meta);
+    g_clear_object(&meta);
     g_free(header_tag);
     if (names_colors) {
         g_array_free(names_colors->colors, TRUE);
@@ -747,12 +733,12 @@ fail:
     /* The strings are already eaten by meta. */
     g_free(recording);
 
-    return container;
+    return file;
 }
 
 static LSMHeaderTag*
-lsm_read_header_tag(const GwyTIFF *tiff,
-                    const GwyTIFFEntry *tag, GError **error)
+read_header_tag(const GwyTIFF *tiff,
+                const GwyTIFFEntry *tag, GError **error)
 {
     LSMHeaderTag *header_tag;
     const guchar *p;
@@ -816,9 +802,9 @@ lsm_read_header_tag(const GwyTIFF *tiff,
 }
 
 static LSMNamesColors *
-lsm_read_names_colors(const GwyTIFF *tiff,
-                      guint32 offset,
-                      G_GNUC_UNUSED GError **error)
+read_names_colors(const GwyTIFF *tiff,
+                  guint32 offset,
+                  G_GNUC_UNUSED GError **error)
 {
     LSMNamesColors *names_colors;
     const guchar *p;
@@ -852,7 +838,7 @@ lsm_read_names_colors(const GwyTIFF *tiff,
     p = tiff->data + offset + names_colors->offset_colors;
     for (i = 0; i < names_colors->numcolors; i++) {
         color = gwy_get_gint32_le(&p);
-        g_array_append_val (names_colors->colors, color);
+        g_array_append_val(names_colors->colors, color);
         gwy_debug("color [%d] = %d", i, color);
     }
     p = tiff->data + offset + names_colors->offset_names;
@@ -907,10 +893,10 @@ set_meta_double(const LSMEntry *entry, const gchar *name, const gchar *unit, Gwy
 }
 
 static LSMEntryRecording*
-lsm_read_recording(const GwyTIFF *tiff,
-                   GwyContainer *meta,
-                   guint32 offset,
-                   G_GNUC_UNUSED GError **error)
+read_recording(const GwyTIFF *tiff,
+               GwyContainer *meta,
+               guint32 offset,
+               G_GNUC_UNUSED GError **error)
 {
     LSMEntryRecording *recording;
     LSMEntry *entry;
@@ -927,7 +913,7 @@ lsm_read_recording(const GwyTIFF *tiff,
     while (entry->entry != LSM_SUBBLOCK_END) {
         if (entry)
             g_free(entry);
-        entry = lsm_read_entry(tiff, offset, error);
+        entry = read_entry(tiff, offset, error);
 
         gwy_debug("entry = 0x%x type=%d size=%d", entry->entry, entry->type, entry->size);
         if (entry->entry == LSM_RECORDING_ENTRY_NAME)
@@ -1067,9 +1053,9 @@ lsm_read_recording(const GwyTIFF *tiff,
 }
 
 static LSMEntry*
-lsm_read_entry(const GwyTIFF *tiff,
-               guint32 offset,
-               G_GNUC_UNUSED GError **error)
+read_entry(const GwyTIFF *tiff,
+           guint32 offset,
+           G_GNUC_UNUSED GError **error)
 {
     LSMEntry *entry;
     const guchar *p;
