@@ -1,6 +1,6 @@
 /*
- *  $Id: zemax.c 25670 2023-09-15 08:23:12Z yeti-dn $
- *  Copyright (C) 2013 David Necas (Yeti).
+ *  $Id: zemax.c 28805 2025-11-05 12:28:20Z yeti-dn $
+ *  Copyright (C) 2013-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -23,11 +23,9 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/stats.h>
-#include <app/gwymoduleutils-file.h>
-#include <app/data-browser.h>
+#include <gwy.h>
 #include "err.h"
 
 #define EXTENSION ".dat"
@@ -50,24 +48,24 @@ typedef struct {
     gdouble yoff;
 } ZemaxFile;
 
-static gboolean      module_register  (void);
-static gint          zemax_detect     (const GwyFileDetectInfo *fileinfo,
-                                       gboolean only_name);
-static GwyContainer* zemax_load       (const gchar *filename,
-                                       GwyRunType mode,
-                                       GError **error);
-static guint         zemax_read_header(const gchar *header,
-                                       guint len,
-                                       ZemaxFile *zmxfile,
-                                       GError **error);
-static gboolean      is_empty         (GwyDataField *field);
+static gboolean module_register(void);
+static gint     detect_file    (const GwyFileDetectInfo *fileinfo,
+                                gboolean only_name);
+static GwyFile* load_file      (const gchar *filename,
+                                GwyRunModeFlags mode,
+                                GError **error);
+static guint    read_header    (const gchar *header,
+                                guint len,
+                                ZemaxFile *zmxfile,
+                                GError **error);
+static gboolean is_empty       (GwyField *field);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     module_register,
     N_("Imports Zemax grid sag data files."),
     "Yeti <yeti@gwyddion.net>",
-    "1.1",
+    "2.0",
     "David Nečas (Yeti)",
     "2013",
 };
@@ -79,32 +77,29 @@ module_register(void)
 {
     gwy_file_func_register("zemax",
                            N_("Zemax grid sag data (.dat)"),
-                           (GwyFileDetectFunc)&zemax_detect,
-                           (GwyFileLoadFunc)&zemax_load,
-                           NULL,
-                           NULL);
+                           detect_file, load_file, NULL, NULL);
 
     return TRUE;
 }
 
 static gint
-zemax_detect(const GwyFileDetectInfo *fileinfo, gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     ZemaxFile zmxfile;
 
     if (only_name)
         return g_str_has_suffix(fileinfo->name_lowercase, EXTENSION) ? 20 : 0;
 
-    if (zemax_read_header(fileinfo->head, fileinfo->buffer_len, &zmxfile, NULL))
+    if (read_header(fileinfo->head, fileinfo->buffer_len, &zmxfile, NULL))
         return 60;
 
     return 0;
 }
 
-static GwyContainer*
-zemax_load(const gchar *filename,
-           G_GNUC_UNUSED GwyRunType mode,
-           GError **error)
+static GwyFile*
+load_file(const gchar *filename,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
+          GError **error)
 {
     /* The last field is special, it is the mask of invalid pixels. */
     static const gchar *titles[4] = {
@@ -115,8 +110,10 @@ zemax_load(const gchar *filename,
     };
 
     ZemaxFile zmxfile;
-    GwyContainer *container = NULL, *meta;
-    GwyDataField *fields[5];
+    GwyFile *file = NULL;
+    GwyContainer *meta;
+    GwyField *fields[5];
+    gdouble *field_data[5];
     gchar *p, *line, *end, *buffer = NULL;
     GError *err = NULL;
     gdouble q = 1.0;
@@ -131,7 +128,7 @@ zemax_load(const gchar *filename,
     }
 
     gwy_clear(fields, G_N_ELEMENTS(fields));
-    if (!(len = zemax_read_header(buffer, size, &zmxfile, error)))
+    if (!(len = read_header(buffer, size, &zmxfile, error)))
         goto fail;
 
     p = buffer + len;
@@ -144,10 +141,11 @@ zemax_load(const gchar *filename,
         q = 0.0254;
 
     for (i = 0; i < G_N_ELEMENTS(fields); i++) {
-        fields[i] = gwy_data_field_new(zmxfile.xres, zmxfile.yres,
-                                       zmxfile.xres*zmxfile.dx*q, zmxfile.yres*zmxfile.dy*q,
-                                       FALSE);
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(fields[i]), "m");
+        fields[i] = gwy_field_new(zmxfile.xres, zmxfile.yres,
+                                  zmxfile.xres*zmxfile.dx*q, zmxfile.yres*zmxfile.dy*q,
+                                  FALSE);
+        field_data[i] = gwy_field_get_data(fields[i]);
+        gwy_unit_set_from_string(gwy_field_get_unit_xy(fields[i]), "m");
     }
 
     for (k = 0; k < zmxfile.xres*zmxfile.yres; k++) {
@@ -162,43 +160,43 @@ zemax_load(const gchar *filename,
         }
 
         for (i = 0; i < G_N_ELEMENTS(fields); i++) {
-            fields[i]->data[k] = g_ascii_strtod(line, &end);
+            field_data[i][k] = g_ascii_strtod(line, &end);
             if (end == line) {
                 if (i < 4) {
                     g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
                                 _("Data line %u does not contain four items."), k+1);
                     goto fail;
                 }
-                fields[i]->data[k] = 0.0;
+                field_data[i][k] = 0.0;
             }
             line = end;
         }
-        if (fields[4]->data[k])
+        if (field_data[4][k])
             havemask = TRUE;
     }
 
-    gwy_data_field_multiply(fields[0], q);
-    gwy_data_field_multiply(fields[3], 1.0/q);
+    gwy_field_multiply(fields[0], q);
+    gwy_field_multiply(fields[3], 1.0/q);
 
     nchannels = G_N_ELEMENTS(titles);
     if (is_empty(fields[1]) && is_empty(fields[2]) && is_empty(fields[3]))
         nchannels = 1;
 
-    meta = gwy_container_new();
+    meta = gwy_container_new_in_construction();
     gwy_container_set_string_by_name(meta, "Decenter X", g_strdup_printf("%g m", q*zmxfile.xoff));
     gwy_container_set_string_by_name(meta, "Decenter Y", g_strdup_printf("%g m", q*zmxfile.yoff));
 
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     str = g_string_new(NULL);
     for (i = 0; i < nchannels; i++) {
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(fields[i]), units[i]);
-        gwy_container_set_object(container, gwy_app_get_data_key_for_id(i), fields[i]);
-        gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(i), titles[i]);
+        gwy_unit_set_from_string(gwy_field_get_unit_z(fields[i]), units[i]);
+        gwy_file_set_image(file, i, fields[i]);
+        gwy_file_set_title(file, GWY_FILE_IMAGE, i, titles[i], FALSE);
         if (havemask)
-            gwy_container_pass_object(container, gwy_app_get_mask_key_for_id(i), gwy_data_field_duplicate(fields[4]));
-        gwy_container_pass_object(container, gwy_app_get_data_meta_key_for_id(i), gwy_container_duplicate(meta));
+            gwy_file_pass_image_mask(file, i, gwy_field_copy(fields[4]));
+        gwy_file_pass_meta(file, GWY_FILE_IMAGE, i, gwy_container_copy(meta));
 
-        gwy_file_channel_import_log_add(container, i, NULL, filename);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, i, NULL, filename);
     }
     g_string_free(str, TRUE);
     g_object_unref(meta);
@@ -206,17 +204,17 @@ zemax_load(const gchar *filename,
 fail:
     g_free(buffer);
     for (i = 0; i < G_N_ELEMENTS(fields); i++)
-        GWY_OBJECT_UNREF(fields[i]);
+        g_clear_object(fields + i);
 
-    return container;
+    return file;
 }
 
 /* XXX: This is a weak detection.  We basically accept anything that has seven reasonable numbers on the first line. */
 static guint
-zemax_read_header(const gchar *header,
-                  guint len,
-                  ZemaxFile *zmxfile,
-                  GError **error)
+read_header(const gchar *header,
+            guint len,
+            ZemaxFile *zmxfile,
+            GError **error)
 {
     const gchar *p = header, *lineend;
     gchar *end, *line = NULL;
@@ -325,10 +323,10 @@ fail:
 }
 
 static gboolean
-is_empty(GwyDataField *field)
+is_empty(GwyField *field)
 {
     gdouble min, max;
-    gwy_data_field_get_min_max(field, &min, &max);
+    gwy_field_get_min_max(field, &min, &max);
     return min == 0.0 && max == 0.0;
 }
 
