@@ -1,5 +1,5 @@
 /*
- *  $Id: zyvex.c 27575 2025-03-20 17:42:36Z yeti-dn $
+ *  $Id: zyvex.c 28911 2025-11-24 18:27:42Z yeti-dn $
  *  ZyVector Scanz .zad file format import module
  *  Copyright (C) 2025 Zyvex Labs LLC.
  *  Author: Moutaz Haq.
@@ -16,30 +16,12 @@
  *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
  *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
-
-/**
- * [FILE-MAGIC-FREEDESKTOP]
- * <mime-type type="application/x-zyvector-scanz-spm">
- *   <comment>ZyVector Scanz SPM data</comment>
- *   <glob pattern="*.zad"/>
- *   <glob pattern="*.ZAD"/>
- * </mime-type>
- **/
-
-/**
- * [FILE-MAGIC-USERGUIDE]
- * ZyVector Scanz
- * .zad
- * Read
- **/
-
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
-#include <libgwymodule/gwymodule-file.h>
-#include <app/data-browser.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
 #include <libxml/parser.h>
 #include <libxml/tree.h>
 #include <libxml/xpath.h>
@@ -53,12 +35,12 @@
 #define MIN_ZAD_VERSION 5
 #define MAX_ZAD_VERSION 7
 
-static gboolean      module_register(void);
-static gint          detect_file    (const GwyFileDetectInfo *fileinfo,
-                                     gboolean only_name);
-static GwyContainer* load_file      (const gchar *filename,
-                                     GwyRunType mode,
-                                     GError **error);
+static gboolean module_register(void);
+static gint     detect_file    (const GwyFileDetectInfo *fileinfo,
+                                gboolean only_name);
+static GwyFile* load_file      (const gchar *filename,
+                                GwyRunModeFlags mode,
+                                GError **error);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
@@ -77,10 +59,7 @@ module_register(void)
 {
     gwy_file_func_register("zyvex",
                            N_("ZyVector Scanz files (.zad)"),
-                           (GwyFileDetectFunc)&detect_file,
-                           (GwyFileLoadFunc)&load_file,
-                           NULL,
-                           NULL);
+                           &detect_file, &load_file, NULL, NULL);
     return TRUE;
 }
 
@@ -196,8 +175,7 @@ err_DATA_TYPE_NAME(GError **error,
 }
 
 static gint
-detect_file(const GwyFileDetectInfo *fileinfo,
-            gboolean only_name)
+detect_file(const GwyFileDetectInfo *fileinfo, gboolean only_name)
 {
     const gchar *head, *scanz_data, *version;
     gint64 version_num;
@@ -536,12 +514,10 @@ find_elems_by_name(xmlXPathContextPtr xpathCtx,
 {
     xmlXPathObjectPtr xpathObj;
     xmlNodeSetPtr nodeset;
-    gsize loc_size;
-    gchar* loc;
+    gsize loc_size = 3 + strlen(name) + 1;
+    gchar loc[loc_size];
 
-    loc_size = 3 + strlen(name) + 1;
-    loc = g_alloca(loc_size);
-    memset(loc, 0, loc_size);
+    gwy_clear(loc, loc_size);
     g_strlcat(loc, ".//", loc_size);
     g_strlcat(loc, name, loc_size);
     xpathObj = xmlXPathNodeEval(node, (xmlChar*)loc, xpathCtx);
@@ -917,7 +893,7 @@ read_system_data(int version,
             err_MISSING_FIELD(error, "SystemData.PreampMaxCurr");
             goto fail;
         }
-        system_data->preamp_max_curr = pow10(10 - preamp_gain);
+        system_data->preamp_max_curr = gwy_exp10(10 - preamp_gain);
     }
 
     success = get_attr_float(node, "VernierZ", 0.0, &system_data->vernier_z, &missing);
@@ -937,15 +913,15 @@ fail:
     return NULL;
 }
 
-static GwyDataField*
-create_data_field(const SystemData* system_data,
-                  const ScanData* scan_data,
-                  const ScanBuf* scan_buf,
-                  GError **error)
+static GwyField*
+create_field(const SystemData* system_data,
+             const ScanData* scan_data,
+             const ScanBuf* scan_buf,
+             GError **error)
 {
     guchar *data;
-    GwyDataField *field;
-    gint z_pow10;
+    GwyField *field;
+    gint z_gwy_exp10;
     gdouble z_scale;
 
     z_scale = get_z_scale(system_data, scan_data, scan_buf);
@@ -954,21 +930,21 @@ create_data_field(const SystemData* system_data,
     if (!data)
         return NULL;
 
-    field = gwy_data_field_new(scan_buf->dims[0], scan_buf->dims[1],
-                               scan_data->scan_size[0] * Nanometer,
-                               scan_data->scan_size[1] * Nanometer,
-                               FALSE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(field), "m");
-    gwy_si_unit_set_from_string_parse(gwy_data_field_get_si_unit_z(field), scan_buf->units, &z_pow10);
-    z_scale *= pow10(z_pow10);  /* converts from prefixed to base SI units */
+    field = gwy_field_new(scan_buf->dims[0], scan_buf->dims[1],
+                          scan_data->scan_size[0] * Nanometer,
+                          scan_data->scan_size[1] * Nanometer,
+                          FALSE);
+    gwy_unit_set_from_string(gwy_field_get_unit_xy(field), "m");
+    z_gwy_exp10 = gwy_unit_set_from_string(gwy_field_get_unit_z(field), scan_buf->units);
+    z_scale *= gwy_exp10(z_gwy_exp10);  /* converts from prefixed to base SI units */
     gwy_convert_raw_data(data, scan_buf->dims[0] * scan_buf->dims[1], 1,
                          scan_buf->data_type, GWY_BYTE_ORDER_LITTLE_ENDIAN,
-                         gwy_data_field_get_data(field), z_scale, 0.0);
+                         gwy_field_get_data(field), z_scale, 0.0);
     g_free(data);
 
     if (scan_buf->valid_rows < scan_buf->dims[1]) {
         /* Set invalid rows to NAN */
-        gdouble *raw_data = gwy_data_field_get_data(field);
+        gdouble *raw_data = gwy_field_get_data(field);
         gint i = scan_buf->valid_rows * scan_buf->dims[0];
         gint max_i = scan_buf->dims[0] * scan_buf->dims[1];
         while (i < max_i) {
@@ -980,12 +956,12 @@ create_data_field(const SystemData* system_data,
     return field;
 }
 
-static GwyContainer*
+static GwyFile*
 load_file(const gchar *filename,
-          G_GNUC_UNUSED GwyRunType mode,
+          G_GNUC_UNUSED GwyRunModeFlags mode,
           GError **error)
 {
-    GwyContainer* container = NULL;
+    GwyFile* file = NULL;
     xmlDocPtr doc = NULL;
     xmlXPathContextPtr xpathCtx = NULL;
     xmlNodePtr root, system_data_node, scan_data_node;
@@ -1035,22 +1011,22 @@ load_file(const gchar *filename,
         goto finish;
 
     /* read_scan_data() already checked for err_NO_DATA so no need to check here. */
-    container = gwy_container_new();
+    file = gwy_file_new_in_construction();
     for (i = 0; i < scan_data_num_scan_bufs(scan_data); i++) {
         ScanBuf *scan_buf = scan_data_get_scan_buf(scan_data, i);
-        GwyDataField *field = create_data_field(system_data, scan_data, scan_buf, error);
-        GwyDataField *mask = NULL;
+        GwyField *field = create_field(system_data, scan_data, scan_buf, error);
+        GwyField *mask = NULL;
         if (!field) {
-            GWY_OBJECT_UNREF(container);
+            g_clear_object(&file);
             goto finish;
         }
         mask = gwy_app_channel_mask_of_nans(field, TRUE);
-        gwy_container_pass_object(container, gwy_app_get_data_key_for_id(i), field);
-        gwy_container_set_const_string(container, gwy_app_get_data_title_key_for_id(i), (const guchar*)scan_buf->name);
+        gwy_file_pass_image(file, i, field);
+        gwy_file_set_title(file, GWY_FILE_IMAGE, i, (const guchar*)scan_buf->name, FALSE);
         if (mask)
-            gwy_container_pass_object(container, gwy_app_get_mask_key_for_id(i), mask);
-        gwy_app_channel_check_nonsquare(container, i);
-        gwy_file_channel_import_log_add(container, i, NULL, filename);
+            gwy_file_pass_image_mask(file, i, mask);
+        gwy_check_nonsquare_image(file, i);
+        gwy_log_add_import(file, GWY_FILE_IMAGE, i, NULL, filename);
     }
 
 finish:
@@ -1060,7 +1036,7 @@ finish:
         xmlXPathFreeContext(xpathCtx);
     if (doc)
         xmlFreeDoc(doc);
-    return container;
+    return file;
 }
 
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
