@@ -1,41 +1,27 @@
 /*
- *  $Id: graph_align.c 25076 2022-10-11 13:48:10Z yeti-dn $
+ *  $Id: graph_align.c 28803 2025-11-05 11:58:19Z yeti-dn $
  *  Copyright (C) 2015-2017 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
+ *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
+ *  later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/correct.h>
-#include <libgwydgets/gwygraphmodel.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-graph.h>
-#include <app/gwyapp.h>
-
-typedef struct {
-    gdouble x;
-    gdouble y;
-} PointXY;
+#include <gwy.h>
 
 static gboolean module_register (void);
-static void     graph_align     (GwyGraph *graph);
+static void     module_main     (GwyGraph *graph);
 static void     align_two_curves(GwyGraphCurveModel *base,
                                  GwyGraphCurveModel *cmodel);
 static gdouble  find_best_offset(const gdouble *a,
@@ -44,7 +30,7 @@ static gdouble  find_best_offset(const gdouble *a,
                                  gint nb,
                                  gint off_from,
                                  gint off_to);
-static gdouble* regularise      (const PointXY *xydata,
+static gdouble* regularise      (const GwyXY *xydata,
                                  gint ndata,
                                  gdouble dx,
                                  gint *pn);
@@ -70,9 +56,9 @@ static gboolean
 module_register(void)
 {
     gwy_graph_func_register("graph_align",
-                            (GwyGraphFunc)&graph_align,
+                            &module_main,
                             N_("/_Correct Data/_Align"),
-                            GWY_STOCK_GRAPH_ALIGN,
+                            GWY_ICON_GRAPH_ALIGN,
                             GWY_MENU_FLAG_GRAPH_CURVE,
                             N_("Align curves"));
 
@@ -80,16 +66,18 @@ module_register(void)
 }
 
 static void
-graph_align(GwyGraph *graph)
+module_main(GwyGraph *graph)
 {
-    GwyContainer *data;
+    GwyFile *data;
     GwyGraphModel *gmodel;
     GwyGraphCurveModel *cmodel, *basecmodel = NULL;
     gint i, ncurves, ndata, ndatamax = 0;
     const gdouble *xdata;
     gdouble len, maxlen = 0.0;
+    gint id;
     GQuark quark;
 
+    gwy_data_browser_get_current(GWY_APP_GRAPH_MODEL_ID, &id, 0);
     gmodel = gwy_graph_get_model(graph);
     ncurves = gwy_graph_model_get_n_curves(gmodel);
     if (ncurves < 2) {
@@ -116,10 +104,11 @@ graph_align(GwyGraph *graph)
         return;
     }
 
-    gwy_app_data_browser_get_current(GWY_APP_CONTAINER, &data,
-                                     GWY_APP_GRAPH_MODEL_KEY, &quark,
-                                     0);
-    gwy_app_undo_qcheckpointv(data, 1, &quark);
+    gwy_data_browser_get_current(GWY_APP_CONTAINER, &data,
+                                 GWY_APP_GRAPH_MODEL_KEY, &quark,
+                                 0);
+    gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &quark);
+    gwy_log_add(data, GWY_FILE_GRAPH, id, id);
 
     for (i = 0; i < ncurves; i++) {
         cmodel = gwy_graph_model_get_curve(gmodel, i);
@@ -132,19 +121,19 @@ graph_align(GwyGraph *graph)
     }
 }
 
-static PointXY*
+static GwyXY*
 extract_xy_data(GwyGraphCurveModel *gcmodel)
 {
     guint i, ndata = gwy_graph_curve_model_get_ndata(gcmodel);
     const gdouble *xdata = gwy_graph_curve_model_get_xdata(gcmodel);
     const gdouble *ydata = gwy_graph_curve_model_get_ydata(gcmodel);
-    PointXY *pts = g_new(PointXY, ndata);
+    GwyXY *pts = g_new(GwyXY, ndata);
 
     for (i = 0; i < ndata; i++) {
         pts[i].x = xdata[i];
         pts[i].y = ydata[i];
     }
-    qsort(pts, ndata, sizeof(PointXY), gwy_compare_double);
+    qsort(pts, ndata, sizeof(GwyXY), gwy_compare_double);
 
     return pts;
 }
@@ -153,7 +142,7 @@ static void
 align_two_curves(GwyGraphCurveModel *base,
                  GwyGraphCurveModel *cmodel)
 {
-    PointXY *cxydata, *bxydata;
+    GwyXY *cxydata, *bxydata;
     gdouble *cline, *bline, *newcxdata, *newcydata;
     gint cndata, bndata, cn, bn, i, off_from, off_to;
     gdouble clen, dx, blen, off;
@@ -257,21 +246,18 @@ find_best_offset(const gdouble *a, gint na,
     }
 
     gwy_debug("best offset %d [%g %g %g]",
-              bestoff,
-              scores[0]/bestscore, scores[1]/bestscore, scores[2]/bestscore);
+              bestoff, scores[0]/bestscore, scores[1]/bestscore, scores[2]/bestscore);
 
     if (scores[0] > scores[1] && scores[2] > scores[1]) {
-        subpixoff = 0.5*(scores[0] - scores[2])/(scores[0] + scores[2]
-                                                 - 2.0*scores[1]);
+        subpixoff = 0.5*(scores[0] - scores[2])/(scores[0] + scores[2] - 2.0*scores[1]);
         gwy_debug("subpix %g", subpixoff);
     }
 
     return bestoff + subpixoff;
 }
 
-/* @boff is how much we move @b.  If @boff < 0 we move it to the left and
- * if @boff > 0 we move it to the right.  If @boff = 0 then @a and @b left
- * edges are aligned. */
+/* @boff is how much we move @b.  If @boff < 0 we move it to the left and if @boff > 0 we move it to the right.  If
+ * @boff = 0 then @a and @b left edges are aligned. */
 static gdouble
 difference_score(const gdouble *a, gint na,
                  const gdouble *b, gint nb,
@@ -304,17 +290,17 @@ difference_score(const gdouble *a, gint na,
 }
 
 static gdouble*
-regularise(const PointXY *xydata, gint ndata, gdouble dx, gint *pn)
+regularise(const GwyXY *xydata, gint ndata, gdouble dx, gint *pn)
 {
     gint n = floor((xydata[ndata-1].x - xydata[0].x)/dx) + 1;
     gint i, ic, nzero;
-    GwyDataLine *dline, *wline;
+    GwyLine *dline, *wline;
     gdouble *data, *weight;
 
-    dline = gwy_data_line_new(n, n, TRUE);
-    wline = gwy_data_line_new(n, n, TRUE);
-    data = gwy_data_line_get_data(dline);
-    weight = gwy_data_line_get_data(wline);
+    dline = gwy_line_new(n, n, TRUE);
+    wline = gwy_line_new(n, n, TRUE);
+    data = gwy_line_get_data(dline);
+    weight = gwy_line_get_data(wline);
     *pn = n;
 
     for (ic = 0; ic < ndata; ic++) {
@@ -346,7 +332,7 @@ regularise(const PointXY *xydata, gint ndata, gdouble dx, gint *pn)
     if (nzero) {
         for (i = 0; i < n; i++)
             weight[i] = (weight[i] <= 0.0);
-        gwy_data_line_correct_laplace(dline, wline);
+        gwy_line_correct_laplace(dline, wline);
     }
     data = g_memdup(data, n*sizeof(gdouble));
 
@@ -356,4 +342,4 @@ regularise(const PointXY *xydata, gint ndata, gdouble dx, gint *pn)
     return data;
 }
 
-/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
+/* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
