@@ -1,5 +1,5 @@
 /*
- *  $Id: graph_cut.c 25231 2023-02-22 12:49:25Z yeti-dn $
+ *  $Id: graph_cut.c 28645 2025-10-13 17:19:42Z yeti-dn $
  *  Copyright (C) 2007-2023 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,20 +16,11 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwydgets/gwygraph.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwydgets/gwyinventorystore.h>
-#include <libgwydgets/gwycombobox.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwydgets/gwydgetutils.h>
-#include <libgwymodule/gwymodule-graph.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
+#include <gwy.h>
 
 enum {
     PARAM_CURVE,
@@ -52,12 +43,12 @@ typedef struct {
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             cut                 (GwyGraph *graph);
+static void             module_main         (GwyGraph *graph);
 static GwyDialogOutcome run_gui             (ModuleArgs *args);
 static void             param_changed       (ModuleGUI *gui,
                                              gint id);
 static void             execute             (ModuleArgs *args,
-                                             GwyContainer *data);
+                                             GwyFile *data);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
@@ -75,9 +66,9 @@ static gboolean
 module_register(void)
 {
     gwy_graph_func_register("graph_cut",
-                            (GwyGraphFunc)&cut,
+                            &module_main,
                             N_("/_Basic Operations/_Cut..."),
-                            GWY_STOCK_GRAPH_CUT,
+                            GWY_ICON_GRAPH_CUT,
                             GWY_MENU_FLAG_GRAPH_CURVE,
                             N_("Extract part of graph into new one"));
 
@@ -104,14 +95,14 @@ define_module_params(void)
 }
 
 static void
-cut(GwyGraph *graph)
+module_main(GwyGraph *graph)
 {
-    GwyContainer *data;
+    GwyFile *data;
     GwyDialogOutcome outcome;
     ModuleArgs args;
 
-    gwy_app_data_browser_get_current(GWY_APP_CONTAINER, &data, 0);
-    gwy_clear(&args, 1);
+    gwy_data_browser_get_current(GWY_APP_CONTAINER, &data, 0);
+    gwy_clear1(args);
     args.params = gwy_params_new_from_settings(define_module_params());
     args.gmodel = gwy_graph_get_model(graph);
 
@@ -131,7 +122,7 @@ run_gui(ModuleArgs *args)
     GwyDialogOutcome outcome;
     ModuleGUI gui;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
     gui.gmodel = gwy_graph_model_new_alike(args->gmodel);
 
@@ -139,9 +130,9 @@ run_gui(ModuleArgs *args)
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    hbox = gwy_hbox_new(0);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
     gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
-    gwy_dialog_add_content(GWY_DIALOG(gui.dialog), hbox, FALSE, FALSE, 0);
+    gwy_dialog_add_content(dialog, hbox, FALSE, FALSE, 0);
 
     graph = gwy_graph_new(gui.gmodel);
     gtk_widget_set_size_request(graph, 480, 360);
@@ -166,7 +157,7 @@ run_gui(ModuleArgs *args)
 }
 
 static void
-execute(ModuleArgs *args, GwyContainer *data)
+execute(ModuleArgs *args, GwyFile *data)
 {
     GwyParams *params = args->params;
     gint i, k, ndata, cstart, cend;
@@ -204,7 +195,7 @@ execute(ModuleArgs *args, GwyContainer *data)
             }
         }
         if (newxdata->len) {
-            newgcmodel = gwy_graph_curve_model_duplicate(gcmodel);
+            newgcmodel = gwy_graph_curve_model_copy(gcmodel);
             gwy_graph_curve_model_set_data(newgcmodel,
                                            &g_array_index(newxdata, gdouble, 0), &g_array_index(newydata, gdouble, 0),
                                            newxdata->len);
@@ -214,8 +205,10 @@ execute(ModuleArgs *args, GwyContainer *data)
     }
 
     /* XXX: This may confuse the user if we do not add anything at all. */
-    if (gwy_graph_model_get_n_curves(gmodel))
-        gwy_app_data_browser_add_graph_model(gmodel, data, TRUE);
+    if (gwy_graph_model_get_n_curves(gmodel)) {
+        gint newid = gwy_file_add_graph(data, gmodel);
+        gwy_file_set_visible(data, GWY_FILE_GRAPH, newid, TRUE);
+    }
 
     g_object_unref(gmodel);
 }
