@@ -1,5 +1,5 @@
 /*
- *  $Id: graph_export_ascii.c 26506 2024-08-14 15:28:42Z yeti-dn $
+ *  $Id: graph_export_ascii.c 28734 2025-10-29 10:18:03Z yeti-dn $
  *  Copyright (C) 2006-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,14 +16,9 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwydgets/gwygraphmodel.h>
-#include <libgwydgets/gwycombobox.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-graph.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
+#include <gwy.h>
 
 enum {
     PARAM_STYLE,
@@ -46,7 +41,7 @@ typedef struct {
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             export              (GwyGraph *graph);
+static void             module_main         (GwyGraph *graph);
 static GwyDialogOutcome run_gui             (ModuleArgs *args);
 static void             param_changed       (ModuleGUI *gui,
                                              gint id);
@@ -69,9 +64,9 @@ static gboolean
 module_register(void)
 {
     gwy_graph_func_register("graph_export_ascii",
-                            (GwyGraphFunc)&export,
+                            &module_main,
                             N_("/_Export/_Text..."),
-                            GWY_STOCK_GRAPH_EXPORT_ASCII,
+                            GWY_ICON_GRAPH_EXPORT_ASCII,
                             GWY_MENU_FLAG_GRAPH_CURVE,
                             N_("Export graph data to a text file"));
 
@@ -106,12 +101,12 @@ define_module_params(void)
 }
 
 static void
-export(GwyGraph *graph)
+module_main(GwyGraph *graph)
 {
     GwyDialogOutcome outcome;
     ModuleArgs args;
 
-    gwy_clear(&args, 1);
+    gwy_clear1(args);
     args.params = gwy_params_new_from_settings(define_module_params());
     outcome = run_gui(&args);
     gwy_params_save_to_settings(args.params);
@@ -183,7 +178,7 @@ execute(GwyGraph *graph, ModuleArgs *args)
                                        gwy_params_get_boolean(params, PARAM_METADATA),
                                        style,
                                        NULL);
-    gwy_save_auxiliary_data(_("Export to Text File"), NULL, -1, str->str);
+    gwy_save_auxiliary_data(_("Export to Text File"), NULL, str->str, str->len, FALSE);
     g_string_free(str, TRUE);
 }
 
