@@ -1,5 +1,5 @@
 /*
- *  $Id: graph_measure_period.c 26524 2024-08-15 18:14:00Z yeti-dn $
+ *  $Id: graph_measure_period.c 29061 2026-01-02 15:01:53Z yeti-dn $
  *  Copyright (C) 2023 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,16 +16,11 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
 #include <string.h>
 #include <fftw3.h>
-#include <libgwyddion/gwymacros.h>
-#include <libprocess/gwyprocess.h>
-#include <libgwydgets/gwygraphmodel.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-graph.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 
 #define CNORM(x) (x[0]*x[0] + x[1]*x[1])
 
@@ -64,9 +59,10 @@ typedef struct {
 
 static gboolean     module_register     (void);
 static GwyParamDef* define_module_params(void);
-static void         graph_measure_period(GwyGraph *graph);
+static void         module_main         (GwyGraph *graph);
 static void         run_gui             (ModuleArgs *args,
-                                         GwyContainer *data);
+                                         GwyFile *data,
+                                         gint id);
 static gint         execute             (ModuleArgs *args,
                                          GwyResults *results);
 static void         param_changed       (ModuleGUI *gui,
@@ -74,10 +70,10 @@ static void         param_changed       (ModuleGUI *gui,
 static void         dialog_response     (ModuleGUI *gui,
                                          gint response);
 static void         preview             (gpointer user_data);
-static GwyResults*  create_results      (GwyContainer *data,
-                                         GwyGraphModel *gmodel,
-                                         GwySIUnit *xunit,
-                                         GwySIUnit *yunit);
+static GwyResults*  create_results      (GwyFile *data,
+                                         gint id,
+                                         GwyUnit *xunit,
+                                         GwyUnit *yunit);
 
 static const gchar* results_period[] = {
     "simple-fft", "zoom-fft",
@@ -101,9 +97,9 @@ static gboolean
 module_register(void)
 {
     gwy_graph_func_register("graph_measure_period",
-                            (GwyGraphFunc)&graph_measure_period,
+                            module_main,
                             N_("/Measure _Features/_Period..."),
-                            GWY_STOCK_GRAPH_PERIOD_MEASURE,
+                            GWY_ICON_GRAPH_PERIOD_MEASURE,
                             GWY_MENU_FLAG_GRAPH_CURVE,
                             N_("Measure grating period"));
 
@@ -132,42 +128,45 @@ define_module_params(void)
 }
 
 static void
-graph_measure_period(GwyGraph *graph)
+module_main(GwyGraph *graph)
 {
-    GwyContainer *data;
+    GwyFile *data;
     ModuleArgs args;
+    gint id;
 
-    gwy_app_data_browser_get_current(GWY_APP_CONTAINER, &data, 0);
-    gwy_clear(&args, 1);
+    gwy_data_browser_get_current(GWY_APP_CONTAINER, &data,
+                                 GWY_APP_GRAPH_MODEL_ID, &id,
+                                 0);
+    gwy_clear1(args);
     args.params = gwy_params_new_from_settings(define_module_params());
     args.gmodel = gwy_graph_get_model(graph);
 
-    run_gui(&args, data);
+    run_gui(&args, data, id);
     gwy_params_save_to_settings(args.params);
     g_object_unref(args.params);
-    GWY_OBJECT_UNREF(args.levelled);
+    g_clear_object(&args.levelled);
 }
 
 static void
-run_gui(ModuleArgs *args, GwyContainer *data)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     GtkWidget *hbox, *graph;
     GwyDialog *dialog;
     GwyParamTable *table;
-    GwySIUnit *xunit, *yunit;
+    GwyUnit *xunit, *yunit;
     ModuleGUI gui;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
     gui.gmodel = gwy_graph_model_new_alike(args->gmodel);
-    g_object_get(args->gmodel, "si-unit-x", &xunit, "si-unit-y", &yunit, NULL);
-    gui.results = create_results(data, args->gmodel, xunit, yunit);
+    g_object_get(args->gmodel, "unit-x", &xunit, "unit-y", &yunit, NULL);
+    gui.results = create_results(data, id, xunit, yunit);
 
     gui.dialog = gwy_dialog_new(_("Measure Period"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_CLEAR, GTK_RESPONSE_OK, 0);
 
-    hbox = gwy_hbox_new(0);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
     gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
     gwy_dialog_add_content(GWY_DIALOG(gui.dialog), hbox, FALSE, FALSE, 0);
 
@@ -228,8 +227,8 @@ dialog_response(ModuleGUI *gui, gint response)
 }
 
 static GwyResults*
-create_results(GwyContainer *data, GwyGraphModel *gmodel,
-               GwySIUnit *xunit, GwySIUnit *yunit)
+create_results(GwyFile *data, gint id,
+               GwyUnit *xunit, GwyUnit *yunit)
 {
     GwyResults *results;
 
@@ -258,7 +257,7 @@ create_results(GwyContainer *data, GwyGraphModel *gmodel,
     gwy_results_set_unit(results, "z", yunit);
 
     gwy_results_fill_filename(results, "file", data);
-    gwy_results_fill_graph(results, "graph", gmodel);
+    gwy_results_fill_data_name(results, "graph", data, GWY_FILE_GRAPH, id);
 
     return results;
 }
@@ -290,10 +289,10 @@ preview(gpointer user_data)
 /* An ad-hoc curve regularisation method. If the points are exactly regular, the net result is basically some
  * pointless interpolation (which includes points exactly correspoding to the curve points). Otherwise it just
  * does some kind of linear averaging/interpolation thing. */
-static GwyDataLine*
+static GwyLine*
 regularise_curve(GwyGraphCurveModel *gcmodel, gint pos, gint npts)
 {
-    GwyDataLine *line, *mask;
+    GwyLine *line, *mask;
     gint ndata, i, res = 4*(npts - 1) + 3;
     const gdouble *xdata, *ydata;
     gdouble *data, *mdata;
@@ -306,10 +305,10 @@ regularise_curve(GwyGraphCurveModel *gcmodel, gint pos, gint npts)
     xend = xdata[pos + npts-1];
     dx = (xend - xbegin)/(res - 3);
     xoff = xbegin - dx;
-    line = gwy_data_line_new(res, dx*res, TRUE);
-    mask = gwy_data_line_new_alike(line, TRUE);
-    data = gwy_data_line_get_data(line);
-    mdata = gwy_data_line_get_data(mask);
+    line = gwy_line_new(res, dx*res, TRUE);
+    mask = gwy_line_new_alike(line, TRUE);
+    data = gwy_line_get_data(line);
+    mdata = gwy_line_get_data(mask);
 
     for (i = MAX(pos-3, 0); i < MIN(pos + npts+3, ndata); i++) {
         gdouble x = xdata[i], y = ydata[i];
@@ -331,7 +330,7 @@ regularise_curve(GwyGraphCurveModel *gcmodel, gint pos, gint npts)
             mdata[i] = 1.0;
     }
 
-    gwy_data_line_correct_laplace(line, mask);
+    gwy_line_correct_laplace(line, mask);
     g_object_unref(mask);
 
     return line;
@@ -401,11 +400,11 @@ linear_fit_slope_with_error_estimate(gint n, const gdouble *xdata, const gdouble
 
 /* Estimate period in pixels. */
 static gdouble
-coarse_period_estimate(GwyDataLine *line, gdouble oversample)
+coarse_period_estimate(GwyLine *line, gdouble oversample)
 {
-    gint i, imax = 0, res = gwy_data_line_get_res(line);
+    gint i, imax = 0, res = gwy_line_get_res(line);
     gint extres = gwy_fft_find_nice_size((gint)ceil(oversample*res));
-    GwyDataLine *wline = gwy_data_line_new(res, res, TRUE);
+    GwyLine *wline = gwy_line_new(res, res, TRUE);
     fftw_complex *data = fftw_alloc_complex(extres);
     fftw_complex *f = fftw_alloc_complex(extres);
     gdouble *wdata;
@@ -414,10 +413,10 @@ coarse_period_estimate(GwyDataLine *line, gdouble oversample)
 
     plan = fftw_plan_dft_1d(extres, data, f, FFTW_FORWARD, FFTW_DESTROY_INPUT | FFTW_ESTIMATE);
 
-    gwy_data_line_copy(line, wline);
-    gwy_data_line_fft_window(wline, GWY_WINDOWING_HANN);
+    gwy_line_copy_data(line, wline);
+    gwy_line_fft_window(wline, GWY_WINDOWING_HANN);
     gwy_clear(data, extres);
-    wdata = gwy_data_line_get_data(wline);
+    wdata = gwy_line_get_data(wline);
     for (i = 0; i < res; i++)
         data[i][0] = wdata[i];
     g_object_unref(wline);
@@ -443,11 +442,11 @@ coarse_period_estimate(GwyDataLine *line, gdouble oversample)
 
 /* Estimate period in pixels. */
 static gdouble
-zoom_period_estimate(GwyDataLine *line, gdouble Tcoarse)
+zoom_period_estimate(GwyLine *line, gdouble Tcoarse)
 {
-    gint i, imax, res = gwy_data_line_get_res(line);
-    GwyDataLine *wline = gwy_data_line_duplicate(line);
-    GwyDataLine *rzfft = gwy_data_line_new_alike(line, FALSE), *izfft = gwy_data_line_new_alike(line, FALSE);
+    gint i, imax, res = gwy_line_get_res(line);
+    GwyLine *wline = gwy_line_copy(line);
+    GwyLine *rzfft = gwy_line_new_alike(line, FALSE), *izfft = gwy_line_new_alike(line, FALSE);
     gdouble *rdata, *idata;
     gdouble f0, f1, fm, x = 0.0;
 
@@ -455,18 +454,18 @@ zoom_period_estimate(GwyDataLine *line, gdouble Tcoarse)
 
     f0 = fmax(res/Tcoarse - 1.05, 0.0);
     f1 = res/Tcoarse + 1.05;
-    gwy_data_line_fft_window(wline, GWY_WINDOWING_HANN);
-    gwy_data_line_zoom_fft(wline, NULL, rzfft, izfft, res, f0, f1);
+    gwy_line_fft_window(wline, GWY_WINDOWING_HANN);
+    gwy_line_zoom_fft(wline, NULL, rzfft, izfft, res, f0, f1);
     g_object_unref(wline);
 
-    rdata = gwy_data_line_get_data(rzfft);
-    idata = gwy_data_line_get_data(izfft);
+    rdata = gwy_line_get_data(rzfft);
+    idata = gwy_line_get_data(izfft);
     imax = res/2;
     for (i = 0; i < res; i++)
         rdata[i] = rdata[i]*rdata[i] + idata[i]*idata[i];
     g_object_unref(izfft);
 
-    imax = GWY_ROUND(gwy_data_line_max_pos_i(rzfft));
+    imax = GWY_ROUND(gwy_line_max_pos_i(rzfft));
     if (imax > 0 && imax < res-1)
         gwy_math_refine_maximum_1d(rdata + imax-1, &x);
 
@@ -540,7 +539,7 @@ fit_slope_of_centres(GArray *centres, gdouble tolerance, gdouble *err_estim)
         xdata[i] = cd[i].l;
 
     p = 92.0;
-    gwy_math_percentiles(n, xdata, GWY_PERCENTILE_INTERPOLATION_HIGHER, 1, &p, &m);
+    gwy_math_percentiles(xdata, n, GWY_PERCENTILE_INTERPOLATION_HIGHER, &p, 1, &m);
     for (i = ngood = 0; i < n; i++) {
         if (cd[i].l >= 0.1*m && cd[i].l <= 1.4*m) {
             cd[ngood] = cd[i];
@@ -555,7 +554,7 @@ fit_slope_of_centres(GArray *centres, gdouble tolerance, gdouble *err_estim)
     /* Now we got rid of noise-created too short segments (possibly many) and maybe some too long ones (a few).  So
      * take percentile closer to median and filter again. */
     p = 50.0;
-    gwy_math_percentiles(n, xdata, GWY_PERCENTILE_INTERPOLATION_LINEAR, 1, &p, &m);
+    gwy_math_percentiles(xdata, n, GWY_PERCENTILE_INTERPOLATION_LINEAR, &p, 1, &m);
     for (i = ngood = 0; i < n; i++) {
         if (cd[i].l >= 0.7*m && cd[i].l <= 1.4*m) {
             cd[ngood] = cd[i];
@@ -577,7 +576,7 @@ fit_slope_of_centres(GArray *centres, gdouble tolerance, gdouble *err_estim)
      * XXX: Do not use plain mean because it is obviously unstable.  However, do not use median either because in
      * some odd cases the distances basically form two groups, differing by 1 pixel. So the median will be either
      * 1/2 pixel too large or too small, but in either case it will be bad. */
-    m = gwy_math_trimmed_mean(n, xdata, n/4, n/4);
+    m = gwy_math_trimmed_mean(xdata, n, n/4, n/4);
 
     /* Find three consecutive points which are approximately m apart.  We cannot trust the first centre to be good. */
     for (firstgood = 0; firstgood < n-3; firstgood++) {
@@ -625,7 +624,7 @@ fit_slope_of_centres(GArray *centres, gdouble tolerance, gdouble *err_estim)
         }
     }
 
-    //gwy_math_fit_polynom(ngood, idata, xdata, 1, coeffs);
+    //gwy_math_fit_poly(ngood, idata, xdata, 1, coeffs);
     T = linear_fit_slope_with_error_estimate(ngood, idata, xdata, err_estim);
     g_free(xdata);
 
@@ -633,11 +632,11 @@ fit_slope_of_centres(GArray *centres, gdouble tolerance, gdouble *err_estim)
 }
 
 static gdouble
-gravcentre_period_estimate(GwyDataLine *line, gdouble *err_estim)
+gravcentre_period_estimate(GwyLine *line, gdouble *err_estim)
 {
     const gdouble threshold = 0.0;
-    gint i, istart, iend, res = gwy_data_line_get_res(line);
-    const gdouble *d = gwy_data_line_get_data(line);
+    gint i, istart, iend, res = gwy_line_get_res(line);
+    const gdouble *d = gwy_line_get_data(line);
     GArray *xcabove, *xcbelow;
     gdouble Tabove, Tbelow, err_above, err_below;
     GravityCentre gc;
@@ -727,7 +726,7 @@ fit_slope_of_crossings(GArray *crossings, gdouble tolerance, gdouble *err_estim)
      * XXX: Do not use plain mean because it is obviously unstable.  However, do not use median either because in
      * some odd cases the distances basically form two groups, differing by 1 pixel. So the median will be either
      * 1/2 pixel too large or too small, but in either case it will be bad. */
-    m = gwy_math_trimmed_mean(n, xdata, n/4, n/4);
+    m = gwy_math_trimmed_mean(xdata, n, n/4, n/4);
 
     /* Find three consecutive points which are approximately m apart.  We cannot trust the first centre to be good. */
     for (firstgood = 0; firstgood < n-3; firstgood++) {
@@ -782,11 +781,11 @@ fit_slope_of_crossings(GArray *crossings, gdouble tolerance, gdouble *err_estim)
 }
 
 static gdouble
-zerocross_period_estimate(GwyDataLine *line, gdouble *err_estim)
+zerocross_period_estimate(GwyLine *line, gdouble *err_estim)
 {
     GArray *zcdown, *zcup, *zxdata;
     gint i, istart, iend, res = line->res;
-    const gdouble *d = line->data;
+    const gdouble *d = gwy_line_get_data_const(line);
     gdouble threshold = 0.0;
     gdouble Tup, Tdown, err_up, err_down;
 
@@ -827,8 +826,8 @@ zerocross_period_estimate(GwyDataLine *line, gdouble *err_estim)
             g_array_index(zxdata, gdouble, ii-istart) = d[ii];
             g_array_index(zxdata, gdouble, seglen + ii-istart) = ii - i;
         }
-        gwy_math_fit_polynom(seglen, &g_array_index(zxdata, gdouble, 0), &g_array_index(zxdata, gdouble, seglen),
-                             1, coeffs);
+        gwy_math_fit_poly(&g_array_index(zxdata, gdouble, 0), &g_array_index(zxdata, gdouble, seglen), seglen,
+                          1, coeffs);
         x = i + coeffs[0];
         if (d[i] > 0.0)
             g_array_append_val(zcdown, x);
@@ -884,33 +883,33 @@ find_first_acf_peak(const gdouble *d, gint res)
 }
 
 static gdouble
-acf_period_estimate(GwyDataLine *line)
+acf_period_estimate(GwyLine *line)
 {
-    GwyDataLine *acf = gwy_data_line_new_alike(line, FALSE);
+    GwyLine *acf = gwy_line_new_alike(line, FALSE);
     gdouble x = 0.0;
     gint peakpos;
 
-    gwy_data_line_acf(line, acf);
-    peakpos = find_first_acf_peak(acf->data, acf->res);
+    gwy_line_acf(line, acf);
+    const gdouble *d = gwy_line_get_data_const(acf);
+    peakpos = find_first_acf_peak(d, acf->res);
     if (!(peakpos > 0))
         return -1.0;
-    gwy_math_refine_maximum_1d(acf->data + peakpos-1, &x);
+    gwy_math_refine_maximum_1d(d + peakpos-1, &x);
     g_object_unref(acf);
 
     return peakpos + x;
 }
 
 static gdouble
-multiacf_period_estimate(GwyDataLine *line, gdouble *err_estim)
+multiacf_period_estimate(GwyLine *line, gdouble *err_estim)
 {
-    GwyDataLine *acf = gwy_data_line_new_alike(line, FALSE);
+    GwyLine *acf = gwy_line_new_alike(line, FALSE);
     GArray *xdata, *ydata;
     gdouble x = 0.0, sxx, sxy, f, m0, t;
-    gdouble *d;
     gint i, j, res, peakpos;
 
-    gwy_data_line_acf(line, acf);
-    d = acf->data;
+    gwy_line_acf(line, acf);
+    const gdouble *d = gwy_line_get_data_const(acf);
     res = acf->res;
     peakpos = find_first_acf_peak(d, res);
     if (!(peakpos > 0))
@@ -977,18 +976,18 @@ multiacf_period_estimate(GwyDataLine *line, gdouble *err_estim)
 }
 
 static void
-remove_envelope(GwyDataLine *line)
+remove_envelope(GwyLine *line)
 {
-    GwyDataField *background;
-    GwyDataLine *tmp;
+    GwyField *background;
+    GwyLine *tmp;
     gdouble *bg, *d;
     gdouble a, b, T;
     gint m, i, res;
 
     /* If the profile is way off zero line, weird things may happen. Get it approximately right first. */
-    gwy_data_line_get_line_coeffs(line, &a, &b);
-    gwy_data_line_line_level(line, a, b);
-    res = gwy_data_line_get_res(line);
+    gwy_line_fit_line(line, &a, &b);
+    gwy_line_line_level(line, a, b);
+    res = gwy_line_get_res(line);
 
     /* Then use the envelope method. */
     T = coarse_period_estimate(line, 2.0);
@@ -999,20 +998,20 @@ remove_envelope(GwyDataLine *line)
     if (m >= res-1)
         return;
 
-    background = gwy_data_field_new(res, 2, res, 2, FALSE);
-    tmp = gwy_data_line_new_alike(line, FALSE);
+    background = gwy_field_new(res, 2, res, 2, FALSE);
+    tmp = gwy_line_new_alike(line, FALSE);
 
-    gwy_data_line_copy(line, tmp);
-    gwy_data_line_part_filter_kth_rank(tmp, m, 0, res, GWY_ROUND(0.05*m));
-    gwy_data_field_set_row(background, tmp, 0);
+    gwy_line_copy_data(line, tmp);
+    gwy_line_part_filter_kth_rank(tmp, m, 0, res, GWY_ROUND(0.05*m));
+    gwy_field_set_row(background, tmp, 0);
 
-    gwy_data_line_copy(line, tmp);
-    gwy_data_line_part_filter_kth_rank(tmp, m, 0, res, GWY_ROUND(0.95*m));
-    gwy_data_field_set_row(background, tmp, 1);
+    gwy_line_copy_data(line, tmp);
+    gwy_line_part_filter_kth_rank(tmp, m, 0, res, GWY_ROUND(0.95*m));
+    gwy_field_set_row(background, tmp, 1);
 
-    gwy_data_field_row_gaussian(background, 0.5*T);
-    bg = gwy_data_field_get_data(background);
-    d = gwy_data_line_get_data(line);
+    gwy_field_row_gaussian(background, 0.5*T);
+    bg = gwy_field_get_data(background);
+    d = gwy_line_get_data(line);
 
     /* Remove the envelope. */
     for (i = 0; i < res; i++)
@@ -1023,33 +1022,33 @@ remove_envelope(GwyDataLine *line)
 }
 
 static void
-fix_zero_line(GwyDataLine *line)
+fix_zero_line(GwyLine *line)
 {
     /* For some gratings this is not very good, but it is a one-liner and the method based on peaks in height
      * distribution has its own problems. Do not use plain median. If the fill ratio is relatively far from 1/2,
      * mean becomes just somewhat off, whereas median can easily move to the top or bottom level. */
-    gwy_data_line_add(line, -gwy_data_line_get_avg(line));
+    gwy_line_add(line, -gwy_line_avg(line));
 }
 
 /* The way we do levelling makes a bit difficult to show it for the original curve. Try to reconstruct it at least
  * in spirit by roughly sampling it back. */
 static GwyGraphCurveModel*
-rough_levelled_graph_curve(GwyGraphCurveModel *origmodel, GwyDataLine *dline, gdouble from)
+rough_levelled_graph_curve(GwyGraphCurveModel *origmodel, GwyLine *dline, gdouble from)
 {
     GwyGraphCurveModel *levelledcmodel;
-    GwyDataLine *resampled;
+    GwyLine *resampled;
     gint res, ndata;
 
-    res = gwy_data_line_get_res(dline);
+    res = gwy_line_get_res(dline);
     ndata = res/4;
-    resampled = gwy_data_line_new_resampled(dline, ndata, GWY_INTERPOLATION_KEY);
-    gwy_data_line_set_offset(resampled, from);
+    resampled = gwy_line_new_resampled(dline, ndata, GWY_INTERPOLATION_KEY);
+    gwy_line_set_offset(resampled, from);
     levelledcmodel = gwy_graph_curve_model_new_alike(origmodel);
     g_object_set(levelledcmodel,
-                 "line-style", GDK_LINE_ON_OFF_DASH,
+                 "line-style", GWY_GRAPH_LINE_DASH,
                  "description", _("Leveled profile"),
                  NULL);
-    gwy_graph_curve_model_set_data_from_dataline(levelledcmodel, resampled, 0, 0);
+    gwy_graph_curve_model_set_data_from_line(levelledcmodel, resampled, 0, 0);
     g_object_unref(resampled);
 
     return levelledcmodel;
@@ -1086,7 +1085,7 @@ execute(ModuleArgs *args, GwyResults *results)
     gboolean level = gwy_params_get_boolean(params, PARAM_LEVEL);
     gboolean show_levelled = gwy_params_get_boolean(params, PARAM_SHOW_LEVELED);
     GwyGraphCurveModel *gcmodel = gwy_graph_model_get_curve(args->gmodel, curve);
-    GwyDataLine *line;
+    GwyLine *line;
     const gdouble *xdata;
     guint ndata, pos, npts;
     gdouble xoff, dx, Tfft, Tzoomfft, Tacf, Tmultiacf, Tgc, Tzc, err_gc, err_zc, err_multiacf = -1 /* shut up clang */;
@@ -1115,7 +1114,7 @@ execute(ModuleArgs *args, GwyResults *results)
         fix_zero_line(line);
     }
 
-    dx = gwy_data_line_get_dx(line);
+    dx = gwy_line_get_dx(line);
     Tfft = coarse_period_estimate(line, 2.0);
     Tzoomfft = Tfft > 0.0 ? zoom_period_estimate(line, Tfft) : -1.0;
     Tacf = acf_period_estimate(line);
@@ -1123,7 +1122,7 @@ execute(ModuleArgs *args, GwyResults *results)
     Tgc = gravcentre_period_estimate(line, &err_gc);
     Tzc = zerocross_period_estimate(line, &err_zc);
 
-    GWY_OBJECT_UNREF(args->levelled);
+    g_clear_object(&args->levelled);
     if (level && show_levelled)
         args->levelled = rough_levelled_graph_curve(gcmodel, line, xoff);
     g_object_unref(line);
