@@ -1,5 +1,5 @@
 /*
- *  $Id: graph_peaks.c 24444 2021-11-01 10:14:22Z yeti-dn $
+ *  $Id: graph_peaks.c 28613 2025-09-23 17:17:53Z yeti-dn $
  *  Copyright (C) 2016-2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,19 +16,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
 #include <string.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/peaks.h>
-#include <libgwydgets/gwygraphmodel.h>
-#include <libgwydgets/gwycombobox.h>
-#include <libgwydgets/gwynullstore.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwydgets/gwydgetutils.h>
-#include <libgwymodule/gwymodule-graph.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 
 enum {
     NPEAKQUANT = GWY_PEAK_WIDTH + 1
@@ -71,14 +62,14 @@ typedef struct {
     GwyParamTable *table_peaks;
     GArray *peaks;
     GArray *peaks_sorted;
-    GwySIValueFormat *vf[NPEAKQUANT];
+    GwyValueFormat *vf[NPEAKQUANT];
     gboolean in_init;
     gboolean peaks_valid;
 } ModuleGUI;
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             graph_peaks         (GwyGraph *graph);
+static void             module_main         (GwyGraph *graph);
 static GwyDialogOutcome run_gui             (ModuleArgs *args);
 static void             param_changed       (ModuleGUI *gui,
                                              gint id);
@@ -115,9 +106,9 @@ static gboolean
 module_register(void)
 {
     gwy_graph_func_register("graph_peaks",
-                            (GwyGraphFunc)&graph_peaks,
+                            module_main,
                             N_("/Measure _Features/Find _Peaks..."),
-                            GWY_STOCK_FIND_PEAKS,
+                            GWY_ICON_FIND_PEAKS,
                             GWY_MENU_FLAG_GRAPH_CURVE,
                             N_("Find graph curve peaks"));
 
@@ -155,7 +146,7 @@ define_module_params(void)
 }
 
 static void
-graph_peaks(GwyGraph *graph)
+module_main(GwyGraph *graph)
 {
     ModuleArgs args;
 
@@ -177,7 +168,7 @@ run_gui(ModuleArgs *args)
     ModuleGUI gui;
     guint i;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
     gui.peaks = g_array_new(FALSE, FALSE, sizeof(Peak));
     gui.peaks_sorted = g_array_new(FALSE, FALSE, sizeof(Peak));
@@ -191,7 +182,7 @@ run_gui(ModuleArgs *args)
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GTK_RESPONSE_OK, 0);
 
-    hbox = gwy_hbox_new(0);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
     gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
     gwy_dialog_add_content(dialog, hbox, FALSE, FALSE, 0);
 
@@ -204,7 +195,7 @@ run_gui(ModuleArgs *args)
     gwy_graph_area_set_selection_editable(area, FALSE);
     gui.selection = gwy_graph_area_get_selection(area, GWY_GRAPH_STATUS_XLINES);
 
-    vbox = gwy_vbox_new(0);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
     gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, TRUE, 0);
 
     table = gui.table = gwy_param_table_new(args->params);
@@ -225,7 +216,7 @@ run_gui(ModuleArgs *args)
     gwy_param_table_append_report(table, PARAM_REPORT_STYLE);
     gwy_param_table_report_set_formatter(table, PARAM_REPORT_STYLE, format_report, &gui, NULL);
     /* XXX: Silly.  Just want to right-align the export controls for consistency. */
-    hbox2 = gwy_hbox_new(0);
+    hbox2 = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
     gtk_box_pack_start(GTK_BOX(vbox), hbox2, FALSE, FALSE, 0);
     gtk_box_pack_end(GTK_BOX(hbox2), gwy_param_table_widget(table), FALSE, FALSE, 0);
     gwy_dialog_add_param_table(GWY_DIALOG(gui.dialog), table);
@@ -240,7 +231,7 @@ run_gui(ModuleArgs *args)
     g_array_free(gui.peaks_sorted, TRUE);
     g_array_free(gui.peaks, TRUE);
     for (i = 0; i < NPEAKQUANT; i++)
-        GWY_SI_VALUE_FORMAT_FREE(gui.vf[i]);
+        GWY_FREE_VALUE_FORMAT(gui.vf[i]);
 
     return outcome;
 }
@@ -374,7 +365,7 @@ update_value_formats(ModuleGUI *gui)
 {
     gint curve = gwy_params_get_int(gui->args->params, PARAM_CURVE);
     GwyGraphCurveModel *gcmodel = gwy_graph_model_get_curve(gui->args->gmodel, curve);
-    GwySIUnit *xunit, *yunit, *areaunit;
+    GwyUnit *xunit, *yunit, *areaunit;
     GtkTreeView *treeview;
     GtkTreeViewColumn *column;
     GtkLabel *label;
@@ -384,15 +375,15 @@ update_value_formats(ModuleGUI *gui)
     guint i;
 
     g_object_get(gui->gmodel,
-                 "si-unit-x", &xunit,
-                 "si-unit-y", &yunit,
+                 "unit-x", &xunit,
+                 "unit-y", &yunit,
                  NULL);
-    areaunit = gwy_si_unit_multiply(xunit, yunit, NULL);
+    areaunit = gwy_unit_multiply(xunit, yunit, NULL);
 
     gwy_graph_curve_model_get_x_range(gcmodel, &min, &max);
     xrange = max - min;
-    gui->vf[GWY_PEAK_ABSCISSA] = gwy_si_unit_get_format_with_digits(xunit, GWY_SI_UNIT_FORMAT_MARKUP, xrange, 4,
-                                                                    gui->vf[GWY_PEAK_ABSCISSA]);
+    gui->vf[GWY_PEAK_ABSCISSA] = gwy_unit_get_format_with_digits(xunit, GWY_UNIT_FORMAT_MARKUP, xrange, 4,
+                                                                 gui->vf[GWY_PEAK_ABSCISSA]);
 
     max = get_peak_max(peaks, GWY_PEAK_HEIGHT);
     if (!(max > 0.0)) {
@@ -400,22 +391,22 @@ update_value_formats(ModuleGUI *gui)
         max = 0.4*(max - min);
     }
     yrange = max;
-    gui->vf[GWY_PEAK_HEIGHT] = gwy_si_unit_get_format_with_digits(yunit, GWY_SI_UNIT_FORMAT_MARKUP, yrange, 4,
-                                                                  gui->vf[GWY_PEAK_HEIGHT]);
+    gui->vf[GWY_PEAK_HEIGHT] = gwy_unit_get_format_with_digits(yunit, GWY_UNIT_FORMAT_MARKUP, yrange, 4,
+                                                               gui->vf[GWY_PEAK_HEIGHT]);
 
     max = 0.0;
     max = get_peak_max(peaks, GWY_PEAK_AREA);
     if (!(max > 0.0))
         max = 0.1*xrange*yrange;
-    gui->vf[GWY_PEAK_AREA] = gwy_si_unit_get_format_with_digits(areaunit, GWY_SI_UNIT_FORMAT_MARKUP, 0.5*max, 4,
-                                                                gui->vf[GWY_PEAK_AREA]);
+    gui->vf[GWY_PEAK_AREA] = gwy_unit_get_format_with_digits(areaunit, GWY_UNIT_FORMAT_MARKUP, 0.5*max, 4,
+                                                             gui->vf[GWY_PEAK_AREA]);
 
     max = 0.0;
     max = get_peak_max(peaks, GWY_PEAK_WIDTH);
     if (!(max > 0.0))
         max = 0.05*xrange;
-    gui->vf[GWY_PEAK_WIDTH] = gwy_si_unit_get_format_with_digits(xunit, GWY_SI_UNIT_FORMAT_MARKUP, max, 3,
-                                                                 gui->vf[GWY_PEAK_WIDTH]);
+    gui->vf[GWY_PEAK_WIDTH] = gwy_unit_get_format_with_digits(xunit, GWY_UNIT_FORMAT_MARKUP, max, 3,
+                                                              gui->vf[GWY_PEAK_WIDTH]);
 
     g_object_unref(areaunit);
     g_object_unref(yunit);
@@ -504,7 +495,7 @@ format_report(gpointer user_data)
     for (i = 0; i < peaks->len; i++) {
         const Peak *peak = &g_array_index(peaks, Peak, i);
         for (j = 0; j < NCOLUMNS; j++) {
-            GwySIValueFormat *vf = gui->vf[quantities[j]];
+            GwyValueFormat *vf = gui->vf[quantities[j]];
             g_string_append_printf(text, "%.*f%c",
                                    vf->precision, peak->v[quantities[j]]/vf->magnitude, j == NCOLUMNS-1 ? '\n' : '\t');
         }
