@@ -1,5 +1,5 @@
 /*
- *  $Id: graph_sfuncs.c 24431 2021-10-27 12:52:11Z yeti-dn $
+ *  $Id: graph_sfuncs.c 29023 2025-12-19 13:06:16Z yeti-dn $
  *  Copyright (C) 2019-2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,15 +16,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
 #include <string.h>
-#include <libgwyddion/gwymacros.h>
-#include <libprocess/gwyprocess.h>
-#include <libgwydgets/gwygraphmodel.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-graph.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 
 enum {
     PARAM_CURVE,
@@ -62,17 +57,17 @@ typedef struct {
 
 static gboolean         module_register              (void);
 static GwyParamDef*     define_module_params         (void);
-static void             graph_sfuncs                 (GwyGraph *graph);
+static void             module_main                  (GwyGraph *graph);
 static GwyDialogOutcome run_gui                      (ModuleArgs *args);
 static void             execute                      (ModuleArgs *args);
 static void             param_changed                (ModuleGUI *gui,
                                                       gint id);
 static void             preview                      (gpointer user_data);
 static void             calculate_stats              (GwyGraphCurveModel *gcmodel,
-                                                      GwySIUnit *xunit,
-                                                      GwySIUnit *yunit,
+                                                      GwyUnit *xunit,
+                                                      GwyUnit *yunit,
                                                       GwyParams *params,
-                                                      GwyDataLine *dline);
+                                                      GwyLine *dline);
 static gboolean         sfunction_has_native_sampling(GwySFOutputType type);
 
 static const GwyEnum sf_types[] =  {
@@ -101,7 +96,7 @@ static gboolean
 module_register(void)
 {
     gwy_graph_func_register("graph_sfuncs",
-                            (GwyGraphFunc)&graph_sfuncs,
+                            module_main,
                             N_("/_Statistics/Statistical _Functions..."),
                             NULL,
                             GWY_MENU_FLAG_GRAPH_CURVE,
@@ -133,14 +128,14 @@ define_module_params(void)
 }
 
 static void
-graph_sfuncs(GwyGraph *graph)
+module_main(GwyGraph *graph)
 {
     GwyDialogOutcome outcome;
     GwyAppDataId target_graph_id;
-    GwyContainer *data;
+    GwyFile *data;
     ModuleArgs args;
 
-    gwy_clear(&args, 1);
+    gwy_clear1(args);
     args.params = gwy_params_new_from_settings(define_module_params());
     args.gmodel = gwy_graph_get_model(graph);
     args.result = gwy_graph_model_new();
@@ -153,7 +148,7 @@ graph_sfuncs(GwyGraph *graph)
         execute(&args);
 
     target_graph_id = gwy_params_get_data_id(args.params, PARAM_TARGET_GRAPH);
-    gwy_app_data_browser_get_current(GWY_APP_CONTAINER, &data, 0);
+    gwy_data_browser_get_current(GWY_APP_CONTAINER, &data, 0);
     gwy_app_add_graph_or_curves(args.result, data, &target_graph_id, 1);
 
 end:
@@ -173,7 +168,7 @@ run_gui(ModuleArgs *args)
     /* This is to get the target graph filter right. */
     execute(args);
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
     g_object_set(args->result, "label-visible", FALSE, NULL);
 
@@ -182,7 +177,7 @@ run_gui(ModuleArgs *args)
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
     gwy_dialog_have_result(dialog);
 
-    hbox = gwy_hbox_new(0);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
     gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
     gwy_dialog_add_content(dialog, hbox, FALSE, FALSE, 0);
 
@@ -274,14 +269,14 @@ execute(ModuleArgs *args)
     gint ifrom = (all_curves ? 0 : curve);
     gint ito = (all_curves ? gwy_graph_model_get_n_curves(gmodel) : curve+1);
     GwyGraphCurveModel *gcmodel;
-    GwySIUnit *xunit, *yunit;
+    GwyUnit *xunit, *yunit;
     const gchar *title;
-    GwyDataLine *dline;
+    GwyLine *dline;
     gint i;
     gchar *s;
 
-    dline = gwy_data_line_new(1, 1.0, FALSE);
-    g_object_get(gmodel, "si-unit-x", &xunit, "si-unit-y", &yunit, NULL);
+    dline = gwy_line_new(1, 1.0, FALSE);
+    g_object_get(gmodel, "unit-x", &xunit, "unit-y", &yunit, NULL);
     gwy_graph_model_remove_all_curves(result);
     title = _(gwy_enum_to_string(output_type, sf_types, G_N_ELEMENTS(sf_types)));
     g_object_set(result,
@@ -294,7 +289,7 @@ execute(ModuleArgs *args)
         gcmodel = gwy_graph_model_get_curve(gmodel, i);
         calculate_stats(gcmodel, xunit, yunit, params, dline);
         gcmodel = gwy_graph_curve_model_new();
-        gwy_graph_curve_model_set_data_from_dataline(gcmodel, dline, 0, 0);
+        gwy_graph_curve_model_set_data_from_line(gcmodel, dline, 0, 0);
         g_object_set(gcmodel, "mode", GWY_GRAPH_CURVE_LINE, NULL);
         if (all_curves) {
             s = g_strdup_printf("%s %d", title, i+1);
@@ -310,26 +305,26 @@ execute(ModuleArgs *args)
         g_object_unref(gcmodel);
     }
 
-    gwy_graph_model_set_units_from_data_line(result, dline);
+    gwy_graph_model_set_units_from_line(result, dline);
     g_object_unref(dline);
     g_object_unref(xunit);
     g_object_unref(yunit);
 }
 
 static void
-oversample_curve(const gdouble *xdata, const gdouble *ydata, guint ndata, GwyDataLine *oversampled)
+oversample_curve(const gdouble *xdata, const gdouble *ydata, guint ndata, GwyLine *oversampled)
 {
     guint i, j, nover;
     gdouble xfrom, xto;
     gdouble *d;
 
     if (ndata == 1)
-        gwy_data_line_fill(oversampled, ydata[0]);
+        gwy_line_fill(oversampled, ydata[0]);
 
     xfrom = xdata[0];
     xto = xdata[ndata-1];
-    nover = gwy_data_line_get_res(oversampled);
-    d = gwy_data_line_get_data(oversampled);
+    nover = gwy_line_get_res(oversampled);
+    d = gwy_line_get_data(oversampled);
     for (i = j = 0; i < nover; i++) {
         gdouble t, x = i/(nover - 1.0)*(xto - xfrom) + xfrom;
 
@@ -351,16 +346,16 @@ oversample_curve(const gdouble *xdata, const gdouble *ydata, guint ndata, GwyDat
 
 static void
 calculate_stats(GwyGraphCurveModel *gcmodel,
-                GwySIUnit *xunit, GwySIUnit *yunit,
+                GwyUnit *xunit, GwyUnit *yunit,
                 GwyParams *params,
-                GwyDataLine *dline)
+                GwyLine *dline)
 {
     GwySFOutputType output_type = gwy_params_get_enum(params, PARAM_OUTPUT_TYPE);
     gdouble oversample = gwy_params_get_double(params, PARAM_OVERSAMPLE);
     gboolean fixres = gwy_params_get_boolean(params, PARAM_FIXRES);
     gint resolution = gwy_params_get_int(params, PARAM_RESOLUTION);
     GwyWindowingType window = gwy_params_get_enum(params, PARAM_WINDOW);
-    GwyDataLine *oversampled;
+    GwyLine *oversampled;
     const gdouble *xdata, *ydata;
     guint ndata, i, j, nover;
     gdouble xfrom, xto;
@@ -384,9 +379,9 @@ calculate_stats(GwyGraphCurveModel *gcmodel,
         }
     }
 
-    oversampled = gwy_data_line_new(nover, xto - xfrom, FALSE);
-    gwy_si_unit_assign(gwy_data_line_get_si_unit_x(oversampled), xunit);
-    gwy_si_unit_assign(gwy_data_line_get_si_unit_y(oversampled), yunit);
+    oversampled = gwy_line_new(nover, xto - xfrom, FALSE);
+    gwy_unit_assign(gwy_line_get_unit_x(oversampled), xunit);
+    gwy_unit_assign(gwy_line_get_unit_y(oversampled), yunit);
 
     /* Oversample derivatives, not values for DA and CDA. */
     if (output_type == GWY_SF_DA || output_type == GWY_SF_CDA) {
@@ -406,33 +401,30 @@ calculate_stats(GwyGraphCurveModel *gcmodel,
         }
         ydata = diffdata;
 
-        gwy_si_unit_divide(yunit, xunit, gwy_data_line_get_si_unit_y(oversampled));
+        gwy_unit_divide(yunit, xunit, gwy_line_get_unit_y(oversampled));
     }
 
     oversample_curve(xdata, ydata, ndata, oversampled);
 
     if (output_type == GWY_SF_DH || output_type == GWY_SF_DA) {
-        gwy_data_line_distribution(oversampled, dline, 0.0, 0.0, TRUE, fixres ? resolution : -1);
+        gwy_line_distribution(oversampled, dline, 0.0, 0.0, TRUE, fixres ? resolution : -1);
     }
     else if (output_type == GWY_SF_CDH || output_type == GWY_SF_CDA) {
-        gwy_data_line_distribution(oversampled, dline, 0.0, 0.0, TRUE, fixres ? resolution : -1);
-        gwy_data_line_cumulate(dline);
-        i = gwy_data_line_get_res(dline);
-        gwy_data_line_multiply(dline, 1.0/gwy_data_line_get_val(dline, i-1));
-        gwy_si_unit_set_from_string(gwy_data_line_get_si_unit_y(dline), NULL);
+        gwy_line_distribution(oversampled, dline, 0.0, 0.0, TRUE, fixres ? resolution : -1);
+        gwy_line_cumulate(dline, TRUE);
     }
     else if (output_type == GWY_SF_ACF) {
-        gwy_data_line_add(oversampled, -gwy_data_line_get_avg(oversampled));
-        gwy_data_line_acf(oversampled, dline);
+        gwy_line_add(oversampled, -gwy_line_avg(oversampled));
+        gwy_line_acf(oversampled, dline);
     }
     else if (output_type == GWY_SF_HHCF) {
-        gwy_data_line_add(oversampled, -gwy_data_line_get_avg(oversampled));
-        gwy_data_line_hhcf(oversampled, dline);
+        gwy_line_add(oversampled, -gwy_line_avg(oversampled));
+        gwy_line_hhcf(oversampled, dline);
     }
     else if (output_type == GWY_SF_PSDF) {
-        gwy_data_line_add(oversampled, -gwy_data_line_get_avg(oversampled));
+        gwy_line_add(oversampled, -gwy_line_avg(oversampled));
         /* Interpolation is ignored. */
-        gwy_data_line_psdf(oversampled, dline, window, GWY_INTERPOLATION_LINEAR);
+        gwy_line_psdf(oversampled, dline, window);
     }
     else {
         g_assert_not_reached();
@@ -445,7 +437,7 @@ calculate_stats(GwyGraphCurveModel *gcmodel,
 static gboolean
 sfunction_has_native_sampling(GwySFOutputType type)
 {
-    return (type == GWY_SF_ACF || type == GWY_SF_HHCF || type == GWY_SF_PSDF);
+    return type == GWY_SF_ACF || type == GWY_SF_HHCF || type == GWY_SF_PSDF;
 }
 
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
