@@ -1,5 +1,5 @@
 /*
- *  $Id: graph_simplemech.c 26510 2024-08-15 17:12:23Z yeti-dn $
+ *  $Id: graph_simplemech.c 28783 2025-11-04 17:09:52Z yeti-dn $
  *  Copyright (C) 2019-2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,16 +16,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
 #include <string.h>
-#include <libgwyddion/gwymacros.h>
-#include <libprocess/gwyprocess.h>
-#include <libgwydgets/gwygraphmodel.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-graph.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
-
+#include <gwy.h>
 
 enum {
     PARAM_CURVE_APPROACH,
@@ -52,23 +46,20 @@ typedef struct {
     GwyParamTable *table;
 } ModuleGUI;
 
-static gboolean         module_register              (void);
-static GwyParamDef*     define_module_params         (void);
-static void             graph_simplemech             (GwyGraph *graph);
-static GwyDialogOutcome run_gui                      (ModuleArgs *args,
-                                                      GwyContainer *data);
-static void             execute                      (ModuleArgs *args,
-                                                      GwyResults *results);
-static void             param_changed                (ModuleGUI *gui,
-                                                      gint id);
-static GwyResults*      create_results               (GwySIUnit *xunit,
-                                                      GwySIUnit *yunit);
-
-static void             preview                      (gpointer user_data);
-
-
-static const gchar* fitresults[] = { "modulus", "adhesion", "deformation", "dissipation", "baseline", "peak",};
+static gboolean         module_register     (void);
+static GwyParamDef*     define_module_params(void);
+static void             module_main         (GwyGraph *graph);
+static GwyDialogOutcome run_gui             (ModuleArgs *args,
+                                             GwyContainer *data);
+static void             execute             (ModuleArgs *args,
+                                             GwyResults *results);
+static void             param_changed       (ModuleGUI *gui,
+                                             gint id);
+static GwyResults*      create_results      (GwyUnit *xunit,
+                                             GwyUnit *yunit);
+static void             preview             (gpointer user_data);
 
+static const gchar* fitresults[] = { "modulus", "adhesion", "deformation", "dissipation", "baseline", "peak", };
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
@@ -86,7 +77,7 @@ static gboolean
 module_register(void)
 {
     gwy_graph_func_register("graph_simplemech",
-                            (GwyGraphFunc)&graph_simplemech,
+                            module_main,
                             N_("/_Force Distance/_Nanomechanical Fit..."),
                             NULL,
                             GWY_MENU_FLAG_GRAPH_CURVE,
@@ -122,19 +113,19 @@ define_module_params(void)
 }
 
 static void
-graph_simplemech(G_GNUC_UNUSED GwyGraph *graph)
+module_main(G_GNUC_UNUSED GwyGraph *graph)
 {
     GwyDialogOutcome outcome;
     GwyContainer *data;
     ModuleArgs args;
     GwyAppDataId graph_id;
 
-    gwy_app_data_browser_get_current(GWY_APP_CONTAINER, &data, 0);
-    gwy_app_data_browser_get_current(GWY_APP_CONTAINER_ID, &graph_id.datano,
-                                     GWY_APP_GRAPH_MODEL_ID, &graph_id.id,
-                                     0);
+    gwy_data_browser_get_current(GWY_APP_CONTAINER, &data, 0);
+    gwy_data_browser_get_current(GWY_APP_CONTAINER_ID, &graph_id.datano,
+                                 GWY_APP_GRAPH_MODEL_ID, &graph_id.id,
+                                 0);
 
-    gwy_clear(&args, 1);
+    gwy_clear1(args);
     args.params = gwy_params_new_from_settings(define_module_params());
     args.result = gwy_graph_model_new();
 
@@ -161,18 +152,19 @@ run_gui(ModuleArgs *args, G_GNUC_UNUSED GwyContainer *data)
     GtkWidget *hbox, *graph;
     GwyDialog *dialog;
     GwyParamTable *table;
-    GwySIUnit *xunit, *yunit;
+    GwyUnit *xunit, *yunit;
     ModuleGUI gui;
 
     /* This is to get the target graph filter right. */
     execute(args, NULL);
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
     g_object_set(args->result, "label-visible", FALSE, NULL);
 
     g_object_get(gwy_params_get_graph(args->params, PARAM_GRAPH_APPROACH),
-                           "si-unit-x", &xunit, "si-unit-y", &yunit, NULL);
+                 "unit-x", &xunit, "unit-y", &yunit,
+                 NULL);
     gui.results = create_results(xunit, yunit);
 
     gui.dialog = gwy_dialog_new(_("Nanomechanical Fit"));
@@ -180,7 +172,7 @@ run_gui(ModuleArgs *args, G_GNUC_UNUSED GwyContainer *data)
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
     gwy_dialog_have_result(dialog);
 
-    hbox = gwy_hbox_new(0);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
     gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
     gwy_dialog_add_content(GWY_DIALOG(gui.dialog), hbox, FALSE, FALSE, 0);
 
@@ -188,17 +180,17 @@ run_gui(ModuleArgs *args, G_GNUC_UNUSED GwyContainer *data)
     gtk_widget_set_size_request(graph, 480, 300);
     gtk_box_pack_end(GTK_BOX(hbox), graph, TRUE, TRUE, 0);
     gwy_graph_enable_user_input(GWY_GRAPH(graph), FALSE);
-    g_object_set(args->result, "si-unit-x", xunit, "si-unit-y", yunit, NULL);
+    g_object_set(args->result, "unit-x", xunit, "unit-y", yunit, NULL);
 
     table = gui.table = gwy_param_table_new(args->params);
 
     gwy_param_table_append_graph_id(table, PARAM_GRAPH_APPROACH);
-    gwy_param_table_append_graph_curve(table, PARAM_CURVE_APPROACH, gwy_params_get_graph(args->params,
-                                                                                PARAM_GRAPH_APPROACH));
+    gwy_param_table_append_graph_curve(table, PARAM_CURVE_APPROACH,
+                                       gwy_params_get_graph(args->params, PARAM_GRAPH_APPROACH));
 
     gwy_param_table_append_graph_id(table, PARAM_GRAPH_RETRACT);
-    gwy_param_table_append_graph_curve(table, PARAM_CURVE_RETRACT, gwy_params_get_graph(args->params,
-                                                                                PARAM_GRAPH_RETRACT));
+    gwy_param_table_append_graph_curve(table, PARAM_CURVE_RETRACT,
+                                       gwy_params_get_graph(args->params, PARAM_GRAPH_RETRACT));
 
     gwy_param_table_append_slider(table, PARAM_BASELINE_RANGE);
     gwy_param_table_slider_set_factor(table, PARAM_BASELINE_RANGE, 100.0);
@@ -235,13 +227,11 @@ run_gui(ModuleArgs *args, G_GNUC_UNUSED GwyContainer *data)
 
     g_object_set(args->result, "label-visible", TRUE, NULL);
 
-
 end:
     g_object_unref(gui.results);
     g_object_unref(xunit);
     g_object_unref(yunit);
 
-
     return outcome;
 }
 
@@ -253,21 +243,21 @@ param_changed(ModuleGUI *gui, gint id)
     gchar *label_bottom, *label_left;
 
     if (id == PARAM_GRAPH_APPROACH || id == PARAM_GRAPH_RETRACT) {
-
         gwy_param_table_graph_curve_set_model(table, PARAM_CURVE_APPROACH,
                                               gwy_params_get_graph(params, PARAM_GRAPH_APPROACH));
         gwy_param_table_graph_curve_set_model(table, PARAM_CURVE_RETRACT,
                                               gwy_params_get_graph(params, PARAM_GRAPH_RETRACT));
 
         g_object_get(gwy_params_get_graph(gui->args->params, PARAM_GRAPH_APPROACH),
-                           "axis-label-bottom", &label_bottom,
-                           "axis-label-left", &label_left, NULL);
-        g_object_set(gui->args->result, "axis-label-bottom", label_bottom,
-                     "axis-label-left", label_left, NULL);
-
+                     "axis-label-bottom", &label_bottom,
+                     "axis-label-left", &label_left,
+                     NULL);
+        g_object_set(gui->args->result,
+                     "axis-label-bottom", label_bottom,
+                     "axis-label-left", label_left,
+                     NULL);
     }
     gwy_dialog_invalidate(GWY_DIALOG(gui->dialog));
-
 }
 
 static void
@@ -278,16 +268,14 @@ preview(gpointer user_data)
     execute(gui->args, gui->results);
     gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
     gwy_param_table_results_fill(gui->table, WIDGET_RESULTS);
-
 }
 
 //param[0]: F_ad, param[1]: E (modulus), param[2]: R, param[3]: xshift, param[4] nu
 static gdouble
 func_dmt(gdouble x,
-                 G_GNUC_UNUSED gint n_param,
-                 const gdouble *param,
-                 G_GNUC_UNUSED gpointer user_data,
-                 gboolean *fres)
+         const gdouble *param,
+         G_GNUC_UNUSED gpointer user_data,
+         gboolean *fres)
 {
     /*xc, F_ad, R, E, nu*/
     double xr = fabs(param[0] - x); //this should be deformation and should be already entered only positive
@@ -331,10 +319,9 @@ execute(ModuleArgs *args, GwyResults *results)
 
     gwy_graph_model_remove_all_curves(result);
     gcmodel_approach = gwy_graph_model_get_curve(gwy_params_get_graph(params, PARAM_GRAPH_APPROACH),
-                                                       gwy_params_get_int(params, PARAM_CURVE_APPROACH));
+                                                 gwy_params_get_int(params, PARAM_CURVE_APPROACH));
     gcmodel_retract = gwy_graph_model_get_curve(gwy_params_get_graph(params, PARAM_GRAPH_RETRACT),
-                                                       gwy_params_get_int(params, PARAM_CURVE_RETRACT));
-
+                                                gwy_params_get_int(params, PARAM_CURVE_RETRACT));
 
     gwy_graph_model_add_curve(result, gcmodel_approach);
     gwy_graph_model_add_curve(result, gcmodel_retract);
@@ -359,37 +346,34 @@ execute(ModuleArgs *args, GwyResults *results)
 
     //get peak force, approach curve range and force integral in approach direction
     adis = 0;
-    for (i=0; i<nadata; i++) {
-        if (peak < yadata[i])
-        {
+    for (i = 0; i < nadata; i++) {
+        if (peak < yadata[i]) {
             peak = yadata[i];
             xpeak = xadata[i];
         }
-        if (xadata[i]<afrom) afrom = xadata[i];
-        if (xadata[i]>ato) ato = xadata[i];
+        afrom = fmin(afrom, xadata[i]);
+        ato = fmax(ato, xadata[i]);
 
-        if (i<(nadata-1)) adis += fabs(xadata[i]-xadata[i+1])*(yadata[i]+yadata[i+1])/2.0;
+        if (i < nadata-1)
+            adis += fabs(xadata[i] - xadata[i+1])*(yadata[i] + yadata[i+1])/2.0;
     }
 
     //fit baseline - average value on approach curve flat part
     baseline = 0;
     nbaseline = 0;
-    for (i=0; i<nadata; i++) {
-        if (xadata[i]>(ato - baseline_range*(ato-afrom)))
-        {
+    for (i = 0; i < nadata; i++) {
+        if (xadata[i] > ato - baseline_range*(ato-afrom)) {
             baseline += yadata[i];
             nbaseline++;
         }
-     }
+    }
     baseline /= nbaseline;
 
     //find zero force - point where we reach baseline force on approach curve when going from peak value
     xzero = xadata[nadata-1]; //safe defaults
     yzero = yadata[nadata-1];
-    for (i=(nadata-1); i>1; i--)
-    {
-        if (yadata[i]>=baseline && yadata[i+1]<baseline)
-        {
+    for (i = nadata-1; i > 1; i--) {
+        if (yadata[i] >= baseline && yadata[i+1] < baseline) {
             xzero = xadata[i];
             yzero = yadata[i];
         }
@@ -405,24 +389,24 @@ execute(ModuleArgs *args, GwyResults *results)
     rfrom = G_MAXDOUBLE;
     rto = -G_MAXDOUBLE;
     rdis = 0;
-    for (i=0; i<nrdata; i++) {
+    for (i = 0; i < nrdata; i++) {
         //printf("yrdata %d %g %g\n", i, xrdata[i], yrdata[i]);
-        if (adhesion > yrdata[i])
-        {
+        if (adhesion > yrdata[i]) {
             adhesion = yrdata[i];
             xadhesion = xrdata[i];
             iadhesion = i;
         }
-        if (peak < yrdata[i])
-        {
+        if (peak < yrdata[i]) {
             peak = yrdata[i];
             xpeak = xrdata[i];
         }
-        if (xrdata[i]<rfrom) rfrom = xrdata[i];
-        if (xrdata[i]>rto) rto = xrdata[i];
-
-        if (i<nrdata-1) rdis += fabs(xrdata[i]-xrdata[i+1])*(yrdata[i]+yrdata[i+1])/2.0;
+        if (xrdata[i] < rfrom)
+            rfrom = xrdata[i];
+        if (xrdata[i] > rto)
+            rto = xrdata[i];
 
+        if (i < nrdata - 1)
+            rdis += fabs(xrdata[i] - xrdata[i + 1]) * (yrdata[i] + yrdata[i + 1])/2.0;
     }
 
     //find dmt fit limits as points where we reach the points on retract curve when going from peak value
@@ -433,15 +417,12 @@ execute(ModuleArgs *args, GwyResults *results)
     upperval = adhesion + fit_upper*(peak-adhesion);
     lowerval = adhesion + fit_lower*(peak-adhesion);
 
-    for (i=(nrdata-1); i>1; i--)
-    {
-        if (yrdata[i]>=upperval && yrdata[i+1]<upperval)
-        {
+    for (i = nrdata-1; i > 1; i--) {
+        if (yrdata[i] >= upperval && yrdata[i+1] < upperval) {
             xupper = xrdata[i];
             iupper = i;
         }
-        if (yrdata[i]>=lowerval && yrdata[i+1]<lowerval)
-        {
+        if (yrdata[i] >= lowerval && yrdata[i+1] < lowerval) {
             xlower = xrdata[i];
             ilower = i;
         }
@@ -452,10 +433,10 @@ execute(ModuleArgs *args, GwyResults *results)
     fit_done = FALSE;
     nfitdata = 0;
     modulus = 5e7;
-    if ((ilower-iupper)>4) //random criterion
-    {
+    //random criterion
+    if (ilower - iupper > 4) {
        //fit dmt
-       fitter = gwy_math_nlfit_new((GwyNLFitFunc)func_dmt, gwy_math_nlfit_diff);
+       fitter = gwy_math_nlfit_new(func_dmt, NULL);
 
        param[0] = xadhesion; //adhesion comes from direct measurement
        fix[0] = 0;
@@ -472,9 +453,8 @@ execute(ModuleArgs *args, GwyResults *results)
        param[4] = nu;
        fix[4] = 1;
 
-
        if (gwy_math_nlfit_fit_full(fitter, ilower-iupper, xrdata + iupper, yrdata + iupper, NULL,
-                                                                   5, param, fix, NULL, NULL) >=0) {
+                                   5, param, fix, NULL, NULL) >= 0) {
           //printf("fit ok: params %g %g %g %g\n", param[0], param[1], param[2], param[3]);
 
           modulus = param[3];
@@ -485,7 +465,7 @@ execute(ModuleArgs *args, GwyResults *results)
           nxdata = gwy_math_linspace(NULL, nfitdata, xupper, (xlower - xupper)/nfitdata);
           nydata = g_new(gdouble, nfitdata);
           for (i = 0; i < nfitdata; i++)
-              nydata[i] = func_dmt(nxdata[i], 4, param, NULL, &fres);
+              nydata[i] = func_dmt(nxdata[i], param, NULL, &fres);
 
           fit_done = TRUE;
        }
@@ -505,12 +485,11 @@ execute(ModuleArgs *args, GwyResults *results)
     xp[2] = xzero;
     yp[2] = yzero;
 
-    xb[0] = ato - baseline_range*(ato-afrom);
+    xb[0] = ato - baseline_range*(ato - afrom);
     yb[0] = baseline;
     xb[1] = ato;
     yb[1] = baseline;
 
-
     gwy_graph_curve_model_set_data(gcmodel_points, xp, yp, 3);
     g_object_set(gcmodel_points, "mode", GWY_GRAPH_CURVE_POINTS, NULL);
     g_object_set(gcmodel_points, "description", g_strdup(_("pick points")), NULL);
@@ -525,16 +504,15 @@ execute(ModuleArgs *args, GwyResults *results)
                  NULL);
     gwy_graph_model_add_curve(result, gcmodel_baseline);
 
-    if (nfitdata>0)
-    {
-       gwy_graph_curve_model_set_data(gcmodel_fit, nxdata, nydata, nfitdata);
-       g_object_set(gcmodel_fit,
-                    "mode", GWY_GRAPH_CURVE_LINE,
-                    "color", gwy_graph_get_preset_color(3),
-                    "description", _("DMT fit"),
-                    "line-width", 3,
-                    NULL);
-       gwy_graph_model_add_curve(result, gcmodel_fit);
+    if (nfitdata > 0) {
+        gwy_graph_curve_model_set_data(gcmodel_fit, nxdata, nydata, nfitdata);
+        g_object_set(gcmodel_fit,
+                     "mode", GWY_GRAPH_CURVE_LINE,
+                     "color", gwy_graph_get_preset_color(3),
+                     "description", _("DMT fit"),
+                     "line-width", 3,
+                     NULL);
+        gwy_graph_model_add_curve(result, gcmodel_fit);
     }
 
     if (results) {
@@ -546,20 +524,18 @@ execute(ModuleArgs *args, GwyResults *results)
                                 "dissipation", (adis-rdis)/1.602176634e-19,
                                 NULL);
 
-        if (fit_done) gwy_results_fill_values(results, "modulus", modulus/1e6, NULL);
-
+        if (fit_done)
+            gwy_results_fill_values(results, "modulus", modulus/1e6, NULL);
     }
 
-    if (nfitdata > 0)
-    {
-       g_free(nxdata);
-       g_free(nydata);
+    if (nfitdata > 0) {
+        g_free(nxdata);
+        g_free(nydata);
     }
-
- }
+}
 
 static GwyResults*
-create_results(GwySIUnit *xunit, GwySIUnit *yunit)
+create_results(GwyUnit *xunit, GwyUnit *yunit)
 {
     GwyResults *results;
 
