@@ -1,5 +1,5 @@
 /*
- *  $Id: graph_stats.c 25231 2023-02-22 12:49:25Z yeti-dn $
+ *  $Id: graph_stats.c 29023 2025-12-19 13:06:16Z yeti-dn $
  *  Copyright (C) 2016-2023 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,15 +16,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
 #include <string.h>
-#include <libgwyddion/gwymacros.h>
-#include <libprocess/gwyprocess.h>
-#include <libgwydgets/gwygraphmodel.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-graph.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 
 enum {
     PARAM_CURVE,
@@ -51,18 +46,19 @@ typedef struct {
 
 static gboolean     module_register     (void);
 static GwyParamDef* define_module_params(void);
-static void         graph_stats         (GwyGraph *graph);
+static void         module_main         (GwyGraph *graph);
 static void         run_gui             (ModuleArgs *args,
-                                         GwyContainer *data);
+                                         GwyFile *data,
+                                         gint id);
 static gint         execute             (ModuleArgs *args,
                                          GwyResults *results);
 static void         param_changed       (ModuleGUI *gui,
                                          gint id);
 static void         preview             (gpointer user_data);
-static GwyResults*  create_results      (GwyContainer *data,
-                                         GwyGraphModel *gmodel,
-                                         GwySIUnit *xunit,
-                                         GwySIUnit *yunit);
+static GwyResults*  create_results      (GwyFile *data,
+                                         gint id,
+                                         GwyUnit *xunit,
+                                         GwyUnit *yunit);
 
 
 static const gchar* results_simple[] = { "min", "max", "avg", "median", "ra", "rms", "skew", "kurtosis", };
@@ -86,9 +82,9 @@ static gboolean
 module_register(void)
 {
     gwy_graph_func_register("graph_stats",
-                            (GwyGraphFunc)&graph_stats,
+                            module_main,
                             N_("/_Statistics/_Statistical Quantities..."),
-                            GWY_STOCK_GRAPH_STATISTICS,
+                            GWY_ICON_GRAPH_STATISTICS,
                             GWY_MENU_FLAG_GRAPH_CURVE,
                             N_("Calculate graph curve statistics"));
 
@@ -116,40 +112,43 @@ define_module_params(void)
 }
 
 static void
-graph_stats(GwyGraph *graph)
+module_main(GwyGraph *graph)
 {
-    GwyContainer *data;
+    GwyFile *data;
     ModuleArgs args;
+    gint id;
 
-    gwy_app_data_browser_get_current(GWY_APP_CONTAINER, &data, 0);
-    gwy_clear(&args, 1);
+    gwy_data_browser_get_current(GWY_APP_CONTAINER, &data,
+                                 GWY_APP_GRAPH_MODEL_ID, &id,
+                                 0);
+    gwy_clear1(args);
     args.params = gwy_params_new_from_settings(define_module_params());
     args.gmodel = gwy_graph_get_model(graph);
-    run_gui(&args, data);
+    run_gui(&args, data, id);
     gwy_params_save_to_settings(args.params);
     g_object_unref(args.params);
 }
 
 static void
-run_gui(ModuleArgs *args, GwyContainer *data)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     GtkWidget *hbox, *graph;
     GwyDialog *dialog;
     GwyParamTable *table;
-    GwySIUnit *xunit, *yunit;
+    GwyUnit *xunit, *yunit;
     ModuleGUI gui;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
     gui.gmodel = gwy_graph_model_new_alike(args->gmodel);
-    g_object_get(args->gmodel, "si-unit-x", &xunit, "si-unit-y", &yunit, NULL);
-    gui.results = create_results(data, args->gmodel, xunit, yunit);
+    g_object_get(args->gmodel, "unit-x", &xunit, "unit-y", &yunit, NULL);
+    gui.results = create_results(data, id, xunit, yunit);
 
     gui.dialog = gwy_dialog_new(_("Statistical Quantities"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GTK_RESPONSE_OK, 0);
 
-    hbox = gwy_hbox_new(0);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
     gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
     gwy_dialog_add_content(GWY_DIALOG(gui.dialog), hbox, FALSE, FALSE, 0);
 
@@ -202,8 +201,8 @@ param_changed(ModuleGUI *gui, gint id)
 }
 
 static GwyResults*
-create_results(GwyContainer *data, GwyGraphModel *gmodel,
-               GwySIUnit *xunit, GwySIUnit *yunit)
+create_results(GwyFile *data, gint id,
+               GwyUnit *xunit, GwyUnit *yunit)
 {
     GwyResults *results;
 
@@ -254,7 +253,7 @@ create_results(GwyContainer *data, GwyGraphModel *gmodel,
     gwy_results_set_unit(results, "z", yunit);
 
     gwy_results_fill_filename(results, "file", data);
-    gwy_results_fill_graph(results, "graph", gmodel);
+    gwy_results_fill_data_name(results, "graph", data, GWY_FILE_GRAPH, id);
 
     return results;
 }
@@ -282,11 +281,11 @@ execute(ModuleArgs *args, GwyResults *results)
     gdouble to = gwy_params_get_double(params, PARAM_RANGE_TO);
     gint curve = gwy_params_get_int(params, PARAM_CURVE);
     GwyGraphCurveModel *gcmodel = gwy_graph_model_get_curve(args->gmodel, curve);
-    GwyDataLine *dline;
+    GwyLine *dline;
     const gdouble *xdata, *ydata;
     guint ndata, i, pos, npts;
     gdouble min, max, projlen, length, variation, integralp, integraln, integral, integral2;
-    GwySIUnit *xunit, *yunit;
+    GwyUnit *xunit, *yunit;
 
     xdata = gwy_graph_curve_model_get_xdata(gcmodel);
     ydata = gwy_graph_curve_model_get_ydata(gcmodel);
@@ -307,21 +306,21 @@ execute(ModuleArgs *args, GwyResults *results)
     gwy_results_fill_values(results, "npts", npts, NULL);
     gwy_results_fill_format(results, "range", "from", from, "to", to, NULL);
 
-    /* Calculate simple quantities only depending on the value distribution using DataLine methods. */
-    dline = gwy_data_line_new(npts, 1.0, FALSE);
-    gwy_assign(gwy_data_line_get_data(dline), ydata + pos, npts);
-    gwy_data_line_get_min_max(dline, &min, &max);
+    /* Calculate simple quantities only depending on the value distribution using Line methods. */
+    dline = gwy_line_new(npts, 1.0, FALSE);
+    gwy_assign(gwy_line_get_data(dline), ydata + pos, npts);
+    gwy_line_min_max(dline, &min, &max);
     gwy_results_fill_values(results,
                             "min", min, "max", max,
-                            "avg", gwy_data_line_get_avg(dline),
-                            "median", gwy_data_line_get_median(dline),
+                            "avg", gwy_line_avg(dline),
+                            "median", gwy_line_median(dline),
                             NULL);
     if (npts > 1) {
         gwy_results_fill_values(results,
-                                "rms", gwy_data_line_get_rms(dline),
-                                "ra", gwy_data_line_get_ra(dline),
-                                "skew", gwy_data_line_get_skew(dline),
-                                "kurtosis", gwy_data_line_get_kurtosis(dline),
+                                "rms", gwy_line_rms(dline),
+                                "ra", gwy_line_ra(dline),
+                                "skew", gwy_line_skew(dline),
+                                "kurtosis", gwy_line_kurtosis(dline),
                                 NULL);
     }
     g_object_unref(dline);
@@ -384,8 +383,8 @@ execute(ModuleArgs *args, GwyResults *results)
                             "integral2", sqrt(integral2/projlen),
                             NULL);
 
-    g_object_get(args->gmodel, "si-unit-x", &xunit, "si-unit-y", &yunit, NULL);
-    if (gwy_si_unit_equal(xunit, yunit))
+    g_object_get(args->gmodel, "unit-x", &xunit, "unit-y", &yunit, NULL);
+    if (gwy_unit_equal(xunit, yunit))
         gwy_results_fill_values(results, "length", length, NULL);
     g_object_unref(xunit);
     g_object_unref(yunit);
