@@ -1,5 +1,5 @@
 /*
- *  $Id: graph_terraces.c 26524 2024-08-15 18:14:00Z yeti-dn $
+ *  $Id: graph_terraces.c 29059 2026-01-02 14:27:52Z yeti-dn $
  *  Copyright (C) 2019-2023 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,23 +16,11 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
 #include <string.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwythreads.h>
-#include <libprocess/gwyprocess.h>
-#include <libgwydgets/gwygraphmodel.h>
-#include <libgwydgets/gwycombobox.h>
-#include <libgwydgets/gwynullstore.h>
-#include <libgwydgets/gwycheckboxes.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwydgets/gwydgetutils.h>
-#include <libgwymodule/gwymodule-graph.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
-#include "libgwyddion/gwyomp.h"
-#include "../process/preview.h"
+#include <gwy.h>
+#include "libgwyddion/omp.h"
 
 /* Lower symmetric part indexing */
 /* i MUST be greater or equal than j */
@@ -44,7 +32,7 @@
 #define pwr 0.65
 
 enum {
-    RESPONSE_SURVEY = 1000,
+    RESPONSE_SURVEY = GWY_RESPONSE_USER,
 };
 
 enum {
@@ -119,9 +107,9 @@ typedef struct {
 typedef struct {
     GwyParams *params;
     GwyGraphModel *gmodel;
-    GwyDataLine *edges;
-    GwyDataLine *residuum;
-    GwyDataLine *background;
+    GwyLine *edges;
+    GwyLine *residuum;
+    GwyLine *background;
     GArray *terracesegments;
     FitResult *fres;
 } ModuleArgs;
@@ -133,12 +121,11 @@ typedef struct {
     GtkWidget *graph;
     GwyResults *results;
     GtkWidget *terracelist;
-    GdkPixbuf *colourpixbuf;
     GwyParamTable *table_param;
     GwyParamTable *table_terraces;
     GwyParamTable *table_output;
     GwyParamTable *table_survey;
-    GwySIValueFormat *vf;
+    GwyValueFormat *vf;
 } ModuleGUI;
 
 typedef struct {
@@ -168,9 +155,10 @@ typedef struct {
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             graph_terraces      (GwyGraph *graph);
+static void             module_main         (GwyGraph *graph);
 static GwyDialogOutcome run_gui             (ModuleArgs *args,
-                                             GwyContainer *data);
+                                             GwyFile *data,
+                                             gint id);
 static void             param_changed       (ModuleGUI *gui,
                                              gint id);
 static void             dialog_response     (ModuleGUI *gui,
@@ -181,9 +169,9 @@ static void             fill_preview_graph  (ModuleGUI *gui);
 static FitResult*       terrace_do          (const gdouble *xdata,
                                              const gdouble *ydata,
                                              guint ndata,
-                                             GwyDataLine *edges,
-                                             GwyDataLine *residuum,
-                                             GwyDataLine *background,
+                                             GwyLine *edges,
+                                             GwyLine *residuum,
+                                             GwyLine *background,
                                              GArray *terracesegments,
                                              GwySelection *xsel,
                                              GwyParams *params,
@@ -193,10 +181,11 @@ static GtkWidget*       parameters_tab_new  (ModuleGUI *gui);
 static GtkWidget*       terrace_list_tab_new(ModuleGUI *gui);
 static GtkWidget*       output_tab_new      (ModuleGUI *gui);
 static GtkWidget*       survey_tab_new      (ModuleGUI *gui);
-static GwyResults*      create_results      (GwyContainer *data,
+static GwyResults*      create_results      (GwyFile *data,
+                                             gint id,
                                              GwyGraphModel *gmodel);
 static void             create_output_graphs(ModuleArgs *args,
-                                             GwyContainer *data);
+                                             GwyFile *data);
 static void             free_fit_result     (FitResult *fres);
 static gchar*           format_report       (gpointer user_data);
 static guint            prepare_survey      (GwyParams *params,
@@ -213,7 +202,6 @@ static const GwyEnum output_flags[] = {
     { N_("Polynomial background"), OUTPUT_BACKGROUND, },
 };
 
-
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
@@ -230,9 +218,9 @@ static gboolean
 module_register(void)
 {
     gwy_graph_func_register("graph_terraces",
-                            (GwyGraphFunc)&graph_terraces,
+                            &module_main,
                             N_("/Measure _Features/_Terraces..."),
-                            GWY_STOCK_GRAPH_TERRACE_MEASURE,
+                            GWY_ICON_GRAPH_TERRACE_MEASURE,
                             GWY_MENU_FLAG_GRAPH_CURVE,
                             N_("Fit terraces with polynomial background"));
 
@@ -269,7 +257,7 @@ define_module_params(void)
                              0.0, 0.4, 0.015);
     gwy_param_def_add_int(paramdef, PARAM_POLY_DEGREE, "poly_degree", _("_Polynomial degree"), 0, MAX_DEGREE, 4);
     gwy_param_def_add_boolean(paramdef, PARAM_INDEPENDENT, "independent", _("_Independent heights"), FALSE);
-    gwy_param_def_add_gwyenum(paramdef, PARAM_DISPLAY, NULL, gwy_sgettext("verb|Display"),
+    gwy_param_def_add_gwyenum(paramdef, PARAM_DISPLAY, NULL, C_("verb", "Display"),
                               displays, G_N_ELEMENTS(displays), PREVIEW_DATA_FIT);
     gwy_param_def_add_boolean(paramdef, PARAM_USE_SELECTION, "use_selection", _("Select regions _manually"), FALSE);
     gwy_param_def_add_report_type(paramdef, PARAM_FIT_REPORT_STYLE, "fit_report_style", _("Save Fit Report"),
@@ -293,29 +281,32 @@ define_module_params(void)
 }
 
 static void
-graph_terraces(GwyGraph *graph)
+module_main(GwyGraph *graph)
 {
-    GwyContainer *data;
+    GwyFile *data;
     GwyDialogOutcome outcome;
-    GwySIUnit *unit;
+    GwyUnit *unit;
     ModuleArgs args;
+    gint id;
 
-    gwy_clear(&args, 1);
+    gwy_clear1(args);
     args.params = gwy_params_new_from_settings(define_module_params());
     args.gmodel = gwy_graph_get_model(graph);
-    gwy_app_data_browser_get_current(GWY_APP_CONTAINER, &data, 0);
+    gwy_data_browser_get_current(GWY_APP_CONTAINER, &data,
+                                 GWY_APP_GRAPH_MODEL_ID, &id,
+                                 0);
 
-    args.background = gwy_data_line_new(1, 1.0, TRUE);
-    g_object_get(args.gmodel, "si-unit-x", &unit, NULL);
-    gwy_si_unit_assign(gwy_data_line_get_si_unit_x(args.background), unit);
+    args.background = gwy_line_new(1, 1.0, TRUE);
+    g_object_get(args.gmodel, "unit-x", &unit, NULL);
+    gwy_unit_assign(gwy_line_get_unit_x(args.background), unit);
     g_object_unref(unit);
-    g_object_get(args.gmodel, "si-unit-y", &unit, NULL);
-    gwy_si_unit_assign(gwy_data_line_get_si_unit_y(args.background), unit);
-    args.edges = gwy_data_line_duplicate(args.background);
-    args.residuum = gwy_data_line_duplicate(args.background);
+    g_object_get(args.gmodel, "unit-y", &unit, NULL);
+    gwy_unit_assign(gwy_line_get_unit_y(args.background), unit);
+    args.edges = gwy_line_copy(args.background);
+    args.residuum = gwy_line_copy(args.background);
     args.terracesegments = g_array_new(FALSE, FALSE, sizeof(TerraceSegment));
 
-    outcome = run_gui(&args, data);
+    outcome = run_gui(&args, data, id);
     gwy_params_save_to_settings(args.params);
     if (outcome == GWY_DIALOG_HAVE_RESULT)
         create_output_graphs(&args, data);
@@ -329,7 +320,7 @@ graph_terraces(GwyGraph *graph)
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     GwyDialogOutcome outcome;
     GwyDialog *dialog;
@@ -337,15 +328,10 @@ run_gui(ModuleArgs *args, GwyContainer *data)
     GwyGraphArea *area;
     GwySelection *xsel;
     ModuleGUI gui;
-    gint width, height;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
-    gui.results = create_results(data, args->gmodel);
-
-    gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &width, &height);
-    height |= 1;
-    gui.colourpixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, 8, height, height);
+    gui.results = create_results(data, id, args->gmodel);
 
     gui.gmodel = gwy_graph_model_new_alike(args->gmodel);
     g_object_set(gui.gmodel, "label-visible", FALSE, NULL);
@@ -354,7 +340,7 @@ run_gui(ModuleArgs *args, GwyContainer *data)
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_CLEAR, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    hbox = gwy_hbox_new(0);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
     gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
     gwy_dialog_add_content(dialog, hbox, FALSE, FALSE, 0);
 
@@ -390,7 +376,7 @@ run_gui(ModuleArgs *args, GwyContainer *data)
 
     g_object_unref(gui.results);
     g_object_unref(gui.gmodel);
-    gwy_si_unit_value_format_free(gui.vf);
+    gwy_value_format_free(gui.vf);
 
     return outcome;
 }
@@ -402,14 +388,14 @@ update_value_formats(ModuleGUI *gui)
     GwyGraphCurveModel *gcmodel = gwy_graph_model_get_curve(gui->args->gmodel, curve);
     GtkTreeView *treeview;
     GList *columns, *l;
-    GwySIUnit *yunit;
+    GwyUnit *yunit;
     gdouble min, max, yrange;
 
-    g_object_get(gui->gmodel, "si-unit-y", &yunit, NULL);
+    g_object_get(gui->gmodel, "unit-y", &yunit, NULL);
 
     gwy_graph_curve_model_get_y_range(gcmodel, &min, &max);
     yrange = max - min;
-    gui->vf = gwy_si_unit_get_format_with_digits(yunit, GWY_SI_UNIT_FORMAT_MARKUP, yrange, 4, gui->vf);
+    gui->vf = gwy_unit_get_format_with_digits(yunit, GWY_UNIT_FORMAT_MARKUP, yrange, 4, gui->vf);
 
     g_object_unref(yunit);
 
@@ -477,7 +463,7 @@ render_text_column(GtkTreeViewColumn *column, GtkCellRenderer *renderer,
 {
     guint column_id = GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(column), "column-id"));
     ModuleGUI *gui = (ModuleGUI*)user_data;
-    GwySIValueFormat *vf = gui->vf;
+    GwyValueFormat *vf = gui->vf;
     TerraceSegment *seg;
     gchar buf[32];
     guint i;
@@ -503,16 +489,14 @@ render_text_column(GtkTreeViewColumn *column, GtkCellRenderer *renderer,
 }
 
 static void
-render_colour(G_GNUC_UNUSED GtkTreeViewColumn *column, G_GNUC_UNUSED GtkCellRenderer *renderer,
+render_colour(G_GNUC_UNUSED GtkTreeViewColumn *column, GtkCellRenderer *renderer,
               GtkTreeModel *model, GtkTreeIter *iter,
-              gpointer user_data)
+              G_GNUC_UNUSED gpointer user_data)
 {
-    ModuleGUI *gui = (ModuleGUI*)user_data;
-    guint i, pixel;
+    guint i;
 
     gtk_tree_model_get(model, iter, 0, &i, -1);
-    pixel = 0xff | gwy_rgba_to_pixbuf_pixel(gwy_graph_get_preset_color(i+1));
-    gdk_pixbuf_fill(gui->colourpixbuf, pixel);
+    g_object_set(renderer, "color", gwy_graph_get_preset_color(i+1), NULL);
 }
 
 static GtkTreeViewColumn*
@@ -552,7 +536,7 @@ terrace_list_tab_new(ModuleGUI *gui)
     GwyNullStore *store;
     GwyParamTable *table;
 
-    vbox = gtk_vbox_new(FALSE, 2);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 2);
     gtk_container_set_border_width(GTK_CONTAINER(vbox), 4);
 
     scwin = gtk_scrolled_window_new(NULL, NULL);
@@ -564,9 +548,8 @@ terrace_list_tab_new(ModuleGUI *gui)
     gtk_container_add(GTK_CONTAINER(scwin), gui->terracelist);
 
     column = append_text_column(gui, COLUMN_ID, "n", FALSE);
-    renderer = gtk_cell_renderer_pixbuf_new();
+    renderer = gwy_cell_renderer_color_new();
     gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(column), renderer, FALSE);
-    g_object_set(renderer, "pixbuf", gui->colourpixbuf, NULL);
     gtk_tree_view_column_set_cell_data_func(column, renderer, render_colour, gui, NULL);
     append_text_column(gui, COLUMN_HEIGHT, "h", TRUE);
     append_text_column(gui, COLUMN_LEVEL, "k", FALSE);
@@ -578,7 +561,7 @@ terrace_list_tab_new(ModuleGUI *gui)
     gwy_param_table_append_report(table, PARAM_TERRACE_REPORT_STYLE);
     gwy_param_table_report_set_formatter(table, PARAM_TERRACE_REPORT_STYLE, format_report, gui, NULL);
     /* XXX: Silly.  Just want to right-align the export controls for consistency. */
-    hbox = gwy_hbox_new(0);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
     gtk_box_pack_end(GTK_BOX(hbox), gwy_param_table_widget(table), FALSE, FALSE, 0);
     gwy_dialog_add_param_table(GWY_DIALOG(gui->dialog), table);
@@ -628,10 +611,11 @@ survey_tab_new(ModuleGUI *gui)
 }
 
 static GwyResults*
-create_results(GwyContainer *data,
+create_results(GwyFile *data,
+               gint id,
                GwyGraphModel *gmodel)
 {
-    GwySIUnit *yunit;
+    GwyUnit *yunit;
     GwyResults *results = gwy_results_new();
 
     gwy_results_add_header(results, N_("Fit Results"));
@@ -644,11 +628,11 @@ create_results(GwyContainer *data,
     gwy_results_add_value_z(results, "resid", N_("Mean square difference"));
     gwy_results_add_value_z(results, "discrep", N_("Terrace discrepancy"));
     gwy_results_add_value_int(results, "nterraces", N_("Number of terraces"));
-    g_object_get(gmodel, "si-unit-y", &yunit, NULL);
+    g_object_get(gmodel, "unit-y", &yunit, NULL);
     gwy_results_set_unit(results, "z", yunit);
     g_object_unref(yunit);
     gwy_results_fill_filename(results, "file", data);
-    gwy_results_fill_graph(results, "graph", gmodel);
+    gwy_results_fill_data_name(results, "graph", data, GWY_FILE_GRAPH, id);
 
     return results;
 }
@@ -681,9 +665,9 @@ param_changed(ModuleGUI *gui, gint id)
         gcmodel = gwy_graph_model_get_curve(args->gmodel, curve);
         gwy_graph_model_add_curve(gui->gmodel, gcmodel);
         ndata = gwy_graph_curve_model_get_ndata(gcmodel);
-        gwy_data_line_resample(args->edges, ndata, GWY_INTERPOLATION_NONE);
-        gwy_data_line_resample(args->residuum, ndata, GWY_INTERPOLATION_NONE);
-        gwy_data_line_resample(args->background, ndata, GWY_INTERPOLATION_NONE);
+        gwy_line_resize(args->edges, ndata);
+        gwy_line_resize(args->residuum, ndata);
+        gwy_line_resize(args->background, ndata);
     }
     if (id == PARAM_DISPLAY)
         fill_preview_graph(gui);
@@ -783,7 +767,7 @@ create_segmented_graph_curve(GwyGraphModel *gmodel,
 
     for (g = 0; g < nterraces; g++) {
         const TerraceSegment *seg = &terrace_index(terracesegments, g);
-        gcmodel2 = gwy_graph_curve_model_duplicate(gcmodel);
+        gcmodel2 = gwy_graph_curve_model_copy(gcmodel);
         g_string_printf(str, _("Segment %u"), g+1);
         g_object_set(gcmodel2, "color", gwy_graph_get_preset_color(g+1), "description", str->str, NULL);
         gwy_graph_curve_model_set_data(gcmodel2, xdata + seg->i, ydata + seg->i, seg->npixels);
@@ -815,13 +799,13 @@ create_one_output_graph(ModuleArgs *args,
                         gboolean for_preview)
 {
     GwyParams *params = args->params;
-    GwyDataLine *edges = args->edges, *residuum = args->residuum, *background = args->background;
+    GwyLine *edges = args->edges, *residuum = args->residuum, *background = args->background;
     gint curve = gwy_params_get_int(params, PARAM_CURVE);
     gboolean independent = gwy_params_get_boolean(params, PARAM_INDEPENDENT);
     gdouble edge_threshold = gwy_params_get_double(params, PARAM_EDGE_THRESHOLD);
     GwyGraphCurveModel *gcmodel = gwy_graph_model_get_curve(args->gmodel, curve);
     const gdouble *xdata, *ydata;
-    GwyDataLine *dline;
+    GwyLine *dline;
     gdouble *d;
     guint i, j, ndata, nterraces;
 
@@ -830,7 +814,7 @@ create_one_output_graph(ModuleArgs *args,
     ndata = gwy_graph_curve_model_get_ndata(gcmodel);
 
     if (mode == PREVIEW_DATA_FIT || mode == PREVIEW_DATA_POLY) {
-        gcmodel = gwy_graph_curve_model_duplicate(gcmodel);
+        gcmodel = gwy_graph_curve_model_copy(gcmodel);
         g_object_set(gcmodel, "color", gwy_graph_get_preset_color(0), NULL);
         gwy_graph_model_add_curve(gmodel, gcmodel);
         g_object_unref(gcmodel);
@@ -848,8 +832,8 @@ create_one_output_graph(ModuleArgs *args,
 
     if (mode == PREVIEW_DATA_FIT) {
         /* Add segmented fit */
-        dline = gwy_data_line_duplicate(residuum);
-        d = gwy_data_line_get_data(dline);
+        dline = gwy_line_copy(residuum);
+        d = gwy_line_get_data(dline);
         for (i = 0; i < ndata; i++)
             d[i] = ydata[i] - d[i];
         g_object_set(gcmodel, "line-width", 2, NULL);
@@ -862,16 +846,16 @@ create_one_output_graph(ModuleArgs *args,
         GwyGraphCurveModel *gcmodel2;
         GString *str = g_string_new(NULL);
 
-        dline = gwy_data_line_duplicate(background);
+        dline = gwy_line_copy(background);
         if (independent) {
             for (i = 0; i < nterraces; i++) {
-                gwy_data_line_copy(background, dline);
-                gwy_data_line_add(dline, solution[i]);
+                gwy_line_copy_data(background, dline);
+                gwy_line_add(dline, solution[i]);
 
-                gcmodel2 = gwy_graph_curve_model_duplicate(gcmodel);
+                gcmodel2 = gwy_graph_curve_model_copy(gcmodel);
                 g_string_printf(str, _("Segment %u"), i+1);
                 g_object_set(gcmodel2, "color", gwy_graph_get_preset_color(i+1), "description", str->str, NULL);
-                gwy_graph_curve_model_set_data(gcmodel2, xdata, gwy_data_line_get_data(dline), ndata);
+                gwy_graph_curve_model_set_data(gcmodel2, xdata, gwy_line_get_data(dline), ndata);
                 gwy_graph_model_add_curve(gmodel, gcmodel2);
                 g_object_unref(gcmodel2);
             }
@@ -887,13 +871,13 @@ create_one_output_graph(ModuleArgs *args,
                 if (i && levels[i-1] == levels[i])
                     continue;
 
-                gwy_data_line_copy(background, dline);
-                gwy_data_line_add(dline, solution[1] + levels[i]*solution[0]);
+                gwy_line_copy_data(background, dline);
+                gwy_line_add(dline, solution[1] + levels[i]*solution[0]);
 
-                gcmodel2 = gwy_graph_curve_model_duplicate(gcmodel);
+                gcmodel2 = gwy_graph_curve_model_copy(gcmodel);
                 g_string_printf(str, _("Level %d"), levels[i]);
                 g_object_set(gcmodel2, "description", str->str, NULL);
-                gwy_graph_curve_model_set_data(gcmodel2, xdata, gwy_data_line_get_data(dline), ndata);
+                gwy_graph_curve_model_set_data(gcmodel2, xdata, gwy_line_get_data(dline), ndata);
                 gwy_graph_model_add_curve(gmodel, gcmodel2);
                 g_object_unref(gcmodel2);
             }
@@ -904,14 +888,14 @@ create_one_output_graph(ModuleArgs *args,
     }
     else if (mode == PREVIEW_RESIDUUM) {
         /* Add segmented residuum. */
-        create_segmented_graph_curve(gmodel, gcmodel, terracesegments, xdata, gwy_data_line_get_data(residuum));
+        create_segmented_graph_curve(gmodel, gcmodel, terracesegments, xdata, gwy_line_get_data(residuum));
     }
     else if (mode == PREVIEW_TERRACES) {
         /* Add segmented ideal terraces. */
         const gdouble *solution = fres->solution;
 
-        dline = gwy_data_line_new_alike(background, TRUE);
-        d = gwy_data_line_get_data(dline);
+        dline = gwy_line_new_alike(background, TRUE);
+        d = gwy_line_get_data(dline);
         for (i = 0; i < nterraces; i++) {
             const TerraceSegment *seg = &terrace_index(terracesegments, i);
             gdouble h = (independent ? solution[i] : solution[1] + seg->level*solution[0]);
@@ -924,8 +908,8 @@ create_one_output_graph(ModuleArgs *args,
     }
     else if (mode == PREVIEW_LEVELLED) {
         /* Add segmented data minus background. */
-        dline = gwy_data_line_duplicate(background);
-        d = gwy_data_line_get_data(dline);
+        dline = gwy_line_copy(background);
+        d = gwy_line_get_data(dline);
         for (i = 0; i < ndata; i++)
             d[i] = ydata[i] - d[i];
         gwy_graph_curve_model_set_data(gcmodel, xdata, d, ndata);
@@ -937,7 +921,7 @@ create_one_output_graph(ModuleArgs *args,
     }
     else if (mode == PREVIEW_BACKGROUND) {
         /* Add full background. */
-        gwy_graph_curve_model_set_data(gcmodel, xdata, gwy_data_line_get_data(background), ndata);
+        gwy_graph_curve_model_set_data(gcmodel, xdata, gwy_line_get_data(background), ndata);
         g_object_set(gcmodel, "description", _("Polynomial background"), NULL);
         gwy_graph_model_add_curve(gmodel, gcmodel);
     }
@@ -946,18 +930,18 @@ create_one_output_graph(ModuleArgs *args,
 
         /* Add full step filter result. */
         g_object_set(gcmodel, "color", gwy_graph_get_preset_color(0), NULL);
-        gwy_graph_curve_model_set_data(gcmodel, xdata, gwy_data_line_get_data(edges), ndata);
+        gwy_graph_curve_model_set_data(gcmodel, xdata, gwy_line_get_data(edges), ndata);
         gwy_graph_model_add_curve(gmodel, gcmodel);
         g_object_unref(gcmodel);
 
         gcmodel = gwy_graph_curve_model_new();
         stepxdata[0] = xdata[0];
         stepxdata[1] = xdata[ndata-1];
-        stepydata[0] = stepydata[1] = edge_threshold*gwy_data_line_get_max(edges);
+        stepydata[0] = stepydata[1] = edge_threshold*gwy_line_max(edges);
         gwy_graph_curve_model_set_data(gcmodel, stepxdata, stepydata, 2);
         g_object_set(gcmodel,
                      "mode", GWY_GRAPH_CURVE_LINE,
-                     "line-style", GDK_LINE_ON_OFF_DASH,
+                     "line-style", GWY_GRAPH_LINE_DASH,
                      "color", gwy_graph_get_preset_color(1),
                      NULL);
         gwy_graph_model_add_curve(gmodel, gcmodel);
@@ -980,7 +964,7 @@ preview(gpointer user_data)
 {
     ModuleGUI *gui = (ModuleGUI*)user_data;
     ModuleArgs *args = gui->args;
-    GwyDataLine *edges = args->edges, *residuum = args->residuum, *background = args->background;
+    GwyLine *edges = args->edges, *residuum = args->residuum, *background = args->background;
     GwyParams *params = args->params;
     gint curve = gwy_params_get_int(params, PARAM_CURVE);
     gboolean independent = gwy_params_get_boolean(params, PARAM_INDEPENDENT);
@@ -1071,7 +1055,7 @@ make_segments_from_xsel(GArray *terracesegments,
         GWY_ORDER(gdouble, xseg[0], xseg[1]);
 
         /* Find integer index ranges corresponding to the selection.  Be careful to skip empty segments. */
-        gwy_clear(&seg, 1);
+        gwy_clear1(seg);
         seg.xfrom = xseg[0];
         seg.xto = xseg[1];
         for (j = 0; j < ndata; j++) {
@@ -1117,12 +1101,12 @@ step_gauss_line_integrals(gdouble t, gdouble x1, gdouble x2,
 static void
 apply_gaussian_step_filter(const gdouble *xdata,
                            const gdouble *ydata,
-                           GwyDataLine *filtered,
+                           GwyLine *filtered,
                            gdouble dx,
                            gdouble sigma)
 {
-    gint n = gwy_data_line_get_res(filtered);
-    gdouble *d = gwy_data_line_get_data(filtered);
+    gint n = gwy_line_get_res(filtered);
+    gdouble *d = gwy_line_get_data(filtered);
     gint i;
 
     gwy_clear(d, n);
@@ -1175,11 +1159,11 @@ apply_gaussian_step_filter(const gdouble *xdata,
 }
 
 static void
-enumerate_line_segments(GwyDataLine *marked, const gdouble *xdata,
+enumerate_line_segments(GwyLine *marked, const gdouble *xdata,
                         GArray *terracesegments)
 {
-    gint n = gwy_data_line_get_res(marked);
-    gdouble *md = gwy_data_line_get_data(marked);
+    gint n = gwy_line_get_res(marked);
+    gdouble *md = gwy_line_get_data(marked);
     gint i, prevedge;
     TerraceSegment seg;
 
@@ -1191,7 +1175,7 @@ enumerate_line_segments(GwyDataLine *marked, const gdouble *xdata,
         if (md[i-1] != md[i]) {
             if (md[i] == 0.0) {
                 /* Downwards edge.  The previous segment is a grain. */
-                gwy_clear(&seg, 1);
+                gwy_clear1(seg);
                 seg.xfrom = (prevedge ? 0.5*(xdata[prevedge-1] + xdata[prevedge]) : 1.5*xdata[0] - 0.5*xdata[1]);
                 seg.xto = 0.5*(xdata[i-1] + xdata[i]);
                 seg.i = prevedge;
@@ -1205,7 +1189,7 @@ enumerate_line_segments(GwyDataLine *marked, const gdouble *xdata,
 
     /* Data ending inside a grain is the same as downwards edge. */
     if (md[n-1]) {
-        gwy_clear(&seg, 1);
+        gwy_clear1(seg);
         seg.xfrom = (prevedge ? 0.5*(xdata[prevedge-1] + xdata[prevedge]) : 1.5*xdata[0] - 0.5*xdata[1]);
         seg.xto = 1.5*xdata[n-1] - 0.5*xdata[n-2];
         seg.i = prevedge;
@@ -1216,10 +1200,10 @@ enumerate_line_segments(GwyDataLine *marked, const gdouble *xdata,
 
 /* Shrink grains using real distance (i.e. possibly non-uniform sampling). */
 static void
-shrink_grains(GwyDataLine *marked, const gdouble *xdata, gdouble distance, GArray *terracesegments)
+shrink_grains(GwyLine *marked, const gdouble *xdata, gdouble distance, GArray *terracesegments)
 {
-    gint j, n = gwy_data_line_get_res(marked);
-    gdouble *md = gwy_data_line_get_data(marked);
+    gint j, n = gwy_line_get_res(marked);
+    gdouble *md = gwy_line_get_data(marked);
     guint g, nseg;
 
     nseg = terracesegments->len;
@@ -1245,9 +1229,9 @@ shrink_grains(GwyDataLine *marked, const gdouble *xdata, gdouble distance, GArra
 
 /* Remove grains by size using real distance (i.e. possibly non-uniform sampling). */
 static void
-remove_grains_by_size(GwyDataLine *marked, gdouble minsize, GArray *terracesegments)
+remove_grains_by_size(GwyLine *marked, gdouble minsize, GArray *terracesegments)
 {
-    gdouble *md = gwy_data_line_get_data(marked);
+    gdouble *md = gwy_line_get_data(marked);
     guint g, nseg;
 
     /* Iterate backwards.  We do not need to care how the array changes beyond the current item and the removals are
@@ -1269,8 +1253,8 @@ find_terrace_segments(GArray *terracesegments,
                       const gdouble *ydata,
                       guint ndata,
                       GwyParams *params,
-                      GwyDataLine *edges,
-                      GwyDataLine *marked,
+                      GwyLine *edges,
+                      GwyLine *marked,
                       GwySelection *xsel,
                       gdouble *pxc,
                       gdouble *pxq)
@@ -1290,15 +1274,15 @@ find_terrace_segments(GArray *terracesegments,
         return FALSE;
 
     /* Use a common `pixel' size. */
-    gwy_assign(gwy_data_line_get_data(edges), xdata, ndata);
-    gwy_data_line_get_min_max(edges, &min, &max);
+    gwy_assign(gwy_line_get_data(edges), xdata, ndata);
+    gwy_line_min_max(edges, &min, &max);
     xlen = max - min;
     dx = xlen/ndata;
 
     /* Always calculate the Gaussian filter to have something to display even when we do not use the result for
      * fitting. */
     apply_gaussian_step_filter(xdata, ydata, edges, dx, edge_kernel_size);
-    gwy_data_line_copy(edges, marked);
+    gwy_line_copy_data(edges, marked);
 
     if (use_selection) {
         /* Use provided selection as requested. */
@@ -1306,8 +1290,8 @@ find_terrace_segments(GArray *terracesegments,
     }
     else {
         /* Mark flat areas in the profile. */
-        threshold = edge_threshold*gwy_data_line_get_max(marked);
-        gwy_data_line_threshold(marked, threshold, 1.0, 0.0);
+        threshold = edge_threshold*gwy_line_max(marked);
+        gwy_line_threshold(marked, threshold, 1.0, 0.0);
         enumerate_line_segments(marked, xdata, terracesegments);
         shrink_grains(marked, xdata, edge_broadening*dx, terracesegments);
 
@@ -1414,7 +1398,7 @@ calculate_power_matrix_block(GArray *terracesegments, const gdouble *xdata,
 
 static void
 calculate_residuum(GArray *terracesegments, FitResult *fres,
-                   GwyDataLine *residuum,
+                   GwyLine *residuum,
                    const gdouble *xdata, const gdouble *ydata,
                    gdouble xc, gdouble xq,
                    gint poly_degree,
@@ -1425,8 +1409,8 @@ calculate_residuum(GArray *terracesegments, FitResult *fres,
     guint g, i, k, nterraces = terracesegments->len, npixels;
 
     solution_block = solution + (indep ? nterraces : 2);
-    gwy_data_line_clear(residuum);
-    resdata = gwy_data_line_get_data(residuum);
+    gwy_line_clear(residuum);
+    resdata = gwy_line_get_data(residuum);
 
     fres->msq = fres->deltares = 0.0;
     npixels = 0;
@@ -1466,7 +1450,7 @@ fit_terraces_arbitrary(GArray *terracesegments,
                        gdouble xc, gdouble xq,
                        gint poly_degree,
                        const gdouble *power_block,
-                       GwyDataLine *residuum,
+                       GwyLine *residuum,
                        const gchar **message)
 {
     gint g, k, nterraces, matn, matsize, G_GNUC_UNUSED npixels;
@@ -1584,7 +1568,7 @@ fit_terraces_same_step(GArray *terracesegments,
                        gdouble xc, gdouble xq,
                        gint poly_degree,
                        const gdouble *power_block,
-                       GwyDataLine *residuum,
+                       GwyLine *residuum,
                        const gchar **message)
 {
     gint g, k, nterraces, matn, matsize, npixels;
@@ -1748,7 +1732,7 @@ estimate_step_parameters(const gdouble *heights, guint n,
     }
 
     p = 85.0;
-    gwy_math_percentiles(ns, steps, GWY_PERCENTILE_INTERPOLATION_LINEAR, 1, &p, &sh);
+    gwy_math_percentiles(steps, ns, GWY_PERCENTILE_INTERPOLATION_LINEAR, &p, 1, &sh);
     gwy_debug("estimated step height %g nm", sh/1e-9);
     g_free(steps);
 
@@ -1777,7 +1761,7 @@ estimate_step_parameters(const gdouble *heights, guint n,
 /* XXX: The background is generally bogus far outside the fitted region.  This usually means profile ends because they
  * contain too small terrace bits. It is more meaningful to only calculate it for marked area. */
 static void
-fill_background(GwyDataLine *background,
+fill_background(GwyLine *background,
                 const gdouble *xdata, gdouble xc, gdouble xq,
                 gint poly_degree,
                 const gdouble *coeffs)
@@ -1785,8 +1769,8 @@ fill_background(GwyDataLine *background,
     gint res, i, k;
     gdouble *d;
 
-    res = gwy_data_line_get_res(background);
-    d = gwy_data_line_get_data(background);
+    res = gwy_line_get_res(background);
+    d = gwy_line_get_data(background);
     for (i = 0; i < res; i++) {
         gdouble x = xq*(xdata[i] - xc);
         gdouble xp = 1.0, s = 0.0;
@@ -1801,7 +1785,7 @@ fill_background(GwyDataLine *background,
 
 static FitResult*
 terrace_do(const gdouble *xdata, const gdouble *ydata, guint ndata,
-           GwyDataLine *edges, GwyDataLine *residuum, GwyDataLine *background,
+           GwyLine *edges, GwyLine *residuum, GwyLine *background,
            GArray *terracesegments, GwySelection *xsel,
            GwyParams *params,
            const gchar **message)
@@ -1814,7 +1798,7 @@ terrace_do(const gdouble *xdata, const gdouble *ydata, guint ndata,
     gint g, poly_degree = gwy_params_get_int(params, PARAM_POLY_DEGREE);
     gint *levels;
 
-    /* Use background dataline as a scratch space. */
+    /* Use background line as a scratch space. */
     if (!find_terrace_segments(terracesegments, xdata, ydata, ndata, params, edges, background, xsel, &xc, &xq)) {
         *message = _("No terraces were found");
         return NULL;
@@ -1864,7 +1848,7 @@ finalise:
 }
 
 static void
-create_output_graphs(ModuleArgs *args, GwyContainer *data)
+create_output_graphs(ModuleArgs *args, GwyFile *data)
 {
     const guint output_map[2*G_N_ELEMENTS(output_flags)] = {
         PREVIEW_DATA_FIT,   OUTPUT_DATA_FIT,
@@ -1888,7 +1872,8 @@ create_output_graphs(ModuleArgs *args, GwyContainer *data)
         create_one_output_graph(args, gmodel, output_map[2*i], terracesegments, fres, FALSE);
         title = gwy_enum_to_string(output_map[2*i + 1], output_flags, G_N_ELEMENTS(output_flags));
         g_object_set(gmodel, "title", _(title), NULL);
-        gwy_app_data_browser_add_graph_model(gmodel, data, TRUE);
+        gint newid = gwy_file_add_graph(data, gmodel);
+        gwy_file_set_visible(data, GWY_FILE_GRAPH, newid, TRUE);
         g_object_unref(gmodel);
     }
 }
@@ -1900,9 +1885,9 @@ format_report(gpointer user_data)
     ModuleArgs *args = gui->args;
     GwyResultsReportType report_style = gwy_params_get_report_type(args->params, PARAM_TERRACE_REPORT_STYLE);
     GArray *terracesegments = args->terracesegments;
-    GwySIUnitFormatStyle style = GWY_SI_UNIT_FORMAT_UNICODE;
-    GwySIValueFormat *vfz;
-    GwySIUnit *yunit;
+    GwyUnitFormatStyle style = GWY_UNIT_FORMAT_UNICODE;
+    GwyValueFormat *vfz;
+    GwyUnit *yunit;
     GString *text;
     const gchar *k_header, *Npx_header;
     gchar *h_header, *Delta_header, *r_header;
@@ -1912,8 +1897,8 @@ format_report(gpointer user_data)
     if (!(report_style & GWY_RESULTS_REPORT_MACHINE))
         vfz = gui->vf;
     else {
-        g_object_get(args->gmodel, "si-unit-y", &yunit, NULL);
-        vfz = gwy_si_unit_get_format_for_power10(yunit, style, 0, NULL);
+        g_object_get(args->gmodel, "unit-y", &yunit, NULL);
+        vfz = gwy_unit_get_format_for_power10(yunit, style, 0, NULL);
         g_object_unref(yunit);
     }
 
@@ -1987,7 +1972,7 @@ run_survey(ModuleGUI *gui)
     GwyResultsReportType report_style = gwy_params_get_report_type(args->params, PARAM_TERRACE_REPORT_STYLE);
     gint curve = gwy_params_get_int(args->params, PARAM_CURVE);
     GArray *terracesegments = NULL, *surveyout, *degrees, *broadenings;
-    GwyDataLine *edges, *residuum, *background;
+    GwyLine *edges, *residuum, *background;
     GwyGraphCurveModel *gcmodel;
     GwyGraphArea *area;
     FitResult *fres;
@@ -2007,9 +1992,9 @@ run_survey(ModuleGUI *gui)
     ydata = gwy_graph_curve_model_get_ydata(gcmodel);
     ndata = gwy_graph_curve_model_get_ndata(gcmodel);
 
-    edges = gwy_data_line_new_alike(args->edges, FALSE);
-    residuum = gwy_data_line_new_alike(args->residuum, FALSE);
-    background = gwy_data_line_new_alike(args->background, FALSE);
+    edges = gwy_line_new_alike(args->edges, FALSE);
+    residuum = gwy_line_new_alike(args->residuum, FALSE);
+    background = gwy_line_new_alike(args->background, FALSE);
     terracesegments = g_array_new(FALSE, FALSE, sizeof(TerraceSegment));
     surveyout = g_array_new(FALSE, FALSE, sizeof(TerraceSurveyRow));
 
@@ -2028,7 +2013,7 @@ run_survey(ModuleGUI *gui)
         fres = terrace_do(xdata, ydata, ndata, edges, residuum, background, terracesegments, xsel, surveyparams,
                           &message);
 
-        gwy_clear(&srow, 1);
+        gwy_clear1(srow);
         srow.poly_degree = gwy_params_get_int(surveyparams, PARAM_POLY_DEGREE);
         srow.edge_kernel_size = gwy_params_get_double(surveyparams, PARAM_EDGE_KERNEL_SIZE);
         srow.edge_threshold = gwy_params_get_double(surveyparams, PARAM_EDGE_THRESHOLD);
@@ -2080,7 +2065,7 @@ run_survey(ModuleGUI *gui)
     }
     g_array_free(surveyout, TRUE);
 
-    gwy_save_auxiliary_data(_("Save Terrace Fit Survey"), GTK_WINDOW(gui->dialog), -1, str->str);
+    gwy_save_auxiliary_data(_("Save Terrace Fit Survey"), GTK_WINDOW(gui->dialog), str->str, str->len, FALSE);
     g_string_free(str, TRUE);
 }
 
