@@ -1,5 +1,5 @@
 /*
- *  $Id: acf2d.c 27043 2024-12-13 16:42:03Z yeti-dn $
+ *  $Id: acf2d.c 28910 2025-11-24 18:11:09Z yeti-dn $
  *  Copyright (C) 2007-2022 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,18 +16,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/level.h>
-#include <libprocess/filters.h>
-#include <libprocess/grains.h>
-#include <libprocess/stats.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
@@ -76,10 +68,10 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *mask;
-    GwyDataField *acf;
-    GwyDataField *acfmask;
+    GwyField *field;
+    GwyField *mask;
+    GwyField *acf;
+    GwyField *acfmask;
     GwySelection *selection;
     GwyGraphModel *gmodel;
 } ModuleArgs;
@@ -91,22 +83,21 @@ typedef struct {
     GwyParamTable *table_acf;
     GwyParamTable *table_graph;
     GwyParamTable *table_params;
-    GwyDataLine *line;
-    GwyContainer *data;
+    GwyLine *line;
     GwyResults *results;
     UpdateWhat update;
 } ModuleGUI;
 
 static gboolean         module_register               (void);
 static GwyParamDef*     define_module_params          (void);
-static void             acf2d                         (GwyContainer *data,
-                                                       GwyRunType runtype);
+static void             module_main                   (GwyFile *data,
+                                                       GwyRunModeFlags mode);
 static void             execute                       (ModuleArgs *args);
 static GwyDialogOutcome run_gui                       (ModuleArgs *args,
-                                                       GwyContainer *data,
+                                                       GwyFile *data,
                                                        gint id);
 static GwyResults*      create_results                (ModuleArgs *args,
-                                                       GwyContainer *data,
+                                                       GwyFile *data,
                                                        gint id);
 static void             param_changed                 (ModuleGUI *gui,
                                                        gint id);
@@ -114,19 +105,19 @@ static void             dialog_response               (ModuleGUI *gui,
                                                        gint response);
 static void             preview                       (gpointer user_data);
 static void             calculate_zoomed_fields       (ModuleGUI *gui);
-static GwyDataField*    cut_field_to_zoom             (GwyDataField *field,
+static GwyField*        cut_field_to_zoom             (GwyField *field,
                                                        gint zoom);
 static void             selection_changed             (ModuleGUI *gui,
                                                        gint hint);
 static void             update_sensitivity            (ModuleGUI *gui);
 static void             update_curve                  (ModuleGUI *gui,
                                                        gint i);
-static void             add_line_selection_from_points(GwyContainer *data,
-                                                       GwyDataField *field,
+static void             add_line_selection_from_points(GwyFile *data,
+                                                       GwyField *field,
                                                        gint id,
                                                        GwySelection *pointsel);
 static void             create_acf_mask               (ModuleArgs *args);
-static gboolean         find_decays                   (GwyDataField *mask,
+static gboolean         find_decays                   (GwyField *mask,
                                                        gdouble *r1,
                                                        gdouble *r2,
                                                        gdouble *phi1,
@@ -150,11 +141,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("acf2d",
-                              (GwyProcessFunc)&acf2d,
+                              module_main,
                               N_("/_Statistics/2D Auto_correlation..."),
                               NULL,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Calculate 2D autocorrelation function"));
 
     return TRUE;
@@ -164,9 +155,9 @@ static GwyParamDef*
 define_module_params(void)
 {
     static const GwyEnum levels[] = {
-        { N_("data-adjustment|None"),   LEVELLING_NONE,       },
-        { N_("Mean value subtraction"), LEVELLING_MEAN_VALUE, },
-        { N_("Plane leveling"),         LEVELLING_MEAN_PLANE, },
+        { gwy_NC("data-adjustment", "None"), LEVELLING_NONE,       },
+        { N_("Mean value subtraction"),      LEVELLING_MEAN_VALUE, },
+        { N_("Plane leveling"),              LEVELLING_MEAN_PLANE, },
     };
     static GwyEnum zooms[5];
     static GwyParamDef *paramdef = NULL;
@@ -204,28 +195,28 @@ define_module_params(void)
 }
 
 static void
-acf2d(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     ModuleArgs args;
     gint oldid, newid;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_MASK_FIELD, &args.mask,
-                                     GWY_APP_DATA_FIELD_ID, &oldid,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_MASK_FIELD, &args.mask,
+                                 GWY_APP_FIELD_ID, &oldid,
+                                 0);
     g_return_if_fail(args.field);
 
     args.params = gwy_params_new_from_settings(define_module_params());
-    args.acf = gwy_data_field_new(17, 17, 1.0, 1.0, TRUE);
-    args.acfmask = gwy_data_field_new(17, 17, 1.0, 1.0, TRUE);
+    args.acf = gwy_field_new(17, 17, 1.0, 1.0, TRUE);
+    args.acfmask = gwy_field_new(17, 17, 1.0, 1.0, TRUE);
     // We need to set the units of args->gmodel immediately for target graph filtering.
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(args.acf), gwy_data_field_get_si_unit_xy(args.field));
-    gwy_si_unit_power(gwy_data_field_get_si_unit_z(args.field), 2, gwy_data_field_get_si_unit_z(args.acf));
+    gwy_unit_assign(gwy_field_get_unit_xy(args.acf), gwy_field_get_unit_xy(args.field));
+    gwy_unit_power(gwy_field_get_unit_z(args.field), 2, gwy_field_get_unit_z(args.acf));
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, oldid);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -241,20 +232,22 @@ acf2d(GwyContainer *data, GwyRunType runtype)
     if (gwy_params_get_boolean(args.params, PARAM_CREATE_IMAGE)) {
         gboolean zoomed_image = gwy_params_get_boolean(args.params, PARAM_ZOOMED_IMAGE);
         guint zoom = (zoomed_image ? gwy_params_get_enum(args.params, PARAM_ZOOM) : 1);
-        GwyDataField *zoomed, *zoomedmask;
+        GwyField *zoomed, *zoomedmask;
 
         zoomed = cut_field_to_zoom(args.acf, zoom);
         zoomedmask = cut_field_to_zoom(args.acfmask, zoom);
-        newid = gwy_app_data_browser_add_data_field(zoomed, data, TRUE);
-        gwy_container_set_object(data, gwy_app_get_mask_key_for_id(newid), zoomedmask);
+        newid = gwy_file_add_image(data, zoomed);
+        gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+        gwy_file_pass_image_mask(data, newid, zoomedmask);
         g_object_unref(zoomed);
-        g_object_unref(zoomedmask);
 
         add_line_selection_from_points(data, zoomed, newid, args.selection);
-        gwy_app_set_data_field_title(data, newid, _("2D ACF"));
-        gwy_app_channel_log_add_proc(data, oldid, newid);
+        gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _("2D ACF"), TRUE);
+        gwy_log_add(data, GWY_FILE_IMAGE, oldid, newid);
 
-        gwy_app_sync_data_items(data, data, oldid, newid, FALSE, GWY_DATA_ITEM_MASK_COLOR, 0);
+        gwy_file_sync_items(data, GWY_FILE_IMAGE, oldid,
+                            data, GWY_FILE_IMAGE, newid,
+                            GWY_FILE_ITEM_MASK_COLOR, FALSE);
     }
     if (args.gmodel && gwy_graph_model_get_n_curves(args.gmodel)) {
         GwyGraphCurveModel *gcmodel;
@@ -268,13 +261,14 @@ acf2d(GwyContainer *data, GwyRunType runtype)
             for (i = 0; i < n; i++) {
                 gmodel = gwy_graph_model_new_alike(args.gmodel);
                 gcmodel = gwy_graph_model_get_curve(args.gmodel, i);
-                gcmodel = gwy_graph_curve_model_duplicate(gcmodel);
+                gcmodel = gwy_graph_curve_model_copy(gcmodel);
                 gwy_graph_model_add_curve(gmodel, gcmodel);
                 g_object_unref(gcmodel);
                 g_object_get(gcmodel, "description", &s, NULL);
                 g_object_set(gmodel, "title", s, NULL);
                 g_free(s);
-                gwy_app_data_browser_add_graph_model(gmodel, data, TRUE);
+                newid = gwy_file_add_graph(data, gmodel);
+                gwy_file_set_visible(data, GWY_FILE_GRAPH, newid, TRUE);
                 g_object_unref(gmodel);
             }
         }
@@ -285,15 +279,15 @@ acf2d(GwyContainer *data, GwyRunType runtype)
     }
 
 end:
-    GWY_OBJECT_UNREF(args.selection);
-    GWY_OBJECT_UNREF(args.gmodel);
+    g_clear_object(&args.selection);
+    g_clear_object(&args.gmodel);
     g_object_unref(args.acf);
     g_object_unref(args.acfmask);
     g_object_unref(args.params);
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     ModuleGUI gui;
     GtkWidget *hbox, *vbox, *notebook;
@@ -301,38 +295,36 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     GwyDialog *dialog;
     GwyGraph *graph;
     GwyDialogOutcome outcome;
+    GwyVectorLayer *vlayer;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
     args->gmodel = gwy_graph_model_new();
-    gui.data = gwy_container_new();
     gui.results = create_results(args, data, id);
-    gui.line = gwy_data_line_new(1, 1.0, FALSE);
-    calculate_zoomed_fields(&gui);
+    gui.line = gwy_line_new(1, 1.0, FALSE);
 
     gui.dialog = gwy_dialog_new(_("Autocorrelation Function"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_CLEAR, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
     /***** ACF preview *****/
-    gwy_app_sync_data_items(data, gui.data, id, 0, FALSE,
-                            GWY_DATA_ITEM_PALETTE,
-                            GWY_DATA_ITEM_MASK_COLOR,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
-    gui.dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, TRUE);
-    args->selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(gui.dataview), 0, "Point", 12, TRUE);
+    gui.dataview = gwy_create_preview(NULL, NULL, PREVIEW_SIZE);
+    calculate_zoomed_fields(&gui);
+    gwy_setup_data_view(GWY_DATA_VIEW(gui.dataview), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_MASK_COLOR | GWY_FILE_ITEM_REAL_SQUARE);
+    args->selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(gui.dataview), GWY_TYPE_LAYER_POINT, 12, TRUE);
     g_object_ref(args->selection);
-    g_object_set(gwy_data_view_get_top_layer(GWY_DATA_VIEW(gui.dataview)), "draw-as-vector", TRUE, NULL);
+    vlayer = gwy_data_view_get_interactive_layer(GWY_DATA_VIEW(gui.dataview));
+    gwy_layer_point_set_draw_as_vector(GWY_LAYER_POINT(vlayer), TRUE);
     g_signal_connect_swapped(args->selection, "changed", G_CALLBACK(selection_changed), &gui);
 
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(gui.dataview), FALSE);
 
     /***** Graph *****/
-    vbox = gwy_vbox_new(0);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
     gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 4);
 
-    gwy_graph_model_set_units_from_data_field(args->gmodel, args->acf, 1, 0, 0, 1);
+    gwy_graph_model_set_units_from_field(args->gmodel, args->acf, 1, 0, 0, 1);
     g_object_set(args->gmodel,
                  "title", _("ACF Section"),
                  "axis-label-bottom", "τ",
@@ -378,7 +370,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 
     table = gui.table_params = gwy_param_table_new(args->params);
     gwy_param_table_append_slider(table, PARAM_THRESHOLD);
-    gwy_param_table_append_mask_color(table, PARAM_MASK_COLOR, gui.data, 0, data, id);
+    gwy_param_table_append_image_mask_color(table, PARAM_MASK_COLOR, GWY_DATA_VIEW(gui.dataview), data, id);
     gwy_param_table_append_resultsv(table, WIDGET_RESULTS, gui.results, result_values, G_N_ELEMENTS(result_values));
     gwy_param_table_append_report(table, PARAM_REPORT_STYLE);
     gwy_param_table_report_set_results(table, PARAM_REPORT_STYLE, gui.results);
@@ -393,7 +385,6 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 
     outcome = gwy_dialog_run(dialog);
 
-    g_object_unref(gui.data);
     g_object_unref(gui.results);
     g_object_unref(gui.line);
 
@@ -401,7 +392,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 }
 
 static GwyResults*
-create_results(ModuleArgs *args, GwyContainer *data, gint id)
+create_results(ModuleArgs *args, GwyFile *data, gint id)
 {
     GwyResults *results = gwy_results_new();
 
@@ -419,11 +410,11 @@ create_results(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_results_add_value(results, "Str", N_("Texture aspect ratio"),
                           "symbol", "S<sub>tr</sub>", NULL);
 
-    gwy_results_set_unit(results, "x", gwy_data_field_get_si_unit_xy(args->field));
-    gwy_results_set_unit(results, "y", gwy_data_field_get_si_unit_xy(args->field));
-    gwy_results_set_unit(results, "z", gwy_data_field_get_si_unit_z(args->field));
+    gwy_results_set_unit(results, "x", gwy_field_get_unit_xy(args->field));
+    gwy_results_set_unit(results, "y", gwy_field_get_unit_xy(args->field));
+    gwy_results_set_unit(results, "z", gwy_field_get_unit_z(args->field));
     gwy_results_fill_filename(results, "file", data);
-    gwy_results_fill_channel(results, "image", data, id);
+    gwy_results_fill_data_name(results, "image", data, GWY_FILE_IMAGE, id);
 
     return results;
 }
@@ -514,17 +505,18 @@ preview(gpointer user_data)
         gwy_param_table_results_fill(gui->table_params, WIDGET_RESULTS);
     }
     if (gui->update >= UPDATE_ZOOMED) {
-        GwyDataField *acf;
+        GwyDataView *dataview = GWY_DATA_VIEW(gui->dataview);
+        GwyField *acf;
         gdouble xoff, yoff;
 
-        acf = gwy_container_get_object(gui->data, gwy_app_get_data_key_for_id(0));
-        xoff = gwy_data_field_get_xoffset(acf);
-        yoff = gwy_data_field_get_yoffset(acf);
+        acf = gwy_data_view_get_field(dataview);
+        xoff = gwy_field_get_xoffset(acf);
+        yoff = gwy_field_get_yoffset(acf);
         calculate_zoomed_fields(gui);
-        gwy_set_data_preview_size(GWY_DATA_VIEW(gui->dataview), PREVIEW_SIZE);
-        acf = gwy_container_get_object(gui->data, gwy_app_get_data_key_for_id(0));
-        xoff -= gwy_data_field_get_xoffset(acf);
-        yoff -= gwy_data_field_get_yoffset(acf);
+        gwy_set_data_preview_size(dataview, PREVIEW_SIZE);
+        acf = gwy_data_view_get_field(dataview);
+        xoff -= gwy_field_get_xoffset(acf);
+        yoff -= gwy_field_get_yoffset(acf);
         if (xoff || yoff) {
             gwy_selection_move(args->selection, xoff, yoff);
             gui->update = UPDATE_NOTHING;
@@ -546,7 +538,7 @@ update_curve(ModuleGUI *gui, gint i)
     gint resolution = gwy_params_get_int(params, PARAM_RESOLUTION);
     gint thickness = gwy_params_get_int(params, PARAM_THICKNESS);
     GwyInterpolationType interpolation = gwy_params_get_enum(args->params, PARAM_INTERPOLATION);
-    GwyDataField *zoomedacf, *acf = args->acf;
+    GwyField *zoomedacf, *acf = args->acf;
     GwyGraphCurveModel *gcmodel;
     gdouble xy[4], xofffull, yofffull;
     gint xl0, yl0, xl1, yl1;
@@ -557,17 +549,17 @@ update_curve(ModuleGUI *gui, gint i)
         g_return_if_reached();
     }
 
-    zoomedacf = gwy_container_get_object(gui->data, gwy_app_get_data_key_for_id(0));
-    xy[0] += gwy_data_field_get_xoffset(zoomedacf);
-    xy[1] += gwy_data_field_get_yoffset(zoomedacf);
+    zoomedacf = gwy_data_view_get_field(GWY_DATA_VIEW(gui->dataview));
+    xy[0] += gwy_field_get_xoffset(zoomedacf);
+    xy[1] += gwy_field_get_yoffset(zoomedacf);
 
-    xl0 = gwy_data_field_get_xres(acf)/2;
-    yl0 = gwy_data_field_get_yres(acf)/2;
+    xl0 = gwy_field_get_xres(acf)/2;
+    yl0 = gwy_field_get_yres(acf)/2;
 
-    xofffull = gwy_data_field_get_xoffset(acf);
-    yofffull = gwy_data_field_get_yoffset(acf);
-    xl1 = floor(gwy_data_field_rtoj(acf, xy[0] - xofffull));
-    yl1 = floor(gwy_data_field_rtoi(acf, xy[1] - yofffull));
+    xofffull = gwy_field_get_xoffset(acf);
+    yofffull = gwy_field_get_yoffset(acf);
+    xl1 = floor(gwy_field_rtoj(acf, xy[0] - xofffull));
+    yl1 = floor(gwy_field_rtoi(acf, xy[1] - yofffull));
 
     if (!fixres) {
         lineres = GWY_ROUND(hypot(abs(xl0 - xl1) + 1, abs(yl0 - yl1) + 1));
@@ -576,7 +568,7 @@ update_curve(ModuleGUI *gui, gint i)
     else
         lineres = resolution;
 
-    gwy_data_field_get_profile(acf, gui->line, xl0, yl0, xl1, yl1, lineres, thickness, interpolation);
+    gwy_field_get_profile(acf, gui->line, xl0, yl0, xl1, yl1, lineres, thickness, interpolation);
 
     n = gwy_graph_model_get_n_curves(args->gmodel);
     if (i < n)
@@ -591,8 +583,8 @@ update_curve(ModuleGUI *gui, gint i)
         g_object_unref(gcmodel);
     }
 
-    gwy_graph_curve_model_set_data_from_dataline(gcmodel, gui->line, 0, 0);
-    desc = g_strdup_printf(_("ACF %.0f°"), 180.0/G_PI*atan2(-xy[1], xy[0]));
+    gwy_graph_curve_model_set_data_from_line(gcmodel, gui->line, 0, 0);
+    desc = g_strdup_printf(_("ACF %.0f°"), gwy_rad2deg(atan2(-xy[1], xy[0])));
     g_object_set(gcmodel, "description", desc, NULL);
     g_free(desc);
 }
@@ -602,27 +594,27 @@ calculate_zoomed_fields(ModuleGUI *gui)
 {
     ModuleArgs *args = gui->args;
     guint zoom = gwy_params_get_enum(args->params, PARAM_ZOOM);
-    GwyDataField *zoomed;
+    GwyField *zoomed;
 
     zoomed = cut_field_to_zoom(args->acf, zoom);
-    gwy_container_set_object(gui->data, gwy_app_get_data_key_for_id(0), zoomed);
-    gwy_data_field_data_changed(zoomed);
+    gwy_field_data_changed(zoomed);
+    gwy_data_view_set_field(GWY_DATA_VIEW(gui->dataview), zoomed);
     g_object_unref(zoomed);
 
     zoomed = cut_field_to_zoom(args->acfmask, zoom);
-    gwy_container_set_object(gui->data, gwy_app_get_mask_key_for_id(0), zoomed);
-    gwy_data_field_data_changed(zoomed);
+    gwy_field_data_changed(zoomed);
+    gwy_data_view_set_mask(GWY_DATA_VIEW(gui->dataview), zoomed);
     g_object_unref(zoomed);
 }
 
-static GwyDataField*
-cut_field_to_zoom(GwyDataField *field, gint zoom)
+static GwyField*
+cut_field_to_zoom(GwyField *field, gint zoom)
 {
-    GwyDataField *zoomed;
+    GwyField *zoomed;
     guint xres, yres, width, height;
 
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
     width = (xres/zoom) | 1;
     height = (yres/zoom) | 1;
     if (width < 17)
@@ -633,17 +625,17 @@ cut_field_to_zoom(GwyDataField *field, gint zoom)
     if (width >= xres && height >= yres)
         return g_object_ref(field);
 
-    zoomed = gwy_data_field_area_extract(field, (xres - width)/2, (yres - height)/2, width, height);
-    gwy_data_field_set_xoffset(zoomed, -0.5*gwy_data_field_get_xreal(zoomed));
-    gwy_data_field_set_yoffset(zoomed, -0.5*gwy_data_field_get_yreal(zoomed));
+    zoomed = gwy_field_area_extract(field, (xres - width)/2, (yres - height)/2, width, height);
+    gwy_field_set_xoffset(zoomed, -0.5*gwy_field_get_xreal(zoomed));
+    gwy_field_set_yoffset(zoomed, -0.5*gwy_field_get_yreal(zoomed));
 
     return zoomed;
 }
 
 /* Convert points to lines from the origin, which is assumed to be in the centre. */
 static void
-add_line_selection_from_points(GwyContainer *data,
-                               GwyDataField *field,
+add_line_selection_from_points(GwyFile *data,
+                               GwyField *field,
                                gint id,
                                GwySelection *pointsel)
 {
@@ -651,17 +643,15 @@ add_line_selection_from_points(GwyContainer *data,
     gint nsel, i;
     gdouble *seldata;
     gdouble xreal, yreal;
-    gchar *key;
 
     if (!pointsel || !(nsel = gwy_selection_get_data(pointsel, NULL)))
         return;
 
-    linesel = g_object_new(g_type_from_name("GwySelectionLine"), NULL);
-    g_return_if_fail(linesel);
+    linesel = gwy_selection_line_new();
     gwy_selection_set_max_objects(linesel, 1024);
     seldata = g_new(gdouble, 4*nsel);
-    xreal = gwy_data_field_get_xreal(field);
-    yreal = gwy_data_field_get_yreal(field);
+    xreal = gwy_field_get_xreal(field);
+    yreal = gwy_field_get_yreal(field);
 
     for (i = 0; i < nsel; i++) {
         seldata[4*i + 0] = 0.5*xreal;
@@ -672,8 +662,7 @@ add_line_selection_from_points(GwyContainer *data,
     gwy_selection_set_data(linesel, nsel, seldata);
     g_free(seldata);
 
-    key = g_strdup_printf("/%d/select/line", id);
-    gwy_container_pass_object_by_name(data, key, linesel);
+    gwy_container_pass_object(GWY_CONTAINER(data), gwy_file_key_image_selection(id, "line"), linesel);
 }
 
 static void
@@ -681,73 +670,64 @@ execute(ModuleArgs *args)
 {
     GwyParams *params = args->params;
     LevellingType level = gwy_params_get_enum(params, PARAM_LEVEL);
-    GwyDataField *mask = args->mask, *field = args->field, *acf = args->acf, *acfmask = args->acfmask, *field_for_acf;
+    GwyField *mask = args->mask, *field = args->field, *acf = args->acf, *acfmask = args->acfmask, *field_for_acf;
     GwyMaskingType masking = gwy_params_get_masking(params, PARAM_MASKING, &mask);
-    gint xres = gwy_data_field_get_xres(field);
-    gint yres = gwy_data_field_get_yres(field);
+    gint xres = gwy_field_get_xres(field);
+    gint yres = gwy_field_get_yres(field);
     gdouble a, bx, by;
 
-    /* Reuse acfmask for the field because we do not need it for gwy_data_field_area_2dacf_mask().  Reuse acf for the
+    /* Reuse acfmask for the field because we do not need it for gwy_field_area_2dacf().  Reuse acf for the
      * modified mask because we discard it immediately. */
     if (level == LEVELLING_MEAN_VALUE) {
-        gwy_data_field_resample(acfmask, xres, yres, GWY_INTERPOLATION_NONE);
-        gwy_data_field_copy(field, acfmask, TRUE);
-        a = gwy_data_field_area_get_avg_mask(acfmask, mask, masking, 0, 0, xres, yres);
-        gwy_data_field_add(acfmask, -a);
+        gwy_field_resize(acfmask, xres, yres);
+        gwy_field_assign(acfmask, field);
+        a = gwy_field_area_get_avg(acfmask, mask, masking, 0, 0, xres, yres);
+        gwy_field_add(acfmask, -a);
         field_for_acf = acfmask;
     }
     else if (level == LEVELLING_MEAN_PLANE) {
-        gwy_data_field_resample(acfmask, xres, yres, GWY_INTERPOLATION_NONE);
-        gwy_data_field_copy(field, acfmask, TRUE);
-        if (mask && masking == GWY_MASK_EXCLUDE) {
-            gwy_data_field_resample(acf, xres, yres, GWY_INTERPOLATION_NONE);
-            gwy_data_field_copy(mask, acf, FALSE);
-            gwy_data_field_area_fit_plane(acfmask, acf, 0, 0, xres, yres, &a, &bx, &by);
-        }
-        else if (mask && masking == GWY_MASK_INCLUDE)
-            gwy_data_field_area_fit_plane(acfmask, mask, 0, 0, xres, yres, &a, &bx, &by);
-        else
-            gwy_data_field_fit_plane(acfmask, &a, &bx, &by);
-
-        gwy_data_field_plane_level(acfmask, a, bx, by);
+        gwy_field_resize(acfmask, xres, yres);
+        gwy_field_assign(acfmask, field);
+        gwy_field_area_fit_plane(acfmask, mask, masking, 0, 0, xres, yres, &a, &bx, &by);
+        gwy_field_plane_level(acfmask, a, bx, by);
         field_for_acf = acfmask;
     }
     else
         field_for_acf = field;
 
-    gwy_data_field_area_2dacf_mask(field_for_acf, acf, mask, masking, 0, 0, xres, yres, 0, 0, NULL);
+    gwy_field_area_acf_2d(field_for_acf, mask, masking, acf, 0, 0, xres, yres, 0, 0, NULL);
     create_acf_mask(args);
 }
 
 static void
 create_acf_mask(ModuleArgs *args)
 {
-    GwyDataField *acf = args->acf;
-    GwyDataField *acfmask = args->acfmask;
+    GwyField *acf = args->acf;
+    GwyField *acfmask = args->acfmask;
     gdouble threshold = gwy_params_get_double(args->params, PARAM_THRESHOLD);
-    gint xres = gwy_data_field_get_xres(acf);
-    gint yres = gwy_data_field_get_yres(acf);
+    gint xres = gwy_field_get_xres(acf);
+    gint yres = gwy_field_get_yres(acf);
     gdouble sigma2;
 
-    sigma2 = gwy_data_field_get_val(acf, xres/2, yres/2);
-    gwy_data_field_resample(acfmask, xres, yres, GWY_INTERPOLATION_NONE);
-    gwy_data_field_copy(acf, acfmask, TRUE);
-    gwy_data_field_threshold(acfmask, threshold*sigma2, 0.0, 1.0);
-    gwy_data_field_grains_extract_grain(acfmask, xres/2, yres/2);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(acfmask), NULL);
+    sigma2 = gwy_field_get_val(acf, xres/2, yres/2);
+    gwy_field_resize(acfmask, xres, yres);
+    gwy_field_assign(acfmask, acf);
+    gwy_field_threshold(acfmask, threshold*sigma2, 0.0, 1.0);
+    gwy_field_grains_extract_grain(acfmask, xres/2, yres/2);
+    gwy_unit_clear(gwy_field_get_unit_z(acfmask));
 }
 
 /* XXX: This is not very efficient and probably does not do anything useful when the grain is highly non-convex. But
  * what exactly should be done in such case is anyone's guess... */
 static gboolean
-find_decays(GwyDataField *mask,
+find_decays(GwyField *mask,
             gdouble *r1, gdouble *r2, gdouble *phi1, gdouble *phi2)
 {
-    const gdouble *data = gwy_data_field_get_data_const(mask);
-    guint xres = gwy_data_field_get_xres(mask);
-    guint yres = gwy_data_field_get_yres(mask);
-    gdouble dx = gwy_data_field_get_dx(mask);
-    gdouble dy = gwy_data_field_get_dy(mask);
+    const gdouble *data = gwy_field_get_data_const(mask);
+    guint xres = gwy_field_get_xres(mask);
+    guint yres = gwy_field_get_yres(mask);
+    gdouble dx = gwy_field_get_dx(mask);
+    gdouble dy = gwy_field_get_dy(mask);
     gdouble phimin = 0.0, r2min = G_MAXDOUBLE;
     gdouble phimax = 0.0, r2max = 0.0;
     gdouble xc, yc;
