@@ -1,5 +1,5 @@
 /*
- *  $Id: angle_dist.c 24549 2021-11-26 17:00:39Z yeti-dn $
+ *  $Id: angle_dist.c 28821 2025-11-06 16:00:41Z yeti-dn $
  *  Copyright (C) 2004-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,14 +16,9 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <libprocess/level.h>
-#include <libprocess/filters.h>
-#include <libgwydgets/gwystock.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
 
@@ -37,8 +32,8 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *result;
+    GwyField *field;
+    GwyField *result;
 } ModuleArgs;
 
 typedef struct {
@@ -49,25 +44,25 @@ typedef struct {
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             angle_dist          (GwyContainer *data,
-                                             GwyRunType runtype);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static gboolean         execute             (ModuleArgs *args,
                                              GtkWindow *wait_window);
 static GwyDialogOutcome run_gui             (ModuleArgs *args);
 static void             param_changed       (ModuleGUI *gui,
                                              gint id);
-static void             compute_slopes      (GwyDataField *field,
+static void             compute_slopes      (GwyField *field,
                                              gint kernel_size,
-                                             GwyDataField *xder,
-                                             GwyDataField *yder);
-static gdouble          compute_max_der2    (GwyDataField *xder,
-                                             GwyDataField *yder);
-static gboolean         count_angles        (GwyDataField *xderfield,
-                                             GwyDataField *yderfield,
+                                             GwyField *xder,
+                                             GwyField *yder);
+static gdouble          compute_max_der2    (GwyField *xder,
+                                             GwyField *yder);
+static gboolean         count_angles        (GwyField *xderfield,
+                                             GwyField *yderfield,
                                              gint size,
                                              gulong *count,
                                              gint steps);
-static GwyDataField*    make_datafield      (GwyDataField *old,
+static GwyField*        make_field          (GwyField *old,
                                              gint res,
                                              gulong *count,
                                              gdouble real,
@@ -89,11 +84,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("angle_dist",
-                              (GwyProcessFunc)&angle_dist,
+                              module_main,
                               N_("/_Statistics/An_gle Distribution..."),
-                              GWY_STOCK_DISTRIBUTION_ANGLE,
+                              GWY_ICON_DISTRIBUTION_ANGLE,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Calculate two-dimensional angle distribution"));
 
     return TRUE;
@@ -118,36 +113,38 @@ define_module_params(void)
 }
 
 static void
-angle_dist(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
     gint oldid, newid;
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_DATA_FIELD_ID, &oldid,
-                                     NULL);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_FIELD_ID, &oldid,
+                                 NULL);
     g_return_if_fail(args.field);
     args.params = gwy_params_new_from_settings(define_module_params());
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
             goto end;
     }
     if (outcome != GWY_DIALOG_HAVE_RESULT) {
-        if (!execute(&args, gwy_app_find_window_for_channel(data, oldid)))
+        if (!execute(&args, gwy_data_browser_get_window_for_data(data, GWY_FILE_IMAGE, oldid)))
             goto end;
     }
 
-    newid = gwy_app_data_browser_add_data_field(args.result, data, TRUE);
-    gwy_app_sync_data_items(data, data, oldid, newid, FALSE,
-                            GWY_DATA_ITEM_PALETTE,
-                            0);
-    gwy_app_set_data_field_title(data, newid, _("Angle distribution"));
-    gwy_app_channel_log_add_proc(data, oldid, newid);
+    newid = gwy_file_add_image(data, args.result);
+
+    gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+    gwy_file_sync_items(data, GWY_FILE_IMAGE, oldid,
+                        data, GWY_FILE_IMAGE, newid,
+                        GWY_FILE_ITEM_PALETTE, FALSE);
+    gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _("Angle distribution"), TRUE);
+    gwy_log_add(data, GWY_FILE_IMAGE, oldid, newid);
     g_object_unref(args.result);
 
 end:
@@ -199,23 +196,23 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
     gint kernel_size = gwy_params_get_int(params, PARAM_KERNEL_SIZE);
     gboolean fit_plane = gwy_params_get_boolean(params, PARAM_FIT_PLANE);
     gboolean logscale = gwy_params_get_boolean(params, PARAM_LOGSCALE);
-    GwyDataField *field = args->field, *xder, *yder;
+    GwyField *field = args->field, *xder, *yder;
     gint xres, yres, n;
     gulong *count;
 
     gwy_app_wait_start(wait_window, _("Computing angle distribution..."));
 
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
 
     n = fit_plane ? kernel_size : 2;
     n = (xres - n)*(yres - n);
-    xder = gwy_data_field_new_alike(field, FALSE);
-    yder = gwy_data_field_new_alike(field, FALSE);
+    xder = gwy_field_new_alike(field, FALSE);
+    yder = gwy_field_new_alike(field, FALSE);
     compute_slopes(field, fit_plane ? kernel_size : 0, xder, yder);
     count = g_new0(gulong, size*size);
     if (count_angles(xder, yder, size, count, nsteps))
-        args->result = make_datafield(field, size, count, 2.0*G_PI, logscale);
+        args->result = make_field(field, size, count, 2.0*G_PI, logscale);
 
     g_free(count);
     g_object_unref(yder);
@@ -227,11 +224,11 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
 }
 
 static gdouble
-compute_max_der2(GwyDataField *xder, GwyDataField *yder)
+compute_max_der2(GwyField *xder, GwyField *yder)
 {
-    gint i, n = gwy_data_field_get_xres(xder)*gwy_data_field_get_yres(xder);
-    const gdouble *xd = gwy_data_field_get_data_const(xder);
-    const gdouble *yd = gwy_data_field_get_data_const(yder);
+    gint i, n = gwy_field_get_xres(xder)*gwy_field_get_yres(xder);
+    const gdouble *xd = gwy_field_get_data_const(xder);
+    const gdouble *yd = gwy_field_get_data_const(yder);
     gdouble m = 0.0;
 
     for (i = 0; i < n; i++)
@@ -241,26 +238,26 @@ compute_max_der2(GwyDataField *xder, GwyDataField *yder)
 }
 
 static void
-compute_slopes(GwyDataField *field,
+compute_slopes(GwyField *field,
                gint kernel_size,
-               GwyDataField *xder,
-               GwyDataField *yder)
+               GwyField *xder,
+               GwyField *yder)
 {
     GwyPlaneFitQuantity quantites[2] = { GWY_PLANE_FIT_BX, GWY_PLANE_FIT_BY };
-    GwyDataField *fields[2] = { xder, yder };
+    GwyField *fields[2] = { xder, yder };
 
     if (!kernel_size) {
-        gwy_data_field_filter_slope(field, xder, yder);
+        gwy_field_filter_slope(field, xder, yder);
         return;
     }
 
-    gwy_data_field_fit_local_planes(field, kernel_size, 2, quantites, fields);
-    gwy_data_field_multiply(xder, 1.0/gwy_data_field_get_dx(field));
-    gwy_data_field_multiply(yder, 1.0/gwy_data_field_get_dy(field));
+    gwy_field_fit_local_planes(field, kernel_size, 2, quantites, fields);
+    gwy_field_multiply(xder, 1.0/gwy_field_get_dx(field));
+    gwy_field_multiply(yder, 1.0/gwy_field_get_dy(field));
 }
 
 static gboolean
-count_angles(GwyDataField *xderfield, GwyDataField *yderfield,
+count_angles(GwyField *xderfield, GwyField *yderfield,
              gint size, gulong *count,
              gint steps)
 {
@@ -283,9 +280,9 @@ count_angles(GwyDataField *xderfield, GwyDataField *yderfield,
         st[j] = sin(theta);
     }
 
-    xder = gwy_data_field_get_data_const(xderfield);
-    yder = gwy_data_field_get_data_const(yderfield);
-    n = gwy_data_field_get_xres(xderfield)*gwy_data_field_get_yres(xderfield);
+    xder = gwy_field_get_data_const(xderfield);
+    yder = gwy_field_get_data_const(yderfield);
+    n = gwy_field_get_xres(xderfield)*gwy_field_get_yres(xderfield);
     for (i = 0; i < n; i++) {
         gdouble xd = xder[i], yd = yder[i];
 
@@ -313,28 +310,20 @@ count_angles(GwyDataField *xderfield, GwyDataField *yderfield,
     return ok;
 }
 
-static GwyDataField*
-make_datafield(G_GNUC_UNUSED GwyDataField *old,
-               gint res, gulong *count,
-               gdouble real, gboolean logscale)
+static GwyField*
+make_field(G_GNUC_UNUSED GwyField *old,
+           gint res, gulong *count,
+           gdouble real, gboolean logscale)
 {
-    GwyDataField *field;
-    GwySIUnit *unit;
+    GwyField *field;
     gdouble *d;
     gint i;
 
-    field = gwy_data_field_new(res, res, real, real, FALSE);
-    gwy_data_field_set_xoffset(field, -gwy_data_field_get_xreal(field)/2);
-    gwy_data_field_set_yoffset(field, -gwy_data_field_get_yreal(field)/2);
+    field = gwy_field_new(res, res, real, real, FALSE);
+    gwy_field_set_xoffset(field, -gwy_field_get_xreal(field)/2);
+    gwy_field_set_yoffset(field, -gwy_field_get_yreal(field)/2);
 
-    unit = gwy_si_unit_new(NULL);
-    gwy_data_field_set_si_unit_z(field, unit);
-    g_object_unref(unit);
-    unit = gwy_si_unit_new(NULL);
-    gwy_data_field_set_si_unit_xy(field, unit);
-    g_object_unref(unit);
-
-    d = gwy_data_field_get_data(field);
+    d = gwy_field_get_data(field);
     if (logscale) {
         for (i = 0; i < res*res; i++)
             d[i] = count[i] ? log((gdouble)count[i]) + 1.0 : 0.0;
