@@ -1,5 +1,5 @@
 /*
- *  $Id: arithmetic.c 26750 2024-10-18 15:41:48Z yeti-dn $
+ *  $Id: arithmetic.c 28906 2025-11-24 16:45:11Z yeti-dn $
  *  Copyright (C) 2003-2022 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,21 +16,11 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyexpr.h>
-#include <libprocess/stats.h>
-#include <libprocess/filters.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/correct.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwydgets/gwyradiobuttons.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
+#include "libgwyapp/sanity.h"
 #include "preview.h"
 
 #define RUN_MODES GWY_RUN_INTERACTIVE
@@ -80,13 +70,13 @@ enum {
     PARAM_IMAGE,
 };
 
-typedef GwyDataField* (*MakeFieldFunc)(GwyDataField *dfield);
+typedef GwyField* (*MakeFieldFunc)(GwyField *dfield);
 
 typedef struct {
     GwyParams *params;
     GtkListStore *history;
-    GwyDataField *result;
-    GwyDataField *mask;
+    GwyField *result;
+    GwyField *mask;
 } ModuleArgs;
 
 typedef struct {
@@ -109,14 +99,13 @@ typedef struct {
     GwyParamTable *table;
     GtkWidget *result;
     GSList *dataunits;
-    GwyContainer *data;
 } ModuleGUI;
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             arithmetic          (GwyContainer *data,
-                                             GwyRunType run);
-static GwyDialogOutcome run_gui             (GwyContainer *data,
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
+static GwyDialogOutcome run_gui             (GwyFile *data,
                                              gint id,
                                              ModuleArgs *args,
                                              EvaluationData *evdata);
@@ -140,12 +129,12 @@ static void             make_mask_if_needed (ModuleArgs *args);
 static void             need_data           (const EvaluationData *evdata,
                                              gboolean *nd);
 static void             update_history      (ModuleArgs *args);
-static void             fix_mask_field      (GwyDataField *mask,
+static void             fix_mask_field      (GwyField *mask,
                                              EvaluationData *evdata);
-static GwyDataField*    make_x              (GwyDataField *dfield);
-static GwyDataField*    make_y              (GwyDataField *dfield);
-static GwyDataField*    make_x_der          (GwyDataField *dfield);
-static GwyDataField*    make_y_der          (GwyDataField *dfield);
+static GwyField*        make_x              (GwyField *dfield);
+static GwyField*        make_y              (GwyField *dfield);
+static GwyField*        make_x_der          (GwyField *dfield);
+static GwyField*        make_y_der          (GwyField *dfield);
 static void             sanitise_params     (ModuleArgs *args);
 static GtkListStore*    load_history        (void);
 static void             save_history        (GtkListStore *history);
@@ -166,11 +155,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("arithmetic",
-                              (GwyProcessFunc)&arithmetic,
+                              module_main,
                               N_("/M_ultidata/_Arithmetic..."),
-                              GWY_STOCK_ARITHMETIC,
+                              GWY_ICON_ARITHMETIC,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Arithmetic operations on data"));
 
     return TRUE;
@@ -202,7 +191,7 @@ define_module_params(void)
 }
 
 static void
-arithmetic(GwyContainer *data, GwyRunType run)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
     EvaluationData evdata;
@@ -211,16 +200,16 @@ arithmetic(GwyContainer *data, GwyRunType run)
     gint newid;
     guint i;
 
-    g_return_if_fail(run & RUN_MODES);
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD_ID, &dataid.id,
-                                     GWY_APP_CONTAINER_ID, &dataid.datano,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_FIELD_ID, &dataid.id,
+                                 GWY_APP_CONTAINER_ID, &dataid.datano,
+                                 0);
     g_return_if_fail(gwy_app_data_id_verify_channel(&dataid));
     args.params = gwy_params_new_from_settings(define_module_params());
     gwy_params_set_image_id(args.params, PARAM_IMAGE, dataid);
     args.history = load_history();
-    args.result = gwy_data_field_new(PREVIEW_SIZE, PREVIEW_SIZE, 1.0, 1.0, TRUE);
+    args.result = gwy_field_new(PREVIEW_SIZE, PREVIEW_SIZE, 1.0, 1.0, TRUE);
     sanitise_params(&args);
 
     evdata.ok_masks = g_ptr_array_new();
@@ -245,20 +234,21 @@ arithmetic(GwyContainer *data, GwyRunType run)
     if (outcome != GWY_DIALOG_HAVE_RESULT)
         execute(&args, &evdata);
 
-    newid = gwy_app_data_browser_add_data_field(args.result, data, TRUE);
-    gwy_app_set_data_field_title(data, newid, _("Calculated"));
-    if (args.mask)
-        gwy_container_set_object(data, gwy_app_get_mask_key_for_id(newid), args.mask);
-    gwy_app_sync_data_items(gwy_app_data_browser_get(evdata.first.datano), data, evdata.first.id, newid, FALSE,
-                            GWY_DATA_ITEM_GRADIENT,
-                            GWY_DATA_ITEM_MASK_COLOR,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
-    gwy_app_channel_log_add_proc(data, -1, newid);
+    newid = gwy_file_add_image(data, args.result);
+
+    gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+    gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _("Calculated"), TRUE);
+    if (args.mask) {
+        gwy_file_set_image_mask(data, newid, args.mask);
+        gwy_file_sync_items(gwy_data_browser_get_file(evdata.first.datano), GWY_FILE_IMAGE, evdata.first.id,
+                            data, GWY_FILE_IMAGE, newid,
+                            GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_MASK_COLOR | GWY_FILE_ITEM_REAL_SQUARE, FALSE);
+    }
+    gwy_log_add(data, GWY_FILE_IMAGE, -1, newid);
 
 end:
-    GWY_OBJECT_UNREF(args.result);
-    GWY_OBJECT_UNREF(args.mask);
+    g_clear_object(&args.result);
+    g_clear_object(&args.mask);
     g_object_unref(args.params);
     g_object_unref(args.history);
     g_ptr_array_free(evdata.ok_masks, TRUE);
@@ -268,71 +258,61 @@ end:
 }
 
 static GwyDialogOutcome
-run_gui(GwyContainer *data, gint id,
+run_gui(GwyFile *data, gint id,
         ModuleArgs *args, EvaluationData *evdata)
 {
     gint dataunits = gwy_params_get_int(args->params, PARAM_DATAUNITS);
     const gchar *userunits = gwy_params_get_string(args->params, PARAM_USERUNITS);
-    GtkWidget *hbox, *hbox2, *table, *chooser, *entry, *label, *button;
+    GtkWidget *hbox, *hbox2, *grid, *chooser, *entry, *label, *button;
     ModuleGUI gui;
-    GwyDialogOutcome outcome;
     GwyDialog *dialog;
     guint i, row;
     gchar *s;
 
     gui.args = args;
     gui.evdata = evdata;
-    gui.data = gwy_container_new();
-    gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(0), args->result);
-    gwy_app_sync_data_items(data, gui.data, id, 0, FALSE,
-                            GWY_DATA_ITEM_GRADIENT,
-                            GWY_DATA_ITEM_MASK_COLOR,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
 
     gui.dialog = gwy_dialog_new(_("Arithmetic"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_UPDATE, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    gui.view = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, TRUE);
+    gui.view = gwy_create_preview(args->result, args->mask, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(gui.view), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_MASK_COLOR | GWY_FILE_ITEM_REAL_SQUARE);
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(gui.view), FALSE);
 
-    table = gtk_table_new(6 + NARGS, 3, FALSE);
-    gtk_table_set_row_spacings(GTK_TABLE(table), 2);
-    gtk_table_set_col_spacings(GTK_TABLE(table), 6);
-    gtk_container_set_border_width(GTK_CONTAINER(table), 4);
-    gtk_box_pack_start(GTK_BOX(hbox), table, TRUE, TRUE, 4);
+    grid = gtk_grid_new();
+    gtk_grid_set_row_spacing(GTK_GRID(grid), 2);
+    gtk_grid_set_column_spacing(GTK_GRID(grid), 6);
+    gtk_container_set_border_width(GTK_CONTAINER(grid), 4);
+    gtk_box_pack_start(GTK_BOX(hbox), grid, TRUE, TRUE, 4);
     row = 0;
 
     label = gtk_label_new_with_mnemonic(_("_Expression:"));
-    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
-    gtk_table_attach(GTK_TABLE(table), label, 0, 2, row, row+1, GTK_FILL, 0, 0, 0);
-    row++;
+    gtk_label_set_xalign(GTK_LABEL(label), 0.0);
+    gtk_grid_attach(GTK_GRID(grid), label, 0, row++, 2, 1);
 
     entry = gtk_combo_box_new_with_model_and_entry(GTK_TREE_MODEL(args->history));
     gtk_combo_box_set_entry_text_column(GTK_COMBO_BOX(entry), 0);
     gui.expression = entry;
     gtk_combo_box_set_active(GTK_COMBO_BOX(gui.expression), 0);
-    gtk_table_attach(GTK_TABLE(table), entry, 0, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
+    gtk_grid_attach(GTK_GRID(grid), entry, 0, row++, 3, 1);
     g_signal_connect_swapped(entry, "changed", G_CALLBACK(expr_changed), &gui);
     g_signal_connect_swapped(gtk_bin_get_child(GTK_BIN(entry)), "activate", G_CALLBACK(preview), &gui);
     gtk_label_set_mnemonic_widget(GTK_LABEL(label), entry);
-    row++;
 
     gui.result = label = gtk_label_new(NULL);
-    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
-    gtk_table_attach(GTK_TABLE(table), label, 0, 2, row, row+1, GTK_FILL, 0, 0, 0);
-    row++;
+    gtk_label_set_xalign(GTK_LABEL(label), 0.0);
+    gtk_grid_attach(GTK_GRID(grid), label, 0, row++, 3, 1);
 
-    gtk_table_set_row_spacing(GTK_TABLE(table), row-1, 8);
     label = gtk_label_new(_("Operands"));
-    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
-    gtk_table_attach(GTK_TABLE(table), label, 0, 2, row, row+1, GTK_FILL, 0, 0, 0);
+    gtk_label_set_xalign(GTK_LABEL(label), 0.0);
+    gwy_set_widget_padding(label, -1, -1, 8, -1);
+    gtk_grid_attach(GTK_GRID(grid), label, 0, row, 2, 1);
 
     label = gtk_label_new(_("Units"));
-    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
-    gtk_table_attach(GTK_TABLE(table), label, 2, 3, row, row+1, GTK_FILL, 0, 0, 0);
-    row++;
+    gtk_label_set_xalign(GTK_LABEL(label), 0.0);
+    gtk_grid_attach(GTK_GRID(grid), label, 2, row++, 1, 1);
 
     gui.dataunits = NULL;
     for (i = 0; i < NARGS; i++) {
@@ -342,15 +322,15 @@ run_gui(GwyContainer *data, gint id,
         s = g_strdup_printf("d_%d", i+1);
         label = gtk_label_new_with_mnemonic(evdata->name[i]);
         g_free(s);
-        gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
-        gtk_table_attach(GTK_TABLE(table), label, 0, 1, row, row+1, GTK_FILL, 0, 0, 0);
+        gtk_label_set_xalign(GTK_LABEL(label), 0.0);
+        gtk_grid_attach(GTK_GRID(grid), label, 0, row, 1, 1);
 
-        chooser = gwy_data_chooser_new_channels();
+        chooser = gwy_data_chooser_new(GWY_FILE_IMAGE);
         dataid = gwy_params_get_data_id(args->params, PARAM_IMAGE + i);
         gwy_data_chooser_set_active_id(GWY_DATA_CHOOSER(chooser), &dataid);
         g_signal_connect_swapped(chooser, "changed", G_CALLBACK(data_chosen), &gui);
         g_object_set_data(G_OBJECT(chooser), "index", GUINT_TO_POINTER(i));
-        gtk_table_attach(GTK_TABLE(table), chooser, 1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
+        gtk_grid_attach(GTK_GRID(grid), chooser, 1, row, 1, 1);
         gtk_label_set_mnemonic_widget(GTK_LABEL(label), chooser);
 
         button = gtk_radio_button_new(gui.dataunits);
@@ -359,14 +339,12 @@ run_gui(GwyContainer *data, gint id,
         s = g_strdup_printf(_("Take result units from data d%d"), i+1);
         gtk_widget_set_tooltip_text(button, s);
         g_free(s);
-        gtk_table_attach(GTK_TABLE(table), button, 2, 3, row, row+1, 0, 0, 0, 0);
+        gtk_grid_attach(GTK_GRID(grid), button, 2, row++, 1, 1);
         g_signal_connect_swapped(button, "clicked", G_CALLBACK(dataunits_selected), &gui);
-
-        row++;
     }
 
-    hbox2 = gtk_hbox_new(FALSE, 6);
-    gtk_table_attach(GTK_TABLE(table), hbox2, 0, 2, row, row+1, GTK_FILL, 0, 0, 0);
+    hbox2 = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 6);
+    gtk_grid_attach(GTK_GRID(grid), hbox2, 0, row, 2, 1);
 
     label = gtk_label_new_with_mnemonic(_("Specify un_its:"));
     gtk_box_pack_start(GTK_BOX(hbox2), label, FALSE, FALSE, 0);
@@ -384,28 +362,23 @@ run_gui(GwyContainer *data, gint id,
     gui.dataunits = gtk_radio_button_get_group(GTK_RADIO_BUTTON(button));
     gwy_radio_button_set_value(button, USER_UNITS_ID);
     gtk_widget_set_tooltip_text(button, _("Specify result units explicitly"));
-    gtk_table_attach(GTK_TABLE(table), button, 2, 3, row, row+1, 0, 0, 0, 0);
+    gtk_grid_attach(GTK_GRID(grid), button, 2, row++, 1, 1);
     g_signal_connect_swapped(button, "clicked", G_CALLBACK(dataunits_selected), &gui);
-    row++;
 
     gui.table = gwy_param_table_new(args->params);
     gwy_param_table_append_entry(gui.table, PARAM_FILLER_VALUE);
     gwy_param_table_add_enabler(gui.table, PARAM_FIXED_FILLER, PARAM_FILLER_VALUE);
     gwy_param_table_entry_set_width(gui.table, PARAM_FILLER_VALUE, 12);
-    gtk_table_attach(GTK_TABLE(table), gwy_param_table_widget(gui.table), 0, 2, row, row+1, GTK_FILL, 0, 0, 0);
+    gtk_grid_attach(GTK_GRID(grid), gwy_param_table_widget(gui.table), 0, row++, 3, 1);
     gwy_dialog_add_param_table(dialog, gui.table);
-    row++;
 
     g_signal_connect_swapped(gui.table, "param-changed", G_CALLBACK(param_changed), &gui);
     gwy_dialog_set_preview_func(dialog, GWY_PREVIEW_UPON_REQUEST, preview, &gui, NULL);
     gtk_widget_grab_focus(gui.expression);
     gwy_radio_buttons_set_current(gui.dataunits, dataunits);
     expr_changed(&gui, gui.expression); /* resolves variables */
-    outcome = gwy_dialog_run(dialog);
 
-    g_object_unref(gui.data);
-
-    return outcome;
+    return gwy_dialog_run(dialog);
 }
 
 static void
@@ -453,7 +426,7 @@ expr_changed(ModuleGUI *gui, GtkWidget *entry)
         g_return_if_fail(nvars);
         if (nvars == 1) {
             gdouble v = gwy_expr_execute(evdata->expr, NULL);
-            message = s = g_strdup_printf("%g", v);
+            message = s = g_strdup_printf("%.8g", v);
             evdata->err = ARITHMETIC_NUMERIC;
         }
         else {
@@ -507,33 +480,24 @@ show_state(ModuleGUI *gui, const gchar *message)
     GtkDialog *dialog = GTK_DIALOG(gui->dialog);
     ModuleArgs *args = gui->args;
     EvaluationData *evdata = gui->evdata;
-    gboolean ok;
 
     if (!message && evdata->err != ARITHMETIC_NUMERIC) {
         message = check_fields(args, evdata);
-        gtk_label_set_text(GTK_LABEL(gui->result), evdata->err ? message : NULL);
+        if (!evdata->err)
+            message = NULL;
     }
-    else {
-        if (message)
-            gtk_label_set_text(GTK_LABEL(gui->result), message);
-    }
-
-    ok = (evdata->err == ARITHMETIC_OK);
+    gboolean ok = (evdata->err == ARITHMETIC_OK || evdata->err == ARITHMETIC_NUMERIC);
     gtk_dialog_set_response_sensitive(dialog, GTK_RESPONSE_OK, ok);
     gtk_dialog_set_response_sensitive(dialog, GWY_RESPONSE_UPDATE, ok);
-
-    if (ok)
-        set_widget_as_ok_message(gui->result);
-    else
-        set_widget_as_error_message(gui->result);
+    gwy_set_message_label(GTK_LABEL(gui->result), message, ok ? GTK_MESSAGE_INFO : GTK_MESSAGE_ERROR, FALSE);
 }
 
 static const gchar*
 check_fields(ModuleArgs *args, EvaluationData *evdata)
 {
     guint first = 0, i;
-    GwyDataField *dfirst, *dfield;
-    GwyDataCompatibilityFlags diff;
+    GwyField *dfirst, *dfield;
+    GwyDataMismatchFlags diff;
     gboolean nd[NARGS];
 
     if (evdata->err & (ARITHMETIC_EXPR | ARITHMETIC_NUMERIC))
@@ -552,24 +516,24 @@ check_fields(ModuleArgs *args, EvaluationData *evdata)
         return NULL;
     }
 
-    /* each window must match with first, this is transitive */
+    /* Each image must match the first. This is transitive. */
     dfirst = gwy_params_get_image(args->params, PARAM_IMAGE + first);
     for (i = first+1; i < NARGS; i++) {
         if (!nd[i])
             continue;
 
         dfield = gwy_params_get_image(args->params, PARAM_IMAGE + i);
-        diff = gwy_data_field_check_compatibility(dfirst, dfield,
-                                                  GWY_DATA_COMPATIBILITY_RES
-                                                  | GWY_DATA_COMPATIBILITY_REAL
-                                                  | GWY_DATA_COMPATIBILITY_LATERAL);
+        diff = gwy_field_is_incompatible(dfirst, dfield,
+                                         GWY_DATA_MISMATCH_RES
+                                         | GWY_DATA_MISMATCH_REAL
+                                         | GWY_DATA_MISMATCH_LATERAL);
         if (diff) {
             evdata->err |= ARITHMETIC_DATA;
-            if (diff & GWY_DATA_COMPATIBILITY_RES)
+            if (diff & GWY_DATA_MISMATCH_RES)
                 return _("Pixel dimensions differ");
-            if (diff & GWY_DATA_COMPATIBILITY_LATERAL)
+            if (diff & GWY_DATA_MISMATCH_LATERAL)
                 return _("Lateral dimensions are different physical quantities");
-            if (diff & GWY_DATA_COMPATIBILITY_REAL)
+            if (diff & GWY_DATA_MISMATCH_REAL)
                 return _("Physical dimensions differ");
         }
     }
@@ -591,36 +555,32 @@ preview(gpointer user_data)
     execute(args, gui->evdata);
     g_return_if_fail(args->result);
 
-    gwy_data_field_data_changed(args->result);
-    if (args->mask)
-        gwy_container_set_object(gui->data, gwy_app_get_mask_key_for_id(0), args->mask);
-    else
-        gwy_container_remove(gui->data, gwy_app_get_mask_key_for_id(0));
-
+    gwy_field_data_changed(args->result);
+    gwy_data_view_set_mask(GWY_DATA_VIEW(gui->view), args->mask);
     gwy_set_data_preview_size(GWY_DATA_VIEW(gui->view), PREVIEW_SIZE);
     gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
 }
 
 static void
-update_field(GwyDataField *src, GwyDataField *target, gint *n, gdouble **tdata)
+update_field(GwyField *src, GwyField *target, gint *n, gdouble **tdata)
 {
-    gint xres = gwy_data_field_get_xres(src), yres = gwy_data_field_get_yres(src);
+    gint xres = gwy_field_get_xres(src), yres = gwy_field_get_yres(src);
 
-    gwy_data_field_resample(target, xres, yres, GWY_INTERPOLATION_NONE);
-    gwy_data_field_copy_units(src, target);
-    gwy_data_field_set_xreal(target, gwy_data_field_get_xreal(src));
-    gwy_data_field_set_yreal(target, gwy_data_field_get_yreal(src));
-    gwy_data_field_set_xoffset(target, gwy_data_field_get_xoffset(src));
-    gwy_data_field_set_yoffset(target, gwy_data_field_get_yoffset(src));
+    gwy_field_resize(target, xres, yres);
+    gwy_field_copy_units(src, target);
+    gwy_field_set_xreal(target, gwy_field_get_xreal(src));
+    gwy_field_set_yreal(target, gwy_field_get_yreal(src));
+    gwy_field_set_xoffset(target, gwy_field_get_xoffset(src));
+    gwy_field_set_yoffset(target, gwy_field_get_yoffset(src));
     *n = xres*yres;
-    *tdata = gwy_data_field_get_data(target);
+    *tdata = gwy_field_get_data(target);
 }
 
 static void
 execute(ModuleArgs *args, EvaluationData *evdata)
 {
     static MakeFieldFunc derivers[ARITHMETIC_NVARS] = { NULL, NULL, make_x_der, make_y_der, };
-    GwyDataField **data_fields, *dfield, *mfield, *empty_mask = NULL; /* A zero filled mask. */
+    GwyField **fields, *dfield, *mfield, *empty_mask = NULL; /* A zero filled mask. */
     const gdouble **d;
     gboolean nd[NARGS];
     gdouble *r = NULL;
@@ -631,9 +591,9 @@ execute(ModuleArgs *args, EvaluationData *evdata)
 
     need_data(evdata, nd);
     /* We know the expression can't contain more variables */
-    data_fields = g_new0(GwyDataField*, ARITHMETIC_NARGS);
+    fields = g_new0(GwyField*, ARITHMETIC_NARGS);
     d = g_new0(const gdouble*, ARITHMETIC_NARGS + 1);
-    GWY_OBJECT_UNREF(args->mask);
+    g_clear_object(&args->mask);
 
     /* First get all the data fields we directly have */
     for (i = 0; i < NARGS; i++) {
@@ -641,8 +601,8 @@ execute(ModuleArgs *args, EvaluationData *evdata)
         if (!nd[i])
             continue;
 
-        dfield = data_fields[i] = gwy_params_get_image(args->params, PARAM_IMAGE + i);
-        d[evdata->pos[i]] = gwy_data_field_get_data_const(dfield);
+        dfield = fields[i] = gwy_params_get_image(args->params, PARAM_IMAGE + i);
+        d[evdata->pos[i]] = gwy_field_get_data_const(dfield);
         gwy_debug("d[%u] set to PRIMARY %u", evdata->pos[i], i);
         if (first) {
             first = FALSE;
@@ -663,14 +623,14 @@ execute(ModuleArgs *args, EvaluationData *evdata)
             update_field(dfield, args->result, &n, &r);
             evdata->first = gwy_params_get_data_id(args->params, PARAM_IMAGE + i);
         }
-        if (!(mfield = gwy_params_get_mask(args->params, PARAM_IMAGE + i % NARGS))) {
+        if (!(mfield = gwy_params_get_image_mask(args->params, PARAM_IMAGE + i % NARGS))) {
             if (!empty_mask)
-                empty_mask = gwy_data_field_new_alike(dfield, TRUE);
+                empty_mask = gwy_field_new_alike(dfield, TRUE);
             mfield = empty_mask;
         }
         else
             fix_mask_field(mfield, evdata);
-        d[evdata->pos[i]] = gwy_data_field_get_data_const(mfield);
+        d[evdata->pos[i]] = gwy_field_get_data_const(mfield);
     }
 
     /* Derive derived fields */
@@ -678,47 +638,47 @@ execute(ModuleArgs *args, EvaluationData *evdata)
         if (!evdata->pos[i])
             continue;
 
-        g_return_if_fail(data_fields[i % NARGS]);
-        dfield = derivers[i/NARGS](data_fields[i % NARGS]);
+        g_return_if_fail(fields[i % NARGS]);
+        dfield = derivers[i/NARGS](fields[i % NARGS]);
         gwy_debug("d[%u] set to DERIVED %u (type %u)", evdata->pos[i], i/NARGS, i%NARGS);
-        data_fields[i] = dfield;
-        d[evdata->pos[i]] = gwy_data_field_get_data_const(dfield);
+        fields[i] = dfield;
+        d[evdata->pos[i]] = gwy_field_get_data_const(dfield);
     }
 
     i = NARGS*ARITHMETIC_NVARS + COMMON_COORD_X;
     if (evdata->pos[i]) {
-        dfield = make_x(data_fields[0]);
-        data_fields[i] = dfield;
-        d[evdata->pos[i]] = gwy_data_field_get_data_const(dfield);
+        dfield = make_x(fields[0]);
+        fields[i] = dfield;
+        d[evdata->pos[i]] = gwy_field_get_data_const(dfield);
     }
 
     i = NARGS*ARITHMETIC_NVARS + COMMON_COORD_Y;
     if (evdata->pos[i]) {
-        dfield = make_y(data_fields[0]);
-        data_fields[i] = dfield;
-        d[evdata->pos[i]] = gwy_data_field_get_data_const(dfield);
+        dfield = make_y(fields[0]);
+        fields[i] = dfield;
+        d[evdata->pos[i]] = gwy_field_get_data_const(dfield);
     }
 
     /* Execute */
     gwy_expr_vector_execute(evdata->expr, n, d, r);
-    gwy_data_field_invalidate(args->result);
+    gwy_field_invalidate(args->result);
 
     /* Set units. */
     dataunits = gwy_params_get_int(args->params, PARAM_DATAUNITS);
     if (dataunits == USER_UNITS_ID) {
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(args->result),
-                                    gwy_params_get_string(args->params, PARAM_USERUNITS));
+        gwy_unit_set_from_string(gwy_field_get_unit_z(args->result),
+                                 gwy_params_get_string(args->params, PARAM_USERUNITS));
     }
     else
-        gwy_data_field_copy_units(gwy_params_get_image(args->params, PARAM_IMAGE + dataunits), args->result);
+        gwy_field_copy_units(gwy_params_get_image(args->params, PARAM_IMAGE + dataunits), args->result);
 
     /* Free stuff */
-    GWY_OBJECT_UNREF(empty_mask);
+    g_clear_object(&empty_mask);
     for (i = NARGS; i < ARITHMETIC_NARGS; i++) {
-        if (data_fields[i])
-            g_object_unref(data_fields[i]);
+        if (fields[i])
+            g_object_unref(fields[i]);
     }
-    g_free(data_fields);
+    g_free(fields);
     g_free(d);
 
     make_mask_if_needed(args);
@@ -727,20 +687,20 @@ execute(ModuleArgs *args, EvaluationData *evdata)
 static void
 make_mask_if_needed(ModuleArgs *args)
 {
-    GwyDataField *result = args->result, *mask;
+    GwyField *result = args->result, *mask;
 
     args->mask = mask = gwy_app_channel_mask_of_nans(result, FALSE);
     if (!mask)
         return;
 
     if (gwy_params_get_boolean(args->params, PARAM_FIXED_FILLER)) {
-        gint xres = gwy_data_field_get_xres(result);
-        gint yres = gwy_data_field_get_yres(result);
+        gint xres = gwy_field_get_xres(result);
+        gint yres = gwy_field_get_yres(result);
         gdouble filler_value = gwy_params_get_double(args->params, PARAM_FILLER_VALUE);
-        gwy_data_field_area_fill_mask(result, mask, GWY_MASK_INCLUDE, 0, 0, xres, yres, filler_value);
+        gwy_field_area_fill(result, mask, GWY_MASK_INCLUDE, 0, 0, xres, yres, filler_value);
     }
     else
-        gwy_data_field_laplace_solve(result, mask, -1, 0.25);
+        gwy_field_laplace_solve(result, mask, -1, 0.25);
 }
 
 /* Find which data we need, for derivatives or otherwise */
@@ -766,7 +726,7 @@ need_data(const EvaluationData *evdata, gboolean *nd)
 }
 
 static void
-fix_mask_field(GwyDataField *mask, EvaluationData *evdata)
+fix_mask_field(GwyField *mask, EvaluationData *evdata)
 {
     gdouble min, max;
     guint i;
@@ -779,21 +739,21 @@ fix_mask_field(GwyDataField *mask, EvaluationData *evdata)
 
     /* Silently normalise the mask values if they are outside [0, 1]. We do not actually like any value different from
      * 0 and 1, but let them pass. */
-    gwy_data_field_get_min_max(mask, &min, &max);
+    gwy_field_get_min_max(mask, &min, &max);
     if (min < 0.0 || max > 1.0)
-        gwy_data_field_clamp(mask, 0.0, 1.0);
+        gwy_field_clamp(mask, 0.0, 1.0);
 
     g_ptr_array_add(evdata->ok_masks, mask);
 }
 
-static GwyDataField*
-make_x(GwyDataField *dfield)
+static GwyField*
+make_x(GwyField *dfield)
 {
-    gint xres = gwy_data_field_get_xres(dfield), yres = gwy_data_field_get_yres(dfield);
-    GwyDataField *result = gwy_data_field_new_alike(dfield, FALSE);
-    gdouble dx = gwy_data_field_get_dx(dfield);
-    gdouble xoff = gwy_data_field_get_xoffset(dfield);
-    gdouble *data = gwy_data_field_get_data(result);
+    gint xres = gwy_field_get_xres(dfield), yres = gwy_field_get_yres(dfield);
+    GwyField *result = gwy_field_new_alike(dfield, FALSE);
+    gdouble dx = gwy_field_get_dx(dfield);
+    gdouble xoff = gwy_field_get_xoffset(dfield);
+    gdouble *data = gwy_field_get_data(result);
     gint i;
 
     gwy_math_linspace(data, xres, xoff + 0.5*dx, dx);
@@ -803,14 +763,14 @@ make_x(GwyDataField *dfield)
     return result;
 }
 
-static GwyDataField*
-make_y(GwyDataField *dfield)
+static GwyField*
+make_y(GwyField *dfield)
 {
-    gint xres = gwy_data_field_get_xres(dfield), yres = gwy_data_field_get_yres(dfield);
-    GwyDataField *result = gwy_data_field_new_alike(dfield, FALSE);
-    gdouble dy = gwy_data_field_get_dy(dfield);
-    gdouble yoff = gwy_data_field_get_yoffset(dfield);
-    gdouble *data = gwy_data_field_get_data(result);
+    gint xres = gwy_field_get_xres(dfield), yres = gwy_field_get_yres(dfield);
+    GwyField *result = gwy_field_new_alike(dfield, FALSE);
+    gdouble dy = gwy_field_get_dy(dfield);
+    gdouble yoff = gwy_field_get_yoffset(dfield);
+    gdouble *data = gwy_field_get_data(result);
     gint i, j;
 
     for (i = 0; i < yres; i++) {
@@ -823,18 +783,18 @@ make_y(GwyDataField *dfield)
     return result;
 }
 
-static GwyDataField*
-make_x_der(GwyDataField *dfield)
+static GwyField*
+make_x_der(GwyField *dfield)
 {
-    gint xres = gwy_data_field_get_xres(dfield), yres = gwy_data_field_get_yres(dfield);
-    GwyDataField *result = gwy_data_field_new_alike(dfield, FALSE);
-    gdouble h = 2.0*gwy_data_field_get_dx(dfield);
-    const gdouble *drow, *d = gwy_data_field_get_data_const(dfield);
-    gdouble *rrow, *r = gwy_data_field_get_data(result);
+    gint xres = gwy_field_get_xres(dfield), yres = gwy_field_get_yres(dfield);
+    GwyField *result = gwy_field_new_alike(dfield, FALSE);
+    gdouble h = 2.0*gwy_field_get_dx(dfield);
+    const gdouble *drow, *d = gwy_field_get_data_const(dfield);
+    gdouble *rrow, *r = gwy_field_get_data(result);
     gint i, j;
 
     if (xres < 2) {
-        gwy_data_field_clear(result);
+        gwy_field_clear(result);
         return result;
     }
 
@@ -850,18 +810,18 @@ make_x_der(GwyDataField *dfield)
     return result;
 }
 
-static GwyDataField*
-make_y_der(GwyDataField *dfield)
+static GwyField*
+make_y_der(GwyField *dfield)
 {
-    gint xres = gwy_data_field_get_xres(dfield), yres = gwy_data_field_get_yres(dfield);
-    GwyDataField *result = gwy_data_field_new_alike(dfield, FALSE);
-    gdouble h = 2.0*gwy_data_field_get_dy(dfield);
-    const gdouble *drow, *drowp, *drowm, *d = gwy_data_field_get_data_const(dfield);
-    gdouble *rrow, *r = gwy_data_field_get_data(result);
+    gint xres = gwy_field_get_xres(dfield), yres = gwy_field_get_yres(dfield);
+    GwyField *result = gwy_field_new_alike(dfield, FALSE);
+    gdouble h = 2.0*gwy_field_get_dy(dfield);
+    const gdouble *drow, *drowp, *drowm, *d = gwy_field_get_data_const(dfield);
+    gdouble *rrow, *r = gwy_field_get_data(result);
     guint i, j;
 
     if (yres < 2) {
-        gwy_data_field_clear(result);
+        gwy_field_clear(result);
         return result;
     }
 
