@@ -1,5 +1,5 @@
 /*
- *  $Id: blockstep.c 26087 2024-01-02 16:32:16Z yeti-dn $
+ *  $Id: blockstep.c 29061 2026-01-02 15:01:53Z yeti-dn $
  *  Copyright (C) 2003-2023 David Necas (Yeti), Petr Klapetek, Luke Somers.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net, lsomers@sas.upenn.edu.
  *
@@ -16,17 +16,9 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwythreads.h>
-#include <libprocess/linestats.h>
-#include <libprocess/stats.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwydgets/gwydgetutils.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
+#include <gwy.h>
 
 #include "preview.h"
 
@@ -68,25 +60,25 @@ typedef struct {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *result;
-    GwyDataField *mask;
+    GwyField *field;
+    GwyField *result;
+    GwyField *mask;
 } ModuleArgs;
 
 typedef struct {
     ModuleArgs *args;
     GtkWidget *dialog;
+    GtkWidget *dataview;
     GwyParamTable *table;
-    GwyContainer *data;
 } ModuleGUI;
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             block_correct_step  (GwyContainer *data,
-                                             GwyRunType run);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static gint             execute             (ModuleArgs *args);
 static GwyDialogOutcome run_gui             (ModuleArgs *args,
-                                             GwyContainer *data,
+                                             GwyFile *data,
                                              gint id);
 static void             param_changed       (ModuleGUI *gui,
                                              gint id);
@@ -108,11 +100,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("block_correct_step",
-                              (GwyProcessFunc)&block_correct_step,
+                              module_main,
                               N_("/_Correct Data/Ste_p Block Correction"),
                               NULL,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Correct steps without any line correction"));
 
     return TRUE;
@@ -141,34 +133,33 @@ define_module_params(void)
     gwy_param_def_add_gwyenum(paramdef, PARAM_SCANDIR, "scandir", _("Scanning direction"),
                               scandirs, G_N_ELEMENTS(scandirs), SCANDIR_LTR);
     gwy_param_def_add_instant_updates(paramdef, PARAM_UPDATE, "update", NULL, TRUE);
-    gwy_param_def_add_gwyenum(paramdef, PARAM_DISPLAY, "display", gwy_sgettext("verb|Display"),
+    gwy_param_def_add_gwyenum(paramdef, PARAM_DISPLAY, "display", C_("verb", "Display"),
                               displays, G_N_ELEMENTS(displays), PREVIEW_CORRECTED);
     gwy_param_def_add_mask_color(paramdef, PARAM_MASK_COLOR, NULL, NULL);
     return paramdef;
 }
 
 static void
-block_correct_step(GwyContainer *data,
-                   GwyRunType run)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     ModuleArgs args;
     GQuark dquark;
     gint id;
 
-    g_return_if_fail(run & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_DATA_FIELD_KEY, &dquark,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_FIELD_KEY, &dquark,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(args.field && dquark);
 
-    args.result = gwy_data_field_duplicate(args.field);
-    args.mask = gwy_data_field_new_alike(args.field, TRUE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(args.mask), NULL);
+    args.result = gwy_field_copy(args.field);
+    args.mask = gwy_field_new_alike(args.field, TRUE);
+    gwy_unit_clear(gwy_field_get_unit_z(args.mask));
     args.params = gwy_params_new_from_settings(define_module_params());
 
-    if (run == GWY_RUN_INTERACTIVE) {
+    if (mode == GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, id);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -177,10 +168,10 @@ block_correct_step(GwyContainer *data,
     if (outcome != GWY_DIALOG_HAVE_RESULT)
         execute(&args);
 
-    gwy_app_undo_qcheckpointv(data, 1, &dquark);
-    gwy_data_field_assign(args.field, args.result);
-    gwy_data_field_data_changed(args.field);
-    gwy_app_channel_log_add_proc(data, id, id);
+    gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &dquark);
+    gwy_field_assign(args.field, args.result);
+    gwy_field_data_changed(args.field);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, id);
 
 end:
     g_object_unref(args.mask);
@@ -189,29 +180,23 @@ end:
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     GtkWidget *hbox, *dataview;
-    GwyDialogOutcome outcome;
     GwyParamTable *table;
     GwyDialog *dialog;
     ModuleGUI gui;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
-    gui.data = gwy_container_new();
-    gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(0), args->result);
-    gwy_app_sync_data_items(data, gui.data, id, 0, FALSE,
-                            GWY_DATA_ITEM_PALETTE,
-                            GWY_DATA_ITEM_MASK_COLOR,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
 
     gui.dialog = gwy_dialog_new(_("Block Step Correction"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_UPDATE, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, TRUE);
+    dataview = gwy_create_preview(args->result, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_MASK_COLOR | GWY_FILE_ITEM_REAL_SQUARE);
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), FALSE);
 
     table = gui.table = gwy_param_table_new(args->params);
@@ -224,7 +209,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_param_table_append_separator(table);
     gwy_param_table_append_radio(table, PARAM_DISPLAY);
     gwy_param_table_append_separator(table);
-    gwy_param_table_append_mask_color(table, PARAM_MASK_COLOR, gui.data, 0, NULL, -1);
+    gwy_param_table_append_image_mask_color(table, PARAM_MASK_COLOR, GWY_DATA_VIEW(dataview), NULL, -1);
 
     gtk_box_pack_start(GTK_BOX(hbox), gwy_param_table_widget(table), TRUE, TRUE, 0);
     gwy_dialog_add_param_table(dialog, table);
@@ -232,11 +217,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     g_signal_connect_swapped(table, "param-changed", G_CALLBACK(param_changed), &gui);
     gwy_dialog_set_preview_func(dialog, GWY_PREVIEW_IMMEDIATE, preview, &gui, NULL);
 
-    outcome = gwy_dialog_run(dialog);
-
-    g_object_unref(gui.data);
-
-    return outcome;
+    return gwy_dialog_run(dialog);
 }
 
 static void
@@ -247,14 +228,15 @@ param_changed(ModuleGUI *gui, gint id)
 
     if (id < 0 || id == PARAM_DISPLAY) {
         PreviewType ptype = gwy_params_get_enum(params, PARAM_DISPLAY);
+        GwyDataView *dataview = GWY_DATA_VIEW(gui->dataview);
 
         if (ptype == PREVIEW_CORRECTED) {
-            gwy_container_set_object(gui->data, gwy_app_get_data_key_for_id(0), args->result);
-            gwy_container_remove(gui->data, gwy_app_get_mask_key_for_id(0));
+            gwy_data_view_set_field(dataview, args->result);
+            gwy_data_view_set_mask(dataview, NULL);
         }
         else {
-            gwy_container_set_object(gui->data, gwy_app_get_data_key_for_id(0), args->field);
-            gwy_container_set_object(gui->data, gwy_app_get_mask_key_for_id(0), args->mask);
+            gwy_data_view_set_field(dataview, args->field);
+            gwy_data_view_set_mask(dataview, args->mask);
         }
     }
 
@@ -271,8 +253,8 @@ preview(gpointer user_data)
     guint n;
 
     n = execute(args);
-    gwy_data_field_data_changed(args->result);
-    gwy_data_field_data_changed(args->mask);
+    gwy_field_data_changed(args->result);
+    gwy_field_data_changed(args->mask);
     gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
 
     s = g_strdup_printf("%u", n);
@@ -281,11 +263,11 @@ preview(gpointer user_data)
 }
 
 static LineSplit*
-mark_discontinuities(GwyDataField *field, GwyDataField *mask,
+mark_discontinuities(GwyField *field, GwyField *mask,
                      gdouble threshold, ScanDirection scandir)
 {
-    const gdouble *d = gwy_data_field_get_data_const(field);
-    gint i, xres = gwy_data_field_get_xres(field), yres = gwy_data_field_get_yres(field);
+    const gdouble *d = gwy_field_get_data_const(field);
+    gint i, xres = gwy_field_get_xres(field), yres = gwy_field_get_yres(field);
     gint *imask;
     LineSplit *ls;
 
@@ -371,7 +353,7 @@ mark_discontinuities(GwyDataField *field, GwyDataField *mask,
     }
 
     if (mask) {
-        gdouble *m = gwy_data_field_get_data(mask);
+        gdouble *m = gwy_field_get_data(mask);
 
         for (i = 0; i < xres*yres - xres; i++)
             m[i] = fmax(imask[i], imask[i + xres]);
@@ -401,13 +383,13 @@ process_one_step_segment(const gdouble *row, gdouble *mrow, gdouble *shifts,
 }
 
 static GArray*
-construct_blocks(GwyDataField *field, GwyDataField *mask,
+construct_blocks(GwyField *field, GwyField *mask,
                  const LineSplit *ls, ScanDirection scandir)
 {
-    gint xres = gwy_data_field_get_xres(field), yres = gwy_data_field_get_yres(field);
+    gint xres = gwy_field_get_xres(field), yres = gwy_field_get_yres(field);
     gint i, minlength = (gint)(3*xres/4);
-    const gdouble *d = gwy_data_field_get_data_const(field);
-    gdouble *m = (mask ? gwy_data_field_get_data(mask) : NULL);
+    const gdouble *d = gwy_field_get_data_const(field);
+    gdouble *m = (mask ? gwy_field_get_data(mask) : NULL);
     GArray *blocksteps;
     gdouble *shifts;
     guint k;
@@ -458,7 +440,7 @@ construct_blocks(GwyDataField *field, GwyDataField *mask,
 
     shifts = g_new(gdouble, xres);
     if (mask)
-        gwy_data_field_clear(mask);
+        gwy_field_clear(mask);
 
     for (k = 0; k < blocksteps->len; k++) {
         BlockStep *bs = &g_array_index(blocksteps, BlockStep, k);
@@ -476,7 +458,7 @@ construct_blocks(GwyDataField *field, GwyDataField *mask,
         else
             process_one_step_segment(row, mrow, shifts, xres, 0, fromleft);
 
-        bs->shift = gwy_math_trimmed_mean(xres, shifts, xres/4, xres/4);
+        bs->shift = gwy_math_trimmed_mean(shifts, xres, xres/4, xres/4);
         /* We kind of have not decided on the definition of vertical position… */
         bs->i--;
 
@@ -502,15 +484,15 @@ construct_blocks(GwyDataField *field, GwyDataField *mask,
 }
 
 static void
-apply_correction(GwyDataField *field, const BlockStep *bs, ScanDirection scandir)
+apply_correction(GwyField *field, const BlockStep *bs, ScanDirection scandir)
 {
     gdouble *d, *row;
     gdouble shift = 0.0;
     gint i, j, xres, yres;
 
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
-    d = gwy_data_field_get_data(field);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
+    d = gwy_field_get_data(field);
 
     for (i = bs->i; i < yres; i++) {
         row = d + i*xres;
@@ -543,27 +525,27 @@ apply_correction(GwyDataField *field, const BlockStep *bs, ScanDirection scandir
 static gint
 execute(ModuleArgs *args)
 {
-    GwyDataField *field = args->field, *result = args->result, *mask = args->mask;
+    GwyField *field = args->field, *result = args->result, *mask = args->mask;
     gdouble threshold = gwy_params_get_double(args->params, PARAM_THRESHOLD);
     ScanDirection scandir = gwy_params_get_enum(args->params, PARAM_SCANDIR);
     PreviewType display = gwy_params_get_enum(args->params, PARAM_DISPLAY);
-    GwyDataLine *rmsline;
+    GwyLine *rmsline;
     LineSplit *ls;
     GArray *blocksteps;
     guint nblocks;
     gdouble rms;
 
     /* Get root square mean vertical difference (pixelwise, tan β₀ is in real units). */
-    rmsline = gwy_data_line_new(1, 1, FALSE);
-    gwy_data_field_get_line_stats(field, rmsline, GWY_LINE_STAT_TAN_BETA0, GWY_ORIENTATION_VERTICAL);
-    rms = gwy_data_line_get_avg(rmsline) * gwy_data_field_get_dy(field);
+    rmsline = gwy_line_new(1, 1, FALSE);
+    gwy_field_get_line_stats(field, rmsline, GWY_LINE_STAT_TAN_BETA0, GWY_ORIENTATION_VERTICAL);
+    rms = gwy_line_avg(rmsline) * gwy_field_get_dy(field);
     g_object_unref(rmsline);
     threshold *= rms;
 
     ls = mark_discontinuities(field, (display == PREVIEW_MASK ? mask : NULL), threshold, scandir);
     blocksteps = construct_blocks(field, (display == PREVIEW_BLOCKS ? mask : NULL), ls, scandir);
     g_free(ls);
-    gwy_data_field_assign(result, field);
+    gwy_field_assign(result, field);
     if (!blocksteps)
         return 0;
 
