@@ -1,5 +1,5 @@
 /*
- *  $Id: coerce.c 27054 2024-12-18 15:37:20Z yeti-dn $
+ *  $Id: coerce.c 29061 2026-01-02 15:01:53Z yeti-dn $
  *  Copyright (C) 2016-2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,19 +16,11 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
 #include <string.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwythreads.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/filters.h>
-#include <libprocess/stats.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
-#include "libgwyddion/gwyomp.h"
+#include <gwy.h>
+#include "libgwyddion/omp.h"
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
@@ -68,39 +60,38 @@ typedef struct {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *result;
+    GwyField *field;
+    GwyField *result;
 } ModuleArgs;
 
 typedef struct {
     ModuleArgs *args;
     GtkWidget *dialog;
     GwyParamTable *table;
-    GwyContainer *data;
 } ModuleGUI;
 
 static gboolean         module_register         (void);
 static GwyParamDef*     define_module_params    (void);
-static void             coerce                  (GwyContainer *data,
-                                                 GwyRunType runtype);
+static void             module_main             (GwyFile *data,
+                                                 GwyRunModeFlags mode);
 static void             execute                 (ModuleArgs *args);
 static GwyDialogOutcome run_gui                 (ModuleArgs *args,
-                                                 GwyContainer *data,
+                                                 GwyFile *data,
                                                  gint id);
 static void             param_changed           (ModuleGUI *gui,
                                                  gint id);
 static void             preview                 (gpointer user_data);
-static gboolean         template_filter         (GwyContainer *data,
+static gboolean         template_filter         (GwyFile *data,
                                                  gint id,
                                                  gpointer user_data);
-static void             coerce_do_field         (GwyDataField *field,
-                                                 GwyDataField *result,
+static void             coerce_do_field         (GwyField *field,
+                                                 GwyField *result,
                                                  GwyParams *params);
-static void             coerce_do_field_levels  (GwyDataField *field,
-                                                 GwyDataField *result,
+static void             coerce_do_field_levels  (GwyField *field,
+                                                 GwyField *result,
                                                  GwyParams *params);
-static void             coerce_do_rows          (GwyDataField *field,
-                                                 GwyDataField *result,
+static void             coerce_do_rows          (GwyField *field,
+                                                 GwyField *result,
                                                  GwyParams *params);
 static void             build_values_levels     (const ValuePos *vpos,
                                                  gdouble *z,
@@ -141,11 +132,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("coerce",
-                              (GwyProcessFunc)&coerce,
+                              module_main,
                               N_("/_Distortion/Co_erce..."),
-                              GWY_STOCK_ENFORCE_DISTRIBUTION,
+                              GWY_ICON_ENFORCE_DISTRIBUTION,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               _("Enforce prescribed statistical properties"));
 
     return TRUE;
@@ -155,9 +146,9 @@ static GwyParamDef*
 define_module_params(void)
 {
     static const GwyEnum distributions[] = {
-        { N_("distribution|Uniform"),     COERCE_DISTRIBUTION_UNIFORM,     },
-        { N_("distribution|Gaussian"),    COERCE_DISTRIBUTION_GAUSSIAN,    },
-        { N_("distribution|Skew-normal"), COERCE_DISTRIBUTION_SKEW_NORMAL, },
+        { gwy_NC("distribution", "Uniform"),     COERCE_DISTRIBUTION_UNIFORM,     },
+        { gwy_NC("distribution", "Gaussian"),    COERCE_DISTRIBUTION_GAUSSIAN,    },
+        { gwy_NC("distribution", "Skew-normal"), COERCE_DISTRIBUTION_SKEW_NORMAL, },
         { N_("As another data"),          COERCE_DISTRIBUTION_DATA,        },
         { N_("Discrete levels"),          COERCE_DISTRIBUTION_LEVELS,      },
     };
@@ -166,7 +157,7 @@ define_module_params(void)
         { N_("By row (identically)"), COERCE_PROCESSING_ROWS,  },
     };
     static const GwyEnum leveltypes[] = {
-        { N_("distribution|Uniform"), COERCE_LEVELS_UNIFORM,  },
+        { gwy_NC("distribution", "Uniform"), COERCE_LEVELS_UNIFORM,  },
         { N_("Same area"),            COERCE_LEVELS_EQUIAREA, },
     };
     static GwyParamDef *paramdef = NULL;
@@ -190,22 +181,22 @@ define_module_params(void)
 }
 
 static void
-coerce(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     gint id, newid;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(args.field);
 
-    args.result = gwy_data_field_new_alike(args.field, FALSE);
+    args.result = gwy_field_new_alike(args.field, FALSE);
     args.params = gwy_params_new_from_settings(define_module_params());
     sanitise_params(&args);
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, id);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -214,14 +205,14 @@ coerce(GwyContainer *data, GwyRunType runtype)
     if (outcome == GWY_DIALOG_PROCEED)
         execute(&args);
 
-    newid = gwy_app_data_browser_add_data_field(args.result, data, TRUE);
-    gwy_app_sync_data_items(data, data, id, newid, FALSE,
-                            GWY_DATA_ITEM_PALETTE,
-                            GWY_DATA_ITEM_RANGE_TYPE,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
-    gwy_app_set_data_field_title(data, newid, _("Coerced"));
-    gwy_app_channel_log_add_proc(data, id, newid);
+    newid = gwy_file_add_image(data, args.result);
+
+    gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+    gwy_file_sync_items(data, GWY_FILE_IMAGE, id,
+                        data, GWY_FILE_IMAGE, newid,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_COLOR_MAPPING | GWY_FILE_ITEM_REAL_SQUARE, FALSE);
+    gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _("Coerced"), TRUE);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, newid);
 
 end:
     g_object_unref(args.result);
@@ -229,30 +220,23 @@ end:
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
-    GwyDialogOutcome outcome;
     GtkWidget *hbox, *dataview;
     GwyDialog *dialog;
     GwyParamTable *table;
     ModuleGUI gui;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
-    gui.data = gwy_container_new();
-
-    gwy_container_set_object_by_name(gui.data, "/0/data", args->result);
-    gwy_app_sync_data_items(data, gui.data, id, 0, FALSE,
-                            GWY_DATA_ITEM_PALETTE,
-                            GWY_DATA_ITEM_RANGE,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
 
     gui.dialog = gwy_dialog_new(_("Coerce Statistics"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_UPDATE, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
+    dataview = gwy_create_preview(args->result, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_RANGE | GWY_FILE_ITEM_REAL_SQUARE);
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), FALSE);
 
     table = gui.table = gwy_param_table_new(args->params);
@@ -284,11 +268,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     g_signal_connect_swapped(table, "param-changed", G_CALLBACK(param_changed), &gui);
     gwy_dialog_set_preview_func(dialog, GWY_PREVIEW_IMMEDIATE, preview, &gui, NULL);
 
-    outcome = gwy_dialog_run(dialog);
-
-    g_object_unref(gui.data);
-
-    return outcome;
+    return gwy_dialog_run(dialog);
 }
 
 static void
@@ -312,16 +292,16 @@ param_changed(ModuleGUI *gui, gint id)
 }
 
 static gboolean
-template_filter(GwyContainer *data, gint id, gpointer user_data)
+template_filter(GwyFile *data, gint id, gpointer user_data)
 {
-    GwyDataField *template, *field = (GwyDataField*)user_data;
+    GwyField *template, *field = (GwyField*)user_data;
 
-    if (!gwy_container_gis_object(data, gwy_app_get_data_key_for_id(id), &template))
+    if (!(template = gwy_file_get_image(data, id)))
         return FALSE;
     if (template == field)
         return FALSE;
 
-    return !gwy_data_field_check_compatibility(field, template, GWY_DATA_COMPATIBILITY_VALUE);
+    return !gwy_field_is_incompatible(field, template, GWY_DATA_MISMATCH_VALUE);
 }
 
 static void
@@ -330,7 +310,7 @@ preview(gpointer user_data)
     ModuleGUI *gui = (ModuleGUI*)user_data;
 
     execute(gui->args);
-    gwy_data_field_data_changed(gui->args->result);
+    gwy_field_data_changed(gui->args->result);
     gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
 }
 
@@ -349,11 +329,11 @@ execute(ModuleArgs *args)
 }
 
 static void
-coerce_do_field(GwyDataField *field, GwyDataField *result,
+coerce_do_field(GwyField *field, GwyField *result,
                 GwyParams *params)
 {
-    guint n = gwy_data_field_get_xres(field)*gwy_data_field_get_yres(field);
-    const gdouble *d = gwy_data_field_get_data_const(field);
+    guint n = gwy_field_get_xres(field)*gwy_field_get_yres(field);
+    const gdouble *d = gwy_field_get_data_const(field);
     CoerceDistributionType distribution = gwy_params_get_enum(params, PARAM_DISTRIBUTION);
     CoerceLevelsType level_type = gwy_params_get_enum(params, PARAM_LEVEL_TYPE);
     ValuePos *vpos;
@@ -374,35 +354,35 @@ coerce_do_field(GwyDataField *field, GwyDataField *result,
     qsort(vpos, n, sizeof(ValuePos), gwy_compare_double);
 
     if (distribution == COERCE_DISTRIBUTION_DATA) {
-        GwyDataField *src = gwy_params_get_image(params, PARAM_TEMPLATE);
-        guint nsrc = gwy_data_field_get_xres(src)*gwy_data_field_get_yres(src);
-        build_values_from_data(z, n, gwy_data_field_get_data_const(src), nsrc);
+        GwyField *src = gwy_params_get_image(params, PARAM_TEMPLATE);
+        guint nsrc = gwy_field_get_xres(src)*gwy_field_get_yres(src);
+        build_values_from_data(z, n, gwy_field_get_data_const(src), nsrc);
     }
     else if (distribution == COERCE_DISTRIBUTION_LEVELS) {
         build_values_levels(vpos, z, n, gwy_params_get_int(params, PARAM_NLEVELS));
     }
     else if (distribution == COERCE_DISTRIBUTION_UNIFORM) {
         gdouble min, max;
-        gwy_data_field_get_min_max(field, &min, &max);
+        gwy_field_get_min_max(field, &min, &max);
         build_values_uniform(z, n, min, max);
     }
     else if (distribution == COERCE_DISTRIBUTION_GAUSSIAN) {
         gdouble avg, rms;
-        avg = gwy_data_field_get_avg(field);
-        rms = gwy_data_field_get_rms(field);
+        avg = gwy_field_get_avg(field);
+        rms = gwy_field_get_rms(field);
         build_values_gaussian(z, n, avg, rms);
     }
     else if (distribution == COERCE_DISTRIBUTION_SKEW_NORMAL) {
         gdouble avg, rms;
-        avg = gwy_data_field_get_avg(field);
-        rms = gwy_data_field_get_rms(field);
+        avg = gwy_field_get_avg(field);
+        rms = gwy_field_get_rms(field);
         build_values_skew_normal(z, n, avg, rms, gwy_params_get_double(params, PARAM_SKEW));
     }
     else {
         g_return_if_reached();
     }
 
-    dr = gwy_data_field_get_data(result);
+    dr = gwy_field_get_data(result);
     for (k = 0; k < n; k++)
         dr[vpos[k].k] = z[k];
 
@@ -411,18 +391,18 @@ coerce_do_field(GwyDataField *field, GwyDataField *result,
 }
 
 static void
-coerce_do_field_levels(GwyDataField *field, GwyDataField *result,
+coerce_do_field_levels(GwyField *field, GwyField *result,
                        GwyParams *params)
 {
-    guint n = gwy_data_field_get_xres(field)*gwy_data_field_get_yres(field);
-    const gdouble *d = gwy_data_field_get_data_const(field);
-    gdouble *dr = gwy_data_field_get_data(result);
+    guint n = gwy_field_get_xres(field)*gwy_field_get_yres(field);
+    const gdouble *d = gwy_field_get_data_const(field);
+    gdouble *dr = gwy_field_get_data(result);
     gdouble min, max, q;
     gint k, nlevels = gwy_params_get_int(params, PARAM_NLEVELS);
 
-    gwy_data_field_get_min_max(field, &min, &max);
+    gwy_field_get_min_max(field, &min, &max);
     if (max <= min) {
-        gwy_data_field_fill(result, 0.5*(min + max));
+        gwy_field_fill(result, 0.5*(min + max));
         return;
     }
 
@@ -440,12 +420,12 @@ coerce_do_field_levels(GwyDataField *field, GwyDataField *result,
 }
 
 static void
-coerce_do_rows(GwyDataField *field, GwyDataField *result,
+coerce_do_rows(GwyField *field, GwyField *result,
                GwyParams *params)
 {
-    guint xres = gwy_data_field_get_xres(field),
-          yres = gwy_data_field_get_yres(field);
-    const gdouble *d = gwy_data_field_get_data_const(field);
+    guint xres = gwy_field_get_xres(field),
+          yres = gwy_field_get_yres(field);
+    const gdouble *d = gwy_field_get_data_const(field);
     CoerceDistributionType distribution = gwy_params_get_enum(params, PARAM_DISTRIBUTION);
     ValuePos *vpos;
     gdouble *z, *dr;
@@ -454,11 +434,11 @@ coerce_do_rows(GwyDataField *field, GwyDataField *result,
     /* It is not completely clear what we should do in the case of row-wise processing but this at least ensures that
      * the levels are the same in the entire field but individual rows are transformed individualy. */
     if (distribution == COERCE_DISTRIBUTION_LEVELS) {
-        GwyDataField *tmp = gwy_data_field_duplicate(field);
+        GwyField *tmp = gwy_field_copy(field);
         gdouble min, max;
-        gwy_data_field_get_min_max(field, &min, &max);
+        gwy_field_get_min_max(field, &min, &max);
         for (i = 0; i < yres; i++)
-            gwy_data_field_area_renormalize(tmp, 0, i, xres, 1, max-min, min);
+            gwy_field_area_renormalize(tmp, 0, i, xres, 1, max-min, min);
         coerce_do_field(tmp, result, params);
         g_object_unref(tmp);
         return;
@@ -468,32 +448,32 @@ coerce_do_rows(GwyDataField *field, GwyDataField *result,
     z = g_new(gdouble, xres);
 
     if (distribution == COERCE_DISTRIBUTION_DATA) {
-        GwyDataField *src = gwy_params_get_image(params, PARAM_TEMPLATE);
-        guint nsrc = gwy_data_field_get_xres(src)*gwy_data_field_get_yres(src);
-        build_values_from_data(z, xres, gwy_data_field_get_data_const(src), nsrc);
+        GwyField *src = gwy_params_get_image(params, PARAM_TEMPLATE);
+        guint nsrc = gwy_field_get_xres(src)*gwy_field_get_yres(src);
+        build_values_from_data(z, xres, gwy_field_get_data_const(src), nsrc);
     }
     else if (distribution == COERCE_DISTRIBUTION_UNIFORM) {
         gdouble min, max;
-        gwy_data_field_get_min_max(field, &min, &max);
+        gwy_field_get_min_max(field, &min, &max);
         build_values_uniform(z, xres, min, max);
     }
     else if (distribution == COERCE_DISTRIBUTION_GAUSSIAN) {
         gdouble avg, rms;
-        avg = gwy_data_field_get_avg(field);
-        rms = gwy_data_field_get_rms(field);
+        avg = gwy_field_get_avg(field);
+        rms = gwy_field_get_rms(field);
         build_values_gaussian(z, xres, avg, rms);
     }
     else if (distribution == COERCE_DISTRIBUTION_SKEW_NORMAL) {
         gdouble avg, rms;
-        avg = gwy_data_field_get_avg(field);
-        rms = gwy_data_field_get_rms(field);
+        avg = gwy_field_get_avg(field);
+        rms = gwy_field_get_rms(field);
         build_values_skew_normal(z, xres, avg, rms, gwy_params_get_double(params, PARAM_SKEW));
     }
     else {
         g_return_if_reached();
     }
 
-    dr = gwy_data_field_get_data(result);
+    dr = gwy_field_get_data(result);
     for (i = 0; i < yres; i++) {
         for (j = 0; j < xres; j++) {
             vpos[j].z = d[i*xres + j];
@@ -623,7 +603,7 @@ build_values_from_data(gdouble *z, guint n, const gdouble *data, guint ndata)
 
     if (n == ndata) {
         gwy_assign(z, data, n);
-        gwy_math_sort(n, z);
+        gwy_math_sort(z, n);
         return;
     }
 
@@ -634,7 +614,7 @@ build_values_from_data(gdouble *z, guint n, const gdouble *data, guint ndata)
     }
 
     sorted = g_memdup(data, ndata*sizeof(gdouble));
-    gwy_math_sort(ndata, sorted);
+    gwy_math_sort(sorted, ndata);
 
     if (n < 3) {
         if (n == 1)
@@ -769,7 +749,7 @@ sanitise_params(ModuleArgs *args)
     gboolean is_none = gwy_params_data_id_is_none(params, PARAM_TEMPLATE);
 
     if (distribution == COERCE_DISTRIBUTION_DATA
-        && (is_none || !template_filter(gwy_app_data_browser_get(template.datano), template.id, args->field)))
+        && (is_none || !template_filter(gwy_data_browser_get_file(template.datano), template.id, args->field)))
         gwy_params_reset(params, PARAM_DISTRIBUTION);
 }
 
