@@ -1,6 +1,6 @@
 /*
- *  $Id: correct_perspective.c 25638 2023-09-07 12:23:28Z yeti-dn $
- *  Copyright (C) 2021 David Necas (Yeti).
+ *  $Id: correct_perspective.c 29084 2026-01-05 18:02:33Z yeti-dn $
+ *  Copyright (C) 2021-2026 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -16,18 +16,11 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/correct.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_INTERACTIVE)
@@ -49,7 +42,7 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
+    GwyField *field;
     gdouble xy[8];
 } ModuleArgs;
 
@@ -57,25 +50,24 @@ typedef struct {
     ModuleArgs *args;
     GtkWidget *dialog;
     GwyParamTable *table;
-    GwyContainer *data;
     GtkWidget *view;
     GwyVectorLayer *vlayer;
     GwySelection *selection;
-    gboolean corrected_computed;
+    GwyField *corrected;
 } ModuleGUI;
 
 static gboolean         module_register                (void);
 static GwyParamDef*     define_module_params           (void);
-static void             correct_perspective            (GwyContainer *data,
-                                                        GwyRunType run);
+static void             module_main                    (GwyFile *data,
+                                                        GwyRunModeFlags mode);
 static void             gather_quarks_for_one_image    (GwyContainer *data,
                                                         gint id,
                                                         GArray *quarks);
 static void             apply_correction_to_one_image  (ModuleArgs *args,
-                                                        GwyContainer *data,
+                                                        GwyFile *data,
                                                         gint id);
 static GwyDialogOutcome run_gui                        (ModuleArgs *args,
-                                                        GwyContainer *data,
+                                                        GwyFile *data,
                                                         gint id);
 static void             param_changed                  (ModuleGUI *gui,
                                                         gint id);
@@ -83,13 +75,13 @@ static void             dialog_response                (GwyDialog *dialog,
                                                         gint response,
                                                         ModuleGUI *gui);
 static void             preview                        (gpointer user_data);
-static void             init_coordinates               (GwyDataField *field,
+static void             init_coordinates               (GwyField *field,
                                                         gdouble *xy);
 static void             selection_changed              (ModuleGUI *gui);
 static void             guess_pixel_dimensions         (GwyParamTable *table,
                                                         GwySelection *selection,
-                                                        GwyDataField *field);
-static GwyDataField*    create_corrected_field         (GwyDataField *field,
+                                                        GwyField *field);
+static GwyField*        create_corrected_field         (GwyField *field,
                                                         const gdouble *xy,
                                                         gint xres,
                                                         gint yres,
@@ -118,11 +110,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("correct_perspective",
-                              (GwyProcessFunc)&correct_perspective,
+                              module_main,
                               N_("/_Distortion/_Perspective..."),
-                              GWY_STOCK_PERSPECTIVE_DISTORT,
+                              GWY_ICON_PERSPECTIVE_DISTORT,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Correct perspective distortion"));
 
     return TRUE;
@@ -147,7 +139,7 @@ define_module_params(void)
     gwy_param_def_add_boolean(paramdef, PARAM_FIXRES, "fixres", _("Specify output _dimensions"), FALSE);
     gwy_param_def_add_int(paramdef, PARAM_XRES, "xres", _("_X resolution"), 2, 32768, 512);
     gwy_param_def_add_int(paramdef, PARAM_YRES, "yres", _("_Y resolution"), 2, 32768, 512);
-    gwy_param_def_add_gwyenum(paramdef, PARAM_IMAGE_MODE, NULL, gwy_sgettext("verb|Display"),
+    gwy_param_def_add_gwyenum(paramdef, PARAM_IMAGE_MODE, NULL, C_("verb", "Display"),
                               image_modes, G_N_ELEMENTS(image_modes), IMAGE_DATA);
     gwy_param_def_add_boolean(paramdef, PARAM_NEW_IMAGE, "new-image", _("Create new image"), TRUE);
     gwy_param_def_add_boolean(paramdef, PARAM_DISTRIBUTE, "distribute", _("_Apply to all compatible images"), FALSE);
@@ -155,42 +147,40 @@ define_module_params(void)
 }
 
 static void
-correct_perspective(GwyContainer *data, GwyRunType run)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
-    enum { compat_flags = GWY_DATA_COMPATIBILITY_RES | GWY_DATA_COMPATIBILITY_REAL | GWY_DATA_COMPATIBILITY_LATERAL };
+    enum { compat_flags = GWY_DATA_MISMATCH_RES | GWY_DATA_MISMATCH_REAL | GWY_DATA_MISMATCH_LATERAL };
     ModuleArgs args;
     GwyParams *params;
-    GwyDataField *field;
+    GwyField *field;
     GwySelection *selection;
     GwyDialogOutcome outcome;
     gint i, id;
     gboolean distribute, new_image;
     GArray *undo_quarks;
     gint *image_ids;
-    gchar selkey[40];
 
-    g_return_if_fail(run & RUN_MODES);
-    g_return_if_fail(g_type_from_name("GwyLayerProjective"));
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &field,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &field,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(field);
 
-    gwy_clear(&args, 1);
+    gwy_clear1(args);
     args.field = field;
     args.params = params = gwy_params_new_from_settings(define_module_params());
     init_coordinates(field, args.xy);
-    g_snprintf(selkey, sizeof(selkey), "/%d/select/projective", id);
-    if (gwy_container_gis_object_by_name(data, selkey, &selection)
+    GQuark selection_key = gwy_file_key_image_selection(id, "projective");
+    if (gwy_container_gis_object(GWY_CONTAINER(data), selection_key, &selection)
         && gwy_selection_get_data(GWY_SELECTION(selection), NULL))
         gwy_selection_get_object(GWY_SELECTION(selection), 0, args.xy);
 
     outcome = run_gui(&args, data, id);
     gwy_params_save_to_settings(params);
 
-    selection = g_object_new(g_type_from_name("GwySelectionProjective"), "max-objects", 1, NULL);
+    selection = gwy_selection_quad_new();
     gwy_selection_set_object(GWY_SELECTION(selection), 0, args.xy);
-    gwy_container_pass_object_by_name(data, selkey, selection);
+    gwy_container_pass_object(GWY_CONTAINER(data), selection_key, selection);
 
     if (outcome != GWY_DIALOG_PROCEED)
         goto end;
@@ -200,29 +190,29 @@ correct_perspective(GwyContainer *data, GwyRunType run)
     if (!distribute) {
         if (!new_image) {
             undo_quarks = g_array_new(FALSE, FALSE, sizeof(GQuark));
-            gather_quarks_for_one_image(data, id, undo_quarks);
-            gwy_app_undo_qcheckpointv(data, undo_quarks->len, (GQuark*)undo_quarks->data);
+            gather_quarks_for_one_image(GWY_CONTAINER(data), id, undo_quarks);
+            gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), undo_quarks->len, (GQuark*)undo_quarks->data);
             g_array_free(undo_quarks, TRUE);
         }
         apply_correction_to_one_image(&args, data, id);
         goto end;
     }
 
-    image_ids = gwy_app_data_browser_get_data_ids(data);
+    image_ids = gwy_file_get_ids(data, GWY_FILE_IMAGE);
     g_object_ref(field);
     if (!new_image) {
         undo_quarks = g_array_new(FALSE, FALSE, sizeof(GQuark));
         for (i = 0; image_ids[i] != -1; i++) {
-            GwyDataField *otherfield = gwy_container_get_object(data, gwy_app_get_data_key_for_id(image_ids[i]));
-            if (!gwy_data_field_check_compatibility(field, otherfield, compat_flags))
-                gather_quarks_for_one_image(data, image_ids[i], undo_quarks);
+            GwyField *otherfield = gwy_file_get_image(data, image_ids[i]);
+            if (!gwy_field_is_incompatible(field, otherfield, compat_flags))
+                gather_quarks_for_one_image(GWY_CONTAINER(data), image_ids[i], undo_quarks);
         }
-        gwy_app_undo_qcheckpointv(data, undo_quarks->len, (GQuark*)undo_quarks->data);
+        gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), undo_quarks->len, (GQuark*)undo_quarks->data);
         g_array_free(undo_quarks, TRUE);
     }
     for (i = 0; image_ids[i] != -1; i++) {
-        GwyDataField *otherfield = gwy_container_get_object(data, gwy_app_get_data_key_for_id(image_ids[i]));
-        if (gwy_data_field_check_compatibility(field, otherfield, compat_flags))
+        GwyField *otherfield = gwy_file_get_image(data, image_ids[i]);
+        if (gwy_field_is_incompatible(field, otherfield, compat_flags))
             continue;
 
         apply_correction_to_one_image(&args, data, image_ids[i]);
@@ -240,22 +230,22 @@ gather_quarks_for_one_image(GwyContainer *data, gint id, GArray *quarks)
     GObject *object;
     GQuark quark;
 
-    quark = gwy_app_get_data_key_for_id(id);
+    quark = gwy_file_key_image(id);
     object = gwy_container_get_object(data, quark);
-    g_assert(GWY_IS_DATA_FIELD(object));
+    g_assert(GWY_IS_FIELD(object));
     g_array_append_val(quarks, quark);
 
-    quark = gwy_app_get_mask_key_for_id(id);
-    if (gwy_container_gis_object(data, quark, &object) && GWY_IS_DATA_FIELD(object))
+    quark = gwy_file_key_image_mask(id);
+    if (gwy_container_gis_object(data, quark, &object) && GWY_IS_FIELD(object))
         g_array_append_val(quarks, quark);
 
-    quark = gwy_app_get_show_key_for_id(id);
-    if (gwy_container_gis_object(data, quark, &object) && GWY_IS_DATA_FIELD(object))
+    quark = gwy_file_key_image_picture(id);
+    if (gwy_container_gis_object(data, quark, &object) && GWY_IS_FIELD(object))
         g_array_append_val(quarks, quark);
 }
 
 static void
-apply_correction_to_one_image(ModuleArgs *args, GwyContainer *data, gint id)
+apply_correction_to_one_image(ModuleArgs *args, GwyFile *data, gint id)
 {
     GwyParams *params = args->params;
     GwyInterpolationType interpolation = gwy_params_get_enum(params, PARAM_INTERPOLATION);
@@ -264,25 +254,26 @@ apply_correction_to_one_image(ModuleArgs *args, GwyContainer *data, gint id)
     gboolean fixres = gwy_params_get_boolean(params, PARAM_FIXRES);
     gint xres = (fixres ? gwy_params_get_int(params, PARAM_XRES) : 0);
     gint yres = (fixres ? gwy_params_get_int(params, PARAM_YRES) : 0);
-    GwyDataField *field, *mask = NULL, *show = NULL;
+    GwyField *field = gwy_file_get_image(data, id), *mask = NULL, *show = NULL;
     gchar *title, *newtitle;
     gint newid;
 
-    field = gwy_container_get_object(data, gwy_app_get_data_key_for_id(id));
-    g_assert(GWY_IS_DATA_FIELD(field));
-    gwy_container_gis_object(data, gwy_app_get_mask_key_for_id(id), &mask);
-    gwy_container_gis_object(data, gwy_app_get_show_key_for_id(id), &show);
+    g_assert(GWY_IS_FIELD(field));
+    mask = gwy_file_get_image_mask(data, id);
+    gwy_container_gis_object(GWY_CONTAINER(data), gwy_file_key_image_picture(id), &show);
 
     field = create_corrected_field(field, args->xy, xres, yres, interpolation);
     if (!new_image) {
-        gwy_container_set_object(data, gwy_app_get_data_key_for_id(id), field);
+        gwy_file_set_image(data, id, field);
         newid = id;
     }
     else {
-        newid = gwy_app_data_browser_add_data_field(field, data, !distribute);
-        title = gwy_app_get_data_field_title(data, id);
+        newid = gwy_file_add_image(data, field);
+        if (!distribute)
+            gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+        title = gwy_file_get_display_title(data, GWY_FILE_IMAGE, id);
         newtitle = g_strconcat(title, " ", _("Corrected"), NULL);
-        gwy_app_set_data_field_title(data, newid, newtitle);
+        gwy_file_set_title(data, GWY_FILE_IMAGE, newid, newtitle, TRUE);
         g_free(newtitle);
         g_free(title);
     }
@@ -290,26 +281,25 @@ apply_correction_to_one_image(ModuleArgs *args, GwyContainer *data, gint id)
 
     if (mask) {
         mask = create_corrected_field(mask, args->xy, xres, yres, GWY_INTERPOLATION_ROUND);
-        gwy_container_pass_object(data, gwy_app_get_mask_key_for_id(newid), mask);
+        gwy_file_pass_image_mask(data, newid, mask);
     }
     if (show) {
         show = create_corrected_field(show, args->xy, xres, yres, interpolation);
-        gwy_container_pass_object(data, gwy_app_get_show_key_for_id(newid), show);
+        gwy_container_pass_object(GWY_CONTAINER(data), gwy_file_key_image_picture(newid), show);
     }
 
     if (!new_image) {
-        gwy_app_sync_data_items(data, data, id, newid, FALSE,
-                                GWY_DATA_ITEM_GRADIENT,
-                                GWY_DATA_ITEM_RANGE,
-                                GWY_DATA_ITEM_MASK_COLOR,
-                                GWY_DATA_ITEM_REAL_SQUARE,
-                                0);
+        gwy_file_sync_items(data, GWY_FILE_IMAGE, id,
+                            data, GWY_FILE_IMAGE, newid,
+                            GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_RANGE
+                            | GWY_FILE_ITEM_MASK_COLOR | GWY_FILE_ITEM_REAL_SQUARE,
+                            FALSE);
     }
-    gwy_app_channel_log_add_proc(data, id, newid);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, newid);
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     GtkWidget *hbox;
     GwyDialog *dialog;
@@ -317,25 +307,22 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     ModuleGUI gui;
     GwyDialogOutcome outcome;
 
+    gwy_clear1(gui);
     gui.args = args;
-    gui.data = gwy_container_new();
-    gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(0), args->field);
-    gwy_app_sync_data_items(data, gui.data, id, 0, FALSE,
-                            GWY_DATA_ITEM_RANGE_TYPE,
-                            GWY_DATA_ITEM_RANGE,
-                            GWY_DATA_ITEM_GRADIENT,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
 
     gui.dialog = gwy_dialog_new(_("Projective Correction"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    gui.view = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
-    gui.selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(gui.view), 0, "Projective", 1, TRUE);
+    gui.view = gwy_create_preview(args->field, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(gui.view), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_COLOR_MAPPING | GWY_FILE_ITEM_RANGE
+                        | GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_REAL_SQUARE);
+    gui.selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(gui.view), GWY_TYPE_LAYER_QUAD, 1, TRUE);
     g_object_ref(gui.selection);
     gwy_selection_set_data(gui.selection, 1, args->xy);
-    gui.vlayer = gwy_data_view_get_top_layer(GWY_DATA_VIEW(gui.view));
+    gui.vlayer = gwy_data_view_get_interactive_layer(GWY_DATA_VIEW(gui.view));
+    gwy_layer_quad_set_n_lines(GWY_LAYER_QUAD(gui.vlayer), 3);
     g_object_ref(gui.vlayer);
     g_signal_connect_swapped(gui.selection, "changed", G_CALLBACK(selection_changed), &gui);
 
@@ -373,7 +360,6 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 
     g_object_unref(gui.selection);
     g_object_unref(gui.vlayer);
-    g_object_unref(gui.data);
 
     return outcome;
 }
@@ -392,7 +378,7 @@ param_changed(ModuleGUI *gui, gint id)
             guess_pixel_dimensions(table, gui->selection, gui->args->field);
     }
     if (id < 0 || id == PARAM_INTERPOLATION) {
-        gui->corrected_computed = FALSE;
+        g_clear_object(&gui->corrected);
         if (gwy_params_get_enum(params, PARAM_IMAGE_MODE) == IMAGE_CORRECTED)
             gwy_dialog_invalidate(GWY_DIALOG(gui->dialog));
     }
@@ -420,11 +406,11 @@ selection_changed(ModuleGUI *gui)
     gwy_selection_get_object(gui->selection, 0, args->xy);
     if (!gwy_params_get_boolean(args->params, PARAM_FIXRES))
         guess_pixel_dimensions(gui->table, gui->selection, args->field);
-    gui->corrected_computed = FALSE;
+    g_clear_object(&gui->corrected);
 }
 
 static void
-guess_pixel_dimensions(GwyParamTable *table, GwySelection *selection, GwyDataField *field)
+guess_pixel_dimensions(GwyParamTable *table, GwySelection *selection, GwyField *field)
 {
     gdouble xypix[8], xy[8];
     gdouble newxpix, newypix;
@@ -432,8 +418,8 @@ guess_pixel_dimensions(GwyParamTable *table, GwySelection *selection, GwyDataFie
 
     gwy_selection_get_object(selection, 0, xy);
     for (i = 0; i < 4; i++) {
-        xypix[2*i + 0] = gwy_data_field_rtoj(field, xy[2*i + 0]);
-        xypix[2*i + 1] = gwy_data_field_rtoi(field, xy[2*i + 1]);
+        xypix[2*i + 0] = gwy_field_rtoj(field, xy[2*i + 0]);
+        xypix[2*i + 1] = gwy_field_rtoi(field, xy[2*i + 1]);
     }
     estimate_reasonable_dimensions(xypix, &newxpix, &newypix);
     gwy_param_table_set_int(table, PARAM_XRES, MAX(GWY_ROUND(newxpix + 1.0), 2));
@@ -441,13 +427,13 @@ guess_pixel_dimensions(GwyParamTable *table, GwySelection *selection, GwyDataFie
 }
 
 static void
-init_coordinates(GwyDataField *field, gdouble *xy)
+init_coordinates(GwyField *field, gdouble *xy)
 {
     gdouble xreal, yreal;
     gint i, j;
 
-    xreal = gwy_data_field_get_xreal(field);
-    yreal = gwy_data_field_get_yreal(field);
+    xreal = gwy_field_get_xreal(field);
+    yreal = gwy_field_get_yreal(field);
     for (i = 0; i < 2; i++) {
         for (j = 0; j < 2; j++) {
             xy[2*(2*i + j) + 0] = 0.25*xreal + 0.5*(i ? 1-j : j)*xreal;
@@ -463,24 +449,20 @@ preview(gpointer user_data)
     ModuleArgs *args = gui->args;
     ImageMode mode = gwy_params_get_enum(args->params, PARAM_IMAGE_MODE);
     GwyDataView *dataview = GWY_DATA_VIEW(gui->view);
-    GwyPixmapLayer *layer = gwy_data_view_get_base_layer(dataview);
-    GwyDataField *corrected;
 
     if (mode == IMAGE_DATA) {
-        g_object_set(layer, "data-key", "/0/data", NULL);
-        gwy_data_view_set_top_layer(dataview, gui->vlayer);
+        gwy_data_view_set_field(dataview, args->field);
+        gwy_data_view_set_interactive_layer(dataview, gui->vlayer);
         gwy_set_data_preview_size(dataview, PREVIEW_SIZE);
         return;
     }
 
-    if (!gui->corrected_computed) {
+    if (!gui->corrected) {
         GwyInterpolationType interpolation = gwy_params_get_enum(args->params, PARAM_INTERPOLATION);
-        corrected = create_corrected_field(args->field, args->xy, 0, 0, interpolation);
-        gwy_container_pass_object_by_name(gui->data, "/1/data", corrected);
-        gui->corrected_computed = TRUE;
+        gui->corrected = create_corrected_field(args->field, args->xy, 0, 0, interpolation);
     }
-    g_object_set(layer, "data-key", "/1/data", NULL);
-    gwy_data_view_set_top_layer(dataview, NULL);
+    gwy_data_view_set_field(dataview, gui->corrected);
+    gwy_data_view_set_interactive_layer(dataview, NULL);
     gwy_set_data_preview_size(dataview, PREVIEW_SIZE);
 }
 
@@ -510,21 +492,21 @@ estimate_reasonable_dimensions(const gdouble *xy, gdouble *lx, gdouble *ly)
     *ly = hypot(ly1, ly2)/G_SQRT2;
 }
 
-static GwyDataField*
-create_corrected_field(GwyDataField *field,
+static GwyField*
+create_corrected_field(GwyField *field,
                        const gdouble *xy,
                        gint xres, gint yres,
                        GwyInterpolationType interp)
 {
-    GwyDataField *corrected;
+    GwyField *corrected;
     gdouble matrix[9], xypix[8];
     gdouble newxreal, newyreal, newxpix, newypix;
     gint i, newxres, newyres;
 
     estimate_reasonable_dimensions(xy, &newxreal, &newyreal);
     for (i = 0; i < 4; i++) {
-        xypix[2*i + 0] = gwy_data_field_rtoj(field, xy[2*i + 0]);
-        xypix[2*i + 1] = gwy_data_field_rtoi(field, xy[2*i + 1]);
+        xypix[2*i + 0] = gwy_field_rtoj(field, xy[2*i + 0]);
+        xypix[2*i + 1] = gwy_field_rtoi(field, xy[2*i + 1]);
     }
     if (xres && yres) {
         newxpix = newxres = xres;
@@ -535,11 +517,11 @@ create_corrected_field(GwyDataField *field,
         newxres = MAX(GWY_ROUND(newxpix + 1.0), 2);
         newyres = MAX(GWY_ROUND(newypix + 1.0), 2);
     }
-    corrected = gwy_data_field_new(newxres, newyres, newxreal*newxres/newxpix, newyreal*newyres/newypix, FALSE);
-    gwy_data_field_copy_units(field, corrected);
+    corrected = gwy_field_new(newxres, newyres, newxreal*newxres/newxpix, newyreal*newyres/newypix, FALSE);
+    gwy_field_copy_units(field, corrected);
 
     solve_projection_from_rectangle(xypix, newxres, newyres, matrix);
-    gwy_data_field_distort(field, corrected, project, matrix, interp, GWY_EXTERIOR_MIRROR_EXTEND, 0.0);
+    gwy_field_distort(field, corrected, project, matrix, interp, GWY_EXTERIOR_MIRROR, 0.0);
 
     return corrected;
 }
