@@ -1,5 +1,5 @@
 /*
- *  $Id: crosscor.c 25638 2023-09-07 12:23:28Z yeti-dn $
+ *  $Id: crosscor.c 28910 2025-11-24 18:11:09Z yeti-dn $
  *  Copyright (C) 2004-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,17 +16,9 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/gwyprocess.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/correlation.h>
-#include <libprocess/filters.h>
-#include <libprocess/stats.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 
 #define RUN_MODES GWY_RUN_INTERACTIVE
 
@@ -67,9 +59,9 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *mask;
-    GwyDataField *result[OUTPUT_NTYPES];
+    GwyField *field;
+    GwyField *mask;
+    GwyField *result[OUTPUT_NTYPES];
 } ModuleArgs;
 
 typedef struct {
@@ -82,8 +74,8 @@ typedef struct {
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             crosscor            (GwyContainer *data,
-                                             GwyRunType runtype);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static gboolean         execute             (ModuleArgs *args,
                                              GtkWindow *wait_window);
 static GwyDialogOutcome run_gui             (ModuleArgs *args);
@@ -94,10 +86,10 @@ static void             dialog_response     (ModuleGUI *gui,
 static GtkWidget*       correlation_tab_new (ModuleGUI *gui);
 static GtkWidget*       output_tab_new      (ModuleGUI *gui);
 static GtkWidget*       multichannel_tab_new(ModuleGUI *gui);
-static gboolean         other_image_filter  (GwyContainer *data,
+static gboolean         other_image_filter  (GwyFile *data,
                                              gint id,
                                              gpointer user_data);
-static gboolean         weaker_image_filter (GwyContainer *data,
+static gboolean         weaker_image_filter (GwyFile *data,
                                              gint id,
                                              gpointer user_data);
 static void             guess_offsets       (ModuleGUI *gui);
@@ -127,11 +119,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("crosscor",
-                              (GwyProcessFunc)&crosscor,
+                              module_main,
                               N_("/M_ultidata/_Cross-Correlation..."),
                               NULL,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Cross-correlate two data fields"));
 
     return TRUE;
@@ -170,19 +162,18 @@ define_module_params(void)
 }
 
 static void
-crosscor(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome;
-    GwyDataField *mask;
     ModuleArgs args;
     gint id, newid;
     guint i;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(args.field);
     args.params = gwy_params_new_from_settings(define_module_params());
 
@@ -191,31 +182,30 @@ crosscor(GwyContainer *data, GwyRunType runtype)
     if (outcome == GWY_DIALOG_CANCEL)
         goto end;
 
-    if (!execute(&args, gwy_app_find_window_for_channel(data, id)))
+    if (!execute(&args, gwy_data_browser_get_window_for_data(data, GWY_FILE_IMAGE, id)))
         goto end;
 
     for (i = 0; i < OUTPUT_NTYPES; i++) {
         if (!args.result[i])
             continue;
 
-        newid = gwy_app_data_browser_add_data_field(args.result[i], data, TRUE);
-        gwy_app_sync_data_items(data, data, id, newid, FALSE,
-                                GWY_DATA_ITEM_GRADIENT,
-                                GWY_DATA_ITEM_REAL_SQUARE,
-                                0);
-        gwy_app_set_data_field_title(data, newid, _(gwy_enum_to_string(1 << i, outputs, OUTPUT_NTYPES)));
-        if (args.mask) {
-            mask = gwy_data_field_duplicate(args.mask);
-            gwy_container_pass_object(data, gwy_app_get_mask_key_for_id(newid), mask);
-        }
-        gwy_app_channel_log_add_proc(data, id, newid);
+        newid = gwy_file_add_image(data, args.result[i]);
+
+        gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+        gwy_file_sync_items(data, GWY_FILE_IMAGE, id,
+                            data, GWY_FILE_IMAGE, newid,
+                            GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_REAL_SQUARE, FALSE);
+        gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _(gwy_enum_to_string(1 << i, outputs, OUTPUT_NTYPES)), TRUE);
+        if (args.mask)
+            gwy_file_pass_image_mask(data, newid, gwy_field_copy(args.mask));
+        gwy_log_add(data, GWY_FILE_IMAGE, id, newid);
     }
 
 end:
     g_object_unref(args.params);
-    GWY_OBJECT_UNREF(args.mask);
+    g_clear_object(&args.mask);
     for (i = 0; i < OUTPUT_NTYPES; i++)
-        GWY_OBJECT_UNREF(args.result[i]);
+        g_clear_object(args.result + i);
 }
 
 static GwyDialogOutcome
@@ -247,7 +237,7 @@ run_gui(ModuleArgs *args)
 }
 
 static void
-append_lateral_slider_pair(GwyParamTable *table, gint idx, gint idy, GwyDataField *field)
+append_lateral_slider_pair(GwyParamTable *table, gint idx, gint idy, GwyField *field)
 {
     gwy_param_table_append_slider(table, idx);
     gwy_param_table_slider_add_alt(table, idx);
@@ -359,56 +349,56 @@ dialog_response(ModuleGUI *gui, gint response)
 }
 
 static gboolean
-other_image_filter(GwyContainer *data, gint id, gpointer user_data)
+other_image_filter(GwyFile *data, gint id, gpointer user_data)
 {
-    GwyDataField *otherfield, *field = (GwyDataField*)user_data;
+    GwyField *otherfield, *field = (GwyField*)user_data;
 
-    if (!gwy_container_gis_object(data, gwy_app_get_data_key_for_id(id), &otherfield))
+    if (!(otherfield = gwy_file_get_image(data, id)))
         return FALSE;
     if (otherfield == field)
         return FALSE;
-    return !gwy_data_field_check_compatibility(field, otherfield,
-                                               GWY_DATA_COMPATIBILITY_RES
-                                               | GWY_DATA_COMPATIBILITY_REAL
-                                               | GWY_DATA_COMPATIBILITY_LATERAL
-                                               | GWY_DATA_COMPATIBILITY_VALUE);
+    return !gwy_field_is_incompatible(field, otherfield,
+                                      GWY_DATA_MISMATCH_RES
+                                      | GWY_DATA_MISMATCH_REAL
+                                      | GWY_DATA_MISMATCH_LATERAL
+                                      | GWY_DATA_MISMATCH_VALUE);
 }
 
 static gboolean
-weaker_image_filter(GwyContainer *data, gint id, gpointer user_data)
+weaker_image_filter(GwyFile *data, gint id, gpointer user_data)
 {
-    GwyDataField *otherfield, *field = (GwyDataField*)user_data;
+    GwyField *otherfield, *field = (GwyField*)user_data;
 
-    if (!gwy_container_gis_object(data, gwy_app_get_data_key_for_id(id), &otherfield))
+    if (!(otherfield = gwy_file_get_image(data, id)))
         return FALSE;
     if (otherfield == field)
         return FALSE;
-    return !gwy_data_field_check_compatibility(field, otherfield,
-                                               GWY_DATA_COMPATIBILITY_RES
-                                               | GWY_DATA_COMPATIBILITY_REAL
-                                               | GWY_DATA_COMPATIBILITY_LATERAL);
+    return !gwy_field_is_incompatible(field, otherfield,
+                                      GWY_DATA_MISMATCH_RES
+                                      | GWY_DATA_MISMATCH_REAL
+                                      | GWY_DATA_MISMATCH_LATERAL);
 }
 
 static void
 guess_offsets(ModuleGUI *gui)
 {
     ModuleArgs *args = gui->args;
-    GwyDataField *field1 = args->field, *field2 = gwy_params_get_image(args->params, PARAM_OTHER_IMAGE);
-    GwyDataField *kernel, *score;
+    GwyField *field1 = args->field, *field2 = gwy_params_get_image(args->params, PARAM_OTHER_IMAGE);
+    GwyField *kernel, *score;
     gint xres, yres, xoffset, yoffset, xborder, yborder;
     gdouble maxscore, xoff, yoff;
 
-    xres = gwy_data_field_get_xres(field1);
-    yres = gwy_data_field_get_yres(field1);
+    xres = gwy_field_get_xres(field1);
+    yres = gwy_field_get_yres(field1);
     xborder = xres/5;
     yborder = yres/5;
 
-    kernel = gwy_data_field_area_extract(field2, xborder, yborder, xres - 2*xborder, yres - 2*yborder);
-    score = gwy_data_field_new_alike(field1, FALSE);
-    gwy_data_field_correlate(field1, kernel, score, GWY_CORR_SEARCH_COVARIANCE_SCORE);
+    kernel = gwy_field_area_extract(field2, xborder, yborder, xres - 2*xborder, yres - 2*yborder);
+    score = gwy_field_new_alike(field1, FALSE);
+    gwy_field_correlate(field1, kernel, score, GWY_CORR_SEARCH_COVARIANCE_SCORE);
     g_object_unref(kernel);
 
-    if (gwy_data_field_get_local_maxima_list(score, &xoff, &yoff, &maxscore, 1, 0, 0.0, FALSE)) {
+    if (gwy_field_get_local_maxima_list(score, &xoff, &yoff, &maxscore, 1, 0, 0.0, FALSE)) {
         xoffset = GWY_ROUND(xoff) - xres/2;
         yoffset = GWY_ROUND(yoff) - yres/2;
     }
@@ -420,17 +410,17 @@ guess_offsets(ModuleGUI *gui)
     gwy_param_table_set_int(gui->table_correlation, PARAM_SEARCH_YOFFSET, yoffset);
 }
 
-static GwyDataField*
-dir_field(GwyDataField *fieldx, GwyDataField *fieldy)
+static GwyField*
+dir_field(GwyField *fieldx, GwyField *fieldy)
 {
-    GwyDataField *result = gwy_data_field_new_alike(fieldx, TRUE);
-    const gdouble *xdata = gwy_data_field_get_data(fieldx);
-    const gdouble *ydata = gwy_data_field_get_data(fieldy);
-    gdouble *rdata = gwy_data_field_get_data(result);
+    GwyField *result = gwy_field_new_alike(fieldx, TRUE);
+    const gdouble *xdata = gwy_field_get_data(fieldx);
+    const gdouble *ydata = gwy_field_get_data(fieldy);
+    gdouble *rdata = gwy_field_get_data(result);
     gint i, n;
 
-    n = gwy_data_field_get_xres(fieldx)*gwy_data_field_get_yres(fieldx);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(result), NULL);
+    n = gwy_field_get_xres(fieldx)*gwy_field_get_yres(fieldx);
+    gwy_unit_clear(gwy_field_get_unit_z(result));
 
     for (i = 0; i < n; i++)
         rdata[i] = atan2(ydata[i], xdata[i]);
@@ -442,12 +432,12 @@ static gboolean
 execute(ModuleArgs *args, GtkWindow *wait_window)
 {
     GwyParams *params = args->params;
-    GwyDataField *field1 = args->field,
+    GwyField *field1 = args->field,
                  *field2 = gwy_params_get_image(params, PARAM_OTHER_IMAGE),
                  *field3 = gwy_params_get_image(params, PARAM_SECOND_SOURCE),
                  *field4 = gwy_params_get_image(params, PARAM_SECOND_OTHER);
-    GwyDataField *fieldx, *fieldy, *score;
-    GwyDataField *field2b = NULL, *field4b = NULL, *fieldx2 = NULL, *fieldy2 = NULL, *score2 = NULL;
+    GwyField *fieldx, *fieldy, *score;
+    GwyField *field2b = NULL, *field4b = NULL, *fieldx2 = NULL, *fieldy2 = NULL, *score2 = NULL;
     gboolean multiple = gwy_params_get_boolean(params, PARAM_MULTIPLE);
     gint xoffset = gwy_params_get_int(params, PARAM_SEARCH_XOFFSET);
     gint yoffset = gwy_params_get_int(params, PARAM_SEARCH_YOFFSET);
@@ -461,85 +451,85 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
     gboolean ok = FALSE, state_finalised = FALSE;
     gint xres, yres, i, col, row;
 
-    /* Result fields.  These three are always created because gwy_data_field_crosscorrelate_init() uses them. */
-    fieldx = gwy_data_field_new_alike(field1, FALSE);
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_z(fieldx), gwy_data_field_get_si_unit_xy(field1));
-    fieldy = gwy_data_field_new_alike(fieldx, FALSE);
-    score = gwy_data_field_new_alike(field1, FALSE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(score), NULL);
+    /* Result fields.  These three are always created because gwy_field_crosscorrelate_init() uses them. */
+    fieldx = gwy_field_new_alike(field1, FALSE);
+    gwy_unit_assign(gwy_field_get_unit_z(fieldx), gwy_field_get_unit_xy(field1));
+    fieldy = gwy_field_new_alike(fieldx, FALSE);
+    score = gwy_field_new_alike(field1, FALSE);
+    gwy_unit_clear(gwy_field_get_unit_z(score));
 
-    xres = gwy_data_field_get_xres(field1);
-    yres = gwy_data_field_get_yres(field1);
+    xres = gwy_field_get_xres(field1);
+    yres = gwy_field_get_yres(field1);
 
     if (multiple) {
-        fieldx2 = gwy_data_field_new_alike(field1, FALSE);
-        fieldy2 = gwy_data_field_new_alike(field1, FALSE);
-        score2 = gwy_data_field_new_alike(field1, FALSE);
+        fieldx2 = gwy_field_new_alike(field1, FALSE);
+        fieldy2 = gwy_field_new_alike(field1, FALSE);
+        score2 = gwy_field_new_alike(field1, FALSE);
     }
 
     gwy_app_wait_start(wait_window, _("Initializing..."));
 
     /* If a shift is requested, make a copy and shift it. */
     if (xoffset || yoffset) {
-        field2b = gwy_data_field_new_alike(field2, FALSE);
-        gwy_data_field_fill(field2b, gwy_data_field_get_avg(field2));
-        gwy_data_field_area_copy(field2, field2b, 0, 0, -1, -1, xoffset, yoffset);
+        field2b = gwy_field_new_alike(field2, FALSE);
+        gwy_field_fill(field2b, gwy_field_get_avg(field2));
+        gwy_field_area_copy(field2, field2b, 0, 0, -1, -1, xoffset, yoffset);
         field2 = field2b;
 
         if (multiple) {
-            field4b = gwy_data_field_new_alike(field4, FALSE);
-            gwy_data_field_fill(field4b, gwy_data_field_get_avg(field4));
-            gwy_data_field_area_copy(field4, field4b, 0, 0, -1, -1, xoffset, yoffset);
+            field4b = gwy_field_new_alike(field4, FALSE);
+            gwy_field_fill(field4b, gwy_field_get_avg(field4));
+            gwy_field_area_copy(field4, field4b, 0, 0, -1, -1, xoffset, yoffset);
             field4 = field4b;
         }
     }
 
     /* Compute crosscorelation. */
-    state = gwy_data_field_crosscorrelate_init(field1, field2, fieldx, fieldy, score,
-                                               search_x, search_y, window_x, window_y);
-    gwy_data_field_crosscorrelate_set_weights(state, window);
+    state = gwy_field_crosscorrelate_init(field1, field2, fieldx, fieldy, score,
+                                          search_x, search_y, window_x, window_y);
+    gwy_field_crosscorrelate_set_weights(state, window);
 
     if (!gwy_app_wait_set_message(multiple ? _("Correlating first set...") : _("Correlating...")))
         goto end;
 
     do {
-        gwy_data_field_crosscorrelate_iteration(state);
+        gwy_field_crosscorrelate_iteration(state);
         if (!gwy_app_wait_set_fraction(state->fraction))
             goto end;
     } while (state->state != GWY_COMPUTATION_STATE_FINISHED);
 
-    gwy_data_field_crosscorrelate_finalize(state);
+    gwy_field_crosscorrelate_finalize(state);
 
     /* Compute crosscorelation of second set if it is there. */
     if (multiple) {
-        state = gwy_data_field_crosscorrelate_init(field3, field4, fieldx2, fieldy2, score2,
-                                                   search_x, search_y, window_x, window_y);
-        gwy_data_field_crosscorrelate_set_weights(state, window);
+        state = gwy_field_crosscorrelate_init(field3, field4, fieldx2, fieldy2, score2,
+                                              search_x, search_y, window_x, window_y);
+        gwy_field_crosscorrelate_set_weights(state, window);
 
         if (!gwy_app_wait_set_message(_("Correlating second set...")))
             goto end;
 
         do {
-            gwy_data_field_crosscorrelate_iteration(state);
+            gwy_field_crosscorrelate_iteration(state);
             if (!gwy_app_wait_set_fraction(state->fraction))
                 goto end;
         } while (state->state != GWY_COMPUTATION_STATE_FINISHED);
 
-        gwy_data_field_crosscorrelate_finalize(state);
+        gwy_field_crosscorrelate_finalize(state);
 
-        gwy_data_field_linear_combination(fieldx, 0.5, fieldx, 0.5, fieldx2, 0.0);
-        gwy_data_field_linear_combination(fieldy, 0.5, fieldy, 0.5, fieldy2, 0.0);
-        gwy_data_field_linear_combination(score, 0.5, score, 0.5, score2, 0.0);
+        gwy_field_linear_combination(fieldx, 0.5, fieldx, 0.5, fieldx2, 0.0);
+        gwy_field_linear_combination(fieldy, 0.5, fieldy, 0.5, fieldy2, 0.0);
+        gwy_field_linear_combination(score, 0.5, score, 0.5, score2, 0.0);
     }
     state_finalised = TRUE;
 
     if (xoffset)
-        gwy_data_field_add(fieldx, gwy_data_field_jtor(fieldx, xoffset));
+        gwy_field_add(fieldx, gwy_field_jtor(fieldx, xoffset));
     if (yoffset)
-        gwy_data_field_add(fieldy, gwy_data_field_itor(fieldx, yoffset));
+        gwy_field_add(fieldy, gwy_field_itor(fieldx, yoffset));
 
     if (gwy_params_get_boolean(params, PARAM_EXTEND)) {
-        GwyDataField *mask = gwy_data_field_new_alike(fieldx, TRUE);
+        GwyField *mask = gwy_field_new_alike(fieldx, TRUE);
         gint leftadd, rightadd, topadd, bottomadd;
 
         // Crop data if there was a global shift.
@@ -548,26 +538,26 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
         topadd = (yoffset < 0) ? 0 : yoffset;
         bottomadd = (yoffset < 0) ? -yoffset : 0;
 
-        gwy_data_field_area_fill(mask, 0, 0, window_x/2 + 2 + leftadd, yres, 1.0);
-        gwy_data_field_area_fill(mask, 0, 0, xres, window_y/2 + 2 + topadd, 1.0);
-        gwy_data_field_area_fill(mask, xres - window_x/2 - 2 - rightadd, 0, window_x/2 + 2 + rightadd, yres, 1.0);
-        gwy_data_field_area_fill(mask, 0, yres - window_y/2 - 2 - bottomadd, xres, window_y/2 + 2 + bottomadd, 1.0);
-        gwy_data_field_laplace_solve(fieldx, mask, -1, 0.8);
-        gwy_data_field_laplace_solve(fieldy, mask, -1, 0.8);
+        gwy_field_area_fill(mask, NULL, GWY_MASK_IGNORE, 0, 0, window_x/2 + 2 + leftadd, yres, 1.0);
+        gwy_field_area_fill(mask, NULL, GWY_MASK_IGNORE, 0, 0, xres, window_y/2 + 2 + topadd, 1.0);
+        gwy_field_area_fill(mask, NULL, GWY_MASK_IGNORE, xres - window_x/2 - 2 - rightadd, 0, window_x/2 + 2 + rightadd, yres, 1.0);
+        gwy_field_area_fill(mask, NULL, GWY_MASK_IGNORE, 0, yres - window_y/2 - 2 - bottomadd, xres, window_y/2 + 2 + bottomadd, 1.0);
+        gwy_field_laplace_solve(fieldx, mask, -1, 0.8);
+        gwy_field_laplace_solve(fieldy, mask, -1, 0.8);
         g_object_unref(mask);
     }
 
     if (gwy_params_get_boolean(params, PARAM_GAUSSIAN)) {
         gdouble width = gwy_params_get_double(params, PARAM_GAUSSIAN_WIDTH);
-        gwy_data_field_filter_gaussian(fieldx, width);
-        gwy_data_field_filter_gaussian(fieldy, width);
+        gwy_field_filter_gaussian(fieldx, width);
+        gwy_field_filter_gaussian(fieldy, width);
     }
 
     if (output & (1 << OUTPUT_CORRECTED)) {
-        GwyDataField *corrected = gwy_data_field_new_alike(field2, FALSE);
-        const gdouble *xdata = gwy_data_field_get_data(fieldx);
-        const gdouble *ydata = gwy_data_field_get_data(fieldy);
-        gdouble dx = gwy_data_field_get_dx(field1), dy = gwy_data_field_get_dy(field1);
+        GwyField *corrected = gwy_field_new_alike(field2, FALSE);
+        const gdouble *xdata = gwy_field_get_data(fieldx);
+        const gdouble *ydata = gwy_field_get_data(fieldy);
+        gdouble dx = gwy_field_get_dx(field1), dy = gwy_field_get_dy(field1);
         GwyXY *coords = g_new(GwyXY, xres*yres);
 
         for (row = i = 0; row < yres; row++) {
@@ -576,16 +566,15 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
                 coords[i].y = row + ydata[i]/dy + 0.5 - yoffset;
             }
         }
-        gwy_data_field_sample_distorted(field2, corrected, coords,
-                                        GWY_INTERPOLATION_BILINEAR, GWY_EXTERIOR_BORDER_EXTEND, 0.0);
+        gwy_field_sample_distorted(field2, corrected, coords,
+                                   GWY_INTERPOLATION_LINEAR, GWY_EXTERIOR_BORDER, 0.0);
         args->result[OUTPUT_CORRECTED] = corrected;
         g_free(coords);
     }
 
-
     if (output & (1 << OUTPUT_ABS)) {
-        args->result[OUTPUT_ABS] = gwy_data_field_new_alike(fieldx, FALSE);
-        gwy_data_field_hypot_of_fields(args->result[OUTPUT_ABS], fieldx, fieldy);
+        args->result[OUTPUT_ABS] = gwy_field_new_alike(fieldx, FALSE);
+        gwy_field_hypot_of_fields(args->result[OUTPUT_ABS], fieldx, fieldy);
     }
     if (output & (1 << OUTPUT_DIR))
         args->result[OUTPUT_DIR] = dir_field(fieldx, fieldy);
@@ -596,25 +585,25 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
     if (output & (1 << OUTPUT_SCORE))
         args->result[OUTPUT_SCORE] = g_object_ref(score);
     if (gwy_params_get_boolean(params, PARAM_GAUSSIAN)) {
-        args->mask = gwy_data_field_duplicate(score);
-        gwy_data_field_threshold(args->mask, gwy_params_get_double(params, PARAM_GAUSSIAN_WIDTH), 1.0, 0.0);
+        args->mask = gwy_field_copy(score);
+        gwy_field_threshold(args->mask, gwy_params_get_double(params, PARAM_GAUSSIAN_WIDTH), 1.0, 0.0);
     }
 
     ok = TRUE;
 
 end:
     if (!state_finalised)
-        gwy_data_field_crosscorrelate_finalize(state);
+        gwy_field_crosscorrelate_finalize(state);
     gwy_app_wait_finish();
 
     g_object_unref(score);
     g_object_unref(fieldy);
     g_object_unref(fieldx);
-    GWY_OBJECT_UNREF(fieldx2);
-    GWY_OBJECT_UNREF(fieldy2);
-    GWY_OBJECT_UNREF(score2);
-    GWY_OBJECT_UNREF(field2b);
-    GWY_OBJECT_UNREF(field4b);
+    g_clear_object(&fieldx2);
+    g_clear_object(&fieldy2);
+    g_clear_object(&score2);
+    g_clear_object(&field2b);
+    g_clear_object(&field4b);
 
     return ok;
 }
