@@ -1,5 +1,5 @@
 /*
- *  $Id: curvature.c 25076 2022-10-11 13:48:10Z yeti-dn $
+ *  $Id: curvature.c 28861 2025-11-14 13:02:11Z yeti-dn $
  *  Copyright (C) 2009-2022 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.net.
  *
@@ -16,18 +16,11 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyresults.h>
-#include <libprocess/level.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libgwydgets/gwynullstore.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
@@ -59,8 +52,8 @@ typedef struct {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *mask;
+    GwyField *field;
+    GwyField *mask;
     GwyGraphModel *gmodel;
     GwySelection *selection;
     gdouble results[RESULT_NRESULTS];
@@ -71,23 +64,22 @@ typedef struct {
     GtkWidget *dialog;
     GwyParamTable *table_main;
     GwyParamTable *table_results;
-    GwyContainer *data;
     GwyResults *results;
     GwySelection *selection;
 } ModuleGUI;
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             curvature           (GwyContainer *data,
-                                             GwyRunType runtype);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static gboolean         execute             (ModuleArgs *args);
 static GwyDialogOutcome run_gui             (ModuleArgs *args,
-                                             GwyContainer *data,
+                                             GwyFile *data,
                                              gint id);
 static void             param_changed       (ModuleGUI *gui,
                                              gint id);
 static GwyResults*      create_results      (ModuleArgs *args,
-                                             GwyContainer *data,
+                                             GwyFile *data,
                                              gint id);
 static void             preview             (gpointer user_data);
 
@@ -107,11 +99,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("curvature",
-                              (GwyProcessFunc)&curvature,
+                              module_main,
                               N_("/Measure _Features/_Curvature..."),
-                              GWY_STOCK_CURVATURE,
+                              GWY_ICON_CURVATURE,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Calculate overall curvature"));
 
     return TRUE;
@@ -137,19 +129,18 @@ define_module_params(void)
 }
 
 static void
-curvature(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     GwyParams *params;
     ModuleArgs args;
     gint id;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    g_return_if_fail(g_type_from_name("GwyLayerLine"));
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_MASK_FIELD, &args.mask,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_MASK_FIELD, &args.mask,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(args.field);
 
     if (!gwy_require_image_same_units(args.field, data, id, _("Curvature")))
@@ -157,11 +148,12 @@ curvature(GwyContainer *data, GwyRunType runtype)
 
     args.gmodel = gwy_graph_model_new();
     g_object_set(args.gmodel, "title", _("Curvature Sections"), NULL);
-    gwy_graph_model_set_units_from_data_field(args.gmodel, args.field, 1, 0, 0, 1);
-    args.selection = g_object_new(g_type_from_name("GwySelectionLine"), "max-objects", 1024, NULL);
+    gwy_graph_model_set_units_from_field(args.gmodel, args.field, 1, 0, 0, 1);
+    args.selection = gwy_selection_line_new();
+    gwy_selection_set_max_objects(args.selection, 2);
     args.params = params = gwy_params_new_from_settings(define_module_params());
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, id);
         gwy_params_save_to_settings(params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -171,10 +163,8 @@ curvature(GwyContainer *data, GwyRunType runtype)
         execute(&args);
 
     if (gwy_params_get_boolean(params, PARAM_SET_SELECTION)) {
-        gchar *key = g_strdup_printf("/%d/select/line", id);
-        gwy_container_set_object_by_name(data, key, args.selection);
-        g_free(key);
-        gwy_app_channel_log_add_proc(data, id, id);
+        gwy_container_set_object(GWY_CONTAINER(data), gwy_file_key_image_selection(id, "line"), args.selection);
+        gwy_log_add(data, GWY_FILE_IMAGE, id, id);
     }
     if (gwy_params_get_boolean(params, PARAM_PLOT_GRAPH)) {
         GwyAppDataId target_graph_id = gwy_params_get_data_id(params, PARAM_TARGET_GRAPH);
@@ -259,7 +249,7 @@ intersect_with_boundary(gdouble x_0, gdouble y_0, gdouble phi, gdouble w, gdoubl
 
 /* Does not include x and y offsets of the data field */
 static gboolean
-curvature_calculate(GwyDataField *field, GwyDataField *mask, GwyMaskingType masking,
+curvature_calculate(GwyField *field, GwyField *mask, GwyMaskingType masking,
                     gdouble *r, Intersection *i1, Intersection *i2)
 {
     enum { DEGREE = 2 };
@@ -279,16 +269,16 @@ curvature_calculate(GwyDataField *field, GwyDataField *mask, GwyMaskingType mask
         }
     }
 
-    gwy_data_field_fit_poly(field, mask, NTERMS, term_powers, masking != GWY_MASK_INCLUDE, coeffs);
+    gwy_field_fit_poly(field, mask, masking, NTERMS, term_powers, coeffs);
     gwy_debug("NORM a=%g, bx=%g, by=%g, cxx=%g, cxy=%g, cyy=%g",
               coeffs[A], coeffs[BX], coeffs[BY], coeffs[CXX], coeffs[CXY], coeffs[CYY]);
 
     /* Transform coeffs from normalized coordinates to coordinates that are
      * still numerically around 1 but have the right aspect ratio. */
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
-    xreal = gwy_data_field_get_xreal(field);
-    yreal = gwy_data_field_get_yreal(field);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
+    xreal = gwy_field_get_xreal(field);
+    yreal = gwy_field_get_yreal(field);
     qx = 2.0/xreal*xres/(xres - 1.0);
     qy = 2.0/yreal*yres/(yres - 1.0);
     q = sqrt(qx*qy);
@@ -315,14 +305,14 @@ curvature_calculate(GwyDataField *field, GwyDataField *mask, GwyMaskingType mask
     for (i = 0; i < 2; i++)
         ok &= intersect_with_boundary(r[RESULT_X0], r[RESULT_Y0], -r[RESULT_PHI1 + i], xreal, yreal, i1 + i, i2 + i);
 
-    r[RESULT_X0] += gwy_data_field_get_xoffset(field);
-    r[RESULT_Y0] += gwy_data_field_get_yoffset(field);
+    r[RESULT_X0] += gwy_field_get_xoffset(field);
+    r[RESULT_Y0] += gwy_field_get_yoffset(field);
 
     return ok;
 }
 
 static gboolean
-curvature_set_selection(GwyDataField *field,
+curvature_set_selection(GwyField *field,
                         const Intersection *i1, const Intersection *i2,
                         GwySelection *selection)
 {
@@ -331,10 +321,10 @@ curvature_set_selection(GwyDataField *field,
     gint xres, yres;
     guint i;
 
-    xreal = gwy_data_field_get_xreal(field);
-    yreal = gwy_data_field_get_yreal(field);
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
+    xreal = gwy_field_get_xreal(field);
+    yreal = gwy_field_get_yreal(field);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
     for (i = 0; i < 2; i++) {
         xy[0] = CLAMP(i1[i].x, 0, xreal*(xres - 1)/xres);
         xy[1] = CLAMP(i1[i].y, 0, yreal*(yres - 1)/yres);
@@ -347,12 +337,12 @@ curvature_set_selection(GwyDataField *field,
 }
 
 static gboolean
-curvature_plot_graph(GwyDataField *field,
+curvature_plot_graph(GwyField *field,
                      const Intersection *i1, const Intersection *i2,
                      GwyGraphModel *gmodel)
 {
     GwyGraphCurveModel *gcmodel;
-    GwyDataLine *dline;
+    GwyLine *dline;
     gint xres, yres;
     gchar *s;
     guint i;
@@ -373,23 +363,23 @@ curvature_plot_graph(GwyDataField *field,
         }
     }
 
-    dline = gwy_data_line_new(1, 1.0, FALSE);
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
+    dline = gwy_line_new(1, 1.0, FALSE);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
     for (i = 0; i < 2; i++) {
-        gint col1 = gwy_data_field_rtoj(field, i1[i].x);
-        gint row1 = gwy_data_field_rtoi(field, i1[i].y);
-        gint col2 = gwy_data_field_rtoj(field, i2[i].x);
-        gint row2 = gwy_data_field_rtoi(field, i2[i].y);
+        gint col1 = gwy_field_rtoj(field, i1[i].x);
+        gint row1 = gwy_field_rtoi(field, i1[i].y);
+        gint col2 = gwy_field_rtoj(field, i2[i].x);
+        gint row2 = gwy_field_rtoi(field, i2[i].y);
 
-        /* FIXME: We should use gwy_data_field_get_profile_mask() here. */
-        gwy_data_field_get_profile(field, dline,
-                                   CLAMP(col1, 0, xres-1), CLAMP(row1, 0, yres-1),
-                                   CLAMP(col2, 0, xres-1), CLAMP(row2, 0, yres-1),
-                                   -1, 1, GWY_INTERPOLATION_BILINEAR);
-        gwy_data_line_set_offset(dline, i1[i].t/(i2[i].t - i1[i].t)*gwy_data_line_get_real(dline));
+        /* FIXME: We should use gwy_field_get_profile_mask() here. */
+        gwy_field_get_profile(field, dline,
+                              CLAMP(col1, 0, xres-1), CLAMP(row1, 0, yres-1),
+                              CLAMP(col2, 0, xres-1), CLAMP(row2, 0, yres-1),
+                              -1, 1, GWY_INTERPOLATION_LINEAR);
+        gwy_line_set_offset(dline, i1[i].t/(i2[i].t - i1[i].t)*gwy_line_get_real(dline));
         gcmodel = gwy_graph_model_get_curve(gmodel, i);
-        gwy_graph_curve_model_set_data_from_dataline(gcmodel, dline, 0, 0);
+        gwy_graph_curve_model_set_data_from_line(gcmodel, dline, 0, 0);
     }
     g_object_unref(dline);
 
@@ -397,7 +387,7 @@ curvature_plot_graph(GwyDataField *field,
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     GwyDialogOutcome outcome;
     GwyDialog *dialog;
@@ -405,37 +395,29 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     GtkWidget *hbox, *vbox, *dataview, *graph;
     ModuleGUI gui;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
     gui.results = create_results(args, data, id);
 
-    gui.data = gwy_container_new();
-    gwy_container_set_object_by_name(gui.data, "/0/data", args->field);
-    if (args->mask)
-        gwy_container_set_object_by_name(gui.data, "/0/mask", args->mask);
-    gwy_app_sync_data_items(data, gui.data, id, 0, FALSE,
-                            GWY_DATA_ITEM_PALETTE,
-                            GWY_DATA_ITEM_MASK_COLOR,
-                            GWY_DATA_ITEM_RANGE,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
-
     gui.dialog = gwy_dialog_new(_("Curvature"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    hbox = gwy_hbox_new(8);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 8);
     gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
     gwy_dialog_add_content(dialog, hbox, FALSE, FALSE, 0);
 
-    vbox = gwy_vbox_new(4);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
     gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);
 
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SMALL_SIZE, FALSE);
-    gui.selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(dataview), 0, "Line", 2, FALSE);
+    dataview = gwy_create_preview(args->field, args->mask, PREVIEW_SMALL_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_MASK_COLOR
+                        | GWY_FILE_ITEM_RANGE | GWY_FILE_ITEM_REAL_SQUARE);
+    gui.selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(dataview), GWY_TYPE_LAYER_LINE, 2, FALSE);
     g_object_ref(gui.selection);
     gwy_selection_assign(gui.selection, args->selection);
-    g_object_set(gui.selection, "max-objects", 2, NULL);
+    gwy_selection_set_max_objects(gui.selection, 2);
     gtk_box_pack_start(GTK_BOX(vbox), dataview, FALSE, FALSE, 0);
 
     table = gui.table_main = gwy_param_table_new(args->params);
@@ -452,7 +434,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gtk_box_pack_start(GTK_BOX(vbox), gwy_param_table_widget(table), FALSE, FALSE, 0);
     gwy_dialog_add_param_table(dialog, table);
 
-    vbox = gwy_vbox_new(4);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
     gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);
 
     graph = gwy_graph_new(args->gmodel);
@@ -476,7 +458,6 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 
     g_object_unref(gui.results);
     g_object_unref(gui.selection);
-    g_object_unref(gui.data);
 
     return outcome;
 }
@@ -497,7 +478,7 @@ param_changed(ModuleGUI *gui, gint id)
 }
 
 static GwyResults*
-create_results(ModuleArgs *args, GwyContainer *data, gint id)
+create_results(ModuleArgs *args, GwyFile *data, gint id)
 {
     GwyResults *results = gwy_results_new();
 
@@ -519,12 +500,12 @@ create_results(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_results_bind_formats(results, "x0", "y0", NULL);
     gwy_results_bind_formats(results, "r1", "r2", NULL);
 
-    gwy_results_set_unit(results, "x", gwy_data_field_get_si_unit_xy(args->field));
-    gwy_results_set_unit(results, "y", gwy_data_field_get_si_unit_xy(args->field));
-    gwy_results_set_unit(results, "z", gwy_data_field_get_si_unit_z(args->field));
+    gwy_results_set_unit(results, "x", gwy_field_get_unit_xy(args->field));
+    gwy_results_set_unit(results, "y", gwy_field_get_unit_xy(args->field));
+    gwy_results_set_unit(results, "z", gwy_field_get_unit_z(args->field));
 
     gwy_results_fill_filename(results, "file", data);
-    gwy_results_fill_channel(results, "image", data, id);
+    gwy_results_fill_data_name(results, "image", data, GWY_FILE_IMAGE, id);
 
     return results;
 }
@@ -532,7 +513,7 @@ create_results(ModuleArgs *args, GwyContainer *data, gint id)
 static gboolean
 execute(ModuleArgs *args)
 {
-    GwyDataField *field = args->field, *mask = args->mask;
+    GwyField *field = args->field, *mask = args->mask;
     GwyMaskingType masking = gwy_params_get_masking(args->params, PARAM_MASKING, &mask);
     Intersection i1[2], i2[2];
 
@@ -552,7 +533,7 @@ preview(gpointer user_data)
 {
     ModuleGUI *gui = (ModuleGUI*)user_data;
     ModuleArgs *args = gui->args;
-    GwyDataField *mask = args->mask;
+    GwyField *mask = args->mask;
     GwyMaskingType masking = gwy_params_get_masking(args->params, PARAM_MASKING, &mask);
     GwyResults *results = gui->results;
     gdouble *r = args->results;
