@@ -1,5 +1,5 @@
 /*
- *  $Id: cwt.c 23666 2021-05-10 21:56:10Z yeti-dn $
+ *  $Id: cwt.c 28821 2025-11-06 16:00:41Z yeti-dn $
  *  Copyright (C) 2003-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,14 +16,9 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libprocess/inttrans.h>
-#include <libprocess/cwt.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
@@ -35,7 +30,7 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
+    GwyField *field;
 } ModuleArgs;
 
 typedef struct {
@@ -46,8 +41,8 @@ typedef struct {
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             cwt                 (GwyContainer *data,
-                                             GwyRunType runtype);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static GwyDialogOutcome run_gui             (ModuleArgs *args);
 
 static GwyModuleInfo module_info = {
@@ -66,11 +61,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("cwt",
-                              (GwyProcessFunc)&cwt,
+                              module_main,
                               N_("/_Integral Transforms/2D _CWT..."),
-                              GWY_STOCK_CWT,
+                              GWY_ICON_CWT,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Compute continuous wavelet transform"));
 
     return TRUE;
@@ -87,49 +82,47 @@ define_module_params(void)
     paramdef = gwy_param_def_new();
     gwy_param_def_set_function_name(paramdef, gwy_process_func_current());
     gwy_param_def_add_gwyenum(paramdef, PARAM_WAVELET, "wavelet", _("_Wavelet type"),
-                              gwy_2d_cwt_wavelet_type_get_enum(), -1, GWY_2DCWT_GAUSS);
+                              gwy_cwt_2d_wavelet_type_get_enum(), -1, GWY_CWT_2D_GAUSS);
     gwy_param_def_add_double(paramdef, PARAM_SCALE, "scale", _("_Scale"), 0.0, 1000.0, 10.0);
     return paramdef;
 }
 
 static void
-cwt(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
-    GwyDataField *field;
+    GwyField *field;
     GwyParams *params;
     ModuleArgs args;
     gint oldid, newid;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &field,
-                                     GWY_APP_DATA_FIELD_ID, &oldid,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &field,
+                                 GWY_APP_FIELD_ID, &oldid,
+                                 0);
     g_return_if_fail(field);
 
     args.field = field;
     args.params = params = gwy_params_new_from_settings(define_module_params());
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         GwyDialogOutcome outcome = run_gui(&args);
         gwy_params_save_to_settings(params);
         if (outcome == GWY_DIALOG_CANCEL)
             goto end;
     }
 
-    field = gwy_data_field_duplicate(field);
-    gwy_data_field_cwt(field, GWY_INTERPOLATION_LINEAR, /* ignored */
-                       gwy_params_get_double(params, PARAM_SCALE),
-                       gwy_params_get_enum(params, PARAM_WAVELET));
+    field = gwy_field_copy(field);
+    gwy_field_cwt(field, gwy_params_get_double(params, PARAM_SCALE), gwy_params_get_enum(params, PARAM_WAVELET));
 
-    newid = gwy_app_data_browser_add_data_field(field, data, TRUE);
-    gwy_app_sync_data_items(data, data, oldid, newid, FALSE,
-                            GWY_DATA_ITEM_GRADIENT,
-                            GWY_DATA_ITEM_MASK_COLOR,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
+    newid = gwy_file_add_image(data, field);
+
+    gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+    gwy_file_sync_items(data, GWY_FILE_IMAGE, oldid,
+                        data, GWY_FILE_IMAGE, newid,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_MASK_COLOR | GWY_FILE_ITEM_REAL_SQUARE, FALSE);
 
     g_object_unref(field);
-    gwy_app_set_data_field_title(data, newid, _("CWT"));
-    gwy_app_channel_log_add_proc(data, oldid, newid);
+    gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _("CWT"), TRUE);
+    gwy_log_add(data, GWY_FILE_IMAGE, oldid, newid);
 
 end:
     g_object_unref(args.params);
@@ -142,7 +135,7 @@ run_gui(ModuleArgs *args)
     GwyParamTable *table;
     ModuleGUI gui;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
 
     gui.dialog = gwy_dialog_new(_("2D CWT"));
