@@ -1,5 +1,5 @@
 /*
- *  $Id: deconvolve.c 24658 2022-03-10 13:16:56Z yeti-dn $
+ *  $Id: deconvolve.c 28858 2025-11-13 14:32:56Z yeti-dn $
  *  Copyright (C) 2018-2022 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,30 +16,20 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/filters.h>
-#include <libprocess/inttrans.h>
-#include <libprocess/stats.h>
-#include <libprocess/simplefft.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES GWY_RUN_INTERACTIVE
 
 #define field_convolve_default(field, kernel) \
-    gwy_data_field_area_ext_convolve((field), \
+    gwy_field_area_ext_convolve((field), \
                                      0, 0, \
-                                     gwy_data_field_get_xres(field), \
-                                     gwy_data_field_get_yres(field), \
+                                     gwy_field_get_xres(field), \
+                                     gwy_field_get_yres(field), \
                                      (field), (kernel), \
-                                     GWY_EXTERIOR_BORDER_EXTEND, 0.0, TRUE)
+                                     GWY_EXTERIOR_BORDER, 0.0, TRUE)
 
 
 enum {
@@ -92,17 +82,17 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *deconvolved;
-    GwyDataField *difference;
+    GwyField *field;
+    GwyField *deconvolved;
+    GwyField *difference;
 } ModuleArgs;
 
 typedef struct {
     ModuleArgs *args;
     GtkWidget *dialog;
+    GtkWidget *dataview;
     GwyParamTable *table_param;
     GwyParamTable *table_output;
-    GwyContainer *data;
     GwyGraphModel *gmodel;
     GwyGraphArea *area;
     GwySelection *selection;
@@ -113,11 +103,11 @@ typedef struct {
 
 static gboolean         module_register              (void);
 static GwyParamDef*     define_module_params         (void);
-static void             deconvolve                   (GwyContainer *data,
-                                                      GwyRunType runtype);
+static void             module_main                  (GwyFile *data,
+                                                      GwyRunModeFlags mode);
 static void             execute                      (ModuleArgs *args);
 static GwyDialogOutcome run_gui                      (ModuleArgs *args,
-                                                      GwyContainer *data,
+                                                      GwyFile *data,
                                                       gint id);
 static void             param_changed                (ModuleGUI *gui,
                                                       gint id);
@@ -129,17 +119,17 @@ static void             switch_display               (ModuleGUI *gui);
 static void             switch_lcurve                (ModuleGUI *gui);
 static void             clear_lcurve                 (ModuleGUI *gui);
 static void             calculate_lcurve             (ModuleGUI *gui);
-static gboolean         kernel_filter                (GwyContainer *data,
+static gboolean         kernel_filter                (GwyFile *data,
                                                       gint id,
                                                       gpointer user_data);
-static void             deconvolve_with_kernel       (GwyDataField *measured,
-                                                      GwyDataField *tf,
-                                                      GwyDataField *deconv,
-                                                      GwyDataField *difference,
+static void             deconvolve_with_kernel       (GwyField *measured,
+                                                      GwyField *tf,
+                                                      GwyField *deconv,
+                                                      GwyField *difference,
                                                       gdouble sigma);
-static void             adjust_deconv_to_non_integral(GwyDataField *psf);
-static gint             create_output_field          (GwyDataField *field,
-                                                      GwyContainer *data,
+static void             adjust_deconv_to_non_integral(GwyField *psf);
+static gint             create_output_field          (GwyField *field,
+                                                      GwyFile *data,
                                                       gint id,
                                                       const gchar *name);
 
@@ -166,11 +156,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("deconvolve",
-                              (GwyProcessFunc)&deconvolve,
+                              module_main,
                               N_("/M_ultidata/_Deconvolve..."),
-                              GWY_STOCK_DECONVOLVE,
+                              GWY_ICON_DECONVOLVE,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Deconvolve image"));
 
     return TRUE;
@@ -199,7 +189,7 @@ define_module_params(void)
     gwy_param_def_add_boolean(paramdef, PARAM_AS_INTEGRAL, "as_integral", "Normalize as _integral", TRUE);
     gwy_param_def_add_gwyflags(paramdef, PARAM_OUTPUT_TYPE, "output_type", _("Output"),
                                outputs, G_N_ELEMENTS(outputs), (1 << DECONV_OUTPUT_DECONVOLVED));
-    gwy_param_def_add_gwyenum(paramdef, PARAM_DISPLAY, "display", gwy_sgettext("verb|_Display"),
+    gwy_param_def_add_gwyenum(paramdef, PARAM_DISPLAY, "display", C_("verb", "_Display"),
                               displays, G_N_ELEMENTS(displays), DECONV_DISPLAY_DECONVOLVED);
     gwy_param_def_add_gwyenum(paramdef, PARAM_LCURVE, "lcurve", _("_L-curve display"),
                               lcurves, G_N_ELEMENTS(lcurves), DECONV_LCURVE_CURVATURE);
@@ -209,23 +199,23 @@ define_module_params(void)
 }
 
 static void
-deconvolve(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
     DeconvOutputType output;
     GwyDialogOutcome outcome;
     gint id;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
-    g_return_if_fail(GWY_IS_DATA_FIELD(args.field));
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
+    g_return_if_fail(GWY_IS_FIELD(args.field));
 
     args.params = gwy_params_new_from_settings(define_module_params());
-    args.deconvolved = gwy_data_field_new_alike(args.field, TRUE);
-    args.difference = gwy_data_field_new_alike(args.field, TRUE);
+    args.deconvolved = gwy_field_new_alike(args.field, TRUE);
+    args.difference = gwy_field_new_alike(args.field, TRUE);
 
     outcome = run_gui(&args, data, id);
     gwy_params_save_to_settings(args.params);
@@ -251,7 +241,7 @@ end:
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     GtkWidget *hbox, *vbox, *dataview, *notebook, *graph;
     GwyParamTable *table;
@@ -259,26 +249,22 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     GwyDialogOutcome outcome;
     ModuleGUI gui;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
-    gui.data = gwy_container_new();
-    gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(0), args->field);
-    gwy_app_sync_data_items(data, gui.data, id, 0, FALSE,
-                            GWY_DATA_ITEM_GRADIENT,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
 
     gui.dialog = gwy_dialog_new(_("Deconvolve"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
+    dataview = gui.dataview = gwy_create_preview(args->field, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_REAL_SQUARE);
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), FALSE);
 
     notebook = gtk_notebook_new();
     gtk_box_pack_start(GTK_BOX(hbox), notebook, TRUE, TRUE, 0);
 
-    vbox = gwy_vbox_new(4);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
     gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox, gtk_label_new("Parameters"));
 
     gui.gmodel = gwy_graph_model_new();
@@ -325,7 +311,6 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 
     outcome = gwy_dialog_run(dialog);
 
-    g_object_unref(gui.data);
     g_object_unref(gui.gmodel);
     g_free(gui.lcurvedata);
 
@@ -354,7 +339,7 @@ param_changed(ModuleGUI *gui, gint id)
     }
     if (id < 0 || id == PARAM_SIGMA) {
         gdouble sigma = gwy_params_get_double(params, PARAM_SIGMA);
-        gchar *s = g_strdup_printf("%.4g", pow10(sigma));
+        gchar *s = g_strdup_printf("%.4g", gwy_exp10(sigma));
         gwy_selection_set_data(gui->selection, 1, &sigma);
         gwy_param_table_info_set_valuestr(gui->table_param, LABEL_SIGMA, s);
         g_free(s);
@@ -454,13 +439,14 @@ switch_display(ModuleGUI *gui)
 {
     ModuleArgs *args = gui->args;
     DeconvDisplayType display = gwy_params_get_enum(args->params, PARAM_DISPLAY);
+    GwyDataView *dataview = GWY_DATA_VIEW(gui->dataview);
 
     if (display == DECONV_DISPLAY_DATA)
-        gwy_container_set_object(gui->data, gwy_app_get_data_key_for_id(0), args->field);
+        gwy_data_view_set_field(dataview, args->field);
     else if (display == DECONV_DISPLAY_DECONVOLVED)
-        gwy_container_set_object(gui->data, gwy_app_get_data_key_for_id(0), args->deconvolved);
+        gwy_data_view_set_field(dataview, args->deconvolved);
     else
-        gwy_container_set_object(gui->data, gwy_app_get_data_key_for_id(0), args->difference);
+        gwy_data_view_set_field(dataview, args->difference);
 }
 
 static void
@@ -470,26 +456,25 @@ preview(gpointer user_data)
     ModuleArgs *args = gui->args;
 
     execute(args);
-    gwy_data_field_data_changed(args->deconvolved);
-    gwy_data_field_data_changed(args->difference);
+    gwy_field_data_changed(args->deconvolved);
+    gwy_field_data_changed(args->difference);
     gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
 }
 
 static gboolean
-kernel_filter(GwyContainer *data, gint id, gpointer user_data)
+kernel_filter(GwyFile *data, gint id, gpointer user_data)
 {
-    GwyDataField *field = (GwyDataField*)user_data;
-    GwyDataField *kernel = gwy_container_get_object(data, gwy_app_get_data_key_for_id(id));
+    GwyField *field = (GwyField*)user_data;
+    GwyField *kernel = gwy_file_get_image(data, id);
 
-    if (kernel == field)
+    if (!kernel || kernel == field)
         return FALSE;
-    if (gwy_data_field_get_xres(kernel) > gwy_data_field_get_xres(field))
+    if (gwy_field_get_xres(kernel) > gwy_field_get_xres(field))
         return FALSE;
-    if (gwy_data_field_get_yres(kernel) > gwy_data_field_get_yres(field))
+    if (gwy_field_get_yres(kernel) > gwy_field_get_yres(field))
         return FALSE;
 
-    return !gwy_data_field_check_compatibility(kernel, field,
-                                               GWY_DATA_COMPATIBILITY_MEASURE | GWY_DATA_COMPATIBILITY_LATERAL);
+    return !gwy_field_is_incompatible(kernel, field, GWY_DATA_MISMATCH_MEASURE | GWY_DATA_MISMATCH_LATERAL);
 }
 
 static void
@@ -498,44 +483,44 @@ execute(ModuleArgs *args)
     GwyParams *params = args->params;
     gdouble lsigma = gwy_params_get_double(args->params, PARAM_SIGMA);
     gboolean as_integral = gwy_params_get_boolean(args->params, PARAM_AS_INTEGRAL);
-    GwyDataField *kernel = gwy_params_get_image(params, PARAM_KERNEL);
-    GwyDataField *field = args->field, *deconv = args->deconvolved, *difference = args->difference;
+    GwyField *kernel = gwy_params_get_image(params, PARAM_KERNEL);
+    GwyField *field = args->field, *deconv = args->deconvolved, *difference = args->difference;
 
     if (kernel) {
-        deconvolve_with_kernel(field, kernel, deconv, difference, pow10(lsigma));
+        deconvolve_with_kernel(field, kernel, deconv, difference, gwy_exp10(lsigma));
         if (!as_integral)
             adjust_deconv_to_non_integral(deconv);
     }
 }
 
 static void
-deconvolve_with_kernel(GwyDataField *measured, GwyDataField *kernel,
-                       GwyDataField *deconv, GwyDataField *difference,
+deconvolve_with_kernel(GwyField *measured, GwyField *kernel,
+                       GwyField *deconv, GwyField *difference,
                        gdouble sigma)
 {
-    GwyDataField *xm, *xkernel;
+    GwyField *xm, *xkernel;
     gint xres, yres, extx, exty, txres, tyres;
 
-    xres = gwy_data_field_get_xres(measured);
-    yres = gwy_data_field_get_yres(measured);
-    txres = gwy_data_field_get_xres(kernel);
-    tyres = gwy_data_field_get_yres(kernel);
+    xres = gwy_field_get_xres(measured);
+    yres = gwy_field_get_yres(measured);
+    txres = gwy_field_get_xres(kernel);
+    tyres = gwy_field_get_yres(kernel);
     extx = txres/2 + 1;
     exty = tyres/2 + 1;
-    xm = gwy_data_field_extend(measured, extx, extx, exty, exty, GWY_EXTERIOR_MIRROR_EXTEND, 0.0, FALSE);
-    xkernel = gwy_data_field_new_alike(xm, TRUE);
-    gwy_data_field_copy_units(kernel, xkernel);
-    gwy_data_field_area_copy(kernel, xkernel,
-                             0, 0, txres, tyres,
-                             xres/2 + extx - txres/2, yres/2 + exty - tyres/2);
-    gwy_data_field_deconvolve_regularized(xm, xkernel, deconv, sigma);
+    xm = gwy_field_extend(measured, extx, extx, exty, exty, GWY_EXTERIOR_MIRROR, 0.0, FALSE);
+    xkernel = gwy_field_new_alike(xm, TRUE);
+    gwy_field_copy_units(kernel, xkernel);
+    gwy_field_area_copy(kernel, xkernel,
+                        0, 0, txres, tyres,
+                        xres/2 + extx - txres/2, yres/2 + exty - tyres/2);
+    gwy_field_deconvolve_regularized(xm, xkernel, deconv, sigma);
     g_object_unref(xkernel);
     g_object_unref(xm);
-    gwy_data_field_resize(deconv, extx, exty, xres + extx, yres + exty);
+    gwy_field_crop(deconv, extx, exty, xres, yres);
 
-    gwy_data_field_copy(deconv, difference, TRUE);
+    gwy_field_assign(difference, deconv);
     field_convolve_default(difference, kernel);
-    gwy_data_field_subtract_fields(difference, measured, difference);
+    gwy_field_subtract_fields(difference, measured, difference);
 }
 
 static void
@@ -568,8 +553,8 @@ calculate_lcurve(ModuleGUI *gui)
 {
     ModuleArgs *args = gui->args;
     GwyParams *params = args->params;
-    GwyDataField *field = args->field, *lfield, *deconv, *difference;
-    GwyDataField *kernel = gwy_params_get_image(params, PARAM_KERNEL);
+    GwyField *field = args->field, *lfield, *deconv, *difference;
+    GwyField *kernel = gwy_params_get_image(params, PARAM_KERNEL);
     gdouble mean_sigma = gwy_params_get_double(params, PARAM_SIGMA);
     gdouble sigma_range = gwy_params_get_double(params, PARAM_SIGMA_RANGE);
     gdouble lsigma, sigma, max, maxsigma, x;
@@ -584,11 +569,11 @@ calculate_lcurve(ModuleGUI *gui)
 
     gui->nsteps = nsteps = NSTEPS;
     gui->lcurvedata = lcurvedata = g_renew(gdouble, gui->lcurvedata, nsteps*LCURVE_DATA_NTYPES);
-    lfield = gwy_data_field_new_alike(field, TRUE);
-    deconv = gwy_data_field_new_alike(field, TRUE);
-    difference = gwy_data_field_new_alike(field, TRUE);
-    gwy_data_field_copy(field, lfield, TRUE);
-    gwy_data_field_add(lfield, -gwy_data_field_get_avg(field));
+    lfield = gwy_field_new_alike(field, TRUE);
+    deconv = gwy_field_new_alike(field, TRUE);
+    difference = gwy_field_new_alike(field, TRUE);
+    gwy_field_assign(lfield, field);
+    gwy_field_add(lfield, -gwy_field_get_avg(field));
 
     for (i = 0; i < nsteps; i++) {
         if (!gwy_app_wait_set_fraction((gdouble)i/nsteps)) {
@@ -598,13 +583,13 @@ calculate_lcurve(ModuleGUI *gui)
 
         lsigma = mean_sigma - sigma_range/2 + i*sigma_range/(nsteps - 1);
         lcurvedata[nsteps*LCURVE_DATA_LOG10SIGMA + i] = lsigma;
-        sigma = pow10(lsigma);
-        gwy_data_field_fill(deconv, 0);
+        sigma = gwy_exp10(lsigma);
+        gwy_field_fill(deconv, 0);
 
         deconvolve_with_kernel(lfield, kernel, deconv, difference, sigma);
 
-        lcurvedata[nsteps*LCURVE_DATA_DIFFERENCE + i] = sqrt(gwy_data_field_get_mean_square(difference));
-        lcurvedata[nsteps*LCURVE_DATA_RMS + i] = gwy_data_field_get_rms(deconv);
+        lcurvedata[nsteps*LCURVE_DATA_DIFFERENCE + i] = sqrt(gwy_field_get_mean_square(difference));
+        lcurvedata[nsteps*LCURVE_DATA_RMS + i] = gwy_field_get_rms(deconv);
         if (!lcurvedata[nsteps*LCURVE_DATA_RMS + i] || !lcurvedata[nsteps*LCURVE_DATA_DIFFERENCE + i]) {
             clear_lcurve(gui);
             goto end;
@@ -634,7 +619,7 @@ calculate_lcurve(ModuleGUI *gui)
     }
     gui->best_sigma = maxsigma;
 
-    s = g_strdup_printf("%.4g (log<sub>10</sub>: %.4f)", pow10(maxsigma), maxsigma);
+    s = g_strdup_printf("%.4g (log<sub>10</sub>: %.4f)", gwy_exp10(maxsigma), maxsigma);
     gwy_param_table_info_set_valuestr(gui->table_param, LABEL_BEST_SIGMA, s);
     g_free(s);
 
@@ -646,35 +631,36 @@ end:
 }
 
 static gint
-create_output_field(GwyDataField *field,
-                    GwyContainer *data,
+create_output_field(GwyField *field,
+                    GwyFile *data,
                     gint id,
                     const gchar *name)
 {
     gint newid;
 
-    newid = gwy_app_data_browser_add_data_field(field, data, TRUE);
-    gwy_app_sync_data_items(data, data, id, newid, FALSE,
-                            GWY_DATA_ITEM_GRADIENT,
-                            GWY_DATA_ITEM_MASK_COLOR,
-                            0);
-    gwy_app_set_data_field_title(data, newid, name);
-    gwy_app_channel_log_add_proc(data, id, newid);
+    newid = gwy_file_add_image(data, field);
+
+    gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+    gwy_file_sync_items(data, GWY_FILE_IMAGE, id,
+                        data, GWY_FILE_IMAGE, newid,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_MASK_COLOR, FALSE);
+    gwy_file_set_title(data, GWY_FILE_IMAGE, newid, name, TRUE);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, newid);
 
     return newid;
 }
 
 static void
-adjust_deconv_to_non_integral(GwyDataField *deconv)
+adjust_deconv_to_non_integral(GwyField *deconv)
 {
-    GwySIUnit *xyunit, *zunit;
+    GwyUnit *xyunit, *zunit;
 
-    xyunit = gwy_data_field_get_si_unit_xy(deconv);
-    zunit = gwy_data_field_get_si_unit_z(deconv);
-    gwy_si_unit_power_multiply(zunit, 1, xyunit, 2, zunit);
+    xyunit = gwy_field_get_unit_xy(deconv);
+    zunit = gwy_field_get_unit_z(deconv);
+    gwy_unit_power_multiply(zunit, 1, xyunit, 2, zunit);
 
-    gwy_data_field_multiply(deconv, gwy_data_field_get_dx(deconv) * gwy_data_field_get_dy(deconv));
-    gwy_data_field_data_changed(deconv);
+    gwy_field_multiply(deconv, gwy_field_get_dx(deconv) * gwy_field_get_dy(deconv));
+    gwy_field_data_changed(deconv);
 }
 
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
