@@ -1,5 +1,5 @@
 /*
- *  $Id: displfield.c 24796 2022-04-28 15:20:59Z yeti-dn $
+ *  $Id: displfield.c 28905 2025-11-24 15:51:18Z yeti-dn $
  *  Copyright (C) 2019-2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,20 +16,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwythreads.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/correct.h>
-#include <libprocess/filters.h>
-#include <libprocess/stats.h>
-#include <libprocess/synth.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
-#include "libgwyddion/gwyomp.h"
+#include <gwy.h>
+#include "libgwyddion/omp.h"
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
@@ -66,37 +56,36 @@ typedef struct {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *result;
+    GwyField *field;
+    GwyField *result;
 } ModuleArgs;
 
 typedef struct {
     ModuleArgs *args;
     GtkWidget *dialog;
     GwyParamTable *table;
-    GwyContainer *data;
 } ModuleGUI;
 
 static gboolean         module_register      (void);
 static GwyParamDef*     define_module_params (void);
-static void             displ_field          (GwyContainer *data,
-                                              GwyRunType runtype);
+static void             module_main          (GwyFile *data,
+                                              GwyRunModeFlags mode);
 static void             execute              (ModuleArgs *args);
 static GwyDialogOutcome run_gui              (ModuleArgs *args,
-                                              GwyContainer *data,
+                                              GwyFile *data,
                                               gint id);
 static void             param_changed        (ModuleGUI *gui,
                                               gint id);
 static void             preview              (gpointer user_data);
-static gboolean         displ_field_filter   (GwyContainer *data,
+static gboolean         displ_field_filter   (GwyFile *data,
                                               gint id,
                                               gpointer user_data);
-static GwyDataField*    make_displacement_map(guint xres,
+static GwyField*        make_displacement_map(guint xres,
                                               guint yres,
                                               gdouble sigma,
                                               gdouble tau,
                                               GRand *rng);
-static GwyDataField*    make_tear_map        (guint xres,
+static GwyField*        make_tear_map        (guint xres,
                                               guint yres,
                                               gdouble sigma,
                                               gdouble density,
@@ -120,11 +109,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("displfield",
-                              (GwyProcessFunc)&displ_field,
+                              module_main,
                               N_("/_Distortion/Displacement _Field..."),
-                              GWY_STOCK_DISPLACEMENT_FIELD,
+                              GWY_ICON_DISPLACEMENT_FIELD,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               _("Deform image or scan lines in plane"));
 
     return TRUE;
@@ -141,10 +130,10 @@ define_module_params(void)
         { N_("Images (two-dimensional)"),   DISPL_FIELD_METHOD_2D_IMAGES,   },
     };
     static const GwyEnum exteriors[] = {
-        { N_("exterior|Border"),   GWY_EXTERIOR_BORDER_EXTEND, },
-        { N_("exterior|Mirror"),   GWY_EXTERIOR_MIRROR_EXTEND, },
-        { N_("exterior|Periodic"), GWY_EXTERIOR_PERIODIC,      },
-        { N_("exterior|Laplace"),  GWY_EXTERIOR_LAPLACE,       },
+        { gwy_NC("exterior", "Border"),   GWY_EXTERIOR_BORDER, },
+        { gwy_NC("exterior", "Mirror"),   GWY_EXTERIOR_MIRROR, },
+        { gwy_NC("exterior", "Periodic"), GWY_EXTERIOR_PERIODIC,      },
+        { gwy_NC("exterior", "Laplace"),  GWY_EXTERIOR_LAPLACE,       },
     };
     static GwyParamDef *paramdef = NULL;
 
@@ -155,7 +144,7 @@ define_module_params(void)
     gwy_param_def_set_function_name(paramdef, gwy_process_func_current());
     gwy_param_def_add_double(paramdef, PARAM_DENSITY, "density", _("Densi_ty"), 0.0001, 0.25, 0.02);
     gwy_param_def_add_gwyenum(paramdef, PARAM_EXTERIOR, "exterior", _("_Exterior type"),
-                              exteriors, G_N_ELEMENTS(exteriors), GWY_EXTERIOR_BORDER_EXTEND);
+                              exteriors, G_N_ELEMENTS(exteriors), GWY_EXTERIOR_BORDER);
     gwy_param_def_add_enum(paramdef, PARAM_INTERP, "interp", NULL,
                            GWY_TYPE_INTERPOLATION_TYPE, GWY_INTERPOLATION_LINEAR);
     gwy_param_def_add_gwyenum(paramdef, PARAM_METHOD, "method", _("_Method"),
@@ -171,21 +160,21 @@ define_module_params(void)
 }
 
 static void
-displ_field(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     ModuleArgs args;
     gint id, newid;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
-    args.result = gwy_data_field_duplicate(args.field);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
+    args.result = gwy_field_copy(args.field);
     args.params = gwy_params_new_from_settings(define_module_params());
     sanitise_params(&args);
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, id);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -194,14 +183,14 @@ displ_field(GwyContainer *data, GwyRunType runtype)
     if (outcome != GWY_DIALOG_HAVE_RESULT)
         execute(&args);
 
-    newid = gwy_app_data_browser_add_data_field(args.result, data, TRUE);
-    gwy_app_sync_data_items(data, data, id, newid, FALSE,
-                            GWY_DATA_ITEM_PALETTE,
-                            GWY_DATA_ITEM_RANGE_TYPE,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
-    gwy_app_set_data_field_title(data, newid, _("Distorted"));
-    gwy_app_channel_log_add_proc(data, id, newid);
+    newid = gwy_file_add_image(data, args.result);
+
+    gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+    gwy_file_sync_items(data, GWY_FILE_IMAGE, id,
+                        data, GWY_FILE_IMAGE, newid,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_COLOR_MAPPING | GWY_FILE_ITEM_REAL_SQUARE, FALSE);
+    gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _("Distorted"), TRUE);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, newid);
 
 end:
     g_object_unref(args.params);
@@ -209,30 +198,23 @@ end:
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     ModuleGUI gui;
     GtkWidget *hbox, *dataview;
     GwyDialog *dialog;
     GwyParamTable *table;
-    GwyDialogOutcome outcome;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
-    gui.data = gwy_container_new();
-    gwy_container_set_object_by_name(gui.data, "/0/data", args->result);
-    gwy_app_sync_data_items(data, gui.data, id, 0, FALSE,
-                            GWY_DATA_ITEM_PALETTE,
-                            GWY_DATA_ITEM_MASK_COLOR,
-                            GWY_DATA_ITEM_RANGE,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
 
     gui.dialog = gwy_dialog_new(_("Displacement Field"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_UPDATE, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
+    dataview = gwy_create_preview(args->result, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_RANGE | GWY_FILE_ITEM_REAL_SQUARE);
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), FALSE);
 
     table = gui.table = gwy_param_table_new(args->params);
@@ -267,11 +249,8 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 
     g_signal_connect_swapped(table, "param-changed", G_CALLBACK(param_changed), &gui);
     gwy_dialog_set_preview_func(dialog, GWY_PREVIEW_IMMEDIATE, preview, &gui, NULL);
-    outcome = gwy_dialog_run(dialog);
 
-    g_object_unref(gui.data);
-
-    return outcome;
+    return gwy_dialog_run(dialog);
 }
 
 static void
@@ -302,26 +281,24 @@ param_changed(ModuleGUI *gui, gint id)
 }
 
 static gboolean
-displ_field_filter(GwyContainer *data, gint id, gpointer user_data)
+displ_field_filter(GwyFile *data, gint id, gpointer user_data)
 {
-    GwyDataField *other_image, *field = (GwyDataField*)user_data;
-    GwySIUnit *fxyunit, *dzunit;
+    GwyField *other_image, *field = (GwyField*)user_data;
+    GwyUnit *fxyunit, *dzunit;
 
-    if (!gwy_container_gis_object(data, gwy_app_get_data_key_for_id(id), &other_image))
+    if (!(other_image = gwy_file_get_image(data, id)))
         return FALSE;
     if (other_image == field)
         return FALSE;
-    if (gwy_data_field_check_compatibility(other_image, field,
-                                           GWY_DATA_COMPATIBILITY_RES
-                                           | GWY_DATA_COMPATIBILITY_REAL
-                                           | GWY_DATA_COMPATIBILITY_LATERAL))
+    if (gwy_field_is_incompatible(other_image, field,
+                                  GWY_DATA_MISMATCH_RES | GWY_DATA_MISMATCH_REAL | GWY_DATA_MISMATCH_LATERAL))
         return FALSE;
 
     /* Values of the distortion field are lateral offsets in the image. */
-    fxyunit = gwy_data_field_get_si_unit_xy(field);
-    dzunit = gwy_data_field_get_si_unit_z(other_image);
+    fxyunit = gwy_field_get_unit_xy(field);
+    dzunit = gwy_field_get_unit_z(other_image);
 
-    return gwy_si_unit_equal(fxyunit, dzunit);
+    return gwy_unit_equal(fxyunit, dzunit);
 }
 
 static void
@@ -330,7 +307,7 @@ preview(gpointer user_data)
     ModuleGUI *gui = (ModuleGUI*)user_data;
 
     execute(gui->args);
-    gwy_data_field_data_changed(gui->args->result);
+    gwy_field_data_changed(gui->args->result);
     gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
 }
 
@@ -349,17 +326,17 @@ distort_func_2d(gdouble x, gdouble y, gdouble *px, gdouble *py, gpointer user_da
     *py = y - dd->ydata[k];
 }
 
-static GwyDataField*
+static GwyField*
 create_displ_field(GwyParams *params, gint id)
 {
-    GwyDataField *displfield = gwy_params_get_image(params, id);
-    return displfield ? gwy_data_field_duplicate(displfield) : NULL;
+    GwyField *displfield = gwy_params_get_image(params, id);
+    return displfield ? gwy_field_copy(displfield) : NULL;
 }
 
 static void
 execute(ModuleArgs *args)
 {
-    GwyDataField *result = args->result, *field = args->field, *xdisplfield = NULL, *ydisplfield = NULL;
+    GwyField *result = args->result, *field = args->field, *xdisplfield = NULL, *ydisplfield = NULL;
     GwyParams *params = args->params;
     DisplFieldMethod method = gwy_params_get_enum(params, PARAM_METHOD);
     gdouble sigma = gwy_params_get_double(params, PARAM_SIGMA);
@@ -371,9 +348,9 @@ execute(ModuleArgs *args)
     GRand *rng;
     gint xres, yres;
 
-    gwy_clear(&dd, 1);
-    xres = dd.xres = gwy_data_field_get_xres(field);
-    yres = dd.yres = gwy_data_field_get_yres(field);
+    gwy_clear1(dd);
+    xres = dd.xres = gwy_field_get_xres(field);
+    yres = dd.yres = gwy_field_get_yres(field);
     rng = g_rand_new_with_seed(gwy_params_get_int(params, PARAM_SEED));
 
     if (method == DISPL_FIELD_METHOD_2D_GAUSSIAN) {
@@ -386,33 +363,33 @@ execute(ModuleArgs *args)
         xdisplfield = make_tear_map(xres, yres, sigma, density, tau, rng);
     else if (method == DISPL_FIELD_METHOD_1D_IMAGE) {
         if ((xdisplfield = create_displ_field(params, PARAM_X_DISPLACEMENT)))
-            gwy_data_field_multiply(xdisplfield, 1.0/gwy_data_field_get_dx(field));
+            gwy_field_multiply(xdisplfield, 1.0/gwy_field_get_dx(field));
     }
     else if (method == DISPL_FIELD_METHOD_2D_IMAGES) {
         if ((xdisplfield = create_displ_field(params, PARAM_X_DISPLACEMENT))
             && (ydisplfield = create_displ_field(params, PARAM_Y_DISPLACEMENT))) {
-            gwy_data_field_multiply(xdisplfield, 1.0/gwy_data_field_get_dx(field));
-            gwy_data_field_multiply(ydisplfield, 1.0/gwy_data_field_get_dy(field));
+            gwy_field_multiply(xdisplfield, 1.0/gwy_field_get_dx(field));
+            gwy_field_multiply(ydisplfield, 1.0/gwy_field_get_dy(field));
         }
         else {
-            GWY_OBJECT_UNREF(xdisplfield);
-            GWY_OBJECT_UNREF(ydisplfield);
+            g_clear_object(&xdisplfield);
+            g_clear_object(&ydisplfield);
         }
     }
 
     if (xdisplfield) {
         if (!ydisplfield)
-            ydisplfield = gwy_data_field_new_alike(xdisplfield, TRUE);
+            ydisplfield = gwy_field_new_alike(xdisplfield, TRUE);
 
-        dd.xdata = gwy_data_field_get_data_const(xdisplfield);
-        dd.ydata = gwy_data_field_get_data_const(ydisplfield);
-        gwy_data_field_distort(field, result, &distort_func_2d, &dd, interp, exterior, 0.0);
+        dd.xdata = gwy_field_get_data_const(xdisplfield);
+        dd.ydata = gwy_field_get_data_const(ydisplfield);
+        gwy_field_distort(field, result, &distort_func_2d, &dd, interp, exterior, 0.0);
     }
     else
-        gwy_data_field_copy(field, result, FALSE);
+        gwy_field_copy_data(field, result);
 
-    GWY_OBJECT_UNREF(xdisplfield);
-    GWY_OBJECT_UNREF(ydisplfield);
+    g_clear_object(&xdisplfield);
+    g_clear_object(&ydisplfield);
     g_rand_free(rng);
 }
 
@@ -465,33 +442,33 @@ fill_tear(gdouble *d, gdouble *m,
     }
 }
 
-static GwyDataField*
+static GwyField*
 make_displacement_map(guint xres, guint yres,
                       gdouble sigma, gdouble tau,
                       GRand *rng)
 {
-    GwyDataField *field = gwy_data_field_new(xres, yres, 1.0, 1.0, TRUE);
-    gwy_data_field_synth_gaussian_displacement(field, sigma, tau, rng);
+    GwyField *field = gwy_field_new(xres, yres, 1.0, 1.0, TRUE);
+    gwy_field_synth_gaussian_displacement(field, sigma, tau, rng);
     return field;
 }
 
-static GwyDataField*
+static GwyField*
 make_tear_map(guint xres, guint yres,
               gdouble sigma, gdouble density, gdouble length,
               GRand *rng)
 {
-    GwyDataField *field, *mask;
+    GwyField *field, *mask;
     gdouble *d, *m;
     guint n, k;
 
-    field = gwy_data_field_new(xres, yres, 1.0, 1.0, TRUE);
+    field = gwy_field_new(xres, yres, 1.0, 1.0, TRUE);
     n = GWY_ROUND(0.5*xres*yres*density/length);
     if (!n)
         return field;
 
-    mask = gwy_data_field_new(xres, yres, 1.0, 1.0, TRUE);
-    d = gwy_data_field_get_data(field);
-    m = gwy_data_field_get_data(mask);
+    mask = gwy_field_new(xres, yres, 1.0, 1.0, TRUE);
+    d = gwy_field_get_data(field);
+    m = gwy_field_get_data(mask);
 
     for (k = 0; k < n; k++) {
         gint col, row, len, count = 0;
@@ -515,7 +492,7 @@ make_tear_map(guint xres, guint yres,
         fill_tear(d, m, xres, yres, col, row, len, value);
     }
 
-    gwy_data_field_laplace_solve(field, mask, 0, 0.5);
+    gwy_field_laplace_solve(field, mask, 0, 0.5);
     g_object_unref(mask);
 
     return field;
@@ -534,12 +511,12 @@ sanitise_params(ModuleArgs *args)
     if (method == DISPL_FIELD_METHOD_2D_IMAGES
         && (x_is_none
             || y_is_none
-            || !displ_field_filter(gwy_app_data_browser_get(xdisplid.datano), xdisplid.id, args->field)
-            || !displ_field_filter(gwy_app_data_browser_get(ydisplid.datano), ydisplid.id, args->field)))
+            || !displ_field_filter(gwy_data_browser_get_file(xdisplid.datano), xdisplid.id, args->field)
+            || !displ_field_filter(gwy_data_browser_get_file(ydisplid.datano), ydisplid.id, args->field)))
         gwy_params_reset(params, PARAM_METHOD);
 
     if (method == DISPL_FIELD_METHOD_1D_IMAGE
-        && (x_is_none || !displ_field_filter(gwy_app_data_browser_get(xdisplid.datano), xdisplid.id, args->field)))
+        && (x_is_none || !displ_field_filter(gwy_data_browser_get_file(xdisplid.datano), xdisplid.id, args->field)))
         gwy_params_reset(params, PARAM_METHOD);
 }
 
