@@ -1,5 +1,5 @@
 /*
- *  $Id: drift.c 25638 2023-09-07 12:23:28Z yeti-dn $
+ *  $Id: drift.c 29023 2025-12-19 13:06:16Z yeti-dn $
  *  Copyright (C) 2007-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -18,20 +18,11 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwythreads.h>
-#include <libprocess/linestats.h>
-#include <libprocess/gwyprocess.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libprocess/interpolation.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
-#include "libgwyddion/gwyomp.h"
+#include <gwy.h>
+#include "libgwyddion/omp.h"
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_INTERACTIVE | GWY_RUN_IMMEDIATE)
@@ -56,10 +47,10 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *mask;
-    GwyDataField *result;
-    GwyDataLine *drift;
+    GwyField *field;
+    GwyField *mask;
+    GwyField *result;
+    GwyLine *drift;
 } ModuleArgs;
 
 typedef struct {
@@ -67,32 +58,31 @@ typedef struct {
     GtkWidget *dialog;
     GwyParamTable *table;
     GwyGraphModel *gmodel;
-    GwyDataView *dataview;
-    GwyPixmapLayer *mlayer;
-    GwyPixmapLayer *blayer;
-    GwyContainer *data;
+    GtkWidget *dataview;
+    GwyDataViewLayer *mlayer;
+    GwyDataViewLayer *blayer;
 } ModuleGUI;
 
 static gboolean         module_register              (void);
 static GwyParamDef*     define_module_params         (void);
-static void             compensate_drift             (GwyContainer *data,
-                                                      GwyRunType runtype);
+static void             module_main                  (GwyFile *data,
+                                                      GwyRunModeFlags mode);
 static void             gather_quarks_for_one_image  (GwyContainer *data,
                                                       gint id,
                                                       GArray *quarks);
 static void             apply_correction_to_one_image(ModuleArgs *args,
-                                                      GwyContainer *data,
+                                                      GwyFile *data,
                                                       gint id);
 static void             execute                      (ModuleArgs *args);
 static GwyDialogOutcome run_gui                      (ModuleArgs *args,
-                                                      GwyContainer *data,
+                                                      GwyFile *data,
                                                       gint id);
 static void             param_changed                (ModuleGUI *gui,
                                                       gint id);
 static void             preview                      (gpointer user_data);
-static void             mask_process                 (GwyDataField *maskfield,
-                                                      GwyDataLine *drift);
-static void             gwy_data_field_normalize_rows(GwyDataField *field);
+static void             mask_process                 (GwyField *maskfield,
+                                                      GwyLine *drift);
+static void             gwy_field_normalize_rows     (GwyField *field);
 static void             match_line                   (gint res,
                                                       const gdouble *ref,
                                                       const gdouble *cmp,
@@ -100,18 +90,18 @@ static void             match_line                   (gint res,
                                                       gdouble *offset,
                                                       gdouble *score,
                                                       gdouble *d);
-static void             calculate_correlation_scores (GwyDataField *field,
+static void             calculate_correlation_scores (GwyField *field,
                                                       gint range,
                                                       gint maxoffset,
                                                       gdouble supersample,
                                                       GwyInterpolationType interp,
-                                                      GwyDataField *scores,
-                                                      GwyDataField *offsets);
-static void             calculate_drift_very_naive   (GwyDataField *offsets,
-                                                      GwyDataField *scores,
-                                                      GwyDataLine *drift);
-static void             apply_drift                  (GwyDataField *field,
-                                                      GwyDataLine *drift,
+                                                      GwyField *scores,
+                                                      GwyField *offsets);
+static void             calculate_drift_very_naive   (GwyField *offsets,
+                                                      GwyField *scores,
+                                                      GwyLine *drift);
+static void             apply_drift                  (GwyField *field,
+                                                      GwyLine *drift,
                                                       GwyInterpolationType interp);
 
 static GwyModuleInfo module_info = {
@@ -130,11 +120,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("drift",
-                              (GwyProcessFunc)&compensate_drift,
+                              module_main,
                               N_("/_Distortion/Compensate _Drift..."),
-                              GWY_STOCK_DRIFT,
+                              GWY_ICON_DRIFT,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Evaluate/correct thermal drift in fast scan axis"));
 
     return TRUE;
@@ -162,7 +152,7 @@ define_module_params(void)
     gwy_param_def_add_boolean(paramdef, PARAM_DO_CORRECT, "do-correct", _("Correct _data"), TRUE);
     gwy_param_def_add_boolean(paramdef, PARAM_DO_PLOT, "do-plot", _("Plot drift _graph"), TRUE);
     gwy_param_def_add_boolean(paramdef, PARAM_EXCLUDE_LINEAR, "exclude-linear", _("_Exclude linear skew"), FALSE);
-    gwy_param_def_add_gwyenum(paramdef, PARAM_DISPLAY, "display", gwy_sgettext("verb|Display"),
+    gwy_param_def_add_gwyenum(paramdef, PARAM_DISPLAY, "display", C_("verb", "Display"),
                               previews, G_N_ELEMENTS(previews), PREVIEW_MASK);
     gwy_param_def_add_target_graph(paramdef, PARAM_TARGET_GRAPH, "target_graph", NULL);
     gwy_param_def_add_mask_color(paramdef, PARAM_MASK_COLOR, NULL, NULL);
@@ -170,34 +160,34 @@ define_module_params(void)
 }
 
 static void
-compensate_drift(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
-    enum { compat_flags = GWY_DATA_COMPATIBILITY_RES | GWY_DATA_COMPATIBILITY_REAL | GWY_DATA_COMPATIBILITY_LATERAL };
+    enum { compat_flags = GWY_DATA_MISMATCH_RES | GWY_DATA_MISMATCH_REAL | GWY_DATA_MISMATCH_LATERAL };
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     ModuleArgs args;
     GwyParams *params;
-    GwyDataField *field, *mask, *sfield;
+    GwyField *field, *mask, *sfield;
     gboolean do_plot, do_correct, new_image, distribute;
     GArray *undo_quarks;
     gint *image_ids;
     gint id, i;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &field,
-                                     GWY_APP_MASK_FIELD, &mask,
-                                     GWY_APP_SHOW_FIELD, &sfield,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &field,
+                                 GWY_APP_MASK_FIELD, &mask,
+                                 GWY_APP_SHOW_FIELD, &sfield,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(field);
 
     args.field = field;
-    args.mask = gwy_data_field_new_alike(field, TRUE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(args.mask), NULL);
-    args.result = gwy_data_field_new_alike(field, TRUE);
-    args.drift = gwy_data_line_new(gwy_data_field_get_yres(field), gwy_data_field_get_yreal(field), TRUE);
+    args.mask = gwy_field_new_alike(field, TRUE);
+    gwy_unit_clear(gwy_field_get_unit_z(args.mask));
+    args.result = gwy_field_new_alike(field, TRUE);
+    args.drift = gwy_line_new(gwy_field_get_yres(field), gwy_field_get_yreal(field), TRUE);
     args.params = params = gwy_params_new_from_settings(define_module_params());
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, id);
         gwy_params_save_to_settings(params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -216,7 +206,7 @@ compensate_drift(GwyContainer *data, GwyRunType runtype)
         GwyGraphCurveModel *gcmodel;
 
         gmodel = gwy_graph_model_new();
-        gwy_graph_model_set_units_from_data_line(gmodel, args.drift);
+        gwy_graph_model_set_units_from_line(gmodel, args.drift);
         g_object_set(gmodel,
                      "title", _("Drift"),
                      "axis-label-left", _("drift"),
@@ -224,7 +214,7 @@ compensate_drift(GwyContainer *data, GwyRunType runtype)
                      NULL);
 
         gcmodel = gwy_graph_curve_model_new();
-        gwy_graph_curve_model_set_data_from_dataline(gcmodel, args.drift, -1, -1);
+        gwy_graph_curve_model_set_data_from_line(gcmodel, args.drift, -1, -1);
         g_object_set(gcmodel, "description", _("x-axis drift"), NULL);
         gwy_graph_model_add_curve(gmodel, gcmodel);
         g_object_unref(gcmodel);
@@ -238,28 +228,28 @@ compensate_drift(GwyContainer *data, GwyRunType runtype)
     if (!distribute) {
         if (!new_image) {
             undo_quarks = g_array_new(FALSE, FALSE, sizeof(GQuark));
-            gather_quarks_for_one_image(data, id, undo_quarks);
-            gwy_app_undo_qcheckpointv(data, undo_quarks->len, (GQuark*)undo_quarks->data);
+            gather_quarks_for_one_image(GWY_CONTAINER(data), id, undo_quarks);
+            gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), undo_quarks->len, (GQuark*)undo_quarks->data);
             g_array_free(undo_quarks, TRUE);
         }
         apply_correction_to_one_image(&args, data, id);
         goto end;
     }
 
-    image_ids = gwy_app_data_browser_get_data_ids(data);
+    image_ids = gwy_file_get_ids(data, GWY_FILE_IMAGE);
     if (!new_image) {
         undo_quarks = g_array_new(FALSE, FALSE, sizeof(GQuark));
         for (i = 0; image_ids[i] != -1; i++) {
-            GwyDataField *otherfield = gwy_container_get_object(data, gwy_app_get_data_key_for_id(image_ids[i]));
-            if (!gwy_data_field_check_compatibility(field, otherfield, compat_flags))
-                gather_quarks_for_one_image(data, image_ids[i], undo_quarks);
+            GwyField *otherfield = gwy_file_get_image(data, image_ids[i]);
+            if (!gwy_field_is_incompatible(field, otherfield, compat_flags))
+                gather_quarks_for_one_image(GWY_CONTAINER(data), image_ids[i], undo_quarks);
         }
-        gwy_app_undo_qcheckpointv(data, undo_quarks->len, (GQuark*)undo_quarks->data);
+        gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), undo_quarks->len, (GQuark*)undo_quarks->data);
         g_array_free(undo_quarks, TRUE);
     }
     for (i = 0; image_ids[i] != -1; i++) {
-        GwyDataField *otherfield = gwy_container_get_object(data, gwy_app_get_data_key_for_id(image_ids[i]));
-        if (gwy_data_field_check_compatibility(field, otherfield, compat_flags))
+        GwyField *otherfield = gwy_file_get_image(data, image_ids[i]);
+        if (gwy_field_is_incompatible(field, otherfield, compat_flags))
             continue;
 
         apply_correction_to_one_image(&args, data, image_ids[i]);
@@ -279,81 +269,81 @@ gather_quarks_for_one_image(GwyContainer *data, gint id, GArray *quarks)
     GObject *object;
     GQuark quark;
 
-    quark = gwy_app_get_data_key_for_id(id);
+    quark = gwy_file_key_image(id);
     object = gwy_container_get_object(data, quark);
-    g_assert(GWY_IS_DATA_FIELD(object));
+    g_assert(GWY_IS_FIELD(object));
     g_array_append_val(quarks, quark);
 
-    quark = gwy_app_get_mask_key_for_id(id);
-    if (gwy_container_gis_object(data, quark, &object) && GWY_IS_DATA_FIELD(object))
+    quark = gwy_file_key_image_mask(id);
+    if (gwy_container_gis_object(data, quark, &object) && GWY_IS_FIELD(object))
         g_array_append_val(quarks, quark);
 
-    quark = gwy_app_get_show_key_for_id(id);
-    if (gwy_container_gis_object(data, quark, &object) && GWY_IS_DATA_FIELD(object))
+    quark = gwy_file_key_image_picture(id);
+    if (gwy_container_gis_object(data, quark, &object) && GWY_IS_FIELD(object))
         g_array_append_val(quarks, quark);
 }
 
 static void
-apply_correction_to_one_image(ModuleArgs *args, GwyContainer *data, gint id)
+apply_correction_to_one_image(ModuleArgs *args, GwyFile *data, gint id)
 {
     GwyInterpolationType interp = gwy_params_get_enum(args->params, PARAM_INTERP);
     gboolean new_image = gwy_params_get_boolean(args->params, PARAM_NEW_IMAGE);
     gboolean distribute = gwy_params_get_boolean(args->params, PARAM_DISTRIBUTE);
-    GwyDataField *field, *mask = NULL, *show = NULL;
+    GwyField *field = gwy_file_get_image(data, id), *mask = NULL, *show = NULL;
     gchar *title, *newtitle;
     gint newid;
 
-    field = gwy_container_get_object(data, gwy_app_get_data_key_for_id(id));
-    g_assert(GWY_IS_DATA_FIELD(field));
-    gwy_container_gis_object(data, gwy_app_get_mask_key_for_id(id), &mask);
-    gwy_container_gis_object(data, gwy_app_get_show_key_for_id(id), &show);
+    g_assert(GWY_IS_FIELD(field));
+    mask = gwy_file_get_image_mask(data, id);
+    gwy_container_gis_object(GWY_CONTAINER(data), gwy_file_key_image_picture(id), &show);
 
     if (!new_image) {
         apply_drift(field, args->drift, interp);
-        gwy_data_field_data_changed(field);
+        gwy_field_data_changed(field);
         if (mask) {
             apply_drift(mask, args->drift, GWY_INTERPOLATION_ROUND);
-            gwy_data_field_data_changed(mask);
+            gwy_field_data_changed(mask);
         }
         if (show) {
             apply_drift(show, args->drift, interp);
-            gwy_data_field_data_changed(show);
+            gwy_field_data_changed(show);
         }
-        gwy_app_channel_log_add_proc(data, id, id);
+        gwy_log_add(data, GWY_FILE_IMAGE, id, id);
         return;
     }
 
-    field = gwy_data_field_duplicate(field);
+    field = gwy_field_copy(field);
     apply_drift(field, args->drift, interp);
-    newid = gwy_app_data_browser_add_data_field(field, data, !distribute);
+    newid = gwy_file_add_image(data, field);
+    if (!distribute)
+        gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
     g_object_unref(field);
-    title = gwy_app_get_data_field_title(data, id);
+    title = gwy_file_get_display_title(data, GWY_FILE_IMAGE, id);
     newtitle = g_strdup_printf("%s (%s)", title, _("Drift-corrected"));
-    gwy_app_set_data_field_title(data, newid, newtitle);
+    gwy_file_set_title(data, GWY_FILE_IMAGE, newid, newtitle, TRUE);
     g_free(newtitle);
     g_free(title);
-    gwy_app_sync_data_items(data, data, id, newid, FALSE,
-                            GWY_DATA_ITEM_GRADIENT,
-                            GWY_DATA_ITEM_RANGE,
-                            GWY_DATA_ITEM_MASK_COLOR,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
-    gwy_app_channel_log_add_proc(data, id, newid);
+    gwy_file_sync_items(data, GWY_FILE_IMAGE, id,
+                        data, GWY_FILE_IMAGE, newid,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_RANGE
+                        | GWY_FILE_ITEM_MASK_COLOR | GWY_FILE_ITEM_REAL_SQUARE,
+                        FALSE);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, newid);
 
     if (mask) {
-        mask = gwy_data_field_duplicate(mask);
+        mask = gwy_field_copy(mask);
         apply_drift(mask, args->drift, GWY_INTERPOLATION_ROUND);
-        gwy_container_pass_object(data, gwy_app_get_mask_key_for_id(newid), mask);
+        gwy_file_pass_image_mask(data, newid, mask);
     }
     if (show) {
-        show = gwy_data_field_duplicate(show);
+        show = gwy_field_copy(show);
         apply_drift(show, args->drift, interp);
-        gwy_container_pass_object(data, gwy_app_get_show_key_for_id(newid), show);
+        gwy_container_pass_object(GWY_CONTAINER(data), gwy_file_key_image_picture(newid), show);
     }
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     GwyDialogOutcome outcome;
     GwyDialog *dialog;
@@ -361,32 +351,21 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     GtkWidget *hbox, *dataview;
     ModuleGUI gui;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
-    gui.data = gwy_container_new();
     /* Create an empty graph model just for compatibility check. */
     gui.gmodel = gwy_graph_model_new();
-    gwy_graph_model_set_units_from_data_field(gui.gmodel, args->field, 1, 0, 1, 0);
-    gwy_container_set_object_by_name(gui.data, "/0/data", args->field);
-    gwy_container_set_object_by_name(gui.data, "/0/mask", args->mask);
-    gwy_container_set_object_by_name(gui.data, "/1/data", args->result);
-    gwy_app_sync_data_items(data, gui.data, id, 0, FALSE,
-                            GWY_DATA_ITEM_PALETTE,
-                            GWY_DATA_ITEM_RANGE,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            GWY_DATA_ITEM_MASK_COLOR,
-                            0);
+    gwy_graph_model_set_units_from_field(gui.gmodel, args->field, 1, 0, 1, 0);
 
     gui.dialog = gwy_dialog_new(_("Compensate Drift"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_UPDATE, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, TRUE);
-    gui.dataview = GWY_DATA_VIEW(dataview);
-    gui.blayer = g_object_ref(gwy_data_view_get_base_layer(gui.dataview));
-    gui.mlayer = g_object_ref(gwy_data_view_get_alpha_layer(gui.dataview));
-
-    hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), gui.dataview, FALSE);
+    dataview = gui.dataview = gwy_create_preview(args->field, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_RANGE
+                        | GWY_FILE_ITEM_REAL_SQUARE | GWY_FILE_ITEM_MASK_COLOR);
+    hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), FALSE);
 
     table = gui.table = gwy_param_table_new(args->params);
 
@@ -398,7 +377,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_param_table_append_separator(table);
     gwy_param_table_append_radio(table, PARAM_DISPLAY);
     gwy_param_table_append_separator(table);
-    gwy_param_table_append_mask_color(table, PARAM_MASK_COLOR, gui.data, 0, NULL, -1);
+    gwy_param_table_append_image_mask_color(table, PARAM_MASK_COLOR, GWY_DATA_VIEW(gui.dataview), NULL, -1);
 
     gwy_param_table_append_header(table, -1, _("Output"));
     gwy_param_table_append_checkbox(table, PARAM_DO_CORRECT);
@@ -416,7 +395,6 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     param_changed(&gui, PARAM_DISPLAY);
     outcome = gwy_dialog_run(dialog);
 
-    g_object_unref(gui.data);
     g_object_unref(gui.mlayer);
     g_object_unref(gui.blayer);
     g_object_unref(gui.gmodel);
@@ -429,6 +407,7 @@ param_changed(ModuleGUI *gui, gint id)
 {
     GwyParams *params = gui->args->params;
     GwyParamTable *table = gui->table;
+    ModuleArgs *args = gui->args;
 
     if (id < 0 || id == PARAM_DO_PLOT) {
         gboolean do_plot = gwy_params_get_boolean(params, PARAM_DO_PLOT);
@@ -436,13 +415,14 @@ param_changed(ModuleGUI *gui, gint id)
     }
     if (id == PARAM_DISPLAY) {
         DriftPreviewType display = gwy_params_get_enum(params, PARAM_DISPLAY);
+        GwyDataView *dataview = GWY_DATA_VIEW(gui->dataview);
         if (display == PREVIEW_CORRECTED) {
-            gwy_pixmap_layer_set_data_key(gui->blayer, "/1/data");
-            gwy_data_view_set_alpha_layer(gui->dataview, NULL);
+            gwy_data_view_set_field(dataview, args->result);
+            gwy_data_view_set_mask(dataview, NULL);
         }
         else {
-            gwy_pixmap_layer_set_data_key(gui->blayer, "/0/data");
-            gwy_data_view_set_alpha_layer(gui->dataview, gui->mlayer);
+            gwy_data_view_set_field(dataview, args->field);
+            gwy_data_view_set_mask(dataview, args->mask);
         }
     }
     if (id < 0 || id == PARAM_TARGET_GRAPH || id == PARAM_INTERP || id == PARAM_EXCLUDE_LINEAR)
@@ -457,29 +437,29 @@ preview(gpointer user_data)
 
     execute(args);
     mask_process(args->mask, args->drift);
-    gwy_data_field_data_changed(args->result);
-    gwy_data_field_data_changed(args->mask);
+    gwy_field_data_changed(args->result);
+    gwy_field_data_changed(args->mask);
     gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
 }
 
 static void
-mask_process(GwyDataField *maskfield, GwyDataLine *drift)
+mask_process(GwyField *maskfield, GwyLine *drift)
 {
     gint i, j, k, step, pos, xres, yres, w, from, to;
     gdouble *mdata, *rdata;
 
-    gwy_data_field_clear(maskfield);
-    xres = gwy_data_field_get_xres(maskfield);
-    yres = gwy_data_field_get_yres(maskfield);
+    gwy_field_clear(maskfield);
+    xres = gwy_field_get_xres(maskfield);
+    yres = gwy_field_get_yres(maskfield);
 
     step = xres/10;
     w = (xres + 3*PREVIEW_SIZE/4)/PREVIEW_SIZE;
     w = MAX(w, 1);
-    rdata = gwy_data_line_get_data(drift);
-    mdata = gwy_data_field_get_data(maskfield);
+    rdata = gwy_line_get_data(drift);
+    mdata = gwy_field_get_data(maskfield);
     for (i = 0; i < yres; i++) {
         for (j = -2*step - step/2; j <= xres + 2*step + step/2; j += step) {
-            pos = j + GWY_ROUND(gwy_data_line_rtoi(drift, rdata[i]));
+            pos = j + GWY_ROUND(gwy_line_rtoi(drift, rdata[i]));
             from = MAX(0, pos - w/2);
             to = MIN(pos + (w - w/2) - 1, xres-1);
             for (k = from; k <= to; k++)
@@ -489,22 +469,22 @@ mask_process(GwyDataField *maskfield, GwyDataLine *drift)
 }
 
 /**
- * gwy_data_field_normalize_rows:
+ * gwy_field_normalize_rows:
  * @field: A data field.
  *
  * Normalizes all rows to have mean value 0 and rms 1 (unless they consist of
  * all identical values, then they will have rms 0).
  **/
 static void
-gwy_data_field_normalize_rows(GwyDataField *field)
+gwy_field_normalize_rows(GwyField *field)
 {
     gdouble *data, *row;
     gdouble avg, rms;
     gint xres, yres, i, j;
 
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
-    data = gwy_data_field_get_data(field);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
+    data = gwy_field_get_data(field);
 
     for (i = 0; i < yres; i++) {
         row = data + i*xres;
@@ -579,35 +559,35 @@ match_line(gint res,
 }
 
 static void
-calculate_correlation_scores(GwyDataField *field,
+calculate_correlation_scores(GwyField *field,
                              gint range,
                              gint maxoffset,
                              gdouble supersample,
                              GwyInterpolationType interp,
-                             GwyDataField *scores,
-                             GwyDataField *offsets)
+                             GwyField *scores,
+                             GwyField *offsets)
 {
-    GwyDataField *dsuper;
-    GwySIUnit *siunit;
+    GwyField *dsuper;
+    GwyUnit *siunit;
     gint xres, yres, rangeres;
     gint maxoff, i, ii;
     const gdouble *ds;
     gdouble *sdata, *odata;
     gdouble dx;
 
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
 
     maxoff = (gint)ceil(supersample*maxoffset);
     xres *= supersample;
-    dsuper = gwy_data_field_new_resampled(field, xres, yres, interp);
-    gwy_data_field_normalize_rows(dsuper);
+    dsuper = gwy_field_new_resampled(field, xres, yres, interp);
+    gwy_field_normalize_rows(dsuper);
 
     rangeres = 2*range + 1;
-    ds = gwy_data_field_get_data_const(dsuper);
-    sdata = gwy_data_field_get_data(scores);
-    odata = gwy_data_field_get_data(offsets);
-    dx = gwy_data_field_get_dx(dsuper);
+    ds = gwy_field_get_data_const(dsuper);
+    sdata = gwy_field_get_data(scores);
+    odata = gwy_field_get_data(offsets);
+    dx = gwy_field_get_dx(dsuper);
 #ifdef _OPENMP
 #pragma omp parallel if (gwy_threads_are_enabled()) default(none) \
             private(i,ii) \
@@ -658,23 +638,23 @@ calculate_correlation_scores(GwyDataField *field,
         }
     }
 
-    gwy_data_field_set_yreal(scores, gwy_data_field_get_yreal(field));
-    gwy_data_field_set_xreal(scores, gwy_data_field_itor(field, rangeres));
-    gwy_data_field_set_xoffset(scores, gwy_data_field_itor(field, -range - 0.5));
-    gwy_data_field_set_yreal(offsets, gwy_data_field_get_yreal(field));
-    gwy_data_field_set_xreal(offsets, gwy_data_field_itor(field, rangeres));
-    gwy_data_field_set_xoffset(offsets, gwy_data_field_itor(field, -range - 0.5));
+    gwy_field_set_yreal(scores, gwy_field_get_yreal(field));
+    gwy_field_set_xreal(scores, gwy_field_itor(field, rangeres));
+    gwy_field_set_xoffset(scores, gwy_field_itor(field, -range - 0.5));
+    gwy_field_set_yreal(offsets, gwy_field_get_yreal(field));
+    gwy_field_set_xreal(offsets, gwy_field_itor(field, rangeres));
+    gwy_field_set_xoffset(offsets, gwy_field_itor(field, -range - 0.5));
 
-    siunit = gwy_data_field_get_si_unit_xy(field);
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(scores), siunit);
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(offsets), siunit);
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_z(offsets), siunit);
+    siunit = gwy_field_get_unit_xy(field);
+    gwy_unit_assign(gwy_field_get_unit_xy(scores), siunit);
+    gwy_unit_assign(gwy_field_get_unit_xy(offsets), siunit);
+    gwy_unit_assign(gwy_field_get_unit_z(offsets), siunit);
 }
 
 static void
-calculate_drift_very_naive(GwyDataField *offsets,
-                           GwyDataField *scores,
-                           GwyDataLine *drift)
+calculate_drift_very_naive(GwyField *offsets,
+                           GwyField *scores,
+                           GwyLine *drift)
 {
     const gdouble *doff, *dsco;
     gdouble *dd;
@@ -682,16 +662,16 @@ calculate_drift_very_naive(GwyDataField *offsets,
     gint i, j;
     gdouble dm;
 
-    yres = gwy_data_field_get_yres(offsets);
-    xres = gwy_data_field_get_xres(offsets);
+    yres = gwy_field_get_yres(offsets);
+    xres = gwy_field_get_xres(offsets);
     range = (xres - 1)/2;
 
-    doff = gwy_data_field_get_data_const(offsets);
-    dsco = gwy_data_field_get_data_const(scores);
-    gwy_data_line_resample(drift, yres, GWY_INTERPOLATION_NONE);
-    gwy_data_field_copy_units_to_data_line(offsets, drift);
-    gwy_data_line_set_real(drift, gwy_data_field_get_yreal(offsets));
-    dd = gwy_data_line_get_data(drift);
+    doff = gwy_field_get_data_const(offsets);
+    dsco = gwy_field_get_data_const(scores);
+    gwy_line_resize(drift, yres);
+    gwy_field_copy_units_to_line(offsets, drift);
+    gwy_line_set_real(drift, gwy_field_get_yreal(offsets));
+    dd = gwy_line_get_data(drift);
 
     for (i = 0; i < yres; i++) {
         gdouble w, sxx, sxz, q;
@@ -722,26 +702,26 @@ calculate_drift_very_naive(GwyDataField *offsets,
         dm = d;
     }
 
-    gwy_data_line_cumulate(drift);
+    gwy_line_cumulate(drift, FALSE);
 }
 
 static void
-apply_drift(GwyDataField *field, GwyDataLine *drift, GwyInterpolationType interp)
+apply_drift(GwyField *field, GwyLine *drift, GwyInterpolationType interp)
 {
     gdouble *coeff, *data;
     gint xres, yres, i;
     gdouble corr;
 
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
-    data = gwy_data_field_get_data(field);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
+    data = gwy_field_get_data(field);
     coeff = g_new(gdouble, xres);
 
     for (i = 0; i < yres; i++) {
-        corr = gwy_data_field_rtoj(field, gwy_data_line_get_val(drift, i));
+        corr = gwy_field_rtoj(field, gwy_line_get_val(drift, i));
         gwy_assign(coeff, data + i*xres, xres);
         gwy_interpolation_shift_block_1d(xres, coeff, corr, data + i*xres,
-                                         interp, GWY_EXTERIOR_BORDER_EXTEND, 0.0, FALSE);
+                                         interp, GWY_EXTERIOR_BORDER, 0.0, FALSE);
     }
 
     g_free(coeff);
@@ -753,16 +733,16 @@ execute(ModuleArgs *args)
     gint range = gwy_params_get_int(args->params, PARAM_RANGE);
     GwyInterpolationType interp = gwy_params_get_enum(args->params, PARAM_INTERP);
     gboolean exclude_linear = gwy_params_get_boolean(args->params, PARAM_EXCLUDE_LINEAR);
-    GwyDataField *field = args->field, *result = args->result;
-    GwyDataField *offsets, *scores;
-    GwyDataLine *drift = args->drift;
+    GwyField *field = args->field, *result = args->result;
+    GwyField *offsets, *scores;
+    GwyLine *drift = args->drift;
     gint yres, maxoffset;
 
-    yres = gwy_data_field_get_yres(field);
-    gwy_data_field_copy(field, result, FALSE);
+    yres = gwy_field_get_yres(field);
+    gwy_field_copy_data(field, result);
 
-    offsets = gwy_data_field_new(2*range + 1, yres, 1.0, 1.0, FALSE);
-    scores = gwy_data_field_new(2*range + 1, yres, 1.0, 1.0, FALSE);
+    offsets = gwy_field_new(2*range + 1, yres, 1.0, 1.0, FALSE);
+    scores = gwy_field_new(2*range + 1, yres, 1.0, 1.0, FALSE);
     maxoffset = MAX(1, range/5);
     calculate_correlation_scores(field, range, maxoffset, 4.0, interp, scores, offsets);
     calculate_drift_very_naive(offsets, scores, drift);
@@ -772,10 +752,10 @@ execute(ModuleArgs *args)
     if (exclude_linear) {
         gdouble a, b;
 
-        gwy_data_line_get_line_coeffs(drift, &a, &b);
-        gwy_data_line_line_level(drift, a, b);
+        gwy_line_fit_line(drift, &a, &b);
+        gwy_line_line_level(drift, a, b);
     }
-    gwy_data_line_add(drift, -gwy_data_line_get_median(drift));
+    gwy_line_add(drift, -gwy_line_median(drift));
 
     apply_drift(result, drift, interp);
 }
