@@ -1,5 +1,5 @@
 /*
- *  $Id: dwtanisotropy.c 23666 2021-05-10 21:56:10Z yeti-dn $
+ *  $Id: dwtanisotropy.c 28910 2025-11-24 18:11:09Z yeti-dn $
  *  Copyright (C) 2003-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,17 +16,9 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyenum.h>
-#include <libprocess/stats.h>
-#include <libprocess/inttrans.h>
-#include <libprocess/dwt.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
+#include <gwy.h>
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
 
@@ -39,8 +31,8 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *result;
+    GwyField *field;
+    GwyField *result;
     /* Cached input image properties. */
     gint goodsize;
 } ModuleArgs;
@@ -53,8 +45,8 @@ typedef struct {
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             dwt_anisotropy      (GwyContainer *data,
-                                             GwyRunType runtype);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static void             execute             (ModuleArgs *args);
 static GwyDialogOutcome run_gui             (ModuleArgs *args);
 
@@ -74,11 +66,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("dwtanisotropy",
-                              (GwyProcessFunc)&dwt_anisotropy,
+                              module_main,
                               N_("/_Integral Transforms/DWT _Anisotropy..."),
                               NULL,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("DWT anisotropy detection"));
 
     return TRUE;
@@ -104,30 +96,30 @@ define_module_params(void)
 }
 
 static void
-dwt_anisotropy(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome;
     ModuleArgs args;
     GQuark mquark;
     gint id, i;
 
-    gwy_clear(&args, 1);
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     GWY_APP_MASK_FIELD_KEY, &mquark,
-                                     0);
+    gwy_clear1(args);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_FIELD_ID, &id,
+                                 GWY_APP_MASK_FIELD_KEY, &mquark,
+                                 0);
     g_return_if_fail(args.field);
 
     if (!gwy_require_square_image(args.field, data, id, _("DWT Anisotropy")))
         return;
 
     args.goodsize = 1;
-    for (i = gwy_data_field_get_xres(args.field)-1; i; i >>= 1)
+    for (i = gwy_field_get_xres(args.field)-1; i; i >>= 1)
         args.goodsize <<= 1;
 
     args.params = gwy_params_new_from_settings(define_module_params());
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -135,22 +127,22 @@ dwt_anisotropy(GwyContainer *data, GwyRunType runtype)
     }
     execute(&args);
 
-    gwy_app_undo_qcheckpointv(data, 1, &mquark);
-    if (gwy_data_field_get_max(args.result) > 0.0)
-        gwy_container_set_object(data, mquark, args.result);
+    gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &mquark);
+    if (gwy_field_get_max(args.result) > 0.0)
+        gwy_container_set_object(GWY_CONTAINER(data), mquark, args.result);
     else
-        gwy_container_remove(data, mquark);
-    gwy_app_channel_log_add_proc(data, id, id);
+        gwy_container_remove(GWY_CONTAINER(data), mquark);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, id);
 
 end:
-    GWY_OBJECT_UNREF(args.result);
+    g_clear_object(&args.result);
     g_object_unref(args.params);
 }
 
 static GwyDialogOutcome
 run_gui(ModuleArgs *args)
 {
-    gint xres = gwy_data_field_get_xres(args->field);
+    gint xres = gwy_field_get_xres(args->field);
     GwyDialog *dialog;
     GwyParamTable *table;
     ModuleGUI gui;
@@ -194,23 +186,23 @@ execute(ModuleArgs *args)
     GwyInterpolationType interp = gwy_params_get_enum(args->params, PARAM_INTERP);
     gdouble ratio = gwy_params_get_double(args->params, PARAM_RATIO);
     gint limit = gwy_params_get_int(args->params, PARAM_LOWLIMIT);
-    gint xres = gwy_data_field_get_xres(args->field);
-    GwyDataField *resampled, *mask;
-    GwyDataLine *wtcoefs;
+    gint xres = gwy_field_get_xres(args->field);
+    GwyField *resampled, *mask;
+    GwyLine *wtcoefs;
 
-    resampled = gwy_data_field_new_resampled(args->field, args->goodsize, args->goodsize, interp);
-    gwy_data_field_add(resampled, -gwy_data_field_get_avg(resampled));
+    resampled = gwy_field_new_resampled(args->field, args->goodsize, args->goodsize, interp);
+    gwy_field_add(resampled, -gwy_field_get_avg(resampled));
 
-    mask = args->result = gwy_data_field_new_alike(resampled, TRUE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(mask), NULL);
+    mask = args->result = gwy_field_new_alike(resampled, TRUE);
+    gwy_unit_clear(gwy_field_get_unit_z(mask));
 
-    wtcoefs = gwy_data_line_new(1, 1.0, TRUE);
+    wtcoefs = gwy_line_new(1, 1.0, TRUE);
     wtcoefs = gwy_dwt_set_coefficients(wtcoefs, wavelet);
-    gwy_data_field_dwt_mark_anisotropy(resampled, mask, wtcoefs, ratio, (1u << limit));
+    gwy_field_dwt_mark_anisotropy(resampled, mask, wtcoefs, ratio, (1u << limit));
     g_object_unref(wtcoefs);
     g_object_unref(resampled);
 
-    gwy_data_field_resample(mask, xres, xres, GWY_INTERPOLATION_ROUND);
+    gwy_field_resample(mask, xres, xres, GWY_INTERPOLATION_ROUND);
 }
 
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
