@@ -1,5 +1,5 @@
 /*
- *  $Id: edge.c 25638 2023-09-07 12:23:28Z yeti-dn $
+ *  $Id: edge.c 29061 2026-01-02 15:01:53Z yeti-dn $
  *  Copyright (C) 2003-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,46 +16,40 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/gwyprocess.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 
-#define EDGE_RUN_MODES GWY_RUN_IMMEDIATE
+#define RUN_MODES GWY_RUN_IMMEDIATE
 
 static gboolean module_register(void);
-static void     edge           (GwyContainer *data,
-                                GwyRunType run,
-                                const gchar *name);
-static void     laplacian_do   (GwyDataField *dfield,
-                                GwyDataField *show);
-static void     canny_do       (GwyDataField *dfield,
-                                GwyDataField *show);
-static void     rms_do         (GwyDataField *dfield,
-                                GwyDataField *show);
-static void     rms_edge_do    (GwyDataField *dfield,
-                                GwyDataField *show);
-static void     nonlinearity_do(GwyDataField *dfield,
-                                GwyDataField *show);
-static void     hough_lines_do (GwyDataField *dfield,
-                                GwyDataField *show);
-static void     harris_do      (GwyDataField *dfield,
-                                GwyDataField *show);
-static void     inclination_do (GwyDataField *dfield,
-                                GwyDataField *show);
-static void     step_do        (GwyDataField *dfield,
-                                GwyDataField *show);
-static void     sobel_do       (GwyDataField *dfield,
-                                GwyDataField *show);
-static void     prewitt_do     (GwyDataField *dfield,
-                                GwyDataField *show);
-static void     slope_map      (GwyContainer *data,
-                                GwyRunType run);
-
+static void     module_main    (GwyFile *data,
+                                GwyRunModeFlags mode);
+static void     laplacian_do   (GwyField *dfield,
+                                GwyField *show);
+static void     canny_do       (GwyField *dfield,
+                                GwyField *show);
+static void     rms_do         (GwyField *dfield,
+                                GwyField *show);
+static void     rms_edge_do    (GwyField *dfield,
+                                GwyField *show);
+static void     nonlinearity_do(GwyField *dfield,
+                                GwyField *show);
+static void     hough_lines_do (GwyField *dfield,
+                                GwyField *show);
+static void     harris_do      (GwyField *dfield,
+                                GwyField *show);
+static void     inclination_do (GwyField *dfield,
+                                GwyField *show);
+static void     step_do        (GwyField *dfield,
+                                GwyField *show);
+static void     sobel_do       (GwyField *dfield,
+                                GwyField *show);
+static void     prewitt_do     (GwyField *dfield,
+                                GwyField *show);
+static void     slope_map      (GwyFile *data,
+                                GwyRunModeFlags mode);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
@@ -73,104 +67,104 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("edge_laplacian",
-                              (GwyProcessFunc)&edge,
+                              module_main,
                               N_("/_Presentation/_Edge Detection/_Laplacian of Gaussian"),
                               NULL,
-                              EDGE_RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              RUN_MODES,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Laplacian of Gaussian step detection "
                                  "presentation"));
     gwy_process_func_register("edge_canny",
-                              (GwyProcessFunc)&edge,
+                              module_main,
                               N_("/_Presentation/_Edge Detection/_Canny"),
                               NULL,
-                              EDGE_RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              RUN_MODES,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Canny edge detection presentation"));
     gwy_process_func_register("edge_rms",
-                              (GwyProcessFunc)&edge,
+                              module_main,
                               N_("/_Presentation/_Edge Detection/_RMS"),
                               NULL,
-                              EDGE_RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              RUN_MODES,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Local RMS value based step detection "
                                  "presentation"));
     gwy_process_func_register("edge_rms_edge",
-                              (GwyProcessFunc)&edge,
+                              module_main,
                               N_("/_Presentation/_Edge Detection/RMS _Edge"),
                               NULL,
-                              EDGE_RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              RUN_MODES,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Local RMS value based step detection with postprocessing"));
     gwy_process_func_register("edge_nonlinearity",
-                              (GwyProcessFunc)&edge,
+                              module_main,
                               N_("/_Presentation/_Edge Detection/Local _Nonlinearity"),
                               NULL,
-                              EDGE_RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              RUN_MODES,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Local nonlinearity based edge detection "
                                  "presentation"));
     gwy_process_func_register("edge_hough_lines",
-                              (GwyProcessFunc)&edge,
+                              module_main,
                               N_("/_Presentation/_Edge Detection/_Hough Lines"),
                               NULL,
-                              EDGE_RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              RUN_MODES,
+                              GWY_MENU_FLAG_IMAGE,
                               /* FIXME */
                               N_("Hough lines presentation"));
     gwy_process_func_register("edge_harris",
-                              (GwyProcessFunc)&edge,
+                              module_main,
                               N_("/_Presentation/_Edge Detection/_Harris Corner"),
                               NULL,
-                              EDGE_RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              RUN_MODES,
+                              GWY_MENU_FLAG_IMAGE,
                               /* FIXME */
                               N_("Harris corner presentation"));
     gwy_process_func_register("edge_inclination",
-                              (GwyProcessFunc)&edge,
+                              module_main,
                               N_("/_Presentation/_Edge Detection/_Inclination"),
                               NULL,
-                              EDGE_RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              RUN_MODES,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Local inclination visualization presentation"));
     gwy_process_func_register("edge_step",
-                              (GwyProcessFunc)&edge,
+                              module_main,
                               N_("/_Presentation/_Edge Detection/_Step"),
                               NULL,
-                              EDGE_RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              RUN_MODES,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Fine step detection presentation"));
     gwy_process_func_register("edge_sobel",
-                              (GwyProcessFunc)&edge,
+                              module_main,
                               N_("/_Presentation/_Edge Detection/_Sobel"),
                               NULL,
-                              EDGE_RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              RUN_MODES,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Sobel edge presentation"));
     gwy_process_func_register("edge_prewitt",
-                              (GwyProcessFunc)&edge,
+                              module_main,
                               N_("/_Presentation/_Edge Detection/_Prewitt"),
                               NULL,
-                              EDGE_RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              RUN_MODES,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Prewitt edge presentation"));
     gwy_process_func_register("slope_map",
-                              (GwyProcessFunc)&slope_map,
+                              slope_map,
                               N_("/_Integral Transforms/Local Slope"),
                               NULL,
-                              EDGE_RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              RUN_MODES,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("First derivative slope transformation"));
 
     return TRUE;
 }
 
 static void
-edge(GwyContainer *data, GwyRunType run, const gchar *name)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     static const struct {
         const gchar *name;
-        void (*func)(GwyDataField *dfield, GwyDataField *show);
+        void (*func)(GwyField *dfield, GwyField *show);
     }
     functions[] = {
         { "edge_canny",         canny_do,         },
@@ -185,28 +179,26 @@ edge(GwyContainer *data, GwyRunType run, const gchar *name)
         { "edge_sobel",         sobel_do,         },
         { "edge_prewitt",       prewitt_do,       },
     };
-    GwyDataField *dfield, *showfield;
+    const gchar *name = gwy_process_func_current();
+    GwyField *dfield, *showfield;
     GQuark dquark, squark;
-    GwySIUnit *siunit;
     gint id;
     guint i;
 
-    g_return_if_fail(run & EDGE_RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD_KEY, &dquark,
-                                     GWY_APP_DATA_FIELD, &dfield,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     GWY_APP_SHOW_FIELD_KEY, &squark,
-                                     GWY_APP_SHOW_FIELD, &showfield,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD_KEY, &dquark,
+                                 GWY_APP_FIELD, &dfield,
+                                 GWY_APP_FIELD_ID, &id,
+                                 GWY_APP_SHOW_FIELD_KEY, &squark,
+                                 GWY_APP_SHOW_FIELD, &showfield,
+                                 0);
     g_return_if_fail(dfield && dquark && squark);
 
-    gwy_app_undo_qcheckpointv(data, 1, &squark);
+    gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &squark);
     if (!showfield) {
-        showfield = gwy_data_field_new_alike(dfield, FALSE);
-        siunit = gwy_si_unit_new(NULL);
-        gwy_data_field_set_si_unit_z(showfield, siunit);
-        g_object_unref(siunit);
-        gwy_container_pass_object(data, squark, showfield);
+        showfield = gwy_field_new_alike(dfield, FALSE);
+        gwy_unit_clear(gwy_field_get_unit_z(showfield));
+        gwy_container_pass_object(GWY_CONTAINER(data), squark, showfield);
     }
 
     for (i = 0; i < G_N_ELEMENTS(functions); i++) {
@@ -217,55 +209,55 @@ edge(GwyContainer *data, GwyRunType run, const gchar *name)
     }
     if (i == G_N_ELEMENTS(functions)) {
         g_warning("edge does not provide function `%s'", name);
-        gwy_data_field_copy(dfield, showfield, FALSE);
+        gwy_field_copy_data(dfield, showfield);
     }
 
-    gwy_data_field_normalize(showfield);
-    gwy_data_field_data_changed(showfield);
-    gwy_app_channel_log_add_proc(data, id, id);
+    gwy_field_normalize(showfield);
+    gwy_field_data_changed(showfield);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, id);
 }
 
 /* Note this is the limiting case when LoG reduces for discrete data just to Laplacian */
 static void
-laplacian_do(GwyDataField *dfield, GwyDataField *show)
+laplacian_do(GwyField *dfield, GwyField *show)
 {
-    gwy_data_field_copy(dfield, show, FALSE);
-    gwy_data_field_filter_laplacian(show);
+    gwy_field_copy_data(dfield, show);
+    gwy_field_filter_laplacian(show);
 }
 
 static void
-canny_do(GwyDataField *dfield, GwyDataField *show)
+canny_do(GwyField *dfield, GwyField *show)
 {
     /* Now we use fixed threshold, but in future, there could be API with some setting. We could also do smooting
      * before applying filter.*/
-    gwy_data_field_copy(dfield, show, FALSE);
-    gwy_data_field_filter_canny(show, 0.1);
+    gwy_field_copy_data(dfield, show);
+    gwy_field_filter_canny(show, 0.1);
 }
 
 static void
-rms_do(GwyDataField *dfield, GwyDataField *show)
+rms_do(GwyField *dfield, GwyField *show)
 {
-    gwy_data_field_copy(dfield, show, FALSE);
-    gwy_data_field_filter_rms(show, 5);
+    gwy_field_copy_data(dfield, show);
+    gwy_field_filter_rms(show, 5);
 }
 
 static void
-rms_edge_do(GwyDataField *dfield, GwyDataField *show)
+rms_edge_do(GwyField *dfield, GwyField *show)
 {
-    GwyDataField *tmp;
+    GwyField *tmp;
     gint xres, yres, i, j;
     gdouble *d;
     const gdouble *t;
     gdouble s;
 
-    gwy_data_field_copy(dfield, show, FALSE);
-    xres = gwy_data_field_get_xres(show);
-    yres = gwy_data_field_get_yres(show);
+    gwy_field_copy_data(dfield, show);
+    xres = gwy_field_get_xres(show);
+    yres = gwy_field_get_yres(show);
 
-    tmp = gwy_data_field_duplicate(show);
-    gwy_data_field_filter_rms(tmp, 5);
-    t = gwy_data_field_get_data_const(tmp);
-    d = gwy_data_field_get_data(show);
+    tmp = gwy_field_copy(show);
+    gwy_field_filter_rms(tmp, 5);
+    t = gwy_field_get_data_const(tmp);
+    d = gwy_field_get_data(show);
     for (i = 0; i < yres; i++) {
         gint iim = MAX(i-2, 0)*xres;
         gint iip = MIN(i+2, yres-1)*xres;
@@ -320,7 +312,7 @@ fit_local_plane_by_pos(gint n,
 }
 
 static void
-nonlinearity_do(GwyDataField *dfield, GwyDataField *show)
+nonlinearity_do(GwyField *dfield, GwyField *show)
 {
     static const gdouble r = 2.5;
     gint xres, yres, i, j, size;
@@ -328,13 +320,13 @@ nonlinearity_do(GwyDataField *dfield, GwyDataField *show)
     gint *xp, *yp;
     gdouble *d, *z;
 
-    xres = gwy_data_field_get_xres(dfield);
-    yres = gwy_data_field_get_yres(dfield);
-    d = gwy_data_field_get_data(show);
-    qx = gwy_data_field_get_dx(dfield);
-    qy = gwy_data_field_get_dx(dfield);
+    xres = gwy_field_get_xres(dfield);
+    yres = gwy_field_get_yres(dfield);
+    d = gwy_field_get_data(show);
+    qx = gwy_field_get_dx(dfield);
+    qy = gwy_field_get_dx(dfield);
 
-    size = gwy_data_field_get_circular_area_size(r);
+    size = gwy_field_get_circular_area_size(r);
     z = g_new(gdouble, size);
     xp = g_new(gint, 2*size);
     yp = xp + size;
@@ -343,7 +335,7 @@ nonlinearity_do(GwyDataField *dfield, GwyDataField *show)
             gdouble bx, by, s0r;
             gint n;
 
-            n = gwy_data_field_circular_area_extract_with_pos(dfield, j, i, r, z, xp, yp);
+            n = gwy_field_circular_area_extract_with_pos(dfield, j, i, r, z, xp, yp);
             s0r = fit_local_plane_by_pos(n, xp, yp, z, &bx, &by);
             bx /= qx;
             by /= qy;
@@ -355,7 +347,7 @@ nonlinearity_do(GwyDataField *dfield, GwyDataField *show)
 }
 
 static void
-inclination_do(GwyDataField *dfield, GwyDataField *show)
+inclination_do(GwyField *dfield, GwyField *show)
 {
     static const gdouble r = 2.5;
     gint xres, yres, i, j, size;
@@ -363,13 +355,13 @@ inclination_do(GwyDataField *dfield, GwyDataField *show)
     gint *xp, *yp;
     gdouble *d, *z;
 
-    xres = gwy_data_field_get_xres(dfield);
-    yres = gwy_data_field_get_yres(dfield);
-    d = gwy_data_field_get_data(show);
-    qx = gwy_data_field_get_dx(dfield);
-    qy = gwy_data_field_get_dx(dfield);
+    xres = gwy_field_get_xres(dfield);
+    yres = gwy_field_get_yres(dfield);
+    d = gwy_field_get_data(show);
+    qx = gwy_field_get_dx(dfield);
+    qy = gwy_field_get_dx(dfield);
 
-    size = gwy_data_field_get_circular_area_size(r);
+    size = gwy_field_get_circular_area_size(r);
     z = g_new(gdouble, size);
     xp = g_new(gint, 2*size);
     yp = xp + size;
@@ -378,7 +370,7 @@ inclination_do(GwyDataField *dfield, GwyDataField *show)
             gdouble bx, by;
             gint n;
 
-            n = gwy_data_field_circular_area_extract_with_pos(dfield, j, i, r, z, xp, yp);
+            n = gwy_field_circular_area_extract_with_pos(dfield, j, i, r, z, xp, yp);
             fit_local_plane_by_pos(n, xp, yp, z, &bx, &by);
             bx /= qx;
             by /= qy;
@@ -390,24 +382,24 @@ inclination_do(GwyDataField *dfield, GwyDataField *show)
 }
 
 static void
-step_do(GwyDataField *dfield, GwyDataField *show)
+step_do(GwyField *dfield, GwyField *show)
 {
     static const gdouble r = 2.5;
     gint xres, yres, i, j, size;
     gdouble *d, *z;
 
-    xres = gwy_data_field_get_xres(dfield);
-    yres = gwy_data_field_get_yres(dfield);
-    d = gwy_data_field_get_data(show);
+    xres = gwy_field_get_xres(dfield);
+    yres = gwy_field_get_yres(dfield);
+    d = gwy_field_get_data(show);
 
-    size = gwy_data_field_get_circular_area_size(r);
+    size = gwy_field_get_circular_area_size(r);
     z = g_new(gdouble, size);
     for (i = 0; i < yres; i++) {
         for (j = 0; j < xres; j++) {
             gint n;
 
-            n = gwy_data_field_circular_area_extract(dfield, j, i, r, z);
-            gwy_math_sort(n, z);
+            n = gwy_field_circular_area_extract(dfield, j, i, r, z);
+            gwy_math_sort(z, n);
             d[i*xres + j] = sqrt(z[n-1 - n/3] - z[n/3]);
         }
     }
@@ -415,75 +407,77 @@ step_do(GwyDataField *dfield, GwyDataField *show)
 }
 
 static void
-hough_lines_do(GwyDataField *dfield, GwyDataField *show)
+hough_lines_do(GwyField *dfield, GwyField *show)
 {
-    GwyDataField *x_gradient, *y_gradient;
+    GwyField *x_gradient, *y_gradient;
 
-    gwy_data_field_copy(dfield, show, FALSE);
-    gwy_data_field_filter_canny(show, 0.1);
+    gwy_field_copy_data(dfield, show);
+    gwy_field_filter_canny(show, 0.1);
 
-    x_gradient = gwy_data_field_duplicate(dfield);
-    gwy_data_field_filter_sobel(x_gradient, GWY_ORIENTATION_HORIZONTAL);
-    y_gradient = gwy_data_field_duplicate(dfield);
-    gwy_data_field_filter_sobel(y_gradient, GWY_ORIENTATION_VERTICAL);
+    x_gradient = gwy_field_copy(dfield);
+    gwy_field_filter_sobel(x_gradient, GWY_ORIENTATION_HORIZONTAL);
+    y_gradient = gwy_field_copy(dfield);
+    gwy_field_filter_sobel(y_gradient, GWY_ORIENTATION_VERTICAL);
 
-    gwy_data_field_hough_line_strenghten(show, x_gradient, y_gradient, 1, 0.2);
+    gwy_field_hough_line_strenghten(show, x_gradient, y_gradient, 1, 0.2);
 }
 static void
-harris_do(GwyDataField *dfield, GwyDataField *show)
+harris_do(GwyField *dfield, GwyField *show)
 {
-    GwyDataField *x_gradient, *y_gradient;
+    GwyField *x_gradient, *y_gradient;
 
-    gwy_data_field_copy(dfield, show, FALSE);
-    x_gradient = gwy_data_field_duplicate(dfield);
-    gwy_data_field_filter_sobel(x_gradient, GWY_ORIENTATION_HORIZONTAL);
-    y_gradient = gwy_data_field_duplicate(dfield);
-    gwy_data_field_filter_sobel(y_gradient, GWY_ORIENTATION_VERTICAL);
+    gwy_field_copy_data(dfield, show);
+    x_gradient = gwy_field_copy(dfield);
+    gwy_field_filter_sobel(x_gradient, GWY_ORIENTATION_HORIZONTAL);
+    y_gradient = gwy_field_copy(dfield);
+    gwy_field_filter_sobel(y_gradient, GWY_ORIENTATION_VERTICAL);
 
-    gwy_data_field_filter_harris(x_gradient, y_gradient, show, 20, 0.1);
+    gwy_field_filter_harris(x_gradient, y_gradient, show, 20, 0.1);
 }
 
 static void
-sobel_do(GwyDataField *dfield, GwyDataField *show)
+sobel_do(GwyField *dfield, GwyField *show)
 {
-    gwy_data_field_copy(dfield, show, FALSE);
-    gwy_data_field_filter_sobel_total(show);
+    gwy_field_copy_data(dfield, show);
+    gwy_field_filter_sobel_total(show);
 }
 
 static void
-prewitt_do(GwyDataField *dfield, GwyDataField *show)
+prewitt_do(GwyField *dfield, GwyField *show)
 {
-    gwy_data_field_copy(dfield, show, FALSE);
-    gwy_data_field_filter_prewitt_total(show);
+    gwy_field_copy_data(dfield, show);
+    gwy_field_filter_prewitt_total(show);
 }
 
 static void
-slope_map(GwyContainer *data, GwyRunType run)
+slope_map(GwyFile *data, GwyRunModeFlags mode)
 {
-    GwyDataField *dfield, *sfield, *buf;
-    GwySIUnit *xyunit, *zunit;
+    GwyField *dfield, *sfield, *buf;
+    GwyUnit *xyunit, *zunit;
     gint oldid, newid;
 
-    g_return_if_fail(run & EDGE_RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &dfield,
-                                     GWY_APP_DATA_FIELD_ID, &oldid,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &dfield,
+                                 GWY_APP_FIELD_ID, &oldid,
+                                 0);
     g_return_if_fail(dfield);
 
-    sfield = gwy_data_field_new_alike(dfield, FALSE);
-    buf = gwy_data_field_new_alike(dfield, FALSE);
+    sfield = gwy_field_new_alike(dfield, FALSE);
+    buf = gwy_field_new_alike(dfield, FALSE);
 
-    gwy_data_field_filter_slope(dfield, sfield, buf);
-    gwy_data_field_hypot_of_fields(sfield, sfield, buf);
+    gwy_field_filter_slope(dfield, sfield, buf);
+    gwy_field_hypot_of_fields(sfield, sfield, buf);
     g_object_unref(buf);
 
-    xyunit = gwy_data_field_get_si_unit_xy(sfield);
-    zunit = gwy_data_field_get_si_unit_z(sfield);
-    gwy_si_unit_divide(zunit, xyunit, zunit);
+    xyunit = gwy_field_get_unit_xy(sfield);
+    zunit = gwy_field_get_unit_z(sfield);
+    gwy_unit_divide(zunit, xyunit, zunit);
 
-    newid = gwy_app_data_browser_add_data_field(sfield, data, TRUE);
-    gwy_app_set_data_field_title(data, newid, _("Slope map"));
-    gwy_app_channel_log_add_proc(data, oldid, newid);
+    newid = gwy_file_add_image(data, sfield);
+
+    gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+    gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _("Slope map"), TRUE);
+    gwy_log_add(data, GWY_FILE_IMAGE, oldid, newid);
     g_object_unref(sfield);
 }
 
