@@ -1,5 +1,5 @@
 /*
- *  $Id: entropy.c 24441 2021-10-29 15:30:17Z yeti-dn $
+ *  $Id: entropy.c 29023 2025-12-19 13:06:16Z yeti-dn $
  *  Copyright (C) 2015-2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,19 +16,9 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/level.h>
-#include <libprocess/filters.h>
-#include <libprocess/stats.h>
-#include <libprocess/linestats.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwydgets/gwydgetutils.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_INTERACTIVE)
@@ -53,8 +43,8 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *mask;
+    GwyField *field;
+    GwyField *mask;
     /* Cached input data properties. */
     gboolean same_units;
 } ModuleArgs;
@@ -69,18 +59,18 @@ typedef struct {
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             entropy             (GwyContainer *data,
-                                             GwyRunType runtype);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static GwyDialogOutcome run_gui             (ModuleArgs *args);
 static GwyResults*      create_results      (ModuleArgs *args);
 static void             param_changed       (ModuleGUI *gui,
                                              gint id);
 static void             set_graph_zoom      (ModuleGUI *gui);
 static void             preview             (gpointer user_data);
-static void             compute_slopes      (GwyDataField *field,
+static void             compute_slopes      (GwyField *field,
                                              gint kernel_size,
-                                             GwyDataField *xder,
-                                             GwyDataField *yder);
+                                             GwyField *xder,
+                                             GwyField *yder);
 static void             sanitise_params     (ModuleArgs *args);
 
 static GwyModuleInfo module_info = {
@@ -99,11 +89,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("entropy",
-                              (GwyProcessFunc)&entropy,
+                              module_main,
                               N_("/_Statistics/_Entropy..."),
-                              GWY_STOCK_ENTROPY,
+                              GWY_ICON_ENTROPY,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Calculate entropy of value and slope distributions"));
 
     return TRUE;
@@ -133,18 +123,18 @@ define_module_params(void)
 }
 
 static void
-entropy(G_GNUC_UNUSED GwyContainer *data, GwyRunType runtype)
+module_main(G_GNUC_UNUSED GwyFile *data, GwyRunModeFlags mode)
 {
-    GwyDataField *field;
+    GwyField *field;
     ModuleArgs args;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &field,
-                                     GWY_APP_MASK_FIELD, &args.mask,
-                                     0);
-    g_return_if_fail(GWY_IS_DATA_FIELD(field));
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &field,
+                                 GWY_APP_MASK_FIELD, &args.mask,
+                                 0);
+    g_return_if_fail(GWY_IS_FIELD(field));
     args.field = field;
-    args.same_units = gwy_si_unit_equal(gwy_data_field_get_si_unit_xy(field), gwy_data_field_get_si_unit_z(field));
+    args.same_units = gwy_unit_equal(gwy_field_get_unit_xy(field), gwy_field_get_unit_z(field));
     args.params = gwy_params_new_from_settings(define_module_params());
     sanitise_params(&args);
     run_gui(&args);
@@ -161,7 +151,7 @@ run_gui(ModuleArgs *args)
     GwyDialog *dialog;
     ModuleGUI gui;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
     gui.results = create_results(args);
     gui.gmodel = gwy_graph_model_new();
@@ -170,7 +160,7 @@ run_gui(ModuleArgs *args)
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_UPDATE, GTK_RESPONSE_OK, 0);
 
-    hbox = gwy_hbox_new(8);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 8);
     gwy_dialog_add_content(dialog, hbox, FALSE, FALSE, 0);
 
     table = gui.table = gwy_param_table_new(args->params);
@@ -289,11 +279,11 @@ set_graph_zoom(ModuleGUI *gui)
 /* This does not transform to spherical (theta,phi) but to a planar coordinate system with unit |J| so the entropy
  * should be preserved.  It is the same transformation as in facet analysis. */
 static void
-transform_to_sphere(GwyDataField *xder, GwyDataField *yder)
+transform_to_sphere(GwyField *xder, GwyField *yder)
 {
-    gdouble *xdata = gwy_data_field_get_data(xder);
-    gdouble *ydata = gwy_data_field_get_data(yder);
-    guint i, n = gwy_data_field_get_xres(xder)*gwy_data_field_get_yres(xder);
+    gdouble *xdata = gwy_field_get_data(xder);
+    gdouble *ydata = gwy_field_get_data(yder);
+    guint i, n = gwy_field_get_xres(xder)*gwy_field_get_yres(xder);
 
     for (i = 0; i < n; i++) {
         gdouble x = xdata[i], y = ydata[i];
@@ -308,28 +298,28 @@ transform_to_sphere(GwyDataField *xder, GwyDataField *yder)
 }
 
 static gdouble
-calculate_sigma2_2d(GwyDataField *xfield, GwyDataField *yfield)
+calculate_sigma2_2d(GwyField *xfield, GwyField *yfield)
 {
-    gdouble xc = gwy_data_field_get_avg(xfield);
-    gdouble yc = gwy_data_field_get_avg(yfield);
-    const gdouble *xdata = gwy_data_field_get_data(xfield);
-    const gdouble *ydata = gwy_data_field_get_data(yfield);
+    gdouble xc = gwy_field_get_avg(xfield);
+    gdouble yc = gwy_field_get_avg(yfield);
+    const gdouble *xdata = gwy_field_get_data(xfield);
+    const gdouble *ydata = gwy_field_get_data(yfield);
     gdouble s2 = 0.0;
     guint n, i;
 
-    n = gwy_data_field_get_xres(xfield)*gwy_data_field_get_yres(xfield);
+    n = gwy_field_get_xres(xfield)*gwy_field_get_yres(xfield);
     for (i = 0; i < n; i++)
         s2 += (xdata[i] - xc)*(xdata[i] - xc) + (ydata[i] - yc)*(ydata[i] - yc);
 
     return s2/n;
 }
 
-static GwyDataField*
-fake_mask(GwyDataField *field, GwyDataField *mask, GwyMaskingType masking)
+static GwyField*
+fake_mask(GwyField *field, GwyField *mask, GwyMaskingType masking)
 {
-    GwyDataField *masked;
-    gint xres = gwy_data_field_get_xres(field);
-    gint yres = gwy_data_field_get_yres(field);
+    GwyField *masked;
+    gint xres = gwy_field_get_xres(field);
+    gint yres = gwy_field_get_yres(field);
     const gdouble *d, *m;
     gdouble *md;
     gint i, n;
@@ -337,17 +327,17 @@ fake_mask(GwyDataField *field, GwyDataField *mask, GwyMaskingType masking)
     if (!mask || masking == GWY_MASK_IGNORE)
         return field;
 
-    gwy_data_field_area_count_in_range(mask, NULL, 0, 0, xres, yres, G_MAXDOUBLE, 1.0, NULL, &n);
+    gwy_field_area_count_in_range(mask, NULL, GWY_MASK_IGNORE, 0, 0, xres, yres, G_MAXDOUBLE, 1.0, NULL, &n);
     if (masking == GWY_MASK_EXCLUDE)
         n = xres*yres - n;
 
     if (n == xres*yres)
         return field;
 
-    masked = gwy_data_field_new(n, 1, n, 1.0, FALSE);
-    md = gwy_data_field_get_data(masked);
-    d = gwy_data_field_get_data_const(field);
-    m = gwy_data_field_get_data_const(mask);
+    masked = gwy_field_new(n, 1, n, 1.0, FALSE);
+    md = gwy_field_get_data(masked);
+    d = gwy_field_get_data_const(field);
+    m = gwy_field_get_data_const(mask);
     n = 0;
     for (i = 0; i < xres*yres; i++) {
         gboolean mi = (m[i] >= 1.0);
@@ -364,26 +354,26 @@ preview(gpointer user_data)
 {
     ModuleGUI *gui = (ModuleGUI*)user_data;
     ModuleArgs *args = gui->args;
-    GwyDataField *field = args->field, *mask = args->mask;
+    GwyField *field = args->field, *mask = args->mask;
     EntropyMode mode = gwy_params_get_enum(args->params, PARAM_MODE);
     GwyMaskingType masking = gwy_params_get_masking(args->params, PARAM_MASKING, &mask);
     gboolean fit_plane = gwy_params_get_boolean(args->params, PARAM_FIT_PLANE);
     gint kernel_size = gwy_params_get_int(args->params, PARAM_KERNEL_SIZE);
-    gint xres = gwy_data_field_get_xres(field), yres = gwy_data_field_get_yres(field);
+    gint xres = gwy_field_get_xres(field), yres = gwy_field_get_yres(field);
     GwyGraphModel *gmodel = gui->gmodel;
     GwyGraphCurveModel *gcmodel;
-    GwyDataLine *ecurve;
+    GwyLine *ecurve;
     gdouble S, s, Smax = 0.0;
 
-    ecurve = gwy_data_line_new(1, 1.0, FALSE);
+    ecurve = gwy_line_new(1, 1.0, FALSE);
     if (mode == ENTROPY_VALUES) {
-        S = gwy_data_field_area_get_entropy_at_scales(field, ecurve, mask, masking, 0, 0, xres, yres, 0);
-        s = gwy_data_field_area_get_rms_mask(field, mask, masking, 0, 0, xres, yres);
+        S = gwy_field_area_get_entropy_at_scales(field, mask, masking, ecurve, 0, 0, xres, yres, 0);
+        s = gwy_field_area_get_rms(field, mask, masking, 0, 0, xres, yres);
         Smax = ENTROPY_NORMAL + log(s);
     }
     else {
-        GwyDataField *xder = gwy_data_field_new_alike(field, FALSE);
-        GwyDataField *yder = gwy_data_field_new_alike(field, FALSE);
+        GwyField *xder = gwy_field_new_alike(field, FALSE);
+        GwyField *yder = gwy_field_new_alike(field, FALSE);
 
         compute_slopes(args->field, fit_plane ? kernel_size : 0, xder, yder);
         xder = fake_mask(xder, mask, masking);
@@ -391,7 +381,7 @@ preview(gpointer user_data)
         if (mode == ENTROPY_ANGLES)
             transform_to_sphere(xder, yder);
 
-        S = gwy_data_field_get_entropy_2d_at_scales(xder, yder, ecurve, 0);
+        S = gwy_field_get_entropy_2d_at_scales(xder, yder, ecurve, 0);
         if (mode == ENTROPY_SLOPES) {
             s = calculate_sigma2_2d(xder, yder);
             Smax = ENTROPY_NORMAL_2D + log(s);
@@ -412,25 +402,25 @@ preview(gpointer user_data)
     g_object_set(gmodel,
                  "axis-label-bottom", "log h",
                  "axis-label-left", "S",
-                 "label-position", GWY_GRAPH_LABEL_NORTHWEST,
+                 "label-position", GWY_GRAPH_KEY_NORTHWEST,
                  NULL);
 
-    if (gwy_data_line_get_min(ecurve) > -0.5*G_MAXDOUBLE) {
+    if (gwy_line_min(ecurve) > -0.5*G_MAXDOUBLE) {
         gcmodel = gwy_graph_curve_model_new();
         g_object_set(gcmodel,
                      "description", _("Entropy at scales"),
                      "mode", GWY_GRAPH_CURVE_LINE_POINTS,
                      "color", gwy_graph_get_preset_color(0),
                      NULL);
-        gwy_graph_curve_model_set_data_from_dataline(gcmodel, ecurve, 0, 0);
+        gwy_graph_curve_model_set_data_from_line(gcmodel, ecurve, 0, 0);
         gwy_graph_model_add_curve(gmodel, gcmodel);
         g_object_unref(gcmodel);
     }
 
     if (S > -0.5*G_MAXDOUBLE) {
-        GwyDataLine *best = gwy_data_line_duplicate(ecurve);
-        gdouble *ydata = gwy_data_line_get_data(best);
-        guint i, res = gwy_data_line_get_res(best);
+        GwyLine *best = gwy_line_copy(ecurve);
+        gdouble *ydata = gwy_line_get_data(best);
+        guint i, res = gwy_line_get_res(best);
 
         for (i = 0; i < res; i++)
             ydata[i] = S;
@@ -441,7 +431,7 @@ preview(gpointer user_data)
                      "mode", GWY_GRAPH_CURVE_LINE,
                      "color", gwy_graph_get_preset_color(1),
                      NULL);
-        gwy_graph_curve_model_set_data_from_dataline(gcmodel, best, 0, 0);
+        gwy_graph_curve_model_set_data_from_line(gcmodel, best, 0, 0);
         gwy_graph_model_add_curve(gmodel, gcmodel);
         g_object_unref(gcmodel);
         g_object_unref(best);
@@ -453,23 +443,23 @@ preview(gpointer user_data)
 }
 
 static void
-compute_slopes(GwyDataField *field, gint kernel_size,
-               GwyDataField *xder, GwyDataField *yder)
+compute_slopes(GwyField *field, gint kernel_size,
+               GwyField *xder, GwyField *yder)
 {
     gint xres, yres;
 
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
     if (kernel_size) {
         GwyPlaneFitQuantity quantites[] = { GWY_PLANE_FIT_BX, GWY_PLANE_FIT_BY };
-        GwyDataField *fields[2] = { xder, yder };
+        GwyField *fields[2] = { xder, yder };
 
-        gwy_data_field_fit_local_planes(field, kernel_size, 2, quantites, fields);
-        gwy_data_field_multiply(xder, xres/gwy_data_field_get_xreal(field));
-        gwy_data_field_multiply(yder, yres/gwy_data_field_get_yreal(field));
+        gwy_field_fit_local_planes(field, kernel_size, 2, quantites, fields);
+        gwy_field_multiply(xder, xres/gwy_field_get_xreal(field));
+        gwy_field_multiply(yder, yres/gwy_field_get_yreal(field));
     }
     else
-        gwy_data_field_filter_slope(field, xder, yder);
+        gwy_field_filter_slope(field, xder, yder);
 }
 
 static void
