@@ -1,5 +1,5 @@
 /*
- *  $Id: extend.c 23985 2021-08-16 12:27:37Z yeti-dn $
+ *  $Id: extend.c 28844 2025-11-10 16:07:51Z yeti-dn $
  *  Copyright (C) 2010-2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,17 +16,11 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/stats.h>
-#include <libprocess/arithmetic.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
@@ -50,8 +44,8 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *result;
+    GwyField *field;
+    GwyField *result;
     /* Cached values for dimensions display. */
     gint xres;
     gint yres;
@@ -61,17 +55,16 @@ typedef struct {
     ModuleArgs *args;
     GtkWidget *dialog;
     GwyParamTable *table;
-    GwyContainer *data;
     GtkWidget *view;
     gint last_active;
 } ModuleGUI;
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             extend              (GwyContainer *data,
-                                             GwyRunType runtype);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static GwyDialogOutcome run_gui             (ModuleArgs *args,
-                                             GwyContainer *data,
+                                             GwyFile *data,
                                              gint id);
 static void             param_changed       (ModuleGUI *gui,
                                              gint id);
@@ -94,11 +87,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("extend",
-                              (GwyProcessFunc)&extend,
+                              module_main,
                               N_("/_Basic Operations/E_xtend..."),
-                              GWY_STOCK_EXTEND,
+                              GWY_ICON_EXTEND,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Extend by adding borders"));
 
     return TRUE;
@@ -109,10 +102,10 @@ define_module_params(void)
 {
     static const GwyEnum exteriors[] = {
         { N_("Mean"),              GWY_EXTERIOR_FIXED_VALUE,   },
-        { N_("exterior|Border"),   GWY_EXTERIOR_BORDER_EXTEND, },
-        { N_("exterior|Mirror"),   GWY_EXTERIOR_MIRROR_EXTEND, },
-        { N_("exterior|Periodic"), GWY_EXTERIOR_PERIODIC,      },
-        { N_("exterior|Laplace"),  GWY_EXTERIOR_LAPLACE,       },
+        { gwy_NC("exterior", "Border"),   GWY_EXTERIOR_BORDER, },
+        { gwy_NC("exterior", "Mirror"),   GWY_EXTERIOR_MIRROR, },
+        { gwy_NC("exterior", "Periodic"), GWY_EXTERIOR_PERIODIC,      },
+        { gwy_NC("exterior", "Laplace"),  GWY_EXTERIOR_LAPLACE,       },
     };
 
     static GwyParamDef *paramdef = NULL;
@@ -128,7 +121,7 @@ define_module_params(void)
     gwy_param_def_add_int(paramdef, PARAM_RIGHT, "right", _("_Right"), 0, EXTEND_MAX, 0);
     gwy_param_def_add_boolean(paramdef, PARAM_SYMMETRIC, "symmetric", _("Extend _symmetrically"), TRUE);
     gwy_param_def_add_gwyenum(paramdef, PARAM_EXTERIOR, "exterior", _("_Exterior type"),
-                              exteriors, G_N_ELEMENTS(exteriors), GWY_EXTERIOR_MIRROR_EXTEND);
+                              exteriors, G_N_ELEMENTS(exteriors), GWY_EXTERIOR_MIRROR);
     gwy_param_def_add_boolean(paramdef, PARAM_KEEP_OFFSETS, "keep_offsets", _("Keep lateral offsets"), FALSE);
     gwy_param_def_add_boolean(paramdef, PARAM_NEW_CHANNEL, "new_channel", _("Create new image"), FALSE);
     gwy_param_def_add_instant_updates(paramdef, PARAM_UPDATE, "update", NULL, FALSE);
@@ -136,36 +129,36 @@ define_module_params(void)
 }
 
 static void
-extend(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
-    GwyDataField *fields[3], *result;
-    GQuark quarks[3], quark;
+    GwyField *fields[3], *result;
+    GQuark quarks[3];
     GwyParams *params;
     ModuleArgs args;
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     gint n, id, newid, up, down, left, right, exterior;
     gboolean new_channel, keep_offsets;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, fields + 0,
-                                     GWY_APP_MASK_FIELD, fields + 1,
-                                     GWY_APP_SHOW_FIELD, fields + 2,
-                                     GWY_APP_DATA_FIELD_KEY, quarks + 0,
-                                     GWY_APP_MASK_FIELD_KEY, quarks + 1,
-                                     GWY_APP_SHOW_FIELD_KEY, quarks + 2,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_FIELD, fields + 0,
+                                 GWY_APP_MASK_FIELD, fields + 1,
+                                 GWY_APP_SHOW_FIELD, fields + 2,
+                                 GWY_APP_FIELD_KEY, quarks + 0,
+                                 GWY_APP_MASK_FIELD_KEY, quarks + 1,
+                                 GWY_APP_SHOW_FIELD_KEY, quarks + 2,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(fields[0]);
 
     args.field = fields[0];
-    args.xres = gwy_data_field_get_xres(args.field);
-    args.yres = gwy_data_field_get_yres(args.field);
+    args.xres = gwy_field_get_xres(args.field);
+    args.yres = gwy_field_get_yres(args.field);
     args.params = params = gwy_params_new_from_settings(define_module_params());
     sanitise_params(&args);
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
-        args.result = gwy_data_field_duplicate(args.field);
+    if (mode ==GWY_RUN_INTERACTIVE) {
+        args.result = gwy_field_copy(args.field);
         outcome = run_gui(&args, data, id);
         gwy_params_save_to_settings(params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -192,7 +185,7 @@ extend(GwyContainer *data, GwyRunType runtype)
             quarks[2] = 0;
             n++;
         }
-        gwy_app_undo_qcheckpointv(data, n, quarks);
+        gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), n, quarks);
     }
 
     if (outcome == GWY_DIALOG_HAVE_RESULT) {
@@ -200,26 +193,25 @@ extend(GwyContainer *data, GwyRunType runtype)
         args.result = NULL;
     }
     else {
-        result = gwy_data_field_extend(fields[0], left, right, up, down, exterior, gwy_data_field_get_avg(fields[0]),
-                                       keep_offsets);
+        result = gwy_field_extend(fields[0], left, right, up, down, exterior, gwy_field_get_avg(fields[0]),
+                                  keep_offsets);
     }
 
     if (!new_channel) {
-        gwy_data_field_assign(fields[0], result);
-        gwy_data_field_data_changed(fields[0]);
-        gwy_app_channel_log_add_proc(data, id, id);
+        gwy_field_assign(fields[0], result);
+        gwy_field_data_changed(fields[0]);
+        gwy_log_add(data, GWY_FILE_IMAGE, id, id);
     }
     else {
-        newid = gwy_app_data_browser_add_data_field(result, data, TRUE);
-        gwy_app_sync_data_items(data, data, id, newid, FALSE,
-                                GWY_DATA_ITEM_GRADIENT,
-                                GWY_DATA_ITEM_MASK_COLOR,
-                                GWY_DATA_ITEM_RANGE,
-                                GWY_DATA_ITEM_REAL_SQUARE,
-                                GWY_DATA_ITEM_SELECTIONS,
-                                0);
-        gwy_app_set_data_field_title(data, newid, _("Extended"));
-        gwy_app_channel_log_add_proc(data, id, newid);
+        newid = gwy_file_add_image(data, result);
+        gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+        gwy_file_sync_items(data, GWY_FILE_IMAGE, id,
+                            data, GWY_FILE_IMAGE, newid,
+                            GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_MASK_COLOR | GWY_FILE_ITEM_RANGE
+                            | GWY_FILE_ITEM_REAL_SQUARE | GWY_FILE_ITEM_SELECTIONS,
+                            FALSE);
+        gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _("Extended"), TRUE);
+        gwy_log_add(data, GWY_FILE_IMAGE, id, newid);
     }
     g_object_unref(result);
 
@@ -232,67 +224,60 @@ extend(GwyContainer *data, GwyRunType runtype)
         if (mask_exterior == GWY_EXTERIOR_FIXED_VALUE) {
             guint n_unmasked, xres, yres;
 
-            xres = gwy_data_field_get_xres(fields[1]);
-            yres = gwy_data_field_get_yres(fields[1]);
-            gwy_data_field_area_count_in_range(fields[1], NULL, 0, 0, xres, yres, 0.0, 0.0, &n_unmasked, NULL);
+            xres = gwy_field_get_xres(fields[1]);
+            yres = gwy_field_get_yres(fields[1]);
+            gwy_field_area_count_in_range(fields[1], NULL, GWY_MASK_IGNORE,
+                                          0, 0, xres, yres, 0.0, 0.0, &n_unmasked, NULL);
             fill_value = (n_unmasked > 0.5*xres*yres);
         }
-        result = gwy_data_field_extend(fields[1], left, right, up, down, mask_exterior, fill_value, keep_offsets);
+        result = gwy_field_extend(fields[1], left, right, up, down, mask_exterior, fill_value, keep_offsets);
         if (!new_channel) {
-            gwy_data_field_assign(fields[1], result);
-            gwy_data_field_data_changed(fields[1]);
+            gwy_field_assign(fields[1], result);
+            gwy_field_data_changed(fields[1]);
         }
         else {
-            quark = gwy_app_get_mask_key_for_id(newid);
-            gwy_container_set_object(data, quark, result);
+            gwy_file_set_image_mask(data, newid, result);
         }
         g_object_unref(result);
     }
 
     if (fields[2]) {
-        result = gwy_data_field_extend(fields[2], left, right, up, down, exterior, gwy_data_field_get_avg(fields[2]),
-                                       keep_offsets);
+        result = gwy_field_extend(fields[2], left, right, up, down, exterior, gwy_field_get_avg(fields[2]),
+                                  keep_offsets);
         if (!new_channel) {
-            gwy_data_field_assign(fields[2], result);
-            gwy_data_field_data_changed(fields[2]);
+            gwy_field_assign(fields[2], result);
+            gwy_field_data_changed(fields[2]);
         }
         else {
-            quark = gwy_app_get_show_key_for_id(newid);
-            gwy_container_set_object(data, quark, result);
+            gwy_container_set_object(GWY_CONTAINER(data), gwy_file_key_image_picture(newid), result);
         }
         g_object_unref(result);
     }
 
 end:
     g_object_unref(params);
-    GWY_OBJECT_UNREF(args.result);
+    g_clear_object(&args.result);
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
-    GwyDialogOutcome outcome;
     GtkWidget *hbox;
     GwyDialog *dialog;
     GwyParamTable *table;
     ModuleGUI gui;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
     gui.last_active = PARAM_UP;
-    gui.data = gwy_container_new();
-
-    gwy_container_set_object_by_name(gui.data, "/0/data", args->result);
-    gwy_app_sync_data_items(data, gui.data, id, 0, FALSE,
-                            GWY_DATA_ITEM_PALETTE,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
 
     gui.dialog = gwy_dialog_new(_("Extend"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_UPDATE, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    gui.view = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
+    gui.view = gwy_create_preview(args->result, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(gui.view), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_REAL_SQUARE);
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(gui.view), FALSE);
 
     table = gui.table = gwy_param_table_new(args->params);
@@ -328,11 +313,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     g_signal_connect_swapped(table, "param-changed", G_CALLBACK(param_changed), &gui);
     gwy_dialog_set_preview_func(dialog, GWY_PREVIEW_IMMEDIATE, preview, &gui, NULL);
 
-    outcome = gwy_dialog_run(dialog);
-
-    g_object_unref(gui.data);
-
-    return outcome;
+    return gwy_dialog_run(dialog);
 }
 
 static void
@@ -381,7 +362,7 @@ preview(gpointer user_data)
     ModuleGUI *gui = (ModuleGUI*)user_data;
     ModuleArgs *args = gui->args;
     GwyParams *params = args->params;
-    GwyDataField *field = args->field, *result;
+    GwyField *field = args->field, *result;
     gint up = gwy_params_get_int(params, PARAM_UP);
     gint down = gwy_params_get_int(params, PARAM_DOWN);
     gint left = gwy_params_get_int(params, PARAM_LEFT);
@@ -389,10 +370,10 @@ preview(gpointer user_data)
     GwyExteriorType exterior = gwy_params_get_int(params, PARAM_EXTERIOR);
     gboolean keep_offsets = gwy_params_get_boolean(params, PARAM_KEEP_OFFSETS);
 
-    result = gwy_data_field_extend(field, left, right, up, down, exterior, gwy_data_field_get_avg(field), keep_offsets);
-    gwy_data_field_assign(args->result, result);
+    result = gwy_field_extend(field, left, right, up, down, exterior, gwy_field_get_avg(field), keep_offsets);
+    gwy_field_assign(args->result, result);
     g_object_unref(result);
-    gwy_data_field_data_changed(args->result);
+    gwy_field_data_changed(args->result);
     gwy_set_data_preview_size(GWY_DATA_VIEW(gui->view), PREVIEW_SIZE);
     gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
 }
