@@ -1,5 +1,5 @@
 /*
- *  $Id: extract_path.c 26510 2024-08-15 17:12:23Z yeti-dn $
+ *  $Id: extract_path.c 28821 2025-11-06 16:00:41Z yeti-dn $
  *  Copyright (C) 2016-2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,17 +16,11 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libprocess/spline.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 
 #define RUN_MODES (GWY_RUN_INTERACTIVE | GWY_RUN_IMMEDIATE)
 
@@ -40,7 +34,7 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
+    GwyField *field;
     gboolean realsquare;
     GwySelection *selection;
     GwyGraphModel *gmodel_r;
@@ -55,8 +49,8 @@ typedef struct {
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             extract_path        (GwyContainer *data,
-                                             GwyRunType runtype);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static void             execute             (ModuleArgs *args);
 static GwyDialogOutcome run_gui             (ModuleArgs *args);
 static void             param_changed       (ModuleGUI *gui,
@@ -78,11 +72,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("extract_path",
-                              (GwyProcessFunc)&extract_path,
+                              module_main,
                               N_("/_Distortion/Extract _Path Selection..."),
-                              GWY_STOCK_EXTRACT_PATH,
+                              GWY_ICON_EXTRACT_PATH,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Extract path selection data"));
 
     return TRUE;
@@ -106,29 +100,26 @@ define_module_params(void)
 }
 
 static void
-extract_path(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyAppDataId nullid = GWY_APP_DATA_ID_NONE;
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     ModuleArgs args;
-    gchar key[48];
     gint id;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    g_return_if_fail(g_type_from_name("GwyLayerPath"));
+    g_return_if_fail(mode & RUN_MODES);
 
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(args.field);
     args.params = gwy_params_new_from_settings(define_module_params());
 
-    g_snprintf(key, sizeof(key), "/%d/select/path", id);
-    gwy_container_gis_object_by_name(data, key, &args.selection);
-    gwy_container_gis_boolean(data, gwy_app_get_data_real_square_key_for_id(id), &args.realsquare);
+    gwy_container_gis_object(GWY_CONTAINER(data), gwy_file_key_image_selection(id, "path"), &args.selection);
+    gwy_container_gis_boolean(GWY_CONTAINER(data), gwy_file_key_image_real_square(id), &args.realsquare);
 
-    if (runtype == GWY_RUN_IMMEDIATE) {
+    if (mode ==GWY_RUN_IMMEDIATE) {
         if (!args.selection)
             goto end;
     }
@@ -161,7 +152,7 @@ run_gui(ModuleArgs *args)
     GwyParamTable *table;
     gchar buf[16];
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
     gui.dialog = gwy_dialog_new(_("Extract Path Selection"));
     dialog = GWY_DIALOG(gui.dialog);
@@ -252,15 +243,15 @@ create_graph_model(const GwyXY *points, const gdouble *xdata, gdouble *ydata, gu
 
 /* XXX: This replicates straighten_path.c */
 static GwyXY*
-rescale_points(GwySelection *selection, GwyDataField *field, gboolean realsquare,
+rescale_points(GwySelection *selection, GwyField *field, gboolean realsquare,
                gdouble *pdx, gdouble *pdy, gdouble *pqx, gdouble *pqy)
 {
     gdouble dx, dy, qx, qy, h;
     GwyXY *points;
     guint n, i;
 
-    dx = gwy_data_field_get_dx(field);
-    dy = gwy_data_field_get_dy(field);
+    dx = gwy_field_get_dx(field);
+    dy = gwy_field_get_dy(field);
     h = MIN(dx, dy);
     if (realsquare) {
         qx = h/dx;
@@ -298,7 +289,7 @@ execute(ModuleArgs *args)
     GwyGraphModel *gmodel;
     GwySpline *spline;
     GwyXY *points, *tangents;
-    GwySIUnit *xyunit;
+    GwyUnit *xyunit;
     gdouble dx, dy, qx, qy, h, l, length, slackness;
     gdouble *xdata, *ydata;
     guint n, i;
@@ -340,14 +331,14 @@ execute(ModuleArgs *args)
         }
     }
 
-    xyunit = gwy_data_field_get_si_unit_xy(args->field);
+    xyunit = gwy_field_get_unit_xy(args->field);
     if ((gmodel = create_graph_model(points, xdata, ydata, n, _("Distance"), _("Position"), make_x, make_y))) {
-        g_object_set(gmodel, "si-unit-x", xyunit, "si-unit-y", xyunit, NULL);
+        g_object_set(gmodel, "unit-x", xyunit, "unit-y", xyunit, NULL);
         args->gmodel_r = gmodel;
     }
 
     if ((gmodel = create_graph_model(tangents, xdata, ydata, n, _("Distance"), _("Position"), make_vx, make_vy))) {
-        g_object_set(gmodel, "si-unit-x", xyunit, NULL);
+        g_object_set(gmodel, "unit-x", xyunit, NULL);
         args->gmodel_v = gmodel;
     }
 
