@@ -1,5 +1,5 @@
 /*
- *  $Id: facet_measure.c 26752 2024-10-18 15:49:13Z yeti-dn $
+ *  $Id: facet_measure.c 28910 2025-11-24 18:11:09Z yeti-dn $
  *  Copyright (C) 2019-2024 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -21,22 +21,13 @@
  **/
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwythreads.h>
-#include <libprocess/stats.h>
-#include <libprocess/level.h>
-#include <libprocess/filters.h>
-#include <libprocess/grains.h>
-#include <libprocess/elliptic.h>
-#include <libgwydgets/gwynullstore.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "preview.h"
+#include "libgwyapp/sanity.h"
 
 #define RUN_MODES (GWY_RUN_INTERACTIVE)
 
@@ -93,16 +84,16 @@ typedef struct {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *mask;
-    GwyDataField *theta;
-    GwyDataField *phi;
-    GwyDataField *result;
+    GwyField *field;
+    GwyField *mask;
+    GwyField *theta;
+    GwyField *phi;
+    GwyField *result;
 } ModuleArgs;
 
 typedef struct {
     ModuleArgs *args;
-    GwyContainer *args_data;
+    GwyFile *args_data;
     GtkWidget *dialog;
     GtkWidget *mark_button;
     GtkWidget *delete;
@@ -110,97 +101,95 @@ typedef struct {
     GtkWidget *theta_0_label;
     GtkWidget *theta_max_label;
     GwyParamTable *table;
-    GwyContainer *data;
-    GwyContainer *fdata;
-    GwyDataField *dist;
-    GwyDataField *mask;
+    GwyField *dist;
+    GwyField *mask;
     GwyNullStore *store;
     GwySelection *fselection;
     GwySelection *iselection;
     GArray *measured_data;
-    gchar *selkey;
+    GQuark selkey;
     gdouble q;
     gint selid;
     gboolean did_init;
 } ModuleGUI;
 
-static gboolean         module_register                  (void);
-static GwyParamDef*     define_module_params             (void);
-static void             facet_measure                    (GwyContainer *data,
-                                                          GwyRunType runtype);
-static void             execute                          (ModuleArgs *args);
-static GwyDialogOutcome run_gui                          (ModuleArgs *args,
-                                                          GwyContainer *data,
-                                                          gint id);
-static void             param_changed                    (ModuleGUI *gui,
-                                                          gint id);
-static void             dialog_response                  (ModuleGUI *gui,
-                                                          gint response);
-static void             save_facet_selection             (ModuleGUI *gui);
-static GtkWidget*       create_theta_box                 (ModuleGUI *gui);
-static GtkWidget*       create_point_list                (ModuleGUI *gui);
-static void             point_list_selection_changed     (GtkTreeSelection *treesel,
-                                                          ModuleGUI *gui);
-static void             render_id                        (GtkCellLayout *layout,
-                                                          GtkCellRenderer *renderer,
-                                                          GtkTreeModel *model,
-                                                          GtkTreeIter *iter,
-                                                          gpointer user_data);
-static void             render_npoints                   (GtkCellLayout *layout,
-                                                          GtkCellRenderer *renderer,
-                                                          GtkTreeModel *model,
-                                                          GtkTreeIter *iter,
-                                                          gpointer user_data);
-static void             render_facet_angle               (GtkCellLayout *layout,
-                                                          GtkCellRenderer *renderer,
-                                                          GtkTreeModel *model,
-                                                          GtkTreeIter *iter,
-                                                          gpointer user_data);
-static void             render_facet_coordinate          (GtkCellLayout *layout,
-                                                          GtkCellRenderer *renderer,
-                                                          GtkTreeModel *model,
-                                                          GtkTreeIter *iter,
-                                                          gpointer user_data);
-static void             clear_measurements               (ModuleGUI *gui);
-static void             delete_measurement               (ModuleGUI *gui);
-static void             refine_facet                     (ModuleGUI *gui);
-static void             mark_facet                       (gpointer user_data);
-static void             measure_facet                    (ModuleGUI *gui);
-static gchar*           format_facet_table               (gpointer user_data);
-static gboolean         point_list_key_pressed           (ModuleGUI *gui,
-                                                          GdkEventKey *event);
-static void             recalculate_distribution         (ModuleGUI *gui);
-static void             update_theta_range               (ModuleGUI *gui);
-static void             facet_view_select_angle          (ModuleGUI *gui,
-                                                          gdouble theta,
-                                                          gdouble phi);
-static void             facet_view_selection_updated     (GwySelection *selection,
-                                                          gint hint,
-                                                          ModuleGUI *gui);
-static void             preview_selection_updated        (GwySelection *selection,
-                                                          gint id,
-                                                          ModuleGUI *gui);
-static void             gwy_data_field_mark_facets       (GwyDataField *dtheta,
-                                                          GwyDataField *dphi,
-                                                          gdouble theta0,
-                                                          gdouble phi0,
-                                                          gdouble tolerance,
-                                                          GwyDataField *mask);
-static void             calculate_average_angle          (GwyDataField *dtheta,
-                                                          GwyDataField *dphi,
-                                                          gdouble theta0,
-                                                          gdouble phi0,
-                                                          gdouble tolerance,
-                                                          FacetMeasurement *fmeas);
-static gdouble          calc_facet_distribution          (ModuleArgs *args,
-                                                          GwyDataField *dist);
-static void             mark_fdata                       (GwyDataField *mask,
-                                                          gdouble q,
-                                                          gdouble theta0,
-                                                          gdouble phi0,
-                                                          gdouble tolerance);
-static GwyDataField*    make_fdist_field                 (GwyDataField *field);
-static void             sanitise_params                  (ModuleArgs *args);
+static gboolean         module_register             (void);
+static GwyParamDef*     define_module_params        (void);
+static void             module_main                 (GwyFile *data,
+                                                     GwyRunModeFlags mode);
+static void             execute                     (ModuleArgs *args);
+static GwyDialogOutcome run_gui                     (ModuleArgs *args,
+                                                     GwyFile *data,
+                                                     gint id);
+static void             param_changed               (ModuleGUI *gui,
+                                                     gint id);
+static void             dialog_response             (ModuleGUI *gui,
+                                                     gint response);
+static void             save_facet_selection        (ModuleGUI *gui);
+static GtkWidget*       create_theta_box            (ModuleGUI *gui);
+static GtkWidget*       create_point_list           (ModuleGUI *gui);
+static void             point_list_selection_changed(GtkTreeSelection *treesel,
+                                                     ModuleGUI *gui);
+static void             render_id                   (GtkCellLayout *layout,
+                                                     GtkCellRenderer *renderer,
+                                                     GtkTreeModel *model,
+                                                     GtkTreeIter *iter,
+                                                     gpointer user_data);
+static void             render_npoints              (GtkCellLayout *layout,
+                                                     GtkCellRenderer *renderer,
+                                                     GtkTreeModel *model,
+                                                     GtkTreeIter *iter,
+                                                     gpointer user_data);
+static void             render_facet_angle          (GtkCellLayout *layout,
+                                                     GtkCellRenderer *renderer,
+                                                     GtkTreeModel *model,
+                                                     GtkTreeIter *iter,
+                                                     gpointer user_data);
+static void             render_facet_coordinate     (GtkCellLayout *layout,
+                                                     GtkCellRenderer *renderer,
+                                                     GtkTreeModel *model,
+                                                     GtkTreeIter *iter,
+                                                     gpointer user_data);
+static void             clear_measurements          (ModuleGUI *gui);
+static void             delete_measurement          (ModuleGUI *gui);
+static void             refine_facet                (ModuleGUI *gui);
+static void             mark_facet                  (gpointer user_data);
+static void             measure_facet               (ModuleGUI *gui);
+static gchar*           format_facet_table          (gpointer user_data);
+static gboolean         point_list_key_pressed      (ModuleGUI *gui,
+                                                     GdkEventKey *event);
+static void             recalculate_distribution    (ModuleGUI *gui);
+static void             update_theta_range          (ModuleGUI *gui);
+static void             facet_view_select_angle     (ModuleGUI *gui,
+                                                     gdouble theta,
+                                                     gdouble phi);
+static void             facet_view_selection_updated(GwySelection *selection,
+                                                     gint hint,
+                                                     ModuleGUI *gui);
+static void             preview_selection_updated   (GwySelection *selection,
+                                                     gint id,
+                                                     ModuleGUI *gui);
+static void             gwy_field_mark_facets       (GwyField *dtheta,
+                                                     GwyField *dphi,
+                                                     gdouble theta0,
+                                                     gdouble phi0,
+                                                     gdouble tolerance,
+                                                     GwyField *mask);
+static void             calculate_average_angle     (GwyField *dtheta,
+                                                     GwyField *dphi,
+                                                     gdouble theta0,
+                                                     gdouble phi0,
+                                                     gdouble tolerance,
+                                                     FacetMeasurement *fmeas);
+static gdouble          calc_facet_distribution     (ModuleArgs *args,
+                                                     GwyField *dist);
+static void             mark_fdata                  (GwyField *mask,
+                                                     gdouble q,
+                                                     gdouble theta0,
+                                                     gdouble phi0,
+                                                     gdouble tolerance);
+static GwyField*        make_fdist_field            (GwyField *field);
+static void             sanitise_params             (ModuleArgs *args);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
@@ -218,11 +207,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("facet_measure",
-                              (GwyProcessFunc)&facet_measure,
+                              module_main,
                               N_("/Measure _Features/Facet _Measurement..."),
-                              GWY_STOCK_FACET_MEASURE,
+                              GWY_ICON_FACET_MEASURE,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Measure facet angles"));
 
     return TRUE;
@@ -239,7 +228,7 @@ define_module_params(void)
     paramdef = gwy_param_def_new();
     gwy_param_def_set_function_name(paramdef, gwy_process_func_current());
     gwy_param_def_add_int(paramdef, PARAM_KERNEL_SIZE, "kernel-size", _("_Facet plane size"), 0, MAX_PLANE_SIZE, 3);
-    gwy_param_def_add_double(paramdef, PARAM_TOLERANCE, "tolerance", _("_Tolerance"), 0.0, G_PI/6.0, 3.0*G_PI/180.0);
+    gwy_param_def_add_double(paramdef, PARAM_TOLERANCE, "tolerance", _("_Tolerance"), 0.0, G_PI/6.0, gwy_deg2rad(3.0));
     gwy_param_def_add_angle(paramdef, PARAM_PHI0, "phi0", _("Selected φ"), FALSE, 1, 0.0);
     /* The real folding is 4, not 2, but the facet map contains regions outside the possible angles. */
     gwy_param_def_add_angle(paramdef, PARAM_THETA0, "theta0", _("Selected ϑ"), TRUE, 2, 0.0);
@@ -253,30 +242,29 @@ define_module_params(void)
 }
 
 static void
-facet_measure(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome;
     ModuleArgs args;
     GQuark mquark;
     gint id;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    g_return_if_fail(g_type_from_name("GwyLayerPoint"));
+    g_return_if_fail(mode & RUN_MODES);
 
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_MASK_FIELD, &args.mask,
-                                     GWY_APP_MASK_FIELD_KEY, &mquark,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_MASK_FIELD, &args.mask,
+                                 GWY_APP_MASK_FIELD_KEY, &mquark,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(args.field && mquark);
 
     if (!gwy_require_image_same_units(args.field, data, id, _("Measure Facets")))
         return;
 
-    args.result = gwy_data_field_new_alike(args.field, TRUE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(args.result), NULL);
-    args.theta = gwy_data_field_new_alike(args.result, FALSE);
-    args.phi = gwy_data_field_new_alike(args.result, FALSE);
+    args.result = gwy_field_new_alike(args.field, TRUE);
+    gwy_unit_clear(gwy_field_get_unit_z(args.result));
+    args.theta = gwy_field_new_alike(args.result, FALSE);
+    args.phi = gwy_field_new_alike(args.result, FALSE);
     args.params = gwy_params_new_from_settings(define_module_params());
     sanitise_params(&args);
 
@@ -287,12 +275,12 @@ facet_measure(GwyContainer *data, GwyRunType runtype)
 
     execute(&args);
 
-    gwy_app_undo_qcheckpointv(data, 1, &mquark);
-    if (gwy_data_field_get_max(args.result) > 0.0)
-        gwy_container_set_object(data, mquark, args.result);
+    gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &mquark);
+    if (gwy_field_get_max(args.result) > 0.0)
+        gwy_container_set_object(GWY_CONTAINER(data), mquark, args.result);
     else
-        gwy_container_remove(data, mquark);
-    gwy_app_channel_log_add_proc(data, id, id);
+        gwy_container_remove(GWY_CONTAINER(data), mquark);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, id);
 
 end:
     g_object_unref(args.theta);
@@ -302,56 +290,41 @@ end:
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     static const GwyRGBA facet_mask_color = { 0.56, 0.39, 0.07, 0.5 };
 
-    GtkWidget *hbox, *vbox, *auxbox, *scwin, *button, *dataview, *alignment, *pointlist, *widget;
+    GtkWidget *hbox, *vbox, *auxbox, *scwin, *button, *dataview, *pointlist, *widget;
     GwyDialogOutcome outcome;
     GwyDialog *dialog;
     GwyParamTable *table, *extable;
     ModuleGUI gui;
     GtkSizeGroup *sizegroup;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
     gui.selid = -1;
     gui.args_data = data;
-    gui.selkey = g_strdup_printf("/%d/select/_facets", id);
-
-    gui.data = gwy_container_new();
-    gwy_container_set_object_by_name(gui.data, "/0/data", args->field);
-    gwy_container_set_object_by_name(gui.data, "/0/mask", args->result);
-    gwy_app_sync_data_items(data, gui.data, id, 0, FALSE,
-                            GWY_DATA_ITEM_PALETTE,
-                            GWY_DATA_ITEM_RANGE,
-                            GWY_DATA_ITEM_MASK_COLOR,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
-
-    gui.fdata = gwy_container_new();
-    gui.dist = make_fdist_field(args->field);
-    gui.mask = gwy_data_field_new_alike(gui.dist, TRUE);
-    gwy_container_set_object_by_name(gui.fdata, "/0/data", gui.dist);
-    gwy_container_set_object_by_name(gui.fdata, "/0/mask", gui.mask);
-    gwy_container_set_const_string_by_name(gui.fdata, "/0/base/palette", FVIEW_GRADIENT);
-    gwy_rgba_store_to_container(&facet_mask_color, gui.fdata, "/0/mask");
+    gui.selkey = gwy_file_key_image_selection(id, "_facets");
 
     gui.dialog = gwy_dialog_new(_("Measure Facets"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
     /* First row: Image + Options */
-    hbox = gwy_hbox_new(8);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 8);
     gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
     gwy_dialog_add_content(dialog, hbox, FALSE, FALSE, 0);
 
-    dataview = gwy_create_preview(gui.data, 0, IMAGEVIEW_SIZE, TRUE);
-    alignment = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-    gtk_container_add(GTK_CONTAINER(alignment), dataview);
-    gtk_box_pack_start(GTK_BOX(hbox), alignment, FALSE, FALSE, 0);
+    dataview = gwy_create_preview(args->field, args->result, IMAGEVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_RANGE
+                        | GWY_FILE_ITEM_MASK_COLOR | GWY_FILE_ITEM_REAL_SQUARE);
+    gtk_widget_set_halign(dataview, GTK_ALIGN_START);
+    gtk_widget_set_valign(dataview, GTK_ALIGN_START);
+    gtk_box_pack_start(GTK_BOX(hbox), dataview, FALSE, FALSE, 0);
 
-    gui.iselection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(dataview), 0, "Point", 1, TRUE);
+    gui.iselection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(dataview), GWY_TYPE_LAYER_POINT, 1, TRUE);
 
     table = gui.table = gwy_param_table_new(args->params);
     gwy_param_table_append_slider(table, PARAM_KERNEL_SIZE);
@@ -373,7 +346,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_param_table_append_button(table, BUTTON_MEASURE, BUTTON_MARK, RESPONSE_MEASURE, _("Mea_sure"));
 
     gwy_param_table_append_separator(table);
-    gwy_param_table_append_mask_color(table, PARAM_MASK_COLOR, gui.data, 0, data, id);
+    gwy_param_table_append_image_mask_color(table, PARAM_MASK_COLOR, GWY_DATA_VIEW(dataview), data, id);
     if (args->mask) {
         gwy_param_table_append_radio_buttons(table, PARAM_COMBINE_TYPE, NULL);
         gwy_param_table_add_enabler(table, PARAM_COMBINE, PARAM_COMBINE_TYPE);
@@ -384,21 +357,25 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_dialog_add_param_table(dialog, table);
 
     /* Second row: Facet view + Facet list */
-    hbox = gwy_hbox_new(8);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 8);
     gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
     gwy_dialog_add_content(dialog, hbox, FALSE, FALSE, 0);
 
-    vbox = gwy_vbox_new(2);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 2);
     gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);
 
-    dataview = gwy_create_preview(gui.fdata, 0, FACETVIEW_SIZE, TRUE);
+    gui.dist = make_fdist_field(args->field);
+    gui.mask = gwy_field_new_alike(gui.dist, TRUE);
+    dataview = gwy_create_preview(gui.dist, gui.mask, FACETVIEW_SIZE);
+    gwy_data_view_set_gradient(GWY_DATA_VIEW(dataview), gwy_gradients_get_gradient(FVIEW_GRADIENT));
+    gwy_data_view_set_mask_color(GWY_DATA_VIEW(dataview), &facet_mask_color);
     gtk_box_pack_start(GTK_BOX(vbox), dataview, FALSE, FALSE, 0);
-    gui.fselection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(dataview), 0, "Point", 1, TRUE);
+    gui.fselection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(dataview), GWY_TYPE_LAYER_POINT, 1, TRUE);
     g_object_ref(gui.fselection);
 
     gtk_box_pack_start(GTK_BOX(vbox), create_theta_box(&gui), FALSE, FALSE, 0);
 
-    vbox = gwy_vbox_new(0);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
     gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);
 
     pointlist = create_point_list(&gui);
@@ -407,7 +384,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gtk_container_add(GTK_CONTAINER(scwin), pointlist);
     gtk_box_pack_start(GTK_BOX(vbox), scwin, TRUE, TRUE, 0);
 
-    auxbox = gwy_hbox_new(0);
+    auxbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
     gtk_box_pack_start(GTK_BOX(vbox), auxbox, FALSE, FALSE, 0);
 
     extable = gwy_param_table_new(args->params);
@@ -421,12 +398,12 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_dialog_add_param_table(dialog, extable);
 
     sizegroup = gtk_size_group_new(GTK_SIZE_GROUP_BOTH);
-    button = gwy_stock_like_button_new(_("_Clear"), GTK_STOCK_CLEAR);
+    button = gwy_create_stock_button(_("_Clear"), GWY_ICON_GTK_CLEAR);
     gtk_box_pack_start(GTK_BOX(auxbox), button, FALSE, FALSE, 0);
     gtk_size_group_add_widget(sizegroup, button);
     g_signal_connect_swapped(button, "clicked", G_CALLBACK(clear_measurements), &gui);
 
-    button = gwy_stock_like_button_new(_("_Delete"), GTK_STOCK_DELETE);
+    button = gwy_create_stock_button(_("_Delete"), GWY_ICON_GTK_DELETE);
     gui.delete = button;
     gtk_box_pack_start(GTK_BOX(auxbox), button, FALSE, FALSE, 0);
     gtk_size_group_add_widget(sizegroup, button);
@@ -444,12 +421,9 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 
     save_facet_selection(&gui);
     g_object_unref(gui.fselection);
-    g_object_unref(gui.data);
-    g_object_unref(gui.fdata);
     g_object_unref(gui.dist);
     g_object_unref(gui.mask);
     g_array_free(gui.measured_data, TRUE);
-    g_free(gui.selkey);
 
     return outcome;
 }
@@ -457,18 +431,18 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 static GtkWidget*
 create_theta_box(ModuleGUI *gui)
 {
-    GtkWidget *thetabox = gwy_hbox_new(0);
+    GtkWidget *thetabox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 
     gui->theta_min_label = gtk_label_new(NULL);
-    gtk_misc_set_alignment(GTK_MISC(gui->theta_min_label), 0.0, 0.5);
+    gtk_label_set_xalign(GTK_LABEL(gui->theta_min_label), 0.0);
     gtk_box_pack_start(GTK_BOX(thetabox), gui->theta_min_label, TRUE, TRUE, 0);
 
     gui->theta_0_label = gtk_label_new(NULL);
-    gtk_misc_set_alignment(GTK_MISC(gui->theta_0_label), 0.5, 0.5);
+    gtk_label_set_xalign(GTK_LABEL(gui->theta_0_label), 0.5);
     gtk_box_pack_start(GTK_BOX(thetabox), gui->theta_0_label, TRUE, TRUE, 0);
 
     gui->theta_max_label = gtk_label_new(NULL);
-    gtk_misc_set_alignment(GTK_MISC(gui->theta_max_label), 1.0, 0.5);
+    gtk_label_set_xalign(GTK_LABEL(gui->theta_max_label), 1.0);
     gtk_box_pack_start(GTK_BOX(thetabox), gui->theta_max_label, TRUE, TRUE, 0);
 
     return thetabox;
@@ -670,7 +644,7 @@ render_facet_angle(GtkCellLayout *layout,
         g_assert_not_reached();
     }
 
-    g_snprintf(buf, sizeof(buf), "%.3f", 180.0/G_PI*u);
+    g_snprintf(buf, sizeof(buf), "%.3f", gwy_rad2deg(u));
     g_object_set(renderer, "text", buf, NULL);
 }
 
@@ -722,22 +696,23 @@ param_changed(ModuleGUI *gui, gint id)
     /* This requires gui->q already calculated because we set selection on the facet view.  Life would be easier
      * if selections used offset coordinates. */
     if (id < 0 && !gui->did_init) {
-        GwyContainer *data = gui->args_data;
+        GwyFile *data = gui->args_data;
         GwySelection *sel;
         gdouble xy[2];
 
         gui->did_init = TRUE;
-        if (gwy_container_gis_object_by_name(data, gui->selkey, &sel) && gwy_selection_get_object(sel, 0, xy) > 0)
+        if (gwy_container_gis_object(GWY_CONTAINER(data), gui->selkey, &sel)
+            && gwy_selection_get_object(sel, 0, xy) > 0)
             slopes_to_angles(xy[0], xy[1], &theta, &phi);
         /* XXX: recursion? */
         facet_view_select_angle(gui, theta, phi);
     }
     if (id < 0 || id == PARAM_THETA0) {
-        g_snprintf(buf, sizeof(buf), "%.2f", 180.0/G_PI*theta);
+        g_snprintf(buf, sizeof(buf), "%.2f", gwy_rad2deg(theta));
         gwy_param_table_info_set_valuestr(table, INFO_THETA, buf);
     }
     if (id < 0 || id == PARAM_PHI0) {
-        g_snprintf(buf, sizeof(buf), "%.2f", 180.0/G_PI*phi);
+        g_snprintf(buf, sizeof(buf), "%.2f", gwy_rad2deg(phi));
         gwy_param_table_info_set_valuestr(table, INFO_PHI, buf);
     }
     if (id < 0 || id == PARAM_UPDATE)
@@ -765,7 +740,7 @@ save_facet_selection(ModuleGUI *gui)
     gdouble theta, phi, xy[2];
 
     if (!gwy_selection_get_data(gui->fselection, NULL)) {
-        gwy_container_remove_by_name(gui->args_data, gui->selkey);
+        gwy_container_remove(GWY_CONTAINER(gui->args_data), gui->selkey);
         return;
     }
 
@@ -773,9 +748,10 @@ save_facet_selection(ModuleGUI *gui)
     xy_to_angles(xy[0], xy[1], gui->q, &theta, &phi);
     angles_to_slopes(theta, phi, xy+0, xy+1);
     /* Create a new object.  We have signals connected to the old one. */
-    selection = g_object_new(g_type_from_name("GwySelectionPoint"), "max-objects", 1, NULL);
+    selection = gwy_selection_point_new();
+    gwy_selection_set_max_objects(selection, 1);
     gwy_selection_set_data(selection, 1, xy);
-    gwy_container_pass_object_by_name(gui->args_data, gui->selkey, selection);
+    gwy_container_pass_object(GWY_CONTAINER(gui->args_data), gui->selkey, selection);
 }
 
 static void
@@ -819,19 +795,19 @@ static void
 refine_facet(ModuleGUI *gui)
 {
     gdouble tolerance = gwy_params_get_double(gui->args->params, PARAM_TOLERANCE);
-    GwyDataField *dist = gui->dist;
+    GwyField *dist = gui->dist;
     gdouble xy[2], theta, phi, x, y, h;
     gint fres, range;
 
     gwy_selection_get_object(gui->fselection, 0, xy);
     xy_to_angles(xy[0], xy[1], gui->q, &theta, &phi);
 
-    fres = gwy_data_field_get_xres(dist);
-    h = gwy_data_field_get_dx(dist);
+    fres = gwy_field_get_xres(dist);
+    h = gwy_field_get_dx(dist);
     range = GWY_ROUND(fres/gui->q * 0.5/G_SQRT2 * cos(0.5*theta) * tolerance);
     x = xy[0]/h;
     y = xy[1]/h;
-    gwy_data_field_local_maximum(dist, &x, &y, range, range);
+    gwy_field_local_maximum(dist, &x, &y, range, range);
     xy[0] = x*h;
     xy[1] = y*h;
     gwy_selection_set_object(gui->fselection, 0, xy);
@@ -848,8 +824,8 @@ mark_facet(gpointer user_data)
 
     execute(args);
     mark_fdata(gui->mask, gui->q, theta0, phi0, tolerance);
-    gwy_data_field_data_changed(args->result);
-    gwy_data_field_data_changed(gui->mask);
+    gwy_field_data_changed(args->result);
+    gwy_field_data_changed(gui->mask);
     gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
 }
 
@@ -872,7 +848,7 @@ static gboolean
 point_list_key_pressed(ModuleGUI *gui,
                        GdkEventKey *event)
 {
-    if (event->keyval == GDK_Delete) {
+    if (event->keyval == GDK_KEY_Delete) {
         delete_measurement(gui);
         return TRUE;
     }
@@ -934,9 +910,9 @@ recalculate_distribution(ModuleGUI *gui)
         gwy_app_wait_cursor_start(GTK_WINDOW(gui->dialog));
     gui->q = calc_facet_distribution(args, gui->dist);
 
-    gwy_data_field_clear(gui->mask);
-    gwy_data_field_data_changed(gui->mask);
-    gwy_data_field_data_changed(gui->dist);
+    gwy_field_clear(gui->mask);
+    gwy_field_data_changed(gui->mask);
+    gwy_field_data_changed(gui->dist);
     update_theta_range(gui);
 
     for (i = 0; i < n; i++)
@@ -954,11 +930,11 @@ update_theta_range(ModuleGUI *gui)
     gchar buf[32];
 
     xy_to_angles(gui->q, 0.0, gui->q, &theta, &phi);
-    g_snprintf(buf, sizeof(buf), "%.1f %s", -180.0/G_PI*theta, _("deg"));
+    g_snprintf(buf, sizeof(buf), "%.1f %s", -gwy_rad2deg(theta), _("deg"));
     gtk_label_set_text(GTK_LABEL(gui->theta_min_label), buf);
     g_snprintf(buf, sizeof(buf), "0 %s", _("deg"));
     gtk_label_set_text(GTK_LABEL(gui->theta_0_label), buf);
-    g_snprintf(buf, sizeof(buf), "%.1f %s", 180.0/G_PI*theta, _("deg"));
+    g_snprintf(buf, sizeof(buf), "%.1f %s", gwy_rad2deg(theta), _("deg"));
     gtk_label_set_text(GTK_LABEL(gui->theta_max_label), buf);
 }
 
@@ -994,7 +970,7 @@ preview_selection_updated(GwySelection *selection,
                           ModuleGUI *gui)
 {
     ModuleArgs *args = gui->args;
-    GwyDataField *field = args->field, *dtheta = args->theta, *dphi = args->phi;
+    GwyField *field = args->field, *dtheta = args->theta, *dphi = args->phi;
     gdouble theta, phi, xy[2];
     gint i, j;
 
@@ -1002,10 +978,10 @@ preview_selection_updated(GwySelection *selection,
         return;
 
     gwy_selection_get_object(selection, 0, xy);
-    j = gwy_data_field_rtoj(field, xy[0]);
-    i = gwy_data_field_rtoi(field, xy[1]);
-    theta = gwy_data_field_get_val(dtheta, j, i);
-    phi = gwy_data_field_get_val(dphi, j, i);
+    j = gwy_field_rtoj(field, xy[0]);
+    i = gwy_field_rtoi(field, xy[1]);
+    theta = gwy_field_get_val(dtheta, j, i);
+    phi = gwy_field_get_val(dphi, j, i);
     facet_view_select_angle(gui, theta, phi);
 }
 
@@ -1018,27 +994,27 @@ execute(ModuleArgs *args)
     gdouble tolerance = gwy_params_get_double(params, PARAM_TOLERANCE);
     GwyMergeType combine_type = gwy_params_get_enum(params, PARAM_COMBINE_TYPE);
     gboolean combine = gwy_params_get_boolean(params, PARAM_COMBINE);
-    GwyDataField *result = args->result, *mask = args->mask;
+    GwyField *result = args->result, *mask = args->mask;
 
-    gwy_data_field_mark_facets(args->theta, args->phi, theta0, phi0, tolerance, result);
+    gwy_field_mark_facets(args->theta, args->phi, theta0, phi0, tolerance, result);
     if (mask && combine) {
         if (combine_type == GWY_MERGE_UNION)
-            gwy_data_field_grains_add(result, mask);
+            gwy_field_grains_add(result, mask);
         else if (combine_type == GWY_MERGE_INTERSECTION)
-            gwy_data_field_grains_intersect(result, mask);
+            gwy_field_grains_intersect(result, mask);
     }
 }
 
 static void
-gwy_data_field_mark_facets(GwyDataField *dtheta, GwyDataField *dphi,
-                           gdouble theta0, gdouble phi0, gdouble tolerance,
-                           GwyDataField *mask)
+gwy_field_mark_facets(GwyField *dtheta, GwyField *dphi,
+                      gdouble theta0, gdouble phi0, gdouble tolerance,
+                      GwyField *mask)
 {
     gdouble ctol = cos(tolerance), cth0 = cos(theta0), sth0 = sin(theta0);
-    const gdouble *td = gwy_data_field_get_data_const(dtheta);
-    const gdouble *pd = gwy_data_field_get_data_const(dphi);
-    gdouble *md = gwy_data_field_get_data(mask);
-    guint i, n = gwy_data_field_get_xres(dtheta)*gwy_data_field_get_yres(dtheta);
+    const gdouble *td = gwy_field_get_data_const(dtheta);
+    const gdouble *pd = gwy_field_get_data_const(dphi);
+    gdouble *md = gwy_field_get_data(mask);
+    guint i, n = gwy_field_get_xres(dtheta)*gwy_field_get_yres(dtheta);
 
 #ifdef _OPENMP
 #pragma omp parallel for if(gwy_threads_are_enabled()) default(none) \
@@ -1052,16 +1028,16 @@ gwy_data_field_mark_facets(GwyDataField *dtheta, GwyDataField *dphi,
 }
 
 static gdouble
-calc_facet_distribution(ModuleArgs *args, GwyDataField *dist)
+calc_facet_distribution(ModuleArgs *args, GwyField *dist)
 {
     gint kernel_size = gwy_params_get_int(args->params, PARAM_KERNEL_SIZE);
-    gint i, fres = gwy_data_field_get_xres(dist);
+    gint i, fres = gwy_field_get_xres(dist);
     gdouble *d;
 
-    gwy_data_field_facet_distribution(args->field, dist, (fres - 1)/2, 2*kernel_size+1, args->theta, args->phi);
+    gwy_field_facet_distribution(args->field, dist, (fres - 1)/2, 2*kernel_size+1, args->theta, args->phi);
 
     /* Transform values for visualisation. */
-    d = gwy_data_field_get_data(dist);
+    d = gwy_field_get_data(dist);
 #ifdef _OPENMP
 #pragma omp parallel for if(gwy_threads_are_enabled()) default(none) \
             private(i) \
@@ -1070,11 +1046,11 @@ calc_facet_distribution(ModuleArgs *args, GwyDataField *dist)
     for (i = 0; i < fres*fres; i++)
         d[i] = cbrt(d[i]);
 
-    return 0.5*gwy_data_field_get_xreal(dist);
+    return 0.5*gwy_field_get_xreal(dist);
 }
 
 static void
-mark_fdata(GwyDataField *mask, gdouble q, gdouble theta0, gdouble phi0, gdouble tolerance)
+mark_fdata(GwyField *mask, gdouble q, gdouble theta0, gdouble phi0, gdouble tolerance)
 {
     gdouble r, r2, cr, cro, cth0, sth0, cphi0, sphi0;
     gint fres, hres, i, j;
@@ -1085,10 +1061,10 @@ mark_fdata(GwyDataField *mask, gdouble q, gdouble theta0, gdouble phi0, gdouble
     sth0 = sin(theta0);
     cphi0 = cos(phi0);
     sphi0 = sin(phi0);
-    fres = gwy_data_field_get_xres(mask);
-    g_assert(gwy_data_field_get_yres(mask) == fres);
+    fres = gwy_field_get_xres(mask);
+    g_assert(gwy_field_get_yres(mask) == fres);
     hres = (fres - 1)/2;
-    m = gwy_data_field_get_data(mask);
+    m = gwy_field_get_data(mask);
 
 #ifdef _OPENMP
 #pragma omp parallel for if(gwy_threads_are_enabled()) default(none) \
@@ -1118,7 +1094,7 @@ mark_fdata(GwyDataField *mask, gdouble q, gdouble theta0, gdouble phi0, gdouble
 }
 
 static void
-calculate_average_angle(GwyDataField *dtheta, GwyDataField *dphi,
+calculate_average_angle(GwyField *dtheta, GwyField *dphi,
                         gdouble theta0, gdouble phi0, gdouble tolerance,
                         FacetMeasurement *fmeas)
 {
@@ -1134,9 +1110,9 @@ calculate_average_angle(GwyDataField *dtheta, GwyDataField *dphi,
     sth0 = sin(theta0);
     ctol = cos(tolerance);
 
-    td = gwy_data_field_get_data_const(dtheta);
-    pd = gwy_data_field_get_data_const(dphi);
-    n = gwy_data_field_get_xres(dtheta)*gwy_data_field_get_yres(dtheta);
+    td = gwy_field_get_data_const(dtheta);
+    pd = gwy_field_get_data_const(dphi);
+    n = gwy_field_get_xres(dtheta)*gwy_field_get_yres(dtheta);
     count = 0;
     sx = sy = sz = 0.0;
 
@@ -1196,16 +1172,16 @@ calculate_average_angle(GwyDataField *dtheta, GwyDataField *dphi,
     fmeas->error = sqrt(s2/(count - 1));
 }
 
-static GwyDataField*
-make_fdist_field(GwyDataField *field)
+static GwyField*
+make_fdist_field(GwyField *field)
 {
-    gint n = gwy_data_field_get_xres(field)*gwy_data_field_get_yres(field);
+    gint n = gwy_field_get_xres(field)*gwy_field_get_yres(field);
     gint fres = 2*GWY_ROUND(cbrt(3.49*n)) + 1;
-    GwyDataField *fdist;
+    GwyField *fdist;
 
-    fdist = gwy_data_field_new(fres, fres, 1.0, 1.0, FALSE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(fdist), NULL);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(fdist), NULL);
+    fdist = gwy_field_new(fres, fres, 1.0, 1.0, FALSE);
+    gwy_unit_clear(gwy_field_get_unit_xy(fdist));
+    gwy_unit_clear(gwy_field_get_unit_z(fdist));
     return fdist;
 }
 
