@@ -1,5 +1,5 @@
 /*
- *  $Id: fft.c 24116 2021-09-13 15:37:17Z yeti-dn $
+ *  $Id: fft.c 28910 2025-11-24 18:11:09Z yeti-dn $
  *  Copyright (C) 2003-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,20 +16,9 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwythreads.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/inttrans.h>
-#include <libprocess/stats.h>
-#include <libgwydgets/gwydgetutils.h>
-#include <libgwydgets/gwycombobox.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
@@ -54,7 +43,7 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
+    GwyField *field;
 } ModuleArgs;
 
 typedef struct {
@@ -65,24 +54,24 @@ typedef struct {
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             fft                 (GwyContainer *data,
-                                             GwyRunType runtype);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static GwyDialogOutcome run_gui             (ModuleArgs *args);
 static void             param_changed       (ModuleGUI *gui,
                                              gint id);
-static gboolean         imagpart_filter     (GwyContainer *data,
+static gboolean         imagpart_filter     (GwyFile *data,
                                              gint id,
                                              gpointer user_data);
-static void             create_output       (GwyContainer *data,
+static void             create_output       (GwyFile *data,
                                              gint id,
-                                             GwyDataField *field,
+                                             GwyField *field,
                                              const gchar *output_name,
                                              gboolean itransform,
                                              gboolean is_phase);
-static GwyDataField*    make_modulus        (GwyDataField *re,
-                                             GwyDataField *im);
-static GwyDataField*    make_phase          (GwyDataField *re,
-                                             GwyDataField *im);
+static GwyField*        make_modulus        (GwyField *re,
+                                             GwyField *im);
+static GwyField*        make_phase          (GwyField *re,
+                                             GwyField *im);
 static void             sanitise_params     (ModuleArgs *args);
 
 static GwyModuleInfo module_info = {
@@ -101,11 +90,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("fft",
-                              (GwyProcessFunc)&fft,
+                              module_main,
                               N_("/_Integral Transforms/2D _FFT..."),
-                              GWY_STOCK_FFT,
+                              GWY_ICON_FFT,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Compute Fast Fourier Transform"));
 
     return TRUE;
@@ -140,9 +129,9 @@ define_module_params(void)
 }
 
 static void
-fft(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
-    GwyDataField *field, *imagpart, *raout, *ipout;
+    GwyField *field, *imagpart, *raout, *ipout;
     GwyDialogOutcome outcome;
     ModuleArgs args;
     GwyParams *params;
@@ -150,19 +139,19 @@ fft(GwyContainer *data, GwyRunType runtype)
     OutputFlags out;
     gint id, datano;
 
-    g_return_if_fail(runtype & RUN_MODES);
+    g_return_if_fail(mode & RUN_MODES);
 
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &field,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     GWY_APP_CONTAINER_ID, &datano,
-                                     0);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &field,
+                                 GWY_APP_FIELD_ID, &id,
+                                 GWY_APP_CONTAINER_ID, &datano,
+                                 0);
     g_return_if_fail(field);
 
     args.field = field;
     args.params = params = gwy_params_new_from_settings(define_module_params());
     sanitise_params(&args);
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args);
         gwy_params_save_to_settings(params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -172,8 +161,8 @@ fft(GwyContainer *data, GwyRunType runtype)
     if (!out)
         goto end;
 
-    raout = gwy_data_field_new_alike(field, FALSE);
-    ipout = gwy_data_field_new_alike(field, FALSE);
+    raout = gwy_field_new_alike(field, FALSE);
+    ipout = gwy_field_new_alike(field, FALSE);
 
     use_imagpart = gwy_params_get_boolean(params, PARAM_USE_IMAG_PART);
     raw_transform = gwy_params_get_boolean(params, PARAM_RAW_TRANSFORM);
@@ -181,32 +170,31 @@ fft(GwyContainer *data, GwyRunType runtype)
     imagpart = use_imagpart ? gwy_params_get_image(params, PARAM_IMAG_PART) : NULL;
 
     if (is_inv) {
-        GwyDataField *rein = gwy_data_field_duplicate(field);
-        GwyDataField *imin = imagpart ? gwy_data_field_duplicate(imagpart) : NULL;
+        GwyField *rein = gwy_field_copy(field);
+        GwyField *imin = imagpart ? gwy_field_copy(imagpart) : NULL;
 
-        gwy_data_field_2dfft_dehumanize(rein);
-        gwy_data_field_fft_postprocess(rein, FALSE);
+        gwy_field_fft_2d_decenter(rein);
+        gwy_field_fft_postprocess(rein, FALSE);
         if (imin) {
-            gwy_data_field_2dfft_dehumanize(imin);
-            gwy_data_field_fft_postprocess(imin, FALSE);
+            gwy_field_fft_2d_decenter(imin);
+            gwy_field_fft_postprocess(imin, FALSE);
         }
-        gwy_data_field_2dfft_raw(rein, imin, raout, ipout, GWY_TRANSFORM_DIRECTION_BACKWARD);
+        gwy_field_fft_2d_raw(rein, imin, raout, ipout, GWY_TRANSFORM_DIRECTION_BACKWARD);
         g_object_unref(rein);
-        GWY_OBJECT_UNREF(imin);
+        g_clear_object(&imin);
         humanize = FALSE;
     }
     else if (raw_transform)
-        gwy_data_field_2dfft_raw(field, imagpart, raout, ipout, GWY_TRANSFORM_DIRECTION_FORWARD);
+        gwy_field_fft_2d_raw(field, imagpart, raout, ipout, GWY_TRANSFORM_DIRECTION_FORWARD);
     else {
         gboolean preserverms = gwy_params_get_boolean(params, PARAM_PRESERVERMS);
         gboolean zeromean = gwy_params_get_boolean(params, PARAM_ZEROMEAN);
         GwyWindowingType window = gwy_params_get_enum(params, PARAM_WINDOW);
-        gwy_data_field_2dfft(field, imagpart, raout, ipout,
-                             window, GWY_TRANSFORM_DIRECTION_FORWARD, GWY_INTERPOLATION_LINEAR,  /* ignored */
-                             preserverms, zeromean ? 1 : 0);
+        gwy_field_fft_2d(field, imagpart, raout, ipout,
+                         window, GWY_TRANSFORM_DIRECTION_FORWARD, preserverms, zeromean ? 1 : 0);
     }
-    gwy_data_field_fft_postprocess(raout, humanize);
-    gwy_data_field_fft_postprocess(ipout, humanize);
+    gwy_field_fft_postprocess(raout, humanize);
+    gwy_field_fft_postprocess(ipout, humanize);
 
     if (out & OUTPUT_REAL)
         create_output(data, id, g_object_ref(raout), _("FFT Real"), is_inv, FALSE);
@@ -225,38 +213,40 @@ end:
 }
 
 static void
-create_output(GwyContainer *data, gint id, GwyDataField *field, const gchar *output_name,
+create_output(GwyFile *data, gint id, GwyField *field, const gchar *output_name,
               gboolean itransform, gboolean is_phase)
 {
     gint newid;
 
-    newid = gwy_app_data_browser_add_data_field(field, data, TRUE);
-    gwy_app_set_data_field_title(data, newid, output_name);
-    gwy_app_channel_log_add_proc(data, id, newid);
+    newid = gwy_file_add_image(data, field);
+
+    gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+    gwy_file_set_title(data, GWY_FILE_IMAGE, newid, output_name, TRUE);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, newid);
     g_object_unref(field);
 
     if (itransform)
         return;
 
     /* make fft more visible by choosing a good gradient and using auto range */
-    gwy_container_set_const_string(data, gwy_app_get_data_palette_key_for_id(newid), "DFit");
+    gwy_file_set_palette(data, GWY_FILE_IMAGE, newid, "DFit");
     if (!is_phase)
-        gwy_container_set_enum(data, gwy_app_get_data_range_type_key_for_id(newid), GWY_LAYER_BASIC_RANGE_AUTO);
+        gwy_container_set_enum(GWY_CONTAINER(data), gwy_file_key_image_color_mapping(newid), GWY_COLOR_MAPPING_AUTO);
 }
 
-static GwyDataField*
-make_modulus(GwyDataField *re, GwyDataField *im)
+static GwyField*
+make_modulus(GwyField *re, GwyField *im)
 {
-    GwyDataField *modulus;
+    GwyField *modulus;
     const gdouble *datare, *dataim;
     gdouble *data;
     guint n, i;
 
-    modulus = gwy_data_field_new_alike(re, FALSE);
-    n = gwy_data_field_get_xres(re)*gwy_data_field_get_yres(re);
-    datare = gwy_data_field_get_data_const(re);
-    dataim = gwy_data_field_get_data_const(im);
-    data = gwy_data_field_get_data(modulus);
+    modulus = gwy_field_new_alike(re, FALSE);
+    n = gwy_field_get_xres(re)*gwy_field_get_yres(re);
+    datare = gwy_field_get_data_const(re);
+    dataim = gwy_field_get_data_const(im);
+    data = gwy_field_get_data(modulus);
 #ifdef _OPENMP
 #pragma omp parallel for if (gwy_threads_are_enabled()) default(none) \
             shared(data,datare,dataim,n) \
@@ -268,20 +258,20 @@ make_modulus(GwyDataField *re, GwyDataField *im)
     return modulus;
 }
 
-static GwyDataField*
-make_phase(GwyDataField *re, GwyDataField *im)
+static GwyField*
+make_phase(GwyField *re, GwyField *im)
 {
-    GwyDataField *phase;
+    GwyField *phase;
     const gdouble *datare, *dataim;
     gdouble *data;
     guint n, i;
 
-    phase = gwy_data_field_new_alike(re, FALSE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(phase), NULL);
-    n = gwy_data_field_get_xres(re)*gwy_data_field_get_yres(re);
-    datare = gwy_data_field_get_data_const(re);
-    dataim = gwy_data_field_get_data_const(im);
-    data = gwy_data_field_get_data(phase);
+    phase = gwy_field_new_alike(re, FALSE);
+    gwy_unit_clear(gwy_field_get_unit_z(phase));
+    n = gwy_field_get_xres(re)*gwy_field_get_yres(re);
+    datare = gwy_field_get_data_const(re);
+    dataim = gwy_field_get_data_const(im);
+    data = gwy_field_get_data(phase);
 #ifdef _OPENMP
 #pragma omp parallel for if (gwy_threads_are_enabled()) default(none) \
             shared(data,datare,dataim,n) private(i)
@@ -344,13 +334,17 @@ param_changed(ModuleGUI *gui, gint id)
 }
 
 static gboolean
-imagpart_filter(GwyContainer *data, gint id, gpointer user_data)
+imagpart_filter(GwyFile *data, gint id, gpointer user_data)
 {
-    GwyDataField *imagpart, *field = (GwyDataField*)user_data;
+    GwyField *imagpart, *field = (GwyField*)user_data;
 
-    if (!gwy_container_gis_object(data, gwy_app_get_data_key_for_id(id), &imagpart))
+    if (!(imagpart = gwy_file_get_image(data, id)))
         return FALSE;
-    return !gwy_data_field_check_compatibility(imagpart, field, GWY_DATA_COMPATIBILITY_ALL);
+    return !gwy_field_is_incompatible(imagpart, field,
+                                      GWY_DATA_MISMATCH_RES
+                                      | GWY_DATA_MISMATCH_REAL
+                                      | GWY_DATA_MISMATCH_LATERAL
+                                      | GWY_DATA_MISMATCH_VALUE);
 }
 
 static void
@@ -362,7 +356,7 @@ sanitise_params(ModuleArgs *args)
     gboolean use_imagpart = gwy_params_get_boolean(params, PARAM_USE_IMAG_PART);
 
     if (use_imagpart
-        && (is_none || !imagpart_filter(gwy_app_data_browser_get(imagpart.datano), imagpart.id, args->field)))
+        && (is_none || !imagpart_filter(gwy_data_browser_get_file(imagpart.datano), imagpart.id, args->field)))
         gwy_params_set_boolean(params, PARAM_USE_IMAG_PART, FALSE);
 }
 
