@@ -1,5 +1,5 @@
 /*
- *  $Id: fft_filter_1d.c 26631 2024-09-18 15:01:53Z klapetek $
+ *  $Id: fft_filter_1d.c 29023 2025-12-19 13:06:16Z yeti-dn $
  *  Copyright (C) 2004-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,20 +16,9 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libprocess/inttrans.h>
-#include <libprocess/stats.h>
-#include <libprocess/linestats.h>
-#include <libprocess/correct.h>
-#include <libprocess/arithmetic.h>
-#include <libgwydgets/gwygraph.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_INTERACTIVE)
@@ -54,26 +43,25 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *result;
-    GwyDataLine *modulus;
+    GwyField *field;
+    GwyField *result;
+    GwyLine *modulus;
 } ModuleArgs;
 
 typedef struct {
     ModuleArgs *args;
     GtkWidget *dialog;
     GwyParamTable *table;
-    GwyContainer *data;
     GwyGraphModel *gmodel;
     GwySelection *selection;
 } ModuleGUI;
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             fftf_1d             (GwyContainer *data,
-                                             GwyRunType runtype);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static GwyDialogOutcome run_gui             (ModuleArgs *args,
-                                             GwyContainer *data,
+                                             GwyFile *data,
                                              gint id);
 static void             param_changed       (ModuleGUI *gui,
                                              gint id);
@@ -83,7 +71,7 @@ static void             preview             (gpointer user_data);
 static void             graph_selected      (ModuleGUI *gui);
 static void             ensure_modulus      (ModuleArgs *args);
 static void             plot_modulus        (ModuleGUI *gui);
-static GwyDataLine*     calculate_weights   (ModuleArgs *args,
+static GwyLine*         calculate_weights   (ModuleArgs *args,
                                              GwySelection *selection);
 
 static GwyModuleInfo module_info = {
@@ -102,11 +90,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("fft_filter_1d",
-                              (GwyProcessFunc)&fftf_1d,
+                              module_main,
                               N_("/_Correct Data/1D _FFT Filtering..."),
-                              GWY_STOCK_FFT_FILTER_1D,
+                              GWY_ICON_FFT_FILTER_1D,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("1D FFT Filtering"));
     return TRUE;
 }
@@ -142,20 +130,20 @@ define_module_params(void)
 }
 
 static void
-fftf_1d(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
     GwyDialogOutcome outcome;
     gint oldid, newid;
 
-    g_return_if_fail(runtype & RUN_MODES);
+    g_return_if_fail(mode & RUN_MODES);
 
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_DATA_FIELD_ID, &oldid,
-                                     0);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_FIELD_ID, &oldid,
+                                 0);
     g_return_if_fail(args.field);
 
-    args.result = gwy_data_field_new_alike(args.field, TRUE);
+    args.result = gwy_field_new_alike(args.field, TRUE);
     args.modulus = NULL;
     args.params = gwy_params_new_from_settings(define_module_params());
 
@@ -164,24 +152,25 @@ fftf_1d(GwyContainer *data, GwyRunType runtype)
     if (outcome != GWY_DIALOG_HAVE_RESULT)
         goto end;
 
-    newid = gwy_app_data_browser_add_data_field(args.result, data, TRUE);
-    gwy_app_sync_data_items(data, data, oldid, newid, FALSE,
-                            GWY_DATA_ITEM_GRADIENT,
-                            GWY_DATA_ITEM_RANGE,
-                            0);
-    gwy_app_set_data_field_title(data, newid, _("1D FFT Filtered Data"));
-    gwy_app_channel_log_add_proc(data, oldid, newid);
+    newid = gwy_file_add_image(data, args.result);
+
+    gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+    gwy_file_sync_items(data, GWY_FILE_IMAGE, oldid,
+                        data, GWY_FILE_IMAGE, newid,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_RANGE, FALSE);
+    gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _("1D FFT Filtered Data"), TRUE);
+    gwy_log_add(data, GWY_FILE_IMAGE, oldid, newid);
 
 end:
-    GWY_OBJECT_UNREF(args.modulus);
+    g_clear_object(&args.modulus);
     g_object_unref(args.result);
     g_object_unref(args.params);
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
-    GtkWidget *hbox, *align, *dataview;
+    GtkWidget *hbox, *dataview;
     GwyGraph *graph;
     GwyDialog *dialog;
     GwyParamTable *table;
@@ -189,37 +178,28 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     ModuleGUI gui;
     GwySelection *selection;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
-    gui.data = gwy_container_new();
-    gwy_container_set_object_by_name(gui.data, "/0/data", args->field);
-    gwy_app_sync_data_items(data, gui.data, id, 0, FALSE,
-                            GWY_DATA_ITEM_GRADIENT,
-                            GWY_DATA_ITEM_RANGE,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
-    gwy_container_set_object_by_name(gui.data, "/1/data", args->result);
-    gwy_app_sync_data_items(data, gui.data, id, 1, FALSE,
-                            GWY_DATA_ITEM_GRADIENT,
-                            GWY_DATA_ITEM_RANGE,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
 
     gui.dialog = gwy_dialog_new(_("1D FFT filter"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_UPDATE, GWY_RESPONSE_CLEAR, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    hbox = gwy_hbox_new(0);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
     gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
     gwy_dialog_add_content(dialog, hbox, FALSE, FALSE, 0);
 
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SMALL_SIZE, FALSE);
+    dataview = gwy_create_preview(args->field, NULL, PREVIEW_SMALL_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_RANGE | GWY_FILE_ITEM_REAL_SQUARE);
     gtk_box_pack_start(GTK_BOX(hbox), dataview, FALSE, FALSE, 4);
 
-    dataview = gwy_create_preview(gui.data, 1, PREVIEW_SMALL_SIZE, FALSE);
+    dataview = gwy_create_preview(args->result, NULL, PREVIEW_SMALL_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_RANGE | GWY_FILE_ITEM_REAL_SQUARE);
     gtk_box_pack_start(GTK_BOX(hbox), dataview, FALSE, FALSE, 4);
 
-    hbox = gwy_hbox_new(0);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
     gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
     gwy_dialog_add_content(dialog, hbox, FALSE, FALSE, 0);
 
@@ -242,10 +222,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_param_table_append_combo(table, PARAM_INTERPOLATION);
     gwy_param_table_append_checkbox(table, PARAM_UPDATE);
 
-    align = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-    gtk_container_add(GTK_CONTAINER(align), gwy_param_table_widget(table));
-
-    gtk_box_pack_start(GTK_BOX(hbox), align, FALSE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(hbox), gwy_param_table_widget(table), FALSE, FALSE, 0);
     gwy_dialog_add_param_table(dialog, table);
 
     g_signal_connect_swapped(table, "param-changed", G_CALLBACK(param_changed), &gui);
@@ -255,7 +232,6 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 
     outcome = gwy_dialog_run(dialog);
 
-    g_object_unref(gui.data);
     g_object_unref(gui.gmodel);
 
     return outcome;
@@ -264,8 +240,8 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 static void
 param_changed(ModuleGUI *gui, gint id)
 {
-    GwyParamTable *table = gui->table;
-    GwyParams *params = gui->args->params;
+    //GwyParamTable *table = gui->table;
+    //GwyParams *params = gui->args->params;
 
     /*if (id < 0 || id == PARAM_SUPPRESS) {
         GwyFFTFilt1DSuppressType suppres = gwy_params_get_enum(params, PARAM_SUPPRESS);
@@ -275,7 +251,7 @@ param_changed(ModuleGUI *gui, gint id)
         gwy_param_table_set_sensitive(table, PARAM_OUTPUT, suppres == SUPPRESS_NULL);
     }*/
     if (id < 0 || id == PARAM_DIRECTION) {
-        GWY_OBJECT_UNREF(gui->args->modulus);
+        g_clear_object(&gui->args->modulus);
         gwy_selection_clear(gui->selection);
         ensure_modulus(gui->args);
         plot_modulus(gui);
@@ -298,12 +274,12 @@ preview(gpointer user_data)
     ModuleArgs *args = gui->args;
     GwyOrientation direction = gwy_params_get_enum(args->params, PARAM_DIRECTION);
     GwyInterpolationType interpolation = gwy_params_get_enum(args->params, PARAM_INTERPOLATION);
-    GwyDataLine *weights;
+    GwyLine *weights;
 
     weights = calculate_weights(args, gui->selection);
-    gwy_data_field_fft_filter_1d(args->field, args->result, weights, direction, interpolation);
+    gwy_field_fft_filter_1d(args->field, args->result, weights, direction, interpolation);
 
-    gwy_data_field_data_changed(args->result);
+    gwy_field_data_changed(args->result);
     gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
 }
 
@@ -319,7 +295,7 @@ static void
 ensure_modulus(ModuleArgs *args)
 {
     GwyOrientation direction = gwy_params_get_enum(args->params, PARAM_DIRECTION);
-    GwyDataLine *modulus;
+    GwyLine *modulus;
     gint i, res;
     gdouble m;
     gdouble *d;
@@ -327,14 +303,14 @@ ensure_modulus(ModuleArgs *args)
     if (args->modulus)
         return;
 
-    modulus = args->modulus = gwy_data_line_new(1, 1.0, FALSE);
+    modulus = args->modulus = gwy_line_new(1, 1.0, FALSE);
     /* There is no interpolation.  Pass anything. */
-    gwy_data_field_psdf(args->field, modulus, direction, GWY_INTERPOLATION_LINEAR, GWY_WINDOWING_RECT, -1);
-    m = gwy_data_line_get_max(modulus);
+    gwy_field_psdf(args->field, modulus, direction, GWY_WINDOWING_RECT);
+    m = gwy_line_max(modulus);
     if (!m)
         m = 1.0;
-    d = gwy_data_line_get_data(modulus);
-    res = gwy_data_line_get_res(modulus);
+    d = gwy_line_get_data(modulus);
+    res = gwy_line_get_res(modulus);
     for (i = 0; i < res; i++)
         d[i] = (d[i] > 0.0 ? sqrt(d[i]/m) : 0.0);
 }
@@ -343,18 +319,18 @@ static void
 plot_modulus(ModuleGUI *gui)
 {
     GwyGraphCurveModel *cmodel;
-    GwyDataLine *modulus = gui->args->modulus;
+    GwyLine *modulus = gui->args->modulus;
 
     gwy_graph_model_remove_all_curves(gui->gmodel);
 
     cmodel = gwy_graph_curve_model_new();
-    gwy_graph_curve_model_set_data_from_dataline(cmodel, modulus, 0, 0);
+    gwy_graph_curve_model_set_data_from_line(cmodel, modulus, 0, 0);
     g_object_set(cmodel,
                  "mode", GWY_GRAPH_CURVE_LINE,
                  "description", _("FFT Modulus"),
                  NULL);
     g_object_set(gui->gmodel,
-                 "si-unit-x", gwy_data_line_get_si_unit_x(modulus),
+                 "unit-x", gwy_line_get_unit_x(modulus),
                  "axis-label-bottom", "k",
                  "axis-label-left", "",
                  NULL);
@@ -363,30 +339,32 @@ plot_modulus(ModuleGUI *gui)
     g_object_unref(cmodel);
 }
 
-static GwyDataLine*
+/* XXX: This function is identical among image 1D FFT filter, volume 1D FFT filter and XYZ FFT.
+ * Make a library function! */
+static GwyLine*
 calculate_weights(ModuleArgs *args, GwySelection *selection)
 {
     GwyFFTFilt1DSuppressType suppres = gwy_params_get_enum(args->params, PARAM_SUPPRESS);
     GwyFFTFilt1DOutputType output = gwy_params_get_enum(args->params, PARAM_OUTPUT);
-    GwyDataLine *weights, *modulus = args->modulus;
+    GwyLine *weights, *modulus = args->modulus;
     gint res, k, nsel;
     gdouble sel[2], *w;
     gint fill_from, fill_to;
 
-    res = gwy_data_line_get_res(modulus);
-    weights = gwy_data_line_new_alike(modulus, TRUE);
+    res = gwy_line_get_res(modulus);
+    weights = gwy_line_new_alike(modulus, TRUE);
 
     nsel = gwy_selection_get_data(selection, NULL);
     for (k = 0; k < nsel; k++) {
         gwy_selection_get_object(selection, k, sel);
         GWY_ORDER(gdouble, sel[0], sel[1]);
-        fill_from = MAX(0, gwy_data_line_rtoi(weights, sel[0]));
+        fill_from = MAX(0, gwy_line_rtoi(weights, sel[0]));
         fill_from = MIN(res, fill_from);
-        fill_to = MIN(res, gwy_data_line_rtoi(weights, sel[1]));
-        gwy_data_line_part_fill(weights, fill_from, fill_to, 1.0);
+        fill_to = MIN(res, gwy_line_rtoi(weights, sel[1]));
+        gwy_line_part_fill(weights, fill_from, fill_to - fill_from, 1.0);
     }
 
-    w = gwy_data_line_get_data(weights);
+    w = gwy_line_get_data(weights);
     if (output == OUTPUT_MARKED) {
         for (k = 0; k < res; k++)
             w[k] = 1.0 - w[k];
@@ -395,22 +373,22 @@ calculate_weights(ModuleArgs *args, GwySelection *selection)
     /* For Suppress, interpolate PSDF linearly between endpoints.  Since we pass weights to the filter, not PSDF
      * itself, we have to divide to get the weight. */
     if (suppres == SUPPRESS_NEIGBOURHOOD) {
-        GwyDataLine *buf = gwy_data_line_duplicate(modulus);
+        GwyLine *buf = gwy_line_copy(modulus);
         gdouble *b, *m;
 
         /*invert the weights as the laplace correction treats it as a mask*/
        for (k = 0; k < res; k++)
             w[k] = 1.0 - w[k];
 
-        gwy_data_line_correct_laplace(buf, weights);
-        b = gwy_data_line_get_data(buf);
-        m = gwy_data_line_get_data(modulus);
+        gwy_line_correct_laplace(buf, weights);
+        b = gwy_line_get_data(buf);
+        m = gwy_line_get_data(modulus);
 
         for (k = 0; k < res; k++)
             w[k] = (m[k] > 0.0 ? fmin(b[k]/m[k], 1.0) : 0.0);
         g_object_unref(buf);
     }
-    
+
     return weights;
 }
 
