@@ -1,5 +1,5 @@
 /*
- *  $Id: flatten_base.c 26550 2024-08-18 07:49:41Z yeti-dn $
+ *  $Id: flatten_base.c 28821 2025-11-06 16:00:41Z yeti-dn $
  *  Copyright (C) 2014 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,23 +16,16 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwynlfitpreset.h>
-#include <libprocess/level.h>
-#include <libprocess/stats.h>
-#include <libprocess/grains.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 
-#define FLATTEN_RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
+#define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
 
 static gboolean  module_register     (void);
-static void      flatten_base        (GwyContainer *data,
-                                      GwyRunType run);
+static void      module_main         (GwyFile *data,
+                                      GwyRunModeFlags mode);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
@@ -50,30 +43,30 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("flatten_base",
-                              (GwyProcessFunc)&flatten_base,
+                              module_main,
                               N_("/_Level/Flatten _Base"),
-                              GWY_STOCK_LEVEL_FLATTEN_BASE,
-                              FLATTEN_RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_ICON_LEVEL_FLATTEN_BASE,
+                              RUN_MODES,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Flatten base of surface with positive features"));
 
     return TRUE;
 }
 
 static gboolean
-find_base_peak(GwyDataField *dfield, gdouble *mean, gdouble *rms)
+find_base_peak(GwyField *dfield, gdouble *mean, gdouble *rms)
 {
     GwyNLFitPreset *gaussian;
     GwyNLFitter *fitter;
-    GwyDataLine *dh = gwy_data_line_new(1, 1.0, FALSE);
+    GwyLine *dh = gwy_line_new(1, 1.0, FALSE);
     gdouble *d, *xdata, *ydata;
     gdouble real, off, dhmax = -G_MAXDOUBLE;
     gdouble param[4];
     gint i, res, from, to, ndata, m = 0;
     gboolean retval;
 
-    gwy_data_field_dh(dfield, dh, 0);
-    d = gwy_data_line_get_data(dh);
+    gwy_field_dh(dfield, dh, 0);
+    d = gwy_line_get_data(dh);
     res = dh->res;
     real = dh->real;
     off = dh->off;
@@ -127,7 +120,7 @@ find_base_peak(GwyDataField *dfield, gdouble *mean, gdouble *rms)
 }
 
 static gboolean
-polylevel_with_mask(GwyDataField *dfield, GwyDataField *mask,
+polylevel_with_mask(GwyField *dfield, GwyField *mask,
                     gint max_degree,
                     gdouble mean, gdouble rms)
 {
@@ -137,15 +130,15 @@ polylevel_with_mask(GwyDataField *dfield, GwyDataField *mask,
     gdouble min, max, threshold, threshval;
     gdouble *coeffs;
 
-    gwy_data_field_get_min_max(dfield, &min, &max);
+    gwy_field_get_min_max(dfield, &min, &max);
     if (max <= min)
         return FALSE;
 
     threshold = mean + 3*rms;
     threshval = 100.0*(threshold - min)/(max - min);
     gwy_debug("min %g, max %g, threshold %g => threshval %g", min, max, threshold, threshval);
-    gwy_data_field_grains_mark_height(dfield, mask, threshval, FALSE);
-    gwy_data_field_grains_grow(mask, 1 + max_degree/2, GWY_DISTANCE_TRANSFORM_CONN4, FALSE);
+    gwy_field_grains_mark_height(dfield, mask, threshval, FALSE);
+    gwy_field_grains_grow(mask, 1 + max_degree/2, GWY_DISTANCE_TRANSFORM_CONN4, FALSE);
 
     k = 0;
     for (i = 0; i <= max_degree; i++) {
@@ -155,40 +148,40 @@ polylevel_with_mask(GwyDataField *dfield, GwyDataField *mask,
         }
     }
 
-    coeffs = gwy_data_field_fit_poly(dfield, mask, nterms, term_powers, TRUE, NULL);
-    gwy_data_field_subtract_poly(dfield, nterms, term_powers, coeffs);
+    coeffs = gwy_field_fit_poly(dfield, mask, GWY_MASK_EXCLUDE, nterms, term_powers, NULL);
+    gwy_field_subtract_poly(dfield, nterms, term_powers, coeffs);
     g_free(coeffs);
 
     return TRUE;
 }
 
 static void
-flatten_base(GwyContainer *data, GwyRunType run)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
-    GwyDataField *dfield, *origfield, *mfield = NULL;
+    GwyField *dfield, *origfield, *mfield = NULL;
     GQuark quark;
     gdouble mean, sigma, min, a, bx, by;
     gboolean found_peak;
     gint id, i, totalsteps = 5 + 4, step = 0;
 
-    g_return_if_fail(run & FLATTEN_RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD_KEY, &quark,
-                                     GWY_APP_DATA_FIELD, &origfield,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD_KEY, &quark,
+                                 GWY_APP_FIELD, &origfield,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(origfield && quark);
 
-    gwy_app_wait_start(gwy_app_find_window_for_channel(data, id), _("Facet-leveling..."));
+    gwy_app_wait_start(gwy_data_browser_get_window_for_data(data, GWY_FILE_IMAGE, id), _("Facet-leveling..."));
 
-    dfield = gwy_data_field_duplicate(origfield);
+    dfield = gwy_field_copy(origfield);
     found_peak = find_base_peak(dfield, &mean, &sigma);
     gwy_debug("initial peak: %s (mean=%g, rms=%g)", found_peak ? "OK" : "NOT FOUND", mean, sigma);
 
     for (i = 0; i < 5; i++) {
-        if (!gwy_data_field_fit_facet_plane(dfield, NULL, GWY_MASK_IGNORE, &a, &bx, &by))
+        if (!gwy_field_fit_facet_plane(dfield, NULL, GWY_MASK_IGNORE, &a, &bx, &by))
             break;
 
-        gwy_data_field_plane_level(dfield, a, bx, by);
+        gwy_field_plane_level(dfield, a, bx, by);
         found_peak = find_base_peak(dfield, &mean, &sigma);
         gwy_debug("facet[%d] peak: %s (mean=%g, rms=%g)", i, found_peak ? "OK" : "NOT FOUND", mean, sigma);
         if (!found_peak)
@@ -202,7 +195,7 @@ flatten_base(GwyContainer *data, GwyRunType run)
     if (!gwy_app_wait_set_message(_("Polynomial leveling...")))
         goto cancelled;
 
-    mfield = gwy_data_field_new_alike(dfield, FALSE);
+    mfield = gwy_field_new_alike(dfield, FALSE);
     for (i = 2; i <= 5; i++) {
         polylevel_with_mask(dfield, mfield, i, mean, sigma);
         found_peak = find_base_peak(dfield, &mean, &sigma);
@@ -217,18 +210,18 @@ flatten_base(GwyContainer *data, GwyRunType run)
     gwy_app_wait_finish();
 
     if (found_peak)
-        gwy_data_field_add(dfield, -mean);
+        gwy_field_add(dfield, -mean);
 
-    if ((min = gwy_data_field_get_min(dfield)) > 0.0)
-        gwy_data_field_add(dfield, -min);
+    if ((min = gwy_field_get_min(dfield)) > 0.0)
+        gwy_field_add(dfield, -min);
 
-    gwy_app_undo_qcheckpoint(data, quark, NULL);
-    gwy_data_field_copy(dfield, origfield, FALSE);
-    gwy_app_channel_log_add_proc(data, id, id);
-    gwy_data_field_data_changed(origfield);
+    gwy_app_undo_qcheckpoint(GWY_CONTAINER(data), quark, NULL);
+    gwy_field_copy_data(dfield, origfield);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, id);
+    gwy_field_data_changed(origfield);
 
 cancelled:
-    GWY_OBJECT_UNREF(mfield);
+    g_clear_object(&mfield);
     g_object_unref(dfield);
 }
 
