@@ -1,5 +1,5 @@
 /*
- *  $Id: fractal.c 26283 2024-04-17 13:24:13Z yeti-dn $
+ *  $Id: fractal.c 28860 2025-11-14 12:59:38Z yeti-dn $
  *  Copyright (C) 2003-2024 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,21 +16,9 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyresults.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libprocess/fractals.h>
-#include <libprocess/stats.h>
-#include <libgwydgets/gwygraph.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwydgets/gwycombobox.h>
-#include <libgwydgets/gwydgetutils.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES GWY_RUN_INTERACTIVE
@@ -44,12 +32,12 @@ typedef enum {
     FRACTAL_NMETHODS
 } FractalMethodType;
 
-typedef void (*FractalMethodFunc)(GwyDataField *data_field,
-                                  GwyDataLine *xresult,
-                                  GwyDataLine *yresult,
+typedef void (*FractalMethodFunc)(GwyField *field,
+                                  GwyLine *xresult,
+                                  GwyLine *yresult,
                                   GwyInterpolationType interpolation);
-typedef gdouble (*FractalDimFunc)(GwyDataLine *xresult,
-                                  GwyDataLine *yresult,
+typedef gdouble (*FractalDimFunc)(GwyLine *xresult,
+                                  GwyLine *yresult,
                                   gdouble *a,
                                   gdouble *b);
 
@@ -80,11 +68,11 @@ typedef struct {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
+    GwyField *field;
     GwyGraphModel *gmodel;
     FractalMethodComp comp[FRACTAL_NMETHODS];
-    GwyDataLine *xline;
-    GwyDataLine *yline;
+    GwyLine *xline;
+    GwyLine *yline;
 } ModuleArgs;
 
 typedef struct {
@@ -97,13 +85,13 @@ typedef struct {
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             fractal             (GwyContainer *data,
-                                             GwyRunType runtype);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static gboolean         execute             (ModuleArgs *args);
 static GwyDialogOutcome run_gui             (ModuleArgs *args,
-                                             GwyContainer *data,
+                                             GwyFile *data,
                                              gint id);
-static GwyResults*      create_results      (GwyContainer *data,
+static GwyResults*      create_results      (GwyFile *data,
                                              gint id);
 static void             param_changed       (ModuleGUI *gui,
                                              gint id);
@@ -111,42 +99,42 @@ static void             preview             (gpointer user_data);
 static void             graph_selected      (ModuleGUI *gui,
                                              gint hint,
                                              GwySelection *selection);
-static gboolean         remove_datapoints   (GwyDataLine *xline,
-                                             GwyDataLine *yline,
-                                             GwyDataLine *newxline,
-                                             GwyDataLine *newyline,
+static gboolean         remove_datapoints   (GwyLine *xline,
+                                             GwyLine *yline,
+                                             GwyLine *newxline,
+                                             GwyLine *newyline,
                                              const gdouble *fromto);
 
 static inline void
-gwy_data_field_fractal_ihhcf(GwyDataField *data_field,
-                             GwyDataLine *xresult,
-                             GwyDataLine *yresult,
-                             G_GNUC_UNUSED GwyInterpolationType interpolation)
+gwy_field_fractal_ihhcf(GwyField *field,
+                        GwyLine *xresult,
+                        GwyLine *yresult,
+                        G_GNUC_UNUSED GwyInterpolationType interpolation)
 {
-    return gwy_data_field_fractal_hhcf(data_field, xresult, yresult);
+    return gwy_field_fractal_hhcf(field, xresult, yresult);
 }
 
 /* Indexed directly by FractalMethodType.  Change GUI order using gwy_enum_fill_from_struct() if necessary. */
 static const FractalMethodInfo methods[FRACTAL_NMETHODS] = {
     {
         "partitioning", N_("Partitioning"), "log h", "log S",
-        gwy_data_field_fractal_partitioning, gwy_data_field_fractal_partitioning_dim,
+        gwy_field_fractal_partitioning, gwy_field_fractal_partitioning_dim,
     },
     {
         "cubecounting", N_("Cube counting"), "log h", "log N",
-        gwy_data_field_fractal_cubecounting, gwy_data_field_fractal_cubecounting_dim,
+        gwy_field_fractal_cubecounting, gwy_field_fractal_cubecounting_dim,
     },
     {
         "triangulation", N_("Triangulation"), "log h", "log A",
-        gwy_data_field_fractal_triangulation, gwy_data_field_fractal_triangulation_dim,
+        gwy_field_fractal_triangulation, gwy_field_fractal_triangulation_dim,
     },
     {
         "psdf", N_("Power spectrum"), "log k", "log W",
-        gwy_data_field_fractal_psdf, gwy_data_field_fractal_psdf_dim,
+        gwy_field_fractal_psdf, gwy_field_fractal_psdf_dim,
     },
     {
         "hhcf", N_("Structure function"), "log h", "log H",
-        gwy_data_field_fractal_ihhcf, gwy_data_field_fractal_hhcf_dim,
+        gwy_field_fractal_ihhcf, gwy_field_fractal_hhcf_dim,
     },
 };
 
@@ -167,11 +155,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("fractal",
-                              (GwyProcessFunc)&fractal,
+                              module_main,
                               N_("/_Statistics/_Fractal Dimension..."),
-                              GWY_STOCK_FRACTAL_MEASURE,
+                              GWY_ICON_FRACTAL_MEASURE,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Calculate fractal dimension"));
 
     return TRUE;
@@ -202,18 +190,18 @@ define_module_params(void)
 }
 
 static void
-fractal(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome;
     GwyAppDataId target_graph_id;
     ModuleArgs args;
     gint id;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     args.params = gwy_params_new_from_settings(define_module_params());
     args.gmodel = gwy_graph_model_new();
     outcome = run_gui(&args, data, id);
@@ -227,14 +215,14 @@ fractal(GwyContainer *data, GwyRunType runtype)
     gwy_app_add_graph_or_curves(args.gmodel, data, &target_graph_id, 2);
 
 end:
-    GWY_OBJECT_UNREF(args.xline);
-    GWY_OBJECT_UNREF(args.yline);
+    g_clear_object(&args.xline);
+    g_clear_object(&args.yline);
     g_object_unref(args.params);
     g_object_unref(args.gmodel);
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     GtkWidget *hbox, *graph;
     GwyDialog *dialog;
@@ -245,7 +233,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     const gchar *values[FRACTAL_NMETHODS];
     guint i;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
     gui.results = create_results(data, id);
 
@@ -261,7 +249,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    hbox = gwy_hbox_new(0);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
     gwy_dialog_add_content(dialog, hbox, FALSE, FALSE, 0);
 
     table = gui.table = gwy_param_table_new(args->params);
@@ -303,7 +291,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 }
 
 static GwyResults*
-create_results(GwyContainer *data, gint id)
+create_results(GwyFile *data, gint id)
 {
     GwyResults *results = gwy_results_new();
     guint i;
@@ -313,10 +301,10 @@ create_results(GwyContainer *data, gint id)
     gwy_results_add_value_str(results, "image", N_("Image"));
     gwy_results_add_separator(results);
     for (i = 0; i < FRACTAL_NMETHODS; i++)
-        gwy_results_add_value_plain(results, methods[i].id, gwy_sgettext(methods[i].name));
+        gwy_results_add_value_plain(results, methods[i].id, gwy_C(methods[i].name));
 
     gwy_results_fill_filename(results, "file", data);
-    gwy_results_fill_channel(results, "image", data, id);
+    gwy_results_fill_data_name(results, "image", data, GWY_FILE_IMAGE, id);
 
     return results;
 }
@@ -338,8 +326,8 @@ param_changed(ModuleGUI *gui, gint id)
     if (id < 0 || id == PARAM_METHOD || id == PARAM_INTERP) {
         /* These only change when the method or interpolation changes.  But, cruciually, not when the graph
          * selection changes. */
-        GWY_OBJECT_UNREF(args->xline);
-        GWY_OBJECT_UNREF(args->yline);
+        g_clear_object(&args->xline);
+        g_clear_object(&args->yline);
         gwy_dialog_invalidate(GWY_DIALOG(gui->dialog));
     }
 }
@@ -364,7 +352,7 @@ preview(gpointer user_data)
 static gboolean
 execute(ModuleArgs *args)
 {
-    GwyDataLine *yfit, *xnline, *ynline;
+    GwyLine *yfit, *xnline, *ynline;
     FractalMethodType method = gwy_params_get_enum(args->params, PARAM_METHOD);
     GwyInterpolationType interpolation = gwy_params_get_enum(args->params, PARAM_INTERP);
     FractalMethodComp *comp = args->comp + method;
@@ -375,35 +363,35 @@ execute(ModuleArgs *args)
     gboolean line_ok;
     gint i, res;
 
-    xnline = gwy_data_line_new(1, 1.0, FALSE);
-    ynline = gwy_data_line_new(1, 1.0, FALSE);
+    xnline = gwy_line_new(1, 1.0, FALSE);
+    ynline = gwy_line_new(1, 1.0, FALSE);
 
     if (!args->xline) {
-        args->xline = gwy_data_line_new(1, 1.0, FALSE);
-        args->yline = gwy_data_line_new(1, 1.0, FALSE);
+        args->xline = gwy_line_new(1, 1.0, FALSE);
+        args->yline = gwy_line_new(1, 1.0, FALSE);
         methods[method].calculate(args->field, args->xline, args->yline, interpolation);
     }
     if ((line_ok = remove_datapoints(args->xline, args->yline, xnline, ynline, comp->fromto)))
         comp->dim = methods[method].getdim(xnline, ynline, &a, &b);
 
     g_object_set(gmodel,
-                 "title", gwy_sgettext(methods[method].name),
+                 "title", gwy_C(methods[method].name),
                  "axis-label-bottom", methods[method].abscissa,
                  "axis-label-left", methods[method].ordinate,
                  NULL);
 
     gcmodel = gwy_graph_model_get_curve(gmodel, 0);
-    g_object_set(gcmodel, "description", gwy_sgettext(methods[method].name), NULL);
+    g_object_set(gcmodel, "description", gwy_C(methods[method].name), NULL);
     gwy_graph_curve_model_set_data(gcmodel,
-                                   gwy_data_line_get_data_const(args->xline),
-                                   gwy_data_line_get_data_const(args->yline),
-                                   gwy_data_line_get_res(args->xline));
+                                   gwy_line_get_data_const(args->xline),
+                                   gwy_line_get_data_const(args->yline),
+                                   gwy_line_get_res(args->xline));
 
-    res = gwy_data_line_get_res(xnline);
+    res = gwy_line_get_res(xnline);
     if (line_ok) {
-        yfit = gwy_data_line_new(res, res, FALSE);
-        xdata = gwy_data_line_get_data(xnline);
-        ydata = gwy_data_line_get_data(yfit);
+        yfit = gwy_line_new(res, res, FALSE);
+        xdata = gwy_line_get_data(xnline);
+        ydata = gwy_line_get_data(yfit);
         for (i = 0; i < res; i++)
             ydata[i] = xdata[i]*a + b;
 
@@ -467,29 +455,29 @@ graph_selected(ModuleGUI *gui, G_GNUC_UNUSED gint hint, GwySelection *selection)
 /* Remove datapoints that are below or above selection. New data are in newxline and newyline and can be directly used
  * for fitting and fractal dimension evaluation. */
 static gboolean
-remove_datapoints(GwyDataLine *xline, GwyDataLine *yline,
-                  GwyDataLine *newxline, GwyDataLine *newyline,
+remove_datapoints(GwyLine *xline, GwyLine *yline,
+                  GwyLine *newxline, GwyLine *newyline,
                   const gdouble *fromto)
 {
     const gdouble *xdata, *ydata;
     gdouble *newxdata, *newydata;
     gint i, j, res;
 
-    res = gwy_data_line_get_res(xline);
-    g_assert(res == gwy_data_line_get_res(yline));
-    gwy_data_line_resample(newxline, res, GWY_INTERPOLATION_NONE);
-    gwy_data_line_resample(newyline, res, GWY_INTERPOLATION_NONE);
+    res = gwy_line_get_res(xline);
+    g_assert(res == gwy_line_get_res(yline));
+    gwy_line_resize(newxline, res);
+    gwy_line_resize(newyline, res);
     if (fromto[0] == fromto[1]) {
-        gwy_data_line_copy(xline, newxline);
-        gwy_data_line_copy(yline, newyline);
+        gwy_line_copy_data(xline, newxline);
+        gwy_line_copy_data(yline, newyline);
         return res >= 2;
     }
 
     j = 0;
-    xdata = gwy_data_line_get_data_const(xline);
-    ydata = gwy_data_line_get_data_const(yline);
-    newxdata = gwy_data_line_get_data(newxline);
-    newydata = gwy_data_line_get_data(newyline);
+    xdata = gwy_line_get_data_const(xline);
+    ydata = gwy_line_get_data_const(yline);
+    newxdata = gwy_line_get_data(newxline);
+    newydata = gwy_line_get_data(newyline);
     for (i = 0; i < res; i++) {
         if (xdata[i] >= fromto[0] && xdata[i] <= fromto[1]) {
             newxdata[j] = xdata[i];
@@ -500,8 +488,8 @@ remove_datapoints(GwyDataLine *xline, GwyDataLine *yline,
     if (j < 2)
         return FALSE;
 
-    gwy_data_line_resize(newxline, 0, j);
-    gwy_data_line_resize(newyline, 0, j);
+    gwy_line_crop(newxline, 0, j);
+    gwy_line_crop(newyline, 0, j);
 
     return TRUE;
 }
