@@ -1,5 +1,5 @@
 /*
- *  $Id: freq_split.c 27314 2025-02-20 16:54:22Z yeti-dn $
+ *  $Id: freq_split.c 29023 2025-12-19 13:06:16Z yeti-dn $
  *  Copyright (C) 2018-2024 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,19 +16,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwythreads.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/inttrans.h>
-#include <libprocess/stats.h>
-#include <libprocess/linestats.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
@@ -64,12 +55,12 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *highpass;
-    GwyDataField *lowpass;
+    GwyField *field;
+    GwyField *highpass;
+    GwyField *lowpass;
     /* Cached expensive intermediate calculation. */
-    GwyDataField *extfftre;
-    GwyDataField *extfftim;
+    GwyField *extfftre;
+    GwyField *extfftim;
     gint leftext;
     gint topext;
 } ModuleArgs;
@@ -77,27 +68,27 @@ typedef struct {
 typedef struct {
     ModuleArgs *args;
     GtkWidget *dialog;
+    GtkWidget *dataview;
     GwyParamTable *table;
-    GwyContainer *data;
     GwyGraphModel *gmodel;
-    GwySIValueFormat *vf;
+    GwyValueFormat *vf;
 } ModuleGUI;
 
 static gboolean         module_register      (void);
 static GwyParamDef*     define_module_params (void);
-static void             freq_split           (GwyContainer *data,
-                                              GwyRunType runtype);
+static void             module_main          (GwyFile *data,
+                                              GwyRunModeFlags mode);
 static void             execute              (ModuleArgs *args);
 static GwyDialogOutcome run_gui              (ModuleArgs *args,
-                                              GwyContainer *data,
+                                              GwyFile *data,
                                               gint id);
 static void             param_changed        (ModuleGUI *gui,
                                               gint id);
 static void             preview              (gpointer user_data);
 static void             update_real_frequency(ModuleGUI *gui);
-static void             extend_and_fft       (GwyDataField *field,
-                                              GwyDataField **extfftre,
-                                              GwyDataField **extfftim,
+static void             extend_and_fft       (GwyField *field,
+                                              GwyField **extfftre,
+                                              GwyField **extfftim,
                                               gint *leftext,
                                               gint *topext,
                                               FSplitBoundaryType boundary);
@@ -109,7 +100,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Splits image into low and high frequency components."),
     "Yeti <yeti@gwyddion.net>",
-    "2.2",
+    "2.1",
     "David Nečas (Yeti)",
     "2018",
 };
@@ -120,11 +111,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("freq_split",
-                              (GwyProcessFunc)&freq_split,
+                              module_main,
                               N_("/_Level/_Frequency Split..."),
-                              GWY_STOCK_FREQUENCY_SPLIT,
+                              GWY_ICON_FREQUENCY_SPLIT,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Split into low and high frequencies"));
 
     return TRUE;
@@ -134,7 +125,7 @@ static GwyParamDef*
 define_module_params(void)
 {
     static const GwyEnum boundaries[] = {
-        { N_("boundary-handling|None"), FSPLIT_BOUNDARY_NONE,    },
+        { gwy_NC("boundary-handling", "None"), FSPLIT_BOUNDARY_NONE,    },
         { N_("Laplace"),                FSPLIT_BOUNDARY_LAPLACE, },
         { N_("Smooth connect"),         FSPLIT_BOUNDARY_SMCONN,  },
         { N_("Mirror"),                 FSPLIT_BOUNDARY_MIRROR,  },
@@ -161,14 +152,14 @@ define_module_params(void)
                               boundaries, G_N_ELEMENTS(boundaries), FSPLIT_BOUNDARY_NONE);
     gwy_param_def_add_gwyflags(paramdef, PARAM_OUTPUT, "output", _("Output type"),
                                outputs, G_N_ELEMENTS(outputs), FSPLIT_OUTPUT_BOTH);
-    gwy_param_def_add_gwyenum(paramdef, PARAM_PREVIEW, "preview", gwy_sgettext("verb|Display"),
+    gwy_param_def_add_gwyenum(paramdef, PARAM_PREVIEW, "preview", C_("verb", "Display"),
                               previews, G_N_ELEMENTS(previews), FSPLIT_PREVIEW_HIGH_PASS);
     gwy_param_def_add_instant_updates(paramdef, PARAM_UPDATE, "update", NULL, TRUE);
     return paramdef;
 }
 
 static void
-freq_split(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     FSplitOutputType output;
@@ -176,17 +167,17 @@ freq_split(GwyContainer *data, GwyRunType runtype)
     GwyParams *params;
     gint id, newid;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(args.field);
-    args.highpass = gwy_data_field_new_alike(args.field, TRUE);
-    args.lowpass = gwy_data_field_new_alike(args.field, TRUE);
+    args.highpass = gwy_field_new_alike(args.field, TRUE);
+    args.lowpass = gwy_field_new_alike(args.field, TRUE);
     args.params = params = gwy_params_new_from_settings(define_module_params());
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, id);
         gwy_params_save_to_settings(params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -198,35 +189,35 @@ freq_split(GwyContainer *data, GwyRunType runtype)
     output = gwy_params_get_flags(params, PARAM_OUTPUT);
 
     if (output & FSPLIT_OUTPUT_LOW_PASS) {
-        newid = gwy_app_data_browser_add_data_field(args.lowpass, data, TRUE);
-        gwy_app_sync_data_items(data, data, id, newid, FALSE,
-                                GWY_DATA_ITEM_GRADIENT,
-                                GWY_DATA_ITEM_REAL_SQUARE,
-                                0);
-        gwy_app_set_data_field_title(data, newid, _("Low-pass"));
-        gwy_app_channel_log_add_proc(data, id, newid);
+        newid = gwy_file_add_image(data, args.lowpass);
+        gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+        gwy_file_sync_items(data, GWY_FILE_IMAGE, id,
+                            data, GWY_FILE_IMAGE, newid,
+                            GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_REAL_SQUARE, FALSE);
+        gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _("Low-pass"), TRUE);
+        gwy_log_add(data, GWY_FILE_IMAGE, id, newid);
     }
 
     if (output & FSPLIT_OUTPUT_HIGH_PASS) {
-        newid = gwy_app_data_browser_add_data_field(args.highpass, data, TRUE);
-        gwy_app_sync_data_items(data, data, id, newid, FALSE,
-                                GWY_DATA_ITEM_GRADIENT,
-                                GWY_DATA_ITEM_REAL_SQUARE,
-                                0);
-        gwy_app_set_data_field_title(data, newid, _("High-pass"));
-        gwy_app_channel_log_add_proc(data, id, newid);
+        newid = gwy_file_add_image(data, args.highpass);
+        gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+        gwy_file_sync_items(data, GWY_FILE_IMAGE, id,
+                            data, GWY_FILE_IMAGE, newid,
+                            GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_REAL_SQUARE, FALSE);
+        gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _("High-pass"), TRUE);
+        gwy_log_add(data, GWY_FILE_IMAGE, id, newid);
     }
 
 end:
-    GWY_OBJECT_UNREF(args.extfftre);
-    GWY_OBJECT_UNREF(args.extfftim);
+    g_clear_object(&args.extfftre);
+    g_clear_object(&args.extfftim);
     g_object_unref(args.params);
     g_object_unref(args.highpass);
     g_object_unref(args.lowpass);
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     GtkWidget *hbox, *vbox, *dataview;
     GwyGraph *graph;
@@ -235,25 +226,20 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     GwyDialog *dialog;
     ModuleGUI gui;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
-    gui.data = gwy_container_new();
-    gwy_container_set_object_by_name(gui.data, "/0/data", args->field);
-    gwy_app_sync_data_items(data, gui.data, id, 0, FALSE,
-                            GWY_DATA_ITEM_PALETTE,
-                            GWY_DATA_ITEM_RANGE,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
-    gui.vf = gwy_data_field_get_value_format_xy(args->field, GWY_SI_UNIT_FORMAT_VFMARKUP, NULL);
+    gui.vf = gwy_field_get_value_format_xy(args->field, GWY_UNIT_FORMAT_VFMARKUP, NULL);
 
     gui.dialog = gwy_dialog_new(_("Frequency Split"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_UPDATE, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
+    dataview = gui.dataview = gwy_create_preview(args->field, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_RANGE | GWY_FILE_ITEM_REAL_SQUARE);
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), FALSE);
 
-    vbox = gwy_vbox_new(4);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
     gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);
 
     gui.gmodel = gwy_graph_model_new();
@@ -264,8 +250,9 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_graph_set_axis_visible(graph, GTK_POS_TOP, FALSE);
     gwy_graph_set_axis_visible(graph, GTK_POS_BOTTOM, FALSE);
     gwy_graph_enable_user_input(graph, FALSE);
-    gwy_graph_area_enable_user_input(GWY_GRAPH_AREA(gwy_graph_get_area(graph)), FALSE);
-    gtk_widget_set_size_request(GTK_WIDGET(graph), -1, 120);
+    GwyGraphArea *area = GWY_GRAPH_AREA(gwy_graph_get_area(graph));
+    gwy_graph_area_enable_user_input(area, FALSE);
+    gwy_graph_area_set_natural_size(area, 2*PREVIEW_SIZE/3, PREVIEW_SIZE/4);
     gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(graph), TRUE, TRUE, 0);
 
     table = gui.table = gwy_param_table_new(args->params);
@@ -291,7 +278,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 
     outcome = gwy_dialog_run(dialog);
 
-    gwy_si_unit_value_format_free(gui.vf);
+    gwy_value_format_free(gui.vf);
     g_object_unref(gui.gmodel);
 
     return outcome;
@@ -312,21 +299,21 @@ param_changed(ModuleGUI *gui, gint id)
         /* Recalculate everything when
          * (a) We must because the boundary treatment has changed.
          * (b) We are currently recalculating nothing because original data are shown. */
-        GWY_OBJECT_UNREF(args->extfftre);
-        GWY_OBJECT_UNREF(args->extfftim);
+        g_clear_object(&args->extfftre);
+        g_clear_object(&args->extfftim);
     }
 
     if (id < 0 || id == PARAM_BOUNDARY || filter_changed)
         gwy_dialog_invalidate(GWY_DIALOG(gui->dialog));
 
     if (id < 0 || id == PARAM_PREVIEW) {
-        GQuark quark = gwy_app_get_data_key_for_id(0);
+        GwyDataView *dataview = GWY_DATA_VIEW(gui->dataview);
         if (display == FSPLIT_PREVIEW_LOW_PASS)
-            gwy_container_set_object(gui->data, quark, args->lowpass);
+            gwy_data_view_set_field(dataview, args->lowpass);
         else if (display == FSPLIT_PREVIEW_HIGH_PASS)
-            gwy_container_set_object(gui->data, quark, args->highpass);
+            gwy_data_view_set_field(dataview, args->highpass);
         else
-            gwy_container_set_object(gui->data, quark, args->field);
+            gwy_data_view_set_field(dataview, args->field);
     }
 
     if (id < 0 || filter_changed)
@@ -337,7 +324,7 @@ static void
 update_real_frequency(ModuleGUI *gui)
 {
     GwyParamTable *table = gui->table;
-    GwySIValueFormat *vf = gui->vf;
+    GwyValueFormat *vf = gui->vf;
     gdouble dx, v = gwy_params_get_double(gui->args->params, PARAM_CUTOFF);
     gchar *s;
 
@@ -345,7 +332,7 @@ update_real_frequency(ModuleGUI *gui)
         gwy_param_table_info_set_valuestr(table, INFO_WAVELENGTH, "∞");
         return;
     }
-    dx = gwy_data_field_get_dx(gui->args->field);
+    dx = gwy_field_get_dx(gui->args->field);
     v = 2.0*dx/v;
     s = g_strdup_printf("%.*f", vf->precision+1, v/vf->magnitude);
     gwy_param_table_info_set_valuestr(table, INFO_WAVELENGTH, s);
@@ -359,8 +346,8 @@ preview(gpointer user_data)
     ModuleArgs *args = gui->args;
 
     execute(args);
-    gwy_data_field_data_changed(args->highpass);
-    gwy_data_field_data_changed(args->lowpass);
+    gwy_field_data_changed(args->highpass);
+    gwy_field_data_changed(args->lowpass);
     gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
 }
 
@@ -407,38 +394,38 @@ extend_one_row(const gdouble *row, guint n, gdouble *extrow, guint next)
     }
 }
 
-static GwyDataField*
-extend_data_field_smconn(GwyDataField *field)
+static GwyField*
+extend_field_smconn(GwyField *field)
 {
-    GwyDataField *extfield, *flipped;
-    gdouble dx = gwy_data_field_get_dx(field), dy = gwy_data_field_get_dy(field);
+    GwyField *extfield, *flipped;
+    gdouble dx = gwy_field_get_dx(field), dy = gwy_field_get_dy(field);
     gint xres, yres, extxres, extyres;
     const gdouble *data;
     gdouble *extdata, *buf;
     gint i, j;
 
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
     extxres = gwy_fft_find_nice_size(4*xres/3);
     extyres = gwy_fft_find_nice_size(4*yres/3);
     if (extxres >= 3*xres || extyres >= 3*extyres) {
         /* This is a silly case.  We just do not want to hit the assertion in extend_one_row(). */
-        return gwy_data_field_extend(field, 0, 0, extxres - xres, extyres - yres,
-                                     GWY_EXTERIOR_FIXED_VALUE, gwy_data_field_get_avg(field), FALSE);
+        return gwy_field_extend(field, 0, 0, extxres - xres, extyres - yres,
+                                GWY_EXTERIOR_FIXED_VALUE, gwy_field_get_avg(field), FALSE);
     }
 
-    extfield = gwy_data_field_new(extxres, extyres, extxres*dx, extyres*dy, FALSE);
-    flipped = gwy_data_field_new(extyres, extxres, extyres*dx, extxres*dy, FALSE);
-    data = gwy_data_field_get_data_const(field);
+    extfield = gwy_field_new(extxres, extyres, extxres*dx, extyres*dy, FALSE);
+    flipped = gwy_field_new(extyres, extxres, extyres*dx, extxres*dy, FALSE);
+    data = gwy_field_get_data_const(field);
 
     /* Extend rows horizontally. */
-    extdata = gwy_data_field_get_data(extfield);
+    extdata = gwy_field_get_data(extfield);
     for (i = 0; i < yres; i++)
         extend_one_row(data + i*xres, xres, extdata + i*extxres, extxres);
 
     /* Extend columns, including the newly created ones. */
-    gwy_data_field_flip_xy(extfield, flipped, FALSE);
-    extdata = gwy_data_field_get_data(flipped);
+    gwy_field_transpose(extfield, flipped, FALSE);
+    extdata = gwy_field_get_data(flipped);
     buf = g_new(gdouble, extyres);
     for (i = 0; i < extxres; i++) {
         extend_one_row(extdata + i*extyres, yres, buf, extyres);
@@ -447,9 +434,9 @@ extend_data_field_smconn(GwyDataField *field)
 
     /* Copy it back, extend the remaining rows and use the average to fill
      * the area unreachable by a single extension. */
-    gwy_data_field_flip_xy(flipped, extfield, FALSE);
+    gwy_field_transpose(flipped, extfield, FALSE);
     g_object_unref(flipped);
-    extdata = gwy_data_field_get_data(extfield);
+    extdata = gwy_field_get_data(extfield);
     buf = g_renew(gdouble, buf, extxres);
     for (i = yres; i < extyres; i++) {
         extend_one_row(extdata + i*extxres, xres, buf, extxres);
@@ -460,22 +447,22 @@ extend_data_field_smconn(GwyDataField *field)
     return extfield;
 }
 
-static GwyDataField*
-extend_data_field_mirror(GwyDataField *field)
+static GwyField*
+extend_field_mirror(GwyField *field)
 {
-    GwyDataField *extfield;
+    GwyField *extfield;
     gint extxres, extyres, xres, yres, i, j;
-    gdouble dx = gwy_data_field_get_dx(field), dy = gwy_data_field_get_dy(field);
+    gdouble dx = gwy_field_get_dx(field), dy = gwy_field_get_dy(field);
     const gdouble *data, *srow;
     gdouble *extdata, *trow;
 
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
     extxres = 2*xres;
     extyres = 2*yres;
-    extfield = gwy_data_field_new(extxres, extyres, extxres*dx, extyres*dy, FALSE);
-    data = gwy_data_field_get_data_const(field);
-    extdata = gwy_data_field_get_data(extfield);
+    extfield = gwy_field_new(extxres, extyres, extxres*dx, extyres*dy, FALSE);
+    data = gwy_field_get_data_const(field);
+    extdata = gwy_field_get_data(extfield);
 
     for (i = 0; i < yres; i++) {
         srow = data + i*xres;
@@ -493,50 +480,50 @@ extend_data_field_mirror(GwyDataField *field)
 }
 
 static void
-extend_and_fft(GwyDataField *field, GwyDataField **extfftre, GwyDataField **extfftim,
+extend_and_fft(GwyField *field, GwyField **extfftre, GwyField **extfftim,
                gint *leftext, gint *topext, FSplitBoundaryType boundary)
 {
-    GwyDataField *extfield;
+    GwyField *extfield;
     gint xres, yres, xext, yext;
 
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
 
     *leftext = *topext = 0;
     if (boundary == FSPLIT_BOUNDARY_LAPLACE) {
         xext = gwy_fft_find_nice_size(5*xres/3);
         yext = gwy_fft_find_nice_size(5*yres/3);
-        extfield = gwy_data_field_extend(field, xext/2, xext - xext/2, yext/2, yext - yext/2,
-                                         GWY_EXTERIOR_LAPLACE, 0.0, FALSE);
+        extfield = gwy_field_extend(field, xext/2, xext - xext/2, yext/2, yext - yext/2,
+                                    GWY_EXTERIOR_LAPLACE, 0.0, FALSE);
         *leftext = xext/2;
         *topext = yext/2;
     }
     else if (boundary == FSPLIT_BOUNDARY_SMCONN) {
         /* The extension is asymmetrical, just to the right and bottom. */
-        extfield = extend_data_field_smconn(field);
+        extfield = extend_field_smconn(field);
     }
     else if (boundary == FSPLIT_BOUNDARY_MIRROR) {
         /* The extension is asymmetrical, just to the right and bottom. */
-        extfield = extend_data_field_mirror(field);
+        extfield = extend_field_mirror(field);
     }
     else {
         extfield = g_object_ref(field);
     }
 
-    *extfftre = gwy_data_field_new_alike(extfield, FALSE);
-    *extfftim = gwy_data_field_new_alike(extfield, FALSE);
-    gwy_data_field_2dfft_raw(extfield, NULL, *extfftre, *extfftim, GWY_TRANSFORM_DIRECTION_FORWARD);
+    *extfftre = gwy_field_new_alike(extfield, FALSE);
+    *extfftim = gwy_field_new_alike(extfield, FALSE);
+    gwy_field_fft_2d_raw(extfield, NULL, *extfftre, *extfftim, GWY_TRANSFORM_DIRECTION_FORWARD);
     g_object_unref(extfield);
 }
 
 static void
-filter_frequencies(GwyDataField *refield, GwyDataField *imfield,
+filter_frequencies(GwyField *refield, GwyField *imfield,
                    gdouble cutoff, gdouble width)
 {
-    gint xres = gwy_data_field_get_xres(refield);
-    gint yres = gwy_data_field_get_yres(refield);
-    gdouble *re = gwy_data_field_get_data(refield);
-    gdouble *im = gwy_data_field_get_data(imfield);
+    gint xres = gwy_field_get_xres(refield);
+    gint yres = gwy_field_get_yres(refield);
+    gdouble *re = gwy_field_get_data(refield);
+    gdouble *im = gwy_field_get_data(imfield);
     gint i, j;
 
 #ifdef _OPENMP
@@ -565,8 +552,8 @@ static void
 execute(ModuleArgs *args)
 {
     GwyParams *params = args->params;
-    GwyDataField *field = args->field, *lowpass = args->lowpass, *highpass = args->highpass;
-    GwyDataField *tmpre, *tmpim, *fre, *fim;
+    GwyField *field = args->field, *lowpass = args->lowpass, *highpass = args->highpass;
+    GwyField *tmpre, *tmpim, *fre, *fim;
     FSplitBoundaryType boundary = gwy_params_get_enum(params, PARAM_BOUNDARY);
     gdouble cutoff = gwy_params_get_double(params, PARAM_CUTOFF);
     gdouble width = gwy_params_get_double(params, PARAM_WIDTH);
@@ -574,39 +561,39 @@ execute(ModuleArgs *args)
 
     if (!args->extfftre)
         extend_and_fft(args->field, &args->extfftre, &args->extfftim, &args->leftext, &args->topext, boundary);
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
-    tmpre = gwy_data_field_new_alike(args->extfftre, FALSE);
-    tmpim = gwy_data_field_new_alike(args->extfftre, FALSE);
-    fre = gwy_data_field_duplicate(args->extfftre);
-    fim = gwy_data_field_duplicate(args->extfftim);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
+    tmpre = gwy_field_new_alike(args->extfftre, FALSE);
+    tmpim = gwy_field_new_alike(args->extfftre, FALSE);
+    fre = gwy_field_copy(args->extfftre);
+    fim = gwy_field_copy(args->extfftim);
     filter_frequencies(fre, fim, cutoff, width);
-    gwy_data_field_2dfft_raw(fre, fim, tmpre, tmpim, GWY_TRANSFORM_DIRECTION_BACKWARD);
+    gwy_field_fft_2d_raw(fre, fim, tmpre, tmpim, GWY_TRANSFORM_DIRECTION_BACKWARD);
     g_object_unref(tmpim);
     g_object_unref(fre);
     g_object_unref(fim);
-    gwy_data_field_area_copy(tmpre, highpass, args->leftext, args->topext, xres, yres, 0, 0);
+    gwy_field_area_copy(tmpre, highpass, args->leftext, args->topext, xres, yres, 0, 0);
     g_object_unref(tmpre);
-    gwy_data_field_subtract_fields(lowpass, field, highpass);
+    gwy_field_subtract_fields(lowpass, field, highpass);
 }
 
 static void
 make_psdf_curve(ModuleGUI *gui)
 {
-    GwyDataField *field = gui->args->field;
-    GwyDataLine *rpsdf = gwy_data_line_new(1, 1, FALSE);
+    GwyField *field = gui->args->field;
+    GwyLine *rpsdf = gwy_line_new(1, 1, FALSE);
     GwyGraphCurveModel *gcmodel;
     gdouble q;
 
-    gwy_data_field_rpsdf(field, rpsdf, GWY_INTERPOLATION_KEY, GWY_WINDOWING_BLACKMANN, -1);
+    gwy_field_rpsdf(field, rpsdf, GWY_WINDOWING_BLACKMANN, -1);
     /* Transform to units of Nyquist frequency. */
-    q = 1.0/(G_PI/gwy_data_field_get_dx(field));
-    gwy_data_line_set_real(rpsdf, gwy_data_line_get_real(rpsdf)*q);
-    gwy_data_line_set_offset(rpsdf, gwy_data_line_get_offset(rpsdf)*q);
-    gwy_data_line_multiply(rpsdf, 1.0/fmax(gwy_data_line_get_max(rpsdf), G_MINDOUBLE));
+    q = 1.0/(G_PI/gwy_field_get_dx(field));
+    gwy_line_set_real(rpsdf, gwy_line_get_real(rpsdf)*q);
+    gwy_line_set_offset(rpsdf, gwy_line_get_offset(rpsdf)*q);
+    gwy_line_multiply(rpsdf, 1.0/fmax(gwy_line_max(rpsdf), G_MINDOUBLE));
 
     gcmodel = gwy_graph_curve_model_new();
-    gwy_graph_curve_model_set_data_from_dataline(gcmodel, rpsdf, 0, 0);
+    gwy_graph_curve_model_set_data_from_line(gcmodel, rpsdf, 0, 0);
     g_object_set(gcmodel,
                  "mode", GWY_GRAPH_CURVE_LINE,
                  "color", gwy_graph_get_preset_color(0),
