@@ -1,5 +1,5 @@
 /*
- *  $Id: gaussian_step.c 23666 2021-05-10 21:56:10Z yeti-dn $
+ *  $Id: gaussian_step.c 28910 2025-11-24 18:11:09Z yeti-dn $
  *  Copyright (C) 2003-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,14 +16,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/gwyprocess.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
@@ -37,24 +33,23 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *result;
+    GwyField *field;
+    GwyField *result;
 } ModuleArgs;
 
 typedef struct {
     ModuleArgs *args;
     GtkWidget *dialog;
     GwyParamTable *table;
-    GwyContainer *data;
 } ModuleGUI;
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             gaussian_step       (GwyContainer *data,
-                                             GwyRunType runtype);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static void             execute             (ModuleArgs *args);
 static GwyDialogOutcome run_gui             (ModuleArgs *args,
-                                             GwyContainer *data,
+                                             GwyFile *data,
                                              gint id);
 static void             param_changed       (ModuleGUI *gui,
                                              gint id);
@@ -76,11 +71,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("gaussian_step",
-                              (GwyProcessFunc)&gaussian_step,
+                              module_main,
                               N_("/_Presentation/_Edge Detection/_Gaussian Step..."),
                               NULL,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Detect edges using Gaussian step filter"));
 
     return TRUE;
@@ -102,25 +97,25 @@ define_module_params(void)
 }
 
 static void
-gaussian_step(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     ModuleArgs args;
     GQuark squark;
     gint id;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     GWY_APP_SHOW_FIELD_KEY, &squark,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_FIELD_ID, &id,
+                                 GWY_APP_SHOW_FIELD_KEY, &squark,
+                                 0);
     g_return_if_fail(args.field && squark);
 
-    args.result = gwy_data_field_new_alike(args.field, TRUE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(args.result), NULL);
+    args.result = gwy_field_new_alike(args.field, TRUE);
+    gwy_unit_clear(gwy_field_get_unit_z(args.result));
     args.params = gwy_params_new_from_settings(define_module_params());
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, id);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -129,9 +124,9 @@ gaussian_step(GwyContainer *data, GwyRunType runtype)
     if (outcome != GWY_DIALOG_HAVE_RESULT)
         execute(&args);
 
-    gwy_app_undo_qcheckpointv(data, 1, &squark);
-    gwy_container_set_object(data, squark, args.result);
-    gwy_app_channel_log_add_proc(data, id, id);
+    gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &squark);
+    gwy_container_set_object(GWY_CONTAINER(data), squark, args.result);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, id);
 
 end:
     g_object_unref(args.result);
@@ -139,28 +134,23 @@ end:
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     GtkWidget *hbox, *dataview;
-    GwyDialogOutcome outcome;
     GwyParamTable *table;
     GwyDialog *dialog;
     ModuleGUI gui;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
-    gui.data = gwy_container_new();
-    gwy_container_set_object_by_name(gui.data, "/0/data", args->result);
-    gwy_app_sync_data_items(data, gui.data, id, 0, FALSE,
-                            GWY_DATA_ITEM_PALETTE,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
 
     gui.dialog = gwy_dialog_new(_("Gaussian Step Detection"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_UPDATE, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
+    dataview = gwy_create_preview(args->result, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_REAL_SQUARE);
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), FALSE);
 
     table = gui.table = gwy_param_table_new(args->params);
@@ -175,11 +165,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     g_signal_connect_swapped(table, "param-changed", G_CALLBACK(param_changed), &gui);
     gwy_dialog_set_preview_func(dialog, GWY_PREVIEW_IMMEDIATE, preview, &gui, NULL);
 
-    outcome = gwy_dialog_run(dialog);
-
-    g_object_unref(gui.data);
-
-    return outcome;
+    return gwy_dialog_run(dialog);
 }
 
 static void
@@ -196,7 +182,7 @@ preview(gpointer user_data)
     ModuleArgs *args = gui->args;
 
     execute(args);
-    gwy_data_field_data_changed(args->result);
+    gwy_field_data_changed(args->result);
     gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
 }
 
@@ -205,11 +191,11 @@ execute(ModuleArgs *args)
 {
     GwyParams *params = args->params;
     gdouble gaussian_fwhm = gwy_params_get_double(params, PARAM_GAUSSIAN_FWHM);
-    GwyDataField *result = args->result, *field = args->field;
+    GwyField *result = args->result, *field = args->field;
 
-    gwy_data_field_copy(field, result, FALSE);
-    gwy_data_field_filter_gauss_step(result, gaussian_fwhm*FWHM2SIGMA);
-    gwy_data_field_normalize(result);
+    gwy_field_copy_data(field, result);
+    gwy_field_filter_gauss_step(result, gaussian_fwhm*FWHM2SIGMA);
+    gwy_field_normalize(result);
 }
 
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
