@@ -1,5 +1,5 @@
 /*
- *  $Id: good_profile.c 25638 2023-09-07 12:23:28Z yeti-dn $
+ *  $Id: good_profile.c 29061 2026-01-02 15:01:53Z yeti-dn $
  *  Copyright (C) 2020-2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,19 +16,9 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/level.h>
-#include <libprocess/stats.h>
-#include <libprocess/linestats.h>
-#include <libprocess/correct.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
@@ -57,8 +47,8 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *mask;
+    GwyField *field;
+    GwyField *mask;
     GwyGraphModel *gmodel;
 } ModuleArgs;
 
@@ -66,24 +56,23 @@ typedef struct {
     ModuleArgs *args;
     GtkWidget *dialog;
     GwyParamTable *table;
-    GwyContainer *data;
     GtkWidget *dataview;
     GtkWidget *graph;
-    GwySIValueFormat *varvf;
+    GwyValueFormat *varvf;
 } ModuleGUI;
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             good_profile        (GwyContainer *data,
-                                             GwyRunType runtype);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static void             execute             (ModuleArgs *args,
                                              gdouble *variation);
 static GwyDialogOutcome run_gui             (ModuleArgs *args,
-                                             GwyContainer *data,
+                                             GwyFile *data,
                                              gint id);
 static void             param_changed       (ModuleGUI *gui,
                                              gint id);
-static gboolean         other_image_filter  (GwyContainer *data,
+static gboolean         other_image_filter  (GwyFile *data,
                                              gint id,
                                              gpointer user_data);
 static void             preview             (gpointer user_data);
@@ -105,11 +94,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("good_profile",
-                              (GwyProcessFunc)&good_profile,
+                              module_main,
                               N_("/_Correct Data/_Good Mean Profile..."),
                               NULL,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Calculate good mean profile"));
 
     return TRUE;
@@ -134,7 +123,7 @@ define_module_params(void)
     paramdef = gwy_param_def_new();
     gwy_param_def_set_function_name(paramdef, gwy_process_func_current());
     gwy_param_def_add_boolean(paramdef, PARAM_CREATE_MASK, "create_mask", _("Create _mask"), TRUE);
-    gwy_param_def_add_gwyenum(paramdef, PARAM_DISPLAY, "display", gwy_sgettext("verb|Display"),
+    gwy_param_def_add_gwyenum(paramdef, PARAM_DISPLAY, "display", C_("verb", "Display"),
                               displays, G_N_ELEMENTS(displays), GOOD_PROF_GRAPH);
     gwy_param_def_add_gwyenum(paramdef, PARAM_MODE, "mode", _("Mode"),
                               modes, G_N_ELEMENTS(modes), GOOD_PROF_SINGLE);
@@ -147,36 +136,36 @@ define_module_params(void)
 }
 
 static void
-good_profile(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     GwyAppDataId target_graph_id;
-    GwyDataField *field;
+    GwyField *field;
     GwyParams *params;
     ModuleArgs args;
     GQuark mquark;
     gint id;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &field,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     GWY_APP_MASK_FIELD_KEY, &mquark,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &field,
+                                 GWY_APP_FIELD_ID, &id,
+                                 GWY_APP_MASK_FIELD_KEY, &mquark,
+                                 0);
     g_return_if_fail(field);
 
     args.field = field;
     args.mask = NULL;
     args.gmodel = gwy_graph_model_new();
-    gwy_graph_model_set_units_from_data_field(args.gmodel, field, 1, 0, 0, 1);
+    gwy_graph_model_set_units_from_field(args.gmodel, field, 1, 0, 0, 1);
     args.params = params = gwy_params_new_from_settings(define_module_params());
     /* Avoid Laplace interpolation in execute() if we do not need it. */
-    if (runtype == GWY_RUN_INTERACTIVE || gwy_params_get_boolean(params, PARAM_CREATE_MASK)) {
-        args.mask = gwy_data_field_new_alike(field, TRUE);
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(args.mask), NULL);
+    if (mode ==GWY_RUN_INTERACTIVE || gwy_params_get_boolean(params, PARAM_CREATE_MASK)) {
+        args.mask = gwy_field_new_alike(field, TRUE);
+        gwy_unit_clear(gwy_field_get_unit_z(args.mask));
     }
     sanitise_params(&args);
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, id);
         gwy_params_save_to_settings(params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -194,37 +183,35 @@ good_profile(GwyContainer *data, GwyRunType runtype)
          * undo checkpoints then, one in each file. */
         if (gwy_params_get_enum(params, PARAM_MODE) == GOOD_PROF_MULTIPLE) {
             GwyAppDataId other_image_id = gwy_params_get_data_id(params, PARAM_OTHER_IMAGE);
-            GwyContainer *otherdata = gwy_app_data_browser_get(other_image_id.datano);
-            GQuark quarks[2];
+            GwyFile *otherdata = gwy_data_browser_get_file(other_image_id.datano);
+            GQuark quarks[2] = { mquark, gwy_file_key_image_mask(other_image_id.id) };
 
-            quarks[0] = mquark;
-            quarks[1] = gwy_app_get_mask_key_for_id(other_image_id.id);
             if (otherdata == data)
-                gwy_app_undo_qcheckpointv(data, 2, quarks);
+                gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 2, quarks);
             else {
-                gwy_app_undo_qcheckpointv(data, 1, quarks);
-                gwy_app_undo_qcheckpointv(otherdata, 1, quarks+1);
+                gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, quarks);
+                gwy_app_undo_qcheckpointv(GWY_CONTAINER(otherdata), 1, quarks+1);
                 /* XXX: We should sync the mask colour, but only if the user has changed it.  Otherwise the second
                  * channel must be left alone. */
             }
-            field = gwy_data_field_duplicate(args.mask);
-            gwy_container_set_object(data, quarks[0], args.mask);
-            gwy_container_pass_object(otherdata, quarks[1], field);
+            field = gwy_field_copy(args.mask);
+            gwy_file_set_image_mask(data, id, args.mask);
+            gwy_file_pass_image(otherdata, other_image_id.id, field);
         }
         else {
-            gwy_app_undo_qcheckpointv(data, 1, &mquark);
-            gwy_container_set_object(data, mquark, args.mask);
+            gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &mquark);
+            gwy_file_set_image_mask(data, id, args.mask);
         }
     }
 
 end:
     g_object_unref(params);
-    GWY_OBJECT_UNREF(args.mask);
+    g_clear_object(&args.mask);
     g_object_unref(args.gmodel);
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     GoodProfDisplay display = gwy_params_get_enum(args->params, PARAM_DISPLAY);
     GwyDialogOutcome outcome;
@@ -233,32 +220,26 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     GwyParamTable *table;
     ModuleGUI gui;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
-    gui.varvf = gwy_data_field_get_value_format_z(args->field, GWY_SI_UNIT_FORMAT_VFMARKUP, NULL);
+    gui.varvf = gwy_field_get_value_format_z(args->field, GWY_UNIT_FORMAT_VFMARKUP, NULL);
     gui.varvf->precision++;
-    gui.data = gwy_container_new();
-    gwy_container_set_object_by_name(gui.data, "/0/data", args->field);
-    gwy_container_set_object_by_name(gui.data, "/0/mask", args->mask);
-    gwy_app_sync_data_items(data, gui.data, id, 0, FALSE,
-                            GWY_DATA_ITEM_PALETTE,
-                            GWY_DATA_ITEM_MASK_COLOR,
-                            GWY_DATA_ITEM_RANGE,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
 
     gui.dialog = gwy_dialog_new(_("Good Mean Profile"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_UPDATE, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    hbox = gwy_hbox_new(0);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
     gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
     gwy_dialog_add_content(dialog, hbox, FALSE, FALSE, 0);
 
-    vbox = gwy_vbox_new(0);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
     gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 4);
 
-    gui.dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, TRUE);
+    gui.dataview = gwy_create_preview(args->field, args->mask, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(gui.dataview), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_MASK_COLOR
+                        | GWY_FILE_ITEM_RANGE | GWY_FILE_ITEM_REAL_SQUARE);
     gtk_box_pack_start(GTK_BOX(vbox), gui.dataview, FALSE, FALSE, 0);
     if (display != GOOD_PROF_IMAGE)
         gtk_widget_set_no_show_all(gui.dataview, TRUE);
@@ -287,7 +268,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_param_table_append_checkbox(table, PARAM_CREATE_MASK);
 
     gwy_param_table_append_header(table, -1, _("Options"));
-    gwy_param_table_append_mask_color(table, PARAM_MASK_COLOR, gui.data, 0, NULL, -1);
+    gwy_param_table_append_image_mask_color(table, PARAM_MASK_COLOR, GWY_DATA_VIEW(gui.dataview), NULL, -1);
     gwy_param_table_append_checkbox(table, PARAM_UPDATE);
 
     gtk_box_pack_start(GTK_BOX(hbox), gwy_param_table_widget(table), TRUE, TRUE, 0);
@@ -298,11 +279,12 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 
     outcome = gwy_dialog_run(dialog);
 
-    if (outcome != GWY_DIALOG_CANCEL && gwy_params_get_boolean(args->params, PARAM_CREATE_MASK))
-        gwy_app_sync_data_items(gui.data, data, 0, id, FALSE, GWY_DATA_ITEM_MASK_COLOR, 0);
+    if (outcome != GWY_DIALOG_CANCEL && gwy_params_get_boolean(args->params, PARAM_CREATE_MASK)) {
+        GwyRGBA color = gwy_params_get_color(args->params, PARAM_MASK_COLOR);
+        gwy_container_set_boxed(GWY_CONTAINER(data), GWY_TYPE_RGBA, gwy_file_key_image_mask_color(id), &color);
+    }
 
-    g_object_unref(gui.data);
-    gwy_si_unit_value_format_free(gui.varvf);
+    gwy_value_format_free(gui.varvf);
 
     return outcome;
 }
@@ -341,16 +323,20 @@ param_changed(ModuleGUI *gui, gint id)
 }
 
 static gboolean
-other_image_filter(GwyContainer *data, gint id, gpointer user_data)
+other_image_filter(GwyFile *data, gint id, gpointer user_data)
 {
-    GwyDataField *other_image, *field = (GwyDataField*)user_data;
+    GwyField *other_image, *field = (GwyField*)user_data;
 
-    if (!gwy_container_gis_object(data, gwy_app_get_data_key_for_id(id), &other_image))
+    if (!(other_image = gwy_file_get_image(data, id)))
         return FALSE;
     if (other_image == field)
         return FALSE;
 
-    return !gwy_data_field_check_compatibility(other_image, field, GWY_DATA_COMPATIBILITY_ALL);
+    return !gwy_field_is_incompatible(other_image, field,
+                                      GWY_DATA_MISMATCH_RES
+                                      | GWY_DATA_MISMATCH_REAL
+                                      | GWY_DATA_MISMATCH_LATERAL
+                                      | GWY_DATA_MISMATCH_VALUE);
 }
 
 static void
@@ -358,12 +344,12 @@ preview(gpointer user_data)
 {
     ModuleGUI *gui = (ModuleGUI*)user_data;
     ModuleArgs *args = gui->args;
-    GwySIValueFormat *vf = gui->varvf;
+    GwyValueFormat *vf = gui->varvf;
     gdouble variation;
     gchar *s;
 
     execute(args, &variation);
-    gwy_data_field_data_changed(args->mask);
+    gwy_field_data_changed(args->mask);
     s = g_strdup_printf("%.*f", vf->precision, variation/vf->magnitude);
     gwy_param_table_info_set_valuestr(gui->table, INFO_VARIATION, s);
     g_free(s);
@@ -371,28 +357,28 @@ preview(gpointer user_data)
 }
 
 static void
-good_profile_do_single(GwyDataLine *profile, ModuleArgs *args)
+good_profile_do_single(GwyLine *profile, ModuleArgs *args)
 {
-    GwyDataField *ffield, *fmask = NULL, *field = args->field, *mask = args->mask;
+    GwyField *ffield, *fmask = NULL, *field = args->field, *mask = args->mask;
     gdouble trim_fraction = gwy_params_get_double(args->params, PARAM_TRIM_FRACTION);
     const gdouble *ddata, *drow;
     gdouble *data, *mdata = NULL, *row, *mrow, *ldata, *low = NULL, *high = NULL;
     gint xres, yres, ntrim, i, j, k[2];
     gdouble v[2];
 
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
-    ldata = gwy_data_line_get_data(profile);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
+    ldata = gwy_line_get_data(profile);
 
     /* Work with a flipped field because we process it by column.  Note
      * that this means xres and yres have swapped meaning! */
-    ffield = gwy_data_field_new_alike(field, FALSE);
-    gwy_data_field_flip_xy(field, ffield, FALSE);
-    data = gwy_data_field_get_data(ffield);
+    ffield = gwy_field_new_alike(field, FALSE);
+    gwy_field_transpose(field, ffield, FALSE);
+    data = gwy_field_get_data(ffield);
     if (mask) {
-        fmask = gwy_data_field_new_alike(mask, FALSE);
-        gwy_data_field_flip_xy(mask, fmask, FALSE);
-        mdata = gwy_data_field_get_data(fmask);
+        fmask = gwy_field_new_alike(mask, FALSE);
+        gwy_field_transpose(mask, fmask, FALSE);
+        mdata = gwy_field_get_data(fmask);
         low = g_new(gdouble, xres);
         high = g_new(gdouble, xres);
     }
@@ -406,10 +392,10 @@ good_profile_do_single(GwyDataLine *profile, ModuleArgs *args)
     k[1] = yres-1-ntrim;
     for (i = 0; i < xres; i++) {
         row = data + i*yres;
-        ldata[i] = gwy_math_trimmed_mean(yres, row, ntrim, ntrim);
+        ldata[i] = gwy_math_trimmed_mean(row, yres, ntrim, ntrim);
         if (mdata) {
             mrow = mdata + i*yres;
-            gwy_math_kth_ranks(yres, row, 2, k, v);
+            gwy_math_kth_ranks(row, yres, k, 2, v);
             low[i] = v[0];
             high[i] = v[1];
         }
@@ -418,8 +404,8 @@ good_profile_do_single(GwyDataLine *profile, ModuleArgs *args)
     g_object_unref(ffield);
     if (fmask) {
         g_object_unref(fmask);
-        ddata = gwy_data_field_get_data_const(field);
-        mdata = gwy_data_field_get_data(mask);
+        ddata = gwy_field_get_data_const(field);
+        mdata = gwy_field_get_data(mask);
         for (i = 0; i < yres; i++) {
             drow = ddata + i*xres;
             mrow = mdata + i*xres;
@@ -432,47 +418,47 @@ good_profile_do_single(GwyDataLine *profile, ModuleArgs *args)
 }
 
 static void
-good_profile_do_multiple(GwyDataLine *profile, GwyDataLine *weights, ModuleArgs *args)
+good_profile_do_multiple(GwyLine *profile, GwyLine *weights, ModuleArgs *args)
 {
-    GwyDataField *field = args->field, *mask = args->mask;
+    GwyField *field = args->field, *mask = args->mask;
     gdouble trim_fraction = gwy_params_get_double(args->params, PARAM_TRIM_FRACTION);
-    GwyDataField *field2 = gwy_params_get_image(args->params, PARAM_OTHER_IMAGE);
-    GwyDataField *buf;
+    GwyField *field2 = gwy_params_get_image(args->params, PARAM_OTHER_IMAGE);
+    GwyField *buf;
     const gdouble *d1, *d2;
     gdouble *b, *m;
     gdouble p, threshold;
     gint xres, yres, n, i;
 
-    d1 = gwy_data_field_get_data_const(field);
-    d2 = gwy_data_field_get_data_const(field2);
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
+    d1 = gwy_field_get_data_const(field);
+    d2 = gwy_field_get_data_const(field2);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
     n = xres*yres;
 
-    buf = gwy_data_field_new_alike(field, FALSE);
-    b = gwy_data_field_get_data(buf);
+    buf = gwy_field_new_alike(field, FALSE);
+    b = gwy_field_get_data(buf);
 
     if (!mask) {
-        mask = gwy_data_field_new_alike(field, FALSE);
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(mask), NULL);
+        mask = gwy_field_new_alike(field, FALSE);
+        gwy_unit_clear(gwy_field_get_unit_z(mask));
     }
     else
         g_object_ref(mask);
-    m = gwy_data_field_get_data(mask);
+    m = gwy_field_get_data(mask);
 
     /* Use m[] temporarily for the difference we pass to gwy_math_percentiles() and they get shuffled. */
     for (i = 0; i < n; i++)
         b[i] = m[i] = fabs(d1[i] - d2[i]);
 
     p = 100.0*(1.0 - trim_fraction);
-    gwy_math_percentiles(n, m, GWY_PERCENTILE_INTERPOLATION_MIDPOINT, 1, &p, &threshold);
+    gwy_math_percentiles(m, n, GWY_PERCENTILE_INTERPOLATION_MIDPOINT, &p, 1, &threshold);
 
     for (i = 0; i < n; i++) {
         m[i] = (b[i] > threshold);
         b[i] = 0.5*(d1[i] + d2[i]);
     }
-    gwy_data_field_get_line_stats_mask(buf, mask, GWY_MASK_EXCLUDE, profile, weights, 0, 0, xres, yres,
-                                       GWY_LINE_STAT_MEAN, GWY_ORIENTATION_VERTICAL);
+    gwy_field_area_get_line_stats(buf, mask, GWY_MASK_EXCLUDE, profile, weights, 0, 0, xres, yres,
+                                  GWY_LINE_STAT_MEAN, GWY_ORIENTATION_VERTICAL);
 
     g_object_unref(mask);
     g_object_unref(buf);
@@ -483,21 +469,21 @@ execute(ModuleArgs *args, gdouble *variation)
 {
     GoodProfMode mode = gwy_params_get_enum(args->params, PARAM_MODE);
     GwyGraphCurveModel *gcmodel;
-    GwyDataField *field = args->field;
-    GwyDataLine *profile, *weights = NULL;
+    GwyField *field = args->field;
+    GwyLine *profile, *weights = NULL;
     gdouble dx, xoff;
     gint xres, i, n;
     gdouble *d, *w;
     GwyXY *xy;
 
-    xres = gwy_data_field_get_xres(field);
-    profile = gwy_data_line_new(xres, gwy_data_field_get_xreal(field), TRUE);
-    gwy_data_field_copy_units_to_data_line(field, profile);
+    xres = gwy_field_get_xres(field);
+    profile = gwy_line_new(xres, gwy_field_get_xreal(field), TRUE);
+    gwy_field_copy_units_to_line(field, profile);
 
     if (mode == GOOD_PROF_SINGLE)
         good_profile_do_single(profile, args);
     else if (mode == GOOD_PROF_MULTIPLE) {
-        weights = gwy_data_line_new_alike(profile, FALSE);
+        weights = gwy_line_new_alike(profile, FALSE);
         good_profile_do_multiple(profile, weights, args);
     }
     else {
@@ -518,13 +504,13 @@ execute(ModuleArgs *args, gdouble *variation)
     }
 
     if (mode == GOOD_PROF_SINGLE)
-        gwy_graph_curve_model_set_data_from_dataline(gcmodel, profile, 0, 0);
+        gwy_graph_curve_model_set_data_from_line(gcmodel, profile, 0, 0);
     else {
-        xres = gwy_data_line_get_res(profile);
-        dx = gwy_data_line_get_dx(profile);
-        xoff = gwy_data_line_get_offset(profile);
-        d = gwy_data_line_get_data(profile);
-        w = gwy_data_line_get_data(weights);
+        xres = gwy_line_get_res(profile);
+        dx = gwy_line_get_dx(profile);
+        xoff = gwy_line_get_offset(profile);
+        d = gwy_line_get_data(profile);
+        w = gwy_line_get_data(weights);
         xy = g_new(GwyXY, xres);
         for (i = n = 0; i < xres; i++) {
             if (w[i] > 0.0) {
@@ -539,15 +525,15 @@ execute(ModuleArgs *args, gdouble *variation)
         if (variation) {
             for (i = 0; i < xres; i++)
                 w[i] = (w[i] <= 0.0);
-            gwy_data_line_correct_laplace(profile, weights);
+            gwy_line_correct_laplace(profile, weights);
         }
     }
 
     if (variation)
-        *variation = gwy_data_line_get_variation(profile);
+        *variation = gwy_line_variation(profile);
 
     g_object_unref(profile);
-    GWY_OBJECT_UNREF(weights);
+    g_clear_object(&weights);
 }
 
 static void
@@ -559,7 +545,7 @@ sanitise_params(ModuleArgs *args)
     gboolean is_none = gwy_params_data_id_is_none(params, PARAM_OTHER_IMAGE);
 
     if (mode == GOOD_PROF_MULTIPLE
-        && (is_none || !other_image_filter(gwy_app_data_browser_get(other_image.datano), other_image.id, args->field)))
+        && (is_none || !other_image_filter(gwy_data_browser_get_file(other_image.datano), other_image.id, args->field)))
         gwy_params_reset(params, PARAM_MODE);
 }
 
