@@ -1,6 +1,6 @@
 /*
- *  $Id: grain_cross.c 25076 2022-10-11 13:48:10Z yeti-dn $
- *  Copyright (C) 2007-2022 David Necas (Yeti).
+ *  $Id: grain_cross.c 28978 2025-12-11 10:41:31Z yeti-dn $
+ *  Copyright (C) 2007-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -16,18 +16,11 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/grains.h>
-#include <libgwydgets/gwygrainvaluemenu.h>
-#include <libgwydgets/gwydgetutils.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
@@ -44,12 +37,11 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *mask;
+    GwyField *field;
+    GwyNield *grains;
     GwyGraphModel *gmodel;
     /* Cached input data properties. */
     guint ngrains;
-    gint *grains;
 } ModuleArgs;
 
 typedef struct {
@@ -62,24 +54,24 @@ typedef struct {
 
 static gboolean         module_register      (void);
 static GwyParamDef*     define_module_params (void);
-static void             grain_cross          (GwyContainer *data,
-                                              GwyRunType runtype);
+static void             module_main          (GwyFile *data,
+                                              GwyRunModeFlags mode);
 static void             execute              (ModuleArgs *args);
 static GwyDialogOutcome run_gui              (ModuleArgs *args);
 static void             param_changed        (ModuleGUI *gui,
                                               gint id);
 static void             preview              (gpointer user_data);
 static void             axis_quantity_changed(ModuleGUI *gui);
-static gboolean         other_image_filter   (GwyContainer *data,
+static gboolean         other_image_filter   (GwyFile *data,
                                               gint id,
                                               gpointer user_data);
-static GwyDataField*    get_ordinate_field   (ModuleArgs *args);
-static GtkTreeView*     attach_axis_list     (GtkWidget *table,
+static GwyField*        get_ordinate_field   (ModuleArgs *args);
+static GtkTreeView*     attach_axis_list     (GtkGrid *grid,
                                               const gchar *name,
                                               gint column,
                                               gint idvalue,
                                               gint idexpanded,
-                                              GwyDataField *field,
+                                              GwyField *field,
                                               ModuleGUI *gui);
 static void             set_graph_model_units(ModuleArgs *args);
 
@@ -99,11 +91,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("grain_cross",
-                              (GwyProcessFunc)&grain_cross,
+                              module_main,
                               N_("/_Grains/_Correlate..."),
-                              GWY_STOCK_GRAIN_CORRELATION,
+                              GWY_ICON_GRAIN_CORRELATION,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA | GWY_MENU_FLAG_DATA_MASK,
+                              GWY_MENU_FLAG_IMAGE | GWY_MENU_FLAG_IMAGE_MASK,
                               N_("Correlate grain characteristics"));
 
     return TRUE;
@@ -122,11 +114,11 @@ define_module_params(void)
     gwy_param_def_add_resource(paramdef, PARAM_ABSCISSA, "abscissa", _("_Abscissa"),
                                gwy_grain_values(), "Equivalent disc radius");
     gwy_param_def_add_grain_groups(paramdef, PARAM_ABSCISSA_EXPANDED, "abscissa_expanded", NULL,
-                                   1 << GWY_GRAIN_VALUE_GROUP_AREA);
+                                   1 << GWY_GRAIN_GROUP_AREA);
     gwy_param_def_add_resource(paramdef, PARAM_ORDINATE, "ordinate", _("O_rdinate"),
                                gwy_grain_values(), "Projected boundary length");
     gwy_param_def_add_grain_groups(paramdef, PARAM_ORDINATE_EXPANDED, "ordinate_expanded", NULL,
-                                   1 << GWY_GRAIN_VALUE_GROUP_BOUNDARY);
+                                   1 << GWY_GRAIN_GROUP_BOUNDARY);
     gwy_param_def_add_boolean(paramdef, PARAM_DIFFERENT_ORDINATE, "different_ordinate",
                               _("Ordinate data calculated from different image"), FALSE);
     gwy_param_def_add_image_id(paramdef, PARAM_OTHER_IMAGE, "other_image", _("Ordinate _image"));
@@ -136,19 +128,19 @@ define_module_params(void)
 
 static gboolean
 check_same_units(GwyParams *params, gint idvalue, gint idexpanded,
-                 GwyDataField *field, GwyContainer *data, gint id,
-                 GwyRunType runtype)
+                 GwyField *field, GwyFile *data, gint id,
+                 GwyRunModeFlags mode)
 {
     GwyGrainValue *gvalue = GWY_GRAIN_VALUE(gwy_params_get_resource(params, idvalue));
     GwyGrainValueFlags flags = gwy_grain_value_get_flags(gvalue);
 
-    if (!(flags & GWY_GRAIN_VALUE_SAME_UNITS))
+    if (!(flags & GWY_GRAIN_SAME_UNITS))
         return TRUE;
-    if (gwy_si_unit_equal(gwy_data_field_get_si_unit_xy(field), gwy_data_field_get_si_unit_z(field)))
+    if (gwy_unit_equal(gwy_field_get_unit_xy(field), gwy_field_get_unit_z(field)))
         return TRUE;
 
     /* Non-interactively, we complain.  But interactively we just reset to valid values. */
-    if (runtype == GWY_RUN_IMMEDIATE) {
+    if (mode ==GWY_RUN_IMMEDIATE) {
         gwy_require_image_same_units(field, data, id, _("Grain Correlations"));
         return FALSE;
     }
@@ -159,35 +151,35 @@ check_same_units(GwyParams *params, gint idvalue, gint idexpanded,
 }
 
 static void
-grain_cross(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     GwyAppDataId target_graph_id;
     ModuleArgs args;
-    GwyDataField *ordfield;
+    GwyField *ordfield;
     GwyParams *params;
     gint id;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_MASK_FIELD, &args.mask,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
-    g_return_if_fail(args.field && args.mask);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 /* TODO Nield */
+                                 GWY_APP_MASK_FIELD, &args.grains,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
+    g_return_if_fail(args.field && args.grains);
     args.params = params = gwy_params_new_from_settings(define_module_params());
     if (gwy_params_data_id_is_none(params, PARAM_OTHER_IMAGE))
         gwy_params_set_boolean(params, PARAM_DIFFERENT_ORDINATE, FALSE);
     ordfield = get_ordinate_field(&args);
-    if (!check_same_units(params, PARAM_ABSCISSA, PARAM_ABSCISSA_EXPANDED, args.field, data, id, runtype)
-        || !check_same_units(params, PARAM_ORDINATE, PARAM_ORDINATE_EXPANDED, ordfield, data, id, runtype))
+    if (!check_same_units(params, PARAM_ABSCISSA, PARAM_ABSCISSA_EXPANDED, args.field, data, id, mode)
+        || !check_same_units(params, PARAM_ORDINATE, PARAM_ORDINATE_EXPANDED, ordfield, data, id, mode))
         goto end;
 
     args.gmodel = gwy_graph_model_new();
     set_graph_model_units(&args);
-    args.grains = g_new0(gint, gwy_data_field_get_xres(args.mask)*gwy_data_field_get_yres(args.mask));
-    args.ngrains = gwy_data_field_number_grains(args.mask, args.grains);
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    args.ngrains = gwy_nield_max(args.grains);
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args);
         gwy_params_save_to_settings(params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -200,8 +192,7 @@ grain_cross(GwyContainer *data, GwyRunType runtype)
     gwy_app_add_graph_or_curves(args.gmodel, data, &target_graph_id, 1);
 
 end:
-    g_free(args.grains);
-    GWY_OBJECT_UNREF(args.gmodel);
+    g_clear_object(&args.gmodel);
     g_object_unref(args.params);
 }
 
@@ -210,7 +201,7 @@ run_gui(ModuleArgs *args)
 {
     ModuleGUI gui;
     GwyDialog *dialog;
-    GtkWidget *gtktable, *graph;
+    GtkWidget *grid, *graph;
     GwyParamTable *table;
 
     gui.args = args;
@@ -220,20 +211,20 @@ run_gui(ModuleArgs *args)
     gtk_window_set_default_size(GTK_WINDOW(dialog), 860, 520);
     gtk_dialog_set_response_sensitive(GTK_DIALOG(dialog), GTK_RESPONSE_OK, !!args->ngrains);
 
-    gtktable = gtk_table_new(3, 3, FALSE);
-    gtk_table_set_row_spacings(GTK_TABLE(gtktable), 2);
-    gtk_table_set_col_spacings(GTK_TABLE(gtktable), 6);
-    gtk_container_set_border_width(GTK_CONTAINER(gtktable), 4);
-    gwy_dialog_add_content(dialog, gtktable, TRUE, TRUE, 0);
+    grid = gtk_grid_new();
+    gtk_grid_set_row_spacing(GTK_GRID(grid), 2);
+    gtk_grid_set_column_spacing(GTK_GRID(grid), 6);
+    gtk_container_set_border_width(GTK_CONTAINER(grid), 4);
+    gwy_dialog_add_content(dialog, grid, TRUE, TRUE, 0);
 
     graph = gwy_graph_new(args->gmodel);
     gtk_widget_set_size_request(graph, PREVIEW_SMALL_SIZE, -1);
     gwy_graph_enable_user_input(GWY_GRAPH(graph), FALSE);
-    gtk_table_attach(GTK_TABLE(gtktable), graph, 0, 1, 0, 3, GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+    gtk_grid_attach(GTK_GRID(grid), graph, 0, 0, 1, 3);
 
-    gui.abscissa = attach_axis_list(gtktable, _("_Abscissa"), 1, PARAM_ABSCISSA, PARAM_ABSCISSA_EXPANDED,
+    gui.abscissa = attach_axis_list(GTK_GRID(grid), _("_Abscissa"), 1, PARAM_ABSCISSA, PARAM_ABSCISSA_EXPANDED,
                                     args->field, &gui);
-    gui.ordinate = attach_axis_list(gtktable, _("O_rdinate"), 2, PARAM_ORDINATE, PARAM_ORDINATE_EXPANDED,
+    gui.ordinate = attach_axis_list(GTK_GRID(grid), _("O_rdinate"), 2, PARAM_ORDINATE, PARAM_ORDINATE_EXPANDED,
                                     get_ordinate_field(args), &gui);
 
     table = gui.table = gwy_param_table_new(args->params);
@@ -242,7 +233,7 @@ run_gui(ModuleArgs *args)
     gwy_param_table_data_id_set_filter(table, PARAM_OTHER_IMAGE, other_image_filter, args->field, NULL);
     gwy_param_table_append_target_graph(table, PARAM_TARGET_GRAPH, args->gmodel);
 
-    gtk_table_attach(GTK_TABLE(gtktable), gwy_param_table_widget(table), 1, 3, 2, 3, GTK_FILL, 0, 0, 0);
+    gtk_grid_attach(GTK_GRID(grid), gwy_param_table_widget(table), 1, 2, 2, 1);
     gwy_dialog_add_param_table(dialog, table);
 
     g_signal_connect_swapped(table, "param-changed", G_CALLBACK(param_changed), &gui);
@@ -267,7 +258,7 @@ axis_quantity_changed(ModuleGUI *gui)
 
     gwy_params_set_flags(params, PARAM_ABSCISSA_EXPANDED, gwy_grain_value_tree_view_get_expanded_groups(gui->abscissa));
     if (gtk_tree_selection_get_selected(gtk_tree_view_get_selection(gui->abscissa), &model, &iter)) {
-        gtk_tree_model_get(model, &iter, GWY_GRAIN_VALUE_STORE_COLUMN_ITEM, &gvalue, -1);
+        gtk_tree_model_get(model, &iter, GWY_GRAIN_STORE_COLUMN_ITEM, &gvalue, -1);
         gwy_params_set_resource(params, PARAM_ABSCISSA, gwy_resource_get_name(GWY_RESOURCE(gvalue)));
     }
     else
@@ -275,7 +266,7 @@ axis_quantity_changed(ModuleGUI *gui)
 
     gwy_params_set_flags(params, PARAM_ORDINATE_EXPANDED, gwy_grain_value_tree_view_get_expanded_groups(gui->ordinate));
     if (gtk_tree_selection_get_selected(gtk_tree_view_get_selection(gui->ordinate), &model, &iter)) {
-        gtk_tree_model_get(model, &iter, GWY_GRAIN_VALUE_STORE_COLUMN_ITEM, &gvalue, -1);
+        gtk_tree_model_get(model, &iter, GWY_GRAIN_STORE_COLUMN_ITEM, &gvalue, -1);
         gwy_params_set_resource(params, PARAM_ORDINATE, gwy_resource_get_name(GWY_RESOURCE(gvalue)));
     }
     else
@@ -294,13 +285,13 @@ param_changed(ModuleGUI *gui, gint id)
     if (id < 0 || id == PARAM_DIFFERENT_ORDINATE || id == PARAM_OTHER_IMAGE) {
         GwyGrainValue *gvalue = GWY_GRAIN_VALUE(gwy_params_get_resource(params, PARAM_ORDINATE));
         GwyGrainValueFlags flags = gwy_grain_value_get_flags(gvalue);
-        GwyDataField *ordfield = get_ordinate_field(args);
-        gboolean same_units = gwy_si_unit_equal(gwy_data_field_get_si_unit_xy(ordfield),
-                                                gwy_data_field_get_si_unit_z(ordfield));
+        GwyField *ordfield = get_ordinate_field(args);
+        gboolean same_units = gwy_unit_equal(gwy_field_get_unit_xy(ordfield),
+                                             gwy_field_get_unit_z(ordfield));
         /* XXX: The grain value tree view is not smart enough to move the selection away from a disabled item
          * – basically it does not know where.  And GtkTreeView does not allow selecting something else if an
          * insensitive item is currently selected.  So we have to fix it manually. */
-        if ((flags & GWY_GRAIN_VALUE_SAME_UNITS) && !same_units) {
+        if ((flags & GWY_GRAIN_SAME_UNITS) && !same_units) {
             gwy_params_reset(params, PARAM_ORDINATE);
             gvalue = GWY_GRAIN_VALUE(gwy_params_get_resource(params, PARAM_ORDINATE));
             gwy_grain_value_tree_view_select(gui->ordinate, gvalue);
@@ -327,19 +318,17 @@ preview(gpointer user_data)
 }
 
 static gboolean
-other_image_filter(GwyContainer *data, gint id, gpointer user_data)
+other_image_filter(GwyFile *data, gint id, gpointer user_data)
 {
-    GwyDataField *field = (GwyDataField*)user_data;
-    GwyDataField *otherfield = gwy_container_get_object(data, gwy_app_get_data_key_for_id(id));
+    GwyField *field = (GwyField*)user_data;
+    GwyField *otherfield = gwy_file_get_image(data, id);
 
     /* Do not reject the field itself.  This ensures the chooser is non-empty. */
-    return !gwy_data_field_check_compatibility(field, otherfield,
-                                               GWY_DATA_COMPATIBILITY_RES
-                                               | GWY_DATA_COMPATIBILITY_REAL
-                                               | GWY_DATA_COMPATIBILITY_LATERAL);
+    return !gwy_field_is_incompatible(field, otherfield,
+                                      GWY_DATA_MISMATCH_RES | GWY_DATA_MISMATCH_REAL | GWY_DATA_MISMATCH_LATERAL);
 }
 
-static GwyDataField*
+static GwyField*
 get_ordinate_field(ModuleArgs *args)
 {
     gboolean different_ordinate = gwy_params_get_boolean(args->params, PARAM_DIFFERENT_ORDINATE);
@@ -347,9 +336,9 @@ get_ordinate_field(ModuleArgs *args)
 }
 
 static GtkTreeView*
-attach_axis_list(GtkWidget *table, const gchar *name,
+attach_axis_list(GtkGrid *grid, const gchar *name,
                  gint column, gint idvalue, gint idexpanded,
-                 GwyDataField *field, ModuleGUI *gui)
+                 GwyField *field, ModuleGUI *gui)
 {
     guint expanded = gwy_params_get_flags(gui->args->params, idexpanded);
     GwyGrainValue *gvalue = GWY_GRAIN_VALUE(gwy_params_get_resource(gui->args->params, idvalue));
@@ -358,19 +347,22 @@ attach_axis_list(GtkWidget *table, const gchar *name,
     gboolean same_units;
 
     label = gtk_label_new_with_mnemonic(name);
-    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
-    gtk_table_attach(GTK_TABLE(table), label, column, column+1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
+    gtk_label_set_xalign(GTK_LABEL(label), 0.0);
+    gtk_widget_set_vexpand(label, FALSE);
+    gtk_grid_attach(grid, label, column, 0, 1, 1);
 
     scwin = gtk_scrolled_window_new(NULL, NULL);
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scwin), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-    gtk_table_attach(GTK_TABLE(table), scwin, column, column+1, 1, 2,
-                     GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+    gtk_grid_attach(grid, scwin, column, 1, 1, 1);
+    gtk_widget_set_vexpand(scwin, TRUE);
+    gtk_widget_set_hexpand(scwin, TRUE);
+    gtk_widget_set_size_request(scwin, 240, -1);
 
     widget = gwy_grain_value_tree_view_new(FALSE, "name", NULL);
     list = GTK_TREE_VIEW(widget);
     gtk_tree_view_set_headers_visible(list, FALSE);
     gwy_grain_value_tree_view_set_expanded_groups(list, expanded);
-    same_units = gwy_si_unit_equal(gwy_data_field_get_si_unit_xy(field), gwy_data_field_get_si_unit_z(field));
+    same_units = gwy_unit_equal(gwy_field_get_unit_xy(field), gwy_field_get_unit_z(field));
     gwy_grain_value_tree_view_set_same_units(list, same_units);
     gwy_grain_value_tree_view_select(list, gvalue);
     gtk_label_set_mnemonic_widget(GTK_LABEL(label), widget);
@@ -384,7 +376,7 @@ execute(ModuleArgs *args)
 {
     GwyParams *params = args->params;
     GwyGraphModel *gmodel = args->gmodel;
-    GwyDataField *absfield = args->field, *ordfield = get_ordinate_field(args);
+    GwyField *absfield = args->field, *ordfield = get_ordinate_field(args);
     GwyGraphCurveModel *cmodel;
     GwyGrainValue *gvalues[2];
     gdouble *xdata, *ydata, *bothdata, *rdata[2];
@@ -399,11 +391,11 @@ execute(ModuleArgs *args)
     rdata[1] = ydata = bothdata + 3*ngrains + 1;
 
     if (ordfield != absfield) {
-        gwy_grain_values_calculate(1, gvalues + 0, rdata + 0, absfield, ngrains, args->grains);
-        gwy_grain_values_calculate(1, gvalues + 1, rdata + 1, ordfield, ngrains, args->grains);
+        gwy_grain_values_calculate(gvalues + 0, rdata + 0, 1, absfield, args->grains);
+        gwy_grain_values_calculate(gvalues + 1, rdata + 1, 1, ordfield, args->grains);
     }
     else
-        gwy_grain_values_calculate(2, gvalues, rdata, absfield, ngrains, args->grains);
+        gwy_grain_values_calculate(gvalues, rdata, 2, absfield, args->grains);
 
     for (i = 0; i < ngrains; i++) {
         bothdata[2*i + 0] = xdata[i+1];
@@ -420,6 +412,7 @@ execute(ModuleArgs *args)
     g_object_set(cmodel,
                  "description", title,
                  "mode", GWY_GRAPH_CURVE_POINTS,
+                 "point-size", 4,
                  NULL);
     gwy_graph_curve_model_set_data(cmodel, xdata, ydata, ngrains);
     g_free(bothdata);
@@ -439,21 +432,21 @@ execute(ModuleArgs *args)
 static void
 set_graph_model_units(ModuleArgs *args)
 {
-    GwyDataField *absfield = args->field, *ordfield = get_ordinate_field(args);
+    GwyField *absfield = args->field, *ordfield = get_ordinate_field(args);
     GwyGrainValue *gvalues[2];
-    GwySIUnit *unit = gwy_si_unit_new(NULL);
+    GwyUnit *unit = gwy_unit_new(NULL);
 
     gvalues[0] = GWY_GRAIN_VALUE(gwy_params_get_resource(args->params, PARAM_ABSCISSA));
     gvalues[1] = GWY_GRAIN_VALUE(gwy_params_get_resource(args->params, PARAM_ORDINATE));
 
-    gwy_si_unit_power_multiply(gwy_data_field_get_si_unit_xy(absfield), gwy_grain_value_get_power_xy(gvalues[0]),
-                               gwy_data_field_get_si_unit_z(absfield), gwy_grain_value_get_power_z(gvalues[0]),
-                               unit);
-    g_object_set(args->gmodel, "si-unit-x", unit, NULL);
-    gwy_si_unit_power_multiply(gwy_data_field_get_si_unit_xy(ordfield), gwy_grain_value_get_power_xy(gvalues[1]),
-                               gwy_data_field_get_si_unit_z(ordfield), gwy_grain_value_get_power_z(gvalues[1]),
-                               unit);
-    g_object_set(args->gmodel, "si-unit-y", unit, NULL);
+    gwy_unit_power_multiply(gwy_field_get_unit_xy(absfield), gwy_grain_value_get_power_xy(gvalues[0]),
+                            gwy_field_get_unit_z(absfield), gwy_grain_value_get_power_z(gvalues[0]),
+                            unit);
+    g_object_set(args->gmodel, "unit-x", unit, NULL);
+    gwy_unit_power_multiply(gwy_field_get_unit_xy(ordfield), gwy_grain_value_get_power_xy(gvalues[1]),
+                            gwy_field_get_unit_z(ordfield), gwy_grain_value_get_power_z(gvalues[1]),
+                            unit);
+    g_object_set(args->gmodel, "unit-y", unit, NULL);
     g_object_unref(unit);
 }
 
