@@ -1,5 +1,5 @@
 /*
- *  $Id: grain_dist.c 27037 2024-12-11 15:03:44Z yeti-dn $
+ *  $Id: grain_dist.c 28978 2025-12-11 10:41:31Z yeti-dn $
  *  Copyright (C) 2003-2021 David Necas (Yeti), Petr Klapetek, Sven Neumann.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net, neumann@jpk.com.
  *
@@ -19,17 +19,11 @@
  * incompatible quantities? */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libprocess/grains.h>
-#include <libprocess/linestats.h>
-#include <libgwydgets/gwygrainvaluemenu.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
+#include <gwy.h>
 
 #define RUN_MODES (GWY_RUN_INTERACTIVE | GWY_RUN_IMMEDIATE)
 
@@ -58,11 +52,10 @@ typedef struct {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *mask;
+    GwyField *field;
+    GwyNield *grains;
     /* Cached precalculated data. */
     gboolean units_equal;
-    gint *grains;
     guint ngrains;
 } ModuleArgs;
 
@@ -70,7 +63,7 @@ typedef struct {
     ModuleArgs *args;
     guint nvalues;
     GwyGrainValue **gvalues;
-    GwyDataLine **rawvalues;
+    GwyLine **rawvalues;
     gboolean add_comment;
 } GrainDistExportData;
 
@@ -84,10 +77,10 @@ typedef struct {
 
 static gboolean         module_register            (void);
 static GwyParamDef*     define_module_params       (void);
-static void             grain_dist                 (GwyContainer *data,
-                                                    GwyRunType runtype);
+static void             module_main                (GwyFile *data,
+                                                    GwyRunModeFlags mode);
 static void             execute                    (ModuleArgs *args,
-                                                    GwyContainer *data);
+                                                    GwyFile *data);
 static GwyDialogOutcome run_gui                    (ModuleArgs *args);
 static void             param_changed              (ModuleGUI *gui,
                                                     gint id);
@@ -99,8 +92,10 @@ static void             row_expanded_collapsed     (ModuleGUI *gui);
 static void             add_one_distribution       (GwyGraphModel *gmodel,
                                                     GrainDistExportData *expdata,
                                                     guint i);
-static gchar*           grain_dist_export_create   (gpointer user_data,
+static gchar*           create_raw_dump            (gpointer user_data,
                                                     gssize *data_len);
+static void             free_raw_dump              (gchar *data,
+                                                    gpointer user_data);
 static gchar*           rectify_grain_quantity_list(const gchar *s);
 
 static GwyModuleInfo module_info = {
@@ -119,11 +114,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("grain_dist",
-                              (GwyProcessFunc)&grain_dist,
+                              module_main,
                               N_("/_Grains/_Distributions..."),
-                              GWY_STOCK_GRAINS_GRAPH,
+                              GWY_ICON_GRAINS_GRAPH,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA | GWY_MENU_FLAG_DATA_MASK,
+                              GWY_MENU_FLAG_IMAGE | GWY_MENU_FLAG_IMAGE_MASK,
                               N_("Distributions of various grain characteristics"));
 
     return TRUE;
@@ -155,24 +150,24 @@ define_module_params(void)
 }
 
 static void
-grain_dist(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     ModuleArgs args;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_MASK_FIELD, &args.mask,
-                                     0);
-    g_return_if_fail(args.field && args.mask);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 /* TODO Nield */
+                                 GWY_APP_MASK_FIELD, &args.grains,
+                                 0);
+    g_return_if_fail(args.field && args.grains);
 
-    args.units_equal = gwy_si_unit_equal(gwy_data_field_get_si_unit_xy(args.field),
-                                         gwy_data_field_get_si_unit_z(args.field));
-    args.grains = g_new0(gint, gwy_data_field_get_xres(args.mask) * gwy_data_field_get_yres(args.mask));
-    args.ngrains = gwy_data_field_number_grains(args.mask, args.grains);
+    args.units_equal = gwy_unit_equal(gwy_field_get_unit_xy(args.field),
+                                      gwy_field_get_unit_z(args.field));
+    args.ngrains = gwy_nield_max(args.grains);
     args.params = gwy_params_new_from_settings(define_module_params());
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -181,7 +176,6 @@ grain_dist(GwyContainer *data, GwyRunType runtype)
     execute(&args, data);
 
 end:
-    g_free(args.grains);
     g_object_unref(args.params);
 }
 
@@ -205,7 +199,7 @@ run_gui(ModuleArgs *args)
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_CLEAR, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
     gtk_window_set_default_size(GTK_WINDOW(dialog), -1, 520);
 
-    hbox = gwy_hbox_new(0);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
     gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
     gwy_dialog_add_content(dialog, hbox, TRUE, TRUE, 0);
 
@@ -215,7 +209,7 @@ run_gui(ModuleArgs *args)
     gwy_graph_enable_user_input(GWY_GRAPH(graph), FALSE);
     gtk_box_pack_start(GTK_BOX(hbox), graph, TRUE, TRUE, 4);
 
-    vbox = gwy_vbox_new(2);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 2);
     gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 4);
 
     scwin = gtk_scrolled_window_new(NULL, NULL);
@@ -326,7 +320,7 @@ preview(gpointer user_data)
     GtkTreeModel *model;
     GrainDistExportData expdata;
     GwyGrainValue *gvalue;
-    GwyDataLine *dline;
+    GwyLine *dline;
     GtkTreeIter iter;
     gdouble *d;
 
@@ -338,10 +332,10 @@ preview(gpointer user_data)
     expdata.args = args;
     expdata.nvalues = 1;
     expdata.gvalues = &gvalue;
-    dline = gwy_data_line_new(args->ngrains+1, 1.0, FALSE);
-    d = gwy_data_line_get_data(dline);
+    dline = gwy_line_new(args->ngrains+1, 1.0, FALSE);
+    d = gwy_line_get_data(dline);
     expdata.rawvalues = &dline;
-    gwy_grain_values_calculate(1, expdata.gvalues, &d, args->field, args->ngrains, args->grains);
+    gwy_grain_values_calculate(expdata.gvalues, &d, 1, args->field, args->grains);
     add_one_distribution(gui->gmodel, &expdata, 0);
     g_object_unref(dline);
 }
@@ -350,50 +344,50 @@ static void
 add_one_distribution(GwyGraphModel *gmodel, GrainDistExportData *expdata, guint i)
 {
     GwyParams *params = expdata->args->params;
-    GwyDataField *field = expdata->args->field;
+    GwyField *field = expdata->args->field;
     GwyGraphCurveModel *cmodel;
-    GwyDataLine *dline, *distribution;
+    GwyLine *dline, *distribution;
     GwyGrainValue *gvalue;
     const gchar *name;
     gint res;
 
     dline = expdata->rawvalues[i];
     gvalue = expdata->gvalues[i];
-    gwy_si_unit_power_multiply(gwy_data_field_get_si_unit_xy(field), gwy_grain_value_get_power_xy(gvalue),
-                               gwy_data_field_get_si_unit_z(field), gwy_grain_value_get_power_z(gvalue),
-                               gwy_data_line_get_si_unit_y(dline));
+    gwy_unit_power_multiply(gwy_field_get_unit_xy(field), gwy_grain_value_get_power_xy(gvalue),
+                            gwy_field_get_unit_z(field), gwy_grain_value_get_power_z(gvalue),
+                            gwy_line_get_unit_y(dline));
     /* Get rid of the zeroth bogus item corresponding to no grain. */
-    gwy_data_line_resize(dline, 1, gwy_data_line_get_res(dline));
+    gwy_line_crop(dline, 1, gwy_line_get_res(dline)-1);
     res = gwy_params_get_boolean(params, PARAM_FIXRES) ? gwy_params_get_int(params, PARAM_RESOLUTION) : 0;
-    distribution = gwy_data_line_new(res ? res : 1, 1.0, FALSE);
-    gwy_data_line_distribution(dline, distribution, 0.0, 0.0, FALSE, res);
-    /* Make the values centered in bins.  Changing gwy_data_line_distribution() to do that itself would be
-     * incompatible and of course changing gwy_graph_curve_model_set_data_from_dataline() is impossible. */
-    gwy_data_line_set_offset(distribution, distribution->off + 0.5*distribution->real/distribution->res);
+    distribution = gwy_line_new(res ? res : 1, 1.0, FALSE);
+    gwy_line_distribution(dline, distribution, 0.0, 0.0, FALSE, res);
+    /* Make the values centered in bins.  Changing gwy_line_distribution() to do that itself would be
+     * incompatible and of course changing gwy_graph_curve_model_set_data_from_line() is impossible. */
+    gwy_line_set_offset(distribution, distribution->off + 0.5*distribution->real/distribution->res);
 
     cmodel = gwy_graph_curve_model_new();
     gwy_graph_model_add_curve(gmodel, cmodel);
     g_object_unref(cmodel);
 
-    name = gettext(gwy_resource_get_name(GWY_RESOURCE(gvalue)));
+    name = _(gwy_resource_get_name(GWY_RESOURCE(gvalue)));
     g_object_set(gmodel,
                  "title", name,
                  "axis-label-left", _("count"),
                  "axis-label-bottom", gwy_grain_value_get_symbol_markup(gvalue),
                  NULL);
-    gwy_graph_model_set_units_from_data_line(gmodel, distribution);
+    gwy_graph_model_set_units_from_line(gmodel, distribution);
     g_object_set(cmodel, "description", name, NULL);
-    gwy_graph_curve_model_set_data_from_dataline(cmodel, distribution, 0, 0);
+    gwy_graph_curve_model_set_data_from_line(cmodel, distribution, 0, 0);
     g_object_unref(distribution);
 }
 
 static void
-execute(ModuleArgs *args, GwyContainer *data)
+execute(ModuleArgs *args, GwyFile *data)
 {
     GrainDistMode mode = gwy_params_get_enum(args->params, PARAM_MODE);
     GrainDistExportData expdata;
     GwyGrainValue *gvalue;
-    GwyDataLine *dline;
+    GwyLine *dline;
     gchar **selected_quantities;
     guint i, nvalues;
     gdouble **results;
@@ -402,25 +396,25 @@ execute(ModuleArgs *args, GwyContainer *data)
     selected_quantities = g_strsplit(gwy_params_get_string(args->params, PARAM_SELECTED), "\n", 0);
     nvalues = g_strv_length(selected_quantities);
     expdata.gvalues = g_new(GwyGrainValue*, nvalues);
-    expdata.rawvalues = g_new(GwyDataLine*, nvalues);
+    expdata.rawvalues = g_new(GwyLine*, nvalues);
     expdata.add_comment = gwy_params_get_boolean(args->params, PARAM_ADD_COMMENT);
     results = g_new(gdouble*, nvalues);
     for (nvalues = i = 0; selected_quantities[i]; i++) {
         gvalue = gwy_grain_values_get_grain_value(selected_quantities[nvalues]);
         if (!gvalue)
             continue;
-        if (!args->units_equal && (gwy_grain_value_get_flags(gvalue) & GWY_GRAIN_VALUE_SAME_UNITS))
+        if (!args->units_equal && (gwy_grain_value_get_flags(gvalue) & GWY_GRAIN_SAME_UNITS))
             continue;
         expdata.gvalues[nvalues] = gvalue;
-        dline = gwy_data_line_new(args->ngrains+1, 1.0, FALSE);
+        dline = gwy_line_new(args->ngrains+1, 1.0, FALSE);
         expdata.rawvalues[nvalues] = dline;
-        results[nvalues] = gwy_data_line_get_data(dline);
+        results[nvalues] = gwy_line_get_data(dline);
         nvalues++;
     }
     expdata.nvalues = nvalues;
     g_strfreev(selected_quantities);
 
-    gwy_grain_values_calculate(nvalues, expdata.gvalues, results, args->field, args->ngrains, args->grains);
+    gwy_grain_values_calculate(expdata.gvalues, results, nvalues, args->field, args->grains);
     g_free(results);
 
     if (mode == MODE_GRAPH) {
@@ -428,13 +422,14 @@ execute(ModuleArgs *args, GwyContainer *data)
             GwyGraphModel *gmodel = gwy_graph_model_new();
 
             add_one_distribution(gmodel, &expdata, i);
-            gwy_app_data_browser_add_graph_model(gmodel, data, TRUE);
+            gint newid = gwy_file_add_graph(data, gmodel);
+            gwy_file_set_visible(data, GWY_FILE_GRAPH, newid, TRUE);
             g_object_unref(gmodel);
         }
     }
     else if (mode == MODE_RAW) {
         gwy_save_auxiliary_with_callback(_("Export Raw Grain Values"), NULL,
-                                         grain_dist_export_create, (GwySaveAuxiliaryDestroy)g_free, &expdata);
+                                         create_raw_dump, free_raw_dump, FALSE, &expdata);
     }
     else {
         g_assert_not_reached();
@@ -446,8 +441,14 @@ execute(ModuleArgs *args, GwyContainer *data)
     g_free(expdata.gvalues);
 }
 
+static void
+free_raw_dump(gchar *data, G_GNUC_UNUSED gpointer user_data)
+{
+    g_free(data);
+}
+
 static gchar*
-grain_dist_export_create(gpointer user_data, gssize *data_len)
+create_raw_dump(gpointer user_data, gssize *data_len)
 {
     const GrainDistExportData *expdata = (const GrainDistExportData*)user_data;
     GString *report;
@@ -457,7 +458,7 @@ grain_dist_export_create(gpointer user_data, gssize *data_len)
     gdouble val;
 
     if (expdata->nvalues)
-        ngrains = gwy_data_line_get_res(expdata->rawvalues[0]) - 1;
+        ngrains = gwy_line_get_res(expdata->rawvalues[0]) - 1;
 
     report = g_string_sized_new(12*ngrains*expdata->nvalues);
 
@@ -472,7 +473,7 @@ grain_dist_export_create(gpointer user_data, gssize *data_len)
 
     for (gno = 1; gno <= ngrains; gno++) {
         for (i = 0; i < expdata->nvalues; i++) {
-            val = gwy_data_line_get_val(expdata->rawvalues[i], gno);
+            val = gwy_line_get_val(expdata->rawvalues[i], gno);
             g_ascii_formatd(buffer, sizeof(buffer), "%g", val);
             g_string_append(report, buffer);
             g_string_append_c(report, i == expdata->nvalues-1 ? '\n' : '\t');
