@@ -1,5 +1,5 @@
 /*
- *  $Id: grain_edge.c 23666 2021-05-10 21:56:10Z yeti-dn $
+ *  $Id: grain_edge.c 28910 2025-11-24 18:11:09Z yeti-dn $
  *  Copyright (C) 2011-2021 David Necas (Yeti), Petr Klapetek, Daniil Bratashov.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net, dn2010@gmail.com.
  *
@@ -16,16 +16,9 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/filters.h>
-#include <libprocess/grains.h>
-#include <libprocess/stats.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
@@ -40,25 +33,24 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *mask;
-    GwyDataField *result;
+    GwyField *field;
+    GwyField *mask;
+    GwyField *result;
 } ModuleArgs;
 
 typedef struct {
     ModuleArgs *args;
     GtkWidget *dialog;
     GwyParamTable *table;
-    GwyContainer *data;
 } ModuleGUI;
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             grain_edge          (GwyContainer *data,
-                                             GwyRunType runtype);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static void             execute             (ModuleArgs *args);
 static GwyDialogOutcome run_gui             (ModuleArgs *args,
-                                             GwyContainer *data,
+                                             GwyFile *data,
                                              gint id);
 static void             param_changed       (ModuleGUI *gui,
                                              gint id);
@@ -80,11 +72,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("grain_edge",
-                              (GwyProcessFunc)&grain_edge,
+                              module_main,
                               N_("/_Grains/Mark by _Edge Detection..."),
-                              GWY_STOCK_GRAINS_EDGE,
+                              GWY_ICON_GRAINS_EDGE,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Mark grains with edge detection mechanism"));
 
     return TRUE;
@@ -109,26 +101,26 @@ define_module_params(void)
 }
 
 static void
-grain_edge(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     GQuark mquark;
     gint id;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     GWY_APP_MASK_FIELD_KEY, &mquark,
-                                     GWY_APP_MASK_FIELD, &args.mask,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_FIELD_ID, &id,
+                                 GWY_APP_MASK_FIELD_KEY, &mquark,
+                                 GWY_APP_MASK_FIELD, &args.mask,
+                                 0);
     g_return_if_fail(args.field && mquark);
 
-    args.result = gwy_data_field_new_alike(args.field, TRUE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(args.result), NULL);
+    args.result = gwy_field_new_alike(args.field, TRUE);
+    gwy_unit_clear(gwy_field_get_unit_z(args.result));
     args.params = gwy_params_new_from_settings(define_module_params());
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, id);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -137,12 +129,12 @@ grain_edge(GwyContainer *data, GwyRunType runtype)
     if (outcome != GWY_DIALOG_HAVE_RESULT)
         execute(&args);
 
-    gwy_app_undo_qcheckpointv(data, 1, &mquark);
-    if (gwy_data_field_get_max(args.result) > 0.0)
-        gwy_container_set_object(data, mquark, args.result);
+    gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &mquark);
+    if (gwy_field_get_max(args.result) > 0.0)
+        gwy_container_set_object(GWY_CONTAINER(data), mquark, args.result);
     else
-        gwy_container_remove(data, mquark);
-    gwy_app_channel_log_add_proc(data, id, id);
+        gwy_container_remove(GWY_CONTAINER(data), mquark);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, id);
 
 end:
     g_object_unref(args.params);
@@ -150,38 +142,31 @@ end:
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
-    GwyDialogOutcome outcome;
     GtkWidget *hbox, *dataview;
     GwyDialog *dialog;
     GwyParamTable *table;
     ModuleGUI gui;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
-    gui.data = gwy_container_new();
-
-    gwy_container_set_object_by_name(gui.data, "/0/data", args->field);
-    gwy_container_set_object_by_name(gui.data, "/0/mask", args->result);
-    gwy_app_sync_data_items(data, gui.data, id, 0, FALSE,
-                            GWY_DATA_ITEM_PALETTE,
-                            GWY_DATA_ITEM_RANGE,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
 
     gui.dialog = gwy_dialog_new(_("Mark Grains by Edge Detection"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_UPDATE, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, TRUE);
+    dataview = gwy_create_preview(args->field, args->result, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_MASK_COLOR
+                        | GWY_FILE_ITEM_RANGE | GWY_FILE_ITEM_REAL_SQUARE);
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), FALSE);
 
     table = gui.table = gwy_param_table_new(args->params);
     gwy_param_table_append_header(table, -1, _("Threshold"));
     gwy_param_table_append_slider(table, PARAM_THRESHOLD_LAPLACIAN);
     gwy_param_table_append_header(table, -1, _("Options"));
-    gwy_param_table_append_mask_color(table, PARAM_MASK_COLOR, gui.data, 0, data, id);
+    gwy_param_table_append_image_mask_color(table, PARAM_MASK_COLOR, GWY_DATA_VIEW(dataview), data, id);
     if (args->mask) {
         gwy_param_table_append_radio_buttons(table, PARAM_COMBINE_TYPE, NULL);
         gwy_param_table_add_enabler(table, PARAM_COMBINE, PARAM_COMBINE_TYPE);
@@ -194,11 +179,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     g_signal_connect_swapped(table, "param-changed", G_CALLBACK(param_changed), &gui);
     gwy_dialog_set_preview_func(dialog, GWY_PREVIEW_IMMEDIATE, preview, &gui, NULL);
 
-    outcome = gwy_dialog_run(dialog);
-
-    g_object_unref(gui.data);
-
-    return outcome;
+    return gwy_dialog_run(dialog);
 }
 
 static void
@@ -215,7 +196,7 @@ preview(gpointer user_data)
     ModuleArgs *args = gui->args;
 
     execute(args);
-    gwy_data_field_data_changed(args->result);
+    gwy_field_data_changed(args->result);
     gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
 }
 
@@ -226,18 +207,18 @@ execute(ModuleArgs *args)
     gboolean combine = gwy_params_get_boolean(params, PARAM_COMBINE);
     GwyMergeType combine_type = gwy_params_get_enum(params, PARAM_COMBINE_TYPE);
     gdouble threshold_laplacian = gwy_params_get_double(params, PARAM_THRESHOLD_LAPLACIAN);
-    GwyDataField *temp_field, *mask = args->mask;
+    GwyField *temp_field, *mask = args->mask;
 
-    temp_field = gwy_data_field_duplicate(args->field);
-    gwy_data_field_filter_laplacian_of_gaussians(temp_field);
-    gwy_data_field_grains_mark_height(temp_field, args->result, 100.0*threshold_laplacian, TRUE);
+    temp_field = gwy_field_copy(args->field);
+    gwy_field_filter_laplacian_of_gaussians(temp_field);
+    gwy_field_grains_mark_height(temp_field, args->result, 100.0*threshold_laplacian, TRUE);
     g_object_unref(temp_field);
 
     if (mask && combine) {
         if (combine_type == GWY_MERGE_UNION)
-            gwy_data_field_grains_add(args->result, mask);
+            gwy_field_grains_add(args->result, mask);
         else if (combine_type == GWY_MERGE_INTERSECTION)
-            gwy_data_field_grains_intersect(args->result, mask);
+            gwy_field_grains_intersect(args->result, mask);
     }
 }
 
