@@ -1,5 +1,5 @@
 /*
- *  $Id: grain_mark.c 26238 2024-03-01 15:46:29Z yeti-dn $
+ *  $Id: grain_mark.c 29061 2026-01-02 15:01:53Z yeti-dn $
  *  Copyright (C) 2004-2024 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,15 +16,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/grains.h>
-#include <libprocess/stats.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
@@ -48,9 +43,9 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *mask;
-    GwyDataField *result;
+    GwyField *field;
+    GwyField *mask;
+    GwyField *result;
     /* Cached rank data. */
     gint *rank;
 } ModuleArgs;
@@ -59,23 +54,22 @@ typedef struct {
     ModuleArgs *args;
     GtkWidget *dialog;
     GwyParamTable *table;
-    GwyContainer *data;
 } ModuleGUI;
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             grain_mark          (GwyContainer *data,
-                                             GwyRunType runtype);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static void             execute             (ModuleArgs *args);
 static GwyDialogOutcome run_gui             (ModuleArgs *args,
-                                             GwyContainer *data,
+                                             GwyFile *data,
                                              gint id);
 static void             param_changed       (ModuleGUI *gui,
                                              gint id);
 static void             preview             (gpointer user_data);
-static gint*            calculate_ranks     (GwyDataField *field);
+static gint*            calculate_ranks     (GwyField *field);
 static void             mark_by_rank        (const gint *rank,
-                                             GwyDataField *result,
+                                             GwyField *result,
                                              gdouble rank_frac,
                                              gboolean inverted);
 
@@ -95,11 +89,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("grain_mark",
-                              (GwyProcessFunc)&grain_mark,
+                              module_main,
                               N_("/_Grains/_Mark by Threshold..."),
-                              GWY_STOCK_GRAINS,
+                              GWY_ICON_GRAINS,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Mark grains by threshold"));
 
     return TRUE;
@@ -134,27 +128,27 @@ define_module_params(void)
 }
 
 static void
-grain_mark(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     GQuark mquark;
     gint id;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     GWY_APP_MASK_FIELD_KEY, &mquark,
-                                     GWY_APP_MASK_FIELD, &args.mask,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_FIELD_ID, &id,
+                                 GWY_APP_MASK_FIELD_KEY, &mquark,
+                                 GWY_APP_MASK_FIELD, &args.mask,
+                                 0);
     g_return_if_fail(args.field && mquark);
 
-    args.result = gwy_data_field_new_alike(args.field, TRUE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(args.result), NULL);
+    args.result = gwy_field_new_alike(args.field, TRUE);
+    gwy_unit_clear(gwy_field_get_unit_z(args.result));
     args.params = gwy_params_new_from_settings(define_module_params());
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, id);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -163,12 +157,12 @@ grain_mark(GwyContainer *data, GwyRunType runtype)
     if (outcome != GWY_DIALOG_HAVE_RESULT)
         execute(&args);
 
-    gwy_app_undo_qcheckpointv(data, 1, &mquark);
-    if (gwy_data_field_get_max(args.result) > 0.0)
-        gwy_container_set_object(data, mquark, args.result);
+    gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &mquark);
+    if (gwy_field_get_max(args.result) > 0.0)
+        gwy_container_set_object(GWY_CONTAINER(data), mquark, args.result);
     else
-        gwy_container_remove(data, mquark);
-    gwy_app_channel_log_add_proc(data, id, id);
+        gwy_container_remove(GWY_CONTAINER(data), mquark);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, id);
 
 end:
     g_object_unref(args.params);
@@ -177,31 +171,23 @@ end:
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
-    GwyDialogOutcome outcome;
     GtkWidget *hbox, *dataview;
     GwyDialog *dialog;
     GwyParamTable *table;
     ModuleGUI gui;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
-    gui.data = gwy_container_new();
-
-    gwy_container_set_object_by_name(gui.data, "/0/data", args->field);
-    gwy_container_set_object_by_name(gui.data, "/0/mask", args->result);
-    gwy_app_sync_data_items(data, gui.data, id, 0, FALSE,
-                            GWY_DATA_ITEM_PALETTE,
-                            GWY_DATA_ITEM_RANGE,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
 
     gui.dialog = gwy_dialog_new(_("Mark Grains by Threshold"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_UPDATE, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, TRUE);
+    dataview = gwy_create_preview(args->field, args->result, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_RANGE | GWY_FILE_ITEM_REAL_SQUARE);
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), FALSE);
 
     table = gui.table = gwy_param_table_new(args->params);
@@ -222,7 +208,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_param_table_append_radio_buttons(table, PARAM_MERGE_TYPE, NULL);
 
     gwy_param_table_append_header(table, -1, _("Options"));
-    gwy_param_table_append_mask_color(table, PARAM_MASK_COLOR, gui.data, 0, data, id);
+    gwy_param_table_append_image_mask_color(table, PARAM_MASK_COLOR, GWY_DATA_VIEW(dataview), data, id);
     if (args->mask) {
         gwy_param_table_append_radio_buttons(table, PARAM_COMBINE_TYPE, NULL);
         gwy_param_table_add_enabler(table, PARAM_COMBINE, PARAM_COMBINE_TYPE);
@@ -235,11 +221,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     g_signal_connect_swapped(table, "param-changed", G_CALLBACK(param_changed), &gui);
     gwy_dialog_set_preview_func(dialog, GWY_PREVIEW_IMMEDIATE, preview, &gui, NULL);
 
-    outcome = gwy_dialog_run(dialog);
-
-    g_object_unref(gui.data);
-
-    return outcome;
+    return gwy_dialog_run(dialog);
 }
 
 static void
@@ -256,17 +238,17 @@ preview(gpointer user_data)
     ModuleArgs *args = gui->args;
 
     execute(args);
-    gwy_data_field_data_changed(args->result);
+    gwy_field_data_changed(args->result);
     gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
 }
 
 static void
-combine_masks(GwyDataField *result, GwyDataField *operand, GwyMergeType merge_type)
+combine_masks(GwyField *result, GwyField *operand, GwyMergeType merge_type)
 {
     if (merge_type == GWY_MERGE_UNION)
-        gwy_data_field_grains_add(result, operand);
+        gwy_field_grains_add(result, operand);
     else if (merge_type == GWY_MERGE_INTERSECTION)
-        gwy_data_field_grains_intersect(result, operand);
+        gwy_field_grains_intersect(result, operand);
 }
 
 static void
@@ -280,18 +262,18 @@ execute(ModuleArgs *args)
     gboolean inverted = gwy_params_get_boolean(params, PARAM_INVERTED);
     GwyMergeType merge_type = gwy_params_get_enum(params, PARAM_MERGE_TYPE);
     GwyMergeType combine_type = gwy_params_get_enum(params, PARAM_COMBINE_TYPE);
-    GwyDataField *field = args->field, *mask = args->mask, *result = args->result, *tmp = NULL;
+    GwyField *field = args->field, *mask = args->mask, *result = args->result, *tmp = NULL;
     gboolean field_been_set = FALSE;
 
     if (gwy_params_get_boolean(params, PARAM_IS_HEIGHT)) {
-        gwy_data_field_grains_mark_height(field, result, height_perc, inverted);
+        gwy_field_grains_mark_height(field, result, height_perc, inverted);
         field_been_set = TRUE;
     }
     if (gwy_params_get_boolean(params, PARAM_IS_RANK)) {
         if (!args->rank)
             args->rank = calculate_ranks(field);
         if (field_been_set) {
-            tmp = gwy_data_field_new_alike(result, FALSE);
+            tmp = gwy_field_new_alike(result, FALSE);
             mark_by_rank(args->rank, tmp, rank_frac, inverted);
             combine_masks(result, tmp, merge_type);
         }
@@ -301,53 +283,53 @@ execute(ModuleArgs *args)
     }
     if (gwy_params_get_boolean(params, PARAM_IS_SLOPE)) {
         if (field_been_set) {
-            tmp = gwy_data_field_new_alike(result, FALSE);
-            gwy_data_field_grains_mark_slope(field, tmp, slope_perc, FALSE);
+            tmp = gwy_field_new_alike(result, FALSE);
+            gwy_field_grains_mark_slope(field, tmp, slope_perc, FALSE);
             combine_masks(result, tmp, merge_type);
         }
         else
-            gwy_data_field_grains_mark_slope(field, result, slope_perc, FALSE);
+            gwy_field_grains_mark_slope(field, result, slope_perc, FALSE);
         field_been_set = TRUE;
     }
     if (gwy_params_get_boolean(params, PARAM_IS_LAP)) {
         if (field_been_set) {
             if (!tmp)
-                tmp = gwy_data_field_new_alike(result, FALSE);
-            gwy_data_field_grains_mark_curvature(field, tmp, curvature_perc, FALSE);
+                tmp = gwy_field_new_alike(result, FALSE);
+            gwy_field_grains_mark_curvature(field, tmp, curvature_perc, FALSE);
             combine_masks(result, tmp, merge_type);
         }
         else
-            gwy_data_field_grains_mark_curvature(field, result, curvature_perc, FALSE);
+            gwy_field_grains_mark_curvature(field, result, curvature_perc, FALSE);
         field_been_set = TRUE;
     }
     if (!field_been_set)
-        gwy_data_field_clear(result);
+        gwy_field_clear(result);
     if (mask && gwy_params_get_boolean(params, PARAM_COMBINE))
         combine_masks(result, mask, combine_type);
 
-    GWY_OBJECT_UNREF(tmp);
+    g_clear_object(&tmp);
 }
 
 static gint*
-calculate_ranks(GwyDataField *field)
+calculate_ranks(GwyField *field)
 {
-    gint i, n = gwy_data_field_get_xres(field)*gwy_data_field_get_yres(field);
+    gint i, n = gwy_field_get_xres(field)*gwy_field_get_yres(field);
     gdouble *d = g_new(gdouble, n);
     gint *rank = g_new(gint, n);
 
     for (i = 0; i < n; i++)
         rank[i] = i;
-    gwy_assign(d, gwy_data_field_get_data_const(field), n);
-    gwy_math_sort_with_index(n, d, rank);
+    gwy_assign(d, gwy_field_get_data_const(field), n);
+    gwy_math_sort_with_index(d, n, rank);
 
     return rank;
 }
 
 static void
-mark_by_rank(const gint *rank, GwyDataField *result,
+mark_by_rank(const gint *rank, GwyField *result,
              gdouble rank_frac, gboolean inverted)
 {
-    gint n = gwy_data_field_get_xres(result)*gwy_data_field_get_yres(result);
+    gint n = gwy_field_get_xres(result)*gwy_field_get_yres(result);
     gint i, ifrom, ito;
     gdouble *d;
 
@@ -359,8 +341,8 @@ mark_by_rank(const gint *rank, GwyDataField *result,
         ifrom = MIN(n, GWY_ROUND(rank_frac*n) + 1);
         ito = n;
     }
-    gwy_data_field_clear(result);
-    d = gwy_data_field_get_data(result);
+    gwy_field_clear(result);
+    d = gwy_field_get_data(result);
     for (i = ifrom; i < ito; i++)
         d[rank[i]] = 1.0;
 }
