@@ -1,5 +1,5 @@
 /*
- *  $Id: grain_stat.c 23666 2021-05-10 21:56:10Z yeti-dn $
+ *  $Id: grain_stat.c 29061 2026-01-02 15:01:53Z yeti-dn $
  *  Copyright (C) 2014-2021 David Necas (Yeti), Petr Klapetek, Sven Neumann.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net, neumann@jpk.com.
  *
@@ -16,16 +16,11 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/grains.h>
-#include <libgwydgets/gwygrainvaluemenu.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
+#include <gwy.h>
 
 #define RUN_MODES GWY_RUN_INTERACTIVE
 
@@ -41,13 +36,13 @@ typedef struct {
     gdouble rms;
     gdouble q25;
     gdouble q75;
-    GwySIValueFormat *vf;   /* For the treeview */
+    GwyValueFormat *vf;   /* For the treeview */
 } GrainQuantityStats;
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *mask;
+    GwyField *field;
+    GwyNield *grains;
     /* Cached input image parameters. */
     gboolean same_units;
 } ModuleArgs;
@@ -58,13 +53,12 @@ typedef struct {
     GtkWidget *dialog;
     GwyParamTable *table;
     GtkWidget *treeview;
-    GwyContainer *data;
 } ModuleGUI;
 
 static gboolean            module_register        (void);
 static GwyParamDef*        define_module_params   (void);
-static void                grain_stat             (GwyContainer *data,
-                                                   GwyRunType runtype);
+static void                module_main            (GwyFile *data,
+                                                   GwyRunModeFlags mode);
 static GwyDialogOutcome    run_gui                (ModuleArgs *args);
 static void                row_expanded_collapsed (ModuleGUI *gui);
 static void                render_grain_stat      (GtkTreeViewColumn *column,
@@ -72,8 +66,8 @@ static void                render_grain_stat      (GtkTreeViewColumn *column,
                                                    GtkTreeModel *model,
                                                    GtkTreeIter *iter,
                                                    gpointer user_data);
-static GrainQuantityStats* calculate_stats        (GwyDataField *field,
-                                                   GwyDataField *mask);
+static GrainQuantityStats* calculate_stats        (GwyField *field,
+                                                   GwyNield *grains);
 static gdouble             calc_average           (gdouble *values,
                                                    guint n);
 static gdouble             calc_rms               (const gdouble *values,
@@ -110,11 +104,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("grain_stat",
-                              (GwyProcessFunc)&grain_stat,
+                              module_main,
                               N_("/_Grains/S_tatistics..."),
-                              GWY_STOCK_GRAINS_STATISTICS,
+                              GWY_ICON_GRAINS_STATISTICS,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA | GWY_MENU_FLAG_DATA_MASK,
+                              GWY_MENU_FLAG_IMAGE | GWY_MENU_FLAG_IMAGE_MASK,
                               N_("Grain property statistics"));
 
     return TRUE;
@@ -137,17 +131,17 @@ define_module_params(void)
 }
 
 static void
-grain_stat(G_GNUC_UNUSED GwyContainer *data, GwyRunType runtype)
+module_main(G_GNUC_UNUSED GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_MASK_FIELD, &args.mask,
-                                     0);
-    g_return_if_fail(args.field && args.mask);
-    args.same_units = gwy_si_unit_equal(gwy_data_field_get_si_unit_xy(args.field),
-                                        gwy_data_field_get_si_unit_z(args.field));
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 /* TODO Nield */
+                                 GWY_APP_MASK_FIELD, &args.grains,
+                                 0);
+    g_return_if_fail(args.field && args.grains);
+    args.same_units = gwy_unit_equal(gwy_field_get_unit_xy(args.field), gwy_field_get_unit_z(args.field));
     args.params = gwy_params_new_from_settings(define_module_params());
     run_gui(&args);
     gwy_params_save_to_settings(args.params);
@@ -172,7 +166,7 @@ run_gui(ModuleArgs *args)
     guint n, i;
 
     gui.args = args;
-    gui.stats = calculate_stats(args->field, args->mask);
+    gui.stats = calculate_stats(args->field, args->grains);
 
     gui.dialog = gwy_dialog_new(_("Grain Statistics"));
     dialog = GWY_DIALOG(gui.dialog);
@@ -210,7 +204,7 @@ run_gui(ModuleArgs *args)
     gwy_param_table_append_report(table, PARAM_REPORT_STYLE);
     gwy_param_table_report_set_formatter(table, PARAM_REPORT_STYLE, format_report, &gui, NULL);
     /* XXX: Silly.  Just want to right-align the export controls for consistency. */
-    auxbox = gwy_hbox_new(0);
+    auxbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
     gwy_dialog_add_content(dialog, auxbox, FALSE, FALSE, 0);
     gtk_box_pack_end(GTK_BOX(auxbox), gwy_param_table_widget(table), FALSE, FALSE, 0);
     gwy_dialog_add_param_table(dialog, table);
@@ -223,7 +217,7 @@ run_gui(ModuleArgs *args)
 
     n = gwy_inventory_get_n_items(gwy_grain_values());
     for (i = 0; i < n; i++)
-        gwy_si_unit_value_format_free(gui.stats[i].vf);
+        gwy_value_format_free(gui.stats[i].vf);
     g_free(gui.stats);
 
     return outcome;
@@ -250,14 +244,14 @@ render_grain_stat(GtkTreeViewColumn *column, GtkCellRenderer *renderer,
     const gchar *name;
     gint i, id;
 
-    gtk_tree_model_get(model, iter, GWY_GRAIN_VALUE_STORE_COLUMN_ITEM, &gvalue, -1);
+    gtk_tree_model_get(model, iter, GWY_GRAIN_STORE_COLUMN_ITEM, &gvalue, -1);
     if (!gvalue) {
         g_object_set(renderer, "text", "", NULL);
         return;
     }
 
     g_object_unref(gvalue);
-    if (!gui->args->same_units && (gwy_grain_value_get_flags(gvalue) & GWY_GRAIN_VALUE_SAME_UNITS)) {
+    if (!gui->args->same_units && (gwy_grain_value_get_flags(gvalue) & GWY_GRAIN_SAME_UNITS)) {
         g_object_set(renderer, "text", _("N.A."), NULL);
         return;
     }
@@ -291,25 +285,21 @@ render_grain_stat(GtkTreeViewColumn *column, GtkCellRenderer *renderer,
 }
 
 static GrainQuantityStats*
-calculate_stats(GwyDataField *field, GwyDataField *mask)
+calculate_stats(GwyField *field, GwyNield *grains)
 {
-    GwySIUnitFormatStyle style = GWY_SI_UNIT_FORMAT_VFMARKUP;
+    GwyUnitFormatStyle style = GWY_UNIT_FORMAT_VFMARKUP;
     GwyInventory *inventory;
     GwyGrainValue **gvalues;
     GrainQuantityStats *stats, *stat;
     GwyGrainQuantity quantity;
-    GwySIUnit *siunitxy, *siunitz, *siunit;
-    guint n, i, j, xres, yres, ngrains;
+    GwyUnit *siunitxy, *siunitz, *siunit;
+    guint n, i, j, ngrains;
     gdouble p[3], pv[3], v;
-    gint *grains;
     gint powerxy, powerz;
     gboolean is_angle;
     gdouble **values;
 
-    xres = gwy_data_field_get_xres(mask);
-    yres = gwy_data_field_get_yres(mask);
-    grains = g_new0(gint, xres*yres);
-    ngrains = gwy_data_field_number_grains(mask, grains);
+    ngrains = gwy_nield_max(grains);
 
     inventory = gwy_grain_values();
     n = gwy_inventory_get_n_items(inventory);
@@ -321,8 +311,7 @@ calculate_stats(GwyDataField *field, GwyDataField *mask)
         values[i] = g_new(gdouble, ngrains + 1);
     }
 
-    gwy_grain_values_calculate(n, gvalues, values, field, ngrains, grains);
-    g_free(grains);
+    gwy_grain_values_calculate(gvalues, values, n, field, grains);
 
     stats = g_new(GrainQuantityStats, n);
     p[0] = 25.0;
@@ -332,7 +321,7 @@ calculate_stats(GwyDataField *field, GwyDataField *mask)
     for (i = 0; i < n; i++) {
         stat = stats + i;
         stat->gvalue = gvalues[i];
-        is_angle = (gwy_grain_value_get_flags(stat->gvalue) & GWY_GRAIN_VALUE_IS_ANGLE);
+        is_angle = (gwy_grain_value_get_flags(stat->gvalue) & GWY_GRAIN_IS_ANGLE);
         /* Exclude the zeroth value of the array as no-grain. */
         if (is_angle) {
             stat->mean = calc_semicirc_average(values[i]+1, ngrains);
@@ -341,7 +330,7 @@ calculate_stats(GwyDataField *field, GwyDataField *mask)
             calc_semicirc_quartiles(values[i]+1, ngrains, j, &stat->q25, &stat->q75);
         }
         else {
-            gwy_math_percentiles(ngrains, values[i]+1, GWY_PERCENTILE_INTERPOLATION_LINEAR, 3, p, pv);
+            gwy_math_percentiles(values[i] + 1, ngrains, GWY_PERCENTILE_INTERPOLATION_LINEAR, p, 3, pv);
             stat->q25 = pv[0];
             stat->median = pv[1];
             stat->q75 = pv[2];
@@ -350,26 +339,26 @@ calculate_stats(GwyDataField *field, GwyDataField *mask)
         }
 
         quantity = gwy_grain_value_get_quantity(stat->gvalue);
-        if (quantity == GWY_GRAIN_VALUE_PIXEL_AREA) {
-            stat->vf = gwy_si_unit_value_format_new(1.0, 1, _("px<sup>2</sup>"));
+        if (quantity == GWY_GRAIN_PIXEL_AREA) {
+            stat->vf = gwy_value_format_new(1.0, 1, _("px<sup>2</sup>"));
             continue;
         }
         if (is_angle) {
-            stat->vf = gwy_si_unit_value_format_new(G_PI/180.0, 2, _("deg"));
+            stat->vf = gwy_value_format_new(G_PI/180.0, 2, _("deg"));
             continue;
         }
 
-        siunitxy = gwy_data_field_get_si_unit_xy(field);
-        siunitz = gwy_data_field_get_si_unit_z(field);
+        siunitxy = gwy_field_get_unit_xy(field);
+        siunitz = gwy_field_get_unit_z(field);
         powerxy = gwy_grain_value_get_power_xy(stat->gvalue);
         powerz = gwy_grain_value_get_power_z(stat->gvalue);
-        siunit = gwy_si_unit_power_multiply(siunitxy, powerxy, siunitz, powerz, siunit);
+        siunit = gwy_unit_power_multiply(siunitxy, powerxy, siunitz, powerz, siunit);
         v = fmax(fabs(stat->mean), fabs(stat->median));
         v = fmax(v, stat->rms);
         v = fmax(v, stat->q75 - stat->q25);
-        stat->vf = gwy_si_unit_get_format_with_digits(siunit, style, v, 3, NULL);
+        stat->vf = gwy_unit_get_format_with_digits(siunit, style, v, 3, NULL);
     }
-    gwy_object_unref(siunit);
+    g_clear_object(&siunit);
     g_free(values);
     g_free(gvalues);
 
@@ -379,7 +368,7 @@ calculate_stats(GwyDataField *field, GwyDataField *mask)
 static gdouble
 calc_average(gdouble *values, guint n)
 {
-    return gwy_math_trimmed_mean(n, values, 0, 0);
+    return gwy_math_mean(values, n);
 }
 
 static gdouble
@@ -446,7 +435,7 @@ calc_semicirc_median(gdouble *angles, guint n, guint *medpos)
         return angles[0];
     }
 
-    gwy_math_sort(n, angles);
+    gwy_math_sort(angles, n);
     /* Choose one value to be a speculative median at random.  Calculate the
      * sums of distances.  Find the first angle which is closer in the opposite
      * direction.  */
@@ -536,7 +525,7 @@ append_separator(GString *str, GwyResultsReportType base_type)
 }
 
 static void
-format_value(GString *str, gdouble v, GwySIValueFormat *vf)
+format_value(GString *str, gdouble v, GwyValueFormat *vf)
 {
     if (vf)
         g_string_append_printf(str, "%.*f", vf->precision, v/vf->magnitude);
@@ -554,15 +543,15 @@ format_report(gpointer user_data)
     ModuleGUI *gui = (ModuleGUI*)user_data;
     const GrainQuantityStats *stats = gui->stats;
     GwyResultsReportType report_style = gwy_params_get_report_type(gui->args->params, PARAM_REPORT_STYLE);
-    GwyDataField *field = gui->args->field;
+    GwyField *field = gui->args->field;
     GwyResultsReportType base_style;
     const GrainQuantityStats *stat;
     GwyGrainQuantity quantity;
-    GwyGrainValueGroup idgroup = GWY_GRAIN_VALUE_GROUP_ID;
-    GwySIUnit *siunit, *siunitxy, *siunitz;
-    GwySIUnitFormatStyle style;
+    GwyGrainValueGroup idgroup = GWY_GRAIN_GROUP_ID;
+    GwyUnit *siunit, *siunitxy, *siunitz;
+    GwyUnitFormatStyle style;
     gboolean is_angle, for_machine;
-    GwySIValueFormat *vf = NULL;
+    GwyValueFormat *vf = NULL;
     GString *str;
     const gchar *name, *s;
     gchar *padding = NULL, *unitstr = NULL;
@@ -575,7 +564,7 @@ format_report(gpointer user_data)
     n = gwy_inventory_get_n_items(gwy_grain_values());
     str = g_string_new(NULL);
     siunit = NULL;
-    style = (for_machine ? GWY_SI_UNIT_FORMAT_PLAIN : GWY_SI_UNIT_FORMAT_VFUNICODE);
+    style = (for_machine ? GWY_UNIT_FORMAT_PLAIN : GWY_UNIT_FORMAT_VFUNICODE);
 
     maxwidth = 0;
     if (base_style == GWY_RESULTS_REPORT_COLON) {
@@ -598,7 +587,7 @@ format_report(gpointer user_data)
         stat = stats + i;
         if (gwy_grain_value_get_group(stat->gvalue) == idgroup)
             continue;
-        is_angle = (gwy_grain_value_get_flags(stat->gvalue) & GWY_GRAIN_VALUE_IS_ANGLE);
+        is_angle = (gwy_grain_value_get_flags(stat->gvalue) & GWY_GRAIN_IS_ANGLE);
         quantity = gwy_grain_value_get_quantity(stat->gvalue);
 
         /* Name */
@@ -618,30 +607,30 @@ format_report(gpointer user_data)
         /* Value format */
         if (!for_machine && is_angle) {
             if (vf)
-                gwy_si_unit_value_format_free(vf);
-            vf = gwy_si_unit_value_format_new(G_PI/180.0, 2, _("deg"));
+                gwy_value_format_free(vf);
+            vf = gwy_value_format_new(G_PI/180.0, 2, _("deg"));
         }
         else {
-            if (quantity == GWY_GRAIN_VALUE_PIXEL_AREA) {
-                gwy_object_unref(siunit);
-                siunit = gwy_si_unit_new("px^2");
+            if (quantity == GWY_GRAIN_PIXEL_AREA) {
+                g_clear_object(&siunit);
+                siunit = gwy_unit_new("px^2");
             }
             else {
-                siunitxy = gwy_data_field_get_si_unit_xy(field);
-                siunitz = gwy_data_field_get_si_unit_z(field);
+                siunitxy = gwy_field_get_unit_xy(field);
+                siunitz = gwy_field_get_unit_z(field);
                 powerxy = gwy_grain_value_get_power_xy(stat->gvalue);
                 powerz = gwy_grain_value_get_power_z(stat->gvalue);
-                siunit = gwy_si_unit_power_multiply(siunitxy, powerxy, siunitz, powerz, siunit);
+                siunit = gwy_unit_power_multiply(siunitxy, powerxy, siunitz, powerz, siunit);
             }
             if (for_machine) {
                 g_free(unitstr);
-                unitstr = gwy_si_unit_get_string(siunit, style);
+                unitstr = gwy_unit_get_string(siunit, style);
             }
             else {
                 v = fmax(fabs(stat->mean), fabs(stat->median));
                 v = fmax(v, stat->rms);
                 v = fmax(v, stat->q75 - stat->q25);
-                vf = gwy_si_unit_get_format_with_digits(siunit, style, v, 3, vf);
+                vf = gwy_unit_get_format_with_digits(siunit, style, v, 3, vf);
             }
         }
 
@@ -679,11 +668,11 @@ format_report(gpointer user_data)
         }
         g_string_append(str, "\n");
     }
-    gwy_object_unref(siunit);
+    g_clear_object(&siunit);
     g_free(unitstr);
     g_free(padding);
     if (vf)
-        gwy_si_unit_value_format_free(vf);
+        gwy_value_format_free(vf);
 
     return g_string_free(str, FALSE);
 }
