@@ -1,5 +1,5 @@
 /*
- *  $Id: grain_wshed.c 26353 2024-05-21 08:20:43Z yeti-dn $
+ *  $Id: grain_wshed.c 28910 2025-11-24 18:11:09Z yeti-dn $
  *  Copyright (C) 2004-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,16 +16,9 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/stats.h>
-#include <libprocess/filters.h>
-#include <libprocess/grains.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
@@ -44,26 +37,25 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *mask;
-    GwyDataField *result;
+    GwyField *field;
+    GwyField *mask;
+    GwyField *result;
 } ModuleArgs;
 
 typedef struct {
     ModuleArgs *args;
     GtkWidget *dialog;
     GwyParamTable *table;
-    GwyContainer *data;
 } ModuleGUI;
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             grain_wshed         (GwyContainer *data,
-                                             GwyRunType runtype);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static gboolean         execute             (ModuleArgs *args,
                                              GtkWindow *wait_window);
 static GwyDialogOutcome run_gui             (ModuleArgs *args,
-                                             GwyContainer *data,
+                                             GwyFile *data,
                                              gint id);
 static void             param_changed       (ModuleGUI *gui,
                                              gint id);
@@ -85,11 +77,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("grain_wshed",
-                              (GwyProcessFunc)&grain_wshed,
+                              module_main,
                               N_("/_Grains/Mark by _Watershed..."),
-                              GWY_STOCK_GRAINS_WATER,
+                              GWY_ICON_GRAINS_WATER,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Mark grains by watershed"));
 
     return TRUE;
@@ -118,42 +110,42 @@ define_module_params(void)
 }
 
 static void
-grain_wshed(GwyContainer *data, GwyRunType run)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     GQuark mquark;
     gint id;
 
-    g_return_if_fail(run & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_MASK_FIELD_KEY, &mquark,
-                                     GWY_APP_MASK_FIELD, &args.mask,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_MASK_FIELD_KEY, &mquark,
+                                 GWY_APP_MASK_FIELD, &args.mask,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(args.field && mquark);
 
-    args.result = gwy_data_field_new_alike(args.field, TRUE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(args.result), NULL);
+    args.result = gwy_field_new_alike(args.field, TRUE);
+    gwy_unit_clear(gwy_field_get_unit_z(args.result));
     args.params = gwy_params_new_from_settings(define_module_params());
 
-    if (run == GWY_RUN_INTERACTIVE) {
+    if (mode == GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, id);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
             goto end;
     }
     if (outcome != GWY_DIALOG_HAVE_RESULT) {
-        if (!execute(&args, gwy_app_find_window_for_channel(data, id)))
+        if (!execute(&args, gwy_data_browser_get_window_for_data(data, GWY_FILE_IMAGE, id)))
             goto end;
     }
 
-    gwy_app_undo_qcheckpointv(data, 1, &mquark);
-    if (gwy_data_field_get_max(args.result) > 0.0)
-        gwy_container_set_object(data, mquark, args.result);
+    gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &mquark);
+    if (gwy_field_get_max(args.result) > 0.0)
+        gwy_container_set_object(GWY_CONTAINER(data), mquark, args.result);
     else
-        gwy_container_remove(data, mquark);
-    gwy_app_channel_log_add_proc(data, id, id);
+        gwy_container_remove(GWY_CONTAINER(data), mquark);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, id);
 
 end:
     g_object_unref(args.result);
@@ -161,30 +153,23 @@ end:
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     GtkWidget *hbox, *dataview;
-    GwyDialogOutcome outcome;
     GwyParamTable *table;
     GwyDialog *dialog;
     ModuleGUI gui;
     gchar *s;
 
     gui.args = args;
-    gui.data = gwy_container_new();
-    gwy_container_set_object_by_name(gui.data, "/0/data", args->field);
-    gwy_container_set_object_by_name(gui.data, "/0/mask", args->result);
-    gwy_app_sync_data_items(data, gui.data, id, 0, FALSE,
-                            GWY_DATA_ITEM_PALETTE,
-                            GWY_DATA_ITEM_RANGE,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
 
     gui.dialog = gwy_dialog_new(_("Mark Grains by Watershed"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_UPDATE, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, TRUE);
+    dataview = gwy_create_preview(args->field, args->result, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_RANGE | GWY_FILE_ITEM_REAL_SQUARE);
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), FALSE);
 
     table = gui.table = gwy_param_table_new(args->params);
@@ -206,7 +191,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_param_table_set_unitstr(table, PARAM_WSHED_DROPSIZE, "%");
 
     gwy_param_table_append_header(table, -1, _("Options"));
-    gwy_param_table_append_mask_color(table, PARAM_MASK_COLOR, gui.data, 0, data, id);
+    gwy_param_table_append_image_mask_color(table, PARAM_MASK_COLOR, GWY_DATA_VIEW(dataview), data, id);
     gwy_param_table_append_checkbox(table, PARAM_INVERTED);
     if (args->mask) {
         gwy_param_table_append_radio_buttons(table, PARAM_COMBINE_TYPE, NULL);
@@ -219,11 +204,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     g_signal_connect_swapped(table, "param-changed", G_CALLBACK(param_changed), &gui);
     gwy_dialog_set_preview_func(dialog, GWY_PREVIEW_UPON_REQUEST, preview, &gui, NULL);
 
-    outcome = gwy_dialog_run(dialog);
-
-    g_object_unref(gui.data);
-
-    return outcome;
+    return gwy_dialog_run(dialog);
 }
 
 static void
@@ -240,11 +221,11 @@ preview(gpointer user_data)
     ModuleArgs *args = gui->args;
 
     if (execute(args, GTK_WINDOW(gui->dialog))) {
-        gwy_data_field_data_changed(args->result);
+        gwy_field_data_changed(args->result);
         gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
     }
     else
-        gwy_data_field_clear(args->result);
+        gwy_field_clear(args->result);
 }
 
 static gboolean
@@ -259,25 +240,25 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
     gint wshed_steps = gwy_params_get_int(params, PARAM_WSHED_STEPS);
     gdouble wshed_dropsize = gwy_params_get_double(params, PARAM_WSHED_DROPSIZE);
     gboolean inverted = gwy_params_get_boolean(params, PARAM_INVERTED);
-    GwyDataField *newmask, *mask = args->mask;
+    GwyField *newmask, *mask = args->mask;
     gdouble max, min, q;
     GwyComputationState *state;
     GwyWatershedStateType oldstate = -1;
     gboolean ok = FALSE;
 
-    max = gwy_data_field_get_max(args->field);
-    min = gwy_data_field_get_min(args->field);
+    max = gwy_field_get_max(args->field);
+    min = gwy_field_get_min(args->field);
     q = (max - min)/5000.0 * 100.0;
 
-    newmask = gwy_data_field_new_alike(args->result, FALSE);
-    state = gwy_data_field_grains_watershed_init(args->field, newmask,
-                                                 locate_steps, locate_thresh, locate_dropsize*q,
-                                                 wshed_steps, wshed_dropsize*q,
-                                                 FALSE, inverted);
+    newmask = gwy_field_new_alike(args->result, FALSE);
+    state = gwy_field_grains_watershed_init(args->field, newmask,
+                                            locate_steps, locate_thresh, locate_dropsize*q,
+                                            wshed_steps, wshed_dropsize*q,
+                                            FALSE, inverted);
     gwy_app_wait_start(wait_window, _("Initializing..."));
 
     do {
-        gwy_data_field_grains_watershed_iteration(state);
+        gwy_field_grains_watershed_iteration(state);
         if (oldstate != state->state) {
             if (state->state == GWY_WATERSHED_STATE_MIN)
                 ok = gwy_app_wait_set_message(_("Finding minima..."));
@@ -297,17 +278,17 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
     ok = (state->state == GWY_WATERSHED_STATE_FINISHED);
 
     gwy_app_wait_finish();
-    gwy_data_field_grains_watershed_finalize(state);
+    gwy_field_grains_watershed_finalize(state);
 
     if (ok) {
         if (mask && combine) {
             if (combine_type == GWY_MERGE_UNION)
-                gwy_data_field_grains_add(newmask, mask);
+                gwy_field_grains_add(newmask, mask);
             else if (combine_type == GWY_MERGE_INTERSECTION)
-                gwy_data_field_grains_intersect(newmask, mask);
+                gwy_field_grains_intersect(newmask, mask);
         }
-        gwy_data_field_threshold(newmask, 0.5, 0.0, 1.0);
-        gwy_data_field_copy(newmask, args->result, FALSE);
+        gwy_field_threshold(newmask, 0.5, 0.0, 1.0);
+        gwy_field_copy_data(newmask, args->result);
     }
     g_object_unref(newmask);
 
