@@ -1,5 +1,5 @@
 /*
- *  $Id: hough.c 23666 2021-05-10 21:56:10Z yeti-dn $
+ *  $Id: hough.c 28910 2025-11-24 18:11:09Z yeti-dn $
  *  Copyright (C) 2003-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,16 +16,9 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libprocess/hough.h>
-#include <libprocess/filters.h>
-#include <libprocess/cwt.h>
-#include <libgwydgets/gwydgetutils.h>
-#include <libgwydgets/gwycombobox.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
@@ -42,8 +35,8 @@ enum {
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             hough               (GwyContainer *data,
-                                             GwyRunType run);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static GwyDialogOutcome run_gui             (GwyParams *params);
 static void             param_changed       (GwyParamTable *table,
                                              gint id);
@@ -64,11 +57,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("hough",
-                              (GwyProcessFunc)&hough,
+                              module_main,
                               N_("/_Integral Transforms/_Hough..."),
-                              GWY_STOCK_HOUGH,
+                              GWY_ICON_HOUGH,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Compute Hough transform"));
 
     return TRUE;
@@ -95,23 +88,22 @@ define_module_params(void)
 }
 
 static void
-hough(GwyContainer *data, GwyRunType run)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
-    GwyDataField *dfield, *edgefield, *result, *f1, *f2;
-    GwySIUnit *siunit;
+    GwyField *dfield, *edgefield, *result, *f1, *f2;
     GwyParams *params;
     GwyHoughOutputType output;
     gint id, newid, circle_size;
     gchar *title;
 
-    g_return_if_fail(run & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &dfield,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &dfield,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(dfield);
 
     params = gwy_params_new_from_settings(define_module_params());
-    if (run == GWY_RUN_INTERACTIVE) {
+    if (mode == GWY_RUN_INTERACTIVE) {
         GwyDialogOutcome outcome = run_gui(params);
         gwy_params_save_to_settings(params);
         if (outcome != GWY_DIALOG_PROCEED) {
@@ -123,40 +115,40 @@ hough(GwyContainer *data, GwyRunType run)
     output = gwy_params_get_enum(params, PARAM_OUTPUT);
     circle_size = gwy_params_get_int(params, PARAM_CIRCLE_SIZE);
 
-    result = gwy_data_field_new_alike(dfield, FALSE);
-    siunit = gwy_si_unit_new(NULL);
-    gwy_data_field_set_si_unit_z(result, siunit);
-    g_object_unref(siunit);
+    result = gwy_field_new_alike(dfield, FALSE);
+    gwy_unit_clear(gwy_field_get_unit_z(result));
 
-    edgefield = gwy_data_field_duplicate(dfield);
+    edgefield = gwy_field_copy(dfield);
 
-    f1 = gwy_data_field_duplicate(dfield);
-    f2 = gwy_data_field_duplicate(dfield);
+    f1 = gwy_field_copy(dfield);
+    f2 = gwy_field_copy(dfield);
 
-    gwy_data_field_filter_canny(edgefield, 0.1);
-    gwy_data_field_filter_sobel(f1, GWY_ORIENTATION_HORIZONTAL);
-    gwy_data_field_filter_sobel(f2, GWY_ORIENTATION_VERTICAL);
+    gwy_field_filter_canny(edgefield, 0.1);
+    gwy_field_filter_sobel(f1, GWY_ORIENTATION_HORIZONTAL);
+    gwy_field_filter_sobel(f2, GWY_ORIENTATION_VERTICAL);
     if (output == GWY_HOUGH_OUTPUT_LINE) {
         title = g_strdup(_("Hough line"));
-        gwy_data_field_hough_line(edgefield, f1, f2, result, 1, FALSE);
+        gwy_field_hough_line(edgefield, f1, f2, result, 1, FALSE);
     }
     else  {
         title = g_strdup_printf(_("Hough circle r=%d"), circle_size);
-        gwy_data_field_hough_circle(edgefield, f1, f2, result, circle_size);
+        gwy_field_hough_circle(edgefield, f1, f2, result, circle_size);
     }
 
-    newid = gwy_app_data_browser_add_data_field(result, data, TRUE);
+    newid = gwy_file_add_image(data, result);
+
+    gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
     g_object_unref(result);
 
-    gwy_app_set_data_field_title(data, newid, title);
-    gwy_data_field_data_changed(result);
+    gwy_file_set_title(data, GWY_FILE_IMAGE, newid, title, TRUE);
+    gwy_field_data_changed(result);
     g_object_unref(edgefield);
     g_object_unref(f1);
     g_object_unref(f2);
     g_object_unref(params);
     g_free(title);
 
-    gwy_app_channel_log_add_proc(data, id, newid);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, newid);
 }
 
 static GwyDialogOutcome
