@@ -1,5 +1,5 @@
 /*
- *  $Id: immerse.c 26752 2024-10-18 15:49:13Z yeti-dn $
+ *  $Id: immerse.c 28844 2025-11-10 16:07:51Z yeti-dn $
  *  Copyright (C) 2004-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,19 +16,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/stats.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/correlation.h>
-#include <libprocess/hough.h>
-#include <libdraw/gwypixfield.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES GWY_RUN_INTERACTIVE
@@ -62,8 +53,10 @@ enum {
     PARAM_LEVELING,
     PARAM_DRAW_FRAME,
     PARAM_DRAW_DETAIL,
+    PARAM_ADAPT_COLOR_RANGE,
     PARAM_XPOS,
     PARAM_YPOS,
+
     BUTTON_LOCATE,
     BUTTON_REFINE,
     INFO_XYPOS,
@@ -71,55 +64,48 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *result;
+    GwyField *field;
+    GwyField *result;
 } ModuleArgs;
 
 typedef struct {
     ModuleArgs *args;
     GtkWidget *dialog;
     GwyParamTable *table;
-    GwyContainer *data;
     GtkWidget *view;
     GdkPixbuf *detail;
-    GwySIValueFormat *vf;
+    GwyValueFormat *vf;
     gdouble xmax;
     gdouble ymax;
-    gint xc;
-    gint yc;
+    gdouble xc;
+    gdouble yc;
     gint button;
-    GdkCursor *near_cursor;
-    GdkCursor *move_cursor;
 } ModuleGUI;
 
 static gboolean         module_register            (void);
 static GwyParamDef*     define_module_params       (void);
-static void             immerse                    (GwyContainer *data,
-                                                    GwyRunType runtype);
+static void             module_main                (GwyFile *data,
+                                                    GwyRunModeFlags mode);
 static void             execute                    (ModuleArgs *args);
 static GwyDialogOutcome run_gui                    (ModuleArgs *args,
-                                                    GwyContainer *data,
+                                                    GwyFile *data,
                                                     gint id);
-static void             dialog_mapped              (GtkWidget *dialog,
-                                                    ModuleGUI *gui);
-static void             dialog_unmapped            (GtkWidget *dialog,
-                                                    ModuleGUI *gui);
 static void             param_changed              (ModuleGUI *gui,
                                                     gint id);
 static void             dialog_response            (ModuleGUI *gui,
                                                     gint response);
 static void             immerse_update_detail      (ModuleGUI *gui);
-static gboolean         detail_filter              (GwyContainer *data,
+static gboolean         detail_filter              (GwyFile *data,
                                                     gint id,
                                                     gpointer user_data);
 static void             immerse_search             (ModuleGUI *gui,
                                                     gint search_type);
-static void             immerse_correlate          (GwyDataField *image,
-                                                    GwyDataField *kernel,
+static void             immerse_correlate          (GwyField *image,
+                                                    GwyField *kernel,
                                                     gint *col,
                                                     gint *row);
-static gboolean         immerse_view_expose        (GtkWidget *view,
-                                                    GdkEventExpose *event,
+static gboolean         immerse_view_draw          (GtkWidget *view,
+                                                    cairo_t *cr,
                                                     ModuleGUI *gui);
 static gboolean         immerse_view_button_press  (GtkWidget *view,
                                                     GdkEventButton *event,
@@ -143,7 +129,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Immerse high resolution detail into overall image."),
     "Petr Klapetek <klapetek@gwyddion.net>",
-    "3.0",
+    "4.0",
     "David Nečas (Yeti) & Petr Klapetek",
     "2006",
 };
@@ -154,11 +140,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("immerse",
-                              (GwyProcessFunc)&immerse,
+                              module_main,
                               N_("/M_ultidata/_Immerse Detail..."),
-                              GWY_STOCK_IMMERSE,
+                              GWY_ICON_IMMERSE,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Immerse a detail into image"));
 
     return TRUE;
@@ -172,7 +158,7 @@ define_module_params(void)
         { N_("_Downsample detail"),    GWY_IMMERSE_SAMPLING_DOWN, },
     };
     static const GwyEnum levelings[] = {
-        { N_("levelling|_None"), GWY_IMMERSE_LEVEL_NONE, },
+        { gwy_NC("levelling", "_None"), GWY_IMMERSE_LEVEL_NONE, },
         { N_("_Mean value"),     GWY_IMMERSE_LEVEL_MEAN, },
     };
     static GwyParamDef *paramdef = NULL;
@@ -189,23 +175,25 @@ define_module_params(void)
                               levelings, G_N_ELEMENTS(levelings), GWY_IMMERSE_LEVEL_MEAN);
     gwy_param_def_add_boolean(paramdef, PARAM_DRAW_FRAME, "draw_frame", _("Show _frame"), TRUE);
     gwy_param_def_add_boolean(paramdef, PARAM_DRAW_DETAIL, "draw_detail", _("Show _detail"), TRUE);
+    gwy_param_def_add_boolean(paramdef, PARAM_ADAPT_COLOR_RANGE, "adapt-color-range",
+                              _("Use image's color range for detail"), FALSE);
     gwy_param_def_add_double(paramdef, PARAM_XPOS, NULL, _("X position"), -G_MAXDOUBLE, G_MAXDOUBLE, 0.0);
     gwy_param_def_add_double(paramdef, PARAM_YPOS, NULL, _("Y position"), -G_MAXDOUBLE, G_MAXDOUBLE, 0.0);
     return paramdef;
 }
 
 static void
-immerse(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome;
     ModuleArgs args;
     gint id, newid;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(args.field);
 
     args.params = gwy_params_new_from_settings(define_module_params());
@@ -216,24 +204,24 @@ immerse(GwyContainer *data, GwyRunType runtype)
 
     execute(&args);
 
-    newid = gwy_app_data_browser_add_data_field(args.result, data, TRUE);
-    gwy_app_set_data_field_title(data, newid, _("Immersed detail"));
-    gwy_app_channel_log_add_proc(data, id, newid);
-    gwy_app_sync_data_items(data, data, id, newid, FALSE,
-                            GWY_DATA_ITEM_GRADIENT,
-                            GWY_DATA_ITEM_MASK_COLOR,
-                            GWY_DATA_ITEM_RANGE_TYPE,
-                            GWY_DATA_ITEM_RANGE,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
+    newid = gwy_file_add_image(data, args.result);
+
+    gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+    gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _("Immersed detail"), TRUE);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, newid);
+    gwy_file_sync_items(data, GWY_FILE_IMAGE, id,
+                        data, GWY_FILE_IMAGE, newid,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_MASK_COLOR
+                        | GWY_FILE_ITEM_COLOR_MAPPING | GWY_FILE_ITEM_RANGE | GWY_FILE_ITEM_REAL_SQUARE,
+                        FALSE);
 
 end:
     g_object_unref(args.params);
-    GWY_OBJECT_UNREF(args.result);
+    g_clear_object(&args.result);
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     ModuleGUI gui;
     GwyDialog *dialog;
@@ -241,27 +229,22 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     GtkWidget *hbox;
     GwyDialogOutcome outcome;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
 
-    gui.data = gwy_container_new();
-    gwy_container_set_object_by_name(gui.data, "/0/data", args->field);
-    gwy_container_set_boolean_by_name(gui.data, "/0/data/realsquare", TRUE);
-    gwy_app_sync_data_items(data, gui.data, id, 0, FALSE,
-                            GWY_DATA_ITEM_PALETTE,
-                            GWY_DATA_ITEM_MASK_COLOR,
-                            GWY_DATA_ITEM_RANGE,
-                            0);
-    gui.vf = gwy_data_field_get_value_format_xy(args->field, GWY_SI_UNIT_FORMAT_VFMARKUP, NULL);
+    gui.vf = gwy_field_get_value_format_xy(args->field, GWY_UNIT_FORMAT_VFMARKUP, NULL);
 
     gui.dialog = gwy_dialog_new(_("Immerse Detail"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    gui.view = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
+    gui.view = gwy_create_preview(args->field, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(gui.view), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_RANGE);
+    gwy_data_view_set_real_square(GWY_DATA_VIEW(gui.view), TRUE);
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(gui.view), FALSE);
 
-    g_signal_connect_after(gui.view, "expose-event", G_CALLBACK(immerse_view_expose), &gui);
+    g_signal_connect_after(gui.view, "draw", G_CALLBACK(immerse_view_draw), &gui);
     g_signal_connect(gui.view, "button-press-event", G_CALLBACK(immerse_view_button_press), &gui);
     g_signal_connect(gui.view, "button-release-event", G_CALLBACK(immerse_view_button_release), &gui);
     g_signal_connect(gui.view, "motion-notify-event", G_CALLBACK(immerse_view_motion_notify), &gui);
@@ -283,46 +266,26 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_param_table_append_separator(table);
     gwy_param_table_append_checkbox(table, PARAM_DRAW_FRAME);
     gwy_param_table_append_checkbox(table, PARAM_DRAW_DETAIL);
+    gwy_param_table_append_checkbox(table, PARAM_ADAPT_COLOR_RANGE);
 
     gtk_box_pack_start(GTK_BOX(hbox), gwy_param_table_widget(table), TRUE, TRUE, 0);
     gwy_dialog_add_param_table(dialog, table);
 
     g_signal_connect_swapped(table, "param-changed", G_CALLBACK(param_changed), &gui);
     g_signal_connect_swapped(dialog, "response", G_CALLBACK(dialog_response), &gui);
-    g_signal_connect_after(dialog, "map", G_CALLBACK(dialog_mapped), &gui);
-    g_signal_connect(dialog, "unmap", G_CALLBACK(dialog_unmapped), &gui);
 
     outcome = gwy_dialog_run(dialog);
 
-    gwy_si_unit_value_format_free(gui.vf);
-    g_object_unref(gui.data);
-    GWY_OBJECT_UNREF(gui.detail);
+    gwy_value_format_free(gui.vf);
+    g_clear_object(&gui.detail);
 
     return outcome;
 }
 
-static void
-dialog_mapped(GtkWidget *dialog, ModuleGUI *gui)
-{
-    GdkDisplay *display = gtk_widget_get_display(dialog);
-
-    gui->near_cursor = gdk_cursor_new_for_display(display, GDK_FLEUR);
-    gui->move_cursor = gdk_cursor_new_for_display(display, GDK_CROSS);
-    immerse_update_detail(gui);
-}
-
-static void
-dialog_unmapped(G_GNUC_UNUSED GtkWidget *dialog, ModuleGUI *gui)
-{
-    gdk_cursor_unref(gui->near_cursor);
-    gdk_cursor_unref(gui->move_cursor);
-    gui->near_cursor = gui->move_cursor = NULL;
-}
-
 static void
 param_changed(ModuleGUI *gui, gint id)
 {
-    if (id < 0 || id == PARAM_DETAIL)
+    if (id < 0 || id == PARAM_DETAIL || id == PARAM_ADAPT_COLOR_RANGE)
         immerse_update_detail(gui);
 
     if (id != PARAM_SAMPLING && id != PARAM_LEVELING)
@@ -342,17 +305,16 @@ static void
 immerse_update_detail(ModuleGUI *gui)
 {
     ModuleArgs *args = gui->args;
-    GwyDataField *field = args->field, *detail = gwy_params_get_image(args->params, PARAM_DETAIL);
+    GwyField *field = args->field, *detail = gwy_params_get_image(args->params, PARAM_DETAIL);
     gdouble xpos = gwy_params_get_double(args->params, PARAM_XPOS);
     gdouble ypos = gwy_params_get_double(args->params, PARAM_YPOS);
     GwyAppDataId dataid = gwy_params_get_data_id(args->params, PARAM_DETAIL);
+    GwyFile *file = gwy_data_browser_get_file(dataid.datano);
+    gint id = dataid.id;
     gdouble xreald, yreald;
-    GwyGradient *gradient;
-    GdkPixbuf *pixbuf;
-    const guchar *name;
     gint w, h, xresd, yresd;
 
-    GWY_OBJECT_UNREF(gui->detail);
+    g_clear_object(&gui->detail);
     gtk_dialog_set_response_sensitive(GTK_DIALOG(gui->dialog), GTK_RESPONSE_OK, !!detail);
     gtk_dialog_set_response_sensitive(GTK_DIALOG(gui->dialog), RESPONSE_ESTIMATE, !!detail);
     gtk_dialog_set_response_sensitive(GTK_DIALOG(gui->dialog), RESPONSE_REFINE, !!detail);
@@ -360,49 +322,67 @@ immerse_update_detail(ModuleGUI *gui)
     if (!detail || !gtk_widget_is_drawable(gui->view))
         return;
 
-    xresd = gwy_data_field_get_xres(detail);
-    yresd = gwy_data_field_get_yres(detail);
-    xreald = gwy_data_field_get_xreal(detail);
-    yreald = gwy_data_field_get_yreal(detail);
-    gui->xmax = gwy_data_field_get_xreal(field) - xreald + gwy_data_field_get_dx(field)/2.0;
-    gui->ymax = gwy_data_field_get_yreal(field) - yreald + gwy_data_field_get_dy(field)/2.0;
+    xresd = gwy_field_get_xres(detail);
+    yresd = gwy_field_get_yres(detail);
+    xreald = gwy_field_get_xreal(detail);
+    yreald = gwy_field_get_yreal(detail);
+    gui->xmax = gwy_field_get_xreal(field) - xreald + gwy_field_get_dx(field)/2.0;
+    gui->ymax = gwy_field_get_yreal(field) - yreald + gwy_field_get_dy(field)/2.0;
 
     clamp_detail_offset(gui, xpos, ypos);
 
-    name = NULL;
-    gwy_container_gis_string(gwy_app_data_browser_get(dataid.datano),
-                             gwy_app_get_data_palette_key_for_id(dataid.id),
-                             &name);
-    gradient = gwy_gradients_get_gradient(name);
+    const gchar *name = gwy_file_get_palette(file, GWY_FILE_IMAGE, id);
+    GwyGradient *gradient = gwy_gradients_get_gradient(name);
 
     /* Handle real-square properly by using an intermediate pixel-square pixbuf with sufficient resolution */
-    gwy_data_view_coords_real_to_xy(GWY_DATA_VIEW(gui->view), xreald, yreald, &w, &h);
+    gdouble wreal, hreal;
+    gwy_data_view_coords_real_to_widget(GWY_DATA_VIEW(gui->view), xreald, yreald, &wreal, &hreal);
+    w = GWY_ROUND(wreal);
+    h = GWY_ROUND(hreal);
     gwy_debug("%dx%d", w, h);
     w = MAX(w, 2);
     h = MAX(h, 2);
 
-    pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, 8, xresd, yresd);
-    gwy_pixbuf_draw_data_field(pixbuf, detail, gradient);
+    GdkPixbuf *pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, 8, xresd, yresd);
+    if (gwy_params_get_boolean(args->params, PARAM_ADAPT_COLOR_RANGE)) {
+        GwyColorMappingType range_type;
+        gdouble min, max;
+        gwy_container_gis_enum(GWY_CONTAINER(file), gwy_file_key_image_color_mapping(id), &range_type);
+        if (range_type == GWY_COLOR_MAPPING_AUTO)
+            gwy_field_get_autorange(field, &min, &max);
+        else {
+            gwy_field_get_min_max(field, &min, &max);
+            if (range_type == GWY_COLOR_MAPPING_FIXED) {
+                gwy_container_gis_double(GWY_CONTAINER(file), gwy_file_key_image_range_min(id), &min);
+                gwy_container_gis_double(GWY_CONTAINER(file), gwy_file_key_image_range_max(id), &max);
+            }
+            /* FIXME: Adaptive mapping using the distribution of different data field is currently not possible. */
+        }
+        gwy_draw_field_pixbuf_with_range(pixbuf, detail, gradient, min, max);
+    }
+    else
+        gwy_draw_field_pixbuf_full(pixbuf, detail, gradient);
+
     gui->detail = gdk_pixbuf_scale_simple(pixbuf, w, h, GDK_INTERP_TILES);
+
     g_object_unref(pixbuf);
 
     redraw(gui);
 }
 
 static gboolean
-detail_filter(GwyContainer *data, gint id, gpointer user_data)
+detail_filter(GwyFile *data, gint id, gpointer user_data)
 {
-    GwyDataField *detail, *field = (GwyDataField*)user_data;
+    GwyField *detail, *field = (GwyField*)user_data;
 
-    if (!gwy_container_gis_object(data, gwy_app_get_data_key_for_id(id), &detail))
+    if (!(detail = gwy_file_get_image(data, id)))
         return FALSE;
     if (detail == field)
         return FALSE;
-    if (gwy_data_field_check_compatibility(field, detail,
-                                           GWY_DATA_COMPATIBILITY_LATERAL | GWY_DATA_COMPATIBILITY_VALUE))
+    if (gwy_field_is_incompatible(field, detail, GWY_DATA_MISMATCH_LATERAL | GWY_DATA_MISMATCH_VALUE))
         return FALSE;
-    if (gwy_data_field_get_xreal(field) < gwy_data_field_get_xreal(detail)
-        || gwy_data_field_get_yreal(field) < gwy_data_field_get_yreal(detail))
+    if (gwy_field_get_xreal(field) < gwy_field_get_xreal(detail)
+        || gwy_field_get_yreal(field) < gwy_field_get_yreal(detail))
         return FALSE;
 
     return TRUE;
@@ -414,16 +394,16 @@ immerse_search(ModuleGUI *gui, gboolean is_refine)
     ModuleArgs *args = gui->args;
     gdouble xpos = gwy_params_get_double(args->params, PARAM_XPOS);
     gdouble ypos = gwy_params_get_double(args->params, PARAM_YPOS);
-    GwyDataField *field = args->field, *detail = gwy_params_get_image(args->params, PARAM_DETAIL);
-    GwyDataField *subfield, *iarea;
+    GwyField *field = args->field, *detail = gwy_params_get_image(args->params, PARAM_DETAIL);
+    GwyField *subfield, *iarea;
     gdouble wr, hr, deltax, deltay;
     gint w, h, xfrom, xto, yfrom, yto, ixres, iyres, col, row;
 
-    ixres = gwy_data_field_get_xres(field);
-    iyres = gwy_data_field_get_yres(field);
+    ixres = gwy_field_get_xres(field);
+    iyres = gwy_field_get_yres(field);
 
-    wr = gwy_data_field_get_xreal(detail)/gwy_data_field_get_dx(field);
-    hr = gwy_data_field_get_yreal(detail)/gwy_data_field_get_dy(field);
+    wr = gwy_field_get_xreal(detail)/gwy_field_get_dx(field);
+    hr = gwy_field_get_yreal(detail)/gwy_field_get_dy(field);
     if (wr*hr < 6.0) {
         g_warning("Detail image is too small for correlation");
         return;
@@ -434,8 +414,8 @@ immerse_search(ModuleGUI *gui, gboolean is_refine)
     gwy_debug("w: %d, h: %d", w, h);
     g_assert(w <= ixres && h <= iyres);
     if (is_refine) {
-        xfrom = gwy_data_field_rtoj(field, xpos);
-        yfrom = gwy_data_field_rtoi(field, ypos);
+        xfrom = gwy_field_rtoj(field, xpos);
+        yfrom = gwy_field_rtoi(field, ypos);
         /* Calculate the area we will search the detail in */
         deltax = improve_search_window(w, ixres);
         deltay = improve_search_window(h, iyres);
@@ -456,16 +436,16 @@ immerse_search(ModuleGUI *gui, gboolean is_refine)
     if (xfrom == 0 && yfrom == 0 && xto == ixres && yto == iyres)
         iarea = g_object_ref(field);
     else
-        iarea = gwy_data_field_area_extract(field, xfrom, yfrom, xto - xfrom, yto - yfrom);
+        iarea = gwy_field_area_extract(field, xfrom, yfrom, xto - xfrom, yto - yfrom);
 
-    subfield = gwy_data_field_new_resampled(detail, w, h, GWY_INTERPOLATION_LINEAR);
+    subfield = gwy_field_new_resampled(detail, w, h, GWY_INTERPOLATION_LINEAR);
 
     immerse_correlate(iarea, subfield, &col, &row);
     gwy_debug("[c] col: %d, row: %d", col, row);
     col += xfrom;
     row += yfrom;
-    xpos = gwy_data_field_jtor(subfield, col + 0.5);
-    ypos = gwy_data_field_itor(subfield, row + 0.5);
+    xpos = gwy_field_jtor(subfield, col + 0.5);
+    ypos = gwy_field_itor(subfield, row + 0.5);
     g_object_unref(iarea);
     g_object_unref(subfield);
     gwy_debug("[C] col: %d, row: %d", col, row);
@@ -476,15 +456,15 @@ immerse_search(ModuleGUI *gui, gboolean is_refine)
     xto = MIN(col + w + 1, ixres);
     yto = MIN(row + h + 1, iyres);
     gwy_debug("x: %d..%d, y: %d..%d", xfrom, xto, yfrom, yto);
-    iarea = gwy_data_field_area_extract(field, xfrom, yfrom, xto - xfrom, yto - yfrom);
-    wr = gwy_data_field_get_xreal(iarea)/gwy_data_field_get_dx(detail);
-    hr = gwy_data_field_get_yreal(iarea)/gwy_data_field_get_dy(detail);
-    gwy_data_field_resample(iarea, GWY_ROUND(wr), GWY_ROUND(hr), GWY_INTERPOLATION_LINEAR);
+    iarea = gwy_field_area_extract(field, xfrom, yfrom, xto - xfrom, yto - yfrom);
+    wr = gwy_field_get_xreal(iarea)/gwy_field_get_dx(detail);
+    hr = gwy_field_get_yreal(iarea)/gwy_field_get_dy(detail);
+    gwy_field_resample(iarea, GWY_ROUND(wr), GWY_ROUND(hr), GWY_INTERPOLATION_LINEAR);
     immerse_correlate(iarea, detail, &col, &row);
     gwy_debug("[U] col: %d, row: %d", col, row);
 
-    xpos = gwy_data_field_jtor(detail, col + 0.5) + gwy_data_field_jtor(field, xfrom);
-    ypos = gwy_data_field_itor(detail, row + 0.5) + gwy_data_field_itor(field, yfrom);
+    xpos = gwy_field_jtor(detail, col + 0.5) + gwy_field_jtor(field, xfrom);
+    ypos = gwy_field_itor(detail, row + 0.5) + gwy_field_itor(field, yfrom);
 
     g_object_unref(iarea);
     clamp_detail_offset(gui, xpos, ypos);
@@ -492,20 +472,20 @@ immerse_search(ModuleGUI *gui, gboolean is_refine)
 }
 
 static void
-immerse_correlate(GwyDataField *image, GwyDataField *kernel,
+immerse_correlate(GwyField *image, GwyField *kernel,
                   gint *col, gint *row)
 {
-    GwyDataField *subimage, *subkernel, *score, *imagearea;
+    GwyField *subimage, *subkernel, *score, *imagearea;
     gdouble factor, sxreal, syreal, zunused;
     gint ixres, iyres, kxres, kyres;
     gint sixres, siyres, skxres, skyres;
     gint xfrom, yfrom, xto, yto;
     gint sx, sy, delta;
 
-    ixres = gwy_data_field_get_xres(image);
-    iyres = gwy_data_field_get_yres(image);
-    kxres = gwy_data_field_get_xres(kernel);
-    kyres = gwy_data_field_get_yres(kernel);
+    ixres = gwy_field_get_xres(image);
+    iyres = gwy_field_get_yres(image);
+    kxres = gwy_field_get_xres(kernel);
+    kyres = gwy_field_get_yres(kernel);
     gwy_debug("kernel: %dx%d, image: %dx%d", kxres, kyres, ixres, iyres);
 
     factor = MAX(downsample_factor, downsample_limit/sqrt(kxres*kyres));
@@ -517,13 +497,13 @@ immerse_correlate(GwyDataField *image, GwyDataField *kernel,
     siyres = GWY_ROUND(factor*iyres);
     gwy_debug("skernel: %dx%d, simage: %dx%d", skxres, skyres, sixres, siyres);
 
-    subimage = gwy_data_field_new_resampled(image, sixres, siyres, GWY_INTERPOLATION_LINEAR);
-    score = gwy_data_field_new_alike(subimage, FALSE);
-    subkernel = gwy_data_field_new_resampled(kernel, skxres, skyres, GWY_INTERPOLATION_LINEAR);
+    subimage = gwy_field_new_resampled(image, sixres, siyres, GWY_INTERPOLATION_LINEAR);
+    score = gwy_field_new_alike(subimage, FALSE);
+    subkernel = gwy_field_new_resampled(kernel, skxres, skyres, GWY_INTERPOLATION_LINEAR);
 
-    gwy_data_field_correlation_search(subimage, subkernel, NULL, score,
-                                      GWY_CORR_SEARCH_COVARIANCE_SCORE, 0.01, GWY_EXTERIOR_BORDER_EXTEND, 0.0);
-    gwy_data_field_get_local_maxima_list(score, &sxreal, &syreal, &zunused, 1, 0, 0.0, FALSE);
+    gwy_field_correlation_search(subimage, subkernel, NULL, score,
+                                 GWY_CORR_SEARCH_COVARIANCE_SCORE, 0.01, GWY_EXTERIOR_BORDER, 0.0);
+    gwy_field_get_local_maxima_list(score, &sxreal, &syreal, &zunused, 1, 0, 0.0, FALSE);
     sx = GWY_ROUND(sxreal);
     sy = GWY_ROUND(syreal);
     gwy_debug("sx: %d, sy: %d", sx, sy);
@@ -545,11 +525,11 @@ immerse_correlate(GwyDataField *image, GwyDataField *kernel,
     xto = MIN(sx + kxres + delta, ixres);
     yto = MIN(sy + kyres + delta, iyres);
 
-    imagearea = gwy_data_field_area_extract(image, xfrom, yfrom, xto - xfrom, yto - yfrom);
-    score = gwy_data_field_new_alike(imagearea, FALSE);
-    gwy_data_field_correlation_search(imagearea, kernel, NULL, score,
-                                      GWY_CORR_SEARCH_COVARIANCE_SCORE, 0.01, GWY_EXTERIOR_BORDER_EXTEND, 0.0);
-    gwy_data_field_get_local_maxima_list(score, &sxreal, &syreal, &zunused, 1, 0, 0.0, FALSE);
+    imagearea = gwy_field_area_extract(image, xfrom, yfrom, xto - xfrom, yto - yfrom);
+    score = gwy_field_new_alike(imagearea, FALSE);
+    gwy_field_correlation_search(imagearea, kernel, NULL, score,
+                                 GWY_CORR_SEARCH_COVARIANCE_SCORE, 0.01, GWY_EXTERIOR_BORDER, 0.0);
+    gwy_field_get_local_maxima_list(score, &sxreal, &syreal, &zunused, 1, 0, 0.0, FALSE);
     g_object_unref(score);
     g_object_unref(imagearea);
 
@@ -560,8 +540,8 @@ immerse_correlate(GwyDataField *image, GwyDataField *kernel,
 static void
 execute(ModuleArgs *args)
 {
-    GwyDataField *field = args->field, *detail = gwy_params_get_image(args->params, PARAM_DETAIL);
-    GwyDataField *resampled, *result;
+    GwyField *field = args->field, *detail = gwy_params_get_image(args->params, PARAM_DETAIL);
+    GwyField *resampled, *result;
     GwyImmerseLevelType levelling = gwy_params_get_enum(args->params, PARAM_LEVELING);
     GwyImmerseSamplingType sampling = gwy_params_get_enum(args->params, PARAM_SAMPLING);
     gdouble xpos = gwy_params_get_double(args->params, PARAM_XPOS);
@@ -570,41 +550,41 @@ execute(ModuleArgs *args)
     gint x, y, w, h;
     gdouble iavg, davg;
 
-    davg = gwy_data_field_get_avg(detail);
-    kxres = gwy_data_field_get_xres(detail);
-    kyres = gwy_data_field_get_yres(detail);
+    davg = gwy_field_get_avg(detail);
+    kxres = gwy_field_get_xres(detail);
+    kyres = gwy_field_get_yres(detail);
 
     if (sampling == GWY_IMMERSE_SAMPLING_DOWN) {
-        result = args->result = gwy_data_field_duplicate(field);
-        x = gwy_data_field_rtoj(field, xpos);
-        y = gwy_data_field_rtoi(field, ypos);
-        w = GWY_ROUND(gwy_data_field_get_xreal(detail)/gwy_data_field_get_dx(field));
-        h = GWY_ROUND(gwy_data_field_get_yreal(detail)/gwy_data_field_get_dy(field));
+        result = args->result = gwy_field_copy(field);
+        x = gwy_field_rtoj(field, xpos);
+        y = gwy_field_rtoi(field, ypos);
+        w = GWY_ROUND(gwy_field_get_xreal(detail)/gwy_field_get_dx(field));
+        h = GWY_ROUND(gwy_field_get_yreal(detail)/gwy_field_get_dy(field));
         w = MAX(w, 1);
         h = MAX(h, 1);
         gwy_debug("w: %d, h: %d", w, h);
-        resampled = gwy_data_field_new_resampled(detail, w, h, GWY_INTERPOLATION_LINEAR);
+        resampled = gwy_field_new_resampled(detail, w, h, GWY_INTERPOLATION_LINEAR);
         if (levelling == GWY_IMMERSE_LEVEL_MEAN) {
-            iavg = gwy_data_field_area_get_avg_mask(result, NULL, GWY_MASK_IGNORE, x, y, w, h);
-            gwy_data_field_add(resampled, iavg - davg);
+            iavg = gwy_field_area_get_avg(result, NULL, GWY_MASK_IGNORE, x, y, w, h);
+            gwy_field_add(resampled, iavg - davg);
         }
-        gwy_data_field_area_copy(resampled, result, 0, 0, w, h, x, y);
+        gwy_field_area_copy(resampled, result, 0, 0, w, h, x, y);
         g_object_unref(resampled);
     }
     else if (sampling == GWY_IMMERSE_SAMPLING_UP) {
-        w = GWY_ROUND(gwy_data_field_get_xreal(field)/gwy_data_field_get_dx(detail));
-        h = GWY_ROUND(gwy_data_field_get_yreal(field)/gwy_data_field_get_dy(detail));
+        w = GWY_ROUND(gwy_field_get_xreal(field)/gwy_field_get_dx(detail));
+        h = GWY_ROUND(gwy_field_get_yreal(field)/gwy_field_get_dy(detail));
         gwy_debug("w: %d, h: %d", w, h);
-        result = args->result = gwy_data_field_new_resampled(field, w, h, GWY_INTERPOLATION_LINEAR);
-        x = gwy_data_field_rtoj(result, xpos);
-        y = gwy_data_field_rtoi(result, ypos);
+        result = args->result = gwy_field_new_resampled(field, w, h, GWY_INTERPOLATION_LINEAR);
+        x = gwy_field_rtoj(result, xpos);
+        y = gwy_field_rtoi(result, ypos);
         if (levelling == GWY_IMMERSE_LEVEL_MEAN) {
-            iavg = gwy_data_field_area_get_avg_mask(result, NULL, GWY_MASK_IGNORE, x, y, kxres, kyres);
-            gwy_data_field_area_copy(detail, result, 0, 0, kxres, kyres, x, y);
-            gwy_data_field_area_add(result, x, y, kxres, kyres, iavg - davg);
+            iavg = gwy_field_area_get_avg(result, NULL, GWY_MASK_IGNORE, x, y, kxres, kyres);
+            gwy_field_area_copy(detail, result, 0, 0, kxres, kyres, x, y);
+            gwy_field_area_add(result, x, y, kxres, kyres, iavg - davg);
         }
         else
-            gwy_data_field_area_copy(detail, result, 0, 0, kxres, kyres, x, y);
+            gwy_field_area_copy(detail, result, 0, 0, kxres, kyres, x, y);
     }
     else {
         g_return_if_reached();
@@ -612,34 +592,40 @@ execute(ModuleArgs *args)
 }
 
 static gboolean
-immerse_view_expose(GtkWidget *view, GdkEventExpose *event, ModuleGUI *gui)
+immerse_view_draw(GtkWidget *view, cairo_t *cr, ModuleGUI *gui)
 {
-    GdkColor white = { 0, 0xffff, 0xffff, 0xffff };
-    GwyParams *params = gui->args->params;
-    gdouble xpos = gwy_params_get_double(params, PARAM_XPOS);
-    gdouble ypos = gwy_params_get_double(params, PARAM_YPOS);
-    GdkGC *gc;
-    gint w, h, xoff, yoff;
-
-    if (event->count > 0)
-        return FALSE;
-
     if (!gui->detail)
         return FALSE;
 
-    gwy_data_view_coords_real_to_xy(GWY_DATA_VIEW(view), xpos, ypos, &xoff, &yoff);
-    w = gdk_pixbuf_get_width(gui->detail);
-    h = gdk_pixbuf_get_height(gui->detail);
-    /* gwy_debug("(%d,%d) %dx%d", xoff, yoff, w, h); */
-    gc = gdk_gc_new(view->window);
-    if (gwy_params_get_boolean(params, PARAM_DRAW_DETAIL))
-        gdk_draw_pixbuf(view->window, gc, gui->detail, 0, 0, xoff, yoff, w, h, GDK_RGB_DITHER_NORMAL, 0, 0);
-    if (gwy_params_get_boolean(params, PARAM_DRAW_FRAME)) {
-        gdk_gc_set_function(gc, GDK_XOR);
-        gdk_gc_set_rgb_fg_color(gc, &white);
-        gdk_draw_rectangle(view->window, gc, FALSE, xoff, yoff, w-1, h-1);
+    GwyParams *params = gui->args->params;
+    gdouble xpos = gwy_params_get_double(params, PARAM_XPOS);
+    gdouble ypos = gwy_params_get_double(params, PARAM_YPOS);
+    gdouble xoff, yoff;
+    gwy_data_view_coords_real_to_widget(GWY_DATA_VIEW(view), xpos, ypos, &xoff, &yoff);
+
+    /* Rounding fixes ugly black frame around the detail. */
+    xoff = GWY_ROUND(xoff);
+    yoff = GWY_ROUND(yoff);
+
+    gint w = gdk_pixbuf_get_width(gui->detail);
+    gint h = gdk_pixbuf_get_height(gui->detail);
+
+    if (gwy_params_get_boolean(params, PARAM_DRAW_DETAIL)) {
+        cairo_save(cr);
+        gdk_cairo_set_source_pixbuf(cr, gui->detail, xoff, yoff);
+        cairo_paint(cr);
+        cairo_restore(cr);
     }
-    g_object_unref(gc);
+    if (gwy_params_get_boolean(params, PARAM_DRAW_FRAME)) {
+        cairo_save(cr);
+        cairo_set_source_rgb(cr, 1.0, 1.0, 1.0);
+        cairo_set_line_width(cr, 1.0);
+        cairo_set_operator(cr, CAIRO_OPERATOR_DIFFERENCE);
+        cairo_rectangle(cr, xoff + 0.5, yoff + 0.5, w-1, h-1);
+        cairo_stroke(cr);
+        cairo_restore(cr);
+    }
+
     return FALSE;
 }
 
@@ -649,17 +635,17 @@ immerse_view_button_press(GtkWidget *view, GdkEventButton *event, ModuleGUI *gui
     GwyParams *params = gui->args->params;
     gdouble xpos = gwy_params_get_double(params, PARAM_XPOS);
     gdouble ypos = gwy_params_get_double(params, PARAM_YPOS);
-    gint xoff, yoff;
+    gdouble xoff, yoff;
 
     if (event->button != 1 || !immerse_view_inside_detail(gui, event->x, event->y))
         return FALSE;
 
-    gwy_data_view_coords_real_to_xy(GWY_DATA_VIEW(view), xpos, ypos, &xoff, &yoff);
+    gwy_data_view_coords_real_to_widget(GWY_DATA_VIEW(view), xpos, ypos, &xoff, &yoff);
     gui->button = event->button;
     /* Cursor offset wrt top left corner */
     gui->xc = event->x - xoff;
     gui->yc = event->y - yoff;
-    gdk_window_set_cursor(view->window, gui->move_cursor);
+    gwy_data_view_set_named_cursor(GWY_DATA_VIEW(view), "crosshair");
 
     return TRUE;
 }
@@ -673,9 +659,9 @@ immerse_view_button_release(GtkWidget *view, GdkEventButton *event, ModuleGUI *g
         return FALSE;
 
     gui->button = 0;
-    gwy_data_view_coords_xy_to_real(GWY_DATA_VIEW(view), event->x - gui->xc, event->y - gui->yc, &xpos, &ypos);
+    gwy_data_view_coords_widget_to_real(GWY_DATA_VIEW(view), event->x - gui->xc, event->y - gui->yc, &xpos, &ypos);
     clamp_detail_offset(gui, xpos, ypos);
-    gdk_window_set_cursor(view->window, gui->near_cursor);
+    gwy_data_view_set_named_cursor(GWY_DATA_VIEW(view), "move");
     redraw(gui);
 
     return TRUE;
@@ -684,29 +670,17 @@ immerse_view_button_release(GtkWidget *view, GdkEventButton *event, ModuleGUI *g
 static gboolean
 immerse_view_motion_notify(GtkWidget *view, GdkEventMotion *event, ModuleGUI *gui)
 {
-    GdkWindow *window;
-    gdouble xpos, ypos;
-    gint x, y;
-
     if (!gui->detail)
         return FALSE;
 
-    window = view->window;
-    if (event->is_hint)
-        gdk_window_get_pointer(window, &x, &y, NULL);
-    else {
-        x = event->x;
-        y = event->y;
-    }
+    gdouble xpos, ypos;
+    gdouble x = event->x, y = event->y;
 
     if (!gui->button) {
-        if (immerse_view_inside_detail(gui, x, y))
-            gdk_window_set_cursor(window, gui->near_cursor);
-        else
-            gdk_window_set_cursor(window, NULL);
+        gwy_data_view_set_named_cursor(GWY_DATA_VIEW(view), immerse_view_inside_detail(gui, x, y) ? "move" : NULL);
     }
     else {
-        gwy_data_view_coords_xy_to_real(GWY_DATA_VIEW(view), x - gui->xc, y - gui->yc, &xpos, &ypos);
+        gwy_data_view_coords_widget_to_real(GWY_DATA_VIEW(view), x - gui->xc, y - gui->yc, &xpos, &ypos);
         clamp_detail_offset(gui, xpos, ypos);
         redraw(gui);
     }
@@ -720,12 +694,13 @@ immerse_view_inside_detail(ModuleGUI *gui, gint x, gint y)
     GwyParams *params = gui->args->params;
     gdouble xpos = gwy_params_get_double(params, PARAM_XPOS);
     gdouble ypos = gwy_params_get_double(params, PARAM_YPOS);
-    gint xoff, yoff, w, h;
+    gdouble xoff, yoff;
+    gint w, h;
 
     if (!gui->detail)
         return FALSE;
 
-    gwy_data_view_coords_real_to_xy(GWY_DATA_VIEW(gui->view), xpos, ypos, &xoff, &yoff);
+    gwy_data_view_coords_real_to_widget(GWY_DATA_VIEW(gui->view), xpos, ypos, &xoff, &yoff);
     w = gdk_pixbuf_get_width(gui->detail);
     h = gdk_pixbuf_get_height(gui->detail);
     return x >= xoff && x < xoff + w && y >= yoff && y < yoff + h;
