@@ -1,5 +1,5 @@
 /*
- *  $Id: indent_analyze.c 26524 2024-08-15 18:14:00Z yeti-dn $
+ *  $Id: indent_analyze.c 28910 2025-11-24 18:11:09Z yeti-dn $
  *  Copyright (C) 2006-2022 Lukas Chvatal, David Necas (Yeti), Petr Klapetek.
  *  E-mail: chvatal@physics.muni.cz, yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,18 +16,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwythreads.h>
-#include <libgwymodule/gwymodule.h>
-#include <libprocess/datafield.h>
-#include <libprocess/gwyprocess.h>
-#include <libgwydgets/gwydgets.h>
-#include <app/settings.h>
-#include <app/app.h>
-#include <app/gwyapp.h>
 #include <gtk/gtk.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES GWY_RUN_INTERACTIVE
@@ -101,11 +93,11 @@ typedef struct {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *adjusted;
-    GwyDataField *xder;
-    GwyDataField *yder;
-    GwyDataField *result[RESULT_NTYPES];
+    GwyField *field;
+    GwyField *adjusted;
+    GwyField *xder;
+    GwyField *yder;
+    GwyField *result[RESULT_NTYPES];
     GwySelection *selection;
     ImprintParameters imp;
 } ModuleArgs;
@@ -113,62 +105,62 @@ typedef struct {
 typedef struct {
     ModuleArgs *args;
     GtkWidget *dialog;
+    GtkWidget *dataview;
     GwyParamTable *table_param;
     GwyParamTable *table_results;
-    GwyContainer *data;
     GwyResults *results;
 } ModuleGUI;
 
 static gboolean         module_register         (void);
 static GwyParamDef*     define_module_params    (void);
-static void             indent_analyze          (GwyContainer *data,
-                                                 GwyRunType runtype);
+static void             module_main             (GwyFile *data,
+                                                 GwyRunModeFlags mode);
 static void             execute                 (ModuleArgs *args);
 static GwyDialogOutcome run_gui                 (ModuleArgs *args,
-                                                 GwyContainer *data,
+                                                 GwyFile *data,
                                                  gint id);
 static GwyResults*      create_results          (ModuleArgs *args,
-                                                 GwyContainer *data,
+                                                 GwyFile *data,
                                                  gint id);
 static void             param_changed           (ModuleGUI *gui,
                                                  gint id);
 static void             preview                 (gpointer user_data);
-static void             level_using_exterior    (GwyDataField *field,
-                                                 GwyDataField *mask,
+static void             level_using_exterior    (GwyField *field,
+                                                 GwyField *mask,
                                                  gdouble wfrac,
                                                  gboolean do_level);
-static void             compute_slopes          (GwyDataField *field,
+static void             compute_slopes          (GwyField *field,
                                                  gint kernel_size,
-                                                 GwyDataField *xder,
-                                                 GwyDataField *yder,
-                                                 GwyDataField *atanfield);
-static gdouble          find_prevalent_phi      (GwyDataField *field,
-                                                 GwyDataField *exclmask,
-                                                 GwyDataField *xder,
-                                                 GwyDataField *yder,
-                                                 GwyDataField *atanfield);
-static void             find_imprint_side_centre(GwyDataField *field,
+                                                 GwyField *xder,
+                                                 GwyField *yder,
+                                                 GwyField *atanfield);
+static gdouble          find_prevalent_phi      (GwyField *field,
+                                                 GwyField *exclmask,
+                                                 GwyField *xder,
+                                                 GwyField *yder,
+                                                 GwyField *atanfield);
+static void             find_imprint_side_centre(GwyField *field,
                                                  gdouble search_to_z,
                                                  gdouble phi,
                                                  gdouble *xcentre,
                                                  gdouble *ycentre);
-static void             calc_mean_normal        (GwyDataField *xder,
-                                                 GwyDataField *yder,
+static void             calc_mean_normal        (GwyField *xder,
+                                                 GwyField *yder,
                                                  gdouble x,
                                                  gdouble y,
                                                  gdouble r,
                                                  GwyXYZ *v);
-static void             mark_facet              (GwyDataField *xder,
-                                                 GwyDataField *yder,
-                                                 GwyDataField *mask,
+static void             mark_facet              (GwyField *xder,
+                                                 GwyField *yder,
+                                                 GwyField *mask,
                                                  gdouble theta0,
                                                  gdouble phi0,
                                                  gdouble tolerance);
-static void             mark_pileup             (GwyDataField *above,
-                                                 GwyDataField *imprint,
-                                                 GwyDataField *pileup);
-static void             calc_surface_and_volume (GwyDataField *field,
-                                                 GwyDataField *mask,
+static void             mark_pileup             (GwyField *above,
+                                                 GwyField *imprint,
+                                                 GwyField *pileup);
+static void             calc_surface_and_volume (GwyField *field,
+                                                 GwyField *mask,
                                                  gdouble *Aproj,
                                                  gdouble *Asurf,
                                                  gdouble *V);
@@ -189,11 +181,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("indent_analyze",
-                              (GwyProcessFunc)&indent_analyze,
+                              module_main,
                               N_("/SPM M_odes/_Force and Indentation/_Analyze Imprint..."),
-                              GWY_STOCK_TIP_INDENT_ANALYZE,
+                              GWY_ICON_TIP_INDENT_ANALYZE,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               _("Analyze indentation imprint"));
 
     return TRUE;
@@ -229,10 +221,10 @@ define_module_params(void)
     gwy_param_def_add_boolean(paramdef, PARAM_DO_LEVEL, "do_level", _("Level using imprint exterior"), TRUE);
     gwy_param_def_add_double(paramdef, PARAM_EXTERIOR, "border", _("Exterior width"), 1.0, 40.0, 5.0);
     gwy_param_def_add_double(paramdef, PARAM_PLANE_TOL, "plane_tol", _("Ref. plane _tolerance"), 0.0, 8.0, 2.0);
-    gwy_param_def_add_double(paramdef, PARAM_PHI_TOL, "phi_tol", _("_Angle tolerance"), 0.0, G_PI, 8.0*G_PI/180.0);
+    gwy_param_def_add_double(paramdef, PARAM_PHI_TOL, "phi_tol", _("_Angle tolerance"), 0.0, G_PI, gwy_deg2rad(8.0));
     gwy_param_def_add_gwyenum(paramdef, PARAM_INDENTER, "indentor", _("_Indenter shape"),
                               indenters, G_N_ELEMENTS(indenters), INDENTER_PYRAMID3);
-    gwy_param_def_add_gwyenum(paramdef, PARAM_DISPLAY, "display", gwy_sgettext("verb|_Display"),
+    gwy_param_def_add_gwyenum(paramdef, PARAM_DISPLAY, "display", C_("verb", "_Display"),
                               displays, G_N_ELEMENTS(displays), RESULT_NOTHING);
     gwy_param_def_add_boolean(paramdef, PARAM_SET_MASK, "set_mask", _("Create _mask"), TRUE);
     gwy_param_def_add_mask_color(paramdef, PARAM_MASK_COLOR, NULL, NULL);
@@ -242,7 +234,7 @@ define_module_params(void)
 }
 
 static void
-indent_analyze(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     IndentDisplayType display;
@@ -251,24 +243,24 @@ indent_analyze(GwyContainer *data, GwyRunType runtype)
     GQuark mquark;
     gint id, i;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     GWY_APP_MASK_FIELD_KEY, &mquark,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_FIELD_ID, &id,
+                                 GWY_APP_MASK_FIELD_KEY, &mquark,
+                                 0);
     g_return_if_fail(args.field && mquark);
 
     if (!gwy_require_image_same_units(args.field, data, id, _("Analyze imprint")))
         return;
 
-    args.adjusted = gwy_data_field_duplicate(args.field);
+    args.adjusted = gwy_field_copy(args.field);
     for (i = 1; i < RESULT_NTYPES; i++) {
-        args.result[i] = gwy_data_field_new_alike(args.field, TRUE);
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(args.result[i]), NULL);
+        args.result[i] = gwy_field_new_alike(args.field, TRUE);
+        gwy_unit_clear(gwy_field_get_unit_z(args.result[i]));
     }
-    args.xder = gwy_data_field_new_alike(args.field, TRUE);
-    args.yder = gwy_data_field_new_alike(args.field, TRUE);
+    args.xder = gwy_field_new_alike(args.field, TRUE);
+    args.yder = gwy_field_new_alike(args.field, TRUE);
     args.params = gwy_params_new_from_settings(define_module_params());
 
     outcome = run_gui(&args, data, id);
@@ -281,12 +273,12 @@ indent_analyze(GwyContainer *data, GwyRunType runtype)
     if (set_mask && display != RESULT_NOTHING) {
         if (outcome != GWY_DIALOG_HAVE_RESULT)
             execute(&args);
-        gwy_app_undo_qcheckpointv(data, 1, &mquark);
-        if (gwy_data_field_get_max(args.result[display]) > 0.0)
-            gwy_container_set_object(data, mquark, args.result[display]);
+        gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &mquark);
+        if (gwy_field_get_max(args.result[display]) > 0.0)
+            gwy_container_set_object(GWY_CONTAINER(data), mquark, args.result[display]);
         else
-            gwy_container_remove(data, mquark);
-        gwy_app_channel_log_add_proc(data, id, id);
+            gwy_container_remove(GWY_CONTAINER(data), mquark);
+        gwy_log_add(data, GWY_FILE_IMAGE, id, id);
     }
 
 end:
@@ -297,11 +289,11 @@ end:
     g_object_unref(args.xder);
     g_object_unref(args.yder);
     /* TODO: We can set the selection on the image. */
-    GWY_OBJECT_UNREF(args.selection);
+    g_clear_object(&args.selection);
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     GwyDialogOutcome outcome;
     GtkWidget *hbox, *dataview;
@@ -309,25 +301,19 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     GwyParamTable *table;
     ModuleGUI gui;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
-    gui.data = gwy_container_new();
     gui.results = create_results(args, data, id);
 
-    gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(0), args->adjusted);
-    gwy_app_sync_data_items(data, gui.data, id, 0, FALSE,
-                            GWY_DATA_ITEM_PALETTE,
-                            GWY_DATA_ITEM_RANGE,
-                            GWY_DATA_ITEM_MASK_COLOR,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
-
     gui.dialog = gwy_dialog_new(_("Analyze Imprint"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, TRUE);
-    args->selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(dataview), 0, "Point", 5, FALSE);
+    dataview = gui.dataview = gwy_create_preview(args->adjusted, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_RANGE
+                        | GWY_FILE_ITEM_MASK_COLOR | GWY_FILE_ITEM_REAL_SQUARE);
+    args->selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(dataview), GWY_TYPE_LAYER_POINT, 5, FALSE);
     g_object_ref(args->selection);
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), FALSE);
 
@@ -346,7 +332,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_param_table_set_unitstr(table, PARAM_PHI_TOL, _("deg"));
     gwy_param_table_append_radio(table, PARAM_DISPLAY);
     gwy_param_table_append_separator(table);
-    gwy_param_table_append_mask_color(table, PARAM_MASK_COLOR, gui.data, 0, data, id);
+    gwy_param_table_append_image_mask_color(table, PARAM_MASK_COLOR, GWY_DATA_VIEW(dataview), data, id);
 
     gtk_box_pack_start(GTK_BOX(hbox), gwy_param_table_widget(table), TRUE, TRUE, 0);
     gwy_dialog_add_param_table(dialog, table);
@@ -374,14 +360,13 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 
     outcome = gwy_dialog_run(dialog);
 
-    g_object_unref(gui.data);
     g_object_unref(gui.results);
 
     return outcome;
 }
 
 static GwyResults*
-create_results(ModuleArgs *args, GwyContainer *data, gint id)
+create_results(ModuleArgs *args, GwyFile *data, gint id)
 {
     GwyResults *results = gwy_results_new();
 
@@ -415,10 +400,10 @@ create_results(ModuleArgs *args, GwyContainer *data, gint id)
                              NULL);
     gwy_results_bind_formats(results, "Vimp", "Vpileup", NULL);
 
-    gwy_results_set_unit(results, "x", gwy_data_field_get_si_unit_xy(args->field));
-    gwy_results_set_unit(results, "z", gwy_data_field_get_si_unit_z(args->field));
+    gwy_results_set_unit(results, "x", gwy_field_get_unit_xy(args->field));
+    gwy_results_set_unit(results, "z", gwy_field_get_unit_z(args->field));
     gwy_results_fill_filename(results, "file", data);
-    gwy_results_fill_channel(results, "image", data, id);
+    gwy_results_fill_data_name(results, "image", data, GWY_FILE_IMAGE, id);
 
     return results;
 }
@@ -431,11 +416,12 @@ param_changed(ModuleGUI *gui, gint id)
 
     if (id < 0 || id == PARAM_DISPLAY) {
         IndentDisplayType display = gwy_params_get_enum(params, PARAM_DISPLAY);
+        GwyDataView *dataview = GWY_DATA_VIEW(gui->dataview);
 
         if (display == RESULT_NOTHING)
-            gwy_container_remove(gui->data, gwy_app_get_mask_key_for_id(0));
+            gwy_data_view_set_mask(dataview, NULL);
         else
-            gwy_container_set_object(gui->data, gwy_app_get_mask_key_for_id(0), args->result[display]);
+            gwy_data_view_set_mask(dataview, args->result[display]);
     }
     if (id < 0 || id == PARAM_INDENTER) {
         GwyIndenterType indenter = gwy_params_get_enum(params, PARAM_INDENTER);
@@ -459,9 +445,9 @@ preview(gpointer user_data)
     gint i;
 
     execute(args);
-    gwy_data_field_data_changed(args->adjusted);
+    gwy_field_data_changed(args->adjusted);
     for (i = 1; i < RESULT_NTYPES; i++)
-        gwy_data_field_data_changed(args->result[i]);
+        gwy_field_data_changed(args->result[i]);
     gwy_results_fill_values(gui->results,
                             "x0", imp->x0, "y0", imp->y0, "zmin", imp->zmin, "zmax", imp->zmax,
                             "Aproj_imp", imp->Aproj_imp, "Asurf_imp", imp->Asurf_imp, "Acontact", imp->Acontact,
@@ -479,35 +465,34 @@ execute(ModuleArgs *args)
 {
     GwyParams *params = args->params;
     ImprintParameters *imp = &args->imp;
-    GwyDataField *adjusted = args->adjusted, *xder = args->xder, *yder = args->yder, *buf, *atanfield, *kernel;
-    GwyDataField *imprint = args->result[RESULT_IMPRINT];
-    GwyDataField *above = args->result[RESULT_ABOVE];
-    GwyDataField *below = args->result[RESULT_BELOW];
-    GwyDataField *impfaces = args->result[RESULT_IMPRINT_FACES];
-    GwyDataField *facesborder = args->result[RESULT_FACES_BORDER];
-    GwyDataField *pileup = args->result[RESULT_PILEUP];
+    GwyField *adjusted = args->adjusted, *xder = args->xder, *yder = args->yder, *buf, *atanfield, *kernel;
+    GwyField *imprint = args->result[RESULT_IMPRINT];
+    GwyField *above = args->result[RESULT_ABOVE];
+    GwyField *below = args->result[RESULT_BELOW];
+    GwyField *impfaces = args->result[RESULT_IMPRINT_FACES];
+    GwyField *facesborder = args->result[RESULT_FACES_BORDER];
+    GwyField *pileup = args->result[RESULT_PILEUP];
     gboolean do_level = gwy_params_get_boolean(params, PARAM_DO_LEVEL);
     gdouble exterior = gwy_params_get_double(params, PARAM_EXTERIOR);
     gdouble plane_tol = gwy_params_get_double(params, PARAM_PLANE_TOL);
     gdouble phi_tol = gwy_params_get_double(params, PARAM_PHI_TOL);
     GwyIndenterType indenter = gwy_params_get_enum(params, PARAM_INDENTER);
-    gint xres = gwy_data_field_get_xres(args->field), yres = gwy_data_field_get_yres(args->field);
+    gint xres = gwy_field_get_xres(args->field), yres = gwy_field_get_yres(args->field);
     gint i, ksize, nsides = indenter;
     gdouble points[2*5];
     gdouble search_to_z, dx, dy;
 
     /* Level (if requested) using exterior and find the base plane and its rms. */
-    gwy_data_field_copy(args->field, adjusted, FALSE);
-    dx = gwy_data_field_get_dx(adjusted);
-    dy = gwy_data_field_get_dy(adjusted);
+    gwy_field_copy_data(args->field, adjusted);
+    dx = gwy_field_get_dx(adjusted);
+    dy = gwy_field_get_dy(adjusted);
     level_using_exterior(adjusted, args->result[RESULT_EXTERIOR], exterior/100.0, do_level);
-    if (!gwy_data_field_get_local_minima_list(adjusted, &imp->x0, &imp->y0, &imp->zmin, 1, 0, G_MAXDOUBLE, TRUE)) {
+    if (!gwy_field_get_local_minima_list(adjusted, &imp->x0, &imp->y0, &imp->zmin, 1, 0, G_MAXDOUBLE, TRUE)) {
         g_printerr("cannot find any minimum! fixme to handle this!\n");
         return;
     }
-    imp->zmax = gwy_data_field_get_max(adjusted);
-    imp->rms_base = gwy_data_field_area_get_rms_mask(adjusted, args->result[RESULT_EXTERIOR], GWY_MASK_INCLUDE,
-                                                     0, 0, xres, yres);
+    imp->zmax = gwy_field_get_max(adjusted);
+    imp->rms_base = gwy_field_area_get_rms(adjusted, args->result[RESULT_EXTERIOR], GWY_MASK_INCLUDE, 0, 0, xres, yres);
     plane_tol *= imp->rms_base;
 
     /* Calculate slopes and find the imprint face. */
@@ -515,7 +500,7 @@ execute(ModuleArgs *args)
     atanfield = args->result[RESULT_OUTER_PILEUP];
     buf = args->result[RESULT_FACES_BORDER];
     compute_slopes(adjusted, 5, xder, yder, atanfield);
-    gwy_data_field_clear(impfaces);
+    gwy_field_clear(impfaces);
     imp->phi = find_prevalent_phi(adjusted, impfaces, xder, yder, atanfield);
     search_to_z = imp->zmin - 0.3*imp->zmin;
     gwy_debug("search side from to %g within [%g..0.0]", search_to_z, imp->zmin);
@@ -531,49 +516,49 @@ execute(ModuleArgs *args)
             calc_mean_normal(xder, yder, xc, yc, 4.5, &v);
             phi0 = atan2(v.y, v.x);
             theta0 = atan(hypot(v.x, v.y));
-            gwy_debug("mean normal phi0=%g, theta0=%g", 180.0/G_PI*phi0, 180.0/G_PI*theta0);
+            gwy_debug("mean normal phi0=%g, theta0=%g", gwy_rad2deg(phi0), gwy_rad2deg(theta0));
             mark_facet(xder, yder, buf, theta0, phi0, phi_tol);
-            gwy_data_field_grains_extract_grain(buf, (gint)floor(xc), (gint)floor(yc));
-            gwy_data_field_fill_voids(buf, TRUE);
-            gwy_data_field_max_of_fields(impfaces, buf, impfaces);
+            gwy_field_grains_extract_grain(buf, (gint)floor(xc), (gint)floor(yc));
+            gwy_field_fill_voids(buf, TRUE);
+            gwy_field_max_of_fields(impfaces, buf, impfaces);
         }
     }
 
     /* Mark various derived regions. */
-    gwy_data_field_copy(adjusted, above, FALSE);
-    gwy_data_field_copy(adjusted, below, FALSE);
-    gwy_data_field_threshold(above, plane_tol, 0.0, 1.0);
-    gwy_data_field_threshold(below, -plane_tol, 1.0, 0.0);
-    gwy_data_field_max_of_fields(args->result[RESULT_PLANE], above, below);
-    gwy_data_field_threshold(args->result[RESULT_PLANE], 0.5, 1.0, 0.0);
+    gwy_field_copy_data(adjusted, above);
+    gwy_field_copy_data(adjusted, below);
+    gwy_field_threshold(above, plane_tol, 0.0, 1.0);
+    gwy_field_threshold(below, -plane_tol, 1.0, 0.0);
+    gwy_field_max_of_fields(args->result[RESULT_PLANE], above, below);
+    gwy_field_threshold(args->result[RESULT_PLANE], 0.5, 1.0, 0.0);
 
-    gwy_data_field_copy(below, imprint, FALSE);
-    gwy_data_field_grains_extract_grain(imprint, (gint)floor(imp->x0), (gint)floor(imp->y0));
-    gwy_data_field_fill_voids(imprint, TRUE);
+    gwy_field_copy_data(below, imprint);
+    gwy_field_grains_extract_grain(imprint, (gint)floor(imp->x0), (gint)floor(imp->y0));
+    gwy_field_fill_voids(imprint, TRUE);
     /* FIXME */
     if (!nsides)
-        gwy_data_field_copy(imprint, impfaces, FALSE);
+        gwy_field_copy_data(imprint, impfaces);
 
     mark_pileup(above, imprint, pileup);
-    gwy_data_field_min_of_fields(args->result[RESULT_INNER_PILEUP], pileup, impfaces);
-    gwy_data_field_subtract_fields(args->result[RESULT_OUTER_PILEUP], pileup, impfaces);
-    gwy_data_field_threshold(args->result[RESULT_INNER_PILEUP], 0.5, 0.0, 1.0);
+    gwy_field_min_of_fields(args->result[RESULT_INNER_PILEUP], pileup, impfaces);
+    gwy_field_subtract_fields(args->result[RESULT_OUTER_PILEUP], pileup, impfaces);
+    gwy_field_threshold(args->result[RESULT_INNER_PILEUP], 0.5, 0.0, 1.0);
 
     ksize = GWY_ROUND(sqrt(xres*yres)/150.0);
     ksize = MAX(ksize, 2);
-    kernel = gwy_data_field_new(ksize, ksize, 1.0, 1.0, TRUE);
-    gwy_data_field_elliptic_area_fill(kernel, 0, 0, ksize, ksize, 1.0);
-    gwy_data_field_copy(impfaces, args->result[RESULT_CONTACT_AREA], FALSE);
-    gwy_data_field_area_filter_min_max(args->result[RESULT_CONTACT_AREA], kernel, GWY_MIN_MAX_FILTER_CLOSING,
-                                       0, 0, xres, yres);
+    kernel = gwy_field_new(ksize, ksize, 1.0, 1.0, TRUE);
+    gwy_field_elliptic_area_fill(kernel, 0, 0, ksize, ksize, 1.0);
+    gwy_field_copy_data(impfaces, args->result[RESULT_CONTACT_AREA]);
+    gwy_field_area_filter_min_max(args->result[RESULT_CONTACT_AREA], kernel, GWY_MIN_MAX_FILTER_CLOSING,
+                                  0, 0, xres, yres);
     g_object_unref(kernel);
-    gwy_data_field_fill_voids(args->result[RESULT_CONTACT_AREA], TRUE);
+    gwy_field_fill_voids(args->result[RESULT_CONTACT_AREA], TRUE);
 
-    gwy_data_field_copy(impfaces, facesborder, FALSE);
-    gwy_data_field_grains_grow(facesborder, 1.0, GWY_DISTANCE_TRANSFORM_CHESS, FALSE);
-    gwy_data_field_threshold(facesborder, 0.5, 0.0, 1.0);
-    gwy_data_field_subtract_fields(facesborder, facesborder, impfaces);
-    gwy_data_field_threshold(facesborder, 0.5, 0.0, 1.0);
+    gwy_field_copy_data(impfaces, facesborder);
+    gwy_field_grains_grow(facesborder, 1.0, GWY_DISTANCE_TRANSFORM_CHESS, FALSE);
+    gwy_field_threshold(facesborder, 0.5, 0.0, 1.0);
+    gwy_field_subtract_fields(facesborder, facesborder, impfaces);
+    gwy_field_threshold(facesborder, 0.5, 0.0, 1.0);
 
     /* Calculate surfaces and volumes. */
     calc_surface_and_volume(adjusted, impfaces, &imp->Aproj_imp, &imp->Asurf_imp, &imp->Vimp);
@@ -596,49 +581,49 @@ execute(ModuleArgs *args)
 
 /* Always make average height over the exterior zero.  Plane-level only if requested. */
 static void
-level_using_exterior(GwyDataField *field, GwyDataField *mask, gdouble wfrac, gboolean do_level)
+level_using_exterior(GwyField *field, GwyField *mask, gdouble wfrac, gboolean do_level)
 {
-    gint xres = gwy_data_field_get_xres(field), yres = gwy_data_field_get_yres(field);
+    gint xres = gwy_field_get_xres(field), yres = gwy_field_get_yres(field);
     gint wx = MAX(GWY_ROUND(xres*wfrac), 1);
     gint wy = MAX(GWY_ROUND(yres*wfrac), 1);
     gdouble a, bx, by;
 
-    gwy_data_field_clear(mask);
-    gwy_data_field_area_fill(mask, 0, 0, xres, wy, 1.0);
-    gwy_data_field_area_fill(mask, 0, 0, wx, yres, 1.0);
-    gwy_data_field_area_fill(mask, 0, yres-wy, xres, wy, 1.0);
-    gwy_data_field_area_fill(mask, xres-wx, 0, wx, yres, 1.0);
+    gwy_field_clear(mask);
+    gwy_field_area_fill(mask, NULL, GWY_MASK_IGNORE, 0, 0, xres, wy, 1.0);
+    gwy_field_area_fill(mask, NULL, GWY_MASK_IGNORE, 0, 0, wx, yres, 1.0);
+    gwy_field_area_fill(mask, NULL, GWY_MASK_IGNORE, 0, yres-wy, xres, wy, 1.0);
+    gwy_field_area_fill(mask, NULL, GWY_MASK_IGNORE, xres-wx, 0, wx, yres, 1.0);
     if (do_level) {
-        gwy_data_field_area_fit_plane_mask(field, mask, GWY_MASK_INCLUDE, 0, 0, xres, yres, &a, &bx, &by);
+        gwy_field_area_fit_plane(field, mask, GWY_MASK_INCLUDE, 0, 0, xres, yres, &a, &bx, &by);
         gwy_debug("fitted plane bx=%g, by=%g", bx, by);
-        gwy_data_field_plane_level(field, a, bx, by);
+        gwy_field_plane_level(field, a, bx, by);
     }
-    a = gwy_data_field_area_get_avg_mask(field, mask, GWY_MASK_INCLUDE, 0, 0, xres, yres);
-    gwy_data_field_add(field, -a);
+    a = gwy_field_area_get_avg(field, mask, GWY_MASK_INCLUDE, 0, 0, xres, yres);
+    gwy_field_add(field, -a);
 }
 
 static void
-compute_slopes(GwyDataField *field, gint kernel_size,
-               GwyDataField *xder, GwyDataField *yder, GwyDataField *atanfield)
+compute_slopes(GwyField *field, gint kernel_size,
+               GwyField *xder, GwyField *yder, GwyField *atanfield)
 {
-    gint xres = gwy_data_field_get_xres(field), yres = gwy_data_field_get_yres(field);
+    gint xres = gwy_field_get_xres(field), yres = gwy_field_get_yres(field);
     GwyPlaneFitQuantity quantites[2] = { GWY_PLANE_FIT_BX, GWY_PLANE_FIT_BY };
-    GwyDataField *fields[2] = { xder, yder };
+    GwyField *fields[2] = { xder, yder };
     const gdouble *xd, *yd;
     gdouble *a;
     gint i, n = xres*yres;
 
     if (!kernel_size)
-        gwy_data_field_filter_slope(field, xder, yder);
+        gwy_field_filter_slope(field, xder, yder);
     else {
-        gwy_data_field_fit_local_planes(field, kernel_size, 2, quantites, fields);
-        gwy_data_field_multiply(xder, 1.0/gwy_data_field_get_dx(field));
-        gwy_data_field_multiply(yder, 1.0/gwy_data_field_get_dy(field));
+        gwy_field_fit_local_planes(field, kernel_size, 2, quantites, fields);
+        gwy_field_multiply(xder, 1.0/gwy_field_get_dx(field));
+        gwy_field_multiply(yder, 1.0/gwy_field_get_dy(field));
     }
 
-    xd = gwy_data_field_get_data_const(xder);
-    yd = gwy_data_field_get_data_const(yder);
-    a = gwy_data_field_get_data(atanfield);
+    xd = gwy_field_get_data_const(xder);
+    yd = gwy_field_get_data_const(yder);
+    a = gwy_field_get_data(atanfield);
 #ifdef _OPENMP
 #pragma omp parallel for if(gwy_threads_are_enabled()) default(none) \
             private(i) \
@@ -650,27 +635,27 @@ compute_slopes(GwyDataField *field, gint kernel_size,
 }
 
 static gdouble
-find_prevalent_phi(GwyDataField *field, GwyDataField *exclmask,
-                   GwyDataField *xder, GwyDataField *yder, GwyDataField *atanfield)
+find_prevalent_phi(GwyField *field, GwyField *exclmask,
+                   GwyField *xder, GwyField *yder, GwyField *atanfield)
 {
-    const gdouble *xd = gwy_data_field_get_data_const(xder);
-    const gdouble *yd = gwy_data_field_get_data_const(yder);
-    const gdouble *m = gwy_data_field_get_data_const(exclmask);
-    const gdouble *a = gwy_data_field_get_data_const(atanfield);
-    gint xres = gwy_data_field_get_xres(field), yres = gwy_data_field_get_yres(field);
-    GwyDataLine *phidist;
+    const gdouble *xd = gwy_field_get_data_const(xder);
+    const gdouble *yd = gwy_field_get_data_const(yder);
+    const gdouble *m = gwy_field_get_data_const(exclmask);
+    const gdouble *a = gwy_field_get_data_const(atanfield);
+    gint xres = gwy_field_get_xres(field), yres = gwy_field_get_yres(field);
+    GwyLine *phidist;
     gdouble *data;
     gdouble d, phi, x, y[3];
     gint n, i, iphi, size;
 
     n = xres*yres;
-    gwy_data_field_area_count_in_range(exclmask, NULL, 0, 0, xres, yres, 0.5, 0.5, &size, NULL);
+    gwy_field_area_count_in_range(exclmask, NULL, GWY_MASK_IGNORE, 0, 0, xres, yres, 0.5, 0.5, &size, NULL);
     size = (gint)floor(5.49*cbrt(size) + 0.5);
     size = MAX(size, 24);
     gwy_debug("phi dist size %d", size);
 
-    phidist = gwy_data_line_new(size, 2.0*G_PI, TRUE);
-    data = gwy_data_line_get_data(phidist);
+    phidist = gwy_line_new(size, 2.0*G_PI, TRUE);
+    data = gwy_line_get_data(phidist);
     for (i = 0; i < n; i++) {
         if (m[i] <= 0.0) {
             d = xd[i]*xd[i] + yd[i]*yd[i];
@@ -679,16 +664,16 @@ find_prevalent_phi(GwyDataField *field, GwyDataField *exclmask,
             data[iphi] += sqrt(d);
         }
     }
-    i = (gint)gwy_data_line_max_pos_i(phidist);
+    i = (gint)gwy_line_max_pos_i(phidist);
 
     gwy_debug("phi maximum at %d (%g deg)", i, 360.0*(i + 0.5)/size);
-    y[0] = gwy_data_line_get_val(phidist, (i + size-1) % size);
-    y[1] = gwy_data_line_get_val(phidist, i);
-    y[2] = gwy_data_line_get_val(phidist, (i+1) % size);
+    y[0] = gwy_line_get_val(phidist, (i + size-1) % size);
+    y[1] = gwy_line_get_val(phidist, i);
+    y[2] = gwy_line_get_val(phidist, (i+1) % size);
     x = 0.0;
     gwy_math_refine_maximum_1d(y, &x);
-    phi = gwy_data_line_get_dx(phidist)*(i + 0.5 + x);
-    gwy_debug("refined phi to %g deg", phi*180.0/G_PI);
+    phi = gwy_line_get_dx(phidist)*(i + 0.5 + x);
+    gwy_debug("refined phi to %g deg", gwy_rad2deg(phi));
 
     g_object_unref(phidist);
 
@@ -696,16 +681,16 @@ find_prevalent_phi(GwyDataField *field, GwyDataField *exclmask,
 }
 
 static void
-find_imprint_side_centre(GwyDataField *field,
+find_imprint_side_centre(GwyField *field,
                          gdouble search_to_z, gdouble phi,
                          gdouble *xcentre, gdouble *ycentre)
 {
-    gint xres = gwy_data_field_get_xres(field), yres = gwy_data_field_get_yres(field);
+    gint xres = gwy_field_get_xres(field), yres = gwy_field_get_yres(field);
     gdouble x, y, cphi = cos(phi), sphi = sin(phi);
-    const gdouble *d = gwy_data_field_get_data_const(field);
+    const gdouble *d = gwy_field_get_data_const(field);
     gint i, j, k;
 
-    gwy_debug("direction %g deg", 180.0/G_PI*phi);
+    gwy_debug("direction %g deg", gwy_rad2deg(phi));
 
     /* Go from the minimum until we reach search_to_z. */
     k = 1;
@@ -727,11 +712,11 @@ find_imprint_side_centre(GwyDataField *field,
 }
 
 static void
-calc_mean_normal(GwyDataField *xder, GwyDataField *yder, gdouble x, gdouble y, gdouble r, GwyXYZ *v)
+calc_mean_normal(GwyField *xder, GwyField *yder, gdouble x, gdouble y, gdouble r, GwyXYZ *v)
 {
-    const gdouble *xd = gwy_data_field_get_data_const(xder);
-    const gdouble *yd = gwy_data_field_get_data_const(yder);
-    gint xres = gwy_data_field_get_xres(xder), yres = gwy_data_field_get_yres(yder);
+    const gdouble *xd = gwy_field_get_data_const(xder);
+    const gdouble *yd = gwy_field_get_data_const(yder);
+    gint xres = gwy_field_get_xres(xder), yres = gwy_field_get_yres(yder);
     gint j = (gint)floor(x);
     gint i = (gint)floor(y);
     gint ii, jj, k, G_GNUC_UNUSED n;
@@ -761,18 +746,18 @@ calc_mean_normal(GwyDataField *xder, GwyDataField *yder, gdouble x, gdouble y, g
     v->y /= s;
     v->z /= s;
     gwy_debug("mean local normal (%g,%g,%g) direction %g deg (%d samples)",
-              v->x, v->y, v->z, 180.0/G_PI*gwy_canonicalize_angle(atan2(-v->y, v->x), TRUE, TRUE), n);
+              v->x, v->y, v->z, gwy_rad2deg(gwy_canonicalize_angle(atan2(-v->y, v->x), TRUE, TRUE)), n);
 }
 
 static void
-mark_facet(GwyDataField *xder, GwyDataField *yder, GwyDataField *mask,
+mark_facet(GwyField *xder, GwyField *yder, GwyField *mask,
            gdouble theta0, gdouble phi0, gdouble tolerance)
 {
     gdouble ctol = cos(tolerance), cth0 = cos(theta0), sth0 = sin(theta0), cphi0 = cos(phi0), sphi0 = sin(phi0);
-    const gdouble *xd = gwy_data_field_get_data_const(xder);
-    const gdouble *yd = gwy_data_field_get_data_const(yder);
-    gdouble *md = gwy_data_field_get_data(mask);
-    guint i, n = gwy_data_field_get_xres(xder)*gwy_data_field_get_yres(xder);
+    const gdouble *xd = gwy_field_get_data_const(xder);
+    const gdouble *yd = gwy_field_get_data_const(yder);
+    gdouble *md = gwy_field_get_data(mask);
+    guint i, n = gwy_field_get_xres(xder)*gwy_field_get_yres(xder);
 
 #ifdef _OPENMP
 #pragma omp parallel for if(gwy_threads_are_enabled()) default(none) \
@@ -791,7 +776,7 @@ mark_facet(GwyDataField *xder, GwyDataField *yder, GwyDataField *mask,
 
 #if 0
 static gboolean
-fit_pyramid(GwyDataField *field, GwyDataField *impfaces,
+fit_pyramid(GwyField *field, GwyField *impfaces,
             GwyIndenterType indenter, PyramidParameters *pyrpar)
 {
     static const GwyEnum indenters[] = {
@@ -815,7 +800,7 @@ fit_pyramid(GwyDataField *field, GwyDataField *impfaces,
         return FALSE;
 
     surface = gwy_surface_new();
-    gwy_surface_set_from_data_field(surface, field);
+    gwy_surface_set_from_field(surface, field);
     points = gwy_surface_get_data_const(surface);
     npoints = gwy_surface_get_npoints(surface);
     nparams = gwy_shape_fit_preset_get_nparams(preset);
@@ -846,7 +831,7 @@ fit_pyramid(GwyDataField *field, GwyDataField *impfaces,
 end:
     if (fitter)
         gwy_math_nlfit_free(fitter);
-    GWY_OBJECT_UNREF(surface);
+    g_clear_object(&surface);
     g_free(params);
 
     return ok;
@@ -855,24 +840,24 @@ end:
 
 /* Keep connected components not too far from imprint in pileup. */
 static void
-mark_pileup(GwyDataField *above, GwyDataField *imprint, GwyDataField *pileup)
+mark_pileup(GwyField *above, GwyField *imprint, GwyField *pileup)
 {
-    gint xres = gwy_data_field_get_xres(imprint), yres = gwy_data_field_get_yres(imprint);
+    gint xres = gwy_field_get_xres(imprint), yres = gwy_field_get_yres(imprint);
     gint ngrains, i, n = xres*yres;
     gint *grains, *keep;
     gdouble *d;
     gdouble maxdist;
 
-    maxdist = fmax(0.02*sqrt(gwy_data_field_get_sum(imprint)), 2.1);
+    maxdist = fmax(0.02*sqrt(gwy_field_get_sum(imprint)), 2.1);
     gwy_debug("using maxdist %g", maxdist);
 
-    gwy_data_field_copy(imprint, pileup, FALSE);
-    gwy_data_field_grains_invert(pileup);
-    gwy_data_field_grain_simple_dist_trans(pileup, GWY_DISTANCE_TRANSFORM_EUCLIDEAN, FALSE);
-    d = gwy_data_field_get_data(pileup);
+    gwy_field_copy_data(imprint, pileup);
+    gwy_field_grains_invert(pileup);
+    gwy_field_grain_simple_dist_trans(pileup, GWY_DISTANCE_TRANSFORM_EUCLIDEAN, FALSE);
+    d = gwy_field_get_data(pileup);
 
     grains = g_new0(gint, n);
-    ngrains = gwy_data_field_number_grains(above, grains);
+    ngrains = gwy_field_number_grains(above, grains);
     keep = g_new0(gint, ngrains+1);
     for (i = 0; i < n; i++)
         keep[grains[i]] |= (d[i] > 0.0 && d[i] < maxdist);
@@ -885,18 +870,18 @@ mark_pileup(GwyDataField *above, GwyDataField *imprint, GwyDataField *pileup)
 }
 
 static void
-calc_surface_and_volume(GwyDataField *field, GwyDataField *mask,
+calc_surface_and_volume(GwyField *field, GwyField *mask,
                         gdouble *Aproj, gdouble *Asurf, gdouble *V)
 {
-    gint xres = gwy_data_field_get_xres(field), yres = gwy_data_field_get_yres(field);
-    gdouble dA = gwy_data_field_get_dx(field)*gwy_data_field_get_dy(field);
+    gint xres = gwy_field_get_xres(field), yres = gwy_field_get_yres(field);
+    gdouble dA = gwy_field_get_dx(field)*gwy_field_get_dy(field);
 
     if (Aproj)
-        *Aproj = dA*gwy_data_field_get_sum(mask);
+        *Aproj = dA*gwy_field_get_sum(mask);
     if (Asurf)
-        *Asurf = gwy_data_field_area_get_surface_area_mask(field, mask, GWY_MASK_INCLUDE, 0, 0, xres, yres);
+        *Asurf = gwy_field_area_get_surface_area(field, mask, GWY_MASK_INCLUDE, 0, 0, xres, yres);
     if (V)
-        *V = gwy_data_field_area_get_volume(field, NULL, mask, 0, 0, xres, yres);
+        *V = gwy_field_area_get_volume(field, NULL, mask, GWY_MASK_INCLUDE, 0, 0, xres, yres);
 }
 
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
