@@ -1,5 +1,5 @@
 /*
- *  $Id: level.c 24796 2022-04-28 15:20:59Z yeti-dn $
+ *  $Id: level.c 28821 2025-11-06 16:00:41Z yeti-dn $
  *  Copyright (C) 2003-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,17 +16,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/level.h>
-#include <libprocess/grains.h>
-#include <libprocess/stats.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
 
@@ -38,9 +31,8 @@ static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
 static GwyDialogOutcome run_gui             (GwyParams *params,
                                              const gchar *funcname);
-static void             level_func          (GwyContainer *data,
-                                             GwyRunType run,
-                                             const gchar *funcname);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
@@ -58,39 +50,39 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("level",
-                              &level_func,
+                              module_main,
                               N_("/_Level/Plane _Level"),
-                              GWY_STOCK_LEVEL,
+                              GWY_ICON_LEVEL,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Level data by mean plane subtraction"));
     gwy_process_func_register("level_rotate",
-                              &level_func,
+                              module_main,
                               N_("/_Level/Level _Rotate"),
                               NULL,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Automatically level data by plane rotation"));
     gwy_process_func_register("fix_zero",
-                              &level_func,
+                              module_main,
                               N_("/_Level/Fix _Zero"),
-                              GWY_STOCK_FIX_ZERO,
+                              GWY_ICON_FIX_ZERO,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Shift minimum data value to zero"));
     gwy_process_func_register("zero_mean",
-                              &level_func,
+                              module_main,
                               N_("/_Level/Zero _Mean Value"),
-                              GWY_STOCK_ZERO_MEAN,
+                              GWY_ICON_ZERO_MEAN,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Shift mean data value to zero"));
     gwy_process_func_register("zero_max",
-                              &level_func,
+                              module_main,
                               N_("/_Level/Zero Ma_ximum Value"),
-                              GWY_STOCK_ZERO_MAXIMUM,
+                              GWY_ICON_ZERO_MAXIMUM,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Shift maximum data value to zero"));
 
     return TRUE;
@@ -110,31 +102,30 @@ define_module_params(void)
 }
 
 static void
-level_func(GwyContainer *data,
-           GwyRunType run,
-           const gchar *funcname)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
+    const gchar *funcname = gwy_process_func_current();
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     GwyMaskingType masking;
     GwyParams *params;
-    GwyDataField *dfield, *mask;
+    GwyField *dfield, *mask;
     gdouble c, bx, by;
     gint xres, yres;
     GQuark quark;
     gint id;
 
-    g_return_if_fail(run & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD_KEY, &quark,
-                                     GWY_APP_DATA_FIELD, &dfield,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     GWY_APP_MASK_FIELD, &mask,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD_KEY, &quark,
+                                 GWY_APP_FIELD, &dfield,
+                                 GWY_APP_FIELD_ID, &id,
+                                 GWY_APP_MASK_FIELD, &mask,
+                                 0);
     g_return_if_fail(dfield && quark);
 
     /* We have multiple functions with the same set of parameters. */
     gwy_param_def_set_function_name(define_module_params(), funcname);
     params = gwy_params_new_from_settings(define_module_params());
-    if (run != GWY_RUN_IMMEDIATE && mask) {
+    if (mode != GWY_RUN_IMMEDIATE && mask) {
         outcome = run_gui(params, funcname);
         gwy_params_save_to_settings(params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -142,66 +133,52 @@ level_func(GwyContainer *data,
     }
     masking = gwy_params_get_masking(params, PARAM_MASKING, &mask);
 
-    xres = gwy_data_field_get_xres(dfield);
-    yres = gwy_data_field_get_yres(dfield);
+    xres = gwy_field_get_xres(dfield);
+    yres = gwy_field_get_yres(dfield);
 
-    gwy_app_undo_qcheckpoint(data, quark, NULL);
+    gwy_app_undo_qcheckpoint(GWY_CONTAINER(data), quark, NULL);
 
     if (gwy_stramong(funcname, "level", "level_rotate", NULL)) {
-        if (mask) {
-            if (masking == GWY_MASK_EXCLUDE) {
-                mask = gwy_data_field_duplicate(mask);
-                gwy_data_field_grains_invert(mask);
-            }
-            else
-                g_object_ref(mask);
-        }
-
-        if (mask)
-            gwy_data_field_area_fit_plane(dfield, mask, 0, 0, xres, yres, &c, &bx, &by);
-        else
-            gwy_data_field_fit_plane(dfield, &c, &bx, &by);
-
+        gwy_field_area_fit_plane(dfield, mask, masking, 0, 0, xres, yres, &c, &bx, &by);
         if (gwy_strequal(funcname, "level_rotate")) {
-            bx = gwy_data_field_rtoj(dfield, bx);
-            by = gwy_data_field_rtoi(dfield, by);
-            gwy_data_field_plane_rotate(dfield, atan2(bx, 1), atan2(by, 1), GWY_INTERPOLATION_LINEAR);
+            bx = gwy_field_rtoj(dfield, bx);
+            by = gwy_field_rtoi(dfield, by);
+            gwy_field_plane_rotate(dfield, atan2(bx, 1), atan2(by, 1), GWY_INTERPOLATION_LINEAR);
             gwy_debug("b = %g, alpha = %g deg, c = %g, beta = %g deg",
-                      bx, 180/G_PI*atan2(bx, 1), by, 180/G_PI*atan2(by, 1));
+                      bx, gwy_rad2deg(atan2(bx, 1)), by, gwy_rad2deg(atan2(by, 1)));
         }
         else {
-            c = -0.5*(bx*gwy_data_field_get_xres(dfield) + by*gwy_data_field_get_yres(dfield));
-            gwy_data_field_plane_level(dfield, c, bx, by);
+            c = -0.5*(bx*gwy_field_get_xres(dfield) + by*gwy_field_get_yres(dfield));
+            gwy_field_plane_level(dfield, c, bx, by);
         }
-        GWY_OBJECT_UNREF(mask);
     }
     else if (gwy_strequal(funcname, "fix_zero")) {
         if (mask)
-            gwy_data_field_area_get_min_max_mask(dfield, mask, masking, 0, 0, xres, yres, &c, NULL);
+            gwy_field_area_get_min_max(dfield, mask, masking, 0, 0, xres, yres, &c, NULL);
         else
-            c = gwy_data_field_get_min(dfield);
-        gwy_data_field_add(dfield, -c);
+            c = gwy_field_get_min(dfield);
+        gwy_field_add(dfield, -c);
     }
     else if (gwy_strequal(funcname, "zero_mean")) {
         if (mask)
-            c = gwy_data_field_area_get_avg_mask(dfield, mask, masking, 0, 0, xres, yres);
+            c = gwy_field_area_get_avg(dfield, mask, masking, 0, 0, xres, yres);
         else
-            c = gwy_data_field_get_avg(dfield);
-        gwy_data_field_add(dfield, -c);
+            c = gwy_field_get_avg(dfield);
+        gwy_field_add(dfield, -c);
     }
     else if (gwy_strequal(funcname, "zero_max")) {
         if (mask)
-            gwy_data_field_area_get_min_max_mask(dfield, mask, masking, 0, 0, xres, yres, NULL, &c);
+            gwy_field_area_get_min_max(dfield, mask, masking, 0, 0, xres, yres, NULL, &c);
         else
-            c = gwy_data_field_get_max(dfield);
-        gwy_data_field_add(dfield, -c);
+            c = gwy_field_get_max(dfield);
+        gwy_field_add(dfield, -c);
     }
     else {
         g_assert_not_reached();
     }
 
-    gwy_app_channel_log_add_proc(data, id, id);
-    gwy_data_field_data_changed(dfield);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, id);
+    gwy_field_data_changed(dfield);
 
 end:
     g_object_unref(params);
