@@ -1,5 +1,5 @@
 /*
- *  $Id: level_grains.c 23666 2021-05-10 21:56:10Z yeti-dn $
+ *  $Id: level_grains.c 28873 2025-11-18 14:59:29Z yeti-dn $
  *  Copyright (C) 2011-2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,15 +16,9 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/stats.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/gwygrainvalue.h>
-#include <libprocess/correct.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 
 #define RUN_MODES (GWY_RUN_INTERACTIVE | GWY_RUN_IMMEDIATE)
 
@@ -35,10 +29,10 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *mask;
-    GwyDataField *result;
-    GwyDataField *bg;
+    GwyField *field;
+    GwyField *mask;
+    GwyField *result;
+    GwyField *bg;
 } ModuleArgs;
 
 typedef struct {
@@ -49,8 +43,8 @@ typedef struct {
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             level_grains        (GwyContainer *data,
-                                             GwyRunType runtype);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static void             execute             (ModuleArgs *args);
 static GwyDialogOutcome run_gui             (ModuleArgs *args);
 
@@ -70,11 +64,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("level_grains",
-                              (GwyProcessFunc)&level_grains,
+                              module_main,
                               N_("/_Grains/_Level Grains..."),
                               NULL,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA_MASK | GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE_MASK | GWY_MENU_FLAG_IMAGE,
                               N_("Level individual grains, interpolating the shifts between using Laplacian "
                                  "interpolation"));
 
@@ -85,12 +79,12 @@ static GwyParamDef*
 define_module_params(void)
 {
     static GwyEnum bases[] = {
-        { NULL, GWY_GRAIN_VALUE_MINIMUM,          },
-        { NULL, GWY_GRAIN_VALUE_MAXIMUM,          },
-        { NULL, GWY_GRAIN_VALUE_MEAN,             },
-        { NULL, GWY_GRAIN_VALUE_MEDIAN,           },
-        { NULL, GWY_GRAIN_VALUE_BOUNDARY_MINIMUM, },
-        { NULL, GWY_GRAIN_VALUE_BOUNDARY_MAXIMUM, },
+        { NULL, GWY_GRAIN_MINIMUM,          },
+        { NULL, GWY_GRAIN_MAXIMUM,          },
+        { NULL, GWY_GRAIN_MEAN,             },
+        { NULL, GWY_GRAIN_MEDIAN,           },
+        { NULL, GWY_GRAIN_BOUNDARY_MINIMUM, },
+        { NULL, GWY_GRAIN_BOUNDARY_MAXIMUM, },
     };
     static GwyParamDef *paramdef = NULL;
     guint i;
@@ -106,47 +100,50 @@ define_module_params(void)
     paramdef = gwy_param_def_new();
     gwy_param_def_set_function_name(paramdef, gwy_process_func_current());
     gwy_param_def_add_gwyenum(paramdef, PARAM_BASE, "base", _("Quantity to level"),
-                              bases, G_N_ELEMENTS(bases), GWY_GRAIN_VALUE_MINIMUM);
+                              bases, G_N_ELEMENTS(bases), GWY_GRAIN_MINIMUM);
     gwy_param_def_add_boolean(paramdef, PARAM_DO_EXTRACT, "do_extract", _("E_xtract background"), FALSE);
     return paramdef;
 }
 
 static void
-level_grains(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyParams *params;
     ModuleArgs args;
     GQuark quark;
     gint id, newid;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD_KEY, &quark,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_MASK_FIELD, &args.mask,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD_KEY, &quark,
+                                 GWY_APP_FIELD_ID, &id,
+                                 GWY_APP_FIELD, &args.field,
+                                 GWY_APP_MASK_FIELD, &args.mask,
+                                 0);
     g_return_if_fail(args.field && quark);
 
     args.result = g_object_ref(args.field);   /* Change to a copy once we have a preview. */
-    args.bg = gwy_data_field_new_alike(args.field, FALSE);
+    args.bg = gwy_field_new_alike(args.field, FALSE);
     args.params = params = gwy_params_new_from_settings(define_module_params());
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         GwyDialogOutcome outcome = run_gui(&args);
         gwy_params_save_to_settings(params);
         if (outcome != GWY_DIALOG_PROCEED)
             goto end;
     }
-    gwy_app_undo_qcheckpointv(data, 1, &quark);
+    gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &quark);
     execute(&args);
-    gwy_app_channel_log_add_proc(data, id, id);
-    gwy_data_field_data_changed(args.field);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, id);
+    gwy_field_data_changed(args.field);
 
     if (gwy_params_get_boolean(params, PARAM_DO_EXTRACT)) {
-        newid = gwy_app_data_browser_add_data_field(args.bg, data, TRUE);
-        gwy_app_sync_data_items(data, data, id, newid, FALSE, GWY_DATA_ITEM_GRADIENT, 0);
-        gwy_app_set_data_field_title(data, newid, _("Background"));
-        gwy_app_channel_log_add_proc(data, id, newid);
+        newid = gwy_file_add_image(data, args.bg);
+        gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+        gwy_file_sync_items(data, GWY_FILE_IMAGE, id,
+                            data, GWY_FILE_IMAGE, newid,
+                            GWY_FILE_ITEM_PALETTE, FALSE);
+        gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _("Background"), TRUE);
+        gwy_log_add(data, GWY_FILE_IMAGE, id, newid);
     }
 
 end:
@@ -162,7 +159,7 @@ run_gui(ModuleArgs *args)
     GwyDialog *dialog;
     GwyParamTable *table;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
 
     gui.dialog = gwy_dialog_new(_("Level Grains"));
@@ -183,38 +180,38 @@ run_gui(ModuleArgs *args)
 static void
 execute(ModuleArgs *args)
 {
-    GwyDataField *invmask, *background = args->bg, *mask = args->mask;
+    GwyField *invmask, *background = args->bg, *mask = args->mask;
     GwyGrainQuantity base = gwy_params_get_enum(args->params, PARAM_BASE);
     gdouble *heights, *bgdata;
     gint *grains;
     gint i, xres, yres, ngrains;
 
-    xres = gwy_data_field_get_xres(mask);
-    yres = gwy_data_field_get_yres(mask);
+    xres = gwy_field_get_xres(mask);
+    yres = gwy_field_get_yres(mask);
     grains = g_new0(gint, xres*yres);
-    ngrains = gwy_data_field_number_grains(mask, grains);
+    ngrains = gwy_field_number_grains(mask, grains);
     if (!ngrains) {
         g_free(grains);
         return;
     }
 
     heights = g_new(gdouble, ngrains+1);
-    gwy_data_field_grains_get_values(args->field, heights, ngrains, grains, base);
+    gwy_field_grains_get_values(args->field, heights, ngrains, grains, base);
     heights[0] = 0.0;
 
-    bgdata = gwy_data_field_get_data(background);
+    bgdata = gwy_field_get_data(background);
     for (i = 0; i < xres*yres; i++)
         bgdata[i] = -heights[grains[i]];
     g_free(grains);
     g_free(heights);
 
-    invmask = gwy_data_field_duplicate(mask);
-    gwy_data_field_grains_invert(invmask);
-    gwy_data_field_laplace_solve(background, invmask, -1, 0.8);
+    invmask = gwy_field_copy(mask);
+    gwy_field_grains_invert(invmask);
+    gwy_field_laplace_solve(background, invmask, -1, 0.8);
     g_object_unref(invmask);
 
-    gwy_data_field_invert(background, FALSE, FALSE, TRUE);
-    gwy_data_field_subtract_fields(args->result, args->field, background);
+    gwy_field_invert_value(background);
+    gwy_field_subtract_fields(args->result, args->field, background);
 }
 
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
