@@ -1,5 +1,5 @@
 /*
- *  $Id: linecorrect.c 26059 2023-12-18 17:46:59Z yeti-dn $
+ *  $Id: linecorrect.c 28910 2025-11-24 18:11:09Z yeti-dn $
  *  Copyright (C) 2003-2023 David Necas (Yeti), Petr Klapetek, Luke Somers.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net, lsomers@sas.upenn.edu.
  *
@@ -16,17 +16,9 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/correct.h>
-#include <libprocess/filters.h>
-#include <libprocess/linestats.h>
-#include <libprocess/stats.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <libgwydgets/gwystock.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 
 #define RUN_MODES GWY_RUN_IMMEDIATE
 
@@ -36,11 +28,11 @@ typedef struct {
     guint n;
 } MedianLineData;
 
-static gboolean module_register                    (void);
-static void     line_correct_step                  (GwyContainer *data,
-                                                    GwyRunType run);
-static void     mark_inverted_lines                (GwyContainer *data,
-                                                    GwyRunType run);
+static gboolean module_register    (void);
+static void     line_correct_step  (GwyFile *data,
+                                    GwyRunModeFlags mode);
+static void     mark_inverted_lines(GwyFile *data,
+                                    GwyRunModeFlags mode);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
@@ -58,18 +50,18 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("line_correct_step",
-                              (GwyProcessFunc)&line_correct_step,
+                              &line_correct_step,
                               N_("/_Correct Data/Ste_p Line Correction"),
-                              GWY_STOCK_LINE_LEVEL,
+                              GWY_ICON_LINE_LEVEL,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Correct steps in lines"));
     gwy_process_func_register("line_correct_inverted",
-                              (GwyProcessFunc)&mark_inverted_lines,
+                              &mark_inverted_lines,
                               N_("/_Correct Data/Mark _Inverted Rows"),
                               NULL,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Mark lines with inverted sign"));
 
     return TRUE;
@@ -100,8 +92,8 @@ calculate_segment_correction(const gdouble *drow,
 }
 
 static void
-line_correct_step_iter(GwyDataField *dfield,
-                       GwyDataField *mask)
+line_correct_step_iter(GwyField *dfield,
+                       GwyField *mask)
 {
     const gdouble threshold = 3.0;
     gint xres, yres, i, j, len;
@@ -109,10 +101,10 @@ line_correct_step_iter(GwyDataField *dfield,
     const gdouble *d, *drow;
     gdouble *m, *mrow;
 
-    yres = gwy_data_field_get_yres(dfield);
-    xres = gwy_data_field_get_xres(dfield);
-    d = gwy_data_field_get_data_const(dfield);
-    m = gwy_data_field_get_data(mask);
+    yres = gwy_field_get_yres(dfield);
+    xres = gwy_field_get_xres(dfield);
+    d = gwy_field_get_data_const(dfield);
+    m = gwy_field_get_data(mask);
 
     w = 0.0;
     for (i = 0; i < yres-1; i++) {
@@ -153,42 +145,42 @@ line_correct_step_iter(GwyDataField *dfield,
             calculate_segment_correction(drow + j-len, mrow + j-len, xres, len);
     }
 
-    gwy_data_field_sum_fields(dfield, dfield, mask);
+    gwy_field_sum_fields(dfield, dfield, mask);
 }
 
 static void
-line_correct_step(GwyContainer *data,
-                  GwyRunType run)
+line_correct_step(GwyFile *data,
+                  GwyRunModeFlags mode)
 {
-    GwyDataField *dfield, *mask;
-    GwyDataLine *shifts;
+    GwyField *dfield, *mask;
+    GwyLine *shifts;
     GQuark dquark;
     gdouble avg;
     gint id;
 
-    g_return_if_fail(run & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &dfield,
-                                     GWY_APP_DATA_FIELD_KEY, &dquark,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &dfield,
+                                 GWY_APP_FIELD_KEY, &dquark,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(dfield && dquark);
-    gwy_app_undo_qcheckpointv(data, 1, &dquark);
+    gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &dquark);
 
-    avg = gwy_data_field_get_avg(dfield);
-    shifts = gwy_data_field_find_row_shifts_trimmed_mean(dfield, NULL, GWY_MASK_IGNORE, 0.5, 0);
-    gwy_data_field_subtract_row_shifts(dfield, shifts);
+    avg = gwy_field_get_avg(dfield);
+    shifts = gwy_field_find_row_shifts_trimmed_mean(dfield, NULL, GWY_MASK_IGNORE, 0.5, 0);
+    gwy_field_subtract_row_shifts(dfield, shifts);
     g_object_unref(shifts);
 
-    mask = gwy_data_field_new_alike(dfield, TRUE);
+    mask = gwy_field_new_alike(dfield, TRUE);
     line_correct_step_iter(dfield, mask);
-    gwy_data_field_clear(mask);
+    gwy_field_clear(mask);
     line_correct_step_iter(dfield, mask);
     g_object_unref(mask);
 
-    gwy_data_field_filter_conservative(dfield, 5);
-    gwy_data_field_add(dfield, avg - gwy_data_field_get_avg(dfield));
-    gwy_data_field_data_changed(dfield);
-    gwy_app_channel_log_add_proc(data, id, id);
+    gwy_field_filter_conservative(dfield, 5);
+    gwy_field_add(dfield, avg - gwy_field_get_avg(dfield));
+    gwy_field_data_changed(dfield);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, id);
 }
 
 static gdouble
@@ -207,11 +199,11 @@ row_correlation(const gdouble *row1, gdouble avg1, gdouble rms1,
 }
 
 static void
-mark_inverted_lines(GwyContainer *data,
-                    GwyRunType run)
+mark_inverted_lines(GwyFile *data,
+                    GwyRunModeFlags mode)
 {
-    GwyDataField *dfield, *existing_mask, *mask;
-    GwyDataLine *avgline, *rmsline;
+    GwyField *dfield, *existing_mask, *mask;
+    GwyLine *avgline, *rmsline;
     GQuark mquark;
     gdouble *weight;
     const gdouble *avg, *rms;
@@ -220,29 +212,29 @@ mark_inverted_lines(GwyContainer *data,
     gboolean have_negative = FALSE, inverted;
     const gdouble *d;
 
-    g_return_if_fail(run & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &dfield,
-                                     GWY_APP_MASK_FIELD, &existing_mask,
-                                     GWY_APP_MASK_FIELD_KEY, &mquark,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &dfield,
+                                 GWY_APP_MASK_FIELD, &existing_mask,
+                                 GWY_APP_MASK_FIELD_KEY, &mquark,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(dfield && mquark);
 
-    total_rms = gwy_data_field_get_rms(dfield);
-    xres = gwy_data_field_get_xres(dfield);
-    yres = gwy_data_field_get_yres(dfield);
+    total_rms = gwy_field_get_rms(dfield);
+    xres = gwy_field_get_xres(dfield);
+    yres = gwy_field_get_yres(dfield);
     if (total_rms <= 0.0 || yres < 3 || xres < 3)
         return;
 
-    avgline = gwy_data_line_new(yres, yres, FALSE);
-    gwy_data_field_get_line_stats(dfield, avgline, GWY_LINE_STAT_MEAN, GWY_ORIENTATION_HORIZONTAL);
-    avg = gwy_data_line_get_data(avgline);
+    avgline = gwy_line_new(yres, yres, FALSE);
+    gwy_field_get_line_stats(dfield, avgline, GWY_LINE_STAT_MEAN, GWY_ORIENTATION_HORIZONTAL);
+    avg = gwy_line_get_data(avgline);
 
-    rmsline = gwy_data_line_new(yres, yres, FALSE);
-    gwy_data_field_get_line_stats(dfield, rmsline, GWY_LINE_STAT_RMS, GWY_ORIENTATION_HORIZONTAL);
-    rms = gwy_data_line_get_data(rmsline);
+    rmsline = gwy_line_new(yres, yres, FALSE);
+    gwy_field_get_line_stats(dfield, rmsline, GWY_LINE_STAT_RMS, GWY_ORIENTATION_HORIZONTAL);
+    rms = gwy_line_get_data(rmsline);
 
-    d = gwy_data_field_get_data_const(dfield);
+    d = gwy_field_get_data_const(dfield);
     /* Calculate neighbour row correlations. */
     weight = g_new0(gdouble, yres-1);
     for (i = 0; i < yres-1; i++) {
@@ -289,8 +281,8 @@ mark_inverted_lines(GwyContainer *data,
     g_object_unref(avgline);
     g_object_unref(rmsline);
 
-    mask = gwy_data_field_new_alike(dfield, TRUE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(mask), NULL);
+    mask = gwy_field_new_alike(dfield, TRUE);
+    gwy_unit_clear(gwy_field_get_unit_z(mask));
 
     /* Extend the sign of this block downwards.  Note weight[i] represents the sign between rows i and i+1. */
     inverted = FALSE;
@@ -298,7 +290,7 @@ mark_inverted_lines(GwyContainer *data,
         if (weight[i] < 0.0)
             inverted = !inverted;
         if (inverted)
-            gwy_data_field_area_fill(mask, 0, i+1, xres, 1, 1.0);
+            gwy_field_area_fill(mask, NULL, GWY_MASK_IGNORE, 0, i+1, xres, 1, 1.0);
     }
 
     /* Extend the sign of this block upwards.  Note weight[i] represents the sign between rows i and i+1. */
@@ -307,27 +299,27 @@ mark_inverted_lines(GwyContainer *data,
         if (weight[i] < 0.0)
             inverted = !inverted;
         if (inverted)
-            gwy_data_field_area_fill(mask, 0, i, xres, 1, 1.0);
+            gwy_field_area_fill(mask, NULL, GWY_MASK_IGNORE, 0, i, xres, 1, 1.0);
     }
 
     g_free(weight);
 
-    if (!existing_mask && gwy_data_field_get_max(mask) <= 0.0) {
+    if (!existing_mask && gwy_field_get_max(mask) <= 0.0) {
         g_object_unref(mask);
         return;
     }
-    gwy_app_undo_qcheckpointv(data, 1, &mquark);
+    gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &mquark);
 
     if (existing_mask) {
-        gwy_data_field_copy(mask, existing_mask, FALSE);
-        gwy_data_field_data_changed(existing_mask);
+        gwy_field_copy_data(mask, existing_mask);
+        gwy_field_data_changed(existing_mask);
     }
     else {
-        gwy_container_set_object(data, mquark, mask);
+        gwy_container_set_object(GWY_CONTAINER(data), mquark, mask);
     }
     g_object_unref(mask);
 
-    gwy_app_channel_log_add_proc(data, id, id);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, id);
 }
 
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
