@@ -1,5 +1,5 @@
 /*
- *  $Id: linematch.c 26554 2024-08-19 11:16:05Z yeti-dn $
+ *  $Id: linematch.c 29061 2026-01-02 15:01:53Z yeti-dn $
  *  Copyright (C) 2015-2024 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,23 +16,11 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwythreads.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/level.h>
-#include <libprocess/correct.h>
-#include <libprocess/stats.h>
-#include <libprocess/linestats.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
-#include "libgwyddion/gwyomp.h"
+#include <gwy.h>
+#include "libgwyddion/omp.h"
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
@@ -68,11 +56,11 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *mask;
-    GwyDataField *result;
-    GwyDataField *bg;
-    GwyDataLine *shifts;
+    GwyField *field;
+    GwyField *mask;
+    GwyField *result;
+    GwyField *bg;
+    GwyLine *shifts;
 } ModuleArgs;
 
 typedef struct {
@@ -80,50 +68,49 @@ typedef struct {
     GtkWidget *dialog;
     GwyParamTable *table;
     GwyGraphModel *gmodel;
-    GwyContainer *data;
 } ModuleGUI;
 
 static gboolean         module_register          (void);
 static GwyParamDef*     define_module_params     (void);
-static void             linematch                (GwyContainer *data,
-                                                  GwyRunType runtype);
+static void             module_main              (GwyFile *data,
+                                                  GwyRunModeFlags mode);
 static void             execute                  (ModuleArgs *args);
 static GwyDialogOutcome run_gui                  (ModuleArgs *args,
-                                                  GwyContainer *data,
+                                                  GwyFile *data,
                                                   gint id);
 static void             param_changed            (ModuleGUI *gui,
                                                   gint id);
 static void             preview                  (gpointer user_data);
-static void             linematch_do_trimmed_mean(GwyDataField *field,
-                                                  GwyDataField *mask,
-                                                  GwyDataLine *shifts,
+static void             linematch_do_trimmed_mean(GwyField *field,
+                                                  GwyField *mask,
+                                                  GwyLine *shifts,
                                                   GwyMaskingType masking,
                                                   gdouble trim_fraction);
-static void             linematch_do_trimmed_diff(GwyDataField *field,
-                                                  GwyDataField *mask,
-                                                  GwyDataLine *shifts,
+static void             linematch_do_trimmed_diff(GwyField *field,
+                                                  GwyField *mask,
+                                                  GwyLine *shifts,
                                                   GwyMaskingType masking,
                                                   gdouble trim_fraction);
-static void             linematch_do_modus       (GwyDataField *field,
-                                                  GwyDataField *mask,
-                                                  GwyDataLine *shifts,
+static void             linematch_do_modus       (GwyField *field,
+                                                  GwyField *mask,
+                                                  GwyLine *shifts,
                                                   GwyMaskingType masking);
-static void             linematch_do_match       (GwyDataField *field,
-                                                  GwyDataField *mask,
-                                                  GwyDataLine *shifts,
+static void             linematch_do_match       (GwyField *field,
+                                                  GwyField *mask,
+                                                  GwyLine *shifts,
                                                   GwyMaskingType masking);
-static void             linematch_do_facet_tilt  (GwyDataField *field,
-                                                  GwyDataField *mask,
-                                                  GwyDataLine *shifts,
+static void             linematch_do_facet_tilt  (GwyField *field,
+                                                  GwyField *mask,
+                                                  GwyLine *shifts,
                                                   GwyMaskingType masking);
-static void             zero_level_row_shifts    (GwyDataLine *shifts);
+static void             zero_level_row_shifts    (GwyLine *shifts);
 
 static const GwyEnum methods[] = {
-    { N_("linematch|Polynomial"),        LINE_MATCH_POLY,         },
+    { gwy_NC("linematch", "Polynomial"), LINE_MATCH_POLY,         },
     { N_("Median"),                      LINE_MATCH_MEDIAN,       },
     { N_("Median of differences"),       LINE_MATCH_MEDIAN_DIFF,  },
     { N_("Modus"),                       LINE_MATCH_MODUS,        },
-    { N_("linematch|Matching"),          LINE_MATCH_MATCH,        },
+    { gwy_NC("linematch", "Matching"),   LINE_MATCH_MATCH,        },
     { N_("Trimmed mean"),                LINE_MATCH_TRIMMED_MEAN, },
     { N_("Trimmed mean of differences"), LINE_MATCH_TMEAN_DIFF,   },
     { N_("Facet-level tilt"),            LINE_MATCH_FACET_TILT,   },
@@ -145,11 +132,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("align_rows",
-                              (GwyProcessFunc)&linematch,
+                              module_main,
                               N_("/_Correct Data/_Align Rows..."),
-                              GWY_STOCK_LINE_LEVEL,
+                              GWY_ICON_LINE_LEVEL,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Align rows using various methods"));
 
     return TRUE;
@@ -179,9 +166,9 @@ define_module_params(void)
 }
 
 static void
-linematch(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
-    GwyDataField *field, *mask;
+    GwyField *field, *mask;
     GQuark quark;
     GwyParams *params;
     ModuleArgs args;
@@ -189,51 +176,54 @@ linematch(GwyContainer *data, GwyRunType runtype)
     const gchar *methodname;
     gchar *title;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD_KEY, &quark,
-                                     GWY_APP_DATA_FIELD, &field,
-                                     GWY_APP_MASK_FIELD, &mask,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD_KEY, &quark,
+                                 GWY_APP_FIELD, &field,
+                                 GWY_APP_MASK_FIELD, &mask,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(field && quark);
 
     args.field = field;
     args.mask = mask;
-    args.bg = gwy_data_field_new_alike(field, FALSE);
-    args.shifts = gwy_data_line_new(field->yres, field->yreal, FALSE);
-    gwy_data_field_copy_units_to_data_line(field, args.shifts);
+    args.bg = gwy_field_new_alike(field, FALSE);
+    args.shifts = gwy_line_new(field->yres, field->yreal, FALSE);
+    gwy_field_copy_units_to_line(field, args.shifts);
     args.params = params = gwy_params_new_from_settings(define_module_params());
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         GwyDialogOutcome outcome = run_gui(&args, data, id);
         gwy_params_save_to_settings(params);
         if (outcome != GWY_DIALOG_HAVE_RESULT)
             goto end;
-        gwy_app_undo_qcheckpointv(data, 1, &quark);
-        gwy_data_field_copy(args.result, field, FALSE);
+        gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &quark);
+        gwy_field_copy_data(args.result, field);
     }
     else {
-        gwy_app_undo_qcheckpointv(data, 1, &quark);
+        gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &quark);
         args.result = g_object_ref(field);
         execute(&args);
     }
 
-    gwy_data_field_data_changed(field);
-    gwy_app_channel_log_add(data, id, id, "proc::align_rows",
-                            "settings-name", "linematch",
-                            NULL);
+    gwy_field_data_changed(field);
+    gwy_log_add_full(data, GWY_FILE_IMAGE, id, id, "proc::align_rows",
+                     "settings-name", "linematch",
+                     NULL);
 
     methodname = gwy_enum_to_string(gwy_params_get_enum(params, PARAM_METHOD), methods, G_N_ELEMENTS(methods));
-    methodname = gwy_sgettext(methodname);
+    methodname = gwy_C(methodname);
     title = g_strdup_printf("%s (%s)", _("Row background"), methodname);
 
     if (gwy_params_get_boolean(params, PARAM_DO_EXTRACT)) {
-        newid = gwy_app_data_browser_add_data_field(args.bg, data, TRUE);
-        gwy_app_sync_data_items(data, data, id, newid, FALSE, GWY_DATA_ITEM_GRADIENT, 0);
-        gwy_app_set_data_field_title(data, newid, title);
-        gwy_app_channel_log_add(data, id, newid, "proc::align_rows",
-                                "settings-name", "linematch",
-                                NULL);
+        newid = gwy_file_add_image(data, args.bg);
+        gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+        gwy_file_sync_items(data, GWY_FILE_IMAGE, id,
+                            data, GWY_FILE_IMAGE, newid,
+                            GWY_FILE_ITEM_PALETTE, FALSE);
+        gwy_file_set_title(data, GWY_FILE_IMAGE, newid, title, TRUE);
+        gwy_log_add_full(data, GWY_FILE_IMAGE, id, newid, "proc::align_rows",
+                         "settings-name", "linematch",
+                         NULL);
     }
 
     if (gwy_params_get_boolean(params, PARAM_DO_PLOT)) {
@@ -241,7 +231,7 @@ linematch(GwyContainer *data, GwyRunType runtype)
         GwyGraphCurveModel *gcmodel = gwy_graph_curve_model_new();
         GwyAppDataId target_graph_id = gwy_params_get_data_id(params, PARAM_TARGET_GRAPH);
 
-        gwy_graph_curve_model_set_data_from_dataline(gcmodel, args.shifts, 0, 0);
+        gwy_graph_curve_model_set_data_from_line(gcmodel, args.shifts, 0, 0);
         g_object_set(gcmodel,
                      "description", title,
                      "mode", GWY_GRAPH_CURVE_LINE,
@@ -255,7 +245,7 @@ linematch(GwyContainer *data, GwyRunType runtype)
                      "axis-label-bottom", _("Vertical position"),
                      "axis-label-left", _("Corrected offset"),
                      NULL);
-        gwy_graph_model_set_units_from_data_line(gmodel, args.shifts);
+        gwy_graph_model_set_units_from_line(gmodel, args.shifts);
         gwy_app_add_graph_or_curves(gmodel, data, &target_graph_id, 1);
         g_object_unref(gmodel);
     }
@@ -264,13 +254,13 @@ linematch(GwyContainer *data, GwyRunType runtype)
 
 end:
     g_object_unref(params);
-    GWY_OBJECT_UNREF(args.result);
-    GWY_OBJECT_UNREF(args.shifts);
-    GWY_OBJECT_UNREF(args.bg);
+    g_clear_object(&args.result);
+    g_clear_object(&args.shifts);
+    g_clear_object(&args.bg);
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     GwyDialogOutcome outcome;
     GtkWidget *hbox, *dataview;
@@ -278,27 +268,21 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     GwyParamTable *table;
     ModuleGUI gui;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
-    gui.data = gwy_container_new();
     /* Create an empty graph model just for compatibility check. */
     gui.gmodel = gwy_graph_model_new();
-    gwy_graph_model_set_units_from_data_field(gui.gmodel, args->field, 1, 0, 0, 1);
+    gwy_graph_model_set_units_from_field(gui.gmodel, args->field, 1, 0, 0, 1);
 
-    args->result = gwy_data_field_duplicate(args->field);
-
-    gwy_container_set_object_by_name(gui.data, "/0/data", args->result);
-    gwy_app_sync_data_items(data, gui.data, id, 0, FALSE,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            GWY_DATA_ITEM_GRADIENT,
-                            GWY_DATA_ITEM_RANGE_TYPE,
-                            0);
+    args->result = gwy_field_copy(args->field);
 
     gui.dialog = gwy_dialog_new(_("Align Rows"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
+    dataview = gwy_create_preview(args->result, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_REAL_SQUARE | GWY_FILE_ITEM_COLOR_MAPPING);
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), FALSE);
 
     table = gui.table = gwy_param_table_new(args->params);
@@ -335,7 +319,6 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_dialog_set_preview_func(dialog, GWY_PREVIEW_IMMEDIATE, preview, &gui, NULL);
     outcome = gwy_dialog_run(dialog);
 
-    g_object_unref(gui.data);
     g_object_unref(gui.gmodel);
 
     return outcome;
@@ -344,39 +327,39 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 static void
 execute(ModuleArgs *args)
 {
-    GwyDataField *field = args->field, *mask = args->mask;
-    GwyDataLine *shifts = args->shifts;
-    GwyDataField *myfield, *mymask;
+    GwyField *field = args->field, *mask = args->mask;
+    GwyLine *shifts = args->shifts;
+    GwyField *myfield, *mymask;
     LineMatchMethod method = gwy_params_get_enum(args->params, PARAM_METHOD);
     GwyMaskingType masking = gwy_params_get_masking(args->params, PARAM_MASKING, &mask);
     GwyOrientation direction = gwy_params_get_enum(args->params, PARAM_DIRECTION);
     gdouble trim_fraction = gwy_params_get_double(args->params, PARAM_TRIM_FRACTION);
     gint max_degree = gwy_params_get_int(args->params, PARAM_MAX_DEGREE);
 
-    gwy_data_field_copy(field, args->result, TRUE);
-    gwy_data_field_copy(field, args->bg, TRUE);
+    gwy_field_assign(args->result, field);
+    gwy_field_assign(args->bg, field);
 
     /* Transpose the fields if necessary. */
     mymask = mask;
     myfield = args->result;
     if (direction == GWY_ORIENTATION_VERTICAL) {
-        myfield = gwy_data_field_new_alike(args->result, FALSE);
-        gwy_data_field_flip_xy(args->result, myfield, FALSE);
+        myfield = gwy_field_new_alike(args->result, FALSE);
+        gwy_field_transpose(args->result, myfield, FALSE);
         if (mask) {
-            mymask = gwy_data_field_new_alike(mask, FALSE);
-            gwy_data_field_flip_xy(mask, mymask, FALSE);
+            mymask = gwy_field_new_alike(mask, FALSE);
+            gwy_field_transpose(mask, mymask, FALSE);
         }
     }
 
-    gwy_data_line_resample(shifts, gwy_data_field_get_yres(myfield), GWY_INTERPOLATION_NONE);
-    gwy_data_line_set_real(shifts, gwy_data_field_get_yreal(myfield));
+    gwy_line_resize(shifts, gwy_field_get_yres(myfield));
+    gwy_line_set_real(shifts, gwy_field_get_yreal(myfield));
 
     /* Perform the correction. */
     if (method == LINE_MATCH_POLY) {
         if (max_degree == 0)
             linematch_do_trimmed_mean(myfield, mymask, shifts, masking, 0.0);
         else
-            gwy_data_field_row_level_poly(myfield, mymask, masking, max_degree, shifts);
+            gwy_field_row_level_poly(myfield, mymask, masking, max_degree, shifts);
     }
     else if (method == LINE_MATCH_MEDIAN)
         linematch_do_trimmed_mean(myfield, mymask, shifts, masking, 0.5);
@@ -398,11 +381,11 @@ execute(ModuleArgs *args)
 
     /* Transpose back if necessary. */
     if (direction == GWY_ORIENTATION_VERTICAL) {
-        GWY_OBJECT_UNREF(mymask);
-        gwy_data_field_flip_xy(myfield, args->result, FALSE);
+        g_clear_object(&mymask);
+        gwy_field_transpose(myfield, args->result, FALSE);
         g_object_unref(myfield);
     }
-    gwy_data_field_subtract_fields(args->bg, args->bg, args->result);
+    gwy_field_subtract_fields(args->bg, args->bg, args->result);
 }
 
 static void
@@ -432,56 +415,56 @@ preview(gpointer user_data)
     ModuleArgs *args = gui->args;
 
     execute(args);
-    gwy_data_field_data_changed(args->result);
+    gwy_field_data_changed(args->result);
     gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
 }
 
 static void
-linematch_do_trimmed_mean(GwyDataField *field, GwyDataField *mask, GwyDataLine *shifts,
+linematch_do_trimmed_mean(GwyField *field, GwyField *mask, GwyLine *shifts,
                           GwyMaskingType masking, gdouble trimfrac)
 {
-    GwyDataLine *myshifts;
+    GwyLine *myshifts;
 
-    myshifts = gwy_data_field_find_row_shifts_trimmed_mean(field, mask, masking, trimfrac, 0);
-    gwy_data_field_subtract_row_shifts(field, myshifts);
-    gwy_data_line_assign(shifts, myshifts);
+    myshifts = gwy_field_find_row_shifts_trimmed_mean(field, mask, masking, trimfrac, 0);
+    gwy_field_subtract_row_shifts(field, myshifts);
+    gwy_line_assign(shifts, myshifts);
     g_object_unref(myshifts);
 }
 
 static void
-linematch_do_trimmed_diff(GwyDataField *field, GwyDataField *mask, GwyDataLine *shifts,
+linematch_do_trimmed_diff(GwyField *field, GwyField *mask, GwyLine *shifts,
                           GwyMaskingType masking, gdouble trimfrac)
 {
-    GwyDataLine *myshifts;
+    GwyLine *myshifts;
 
-    myshifts = gwy_data_field_find_row_shifts_trimmed_diff(field, mask, masking, trimfrac, 0);
-    gwy_data_field_subtract_row_shifts(field, myshifts);
-    gwy_data_line_assign(shifts, myshifts);
+    myshifts = gwy_field_find_row_shifts_trimmed_diff(field, mask, masking, trimfrac, 0);
+    gwy_field_subtract_row_shifts(field, myshifts);
+    gwy_line_assign(shifts, myshifts);
     g_object_unref(myshifts);
 }
 
 static void
-linematch_do_modus(GwyDataField *field, GwyDataField *mask, GwyDataLine *modi,
+linematch_do_modus(GwyField *field, GwyField *mask, GwyLine *modi,
                    GwyMaskingType masking)
 {
     gint xres, yres;
     const gdouble *d, *m;
     gdouble total_median;
 
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
-    total_median = gwy_data_field_area_get_median_mask(field, mask, masking, 0, 0, xres, yres);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
+    total_median = gwy_field_area_get_median(field, mask, masking, 0, 0, xres, yres);
 
-    d = gwy_data_field_get_data(field);
-    m = mask ? gwy_data_field_get_data_const(mask) : NULL;
+    d = gwy_field_get_data(field);
+    m = mask ? gwy_field_get_data_const(mask) : NULL;
 
 #ifdef _OPENMP
 #pragma omp parallel if(gwy_threads_are_enabled()) default(none) \
             shared(d,m,modi,xres,yres,masking,total_median)
 #endif
     {
-        GwyDataLine *line = gwy_data_line_new(xres, 1.0, FALSE);
-        gdouble *buf = gwy_data_line_get_data(line);
+        GwyLine *line = gwy_line_new(xres, 1.0, FALSE);
+        gdouble *buf = gwy_line_get_data(line);
         gint ifrom = gwy_omp_chunk_start(yres), ito = gwy_omp_chunk_end(yres);
         gint i;
 
@@ -502,12 +485,12 @@ linematch_do_modus(GwyDataField *field, GwyDataField *mask, GwyDataLine *modi,
             if (!count)
                 modus = total_median;
             else if (count < 9)
-                modus = gwy_math_median(count, buf);
+                modus = gwy_math_median(buf, count);
             else {
                 gint seglen = GWY_ROUND(sqrt(count)), bestj = 0;
                 gdouble diff, bestdiff = G_MAXDOUBLE;
 
-                gwy_math_sort(count, buf);
+                gwy_math_sort(buf, count);
                 for (j = 0; j + seglen-1 < count; j++) {
                     diff = buf[j + seglen-1] - buf[j];
                     if (diff < bestdiff) {
@@ -522,29 +505,29 @@ linematch_do_modus(GwyDataField *field, GwyDataField *mask, GwyDataLine *modi,
                 modus /= count;
             }
 
-            gwy_data_line_set_val(modi, i, modus);
+            gwy_line_set_val(modi, i, modus);
         }
 
         g_object_unref(line);
     }
 
     zero_level_row_shifts(modi);
-    gwy_data_field_subtract_row_shifts(field, modi);
+    gwy_field_subtract_row_shifts(field, modi);
 }
 
 static void
-linematch_do_match(GwyDataField *field, GwyDataField *mask, GwyDataLine *shifts,
+linematch_do_match(GwyField *field, GwyField *mask, GwyLine *shifts,
                    GwyMaskingType masking)
 {
     gint xres, yres, k;
     const gdouble *d, *m;
     gdouble *s;
 
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
-    d = gwy_data_field_get_data(field);
-    m = mask ? gwy_data_field_get_data_const(mask) : NULL;
-    s = gwy_data_line_get_data(shifts);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
+    d = gwy_field_get_data(field);
+    m = mask ? gwy_field_get_data_const(mask) : NULL;
+    s = gwy_line_get_data(shifts);
 
 #ifdef _OPENMP
 #pragma omp parallel if(gwy_threads_are_enabled()) default(none) \
@@ -619,7 +602,7 @@ linematch_do_match(GwyDataField *field, GwyDataField *mask, GwyDataLine *shifts,
         s[k] += s[k-1];
 
     zero_level_row_shifts(shifts);
-    gwy_data_field_subtract_row_shifts(field, shifts);
+    gwy_field_subtract_row_shifts(field, shifts);
 }
 
 static gdouble
@@ -717,7 +700,7 @@ untilt_row(gdouble *drow, guint res, gdouble bx)
 }
 
 static void
-linematch_do_facet_tilt(GwyDataField *field, GwyDataField *mask, GwyDataLine *shifts,
+linematch_do_facet_tilt(GwyField *field, GwyField *mask, GwyLine *shifts,
                         GwyMaskingType masking)
 {
     guint xres, yres, i, j, mincount;
@@ -725,13 +708,13 @@ linematch_do_facet_tilt(GwyDataField *field, GwyDataField *mask, GwyDataLine *sh
     gdouble *data, *drow;
     gdouble dx, tilt;
 
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
-    dx = gwy_data_field_get_dx(field);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
+    dx = gwy_field_get_dx(field);
     mincount = GWY_ROUND(log(xres) + 1);
 
-    data = gwy_data_field_get_data(field);
-    mdata = mask ? gwy_data_field_get_data_const(mask) : NULL;
+    data = gwy_field_get_data(field);
+    mdata = mask ? gwy_field_get_data_const(mask) : NULL;
     for (i = 0; i < yres; i++) {
         drow = data + i*xres;
         mrow = mdata ? mdata + i*xres : NULL;
@@ -745,13 +728,13 @@ linematch_do_facet_tilt(GwyDataField *field, GwyDataField *mask, GwyDataLine *sh
 
     /* FIXME: Should we put the tilts there to confuse the user?  We need to
      * make sure all functions set the units correctly in such case. */
-    gwy_data_line_clear(shifts);
+    gwy_line_clear(shifts);
 }
 
 static void
-zero_level_row_shifts(GwyDataLine *shifts)
+zero_level_row_shifts(GwyLine *shifts)
 {
-    gwy_data_line_add(shifts, -gwy_data_line_get_avg(shifts));
+    gwy_line_add(shifts, -gwy_line_avg(shifts));
 }
 
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
