@@ -1,5 +1,5 @@
 /*
- *  $Id: local_contrast.c 23666 2021-05-10 21:56:10Z yeti-dn $
+ *  $Id: local_contrast.c 28910 2025-11-24 18:11:09Z yeti-dn $
  *  Copyright (C) 2003-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,14 +16,9 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwythreads.h>
-#include <libprocess/stats.h>
-#include <libprocess/filters.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
 
@@ -35,16 +30,16 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *result;
+    GwyField *field;
+    GwyField *result;
 } ModuleArgs;
 
-static gboolean         module_register        (void);
-static GwyParamDef*     define_module_params   (void);
-static void             maximize_local_contrast(GwyContainer *data,
-                                                GwyRunType runtype);
-static void             execute                (ModuleArgs *args);
-static GwyDialogOutcome run_gui                (ModuleArgs *args);
+static gboolean         module_register     (void);
+static GwyParamDef*     define_module_params(void);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
+static void             execute             (ModuleArgs *args);
+static GwyDialogOutcome run_gui             (ModuleArgs *args);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
@@ -62,11 +57,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("local_contrast",
-                              (GwyProcessFunc)&maximize_local_contrast,
+                              module_main,
                               N_("/_Presentation/Local _Contrast..."),
                               NULL,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Presentation with maximized local contrast"));
 
     return TRUE;
@@ -89,40 +84,40 @@ define_module_params(void)
 }
 
 static void
-maximize_local_contrast(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
     GQuark squark;
     gint id;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     GWY_APP_SHOW_FIELD_KEY, &squark,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_FIELD_ID, &id,
+                                 GWY_APP_SHOW_FIELD_KEY, &squark,
+                                 0);
     g_return_if_fail(args.field && squark);
 
     args.result = NULL;  /* Create it here once we have a GUI. */
     args.params = gwy_params_new_from_settings(define_module_params());
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         GwyDialogOutcome outcome = run_gui(&args);
         gwy_params_save_to_settings(args.params);
         if (outcome != GWY_DIALOG_PROCEED)
             goto end;
     }
 
-    args.result = gwy_data_field_new_alike(args.field, FALSE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(args.result), NULL);
+    args.result = gwy_field_new_alike(args.field, FALSE);
+    gwy_unit_clear(gwy_field_get_unit_z(args.result));
     execute(&args);
 
-    gwy_data_field_normalize(args.result);
-    gwy_app_undo_qcheckpointv(data, 1, &squark);
-    gwy_container_set_object(data, squark, args.result);
-    gwy_app_channel_log_add_proc(data, id, id);
+    gwy_field_normalize(args.result);
+    gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &squark);
+    gwy_container_set_object(GWY_CONTAINER(data), squark, args.result);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, id);
 
 end:
-    GWY_OBJECT_UNREF(args.result);
+    g_clear_object(&args.result);
     g_object_unref(args.params);
 }
 
@@ -151,7 +146,7 @@ run_gui(ModuleArgs *args)
 static void
 execute(ModuleArgs *args)
 {
-    GwyDataField *field = args->field, *result = args->result, *minfield, *maxfield;
+    GwyField *field = args->field, *result = args->result, *minfield, *maxfield;
     gint size = gwy_params_get_int(args->params, PARAM_SIZE);
     gint depth = gwy_params_get_int(args->params, PARAM_DEPTH);
     gdouble weight = gwy_params_get_double(args->params, PARAM_WEIGHT);
@@ -160,25 +155,25 @@ execute(ModuleArgs *args)
     gdouble sum, gmin, gmax;
     gint xres, yres, i, j;
 
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
-    gmin = gwy_data_field_get_min(field);
-    gmax = gwy_data_field_get_max(field);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
+    gmin = gwy_field_get_min(field);
+    gmax = gwy_field_get_max(field);
     if (gmax == gmin) {
-        gwy_data_field_clear(result);
+        gwy_field_clear(result);
         return;
     }
 
-    minfield = gwy_data_field_duplicate(field);
-    gwy_data_field_filter_minimum(minfield, size);
+    minfield = gwy_field_copy(field);
+    gwy_field_filter_minimum(minfield, size);
 
-    maxfield = gwy_data_field_duplicate(field);
-    gwy_data_field_filter_maximum(maxfield, size);
+    maxfield = gwy_field_copy(field);
+    gwy_field_filter_maximum(maxfield, size);
 
-    dat = gwy_data_field_get_data_const(field);
-    min = gwy_data_field_get_data_const(minfield);
-    max = gwy_data_field_get_data_const(maxfield);
-    show = gwy_data_field_get_data(result);
+    dat = gwy_field_get_data_const(field);
+    min = gwy_field_get_data_const(minfield);
+    max = gwy_field_get_data_const(maxfield);
+    show = gwy_field_get_data(result);
 
     weights = g_new(gdouble, depth);
     sum = 0.0;
