@@ -1,5 +1,5 @@
 /*
- *  $Id: logistic.c 23666 2021-05-10 21:56:10Z yeti-dn $
+ *  $Id: logistic.c 28910 2025-11-24 18:11:09Z yeti-dn $
  *  Copyright (C) 2016-2021 David Necas (Yeti), Petr Klapetek, Daniil Bratashov.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net, dn2010@gwyddion.net
  *
@@ -16,18 +16,12 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
 #include <glib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/stats.h>
-#include <libprocess/filters.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
-#include "libgwyddion/gwyomp.h"
+#include <gwy.h>
+#include "libgwyddion/omp.h"
 #include "preview.h"
 
 #define RUN_MODES GWY_RUN_INTERACTIVE
@@ -57,10 +51,10 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataLine *thetas;
-    GwyDataField *field;
-    GwyDataField *mask;
-    GwyDataField *result;
+    GwyLine *thetas;
+    GwyField *field;
+    GwyField *mask;
+    GwyField *result;
 } ModuleArgs;
 
 typedef struct {
@@ -73,28 +67,28 @@ typedef struct {
 
 static gboolean         module_register        (void);
 static GwyParamDef*     define_module_params   (void);
-static void             logistic               (GwyContainer *data,
-                                                GwyRunType runtype);
+static void             module_main            (GwyFile *data,
+                                                GwyRunModeFlags mode);
 static GwyDialogOutcome run_gui                (ModuleArgs *args);
 static void             param_changed          (ModuleGUI *gui,
                                                 gint id);
-static GwyBrick*        create_feature_vector  (GwyDataField *field,
+static GwyBrick*        create_feature_vector  (GwyField *field,
                                                 ModuleArgs *args);
 static gdouble          cost_function          (GwyBrick *brick,
-                                                GwyDataField *mask,
+                                                GwyField *mask,
                                                 gdouble *thetas,
                                                 gdouble *grad,
                                                 gdouble lambda);
-static void             train_logistic         (GwyContainer *container,
+static void             train_logistic         (GwyFile *container,
                                                 gint id,
                                                 GwyBrick *features,
-                                                GwyDataField *mask,
+                                                GwyField *mask,
                                                 gdouble *thetas,
                                                 gdouble lambda);
 static void             predict_mask           (GwyBrick *brick,
                                                 gdouble *thetas,
-                                                GwyDataField *mask);
-static void             logistic_filter_hessian(GwyDataField *field,
+                                                GwyField *mask);
+static void             logistic_filter_hessian(GwyField *field,
                                                 LogisticHessianFilter filter_type);
 static gint             logistic_nfeatures     (ModuleArgs *args);
 static void             load_thetas            (ModuleArgs *args);
@@ -116,11 +110,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("logistic_regression",
-                              (GwyProcessFunc)&logistic,
+                              module_main,
                               N_("/_Grains/Logistic _Regression..."),
-                              GWY_STOCK_GRAINS,
+                              GWY_ICON_GRAINS,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Mark grains with logistic regression"));
 
     return TRUE;
@@ -152,26 +146,26 @@ define_module_params(void)
 }
 
 static void
-logistic(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
-    GwyDataField *field, *mask;
+    GwyField *field, *mask;
     GwyDialogOutcome outcome;
     ModuleArgs args;
     GwyBrick *features;
     gint id, nfeatures;
     GQuark quark;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &field,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     GWY_APP_MASK_FIELD, &mask,
-                                     GWY_APP_MASK_FIELD_KEY, &quark,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &field,
+                                 GWY_APP_FIELD_ID, &id,
+                                 GWY_APP_MASK_FIELD, &mask,
+                                 GWY_APP_MASK_FIELD_KEY, &quark,
+                                 0);
 
     args.field = field;
     args.mask = mask;
-    args.result = gwy_data_field_new_alike(field, TRUE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(args.result), NULL);
+    args.result = gwy_field_new_alike(field, TRUE);
+    gwy_unit_clear(gwy_field_get_unit_z(args.result));
     args.params = gwy_params_new_from_settings(define_module_params());
     load_thetas(&args);
 
@@ -188,16 +182,16 @@ logistic(GwyContainer *data, GwyRunType runtype)
         if (mask) {
             gdouble lambda = gwy_params_get_double(args.params, PARAM_LAMBDA);
 
-            gwy_data_line_resize(args.thetas, nfeatures, GWY_INTERPOLATION_NONE);
-            train_logistic(data, id, features, mask, gwy_data_line_get_data(args.thetas), lambda);
+            gwy_line_resize(args.thetas, nfeatures);
+            train_logistic(data, id, features, mask, gwy_line_get_data(args.thetas), lambda);
             save_thetas(&args);
         }
     }
     else {
-        gwy_app_undo_qcheckpointv(data, 1, &quark);
-        predict_mask(features, gwy_data_line_get_data(args.thetas), args.result);
-        gwy_container_set_object(data, quark, args.result);
-        gwy_app_channel_log_add_proc(data, id, id);
+        gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &quark);
+        predict_mask(features, gwy_line_get_data(args.thetas), args.result);
+        gwy_file_set_image_mask(data, id, args.result);
+        gwy_log_add(data, GWY_FILE_IMAGE, id, id);
     }
     g_object_unref(features);
 
@@ -215,7 +209,7 @@ run_gui(ModuleArgs *args)
     GwyParamTable *table;
     ModuleGUI gui;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
     gui.orig_params = gwy_params_duplicate(args->params);
 
@@ -282,56 +276,56 @@ param_changed(ModuleGUI *gui, gint id)
 }
 
 static void
-assign_feature(GwyDataField *feature, GwyBrick *brick, gint *z)
+assign_feature(GwyField *feature, GwyBrick *brick, gint *z)
 {
-    gwy_data_field_normalize(feature);
-    gwy_data_field_add(feature, -gwy_data_field_get_avg(feature));
+    gwy_field_normalize(feature);
+    gwy_field_add(feature, -gwy_field_get_avg(feature));
     gwy_brick_set_xy_plane(brick, feature, *z);
     (*z)++;
 }
 
 static void
-assign_all_features(GwyDataField *feature, GwyDataField *feature0, GwyBrick *brick, gint *z,
+assign_all_features(GwyField *feature, GwyField *feature0, GwyBrick *brick, gint *z,
                     gboolean use_laplacian, gboolean use_sobel, gboolean use_hessian)
 {
     assign_feature(feature0, brick, z);
 
     if (use_laplacian) {
-        gwy_data_field_copy(feature0, feature, FALSE);
-        gwy_data_field_filter_laplacian(feature);
+        gwy_field_copy_data(feature0, feature);
+        gwy_field_filter_laplacian(feature);
         assign_feature(feature, brick, z);
     }
 
     if (use_sobel) {
-        gwy_data_field_copy(feature0, feature, FALSE);
-        gwy_data_field_filter_sobel(feature, GWY_ORIENTATION_HORIZONTAL);
+        gwy_field_copy_data(feature0, feature);
+        gwy_field_filter_sobel(feature, GWY_ORIENTATION_HORIZONTAL);
         assign_feature(feature, brick, z);
 
-        gwy_data_field_copy(feature0, feature, FALSE);
-        gwy_data_field_filter_sobel(feature, GWY_ORIENTATION_VERTICAL);
+        gwy_field_copy_data(feature0, feature);
+        gwy_field_filter_sobel(feature, GWY_ORIENTATION_VERTICAL);
         assign_feature(feature, brick, z);
     }
 
     if (use_hessian) {
-        gwy_data_field_copy(feature0, feature, FALSE);
+        gwy_field_copy_data(feature0, feature);
         logistic_filter_hessian(feature, LOGISTIC_HESSIAN_DX2);
         assign_feature(feature, brick, z);
 
-        gwy_data_field_copy(feature0, feature, FALSE);
+        gwy_field_copy_data(feature0, feature);
         logistic_filter_hessian(feature, LOGISTIC_HESSIAN_DY2);
         assign_feature(feature, brick, z);
 
-        gwy_data_field_copy(feature0, feature, FALSE);
+        gwy_field_copy_data(feature0, feature);
         logistic_filter_hessian(feature, LOGISTIC_HESSIAN_DXDY);
         assign_feature(feature, brick, z);
     }
 }
 
 static GwyBrick*
-create_feature_vector(GwyDataField *field, ModuleArgs *args)
+create_feature_vector(GwyField *field, ModuleArgs *args)
 {
     GwyBrick *features, *transposed;
-    GwyDataField *feature0, *featureg, *feature;
+    GwyField *feature0, *featureg, *feature;
     gdouble xreal, yreal, size;
     gint xres, yres, z, zres, i;
     gint ngauss = gwy_params_get_int(args->params, PARAM_NGAUSSIANS);
@@ -340,13 +334,13 @@ create_feature_vector(GwyDataField *field, ModuleArgs *args)
     gboolean use_sobel = gwy_params_get_boolean(args->params, PARAM_USE_SOBEL);
     gboolean use_hessian = gwy_params_get_boolean(args->params, PARAM_USE_HESSIAN);
 
-    feature0 = gwy_data_field_duplicate(field);
-    feature = gwy_data_field_new_alike(feature0, FALSE);
-    featureg = gwy_data_field_new_alike(feature0, FALSE);
-    xres = gwy_data_field_get_xres(feature0);
-    yres = gwy_data_field_get_yres(feature0);
-    xreal = gwy_data_field_get_xreal(feature0);
-    yreal = gwy_data_field_get_yreal(feature0);
+    feature0 = gwy_field_copy(field);
+    feature = gwy_field_new_alike(feature0, FALSE);
+    featureg = gwy_field_new_alike(feature0, FALSE);
+    xres = gwy_field_get_xres(feature0);
+    yres = gwy_field_get_yres(feature0);
+    xreal = gwy_field_get_xreal(feature0);
+    yreal = gwy_field_get_yreal(feature0);
     if (!use_gaussians)
         ngauss = 0;
     zres = logistic_nfeatures(args);
@@ -355,8 +349,8 @@ create_feature_vector(GwyDataField *field, ModuleArgs *args)
     z = 0;
     assign_all_features(feature, feature0, features, &z, use_laplacian, use_sobel, use_hessian);
     for (i = 0, size = 2.0; i < ngauss; i++, size *= 2.0) {
-        gwy_data_field_copy(feature0, featureg, FALSE);
-        gwy_data_field_filter_gaussian(featureg, size * FWHM2SIGMA);
+        gwy_field_copy_data(feature0, featureg);
+        gwy_field_filter_gaussian(featureg, size * FWHM2SIGMA);
         assign_all_features(feature, featureg, features, &z, use_laplacian, use_sobel, use_hessian);
     }
     g_object_unref(featureg);
@@ -379,8 +373,8 @@ sigmoid(gdouble z)
 }
 
 static void
-train_logistic(GwyContainer *container, gint id,
-               GwyBrick *features, GwyDataField *mask, gdouble *thetas, gdouble lambda)
+train_logistic(GwyFile *container, gint id,
+               GwyBrick *features, GwyField *mask, gdouble *thetas, gdouble lambda)
 {
     gdouble *grad, *oldgrad;
     gdouble epsilon, alpha, sum;
@@ -395,7 +389,7 @@ train_logistic(GwyContainer *container, gint id,
     alpha = 10.0;
     iter = 0;
     maxiter = 2000;
-    gwy_app_wait_start(gwy_app_find_window_for_channel(container, id), _("Training..."));
+    gwy_app_wait_start(gwy_data_browser_get_window_for_data(container, GWY_FILE_IMAGE, id), _("Training..."));
     while (!converged && !cancelled) {
         if (!gwy_app_wait_set_fraction((gdouble)iter/maxiter)) {
             cancelled = TRUE;
@@ -434,7 +428,7 @@ train_logistic(GwyContainer *container, gint id,
 }
 
 static gdouble
-cost_function(GwyBrick *brick, GwyDataField *mask,
+cost_function(GwyBrick *brick, GwyField *mask,
               gdouble *thetas, gdouble *grad, gdouble lambda)
 {
     gint i, j, m, xres, yres, fres;
@@ -445,10 +439,10 @@ cost_function(GwyBrick *brick, GwyDataField *mask,
     xres = gwy_brick_get_yres(brick);
     yres = gwy_brick_get_zres(brick);
     fdata = gwy_brick_get_data_const(brick);
-    g_assert(xres == gwy_data_field_get_xres(mask));
-    g_assert(yres == gwy_data_field_get_yres(mask));
+    g_assert(xres == gwy_field_get_xres(mask));
+    g_assert(yres == gwy_field_get_yres(mask));
     m = xres*yres;
-    mdata = gwy_data_field_get_data_const(mask);
+    mdata = gwy_field_get_data_const(mask);
 
     jsum = 0;
     gwy_clear(grad, fres);
@@ -492,7 +486,7 @@ cost_function(GwyBrick *brick, GwyDataField *mask,
 }
 
 static void
-predict_mask(GwyBrick *brick, gdouble *thetas, GwyDataField *mask)
+predict_mask(GwyBrick *brick, gdouble *thetas, GwyField *mask)
 {
     gint i, j, xres, yres, fres;
     const gdouble *fdata;
@@ -502,9 +496,9 @@ predict_mask(GwyBrick *brick, gdouble *thetas, GwyDataField *mask)
     xres = gwy_brick_get_yres(brick);
     yres = gwy_brick_get_zres(brick);
     fdata = gwy_brick_get_data_const(brick);
-    mdata = gwy_data_field_get_data(mask);
-    g_assert(xres == gwy_data_field_get_xres(mask));
-    g_assert(yres == gwy_data_field_get_yres(mask));
+    mdata = gwy_field_get_data(mask);
+    g_assert(xres == gwy_field_get_xres(mask));
+    g_assert(yres == gwy_field_get_yres(mask));
 
 #ifdef _OPENMP
 #pragma omp parallel for if (gwy_threads_are_enabled()) default(none) \
@@ -525,7 +519,7 @@ predict_mask(GwyBrick *brick, gdouble *thetas, GwyDataField *mask)
 }
 
 static void
-logistic_filter_hessian(GwyDataField *field, LogisticHessianFilter filter_type)
+logistic_filter_hessian(GwyField *field, LogisticHessianFilter filter_type)
 {
     static const gdouble dx2_kernel[] = {
         0.125, -0.25, 0.125,
@@ -542,8 +536,8 @@ logistic_filter_hessian(GwyDataField *field, LogisticHessianFilter filter_type)
         0,    0, 0,
         -0.5, 0, 0.5,
     };
-    GwyDataField *kernel = gwy_data_field_new(3, 3, 3, 3, FALSE);
-    gdouble *kdata = gwy_data_field_get_data(kernel);
+    GwyField *kernel = gwy_field_new(3, 3, 3, 3, FALSE);
+    gdouble *kdata = gwy_field_get_data(kernel);
 
     if (filter_type == LOGISTIC_HESSIAN_DX2)
         gwy_assign(kdata, dx2_kernel, 3*3);
@@ -554,7 +548,7 @@ logistic_filter_hessian(GwyDataField *field, LogisticHessianFilter filter_type)
     else {
         g_return_if_reached();
     }
-    gwy_data_field_convolve(field, kernel);
+    gwy_field_convolve(field, kernel);
     g_object_unref(kernel);
 }
 
@@ -591,11 +585,11 @@ load_thetas(ModuleArgs *args)
     gdouble *thetas;
 
     nfeatures = logistic_nfeatures(args);
-    args->thetas = gwy_data_line_new(nfeatures, nfeatures, TRUE);
+    args->thetas = gwy_line_new(nfeatures, nfeatures, TRUE);
     if (!gwy_module_data_load("logistic", "thetas", &buffer, &size, NULL))
         return;
 
-    thetas = gwy_data_line_get_data(args->thetas);
+    thetas = gwy_line_get_data(args->thetas);
     p = buffer;
     i = 0;
     while ((line = gwy_str_next_line(&p)) && i < nfeatures)
@@ -610,7 +604,7 @@ save_thetas(ModuleArgs *args)
     gint i, nfeatures;
     FILE *fh;
 
-    thetas = gwy_data_line_get_data(args->thetas);
+    thetas = gwy_line_get_data(args->thetas);
     if (!(fh = gwy_module_data_fopen("logistic", "thetas", "w", NULL)))
         return;
 
