@@ -1,6 +1,6 @@
 /*
- *  $Id: mark_disconn.c 25745 2023-10-01 18:46:03Z yeti-dn $
- *  Copyright (C) 2015-2021 David Necas (Yeti).
+ *  $Id: mark_disconn.c 29073 2026-01-02 17:26:12Z yeti-dn $
+ *  Copyright (C) 2015-2026 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -16,30 +16,13 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/elliptic.h>
-#include <libprocess/filters.h>
-#include <libprocess/grains.h>
-#include <libprocess/stats.h>
-#include <libprocess/linestats.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
 
-/* The same as in scars. */
-typedef enum {
-    FEATURES_POSITIVE = 1 << 0,
-    FEATURES_NEGATIVE = 1 << 2,
-    FEATURES_BOTH     = (FEATURES_POSITIVE | FEATURES_NEGATIVE),
-} GwyFeaturesType;
-
 enum {
     PARAM_TYPE,
     PARAM_RADIUS,
@@ -51,26 +34,25 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *mask;
-    GwyDataField *result;
+    GwyField *field;
+    GwyField *mask;
+    GwyField *result;
 } ModuleArgs;
 
 typedef struct {
     ModuleArgs *args;
     GtkWidget *dialog;
     GwyParamTable *table;
-    GwyContainer *data;
 } ModuleGUI;
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             mark_disconn        (GwyContainer *data,
-                                             GwyRunType run);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static gboolean         execute             (ModuleArgs *args,
                                              GtkWindow *wait_window);
 static GwyDialogOutcome run_gui             (ModuleArgs *args,
-                                             GwyContainer *data,
+                                             GwyFile *data,
                                              gint id);
 static void             param_changed       (ModuleGUI *gui,
                                              gint id);
@@ -81,7 +63,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Creates mask of values disconnected to the rest."),
     "Yeti <yeti@gwyddion.net>",
-    "2.0",
+    "2.1",
     "David Nečas (Yeti)",
     "2015",
 };
@@ -92,11 +74,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("mark_disconn",
-                              (GwyProcessFunc)&mark_disconn,
+                              module_main,
                               N_("/_Correct Data/Mask of _Disconnected..."),
-                              GWY_STOCK_DISCONNECTED,
+                              GWY_ICON_DISCONNECTED,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Mark data disconnected from other values"));
 
     return TRUE;
@@ -106,9 +88,9 @@ static GwyParamDef*
 define_module_params(void)
 {
     static const GwyEnum feature_types[] = {
-        { N_("Positive"), FEATURES_POSITIVE, },
-        { N_("Negative"), FEATURES_NEGATIVE, },
-        { N_("Both"),     FEATURES_BOTH,     },
+        { N_("Positive"), GWY_SIGN_POSITIVE, },
+        { N_("Negative"), GWY_SIGN_NEGATIVE, },
+        { N_("Both"),     GWY_SIGN_BOTH,     },
     };
     static GwyParamDef *paramdef = NULL;
 
@@ -118,7 +100,7 @@ define_module_params(void)
     paramdef = gwy_param_def_new();
     gwy_param_def_set_function_name(paramdef, gwy_process_func_current());
     gwy_param_def_add_gwyenum(paramdef, PARAM_TYPE, "type", _("Defect type"),
-                              feature_types, G_N_ELEMENTS(feature_types), FEATURES_BOTH);
+                              feature_types, G_N_ELEMENTS(feature_types), GWY_SIGN_BOTH);
     gwy_param_def_add_double(paramdef, PARAM_THRESHOLD, "threshold", _("_Threshold"), 0.0, 1.0, 0.1);
     gwy_param_def_add_int(paramdef, PARAM_RADIUS, "radius", _("Defect _radius"), 1, 240, 5);
     gwy_param_def_add_enum(paramdef, PARAM_COMBINE_TYPE, "combine_type", NULL, GWY_TYPE_MERGE_TYPE, GWY_MERGE_UNION);
@@ -128,42 +110,42 @@ define_module_params(void)
 }
 
 static void
-mark_disconn(GwyContainer *data, GwyRunType run)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     GQuark mquark;
     gint id;
 
-    g_return_if_fail(run & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_MASK_FIELD_KEY, &mquark,
-                                     GWY_APP_MASK_FIELD, &args.mask,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_MASK_FIELD_KEY, &mquark,
+                                 GWY_APP_MASK_FIELD, &args.mask,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(args.field && mquark);
 
-    args.result = gwy_data_field_new_alike(args.field, TRUE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(args.result), NULL);
+    args.result = gwy_field_new_alike(args.field, TRUE);
+    gwy_unit_clear(gwy_field_get_unit_z(args.result));
     args.params = gwy_params_new_from_settings(define_module_params());
 
-    if (run == GWY_RUN_INTERACTIVE) {
+    if (mode == GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, id);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
             goto end;
     }
     if (outcome != GWY_DIALOG_HAVE_RESULT) {
-        if (!execute(&args, gwy_app_find_window_for_channel(data, id)))
+        if (!execute(&args, gwy_data_browser_get_window_for_data(data, GWY_FILE_IMAGE, id)))
             goto end;
     }
 
-    gwy_app_undo_qcheckpointv(data, 1, &mquark);
-    if (gwy_data_field_get_max(args.result) > 0.0)
-        gwy_container_set_object(data, mquark, args.result);
+    gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &mquark);
+    if (gwy_field_get_max(args.result) > 0.0)
+        gwy_container_set_object(GWY_CONTAINER(data), mquark, args.result);
     else
-        gwy_container_remove(data, mquark);
-    gwy_app_channel_log_add_proc(data, id, id);
+        gwy_container_remove(GWY_CONTAINER(data), mquark);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, id);
 
 end:
     g_object_unref(args.result);
@@ -171,31 +153,23 @@ end:
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
-    GwyDialogOutcome outcome;
     GtkWidget *hbox, *dataview;
     GwyDialog *dialog;
     GwyParamTable *table;
     ModuleGUI gui;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
-    gui.data = gwy_container_new();
-
-    gwy_container_set_object_by_name(gui.data, "/0/data", args->field);
-    gwy_container_set_object_by_name(gui.data, "/0/mask", args->result);
-    gwy_app_sync_data_items(data, gui.data, id, 0, FALSE,
-                            GWY_DATA_ITEM_PALETTE,
-                            GWY_DATA_ITEM_RANGE,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
 
     gui.dialog = gwy_dialog_new(_("Mark Disconnected"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_UPDATE, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, TRUE);
+    dataview = gwy_create_preview(args->field, args->result, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_RANGE | GWY_FILE_ITEM_REAL_SQUARE);
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), FALSE);
 
     table = gui.table = gwy_param_table_new(args->params);
@@ -210,7 +184,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_param_table_slider_set_digits(table, PARAM_THRESHOLD, 4);
 
     gwy_param_table_append_header(table, -1, _("Options"));
-    gwy_param_table_append_mask_color(table, PARAM_MASK_COLOR, gui.data, 0, data, id);
+    gwy_param_table_append_image_mask_color(table, PARAM_MASK_COLOR, GWY_DATA_VIEW(dataview), data, id);
     if (args->mask) {
         gwy_param_table_append_radio_buttons(table, PARAM_COMBINE_TYPE, NULL);
         gwy_param_table_add_enabler(table, PARAM_COMBINE, PARAM_COMBINE_TYPE);
@@ -222,11 +196,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     g_signal_connect_swapped(table, "param-changed", G_CALLBACK(param_changed), &gui);
     gwy_dialog_set_preview_func(dialog, GWY_PREVIEW_UPON_REQUEST, preview, &gui, NULL);
 
-    outcome = gwy_dialog_run(dialog);
-
-    g_object_unref(gui.data);
-
-    return outcome;
+    return gwy_dialog_run(dialog);
 }
 
 static void
@@ -243,33 +213,33 @@ preview(gpointer user_data)
     ModuleArgs *args = gui->args;
 
     if (execute(args, GTK_WINDOW(gui->dialog))) {
-        gwy_data_field_data_changed(args->result);
+        gwy_field_data_changed(args->result);
         gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
     }
     else
-        gwy_data_field_clear(args->result);
+        gwy_field_clear(args->result);
 }
 
-/* Remove from mask pixels with values that do not belong to the largest
- * contiguous block of values in the height distribution. */
+/* Remove from mask pixels with values that do not belong to the largest contiguous block of values in the height
+ * distribution. */
 static guint
-unmark_disconnected_values(GwyDataField *dfield, GwyDataField *inclmask,
+unmark_disconnected_values(GwyField *dfield, GwyField *inclmask,
                            guint n, gdouble threshold)
 {
-    guint xres = gwy_data_field_get_xres(dfield);
-    guint yres = gwy_data_field_get_yres(dfield);
+    guint xres = gwy_field_get_xres(dfield);
+    guint yres = gwy_field_get_yres(dfield);
     guint lineres = (guint)floor(2.5*cbrt(xres*yres - n) + 0.5);
-    GwyDataLine *dline = gwy_data_line_new(lineres, lineres, FALSE);
+    GwyLine *dline = gwy_line_new(lineres, lineres, FALSE);
     const gdouble *d;
     gdouble *m;
     guint nn, i, blockstart = 0, bestblockstart = 0, bestblocklen = 0;
     gdouble blocksum = 0.0, bestblocksum = 0.0;
     gdouble real, off, min, max, rho_zero;
 
-    gwy_data_field_area_dh(dfield, inclmask, dline, 0, 0, xres, yres, lineres);
-    rho_zero = gwy_data_line_get_max(dline)/sqrt(xres*yres - n)*threshold;
-    d = gwy_data_line_get_data_const(dline);
-    lineres = gwy_data_line_get_res(dline);
+    gwy_field_area_dh(dfield, inclmask, GWY_MASK_INCLUDE, dline, 0, 0, xres, yres, lineres);
+    rho_zero = gwy_line_max(dline)/sqrt(xres*yres - n)*threshold;
+    d = gwy_line_get_data_const(dline);
+    lineres = gwy_line_get_res(dline);
 
     for (i = 0; i <= lineres; i++) {
         if (i == lineres || (i && d[i] + d[i-1] < rho_zero)) {
@@ -290,12 +260,12 @@ unmark_disconnected_values(GwyDataField *dfield, GwyDataField *inclmask,
         return 0;
     }
 
-    real = gwy_data_line_get_real(dline);
-    off = gwy_data_line_get_offset(dline);
+    real = gwy_line_get_real(dline);
+    off = gwy_line_get_offset(dline);
     min = off + real/lineres*bestblockstart;
     max = off + real/lineres*(bestblockstart + bestblocklen + 1);
-    m = gwy_data_field_get_data(inclmask);
-    d = gwy_data_field_get_data_const(dfield);
+    m = gwy_field_get_data(inclmask);
+    d = gwy_field_get_data_const(dfield);
     nn = 0;
     for (i = 0; i < xres*yres; i++) {
         if (m[i] > 0.0 && (d[i] < min || d[i] > max)) {
@@ -316,58 +286,61 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
     GwyMergeType combine_type = gwy_params_get_enum(params, PARAM_COMBINE_TYPE);
     gint size = 2*gwy_params_get_int(params, PARAM_RADIUS) + 1;
     gdouble threshold = gwy_params_get_double(params, PARAM_THRESHOLD);
-    GwyFeaturesType type = gwy_params_get_enum(params, PARAM_TYPE);
-    GwyDataField *field = args->field, *mask = args->mask, *result = args->result;
-    GwyDataField *difffield = NULL, *kernel = NULL;
-    gint xres = gwy_data_field_get_xres(field);
-    gint yres = gwy_data_field_get_yres(field);
+    GwySignFlags type = gwy_params_get_enum(params, PARAM_TYPE);
+    GwyField *field = args->field, *mask = args->mask, *result = args->result;
+    GwyField *difffield = NULL, *buffer = NULL, *kernel = NULL;
+    gint xres = gwy_field_get_xres(field);
+    gint yres = gwy_field_get_yres(field);
     guint n, nn;
     gboolean ok = FALSE;
 
     gwy_app_wait_start(wait_window, _("Initializing..."));
     /* Remove the positive, negative (or both) defects using a filter.  This produces a defect-free field. */
-    gwy_data_field_copy(field, result, FALSE);
+    buffer = gwy_field_new_alike(result, FALSE);
+    gwy_field_copy_data(field, buffer);
     if (!gwy_app_wait_set_message(_("Filtering...")))
         goto finish;
 
-    kernel = gwy_data_field_new(size, size, size, size, TRUE);
-    n = gwy_data_field_elliptic_area_fill(kernel, 0, 0, size, size, 1.0);
-    if (type == FEATURES_POSITIVE || type == FEATURES_NEGATIVE) {
-        GwyMinMaxFilterType filtertpe = (type == FEATURES_POSITIVE
+    kernel = gwy_field_new(size, size, size, size, TRUE);
+    n = gwy_field_elliptic_area_fill(kernel, 0, 0, size, size, 1.0);
+    if (type == GWY_SIGN_POSITIVE || type == GWY_SIGN_NEGATIVE) {
+        GwyMinMaxFilterType filtertpe = (type == GWY_SIGN_POSITIVE
                                          ? GWY_MIN_MAX_FILTER_OPENING
                                          : GWY_MIN_MAX_FILTER_CLOSING);
-        gwy_data_field_area_filter_min_max(result, kernel, filtertpe, 0, 0, xres, yres);
+        gwy_field_area_filter_min_max(buffer, kernel, filtertpe, 0, 0, xres, yres);
     }
     else {
-        if (!gwy_data_field_area_filter_kth_rank(result, kernel, 0, 0, xres, yres, n/2, gwy_app_wait_set_fraction))
+        if (!gwy_field_area_filter_kth_rank(buffer, kernel, 0, 0, xres, yres, n/2, gwy_app_wait_set_fraction))
             goto finish;
     }
 
     /* Then find look at the difference and mark any outliers in it because these must be defects. */
-    difffield = gwy_data_field_new_alike(field, FALSE);
-    gwy_data_field_subtract_fields(difffield, field, result);
-    gwy_data_field_fill(result, 1.0);
+    difffield = gwy_field_new_alike(field, FALSE);
+    gwy_field_subtract_fields(difffield, field, buffer);
+    gwy_field_fill(buffer, 1.0);
 
     if (!gwy_app_wait_set_message(_("Marking outliers...")))
         goto finish;
 
     n = 0;
-    while ((nn = unmark_disconnected_values(difffield, result, n, 4.0*threshold)))
+    while ((nn = unmark_disconnected_values(difffield, buffer, n, 4.0*threshold)))
         n += nn;
 
-    gwy_data_field_grains_invert(result);
+    gwy_field_grains_invert(buffer);
     if (mask && combine) {
         if (combine_type == GWY_MERGE_UNION)
-            gwy_data_field_grains_add(result, mask);
+            gwy_field_grains_add(buffer, mask);
         else if (combine_type == GWY_MERGE_INTERSECTION)
-            gwy_data_field_grains_intersect(result, mask);
+            gwy_field_grains_intersect(buffer, mask);
     }
+    gwy_field_copy_data(buffer, result);
     ok = TRUE;
 
 finish:
     gwy_app_wait_finish();
-    GWY_OBJECT_UNREF(kernel);
-    GWY_OBJECT_UNREF(difffield);
+    g_clear_object(&kernel);
+    g_clear_object(&buffer);
+    g_clear_object(&difffield);
     return ok;
 }
 
