@@ -1,5 +1,5 @@
 /*
- *  $Id: mark_with.c 24975 2022-08-31 12:05:41Z yeti-dn $
+ *  $Id: mark_with.c 28910 2025-11-24 18:11:09Z yeti-dn $
  *  Copyright (C) 2009-2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,20 +16,12 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libprocess/stats.h>
-#include <libprocess/arithmetic.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwydgets/gwyradiobuttons.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "preview.h"
 
-#define MARK_RUN_MODES GWY_RUN_INTERACTIVE
+#define RUN_MODES GWY_RUN_INTERACTIVE
 
 typedef enum {
     MARK_WITH_MASK = 0,
@@ -61,44 +53,44 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *mask;
-    GwyDataField *source;
-    GwyDataField *result;
+    GwyField *field;
+    GwyField *mask;
+    GwyField *source;
+    GwyField *result;
 } ModuleArgs;
 
 typedef struct {
     ModuleArgs *args;
     GtkWidget *dialog;
+    GtkWidget *srcview;
     GwyParamTable *table;
     GSList *withwhat;
     GtkWidget *operand[3];
     gboolean has_any[3];
     GtkSizeGroup *sizegroup;
-    GwyContainer *data;
 } ModuleGUI;
 
 static gboolean         module_register                (void);
 static GwyParamDef*     define_module_params           (void);
-static void             mark_with                      (GwyContainer *data,
-                                                        GwyRunType runtype);
+static void             module_main                    (GwyFile *data,
+                                                        GwyRunModeFlags mode);
 static GwyDialogOutcome run_gui                        (ModuleArgs *args,
-                                                        GwyContainer *data,
+                                                        GwyFile *data,
                                                         gint id);
 static void             execute                        (ModuleArgs *args);
 static void             param_changed                  (ModuleGUI *gui,
                                                         gint id);
 static void             preview                        (gpointer user_data);
-static GwyDataField*    get_other_field                (GwyParams *params,
-                                                        GwyContainer **pdata,
+static GwyField*        get_other_field                (GwyParams *params,
+                                                        GwyFile **pdata,
                                                         gint *id,
                                                         gboolean base_field);
 static void             dialog_response                (GwyDialog *dialog,
                                                         gint response,
                                                         ModuleGUI *gui);
 static void             ensure_something_is_selected   (ModuleGUI *gui);
-static void             gwy_data_field_threshold_to    (GwyDataField *source,
-                                                        GwyDataField *dest,
+static void             gwy_field_threshold_to         (GwyField *source,
+                                                        GwyField *dest,
                                                         gdouble min,
                                                         gdouble max);
 static GtkWidget*       create_operand_row_mask        (gpointer user_data);
@@ -106,22 +98,22 @@ static GtkWidget*       create_operand_row_image       (gpointer user_data);
 static GtkWidget*       create_operand_row_presentation(gpointer user_data);
 static void             operand_changed                (GwyDataChooser *chooser,
                                                         ModuleGUI *gui);
-static gboolean         operand_filter_mask            (GwyContainer *source,
+static gboolean         operand_filter_mask            (GwyFile *source,
                                                         gint id,
                                                         gpointer user_data);
-static gboolean         operand_filter_image           (GwyContainer *source,
+static gboolean         operand_filter_image           (GwyFile *source,
                                                         gint id,
                                                         gpointer user_data);
-static gboolean         operand_filter_presentation    (GwyContainer *source,
+static gboolean         operand_filter_presentation    (GwyFile *source,
                                                         gint id,
                                                         gpointer user_data);
 static void             with_what_selected             (GtkRadioButton *button,
                                                         ModuleGUI *gui);
 
 static const GwyEnum withwhats[] = {
-    { "with|_Mask:",         MARK_WITH_MASK,  },
-    { "with|_Image:",        MARK_WITH_IMAGE, },
-    { "with|_Presentation:", MARK_WITH_SHOW,  },
+    { gwy_NC("with", "_Mask:"),         MARK_WITH_MASK,  },
+    { gwy_NC("with", "_Image:"),        MARK_WITH_IMAGE, },
+    { gwy_NC("with", "_Presentation:"), MARK_WITH_SHOW,  },
 };
 
 static GwyModuleInfo module_info = {
@@ -140,11 +132,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("mark_with",
-                              (GwyProcessFunc)&mark_with,
+                              module_main,
                               N_("/_Mask/Mark _With..."),
-                              GWY_STOCK_MARK_WITH,
-                              MARK_RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_ICON_MARK_WITH,
+                              RUN_MODES,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Mask combining and modification"));
     return TRUE;
 }
@@ -181,26 +173,26 @@ define_module_params(void)
 }
 
 static void
-mark_with(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome;
     ModuleArgs args;
     GQuark mquark;
     gint id;
 
-    g_return_if_fail(runtype & MARK_RUN_MODES);
+    g_return_if_fail(mode & RUN_MODES);
 
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD_ID, &id,
-                                     GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_MASK_FIELD, &args.mask,
-                                     GWY_APP_MASK_FIELD_KEY, &mquark,
-                                     0);
+    gwy_data_browser_get_current(GWY_APP_FIELD_ID, &id,
+                                 GWY_APP_FIELD, &args.field,
+                                 GWY_APP_MASK_FIELD, &args.mask,
+                                 GWY_APP_MASK_FIELD_KEY, &mquark,
+                                 0);
     g_return_if_fail(args.field && mquark);
 
-    args.source = gwy_data_field_new_alike(args.field, TRUE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(args.source), NULL);
-    args.result = gwy_data_field_new_alike(args.field, TRUE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(args.result), NULL);
+    args.source = gwy_field_new_alike(args.field, TRUE);
+    gwy_unit_clear(gwy_field_get_unit_z(args.source));
+    args.result = gwy_field_new_alike(args.field, TRUE);
+    gwy_unit_clear(gwy_field_get_unit_z(args.result));
     args.params = gwy_params_new_from_settings(define_module_params());
 
     outcome = run_gui(&args, data, id);
@@ -209,9 +201,9 @@ mark_with(GwyContainer *data, GwyRunType runtype)
     if (outcome == GWY_DIALOG_PROCEED)
         execute(&args);
 
-    gwy_app_undo_qcheckpointv(data, 1, &mquark);
-    gwy_container_set_object(data, mquark, args.result);
-    gwy_app_channel_log_add_proc(data, id, id);
+    gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &mquark);
+    gwy_container_set_object(GWY_CONTAINER(data), mquark, args.result);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, id);
 
 end:
     gwy_params_save_to_settings(args.params);
@@ -221,71 +213,57 @@ end:
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
-    GwyDialogOutcome outcome;
     GtkWidget *hbox, *vbox, *label, *vbox2, *dataview;
     GwyDialog *dialog;
     GwyParamTable *table;
     ModuleGUI gui;
     gchar *s;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
-    gui.data = gwy_container_new();
-
-    /* Store just something with correct dimensions there.  We need to update it according to what is selected. */
-    gwy_container_set_object_by_name(gui.data, "/0/data", args->field);
-    gwy_container_set_object_by_name(gui.data, "/0/mask", args->source);
-    gwy_app_sync_data_items(data, gui.data, id, 0, FALSE,
-                            GWY_DATA_ITEM_PALETTE,
-                            GWY_DATA_ITEM_MASK_COLOR,
-                            GWY_DATA_ITEM_RANGE,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
-
-    gwy_container_set_object_by_name(gui.data, "/1/data", args->field);
-    gwy_container_set_object_by_name(gui.data, "/1/mask", args->result);
-    gwy_app_sync_data_items(data, gui.data, id, 1, FALSE,
-                            GWY_DATA_ITEM_PALETTE,
-                            GWY_DATA_ITEM_RANGE,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
 
     gui.dialog = gwy_dialog_new(_("Mark With"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_UPDATE, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    hbox = gwy_hbox_new(8);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 8);
     gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
     gwy_dialog_add_content(dialog, hbox, FALSE, FALSE, 0);
 
-    vbox = gwy_vbox_new(8);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 8);
     gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);
 
     /* Source mask preview. */
-    vbox2 = gtk_vbox_new(FALSE, 2);
+    vbox2 = gtk_box_new(GTK_ORIENTATION_VERTICAL, 2);
     gtk_box_pack_start(GTK_BOX(vbox), vbox2, FALSE, FALSE, 0);
 
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_HALF_SIZE, TRUE);
+    dataview = gui.srcview = gwy_create_preview(args->field, args->source, PREVIEW_HALF_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_MASK_COLOR
+                        | GWY_FILE_ITEM_RANGE | GWY_FILE_ITEM_REAL_SQUARE);
     gtk_box_pack_start(GTK_BOX(vbox2), dataview, FALSE, FALSE, 0);
 
     label = gtk_label_new(_("Operand"));
-    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+    gtk_label_set_xalign(GTK_LABEL(label), 0.0);
     gtk_box_pack_start(GTK_BOX(vbox2), label, FALSE, FALSE, 0);
 
     /* Result preview */
-    vbox2 = gtk_vbox_new(FALSE, 2);
+    vbox2 = gtk_box_new(GTK_ORIENTATION_VERTICAL, 2);
     gtk_box_pack_start(GTK_BOX(vbox), vbox2, FALSE, FALSE, 0);
 
-    dataview = gwy_create_preview(gui.data, 1, PREVIEW_HALF_SIZE, TRUE);
+    dataview = gwy_create_preview(args->field, args->result, PREVIEW_HALF_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_MASK_COLOR
+                        | GWY_FILE_ITEM_RANGE | GWY_FILE_ITEM_REAL_SQUARE);
     gtk_box_pack_start(GTK_BOX(vbox2), dataview, FALSE, FALSE, 0);
 
     label = gtk_label_new(_("Result"));
-    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+    gtk_label_set_xalign(GTK_LABEL(label), 0.0);
     gtk_box_pack_start(GTK_BOX(vbox2), label, FALSE, FALSE, 0);
 
-    vbox = gwy_vbox_new(8);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 8);
     gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);
 
     table = gui.table = gwy_param_table_new(args->params);
@@ -301,6 +279,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_param_table_append_foreign(table, PARAM_OPERAND_MASK, create_operand_row_mask, &gui, NULL);
     gwy_param_table_append_foreign(table, PARAM_OPERAND_IMAGE, create_operand_row_image, &gui, NULL);
     gwy_param_table_append_foreign(table, PARAM_OPERAND_SHOW, create_operand_row_presentation, &gui, NULL);
+    g_object_unref(gui.sizegroup);
 
     s = g_strconcat(_("Marked data range"), ":", NULL);
     gwy_param_table_append_message(table, LABEL_RANGE, s);
@@ -313,7 +292,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_param_table_append_separator(table);
     gwy_param_table_append_checkbox(table, PARAM_UPDATE);
     /* The selectable mask colour is for the result.  Source is displayed as-is. */
-    gwy_param_table_append_mask_color(table, PARAM_MASK_COLOR, gui.data, 1, data, id);
+    gwy_param_table_append_image_mask_color(table, PARAM_MASK_COLOR, GWY_DATA_VIEW(dataview), data, id);
 
     gtk_box_pack_start(GTK_BOX(vbox), gwy_param_table_widget(gui.table), FALSE, TRUE, 0);
     gwy_dialog_add_param_table(dialog, gui.table);
@@ -323,12 +302,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_dialog_set_preview_func(dialog, GWY_PREVIEW_IMMEDIATE, preview, &gui, NULL);
     ensure_something_is_selected(&gui);
 
-    outcome = gwy_dialog_run(dialog);
-
-    g_object_unref(gui.data);
-    g_object_unref(gui.sizegroup);
-
-    return outcome;
+    return gwy_dialog_run(dialog);
 }
 
 static void
@@ -340,41 +314,41 @@ execute(ModuleArgs *args)
     gdouble min = gwy_params_get_double(params, PARAM_MIN);
     gdouble max = gwy_params_get_double(params, PARAM_MAX);
     gdouble data_min, data_max, d;
-    GwyDataField *otherfield;
+    GwyField *otherfield;
 
     /* XXX: This does not have to be done when just the operation changes.  But well… */
     otherfield = get_other_field(params, NULL, NULL, FALSE);
     if (withwhat == MARK_WITH_MASK)
-        gwy_data_field_copy(otherfield, args->source, FALSE);
+        gwy_field_copy_data(otherfield, args->source);
     else {
-        gwy_data_field_get_min_max(otherfield, &data_min, &data_max);
+        gwy_field_get_min_max(otherfield, &data_min, &data_max);
         d = data_max - data_min;
-        gwy_data_field_threshold_to(otherfield, args->source, data_min + d*min, data_min + d*max);
+        gwy_field_threshold_to(otherfield, args->source, data_min + d*min, data_min + d*max);
     }
 
     /* Simple cases. */
     if (!args->mask || operation == MASK_EDIT_SET) {
         if (operation == MASK_EDIT_SET || operation == MASK_EDIT_ADD)
-            gwy_data_field_copy(args->source, args->result, FALSE);
+            gwy_field_copy_data(args->source, args->result);
         else
-            gwy_data_field_clear(args->result);
+            gwy_field_clear(args->result);
         return;
     }
 
     /* Not so simple cases. */
     if (operation == MASK_EDIT_ADD)
-        gwy_data_field_max_of_fields(args->result, args->mask, args->source);
+        gwy_field_max_of_fields(args->result, args->mask, args->source);
     else if (operation == MASK_EDIT_INTERSECT)
-        gwy_data_field_min_of_fields(args->result, args->mask, args->source);
+        gwy_field_min_of_fields(args->result, args->mask, args->source);
     else if (operation == MASK_EDIT_REMOVE) {
         const gdouble *sdata, *odata;
         gdouble *mdata;
         gint i, n;
 
-        n = gwy_data_field_get_xres(args->source) * gwy_data_field_get_yres(args->source);
-        mdata = gwy_data_field_get_data(args->result);
-        odata = gwy_data_field_get_data_const(args->mask);
-        sdata = gwy_data_field_get_data_const(args->source);
+        n = gwy_field_get_xres(args->source) * gwy_field_get_yres(args->source);
+        mdata = gwy_field_get_data(args->result);
+        odata = gwy_field_get_data_const(args->mask);
+        sdata = gwy_field_get_data_const(args->source);
         for (i = 0; i < n; i++)
             mdata[i] = MIN(odata[i], 1.0 - sdata[i]);
     }
@@ -408,58 +382,55 @@ preview(gpointer user_data)
     ModuleGUI *gui = (ModuleGUI*)user_data;
     ModuleArgs *args = gui->args;
     GwyParams *params = args->params;
-    GwyDataField *otherfield;
-    GwyContainer *data;
+    GwyFile *data;
+    GwyField *otherfield;
     gint id;
 
     if (!(otherfield = get_other_field(params, &data, &id, TRUE)))
         return;
     execute(args);
-    gwy_data_field_data_changed(args->source);
-    gwy_data_field_data_changed(args->result);
-    gwy_container_set_object_by_name(gui->data, "/0/data", otherfield);
-    gwy_app_sync_data_items(data, gui->data, id, 0, FALSE,
-                            GWY_DATA_ITEM_PALETTE,
-                            GWY_DATA_ITEM_MASK_COLOR,
-                            GWY_DATA_ITEM_RANGE,
-                            0);
+    gwy_field_data_changed(args->source);
+    gwy_field_data_changed(args->result);
+    gwy_data_view_set_field(GWY_DATA_VIEW(gui->srcview), otherfield);
+    gwy_setup_data_view(GWY_DATA_VIEW(gui->srcview), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_MASK_COLOR | GWY_FILE_ITEM_RANGE);
     gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
 }
 
-static GwyDataField*
-get_other_field(GwyParams *params, GwyContainer **pdata, gint *id,
+static GwyField*
+get_other_field(GwyParams *params, GwyFile **pdata, gint *id,
                 gboolean base_field)
 {
     MarkWithWhat withwhat = gwy_params_get_enum(params, PARAM_MARK_WITH);
-    GwyContainer *data;
     GwyAppDataId dataid;
+    GwyFile *data;
     GQuark quark;
 
     if (withwhat == MARK_WITH_MASK) {
         if (gwy_params_data_id_is_none(params, PARAM_OPERAND_MASK))
             return NULL;
         dataid = gwy_params_get_data_id(params, PARAM_OPERAND_MASK);
-        quark = base_field ? gwy_app_get_data_key_for_id(dataid.id) : gwy_app_get_mask_key_for_id(dataid.id);
+        quark = base_field ? gwy_file_key_image(dataid.id) : gwy_file_key_image_mask(dataid.id);
     }
     else if (withwhat == MARK_WITH_SHOW) {
         if (gwy_params_data_id_is_none(params, PARAM_OPERAND_SHOW))
             return NULL;
         dataid = gwy_params_get_data_id(params, PARAM_OPERAND_SHOW);
-        quark = gwy_app_get_show_key_for_id(dataid.id);
+        quark = gwy_file_key_image_picture(dataid.id);
     }
     else {
         if (gwy_params_data_id_is_none(params, PARAM_OPERAND_IMAGE))
             return NULL;
         dataid = gwy_params_get_data_id(params, PARAM_OPERAND_IMAGE);
-        quark = gwy_app_get_data_key_for_id(dataid.id);
+        quark = gwy_file_key_image(dataid.id);
     }
-    data = gwy_app_data_browser_get(dataid.datano);
+    data = gwy_data_browser_get_file(dataid.datano);
     if (pdata)
         *pdata = data;
     if (id)
         *id = dataid.id;
 
-    return gwy_container_get_object(data, quark);
+    return gwy_container_get_object(GWY_CONTAINER(data), quark);
 }
 
 static void
@@ -476,17 +447,18 @@ ensure_something_is_selected(ModuleGUI *gui)
         gwy_radio_buttons_set_current(gui->withwhat, MARK_WITH_IMAGE);
 }
 
+/* Use gwy_nield_mark_by_threshold(). */
 static void
-gwy_data_field_threshold_to(GwyDataField *source, GwyDataField *dest,
-                            gdouble min, gdouble max)
+gwy_field_threshold_to(GwyField *source, GwyField *dest,
+                       gdouble min, gdouble max)
 {
     const gdouble *dsrc;
     gdouble *ddst;
     gint i, n;
 
-    n = gwy_data_field_get_xres(source) * gwy_data_field_get_yres(source);
-    dsrc = gwy_data_field_get_data_const(source);
-    ddst = gwy_data_field_get_data(dest);
+    n = gwy_field_get_xres(source) * gwy_field_get_yres(source);
+    dsrc = gwy_field_get_data_const(source);
+    ddst = gwy_field_get_data(dest);
 
     if (min <= max) {
         for (i = 0; i < n; i++)
@@ -510,10 +482,10 @@ create_operand_row(ModuleGUI *gui, gint i, gint param_id)
     GSList *l;
 
     l = g_slist_nth(gui->withwhat, i);
-    hbox = gwy_hbox_new(6);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 6);
     gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(l->data), FALSE, FALSE, 0);
 
-    gui->operand[i] = gwy_data_chooser_new_channels();
+    gui->operand[i] = gwy_data_chooser_new(GWY_FILE_IMAGE);
     g_object_set_data(G_OBJECT(gui->operand[i]), "param-id", GINT_TO_POINTER(param_id));
     chooser = GWY_DATA_CHOOSER(gui->operand[i]);
     gtk_size_group_add_widget(gui->sizegroup, gui->operand[i]);
@@ -560,36 +532,34 @@ create_operand_row_presentation(gpointer user_data)
 }
 
 static gboolean
-operand_filter(GwyContainer *source, GQuark quark, ModuleGUI *gui)
+operand_filter(GwyFile *source, GQuark quark, ModuleGUI *gui)
 {
-    GwyDataField *target_field = gui->args->field;
-    GwyDataField *source_field;
+    GwyField *target_field = gui->args->field;
+    GwyField *source_field;
 
-    if (!gwy_container_gis_object(source, quark, &source_field))
+    if (!gwy_container_gis_object(GWY_CONTAINER(source), quark, &source_field))
         return FALSE;
 
-    return !gwy_data_field_check_compatibility(source_field, target_field,
-                                               GWY_DATA_COMPATIBILITY_RES
-                                               | GWY_DATA_COMPATIBILITY_REAL
-                                               | GWY_DATA_COMPATIBILITY_LATERAL);
+    return !gwy_field_is_incompatible(source_field, target_field,
+                                      GWY_DATA_MISMATCH_RES | GWY_DATA_MISMATCH_REAL | GWY_DATA_MISMATCH_LATERAL);
 }
 
 static gboolean
-operand_filter_mask(GwyContainer *source, gint id, gpointer user_data)
+operand_filter_mask(GwyFile *source, gint id, gpointer user_data)
 {
-    return operand_filter(source, gwy_app_get_mask_key_for_id(id), (ModuleGUI*)user_data);
+    return operand_filter(source, gwy_file_key_image_mask(id), (ModuleGUI*)user_data);
 }
 
 static gboolean
-operand_filter_image(GwyContainer *source, gint id, gpointer user_data)
+operand_filter_image(GwyFile *source, gint id, gpointer user_data)
 {
-    return operand_filter(source, gwy_app_get_data_key_for_id(id), (ModuleGUI*)user_data);
+    return operand_filter(source, gwy_file_key_image(id), (ModuleGUI*)user_data);
 }
 
 static gboolean
-operand_filter_presentation(GwyContainer *source, gint id, gpointer user_data)
+operand_filter_presentation(GwyFile *source, gint id, gpointer user_data)
 {
-    return operand_filter(source, gwy_app_get_show_key_for_id(id), (ModuleGUI*)user_data);
+    return operand_filter(source, gwy_file_key_image_picture(id), (ModuleGUI*)user_data);
 }
 
 static void
