@@ -1,5 +1,5 @@
 /*
- *  $Id: mask_distribute.c 25638 2023-09-07 12:23:28Z yeti-dn $
+ *  $Id: mask_distribute.c 28821 2025-11-06 16:00:41Z yeti-dn $
  *  Copyright (C) 2003-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,13 +16,8 @@
  */
 
 #include "config.h"
-#include <libgwyddion/gwymacros.h>
-#include <libprocess/arithmetic.h>
-#include <libgwydgets/gwydgetutils.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <glib/gi18n-lib.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
@@ -39,13 +34,13 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *mask;
+    GwyField *mask;
 } ModuleArgs;
 
 typedef struct {
     ModuleArgs *args;
     GArray *undo_quarks;
-    GwyContainer *container;
+    GwyFile *file;
     gint id;
 } MaskDistribData;
 
@@ -57,11 +52,11 @@ typedef struct {
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             mask_distribute     (GwyContainer *data,
-                                             GwyRunType runtype);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static GwyDialogOutcome run_gui             (ModuleArgs *args);
-static void             distribute_in_one   (GwyContainer *container,
-                                             MaskDistribData *distdata);
+static void             distribute_in_one   (GwyFile *file,
+                                             gpointer user_data);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
@@ -79,11 +74,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("mask_distribute",
-                              (GwyProcessFunc)&mask_distribute,
+                              module_main,
                               N_("/_Mask/_Distribute..."),
-                              GWY_STOCK_MASK_DISTRIBUTE,
+                              GWY_ICON_MASK_DISTRIBUTE,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA_MASK | GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE_MASK | GWY_MENU_FLAG_IMAGE,
                               N_("Distribute mask to other channels"));
 
     return TRUE;
@@ -110,33 +105,33 @@ define_module_params(void)
 }
 
 static void
-mask_distribute(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     MaskDistribData distdata;
     ModuleArgs args;
     gint id;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_MASK_FIELD, &args.mask,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_MASK_FIELD, &args.mask,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(args.mask && id >= 0);
 
     args.params = gwy_params_new_from_settings(define_module_params());
-    if (runtype != GWY_RUN_IMMEDIATE) {
+    if (mode !=GWY_RUN_IMMEDIATE) {
         GwyDialogOutcome outcome = run_gui(&args);
         gwy_params_save_to_settings(args.params);
         if (outcome != GWY_DIALOG_PROCEED)
             goto end;
     }
 
-    distdata.container = data;
+    distdata.file = data;
     distdata.id = id;
     distdata.args = &args;
     distdata.undo_quarks = g_array_new(FALSE, FALSE, sizeof(GQuark));
 
     if (gwy_params_get_enum(args.params, PARAM_MODE) == DISTRIBUTE_TO_ALL_FILES)
-        gwy_app_data_browser_foreach((GwyAppDataForeachFunc)distribute_in_one, &distdata);
+        gwy_data_browser_foreach(distribute_in_one, &distdata, NULL);
     else
         distribute_in_one(data, &distdata);
 
@@ -171,10 +166,11 @@ run_gui(ModuleArgs *args)
 }
 
 static void
-distribute_in_one(GwyContainer *container, MaskDistribData *distdata)
+distribute_in_one(GwyFile *file, gpointer user_data)
 {
-    GwyDataField *field, *mask;
-    GwyDataCompatibilityFlags flags = GWY_DATA_COMPATIBILITY_RES | GWY_DATA_COMPATIBILITY_REAL;
+    MaskDistribData *distdata = (MaskDistribData*)user_data;
+    GwyField *field, *mask;
+    GwyDataMismatchFlags flags = GWY_DATA_MISMATCH_RES | GWY_DATA_MISMATCH_REAL;
     gboolean keep_existing = gwy_params_get_boolean(distdata->args->params, PARAM_KEEP_EXISTING);
     GArray *undo_quarks = distdata->undo_quarks;
     GQuark quark;
@@ -182,38 +178,34 @@ distribute_in_one(GwyContainer *container, MaskDistribData *distdata)
     guint i;
 
     g_array_set_size(undo_quarks, 0);
-    ids = gwy_app_data_browser_get_data_ids(container);
+    ids = gwy_file_get_ids(file, GWY_FILE_IMAGE);
 
     for (i = 0; ids[i] >= 0; i++) {
-        if (container == distdata->container && ids[i] == distdata->id)
+        if (file == distdata->file && ids[i] == distdata->id)
             continue;
 
-        quark = gwy_app_get_data_key_for_id(ids[i]);
-        field = gwy_container_get_object(container, quark);
+        field = gwy_file_get_image(file, ids[i]);
         g_return_if_fail(field);
-
-        quark = gwy_app_get_mask_key_for_id(ids[i]);
-        if (!gwy_container_gis_object(container, quark, &mask))
-            mask = NULL;
+        mask = gwy_file_get_image_mask(file, ids[i]);
 
         if (mask && keep_existing)
             continue;
-        if (gwy_data_field_check_compatibility(distdata->args->mask, field, flags))
+        if (gwy_field_is_incompatible(distdata->args->mask, field, flags))
             continue;
 
         g_array_append_val(undo_quarks, quark);
-        gwy_app_channel_log_add_proc(container, -1, ids[i]);
+        gwy_log_add(file, GWY_FILE_IMAGE, -1, ids[i]);
     }
     g_free(ids);
 
     if (!undo_quarks->len)
         return;
 
-    gwy_app_undo_qcheckpointv(container, undo_quarks->len, (GQuark*)undo_quarks->data);
+    gwy_app_undo_qcheckpointv(GWY_CONTAINER(file), undo_quarks->len, (GQuark*)undo_quarks->data);
     for (i = 0; i < undo_quarks->len; i++) {
         quark = g_array_index(undo_quarks, GQuark, i);
-        mask = gwy_data_field_duplicate(distdata->args->mask);
-        gwy_container_pass_object(container, quark, mask);
+        mask = gwy_field_copy(distdata->args->mask);
+        gwy_container_pass_object(GWY_CONTAINER(file), quark, mask);
     }
 }
 
