@@ -1,5 +1,5 @@
 /*
- *  $Id: mask_morph.c 23666 2021-05-10 21:56:10Z yeti-dn $
+ *  $Id: mask_morph.c 28821 2025-11-06 16:00:41Z yeti-dn $
  *  Copyright (C) 2015-2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,17 +16,9 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/elliptic.h>
-#include <libprocess/filters.h>
-#include <libprocess/grains.h>
-#include <libprocess/stats.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
@@ -59,30 +51,29 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *mask;
-    GwyDataField *result;
+    GwyField *field;
+    GwyField *mask;
+    GwyField *result;
 } ModuleArgs;
 
 typedef struct {
     ModuleArgs *args;
     GtkWidget *dialog;
     GwyParamTable *table;
-    GwyContainer *data;
 } ModuleGUI;
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             mask_morph          (GwyContainer *data,
-                                             GwyRunType run);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static void             execute             (ModuleArgs *args);
 static GwyDialogOutcome run_gui             (ModuleArgs *args,
-                                             GwyContainer *data,
+                                             GwyFile *data,
                                              gint id);
 static void             param_changed       (ModuleGUI *gui,
                                              gint id);
 static void             preview             (gpointer user_data);
-static gboolean         kernel_filter       (GwyContainer *data,
+static gboolean         kernel_filter       (GwyFile *data,
                                              gint id,
                                              gpointer user_data);
 static void             sanitise_params     (ModuleArgs *args);
@@ -103,11 +94,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("mask_morph",
-                              (GwyProcessFunc)&mask_morph,
+                              module_main,
                               N_("/_Mask/Morpho_logical Operation..."),
-                              GWY_STOCK_MASK_MORPH,
+                              GWY_ICON_MASK_MORPH,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA_MASK | GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE_MASK | GWY_MENU_FLAG_IMAGE,
                               N_("Morphological operation with mask"));
 
     return TRUE;
@@ -119,8 +110,8 @@ define_module_params(void)
     static const GwyEnum operations[] = {
         { N_("Erosion"),        MASKMORPH_EROSION,     },
         { N_("Dilation"),       MASKMORPH_DILATION,    },
-        { N_("filter|Opening"), MASKMORPH_OPENING,     },
-        { N_("filter|Closing"), MASKMORPH_CLOSING,     },
+        { gwy_NC("filter", "Opening"), MASKMORPH_OPENING,     },
+        { gwy_NC("filter", "Closing"), MASKMORPH_CLOSING,     },
         { N_("ASF Opening"),    MASKMORPH_ASF_OPENING, },
         { N_("ASF Closing"),    MASKMORPH_ASF_CLOSING, },
     };
@@ -150,25 +141,25 @@ define_module_params(void)
 }
 
 static void
-mask_morph(GwyContainer *data, GwyRunType run)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     GQuark mquark;
     gint id;
 
-    g_return_if_fail(run & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_MASK_FIELD, &args.mask,
-                                     GWY_APP_MASK_FIELD_KEY, &mquark,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_MASK_FIELD, &args.mask,
+                                 GWY_APP_MASK_FIELD_KEY, &mquark,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(args.mask);
 
-    args.result = gwy_data_field_new_alike(args.mask, TRUE);
+    args.result = gwy_field_new_alike(args.mask, TRUE);
     args.params = gwy_params_new_from_settings(define_module_params());
     sanitise_params(&args);
-    if (run == GWY_RUN_INTERACTIVE) {
+    if (mode == GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, id);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -177,12 +168,12 @@ mask_morph(GwyContainer *data, GwyRunType run)
     if (outcome != GWY_DIALOG_HAVE_RESULT)
         execute(&args);
 
-    gwy_app_undo_qcheckpointv(data, 1, &mquark);
-    if (gwy_data_field_get_max(args.result) > 0.0)
-        gwy_container_set_object(data, mquark, args.result);
+    gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &mquark);
+    if (gwy_field_get_max(args.result) > 0.0)
+        gwy_container_set_object(GWY_CONTAINER(data), mquark, args.result);
     else
-        gwy_container_remove(data, mquark);
-    gwy_app_channel_log_add_proc(data, id, id);
+        gwy_container_remove(GWY_CONTAINER(data), mquark);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, id);
 
 end:
     g_object_unref(args.result);
@@ -190,31 +181,24 @@ end:
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
-    GwyDialogOutcome outcome;
     GtkWidget *hbox, *dataview;
     GwyDialog *dialog;
     GwyParamTable *table;
     ModuleGUI gui;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
-    gui.data = gwy_container_new();
-
-    gwy_container_set_object_by_name(gui.data, "/0/data", args->field);
-    gwy_container_set_object_by_name(gui.data, "/0/mask", args->result);
-    gwy_app_sync_data_items(data, gui.data, id, 0, FALSE,
-                            GWY_DATA_ITEM_PALETTE,
-                            GWY_DATA_ITEM_RANGE,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
 
     gui.dialog = gwy_dialog_new(_("Morphological Operation"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_UPDATE, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, TRUE);
+    dataview = gwy_create_preview(args->field, args->result, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_MASK_COLOR
+                        | GWY_FILE_ITEM_RANGE | GWY_FILE_ITEM_REAL_SQUARE);
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), FALSE);
 
     table = gui.table = gwy_param_table_new(args->params);
@@ -227,7 +211,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_param_table_append_image_id(table, PARAM_KERNEL);
     gwy_param_table_data_id_set_filter(table, PARAM_KERNEL, kernel_filter, args->mask, NULL);
     gwy_param_table_append_checkbox(table, PARAM_CROP_KERNEL);
-    gwy_param_table_append_mask_color(table, PARAM_MASK_COLOR, gui.data, 0, data, id);
+    gwy_param_table_append_image_mask_color(table, PARAM_MASK_COLOR, GWY_DATA_VIEW(dataview), data, id);
 
     gtk_box_pack_start(GTK_BOX(hbox), gwy_param_table_widget(table), TRUE, TRUE, 0);
     gwy_dialog_add_param_table(dialog, table);
@@ -235,11 +219,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     g_signal_connect_swapped(table, "param-changed", G_CALLBACK(param_changed), &gui);
     gwy_dialog_set_preview_func(dialog, GWY_PREVIEW_UPON_REQUEST, preview, &gui, NULL);
 
-    outcome = gwy_dialog_run(dialog);
-
-    g_object_unref(gui.data);
-
-    return outcome;
+    return gwy_dialog_run(dialog);
 }
 
 static void
@@ -277,24 +257,24 @@ preview(gpointer user_data)
     ModuleGUI *gui = (ModuleGUI*)user_data;
 
     execute(gui->args);
-    gwy_data_field_data_changed(gui->args->result);
+    gwy_field_data_changed(gui->args->result);
     gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
 }
 
-static GwyDataField*
+static GwyField*
 create_kernel(MaskMorphShapeType shape, gint radius)
 {
-    GwyDataField *kernel;
+    GwyField *kernel;
     gint i, j, res;
     gdouble *d;
 
     res = 2*radius + 1;
-    kernel = gwy_data_field_new(res, res, res, res, TRUE);
+    kernel = gwy_field_new(res, res, res, res, TRUE);
     if (shape == MASKMORPH_DISC)
-        gwy_data_field_elliptic_area_fill(kernel, 0, 0, res, res, 1.0);
+        gwy_field_elliptic_area_fill(kernel, 0, 0, res, res, 1.0);
     else if (shape == MASKMORPH_OCTAGON || shape == MASKMORPH_DIAMOND) {
         gint rlim = (shape == MASKMORPH_OCTAGON ? GWY_ROUND(res/G_SQRT2) : radius);
-        d = gwy_data_field_get_data(kernel);
+        d = gwy_field_get_data(kernel);
         for (i = 0; i < res; i++) {
             gint ii = ABS(i - radius);
             for (j = 0; j < res; j++) {
@@ -305,7 +285,7 @@ create_kernel(MaskMorphShapeType shape, gint radius)
         }
     }
     else if (shape == MASKMORPH_SQUARE)
-        gwy_data_field_fill(kernel, 1.0);
+        gwy_field_fill(kernel, 1.0);
     else {
         g_assert_not_reached();
     }
@@ -327,7 +307,7 @@ execute(ModuleArgs *args)
         { GWY_MIN_MAX_FILTER_CLOSING,  MASKMORPH_CLOSING,  },
     };
 
-    GwyDataField *kernel, *mask = args->result;
+    GwyField *kernel, *mask = args->result;
     GwyParams *params = args->params;
     MaskMorphOperation operation = gwy_params_get_enum(params, PARAM_MODE);
     MaskMorphShapeType shape = gwy_params_get_enum(params, PARAM_SHAPE);
@@ -336,24 +316,24 @@ execute(ModuleArgs *args)
     GwyMinMaxFilterType filtertype1, filtertype2;
     guint i, xres, yres;
 
-    xres = gwy_data_field_get_xres(mask);
-    yres = gwy_data_field_get_yres(mask);
-    gwy_data_field_copy(args->mask, mask, FALSE);
+    xres = gwy_field_get_xres(mask);
+    yres = gwy_field_get_yres(mask);
+    gwy_field_copy_data(args->mask, mask);
     for (i = 0; i < G_N_ELEMENTS(operation_map); i++) {
         if (operation_map[i].operation != operation)
             continue;
 
         if (shape == MASKMORPH_USER_KERNEL) {
-            if (!(kernel = gwy_params_get_mask(params, PARAM_KERNEL)))
+            if (!(kernel = gwy_params_get_image_mask(params, PARAM_KERNEL)))
                 continue;
-            kernel = gwy_data_field_duplicate(kernel);
+            kernel = gwy_field_copy(kernel);
             if (crop_kernel)
-                gwy_data_field_grains_autocrop(kernel, FALSE, NULL, NULL, NULL, NULL);
+                gwy_field_grains_autocrop(kernel, FALSE, NULL, NULL, NULL, NULL);
         }
         else
             kernel = create_kernel(shape, radius);
 
-        gwy_data_field_area_filter_min_max(mask, kernel, operation_map[i].filtertype, 0, 0, xres, yres);
+        gwy_field_area_filter_min_max(mask, kernel, operation_map[i].filtertype, 0, 0, xres, yres);
         g_object_unref(kernel);
         return;
     }
@@ -365,7 +345,7 @@ execute(ModuleArgs *args)
         return;
 
     if (shape == MASKMORPH_DISC) {
-        gwy_data_field_area_filter_disc_asf(mask, radius, operation == MASKMORPH_ASF_CLOSING, 0, 0, xres, yres);
+        gwy_field_area_filter_disc_asf(mask, radius, operation == MASKMORPH_ASF_CLOSING, 0, 0, xres, yres);
         return;
     }
 
@@ -380,18 +360,18 @@ execute(ModuleArgs *args)
 
     for (i = 1; i <= radius; i++) {
         kernel = create_kernel(shape, i);
-        gwy_data_field_area_filter_min_max(mask, kernel, filtertype1, 0, 0, xres, yres);
-        gwy_data_field_area_filter_min_max(mask, kernel, filtertype2, 0, 0, xres, yres);
+        gwy_field_area_filter_min_max(mask, kernel, filtertype1, 0, 0, xres, yres);
+        gwy_field_area_filter_min_max(mask, kernel, filtertype2, 0, 0, xres, yres);
         g_object_unref(kernel);
     }
 }
 
 static gboolean
-kernel_filter(GwyContainer *data, gint id, gpointer user_data)
+kernel_filter(GwyFile *data, gint id, gpointer user_data)
 {
-    GwyDataField *kernel, *mask = (GwyDataField*)user_data;
+    GwyField *kernel, *mask = (GwyField*)user_data;
 
-    if (!gwy_container_gis_object(data, gwy_app_get_mask_key_for_id(id), &kernel))
+    if (!(kernel = gwy_file_get_image_mask(data, id)))
         return FALSE;
     if (kernel->xres > mask->xres/2 || kernel->yres > mask->yres/2)
         return FALSE;
@@ -408,7 +388,7 @@ sanitise_params(ModuleArgs *args)
     gboolean is_none = gwy_params_data_id_is_none(params, PARAM_KERNEL);
 
     if (shape == MASKMORPH_USER_KERNEL
-        && (is_none || !kernel_filter(gwy_app_data_browser_get(kernel.datano), kernel.id, args->mask)))
+        && (is_none || !kernel_filter(gwy_data_browser_get_file(kernel.datano), kernel.id, args->mask)))
         gwy_params_reset(params, PARAM_SHAPE);
 }
 
