@@ -1,5 +1,5 @@
 /*
- *  $Id: mask_noisify.c 24796 2022-04-28 15:20:59Z yeti-dn $
+ *  $Id: mask_noisify.c 28821 2025-11-06 16:00:41Z yeti-dn $
  *  Copyright (C) 2017-2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,23 +16,12 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwyrandgenset.h>
-#include <libprocess/grains.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwydgets/gwydgetutils.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
 
-typedef enum {
-    NOISE_DIRECTION_BOTH = 0,
-    NOISE_DIRECTION_UP   = 1,
-    NOISE_DIRECTION_DOWN = 2,
-} NoiseDirectionType;
-
 enum {
     PARAM_DENSITY,
     PARAM_DIRECTION,
@@ -40,8 +29,8 @@ enum {
 };
 
 typedef struct {
-    GwyDataField *mask;
-    GwyDataField *result;
+    GwyField *mask;
+    GwyField *result;
     GwyParams *params;
 } ModuleArgs;
 
@@ -53,8 +42,8 @@ typedef struct {
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             mask_noisify        (GwyContainer *data,
-                                             GwyRunType runtype);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static void             execute             (ModuleArgs *args);
 static GwyDialogOutcome run_gui             (ModuleArgs *args);
 
@@ -74,11 +63,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("mask_noisify",
-                              (GwyProcessFunc)&mask_noisify,
+                              module_main,
                               N_("/_Mask/_Noisify..."),
-                              GWY_STOCK_MASK_NOISIFY,
+                              GWY_ICON_MASK_NOISIFY,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA_MASK | GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE_MASK | GWY_MENU_FLAG_IMAGE,
                               N_("Add noise to mask"));
 
     return TRUE;
@@ -88,9 +77,9 @@ static GwyParamDef*
 define_module_params(void)
 {
     static const GwyEnum directions[] = {
-        { N_("S_ymmetrical"),        NOISE_DIRECTION_BOTH, },
-        { N_("One-sided _positive"), NOISE_DIRECTION_UP,   },
-        { N_("One-sided _negative"), NOISE_DIRECTION_DOWN, },
+        { N_("S_ymmetrical"),        GWY_SIGN_BOTH,     },
+        { N_("One-sided _positive"), GWY_SIGN_POSITIVE, },
+        { N_("One-sided _negative"), GWY_SIGN_NEGATIVE, },
     };
     static GwyParamDef *paramdef = NULL;
 
@@ -101,38 +90,38 @@ define_module_params(void)
     gwy_param_def_set_function_name(paramdef, gwy_process_func_current());
     gwy_param_def_add_double(paramdef, PARAM_DENSITY, "density", _("Densi_ty"), 0.0, 1.0, 0.1);
     gwy_param_def_add_gwyenum(paramdef, PARAM_DIRECTION, "direction", _("Noise type"),
-                              directions, G_N_ELEMENTS(directions), NOISE_DIRECTION_BOTH);
+                              directions, G_N_ELEMENTS(directions), GWY_SIGN_BOTH);
     gwy_param_def_add_boolean(paramdef, PARAM_ONLY_BOUNDARIES, "only_boundaries", _("_Alter only boundaries"), FALSE);
     return paramdef;
 }
 
 static void
-mask_noisify(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
     GQuark quark;
     gint id;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_MASK_FIELD, &args.mask,
-                                     GWY_APP_MASK_FIELD_KEY, &quark,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_MASK_FIELD, &args.mask,
+                                 GWY_APP_MASK_FIELD_KEY, &quark,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(args.mask);
 
     args.result = g_object_ref(args.mask);   /* Change to a copy once we have a preview. */
     args.params = gwy_params_new_from_settings(define_module_params());
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         GwyDialogOutcome outcome = run_gui(&args);
         gwy_params_save_to_settings(args.params);
         if (outcome != GWY_DIALOG_PROCEED)
             goto end;
     }
-    gwy_app_undo_qcheckpointv(data, 1, &quark);
+    gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &quark);
     execute(&args);
-    gwy_app_channel_log_add_proc(data, id, id);
-    gwy_data_field_data_changed(args.mask);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, id);
+    gwy_field_data_changed(args.mask);
 
 end:
     g_object_unref(args.result);
@@ -146,7 +135,7 @@ run_gui(ModuleArgs *args)
     GwyDialog *dialog;
     GwyParamTable *table;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
 
     gui.dialog = gwy_dialog_new(_("Noisify Mask"));
@@ -170,8 +159,8 @@ static void
 execute(ModuleArgs *args)
 {
     GwyRandGenSet *rngset = gwy_rand_gen_set_new(1);
-    GwyDataField *mask = args->result;
-    NoiseDirectionType direction = gwy_params_get_enum(args->params, PARAM_DIRECTION);
+    GwyField *mask = args->result;
+    GwySignFlags direction = gwy_params_get_enum(args->params, PARAM_DIRECTION);
     gboolean only_boundaries = gwy_params_get_boolean(args->params, PARAM_ONLY_BOUNDARIES);
     gdouble density = gwy_params_get_double(args->params, PARAM_DENSITY);
     gboolean on_boundary;
@@ -181,22 +170,22 @@ execute(ModuleArgs *args)
     gdouble *m;
     guint32 r;
 
-    gwy_data_field_copy(args->mask, mask, FALSE);
-    xres = gwy_data_field_get_xres(mask);
-    yres = gwy_data_field_get_yres(mask);
+    gwy_field_copy_data(args->mask, mask);
+    xres = gwy_field_get_xres(mask);
+    yres = gwy_field_get_yres(mask);
     n = xres*yres;
     nind = GWY_ROUND(n*density);
     indices = gwy_rand_gen_set_choose_shuffle(rngset, 0, n, nind);
-    m = gwy_data_field_get_data(mask);
+    m = gwy_field_get_data(mask);
 
     for (iter = 0; iter < nind; iter++) {
         k = indices[iter];
 
         /* No-ops. */
         is_m = (m[k] > 0.0);
-        if (direction == NOISE_DIRECTION_UP)
+        if (direction == GWY_SIGN_POSITIVE)
             change_to = 1;
-        else if (direction == NOISE_DIRECTION_DOWN)
+        else if (direction == GWY_SIGN_NEGATIVE)
             change_to = 0;
         else {
             if (!have_bits) {
