@@ -1,5 +1,5 @@
 /*
- *  $Id: mask_shift.c 23666 2021-05-10 21:56:10Z yeti-dn $
+ *  $Id: mask_shift.c 28844 2025-11-10 16:07:51Z yeti-dn $
  *  Copyright (C) 2020-2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,14 +16,9 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/arithmetic.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
@@ -42,9 +37,9 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *mask;
-    GwyDataField *result;
+    GwyField *field;
+    GwyField *mask;
+    GwyField *result;
     /* Cached input image properties. */
     gint hmove_max;
     gint vmove_max;
@@ -54,16 +49,15 @@ typedef struct {
     ModuleArgs *args;
     GtkWidget *dialog;
     GwyParamTable *table;
-    GwyContainer *data;
 } ModuleGUI;
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             mask_shift          (GwyContainer *data,
-                                             GwyRunType runtype);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static void             execute             (ModuleArgs *args);
 static GwyDialogOutcome run_gui             (ModuleArgs *args,
-                                             GwyContainer *data,
+                                             GwyFile *data,
                                              gint id);
 static void             param_changed       (ModuleGUI *gui,
                                              gint id);
@@ -86,11 +80,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("mask_shift",
-                              (GwyProcessFunc)&mask_shift,
+                              module_main,
                               N_("/_Mask/_Shift..."),
-                              GWY_STOCK_MASK_SHIFT,
+                              GWY_ICON_MASK_SHIFT,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA_MASK | GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE_MASK | GWY_MENU_FLAG_IMAGE,
                               N_("Shift mask"));
 
     return TRUE;
@@ -100,11 +94,11 @@ static GwyParamDef*
 define_module_params(void)
 {
     static const GwyEnum exteriors[] = {
-        { N_("exterior|Empty"),    MASKSHIFT_EXTERIOR_EMPTY,   },
-        { N_("exterior|Filled"),   MASKSHIFT_EXTERIOR_FILLED,  },
-        { N_("exterior|Border"),   GWY_EXTERIOR_BORDER_EXTEND, },
-        { N_("exterior|Mirror"),   GWY_EXTERIOR_MIRROR_EXTEND, },
-        { N_("exterior|Periodic"), GWY_EXTERIOR_PERIODIC,      },
+        { gwy_NC("exterior", "Empty"),    MASKSHIFT_EXTERIOR_EMPTY,   },
+        { gwy_NC("exterior", "Filled"),   MASKSHIFT_EXTERIOR_FILLED,  },
+        { gwy_NC("exterior", "Border"),   GWY_EXTERIOR_BORDER, },
+        { gwy_NC("exterior", "Mirror"),   GWY_EXTERIOR_MIRROR, },
+        { gwy_NC("exterior", "Periodic"), GWY_EXTERIOR_PERIODIC,      },
     };
     static GwyParamDef *paramdef = NULL;
 
@@ -114,7 +108,7 @@ define_module_params(void)
     paramdef = gwy_param_def_new();
     gwy_param_def_set_function_name(paramdef, gwy_process_func_current());
     gwy_param_def_add_gwyenum(paramdef, PARAM_EXTERIOR, "exterior", _("_Exterior type"),
-                              exteriors, G_N_ELEMENTS(exteriors), GWY_EXTERIOR_BORDER_EXTEND);
+                              exteriors, G_N_ELEMENTS(exteriors), GWY_EXTERIOR_BORDER);
     gwy_param_def_add_int(paramdef, PARAM_HMOVE, "hmove", _("_Horizontal shift"), -32768, 32768, 0);
     gwy_param_def_add_int(paramdef, PARAM_VMOVE, "vmove", _("_Vertical shift"), -32768, 32768, 0);
     gwy_param_def_add_mask_color(paramdef, PARAM_MASK_COLOR, NULL, NULL);
@@ -122,28 +116,28 @@ define_module_params(void)
 }
 
 static void
-mask_shift(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     ModuleArgs args;
     GQuark quark;
     gint id;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_MASK_FIELD, &args.mask,
-                                     GWY_APP_MASK_FIELD_KEY, &quark,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_MASK_FIELD, &args.mask,
+                                 GWY_APP_MASK_FIELD_KEY, &quark,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(args.mask);
 
-    args.result = gwy_data_field_duplicate(args.mask);
-    args.hmove_max = (gwy_data_field_get_xres(args.field) + 1)/2;
-    args.vmove_max = (gwy_data_field_get_yres(args.field) + 1)/2;
+    args.result = gwy_field_copy(args.mask);
+    args.hmove_max = (gwy_field_get_xres(args.field) + 1)/2;
+    args.vmove_max = (gwy_field_get_yres(args.field) + 1)/2;
     args.params = gwy_params_new_from_settings(define_module_params());
     sanitise_params(&args);
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, id);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -152,9 +146,9 @@ mask_shift(GwyContainer *data, GwyRunType runtype)
     if (outcome != GWY_DIALOG_HAVE_RESULT)
         execute(&args);
 
-    gwy_app_undo_qcheckpointv(data, 1, &quark);
-    gwy_container_set_object(data, quark, args.result);
-    gwy_app_channel_log_add_proc(data, id, id);
+    gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &quark);
+    gwy_container_set_object(GWY_CONTAINER(data), quark, args.result);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, id);
 
 end:
     g_object_unref(args.result);
@@ -162,31 +156,23 @@ end:
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
-    GwyDialogOutcome outcome;
     GtkWidget *hbox, *dataview;
     GwyDialog *dialog;
     GwyParamTable *table;
     ModuleGUI gui;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
-    gui.data = gwy_container_new();
-
-    gwy_container_set_object_by_name(gui.data, "/0/data", args->field);
-    gwy_container_set_object_by_name(gui.data, "/0/mask", args->result);
-    gwy_app_sync_data_items(data, gui.data, id, 0, FALSE,
-                            GWY_DATA_ITEM_PALETTE,
-                            GWY_DATA_ITEM_RANGE,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
 
     gui.dialog = gwy_dialog_new(_("Shift Mask"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, TRUE);
+    dataview = gwy_create_preview(args->field, args->result, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_MASK_COLOR | GWY_FILE_ITEM_RANGE | GWY_FILE_ITEM_REAL_SQUARE);
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), FALSE);
 
     table = gui.table = gwy_param_table_new(args->params);
@@ -205,7 +191,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_param_table_slider_add_alt(table, PARAM_VMOVE);
     gwy_param_table_alt_set_field_pixel_y(table, PARAM_VMOVE, args->field);
 
-    gwy_param_table_append_mask_color(table, PARAM_MASK_COLOR, gui.data, 0, data, id);
+    gwy_param_table_append_image_mask_color(table, PARAM_MASK_COLOR, GWY_DATA_VIEW(dataview), data, id);
 
     gtk_box_pack_start(GTK_BOX(hbox), gwy_param_table_widget(table), TRUE, TRUE, 0);
     gwy_dialog_add_param_table(dialog, table);
@@ -213,11 +199,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     g_signal_connect_swapped(table, "param-changed", G_CALLBACK(param_changed), &gui);
     gwy_dialog_set_preview_func(dialog, GWY_PREVIEW_IMMEDIATE, preview, &gui, NULL);
 
-    outcome = gwy_dialog_run(dialog);
-
-    g_object_unref(gui.data);
-
-    return outcome;
+    return gwy_dialog_run(dialog);
 }
 
 static void
@@ -233,7 +215,7 @@ preview(gpointer user_data)
     ModuleGUI *gui = (ModuleGUI*)user_data;
 
     execute(gui->args);
-    gwy_data_field_data_changed(gui->args->result);
+    gwy_field_data_changed(gui->args->result);
     gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
 }
 
@@ -244,7 +226,7 @@ execute(ModuleArgs *args)
     guint exterior = gwy_params_get_enum(params, PARAM_EXTERIOR);
     gint hmove = gwy_params_get_int(params, PARAM_HMOVE);
     gint vmove = gwy_params_get_int(params, PARAM_VMOVE);
-    GwyDataField *extended, *mask = args->mask;
+    GwyField *extended, *mask = args->mask;
     gint xres, yres;
     gdouble fillvalue = 0.0;
 
@@ -257,11 +239,11 @@ execute(ModuleArgs *args)
         fillvalue = 1.0;
     }
 
-    xres = gwy_data_field_get_xres(mask);
-    yres = gwy_data_field_get_yres(mask);
-    extended = gwy_data_field_extend(mask, MAX(hmove, 0), MAX(-hmove, 0), MAX(vmove, 0), MAX(-vmove, 0),
+    xres = gwy_field_get_xres(mask);
+    yres = gwy_field_get_yres(mask);
+    extended = gwy_field_extend(mask, MAX(hmove, 0), MAX(-hmove, 0), MAX(vmove, 0), MAX(-vmove, 0),
                                      exterior, fillvalue, FALSE);
-    gwy_data_field_area_copy(extended, args->result, MAX(-hmove, 0), MAX(-vmove, 0), xres, yres, 0, 0);
+    gwy_field_area_copy(extended, args->result, MAX(-hmove, 0), MAX(-vmove, 0), xres, yres, 0, 0);
 }
 
 static void
