@@ -1,5 +1,5 @@
 /*
- *  $Id: mcrop.c 26398 2024-06-20 10:08:44Z yeti-dn $
+ *  $Id: mcrop.c 28858 2025-11-13 14:32:56Z yeti-dn $
  *  Copyright (C) 2010-2024 David Necas (Yeti), Petr Klapetek, Daniil Bratashov.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net, dn2010@gmail.com
  *
@@ -16,17 +16,9 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/correlation.h>
-#include <libprocess/stats.h>
-#include <libprocess/hough.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES GWY_RUN_INTERACTIVE
@@ -56,11 +48,11 @@ typedef enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *otherfield;
-    GwyDataField *cropped;
-    GwyDataField *croppedother;
-    GwyDataField *diff;
+    GwyField *field;
+    GwyField *otherfield;
+    GwyField *cropped;
+    GwyField *croppedother;
+    GwyField *diff;
     gint xoff;
     gint yoff;
 } ModuleArgs;
@@ -70,21 +62,21 @@ typedef struct {
     GtkWidget *dialog;
     GtkWidget *dataview;
     GwyParamTable *table;
-    GwyContainer *data;
+    GwyGradient *orig_gradient;
     GwyGradient *diff_gradient;
 } ModuleGUI;
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             mcrop               (GwyContainer *data,
-                                             GwyRunType runtype);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static GwyDialogOutcome run_gui             (ModuleArgs *args,
-                                             GwyContainer *data,
+                                             GwyFile *data,
                                              gint id);
 static void             param_changed       (ModuleGUI *gui,
                                              gint id);
 static void             preview             (gpointer user_data);
-static gboolean         other_image_filter  (GwyContainer *data,
+static gboolean         other_image_filter  (GwyFile *data,
                                              gint id,
                                              gpointer user_data);
 static gboolean         execute             (ModuleArgs *args,
@@ -106,11 +98,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("mcrop",
-                              (GwyProcessFunc)&mcrop,
+                              module_main,
                               N_("/M_ultidata/Mutual C_rop..."),
-                              GWY_STOCK_MUTUAL_CROP,
+                              GWY_ICON_MUTUAL_CROP,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Crop non-intersecting regions of two images"));
 
     return TRUE;
@@ -139,7 +131,7 @@ define_module_params(void)
     gwy_param_def_set_function_name(paramdef, gwy_process_func_current());
     gwy_param_def_add_image_id(paramdef, PARAM_OTHER_IMAGE, "other_image", _("Second _image"));
     gwy_param_def_add_boolean(paramdef, PARAM_NEW_IMAGE, "new-image", _("Create new images"), TRUE);
-    gwy_param_def_add_gwyenum(paramdef, PARAM_DISPLAY, "display", gwy_sgettext("verb|_Display"),
+    gwy_param_def_add_gwyenum(paramdef, PARAM_DISPLAY, "display", C_("verb", "_Display"),
                               displays, G_N_ELEMENTS(displays), MCROP_DISPLAY_DIFFERENCE);
     gwy_param_def_add_boolean(paramdef, PARAM_DIFF_COLOURMAP, "diff_colourmap",
                               _("Show differences with _adapted color map"), TRUE);
@@ -151,27 +143,27 @@ define_module_params(void)
 }
 
 static void
-mcrop(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome;
     GwyAppDataId dataid;
-    GwyContainer *otherdata;
+    GwyFile *otherdata;
     ModuleArgs args;
     gint id, otherid, newid;
     GQuark quark, otherquark;
 
-    g_return_if_fail(runtype & RUN_MODES);
+    g_return_if_fail(mode & RUN_MODES);
 
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(args.field);
 
     args.params = gwy_params_new_from_settings(define_module_params());
-    args.cropped = gwy_data_field_new(1, 1, 1, 1, TRUE);
-    args.croppedother = gwy_data_field_new(1, 1, 1, 1, TRUE);
-    args.diff = gwy_data_field_new(1, 1, 1, 1, TRUE);
+    args.cropped = gwy_field_new(1, 1, 1, 1, TRUE);
+    args.croppedother = gwy_field_new(1, 1, 1, 1, TRUE);
+    args.diff = gwy_field_new(1, 1, 1, 1, TRUE);
 
     outcome = run_gui(&args, data, id);
     gwy_params_save_to_settings(args.params);
@@ -179,33 +171,37 @@ mcrop(GwyContainer *data, GwyRunType runtype)
         goto end;
 
     dataid = gwy_params_get_data_id(args.params, PARAM_OTHER_IMAGE);
-    otherdata = gwy_app_data_browser_get(dataid.datano);
+    otherdata = gwy_data_browser_get_file(dataid.datano);
     otherid = dataid.id;
 
     /* We may act on images from two different files.  Undo and sorting things into files are a bit complicated. */
-    if (outcome != GWY_DIALOG_HAVE_RESULT && !execute(&args, gwy_app_find_window_for_channel(data, id)))
+    if (outcome != GWY_DIALOG_HAVE_RESULT
+        && !execute(&args, gwy_data_browser_get_window_for_data(data, GWY_FILE_IMAGE, id)))
         goto end;
 
     if (gwy_params_get_boolean(args.params, PARAM_NEW_IMAGE)) {
-        newid = gwy_app_data_browser_add_data_field(args.cropped, data, TRUE);
-        gwy_app_channel_log_add_proc(data, id, newid);
+        newid = gwy_file_add_image(data, args.cropped);
+        gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+        gwy_log_add(data, GWY_FILE_IMAGE, id, newid);
 
-        newid = gwy_app_data_browser_add_data_field(args.croppedother, otherdata, TRUE);
-        gwy_app_channel_log_add_proc(otherdata, otherid, newid);
+        newid = gwy_file_add_image(otherdata, args.croppedother);
+
+        gwy_file_set_visible(otherdata, GWY_FILE_IMAGE, newid, TRUE);
+        gwy_log_add(otherdata, GWY_FILE_IMAGE, otherid, newid);
     }
     else {
-        quark = gwy_app_get_data_key_for_id(id);
-        otherquark = gwy_app_get_data_key_for_id(otherid);
+        quark = gwy_file_key_image(id);
+        otherquark = gwy_file_key_image(otherid);
         if (otherdata == data)
-            gwy_app_undo_qcheckpoint(data, quark, otherquark, 0);
+            gwy_app_undo_qcheckpoint(GWY_CONTAINER(data), quark, otherquark, 0);
         else {
-            gwy_app_undo_qcheckpoint(data, quark, 0);
-            gwy_app_undo_qcheckpoint(otherdata, otherquark, 0);
+            gwy_app_undo_qcheckpoint(GWY_CONTAINER(data), quark, 0);
+            gwy_app_undo_qcheckpoint(GWY_CONTAINER(otherdata), otherquark, 0);
         }
-        gwy_container_set_object(data, quark, args.cropped);
-        gwy_container_set_object(otherdata, otherquark, args.croppedother);
-        gwy_app_channel_log_add_proc(data, id, id);
-        gwy_app_channel_log_add_proc(otherdata, otherid, otherid);
+        gwy_file_set_image(data, id, args.cropped);
+        gwy_file_set_image(otherdata, otherid, args.croppedother);
+        gwy_log_add(data, GWY_FILE_IMAGE, id, id);
+        gwy_log_add(otherdata, GWY_FILE_IMAGE, otherid, otherid);
     }
 
 end:
@@ -216,7 +212,7 @@ end:
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     GwyDialogOutcome outcome;
     ModuleGUI gui;
@@ -225,21 +221,16 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     GwyParamTable *table;
 
     gui.args = args;
-    gui.data = gwy_container_new();
-    gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(0), args->diff);
-    gwy_app_sync_data_items(data, gui.data, id, 0, FALSE,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
-    gwy_container_set_const_string(gui.data, gwy_app_get_data_palette_key_for_id(1), FIT_GRADIENT_NAME);
-
-    gui.diff_gradient = gwy_inventory_new_item(gwy_gradients(), GWY_GRADIENT_DEFAULT, FIT_GRADIENT_NAME);
-    gwy_resource_use(GWY_RESOURCE(gui.diff_gradient));
+    gui.orig_gradient = gwy_gradients_get_gradient(gwy_file_get_palette(data, GWY_FILE_IMAGE, id));
+    gui.diff_gradient = g_object_new(GWY_TYPE_GRADIENT, "name", FIT_GRADIENT_NAME, NULL);
 
     gui.dialog = gwy_dialog_new(_("Mutual Crop"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_UPDATE, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    gui.dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
+    gui.dataview = gwy_create_preview(args->diff, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(gui.dataview), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_REAL_SQUARE);
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(gui.dataview), FALSE);
 
     table = gui.table = gwy_param_table_new(args->params);
@@ -263,9 +254,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 
     outcome = gwy_dialog_run(dialog);
 
-    gwy_resource_release(GWY_RESOURCE(gui.diff_gradient));
-    gwy_inventory_delete_item(gwy_gradients(), FIT_GRADIENT_NAME);
-    g_object_unref(gui.data);
+    g_object_unref(gui.diff_gradient);
 
     return outcome;
 }
@@ -274,36 +263,35 @@ static void
 update_display(ModuleGUI *gui)
 {
     ModuleArgs *args = gui->args;
-    GwyLayerBasic *blayer = GWY_LAYER_BASIC(gwy_data_view_get_base_layer(GWY_DATA_VIEW(gui->dataview)));
     MCropDisplayType display = gwy_params_get_enum(args->params, PARAM_DISPLAY);
     gboolean diff_colourmap = gwy_params_get_boolean(gui->args->params, PARAM_DIFF_COLOURMAP);
-    GwyLayerBasicRangeType range_type = GWY_LAYER_BASIC_RANGE_FULL;
-    GQuark quark = gwy_app_get_data_key_for_id(0);
+    GwyColorMappingType mapping = GWY_COLOR_MAPPING_FULL;
+    GwyDataView *dataview = GWY_DATA_VIEW(gui->dataview);
     gdouble min, max, dispmin, dispmax;
-    gint i = 0;
 
     if (display == MCROP_DISPLAY_CROPPED) {
-        gwy_container_set_object(gui->data, quark, args->cropped);
+        gwy_data_view_set_field(dataview, args->cropped);
     }
     else if (display == MCROP_DISPLAY_CROPPEDOTHER) {
-        gwy_container_set_object(gui->data, quark, args->croppedother);
+        gwy_data_view_set_field(dataview, args->croppedother);
     }
     else if (display == MCROP_DISPLAY_DIFFERENCE) {
-        gwy_container_set_object(gui->data, quark, args->diff);
+        gwy_data_view_set_field(dataview, args->diff);
         if (diff_colourmap) {
-            range_type = GWY_LAYER_BASIC_RANGE_FIXED;
-            gwy_data_field_get_min_max(args->diff, &min, &max);
-            gwy_data_field_get_autorange(args->diff, &dispmin, &dispmax);
+            mapping = GWY_COLOR_MAPPING_FIXED;
+            gwy_field_get_min_max(args->diff, &min, &max);
+            gwy_field_get_autorange(args->diff, &dispmin, &dispmax);
             gwy_set_gradient_for_residuals(gui->diff_gradient, min, max, &dispmin, &dispmax);
-            gwy_container_set_double_by_name(gui->data, "/0/base/min", dispmin);
-            gwy_container_set_double_by_name(gui->data, "/0/base/max", dispmax);
-            i = 1;
+            gwy_data_view_set_fixed_color_range(dataview, dispmin, dispmax);
+            gwy_data_view_set_gradient(dataview, gui->diff_gradient);
         }
         else
-            range_type = GWY_LAYER_BASIC_RANGE_AUTO;
+            mapping = GWY_COLOR_MAPPING_AUTO;
     }
-    gwy_container_set_enum(gui->data, gwy_app_get_data_range_type_key_for_id(0), range_type);
-    gwy_layer_basic_set_gradient_key(blayer, g_quark_to_string(gwy_app_get_data_palette_key_for_id(i)));
+
+    gwy_data_view_set_color_mapping(dataview, mapping);
+    if (mapping != GWY_COLOR_MAPPING_FIXED)
+        gwy_data_view_set_gradient(dataview, gui->orig_gradient);
     gwy_set_data_preview_size(GWY_DATA_VIEW(gui->dataview), PREVIEW_SIZE);
     /* Prevent the size changing wildly the moment someone touches the size adjbars. */
     gtk_widget_set_size_request(gui->dataview, PREVIEW_SIZE, PREVIEW_SIZE);
@@ -341,7 +329,7 @@ preview(gpointer user_data)
         return;
     }
 
-    gwy_data_field_data_changed(args->diff);
+    gwy_field_data_changed(args->diff);
     update_display(gui);
     s = g_strdup_printf("(%d, %d)", args->xoff, args->yoff);
     gwy_param_table_info_set_valuestr(gui->table, INFO_OFFSET, s);
@@ -351,35 +339,35 @@ preview(gpointer user_data)
 }
 
 static gboolean
-other_image_filter(GwyContainer *data, gint id, gpointer user_data)
+other_image_filter(GwyFile *data, gint id, gpointer user_data)
 {
-    GwyDataField *otherfield, *field = (GwyDataField*)user_data;
+    GwyField *otherfield, *field = (GwyField*)user_data;
 
-    if (!gwy_container_gis_object(data, gwy_app_get_data_key_for_id(id), &otherfield))
+    if (!(otherfield = gwy_file_get_image(data, id)))
         return FALSE;
     if (otherfield == field)
         return FALSE;
-    return !gwy_data_field_check_compatibility(otherfield, field,
-                                               GWY_DATA_COMPATIBILITY_MEASURE
-                                               | GWY_DATA_COMPATIBILITY_LATERAL
-                                               | GWY_DATA_COMPATIBILITY_VALUE);
+    return !gwy_field_is_incompatible(otherfield, field,
+                                      GWY_DATA_MISMATCH_MEASURE
+                                      | GWY_DATA_MISMATCH_LATERAL
+                                      | GWY_DATA_MISMATCH_VALUE);
 }
 
 static gboolean
-get_score_iteratively(GwyDataField *data_field, GwyDataField *kernel_field, GwyDataField *score,
+get_score_iteratively(GwyField *field, GwyField *kernel_field, GwyField *score,
                       GtkWindow *wait_window)
 {
     GwyComputationState *state;
     gboolean ok = FALSE;
 
-    state = gwy_data_field_correlate_init(data_field, kernel_field, score);
+    state = gwy_field_correlate_init(field, kernel_field, score);
     gwy_app_wait_start(wait_window, _("Initializing..."));
-    gwy_data_field_correlate_iteration(state);
+    gwy_field_correlate_iteration(state);
     if (!gwy_app_wait_set_message(_("Correlating...")))
         goto cancelled;
 
     do {
-        gwy_data_field_correlate_iteration(state);
+        gwy_field_correlate_iteration(state);
         if (!gwy_app_wait_set_fraction(state->fraction))
             goto cancelled;
     } while (state->state != GWY_COMPUTATION_STATE_FINISHED);
@@ -387,7 +375,7 @@ get_score_iteratively(GwyDataField *data_field, GwyDataField *kernel_field, GwyD
     ok = TRUE;
 
 cancelled:
-    gwy_data_field_correlate_finalize(state);
+    gwy_field_correlate_finalize(state);
     gwy_app_wait_finish();
 
     return ok;
@@ -397,8 +385,8 @@ static gboolean
 execute(ModuleArgs *args, GtkWindow *wait_window)
 {
     GwyParams *params = args->params;
-    GwyDataField *field1 = args->field, *field2 = args->otherfield;
-    GwyDataField *correlation_data, *correlation_kernel, *correlation_score;
+    GwyField *field1 = args->field, *field2 = args->otherfield;
+    GwyField *correlation_data, *correlation_kernel, *correlation_score;
     guint method = gwy_params_get_enum(params, PARAM_METHOD);
     gdouble kfrac = gwy_params_get_double(params, PARAM_KERNEL_FRAC);
     GdkRectangle cdata;
@@ -409,13 +397,13 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
     gdouble x, y, z;
     gboolean ok = FALSE, swapped = FALSE;
 
-    xres1 = gwy_data_field_get_xres(field1);
-    xres2 = gwy_data_field_get_xres(field2);
-    yres1 = gwy_data_field_get_yres(field1);
-    yres2 = gwy_data_field_get_yres(field2);
+    xres1 = gwy_field_get_xres(field1);
+    xres2 = gwy_field_get_xres(field2);
+    yres1 = gwy_field_get_yres(field1);
+    yres2 = gwy_field_get_yres(field2);
 
     if (xres1*yres1 < xres2*yres2) {
-        GWY_SWAP(GwyDataField*, field1, field2);
+        GWY_SWAP(GwyField*, field1, field2);
         GWY_SWAP(gint, xres1, xres2);
         GWY_SWAP(gint, yres1, yres2);
         swapped = TRUE;
@@ -432,20 +420,20 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
     kx = MAX(0, xres2/2 - kxres/2);
     ky = MAX(0, yres2/2 - kyres/2);
 
-    correlation_data = gwy_data_field_area_extract(field1, cdata.x, cdata.y, cdata.width, cdata.height);
-    correlation_kernel = gwy_data_field_area_extract(field2, kx, ky, kxres, kyres);
-    correlation_score = gwy_data_field_new_alike(correlation_data, FALSE);
+    correlation_data = gwy_field_area_extract(field1, cdata.x, cdata.y, cdata.width, cdata.height);
+    correlation_kernel = gwy_field_area_extract(field2, kx, ky, kxres, kyres);
+    correlation_score = gwy_field_new_alike(correlation_data, FALSE);
 
     if (method == MCROP_CORR_SEARCH_BRUTE_FORCE) {
         if (!get_score_iteratively(correlation_data, correlation_kernel, correlation_score, wait_window))
             goto end;
     }
     else {
-        gwy_data_field_correlation_search(correlation_data, correlation_kernel, NULL, correlation_score,
-                                          method, 0.1, GWY_EXTERIOR_BORDER_EXTEND, 0.0);
+        gwy_field_correlation_search(correlation_data, correlation_kernel, NULL, correlation_score,
+                                     method, 0.1, GWY_EXTERIOR_BORDER, 0.0);
     }
 
-    gwy_data_field_get_local_maxima_list(correlation_score, &x, &y, &z, 1, 0.0, -G_MAXDOUBLE, FALSE);
+    gwy_field_get_local_maxima_list(correlation_score, &x, &y, &z, 1, 0.0, -G_MAXDOUBLE, FALSE);
     max_col = GWY_ROUND(x);
     max_row = GWY_ROUND(y);
     gwy_debug("c: %d %d %dx%d  k: %d %d %dx%d",
@@ -469,20 +457,20 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
     gwy_debug("%d %d %d %d", x1l, y1b, x1r, y1t);
     gwy_debug("%d %d %d %d", x2l, y2b, x2r, y2t);
 
-    gwy_data_field_assign(args->cropped, field1);
-    gwy_data_field_resize(args->cropped, x1l, y1b, x1r, y1t);
+    gwy_field_assign(args->cropped, field1);
+    gwy_field_crop(args->cropped, x1l, y1b, x1r - x1l, y1t - y1b);
 
-    gwy_data_field_assign(args->croppedother, field2);
-    gwy_data_field_resize(args->croppedother, x2l, y2b, x2r, y2t);
+    gwy_field_assign(args->croppedother, field2);
+    gwy_field_crop(args->croppedother, x2l, y2b, x2r - x2l, y2t - y2b);
 
     if (swapped) {
-        GWY_SWAP(GwyDataField*, args->cropped, args->croppedother);
+        GWY_SWAP(GwyField*, args->cropped, args->croppedother);
         args->xoff = -args->xoff;
         args->yoff = -args->yoff;
     }
 
-    gwy_data_field_assign(args->diff, args->cropped);
-    gwy_data_field_subtract_fields(args->diff, args->diff, args->croppedother);
+    gwy_field_assign(args->diff, args->cropped);
+    gwy_field_subtract_fields(args->diff, args->diff, args->croppedother);
 
     ok = TRUE;
 
