@@ -1,6 +1,6 @@
 /*
- *  $Id: measure_lattice.c 25638 2023-09-07 12:23:28Z yeti-dn $
- *  Copyright (C) 2015-2023 David Necas (Yeti).
+ *  $Id: measure_lattice.c 29083 2026-01-05 17:55:55Z yeti-dn $
+ *  Copyright (C) 2015-2026 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -14,26 +14,13 @@
  *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
  *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
-
+#define DEBUG 1
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwythreads.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/stats.h>
-#include <libprocess/correct.h>
-#include <libprocess/simplefft.h>
-#include <libprocess/inttrans.h>
-#include <libgwydgets/gwydataview.h>
-#include <libgwydgets/gwylayer-basic.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_INTERACTIVE)
@@ -81,8 +68,8 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *mask;
+    GwyField *field;
+    GwyField *mask;
     /* We always keep the direct-space selection here. */
     gboolean have_xy;
     gdouble xy[4];
@@ -92,28 +79,29 @@ typedef struct {
 
 typedef struct {
     ModuleArgs *args;
+    GwyFile *data;
+    gint id;
     GtkWidget *dialog;
     GtkWidget *dataview;
     GtkWidget *value_labels[VALUE_NVALUES];
     GwyParamTable *table;
     GwyResults *results;
-    GwyContainer *data;
-    GwyDataField *acf;
-    GwyDataField *psdf;
-    GwySIValueFormat *xyvf;
-    GwySIValueFormat *phivf;
+    GwyField *acf;
+    GwyField *psdf;
+    GwyValueFormat *xyvf;
+    GwyValueFormat *phivf;
     gulong selection_id;
 } ModuleGUI;
 
 static gboolean         module_register       (void);
 static GwyParamDef*     define_module_params  (void);
-static void             measure_lattice       (GwyContainer *data,
-                                               GwyRunType runtype);
+static void             module_main           (GwyFile *data,
+                                               GwyRunModeFlags mode);
 static GwyDialogOutcome run_gui               (ModuleArgs *args,
-                                               GwyContainer *data,
+                                               GwyFile *data,
                                                gint id);
 static GwyResults*      create_results        (ModuleArgs *args,
-                                               GwyContainer *data,
+                                               GwyFile *data,
                                                gint id);
 static void             param_changed         (ModuleGUI *gui,
                                                gint id);
@@ -130,7 +118,7 @@ static void             calculate_psdf        (ModuleGUI *gui);
 static void             calculate_zoomed_field(ModuleGUI *gui);
 static void             selection_changed     (ModuleGUI *gui);
 static gboolean         transform_selection   (gdouble *xy);
-static void             center_selection      (GwyDataField *field,
+static void             center_selection      (GwyField *field,
                                                gdouble *xy,
                                                gint n,
                                                gint sign);
@@ -154,11 +142,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("measure_lattice",
-                              (GwyProcessFunc)&measure_lattice,
+                              module_main,
                               N_("/Measure _Features/_Lattice..."),
-                              GWY_STOCK_MEASURE_LATTICE,
+                              GWY_ICON_MEASURE_LATTICE,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Measure lattice"));
 
     return TRUE;
@@ -208,32 +196,31 @@ define_module_params(void)
 }
 
 static void
-measure_lattice(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome;
     GwySelection *selection;
     ModuleArgs args;
-    gchar *selkey;
     gint id;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_MASK_FIELD, &args.mask,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_MASK_FIELD, &args.mask,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(args.field);
 
     /* Replace the field with an adjusted one. Keep the average for adjusting the fixed range for preview. */
-    args.field = gwy_data_field_duplicate(args.field);
-    args.avg = gwy_data_field_get_avg(args.field);
-    gwy_data_field_add(args.field, -args.avg);
-    gwy_data_field_set_xoffset(args.field, -0.5*gwy_data_field_get_xreal(args.field));
-    gwy_data_field_set_yoffset(args.field, -0.5*gwy_data_field_get_yreal(args.field));
+    args.field = gwy_field_copy(args.field);
+    args.avg = gwy_field_get_avg(args.field);
+    gwy_field_add(args.field, -args.avg);
+    gwy_field_set_xoffset(args.field, -0.5*gwy_field_get_xreal(args.field));
+    gwy_field_set_yoffset(args.field, -0.5*gwy_field_get_yreal(args.field));
 
     /* Restore lattice from data if any is present. */
-    selkey = g_strdup_printf("/%d/select/lattice", id);
-    if (gwy_container_gis_object_by_name(data, selkey, &selection))
+    GQuark selection_key = gwy_file_key_image_selection(id, "lattice");
+    if (gwy_container_gis_object(GWY_CONTAINER(data), selection_key, &selection))
         args.have_xy = gwy_selection_get_object(selection, 0, args.xy);
 
     args.params = gwy_params_new_from_settings(define_module_params());
@@ -242,59 +229,48 @@ measure_lattice(GwyContainer *data, GwyRunType runtype)
 
     /* Save lattice to data if any is present. */
     if (args.have_xy && outcome == GWY_DIALOG_HAVE_RESULT) {
-        selection = g_object_new(g_type_from_name("GwySelectionLattice"), "max-objects", 1, NULL);
+        selection = gwy_selection_lattice_new();
         gwy_selection_set_data(selection, 1, args.xy);
-        gwy_container_pass_object_by_name(data, selkey, selection);
+        gwy_container_pass_object(GWY_CONTAINER(data), selection_key, selection);
     }
 
-    g_free(selkey);
     g_object_unref(args.params);
     g_object_unref(args.field);
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     GtkWidget *hbox;
     GwyDialog *dialog;
     GwyDialogOutcome outcome;
     GwyParamTable *table;
     ModuleGUI gui;
-    gdouble z;
-    gint i;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
+    gui.data = data;
+    gui.id = id;
     gui.results = create_results(args, data, id);
-    gui.xyvf = gwy_data_field_get_value_format_xy(args->field, GWY_SI_UNIT_FORMAT_MARKUP, NULL);
+    gui.xyvf = gwy_field_get_value_format_xy(args->field, GWY_UNIT_FORMAT_MARKUP, NULL);
     gui.xyvf->precision += 2;
 
-    gui.phivf = gwy_si_unit_value_format_new(G_PI/180.0, 2, _("deg"));
-    gui.data = gwy_container_new();
-    gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(IMAGE_DATA), args->field);
-    gui.acf = gwy_data_field_new_alike(args->field, FALSE);
-    gui.psdf = gwy_data_field_new_alike(args->field, FALSE);
-    for (i = 0; i < IMAGE_NMODES; i++)
-        gwy_app_sync_data_items(data, gui.data, id, i, FALSE, GWY_DATA_ITEM_PALETTE, GWY_DATA_ITEM_REAL_SQUARE, 0);
-    gwy_app_sync_data_items(data, gui.data, id, IMAGE_DATA, FALSE, GWY_DATA_ITEM_RANGE_TYPE, GWY_DATA_ITEM_RANGE, 0);
-    if (gwy_container_gis_double(gui.data, gwy_app_get_data_range_min_key_for_id(IMAGE_DATA), &z))
-        gwy_container_set_double(gui.data, gwy_app_get_data_range_min_key_for_id(IMAGE_DATA), z - args->avg);
-    if (gwy_container_gis_double(gui.data, gwy_app_get_data_range_max_key_for_id(IMAGE_DATA), &z))
-        gwy_container_set_double(gui.data, gwy_app_get_data_range_max_key_for_id(IMAGE_DATA), z - args->avg);
-    gwy_container_set_enum(gui.data, gwy_app_get_data_range_type_key_for_id(IMAGE_ACF), GWY_LAYER_BASIC_RANGE_AUTO);
-    gwy_container_set_enum(gui.data, gwy_app_get_data_range_type_key_for_id(IMAGE_PSDF), GWY_LAYER_BASIC_RANGE_AUTO);
-    gwy_container_set_const_string(gui.data, gwy_app_get_data_palette_key_for_id(IMAGE_PSDF), "DFit");
+    gui.phivf = gwy_value_format_new(G_PI/180.0, 2, _("deg"));
+    gui.acf = gwy_field_new_alike(args->field, FALSE);
+    gui.psdf = gwy_field_new_alike(args->field, FALSE);
 
     gui.dialog = gwy_dialog_new(_("Measure Lattice"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_RESET, 0);
-    gtk_dialog_add_button(GTK_DIALOG(dialog), gwy_sgettext("verb|_Estimate"), RESPONSE_ESTIMATE);
+    gtk_dialog_add_button(GTK_DIALOG(dialog), C_("verb", "_Estimate"), RESPONSE_ESTIMATE);
     gtk_dialog_add_button(GTK_DIALOG(dialog), _("_Refine"), RESPONSE_REFINE);
     gwy_dialog_add_buttons(dialog, GTK_RESPONSE_OK, 0);
 
-    gui.dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
+    gui.dataview = gwy_create_preview(args->field, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(gui.dataview), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_REAL_SQUARE);
     /* Just for bootstrapping.  It will be actually set up properly in the initial param_changed(). */
-    gwy_create_preview_vector_layer(GWY_DATA_VIEW(gui.dataview), 0, "Point", 2, TRUE);
+    gwy_create_preview_vector_layer(GWY_DATA_VIEW(gui.dataview), GWY_TYPE_LAYER_POINT, 2, TRUE);
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(gui.dataview), FALSE);
 
     table = gui.table = gwy_param_table_new(args->params);
@@ -320,9 +296,8 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 
     outcome = gwy_dialog_run(dialog);
 
-    gwy_si_unit_value_format_free(gui.xyvf);
-    gwy_si_unit_value_format_free(gui.phivf);
-    g_object_unref(gui.data);
+    gwy_value_format_free(gui.xyvf);
+    gwy_value_format_free(gui.phivf);
     g_object_unref(gui.results);
     g_object_unref(gui.acf);
     g_object_unref(gui.psdf);
@@ -331,7 +306,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 }
 
 static GwyResults*
-create_results(ModuleArgs *args, GwyContainer *data, gint id)
+create_results(ModuleArgs *args, GwyFile *data, gint id)
 {
     GwyResults *results = gwy_results_new();
 
@@ -360,11 +335,11 @@ create_results(ModuleArgs *args, GwyContainer *data, gint id)
 
     gwy_results_bind_formats(results, "a1x", "a1y", "a1", "a2x", "a2y", "a2", NULL);
 
-    gwy_results_set_unit(results, "x", gwy_data_field_get_si_unit_xy(args->field));
-    gwy_results_set_unit(results, "y", gwy_data_field_get_si_unit_xy(args->field));
-    gwy_results_set_unit(results, "z", gwy_data_field_get_si_unit_z(args->field));
+    gwy_results_set_unit(results, "x", gwy_field_get_unit_xy(args->field));
+    gwy_results_set_unit(results, "y", gwy_field_get_unit_xy(args->field));
+    gwy_results_set_unit(results, "z", gwy_field_get_unit_z(args->field));
     gwy_results_fill_filename(results, "file", data);
-    gwy_results_fill_channel(results, "image", data, id);
+    gwy_results_fill_data_name(results, "image", data, GWY_FILE_IMAGE, id);
 
     return results;
 }
@@ -374,7 +349,7 @@ create_xaligned_label(const gchar *markup, gdouble xalign, gint width_chars)
 {
     GtkWidget *label = gtk_label_new(NULL);
     gtk_label_set_markup(GTK_LABEL(label), markup);
-    gtk_misc_set_alignment(GTK_MISC(label), xalign, 0.5);
+    gtk_label_set_xalign(GTK_LABEL(label), xalign);
     if (width_chars > 0)
         gtk_label_set_width_chars(GTK_LABEL(label), width_chars);
     return label;
@@ -386,50 +361,43 @@ create_lattice_table(gpointer user_data)
     static const gchar *headers[] = { "x", "y", N_("length"), N_("angle") };
     static const gboolean header_translatable[] = { FALSE, FALSE, TRUE, TRUE };
     ModuleGUI *gui = (ModuleGUI*)user_data;
-    GString *str;
-    GwySIValueFormat *vf;
-    GtkWidget *label;
-    GtkTable *table;
-    guint i;
 
-    table = GTK_TABLE(gtk_table_new(4, 5, FALSE));
-    gtk_table_set_row_spacings(table, 2);
-    gtk_table_set_col_spacings(table, 6);
+    GtkGrid *grid = GTK_GRID(gtk_grid_new());
+    gtk_grid_set_row_spacing(grid, 2);
+    gtk_grid_set_column_spacing(grid, 6);
 
     /* Header row. */
-    str = g_string_new(NULL);
-    for (i = 0; i < G_N_ELEMENTS(headers); i++) {
+    GString *str = g_string_new(NULL);
+    for (guint i = 0; i < G_N_ELEMENTS(headers); i++) {
         g_string_assign(str, header_translatable[i] ? _(headers[i]) : headers[i]);
-        vf = (i == G_N_ELEMENTS(headers)-1 ? gui->phivf : gui->xyvf);
+        GwyValueFormat *vf = (i == G_N_ELEMENTS(headers)-1 ? gui->phivf : gui->xyvf);
         if (strlen(vf->units))
             g_string_append_printf(str, " [%s]", vf->units);
-        gtk_table_attach(table, create_xaligned_label(str->str, 0.5, -1), i+1, i+2, 0, 1, GTK_FILL, 0, 0, 0);
+        gtk_grid_attach(grid, create_xaligned_label(str->str, 0.5, -1), i+1, 0, 1, 1);
     }
     g_string_free(str, TRUE);
 
     /* a1 */
-    gtk_table_attach(table, create_xaligned_label("a<sub>1</sub>:", 0.0, -1),
-                     0, 1, 1, 2, GTK_EXPAND | GTK_FILL, 0, 0, 0);
-    for (i = VALUE_A1_X; i <= VALUE_PHI1; i++) {
+    GtkWidget *label;
+    gtk_grid_attach(grid, create_xaligned_label("a<sub>1</sub>:", 0.0, -1), 0, 1, 1, 1);
+    for (guint i = VALUE_A1_X; i <= VALUE_PHI1; i++) {
         gui->value_labels[i] = label = create_xaligned_label(NULL, 1.0, 8);
-        gtk_table_attach(table, label, i+1 - VALUE_A1_X, i+2 - VALUE_A1_X, 1, 2, GTK_EXPAND | GTK_FILL, 0, 0, 0);
+        gtk_grid_attach(grid, label, i+1 - VALUE_A1_X, 1, 1, 1);
     }
 
     /* a2 */
-    gtk_table_attach(table, create_xaligned_label("a<sub>2</sub>:", 0.0, -1),
-                     0, 1, 2, 3, GTK_EXPAND | GTK_FILL, 0, 0, 0);
-    for (i = VALUE_A2_X; i <= VALUE_PHI2; i++) {
+    gtk_grid_attach(grid, create_xaligned_label("a<sub>2</sub>:", 0.0, -1), 0, 2, 1, 1);
+    for (guint i = VALUE_A2_X; i <= VALUE_PHI2; i++) {
         gui->value_labels[i] = label = create_xaligned_label(NULL, 1.0, 8);
-        gtk_table_attach(table, label, i+1 - VALUE_A2_X, i+2 - VALUE_A2_X, 2, 3, GTK_EXPAND | GTK_FILL, 0, 0, 0);
+        gtk_grid_attach(grid, label, i+1 - VALUE_A2_X, 2, 1, 1);
     }
 
     /* phi */
-    gtk_table_attach(GTK_TABLE(table), create_xaligned_label("ϕ:", 1.0, -1),
-                     3, 4, 3, 4, GTK_EXPAND | GTK_FILL, 0, 0, 0);
+    gtk_grid_attach(grid, create_xaligned_label("ϕ:", 1.0, -1), 3, 3, 1, 1);
     gui->value_labels[VALUE_PHI] = label = create_xaligned_label(NULL, 1.0, 8);
-    gtk_table_attach(GTK_TABLE(table), label, 4, 5, 3, 4, GTK_EXPAND | GTK_FILL, 0, 0, 0);
+    gtk_grid_attach(grid, label, 4, 3, 1, 1);
 
-    return GTK_WIDGET(table);
+    return GTK_WIDGET(grid);
 }
 
 static void
@@ -438,9 +406,10 @@ set_selection(ModuleGUI *gui)
     ModuleArgs *args = gui->args;
     ImageMode image_mode = gwy_params_get_enum(args->params, PARAM_IMAGE_MODE);
     gboolean single_vector = gwy_params_get_boolean(args->params, PARAM_SINGLE_VECTOR);
-    GwyVectorLayer *vlayer = gwy_data_view_get_top_layer(GWY_DATA_VIEW(gui->dataview));
+    GwyDataView *dataview = GWY_DATA_VIEW(gui->dataview);
+    GwyVectorLayer *vlayer = gwy_data_view_get_interactive_layer(dataview);
     GwySelection *selection = gwy_vector_layer_ensure_selection(vlayer);
-    GwyDataField *field = gwy_container_get_object(gui->data, gwy_app_get_data_key_for_id(image_mode));
+    GwyField *field = gwy_data_view_get_field(dataview);
     gdouble xy[4];
 
     gwy_assign(xy, args->xy, 4);
@@ -449,9 +418,9 @@ set_selection(ModuleGUI *gui)
         transform_selection(xy);
 
     gwy_debug("real-space sel: (%g, %g) (%g, %g)", xy[0], xy[1], xy[2], xy[3]);
-    if (g_type_is_a(G_OBJECT_TYPE(selection), g_type_from_name("GwySelectionLattice")))
+    if (GWY_IS_SELECTION_LATTICE(selection))
         gwy_selection_set_data(selection, 1, xy);
-    else if (g_type_is_a(G_OBJECT_TYPE(selection), g_type_from_name("GwySelectionPoint"))) {
+    else if (GWY_IS_SELECTION_POINT(selection)) {
         /* Point selections have origin of real coordinates in the top left corner. */
         center_selection(field, xy, 2, 1);
         gwy_selection_set_data(selection, single_vector ? 1 : 2, xy);
@@ -464,9 +433,10 @@ get_selection(ModuleGUI *gui, gdouble *xy)
     ModuleArgs *args = gui->args;
     ImageMode image_mode = gwy_params_get_enum(args->params, PARAM_IMAGE_MODE);
     gboolean single_vector = gwy_params_get_boolean(args->params, PARAM_SINGLE_VECTOR);
-    GwyVectorLayer *vlayer = gwy_data_view_get_top_layer(GWY_DATA_VIEW(gui->dataview));
+    GwyDataView *dataview = GWY_DATA_VIEW(gui->dataview);
+    GwyVectorLayer *vlayer = gwy_data_view_get_interactive_layer(dataview);
     GwySelection *selection = gwy_vector_layer_ensure_selection(vlayer);
-    GwyDataField *field = gwy_container_get_object(gui->data, gwy_app_get_data_key_for_id(image_mode));
+    GwyField *field = gwy_data_view_get_field(dataview);
 
     if (!gwy_selection_is_full(selection))
         return FALSE;
@@ -474,7 +444,7 @@ get_selection(ModuleGUI *gui, gdouble *xy)
     gwy_selection_get_data(selection, xy);
     gwy_debug("image-space sel: (%g, %g) (%g, %g)", xy[0], xy[1], xy[2], xy[3]);
     /* Point selections have origin of real coordinates in the top left corner. */
-    if (g_type_is_a(G_OBJECT_TYPE(selection), g_type_from_name("GwySelectionPoint")))
+    if (GWY_IS_SELECTION_POINT(selection))
         center_selection(field, xy, single_vector ? 1 : 2, -1);
     /* Keep an orthogonal vector of the same length (which we do not show) in the single vector mode.  It makes life
      * easier because we can basically pretend we have a lattice then. */
@@ -494,16 +464,28 @@ switch_display(ModuleGUI *gui)
 {
     ModuleArgs *args = gui->args;
     GwyParams *params = args->params;
-    GwyPixmapLayer *player = gwy_data_view_get_base_layer(GWY_DATA_VIEW(gui->dataview));
+    GwyDataView *dataview = GWY_DATA_VIEW(gui->dataview);
     ImageMode image_mode = gwy_params_get_enum(params, PARAM_IMAGE_MODE);
 
     calculate_zoomed_field(gui);
-    g_object_set(player,
-                 "gradient-key", g_quark_to_string(gwy_app_get_data_palette_key_for_id(image_mode)),
-                 "data-key", g_quark_to_string(gwy_app_get_data_key_for_id(image_mode)),
-                 "range-type-key", g_quark_to_string(gwy_app_get_data_range_type_key_for_id(image_mode)),
-                 "min-max-key", g_quark_to_string(gwy_app_get_data_base_key_for_id(image_mode)),
-                 NULL);
+
+    if (image_mode == IMAGE_DATA) {
+        gdouble dmin, dmax;
+        gwy_setup_data_view(dataview, gui->data, GWY_FILE_IMAGE, gui->id,
+                            GWY_FILE_ITEM_COLOR_MAPPING | GWY_FILE_ITEM_RANGE);
+        gwy_data_view_get_fixed_color_range(dataview, &dmin, &dmax);
+        gwy_data_view_set_fixed_color_range(dataview, dmin - args->avg, dmax - args->avg);
+    }
+    else
+        gwy_data_view_set_color_mapping(dataview, GWY_COLOR_MAPPING_AUTO);
+
+    if (image_mode == IMAGE_DATA) {
+        gwy_data_view_set_gradient(dataview, NULL);
+        gwy_setup_data_view(dataview, gui->data, GWY_FILE_IMAGE, gui->id, GWY_FILE_ITEM_PALETTE);
+    }
+    else
+        gwy_data_view_set_gradient(dataview, gwy_gradients_get_gradient("DFit"));
+
     gwy_set_data_preview_size(GWY_DATA_VIEW(gui->dataview), PREVIEW_SIZE);
     set_selection(gui);
 }
@@ -516,20 +498,19 @@ switch_selection_mode(ModuleGUI *gui)
     gboolean show_numbers = gwy_params_get_boolean(args->params, PARAM_SHOW_NUMBERS);
     gboolean single_vector = gwy_params_get_boolean(args->params, PARAM_SINGLE_VECTOR);
     GwyDataView *dataview = GWY_DATA_VIEW(gui->dataview);
-    GwyVectorLayer *vlayer = gwy_data_view_get_top_layer(dataview);
+    GwyVectorLayer *vlayer = gwy_data_view_get_interactive_layer(dataview);
     GwySelection *selection = gwy_vector_layer_ensure_selection(vlayer);
 
-    GWY_SIGNAL_HANDLER_DISCONNECT(selection, gui->selection_id);
+    g_clear_signal_handler(&gui->selection_id, selection);
     if (sel_mode == SELECTION_LATTICE) {
-        selection = gwy_create_preview_vector_layer(dataview, 0, "Lattice", 1, TRUE);
+        selection = gwy_create_preview_vector_layer(dataview, GWY_TYPE_LAYER_LATTICE, 1, TRUE);
         gwy_param_table_set_sensitive(gui->table, PARAM_SHOW_NUMBERS, FALSE);
     }
     else {
-        selection = gwy_create_preview_vector_layer(dataview, 0, "Point", single_vector ? 1 : 2, TRUE);
-        g_object_set(gwy_data_view_get_top_layer(dataview),
-                     "draw-as-vector", TRUE,
-                     "point-numbers", show_numbers && !single_vector,
-                     NULL);
+        selection = gwy_create_preview_vector_layer(dataview, GWY_TYPE_LAYER_POINT, single_vector ? 1 : 2, TRUE);
+        vlayer = gwy_data_view_get_interactive_layer(dataview);
+        gwy_layer_point_set_draw_as_vector(GWY_LAYER_POINT(vlayer), TRUE);
+        gwy_layer_point_set_numbered(GWY_LAYER_POINT(vlayer), show_numbers && !single_vector);
         gwy_param_table_set_sensitive(gui->table, PARAM_SHOW_NUMBERS, !single_vector);
     }
     set_selection(gui);
@@ -541,20 +522,20 @@ calculate_acf(ModuleGUI *gui)
 {
     ModuleArgs *args = gui->args;
     GwyParams *params = args->params;
-    GwyDataField *acf = gui->acf, *mid, *mask = args->mask, *field = args->field;
+    GwyField *acf = gui->acf, *mid, *mask = args->mask, *field = args->field;
     GwyMaskingType masking = gwy_params_get_masking(params, PARAM_MASKING, &mask);
     gboolean fix_hacf = gwy_params_get_boolean(params, PARAM_FIX_HACF);
-    gint xres = gwy_data_field_get_xres(field), yres = gwy_data_field_get_yres(field);
+    gint xres = gwy_field_get_xres(field), yres = gwy_field_get_yres(field);
     guint acfwidth = xres/2, acfheight = yres/2;
 
-    gwy_data_field_area_2dacf_mask(field, acf, mask, masking, 0, 0, xres, yres, acfwidth, acfheight, NULL);
+    gwy_field_area_acf_2d(field, mask, masking, acf, 0, 0, xres, yres, acfwidth, acfheight, NULL);
     if (fix_hacf) {
-        mid = gwy_data_field_area_extract(acf, 0, acfheight/2-1, acfwidth, 3);
-        mask = gwy_data_field_new(acfwidth, 3, acfwidth, 3, TRUE);
-        gwy_data_field_area_fill(mask, 0, 1, acfwidth, 1, 1.0);
-        gwy_data_field_set_val(mask, acfwidth/2, 1, 0.0);
-        gwy_data_field_laplace_solve(mid, mask, -1, 1.0);
-        gwy_data_field_area_copy(mid, acf, 0, 1, acfwidth, 1, 0, acfheight/2-1);
+        mid = gwy_field_area_extract(acf, 0, acfheight/2-1, acfwidth, 3);
+        mask = gwy_field_new(acfwidth, 3, acfwidth, 3, TRUE);
+        gwy_field_area_fill(mask, NULL, GWY_MASK_IGNORE, 0, 1, acfwidth, 1, 1.0);
+        gwy_field_set_val(mask, acfwidth/2, 1, 0.0);
+        gwy_field_laplace_solve(mid, mask, -1, 1.0);
+        gwy_field_area_copy(mid, acf, 0, 1, acfwidth, 1, 0, acfheight/2-1);
         g_object_unref(mask);
         g_object_unref(mid);
     }
@@ -575,48 +556,48 @@ calculate_psdf(ModuleGUI *gui)
 {
     ModuleArgs *args = gui->args;
     GwyParams *params = args->params;
-    GwyDataField *psdf = gui->psdf, *mask = args->mask, *field = args->field;
-    GwyDataField *extmask = NULL, *extfield, *fullpsdf;
+    GwyField *psdf = gui->psdf, *mask = args->mask, *field = args->field;
+    GwyField *extmask = NULL, *extfield, *fullpsdf;
     GwyMaskingType masking = gwy_params_get_masking(params, PARAM_MASKING, &mask);
     GwyWindowingType windowing = GWY_WINDOWING_HANN;
-    gint xres = gwy_data_field_get_xres(field), yres = gwy_data_field_get_yres(field);
+    gint xres = gwy_field_get_xres(field), yres = gwy_field_get_yres(field);
     gint extxres = gwy_fft_find_nice_size(2*xres), extyres = gwy_fft_find_nice_size(2*yres);
     gint i, j, n, psdfwidth, psdfheight;
     gdouble *p;
 
-    field = gwy_data_field_duplicate(field);
-    gwy_data_field_add(field, -gwy_data_field_get_avg(field));
-    gwy_data_field_fft_window(field, windowing);
-    extfield = gwy_data_field_extend(field, 0, extxres - xres, 0, extyres - yres,
-                                     GWY_EXTERIOR_FIXED_VALUE, 0.0, FALSE);
-    GWY_OBJECT_UNREF(field);
+    field = gwy_field_copy(field);
+    gwy_field_add(field, -gwy_field_get_avg(field));
+    gwy_field_fft_window(field, windowing);
+    extfield = gwy_field_extend(field, 0, extxres - xres, 0, extyres - yres,
+                                GWY_EXTERIOR_FIXED_VALUE, 0.0, FALSE);
+    g_clear_object(&field);
     if (mask) {
-        extmask = gwy_data_field_extend(mask, 0, extxres - xres, 0, extyres - yres,
-                                        GWY_EXTERIOR_FIXED_VALUE, masking == GWY_MASK_INCLUDE ? 1.0 : 0.0, FALSE);
+        extmask = gwy_field_extend(mask, 0, extxres - xres, 0, extyres - yres,
+                                   GWY_EXTERIOR_FIXED_VALUE, masking == GWY_MASK_INCLUDE ? 1.0 : 0.0, FALSE);
     }
 
-    fullpsdf = gwy_data_field_new_alike(extfield, FALSE);
-    gwy_data_field_area_2dpsdf_mask(extfield, fullpsdf, mask, masking, 0, 0, extxres, extyres, GWY_WINDOWING_NONE, 0);
+    fullpsdf = gwy_field_new_alike(extfield, FALSE);
+    gwy_field_area_psdf_2d(extfield, mask, masking, fullpsdf, 0, 0, extxres, extyres, GWY_WINDOWING_NONE, 0);
     g_object_unref(extfield);
-    GWY_OBJECT_UNREF(extmask);
+    g_clear_object(&extmask);
 
     psdfwidth = reduce_size(extxres);
     psdfheight = reduce_size(extyres);
     i = extyres - psdfheight - (extyres - psdfheight)/2;
     j = extxres - psdfwidth - (extxres - psdfwidth)/2;
-    gwy_data_field_resample(psdf, psdfwidth, psdfheight, GWY_INTERPOLATION_NONE);
-    gwy_data_field_area_copy(fullpsdf, psdf, j, i, psdfwidth, psdfheight, 0, 0);
+    gwy_field_resize(psdf, psdfwidth, psdfheight);
+    gwy_field_area_copy(fullpsdf, psdf, j, i, psdfwidth, psdfheight, 0, 0);
     /* Switch from our usual circular frequencies to plain frequencies to avoid factors in make matrix inversion,
      * making it identical for forward and backward transformations. */
-    gwy_data_field_set_xreal(psdf, psdfwidth*gwy_data_field_get_dx(fullpsdf)/(2.0*G_PI));
-    gwy_data_field_set_yreal(psdf, psdfheight*gwy_data_field_get_dy(fullpsdf)/(2.0*G_PI));
-    gwy_data_field_set_xoffset(psdf, -0.5*gwy_data_field_get_xreal(psdf));
-    gwy_data_field_set_yoffset(psdf, -0.5*gwy_data_field_get_yreal(psdf));
+    gwy_field_set_xreal(psdf, psdfwidth*gwy_field_get_dx(fullpsdf)/(2.0*G_PI));
+    gwy_field_set_yreal(psdf, psdfheight*gwy_field_get_dy(fullpsdf)/(2.0*G_PI));
+    gwy_field_set_xoffset(psdf, -0.5*gwy_field_get_xreal(psdf));
+    gwy_field_set_yoffset(psdf, -0.5*gwy_field_get_yreal(psdf));
     g_object_unref(fullpsdf);
 
     /* We do not really care about modulus units nor its absolute scale.
      * We just have it to display the square root... */
-    p = gwy_data_field_get_data(psdf);
+    p = gwy_field_get_data(psdf);
     n = psdfwidth*psdfheight;
 #ifdef _OPENMP
 #pragma omp parallel for if (gwy_threads_are_enabled()) default(none) \
@@ -631,7 +612,7 @@ calculate_zoomed_field(ModuleGUI *gui)
 {
     ModuleArgs *args = gui->args;
     ImageMode image_mode = gwy_params_get_enum(args->params, PARAM_IMAGE_MODE);
-    GwyDataField *field, *zoomed;
+    GwyField *field, *zoomed;
     guint xres, yres, width, height, zoom;
 
     if (image_mode == IMAGE_ACF) {
@@ -642,11 +623,13 @@ calculate_zoomed_field(ModuleGUI *gui)
         zoom = gwy_params_get_enum(args->params, PARAM_ZOOM_PSDF);
         field = gui->psdf;
     }
-    else
+    else {
+        gwy_data_view_set_field(GWY_DATA_VIEW(gui->dataview), args->field);
         return;
+    }
 
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
 
     if (zoom == 1)
         zoomed = g_object_ref(field);
@@ -660,11 +643,12 @@ calculate_zoomed_field(ModuleGUI *gui)
         if (height < 17)
             height = MAX(height, MIN(17, yres));
 
-        zoomed = gwy_data_field_area_extract(field, (xres - width + 1)/2, (yres - height + 1)/2, width, height);
-        gwy_data_field_set_xoffset(zoomed, -0.5*gwy_data_field_get_xreal(zoomed));
-        gwy_data_field_set_yoffset(zoomed, -0.5*gwy_data_field_get_yreal(zoomed));
+        zoomed = gwy_field_area_extract(field, (xres - width + 1)/2, (yres - height + 1)/2, width, height);
+        gwy_field_set_xoffset(zoomed, -0.5*gwy_field_get_xreal(zoomed));
+        gwy_field_set_yoffset(zoomed, -0.5*gwy_field_get_yreal(zoomed));
     }
-    gwy_container_pass_object(gui->data, gwy_app_get_data_key_for_id(image_mode), zoomed);
+
+    gwy_data_view_set_field(GWY_DATA_VIEW(gui->dataview), zoomed);
 }
 
 static void
@@ -701,10 +685,10 @@ param_changed(ModuleGUI *gui, gint id)
     if (id < 0 || id == PARAM_SELECTION_MODE || id == PARAM_SINGLE_VECTOR)
         switch_selection_mode(gui);
     if (id < 0 || id == PARAM_SHOW_NUMBERS) {
-        GwyVectorLayer *vlayer = gwy_data_view_get_top_layer(GWY_DATA_VIEW(gui->dataview));
-        if (g_type_is_a(G_OBJECT_TYPE(vlayer), g_type_from_name("GwyLayerPoint"))) {
+        GwyVectorLayer *vlayer = gwy_data_view_get_interactive_layer(GWY_DATA_VIEW(gui->dataview));
+        if (GWY_IS_LAYER_POINT(vlayer)) {
             gboolean show_numbers = gwy_params_get_boolean(args->params, PARAM_SHOW_NUMBERS);
-            g_object_set(vlayer, "point-numbers", show_numbers && !single_vector, NULL);
+            gwy_layer_point_set_numbered(GWY_LAYER_POINT(vlayer), show_numbers && !single_vector);
         }
     }
 
@@ -720,8 +704,8 @@ preview(gpointer user_data)
     calculate_acf(gui);
     calculate_psdf(gui);
     switch_display(gui);
-    gwy_data_field_data_changed(gui->acf);
-    gwy_data_field_data_changed(gui->psdf);
+    gwy_field_data_changed(gui->acf);
+    gwy_field_data_changed(gui->psdf);
     if (!gui->args->have_xy)
         dialog_response(GWY_DIALOG(gui->dialog), RESPONSE_ESTIMATE, gui);
     /* Does not do anything really useful. */
@@ -749,9 +733,9 @@ dialog_response(G_GNUC_UNUSED GwyDialog *dialog, gint response, ModuleGUI *gui)
         gwy_clear(xy, 4);
 
     if (image_mode == IMAGE_PSDF)
-        ok = gwy_data_field_measure_lattice_psdf(gui->psdf, xy);
+        ok = gwy_field_measure_lattice_psdf(gui->psdf, xy);
     else
-        ok = gwy_data_field_measure_lattice_acf(gui->acf, xy);
+        ok = gwy_field_measure_lattice_acf(gui->acf, xy);
 
     gwy_debug("%s from %s: %s",
               (response == RESPONSE_ESTIMATE ? "estimate" : "refine"),
@@ -765,8 +749,8 @@ dialog_response(G_GNUC_UNUSED GwyDialog *dialog, gint response, ModuleGUI *gui)
     }
 
     if (response == RESPONSE_ESTIMATE) {
-        args->xy[0] = gwy_data_field_get_xreal(args->field)/20.0;
-        args->xy[3] = -gwy_data_field_get_yreal(args->field)/20.0;
+        args->xy[0] = gwy_field_get_xreal(args->field)/20.0;
+        args->xy[3] = -gwy_field_get_yreal(args->field)/20.0;
         args->xy[1] = args->xy[2] = 0.0;
         args->have_xy = FALSE;
         set_selection(gui);
@@ -775,7 +759,7 @@ dialog_response(G_GNUC_UNUSED GwyDialog *dialog, gint response, ModuleGUI *gui)
 }
 
 static void
-update_value_label(GtkWidget *label, gdouble value, GwySIValueFormat *vf, GString *str)
+update_value_label(GtkWidget *label, gdouble value, GwyValueFormat *vf, GString *str)
 {
     g_string_printf(str, "%.*f", vf->precision, value/vf->magnitude);
     gtk_label_set_text(GTK_LABEL(label), str->str);
@@ -838,10 +822,10 @@ selection_changed(ModuleGUI *gui)
 }
 
 static void
-center_selection(GwyDataField *field, gdouble *xy, gint n, gint sign)
+center_selection(GwyField *field, gdouble *xy, gint n, gint sign)
 {
-    gdouble xoff = 0.5*gwy_data_field_get_xreal(field);
-    gdouble yoff = 0.5*gwy_data_field_get_yreal(field);
+    gdouble xoff = 0.5*gwy_field_get_xreal(field);
+    gdouble yoff = 0.5*gwy_field_get_yreal(field);
     gint i;
 
     for (i = 0; i < n; i++) {
