@@ -1,5 +1,5 @@
 /*
- *  $Id: median-bg.c 27064 2024-12-19 10:25:27Z yeti-dn $
+ *  $Id: median-bg.c 28821 2025-11-06 16:00:41Z yeti-dn $
  *  Copyright (C) 2004-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,17 +16,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/datafield.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/elliptic.h>
-#include <libprocess/filters.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
@@ -38,26 +31,25 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *result;
-    GwyDataField *bg;
+    GwyField *field;
+    GwyField *result;
+    GwyField *bg;
 } ModuleArgs;
 
 typedef struct {
     ModuleArgs *args;
     GtkWidget *dialog;
     GwyParamTable *table;
-    GwyContainer *data;
 } ModuleGUI;
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             median_bg           (GwyContainer *data,
-                                             GwyRunType runtype);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static gboolean         execute             (ModuleArgs *args,
                                              GtkWindow *wait_window);
 static GwyDialogOutcome run_gui             (ModuleArgs *args,
-                                             GwyContainer *data,
+                                             GwyFile *data,
                                              gint id);
 static void             param_changed       (ModuleGUI *gui,
                                              gint id);
@@ -68,7 +60,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Subtracts background using a rank-based algorithm."),
     "Yeti <yeti@gwyddion.net>",
-    "2.1",
+    "2.0",
     "David Nečas (Yeti) & Petr Klapetek",
     "2004",
 };
@@ -79,11 +71,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("median-bg",
-                              (GwyProcessFunc)&median_bg,
+                              module_main,
                               N_("/_Level/_Median Level..."),
-                              GWY_STOCK_LEVEL_MEDIAN,
+                              GWY_ICON_LEVEL_MEDIAN,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Level data by local median subtraction"));
 
     return TRUE;
@@ -105,45 +97,46 @@ define_module_params(void)
 }
 
 static void
-median_bg(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     ModuleArgs args;
     gint id, newid;
     GQuark quark;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_DATA_FIELD_KEY, &quark,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_FIELD_KEY, &quark,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(args.field && quark);
 
-    args.result = gwy_data_field_new_alike(args.field, TRUE);
-    args.bg = gwy_data_field_new_alike(args.field, TRUE);
+    args.result = gwy_field_new_alike(args.field, TRUE);
+    args.bg = gwy_field_new_alike(args.field, TRUE);
 
     args.params = gwy_params_new_from_settings(define_module_params());
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, id);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
             goto end;
     }
-    if (outcome != GWY_DIALOG_HAVE_RESULT && !execute(&args, gwy_app_find_window_for_channel(data, id)))
+    if (outcome != GWY_DIALOG_HAVE_RESULT
+        && !execute(&args, gwy_data_browser_get_window_for_data(data, GWY_FILE_IMAGE, id)))
         goto end;
 
-    gwy_app_undo_qcheckpointv(data, 1, &quark);
-    gwy_container_set_object(data, gwy_app_get_data_key_for_id(id), args.result);
-    gwy_app_channel_log_add_proc(data, id, id);
+    gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &quark);
+    gwy_file_set_image(data, id, args.result);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, id);
 
     if (gwy_params_get_boolean(args.params, PARAM_DO_EXTRACT)) {
-        newid = gwy_app_data_browser_add_data_field(args.bg, data, TRUE);
-        gwy_app_sync_data_items(data, data, id, newid, FALSE,
-                                GWY_DATA_ITEM_GRADIENT,
-                                GWY_DATA_ITEM_REAL_SQUARE,
-                                0);
-        gwy_app_set_data_field_title(data, newid, _("Background"));
-        gwy_app_channel_log_add(data, id, newid, NULL, NULL);
+        newid = gwy_file_add_image(data, args.bg);
+        gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+        gwy_file_sync_items(data, GWY_FILE_IMAGE, id,
+                            data, GWY_FILE_IMAGE, newid,
+                            GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_REAL_SQUARE, FALSE);
+        gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _("Background"), TRUE);
+        gwy_log_add_full(data, GWY_FILE_IMAGE, id, newid, NULL, NULL);
     }
 
 end:
@@ -153,30 +146,23 @@ end:
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
-    GwyDialogOutcome outcome;
     GwyDialog *dialog;
     GwyParamTable *table;
     GtkWidget *dataview, *hbox;
     ModuleGUI gui;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
 
-    gui.data = gwy_container_new();
-    gwy_container_set_object_by_name(gui.data, "/0/data", args->result);
-    gwy_app_sync_data_items(data, gui.data, id, 0, FALSE,
-                            GWY_DATA_ITEM_PALETTE,
-                            GWY_DATA_ITEM_RANGE,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
-
     gui.dialog = gwy_dialog_new(_("Median Level"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_UPDATE, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
+    dataview = gwy_create_preview(args->result, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_RANGE | GWY_FILE_ITEM_REAL_SQUARE);
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), FALSE);
 
     table = gui.table = gwy_param_table_new(args->params);
@@ -191,11 +177,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     g_signal_connect_swapped(table, "param-changed", G_CALLBACK(param_changed), &gui);
     gwy_dialog_set_preview_func(dialog, GWY_PREVIEW_UPON_REQUEST, preview, &gui, NULL);
 
-    outcome = gwy_dialog_run(dialog);
-
-    g_object_unref(gui.data);
-
-    return outcome;
+    return gwy_dialog_run(dialog);
 }
 
 static void
@@ -212,7 +194,7 @@ preview(gpointer user_data)
     ModuleArgs *args = gui->args;
 
     if (execute(args, GTK_WINDOW(gui->dialog))) {
-        gwy_data_field_data_changed(args->result);
+        gwy_field_data_changed(args->result);
         gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
     }
 }
@@ -220,25 +202,25 @@ preview(gpointer user_data)
 static gboolean
 execute(ModuleArgs *args, GtkWindow *wait_window)
 {
-    GwyDataField *bg = args->bg, *result = args->result, *kernel;
+    GwyField *bg = args->bg, *result = args->result, *kernel;
     gint size = gwy_params_get_int(args->params, PARAM_RADIUS);
     gint xres, yres, kres = 2*size + 1, n;
     gboolean ok;
 
     gwy_app_wait_start(wait_window, _("Median-leveling..."));
 
-    kernel = gwy_data_field_new(kres, kres, 1.0, 1.0, TRUE);
-    n = gwy_data_field_elliptic_area_fill(kernel, 0, 0, kres, kres, 1.0);
-    gwy_data_field_copy(args->field, bg, FALSE);
-    xres = gwy_data_field_get_xres(result);
-    yres = gwy_data_field_get_yres(result);
-    ok = gwy_data_field_area_filter_kth_rank(bg, kernel, 0, 0, xres, yres, n/2, gwy_app_wait_set_fraction);
+    kernel = gwy_field_new(kres, kres, 1.0, 1.0, TRUE);
+    n = gwy_field_elliptic_area_fill(kernel, 0, 0, kres, kres, 1.0);
+    gwy_field_copy_data(args->field, bg);
+    xres = gwy_field_get_xres(result);
+    yres = gwy_field_get_yres(result);
+    ok = gwy_field_area_filter_kth_rank(bg, kernel, 0, 0, xres, yres, n/2, gwy_app_wait_set_fraction);
 
     gwy_app_wait_finish();
 
     g_object_unref(kernel);
     if (ok)
-        gwy_data_field_subtract_fields(result, args->field, bg);
+        gwy_field_subtract_fields(result, args->field, bg);
 
     return ok;
 }
