@@ -1,5 +1,5 @@
 /*
- *  $Id: merge.c 23666 2021-05-10 21:56:10Z yeti-dn $
+ *  $Id: merge.c 29059 2026-01-02 14:27:52Z yeti-dn $
  *  Copyright (C) 2004-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,18 +16,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/correlation.h>
-#include <libprocess/hough.h>
-#include <libprocess/stats.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 
 #define RUN_MODES GWY_RUN_INTERACTIVE
 
@@ -74,9 +66,9 @@ typedef struct {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *result;
-    GwyDataField *result_mask;
+    GwyField *field;
+    GwyField *result;
+    GwyField *result_mask;
 } ModuleArgs;
 
 typedef struct {
@@ -87,36 +79,36 @@ typedef struct {
 
 static gboolean         module_register      (void);
 static GwyParamDef*     define_module_params (void);
-static void             merge                (GwyContainer *data,
-                                              GwyRunType runtype);
+static void             module_main          (GwyFile *data,
+                                              GwyRunModeFlags mode);
 static GwyDialogOutcome run_gui              (ModuleArgs *args);
 static void             param_changed        (ModuleGUI *gui,
                                               gint id);
-static gboolean         other_image_filter   (GwyContainer *data,
+static gboolean         other_image_filter   (GwyFile *data,
                                               gint id,
                                               gpointer user_data);
 static void             execute_correlate    (ModuleArgs *args);
 static void             execute_join         (ModuleArgs *args);
 static void             execute_none         (ModuleArgs *args);
-static gdouble          get_row_difference   (GwyDataField *field1,
+static gdouble          get_row_difference   (GwyField *field1,
                                               gint col1,
                                               gint row1,
-                                              GwyDataField *field2,
+                                              GwyField *field2,
                                               gint col2,
                                               gint row2,
                                               gint width,
                                               gint height);
-static gdouble          get_column_difference(GwyDataField *field1,
+static gdouble          get_column_difference(GwyField *field1,
                                               gint col1,
                                               gint row1,
-                                              GwyDataField *field2,
+                                              GwyField *field2,
                                               gint col2,
                                               gint row2,
                                               gint width,
                                               gint height);
-static void             merge_boundary       (GwyDataField *field1,
-                                              GwyDataField *field2,
-                                              GwyDataField *result,
+static void             merge_boundary       (GwyField *field1,
+                                              GwyField *field2,
+                                              GwyField *result,
                                               GwyRectangle res_rect,
                                               GwyCoord f1_pos,
                                               GwyCoord f2_pos,
@@ -130,18 +122,18 @@ static void             create_merged_field  (ModuleArgs *args,
                                               GwyMergeDirectionType direction,
                                               gboolean create_mask,
                                               gboolean crop_to_rectangle);
-static void             put_fields           (GwyDataField *field1,
-                                              GwyDataField *field2,
-                                              GwyDataField *result,
-                                              GwyDataField *outsidemask,
+static void             put_fields           (GwyField *field1,
+                                              GwyField *field2,
+                                              GwyField *result,
+                                              GwyField *outsidemask,
                                               GwyMergeBoundaryType boundary,
                                               gint px1,
                                               gint py1,
                                               gint px2,
                                               gint py2);
-static void             crop_result          (GwyDataField *result,
-                                              GwyDataField *field1,
-                                              GwyDataField *field2,
+static void             crop_result          (GwyField *result,
+                                              GwyField *field1,
+                                              GwyField *field2,
                                               GwyOrientation orientation,
                                               gint px1,
                                               gint py1,
@@ -164,11 +156,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("merge",
-                              (GwyProcessFunc)&merge,
+                              module_main,
                               N_("/M_ultidata/_Merge..."),
-                              GWY_STOCK_MERGE,
+                              GWY_ICON_MERGE,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Merge two images"));
 
     return TRUE;
@@ -180,13 +172,13 @@ define_module_params(void)
     static const GwyEnum directions[] = {
         { N_("Up"),           GWY_MERGE_DIRECTION_UP,    },
         { N_("Down"),         GWY_MERGE_DIRECTION_DOWN,  },
-        { N_("adverb|Right"), GWY_MERGE_DIRECTION_RIGHT, },
-        { N_("adverb|Left"),  GWY_MERGE_DIRECTION_LEFT,  },
+        { gwy_NC("adverb", "Right"), GWY_MERGE_DIRECTION_RIGHT, },
+        { gwy_NC("adverb", "Left"),  GWY_MERGE_DIRECTION_LEFT,  },
     };
     static const GwyEnum modes[] = {
         { N_("Correlation"),     GWY_MERGE_MODE_CORRELATE, },
-        { N_("merge-mode|Join"), GWY_MERGE_MODE_JOIN,      },
-        { N_("merge-mode|None"), GWY_MERGE_MODE_NONE,      },
+        { gwy_NC("merge-mode", "Join"), GWY_MERGE_MODE_JOIN,      },
+        { gwy_NC("merge-mode", "None"), GWY_MERGE_MODE_NONE,      },
     };
     static const GwyEnum boundaries[] = {
         { N_("First image"),   GWY_MERGE_BOUNDARY_FIRST,       },
@@ -215,18 +207,18 @@ define_module_params(void)
 }
 
 static void
-merge(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome;
-    GwyMergeModeType mode;
+    GwyMergeModeType merge_mode;
     ModuleArgs args;
     gint id, newid;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(args.field);
 
     args.params = gwy_params_new_from_settings(define_module_params());
@@ -235,29 +227,29 @@ merge(GwyContainer *data, GwyRunType runtype)
     if (outcome == GWY_DIALOG_CANCEL)
         goto end;
 
-    mode = gwy_params_get_enum(args.params, PARAM_MODE);
-    if (mode == GWY_MERGE_MODE_NONE)
+    merge_mode = gwy_params_get_enum(args.params, PARAM_MODE);
+    if (merge_mode == GWY_MERGE_MODE_NONE)
         execute_none(&args);
-    else if (mode == GWY_MERGE_MODE_JOIN)
+    else if (merge_mode == GWY_MERGE_MODE_JOIN)
         execute_join(&args);
     else
         execute_correlate(&args);
 
-    newid = gwy_app_data_browser_add_data_field(args.result, data, TRUE);
-    gwy_app_set_data_field_title(data, newid, _("Merged images"));
-    gwy_app_sync_data_items(data, data, id, newid, FALSE,
-                            GWY_DATA_ITEM_PALETTE,
-                            GWY_DATA_ITEM_MASK_COLOR,
-                            GWY_DATA_ITEM_RANGE,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
-    if (args.result_mask && gwy_data_field_get_max(args.result_mask) > 0.0)
-        gwy_container_set_object(data, gwy_app_get_mask_key_for_id(newid), args.result_mask);
-    gwy_app_channel_log_add_proc(data, -1, newid);
+    newid = gwy_file_add_image(data, args.result);
+
+    gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+    gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _("Merged images"), TRUE);
+    gwy_file_sync_items(data, GWY_FILE_IMAGE, id,
+                        data, GWY_FILE_IMAGE, newid,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_MASK_COLOR
+                        | GWY_FILE_ITEM_RANGE | GWY_FILE_ITEM_REAL_SQUARE, FALSE);
+    if (args.result_mask && gwy_field_get_max(args.result_mask) > 0.0)
+        gwy_file_set_image_mask(data, newid, args.result_mask);
+    gwy_log_add(data, GWY_FILE_IMAGE, -1, newid);
 
 end:
-    GWY_OBJECT_UNREF(args.result);
-    GWY_OBJECT_UNREF(args.result_mask);
+    g_clear_object(&args.result);
+    g_clear_object(&args.result_mask);
     g_object_unref(args.params);
 }
 
@@ -268,7 +260,7 @@ run_gui(ModuleArgs *args)
     GwyDialog *dialog;
     GwyParamTable *table;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
 
     gui.dialog = gwy_dialog_new(_("Merge Data"));
@@ -319,27 +311,25 @@ param_changed(ModuleGUI *gui, gint id)
 }
 
 static gboolean
-other_image_filter(GwyContainer *data, gint id, gpointer user_data)
+other_image_filter(GwyFile *data, gint id, gpointer user_data)
 {
     ModuleArgs *args = (ModuleArgs*)user_data;
-    GwyDataField *field = args->field, *otherfield;
+    GwyField *field = args->field, *otherfield;
     GwyMergeModeType mode = gwy_params_get_enum(args->params, PARAM_MODE);
     GwyMergeDirectionType direction = gwy_params_get_enum(args->params, PARAM_DIRECTION);
 
-    if (!gwy_container_gis_object(data, gwy_app_get_data_key_for_id(id), &otherfield))
+    if (!(otherfield = gwy_file_get_image(data, id)))
         return FALSE;
     if (otherfield == field)
         return FALSE;
-    if (gwy_data_field_check_compatibility(field, otherfield,
-                                           GWY_DATA_COMPATIBILITY_MEASURE
-                                           | GWY_DATA_COMPATIBILITY_LATERAL
-                                           | GWY_DATA_COMPATIBILITY_VALUE))
+    if (gwy_field_is_incompatible(field, otherfield,
+                                  GWY_DATA_MISMATCH_MEASURE | GWY_DATA_MISMATCH_LATERAL | GWY_DATA_MISMATCH_VALUE))
         return FALSE;
     if (mode == GWY_MERGE_MODE_JOIN) {
         if (direction == GWY_MERGE_DIRECTION_UP || direction == GWY_MERGE_DIRECTION_DOWN)
-            return gwy_data_field_get_xres(otherfield) == gwy_data_field_get_xres(field);
+            return gwy_field_get_xres(otherfield) == gwy_field_get_xres(field);
         if (direction == GWY_MERGE_DIRECTION_LEFT || direction == GWY_MERGE_DIRECTION_RIGHT)
-            return gwy_data_field_get_yres(otherfield) == gwy_data_field_get_yres(field);
+            return gwy_field_get_yres(otherfield) == gwy_field_get_yres(field);
     }
     return TRUE;
 }
@@ -352,8 +342,8 @@ execute_correlate(ModuleArgs *args)
     GwyMergeBoundaryType boundary = gwy_params_get_enum(params, PARAM_BOUNDARY);
     gboolean create_mask = gwy_params_get_boolean(params, PARAM_CREATE_MASK);
     gboolean crop_to_rectangle = gwy_params_get_boolean(params, PARAM_CROP_TO_RECTANGLE);
-    GwyDataField *field1 = args->field, *field2 = gwy_params_get_image(params, PARAM_OHTER_IMAGE);
-    GwyDataField *correlation_data, *correlation_kernel, *correlation_score;
+    GwyField *field1 = args->field, *field2 = gwy_params_get_image(params, PARAM_OHTER_IMAGE);
+    GwyField *correlation_data, *correlation_kernel, *correlation_score;
     GwyRectangle cdata, kdata;
     gint max_col, max_row;
     gint xres1, xres2, yres1, yres2;
@@ -361,7 +351,7 @@ execute_correlate(ModuleArgs *args)
     gdouble xoff, yoff, maxscore;
 
     if ((field1->xres*field1->yres) < (field2->xres*field2->yres)) {
-        GWY_SWAP(GwyDataField*, field1, field2);
+        GWY_SWAP(GwyField*, field1, field2);
         if (direction == GWY_MERGE_DIRECTION_UP)
             direction = GWY_MERGE_DIRECTION_DOWN;
         else if (direction == GWY_MERGE_DIRECTION_DOWN)
@@ -379,10 +369,10 @@ execute_correlate(ModuleArgs *args)
             boundary = GWY_MERGE_BOUNDARY_FIRST;
     }
 
-    xres1 = gwy_data_field_get_xres(field1);
-    xres2 = gwy_data_field_get_xres(field2);
-    yres1 = gwy_data_field_get_yres(field1);
-    yres2 = gwy_data_field_get_yres(field2);
+    xres1 = gwy_field_get_xres(field1);
+    xres2 = gwy_field_get_xres(field2);
+    yres1 = gwy_field_get_yres(field1);
+    yres2 = gwy_field_get_yres(field2);
 
     /* Cut data for correlation. */
     if (direction == GWY_MERGE_DIRECTION_UP) {
@@ -429,14 +419,14 @@ execute_correlate(ModuleArgs *args)
         g_assert_not_reached();
     }
 
-    correlation_data = gwy_data_field_area_extract(field1, cdata.x, cdata.y, cdata.width, cdata.height);
-    correlation_kernel = gwy_data_field_area_extract(field2, kdata.x, kdata.y, kdata.width, kdata.height);
-    correlation_score = gwy_data_field_new_alike(correlation_data, FALSE);
+    correlation_data = gwy_field_area_extract(field1, cdata.x, cdata.y, cdata.width, cdata.height);
+    correlation_kernel = gwy_field_area_extract(field2, kdata.x, kdata.y, kdata.width, kdata.height);
+    correlation_score = gwy_field_new_alike(correlation_data, FALSE);
 
     max_col = max_row = 0;
-    gwy_data_field_correlation_search(correlation_data, correlation_kernel, NULL, correlation_score,
-                                      GWY_CORR_SEARCH_COVARIANCE_SCORE, 0.01, GWY_EXTERIOR_BORDER_EXTEND, 0.0);
-    if (gwy_data_field_get_local_maxima_list(correlation_score, &xoff, &yoff, &maxscore, 1, 0, 0.0, FALSE)) {
+    gwy_field_correlation_search(correlation_data, correlation_kernel, NULL, correlation_score,
+                                 GWY_CORR_SEARCH_COVARIANCE_SCORE, 0.01, GWY_EXTERIOR_BORDER, 0.0);
+    if (gwy_field_get_local_maxima_list(correlation_score, &xoff, &yoff, &maxscore, 1, 0, 0.0, FALSE)) {
         max_col = GWY_ROUND(xoff);
         max_row = GWY_ROUND(yoff);
     }
@@ -471,7 +461,7 @@ execute_join(ModuleArgs *args)
     GwyParams *params = args->params;
     GwyMergeDirectionType direction = gwy_params_get_enum(params, PARAM_DIRECTION);
     GwyMergeBoundaryType boundary = gwy_params_get_enum(params, PARAM_BOUNDARY);
-    GwyDataField *field1 = args->field, *field2 = gwy_params_get_image(params, PARAM_OHTER_IMAGE);
+    GwyField *field1 = args->field, *field2 = gwy_params_get_image(params, PARAM_OHTER_IMAGE);
     gint xres1, xres2, yres1, yres2;
     gint maxover, i, off = 0;
     gint px1, py1, px2, py2;
@@ -479,7 +469,7 @@ execute_join(ModuleArgs *args)
 
     /* Reduce joining to two cases. */
     if (direction == GWY_MERGE_DIRECTION_UP || direction == GWY_MERGE_DIRECTION_LEFT) {
-        GWY_SWAP(GwyDataField*, field1, field2);
+        GWY_SWAP(GwyField*, field1, field2);
 
         if (direction == GWY_MERGE_DIRECTION_UP)
             direction = GWY_MERGE_DIRECTION_DOWN;
@@ -495,10 +485,10 @@ execute_join(ModuleArgs *args)
             boundary = GWY_MERGE_BOUNDARY_FIRST;
     }
 
-    xres1 = gwy_data_field_get_xres(field1);
-    yres1 = gwy_data_field_get_yres(field1);
-    xres2 = gwy_data_field_get_xres(field2);
-    yres2 = gwy_data_field_get_yres(field2);
+    xres1 = gwy_field_get_xres(field1);
+    yres1 = gwy_field_get_yres(field1);
+    xres2 = gwy_field_get_xres(field2);
+    yres2 = gwy_field_get_yres(field2);
 
     if (direction == GWY_MERGE_DIRECTION_DOWN) {
         g_return_if_fail(xres1 == xres2);
@@ -549,14 +539,14 @@ execute_none(ModuleArgs *args)
     GwyMergeBoundaryType boundary = gwy_params_get_enum(params, PARAM_BOUNDARY);
     gboolean create_mask = gwy_params_get_boolean(params, PARAM_CREATE_MASK);
     gboolean crop_to_rectangle = gwy_params_get_boolean(params, PARAM_CROP_TO_RECTANGLE);
-    GwyDataField *field1 = args->field, *field2 = gwy_params_get_image(params, PARAM_OHTER_IMAGE);
+    GwyField *field1 = args->field, *field2 = gwy_params_get_image(params, PARAM_OHTER_IMAGE);
     gint xres1, xres2, yres1, yres2;
     gint px1, py1, px2, py2;
 
-    xres1 = gwy_data_field_get_xres(field1);
-    xres2 = gwy_data_field_get_xres(field2);
-    yres1 = gwy_data_field_get_yres(field1);
-    yres2 = gwy_data_field_get_yres(field2);
+    xres1 = gwy_field_get_xres(field1);
+    xres2 = gwy_field_get_xres(field2);
+    yres1 = gwy_field_get_yres(field1);
+    yres2 = gwy_field_get_yres(field2);
 
     if (direction == GWY_MERGE_DIRECTION_UP) {
         px1 = px2 = 0;
@@ -590,14 +580,14 @@ create_merged_field(ModuleArgs *args, gint px1, gint py1, gint px2, gint py2,
                     GwyMergeBoundaryType boundary, GwyMergeDirectionType direction,
                     gboolean create_mask, gboolean crop_to_rectangle)
 {
-    GwyDataField *field1 = args->field, *field2 = gwy_params_get_image(args->params, PARAM_OHTER_IMAGE);
-    GwyDataField *result, *outsidemask = NULL;
+    GwyField *field1 = args->field, *field2 = gwy_params_get_image(args->params, PARAM_OHTER_IMAGE);
+    GwyField *result, *outsidemask = NULL;
     gint xres1, xres2, yres1, yres2, newxres, newyres;
 
-    xres1 = gwy_data_field_get_xres(field1);
-    xres2 = gwy_data_field_get_xres(field2);
-    yres1 = gwy_data_field_get_yres(field1);
-    yres2 = gwy_data_field_get_yres(field2);
+    xres1 = gwy_field_get_xres(field1);
+    xres2 = gwy_field_get_xres(field2);
+    yres1 = gwy_field_get_yres(field1);
+    yres2 = gwy_field_get_yres(field2);
 
     gwy_debug("field1 %dx%d", xres1, yres1);
     gwy_debug("field2 %dx%d", xres2, yres2);
@@ -605,12 +595,12 @@ create_merged_field(ModuleArgs *args, gint px1, gint py1, gint px2, gint py2,
     newxres = MAX(xres1 + px1, xres2 + px2);
     newyres = MAX(yres1 + py1, yres2 + py2);
 
-    result = args->result = gwy_data_field_new_alike(field1, FALSE);
-    gwy_data_field_resample(result, newxres, newyres, GWY_INTERPOLATION_NONE);
+    result = args->result = gwy_field_new_alike(field1, FALSE);
+    gwy_field_resize(result, newxres, newyres);
 
     if (create_mask && !crop_to_rectangle) {
-        outsidemask = args->result_mask = gwy_data_field_new_alike(result, FALSE);
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(outsidemask), NULL);
+        outsidemask = args->result_mask = gwy_field_new_alike(result, FALSE);
+        gwy_unit_clear(gwy_field_get_unit_z(outsidemask));
     }
     put_fields(field1, field2, result, outsidemask, boundary, px1, py1, px2, py2);
 
@@ -624,8 +614,8 @@ create_merged_field(ModuleArgs *args, gint px1, gint py1, gint px2, gint py2,
 }
 
 static void
-put_fields(GwyDataField *field1, GwyDataField *field2,
-           GwyDataField *result, GwyDataField *outsidemask,
+put_fields(GwyField *field1, GwyField *field2,
+           GwyField *result, GwyField *outsidemask,
            GwyMergeBoundaryType boundary,
            gint px1, gint py1, gint px2, gint py2)
 {
@@ -639,26 +629,26 @@ put_fields(GwyDataField *field1, GwyDataField *field2,
     gwy_debug("result %dx%d", result->xres, result->yres);
     gwy_debug("px1: %d, py1: %d, px2: %d, py2: %d", px1, py1, px2, py2);
 
-    gwy_data_field_fill(result, fmin(gwy_data_field_get_min(field1), gwy_data_field_get_min(field2)));
+    gwy_field_fill(result, fmin(gwy_field_get_min(field1), gwy_field_get_min(field2)));
 
-    w1 = gwy_data_field_get_xres(field1);
-    h1 = gwy_data_field_get_yres(field1);
-    w2 = gwy_data_field_get_xres(field2);
-    h2 = gwy_data_field_get_yres(field2);
+    w1 = gwy_field_get_xres(field1);
+    h1 = gwy_field_get_yres(field1);
+    w2 = gwy_field_get_xres(field2);
+    h2 = gwy_field_get_yres(field2);
 
     if (boundary == GWY_MERGE_BOUNDARY_SECOND) {
-        gwy_data_field_area_copy(field1, result, 0, 0, w1, h1, px1, py1);
-        gwy_data_field_area_copy(field2, result, 0, 0, w2, h2, px2, py2);
+        gwy_field_area_copy(field1, result, 0, 0, w1, h1, px1, py1);
+        gwy_field_area_copy(field2, result, 0, 0, w2, h2, px2, py2);
     }
     else {
-        gwy_data_field_area_copy(field2, result, 0, 0, w2, h2, px2, py2);
-        gwy_data_field_area_copy(field1, result, 0, 0, w1, h1, px1, py1);
+        gwy_field_area_copy(field2, result, 0, 0, w2, h2, px2, py2);
+        gwy_field_area_copy(field1, result, 0, 0, w1, h1, px1, py1);
     }
 
     if (outsidemask) {
-        gwy_data_field_fill(outsidemask, 1.0);
-        gwy_data_field_area_clear(outsidemask, px1, py1, w1, h1);
-        gwy_data_field_area_clear(outsidemask, px2, py2, w2, h2);
+        gwy_field_fill(outsidemask, 1.0);
+        gwy_field_area_clear(outsidemask, px1, py1, w1, h1);
+        gwy_field_area_clear(outsidemask, px2, py2, w2, h2);
     }
 
     /* adjust boundary to be as smooth as possible */
@@ -695,44 +685,44 @@ put_fields(GwyDataField *field1, GwyDataField *field2,
     }
 
     /* Use the pixels sizes of field 1 -- they must be identical. */
-    xreal = result->xres * gwy_data_field_get_dx(field1);
-    yreal = result->yres * gwy_data_field_get_dy(field1);
-    gwy_data_field_set_xreal(result, xreal);
-    gwy_data_field_set_yreal(result, yreal);
+    xreal = result->xres * gwy_field_get_dx(field1);
+    yreal = result->yres * gwy_field_get_dy(field1);
+    gwy_field_set_xreal(result, xreal);
+    gwy_field_set_yreal(result, yreal);
     if (outsidemask) {
-        gwy_data_field_set_xreal(outsidemask, xreal);
-        gwy_data_field_set_yreal(outsidemask, yreal);
+        gwy_field_set_xreal(outsidemask, xreal);
+        gwy_field_set_yreal(outsidemask, yreal);
     }
 }
 
 static void
-crop_result(GwyDataField *result, GwyDataField *field1, GwyDataField *field2,
+crop_result(GwyField *result, GwyField *field1, GwyField *field2,
             GwyOrientation orientation,
             gint px1, gint py1, gint px2, gint py2)
 {
     if (orientation == GWY_ORIENTATION_HORIZONTAL) {
         gint top = MAX(MAX(py1, py2), 0);
         gint bot = MIN(MIN(field1->yres + py1, field2->yres + py2), result->yres);
-        gdouble yreal = (bot - top)*gwy_data_field_get_dy(result);
+        gdouble yreal = (bot - top)*gwy_field_get_dy(result);
         g_return_if_fail(bot > top);
-        gwy_data_field_resize(result, 0, top, result->xres, bot);
-        gwy_data_field_set_yreal(result, yreal);
+        gwy_field_crop(result, 0, top, result->xres, bot - top);
+        gwy_field_set_yreal(result, yreal);
     }
     else {
         gint left = MAX(MAX(px1, px2), 0);
         gint right = MIN(MIN(field1->xres + px1, field2->xres + px2), result->xres);
-        gdouble xreal = (right - left)*gwy_data_field_get_dx(result);
+        gdouble xreal = (right - left)*gwy_field_get_dx(result);
         g_return_if_fail(right > left);
-        gwy_data_field_resize(result, left, 0, right, result->yres);
-        gwy_data_field_set_xreal(result, xreal);
+        gwy_field_crop(result, left, 0, right - left, result->yres);
+        gwy_field_set_xreal(result, xreal);
     }
 }
 
 /* Note this is not a correlation score since we care also about absolute
  * differences and try to suppress the influence of outliers. */
 static gdouble
-get_row_difference(GwyDataField *field1, gint col1, gint row1,
-                   GwyDataField *field2, gint col2, gint row2,
+get_row_difference(GwyField *field1, gint col1, gint row1,
+                   GwyField *field2, gint col2, gint row2,
                    gint width, gint height)
 {
     gint xres1, yres1, xres2, yres2, i, j;
@@ -743,12 +733,12 @@ get_row_difference(GwyDataField *field1, gint col1, gint row1,
     g_return_val_if_fail(width > 0, G_MAXDOUBLE);
     g_return_val_if_fail(height > 0, G_MAXDOUBLE);
 
-    xres1 = gwy_data_field_get_xres(field1);
-    yres1 = gwy_data_field_get_yres(field1);
-    xres2 = gwy_data_field_get_xres(field2);
-    yres2 = gwy_data_field_get_yres(field2);
-    data1 = gwy_data_field_get_data_const(field1);
-    data2 = gwy_data_field_get_data_const(field2);
+    xres1 = gwy_field_get_xres(field1);
+    yres1 = gwy_field_get_yres(field1);
+    xres2 = gwy_field_get_xres(field2);
+    yres2 = gwy_field_get_yres(field2);
+    data1 = gwy_field_get_data_const(field1);
+    data2 = gwy_field_get_data_const(field2);
 
     g_return_val_if_fail(col1 + width <= xres1, G_MAXDOUBLE);
     g_return_val_if_fail(col2 + width <= xres2, G_MAXDOUBLE);
@@ -766,7 +756,7 @@ get_row_difference(GwyDataField *field1, gint col1, gint row1,
             d = d1[j] - d2[j];
             row[j] = d;
         }
-        d = gwy_math_median(width, row);
+        d = gwy_math_median(row, width);
         s += d*d;
     }
 
@@ -776,8 +766,8 @@ get_row_difference(GwyDataField *field1, gint col1, gint row1,
 }
 
 static gdouble
-get_column_difference(GwyDataField *field1, gint col1, gint row1,
-                      GwyDataField *field2, gint col2, gint row2,
+get_column_difference(GwyField *field1, gint col1, gint row1,
+                      GwyField *field2, gint col2, gint row2,
                       gint width, gint height)
 {
     gint xres1, yres1, xres2, yres2, i, j;
@@ -788,12 +778,12 @@ get_column_difference(GwyDataField *field1, gint col1, gint row1,
     g_return_val_if_fail(width > 0, G_MAXDOUBLE);
     g_return_val_if_fail(height > 0, G_MAXDOUBLE);
 
-    xres1 = gwy_data_field_get_xres(field1);
-    yres1 = gwy_data_field_get_yres(field1);
-    xres2 = gwy_data_field_get_xres(field2);
-    yres2 = gwy_data_field_get_yres(field2);
-    data1 = gwy_data_field_get_data_const(field1);
-    data2 = gwy_data_field_get_data_const(field2);
+    xres1 = gwy_field_get_xres(field1);
+    yres1 = gwy_field_get_yres(field1);
+    xres2 = gwy_field_get_xres(field2);
+    yres2 = gwy_field_get_yres(field2);
+    data1 = gwy_field_get_data_const(field1);
+    data2 = gwy_field_get_data_const(field2);
 
     g_return_val_if_fail(col1 + width <= xres1, G_MAXDOUBLE);
     g_return_val_if_fail(col2 + width <= xres2, G_MAXDOUBLE);
@@ -811,7 +801,7 @@ get_column_difference(GwyDataField *field1, gint col1, gint row1,
             d = d1[i*xres1] - d2[i*xres2];
             column[i] = d;
         }
-        d = gwy_math_median(height, column);
+        d = gwy_math_median(column, height);
         s += d*d;
     }
 
@@ -833,7 +823,7 @@ assign_edge(gint edgepos, gint pos1, gint pos2, gint *w1, gint *w2)
 }
 
 static void
-merge_boundary(GwyDataField *field1, GwyDataField *field2, GwyDataField *result,
+merge_boundary(GwyField *field1, GwyField *field2, GwyField *result,
                GwyRectangle res_rect, GwyCoord f1_pos, GwyCoord f2_pos,
                GwyMergeBoundaryType boundary)
 {
@@ -864,9 +854,9 @@ merge_boundary(GwyDataField *field1, GwyDataField *field2, GwyDataField *result,
     assign_edge(res_rect.height, yres1 - f1_pos.y, yres2 - f2_pos.y, &w1bot, &w2bot);
     gwy_debug("bot: %d %d", w1bot, w2bot);
 
-    d1 = gwy_data_field_get_data_const(field1);
-    d2 = gwy_data_field_get_data_const(field2);
-    d = gwy_data_field_get_data(result);
+    d1 = gwy_field_get_data_const(field1);
+    d2 = gwy_field_get_data_const(field2);
+    d = gwy_field_get_data(result);
 
     for (row = 0; row < res_rect.height; row++) {
         gint dtop = row + 1, dbot = res_rect.height - row;
