@@ -1,5 +1,5 @@
 /*
- *  $Id: mfm_recalc.c 25108 2022-10-19 12:33:15Z yeti-dn $
+ *  $Id: mfm_recalc.c 28821 2025-11-06 16:00:41Z yeti-dn $
  *  Copyright (C) 2018-2022 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,24 +16,13 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyenum.h>
-#include <libprocess/stats.h>
-#include <libprocess/inttrans.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libprocess/mfm.h>
-#include <libgwydgets/gwydgetutils.h>
-#include <libgwydgets/gwycombobox.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwydgets/gwyradiobuttons.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "preview.h"
 #include "mfmops.h"
 
-#define MFM_RECALC_RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
+#define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
 
 enum {
     PARAM_SIGNAL,
@@ -47,8 +36,8 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *result;
+    GwyField *field;
+    GwyField *result;
     /* Cached input image properties. */
 //    gint goodsize;
 } ModuleArgs;
@@ -82,33 +71,33 @@ typedef struct {
 typedef struct {
     MfmRecalcArgs *args;
     GSList *signal;
-    GtkObject *spring_constant;
-    GtkObject *quality;
-    GtkObject *base_frequency;
-    GtkObject *base_amplitude;
+    GObject *spring_constant;
+    GObject *quality;
+    GObject *base_frequency;
+    GObject *base_amplitude;
     GtkWidget *new_channel;
     GtkWidget *result;
 } MfmRecalcControls;
 */
-static gboolean         module_register     (void);
-static GwyParamDef*     define_module_params(void);
-static void             mfm_recalc          (GwyContainer *data,
-                                             GwyRunType runtype);
-static void             execute             (ModuleArgs *args);
+static gboolean     module_register     (void);
+static GwyParamDef* define_module_params(void);
+static void         module_main         (GwyFile *data,
+                                         GwyRunModeFlags mode);
+static void         execute             (ModuleArgs *args);
     static GwyDialogOutcome run_gui             (ModuleArgs *args);
 /*static gboolean mfm_recalc_dialog              (MfmRecalcArgs *args,
                                                 MfmRecalcSignal guess);
-static void     mfm_recalc_dialog_update       (MfmRecalcControls *controls);
-static void     mfm_recalc_load_args           (GwyContainer *container,
-                                                MfmRecalcArgs *args);
-static void     mfm_recalc_save_args           (GwyContainer *container,
-                                                MfmRecalcArgs *args);
-static void     mfm_recalc_sanitize_args       (MfmRecalcArgs *args);
-static void     update_sensitivity             (MfmRecalcControls *controls);
-static void     signal_changed                 (GtkToggleButton *toggle,
-                                                MfmRecalcControls *controls);
-static void     new_channel_changed            (GtkToggleButton *check,
-                                                MfmRecalcControls *controls);
+static void mfm_recalc_dialog_update(MfmRecalcControls *controls);
+static void mfm_recalc_load_args    (GwyFile *container,
+                                     MfmRecalcArgs *args);
+static void mfm_recalc_save_args    (GwyFile *container,
+                                     MfmRecalcArgs *args);
+static void mfm_recalc_sanitize_args(MfmRecalcArgs *args);
+static void update_sensitivity      (MfmRecalcControls *controls);
+static void signal_changed          (GtkToggleButton *toggle,
+                                     MfmRecalcControls *controls);
+static void new_channel_changed     (GtkToggleButton *check,
+                                     MfmRecalcControls *controls);
 static void     mfm_recalc_dialog_update_values(MfmRecalcControls *controls,
                                                 MfmRecalcArgs *args);*/
 /*
@@ -132,11 +121,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("mfm_recalc",
-                              (GwyProcessFunc)&mfm_recalc,
+                              module_main,
                               N_("/SPM M_odes/_Magnetic/_Recalculate to Force Gradient..."),
-                              GWY_STOCK_MFM_CONVERT_TO_FORCE,
-                              MFM_RECALC_RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_ICON_MFM_CONVERT_TO_FORCE,
+                              RUN_MODES,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Recalculate to force gradient"));
 
     return TRUE;
@@ -203,13 +192,13 @@ issue_warning(GtkWindow *window)
 }
 
 static void
-mfm_recalc(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
-    /*GwyDataField *dfield, *out;
+    /*GwyField *dfield, *out;
     MfmRecalcArgs args;
     gboolean ok;
     gint newid, oldid;*/
-    GwySIUnit *zunit;
+    GwyUnit *zunit;
     GQuark quark;
     MfmRecalcSignal guess;
     GwyDialogOutcome outcome;
@@ -217,36 +206,31 @@ mfm_recalc(GwyContainer *data, GwyRunType runtype)
     gint id, newid;
     gboolean new_channel;
 
-    gwy_clear(&args, 1);
-    g_return_if_fail(runtype & MFM_RECALC_RUN_MODES);
-    //mfm_recalc_load_args(gwy_app_settings_get(), &args);
-    /*gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &dfield,
-                                     GWY_APP_DATA_FIELD_ID, &oldid,
-                                     GWY_APP_DATA_FIELD_KEY, &dquark,
-                                     0);*/
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     GWY_APP_DATA_FIELD_KEY, &quark,
-                                     0);
+    gwy_clear1(args);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_FIELD_ID, &id,
+                                 GWY_APP_FIELD_KEY, &quark,
+                                 0);
 
     //g_return_if_fail(dfield);
 
     g_return_if_fail(args.field && quark);  //should && quark be here? it was not, but I thought it is strange
 
     //guess the units
-    zunit = gwy_data_field_get_si_unit_z(args.field);
-    if (gwy_si_unit_equal_string(zunit, "deg"))
+    zunit = gwy_field_get_unit_z(args.field);
+    if (gwy_unit_equal_string(zunit, "deg"))
         guess = SIGNAL_PHASE_DEG;
-    else if (gwy_si_unit_equal_string(zunit, "rad"))
+    else if (gwy_unit_equal_string(zunit, "rad"))
         guess = SIGNAL_PHASE_RAD;
-    else if (gwy_si_unit_equal_string(zunit, "Hz"))
+    else if (gwy_unit_equal_string(zunit, "Hz"))
         guess = SIGNAL_FREQUENCY;
-    else if (gwy_si_unit_equal_string(zunit, "V"))
+    else if (gwy_unit_equal_string(zunit, "V"))
         guess = SIGNAL_AMPLITUDE_V;
-    else if (gwy_si_unit_equal_string(zunit, "m"))
+    else if (gwy_unit_equal_string(zunit, "m"))
         guess = SIGNAL_AMPLITUDE_M;
     else {
-        issue_warning(gwy_app_find_window_for_channel(data, id));  //data is not in this version
+        issue_warning(gwy_data_browser_get_window_for_data(data, GWY_FILE_IMAGE, id));  //data is not in this version
         return;
     }
 
@@ -254,11 +238,7 @@ mfm_recalc(GwyContainer *data, GwyRunType runtype)
 
     gwy_params_set_enum(args.params, PARAM_SIGNAL, guess);
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
-        /*ok = mfm_recalc_dialog(&args, guess);
-        mfm_recalc_save_args(gwy_app_settings_get(), &args);
-        if (!ok)
-            return;*/
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -266,25 +246,27 @@ mfm_recalc(GwyContainer *data, GwyRunType runtype)
     }
 
     new_channel = gwy_params_get_boolean(args.params, PARAM_NEW_CHANNEL);
-    gwy_app_undo_qcheckpointv(data, 1, &quark);
+    gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &quark);
     execute(&args);
 
     if (new_channel) {
-        newid = gwy_app_data_browser_add_data_field(args.result, data, TRUE);
-        gwy_app_set_data_field_title(data, newid, _("Recalculated MFM data"));
-        gwy_app_sync_data_items(data, data, id, newid, FALSE,
-                                GWY_DATA_ITEM_PALETTE, 0);
-        gwy_app_channel_log_add_proc(data, id, newid);
+        newid = gwy_file_add_image(data, args.result);
+        gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+        gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _("Recalculated MFM data"), TRUE);
+        gwy_file_sync_items(data, GWY_FILE_IMAGE, id,
+                            data, GWY_FILE_IMAGE, newid,
+                            GWY_FILE_ITEM_PALETTE, FALSE);
+        gwy_log_add(data, GWY_FILE_IMAGE, id, newid);
         //g_object_unref(out);
     }
     else {
-        gwy_data_field_data_changed(args.result);
-        gwy_app_channel_log_add_proc(data, id, id);
+        gwy_field_data_changed(args.result);
+        gwy_log_add(data, GWY_FILE_IMAGE, id, id);
     }
 
 
 end:
-    GWY_OBJECT_UNREF(args.result);
+    g_clear_object(&args.result);
     g_object_unref(args.params);
 }
 
@@ -298,7 +280,7 @@ run_gui(ModuleArgs *args)   //add handling reset, as in older version? what abou
     ModuleGUI gui;
     gint signal;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
 
     gui.dialog = gwy_dialog_new(_("MFM Recalculate Data"));
@@ -336,6 +318,48 @@ run_gui(ModuleArgs *args)   //add handling reset, as in older version? what abou
     return gwy_dialog_run(dialog);
 }
 
+static void
+gwy_field_mfm_phase_to_force_gradient(GwyField *dfield,
+                                      gdouble spring_constant,
+                                      gdouble quality,
+                                      GwyMFMGradientType type)
+{
+    gdouble dx = gwy_field_get_dx(dfield);
+    gdouble dy = gwy_field_get_dy(dfield);
+    gdouble factor = spring_constant/(quality);
+
+    gwy_field_multiply(dfield, factor*mfm_factor(type, dx, dy));
+    gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), mfm_unit(type));
+}
+
+static void
+gwy_field_mfm_frequency_shift_to_force_gradient(GwyField *dfield,
+                                                gdouble spring_constant,
+                                                gdouble base_frequency,
+                                                GwyMFMGradientType type)
+{
+    gdouble dx = gwy_field_get_dx(dfield);
+    gdouble dy = gwy_field_get_dy(dfield);
+    gdouble factor = 2*spring_constant/(base_frequency);
+
+    gwy_field_multiply(dfield, factor*mfm_factor(type, dx, dy));
+    gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), mfm_unit(type));
+}
+
+static void
+gwy_field_mfm_amplitude_shift_to_force_gradient(GwyField *dfield,
+                                                gdouble spring_constant,
+                                                gdouble quality,
+                                                gdouble base_amplitude,
+                                                GwyMFMGradientType type)
+{
+    gdouble dx = gwy_field_get_dx(dfield);
+    gdouble dy = gwy_field_get_dy(dfield);
+    gdouble factor = 3*sqrt(3)*spring_constant/(2*base_amplitude*quality);
+
+    gwy_field_multiply(dfield, factor*mfm_factor(type, dx, dy));
+    gwy_unit_set_from_string(gwy_field_get_unit_z(dfield), mfm_unit(type));
+}
 static void
 execute(ModuleArgs *args)
 {
@@ -343,46 +367,28 @@ execute(ModuleArgs *args)
     gdouble spring_constant = gwy_params_get_double(args->params, PARAM_SPRING_CONSTANT);
     gdouble quality = gwy_params_get_double(args->params, PARAM_QUALITY);
     gdouble base_frequency = gwy_params_get_double(args->params, PARAM_BASE_FREQUENCY);
-    gdouble base_amplitude = gwy_params_get_double(args->params, PARAM_BASE_AMPLITUDE);
+    gdouble base_amplitude = 1e-9 * gwy_params_get_double(args->params, PARAM_BASE_AMPLITUDE);
     gboolean new_channel = gwy_params_get_boolean(args->params, PARAM_NEW_CHANNEL);
     gint result_type = gwy_params_get_enum(args->params, PARAM_RESULT);  //should be gint?
-    GwyDataField *field = args->field;
-
-
+    GwyField *field = args->field, *result;
 
     if (new_channel)
-        args->result = gwy_data_field_duplicate(field);
+        args->result = result = gwy_field_copy(field);
     else {
-        //gwy_app_undo_qcheckpointv(data, 1, &dquark);  //checkpoint moved to mfmrecalc
-        args->result = field;
+        //gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &dquark);  //checkpoint moved to mfmrecalc
+        args->result = result = field;
     }
 
     if (signal == SIGNAL_PHASE_DEG) {
-        gwy_data_field_mfm_phase_to_force_gradient(args->result,
-                                                   spring_constant,
-                                                   quality,
-                                                   result_type);
-        gwy_data_field_multiply(args->result, M_PI/180.0);
-    }
-    else if (signal == SIGNAL_PHASE_RAD) {
-        gwy_data_field_mfm_phase_to_force_gradient(args->result,
-                                                   spring_constant,
-                                                   quality,
-                                                   result_type);
-    }
-    else if (signal == SIGNAL_FREQUENCY) {
-        gwy_data_field_mfm_frequency_shift_to_force_gradient(args->result,
-                                                             spring_constant,
-                                                             base_frequency,
-                                                             result_type);
-    }
-    else if (signal == SIGNAL_AMPLITUDE_M) {
-        gwy_data_field_mfm_amplitude_shift_to_force_gradient(args->result,
-                                                             spring_constant,
-                                                             quality,
-                                                             base_amplitude*1e-9,
-                                                             result_type);
+        gwy_field_mfm_phase_to_force_gradient(result, spring_constant, quality, result_type);
+        gwy_field_multiply(result, M_PI/180.0);
     }
+    else if (signal == SIGNAL_PHASE_RAD)
+        gwy_field_mfm_phase_to_force_gradient(result, spring_constant, quality, result_type);
+    else if (signal == SIGNAL_FREQUENCY)
+        gwy_field_mfm_frequency_shift_to_force_gradient(result, spring_constant, base_frequency, result_type);
+    else if (signal == SIGNAL_AMPLITUDE_M)
+        gwy_field_mfm_amplitude_shift_to_force_gradient(result, spring_constant, quality, base_amplitude, result_type);
     else {
         g_assert_not_reached();
     }
