@@ -1,5 +1,5 @@
 /*
- *  $Id: mfm_shift.c 25583 2023-07-31 12:07:31Z yeti-dn $
+ *  $Id: mfm_shift.c 28821 2025-11-06 16:00:41Z yeti-dn $
  *  Copyright (C) 2017 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,21 +16,9 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/inttrans.h>
-#include <libprocess/stats.h>
-#include <libprocess/mfm.h>
-#include <libgwydgets/gwydgetutils.h>
-#include <libgwydgets/gwycombobox.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
-#include "mfmops.h"
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
@@ -42,23 +30,22 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *result;
+    GwyField *field;
+    GwyField *result;
 } ModuleArgs;
 
 typedef struct {
     ModuleArgs *args;
     GtkWidget *dialog;
     GwyParamTable *table;
-    GwyContainer *data;
 } ModuleGUI;
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             mfm_shift           (GwyContainer *data,
-                                             GwyRunType runtype);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static GwyDialogOutcome run_gui             (ModuleArgs *args,
-                                             GwyContainer *data,
+                                             GwyFile *data,
                                              gint id);
 static void             param_changed       (ModuleGUI *gui,
                                              gint id);
@@ -81,11 +68,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("mfm_shift",
-                              (GwyProcessFunc)&mfm_shift,
+                              module_main,
                               N_("/SPM M_odes/_Magnetic/_Field Shift in Z..."),
-                              GWY_STOCK_MFM_FIELD_SHIFT,
+                              GWY_ICON_MFM_FIELD_SHIFT,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Compute stray field shift for another z level"));
 
     return TRUE;
@@ -107,24 +94,24 @@ define_module_params(void)
 }
 
 static void
-mfm_shift(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome;
     ModuleArgs args;
     gint id, newid, datano;
 
-    gwy_clear(&args, 1);
-    g_return_if_fail(runtype & RUN_MODES);
+    gwy_clear1(args);
+    g_return_if_fail(mode & RUN_MODES);
 
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     GWY_APP_CONTAINER_ID, &datano,
-                                     0);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_FIELD_ID, &id,
+                                 GWY_APP_CONTAINER_ID, &datano,
+                                 0);
     g_return_if_fail(args.field);
-    args.result = gwy_data_field_new_alike(args.field, FALSE);
+    args.result = gwy_field_new_alike(args.field, FALSE);
 
     args.params = gwy_params_new_from_settings(define_module_params());
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, id);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -132,44 +119,39 @@ mfm_shift(GwyContainer *data, GwyRunType runtype)
     }
     execute(&args);
 
-    newid = gwy_app_data_browser_add_data_field(args.result, data, TRUE);
-    gwy_app_sync_data_items(data, data, id, newid, FALSE,
-                            GWY_DATA_ITEM_GRADIENT,
-                            0);
+    newid = gwy_file_add_image(data, args.result);
 
-    gwy_app_set_data_field_title(data, newid, _("Shifted field"));
-    gwy_app_channel_log_add_proc(data, id, newid);
+    gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+    gwy_file_sync_items(data, GWY_FILE_IMAGE, id,
+                        data, GWY_FILE_IMAGE, newid,
+                        GWY_FILE_ITEM_PALETTE, FALSE);
+
+    gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _("Shifted field"), TRUE);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, newid);
 
 end:
-    GWY_OBJECT_UNREF(args.result);
+    g_clear_object(&args.result);
     g_object_unref(args.params);
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
-    GwyDialogOutcome outcome;
     GtkWidget *hbox, *dataview;
     GwyDialog *dialog;
     GwyParamTable *table;
     ModuleGUI gui;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
-    gui.data = gwy_container_new();
-
-    gwy_container_set_object_by_name(gui.data, "/0/data", args->result);
-    gwy_app_sync_data_items(data, gui.data, id, 0, FALSE,
-                            GWY_DATA_ITEM_PALETTE,
-                            GWY_DATA_ITEM_RANGE,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
 
     gui.dialog = gwy_dialog_new(_("Stray Field Plane Shift"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_UPDATE, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, TRUE);
+    dataview = gwy_create_preview(args->result, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_RANGE | GWY_FILE_ITEM_REAL_SQUARE);
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), FALSE);
 
     table = gui.table = gwy_param_table_new(args->params);
@@ -183,11 +165,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     g_signal_connect_swapped(table, "param-changed", G_CALLBACK(param_changed), &gui);
     gwy_dialog_set_preview_func(dialog, GWY_PREVIEW_IMMEDIATE, preview, &gui, NULL);
 
-    outcome = gwy_dialog_run(dialog);
-
-    g_object_unref(gui.data);
-
-    return outcome;
+    return gwy_dialog_run(dialog);
 }
 
 static void
@@ -202,16 +180,16 @@ preview(gpointer user_data)
     ModuleGUI *gui = (ModuleGUI*)user_data;
 
     execute(gui->args);
-    gwy_data_field_data_changed(gui->args->result);
+    gwy_field_data_changed(gui->args->result);
 }
 
 static void
 execute(ModuleArgs *args)
 {
-    GwyDataField *field = args->field, *result = args->result;
+    GwyField *field = args->field, *result = args->result;
     gdouble height = gwy_params_get_double(args->params, PARAM_HEIGHT)*1e-9;
 
-    gwy_data_field_mfm_shift_z(field, result, height);
+    gwy_field_mfm_shift_z(field, result, height);
 }
 
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
