@@ -1,6 +1,6 @@
 /*
- *  $Id: multiprofile.c 24975 2022-08-31 12:05:41Z yeti-dn $
- *  Copyright (C) 2020-2021 David Necas (Yeti).
+ *  $Id: multiprofile.c 28906 2025-11-24 16:45:11Z yeti-dn $
+ *  Copyright (C) 2020-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -16,18 +16,9 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/stats.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libgwydgets/gwyradiobuttons.h>
-#include <libgwydgets/gwycombobox.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES GWY_RUN_INTERACTIVE
@@ -57,8 +48,8 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *mask;
+    GwyField *field;
+    GwyField *mask;
     GwyGraphModel *gmodel;
 } ModuleArgs;
 
@@ -69,7 +60,6 @@ typedef struct {
     GtkWidget *image[NARGS];
     GtkWidget *enabled[NARGS];
     GtkWidget *display[NARGS];
-    GwyContainer *data;
     GwySelection *selection;
     GtkWidget *view;
     gboolean in_update;
@@ -77,11 +67,11 @@ typedef struct {
 
 static gboolean         module_register         (void);
 static GwyParamDef*     define_module_params    (void);
-static void             multiprofile            (GwyContainer *data,
-                                                 GwyRunType runtype);
+static void             module_main             (GwyFile *data,
+                                                 GwyRunModeFlags mode);
 static void             execute                 (ModuleArgs *args);
 static GwyDialogOutcome run_gui                 (ModuleArgs *args,
-                                                 GwyContainer *data,
+                                                 GwyFile *data,
                                                  gint id);
 static GtkWidget*       create_image_table      (ModuleGUI *gui);
 static void             param_changed           (ModuleGUI *gui,
@@ -101,7 +91,7 @@ static void             image_selected          (ModuleGUI *gui,
                                                  GwyDataChooser *chooser);
 static void             display_changed         (ModuleGUI *gui,
                                                  GtkToggleButton *toggle);
-static gboolean         image_filter            (GwyContainer *data,
+static gboolean         image_filter            (GwyFile *data,
                                                  gint id,
                                                  gpointer user_data);
 
@@ -121,11 +111,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("multiprofile",
-                              (GwyProcessFunc)&multiprofile,
+                              module_main,
                               N_("/M_ultidata/_Multiprofile..."),
-                              GWY_STOCK_PROFILE_MULTIPLE,
+                              GWY_ICON_PROFILE_MULTIPLE,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Read lines from multiple images simultaneously"));
 
     return TRUE;
@@ -156,7 +146,7 @@ define_module_params(void)
     gwy_param_def_add_gwyenum(paramdef, PARAM_MODE, "mode", _("_Mode"),
                               modes, G_N_ELEMENTS(modes), MULTIPROF_MODE_PROFILES);
     gwy_param_def_add_target_graph(paramdef, PARAM_TARGET_GRAPH, "target_graph", NULL);
-    gwy_param_def_add_int(paramdef, PARAM_DISPLAY, NULL, gwy_sgettext("verb|Display"), 0, NARGS-1, 0);
+    gwy_param_def_add_int(paramdef, PARAM_DISPLAY, NULL, C_("verb", "Display"), 0, NARGS-1, 0);
     /* The strings must be static so just ‘leak’ them. */
     for (i = 0; i < NARGS; i++) {
         gwy_param_def_add_image_id(paramdef, PARAM_IMAGE_0 + i,
@@ -171,26 +161,26 @@ define_module_params(void)
 }
 
 static void
-multiprofile(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome;
     ModuleArgs args;
     GwyAppDataId dataid;
     gint id;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_MASK_FIELD, &args.mask,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_MASK_FIELD, &args.mask,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(args.field);
 
     args.gmodel = gwy_graph_model_new();
-    gwy_graph_model_set_units_from_data_field(args.gmodel, args.field, 1, 0, 0, 1);
+    gwy_graph_model_set_units_from_field(args.gmodel, args.field, 1, 0, 0, 1);
 
     args.params = gwy_params_new_from_settings(define_module_params());
     /* The first image is always the current image; it is always enabled and always displayed. */
-    dataid.datano = gwy_app_data_browser_get_number(data);
+    dataid.datano = gwy_file_get_id(data);
     dataid.id = id;
     gwy_params_set_image_id(args.params, PARAM_IMAGE_0, dataid);
     gwy_params_set_boolean(args.params, PARAM_ENABLED_0, TRUE);
@@ -211,44 +201,37 @@ end:
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     GwyDialog *dialog;
     GwyParamTable *table;
     GtkWidget *graph, *dataview, *hbox;
     ModuleGUI gui;
-    GwyDialogOutcome outcome;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
 
-    gui.data = gwy_container_new();
-    gwy_container_set_object_by_name(gui.data, "/0/data", args->field);
-    gwy_app_sync_data_items(data, gui.data, id, 0, FALSE,
-                            GWY_DATA_ITEM_PALETTE,
-                            GWY_DATA_ITEM_RANGE,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
-
     gui.dialog = gwy_dialog_new(_("Multiprofile"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    hbox = gwy_hbox_new(0);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
     gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
     gwy_dialog_add_content(dialog, hbox, TRUE, TRUE, 0);
 
-    dataview = gui.view = gwy_create_preview(gui.data, 0, PREVIEW_SMALL_SIZE, FALSE);
+    dataview = gui.view = gwy_create_preview(args->field, NULL, PREVIEW_SMALL_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_RANGE | GWY_FILE_ITEM_REAL_SQUARE);
     gtk_box_pack_start(GTK_BOX(hbox), dataview, FALSE, FALSE, 0);
-    gui.selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(dataview), 0, "Axis", 1, TRUE);
-    g_object_set(gui.selection, "orientation", GWY_ORIENTATION_HORIZONTAL, NULL);
+    gui.selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(dataview), GWY_TYPE_LAYER_AXIS, 1, TRUE);
+    gwy_selection_axis_set_orientation(GWY_SELECTION_AXIS(gui.selection), GWY_ORIENTATION_VERTICAL);
 
     graph = gwy_graph_new(args->gmodel);
     gtk_widget_set_size_request(graph, PREVIEW_SIZE, PREVIEW_SMALL_SIZE);
     gwy_graph_enable_user_input(GWY_GRAPH(graph), FALSE);
     gtk_box_pack_start(GTK_BOX(hbox), graph, TRUE, TRUE, 0);
 
-    hbox = gwy_hbox_new(20);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 20);
     gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
     gwy_dialog_add_content(dialog, hbox, FALSE, FALSE, 0);
 
@@ -282,11 +265,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     g_signal_connect_after(dialog, "response", G_CALLBACK(dialog_response), &gui);
     gwy_dialog_set_preview_func(dialog, GWY_PREVIEW_IMMEDIATE, preview, &gui, NULL);
 
-    outcome = gwy_dialog_run(dialog);
-
-    g_object_unref(gui.data);
-
-    return outcome;
+    return gwy_dialog_run(dialog);
 }
 
 static GtkWidget*
@@ -294,44 +273,44 @@ create_image_table(ModuleGUI *gui)
 {
     GwyParams *params = gui->args->params;
     GwyDataChooser *chooser;
-    GtkTable *table;
+    GtkGrid *grid;
     GtkWidget *label, *check, *button;
     GwyAppDataId dataid;
     GSList *group = NULL;
     gchar *s;
     guint i;
 
-    table = GTK_TABLE(gtk_table_new(1+NARGS, 4, FALSE));
-    gtk_table_set_row_spacings(table, 2);
-    gtk_table_set_col_spacings(table, 6);
-    gtk_container_set_border_width(GTK_CONTAINER(table), 4);
+    grid = GTK_GRID(gtk_grid_new());
+    gtk_grid_set_row_spacing(grid, 2);
+    gtk_grid_set_column_spacing(grid, 6);
+    gtk_container_set_border_width(GTK_CONTAINER(grid), 4);
 
-    gtk_table_attach(table, gwy_label_new_header(_("Images")), 0, 3, 0, 1, GTK_FILL, 0, 0, 0);
-    gtk_table_attach(table, gtk_label_new(_("Show")), 3, 4, 0, 1, GTK_FILL, 0, 0, 0);
+    gtk_grid_attach(grid, gwy_label_new_header(_("Images")), 0, 0, 3, 1);
+    gtk_grid_attach(grid, gtk_label_new(_("Show")), 3, 0, 1, 1);
 
     for (i = 0; i < NARGS; i++) {
         s = g_strdup_printf("%u", i+1);
         label = gtk_label_new(s);
         g_free(s);
-        gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
-        gtk_table_attach(table, label, 0, 1, i+1, i+2, GTK_FILL, 0, 0, 0);
+        gtk_label_set_xalign(GTK_LABEL(label), 1.0);
+        gtk_grid_attach(grid, label, 0, i+1, 1, 1);
 
         gui->enabled[i] = check = gtk_check_button_new();
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), gwy_params_get_boolean(params, PARAM_ENABLED_0 + i));
         g_object_set_data(G_OBJECT(check), "id", GUINT_TO_POINTER(i));
         /* Not showing any check box looks odd, but it needs to be always checked. */
         gtk_widget_set_sensitive(check, i);
-        gtk_table_attach(table, check, 1, 2, i+1, i+2, GTK_FILL, 0, 0, 0);
+        gtk_grid_attach(grid, check, 1, i+1, 1, 1);
 
-        gui->image[i] = gwy_data_chooser_new_channels();
+        gui->image[i] = gwy_data_chooser_new(GWY_FILE_IMAGE);
         g_object_set_data(G_OBJECT(gui->image[i]), "id", GUINT_TO_POINTER(i));
-        gtk_table_attach(table, gui->image[i], 2, 3, i+1, i+2, GTK_FILL, 0, 0, 0);
+        gtk_grid_attach(grid, gui->image[i], 2, i+1, 1, 1);
 
         gui->display[i] = button = gtk_radio_button_new(group);
         g_object_set_data(G_OBJECT(button), "id", GUINT_TO_POINTER(i));
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), !i);
         group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(button));
-        gtk_table_attach(table, button, 3, 4, i+1, i+2, GTK_FILL, 0, 0, 0);
+        gtk_grid_attach(grid, button, 3, i+1, 1, 1);
     }
 
     dataid = gwy_params_get_data_id(params, PARAM_IMAGE_0);
@@ -352,7 +331,7 @@ create_image_table(ModuleGUI *gui)
         g_signal_connect_swapped(gui->display[i], "toggled", G_CALLBACK(display_changed), gui);
     }
 
-    return GTK_WIDGET(table);
+    return GTK_WIDGET(grid);
 }
 
 static void
@@ -361,7 +340,7 @@ param_changed(ModuleGUI *gui, gint id)
     ModuleArgs *args = gui->args;
     GwyParams *params = args->params;
     GwyParamTable *table = gui->table;
-    GwySIValueFormat *vf;
+    GwyValueFormat *vf;
     guint i, j;
     gint yres, lineno;
     gboolean enabled;
@@ -377,16 +356,16 @@ param_changed(ModuleGUI *gui, gint id)
     i = gwy_params_get_int(params, PARAM_DISPLAY);
     if (id < 0) {
         args->field = gwy_params_get_image(params, PARAM_IMAGE_0);
-        args->mask = gwy_params_get_mask(params, PARAM_IMAGE_0);
+        args->mask = gwy_params_get_image_mask(params, PARAM_IMAGE_0);
 
-        yres = gwy_data_field_get_yres(args->field);
-        yreal = gwy_data_field_get_yreal(args->field);
+        yres = gwy_field_get_yres(args->field);
+        yreal = gwy_field_get_yreal(args->field);
         gwy_param_table_slider_set_factor(table, PARAM_LINENO_FRAC, fmax(yres - 1.0, 1.0));
         gwy_param_table_slider_set_steps(table, PARAM_LINENO_FRAC, 1.0/yres, 10.0/yres);
         gwy_param_table_slider_set_digits(table, PARAM_LINENO_FRAC, 0);
-        vf = gwy_data_field_get_value_format_xy(args->field, GWY_SI_UNIT_FORMAT_VFMARKUP, NULL);
+        vf = gwy_field_get_value_format_xy(args->field, GWY_UNIT_FORMAT_VFMARKUP, NULL);
         gwy_param_table_alt_set_linear(table, PARAM_LINENO_FRAC, yreal/vf->magnitude, 0.0, vf->units);
-        gwy_si_unit_value_format_free(vf);
+        gwy_value_format_free(vf);
         gwy_param_table_alt_set_field_pixel_y(table, PARAM_THICKNESS, args->field);
 
         for (j = 1; j < NARGS; j++) {
@@ -396,20 +375,20 @@ param_changed(ModuleGUI *gui, gint id)
             gwy_data_chooser_refilter(GWY_DATA_CHOOSER(gui->image[j]));
         }
 
-        gwy_graph_model_set_units_from_data_field(args->gmodel, args->field, 1, 0, 0, 1);
+        gwy_graph_model_set_units_from_field(args->gmodel, args->field, 1, 0, 0, 1);
         gwy_param_table_data_id_refilter(table, PARAM_TARGET_GRAPH);
     }
 
     if (id < 0 || id == PARAM_LINENO_FRAC) {
-        yres = gwy_data_field_get_yres(args->field);
+        yres = gwy_field_get_yres(args->field);
         lineno = GWY_ROUND(gwy_params_get_double(params, PARAM_LINENO_FRAC)*(yres - 1.0));
         lineno = CLAMP(lineno, 0, yres-1);
-        y = gwy_data_field_itor(args->field, lineno + 0.5);
+        y = gwy_field_itor(args->field, lineno + 0.5);
         gwy_selection_set_data(gui->selection, 1, &y);
     }
 
     if (id < 0 || id == PARAM_MASKING || id == PARAM_MODE) {
-        GwyDataField *mask = args->mask;
+        GwyField *mask = args->mask;
         MultiprofMode mode = gwy_params_get_enum(params, PARAM_MODE);
         GwyMaskingType masking = gwy_params_get_masking(params, PARAM_MASKING, &mask);
         gwy_param_table_set_sensitive(table, PARAM_USE_FIRST_MASK,
@@ -429,9 +408,10 @@ param_changed(ModuleGUI *gui, gint id)
         }
     }
     if (id < 0 || (id >= PARAM_IMAGE_0 && id < PARAM_IMAGE_0 + NARGS) || id == PARAM_DISPLAY) {
-        GwyDataField *field = gwy_params_get_image(params, PARAM_IMAGE_0 + i);
-        gwy_container_set_object_by_name(gui->data, "/0/data", field);
-        gwy_set_data_preview_size(GWY_DATA_VIEW(gui->view), PREVIEW_SMALL_SIZE);
+        GwyField *field = gwy_params_get_image(params, PARAM_IMAGE_0 + i);
+        GwyDataView *dataview = GWY_DATA_VIEW(gui->view);
+        gwy_data_view_set_field(dataview, field);
+        gwy_set_data_preview_size(dataview, PREVIEW_SMALL_SIZE);
     }
 
     gui->in_update = FALSE;
@@ -467,7 +447,7 @@ selection_changed(ModuleGUI *gui, G_GNUC_UNUSED gint hint, GwySelection *selecti
         return;
     if (!gwy_selection_get_object(selection, 0, &y))
         return;
-    gwy_param_table_set_double(gui->table, PARAM_LINENO_FRAC, y/gwy_data_field_get_yreal(gui->args->field));
+    gwy_param_table_set_double(gui->table, PARAM_LINENO_FRAC, y/gwy_field_get_yreal(gui->args->field));
 }
 
 static void
@@ -507,25 +487,31 @@ display_changed(ModuleGUI *gui, GtkToggleButton *toggle)
         gwy_param_table_param_changed(gui->table, PARAM_DISPLAY);
 }
 
-static GwyDataField*
+static GwyField*
 get_chosen_image(ModuleArgs *args, guint i, gboolean want_mask)
 {
     GwyParams *params = args->params;
 
     if (!gwy_params_get_boolean(params, PARAM_ENABLED_0 + i))
         return NULL;
-    return want_mask ? gwy_params_get_mask(params, PARAM_IMAGE_0 + i) : gwy_params_get_image(params, PARAM_IMAGE_0 + i);
+    return (want_mask
+            ? gwy_params_get_image_mask(params, PARAM_IMAGE_0 + i)
+            : gwy_params_get_image(params, PARAM_IMAGE_0 + i));
 }
 
 static gboolean
-image_filter(GwyContainer *data, gint id, gpointer user_data)
+image_filter(GwyFile *data, gint id, gpointer user_data)
 {
     ModuleArgs *args = (ModuleArgs*)user_data;
-    GwyDataField *otherfield, *field = args->field;
+    GwyField *otherfield, *field = args->field;
 
-    if (!gwy_container_gis_object(data, gwy_app_get_data_key_for_id(id), &otherfield))
+    if (!(otherfield = gwy_file_get_image(data, id)))
         return FALSE;
-    return !gwy_data_field_check_compatibility(field, otherfield, GWY_DATA_COMPATIBILITY_ALL);
+    return !gwy_field_is_incompatible(field, otherfield,
+                                      GWY_DATA_MISMATCH_RES
+                                      | GWY_DATA_MISMATCH_REAL
+                                      | GWY_DATA_MISMATCH_LATERAL
+                                      | GWY_DATA_MISMATCH_VALUE);
 }
 
 static void
@@ -548,7 +534,7 @@ execute(ModuleArgs *args)
 
 /* XXX: Duplicate code with tools/cprofile.c */
 static void
-extract_row_profile(GwyDataField *field, GwyDataField *mask, GwyMaskingType masking,
+extract_row_profile(GwyField *field, GwyField *mask, GwyMaskingType masking,
                     GArray *xydata, gint row, gint thickness)
 {
     gint xres, yres, ifrom, ito, i, j;
@@ -556,10 +542,10 @@ extract_row_profile(GwyDataField *field, GwyDataField *mask, GwyMaskingType mask
     gdouble dx;
     GwyXY *xy;
 
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
-    dx = gwy_data_field_get_dx(field);
-    d = gwy_data_field_get_data_const(field);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
+    dx = gwy_field_get_dx(field);
+    d = gwy_field_get_data_const(field);
 
     g_array_set_size(xydata, xres);
     xy = &g_array_index(xydata, GwyXY, 0);
@@ -571,7 +557,7 @@ extract_row_profile(GwyDataField *field, GwyDataField *mask, GwyMaskingType mask
     ito = MIN(ito, yres);
 
     if (mask && masking != GWY_MASK_IGNORE)
-        m = gwy_data_field_get_data_const(mask);
+        m = gwy_field_get_data_const(mask);
     else {
         m = NULL;
         for (j = 0; j < xres; j++)
@@ -624,14 +610,14 @@ multiprofile_do_profiles(ModuleArgs *args)
     gboolean use_first_mask = gwy_params_get_boolean(args->params, PARAM_USE_FIRST_MASK);
     GwyGraphModel *gmodel = args->gmodel;
     GwyGraphCurveModel *gcmodel;
-    GwyDataField *field, *mask = NULL;
+    GwyField *field, *mask = NULL;
     GArray *xydata;
     gint lineno, yres;
     gchar *title;
     guint i;
 
     field = get_chosen_image(args, 0, FALSE);
-    yres = gwy_data_field_get_yres(field);
+    yres = gwy_field_get_yres(field);
     lineno = GWY_ROUND(lineno_frac*(yres - 1.0));
     lineno = CLAMP(lineno, 0, yres-1);
 
@@ -663,7 +649,7 @@ multiprofile_do_profiles(ModuleArgs *args)
 }
 
 static GwyGraphCurveModel*
-add_curve(GwyGraphModel *gmodel, GwyDataLine *dline, GwyDataLine *weight,
+add_curve(GwyGraphModel *gmodel, GwyLine *dline, GwyLine *weight,
           const GwyRGBA *colour, const gchar *description,
           GArray *xydata)
 {
@@ -673,10 +659,10 @@ add_curve(GwyGraphModel *gmodel, GwyDataLine *dline, GwyDataLine *weight,
     gdouble dx;
     GwyXY xy;
 
-    res = gwy_data_line_get_res(dline);
-    dx = gwy_data_line_get_dx(dline);
-    d = gwy_data_line_get_data(dline);
-    w = gwy_data_line_get_data(weight);
+    res = gwy_line_get_res(dline);
+    dx = gwy_line_get_dx(dline);
+    d = gwy_line_get_data(dline);
+    w = gwy_line_get_data(weight);
 
     g_array_set_size(xydata, 0);
     for (i = 0; i < res; i++) {
@@ -711,8 +697,8 @@ multiprofile_do_stats(ModuleArgs *args)
     GwyMaskingType masking = gwy_params_get_masking(args->params, PARAM_MASKING, NULL);
     MultiprofMode mode = gwy_params_get_enum(args->params, PARAM_MODE);
     GwyGraphModel *gmodel = args->gmodel;
-    GwyDataField *field, *mask, *bigfield, *bigmask = NULL;
-    GwyDataLine *avg, *aux1, *aux2, *weight;
+    GwyField *field, *mask, *bigfield, *bigmask = NULL;
+    GwyLine *avg, *aux1, *aux2, *weight;
     GArray *xydata;
     gint lineno, xres, yres, ifrom, ito, blockheight;
     gdouble dx;
@@ -721,9 +707,9 @@ multiprofile_do_stats(ModuleArgs *args)
     gwy_graph_model_remove_all_curves(gmodel);
 
     field = get_chosen_image(args, 0, FALSE);
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
-    dx = gwy_data_field_get_dx(field);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
+    dx = gwy_field_get_dx(field);
     lineno = GWY_ROUND(lineno_frac*(yres - 1.0));
     lineno = CLAMP(lineno, 0, yres-1);
 
@@ -740,56 +726,56 @@ multiprofile_do_stats(ModuleArgs *args)
     }
     g_return_if_fail(nimages);
 
-    bigfield = gwy_data_field_new(xres, nimages*blockheight, dx*xres, 1.0, FALSE);
+    bigfield = gwy_field_new(xres, nimages*blockheight, dx*xres, 1.0, FALSE);
     if (masking != GWY_MASK_IGNORE)
-        bigmask = gwy_data_field_new_alike(bigfield, FALSE);
+        bigmask = gwy_field_new_alike(bigfield, FALSE);
 
     nimages = 0;
     for (i = 0; i < NARGS; i++) {
         if (!(field = get_chosen_image(args, i, FALSE)))
             continue;
 
-        gwy_data_field_area_copy(field, bigfield, 0, ifrom, xres, blockheight, 0, nimages*blockheight);
+        gwy_field_area_copy(field, bigfield, 0, ifrom, xres, blockheight, 0, nimages*blockheight);
         if (bigmask) {
             mask = get_chosen_image(args, i, TRUE);
             if (mask) {
-                gwy_data_field_area_copy(mask, bigmask, 0, ifrom, xres, blockheight, 0, nimages*blockheight);
+                gwy_field_area_copy(mask, bigmask, 0, ifrom, xres, blockheight, 0, nimages*blockheight);
             }
             else {
-                gwy_data_field_area_fill(bigmask, 0, nimages*blockheight, xres, blockheight,
-                                         masking == GWY_MASK_INCLUDE);
+                gwy_field_area_fill(bigmask, NULL, GWY_MASK_IGNORE, 0, nimages*blockheight, xres, blockheight,
+                                    masking == GWY_MASK_INCLUDE);
             }
         }
         nimages++;
     }
 
     xydata = g_array_new(FALSE, FALSE, sizeof(GwyXY));
-    avg = gwy_data_line_new(1, 1.0, FALSE);
-    weight = gwy_data_line_new(1, 1.0, FALSE);
+    avg = gwy_line_new(1, 1.0, FALSE);
+    weight = gwy_line_new(1, 1.0, FALSE);
 
-    gwy_data_field_get_line_stats_mask(bigfield, bigmask, masking, avg, weight,
-                                       0, 0, xres, nimages*blockheight,
-                                       GWY_LINE_STAT_MEAN, GWY_ORIENTATION_VERTICAL);
+    gwy_field_area_get_line_stats(bigfield, bigmask, masking, avg, weight,
+                                  0, 0, xres, nimages*blockheight,
+                                  GWY_LINE_STAT_MEAN, GWY_ORIENTATION_VERTICAL);
     add_curve(gmodel, avg, weight, gwy_graph_get_preset_color(0), _("Mean"), xydata);
 
-    aux1 = gwy_data_line_new_alike(avg, FALSE);
-    aux2 = gwy_data_line_new_alike(avg, FALSE);
+    aux1 = gwy_line_new_alike(avg, FALSE);
+    aux2 = gwy_line_new_alike(avg, FALSE);
     if (mode == MULTIPROF_MODE_MEAN_RMS) {
-        gwy_data_field_get_line_stats_mask(bigfield, bigmask, masking, aux1, NULL,
-                                           0, 0, xres, nimages*blockheight,
-                                           GWY_LINE_STAT_RMS, GWY_ORIENTATION_VERTICAL);
-        gwy_data_line_subtract_lines(aux2, avg, aux1);
+        gwy_field_area_get_line_stats(bigfield, bigmask, masking, aux1, NULL,
+                                      0, 0, xres, nimages*blockheight,
+                                      GWY_LINE_STAT_RMS, GWY_ORIENTATION_VERTICAL);
+        gwy_line_subtract_lines(aux2, avg, aux1);
         add_curve(gmodel, aux2, weight, &lower_colour, _("Lower"), xydata);
-        gwy_data_line_sum_lines(aux2, avg, aux1);
+        gwy_line_sum_lines(aux2, avg, aux1);
         add_curve(gmodel, aux2, weight, &upper_colour, _("Upper"), xydata);
     }
     else {
-        gwy_data_field_get_line_stats_mask(bigfield, bigmask, masking, aux1, NULL,
-                                           0, 0, xres, nimages*blockheight,
-                                           GWY_LINE_STAT_MINIMUM, GWY_ORIENTATION_VERTICAL);
-        gwy_data_field_get_line_stats_mask(bigfield, bigmask, masking, aux2, NULL,
-                                           0, 0, xres, nimages*blockheight,
-                                           GWY_LINE_STAT_MAXIMUM, GWY_ORIENTATION_VERTICAL);
+        gwy_field_area_get_line_stats(bigfield, bigmask, masking, aux1, NULL,
+                                      0, 0, xres, nimages*blockheight,
+                                      GWY_LINE_STAT_MINIMUM, GWY_ORIENTATION_VERTICAL);
+        gwy_field_area_get_line_stats(bigfield, bigmask, masking, aux2, NULL,
+                                      0, 0, xres, nimages*blockheight,
+                                      GWY_LINE_STAT_MAXIMUM, GWY_ORIENTATION_VERTICAL);
         add_curve(gmodel, aux1, weight, &lower_colour, _("Lower"), xydata);
         add_curve(gmodel, aux2, weight, &upper_colour, _("Upper"), xydata);
     }
@@ -799,7 +785,7 @@ multiprofile_do_stats(ModuleArgs *args)
     g_object_unref(aux2);
     g_object_unref(weight);
     g_object_unref(bigfield);
-    GWY_OBJECT_UNREF(bigmask);
+    g_clear_object(&bigmask);
     g_array_free(xydata, TRUE);
 }
 
