@@ -1,5 +1,5 @@
 /*
- *  $Id: neural.c 28641 2025-10-12 07:13:26Z klapetek $
+ *  $Id: neural.c 28906 2025-11-24 16:45:11Z yeti-dn $
  *  Copyright (C) 2012-2024 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,30 +16,15 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyexpr.h>
-#include <libprocess/datafield.h>
-#include <libprocess/filters.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/stats.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwydgets/gwyradiobuttons.h>
-#include <libgwydgets/gwycombobox.h>
-#include <libgwydgets/gwygraph.h>
-#include <libgwydgets/gwyinventorystore.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
+#include <gwy.h>
 #include "neuraldata.h"
 #include "preview.h"
 
-#define NEURAL_TRAIN_RUN_MODES GWY_RUN_INTERACTIVE
-#define NEURAL_APPLY_RUN_MODES GWY_RUN_INTERACTIVE
+#define RUN_MODES GWY_RUN_INTERACTIVE
 
 enum {
     RESPONSE_TRAIN = 1000,
@@ -97,7 +82,8 @@ typedef struct {
     gboolean loading_network;
     GwyParamTable *table_training;
     GwyParamTable *table_network;
-    GwyContainer *data;
+    GwyField *result;
+    GwyField *diff;
     GtkWidget *dialog;
     GtkWidget *dataview;
     GwyGraphModel *gmodel;
@@ -113,8 +99,8 @@ typedef struct {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *result;
+    GwyField *field;
+    GwyField *result;
     /* This is the actual resource. */
     GwyNeuralNetwork *nn;
 } NeuralApplyArgs;
@@ -128,14 +114,14 @@ typedef struct {
 } NeuralApplyGUI;
 
 static gboolean          module_register            (void);
-static void              neural_train               (GwyContainer *data,
-                                                     GwyRunType run);
-static void              neural_apply               (GwyContainer *data,
-                                                     GwyRunType run);
+static void              neural_train               (GwyFile *data,
+                                                     GwyRunModeFlags mode);
+static void              neural_apply               (GwyFile *data,
+                                                     GwyRunModeFlags mode);
 static GwyDialogOutcome  run_gui_train              (NeuralTrainArgs *args);
 static GwyDialogOutcome  run_gui_apply              (NeuralApplyArgs *args);
 static gboolean          execute_apply              (NeuralApplyArgs *args,
-                                                     GwyContainer *data,
+                                                     GwyFile *data,
                                                      gint id);
 static gboolean          can_select_network         (GtkTreeSelection *selection,
                                                      GtkTreeModel *model,
@@ -150,8 +136,7 @@ static GtkWidget*        training_tab_new           (NeuralTrainGUI *gui);
 static GtkWidget*        parameters_tab_new         (NeuralTrainGUI *gui);
 static GtkWidget*        networks_tab_new           (NeuralTrainGUI *gui);
 static void              train_network              (NeuralTrainGUI *gui);
-static void              set_layer_channel          (GwyPixmapLayer *layer,
-                                                     gint channel);
+static void              switch_preview             (NeuralTrainGUI *gui);
 static void              network_load               (NeuralTrainGUI *gui);
 static void              network_store              (NeuralTrainGUI *gui);
 static void              network_delete             (NeuralTrainGUI *gui);
@@ -194,18 +179,18 @@ module_register(void)
     }
 
     gwy_process_func_register("neural_train",
-                              (GwyProcessFunc)&neural_train,
+                              &neural_train,
                               N_("/M_ultidata/Neural Network _Training..."),
-                              GWY_STOCK_NEURAL_TRAIN,
-                              NEURAL_TRAIN_RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_ICON_NEURAL_TRAIN,
+                              RUN_MODES,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Train a neural network for image processing"));
     gwy_process_func_register("neural_apply",
-                              (GwyProcessFunc)&neural_apply,
+                              &neural_apply,
                               N_("/M_ultidata/Apply _Neural Network..."),
-                              GWY_STOCK_NEURAL_APPLY,
-                              NEURAL_APPLY_RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_ICON_NEURAL_APPLY,
+                              RUN_MODES,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Process data using a trained neural network"));
 
     return TRUE;
@@ -266,28 +251,28 @@ define_module_params_apply(void)
 }
 
 static void
-neural_train(G_GNUC_UNUSED GwyContainer *data, GwyRunType run)
+neural_train(G_GNUC_UNUSED GwyFile *data, GwyRunModeFlags mode)
 {
     NeuralTrainArgs args;
-    GwyDataField *field, *signalfield;
+    GwyField *field, *signalfield;
     GwyParams *params;
     GwyAppDataId dataid;
 
-    g_return_if_fail(run & NEURAL_TRAIN_RUN_MODES);
-    gwy_clear(&args, 1);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_clear1(args);
 
     args.params = params = gwy_params_new_from_settings(define_module_params_train());
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD_ID, &dataid.id,
-                                     GWY_APP_DATA_FIELD, &field,
-                                     GWY_APP_CONTAINER_ID, &dataid.datano,
-                                     0);
+    gwy_data_browser_get_current(GWY_APP_FIELD_ID, &dataid.id,
+                                 GWY_APP_FIELD, &field,
+                                 GWY_APP_CONTAINER_ID, &dataid.datano,
+                                 0);
     gwy_params_set_image_id(params, PARAM_MODEL, dataid);
     signalfield = gwy_params_get_image(params, PARAM_SIGNAL);
     /* Init signal to a field compatible with model. */
-    if (!signalfield || gwy_data_field_check_compatibility(field, signalfield,
-                                                           GWY_DATA_COMPATIBILITY_RES
-                                                           | GWY_DATA_COMPATIBILITY_REAL
-                                                           | GWY_DATA_COMPATIBILITY_LATERAL))
+    if (!signalfield || gwy_field_is_incompatible(field, signalfield,
+                                                  GWY_DATA_MISMATCH_RES
+                                                  | GWY_DATA_MISMATCH_REAL
+                                                  | GWY_DATA_MISMATCH_LATERAL))
         gwy_params_set_image_id(params, PARAM_SIGNAL, dataid);
     args.nn = ensure_network(params);
     gwy_debug("switching args->nn to %s", gwy_resource_get_name(GWY_RESOURCE(args.nn)));
@@ -298,82 +283,56 @@ neural_train(G_GNUC_UNUSED GwyContainer *data, GwyRunType run)
 }
 
 static void
-neural_apply(GwyContainer *data, GwyRunType run)
+neural_apply(GwyFile *data, GwyRunModeFlags mode)
 {
     NeuralApplyArgs args;
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     gint id, newid;
 
-    g_return_if_fail(run & NEURAL_APPLY_RUN_MODES);
-    gwy_clear(&args, 1);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_clear1(args);
 
     args.params = gwy_params_new_from_settings(define_module_params_apply());
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD_ID, &id,
-                                     GWY_APP_DATA_FIELD, &args.field,
-                                     0);
+    gwy_data_browser_get_current(GWY_APP_FIELD_ID, &id,
+                                 GWY_APP_FIELD, &args.field,
+                                 0);
     g_return_if_fail(args.field);
 
     args.nn = ensure_network(args.params);
     gwy_debug("switching args->nn to %s", gwy_resource_get_name(GWY_RESOURCE(args.nn)));
 
-    if (run == GWY_RUN_INTERACTIVE)
+    if (mode == GWY_RUN_INTERACTIVE)
         outcome = run_gui_apply(&args);
     if (outcome == GWY_DIALOG_CANCEL)
         goto end;
 
     if (execute_apply(&args, data, id)) {
-        newid = gwy_app_data_browser_add_data_field(args.result, data, TRUE);
-        gwy_app_set_data_field_title(data, newid, _("Evaluated signal"));
-        gwy_app_sync_data_items(data, data, id, newid, FALSE, GWY_DATA_ITEM_GRADIENT, 0);
-        gwy_app_channel_log_add(data, id, newid, "proc::neural_apply", "settings-name", "neural", NULL);
+        newid = gwy_file_add_image(data, args.result);
+        gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+        gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _("Evaluated signal"), TRUE);
+        gwy_file_sync_items(data, GWY_FILE_IMAGE, id,
+                            data, GWY_FILE_IMAGE, newid,
+                            GWY_FILE_ITEM_PALETTE, FALSE);
+        gwy_log_add_full(data, GWY_FILE_IMAGE, id, newid, "proc::neural_apply", "settings-name", "neural", NULL);
     }
 
 end:
-    GWY_OBJECT_UNREF(args.result);
+    g_clear_object(&args.result);
     gwy_params_save_to_settings(args.params);
     g_object_unref(args.params);
 }
 
 /* This assumes model and singal are compatible! */
 static void
-setup_container(GwyContainer *data, NeuralTrainArgs *args)
+setup_images(NeuralTrainGUI *gui)
 {
-    GwyParams *params = args->params;
-    GwyContainer *modeldata, *signaldata;
-    GwyDataField *fields[3], *model;
-    GwyAppDataId dataid;
-    gint i;
+    GwyParams *params = gui->args->params;
+    GwyField *signal = gwy_params_get_image(params, PARAM_SIGNAL);
 
-    model = gwy_params_get_image(params, PARAM_MODEL);
-    dataid = gwy_params_get_data_id(params, PARAM_MODEL);
-    modeldata = gwy_app_data_browser_get(dataid.datano);
-    gwy_container_set_object(data, gwy_app_get_data_key_for_id(0), model);
-    gwy_app_sync_data_items(modeldata, data, dataid.id, 0, FALSE,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            GWY_DATA_ITEM_GRADIENT,
-                            GWY_DATA_ITEM_RANGE_TYPE,
-                            GWY_DATA_ITEM_RANGE,
-                            0);
-
-    fields[0] = g_object_ref(gwy_params_get_image(params, PARAM_SIGNAL));
-    fields[1] = gwy_data_field_new_alike(fields[0], TRUE);
-    fields[2] = gwy_data_field_new_alike(fields[0], TRUE);
-
-    dataid = gwy_params_get_data_id(params, PARAM_MODEL);
-    signaldata = gwy_app_data_browser_get(dataid.datano);
-    for (i = 0; i < 3; i++) {
-        gwy_container_pass_object(data, gwy_app_get_data_key_for_id(i+1), fields[i]);
-        gwy_app_sync_data_items(signaldata, data, dataid.id, 1+1, FALSE,
-                                GWY_DATA_ITEM_REAL_SQUARE,
-                                GWY_DATA_ITEM_GRADIENT,
-                                0);
-        if (i < 2) {
-            gwy_app_sync_data_items(signaldata, data, dataid.id, 1+1, FALSE,
-                                    GWY_DATA_ITEM_RANGE_TYPE,
-                                    GWY_DATA_ITEM_RANGE,
-                                    0);
-        }
-    }
+    g_clear_object(&gui->result);
+    g_clear_object(&gui->diff);
+    gui->result = gwy_field_new_alike(signal, TRUE);
+    gui->diff = gwy_field_new_alike(signal, TRUE);
 }
 
 static void
@@ -425,7 +384,6 @@ create_network_list(GtkTreeModel *model, GtkWidget **scroll)
     guint i;
 
     networklist = gtk_tree_view_new_with_model(model);
-    gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(networklist), TRUE);
 
     for (i = 0; i < G_N_ELEMENTS(columns); i++) {
         renderer = gtk_cell_renderer_text_new();
@@ -446,31 +404,29 @@ static GwyDialogOutcome
 run_gui_train(NeuralTrainArgs *args)
 {
     NeuralTrainGUI gui;
-    GwyContainer *data;
     GwyDialog *dialog;
     GtkWidget *hbox, *vbox, *notebook, *graph;
     GwyGraphCurveModel *gcmodel;
     GwyDialogOutcome outcome;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
     gui.calculated = FALSE;
     gui.compatible = TRUE;
 
-    gui.data = data = gwy_container_new();
-    setup_container(data, args);
+    setup_images(&gui);
 
     gui.dialog = gwy_dialog_new(_("Neural Network Training"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    hbox = gtk_hbox_new(FALSE, 0);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
     gwy_dialog_add_content(dialog, hbox, TRUE, TRUE, 4);
 
-    vbox = gtk_vbox_new(FALSE, 4);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
     gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 4);
 
-    gui.dataview = gwy_create_preview(data, 0, PREVIEW_SMALL_SIZE, FALSE);
+    gui.dataview = gwy_create_preview(gwy_params_get_image(args->params, PARAM_MODEL), NULL, PREVIEW_SMALL_SIZE);
     gtk_box_pack_start(GTK_BOX(vbox), gui.dataview, FALSE, FALSE, 0);
 
     gui.gmodel = gwy_graph_model_new();
@@ -507,8 +463,9 @@ run_gui_train(NeuralTrainArgs *args)
 
     outcome = gwy_dialog_run(dialog);
 
+    g_clear_object(&gui.result);
+    g_clear_object(&gui.diff);
     g_object_unref(gui.gmodel);
-    g_object_unref(gui.data);
 
     return outcome;
 }
@@ -525,7 +482,7 @@ training_tab_new(NeuralTrainGUI *gui)
     gwy_param_table_append_slider(table, PARAM_TRAINSTEPS);
     gwy_param_table_append_separator(table);
     gwy_param_table_append_radio(table, PARAM_PREVIEW);
-    gwy_param_table_append_button(table, BUTTON_TRAIN, -1, RESPONSE_TRAIN, gwy_sgettext("verb|_Train"));
+    gwy_param_table_append_button(table, BUTTON_TRAIN, -1, RESPONSE_TRAIN, C_("verb", "_Train"));
     gwy_param_table_append_button(table, BUTTON_REINITIALIZE, BUTTON_TRAIN, RESPONSE_REINITIALIZE, _("Re_initialize"));
     gwy_param_table_append_message(table, MESSAGE_TRAINING, NULL);
     gwy_param_table_append_separator(table);
@@ -567,7 +524,7 @@ networks_tab_new(NeuralTrainGUI *gui)
     GtkWidget *vbox, *scroll, *hbox, *bbox, *button, *label;
     GtkTreeModel *filtermodel;
 
-    vbox = gtk_vbox_new(FALSE, 0);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
     gtk_container_set_border_width(GTK_CONTAINER(vbox), 4);
 
     gui->store = gwy_inventory_store_new(gwy_neural_networks());
@@ -577,16 +534,16 @@ networks_tab_new(NeuralTrainGUI *gui)
     g_object_unref(filtermodel);
     gtk_box_pack_start(GTK_BOX(vbox), scroll, TRUE, TRUE, 0);
 
-    bbox = gtk_hbutton_box_new();
-    gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox), GTK_BUTTONBOX_START);
+    bbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
+    gtk_box_set_homogeneous(GTK_BOX(bbox), TRUE);
     gtk_box_pack_start(GTK_BOX(vbox), bbox, FALSE, FALSE, 0);
 
-    button = gtk_button_new_with_mnemonic(gwy_sgettext("verb|_Load"));
+    button = gtk_button_new_with_mnemonic(C_("verb", "_Load"));
     gui->load = button;
     gtk_container_add(GTK_CONTAINER(bbox), button);
     g_signal_connect_swapped(button, "clicked", G_CALLBACK(network_load), gui);
 
-    button = gtk_button_new_with_mnemonic(gwy_sgettext("verb|_Store"));
+    button = gtk_button_new_with_mnemonic(C_("verb", "_Store"));
     gui->save = button;
     gtk_container_add(GTK_CONTAINER(bbox), button);
     g_signal_connect_swapped(button, "clicked", G_CALLBACK(network_store), gui);
@@ -601,7 +558,7 @@ networks_tab_new(NeuralTrainGUI *gui)
     gtk_container_add(GTK_CONTAINER(bbox), button);
     g_signal_connect_swapped(button, "clicked", G_CALLBACK(network_delete), gui);
 
-    hbox = gwy_hbox_new(6);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 6);
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 4);
 
     gui->networkname = gtk_entry_new();
@@ -662,7 +619,7 @@ can_select_network(G_GNUC_UNUSED GtkTreeSelection *selection,
                    G_GNUC_UNUSED gboolean path_currently_selected,
                    gpointer data)
 {
-    GwyDataField *field = (GwyDataField*)data;
+    GwyField *field = (GwyField*)data;
     GwyNeuralNetwork *network;
     GtkTreeIter iter;
 
@@ -670,8 +627,8 @@ can_select_network(G_GNUC_UNUSED GtkTreeSelection *selection,
         return FALSE;
 
     gtk_tree_model_get(model, &iter, 0, &network, -1);
-    return (gwy_data_field_get_xres(field) > network->data.width
-            && gwy_data_field_get_yres(field) > network->data.height);
+    return (gwy_field_get_xres(field) > network->data.width
+            && gwy_field_get_yres(field) > network->data.height);
 }
 
 static void
@@ -682,23 +639,23 @@ param_changed(NeuralTrainGUI *gui, gint id)
     if (id < 0 || id == PARAM_MODEL || id == PARAM_SIGNAL) {
         NeuralNetworkData *nndata = &gui->args->nn->data;
         GwyParamTable *table = gui->table_training;
-        GwyDataField *model = gwy_params_get_image(params, PARAM_MODEL);
-        GwyDataField *signal = gwy_params_get_image(params, PARAM_SIGNAL);
+        GwyField *model = gwy_params_get_image(params, PARAM_MODEL);
+        GwyField *signal = gwy_params_get_image(params, PARAM_SIGNAL);
         PreviewType previewtype = gwy_params_get_enum(params, PARAM_PREVIEW);
         gboolean ok;
         const gchar *message = "";
 
-        ok = !gwy_data_field_check_compatibility(model, signal,
-                                                 GWY_DATA_COMPATIBILITY_RES
-                                                 | GWY_DATA_COMPATIBILITY_REAL
-                                                 | GWY_DATA_COMPATIBILITY_LATERAL);
+        ok = !gwy_field_is_incompatible(model, signal,
+                                        GWY_DATA_MISMATCH_RES
+                                        | GWY_DATA_MISMATCH_REAL
+                                        | GWY_DATA_MISMATCH_LATERAL);
         if (!ok)
             message = _("Model and signal are not compatible.");
 
-        if (ok && (gwy_data_field_get_xres(model) <= nndata->width
-                   || gwy_data_field_get_yres(model) <= nndata->height
-                   || gwy_data_field_get_xres(signal) <= nndata->width
-                   || gwy_data_field_get_yres(signal) <= nndata->height)) {
+        if (ok && (gwy_field_get_xres(model) <= nndata->width
+                   || gwy_field_get_yres(model) <= nndata->height
+                   || gwy_field_get_xres(signal) <= nndata->width
+                   || gwy_field_get_yres(signal) <= nndata->height)) {
             ok = FALSE;
             message = _("A field dimension is too small for chosen window size.");
         }
@@ -708,7 +665,7 @@ param_changed(NeuralTrainGUI *gui, gint id)
 
         gui->calculated = FALSE;
         gui->compatible = ok;
-        setup_container(gui->data, gui->args);
+        setup_images(gui);
         if (previewtype != PREVIEW_MODEL && previewtype != PREVIEW_SIGNAL) {
             previewtype = (id == PARAM_SIGNAL ? PREVIEW_SIGNAL : PREVIEW_MODEL);
             gwy_param_table_set_enum(table, PARAM_PREVIEW, previewtype);
@@ -718,8 +675,7 @@ param_changed(NeuralTrainGUI *gui, gint id)
     }
 
     if (id < 0 || id == PARAM_PREVIEW || id == PARAM_MODEL || id == PARAM_SIGNAL) {
-        set_layer_channel(gwy_data_view_get_base_layer(GWY_DATA_VIEW(gui->dataview)),
-                          gwy_params_get_enum(params, PARAM_PREVIEW));
+        switch_preview(gui);
         gwy_set_data_preview_size(GWY_DATA_VIEW(gui->dataview), PREVIEW_SMALL_SIZE);
     }
 
@@ -738,7 +694,7 @@ param_changed(NeuralTrainGUI *gui, gint id)
         gwy_assign_string(&nndata->outunits, gwy_params_get_string(params, PARAM_OUTUNITS));
         if (id == PARAM_WIN_WIDTH || id == PARAM_WIN_HEIGHT || id == PARAM_NHIDDEN)
             neural_network_data_resize(nndata);
-        GWY_RESOURCE(gui->args->nn)->is_modified = TRUE;
+        gwy_resource_data_changed(GWY_RESOURCE(gui->args->nn));
         gui->calculated = FALSE;
     }
 }
@@ -750,7 +706,7 @@ dialog_response(NeuralTrainGUI *gui, gint response)
         NeuralNetworkData *nndata = &gui->args->nn->data;
         neural_network_data_init(nndata, NULL);
         gui->calculated = FALSE;
-        GWY_RESOURCE(gui->args->nn)->is_modified = TRUE;
+        gwy_resource_data_changed(GWY_RESOURCE(gui->args->nn));
     }
     else if (response == RESPONSE_TRAIN) {
         train_network(gui);
@@ -758,23 +714,23 @@ dialog_response(NeuralTrainGUI *gui, gint response)
 }
 
 static void
-calculate_scaling(GwyDataField *field, gdouble *factor, gdouble *shift)
+calculate_scaling(GwyField *field, gdouble *factor, gdouble *shift)
 {
     gdouble min, max;
 
-    gwy_data_field_get_min_max(field, &min, &max);
+    gwy_field_get_min_max(field, &min, &max);
     *shift = min;
     *factor = 1.0/(max - *shift);
 }
 
 static gboolean
-train_do(GwyNeuralNetwork *nn, GwyDataLine *errors, GwyGraphCurveModel *gcmodel,
-         GwyDataField *model, GwyDataField *signal, GwyDataField *mask,
+train_do(GwyNeuralNetwork *nn, GwyLine *errors, GwyGraphCurveModel *gcmodel,
+         GwyField *model, GwyField *signal, GwyField *mask,
          GwyMaskingType masking, gdouble sfactor, gdouble sshift,
          guint trainsteps)
 {
     NeuralNetworkData *nndata = &nn->data;
-    GwyDataField *scaled;
+    GwyField *scaled;
     guint width = nndata->width, height = nndata->height, xres, yres;
     guint n, k, npixels, irow, col, row;
     guint *indices;
@@ -788,7 +744,7 @@ train_do(GwyNeuralNetwork *nn, GwyDataLine *errors, GwyGraphCurveModel *gcmodel,
     if (!trainsteps)
         return TRUE;
 
-    gwy_graph_curve_model_set_data_from_dataline(gcmodel, errors, 0, 1);
+    gwy_graph_curve_model_set_data_from_line(gcmodel, errors, 0, 1);
     if (!gwy_app_wait_set_message(_("Training...")))
         return FALSE;
     timer = g_timer_new();
@@ -797,14 +753,14 @@ train_do(GwyNeuralNetwork *nn, GwyDataLine *errors, GwyGraphCurveModel *gcmodel,
     nndata->outshift = sshift;
     calculate_scaling(model, &nndata->infactor, &nndata->inshift);
 
-    scaled = gwy_data_field_duplicate(model);
-    gwy_data_field_normalize(scaled);
-    xres = gwy_data_field_get_xres(scaled);
-    yres = gwy_data_field_get_yres(scaled);
-    dtmodel = gwy_data_field_get_data_const(scaled);
-    dtsignal = gwy_data_field_get_data(signal);
+    scaled = gwy_field_copy(model);
+    gwy_field_normalize(scaled);
+    xres = gwy_field_get_xres(scaled);
+    yres = gwy_field_get_yres(scaled);
+    dtmodel = gwy_field_get_data_const(scaled);
+    dtsignal = gwy_field_get_data(signal);
     if (masking != GWY_MASK_IGNORE && mask)
-        dtmask = gwy_data_field_get_data_const(mask);
+        dtmask = gwy_field_get_data_const(mask);
     else
         masking = GWY_MASK_IGNORE;
 
@@ -830,9 +786,9 @@ train_do(GwyNeuralNetwork *nn, GwyDataLine *errors, GwyGraphCurveModel *gcmodel,
             nn->target[0] = sfactor*(dtsignal[indices[k] + height/2*xres + width/2] - sshift);
             gwy_neural_network_train_step(nn, 0.3, 0.3, &eo, &eh);
         }
-        gwy_data_line_set_val(errors, n, eo + eh);
+        gwy_line_set_val(errors, n, eo + eh);
         if (g_timer_elapsed(timer, NULL) - lasttime >= 0.2) {
-            gwy_graph_curve_model_set_data_from_dataline(gcmodel, errors, 0, n+1);
+            gwy_graph_curve_model_set_data_from_line(gcmodel, errors, 0, n+1);
             if (!gwy_app_wait_set_fraction((gdouble)n/trainsteps))
                 goto fail;
             lasttime = g_timer_elapsed(timer, NULL);
@@ -850,12 +806,12 @@ fail:
 
 static gboolean
 evaluate(GwyNeuralNetwork *nn,
-         GwyDataField *model, GwyDataField *result,
+         GwyField *model, GwyField *result,
          gdouble sfactor, gdouble sshift)
 {
     NeuralNetworkData *nndata = &nn->data;
-    GwyDataField *scaled;
-    GwySIUnit *unit;
+    GwyField *scaled;
+    GwyUnit *unit;
     guint width = nndata->width, height = nndata->height, xres, yres;
     guint col, row, irow;
     const gdouble *drmodel;
@@ -866,12 +822,12 @@ evaluate(GwyNeuralNetwork *nn,
     if (!gwy_app_wait_set_message(_("Evaluating...")))
         return FALSE;
 
-    scaled = gwy_data_field_duplicate(model);
-    gwy_data_field_normalize(scaled);
-    xres = gwy_data_field_get_xres(scaled);
-    yres = gwy_data_field_get_yres(scaled);
-    drmodel = gwy_data_field_get_data_const(scaled);
-    dresult = gwy_data_field_get_data(result);
+    scaled = gwy_field_copy(model);
+    gwy_field_normalize(scaled);
+    xres = gwy_field_get_xres(scaled);
+    yres = gwy_field_get_yres(scaled);
+    drmodel = gwy_field_get_data_const(scaled);
+    dresult = gwy_field_get_data(result);
 
     for (row = height/2; row < yres + height/2 - height; row++) {
         for (col = width/2; col < xres + width/2 - width; col++) {
@@ -884,18 +840,17 @@ evaluate(GwyNeuralNetwork *nn,
             goto fail;
     }
     ok = TRUE;
-    unit = gwy_data_field_get_si_unit_z(result);
-    gwy_si_unit_set_from_string(unit, nndata->outunits);
-    gwy_si_unit_power_multiply(unit, 1, gwy_data_field_get_si_unit_xy(model), nndata->inpowerxy, unit);
-    gwy_si_unit_power_multiply(unit, 1, gwy_data_field_get_si_unit_z(model), nndata->inpowerz, unit);
+    unit = gwy_field_get_unit_z(result);
+    gwy_unit_set_from_string(unit, nndata->outunits);
+    gwy_unit_power_multiply(unit, 1, gwy_field_get_unit_xy(model), nndata->inpowerxy, unit);
+    gwy_unit_power_multiply(unit, 1, gwy_field_get_unit_z(model), nndata->inpowerz, unit);
 
     /* Fill the borders with the average of result. */
-    avg = gwy_data_field_area_get_avg_mask(result, NULL, GWY_MASK_IGNORE,
-                                           width/2, height/2, xres - width, yres - height);
-    gwy_data_field_area_fill(result, 0, 0, xres, height/2, avg);
-    gwy_data_field_area_fill(result, 0, height/2, width/2, yres - height, avg);
-    gwy_data_field_area_fill(result, xres + width/2 - width, height/2, width - width/2, yres - height, avg);
-    gwy_data_field_area_fill(result, 0, yres + height/2 - height, xres, height - height/2, avg);
+    avg = gwy_field_area_get_avg(result, NULL, GWY_MASK_IGNORE, width/2, height/2, xres - width, yres - height);
+    gwy_field_area_fill(result, NULL, GWY_MASK_IGNORE, 0, 0, xres, height/2, avg);
+    gwy_field_area_fill(result, NULL, GWY_MASK_IGNORE, 0, height/2, width/2, yres - height, avg);
+    gwy_field_area_fill(result, NULL, GWY_MASK_IGNORE, xres + width/2 - width, height/2, width - width/2, yres - height, avg);
+    gwy_field_area_fill(result, NULL, GWY_MASK_IGNORE, 0, yres + height/2 - height, xres, height - height/2, avg);
 
 fail:
     g_object_unref(scaled);
@@ -903,15 +858,15 @@ fail:
 }
 
 static gdouble
-calculate_mean_error(GwyDataField *field)
+calculate_mean_error(GwyField *field)
 {
     guint xres, yres, k;
     const gdouble *data;
     gdouble s = 0.0;
 
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
-    data = gwy_data_field_get_data_const(field);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
+    data = gwy_field_get_data_const(field);
     for (k = xres*yres; k; k--, data++)
         s += fabs(*data);
 
@@ -927,34 +882,31 @@ train_network(NeuralTrainGUI *gui)
     GwyMaskingType masking = gwy_params_get_enum(params, PARAM_MASKING);
     GwyParamTable *table = gui->table_training;
     GwyNeuralNetwork *nn = args->nn;
-    GwyDataField *model, *signal, *result, *diff, *mask;
-    GwyDataLine *errors;
+    GwyField *model = gwy_params_get_image(params, PARAM_MODEL);
+    GwyField *signal = gwy_params_get_image(params, PARAM_SIGNAL);
+    GwyField *mask = gwy_params_get_image_mask(params, PARAM_SIGNAL);
+    GwyField *result = gui->result, *diff = gui->diff;
+    GwyLine *errors;
     GwyGraphCurveModel *gcmodel;
     NeuralNetworkData backup;
-    GwySIValueFormat *vf;
+    GwyValueFormat *vf;
     gdouble sfactor, sshift;
     gchar *s;
     gboolean ok;
 
     gwy_app_wait_start(GTK_WINDOW(gui->dialog), _("Initializing..."));
 
-    model = gwy_container_get_object(gui->data, gwy_app_get_data_key_for_id(0));
-    signal = gwy_container_get_object(gui->data, gwy_app_get_data_key_for_id(1));
-    result = gwy_container_get_object(gui->data, gwy_app_get_data_key_for_id(2));
-    diff = gwy_container_get_object(gui->data, gwy_app_get_data_key_for_id(3));
-    mask = gwy_params_get_mask(params, PARAM_SIGNAL);
-
-    errors = gwy_data_line_new(trainsteps, trainsteps, TRUE);
+    errors = gwy_line_new(trainsteps, trainsteps, TRUE);
     gcmodel = gwy_graph_model_get_curve(gui->gmodel, 0);
     calculate_scaling(signal, &sfactor, &sshift);
 
-    gwy_clear(&backup, 1);
+    gwy_clear1(backup);
     neural_network_data_copy(&nn->data, &backup);
 
-    gwy_resource_use(GWY_RESOURCE(nn));
+    gwy_neural_network_ensure_state(nn);
     ok = (train_do(nn, errors, gcmodel, model, signal, mask, masking, sfactor, sshift, trainsteps)
           && evaluate(nn, model, result, sfactor, sshift));
-    gwy_resource_release(GWY_RESOURCE(nn));
+    gwy_neural_network_free_state(nn);
 
     gwy_app_wait_finish();
 
@@ -968,25 +920,25 @@ train_network(NeuralTrainGUI *gui)
         return;
     }
 
-    gwy_data_field_subtract_fields(diff, result, signal);
-    gwy_data_field_data_changed(result);
-    gwy_data_field_data_changed(diff);
+    gwy_field_subtract_fields(diff, result, signal);
+    gwy_field_data_changed(result);
+    gwy_field_data_changed(diff);
 
-    vf = gwy_data_field_get_value_format_z(diff, GWY_SI_UNIT_FORMAT_VFMARKUP, NULL);
+    vf = gwy_field_get_value_format_z(diff, GWY_UNIT_FORMAT_VFMARKUP, NULL);
     s = g_strdup_printf(_("Mean difference: %.*f %s"),
                         vf->precision, calculate_mean_error(diff)/vf->magnitude, vf->units);
-    gwy_si_unit_value_format_free(vf);
+    gwy_value_format_free(vf);
     gwy_param_table_set_label(table, MESSAGE_TRAINING, s);
     g_free(s);
 
-    GWY_RESOURCE(nn)->is_modified = TRUE;
     gui->calculated = TRUE;
+    gwy_resource_data_changed(GWY_RESOURCE(gui->args->nn));
     gwy_param_table_radio_set_sensitive(table, PARAM_PREVIEW, PREVIEW_RESULT, TRUE);
     gwy_param_table_radio_set_sensitive(table, PARAM_PREVIEW, PREVIEW_DIFFERENCE, TRUE);
 }
 
 static gboolean
-execute_apply(NeuralApplyArgs *args, GwyContainer *data, gint id)
+execute_apply(NeuralApplyArgs *args, GwyFile *data, gint id)
 {
     GwyParams *params = args->params;
     const gchar *name = gwy_params_get_string(params, PARAM_NAME);
@@ -996,14 +948,14 @@ execute_apply(NeuralApplyArgs *args, GwyContainer *data, gint id)
     gdouble factor, shift;
     gboolean ok;
 
-    gwy_app_wait_start(gwy_app_find_window_for_channel(data, id), _("Evaluating..."));
+    gwy_app_wait_start(gwy_data_browser_get_window_for_data(data, GWY_FILE_IMAGE, id), _("Evaluating..."));
 
     network = gwy_inventory_get_item(gwy_neural_networks(), name);
     g_assert(network);
     nndata = &network->data;
 
-    gwy_resource_use(GWY_RESOURCE(network));
-    args->result = gwy_data_field_new_alike(args->field, TRUE);
+    gwy_neural_network_ensure_state(network);
+    args->result = gwy_field_new_alike(args->field, TRUE);
     factor = nndata->outfactor;
     shift = nndata->outshift;
     if (scale_output) {
@@ -1012,7 +964,7 @@ execute_apply(NeuralApplyArgs *args, GwyContainer *data, gint id)
         factor /= ifactor/nndata->infactor;
     }
     ok = evaluate(network, args->field, args->result, factor, shift);
-    gwy_resource_release(GWY_RESOURCE(network));
+    gwy_neural_network_free_state(network);
 
     gwy_app_wait_finish();
 
@@ -1209,14 +1161,33 @@ network_validate_name(NeuralTrainGUI *gui,
 }
 
 static void
-set_layer_channel(GwyPixmapLayer *layer, gint channel)
+switch_preview(NeuralTrainGUI *gui)
 {
-    GwyLayerBasic *blayer = GWY_LAYER_BASIC(layer);
+    NeuralTrainArgs *args = gui->args;
+    GwyParams *params = args->params;
+    PreviewType display = gwy_params_get_enum(args->params, PARAM_PREVIEW);
+    GwyDataView *dataview = GWY_DATA_VIEW(gui->dataview);
+    gint param = (display == PREVIEW_MODEL ? PARAM_MODEL : PARAM_SIGNAL);
+    GwyField *model = gwy_params_get_image(params, param);
+    GwyAppDataId dataid = gwy_params_get_data_id(params, param);
+    GwyFile *data = gwy_data_browser_get_file(dataid.datano);
 
-    gwy_pixmap_layer_set_data_key(layer, g_quark_to_string(gwy_app_get_data_key_for_id(channel)));
-    gwy_layer_basic_set_gradient_key(blayer, g_quark_to_string(gwy_app_get_data_palette_key_for_id(channel)));
-    gwy_layer_basic_set_min_max_key(blayer, g_quark_to_string(gwy_app_get_data_range_max_key_for_id(channel)));
-    gwy_layer_basic_set_range_type_key(blayer, g_quark_to_string(gwy_app_get_data_range_type_key_for_id(channel)));
+    if (display == PREVIEW_MODEL || display == PREVIEW_SIGNAL) {
+        gwy_data_view_set_field(dataview, model);
+        gwy_setup_data_view(dataview, data, GWY_FILE_IMAGE, dataid.id,
+                            GWY_FILE_ITEM_REAL_SQUARE | GWY_FILE_ITEM_COLOR_MAPPING | GWY_FILE_ITEM_RANGE);
+    }
+    else if (display == PREVIEW_RESULT) {
+        gwy_data_view_set_field(dataview, gui->result);
+        gwy_data_view_set_color_mapping(dataview, GWY_COLOR_MAPPING_FULL);
+    }
+    else {
+        gwy_data_view_set_field(dataview, gui->diff);
+        gwy_data_view_set_color_mapping(dataview, GWY_COLOR_MAPPING_FULL);
+    }
+
+    /* Use signals' gradient for eveything except the model which we show in its own colours. */
+    gwy_setup_data_view(dataview, data, GWY_FILE_IMAGE, dataid.id, GWY_FILE_ITEM_PALETTE);
 }
 
 static GwyNeuralNetwork*
@@ -1224,8 +1195,6 @@ ensure_network(GwyParams *params)
 {
     GwyNeuralNetwork *nn;
 
-    printf("ensstring %s\n", gwy_params_get_string(params, PARAM_NAME));
-
     if ((nn = gwy_inventory_get_item(gwy_neural_networks(), gwy_params_get_string(params, PARAM_NAME))))
         return nn;
 
@@ -1240,10 +1209,10 @@ gwy_neural_network_save(GwyNeuralNetwork *nn)
     GwyResource *resource = GWY_RESOURCE(nn);
     GError *error = NULL;
 
-    if (!resource->is_modified)
+    if (!gwy_resource_is_modified(resource))
         return TRUE;
 
-    if (!gwy_resource_get_is_modifiable(resource)) {
+    if (!gwy_resource_is_modifiable(resource)) {
         g_warning("Non-modifiable resource was modified and is about to be saved");
         return FALSE;
     }
