@@ -1,22 +1,18 @@
 /*
- *  $Id: neuraldata.h 25040 2022-09-22 12:32:22Z yeti-dn $
+ *  $Id: neuraldata.h 28453 2025-08-26 07:42:56Z yeti-dn $
  *  Copyright (C) 2012 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
+ *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
+ *  later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #define GWY_NEURAL_NETWORK_UNTITLED "__untitled__"
@@ -72,23 +68,22 @@ struct _GwyNeuralNetworkClass {
     GwyResourceClass parent_class;
 };
 
-static GType             gwy_neural_network_get_type(void)                          G_GNUC_CONST;
-static void              gwy_neural_network_finalize(GObject *object);
-static void              gwy_neural_network_use     (GwyResource *resource);
-static void              gwy_neural_network_release (GwyResource *resource);
-static GwyNeuralNetwork* gwy_neural_network_new     (const gchar *name,
-                                                     const NeuralNetworkData *data,
-                                                     gboolean is_const);
-static void              gwy_neural_network_dump    (GwyResource *resource,
-                                                     GString *str);
-static GwyResource*      gwy_neural_network_parse   (const gchar *text,
-                                                     gboolean is_const);
-static void              neural_network_data_resize (NeuralNetworkData *nndata);
-static void              neural_network_data_init   (NeuralNetworkData *nndata,
-                                                     GRand *rng);
-static void              neural_network_data_copy   (const NeuralNetworkData *src,
-                                                     NeuralNetworkData *dest);
-static void              neural_network_data_free   (NeuralNetworkData *nndata);
+static GType             gwy_neural_network_get_type    (void)                          G_GNUC_CONST;
+static void              gwy_neural_network_finalize    (GObject *object);
+static GwyNeuralNetwork* gwy_neural_network_new         (const gchar *name,
+                                                         const NeuralNetworkData *data,
+                                                         gboolean is_const);
+static void              gwy_neural_network_dump        (GwyResource *resource,
+                                                         GString *str);
+static GwyResource*      gwy_neural_network_parse       (const gchar *text);
+static void              gwy_neural_network_ensure_state(GwyNeuralNetwork *nn);
+static void              gwy_neural_network_free_state  (GwyNeuralNetwork *nn);
+static void              neural_network_data_resize     (NeuralNetworkData *nndata);
+static void              neural_network_data_init       (NeuralNetworkData *nndata,
+                                                         GRand *rng);
+static void              neural_network_data_copy       (const NeuralNetworkData *src,
+                                                         NeuralNetworkData *dest);
+static void              neural_network_data_free       (NeuralNetworkData *nndata);
 
 static const NeuralNetworkData neuralnetworkdata_default = {
     1, 11, 11, 7, 1,     /* sizes */
@@ -114,8 +109,6 @@ gwy_neural_network_class_init(GwyNeuralNetworkClass *klass)
 
     res_class->name = "neuralnetwork";
     res_class->inventory = gwy_inventory_new(&res_class->item_type);
-    res_class->use = gwy_neural_network_use;
-    res_class->release = gwy_neural_network_release;
     res_class->dump = gwy_neural_network_dump;
     res_class->parse = gwy_neural_network_parse;
 }
@@ -128,7 +121,7 @@ gwy_neural_network_init(GwyNeuralNetwork *nn)
 }
 
 static void
-gwy_neural_network_resize(GwyNeuralNetwork *nn)
+gwy_neural_network_ensure_state(GwyNeuralNetwork *nn)
 {
     NeuralNetworkData *nndata = &nn->data;
     guint ninput = nndata->width * nndata->height;
@@ -165,17 +158,8 @@ gwy_neural_network_finalize(GObject *object)
 }
 
 static void
-gwy_neural_network_use(GwyResource *resource)
+gwy_neural_network_free_state(GwyNeuralNetwork *nn)
 {
-    GwyNeuralNetwork *nn = GWY_NEURAL_NETWORK(resource);
-    gwy_neural_network_resize(nn);
-}
-
-static void
-gwy_neural_network_release(GwyResource *resource)
-{
-    GwyNeuralNetwork *nn = GWY_NEURAL_NETWORK(resource);
-
     g_free(nn->input);
     g_free(nn->hidden);
     g_free(nn->output);
@@ -192,15 +176,11 @@ gwy_neural_network_new(const gchar *name,
                        const NeuralNetworkData *data,
                        gboolean is_const)
 {
-    GwyNeuralNetwork *nn;
-
-    nn = g_object_new(GWY_TYPE_NEURAL_NETWORK,
-                      "is-const", is_const,
-                      NULL);
+    GwyNeuralNetwork *nn = g_object_new(GWY_TYPE_NEURAL_NETWORK,
+                                        "name", name,
+                                        "const", is_const,
+                                        NULL);
     neural_network_data_copy(data, &nn->data);
-    g_string_assign(GWY_RESOURCE(nn)->name, name);
-    /* New non-const resources start as modified */
-    GWY_RESOURCE(nn)->is_modified = !is_const;
 
     return nn;
 }
@@ -288,8 +268,7 @@ gwy_neural_network_read_weights(gdouble *weights,
 }
 
 static GwyResource*
-gwy_neural_network_parse(const gchar *text,
-                         gboolean is_const)
+gwy_neural_network_parse(const gchar *text)
 {
     NeuralNetworkData nndata;
     GwyNeuralNetwork *nn = NULL;
@@ -314,14 +293,10 @@ gwy_neural_network_parse(const gchar *text,
             if (layer == 0) {
                 guint ninput = nndata.width * nndata.height;
                 neural_network_data_resize(&nndata);
-                gwy_neural_network_read_weights(nndata.winput,
-                                                (ninput + 1)*nndata.nhidden,
-                                                line);
+                gwy_neural_network_read_weights(nndata.winput, (ninput + 1)*nndata.nhidden, line);
             }
             else if (layer == 1)
-                gwy_neural_network_read_weights(nndata.whidden,
-                                                (nndata.nhidden + 1)*nndata.noutput,
-                                                line);
+                gwy_neural_network_read_weights(nndata.whidden, (nndata.nhidden + 1)*nndata.noutput, line);
             else {
                 g_warning("Too many neuron weight lines.");
             }
@@ -382,8 +357,7 @@ gwy_neural_network_parse(const gchar *text,
             g_warning("Unknown field `%s'.", key);
     }
 
-    nn = gwy_neural_network_new("", &nndata, is_const);
-    GWY_RESOURCE(nn)->is_modified = FALSE;
+    nn = gwy_neural_network_new("", &nndata, FALSE);
     /* FIXME: gwy_neural_network_data_sanitize(&nn->data); */
     g_free(str);
     neural_network_data_free(&nndata);
@@ -564,10 +538,8 @@ gwy_neural_network_train_step(GwyNeuralNetwork *nn,
     NeuralNetworkData *nndata = &nn->data;
     guint ninput = nndata->width * nndata->height;
 
-    layer_forward(nn->input, nn->hidden,
-                  nndata->winput, ninput, nndata->nhidden);
-    layer_forward(nn->hidden, nn->output,
-                  nndata->whidden, nndata->nhidden, nndata->noutput);
+    layer_forward(nn->input, nn->hidden, nndata->winput, ninput, nndata->nhidden);
+    layer_forward(nn->hidden, nn->output, nndata->whidden, nndata->nhidden, nndata->noutput);
 
     *err_o = output_error(nn->output, nndata->noutput, nn->target, nn->doutput);
     *err_h = hidden_error(nn->hidden, nndata->nhidden, nn->dhidden,
@@ -585,10 +557,8 @@ gwy_neural_network_forward_feed(GwyNeuralNetwork *nn)
     NeuralNetworkData *nndata = &nn->data;
     guint ninput = nndata->width * nndata->height;
 
-    layer_forward(nn->input, nn->hidden, nndata->winput,
-                  ninput, nndata->nhidden);
-    layer_forward(nn->hidden, nn->output, nndata->whidden,
-                  nndata->nhidden, nndata->noutput);
+    layer_forward(nn->input, nn->hidden, nndata->winput, ninput, nndata->nhidden);
+    layer_forward(nn->hidden, nn->output, nndata->whidden, nndata->nhidden, nndata->noutput);
 }
 
-/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
+/* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
