@@ -1,5 +1,5 @@
 /*
- *  $Id: otsu_threshold.c 25638 2023-09-07 12:23:28Z yeti-dn $
+ *  $Id: otsu_threshold.c 28910 2025-11-24 18:11:09Z yeti-dn $
  *  Copyright (C) 2013 Brazilian Nanotechnology National Laboratory
  *  E-mail: Vinicius Barboza <vinicius.barboza@lnnano.cnpem.br>
  *
@@ -16,18 +16,15 @@
  */
 
 #include "config.h"
-#include <libgwyddion/gwymath.h>
-#include <libprocess/grains.h>
-#include <libprocess/filters.h>
-#include <libgwymodule/gwymodule.h>
-#include <app/gwyapp.h>
+#include <glib/gi18n-lib.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES GWY_RUN_IMMEDIATE
 
 static gboolean module_register(void);
-static void     otsu_threshold (GwyContainer *data,
-                                GwyRunType run);
+static void     module_main    (GwyFile *data,
+                                GwyRunModeFlags mode);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
@@ -45,52 +42,52 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("otsu-threshold",
-                              (GwyProcessFunc)&otsu_threshold,
+                              module_main,
                               N_("/_Grains/_Mark by Otsu's"),
-                              GWY_STOCK_GRAINS_OTSU,
+                              GWY_ICON_GRAINS_OTSU,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Automated threshold using Otsu's method on heights."));
     return TRUE;
 }
 
 static void
-otsu_threshold(GwyContainer *data,
-               GwyRunType run)
+module_main(GwyFile *data,
+            GwyRunModeFlags mode)
 {
-    GwyDataField *dfield, *mfield;
+    GwyField *dfield, *mfield;
     GQuark mquark;
     gdouble thresh;
     gint id;
 
     /* Running smoothly */
-    g_return_if_fail(run & RUN_MODES);
+    g_return_if_fail(mode & RUN_MODES);
 
     /* Getting fields and quarks */
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &dfield,
-                                     GWY_APP_MASK_FIELD, &mfield,
-                                     GWY_APP_MASK_FIELD_KEY, &mquark,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &dfield,
+                                 GWY_APP_MASK_FIELD, &mfield,
+                                 GWY_APP_MASK_FIELD_KEY, &mquark,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
 
     /* Setting checkopint */
-    gwy_app_undo_qcheckpointv(data, 1, &mquark);
+    gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &mquark);
 
     /* Checking for mask and creating a new one */
     if (!mfield) {
-        mfield = gwy_data_field_new_alike(dfield, TRUE);
-        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(mfield), NULL);
-        gwy_container_pass_object(data, mquark, mfield);
+        mfield = gwy_field_new_alike(dfield, TRUE);
+        gwy_unit_clear(gwy_field_get_unit_z(mfield));
+        gwy_file_pass_image_mask(data, id, mfield);
     }
 
     /* Copying the information to the mask field */
-    gwy_data_field_copy(dfield, mfield, FALSE);
+    gwy_field_copy_data(dfield, mfield);
 
     /* Apply threshold to mask field */
-    thresh = gwy_data_field_otsu_threshold(mfield);
-    gwy_data_field_threshold(mfield, thresh, 0.0, 1.0);
-    gwy_data_field_data_changed(mfield);
-    gwy_app_channel_log_add_proc(data, id, id);
+    thresh = gwy_field_otsu_threshold(mfield);
+    gwy_field_threshold(mfield, thresh, 0.0, 1.0);
+    gwy_field_data_changed(mfield);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, id);
 }
 
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
