@@ -1,5 +1,5 @@
 /*
- *  $Id: pfm.c 27319 2025-02-21 15:36:00Z klapetek $
+ *  $Id: pfm.c 28861 2025-11-14 13:02:11Z yeti-dn $
  *  Copyright (C) 2024 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,17 +16,9 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/gwyprocess.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/correlation.h>
-#include <libprocess/filters.h>
-#include <libprocess/stats.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 
 #define RUN_MODES GWY_RUN_INTERACTIVE
 
@@ -67,8 +59,8 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *result[MAXRESULTS];
+    GwyField *field;
+    GwyField *result[MAXRESULTS];
 } ModuleArgs;
 
 typedef struct {
@@ -80,14 +72,14 @@ typedef struct {
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             pfm                 (GwyContainer *data,
-                                             GwyRunType runtype);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static gboolean         execute             (ModuleArgs *args,
                                              GtkWindow *wait_window);
 static GwyDialogOutcome run_gui             (ModuleArgs *args);
 static void             param_changed       (ModuleGUI *gui,
                                              gint id);
-static gboolean         image_filter        (GwyContainer *data,
+static gboolean         image_filter        (GwyFile *data,
                                              gint id,
                                              gpointer user_data);
 
@@ -108,11 +100,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("pfm",
-                              (GwyProcessFunc)&pfm,
+                              module_main,
                               N_("/_SPM Modes/_Electrical/_PFM..."),
                               NULL,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Processes PFM data from multiple rotations"));
 
     return TRUE;
@@ -125,7 +117,7 @@ define_module_params(void)
     static const GwyEnum rotates[] = {
         { N_("Clockwise"),        ROTATE_CW,    },
         { N_("Counterclocwise"),  ROTATE_CCW,   },
-        { N_("rotation|None"),    ROTATE_NONE,  },
+        { gwy_NC("rotation", "None"),    ROTATE_NONE,  },
     };
     static const GwyEnum outputs[] = {
         { N_("In-plane angles"),  OUTPUT_2D,    },
@@ -165,7 +157,7 @@ define_module_params(void)
 }
 
 static void
-pfm(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome;
     ModuleArgs args;
@@ -174,18 +166,18 @@ pfm(GwyContainer *data, GwyRunType runtype)
     guint i;
     PFMResult output;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(args.field);
     args.params = gwy_params_new_from_settings(define_module_params());
     args.result[0] = NULL;
     args.result[1] = NULL;
     args.result[2] = NULL;
 
-    dataid.datano = gwy_app_data_browser_get_number(data);
+    dataid.datano = gwy_file_get_id(data);
     dataid.id = id;
     gwy_params_set_image_id(args.params, PARAM_IMAGE_VPFM_ROT0_AMPLITUDE, dataid);
     gwy_params_set_image_id(args.params, PARAM_IMAGE_VPFM_ROT0_PHASE, dataid);
@@ -201,61 +193,64 @@ pfm(GwyContainer *data, GwyRunType runtype)
     if (outcome == GWY_DIALOG_CANCEL)
         goto end;
 
-    if (!execute(&args, gwy_app_find_window_for_channel(data, id)))
+    if (!execute(&args, gwy_data_browser_get_window_for_data(data, GWY_FILE_IMAGE, id)))
         goto end;
 
     output = gwy_params_get_enum(args.params, PARAM_OUTPUT);
 
     if (output == OUTPUT_2D) {
         if (args.result[0] && args.result[1]) {
-            newid = gwy_app_data_browser_add_data_field(args.result[0], data, TRUE);
-            gwy_app_sync_data_items(data, data, id, newid, FALSE,
-                                    GWY_DATA_ITEM_GRADIENT,
-                                    GWY_DATA_ITEM_REAL_SQUARE,
-                                    0);
-            gwy_app_set_data_field_title(data, newid, _("Magnitude"));
-            gwy_app_channel_log_add_proc(data, id, newid);
+            newid = gwy_file_add_image(data, args.result[0]);
+            gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+            gwy_file_sync_items(data, GWY_FILE_IMAGE, id,
+                                data, GWY_FILE_IMAGE, newid,
+                                GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_REAL_SQUARE, FALSE);
+            gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _("Magnitude"), TRUE);
+            gwy_log_add(data, GWY_FILE_IMAGE, id, newid);
 
-            newid = gwy_app_data_browser_add_data_field(args.result[1], data, TRUE);
-            gwy_app_sync_data_items(data, data, id, newid, FALSE,
-                                    GWY_DATA_ITEM_GRADIENT,
-                                    GWY_DATA_ITEM_REAL_SQUARE,
-                                    0);
-            gwy_app_set_data_field_title(data, newid, _("Azimuth"));
-            gwy_app_channel_log_add_proc(data, id, newid);
+            newid = gwy_file_add_image(data, args.result[1]);
+
+            gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+            gwy_file_sync_items(data, GWY_FILE_IMAGE, id,
+                                data, GWY_FILE_IMAGE, newid,
+                                GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_REAL_SQUARE, FALSE);
+            gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _("Azimuth"), TRUE);
+            gwy_log_add(data, GWY_FILE_IMAGE, id, newid);
         }
     } else {
         if (args.result[0] && args.result[1] && args.result[2]) {
-            newid = gwy_app_data_browser_add_data_field(args.result[0], data, TRUE);
-            gwy_app_sync_data_items(data, data, id, newid, FALSE,
-                                    GWY_DATA_ITEM_GRADIENT,
-                                    GWY_DATA_ITEM_REAL_SQUARE,
-                                    0);
-            gwy_app_set_data_field_title(data, newid, _("Magnitude"));
-            gwy_app_channel_log_add_proc(data, id, newid);
+            newid = gwy_file_add_image(data, args.result[0]);
+            gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+            gwy_file_sync_items(data, GWY_FILE_IMAGE, id,
+                                data, GWY_FILE_IMAGE, newid,
+                                GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_REAL_SQUARE, FALSE);
+            gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _("Magnitude"), TRUE);
+            gwy_log_add(data, GWY_FILE_IMAGE, id, newid);
 
-            newid = gwy_app_data_browser_add_data_field(args.result[1], data, TRUE);
-            gwy_app_sync_data_items(data, data, id, newid, FALSE,
-                                    GWY_DATA_ITEM_GRADIENT,
-                                    GWY_DATA_ITEM_REAL_SQUARE,
-                                    0);
-            gwy_app_set_data_field_title(data, newid, _("Azimuth"));
-            gwy_app_channel_log_add_proc(data, id, newid);
+            newid = gwy_file_add_image(data, args.result[1]);
 
-            newid = gwy_app_data_browser_add_data_field(args.result[2], data, TRUE);
-            gwy_app_sync_data_items(data, data, id, newid, FALSE,
-                                    GWY_DATA_ITEM_GRADIENT,
-                                    GWY_DATA_ITEM_REAL_SQUARE,
-                                    0);
-            gwy_app_set_data_field_title(data, newid, _("Inclination"));
-            gwy_app_channel_log_add_proc(data, id, newid);
+            gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+            gwy_file_sync_items(data, GWY_FILE_IMAGE, id,
+                                data, GWY_FILE_IMAGE, newid,
+                                GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_REAL_SQUARE, FALSE);
+            gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _("Azimuth"), TRUE);
+            gwy_log_add(data, GWY_FILE_IMAGE, id, newid);
+
+            newid = gwy_file_add_image(data, args.result[2]);
+
+            gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+            gwy_file_sync_items(data, GWY_FILE_IMAGE, id,
+                                data, GWY_FILE_IMAGE, newid,
+                                GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_REAL_SQUARE, FALSE);
+            gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _("Inclination"), TRUE);
+            gwy_log_add(data, GWY_FILE_IMAGE, id, newid);
         }
     }
 
 end:
     g_object_unref(args.params);
     for (i = 0; i < OUTPUT_NTYPES; i++)
-        GWY_OBJECT_UNREF(args.result[i]);
+        g_clear_object(args.result + i);
 }
 
 static GwyDialogOutcome
@@ -272,7 +267,7 @@ run_gui(ModuleArgs *args)
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    hbox = gwy_hbox_new(20);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 20);
     gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
     gwy_dialog_add_content(dialog, hbox, FALSE, FALSE, 0);
 
@@ -338,8 +333,8 @@ param_changed(ModuleGUI *gui, gint id)
     GwyParamTable *table = gui->table;
     GwyParamTable *table_data = gui->table_data;
     gboolean phase_error = FALSE, amplitude_error = FALSE;
-    GwyDataField *vpfm_0_a, *vpfm_0_p, *vpfm_90_a, *vpfm_90_p;
-    GwyDataField *lpfm_0_a, *lpfm_0_p, *lpfm_90_a, *lpfm_90_p;
+    GwyField *vpfm_0_a, *vpfm_0_p, *vpfm_90_a, *vpfm_90_p;
+    GwyField *lpfm_0_a, *lpfm_0_p, *lpfm_90_a, *lpfm_90_p;
 
     if (id < 0 || id == PARAM_OUTPUT) {
         PFMResult output = gwy_params_get_enum(params, PARAM_OUTPUT);
@@ -357,9 +352,9 @@ param_changed(ModuleGUI *gui, gint id)
        lpfm_0_p = gwy_params_get_image(params, PARAM_IMAGE_LPFM_ROT0_PHASE);
        lpfm_90_p = gwy_params_get_image(params, PARAM_IMAGE_LPFM_ROT90_PHASE);
 
-       if (!(gwy_si_unit_equal(gwy_data_field_get_si_unit_z(vpfm_0_p), gwy_data_field_get_si_unit_z(vpfm_90_p))
-             && gwy_si_unit_equal(gwy_data_field_get_si_unit_z(vpfm_0_p), gwy_data_field_get_si_unit_z(lpfm_0_p))
-             && gwy_si_unit_equal(gwy_data_field_get_si_unit_z(vpfm_0_p), gwy_data_field_get_si_unit_z(lpfm_90_p)))) {
+       if (!(gwy_unit_equal(gwy_field_get_unit_z(vpfm_0_p), gwy_field_get_unit_z(vpfm_90_p))
+             && gwy_unit_equal(gwy_field_get_unit_z(vpfm_0_p), gwy_field_get_unit_z(lpfm_0_p))
+             && gwy_unit_equal(gwy_field_get_unit_z(vpfm_0_p), gwy_field_get_unit_z(lpfm_90_p)))) {
           phase_error = TRUE;
        }
     }
@@ -370,9 +365,9 @@ param_changed(ModuleGUI *gui, gint id)
        lpfm_0_a = gwy_params_get_image(params, PARAM_IMAGE_LPFM_ROT0_AMPLITUDE);
        lpfm_90_a = gwy_params_get_image(params, PARAM_IMAGE_LPFM_ROT90_AMPLITUDE);
 
-       if (!(gwy_si_unit_equal(gwy_data_field_get_si_unit_z(vpfm_0_a), gwy_data_field_get_si_unit_z(vpfm_90_a))
-             && gwy_si_unit_equal(gwy_data_field_get_si_unit_z(vpfm_0_a), gwy_data_field_get_si_unit_z(lpfm_0_a))
-             && gwy_si_unit_equal(gwy_data_field_get_si_unit_z(vpfm_0_a), gwy_data_field_get_si_unit_z(lpfm_90_a)))) {
+       if (!(gwy_unit_equal(gwy_field_get_unit_z(vpfm_0_a), gwy_field_get_unit_z(vpfm_90_a))
+             && gwy_unit_equal(gwy_field_get_unit_z(vpfm_0_a), gwy_field_get_unit_z(lpfm_0_a))
+             && gwy_unit_equal(gwy_field_get_unit_z(vpfm_0_a), gwy_field_get_unit_z(lpfm_90_a)))) {
            amplitude_error = TRUE;
        }
     }
@@ -390,25 +385,24 @@ param_changed(ModuleGUI *gui, gint id)
 }
 
 static gboolean
-image_filter(GwyContainer *data, gint id, gpointer user_data)
+image_filter(GwyFile *data, gint id, gpointer user_data)
 {
     ModuleArgs *args = (ModuleArgs*)user_data;
-    GwyDataField *otherfield, *field = args->field;
+    GwyField *otherfield, *field = args->field;
 
-    if (!gwy_container_gis_object(data, gwy_app_get_data_key_for_id(id), &otherfield))
+    if (!(otherfield = gwy_file_get_image(data, id)))
         return FALSE;
 
-    return !gwy_data_field_check_compatibility(field, otherfield,
-                                               GWY_DATA_COMPATIBILITY_REAL | GWY_DATA_COMPATIBILITY_RES);
+    return !gwy_field_is_incompatible(field, otherfield, GWY_DATA_MISMATCH_REAL | GWY_DATA_MISMATCH_RES);
 }
 
 static gboolean
 execute(ModuleArgs *args, GtkWindow *wait_window)
 {
     GwyParams *params = args->params;
-    GwyDataField *vpfm_0_a, *vpfm_0_p, *lpfm_0_a, *lpfm_0_p;
-    GwyDataField *vpfm_90_a, *vpfm_90_p, *lpfm_90_a, *lpfm_90_p;
-    GwyDataField *rvpfm_90_a, *rvpfm_90_p, *rlpfm_90_a, *rlpfm_90_p;
+    GwyField *vpfm_0_a, *vpfm_0_p, *lpfm_0_a, *lpfm_0_p;
+    GwyField *vpfm_90_a, *vpfm_90_p, *lpfm_90_a, *lpfm_90_p;
+    GwyField *rvpfm_90_a, *rvpfm_90_p, *rlpfm_90_a, *rlpfm_90_p;
     gdouble latfactor1, vertfactor;
     gdouble *lpfm_0_a_data, *lpfm_0_p_data, *lpfm_90_a_data, *lpfm_90_p_data;
     gdouble *vpfm_0_a_data, *vpfm_0_p_data, *vpfm_90_a_data, *vpfm_90_p_data;
@@ -417,7 +411,7 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
     gint row, col, pos, rotcol, rotrow, rotpos, xres, yres, i, j, n;
     gboolean rotate;
     gdouble rotrange, bestrot, rotscore, bestrotscore, *rotdata, angle, xrot, maxscore;
-    GwyDataField *rotdfield, *kernel, *score;
+    GwyField *rotdfield, *kernel, *score;
     gint m, bestrotm, nrot, xborder, yborder;
 
     PFMResult output = gwy_params_get_enum(params, PARAM_OUTPUT);
@@ -438,8 +432,8 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
     lpfm_90_a = gwy_params_get_image(params, PARAM_IMAGE_LPFM_ROT90_AMPLITUDE);
     lpfm_90_p = gwy_params_get_image(params, PARAM_IMAGE_LPFM_ROT90_PHASE);
 
-    xres = gwy_data_field_get_xres(lpfm_0_a);
-    yres = gwy_data_field_get_yres(lpfm_0_a);
+    xres = gwy_field_get_xres(lpfm_0_a);
+    yres = gwy_field_get_yres(lpfm_0_a);
 
     gwy_app_wait_start(wait_window, _("Running computation..."));
 
@@ -449,8 +443,8 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
     rotrange = 0.3491; //+-10 deg, now fixed, could user chosen
     xborder = xres/10; //now fixed, could be user chosen
     yborder = yres/10;
-    kernel = gwy_data_field_area_extract(vpfm_90_a, xborder, yborder, xres - 2*xborder, yres - 2*yborder);
-    score = gwy_data_field_new_alike(vpfm_90_a, FALSE);
+    kernel = gwy_field_area_extract(vpfm_90_a, xborder, yborder, xres - 2*xborder, yres - 2*yborder);
+    score = gwy_field_new_alike(vpfm_90_a, FALSE);
 
     if (rotate) {
         bestrot = 0;
@@ -461,17 +455,17 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
         for (m = 0; m < nrot; m++) {
             angle = -rotrange/2 + m*rotrange/nrot;
 
-            rotdfield = gwy_data_field_new_rotated(vpfm_90_a, NULL, angle,
-                                                   GWY_INTERPOLATION_BILINEAR,
-                                                   GWY_ROTATE_RESIZE_SAME_SIZE);
-            gwy_data_field_area_copy(rotdfield, kernel, xborder, yborder, xres - 2*xborder, yres - 2*yborder, 0, 0);
+            rotdfield = gwy_field_new_rotated(vpfm_90_a, NULL, angle,
+                                              GWY_INTERPOLATION_LINEAR,
+                                              GWY_ROTATE_RESIZE_SAME_SIZE);
+            gwy_field_area_copy(rotdfield, kernel, xborder, yborder, xres - 2*xborder, yres - 2*yborder, 0, 0);
 
-            gwy_data_field_correlation_search(vpfm_0_a, kernel, NULL, score, GWY_CORRELATION_POC,
-                                              0.1, GWY_EXTERIOR_MIRROR_EXTEND, 0);
+            gwy_field_correlation_search(vpfm_0_a, kernel, NULL, score, GWY_CORRELATION_POC,
+                                         0.1, GWY_EXTERIOR_MIRROR, 0);
 
-            rotscore = gwy_data_field_get_max(score);
+            rotscore = gwy_field_get_max(score);
             rotdata[m] = rotscore;
-            //printf("%g %g\n", angle*180/G_PI, rotscore);
+            //printf("%g %g\n", gwy_rad2deg(angle), rotscore);
             if (rotscore > bestrotscore) {
                 bestrotscore = rotscore;
                 bestrot = angle;
@@ -482,27 +476,27 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
         }
         if (bestrotm > 0 && bestrotm < (nrot-1)) {
             gwy_math_refine_maximum_1d(rotdata + bestrotm - 1, &xrot);
-            //printf("rot %g     %g -> %g\n", xrot, bestrot*180/G_PI,( bestrot+xrot*rotrange/nrot)*180/G_PI);
+            //printf("rot %g     %g -> %g\n", xrot, gwy_rad2deg(bestrot),gwy_rad2deg(bestrot+xrot*rotrange/nrot));
             bestrot += xrot*rotrange/nrot;
         }
 
         g_free(rotdata);
-        //printf("rotrange %g  bestangle %g\n", rotrange, bestrot*180/G_PI);
+        //printf("rotrange %g  bestangle %g\n", rotrange, gwy_rad2deg(bestrot));
     }
     if (rotate)
-        gwy_data_field_rotate(vpfm_90_a, bestrot, GWY_INTERPOLATION_BILINEAR);
+        gwy_field_rotate(vpfm_90_a, bestrot, GWY_INTERPOLATION_LINEAR);
 
-    gwy_data_field_area_copy(vpfm_90_a, kernel, xborder, yborder, xres - 2*xborder, yres - 2*yborder, 0, 0);
+    gwy_field_area_copy(vpfm_90_a, kernel, xborder, yborder, xres - 2*xborder, yres - 2*yborder, 0, 0);
 
-    gwy_data_field_correlation_search(vpfm_0_a, kernel, NULL, score, GWY_CORRELATION_POC,
-                                      0.1, GWY_EXTERIOR_MIRROR_EXTEND, 0);
+    gwy_field_correlation_search(vpfm_0_a, kernel, NULL, score, GWY_CORRELATION_POC,
+                                 0.1, GWY_EXTERIOR_MIRROR, 0);
 
-    if (gwy_data_field_get_local_maxima_list(score, &xoff, &yoff, &maxscore, 1, 0, 0.0, FALSE)) {
+    if (gwy_field_get_local_maxima_list(score, &xoff, &yoff, &maxscore, 1, 0, 0.0, FALSE)) {
         if (xoff > 0 && xoff < (xres - 1) && yoff > 0 && yoff < (yres - 1)) {
             n = 0;
             for (j = -1; j <= 1; j++) {
                 for (i = -1; i <= 1; i++) {
-                    zvals[n++] = gwy_data_field_get_val(score, xoff+i, yoff+j);
+                    zvals[n++] = gwy_field_get_val(score, xoff+i, yoff+j);
                 }
             }
             gwy_math_refine_maximum_2d(zvals, &xoffset, &yoffset);
@@ -517,47 +511,47 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
 
     g_object_unref(kernel);
     g_object_unref(score);
-    //printf("x offset %g  y offset %g  rotation %g deg\n", xoffset, yoffset, bestrot*180/G_PI);
+    //printf("x offset %g  y offset %g  rotation %g deg\n", xoffset, yoffset, gwy_rad2deg(bestrot));
 
     if (bestrot != 0) {
-        rvpfm_90_a = gwy_data_field_new_rotated(vpfm_90_a, NULL, bestrot,
-                                                GWY_INTERPOLATION_BILINEAR,
-                                                GWY_ROTATE_RESIZE_SAME_SIZE);
-        rvpfm_90_p = gwy_data_field_new_rotated(vpfm_90_p, NULL, bestrot,
-                                                GWY_INTERPOLATION_BILINEAR,
-                                                GWY_ROTATE_RESIZE_SAME_SIZE);
-        rlpfm_90_a = gwy_data_field_new_rotated(lpfm_90_a, NULL, bestrot,
-                                                GWY_INTERPOLATION_BILINEAR,
-                                                GWY_ROTATE_RESIZE_SAME_SIZE);
-        rlpfm_90_p = gwy_data_field_new_rotated(lpfm_90_p, NULL, bestrot,
-                                                GWY_INTERPOLATION_BILINEAR,
-                                                GWY_ROTATE_RESIZE_SAME_SIZE);
+        rvpfm_90_a = gwy_field_new_rotated(vpfm_90_a, NULL, bestrot,
+                                           GWY_INTERPOLATION_LINEAR,
+                                           GWY_ROTATE_RESIZE_SAME_SIZE);
+        rvpfm_90_p = gwy_field_new_rotated(vpfm_90_p, NULL, bestrot,
+                                           GWY_INTERPOLATION_LINEAR,
+                                           GWY_ROTATE_RESIZE_SAME_SIZE);
+        rlpfm_90_a = gwy_field_new_rotated(lpfm_90_a, NULL, bestrot,
+                                           GWY_INTERPOLATION_LINEAR,
+                                           GWY_ROTATE_RESIZE_SAME_SIZE);
+        rlpfm_90_p = gwy_field_new_rotated(lpfm_90_p, NULL, bestrot,
+                                           GWY_INTERPOLATION_LINEAR,
+                                           GWY_ROTATE_RESIZE_SAME_SIZE);
 
-        vpfm_90_a_data = gwy_data_field_get_data(rvpfm_90_a);
-        vpfm_90_p_data = gwy_data_field_get_data(rvpfm_90_p);
-        lpfm_90_a_data = gwy_data_field_get_data(rlpfm_90_a);
-        lpfm_90_p_data = gwy_data_field_get_data(rlpfm_90_p);
+        vpfm_90_a_data = gwy_field_get_data(rvpfm_90_a);
+        vpfm_90_p_data = gwy_field_get_data(rvpfm_90_p);
+        lpfm_90_a_data = gwy_field_get_data(rlpfm_90_a);
+        lpfm_90_p_data = gwy_field_get_data(rlpfm_90_p);
     } else {
-        vpfm_90_a_data = gwy_data_field_get_data(vpfm_90_a);
-        vpfm_90_p_data = gwy_data_field_get_data(vpfm_90_p);
-        lpfm_90_a_data = gwy_data_field_get_data(lpfm_90_a);
-        lpfm_90_p_data = gwy_data_field_get_data(lpfm_90_p);
+        vpfm_90_a_data = gwy_field_get_data(vpfm_90_a);
+        vpfm_90_p_data = gwy_field_get_data(vpfm_90_p);
+        lpfm_90_a_data = gwy_field_get_data(lpfm_90_a);
+        lpfm_90_p_data = gwy_field_get_data(lpfm_90_p);
     }
 
-    vpfm_0_a_data = gwy_data_field_get_data(vpfm_0_a);
-    vpfm_0_p_data = gwy_data_field_get_data(vpfm_0_p);
-    lpfm_0_a_data = gwy_data_field_get_data(lpfm_0_a);
-    lpfm_0_p_data = gwy_data_field_get_data(lpfm_0_p);
+    vpfm_0_a_data = gwy_field_get_data(vpfm_0_a);
+    vpfm_0_p_data = gwy_field_get_data(vpfm_0_p);
+    lpfm_0_a_data = gwy_field_get_data(lpfm_0_a);
+    lpfm_0_p_data = gwy_field_get_data(lpfm_0_p);
 
     //run the calculation
     if (output == OUTPUT_2D) {
-        args->result[0] = gwy_data_field_new_alike(lpfm_0_a, FALSE);
-        args->result[1] = gwy_data_field_new_alike(lpfm_0_a, FALSE);
-        gwy_data_field_set_si_unit_z(args->result[0], gwy_si_unit_new("m"));
-        gwy_data_field_set_si_unit_z(args->result[1], gwy_si_unit_new("rad"));
+        args->result[0] = gwy_field_new_alike(lpfm_0_a, FALSE);
+        args->result[1] = gwy_field_new_alike(lpfm_0_a, FALSE);
+        gwy_unit_set_from_string(gwy_field_get_unit_z(args->result[0]), "m");
+        gwy_unit_set_from_string(gwy_field_get_unit_z(args->result[1]), "rad");
 
-        res0data = gwy_data_field_get_data(args->result[0]);
-        res1data = gwy_data_field_get_data(args->result[1]);
+        res0data = gwy_field_get_data(args->result[0]);
+        res1data = gwy_field_get_data(args->result[1]);
 
         for (row = 0; row < yres; row++) {
             for (col = 0; col < xres; col++) {
@@ -590,17 +584,17 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
             }
         }
     } else {
-        args->result[0] = gwy_data_field_new_alike(lpfm_0_a, FALSE);
-        args->result[1] = gwy_data_field_new_alike(lpfm_0_a, FALSE);
-        args->result[2] = gwy_data_field_new_alike(lpfm_0_a, FALSE);
+        args->result[0] = gwy_field_new_alike(lpfm_0_a, FALSE);
+        args->result[1] = gwy_field_new_alike(lpfm_0_a, FALSE);
+        args->result[2] = gwy_field_new_alike(lpfm_0_a, FALSE);
 
-        gwy_data_field_set_si_unit_z(args->result[0], gwy_si_unit_new("m"));
-        gwy_data_field_set_si_unit_z(args->result[1], gwy_si_unit_new("rad"));
-        gwy_data_field_set_si_unit_z(args->result[2], gwy_si_unit_new("rad"));
+        gwy_unit_set_from_string(gwy_field_get_unit_z(args->result[0]), "m");
+        gwy_unit_set_from_string(gwy_field_get_unit_z(args->result[1]), "rad");
+        gwy_unit_set_from_string(gwy_field_get_unit_z(args->result[2]), "rad");
 
-        res0data = gwy_data_field_get_data(args->result[0]);
-        res1data = gwy_data_field_get_data(args->result[1]);
-        res2data = gwy_data_field_get_data(args->result[2]);
+        res0data = gwy_field_get_data(args->result[0]);
+        res1data = gwy_field_get_data(args->result[1]);
+        res2data = gwy_field_get_data(args->result[2]);
 
         for (row = 0; row < yres; row++) {
             for (col = 0; col < xres; col++) {
