@@ -1,5 +1,5 @@
 /*
- *  $Id: pid.c 24365 2021-10-13 12:45:57Z yeti-dn $
+ *  $Id: pid.c 28910 2025-11-24 18:11:09Z yeti-dn $
  *  Copyright (C) 2012-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,16 +16,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <math.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libprocess/inttrans.h>
-#include <libprocess/stats.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <libgwyddion/gwymathfallback.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
@@ -63,26 +57,26 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *result[OUTPUT_NTYPES];
+    GwyField *field;
+    GwyField *result[OUTPUT_NTYPES];
 } ModuleArgs;
 
 typedef struct {
     ModuleArgs *args;
     GtkWidget *dialog;
+    GtkWidget *dataview;
     GwyParamTable *table;
-    GwyContainer *data;
 } ModuleGUI;
 
 static gboolean         module_register          (void);
 static GwyParamDef*     define_module_params     (void);
-static void             pid                      (GwyContainer *data,
-                                                  GwyRunType runtype);
+static void             module_main              (GwyFile *data,
+                                                  GwyRunModeFlags mode);
 static void             warn_computation_diverged(GtkWindow *parent_window);
 static gint             execute                  (ModuleArgs *args,
                                                   GtkWindow *wait_window);
 static GwyDialogOutcome run_gui                  (ModuleArgs *args,
-                                                  GwyContainer *data,
+                                                  GwyFile *data,
                                                   gint id);
 static void             param_changed            (ModuleGUI *gui,
                                                   gint id);
@@ -111,11 +105,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("pid",
-                              (GwyProcessFunc)&pid,
+                              module_main,
                               N_("/SPM M_odes/_Force and Indentation/_PID Simulation..."),
-                              GWY_STOCK_TIP_PID,
+                              GWY_ICON_TIP_PID,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Simulate PID effects on measurement"));
 
     return TRUE;
@@ -146,7 +140,7 @@ define_module_params(void)
     gwy_param_def_add_int(paramdef, PARAM_TAU, "tau", _("_Integration steps"), 2, 1000, 100);
     gwy_param_def_add_double(paramdef, PARAM_FSTRENGTH, "fstrength", _("Force strength"), 0.0, 1000.0, 1.0);
     gwy_param_def_add_double(paramdef, PARAM_FSETPOINT, "fsetpoint", _("Force setpoint"), 0.0, 1000.0, 10.0);
-    gwy_param_def_add_gwyenum(paramdef, PARAM_DISPLAY, "display", gwy_sgettext("verb|Display"),
+    gwy_param_def_add_gwyenum(paramdef, PARAM_DISPLAY, "display", C_("verb", "Display"),
                               displays, G_N_ELEMENTS(displays), DISPLAY_DATA);
     gwy_param_def_add_gwyflags(paramdef, PARAM_OUTPUT, "output", _("Output"),
                                outputs, G_N_ELEMENTS(outputs), (1 << OUTPUT_NTYPES)-1);
@@ -154,35 +148,35 @@ define_module_params(void)
 }
 
 static void
-pid(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     ModuleArgs args;
     gint id, newid;
     guint output, i;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(args.field);
 
     for (i = 0; i < OUTPUT_NTYPES; i++) {
-        args.result[i] = gwy_data_field_new_alike(args.field, TRUE);
+        args.result[i] = gwy_field_new_alike(args.field, TRUE);
         if (i == OUTPUT_FORCE_FWD || i == OUTPUT_FORCE_REV)
-            gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(args.result[i]), NULL);
+            gwy_unit_clear(gwy_field_get_unit_z(args.result[i]));
     }
 
     args.params = gwy_params_new_from_settings(define_module_params());
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, id);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
             goto end;
     }
     if (outcome != GWY_DIALOG_HAVE_RESULT) {
-        GtkWindow *window = gwy_app_find_window_for_channel(data, id);
+        GtkWindow *window = gwy_data_browser_get_window_for_data(data, GWY_FILE_IMAGE, id);
         gint result = execute(&args, window);
 
         if (result != COMPUTATION_OK) {
@@ -197,19 +191,20 @@ pid(GwyContainer *data, GwyRunType runtype)
         if (!((1 << i) & output))
             continue;
 
-        newid = gwy_app_data_browser_add_data_field(args.result[i], data, TRUE);
-        gwy_app_sync_data_items(data, data, id, newid, FALSE,
-                                GWY_DATA_ITEM_GRADIENT,
-                                GWY_DATA_ITEM_REAL_SQUARE,
-                                0);
-        gwy_app_set_data_field_title(data, newid, gwy_enum_to_string(1 << i, outputs, OUTPUT_NTYPES));
-        gwy_app_channel_log_add_proc(data, id, newid);
+        newid = gwy_file_add_image(data, args.result[i]);
+
+        gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+        gwy_file_sync_items(data, GWY_FILE_IMAGE, id,
+                            data, GWY_FILE_IMAGE, newid,
+                            GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_REAL_SQUARE, FALSE);
+        gwy_file_set_title(data, GWY_FILE_IMAGE, newid, gwy_enum_to_string(1 << i, outputs, OUTPUT_NTYPES), TRUE);
+        gwy_log_add(data, GWY_FILE_IMAGE, id, newid);
     }
 
 end:
     g_object_unref(args.params);
     for (i = 0; i < OUTPUT_NTYPES; i++)
-        GWY_OBJECT_UNREF(args.result[i]);
+        g_clear_object(args.result + i);
 }
 
 static void
@@ -217,7 +212,7 @@ warn_computation_diverged(GtkWindow *parent_window)
 {
     GtkWidget *dialog;
 
-    if (!gwy_app_data_browser_get_gui_enabled() && !gwy_app_wait_get_enabled())
+    if (!gwy_data_browser_get_gui_enabled() && !gwy_app_wait_get_enabled())
         return;
 
     dialog = gtk_message_dialog_new(parent_window,
@@ -230,30 +225,23 @@ warn_computation_diverged(GtkWindow *parent_window)
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     GwyDialog *dialog;
     GwyParamTable *table;
     GtkWidget *hbox, *dataview;
-    GwyDialogOutcome outcome;
     ModuleGUI gui;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
 
-    gui.data = gwy_container_new();
-    gwy_container_set_object_by_name(gui.data, "/0/data", args->result[0]);
-    gwy_app_sync_data_items(data, gui.data, id, 0, FALSE,
-                            GWY_DATA_ITEM_PALETTE,
-                            GWY_DATA_ITEM_RANGE,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
-
     gui.dialog = gwy_dialog_new(_("PID Simulation"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_UPDATE, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
+    dataview = gwy_create_preview(args->result[0], NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_RANGE | GWY_FILE_ITEM_REAL_SQUARE);
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), FALSE);
 
     table = gui.table = gwy_param_table_new(args->params);
@@ -278,11 +266,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     g_signal_connect_swapped(table, "param-changed", G_CALLBACK(param_changed), &gui);
     gwy_dialog_set_preview_func(dialog, GWY_PREVIEW_UPON_REQUEST, preview, &gui, NULL);
 
-    outcome = gwy_dialog_run(dialog);
-
-    g_object_unref(gui.data);
-
-    return outcome;
+    return gwy_dialog_run(dialog);
 }
 
 static void
@@ -292,11 +276,12 @@ param_changed(ModuleGUI *gui, gint id)
     GwyParams *params = args->params;
 
     if (id < 0 || id == PARAM_DISPLAY) {
+        GwyDataView *dataview = GWY_DATA_VIEW(gui->dataview);
         gint i = gwy_params_get_enum(params, PARAM_DISPLAY);
         if (i == DISPLAY_DATA)
-            gwy_container_set_object_by_name(gui->data, "/0/data", args->field);
+            gwy_data_view_set_field(dataview, args->field);
         else
-            gwy_container_set_object_by_name(gui->data, "/0/data", args->result[i]);
+            gwy_data_view_set_field(dataview, args->result[i]);
     }
 
     if (id != PARAM_DISPLAY && id != PARAM_OUTPUT)
@@ -311,7 +296,7 @@ preview(gpointer user_data)
     guint i;
 
     for (i = 0; i < OUTPUT_NTYPES; i++)
-        gwy_data_field_data_changed(gui->args->result[i]);
+        gwy_field_data_changed(gui->args->result[i]);
 
     if (result == COMPUTATION_OK) {
         gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
@@ -330,9 +315,9 @@ preview(gpointer user_data)
 static gint
 execute(ModuleArgs *args, GtkWindow *wait_window)
 {
-    GwyDataField *field = args->field;
-    GwyDataField *fw = args->result[OUTPUT_RESULT_FWD], *rev = args->result[OUTPUT_RESULT_REV];
-    GwyDataField *ffw = args->result[OUTPUT_FORCE_FWD], *frev = args->result[OUTPUT_FORCE_REV];
+    GwyField *field = args->field;
+    GwyField *fw = args->result[OUTPUT_RESULT_FWD], *rev = args->result[OUTPUT_RESULT_REV];
+    GwyField *ffw = args->result[OUTPUT_FORCE_FWD], *frev = args->result[OUTPUT_FORCE_REV];
     gdouble proportional = gwy_params_get_double(args->params, PARAM_PROPORTIONAL);
     gdouble integral = gwy_params_get_double(args->params, PARAM_INTEGRAL);
     gdouble derivative = gwy_params_get_double(args->params, PARAM_DERIVATIVE);
@@ -348,22 +333,22 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
 
     gwy_app_wait_start(wait_window, _("Initializing..."));
 
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
-    gwy_data_field_clear(fw);
-    gwy_data_field_clear(ffw);
-    gwy_data_field_clear(rev);
-    gwy_data_field_clear(frev);
-    dfw = gwy_data_field_get_data(fw);
-    dffw = gwy_data_field_get_data(ffw);
-    drev = gwy_data_field_get_data(rev);
-    dfrev = gwy_data_field_get_data(frev);
-    surface = gwy_data_field_get_data_const(field);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
+    gwy_field_clear(fw);
+    gwy_field_clear(ffw);
+    gwy_field_clear(rev);
+    gwy_field_clear(frev);
+    dfw = gwy_field_get_data(fw);
+    dffw = gwy_field_get_data(ffw);
+    drev = gwy_field_get_data(rev);
+    dfrev = gwy_field_get_data(frev);
+    surface = gwy_field_get_data_const(field);
 
     /* Primitive normalisation. */
     zpos = surface[0];
     accumulator = 0;
-    zrange = (gwy_data_field_get_max(field) - gwy_data_field_get_min(field));
+    zrange = (gwy_field_get_max(field) - gwy_field_get_min(field));
     strength = fstrength/zrange;
 
     /* Scan, forming forward and backward data and error signals. */
@@ -436,7 +421,7 @@ end:
 
     if (outcome != COMPUTATION_OK) {
         for (i = 0; i < OUTPUT_NTYPES; i++)
-            gwy_data_field_clear(args->result[i]);
+            gwy_field_clear(args->result[i]);
     }
 
     return outcome;
