@@ -1,6 +1,6 @@
 /*
- *  $Id: polylevel.c 26100 2024-01-04 14:09:03Z yeti-dn $
- *  Copyright (C) 2004-2021 David Necas (Yeti), Petr Klapetek.
+ *  $Id: polylevel.c 28821 2025-11-06 16:00:41Z yeti-dn $
+ *  Copyright (C) 2004-2025 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -16,15 +16,9 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/level.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
@@ -45,29 +39,30 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *mask;
-    GwyDataField *result;
-    GwyDataField *bg;
+    GwyField *field;
+    GwyField *mask;
+    GwyField *result;
+    GwyField *bg;
 } ModuleArgs;
 
 typedef struct {
     ModuleArgs args;    /* This is a copy with downscaled args for the preview. */
     GtkWidget *dialog;
+    GtkWidget *result_view;
+    GtkWidget *bg_view;
     GwyParamTable *table;
     GtkListStore *coeffmodel;
     GtkWidget *coefflist;
-    GwyContainer *data;
 } ModuleGUI;
 
 static gboolean         module_register             (void);
 static GwyParamDef*     define_module_params        (void);
-static void             poly_level                  (GwyContainer *data,
-                                                     GwyRunType runtype);
+static void             module_main                 (GwyFile *data,
+                                                     GwyRunModeFlags mode);
 static void             execute                     (ModuleArgs *args,
                                                      GtkListStore *coeffmodel);
 static GwyDialogOutcome run_gui                     (ModuleArgs *args,
-                                                     GwyContainer *data,
+                                                     GwyFile *data,
                                                      gint id);
 static void             param_changed               (ModuleGUI *gui,
                                                      gint id);
@@ -84,16 +79,14 @@ static void             render_coeff_value          (GtkTreeViewColumn *column,
                                                      GtkTreeModel *model,
                                                      GtkTreeIter *iter,
                                                      gpointer user_data);
-static void             convert_coefficients_to_real(GwyDataField *field,
+static void             convert_coefficients_to_real(GwyField *field,
                                                      GtkListStore *store);
 static gchar*           format_coefficient          (ModuleGUI *gui,
                                                      gint j,
                                                      gint i,
                                                      gdouble v,
-                                                     GwySIUnitFormatStyle style);
-static void             save_coeffs                 (ModuleGUI *gui);
-static void             copy_coeffs                 (ModuleGUI *gui);
-static gchar*           create_report               (ModuleGUI *gui);
+                                                     GwyUnitFormatStyle style);
+static gchar*           create_report               (gpointer user_data);
 static void             sanitise_params             (ModuleArgs *args);
 
 /* We have just two modes distinguished by TRUE/FALSE, but technically it is an enum and we could have more modes. */
@@ -118,11 +111,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("polylevel",
-                              (GwyProcessFunc)&poly_level,
+                              module_main,
                               N_("/_Level/_Polynomial Background..."),
-                              GWY_STOCK_POLYNOM_LEVEL,
+                              GWY_ICON_POLYNOM_LEVEL,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Remove polynomial background"));
 
     return TRUE;
@@ -152,51 +145,54 @@ define_module_params(void)
 }
 
 static void
-poly_level(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GQuark quark;
     ModuleArgs args;
     gboolean do_extract;
     gint oldid, newid;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD_KEY, &quark,
-                                     GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_MASK_FIELD, &args.mask,
-                                     GWY_APP_DATA_FIELD_ID, &oldid,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD_KEY, &quark,
+                                 GWY_APP_FIELD, &args.field,
+                                 GWY_APP_MASK_FIELD, &args.mask,
+                                 GWY_APP_FIELD_ID, &oldid,
+                                 0);
     g_return_if_fail(args.field && quark);
 
     args.bg = args.result = NULL;
     args.params = gwy_params_new_from_settings(define_module_params());
     sanitise_params(&args);
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         GwyDialogOutcome outcome = run_gui(&args, data, oldid);
         gwy_params_save_to_settings(args.params);
         if (outcome != GWY_DIALOG_PROCEED)
             goto end;
     }
-    gwy_app_undo_qcheckpointv(data, 1, &quark);
+    gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &quark);
 
     do_extract = gwy_params_get_boolean(args.params, PARAM_DO_EXTRACT);
     args.result = g_object_ref(args.field);
     if (do_extract)
-        args.bg = gwy_data_field_new_alike(args.field, FALSE);
+        args.bg = gwy_field_new_alike(args.field, FALSE);
 
     execute(&args, NULL);
-    gwy_data_field_data_changed(args.result);
-    gwy_app_channel_log_add_proc(data, oldid, oldid);
+    gwy_field_data_changed(args.result);
+    gwy_log_add(data, GWY_FILE_IMAGE, oldid, oldid);
 
     if (do_extract) {
-        newid = gwy_app_data_browser_add_data_field(args.bg, data, TRUE);
-        gwy_app_sync_data_items(data, data, oldid, newid, FALSE, GWY_DATA_ITEM_GRADIENT, 0);
-        gwy_app_set_data_field_title(data, newid, _("Background"));
-        gwy_app_channel_log_add(data, oldid, newid, NULL, NULL);
+        newid = gwy_file_add_image(data, args.bg);
+        gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+        gwy_file_sync_items(data, GWY_FILE_IMAGE, oldid,
+                            data, GWY_FILE_IMAGE, newid,
+                            GWY_FILE_ITEM_PALETTE, FALSE);
+        gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _("Background"), TRUE);
+        gwy_log_add_full(data, GWY_FILE_IMAGE, oldid, newid, NULL, NULL);
     }
 
 end:
-    GWY_OBJECT_UNREF(args.bg);
+    g_clear_object(&args.bg);
     g_object_unref(args.params);
 }
 
@@ -204,7 +200,7 @@ static void
 execute(ModuleArgs *args, GtkListStore *coeffmodel)
 {
     GwyParams *params = args->params;
-    GwyDataField *field = args->field, *mask = args->mask, *result = args->result, *bg = args->bg;
+    GwyField *field = args->field, *mask = args->mask, *result = args->result, *bg = args->bg;
     GwyMaskingType masking = gwy_params_get_masking(params, PARAM_MASKING, &mask);
     gint max_degree = gwy_params_get_int(params, PARAM_MAX_DEGREE);
     gint col_degree = gwy_params_get_int(params, PARAM_COL_DEGREE);
@@ -236,14 +232,14 @@ execute(ModuleArgs *args, GtkListStore *coeffmodel)
         }
     }
 
-    coeffs = gwy_data_field_fit_poly(field, mask, nterms, term_powers, masking == GWY_MASK_EXCLUDE, NULL);
-    gwy_data_field_copy(field, result, FALSE);
-    gwy_data_field_subtract_poly(result, nterms, term_powers, coeffs);
+    coeffs = gwy_field_fit_poly(field, mask, masking, nterms, term_powers, NULL);
+    gwy_field_copy_data(field, result);
+    gwy_field_subtract_poly(result, nterms, term_powers, coeffs);
 
     if (bg) {
-        gwy_data_field_clear(bg);
-        gwy_data_field_subtract_poly(bg, nterms, term_powers, coeffs);
-        gwy_data_field_multiply(bg, -1.0);
+        gwy_field_clear(bg);
+        gwy_field_subtract_poly(bg, nterms, term_powers, coeffs);
+        gwy_field_multiply(bg, -1.0);
     }
 
     if (coeffmodel) {
@@ -264,26 +260,23 @@ execute(ModuleArgs *args, GtkListStore *coeffmodel)
     g_free(term_powers);
 }
 
-static GwyContainer*
-create_preview_data(GwyContainer *data, gint id,
-                    ModuleArgs *args, ModuleArgs *preview_args)
+static void
+create_preview_data(ModuleArgs *args, ModuleArgs *preview_args)
 {
-    GwyContainer *preview_data;
     gint xres, yres;
     gdouble zoomval;
 
     preview_args->params = args->params;
     preview_args->mask = NULL;
-    preview_data = gwy_container_new();
-    xres = gwy_data_field_get_xres(args->field);
-    yres = gwy_data_field_get_yres(args->field);
+    xres = gwy_field_get_xres(args->field);
+    yres = gwy_field_get_yres(args->field);
     zoomval = (gdouble)PREVIEW_SIZE/MAX(xres, yres);
     if (zoomval <= 1.0) {
         xres = MAX(xres*zoomval, 3);
         yres = MAX(yres*zoomval, 3);
-        preview_args->field = gwy_data_field_new_resampled(args->field, xres, yres, GWY_INTERPOLATION_ROUND);
+        preview_args->field = gwy_field_new_resampled(args->field, xres, yres, GWY_INTERPOLATION_ROUND);
         if (args->mask)
-            preview_args->mask = gwy_data_field_new_resampled(args->mask, xres, yres, GWY_INTERPOLATION_ROUND);
+            preview_args->mask = gwy_field_new_resampled(args->mask, xres, yres, GWY_INTERPOLATION_ROUND);
     }
     else {
         preview_args->field = g_object_ref(args->field);
@@ -291,21 +284,12 @@ create_preview_data(GwyContainer *data, gint id,
             preview_args->mask = g_object_ref(args->mask);
     }
 
-    preview_args->result = gwy_data_field_new_alike(preview_args->field, TRUE);
-    gwy_container_set_object_by_name(preview_data, "/0/data", preview_args->result);
-    g_object_unref(preview_args->result);
-    gwy_app_sync_data_items(data, preview_data, id, 0, FALSE, GWY_DATA_ITEM_GRADIENT, 0);
-
-    preview_args->bg = gwy_data_field_new_alike(preview_args->field, TRUE);
-    gwy_container_set_object_by_name(preview_data, "/1/data", preview_args->bg);
-    g_object_unref(preview_args->bg);
-    gwy_app_sync_data_items(data, preview_data, id, 1, FALSE, GWY_DATA_ITEM_GRADIENT, 0);
-
-    return preview_data;
+    preview_args->result = gwy_field_new_alike(preview_args->field, TRUE);
+    preview_args->bg = gwy_field_new_alike(preview_args->field, TRUE);
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     GwyDialog *dialog;
     GwyParamTable *table;
@@ -313,43 +297,47 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     ModuleGUI gui;
     GwyDialogOutcome outcome;
 
-    gwy_clear(&gui, 1);
-    gui.data = create_preview_data(data, id, args, &gui.args);
+    gwy_clear1(gui);
+    create_preview_data(args, &gui.args);
 
     gui.dialog = gwy_dialog_new(_("Remove Polynomial Background"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    hbox = gwy_hbox_new(0);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
     gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
     gwy_dialog_add_content(dialog, hbox, FALSE, FALSE, 0);
 
-    vbox = gwy_vbox_new(0);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
     gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);
     create_coeff_view(&gui, GTK_BOX(hbox));
 
-    hbox = gwy_hbox_new(0);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 
-    preview_table = gtk_table_new(2, 2, FALSE);
-    gtk_table_set_row_spacings(GTK_TABLE(preview_table), 2);
-    gtk_table_set_col_spacings(GTK_TABLE(preview_table), 6);
+    preview_table = gtk_grid_new();
+    gtk_grid_set_row_spacing(GTK_GRID(preview_table), 2);
+    gtk_grid_set_column_spacing(GTK_GRID(preview_table), 6);
     gtk_container_set_border_width(GTK_CONTAINER(preview_table), 4);
     gtk_box_pack_start(GTK_BOX(hbox), preview_table, FALSE, FALSE, 0);
 
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_HALF_SIZE, FALSE);
-    gtk_table_attach(GTK_TABLE(preview_table), dataview, 0, 1, 0, 1, 0, 0, 0, 0);
+    dataview = gui.result_view = gwy_create_preview(gui.args.result, NULL, PREVIEW_HALF_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_REAL_SQUARE);
+    gtk_grid_attach(GTK_GRID(preview_table), dataview, 0, 0, 1, 1);
 
-    dataview = gwy_create_preview(gui.data, 1, PREVIEW_HALF_SIZE, FALSE);
-    gtk_table_attach(GTK_TABLE(preview_table), dataview, 1, 2, 0, 1, 0, 0, 0, 0);
+    dataview = gui.bg_view = gwy_create_preview(gui.args.bg, NULL, PREVIEW_HALF_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_REAL_SQUARE);
+    gtk_grid_attach(GTK_GRID(preview_table), dataview, 1, 0, 1, 1);
 
     label = gtk_label_new(_("Leveled data"));
-    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
-    gtk_table_attach(GTK_TABLE(preview_table), label, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
+    gtk_label_set_xalign(GTK_LABEL(label), 0.0);
+    gtk_grid_attach(GTK_GRID(preview_table), label, 0, 1, 1, 1);
 
     label = gtk_label_new(_("Background"));
-    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
-    gtk_table_attach(GTK_TABLE(preview_table), label, 1, 2, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
+    gtk_label_set_xalign(GTK_LABEL(label), 0.0);
+    gtk_grid_attach(GTK_GRID(preview_table), label, 1, 1, 1, 1);
 
     table = gui.table = gwy_param_table_new(args->params);
     gwy_param_table_append_radio_item(table, PARAM_INDEPENDENT, TRUE);
@@ -373,8 +361,9 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_dialog_set_preview_func(dialog, GWY_PREVIEW_IMMEDIATE, preview, &gui, NULL);
     outcome = gwy_dialog_run(dialog);
 
-    g_object_unref(gui.data);
     g_object_unref(gui.coeffmodel);
+    g_object_unref(gui.args.result);
+    g_object_unref(gui.args.bg);
 
     return outcome;
 }
@@ -382,17 +371,17 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 static void
 create_coeff_view(ModuleGUI *gui, GtkBox *hbox)
 {
-    GtkWidget *treeview, *button, *hbox2, *scwin, *label, *coeffvbox;
+    GtkWidget *treeview, *rexport, *scwin, *label, *coeffvbox;
     GtkTreeViewColumn *column;
     GtkCellRenderer *renderer;
     GtkTreeSelection *selection;
 
-    coeffvbox = gtk_vbox_new(FALSE, 0);
+    coeffvbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
     gtk_box_pack_start(hbox, coeffvbox, FALSE, FALSE, 0);
     gtk_container_set_border_width(GTK_CONTAINER(coeffvbox), 4);
 
     label = gtk_label_new(_("Polynomial Coefficients"));
-    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+    gtk_label_set_xalign(GTK_LABEL(label), 0.0);
     gtk_box_pack_start(GTK_BOX(coeffvbox), label, FALSE, FALSE, 0);
 
     scwin = gtk_scrolled_window_new(NULL, NULL);
@@ -424,22 +413,10 @@ create_coeff_view(ModuleGUI *gui, GtkBox *hbox)
     gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(column), renderer, TRUE);
     gtk_tree_view_column_set_cell_data_func(column, renderer, render_coeff_value, gui, NULL);
 
-    hbox2 = gwy_hbox_new(0);
-    gtk_box_pack_start(GTK_BOX(coeffvbox), hbox2, FALSE, FALSE, 0);
-
-    button = gtk_button_new();
-    gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
-    gtk_widget_set_tooltip_text(button, _("Save table to a file"));
-    gtk_container_add(GTK_CONTAINER(button), gtk_image_new_from_stock(GTK_STOCK_SAVE, GTK_ICON_SIZE_SMALL_TOOLBAR));
-    gtk_box_pack_end(GTK_BOX(hbox2), button, FALSE, FALSE, 0);
-    g_signal_connect_swapped(button, "clicked", G_CALLBACK(save_coeffs), gui);
-
-    button = gtk_button_new();
-    gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
-    gtk_widget_set_tooltip_text(button, _("Copy table to clipboard"));
-    gtk_container_add(GTK_CONTAINER(button), gtk_image_new_from_stock(GTK_STOCK_COPY, GTK_ICON_SIZE_SMALL_TOOLBAR));
-    gtk_box_pack_end(GTK_BOX(hbox2), button, FALSE, FALSE, 0);
-    g_signal_connect_swapped(button, "clicked", G_CALLBACK(copy_coeffs), gui);
+    rexport = gwy_results_export_new(GWY_RESULTS_REPORT_TABSEP | GWY_RESULTS_REPORT_MACHINE);
+    gwy_results_export_set_style(GWY_RESULTS_EXPORT(rexport), GWY_RESULTS_EXPORT_FIXED_FORMAT);
+    gwy_results_export_set_format_func(GWY_RESULTS_EXPORT(rexport), create_report, gui);
+    gtk_box_pack_start(GTK_BOX(coeffvbox), rexport, FALSE, FALSE, 0);
 }
 
 static void
@@ -470,7 +447,7 @@ render_coeff_value(G_GNUC_UNUSED GtkTreeViewColumn *column,
     gchar *buf;
 
     gtk_tree_model_get(model, iter, 0, &j, 1, &i, 2, &v, -1);
-    buf = format_coefficient(gui, j, i, v, GWY_SI_UNIT_FORMAT_VFMARKUP);
+    buf = format_coefficient(gui, j, i, v, GWY_UNIT_FORMAT_VFMARKUP);
     g_object_set(renderer, "markup", buf, NULL);
     g_free(buf);
 }
@@ -518,12 +495,12 @@ preview(gpointer user_data)
     gtk_tree_view_set_model(GTK_TREE_VIEW(gui->coefflist), NULL);
     execute(&gui->args, gui->coeffmodel);
     gtk_tree_view_set_model(GTK_TREE_VIEW(gui->coefflist), GTK_TREE_MODEL(gui->coeffmodel));
-    gwy_data_field_data_changed(gui->args.result);
-    gwy_data_field_data_changed(gui->args.bg);
+    gwy_field_data_changed(gui->args.result);
+    gwy_field_data_changed(gui->args.bg);
 }
 
 static void
-convert_coefficients_to_real(GwyDataField *field,
+convert_coefficients_to_real(GwyField *field,
                              GtkListStore *store)
 {
     gdouble cx = field->xoff + 0.5*field->xreal,
@@ -591,46 +568,23 @@ convert_coefficients_to_real(GwyDataField *field,
 static gchar*
 format_coefficient(ModuleGUI *gui,
                    gint j, gint i, gdouble v,
-                   GwySIUnitFormatStyle style)
+                   GwyUnitFormatStyle style)
 {
-    GwySIUnit *zunit = gwy_data_field_get_si_unit_z(gui->args.field),
-              *xyunit = gwy_data_field_get_si_unit_xy(gui->args.field);
-    GwySIUnit *unit = gwy_si_unit_power_multiply(zunit, 1, xyunit, -(i + j), NULL);
-    GwySIValueFormat *vf = gwy_si_unit_get_format_with_digits(unit, style, fabs(v), 4, NULL);
+    GwyUnit *zunit = gwy_field_get_unit_z(gui->args.field),
+              *xyunit = gwy_field_get_unit_xy(gui->args.field);
+    GwyUnit *unit = gwy_unit_power_multiply(zunit, 1, xyunit, -(i + j), NULL);
+    GwyValueFormat *vf = gwy_unit_get_format_with_digits(unit, style, fabs(v), 4, NULL);
     gchar *retval = g_strdup_printf("%.*f%s%s", vf->precision, v/vf->magnitude, *vf->units ? " " : "", vf->units);
-    gwy_si_unit_value_format_free(vf);
+    gwy_value_format_free(vf);
     g_object_unref(unit);
 
     return retval;
 }
 
-static void
-save_coeffs(ModuleGUI *gui)
-{
-    gchar *text;
-
-    text = create_report(gui);
-    gwy_save_auxiliary_data(_("Save Table"), GTK_WINDOW(gui->dialog), -1, text);
-    g_free(text);
-}
-
-static void
-copy_coeffs(ModuleGUI *gui)
-{
-    GtkClipboard *clipboard;
-    GdkDisplay *display;
-    gchar *text;
-
-    text = create_report(gui);
-    display = gtk_widget_get_display(gui->dialog);
-    clipboard = gtk_clipboard_get_for_display(display, GDK_SELECTION_CLIPBOARD);
-    gtk_clipboard_set_text(clipboard, text, -1);
-    g_free(text);
-}
-
 static gchar*
-create_report(ModuleGUI *gui)
+create_report(gpointer user_data)
 {
+    ModuleGUI *gui = (ModuleGUI*)user_data;
     GtkTreeIter iter;
     GString *text;
     GtkTreeModel *model = GTK_TREE_MODEL(gui->coeffmodel);
@@ -644,7 +598,7 @@ create_report(ModuleGUI *gui)
         gdouble v;
         gchar *buf;
         gtk_tree_model_get(model, &iter, 0, &j, 1, &i, 2, &v, -1);
-        buf = format_coefficient(gui, j, i, v, GWY_SI_UNIT_FORMAT_PLAIN);
+        buf = format_coefficient(gui, j, i, v, GWY_UNIT_FORMAT_PLAIN);
         g_string_append_printf(text, "a[%u,%u] = %s\n", j, i, buf);
         g_free(buf);
     } while (gtk_tree_model_iter_next(model, &iter));
