@@ -1,5 +1,5 @@
 /*
- *  $Id: presentationops.c 25638 2023-09-07 12:23:28Z yeti-dn $
+ *  $Id: presentationops.c 28860 2025-11-14 12:59:38Z yeti-dn $
  *  Copyright (C) 2003-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,17 +16,12 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/filters.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 
-#define PRESENTATIONOPS_RUN_MODES GWY_RUN_IMMEDIATE
-#define PRESENTATION_ATTACH_RUN_MODES GWY_RUN_INTERACTIVE
+#define RUN_MODES GWY_RUN_IMMEDIATE
+#define ATTACH_RUN_MODES GWY_RUN_INTERACTIVE
 
 enum {
     PARAM_SOURCE,
@@ -34,21 +29,21 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *target;
+    GwyField *target;
 } ModuleArgs;
 
 static gboolean         module_register      (void);
 static GwyParamDef*     define_attach_params (void);
-static void             presentation_remove  (GwyContainer *data,
-                                              GwyRunType runtype);
-static void             presentation_extract (GwyContainer *data,
-                                              GwyRunType runtype);
-static void             presentation_logscale(GwyContainer *data,
-                                              GwyRunType runtype);
-static void             presentation_attach  (GwyContainer *data,
-                                              GwyRunType runtype);
+static void             presentation_remove  (GwyFile *data,
+                                              GwyRunModeFlags mode);
+static void             presentation_extract (GwyFile *data,
+                                              GwyRunModeFlags mode);
+static void             presentation_logscale(GwyFile *data,
+                                              GwyRunModeFlags mode);
+static void             presentation_attach  (GwyFile *data,
+                                              GwyRunModeFlags mode);
 static GwyDialogOutcome run_attach_gui       (ModuleArgs *args);
-static gboolean         attach_source_filter (GwyContainer *source,
+static gboolean         attach_source_filter (GwyFile *source,
                                               gint id,
                                               gpointer user_data);
 
@@ -82,105 +77,108 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("presentation_remove",
-                              (GwyProcessFunc)&presentation_remove,
+                              &presentation_remove,
                               N_("/_Presentation/_Remove Presentation"),
                               NULL,
-                              PRESENTATIONOPS_RUN_MODES,
-                              GWY_MENU_FLAG_DATA_SHOW | GWY_MENU_FLAG_DATA,
+                              RUN_MODES,
+                              GWY_MENU_FLAG_IMAGE_SHOW | GWY_MENU_FLAG_IMAGE,
                               N_("Remove presentation from data"));
     gwy_process_func_register("presentation_extract",
-                              (GwyProcessFunc)&presentation_extract,
+                              &presentation_extract,
                               N_("/_Presentation/E_xtract Presentation"),
                               NULL,
-                              PRESENTATIONOPS_RUN_MODES,
-                              GWY_MENU_FLAG_DATA_SHOW | GWY_MENU_FLAG_DATA,
+                              RUN_MODES,
+                              GWY_MENU_FLAG_IMAGE_SHOW | GWY_MENU_FLAG_IMAGE,
                               N_("Extract presentation to a new image"));
     gwy_process_func_register("presentation_attach",
-                              (GwyProcessFunc)&presentation_attach,
+                              &presentation_attach,
                               N_("/_Presentation/_Attach Presentation..."),
                               NULL,
-                              PRESENTATION_ATTACH_RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              ATTACH_RUN_MODES,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Attach another data field as presentation"));
     gwy_process_func_register("presentation_logscale",
-                              (GwyProcessFunc)&presentation_logscale,
+                              &presentation_logscale,
                               N_("/_Presentation/_Logscale"),
                               NULL,
-                              PRESENTATIONOPS_RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              RUN_MODES,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Creates a presentation with logarithmic color scale"));
 
     return TRUE;
 }
 
 static void
-presentation_remove(GwyContainer *data, GwyRunType runtype)
+presentation_remove(GwyFile *data, GwyRunModeFlags mode)
 {
     GQuark quark;
     gint id;
 
-    g_return_if_fail(runtype & PRESENTATIONOPS_RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_SHOW_FIELD_KEY, &quark,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_SHOW_FIELD_KEY, &quark,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(quark);
-    gwy_app_undo_qcheckpointv(data, 1, &quark);
-    gwy_container_remove(data, quark);
-    gwy_app_channel_log_add_proc(data, id, id);
+    gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &quark);
+    gwy_container_remove(GWY_CONTAINER(data), quark);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, id);
 }
 
 static void
-presentation_extract(GwyContainer *data, GwyRunType runtype)
+presentation_extract(GwyFile *data, GwyRunModeFlags mode)
 {
-    GwyDataField *dfield;
+    GwyField *dfield;
     GQuark quark;
     gint oldid, newid;
 
-    g_return_if_fail(runtype & PRESENTATIONOPS_RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD_ID, &oldid,
-                                     GWY_APP_SHOW_FIELD_KEY, &quark,
-                                     GWY_APP_SHOW_FIELD, &dfield,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD_ID, &oldid,
+                                 GWY_APP_SHOW_FIELD_KEY, &quark,
+                                 GWY_APP_SHOW_FIELD, &dfield,
+                                 0);
     g_return_if_fail(dfield && quark);
 
-    dfield = gwy_data_field_duplicate(dfield);
-    newid = gwy_app_data_browser_add_data_field(dfield, data, TRUE);
+    dfield = gwy_field_copy(dfield);
+    newid = gwy_file_add_image(data, dfield);
+    gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
     g_object_unref(dfield);
-    gwy_app_sync_data_items(data, data, oldid, newid, FALSE, GWY_DATA_ITEM_GRADIENT, 0);
-    gwy_app_set_data_field_title(data, newid, NULL);
-    gwy_app_channel_log_add_proc(data, oldid, newid);
+    gwy_file_sync_items(data, GWY_FILE_IMAGE, oldid,
+                        data, GWY_FILE_IMAGE, newid,
+                        GWY_FILE_ITEM_PALETTE, FALSE);
+    gwy_file_set_title(data, GWY_FILE_IMAGE, newid, NULL, TRUE);
+    gwy_log_add(data, GWY_FILE_IMAGE, oldid, newid);
 }
 
 static void
-presentation_logscale(GwyContainer *data, GwyRunType runtype)
+presentation_logscale(GwyFile *data, GwyRunModeFlags mode)
 {
-    GwyDataField *dfield, *sfield;
+    GwyField *dfield, *sfield;
     GQuark squark;
     gdouble *d;
     gdouble min, max, m0;
     gint xres, yres, i, zeroes, id;
 
-    g_return_if_fail(runtype & PRESENTATIONOPS_RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &dfield,
-                                     GWY_APP_SHOW_FIELD_KEY, &squark,
-                                     GWY_APP_SHOW_FIELD, &sfield,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &dfield,
+                                 GWY_APP_SHOW_FIELD_KEY, &squark,
+                                 GWY_APP_SHOW_FIELD, &sfield,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(dfield && squark);
 
-    xres = gwy_data_field_get_xres(dfield);
-    yres = gwy_data_field_get_yres(dfield);
-    gwy_app_undo_qcheckpointv(data, 1, &squark);
+    xres = gwy_field_get_xres(dfield);
+    yres = gwy_field_get_yres(dfield);
+    gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &squark);
     if (!sfield) {
-        sfield = gwy_data_field_duplicate(dfield);
-        gwy_container_pass_object(data, squark, sfield);
+        sfield = gwy_field_copy(dfield);
+        gwy_container_pass_object(GWY_CONTAINER(data), gwy_file_key_image_picture(id), sfield);
     }
     else {
-        gwy_data_field_resample(sfield, xres, yres, GWY_INTERPOLATION_NONE);
-        gwy_data_field_copy(dfield, sfield, FALSE);
+        gwy_field_resize(sfield, xres, yres);
+        gwy_field_copy_data(dfield, sfield);
     }
 
-    d = gwy_data_field_get_data(sfield);
+    d = gwy_field_get_data(sfield);
     zeroes = 0;
     max = 0;
     min = G_MAXDOUBLE;
@@ -206,24 +204,24 @@ presentation_logscale(GwyContainer *data, GwyRunType runtype)
             d[i] = d[i] ? log(d[i]) : m0;
     }
 
-    gwy_data_field_data_changed(sfield);
-    gwy_app_channel_log_add_proc(data, id, id);
+    gwy_field_data_changed(sfield);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, id);
 }
 
 static void
-presentation_attach(GwyContainer *data, GwyRunType runtype)
+presentation_attach(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome;
-    GwyDataField *dfield;
+    GwyField *dfield;
     ModuleArgs args;
     GQuark squark;
     gint id;
 
-    g_return_if_fail(runtype & PRESENTATION_ATTACH_RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.target,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     GWY_APP_SHOW_FIELD_KEY, &squark,
-                                     0);
+    g_return_if_fail(mode & ATTACH_RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.target,
+                                 GWY_APP_FIELD_ID, &id,
+                                 GWY_APP_SHOW_FIELD_KEY, &squark,
+                                 0);
 
     args.params = gwy_params_new_from_settings(define_attach_params());
     outcome = run_attach_gui(&args);
@@ -231,9 +229,9 @@ presentation_attach(GwyContainer *data, GwyRunType runtype)
     if (outcome == GWY_DIALOG_CANCEL)
         goto end;
 
-    dfield = gwy_data_field_duplicate(args.target);
-    gwy_app_undo_qcheckpointv(data, 1, &squark);
-    gwy_container_pass_object(data, squark, dfield);
+    dfield = gwy_field_copy(args.target);
+    gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &squark);
+    gwy_container_pass_object(GWY_CONTAINER(data), gwy_file_key_image_picture(id), dfield);
 
 end:
     g_object_unref(args.params);
@@ -259,15 +257,14 @@ run_attach_gui(ModuleArgs *args)
 }
 
 static gboolean
-attach_source_filter(GwyContainer *data, gint id, gpointer user_data)
+attach_source_filter(GwyFile *data, gint id, gpointer user_data)
 {
-    GwyDataField *source, *target = (GwyDataField*)user_data;
+    GwyField *source, *target = (GwyField*)user_data;
 
-    source = gwy_container_get_object(data, gwy_app_get_data_key_for_id(id));
-    return !gwy_data_field_check_compatibility(source, target,
-                                               GWY_DATA_COMPATIBILITY_RES
-                                               | GWY_DATA_COMPATIBILITY_REAL
-                                               | GWY_DATA_COMPATIBILITY_LATERAL);
+    if (!(source = gwy_file_get_image(data, id)))
+        return FALSE;
+    return !gwy_field_is_incompatible(source, target,
+                                      GWY_DATA_MISMATCH_RES | GWY_DATA_MISMATCH_REAL | GWY_DATA_MISMATCH_LATERAL);
 }
 
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
