@@ -1,6 +1,6 @@
 /* This is a GENERATED file. */
 #include <glib.h>
-#include <libgwymodule/gwymodule.h>
+#include <libgwyapp/gwyapp.h>
 
 static const GwyModuleRecord* register_bundle(void);
 
@@ -12,42 +12,27 @@ static GwyModuleInfo module_info = {
 
 GWY_MODULE_QUERY(module_info)
 
-GwyModuleInfo* _gwy_module_query__smm(void);
-GwyModuleInfo* _gwy_module_query__smm_apply(void);
 GwyModuleInfo* _gwy_module_query__acf2d(void);
 GwyModuleInfo* _gwy_module_query__angle_dist(void);
-GwyModuleInfo* _gwy_module_query__anneal_synth(void);
 GwyModuleInfo* _gwy_module_query__arc_revolve(void);
 GwyModuleInfo* _gwy_module_query__arithmetic(void);
 GwyModuleInfo* _gwy_module_query__averaging(void);
 GwyModuleInfo* _gwy_module_query__basicops(void);
-GwyModuleInfo* _gwy_module_query__bdep_synth(void);
 GwyModuleInfo* _gwy_module_query__binning(void);
 GwyModuleInfo* _gwy_module_query__blockstep(void);
-GwyModuleInfo* _gwy_module_query__calcoefs_load(void);
-GwyModuleInfo* _gwy_module_query__calcoefs_new(void);
-GwyModuleInfo* _gwy_module_query__calcoefs_simple(void);
-GwyModuleInfo* _gwy_module_query__calcoefs_view(void);
 GwyModuleInfo* _gwy_module_query__calibrate(void);
 GwyModuleInfo* _gwy_module_query__classify(void);
 GwyModuleInfo* _gwy_module_query__coerce(void);
-GwyModuleInfo* _gwy_module_query__col_synth(void);
 GwyModuleInfo* _gwy_module_query__convolution_filter(void);
 GwyModuleInfo* _gwy_module_query__convolve(void);
 GwyModuleInfo* _gwy_module_query__correct_affine(void);
 GwyModuleInfo* _gwy_module_query__correct_perspective(void);
-GwyModuleInfo* _gwy_module_query__cpde_synth(void);
 GwyModuleInfo* _gwy_module_query__crosscor(void);
 GwyModuleInfo* _gwy_module_query__curvature(void);
 GwyModuleInfo* _gwy_module_query__cwt(void);
 GwyModuleInfo* _gwy_module_query__deconvolve(void);
 GwyModuleInfo* _gwy_module_query__displfield(void);
-GwyModuleInfo* _gwy_module_query__deposit_synth(void);
-GwyModuleInfo* _gwy_module_query__diff_synth(void);
-GwyModuleInfo* _gwy_module_query__disc_synth(void);
-GwyModuleInfo* _gwy_module_query__domain_synth(void);
 GwyModuleInfo* _gwy_module_query__drift(void);
-GwyModuleInfo* _gwy_module_query__dune_synth(void);
 GwyModuleInfo* _gwy_module_query__dwt(void);
 GwyModuleInfo* _gwy_module_query__dwtanisotropy(void);
 GwyModuleInfo* _gwy_module_query__edge(void);
@@ -57,13 +42,10 @@ GwyModuleInfo* _gwy_module_query__extract_path(void);
 GwyModuleInfo* _gwy_module_query__facet_level(void);
 GwyModuleInfo* _gwy_module_query__facet_analysis(void);
 GwyModuleInfo* _gwy_module_query__facet_measure(void);
-GwyModuleInfo* _gwy_module_query__fbm_synth(void);
 GwyModuleInfo* _gwy_module_query__fft(void);
 GwyModuleInfo* _gwy_module_query__fft_filter_1d(void);
 GwyModuleInfo* _gwy_module_query__fft_filter_2d(void);
 GwyModuleInfo* _gwy_module_query__psdf2d(void);
-GwyModuleInfo* _gwy_module_query__fft_synth(void);
-GwyModuleInfo* _gwy_module_query__fibre_synth(void);
 GwyModuleInfo* _gwy_module_query__fit_shape(void);
 GwyModuleInfo* _gwy_module_query__flatten_base(void);
 GwyModuleInfo* _gwy_module_query__fraccor(void);
@@ -74,8 +56,8 @@ GwyModuleInfo* _gwy_module_query__good_profile(void);
 GwyModuleInfo* _gwy_module_query__gradient(void);
 GwyModuleInfo* _gwy_module_query__grain_cross(void);
 GwyModuleInfo* _gwy_module_query__grain_dist(void);
-GwyModuleInfo* _gwy_module_query__grain_edge(void);
 GwyModuleInfo* _gwy_module_query__grain_filter(void);
+GwyModuleInfo* _gwy_module_query__grain_edge(void);
 GwyModuleInfo* _gwy_module_query__grain_makesel(void);
 GwyModuleInfo* _gwy_module_query__grain_mark(void);
 GwyModuleInfo* _gwy_module_query__grain_stat(void);
@@ -87,12 +69,10 @@ GwyModuleInfo* _gwy_module_query__immerse(void);
 GwyModuleInfo* _gwy_module_query__indent_analyze(void);
 GwyModuleInfo* _gwy_module_query__laplace(void);
 GwyModuleInfo* _gwy_module_query__latsim(void);
-GwyModuleInfo* _gwy_module_query__lat_synth(void);
 GwyModuleInfo* _gwy_module_query__level(void);
 GwyModuleInfo* _gwy_module_query__level_grains(void);
 GwyModuleInfo* _gwy_module_query__linecorrect(void);
 GwyModuleInfo* _gwy_module_query__linematch(void);
-GwyModuleInfo* _gwy_module_query__lno_synth(void);
 GwyModuleInfo* _gwy_module_query__local_contrast(void);
 GwyModuleInfo* _gwy_module_query__logistic(void);
 GwyModuleInfo* _gwy_module_query__mark_disconn(void);
@@ -108,26 +88,18 @@ GwyModuleInfo* _gwy_module_query__mcrop(void);
 GwyModuleInfo* _gwy_module_query__measure_lattice(void);
 GwyModuleInfo* _gwy_module_query__median_bg(void);
 GwyModuleInfo* _gwy_module_query__merge(void);
-GwyModuleInfo* _gwy_module_query__mfm_current(void);
 GwyModuleInfo* _gwy_module_query__mfm_field(void);
 GwyModuleInfo* _gwy_module_query__mfm_findshift(void);
-GwyModuleInfo* _gwy_module_query__mfm_parallel(void);
 GwyModuleInfo* _gwy_module_query__mfm_recalc(void);
 GwyModuleInfo* _gwy_module_query__mfm_shift(void);
 GwyModuleInfo* _gwy_module_query__multiprofile(void);
 GwyModuleInfo* _gwy_module_query__neural(void);
-GwyModuleInfo* _gwy_module_query__noise_synth(void);
-GwyModuleInfo* _gwy_module_query__obj_synth(void);
 GwyModuleInfo* _gwy_module_query__otsu_threshold(void);
 GwyModuleInfo* _gwy_module_query__outliers(void);
-GwyModuleInfo* _gwy_module_query__pat_synth(void);
 GwyModuleInfo* _gwy_module_query__pfm(void);
-GwyModuleInfo* _gwy_module_query__phase_synth(void);
 GwyModuleInfo* _gwy_module_query__pid(void);
-GwyModuleInfo* _gwy_module_query__pileup_synth(void);
-GwyModuleInfo* _gwy_module_query__plateau_synth(void);
-GwyModuleInfo* _gwy_module_query__polylevel(void);
 GwyModuleInfo* _gwy_module_query__polydistort(void);
+GwyModuleInfo* _gwy_module_query__polylevel(void);
 GwyModuleInfo* _gwy_module_query__presentationops(void);
 GwyModuleInfo* _gwy_module_query__psdf_logphi(void);
 GwyModuleInfo* _gwy_module_query__psf(void);
@@ -139,14 +111,14 @@ GwyModuleInfo* _gwy_module_query__raveraging(void);
 GwyModuleInfo* _gwy_module_query__relate(void);
 GwyModuleInfo* _gwy_module_query__reorder(void);
 GwyModuleInfo* _gwy_module_query__resample(void);
-GwyModuleInfo* _gwy_module_query__residue_synth(void);
-GwyModuleInfo* _gwy_module_query__roddeposit_synth(void);
 GwyModuleInfo* _gwy_module_query__rotate(void);
 GwyModuleInfo* _gwy_module_query__semsim(void);
 GwyModuleInfo* _gwy_module_query__scale(void);
 GwyModuleInfo* _gwy_module_query__scars(void);
 GwyModuleInfo* _gwy_module_query__shade(void);
 GwyModuleInfo* _gwy_module_query__slope_dist(void);
+GwyModuleInfo* _gwy_module_query__smm(void);
+GwyModuleInfo* _gwy_module_query__smm_apply(void);
 GwyModuleInfo* _gwy_module_query__sphere_revolve(void);
 GwyModuleInfo* _gwy_module_query__stitch(void);
 GwyModuleInfo* _gwy_module_query__straighten_path(void);
@@ -160,8 +132,6 @@ GwyModuleInfo* _gwy_module_query__tipshape(void);
 GwyModuleInfo* _gwy_module_query__trimmed_mean(void);
 GwyModuleInfo* _gwy_module_query__volumize(void);
 GwyModuleInfo* _gwy_module_query__volumize_layers(void);
-GwyModuleInfo* _gwy_module_query__wave_synth(void);
-GwyModuleInfo* _gwy_module_query__wfr_synth(void);
 GwyModuleInfo* _gwy_module_query__wpour_mark(void);
 GwyModuleInfo* _gwy_module_query__wrapvalue(void);
 GwyModuleInfo* _gwy_module_query__xydenoise(void);
@@ -170,42 +140,27 @@ GwyModuleInfo* _gwy_module_query__xyzize(void);
 GwyModuleInfo* _gwy_module_query__zero_crossing(void);
 
 static const GwyModuleRecord modules[] = {
-  { _gwy_module_query__smm, "smm", },
-  { _gwy_module_query__smm_apply, "smm_apply", },
   { _gwy_module_query__acf2d, "acf2d", },
   { _gwy_module_query__angle_dist, "angle_dist", },
-  { _gwy_module_query__anneal_synth, "anneal_synth", },
   { _gwy_module_query__arc_revolve, "arc-revolve", },
   { _gwy_module_query__arithmetic, "arithmetic", },
   { _gwy_module_query__averaging, "averaging", },
   { _gwy_module_query__basicops, "basicops", },
-  { _gwy_module_query__bdep_synth, "bdep_synth", },
   { _gwy_module_query__binning, "binning", },
   { _gwy_module_query__blockstep, "blockstep", },
-  { _gwy_module_query__calcoefs_load, "calcoefs_load", },
-  { _gwy_module_query__calcoefs_new, "calcoefs_new", },
-  { _gwy_module_query__calcoefs_simple, "calcoefs_simple", },
-  { _gwy_module_query__calcoefs_view, "calcoefs_view", },
   { _gwy_module_query__calibrate, "calibrate", },
   { _gwy_module_query__classify, "classify", },
   { _gwy_module_query__coerce, "coerce", },
-  { _gwy_module_query__col_synth, "col_synth", },
   { _gwy_module_query__convolution_filter, "convolution_filter", },
   { _gwy_module_query__convolve, "convolve", },
   { _gwy_module_query__correct_affine, "correct_affine", },
   { _gwy_module_query__correct_perspective, "correct_perspective", },
-  { _gwy_module_query__cpde_synth, "cpde_synth", },
   { _gwy_module_query__crosscor, "crosscor", },
   { _gwy_module_query__curvature, "curvature", },
   { _gwy_module_query__cwt, "cwt", },
   { _gwy_module_query__deconvolve, "deconvolve", },
   { _gwy_module_query__displfield, "displfield", },
-  { _gwy_module_query__deposit_synth, "deposit_synth", },
-  { _gwy_module_query__diff_synth, "diff_synth", },
-  { _gwy_module_query__disc_synth, "disc_synth", },
-  { _gwy_module_query__domain_synth, "domain_synth", },
   { _gwy_module_query__drift, "drift", },
-  { _gwy_module_query__dune_synth, "dune_synth", },
   { _gwy_module_query__dwt, "dwt", },
   { _gwy_module_query__dwtanisotropy, "dwtanisotropy", },
   { _gwy_module_query__edge, "edge", },
@@ -215,13 +170,10 @@ static const GwyModuleRecord modules[] = {
   { _gwy_module_query__facet_level, "facet-level", },
   { _gwy_module_query__facet_analysis, "facet_analysis", },
   { _gwy_module_query__facet_measure, "facet_measure", },
-  { _gwy_module_query__fbm_synth, "fbm_synth", },
   { _gwy_module_query__fft, "fft", },
   { _gwy_module_query__fft_filter_1d, "fft_filter_1d", },
   { _gwy_module_query__fft_filter_2d, "fft_filter_2d", },
   { _gwy_module_query__psdf2d, "psdf2d", },
-  { _gwy_module_query__fft_synth, "fft_synth", },
-  { _gwy_module_query__fibre_synth, "fibre_synth", },
   { _gwy_module_query__fit_shape, "fit-shape", },
   { _gwy_module_query__flatten_base, "flatten_base", },
   { _gwy_module_query__fraccor, "fraccor", },
@@ -232,8 +184,8 @@ static const GwyModuleRecord modules[] = {
   { _gwy_module_query__gradient, "gradient", },
   { _gwy_module_query__grain_cross, "grain_cross", },
   { _gwy_module_query__grain_dist, "grain_dist", },
-  { _gwy_module_query__grain_edge, "grain_edge", },
   { _gwy_module_query__grain_filter, "grain_filter", },
+  { _gwy_module_query__grain_edge, "grain_edge", },
   { _gwy_module_query__grain_makesel, "grain_makesel", },
   { _gwy_module_query__grain_mark, "grain_mark", },
   { _gwy_module_query__grain_stat, "grain_stat", },
@@ -245,12 +197,10 @@ static const GwyModuleRecord modules[] = {
   { _gwy_module_query__indent_analyze, "indent_analyze", },
   { _gwy_module_query__laplace, "laplace", },
   { _gwy_module_query__latsim, "latsim", },
-  { _gwy_module_query__lat_synth, "lat_synth", },
   { _gwy_module_query__level, "level", },
   { _gwy_module_query__level_grains, "level_grains", },
   { _gwy_module_query__linecorrect, "linecorrect", },
   { _gwy_module_query__linematch, "linematch", },
-  { _gwy_module_query__lno_synth, "lno_synth", },
   { _gwy_module_query__local_contrast, "local_contrast", },
   { _gwy_module_query__logistic, "logistic", },
   { _gwy_module_query__mark_disconn, "mark_disconn", },
@@ -266,26 +216,18 @@ static const GwyModuleRecord modules[] = {
   { _gwy_module_query__measure_lattice, "measure_lattice", },
   { _gwy_module_query__median_bg, "median-bg", },
   { _gwy_module_query__merge, "merge", },
-  { _gwy_module_query__mfm_current, "mfm_current", },
   { _gwy_module_query__mfm_field, "mfm_field", },
   { _gwy_module_query__mfm_findshift, "mfm_findshift", },
-  { _gwy_module_query__mfm_parallel, "mfm_parallel", },
   { _gwy_module_query__mfm_recalc, "mfm_recalc", },
   { _gwy_module_query__mfm_shift, "mfm_shift", },
   { _gwy_module_query__multiprofile, "multiprofile", },
   { _gwy_module_query__neural, "neural", },
-  { _gwy_module_query__noise_synth, "noise_synth", },
-  { _gwy_module_query__obj_synth, "obj_synth", },
   { _gwy_module_query__otsu_threshold, "otsu_threshold", },
   { _gwy_module_query__outliers, "outliers", },
-  { _gwy_module_query__pat_synth, "pat_synth", },
   { _gwy_module_query__pfm, "pfm", },
-  { _gwy_module_query__phase_synth, "phase_synth", },
   { _gwy_module_query__pid, "pid", },
-  { _gwy_module_query__pileup_synth, "pileup_synth", },
-  { _gwy_module_query__plateau_synth, "plateau_synth", },
-  { _gwy_module_query__polylevel, "polylevel", },
   { _gwy_module_query__polydistort, "polydistort", },
+  { _gwy_module_query__polylevel, "polylevel", },
   { _gwy_module_query__presentationops, "presentationops", },
   { _gwy_module_query__psdf_logphi, "psdf_logphi", },
   { _gwy_module_query__psf, "psf", },
@@ -297,14 +239,14 @@ static const GwyModuleRecord modules[] = {
   { _gwy_module_query__relate, "relate", },
   { _gwy_module_query__reorder, "reorder", },
   { _gwy_module_query__resample, "resample", },
-  { _gwy_module_query__residue_synth, "residue_synth", },
-  { _gwy_module_query__roddeposit_synth, "roddeposit_synth", },
   { _gwy_module_query__rotate, "rotate", },
   { _gwy_module_query__semsim, "semsim", },
   { _gwy_module_query__scale, "scale", },
   { _gwy_module_query__scars, "scars", },
   { _gwy_module_query__shade, "shade", },
   { _gwy_module_query__slope_dist, "slope_dist", },
+  { _gwy_module_query__smm, "smm", },
+  { _gwy_module_query__smm_apply, "smm_apply", },
   { _gwy_module_query__sphere_revolve, "sphere-revolve", },
   { _gwy_module_query__stitch, "stitch", },
   { _gwy_module_query__straighten_path, "straighten_path", },
@@ -318,8 +260,6 @@ static const GwyModuleRecord modules[] = {
   { _gwy_module_query__trimmed_mean, "trimmed-mean", },
   { _gwy_module_query__volumize, "volumize", },
   { _gwy_module_query__volumize_layers, "volumize_layers", },
-  { _gwy_module_query__wave_synth, "wave_synth", },
-  { _gwy_module_query__wfr_synth, "wfr_synth", },
   { _gwy_module_query__wpour_mark, "wpour_mark", },
   { _gwy_module_query__wrapvalue, "wrapvalue", },
   { _gwy_module_query__xydenoise, "xydenoise", },
