@@ -1,5 +1,5 @@
 /*
- *  $Id: psdf2d.c 27043 2024-12-13 16:42:03Z yeti-dn $
+ *  $Id: psdf2d.c 29023 2025-12-19 13:06:16Z yeti-dn $
  *  Copyright (C) 2009-2022 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,18 +16,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwythreads.h>
-#include <libprocess/filters.h>
-#include <libprocess/stats.h>
-#include <libprocess/linestats.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
@@ -66,10 +58,10 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *mask;
-    GwyDataField *psdf;
-    GwyDataField *modulus;
+    GwyField *field;
+    GwyField *mask;
+    GwyField *psdf;
+    GwyField *modulus;
     GwySelection *selection;
     GwyGraphModel *gmodel;
 } ModuleArgs;
@@ -81,22 +73,21 @@ typedef struct {
     GwyParamTable *table_psdf;
     GwyParamTable *table_graph;
     GwyParamTable *table_params;
-    GwyDataLine *line;
-    GwyContainer *data;
+    GwyLine *line;
     GwyResults *results;
     UpdateWhat update;
 } ModuleGUI;
 
 static gboolean         module_register               (void);
 static GwyParamDef*     define_module_params          (void);
-static void             psdf2d                        (GwyContainer *data,
-                                                       GwyRunType runtype);
+static void             module_main                   (GwyFile *data,
+                                                       GwyRunModeFlags mode);
 static void             execute                       (ModuleArgs *args);
 static GwyDialogOutcome run_gui                       (ModuleArgs *args,
-                                                       GwyContainer *data,
+                                                       GwyFile *data,
                                                        gint id);
 static GwyResults*      create_results                (ModuleArgs *args,
-                                                       GwyContainer *data,
+                                                       GwyFile *data,
                                                        gint id);
 static void             param_changed                 (ModuleGUI *gui,
                                                        gint id);
@@ -104,15 +95,15 @@ static void             dialog_response               (ModuleGUI *gui,
                                                        gint response);
 static void             preview                       (gpointer user_data);
 static void             calculate_zoomed_fields       (ModuleGUI *gui);
-static GwyDataField*    cut_field_to_zoom             (GwyDataField *field,
+static GwyField*        cut_field_to_zoom             (GwyField *field,
                                                        gint zoom);
 static void             selection_changed             (ModuleGUI *gui,
                                                        gint hint);
 static void             update_sensitivity            (ModuleGUI *gui);
 static void             update_curve                  (ModuleGUI *gui,
                                                        gint i);
-static void             add_line_selection_from_points(GwyContainer *data,
-                                                       GwyDataField *field,
+static void             add_line_selection_from_points(GwyFile *data,
+                                                       GwyField *field,
                                                        gint id,
                                                        GwySelection *pointsel);
 
@@ -134,11 +125,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("psdf2d",
-                              (GwyProcessFunc)&psdf2d,
+                              module_main,
                               N_("/_Statistics/2D _PSDF..."),
                               NULL,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Calculate 2D power spectrum density"));
 
     return TRUE;
@@ -181,31 +172,31 @@ define_module_params(void)
 }
 
 static void
-psdf2d(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     ModuleArgs args;
     gint oldid, newid;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_MASK_FIELD, &args.mask,
-                                     GWY_APP_DATA_FIELD_ID, &oldid,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_MASK_FIELD, &args.mask,
+                                 GWY_APP_FIELD_ID, &oldid,
+                                 0);
     g_return_if_fail(args.field);
 
     args.params = gwy_params_new_from_settings(define_module_params());
-    args.psdf = gwy_data_field_new(17, 17, 1.0, 1.0, TRUE);
-    args.modulus = gwy_data_field_new(17, 17, 1.0, 1.0, TRUE);
+    args.psdf = gwy_field_new(17, 17, 1.0, 1.0, TRUE);
+    args.modulus = gwy_field_new(17, 17, 1.0, 1.0, TRUE);
     // We need to set the units of args->gmodel immediately for target graph filtering.  We do not care about modulus;
     // it is just for the show.
-    gwy_si_unit_power(gwy_data_field_get_si_unit_xy(args.field), -1, gwy_data_field_get_si_unit_xy(args.psdf));
-    gwy_si_unit_power_multiply(gwy_data_field_get_si_unit_z(args.field), 2,
-                               gwy_data_field_get_si_unit_xy(args.field), 2,
-                               gwy_data_field_get_si_unit_z(args.psdf));
+    gwy_unit_power(gwy_field_get_unit_xy(args.field), -1, gwy_field_get_unit_xy(args.psdf));
+    gwy_unit_power_multiply(gwy_field_get_unit_z(args.field), 2,
+                            gwy_field_get_unit_xy(args.field), 2,
+                            gwy_field_get_unit_z(args.psdf));
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, oldid);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -219,17 +210,18 @@ psdf2d(GwyContainer *data, GwyRunType runtype)
     if (gwy_params_get_boolean(args.params, PARAM_CREATE_IMAGE)) {
         gboolean zoomed_image = gwy_params_get_boolean(args.params, PARAM_ZOOMED_IMAGE);
         guint zoom = (zoomed_image ? gwy_params_get_enum(args.params, PARAM_ZOOM) : 1);
-        GwyDataField *zoomed;
+        GwyField *zoomed;
 
         zoomed = cut_field_to_zoom(args.psdf, zoom);
-        newid = gwy_app_data_browser_add_data_field(zoomed, data, TRUE);
+        newid = gwy_file_add_image(data, zoomed);
+        gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
         g_object_unref(zoomed);
 
         add_line_selection_from_points(data, zoomed, newid, args.selection);
-        gwy_app_set_data_field_title(data, newid, _("2D PSDF"));
-        gwy_container_set_const_string(data, gwy_app_get_data_palette_key_for_id(newid), "DFit");
-        gwy_container_set_enum(data, gwy_app_get_data_range_type_key_for_id(newid), GWY_LAYER_BASIC_RANGE_AUTO);
-        gwy_app_channel_log_add_proc(data, oldid, newid);
+        gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _("2D PSDF"), TRUE);
+        gwy_file_set_palette(data, GWY_FILE_IMAGE, newid, "DFit");
+        gwy_container_set_enum(GWY_CONTAINER(data), gwy_file_key_image_color_mapping(newid), GWY_COLOR_MAPPING_AUTO);
+        gwy_log_add(data, GWY_FILE_IMAGE, oldid, newid);
     }
     if (args.gmodel && gwy_graph_model_get_n_curves(args.gmodel)) {
         GwyGraphCurveModel *gcmodel;
@@ -243,13 +235,14 @@ psdf2d(GwyContainer *data, GwyRunType runtype)
             for (i = 0; i < n; i++) {
                 gmodel = gwy_graph_model_new_alike(args.gmodel);
                 gcmodel = gwy_graph_model_get_curve(args.gmodel, i);
-                gcmodel = gwy_graph_curve_model_duplicate(gcmodel);
+                gcmodel = gwy_graph_curve_model_copy(gcmodel);
                 gwy_graph_model_add_curve(gmodel, gcmodel);
                 g_object_unref(gcmodel);
                 g_object_get(gcmodel, "description", &s, NULL);
                 g_object_set(gmodel, "title", s, NULL);
                 g_free(s);
-                gwy_app_data_browser_add_graph_model(gmodel, data, TRUE);
+                newid = gwy_file_add_graph(data, gmodel);
+                gwy_file_set_visible(data, GWY_FILE_GRAPH, newid, TRUE);
                 g_object_unref(gmodel);
             }
         }
@@ -260,15 +253,15 @@ psdf2d(GwyContainer *data, GwyRunType runtype)
     }
 
 end:
-    GWY_OBJECT_UNREF(args.selection);
-    GWY_OBJECT_UNREF(args.gmodel);
+    g_clear_object(&args.selection);
+    g_clear_object(&args.gmodel);
     g_object_unref(args.psdf);
     g_object_unref(args.modulus);
     g_object_unref(args.params);
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     ModuleGUI gui;
     GtkWidget *hbox, *vbox, *notebook;
@@ -276,38 +269,37 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     GwyDialog *dialog;
     GwyGraph *graph;
     GwyDialogOutcome outcome;
+    GwyVectorLayer *vlayer;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
     args->gmodel = gwy_graph_model_new();
-    gui.data = gwy_container_new();
     gui.results = create_results(args, data, id);
-    gui.line = gwy_data_line_new(1, 1.0, FALSE);
-    calculate_zoomed_fields(&gui);
+    gui.line = gwy_line_new(1, 1.0, FALSE);
 
     gui.dialog = gwy_dialog_new(_("Power Spectrum Density"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_CLEAR, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
     /***** PSDF (actually, modulus) preview *****/
-    gwy_container_set_const_string(gui.data, gwy_app_get_data_palette_key_for_id(0), "DFit");
-    gwy_container_set_enum(gui.data, gwy_app_get_data_range_type_key_for_id(0), GWY_LAYER_BASIC_RANGE_AUTO);
-    gwy_app_sync_data_items(data, gui.data, id, 0, FALSE,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
-    gui.dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
-    args->selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(gui.dataview), 0, "Point", 12, TRUE);
+    gui.dataview = gwy_create_preview(NULL, NULL, PREVIEW_SIZE);
+    calculate_zoomed_fields(&gui);
+    gwy_setup_data_view(GWY_DATA_VIEW(gui.dataview), data, GWY_FILE_IMAGE, id, GWY_FILE_ITEM_REAL_SQUARE);
+    gwy_data_view_set_gradient(GWY_DATA_VIEW(gui.dataview), gwy_gradients_get_gradient("DFit"));
+    gwy_data_view_set_color_mapping(GWY_DATA_VIEW(gui.dataview), GWY_COLOR_MAPPING_AUTO);
+    args->selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(gui.dataview), GWY_TYPE_LAYER_POINT, 12, TRUE);
     g_object_ref(args->selection);
-    g_object_set(gwy_data_view_get_top_layer(GWY_DATA_VIEW(gui.dataview)), "draw-as-vector", TRUE, NULL);
+    vlayer = gwy_data_view_get_interactive_layer(GWY_DATA_VIEW(gui.dataview));
+    gwy_layer_point_set_draw_as_vector(GWY_LAYER_POINT(vlayer), TRUE);
     g_signal_connect_swapped(args->selection, "changed", G_CALLBACK(selection_changed), &gui);
 
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(gui.dataview), FALSE);
 
     /***** Graph *****/
-    vbox = gwy_vbox_new(0);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
     gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 4);
 
-    gwy_graph_model_set_units_from_data_field(args->gmodel, args->psdf, 1, 0, 0, 1);
+    gwy_graph_model_set_units_from_field(args->gmodel, args->psdf, 1, 0, 0, 1);
     g_object_set(args->gmodel,
                  "title", _("PSDF Section"),
                  "axis-label-bottom", "k",
@@ -366,7 +358,6 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 
     outcome = gwy_dialog_run(dialog);
 
-    g_object_unref(gui.data);
     g_object_unref(gui.results);
     g_object_unref(gui.line);
 
@@ -374,7 +365,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 }
 
 static GwyResults*
-create_results(G_GNUC_UNUSED ModuleArgs *args, GwyContainer *data, gint id)
+create_results(G_GNUC_UNUSED ModuleArgs *args, GwyFile *data, gint id)
 {
     GwyResults *results = gwy_results_new();
 
@@ -393,7 +384,7 @@ create_results(G_GNUC_UNUSED ModuleArgs *args, GwyContainer *data, gint id)
                           NULL);
 
     gwy_results_fill_filename(results, "file", data);
-    gwy_results_fill_channel(results, "image", data, id);
+    gwy_results_fill_data_name(results, "image", data, GWY_FILE_IMAGE, id);
 
     return results;
 }
@@ -466,15 +457,15 @@ preview(gpointer user_data)
 
     if (gui->update >= UPDATE_PSDF) {
         gboolean is_masking = (gwy_params_get_masking(params, PARAM_MASKING, NULL) != GWY_MASK_IGNORE);
-        GwyDataLine *angspec;
+        GwyLine *angspec;
         gdouble m;
 
         execute(args);
-        angspec = gwy_data_field_psdf_to_angular_spectrum(args->psdf, -1);
-        m = gwy_data_line_get_max(angspec);
+        angspec = gwy_field_psdf_to_angular_spectrum(args->psdf, -1);
+        m = gwy_line_max(angspec);
         if (m > 0.0) {
-            gdouble Std = gwy_data_line_max_pos_r(angspec);
-            gdouble Stdi = gwy_data_line_get_avg(angspec)/m;
+            gdouble Std = gwy_line_max_pos_r(angspec);
+            gdouble Stdi = gwy_line_avg(angspec)/m;
 
             gwy_results_fill_values(gui->results, "Std", Std, "Stdi", Stdi, NULL);
         }
@@ -485,17 +476,18 @@ preview(gpointer user_data)
         g_object_unref(angspec);
     }
     if (gui->update >= UPDATE_ZOOMED) {
-        GwyDataField *modulus;
+        GwyDataView *dataview = GWY_DATA_VIEW(gui->dataview);
+        GwyField *modulus;
         gdouble xoff, yoff;
 
-        modulus = gwy_container_get_object(gui->data, gwy_app_get_data_key_for_id(0));
-        xoff = gwy_data_field_get_xoffset(modulus);
-        yoff = gwy_data_field_get_yoffset(modulus);
+        modulus = gwy_data_view_get_field(dataview);
+        xoff = gwy_field_get_xoffset(modulus);
+        yoff = gwy_field_get_yoffset(modulus);
         calculate_zoomed_fields(gui);
         gwy_set_data_preview_size(GWY_DATA_VIEW(gui->dataview), PREVIEW_SIZE);
-        modulus = gwy_container_get_object(gui->data, gwy_app_get_data_key_for_id(0));
-        xoff -= gwy_data_field_get_xoffset(modulus);
-        yoff -= gwy_data_field_get_yoffset(modulus);
+        modulus = gwy_data_view_get_field(dataview);
+        xoff -= gwy_field_get_xoffset(modulus);
+        yoff -= gwy_field_get_yoffset(modulus);
         if (xoff || yoff) {
             gwy_selection_move(args->selection, xoff, yoff);
             gui->update = UPDATE_NOTHING;
@@ -517,7 +509,7 @@ update_curve(ModuleGUI *gui, gint i)
     gint resolution = gwy_params_get_int(params, PARAM_RESOLUTION);
     gint thickness = gwy_params_get_int(params, PARAM_THICKNESS);
     GwyInterpolationType interpolation = gwy_params_get_enum(args->params, PARAM_INTERPOLATION);
-    GwyDataField *zoomedmodulus, *psdf = args->psdf;
+    GwyField *zoomedmodulus, *psdf = args->psdf;
     GwyGraphCurveModel *gcmodel;
     gdouble xy[4], xofffull, yofffull; //, h, hx, hy;
     gint xl0, yl0, xl1, yl1;
@@ -528,23 +520,23 @@ update_curve(ModuleGUI *gui, gint i)
         g_return_if_reached();
     }
 
-    zoomedmodulus = gwy_container_get_object(gui->data, gwy_app_get_data_key_for_id(0));
-    xy[0] += gwy_data_field_get_xoffset(zoomedmodulus);
-    xy[1] += gwy_data_field_get_yoffset(zoomedmodulus);
+    zoomedmodulus = gwy_data_view_get_field(GWY_DATA_VIEW(gui->dataview));
+    xy[0] += gwy_field_get_xoffset(zoomedmodulus);
+    xy[1] += gwy_field_get_yoffset(zoomedmodulus);
 
-    xl0 = gwy_data_field_get_xres(psdf)/2;
-    yl0 = gwy_data_field_get_yres(psdf)/2;
+    xl0 = gwy_field_get_xres(psdf)/2;
+    yl0 = gwy_field_get_yres(psdf)/2;
 
-    xofffull = gwy_data_field_get_xoffset(psdf);
-    yofffull = gwy_data_field_get_yoffset(psdf);
-    xl1 = floor(gwy_data_field_rtoj(psdf, xy[0] - xofffull));
-    yl1 = floor(gwy_data_field_rtoi(psdf, xy[1] - yofffull));
+    xofffull = gwy_field_get_xoffset(psdf);
+    yofffull = gwy_field_get_yoffset(psdf);
+    xl1 = floor(gwy_field_rtoj(psdf, xy[0] - xofffull));
+    yl1 = floor(gwy_field_rtoi(psdf, xy[1] - yofffull));
 
     /* FIXME FIXME FIXME: Why were we doing this? We always say we plot a section of the PSDF. So just do that. It has
      * must have the same units as the 2D PSDF and it is not itself any kind of PSDF. */
 #if 0
-    hx = gwy_data_field_get_dx(args->field)/(2*G_PI);
-    hy = gwy_data_field_get_dx(args->field)/(2*G_PI);
+    hx = gwy_field_get_dx(args->field)/(2*G_PI);
+    hy = gwy_field_get_dx(args->field)/(2*G_PI);
     h = hypot(hx*xy[0], hy*xy[1])/hypot(xy[0], xy[1]);
 #endif
 
@@ -555,9 +547,9 @@ update_curve(ModuleGUI *gui, gint i)
     else
         lineres = resolution;
 
-    gwy_data_field_get_profile(psdf, gui->line, xl0, yl0, xl1, yl1, lineres, thickness, interpolation);
+    gwy_field_get_profile(psdf, gui->line, xl0, yl0, xl1, yl1, lineres, thickness, interpolation);
 #if 0
-    gwy_data_line_multiply(gui->line, h);
+    gwy_line_multiply(gui->line, h);
 #endif
 
     n = gwy_graph_model_get_n_curves(args->gmodel);
@@ -573,8 +565,8 @@ update_curve(ModuleGUI *gui, gint i)
         g_object_unref(gcmodel);
     }
 
-    gwy_graph_curve_model_set_data_from_dataline(gcmodel, gui->line, 0, 0);
-    desc = g_strdup_printf(_("PSDF %.0f°"), 180.0/G_PI*atan2(-xy[1], xy[0]));
+    gwy_graph_curve_model_set_data_from_line(gcmodel, gui->line, 0, 0);
+    desc = g_strdup_printf(_("PSDF %.0f°"), gwy_rad2deg(atan2(-xy[1], xy[0])));
     g_object_set(gcmodel, "description", desc, NULL);
     g_free(desc);
 }
@@ -584,22 +576,22 @@ calculate_zoomed_fields(ModuleGUI *gui)
 {
     ModuleArgs *args = gui->args;
     guint zoom = gwy_params_get_enum(args->params, PARAM_ZOOM);
-    GwyDataField *zoomed;
+    GwyField *zoomed;
 
     zoomed = cut_field_to_zoom(args->modulus, zoom);
-    gwy_container_set_object(gui->data, gwy_app_get_data_key_for_id(0), zoomed);
-    gwy_data_field_data_changed(zoomed);
+    gwy_field_data_changed(zoomed);
+    gwy_data_view_set_field(GWY_DATA_VIEW(gui->dataview), zoomed);
     g_object_unref(zoomed);
 }
 
-static GwyDataField*
-cut_field_to_zoom(GwyDataField *field, gint zoom)
+static GwyField*
+cut_field_to_zoom(GwyField *field, gint zoom)
 {
-    GwyDataField *zoomed;
+    GwyField *zoomed;
     guint xres, yres, width, height;
 
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
     width = (xres/zoom) | 1;
     height = (yres/zoom) | 1;
     if (width < 17)
@@ -610,17 +602,17 @@ cut_field_to_zoom(GwyDataField *field, gint zoom)
     if (width >= xres && height >= yres)
         return g_object_ref(field);
 
-    zoomed = gwy_data_field_area_extract(field, (xres - width + 1)/2, (yres - height + 1)/2, width, height);
-    gwy_data_field_set_xoffset(zoomed, -0.5*gwy_data_field_get_xreal(zoomed));
-    gwy_data_field_set_yoffset(zoomed, -0.5*gwy_data_field_get_yreal(zoomed));
+    zoomed = gwy_field_area_extract(field, (xres - width + 1)/2, (yres - height + 1)/2, width, height);
+    gwy_field_set_xoffset(zoomed, -0.5*gwy_field_get_xreal(zoomed));
+    gwy_field_set_yoffset(zoomed, -0.5*gwy_field_get_yreal(zoomed));
 
     return zoomed;
 }
 
 /* Convert points to lines from the origin, which is assumed to be in the centre. */
 static void
-add_line_selection_from_points(GwyContainer *data,
-                               GwyDataField *field,
+add_line_selection_from_points(GwyFile *data,
+                               GwyField *field,
                                gint id,
                                GwySelection *pointsel)
 {
@@ -628,17 +620,15 @@ add_line_selection_from_points(GwyContainer *data,
     gint nsel, i;
     gdouble *seldata;
     gdouble xreal, yreal;
-    gchar *key;
 
     if (!pointsel || !(nsel = gwy_selection_get_data(pointsel, NULL)))
         return;
 
-    linesel = g_object_new(g_type_from_name("GwySelectionLine"), NULL);
-    g_return_if_fail(linesel);
+    linesel = gwy_selection_line_new();
     gwy_selection_set_max_objects(linesel, 1024);
     seldata = g_new(gdouble, 4*nsel);
-    xreal = gwy_data_field_get_xreal(field);
-    yreal = gwy_data_field_get_yreal(field);
+    xreal = gwy_field_get_xreal(field);
+    yreal = gwy_field_get_yreal(field);
 
     for (i = 0; i < nsel; i++) {
         seldata[4*i + 0] = 0.5*xreal;
@@ -649,29 +639,28 @@ add_line_selection_from_points(GwyContainer *data,
     gwy_selection_set_data(linesel, nsel, seldata);
     g_free(seldata);
 
-    key = g_strdup_printf("/%d/select/line", id);
-    gwy_container_pass_object_by_name(data, key, linesel);
+    gwy_container_pass_object(GWY_CONTAINER(data), gwy_file_key_image_selection(id, "line"), linesel);
 }
 
 static void
 execute(ModuleArgs *args)
 {
     GwyParams *params = args->params;
-    GwyDataField *mask = args->mask, *field = args->field, *psdf = args->psdf, *modulus = args->modulus;
+    GwyField *mask = args->mask, *field = args->field, *psdf = args->psdf, *modulus = args->modulus;
     GwyMaskingType masking = gwy_params_get_masking(params, PARAM_MASKING, &mask);
     GwyWindowingType windowing = gwy_params_get_enum(params, PARAM_WINDOWING);
-    gint xres = gwy_data_field_get_xres(field);
-    gint yres = gwy_data_field_get_yres(field);
+    gint xres = gwy_field_get_xres(field);
+    gint yres = gwy_field_get_yres(field);
     guint n, i;
     gdouble *p;
 
     n = xres*yres;
-    gwy_data_field_area_2dpsdf_mask(field, psdf, mask, masking, 0, 0, xres, yres, windowing, 1);
+    gwy_field_area_psdf_2d(field, mask, masking, psdf, 0, 0, xres, yres, windowing, 1);
 
     /* We do not really care about modulus units nor its absolute scale.
      * We just have it to display the square root... */
-    gwy_data_field_assign(modulus, psdf);
-    p = gwy_data_field_get_data(modulus);
+    gwy_field_assign(modulus, psdf);
+    p = gwy_field_get_data(modulus);
 #ifdef _OPENMP
 #pragma omp parallel for if (gwy_threads_are_enabled()) default(none) \
             shared(p,n) private(i)
