@@ -1,5 +1,5 @@
 /*
- *  $Id: psdf_logphi.c 23666 2021-05-10 21:56:10Z yeti-dn $
+ *  $Id: psdf_logphi.c 28910 2025-11-24 18:11:09Z yeti-dn $
  *  Copyright (C) 2015-2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,16 +16,9 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwythreads.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libprocess/inttrans.h>
-#include <libprocess/filters.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
 
@@ -36,8 +29,8 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *result;
+    GwyField *field;
+    GwyField *result;
 } ModuleArgs;
 
 typedef struct {
@@ -48,8 +41,8 @@ typedef struct {
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             psdflp              (GwyContainer *data,
-                                             GwyRunType runtype);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static void             execute             (ModuleArgs *args);
 static GwyDialogOutcome run_gui             (ModuleArgs *args);
 
@@ -69,11 +62,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("psdf_logphi",
-                              (GwyProcessFunc)&psdflp,
+                              module_main,
                               N_("/_Statistics/_Log-Phi PSDF..."),
-                              GWY_STOCK_PSDF_LOG_PHI,
+                              GWY_ICON_PSDF_LOG_PHI,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Compute PSDF in Log-Phi coordinates"));
 
     return TRUE;
@@ -95,22 +88,22 @@ define_module_params(void)
 }
 
 static void
-psdflp(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     ModuleArgs args;
     gint id, newid;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(args.field);
 
-    args.result = gwy_data_field_new(1, 1, 1.0, 1.0, FALSE);
+    args.result = gwy_field_new(1, 1, 1.0, 1.0, FALSE);
     args.params = gwy_params_new_from_settings(define_module_params());
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -118,9 +111,11 @@ psdflp(GwyContainer *data, GwyRunType runtype)
     }
     execute(&args);
 
-    newid = gwy_app_data_browser_add_data_field(args.result, data, TRUE);
-    gwy_app_set_data_field_title(data, newid, "Log-phi PSDF");
-    gwy_app_channel_log_add_proc(data, id, newid);
+    newid = gwy_file_add_image(data, args.result);
+
+    gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+    gwy_file_set_title(data, GWY_FILE_IMAGE, newid, "Log-phi PSDF", TRUE);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, newid);
 
 end:
     g_object_unref(args.result);
@@ -155,37 +150,36 @@ execute(ModuleArgs *args)
 {
     enum { N = 4 };
 
-    GwyDataField *field = args->field, *lpsdf = args->result, *reout, *imout;
+    GwyField *field = args->field, *lpsdf = args->result, *reout, *imout;
     gint pxres, pyres, fxres, fyres, i, j;
     gdouble *ldata, *redata, *imdata, *cosphi, *sinphi;
     gdouble xreal, yreal, f0, f_max, b;
     gdouble sigma = gwy_params_get_double(args->params, PARAM_SIGMA);
     GwyWindowingType window = gwy_params_get_enum(args->params, PARAM_WINDOW);
 
-    reout = gwy_data_field_new_alike(field, FALSE);
-    imout = gwy_data_field_new_alike(field, FALSE);
-    gwy_data_field_2dfft(field, NULL, reout, imout,
-                         window, GWY_TRANSFORM_DIRECTION_FORWARD, GWY_INTERPOLATION_ROUND, TRUE, 1);
+    reout = gwy_field_new_alike(field, FALSE);
+    imout = gwy_field_new_alike(field, FALSE);
+    gwy_field_fft_2d(field, NULL, reout, imout, window, GWY_TRANSFORM_DIRECTION_FORWARD, TRUE, 1);
 
-    pxres = gwy_data_field_get_xres(reout);
-    pyres = gwy_data_field_get_yres(reout);
-    redata = gwy_data_field_get_data(reout);
-    imdata = gwy_data_field_get_data(imout);
+    pxres = gwy_field_get_xres(reout);
+    pyres = gwy_field_get_yres(reout);
+    redata = gwy_field_get_data(reout);
+    imdata = gwy_field_get_data(imout);
     for (i = 0; i < pxres*pyres; i++)
         redata[i] = redata[i]*redata[i] + imdata[i]*imdata[i];
-    gwy_data_field_2dfft_humanize(reout);
-    gwy_data_field_filter_gaussian(reout, sigma);
-    redata = gwy_data_field_get_data(reout);
+    gwy_field_fft_2d_center(reout);
+    gwy_field_filter_gaussian(reout, sigma);
+    redata = gwy_field_get_data(reout);
     for (i = 0; i < pxres*pyres; i++)
         redata[i] = sqrt(redata[i]);
 
     fxres = pxres/2;
     fyres = pyres/2;
-    gwy_data_field_resample(lpsdf, fxres, fyres, GWY_INTERPOLATION_NONE);
-    ldata = gwy_data_field_get_data(lpsdf);
+    gwy_field_resize(lpsdf, fxres, fyres);
+    ldata = gwy_field_get_data(lpsdf);
 
-    xreal = gwy_data_field_get_xreal(field);
-    yreal = gwy_data_field_get_yreal(field);
+    xreal = gwy_field_get_xreal(field);
+    yreal = gwy_field_get_yreal(field);
     f0 = 2.0/MIN(xreal, yreal);
     f_max = 0.5*MIN(pxres/xreal, pyres/yreal);
     if (f_max <= f0) {
@@ -235,7 +229,7 @@ execute(ModuleArgs *args)
                     if (G_UNLIKELY(x < 0.5 || y < 0.5 || x > pxres - 1.5 || y > pyres - 1.5))
                         continue;
 
-                    p = gwy_data_field_get_dval(reout, x, y, GWY_INTERPOLATION_SCHAUM);
+                    p = gwy_field_get_dval(reout, x, y, GWY_INTERPOLATION_SCHAUM);
                     s += p;
                     n++;
                 }
@@ -248,13 +242,13 @@ execute(ModuleArgs *args)
     g_object_unref(imout);
     g_object_unref(reout);
 
-    gwy_data_field_set_xreal(lpsdf, 2.0*G_PI);
-    gwy_data_field_set_xoffset(lpsdf, 0.0);
-    gwy_data_field_set_yreal(lpsdf, log(f_max/f0));
-    gwy_data_field_set_yoffset(lpsdf, log(f0));
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_xy(lpsdf), NULL);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(lpsdf), NULL);
-    gwy_data_field_normalize(lpsdf);
+    gwy_field_set_xreal(lpsdf, 2.0*G_PI);
+    gwy_field_set_xoffset(lpsdf, 0.0);
+    gwy_field_set_yreal(lpsdf, log(f_max/f0));
+    gwy_field_set_yoffset(lpsdf, log(f0));
+    gwy_unit_clear(gwy_field_get_unit_xy(lpsdf));
+    gwy_unit_clear(gwy_field_get_unit_z(lpsdf));
+    gwy_field_normalize(lpsdf);
 }
 
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
