@@ -1,5 +1,5 @@
 /*
- *  $Id: psf-fit.c 24833 2022-05-19 13:45:20Z yeti-dn $
+ *  $Id: psf-fit.c 28844 2025-11-10 16:07:51Z yeti-dn $
  *  Copyright (C) 2017-2022 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,31 +16,20 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwynlfit.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/simplefft.h>
-#include <libprocess/stats.h>
-#include <libprocess/filters.h>
-#include <libprocess/inttrans.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES GWY_RUN_INTERACTIVE
 
 #define field_convolve_default(field, kernel) \
-    gwy_data_field_area_ext_convolve((field), \
+    gwy_field_area_ext_convolve((field), \
                                      0, 0, \
-                                     gwy_data_field_get_xres(field), \
-                                     gwy_data_field_get_yres(field), \
+                                     gwy_field_get_xres(field), \
+                                     gwy_field_get_yres(field), \
                                      (field), (kernel), \
-                                     GWY_EXTERIOR_BORDER_EXTEND, 0.0, TRUE)
+                                     GWY_EXTERIOR_BORDER, 0.0, TRUE)
 
 #define DECLARE_PSF(name) \
     static gdouble  psf_##name##_fit_func   (guint i, \
@@ -54,19 +43,19 @@
                                              gpointer user_data, \
                                              gdouble *der, \
                                              gboolean *success); \
-    static gboolean psf_##name##_init_params(GwyDataField *model_re, \
-                                             GwyDataField *model_im, \
-                                             GwyDataField *data_re, \
-                                             GwyDataField *data_im, \
-                                             GwyDataField *freq_x, \
-                                             GwyDataField *freq_y, \
+    static gboolean psf_##name##_init_params(GwyField *model_re, \
+                                             GwyField *model_im, \
+                                             GwyField *data_re, \
+                                             GwyField *data_im, \
+                                             GwyField *freq_x, \
+                                             GwyField *freq_y, \
                                              gdouble *params); \
-    static void     psf_##name##_fill_psf   (GwyDataField *freq_x, \
-                                             GwyDataField *freq_y, \
-                                             GwyDataField *buf_re, \
-                                             GwyDataField *buf_im, \
-                                             GwyDataField *psf, \
-                                             GwyDataField *psf_fft, \
+    static void     psf_##name##_fill_psf   (GwyField *freq_x, \
+                                             GwyField *freq_y, \
+                                             GwyField *buf_re, \
+                                             GwyField *buf_im, \
+                                             GwyField *psf, \
+                                             GwyField *psf_fft, \
                                              const gdouble *param);
 
 /* NB: The values directly index the functions[] array. */
@@ -91,19 +80,19 @@ enum {
     PARAM_IDEAL,
 };
 
-typedef gboolean (*PSFParamInitFunc)(GwyDataField *model_re,
-                                     GwyDataField *model_im,
-                                     GwyDataField *data_re,
-                                     GwyDataField *data_im,
-                                     GwyDataField *freq_x,
-                                     GwyDataField *freq_y,
+typedef gboolean (*PSFParamInitFunc)(GwyField *model_re,
+                                     GwyField *model_im,
+                                     GwyField *data_re,
+                                     GwyField *data_im,
+                                     GwyField *freq_x,
+                                     GwyField *freq_y,
                                      gdouble *params);
-typedef void (*PSFFillFunc)(GwyDataField *freq_x,
-                            GwyDataField *freq_y,
-                            GwyDataField *buf_re,
-                            GwyDataField *buf_im,
-                            GwyDataField *psf,
-                            GwyDataField *psf_fft,
+typedef void (*PSFFillFunc)(GwyField *freq_x,
+                            GwyField *freq_y,
+                            GwyField *buf_re,
+                            GwyField *buf_im,
+                            GwyField *psf,
+                            GwyField *psf_fft,
                             const gdouble *params);
 
 typedef struct {
@@ -128,8 +117,8 @@ typedef struct {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *psf;
+    GwyField *field;
+    GwyField *psf;
 } ModuleArgs;
 
 typedef struct {
@@ -140,28 +129,28 @@ typedef struct {
 
 static gboolean         module_register            (void);
 static GwyParamDef*     define_module_params       (void);
-static void             psf                        (GwyContainer *data,
-                                                    GwyRunType runtype);
+static void             module_main                (GwyFile *data,
+                                                    GwyRunModeFlags mode);
 static void             execute                    (ModuleArgs *args);
 static GwyDialogOutcome run_gui                    (ModuleArgs *args);
 static void             param_changed              (ModuleGUI *gui,
                                                     gint id);
-static gboolean         ideal_image_filter         (GwyContainer *data,
+static gboolean         ideal_image_filter         (GwyFile *data,
                                                     gint id,
                                                     gpointer user_data);
-static gint             create_output_field        (GwyDataField *field,
-                                                    GwyContainer *data,
+static gint             create_output_field        (GwyField *field,
+                                                    GwyFile *data,
                                                     gint id,
                                                     const gchar *name);
-static void             fit_psf                    (GwyDataField *model,
-                                                    GwyDataField *data,
-                                                    GwyDataField *psf,
+static void             fit_psf                    (GwyField *model,
+                                                    GwyField *data,
+                                                    GwyField *psf,
                                                     const PSFFunction *func,
                                                     GwyWindowingType windowing);
-static void             adjust_tf_to_non_integral  (GwyDataField *psf);
-static void             set_transfer_function_units(GwyDataField *ideal,
-                                                    GwyDataField *measured,
-                                                    GwyDataField *transferfunc);
+static void             adjust_tf_to_non_integral  (GwyField *psf);
+static void             set_transfer_function_units(GwyField *ideal,
+                                                    GwyField *measured,
+                                                    GwyField *transferfunc);
 
 DECLARE_PSF(gaussian);
 DECLARE_PSF(agaussian);
@@ -201,11 +190,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("psf-fit",
-                              (GwyProcessFunc)&psf,
+                              module_main,
                               N_("/_Statistics/Transfer _Function Fit..."),
                               NULL,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Fit transfer function from known data and image"));
 
     return TRUE;
@@ -241,23 +230,23 @@ define_module_params(void)
 }
 
 static void
-psf(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
-    GwyDataField *convolved = NULL, *difference, *field, *ideal;
+    GwyField *convolved = NULL, *difference, *field, *ideal;
     ModuleArgs args;
     guint output;
     gint id, newid = -1;
 
-    g_return_if_fail(runtype & RUN_MODES);
+    g_return_if_fail(mode & RUN_MODES);
 
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &field,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &field,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     args.field = field;
-    args.psf = gwy_data_field_new_alike(field, TRUE);
+    args.psf = gwy_field_new_alike(field, TRUE);
     args.params = gwy_params_new_from_settings(define_module_params());
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -269,11 +258,11 @@ psf(GwyContainer *data, GwyRunType runtype)
 
     execute(&args);
     if (output & ((1 << PSF_OUTPUT_CONVOLVED) | (1 << PSF_OUTPUT_DIFFERECE))) {
-        convolved = gwy_data_field_new_alike(field, FALSE);
-        gwy_data_field_copy(ideal, convolved, TRUE);
-        gwy_data_field_add(convolved, -gwy_data_field_get_avg(convolved));
+        convolved = gwy_field_new_alike(field, FALSE);
+        gwy_field_assign(convolved, ideal);
+        gwy_field_add(convolved, -gwy_field_get_avg(convolved));
         field_convolve_default(convolved, args.psf);
-        gwy_data_field_add(convolved, gwy_data_field_get_avg(field));
+        gwy_field_add(convolved, gwy_field_get_avg(field));
     }
 
     if (output & (1 << PSF_OUTPUT_PSF))
@@ -281,8 +270,8 @@ psf(GwyContainer *data, GwyRunType runtype)
     if (output & (1 << PSF_OUTPUT_CONVOLVED))
         create_output_field(convolved, data, id, _("Convolved"));
     if (output & (1 << PSF_OUTPUT_DIFFERECE)) {
-        difference = gwy_data_field_new_alike(field, FALSE);
-        gwy_data_field_subtract_fields(difference, convolved, field);
+        difference = gwy_field_new_alike(field, FALSE);
+        gwy_field_subtract_fields(difference, convolved, field);
         create_output_field(difference, data, id, _("Difference"));
         g_object_unref(difference);
     }
@@ -290,13 +279,13 @@ psf(GwyContainer *data, GwyRunType runtype)
     /* Change the normalisation to the discrete (i.e. wrong) one after all calculations are done. */
     if (newid != -1 && !gwy_params_get_boolean(args.params, PARAM_AS_INTEGRAL)) {
         adjust_tf_to_non_integral(args.psf);
-        gwy_data_field_data_changed(args.psf);
+        gwy_field_data_changed(args.psf);
     }
 
 end:
     g_object_unref(args.params);
     g_object_unref(args.psf);
-    GWY_OBJECT_UNREF(convolved);
+    g_clear_object(&convolved);
 }
 
 static GwyDialogOutcome
@@ -306,7 +295,7 @@ run_gui(ModuleArgs *args)
     GwyParamTable *table;
     ModuleGUI gui;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
 
     gui.dialog = gwy_dialog_new(_("Fit Transfer Function"));
@@ -343,18 +332,16 @@ param_changed(ModuleGUI *gui, G_GNUC_UNUSED gint id)
 }
 
 static gboolean
-ideal_image_filter(GwyContainer *data, gint id, gpointer user_data)
+ideal_image_filter(GwyFile *data, gint id, gpointer user_data)
 {
-    GwyDataField *field = (GwyDataField*)user_data;
-    GwyDataField *ideal = gwy_container_get_object(data, gwy_app_get_data_key_for_id(id));
+    GwyField *field = (GwyField*)user_data;
+    GwyField *ideal = gwy_file_get_image(data, id);
 
-    if (ideal == field)
+    if (!ideal || ideal == field)
         return FALSE;
 
-    return !gwy_data_field_check_compatibility(ideal, field,
-                                               GWY_DATA_COMPATIBILITY_RES
-                                               | GWY_DATA_COMPATIBILITY_REAL
-                                               | GWY_DATA_COMPATIBILITY_LATERAL);
+    return !gwy_field_is_incompatible(ideal, field,
+                                      GWY_DATA_MISMATCH_RES | GWY_DATA_MISMATCH_REAL | GWY_DATA_MISMATCH_LATERAL);
 }
 
 static void
@@ -363,29 +350,30 @@ execute(ModuleArgs *args)
     GwyParams *params = args->params;
     GwyWindowingType windowing = gwy_params_get_enum(params, PARAM_WINDOWING);
     PSFFunctionType func = gwy_params_get_enum(params, PARAM_FUNCTION);
-    GwyDataField *field = args->field, *psf = args->psf;
-    GwyDataField *ideal = gwy_params_get_image(params, PARAM_IDEAL);
+    GwyField *field = args->field, *psf = args->psf;
+    GwyField *ideal = gwy_params_get_image(params, PARAM_IDEAL);
     gdouble q;
 
     fit_psf(ideal, field, psf, functions + func, windowing);
     /* See psf.c for normalisation convention. */
-    q = sqrt(gwy_data_field_get_xres(field)*gwy_data_field_get_yres(field))
-        /(gwy_data_field_get_xreal(field)*gwy_data_field_get_yreal(field));
-    gwy_data_field_multiply(psf, q);
+    q = sqrt(gwy_field_get_xres(field)*gwy_field_get_yres(field))
+        /(gwy_field_get_xreal(field)*gwy_field_get_yreal(field));
+    gwy_field_multiply(psf, q);
 }
 
 static gint
-create_output_field(GwyDataField *field, GwyContainer *data, gint id, const gchar *name)
+create_output_field(GwyField *field, GwyFile *data, gint id, const gchar *name)
 {
     gint newid;
 
-    newid = gwy_app_data_browser_add_data_field(field, data, TRUE);
-    gwy_app_sync_data_items(data, data, id, newid, FALSE,
-                            GWY_DATA_ITEM_GRADIENT,
-                            GWY_DATA_ITEM_MASK_COLOR,
-                            0);
-    gwy_app_set_data_field_title(data, newid, name);
-    gwy_app_channel_log_add_proc(data, id, newid);
+    newid = gwy_file_add_image(data, field);
+
+    gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+    gwy_file_sync_items(data, GWY_FILE_IMAGE, id,
+                        data, GWY_FILE_IMAGE, newid,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_MASK_COLOR, FALSE);
+    gwy_file_set_title(data, GWY_FILE_IMAGE, newid, name, TRUE);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, newid);
 
     return newid;
 }
@@ -404,12 +392,12 @@ calculate_root_mean_square_complex(const gdouble *pre, const gdouble *pim,
 }
 
 static void
-precalculate_frequencies(GwyDataField *model, GwyDataField *freq_x, GwyDataField *freq_y)
+precalculate_frequencies(GwyField *model, GwyField *freq_x, GwyField *freq_y)
 {
-    guint xres = gwy_data_field_get_xres(model), yres = gwy_data_field_get_yres(model);
-    gdouble sx = 1.0/gwy_data_field_get_xreal(model), sy = 1.0/gwy_data_field_get_yreal(model);
-    gdouble *fx = gwy_data_field_get_data(freq_x);
-    gdouble *fy = gwy_data_field_get_data(freq_y);
+    guint xres = gwy_field_get_xres(model), yres = gwy_field_get_yres(model);
+    gdouble sx = 1.0/gwy_field_get_xreal(model), sy = 1.0/gwy_field_get_yreal(model);
+    gdouble *fx = gwy_field_get_data(freq_x);
+    gdouble *fy = gwy_field_get_data(freq_y);
     gdouble vx, vy;
     guint i, j;
 
@@ -449,12 +437,12 @@ precalculate_frequencies(GwyDataField *model, GwyDataField *freq_x, GwyDataField
  * Do not bother with weighting inside the fitting when we can just premultiply model and data to achieve the same
  * effect on cheap. */
 static void
-weight_fourier_components(GwyDataField *fftfield, GwyDataField *freq_x, GwyDataField *freq_y)
+weight_fourier_components(GwyField *fftfield, GwyField *freq_x, GwyField *freq_y)
 {
-    guint xres = gwy_data_field_get_xres(fftfield), yres = gwy_data_field_get_yres(fftfield);
-    const gdouble *fx = gwy_data_field_get_data(freq_x);
-    const gdouble *fy = gwy_data_field_get_data(freq_y);
-    gdouble *d = gwy_data_field_get_data(fftfield);
+    guint xres = gwy_field_get_xres(fftfield), yres = gwy_field_get_yres(fftfield);
+    const gdouble *fx = gwy_field_get_data(freq_x);
+    const gdouble *fy = gwy_field_get_data(freq_y);
+    gdouble *d = gwy_field_get_data(fftfield);
     guint k = (yres/2)*xres + xres/2;
     gdouble fmax2 = fx[k]*fx[k] + fy[k]*fy[k];
     gdouble factor = 3.0/fmax2;
@@ -480,25 +468,25 @@ debug_print_params(const PSFFunction *func, const gdouble *params)
 #define debug_print_params(func,params) /* */
 #endif
 
-static GwyDataField*
-prepare_field(GwyDataField *field, GwyWindowingType window)
+static GwyField*
+prepare_field(GwyField *field, GwyWindowingType window)
 {
-    GwyDataField *wfield;
+    GwyField *wfield;
 
-    wfield = gwy_data_field_duplicate(field);
-    gwy_data_field_add(wfield, -gwy_data_field_get_avg(wfield));
-    gwy_data_field_fft_window(wfield, window);
+    wfield = gwy_field_copy(field);
+    gwy_field_add(wfield, -gwy_field_get_avg(wfield));
+    gwy_field_fft_window(wfield, window);
 
     return wfield;
 }
 
 static void
-fit_psf(GwyDataField *model, GwyDataField *data, GwyDataField *psf,
+fit_psf(GwyField *model, GwyField *data, GwyField *psf,
         const PSFFunction *func, GwyWindowingType windowing)
 {
     PSFEstimateData psfedata;
-    guint xres = gwy_data_field_get_xres(model), yres = gwy_data_field_get_yres(model);
-    GwyDataField *model_re, *model_im, *data_re, *data_im, *freq_x, *freq_y, *xd, *xm;
+    guint xres = gwy_field_get_xres(model), yres = gwy_field_get_yres(model);
+    GwyField *model_re, *model_im, *data_re, *data_im, *freq_x, *freq_y, *xd, *xm;
     GwyNLFitter *fitter = NULL;
     gdouble rss;
     gdouble *params = g_new(gdouble, func->nparams);
@@ -507,20 +495,20 @@ fit_psf(GwyDataField *model, GwyDataField *data, GwyDataField *psf,
     xm = freq_x = prepare_field(model, windowing);
     xd = freq_y = prepare_field(data, windowing);
 
-    model_re = gwy_data_field_new_alike(xm, FALSE);
-    model_im = gwy_data_field_new_alike(xm, FALSE);
-    gwy_data_field_2dfft_raw(xm, NULL, model_re, model_im, GWY_TRANSFORM_DIRECTION_FORWARD);
+    model_re = gwy_field_new_alike(xm, FALSE);
+    model_im = gwy_field_new_alike(xm, FALSE);
+    gwy_field_fft_2d_raw(xm, NULL, model_re, model_im, GWY_TRANSFORM_DIRECTION_FORWARD);
 
-    data_re = gwy_data_field_new_alike(xd, FALSE);
-    data_im = gwy_data_field_new_alike(xd, FALSE);
-    gwy_data_field_2dfft_raw(xd, NULL, data_re, data_im, GWY_TRANSFORM_DIRECTION_FORWARD);
+    data_re = gwy_field_new_alike(xd, FALSE);
+    data_im = gwy_field_new_alike(xd, FALSE);
+    gwy_field_fft_2d_raw(xd, NULL, data_re, data_im, GWY_TRANSFORM_DIRECTION_FORWARD);
 
     precalculate_frequencies(model, freq_x, freq_y);
 
     if (!func->initpar(model_re, model_im, data_re, data_im, freq_x, freq_y, params)) {
         g_warning("Initial parameter estimation failed.");
-        gwy_data_field_clear(psf);
-        gwy_data_field_set_val(psf, 0, 0, 1.0);
+        gwy_field_clear(psf);
+        gwy_field_set_val(psf, 0, 0, 1.0);
         goto fail;
     }
 
@@ -531,20 +519,20 @@ fit_psf(GwyDataField *model, GwyDataField *data, GwyDataField *psf,
 
     psfedata.xres = xres;
     psfedata.yres = yres;
-    psfedata.model_re = gwy_data_field_get_data(model_re);
-    psfedata.model_im = gwy_data_field_get_data(model_im);
-    psfedata.data_re = gwy_data_field_get_data(data_re);
-    psfedata.data_im = gwy_data_field_get_data(data_im);
-    psfedata.xfreq = gwy_data_field_get_data(freq_x);
-    psfedata.yfreq = gwy_data_field_get_data(freq_y);
+    psfedata.model_re = gwy_field_get_data(model_re);
+    psfedata.model_im = gwy_field_get_data(model_im);
+    psfedata.data_re = gwy_field_get_data(data_re);
+    psfedata.data_im = gwy_field_get_data(data_im);
+    psfedata.xfreq = gwy_field_get_data(freq_x);
+    psfedata.yfreq = gwy_field_get_data(freq_y);
 
     fitter = gwy_math_nlfit_new_idx(func->func, func->diff);
     rss = gwy_math_nlfit_fit_idx(fitter, 2*xres*yres, func->nparams, params, &psfedata);
     gwy_debug("Fitted rss %g", rss);
     if (!(rss >= 0.0)) {
         g_warning("Initial parameter estimation failed.");
-        gwy_data_field_clear(psf);
-        gwy_data_field_set_val(psf, 0, 0, 1.0);
+        gwy_field_clear(psf);
+        gwy_field_set_val(psf, 0, 0, 1.0);
         goto fail;
     }
     debug_print_params(func, params);
@@ -567,27 +555,27 @@ fail:
 }
 
 static void
-set_transfer_function_units(GwyDataField *ideal, GwyDataField *measured, GwyDataField *transferfunc)
+set_transfer_function_units(GwyField *ideal, GwyField *measured, GwyField *transferfunc)
 {
-    GwySIUnit *sunit, *iunit, *tunit, *xyunit;
+    GwyUnit *sunit, *iunit, *tunit, *xyunit;
 
-    xyunit = gwy_data_field_get_si_unit_xy(measured);
-    sunit = gwy_data_field_get_si_unit_z(ideal);
-    iunit = gwy_data_field_get_si_unit_z(measured);
-    tunit = gwy_data_field_get_si_unit_z(transferfunc);
-    gwy_si_unit_divide(iunit, sunit, tunit);
-    gwy_si_unit_power_multiply(tunit, 1, xyunit, -2, tunit);
+    xyunit = gwy_field_get_unit_xy(measured);
+    sunit = gwy_field_get_unit_z(ideal);
+    iunit = gwy_field_get_unit_z(measured);
+    tunit = gwy_field_get_unit_z(transferfunc);
+    gwy_unit_divide(iunit, sunit, tunit);
+    gwy_unit_power_multiply(tunit, 1, xyunit, -2, tunit);
 }
 
 static void
-adjust_tf_to_non_integral(GwyDataField *psf)
+adjust_tf_to_non_integral(GwyField *psf)
 {
-    GwySIUnit *xyunit, *zunit;
+    GwyUnit *xyunit, *zunit;
 
-    xyunit = gwy_data_field_get_si_unit_xy(psf);
-    zunit = gwy_data_field_get_si_unit_z(psf);
-    gwy_si_unit_power_multiply(zunit, 1, xyunit, 2, zunit);
-    gwy_data_field_multiply(psf, gwy_data_field_get_dx(psf) * gwy_data_field_get_dy(psf));
+    xyunit = gwy_field_get_unit_xy(psf);
+    zunit = gwy_field_get_unit_z(psf);
+    gwy_unit_power_multiply(zunit, 1, xyunit, 2, zunit);
+    gwy_field_multiply(psf, gwy_field_get_dx(psf) * gwy_field_get_dy(psf));
 }
 
 static gdouble
@@ -655,18 +643,18 @@ psf_gaussian_fit_diff(guint i,
 }
 
 static gboolean
-psf_gaussian_init_params(GwyDataField *model_re, GwyDataField *model_im,
-                         GwyDataField *data_re, GwyDataField *data_im,
-                         GwyDataField *freq_x, GwyDataField *freq_y,
+psf_gaussian_init_params(GwyField *model_re, GwyField *model_im,
+                         GwyField *data_re, GwyField *data_im,
+                         GwyField *freq_x, GwyField *freq_y,
                          gdouble *params)
 {
-    guint xres = gwy_data_field_get_xres(model_re), yres = gwy_data_field_get_yres(model_re);
-    const gdouble *mre = gwy_data_field_get_data_const(model_re);
-    const gdouble *mim = gwy_data_field_get_data_const(model_im);
-    const gdouble *dre = gwy_data_field_get_data_const(data_re);
-    const gdouble *dim = gwy_data_field_get_data_const(data_im);
-    const gdouble *fx = gwy_data_field_get_data_const(freq_x);
-    const gdouble *fy = gwy_data_field_get_data_const(freq_y);
+    guint xres = gwy_field_get_xres(model_re), yres = gwy_field_get_yres(model_re);
+    const gdouble *mre = gwy_field_get_data_const(model_re);
+    const gdouble *mim = gwy_field_get_data_const(model_im);
+    const gdouble *dre = gwy_field_get_data_const(data_re);
+    const gdouble *dim = gwy_field_get_data_const(data_im);
+    const gdouble *fx = gwy_field_get_data_const(freq_x);
+    const gdouble *fy = gwy_field_get_data_const(freq_y);
     gdouble q_model, q_data, w_model, w_data;
 
     /* Amplitude. */
@@ -688,16 +676,16 @@ psf_gaussian_init_params(GwyDataField *model_re, GwyDataField *model_im,
 }
 
 static void
-psf_gaussian_fill_psf(GwyDataField *freq_x, GwyDataField *freq_y,
-                      G_GNUC_UNUSED GwyDataField *buf_re, GwyDataField *buf_im,
-                      GwyDataField *psf, GwyDataField *psf_fft,
+psf_gaussian_fill_psf(GwyField *freq_x, GwyField *freq_y,
+                      G_GNUC_UNUSED GwyField *buf_re, GwyField *buf_im,
+                      GwyField *psf, GwyField *psf_fft,
                       const gdouble *param)
 {
     gdouble A = param[0], w = param[1];
-    guint xres = gwy_data_field_get_xres(freq_x), yres = gwy_data_field_get_yres(freq_x);
-    const gdouble *fx = gwy_data_field_get_data_const(freq_x);
-    const gdouble *fy = gwy_data_field_get_data_const(freq_y);
-    gdouble *pf = gwy_data_field_get_data(psf_fft);
+    guint xres = gwy_field_get_xres(freq_x), yres = gwy_field_get_yres(freq_x);
+    const gdouble *fx = gwy_field_get_data_const(freq_x);
+    const gdouble *fy = gwy_field_get_data_const(freq_y);
+    gdouble *pf = gwy_field_get_data(psf_fft);
     gdouble r2, g;
     guint k;
 
@@ -707,8 +695,8 @@ psf_gaussian_fill_psf(GwyDataField *freq_x, GwyDataField *freq_y,
         pf[k] = A*g;
     }
 
-    gwy_data_field_2dfft_raw(psf_fft, NULL, psf, buf_im, GWY_TRANSFORM_DIRECTION_BACKWARD);
-    gwy_data_field_2dfft_humanize(psf);
+    gwy_field_fft_2d_raw(psf_fft, NULL, psf, buf_im, GWY_TRANSFORM_DIRECTION_BACKWARD);
+    gwy_field_fft_2d_center(psf);
 }
 
 static gdouble
@@ -762,9 +750,9 @@ psf_agaussian_fit_diff(guint i,
 }
 
 static gboolean
-psf_agaussian_init_params(GwyDataField *model_re, GwyDataField *model_im,
-                          GwyDataField *data_re, GwyDataField *data_im,
-                          GwyDataField *freq_x, GwyDataField *freq_y,
+psf_agaussian_init_params(GwyField *model_re, GwyField *model_im,
+                          GwyField *data_re, GwyField *data_im,
+                          GwyField *freq_x, GwyField *freq_y,
                           gdouble *params)
 {
     if (!psf_gaussian_init_params(model_re, model_im, data_re, data_im, freq_x, freq_y, params))
@@ -775,16 +763,16 @@ psf_agaussian_init_params(GwyDataField *model_re, GwyDataField *model_im,
 }
 
 static void
-psf_agaussian_fill_psf(GwyDataField *freq_x, GwyDataField *freq_y,
-                       G_GNUC_UNUSED GwyDataField *buf_re, GwyDataField *buf_im,
-                       GwyDataField *psf, GwyDataField *psf_fft,
+psf_agaussian_fill_psf(GwyField *freq_x, GwyField *freq_y,
+                       G_GNUC_UNUSED GwyField *buf_re, GwyField *buf_im,
+                       GwyField *psf, GwyField *psf_fft,
                        const gdouble *param)
 {
     gdouble A = param[0], wx = param[1], wy = param[2];
-    guint xres = gwy_data_field_get_xres(freq_x), yres = gwy_data_field_get_yres(freq_x);
-    const gdouble *fx = gwy_data_field_get_data_const(freq_x);
-    const gdouble *fy = gwy_data_field_get_data_const(freq_y);
-    gdouble *pf = gwy_data_field_get_data(psf_fft);
+    guint xres = gwy_field_get_xres(freq_x), yres = gwy_field_get_yres(freq_x);
+    const gdouble *fx = gwy_field_get_data_const(freq_x);
+    const gdouble *fy = gwy_field_get_data_const(freq_y);
+    gdouble *pf = gwy_field_get_data(psf_fft);
     gdouble r2, g;
     guint k;
 
@@ -794,8 +782,8 @@ psf_agaussian_fill_psf(GwyDataField *freq_x, GwyDataField *freq_y,
         pf[k] = A*g;
     }
 
-    gwy_data_field_2dfft_raw(psf_fft, NULL, psf, buf_im, GWY_TRANSFORM_DIRECTION_BACKWARD);
-    gwy_data_field_2dfft_humanize(psf);
+    gwy_field_fft_2d_raw(psf_fft, NULL, psf, buf_im, GWY_TRANSFORM_DIRECTION_BACKWARD);
+    gwy_field_fft_2d_center(psf);
 }
 
 /* We fit G*model on data, so our residuum function is G*model-data. */
@@ -849,18 +837,18 @@ psf_exponential_fit_diff(guint i,
 }
 
 static gboolean
-psf_exponential_init_params(GwyDataField *model_re, GwyDataField *model_im,
-                            GwyDataField *data_re, GwyDataField *data_im,
-                            GwyDataField *freq_x, GwyDataField *freq_y,
+psf_exponential_init_params(GwyField *model_re, GwyField *model_im,
+                            GwyField *data_re, GwyField *data_im,
+                            GwyField *freq_x, GwyField *freq_y,
                             gdouble *params)
 {
-    guint xres = gwy_data_field_get_xres(model_re), yres = gwy_data_field_get_yres(model_re);
-    const gdouble *mre = gwy_data_field_get_data_const(model_re);
-    const gdouble *mim = gwy_data_field_get_data_const(model_im);
-    const gdouble *dre = gwy_data_field_get_data_const(data_re);
-    const gdouble *dim = gwy_data_field_get_data_const(data_im);
-    const gdouble *fx = gwy_data_field_get_data_const(freq_x);
-    const gdouble *fy = gwy_data_field_get_data_const(freq_y);
+    guint xres = gwy_field_get_xres(model_re), yres = gwy_field_get_yres(model_re);
+    const gdouble *mre = gwy_field_get_data_const(model_re);
+    const gdouble *mim = gwy_field_get_data_const(model_im);
+    const gdouble *dre = gwy_field_get_data_const(data_re);
+    const gdouble *dim = gwy_field_get_data_const(data_im);
+    const gdouble *fx = gwy_field_get_data_const(freq_x);
+    const gdouble *fy = gwy_field_get_data_const(freq_y);
     gdouble q_model, q_data, w_model, w_data;
 
     /* Amplitude. */
@@ -882,17 +870,17 @@ psf_exponential_init_params(GwyDataField *model_re, GwyDataField *model_im,
 }
 
 static void
-psf_exponential_fill_psf(GwyDataField *freq_x, GwyDataField *freq_y,
-                         G_GNUC_UNUSED GwyDataField *buf_re,
-                         GwyDataField *buf_im,
-                         GwyDataField *psf, GwyDataField *psf_fft,
+psf_exponential_fill_psf(GwyField *freq_x, GwyField *freq_y,
+                         G_GNUC_UNUSED GwyField *buf_re,
+                         GwyField *buf_im,
+                         GwyField *psf, GwyField *psf_fft,
                          const gdouble *param)
 {
     gdouble A = param[0], w = param[1];
-    guint xres = gwy_data_field_get_xres(freq_x), yres = gwy_data_field_get_yres(freq_x);
-    const gdouble *fx = gwy_data_field_get_data_const(freq_x);
-    const gdouble *fy = gwy_data_field_get_data_const(freq_y);
-    gdouble *pf = gwy_data_field_get_data(psf_fft);
+    guint xres = gwy_field_get_xres(freq_x), yres = gwy_field_get_yres(freq_x);
+    const gdouble *fx = gwy_field_get_data_const(freq_x);
+    const gdouble *fy = gwy_field_get_data_const(freq_y);
+    gdouble *pf = gwy_field_get_data(psf_fft);
     gdouble r2, g;
     guint k;
 
@@ -902,8 +890,8 @@ psf_exponential_fill_psf(GwyDataField *freq_x, GwyDataField *freq_y,
         pf[k] = A*g;
     }
 
-    gwy_data_field_2dfft_raw(psf_fft, NULL, psf, buf_im, GWY_TRANSFORM_DIRECTION_BACKWARD);
-    gwy_data_field_2dfft_humanize(psf);
+    gwy_field_fft_2d_raw(psf_fft, NULL, psf, buf_im, GWY_TRANSFORM_DIRECTION_BACKWARD);
+    gwy_field_fft_2d_center(psf);
 }
 
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
