@@ -1,5 +1,5 @@
 /*
- *  $Id: rank-filter.c 23666 2021-05-10 21:56:10Z yeti-dn $
+ *  $Id: rank-filter.c 28821 2025-11-06 16:00:41Z yeti-dn $
  *  Copyright (C) 2019-2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,17 +16,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/datafield.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/elliptic.h>
-#include <libprocess/filters.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
@@ -49,9 +42,9 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *result;
-    GwyDataField *result2;
+    GwyField *field;
+    GwyField *result;
+    GwyField *result2;
 } ModuleArgs;
 
 typedef struct {
@@ -63,16 +56,16 @@ typedef struct {
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             rank_filter         (GwyContainer *data,
-                                             GwyRunType runtype);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static gboolean         execute             (ModuleArgs *args,
                                              GtkWindow *wait_window);
 static GwyDialogOutcome run_gui             (ModuleArgs *args);
 static void             param_changed       (ModuleGUI *gui,
                                              gint id);
-static void             add_new_field       (GwyContainer *data,
+static void             add_new_field       (GwyFile *data,
                                              gint oldid,
-                                             GwyDataField *field,
+                                             GwyField *field,
                                              gdouble fraction);
 static void             sanitise_params     (ModuleArgs *args);
 
@@ -92,11 +85,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("rank_filter",
-                              (GwyProcessFunc)&rank_filter,
+                              module_main,
                               N_("/_Integral Transforms/_Rank Filter..."),
                               NULL,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("K-th rank filter"));
 
     return TRUE;
@@ -124,37 +117,37 @@ define_module_params(void)
 }
 
 static void
-rank_filter(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     ModuleArgs args;
     gint id;
     GQuark quark;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_DATA_FIELD_KEY, &quark,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_FIELD_KEY, &quark,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(args.field);
 
-    args.result = gwy_data_field_new_alike(args.field, FALSE);
+    args.result = gwy_field_new_alike(args.field, FALSE);
     args.params = gwy_params_new_from_settings(define_module_params());
     sanitise_params(&args);
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
             goto end;
     }
-    if (!execute(&args, gwy_app_find_window_for_channel(data, id)))
+    if (!execute(&args, gwy_data_browser_get_window_for_data(data, GWY_FILE_IMAGE, id)))
         goto end;
 
     if (gwy_params_get_boolean(args.params, PARAM_BOTH)) {
         if (gwy_params_get_boolean(args.params, PARAM_DIFFERENCE)) {
-            gwy_data_field_subtract_fields(args.result, args.result, args.result2);
+            gwy_field_subtract_fields(args.result, args.result, args.result2);
             add_new_field(data, id, args.result, -1.0);
         }
         else {
@@ -166,36 +159,38 @@ rank_filter(GwyContainer *data, GwyRunType runtype)
         if (gwy_params_get_boolean(args.params, PARAM_NEW_IMAGE))
             add_new_field(data, id, args.result, gwy_params_get_double(args.params, PARAM_PERCENTILE1));
         else {
-            gwy_app_undo_qcheckpointv(data, 1, &quark);
-            gwy_container_set_object(data, quark, args.result);
-            gwy_app_channel_log_add_proc(data, id, id);
+            gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &quark);
+            gwy_file_pass_image(data, id, args.result);
+            gwy_log_add(data, GWY_FILE_IMAGE, id, id);
+            args.result = NULL;
         }
     }
 
 end:
-    GWY_OBJECT_UNREF(args.result2);
-    g_object_unref(args.result);
+    g_clear_object(&args.result2);
+    g_clear_object(&args.result);
     g_object_unref(args.params);
 }
 
 static void
-add_new_field(GwyContainer *data, gint oldid, GwyDataField *field, gdouble fraction)
+add_new_field(GwyFile *data, gint oldid, GwyField *field, gdouble fraction)
 {
     gint newid;
 
-    newid = gwy_app_data_browser_add_data_field(field, data, TRUE);
-    gwy_app_sync_data_items(data, data, oldid, newid, FALSE,
-                            GWY_DATA_ITEM_GRADIENT,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
+    newid = gwy_file_add_image(data, field);
+
+    gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+    gwy_file_sync_items(data, GWY_FILE_IMAGE, oldid,
+                        data, GWY_FILE_IMAGE, newid,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_REAL_SQUARE, FALSE);
     if (fraction < 0.0)
-        gwy_app_set_data_field_title(data, newid, _("Rank difference"));
+        gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _("Rank difference"), TRUE);
     else {
         gchar *s = g_strdup_printf(_("Rank filtered (%.1f %%)"), 100.0*fraction);
-        gwy_app_set_data_field_title(data, newid, s);
+        gwy_file_set_title(data, GWY_FILE_IMAGE, newid, s, TRUE);
         g_free(s);
     }
-    gwy_app_channel_log_add(data, oldid, newid, NULL, NULL);
+    gwy_log_add_full(data, GWY_FILE_IMAGE, oldid, newid, NULL, NULL);
 }
 
 static GwyDialogOutcome
@@ -205,7 +200,7 @@ run_gui(ModuleArgs *args)
     GwyDialog *dialog;
     GwyParamTable *table;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
 
     gui.dialog = gwy_dialog_new(_("Rank Filter"));
@@ -254,7 +249,7 @@ param_changed(ModuleGUI *gui, gint id)
 
     if (id < 0 || id == PARAM_SIZE) {
         gint kres = 2*size + 1;
-        gint n = gwy_data_field_get_elliptic_area_size(kres, kres);
+        gint n = gwy_field_get_elliptic_area_size(kres, kres);
 
         percentscale = gui->percentscale = 1.0/n;
         gwy_param_table_slider_restrict_range(table, PARAM_RANK1, 0.0, n - 1.0);
@@ -282,26 +277,26 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
     gdouble percentile1 = gwy_params_get_double(args->params, PARAM_PERCENTILE1);
     gdouble percentile2 = gwy_params_get_double(args->params, PARAM_PERCENTILE2);
     gint xres, yres, rank, kres = 2*size + 1;
-    gint n = gwy_data_field_get_elliptic_area_size(kres, kres);
-    GwyDataField *kernel, *field = args->field, *result = args->result;
+    gint n = gwy_field_get_elliptic_area_size(kres, kres);
+    GwyField *kernel, *field = args->field, *result = args->result;
     gboolean ok = FALSE;
 
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
 
     gwy_app_wait_start(wait_window, _("Filtering..."));
 
-    kernel = gwy_data_field_new(kres, kres, 1.0, 1.0, TRUE);
-    gwy_data_field_elliptic_area_fill(kernel, 0, 0, kres, kres, 1.0);
-    gwy_data_field_copy(field, result, FALSE);
+    kernel = gwy_field_new(kres, kres, 1.0, 1.0, TRUE);
+    gwy_field_elliptic_area_fill(kernel, 0, 0, kres, kres, 1.0);
+    gwy_field_copy_data(field, result);
     rank = GWY_ROUND(percentile1*(n - 1));
-    if (!gwy_data_field_area_filter_kth_rank(result, kernel, 0, 0, xres, yres, rank, gwy_app_wait_set_fraction))
+    if (!gwy_field_area_filter_kth_rank(result, kernel, 0, 0, xres, yres, rank, gwy_app_wait_set_fraction))
         goto end;
 
     if (both) {
-        args->result2 = result = gwy_data_field_duplicate(field);
+        args->result2 = result = gwy_field_copy(field);
         rank = GWY_ROUND(percentile2*(n - 1));
-        if (!gwy_data_field_area_filter_kth_rank(result, kernel, 0, 0, xres, yres, rank, gwy_app_wait_set_fraction))
+        if (!gwy_field_area_filter_kth_rank(result, kernel, 0, 0, xres, yres, rank, gwy_app_wait_set_fraction))
             goto end;
     }
 
