@@ -1,5 +1,5 @@
 /*
- *  $Id: raveraging.c 23666 2021-05-10 21:56:10Z yeti-dn $
+ *  $Id: raveraging.c 28844 2025-11-10 16:07:51Z yeti-dn $
  *  Copyright (C) 2019-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,15 +16,9 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libprocess/correct.h>
-#include <libprocess/filters.h>
-#include <libprocess/arithmetic.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
@@ -43,7 +37,7 @@ typedef struct {
 } TransformData;
 
 typedef struct {
-    GwyDataField *field;
+    GwyField *field;
     GwyParams *params;
 } ModuleArgs;
 
@@ -55,10 +49,10 @@ typedef struct {
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             raveraging          (GwyContainer *data,
-                                             GwyRunType runtype);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static GwyDialogOutcome run_gui             (ModuleArgs *args);
-static void             filter_radial       (GwyDataField *field,
+static void             filter_radial       (GwyField *field,
                                              gdouble radius,
                                              gdouble angle,
                                              GwyInterpolationType interp);
@@ -79,11 +73,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("raveraging",
-                              (GwyProcessFunc)&raveraging,
+                              module_main,
                               N_("/_Distortion/_Radial Smoothing..."),
                               NULL,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Smooth image in polar coordinates"));
 
     return TRUE;
@@ -99,7 +93,7 @@ define_module_params(void)
 
     paramdef = gwy_param_def_new();
     gwy_param_def_set_function_name(paramdef, gwy_process_func_current());
-    gwy_param_def_add_angle(paramdef, PARAM_ANGLE, "angle", _("_Angle"), TRUE, 2, 10.0*G_PI/180.0);
+    gwy_param_def_add_angle(paramdef, PARAM_ANGLE, "angle", _("_Angle"), TRUE, 2, gwy_deg2rad(10.0));
     gwy_param_def_add_double(paramdef, PARAM_RADIUS, "radius", _("_Radius"), 0.0, 1000.0, 10.0);
     gwy_param_def_add_enum(paramdef, PARAM_INTERP, "interp", NULL, GWY_TYPE_INTERPOLATION_TYPE,
                            GWY_INTERPOLATION_LINEAR);
@@ -107,42 +101,41 @@ define_module_params(void)
 }
 
 static void
-raveraging(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
-    GwyDataField *field;
+    GwyField *field;
     GwyParams *params;
     ModuleArgs args;
     gint oldid, newid;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &field,
-                                     GWY_APP_DATA_FIELD_ID, &oldid,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &field,
+                                 GWY_APP_FIELD_ID, &oldid,
+                                 0);
     g_return_if_fail(field);
 
     args.field = field;
     args.params = params = gwy_params_new_from_settings(define_module_params());
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         GwyDialogOutcome outcome = run_gui(&args);
         gwy_params_save_to_settings(params);
         if (outcome != GWY_DIALOG_PROCEED)
             goto end;
     }
 
-    field = gwy_data_field_duplicate(field);
+    field = gwy_field_copy(field);
     filter_radial(field,
                   gwy_params_get_double(params, PARAM_RADIUS), gwy_params_get_double(params, PARAM_ANGLE),
                   gwy_params_get_enum(params, PARAM_INTERP));
-    newid = gwy_app_data_browser_add_data_field(field, data, TRUE);
-    gwy_app_sync_data_items(data, data, oldid, newid, FALSE,
-                            GWY_DATA_ITEM_GRADIENT,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            GWY_DATA_ITEM_MASK_COLOR,
-                            0);
+    newid = gwy_file_add_image(data, field);
+    gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+    gwy_file_sync_items(data, GWY_FILE_IMAGE, oldid,
+                        data, GWY_FILE_IMAGE, newid,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_REAL_SQUARE | GWY_FILE_ITEM_MASK_COLOR, FALSE);
     g_object_unref(field);
-    gwy_app_set_data_field_title(data, newid, _("Radially smoothed"));
-    gwy_app_channel_log_add_proc(data, oldid, newid);
+    gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _("Radially smoothed"), TRUE);
+    gwy_log_add(data, GWY_FILE_IMAGE, oldid, newid);
 
 end:
     g_object_unref(params);
@@ -155,7 +148,7 @@ run_gui(ModuleArgs *args)
     GwyParamTable *table;
     ModuleGUI gui;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
 
     gui.dialog = gwy_dialog_new(_("Radial Smoothing"));
@@ -171,7 +164,7 @@ run_gui(ModuleArgs *args)
     gwy_param_table_alt_set_field_pixel_x(table, PARAM_RADIUS, args->field);
 
     gwy_param_table_append_slider(table, PARAM_ANGLE);
-    gwy_param_table_slider_set_steps(table, PARAM_ANGLE, 0.1*G_PI/180.0, 10.0*G_PI/180.0);
+    gwy_param_table_slider_set_steps(table, PARAM_ANGLE, gwy_deg2rad(0.1), gwy_deg2rad(10.0));
     gwy_param_table_slider_set_digits(table, PARAM_ANGLE, 2);
     gwy_param_table_append_combo(table, PARAM_INTERP);
 
@@ -205,14 +198,14 @@ raverage_xy_to_rphi(gdouble x, gdouble y,
 }
 
 static void
-filter_radial(GwyDataField *field, gdouble radius, gdouble angle, GwyInterpolationType interp)
+filter_radial(GwyField *field, gdouble radius, gdouble angle, GwyInterpolationType interp)
 {
-    GwyDataField *rdfield, *erdfield;
+    GwyField *rdfield, *erdfield;
     gint xres, yres, rres, ares;
     TransformData transformdata;
 
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
     rres = sqrt(xres*xres + yres*yres)/2;
     ares = (GWY_ROUND(G_PI*MAX(xres, yres)) + 1)/2*2;
     transformdata.phiscale = 2.0*G_PI/ares;
@@ -220,21 +213,21 @@ filter_radial(GwyDataField *field, gdouble radius, gdouble angle, GwyInterpolati
     transformdata.xc = xres/2 + 0.5;
     transformdata.yc = yres/2 + 0.5;
 
-    rdfield = gwy_data_field_new(rres, ares, rres, ares, FALSE);
-    gwy_data_field_distort(field, rdfield, raverage_rphi_to_xy, &transformdata,
-                           interp, GWY_EXTERIOR_BORDER_EXTEND, 0.0);
+    rdfield = gwy_field_new(rres, ares, rres, ares, FALSE);
+    gwy_field_distort(field, rdfield, raverage_rphi_to_xy, &transformdata,
+                      interp, GWY_EXTERIOR_BORDER, 0.0);
 
     /* Additional 180 to cover large angular sigma. */
-    erdfield = gwy_data_field_extend(rdfield, 0, 0, ares, ares, GWY_EXTERIOR_PERIODIC, 0.0, FALSE);
+    erdfield = gwy_field_extend(rdfield, 0, 0, ares, ares, GWY_EXTERIOR_PERIODIC, 0.0, FALSE);
     g_object_unref(rdfield);
 
     if (radius > 0)
-        gwy_data_field_row_gaussian(erdfield, radius);
+        gwy_field_row_gaussian(erdfield, radius);
     if (angle > 0)
-        gwy_data_field_column_gaussian(erdfield, angle/(2.0*G_PI)*ares);
+        gwy_field_column_gaussian(erdfield, angle/(2.0*G_PI)*ares);
 
-    gwy_data_field_distort(erdfield, field, raverage_xy_to_rphi, &transformdata,
-                           interp, GWY_EXTERIOR_BORDER_EXTEND, 0.0);
+    gwy_field_distort(erdfield, field, raverage_xy_to_rphi, &transformdata,
+                      interp, GWY_EXTERIOR_BORDER, 0.0);
     g_object_unref(erdfield);
 }
 
