@@ -1,5 +1,5 @@
 /*
- *  $Id: relate.c 23788 2021-05-26 12:14:14Z yeti-dn $
+ *  $Id: relate.c 28821 2025-11-06 16:00:41Z yeti-dn $
  *  Copyright (C) 2018-2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,15 +16,9 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwynlfit.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/stats.h>
-#include <libgwydgets/gwygraph.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_INTERACTIVE)
@@ -85,8 +79,8 @@ typedef struct {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *mask;
+    GwyField *field;
+    GwyField *mask;
     GwyGraphModel *gmodel;
     gdouble *xdata;
     gdouble *ydata;
@@ -98,15 +92,10 @@ typedef struct {
     GtkWidget *dialog;
     GwyParamTable *table;
     GtkWidget *fit_table;
-    GtkWidget *param_name[MAX_PARAMS];
-    GtkWidget *param_equal[MAX_PARAMS];
-    GtkWidget *param_value[MAX_PARAMS];
-    GtkWidget *param_pm[MAX_PARAMS];
-    GtkWidget *param_error[MAX_PARAMS];
     GtkWidget *rss_label;
 
     GwyResults *results;
-    GwyContainer *args_data;
+    GwyFile *args_data;
     gint id;
 
     gdouble param[MAX_PARAMS];
@@ -129,16 +118,16 @@ DECLARE_FIT_FUNC(log);
 
 static gboolean              module_register        (void);
 static GwyParamDef*          define_module_params   (void);
-static void                  relate                 (GwyContainer *data,
-                                                     GwyRunType runtype);
+static void                  module_main            (GwyFile *data,
+                                                     GwyRunModeFlags mode);
 static GwyDialogOutcome      run_gui                (ModuleArgs *args,
-                                                     GwyContainer *data,
+                                                     GwyFile *data,
                                                      gint id);
 static GtkWidget*            create_fit_table       (gpointer user_data);
 static void                  param_changed          (ModuleGUI *gui,
                                                      gint id);
 static void                  preview                (gpointer user_data);
-static gboolean              other_image_filter     (GwyContainer *data,
+static gboolean              other_image_filter     (GwyFile *data,
                                                      gint id,
                                                      gpointer user_data);
 static const RelateFuncInfo* find_relate_func       (RelateFunction id);
@@ -150,7 +139,6 @@ static void                  update_results         (ModuleGUI *gui);
 static void                  fill_results           (ModuleGUI *gui);
 static void                  plot_fit               (ModuleGUI *gui);
 static gdouble               nlfitter_fit_func      (gdouble x,
-                                                     gint nparam,
                                                      const gdouble *param,
                                                      gpointer user_data,
                                                      gboolean *success);
@@ -234,7 +222,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Plots one image data as a function of another and finds relations."),
     "Yeti <yeti@gwyddion.net>",
-    "2.0",
+    "2.1",
     "David Nečas",
     "2018",
 };
@@ -245,11 +233,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("relate",
-                              (GwyProcessFunc)&relate,
+                              module_main,
                               N_("/_Multidata/_Relation..."),
-                              GWY_STOCK_IMAGE_RELATION,
+                              GWY_ICON_IMAGE_RELATION,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Find simple relations between data"));
 
     return TRUE;
@@ -280,18 +268,18 @@ define_module_params(void)
 }
 
 static void
-relate(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome;
     GwyAppDataId target_graph_id;
     ModuleArgs args;
     gint id, n;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_MASK_FIELD, &args.mask,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_MASK_FIELD, &args.mask,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(args.field);
 
     args.gmodel = gwy_graph_model_new();
@@ -301,7 +289,7 @@ relate(GwyContainer *data, GwyRunType runtype)
      * 2. After a second image is chosen (if any available), filter graphs.
      * Only this way the target graph setting can really be preserved.  Leave it to the image chooser ‘select
      * anything’ to select something.  */
-    n = gwy_data_field_get_xres(args.field) * gwy_data_field_get_yres(args.field);
+    n = gwy_field_get_xres(args.field) * gwy_field_get_yres(args.field);
     args.xdata = g_new(gdouble, 2*n);
     args.ydata = args.xdata + n;
 
@@ -320,14 +308,14 @@ end:
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     ModuleGUI gui;
     GtkWidget *hbox, *graph;
     GwyDialog *dialog;
     GwyParamTable *table;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
     gui.args_data = data;
     gui.id = id;
@@ -336,7 +324,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    hbox = gwy_hbox_new(8);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 8);
     gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
     gwy_dialog_add_content(dialog, hbox, TRUE, TRUE, 0);
 
@@ -380,11 +368,7 @@ static GtkWidget*
 create_fit_table(gpointer user_data)
 {
     ModuleGUI *gui = (ModuleGUI*)user_data;
-
-    gui->fit_table = gtk_table_new(1, 5, FALSE);
-    gtk_table_set_row_spacings(GTK_TABLE(gui->fit_table), 2);
-    gtk_table_set_col_spacings(GTK_TABLE(gui->fit_table), 8);
-    return gui->fit_table;
+    return gui->fit_table = gwy_fit_table_new();
 }
 
 static void
@@ -422,18 +406,16 @@ preview(gpointer user_data)
 }
 
 static gboolean
-other_image_filter(GwyContainer *data, gint id, gpointer user_data)
+other_image_filter(GwyFile *data, gint id, gpointer user_data)
 {
-    GwyDataField *otherfield, *field = (GwyDataField*)user_data;
+    GwyField *otherfield, *field = (GwyField*)user_data;
 
-    if (!gwy_container_gis_object(data, gwy_app_get_data_key_for_id(id), &otherfield))
+    if (!(otherfield = gwy_file_get_image(data, id)))
         return FALSE;
     if (otherfield == field)
         return FALSE;
-    return !gwy_data_field_check_compatibility(field, otherfield,
-                                               GWY_DATA_COMPATIBILITY_RES
-                                               | GWY_DATA_COMPATIBILITY_REAL
-                                               | GWY_DATA_COMPATIBILITY_LATERAL);
+    return !gwy_field_is_incompatible(field, otherfield,
+                                      GWY_DATA_MISMATCH_RES | GWY_DATA_MISMATCH_REAL | GWY_DATA_MISMATCH_LATERAL);
 }
 
 static const RelateFuncInfo*
@@ -453,8 +435,8 @@ replot_data(ModuleGUI *gui)
 {
     ModuleArgs *args = gui->args;
     GwyGraphModel *gmodel = args->gmodel;
-    GwyDataField *field = args->field, *mask = args->mask;
-    GwyDataField *otherfield = gwy_params_get_image(args->params, PARAM_OTHER_IMAGE);
+    GwyField *field = args->field, *mask = args->mask;
+    GwyField *otherfield = gwy_params_get_image(args->params, PARAM_OTHER_IMAGE);
     GwyMaskingType masking = gwy_params_get_masking(args->params, PARAM_MASKING, &mask);
     GwyGraphCurveModel *gcmodel;
     const gdouble *d1, *d2, *m;
@@ -481,20 +463,20 @@ replot_data(ModuleGUI *gui)
         return;
 
     g_object_set(gmodel,
-                 "si-unit-x", gwy_data_field_get_si_unit_z(field),
-                 "si-unit-y", gwy_data_field_get_si_unit_z(otherfield),
+                 "unit-x", gwy_field_get_unit_z(field),
+                 "unit-y", gwy_field_get_unit_z(otherfield),
                  NULL);
 
-    n = gwy_data_field_get_xres(field) * gwy_data_field_get_yres(field);
-    d1 = gwy_data_field_get_data_const(field);
-    d2 = gwy_data_field_get_data_const(otherfield);
+    n = gwy_field_get_xres(field) * gwy_field_get_yres(field);
+    d1 = gwy_field_get_data_const(field);
+    d2 = gwy_field_get_data_const(otherfield);
     if (!mask) {
         gwy_assign(xdata, d1, n);
         gwy_assign(ydata, d2, n);
         ndata = n;
     }
     else {
-        m = gwy_data_field_get_data_const(mask);
+        m = gwy_field_get_data_const(mask);
         ndata = 0;
         for (i = 0; i < n; i++) {
             if ((masking == GWY_MASK_INCLUDE && m[i] >= 1.0) || (masking == GWY_MASK_EXCLUDE && m[i] <= 0.0)) {
@@ -573,83 +555,44 @@ update_fit_result_table(ModuleGUI *gui)
 {
     RelateFunction func = gwy_params_get_enum(gui->args->params, PARAM_FUNC);
     const RelateFuncInfo *finfo = find_relate_func(func);
-    GtkTable *table = GTK_TABLE(gui->fit_table);
-    GtkWidget *label;
-    guint i, nparam = finfo->nparams;
+    GwyFitTable *table = GWY_FIT_TABLE(gui->fit_table);
+    guint nparam = finfo->nparams;
 
-    for (i = 0; i < MAX_PARAMS && gui->param_name[i]; i++) {
-        gtk_widget_destroy(gui->param_name[i]);
-        gtk_widget_destroy(gui->param_equal[i]);
-        gtk_widget_destroy(gui->param_value[i]);
-        gtk_widget_destroy(gui->param_pm[i]);
-        gtk_widget_destroy(gui->param_error[i]);
+    gwy_fit_table_resize(table, nparam);
+    for (guint i = 0; i < nparam; i++) {
+        gwy_fit_table_set_name(table, i, finfo->paraminfo[i].name);
+        gwy_fit_table_set_value(table, i, 0.0);
     }
-    gwy_clear(gui->param_name, i);
-    gwy_clear(gui->param_equal, i);
-    gwy_clear(gui->param_value, i);
-    gwy_clear(gui->param_pm, i);
-    gwy_clear(gui->param_error, i);
-
-    gtk_table_resize(table, nparam, 5);
-    for (i = 0; i < nparam; i++) {
-        label = gui->param_name[i] = gtk_label_new(NULL);
-        gtk_label_set_markup(GTK_LABEL(label), finfo->paraminfo[i].name);
-        gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
-        gtk_table_attach(table, label, 0, 1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
-
-        label = gui->param_equal[i] = gtk_label_new("=");
-        gtk_table_attach(table, label, 1, 2, i, i+1, GTK_FILL, 0, 0, 0);
-
-        label = gui->param_value[i] = gtk_label_new(NULL);
-        gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
-        gtk_table_attach(table, label, 2, 3, i, i+1, GTK_FILL, 0, 0, 0);
-
-        label = gui->param_pm[i] = gtk_label_new("±");
-        gtk_table_attach(table, label, 3, 4, i, i+1, GTK_FILL, 0, 0, 0);
-
-        label = gui->param_error[i] = gtk_label_new(NULL);
-        gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
-        gtk_table_attach(table, label, 4, 5, i, i+1, GTK_FILL, 0, 0, 0);
-    }
-    gtk_widget_show_all(gui->fit_table);
+    gwy_fit_table_clear_errors(table);
 }
 
 static void
 fill_fit_result_table(ModuleGUI *gui)
 {
-    const GwySIUnitFormatStyle style = GWY_SI_UNIT_FORMAT_VFMARKUP;
+    const GwyUnitFormatStyle style = GWY_UNIT_FORMAT_VFMARKUP;
     ModuleArgs *args = gui->args;
     RelateFunction func = gwy_params_get_enum(args->params, PARAM_FUNC);
     const RelateFuncInfo *finfo = find_relate_func(func);
-    guint i, nparam = finfo->nparams;
-    GwySIUnit *unit, *xunit, *yunit;
-    GwySIValueFormat *vf = NULL;
-    gchar buf[80];
+    GwyFitTable *table = GWY_FIT_TABLE(gui->fit_table);
+    guint nparam = finfo->nparams;
+    GwyUnit *unit, *xunit, *yunit;
 
-    xunit = gwy_data_field_get_si_unit_z(args->field);
-    yunit = gwy_data_field_get_si_unit_z(gwy_params_get_image(args->params, PARAM_OTHER_IMAGE));
-    unit = gwy_si_unit_new(NULL);
-    for (i = 0; i < nparam; i++) {
-        gwy_si_unit_power_multiply(xunit, finfo->paraminfo[i].power_x, yunit, finfo->paraminfo[i].power_y, unit);
-        vf = gwy_si_unit_get_format(unit, style, gui->param[i], vf);
-        vf->precision += 3;
-        g_snprintf(buf, sizeof(buf), "%.*f%s%s",
-                   vf->precision, gui->param[i]/vf->magnitude, *vf->units ? " " : "", vf->units);
-        gtk_label_set_markup(GTK_LABEL(gui->param_value[i]), buf);
-
-        vf = gwy_si_unit_get_format(unit, style, gui->error[i], vf);
-        g_snprintf(buf, sizeof(buf), "%.*f%s%s",
-                   vf->precision, gui->error[i]/vf->magnitude, *vf->units ? " " : "", vf->units);
-        gtk_label_set_markup(GTK_LABEL(gui->param_error[i]), buf);
+    xunit = gwy_field_get_unit_z(args->field);
+    yunit = gwy_field_get_unit_z(gwy_params_get_image(args->params, PARAM_OTHER_IMAGE));
+    unit = gwy_unit_new(NULL);
+    for (guint i = 0; i < nparam; i++) {
+        gwy_unit_power_multiply(xunit, finfo->paraminfo[i].power_x, yunit, finfo->paraminfo[i].power_y, unit);
+        gwy_fit_table_set_unit(table, i, unit);
+        gwy_fit_table_set_value(table, i, gui->param[i]);
+        gwy_fit_table_set_error(table, i, gui->error[i]);
     }
-
-    vf = gwy_si_unit_get_format(yunit, style, gui->rss, vf);
-    g_snprintf(buf, sizeof(buf), "%.*f %s",
-               vf->precision, gui->rss/vf->magnitude, vf->units);
-    gwy_param_table_info_set_valuestr(gui->table, INFO_RSS, buf);
-
-    gwy_si_unit_value_format_free(vf);
     g_object_unref(unit);
+
+    GwyValueFormat *vf = gwy_unit_get_format(yunit, style, gui->rss, NULL);
+    gchar *s = g_strdup_printf("%.*f %s", vf->precision, gui->rss/vf->magnitude, vf->units);
+    gwy_param_table_info_set_valuestr(gui->table, INFO_RSS, s);
+    g_free(s);
+    gwy_value_format_free(vf);
 }
 
 static void
@@ -660,7 +603,7 @@ update_results(ModuleGUI *gui)
     guint i, nparam = finfo->nparams;
     GwyResults *results;
 
-    GWY_OBJECT_UNREF(gui->results);
+    g_clear_object(&gui->results);
     results = gui->results = gwy_results_new();
     gwy_results_add_header(results, N_("Fit Results"));
     gwy_results_add_value_str(results, "file", N_("File"));
@@ -692,17 +635,18 @@ fill_results(ModuleGUI *gui)
     ModuleArgs *args = gui->args;
     RelateFunction func = gwy_params_get_enum(args->params, PARAM_FUNC);
     const RelateFuncInfo *finfo = find_relate_func(func);
-    GwyDataField *field = args->field, *otherfield = gwy_params_get_image(args->params, PARAM_OTHER_IMAGE);
+    GwyField *field = args->field, *otherfield = gwy_params_get_image(args->params, PARAM_OTHER_IMAGE);
     GwyResults *results = gui->results;
     const GwyAppDataId dataid = gwy_params_get_data_id(args->params, PARAM_OTHER_IMAGE);
     guint n, i;
 
-    n = gwy_data_field_get_xres(field) * gwy_data_field_get_yres(field);
-    gwy_results_fill_channel(results, "channel1", gui->args_data, gui->id);
-    gwy_results_fill_channel(results, "channel2", gwy_app_data_browser_get(dataid.datano), dataid.id);
-    gwy_results_set_unit(results, "x", gwy_data_field_get_si_unit_z(field));
-    gwy_results_set_unit(results, "y", gwy_data_field_get_si_unit_z(otherfield));
-    gwy_results_set_unit(results, "z", gwy_data_field_get_si_unit_z(otherfield));
+    n = gwy_field_get_xres(field) * gwy_field_get_yres(field);
+    gwy_results_fill_data_name(results, "channel1", gui->args_data, GWY_FILE_IMAGE, gui->id);
+    gwy_results_fill_data_name(results, "channel2",
+                               gwy_data_browser_get_file(dataid.datano), GWY_FILE_IMAGE, dataid.id);
+    gwy_results_set_unit(results, "x", gwy_field_get_unit_z(field));
+    gwy_results_set_unit(results, "y", gwy_field_get_unit_z(otherfield));
+    gwy_results_set_unit(results, "z", gwy_field_get_unit_z(otherfield));
 
     gwy_results_fill_filename(results, "file", gui->args_data);
     gwy_results_fill_values(results,
@@ -723,7 +667,7 @@ plot_fit(ModuleGUI *gui)
 {
     ModuleArgs *args = gui->args;
     RelateFunction func = gwy_params_get_enum(args->params, PARAM_FUNC);
-    GwyDataField *field = args->field, *mask = args->mask;
+    GwyField *field = args->field, *mask = args->mask;
     GwyMaskingType masking = gwy_params_get_masking(args->params, PARAM_MASKING, &mask);
     const RelateFuncInfo *finfo = find_relate_func(func);
     GwyGraphModel *gmodel = args->gmodel;
@@ -750,9 +694,9 @@ plot_fit(ModuleGUI *gui)
     ndata = PLOT_FUNC_SAMPLES;
     xdata = g_new(gdouble, 2*ndata);
     ydata = xdata + ndata;
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
-    gwy_data_field_area_get_min_max_mask(field, mask, masking, 0, 0, xres, yres, &min, &max);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
+    gwy_field_area_get_min_max(field, mask, masking, 0, 0, xres, yres, &min, &max);
 
     evalfunc = finfo->func;
     for (i = 0; i < ndata; i++) {
@@ -783,7 +727,6 @@ shuffle_array_stable(gdouble *a, guint n, guint nhead)
 
 static gdouble
 nlfitter_fit_func(gdouble x,
-                  G_GNUC_UNUSED gint nparam,
                   const gdouble *param,
                   gpointer user_data,
                   gboolean *success)
