@@ -1,5 +1,5 @@
 /*
- *  $Id: rotate.c 25638 2023-09-07 12:23:28Z yeti-dn $
+ *  $Id: rotate.c 28821 2025-11-06 16:00:41Z yeti-dn $
  *  Copyright (C) 2003-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,15 +16,9 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libprocess/stats.h>
-#include <libprocess/correct.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
@@ -44,7 +38,7 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
+    GwyField *field;
     gboolean is_unrotate;
     /* Cached estimated values for input data field (unrotate). */
     GwyPlaneSymmetry symm;
@@ -53,7 +47,7 @@ typedef struct {
 
 typedef struct {
     ModuleArgs *args;
-    GwyContainer *data;
+    GwyField *smallfield;
     GtkWidget *dialog;
     GtkWidget *dataview;
     GwyParamTable *table;
@@ -63,23 +57,19 @@ typedef struct {
 static gboolean         module_register        (void);
 static GwyParamDef*     define_rotate_params   (void);
 static GwyParamDef*     define_unrotate_params (void);
-static void             rotate_unrotate        (GwyContainer *data,
-                                                GwyRunType run,
-                                                const gchar *name);
+static void             module_main            (GwyFile *data,
+                                                GwyRunModeFlags mode);
 static GwyDialogOutcome run_gui                (ModuleArgs *args,
-                                                GwyContainer *data,
-                                                gint id);
-static GwyContainer*    create_preview_data    (GwyContainer *data,
-                                                GwyDataField *field,
+                                                GwyFile *data,
                                                 gint id);
 static void             param_changed          (ModuleGUI *gui,
                                                 gint id);
 static void             preview                (gpointer user_data);
 static void             update_grid            (ModuleGUI *gui);
-static void             fix_mask_exterior      (GwyDataField *mask,
-                                                GwyDataField *exterior_mask,
+static void             fix_mask_exterior      (GwyField *mask,
+                                                GwyField *exterior_mask,
                                                 gdouble fill_value);
-static GwyPlaneSymmetry guess_unrotate_symmetry(GwyDataField *field,
+static GwyPlaneSymmetry guess_unrotate_symmetry(GwyField *field,
                                                 gdouble *corrections);
 static gdouble          get_rotation_angle     (ModuleArgs *args);
 
@@ -99,18 +89,18 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("rotate",
-                              &rotate_unrotate,
+                              module_main,
                               N_("/_Basic Operations/Rotate by _Angle..."),
-                              GWY_STOCK_ROTATE,
+                              GWY_ICON_ROTATE,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Rotate by arbitrary angle"));
     gwy_process_func_register("unrotate",
-                              &rotate_unrotate,
+                              module_main,
                               N_("/_Correct Data/_Unrotate..."),
-                              GWY_STOCK_UNROTATE,
+                              GWY_ICON_UNROTATE,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Automatically correct rotation in horizontal plane"));
     return TRUE;
 }
@@ -162,9 +152,10 @@ define_unrotate_params(void)
 }
 
 static void
-rotate_unrotate(GwyContainer *data, GwyRunType runtype, const gchar *name)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
-    GwyDataField *fields[3], *exterior_mask = NULL;
+    const gchar *name = gwy_process_func_current();
+    GwyField *fields[3], *exterior_mask = NULL;
     gint oldid, newid, i;
     ModuleArgs args;
     GwyParams *params;
@@ -173,12 +164,12 @@ rotate_unrotate(GwyContainer *data, GwyRunType runtype, const gchar *name)
     gboolean create_mask;
     gdouble angle;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, fields + 0,
-                                     GWY_APP_MASK_FIELD, fields + 1,
-                                     GWY_APP_SHOW_FIELD, fields + 2,
-                                     GWY_APP_DATA_FIELD_ID, &oldid,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, fields + 0,
+                                 GWY_APP_MASK_FIELD, fields + 1,
+                                 GWY_APP_SHOW_FIELD, fields + 2,
+                                 GWY_APP_FIELD_ID, &oldid,
+                                 0);
     g_return_if_fail(fields[0]);
 
     args.field = fields[0];
@@ -190,7 +181,7 @@ rotate_unrotate(GwyContainer *data, GwyRunType runtype, const gchar *name)
     else
         params = args.params = gwy_params_new_from_settings(define_rotate_params());
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         GwyDialogOutcome outcome = run_gui(&args, data, oldid);
         gwy_params_save_to_settings(params);
         if (outcome != GWY_DIALOG_PROCEED)
@@ -203,11 +194,11 @@ rotate_unrotate(GwyContainer *data, GwyRunType runtype, const gchar *name)
     create_mask = gwy_params_get_boolean(params, PARAM_CREATE_MASK);
 
     if (fields[1] || (create_mask && resize != GWY_ROTATE_RESIZE_CUT))
-        exterior_mask = gwy_data_field_new(1, 1, 1.0, 1.0, FALSE);
+        exterior_mask = gwy_field_new(1, 1, 1.0, 1.0, FALSE);
 
-    fields[0] = gwy_data_field_new_rotated(fields[0], exterior_mask, angle, interp, resize);
+    fields[0] = gwy_field_new_rotated(fields[0], exterior_mask, angle, interp, resize);
     if (fields[1]) {
-        fields[1] = gwy_data_field_new_rotated(fields[1], NULL, angle, GWY_INTERPOLATION_ROUND, resize);
+        fields[1] = gwy_field_new_rotated(fields[1], NULL, angle, GWY_INTERPOLATION_ROUND, resize);
         /* The rotation fill exterior with average value of inside; which is kind of random and anyway unwanted for
          * masks.  Fill the exterior with either 0 or 1 (if we were asked to add an exterior mask). */
         fix_mask_exterior(fields[1], exterior_mask, create_mask ? 1.0 : 0.0);
@@ -219,52 +210,62 @@ rotate_unrotate(GwyContainer *data, GwyRunType runtype, const gchar *name)
             fields[1] = exterior_mask;
         }
     }
-    GWY_OBJECT_UNREF(exterior_mask);
+    g_clear_object(&exterior_mask);
 
     if (fields[2])
-        fields[2] = gwy_data_field_new_rotated(fields[2], NULL, angle, interp, resize);
+        fields[2] = gwy_field_new_rotated(fields[2], NULL, angle, interp, resize);
 
-    newid = gwy_app_data_browser_add_data_field(fields[0], data, TRUE);
-    gwy_app_sync_data_items(data, data, oldid, newid, FALSE,
-                            GWY_DATA_ITEM_GRADIENT,
-                            GWY_DATA_ITEM_RANGE,
-                            GWY_DATA_ITEM_MASK_COLOR,
-                            0);
+    newid = gwy_file_add_image(data, fields[0]);
+
+    gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+    gwy_file_sync_items(data, GWY_FILE_IMAGE, oldid,
+                        data, GWY_FILE_IMAGE, newid,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_RANGE | GWY_FILE_ITEM_MASK_COLOR, FALSE);
     if (fields[1])
-        gwy_container_set_object(data, gwy_app_get_mask_key_for_id(newid), fields[1]);
+        gwy_file_set_image_mask(data, newid, fields[1]);
     if (fields[2])
-        gwy_container_set_object(data, gwy_app_get_show_key_for_id(newid), fields[2]);
+        gwy_container_set_object(GWY_CONTAINER(data), gwy_file_key_image_picture(newid), fields[2]);
 
-    gwy_app_set_data_field_title(data, newid, _("Rotated Data"));
-    gwy_app_channel_log_add_proc(data, oldid, newid);
+    gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _("Rotated Data"), TRUE);
+    gwy_log_add(data, GWY_FILE_IMAGE, oldid, newid);
 
     for (i = 0; i < 3; i++)
-        GWY_OBJECT_UNREF(fields[i]);
+        g_clear_object(fields + i);
 
 end:
     g_object_unref(params);
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     GwyDialogOutcome outcome;
     GtkWidget *hbox, *dataview;
     GwyDialog *dialog;
     GwyParamTable *table;
+    GwyField *rfield;
     const gchar *s;
+    gint xres, yres;
+    gdouble zoomval;
     ModuleGUI gui;
 
     gui.args = args;
-    gui.data = create_preview_data(data, args->field, id);
+
+    xres = gwy_field_get_xres(args->field);
+    yres = gwy_field_get_yres(args->field);
+    zoomval = (gdouble)PREVIEW_SIZE/MAX(xres, yres);
+    gui.smallfield = gwy_field_new_resampled(args->field, xres*zoomval, yres*zoomval, GWY_INTERPOLATION_LINEAR);
+    rfield = gwy_field_copy(gui.smallfield);
 
     gui.dialog = gwy_dialog_new(args->is_unrotate ? _("Correct Rotation") : _("Rotate"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    dataview = gui.dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
-    /* TODO: Add selection setup helper outside preview.h. */
-    gui.selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(dataview), 0, "Lattice", 1, FALSE);
+    dataview = gui.dataview = gwy_create_preview(rfield, NULL, PREVIEW_SIZE);
+    g_object_unref(rfield);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_RANGE | GWY_FILE_ITEM_MASK_COLOR);
+    gui.selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(dataview), GWY_TYPE_LAYER_LATTICE, 1, FALSE);
 
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), TRUE);
 
@@ -273,7 +274,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     if (args->is_unrotate) {
         gwy_param_table_append_header(table, -1, _("Structure"));
         gwy_param_table_append_info(table, INFO_DETECTED, _("Detected"));
-        s = gwy_sgettext(gwy_enum_to_string(args->symm, gwy_plane_symmetry_get_enum(), -1));
+        s = gwy_C(gwy_enum_to_string(args->symm, gwy_plane_symmetry_get_enum(), -1));
         gwy_param_table_info_set_valuestr(table, INFO_DETECTED, s);
         gwy_param_table_append_combo(table, PARAM_SYMMETRY);
         gwy_param_table_append_info(table, INFO_CORRECTION, _("Correction"));
@@ -282,7 +283,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     else {
         gwy_param_table_append_header(table, -1, _("Rotate"));
         gwy_param_table_append_slider(table, PARAM_ANGLE);
-        gwy_param_table_slider_set_steps(table, PARAM_ANGLE, 0.01 * G_PI/180.0, 5.0 * G_PI/180.0);
+        gwy_param_table_slider_set_steps(table, PARAM_ANGLE, gwy_deg2rad(0.01), gwy_deg2rad(5.0));
     }
 
     gwy_param_table_append_header(table, -1, _("Options"));
@@ -296,39 +297,14 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 
     g_signal_connect_swapped(table, "param-changed", G_CALLBACK(param_changed), &gui);
     gwy_dialog_set_preview_func(dialog, GWY_PREVIEW_IMMEDIATE, preview, &gui, NULL);
+
     outcome = gwy_dialog_run(dialog);
 
-    g_object_unref(gui.data);
+    g_object_unref(gui.smallfield);
 
     return outcome;
 }
 
-static GwyContainer*
-create_preview_data(GwyContainer *data, GwyDataField *field, gint id)
-{
-    GwyContainer *preview_data;
-    GwyDataField *small_field, *rotated_field;
-    gint xres, yres;
-    gdouble zoomval;
-
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
-    zoomval = (gdouble)PREVIEW_SIZE/MAX(xres, yres);
-    small_field = gwy_data_field_new_resampled(field, xres*zoomval, yres*zoomval, GWY_INTERPOLATION_LINEAR);
-    rotated_field = gwy_data_field_duplicate(small_field);
-
-    preview_data = gwy_container_new();
-    gwy_container_pass_object_by_name(preview_data, "/1/data", small_field);
-    gwy_container_pass_object_by_name(preview_data, "/0/data", rotated_field);
-
-    gwy_app_sync_data_items(data, preview_data, id, 0, FALSE,
-                            GWY_DATA_ITEM_GRADIENT,
-                            GWY_DATA_ITEM_RANGE,
-                            GWY_DATA_ITEM_MASK_COLOR,
-                            0);
-    return preview_data;
-}
-
 static void
 param_changed(ModuleGUI *gui, gint id)
 {
@@ -345,7 +321,7 @@ param_changed(ModuleGUI *gui, gint id)
         update_grid(gui);
 
     if (args->is_unrotate && (id < 0 || id == PARAM_SYMMETRY)) {
-        gchar *s = g_strdup_printf("%.2f", 180.0/G_PI*get_rotation_angle(args));
+        gchar *s = g_strdup_printf("%.2f", gwy_rad2deg(get_rotation_angle(args)));
         gwy_param_table_info_set_valuestr(table, INFO_CORRECTION, s);
         g_free(s);
     }
@@ -360,15 +336,16 @@ preview(gpointer user_data)
     ModuleGUI *gui = (ModuleGUI*)user_data;
     ModuleArgs *args = gui->args;
     GwyParams *params = args->params;
-    GwyDataField *field, *rfield;
+    GwyField *rfield;
     gdouble angle = get_rotation_angle(args);
     GwyInterpolationType interp = gwy_params_get_enum(params, PARAM_INTERPOLATION);
     GwyRotateResizeType resize = gwy_params_get_enum(params, PARAM_RESIZE);
+    GwyDataView *dataview = GWY_DATA_VIEW(gui->dataview);
 
-    field = GWY_DATA_FIELD(gwy_container_get_object_by_name(gui->data, "/1/data"));
-    rfield = gwy_data_field_new_rotated(field, NULL, angle, interp, resize);
-    gwy_container_set_object_by_name(gui->data, "/0/data", rfield);
-    gwy_set_data_preview_size(GWY_DATA_VIEW(gui->dataview), PREVIEW_SIZE);
+    rfield = gwy_field_new_rotated(gui->smallfield, NULL, angle, interp, resize);
+    gwy_data_view_set_field(dataview, rfield);
+    g_object_unref(rfield);
+    gwy_set_data_preview_size(dataview, PREVIEW_SIZE);
     gtk_widget_set_size_request(gui->dataview, PREVIEW_SIZE, -1);
     update_grid(gui);
 }
@@ -378,7 +355,7 @@ update_grid(ModuleGUI *gui)
 {
     GwySelection *selection = gui->selection;
     GwyParams *params = gui->args->params;
-    GwyDataField *field;
+    GwyField *field = gui->smallfield;
     gdouble xy[4];
 
     if (!gwy_params_get_boolean(params, PARAM_SHOW_GRID)) {
@@ -386,22 +363,21 @@ update_grid(ModuleGUI *gui)
         return;
     }
 
-    field = gwy_container_get_object_by_name(gui->data, "/0/data");
-    xy[0] = gwy_data_field_get_xreal(field)/12.0;
+    xy[0] = gwy_field_get_xreal(field)/12.0;
     xy[1] = xy[2] = 0.0;
-    xy[3] = gwy_data_field_get_yreal(field)/12.0;
+    xy[3] = gwy_field_get_yreal(field)/12.0;
     gwy_selection_set_data(selection, 1, xy);
 }
 
 static void
-fix_mask_exterior(GwyDataField *mask, GwyDataField *exterior_mask,
+fix_mask_exterior(GwyField *mask, GwyField *exterior_mask,
                   gdouble fill_value)
 {
-    const gdouble *exm = gwy_data_field_get_data_const(exterior_mask);
-    gdouble *m = gwy_data_field_get_data(mask);
+    const gdouble *exm = gwy_field_get_data_const(exterior_mask);
+    gdouble *m = gwy_field_get_data(mask);
     gint n, k;
 
-    n = gwy_data_field_get_xres(mask) * gwy_data_field_get_yres(mask);
+    n = gwy_field_get_xres(mask) * gwy_field_get_yres(mask);
     for (k = 0; k < n; k++) {
         if (exm[k])
             m[k] = fill_value;
@@ -409,15 +385,15 @@ fix_mask_exterior(GwyDataField *mask, GwyDataField *exterior_mask,
 }
 
 static GwyPlaneSymmetry
-guess_unrotate_symmetry(GwyDataField *field, gdouble *corrections)
+guess_unrotate_symmetry(GwyField *field, gdouble *corrections)
 {
     enum { nder = 4800 };
-    GwyDataLine *derdist;
+    GwyLine *derdist;
     GwyPlaneSymmetry symm;
 
-    derdist = gwy_data_line_new(nder, 2.0*G_PI, FALSE);
-    gwy_data_field_slope_distribution(field, derdist, 5);
-    symm = gwy_data_field_unrotate_find_corrections(derdist, corrections);
+    derdist = gwy_line_new(nder, 2.0*G_PI, FALSE);
+    gwy_field_slope_distribution(field, derdist, 5);
+    symm = gwy_field_unrotate_find_corrections(derdist, corrections);
     g_object_unref(derdist);
 
     return symm;
