@@ -1,5 +1,5 @@
 /*
- *  $Id: scars.c 23666 2021-05-10 21:56:10Z yeti-dn $
+ *  $Id: scars.c 28910 2025-11-24 18:11:09Z yeti-dn $
  *  Copyright (C) 2003-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,32 +16,18 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/stats.h>
-#include <libprocess/grains.h>
-#include <libprocess/correct.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "preview.h"
 
-#define SCARS_MARK_RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
-#define SCARS_REMOVE_RUN_MODES GWY_RUN_IMMEDIATE
+#define MARK_RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
+#define REMOVE_RUN_MODES GWY_RUN_IMMEDIATE
 
 enum {
     MAX_LENGTH = 1024
 };
 
-/* The same as in mark_disconn. */
-typedef enum {
-    FEATURES_POSITIVE = 1 << 0,
-    FEATURES_NEGATIVE = 1 << 2,
-    FEATURES_BOTH     = (FEATURES_POSITIVE | FEATURES_NEGATIVE),
-} FeatureType;
-
 enum {
     PARAM_TYPE,
     PARAM_THRESHOLD_HIGH,
@@ -56,27 +42,26 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *mask;
-    GwyDataField *result;
+    GwyField *field;
+    GwyField *mask;
+    GwyField *result;
 } ModuleArgs;
 
 typedef struct {
     ModuleArgs *args;
     GtkWidget *dialog;
     GwyParamTable *table;
-    GwyContainer *data;
 } ModuleGUI;
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             scars_remove        (GwyContainer *data,
-                                             GwyRunType run);
-static void             scars_mark          (GwyContainer *data,
-                                             GwyRunType run);
+static void             scars_remove        (GwyFile *data,
+                                             GwyRunModeFlags mode);
+static void             scars_mark          (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static void             execute             (ModuleArgs *args);
 static GwyDialogOutcome run_gui             (ModuleArgs *args,
-                                             GwyContainer *data,
+                                             GwyFile *data,
                                              gint id);
 static void             param_changed       (ModuleGUI *gui,
                                              gint id);
@@ -99,18 +84,18 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("scars_mark",
-                              (GwyProcessFunc)&scars_mark,
+                              &scars_mark,
                               N_("/_Correct Data/M_ark Scars..."),
-                              GWY_STOCK_MARK_SCARS,
-                              SCARS_MARK_RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_ICON_MARK_SCARS,
+                              MARK_RUN_MODES,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Mark horizontal scars (strokes)"));
     gwy_process_func_register("scars_remove",
-                              (GwyProcessFunc)&scars_remove,
+                              &scars_remove,
                               N_("/_Correct Data/Remove _Scars"),
-                              GWY_STOCK_SCARS,
-                              SCARS_REMOVE_RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_ICON_SCARS,
+                              REMOVE_RUN_MODES,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Correct horizontal scars (strokes)"));
 
     return TRUE;
@@ -120,9 +105,9 @@ static GwyParamDef*
 define_module_params(void)
 {
     static const GwyEnum feature_types[] = {
-        { N_("Positive"), FEATURES_POSITIVE, },
-        { N_("Negative"), FEATURES_NEGATIVE, },
-        { N_("Both"),     FEATURES_BOTH,     },
+        { N_("Positive"), GWY_SIGN_POSITIVE, },
+        { N_("Negative"), GWY_SIGN_NEGATIVE, },
+        { N_("Both"),     GWY_SIGN_BOTH,     },
     };
     static GwyParamDef *paramdef = NULL;
 
@@ -132,7 +117,7 @@ define_module_params(void)
     paramdef = gwy_param_def_new();
     gwy_param_def_set_function_name(paramdef, "scars");
     gwy_param_def_add_gwyenum(paramdef, PARAM_TYPE, "type", _("Scars type"),
-                              feature_types, G_N_ELEMENTS(feature_types), FEATURES_BOTH);
+                              feature_types, G_N_ELEMENTS(feature_types), GWY_SIGN_BOTH);
     gwy_param_def_add_double(paramdef, PARAM_THRESHOLD_HIGH, "threshold_high", _("_Hard threshold"), 0.0, 2.0, 0.666);
     gwy_param_def_add_double(paramdef, PARAM_THRESHOLD_LOW, "threshold_low", _("_Soft threshold"), 0.0, 2.0, 0.25);
     gwy_param_def_add_int(paramdef, PARAM_MIN_LENGTH, "min_len", _("Minimum _length"), 1, MAX_LENGTH, 16);
@@ -145,82 +130,82 @@ define_module_params(void)
 }
 
 static void
-mark_scars(GwyDataField *field, GwyDataField *mask,
+mark_scars(GwyField *field, GwyField *mask,
            GwyParams *params)
 {
-    FeatureType type = gwy_params_get_enum(params, PARAM_TYPE);
+    GwySignFlags type = gwy_params_get_enum(params, PARAM_TYPE);
     gdouble threshold_high = gwy_params_get_double(params, PARAM_THRESHOLD_HIGH);
     gdouble threshold_low = gwy_params_get_double(params, PARAM_THRESHOLD_LOW);
     gint min_len = gwy_params_get_int(params, PARAM_MIN_LENGTH);
     gint max_width = gwy_params_get_int(params, PARAM_MAX_WIDTH);
-    GwyDataField *tmp;
+    GwyField *tmp;
 
-    if (type == FEATURES_POSITIVE || type == FEATURES_NEGATIVE) {
-        gwy_data_field_mark_scars(field, mask, threshold_high, threshold_low,
-                                  min_len, max_width, type == FEATURES_NEGATIVE);
+    if (type == GWY_SIGN_POSITIVE || type == GWY_SIGN_NEGATIVE) {
+        gwy_field_mark_scars(field, mask, threshold_high, threshold_low,
+                             min_len, max_width, type == GWY_SIGN_NEGATIVE);
         return;
     }
 
-    gwy_data_field_mark_scars(field, mask, threshold_high, threshold_low, min_len, max_width, FALSE);
-    tmp = gwy_data_field_new_alike(field, FALSE);
-    gwy_data_field_mark_scars(field, tmp, threshold_high, threshold_low, min_len, max_width, TRUE);
-    gwy_data_field_max_of_fields(mask, mask, tmp);
+    gwy_field_mark_scars(field, mask, threshold_high, threshold_low, min_len, max_width, FALSE);
+    tmp = gwy_field_new_alike(field, FALSE);
+    gwy_field_mark_scars(field, tmp, threshold_high, threshold_low, min_len, max_width, TRUE);
+    gwy_field_max_of_fields(mask, mask, tmp);
     g_object_unref(tmp);
 }
 
 static void
-scars_remove(GwyContainer *data, GwyRunType run)
+scars_remove(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyParams *params;
-    GwyDataField *field, *mask;
+    GwyField *field, *mask;
     GQuark dquark;
     gint id;
 
-    g_return_if_fail(run & SCARS_REMOVE_RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD_KEY, &dquark,
-                                     GWY_APP_DATA_FIELD, &field,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & REMOVE_RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD_KEY, &dquark,
+                                 GWY_APP_FIELD, &field,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(field && dquark);
 
     params = gwy_params_new_from_settings(define_module_params());
     sanitize_params(params);
-    gwy_app_undo_qcheckpointv(data, 1, &dquark);
+    gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &dquark);
 
-    mask = gwy_data_field_new_alike(field, TRUE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(mask), NULL);
+    mask = gwy_field_new_alike(field, TRUE);
+    gwy_unit_clear(gwy_field_get_unit_z(mask));
     mark_scars(field, mask, params);
-    gwy_data_field_laplace_solve(field, mask, -1, 1.0);
+    gwy_field_laplace_solve(field, mask, -1, 1.0);
     g_object_unref(mask);
     g_object_unref(params);
 
-    gwy_data_field_data_changed(field);
-    gwy_app_channel_log_add(data, id, id, "proc::scars_remove",
-                            "settings-name", "scars",
-                            NULL);
+    gwy_field_data_changed(field);
+    gwy_log_add_full(data, GWY_FILE_IMAGE, id, id, "proc::scars_remove",
+                     "settings-name", "scars",
+                     NULL);
 }
 
 static void
-scars_mark(GwyContainer *data, GwyRunType run)
+scars_mark(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     GQuark mquark;
     gint id;
 
-    g_return_if_fail(run & SCARS_MARK_RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_MASK_FIELD_KEY, &mquark,
-                                     GWY_APP_MASK_FIELD, &args.mask,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & MARK_RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_MASK_FIELD_KEY, &mquark,
+                                 GWY_APP_MASK_FIELD, &args.mask,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(args.field && mquark);
 
-    args.result = gwy_data_field_new_alike(args.field, TRUE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(args.result), NULL);
+    args.result = gwy_field_new_alike(args.field, TRUE);
+    gwy_unit_clear(gwy_field_get_unit_z(args.result));
     args.params = gwy_params_new_from_settings(define_module_params());
     sanitize_params(args.params);
-    if (run == GWY_RUN_INTERACTIVE) {
+    if (mode == GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, id);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -229,14 +214,14 @@ scars_mark(GwyContainer *data, GwyRunType run)
     if (outcome != GWY_DIALOG_HAVE_RESULT)
         execute(&args);
 
-    gwy_app_undo_qcheckpointv(data, 1, &mquark);
-    if (gwy_data_field_get_max(args.result) > 0.0)
-        gwy_container_set_object(data, mquark, args.result);
+    gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &mquark);
+    if (gwy_field_get_max(args.result) > 0.0)
+        gwy_file_set_image_mask(data, id, args.result);
     else
-        gwy_container_remove(data, mquark);
-    gwy_app_channel_log_add(data, id, id, "proc::scars_remove",
-                            "settings-name", "scars",
-                            NULL);
+        gwy_container_remove(GWY_CONTAINER(data), mquark);
+    gwy_log_add_full(data, GWY_FILE_IMAGE, id, id, "proc::scars_remove",
+                     "settings-name", "scars",
+                     NULL);
 
 end:
     g_object_unref(args.result);
@@ -252,38 +237,31 @@ execute(ModuleArgs *args)
     mark_scars(args->field, args->result, args->params);
     if (args->mask && combine) {
         if (combine_type == GWY_MERGE_UNION)
-            gwy_data_field_grains_add(args->result, args->mask);
+            gwy_field_grains_add(args->result, args->mask);
         else if (combine_type == GWY_MERGE_INTERSECTION)
-            gwy_data_field_grains_intersect(args->result, args->mask);
+            gwy_field_grains_intersect(args->result, args->mask);
     }
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
-    GwyDialogOutcome outcome;
     GtkWidget *hbox, *dataview;
     GwyDialog *dialog;
     GwyParamTable *table;
     ModuleGUI gui;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
-    gui.data = gwy_container_new();
-
-    gwy_container_set_object_by_name(gui.data, "/0/data", args->field);
-    gwy_container_set_object_by_name(gui.data, "/0/mask", args->result);
-    gwy_app_sync_data_items(data, gui.data, id, 0, FALSE,
-                            GWY_DATA_ITEM_PALETTE,
-                            GWY_DATA_ITEM_RANGE,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
 
     gui.dialog = gwy_dialog_new(_("Mark Scars"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_UPDATE, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, TRUE);
+    dataview = gwy_create_preview(args->field, args->result, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_MASK_COLOR
+                        | GWY_FILE_ITEM_RANGE | GWY_FILE_ITEM_REAL_SQUARE);
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), FALSE);
 
     table = gui.table = gwy_param_table_new(args->params);
@@ -303,7 +281,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_param_table_append_radio(table, PARAM_TYPE);
 
     gwy_param_table_append_header(table, -1, _("Options"));
-    gwy_param_table_append_mask_color(table, PARAM_MASK_COLOR, gui.data, 0, data, id);
+    gwy_param_table_append_image_mask_color(table, PARAM_MASK_COLOR, GWY_DATA_VIEW(dataview), data, id);
     if (args->mask) {
         gwy_param_table_append_radio_buttons(table, PARAM_COMBINE_TYPE, NULL);
         gwy_param_table_add_enabler(table, PARAM_COMBINE, PARAM_COMBINE_TYPE);
@@ -316,11 +294,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     g_signal_connect_swapped(table, "param-changed", G_CALLBACK(param_changed), &gui);
     gwy_dialog_set_preview_func(dialog, GWY_PREVIEW_IMMEDIATE, preview, &gui, NULL);
 
-    outcome = gwy_dialog_run(dialog);
-
-    g_object_unref(gui.data);
-
-    return outcome;
+    return gwy_dialog_run(dialog);
 }
 
 static void
@@ -350,7 +324,7 @@ preview(gpointer user_data)
     ModuleArgs *args = gui->args;
 
     execute(args);
-    gwy_data_field_data_changed(args->result);
+    gwy_field_data_changed(args->result);
     gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
 }
 
