@@ -1,5 +1,5 @@
 /*
- *  $Id: semsim.c 23666 2021-05-10 21:56:10Z yeti-dn $
+ *  $Id: semsim.c 28910 2025-11-24 18:11:09Z yeti-dn $
  *  Copyright (C) 2014-2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,17 +16,11 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwythreads.h>
-#include <libprocess/stats.h>
-#include <libprocess/filters.h>
-#include <libprocess/arithmetic.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwyapp.h>
-#include "libgwyddion/gwyomp.h"
+#include <gwy.h>
+#include "libgwyddion/omp.h"
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
@@ -60,13 +54,13 @@ typedef struct {
     gint exth;
     gint extxres;
     gint extyres;
-    GwyDataField *extfield;
+    GwyField *extfield;
 } SEMSimCommon;
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *result;
+    GwyField *field;
+    GwyField *result;
 } ModuleArgs;
 
 typedef struct {
@@ -77,8 +71,8 @@ typedef struct {
 
 static gboolean         module_register      (void);
 static GwyParamDef*     define_module_params (void);
-static void             semsim               (GwyContainer *data,
-                                              GwyRunType runtype);
+static void             module_main          (GwyFile *data,
+                                              GwyRunModeFlags mode);
 static gboolean         execute              (ModuleArgs *args,
                                               GtkWindow *wait_window);
 static GwyDialogOutcome run_gui              (ModuleArgs *args);
@@ -88,7 +82,7 @@ static gboolean         semsim_do_integration(SEMSimCommon *common,
                                               ModuleArgs *args);
 static gboolean         semsim_do_montecarlo (SEMSimCommon *common,
                                               ModuleArgs *args);
-static gdouble*         create_erf_table     (GwyDataField *field,
+static gdouble*         create_erf_table     (GwyField *field,
                                               gdouble sigma,
                                               gdouble *zstep);
 
@@ -108,11 +102,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("semsim",
-                              (GwyProcessFunc)&semsim,
+                              module_main,
                               N_("/_Presentation/_SEM Image..."),
                               NULL,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Simple SEM simulation from topography"));
 
     return TRUE;
@@ -140,38 +134,38 @@ define_module_params(void)
 }
 
 static void
-semsim(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     ModuleArgs args;
     GQuark squark;
     gint id;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     GWY_APP_SHOW_FIELD_KEY, &squark,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_FIELD_ID, &id,
+                                 GWY_APP_SHOW_FIELD_KEY, &squark,
+                                 0);
     g_return_if_fail(args.field && squark);
 
     if (!gwy_require_image_same_units(args.field, data, id, _("SEM Image")))
         return;
 
-    args.result = gwy_data_field_new_alike(args.field, TRUE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(args.result), NULL);
+    args.result = gwy_field_new_alike(args.field, TRUE);
+    gwy_unit_clear(gwy_field_get_unit_z(args.result));
     args.params = gwy_params_new_from_settings(define_module_params());
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
             goto end;
     }
 
-    if (execute(&args, gwy_app_find_window_for_channel(data, id))) {
-        gwy_app_undo_qcheckpointv(data, 1, &squark);
-        gwy_container_set_object(data, squark, args.result);
-        gwy_app_channel_log_add_proc(data, id, id);
+    if (execute(&args, gwy_data_browser_get_window_for_data(data, GWY_FILE_IMAGE, id))) {
+        gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &squark);
+        gwy_container_set_object(GWY_CONTAINER(data), gwy_file_key_image_picture(id), args.result);
+        gwy_log_add(data, GWY_FILE_IMAGE, id, id);
     }
 
 end:
@@ -225,25 +219,25 @@ static gboolean
 execute(ModuleArgs *args, GtkWindow *wait_window)
 {
     GwyParams *params = args->params;
-    GwyDataField *field = args->field;
+    GwyField *field = args->field;
     SEMSimMethod method = gwy_params_get_enum(params, PARAM_METHOD);
     SEMSimCommon common;
-    gint xres = gwy_data_field_get_xres(field), yres = gwy_data_field_get_yres(field);
+    gint xres = gwy_field_get_xres(field), yres = gwy_field_get_yres(field);
     gdouble sigma;
     gboolean ok;
 
     gwy_app_wait_start(wait_window, _("SEM image simulation..."));
 
-    common.dx = gwy_data_field_get_dx(field);
-    common.dy = gwy_data_field_get_dy(field);
+    common.dx = gwy_field_get_dx(field);
+    common.dy = gwy_field_get_dy(field);
     sigma = gwy_params_get_double(params, PARAM_SIGMA) * common.dx;
     common.exth = (gint)ceil(5.5*sigma/common.dx);
     common.extv = (gint)ceil(5.5*sigma/common.dy);
     common.extxres = xres + 2*common.exth;
     common.extyres = yres + 2*common.extv;
-    common.extfield = gwy_data_field_extend(args->field,
-                                            common.exth, common.exth, common.extv, common.extv,
-                                            GWY_EXTERIOR_BORDER_EXTEND, 0.0, FALSE);
+    common.extfield = gwy_field_extend(args->field,
+                                       common.exth, common.exth, common.extv, common.extv,
+                                       GWY_EXTERIOR_BORDER, 0.0, FALSE);
     common.erftable = create_erf_table(field, sigma, &common.dz);
 
     if (method == SEMSIM_METHOD_INTEGRATION)
@@ -254,7 +248,7 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
     gwy_app_wait_finish();
 
     if (ok)
-        gwy_data_field_normalize(args->result);
+        gwy_field_normalize(args->result);
 
     g_free(common.erftable);
     g_object_unref(common.extfield);
@@ -265,13 +259,13 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
 static gboolean
 semsim_do_integration(SEMSimCommon *common, ModuleArgs *args)
 {
-    GwyDataField *show = args->result;
-    gint xres = gwy_data_field_get_xres(show), yres = gwy_data_field_get_yres(show);
+    GwyField *show = args->result;
+    gint xres = gwy_field_get_xres(show), yres = gwy_field_get_yres(show);
     gint i, j, k;
     gdouble dx = common->dx, dy = common->dy, dz = common->dz;
     gdouble sigma_r2 = G_SQRT2*gwy_params_get_double(args->params, PARAM_SIGMA) * dx;
-    const gdouble *d = gwy_data_field_get_data_const(common->extfield);
-    gdouble *s = gwy_data_field_get_data(show);
+    const gdouble *d = gwy_field_get_data_const(common->extfield);
+    gdouble *s = gwy_field_get_data(show);
     gdouble *erftable = common->erftable;
     WeightItem *weight_table;
     gint exth = common->exth, extv = common->extv, extxres = common->extxres;
@@ -326,17 +320,17 @@ semsim_do_integration(SEMSimCommon *common, ModuleArgs *args)
 static gboolean
 semsim_do_montecarlo(SEMSimCommon *common, ModuleArgs *args)
 {
-    GwyDataField *show = args->result;
-    gint xres = gwy_data_field_get_xres(show), yres = gwy_data_field_get_yres(show);
+    GwyField *show = args->result;
+    gint xres = gwy_field_get_xres(show), yres = gwy_field_get_yres(show);
     gdouble dx = common->dx, dy = common->dy, dz = common->dz;
     gdouble quality = gwy_params_get_double(args->params, PARAM_QUALITY);
     gdouble sigma_r2 = G_SQRT2*gwy_params_get_double(args->params, PARAM_SIGMA) * dx;
-    const gdouble *d = gwy_data_field_get_data_const(common->extfield);
-    gdouble *s = gwy_data_field_get_data(show);
+    const gdouble *d = gwy_field_get_data_const(common->extfield);
+    gdouble *s = gwy_field_get_data(show);
     gdouble *erftable = common->erftable;
     gint exth = common->exth, extv = common->extv;
     gint extxres = common->extxres, extyres = common->extyres;
-    gdouble noise_limit = pow10(-quality);
+    gdouble noise_limit = gwy_exp10(-quality);
     gint miniter = (gint)ceil(10*quality);
     gboolean cancelled = FALSE, *pcancelled = &cancelled;
 
@@ -402,13 +396,13 @@ semsim_do_montecarlo(SEMSimCommon *common, ModuleArgs *args)
 }
 
 static gdouble*
-create_erf_table(GwyDataField *field, gdouble sigma, gdouble *zstep)
+create_erf_table(GwyField *field, gdouble sigma, gdouble *zstep)
 {
     gdouble min, max, dz;
     gdouble *table;
     gint i;
 
-    gwy_data_field_get_min_max(field, &min, &max);
+    gwy_field_get_min_max(field, &min, &max);
     dz = (max - min)/(ERF_TABLE_SIZE - 1);
     table = g_new(gdouble, ERF_TABLE_SIZE + 1);
     for (i = 0; i <= ERF_TABLE_SIZE; i++)
