@@ -1,5 +1,5 @@
 /*
- *  $Id: shade.c 25111 2022-10-19 15:02:39Z yeti-dn $
+ *  $Id: shade.c 28910 2025-11-24 18:11:09Z yeti-dn $
  *  Copyright (C) 2003-2022 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,17 +16,11 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/filters.h>
-#include <libprocess/stats.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwydgets/gwyshader.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "preview.h"
+#include "libgwyapp/sanity.h"
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
 
@@ -39,8 +33,8 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *result;
+    GwyField *field;
+    GwyField *result;
 } ModuleArgs;
 
 typedef struct {
@@ -48,24 +42,23 @@ typedef struct {
     GtkWidget *dialog;
     GwyParamTable *table;
     GtkWidget *shader;
-    GwyContainer *data;
 } ModuleGUI;
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
 static void             execute             (ModuleArgs *args);
 static GwyDialogOutcome run_gui             (ModuleArgs *args,
-                                             GwyContainer *data,
+                                             GwyFile *data,
                                              gint id);
 static void             param_changed       (ModuleGUI *gui,
                                              gint id);
 static void             preview             (gpointer user_data);
-static void             shade               (GwyContainer *data,
-                                             GwyRunType runtype);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static void             shade_changed       (ModuleGUI *gui,
                                              GwyShader *shader);
-static void             mix_with_plane      (GwyDataField *shaded,
-                                             GwyDataField *plane,
+static void             mix_with_plane      (GwyField *shaded,
+                                             GwyField *plane,
                                              gdouble mixfraction);
 
 static GwyModuleInfo module_info = {
@@ -84,11 +77,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("shade",
-                              (GwyProcessFunc)&shade,
+                              module_main,
                               N_("/_Presentation/_Shading..."),
-                              GWY_STOCK_SHADER,
+                              GWY_ICON_SHADER,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Shade data"));
 
     return TRUE;
@@ -112,25 +105,25 @@ define_module_params(void)
 }
 
 static void
-shade(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     ModuleArgs args;
     GQuark squark;
     gint id;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     GWY_APP_SHOW_FIELD_KEY, &squark,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_FIELD_ID, &id,
+                                 GWY_APP_SHOW_FIELD_KEY, &squark,
+                                 0);
     g_return_if_fail(args.field && squark);
 
     args.params = gwy_params_new_from_settings(define_module_params());
-    args.result = gwy_data_field_new_alike(args.field, TRUE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(args.result), NULL);
+    args.result = gwy_field_new_alike(args.field, TRUE);
+    gwy_unit_clear(gwy_field_get_unit_z(args.result));
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, id);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -139,10 +132,10 @@ shade(GwyContainer *data, GwyRunType runtype)
     if (outcome != GWY_DIALOG_HAVE_RESULT)
         execute(&args);
 
-    gwy_app_undo_qcheckpointv(data, 1, &squark);
+    gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &squark);
 
-    gwy_container_set_object(data, squark, args.result);
-    gwy_app_channel_log_add_proc(data, id, id);
+    gwy_container_set_object(GWY_CONTAINER(data), gwy_file_key_image_picture(id), args.result);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, id);
 
 end:
     g_object_unref(args.result);
@@ -150,45 +143,37 @@ end:
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
-    GwyDialogOutcome outcome;
-    GtkWidget *hbox, *vbox, *align, *dataview;
+    GtkWidget *hbox, *vbox, *dataview;
     GwyParamTable *table;
-    const guchar *pal = NULL;
+    const gchar *gradient = NULL;
     ModuleGUI gui;
     GwyDialog *dialog;
 
     gui.args = args;
-    gui.data = gwy_container_new();
-
-    gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(0), args->result);
-    gwy_container_gis_string(data, gwy_app_get_data_palette_key_for_id(id), &pal);
-    gwy_app_sync_data_items(data, gui.data, id, 0, FALSE,
-                            GWY_DATA_ITEM_PALETTE,
-                            GWY_DATA_ITEM_RANGE,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
 
     gui.dialog = gwy_dialog_new(_("Shading"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, TRUE);
+    dataview = gwy_create_preview(args->result, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_RANGE | GWY_FILE_ITEM_REAL_SQUARE);
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), FALSE);
 
-    vbox = gwy_vbox_new(0);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
     gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);
 
-    align = gtk_alignment_new(0.5, 0.5, 1.0, 1.0);
-    gtk_alignment_set_padding(GTK_ALIGNMENT(align), 4, 4, 4, 4);
-    gtk_box_pack_start(GTK_BOX(vbox), align, FALSE, TRUE, 0);
-    gui.shader = gwy_shader_new(pal);
+    gui.shader = gwy_shader_new();
+    gwy_set_widget_padding(gui.shader, 4, 4, 4, 4);
+    gradient = gwy_file_get_palette(data, GWY_FILE_IMAGE, id);
+    gwy_shader_set_gradient(GWY_SHADER(gui.shader), gwy_gradients_get_gradient(gradient));
     gwy_shader_set_angle(GWY_SHADER(gui.shader),
                          gwy_params_get_double(args->params, PARAM_THETA),
                          gwy_params_get_double(args->params, PARAM_PHI));
-    gtk_widget_set_size_request(gui.shader, 120, 120);
-    gtk_container_add(GTK_CONTAINER(align), gui.shader);
+    gtk_widget_set_size_request(gui.shader, -1, 180);
+    gtk_box_pack_start(GTK_BOX(vbox), gui.shader, FALSE, TRUE, 0);
 
     table = gui.table = gwy_param_table_new(args->params);
     gwy_param_table_append_slider(table, PARAM_THETA);
@@ -203,11 +188,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     g_signal_connect_swapped(table, "param-changed", G_CALLBACK(param_changed), &gui);
     gwy_dialog_set_preview_func(dialog, GWY_PREVIEW_IMMEDIATE, preview, &gui, NULL);
 
-    outcome = gwy_dialog_run(dialog);
-
-    g_object_unref(gui.data);
-
-    return outcome;
+    return gwy_dialog_run(dialog);
 }
 
 static void
@@ -241,36 +222,36 @@ preview(gpointer user_data)
     ModuleGUI *gui = (ModuleGUI*)user_data;
 
     execute(gui->args);
-    gwy_data_field_data_changed(gui->args->result);
+    gwy_field_data_changed(gui->args->result);
 }
 
 static void
 execute(ModuleArgs *args)
 {
     GwyParams *params = args->params;
-    GwyDataField *field = args->field, *result = args->result;
+    GwyField *field = args->field, *result = args->result;
 
-    gwy_data_field_shade(field, result,
-                         gwy_params_get_double(params, PARAM_THETA), gwy_params_get_double(params, PARAM_PHI));
+    gwy_field_shade(field, result,
+                    gwy_params_get_double(params, PARAM_THETA), gwy_params_get_double(params, PARAM_PHI));
     if (gwy_params_get_boolean(params, PARAM_DO_MIX))
         mix_with_plane(result, field, gwy_params_get_double(params, PARAM_MIX));
-    gwy_data_field_normalize(result);
+    gwy_field_normalize(result);
 }
 
 static void
-mix_with_plane(GwyDataField *shaded,
-               GwyDataField *plane,
+mix_with_plane(GwyField *shaded,
+               GwyField *plane,
                gdouble mixfraction)
 {
     gdouble plane_min, plane_max, shade_min, shade_max, plane_range, shade_range, q;
 
-    gwy_data_field_get_min_max(plane, &plane_min, &plane_max);
-    gwy_data_field_get_min_max(shaded, &shade_min, &shade_max);
+    gwy_field_get_min_max(plane, &plane_min, &plane_max);
+    gwy_field_get_min_max(shaded, &shade_min, &shade_max);
     plane_range = plane_max - plane_min;
     shade_range = shade_max - shade_min;
 
     q = (plane_range > 0.0 ? mixfraction*shade_range/plane_range : 1.0);
-    gwy_data_field_linear_combination(shaded, 1.0 - mixfraction, shaded, q, plane, 0.0);
+    gwy_field_linear_combination(shaded, 1.0 - mixfraction, shaded, q, plane, 0.0);
 }
 
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
