@@ -1,5 +1,5 @@
 /*
- *  $Id: slope_dist.c 24550 2021-11-26 17:01:11Z yeti-dn $
+ *  $Id: slope_dist.c 28821 2025-11-06 16:00:41Z yeti-dn $
  *  Copyright (C) 2004-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,19 +16,9 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/level.h>
-#include <libprocess/stats.h>
-#include <libprocess/filters.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libgwydgets/gwyradiobuttons.h>
-#include <libgwydgets/gwycombobox.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
@@ -54,49 +44,48 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *mask;
-    GwyDataField *result;
+    GwyField *field;
+    GwyField *mask;
+    GwyField *result;
     GwyGraphModel *gmodel;
     /* Cached input data properties. */
     gboolean same_units;
-    GwyDataField *xder;
-    GwyDataField *yder;
+    GwyField *xder;
+    GwyField *yder;
 } ModuleArgs;
 
 typedef struct {
     ModuleArgs *args;
     GtkWidget *dialog;
     GwyParamTable *table;
-    GwyContainer *data;
     GtkWidget *dataview;
     GtkWidget *graph;
 } ModuleGUI;
 
 static gboolean         module_register           (void);
 static GwyParamDef*     define_module_params      (void);
-static void             slope_dist                (GwyContainer *data,
-                                                   GwyRunType runtype);
+static void             module_main               (GwyFile *data,
+                                                   GwyRunModeFlags mode);
 static void             execute                   (ModuleArgs *args);
 static void             sanitise_params           (ModuleArgs *args);
 static GwyDialogOutcome run_gui                   (ModuleArgs *args,
-                                                   GwyContainer *data,
+                                                   GwyFile *data,
                                                    gint id);
 static void             param_changed             (ModuleGUI *gui,
                                                    gint id);
 static void             preview                   (gpointer user_data);
-static GwyDataField*    slope_do_2d               (ModuleArgs *args);
+static GwyField*        slope_do_2d               (ModuleArgs *args);
 static void             slope_do_graph_phi        (ModuleArgs *args);
 static void             slope_do_graph_theta      (ModuleArgs *args);
 static void             slope_do_graph_gradient   (ModuleArgs *args);
 static void             set_graph_model_properties(GwyGraphModel *gmodel,
-                                                   GwyDataField *field,
+                                                   GwyField *field,
                                                    SlopeOutput output_type);
-static void             compute_slopes            (GwyDataField *field,
+static void             compute_slopes            (GwyField *field,
                                                    gint kernel_size,
-                                                   GwyDataField *xder,
-                                                   GwyDataField *yder);
-static GwyDataField*    make_datafield            (GwyDataField *old,
+                                                   GwyField *xder,
+                                                   GwyField *yder);
+static GwyField*        make_field                (GwyField *old,
                                                    gint res,
                                                    gulong *count,
                                                    gdouble real,
@@ -118,11 +107,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("slope_dist",
-                              (GwyProcessFunc)&slope_dist,
+                              module_main,
                               N_("/_Statistics/_Slope Distribution..."),
-                              GWY_STOCK_DISTRIBUTION_SLOPE,
+                              GWY_ICON_DISTRIBUTION_SLOPE,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Calculate angular slope distribution"));
 
     return TRUE;
@@ -157,28 +146,27 @@ define_module_params(void)
 }
 
 static void
-slope_dist(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
     gint oldid, newid;
     GwyAppDataId target_graph_id;
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_DATA_FIELD_ID, &oldid,
-                                     GWY_APP_MASK_FIELD, &args.mask,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_FIELD_ID, &oldid,
+                                 GWY_APP_MASK_FIELD, &args.mask,
+                                 0);
     g_return_if_fail(args.field);
-    args.same_units = gwy_si_unit_equal(gwy_data_field_get_si_unit_xy(args.field),
-                                        gwy_data_field_get_si_unit_z(args.field));
+    args.same_units = gwy_unit_equal(gwy_field_get_unit_xy(args.field), gwy_field_get_unit_z(args.field));
     args.params = gwy_params_new_from_settings(define_module_params());
     sanitise_params(&args);
-    args.result = gwy_data_field_new(PREVIEW_SIZE, PREVIEW_SIZE, 1.0, 1.0, TRUE);
+    args.result = gwy_field_new(PREVIEW_SIZE, PREVIEW_SIZE, 1.0, 1.0, TRUE);
     args.gmodel = gwy_graph_model_new();
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, oldid);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -188,12 +176,13 @@ slope_dist(GwyContainer *data, GwyRunType runtype)
         execute(&args);
 
     if (gwy_params_get_enum(args.params, PARAM_OUTPUT_TYPE) == SLOPE_DIST_2D_DIST) {
-        newid = gwy_app_data_browser_add_data_field(args.result, data, TRUE);
-        gwy_app_sync_data_items(data, data, oldid, newid, FALSE,
-                                GWY_DATA_ITEM_PALETTE,
-                                0);
-        gwy_app_set_data_field_title(data, newid, _("Slope distribution"));
-        gwy_app_channel_log_add_proc(data, oldid, newid);
+        newid = gwy_file_add_image(data, args.result);
+        gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+        gwy_file_sync_items(data, GWY_FILE_IMAGE, oldid,
+                            data, GWY_FILE_IMAGE, newid,
+                            GWY_FILE_ITEM_PALETTE, FALSE);
+        gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _("Slope distribution"), TRUE);
+        gwy_log_add(data, GWY_FILE_IMAGE, oldid, newid);
     }
     else {
         target_graph_id = gwy_params_get_data_id(args.params, PARAM_TARGET_GRAPH);
@@ -201,45 +190,40 @@ slope_dist(GwyContainer *data, GwyRunType runtype)
     }
 
 end:
-    GWY_OBJECT_UNREF(args.xder);
-    GWY_OBJECT_UNREF(args.yder);
+    g_clear_object(&args.xder);
+    g_clear_object(&args.yder);
     g_object_unref(args.result);
     g_object_unref(args.gmodel);
     g_object_unref(args.params);
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     GtkWidget *hbox, *vbox;
     GwyDialog *dialog;
     GwyParamTable *table;
     ModuleGUI gui;
     SlopeOutput output_type;
-    GwyDialogOutcome outcome;
 
     output_type = gwy_params_get_enum(args->params, PARAM_OUTPUT_TYPE);
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
     gui.dialog = gwy_dialog_new(_("Slope Distribution"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_UPDATE, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    hbox = gwy_hbox_new(8);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 8);
     gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
     gwy_dialog_add_content(dialog, hbox, TRUE, TRUE, 0);
 
-    gui.data = gwy_container_new();
-    gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(0), args->result);
-    gwy_app_sync_data_items(data, gui.data, id, 0, FALSE,
-                            GWY_DATA_ITEM_PALETTE,
-                            0);
-    vbox = gwy_vbox_new(0);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
     gtk_widget_set_size_request(vbox, PREVIEW_SIZE, -1);
     gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);
 
-    gui.dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
+    gui.dataview = gwy_create_preview(args->result, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(gui.dataview), data, GWY_FILE_IMAGE, id, GWY_FILE_ITEM_PALETTE);
     gtk_box_pack_start(GTK_BOX(vbox), gui.dataview, FALSE, FALSE, 0);
     if (output_type != SLOPE_DIST_2D_DIST)
         gtk_widget_set_no_show_all(gui.dataview, TRUE);
@@ -270,11 +254,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     g_signal_connect_swapped(table, "param-changed", G_CALLBACK(param_changed), &gui);
     gwy_dialog_set_preview_func(dialog, GWY_PREVIEW_IMMEDIATE, preview, &gui, NULL);
 
-    outcome = gwy_dialog_run(dialog);
-
-    g_object_unref(gui.data);
-
-    return outcome;
+    return gwy_dialog_run(dialog);
 }
 
 static void
@@ -322,22 +302,22 @@ preview(gpointer user_data)
 
     execute(args);
     if (output_type == SLOPE_DIST_2D_DIST) {
-        gwy_data_field_data_changed(args->result);
+        gwy_field_data_changed(args->result);
         gwy_set_data_preview_size(GWY_DATA_VIEW(gui->dataview), PREVIEW_SIZE);
     }
     gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
 }
 
 static inline gboolean
-is_counted(GwyDataField *mask, guint k, GwyMaskingType masking)
+is_counted(const gdouble *m, guint k, GwyMaskingType masking)
 {
-    if (!mask || masking == GWY_MASK_IGNORE)
+    if (!m || masking == GWY_MASK_IGNORE)
         return TRUE;
 
     if (masking == GWY_MASK_INCLUDE)
-        return mask->data[k] > 0.0;
+        return m[k] > 0.0;
     else
-        return mask->data[k] <= 0.0;
+        return m[k] <= 0.0;
 }
 
 static void
@@ -348,18 +328,18 @@ execute(ModuleArgs *args)
     gint kernel_size = gwy_params_get_int(params, PARAM_KERNEL_SIZE);
     SlopeOutput output_type = gwy_params_get_enum(params, PARAM_OUTPUT_TYPE);
     GwyGraphCurveModel *gcmodel;
-    GwyDataField *result;
+    GwyField *result;
 
     if (!args->xder)
-        args->xder = gwy_data_field_new_alike(args->field, FALSE);
+        args->xder = gwy_field_new_alike(args->field, FALSE);
     if (!args->yder)
-        args->yder = gwy_data_field_new_alike(args->field, FALSE);
+        args->yder = gwy_field_new_alike(args->field, FALSE);
 
     compute_slopes(args->field, fit_plane ? kernel_size : 0, args->xder, args->yder);
 
     if (output_type == SLOPE_DIST_2D_DIST) {
         result = slope_do_2d(args);
-        gwy_data_field_assign(args->result, result);
+        gwy_field_assign(args->result, result);
         g_object_unref(result);
     }
     else {
@@ -379,11 +359,11 @@ execute(ModuleArgs *args)
     }
 }
 
-static GwyDataField*
+static GwyField*
 slope_do_2d(ModuleArgs *args)
 {
     GwyParams *params = args->params;
-    GwyDataField *field = args->field, *mask = args->mask;
+    GwyField *field = args->field, *mask = args->mask;
     GwyMaskingType masking = gwy_params_get_masking(params, PARAM_MASKING, &mask);
     gint kernel_size = gwy_params_get_int(params, PARAM_KERNEL_SIZE);
     gint size = gwy_params_get_int(params, PARAM_SIZE);
@@ -395,25 +375,26 @@ slope_do_2d(ModuleArgs *args)
     gint xider, yider, i;
     gulong *count;
 
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
 
     n = fit_plane ? kernel_size : 2;
     n = (xres - n)*(yres - n);
 
-    gwy_data_field_get_min_max(args->xder, &minxd, &maxxd);
+    gwy_field_get_min_max(args->xder, &minxd, &maxxd);
     maxxd = MAX(fabs(minxd), fabs(maxxd));
-    gwy_data_field_get_min_max(args->yder, &minyd, &maxyd);
+    gwy_field_get_min_max(args->yder, &minyd, &maxyd);
     maxyd = MAX(fabs(minyd), fabs(maxyd));
     max = MAX(maxxd, maxyd);
     if (!max)
         max = 1.0;
 
     count = g_new0(gulong, size*size);
-    xd = gwy_data_field_get_data_const(args->xder);
-    yd = gwy_data_field_get_data_const(args->yder);
+    xd = gwy_field_get_data_const(args->xder);
+    yd = gwy_field_get_data_const(args->yder);
+    const gdouble *m = mask ? gwy_field_get_data(mask) : NULL;
     for (i = 0; i < n; i++) {
-        if (is_counted(mask, i, masking)) {
+        if (is_counted(m, i, masking)) {
             xider = size*(xd[i]/(2.0*max) + 0.5);
             xider = CLAMP(xider, 0, size-1);
             yider = size*(yd[i]/(2.0*max) + 0.5);
@@ -423,35 +404,36 @@ slope_do_2d(ModuleArgs *args)
         }
     }
 
-    return make_datafield(field, size, count, 2.0*max, logscale);
+    return make_field(field, size, count, 2.0*max, logscale);
 }
 
 static void
 slope_do_graph_phi(ModuleArgs *args)
 {
     GwyParams *params = args->params;
-    GwyDataField *field = args->field, *mask = args->mask;
+    GwyField *field = args->field, *mask = args->mask;
     GwyMaskingType masking = gwy_params_get_masking(params, PARAM_MASKING, &mask);
     gint kernel_size = gwy_params_get_int(params, PARAM_KERNEL_SIZE);
     gint size = gwy_params_get_int(params, PARAM_SIZE);
     gboolean fit_plane = gwy_params_get_boolean(params, PARAM_FIT_PLANE);
-    GwyDataLine *dataline;
+    GwyLine *line;
     const gdouble *xd, *yd;
     gdouble *data;
     gint xres, yres, n, i;
 
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
 
     n = fit_plane ? kernel_size : 2;
     n = (xres - n)*(yres - n);
 
-    dataline = gwy_data_line_new(size, 360, TRUE);
-    data = gwy_data_line_get_data(dataline);
-    xd = gwy_data_field_get_data_const(args->xder);
-    yd = gwy_data_field_get_data_const(args->yder);
+    line = gwy_line_new(size, 360, TRUE);
+    data = gwy_line_get_data(line);
+    xd = gwy_field_get_data_const(args->xder);
+    yd = gwy_field_get_data_const(args->yder);
+    const gdouble *m = mask ? gwy_field_get_data(mask) : NULL;
     for (i = 0; i < n; i++) {
-        if (is_counted(mask, i, masking)) {
+        if (is_counted(m, i, masking)) {
             gdouble phi = gwy_canonicalize_angle(atan2(yd[i], -xd[i]), TRUE, TRUE);
             gdouble d = (xd[i]*xd[i] + yd[i]*yd[i]);
             gint iphi = floor(size*phi/(2.0*G_PI));
@@ -461,42 +443,43 @@ slope_do_graph_phi(ModuleArgs *args)
         }
     }
 
-    gwy_graph_curve_model_set_data_from_dataline(gwy_graph_model_get_curve(args->gmodel, 0), dataline, 0, 0);
-    g_object_unref(dataline);
+    gwy_graph_curve_model_set_data_from_line(gwy_graph_model_get_curve(args->gmodel, 0), line, 0, 0);
+    g_object_unref(line);
 }
 
 static void
 slope_do_graph_theta(ModuleArgs *args)
 {
     GwyParams *params = args->params;
-    GwyDataField *field = args->field, *mask = args->mask;
+    GwyField *field = args->field, *mask = args->mask;
     GwyMaskingType masking = gwy_params_get_masking(params, PARAM_MASKING, &mask);
     gint kernel_size = gwy_params_get_int(params, PARAM_KERNEL_SIZE);
     gint size = gwy_params_get_int(params, PARAM_SIZE);
     gboolean fit_plane = gwy_params_get_boolean(params, PARAM_FIT_PLANE);
-    GwyDataLine *dataline;
+    GwyLine *line;
     const gdouble *yd;
     gdouble *xd, *data;
     gint xres, yres, n, i, nc;
     gdouble max;
 
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
 
     n = fit_plane ? kernel_size : 2;
     n = (xres - n)*(yres - n);
 
-    dataline = gwy_data_line_new(size, 90, TRUE);
-    data = gwy_data_line_get_data(dataline);
-    xd = gwy_data_field_get_data(args->xder);
-    yd = gwy_data_field_get_data_const(args->yder);
+    line = gwy_line_new(size, 90, TRUE);
+    data = gwy_line_get_data(line);
+    xd = gwy_field_get_data(args->xder);
+    yd = gwy_field_get_data_const(args->yder);
     for (i = 0; i < n; i++)
-        xd[i] = 180.0/G_PI*atan(hypot(xd[i], yd[i]));
-    gwy_data_field_area_get_min_max_mask(args->xder, mask, masking, 0, 0, xres, yres, NULL, &max);
-    gwy_data_line_set_real(dataline, max);
+        xd[i] = gwy_rad2deg(atan(hypot(xd[i], yd[i])));
+    gwy_field_area_get_min_max(args->xder, mask, masking, 0, 0, xres, yres, NULL, &max);
+    gwy_line_set_real(line, max);
     nc = 0;
+    const gdouble *m = mask ? gwy_field_get_data(mask) : NULL;
     for (i = 0; i < n; i++) {
-        if (is_counted(mask, i, masking)) {
+        if (is_counted(m, i, masking)) {
             gint itheta = floor(size*xd[i]/max);
 
             itheta = CLAMP(itheta, 0, size-1);
@@ -506,44 +489,45 @@ slope_do_graph_theta(ModuleArgs *args)
     }
 
     if (nc && max)
-        gwy_data_line_multiply(dataline, size/(nc*max));
+        gwy_line_multiply(line, size/(nc*max));
 
-    gwy_graph_curve_model_set_data_from_dataline(gwy_graph_model_get_curve(args->gmodel, 0), dataline, 0, 0);
-    g_object_unref(dataline);
+    gwy_graph_curve_model_set_data_from_line(gwy_graph_model_get_curve(args->gmodel, 0), line, 0, 0);
+    g_object_unref(line);
 }
 
 static void
 slope_do_graph_gradient(ModuleArgs *args)
 {
     GwyParams *params = args->params;
-    GwyDataField *field = args->field, *mask = args->mask;
+    GwyField *field = args->field, *mask = args->mask;
     GwyMaskingType masking = gwy_params_get_masking(params, PARAM_MASKING, &mask);
     gint kernel_size = gwy_params_get_int(params, PARAM_KERNEL_SIZE);
     gint size = gwy_params_get_int(params, PARAM_SIZE);
     gboolean fit_plane = gwy_params_get_boolean(params, PARAM_FIT_PLANE);
-    GwyDataLine *dataline;
+    GwyLine *line;
     const gdouble *yd;
     gdouble *xd, *data;
     gint xres, yres, n, i, nc;
     gdouble max;
 
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
 
     n = fit_plane ? kernel_size : 2;
     n = (xres - n)*(yres - n);
 
-    xd = gwy_data_field_get_data(args->xder);
-    yd = gwy_data_field_get_data_const(args->yder);
+    xd = gwy_field_get_data(args->xder);
+    yd = gwy_field_get_data_const(args->yder);
     for (i = 0; i < n; i++)
         xd[i] = hypot(xd[i], yd[i]);
-    gwy_data_field_area_get_min_max_mask(args->xder, mask, masking, 0, 0, xres, yres, NULL, &max);
+    gwy_field_area_get_min_max(args->xder, mask, masking, 0, 0, xres, yres, NULL, &max);
 
-    dataline = gwy_data_line_new(size, max, TRUE);
-    data = gwy_data_line_get_data(dataline);
+    line = gwy_line_new(size, max, TRUE);
+    data = gwy_line_get_data(line);
     nc = 0;
+    const gdouble *m = mask ? gwy_field_get_data(mask) : NULL;
     for (i = 0; i < n; i++) {
-        if (is_counted(mask, i, masking)) {
+        if (is_counted(m, i, masking)) {
             gint ider = floor(size*xd[i]/max);
 
             ider = CLAMP(ider, 0, size-1);
@@ -553,41 +537,41 @@ slope_do_graph_gradient(ModuleArgs *args)
     }
 
     if (nc && max)
-        gwy_data_line_multiply(dataline, size/(nc*max));
+        gwy_line_multiply(line, size/(nc*max));
 
-    gwy_graph_curve_model_set_data_from_dataline(gwy_graph_model_get_curve(args->gmodel, 0), dataline, 0, 0);
-    g_object_unref(dataline);
+    gwy_graph_curve_model_set_data_from_line(gwy_graph_model_get_curve(args->gmodel, 0), line, 0, 0);
+    g_object_unref(line);
 }
 
 static void
 set_graph_model_properties(GwyGraphModel *gmodel,
-                           GwyDataField *field,
+                           GwyField *field,
                            SlopeOutput output_type)
 {
-    GwySIUnit *siunitx, *siunity;
+    GwyUnit *siunitx, *siunity;
     GwyGraphCurveModel *gcmodel;
     const gchar *xlabel, *ylabel, *title, *desc;
 
     if (output_type == SLOPE_DIST_GRAPH_PHI) {
-        siunitx = gwy_si_unit_new("deg");
-        siunity = gwy_si_unit_divide(gwy_data_field_get_si_unit_z(field), gwy_data_field_get_si_unit_xy(field), NULL);
-        gwy_si_unit_power(siunity, 2, siunity);
+        siunitx = gwy_unit_new("deg");
+        siunity = gwy_unit_divide(gwy_field_get_unit_z(field), gwy_field_get_unit_xy(field), NULL);
+        gwy_unit_power(siunity, 2, siunity);
         xlabel = "φ";
         ylabel = "w";
         title = _("Angular Slope Distribution");
         desc = _("Slopes");
     }
     else if (output_type == SLOPE_DIST_GRAPH_THETA) {
-        siunitx = gwy_si_unit_new("deg");
-        siunity = gwy_si_unit_power(siunitx, -1, NULL);
+        siunitx = gwy_unit_new("deg");
+        siunity = gwy_unit_power(siunitx, -1, NULL);
         xlabel = "θ";
         ylabel = "ρ";
         title = _("Inclination Distribution");
         desc = _("Inclinations");
     }
     else if (output_type == SLOPE_DIST_GRAPH_GRADIENT) {
-        siunitx = gwy_si_unit_divide(gwy_data_field_get_si_unit_z(field), gwy_data_field_get_si_unit_xy(field), NULL);
-        siunity = gwy_si_unit_power(siunitx, -1, NULL);
+        siunitx = gwy_unit_divide(gwy_field_get_unit_z(field), gwy_field_get_unit_xy(field), NULL);
+        siunity = gwy_unit_power(siunitx, -1, NULL);
         xlabel = "η";
         ylabel = "ρ";
         title = _("Inclination Distribution");
@@ -599,8 +583,8 @@ set_graph_model_properties(GwyGraphModel *gmodel,
     }
 
     g_object_set(gmodel,
-                 "si-unit-x", siunitx,
-                 "si-unit-y", siunity,
+                 "unit-x", siunitx,
+                 "unit-y", siunity,
                  "axis-label-bottom", xlabel,
                  "axis-label-left", ylabel,
                  "title", title,
@@ -618,47 +602,40 @@ set_graph_model_properties(GwyGraphModel *gmodel,
 }
 
 static void
-compute_slopes(GwyDataField *field,
+compute_slopes(GwyField *field,
                gint kernel_size,
-               GwyDataField *xder,
-               GwyDataField *yder)
+               GwyField *xder,
+               GwyField *yder)
 {
     GwyPlaneFitQuantity quantites[2] = { GWY_PLANE_FIT_BX, GWY_PLANE_FIT_BY };
-    GwyDataField *fields[2] = { xder, yder };
+    GwyField *fields[2] = { xder, yder };
 
     if (!kernel_size) {
-        gwy_data_field_filter_slope(field, xder, yder);
+        gwy_field_filter_slope(field, xder, yder);
         return;
     }
 
-    gwy_data_field_fit_local_planes(field, kernel_size, 2, quantites, fields);
-    gwy_data_field_multiply(xder, 1.0/gwy_data_field_get_dx(field));
-    gwy_data_field_multiply(yder, 1.0/gwy_data_field_get_dy(field));
+    gwy_field_fit_local_planes(field, kernel_size, 2, quantites, fields);
+    gwy_field_multiply(xder, 1.0/gwy_field_get_dx(field));
+    gwy_field_multiply(yder, 1.0/gwy_field_get_dy(field));
 }
 
-static GwyDataField*
-make_datafield(GwyDataField *old,
-               gint res, gulong *count,
-               gdouble real, gboolean logscale)
+static GwyField*
+make_field(GwyField *old,
+           gint res, gulong *count,
+           gdouble real, gboolean logscale)
 {
-    GwyDataField *field;
-    GwySIUnit *unit;
+    GwyField *field;
     gdouble *d;
     gint i;
 
-    field = gwy_data_field_new(res, res, real, real, FALSE);
-    gwy_data_field_set_xoffset(field, -real/2);
-    gwy_data_field_set_yoffset(field, -real/2);
+    field = gwy_field_new(res, res, real, real, FALSE);
+    gwy_field_set_xoffset(field, -real/2);
+    gwy_field_set_yoffset(field, -real/2);
 
-    unit = gwy_si_unit_new(NULL);
-    gwy_data_field_set_si_unit_z(field, unit);
-    g_object_unref(unit);
+    gwy_unit_divide(gwy_field_get_unit_z(old), gwy_field_get_unit_xy(old), gwy_field_get_unit_xy(field));
 
-    unit = gwy_si_unit_divide(gwy_data_field_get_si_unit_z(old), gwy_data_field_get_si_unit_xy(old), NULL);
-    gwy_data_field_set_si_unit_xy(field, unit);
-    g_object_unref(unit);
-
-    d = gwy_data_field_get_data(field);
+    d = gwy_field_get_data(field);
     if (logscale) {
         for (i = 0; i < res*res; i++)
             d[i] = count[i] ? log((gdouble)count[i]) + 1.0 : 0.0;
