@@ -1,5 +1,5 @@
 /*
- *  $Id: smm.c 26520 2024-08-15 18:09:25Z yeti-dn $
+ *  $Id: smm.c 28821 2025-11-06 16:00:41Z yeti-dn $
  *  Copyright (C) 2024 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,20 +16,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <complex.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwynlfit.h>
-#include <libprocess/stats.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libgwydgets/gwyradiobuttons.h>
-#include <libgwydgets/gwycombobox.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES GWY_RUN_INTERACTIVE
@@ -76,7 +66,7 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
+    GwyField *field;
     GwyGraphModel *gmodel;
     GwyGraphCurveModel *gcmodel_nominal;
     GwyGraphCurveModel *gcmodel_measured;
@@ -107,74 +97,71 @@ typedef struct {
     GtkWidget *channel_a;
     GtkWidget *channel_b;
     GtkWidget *mask[NARGS];
-    GtkObject *capacitance_adj[NARGS];
+    GtkAdjustment *capacitance_adj[NARGS];
     GtkWidget *capacitance_spin[NARGS];
     GtkWidget *enabled[NARGS];
     GtkWidget *fit[NARGS];
     GtkWidget *display[NARGS];
-    GtkObject *ifit_adj[NFITS];
+    GtkAdjustment *ifit_adj[NFITS];
     GtkWidget *ifit_spin[NFITS];
     GtkWidget *dofit[NFITS];
     GtkWidget *rfit[NFITS];
     GtkWidget *result[NFITS];
     GtkWidget *rss;
-    GwyContainer *data;
     GtkWidget *view;
     gboolean in_update;
 } ModuleGUI;
 
-static gboolean         module_register         (void);
-static GwyParamDef*     define_module_params    (void);
-static void             smm                     (GwyContainer *data,
-                                                 GwyRunType runtype);
-static void             execute                 (ModuleArgs *args);
-static GwyDialogOutcome run_gui                 (ModuleArgs *args,
-                                                 GwyContainer *data,
-                                                 gint id);
-static GtkWidget*       create_image_table      (ModuleGUI *gui);
-static GtkWidget*       create_fit_table        (ModuleGUI *gui);
-static void             param_changed           (ModuleGUI *gui,
-                                                 gint id);
-static void             dialog_response         (GwyDialog *dialog,
-                                                 gint response,
-                                                 ModuleGUI *gui);
-static void             preview                 (gpointer user_data);
-static void             estimate                (ModuleGUI *gui);
-static void             fit                     (ModuleGUI *gui);
-static void             enabled_changed         (ModuleGUI *gui,
-                                                 GtkToggleButton *check);
-static void             fit_changed             (ModuleGUI *gui,
-                                                 GtkToggleButton *check);
-static void             image_selected          (ModuleGUI *gui,
-                                                 GwyDataChooser *chooser);
-static void             display_changed         (ModuleGUI *gui,
-                                                 GtkToggleButton *toggle);
-static void             capacitance_changed     (ModuleGUI *gui,
-                                                 GtkAdjustment *adj);
-static void             dofit_changed           (ModuleGUI *gui,
-                                                 GtkToggleButton *check);
-static void             copy_inits              (GObject *button,
-                                                 ModuleGUI *gui);
-static void             ifit_changed            (ModuleGUI *gui,
-                                                 GtkAdjustment *adj);
-static void             smmeval                 (double complex s11m[3],
-                                                 double complex s11[3],
-                                                 double complex *re00,
-                                                 double complex *re01,
-                                                 double complex *re11);
-static gdouble          smmfit                  (double complex *s11m,
-                                                 double complex *s11,
-                                                 gint n,
-                                                 double complex *re00,
-                                                 double complex *re01,
-                                                 double complex *re11,
-                                                 gboolean *fix);
-static gboolean         image_filter            (GwyContainer *data,
-                                                 gint id,
-                                                 gpointer user_data);
-static gchar*           create_report           (ModuleGUI *gui);
-static void             save_coeffs             (ModuleGUI *gui);
-static void             copy_coeffs             (ModuleGUI *gui);
+static gboolean         module_register     (void);
+static GwyParamDef*     define_module_params(void);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
+static void             execute             (ModuleArgs *args);
+static GwyDialogOutcome run_gui             (ModuleArgs *args,
+                                             GwyFile *data,
+                                             gint id);
+static GtkWidget*       create_image_table  (ModuleGUI *gui);
+static GtkWidget*       create_fit_table    (ModuleGUI *gui);
+static void             param_changed       (ModuleGUI *gui,
+                                             gint id);
+static void             dialog_response     (GwyDialog *dialog,
+                                             gint response,
+                                             ModuleGUI *gui);
+static void             preview             (gpointer user_data);
+static void             estimate            (ModuleGUI *gui);
+static void             fit                 (ModuleGUI *gui);
+static void             enabled_changed     (ModuleGUI *gui,
+                                             GtkToggleButton *check);
+static void             fit_changed         (ModuleGUI *gui,
+                                             GtkToggleButton *check);
+static void             image_selected      (ModuleGUI *gui,
+                                             GwyDataChooser *chooser);
+static void             display_changed     (ModuleGUI *gui,
+                                             GtkToggleButton *toggle);
+static void             capacitance_changed (ModuleGUI *gui,
+                                             GtkAdjustment *adj);
+static void             dofit_changed       (ModuleGUI *gui,
+                                             GtkToggleButton *check);
+static void             copy_inits          (GObject *button,
+                                             ModuleGUI *gui);
+static void             ifit_changed        (ModuleGUI *gui,
+                                             GtkAdjustment *adj);
+static void             smmeval             (double complex s11m[3],
+                                             double complex s11[3],
+                                             double complex *re00,
+                                             double complex *re01,
+                                             double complex *re11);
+static gdouble          smmfit              (double complex *s11m,
+                                             double complex *s11,
+                                             gint n,
+                                             double complex *re00,
+                                             double complex *re01,
+                                             double complex *re11,
+                                             gboolean *fix);
+static gboolean         image_filter        (GwyFile *data,
+                                             gint id,
+                                             gpointer user_data);
+static gchar*           create_report       (gpointer user_data);
 
 
 static GwyModuleInfo module_info = {
@@ -193,11 +180,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("smm",
-                              (GwyProcessFunc)&smm,
+                              module_main,
                               N_("/_SPM Modes/_Electrical/_SMM Calibration..."),
                               NULL,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Calibrate SMM using capacitors calibration sample"));
 
     return TRUE;
@@ -229,7 +216,7 @@ define_module_params(void)
     paramdef = gwy_param_def_new();
     gwy_param_def_set_function_name(paramdef, gwy_process_func_current());
     gwy_param_def_add_target_graph(paramdef, PARAM_TARGET_GRAPH, "target_graph", NULL);
-    gwy_param_def_add_int(paramdef, PARAM_DISPLAY, NULL, gwy_sgettext("verb|Display"), 0, NARGS-1, 0);
+    gwy_param_def_add_int(paramdef, PARAM_DISPLAY, NULL, C_("verb", "Display"), 0, NARGS-1, 0);
     gwy_param_def_add_image_id(paramdef, PARAM_IMAGE_A, "image_a", _("First channel"));
     gwy_param_def_add_image_id(paramdef, PARAM_IMAGE_B, "image_b", _("Second channel"));
     gwy_param_def_add_gwyenum(paramdef, PARAM_SIGNAL_A, "signal_a", "",
@@ -257,7 +244,7 @@ define_module_params(void)
     }
     for (i = 0; i < NARGS; i++) {
         gwy_param_def_add_boolean(paramdef, PARAM_FIT_0 + i,
-                                  g_strdup_printf("fit/%u", i), g_strdup_printf("%s %u", gwy_sgettext("verb|Fit"), i),
+                                  g_strdup_printf("fit/%u", i), g_strdup_printf("%s %u", C_("verb", "Fit"), i),
                                   i == 0 || i == 1);
     }
     for (i = 0; i < NARGS; i++) {
@@ -280,17 +267,17 @@ define_module_params(void)
 }
 
 static void
-smm(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome;
     ModuleArgs args;
     GwyAppDataId dataid;
     gint id;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(args.field);
 
     args.gmodel = gwy_graph_model_new();
@@ -301,7 +288,7 @@ smm(GwyContainer *data, GwyRunType runtype)
                  "title", _("Capacitance"),
                  "axis-label-left", _("S<sub>11</sub>"),
                  "axis-label-bottom", _("Capacitance"),
-                 "si-unit_x", gwy_si_unit_new("F"),
+                 "unit-x", gwy_unit_new("F"),
                  NULL);
     g_object_set(args.gcmodel_nominal,
                  "mode", GWY_GRAPH_CURVE_POINTS,
@@ -319,7 +306,7 @@ smm(GwyContainer *data, GwyRunType runtype)
 
     args.params = gwy_params_new_from_settings(define_module_params());
     /* The first image is always the current image; it is always enabled and always displayed. */
-    dataid.datano = gwy_app_data_browser_get_number(data);
+    dataid.datano = gwy_file_get_id(data);
     dataid.id = id;
     gwy_params_set_image_id(args.params, PARAM_IMAGE_0, dataid);
     gwy_params_set_image_id(args.params, PARAM_IMAGE_A, dataid);
@@ -341,39 +328,30 @@ end:
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     GwyDialog *dialog;
     GwyParamTable *table;
     GtkWidget *graph, *dataview, *hbox;
     ModuleGUI gui;
-    GwyDialogOutcome outcome;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
 
-    gui.data = gwy_container_new();
-    gwy_container_set_object_by_name(gui.data, "/0/data", args->field);
-    gwy_app_sync_data_items(data, gui.data, id, 0, FALSE,
-                            GWY_DATA_ITEM_PALETTE,
-                            0);
-
     gui.dialog = gwy_dialog_new(_("SMM Calibration"));
     dialog = GWY_DIALOG(gui.dialog);
 
     gtk_dialog_add_button(GTK_DIALOG(dialog), _("_Plot Inits"), RESPONSE_PLOTINITS);
-    gtk_dialog_add_button(GTK_DIALOG(dialog), gwy_sgettext("verb|_Estimate"), RESPONSE_ESTIMATE);
-    gtk_dialog_add_button(GTK_DIALOG(dialog), gwy_sgettext("verb|_Fit"), RESPONSE_FIT);
-    gtk_dialog_add_button(GTK_DIALOG(dialog), _("_Reset"), GWY_RESPONSE_RESET);
-    gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
-    gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_OK, GTK_RESPONSE_OK);
-    gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);
+    gtk_dialog_add_button(GTK_DIALOG(dialog), C_("verb", "_Estimate"), RESPONSE_ESTIMATE);
+    gtk_dialog_add_button(GTK_DIALOG(dialog), C_("verb", "_Fit"), RESPONSE_FIT);
+    gwy_dialog_add_buttons(dialog, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    hbox = gwy_hbox_new(0);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
     gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
     gwy_dialog_add_content(dialog, hbox, TRUE, TRUE, 0);
 
-    dataview = gui.view = gwy_create_preview(gui.data, 0, PREVIEW_SMALL_SIZE, FALSE);
+    dataview = gui.view = gwy_create_preview(args->field, NULL, PREVIEW_SMALL_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(gui.view), data, GWY_FILE_IMAGE, id, GWY_FILE_ITEM_PALETTE);
     gtk_box_pack_start(GTK_BOX(hbox), dataview, FALSE, FALSE, 0);
 
     graph = gwy_graph_new(args->gmodel);
@@ -381,7 +359,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_graph_enable_user_input(GWY_GRAPH(graph), FALSE);
     gtk_box_pack_start(GTK_BOX(hbox), graph, TRUE, TRUE, 0);
 
-    hbox = gwy_hbox_new(20);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 20);
     gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
     gwy_dialog_add_content(dialog, hbox, FALSE, FALSE, 0);
 
@@ -413,18 +391,14 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     g_signal_connect_after(dialog, "response", G_CALLBACK(dialog_response), &gui);
     gwy_dialog_set_preview_func(dialog, GWY_PREVIEW_IMMEDIATE, preview, &gui, NULL);
 
-    outcome = gwy_dialog_run(dialog);
-
-    g_object_unref(gui.data);
-
-    return outcome;
+    return gwy_dialog_run(dialog);
 }
 
 static GtkWidget*
 create_image_table(ModuleGUI *gui)
 {
     GwyParams *params = gui->args->params;
-    GtkTable *table;
+    GtkGrid *grid;
     GwyDataChooser *chooser;
     GtkWidget *label, *check, *button;
     GwyAppDataId dataid;
@@ -432,52 +406,50 @@ create_image_table(ModuleGUI *gui)
     gchar *s;
     guint i;
 
-    table = GTK_TABLE(gtk_table_new(1+NARGS, 6, FALSE));
-    gtk_table_set_row_spacings(table, 2);
-    gtk_table_set_col_spacings(table, 6);
-    gtk_container_set_border_width(GTK_CONTAINER(table), 4);
-
-    gtk_table_attach(table, gwy_label_new_header(_("Masks")), 0, 3, 0, 1, GTK_FILL, 0, 0, 0);
-    gtk_table_attach(table, gwy_label_new_header(_("Capacitance [fF]")), 3, 4, 0, 1, GTK_FILL, 0, 0, 0);
-    gtk_table_attach(table, gwy_label_new_header(_("Use")), 4, 5, 0, 1, GTK_FILL, 0, 0, 0);
-    gtk_table_attach(table, gwy_label_new_header(_("Show")), 5, 6, 0, 1, GTK_FILL, 0, 0, 0);
+    grid = GTK_GRID(gtk_grid_new());
+    gtk_grid_set_row_spacing(grid, 2);
+    gtk_grid_set_column_spacing(grid, 6);
+    gtk_container_set_border_width(GTK_CONTAINER(grid), 4);
 
+    gtk_grid_attach(grid, gwy_label_new_header(_("Masks")), 0, 0, 3, 1);
+    gtk_grid_attach(grid, gwy_label_new_header(_("Capacitance [fF]")), 3, 0, 1, 1);
+    gtk_grid_attach(grid, gwy_label_new_header(_("Use")), 4, 0, 1, 1);
+    gtk_grid_attach(grid, gwy_label_new_header(_("Show")), 5, 0, 1, 1);
 
     for (i = 0; i < NARGS; i++) {
         s = g_strdup_printf("%u", i+1);
         label = gtk_label_new(s);
         g_free(s);
-        gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
-        gtk_table_attach(table, label, 0, 1, i+1, i+2, GTK_FILL, 0, 0, 0);
+        gtk_label_set_xalign(GTK_LABEL(label), 1.0);
+        gtk_grid_attach(grid, label, 0, i+1, 1, 1);
 
         gui->enabled[i] = check = gtk_check_button_new();
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), gwy_params_get_boolean(params, PARAM_ENABLED_0 + i));
         g_object_set_data(G_OBJECT(check), "id", GUINT_TO_POINTER(i));
-        gtk_table_attach(table, check, 1, 2, i+1, i+2, GTK_FILL, 0, 0, 0);
+        gtk_grid_attach(grid, check, 1, i+1, 1, 1);
 
-        gui->mask[i] = gwy_data_chooser_new_channels();
+        gui->mask[i] = gwy_data_chooser_new(GWY_FILE_IMAGE);
         g_object_set_data(G_OBJECT(gui->mask[i]), "id", GUINT_TO_POINTER(i));
-        gtk_table_attach(table, gui->mask[i], 2, 3, i+1, i+2, GTK_FILL, 0, 0, 0);
+        gtk_grid_attach(grid, gui->mask[i], 2, i+1, 1, 1);
 
         gui->capacitance_adj[i] = gtk_adjustment_new(gwy_params_get_double(params, PARAM_CAPACITANCE_0 + i)/1e-15,
                                                      0.001, 1000, 0.001, 0.01, 0);
-        gui->capacitance_spin[i] = gtk_spin_button_new(GTK_ADJUSTMENT(gui->capacitance_adj[i]), 1, 2);
+        gui->capacitance_spin[i] = gtk_spin_button_new(gui->capacitance_adj[i], 1, 2);
         gtk_spin_button_set_digits(GTK_SPIN_BUTTON(gui->capacitance_spin[i]), 3);
         g_object_set_data(G_OBJECT(gui->capacitance_adj[i]), "id", GUINT_TO_POINTER(i));
         gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(gui->capacitance_spin[i]), TRUE);
-        gtk_table_attach(GTK_TABLE(table), gui->capacitance_spin[i],
-                         3, 4, i+1, i+2, GTK_EXPAND | GTK_FILL, 0, 0, 0);
+        gtk_grid_attach(grid, gui->capacitance_spin[i], 3, i+1, 1, 1);
 
         gui->fit[i] = check = gtk_check_button_new();
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), gwy_params_get_boolean(params, PARAM_FIT_0 + i));
         g_object_set_data(G_OBJECT(check), "id", GUINT_TO_POINTER(i));
-        gtk_table_attach(table, check, 4, 5, i+1, i+2, GTK_FILL, 0, 0, 0);
+        gtk_grid_attach(grid, check, 4, i+1, 1, 1);
 
         gui->display[i] = button = gtk_radio_button_new(group);
         g_object_set_data(G_OBJECT(button), "id", GUINT_TO_POINTER(i));
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), !i);
         group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(button));
-        gtk_table_attach(table, button, 5, 6, i+1, i+2, GTK_FILL, 0, 0, 0);
+        gtk_grid_attach(grid, button, 5, i+1, 1, 1);
     }
 
     dataid = gwy_params_get_data_id(params, PARAM_IMAGE_0);
@@ -497,97 +469,79 @@ create_image_table(ModuleGUI *gui)
         g_signal_connect_swapped(gui->fit[i], "toggled", G_CALLBACK(fit_changed), gui);
         g_signal_connect_swapped(gui->mask[i], "changed", G_CALLBACK(image_selected), gui);
         g_signal_connect_swapped(gui->display[i], "toggled", G_CALLBACK(display_changed), gui);
-        g_signal_connect_swapped(gui->capacitance_adj[i], "value-changed",
-                                 G_CALLBACK(capacitance_changed), gui);
+        g_signal_connect_swapped(gui->capacitance_adj[i], "value-changed", G_CALLBACK(capacitance_changed), gui);
     }
 
-    return GTK_WIDGET(table);
+    return GTK_WIDGET(grid);
 }
 
+/* FIXME: This should be probably rewritten to GwyFitTable. */
 static GtkWidget*
 create_fit_table(ModuleGUI *gui)
 {
     GwyParams *params = gui->args->params;
-    GtkTable *table;
-    GtkWidget *label, *check, *button, *hbox;
+    GtkGrid *grid;
+    GtkWidget *label, *check, *button;
     guint i;
 
-    table = GTK_TABLE(gtk_table_new(1+NFITS, 6, FALSE));
-    gtk_table_set_row_spacings(table, 2);
-    gtk_table_set_col_spacings(table, 6);
-    gtk_container_set_border_width(GTK_CONTAINER(table), 4);
-
-    gtk_table_attach(table, gwy_label_new_header(_("Parameter")), 0, 1, 0, 1, GTK_FILL, 0, 0, 0);
-    gtk_table_attach(table, gwy_label_new_header(_("Fit")), 1, 2, 0, 1, GTK_FILL, 0, 0, 0);
-    gtk_table_attach(table, gwy_label_new_header(_("Result")), 2, 3, 0, 1, GTK_FILL, 0, 0, 0);
-    gtk_table_attach(table, gwy_label_new_header(_("Initial")), 4, 5, 0, 1, GTK_FILL, 0, 0, 0);
+    grid = GTK_GRID(gtk_grid_new());
+    gtk_grid_set_row_spacing(grid, 2);
+    gtk_grid_set_column_spacing(grid, 6);
+    gtk_container_set_border_width(GTK_CONTAINER(grid), 4);
 
+    gtk_grid_attach(grid, gwy_label_new_header(_("Parameter")), 0, 0, 1, 1);
+    gtk_grid_attach(grid, gwy_label_new_header(_("Fit")), 1, 0, 1, 1);
+    gtk_grid_attach(grid, gwy_label_new_header(_("Result")), 2, 0, 1, 1);
+    gtk_grid_attach(grid, gwy_label_new_header(_("Initial")), 4, 0, 1, 1);
 
     for (i = 0; i < NFITS; i++) {
-        label = gtk_label_new(gwy_param_def_get_param_label(gwy_params_get_def(params),
-                                                            PARAM_IFIT_0 + i));
-        gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
-        gtk_table_attach(table, label, 0, 1, i+1, i+2, GTK_FILL, 0, 0, 0);
+        label = gtk_label_new(gwy_param_def_get_param_label(gwy_params_get_def(params), PARAM_IFIT_0 + i));
+        gtk_label_set_xalign(GTK_LABEL(label), 1.0);
+        gtk_grid_attach(grid, label, 0, i+1, 1, 1);
 
         gui->dofit[i] = check = gtk_check_button_new();
         g_object_set_data(G_OBJECT(check), "id", GUINT_TO_POINTER(i));
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), gwy_params_get_boolean(params, PARAM_DOFIT_0 + i));
-        gtk_table_attach(table, check, 1, 2, i+1, i+2, GTK_FILL, 0, 0, 0);
+        gtk_grid_attach(grid, check, 1, i+1, 1, 1);
 
         gui->result[i] = label = gtk_label_new(NULL);
-        gtk_table_attach(table, label, 2, 3, i+1, i+2, GTK_FILL, 0, 0, 0);
+        gtk_grid_attach(grid, label, 2, i+1, 1, 1);
 
         button = gtk_button_new_with_label("→");
         gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
-        gtk_table_attach(table, button, 3, 4, i+1, i+2, 0, 0, 0, 0);
+        gtk_grid_attach(grid, button, 3, i+1, 1, 1);
         g_object_set_data(G_OBJECT(button), "id", GUINT_TO_POINTER(i));
         g_signal_connect(button, "clicked", G_CALLBACK(copy_inits), gui);
 
         gui->ifit_adj[i] = gtk_adjustment_new(gwy_params_get_double(params, PARAM_IFIT_0 + i),
                                               -100, 100, 0.001, 0.01, 0);
         g_object_set_data(G_OBJECT(gui->ifit_adj[i]), "id", GUINT_TO_POINTER(i));
-        gui->ifit_spin[i] = gtk_spin_button_new(GTK_ADJUSTMENT(gui->ifit_adj[i]), 1, 2);
+        gui->ifit_spin[i] = gtk_spin_button_new(gui->ifit_adj[i], 1, 2);
         gtk_spin_button_set_digits(GTK_SPIN_BUTTON(gui->ifit_spin[i]), 3);
         gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(gui->ifit_spin[i]), TRUE);
-        gtk_table_attach(GTK_TABLE(table), gui->ifit_spin[i],
-                         4, 5, i+1, i+2, GTK_EXPAND | GTK_FILL, 0, 0, 0);
+        gtk_grid_attach(grid, gui->ifit_spin[i], 4, i+1, 1, 1);
     }
 
     for (i = 0; i < NFITS; i++) {
         g_signal_connect_swapped(gui->dofit[i], "toggled", G_CALLBACK(dofit_changed), gui);
-        g_signal_connect_swapped(gui->ifit_adj[i], "value-changed",
-                                 G_CALLBACK(ifit_changed), gui);
+        g_signal_connect_swapped(gui->ifit_adj[i], "value-changed", G_CALLBACK(ifit_changed), gui);
     }
 
     gui->rss = gtk_label_new(_("Data not fitted."));
-    gtk_misc_set_alignment(GTK_MISC(gui->rss), 0.0, 0.5);
-    gtk_table_attach(table, gui->rss, 0, 5, i+1, i+2, GTK_FILL, 0, 0, 0);
+    gtk_label_set_xalign(GTK_LABEL(gui->rss), 0.0);
+    gtk_grid_attach(grid, gui->rss, 0, i+1, 5, 1);
     i++;
 
-    hbox = gwy_hbox_new(0);
-    gtk_table_attach(GTK_TABLE(table), hbox,
-                     4, 5, i+1, i+2, GTK_EXPAND | GTK_FILL, 0, 0, 0);
+    GtkWidget *rexport = gwy_results_export_new(GWY_RESULTS_REPORT_TABSEP);
+    gwy_results_export_set_style(GWY_RESULTS_EXPORT(rexport), GWY_RESULTS_EXPORT_FIXED_FORMAT);
+    gwy_results_export_set_format_func(GWY_RESULTS_EXPORT(rexport), create_report, gui);
+    GtkWidget *hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
+    gtk_box_pack_end(GTK_BOX(hbox), rexport, FALSE, FALSE, 0);
+    gtk_grid_attach(grid, hbox, 4, i+1, 1, 1);
 
-
-    button = gtk_button_new();
-    gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
-    gtk_widget_set_tooltip_text(button, _("Save table to a file"));
-    gtk_container_add(GTK_CONTAINER(button), gtk_image_new_from_stock(GTK_STOCK_SAVE, GTK_ICON_SIZE_SMALL_TOOLBAR));
-    gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 0);
-    g_signal_connect_swapped(button, "clicked", G_CALLBACK(save_coeffs), gui);
-
-    button = gtk_button_new();
-    gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
-    gtk_widget_set_tooltip_text(button, _("Copy table to clipboard"));
-    gtk_container_add(GTK_CONTAINER(button), gtk_image_new_from_stock(GTK_STOCK_COPY, GTK_ICON_SIZE_SMALL_TOOLBAR));
-    gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 0);
-    g_signal_connect_swapped(button, "clicked", G_CALLBACK(copy_coeffs), gui);
-
-
-    return GTK_WIDGET(table);
+    return GTK_WIDGET(grid);
 }
 
-
 static void
 param_changed(ModuleGUI *gui, gint id)
 {
@@ -631,12 +585,10 @@ param_changed(ModuleGUI *gui, gint id)
         if (signalb == SMM_SIGNAL_REAL && signala == SMM_SIGNAL_IMAGINARY)
             sens = TRUE;
 
-        if (signalb == SMM_SIGNAL_PHASE
-            && (signala == SMM_SIGNAL_MAG || signala == SMM_SIGNAL_LOGMAG))
+        if (signalb == SMM_SIGNAL_PHASE && (signala == SMM_SIGNAL_MAG || signala == SMM_SIGNAL_LOGMAG))
             sens = TRUE;
 
-        if (signala == SMM_SIGNAL_PHASE
-            && (signalb == SMM_SIGNAL_MAG || signalb == SMM_SIGNAL_LOGMAG))
+        if (signala == SMM_SIGNAL_PHASE && (signalb == SMM_SIGNAL_MAG || signalb == SMM_SIGNAL_LOGMAG))
             sens = TRUE;
 
          gtk_dialog_set_response_sensitive(GTK_DIALOG(gui->dialog), RESPONSE_ESTIMATE, nfits > 2 && sens);
@@ -655,10 +607,10 @@ param_changed(ModuleGUI *gui, gint id)
     }
 
     if (id < 0 || (id >= PARAM_IMAGE_0 && id < PARAM_IMAGE_0 + NARGS) || id == PARAM_DISPLAY) {
-        GwyDataField *field = gwy_params_get_image(params, PARAM_IMAGE_0 + i);
-        gwy_container_set_object_by_name(gui->data, "/0/data", field);
-        gwy_data_field_data_changed(field);
-        gwy_set_data_preview_size(GWY_DATA_VIEW(gui->view), PREVIEW_SMALL_SIZE);
+        GwyField *field = gwy_params_get_image(params, PARAM_IMAGE_0 + i);
+        GwyDataView *dataview = GWY_DATA_VIEW(gui->view);
+        gwy_data_view_set_field(dataview, field);
+        gwy_set_data_preview_size(dataview, PREVIEW_SMALL_SIZE);
     }
 
     if (id >= PARAM_ENABLED_0 && id < PARAM_ENABLED_0 + NARGS) {
@@ -695,7 +647,8 @@ dialog_response(G_GNUC_UNUSED GwyDialog *dialog, gint response, ModuleGUI *gui)
         }
         for (i = 0; i < NFITS; i++)
             gtk_label_set_text(GTK_LABEL(gui->result[i]), "");
-    } else if (response == RESPONSE_ESTIMATE)
+    }
+    else if (response == RESPONSE_ESTIMATE)
         estimate(gui);
     else if (response == RESPONSE_FIT)
         fit(gui);
@@ -805,35 +758,35 @@ copy_inits(GObject *button, ModuleGUI *gui)
     if (args->fitted) {
         if (i == 0) {
             args->init_e00 = creal(args->e00) + I*cimag(args->init_e00);
-            gtk_adjustment_set_value(GTK_ADJUSTMENT(gui->ifit_adj[0]), creal(args->init_e00));
+            gtk_adjustment_set_value(gui->ifit_adj[0], creal(args->init_e00));
         } else if (i == 1) {
             args->init_e00 = creal(args->init_e00) + I*cimag(args->e00);
-            gtk_adjustment_set_value(GTK_ADJUSTMENT(gui->ifit_adj[1]), cimag(args->init_e00));
+            gtk_adjustment_set_value(gui->ifit_adj[1], cimag(args->init_e00));
         } else if (i == 2) {
             args->init_e01 = creal(args->e01) + I*cimag(args->init_e01);
-            gtk_adjustment_set_value(GTK_ADJUSTMENT(gui->ifit_adj[2]), creal(args->init_e01));
+            gtk_adjustment_set_value(gui->ifit_adj[2], creal(args->init_e01));
         } else if (i == 3) {
             args->init_e01 = creal(args->init_e01) + I*cimag(args->e01);
-            gtk_adjustment_set_value(GTK_ADJUSTMENT(gui->ifit_adj[3]), cimag(args->init_e01));
+            gtk_adjustment_set_value(gui->ifit_adj[3], cimag(args->init_e01));
         } else if (i == 4) {
             args->init_e11 = creal(args->e11) + I*cimag(args->init_e11);
-            gtk_adjustment_set_value(GTK_ADJUSTMENT(gui->ifit_adj[4]), creal(args->init_e11));
+            gtk_adjustment_set_value(gui->ifit_adj[4], creal(args->init_e11));
         } else if (i == 5) {
             args->init_e11 = creal(args->init_e11) + I*cimag(args->e11);
-            gtk_adjustment_set_value(GTK_ADJUSTMENT(gui->ifit_adj[5]), cimag(args->init_e11));
+            gtk_adjustment_set_value(gui->ifit_adj[5], cimag(args->init_e11));
         }
     }
 }
 
 static gboolean
-image_filter(GwyContainer *data, gint id, gpointer user_data)
+image_filter(GwyFile *data, gint id, gpointer user_data)
 {
     ModuleArgs *args = (ModuleArgs*)user_data;
-    GwyDataField *otherfield, *field = args->field;
+    GwyField *otherfield, *field = args->field;
 
-    if (!gwy_container_gis_object(data, gwy_app_get_data_key_for_id(id), &otherfield))
+    if (!(otherfield = gwy_file_get_image(data, id)))
         return FALSE;
-    return !gwy_data_field_check_compatibility(field, otherfield, GWY_DATA_COMPATIBILITY_RES);
+    return !gwy_field_is_incompatible(field, otherfield, GWY_DATA_MISMATCH_RES);
 }
 
 static void
@@ -849,7 +802,7 @@ static void
 execute(ModuleArgs *args)
 {
     GwyParams *params = args->params;
-    GwyDataField *mask, *dfa, *dfb;
+    GwyField *mask, *dfa, *dfb;
     gdouble xdata[NARGS], freq, refimp, aval, bval, res11m, ims11m;
     double complex s11m, s11n, s11c, zs, zr;
     gint i, ndata, show, signala, signalb;
@@ -863,24 +816,20 @@ execute(ModuleArgs *args)
     show = gwy_params_get_enum(params, PARAM_SHOW);
     freq = gwy_params_get_double(params, PARAM_FREQ)*1e9;
     refimp = gwy_params_get_double(params, PARAM_REFIMP);
-  
-    //the situations below should be prevented elsewhere 
-    if (gwy_data_field_check_compatibility(dfa, dfb, 
-                                           GWY_DATA_COMPATIBILITY_RES | GWY_DATA_COMPATIBILITY_REAL)) {
+
+    //the situations below should be prevented elsewhere
+    if (gwy_field_is_incompatible(dfa, dfb, GWY_DATA_MISMATCH_RES | GWY_DATA_MISMATCH_REAL))
         return;
-    }
 
     for (i = 0; i < NARGS; i++) {
         if (gwy_params_get_boolean(params, PARAM_ENABLED_0 + i)) {
-           mask = gwy_params_get_image(params, PARAM_IMAGE_0 + i);
-           if (gwy_data_field_check_compatibility(dfa, mask,
-                                                  GWY_DATA_COMPATIBILITY_RES | GWY_DATA_COMPATIBILITY_REAL)) {
-               return;
-           }
+            mask = gwy_params_get_image(params, PARAM_IMAGE_0 + i);
+            if (gwy_field_is_incompatible(dfa, mask, GWY_DATA_MISMATCH_RES | GWY_DATA_MISMATCH_REAL)) {
+                return;
+            }
         }
     }
 
- 
     zr = refimp;
 
     ndata = 0;
@@ -898,45 +847,45 @@ execute(ModuleArgs *args)
 
             mask = gwy_params_get_image(params, PARAM_IMAGE_0 + i);
 
-            aval =  gwy_data_field_area_get_avg(dfa, mask, 0, 0,
-                                                gwy_data_field_get_xres(dfa),
-                                                gwy_data_field_get_yres(dfa));
+            aval =  gwy_field_area_get_avg(dfa, mask, GWY_MASK_INCLUDE, 0, 0,
+                                           gwy_field_get_xres(dfa), gwy_field_get_yres(dfa));
 
-            bval =  gwy_data_field_area_get_avg(dfb, mask, 0, 0,
-                                                gwy_data_field_get_xres(dfb),
-                                                gwy_data_field_get_yres(dfb));
+            bval =  gwy_field_area_get_avg(dfb, mask, GWY_MASK_INCLUDE, 0, 0,
+                                           gwy_field_get_xres(dfb), gwy_field_get_yres(dfb));
 
             if (signala == SMM_SIGNAL_REAL && signalb == SMM_SIGNAL_IMAGINARY) {
                 res11m = aval;
                 ims11m = bval;
-            } else if (signalb == SMM_SIGNAL_REAL && signala == SMM_SIGNAL_IMAGINARY) {
+            }
+            else if (signalb == SMM_SIGNAL_REAL && signala == SMM_SIGNAL_IMAGINARY) {
                 res11m = bval;
                 ims11m = aval;
-            } else if (signalb == SMM_SIGNAL_PHASE
-                       && (signala == SMM_SIGNAL_MAG || signala == SMM_SIGNAL_LOGMAG)) {
+            }
+            else if (signalb == SMM_SIGNAL_PHASE && (signala == SMM_SIGNAL_MAG || signala == SMM_SIGNAL_LOGMAG)) {
                 if (signala == SMM_SIGNAL_LOGMAG)
-                    aval = pow(10, aval/20);
+                    aval = gwy_exp10(aval/20);
 
-                if (gwy_si_unit_equal_string(gwy_data_field_get_si_unit_z(dfb), "rad"))
+                if (gwy_unit_equal_string(gwy_field_get_unit_z(dfb), "rad"))
                     mult = 1;
                 else
                     mult = G_PI/180;
 
                 res11m = aval*cos(bval*mult);
                 ims11m = aval*sin(bval*mult);
-            } else if (signala == SMM_SIGNAL_PHASE
-                       && (signalb == SMM_SIGNAL_MAG || signalb == SMM_SIGNAL_LOGMAG)) {
+            }
+            else if (signala == SMM_SIGNAL_PHASE && (signalb == SMM_SIGNAL_MAG || signalb == SMM_SIGNAL_LOGMAG)) {
                 if (signalb == SMM_SIGNAL_LOGMAG)
-                    bval = pow(10, bval/20);
+                    bval = gwy_exp10(bval/20);
 
-                if (gwy_si_unit_equal_string(gwy_data_field_get_si_unit_z(dfa), "rad"))
+                if (gwy_unit_equal_string(gwy_field_get_unit_z(dfa), "rad"))
                     mult = 1;
                 else
                     mult = G_PI/180;
 
                 res11m = bval*cos(aval*mult);
                 ims11m = bval*sin(aval*mult);
-            } else { //should not be reached
+            }
+            else { //should not be reached
                 res11m = 0;
                 ims11m = 0;
             }
@@ -944,7 +893,8 @@ execute(ModuleArgs *args)
             if (args->fitted) {
                s11m = res11m + I*ims11m;
                s11c = (s11m - args->e00)/(args->e01 + args->e11*(s11m - args->e00));
-            } else {
+            }
+            else {
                s11m = res11m + I*ims11m;
                s11c = (s11m - args->init_e00)/(args->init_e01 + args->init_e11*(s11m - args->init_e00));
             }
@@ -990,7 +940,8 @@ execute(ModuleArgs *args)
                      "title", _("Im(S<sub>11</sub>)"),
                      "axis-label-left", _("Im(S<sub>11</sub>)"),
                      NULL);
-    } else {
+    }
+    else {
         gwy_graph_curve_model_set_data(args->gcmodel_nominal, xdata, args->capacitance, ndata);
         gwy_graph_curve_model_set_data(args->gcmodel_measured, xdata, args->capcalc, ndata);
         g_object_set(args->gmodel,
@@ -1051,12 +1002,12 @@ estimate(ModuleGUI *gui)
     args->init_e01 = e01;
     args->init_e11 = e11;
 
-    gtk_adjustment_set_value(GTK_ADJUSTMENT(gui->ifit_adj[0]), creal(e00));
-    gtk_adjustment_set_value(GTK_ADJUSTMENT(gui->ifit_adj[1]), cimag(e00));
-    gtk_adjustment_set_value(GTK_ADJUSTMENT(gui->ifit_adj[2]), creal(e01));
-    gtk_adjustment_set_value(GTK_ADJUSTMENT(gui->ifit_adj[3]), cimag(e01));
-    gtk_adjustment_set_value(GTK_ADJUSTMENT(gui->ifit_adj[4]), creal(e11));
-    gtk_adjustment_set_value(GTK_ADJUSTMENT(gui->ifit_adj[5]), cimag(e11));
+    gtk_adjustment_set_value(gui->ifit_adj[0], creal(e00));
+    gtk_adjustment_set_value(gui->ifit_adj[1], cimag(e00));
+    gtk_adjustment_set_value(gui->ifit_adj[2], creal(e01));
+    gtk_adjustment_set_value(gui->ifit_adj[3], cimag(e01));
+    gtk_adjustment_set_value(gui->ifit_adj[4], creal(e11));
+    gtk_adjustment_set_value(gui->ifit_adj[5], cimag(e11));
 
     gwy_param_table_param_changed(gui->table, PARAM_IFIT_0);
 }
@@ -1193,8 +1144,9 @@ smmfit(double complex *s11m, double complex *s11, gint n,
 }
 
 static gchar*
-create_report(ModuleGUI *gui)
+create_report(gpointer user_data)
 {
+    ModuleGUI *gui = (ModuleGUI*)user_data;
     ModuleArgs *args = gui->args;
     GString *text = g_string_new(NULL);
 
@@ -1213,32 +1165,6 @@ create_report(ModuleGUI *gui)
     return g_string_free(text, FALSE);
 }
 
-
-static void
-save_coeffs(ModuleGUI *gui)
-{
-    gchar *text;
-
-    text = create_report(gui);
-    gwy_save_auxiliary_data(_("Save Table"), GTK_WINDOW(gui->dialog), -1, text);
-    g_free(text);
-}
-
-static void
-copy_coeffs(ModuleGUI *gui)
-{
-    GtkClipboard *clipboard;
-    GdkDisplay *display;
-    gchar *text;
-
-    text = create_report(gui);
-    display = gtk_widget_get_display(gui->dialog);
-    clipboard = gtk_clipboard_get_for_display(display, GDK_SELECTION_CLIPBOARD);
-    gtk_clipboard_set_text(clipboard, text, -1);
-    g_free(text);
-}
-
-
 static void
 smmeval(double complex s11m[3], double complex s11[3],
         double complex *re00, double complex *re01, double complex *re11)
@@ -1261,5 +1187,4 @@ smmeval(double complex s11m[3], double complex s11[3],
     *re11 = e11;
 }
 
-
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
