@@ -1,5 +1,5 @@
 /*
- *  $Id: smm_apply.c 26520 2024-08-15 18:09:25Z yeti-dn $
+ *  $Id: smm_apply.c 28910 2025-11-24 18:11:09Z yeti-dn $
  *  Copyright (C) 2024 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,18 +16,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <complex.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/gwyprocess.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/correlation.h>
-#include <libprocess/filters.h>
-#include <libprocess/stats.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 
 #define RUN_MODES GWY_RUN_INTERACTIVE
 
@@ -65,9 +57,9 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *mask;
-    GwyDataField *result[OUTPUT_NTYPES];
+    GwyField *field;
+    GwyField *mask;
+    GwyField *result[OUTPUT_NTYPES];
 } ModuleArgs;
 
 typedef struct {
@@ -81,13 +73,13 @@ typedef struct {
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             smm_apply           (GwyContainer *data,
-                                             GwyRunType runtype);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static gboolean         execute             (ModuleArgs *args);
 static GwyDialogOutcome run_gui             (ModuleArgs *args);
 static void             param_changed       (ModuleGUI *gui,
                                              gint id);
-static gboolean         image_filter        (GwyContainer *data,
+static gboolean         image_filter        (GwyFile *data,
                                              gint id,
                                              gpointer user_data);
 
@@ -115,11 +107,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("smm_apply",
-                              (GwyProcessFunc)&smm_apply,
+                              module_main,
                               N_("/_SPM Modes/_Electrical/_Apply SMM Calibration..."),
                               NULL,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Apply SMM calibration coefficients"));
 
     return TRUE;
@@ -166,24 +158,23 @@ define_module_params(void)
 }
 
 static void
-smm_apply(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome;
-    GwyDataField *mask;
     ModuleArgs args;
     GwyAppDataId dataid;
     gint id, newid;
     guint i;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(args.field);
     args.params = gwy_params_new_from_settings(define_module_params());
 
-    dataid.datano = gwy_app_data_browser_get_number(data);
+    dataid.datano = gwy_file_get_id(data);
     dataid.id = id;
     gwy_params_set_image_id(args.params, PARAM_IMAGE_A, dataid);
     gwy_params_set_image_id(args.params, PARAM_IMAGE_B, dataid);
@@ -200,25 +191,23 @@ smm_apply(GwyContainer *data, GwyRunType runtype)
         if (!args.result[i])
             continue;
 
-        newid = gwy_app_data_browser_add_data_field(args.result[i], data, TRUE);
-        gwy_app_sync_data_items(data, data, id, newid, FALSE,
-                                GWY_DATA_ITEM_GRADIENT,
-                                GWY_DATA_ITEM_REAL_SQUARE,
-                                0);
-        gwy_app_set_data_field_title(data, newid, _(gwy_enum_to_string(1 << i, outputs, OUTPUT_NTYPES)));
-        if (args.mask) {
-            mask = gwy_data_field_duplicate(args.mask);
-            gwy_container_pass_object(data, gwy_app_get_mask_key_for_id(newid), mask);
-        }
-        gwy_app_channel_log_add_proc(data, id, newid);
-    }
+        newid = gwy_file_add_image(data, args.result[i]);
 
+        gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+        gwy_file_sync_items(data, GWY_FILE_IMAGE, id,
+                            data, GWY_FILE_IMAGE, newid,
+                            GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_REAL_SQUARE, FALSE);
+        gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _(gwy_enum_to_string(1 << i, outputs, OUTPUT_NTYPES)), TRUE);
+        if (args.mask)
+            gwy_file_pass_image_mask(data, newid, gwy_field_copy(args.mask));
+        gwy_log_add(data, GWY_FILE_IMAGE, id, newid);
+    }
 
 end:
     g_object_unref(args.params);
-    GWY_OBJECT_UNREF(args.mask);
+    g_clear_object(&args.mask);
     for (i = 0; i < OUTPUT_NTYPES; i++)
-        GWY_OBJECT_UNREF(args.result[i]);
+        g_clear_object(args.result + i);
 }
 
 static GwyDialogOutcome
@@ -312,21 +301,21 @@ param_changed(ModuleGUI *gui, gint id)
 }
 
 static gboolean
-image_filter(GwyContainer *data, gint id, gpointer user_data)
+image_filter(GwyFile *data, gint id, gpointer user_data)
 {
     ModuleArgs *args = (ModuleArgs*)user_data;
-    GwyDataField *otherfield, *field = args->field;
+    GwyField *otherfield, *field = args->field;
 
-    if (!gwy_container_gis_object(data, gwy_app_get_data_key_for_id(id), &otherfield))
+    if (!(otherfield = gwy_file_get_image(data, id)))
         return FALSE;
-    return !gwy_data_field_check_compatibility(field, otherfield, GWY_DATA_COMPATIBILITY_RES);
+    return !gwy_field_is_incompatible(field, otherfield, GWY_DATA_MISMATCH_RES);
 }
 
 static gboolean
 execute(ModuleArgs *args)
 {
     GwyParams *params = args->params;
-    GwyDataField *s11_re, *s11_im, *ztip_re, *ztip_im, *cap, *dfa, *dfb;
+    GwyField *s11_re, *s11_im, *ztip_re, *ztip_im, *cap, *dfa, *dfb;
     gdouble *s11redata, *s11imdata, *ztipredata, *ztipimdata, *capdata, *adata, *bdata;
     double complex zr, s11, s11m, e00, e01, e11, ztip;
     gint i, signala, signalb, xres, yres;
@@ -343,27 +332,27 @@ execute(ModuleArgs *args)
     zr = refimp;
     omega = 2*G_PI*freq;
 
-    s11_re = gwy_data_field_new_alike(dfa, FALSE);
-    s11_im = gwy_data_field_new_alike(dfa, FALSE);
-    cap = gwy_data_field_new_alike(dfa, FALSE);
-    ztip_re = gwy_data_field_new_alike(dfa, FALSE);
-    ztip_im = gwy_data_field_new_alike(dfa, FALSE);
+    s11_re = gwy_field_new_alike(dfa, FALSE);
+    s11_im = gwy_field_new_alike(dfa, FALSE);
+    cap = gwy_field_new_alike(dfa, FALSE);
+    ztip_re = gwy_field_new_alike(dfa, FALSE);
+    ztip_im = gwy_field_new_alike(dfa, FALSE);
 
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(s11_re), NULL);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(s11_im), NULL);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(ztip_re), "Ω");
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(ztip_im), "Ω");
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(cap), "F");
+    gwy_unit_clear(gwy_field_get_unit_z(s11_re));
+    gwy_unit_clear(gwy_field_get_unit_z(s11_im));
+    gwy_unit_set_from_string(gwy_field_get_unit_z(ztip_re), "Ω");
+    gwy_unit_set_from_string(gwy_field_get_unit_z(ztip_im), "Ω");
+    gwy_unit_set_from_string(gwy_field_get_unit_z(cap), "F");
 
-    xres = gwy_data_field_get_xres(dfa);
-    yres = gwy_data_field_get_yres(dfa);
-    adata = gwy_data_field_get_data(dfa);
-    bdata = gwy_data_field_get_data(dfb);
-    s11redata = gwy_data_field_get_data(s11_re);
-    s11imdata = gwy_data_field_get_data(s11_im);
-    capdata = gwy_data_field_get_data(cap);
-    ztipredata = gwy_data_field_get_data(ztip_re);
-    ztipimdata = gwy_data_field_get_data(ztip_im);
+    xres = gwy_field_get_xres(dfa);
+    yres = gwy_field_get_yres(dfa);
+    adata = gwy_field_get_data(dfa);
+    bdata = gwy_field_get_data(dfb);
+    s11redata = gwy_field_get_data(s11_re);
+    s11imdata = gwy_field_get_data(s11_im);
+    capdata = gwy_field_get_data(cap);
+    ztipredata = gwy_field_get_data(ztip_re);
+    ztipimdata = gwy_field_get_data(ztip_im);
 
     e00 = gwy_params_get_double(params, PARAM_ERR_0)
           + I*gwy_params_get_double(params, PARAM_ERR_0 + 1);
@@ -384,31 +373,32 @@ execute(ModuleArgs *args)
         } else if (signalb == SMM_SIGNAL_REAL && signala == SMM_SIGNAL_IMAGINARY) {
             res11m = bval;
             ims11m = aval;
-        } else if (signalb == SMM_SIGNAL_PHASE
-                   && (signala == SMM_SIGNAL_MAG || signala == SMM_SIGNAL_LOGMAG)) {
+        }
+        else if (signalb == SMM_SIGNAL_PHASE && (signala == SMM_SIGNAL_MAG || signala == SMM_SIGNAL_LOGMAG)) {
             if (signala == SMM_SIGNAL_LOGMAG)
-                aval = pow(10, aval/20);
+                aval = gwy_exp10(aval/20);
 
-            if (gwy_si_unit_equal_string(gwy_data_field_get_si_unit_z(dfb), "rad"))
+            if (gwy_unit_equal_string(gwy_field_get_unit_z(dfb), "rad"))
                 mult = 1;
             else
                 mult = G_PI/180;
 
             res11m = aval*cos(bval*mult);
             ims11m = aval*sin(bval*mult);
-        } else if (signala == SMM_SIGNAL_PHASE
-                   && (signalb == SMM_SIGNAL_MAG || signalb == SMM_SIGNAL_LOGMAG)) {
+        }
+        else if (signala == SMM_SIGNAL_PHASE && (signalb == SMM_SIGNAL_MAG || signalb == SMM_SIGNAL_LOGMAG)) {
             if (signalb == SMM_SIGNAL_LOGMAG)
-                bval = pow(10, bval/20);
+                bval = gwy_exp10(bval/20);
 
-            if (gwy_si_unit_equal_string(gwy_data_field_get_si_unit_z(dfa), "rad"))
+            if (gwy_unit_equal_string(gwy_field_get_unit_z(dfa), "rad"))
                 mult = 1;
             else
                 mult = G_PI/180;
 
             res11m = bval*cos(aval*mult);
             ims11m = bval*sin(aval*mult);
-        } else { //should not be reached
+        }
+        else { //should not be reached
                 res11m = 0;
                 ims11m = 0;
         }
@@ -443,11 +433,11 @@ execute(ModuleArgs *args)
         args->result[OUTPUT_CAPACITANCE] = g_object_ref(cap);
 
 
-    gwy_object_unref(s11_re);
-    gwy_object_unref(s11_im);
-    gwy_object_unref(ztip_re);
-    gwy_object_unref(ztip_im);
-    gwy_object_unref(cap);
+    g_clear_object(&s11_re);
+    g_clear_object(&s11_im);
+    g_clear_object(&ztip_re);
+    g_clear_object(&ztip_im);
+    g_clear_object(&cap);
 
     return TRUE;
 }
