@@ -1,5 +1,5 @@
 /*
- *  $Id: sphere-revolve.c 23784 2021-05-25 16:56:09Z yeti-dn $
+ *  $Id: sphere-revolve.c 28905 2025-11-24 15:51:18Z yeti-dn $
  *  Copyright (C) 2004-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,17 +16,11 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwythreads.h>
-#include <libprocess/stats.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/filters.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwyapp.h>
-#include "libgwyddion/gwyomp.h"
+#include <gwy.h>
+#include "libgwyddion/omp.h"
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
@@ -39,31 +33,30 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *result;
-    GwyDataField *bg;
+    GwyField *field;
+    GwyField *result;
+    GwyField *bg;
 } ModuleArgs;
 
 typedef struct {
     ModuleArgs *args;
     GtkWidget *dialog;
     GwyParamTable *table;
-    GwyContainer *data;
 } ModuleGUI;
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             sphrev              (GwyContainer *data,
-                                             GwyRunType runtype);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static GwyDialogOutcome run_gui             (ModuleArgs *args,
-                                             GwyContainer *data,
+                                             GwyFile *data,
                                              gint id);
 static void             param_changed       (ModuleGUI *gui,
                                              gint id);
 static void             preview             (gpointer user_data);
 static gboolean         execute             (ModuleArgs *args,
                                              GtkWindow *wait_window);
-static GwyDataField*    make_sphere         (gdouble radius,
+static GwyField*        make_sphere         (gdouble radius,
                                              gint maxres);
 
 static GwyModuleInfo module_info = {
@@ -82,11 +75,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("sphere_revolve",
-                              (GwyProcessFunc)&sphrev,
+                              module_main,
                               N_("/_Level/Revolve _Sphere..."),
-                              GWY_STOCK_REVOLVE_SPHERE,
+                              GWY_ICON_REVOLVE_SPHERE,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Level data by sphere revolution"));
 
     return TRUE;
@@ -109,45 +102,46 @@ define_module_params(void)
 }
 
 static void
-sphrev(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     ModuleArgs args;
     gint id, newid;
     GQuark quark;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_DATA_FIELD_KEY, &quark,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_FIELD_KEY, &quark,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(args.field && quark);
 
-    args.result = gwy_data_field_new_alike(args.field, TRUE);
-    args.bg = gwy_data_field_new_alike(args.field, TRUE);
+    args.result = gwy_field_new_alike(args.field, TRUE);
+    args.bg = gwy_field_new_alike(args.field, TRUE);
 
     args.params = gwy_params_new_from_settings(define_module_params());
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, id);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
             goto end;
     }
-    if (outcome != GWY_DIALOG_HAVE_RESULT && !execute(&args, gwy_app_find_window_for_channel(data, id)))
+    if (outcome != GWY_DIALOG_HAVE_RESULT
+        && !execute(&args, gwy_data_browser_get_window_for_data(data, GWY_FILE_IMAGE, id)))
         goto end;
 
-    gwy_app_undo_qcheckpointv(data, 1, &quark);
-    gwy_container_set_object(data, gwy_app_get_data_key_for_id(id), args.result);
-    gwy_app_channel_log_add_proc(data, id, id);
+    gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &quark);
+    gwy_file_set_image(data, id, args.result);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, id);
 
     if (gwy_params_get_boolean(args.params, PARAM_DO_EXTRACT)) {
-        newid = gwy_app_data_browser_add_data_field(args.bg, data, TRUE);
-        gwy_app_sync_data_items(data, data, id, newid, FALSE,
-                                GWY_DATA_ITEM_GRADIENT,
-                                GWY_DATA_ITEM_REAL_SQUARE,
-                                0);
-        gwy_app_set_data_field_title(data, newid, _("Background"));
-        gwy_app_channel_log_add(data, id, newid, NULL, NULL);
+        newid = gwy_file_add_image(data, args.bg);
+        gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+        gwy_file_sync_items(data, GWY_FILE_IMAGE, id,
+                            data, GWY_FILE_IMAGE, newid,
+                            GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_REAL_SQUARE, FALSE);
+        gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _("Background"), TRUE);
+        gwy_log_add_full(data, GWY_FILE_IMAGE, id, newid, NULL, NULL);
     }
 
 end:
@@ -157,30 +151,23 @@ end:
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
-    GwyDialogOutcome outcome;
     GwyDialog *dialog;
     GwyParamTable *table;
     GtkWidget *dataview, *hbox;
     ModuleGUI gui;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
 
-    gui.data = gwy_container_new();
-    gwy_container_set_object_by_name(gui.data, "/0/data", args->result);
-    gwy_app_sync_data_items(data, gui.data, id, 0, FALSE,
-                            GWY_DATA_ITEM_PALETTE,
-                            GWY_DATA_ITEM_RANGE,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
-
     gui.dialog = gwy_dialog_new(_("Revolve Sphere"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_UPDATE, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
+    dataview = gwy_create_preview(args->result, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_RANGE | GWY_FILE_ITEM_REAL_SQUARE);
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), FALSE);
 
     table = gui.table = gwy_param_table_new(args->params);
@@ -196,11 +183,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     g_signal_connect_swapped(table, "param-changed", G_CALLBACK(param_changed), &gui);
     gwy_dialog_set_preview_func(dialog, GWY_PREVIEW_UPON_REQUEST, preview, &gui, NULL);
 
-    outcome = gwy_dialog_run(dialog);
-
-    g_object_unref(gui.data);
-
-    return outcome;
+    return gwy_dialog_run(dialog);
 }
 
 static void
@@ -217,7 +200,7 @@ preview(gpointer user_data)
     ModuleArgs *args = gui->args;
 
     if (execute(args, GTK_WINDOW(gui->dialog))) {
-        gwy_data_field_data_changed(args->result);
+        gwy_field_data_changed(args->result);
         gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
     }
 }
@@ -225,7 +208,7 @@ preview(gpointer user_data)
 static gboolean
 execute(ModuleArgs *args, GtkWindow *wait_window)
 {
-    GwyDataField *meanfield, *rmsfield, *invfield = NULL, *sphere, *field = args->field, *bg = args->bg;
+    GwyField *meanfield, *rmsfield, *invfield = NULL, *sphere, *field = args->field, *bg = args->bg;
     gboolean inverted = gwy_params_get_boolean(args->params, PARAM_INVERTED);
     gdouble radius = gwy_params_get_double(args->params, PARAM_RADIUS);
     gdouble *rdata, *sphdata, *tmp;
@@ -236,40 +219,40 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
     gwy_app_wait_start(wait_window, _("Revolving sphere..."));
 
     if (inverted) {
-        invfield = gwy_data_field_duplicate(field);
-        gwy_data_field_multiply(invfield, -1.0);
+        invfield = gwy_field_copy(field);
+        gwy_field_multiply(invfield, -1.0);
         field = invfield;
     }
 
-    xres = gwy_data_field_get_xres(bg);
-    yres = gwy_data_field_get_yres(bg);
-    rdata = gwy_data_field_get_data(bg);
+    xres = gwy_field_get_xres(bg);
+    yres = gwy_field_get_yres(bg);
+    rdata = gwy_field_get_data(bg);
 
-    q = gwy_data_field_get_rms(field)/sqrt(5.0/6.0);
-    sphere = make_sphere(radius, gwy_data_field_get_xres(field));
+    q = gwy_field_get_rms(field)/sqrt(5.0/6.0);
+    sphere = make_sphere(radius, gwy_field_get_xres(field));
 
     /* Scale-freeing.
      * Data is normalized to have the same RMS as if it was composed from spheres of radius args->radius.  Actually we
      * normalize the sphere instead, but the effect is the same.  */
-    gwy_data_field_multiply(sphere, -q);
-    sphdata = gwy_data_field_get_data(sphere);
-    sres = gwy_data_field_get_xres(sphere);
+    gwy_field_multiply(sphere, -q);
+    sphdata = gwy_field_get_data(sphere);
+    sres = gwy_field_get_xres(sphere);
     size = sres/2;
     sc = size*sres + size;
 
-    meanfield = gwy_data_field_duplicate(field);
-    rmsfield = gwy_data_field_duplicate(field);
+    meanfield = gwy_field_copy(field);
+    rmsfield = gwy_field_copy(field);
     /* XXX: 1D apparently uses size/2 here.  Not sure why, mimic it. */
-    gwy_data_field_filter_mean(meanfield, size/2);
-    gwy_data_field_filter_rms(rmsfield, size/2);
+    gwy_field_filter_mean(meanfield, size/2);
+    gwy_field_filter_rms(rmsfield, size/2);
 
     /* Transform mean value data to avg - 2.5*rms for outlier cut-off.
      * Allows using rmsfield as a scratch buffer for the trimmed data. */
-    gwy_data_field_multiply(rmsfield, 2.5);
-    gwy_data_field_subtract_fields(meanfield, meanfield, rmsfield);
-    gwy_data_field_max_of_fields(rmsfield, meanfield, field);
+    gwy_field_multiply(rmsfield, 2.5);
+    gwy_field_subtract_fields(meanfield, meanfield, rmsfield);
+    gwy_field_max_of_fields(rmsfield, meanfield, field);
 
-    tmp = gwy_data_field_get_data(rmsfield);
+    tmp = gwy_field_get_data(rmsfield);
 
 #ifdef _OPENMP
 #pragma omp parallel if (gwy_threads_are_enabled()) default(none) \
@@ -316,7 +299,7 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
 
     if (invfield) {
         if (!cancelled)
-            gwy_data_field_multiply(bg, -1.0);
+            gwy_field_multiply(bg, -1.0);
         g_object_unref(invfield);
     }
 
@@ -325,15 +308,15 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
     g_object_unref(meanfield);
     g_object_unref(sphere);
     if (!cancelled)
-        gwy_data_field_subtract_fields(args->result, field, bg);
+        gwy_field_subtract_fields(args->result, field, bg);
 
     return !cancelled;
 }
 
-static GwyDataField*
+static GwyField*
 make_sphere(gdouble radius, gint maxres)
 {
-    GwyDataField *sphere;
+    GwyField *sphere;
     gdouble *data;
     gint i, j, size, res, sc;
     gboolean very_flat;
@@ -341,8 +324,8 @@ make_sphere(gdouble radius, gint maxres)
 
     size = GWY_ROUND(MIN(radius, maxres));
     res = 2*size + 1;
-    sphere = gwy_data_field_new(res, res, 1.0, 1.0, FALSE);
-    data = gwy_data_field_get_data(sphere);
+    sphere = gwy_field_new(res, res, 1.0, 1.0, FALSE);
+    data = gwy_field_get_data(sphere);
     sc = size*res + size;
     /* Pathological case: very flat sphere */
     very_flat = radius/8 > maxres;
