@@ -1,5 +1,5 @@
 /*
- *  $Id: straighten_path.c 24626 2022-03-03 12:59:05Z yeti-dn $
+ *  $Id: straighten_path.c 28970 2025-12-10 15:45:57Z yeti-dn $
  *  Copyright (C) 2016-2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,22 +16,12 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
 #include <string.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libprocess/correct.h>
-#include <libprocess/stats.h>
-#include <libprocess/interpolation.h>
-#include <libprocess/spline.h>
-#include <libgwydgets/gwynullstore.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_INTERACTIVE)
@@ -50,9 +40,9 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *result;
-    GwyDataField *result_mask;
+    GwyField *field;
+    GwyField *result;
+    GwyField *result_mask;
     GwySelection *selection;
     /* Cached values for input data field. */
     gboolean realsquare;
@@ -66,17 +56,16 @@ typedef struct {
     GtkWidget *view;
     GtkWidget *view_result;
     GwySelection *selection;
-    GwyContainer *data;
 } ModuleGUI;
 
 static gboolean         module_register        (void);
 static GwyParamDef*     define_module_params   (void);
-static void             straighten_path        (GwyContainer *data,
-                                                GwyRunType runtype);
+static void             module_main            (GwyFile *data,
+                                                GwyRunModeFlags mode);
 static void             execute                (ModuleArgs *args,
                                                 GwySelection *selection);
 static GwyDialogOutcome run_gui                (ModuleArgs *args,
-                                                GwyContainer *data,
+                                                GwyFile *data,
                                                 gint id);
 static void             param_changed          (ModuleGUI *gui,
                                                 gint id);
@@ -116,11 +105,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("straighten_path",
-                              (GwyProcessFunc)&straighten_path,
+                              module_main,
                               N_("/_Distortion/Straighten _Path..."),
-                              GWY_STOCK_STRAIGHTEN_PATH,
+                              GWY_ICON_STRAIGHTEN_PATH,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Straighten along a path"));
 
     return TRUE;
@@ -147,39 +136,36 @@ define_module_params(void)
 }
 
 static void
-straighten_path(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome;
     ModuleArgs args;
     GwyParams *params;
-    GwyDataField *field, *tmp;
+    GwyField *field, *tmp;
     gint id, newid, yres;
     GwySelection *selection;
     gboolean closed;
     gdouble slackness;
-    gchar key[40];
 
-    g_return_if_fail(runtype & RUN_MODES);
-    g_return_if_fail(g_type_from_name("GwyLayerPath"));
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &field,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &field,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(field);
 
-    gwy_clear(&args, 1);
+    gwy_clear1(args);
     args.field = field;
     args.params = params = gwy_params_new_from_settings(define_module_params());
-    g_snprintf(key, sizeof(key), "/%d/data/realsquare", id);
-    gwy_container_gis_boolean_by_name(data, key, &args.realsquare);
-    yres = gwy_data_field_get_yres(field);
-    args.result = gwy_data_field_new(5, yres, 5, yres, TRUE);
-    args.result_mask = gwy_data_field_new_alike(args.result, TRUE);
+    gwy_container_gis_boolean(GWY_CONTAINER(data), gwy_file_key_image_real_square(id), &args.realsquare);
+    yres = gwy_field_get_yres(field);
+    args.result = gwy_field_new(5, yres, 5, yres, TRUE);
+    args.result_mask = gwy_field_new_alike(args.result, TRUE);
 
-    g_snprintf(key, sizeof(key), "/%d/select/path", id);
-    if (gwy_container_gis_object_by_name(data, key, &selection)
+    GQuark selection_key = gwy_file_key_image_selection(id, "path");
+    if (gwy_container_gis_object(GWY_CONTAINER(data), selection_key, &selection)
         && gwy_selection_get_data(GWY_SELECTION(selection), NULL) > 1) {
         gwy_debug("init selection from container");
-        args.selection = gwy_selection_duplicate(selection);
+        args.selection = gwy_selection_copy(selection);
         gwy_selection_set_max_objects(args.selection, 1024);
         g_object_get(selection, "slackness", &slackness, "closed", &closed, NULL);
         gwy_params_set_double(params, PARAM_SLACKNESS, slackness);
@@ -187,14 +173,14 @@ straighten_path(GwyContainer *data, GwyRunType runtype)
     }
     else {
         gwy_debug("init selection afresh");
-        args.selection = g_object_new(g_type_from_name("GwySelectionPath"), NULL);
+        args.selection = gwy_selection_path_new();
         gwy_selection_set_max_objects(args.selection, 1024);
         init_selection(args.selection, &args);
     }
 
     outcome = run_gui(&args, data, id);
     gwy_params_save_to_settings(params);
-    gwy_container_set_object_by_name(data, key, args.selection);
+    gwy_container_set_object(GWY_CONTAINER(data), selection_key, args.selection);
 
     if (outcome == GWY_DIALOG_CANCEL)
         goto end;
@@ -202,22 +188,22 @@ straighten_path(GwyContainer *data, GwyRunType runtype)
         execute(&args, args.selection);
 
     if (gwy_params_get_enum(params, PARAM_ORIENTATION) == GWY_ORIENTATION_HORIZONTAL) {
-        tmp = gwy_data_field_new_rotated_90(args.result, FALSE);
+        tmp = gwy_field_new_rotated_90(args.result, FALSE);
         g_object_unref(args.result);
         args.result = tmp;
     }
 
-    newid = gwy_app_data_browser_add_data_field(args.result, data, TRUE);
-    gwy_app_set_data_field_title(data, newid, _("Straightened"));
-    gwy_app_sync_data_items(data, data, id, newid, FALSE,
-                            GWY_DATA_ITEM_RANGE_TYPE,
-                            GWY_DATA_ITEM_RANGE,
-                            GWY_DATA_ITEM_GRADIENT,
-                            GWY_DATA_ITEM_MASK_COLOR,
-                            0);
-    if (gwy_data_field_get_max(args.result_mask) > 0.0)
-        gwy_container_set_object(data, gwy_app_get_mask_key_for_id(newid), args.result_mask);
-    gwy_app_channel_log_add_proc(data, id, newid);
+    newid = gwy_file_add_image(data, args.result);
+
+    gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+    gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _("Straightened"), TRUE);
+    gwy_file_sync_items(data, GWY_FILE_IMAGE, id,
+                        data, GWY_FILE_IMAGE, newid,
+                        GWY_FILE_ITEM_COLOR_MAPPING | GWY_FILE_ITEM_RANGE
+                        | GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_MASK_COLOR, FALSE);
+    if (gwy_field_get_max(args.result_mask) > 0.0)
+        gwy_file_set_image_mask(data, newid, args.result_mask);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, newid);
 
 end:
     g_object_unref(args.selection);
@@ -227,55 +213,38 @@ end:
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
-    GtkWidget *hbox, *vbox, *alignment, *buttonbox, *button;
+    GtkWidget *hbox, *vbox, *buttonbox, *button;
     GwyDialog *dialog;
     GwyParamTable *table;
     ModuleGUI gui;
-    GwyDataField *field = args->field;
+    GwyField *field = args->field;
     GwyDialogOutcome outcome;
     gint maxthickness, thickness;
 
-    maxthickness = MAX(gwy_data_field_get_xres(field), gwy_data_field_get_yres(field))/2;
+    maxthickness = MAX(gwy_field_get_xres(field), gwy_field_get_yres(field))/2;
     maxthickness = MAX(maxthickness, 3);
     thickness = gwy_params_get_int(args->params, PARAM_THICKNESS);
     if (thickness > maxthickness)
         gwy_params_set_int(args->params, PARAM_THICKNESS, (thickness = maxthickness));
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
-    gui.data = gwy_container_new();
-    gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(0), field);
-    gwy_app_sync_data_items(data, gui.data, id, 0, FALSE,
-                            GWY_DATA_ITEM_RANGE_TYPE,
-                            GWY_DATA_ITEM_RANGE,
-                            GWY_DATA_ITEM_GRADIENT,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
-
-    gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(1), args->result);
-    gwy_container_set_object(gui.data, gwy_app_get_mask_key_for_id(1), args->result_mask);
-    gwy_app_sync_data_items(data, gui.data, id, 1, FALSE,
-                            GWY_DATA_ITEM_RANGE_TYPE,
-                            GWY_DATA_ITEM_RANGE,
-                            GWY_DATA_ITEM_GRADIENT,
-                            GWY_DATA_ITEM_MASK_COLOR,
-                            0);
 
     gui.dialog = gwy_dialog_new(_("Straighten Path"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_UPDATE, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    hbox = gwy_hbox_new(0);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
     gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
     gwy_dialog_add_content(dialog, hbox, FALSE, FALSE, 0);
 
-    vbox = gwy_vbox_new(0);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
     gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);
 
     gtk_box_pack_start(GTK_BOX(vbox), create_coord_list(&gui), TRUE, TRUE, 0);
-    buttonbox = gwy_hbox_new(0);
+    buttonbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
     gtk_box_set_homogeneous(GTK_BOX(buttonbox), TRUE);
     gtk_box_pack_start(GTK_BOX(vbox), buttonbox, FALSE, FALSE, 0);
 
@@ -305,20 +274,24 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gtk_box_pack_start(GTK_BOX(vbox), gwy_param_table_widget(table), FALSE, FALSE, 0);
     gwy_dialog_add_param_table(dialog, table);
 
-    alignment = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-    gtk_box_pack_start(GTK_BOX(hbox), alignment, FALSE, FALSE, 4);
-
-    gui.view = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
-    gui.selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(gui.view), 0, "Path", 1024, TRUE);
+    gui.view = gwy_create_preview(field, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(gui.view), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_COLOR_MAPPING | GWY_FILE_ITEM_RANGE
+                        | GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_REAL_SQUARE);
+    gtk_widget_set_halign(gui.view, GTK_ALIGN_START);
+    gtk_widget_set_valign(gui.view, GTK_ALIGN_START);
+    gui.selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(gui.view), GWY_TYPE_LAYER_PATH, 1024, TRUE);
     g_object_ref(gui.selection);
     gwy_selection_assign(gui.selection, args->selection);
-    gtk_container_add(GTK_CONTAINER(alignment), gui.view);
+    gtk_box_pack_start(GTK_BOX(hbox), gui.view, FALSE, FALSE, 4);
 
-    alignment = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-    gtk_box_pack_start(GTK_BOX(hbox), alignment, FALSE, FALSE, 4);
-
-    gui.view_result = gwy_create_preview(gui.data, 1, PREVIEW_SIZE, TRUE);
-    gtk_container_add(GTK_CONTAINER(alignment), gui.view_result);
+    gui.view_result = gwy_create_preview(args->result, args->result_mask, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(gui.view_result), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_COLOR_MAPPING | GWY_FILE_ITEM_RANGE
+                        | GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_MASK_COLOR);
+    gtk_widget_set_halign(gui.view_result, GTK_ALIGN_START);
+    gtk_widget_set_valign(gui.view_result, GTK_ALIGN_START);
+    gtk_box_pack_start(GTK_BOX(hbox), gui.view_result, FALSE, FALSE, 4);
 
     fill_coord_list(&gui);
     /* We do not get the right value before the data view is shown. */
@@ -331,7 +304,6 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 
     gwy_selection_assign(args->selection, gui.selection);
     g_object_unref(gui.selection);
-    g_object_unref(gui.data);
 
     return outcome;
 }
@@ -368,8 +340,8 @@ set_scaled_thickness(ModuleGUI *gui)
 {
     gint thickness = gwy_params_get_int(gui->args->params, PARAM_THICKNESS);
     gdouble zoom = gwy_data_view_get_real_zoom(GWY_DATA_VIEW(gui->view));
-    GwyVectorLayer *vlayer = gwy_data_view_get_top_layer(GWY_DATA_VIEW(gui->view));
-    g_object_set(vlayer, "thickness", GWY_ROUND(zoom*thickness), NULL);
+    GwyVectorLayer *vlayer = gwy_data_view_get_interactive_layer(GWY_DATA_VIEW(gui->view));
+    gwy_layer_path_set_thickness(GWY_LAYER_PATH(vlayer), zoom*thickness);
 }
 
 static void
@@ -379,8 +351,8 @@ preview(gpointer user_data)
     ModuleArgs *args = gui->args;
 
     execute(args, gui->selection);
-    gwy_data_field_data_changed(args->result);
-    gwy_data_field_data_changed(args->result_mask);
+    gwy_field_data_changed(args->result);
+    gwy_field_data_changed(args->result_mask);
     gwy_set_data_preview_size(GWY_DATA_VIEW(gui->view_result), PREVIEW_SIZE);
     gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
 }
@@ -432,7 +404,7 @@ render_coord_cell(GtkCellLayout *layout,
                   gpointer user_data)
 {
     ModuleGUI *gui = (ModuleGUI*)user_data;
-    GwyDataField *field = gui->args->field;
+    GwyField *field = gui->args->field;
     gchar buf[32];
     guint idx, id;
     gint ival;
@@ -446,9 +418,9 @@ render_coord_cell(GtkCellLayout *layout,
 
         gwy_selection_get_object(gui->selection, idx, xy);
         if (id == COLUMN_X)
-            ival = gwy_data_field_rtoj(field, xy[0]);
+            ival = gwy_field_rtoj(field, xy[0]);
         else
-            ival = gwy_data_field_rtoi(field, xy[1]);
+            ival = gwy_field_rtoi(field, xy[1]);
     }
 
     g_snprintf(buf, sizeof(buf), "%d", ival);
@@ -466,7 +438,7 @@ delete_selection_object(GtkTreeView *treeview,
     GtkTreeIter iter;
     const gint *indices;
 
-    if (event->keyval != GDK_Delete)
+    if (event->keyval != GDK_KEY_Delete)
         return FALSE;
 
     selection = gtk_tree_view_get_selection(treeview);
@@ -488,8 +460,8 @@ delete_selection_object(GtkTreeView *treeview,
 static void
 init_selection(GwySelection *selection, ModuleArgs *args)
 {
-    gdouble xreal = gwy_data_field_get_xreal(args->field);
-    gdouble yreal = gwy_data_field_get_yreal(args->field);
+    gdouble xreal = gwy_field_get_xreal(args->field);
+    gdouble yreal = gwy_field_get_yreal(args->field);
     gboolean closed = gwy_params_get_boolean(args->params, PARAM_CLOSED);
     gdouble xy[8];
 
@@ -510,10 +482,9 @@ init_selection(GwySelection *selection, ModuleArgs *args)
         gwy_selection_set_data(selection, 3, xy);
     }
 
-    g_object_set(selection,
-                 "slackness", gwy_params_get_double(args->params, PARAM_SLACKNESS),
-                 "closed", closed,
-                 NULL);
+    GwySelectionPath *selpath = GWY_SELECTION_PATH(selection);
+    gwy_selection_path_set_slackness(selpath, gwy_params_get_double(args->params, PARAM_SLACKNESS));
+    gwy_selection_path_set_closed(selpath, closed);
 }
 
 static void
@@ -569,9 +540,11 @@ param_changed(ModuleGUI *gui, gint id)
     GwyParams *params = gui->args->params;
 
     if (id < 0 || id == PARAM_CLOSED)
-        g_object_set(gui->selection, "closed", gwy_params_get_boolean(params, PARAM_CLOSED), NULL);
-    if (id < 0 || id == PARAM_SLACKNESS)
-        g_object_set(gui->selection, "slackness", gwy_params_get_double(params, PARAM_SLACKNESS), NULL);
+        gwy_selection_path_set_closed(GWY_SELECTION_PATH(gui->selection), gwy_params_get_boolean(params, PARAM_CLOSED));
+    if (id < 0 || id == PARAM_SLACKNESS) {
+        gwy_selection_path_set_slackness(GWY_SELECTION_PATH(gui->selection),
+                                         gwy_params_get_double(params, PARAM_SLACKNESS));
+    }
     if (id < 0 || id == PARAM_THICKNESS)
         set_scaled_thickness(gui);
 
@@ -580,7 +553,7 @@ param_changed(ModuleGUI *gui, gint id)
 
 /* XXX: This replicates extract_path.c */
 static GwyXY*
-rescale_points(GwySelection *selection, GwyDataField *field,
+rescale_points(GwySelection *selection, GwyField *field,
                gboolean realsquare,
                gdouble *pdx, gdouble *pdy, gdouble *pqx, gdouble *pqy)
 {
@@ -588,8 +561,8 @@ rescale_points(GwySelection *selection, GwyDataField *field,
     GwyXY *points;
     guint n, i;
 
-    dx = gwy_data_field_get_dx(field);
-    dy = gwy_data_field_get_dy(field);
+    dx = gwy_field_get_dx(field);
+    dy = gwy_field_get_dy(field);
     h = MIN(dx, dy);
     if (realsquare) {
         qx = h/dx;
@@ -617,19 +590,19 @@ rescale_points(GwySelection *selection, GwyDataField *field,
 }
 
 static void
-resize_result_field(GwyDataField *result, gint n, gint thickness, gdouble h)
+resize_result_field(GwyField *result, gint n, gint thickness, gdouble h)
 {
-    gwy_data_field_resample(result, thickness, n, GWY_INTERPOLATION_NONE);
-    gwy_data_field_set_xreal(result, h*thickness);
-    gwy_data_field_set_yreal(result, h*n);
-    gwy_data_field_set_xoffset(result, 0.0);
-    gwy_data_field_set_yoffset(result, 0.0);
+    gwy_field_resize(result, thickness, n);
+    gwy_field_set_xreal(result, h*thickness);
+    gwy_field_set_yreal(result, h*n);
+    gwy_field_set_xoffset(result, 0.0);
+    gwy_field_set_yoffset(result, 0.0);
 }
 
 static void
 execute(ModuleArgs *args, GwySelection *selection)
 {
-    GwyDataField *mask = args->result_mask, *field = args->field, *result = args->result;
+    GwyField *mask = args->result_mask, *field = args->field, *result = args->result;
     GwyInterpolationType interp = gwy_params_get_enum(args->params, PARAM_INTERP);
     guint thickness = gwy_params_get_int(args->params, PARAM_THICKNESS);
     GwySpline *spline;
@@ -649,19 +622,19 @@ execute(ModuleArgs *args, GwySelection *selection)
     g_free(points);
 
     length = gwy_spline_length(spline);
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
 
     /* This would give natural sampling for a straight line along some axis. */
     n = GWY_ROUND(length + 1.0);
     resize_result_field(result, n, thickness, h);
-    gwy_data_field_copy_units(field, result);
+    gwy_field_copy_units(field, result);
     resize_result_field(mask, n, thickness, h);
-    gwy_data_field_copy_units(field, mask);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(mask), NULL);
-    gwy_data_field_clear(mask);
+    gwy_field_copy_units(field, mask);
+    gwy_unit_clear(gwy_field_get_unit_z(mask));
+    gwy_field_clear(mask);
     if (n < 2) {
-        gwy_data_field_clear(result);
+        gwy_field_clear(result);
         return;
     }
 
@@ -669,7 +642,7 @@ execute(ModuleArgs *args, GwySelection *selection)
     tangents = g_new(GwyXY, n);
     coords = g_new(GwyXY, n*thickness);
     gwy_spline_sample_uniformly(spline, points, tangents, n);
-    m = gwy_data_field_get_data(mask);
+    m = gwy_field_get_data(mask);
     for (i = k = 0; i < n; i++) {
         gdouble xc = qx*points[i].x, yc = qy*points[i].y;
         gdouble vx = qx*tangents[i].y, vy = -qy*tangents[i].x;
@@ -689,14 +662,14 @@ execute(ModuleArgs *args, GwySelection *selection)
     }
     /* Pass mirror because we handle exterior ourselves here and mirror is the least code which simultaneously does
      * not produce undefined pixels where we disagree with the function on which pixels are numerically outside. */
-    gwy_data_field_sample_distorted(field, result, coords, interp, GWY_EXTERIOR_MIRROR_EXTEND, 0.0);
+    gwy_field_sample_distorted(field, result, coords, interp, GWY_EXTERIOR_MIRROR, 0.0);
 
     g_free(coords);
     g_free(points);
     g_free(tangents);
 
     if (have_exterior)
-        gwy_data_field_correct_average_unmasked(result, mask);
+        gwy_field_correct_average(result, mask);
 }
 
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
