@@ -1,5 +1,5 @@
 /*
- *  $Id: threshold.c 23854 2021-06-14 16:36:54Z yeti-dn $
+ *  $Id: threshold.c 29068 2026-01-02 16:17:44Z yeti-dn $
  *  Copyright (C) 2010-2021 David Necas (Yeti)
  *  E-mail: yeti@gwyddion.net
  *
@@ -16,14 +16,9 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/filters.h>
-#include <libprocess/stats.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
@@ -53,8 +48,8 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *result;
+    GwyField *field;
+    GwyField *result;
     /* Cached values for input data field. */
     gdouble min, max;
     gdouble disp_min, disp_max;
@@ -65,27 +60,26 @@ typedef struct {
     ModuleArgs *args;
     GtkWidget *dialog;
     GwyParamTable *table;
-    GwyContainer *data;
-    GwyDataField *percentfield;
+    GwyField *percentfield;
 } ModuleGUI;
 
 static gboolean         module_register       (void);
 static GwyParamDef*     define_module_params  (void);
-static void             threshold             (GwyContainer *data,
-                                               GwyRunType runtype);
+static void             module_main           (GwyFile *data,
+                                               GwyRunModeFlags mode);
 static void             execute               (ModuleArgs *args,
-                                               GwyDataField *percentfield);
+                                               GwyField *percentfield);
 static GwyDialogOutcome run_gui               (ModuleArgs *args,
-                                               GwyContainer *data,
+                                               GwyFile *data,
                                                gint id);
 static void             dialog_response       (ModuleGUI *gui,
                                                gint response);
 static void             param_changed         (ModuleGUI *gui,
                                                gint id);
 static void             preview               (gpointer user_data);
-static void             find_out_display_range(GwyContainer *container,
+static void             find_out_display_range(GwyFile *data,
                                                gint id,
-                                               GwyDataField *data_field,
+                                               GwyField *field,
                                                gdouble *disp_min,
                                                gdouble *disp_max);
 static void             sanitise_params       (ModuleArgs *args);
@@ -106,11 +100,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("threshold",
-                              (GwyProcessFunc)&threshold,
+                              module_main,
                               N_("/_Basic Operations/Li_mit Range..."),
-                              GWY_STOCK_LIMIT_RANGE,
+                              GWY_ICON_LIMIT_RANGE,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Limit data range"));
 
     return TRUE;
@@ -120,10 +114,10 @@ static GwyParamDef*
 define_module_params(void)
 {
     static const GwyEnum methods[] = {
-        { N_("Specify _thresholds"),   THRESHOLD_RANGE_USER,       },
-        { N_("Use _display range"),    THRESHOLD_RANGE_DISPLAY,    },
-        { N_("Cut off outlier_s"),     THRESHOLD_RANGE_OUTLIERS,   },
-        { N_("Limit to _percentiles"), THRESHOLD_RANGE_PERCENTILE, },
+        { N_("Specify _thresholds"), THRESHOLD_RANGE_USER,       },
+        { N_("Use _display range"),  THRESHOLD_RANGE_DISPLAY,    },
+        { N_("Cut off outlier_s"),   THRESHOLD_RANGE_OUTLIERS,   },
+        { N_("Trim _percentiles"),   THRESHOLD_RANGE_PERCENTILE, },
     };
     static GwyParamDef *paramdef = NULL;
 
@@ -143,85 +137,79 @@ define_module_params(void)
 }
 
 static void
-threshold(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome;
     ModuleArgs args;
-    GwyDataField *field;
+    GwyField *field;
     GQuark quark;
     gint id;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &field,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     GWY_APP_DATA_FIELD_KEY, &quark,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &field,
+                                 GWY_APP_FIELD_ID, &id,
+                                 GWY_APP_FIELD_KEY, &quark,
+                                 0);
     g_return_if_fail(field);
 
     args.field = field;
-    gwy_data_field_get_min_max(field, &args.min, &args.max);
-    args.avg = gwy_data_field_get_avg(field);
-    args.rms = gwy_data_field_get_rms(field);
+    gwy_field_get_min_max(field, &args.min, &args.max);
+    args.avg = gwy_field_get_avg(field);
+    args.rms = gwy_field_get_rms(field);
     find_out_display_range(data, id, field, &args.disp_min, &args.disp_max);
     args.params = gwy_params_new_from_settings(define_module_params());
     sanitise_params(&args);
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
-        args.result = gwy_data_field_new_alike(field, TRUE);
+    if (mode ==GWY_RUN_INTERACTIVE) {
+        args.result = gwy_field_new_alike(field, TRUE);
         outcome = run_gui(&args, data, id);
         gwy_params_save_to_settings(args.params);
         if (outcome != GWY_DIALOG_HAVE_RESULT)
             goto end;
-        gwy_app_undo_qcheckpointv(data, 1, &quark);
-        gwy_data_field_copy(args.result, field, FALSE);
+        gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &quark);
+        gwy_field_copy_data(args.result, field);
     }
     else {
-        gwy_app_undo_qcheckpointv(data, 1, &quark);
+        gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &quark);
         args.result = g_object_ref(field);
         execute(&args, NULL);
     }
 
-    gwy_data_field_data_changed(field);
-    gwy_app_channel_log_add_proc(data, id, id);
+    gwy_field_data_changed(field);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, id);
 
 end:
     g_object_unref(args.params);
-    GWY_OBJECT_UNREF(args.result);
+    g_clear_object(&args.result);
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     static const gint range_sliders[] = { PARAM_LOWER, PARAM_UPPER };
     GwyDialogOutcome outcome;
     GwyDialog *dialog;
     GwyParamTable *table;
     GtkWidget *dataview, *hbox;
-    GwySIValueFormat *vf;
+    GwyValueFormat *vf;
     ModuleGUI gui;
     gdouble range, slider_min, slider_max;
     gchar *s;
     gint i;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
 
-    vf = gwy_data_field_get_value_format_z(args->field, GWY_SI_UNIT_FORMAT_VFMARKUP, NULL);
+    vf = gwy_field_get_value_format_z(args->field, GWY_UNIT_FORMAT_VFMARKUP, NULL);
     vf->precision += 2;
 
-    gui.data = gwy_container_new();
-    gwy_container_set_object_by_name(gui.data, "/0/data", args->result);
-    gwy_app_sync_data_items(data, gui.data, id, 0, FALSE,
-                            GWY_DATA_ITEM_PALETTE,
-                            GWY_DATA_ITEM_RANGE,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
-
     gui.dialog = gwy_dialog_new(_("Limit Range"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
+    dataview = gwy_create_preview(args->result, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_RANGE | GWY_FILE_ITEM_REAL_SQUARE);
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), FALSE);
 
     table = gui.table = gwy_param_table_new(args->params);
@@ -272,9 +260,8 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 
     outcome = gwy_dialog_run(dialog);
 
-    g_object_unref(gui.data);
-    GWY_OBJECT_UNREF(gui.percentfield);
-    gwy_si_unit_value_format_free(vf);
+    g_clear_object(&gui.percentfield);
+    gwy_value_format_free(vf);
 
     return outcome;
 }
@@ -321,31 +308,24 @@ dialog_response(ModuleGUI *gui, gint response)
 }
 
 static void
-find_out_display_range(GwyContainer *container, gint id, GwyDataField *data_field,
+find_out_display_range(GwyFile *data, gint id, GwyField *field,
                        gdouble *disp_min, gdouble *disp_max)
 {
-    GwyLayerBasicRangeType range_type = GWY_LAYER_BASIC_RANGE_FULL;
+    GwyColorMappingType color_mapping = GWY_COLOR_MAPPING_FULL;
+    GwyContainer *container = GWY_CONTAINER(data);
 
-    gwy_container_gis_enum(container, gwy_app_get_data_range_type_key_for_id(id), &range_type);
-    switch (range_type) {
-        case GWY_LAYER_BASIC_RANGE_FULL:
-        case GWY_LAYER_BASIC_RANGE_ADAPT:
-        gwy_data_field_get_min_max(data_field, disp_min, disp_max);
-        break;
-
-        case GWY_LAYER_BASIC_RANGE_FIXED:
-        gwy_data_field_get_min_max(data_field, disp_min, disp_max);
-        gwy_container_gis_double(container, gwy_app_get_data_range_min_key_for_id(id), disp_min);
-        gwy_container_gis_double(container, gwy_app_get_data_range_max_key_for_id(id), disp_max);
-        break;
-
-        case GWY_LAYER_BASIC_RANGE_AUTO:
-        gwy_data_field_get_autorange(data_field, disp_min, disp_max);
-        break;
-
-        default:
+    gwy_container_gis_enum(container, gwy_file_key_image_color_mapping(id), &color_mapping);
+    if (color_mapping == GWY_COLOR_MAPPING_FULL || color_mapping == GWY_COLOR_MAPPING_ADAPT)
+        gwy_field_get_min_max(field, disp_min, disp_max);
+    else if (color_mapping == GWY_COLOR_MAPPING_FIXED) {
+        gwy_field_get_min_max(field, disp_min, disp_max);
+        gwy_container_gis_double(container, gwy_file_key_image_range_min(id), disp_min);
+        gwy_container_gis_double(container, gwy_file_key_image_range_max(id), disp_max);
+    }
+    else if (color_mapping == GWY_COLOR_MAPPING_AUTO)
+        gwy_field_get_autorange(field, disp_min, disp_max);
+    else {
         g_assert_not_reached();
-        break;
     }
 }
 
@@ -387,18 +367,18 @@ preview(gpointer user_data)
      * the data field values and use it repeatedly.  XXX: Since the user is going to move the slider around it would
      * be more efficient to just sort the data.  Then the percentage computation is instant. */
     if (!gui->percentfield)
-        gui->percentfield = gwy_data_field_duplicate(args->field);
+        gui->percentfield = gwy_field_copy(args->field);
     execute(args, gui->percentfield);
-    gwy_data_field_data_changed(args->result);
+    gwy_field_data_changed(args->result);
     gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
 }
 
 static void
-execute(ModuleArgs *args, GwyDataField *percentfield)
+execute(ModuleArgs *args, GwyField *percentfield)
 {
     GwyParams *params = args->params;
     ThresholdRangeMode method = gwy_params_get_enum(params, PARAM_METHOD);
-    GwyDataField *buf = NULL;
+    GwyField *buf = NULL;
     gdouble range[2];
 
     if (method == THRESHOLD_RANGE_USER) {
@@ -421,23 +401,23 @@ execute(ModuleArgs *args, GwyDataField *percentfield)
     else if (method == THRESHOLD_RANGE_PERCENTILE) {
         gdouble lower_p = gwy_params_get_double(params, PARAM_LOWER_P);
         gdouble upper_p = gwy_params_get_double(params, PARAM_UPPER_P);
-        gdouble *pdata;
-        gdouble p[2];
 
         if (!percentfield)
-            percentfield = buf = gwy_data_field_duplicate(args->field);
-        pdata = gwy_data_field_get_data(percentfield);
-        p[0] = 100.0*lower_p;
-        p[1] = 100.0*(1.0 - upper_p);
-        gwy_math_percentiles(percentfield->xres*percentfield->yres, pdata,
-                             GWY_PERCENTILE_INTERPOLATION_LINEAR, 2, p, range);
-        GWY_OBJECT_UNREF(buf);
+            percentfield = buf = gwy_field_copy(args->field);
+        gdouble *pdata = gwy_field_get_data(percentfield);
+        gdouble p[2] = { 100.0*lower_p, 100.0*(1.0 - upper_p) };
+        gwy_math_percentiles(pdata, percentfield->xres*percentfield->yres,
+                             GWY_PERCENTILE_INTERPOLATION_LINEAR, p, 2,
+                             range);
+        g_clear_object(&buf);
     }
     else {
         g_return_if_reached();
     }
-    gwy_data_field_copy(args->field, args->result, FALSE);
-    gwy_data_field_clamp(args->result, range[0], range[1]);
+    gwy_field_copy_data(args->field, args->result);
+    /* Weird rounding errors can make upper smaller than lower. */
+    GWY_ORDER(gdouble, range[0], range[1]);
+    gwy_field_clamp(args->result, range[0], range[1]);
 }
 
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
