@@ -1,5 +1,5 @@
 /*
- *  $Id: tilt.c 26399 2024-06-20 10:09:07Z yeti-dn $
+ *  $Id: tilt.c 28821 2025-11-06 16:00:41Z yeti-dn $
  *  Copyright (C) 2008-2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,15 +16,11 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
+#include <gwy.h>
 #include <gtk/gtk.h>
-#include <libprocess/level.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
@@ -38,8 +34,8 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *result;
+    GwyField *field;
+    GwyField *result;
     /* Cached input image properties. */
     gboolean units_equal;
 } ModuleArgs;
@@ -48,16 +44,15 @@ typedef struct {
     ModuleArgs *args;
     GtkWidget *dialog;
     GwyParamTable *table;
-    GwyContainer *data;
 } ModuleGUI;
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             tilt                (GwyContainer *data,
-                                             GwyRunType run);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static void             execute             (ModuleArgs *args);
 static GwyDialogOutcome run_gui             (ModuleArgs *args,
-                                             GwyContainer *data,
+                                             GwyFile *data,
                                              gint id);
 static void             param_changed       (ModuleGUI *gui,
                                              gint id);
@@ -79,11 +74,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("tilt",
-                              (GwyProcessFunc)&tilt,
+                              module_main,
                               N_("/_Basic Operations/_Tilt..."),
-                              GWY_STOCK_TILT,
+                              GWY_ICON_TILT,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Tilt by specified amount"));
 
     return TRUE;
@@ -107,25 +102,24 @@ define_module_params(void)
 }
 
 static void
-tilt(GwyContainer *data, GwyRunType run)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     ModuleArgs args;
     GQuark quark;
     gint id;
 
-    g_return_if_fail(run & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_DATA_FIELD_KEY, &quark,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_FIELD_KEY, &quark,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(args.field);
-    args.result = gwy_data_field_duplicate(args.field);
-    args.units_equal = gwy_si_unit_equal(gwy_data_field_get_si_unit_z(args.field),
-                                         gwy_data_field_get_si_unit_xy(args.field));
+    args.result = gwy_field_copy(args.field);
+    args.units_equal = gwy_unit_equal(gwy_field_get_unit_z(args.field), gwy_field_get_unit_xy(args.field));
 
     args.params = gwy_params_new_from_settings(define_module_params());
-    if (run == GWY_RUN_INTERACTIVE) {
+    if (mode == GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, id);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -134,9 +128,9 @@ tilt(GwyContainer *data, GwyRunType run)
     if (outcome != GWY_DIALOG_HAVE_RESULT)
         execute(&args);
 
-    gwy_app_undo_qcheckpointv(data, 1, &quark);
-    gwy_container_set_object(data, quark, args.result);
-    gwy_app_channel_log_add_proc(data, id, id);
+    gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &quark);
+    gwy_file_set_image(data, id, args.result);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, id);
 
 end:
     g_object_unref(args.result);
@@ -144,37 +138,31 @@ end:
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     static const gint slope_params[] = { PARAM_DX, PARAM_DY };
-    GwyDialogOutcome outcome;
     ModuleGUI gui;
     GwyDialog *dialog;
     GwyParamTable *table;
     GtkWidget *hbox, *dataview;
-    GwySIUnit *unit;
+    GwyUnit *unit;
     gchar *unitstr;
     gint i;
 
     gui.args = args;
-    gui.data = gwy_container_new();
-    gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(0), args->result);
-    gwy_app_sync_data_items(data, gui.data, id, 0, FALSE,
-                            GWY_DATA_ITEM_PALETTE,
-                            GWY_DATA_ITEM_RANGE,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
 
-    unit = gwy_si_unit_new(NULL);
-    gwy_si_unit_divide(gwy_data_field_get_si_unit_z(args->field), gwy_data_field_get_si_unit_xy(args->field), unit);
-    unitstr = gwy_si_unit_get_string(unit, GWY_SI_UNIT_FORMAT_VFMARKUP);
+    unit = gwy_unit_new(NULL);
+    gwy_unit_divide(gwy_field_get_unit_z(args->field), gwy_field_get_unit_xy(args->field), unit);
+    unitstr = gwy_unit_get_string(unit, GWY_UNIT_FORMAT_VFMARKUP);
     g_object_unref(unit);
 
     gui.dialog = gwy_dialog_new(_("Tilt"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
+    dataview = gwy_create_preview(args->result, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_RANGE | GWY_FILE_ITEM_REAL_SQUARE);
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), FALSE);
 
     table = gui.table = gwy_param_table_new(args->params);
@@ -192,11 +180,11 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
         gwy_param_table_append_slider(table, PARAM_THETA);
         gwy_param_table_slider_set_mapping(table, PARAM_THETA, GWY_SCALE_MAPPING_SQRT);
         gwy_param_table_slider_restrict_range(table, PARAM_THETA, 0.0, atan(G_SQRT2*100.0));
-        gwy_param_table_slider_set_steps(table, PARAM_THETA, 0.01 * G_PI/180.0, 1.0 * G_PI/180.0);
+        gwy_param_table_slider_set_steps(table, PARAM_THETA, gwy_deg2rad(0.01), gwy_deg2rad(1.0));
         gwy_param_table_slider_set_digits(table, PARAM_THETA, 4);
     }
     gwy_param_table_append_slider(table, PARAM_PHI);
-    gwy_param_table_slider_set_steps(table, PARAM_PHI, 0.01 * G_PI/180.0, 1.0 * G_PI/180.0);
+    gwy_param_table_slider_set_steps(table, PARAM_PHI, gwy_deg2rad(0.01), gwy_deg2rad(1.0));
     gwy_param_table_slider_set_digits(table, PARAM_PHI, 4);
 
     gtk_box_pack_start(GTK_BOX(hbox), gwy_param_table_widget(table), TRUE, TRUE, 0);
@@ -205,11 +193,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     g_signal_connect_swapped(table, "param-changed", G_CALLBACK(param_changed), &gui);
     gwy_dialog_set_preview_func(dialog, GWY_PREVIEW_IMMEDIATE, preview, &gui, NULL);
 
-    outcome = gwy_dialog_run(dialog);
-
-    g_object_unref(gui.data);
-
-    return outcome;
+    return gwy_dialog_run(dialog);
 }
 
 static void
@@ -245,24 +229,24 @@ preview(gpointer user_data)
     ModuleGUI *gui = (ModuleGUI*)user_data;
 
     execute(gui->args);
-    gwy_data_field_data_changed(gui->args->result);
+    gwy_field_data_changed(gui->args->result);
     gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
 }
 
 static void
 execute(ModuleArgs *args)
 {
-    GwyDataField *field = args->field, *result = args->result;
+    GwyField *field = args->field, *result = args->result;
     GwyParams *params = args->params;
     gdouble bx, by, c;
 
     /* Use negative values since the module says ‘Tilt’, not ‘Remove tilt’. */
-    bx = -gwy_params_get_double(params, PARAM_DX) * gwy_data_field_get_dx(field);
-    by = -gwy_params_get_double(params, PARAM_DY) * gwy_data_field_get_dy(field);
-    c = -0.5*(bx*gwy_data_field_get_xres(field) + by*gwy_data_field_get_yres(field));
+    bx = -gwy_params_get_double(params, PARAM_DX) * gwy_field_get_dx(field);
+    by = -gwy_params_get_double(params, PARAM_DY) * gwy_field_get_dy(field);
+    c = -0.5*(bx*gwy_field_get_xres(field) + by*gwy_field_get_yres(field));
 
-    gwy_data_field_assign(result, field);
-    gwy_data_field_plane_level(result, c, bx, by);
+    gwy_field_assign(result, field);
+    gwy_field_plane_level(result, c, bx, by);
 }
 
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
