@@ -1,5 +1,5 @@
 /*
- *  $Id: tip_blind.c 24857 2022-07-14 11:08:49Z yeti-dn $
+ *  $Id: tip_blind.c 28860 2025-11-14 12:59:38Z yeti-dn $
  *  Copyright (C) 2004-2022 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,18 +16,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/stats.h>
-#include <libprocess/tip.h>
-#include <libgwydgets/gwydataview.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES GWY_RUN_INTERACTIVE
@@ -55,12 +47,12 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *tip;
+    GwyField *field;
+    GwyField *tip;
     GwyGraphModel *gmodel;
     /* Stripe results */
     guint nstripes;
-    GwyDataField **stripetips;
+    GwyField **stripetips;
     gboolean *goodtips;
 } ModuleArgs;
 
@@ -69,28 +61,27 @@ typedef struct {
     GtkWidget *dialog;
     GtkWidget *dataview;
     GwyParamTable *table;
-    GwyContainer *data;
 } ModuleGUI;
 
-typedef GwyDataField* (*TipFunc)(GwyDataField *tip,
-                                 GwyDataField *surface,
-                                 gdouble threshold,
-                                 gboolean use_edges,
-                                 gint *count,
-                                 GwySetFractionFunc set_fraction,
-                                 GwySetMessageFunc set_message);
+typedef GwyField* (*TipFunc)(GwyField *tip,
+                             GwyField *surface,
+                             gdouble threshold,
+                             gboolean use_edges,
+                             gint *count,
+                             GwySetFractionFunc set_fraction,
+                             GwySetMessageFunc set_message);
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             tip_blind           (GwyContainer *data,
-                                             GwyRunType runtype);
-static gint             create_output_field (GwyDataField *field,
-                                             GwyContainer *data,
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
+static gint             create_output_field (GwyField *field,
+                                             GwyFile *data,
                                              gint id,
                                              gint i,
                                              gint nstripes);
 static GwyDialogOutcome run_gui             (ModuleArgs *args,
-                                             GwyContainer *data,
+                                             GwyFile *data,
                                              gint id);
 static void             param_changed       (ModuleGUI *gui,
                                              gint id);
@@ -101,13 +92,13 @@ static void             dialog_response     (ModuleGUI *gui,
 static gboolean         resize_stripe_tips  (ModuleArgs *args,
                                              guint n);
 static void             clear_tip_images    (ModuleArgs *args);
-static gboolean         source_image_filter (GwyContainer *data,
+static gboolean         source_image_filter (GwyFile *data,
                                              gint id,
                                              gpointer user_data);
 static gboolean         execute             (ModuleArgs *args,
                                              gboolean full,
                                              GtkWindow *wait_window);
-static void             tip_curvatures      (GwyDataField *tipfield,
+static void             tip_curvatures      (GwyField *tipfield,
                                              gdouble *pc1,
                                              gdouble *pc2);
 static void             size_plot           (ModuleArgs *args);
@@ -130,11 +121,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("tip_blind",
-                              (GwyProcessFunc)&tip_blind,
+                              module_main,
                               N_("/SPM M_odes/_Tip/_Blind Estimation..."),
-                              GWY_STOCK_TIP_ESTIMATION,
+                              GWY_ICON_TIP_ESTIMATION,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Blind tip estimation"));
 
     return TRUE;
@@ -166,7 +157,7 @@ define_module_params(void)
 }
 
 static void
-tip_blind(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome;
     GwyParams *params;
@@ -174,16 +165,16 @@ tip_blind(GwyContainer *data, GwyRunType runtype)
     GwyAppDataId dataid;
     gint id, i, nstripes;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     GWY_APP_CONTAINER_ID, &dataid.datano,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_FIELD_ID, &id,
+                                 GWY_APP_CONTAINER_ID, &dataid.datano,
+                                 0);
     g_return_if_fail(args.field);
 
     dataid.id = id;
-    args.tip = gwy_data_field_new(3, 3, 1.0, 1.0, TRUE);
+    args.tip = gwy_field_new(3, 3, 1.0, 1.0, TRUE);
     args.gmodel = gwy_graph_model_new();
     args.params = params = gwy_params_new_from_settings(define_module_params());
     sanitise_params(&args, dataid);
@@ -204,8 +195,10 @@ tip_blind(GwyContainer *data, GwyRunType runtype)
             }
         }
         if (gwy_params_get_boolean(params, PARAM_PLOT_SIZE_GRAPH)) {
-            if (gwy_graph_model_get_n_curves(args.gmodel))
-                gwy_app_data_browser_add_graph_model(args.gmodel, data, TRUE);
+            if (gwy_graph_model_get_n_curves(args.gmodel)) {
+                gint newid = gwy_file_add_graph(data, args.gmodel);
+                gwy_file_set_visible(data, GWY_FILE_GRAPH, newid, TRUE);
+            }
         }
     }
     else
@@ -219,39 +212,35 @@ end:
 }
 
 static gint
-create_output_field(GwyDataField *field, GwyContainer *data, gint id, gint i, gint nstripes)
+create_output_field(GwyField *field, GwyFile *data, gint id, gint i, gint nstripes)
 {
-    gint newid = gwy_app_data_browser_add_data_field(field, data, TRUE);
-    gwy_app_sync_data_items(data, data, id, newid, FALSE, GWY_DATA_ITEM_GRADIENT, 0);
+    gint newid = gwy_file_add_image(data, field);
+    gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+    gwy_file_sync_items(data, GWY_FILE_IMAGE, id,
+                        data, GWY_FILE_IMAGE, newid,
+                        GWY_FILE_ITEM_PALETTE, FALSE);
     if (nstripes > 0) {
-        gchar *title = g_strdup_printf("%s %u/%u", _("Estimated tip"), i+1, nstripes);
-        gwy_container_set_string(data, gwy_app_get_data_title_key_for_id(newid), title);
+        gwy_file_pass_title(data, GWY_FILE_IMAGE, newid,
+                            g_strdup_printf("%s %u/%u", _("Estimated tip"), i+1, nstripes));
     }
     else
-        gwy_app_set_data_field_title(data, newid, _("Estimated tip"));
-    gwy_app_channel_log_add_proc(data, -1, newid);
+        gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _("Estimated tip"), TRUE);
+    gwy_log_add(data, GWY_FILE_IMAGE, -1, newid);
     return newid;
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
-    GtkWidget *hbox, *vbox, *align, *graph;
+    GtkWidget *hbox, *vbox, *graph;
     GwyDialog *dialog;
     GwyParamTable *table;
     ModuleGUI gui;
-    GwyDialogOutcome outcome;
-    GwySIValueFormat *vf;
+    GwyValueFormat *vf;
     gdouble rms;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
-    gui.data = gwy_container_new();
-    gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(0), args->tip);
-    gwy_app_sync_data_items(data, gui.data, id, 0, FALSE,
-                            GWY_DATA_ITEM_GRADIENT,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
 
     gui.dialog = gwy_dialog_new(_("Blind Tip Estimation"));
     dialog = GWY_DIALOG(gui.dialog);
@@ -261,25 +250,26 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_dialog_add_buttons(dialog, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
     gtk_dialog_set_response_sensitive(GTK_DIALOG(gui.dialog), GTK_RESPONSE_OK, FALSE);
 
-    hbox = gwy_hbox_new(8);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 8);
     gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
     gwy_dialog_add_content(GWY_DIALOG(dialog), hbox, FALSE, FALSE, 0);
 
-    vbox = gwy_vbox_new(0);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
     gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);
 
-    align = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-    gtk_widget_set_size_request(align, PREVIEW_SMALL_SIZE, -1);
-    gtk_box_pack_start(GTK_BOX(vbox), align, FALSE, FALSE, 0);
-
-    gui.dataview = gwy_create_preview(gui.data, 0, PREVIEW_SMALL_SIZE, FALSE);
-    gtk_container_add(GTK_CONTAINER(align), GTK_WIDGET(gui.dataview));
+    gui.dataview = gwy_create_preview(args->tip, NULL, PREVIEW_SMALL_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(gui.dataview), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_REAL_SQUARE);
+    gtk_widget_set_size_request(gui.dataview, PREVIEW_SMALL_SIZE, -1);
+    gtk_widget_set_halign(gui.dataview, GTK_ALIGN_START);
+    gtk_widget_set_valign(gui.dataview, GTK_ALIGN_START);
+    gtk_box_pack_start(GTK_BOX(vbox), gui.dataview, FALSE, FALSE, 0);
 
     graph = gwy_graph_new(args->gmodel);
     g_object_set(args->gmodel, "label-visible", FALSE, NULL);
     gwy_graph_enable_user_input(GWY_GRAPH(graph), FALSE);
-    gwy_axis_set_visible(gwy_graph_get_axis(GWY_GRAPH(graph), GTK_POS_LEFT), FALSE);
-    gwy_axis_set_visible(gwy_graph_get_axis(GWY_GRAPH(graph), GTK_POS_BOTTOM), FALSE);
+    gwy_graph_set_axis_visible(GWY_GRAPH(graph), GTK_POS_LEFT, FALSE);
+    gwy_graph_set_axis_visible(GWY_GRAPH(graph), GTK_POS_BOTTOM, FALSE);
     gtk_widget_set_size_request(graph, -1, 120);
     gtk_box_pack_start(GTK_BOX(vbox), graph, TRUE, TRUE, 0);
 
@@ -297,8 +287,8 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_param_table_append_header(table, -1, _("Options"));
     gwy_param_table_append_slider(table, PARAM_THRESHOLD);
     gwy_param_table_slider_set_mapping(table, PARAM_THRESHOLD, GWY_SCALE_MAPPING_LOG);
-    rms = gwy_data_field_get_rms(args->field);
-    vf = gwy_si_unit_get_format(gwy_data_field_get_si_unit_z(args->field), GWY_SI_UNIT_FORMAT_VFMARKUP, 1e-3*rms, NULL);
+    rms = gwy_field_get_rms(args->field);
+    vf = gwy_unit_get_format(gwy_field_get_unit_z(args->field), GWY_UNIT_FORMAT_VFMARKUP, 1e-3*rms, NULL);
     vf->precision++;
     gwy_param_table_slider_set_factor(table, PARAM_THRESHOLD, 1.0/vf->magnitude);
     gwy_param_table_set_unitstr(table, PARAM_THRESHOLD, vf->units);
@@ -318,11 +308,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     g_signal_connect_swapped(gui.table, "param-changed", G_CALLBACK(param_changed), &gui);
     g_signal_connect_swapped(dialog, "response", G_CALLBACK(dialog_response), &gui);
 
-    outcome = gwy_dialog_run(dialog);
-
-    g_object_unref(gui.data);
-
-    return outcome;
+    return gwy_dialog_run(dialog);
 }
 
 static void
@@ -358,15 +344,15 @@ param_changed(ModuleGUI *gui, gint id)
 }
 
 static gboolean
-source_image_filter(GwyContainer *data, gint id, gpointer user_data)
+source_image_filter(GwyFile *data, gint id, gpointer user_data)
 {
-    GwyDataField *field = (GwyDataField*)user_data;
-    GwyDataField *source = gwy_container_get_object(data, gwy_app_get_data_key_for_id(id));
+    GwyField *field = (GwyField*)user_data;
+    GwyField *source = gwy_file_get_image(data, id);
 
-    return !gwy_data_field_check_compatibility(source, field,
-                                               GWY_DATA_COMPATIBILITY_MEASURE
-                                               | GWY_DATA_COMPATIBILITY_LATERAL
-                                               | GWY_DATA_COMPATIBILITY_VALUE);
+    if (!source)
+        return FALSE;
+    return !gwy_field_is_incompatible(source, field,
+                                      GWY_DATA_MISMATCH_MEASURE | GWY_DATA_MISMATCH_LATERAL | GWY_DATA_MISMATCH_VALUE);
 }
 
 static void
@@ -378,14 +364,15 @@ update_tip_preview(ModuleGUI *gui, gboolean must_set_it)
     GwyParams *params = args->params;
     gboolean split_to_stripes = gwy_params_get_boolean(params, PARAM_SPLIT_TO_STRIPES);
     gint preview_stripe = gwy_params_get_int(params, PARAM_PREVIEW_STRIPE)-1;
+    GwyDataView *dataview = GWY_DATA_VIEW(gui->dataview);
 
     if (split_to_stripes && preview_stripe < args->nstripes)
-        gwy_container_set_object(gui->data, gwy_app_get_data_key_for_id(0), args->stripetips[preview_stripe]);
+        gwy_data_view_set_field(dataview, args->stripetips[preview_stripe]);
     else if (must_set_it)
-        gwy_container_set_object(gui->data, gwy_app_get_data_key_for_id(0), args->tip);
+        gwy_data_view_set_field(dataview, args->tip);
     else
         return;
-    gwy_set_data_preview_size(GWY_DATA_VIEW(gui->dataview), PREVIEW_SMALL_SIZE);
+    gwy_set_data_preview_size(dataview, PREVIEW_SMALL_SIZE);
 }
 
 static void
@@ -406,7 +393,7 @@ dialog_response(ModuleGUI *gui, gint response)
     gwy_param_table_slider_restrict_range(gui->table, PARAM_PREVIEW_STRIPE,
                                           1, gwy_params_get_int(args->params, PARAM_NSTRIPES));
     update_tip_preview(gui, TRUE);
-    gwy_data_field_data_changed(gwy_container_get_object(gui->data, gwy_app_get_data_key_for_id(0)));
+    gwy_field_data_changed(gwy_data_view_get_field(GWY_DATA_VIEW(gui->dataview)));
     gtk_dialog_set_response_sensitive(GTK_DIALOG(gui->dialog), GTK_RESPONSE_OK, ok);
     /* Use the invalidate/have-data mechanism for our own means.  Invalidate will schedule a preview, but we have no
      * such thing so the dialog will remain in doesn't-have-data state until we run the tip estimation successfully. */
@@ -417,20 +404,20 @@ dialog_response(ModuleGUI *gui, gint response)
 }
 
 static gboolean
-prepare_tip_field(GwyDataField *tip, GwyDataField *surface, gint xres, gint yres)
+prepare_tip_field(GwyField *tip, GwyField *surface, gint xres, gint yres)
 {
     gboolean keep = TRUE;
 
     /* If user has changed the tip size, start from scratch. */
-    if (gwy_data_field_get_xres(tip) != xres || gwy_data_field_get_yres(tip) != yres) {
-        gwy_data_field_resample(tip, xres, yres, GWY_INTERPOLATION_NONE);
-        gwy_data_field_clear(tip);
+    if (gwy_field_get_xres(tip) != xres || gwy_field_get_yres(tip) != yres) {
+        gwy_field_resize(tip, xres, yres);
+        gwy_field_clear(tip);
         keep = FALSE;
     }
     /* Do this always to be on the safe side (for instance during the dialog init). */
-    gwy_data_field_set_xreal(tip, gwy_data_field_get_dx(surface)*gwy_data_field_get_xres(tip));
-    gwy_data_field_set_yreal(tip, gwy_data_field_get_dy(surface)*gwy_data_field_get_yres(tip));
-    gwy_data_field_copy_units(surface, tip);
+    gwy_field_set_xreal(tip, gwy_field_get_dx(surface)*gwy_field_get_xres(tip));
+    gwy_field_set_yreal(tip, gwy_field_get_dy(surface)*gwy_field_get_yres(tip));
+    gwy_field_copy_units(surface, tip);
     return keep;
 }
 
@@ -446,15 +433,15 @@ prepare_stripe_fields(ModuleArgs *args, gint nstripes, gboolean keep)
     /* Otherwise initialise all the stripe tips from the global one.  It iself could be reset or kept, but whatever is
      * the state, we want to copy it. */
     for (i = 0; i < nstripes; i++)
-        gwy_data_field_copy(args->tip, args->stripetips[i], FALSE);
+        gwy_field_copy_data(args->tip, args->stripetips[i]);
 }
 
 static gboolean
 execute(ModuleArgs *args, gboolean full, GtkWindow *wait_window)
 {
     GwyParams *params = args->params;
-    GwyDataField *surface = gwy_params_get_image(params, PARAM_SOURCE);
-    GwyDataField *tip = args->tip, *stripe;
+    GwyField *surface = gwy_params_get_image(params, PARAM_SOURCE);
+    GwyField *tip = args->tip, *stripe;
     gint txres = gwy_params_get_int(params, PARAM_XRES);
     gint tyres = gwy_params_get_int(params, PARAM_YRES);
     gdouble threshold = gwy_params_get_double(params, PARAM_THRESHOLD);
@@ -471,7 +458,7 @@ execute(ModuleArgs *args, gboolean full, GtkWindow *wait_window)
     prepare_stripe_fields(args, nstripes, keep);
 
     if (split_to_stripes) {
-        gint xres = gwy_data_field_get_xres(surface), yres = gwy_data_field_get_yres(surface);
+        gint xres = gwy_field_get_xres(surface), yres = gwy_field_get_yres(surface);
         gint anygood = 0, i;
         gchar *prefix = NULL;
 
@@ -491,7 +478,7 @@ execute(ModuleArgs *args, gboolean full, GtkWindow *wait_window)
 
             gwy_debug("[%u] (%u, %u) of %u", i, row, height, yres);
             count = -1;
-            stripe = gwy_data_field_area_extract(surface, 0, row, xres, height);
+            stripe = gwy_field_area_extract(surface, 0, row, xres, height);
             ok = !!tipfunc(args->stripetips[i], stripe, threshold, use_boundaries, &count,
                            gwy_app_wait_set_fraction, gwy_app_wait_set_message);
             g_object_unref(stripe);
@@ -526,10 +513,10 @@ clear_tip_images(ModuleArgs *args)
 {
     guint i;
 
-    gwy_data_field_clear(args->tip);
+    gwy_field_clear(args->tip);
     if (args->nstripes) {
         for (i = 0; i < args->nstripes; i++)
-            gwy_data_field_clear(args->stripetips[i]);
+            gwy_field_clear(args->stripetips[i]);
         gwy_clear(args->goodtips, args->nstripes);
     }
 }
@@ -537,25 +524,25 @@ clear_tip_images(ModuleArgs *args)
 static gboolean
 resize_stripe_tips(ModuleArgs *args, guint n)
 {
-    gint xres = gwy_data_field_get_xres(args->tip), yres = gwy_data_field_get_yres(args->tip);
+    gint xres = gwy_field_get_xres(args->tip), yres = gwy_field_get_yres(args->tip);
     guint i;
 
     if (args->nstripes == n) {
-        if (!args->nstripes || (gwy_data_field_get_xres(args->stripetips[0]) == xres
-                                && gwy_data_field_get_yres(args->stripetips[0]) == yres))
+        if (!args->nstripes || (gwy_field_get_xres(args->stripetips[0]) == xres
+                                && gwy_field_get_yres(args->stripetips[0]) == yres))
             return TRUE;
     }
 
     for (i = n; i < args->nstripes; i++)
         g_object_unref(args->stripetips[i]);
     if (n) {
-        args->stripetips = g_renew(GwyDataField*, args->stripetips, n);
+        args->stripetips = g_renew(GwyField*, args->stripetips, n);
         args->goodtips = g_renew(gboolean, args->goodtips, n);
         gwy_clear(args->goodtips, n);
         for (i = args->nstripes; i < n; i++)
-            args->stripetips[i] = gwy_data_field_new_alike(args->tip, TRUE);
+            args->stripetips[i] = gwy_field_new_alike(args->tip, TRUE);
         for (i = 0; i < MIN(n, args->nstripes); i++)
-            gwy_data_field_resample(args->stripetips[i], xres, yres, GWY_INTERPOLATION_NONE);
+            gwy_field_resize(args->stripetips[i], xres, yres);
     }
     else {
         GWY_FREE(args->stripetips);
@@ -566,11 +553,11 @@ resize_stripe_tips(ModuleArgs *args, guint n)
 }
 
 static void
-tip_curvatures(GwyDataField *tip, gdouble *pc1, gdouble *pc2)
+tip_curvatures(GwyField *tip, gdouble *pc1, gdouble *pc2)
 {
-    gint xres = gwy_data_field_get_xres(tip), yres = gwy_data_field_get_yres(tip);
-    gdouble dx = gwy_data_field_get_dx(tip), dy = gwy_data_field_get_dy(tip);
-    const gdouble *data = gwy_data_field_get_data_const(tip);
+    gint xres = gwy_field_get_xres(tip), yres = gwy_field_get_yres(tip);
+    gdouble dx = gwy_field_get_dx(tip), dy = gwy_field_get_dy(tip);
+    const gdouble *data = gwy_field_get_data_const(tip);
     gdouble R = 2 + 0.25*log(xres*yres);
     gdouble sx2 = 0.0, sy2 = 0.0, sx4 = 0.0, sx2y2 = 0.0, sy4 = 0.0;
     gdouble sz = 0.0, szx = 0.0, szy = 0.0, szx2 = 0.0, szxy = 0.0, szy2 = 0.0;
@@ -636,21 +623,21 @@ tip_curvatures(GwyDataField *tip, gdouble *pc1, gdouble *pc2)
 static void
 size_plot(ModuleArgs *args)
 {
-    GwyDataField *surface = gwy_params_get_image(args->params, PARAM_SOURCE);
-    gint tyres = gwy_data_field_get_yres(args->tip);
-    gint yres = gwy_data_field_get_yres(surface);
+    GwyField *surface = gwy_params_get_image(args->params, PARAM_SOURCE);
+    gint tyres = gwy_field_get_yres(args->tip);
+    gint yres = gwy_field_get_yres(surface);
     GwyGraphModel *gmodel = args->gmodel;
     GwyGraphCurveModel *gcmodel;
     gdouble *xdata, *ydata;
     gint i, ngood = 0, nstripes = args->nstripes;
-    gdouble dy = gwy_data_field_get_dy(surface);
+    gdouble dy = gwy_field_get_dy(surface);
 
     g_object_set(gmodel,
                  "title", _("Tip radius evolution"),
                  "axis-label-bottom", "y",
                  "axis-label-left", "r",
                  NULL);
-    gwy_graph_model_set_units_from_data_field(gmodel, args->field, 1, 0, 1, 0);
+    gwy_graph_model_set_units_from_field(gmodel, args->field, 1, 0, 1, 0);
 
     xdata = g_new(gdouble, nstripes);
     ydata = g_new(gdouble, nstripes);
