@@ -1,5 +1,5 @@
 /*
- *  $Id: tipops.c 24682 2022-03-15 17:33:44Z yeti-dn $
+ *  $Id: tipops.c 28821 2025-11-06 16:00:41Z yeti-dn $
  *  Copyright (C) 2004-2022 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,15 +16,9 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/tip.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 
 #define RUN_MODES GWY_RUN_INTERACTIVE
 
@@ -42,8 +36,8 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *result;
+    GwyField *field;
+    GwyField *result;
 } ModuleArgs;
 
 typedef struct {
@@ -54,9 +48,8 @@ typedef struct {
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             tipops              (GwyContainer *data,
-                                             GwyRunType runtype,
-                                             const gchar *name);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static gboolean         execute             (ModuleArgs *args,
                                              TipOperation op,
                                              GtkWindow *wait_window);
@@ -64,7 +57,7 @@ static GwyDialogOutcome run_gui             (ModuleArgs *args,
                                              TipOperation op);
 static void             param_changed       (ModuleGUI *gui,
                                              gint id);
-static gboolean         tip_image_filter    (GwyContainer *data,
+static gboolean         tip_image_filter    (GwyFile *data,
                                              gint id,
                                              gpointer user_data);
 
@@ -84,25 +77,25 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("tip_dilation",
-                              &tipops,
+                              module_main,
                               N_("/SPM M_odes/_Tip/_Dilation..."),
-                              GWY_STOCK_TIP_DILATION,
+                              GWY_ICON_TIP_DILATION,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Surface dilation by defined tip"));
     gwy_process_func_register("tip_reconstruction",
-                              &tipops,
+                              module_main,
                               N_("/SPM M_odes/_Tip/_Surface Reconstruction..."),
-                              GWY_STOCK_TIP_EROSION,
+                              GWY_ICON_TIP_EROSION,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Surface reconstruction by defined tip"));
     gwy_process_func_register("tip_map",
-                              &tipops,
+                              module_main,
                               N_("/SPM M_odes/_Tip/_Certainty Map..."),
-                              GWY_STOCK_TIP_MAP,
+                              GWY_ICON_TIP_MAP,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Tip certainty map"));
 
     return TRUE;
@@ -123,9 +116,7 @@ define_module_params(void)
 }
 
 static void
-tipops(GwyContainer *data,
-       GwyRunType runtype,
-       const gchar *name)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     static const GwyEnum ops[] = {
         { "tip_dilation",       DILATION,      },
@@ -136,22 +127,23 @@ tipops(GwyContainer *data,
         { N_("Dilated data"),           DILATION,      },
         { N_("Surface reconstruction"), EROSION,       },
     };
+    const gchar *name = gwy_process_func_current();
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     TipOperation op;
     ModuleArgs args;
     gint id, newid;
     GQuark quark;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_clear(&args, 1);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_clear1(args);
     if ((op = gwy_string_to_enum(name, ops, G_N_ELEMENTS(ops))) == (TipOperation)-1) {
         g_warning("tipops does not provide function `%s'", name);
         return;
     }
 
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(args.field);
 
     args.params = gwy_params_new_from_settings(define_module_params());
@@ -159,27 +151,28 @@ tipops(GwyContainer *data,
     gwy_params_save_to_settings(args.params);
     if (outcome == GWY_DIALOG_CANCEL)
         goto end;
-    if (!execute(&args, op, gwy_app_find_window_for_channel(data, id)))
+    if (!execute(&args, op, gwy_data_browser_get_window_for_data(data, GWY_FILE_IMAGE, id)))
         goto end;
 
     if (op == DILATION || op == EROSION) {
-        newid = gwy_app_data_browser_add_data_field(args.result, data, TRUE);
-        gwy_app_sync_data_items(data, data, id, newid, FALSE,
-                                GWY_DATA_ITEM_GRADIENT,
-                                GWY_DATA_ITEM_MASK_COLOR,
-                                0);
-        gwy_app_set_data_field_title(data, newid, gwy_enum_to_string(op, data_titles, G_N_ELEMENTS(data_titles)));
-        gwy_app_channel_log_add_proc(data, id, newid);
+        newid = gwy_file_add_image(data, args.result);
+        gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+        gwy_file_sync_items(data, GWY_FILE_IMAGE, id,
+                            data, GWY_FILE_IMAGE, newid,
+                            GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_MASK_COLOR, FALSE);
+        gwy_file_set_title(data, GWY_FILE_IMAGE, newid,
+                           gwy_enum_to_string(op, data_titles, G_N_ELEMENTS(data_titles)), TRUE);
+        gwy_log_add(data, GWY_FILE_IMAGE, id, newid);
     }
     else {
-        quark = gwy_app_get_mask_key_for_id(id);
-        gwy_app_undo_qcheckpointv(data, 1, &quark);
-        gwy_container_set_object(data, quark, args.result);
-        gwy_app_channel_log_add_proc(data, id, id);
+        quark = gwy_file_key_image_mask(id);
+        gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &quark);
+        gwy_file_set_image(data, id, args.result);
+        gwy_log_add(data, GWY_FILE_IMAGE, id, id);
     }
 
 end:
-    GWY_OBJECT_UNREF(args.result);
+    g_clear_object(&args.result);
     g_object_unref(args.params);
 }
 
@@ -213,17 +206,16 @@ run_gui(ModuleArgs *args, TipOperation op)
 }
 
 static gboolean
-tip_image_filter(GwyContainer *data, gint id, gpointer user_data)
+tip_image_filter(GwyFile *data, gint id, gpointer user_data)
 {
-    GwyDataField *field = (GwyDataField*)user_data;
-    GwyDataField *tip = gwy_container_get_object(data, gwy_app_get_data_key_for_id(id));
+    GwyField *field = (GwyField*)user_data;
+    GwyField *tip = gwy_file_get_image(data, id);
 
-    if (gwy_data_field_get_xreal(tip) > gwy_data_field_get_xreal(field)/4
-        || gwy_data_field_get_yreal(tip) > gwy_data_field_get_yreal(field)/4)
+    if (gwy_field_get_xreal(tip) > gwy_field_get_xreal(field)/4
+        || gwy_field_get_yreal(tip) > gwy_field_get_yreal(field)/4)
         return FALSE;
 
-    return !gwy_data_field_check_compatibility(tip, field,
-                                               GWY_DATA_COMPATIBILITY_LATERAL | GWY_DATA_COMPATIBILITY_VALUE);
+    return !gwy_field_is_incompatible(tip, field, GWY_DATA_MISMATCH_LATERAL | GWY_DATA_MISMATCH_VALUE);
 }
 
 static void
@@ -236,20 +228,20 @@ param_changed(ModuleGUI *gui, gint id)
         if (gwy_params_data_id_is_none(params, PARAM_TIP))
             gtk_dialog_set_response_sensitive(GTK_DIALOG(gui->dialog), GTK_RESPONSE_OK, FALSE);
         else {
-            GwyDataField *tip = gwy_params_get_image(params, PARAM_TIP), *field = args->field;
+            GwyField *tip = gwy_params_get_image(params, PARAM_TIP), *field = args->field;
 
-            if (!gwy_data_field_check_compatibility(tip, field, GWY_DATA_COMPATIBILITY_MEASURE))
+            if (!gwy_field_is_incompatible(tip, field, GWY_DATA_MISMATCH_MEASURE))
                 gwy_param_table_set_label(gui->table, MESSAGE_RESAMPLING, NULL);
             else {
-                gint xres = GWY_ROUND(gwy_data_field_get_xreal(tip)/gwy_data_field_get_dx(field));
-                gint yres = GWY_ROUND(gwy_data_field_get_yreal(tip)/gwy_data_field_get_dy(field));
+                gint xres = GWY_ROUND(gwy_field_get_xreal(tip)/gwy_field_get_dx(field));
+                gint yres = GWY_ROUND(gwy_field_get_yreal(tip)/gwy_field_get_dy(field));
                 gchar *s;
 
                 xres = MAX(xres, 1);
                 yres = MAX(yres, 1);
                 s = g_strdup_printf(_("Tip pixel size does not match data.\n"
                                       "It will be resampled from %d×%d to %d×%d."),
-                                    gwy_data_field_get_xres(tip), gwy_data_field_get_yres(tip), xres, yres);
+                                    gwy_field_get_xres(tip), gwy_field_get_yres(tip), xres, yres);
                 gwy_param_table_set_label(gui->table, MESSAGE_RESAMPLING, s);
                 g_free(s);
             }
@@ -260,11 +252,11 @@ param_changed(ModuleGUI *gui, gint id)
 static gboolean
 execute(ModuleArgs *args, TipOperation op, GtkWindow *wait_window)
 {
-    GwyDataField *field = args->field, *result, *tip = gwy_params_get_image(args->params, PARAM_TIP);
+    GwyField *field = args->field, *result, *tip = gwy_params_get_image(args->params, PARAM_TIP);
     gboolean ok = FALSE;
 
     g_return_val_if_fail(tip, FALSE);
-    result = args->result = gwy_data_field_new_alike(field, FALSE);
+    result = args->result = gwy_field_new_alike(field, FALSE);
 
     gwy_app_wait_start(wait_window, _("Initializing..."));
     if (op == DILATION)
