@@ -1,5 +1,5 @@
 /*
- *  $Id: trimmed-mean.c 23666 2021-05-10 21:56:10Z yeti-dn $
+ *  $Id: trimmed-mean.c 28821 2025-11-06 16:00:41Z yeti-dn $
  *  Copyright (C) 2019-2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,17 +16,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/datafield.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/elliptic.h>
-#include <libprocess/filters.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
@@ -49,8 +42,8 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *result;
+    GwyField *field;
+    GwyField *result;
 } ModuleArgs;
 
 typedef struct {
@@ -61,8 +54,8 @@ typedef struct {
 
 static gboolean         module_register           (void);
 static GwyParamDef*     define_module_params      (void);
-static void             trimmed_mean              (GwyContainer *data,
-                                                   GwyRunType runtype);
+static void             module_main               (GwyFile *data,
+                                                   GwyRunModeFlags mode);
 static gboolean         execute                   (ModuleArgs *args,
                                                    GtkWindow *wait_window);
 static GwyDialogOutcome run_gui                   (ModuleArgs *args);
@@ -89,11 +82,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("trimmed_mean",
-                              (GwyProcessFunc)&trimmed_mean,
+                              module_main,
                               N_("/_Level/_Trimmed Mean..."),
                               NULL,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Trimmed mean leveling and filter"));
 
     return TRUE;
@@ -120,48 +113,49 @@ define_module_params(void)
 }
 
 static void
-trimmed_mean(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     ModuleArgs args;
     gint id, newid;
     GQuark quark;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_DATA_FIELD_KEY, &quark,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_FIELD_KEY, &quark,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(args.field && quark);
 
-    args.result = gwy_data_field_new_alike(args.field, FALSE);
+    args.result = gwy_field_new_alike(args.field, FALSE);
     args.params = gwy_params_new_from_settings(define_module_params());
     sanitise_params(&args);
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
             goto end;
     }
-    if (!execute(&args, gwy_app_find_window_for_channel(data, id)))
+    if (!execute(&args, gwy_data_browser_get_window_for_data(data, GWY_FILE_IMAGE, id)))
         goto end;
 
-    gwy_app_undo_qcheckpointv(data, 1, &quark);
-    gwy_data_field_subtract_fields(args.field, args.field, args.result);
-    gwy_data_field_data_changed(args.field);
-    gwy_app_channel_log_add_proc(data, id, id);
+    gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &quark);
+    gwy_field_subtract_fields(args.field, args.field, args.result);
+    gwy_field_data_changed(args.field);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, id);
 
     if (!gwy_params_get_boolean(args.params, PARAM_DO_EXTRACT))
         goto end;
 
-    newid = gwy_app_data_browser_add_data_field(args.result, data, TRUE);
-    gwy_app_sync_data_items(data, data, id, newid, FALSE,
-                            GWY_DATA_ITEM_GRADIENT,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
-    gwy_app_set_data_field_title(data, newid, _("Background"));
-    gwy_app_channel_log_add(data, id, newid, NULL, NULL);
+    newid = gwy_file_add_image(data, args.result);
+
+    gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+    gwy_file_sync_items(data, GWY_FILE_IMAGE, id,
+                        data, GWY_FILE_IMAGE, newid,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_REAL_SQUARE, FALSE);
+    gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _("Background"), TRUE);
+    gwy_log_add_full(data, GWY_FILE_IMAGE, id, newid, NULL, NULL);
 
 end:
     g_object_unref(args.result);
@@ -171,7 +165,7 @@ end:
 static gboolean
 execute(ModuleArgs *args, GtkWindow *wait_window)
 {
-    GwyDataField *field = args->field, *result = args->result, *kernel;
+    GwyField *field = args->field, *result = args->result, *kernel;
     gint size = gwy_params_get_int(args->params, PARAM_SIZE);
     gint kres = 2*size + 1;
     gint xres, yres, nlowest, nhighest;
@@ -180,13 +174,13 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
     gwy_app_wait_start(wait_window, _("Filtering..."));
 
     calculate_nlowest_nhighest(args->params, &nlowest, &nhighest);
-    kernel = gwy_data_field_new(kres, kres, 1.0, 1.0, TRUE);
-    gwy_data_field_elliptic_area_fill(kernel, 0, 0, kres, kres, 1.0);
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
-    gwy_data_field_copy(field, result, FALSE);
-    ok = gwy_data_field_area_filter_trimmed_mean(result, kernel, 0, 0, xres, yres, nlowest, nhighest,
-                                                 gwy_app_wait_set_fraction);
+    kernel = gwy_field_new(kres, kres, 1.0, 1.0, TRUE);
+    gwy_field_elliptic_area_fill(kernel, 0, 0, kres, kres, 1.0);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
+    gwy_field_copy_data(field, result);
+    ok = gwy_field_area_filter_trimmed_mean(result, kernel, 0, 0, xres, yres, nlowest, nhighest,
+                                            gwy_app_wait_set_fraction);
     g_object_unref(kernel);
 
     gwy_app_wait_finish();
@@ -201,7 +195,7 @@ run_gui(ModuleArgs *args)
     GwyParamTable *table;
     ModuleGUI gui;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
 
     gui.dialog = gwy_dialog_new(_("Trimmed Mean"));
@@ -245,7 +239,7 @@ param_changed(ModuleGUI *gui, gint id)
     GwyParamTable *table = gui->table;
     gint size = gwy_params_get_int(args->params, PARAM_SIZE);
     gint nlowest, nhighest, kres = 2*size + 1;
-    gint n = gwy_data_field_get_elliptic_area_size(kres, kres);
+    gint n = gwy_field_get_elliptic_area_size(kres, kres);
     gboolean trim_symm = gwy_params_get_boolean(params, PARAM_TRIM_SYMM);
     gdouble fraction_lowest = gwy_params_get_double(params, PARAM_FRACTION_LOWEST);
     gdouble fraction_highest = gwy_params_get_double(params, PARAM_FRACTION_HIGHEST);
@@ -306,7 +300,7 @@ calculate_nlowest_nhighest(GwyParams *params, gint *nlowest, gint *nhighest)
     gdouble fraction_highest = gwy_params_get_double(params, PARAM_FRACTION_HIGHEST);
     gint size = gwy_params_get_int(params, PARAM_SIZE);
     gint kres = 2*size + 1;
-    gint n = gwy_data_field_get_elliptic_area_size(kres, kres);
+    gint n = gwy_field_get_elliptic_area_size(kres, kres);
 
     *nlowest = (gint)floor(fraction_lowest*n + 1e-12);
     *nhighest = (gint)floor(fraction_highest*n + 1e-12);
