@@ -1,5 +1,5 @@
 /*
- *  $Id: volumize_layers.c 26179 2024-02-14 10:22:24Z yeti-dn $
+ *  $Id: volumize_layers.c 28808 2025-11-05 18:26:20Z yeti-dn $
  *  Copyright (C) 2015-2022 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,21 +16,9 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/stats.h>
-#include <libprocess/brick.h>
-#include <libprocess/filters.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwydgets/gwydataview.h>
-#include <libgwydgets/gwylayer-basic.h>
-#include <libgwydgets/gwylayer-mask.h>
-#include <libgwydgets/gwycombobox.h>
-#include <libgwydgets/gwydgetutils.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
@@ -46,7 +34,7 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
+    GwyField *field;
     GwyBrick *brick;
     gint *ids;
     gint nids;
@@ -54,7 +42,7 @@ typedef struct {
     gdouble *ydata;
     gdouble *zdata;
     gdouble ndata;
-    GwyContainer *data;
+    GwyFile *data;
 } ModuleArgs;
 
 typedef struct {
@@ -65,8 +53,8 @@ typedef struct {
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             volumize_layers     (GwyContainer *data,
-                                             GwyRunType run);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static GwyDialogOutcome run_gui             (ModuleArgs *args);
 static void             param_changed       (ModuleGUI *gui,
                                              gint id);
@@ -75,7 +63,7 @@ static GwyBrick*        execute             (ModuleArgs *args);
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
-    N_("Converts all datafields to 3D volume data."),
+    N_("Converts all fields to 3D volume data."),
     "Petr Klapetek <klapetek@gwyddion.net>",
     "3.1",
     "David Nečas (Yeti) & Petr Klapetek",
@@ -88,12 +76,12 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("volumize_layers",
-                              (GwyProcessFunc)&volumize_layers,
+                              module_main,
                               N_("/_Basic Operations/Volumize Layers..."),
-                              GWY_STOCK_VOLUMIZE_LAYERS,
+                              GWY_ICON_VOLUMIZE_LAYERS,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
-                              N_("Convert all datafields to 3D data"));
+                              GWY_MENU_FLAG_IMAGE,
+                              N_("Convert all fields to 3D data"));
 
     return TRUE;
 }
@@ -118,33 +106,33 @@ define_module_params(void)
 }
 
 static void
-volumize_layers(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome;
     ModuleArgs args;
-    GwyDataField *preview;
+    GwyField *preview;
     GtkWidget *dialog;
     gboolean ok = TRUE;
     gint *ids, i, nids, xres, yres, newid;
     GwyGraphCurveModel *gcmodel;
     GwyGraphModel *gmodel;
 
-    gwy_clear(&args, 1);
-    g_return_if_fail(runtype & RUN_MODES);
+    gwy_clear1(args);
+    g_return_if_fail(mode & RUN_MODES);
 
-    ids = gwy_app_data_browser_get_data_ids(data);
+    ids = gwy_file_get_ids(data, GWY_FILE_IMAGE);
 
     args.params = gwy_params_new_from_settings(define_module_params());
 
     nids = 1;
-    args.field = gwy_container_get_object(data, gwy_app_get_data_key_for_id(ids[0]));
-    xres = gwy_data_field_get_xres(args.field);
-    yres = gwy_data_field_get_yres(args.field);
+    args.field = gwy_file_get_image(data, ids[0]);
+    xres = gwy_field_get_xres(args.field);
+    yres = gwy_field_get_yres(args.field);
 
     i = 0;
     while (ids[i] != -1) {
-        args.field = gwy_container_get_object(data, gwy_app_get_data_key_for_id(ids[i]));
-        if (xres != gwy_data_field_get_xres(args.field) || yres != gwy_data_field_get_yres(args.field)) {
+        args.field = gwy_file_get_image(data, ids[i]);
+        if (xres != gwy_field_get_xres(args.field) || yres != gwy_field_get_yres(args.field)) {
             ok = FALSE;
             break;
         }
@@ -154,9 +142,9 @@ volumize_layers(GwyContainer *data, GwyRunType runtype)
     }
 
     if (!ok) {
-        dialog = gtk_message_dialog_new(gwy_app_find_window_for_channel(data, ids[0]),
+        dialog = gtk_message_dialog_new(gwy_data_browser_get_window_for_data(data, GWY_FILE_IMAGE, ids[0]),
                                         GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
-                                        _("All datafields must have same resolution to make a volume from them."));
+                                        _("All fields must have same resolution to make a volume from them."));
         gtk_dialog_run(GTK_DIALOG(dialog));
         gtk_widget_destroy(dialog);
         goto end;
@@ -173,7 +161,7 @@ volumize_layers(GwyContainer *data, GwyRunType runtype)
     args.zdata = g_new0(gdouble, nids);
     args.ndata = nids-1;
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -183,13 +171,13 @@ volumize_layers(GwyContainer *data, GwyRunType runtype)
 
     xres = gwy_params_get_int(args.params, PARAM_XRES);
     yres = gwy_params_get_int(args.params, PARAM_YRES);
-    preview = gwy_data_field_new(xres, yres, xres, yres, FALSE);
+    preview = gwy_field_new(xres, yres, xres, yres, FALSE);
     gwy_brick_mean_xy_plane(args.brick, preview);
-    newid = gwy_app_data_browser_add_brick(args.brick, preview, data, TRUE);
+    newid = gwy_file_add_volume(data, args.brick, preview);
+    gwy_file_set_visible(data, GWY_FILE_VOLUME, newid, TRUE);
     g_object_unref(args.brick);
     g_object_unref(preview);
-    gwy_app_volume_log_add(data, -1, newid, "proc::volumize_layers", NULL);
-
+    gwy_log_add_full(data, GWY_FILE_VOLUME, -1, newid, "proc::volumize_layers", NULL);
 
     if (gwy_params_get_boolean(args.params, PARAM_OFFSETS)) {
         gcmodel = gwy_graph_curve_model_new();
@@ -206,9 +194,9 @@ volumize_layers(GwyContainer *data, GwyRunType runtype)
                      "axis-label-left", _("X offset"),
                      NULL);
 
-        gwy_graph_model_set_units_from_data_field(gmodel, args.field,
-                                                  0, 0, 0, 1);
-        gwy_app_data_browser_add_graph_model(gmodel, data, TRUE);
+        gwy_graph_model_set_units_from_field(gmodel, args.field, 0, 0, 0, 1);
+        newid = gwy_file_add_graph(data, gmodel);
+        gwy_file_set_visible(data, GWY_FILE_GRAPH, newid, TRUE);
         g_object_unref(gmodel);
 
         gcmodel = gwy_graph_curve_model_new();
@@ -225,18 +213,18 @@ volumize_layers(GwyContainer *data, GwyRunType runtype)
                      "axis-label-left", _("Y offset"),
                      NULL);
 
-        gwy_graph_model_set_units_from_data_field(gmodel, args.field,
-                                                  0, 0, 0, 1);
-        gwy_app_data_browser_add_graph_model(gmodel, data, TRUE);
+        gwy_graph_model_set_units_from_field(gmodel, args.field, 0, 0, 0, 1);
+        newid = gwy_file_add_graph(data, gmodel);
+        gwy_file_set_visible(data, GWY_FILE_GRAPH, newid, TRUE);
         g_object_unref(gmodel);
-     }
+    }
 
 end:
     g_free(ids);
     g_free(args.xdata);
     g_free(args.ydata);
     g_free(args.zdata);
-    gwy_object_unref(args.params);
+    g_clear_object(&args.params);
 }
 
 static GwyDialogOutcome
@@ -278,19 +266,19 @@ param_changed(ModuleGUI *gui, gint id)
 
     if (id < 0 || id == PARAM_ZUNIT) {
         gint power10;
-        GwySIUnit *unit = gwy_params_get_unit(params, PARAM_ZUNIT, &power10);
-        GwySIValueFormat *vf = gwy_si_unit_get_format_for_power10(unit, GWY_SI_UNIT_FORMAT_VFMARKUP, power10, NULL);
+        GwyUnit *unit = gwy_params_get_unit(params, PARAM_ZUNIT, &power10);
+        GwyValueFormat *vf = gwy_unit_get_format_for_power10(unit, GWY_UNIT_FORMAT_VFMARKUP, power10, NULL);
         gwy_param_table_set_unitstr(gui->table, PARAM_ZREAL, vf->units);
-        gwy_si_unit_value_format_free(vf);
+        gwy_value_format_free(vf);
     }
 }
 
 static GwyBrick*
 execute(ModuleArgs *args)
 {
-    GwyDataField *field = args->field;
+    GwyField *field = args->field;
     GwyBrick *brick = args->brick;
-    GwyContainer *data = args->data;
+    GwyFile *data = args->data;
     gint xres = gwy_params_get_int(args->params, PARAM_XRES);
     gint yres = gwy_params_get_int(args->params, PARAM_YRES);
     gint zres = gwy_params_get_int(args->params, PARAM_ZRES);
@@ -299,16 +287,16 @@ execute(ModuleArgs *args)
     gint power10, i, row, col;
     gdouble zreal = gwy_params_get_double(args->params, PARAM_ZREAL);
     gdouble *ddata, *bdata;
-    GwySIUnit *zunit = gwy_params_get_unit(args->params, PARAM_ZUNIT, &power10);
+    GwyUnit *zunit = gwy_params_get_unit(args->params, PARAM_ZUNIT, &power10);
     gboolean offsets = gwy_params_get_boolean(args->params, PARAM_OFFSETS);
 
     brick = gwy_brick_new(xres, yres, nids-1,
-                          gwy_data_field_get_xreal(field), gwy_data_field_get_yreal(field), zreal * pow10(power10),
+                          gwy_field_get_xreal(field), gwy_field_get_yreal(field), zreal * gwy_exp10(power10),
                           FALSE);
     bdata = gwy_brick_get_data(brick);
     for (i = 0; i < nids-1; i++) {
-        field = gwy_container_get_object(data, gwy_app_get_data_key_for_id(ids[i]));
-        ddata = gwy_data_field_get_data(field);
+        field = gwy_file_get_image(data, ids[i]);
+        ddata = gwy_field_get_data(field);
 
         for (row = 0; row < yres; row++) {
             for (col = 0; col < xres; col++)
@@ -317,16 +305,16 @@ execute(ModuleArgs *args)
     }
 
     gwy_brick_resample(brick, xres, yres, zres, GWY_INTERPOLATION_ROUND);
-    gwy_si_unit_assign(gwy_brick_get_si_unit_x(brick), gwy_data_field_get_si_unit_xy(field));
-    gwy_si_unit_assign(gwy_brick_get_si_unit_y(brick), gwy_data_field_get_si_unit_xy(field));
-    gwy_si_unit_assign(gwy_brick_get_si_unit_w(brick), gwy_data_field_get_si_unit_z(field));
-    gwy_brick_set_si_unit_z(brick, zunit);
+    gwy_unit_assign(gwy_brick_get_unit_x(brick), gwy_field_get_unit_xy(field));
+    gwy_unit_assign(gwy_brick_get_unit_y(brick), gwy_field_get_unit_xy(field));
+    gwy_unit_assign(gwy_brick_get_unit_w(brick), gwy_field_get_unit_z(field));
+    gwy_unit_assign(gwy_brick_get_unit_z(brick), zunit);
 
     if (offsets) {
         for (i = 0; i < nids-1; i++) {
-            field = gwy_container_get_object(data, gwy_app_get_data_key_for_id(ids[i]));
-            args->xdata[i] = gwy_data_field_get_xoffset(field);
-            args->ydata[i] = gwy_data_field_get_yoffset(field);
+            field = gwy_file_get_image(data, ids[i]);
+            args->xdata[i] = gwy_field_get_xoffset(field);
+            args->ydata[i] = gwy_field_get_yoffset(field);
             args->zdata[i] = i;
         }
     }
