@@ -1,5 +1,5 @@
 /*
- *  $Id: wpour_mark.c 26524 2024-08-15 18:14:00Z yeti-dn $
+ *  $Id: wpour_mark.c 28910 2025-11-24 18:11:09Z yeti-dn $
  *  Copyright (C) 2014-2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,18 +16,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include "string.h"
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/grains.h>
-#include <libprocess/stats.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/filters.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
@@ -68,46 +60,46 @@ typedef struct {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *preproc;
-    GwyDataField *mask;
-    GwyDataField *result;
+    GwyField *field;
+    GwyField *preproc;
+    GwyField *mask;
+    GwyField *result;
 } ModuleArgs;
 
 typedef struct {
     ModuleArgs *args;
     GtkWidget *dialog;
+    GtkWidget *dataview;
     GwyParamTable *table;
-    GwyContainer *data;
 } ModuleGUI;
 
 static gboolean         module_register         (void);
 static GwyParamDef*     define_module_params    (void);
-static void             wpour_mark              (GwyContainer *data,
-                                                 GwyRunType runtype);
+static void             module_main             (GwyFile *data,
+                                                 GwyRunModeFlags mode);
 static void             execute                 (ModuleArgs *args);
 static GwyDialogOutcome run_gui                 (ModuleArgs *args,
-                                                 GwyContainer *data,
+                                                 GwyFile *data,
                                                  gint id);
 static void             param_changed           (ModuleGUI *gui,
                                                  gint id);
 static void             preview                 (gpointer user_data);
-static void             add_slope_contribs      (GwyDataField *workspace,
-                                                 GwyDataField *field,
+static void             add_slope_contribs      (GwyField *workspace,
+                                                 GwyField *field,
                                                  gdouble gradient_contrib,
                                                  gdouble curvature_contrib);
-static void             normal_vector_difference(GwyDataField *result,
-                                                 GwyDataField *xder,
-                                                 GwyDataField *yder);
-static gdouble          create_barriers         (GwyDataField *field,
+static void             normal_vector_difference(GwyField *result,
+                                                 GwyField *xder,
+                                                 GwyField *yder);
+static gdouble          create_barriers         (GwyField *field,
                                                  gdouble level);
-static void             prefill_minima          (GwyDataField *field,
-                                                 GwyDataField *workspace,
+static void             prefill_minima          (GwyField *field,
+                                                 GwyField *workspace,
                                                  IntList *inqueue,
                                                  IntList *outqueue,
                                                  gdouble depth,
                                                  gdouble height);
-static void             replace_value           (GwyDataField *field,
+static void             replace_value           (GwyField *field,
                                                  gdouble from,
                                                  gdouble to);
 
@@ -127,11 +119,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("wpour_mark",
-                              (GwyProcessFunc)&wpour_mark,
+                              module_main,
                               N_("/_Grains/_Mark by Segmentation..."),
-                              GWY_STOCK_GRAINS_WATER,
+                              GWY_ICON_GRAINS_WATER,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Segment using watershed "));
 
     return TRUE;
@@ -174,27 +166,27 @@ define_module_params(void)
 }
 
 static void
-wpour_mark(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     ModuleArgs args;
     GQuark mquark;
     gint id;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_MASK_FIELD, &args.mask,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     GWY_APP_MASK_FIELD_KEY, &mquark,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_MASK_FIELD, &args.mask,
+                                 GWY_APP_FIELD_ID, &id,
+                                 GWY_APP_MASK_FIELD_KEY, &mquark,
+                                 0);
     g_return_if_fail(args.field && mquark);
 
-    args.result = gwy_data_field_new_alike(args.field, TRUE);
-    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(args.result), NULL);
-    args.preproc = gwy_data_field_new_alike(args.field, TRUE);
+    args.result = gwy_field_new_alike(args.field, TRUE);
+    gwy_unit_clear(gwy_field_get_unit_z(args.result));
+    args.preproc = gwy_field_new_alike(args.field, TRUE);
     args.params = gwy_params_new_from_settings(define_module_params());
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, id);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -203,43 +195,39 @@ wpour_mark(GwyContainer *data, GwyRunType runtype)
     if (outcome != GWY_DIALOG_HAVE_RESULT)
         execute(&args);
 
-    gwy_app_undo_qcheckpointv(data, 1, &mquark);
-    if (gwy_data_field_get_max(args.result) > 0.0)
-        gwy_container_set_object(data, mquark, args.result);
+    gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &mquark);
+    if (gwy_field_get_max(args.result) > 0.0) {
+        gwy_file_pass_image_mask(data, id, args.result);
+        args.result = NULL;
+    }
     else
-        gwy_container_remove(data, mquark);
-    gwy_app_channel_log_add_proc(data, id, id);
+        gwy_container_remove(GWY_CONTAINER(data), mquark);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, id);
 
 end:
     g_object_unref(args.params);
     g_object_unref(args.preproc);
-    g_object_unref(args.result);
+    g_clear_object(&args.result);
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     GtkWidget *hbox, *dataview;
     GwyDialog *dialog;
     GwyParamTable *table;
     ModuleGUI gui;
-    GwyDialogOutcome outcome;
 
     gui.args = args;
-    gui.data = gwy_container_new();
-    gwy_container_set_object_by_name(gui.data, "/0/data", args->field);
-    gwy_container_set_object_by_name(gui.data, "/0/mask", args->result);
-    gwy_app_sync_data_items(data, gui.data, id, 0, FALSE,
-                            GWY_DATA_ITEM_PALETTE,
-                            GWY_DATA_ITEM_RANGE,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
 
     gui.dialog = gwy_dialog_new(_("Segment by Watershed"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_UPDATE, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, TRUE);
+    dataview = gui.dataview = gwy_create_preview(args->field, args->result, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_MASK_COLOR
+                        | GWY_FILE_ITEM_RANGE | GWY_FILE_ITEM_REAL_SQUARE);
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), FALSE);
 
     table = gui.table = gwy_param_table_new(args->params);
@@ -258,7 +246,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 
     gwy_param_table_append_header(table, -1, _("Options"));
     gwy_param_table_append_checkbox(table, PARAM_INVERTED);
-    gwy_param_table_append_mask_color(table, PARAM_MASK_COLOR, gui.data, 0, data, id);
+    gwy_param_table_append_image_mask_color(table, PARAM_MASK_COLOR, GWY_DATA_VIEW(dataview), data, id);
     if (args->mask) {
         gwy_param_table_append_radio_buttons(table, PARAM_COMBINE_TYPE, NULL);
         gwy_param_table_add_enabler(table, PARAM_COMBINE, PARAM_COMBINE_TYPE);
@@ -274,11 +262,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     g_signal_connect_swapped(table, "param-changed", G_CALLBACK(param_changed), &gui);
     gwy_dialog_set_preview_func(dialog, GWY_PREVIEW_IMMEDIATE, preview, &gui, NULL);
 
-    outcome = gwy_dialog_run(dialog);
-
-    g_object_unref(gui.data);
-
-    return outcome;
+    return gwy_dialog_run(dialog);
 }
 
 static void
@@ -289,15 +273,12 @@ param_changed(ModuleGUI *gui, gint id)
 
     if (id < 0 || id == PARAM_IMAGE_PREVIEW) {
         ImagePreviewType image_preview = gwy_params_get_enum(params, PARAM_IMAGE_PREVIEW);
-        GwyDataField *field = (image_preview == IMAGE_PREVIEW_ORIGINAL ? args->field : args->preproc);
-        gwy_container_set_object_by_name(gui->data, "/0/data", field);
+        GwyField *field = (image_preview == IMAGE_PREVIEW_ORIGINAL ? args->field : args->preproc);
+        gwy_data_view_set_field(GWY_DATA_VIEW(gui->dataview), field);
     }
     if (id < 0 || id == PARAM_MASK_PREVIEW) {
         MaskPreviewType mask_preview = gwy_params_get_enum(params, PARAM_MASK_PREVIEW);
-        if (mask_preview == MASK_PREVIEW_MARKED)
-            gwy_container_set_object_by_name(gui->data, "/0/mask", args->result);
-        else
-            gwy_container_remove_by_name(gui->data, "/0/mask");
+        gwy_data_view_set_mask(GWY_DATA_VIEW(gui->dataview), mask_preview == MASK_PREVIEW_MARKED ? args->result : NULL);
     }
 
     if (id != PARAM_MASK_COLOR && id != PARAM_UPDATE && id != PARAM_IMAGE_PREVIEW && id != PARAM_MASK_PREVIEW)
@@ -311,15 +292,15 @@ preview(gpointer user_data)
     ModuleArgs *args = gui->args;
 
     execute(args);
-    gwy_data_field_data_changed(args->preproc);
-    gwy_data_field_data_changed(args->result);
+    gwy_field_data_changed(args->preproc);
+    gwy_field_data_changed(args->result);
     gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
 }
 
 static inline IntList*
 int_list_new(guint prealloc)
 {
-    IntList *list = g_slice_new0(IntList);
+    IntList *list = g_new0(IntList, 1);
     prealloc = MAX(prealloc, 16);
     list->size = prealloc;
     list->data = g_new(gint, list->size);
@@ -361,13 +342,13 @@ static void
 int_list_free(IntList *list)
 {
     g_free(list->data);
-    g_slice_free(IntList, list);
+    g_free(list);
 }
 
 static void
 execute(ModuleArgs *args)
 {
-    GwyDataField *result = args->result, *field = args->field, *preproc = args->preproc, *mask = args->mask;
+    GwyField *result = args->result, *field = args->field, *preproc = args->preproc, *mask = args->mask;
     GwyParams *params = args->params;
     gboolean combine = gwy_params_get_boolean(params, PARAM_COMBINE);
     GwyMergeType combine_type = gwy_params_get_enum(params, PARAM_COMBINE_TYPE);
@@ -383,23 +364,23 @@ execute(ModuleArgs *args)
     guint xres, yres;
     gdouble barmax;
 
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
 
-    gwy_data_field_copy(field, preproc, FALSE);
+    gwy_field_copy_data(field, preproc);
     if (inverted)
-        gwy_data_field_invert(preproc, FALSE, FALSE, TRUE);
+        gwy_field_invert_value(preproc);
 
     /* Use result as a scratch buffer. */
-    gwy_data_field_add(preproc, -gwy_data_field_get_max(preproc));
+    gwy_field_add(preproc, -gwy_field_get_max(preproc));
     if (blur_fwhm)
-        gwy_data_field_area_filter_gaussian(preproc, blur_fwhm*FWHM2SIGMA, 0, 0, xres, yres);
+        gwy_field_area_filter_gaussian(preproc, blur_fwhm*FWHM2SIGMA, 0, 0, xres, yres);
     add_slope_contribs(result, preproc, gradient_contrib, curvature_contrib);
     barmax = create_barriers(preproc, barrier_level);
     prefill_minima(preproc, result, inqueue, outqueue, prefill_level, prefill_height);
 
     replace_value(preproc, barmax, HUGE_VAL);
-    gwy_data_field_waterpour(preproc, result, NULL);
+    gwy_field_waterpour(preproc, result, NULL);
     /* Lower the barriers again to avoid HUGE_VAL in the preview. */
     replace_value(preproc, HUGE_VAL, barmax);
 
@@ -408,15 +389,15 @@ execute(ModuleArgs *args)
 
     if (mask && combine) {
         if (combine_type == GWY_MERGE_UNION)
-            gwy_data_field_grains_add(result, mask);
+            gwy_field_grains_add(result, mask);
         else if (combine_type == GWY_MERGE_INTERSECTION)
-            gwy_data_field_grains_intersect(result, mask);
+            gwy_field_grains_intersect(result, mask);
     }
 
 #if 0
     /* FIXME: neither grains nor gnetwork used for anything at this moment. */
     grains = g_new0(gint, xres*yres);
-    ngrains = gwy_data_field_number_grains(result, grains);
+    ngrains = gwy_field_number_grains(result, grains);
     gnetwork = analyse_grain_network(preproc, grains);
     g_free(grains);
     g_hash_table_destroy(gnetwork);
@@ -424,50 +405,50 @@ execute(ModuleArgs *args)
 }
 
 static void
-add_slope_contribs(GwyDataField *workspace, GwyDataField *field,
+add_slope_contribs(GwyField *workspace, GwyField *field,
                    gdouble gradient_contrib, gdouble curvature_contrib)
 {
-    GwyDataField *xder, *yder;
+    GwyField *xder, *yder;
     gdouble r, rg, rc;
 
     if (!gradient_contrib && !curvature_contrib)
         return;
 
-    r = gwy_data_field_get_rms(field);
+    r = gwy_field_get_rms(field);
     if (!r)
         return;
 
-    xder = gwy_data_field_new_alike(field, FALSE);
-    yder = gwy_data_field_new_alike(field, FALSE);
+    xder = gwy_field_new_alike(field, FALSE);
+    yder = gwy_field_new_alike(field, FALSE);
 
-    gwy_data_field_filter_slope(field, xder, yder);
-    gwy_data_field_multiply(field, 1.0 - MAX(gradient_contrib, curvature_contrib));
+    gwy_field_filter_slope(field, xder, yder);
+    gwy_field_multiply(field, 1.0 - MAX(gradient_contrib, curvature_contrib));
 
     /* We need this for both operations. */
-    gwy_data_field_hypot_of_fields(workspace, xder, yder);
-    rg = gwy_data_field_get_rms(workspace);
+    gwy_field_hypot_of_fields(workspace, xder, yder);
+    rg = gwy_field_get_rms(workspace);
 
     if (gradient_contrib) {
-        gwy_data_field_multiply(workspace, gradient_contrib * r/rg);
-        gwy_data_field_sum_fields(field, field, workspace);
+        gwy_field_multiply(workspace, gradient_contrib * r/rg);
+        gwy_field_sum_fields(field, field, workspace);
     }
 
     if (curvature_contrib) {
-        gwy_data_field_multiply(xder, 1.0/rg);
-        gwy_data_field_multiply(yder, 1.0/rg);
+        gwy_field_multiply(xder, 1.0/rg);
+        gwy_field_multiply(yder, 1.0/rg);
         normal_vector_difference(workspace, xder, yder);
-        rc = gwy_data_field_get_rms(workspace);
+        rc = gwy_field_get_rms(workspace);
         if (rc) {
-            gwy_data_field_multiply(workspace, curvature_contrib * r/rc);
-            gwy_data_field_sum_fields(field, field, workspace);
+            gwy_field_multiply(workspace, curvature_contrib * r/rc);
+            gwy_field_sum_fields(field, field, workspace);
         }
     }
 
     g_object_unref(yder);
     g_object_unref(xder);
 
-    gwy_data_field_invalidate(field);
-    gwy_data_field_invalidate(workspace);
+    gwy_field_invalidate(field);
+    gwy_field_invalidate(workspace);
 }
 
 static void
@@ -481,18 +462,18 @@ normal_vector(gdouble bx, gdouble by, gdouble *nx, gdouble *ny, gdouble *nz)
 }
 
 static void
-normal_vector_difference(GwyDataField *result, GwyDataField *xder, GwyDataField *yder)
+normal_vector_difference(GwyField *result, GwyField *xder, GwyField *yder)
 {
     const gdouble *bx, *by;
     guint xres, yres, i, j;
     gdouble *d;
 
-    gwy_data_field_clear(result);
+    gwy_field_clear(result);
     xres = result->xres;
     yres = result->yres;
-    d = result->data;
-    bx = xder->data;
-    by = yder->data;
+    d = gwy_field_get_data(result);
+    bx = gwy_field_get_data_const(xder);
+    by = gwy_field_get_data_const(yder);
 
     for (i = 0; i < yres; i++) {
         gdouble *row = d + i*xres, *next = row + xres;
@@ -521,67 +502,63 @@ normal_vector_difference(GwyDataField *result, GwyDataField *xder, GwyDataField
             }
         }
     }
-
-    gwy_data_field_invalidate(result);
 }
 
 static gdouble
-create_barriers(GwyDataField *field, gdouble level)
+create_barriers(GwyField *field, gdouble level)
 {
     guint k, xres = field->xres, yres = field->yres;
     gdouble min, max, barmax;
 
-    gwy_data_field_get_min_max(field, &min, &max);
+    gwy_field_get_min_max(field, &min, &max);
     barmax = 1.01*max;
     if (min == max)
         return barmax;
 
     if (level < 1.0) {
         gdouble threshold = level*(max - min) + min;
-        gdouble *d = field->data;
+        gdouble *d = gwy_field_get_data(field);
 
         barmax = max;
         for (k = 0; k < xres*yres; k++) {
             if (d[k] >= threshold)
                 d[k] = barmax;
         }
-        gwy_data_field_invalidate(field);
     }
 
     return barmax;
 }
 
 static void
-prefill_minima(GwyDataField *field, GwyDataField *workspace,
+prefill_minima(GwyField *field, GwyField *workspace,
                IntList *inqueue, IntList *outqueue,
                gdouble depth, gdouble height)
 {
     guint i, j, k, m, xres = field->xres, yres = field->yres;
     gdouble min, max;
 
-    gwy_data_field_get_min_max(field, &min, &max);
+    gwy_field_get_min_max(field, &min, &max);
     if (min == max)
         return;
 
     /* Simple absolute prefilling corresponding to plain mark-by-threshold. */
     if (depth > 0.0) {
         gdouble depththreshold = depth*(max - min) + min;
-        gdouble *d = field->data;
+        gdouble *d = gwy_field_get_data(field);
 
         for (k = 0; k < xres*yres; k++) {
             if (d[k] < depththreshold)
                 d[k] = depththreshold;
         }
-        gwy_data_field_invalidate(field);
     }
 
     /* Simple height prefilling which floods all pixels with heights only
      * little above the minimum. */
     if (height > 0.0) {
         gdouble heightthreshold = height*(max - min);
-        gdouble *d = field->data, *w = workspace->data;
+        gdouble *d = gwy_field_get_data(field), *w = gwy_field_get_data(workspace);
 
-        gwy_data_field_mark_extrema(field, workspace, FALSE);
+        gwy_field_mark_extrema(field, workspace, FALSE);
 
         inqueue->len = 0;
         for (k = 0; k < xres*yres; k++) {
@@ -621,21 +598,20 @@ prefill_minima(GwyDataField *field, GwyDataField *workspace,
             GWY_SWAP(IntList*, inqueue, outqueue);
         }
 
-        gwy_data_field_invalidate(field);
+        gwy_field_invalidate(field);
     }
 }
 
 static void
-replace_value(GwyDataField *field, gdouble from, gdouble to)
+replace_value(GwyField *field, gdouble from, gdouble to)
 {
     guint k, xres = field->xres, yres = field->yres;
-    gdouble *d = field->data;
+    gdouble *d = gwy_field_get_data(field);
 
     for (k = 0; k < xres*yres; k++) {
         if (d[k] == from)
             d[k] = to;
     }
-    gwy_data_field_invalidate(field);
 }
 
 #if 0
@@ -667,13 +643,13 @@ uint_pair_equal(gconstpointer pa, gconstpointer pb)
 static void
 uint_pair_free(gpointer p)
 {
-    g_slice_free(UIntPair, p);
+    g_free(p);
 }
 
 static void
 grain_neighbour_free(gpointer p)
 {
-    g_slice_free(GrainNeighbour, p);
+    g_free(p);
 }
 
 static inline gboolean
@@ -716,7 +692,7 @@ is_merge_pixel(const gint *gc, gint *pg1, gint *pg2)
 }
 
 static GHashTable*
-analyse_grain_network(GwyDataField *field, const gint *grains)
+analyse_grain_network(GwyField *field, const gint *grains)
 {
     GHashTable *gnetwork;
     guint xres = field->xres, yres = field->yres;
@@ -778,8 +754,8 @@ analyse_grain_network(GwyDataField *field, const gint *grains)
                 neighbour->min_bsum = MIN(neighbour->min_bsum, min_bsum);
             }
             else {
-                UIntPair *ppair = g_slice_dup(UIntPair, &pair);
-                neighbour = g_slice_new(GrainNeighbour);
+                UIntPair *ppair = g_memdup2(&pair, sizeof(UIntPair));
+                neighbour = g_new(GrainNeighbour, 1);
                 neighbour->min_barrier = min_barrier;
                 neighbour->min_bsum = min_bsum;
                 g_hash_table_insert(gnetwork, ppair, neighbour);
