@@ -1,5 +1,5 @@
 /*
- *  $Id: xydenoise.c 23666 2021-05-10 21:56:10Z yeti-dn $
+ *  $Id: xydenoise.c 28821 2025-11-06 16:00:41Z yeti-dn $
  *  Copyright (C) 2012-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,18 +16,9 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwythreads.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/correlation.h>
-#include <libprocess/filters.h>
-#include <libprocess/inttrans.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <libgwydgets/gwystock.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 
 #define RUN_MODES GWY_RUN_INTERACTIVE
 
@@ -38,8 +29,8 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
-    GwyDataField *result;
+    GwyField *field;
+    GwyField *result;
 } ModuleArgs;
 
 typedef struct {
@@ -50,13 +41,13 @@ typedef struct {
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             xydenoise           (GwyContainer *data,
-                                             GwyRunType runtype);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static void             execute             (ModuleArgs *args);
 static GwyDialogOutcome run_gui             (ModuleArgs *args);
 static void             param_changed       (ModuleGUI *gui,
                                              gint id);
-static gboolean         other_image_filter  (GwyContainer *data,
+static gboolean         other_image_filter  (GwyFile *data,
                                              gint id,
                                              gpointer user_data);
 
@@ -76,11 +67,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("xydenoise",
-                              (GwyProcessFunc)&xydenoise,
+                              module_main,
                               N_("/M_ultidata/_XY Denoise..."),
-                              GWY_STOCK_XY_DENOISE,
+                              GWY_ICON_XY_DENOISE,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Denoises horizontal/vertical measurement."));
 
     return TRUE;
@@ -102,19 +93,19 @@ define_module_params(void)
 }
 
 static void
-xydenoise(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome;
     ModuleArgs args;
     gint id, newid;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(args.field);
 
-    args.result = gwy_data_field_new_alike(args.field, FALSE);
+    args.result = gwy_field_new_alike(args.field, FALSE);
     args.params = gwy_params_new_from_settings(define_module_params());
     outcome = run_gui(&args);
     gwy_params_save_to_settings(args.params);
@@ -122,10 +113,13 @@ xydenoise(GwyContainer *data, GwyRunType runtype)
         goto end;
 
     execute(&args);
-    newid = gwy_app_data_browser_add_data_field(args.result, data, TRUE);
-    gwy_app_sync_data_items(data, data, id, newid, FALSE, GWY_DATA_ITEM_GRADIENT, 0);
-    gwy_app_set_data_field_title(data, newid, _("Denoised"));
-    gwy_app_channel_log_add_proc(data, id, newid);
+    newid = gwy_file_add_image(data, args.result);
+    gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+    gwy_file_sync_items(data, GWY_FILE_IMAGE, id,
+                        data, GWY_FILE_IMAGE, newid,
+                        GWY_FILE_ITEM_PALETTE, FALSE);
+    gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _("Denoised"), TRUE);
+    gwy_log_add(data, GWY_FILE_IMAGE, id, newid);
 
 end:
     g_object_unref(args.result);
@@ -170,47 +164,44 @@ param_changed(ModuleGUI *gui, gint id)
 }
 
 static gboolean
-other_image_filter(GwyContainer *data, gint id, gpointer user_data)
+other_image_filter(GwyFile *data, gint id, gpointer user_data)
 {
-    GwyDataField *otherimage, *field = (GwyDataField*)user_data;
+    GwyField *otherimage = gwy_file_get_image(data, id), *field = (GwyField*)user_data;
 
-    if (!gwy_container_gis_object(data, gwy_app_get_data_key_for_id(id), &otherimage))
-        return FALSE;
     if (otherimage == field)
         return FALSE;
 
-    return !gwy_data_field_check_compatibility(field, otherimage,
-                                               GWY_DATA_COMPATIBILITY_RES
-                                               | GWY_DATA_COMPATIBILITY_REAL
-                                               | GWY_DATA_COMPATIBILITY_LATERAL
-                                               | GWY_DATA_COMPATIBILITY_VALUE);
+    return !gwy_field_is_incompatible(field, otherimage,
+                                      GWY_DATA_MISMATCH_RES
+                                      | GWY_DATA_MISMATCH_REAL
+                                      | GWY_DATA_MISMATCH_LATERAL
+                                      | GWY_DATA_MISMATCH_VALUE);
 }
 
 static void
 execute(ModuleArgs *args)
 {
-    GwyDataField *fieldx = args->field, *fieldy = gwy_params_get_image(args->params, PARAM_OTHER_IMAGE);
-    GwyDataField *rx, *ix, *ry, *iy, *result = args->result, *iresult;
+    GwyField *fieldx = args->field, *fieldy = gwy_params_get_image(args->params, PARAM_OTHER_IMAGE);
+    GwyField *rx, *ix, *ry, *iy, *result = args->result, *iresult;
     gboolean do_average = gwy_params_get_boolean(args->params, PARAM_DO_AVERAGE);
     gdouble *rxdata, *rydata, *ixdata, *iydata;
     gint i, n;
 
-    n = gwy_data_field_get_xres(fieldx) * gwy_data_field_get_yres(fieldy);
-    iresult = gwy_data_field_new_alike(fieldx, TRUE);
-    rx = gwy_data_field_new_alike(fieldx, TRUE);
-    ix = gwy_data_field_new_alike(fieldx, TRUE);
-    ry = gwy_data_field_new_alike(fieldx, TRUE);
-    iy = gwy_data_field_new_alike(fieldx, TRUE);
+    n = gwy_field_get_xres(fieldx) * gwy_field_get_yres(fieldy);
+    iresult = gwy_field_new_alike(fieldx, TRUE);
+    rx = gwy_field_new_alike(fieldx, TRUE);
+    ix = gwy_field_new_alike(fieldx, TRUE);
+    ry = gwy_field_new_alike(fieldx, TRUE);
+    iy = gwy_field_new_alike(fieldx, TRUE);
 
-    gwy_data_field_2dfft(fieldx, NULL, rx, ix,
-                         GWY_WINDOWING_NONE, GWY_TRANSFORM_DIRECTION_FORWARD, GWY_INTERPOLATION_LINEAR, FALSE, 0);
-    gwy_data_field_2dfft(fieldy, NULL, ry, iy,
-                         GWY_WINDOWING_NONE, GWY_TRANSFORM_DIRECTION_FORWARD, GWY_INTERPOLATION_LINEAR, FALSE, 0);
+    /* FIXME: Why we do not use the raw function? */
+    gwy_field_fft_2d(fieldx, NULL, rx, ix, GWY_WINDOWING_NONE, GWY_TRANSFORM_DIRECTION_FORWARD, FALSE, 0);
+    gwy_field_fft_2d(fieldy, NULL, ry, iy, GWY_WINDOWING_NONE, GWY_TRANSFORM_DIRECTION_FORWARD, FALSE, 0);
 
-    rxdata = gwy_data_field_get_data(rx);
-    rydata = gwy_data_field_get_data(ry);
-    ixdata = gwy_data_field_get_data(ix);
-    iydata = gwy_data_field_get_data(iy);
+    rxdata = gwy_field_get_data(rx);
+    rydata = gwy_field_get_data(ry);
+    ixdata = gwy_field_get_data(ix);
+    iydata = gwy_field_get_data(iy);
 
 #ifdef _OPENMP
 #pragma omp parallel for if(gwy_threads_are_enabled()) default(none) \
@@ -233,8 +224,7 @@ execute(ModuleArgs *args)
         ixdata[i] = modulus*sinxphase;
     }
 
-    gwy_data_field_2dfft(rx, ix, result, iresult,
-                         GWY_WINDOWING_NONE, GWY_TRANSFORM_DIRECTION_BACKWARD, GWY_INTERPOLATION_LINEAR, FALSE, 0);
+    gwy_field_fft_2d(rx, ix, result, iresult, GWY_WINDOWING_NONE, GWY_TRANSFORM_DIRECTION_BACKWARD, FALSE, 0);
 
     g_object_unref(iresult);
     g_object_unref(rx);
