@@ -1,5 +1,5 @@
 /*
- *  $Id: xyz_channels.c 24749 2022-03-25 15:06:08Z yeti-dn $
+ *  $Id: xyz_channels.c 28821 2025-11-06 16:00:41Z yeti-dn $
  *  Copyright (C) 2003-2022 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,19 +16,9 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/inttrans.h>
-#include <libprocess/stats.h>
-#include <libgwydgets/gwydgetutils.h>
-#include <libgwydgets/gwycombobox.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 #include "preview.h"
 
 #define RUN_MODES (GWY_RUN_INTERACTIVE)
@@ -41,7 +31,7 @@ enum {
 
 typedef struct {
     GwyParams *params;
-    GwyDataField *field;
+    GwyField *field;
     GwySurface *surface;
 } ModuleArgs;
 
@@ -53,16 +43,16 @@ typedef struct {
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             xyz_channels        (GwyContainer *data,
-                                             GwyRunType runtype);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static void             execute             (ModuleArgs *args);
 static GwyDialogOutcome run_gui             (ModuleArgs *args);
 static void             param_changed       (ModuleGUI *gui,
                                              gint id);
-static gboolean         xzdata_image_filter (GwyContainer *data,
+static gboolean         xzdata_image_filter (GwyFile *data,
                                              gint id,
                                              gpointer user_data);
-static gboolean         ydata_image_filter  (GwyContainer *data,
+static gboolean         ydata_image_filter  (GwyFile *data,
                                              gint id,
                                              gpointer user_data);
 
@@ -82,11 +72,11 @@ static gboolean
 module_register(void)
 {
     gwy_process_func_register("xyz_channels",
-                              (GwyProcessFunc)&xyz_channels,
+                              module_main,
                               N_("/_Basic Operations/XYZ from C_hannels..."),
                               NULL,
                               RUN_MODES,
-                              GWY_MENU_FLAG_DATA,
+                              GWY_MENU_FLAG_IMAGE,
                               N_("Convert three channels to XYZ data"));
 
     return TRUE;
@@ -109,16 +99,16 @@ define_module_params(void)
 }
 
 static void
-xyz_channels(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
     GwyDialogOutcome outcome;
     gint id, newid;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &args.field,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_FIELD, &args.field,
+                                 GWY_APP_FIELD_ID, &id,
+                                 0);
     g_return_if_fail(args.field);
     args.surface = gwy_surface_new();
 
@@ -131,8 +121,10 @@ xyz_channels(GwyContainer *data, GwyRunType runtype)
     if (outcome != GWY_DIALOG_HAVE_RESULT)
         execute(&args);
 
-    newid = gwy_app_data_browser_add_surface(args.surface, data, TRUE);
-    gwy_app_xyz_log_add(data, -1, newid, "proc::xyzize", NULL);
+    newid = gwy_file_add_xyz(data, args.surface);
+
+    gwy_file_set_visible(data, GWY_FILE_XYZ, newid, TRUE);
+    gwy_log_add_full(data, GWY_FILE_XYZ, -1, newid, "proc::xyzize", NULL);
 
 end:
     g_object_unref(args.surface);
@@ -175,56 +167,52 @@ param_changed(ModuleGUI *gui, gint id)
 }
 
 static gboolean
-xzdata_image_filter(GwyContainer *data, gint id, gpointer user_data)
+xzdata_image_filter(GwyFile *data, gint id, gpointer user_data)
 {
-    GwyDataField *field = (GwyDataField*)user_data;
-    GwyDataField *coordfield = gwy_container_get_object(data, gwy_app_get_data_key_for_id(id));
+    GwyField *field = (GwyField*)user_data;
+    GwyField *coordfield = gwy_file_get_image(data, id);
 
-    return !gwy_data_field_check_compatibility(coordfield, field,
-                                               GWY_DATA_COMPATIBILITY_RES
-                                               | GWY_DATA_COMPATIBILITY_REAL);
+    return !gwy_field_is_incompatible(coordfield, field, GWY_DATA_MISMATCH_RES | GWY_DATA_MISMATCH_REAL);
 }
 
 /* Y cooordinates must have the same units as X. */
 static gboolean
-ydata_image_filter(GwyContainer *data, gint id, gpointer user_data)
+ydata_image_filter(GwyFile *data, gint id, gpointer user_data)
 {
     GwyParams *params = (GwyParams*)user_data;
-    GwyDataField *xfield = gwy_params_get_image(params, PARAM_XDATA);
-    GwyDataField *yfield = gwy_container_get_object(data, gwy_app_get_data_key_for_id(id));
+    GwyField *xfield = gwy_params_get_image(params, PARAM_XDATA);
+    GwyField *yfield = gwy_file_get_image(data, id);
 
-    return !gwy_data_field_check_compatibility(yfield, xfield,
-                                               GWY_DATA_COMPATIBILITY_RES
-                                               | GWY_DATA_COMPATIBILITY_REAL
-                                               | GWY_DATA_COMPATIBILITY_VALUE);
+    return !gwy_field_is_incompatible(yfield, xfield,
+                                      GWY_DATA_MISMATCH_RES | GWY_DATA_MISMATCH_REAL | GWY_DATA_MISMATCH_VALUE);
 }
 
 static void
 execute(ModuleArgs *args)
 {
     GwyParams *params = args->params;
-    GwyDataField *xfield = gwy_params_get_image(params, PARAM_XDATA);
-    GwyDataField *yfield = gwy_params_get_image(params, PARAM_YDATA);
-    GwyDataField *zfield = gwy_params_get_image(params, PARAM_ZDATA);
+    GwyField *xfield = gwy_params_get_image(params, PARAM_XDATA);
+    GwyField *yfield = gwy_params_get_image(params, PARAM_YDATA);
+    GwyField *zfield = gwy_params_get_image(params, PARAM_ZDATA);
     GwySurface *surface = args->surface;
     const gdouble *xd, *yd, *zd;
     GwyXYZ *xyz;
     gint n, i;
 
-    n = gwy_data_field_get_xres(xfield)*gwy_data_field_get_yres(xfield);
+    n = gwy_field_get_xres(xfield)*gwy_field_get_yres(xfield);
     gwy_surface_resize(surface, n);
     xyz = gwy_surface_get_data(surface);
-    xd = gwy_data_field_get_data_const(xfield);
-    yd = gwy_data_field_get_data_const(yfield);
-    zd = gwy_data_field_get_data_const(zfield);
+    xd = gwy_field_get_data_const(xfield);
+    yd = gwy_field_get_data_const(yfield);
+    zd = gwy_field_get_data_const(zfield);
     for (i = 0; i < n; i++) {
         xyz[i].x = xd[i];
         xyz[i].y = yd[i];
         xyz[i].z = zd[i];
     }
 
-    gwy_si_unit_assign(gwy_surface_get_si_unit_xy(surface), gwy_data_field_get_si_unit_z(xfield));
-    gwy_si_unit_assign(gwy_surface_get_si_unit_z(surface), gwy_data_field_get_si_unit_z(zfield));
+    gwy_unit_assign(gwy_surface_get_unit_xy(surface), gwy_field_get_unit_z(xfield));
+    gwy_unit_assign(gwy_surface_get_unit_z(surface), gwy_field_get_unit_z(zfield));
 }
 
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
