@@ -1,5 +1,5 @@
 /*
- *  $Id: gwy.c 26773 2024-10-30 06:46:46Z yeti-dn $
+ *  $Id: gwy.c 27838 2025-04-17 09:08:59Z yeti-dn $
  *  Copyright (C) 2012-2016 David Necas (Yeti), Petr Klapetek, Jozef Vesely.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net, vesely@gjh.sk.
  *
@@ -21,7 +21,7 @@
 
 #include "config.h"
 #include <pygobject.h>
-#include <app/gwyapp.h>
+#include <libgwyapp/gwyapp.h>
 #include "wrap_calls.h"
 #include "pygwy.h"
 
