@@ -1,5 +1,5 @@
 /*
- *  $Id: pygwy-console.c 24708 2022-03-21 17:14:46Z yeti-dn $
+ *  $Id: pygwy-console.c 28808 2025-11-05 18:26:20Z yeti-dn $
  *  Copyright (C) 2008 Jan Horak
  *  E-mail: xhorak@gmail.com
  *  Copyright (C) 2014-2016 David Necas (Yeti).
@@ -28,11 +28,11 @@
 #include <stdio.h>
 #include <glib/gstdio.h>
 #include <gdk/gdkkeysyms.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-process.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
+#include <libgwyddion/macros.h>
+#include <libgwyui/icons.h>
+#include <libgwyapp/module-process.h>
+#include <libgwyapp/gwyapp.h>
+#include <libgwyapp/module-utils.h>
 
 #ifdef HAVE_GTKSOURCEVIEW
 #include <gtksourceview/gtksourceview.h>
@@ -66,7 +66,7 @@ static void       pygwy_console_save_as            (GtkToolButton *button,
 static void       pygwy_console_run                (GtkToolButton *button,
                                                     gpointer user_data);
 static void       pygwy_console                    (GwyContainer *data,
-                                                    GwyRunType run,
+                                                    GwyRunModeFlags mode,
                                                     const gchar *name);
 static void       pygwy_console_command_execute    (GtkEntry *entry,
                                                     gpointer user_data);
@@ -89,7 +89,7 @@ pygwy_register_console(void)
     gwy_process_func_register("pygwy_console",
                               pygwy_console,
                               N_("/Pygwy Console"),
-                              GWY_STOCK_PYGWY,
+                              GWY_ICON_PYGWY,
                               GWY_RUN_IMMEDIATE,
                               0,
                               N_("Python wrapper console"));
@@ -162,7 +162,7 @@ pygwy_console_create_gui(void)
     g_signal_connect(console_win, "key-press-event",
                      G_CALLBACK(key_pressed), console_setup);
 
-    vbox1 = gtk_vbox_new(FALSE, 0);
+    vbox1 = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
     gtk_container_add(GTK_CONTAINER(console_win), vbox1);
 
     /* Buttons. */
@@ -295,7 +295,7 @@ pygwy_console_create_gui(void)
 }
 
 static void
-pygwy_console(GwyContainer *data, GwyRunType run, const gchar *name)
+pygwy_console(GwyContainer *data, GwyRunModeFlags mode, const gchar *name)
 {
     PyObject *d;
 
