@@ -1,5 +1,5 @@
 /*
- *  $Id: pygwy.c 26748 2024-10-18 15:33:49Z yeti-dn $
+ *  $Id: pygwy.c 28808 2025-11-05 18:26:20Z yeti-dn $
  *  Copyright (C) 2004-2017 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -71,7 +71,7 @@ typedef struct {
     const gchar *menu_path;
     const gchar *icon;
     GwyMenuSensFlags sens;
-    GwyRunType run;
+    GwyRunModeFlags mode;
 
     /* Used dynamically during execution */
     PyObject *module;
@@ -82,17 +82,17 @@ typedef struct {
 static gboolean         module_register               (void);
 static gboolean         check_pygtk_availability      (void);
 static void             pygwy_procvolxyz_run          (GwyContainer *data,
-                                                       GwyRunType run,
+                                                       GwyRunModeFlags mode,
                                                        const gchar *name);
 static void             pygwy_graph_run               (GwyGraph *graph,
                                                        const gchar *name);
 static gboolean         pygwy_file_save_run           (GwyContainer *data,
                                                        const gchar *filename,
-                                                       GwyRunType mode,
+                                                       GwyRunModeFlags mode,
                                                        GError **error,
                                                        const gchar *name);
 static GwyContainer*    pygwy_file_load_run           (const gchar *filename,
-                                                       GwyRunType mode,
+                                                       GwyRunModeFlags mode,
                                                        GError **error,
                                                        const gchar *name);
 static gint             pygwy_file_detect_run         (const GwyFileDetectInfo *fileinfo,
@@ -525,7 +525,7 @@ pygwy_get_module_info(const gchar *fullpath, const gchar *filename)
     /* run mode and sensitivity are optional for something-processing modules */
     if (type_is_dataprocess) {
         if (info->type == PYGWY_PROCESS)
-            info->sens = GWY_MENU_FLAG_DATA;
+            info->sens = GWY_MENU_FLAG_IMAGE;
         else if (info->type == PYGWY_GRAPH)
             info->sens = GWY_MENU_FLAG_GRAPH;
         else if (info->type == PYGWY_VOLUME)
@@ -751,7 +751,7 @@ update_module_code(PygwyModuleInfo *info)
 }
 
 static void
-pygwy_procvolxyz_run(GwyContainer *data, GwyRunType run, const gchar *name)
+pygwy_procvolxyz_run(GwyContainer *data, GwyRunModeFlags mode, const gchar *name)
 {
     PygwyModuleInfo *info;
     PyObject *py_container = NULL, *res = NULL;
@@ -817,7 +817,7 @@ fail:
 }
 
 static gboolean
-pygwy_file_save_run(GwyContainer *data, const gchar *filename, GwyRunType mode,
+pygwy_file_save_run(GwyContainer *data, const gchar *filename, GwyRunModeFlags mode,
                     GError **error,
                     const gchar *name)
 {
@@ -856,7 +856,7 @@ fail:
 }
 
 static GwyContainer*
-pygwy_file_load_run(const gchar *filename, GwyRunType mode, GError **error,
+pygwy_file_load_run(const gchar *filename, GwyRunModeFlags mode, GError **error,
                     const gchar *name)
 {
     GwyContainer *container = NULL;
