@@ -1,5 +1,5 @@
 /*
- *  $Id: corrlen.c 26745 2024-10-18 15:17:53Z yeti-dn $
+ *  $Id: corrlen.c 29023 2025-12-19 13:06:16Z yeti-dn $
  *  Copyright (C) 2020-2024 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,22 +16,12 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwythreads.h>
-#include <libgwyddion/gwynlfitpreset.h>
-#include <libgwyddion/gwyresults.h>
-#include <libgwymodule/gwymodule-tool.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libprocess/correct.h>
-#include <libprocess/stats.h>
-#include <libprocess/linestats.h>
-#include <libgwydgets/gwystock.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
-#include "libgwyddion/gwyomp.h"
+#include <gwy.h>
+#include "libgwyddion/omp.h"
+#include "libgwyapp/sanity.h"
 
 /* Lower symmetric part indexing; i MUST be greater or equal than j */
 #define SLi(a, i, j) a[(i)*((i) + 1)/2 + (j)]
@@ -107,9 +97,6 @@ struct _GwyToolCorrLen {
     gint isel[4];
     gint isel_prev[4];
     gdouble rsel[4];
-
-    /* potential class data */
-    GType layer_type_rect;
 };
 
 struct _GwyToolCorrLenClass {
@@ -118,7 +105,7 @@ struct _GwyToolCorrLenClass {
 
 static gboolean     module_register                    (void);
 static GwyParamDef* define_module_params               (void);
-static GType        gwy_tool_corr_len_get_type         (void)                       G_GNUC_CONST;
+static GType        gwy_tool_corr_len_get_type         (void)                      G_GNUC_CONST;
 static void         gwy_tool_corr_len_finalize         (GObject *object);
 static void         gwy_tool_corr_len_init_dialog      (GwyToolCorrLen *tool);
 static void         gwy_tool_corr_len_data_switched    (GwyTool *gwytool,
@@ -171,7 +158,7 @@ static GwyParamDef*
 define_module_params(void)
 {
     static const GwyEnum levellings[] = {
-        { N_("line-leveling|None"), 0, },
+        { gwy_NC("line-leveling", "None"), 0, },
         { N_("Offset"),             1, },
         { N_("Tilt"),               2, },
         { N_("Bow"),                3, },
@@ -217,7 +204,7 @@ gwy_tool_corr_len_class_init(GwyToolCorrLenClass *klass)
 
     gobject_class->finalize = gwy_tool_corr_len_finalize;
 
-    tool_class->stock_id = GWY_STOCK_CORRELATION_LENGTH;
+    tool_class->icon_name = GWY_ICON_CORRELATION_LENGTH;
     tool_class->title = _("Correlation Length");
     tool_class->tooltip = _("Correlation Length");
     tool_class->prefix = "/module/corrlen";
@@ -237,10 +224,10 @@ gwy_tool_corr_len_finalize(GObject *object)
     GwyToolCorrLen *tool = GWY_TOOL_CORR_LEN(object);
 
     gwy_params_save_to_settings(tool->params);
-    GWY_OBJECT_UNREF(tool->params);
-    GWY_OBJECT_UNREF(tool->results);
-    GWY_OBJECT_UNREF(tool->gmodel_acf);
-    GWY_OBJECT_UNREF(tool->gmodel_psdf);
+    g_clear_object(&tool->params);
+    g_clear_object(&tool->results);
+    g_clear_object(&tool->gmodel_acf);
+    g_clear_object(&tool->gmodel_psdf);
 
     G_OBJECT_CLASS(gwy_tool_corr_len_parent_class)->finalize(object);
 }
@@ -250,17 +237,13 @@ gwy_tool_corr_len_init(GwyToolCorrLen *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
 
-    tool->layer_type_rect = gwy_plain_tool_check_layer_type(plain_tool, "GwyLayerRectangle");
-    if (!tool->layer_type_rect)
-        return;
-
     plain_tool->lazy_updates = TRUE;
-    plain_tool->unit_style = GWY_SI_UNIT_FORMAT_VFMARKUP;
+    plain_tool->unit_style = GWY_UNIT_FORMAT_VFMARKUP;
 
     tool->params = gwy_params_new_from_settings(define_module_params());
     tool->results = create_results();
 
-    gwy_plain_tool_connect_selection(plain_tool, tool->layer_type_rect, "rectangle");
+    gwy_plain_tool_connect_selection(plain_tool, GWY_TYPE_LAYER_RECTANGLE, "rectangle");
     memset(tool->isel_prev, 0xff, 4*sizeof(gint));
     gwy_plain_tool_enable_selection_holding(plain_tool);
 
@@ -272,26 +255,28 @@ gwy_tool_corr_len_rect_updated(GwyToolCorrLen *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
 
-    gwy_rect_selection_labels_select(tool->rlabels, plain_tool->selection, plain_tool->data_field);
+    gwy_rect_selection_labels_select(tool->rlabels, plain_tool->selection, plain_tool->field);
 }
 
 static void
 gwy_tool_corr_len_init_dialog(GwyToolCorrLen *tool)
 {
     GtkDialog *dialog = GTK_DIALOG(GWY_TOOL(tool)->dialog);
-    GtkWidget *hbox, *vbox, *image, *graph, *options;
+    GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
+    GtkWidget *hbox, *vbox, *graph, *options;
     GwyParamTable *table;
 
     tool->gmodel_acf = gwy_graph_model_new();
     tool->gmodel_psdf = gwy_graph_model_new();
     update_units(tool);
 
-    hbox = gwy_hbox_new(6);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 6);
     gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(dialog)), hbox, TRUE, TRUE, 0);
 
     /* Selection info */
-    vbox = gwy_vbox_new(0);
-    gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
+    gtk_widget_set_hexpand(vbox, FALSE);
+    gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, TRUE, 0);
 
     tool->rlabels = gwy_rect_selection_labels_new(TRUE, G_CALLBACK(gwy_tool_corr_len_rect_updated), tool);
     gtk_box_pack_start(GTK_BOX(vbox), gwy_rect_selection_labels_get_table(tool->rlabels), FALSE, FALSE, 0);
@@ -310,7 +295,7 @@ gwy_tool_corr_len_init_dialog(GwyToolCorrLen *tool)
     gwy_param_table_append_combo(table, PARAM_ORIENTATION);
     gwy_param_table_append_checkbox(table, PARAM_INSTANT_UPDATE);
     gwy_param_table_append_hold_selection(table, PARAM_HOLD_SELECTION);
-    gwy_plain_tool_add_param_table(GWY_PLAIN_TOOL(tool), table);
+    gwy_plain_tool_add_param_table(plain_tool, table);
     gtk_container_add(GTK_CONTAINER(options), gwy_param_table_widget(table));
 
     /* Results */
@@ -333,11 +318,11 @@ gwy_tool_corr_len_init_dialog(GwyToolCorrLen *tool)
     gwy_param_table_append_header(table, -1, _("Output"));
     gwy_param_table_append_combo(table, PARAM_OUTPUT);
     gwy_param_table_append_target_graph(table, PARAM_TARGET_GRAPH, tool->gmodel_acf);
-    gwy_plain_tool_add_param_table(GWY_PLAIN_TOOL(tool), table);
+    gwy_plain_tool_add_param_table(plain_tool, table);
     gtk_box_pack_start(GTK_BOX(hbox), gwy_param_table_widget(table), FALSE, FALSE, 0);
 
     /* Graphs */
-    vbox = gwy_vbox_new(0);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
     gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);
 
     graph = gwy_graph_new(tool->gmodel_acf);
@@ -348,13 +333,10 @@ gwy_tool_corr_len_init_dialog(GwyToolCorrLen *tool)
     gwy_graph_enable_user_input(GWY_GRAPH(graph), FALSE);
     gtk_box_pack_start(GTK_BOX(vbox), graph, TRUE, TRUE, 2);
 
-    tool->update = gtk_dialog_add_button(dialog, _("_Update"), GWY_TOOL_RESPONSE_UPDATE);
-    image = gtk_image_new_from_stock(GTK_STOCK_EXECUTE, GTK_ICON_SIZE_BUTTON);
-    gtk_button_set_image(GTK_BUTTON(tool->update), image);
-    gwy_plain_tool_add_clear_button(GWY_PLAIN_TOOL(tool));
+    tool->update = gwy_add_button_to_dialog(dialog, _("_Update"), GWY_ICON_GTK_EXECUTE, GWY_RESPONSE_UPDATE);
+    gwy_plain_tool_add_clear_button(plain_tool);
     gwy_tool_add_hide_button(GWY_TOOL(tool), TRUE);
-    gtk_dialog_add_button(dialog, GTK_STOCK_APPLY, GTK_RESPONSE_APPLY);
-    gtk_dialog_set_default_response(dialog, GTK_RESPONSE_APPLY);
+    gwy_plain_tool_add_apply_button(plain_tool);
     gtk_dialog_set_response_sensitive(dialog, GTK_RESPONSE_APPLY, FALSE);
     gwy_help_add_to_tool_dialog(dialog, GWY_TOOL(tool), GWY_HELP_DEFAULT);
 
@@ -381,7 +363,7 @@ gwy_tool_corr_len_data_switched(GwyTool *gwytool,
 
     gwy_param_table_set_sensitive(tool->table_results, PARAM_REPORT_STYLE, FALSE);
     if (data_view) {
-        gwy_object_set_or_reset(plain_tool->layer, tool->layer_type_rect,
+        gwy_object_set_or_reset(plain_tool->layer, GWY_TYPE_LAYER_RECTANGLE,
                                 "editable", TRUE,
                                 "focus", -1,
                                 NULL);
@@ -397,12 +379,12 @@ static void
 update_units(GwyToolCorrLen *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    GwyDataField *field = plain_tool->data_field;
-    GwySIUnit *xyunit, *zunit, *unit = gwy_si_unit_new(NULL);
+    GwyField *field = plain_tool->field;
+    GwyUnit *xyunit, *zunit, *unit = gwy_unit_new(NULL);
 
     if (!field) {
-        g_object_set(tool->gmodel_psdf, "si-unit-x", unit, "si-unit-y", unit, NULL);
-        g_object_set(tool->gmodel_acf, "si-unit-x", unit, "si-unit-y", unit, NULL);
+        g_object_set(tool->gmodel_psdf, "unit-x", unit, "unit-y", unit, NULL);
+        g_object_set(tool->gmodel_acf, "unit-x", unit, "unit-y", unit, NULL);
         gwy_results_set_unit(tool->results, "x", unit);
         gwy_results_set_unit(tool->results, "y", unit);
         gwy_results_set_unit(tool->results, "z", unit);
@@ -410,20 +392,20 @@ update_units(GwyToolCorrLen *tool)
         return;
     }
 
-    xyunit = gwy_data_field_get_si_unit_xy(field);
-    zunit = gwy_data_field_get_si_unit_z(field);
+    xyunit = gwy_field_get_unit_xy(field);
+    zunit = gwy_field_get_unit_z(field);
 
     gwy_results_set_unit(tool->results, "x", xyunit);
     gwy_results_set_unit(tool->results, "y", xyunit);
     gwy_results_set_unit(tool->results, "z", zunit);
 
-    g_object_set(tool->gmodel_acf, "si-unit-x", xyunit, NULL);
-    gwy_si_unit_power(zunit, 2, unit);
-    g_object_set(tool->gmodel_acf, "si-unit-y", unit, NULL);
-    gwy_si_unit_power(xyunit, -1, unit);
-    g_object_set(tool->gmodel_psdf, "si-unit-x", unit, NULL);
-    gwy_si_unit_power_multiply(xyunit, 1, zunit, 2, unit);
-    g_object_set(tool->gmodel_psdf, "si-unit-y", unit, NULL);
+    g_object_set(tool->gmodel_acf, "unit-x", xyunit, NULL);
+    gwy_unit_power(zunit, 2, unit);
+    g_object_set(tool->gmodel_acf, "unit-y", unit, NULL);
+    gwy_unit_power(xyunit, -1, unit);
+    g_object_set(tool->gmodel_psdf, "unit-x", unit, NULL);
+    gwy_unit_power_multiply(xyunit, 1, zunit, 2, unit);
+    g_object_set(tool->gmodel_psdf, "unit-y", unit, NULL);
 
     g_object_unref(unit);
 }
@@ -433,7 +415,7 @@ update_selected_rectangle(GwyToolCorrLen *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
     GwySelection *selection = plain_tool->selection;
-    GwyDataField *field = plain_tool->data_field;
+    GwyField *field = plain_tool->field;
     gint n = selection ? gwy_selection_get_data(selection, NULL) : 0;
 
     gwy_rect_selection_labels_fill(tool->rlabels, n == 1 ? selection : NULL, field, NULL, tool->isel);
@@ -450,7 +432,7 @@ param_changed(GwyToolCorrLen *tool, gint id)
     if (id == PARAM_INSTANT_UPDATE)
         do_update = do_update || gwy_params_get_boolean(params, PARAM_INSTANT_UPDATE);
     if (id == PARAM_MASKING) {
-        do_update = do_update || (GWY_PLAIN_TOOL(tool)->data_field && GWY_PLAIN_TOOL(tool)->mask_field);
+        do_update = do_update || (GWY_PLAIN_TOOL(tool)->field && GWY_PLAIN_TOOL(tool)->mask_field);
     }
     if (id < 0 || id == PARAM_INSTANT_UPDATE)
         update_sensitivity(tool);
@@ -492,7 +474,7 @@ gwy_tool_corr_len_response(GwyTool *tool,
 
     if (response_id == GTK_RESPONSE_APPLY)
         gwy_tool_corr_len_apply(GWY_TOOL_CORR_LEN(tool));
-    else if (response_id == GWY_TOOL_RESPONSE_UPDATE)
+    else if (response_id == GWY_RESPONSE_UPDATE)
         update_labels(GWY_TOOL_CORR_LEN(tool));
 }
 
@@ -534,8 +516,10 @@ gwy_tool_corr_len_apply(GwyToolCorrLen *tool)
 
     if ((gmodel = gwy_params_get_graph(params, PARAM_TARGET_GRAPH)))
         gwy_graph_model_append_curves(gmodel, outgmodel, 1);
-    else
-        gwy_app_data_browser_add_graph_model(outgmodel, plain_tool->container, TRUE);
+    else {
+        gint newid = gwy_file_add_graph(GWY_FILE(plain_tool->container), outgmodel);
+        gwy_file_set_visible(GWY_FILE(plain_tool->container), GWY_FILE_GRAPH, newid, TRUE);
+    }
 
     g_object_unref(outgmodel);
 }
@@ -548,7 +532,7 @@ update_labels(GwyToolCorrLen *tool)
     guint i;
 
     gtk_dialog_set_response_sensitive(GTK_DIALOG(GWY_TOOL(tool)->dialog), GTK_RESPONSE_APPLY, FALSE);
-    if (!plain_tool->data_field) {
+    if (!plain_tool->field) {
         for (i = 0; i < G_N_ELEMENTS(results_ids); i++)
             gwy_param_table_results_clear(tool->table_results, results_ids[i]);
         gwy_graph_model_remove_all_curves(tool->gmodel_acf);
@@ -568,10 +552,10 @@ update_labels(GwyToolCorrLen *tool)
 }
 
 static gdouble
-find_decay_point(GwyDataLine *line, gdouble q)
+find_decay_point(GwyLine *line, gdouble q)
 {
-    const gdouble *d = gwy_data_line_get_data(line);
-    guint res = gwy_data_line_get_res(line);
+    const gdouble *d = gwy_line_get_data(line);
+    guint res = gwy_line_get_res(line);
     gdouble max = d[0];
     gdouble threshold = q*max;
     guint i;
@@ -580,12 +564,12 @@ find_decay_point(GwyDataLine *line, gdouble q)
     for (i = 1; i < res; i++) {
         if (d[i] <= threshold) {
             if (d[i] == threshold)
-                return gwy_data_line_itor(line, i);
+                return gwy_line_itor(line, i);
 
             v0 = d[i-1] - threshold;
             v1 = d[i] - threshold;
             t = v0/(v0 - v1);
-            return gwy_data_line_itor(line, i-1 + t);
+            return gwy_line_itor(line, i-1 + t);
         }
     }
 
@@ -593,14 +577,14 @@ find_decay_point(GwyDataLine *line, gdouble q)
 }
 
 static void
-fit_psdf(GwyDataLine *psdf, const gchar *presetname, gdouble T_estim,
-         gdouble *sigma, gdouble *T, GwyDataLine *fit)
+fit_psdf(GwyLine *psdf, const gchar *presetname, gdouble T_estim,
+         gdouble *sigma, gdouble *T, GwyLine *fit)
 {
     GwyNLFitPreset *preset = gwy_inventory_get_item(gwy_nlfit_presets(), presetname);
-    guint nfit, i, res = gwy_data_line_get_res(psdf);
-    const gdouble *ydata = gwy_data_line_get_data(psdf);
+    guint nfit, i, res = gwy_line_get_res(psdf);
+    const gdouble *ydata = gwy_line_get_data(psdf);
     gdouble *xdata, *fitdata, *xdata_all;
-    gdouble t = 0.0, s = gwy_data_line_get_sum(psdf);
+    gdouble t = 0.0, s = gwy_line_sum(psdf);
     GwyNLFitter *fitter;
     gdouble params[2], errors[2];
     gboolean ok;
@@ -610,7 +594,7 @@ fit_psdf(GwyDataLine *psdf, const gchar *presetname, gdouble T_estim,
         if (t > 0.999*s)
             break;
     }
-    xdata = xdata_all = gwy_math_linspace(NULL, res, 0.0, gwy_data_line_get_dx(psdf));
+    xdata = xdata_all = gwy_math_linspace(NULL, res, 0.0, gwy_line_get_dx(psdf));
 
     /* Try to skip the smallest frequencies.  Unfortunately, we cannot do that for tiny data. */
     for (i = 0; i < 4; i++) {
@@ -621,13 +605,13 @@ fit_psdf(GwyDataLine *psdf, const gchar *presetname, gdouble T_estim,
         }
     }
 
-    params[0] = sqrt(s*gwy_data_line_get_dx(psdf));
+    params[0] = sqrt(s*gwy_line_get_dx(psdf));
     params[1] = T_estim;
     fitter = gwy_nlfit_preset_fit(preset, NULL, nfit, xdata, ydata, params, errors, NULL);
     *sigma = (gwy_math_nlfit_succeeded(fitter) ? params[0] : -1.0);
     *T = (gwy_math_nlfit_succeeded(fitter) ? params[1] : -1.0);
 
-    fitdata = gwy_data_line_get_data(fit);
+    fitdata = gwy_line_get_data(fit);
     for (i = 0; i < res; i++)
         fitdata[i] = gwy_nlfit_preset_get_value(preset, xdata_all[i], params, &ok);
 
@@ -652,15 +636,15 @@ matrix_multiply_right(const gdouble *v, const gdouble *A,
 }
 
 static void
-acf_extrapolate_correction(GwyDataLine *acf, GwyDataLine *corrected_acf)
+acf_extrapolate_correction(GwyLine *acf, GwyLine *corrected_acf)
 {
-    const gdouble *a = gwy_data_line_get_data(acf);
-    const gdouble *c = gwy_data_line_get_data(corrected_acf);
-    gint ares = gwy_data_line_get_res(acf);
-    gint cres = gwy_data_line_get_res(corrected_acf);
+    const gdouble *a = gwy_line_get_data(acf);
+    const gdouble *c = gwy_line_get_data(corrected_acf);
+    gint ares = gwy_line_get_res(acf);
+    gint cres = gwy_line_get_res(corrected_acf);
     gdouble diff = c[cres-1] - a[cres-1];
-    GwyDataLine *extended = gwy_data_line_new_alike(acf, FALSE);
-    gdouble *e = gwy_data_line_get_data(extended);
+    GwyLine *extended = gwy_line_new_alike(acf, FALSE);
+    gdouble *e = gwy_line_get_data(extended);
     gint i;
 
     gwy_assign(e, c, cres);
@@ -668,21 +652,21 @@ acf_extrapolate_correction(GwyDataLine *acf, GwyDataLine *corrected_acf)
     for (i = cres; i < ares; i++)
         e[i] = a[i] + diff;
 
-    gwy_data_line_assign(corrected_acf, extended);
+    gwy_line_assign(corrected_acf, extended);
     g_object_unref(extended);
 }
 
-static GwyDataLine*
-acf_selfcorrect(GwyDataLine *acf, gint degree, gint N, gint K)
+static GwyLine*
+acf_selfcorrect(GwyLine *acf, gint degree, gint N, gint K)
 {
-    GwyDataLine *acf_corr = gwy_data_line_duplicate(acf);
+    GwyLine *acf_corr = gwy_line_copy(acf);
     gdouble *x, *A, *ATA, *rhs;
     gdouble qnN;
     guint m, j, k, ATA_size;
     gint n;
 
-    gwy_data_line_resize(acf_corr, 0, K);
-    x = gwy_data_line_get_data(acf_corr);
+    gwy_line_crop(acf_corr, 0, K);
+    x = gwy_line_get_data(acf_corr);
 
     /**
      * Construct the matrix. It is
@@ -770,8 +754,8 @@ corrected_exponential_fitfunc(guint i,
 }
 
 static gboolean
-fit_acf_with_bias_corrected_model(GwyNLFitIdxFunc func, GwyDataLine *acf, gint degree, gint N, gint K,
-                                  gdouble *sigma, gdouble *T, GwyDataLine *fit)
+fit_acf_with_bias_corrected_model(GwyNLFitIdxFunc func, GwyLine *acf, gint degree, gint N, gint K,
+                                  gdouble *sigma, gdouble *T, GwyLine *fit)
 {
     GwyNLFitter *fitter;
     CorrectedACFModelData model;
@@ -780,7 +764,7 @@ fit_acf_with_bias_corrected_model(GwyNLFitIdxFunc func, GwyDataLine *acf, gint d
     gdouble *f, *zeros;
     gint i, res;
 
-    g_return_val_if_fail(GWY_IS_DATA_LINE(acf), FALSE);
+    g_return_val_if_fail(GWY_IS_LINE(acf), FALSE);
     /* Allow -1 to get uncorrected fits using the same function. */
     g_return_val_if_fail(degree >= -1, FALSE);
     g_return_val_if_fail(N > 0, FALSE);
@@ -789,8 +773,8 @@ fit_acf_with_bias_corrected_model(GwyNLFitIdxFunc func, GwyDataLine *acf, gint d
     if (!sigma && !T)
         return TRUE;
 
-    model.acf = gwy_data_line_get_data(acf);
-    model.dx = gwy_data_line_get_dx(acf);
+    model.acf = gwy_line_get_data(acf);
+    model.dx = gwy_line_get_dx(acf);
     model.degree = degree;
     model.N = N;
 
@@ -808,10 +792,10 @@ fit_acf_with_bias_corrected_model(GwyNLFitIdxFunc func, GwyDataLine *acf, gint d
     if (T)
         *T = fabs(param[1]);
 
-    res = gwy_data_line_get_res(acf);
+    res = gwy_line_get_res(acf);
     model.acf = zeros = g_new0(gdouble, res);
-    gwy_data_line_assign(fit, acf);
-    f = gwy_data_line_get_data(fit);
+    gwy_line_assign(fit, acf);
+    f = gwy_line_get_data(fit);
     for (i = 0; i < res; i++)
         f[i] = func(i, param, &model, &success);
 
@@ -821,7 +805,7 @@ fit_acf_with_bias_corrected_model(GwyNLFitIdxFunc func, GwyDataLine *acf, gint d
 }
 
 static GwyGraphCurveModel*
-make_gcmodel(GwyDataLine *line, const gchar *desc, gint icolor)
+make_gcmodel(GwyLine *line, const gchar *desc, gint icolor)
 {
     GwyGraphCurveModel *gcmodel = gwy_graph_curve_model_new();
 
@@ -830,7 +814,7 @@ make_gcmodel(GwyDataLine *line, const gchar *desc, gint icolor)
                  "description", desc,
                  "color", gwy_graph_get_preset_color(icolor),
                  NULL);
-    gwy_graph_curve_model_set_data_from_dataline(gcmodel, line, 0, 0);
+    gwy_graph_curve_model_set_data_from_line(gcmodel, line, 0, 0);
 
     return gcmodel;
 }
@@ -854,17 +838,17 @@ calculate(GwyToolCorrLen *tool)
 
     GwyParams *params = tool->params;
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    GwyDataField *field = plain_tool->data_field;
-    GwyDataField *mask = plain_tool->mask_field;
+    GwyField *field = plain_tool->field;
+    GwyField *mask = plain_tool->mask_field;
     GwyMaskingType masking = gwy_params_get_masking(params, PARAM_MASKING, &mask);
     GwyOrientation orientation = gwy_params_get_enum(params, PARAM_ORIENTATION);
     CorrlenModelType model = gwy_params_get_enum(params, PARAM_MODEL);
     gint level = gwy_params_get_int(params, PARAM_LEVEL);
-    GwyDataField *field_to_use, *mask_to_use;
+    GwyField *field_to_use, *mask_to_use;
     GwyGraphModel *gmodel;
     GwyGraphCurveModel *gcmodel;
     GwyResults *results = tool->results;
-    GwyDataLine *psdf, *acf, *fit, *acf_corr;
+    GwyLine *psdf, *acf, *fit, *acf_corr;
     gdouble xoff, yoff;
     gdouble L, T_estim, acf_1e, acf_0, T_corr, sigma, sigma_corr;
     gdouble T_acf, sigma_acf, T_psdf, sigma_psdf;
@@ -883,50 +867,50 @@ calculate(GwyToolCorrLen *tool)
     if (w < 4 || h < 4)
         return FALSE;
 
-    xoff = gwy_data_field_get_xoffset(field);
-    yoff = gwy_data_field_get_yoffset(field);
+    xoff = gwy_field_get_xoffset(field);
+    yoff = gwy_field_get_yoffset(field);
 
     mask_to_use = NULL;
     if (orientation == GWY_ORIENTATION_VERTICAL) {
-        field_to_use = gwy_data_field_new(1, 1, 1, 1, FALSE);
-        gwy_data_field_area_flip_xy(field, col, row, w, h, field_to_use, FALSE);
+        field_to_use = gwy_field_new(1, 1, 1, 1, FALSE);
+        gwy_field_area_transpose(field, col, row, w, h, field_to_use, FALSE);
         if (mask && masking != GWY_MASK_IGNORE) {
-            mask_to_use = gwy_data_field_new(1, 1, 1, 1, FALSE);
-            gwy_data_field_area_flip_xy(mask, col, row, w, h, mask_to_use, FALSE);
+            mask_to_use = gwy_field_new(1, 1, 1, 1, FALSE);
+            gwy_field_area_transpose(mask, col, row, w, h, mask_to_use, FALSE);
         }
     }
     else {
-        field_to_use = gwy_data_field_area_extract(field, col, row, w, h);
+        field_to_use = gwy_field_area_extract(field, col, row, w, h);
         if (mask && masking != GWY_MASK_IGNORE)
-            mask_to_use = gwy_data_field_area_extract(mask, col, row, w, h);
+            mask_to_use = gwy_field_area_extract(mask, col, row, w, h);
     }
-    fxres = gwy_data_field_get_xres(field_to_use);
-    fyres = gwy_data_field_get_yres(field_to_use);
-    L = w*gwy_data_field_get_dx(field_to_use);
+    fxres = gwy_field_get_xres(field_to_use);
+    fyres = gwy_field_get_yres(field_to_use);
+    L = w*gwy_field_get_dx(field_to_use);
 
     /* level=0 means really do nothing with individual lines */
     if (level)
-        gwy_data_field_row_level_poly(field_to_use, mask_to_use, masking, level-1, NULL);
-    gwy_data_field_add(field_to_use, -gwy_data_field_get_avg(field));
+        gwy_field_row_level_poly(field_to_use, mask_to_use, masking, level-1, NULL);
+    gwy_field_add(field_to_use, -gwy_field_get_avg(field));
 
     /************************************************************************
      *
      * ACF-based calculations.
      *
      ************************************************************************/
-    acf = gwy_data_field_area_row_acf(field_to_use, mask_to_use, masking, 0, 0, fxres, fyres, 0, NULL);
-    sigma = sqrt(gwy_data_line_get_val(acf, 0));
-    acfres = gwy_data_line_get_res(acf);
+    acf = gwy_field_area_row_acf(field_to_use, mask_to_use, masking, 0, 0, fxres, fyres, 0, NULL);
+    sigma = sqrt(gwy_line_get_val(acf, 0));
+    acfres = gwy_line_get_res(acf);
     acf_1e = find_decay_point(acf, exp(-1.0));
     acf_0 = find_decay_point(acf, 0.0);
     if (acf_0 < 0.0)
-        acf_0 = gwy_data_line_get_real(acf);
-    ncut = 5*gwy_data_line_rtoi(acf, acf_0);
+        acf_0 = gwy_line_get_real(acf);
+    ncut = 5*gwy_line_rtoi(acf, acf_0);
     ncut = MIN(acfres, ncut);
-    K = gwy_data_line_rtoi(acf, 1.15*acf_0);
+    K = gwy_line_rtoi(acf, 1.15*acf_0);
     K = MIN(acfres, K);
 
-    fit = gwy_data_line_new_alike(acf, FALSE);
+    fit = gwy_line_new_alike(acf, FALSE);
     ok = FALSE;
     /* Fit models take the degree, which is not our ‘level’ parameter! */
     if (model == CORRLEN_MODEL_GAUSS) {
@@ -944,9 +928,9 @@ calculate(GwyToolCorrLen *tool)
 
     /* acf_selfcorrect() takes the degree, which is not our ‘level’ parameter! */
     acf_corr = acf_selfcorrect(acf, level-1, acfres, K);
-    sigma_corr = sqrt(gwy_data_line_get_val(acf_corr, 0));
-    gwy_data_line_resize(acf, 0, ncut);
-    gwy_data_line_resize(fit, 0, ncut);
+    sigma_corr = sqrt(gwy_line_get_val(acf_corr, 0));
+    gwy_line_crop(acf, 0, ncut);
+    gwy_line_crop(fit, 0, ncut);
     acf_extrapolate_correction(acf, acf_corr);
 
     T_corr = find_decay_point(acf_corr, exp(-1.0));
@@ -968,7 +952,7 @@ calculate(GwyToolCorrLen *tool)
 
     if (T_corr > 0.0) {
         gcmodel = make_gcmodel(fit, _("Biased fit"), 1);
-        g_object_set(gcmodel, "line-style", GDK_LINE_ON_OFF_DASH, NULL);
+        g_object_set(gcmodel, "line-style", GWY_GRAPH_LINE_DASH, NULL);
         gwy_graph_model_add_curve(gmodel, gcmodel);
         g_object_unref(gcmodel);
     }
@@ -984,8 +968,8 @@ calculate(GwyToolCorrLen *tool)
      ************************************************************************/
     model_name = gwy_enum_to_string(model, psdf_fitfunc_names, G_N_ELEMENTS(psdf_fitfunc_names));
     g_assert(model_name && *model_name);
-    psdf = gwy_data_field_area_row_psdf(field_to_use, mask_to_use, masking, 0, 0, fxres, fyres, GWY_WINDOWING_HANN, 0);
-    fit = gwy_data_line_new_alike(psdf, FALSE);
+    psdf = gwy_field_area_row_psdf(field_to_use, mask_to_use, masking, 0, 0, fxres, fyres, GWY_WINDOWING_HANN, 0);
+    fit = gwy_line_new_alike(psdf, FALSE);
     T_estim = (acf_1e > 0.0 ? acf_1e : 0.05*L);
     fit_psdf(psdf, model_name, T_estim, &sigma_psdf, &T_psdf, fit);
 
@@ -1000,7 +984,7 @@ calculate(GwyToolCorrLen *tool)
     gwy_graph_model_add_curve(gmodel, gcmodel);
     if (T_psdf > 0.0) {
         gcmodel = make_gcmodel(fit, _("fit"), 1);
-        g_object_set(gcmodel, "line-style", GDK_LINE_ON_OFF_DASH, NULL);
+        g_object_set(gcmodel, "line-style", GWY_GRAPH_LINE_DASH, NULL);
         gwy_graph_model_add_curve(gmodel, gcmodel);
     }
     g_object_unref(gcmodel);
@@ -1047,12 +1031,12 @@ calculate(GwyToolCorrLen *tool)
     fill_result_if_positive(results, "sigma_acf", sigma_acf);
     fill_result_if_positive(results, "sigma_corr", sigma_corr);
 
-    gwy_results_fill_filename(results, "file", plain_tool->container);
-    gwy_results_fill_channel(results, "image", plain_tool->container, plain_tool->id);
+    gwy_results_fill_filename(results, "file", GWY_FILE(plain_tool->container));
+    gwy_results_fill_data_name(results, "image", GWY_FILE(plain_tool->container), GWY_FILE_IMAGE, plain_tool->id);
 
     gwy_param_table_set_sensitive(tool->table_results, PARAM_REPORT_STYLE, TRUE);
 
-    GWY_OBJECT_UNREF(mask_to_use);
+    g_clear_object(&mask_to_use);
     g_object_unref(field_to_use);
 
     update_target_graphs(tool);
