@@ -1,5 +1,5 @@
 /*
- *  $Id: cprofile.c 26745 2024-10-18 15:17:53Z yeti-dn $
+ *  $Id: cprofile.c 28788 2025-11-04 17:13:02Z yeti-dn $
  *  Copyright (C) 2019-2022 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,16 +16,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwymodule/gwymodule-tool.h>
-#include <libprocess/datafield.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwydgets/gwynullstore.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 
 enum {
     COLUMN_I, COLUMN_X, COLUMN_Y, NCOLUMNS
@@ -41,13 +35,6 @@ enum {
     PARAM_TARGET_GRAPH,
 };
 
-typedef enum {
-    CPROFILE_MODE_CROSS      = 0,
-    CPROFILE_MODE_HORIZONTAL = 1,
-    CPROFILE_MODE_VERTICAL   = 2,
-    CPROFILE_NMODES,
-} CprofileMode;
-
 #define GWY_TYPE_TOOL_CPROFILE            (gwy_tool_cprofile_get_type())
 #define GWY_TOOL_CPROFILE(obj)            (G_TYPE_CHECK_INSTANCE_CAST((obj), GWY_TYPE_TOOL_CPROFILE, GwyToolCprofile))
 #define GWY_IS_TOOL_CPROFILE(obj)         (G_TYPE_CHECK_INSTANCE_TYPE((obj), GWY_TYPE_TOOL_CPROFILE))
@@ -71,8 +58,7 @@ struct _GwyToolCprofile {
     GwyParamTable *table;
 
     /* potential class data */
-    GwySIValueFormat *pixel_format;
-    GType layer_type_cross;
+    GwyValueFormat *pixel_format;
 };
 
 struct _GwyToolCprofileClass {
@@ -134,9 +120,10 @@ static GwyParamDef*
 define_module_params(void)
 {
     static const GwyEnum modes[] = {
-        { N_("Cross"),      CPROFILE_MODE_CROSS,      },
-        { N_("Horizontal"), CPROFILE_MODE_HORIZONTAL, },
-        { N_("Vertical"),   CPROFILE_MODE_VERTICAL,   },
+        /* NB: These are not swapped. Show the direction of the lines to the user, not the coordinate she selects. */
+        { N_("Vertical"),   GWY_DIRECTION_HORIZONTAL, },
+        { N_("Horizontal"), GWY_DIRECTION_VERTICAL,   },
+        { N_("Cross"),      GWY_DIRECTION_BOTH,       },
     };
     static GwyParamDef *paramdef = NULL;
 
@@ -146,7 +133,7 @@ define_module_params(void)
     paramdef = gwy_param_def_new();
     gwy_param_def_set_function_name(paramdef, "cprofile");
     gwy_param_def_add_gwyenum(paramdef, PARAM_MODE, "mode", _("_Mode"),
-                              modes, G_N_ELEMENTS(modes), CPROFILE_MODE_CROSS);
+                              modes, G_N_ELEMENTS(modes), GWY_DIRECTION_BOTH);
     gwy_param_def_add_enum(paramdef, PARAM_MASKING, "masking", NULL, GWY_TYPE_MASKING_TYPE, GWY_MASK_IGNORE);
     gwy_param_def_add_int(paramdef, PARAM_THICKNESS, "thickness", _("_Thickness"), 1, 128, 1);
     gwy_param_def_add_boolean(paramdef, PARAM_ZERO_CROSS, "zero_cross", _("Cross at _zero"), TRUE);
@@ -166,7 +153,7 @@ gwy_tool_cprofile_class_init(GwyToolCprofileClass *klass)
 
     gobject_class->finalize = gwy_tool_cprofile_finalize;
 
-    tool_class->stock_id = GWY_STOCK_CROSS_PROFILE;
+    tool_class->icon_name = GWY_ICON_CROSS_PROFILE;
     tool_class->title = _("Profiles Along Axes");
     tool_class->tooltip = _("Read horizontal and/or vertical profiles");
     tool_class->prefix = "/module/cprofile";
@@ -185,16 +172,16 @@ gwy_tool_cprofile_finalize(GObject *object)
     GwyToolCprofile *tool = GWY_TOOL_CPROFILE(object);
 
     gwy_params_save_to_settings(tool->params);
-    GWY_OBJECT_UNREF(tool->params);
+    g_clear_object(&tool->params);
     if (tool->xydata)
         g_array_free(tool->xydata, TRUE);
     if (tool->model) {
         gtk_tree_view_set_model(tool->treeview, NULL);
-        GWY_OBJECT_UNREF(tool->model);
+        g_clear_object(&tool->model);
     }
-    GWY_OBJECT_UNREF(tool->gmodel);
-    GWY_OBJECT_UNREF(tool->colorpixbuf);
-    GWY_SI_VALUE_FORMAT_FREE(tool->pixel_format);
+    g_clear_object(&tool->gmodel);
+    g_clear_object(&tool->colorpixbuf);
+    GWY_FREE_VALUE_FORMAT(tool->pixel_format);
 
     G_OBJECT_CLASS(gwy_tool_cprofile_parent_class)->finalize(object);
 }
@@ -205,21 +192,18 @@ gwy_tool_cprofile_init(GwyToolCprofile *tool)
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
     gint width, height;
 
-    tool->layer_type_cross = gwy_plain_tool_check_layer_type(plain_tool, "GwyLayerCross");
-    if (!tool->layer_type_cross)
-        return;
-
-    plain_tool->unit_style = GWY_SI_UNIT_FORMAT_MARKUP;
+    plain_tool->unit_style = GWY_UNIT_FORMAT_MARKUP;
     plain_tool->lazy_updates = TRUE;
 
     tool->params = gwy_params_new_from_settings(define_module_params());
-    tool->pixel_format = gwy_si_unit_value_format_new(1.0, 0, _("px"));
+    tool->pixel_format = gwy_value_format_new(1.0, 0, _("px"));
+    tool->model = GTK_TREE_MODEL(gwy_null_store_new(0));
 
     gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &width, &height);
     height |= 1;
     tool->colorpixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, 8, height, height);
 
-    gwy_plain_tool_connect_selection(plain_tool, tool->layer_type_cross, "cross");
+    gwy_plain_tool_connect_selection(plain_tool, GWY_TYPE_LAYER_CROSS, "cross");
     gwy_plain_tool_enable_selection_holding(plain_tool);
 
     gwy_tool_cprofile_init_dialog(tool);
@@ -236,22 +220,20 @@ gwy_tool_cprofile_init_dialog(GwyToolCprofile *tool)
     GtkCellRenderer *renderer;
     GtkWidget *scwin, *label, *hbox, *vbox, *options, *graph;
     GwyParamTable *table;
-    GwyNullStore *store;
     guint i;
 
     tool->gmodel = gwy_graph_model_new();
     g_object_set(tool->gmodel, "title", _("Profiles"), "label-visible", FALSE, NULL);
 
-    hbox = gwy_hbox_new(4);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
     gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(dialog)), hbox, TRUE, TRUE, 0);
 
     /* Left pane */
-    vbox = gwy_vbox_new(8);
-    gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 8);
+    gtk_widget_set_hexpand(vbox, FALSE);
+    gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, TRUE, 0);
 
     /* Line coordinates */
-    store = gwy_null_store_new(0);
-    tool->model = GTK_TREE_MODEL(store);
     tool->treeview = GTK_TREE_VIEW(gtk_tree_view_new_with_model(tool->model));
     gwy_plain_tool_enable_object_deletion(GWY_PLAIN_TOOL(tool), tool->treeview);
 
@@ -304,7 +286,7 @@ gwy_tool_cprofile_init_dialog(GwyToolCprofile *tool)
 
     gwy_plain_tool_add_clear_button(GWY_PLAIN_TOOL(tool));
     gwy_tool_add_hide_button(GWY_TOOL(tool), FALSE);
-    gtk_dialog_add_button(dialog, GTK_STOCK_APPLY, GTK_RESPONSE_APPLY);
+    gwy_plain_tool_add_apply_button(GWY_PLAIN_TOOL(tool));
     gtk_dialog_set_default_response(dialog, GTK_RESPONSE_APPLY);
     gtk_dialog_set_response_sensitive(dialog, GTK_RESPONSE_APPLY, FALSE);
     gwy_help_add_to_tool_dialog(dialog, GWY_TOOL(tool), GWY_HELP_DEFAULT);
@@ -324,7 +306,7 @@ render_cell(GtkCellLayout *layout,
 {
     GwyToolCprofile *tool = (GwyToolCprofile*)user_data;
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    const GwySIValueFormat *vf;
+    const GwyValueFormat *vf;
     gchar buf[32];
     gdouble line[4];
     gdouble val;
@@ -342,9 +324,9 @@ render_cell(GtkCellLayout *layout,
 
     vf = tool->pixel_format;
     if (id == COLUMN_X)
-        val = floor(gwy_data_field_rtoj(plain_tool->data_field, line[0]));
+        val = floor(gwy_field_rtoj(plain_tool->field, line[0]));
     else if (id == COLUMN_Y)
-        val = floor(gwy_data_field_rtoi(plain_tool->data_field, line[1]));
+        val = floor(gwy_field_rtoi(plain_tool->field, line[1]));
     else {
         g_return_if_reached();
     }
@@ -426,14 +408,14 @@ render_color(G_GNUC_UNUSED GtkCellLayout *layout,
              gpointer user_data)
 {
     GwyToolCprofile *tool = (GwyToolCprofile*)user_data;
-    CprofileMode mode = gwy_params_get_enum(tool->params, PARAM_MODE);
+    GwyDirectionFlags mode = gwy_params_get_enum(tool->params, PARAM_MODE);
     GwyGraphCurveModel *gcmodel;
     GwyRGBA *rgba, *rgba2;
     guint idx, pixel;
 
     gtk_tree_model_get(model, iter, 0, &idx, -1);
 
-    if (mode != CPROFILE_MODE_CROSS) {
+    if (mode != GWY_DIRECTION_BOTH) {
         gcmodel = gwy_graph_model_get_curve(tool->gmodel, idx);
         g_object_get(gcmodel, "color", &rgba, NULL);
         pixel = 0xff | gwy_rgba_to_pixbuf_pixel(rgba);
@@ -465,14 +447,11 @@ gwy_tool_cprofile_data_switched(GwyTool *gwytool,
         return;
 
     if (data_view) {
-        CprofileMode mode = gwy_params_get_enum(tool->params, PARAM_MODE);
-        gboolean is_horiz = (mode == CPROFILE_MODE_CROSS || mode == CPROFILE_MODE_HORIZONTAL);
-        gboolean is_vert = (mode == CPROFILE_MODE_CROSS || mode == CPROFILE_MODE_VERTICAL);
-        gwy_object_set_or_reset(plain_tool->layer, tool->layer_type_cross,
-                                "draw-horizontal", is_horiz,
-                                "draw-vertical", is_vert,
-                                "thickness", gwy_params_get_int(tool->params, PARAM_THICKNESS),
+        gwy_object_set_or_reset(plain_tool->layer, GWY_TYPE_LAYER_CROSS,
+                                "directions", gwy_params_get_enum(tool->params, PARAM_MODE),
+                                "thickness", (gdouble)gwy_params_get_int(tool->params, PARAM_THICKNESS),
                                 "editable", TRUE,
+                                "focus", -1,
                                 NULL);
         gwy_selection_set_max_objects(plain_tool->selection, 1024);
         gwy_plain_tool_hold_selection(plain_tool, gwy_params_get_flags(tool->params, PARAM_HOLD_SELECTION));
@@ -498,22 +477,19 @@ param_changed(GwyToolCprofile *tool, gint id)
     gboolean do_update = (id != PARAM_MASKING && id != PARAM_TARGET_GRAPH && id != PARAM_OPTIONS_VISIBLE);
 
     if (id == PARAM_MASKING)
-        do_update = do_update || (plain_tool->data_field && plain_tool->mask_field);
+        do_update = do_update || (plain_tool->field && plain_tool->mask_field);
     if (id < 0 || id == PARAM_MODE) {
         if (plain_tool->layer) {
-            CprofileMode mode = gwy_params_get_enum(params, PARAM_MODE);
-            gboolean is_horiz = (mode == CPROFILE_MODE_CROSS || mode == CPROFILE_MODE_HORIZONTAL);
-            gboolean is_vert = (mode == CPROFILE_MODE_CROSS || mode == CPROFILE_MODE_VERTICAL);
-            g_object_set(plain_tool->layer,
-                         "draw-horizontal", is_horiz,
-                         "draw-vertical", is_vert,
-                         NULL);
+            gwy_layer_cross_set_directions(GWY_LAYER_CROSS(plain_tool->layer),
+                                           gwy_params_get_enum(params, PARAM_MODE));
         }
         gwy_graph_model_remove_all_curves(tool->gmodel);
     }
     if (id < 0 || id == PARAM_THICKNESS) {
-        if (plain_tool->layer)
-            g_object_set(plain_tool->layer, "thickness", gwy_params_get_int(params, PARAM_THICKNESS), NULL);
+        if (plain_tool->layer) {
+            gwy_layer_cross_set_thickness(GWY_LAYER_CROSS(plain_tool->layer),
+                                          gwy_params_get_int(params, PARAM_THICKNESS));
+        }
     }
     if (do_update)
         update_all_curves(tool);
@@ -592,8 +568,8 @@ update_all_curves(GwyToolCprofile *tool)
 }
 
 static void
-extract_column_profile(GwyDataField *dfield,
-                       GwyDataField *mask, GwyMaskingType masking,
+extract_column_profile(GwyField *dfield,
+                       GwyField *mask, GwyMaskingType masking,
                        GArray *xydata, gint col, gint thickness)
 {
     gint xres, yres, jfrom, jto, i, j, count;
@@ -601,10 +577,10 @@ extract_column_profile(GwyDataField *dfield,
     gdouble dy, z;
     GwyXY xy;
 
-    xres = gwy_data_field_get_xres(dfield);
-    yres = gwy_data_field_get_yres(dfield);
-    dy = gwy_data_field_get_dy(dfield);
-    d = gwy_data_field_get_data_const(dfield);
+    xres = gwy_field_get_xres(dfield);
+    yres = gwy_field_get_yres(dfield);
+    dy = gwy_field_get_dy(dfield);
+    d = gwy_field_get_data_const(dfield);
 
     g_array_set_size(xydata, 0);
 
@@ -613,7 +589,7 @@ extract_column_profile(GwyDataField *dfield,
     jto = col + thickness/2 + 1;
     jto = MIN(jto, xres);
 
-    m = (mask ? gwy_data_field_get_data_const(mask) : NULL);
+    m = (mask ? gwy_field_get_data_const(mask) : NULL);
     for (i = 0; i < yres; i++) {
         drow = d + i*xres + jfrom;
         z = 0.0;
@@ -652,8 +628,8 @@ extract_column_profile(GwyDataField *dfield,
 }
 
 static void
-extract_row_profile(GwyDataField *dfield,
-                    GwyDataField *mask, GwyMaskingType masking,
+extract_row_profile(GwyField *dfield,
+                    GwyField *mask, GwyMaskingType masking,
                     GArray *xydata, gint row, gint thickness)
 {
     gint xres, yres, ifrom, ito, i, j;
@@ -661,10 +637,10 @@ extract_row_profile(GwyDataField *dfield,
     gdouble dx;
     GwyXY *xy;
 
-    xres = gwy_data_field_get_xres(dfield);
-    yres = gwy_data_field_get_yres(dfield);
-    dx = gwy_data_field_get_dx(dfield);
-    d = gwy_data_field_get_data_const(dfield);
+    xres = gwy_field_get_xres(dfield);
+    yres = gwy_field_get_yres(dfield);
+    dx = gwy_field_get_dx(dfield);
+    d = gwy_field_get_data_const(dfield);
 
     g_array_set_size(xydata, xres);
     xy = &g_array_index(xydata, GwyXY, 0);
@@ -675,7 +651,7 @@ extract_row_profile(GwyDataField *dfield,
     ito = row + thickness/2 + 1;
     ito = MIN(ito, yres);
 
-    m = (mask ? gwy_data_field_get_data_const(mask) : NULL);
+    m = (mask ? gwy_field_get_data_const(mask) : NULL);
     if (!mask) {
         for (j = 0; j < xres; j++)
             xy[j].x = ito - ifrom;
@@ -732,7 +708,7 @@ static void
 update_one_curve(GwyToolCprofile *tool, gint i, gint id, gboolean is_vert)
 {
     GwyGraphCurveModel *gcmodel;
-    GwyDataField *data_field;
+    GwyField *field;
     const GwyRGBA *color;
     GArray *xydata;
     gchar *desc;
@@ -764,8 +740,8 @@ update_one_curve(GwyToolCprofile *tool, gint i, gint id, gboolean is_vert)
     g_object_unref(gcmodel);
 
     if (i == 0) {
-        data_field = GWY_PLAIN_TOOL(tool)->data_field;
-        gwy_graph_model_set_units_from_data_field(tool->gmodel, data_field, 1, 0, 0, 1);
+        field = GWY_PLAIN_TOOL(tool)->field;
+        gwy_graph_model_set_units_from_field(tool->gmodel, field, 1, 0, 0, 1);
         gwy_param_table_data_id_refilter(tool->table, PARAM_TARGET_GRAPH);
     }
 }
@@ -774,14 +750,12 @@ static void
 update_curve(GwyToolCprofile *tool, gint i)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    GwyDataField *data_field = plain_tool->data_field;
-    GwyDataField *mask = plain_tool->mask_field;
-    CprofileMode mode = gwy_params_get_enum(tool->params, PARAM_MODE);
+    GwyField *field = plain_tool->field;
+    GwyField *mask = plain_tool->mask_field;
+    GwyDirectionFlags mode = gwy_params_get_enum(tool->params, PARAM_MODE);
     gboolean zero_cross = gwy_params_get_boolean(tool->params, PARAM_ZERO_CROSS);
     gint thickness = gwy_params_get_int(tool->params, PARAM_THICKNESS);
     GwyMaskingType masking = gwy_params_get_masking(tool->params, PARAM_MASKING, &mask);
-    gboolean is_horiz = (mode == CPROFILE_MODE_CROSS || mode == CPROFILE_MODE_HORIZONTAL);
-    gboolean is_vert = (mode == CPROFILE_MODE_CROSS || mode == CPROFILE_MODE_VERTICAL);
     gdouble xy[2];
     gint col, row;
     GArray *xydata;
@@ -794,17 +768,17 @@ update_curve(GwyToolCprofile *tool, gint i)
         tool->xydata = g_array_new(FALSE, FALSE, sizeof(GwyXY));
     xydata = tool->xydata;
 
-    col = gwy_data_field_rtoj(data_field, xy[0]);
-    row = gwy_data_field_rtoi(data_field, xy[1]);
-    if (is_horiz) {
-        extract_row_profile(data_field, mask, masking, xydata, row, thickness);
-        add_x_offset(xydata, zero_cross ? -xy[0] : gwy_data_field_get_xoffset(data_field));
-        update_one_curve(tool, is_vert ? 2*i + 0 : i, i+1, FALSE);
+    col = gwy_field_rtoj(field, xy[0]);
+    row = gwy_field_rtoi(field, xy[1]);
+    if (mode & GWY_DIRECTION_VERTICAL) {
+        extract_row_profile(field, mask, masking, xydata, row, thickness);
+        add_x_offset(xydata, zero_cross ? -xy[0] : gwy_field_get_xoffset(field));
+        update_one_curve(tool, (mode & GWY_DIRECTION_HORIZONTAL) ? 2*i + 0 : i, i+1, FALSE);
     }
-    if (is_vert) {
-        extract_column_profile(data_field, mask, masking, xydata, col, thickness);
-        add_x_offset(xydata, zero_cross ? -xy[1] : gwy_data_field_get_yoffset(data_field));
-        update_one_curve(tool, is_horiz ? 2*i + 1 : i, i+1, TRUE);
+    if (mode & GWY_DIRECTION_HORIZONTAL) {
+        extract_column_profile(field, mask, masking, xydata, col, thickness);
+        add_x_offset(xydata, zero_cross ? -xy[1] : gwy_field_get_yoffset(field));
+        update_one_curve(tool, (mode & GWY_DIRECTION_VERTICAL) ? 2*i + 1 : i, i+1, TRUE);
     }
 }
 
@@ -812,21 +786,21 @@ static void
 gwy_tool_cprofile_apply(GwyToolCprofile *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    GwyGraphModel *gmodel;
-    gint n;
 
     g_return_if_fail(plain_tool->selection);
-    n = gwy_selection_get_data(plain_tool->selection, NULL);
+    gint n = gwy_selection_get_data(plain_tool->selection, NULL);
     g_return_if_fail(n);
 
+    GwyGraphModel *gmodel;
     if ((gmodel = gwy_params_get_graph(tool->params, PARAM_TARGET_GRAPH))) {
         gwy_graph_model_append_curves(gmodel, tool->gmodel, 1);
         return;
     }
 
-    gmodel = gwy_graph_model_duplicate(tool->gmodel);
+    gmodel = gwy_graph_model_copy(tool->gmodel);
     g_object_set(gmodel, "label-visible", TRUE, NULL);
-    gwy_app_data_browser_add_graph_model(gmodel, plain_tool->container, TRUE);
+    gint newid = gwy_file_add_graph(GWY_FILE(plain_tool->container), gmodel);
+    gwy_file_set_visible(GWY_FILE(plain_tool->container), GWY_FILE_GRAPH, newid, TRUE);
     g_object_unref(gmodel);
 }
 
