@@ -1,5 +1,5 @@
 /*
- *  $Id: crop.c 26745 2024-10-18 15:17:53Z yeti-dn $
+ *  $Id: crop.c 28858 2025-11-13 14:32:56Z yeti-dn $
  *  Copyright (C) 2003-2023 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,14 +16,11 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwymodule/gwymodule-tool.h>
-#include <libprocess/arithmetic.h>
-#include <libgwydgets/gwystock.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
+#include "libgwyapp/sanity.h"
 
 enum {
     PARAM_KEEP_OFFSETS,
@@ -49,9 +46,6 @@ struct _GwyToolCrop {
     GwyParamTable *table;
     gdouble rsel[4];
     gint isel[4];
-
-    /* potential class data */
-    GType layer_type_rect;
 };
 
 struct _GwyToolCropClass {
@@ -125,7 +119,7 @@ gwy_tool_crop_class_init(GwyToolCropClass *klass)
 
     gobject_class->finalize = gwy_tool_crop_finalize;
 
-    tool_class->stock_id = GWY_STOCK_CROP;
+    tool_class->icon_name = GWY_ICON_CROP;
     tool_class->title = _("Crop");
     tool_class->tooltip = _("Crop data");
     tool_class->prefix = "/module/crop";
@@ -142,7 +136,7 @@ gwy_tool_crop_finalize(GObject *object)
     GwyToolCrop *tool = GWY_TOOL_CROP(object);
 
     gwy_params_save_to_settings(tool->params);
-    GWY_OBJECT_UNREF(tool->params);
+    g_clear_object(&tool->params);
 
     G_OBJECT_CLASS(gwy_tool_crop_parent_class)->finalize(object);
 }
@@ -152,14 +146,10 @@ gwy_tool_crop_init(GwyToolCrop *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
 
-    tool->layer_type_rect = gwy_plain_tool_check_layer_type(plain_tool, "GwyLayerRectangle");
-    if (!tool->layer_type_rect)
-        return;
-
     tool->params = gwy_params_new_from_settings(define_module_params());
 
     plain_tool->lazy_updates = TRUE;
-    gwy_plain_tool_connect_selection(plain_tool, tool->layer_type_rect, "rectangle");
+    gwy_plain_tool_connect_selection(plain_tool, GWY_TYPE_LAYER_RECTANGLE, "rectangle");
     gwy_plain_tool_enable_selection_holding(plain_tool);
 
     gwy_tool_crop_init_dialog(tool);
@@ -170,30 +160,31 @@ gwy_tool_crop_rect_updated(GwyToolCrop *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
 
-    gwy_rect_selection_labels_select(tool->rlabels, plain_tool->selection, plain_tool->data_field);
+    gwy_rect_selection_labels_select(tool->rlabels, plain_tool->selection, plain_tool->field);
 }
 
 static void
 gwy_tool_crop_init_dialog(GwyToolCrop *tool)
 {
     GtkDialog *dialog = GTK_DIALOG(GWY_TOOL(tool)->dialog);
+    GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
     GwyParamTable *table;
 
     tool->rlabels = gwy_rect_selection_labels_new(TRUE, G_CALLBACK(gwy_tool_crop_rect_updated), tool);
-    gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(dialog)), gwy_rect_selection_labels_get_table(tool->rlabels), FALSE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(dialog)),
+                       gwy_rect_selection_labels_get_table(tool->rlabels), FALSE, FALSE, 0);
 
     table = tool->table = gwy_param_table_new(tool->params);
     gwy_param_table_append_checkbox(table, PARAM_KEEP_OFFSETS);
     gwy_param_table_append_checkbox(table, PARAM_ALL);
     gwy_param_table_append_checkbox(table, PARAM_NEW_CHANNEL);
     gwy_param_table_append_hold_selection(table, PARAM_HOLD_SELECTION);
-    gwy_plain_tool_add_param_table(GWY_PLAIN_TOOL(tool), table);
+    gwy_plain_tool_add_param_table(plain_tool, table);
     gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(dialog)), gwy_param_table_widget(table), FALSE, FALSE, 0);
 
-    gwy_plain_tool_add_clear_button(GWY_PLAIN_TOOL(tool));
+    gwy_plain_tool_add_clear_button(plain_tool);
     gwy_tool_add_hide_button(GWY_TOOL(tool), FALSE);
-    gtk_dialog_add_button(dialog, GTK_STOCK_APPLY, GTK_RESPONSE_APPLY);
-    gtk_dialog_set_default_response(dialog, GTK_RESPONSE_APPLY);
+    gwy_plain_tool_add_apply_button(plain_tool);
     gtk_dialog_set_response_sensitive(dialog, GTK_RESPONSE_APPLY, FALSE);
     gwy_help_add_to_tool_dialog(dialog, GWY_TOOL(tool), GWY_HELP_NO_BUTTON);
 
@@ -218,8 +209,8 @@ gwy_tool_crop_data_switched(GwyTool *gwytool,
         return;
 
     if (data_view) {
-        gwy_object_set_or_reset(plain_tool->layer, tool->layer_type_rect,
-                                "is-crop", TRUE,
+        gwy_object_set_or_reset(plain_tool->layer, GWY_TYPE_LAYER_RECTANGLE,
+                                "draw-as-crop", TRUE,
                                 "editable", TRUE,
                                 "focus", -1,
                                 NULL);
@@ -266,26 +257,26 @@ gwy_tool_crop_selection_changed(GwyPlainTool *plain_tool,
 }
 
 static void
-crop_one_field(GwyDataField *field,
+crop_one_field(GwyField *field,
                const gint *isel,
                const gdouble *sel,
                gboolean keep_offsets)
 {
-    gdouble xoff = (keep_offsets ? gwy_data_field_get_xoffset(field) + sel[0] : 0.0);
-    gdouble yoff = (keep_offsets ? gwy_data_field_get_yoffset(field) + sel[1] : 0.0);
+    gdouble xoff = (keep_offsets ? gwy_field_get_xoffset(field) + sel[0] : 0.0);
+    gdouble yoff = (keep_offsets ? gwy_field_get_yoffset(field) + sel[1] : 0.0);
 
-    gwy_data_field_resize(field, isel[0], isel[1], isel[2]+1, isel[3]+1);
-    gwy_data_field_set_xoffset(field, xoff);
-    gwy_data_field_set_yoffset(field, yoff);
+    gwy_field_crop(field, isel[0], isel[1], isel[2]+1 - isel[0], isel[3]+1 - isel[0]);
+    gwy_field_set_xoffset(field, xoff);
+    gwy_field_set_yoffset(field, yoff);
 }
 
 static void
 crop_into_new_channel(GwyToolCrop *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    GwyContainer *container = plain_tool->container;
+    GwyFile *file = GWY_FILE(plain_tool->container);
     gboolean keep_off = gwy_params_get_boolean(tool->params, PARAM_KEEP_OFFSETS);
-    GwyDataField *field = plain_tool->data_field, *mask = plain_tool->mask_field, *show = plain_tool->show_field;
+    GwyField *field = plain_tool->field, *mask = plain_tool->mask_field, *show = plain_tool->show_field;
     gint id, oldid = plain_tool->id;
     gint isel[4];
     gdouble rsel[4];
@@ -293,29 +284,28 @@ crop_into_new_channel(GwyToolCrop *tool)
     gwy_assign(isel, tool->isel, 4);
     gwy_assign(rsel, tool->rsel, 4);
 
-    field = gwy_data_field_duplicate(field);
+    field = gwy_field_copy(field);
     crop_one_field(field, isel, rsel, keep_off);
-    id = gwy_app_data_browser_add_data_field(field, container, TRUE);
+    id = gwy_file_add_image(file, field);
+    gwy_file_set_visible(file, GWY_FILE_IMAGE, id, TRUE);
     g_object_unref(field);
-    gwy_app_sync_data_items(container, container, oldid, id, FALSE,
-                            GWY_DATA_ITEM_GRADIENT,
-                            GWY_DATA_ITEM_RANGE_TYPE,
-                            GWY_DATA_ITEM_MASK_COLOR,
-                            GWY_DATA_ITEM_REAL_SQUARE,
-                            0);
-    gwy_app_set_data_field_title(container, id, _("Detail"));
-    gwy_app_channel_log_add(container, oldid, id, "tool::GwyToolCrop", NULL);
+    gwy_file_sync_items(file, GWY_FILE_IMAGE, oldid,
+                        file, GWY_FILE_IMAGE, id,
+                        GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_COLOR_MAPPING
+                        | GWY_FILE_ITEM_MASK_COLOR | GWY_FILE_ITEM_REAL_SQUARE, FALSE);
+    gwy_file_set_title(file, GWY_FILE_IMAGE, id, _("Detail"), TRUE);
+    gwy_log_add_full(file, GWY_FILE_IMAGE, oldid, id, "tool::GwyToolCrop", NULL);
 
     if (mask) {
-        mask = gwy_data_field_duplicate(mask);
+        mask = gwy_field_copy(mask);
         crop_one_field(mask, isel, rsel, keep_off);
-        gwy_container_pass_object(container, gwy_app_get_mask_key_for_id(id), mask);
+        gwy_file_pass_image_mask(file, id, mask);
     }
 
     if (show) {
-        show = gwy_data_field_duplicate(show);
+        show = gwy_field_copy(show);
         crop_one_field(show, isel, rsel, keep_off);
-        gwy_container_pass_object(container, gwy_app_get_show_key_for_id(id), show);
+        gwy_container_pass_object(GWY_CONTAINER(file), gwy_file_key_image_picture(id), show);
     }
 }
 
@@ -323,8 +313,8 @@ static void
 crop_in_place_maybe_all(GwyToolCrop *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    GwyContainer *container = plain_tool->container;
-    GwyDataField *field = plain_tool->data_field, *otherfield;
+    GwyFile *file = GWY_FILE(plain_tool->container);
+    GwyField *field = plain_tool->field, *otherfield;
     gboolean keep_off = gwy_params_get_boolean(tool->params, PARAM_KEEP_OFFSETS);
     gboolean crop_all = gwy_params_get_boolean(tool->params, PARAM_ALL);
     guint i, n;
@@ -344,38 +334,38 @@ crop_in_place_maybe_all(GwyToolCrop *tool)
         ids[1] = -1;
     }
     else
-        ids = gwy_app_data_browser_get_data_ids(container);
+        ids = gwy_file_get_ids(file, GWY_FILE_IMAGE);
 
     undo_quarks = g_array_new(FALSE, FALSE, sizeof(GQuark));
     for (i = n = 0; ids[i] > -1; i++) {
-        quark = gwy_app_get_data_key_for_id(ids[i]);
-        otherfield = gwy_container_get_object(container, quark);
-        if (gwy_data_field_check_compatibility(field, otherfield, GWY_DATA_COMPATIBILITY_ALL))
+        quark = gwy_file_key_image(ids[i]);
+        otherfield = gwy_file_get_image(file, ids[i]);
+        if (gwy_field_is_incompatible(field, otherfield, GWY_DATA_MISMATCH_ALL))
             continue;
 
         g_array_append_val(undo_quarks, quark);
 
-        quark = gwy_app_get_mask_key_for_id(ids[i]);
-        if (gwy_container_contains(container, quark))
+        quark = gwy_file_key_image_mask(ids[i]);
+        if (gwy_container_contains(plain_tool->container, quark))
             g_array_append_val(undo_quarks, quark);
 
-        quark = gwy_app_get_show_key_for_id(ids[i]);
-        if (gwy_container_contains(container, quark))
+        quark = gwy_file_key_image_picture(ids[i]);
+        if (gwy_container_contains(plain_tool->container, quark))
             g_array_append_val(undo_quarks, quark);
 
         ids[n++] = ids[i];
     }
     ids[n] = -1;
-    gwy_app_undo_qcheckpointv(container, undo_quarks->len, &g_array_index(undo_quarks, GQuark, 0));
+    gwy_app_undo_qcheckpointv(plain_tool->container, undo_quarks->len, &g_array_index(undo_quarks, GQuark, 0));
     for (i = 0; i < undo_quarks->len; i++) {
-        field = gwy_container_get_object(container, g_array_index(undo_quarks, GQuark, i));
+        field = gwy_container_get_object(plain_tool->container, g_array_index(undo_quarks, GQuark, i));
         crop_one_field(field, isel, rsel, keep_off);
-        gwy_data_field_data_changed(field);
+        gwy_field_data_changed(field);
     }
     qualname = g_strconcat("tool::", G_OBJECT_TYPE_NAME(plain_tool), NULL);
     for (i = 0; ids[i] > -1; i++) {
-        gwy_app_data_clear_selections(container, ids[i]);
-        gwy_app_channel_log_add(plain_tool->container, ids[i], ids[i], qualname, NULL);
+        gwy_file_remove_selections(file, GWY_FILE_IMAGE, ids[i]);
+        gwy_log_add_full(file, GWY_FILE_IMAGE, ids[i], ids[i], qualname, NULL);
     }
     g_free(qualname);
 }
@@ -385,7 +375,7 @@ gwy_tool_crop_apply(GwyToolCrop *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
 
-    g_return_if_fail(plain_tool->id >= 0 && plain_tool->data_field != NULL);
+    g_return_if_fail(plain_tool->id >= 0 && plain_tool->field != NULL);
 
     if (!gwy_selection_get_data(plain_tool->selection, NULL)) {
         g_warning("Apply invoked when no selection is present");
@@ -404,7 +394,7 @@ update_selected_rectangle(GwyToolCrop *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
     GwySelection *selection = plain_tool->selection;
-    GwyDataField *field = plain_tool->data_field;
+    GwyField *field = plain_tool->field;
     gint n = selection ? gwy_selection_get_data(selection, NULL) : 0;
     gint xres, yres;
 
@@ -417,8 +407,8 @@ update_selected_rectangle(GwyToolCrop *tool)
     gwy_rect_selection_labels_fill(tool->rlabels, selection, field, tool->rsel, tool->isel);
 
     /* Make Apply insensitive when the full image is selected, for whatever reason. There is nothting to crop then. */
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
     if (tool->isel[2] - tool->isel[0] == xres-1 && tool->isel[3] - tool->isel[1] == yres-1)
         return;
 
