@@ -1,5 +1,5 @@
 /*
- *  $Id: distance.c 26745 2024-10-18 15:17:53Z yeti-dn $
+ *  $Id: distance.c 28788 2025-11-04 17:13:02Z yeti-dn $
  *  Copyright (C) 2003-2022 Nenad Ocelic, David Necas (Yeti), Petr Klapetek.
  *  E-mail: ocelic@biochem.mpg.de, yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,15 +16,9 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwymodule/gwymodule-tool.h>
-#include <libprocess/stats.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwydgets/gwynullstore.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
+#include <gwy.h>
 
 enum {
     COLUMN_I, COLUMN_DX, COLUMN_DY, COLUMN_PHI, COLUMN_R, COLUMN_DZ, NCOLUMNS
@@ -53,15 +47,8 @@ struct _GwyToolDistance {
     GtkTreeModel *model;
     GwyParamTable *table;
 
-    GwyDataField *xunc;
-    GwyDataField *yunc;
-    GwyDataField *zunc;
-
-    gboolean has_calibration;
-
     /* potential class data */
-    GwySIValueFormat *angle_format;
-    GType layer_type_line;
+    GwyValueFormat *angle_format;
 };
 
 struct _GwyToolDistanceClass {
@@ -138,7 +125,7 @@ gwy_tool_distance_class_init(GwyToolDistanceClass *klass)
 
     gobject_class->finalize = gwy_tool_distance_finalize;
 
-    tool_class->stock_id = GWY_STOCK_DISTANCE;
+    tool_class->icon_name = GWY_ICON_DISTANCE;
     tool_class->title = _("Distance");
     tool_class->tooltip = _("Measure distances and directions between points");
     tool_class->prefix = "/module/distance";
@@ -155,12 +142,12 @@ gwy_tool_distance_finalize(GObject *object)
     GwyToolDistance *tool = GWY_TOOL_DISTANCE(object);
 
     gwy_params_save_to_settings(tool->params);
-    GWY_OBJECT_UNREF(tool->params);
+    g_clear_object(&tool->params);
     if (tool->model) {
         gtk_tree_view_set_model(tool->treeview, NULL);
-        GWY_OBJECT_UNREF(tool->model);
+        g_clear_object(&tool->model);
     }
-    GWY_SI_VALUE_FORMAT_FREE(tool->angle_format);
+    GWY_FREE_VALUE_FORMAT(tool->angle_format);
 
     G_OBJECT_CLASS(gwy_tool_distance_parent_class)->finalize(object);
 }
@@ -169,16 +156,12 @@ static void
 gwy_tool_distance_init(GwyToolDistance *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-
-    tool->layer_type_line = gwy_plain_tool_check_layer_type(plain_tool, "GwyLayerLine");
-    if (!tool->layer_type_line)
-        return;
-
-    plain_tool->unit_style = GWY_SI_UNIT_FORMAT_MARKUP;
+    plain_tool->unit_style = GWY_UNIT_FORMAT_MARKUP;
     plain_tool->lazy_updates = TRUE;
+
     tool->params = gwy_params_new_from_settings(define_module_params());
-    tool->angle_format = gwy_si_unit_value_format_new(1.0, 1, _("deg"));
-    gwy_plain_tool_connect_selection(plain_tool, tool->layer_type_line, "line");
+    tool->angle_format = gwy_value_format_new(1.0, 1, _("deg"));
+    gwy_plain_tool_connect_selection(plain_tool, GWY_TYPE_LAYER_LINE, "line");
     gwy_plain_tool_enable_selection_holding(plain_tool);
 
     gwy_tool_distance_init_dialog(tool);
@@ -245,7 +228,6 @@ gwy_tool_distance_data_switched(GwyTool *gwytool, GwyDataView *data_view)
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(gwytool);
     GwyToolDistance *tool = GWY_TOOL_DISTANCE(gwytool);
     gboolean ignore = (data_view == plain_tool->data_view);
-    gchar xukey[32], yukey[32], zukey[32];
 
     GWY_TOOL_CLASS(gwy_tool_distance_parent_class)->data_switched(gwytool, data_view);
 
@@ -253,20 +235,12 @@ gwy_tool_distance_data_switched(GwyTool *gwytool, GwyDataView *data_view)
         return;
 
     if (data_view) {
-        gwy_object_set_or_reset(plain_tool->layer, tool->layer_type_line,
-                                "line-numbers", gwy_params_get_boolean(tool->params, PARAM_NUMBER_LINES),
-                                "thickness", 1,
+        gwy_object_set_or_reset(plain_tool->layer, GWY_TYPE_LAYER_LINE,
+                                "numbered", gwy_params_get_boolean(tool->params, PARAM_NUMBER_LINES),
                                 "editable", TRUE,
                                 "focus", -1,
                                 NULL);
         gwy_selection_set_max_objects(plain_tool->selection, 1024);
-        g_snprintf(xukey, sizeof(xukey), "/%d/data/cal_xunc", plain_tool->id);
-        g_snprintf(yukey, sizeof(yukey), "/%d/data/cal_yunc", plain_tool->id);
-        g_snprintf(zukey, sizeof(zukey), "/%d/data/cal_zunc", plain_tool->id);
-
-        tool->has_calibration = (gwy_container_gis_object_by_name(plain_tool->container, xukey, &tool->xunc)
-                                 && gwy_container_gis_object_by_name(plain_tool->container, yukey, &tool->yunc)
-                                 && gwy_container_gis_object_by_name(plain_tool->container, zukey, &tool->zunc));
         gwy_plain_tool_hold_selection(plain_tool, gwy_params_get_flags(tool->params, PARAM_HOLD_SELECTION));
     }
     update_headers(tool);
@@ -322,11 +296,9 @@ param_changed(GwyToolDistance *tool, gint id)
 {
     if (id < 0 || id == PARAM_NUMBER_LINES) {
         GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-        if (plain_tool->layer) {
-            g_object_set(plain_tool->layer,
-                         "line-numbers", gwy_params_get_boolean(tool->params, PARAM_NUMBER_LINES),
-                         NULL);
-        }
+        GwyLayerLine *layer = plain_tool->layer ? GWY_LAYER_LINE(plain_tool->layer) : NULL;
+        if (layer)
+            gwy_layer_line_set_numbered(layer, gwy_params_get_boolean(tool->params, PARAM_NUMBER_LINES));
     }
 }
 
@@ -335,7 +307,7 @@ gwy_tool_distance_update_header(GwyToolDistance *tool,
                                 guint col,
                                 GString *str,
                                 const gchar *title,
-                                GwySIValueFormat *vf)
+                                GwyValueFormat *vf)
 {
     GtkTreeViewColumn *column = gtk_tree_view_get_column(tool->treeview, col);
     GtkLabel *label = GTK_LABEL(gtk_tree_view_column_get_widget(column));
@@ -368,20 +340,6 @@ update_headers(GwyToolDistance *tool)
     gwy_param_table_set_sensitive(tool->table, PARAM_REPORT_STYLE, ok);
 }
 
-static gdouble
-get_uncertainty(GwyDataField *uncfield, const gdouble *coords)
-{
-    gdouble unc;
-
-    if (!uncfield)
-        return 0.0;
-    unc = gwy_data_field_get_dval_real(uncfield, coords[0], coords[1], GWY_INTERPOLATION_BILINEAR);
-    unc *= unc;
-    unc += (gwy_data_field_get_dval_real(uncfield, coords[2], coords[3], GWY_INTERPOLATION_BILINEAR)
-            * gwy_data_field_get_dval_real(uncfield, coords[2], coords[3], GWY_INTERPOLATION_BILINEAR));
-    return sqrt(unc);
-}
-
 static void
 render_cell(GtkCellLayout *layout,
             GtkCellRenderer *renderer,
@@ -392,10 +350,10 @@ render_cell(GtkCellLayout *layout,
     GwyToolDistance *tool = (GwyToolDistance*)user_data;
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
     gint id = GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(layout), "id"));
-    const GwySIValueFormat *vf = NULL;
+    const GwyValueFormat *vf = NULL;
     gchar buf[32];
     gdouble line[4];
-    gdouble val, unc = 0.0;
+    gdouble val;
     guint idx;
     gint x, y;
 
@@ -411,12 +369,10 @@ render_cell(GtkCellLayout *layout,
     if (id == COLUMN_DX) {
         vf = plain_tool->coord_format;
         val = line[2] - line[0];
-        unc = get_uncertainty(tool->xunc, line);
     }
     else if (id == COLUMN_DY) {
         vf = plain_tool->coord_format;
         val = line[3] - line[1];
-        unc = get_uncertainty(tool->yunc, line);
     }
     else if (id == COLUMN_R) {
         vf = plain_tool->coord_format;
@@ -424,37 +380,25 @@ render_cell(GtkCellLayout *layout,
     }
     else if (id == COLUMN_PHI) {
         vf = tool->angle_format;
-        val = atan2(line[1] - line[3], line[2] - line[0]) * 180.0/G_PI;
+        val = gwy_rad2deg(atan2(line[1] - line[3], line[2] - line[0]));
     }
     else if (id == COLUMN_DZ) {
-        x = floor(gwy_data_field_rtoj(plain_tool->data_field, line[2]));
-        y = floor(gwy_data_field_rtoi(plain_tool->data_field, line[3]));
-        val = gwy_data_field_get_val(plain_tool->data_field, x, y);
-        x = floor(gwy_data_field_rtoj(plain_tool->data_field, line[0]));
-        y = floor(gwy_data_field_rtoi(plain_tool->data_field, line[1]));
-        val -= gwy_data_field_get_val(plain_tool->data_field, x, y);
+        x = floor(gwy_field_rtoj(plain_tool->field, line[2]));
+        y = floor(gwy_field_rtoi(plain_tool->field, line[3]));
+        val = gwy_field_get_val(plain_tool->field, x, y);
+        x = floor(gwy_field_rtoj(plain_tool->field, line[0]));
+        y = floor(gwy_field_rtoi(plain_tool->field, line[1]));
+        val -= gwy_field_get_val(plain_tool->field, x, y);
         vf = plain_tool->value_format;
-        unc = get_uncertainty(tool->zunc, line);
     }
     else {
         g_return_if_reached();
     }
 
-    if (tool->has_calibration) {
-        if (vf) {
-            g_snprintf(buf, sizeof(buf), "%.*f±%.*f",
-                       vf->precision, val/vf->magnitude,
-                       vf->precision, unc/vf->magnitude);
-        }
-        else
-            g_snprintf(buf, sizeof(buf), "%.3g±%.3g", val, unc);
-    }
-    else {
-        if (vf)
-            g_snprintf(buf, sizeof(buf), "%.*f", vf->precision, val/vf->magnitude);
-        else
-            g_snprintf(buf, sizeof(buf), "%.3g", val);
-    }
+    if (vf)
+        g_snprintf(buf, sizeof(buf), "%.*f", vf->precision, val/vf->magnitude);
+    else
+        g_snprintf(buf, sizeof(buf), "%.3g", val);
     g_object_set(renderer, "text", buf, NULL);
 }
 
@@ -464,29 +408,29 @@ format_report(gpointer user_data)
     GwyToolDistance *tool = (GwyToolDistance*)user_data;
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
     GwyResultsReportType report_style = gwy_params_get_report_type(tool->params, PARAM_REPORT_STYLE);
-    GwySIUnitFormatStyle style = GWY_SI_UNIT_FORMAT_UNICODE;
+    GwyUnitFormatStyle style = GWY_UNIT_FORMAT_UNICODE;
     gint n, x, y, i;
     gdouble line[4], dx, dy, r, phi, dz;
-    GwySIValueFormat *vf_dist, *vf_phi, *vf_dz;
-    GwySIUnit *xyunit, *zunit;
+    GwyValueFormat *vf_dist, *vf_phi, *vf_dz;
+    GwyUnit *xyunit, *zunit;
     GString *text;
     gchar *dx_header, *dy_header, *phi_header, *r_header, *dz_header;
 
     text = g_string_new(NULL);
-    xyunit = gwy_data_field_get_si_unit_xy(plain_tool->data_field);
-    zunit = gwy_data_field_get_si_unit_z(plain_tool->data_field);
+    xyunit = gwy_field_get_unit_xy(plain_tool->field);
+    zunit = gwy_field_get_unit_z(plain_tool->field);
     if (!(report_style & GWY_RESULTS_REPORT_MACHINE)) {
-        dx = gwy_data_field_get_dx(plain_tool->data_field);
-        dy = gwy_data_field_get_dy(plain_tool->data_field);
-        vf_dist = gwy_si_unit_get_format(xyunit, style, fmin(dx, dy), NULL);
-        gwy_data_field_get_min_max(plain_tool->data_field, &dx, &dy);
-        vf_dz = gwy_si_unit_get_format(zunit, style, fmax(fabs(dx), fabs(dy))/120.0, NULL);
-        vf_phi = gwy_si_unit_value_format_new(G_PI/180.0, 0, _("deg"));
+        dx = gwy_field_get_dx(plain_tool->field);
+        dy = gwy_field_get_dy(plain_tool->field);
+        vf_dist = gwy_unit_get_format(xyunit, style, fmin(dx, dy), NULL);
+        gwy_field_get_min_max(plain_tool->field, &dx, &dy);
+        vf_dz = gwy_unit_get_format(zunit, style, fmax(fabs(dx), fabs(dy))/120.0, NULL);
+        vf_phi = gwy_value_format_new(G_PI/180.0, 0, _("deg"));
     }
     else {
-        vf_dist = gwy_si_unit_get_format_for_power10(xyunit, style, 0, NULL);
-        vf_dz = gwy_si_unit_get_format_for_power10(zunit, style, 0, NULL);
-        vf_phi = gwy_si_unit_value_format_new(1.0, 0, "");
+        vf_dist = gwy_unit_get_format_for_power10(xyunit, style, 0, NULL);
+        vf_dz = gwy_unit_get_format_for_power10(zunit, style, 0, NULL);
+        vf_phi = gwy_value_format_new(1.0, 0, "");
     }
 
     /* FIXME: This needs some helper function. */
@@ -511,12 +455,12 @@ format_report(gpointer user_data)
         r = hypot(line[2] - line[0], line[3] - line[1]);
         phi = atan2(line[1] - line[3], line[2] - line[0]);
 
-        x = floor(gwy_data_field_rtoj(plain_tool->data_field, line[2]));
-        y = floor(gwy_data_field_rtoi(plain_tool->data_field, line[3]));
-        dz = gwy_data_field_get_val(plain_tool->data_field, x, y);
-        x = floor(gwy_data_field_rtoj(plain_tool->data_field, line[0]));
-        y = floor(gwy_data_field_rtoi(plain_tool->data_field, line[1]));
-        dz -= gwy_data_field_get_val(plain_tool->data_field, x, y);
+        x = floor(gwy_field_rtoj(plain_tool->field, line[2]));
+        y = floor(gwy_field_rtoi(plain_tool->field, line[3]));
+        dz = gwy_field_get_val(plain_tool->field, x, y);
+        x = floor(gwy_field_rtoj(plain_tool->field, line[0]));
+        y = floor(gwy_field_rtoi(plain_tool->field, line[1]));
+        dz -= gwy_field_get_val(plain_tool->field, x, y);
 
         gwy_format_result_table_row(text, report_style, 5,
                                     dx/vf_dist->magnitude, dy/vf_dist->magnitude,
