@@ -1,5 +1,5 @@
 /*
- *  $Id: filter.c 26745 2024-10-18 15:17:53Z yeti-dn $
+ *  $Id: filter.c 28844 2025-11-10 16:07:51Z yeti-dn $
  *  Copyright (C) 2003-2022 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,16 +16,9 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwymodule/gwymodule-tool.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libprocess/elliptic.h>
-#include <libprocess/filters.h>
-#include <libprocess/linestats.h>
-#include <libgwydgets/gwystock.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 
 #define FWHM2SIGMA (1.0/(2.0*sqrt(2*G_LN2)))
 
@@ -71,9 +64,6 @@ struct _GwyToolFilter {
     GwyRectSelectionLabels *rlabels;
     GwyParamTable *table;
     FilterType oldfilter;
-
-    /* potential class data */
-    GType layer_type_rect;
 };
 
 struct _GwyToolFilterClass {
@@ -97,15 +87,15 @@ static void         param_changed                    (GwyToolFilter *tool,
 static void         update_selected_rectangle        (GwyToolFilter *tool);
 static void         set_up_size_for_mode             (GwyToolFilter *tool);
 static void         gwy_tool_filter_apply            (GwyToolFilter *tool);
-static void         filter_area_sharpen              (GwyDataField *dfield,
+static void         filter_area_sharpen              (GwyField *dfield,
                                                       gdouble sigma,
                                                       gint col,
                                                       gint row,
                                                       gint width,
                                                       gint height);
-static void         apply_masking                    (GwyDataField *dfield,
-                                                      GwyDataField *orig,
-                                                      GwyDataField *mask,
+static void         apply_masking                    (GwyField *dfield,
+                                                      GwyField *orig,
+                                                      GwyField *mask,
                                                       GwyMaskingType masking);
 static gboolean     filter_is_sized                  (FilterType type);
 static gboolean     filter_is_float_sized            (FilterType type);
@@ -143,13 +133,13 @@ define_module_params(void)
         { N_("Conservative denoise"), FILTER_CONSERVATIVE, },
         { N_("Minimum"),              FILTER_MINIMUM,      },
         { N_("Maximum"),              FILTER_MAXIMUM,      },
-        { N_("filter|Opening"),       FILTER_OPENING,      },
-        { N_("filter|Closing"),       FILTER_CLOSING,      },
+        { gwy_NC("filter", "Opening"),       FILTER_OPENING,      },
+        { gwy_NC("filter", "Closing"),       FILTER_CLOSING,      },
         { N_("ASF Opening"),          FILTER_ASF_OPENING,  },
         { N_("ASF Closing"),          FILTER_ASF_CLOSING,  },
         { N_("Kuwahara"),             FILTER_KUWAHARA,     },
         { N_("Dechecker"),            FILTER_DECHECKER,    },
-        { N_("filter|Gaussian"),      FILTER_GAUSSIAN,     },
+        { gwy_NC("filter", "Gaussian"),      FILTER_GAUSSIAN,     },
         { N_("Sharpen"),              FILTER_SHARPEN,      },
     };
     static GwyParamDef *paramdef = NULL;
@@ -180,7 +170,7 @@ gwy_tool_filter_class_init(GwyToolFilterClass *klass)
 
     gobject_class->finalize = gwy_tool_filter_finalize;
 
-    tool_class->stock_id = GWY_STOCK_FILTER;
+    tool_class->icon_name = GWY_ICON_FILTER;
     tool_class->title = _("Filter");
     tool_class->tooltip = _("Basic filters: mean, median, denoise, …");
     tool_class->prefix = "/module/filter";
@@ -197,7 +187,7 @@ gwy_tool_filter_finalize(GObject *object)
     GwyToolFilter *tool = GWY_TOOL_FILTER(object);
 
     gwy_params_save_to_settings(tool->params);
-    GWY_OBJECT_UNREF(tool->params);
+    g_clear_object(&tool->params);
 
     G_OBJECT_CLASS(gwy_tool_filter_parent_class)->finalize(object);
 }
@@ -207,16 +197,11 @@ gwy_tool_filter_init(GwyToolFilter *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
 
-    tool->layer_type_rect = gwy_plain_tool_check_layer_type(plain_tool, "GwyLayerRectangle");
-
-    if (!tool->layer_type_rect)
-        return;
-
     tool->params = gwy_params_new_from_settings(define_module_params());
     tool->oldfilter = FILTER_UNDEFINED;
 
     plain_tool->lazy_updates = TRUE;
-    gwy_plain_tool_connect_selection(plain_tool, tool->layer_type_rect, "rectangle");
+    gwy_plain_tool_connect_selection(plain_tool, GWY_TYPE_LAYER_RECTANGLE, "rectangle");
 
     gwy_tool_filter_init_dialog(tool);
 }
@@ -226,7 +211,7 @@ gwy_tool_filter_rect_updated(GwyToolFilter *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
 
-    gwy_rect_selection_labels_select(tool->rlabels, plain_tool->selection, plain_tool->data_field);
+    gwy_rect_selection_labels_select(tool->rlabels, plain_tool->selection, plain_tool->field);
 }
 
 static void
@@ -237,7 +222,8 @@ gwy_tool_filter_init_dialog(GwyToolFilter *tool)
 
     /* Selection info */
     tool->rlabels = gwy_rect_selection_labels_new(TRUE, G_CALLBACK(gwy_tool_filter_rect_updated), tool);
-    gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(dialog)), gwy_rect_selection_labels_get_table(tool->rlabels), FALSE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(dialog)),
+                       gwy_rect_selection_labels_get_table(tool->rlabels), FALSE, FALSE, 0);
 
     /* Options */
     table = tool->table = gwy_param_table_new(tool->params);
@@ -257,7 +243,7 @@ gwy_tool_filter_init_dialog(GwyToolFilter *tool)
 
     gwy_plain_tool_add_clear_button(GWY_PLAIN_TOOL(tool));
     gwy_tool_add_hide_button(GWY_TOOL(tool), FALSE);
-    gtk_dialog_add_button(dialog, GTK_STOCK_APPLY, GTK_RESPONSE_APPLY);
+    gwy_plain_tool_add_apply_button(GWY_PLAIN_TOOL(tool));
     gtk_dialog_set_default_response(dialog, GTK_RESPONSE_APPLY);
     gtk_dialog_set_response_sensitive(dialog, GTK_RESPONSE_APPLY, FALSE);
     gwy_help_add_to_tool_dialog(dialog, GWY_TOOL(tool), GWY_HELP_DEFAULT);
@@ -272,7 +258,6 @@ gwy_tool_filter_data_switched(GwyTool *gwytool,
                               GwyDataView *data_view)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(gwytool);
-    GwyToolFilter *tool = GWY_TOOL_FILTER(gwytool);
     gboolean ignore = (data_view == plain_tool->data_view);
 
     GWY_TOOL_CLASS(gwy_tool_filter_parent_class)->data_switched(gwytool, data_view);
@@ -281,7 +266,7 @@ gwy_tool_filter_data_switched(GwyTool *gwytool,
         return;
 
     if (data_view) {
-        gwy_object_set_or_reset(plain_tool->layer, tool->layer_type_rect,
+        gwy_object_set_or_reset(plain_tool->layer, GWY_TYPE_LAYER_RECTANGLE,
                                 "editable", TRUE,
                                 "focus", -1,
                                 NULL);
@@ -371,7 +356,7 @@ update_selected_rectangle(GwyToolFilter *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
     GwySelection *selection = plain_tool->selection;
-    GwyDataField *field = plain_tool->data_field;
+    GwyField *field = plain_tool->field;
     gint n = selection ? gwy_selection_get_data(selection, NULL) : 0;
 
     gwy_rect_selection_labels_fill(tool->rlabels, n == 1 ? selection : NULL, field, NULL, tool->isel);
@@ -381,9 +366,9 @@ static void
 gwy_tool_filter_apply(GwyToolFilter *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    GwyDataField *field = plain_tool->data_field;
-    GwyDataField *mask = plain_tool->mask_field;
-    GwyDataField *origfield = NULL, *kernel = NULL;
+    GwyField *field = plain_tool->field;
+    GwyField *mask = plain_tool->mask_field;
+    GwyField *origfield = NULL, *kernel = NULL;
     GwyMaskingType masking = gwy_params_get_masking(tool->params, PARAM_MASKING, &mask);
     FilterType filter_type = gwy_params_get_enum(tool->params, PARAM_FILTER_TYPE);
     gdouble sigma = gwy_params_get_double(tool->params, PARAM_SIZE_GAUSS);
@@ -397,13 +382,13 @@ gwy_tool_filter_apply(GwyToolFilter *tool)
     w = tool->isel[2]+1 - tool->isel[0];
     h = tool->isel[3]+1 - tool->isel[1];
 
-    gwy_app_undo_qcheckpoint(plain_tool->container, gwy_app_get_data_key_for_id(plain_tool->id), 0);
+    gwy_app_undo_qcheckpoint(plain_tool->container, gwy_file_key_image(plain_tool->id), 0);
 
     if (filter_needs_kernel(filter_type)) {
-        kernel = gwy_data_field_new(size, size, size, size, TRUE);
-        n = gwy_data_field_elliptic_area_fill(kernel, 0, 0, size, size, 1.0);
+        kernel = gwy_field_new(size, size, size, size, TRUE);
+        n = gwy_field_elliptic_area_fill(kernel, 0, 0, size, size, 1.0);
         if (filter_type == FILTER_MEAN)
-            gwy_data_field_multiply(kernel, 1.0/n);
+            gwy_field_multiply(kernel, 1.0/n);
     }
 
     /*
@@ -416,34 +401,34 @@ gwy_tool_filter_apply(GwyToolFilter *tool)
      * native masking would not be difficult, but apart from median probably also not worth it.
      */
     if (mask)
-        origfield = gwy_data_field_duplicate(field);
+        origfield = gwy_field_copy(field);
 
     if (filter_type == FILTER_MEAN)
-        gwy_data_field_area_ext_convolve(field, col, row, w, h, field, kernel, GWY_EXTERIOR_BORDER_EXTEND, 0.0, FALSE);
+        gwy_field_area_ext_convolve(field, col, row, w, h, field, kernel, GWY_EXTERIOR_BORDER, 0.0, FALSE);
     else if (filter_type == FILTER_MEDIAN)
-        gwy_data_field_area_filter_kth_rank(field, kernel, col, row, w, h, n/2, NULL);
+        gwy_field_area_filter_kth_rank(field, kernel, col, row, w, h, n/2, NULL);
     else if (filter_type == FILTER_MINIMUM)
-        gwy_data_field_area_filter_min_max(field, kernel, GWY_MIN_MAX_FILTER_MINIMUM, col, row, w, h);
+        gwy_field_area_filter_min_max(field, kernel, GWY_MIN_MAX_FILTER_MINIMUM, col, row, w, h);
     else if (filter_type == FILTER_MAXIMUM)
-        gwy_data_field_area_filter_min_max(field, kernel, GWY_MIN_MAX_FILTER_MAXIMUM, col, row, w, h);
+        gwy_field_area_filter_min_max(field, kernel, GWY_MIN_MAX_FILTER_MAXIMUM, col, row, w, h);
     else if (filter_type == FILTER_CONSERVATIVE)
-        gwy_data_field_area_filter_conservative(field, size, col, row, w, h);
+        gwy_field_area_filter_conservative(field, size, col, row, w, h);
     else if (filter_type == FILTER_KUWAHARA)
-        gwy_data_field_area_filter_kuwahara(field, col, row, w, h);
+        gwy_field_area_filter_kuwahara(field, col, row, w, h);
     else if (filter_type == FILTER_DECHECKER)
-        gwy_data_field_area_filter_dechecker(field, col, row, w, h);
+        gwy_field_area_filter_dechecker(field, col, row, w, h);
     else if (filter_type == FILTER_GAUSSIAN)
-        gwy_data_field_area_filter_gaussian(field, sigma, col, row, w, h);
+        gwy_field_area_filter_gaussian(field, sigma, col, row, w, h);
     else if (filter_type == FILTER_SHARPEN)
         filter_area_sharpen(field, sigma, col, row, w, h);
     else if (filter_type == FILTER_OPENING)
-        gwy_data_field_area_filter_min_max(field, kernel, GWY_MIN_MAX_FILTER_OPENING, col, row, w, h);
+        gwy_field_area_filter_min_max(field, kernel, GWY_MIN_MAX_FILTER_OPENING, col, row, w, h);
     else if (filter_type == FILTER_CLOSING)
-        gwy_data_field_area_filter_min_max(field, kernel, GWY_MIN_MAX_FILTER_CLOSING, col, row, w, h);
+        gwy_field_area_filter_min_max(field, kernel, GWY_MIN_MAX_FILTER_CLOSING, col, row, w, h);
     else if (filter_type == FILTER_ASF_OPENING)
-        gwy_data_field_area_filter_disc_asf(field, size/2, FALSE, col, row, w, h);
+        gwy_field_area_filter_disc_asf(field, size/2, FALSE, col, row, w, h);
     else if (filter_type == FILTER_ASF_CLOSING)
-        gwy_data_field_area_filter_disc_asf(field, size/2, TRUE, col, row, w, h);
+        gwy_field_area_filter_disc_asf(field, size/2, TRUE, col, row, w, h);
     else {
         g_assert_not_reached();
     }
@@ -453,26 +438,27 @@ gwy_tool_filter_apply(GwyToolFilter *tool)
         g_object_unref(origfield);
     }
 
-    GWY_OBJECT_UNREF(kernel);
-    gwy_data_field_data_changed(field);
+    g_clear_object(&kernel);
+    gwy_field_data_changed(field);
     gwy_params_save_to_settings(tool->params);   /* Ensure correct parameters in the log. */
     gwy_plain_tool_log_add(plain_tool);
 }
 
 static void
-filter_area_sharpen(GwyDataField *dfield, gdouble sigma,
+filter_area_sharpen(GwyField *dfield, gdouble sigma,
                     gint col, gint row, gint width, gint height)
 {
-    GwyDataField *origpart;
+    GwyField *origpart;
     gint i, j, xres = dfield->xres;
-    gdouble *d, *p;
 
-    origpart = gwy_data_field_area_extract(dfield, col, row, width, height);
-    gwy_data_field_area_filter_gaussian(dfield, sigma, col, row, width, height);
+    origpart = gwy_field_area_extract(dfield, col, row, width, height);
+    gwy_field_area_filter_gaussian(dfield, sigma, col, row, width, height);
 
+    gdouble *ddata = gwy_field_get_data(dfield);
+    const gdouble *pdata = gwy_field_get_data_const(origpart);
     for (i = 0; i < height; i++) {
-        d = dfield->data + (i + row)*xres + col;
-        p = origpart->data + i*width;
+        gdouble *d = ddata + (i + row)*xres + col;
+        const gdouble *p = pdata + i*width;
         for (j = 0; j < width; j++)
             d[j] = 2*p[j] - d[j];
     }
@@ -481,14 +467,14 @@ filter_area_sharpen(GwyDataField *dfield, gdouble sigma,
 }
 
 static void
-apply_masking(GwyDataField *dfield, GwyDataField *orig, GwyDataField *mask,
+apply_masking(GwyField *dfield, GwyField *orig, GwyField *mask,
               GwyMaskingType masking)
 {
-    const gdouble *r = gwy_data_field_get_data_const(orig);
-    const gdouble *m = gwy_data_field_get_data_const(mask);
-    gdouble *d = gwy_data_field_get_data(dfield);
-    gint xres = gwy_data_field_get_xres(dfield);
-    gint yres = gwy_data_field_get_yres(dfield);
+    const gdouble *r = gwy_field_get_data_const(orig);
+    const gdouble *m = gwy_field_get_data_const(mask);
+    gdouble *d = gwy_field_get_data(dfield);
+    gint xres = gwy_field_get_xres(dfield);
+    gint yres = gwy_field_get_yres(dfield);
     gint k;
 
     if (masking == GWY_MASK_INCLUDE) {
@@ -504,7 +490,7 @@ apply_masking(GwyDataField *dfield, GwyDataField *orig, GwyDataField *mask,
         }
     }
 
-    gwy_data_field_invalidate(dfield);
+    gwy_field_invalidate(dfield);
 }
 
 static gboolean
