@@ -1,5 +1,5 @@
 /*
- *  $Id: grainmeasure.c 26745 2024-10-18 15:17:53Z yeti-dn $
+ *  $Id: grainmeasure.c 28978 2025-12-11 10:41:31Z yeti-dn $
  *  Copyright (C) 2007-2022 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,16 +16,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdarg.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwymodule/gwymodule-tool.h>
-#include <libprocess/grains.h>
-#include <libgwydgets/gwygrainvaluemenu.h>
-#include <libgwydgets/gwydgetutils.h>
-#include <libgwydgets/gwystock.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 
 enum {
     PARAM_EXPANDED,
@@ -52,11 +46,8 @@ struct _GwyToolGrainMeasure {
     gint gno;
 
     gboolean same_units;
-    GwySIUnit *siunit;
-    GwySIValueFormat *vf;
-
-    /* potential class data */
-    GType layer_type_point;
+    GwyUnit *siunit;
+    GwyValueFormat *vf;
 };
 
 struct _GwyToolGrainMeasureClass {
@@ -124,7 +115,7 @@ gwy_tool_grain_measure_class_init(GwyToolGrainMeasureClass *klass)
 
     gobject_class->finalize = gwy_tool_grain_measure_finalize;
 
-    tool_class->stock_id = GWY_STOCK_GRAINS_MEASURE;
+    tool_class->icon_name = GWY_ICON_GRAINS_MEASURE;
     tool_class->title = _("Grain Measure");
     tool_class->tooltip = _("Measure individual grains (continuous parts of mask)");
     tool_class->prefix = "/module/grainmeasure";
@@ -144,15 +135,14 @@ gwy_tool_grain_measure_finalize(GObject *object)
     guint i;
 
     gwy_params_save_to_settings(tool->params);
-    GWY_OBJECT_UNREF(tool->params);
-    GWY_OBJECT_UNREF(tool->siunit);
-    g_free(tool->grains);
+    g_clear_object(&tool->params);
+    g_clear_object(&tool->siunit);
     if (tool->values) {
         for (i = 0; i < tool->values->len; i++)
             g_free(tool->values->pdata[i]);
         g_ptr_array_free(tool->values, TRUE);
     }
-    GWY_SI_VALUE_FORMAT_FREE(tool->vf);
+    GWY_FREE_VALUE_FORMAT(tool->vf);
 
     G_OBJECT_CLASS(gwy_tool_grain_measure_parent_class)->finalize(object);
 }
@@ -162,14 +152,10 @@ gwy_tool_grain_measure_init(GwyToolGrainMeasure *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
 
-    tool->layer_type_point = gwy_plain_tool_check_layer_type(plain_tool, "GwyLayerPoint");
-    if (!tool->layer_type_point)
-        return;
-
     tool->params = gwy_params_new_from_settings(define_module_params());
 
     plain_tool->lazy_updates = TRUE;
-    gwy_plain_tool_connect_selection(plain_tool, tool->layer_type_point, "pointer");
+    gwy_plain_tool_connect_selection(plain_tool, GWY_TYPE_LAYER_POINT, "pointer");
 
     gwy_tool_grain_measure_init_dialog(tool);
 }
@@ -190,15 +176,15 @@ render_value(G_GNUC_UNUSED GtkTreeViewColumn *column,
     const gchar *name;
     gint i;
 
-    gtk_tree_model_get(model, iter, GWY_GRAIN_VALUE_STORE_COLUMN_ITEM, &gvalue, -1);
+    gtk_tree_model_get(model, iter, GWY_GRAIN_STORE_COLUMN_ITEM, &gvalue, -1);
     if (tool->gno <= 0 || !gvalue) {
         g_object_set(renderer, "text", "", NULL);
-        GWY_OBJECT_UNREF(gvalue);
+        g_clear_object(&gvalue);
         return;
     }
 
     g_object_unref(gvalue);
-    if (!tool->same_units && (gwy_grain_value_get_flags(gvalue) & GWY_GRAIN_VALUE_SAME_UNITS)) {
+    if (!tool->same_units && (gwy_grain_value_get_flags(gvalue) & GWY_GRAIN_SAME_UNITS)) {
         g_object_set(renderer, "text", _("N.A."), NULL);
         return;
     }
@@ -222,26 +208,26 @@ render_value(G_GNUC_UNUSED GtkTreeViewColumn *column,
     values = g_ptr_array_index(tool->values, i);
     value = values[tool->gno];
 
-    if (gwy_grain_value_get_flags(gvalue) & GWY_GRAIN_VALUE_IS_ANGLE) {
-        g_snprintf(buf, sizeof(buf), "%.1f deg", 180.0/G_PI*value);
+    if (gwy_grain_value_get_flags(gvalue) & GWY_GRAIN_IS_ANGLE) {
+        g_snprintf(buf, sizeof(buf), "%.1f deg", gwy_rad2deg(value));
         g_object_set(renderer, "text", buf, NULL);
     }
-    else if (quantity == GWY_GRAIN_VALUE_PIXEL_AREA) {
+    else if (quantity == GWY_GRAIN_PIXEL_AREA) {
         g_snprintf(buf, sizeof(buf), "%.0f", value);
         g_object_set(renderer, "text", buf, NULL);
     }
     else {
-        GwySIUnit *siunitxy, *siunitz;
-        GwyDataField *field;
-        GwySIUnitFormatStyle style = GWY_SI_UNIT_FORMAT_VFMARKUP;
+        GwyUnit *siunitxy, *siunitz;
+        GwyField *field;
+        GwyUnitFormatStyle style = GWY_UNIT_FORMAT_VFMARKUP;
 
-        field = GWY_PLAIN_TOOL(tool)->data_field;
-        siunitxy = gwy_data_field_get_si_unit_xy(field);
-        siunitz = gwy_data_field_get_si_unit_z(field);
-        tool->siunit = gwy_si_unit_power_multiply(siunitxy, gwy_grain_value_get_power_xy(gvalue),
-                                                  siunitz, gwy_grain_value_get_power_z(gvalue),
-                                                  tool->siunit);
-        tool->vf = gwy_si_unit_get_format_with_digits(tool->siunit, style, value, 3, tool->vf);
+        field = GWY_PLAIN_TOOL(tool)->field;
+        siunitxy = gwy_field_get_unit_xy(field);
+        siunitz = gwy_field_get_unit_z(field);
+        tool->siunit = gwy_unit_power_multiply(siunitxy, gwy_grain_value_get_power_xy(gvalue),
+                                               siunitz, gwy_grain_value_get_power_z(gvalue),
+                                               tool->siunit);
+        tool->vf = gwy_unit_get_format_with_digits(tool->siunit, style, value, 3, tool->vf);
         g_snprintf(buf, sizeof(buf), "%.*f%s%s",
                    tool->vf->precision, value/tool->vf->magnitude,
                    *tool->vf->units ? " " : "", tool->vf->units);
@@ -306,9 +292,7 @@ gwy_tool_grain_measure_data_switched(GwyTool *gwytool,
         return;
 
     if (data_view) {
-        gwy_object_set_or_reset(plain_tool->layer, tool->layer_type_point,
-                                "draw-marker", TRUE,
-                                "marker-radius", 0,
+        gwy_object_set_or_reset(plain_tool->layer, GWY_TYPE_LAYER_POINT,
                                 "editable", TRUE,
                                 "focus", -1,
                                 NULL);
@@ -354,13 +338,13 @@ gwy_tool_grain_measure_selection_changed(GwyPlainTool *plain_tool,
     if (!plain_tool->mask_field || !gwy_selection_get_object(plain_tool->selection, 0, point))
         tool->gno = 0;
     else {
-        row = floor(gwy_data_field_rtoi(plain_tool->mask_field, point[1]));
-        col = floor(gwy_data_field_rtoj(plain_tool->mask_field, point[0]));
-        if (gwy_data_field_get_val(plain_tool->mask_field, col, row)) {
+        row = floor(gwy_field_rtoi(plain_tool->mask_field, point[1]));
+        col = floor(gwy_field_rtoj(plain_tool->mask_field, point[0]));
+        if (gwy_field_get_val(plain_tool->mask_field, col, row)) {
             if (!tool->grains)
                 recalculate(tool);
 
-            xres = gwy_data_field_get_xres(plain_tool->mask_field);
+            xres = gwy_field_get_xres(plain_tool->mask_field);
             tool->gno = tool->grains[row*xres + col];
         }
         else
@@ -392,27 +376,25 @@ invalidate(GwyToolGrainMeasure *tool)
 static void
 update_units(GwyToolGrainMeasure *tool)
 {
-    GwyDataField *field = GWY_PLAIN_TOOL(tool)->data_field;
-    GwySIUnit *siunitxy = gwy_data_field_get_si_unit_xy(field);
-    GwySIUnit *siunitz = gwy_data_field_get_si_unit_z(field);
+    GwyField *field = GWY_PLAIN_TOOL(tool)->field;
+    GwyUnit *siunitxy = gwy_field_get_unit_xy(field);
+    GwyUnit *siunitz = gwy_field_get_unit_z(field);
 
-    tool->same_units = gwy_si_unit_equal(siunitxy, siunitz);
+    tool->same_units = gwy_unit_equal(siunitxy, siunitz);
 }
 
 static void
 recalculate(GwyToolGrainMeasure *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    GwyDataField *field = plain_tool->data_field;
-    GwyDataField *mask = plain_tool->mask_field;
+    GwyField *field = plain_tool->field;
+    // TODO Nield
+    GwyNield *grains = NULL; //plain_tool->mask_field;
     GwyInventory *inventory;
     GwyGrainValue **gvalues;
     guint n, i;
 
-    if (!tool->grains) {
-        tool->grains = g_new0(gint, gwy_data_field_get_xres(field) *gwy_data_field_get_yres(field));
-        tool->ngrains = gwy_data_field_number_grains(mask, tool->grains);
-    }
+    tool->ngrains = gwy_nield_max(grains);
 
     inventory = gwy_grain_values();
     n = gwy_inventory_get_n_items(inventory);
@@ -428,7 +410,7 @@ recalculate(GwyToolGrainMeasure *tool)
         g_ptr_array_index(tool->values, i) = g_renew(gdouble, g_ptr_array_index(tool->values, i), tool->ngrains+1);
     }
 
-    gwy_grain_values_calculate(n, gvalues, (gdouble**)tool->values->pdata, field, tool->ngrains, tool->grains);
+    gwy_grain_values_calculate(gvalues, (gdouble**)tool->values->pdata, n, field, grains);
     g_free(gvalues);
 }
 
