@@ -1,5 +1,5 @@
 /*
- *  $Id: icolorange.c 26745 2024-10-18 15:17:53Z yeti-dn $
+ *  $Id: icolorange.c 29092 2026-01-06 16:14:51Z yeti-dn $
  *  Copyright (C) 2003-2022 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,22 +16,11 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwyddion/gwycontainer.h>
-#include <libgwymodule/gwymodule-tool.h>
-#include <libprocess/stats.h>
-#include <libprocess/linestats.h>
-#include <libgwydgets/gwygraph.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwydgets/gwyradiobuttons.h>
-#include <libgwydgets/gwylayer-basic.h>
-#include <app/gwyapp.h>
-#include <libgwydgets/gwydgetutils.h>
+#include <gwy.h>
 
 #define APP_RANGE_KEY "/app/default-range-type"
 
@@ -81,7 +70,7 @@ struct _GwyToolColorRange {
 
     GwyGraph *histogram;
     GwyGraphModel *histogram_model;
-    GwyDataLine *heightdist;
+    GwyLine *heightdist;
     GwySelection *graph_selection;
     gint isel[4];
     gdouble rsel[4];
@@ -93,50 +82,47 @@ struct _GwyToolColorRange {
     GtkWidget *is_default;
 
     GQuark key_min_max[2];
-
-    /* potential class data */
-    GType layer_type_rect;
 };
 
 struct _GwyToolColorRangeClass {
     GwyPlainToolClass parent_class;
 };
 
-static gboolean               module_register                       (void);
-static GwyParamDef*           define_module_params                  (void);
-static GType                  gwy_tool_color_range_get_type         (void)                               G_GNUC_CONST;
-static void                   gwy_tool_color_range_finalize         (GObject *object);
-static void                   gwy_tool_color_range_init_dialog      (GwyToolColorRange *tool);
-static void                   gwy_tool_color_range_data_switched    (GwyTool *gwytool,
-                                                                     GwyDataView *data_view);
-static void                   gwy_tool_color_range_data_changed     (GwyPlainTool *plain_tool);
-static void                   gwy_tool_color_range_mask_changed     (GwyPlainTool *plain_tool);
-static void                   gwy_tool_color_range_selection_changed(GwyPlainTool *plain_tool,
-                                                                     gint hint);
-static void                   gwy_tool_color_range_response         (GwyTool *tool,
-                                                                     gint response_id);
-static void                   make_keys                             (GwyToolColorRange *tool,
-                                                                     GwyDataView *data_view);
-static void                   xsel_changed                          (GwySelection *selection,
-                                                                     gint hint,
-                                                                     GwyToolColorRange *tool);
-static void                   type_changed                          (GtkWidget *radio,
-                                                                     GwyToolColorRange *tool);
-static void                   set_default_mode                      (GtkToggleButton *check,
-                                                                     GwyToolColorRange *tool);
-static GwyLayerBasicRangeType get_range_type                        (GwyToolColorRange *tool);
-static void                   set_range_type                        (GwyToolColorRange *tool,
-                                                                     GwyLayerBasicRangeType range_type);
-static void                   get_min_max                           (GwyToolColorRange *tool,
-                                                                     gdouble *selection);
-static void                   set_min_max                           (GwyToolColorRange *tool);
-static void                   update_fullrange                      (GwyToolColorRange *tool);
-static void                   update_histogram                      (GwyToolColorRange *tool);
-static void                   param_changed                         (GwyToolColorRange *tool,
-                                                                     gint id);
-static void                   invert_mapping                        (GwyToolColorRange *tool);
-static void                   set_zero_to_minimum                   (GwyToolColorRange *tool);
-static void                   update_selected_rectangle             (GwyToolColorRange *tool);
+static gboolean            module_register                       (void);
+static GwyParamDef*        define_module_params                  (void);
+static GType               gwy_tool_color_range_get_type         (void)                         G_GNUC_CONST;
+static void                gwy_tool_color_range_finalize         (GObject *object);
+static void                gwy_tool_color_range_init_dialog      (GwyToolColorRange *tool);
+static void                gwy_tool_color_range_data_switched    (GwyTool *gwytool,
+                                                                  GwyDataView *data_view);
+static void                gwy_tool_color_range_data_changed     (GwyPlainTool *plain_tool);
+static void                gwy_tool_color_range_mask_changed     (GwyPlainTool *plain_tool);
+static void                gwy_tool_color_range_selection_changed(GwyPlainTool *plain_tool,
+                                                                  gint hint);
+static void                gwy_tool_color_range_response         (GwyTool *tool,
+                                                                  gint response_id);
+static void                make_keys                             (GwyToolColorRange *tool,
+                                                                  GwyDataView *data_view);
+static void                xsel_changed                          (GwySelection *selection,
+                                                                  gint hint,
+                                                                  GwyToolColorRange *tool);
+static void                type_changed                          (GtkWidget *radio,
+                                                                  GwyToolColorRange *tool);
+static void                set_default_mode                      (GtkToggleButton *check,
+                                                                  GwyToolColorRange *tool);
+static GwyColorMappingType get_range_type                        (GwyToolColorRange *tool);
+static void                set_range_type                        (GwyToolColorRange *tool,
+                                                                  GwyColorMappingType mapping);
+static void                get_min_max                           (GwyToolColorRange *tool,
+                                                                  gdouble *selection);
+static void                set_min_max                           (GwyToolColorRange *tool);
+static void                update_fullrange                      (GwyToolColorRange *tool);
+static void                update_histogram                      (GwyToolColorRange *tool);
+static void                param_changed                         (GwyToolColorRange *tool,
+                                                                  gint id);
+static void                invert_mapping                        (GwyToolColorRange *tool);
+static void                set_zero_to_minimum                   (GwyToolColorRange *tool);
+static void                update_selected_rectangle             (GwyToolColorRange *tool);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
@@ -144,7 +130,7 @@ static GwyModuleInfo module_info = {
     N_("Interactive color range tool, allows selecting the data range false color scale should map to, "
        "either on data or on height distribution histogram."),
     "Yeti <yeti@gwyddion.net>",
-    "4.2",
+    "4.3",
     "David Nečas (Yeti) & Petr Klapetek",
     "2004",
 };
@@ -188,7 +174,7 @@ gwy_tool_color_range_class_init(GwyToolColorRangeClass *klass)
 
     gobject_class->finalize = gwy_tool_color_range_finalize;
 
-    tool_class->stock_id = GWY_STOCK_COLOR_RANGE;
+    tool_class->icon_name = GWY_ICON_COLOR_RANGE;
     tool_class->title = _("Color Range");
     tool_class->tooltip = _("Stretch color range to part of data");
     tool_class->prefix = "/module/colorrange";
@@ -206,8 +192,8 @@ gwy_tool_color_range_finalize(GObject *object)
     GwyToolColorRange *tool = GWY_TOOL_COLOR_RANGE(object);
 
     gwy_params_save_to_settings(tool->params);
-    GWY_OBJECT_UNREF(tool->params);
-    GWY_OBJECT_UNREF(tool->heightdist);
+    g_clear_object(&tool->params);
+    g_clear_object(&tool->heightdist);
 
     G_OBJECT_CLASS(gwy_tool_color_range_parent_class)->finalize(object);
 }
@@ -218,17 +204,13 @@ gwy_tool_color_range_init(GwyToolColorRange *tool)
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
     GwyContainer *settings = gwy_app_settings_get();
 
-    tool->layer_type_rect = gwy_plain_tool_check_layer_type(plain_tool, "GwyLayerRectangle");
-    if (!tool->layer_type_rect)
-        return;
-
     tool->params = gwy_params_new_from_settings(define_module_params());
     if (!gwy_container_contains_by_name(settings, APP_RANGE_KEY))
-        gwy_container_set_enum_by_name(settings, APP_RANGE_KEY, GWY_LAYER_BASIC_RANGE_FULL);
+        gwy_container_set_enum_by_name(settings, APP_RANGE_KEY, GWY_COLOR_MAPPING_FULL);
 
-    plain_tool->unit_style = GWY_SI_UNIT_FORMAT_VFMARKUP;
+    plain_tool->unit_style = GWY_UNIT_FORMAT_VFMARKUP;
 
-    gwy_plain_tool_connect_selection(plain_tool, tool->layer_type_rect, "rectangle");
+    gwy_plain_tool_connect_selection(plain_tool, GWY_TYPE_LAYER_RECTANGLE, "rectangle");
 
     gwy_tool_color_range_init_dialog(tool);
 }
@@ -238,7 +220,7 @@ gwy_tool_crop_rect_updated(GwyToolColorRange *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
 
-    gwy_rect_selection_labels_select(tool->rlabels, plain_tool->selection, plain_tool->data_field);
+    gwy_rect_selection_labels_select(tool->rlabels, plain_tool->selection, plain_tool->field);
 }
 
 static void
@@ -251,26 +233,26 @@ gwy_tool_color_range_init_dialog(GwyToolColorRange *tool)
     }
     const range_types[] = {
         {
-            GWY_LAYER_BASIC_RANGE_FULL, GWY_STOCK_COLOR_RANGE_FULL,
+            GWY_COLOR_MAPPING_FULL, GWY_ICON_COLOR_RANGE_FULL,
             N_("Full color range from minimum to maximum"),
         },
         {
-            GWY_LAYER_BASIC_RANGE_FIXED, GWY_STOCK_COLOR_RANGE_FIXED,
+            GWY_COLOR_MAPPING_FIXED, GWY_ICON_COLOR_RANGE_FIXED,
             N_("Explicitly set fixed color range"),
         },
         {
-            GWY_LAYER_BASIC_RANGE_AUTO, GWY_STOCK_COLOR_RANGE_AUTO,
+            GWY_COLOR_MAPPING_AUTO, GWY_ICON_COLOR_RANGE_AUTO,
             N_("Automatic color range with tails cut off"),
         },
         {
-            GWY_LAYER_BASIC_RANGE_ADAPT, GWY_STOCK_COLOR_RANGE_ADAPTIVE,
+            GWY_COLOR_MAPPING_ADAPT, GWY_ICON_COLOR_RANGE_ADAPTIVE,
             N_("Adaptive nonlinear color mapping"),
         },
     };
 
     GtkDialog *dialog = GTK_DIALOG(GWY_TOOL(tool)->dialog);
     GtkWidget *hbox, *button, *image;
-    GwyLayerBasicRangeType range_type = GWY_LAYER_BASIC_RANGE_FULL;
+    GwyColorMappingType mapping = GWY_COLOR_MAPPING_FULL;
     GtkRadioButton *group;
     GwyParamTable *table;
     GwyGraphCurveModel *cmodel;
@@ -278,18 +260,21 @@ gwy_tool_color_range_init_dialog(GwyToolColorRange *tool)
     gint i;
 
     /* Mode switch */
-    hbox = gwy_hbox_new(0);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
     gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(dialog)), hbox, FALSE, FALSE, 0);
+    GtkWidget *label = gtk_label_new("Mapping type:");
+    gtk_widget_set_margin_end(label, 6);
+    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 
     group = NULL;
     for (i = 0; i < G_N_ELEMENTS(range_types); i++) {
         button = gtk_radio_button_new_from_widget(group);
         g_object_set(button, "draw-indicator", FALSE, NULL);
-        image = gtk_image_new_from_stock(range_types[i].stock_id, GTK_ICON_SIZE_LARGE_TOOLBAR);
+        image = gtk_image_new_from_icon_name(range_types[i].stock_id, GTK_ICON_SIZE_LARGE_TOOLBAR);
         gtk_container_add(GTK_CONTAINER(button), image);
         gwy_radio_button_set_value(button, range_types[i].type);
         gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
-        gtk_widget_set_tooltip_text(button, gettext(range_types[i].text));
+        gtk_widget_set_tooltip_text(button, _(range_types[i].text));
         g_signal_connect(button, "clicked", G_CALLBACK(type_changed), tool);
         if (!group)
             group = GTK_RADIO_BUTTON(button);
@@ -297,12 +282,12 @@ gwy_tool_color_range_init_dialog(GwyToolColorRange *tool)
     tool->modelist = gtk_radio_button_get_group(group);
 
     /* Is default */
-    tool->is_default = gtk_check_button_new_with_mnemonic(_("_default"));
+    tool->is_default = gtk_check_button_new_with_mnemonic(_("is _default"));
     gtk_box_pack_start(GTK_BOX(hbox), tool->is_default, FALSE, FALSE, 4);
     g_signal_connect(tool->is_default, "toggled", G_CALLBACK(set_default_mode), tool);
 
     /* Height distribution */
-    tool->heightdist = gwy_data_line_new(1.0, 1.0, TRUE);
+    tool->heightdist = gwy_line_new(1.0, 1.0, TRUE);
     cmodel = gwy_graph_curve_model_new();
     g_object_set(cmodel, "description", _("Height histogram"), "mode", GWY_GRAPH_CURVE_LINE, NULL);
 
@@ -313,7 +298,6 @@ gwy_tool_color_range_init_dialog(GwyToolColorRange *tool)
     garea = GWY_GRAPH_AREA(gwy_graph_get_area(tool->histogram));
     gtk_widget_set_size_request(GTK_WIDGET(garea), -1, 48);
     tool->graph_selection = gwy_graph_area_get_selection(garea, GWY_GRAPH_STATUS_XSEL);
-    g_return_if_fail(GWY_IS_SELECTION_GRAPH_1DAREA(tool->graph_selection));
     gwy_selection_set_max_objects(tool->graph_selection, 1);
     g_signal_connect(tool->graph_selection, "changed", G_CALLBACK(xsel_changed), tool);
 
@@ -346,18 +330,20 @@ gwy_tool_color_range_init_dialog(GwyToolColorRange *tool)
     gwy_param_table_append_button(table, BUTTON_SET_ZERO, -1, RESPONSE_SET_ZERO, _("Set Zero to Color Map Minimum"));
 
     gwy_plain_tool_add_param_table(GWY_PLAIN_TOOL(tool), table);
-    gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(dialog)), gwy_param_table_widget(table), FALSE, FALSE, 0);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 12);
+    gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(dialog)), hbox, FALSE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(hbox), gwy_param_table_widget(table), TRUE, TRUE, 0);
 
     /* Selection info */
-    tool->rlabels = gwy_rect_selection_labels_new (TRUE, G_CALLBACK(gwy_tool_crop_rect_updated), tool);
-    gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(dialog)), gwy_rect_selection_labels_get_table(tool->rlabels), FALSE, FALSE, 0);
+    tool->rlabels = gwy_rect_selection_labels_new(TRUE, G_CALLBACK(gwy_tool_crop_rect_updated), tool);
+    gtk_box_pack_start(GTK_BOX(hbox), gwy_rect_selection_labels_get_table(tool->rlabels), FALSE, FALSE, 0);
 
     gwy_tool_add_hide_button(GWY_TOOL(tool), TRUE);
     gwy_help_add_to_tool_dialog(dialog, GWY_TOOL(tool), GWY_HELP_DEFAULT);
 
     /* Switch to the default */
-    gwy_container_gis_enum_by_name(gwy_app_settings_get(), APP_RANGE_KEY, &range_type);
-    gwy_radio_buttons_set_current(tool->modelist, range_type);
+    gwy_container_gis_enum_by_name(gwy_app_settings_get(), APP_RANGE_KEY, &mapping);
+    gwy_radio_buttons_set_current(tool->modelist, mapping);
     type_changed(NULL, tool);
 
     g_signal_connect_swapped(tool->table, "param-changed", G_CALLBACK(param_changed), tool);
@@ -372,9 +358,7 @@ gwy_tool_color_range_data_switched(GwyTool *gwytool,
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(gwytool);
     GwyToolColorRange *tool = GWY_TOOL_COLOR_RANGE(gwytool);
     gboolean ignore = (data_view == plain_tool->data_view);
-    GwyLayerBasicRangeType range_type;
-
-    make_keys(tool, data_view);
+    GwyColorMappingType mapping;
 
     gwy_debug("A");
     tool->data_switch = TRUE;
@@ -382,11 +366,13 @@ gwy_tool_color_range_data_switched(GwyTool *gwytool,
     tool->data_switch = FALSE;
     gwy_debug("B");
 
+    make_keys(tool, data_view);
+
     if (plain_tool->init_failed)
         return;
 
     if (data_view) {
-        gwy_object_set_or_reset(plain_tool->layer, tool->layer_type_rect,
+        gwy_object_set_or_reset(plain_tool->layer, GWY_TYPE_LAYER_RECTANGLE,
                                 "editable", TRUE,
                                 "focus", -1,
                                 NULL);
@@ -411,9 +397,9 @@ gwy_tool_color_range_data_switched(GwyTool *gwytool,
     if (ignore)
         return;
 
-    range_type = get_range_type(tool);
+    mapping = get_range_type(tool);
     if (data_view) {
-        if (range_type == GWY_LAYER_BASIC_RANGE_FIXED) {
+        if (mapping == GWY_COLOR_MAPPING_FIXED) {
             gdouble sel[2];
 
             get_min_max(tool, sel);
@@ -427,7 +413,7 @@ gwy_tool_color_range_data_switched(GwyTool *gwytool,
         type_changed(NULL, tool);
         tool->programmatic_update = FALSE;
     }
-    gwy_radio_buttons_set_current(tool->modelist, range_type);
+    gwy_radio_buttons_set_current(tool->modelist, mapping);
     update_fullrange(tool);
     gwy_tool_color_range_mask_changed(plain_tool);
     gwy_debug("set min max after data switch");
@@ -438,26 +424,15 @@ static void
 make_keys(GwyToolColorRange *tool,
           GwyDataView *data_view)
 {
-    GwyPixmapLayer *layer;
-    const gchar *dkey;
-    gchar key[32];
-    gint id;
+    GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
 
-    if (!data_view) {
+    if (!data_view || plain_tool->id < 0) {
         gwy_clear(tool->key_min_max, 2);
         return;
     }
 
-    layer = gwy_data_view_get_base_layer(data_view);
-    g_return_if_fail(GWY_IS_PIXMAP_LAYER(layer));
-    dkey = gwy_pixmap_layer_get_data_key(layer);
-    g_return_if_fail(dkey && dkey[0] == '/' && g_ascii_isdigit(dkey[1]));
-    id = atoi(dkey + 1);
-
-    g_snprintf(key, sizeof(key), "/%d/base/min", id);
-    tool->key_min_max[0] = g_quark_from_string(key);
-    g_snprintf(key, sizeof(key), "/%d/base/max", id);
-    tool->key_min_max[1] = g_quark_from_string(key);
+    tool->key_min_max[0] = gwy_file_key_image_range_min(plain_tool->id);
+    tool->key_min_max[1] = gwy_file_key_image_range_max(plain_tool->id);
 }
 
 static void
@@ -470,7 +445,7 @@ gwy_tool_color_range_data_changed(GwyPlainTool *plain_tool)
     update_fullrange(tool);
     /* When the range is defined by selection, follow data updates. When it is given on the histogram or numerically,
      * don't. */
-    if (plain_tool->data_field
+    if (plain_tool->field
         && tool->range_source == USE_SELECTION
         && gwy_params_get_boolean(tool->params, PARAM_USE_SELECTION))
         gwy_tool_color_range_selection_changed(plain_tool, -1);
@@ -481,8 +456,8 @@ gwy_tool_color_range_mask_changed(GwyPlainTool *plain_tool)
 {
     GwyToolColorRange *tool = GWY_TOOL_COLOR_RANGE(plain_tool);
     gboolean have_mask = !!plain_tool->mask_field;
-    gboolean range_type = get_range_type(tool);
-    gboolean msens = (have_mask && range_type == GWY_LAYER_BASIC_RANGE_FIXED);
+    gboolean mapping = get_range_type(tool);
+    gboolean msens = (have_mask && mapping == GWY_COLOR_MAPPING_FIXED);
 
     gwy_param_table_set_sensitive(tool->table, BUTTON_SET_TO_MASKED, msens);
     gwy_param_table_set_sensitive(tool->table, BUTTON_SET_TO_UNMASKED, msens);
@@ -493,7 +468,7 @@ gwy_tool_color_range_selection_changed(GwyPlainTool *plain_tool,
                                        gint hint)
 {
     GwyToolColorRange *tool = GWY_TOOL_COLOR_RANGE(plain_tool);
-    GwyLayerBasicRangeType range_type;
+    GwyColorMappingType mapping;
     gboolean is_selected = FALSE;
     gdouble range[4];
 
@@ -503,8 +478,8 @@ gwy_tool_color_range_selection_changed(GwyPlainTool *plain_tool,
     if (plain_tool->selection)
         is_selected = gwy_selection_get_data(plain_tool->selection, NULL);
 
-    range_type = get_range_type(tool);
-    if (range_type != GWY_LAYER_BASIC_RANGE_FIXED || !gwy_params_get_boolean(tool->params, PARAM_USE_SELECTION))
+    mapping = get_range_type(tool);
+    if (mapping != GWY_COLOR_MAPPING_FIXED || !gwy_params_get_boolean(tool->params, PARAM_USE_SELECTION))
         return;
 
     if (!tool->programmatic_update)
@@ -562,21 +537,21 @@ type_changed(GtkWidget *radio,
              GwyToolColorRange *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    GwyLayerBasicRangeType range_type, old_mode;
+    GwyColorMappingType mapping, old_mode;
     gboolean fixed_sens = FALSE;
 
     old_mode = get_range_type(tool);
     if (radio) {
-        range_type = gwy_radio_button_get_value(radio);
-        if (old_mode == range_type)
+        mapping = gwy_radio_button_get_value(radio);
+        if (old_mode == mapping)
             return;
     }
     else
-        range_type = old_mode;  /* Initialization */
+        mapping = old_mode;  /* Initialization */
 
     if (plain_tool->container) {
-        fixed_sens = (range_type == GWY_LAYER_BASIC_RANGE_FIXED);
-        set_range_type(tool, range_type);
+        fixed_sens = (mapping == GWY_COLOR_MAPPING_FIXED);
+        set_range_type(tool, mapping);
         if (fixed_sens && !tool->data_switch) {
             gwy_debug("set min max after range type change");
             set_min_max(tool);
@@ -591,8 +566,8 @@ type_changed(GtkWidget *radio,
 
     old_mode = -1;
     gwy_container_gis_enum_by_name(gwy_app_settings_get(), APP_RANGE_KEY, &old_mode);
-    gtk_widget_set_sensitive(tool->is_default, old_mode != range_type);
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tool->is_default), old_mode == range_type);
+    gtk_widget_set_sensitive(tool->is_default, old_mode != mapping);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tool->is_default), old_mode == mapping);
     gwy_tool_color_range_mask_changed(plain_tool);
 }
 
@@ -608,47 +583,28 @@ set_default_mode(GtkToggleButton *check,
     gtk_widget_set_sensitive(tool->is_default, FALSE);
 }
 
-static GwyLayerBasicRangeType
+static GwyColorMappingType
 get_range_type(GwyToolColorRange *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    GwyLayerBasicRangeType range_type = GWY_LAYER_BASIC_RANGE_FULL;
-    GwyPixmapLayer *layer;
+    GwyColorMappingType mapping = GWY_COLOR_MAPPING_FULL;
 
     plain_tool = GWY_PLAIN_TOOL(tool);
-    if (plain_tool->data_view) {
-        layer = gwy_data_view_get_base_layer(plain_tool->data_view);
-        range_type = gwy_layer_basic_get_range_type(GWY_LAYER_BASIC(layer));
-    }
+    if (plain_tool->data_view)
+        mapping = gwy_data_view_get_color_mapping(plain_tool->data_view);
     else
-        gwy_container_gis_enum_by_name(gwy_app_settings_get(), APP_RANGE_KEY, &range_type);
+        gwy_container_gis_enum_by_name(gwy_app_settings_get(), APP_RANGE_KEY, &mapping);
 
-    return range_type;
+    return mapping;
 }
 
 static void
 set_range_type(GwyToolColorRange *tool,
-               GwyLayerBasicRangeType range_type)
+               GwyColorMappingType mapping)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    GwyPixmapLayer *layer;
-    const gchar *key;
-    gchar buf[32];
-
     g_return_if_fail(plain_tool->data_view);
-
-    layer = gwy_data_view_get_base_layer(plain_tool->data_view);
-    key = gwy_layer_basic_get_range_type_key(GWY_LAYER_BASIC(layer));
-    if (!key) {
-        g_warning("Setting range type key.  This should be done by the app.");
-
-        g_snprintf(buf, sizeof(buf), "/%d/base", plain_tool->id);
-        gwy_layer_basic_set_min_max_key(GWY_LAYER_BASIC(layer), buf);
-        strncat(buf, "/range-type", sizeof(buf)-1);
-        gwy_layer_basic_set_range_type_key(GWY_LAYER_BASIC(layer), buf);
-        key = buf;
-    }
-    gwy_container_set_enum_by_name(plain_tool->container, key, range_type);
+    gwy_container_set_enum(plain_tool->container, gwy_file_key_image_color_mapping(plain_tool->id), mapping);
 }
 
 static void
@@ -658,10 +614,10 @@ get_min_max(GwyToolColorRange *tool,
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
     gint i;
 
-    g_return_if_fail(plain_tool->data_view && plain_tool->data_field);
+    g_return_if_fail(plain_tool->data_view && plain_tool->field);
 
     for (i = 0; i < 2; i++) {
-        selection[i] = gwy_data_field_get_min(plain_tool->data_field);
+        selection[i] = gwy_field_get_min(plain_tool->field);
         gwy_container_gis_double(plain_tool->container, tool->key_min_max[i], &selection[i]);
     }
 }
@@ -670,7 +626,7 @@ static void
 set_min_max(GwyToolColorRange *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    const GwySIValueFormat *vf = plain_tool->value_format;
+    const GwyValueFormat *vf = plain_tool->value_format;
     gboolean clear = FALSE;
     gdouble sel[2];
     gint i, col, row, w, h;
@@ -692,8 +648,8 @@ set_min_max(GwyToolColorRange *tool)
         if (!plain_tool->selection || !gwy_selection_get_data(plain_tool->selection, NULL) || w <= 1 || h <= 1)
             clear = TRUE;
         else {
-            gwy_data_field_area_get_min_max_mask(plain_tool->data_field, NULL, GWY_MASK_IGNORE, col, row, w, h,
-                                                 &sel[0], &sel[1]);
+            gwy_field_area_get_min_max(plain_tool->field, NULL, GWY_MASK_IGNORE, col, row, w, h,
+                                       &sel[0], &sel[1]);
         }
     }
     else if (tool->range_source == USE_HISTOGRAM) {
@@ -709,7 +665,7 @@ set_min_max(GwyToolColorRange *tool)
     if (clear) {
         for (i = 0; i < 2; i++)
             gwy_container_remove(plain_tool->container, tool->key_min_max[i]);
-        gwy_data_field_get_min_max(plain_tool->data_field, &sel[0], &sel[1]);
+        gwy_field_get_min_max(plain_tool->field, &sel[0], &sel[1]);
     }
     else {
         for (i = 0; i < 2; i++)
@@ -730,12 +686,12 @@ static void
 update_fullrange(GwyToolColorRange *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    const GwySIValueFormat *vf = plain_tool->value_format;
+    const GwyValueFormat *vf = plain_tool->value_format;
     gdouble min, max;
     gchar buf[64];
 
     if (plain_tool->container) {
-        gwy_data_field_get_min_max(plain_tool->data_field, &min, &max);
+        gwy_field_get_min_max(plain_tool->field, &min, &max);
         g_snprintf(buf, sizeof(buf), "%.*f%s%s", vf->precision, min/vf->magnitude, *vf->units ? " " : "", vf->units);
         gwy_param_table_info_set_valuestr(tool->table, INFO_MINIMUM, buf);
         g_snprintf(buf, sizeof(buf), "%.*f%s%s", vf->precision, max/vf->magnitude, *vf->units ? " " : "", vf->units);
@@ -755,23 +711,23 @@ update_histogram(GwyToolColorRange *tool)
     gdouble data[2] = { 0.0, 0.0 };
 
     cmodel = gwy_graph_model_get_curve(tool->histogram_model, 0);
-    if (!plain_tool->data_field) {
+    if (!plain_tool->field) {
         gwy_graph_curve_model_set_data(cmodel, data, data, G_N_ELEMENTS(data));
         return;
     }
 
-    gwy_data_field_dh(plain_tool->data_field, tool->heightdist, 0);
+    gwy_field_dh(plain_tool->field, tool->heightdist, 0);
     /* rescale to sqrt to make more readable  */
-    gwy_data_line_sqrt(tool->heightdist);
+    gwy_line_sqrt(tool->heightdist);
 
-    gwy_graph_curve_model_set_data_from_dataline(cmodel, tool->heightdist, 0, 0);
+    gwy_graph_curve_model_set_data_from_line(cmodel, tool->heightdist, 0, 0);
 }
 
 static void
 param_changed(GwyToolColorRange *tool, gint id)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    const GwySIValueFormat *vf = plain_tool->value_format;
+    const GwyValueFormat *vf = plain_tool->value_format;
     gdouble sel[2];
     gint i;
 
@@ -799,18 +755,18 @@ static void
 set_range_using_mask(GwyToolColorRange *tool, GwyMaskingType masking)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    GwyDataField *dfield = plain_tool->data_field, *mask = plain_tool->mask_field;
-    const GwySIValueFormat *vf = plain_tool->value_format;
+    GwyField *dfield = plain_tool->field, *mask = plain_tool->mask_field;
+    const GwyValueFormat *vf = plain_tool->value_format;
     gdouble sel[2];
     gint i;
 
     if (!dfield)
         return;
 
-    gwy_data_field_area_get_min_max_mask(dfield, mask, masking, 0, 0, dfield->xres, dfield->yres, &sel[0], &sel[1]);
+    gwy_field_area_get_min_max(dfield, mask, masking, 0, 0, dfield->xres, dfield->yres, &sel[0], &sel[1]);
     /* No valid pixels? */
     if (sel[1] < sel[0])
-        gwy_data_field_get_min_max(dfield, &sel[0], &sel[1]);
+        gwy_field_get_min_max(dfield, &sel[0], &sel[1]);
 
     for (i = 0; i < 2; i++)
         gwy_container_set_double(plain_tool->container, tool->key_min_max[i], sel[i]);
@@ -846,7 +802,7 @@ invert_mapping(GwyToolColorRange *tool)
 {
     gdouble min, max;
 
-    if (!GWY_PLAIN_TOOL(tool)->data_field)
+    if (!GWY_PLAIN_TOOL(tool)->field)
         return;
 
     min = gwy_params_get_double(tool->params, PARAM_START);
@@ -864,8 +820,8 @@ static void
 set_zero_to_minimum(GwyToolColorRange *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    GwyDataField *dfield = plain_tool->data_field;
-    const GwySIValueFormat *vf = plain_tool->value_format;
+    GwyField *dfield = plain_tool->field;
+    const GwyValueFormat *vf = plain_tool->value_format;
     gdouble min, max;
 
     if (!dfield)
@@ -875,8 +831,8 @@ set_zero_to_minimum(GwyToolColorRange *tool)
     max = gwy_params_get_double(tool->params, PARAM_END) * vf->magnitude;
 
     tool->programmatic_update = TRUE;
-    gwy_data_field_add(dfield, -min);
-    gwy_data_field_data_changed(dfield);
+    gwy_field_add(dfield, -min);
+    gwy_field_data_changed(dfield);
     /* XXX: Is this always correct when vf changes? */
     gwy_param_table_set_double(tool->table, PARAM_START, 0.0);
     gwy_param_table_set_double(tool->table, PARAM_END, (max - min)/vf->magnitude);
@@ -890,7 +846,7 @@ update_selected_rectangle(GwyToolColorRange *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
     GwySelection *selection = plain_tool->selection;
-    GwyDataField *field = plain_tool->data_field;
+    GwyField *field = plain_tool->field;
     gint n;
 
     n = selection ? gwy_selection_get_data(selection, NULL) : 0;
