@@ -1,5 +1,5 @@
 /*
- *  $Id: level3.c 26745 2024-10-18 15:17:53Z yeti-dn $
+ *  $Id: level3.c 28788 2025-11-04 17:13:02Z yeti-dn $
  *  Copyright (C) 2003-2022 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,15 +16,9 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwymodule/gwymodule-tool.h>
-#include <libprocess/level.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwydgets/gwynullstore.h>
-#include <libgwydgets/gwydgetutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 
 enum {
     COLUMN_I, COLUMN_X, COLUMN_Y, COLUMN_Z, NCOLUMNS
@@ -53,9 +47,6 @@ struct _GwyToolLevel3 {
     GtkTreeView *treeview;
     GtkTreeModel *model;
     GwyParamTable *table;
-
-    /* potential class data */
-    GType layer_type_point;
 };
 
 struct _GwyToolLevel3Class {
@@ -133,7 +124,7 @@ gwy_tool_level3_class_init(GwyToolLevel3Class *klass)
 
     gobject_class->finalize = gwy_tool_level3_finalize;
 
-    tool_class->stock_id = GWY_STOCK_LEVEL_TRIANGLE;
+    tool_class->icon_name = GWY_ICON_LEVEL_TRIANGLE;
     tool_class->title = _("Three Point Level");
     tool_class->tooltip = _("Level data by fitting a plane through three points");
     tool_class->prefix = "/module/level3";
@@ -151,10 +142,10 @@ gwy_tool_level3_finalize(GObject *object)
     GwyToolLevel3 *tool = GWY_TOOL_LEVEL3(object);
 
     gwy_params_save_to_settings(tool->params);
-    GWY_OBJECT_UNREF(tool->params);
+    g_clear_object(&tool->params);
     if (tool->model) {
         gtk_tree_view_set_model(tool->treeview, NULL);
-        GWY_OBJECT_UNREF(tool->model);
+        g_clear_object(&tool->model);
     }
 
     G_OBJECT_CLASS(gwy_tool_level3_parent_class)->finalize(object);
@@ -165,13 +156,9 @@ gwy_tool_level3_init(GwyToolLevel3 *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
 
-    tool->layer_type_point = gwy_plain_tool_check_layer_type(plain_tool, "GwyLayerPoint");
-    if (!tool->layer_type_point)
-        return;
-
-    plain_tool->unit_style = GWY_SI_UNIT_FORMAT_MARKUP;
+    plain_tool->unit_style = GWY_UNIT_FORMAT_MARKUP;
     tool->params = gwy_params_new_from_settings(define_module_params());
-    gwy_plain_tool_connect_selection(plain_tool, tool->layer_type_point, "point");
+    gwy_plain_tool_connect_selection(plain_tool, GWY_TYPE_LAYER_POINT, "point");
 
     gwy_tool_level3_init_dialog(tool);
 }
@@ -216,7 +203,7 @@ gwy_tool_level3_init_dialog(GwyToolLevel3 *tool)
 
     gwy_plain_tool_add_clear_button(GWY_PLAIN_TOOL(tool));
     gwy_tool_add_hide_button(GWY_TOOL(tool), FALSE);
-    gtk_dialog_add_button(dialog, GTK_STOCK_APPLY, GTK_RESPONSE_APPLY);
+    gwy_plain_tool_add_apply_button(GWY_PLAIN_TOOL(tool));
     gtk_dialog_set_default_response(dialog, GTK_RESPONSE_APPLY);
     gtk_dialog_set_response_sensitive(dialog, GTK_RESPONSE_APPLY, FALSE);
     gwy_help_add_to_tool_dialog(dialog, GWY_TOOL(tool), GWY_HELP_NO_BUTTON);
@@ -241,8 +228,8 @@ gwy_tool_level3_data_switched(GwyTool *gwytool,
         return;
 
     if (data_view) {
-        gwy_object_set_or_reset(plain_tool->layer, tool->layer_type_point,
-                                "marker-radius", gwy_params_get_int(tool->params, PARAM_RADIUS) - 1,
+        gwy_object_set_or_reset(plain_tool->layer, GWY_TYPE_LAYER_POINT,
+                                "marker-radius", gwy_params_get_int(tool->params, PARAM_RADIUS) - 1.0,
                                 "editable", TRUE,
                                 "focus", -1,
                                 NULL);
@@ -320,7 +307,7 @@ param_changed(GwyToolLevel3 *tool, gint id)
 
     if (id < 0 || id == PARAM_RADIUS) {
         if (plain_tool->layer)
-            g_object_set(plain_tool->layer, "marker-radius", gwy_params_get_int(params, PARAM_RADIUS)-1, NULL);
+            g_object_set(plain_tool->layer, "marker-radius", gwy_params_get_int(params, PARAM_RADIUS) - 1.0, NULL);
     }
     if (id < 0 || id == PARAM_INSTANT_APPLY) {
         gtk_dialog_set_response_sensitive(GTK_DIALOG(GWY_TOOL(tool)->dialog), GTK_RESPONSE_APPLY,
@@ -336,7 +323,7 @@ update_header(GwyToolLevel3 *tool,
               guint col,
               GString *str,
               const gchar *title,
-              GwySIValueFormat *vf)
+              GwyValueFormat *vf)
 {
     GtkTreeViewColumn *column = gtk_tree_view_get_column(tool->treeview, col);
     GtkLabel *label = GTK_LABEL(gtk_tree_view_column_get_widget(column));
@@ -373,7 +360,7 @@ render_cell(GtkCellLayout *layout,
     GwyToolLevel3 *tool = (GwyToolLevel3*)user_data;
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
     gint id = GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(layout), "id"));
-    const GwySIValueFormat *vf = NULL;
+    const GwyValueFormat *vf = NULL;
     gchar buf[32];
     gdouble point[2];
     gdouble val;
@@ -401,7 +388,7 @@ render_cell(GtkCellLayout *layout,
     }
     else if (id == COLUMN_Z) {
         vf = plain_tool->value_format;
-        val = gwy_plain_tool_get_z_average(plain_tool->data_field, point,
+        val = gwy_plain_tool_get_z_average(plain_tool->field, point,
                                            gwy_params_get_int(tool->params, PARAM_RADIUS));
     }
     else {
@@ -420,7 +407,7 @@ static void
 gwy_tool_level3_apply(GwyToolLevel3 *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    GwyDataField *field = plain_tool->data_field;
+    GwyField *field = plain_tool->field;
     gint radius = gwy_params_get_int(tool->params, PARAM_RADIUS);
     gboolean set_zero = gwy_params_get_boolean(tool->params, PARAM_SET_ZERO);
     gdouble points[9], z[3], coeffs[3];
@@ -452,20 +439,20 @@ gwy_tool_level3_apply(GwyToolLevel3 *tool)
     gwy_math_lin_solve_rewrite(3, points, z, coeffs);
     /* To keep mean value intact, the mean value of the plane we add to the data has to be zero, i.e., in the center
      * of the data the value must be zero. */
-    coeffs[0] = gwy_data_field_jtor(field, coeffs[0]);
-    coeffs[1] = gwy_data_field_itor(field, coeffs[1]);
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
+    coeffs[0] = gwy_field_jtor(field, coeffs[0]);
+    coeffs[1] = gwy_field_itor(field, coeffs[1]);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
     if (!set_zero)
         coeffs[2] = -0.5*(coeffs[0]*xres + coeffs[1]*yres);
     if (tool->allow_undo) {
-        gwy_app_undo_qcheckpoint(plain_tool->container, gwy_app_get_data_key_for_id(plain_tool->id), 0);
+        gwy_app_undo_qcheckpoint(plain_tool->container, gwy_file_key_image(plain_tool->id), 0);
         gwy_params_save_to_settings(tool->params);   /* Ensure correct parameters in the log. */
         gwy_plain_tool_log_add(plain_tool);
     }
-    gwy_data_field_plane_level(field, coeffs[2], coeffs[0], coeffs[1]);
+    gwy_field_plane_level(field, coeffs[2], coeffs[0], coeffs[1]);
 
-    gwy_data_field_data_changed(field);
+    gwy_field_data_changed(field);
 }
 
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
