@@ -1,5 +1,5 @@
 /*
- *  $Id: linestats.c 26745 2024-10-18 15:17:53Z yeti-dn $
+ *  $Id: linestats.c 29023 2025-12-19 13:06:16Z yeti-dn $
  *  Copyright (C) 2003-2022 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,17 +16,11 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwymodule/gwymodule-tool.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libprocess/stats.h>
-#include <libprocess/linestats.h>
-#include <libgwydgets/gwystock.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
+#include "libgwyapp/sanity.h"
 
 enum {
     PARAM_OUTPUT_TYPE,
@@ -55,8 +49,8 @@ struct _GwyToolLineStats {
 
     GwyRectSelectionLabels *rlabels;
 
-    GwyDataLine *line;
-    GwyDataLine *weights;
+    GwyLine *line;
+    GwyLine *weights;
     gint isel[4];
     gint isel_prev[4];
 
@@ -64,9 +58,6 @@ struct _GwyToolLineStats {
     GtkWidget *update;
     GwyParamTable *table_quantity;
     GwyParamTable *table_options;
-
-    /* potential class data */
-    GType layer_type_rect;
 };
 
 struct _GwyToolLineStatsClass {
@@ -92,12 +83,12 @@ static void         update_curve                         (GwyToolLineStats *tool
 static void         param_changed                        (GwyToolLineStats *tool,
                                                           gint id);
 static void         gwy_tool_line_stats_apply            (GwyToolLineStats *tool);
-static void         calculate_avg_rms_for_rms            (GwyDataLine *dline,
+static void         calculate_avg_rms_for_rms            (GwyLine *dline,
                                                           gdouble *avg,
                                                           gdouble *rms);
-static gint         set_data_from_dataline_filtered      (GwyGraphCurveModel *gcmodel,
-                                                          GwyDataLine *dline,
-                                                          GwyDataLine *weight,
+static gint         set_data_from_line_filtered          (GwyGraphCurveModel *gcmodel,
+                                                          GwyLine *dline,
+                                                          GwyLine *weight,
                                                           gdouble threshold);
 
 static GwyModuleInfo module_info = {
@@ -179,7 +170,7 @@ gwy_tool_line_stats_class_init(GwyToolLineStatsClass *klass)
 
     gobject_class->finalize = gwy_tool_line_stats_finalize;
 
-    tool_class->stock_id = GWY_STOCK_GRAPH_VERTICAL;
+    tool_class->icon_name = GWY_ICON_GRAPH_VERTICAL;
     tool_class->title = _("Row/Column Statistics");
     tool_class->tooltip = _("Calculate row/column statistical functions");
     tool_class->prefix = "/module/line_stats";
@@ -199,10 +190,10 @@ gwy_tool_line_stats_finalize(GObject *object)
     GwyToolLineStats *tool = GWY_TOOL_LINE_STATS(object);
 
     gwy_params_save_to_settings(tool->params);
-    GWY_OBJECT_UNREF(tool->params);
-    GWY_OBJECT_UNREF(tool->line);
-    GWY_OBJECT_UNREF(tool->weights);
-    GWY_OBJECT_UNREF(tool->gmodel);
+    g_clear_object(&tool->params);
+    g_clear_object(&tool->line);
+    g_clear_object(&tool->weights);
+    g_clear_object(&tool->gmodel);
 
     G_OBJECT_CLASS(gwy_tool_line_stats_parent_class)->finalize(object);
 }
@@ -212,19 +203,15 @@ gwy_tool_line_stats_init(GwyToolLineStats *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
 
-    tool->layer_type_rect = gwy_plain_tool_check_layer_type(plain_tool, "GwyLayerRectangle");
-    if (!tool->layer_type_rect)
-        return;
-
-    plain_tool->unit_style = GWY_SI_UNIT_FORMAT_MARKUP;
+    plain_tool->unit_style = GWY_UNIT_FORMAT_MARKUP;
     plain_tool->lazy_updates = TRUE;
 
     tool->params = gwy_params_new_from_settings(define_module_params());
-    tool->line = gwy_data_line_new(4, 1.0, FALSE);
-    tool->weights = gwy_data_line_new(4, 1.0, FALSE);
+    tool->line = gwy_line_new(4, 1.0, FALSE);
+    tool->weights = gwy_line_new(4, 1.0, FALSE);
     memset(tool->isel_prev, 0xff, 4*sizeof(gint));
 
-    gwy_plain_tool_connect_selection(plain_tool, tool->layer_type_rect, "rectangle");
+    gwy_plain_tool_connect_selection(plain_tool, GWY_TYPE_LAYER_RECTANGLE, "rectangle");
     gwy_plain_tool_enable_selection_holding(plain_tool);
 
     gwy_tool_line_stats_init_dialog(tool);
@@ -235,24 +222,25 @@ gwy_tool_line_stats_rect_updated(GwyToolLineStats *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
 
-    gwy_rect_selection_labels_select(tool->rlabels, plain_tool->selection, plain_tool->data_field);
+    gwy_rect_selection_labels_select(tool->rlabels, plain_tool->selection, plain_tool->field);
 }
 
 static void
 gwy_tool_line_stats_init_dialog(GwyToolLineStats *tool)
 {
     GtkDialog *dialog = GTK_DIALOG(GWY_TOOL(tool)->dialog);
-    GtkWidget *hbox, *vbox, *options, *image, *graph;
+    GtkWidget *hbox, *vbox, *options, *graph;
     GwyParamTable *table;
 
     tool->gmodel = gwy_graph_model_new();
 
-    hbox = gwy_hbox_new(4);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
     gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(dialog)), hbox, TRUE, TRUE, 0);
 
     /* Left pane */
-    vbox = gwy_vbox_new(6);
-    gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 6);
+    gtk_widget_set_hexpand(vbox, FALSE);
+    gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, TRUE, 0);
 
     /* Selection info */
     tool->rlabels = gwy_rect_selection_labels_new(TRUE, G_CALLBACK(gwy_tool_line_stats_rect_updated), tool);
@@ -283,12 +271,10 @@ gwy_tool_line_stats_init_dialog(GwyToolLineStats *tool)
     gwy_graph_enable_user_input(GWY_GRAPH(graph), FALSE);
     gtk_box_pack_start(GTK_BOX(hbox), graph, TRUE, TRUE, 2);
 
-    tool->update = gtk_dialog_add_button(dialog, _("_Update"), GWY_TOOL_RESPONSE_UPDATE);
-    image = gtk_image_new_from_stock(GTK_STOCK_EXECUTE, GTK_ICON_SIZE_BUTTON);
-    gtk_button_set_image(GTK_BUTTON(tool->update), image);
+    tool->update = gwy_add_update_button_to_dialog(GTK_DIALOG(dialog));
     gwy_plain_tool_add_clear_button(GWY_PLAIN_TOOL(tool));
     gwy_tool_add_hide_button(GWY_TOOL(tool), FALSE);
-    gtk_dialog_add_button(dialog, GTK_STOCK_APPLY, GTK_RESPONSE_APPLY);
+    gwy_plain_tool_add_apply_button(GWY_PLAIN_TOOL(tool));
     gtk_dialog_set_default_response(dialog, GTK_RESPONSE_APPLY);
     gtk_dialog_set_response_sensitive(dialog, GTK_RESPONSE_APPLY, FALSE);
     gwy_help_add_to_tool_dialog(dialog, GWY_TOOL(tool), GWY_HELP_DEFAULT);
@@ -314,7 +300,7 @@ gwy_tool_line_stats_data_switched(GwyTool *gwytool,
         return;
 
     if (data_view) {
-        gwy_object_set_or_reset(plain_tool->layer, tool->layer_type_rect,
+        gwy_object_set_or_reset(plain_tool->layer, GWY_TYPE_LAYER_RECTANGLE,
                                 "editable", TRUE,
                                 "focus", -1,
                                 NULL);
@@ -333,7 +319,7 @@ gwy_tool_line_stats_response(GwyTool *tool,
 
     if (response_id == GTK_RESPONSE_APPLY)
         gwy_tool_line_stats_apply(GWY_TOOL_LINE_STATS(tool));
-    else if (response_id == GWY_TOOL_RESPONSE_UPDATE)
+    else if (response_id == GWY_RESPONSE_UPDATE)
         update_curve(GWY_TOOL_LINE_STATS(tool));
 }
 
@@ -357,7 +343,7 @@ update_selected_rectangle(GwyToolLineStats *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
     GwySelection *selection = plain_tool->selection;
-    GwyDataField *field = plain_tool->data_field;
+    GwyField *field = plain_tool->field;
     gint n = selection ? gwy_selection_get_data(selection, NULL) : 0;
 
     gwy_rect_selection_labels_fill(tool->rlabels, n == 1 ? selection : NULL, field, NULL, tool->isel);
@@ -387,8 +373,8 @@ static void
 update_curve(GwyToolLineStats *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    GwyDataField *field = plain_tool->data_field;
-    GwyDataField *mask = plain_tool->mask_field;
+    GwyField *field = plain_tool->field;
+    GwyField *mask = plain_tool->mask_field;
     GwyMaskingType masking = gwy_params_get_masking(tool->params, PARAM_MASKING, &mask);
     GwyOrientation dir = gwy_params_get_enum(tool->params, PARAM_DIRECTION);
     GwyLineStatQuantity quantity = gwy_params_get_enum(tool->params, PARAM_OUTPUT_TYPE);
@@ -396,8 +382,8 @@ update_curve(GwyToolLineStats *tool)
     gchar *result;
     gint nsel, col, row, w, h;
     const gchar *title;
-    GwySIUnit *siunit;
-    GwySIValueFormat *format;
+    GwyUnit *siunit;
+    GwyValueFormat *format;
     gdouble avg, rms;
 
     gwy_graph_model_remove_all_curves(tool->gmodel);
@@ -407,7 +393,7 @@ update_curve(GwyToolLineStats *tool)
         return;
 
     if (quantity == GWY_LINE_STAT_LENGTH
-        && !gwy_si_unit_equal(gwy_data_field_get_si_unit_xy(field), gwy_data_field_get_si_unit_z(field)))
+        && !gwy_unit_equal(gwy_field_get_unit_xy(field), gwy_field_get_unit_z(field)))
         return;
 
     if (plain_tool->pending_updates & GWY_PLAIN_TOOL_CHANGED_SELECTION)
@@ -425,11 +411,11 @@ update_curve(GwyToolLineStats *tool)
     if (nsel == 0)
         return;
 
-    gwy_data_field_get_line_stats_mask(field, plain_tool->mask_field, masking, tool->line, tool->weights,
-                                       col, row, w, h, quantity, dir);
+    gwy_field_area_get_line_stats(field, plain_tool->mask_field, masking, tool->line, tool->weights,
+                                  col, row, w, h, quantity, dir);
 
     gcmodel = gwy_graph_curve_model_new();
-    if (!set_data_from_dataline_filtered(gcmodel, tool->line, tool->weights, 5.0)) {
+    if (!set_data_from_line_filtered(gcmodel, tool->line, tool->weights, 5.0)) {
         g_object_unref(gcmodel);
         return;
     }
@@ -437,19 +423,19 @@ update_curve(GwyToolLineStats *tool)
     g_object_set(gcmodel, "mode", GWY_GRAPH_CURVE_LINE, NULL);
     g_object_unref(gcmodel);
 
-    title = gettext(gwy_enum_to_string(quantity, quantities, G_N_ELEMENTS(quantities)));
+    title = _(gwy_enum_to_string(quantity, quantities, G_N_ELEMENTS(quantities)));
     g_object_set(gcmodel, "description", title, NULL);
     g_object_set(tool->gmodel, "title", title, NULL);
-    gwy_graph_model_set_units_from_data_line(tool->gmodel, tool->line);
+    gwy_graph_model_set_units_from_line(tool->gmodel, tool->line);
     gwy_param_table_data_id_refilter(tool->table_options, PARAM_TARGET_GRAPH);
 
-    siunit = gwy_data_line_get_si_unit_y(tool->line);
-    format = gwy_si_unit_get_format(siunit, GWY_SI_UNIT_FORMAT_MARKUP, gwy_data_line_get_avg(tool->line), NULL);
+    siunit = gwy_line_get_unit_y(tool->line);
+    format = gwy_unit_get_format(siunit, GWY_UNIT_FORMAT_MARKUP, gwy_line_avg(tool->line), NULL);
     if (quantity == GWY_LINE_STAT_RMS)
         calculate_avg_rms_for_rms(tool->line, &avg, &rms);
     else {
-        avg = gwy_data_line_get_avg(tool->line);
-        rms = gwy_data_line_get_rms(tool->line);
+        avg = gwy_line_avg(tool->line);
+        rms = gwy_line_rms(tool->line);
     }
 
     if (*format->units)
@@ -472,7 +458,7 @@ param_changed(GwyToolLineStats *tool, gint id)
     if (id == PARAM_INSTANT_UPDATE)
         do_update = do_update || gwy_params_get_boolean(params, PARAM_INSTANT_UPDATE);
     if (id == PARAM_MASKING)
-        do_update = do_update || (GWY_PLAIN_TOOL(tool)->data_field && GWY_PLAIN_TOOL(tool)->mask_field);
+        do_update = do_update || (GWY_PLAIN_TOOL(tool)->field && GWY_PLAIN_TOOL(tool)->mask_field);
     if (id < 0 || id == PARAM_INSTANT_UPDATE || id == PARAM_OUTPUT_TYPE)
         update_sensitivity(tool);
     if (do_update)
@@ -485,28 +471,29 @@ static void
 gwy_tool_line_stats_apply(GwyToolLineStats *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    GwyGraphModel *gmodel;
 
     g_return_if_fail(plain_tool->selection);
 
+    GwyGraphModel *gmodel;
     if ((gmodel = gwy_params_get_graph(tool->params, PARAM_TARGET_GRAPH))) {
         gwy_graph_model_append_curves(gmodel, tool->gmodel, 1);
         return;
     }
 
-    gmodel = gwy_graph_model_duplicate(tool->gmodel);
-    gwy_app_data_browser_add_graph_model(gmodel, plain_tool->container, TRUE);
+    gmodel = gwy_graph_model_copy(tool->gmodel);
+    gint newid = gwy_file_add_graph(GWY_FILE(plain_tool->container), gmodel);
+    gwy_file_set_visible(GWY_FILE(plain_tool->container), GWY_FILE_GRAPH, newid, TRUE);
     g_object_unref(gmodel);
 }
 
 static void
-calculate_avg_rms_for_rms(GwyDataLine *dline, gdouble *avg, gdouble *rms)
+calculate_avg_rms_for_rms(GwyLine *dline, gdouble *avg, gdouble *rms)
 {
-    const gdouble *d = gwy_data_line_get_data_const(dline);
+    const gdouble *d = gwy_line_get_data_const(dline);
     gdouble z, s2 = 0.0, s4 = 0.0;
     gint n, i;
 
-    n = gwy_data_line_get_res(dline);
+    n = gwy_line_get_res(dline);
     for (i = 0; i < n; i++) {
         z = d[i];
         s2 += z*z;
@@ -523,20 +510,20 @@ calculate_avg_rms_for_rms(GwyDataLine *dline, gdouble *avg, gdouble *rms)
 }
 
 static gint
-set_data_from_dataline_filtered(GwyGraphCurveModel *gcmodel,
-                                GwyDataLine *dline, GwyDataLine *weight,
-                                gdouble threshold)
+set_data_from_line_filtered(GwyGraphCurveModel *gcmodel,
+                            GwyLine *dline, GwyLine *weight,
+                            gdouble threshold)
 {
     gint res, i, n = 0;
     gdouble off, dx;
     gdouble *xdata, *ydata;
     const gdouble *d, *w;
 
-    res = gwy_data_line_get_res(dline);
-    dx = gwy_data_line_get_real(dline)/res;
-    off = gwy_data_line_get_offset(dline);
-    d = gwy_data_line_get_data(dline);
-    w = gwy_data_line_get_data(weight);
+    res = gwy_line_get_res(dline);
+    dx = gwy_line_get_real(dline)/res;
+    off = gwy_line_get_offset(dline);
+    d = gwy_line_get_data(dline);
+    w = gwy_line_get_data(weight);
 
     xdata = g_new(gdouble, res);
     ydata = g_new(gdouble, res);
