@@ -1,5 +1,5 @@
 /*
- *  $Id: maskedit.c 26745 2024-10-18 15:17:53Z yeti-dn $
+ *  $Id: maskedit.c 28906 2025-11-24 16:45:11Z yeti-dn $
  *  Copyright (C) 2003-2022 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,17 +16,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwymodule/gwymodule-tool.h>
-#include <libprocess/elliptic.h>
-#include <libprocess/stats.h>
-#include <libprocess/filters.h>
-#include <libprocess/grains.h>
-#include <libgwydgets/gwystock.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 
 enum {
     SENS_DATA = 1 << 0,
@@ -88,10 +81,10 @@ typedef enum {
     MASK_TOOL_FILL_ERASE  = 3,
 } MaskEditTool;
 
-typedef void (*FieldFillFunc)(GwyDataField*, gint, gint, gint, gint, gdouble);
-typedef void (*FieldExtractFunc)(GwyDataField*, gint, gint, gint, gint, gdouble*);
-typedef void (*FieldUnextractFunc)(GwyDataField*, gint, gint, gint, gint, const gdouble*);
-typedef gint (*FieldSizeFunc)(gint, gint);
+typedef void  (*FieldFillFunc)      (GwyField*, gint, gint, gint, gint, gdouble);
+typedef void  (*FieldExtractFunc)   (GwyField*, gint, gint, gint, gint, gdouble*);
+typedef void  (*FieldUnextractFunc) (GwyField*, gint, gint, gint, gint, const gdouble*);
+typedef gsize (*FieldSizeFunc)      (gint, gint);
 
 #define GWY_TYPE_TOOL_MASK_EDITOR            (gwy_tool_mask_editor_get_type())
 #define GWY_TOOL_MASK_EDITOR(obj)            (G_TYPE_CHECK_INSTANCE_CAST((obj), GWY_TYPE_TOOL_MASK_EDITOR, GwyToolMaskEditor))
@@ -113,9 +106,6 @@ struct _GwyToolMaskEditor {
     /* paintrbrush only */
     gboolean drawing_started;
     gint oldisel[2];
-
-    /* potential class data */
-    GType layer_types[MASK_NLAYER_TYPES];
 };
 
 struct _GwyToolMaskEditorClass {
@@ -149,6 +139,8 @@ static void         bucket_fill                            (GwyToolMaskEditor *t
                                                             gint j,
                                                             gint i);
 
+static GType layer_types[MASK_NLAYER_TYPES];
+
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
@@ -160,21 +152,21 @@ static GwyModuleInfo module_info = {
 };
 
 static const GwyEnum mode_icons[] = {
-    { GWY_STOCK_MASK_SET,       MASK_EDIT_OP_SET,       },
-    { GWY_STOCK_MASK_ADD,       MASK_EDIT_OP_ADD,       },
-    { GWY_STOCK_MASK_SUBTRACT,  MASK_EDIT_OP_REMOVE,    },
-    { GWY_STOCK_MASK_INTERSECT, MASK_EDIT_OP_INTERSECT, },
+    { GWY_ICON_MASK_SET,       MASK_EDIT_OP_SET,       },
+    { GWY_ICON_MASK_ADD,       MASK_EDIT_OP_ADD,       },
+    { GWY_ICON_MASK_SUBTRACT,  MASK_EDIT_OP_REMOVE,    },
+    { GWY_ICON_MASK_INTERSECT, MASK_EDIT_OP_INTERSECT, },
 };
 static const GwyEnum shape_icons[] = {
-    { GWY_STOCK_MASK,        MASK_SHAPE_RECTANGLE, },
-    { GWY_STOCK_MASK_CIRCLE, MASK_SHAPE_ELLIPSE,   },
-    { GWY_STOCK_MASK_LINE,   MASK_SHAPE_LINE,      },
+    { GWY_ICON_MASK,        MASK_SHAPE_RECTANGLE, },
+    { GWY_ICON_MASK_CIRCLE, MASK_SHAPE_ELLIPSE,   },
+    { GWY_ICON_MASK_LINE,   MASK_SHAPE_LINE,      },
 };
 static const GwyEnum tool_icons[] = {
-    { GWY_STOCK_MASK_PAINT_DRAW,  MASK_TOOL_PAINT_DRAW,  },
-    { GWY_STOCK_MASK_PAINT_ERASE, MASK_TOOL_PAINT_ERASE, },
-    { GWY_STOCK_MASK_FILL_DRAW,   MASK_TOOL_FILL_DRAW,   },
-    { GWY_STOCK_MASK_FILL_ERASE,  MASK_TOOL_FILL_ERASE,  },
+    { GWY_ICON_MASK_PAINT_DRAW,  MASK_TOOL_PAINT_DRAW,  },
+    { GWY_ICON_MASK_PAINT_ERASE, MASK_TOOL_PAINT_ERASE, },
+    { GWY_ICON_MASK_FILL_DRAW,   MASK_TOOL_FILL_DRAW,   },
+    { GWY_ICON_MASK_FILL_ERASE,  MASK_TOOL_FILL_ERASE,  },
 };
 
 GWY_MODULE_QUERY2(module_info, maskedit)
@@ -184,6 +176,12 @@ G_DEFINE_TYPE(GwyToolMaskEditor, gwy_tool_mask_editor, GWY_TYPE_PLAIN_TOOL)
 static gboolean
 module_register(void)
 {
+    layer_types[MASK_SHAPE_RECTANGLE] = GWY_TYPE_LAYER_RECTANGLE;
+    layer_types[MASK_SHAPE_ELLIPSE] = GWY_TYPE_LAYER_ELLIPSE;
+    layer_types[MASK_SHAPE_LINE] = GWY_TYPE_LAYER_LINE;
+    /* Not a real shape, but permits putting layer-related things into one array. */
+    layer_types[MASK_SHAPE_POINT] = GWY_TYPE_LAYER_POINT;
+
     gwy_tool_func_register(GWY_TYPE_TOOL_MASK_EDITOR);
 
     return TRUE;
@@ -250,7 +248,7 @@ gwy_tool_mask_editor_class_init(GwyToolMaskEditorClass *klass)
 
     gobject_class->finalize = gwy_tool_mask_editor_finalize;
 
-    tool_class->stock_id = GWY_STOCK_MASK_EDITOR;
+    tool_class->icon_name = GWY_ICON_MASK_EDITOR;
     tool_class->title = _("Mask Editor");
     tool_class->tooltip = _("Edit mask");
     tool_class->prefix = "/module/maskeditor";
@@ -267,9 +265,9 @@ gwy_tool_mask_editor_finalize(GObject *object)
     GwyToolMaskEditor *tool = GWY_TOOL_MASK_EDITOR(object);
 
     gwy_params_save_to_settings(tool->params);
-    GWY_OBJECT_UNREF(tool->params);
-    GWY_OBJECT_UNREF(tool->sizegroup);
-    GWY_OBJECT_UNREF(tool->sensgroup);
+    g_clear_object(&tool->params);
+    g_clear_object(&tool->sizegroup);
+    g_clear_object(&tool->sensgroup);
 
     G_OBJECT_CLASS(gwy_tool_mask_editor_parent_class)->finalize(object);
 }
@@ -277,19 +275,6 @@ gwy_tool_mask_editor_finalize(GObject *object)
 static void
 gwy_tool_mask_editor_init(GwyToolMaskEditor *tool)
 {
-    static const gchar *const shape_layer_types[MASK_NLAYER_TYPES] = {
-        "GwyLayerRectangle", "GwyLayerEllipse", "GwyLayerLine", "GwyLayerPoint",
-    };
-
-    GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    guint i;
-
-    for (i = 0; i < MASK_NLAYER_TYPES; i++) {
-        tool->layer_types[i] = gwy_plain_tool_check_layer_type(plain_tool, shape_layer_types[i]);
-        if (!tool->layer_types[i])
-            return;
-    }
-
     tool->params = gwy_params_new_from_settings(define_module_params());
     connect_selection(tool);
     gwy_tool_mask_editor_init_dialog(tool);
@@ -362,35 +347,35 @@ terrible_hack(GwyToolMaskEditor *tool)
     for (l = children; l; l = g_list_next(l)) {
         GtkWidget *widget = GTK_WIDGET(l->data);
 
-        if (GTK_IS_HBOX(widget)) {
+        if (gwy_widget_is_hbox(widget)) {
             GList *l2, *children2 = gtk_container_get_children(GTK_CONTAINER(widget));
 
             for (l2 = children2; l2; l2 = g_list_next(l2)) {
                 GtkWidget *img, *widget2 = GTK_WIDGET(l2->data);
 
                 if (GTK_IS_RADIO_BUTTON(widget2) && (img = gtk_bin_get_child(GTK_BIN(widget2))) && GTK_IS_IMAGE(img)) {
-                    gchar *stock_id = NULL;
+                    const gchar *icon_name = NULL;
 
-                    gtk_image_get_stock(GTK_IMAGE(img), &stock_id, NULL);
-                    if (stock_id) {
+                    gtk_image_get_icon_name(GTK_IMAGE(img), &icon_name, NULL);
+                    if (icon_name) {
                         MaskEditStyle style;
                         gboolean known_style = FALSE;
                         guint i;
 
                         for (i = 0; i < G_N_ELEMENTS(mode_icons) && !known_style; i++) {
-                            if (gwy_strequal(mode_icons[i].name, stock_id)) {
+                            if (gwy_strequal(mode_icons[i].name, icon_name)) {
                                 style = MASK_EDIT_STYLE_SHAPES;
                                 known_style = TRUE;
                             }
                         }
                         for (i = 0; i < G_N_ELEMENTS(shape_icons) && !known_style; i++) {
-                            if (gwy_strequal(shape_icons[i].name, stock_id)) {
+                            if (gwy_strequal(shape_icons[i].name, icon_name)) {
                                 style = MASK_EDIT_STYLE_SHAPES;
                                 known_style = TRUE;
                             }
                         }
                         for (i = 0; i < G_N_ELEMENTS(tool_icons) && !known_style; i++) {
-                            if (gwy_strequal(tool_icons[i].name, stock_id)) {
+                            if (gwy_strequal(tool_icons[i].name, icon_name)) {
                                 style = MASK_EDIT_STYLE_DRAWING;
                                 known_style = TRUE;
                             }
@@ -420,7 +405,7 @@ create_button(GwyToolMaskEditor *tool, GtkBox *hbox,
     GtkWidget *button;
 
     if (stock_id)
-        button = gwy_stock_like_button_new(label, stock_id);
+        button = gwy_create_stock_button(label, stock_id);
     else
         button = gtk_button_new_with_mnemonic(label);
 
@@ -435,11 +420,12 @@ static GtkWidget*
 create_actions(gpointer user_data)
 {
     GwyToolMaskEditor *tool = (GwyToolMaskEditor*)user_data;
-    GtkBox *hbox = GTK_BOX(gtk_hbox_new(TRUE, 0));
+    GtkBox *hbox = GTK_BOX(gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0));
 
-    create_button(tool, hbox, _("_Invert"), GWY_STOCK_MASK_INVERT, ACTION_INVERT, SENS_DATA | SENS_MASK);
-    create_button(tool, hbox, _("_Remove"), GWY_STOCK_MASK_REMOVE, ACTION_REMOVE, SENS_DATA | SENS_MASK);
-    create_button(tool, hbox, _("_Fill"), GWY_STOCK_MASK, ACTION_FILL, SENS_DATA);
+    gtk_box_set_homogeneous(hbox, TRUE);
+    create_button(tool, hbox, _("_Invert"), GWY_ICON_MASK_INVERT, ACTION_INVERT, SENS_DATA | SENS_MASK);
+    create_button(tool, hbox, _("_Remove"), GWY_ICON_MASK_REMOVE, ACTION_REMOVE, SENS_DATA | SENS_MASK);
+    create_button(tool, hbox, _("_Fill"), GWY_ICON_MASK, ACTION_FILL, SENS_DATA);
 
     return GTK_WIDGET(hbox);
 }
@@ -448,7 +434,7 @@ static GtkWidget*
 create_fill_voids(gpointer user_data)
 {
     GwyToolMaskEditor *tool = (GwyToolMaskEditor*)user_data;
-    GtkBox *hbox = GTK_BOX(gtk_hbox_new(FALSE, 4));
+    GtkBox *hbox = GTK_BOX(gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4));
     GtkWidget *checkbox;
 
     create_button(tool, hbox, _("Fill _Voids"), NULL, ACTION_FILL_VOIDS, SENS_DATA | SENS_MASK);
@@ -466,10 +452,11 @@ static GtkWidget*
 create_grow_shrink(gpointer user_data)
 {
     GwyToolMaskEditor *tool = (GwyToolMaskEditor*)user_data;
-    GtkBox *hbox = GTK_BOX(gtk_hbox_new(TRUE, 0));
+    GtkBox *hbox = GTK_BOX(gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0));
 
-    create_button(tool, hbox, _("_Grow"), GWY_STOCK_MASK_GROW, ACTION_GROW, SENS_DATA | SENS_MASK);
-    create_button(tool, hbox, _("Shrin_k"), GWY_STOCK_MASK_SHRINK, ACTION_SHRINK, SENS_DATA | SENS_MASK);
+    gtk_box_set_homogeneous(hbox, TRUE);
+    create_button(tool, hbox, _("_Grow"), GWY_ICON_MASK_GROW, ACTION_GROW, SENS_DATA | SENS_MASK);
+    create_button(tool, hbox, _("Shrin_k"), GWY_ICON_MASK_SHRINK, ACTION_SHRINK, SENS_DATA | SENS_MASK);
 
     return GTK_WIDGET(hbox);
 }
@@ -487,7 +474,7 @@ connect_selection(GwyToolMaskEditor *tool)
                            ? MASK_SHAPE_POINT
                            : gwy_params_get_enum(tool->params, PARAM_SHAPE));
 
-    gwy_plain_tool_connect_selection(plain_tool, tool->layer_types[shape], shape_selection_names[shape]);
+    gwy_plain_tool_connect_selection(plain_tool, layer_types[shape], shape_selection_names[shape]);
 }
 
 static void
@@ -515,10 +502,10 @@ static void
 gwy_tool_mask_editor_mask_changed(GwyPlainTool *plain_tool)
 {
     GwyToolMaskEditor *tool = GWY_TOOL_MASK_EDITOR(plain_tool);
-    GwyDataField *mask = plain_tool->mask_field;
+    GwyField *mask = plain_tool->mask_field;
 
     gwy_sensitivity_group_set_state(tool->sensgroup, SENS_MASK,
-                                    mask && gwy_data_field_get_max(mask) > 0.0 ? SENS_MASK : 0);
+                                    mask && gwy_field_get_max(mask) > 0.0 ? SENS_MASK : 0);
 }
 
 static void
@@ -568,15 +555,21 @@ setup_layer(GwyToolMaskEditor *tool)
     if (!plain_tool->data_view)
         return;
 
-    gwy_debug("setting up layer %s", g_type_name(tool->layer_types[shape]));
-    gwy_object_set_or_reset(plain_tool->layer, tool->layer_types[shape], "editable", TRUE, "focus", -1, NULL);
+    gwy_debug("setting up layer %s", g_type_name(layer_types[shape]));
+    gwy_object_set_or_reset(plain_tool->layer, layer_types[shape],
+                            "editable", TRUE,
+                            "focus", -1,
+                            NULL);
     if (style == MASK_EDIT_STYLE_SHAPES) {
         if (shape == MASK_SHAPE_LINE)
-            g_object_set(plain_tool->layer, "line-numbers", FALSE, "thickness", 1, NULL);
+            g_object_set(plain_tool->layer, "numbered", FALSE, "thickness", 1.0, NULL);
     }
     else {
-        if (drawtool == MASK_TOOL_PAINT_DRAW || drawtool == MASK_TOOL_PAINT_ERASE)
-            g_object_set(plain_tool->layer, "marker-radius", gwy_params_get_int(tool->params, PARAM_RADIUS), NULL);
+        if (drawtool == MASK_TOOL_PAINT_DRAW || drawtool == MASK_TOOL_PAINT_ERASE) {
+            g_object_set(plain_tool->layer,
+                         "marker-radius", (gdouble)gwy_params_get_int(tool->params, PARAM_RADIUS),
+                         NULL);
+        }
         else
             g_object_set(plain_tool->layer, "draw-marker", FALSE, NULL);
     }
@@ -584,12 +577,12 @@ setup_layer(GwyToolMaskEditor *tool)
     gwy_selection_set_max_objects(plain_tool->selection, 1);
 }
 
-static GwyDataField*
+static GwyField*
 ensure_mask(GwyPlainTool *plain_tool)
 {
     if (!plain_tool->mask_field) {
-        GwyDataField *mask = gwy_data_field_new_alike(plain_tool->data_field, TRUE);
-        gwy_container_pass_object(plain_tool->container, gwy_app_get_mask_key_for_id(plain_tool->id), mask);
+        GwyField *mask = gwy_field_new_alike(plain_tool->field, TRUE);
+        gwy_container_pass_object(plain_tool->container, gwy_file_key_image_mask(plain_tool->id), mask);
     }
     g_assert(plain_tool->mask_field);
     return plain_tool->mask_field;
@@ -600,8 +593,8 @@ run_action(GwyToolMaskEditor *tool, GtkButton *button)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
     MaskEditAction action = GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(button), "action"));
-    GwyDataField *field = plain_tool->data_field;
-    GwyDataField *mask = plain_tool->mask_field;
+    GwyField *field = plain_tool->field;
+    GwyField *mask = plain_tool->mask_field;
     gint gsamount = gwy_params_get_int(tool->params, PARAM_GSAMOUNT);
     GwyDistanceTransformType dist_type = gwy_params_get_enum(tool->params, PARAM_DIST_TYPE);
     gboolean prevent_merge = gwy_params_get_boolean(tool->params, PARAM_PREVENT_MERGE);
@@ -611,29 +604,29 @@ run_action(GwyToolMaskEditor *tool, GtkButton *button)
 
     g_return_if_fail(action == ACTION_FILL ? field : mask);
 
-    quark = gwy_app_get_mask_key_for_id(plain_tool->id);
+    quark = gwy_file_key_image_mask(plain_tool->id);
     gwy_app_undo_qcheckpointv(plain_tool->container, 1, &quark);
 
     if (action == ACTION_FILL) {
         mask = ensure_mask(plain_tool);
-        gwy_data_field_fill(mask, 1.0);
+        gwy_field_fill(mask, 1.0);
     }
     else if (action == ACTION_REMOVE)
         gwy_container_remove(plain_tool->container, quark);
     else if (action == ACTION_INVERT)
-        gwy_data_field_grains_invert(mask);
+        gwy_field_grains_invert(mask);
     else if (action == ACTION_GROW)
-        gwy_data_field_grains_grow(mask, gsamount, dist_type, prevent_merge);
+        gwy_field_grains_grow(mask, gsamount, dist_type, prevent_merge);
     else if (action == ACTION_SHRINK)
-        gwy_data_field_grains_shrink(mask, gsamount, dist_type, from_border);
+        gwy_field_grains_shrink(mask, gsamount, dist_type, from_border);
     else if (action == ACTION_FILL_VOIDS)
-        gwy_data_field_fill_voids(mask, fill_nonsimple);
+        gwy_field_fill_voids(mask, fill_nonsimple);
     else {
         g_assert_not_reached();
     }
 
     if (action != ACTION_REMOVE)
-        gwy_data_field_data_changed(plain_tool->mask_field);
+        gwy_field_data_changed(plain_tool->mask_field);
 
     gwy_params_save_to_settings(tool->params);
     gwy_plain_tool_log_add(plain_tool);
@@ -663,24 +656,24 @@ mask_pixel_op(gdouble *field, gint i,
 
 /* Width is always positive; height can be negative to indicate line direction. */
 static void
-gwy_data_field_general_linear_area_operation(GwyDataField *dfield,
-                                             gint col, gint row,
-                                             gint width, gint height,
-                                             gdouble *data,
-                                             const gboolean is_4_connected,
-                                             const MaskPixelOp op)
+gwy_field_general_linear_area_operation(GwyField *dfield,
+                                        gint col, gint row,
+                                        gint width, gint height,
+                                        gdouble *data,
+                                        const gboolean is_4_connected,
+                                        const MaskPixelOp op)
 {
     gboolean is_const = (op == MASK_COUNT || op == MASK_EXTRACT);
     gint i, j, q, xres, yres;
     gdouble *d;
 
-    xres = gwy_data_field_get_xres(dfield);
-    yres = gwy_data_field_get_yres(dfield);
+    xres = gwy_field_get_xres(dfield);
+    yres = gwy_field_get_yres(dfield);
     col = CLAMP(col, 0, xres-1);
     row = CLAMP(row, 0, yres-1);
     width = CLAMP(width, 0, xres-col);
     height = (height >= 0 ? CLAMP(height, 0, yres-row) : CLAMP(height, -row, 0));
-    d = (is_const ? (gdouble*)gwy_data_field_get_data_const(dfield) : gwy_data_field_get_data(dfield));
+    d = (is_const ? (gdouble*)gwy_field_get_data_const(dfield) : gwy_field_get_data(dfield));
 
     j = 0;
     if (ABS(height) >= width) {
@@ -729,77 +722,105 @@ gwy_data_field_general_linear_area_operation(GwyDataField *dfield,
     }
 
     if (!is_const)
-        gwy_data_field_invalidate(dfield);
+        gwy_field_invalidate(dfield);
+}
+
+/* Adaptors. */
+static void
+rectangular_area_fill(GwyField *dfield,
+                      gint col, gint row,
+                      gint width, gint height,
+                      gdouble value)
+{
+    gwy_field_area_fill(dfield, NULL, GWY_MASK_IGNORE, col, row, width, height, value);
+}
+
+static void
+elliptic_area_fill(GwyField *dfield,
+                   gint col, gint row,
+                   gint width, gint height,
+                   gdouble value)
+{
+    gwy_field_elliptic_area_fill(dfield, col, row, width, height, value);
+}
+
+static void
+elliptic_area_extract(GwyField *dfield,
+                      gint col, gint row,
+                      gint width, gint height,
+                      gdouble *data)
+{
+    gwy_field_elliptic_area_extract(dfield, col, row, width, height, data);
 }
 
 /* 8-continuous lines. */
-static gint
-gwy_data_field_get_linear_area_size(gint width, gint height)
+static gsize
+gwy_field_get_linear_area_size(gint width, gint height)
 {
     return MAX(ABS(width), ABS(height));
 }
 
 static void
-gwy_data_field_linear_area_fill(GwyDataField *dfield,
+gwy_field_linear_area_fill(GwyField *dfield,
+                           gint col, gint row,
+                           gint width, gint height,
+                           gdouble value)
+{
+    gwy_field_general_linear_area_operation(dfield, col, row, width, height, &value, FALSE, MASK_FILL);
+}
+
+static void
+gwy_field_linear_area_extract(GwyField *dfield,
+                              gint col, gint row,
+                              gint width, gint height,
+                              gdouble *values)
+{
+    gwy_field_general_linear_area_operation(dfield, col, row, width, height, values, FALSE, MASK_EXTRACT);
+}
+
+static void
+gwy_field_linear_area_unextract(GwyField *dfield,
                                 gint col, gint row,
                                 gint width, gint height,
-                                gdouble value)
+                                const gdouble *values)
 {
-    gwy_data_field_general_linear_area_operation(dfield, col, row, width, height, &value, FALSE, MASK_FILL);
-}
-
-static void
-gwy_data_field_linear_area_extract(GwyDataField *dfield,
-                                   gint col, gint row,
-                                   gint width, gint height,
-                                   gdouble *values)
-{
-    gwy_data_field_general_linear_area_operation(dfield, col, row, width, height, values, FALSE, MASK_EXTRACT);
-}
-
-static void
-gwy_data_field_linear_area_unextract(GwyDataField *dfield,
-                                     gint col, gint row,
-                                     gint width, gint height,
-                                     const gdouble *values)
-{
-    gwy_data_field_general_linear_area_operation(dfield, col, row, width, height, (gdouble*)values,
-                                                 FALSE, MASK_UNEXTRACT);
+    gwy_field_general_linear_area_operation(dfield, col, row, width, height, (gdouble*)values,
+                                            FALSE, MASK_UNEXTRACT);
 }
 
 /* 4-continuous lines. */
-static gint
-gwy_data_field_get_clinear_area_size(gint width, gint height)
+static gsize
+gwy_field_get_clinear_area_size(gint width, gint height)
 {
     return ABS(width) + ABS(height) - 1;
 }
 
 static void
-gwy_data_field_clinear_area_fill(GwyDataField *dfield,
-                                gint col, gint row,
-                                gint width, gint height,
-                                gdouble value)
+gwy_field_clinear_area_fill(GwyField *dfield,
+                            gint col, gint row,
+                            gint width, gint height,
+                            gdouble value)
 {
-    gwy_data_field_general_linear_area_operation(dfield, col, row, width, height, &value, TRUE, MASK_FILL);
+    gwy_field_general_linear_area_operation(dfield, col, row, width, height, &value, TRUE, MASK_FILL);
 }
 
 static void
-gwy_data_field_clinear_area_extract(GwyDataField *dfield,
-                                   gint col, gint row,
-                                   gint width, gint height,
-                                   gdouble *values)
+gwy_field_clinear_area_extract(GwyField *dfield,
+                               gint col, gint row,
+                               gint width, gint height,
+                               gdouble *values)
 {
-    gwy_data_field_general_linear_area_operation(dfield, col, row, width, height, values, TRUE, MASK_EXTRACT);
+    gwy_field_general_linear_area_operation(dfield, col, row, width, height, values, TRUE, MASK_EXTRACT);
 }
 
 static void
-gwy_data_field_clinear_area_unextract(GwyDataField *dfield,
-                                      gint col, gint row,
-                                      gint width, gint height,
-                                      const gdouble *values)
+gwy_field_clinear_area_unextract(GwyField *dfield,
+                                 gint col, gint row,
+                                 gint width, gint height,
+                                 const gdouble *values)
 {
-    gwy_data_field_general_linear_area_operation(dfield, col, row, width, height, (gdouble*)values,
-                                                 TRUE, MASK_UNEXTRACT);
+    gwy_field_general_linear_area_operation(dfield, col, row, width, height, (gdouble*)values,
+                                            TRUE, MASK_UNEXTRACT);
 }
 
 static void
@@ -812,7 +833,7 @@ gwy_tool_mask_editor_selection_finished(GwyPlainTool *plain_tool)
     MaskEditLogicalOp mode = gwy_params_get_enum(tool->params, PARAM_MODE);
     gboolean lines_4_connected = gwy_params_get_boolean(tool->params, PARAM_LINES_4_CONNECTED);
     GwySelection *selection = plain_tool->selection;
-    GwyDataField *mask = plain_tool->mask_field;
+    GwyField *mask = plain_tool->mask_field;
     FieldFillFunc fill_func = NULL;
     FieldExtractFunc extract_func = NULL;
     FieldUnextractFunc unextract_func = NULL;
@@ -823,14 +844,14 @@ gwy_tool_mask_editor_selection_finished(GwyPlainTool *plain_tool)
     gdouble sel[4];
     gint isel[4];
 
-    g_return_if_fail(plain_tool->data_field);
+    g_return_if_fail(plain_tool->field);
 
     tool->drawing_started = FALSE;
     if (!gwy_selection_get_object(selection, 0, sel))
         return;
 
-    isel[0] = floor(gwy_data_field_rtoj(plain_tool->data_field, sel[0]));
-    isel[1] = floor(gwy_data_field_rtoi(plain_tool->data_field, sel[1]));
+    isel[0] = floor(gwy_field_rtoj(plain_tool->field, sel[0]));
+    isel[1] = floor(gwy_field_rtoi(plain_tool->field, sel[1]));
 
     if (style == MASK_EDIT_STYLE_DRAWING) {
         if (drawtool == MASK_TOOL_PAINT_DRAW || drawtool == MASK_TOOL_PAINT_ERASE) {
@@ -841,38 +862,38 @@ gwy_tool_mask_editor_selection_finished(GwyPlainTool *plain_tool)
         }
         bucket_fill(tool, isel[0], isel[1]);
         if (plain_tool->mask_field)
-            gwy_data_field_data_changed(plain_tool->mask_field);
+            gwy_field_data_changed(plain_tool->mask_field);
         return;
     }
 
     if (shape == MASK_SHAPE_RECTANGLE)
-        fill_func = gwy_data_field_area_fill;
+        fill_func = rectangular_area_fill;
     else if (shape == MASK_SHAPE_ELLIPSE) {
-        fill_func = (FieldFillFunc)gwy_data_field_elliptic_area_fill;
-        extract_func = (FieldExtractFunc)gwy_data_field_elliptic_area_extract;
-        unextract_func = gwy_data_field_elliptic_area_unextract;
-        size_func = gwy_data_field_get_elliptic_area_size;
+        fill_func = elliptic_area_fill;
+        extract_func = elliptic_area_extract;
+        unextract_func = gwy_field_elliptic_area_unextract;
+        size_func = gwy_field_get_elliptic_area_size;
     }
     else if (shape == MASK_SHAPE_LINE) {
         if (lines_4_connected) {
-            fill_func = gwy_data_field_clinear_area_fill;
-            extract_func = gwy_data_field_clinear_area_extract;
-            unextract_func = gwy_data_field_clinear_area_unextract;
-            size_func = gwy_data_field_get_clinear_area_size;
+            fill_func = gwy_field_clinear_area_fill;
+            extract_func = gwy_field_clinear_area_extract;
+            unextract_func = gwy_field_clinear_area_unextract;
+            size_func = gwy_field_get_clinear_area_size;
         }
         else {
-            fill_func = gwy_data_field_linear_area_fill;
-            extract_func = gwy_data_field_linear_area_extract;
-            unextract_func = gwy_data_field_linear_area_unextract;
-            size_func = gwy_data_field_get_linear_area_size;
+            fill_func = gwy_field_linear_area_fill;
+            extract_func = gwy_field_linear_area_extract;
+            unextract_func = gwy_field_linear_area_unextract;
+            size_func = gwy_field_get_linear_area_size;
         }
     }
     else {
         g_return_if_reached();
     }
 
-    isel[2] = floor(gwy_data_field_rtoj(plain_tool->data_field, sel[2]));
-    isel[3] = floor(gwy_data_field_rtoi(plain_tool->data_field, sel[3]));
+    isel[2] = floor(gwy_field_rtoj(plain_tool->field, sel[2]));
+    isel[3] = floor(gwy_field_rtoi(plain_tool->field, sel[3]));
     if (shape == MASK_SHAPE_LINE) {
         if (isel[2] < isel[0]) {
             GWY_SWAP(gdouble, isel[0], isel[2]);
@@ -889,34 +910,34 @@ gwy_tool_mask_editor_selection_finished(GwyPlainTool *plain_tool)
     height = isel[3] - isel[1];  /* This one can be negative for lines. */
     gwy_debug("(%d,%d) (%d,%d)", col, row, width, height);
 
-    quark = gwy_app_get_mask_key_for_id(plain_tool->id);
+    quark = gwy_file_key_image_mask(plain_tool->id);
     if (mode == MASK_EDIT_OP_SET) {
         gwy_app_undo_qcheckpointv(plain_tool->container, 1, &quark);
         mask = ensure_mask(plain_tool);
-        gwy_data_field_clear(mask);
+        gwy_field_clear(mask);
         fill_func(mask, col, row, width, height, 1.0);
     }
-    else if (mode == MASK_EDIT_OP_ADD && (!mask || gwy_data_field_get_min(mask) < 1.0)) {
+    else if (mode == MASK_EDIT_OP_ADD && (!mask || gwy_field_get_min(mask) < 1.0)) {
         gwy_app_undo_qcheckpointv(plain_tool->container, 1, &quark);
         mask = ensure_mask(plain_tool);
         fill_func(mask, col, row, width, height, 1.0);
     }
-    else if (mode == MASK_EDIT_OP_REMOVE && mask && gwy_data_field_get_max(mask) > 0.0) {
+    else if (mode == MASK_EDIT_OP_REMOVE && mask && gwy_field_get_max(mask) > 0.0) {
         gwy_app_undo_qcheckpointv(plain_tool->container, 1, &quark);
         fill_func(mask, col, row, width, height, 0.0);
-        if (!(gwy_data_field_get_max(mask) > 0.0)) {
+        if (!(gwy_field_get_max(mask) > 0.0)) {
             gwy_container_remove(plain_tool->container, quark);
             mask = NULL;
         }
     }
-    else if (mode == MASK_EDIT_OP_INTERSECT && mask && gwy_data_field_get_max(mask) > 0.0) {
+    else if (mode == MASK_EDIT_OP_INTERSECT && mask && gwy_field_get_max(mask) > 0.0) {
         gdouble *data;
         gint n;
 
         gwy_app_undo_qcheckpointv(plain_tool->container, 1, &quark);
-        gwy_data_field_clamp(mask, 0.0, 1.0);
+        gwy_field_clamp(mask, 0.0, 1.0);
         if (shape == MASK_SHAPE_RECTANGLE)
-            gwy_data_field_area_add(mask, col, row, width, height, 1.0);
+            gwy_field_area_add(mask, col, row, width, height, 1.0);
         else {
             n = size_func(width, height);
             data = g_new(gdouble, n);
@@ -926,9 +947,9 @@ gwy_tool_mask_editor_selection_finished(GwyPlainTool *plain_tool)
             unextract_func(mask, col, row, width, height, data);
             g_free(data);
         }
-        gwy_data_field_add(mask, -1.0);
-        gwy_data_field_clamp(mask, 0.0, 1.0);
-        if (!(gwy_data_field_get_max(mask) > 0.0)) {
+        gwy_field_add(mask, -1.0);
+        gwy_field_clamp(mask, 0.0, 1.0);
+        if (!(gwy_field_get_max(mask) > 0.0)) {
             gwy_container_remove(plain_tool->container, quark);
             log_it_anyway = TRUE;
             mask = NULL;
@@ -937,7 +958,7 @@ gwy_tool_mask_editor_selection_finished(GwyPlainTool *plain_tool)
 
     gwy_selection_clear(selection);
     if (mask) {
-        gwy_data_field_data_changed(mask);
+        gwy_field_data_changed(mask);
     }
     if (mask || log_it_anyway) {
         gwy_params_save_to_settings(tool->params);
@@ -950,7 +971,7 @@ bucket_fill(GwyToolMaskEditor *tool, gint j, gint i)
 {
     MaskEditTool drawtool = gwy_params_get_enum(tool->params, PARAM_TOOL);
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    GwyDataField *mask;
+    GwyField *mask;
     gint xres, yres, k, gno;
     gint *g, *grains = NULL;
     gboolean draw;
@@ -962,15 +983,15 @@ bucket_fill(GwyToolMaskEditor *tool, gint j, gint i)
     if (!mask) {
         if (drawtool != MASK_TOOL_FILL_DRAW)
             return;
-        quark = gwy_app_get_mask_key_for_id(plain_tool->id);
+        quark = gwy_file_key_image_mask(plain_tool->id);
         gwy_app_undo_qcheckpointv(plain_tool->container, 1, &quark);
         mask = ensure_mask(plain_tool);
-        gwy_data_field_fill(mask, 1.0);
+        gwy_field_fill(mask, 1.0);
         goto end;
     }
 
-    xres = gwy_data_field_get_xres(mask);
-    yres = gwy_data_field_get_yres(mask);
+    xres = gwy_field_get_xres(mask);
+    yres = gwy_field_get_yres(mask);
     if (i < 0 || i >= yres || j < 0 || j >= xres)
         return;
 
@@ -982,17 +1003,17 @@ bucket_fill(GwyToolMaskEditor *tool, gint j, gint i)
         g_return_if_reached();
     }
 
-    data = gwy_data_field_get_data(mask);
+    data = gwy_field_get_data(mask);
     if ((data[i*xres + j] && draw) || (!data[i*xres + j] && !draw))
         return;
 
-    quark = gwy_app_get_mask_key_for_id(plain_tool->id);
+    quark = gwy_file_key_image_mask(plain_tool->id);
     gwy_app_undo_qcheckpointv(plain_tool->container, 1, &quark);
 
     g = grains = g_new0(gint, xres*yres);
     if (draw)
-        gwy_data_field_grains_invert(mask);
-    gwy_data_field_number_grains(mask, grains);
+        gwy_field_grains_invert(mask);
+    gwy_field_number_grains(mask, grains);
     gno = grains[i*xres + j];
 
     for (k = xres*yres; k; k--, data++, g++) {
@@ -1000,7 +1021,7 @@ bucket_fill(GwyToolMaskEditor *tool, gint j, gint i)
             *data = 0.0;
     }
     if (draw)
-        gwy_data_field_grains_invert(mask);
+        gwy_field_grains_invert(mask);
 
     g_free(grains);
 end:
@@ -1010,15 +1031,15 @@ end:
 
 /* FIXME: This is woefully inefficient. */
 static void
-gwy_data_field_paint_wide_line(GwyDataField *dfield,
-                               gint col, gint row,
-                               gint width, gint height,
-                               gdouble radius, gdouble value)
+gwy_field_paint_wide_line(GwyField *dfield,
+                          gint col, gint row,
+                          gint width, gint height,
+                          gdouble radius, gdouble value)
 {
     gint i, q;
 
     if (!width && !height) {
-        gwy_data_field_circular_area_fill(dfield, col, row, radius, value);
+        gwy_field_circular_area_fill(dfield, col, row, radius, value);
         return;
     }
 
@@ -1026,14 +1047,14 @@ gwy_data_field_paint_wide_line(GwyDataField *dfield,
         q = width/2;
         if (height > 0) {
             for (i = 0; i <= height; i++) {
-                gwy_data_field_circular_area_fill(dfield, col + q/height, row + i, radius, value);
+                gwy_field_circular_area_fill(dfield, col + q/height, row + i, radius, value);
                 q += width;
             }
         }
         else {
             height = ABS(height);
             for (i = 0; i <= height; i++) {
-                gwy_data_field_circular_area_fill(dfield, col + q/height, row - i, radius, value);
+                gwy_field_circular_area_fill(dfield, col + q/height, row - i, radius, value);
                 q += width;
             }
         }
@@ -1041,7 +1062,7 @@ gwy_data_field_paint_wide_line(GwyDataField *dfield,
     else {
         q = height/2;
         for (i = 0; i <= width; i++) {
-            gwy_data_field_circular_area_fill(dfield, col+i, row + q/width, radius, value);
+            gwy_field_circular_area_fill(dfield, col+i, row + q/width, radius, value);
             q += height;
         }
     }
@@ -1052,8 +1073,8 @@ gwy_tool_mask_editor_selection_changed(GwyPlainTool *plain_tool,
                                        G_GNUC_UNUSED gint hint)
 {
     GwyToolMaskEditor *tool = GWY_TOOL_MASK_EDITOR(plain_tool);
-    GwyDataField *mask = plain_tool->mask_field;
-    GwyDataField *field = plain_tool->data_field;
+    GwyField *mask = plain_tool->mask_field;
+    GwyField *field = plain_tool->field;
     MaskEditStyle style = gwy_params_get_enum(tool->params, PARAM_STYLE);
     MaskEditTool drawtool = gwy_params_get_enum(tool->params, PARAM_TOOL);
     gint radius = gwy_params_get_int(tool->params, PARAM_RADIUS);
@@ -1078,19 +1099,19 @@ gwy_tool_mask_editor_selection_changed(GwyPlainTool *plain_tool,
     else
         return;
 
-    isel[0] = floor(gwy_data_field_rtoj(plain_tool->data_field, sel[0]));
-    isel[1] = floor(gwy_data_field_rtoi(plain_tool->data_field, sel[1]));
+    isel[0] = floor(gwy_field_rtoj(plain_tool->field, sel[0]));
+    isel[1] = floor(gwy_field_rtoi(plain_tool->field, sel[1]));
 
-    quark = gwy_app_get_mask_key_for_id(plain_tool->id);
-    xres = gwy_data_field_get_xres(plain_tool->data_field);
-    yres = gwy_data_field_get_yres(plain_tool->data_field);
+    quark = gwy_file_key_image_mask(plain_tool->id);
+    xres = gwy_field_get_xres(plain_tool->field);
+    yres = gwy_field_get_yres(plain_tool->field);
 
     r = radius - 0.5;
     if (isel[0] >= 0 && isel[0] < xres && isel[1] >= 0 && isel[1] < yres) {
         if (!tool->drawing_started) {
             gwy_app_undo_qcheckpointv(plain_tool->container, 1, &quark);
             mask = ensure_mask(plain_tool);
-            gwy_data_field_circular_area_fill(mask, isel[0], isel[1], r, fillvalue);
+            gwy_field_circular_area_fill(mask, isel[0], isel[1], r, fillvalue);
         }
         else {
             gint xy[4];
@@ -1105,9 +1126,9 @@ gwy_tool_mask_editor_selection_changed(GwyPlainTool *plain_tool,
             }
             xy[2] -= xy[0];
             xy[3] -= xy[1];
-            gwy_data_field_paint_wide_line(mask, xy[0], xy[1], xy[2], xy[3], r, fillvalue);
+            gwy_field_paint_wide_line(mask, xy[0], xy[1], xy[2], xy[3], r, fillvalue);
         }
-        gwy_data_field_data_changed(mask);
+        gwy_field_data_changed(mask);
         tool->oldisel[0] = isel[0];
         tool->oldisel[1] = isel[1];
         tool->drawing_started = TRUE;
