@@ -1,5 +1,5 @@
 /*
- *  $Id: pathlevel.c 26745 2024-10-18 15:17:53Z yeti-dn $
+ *  $Id: pathlevel.c 28945 2025-12-03 10:10:57Z yeti-dn $
  *  Copyright (C) 2007-2022 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,16 +16,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwymodule/gwymodule-tool.h>
-#include <libprocess/datafield.h>
-#include <libprocess/linestats.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwydgets/gwynullstore.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 
 enum {
     COLUMN_I, COLUMN_X0, COLUMN_Y0, COLUMN_X1, COLUMN_Y1, NCOLUMNS
@@ -59,9 +53,6 @@ struct _GwyToolPathLevel {
     GtkTreeModel *model;
 
     GwyParamTable *table;
-
-    /* potential class data */
-    GType layer_type_line;
 };
 
 struct _GwyToolPathLevelClass {
@@ -137,7 +128,7 @@ gwy_tool_path_level_class_init(GwyToolPathLevelClass *klass)
 
     gobject_class->finalize = gwy_tool_path_level_finalize;
 
-    tool_class->stock_id = GWY_STOCK_PATH_LEVEL;
+    tool_class->icon_name = GWY_ICON_PATH_LEVEL;
     tool_class->title = _("Path Level");
     tool_class->tooltip = _("Level rows using intersections with given lines");
     tool_class->prefix = "/module/pathlevel";
@@ -154,10 +145,10 @@ gwy_tool_path_level_finalize(GObject *object)
     GwyToolPathLevel *tool = GWY_TOOL_PATH_LEVEL(object);
 
     gwy_params_save_to_settings(tool->params);
-    GWY_OBJECT_UNREF(tool->params);
+    g_clear_object(&tool->params);
     if (tool->model) {
         gtk_tree_view_set_model(tool->treeview, NULL);
-        GWY_OBJECT_UNREF(tool->model);
+        g_clear_object(&tool->model);
     }
 
     G_OBJECT_CLASS(gwy_tool_path_level_parent_class)->finalize(object);
@@ -168,16 +159,12 @@ gwy_tool_path_level_init(GwyToolPathLevel *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
 
-    tool->layer_type_line = gwy_plain_tool_check_layer_type(plain_tool, "GwyLayerLine");
-    if (!tool->layer_type_line)
-        return;
-
-    plain_tool->unit_style = GWY_SI_UNIT_FORMAT_MARKUP;
+    plain_tool->unit_style = GWY_UNIT_FORMAT_MARKUP;
     plain_tool->lazy_updates = TRUE;
 
     tool->params = gwy_params_new_from_settings(define_module_params());
 
-    gwy_plain_tool_connect_selection(plain_tool, tool->layer_type_line, "line");
+    gwy_plain_tool_connect_selection(plain_tool, GWY_TYPE_LAYER_LINE, "line");
 
     gwy_tool_path_level_init_dialog(tool);
 }
@@ -230,7 +217,7 @@ gwy_tool_path_level_init_dialog(GwyToolPathLevel *tool)
 
     gwy_plain_tool_add_clear_button(GWY_PLAIN_TOOL(tool));
     gwy_tool_add_hide_button(GWY_TOOL(tool), TRUE);
-    gtk_dialog_add_button(dialog, GTK_STOCK_APPLY, GTK_RESPONSE_APPLY);
+    gwy_plain_tool_add_apply_button(GWY_PLAIN_TOOL(tool));
     gtk_dialog_set_response_sensitive(dialog, GTK_RESPONSE_APPLY, FALSE);
     gwy_help_add_to_tool_dialog(dialog, GWY_TOOL(tool), GWY_HELP_NO_BUTTON);
 
@@ -253,8 +240,8 @@ gwy_tool_path_level_data_switched(GwyTool *gwytool,
         return;
 
     if (data_view) {
-        gwy_object_set_or_reset(plain_tool->layer, tool->layer_type_line,
-                                "thickness", gwy_params_get_int(tool->params, PARAM_THICKNESS),
+        gwy_object_set_or_reset(plain_tool->layer, GWY_TYPE_LAYER_LINE,
+                                "thickness", (gdouble)gwy_params_get_int(tool->params, PARAM_THICKNESS),
                                 "editable", TRUE,
                                 "focus", -1,
                                 NULL);
@@ -317,8 +304,11 @@ param_changed(GwyToolPathLevel *tool, gint id)
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
 
     if (id < 0 || id == PARAM_THICKNESS) {
-        if (plain_tool->layer)
-            g_object_set(plain_tool->layer, "thickness", gwy_params_get_int(tool->params, PARAM_THICKNESS), NULL);
+        if (plain_tool->layer) {
+            g_object_set(plain_tool->layer,
+                         "thickness", (gdouble)gwy_params_get_int(tool->params, PARAM_THICKNESS),
+                         NULL);
+        }
     }
 }
 
@@ -376,20 +366,20 @@ static void
 gwy_tool_path_level_apply(GwyToolPathLevel *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    GwyDataField *field = plain_tool->data_field;
+    GwyField *field = plain_tool->field;
     gint thickness = gwy_params_get_int(tool->params, PARAM_THICKNESS);
-    gint xres = gwy_data_field_get_xres(field);
-    gint yres = gwy_data_field_get_yres(field);
+    gint xres = gwy_field_get_xres(field);
+    gint yres = gwy_field_get_yres(field);
     gint n = gwy_selection_get_data(plain_tool->selection, NULL);
     ChangePoint *cpts;
-    GwyDataLine *corr;
+    GwyLine *corr;
     gint row, i, j, nw, tp, tn;
     gboolean *wset;
     gdouble *cd, *d;
     gdouble s;
     gint *isel;
 
-    gwy_app_undo_qcheckpoint(plain_tool->container, gwy_app_get_data_key_for_id(plain_tool->id), 0);
+    gwy_app_undo_qcheckpoint(plain_tool->container, gwy_file_key_image(plain_tool->id), 0);
 
     cpts = g_new(ChangePoint, 2*n);
     isel = g_new(gint, 4*n);
@@ -405,9 +395,9 @@ gwy_tool_path_level_apply(GwyToolPathLevel *tool)
 
     qsort(cpts, 2*n, sizeof(ChangePoint), change_point_compare);
     wset = g_new0(gboolean, n);
-    corr = gwy_data_line_new(yres, 1.0, TRUE);
-    cd = gwy_data_line_get_data(corr);
-    d = gwy_data_field_get_data(field);
+    corr = gwy_line_new(yres, 1.0, TRUE);
+    cd = gwy_line_get_data(corr);
+    d = gwy_field_get_data(field);
     tp = (thickness - 1)/2;
     tn = thickness/2;
     i = 0;
@@ -456,7 +446,7 @@ gwy_tool_path_level_apply(GwyToolPathLevel *tool)
     g_free(cpts);
     g_free(isel);
 
-    gwy_data_line_cumulate(corr);
+    gwy_line_cumulate(corr, FALSE);
     for (row = 0; row < yres; row++) {
         s = cd[row];
         for (j = 0; j < xres; j++)
@@ -464,7 +454,7 @@ gwy_tool_path_level_apply(GwyToolPathLevel *tool)
     }
     g_object_unref(corr);
 
-    gwy_data_field_data_changed(field);
+    gwy_field_data_changed(field);
     gwy_params_save_to_settings(tool->params);   /* Ensure correct parameters in the log. */
     gwy_plain_tool_log_add(plain_tool);
 }
@@ -473,17 +463,17 @@ static void
 sel_to_isel(GwyToolPathLevel *tool, gint i, gint *isel)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    GwyDataField *field = plain_tool->data_field;
-    gint xres = gwy_data_field_get_xres(field);
-    gint yres = gwy_data_field_get_yres(field);
+    GwyField *field = plain_tool->field;
+    gint xres = gwy_field_get_xres(field);
+    gint yres = gwy_field_get_yres(field);
     gdouble sel[4];
 
     gwy_selection_get_object(plain_tool->selection, i, sel);
 
-    sel[0] = floor(gwy_data_field_rtoj(field, sel[0]));
-    sel[1] = floor(gwy_data_field_rtoi(field, sel[1]));
-    sel[2] = floor(gwy_data_field_rtoj(field, sel[2]));
-    sel[3] = floor(gwy_data_field_rtoi(field, sel[3]));
+    sel[0] = floor(gwy_field_rtoj(field, sel[0]));
+    sel[1] = floor(gwy_field_rtoi(field, sel[1]));
+    sel[2] = floor(gwy_field_rtoj(field, sel[2]));
+    sel[3] = floor(gwy_field_rtoi(field, sel[3]));
     if (sel[1] > sel[3]) {
         GWY_SWAP(gdouble, sel[0], sel[2]);
         GWY_SWAP(gdouble, sel[1], sel[3]);
