@@ -1,5 +1,5 @@
 /*
- *  $Id: profile.c 26745 2024-10-18 15:17:53Z yeti-dn $
+ *  $Id: profile.c 28805 2025-11-05 12:28:20Z yeti-dn $
  *  Copyright (C) 2003-2022 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,19 +16,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwymodule/gwymodule-tool.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libprocess/datafield.h>
-#include <libprocess/stats.h>
-#include <libprocess/linestats.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwydgets/gwynullstore.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 
 enum {
     RESPONSE_IMPROVE = 100,
@@ -51,23 +42,10 @@ enum {
     PARAM_HOLD_SELECTION,
     PARAM_OPTIONS_VISIBLE,
 
-    PARAM_BOTH,
-    PARAM_DISPLAY,
-
     BUTTON_IMPROVE,
     BUTTON_IMPROVE_ALL,
 };
 
-typedef enum {
-    GWY_CC_DISPLAY_NONE   = 0,
-    GWY_CC_DISPLAY_X_CORR = 1,
-    GWY_CC_DISPLAY_Y_CORR = 2,
-    GWY_CC_DISPLAY_Z_CORR = 3,
-    GWY_CC_DISPLAY_X_UNC  = 4,
-    GWY_CC_DISPLAY_Y_UNC  = 5,
-    GWY_CC_DISPLAY_Z_UNC  = 6,
-} GwyCCDisplayType;
-
 #define GWY_TYPE_TOOL_PROFILE            (gwy_tool_profile_get_type())
 #define GWY_TOOL_PROFILE(obj)            (G_TYPE_CHECK_INSTANCE_CAST((obj), GWY_TYPE_TOOL_PROFILE, GwyToolProfile))
 #define GWY_IS_TOOL_PROFILE(obj)         (G_TYPE_CHECK_INSTANCE_TYPE((obj), GWY_TYPE_TOOL_PROFILE))
@@ -84,34 +62,12 @@ struct _GwyToolProfile {
     GtkTreeView *treeview;
     GtkTreeModel *model;
 
-    GwyDataLine *line;
+    GwyLine *line;
     GwyGraphModel *gmodel;
-    GdkPixbuf *colorpixbuf;
 
     GwyParamTable *table;
-    GwyParamTable *table_unc;
 
-    GwyDataField *xerr;
-    GwyDataField *yerr;
-    GwyDataField *zerr;
-    GwyDataField *xunc;
-    GwyDataField *yunc;
-    GwyDataField *zunc;
-
-    /* curves calculated and output*/
-    GwyDataLine *line_xerr;
-    GwyDataLine *line_yerr;
-    GwyDataLine *line_zerr;
-    GwyDataLine *line_xunc;
-    GwyDataLine *line_yunc;
-    GwyDataLine *line_zunc;
-
-    gboolean has_calibration;
-    GwyCCDisplayType display_type;
-
-    /* potential class data */
-    GwySIValueFormat *pixel_format;
-    GType layer_type_line;
+    GwyValueFormat *pixel_format;
 };
 
 struct _GwyToolProfileClass {
@@ -151,7 +107,6 @@ static void         render_color                      (GtkCellLayout *layout,
                                                        GtkTreeModel *model,
                                                        GtkTreeIter *iter,
                                                        gpointer user_data);
-static void         display_changed                   (GwyToolProfile *tool);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
@@ -178,15 +133,6 @@ module_register(void)
 static GwyParamDef*
 define_module_params(void)
 {
-    static const GwyEnum displays[] = {
-        { N_("calib-data|None"), GWY_CC_DISPLAY_NONE,   },
-        { N_("X correction"),    GWY_CC_DISPLAY_X_CORR, },
-        { N_("Y correction"),    GWY_CC_DISPLAY_Y_CORR, },
-        { N_("Z correction"),    GWY_CC_DISPLAY_Z_CORR, },
-        { N_("X uncertainty"),   GWY_CC_DISPLAY_X_UNC,  },
-        { N_("Y uncertainty"),   GWY_CC_DISPLAY_Y_UNC,  },
-        { N_("Z uncertainty"),   GWY_CC_DISPLAY_Z_UNC,  },
-    };
     static GwyParamDef *paramdef = NULL;
 
     if (paramdef)
@@ -206,10 +152,6 @@ define_module_params(void)
     gwy_param_def_add_hold_selection(paramdef, PARAM_HOLD_SELECTION, "hold_selection", NULL);
     gwy_param_def_add_boolean(paramdef, PARAM_OPTIONS_VISIBLE, "options_visible", NULL, FALSE);
 
-    gwy_param_def_add_boolean(paramdef, PARAM_BOTH, "both", _("_Show profile"), TRUE);
-    gwy_param_def_add_gwyenum(paramdef, PARAM_DISPLAY, "display", _("_Calibration data"),
-                              displays, G_N_ELEMENTS(displays), GWY_CC_DISPLAY_NONE);
-
     return paramdef;
 }
 
@@ -222,7 +164,7 @@ gwy_tool_profile_class_init(GwyToolProfileClass *klass)
 
     gobject_class->finalize = gwy_tool_profile_finalize;
 
-    tool_class->stock_id = GWY_STOCK_PROFILE;
+    tool_class->icon_name = GWY_ICON_PROFILE;
     tool_class->title = _("Profiles");
     tool_class->tooltip = _("Extract profiles along arbitrary lines");
     tool_class->prefix = "/module/profile";
@@ -241,21 +183,14 @@ gwy_tool_profile_finalize(GObject *object)
     GwyToolProfile *tool = GWY_TOOL_PROFILE(object);
 
     gwy_params_save_to_settings(tool->params);
-    GWY_OBJECT_UNREF(tool->params);
-    GWY_OBJECT_UNREF(tool->line);
-    GWY_OBJECT_UNREF(tool->line_xerr);
-    GWY_OBJECT_UNREF(tool->line_yerr);
-    GWY_OBJECT_UNREF(tool->line_zerr);
-    GWY_OBJECT_UNREF(tool->line_xunc);
-    GWY_OBJECT_UNREF(tool->line_yunc);
-    GWY_OBJECT_UNREF(tool->line_zunc);
+    g_clear_object(&tool->params);
+    g_clear_object(&tool->line);
     if (tool->model) {
         gtk_tree_view_set_model(tool->treeview, NULL);
-        GWY_OBJECT_UNREF(tool->model);
+        g_clear_object(&tool->model);
     }
-    GWY_OBJECT_UNREF(tool->colorpixbuf);
-    GWY_OBJECT_UNREF(tool->gmodel);
-    GWY_SI_VALUE_FORMAT_FREE(tool->pixel_format);
+    g_clear_object(&tool->gmodel);
+    GWY_FREE_VALUE_FORMAT(tool->pixel_format);
 
     G_OBJECT_CLASS(gwy_tool_profile_parent_class)->finalize(object);
 }
@@ -264,23 +199,14 @@ static void
 gwy_tool_profile_init(GwyToolProfile *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    gint width, height;
 
-    tool->layer_type_line = gwy_plain_tool_check_layer_type(plain_tool, "GwyLayerLine");
-    if (!tool->layer_type_line)
-        return;
-
-    plain_tool->unit_style = GWY_SI_UNIT_FORMAT_MARKUP;
+    plain_tool->unit_style = GWY_UNIT_FORMAT_MARKUP;
     plain_tool->lazy_updates = TRUE;
 
     tool->params = gwy_params_new_from_settings(define_module_params());
-    tool->pixel_format = gwy_si_unit_value_format_new(1.0, 0, _("px"));
+    tool->pixel_format = gwy_value_format_new(1.0, 0, _("px"));
 
-    gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &width, &height);
-    height |= 1;
-    tool->colorpixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, 8, height, height);
-
-    gwy_plain_tool_connect_selection(plain_tool, tool->layer_type_line, "line");
+    gwy_plain_tool_connect_selection(plain_tool, GWY_TYPE_LAYER_LINE, "line");
     gwy_plain_tool_enable_selection_holding(plain_tool);
 
     gwy_tool_profile_init_dialog(tool);
@@ -304,12 +230,38 @@ gwy_tool_profile_init_dialog(GwyToolProfile *tool)
     tool->gmodel = gwy_graph_model_new();
     g_object_set(tool->gmodel, "title", _("Profiles"), "label-visible", FALSE, NULL);
 
-    hbox = gwy_hbox_new(4);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
     gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(dialog)), hbox, TRUE, TRUE, 0);
 
     /* Left pane */
-    vbox = gwy_vbox_new(8);
-    gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 8);
+    /* FIXME GTK3 Setting hexpand=FALSE is the best simple thing we can do to avoid the controls being allocated way
+     * too large.
+     *
+     * It would not happen if the graph had a much larger natural size. GTK+ first tries to satisfy minimum sizes,
+     * then natural sizes, and then it distributes the rest – always uniformly. Unfortunately, we cannot set the
+     * natrual size here; only the widget iself can. And graphs are fine being quite small, we just like to expand
+     * them a lot when there is extra room.
+     *
+     * Another option – complicated – is to create a SpaceEater adaptor widget. Meaning a GtkBin wrapper, which gives
+     * the same minimum size as the child, but cheats for the natural size. It needs to know the requested size of the
+     * other widget in the box (here the controls) and the box size. Then it requests natural size
+     *
+     * natural = 2(1 - α) graph_min + (2α - 1)(size - controls_natural)
+     *
+     * With this formula GTK+ uniform distribution of the extra size will actually give the α fraction to the graph
+     * and (1-α) fraction to the controls. For α < 1/2 the requested size can become negative, which would be a
+     * problem. But we would only need α like 0.9 to expand mostly the graph and expand the controls just a little bit.
+     *
+     * Another option, less complicated but mostly a kluge, would be to use GtkPaned and control the position manually.
+     * It would have the advantage of letting user resize the ratio. But we would probably often change it
+     * programatically, e.g. when controls are expanded/collapsed, so it might not feel nice.
+     *
+     * The ideal option would be a container which allows to set uneven expansion coefficients. We cannot
+     * realistically override just the size allocation method of GtkBox. There is too much interlinked private code.
+     * So it would have to be another box-like container, just for this purpose. */
+    gtk_widget_set_hexpand(vbox, FALSE);
+    gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, TRUE, 0);
 
     /* Line coordinates */
     store = gwy_null_store_new(0);
@@ -327,8 +279,7 @@ gwy_tool_profile_init_dialog(GwyToolProfile *tool)
         gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(column), renderer, TRUE);
         gtk_cell_layout_set_cell_data_func(GTK_CELL_LAYOUT(column), renderer, render_cell, tool, NULL);
         if (i == COLUMN_I) {
-            renderer = gtk_cell_renderer_pixbuf_new();
-            g_object_set(renderer, "pixbuf", tool->colorpixbuf, NULL);
+            renderer = gwy_cell_renderer_color_new();
             gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(column), renderer, FALSE);
             gtk_cell_layout_set_cell_data_func(GTK_CELL_LAYOUT(column), renderer, render_color, tool, NULL);
         }
@@ -349,7 +300,7 @@ gwy_tool_profile_init_dialog(GwyToolProfile *tool)
     options = gwy_create_expander_with_param(_("<b>Options</b>"), tool->params, PARAM_OPTIONS_VISIBLE);
     gtk_box_pack_start(GTK_BOX(vbox), options, FALSE, FALSE, 0);
 
-    vbox = gwy_vbox_new(8);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 8);
     gtk_container_add(GTK_CONTAINER(options), vbox);
 
     table = tool->table = gwy_param_table_new(tool->params);
@@ -368,19 +319,13 @@ gwy_tool_profile_init_dialog(GwyToolProfile *tool)
     gtk_box_pack_start(GTK_BOX(vbox), gwy_param_table_widget(table), FALSE, FALSE, 0);
     gwy_plain_tool_add_param_table(GWY_PLAIN_TOOL(tool), table);
 
-    table = tool->table_unc = gwy_param_table_new(tool->params);
-    gwy_param_table_append_combo(table, PARAM_DISPLAY);
-    gwy_param_table_append_checkbox(table, PARAM_BOTH);
-    gtk_box_pack_start(GTK_BOX(vbox), gwy_param_table_widget(table), FALSE, FALSE, 0);
-    gwy_plain_tool_add_param_table(GWY_PLAIN_TOOL(tool), table);
-
     graph = gwy_graph_new(tool->gmodel);
     gwy_graph_enable_user_input(GWY_GRAPH(graph), FALSE);
     gtk_box_pack_start(GTK_BOX(hbox), graph, TRUE, TRUE, 2);
 
     gwy_plain_tool_add_clear_button(GWY_PLAIN_TOOL(tool));
     gwy_tool_add_hide_button(GWY_TOOL(tool), FALSE);
-    gtk_dialog_add_button(dialog, GTK_STOCK_APPLY, GTK_RESPONSE_APPLY);
+    gwy_plain_tool_add_apply_button(GWY_PLAIN_TOOL(tool));
     gtk_dialog_set_default_response(dialog, GTK_RESPONSE_APPLY);
     gtk_dialog_set_response_sensitive(dialog, GTK_RESPONSE_APPLY, FALSE);
     gwy_help_add_to_tool_dialog(dialog, GWY_TOOL(tool), GWY_HELP_DEFAULT);
@@ -388,7 +333,6 @@ gwy_tool_profile_init_dialog(GwyToolProfile *tool)
     selection = gtk_tree_view_get_selection(tool->treeview);
     g_signal_connect_swapped(selection, "changed", G_CALLBACK(update_symm_sensitivty), tool);
     g_signal_connect_swapped(tool->table, "param-changed", G_CALLBACK(param_changed), tool);
-    g_signal_connect_swapped(tool->table_unc, "param-changed", G_CALLBACK(param_changed), tool);
 
     gtk_widget_show_all(gtk_dialog_get_content_area(dialog));
     gwy_param_table_param_changed(tool->table, -1);
@@ -400,8 +344,6 @@ gwy_tool_profile_data_switched(GwyTool *gwytool, GwyDataView *data_view)
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(gwytool);
     GwyToolProfile *tool = GWY_TOOL_PROFILE(gwytool);
     gboolean ignore = (data_view == plain_tool->data_view);
-    GtkWidget *unc_table_widget;
-    gchar xekey[32], yekey[32], zekey[32], xukey[32], yukey[32], zukey[32];
 
     GWY_TOOL_CLASS(gwy_tool_profile_parent_class)->data_switched(gwytool, data_view);
 
@@ -409,34 +351,13 @@ gwy_tool_profile_data_switched(GwyTool *gwytool, GwyDataView *data_view)
         return;
 
     if (data_view) {
-        gwy_object_set_or_reset(plain_tool->layer, tool->layer_type_line,
-                                "line-numbers", gwy_params_get_boolean(tool->params, PARAM_NUMBER_LINES),
-                                "thickness", gwy_params_get_int(tool->params, PARAM_THICKNESS),
-                                "center-tick", FALSE,
+        gwy_object_set_or_reset(plain_tool->layer, GWY_TYPE_LAYER_LINE,
+                                "numbered", gwy_params_get_boolean(tool->params, PARAM_NUMBER_LINES),
+                                "thickness", (gdouble)gwy_params_get_int(tool->params, PARAM_THICKNESS),
                                 "editable", TRUE,
                                 "focus", -1,
                                 NULL);
         gwy_selection_set_max_objects(plain_tool->selection, 1024);
-
-        g_snprintf(xekey, sizeof(xekey), "/%d/data/cal_xerr", plain_tool->id);
-        g_snprintf(yekey, sizeof(yekey), "/%d/data/cal_yerr", plain_tool->id);
-        g_snprintf(zekey, sizeof(zekey), "/%d/data/cal_zerr", plain_tool->id);
-        g_snprintf(xukey, sizeof(xukey), "/%d/data/cal_xunc", plain_tool->id);
-        g_snprintf(yukey, sizeof(yukey), "/%d/data/cal_yunc", plain_tool->id);
-        g_snprintf(zukey, sizeof(zukey), "/%d/data/cal_zunc", plain_tool->id);
-
-        unc_table_widget = gwy_param_table_widget(tool->table_unc);
-        tool->has_calibration = (gwy_container_gis_object_by_name(plain_tool->container, xekey, &tool->xerr)
-                                 && gwy_container_gis_object_by_name(plain_tool->container, yekey, &tool->yerr)
-                                 && gwy_container_gis_object_by_name(plain_tool->container, zekey, &tool->zerr)
-                                 && gwy_container_gis_object_by_name(plain_tool->container, xukey, &tool->xunc)
-                                 && gwy_container_gis_object_by_name(plain_tool->container, yukey, &tool->yunc)
-                                 && gwy_container_gis_object_by_name(plain_tool->container, zukey, &tool->zunc));
-        gtk_widget_set_no_show_all(unc_table_widget, !tool->has_calibration);
-        if (tool->has_calibration)
-            gtk_widget_show_all(unc_table_widget);
-        else
-            gtk_widget_hide(unc_table_widget);
         gwy_plain_tool_hold_selection(plain_tool, gwy_params_get_flags(tool->params, PARAM_HOLD_SELECTION));
     }
 
@@ -518,14 +439,14 @@ param_changed(GwyToolProfile *tool, gint id)
                           && id != PARAM_TARGET_GRAPH && id != PARAM_OPTIONS_VISIBLE);
 
     if (id == PARAM_MASKING)
-        do_update = do_update || (plain_tool->data_field && plain_tool->mask_field);
+        do_update = do_update || (plain_tool->field && plain_tool->mask_field);
     if (id < 0 || id == PARAM_THICKNESS) {
         if (plain_tool->layer)
-            g_object_set(plain_tool->layer, "thickness", gwy_params_get_int(params, PARAM_THICKNESS), NULL);
+            g_object_set(plain_tool->layer, "thickness", (gdouble)gwy_params_get_int(params, PARAM_THICKNESS), NULL);
     }
     if (id < 0 || id == PARAM_NUMBER_LINES) {
         if (plain_tool->layer)
-            g_object_set(plain_tool->layer, "line-numbers", gwy_params_get_boolean(params, PARAM_NUMBER_LINES), NULL);
+            g_object_set(plain_tool->layer, "numbered", gwy_params_get_boolean(params, PARAM_NUMBER_LINES), NULL);
     }
     if (id < 0 || id == PARAM_SEPARATE) {
         gboolean separate = gwy_params_get_boolean(params, PARAM_SEPARATE);
@@ -537,8 +458,6 @@ param_changed(GwyToolProfile *tool, gint id)
     }
     if (do_update)
         update_all_curves(tool);
-    if (id < 0 || id == PARAM_BOTH || id == PARAM_DISPLAY)
-        display_changed(tool);
 }
 
 static void
@@ -556,215 +475,47 @@ update_symm_sensitivty(GwyToolProfile *tool)
     gwy_param_table_set_sensitive(tool->table, BUTTON_IMPROVE_ALL, has_lines);
 }
 
-static void
-gwy_data_line_sum(GwyDataLine *a, GwyDataLine *b)
-{
-    gint i;
-    g_return_if_fail(GWY_IS_DATA_LINE(a));
-    g_return_if_fail(GWY_IS_DATA_LINE(b));
-    g_return_if_fail(a->res == b->res);
-
-    for (i = 0; i < a->res; i++)
-        a->data[i] += b->data[i];
-}
-
-static void
-gwy_data_line_subtract(GwyDataLine *a, GwyDataLine *b)
-{
-    gint i;
-    g_return_if_fail(GWY_IS_DATA_LINE(a));
-    g_return_if_fail(GWY_IS_DATA_LINE(b));
-    g_return_if_fail(a->res == b->res);
-
-    for (i = 0; i < a->res; i++)
-        a->data[i] -= b->data[i];
-}
-
-static void
-add_hidden_curve(GwyToolProfile *tool, GwyDataLine *line,
-                 gchar *str, const GwyRGBA *color, gboolean hidden)
-{
-    GwyGraphCurveModel *gcmodel;
-
-    gcmodel = gwy_graph_curve_model_new();
-    g_object_set(gcmodel,
-                 "mode", hidden ? GWY_GRAPH_CURVE_HIDDEN : GWY_GRAPH_CURVE_LINE,
-                 "description", str,
-                 "color", color,
-                 "line-style", GDK_LINE_ON_OFF_DASH,
-                 NULL);
-    gwy_graph_curve_model_set_data_from_dataline(gcmodel, line, 0, 0);
-    gwy_graph_model_add_curve(tool->gmodel, gcmodel);
-    g_object_unref(gcmodel);
-}
-
-static void
-add_hidden_unc_curves(GwyToolProfile *tool, gint i, const GwyRGBA *color,
-                      GwyDataLine *upunc, GwyDataLine *lowunc)
-{
-    gchar *desc;
-
-    desc = g_strdup_printf(_("X error %d"), i);
-    add_hidden_curve(tool, tool->line_xerr, desc, color, !(tool->display_type == 1));
-    g_free(desc);
-
-    desc = g_strdup_printf(_("Y error %d"), i);
-    add_hidden_curve(tool, tool->line_yerr, desc, color, !(tool->display_type == 2));
-    g_free(desc);
-
-    desc = g_strdup_printf(_("Z error %d"), i);
-    add_hidden_curve(tool, tool->line_zerr, desc, color, !(tool->display_type == 3));
-    g_free(desc);
-
-    desc = g_strdup_printf(_("X uncertainty %d"), i);
-    add_hidden_curve(tool, tool->line_xunc, desc, color, !(tool->display_type == 4));
-    g_free(desc);
-
-    desc = g_strdup_printf(_("Y uncertainty %d"), i);
-    add_hidden_curve(tool, tool->line_yunc, desc, color, !(tool->display_type == 5));
-    g_free(desc);
-
-    desc = g_strdup_printf(_("Z uncertainty %d"), i);
-    add_hidden_curve(tool, tool->line_zunc, desc, color, TRUE);
-    g_free(desc);
-
-    desc = g_strdup_printf(_("Zunc up bound %d"), i);
-    add_hidden_curve(tool, upunc, desc, color, !(tool->display_type == 6));
-    g_free(desc);
-
-    desc = g_strdup_printf(_("Zunc low bound %d"), i);
-    add_hidden_curve(tool, lowunc, desc, color, !(tool->display_type == 6));
-    g_free(desc);
-}
-
-static void
-get_profile_uncs(GwyToolProfile *tool,
-                 gint xl1, gint yl1, gint xl2, gint yl2,
-                 gint lineres)
-{
-    GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    GwyDataField *data_field = plain_tool->data_field;
-    gint thickness = gwy_params_get_int(tool->params, PARAM_THICKNESS);
-    GwyInterpolationType interp = gwy_params_get_enum(tool->params, PARAM_INTERPOLATION);
-    gdouble calxratio = ((gdouble)gwy_data_field_get_xres(tool->xerr)/gwy_data_field_get_xres(data_field));
-    gdouble calyratio = ((gdouble)gwy_data_field_get_yres(tool->xerr)/gwy_data_field_get_yres(data_field));
-
-    xl1 *= calxratio;
-    yl1 *= calyratio;
-    xl2 *= calxratio;
-    yl2 *= calyratio;
-
-    tool->line_xerr = gwy_data_field_get_profile(tool->xerr, tool->line_xerr, xl1, yl1, xl2, yl2,
-                                                 lineres, thickness, interp);
-    tool->line_yerr = gwy_data_field_get_profile(tool->yerr, tool->line_yerr, xl1, yl1, xl2, yl2,
-                                                 lineres, thickness, interp);
-    tool->line_zerr = gwy_data_field_get_profile(tool->zerr, tool->line_zerr, xl1, yl1, xl2, yl2,
-                                                 lineres, thickness, interp);
-
-    tool->line_xunc = gwy_data_field_get_profile(tool->xunc, tool->line_xunc, xl1, yl1, xl2, yl2,
-                                                 lineres, thickness, interp);
-    tool->line_yunc = gwy_data_field_get_profile(tool->yunc, tool->line_yunc, xl1, yl1, xl2, yl2,
-                                                 lineres, thickness, interp);
-    tool->line_zunc = gwy_data_field_get_profile(tool->zunc, tool->line_zunc, xl1, yl1, xl2, yl2,
-                                                 lineres, thickness, interp);
-}
-
-static void
-set_unc_gcmodel_data(GwyToolProfile *tool, gint i,
-                     GwyDataLine *upunc, GwyDataLine *lowunc)
-{
-    GwyGraphCurveModel *cmodel;
-
-    cmodel = gwy_graph_model_get_curve(tool->gmodel, i+1);
-    gwy_graph_curve_model_set_data_from_dataline(cmodel, tool->line_xerr, 0, 0);
-
-    cmodel = gwy_graph_model_get_curve(tool->gmodel, i+2);
-    gwy_graph_curve_model_set_data_from_dataline(cmodel, tool->line_yerr, 0, 0);
-
-    cmodel = gwy_graph_model_get_curve(tool->gmodel, i+3);
-    gwy_graph_curve_model_set_data_from_dataline(cmodel, tool->line_zerr, 0, 0);
-
-    cmodel = gwy_graph_model_get_curve(tool->gmodel, i+4);
-    gwy_graph_curve_model_set_data_from_dataline(cmodel, tool->line_xunc, 0, 0);
-
-    cmodel = gwy_graph_model_get_curve(tool->gmodel, i+5);
-    gwy_graph_curve_model_set_data_from_dataline(cmodel, tool->line_yunc, 0, 0);
-
-    cmodel = gwy_graph_model_get_curve(tool->gmodel, i+6);
-    gwy_graph_curve_model_set_data_from_dataline(cmodel, tool->line_zunc, 0, 0);
-
-    cmodel = gwy_graph_model_get_curve(tool->gmodel, i+7);
-    gwy_graph_curve_model_set_data_from_dataline(cmodel, upunc, 0, 0);
-
-    cmodel = gwy_graph_model_get_curve(tool->gmodel, i+8);
-    gwy_graph_curve_model_set_data_from_dataline(cmodel, lowunc, 0, 0);
-}
-
 static void
 update_curve(GwyToolProfile *tool, gint i)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    GwyDataField *data_field = plain_tool->data_field;
-    GwyDataField *mask = plain_tool->mask_field;
+    GwyField *field = plain_tool->field;
+    GwyField *mask = plain_tool->mask_field;
     GwyMaskingType masking = gwy_params_get_masking(tool->params, PARAM_MASKING, &mask);
     gint thickness = gwy_params_get_int(tool->params, PARAM_THICKNESS);
     gboolean fixres = gwy_params_get_boolean(tool->params, PARAM_FIXRES);
     gint res = fixres ? gwy_params_get_int(tool->params, PARAM_RESOLUTION) : -1;
     GwyInterpolationType interp = gwy_params_get_enum(tool->params, PARAM_INTERPOLATION);
-    gboolean has_calibration = tool->has_calibration && !mask;
     GwyGraphCurveModel *gcmodel;
     gdouble line[4];
     gint xl1, yl1, xl2, yl2;
-    gint n, multpos;
+    gint n;
     gchar *desc;
     const GwyRGBA *color;
     GwyXY *xydata = NULL;
-    GwyDataLine *upunc = NULL, *lowunc = NULL;
 
     g_return_if_fail(plain_tool->selection);
     if (!gwy_selection_get_object(plain_tool->selection, i, line))
         g_return_if_reached();
 
-    multpos = has_calibration ? 9 : 1;
-    i *= multpos;
-
-    xl1 = floor(gwy_data_field_rtoj(data_field, line[0]));
-    yl1 = floor(gwy_data_field_rtoi(data_field, line[1]));
-    xl2 = floor(gwy_data_field_rtoj(data_field, line[2]));
-    yl2 = floor(gwy_data_field_rtoi(data_field, line[3]));
+    xl1 = floor(gwy_field_rtoj(field, line[0]));
+    yl1 = floor(gwy_field_rtoi(field, line[1]));
+    xl2 = floor(gwy_field_rtoj(field, line[2]));
+    yl2 = floor(gwy_field_rtoi(field, line[3]));
     if (res <= 0) {
         res = GWY_ROUND(hypot(abs(xl1 - xl2) + 1, abs(yl1 - yl2) + 1));
         res = MAX(res, 4);
     }
 
-    if (has_calibration) {
-        /* Use non-masking profiles with calibration. */
-        tool->line = gwy_data_field_get_profile(data_field, tool->line, xl1, yl1, xl2, yl2,
-                                                res, thickness, interp);
-    }
-    else {
-        xydata = gwy_data_field_get_profile_mask(data_field, &res, mask, masking,
-                                                 line[0], line[1], line[2], line[3],
-                                                 res, thickness, interp);
-        if (!xydata) {
-            xydata = g_new(GwyXY, 1);
-            xydata[0].x = 0.0;
-            xydata[0].y = gwy_data_field_get_dval_real(data_field, 0.5*(line[0] + line[2]), 0.5*(line[1] + line[3]),
-                                                       GWY_INTERPOLATION_ROUND);
-            res = 1;
-        }
-    }
-
-    if (has_calibration) {
-        get_profile_uncs(tool, xl1, yl1, xl2, yl2, res);
-
-        upunc = gwy_data_line_new_alike(tool->line, FALSE);
-        gwy_data_line_copy(tool->line, upunc);
-        gwy_data_line_sum(upunc, tool->line_xerr);
-
-        lowunc = gwy_data_line_new_alike(tool->line, FALSE);
-        gwy_data_line_copy(tool->line, lowunc);
-        gwy_data_line_subtract(lowunc, tool->line_xerr);
+    xydata = gwy_field_get_profile_mask(field, &res, mask, masking,
+                                        line[0], line[1], line[2], line[3],
+                                        res, thickness, interp);
+    if (!xydata) {
+        xydata = g_new(GwyXY, 1);
+        xydata[0].x = 0.0;
+        xydata[0].y = gwy_field_get_dval_real(field, 0.5*(line[0] + line[2]), 0.5*(line[1] + line[3]),
+                                              GWY_INTERPOLATION_ROUND);
+        res = 1;
     }
 
     n = gwy_graph_model_get_n_curves(tool->gmodel);
@@ -773,14 +524,11 @@ update_curve(GwyToolProfile *tool, gint i)
         if (xydata)
             gwy_graph_curve_model_set_data_interleaved(gcmodel, (gdouble*)xydata, res);
         else
-            gwy_graph_curve_model_set_data_from_dataline(gcmodel, tool->line, 0, 0);
-
-        if (has_calibration)
-            set_unc_gcmodel_data(tool, i, upunc, lowunc);
+            gwy_graph_curve_model_set_data_from_line(gcmodel, tool->line, 0, 0);
     }
     else {
         gcmodel = gwy_graph_curve_model_new();
-        desc = g_strdup_printf(_("Profile %d"), i/multpos+1);
+        desc = g_strdup_printf(_("Profile %d"), i+1);
         color = gwy_graph_get_preset_color(i);
         g_object_set(gcmodel,
                      "mode", GWY_GRAPH_CURVE_LINE,
@@ -791,18 +539,15 @@ update_curve(GwyToolProfile *tool, gint i)
         if (xydata)
             gwy_graph_curve_model_set_data_interleaved(gcmodel, (gdouble*)xydata, res);
         else
-            gwy_graph_curve_model_set_data_from_dataline(gcmodel, tool->line, 0, 0);
+            gwy_graph_curve_model_set_data_from_line(gcmodel, tool->line, 0, 0);
 
         gwy_graph_model_add_curve(tool->gmodel, gcmodel);
         g_object_unref(gcmodel);
 
         if (i == 0) {
-            gwy_graph_model_set_units_from_data_field(tool->gmodel, data_field, 1, 0, 0, 1);
+            gwy_graph_model_set_units_from_field(tool->gmodel, field, 1, 0, 0, 1);
             gwy_param_table_data_id_refilter(tool->table, PARAM_TARGET_GRAPH);
         }
-
-        if (has_calibration)
-            add_hidden_unc_curves(tool, i/multpos+1, color, upunc, lowunc);
     }
 
     g_free(xydata);
@@ -856,7 +601,7 @@ update_all_curves(GwyToolProfile *tool)
 }
 
 static gdouble
-estimate_orthogonal_variation(GwyDataField *dfield,
+estimate_orthogonal_variation(GwyField *dfield,
                               const gdouble *line, gint thickness)
 {
     gdouble lx, ly, l, dx, dy, h, xreal, yreal;
@@ -869,8 +614,8 @@ estimate_orthogonal_variation(GwyDataField *dfield,
     ly = line[3] - line[1];
     l = hypot(lx, ly);
 
-    dx = gwy_data_field_get_dx(dfield);
-    dy = gwy_data_field_get_dy(dfield);
+    dx = gwy_field_get_dx(dfield);
+    dy = gwy_field_get_dy(dfield);
     h = 2.0*dx*dy/(dx + dy);
 
     /* First orthogonal profile is (xfrom0,yfrom0)--(xto0,yto0), the last is (xfrom1,yfrom1)--(xto1,yto1), between
@@ -884,8 +629,8 @@ estimate_orthogonal_variation(GwyDataField *dfield,
     yfrom1 = line[3] - lx/l*thickness*h;
     yto1 = line[3] + lx/l*thickness*h;
 
-    xreal = gwy_data_field_get_xreal(dfield);
-    yreal = gwy_data_field_get_yreal(dfield);
+    xreal = gwy_field_get_xreal(dfield);
+    yreal = gwy_field_get_yreal(dfield);
     ir = pow(l/h + 1.0, 2.0/3.0);
     res = thickness+1;
     n = 0;
@@ -909,8 +654,8 @@ estimate_orthogonal_variation(GwyDataField *dfield,
         if (yl2 < 0.5*dy || yl2 > yreal - 0.5*dy)
             continue;
 
-        xy = gwy_data_field_get_profile_mask(dfield, &nxy, NULL, GWY_MASK_IGNORE, xl1, yl1, xl2, yl2,
-                                             res, 1, GWY_INTERPOLATION_LINEAR);
+        xy = gwy_field_get_profile_mask(dfield, &nxy, NULL, GWY_MASK_IGNORE, xl1, yl1, xl2, yl2,
+                                        res, 1, GWY_INTERPOLATION_LINEAR);
         if (!xy)
             continue;
 
@@ -929,7 +674,7 @@ estimate_orthogonal_variation(GwyDataField *dfield,
 }
 
 static void
-straighten_at_scale(GwyDataField *dfield, gdouble *line,
+straighten_at_scale(GwyField *dfield, gdouble *line,
                     gint thickness, gdouble phistep, gint n)
 {
     gdouble r, xc, yc, phi0, phi, cphi, sphi, t;
@@ -979,15 +724,15 @@ static void
 straighten_profile(GwyToolProfile *tool, gint id)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    GwyDataField *dfield = plain_tool->data_field;
+    GwyField *dfield = plain_tool->field;
     gint thickness = gwy_params_get_int(tool->params, PARAM_THICKNESS);
     gdouble line[4];
     gdouble dx, dy;
 
     g_return_if_fail(plain_tool->selection);
     g_return_if_fail(gwy_selection_get_object(plain_tool->selection, id, line));
-    dx = gwy_data_field_get_dx(dfield);
-    dy = gwy_data_field_get_dy(dfield);
+    dx = gwy_field_get_dx(dfield);
+    dy = gwy_field_get_dy(dfield);
     thickness = MAX((thickness + 1)/2, 4);
 
     /* Don't attempt to optimise very short lines. It would end up in tears. */
@@ -1010,7 +755,7 @@ render_cell(GtkCellLayout *layout,
     GwyToolProfile *tool = (GwyToolProfile*)user_data;
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
     guint id = GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(layout), "id"));
-    const GwySIValueFormat *vf = tool->pixel_format;
+    const GwyValueFormat *vf = tool->pixel_format;
     gchar buf[32];
     gdouble line[4];
     gdouble val;
@@ -1026,13 +771,13 @@ render_cell(GtkCellLayout *layout,
     gwy_selection_get_object(plain_tool->selection, idx, line);
 
     if (id == COLUMN_X1)
-        val = floor(gwy_data_field_rtoj(plain_tool->data_field, line[0]));
+        val = floor(gwy_field_rtoj(plain_tool->field, line[0]));
     else if (id == COLUMN_Y1)
-        val = floor(gwy_data_field_rtoi(plain_tool->data_field, line[1]));
+        val = floor(gwy_field_rtoi(plain_tool->field, line[1]));
     else if (id == COLUMN_X2)
-        val = floor(gwy_data_field_rtoj(plain_tool->data_field, line[2]));
+        val = floor(gwy_field_rtoj(plain_tool->field, line[2]));
     else if (id == COLUMN_Y2)
-        val = floor(gwy_data_field_rtoi(plain_tool->data_field, line[3]));
+        val = floor(gwy_field_rtoi(plain_tool->field, line[3]));
     else {
         g_return_if_reached();
     }
@@ -1047,7 +792,7 @@ render_cell(GtkCellLayout *layout,
 
 static void
 render_color(G_GNUC_UNUSED GtkCellLayout *layout,
-             G_GNUC_UNUSED GtkCellRenderer *renderer,
+             GtkCellRenderer *renderer,
              GtkTreeModel *model,
              GtkTreeIter *iter,
              gpointer user_data)
@@ -1055,33 +800,27 @@ render_color(G_GNUC_UNUSED GtkCellLayout *layout,
     GwyToolProfile *tool = (GwyToolProfile*)user_data;
     GwyGraphCurveModel *gcmodel;
     GwyRGBA *rgba;
-    guint idx, pixel;
+    guint idx;
 
     gtk_tree_model_get(model, iter, 0, &idx, -1);
     gcmodel = gwy_graph_model_get_curve(tool->gmodel, idx);
     g_object_get(gcmodel, "color", &rgba, NULL);
-    pixel = 0xff | gwy_rgba_to_pixbuf_pixel(rgba);
+    g_object_set(renderer, "color", rgba, NULL);
     gwy_rgba_free(rgba);
-    gdk_pixbuf_fill(tool->colorpixbuf, pixel);
 }
 
 static void
 gwy_tool_profile_apply(GwyToolProfile *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    GwyDataField *mask = plain_tool->mask_field;
-    G_GNUC_UNUSED GwyMaskingType masking = gwy_params_get_masking(tool->params, PARAM_MASKING, &mask);
-    GwyCCDisplayType display_type = gwy_params_get_enum(tool->params, PARAM_DISPLAY);
+    GwyFile *file = GWY_FILE(plain_tool->container);
     gboolean separate = gwy_params_get_boolean(tool->params, PARAM_SEPARATE);
-    gboolean has_calibration = tool->has_calibration && !mask;
-    GwyGraphCurveModel *gcmodel, *gcm;
-    GwyCurveCalibrationData *ccdata;
+    GwyGraphCurveModel *gcmodel;
     GwyGraphModel *gmodel;
     gchar *s;
-    gint i, n, multpos, size;
 
     g_return_if_fail(plain_tool->selection);
-    n = gwy_selection_get_data(plain_tool->selection, NULL);
+    gint n = gwy_selection_get_data(plain_tool->selection, NULL);
     g_return_if_fail(n);
 
     if ((gmodel = gwy_params_get_graph(tool->params, PARAM_TARGET_GRAPH))) {
@@ -1090,91 +829,28 @@ gwy_tool_profile_apply(GwyToolProfile *tool)
     }
 
     if (!separate) {
-        gmodel = gwy_graph_model_duplicate(tool->gmodel);
+        gmodel = gwy_graph_model_copy(tool->gmodel);
         g_object_set(gmodel, "label-visible", TRUE, NULL);
-        gwy_app_data_browser_add_graph_model(gmodel, plain_tool->container, TRUE);
+        gint newid = gwy_file_add_graph(file, gmodel);
+        gwy_file_set_visible(file, GWY_FILE_GRAPH, newid, TRUE);
         g_object_unref(gmodel);
         return;
     }
 
-    multpos = has_calibration ? 9 : 1;
-    for (i = 0; i < n*multpos; i += multpos) {
+    for (gint i = 0; i < n; i++) {
         gmodel = gwy_graph_model_new_alike(tool->gmodel);
         g_object_set(gmodel, "label-visible", TRUE, NULL);
         gcmodel = gwy_graph_model_get_curve(tool->gmodel, i);
-        gcmodel = gwy_graph_curve_model_duplicate(gcmodel);
-
-        /*add calibration data to the curve*/
-        if (has_calibration) {
-            ccdata = g_new(GwyCurveCalibrationData, 1);
-            size = gwy_graph_curve_model_get_ndata(gcmodel) * sizeof(gdouble);
-            gcm = gwy_graph_model_get_curve(tool->gmodel, i+1);
-            ccdata->xerr = g_memdup(gwy_graph_curve_model_get_ydata(gcm), size);
-            gcm = gwy_graph_model_get_curve(tool->gmodel, i+2);
-            ccdata->yerr = g_memdup(gwy_graph_curve_model_get_ydata(gcm), size);
-            gcm = gwy_graph_model_get_curve(tool->gmodel, i+3);
-            ccdata->zerr = g_memdup(gwy_graph_curve_model_get_ydata(gcm), size);
-            gcm = gwy_graph_model_get_curve(tool->gmodel, i+4);
-            ccdata->xunc = g_memdup(gwy_graph_curve_model_get_ydata(gcm), size);
-            gcm = gwy_graph_model_get_curve(tool->gmodel, i+5);
-            ccdata->yunc = g_memdup(gwy_graph_curve_model_get_ydata(gcm), size);
-            gcm = gwy_graph_model_get_curve(tool->gmodel, i+6);
-            ccdata->zunc = g_memdup(gwy_graph_curve_model_get_ydata(gcm), size);
-            gwy_graph_curve_model_set_calibration_data(gcmodel, ccdata);
-        }
+        gcmodel = gwy_graph_curve_model_copy(gcmodel);
 
         gwy_graph_model_add_curve(gmodel, gcmodel);
         g_object_unref(gcmodel);
         g_object_get(gcmodel, "description", &s, NULL);
         g_object_set(gmodel, "title", s, NULL);
         g_free(s);
-        gwy_app_data_browser_add_graph_model(gmodel, plain_tool->container, TRUE);
+        gint newid = gwy_file_add_graph(file, gmodel);
+        gwy_file_set_visible(file, GWY_FILE_GRAPH, newid, TRUE);
         g_object_unref(gmodel);
-
-        if (display_type > 0) {
-            gmodel = gwy_graph_model_new_alike(tool->gmodel);
-            g_object_set(gmodel, "label-visible", TRUE, NULL);
-            gcmodel = gwy_graph_model_get_curve(tool->gmodel, i+display_type);
-            gcmodel = gwy_graph_curve_model_duplicate(gcmodel);
-            gwy_graph_model_add_curve(gmodel, gcmodel);
-            g_object_unref(gcmodel);
-            g_object_get(gcmodel, "description", &s, NULL);
-            g_object_set(gmodel, "title", s, NULL);
-            g_object_set(gcmodel, "mode", GWY_GRAPH_CURVE_LINE, NULL);
-            g_free(s);
-            gwy_app_data_browser_add_graph_model(gmodel, plain_tool->container, TRUE);
-         }
-    }
-}
-
-static void
-display_changed(GwyToolProfile *tool)
-{
-    GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    GwyCCDisplayType display_type = gwy_params_get_enum(tool->params, PARAM_DISPLAY);
-    gboolean both = gwy_params_get_boolean(tool->params, PARAM_BOTH);
-    GwyGraphCurveModel *gcmodel;
-    gint i, n, multpos = 9;
-
-    if (!tool->has_calibration)
-        return;
-
-    g_return_if_fail(plain_tool->selection);
-    n = gwy_selection_get_data(plain_tool->selection, NULL);
-    if (!n)
-        return;
-
-    /*change the visibility of all the affected curves*/
-    for (i = 0; i < n*multpos; i++) {
-        gcmodel = gwy_graph_model_get_curve(tool->gmodel, i);
-
-        if (i % multpos == 0)
-            g_object_set(gcmodel, "mode", both ? GWY_GRAPH_CURVE_LINE : GWY_GRAPH_CURVE_HIDDEN, NULL);
-        else if ((display_type <= 5 && (i - (gint)display_type) >= 0 && (i - (gint)display_type) % multpos == 0)
-                 || (display_type == 6 && ((i-7) % multpos == 0 || (i-8) % multpos == 0)))
-            g_object_set(gcmodel, "mode", GWY_GRAPH_CURVE_LINE, NULL);
-        else
-            g_object_set(gcmodel, "mode", GWY_GRAPH_CURVE_HIDDEN, NULL);
     }
 }
 
