@@ -1,5 +1,5 @@
 /*
- *  $Id: readvalue.c 26745 2024-10-18 15:17:53Z yeti-dn $
+ *  $Id: readvalue.c 28860 2025-11-14 12:59:38Z yeti-dn $
  *  Copyright (C) 2003-2023 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,17 +16,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/elliptic.h>
-#include <libprocess/stats.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwydgets/gwylayer-basic.h>
-#include <libgwymodule/gwymodule-tool.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
+#include <gwy.h>
 
 enum {
     RADIUS_MAX = 40,
@@ -67,9 +60,8 @@ struct _GwyToolReadValue {
     GwyParams *params;
     GwyResults *results;
 
-    GwyContainer *data;
-    GwyDataField *detail;
-    GwyDataField *detail_mask;
+    GwyField *detail;
+    GwyField *detail_mask;
 
     gdouble avg;
     gdouble bx;
@@ -87,7 +79,7 @@ struct _GwyToolReadValue {
     Range yr;
     gint zisel[4];
     gulong palette_id;
-    gulong mask_colour_id[4];
+    gulong mask_colour_id;
 
     GtkWidget *x;
     GtkWidget *xpix;
@@ -102,9 +94,6 @@ struct _GwyToolReadValue {
 
     /* to prevent double-update on data_changed -- badly designed code? */
     gboolean drawn;
-
-    /* potential class data */
-    GType layer_type_point;
 };
 
 struct _GwyToolReadValueClass {
@@ -199,7 +188,7 @@ gwy_tool_read_value_class_init(GwyToolReadValueClass *klass)
 
     gobject_class->finalize = gwy_tool_read_value_finalize;
 
-    tool_class->stock_id = GWY_STOCK_POINTER_MEASURE;
+    tool_class->icon_name = GWY_ICON_POINTER_MEASURE;
     tool_class->title = _("Read Value");
     tool_class->tooltip = _("Read value under mouse cursor");
     tool_class->prefix = "/module/readvalue";
@@ -215,17 +204,14 @@ gwy_tool_read_value_finalize(GObject *object)
 {
     GwyToolReadValue *tool = GWY_TOOL_READ_VALUE(object);
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    guint i;
 
-    GWY_SIGNAL_HANDLER_DISCONNECT(plain_tool->container, tool->palette_id);
-    for (i = 0; i < G_N_ELEMENTS(tool->mask_colour_id); i++)
-        GWY_SIGNAL_HANDLER_DISCONNECT(plain_tool->container, tool->mask_colour_id[i]);
+    g_clear_signal_handler(&tool->palette_id, plain_tool->container);
+    g_clear_signal_handler(&tool->mask_colour_id, plain_tool->container);
     gwy_params_save_to_settings(tool->params);
-    GWY_OBJECT_UNREF(tool->params);
-    GWY_OBJECT_UNREF(tool->results);
-    GWY_OBJECT_UNREF(tool->data);
-    GWY_OBJECT_UNREF(tool->detail);
-    GWY_OBJECT_UNREF(tool->detail_mask);
+    g_clear_object(&tool->params);
+    g_clear_object(&tool->results);
+    g_clear_object(&tool->detail);
+    g_clear_object(&tool->detail_mask);
     g_free(tool->values);
     g_free(tool->xpos);
     g_free(tool->ypos);
@@ -236,28 +222,16 @@ gwy_tool_read_value_finalize(GObject *object)
 static void
 gwy_tool_read_value_init(GwyToolReadValue *tool)
 {
-    GwyPlainTool *plain_tool;
-    GwyResults *results;
-
-    plain_tool = GWY_PLAIN_TOOL(tool);
-    tool->layer_type_point = gwy_plain_tool_check_layer_type(plain_tool, "GwyLayerPoint");
-    if (!tool->layer_type_point)
-        return;
-
-    plain_tool->unit_style = GWY_SI_UNIT_FORMAT_MARKUP;
+    GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
+    plain_tool->unit_style = GWY_UNIT_FORMAT_MARKUP;
     plain_tool->lazy_updates = TRUE;
 
     tool->params = gwy_params_new_from_settings(define_module_params());
-    gwy_plain_tool_connect_selection(plain_tool, tool->layer_type_point, "pointer");
-    tool->data = gwy_container_new();
-    tool->detail = gwy_data_field_new(PREVIEW_SIZE, PREVIEW_SIZE, PREVIEW_SIZE, PREVIEW_SIZE, TRUE);
-    tool->detail_mask = gwy_data_field_new(PREVIEW_SIZE, PREVIEW_SIZE, PREVIEW_SIZE, PREVIEW_SIZE, TRUE);
-    gwy_container_set_object(tool->data, gwy_app_get_data_key_for_id(0), tool->detail);
-    if (gwy_params_get_boolean(tool->params, PARAM_SHOW_MASK))
-        gwy_container_set_object(tool->data, gwy_app_get_mask_key_for_id(0), tool->detail_mask);
-    adapt_colour_range(tool, TRUE);
+    gwy_plain_tool_connect_selection(plain_tool, GWY_TYPE_LAYER_POINT, "pointer");
+    tool->detail = gwy_field_new(PREVIEW_SIZE, PREVIEW_SIZE, PREVIEW_SIZE, PREVIEW_SIZE, TRUE);
+    tool->detail_mask = gwy_field_new(PREVIEW_SIZE, PREVIEW_SIZE, PREVIEW_SIZE, PREVIEW_SIZE, TRUE);
 
-    results = tool->results = gwy_results_new();
+    GwyResults *results = tool->results = gwy_results_new();
     gwy_results_add_value_z(results, "z", "Z");
     gwy_results_add_value_angle(results, "theta", N_("Inclination θ"));
     gwy_results_add_value_angle(results, "phi", N_("Inclination φ"));
@@ -269,30 +243,28 @@ gwy_tool_read_value_init(GwyToolReadValue *tool)
 }
 
 static void
-attach_coord_row(GtkTable *table, const gchar *name, gint row,
+attach_coord_row(GtkGrid *grid, const gchar *name, gint row,
                  GtkWidget **pxspin, GtkWidget **reallabel)
 {
-    GtkObject *adj;
-    GtkWidget *spin, *hbox, *label;
+    GtkWidget *hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
+    gtk_widget_set_hexpand(hbox, TRUE);
+    gtk_grid_attach(grid, hbox, 0, row, 3, 1);
 
-    hbox = gwy_hbox_new(4);
-    gtk_table_attach(table, hbox, 0, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
-
-    label = gtk_label_new(name);
-    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+    GtkWidget *label = gtk_label_new(name);
+    gtk_label_set_xalign(GTK_LABEL(label), 0.0);
     gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
 
     gtk_box_pack_end(GTK_BOX(hbox), gtk_label_new(_("px")), FALSE, FALSE, 0);
 
-    adj = gtk_adjustment_new(1.0, 1.0, 100.0, 1.0, 10.0, 0.0);
-    spin = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.0, 0);
+    GtkAdjustment *adj = gtk_adjustment_new(1.0, 1.0, 100.0, 1.0, 10.0, 0.0);
+    GtkWidget *spin = gtk_spin_button_new(adj, 0.0, 0);
     gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
     gtk_entry_set_width_chars(GTK_ENTRY(spin), 4);
     gtk_entry_set_text(GTK_ENTRY(spin), "");
     gtk_box_pack_end(GTK_BOX(hbox), spin, FALSE, FALSE, 0);
 
     label = gtk_label_new(NULL);
-    gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
+    gtk_label_set_xalign(GTK_LABEL(label), 1.0);
     gtk_box_pack_end(GTK_BOX(hbox), label, FALSE, FALSE, 4);
 
     *pxspin = spin;
@@ -303,44 +275,43 @@ static void
 gwy_tool_read_value_init_dialog(GwyToolReadValue *tool)
 {
     GtkDialog *dialog = GTK_DIALOG(GWY_TOOL(tool)->dialog);
-    GtkTable *table;
     GtkWidget *hbox, *vbox;
     gint row;
 
-    hbox = gwy_hbox_new(8);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 8);
     gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(dialog)), hbox, TRUE, TRUE, 0);
 
     /* Zoom view */
-    vbox = gwy_vbox_new(0);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
     gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);
 
-    tool->zoomview = gwy_create_preview(tool->data, 0, PREVIEW_SIZE*SCALE, TRUE);
-    tool->zselection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(tool->zoomview), 0, "Point", 1, FALSE);
-    g_object_set(gwy_data_view_get_top_layer(GWY_DATA_VIEW(tool->zoomview)),
-                 "marker-radius", gwy_params_get_int(tool->params, PARAM_RADIUS),
-                 "focus", -1,
-                 NULL);
+    tool->zoomview = gwy_create_preview(tool->detail, NULL, PREVIEW_SIZE*SCALE);
+    if (gwy_params_get_boolean(tool->params, PARAM_SHOW_MASK))
+        gwy_data_view_set_mask(GWY_DATA_VIEW(tool->zoomview), tool->detail_mask);
+    adapt_colour_range(tool, TRUE);
+
+    gtk_widget_set_size_request(tool->zoomview, PREVIEW_SIZE*SCALE, PREVIEW_SIZE*SCALE);
+    tool->zselection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(tool->zoomview), GWY_TYPE_LAYER_POINT, 1, FALSE);
+    /* FIXME GTK3 radius is now double. */
+    GwyVectorLayer *layer = gwy_data_view_get_interactive_layer(GWY_DATA_VIEW(tool->zoomview));
+    gwy_layer_point_set_marker_radius(GWY_LAYER_POINT(layer), gwy_params_get_int(tool->params, PARAM_RADIUS));
+    gwy_vector_layer_set_focus(GWY_VECTOR_LAYER(layer), -1);
     gtk_box_pack_start(GTK_BOX(vbox), tool->zoomview, FALSE, FALSE, 0);
 
     /* Right pane */
-    vbox = gwy_vbox_new(4);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
     gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);
 
-    table = GTK_TABLE(gtk_table_new(3, 3, FALSE));
-    gtk_table_set_col_spacings(table, 6);
-    gtk_table_set_row_spacings(table, 2);
-    gtk_container_set_border_width(GTK_CONTAINER(table), 4);
-    gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(table), FALSE, FALSE, 0);
+    GtkGrid *grid = GTK_GRID(gtk_grid_new());
+    gtk_grid_set_row_spacing(grid, 2);
+    gtk_grid_set_column_spacing(grid, 6);
+    gtk_container_set_border_width(GTK_CONTAINER(grid), 4);
+    gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(grid), FALSE, FALSE, 0);
+
     row = 0;
-
-    gtk_table_attach(table, gwy_label_new_header(_("Position")), 0, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
-    row++;
-
-    attach_coord_row(table, "X", row, &tool->xpix, &tool->x);
-    row++;
-    attach_coord_row(table, "Y", row, &tool->ypix, &tool->y);
-    gtk_table_set_row_spacing(table, row, 4);
-    row++;
+    gtk_grid_attach(grid, gwy_label_new_header(_("Position")), 0, row++, 3, 1);
+    attach_coord_row(grid, "X", row++, &tool->xpix, &tool->x);
+    attach_coord_row(grid, "Y", row++, &tool->ypix, &tool->y);
 
     tool->table = gwy_param_table_new(tool->params);
     gwy_param_table_append_header(tool->table, -1, _("Value"));
@@ -361,7 +332,9 @@ gwy_tool_read_value_init_dialog(GwyToolReadValue *tool)
     gwy_param_table_append_checkbox(tool->table, PARAM_ADAPT_COLOR_RANGE);
     gwy_param_table_append_checkbox(tool->table, PARAM_SHOW_MASK);
     gwy_plain_tool_add_param_table(GWY_PLAIN_TOOL(tool), tool->table);
-    gtk_box_pack_start(GTK_BOX(vbox), gwy_param_table_widget(tool->table), FALSE, FALSE, 0);
+    GtkWidget *table_widget = gwy_param_table_widget(tool->table);
+    gtk_widget_set_margin_top(table_widget, 4);
+    gtk_box_pack_start(GTK_BOX(vbox), table_widget, FALSE, FALSE, 0);
 
     gwy_plain_tool_add_clear_button(GWY_PLAIN_TOOL(tool));
     gwy_tool_add_hide_button(GWY_TOOL(tool), TRUE);
@@ -377,14 +350,11 @@ gwy_tool_read_value_init_dialog(GwyToolReadValue *tool)
 }
 
 static gulong
-connect_to_item(GwyContainer *container, GQuark key1, const gchar *key2,
+connect_to_item(GwyContainer *container, GQuark key1,
                 GCallback callback, gpointer callback_data)
 {
-    gchar *sigdetail;
-    gulong signal_id;
-
-    sigdetail = g_strconcat("item-changed::", g_quark_to_string(key1), key2, NULL);
-    signal_id = g_signal_connect_swapped(container, sigdetail, callback, callback_data);
+    gchar *sigdetail = g_strconcat("item-changed::", g_quark_to_string(key1), NULL);
+    gulong signal_id = g_signal_connect_swapped(container, sigdetail, callback, callback_data);
     g_free(sigdetail);
 
     return signal_id;
@@ -393,19 +363,16 @@ connect_to_item(GwyContainer *container, GQuark key1, const gchar *key2,
 static void
 gwy_tool_read_value_data_switched(GwyTool *gwytool, GwyDataView *data_view)
 {
-    static const gchar *colour_components[4] = { "/red", "/green", "/blue", "/alpha" };
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(gwytool);
     GwyToolReadValue *tool = GWY_TOOL_READ_VALUE(gwytool);
     GQuark quark;
     gboolean ignore;
-    guint i;
 
     ignore = (data_view == plain_tool->data_view);
 
     if (!ignore) {
-        GWY_SIGNAL_HANDLER_DISCONNECT(plain_tool->container, tool->palette_id);
-        for (i = 0; i < G_N_ELEMENTS(tool->mask_colour_id); i++)
-            GWY_SIGNAL_HANDLER_DISCONNECT(plain_tool->container, tool->mask_colour_id[i]);
+        g_clear_signal_handler(&tool->palette_id, plain_tool->container);
+        g_clear_signal_handler(&tool->mask_colour_id, plain_tool->container);
     }
 
     GWY_TOOL_CLASS(gwy_tool_read_value_parent_class)->data_switched(gwytool, data_view);
@@ -414,9 +381,9 @@ gwy_tool_read_value_data_switched(GwyTool *gwytool, GwyDataView *data_view)
         return;
 
     if (data_view) {
-        gwy_object_set_or_reset(plain_tool->layer, tool->layer_type_point,
+        gwy_object_set_or_reset(plain_tool->layer, GWY_TYPE_LAYER_POINT,
                                 "draw-marker", gwy_params_get_boolean(tool->params, PARAM_SHOW_SELECTION),
-                                "marker-radius", gwy_params_get_int(tool->params, PARAM_RADIUS),
+                                "marker-radius", (gdouble)gwy_params_get_int(tool->params, PARAM_RADIUS),
                                 "editable", TRUE,
                                 "focus", -1,
                                 NULL);
@@ -428,14 +395,10 @@ gwy_tool_read_value_data_switched(GwyTool *gwytool, GwyDataView *data_view)
 
         /* FIXME: We may want to respond also to colour mapping keys in case something else than icolorange changes
          * them (icolorange can't because it is another tool). */
-        quark = gwy_app_get_data_palette_key_for_id(plain_tool->id);
-        tool->palette_id = connect_to_item(plain_tool->container, quark, NULL,
-                                           G_CALLBACK(palette_changed), tool);
-        quark = gwy_app_get_mask_key_for_id(plain_tool->id);
-        for (i = 0; i < G_N_ELEMENTS(tool->mask_colour_id); i++) {
-            tool->mask_colour_id[i] = connect_to_item(plain_tool->container, quark, colour_components[i],
-                                                      G_CALLBACK(mask_colour_changed), tool);
-        }
+        quark = gwy_file_key_image_palette(plain_tool->id);
+        tool->palette_id = connect_to_item(plain_tool->container, quark, G_CALLBACK(palette_changed), tool);
+        quark = gwy_file_key_image_mask(plain_tool->id);
+        tool->mask_colour_id = connect_to_item(plain_tool->container, quark, G_CALLBACK(mask_colour_changed), tool);
 
         adapt_colour_range(tool, FALSE);
         palette_changed(tool);
@@ -455,41 +418,41 @@ static void
 update_units(GwyToolReadValue *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    GwyDataField *dfield = plain_tool->data_field;
-    GwySIUnit *siunitxy = gwy_data_field_get_si_unit_xy(dfield);
-    GwySIUnit *siunitz = gwy_data_field_get_si_unit_z(dfield);
+    GwyField *dfield = plain_tool->field;
+    GwyUnit *siunitxy = gwy_field_get_unit_xy(dfield);
+    GwyUnit *siunitz = gwy_field_get_unit_z(dfield);
     gint dxres, dyres;
 
-    tool->same_units = gwy_si_unit_equal(siunitxy, siunitz);
+    tool->same_units = gwy_unit_equal(siunitxy, siunitz);
 
     gwy_results_set_unit(tool->results, "x", siunitxy);
     gwy_results_set_unit(tool->results, "z", siunitz);
 
-    gwy_data_field_copy_units(dfield, tool->detail);
-    dxres = gwy_data_field_get_xres(tool->detail);
-    dyres = gwy_data_field_get_yres(tool->detail);
-    gwy_data_field_set_xreal(tool->detail, dxres*gwy_data_field_get_dx(dfield));
-    gwy_data_field_set_yreal(tool->detail, dyres*gwy_data_field_get_dy(dfield));
+    gwy_field_copy_units(dfield, tool->detail);
+    dxres = gwy_field_get_xres(tool->detail);
+    dyres = gwy_field_get_yres(tool->detail);
+    gwy_field_set_xreal(tool->detail, dxres*gwy_field_get_dx(dfield));
+    gwy_field_set_yreal(tool->detail, dyres*gwy_field_get_dy(dfield));
 
-    gtk_spin_button_set_range(GTK_SPIN_BUTTON(tool->xpix), 1, gwy_data_field_get_xres(dfield));
-    gtk_spin_button_set_range(GTK_SPIN_BUTTON(tool->ypix), 1, gwy_data_field_get_yres(dfield));
+    gtk_spin_button_set_range(GTK_SPIN_BUTTON(tool->xpix), 1, gwy_field_get_xres(dfield));
+    gtk_spin_button_set_range(GTK_SPIN_BUTTON(tool->ypix), 1, gwy_field_get_yres(dfield));
 }
 
 static void
 resize_detail(GwyToolReadValue *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    GwyDataField *dfield = plain_tool->data_field;
+    GwyField *dfield = plain_tool->field;
     gint xres, yres, dxres, dyres, minres, maxres, newdxres, newdyres, newmaxr;
     gdouble newzoom;
 
     if (!dfield)
         return;
 
-    xres = gwy_data_field_get_xres(dfield);
-    yres = gwy_data_field_get_yres(dfield);
-    dxres = gwy_data_field_get_xres(tool->detail);
-    dyres = gwy_data_field_get_yres(tool->detail);
+    xres = gwy_field_get_xres(dfield);
+    yres = gwy_field_get_yres(dfield);
+    dxres = gwy_field_get_xres(tool->detail);
+    dyres = gwy_field_get_yres(tool->detail);
     gwy_debug("image %dx%d, detail %dx%d", xres, yres, dxres, dyres);
 
     /* Max determines the displayed region. */
@@ -508,17 +471,17 @@ resize_detail(GwyToolReadValue *tool)
     newmaxr = MAX((MIN(newdyres, newdyres) - 3)/2, 1);
     gwy_param_table_slider_restrict_range(tool->table, PARAM_RADIUS, 1, newmaxr);
 
-    gwy_data_field_resample(tool->detail, newdxres, newdyres, GWY_INTERPOLATION_NONE);
-    gwy_data_field_clear(tool->detail);
+    gwy_field_resize(tool->detail, newdxres, newdyres);
+    gwy_field_clear(tool->detail);
 
-    gwy_data_field_resample(tool->detail_mask, newdxres, newdyres, GWY_INTERPOLATION_NONE);
-    gwy_data_field_clear(tool->detail_mask);
+    gwy_field_resize(tool->detail_mask, newdxres, newdyres);
+    gwy_field_clear(tool->detail_mask);
 
     newzoom = (gdouble)SCALE/MAX(newdxres, newdyres)*PREVIEW_SIZE;
     gwy_debug("updating zoom to %g", newzoom);
     gwy_data_view_set_zoom(GWY_DATA_VIEW(tool->zoomview), newzoom);
-    gwy_data_field_data_changed(tool->detail);
-    gwy_data_field_data_changed(tool->detail_mask);
+    gwy_field_data_changed(tool->detail);
+    gwy_field_data_changed(tool->detail_mask);
 }
 
 static void
@@ -550,12 +513,9 @@ gwy_tool_read_value_mask_changed(GwyPlainTool *plain_tool)
 static void
 palette_changed(GwyToolReadValue *tool)
 {
-    GwyPlainTool *plain_tool;
-
-    plain_tool = GWY_PLAIN_TOOL(tool);
-    gwy_app_sync_data_items(plain_tool->container, tool->data, plain_tool->id, 0, TRUE,
-                            GWY_DATA_ITEM_GRADIENT,
-                            0);
+    GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
+    gwy_setup_data_view(GWY_DATA_VIEW(tool->zoomview), GWY_FILE(plain_tool->container), GWY_FILE_IMAGE, plain_tool->id,
+                        GWY_FILE_ITEM_PALETTE);
 }
 
 static void
@@ -596,7 +556,7 @@ static void
 gwy_tool_read_value_selection_changed(GwyPlainTool *plain_tool, gint hint)
 {
     GwyToolReadValue *tool;
-    GwyDataField *dfield;
+    GwyField *dfield;
     Range xr, yr;
     gboolean has_selection, complete;
     gint xres, yres, dxres, dyres;
@@ -607,7 +567,7 @@ gwy_tool_read_value_selection_changed(GwyPlainTool *plain_tool, gint hint)
     g_return_if_fail(hint <= 0);
 
     has_selection = FALSE;
-    dfield = plain_tool->data_field;
+    dfield = plain_tool->field;
     if (plain_tool->selection)
         has_selection = gwy_selection_get_object(plain_tool->selection, 0, sel);
 
@@ -616,12 +576,12 @@ gwy_tool_read_value_selection_changed(GwyPlainTool *plain_tool, gint hint)
 
     complete = TRUE;
     if (has_selection) {
-        dxres = gwy_data_field_get_xres(tool->detail);
-        dyres = gwy_data_field_get_yres(tool->detail);
-        isel[0] = floor(gwy_data_field_rtoj(dfield, sel[0]));
-        isel[1] = floor(gwy_data_field_rtoi(dfield, sel[1]));
-        xres = gwy_data_field_get_xres(dfield);
-        yres = gwy_data_field_get_yres(dfield);
+        dxres = gwy_field_get_xres(tool->detail);
+        dyres = gwy_field_get_yres(tool->detail);
+        isel[0] = floor(gwy_field_rtoj(dfield, sel[0]));
+        isel[1] = floor(gwy_field_rtoi(dfield, sel[1]));
+        xres = gwy_field_get_xres(dfield);
+        yres = gwy_field_get_yres(dfield);
         complete &= find_subrange(isel[0], xres, dxres, &xr);
         complete &= find_subrange(isel[1], yres, dyres, &yr);
         gwy_debug("complete: %d", complete);
@@ -649,8 +609,8 @@ gwy_tool_read_value_selection_changed(GwyPlainTool *plain_tool, gint hint)
 
     gwy_debug("x: %d - %d => %d", isel[0], tool->xr.from, isel[0] - tool->xr.from);
     gwy_debug("x: %d - %d => %d", isel[1], tool->yr.from, isel[1] - tool->yr.from);
-    sel[0] = gwy_data_field_jtor(dfield, isel[0] - tool->xr.from + 0.5);
-    sel[1] = gwy_data_field_itor(dfield, isel[1] - tool->yr.from + 0.5);
+    sel[0] = gwy_field_jtor(dfield, isel[0] - tool->xr.from + 0.5);
+    sel[1] = gwy_field_itor(dfield, isel[1] - tool->yr.from + 0.5);
     gwy_selection_set_object(tool->zselection, 0, sel);
 }
 
@@ -659,30 +619,28 @@ param_changed(GwyToolReadValue *tool, gint id)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
     GwyParams *params = tool->params;
+    GwyLayerPoint *layer = plain_tool->layer ? GWY_LAYER_POINT(plain_tool->layer) : NULL;
 
     if (id == PARAM_RADIUS || id < 0) {
-        gint radius = gwy_params_get_int(params, PARAM_RADIUS);
-        if (plain_tool->layer)
-            g_object_set(plain_tool->layer, "marker-radius", radius, NULL);
+        gdouble radius = gwy_params_get_int(params, PARAM_RADIUS);
+        if (layer)
+            gwy_layer_point_set_marker_radius(layer, radius);
         if (plain_tool->selection)
             update_values(tool);
 
-        g_object_set(gwy_data_view_get_top_layer(GWY_DATA_VIEW(tool->zoomview)), "marker-radius", radius, NULL);
+        GwyLayerPoint *tool_layer = GWY_LAYER_POINT(gwy_data_view_get_interactive_layer(GWY_DATA_VIEW(tool->zoomview)));
+        gwy_layer_point_set_marker_radius(tool_layer, radius);
     }
     if (id == PARAM_SHOW_SELECTION || id < 0) {
-        if (plain_tool->layer) {
-            g_object_set(plain_tool->layer,
-                         "draw-marker", gwy_params_get_boolean(tool->params, PARAM_SHOW_SELECTION),
-                         NULL);
-        }
+        if (layer)
+            gwy_layer_point_set_draw_marker(layer, gwy_params_get_boolean(tool->params, PARAM_SHOW_SELECTION));
     }
     if (id < 0 || id == PARAM_ADAPT_COLOR_RANGE)
         adapt_colour_range(tool, FALSE);
     if (id < 0 || id == PARAM_SHOW_MASK) {
-        if (gwy_params_get_boolean(tool->params, PARAM_SHOW_MASK))
-            gwy_container_set_object(tool->data, gwy_app_get_mask_key_for_id(0), tool->detail_mask);
-        else
-            gwy_container_remove(tool->data, gwy_app_get_mask_key_for_id(0));
+        GwyDataView *dataview = GWY_DATA_VIEW(tool->zoomview);
+        gwy_data_view_set_mask(dataview,
+                               gwy_params_get_boolean(tool->params, PARAM_SHOW_MASK) ? tool->detail_mask : NULL);
     }
 }
 
@@ -691,14 +649,14 @@ dialog_response(GwyToolReadValue *tool, gint response)
 {
     if (response == RESPONSE_SET_ZERO) {
         GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-        GQuark quark = gwy_app_get_data_key_for_id(plain_tool->id);
+        GQuark quark = gwy_file_key_image(plain_tool->id);
 
-        if (!plain_tool->data_field || !gwy_selection_get_data(plain_tool->selection, NULL) || !tool->avg)
+        if (!plain_tool->field || !gwy_selection_get_data(plain_tool->selection, NULL) || !tool->avg)
             return;
 
         gwy_app_undo_qcheckpointv(plain_tool->container, 1, &quark);
-        gwy_data_field_add(plain_tool->data_field, -tool->avg);
-        gwy_data_field_data_changed(plain_tool->data_field);
+        gwy_field_add(plain_tool->field, -tool->avg);
+        gwy_field_data_changed(plain_tool->field);
     }
 }
 
@@ -709,77 +667,73 @@ draw_zoom(GwyToolReadValue *tool)
     gdouble min;
 
     if (tool->xr.from < 0 || tool->yr.from < 0) {
-        gwy_data_field_clear(tool->detail);
-        gwy_data_field_clear(tool->detail_mask);
+        gwy_field_clear(tool->detail);
+        gwy_field_clear(tool->detail_mask);
         adapt_colour_range(tool, TRUE);
     }
     else {
         plain_tool = GWY_PLAIN_TOOL(tool);
         if (!tool->complete) {
-            min = gwy_data_field_area_get_min(plain_tool->data_field, NULL,
-                                              tool->xr.from, tool->yr.from,
-                                              tool->xr.to - tool->xr.from,
-                                              tool->yr.to - tool->yr.from);
-            gwy_data_field_fill(tool->detail, min);
-            gwy_data_field_clear(tool->detail_mask);
+            min = gwy_field_area_get_min(plain_tool->field, NULL, GWY_MASK_IGNORE,
+                                         tool->xr.from, tool->yr.from,
+                                         tool->xr.to - tool->xr.from,
+                                         tool->yr.to - tool->yr.from);
+            gwy_field_fill(tool->detail, min);
+            gwy_field_clear(tool->detail_mask);
         }
-        gwy_data_field_area_copy(plain_tool->data_field, tool->detail,
-                                 tool->xr.from, tool->yr.from,
-                                 tool->xr.to - tool->xr.from, tool->yr.to - tool->yr.from,
-                                 tool->xr.dest, tool->yr.dest);
+        gwy_field_area_copy(plain_tool->field, tool->detail,
+                            tool->xr.from, tool->yr.from,
+                            tool->xr.to - tool->xr.from, tool->yr.to - tool->yr.from,
+                            tool->xr.dest, tool->yr.dest);
         adapt_colour_range(tool, FALSE);
         if (plain_tool->mask_field) {
-            gwy_data_field_area_copy(plain_tool->mask_field, tool->detail_mask,
-                                     tool->xr.from, tool->yr.from,
-                                     tool->xr.to - tool->xr.from, tool->yr.to - tool->yr.from,
-                                     tool->xr.dest, tool->yr.dest);
+            gwy_field_area_copy(plain_tool->mask_field, tool->detail_mask,
+                                tool->xr.from, tool->yr.from,
+                                tool->xr.to - tool->xr.from, tool->yr.to - tool->yr.from,
+                                tool->xr.dest, tool->yr.dest);
         }
         else
-            gwy_data_field_clear(tool->detail_mask);
+            gwy_field_clear(tool->detail_mask);
     }
-    gwy_data_field_data_changed(tool->detail);
-    gwy_data_field_data_changed(tool->detail_mask);
+    gwy_field_data_changed(tool->detail);
+    gwy_field_data_changed(tool->detail_mask);
 }
 
 static void
 adapt_colour_range(GwyToolReadValue *tool, gboolean make_empty)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    GwyContainer *data = tool->data;
-    GwyDataField *field = plain_tool->data_field;
+    GwyField *field = plain_tool->field;
     gboolean do_adapt = gwy_params_get_boolean(tool->params, PARAM_ADAPT_COLOR_RANGE);
     gint id = plain_tool->id;
-    GwyLayerBasicRangeType range_type = GWY_LAYER_BASIC_RANGE_FULL;
+    GwyColorMappingType range_type = GWY_COLOR_MAPPING_FULL;
+    GwyDataView *dataview = GWY_DATA_VIEW(tool->zoomview);
     gdouble min = 0.0, max = 0.0;
 
     if (make_empty || !field || !plain_tool->container) {
-        gwy_container_set_enum(data, gwy_app_get_data_range_type_key_for_id(0), GWY_LAYER_BASIC_RANGE_FULL);
-        gwy_container_set_double(data, gwy_app_get_data_range_min_key_for_id(0), 0.0);
-        gwy_container_set_double(data, gwy_app_get_data_range_max_key_for_id(0), 0.0);
+        gwy_data_view_set_color_mapping(dataview, GWY_COLOR_MAPPING_FULL);
+        gwy_data_view_set_fixed_color_range(dataview, 0.0, 0.0);
         return;
     }
 
     if (do_adapt) {
-        gwy_container_set_enum(data, gwy_app_get_data_range_type_key_for_id(0), GWY_LAYER_BASIC_RANGE_FULL);
-        gwy_container_remove(data, gwy_app_get_data_range_min_key_for_id(0));
-        gwy_container_remove(data, gwy_app_get_data_range_max_key_for_id(0));
+        gwy_data_view_set_color_mapping(dataview, GWY_COLOR_MAPPING_FULL);
         return;
     }
 
-    gwy_container_gis_enum(plain_tool->container, gwy_app_get_data_range_type_key_for_id(id), &range_type);
-    if (range_type == GWY_LAYER_BASIC_RANGE_AUTO)
-        gwy_data_field_get_autorange(field, &min, &max);
+    gwy_container_gis_enum(plain_tool->container, gwy_file_key_image_color_mapping(id), &range_type);
+    if (range_type == GWY_COLOR_MAPPING_AUTO)
+        gwy_field_get_autorange(field, &min, &max);
     else {
-        gwy_data_field_get_min_max(field, &min, &max);
-        if (range_type == GWY_LAYER_BASIC_RANGE_FIXED) {
-            gwy_container_gis_double(plain_tool->container, gwy_app_get_data_range_min_key_for_id(id), &min);
-            gwy_container_gis_double(plain_tool->container, gwy_app_get_data_range_max_key_for_id(id), &max);
+        gwy_field_get_min_max(field, &min, &max);
+        if (range_type == GWY_COLOR_MAPPING_FIXED) {
+            gwy_container_gis_double(plain_tool->container, gwy_file_key_image_range_min(id), &min);
+            gwy_container_gis_double(plain_tool->container, gwy_file_key_image_range_max(id), &max);
         }
         /* FIXME: Adaptive mapping using the distribution of different data field is currently not possible. */
     }
-    gwy_container_set_enum(data, gwy_app_get_data_range_type_key_for_id(0), GWY_LAYER_BASIC_RANGE_FIXED);
-    gwy_container_set_double(data, gwy_app_get_data_range_min_key_for_id(0), min);
-    gwy_container_set_double(data, gwy_app_get_data_range_max_key_for_id(0), max);
+    gwy_data_view_set_fixed_color_range(dataview, min, max);
+    gwy_data_view_set_color_mapping(dataview, GWY_COLOR_MAPPING_FIXED);
 }
 
 static void
@@ -790,9 +744,9 @@ copy_mask_colour(GwyToolReadValue *tool)
     gint id = plain_tool->id;
     GwyRGBA colour;
 
-    if (!data || !gwy_rgba_get_from_container(&colour, data, g_quark_to_string(gwy_app_get_mask_key_for_id(id))))
-        gwy_rgba_get_from_container(&colour, gwy_app_settings_get(), "/mask");
-    gwy_rgba_store_to_container(&colour, tool->data, g_quark_to_string(gwy_app_get_mask_key_for_id(0)));
+    if (!data || !gwy_container_gis_boxed(data, GWY_TYPE_RGBA, gwy_file_key_image_mask_color(id), &colour))
+        gwy_app_settings_get_default_mask_color(&colour);
+    gwy_data_view_set_mask_color(GWY_DATA_VIEW(tool->zoomview), &colour);
 }
 
 static void
@@ -805,7 +759,7 @@ pix_spinned(GwyToolReadValue *tool)
     if (tool->in_update)
         return;
 
-    if (!plain_tool->selection || !plain_tool->data_field)
+    if (!plain_tool->selection || !plain_tool->field)
         return;
 
     if (!strlen(gtk_entry_get_text(GTK_ENTRY(tool->xpix)))
@@ -813,14 +767,14 @@ pix_spinned(GwyToolReadValue *tool)
         return;
 
     spin = GTK_SPIN_BUTTON(tool->xpix);
-    sel[0] = gwy_data_field_jtor(plain_tool->data_field, gtk_spin_button_get_value(spin) - 0.5);
+    sel[0] = gwy_field_jtor(plain_tool->field, gtk_spin_button_get_value(spin) - 0.5);
     spin = GTK_SPIN_BUTTON(tool->ypix);
-    sel[1] = gwy_data_field_itor(plain_tool->data_field, gtk_spin_button_get_value(spin) - 0.5);
+    sel[1] = gwy_field_itor(plain_tool->field, gtk_spin_button_get_value(spin) - 0.5);
     gwy_selection_set_object(plain_tool->selection, 0, sel);
 }
 
 static void
-update_label(GwySIValueFormat *units,
+update_label(GwyValueFormat *units,
              GtkWidget *label,
              gdouble value)
 {
@@ -844,7 +798,7 @@ update_values(GwyToolReadValue *tool)
     gdouble xoff, yoff;
     gint col, row;
 
-    if (plain_tool->data_field && plain_tool->selection)
+    if (plain_tool->field && plain_tool->selection)
         is_selected = gwy_selection_get_object(plain_tool->selection, 0, point);
 
     if (!is_selected) {
@@ -856,11 +810,11 @@ update_values(GwyToolReadValue *tool)
         return;
     }
 
-    xoff = gwy_data_field_get_xoffset(plain_tool->data_field);
-    yoff = gwy_data_field_get_yoffset(plain_tool->data_field);
+    xoff = gwy_field_get_xoffset(plain_tool->field);
+    yoff = gwy_field_get_yoffset(plain_tool->field);
 
-    col = floor(gwy_data_field_rtoj(plain_tool->data_field, point[0]));
-    row = floor(gwy_data_field_rtoi(plain_tool->data_field, point[1]));
+    col = floor(gwy_field_rtoj(plain_tool->field, point[0]));
+    row = floor(gwy_field_rtoi(plain_tool->field, point[1]));
 
     update_label(plain_tool->coord_format, tool->x, point[0] + xoff);
     update_label(plain_tool->coord_format, tool->y, point[1] + yoff);
@@ -888,28 +842,28 @@ calculate(GwyToolReadValue *tool, gint col, gint row)
 {
     gint radius = gwy_params_get_int(tool->params, PARAM_RADIUS);
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    GwyDataField *dfield = plain_tool->data_field;
+    GwyField *dfield = plain_tool->field;
     gint n, i;
     gdouble m[6], z[3];
 
     if (radius == 1) {
-        tool->avg = gwy_data_field_get_val(dfield, col, row);
-        tool->bx = gwy_data_field_get_xder(dfield, col, row);
-        tool->by = gwy_data_field_get_yder(dfield, col, row);
+        tool->avg = gwy_field_get_val(dfield, col, row);
+        tool->bx = gwy_field_get_xder(dfield, col, row);
+        tool->by = gwy_field_get_yder(dfield, col, row);
         tool->k1 = tool->k2 = 0.0;
         return;
     }
 
     /* Create the arrays the first time radius > 1 is requested */
     if (!tool->values) {
-        n = gwy_data_field_get_circular_area_size(RADIUS_MAX - 0.5);
+        n = gwy_field_get_circular_area_size(RADIUS_MAX - 0.5);
         tool->values = g_new(gdouble, n);
         tool->xpos = g_new(gint, n);
         tool->ypos = g_new(gint, n);
     }
 
-    n = gwy_data_field_circular_area_extract_with_pos(dfield, col, row, radius - 0.5, tool->values,
-                                                      tool->xpos, tool->ypos);
+    n = gwy_field_circular_area_extract_with_pos(dfield, col, row, radius - 0.5, tool->values,
+                                                 tool->xpos, tool->ypos);
     tool->avg = 0.0;
     if (!n) {
         tool->bx = tool->by = 0.0;
@@ -937,11 +891,11 @@ calculate(GwyToolReadValue *tool, gint col, gint row)
     gwy_math_choleski_solve(3, m, z);
     /* The signs may seem odd.  We have to invert y due to coordinate system and then invert both for downward slopes.
      * As a result x is inverted. */
-    tool->bx = -z[1]/gwy_data_field_get_dx(dfield);
-    tool->by = z[2]/gwy_data_field_get_dy(dfield);
+    tool->bx = -z[1]/gwy_field_get_dx(dfield);
+    tool->by = z[2]/gwy_field_get_dy(dfield);
 
     calc_curvatures(tool->values, tool->xpos, tool->ypos, n,
-                    gwy_data_field_get_dx(dfield), gwy_data_field_get_dy(dfield),
+                    gwy_field_get_dx(dfield), gwy_field_get_dy(dfield),
                     &tool->k1, &tool->k2);
 }
 
