@@ -1,5 +1,5 @@
 /*
- *  $Id: roughness.c 26745 2024-10-18 15:17:53Z yeti-dn $
+ *  $Id: roughness.c 29059 2026-01-02 14:27:52Z yeti-dn $
  *  Copyright (C) 2006-2022 Martin Hason, David Necas (Yeti), Petr Klapetek.
  *  E-mail: hasonm@physics.muni.cz, yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,21 +16,11 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyresults.h>
-#include <libprocess/datafield.h>
-#include <libprocess/linestats.h>
-#include <libprocess/inttrans.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwydgets/gwycombobox.h>
-#include <libgwydgets/gwydgetutils.h>
-#include <libgwymodule/gwymodule-tool.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
+#include <gwy.h>
 
 enum {
     NGROUPS = 4
@@ -92,16 +82,13 @@ struct _GwyToolRoughness {
     GtkTreeStore *store;
 
     gboolean have_data;
-    GwyDataLine *dataline;
-    GwyDataLine *datalines[NDATALINES];
+    GwyLine *line;
+    GwyLine *lines[NDATALINES];
 
     GwyGraphModel *gmodel;
     GwyGraphModel *gmodel_profile;
 
     GwyParamTable *table;
-
-    /* potential class data */
-    GType layer_type_line;
 };
 
 struct _GwyToolRoughnessClass {
@@ -130,20 +117,20 @@ static void          recalculate                         (GwyToolRoughness *tool
 static void          update_units                        (GwyToolRoughness *tool);
 static void          update_parameters                   (GwyToolRoughness *tool);
 static void          update_graphs                       (GwyToolRoughness *tool);
-static gint          gwy_data_line_extend                (GwyDataLine *dline,
-                                                          GwyDataLine *extline);
+static gint          extend_line                         (GwyLine *dline,
+                                                          GwyLine *extline);
 static void          set_data_from_profile               (GwyToolRoughness *tool);
-static gdouble       gwy_tool_roughness_Xz               (GwyDataLine *data_line);
-static gdouble       gwy_tool_roughness_Ry               (GwyDataLine *data_line);
-static gdouble       gwy_tool_roughness_Da               (GwyDataLine *data_line);
-static gdouble       gwy_tool_roughness_Sm               (GwyDataLine *dline);
-static gdouble       gwy_tool_roughness_l0               (GwyDataLine *data_line);
-static gdouble       gwy_tool_roughness_H                (GwyDataLine *data_line);
-static void          gwy_tool_roughness_distribution     (GwyDataLine *data_line,
-                                                          GwyDataLine *distr);
-static void          updata_dataline_adf                 (GwyToolRoughness *tool);
-static void          updata_dataline_brc                 (GwyToolRoughness *tool);
-static void          updata_dataline_pc                  (GwyToolRoughness *tool);
+static gdouble       gwy_tool_roughness_Xz               (GwyLine *line);
+static gdouble       gwy_tool_roughness_Ry               (GwyLine *line);
+static gdouble       gwy_tool_roughness_Da               (GwyLine *line);
+static gdouble       gwy_tool_roughness_Sm               (GwyLine *dline);
+static gdouble       gwy_tool_roughness_l0               (GwyLine *line);
+static gdouble       gwy_tool_roughness_H                (GwyLine *line);
+static void          gwy_tool_roughness_distribution     (GwyLine *line,
+                                                          GwyLine *distr);
+static void          upline_adf                          (GwyToolRoughness *tool);
+static void          upline_brc                          (GwyToolRoughness *tool);
+static void          upline_pc                           (GwyToolRoughness *tool);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
@@ -207,7 +194,7 @@ gwy_tool_roughness_class_init(GwyToolRoughnessClass *klass)
 
     gobject_class->finalize = gwy_tool_roughness_finalize;
 
-    tool_class->stock_id = GWY_STOCK_ISO_ROUGHNESS;
+    tool_class->icon_name = GWY_ICON_ISO_ROUGHNESS;
     tool_class->title = _("Roughness");
     tool_class->tooltip = _("Calculate roughness parameters");
     tool_class->prefix = "/module/roughness";
@@ -227,32 +214,27 @@ gwy_tool_roughness_finalize(GObject *object)
     guint i;
 
     gwy_params_save_to_settings(tool->params);
-    GWY_OBJECT_UNREF(tool->params);
-    GWY_OBJECT_UNREF(tool->store);
-    GWY_OBJECT_UNREF(tool->dataline);
+    g_clear_object(&tool->params);
+    g_clear_object(&tool->store);
+    g_clear_object(&tool->line);
     for (i = 0; i < NDATALINES; i++)
-        GWY_OBJECT_UNREF(tool->datalines[i]);
-    GWY_OBJECT_UNREF(tool->gmodel);
-    GWY_OBJECT_UNREF(tool->gmodel_profile);
+        g_clear_object(tool->lines + i);
+    g_clear_object(&tool->gmodel);
+    g_clear_object(&tool->gmodel_profile);
 
     G_OBJECT_CLASS(gwy_tool_roughness_parent_class)->finalize(object);
 
     /* XXX: Window size saving may invoke size request and bad things happen when we no longer have results. */
-    GWY_OBJECT_UNREF(tool->results);
+    g_clear_object(&tool->results);
 }
 
 static void
 gwy_tool_roughness_init(GwyToolRoughness *tool)
 {
-    GwyPlainTool *plain_tool;
+    GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
     guint i;
 
-    plain_tool = GWY_PLAIN_TOOL(tool);
-    tool->layer_type_line = gwy_plain_tool_check_layer_type(plain_tool, "GwyLayerLine");
-    if (!tool->layer_type_line)
-        return;
-
-    plain_tool->unit_style = GWY_SI_UNIT_FORMAT_VFMARKUP;
+    plain_tool->unit_style = GWY_UNIT_FORMAT_VFMARKUP;
     plain_tool->lazy_updates = TRUE;
 
     tool->params = gwy_params_new_from_settings(define_module_params());
@@ -260,14 +242,14 @@ gwy_tool_roughness_init(GwyToolRoughness *tool)
     tool->store = create_tree_store();
     for (i = 0; i < NDATALINES; i++) {
         if (i == DATALINE_ADF || i == DATALINE_BRC)
-            tool->datalines[i] = gwy_data_line_new(101, 1.0, FALSE);
+            tool->lines[i] = gwy_line_new(101, 1.0, FALSE);
         else if (i == DATALINE_PC)
-            tool->datalines[i] = gwy_data_line_new(121, 1.0, FALSE);
+            tool->lines[i] = gwy_line_new(121, 1.0, FALSE);
         else
-            tool->datalines[i] = gwy_data_line_new(1, 1.0, FALSE);
+            tool->lines[i] = gwy_line_new(1, 1.0, FALSE);
     }
 
-    gwy_plain_tool_connect_selection(plain_tool, tool->layer_type_line, "line");
+    gwy_plain_tool_connect_selection(plain_tool, GWY_TYPE_LAYER_LINE, "line");
 
     gwy_tool_roughness_init_dialog(tool);
 }
@@ -283,14 +265,14 @@ gwy_tool_roughness_init_dialog(GwyToolRoughness *tool)
     tool->gmodel = gwy_graph_model_new();
     tool->gmodel_profile = gwy_graph_model_new();
 
-    hbox = gwy_hbox_new(4);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
     gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
     gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(dialog)), hbox, TRUE, TRUE, 0);
 
-    vbox_left = gwy_vbox_new(4);
+    vbox_left = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
     gtk_box_pack_start(GTK_BOX(hbox), vbox_left, TRUE, TRUE, 0);
 
-    vbox_right = gwy_vbox_new(0);
+    vbox_right = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
     gtk_box_pack_start(GTK_BOX(hbox), vbox_right, TRUE, TRUE, 0);
 
     scwin = gtk_scrolled_window_new(NULL, NULL);
@@ -335,7 +317,7 @@ gwy_tool_roughness_init_dialog(GwyToolRoughness *tool)
 
     gwy_plain_tool_add_clear_button(GWY_PLAIN_TOOL(tool));
     gwy_tool_add_hide_button(GWY_TOOL(tool), FALSE);
-    gtk_dialog_add_button(dialog, GTK_STOCK_APPLY, GTK_RESPONSE_APPLY);
+    gwy_plain_tool_add_apply_button(GWY_PLAIN_TOOL(tool));
     gtk_dialog_set_default_response(dialog, GTK_RESPONSE_APPLY);
     gtk_dialog_set_response_sensitive(dialog, GTK_RESPONSE_APPLY, FALSE);
     gwy_param_table_set_sensitive(table, PARAM_REPORT_STYLE, FALSE);
@@ -461,7 +443,7 @@ create_results(void)
                           NULL);
 
     gwy_results_add_separator(results);
-    gwy_results_add_header(results, N_("parameters|Hybrid"));
+    gwy_results_add_header(results, gwy_NC("parameters", "Hybrid"));
     gwy_results_add_value(results, "Deltaa", N_("Average absolute slope"),
                           "power-z", 1, "power-x", -1, "symbol", "Δ<sub>a</sub>",
                           NULL);
@@ -479,7 +461,7 @@ create_results(void)
                           NULL);
 
     gwy_results_add_separator(results);
-    gwy_results_add_header(results, N_("parameters|Functional"));
+    gwy_results_add_header(results, gwy_NC("parameters", "Functional"));
     gwy_results_add_value(results, "H", N_("Swedish height"),
                           "power-z", 1, "symbol", "<i>H</i>",
                           NULL);
@@ -709,9 +691,8 @@ gwy_tool_roughness_data_switched(GwyTool *gwytool,
         return;
 
     if (data_view) {
-        gwy_object_set_or_reset(plain_tool->layer, tool->layer_type_line,
-                                "thickness", gwy_params_get_int(tool->params, PARAM_THICKNESS),
-                                "line-numbers", FALSE,
+        gwy_object_set_or_reset(plain_tool->layer, GWY_TYPE_LAYER_LINE,
+                                "thickness", (gdouble)gwy_params_get_int(tool->params, PARAM_THICKNESS),
                                 "editable", TRUE,
                                 "focus", -1,
                                 NULL);
@@ -767,7 +748,7 @@ param_changed(GwyToolRoughness *tool, gint id)
 
     if (id == PARAM_THICKNESS) {
         if (plain_tool->layer)
-            g_object_set(plain_tool->layer, "thickness", gwy_params_get_int(params, PARAM_THICKNESS), NULL);
+            g_object_set(plain_tool->layer, "thickness", (gdouble)gwy_params_get_int(params, PARAM_THICKNESS), NULL);
     }
     if (id == PARAM_GRAPH) {
         update_graphs(tool);
@@ -798,13 +779,14 @@ gwy_tool_roughness_apply(GwyToolRoughness *tool)
     gmodel = gwy_graph_model_new_alike(tool->gmodel);
     g_object_set(gmodel, "label-visible", TRUE, NULL);
     gcmodel = gwy_graph_model_get_curve(tool->gmodel, 0);
-    gcmodel = gwy_graph_curve_model_duplicate(gcmodel);
+    gcmodel = gwy_graph_curve_model_copy(gcmodel);
     gwy_graph_model_add_curve(gmodel, gcmodel);
     g_object_unref(gcmodel);
     g_object_get(gcmodel, "description", &s, NULL);
     g_object_set(gmodel, "title", s, NULL);
     g_free(s);
-    gwy_app_data_browser_add_graph_model(gmodel, plain_tool->container, TRUE);
+    gint newid = gwy_file_add_graph(GWY_FILE(plain_tool->container), gmodel);
+    gwy_file_set_visible(GWY_FILE(plain_tool->container), GWY_FILE_GRAPH, newid, TRUE);
     g_object_unref(gmodel);
 }
 
@@ -822,9 +804,9 @@ static void
 update_controls(GwyToolRoughness *tool, gboolean have_data)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    const gchar *message = (have_data || !plain_tool->data_field ? NULL : _("No profile in the image selected."));
+    const gchar *message = (have_data || !plain_tool->field ? NULL : _("No profile in the image selected."));
     gdouble cutoff = gwy_params_get_double(tool->params, PARAM_CUTOFF);
-    GwySIValueFormat *vf = plain_tool->coord_format;
+    GwyValueFormat *vf = plain_tool->coord_format;
     gdouble real;
     gint lineres;
     gchar buf[24];
@@ -837,8 +819,8 @@ update_controls(GwyToolRoughness *tool, gboolean have_data)
     gwy_param_table_set_label(tool->table, MESSAGE_MISSING, message);
 
     if (have_data) {
-        lineres = gwy_data_line_get_res(tool->dataline);
-        real = gwy_data_line_get_real(tool->dataline);
+        lineres = gwy_line_get_res(tool->line);
+        real = gwy_line_get_real(tool->line);
         if (cutoff > 0.0) {
             cutoff = 2.0*real/lineres/cutoff;
             g_snprintf(buf, sizeof(buf), "%.*f", vf->precision+1, cutoff/vf->magnitude);
@@ -861,7 +843,7 @@ void
 recalculate(GwyToolRoughness *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    GwyDataField *field = plain_tool->data_field;
+    GwyField *field = plain_tool->field;
     gint thickness = gwy_params_get_int(tool->params, PARAM_THICKNESS);
     GwyInterpolationType interpolation = gwy_params_get_enum(tool->params, PARAM_INTERPOLATION);
     GwyResults *results = tool->results;
@@ -878,10 +860,10 @@ recalculate(GwyToolRoughness *tool)
     g_return_if_fail(plain_tool->selection);
     g_return_if_fail(gwy_selection_get_object(plain_tool->selection, 0, line));
 
-    xl1 = floor(gwy_data_field_rtoj(field, line[0]));
-    yl1 = floor(gwy_data_field_rtoi(field, line[1]));
-    xl2 = floor(gwy_data_field_rtoj(field, line[2]));
-    yl2 = floor(gwy_data_field_rtoi(field, line[3]));
+    xl1 = floor(gwy_field_rtoj(field, line[0]));
+    yl1 = floor(gwy_field_rtoi(field, line[1]));
+    xl2 = floor(gwy_field_rtoj(field, line[2]));
+    yl2 = floor(gwy_field_rtoi(field, line[3]));
 
     lineres = GWY_ROUND(hypot(abs(xl1 - xl2) + 1, abs(yl1 - yl2) + 1));
     if (lineres < 8) {
@@ -891,18 +873,17 @@ recalculate(GwyToolRoughness *tool)
 
     plain_tool->pending_updates = 0;
     tool->have_data = TRUE;
-    xoff = gwy_data_field_get_xoffset(field);
-    yoff = gwy_data_field_get_yoffset(field);
+    xoff = gwy_field_get_xoffset(field);
+    yoff = gwy_field_get_yoffset(field);
     gwy_results_fill_format(results, "isel", "x1", xl1, "y1", yl1, "x2", xl2, "y2", yl2, NULL);
     gwy_results_fill_format(results, "realsel",
                             "x1", line[0] + xoff, "y1", line[1] + yoff,
                             "x2", line[2] + xoff, "y2", line[3] + yoff,
                             NULL);
-    tool->dataline = gwy_data_field_get_profile(field, tool->dataline, xl1, yl1, xl2, yl2,
-                                                lineres, thickness, interpolation);
+    tool->line = gwy_field_get_profile(field, tool->line, xl1, yl1, xl2, yl2, lineres, thickness, interpolation);
 
-    gwy_results_fill_filename(results, "file", plain_tool->container);
-    gwy_results_fill_channel(results, "image", plain_tool->container, plain_tool->id);
+    gwy_results_fill_filename(results, "file", GWY_FILE(plain_tool->container));
+    gwy_results_fill_data_name(results, "image", GWY_FILE(plain_tool->container), GWY_FILE_IMAGE, plain_tool->id);
 
     set_data_from_profile(tool);
 
@@ -915,62 +896,62 @@ static void
 update_units(GwyToolRoughness *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    GwyDataField *field = plain_tool->data_field;
-    GwySIUnit *siunitxy, *siunitz;
+    GwyField *field = plain_tool->field;
+    GwyUnit *siunitxy, *siunitz;
 
-    siunitxy = gwy_data_field_get_si_unit_xy(field);
-    siunitz = gwy_data_field_get_si_unit_z(field);
+    siunitxy = gwy_field_get_unit_xy(field);
+    siunitz = gwy_field_get_unit_z(field);
     gwy_results_set_unit(tool->results, "x", siunitxy);
     gwy_results_set_unit(tool->results, "y", siunitxy);
     gwy_results_set_unit(tool->results, "z", siunitz);
-    tool->same_units = gwy_si_unit_equal(siunitxy, siunitz);
+    tool->same_units = gwy_unit_equal(siunitxy, siunitz);
 
-    gwy_data_field_copy_units_to_data_line(field, tool->datalines[DATALINE_TEXTURE]);
-    gwy_data_field_copy_units_to_data_line(field, tool->datalines[DATALINE_WAVINESS]);
-    gwy_data_field_copy_units_to_data_line(field, tool->datalines[DATALINE_ROUGHNESS]);
+    gwy_field_copy_units_to_line(field, tool->lines[DATALINE_TEXTURE]);
+    gwy_field_copy_units_to_line(field, tool->lines[DATALINE_WAVINESS]);
+    gwy_field_copy_units_to_line(field, tool->lines[DATALINE_ROUGHNESS]);
     /* ADF and BRC are updated upon calculation */
 }
 
 static void
 update_parameters(GwyToolRoughness *tool)
 {
-    GwyDataLine *roughness = tool->datalines[DATALINE_ROUGHNESS];
-    GwyDataLine *waviness = tool->datalines[DATALINE_WAVINESS];
-    GwyDataLine *texture = tool->datalines[DATALINE_TEXTURE];
+    GwyLine *roughness = tool->lines[DATALINE_ROUGHNESS];
+    GwyLine *waviness = tool->lines[DATALINE_WAVINESS];
+    GwyLine *texture = tool->lines[DATALINE_TEXTURE];
     gdouble ra, rq, da, dq, rp, rv, rpm, rvm, rtm, l0, real;
 
     /* This definitely does something because we do not have form removed. Not sure; it affects only RP which seems to
      * be defined for zero-mean profiles. */
-    gwy_data_line_add(texture, -gwy_data_line_get_avg(texture));
+    gwy_line_add(texture, -gwy_line_avg(texture));
     /* This definitely does something because we do not have form removed and it should be also correct. */
-    gwy_data_line_add(waviness, -gwy_data_line_get_avg(waviness));
+    gwy_line_add(waviness, -gwy_line_avg(waviness));
     /* This should essentially do nothing but it is safe. */
-    gwy_data_line_add(roughness, -gwy_data_line_get_avg(roughness));
+    gwy_line_add(roughness, -gwy_line_avg(roughness));
 
-    ra = gwy_data_line_get_ra(roughness);
-    rq = gwy_data_line_get_rms(roughness);
-    rv = gwy_data_line_get_xvm(roughness, 1, 1);
-    rp = gwy_data_line_get_xpm(roughness, 1, 1);
-    rvm = gwy_data_line_get_xvm(roughness, 5, 1);
-    rpm = gwy_data_line_get_xpm(roughness, 5, 1);
+    ra = gwy_line_ra(roughness);
+    rq = gwy_line_rms(roughness);
+    rv = gwy_line_xvm(roughness, 1, 1);
+    rp = gwy_line_xpm(roughness, 1, 1);
+    rvm = gwy_line_xvm(roughness, 5, 1);
+    rpm = gwy_line_xpm(roughness, 5, 1);
     rtm = rvm + rpm;
     da = gwy_tool_roughness_Da(roughness);
-    dq = gwy_data_line_get_tan_beta0(roughness);
-    real = gwy_data_line_get_real(roughness);
+    dq = gwy_line_tan_beta0(roughness);
+    real = gwy_line_get_real(roughness);
     gwy_results_fill_values(tool->results,
                             "Ra", ra, "Rq", rq,
                             "Rv", rv, "Rp", rp, "Rt", rp + rv, "Rvm", rvm, "Rpm", rpm, "Rtm", rtm,
-                            "R3z", gwy_data_line_get_xtm(roughness, 1, 3),
-                            "R3zISO", gwy_data_line_get_xtm(roughness, 5, 3),
+                            "R3z", gwy_line_xtm(roughness, 1, 3),
+                            "R3zISO", gwy_line_xtm(roughness, 5, 3),
                             "Rz", gwy_tool_roughness_Xz(roughness),
                             "RzISO", rtm,
                             "Ry", gwy_tool_roughness_Ry(roughness),
-                            "Rsk", gwy_data_line_get_skew(roughness),
-                            "Rku", gwy_data_line_get_kurtosis(roughness) + 3.0,
-                            "Wa", gwy_data_line_get_ra(waviness),
-                            "Wq", gwy_data_line_get_rms(waviness),
-                            "Wy", gwy_data_line_get_xtm(waviness, 1, 1),
-                            "Pt", gwy_data_line_get_xtm(texture, 1, 1),
+                            "Rsk", gwy_line_skew(roughness),
+                            "Rku", gwy_line_kurtosis(roughness) + 3.0,
+                            "Wa", gwy_line_ra(waviness),
+                            "Wq", gwy_line_rms(waviness),
+                            "Wy", gwy_line_xtm(waviness, 1, 1),
+                            "Pt", gwy_line_xtm(texture, 1, 1),
                             "Deltaa", da, "Deltaq", dq,
                             "Sm", gwy_tool_roughness_Sm(roughness),
                             "lambdaa", 2*G_PI*ra/da, "lambdaq", 2*G_PI*rq/dq,
@@ -984,9 +965,9 @@ update_parameters(GwyToolRoughness *tool)
     else
         gwy_results_set_na(tool->results, "L0", "lr", NULL);
 
-    updata_dataline_adf(tool);
-    updata_dataline_brc(tool);
-    updata_dataline_pc(tool);
+    upline_adf(tool);
+    upline_brc(tool);
+    upline_pc(tool);
 }
 
 static void
@@ -1022,15 +1003,15 @@ update_graphs(GwyToolRoughness *tool)
             g_object_set(gcmodel,
                          "mode", GWY_GRAPH_CURVE_LINE,
                          "color", gwy_graph_get_preset_color(i),
-                         "description", gettext(graph_labels[profile_graphs[i]]),
+                         "description", _(graph_labels[profile_graphs[i]]),
                          NULL);
             gwy_graph_model_add_curve(gmodel, gcmodel);
             g_object_unref(gcmodel);
         }
-        gwy_graph_curve_model_set_data_from_dataline(gcmodel, tool->datalines[i], 0, 0);
+        gwy_graph_curve_model_set_data_from_line(gcmodel, tool->lines[i], 0, 0);
     }
     g_object_set(gmodel, "title", _("Surface Profiles"), NULL);
-    gwy_graph_model_set_units_from_data_line(gmodel, tool->dataline);
+    gwy_graph_model_set_units_from_line(gmodel, tool->line);
 
     gmodel = tool->gmodel;
     if (gwy_graph_model_get_n_curves(gmodel))
@@ -1046,27 +1027,26 @@ update_graphs(GwyToolRoughness *tool)
     }
     g_object_set(gcmodel, "description", graph_labels[graph_type], NULL);
     g_object_set(gmodel, "title", graph_labels[graph_type], NULL);
-    gwy_graph_model_set_units_from_data_line(gmodel, tool->datalines[graph_type]);
-    gwy_graph_curve_model_set_data_from_dataline(gcmodel, tool->datalines[graph_type], 0, 0);
+    gwy_graph_model_set_units_from_line(gmodel, tool->lines[graph_type]);
+    gwy_graph_curve_model_set_data_from_line(gcmodel, tool->lines[graph_type], 0, 0);
 }
 
 static gint
-gwy_data_line_extend(GwyDataLine *dline,
-                     GwyDataLine *extline)
+extend_line(GwyLine *dline, GwyLine *extline)
 {
     enum { SMEAR = 6 };
     gint n, next, k, i;
     gdouble *data, *edata;
     gdouble der0, der1;
 
-    n = gwy_data_line_get_res(dline);
+    n = gwy_line_get_res(dline);
     next = gwy_fft_find_nice_size(4*n/3);
     g_return_val_if_fail(next < 3*n, n);
 
-    gwy_data_line_resample(extline, next, GWY_INTERPOLATION_NONE);
-    gwy_data_line_set_real(extline, next*gwy_data_line_get_real(dline)/n);
-    data = gwy_data_line_get_data(dline);
-    edata = gwy_data_line_get_data(extline);
+    gwy_line_resize(extline, next);
+    gwy_line_set_real(extline, next*gwy_line_get_real(dline)/n);
+    data = gwy_line_get_data(dline);
+    edata = gwy_line_get_data(extline);
 
     gwy_assign(edata, data, n);
     /* 0 and 1 in extension data coordinates, not primary data */
@@ -1108,32 +1088,32 @@ gwy_data_line_extend(GwyDataLine *dline,
 static void
 set_data_from_profile(GwyToolRoughness *tool)
 {
-    GwyDataLine *extline = tool->datalines[DATALINE_EXTLINE];
-    GwyDataLine *iout = tool->datalines[DATALINE_IOUT];
-    GwyDataLine *rout = tool->datalines[DATALINE_ROUT];
-    GwyDataLine *rtmp = tool->datalines[DATALINE_RTMP];
-    GwyDataLine *itmp = tool->datalines[DATALINE_ITMP];
-    GwyDataLine *dline = tool->dataline;
+    GwyLine *extline = tool->lines[DATALINE_EXTLINE];
+    GwyLine *iout = tool->lines[DATALINE_IOUT];
+    GwyLine *rout = tool->lines[DATALINE_ROUT];
+    GwyLine *rtmp = tool->lines[DATALINE_RTMP];
+    GwyLine *itmp = tool->lines[DATALINE_ITMP];
+    GwyLine *dline = tool->line;
     gdouble cutoff = gwy_params_get_double(tool->params, PARAM_CUTOFF);
     gint next, i;
     gdouble *re, *im, *wdata, *rdata;
     const gdouble *tdata, *data;
-    gdouble real = gwy_data_line_get_real(dline);
-    gint n = gwy_data_line_get_res(dline);
+    gdouble real = gwy_line_get_real(dline);
+    gint n = gwy_line_get_res(dline);
 
-    gwy_data_line_assign(tool->datalines[DATALINE_TEXTURE], dline);
-    gwy_data_line_resample(tool->datalines[DATALINE_WAVINESS], n, GWY_INTERPOLATION_NONE);
-    gwy_data_line_set_real(tool->datalines[DATALINE_WAVINESS], real);
-    gwy_data_line_resample(tool->datalines[DATALINE_ROUGHNESS], n, GWY_INTERPOLATION_NONE);
-    gwy_data_line_set_real(tool->datalines[DATALINE_ROUGHNESS], real);
+    gwy_line_assign(tool->lines[DATALINE_TEXTURE], dline);
+    gwy_line_resize(tool->lines[DATALINE_WAVINESS], n);
+    gwy_line_set_real(tool->lines[DATALINE_WAVINESS], real);
+    gwy_line_resize(tool->lines[DATALINE_ROUGHNESS], n);
+    gwy_line_set_real(tool->lines[DATALINE_ROUGHNESS], real);
 
-    next = gwy_data_line_extend(dline, extline);
-    gwy_data_line_resample(rtmp, next, GWY_INTERPOLATION_NONE);
-    gwy_data_line_resample(itmp, next, GWY_INTERPOLATION_NONE);
-    gwy_data_line_fft_raw(extline, NULL, rout, iout, GWY_TRANSFORM_DIRECTION_FORWARD);
+    next = extend_line(dline, extline);
+    gwy_line_resize(rtmp, next);
+    gwy_line_resize(itmp, next);
+    gwy_line_fft_raw(extline, NULL, rout, iout, GWY_TRANSFORM_DIRECTION_FORWARD);
 
-    re = gwy_data_line_get_data(rout);
-    im = gwy_data_line_get_data(iout);
+    re = gwy_line_get_data(rout);
+    im = gwy_line_get_data(iout);
     for (i = 0; i < next; i++) {
         gdouble f = 2.0*MIN(i, next-i)/next;
 
@@ -1141,12 +1121,12 @@ set_data_from_profile(GwyToolRoughness *tool)
             re[i] = im[i] = 0.0;
     }
 
-    gwy_data_line_fft_raw(rout, iout, rtmp, itmp, GWY_TRANSFORM_DIRECTION_BACKWARD);
+    gwy_line_fft_raw(rout, iout, rtmp, itmp, GWY_TRANSFORM_DIRECTION_BACKWARD);
 
-    data = gwy_data_line_get_data_const(extline);
-    tdata = gwy_data_line_get_data_const(rtmp);
-    wdata = gwy_data_line_get_data(tool->datalines[DATALINE_WAVINESS]);
-    rdata = gwy_data_line_get_data(tool->datalines[DATALINE_ROUGHNESS]);
+    data = gwy_line_get_data_const(extline);
+    tdata = gwy_line_get_data_const(rtmp);
+    wdata = gwy_line_get_data(tool->lines[DATALINE_WAVINESS]);
+    rdata = gwy_line_get_data(tool->lines[DATALINE_ROUGHNESS]);
     for (i = 0; i < n; i++) {
         wdata[i] = tdata[i];
         rdata[i] = data[i] - tdata[i];
@@ -1154,24 +1134,24 @@ set_data_from_profile(GwyToolRoughness *tool)
 }
 
 static gdouble
-gwy_tool_roughness_Xz(GwyDataLine *data_line)
+gwy_tool_roughness_Xz(GwyLine *line)
 {
     gdouble p, v;
 
-    gwy_data_line_get_kth_peaks(data_line, 1, 5, TRUE, TRUE, 0.0, 0.0, &p);
-    gwy_data_line_get_kth_peaks(data_line, 1, 5, FALSE, TRUE, 0.0, 0.0, &v);
+    gwy_line_kth_peaks(line, 1, 5, TRUE, TRUE, 0.0, 0.0, &p);
+    gwy_line_kth_peaks(line, 1, 5, FALSE, TRUE, 0.0, 0.0, &v);
 
     return p + v;
 }
 
 static gdouble
-gwy_tool_roughness_Ry(GwyDataLine *data_line)
+gwy_tool_roughness_Ry(GwyLine *line)
 {
     gdouble p[5], v[5], Ry = 0.0;
     guint i;
 
-    gwy_data_line_get_kth_peaks(data_line, 5, 1, TRUE, FALSE, 0.0, 0.0, p);
-    gwy_data_line_get_kth_peaks(data_line, 5, 1, FALSE, FALSE, 0.0, 0.0, v);
+    gwy_line_kth_peaks(line, 5, 1, TRUE, FALSE, 0.0, 0.0, p);
+    gwy_line_kth_peaks(line, 5, 1, FALSE, FALSE, 0.0, 0.0, v);
 
     for (i = 0; i < 5; i++) {
         if (p[i] >= 0.0 && v[i] >= 0.0 && p[i] + v[i] > Ry)
@@ -1182,102 +1162,102 @@ gwy_tool_roughness_Ry(GwyDataLine *data_line)
 }
 
 static gdouble
-gwy_tool_roughness_Da(GwyDataLine *dline)
+gwy_tool_roughness_Da(GwyLine *dline)
 {
-    return gwy_data_line_get_variation(dline)/gwy_data_line_get_real(dline);
+    return gwy_line_variation(dline)/gwy_line_get_real(dline);
 }
 
 static gdouble
-gwy_tool_roughness_Sm(GwyDataLine *dline)
+gwy_tool_roughness_Sm(GwyLine *dline)
 {
-    gint count = gwy_data_line_count_peaks(dline, TRUE, 0.0, 0.0);
-    gdouble real = gwy_data_line_get_real(dline);
+    gint count = gwy_line_count_peaks(dline, TRUE, 0.0, 0.0);
+    gdouble real = gwy_line_get_real(dline);
     return real/count;
 }
 
 static gdouble
-gwy_tool_roughness_l0(GwyDataLine *data_line)
+gwy_tool_roughness_l0(GwyLine *line)
 {
     /* This might be not according to the srandard.  However, the original definitions can give lr < 1 for short lines
      * which is obviously wrong. It has to be corrected for the res vs. res-1 ratio somehow. */
-    return gwy_data_line_get_length(data_line);
+    return gwy_line_length(line);
 }
 
 static gdouble
-gwy_tool_roughness_H(GwyDataLine *dline)
+gwy_tool_roughness_H(GwyLine *dline)
 {
     /* The lower line exposes 90%, i.e. 10% remain below. The upper line exposes 5%, i.e. 95% remain below. */
     gdouble z[2], p[2] = { 10.0, 95.0 };
-    gint n = gwy_data_line_get_res(dline);
+    gint n = gwy_line_get_res(dline);
     gdouble *buf = g_new(gdouble, n);
 
-    gwy_assign(buf, gwy_data_line_get_data(dline), n);
-    gwy_math_percentiles(n, buf, GWY_PERCENTILE_INTERPOLATION_LINEAR, 2, p, z);
+    gwy_assign(buf, gwy_line_get_data(dline), n);
+    gwy_math_percentiles(buf, n, GWY_PERCENTILE_INTERPOLATION_LINEAR, p, 2, z);
     g_free(buf);
 
     return z[1] - z[0];
 }
 
 static void
-gwy_tool_roughness_distribution(GwyDataLine *data_line, GwyDataLine *distr)
+gwy_tool_roughness_distribution(GwyLine *line, GwyLine *distr)
 {
     gdouble max;
 
-    gwy_data_line_dh(data_line, distr, 0.0, 0.0, gwy_data_line_get_res(distr));
-    if (data_line->real == 0.0)
-        data_line->real = 1.0;
+    gwy_line_height_dist(line, distr, 0.0, 0.0, gwy_line_get_res(distr));
+    if (line->real == 0.0)
+        line->real = 1.0;
 
-    max = gwy_data_line_get_max(distr);
+    max = gwy_line_max(distr);
     if (max > 0.0)
-        gwy_data_line_multiply(distr, 1.0/max);
+        gwy_line_multiply(distr, 1.0/max);
 
-    gwy_si_unit_assign(gwy_data_line_get_si_unit_x(distr), gwy_data_line_get_si_unit_y(data_line));
+    gwy_unit_assign(gwy_line_get_unit_x(distr), gwy_line_get_unit_y(line));
 }
 
 static void
-updata_dataline_adf(GwyToolRoughness *tool)
+upline_adf(GwyToolRoughness *tool)
 {
-    GwyDataLine *roughness = tool->datalines[DATALINE_ROUGHNESS];
-    GwyDataLine *adf = tool->datalines[DATALINE_ADF];
+    GwyLine *roughness = tool->lines[DATALINE_ROUGHNESS];
+    GwyLine *adf = tool->lines[DATALINE_ADF];
 
     gwy_tool_roughness_distribution(roughness, adf);
 }
 
 static void
-updata_dataline_brc(GwyToolRoughness *tool)
+upline_brc(GwyToolRoughness *tool)
 {
-    GwyDataLine *roughness = tool->datalines[DATALINE_ROUGHNESS];
-    GwyDataLine *brc = tool->datalines[DATALINE_BRC];
+    GwyLine *roughness = tool->lines[DATALINE_ROUGHNESS];
+    GwyLine *brc = tool->lines[DATALINE_BRC];
     gdouble max;
 
     gwy_tool_roughness_distribution(roughness, brc);
-    gwy_data_line_cumulate(brc);
-    max = gwy_data_line_get_max(brc);
+    gwy_line_cumulate(brc, FALSE);
+    max = gwy_line_max(brc);
     if (max > 0.0)
-        gwy_data_line_multiply(brc, 1.0/max);
+        gwy_line_multiply(brc, 1.0/max);
 }
 
 static void
-updata_dataline_pc(GwyToolRoughness *tool)
+upline_pc(GwyToolRoughness *tool)
 {
-    GwyDataLine *roughness = tool->datalines[DATALINE_ROUGHNESS];
-    GwyDataLine *pc = tool->datalines[DATALINE_PC];
-    gint samples = gwy_data_line_get_res(pc);
+    GwyLine *roughness = tool->lines[DATALINE_ROUGHNESS];
+    GwyLine *pc = tool->lines[DATALINE_PC];
+    gint samples = gwy_line_get_res(pc);
     gdouble ymax, dy, threshold, real;
     gint i, peakcount;
 
-    ymax = gwy_data_line_get_max(roughness);
-    gwy_data_line_set_real(pc, ymax);
-    real = gwy_data_line_get_real(roughness);
+    ymax = gwy_line_max(roughness);
+    gwy_line_set_real(pc, ymax);
+    real = gwy_line_get_real(roughness);
     dy = ymax/samples;
 
-    gwy_si_unit_power(gwy_data_line_get_si_unit_y(roughness), 1, gwy_data_line_get_si_unit_x(pc));
-    gwy_si_unit_power(gwy_data_line_get_si_unit_x(roughness), -1, gwy_data_line_get_si_unit_y(pc));
+    gwy_unit_power(gwy_line_get_unit_y(roughness), 1, gwy_line_get_unit_x(pc));
+    gwy_unit_power(gwy_line_get_unit_x(roughness), -1, gwy_line_get_unit_y(pc));
 
     for (i = 0; i < samples; i++) {
         threshold = dy*i;
-        peakcount = gwy_data_line_count_peaks(roughness, TRUE, threshold, threshold);
-        gwy_data_line_set_val(pc, i, peakcount/real);
+        peakcount = gwy_line_count_peaks(roughness, TRUE, threshold, threshold);
+        gwy_line_set_val(pc, i, peakcount/real);
     }
 }
 
