@@ -1,5 +1,5 @@
 /*
- *  $Id: rprofile.c 26745 2024-10-18 15:17:53Z yeti-dn $
+ *  $Id: rprofile.c 28905 2025-11-24 15:51:18Z yeti-dn $
  *  Copyright (C) 2003-2022 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,20 +16,11 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwythreads.h>
-#include <libgwymodule/gwymodule-tool.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libprocess/datafield.h>
-#include <libprocess/stats.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwydgets/gwynullstore.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
-#include "libgwyddion/gwyomp.h"
+#include <gwy.h>
+#include "libgwyddion/omp.h"
 
 enum {
     RESPONSE_IMPROVE = 100,
@@ -70,16 +61,13 @@ struct _GwyToolRprofile {
     GtkTreeView *treeview;
     GtkTreeModel *model;
 
-    GwyDataLine *line;
+    GwyLine *line;
     GwyGraphModel *gmodel;
-    GdkPixbuf *colorpixbuf;
 
     GtkWidget *options;
     GwyParamTable *table;
 
-    /* potential class data */
-    GwySIValueFormat *pixel_format;
-    GType layer_type_line;
+    GwyValueFormat *pixel_format;
 };
 
 struct _GwyToolRprofileClass {
@@ -119,8 +107,8 @@ static void         render_color                       (GtkCellLayout *layout,
                                                         GtkTreeModel *model,
                                                         GtkTreeIter *iter,
                                                         gpointer user_data);
-static gdouble      angular_average_mismatch           (GwyDataField *data_field,
-                                                        GwyDataField *mask,
+static gdouble      angular_average_mismatch           (GwyField *field,
+                                                        GwyField *mask,
                                                         GwyMaskingType masking,
                                                         gdouble x,
                                                         gdouble y,
@@ -180,7 +168,7 @@ gwy_tool_rprofile_class_init(GwyToolRprofileClass *klass)
 
     gobject_class->finalize = gwy_tool_rprofile_finalize;
 
-    tool_class->stock_id = GWY_STOCK_RADIAL_PROFILE;
+    tool_class->icon_name = GWY_ICON_RADIAL_PROFILE;
     tool_class->title = _("Radial Profiles");
     tool_class->tooltip = _("Extract angularly averaged profiles");
     tool_class->prefix = "/module/rprofile";
@@ -199,15 +187,14 @@ gwy_tool_rprofile_finalize(GObject *object)
     GwyToolRprofile *tool = GWY_TOOL_RPROFILE(object);
 
     gwy_params_save_to_settings(tool->params);
-    GWY_OBJECT_UNREF(tool->params);
-    GWY_OBJECT_UNREF(tool->line);
+    g_clear_object(&tool->params);
+    g_clear_object(&tool->line);
     if (tool->model) {
         gtk_tree_view_set_model(tool->treeview, NULL);
-        GWY_OBJECT_UNREF(tool->model);
+        g_clear_object(&tool->model);
     }
-    GWY_OBJECT_UNREF(tool->colorpixbuf);
-    GWY_OBJECT_UNREF(tool->gmodel);
-    GWY_SI_VALUE_FORMAT_FREE(tool->pixel_format);
+    g_clear_object(&tool->gmodel);
+    GWY_FREE_VALUE_FORMAT(tool->pixel_format);
 
     G_OBJECT_CLASS(gwy_tool_rprofile_parent_class)->finalize(object);
 }
@@ -216,23 +203,14 @@ static void
 gwy_tool_rprofile_init(GwyToolRprofile *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    gint width, height;
 
-    tool->layer_type_line = gwy_plain_tool_check_layer_type(plain_tool, "GwyLayerLine");
-    if (!tool->layer_type_line)
-        return;
-
-    plain_tool->unit_style = GWY_SI_UNIT_FORMAT_MARKUP;
+    plain_tool->unit_style = GWY_UNIT_FORMAT_MARKUP;
     plain_tool->lazy_updates = TRUE;
 
     tool->params = gwy_params_new_from_settings(define_module_params());
-    tool->pixel_format = gwy_si_unit_value_format_new(1.0, 0, _("px"));
+    tool->pixel_format = gwy_value_format_new(1.0, 0, _("px"));
 
-    gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &width, &height);
-    height |= 1;
-    tool->colorpixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, 8, height, height);
-
-    gwy_plain_tool_connect_selection(plain_tool, tool->layer_type_line, "line");
+    gwy_plain_tool_connect_selection(plain_tool, GWY_TYPE_LAYER_LINE, "line");
     gwy_plain_tool_enable_selection_holding(plain_tool);
 
     gwy_tool_rprofile_init_dialog(tool);
@@ -256,12 +234,13 @@ gwy_tool_rprofile_init_dialog(GwyToolRprofile *tool)
     tool->gmodel = gwy_graph_model_new();
     g_object_set(tool->gmodel, "title", _("Radial profiles"), "label-visible", FALSE, NULL);
 
-    hbox = gwy_hbox_new(4);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
     gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(dialog)), hbox, TRUE, TRUE, 0);
 
     /* Left pane */
-    vbox = gwy_vbox_new(8);
-    gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 8);
+    gtk_widget_set_hexpand(vbox, FALSE);
+    gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, TRUE, 0);
 
     /* Line coordinates */
     store = gwy_null_store_new(0);
@@ -279,8 +258,7 @@ gwy_tool_rprofile_init_dialog(GwyToolRprofile *tool)
         gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(column), renderer, TRUE);
         gtk_cell_layout_set_cell_data_func(GTK_CELL_LAYOUT(column), renderer, render_cell, tool, NULL);
         if (i == COLUMN_I) {
-            renderer = gtk_cell_renderer_pixbuf_new();
-            g_object_set(renderer, "pixbuf", tool->colorpixbuf, NULL);
+            renderer = gwy_cell_renderer_color_new();
             gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(column), renderer, FALSE);
             gtk_cell_layout_set_cell_data_func(GTK_CELL_LAYOUT(column), renderer, render_color, tool, NULL);
         }
@@ -321,7 +299,7 @@ gwy_tool_rprofile_init_dialog(GwyToolRprofile *tool)
 
     gwy_plain_tool_add_clear_button(GWY_PLAIN_TOOL(tool));
     gwy_tool_add_hide_button(GWY_TOOL(tool), FALSE);
-    gtk_dialog_add_button(dialog, GTK_STOCK_APPLY, GTK_RESPONSE_APPLY);
+    gwy_plain_tool_add_apply_button(GWY_PLAIN_TOOL(tool));
     gtk_dialog_set_default_response(dialog, GTK_RESPONSE_APPLY);
     gtk_dialog_set_response_sensitive(dialog, GTK_RESPONSE_APPLY, FALSE);
     gwy_help_add_to_tool_dialog(dialog, GWY_TOOL(tool), GWY_HELP_DEFAULT);
@@ -347,9 +325,9 @@ gwy_tool_rprofile_data_switched(GwyTool *gwytool, GwyDataView *data_view)
         return;
 
     if (data_view) {
-        gwy_object_set_or_reset(plain_tool->layer, tool->layer_type_line,
-                                "line-numbers", gwy_params_get_boolean(tool->params, PARAM_NUMBER_LINES),
-                                "thickness", 1,
+        gwy_object_set_or_reset(plain_tool->layer, GWY_TYPE_LAYER_LINE,
+                                "numbered", gwy_params_get_boolean(tool->params, PARAM_NUMBER_LINES),
+                                "thickness", 1.0,
                                 "center-tick", TRUE,
                                 "editable", TRUE,
                                 "focus", -1,
@@ -436,10 +414,10 @@ param_changed(GwyToolRprofile *tool, gint id)
                           && id != PARAM_TARGET_GRAPH && id != PARAM_OPTIONS_VISIBLE);
 
     if (id == PARAM_MASKING)
-        do_update = do_update || (plain_tool->data_field && plain_tool->mask_field);
+        do_update = do_update || (plain_tool->field && plain_tool->mask_field);
     if (id < 0 || id == PARAM_NUMBER_LINES) {
         if (plain_tool->layer)
-            g_object_set(plain_tool->layer, "line-numbers", gwy_params_get_boolean(params, PARAM_NUMBER_LINES), NULL);
+            g_object_set(plain_tool->layer, "numbered", gwy_params_get_boolean(params, PARAM_NUMBER_LINES), NULL);
     }
     if (id < 0 || id == PARAM_SEPARATE) {
         gboolean separate = gwy_params_get_boolean(params, PARAM_SEPARATE);
@@ -471,16 +449,16 @@ update_symm_sensitivty(GwyToolRprofile *tool)
 static gint
 calculate_lineres(GwyToolRprofile *tool, const gdouble *line)
 {
-    GwyDataField *dfield = GWY_PLAIN_TOOL(tool)->data_field;
+    GwyField *dfield = GWY_PLAIN_TOOL(tool)->field;
     gint xl1, xl2, yl1, yl2, lineres;
 
     if (gwy_params_get_boolean(tool->params, PARAM_FIXRES))
         return gwy_params_get_int(tool->params, PARAM_RESOLUTION);
 
-    xl1 = floor(gwy_data_field_rtoj(dfield, line[0]));
-    yl1 = floor(gwy_data_field_rtoi(dfield, line[1]));
-    xl2 = floor(gwy_data_field_rtoj(dfield, line[2]));
-    yl2 = floor(gwy_data_field_rtoi(dfield, line[3]));
+    xl1 = floor(gwy_field_rtoj(dfield, line[0]));
+    yl1 = floor(gwy_field_rtoi(dfield, line[1]));
+    xl2 = floor(gwy_field_rtoj(dfield, line[2]));
+    yl2 = floor(gwy_field_rtoi(dfield, line[3]));
     lineres = GWY_ROUND(hypot(abs(xl1 - xl2) + 1, abs(yl1 - yl2) + 1));
     lineres = MAX(lineres, 4);
 
@@ -491,8 +469,8 @@ static void
 update_curve(GwyToolRprofile *tool, gint i)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    GwyDataField *data_field = plain_tool->data_field;
-    GwyDataField *mask = plain_tool->mask_field;
+    GwyField *field = plain_tool->field;
+    GwyField *mask = plain_tool->mask_field;
     GwyMaskingType masking = gwy_params_get_masking(tool->params, PARAM_MASKING, &mask);
     GwyGraphCurveModel *gcmodel;
     gdouble xc, yc, r, line[4];
@@ -505,20 +483,20 @@ update_curve(GwyToolRprofile *tool, gint i)
         g_return_if_reached();
     lineres = calculate_lineres(tool, line);
 
-    xc = 0.5*(line[0] + line[2]) + data_field->xoff;
-    yc = 0.5*(line[1] + line[3]) + data_field->yoff;
+    xc = 0.5*(line[0] + line[2]) + field->xoff;
+    yc = 0.5*(line[1] + line[3]) + field->yoff;
     r = 0.5*hypot(line[2] - line[0], line[3] - line[1]);
 
     /* Just create some line when there is none. */
     if (!tool->line)
-        tool->line = gwy_data_line_new(1, 1.0, FALSE);
-    r = fmax(r, hypot(gwy_data_field_get_dx(data_field), gwy_data_field_get_dy(data_field)));
-    gwy_data_field_angular_average(data_field, tool->line, mask, masking, xc, yc, r, lineres);
+        tool->line = gwy_line_new(1, 1.0, FALSE);
+    r = fmax(r, hypot(gwy_field_get_dx(field), gwy_field_get_dy(field)));
+    gwy_field_angular_average(field, mask, masking, tool->line, xc, yc, r, lineres);
 
     n = gwy_graph_model_get_n_curves(tool->gmodel);
     if (i < n) {
         gcmodel = gwy_graph_model_get_curve(tool->gmodel, i);
-        gwy_graph_curve_model_set_data_from_dataline(gcmodel, tool->line, 0, 0);
+        gwy_graph_curve_model_set_data_from_line(gcmodel, tool->line, 0, 0);
     }
     else {
         gcmodel = gwy_graph_curve_model_new();
@@ -530,12 +508,12 @@ update_curve(GwyToolRprofile *tool, gint i)
                      "color", color,
                      NULL);
         g_free(desc);
-        gwy_graph_curve_model_set_data_from_dataline(gcmodel, tool->line, 0, 0);
+        gwy_graph_curve_model_set_data_from_line(gcmodel, tool->line, 0, 0);
         gwy_graph_model_add_curve(tool->gmodel, gcmodel);
         g_object_unref(gcmodel);
 
         if (i == 0) {
-            gwy_graph_model_set_units_from_data_field(tool->gmodel, data_field, 1, 0, 0, 1);
+            gwy_graph_model_set_units_from_field(tool->gmodel, field, 1, 0, 0, 1);
             gwy_param_table_data_id_refilter(tool->table, PARAM_TARGET_GRAPH);
         }
     }
@@ -593,8 +571,8 @@ update_all_curves(GwyToolRprofile *tool)
 }
 
 static gdouble
-angular_average_mismatch(GwyDataField *data_field,
-                         GwyDataField *mask,
+angular_average_mismatch(GwyField *field,
+                         GwyField *mask,
                          GwyMaskingType masking,
                          gdouble x,
                          gdouble y,
@@ -606,21 +584,21 @@ angular_average_mismatch(GwyDataField *data_field,
     const gdouble *d, *m;
     gdouble *target, *weight, *sum2;
 
-    g_return_val_if_fail(GWY_IS_DATA_FIELD(data_field), G_MAXDOUBLE);
+    g_return_val_if_fail(GWY_IS_FIELD(field), G_MAXDOUBLE);
     g_return_val_if_fail(r >= 0.0, G_MAXDOUBLE);
-    xres = data_field->xres;
-    yres = data_field->yres;
+    xres = field->xres;
+    yres = field->yres;
 
     if (mask) {
-        g_return_val_if_fail(GWY_IS_DATA_FIELD(mask), G_MAXDOUBLE);
+        g_return_val_if_fail(GWY_IS_FIELD(mask), G_MAXDOUBLE);
         g_return_val_if_fail(mask->xres == xres, G_MAXDOUBLE);
         g_return_val_if_fail(mask->yres == yres, G_MAXDOUBLE);
     }
 
-    xreal = data_field->xreal;
-    yreal = data_field->yreal;
-    xoff = data_field->xoff;
-    yoff = data_field->yoff;
+    xreal = field->xreal;
+    yreal = field->yreal;
+    xoff = field->xoff;
+    yoff = field->yoff;
     g_return_val_if_fail(x >= xoff && x <= xoff + xreal, G_MAXDOUBLE);
     g_return_val_if_fail(y >= yoff && y <= yoff + yreal, G_MAXDOUBLE);
     /* Just for integer overflow; we limit i and j ranges explicitly later. */
@@ -639,21 +617,21 @@ angular_average_mismatch(GwyDataField *data_field,
     }
     h = r/nstats;
 
-    d = data_field->data;
-    m = mask ? mask->data : NULL;
+    d = gwy_field_get_data_const(field);
+    m = mask ? gwy_field_get_data_const(mask) : NULL;
 
     /* Just return something for single-point lines. */
     if (nstats < 2 || r == 0.0)
         return G_MAXDOUBLE;
 
-    ifrom = (gint)floor(gwy_data_field_rtoi(data_field, y - r));
+    ifrom = (gint)floor(gwy_field_rtoi(field, y - r));
     ifrom = MAX(ifrom, 0);
-    ito = (gint)ceil(gwy_data_field_rtoi(data_field, y + r));
+    ito = (gint)ceil(gwy_field_rtoi(field, y + r));
     ito = MIN(ito, yres-1);
 
-    jfrom = (gint)floor(gwy_data_field_rtoj(data_field, x - r));
+    jfrom = (gint)floor(gwy_field_rtoj(field, x - r));
     jfrom = MAX(jfrom, 0);
-    jto = (gint)ceil(gwy_data_field_rtoj(data_field, x + r));
+    jto = (gint)ceil(gwy_field_rtoj(field, x + r));
     jto = MIN(jto, xres-1);
 
     sum2 = g_new0(gdouble, 3*nstats);
@@ -710,8 +688,8 @@ angular_average_mismatch(GwyDataField *data_field,
 }
 
 static gdouble
-calculate_angular_mismatch(GwyDataField *dfield,
-                           GwyDataField *mask, GwyMaskingType masking,
+calculate_angular_mismatch(GwyField *dfield,
+                           GwyField *mask, GwyMaskingType masking,
                            const gdouble *line, gint lineres)
 {
     gdouble xc, yc, r;
@@ -724,8 +702,8 @@ calculate_angular_mismatch(GwyDataField *dfield,
 }
 
 static gboolean
-optimize_profile_at_scale(GwyDataField *dfield,
-                          GwyDataField *mask,
+optimize_profile_at_scale(GwyField *dfield,
+                          GwyField *mask,
                           GwyMaskingType masking,
                           gdouble r,
                           gdouble *line,
@@ -736,8 +714,8 @@ optimize_profile_at_scale(GwyDataField *dfield,
 
     gdouble xreal = dfield->xreal, yreal = dfield->yreal;
     gint ij, besti = 0, bestj = 0;
-    gdouble dx = gwy_data_field_get_dx(dfield);
-    gdouble dy = gwy_data_field_get_dy(dfield);
+    gdouble dx = gwy_field_get_dx(dfield);
+    gdouble dy = gwy_field_get_dy(dfield);
     gdouble allvar[GRID_NN];
     gdouble bestvar = G_MAXDOUBLE;
 
@@ -792,8 +770,8 @@ static void
 symmetrize_profile(GwyToolRprofile *tool, gint id)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    GwyDataField *dfield = GWY_PLAIN_TOOL(tool)->data_field;
-    GwyDataField *mask = plain_tool->mask_field;
+    GwyField *dfield = GWY_PLAIN_TOOL(tool)->field;
+    GwyField *mask = plain_tool->mask_field;
     GwyMaskingType masking = gwy_params_get_masking(tool->params, PARAM_MASKING, &mask);
     gdouble line_coarse[4], line_fine[4];
     gdouble r, dx, dy, h, mismatch_fine, mismatch_coarse;
@@ -803,8 +781,8 @@ symmetrize_profile(GwyToolRprofile *tool, gint id)
     if (!gwy_selection_get_object(plain_tool->selection, id, line_fine))
         g_return_if_reached();
     gwy_assign(line_coarse, line_fine, G_N_ELEMENTS(line_fine));
-    dx = gwy_data_field_get_dx(dfield);
-    dy = gwy_data_field_get_dy(dfield);
+    dx = gwy_field_get_dx(dfield);
+    dy = gwy_field_get_dy(dfield);
     lineres = calculate_lineres(tool, line_fine);
 
     /* Don't attempt to optimise very short lines. It would end up in tears. */
@@ -836,7 +814,7 @@ render_cell(GtkCellLayout *layout,
     GwyToolRprofile *tool = (GwyToolRprofile*)user_data;
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
     guint id = GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(layout), "id"));
-    const GwySIValueFormat *vf = tool->pixel_format;
+    const GwyValueFormat *vf = tool->pixel_format;
     gchar buf[32];
     gdouble line[4];
     gdouble val;
@@ -853,13 +831,13 @@ render_cell(GtkCellLayout *layout,
     gwy_selection_get_object(plain_tool->selection, idx, line);
 
     if (id == COLUMN_X1)
-        val = floor(gwy_data_field_rtoj(plain_tool->data_field, line[0]));
+        val = floor(gwy_field_rtoj(plain_tool->field, line[0]));
     else if (id == COLUMN_Y1)
-        val = floor(gwy_data_field_rtoi(plain_tool->data_field, line[1]));
+        val = floor(gwy_field_rtoi(plain_tool->field, line[1]));
     else if (id == COLUMN_X2)
-        val = floor(gwy_data_field_rtoj(plain_tool->data_field, line[2]));
+        val = floor(gwy_field_rtoj(plain_tool->field, line[2]));
     else if (id == COLUMN_Y2)
-        val = floor(gwy_data_field_rtoi(plain_tool->data_field, line[3]));
+        val = floor(gwy_field_rtoi(plain_tool->field, line[3]));
     else {
         g_return_if_reached();
     }
@@ -874,7 +852,7 @@ render_cell(GtkCellLayout *layout,
 
 static void
 render_color(G_GNUC_UNUSED GtkCellLayout *layout,
-             G_GNUC_UNUSED GtkCellRenderer *renderer,
+             GtkCellRenderer *renderer,
              GtkTreeModel *model,
              GtkTreeIter *iter,
              gpointer user_data)
@@ -882,14 +860,13 @@ render_color(G_GNUC_UNUSED GtkCellLayout *layout,
     GwyToolRprofile *tool = (GwyToolRprofile*)user_data;
     GwyGraphCurveModel *gcmodel;
     GwyRGBA *rgba;
-    guint idx, pixel;
+    guint idx;
 
     gtk_tree_model_get(model, iter, 0, &idx, -1);
     gcmodel = gwy_graph_model_get_curve(tool->gmodel, idx);
     g_object_get(gcmodel, "color", &rgba, NULL);
-    pixel = 0xff | gwy_rgba_to_pixbuf_pixel(rgba);
+    g_object_set(renderer, "color", rgba, NULL);
     gwy_rgba_free(rgba);
-    gdk_pixbuf_fill(tool->colorpixbuf, pixel);
 }
 
 static void
@@ -900,10 +877,9 @@ gwy_tool_rprofile_apply(GwyToolRprofile *tool)
     GwyGraphCurveModel *gcmodel;
     GwyGraphModel *gmodel;
     gchar *s;
-    gint i, n;
 
     g_return_if_fail(plain_tool->selection);
-    n = gwy_selection_get_data(plain_tool->selection, NULL);
+    gint n = gwy_selection_get_data(plain_tool->selection, NULL);
     g_return_if_fail(n);
 
     if ((gmodel = gwy_params_get_graph(tool->params, PARAM_TARGET_GRAPH))) {
@@ -912,25 +888,27 @@ gwy_tool_rprofile_apply(GwyToolRprofile *tool)
     }
 
     if (!separate) {
-        gmodel = gwy_graph_model_duplicate(tool->gmodel);
+        gmodel = gwy_graph_model_copy(tool->gmodel);
         g_object_set(gmodel, "label-visible", TRUE, NULL);
-        gwy_app_data_browser_add_graph_model(gmodel, plain_tool->container, TRUE);
+        gint newid = gwy_file_add_graph(GWY_FILE(plain_tool->container), gmodel);
+        gwy_file_set_visible(GWY_FILE(plain_tool->container), GWY_FILE_GRAPH, newid, TRUE);
         g_object_unref(gmodel);
         return;
     }
 
-    for (i = 0; i < n; i++) {
+    for (gint i = 0; i < n; i++) {
         gmodel = gwy_graph_model_new_alike(tool->gmodel);
         g_object_set(gmodel, "label-visible", TRUE, NULL);
         gcmodel = gwy_graph_model_get_curve(tool->gmodel, i);
-        gcmodel = gwy_graph_curve_model_duplicate(gcmodel);
+        gcmodel = gwy_graph_curve_model_copy(gcmodel);
 
         gwy_graph_model_add_curve(gmodel, gcmodel);
         g_object_unref(gcmodel);
         g_object_get(gcmodel, "description", &s, NULL);
         g_object_set(gmodel, "title", s, NULL);
         g_free(s);
-        gwy_app_data_browser_add_graph_model(gmodel, plain_tool->container, TRUE);
+        gint newid = gwy_file_add_graph(GWY_FILE(plain_tool->container), gmodel);
+        gwy_file_set_visible(GWY_FILE(plain_tool->container), GWY_FILE_GRAPH, newid, TRUE);
         g_object_unref(gmodel);
     }
 }
