@@ -1,6 +1,6 @@
 /*
- *  $Id: selectionmanager.c 26745 2024-10-18 15:17:53Z yeti-dn $
- *  Copyright (C) 2009-2022 David Necas (Yeti).
+ *  $Id: selectionmanager.c 29097 2026-01-07 13:50:22Z yeti-dn $
+ *  Copyright (C) 2009-2026 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -14,36 +14,22 @@
  *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
  *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
-
+#define DEBUG 1
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwymodule/gwymodule-tool.h>
-#include <libgwydgets/gwystock.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
+
+#include "libgwyapp/sanity.h"
 
 /* The GtkTargetEntry for tree model drags.
  * FIXME: Is it Gtk+ private or what? */
-#define GTK_TREE_MODEL_ROW \
-    { "GTK_TREE_MODEL_ROW", GTK_TARGET_SAME_APP, 0 }
-
-/* XXX: Copied from data browser. Kind of internal. */
-#define page_id_key "gwy-app-data-browser-page-id"
-#define container_key "gwy-app-data-browser-container"
+#define GTK_TREE_MODEL_ROW { "GTK_TREE_MODEL_ROW", GTK_TARGET_SAME_APP, 0 }
 
 enum {
-    PAGENO_SHIFT = 16,
-};
-
-enum {
-    NLAYERTYPES = 9
-};
-
-enum {
-    COLUMN_ID, COLUMN_OBJECT, NCOLUMNS
+    COLUMN_KEY, COLUMN_OBJECT, NCOLUMNS
 };
 
 enum {
@@ -53,7 +39,7 @@ enum {
 typedef struct {
     GwySelection *selection;
     const gchar *name;
-    GwySIUnit *xyunit;
+    GwyUnit *xyunit;
     gdouble origin[2];
 } DistributeData;
 
@@ -71,6 +57,12 @@ struct _GwyToolSelectionManager {
     GwyParams *params;
     gboolean in_setup;
 
+    /* Have our own file container pointer to use gwy_set_member_object(). */
+    GwyFile *file;
+    const gchar *prefix;
+    gulong item_changed_id;
+    gboolean doing_mass_change;
+
     GtkListStore *model;
     GtkWidget *treeview;
     GtkWidget *allfiles;
@@ -78,66 +70,78 @@ struct _GwyToolSelectionManager {
     GtkWidget *copy;
     GtkWidget *export;
     GtkWidget *delete;
-
-    /* potential class data */
-    GType layer_types[NLAYERTYPES];
-    GType selection_types[NLAYERTYPES];
 };
 
 struct _GwyToolSelectionManagerClass {
     GwyPlainToolClass parent_class;
 };
 
-static gboolean     module_register                             (void);
-static GwyParamDef* define_module_params                        (void);
-static GType        gwy_tool_selection_manager_get_type         (void)                           G_GNUC_CONST;
-static void         gwy_tool_selection_manager_finalize         (GObject *object);
-static void         gwy_tool_selection_manager_init_dialog      (GwyToolSelectionManager *tool);
-static void         gwy_tool_selection_manager_data_switched    (GwyTool *gwytool,
-                                                                 GwyDataView *data_view);
-static void         gwy_tool_selection_manager_response         (GwyTool *gwytool,
-                                                                 gint response_id);
-static void         clear_selection                             (GwyToolSelectionManager *tool);
-static void         gwy_tool_selection_manager_selection_changed(GwyToolSelectionManager *tool,
-                                                                 GtkTreeSelection *selection);
-static void         gwy_tool_selection_manager_all_files_changed(GwyToolSelectionManager *tool,
-                                                                 GtkToggleButton *button);
-static gboolean     key_pressed                                 (GwyToolSelectionManager *tool,
-                                                                 GdkEventKey *event,
-                                                                 GtkTreeView *treeview);
-static void         distribute                                  (GwyToolSelectionManager *tool);
-static void         distribute_one                              (GwyContainer *container,
-                                                                 DistributeData *distdata);
-static void         copy_selection_to_clipboard                 (GwyToolSelectionManager *tool);
-static void         export_selection                            (GwyToolSelectionManager *tool);
-static void         delete_selection                            (GwyToolSelectionManager *tool);
-static gchar*       create_report                               (GwyToolSelectionManager *tool);
-static void         setup_layer                                 (GwyToolSelectionManager *tool,
-                                                                 guint current,
-                                                                 GQuark quark);
+static GType gwy_tool_selection_manager_get_type(void) G_GNUC_CONST;
+
+static gboolean     module_register            (void);
+static GwyParamDef* define_module_params       (void);
+static void         finalize                   (GObject *object);
+static void         init_dialog                (GwyToolSelectionManager *tool);
+static void         data_switched              (GwyTool *gwytool,
+                                                GwyDataView *data_view);
+static void         file_item_changed          (GwyToolSelectionManager *tool,
+                                                GQuark key,
+                                                GwyContainer *data);
+static void         update_selection_item      (GwyToolSelectionManager *tool,
+                                                GQuark quark,
+                                                gboolean addit);
+static void         tool_response              (GwyTool *gwytool,
+                                                gint response_id);
+static void         clear_selection            (GwyToolSelectionManager *tool);
+static void         list_selection_changed     (GwyToolSelectionManager *tool,
+                                                GtkTreeSelection *selection);
+static void         all_files_toggled          (GwyToolSelectionManager *tool,
+                                                GtkToggleButton *button);
+static gboolean     key_pressed                (GwyToolSelectionManager *tool,
+                                                GdkEventKey *event,
+                                                GtkTreeView *treeview);
+static void         distribute                 (GwyToolSelectionManager *tool);
+static void         distribute_one             (GwyFile *file,
+                                                gpointer user_data);
+static void         copy_selection_to_clipboard(GwyToolSelectionManager *tool);
+static void         export_selection           (GwyToolSelectionManager *tool);
+static void         delete_selection           (GwyToolSelectionManager *tool);
+static gchar*       create_report              (GwyToolSelectionManager *tool);
+static void         setup_layer                (GwyToolSelectionManager *tool,
+                                                guint current,
+                                                GQuark quark);
+
+/* NB: This is not 1:1 mapping because LayerCross also uses SelectionPoint. We might also want to add some knowledge
+ * of selection names in the file here. We need to use get-type functions because the types might not have been
+ * registered yet and type macros cannot be used to initialise static data. */
+static struct {
+    const gchar *humanname;
+    GType layer_type;
+    GType selection_type;
+    GType (*get_type)(void);
+}
+selection_info[] = {
+    { N_("Axis lines"),     0, 0, gwy_layer_axis_get_type,      },
+    { N_("Cross lines"),    0, 0, gwy_layer_cross_get_type,     },
+    { N_("Ellipses"),       0, 0, gwy_layer_ellipse_get_type,   },
+    { N_("Lattice"),        0, 0, gwy_layer_lattice_get_type,   },
+    { N_("Lines"),          0, 0, gwy_layer_line_get_type,      },
+    { N_("Points"),         0, 0, gwy_layer_point_get_type,     },
+    { N_("Quadrilaterals"), 0, 0, gwy_layer_quad_get_type,      },
+    { N_("Rectangles"),     0, 0, gwy_layer_rectangle_get_type, },
+    { N_("Spline path"),    0, 0, gwy_layer_path_get_type,      },
+};
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Selection manager tool which lists, clears and copies selections."),
     "Yeti <yeti@gwyddion.net>",
-    "3.1",
+    "4.0",
     "David Nečas (Yeti)",
     "2009",
 };
 
-static const gchar *const layer_types[NLAYERTYPES] = {
-    "GwyLayerAxis",
-    "GwyLayerCross",
-    "GwyLayerEllipse",
-    "GwyLayerLattice",
-    "GwyLayerLine",
-    "GwyLayerPath",
-    "GwyLayerPoint",
-    "GwyLayerProjective",
-    "GwyLayerRectangle",
-};
-
 GWY_MODULE_QUERY2(module_info, selectionmanager)
 
 G_DEFINE_TYPE(GwyToolSelectionManager, gwy_tool_selection_manager, GWY_TYPE_PLAIN_TOOL)
@@ -145,6 +149,16 @@ G_DEFINE_TYPE(GwyToolSelectionManager, gwy_tool_selection_manager, GWY_TYPE_PLAI
 static gboolean
 module_register(void)
 {
+    for (guint i = 0; i < G_N_ELEMENTS(selection_info); i++) {
+        GType type = selection_info[i].get_type();
+        g_assert(g_type_is_a(type, GWY_TYPE_VECTOR_LAYER));
+
+        selection_info[i].layer_type = type;
+        GTypeClass *klass = g_type_class_ref(type);
+        selection_info[i].selection_type = gwy_vector_layer_class_get_selection_type(GWY_VECTOR_LAYER_CLASS(klass));
+        g_type_class_unref(klass);
+    }
+
     gwy_tool_func_register(GWY_TYPE_TOOL_SELECTION_MANAGER);
 
     return TRUE;
@@ -171,25 +185,27 @@ gwy_tool_selection_manager_class_init(GwyToolSelectionManagerClass *klass)
     GwyToolClass *tool_class = GWY_TOOL_CLASS(klass);
     GObjectClass *gobject_class = G_OBJECT_CLASS(klass);
 
-    gobject_class->finalize = gwy_tool_selection_manager_finalize;
+    gobject_class->finalize = finalize;
 
-    tool_class->stock_id = GWY_STOCK_SELECTIONS;
+    tool_class->icon_name = GWY_ICON_SELECTIONS;
     tool_class->title = _("Selection Manager");
     tool_class->tooltip = _("Display, copy and export selections");
     tool_class->prefix = "/module/selectionmanager";
     tool_class->default_height = 280;
-    tool_class->data_switched = gwy_tool_selection_manager_data_switched;
-    tool_class->response = gwy_tool_selection_manager_response;
+    tool_class->data_switched = data_switched;
+    tool_class->response = tool_response;
 }
 
 static void
-gwy_tool_selection_manager_finalize(GObject *object)
+finalize(GObject *object)
 {
     GwyToolSelectionManager *tool = GWY_TOOL_SELECTION_MANAGER(object);
+    GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
 
+    g_clear_signal_handler(&tool->item_changed_id, plain_tool->container);
     gwy_params_save_to_settings(tool->params);
-    GWY_OBJECT_UNREF(tool->params);
-    GWY_OBJECT_UNREF(tool->model);
+    g_clear_object(&tool->params);
+    g_clear_object(&tool->model);
 
     G_OBJECT_CLASS(gwy_tool_selection_manager_parent_class)->finalize(object);
 }
@@ -197,33 +213,19 @@ gwy_tool_selection_manager_finalize(GObject *object)
 static void
 gwy_tool_selection_manager_init(GwyToolSelectionManager *tool)
 {
-    GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
     GtkTreeSelection *selection;
-    guint i;
 
-    for (i = 0; i < NLAYERTYPES; i++) {
-        GwyVectorLayerClass *klass;
-        GType type;
-
-        if (!(type = gwy_plain_tool_check_layer_type(plain_tool, layer_types[i])))
-            return;
-
-        tool->layer_types[i] = type;
-        klass = g_type_class_ref(type);
-        tool->selection_types[i] = gwy_vector_layer_class_get_selection_type(klass);
-        g_type_class_unref(klass);
-    }
     tool->in_setup = TRUE;
     tool->params = gwy_params_new_from_settings(define_module_params());
 
     tool->model = gtk_list_store_new(NCOLUMNS, G_TYPE_INT, G_TYPE_OBJECT);
-    g_object_set_data(G_OBJECT(tool->model), page_id_key, GUINT_TO_POINTER(GWY_PAGE_NOPAGE + PAGENO_SHIFT));
+    gwy_app_set_tree_model_kind(GTK_TREE_MODEL(tool->model), "image-selections");
 
-    gwy_tool_selection_manager_init_dialog(tool);
+    init_dialog(tool);
     tool->in_setup = FALSE;
 
     selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tool->treeview));
-    gwy_tool_selection_manager_selection_changed(tool, selection);
+    list_selection_changed(tool, selection);
 }
 
 static void
@@ -236,7 +238,7 @@ render_name(G_GNUC_UNUSED GtkTreeViewColumn *column,
     GQuark quark;
     const gchar *s;
 
-    gtk_tree_model_get(model, iter, COLUMN_ID, &quark, -1);
+    gtk_tree_model_get(model, iter, COLUMN_KEY, &quark, -1);
     s = g_quark_to_string(quark);
     g_return_if_fail(s);
     s = strrchr(s, GWY_CONTAINER_PATHSEP);
@@ -251,32 +253,15 @@ render_type(G_GNUC_UNUSED GtkTreeViewColumn *column,
             GtkTreeIter *iter,
             G_GNUC_UNUSED gpointer user_data)
 {
-    static const struct {
-        const gchar *typename;
-        const gchar *humanname;
-    }
-    type_names[] = {
-        { "GwySelectionAxis",       N_("Horiz./vert. lines"),    },
-        { "GwySelectionCross",      N_("Crosses"),               },
-        { "GwySelectionEllipse",    N_("Ellipses"),              },
-        { "GwySelectionLattice",    N_("Lattice"),               },
-        { "GwySelectionLine",       N_("Lines"),                 },
-        { "GwySelectionPath",       N_("Spline path"),           },
-        { "GwySelectionPoint",      N_("Points"),                },
-        { "GwySelectionProjective", N_("Projective rectangles"), },
-        { "GwySelectionRectangle",  N_("Rectangles"),            },
-    };
-
     GwySelection *sel;
-    const gchar *name;
-    guint i;
 
     gtk_tree_model_get(model, iter, COLUMN_OBJECT, &sel, -1);
     g_return_if_fail(GWY_IS_SELECTION(sel));
-    name = G_OBJECT_TYPE_NAME(sel);
-    for (i = 0; i < G_N_ELEMENTS(type_names); i++) {
-        if (gwy_strequal(name, type_names[i].typename)) {
-            name = _(type_names[i].humanname);
+    GType type = G_OBJECT_TYPE(sel);
+    const gchar *name = g_type_name(type);
+    for (guint i = 0; i < G_N_ELEMENTS(selection_info); i++) {
+        if (type == selection_info[i].selection_type) {
+            name = _(selection_info[i].humanname);
             break;
         }
     }
@@ -302,7 +287,7 @@ render_objects(G_GNUC_UNUSED GtkTreeViewColumn *column,
 }
 
 static void
-gwy_tool_selection_manager_init_dialog(GwyToolSelectionManager *tool)
+init_dialog(GwyToolSelectionManager *tool)
 {
     static const GtkTargetEntry dnd_target_table[] = { GTK_TREE_MODEL_ROW };
 
@@ -347,11 +332,11 @@ gwy_tool_selection_manager_init_dialog(GwyToolSelectionManager *tool)
     gtk_tree_view_column_set_cell_data_func(column, renderer, render_objects, tool, NULL);
 
     label = gtk_label_new(_("Manage chosen selection"));
-    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
-    gtk_misc_set_padding(GTK_MISC(label), 0, 2);
+    gtk_label_set_xalign(GTK_LABEL(label), 0.0);
+    gwy_set_widget_padding(label, 0, 0, 2, 2);
     gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(dialog)), label, FALSE, FALSE, 0);
 
-    hbox = gwy_hbox_new(6);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 6);
     gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(dialog)), hbox, FALSE, FALSE, 2);
 
     tool->distribute = gtk_button_new_with_mnemonic(_("_Distribute"));
@@ -362,67 +347,45 @@ gwy_tool_selection_manager_init_dialog(GwyToolSelectionManager *tool)
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tool->allfiles),
                                  gwy_params_get_boolean(tool->params, PARAM_ALL_FILES));
 
-    hbox = gwy_hbox_new(6);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 6);
     gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(dialog)), hbox, FALSE, FALSE, 2);
 
-    tool->copy = gwy_stock_like_button_new(_("_Copy"), GTK_STOCK_COPY);
+    tool->copy = gwy_create_stock_button(GWY_STOCK_COPY, GWY_ICON_GTK_COPY);
     gtk_box_pack_start(GTK_BOX(hbox), tool->copy, FALSE, FALSE, 0);
 
-    tool->export = gwy_stock_like_button_new(_("_Export"), GTK_STOCK_SAVE);
+    tool->export = gwy_create_stock_button(GWY_STOCK_EXPORT, GWY_ICON_GTK_SAVE);
     gtk_box_pack_start(GTK_BOX(hbox), tool->export, FALSE, FALSE, 0);
 
-    tool->delete = gwy_stock_like_button_new(_("_Delete"), GTK_STOCK_DELETE);
+    tool->delete = gwy_create_stock_button(GWY_STOCK_DELETE, GWY_ICON_GTK_DELETE);
     gtk_box_pack_start(GTK_BOX(hbox), tool->delete, FALSE, FALSE, 0);
 
     gwy_tool_add_hide_button(GWY_TOOL(tool), TRUE);
-    gtk_dialog_add_button(GTK_DIALOG(GWY_TOOL(tool)->dialog), GTK_STOCK_CLEAR, GWY_TOOL_RESPONSE_CLEAR);
+    gwy_add_button_to_dialog(GTK_DIALOG(GWY_TOOL(tool)->dialog),
+                             GWY_STOCK_CLEAR, GWY_ICON_GTK_CLEAR, GWY_RESPONSE_CLEAR);
     gwy_help_add_to_tool_dialog(dialog, GWY_TOOL(tool), GWY_HELP_DEFAULT);
 
     g_signal_connect_swapped(tool->distribute, "clicked", G_CALLBACK(distribute), tool);
     g_signal_connect_swapped(tool->delete, "clicked", G_CALLBACK(delete_selection), tool);
     g_signal_connect_swapped(tool->copy, "clicked", G_CALLBACK(copy_selection_to_clipboard), tool);
-    g_signal_connect_swapped(tool->allfiles, "toggled", G_CALLBACK(gwy_tool_selection_manager_all_files_changed), tool);
+    g_signal_connect_swapped(tool->allfiles, "toggled", G_CALLBACK(all_files_toggled), tool);
     g_signal_connect_swapped(tool->export, "clicked", G_CALLBACK(export_selection), tool);
     g_signal_connect_swapped(tool->treeview, "key-press-event", G_CALLBACK(key_pressed), tool);
-    g_signal_connect(tool->treeview, "drag-begin", G_CALLBACK(gwy_app_data_browser_block_switching), NULL);
-    g_signal_connect(tool->treeview, "drag-end", G_CALLBACK(gwy_app_data_browser_unblock_switching), NULL);
-    g_signal_connect_swapped(selection, "changed", G_CALLBACK(gwy_tool_selection_manager_selection_changed), tool);
+    g_signal_connect(tool->treeview, "drag-begin", G_CALLBACK(gwy_data_browser_block_switching), NULL);
+    g_signal_connect(tool->treeview, "drag-end", G_CALLBACK(gwy_data_browser_unblock_switching), NULL);
+    g_signal_connect_swapped(selection, "changed", G_CALLBACK(list_selection_changed), tool);
 
     gtk_widget_show_all(gtk_dialog_get_content_area(dialog));
 }
 
 static void
-add_selection(gpointer hkey, gpointer hvalue, gpointer data)
+add_selection(GQuark quark, G_GNUC_UNUSED GValue *value, gpointer user_data)
 {
-    GwyToolSelectionManager *tool = (GwyToolSelectionManager*)data;
-    GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    GQuark quark = GPOINTER_TO_UINT(hkey);
-    GValue *value = (GValue*)hvalue;
-    GwySelection *sel = g_value_get_object(value);
-    GtkTreeIter iter;
-    const gchar *name;
-
-    name = g_quark_to_string(quark);
-    g_return_if_fail(*name == '/');
-    name++;
-    while (g_ascii_isdigit(*name))
-        name++;
-    g_return_if_fail(strncmp(name, "/select/", sizeof("/select/")-1) == 0);
-    name += sizeof("/select/")-1;
-    /* Ignore selections with names starting with an underscore. */
-    if (*name == '_')
-        return;
-
-    g_object_set_data(G_OBJECT(sel), container_key, plain_tool->container);
-    gtk_list_store_insert_with_values(tool->model, &iter, G_MAXINT,
-                                      COLUMN_ID, quark,
-                                      COLUMN_OBJECT, sel,
-                                      -1);
+    GwyToolSelectionManager *tool = (GwyToolSelectionManager*)user_data;
+    update_selection_item(tool, quark, TRUE);
 }
 
 static void
-gwy_tool_selection_manager_data_switched(GwyTool *gwytool,
-                                         GwyDataView *data_view)
+data_switched(GwyTool *gwytool, GwyDataView *data_view)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(gwytool);
     GwyToolSelectionManager *tool = GWY_TOOL_SELECTION_MANAGER(gwytool);
@@ -431,58 +394,132 @@ gwy_tool_selection_manager_data_switched(GwyTool *gwytool,
 
     GWY_TOOL_CLASS(gwy_tool_selection_manager_parent_class)->data_switched(gwytool, data_view);
 
+    tool->prefix = NULL;
     if (ignore || plain_tool->init_failed)
         return;
 
-    /* FIXME: This is very naive because the tool cannot react to selections changed by something else.  Hopefully
-     * only other tools do such things -- and then we get a chance to re-read the selection list.  */
+    gwy_set_member_object(tool, plain_tool->container, GWY_TYPE_FILE, &tool->file,
+                          "item-changed", file_item_changed, &tool->item_changed_id, G_CONNECT_SWAPPED,
+                          NULL);
+
     gtk_list_store_clear(tool->model);
     if (data_view) {
-        gchar *basekey = g_strdup_printf("/%d/select", plain_tool->id);
-        gwy_container_foreach(plain_tool->container, basekey, (GHFunc)&add_selection, tool);
-        g_free(basekey);
+        tool->prefix = g_quark_to_string(gwy_file_key_image_selection(plain_tool->id, NULL));
+        gwy_container_foreach(plain_tool->container, tool->prefix, add_selection, tool);
     }
 
     selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tool->treeview));
-    gwy_tool_selection_manager_selection_changed(tool, selection);
+    list_selection_changed(tool, selection);
 }
 
 static void
-gwy_tool_selection_manager_response(GwyTool *gwytool,
-                                    gint response_id)
+file_item_changed(GwyToolSelectionManager *tool,
+                  GQuark key,
+                  G_GNUC_UNUSED GwyContainer *data)
+{
+    if (tool->doing_mass_change)
+        return;
+
+    g_return_if_fail(tool->prefix);
+
+    const gchar *strkey = g_quark_to_string(key);
+    if (!g_str_has_prefix(strkey, tool->prefix)
+        || strkey[strlen(tool->prefix)] != '/')
+        return;
+
+    gwy_debug("candidate item %s changed", strkey);
+    update_selection_item(tool, key, FALSE);
+}
+
+/* If addit=TRUE, the caller knows it is a new selection item. Otherwise we need to look for it in the tree model to
+ * see if we already have it. */
+static void
+update_selection_item(GwyToolSelectionManager *tool,
+                      GQuark quark,
+                      gboolean addit)
+{
+    const gchar *strkey = g_quark_to_string(quark);
+    GwyFileKeyParsed parsed;
+    gwy_file_parse_key(strkey, &parsed);
+    /* Ignore selections with names starting with an underscore. */
+    if (parsed.piece != GWY_FILE_PIECE_SELECTION || (parsed.suffix && parsed.suffix[0] == '_'))
+        return;
+
+    GwySelection *sel = NULL;
+    gboolean object_exists = gwy_container_gis_object(GWY_PLAIN_TOOL(tool)->container, quark, &sel);
+    if (object_exists && !GWY_IS_SELECTION(sel)) {
+        gwy_debug("item which is not a Selection under /selection?");
+        return;
+    }
+    if (addit && !object_exists) {
+        g_warning("The caller thinks we should add %s, but it does not exist.", strkey);
+    }
+
+    GtkTreeModel *model = GTK_TREE_MODEL(tool->model);
+    GtkTreeIter iter;
+
+    /* If addit=FALSE, we need try to find the item. */
+    gboolean foundit = FALSE;
+    if (!addit && gtk_tree_model_get_iter_first(model, &iter)) {
+        do {
+            GQuark modelquark;
+            gtk_tree_model_get(model, &iter, COLUMN_KEY, &modelquark, -1);
+            if (modelquark == quark) {
+                foundit = TRUE;
+                break;
+            }
+        } while (gtk_tree_model_iter_next(model, &iter));
+    }
+
+    g_return_if_fail(object_exists || foundit);
+
+    /* New selection. Add it. */
+    if (addit || (object_exists && !foundit)) {
+        /* FIXME: Any ordering? Currently the order is “whatever”. */
+        gtk_list_store_insert_with_values(tool->model, &iter, G_MAXINT,
+                                          COLUMN_KEY, quark,
+                                          COLUMN_OBJECT, sel,
+                                          -1);
+        return;
+    }
+
+    /* Removed selection. Delete it. */
+    if (foundit && !object_exists) {
+        gtk_list_store_remove(tool->model, &iter);
+        return;
+    }
+
+    g_assert(object_exists && foundit);
+    gtk_list_store_set(tool->model, &iter, COLUMN_OBJECT, sel, -1);
+}
+
+static void
+tool_response(GwyTool *gwytool, gint response_id)
 {
     GwyToolSelectionManager *tool = GWY_TOOL_SELECTION_MANAGER(gwytool);
 
     GWY_TOOL_CLASS(gwy_tool_selection_manager_parent_class)->response(gwytool, response_id);
 
-    if (response_id == GWY_TOOL_RESPONSE_CLEAR)
+    if (response_id == GWY_RESPONSE_CLEAR)
         clear_selection(tool);
 }
 
 static void
 clear_selection(GwyToolSelectionManager *tool)
 {
-    GtkTreeModel *model = GTK_TREE_MODEL(tool->model);
-    GwyPlainTool *plain_tool;
-    GtkTreeIter iter;
-    GQuark quark;
-
-    if (!gtk_tree_model_get_iter_first(model, &iter))
-        return;
-
-    plain_tool = GWY_PLAIN_TOOL(tool);
-    do {
-        gtk_tree_model_get(model, &iter, COLUMN_ID, &quark, -1);
-        gwy_container_remove(plain_tool->container, quark);
-    } while (gtk_tree_model_iter_next(model, &iter));
-
-    /* XXX: Since the model is not auto-updated, clear it manually. */
-    gtk_list_store_clear(tool->model);
+    GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
+    /* FIXME: Handle data kind generically with AnyTool. */
+    if (plain_tool->container) {
+        tool->doing_mass_change = TRUE;
+        gwy_file_remove_selections(GWY_FILE(plain_tool->container), GWY_FILE_IMAGE, plain_tool->id);
+        /* Just clear it explicitly instead of one by one. */
+        gtk_list_store_clear(tool->model);
+        tool->doing_mass_change = FALSE;
+    }
 }
 
 static void
-gwy_tool_selection_manager_selection_changed(GwyToolSelectionManager *tool,
-                                             GtkTreeSelection *selection)
+list_selection_changed(GwyToolSelectionManager *tool, GtkTreeSelection *selection)
 {
     GtkTreeIter iter;
     gboolean is_selected = gtk_tree_selection_get_selected(selection, NULL, &iter);
@@ -499,23 +536,21 @@ gwy_tool_selection_manager_selection_changed(GwyToolSelectionManager *tool,
 
     if (is_selected) {
         GwySelection *sel;
-        GType type;
-        guint i, n;
 
         gtk_tree_model_get(GTK_TREE_MODEL(tool->model), &iter,
                            COLUMN_OBJECT, &sel,
-                           COLUMN_ID, &quark,
+                           COLUMN_KEY, &quark,
                            -1);
-        n = gwy_selection_get_data(sel, NULL);
+        gint n = gwy_selection_get_data(sel, NULL);
         if (!n) {
             gtk_widget_set_sensitive(tool->distribute, FALSE);
             gtk_widget_set_sensitive(tool->copy, FALSE);
             gtk_widget_set_sensitive(tool->export, FALSE);
         }
 
-        type = G_OBJECT_TYPE(sel);
-        for (i = 0; i < NLAYERTYPES; i++) {
-            if (type == tool->selection_types[i]) {
+        GType type = G_OBJECT_TYPE(sel);
+        for (guint i = 0; i < G_N_ELEMENTS(selection_info); i++) {
+            if (type == selection_info[i].selection_type) {
                 current = i;
                 break;
             }
@@ -527,8 +562,7 @@ gwy_tool_selection_manager_selection_changed(GwyToolSelectionManager *tool,
 }
 
 static void
-gwy_tool_selection_manager_all_files_changed(GwyToolSelectionManager *tool,
-                                             GtkToggleButton *button)
+all_files_toggled(GwyToolSelectionManager *tool, GtkToggleButton *button)
 {
     gwy_params_set_boolean(tool->params, PARAM_ALL_FILES, gtk_toggle_button_get_active(button));
 }
@@ -538,7 +572,7 @@ key_pressed(GwyToolSelectionManager *tool,
             GdkEventKey *event,
             G_GNUC_UNUSED GtkTreeView *treeview)
 {
-    if (event->keyval != GDK_Delete)
+    if (event->keyval != GDK_KEY_Delete)
         return FALSE;
 
     delete_selection(tool);
@@ -550,88 +584,78 @@ distribute(GwyToolSelectionManager *tool)
 {
     GwyPlainTool *plain_tool;
     GtkTreeSelection *treesel;
-    GwyDataField *field;
+    GwyField *field;
     DistributeData distdata;
     GtkTreeIter iter;
     GQuark quark;
-    const gchar *s;
 
     treesel = gtk_tree_view_get_selection(GTK_TREE_VIEW(tool->treeview));
     if (!gtk_tree_selection_get_selected(treesel, NULL, &iter))
         return;
 
     gtk_tree_model_get(GTK_TREE_MODEL(tool->model), &iter,
-                       COLUMN_ID, &quark,
+                       COLUMN_KEY, &quark,
                        COLUMN_OBJECT, &distdata.selection,
                        -1);
-    s = g_quark_to_string(quark);
-    g_return_if_fail(s);
-    distdata.name = strrchr(s, GWY_CONTAINER_PATHSEP);
+
+    GwyFileKeyParsed parsed;
+    gwy_file_parse_key(g_quark_to_string(quark), &parsed);
+    distdata.name = parsed.suffix;
     g_return_if_fail(distdata.name);
 
     plain_tool = GWY_PLAIN_TOOL(tool);
-    field = plain_tool->data_field;
-    distdata.xyunit = gwy_data_field_get_si_unit_xy(field);
-    distdata.origin[0] = gwy_data_field_get_xoffset(field);
-    distdata.origin[1] = gwy_data_field_get_yoffset(field);
-    gwy_debug("source: %p %s", field, s);
+    field = plain_tool->field;
+    distdata.xyunit = gwy_field_get_unit_xy(field);
+    distdata.origin[0] = gwy_field_get_xoffset(field);
+    distdata.origin[1] = gwy_field_get_yoffset(field);
+    gwy_debug("source: %p %s", field, g_quark_to_string(quark));
 
     if (gwy_params_get_boolean(tool->params, PARAM_ALL_FILES))
-        gwy_app_data_browser_foreach((GwyAppDataForeachFunc)distribute_one, &distdata);
+        gwy_data_browser_foreach(distribute_one, &distdata, NULL);
     else
-        distribute_one(plain_tool->container, &distdata);
+        distribute_one(GWY_FILE(plain_tool->container), &distdata);
 }
 
 static void
-distribute_one(GwyContainer *container, DistributeData *distdata)
+distribute_one(GwyFile *file, gpointer user_data)
 {
-    GObject *object, *selobject;
-    GwySelection *selection;
-    GwyDataField *field;
-    GString *str;
-    GQuark quark;
+    DistributeData *distdata = (DistributeData*)user_data;
+    GwyContainer *container = GWY_CONTAINER(file);
     gint *ids;
     gint i;
 
-    gwy_debug("dest: %p", container);
-    ids = gwy_app_data_browser_get_data_ids(container);
-    str = g_string_new(NULL);
-    selobject = G_OBJECT(distdata->selection);
+    gwy_debug("dest: %p", file);
+    ids = gwy_file_get_ids(file, GWY_FILE_IMAGE);
     for (i = 0; ids[i] >= 0; i++) {
-        gdouble xoff, yoff, xreal, yreal;
-
-        g_string_printf(str, "/%d/select%s", ids[i], distdata->name);
-        gwy_debug("%p %s", container, str->str);
-        quark = g_quark_from_string(str->str);
+        GQuark quark = gwy_file_key_image_selection(ids[i], distdata->name);
 
         /* Avoid copying to self */
-        if (gwy_container_gis_object(container, quark, &object) && object == selobject) {
+        GwySelection *selection = gwy_selection_copy(distdata->selection);
+        if (gwy_container_gis_object(container, quark, &selection) && selection == distdata->selection) {
             gwy_debug("avoiding copy-to-self");
             continue;
         }
 
         /* Check units */
-        g_string_printf(str, "/%d/data", ids[i]);
-        if (!gwy_container_gis_object_by_name(container, str->str, &object) || !GWY_IS_DATA_FIELD(object)) {
+        GwyField *field;
+        if (!(field = gwy_file_get_image(file, ids[i]))) {
             gwy_debug("data field not found?!");
             continue;
         }
-        field = GWY_DATA_FIELD(object);
-        if (!gwy_si_unit_equal(gwy_data_field_get_si_unit_xy(field), distdata->xyunit)) {
+        if (!gwy_unit_equal(gwy_field_get_unit_xy(field), distdata->xyunit)) {
             gwy_debug("units differ");
             continue;
         }
 
-        xoff = gwy_data_field_get_xoffset(field);
-        yoff = gwy_data_field_get_yoffset(field);
-        xreal = gwy_data_field_get_xreal(field);
-        yreal = gwy_data_field_get_yreal(field);
-        object = gwy_serializable_duplicate(selobject);
-        selection = GWY_SELECTION(object);
+        gdouble xoff = gwy_field_get_xoffset(field);
+        gdouble yoff = gwy_field_get_yoffset(field);
+        gdouble xreal = gwy_field_get_xreal(field);
+        gdouble yreal = gwy_field_get_yreal(field);
 
         /* Crop the selection, taking into account that the coordinates do not include field offset, and move it
          * relative to the new origin. But for Lattice, which is origin-free, just limit it so that it fits inside. */
-        if (gwy_strequal(G_OBJECT_TYPE_NAME(object), "GwySelectionLattice"))
+        selection = gwy_selection_copy(distdata->selection);
+        if (GWY_IS_SELECTION_LATTICE(selection))
             gwy_selection_crop(selection, -0.5*xreal, -0.5*yreal, 0.5*xreal, 0.5*yreal);
         else {
             gwy_selection_move(selection, distdata->origin[0], distdata->origin[1]);
@@ -640,13 +664,12 @@ distribute_one(GwyContainer *container, DistributeData *distdata)
         }
 
         if (gwy_selection_get_data(selection, NULL))
-            gwy_container_set_object(container, quark, object);
+            gwy_container_pass_object(container, quark, selection);
         else {
             gwy_debug("selection empty after cropping");
+            g_object_unref(selection);
         }
-        g_object_unref(object);
     }
-    g_string_free(str, TRUE);
     g_free(ids);
 }
 
@@ -670,44 +693,25 @@ copy_selection_to_clipboard(GwyToolSelectionManager *tool)
 static void
 export_selection(GwyToolSelectionManager *tool)
 {
-    gchar *text;
-
-    text = create_report(tool);
+    gchar *text = create_report(tool);
     if (!text)
         return;
 
-    gwy_save_auxiliary_data(_("Save Table"), GTK_WINDOW(GWY_TOOL(tool)->dialog), -1, text);
+    gwy_save_auxiliary_data(_("Save Table"), GTK_WINDOW(GWY_TOOL(tool)->dialog), text, -1, FALSE);
     g_free(text);
 }
 
 static void
-add_coordinate_header(GString *text, GwySelection *selection, GwySIUnit *unit)
+add_coordinate_header(GString *text, GwySelection *selection, GwyUnit *unit)
 {
-    const gchar *typename = G_OBJECT_TYPE_NAME(selection);
-    gchar *unitstr = gwy_si_unit_get_string(unit, GWY_SI_UNIT_FORMAT_PLAIN);
+    gchar *unitstr = gwy_unit_get_string(unit, GWY_UNIT_FORMAT_PLAIN);
     gchar *s = (strlen(unitstr) ? g_strconcat(" [", unitstr, "]", NULL) : g_strdup(""));
+    guint objsize = gwy_selection_get_object_size(selection);
 
-    if (gwy_stramong(typename, "GwySelectionProjective", NULL))
-        g_string_append_printf(text, "x0%s\ty0%s\tx1%s\ty1%s\tx2%s\ty2%s\tx3%s\ty3%s\n", s, s, s, s, s, s, s, s);
-    else if (gwy_stramong(typename,
-                          "GwySelectionRectangle", "GwySelectionEllipse", "GwySelectionLine", "GwySelectionLattice",
-                          NULL))
-        g_string_append_printf(text, "x0%s\ty0%s\tx1%s\ty1%s\n", s, s, s, s);
-    else if (gwy_stramong(typename, "GwySelectionCross", "GwySelectionPoint", "GwySelectionPath", NULL))
-        g_string_append_printf(text, "x%s\ty%s\n", s, s);
-    else if (gwy_strequal(typename, "GwySelectionAxis")) {
-        GwyOrientation orientation;
-
-        g_object_get(selection, "orientation", &orientation, NULL);
-        g_string_append_printf(text, "%c%s\n", orientation == GWY_ORIENTATION_VERTICAL ? 'y' : 'x', s);
-    }
-    else {
-        guint j, objsize = gwy_selection_get_object_size(selection);
-
-        for (j = 0; j < objsize; j++) {
-            g_string_append_printf(text, "c%u%s", j, s);
-            g_string_append_c(text, (j == objsize-1) ? '\n' : '\t');
-        }
+    for (guint i = 0; i < objsize; i++) {
+        g_string_append(text, gwy_selection_get_coord_symbol(selection, i));
+        g_string_append(text, s);
+        g_string_append_c(text, (i == objsize-1) ? '\n' : '\t');
     }
 
     g_free(s);
@@ -718,16 +722,15 @@ static gchar*
 create_report(GwyToolSelectionManager *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    GwyDataField *field = plain_tool->data_field;
+    GwyField *field = plain_tool->field;
     GtkTreeSelection *treesel;
     GwySelection *selection, *movedselection = NULL;
-    GwySIUnit *xyunit;
+    GwyUnit *xyunit;
     gdouble xoff, yoff;
     GtkTreeIter iter;
     GString *text;
     guint i, j, n, objsize;
     gdouble *coords;
-    const gchar *typename;
     gchar buf[64];
 
     treesel = gtk_tree_view_get_selection(GTK_TREE_VIEW(tool->treeview));
@@ -735,17 +738,16 @@ create_report(GwyToolSelectionManager *tool)
         return NULL;
 
     gtk_tree_model_get(GTK_TREE_MODEL(tool->model), &iter, COLUMN_OBJECT, &selection, -1);
-    typename = G_OBJECT_TYPE_NAME(selection);
     n = gwy_selection_get_data(selection, NULL);
     if (!n)
         return NULL;
 
-    xyunit = gwy_data_field_get_si_unit_xy(field);
-    xoff = gwy_data_field_get_xoffset(field);
-    yoff = gwy_data_field_get_yoffset(field);
+    xyunit = gwy_field_get_unit_xy(field);
+    xoff = gwy_field_get_xoffset(field);
+    yoff = gwy_field_get_yoffset(field);
     /* Apply the data field offset in a generic manner. */
-    if ((xoff != 0.0 || yoff != 0.0) && !gwy_strequal(typename, "GwySelectionLattice")) {
-        movedselection = gwy_selection_duplicate(selection);
+    if (xoff != 0.0 || yoff != 0.0) {
+        movedselection = gwy_selection_copy(selection);
         gwy_selection_move(movedselection, xoff, yoff);
         selection = movedselection;
     }
@@ -765,7 +767,7 @@ create_report(GwyToolSelectionManager *tool)
     }
 
     g_free(coords);
-    GWY_OBJECT_UNREF(movedselection);
+    g_clear_object(&movedselection);
 
     return g_string_free(text, FALSE);
 }
@@ -773,17 +775,14 @@ create_report(GwyToolSelectionManager *tool)
 static void
 delete_selection(GwyToolSelectionManager *tool)
 {
-    GtkTreeSelection *treesel;
     GtkTreeModel *model;
     GtkTreeIter iter;
-    GQuark quark;
 
-    treesel = gtk_tree_view_get_selection(GTK_TREE_VIEW(tool->treeview));
+    GtkTreeSelection *treesel = gtk_tree_view_get_selection(GTK_TREE_VIEW(tool->treeview));
     if (gtk_tree_selection_get_selected(treesel, &model, &iter)) {
-        gtk_tree_model_get(model, &iter, COLUMN_ID, &quark, -1);
+        GQuark quark;
+        gtk_tree_model_get(model, &iter, COLUMN_KEY, &quark, -1);
         gwy_container_remove(GWY_PLAIN_TOOL(tool)->container, quark);
-        /* XXX: Since the model is not auto-updated, clear it manually. */
-        gtk_list_store_remove(tool->model, &iter);
     }
 }
 
@@ -795,7 +794,7 @@ setup_layer(GwyToolSelectionManager *tool,
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
     const gchar *s;
 
-    if (!plain_tool->data_view || current >= NLAYERTYPES || !quark)
+    if (!plain_tool->data_view || current >= G_N_ELEMENTS(selection_info) || !quark)
         return;
 
     s = g_quark_to_string(quark);
@@ -803,8 +802,8 @@ setup_layer(GwyToolSelectionManager *tool,
     s = strrchr(s, GWY_CONTAINER_PATHSEP);
     g_return_if_fail(s);
 
-    gwy_plain_tool_connect_selection(plain_tool, tool->layer_types[current], s+1);
-    gwy_object_set_or_reset(plain_tool->layer, tool->layer_types[current],
+    gwy_plain_tool_connect_selection(plain_tool, selection_info[current].layer_type, s+1);
+    gwy_object_set_or_reset(plain_tool->layer, selection_info[current].layer_type,
                             "editable", TRUE,
                             "focus", -1,
                             NULL);
