@@ -1,5 +1,5 @@
 /*
- *  $Id: spectro.c 26745 2024-10-18 15:17:53Z yeti-dn $
+ *  $Id: spectro.c 28803 2025-11-05 11:58:19Z yeti-dn $
  *  Copyright (C) 2003-2022 Owain Davies, David Necas (Yeti), Petr Klapetek.
  *  E-mail: owain.davies@blueyonder.co.uk, yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,20 +16,12 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <glib-object.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/datafield.h>
-#include <libprocess/linestats.h>
-#include <libprocess/spectra.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwydgets/gwynullstore.h>
-#include <libgwymodule/gwymodule-tool.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 
 enum {
     MIN_RESOLUTION = 4,
@@ -64,17 +56,13 @@ struct _GwyToolSpectro {
     GtkTreeModel *model;
     GwyParamTable *table;
 
-    GwyDataLine *line;
-    GwyDataLine *weights;
+    GwyLine *line;
+    GwyLine *weights;
     GwyGraphModel *gmodel;
     GwySpectra *spectra;
 
-    GdkPixbuf *colorpixbuf;
     gulong layer_object_chosen_id;
     gboolean ignore_tree_selection;
-
-    /* potential class data */
-    GType layer_type;
 };
 
 struct _GwyToolSpectroClass {
@@ -105,15 +93,15 @@ static void         show_curve                       (GwyToolSpectro *tool,
                                                       gint i);
 static void         gather_curve                     (GwyToolSpectro *tool,
                                                       gint i);
-static void         accumulate_lines                 (GwyDataLine *accum,
-                                                      GwyDataLine *dline,
-                                                      GwyDataLine *weights);
+static void         accumulate_lines                 (GwyLine *accum,
+                                                      GwyLine *dline,
+                                                      GwyLine *weights);
 static void         show_averaged                    (GwyToolSpectro *tool);
 static void         update_header                    (GwyToolSpectro *tool,
                                                       guint col,
                                                       GString *str,
                                                       const gchar *title,
-                                                      GwySIValueFormat *vf);
+                                                      GwyValueFormat *vf);
 static void         render_cell                      (GtkCellLayout *layout,
                                                       GtkCellRenderer *renderer,
                                                       GtkTreeModel *model,
@@ -173,7 +161,7 @@ gwy_tool_spectro_class_init(GwyToolSpectroClass *klass)
 
     gobject_class->finalize = gwy_tool_spectro_finalize;
 
-    tool_class->stock_id = GWY_STOCK_SPECTRUM;
+    tool_class->icon_name = GWY_ICON_SPECTRUM;
     tool_class->title = _("Point Spectroscopy");
     tool_class->tooltip = _("Extract and view point spectroscopy data");
     tool_class->prefix = "/module/spectro";
@@ -193,14 +181,13 @@ gwy_tool_spectro_finalize(GObject *object)
     /* Prevent bad things in the selection callback */
     tool->ignore_tree_selection = TRUE;
 
-    GWY_SIGNAL_HANDLER_DISCONNECT(plain_tool->layer, tool->layer_object_chosen_id);
+    g_clear_signal_handler(&tool->layer_object_chosen_id, plain_tool->layer);
     gtk_tree_view_set_model(tool->treeview, NULL);
     gwy_params_save_to_settings(tool->params);
-    GWY_OBJECT_UNREF(tool->params);
-    GWY_OBJECT_UNREF(tool->colorpixbuf);
-    GWY_OBJECT_UNREF(tool->model);
-    GWY_OBJECT_UNREF(tool->spectra);
-    GWY_OBJECT_UNREF(tool->gmodel);
+    g_clear_object(&tool->params);
+    g_clear_object(&tool->model);
+    g_clear_object(&tool->spectra);
+    g_clear_object(&tool->gmodel);
 
     G_OBJECT_CLASS(gwy_tool_spectro_parent_class)->finalize(object);
 }
@@ -209,22 +196,13 @@ static void
 gwy_tool_spectro_init(GwyToolSpectro *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    gint width, height;
 
-    tool->layer_type = gwy_plain_tool_check_layer_type(plain_tool, "GwyLayerPoint");
-    if (!tool->layer_type)
-        return;
-
-    plain_tool->unit_style = GWY_SI_UNIT_FORMAT_MARKUP;
+    plain_tool->unit_style = GWY_UNIT_FORMAT_MARKUP;
     plain_tool->lazy_updates = TRUE;
 
     tool->params = gwy_params_new_from_settings(define_module_params());
 
-    gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &width, &height);
-    height |= 1;
-    tool->colorpixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, height, height);
-
-    gwy_plain_tool_connect_selection(plain_tool, tool->layer_type, "spec");
+    gwy_plain_tool_connect_selection(plain_tool, GWY_TYPE_LAYER_POINT, "spec");
 
     gwy_tool_spectro_init_dialog(tool);
 }
@@ -246,11 +224,11 @@ gwy_tool_spectro_init_dialog(GwyToolSpectro *tool)
     tool->gmodel = gwy_graph_model_new();
     g_object_set(tool->gmodel, "label-visible", FALSE, NULL);
 
-    hbox = gwy_hbox_new(4);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
     gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(dialog)), hbox, TRUE, TRUE, 0);
 
     /* Left pane */
-    vbox = gwy_vbox_new(8);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 8);
     gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);
 
     /* Point coordinates */
@@ -267,8 +245,7 @@ gwy_tool_spectro_init_dialog(GwyToolSpectro *tool)
         gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(column), renderer, TRUE);
         gtk_cell_layout_set_cell_data_func(GTK_CELL_LAYOUT(column), renderer, render_cell, tool, NULL);
         if (i == COLUMN_I) {
-            renderer = gtk_cell_renderer_pixbuf_new();
-            g_object_set(renderer, "pixbuf", tool->colorpixbuf, NULL);
+            renderer = gwy_cell_renderer_color_new();
             gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(column), renderer, FALSE);
             gtk_cell_layout_set_cell_data_func(GTK_CELL_LAYOUT(column), renderer, render_colour, tool, NULL);
         }
@@ -304,7 +281,7 @@ gwy_tool_spectro_init_dialog(GwyToolSpectro *tool)
     gtk_box_pack_start(GTK_BOX(hbox), graph, TRUE, TRUE, 2);
 
     gwy_tool_add_hide_button(GWY_TOOL(tool), FALSE);
-    gtk_dialog_add_button(dialog, GTK_STOCK_APPLY, GTK_RESPONSE_APPLY);
+    gwy_plain_tool_add_apply_button(GWY_PLAIN_TOOL(tool));
     gtk_dialog_set_default_response(dialog, GTK_RESPONSE_APPLY);
     gtk_dialog_set_response_sensitive(dialog, GTK_RESPONSE_APPLY, FALSE);
     gwy_help_add_to_tool_dialog(dialog, GWY_TOOL(tool), GWY_HELP_DEFAULT);
@@ -327,7 +304,7 @@ gwy_tool_spectro_data_switched(GwyTool *gwytool,
 
     if (!ignore) {
         gwy_debug("disconect obj-chosen handler: %u", (guint)tool->layer_object_chosen_id);
-        GWY_SIGNAL_HANDLER_DISCONNECT(plain_tool->layer, tool->layer_object_chosen_id);
+        g_clear_signal_handler(&tool->layer_object_chosen_id, plain_tool->layer);
     }
 
     GWY_TOOL_CLASS(gwy_tool_spectro_parent_class)->data_switched(gwytool, data_view);
@@ -336,9 +313,9 @@ gwy_tool_spectro_data_switched(GwyTool *gwytool,
         return;
 
     if (plain_tool->layer) {
-        gwy_object_set_or_reset(plain_tool->layer, tool->layer_type,
+        gwy_object_set_or_reset(plain_tool->layer, GWY_TYPE_LAYER_POINT,
                                 "editable", FALSE,
-                                "point-numbers", TRUE,
+                                "numbered", TRUE,
                                 "focus", -1,
                                 NULL);
     }
@@ -348,7 +325,7 @@ gwy_tool_spectro_data_switched(GwyTool *gwytool,
     }
 
     gwy_graph_model_remove_all_curves(tool->gmodel);
-    if (plain_tool->data_field && tool->spectra) {
+    if (plain_tool->field && tool->spectra) {
         gwy_selection_set_max_objects(plain_tool->selection, gwy_spectra_get_n_spectra(tool->spectra));
         fill_locations(tool);
     }
@@ -370,11 +347,10 @@ gwy_tool_spectro_spectra_switched(GwyTool *gwytool,
     gwy_debug("spectra: %p", spectra);
 
     if (spectra) {
-        if (!plain_tool->data_field)
+        if (!plain_tool->field)
             g_warning("Spectra made current without any channel?");
         else {
-            if (!gwy_si_unit_equal(gwy_spectra_get_si_unit_xy(spectra),
-                                   gwy_data_field_get_si_unit_xy(plain_tool->data_field))) {
+            if (!gwy_unit_equal(gwy_spectra_get_unit_xy(spectra), gwy_field_get_unit_xy(plain_tool->field))) {
                 gwy_debug("Spectra and channel units do not match");
                 spectra = NULL;
             }
@@ -391,7 +367,7 @@ gwy_tool_spectro_spectra_switched(GwyTool *gwytool,
         gwy_null_store_set_n_rows(store, 0);
         tool->ignore_tree_selection = FALSE;
         tree_selection_changed(selection, tool);
-        GWY_OBJECT_UNREF(tool->spectra);
+        g_clear_object(&tool->spectra);
         if (plain_tool->selection)
             gwy_selection_clear(plain_tool->selection);
         return;
@@ -399,7 +375,7 @@ gwy_tool_spectro_spectra_switched(GwyTool *gwytool,
 
     g_return_if_fail(GWY_IS_SPECTRA(spectra));
     g_object_ref(spectra);
-    GWY_OBJECT_UNREF(tool->spectra);
+    g_clear_object(&tool->spectra);
     tool->spectra = spectra;
 
     g_object_set(tool->gmodel, "title", gwy_spectra_get_title(tool->spectra), NULL);
@@ -453,7 +429,7 @@ static void
 fill_locations(GwyToolSpectro *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    GwyDataField *field = plain_tool->data_field;
+    GwyField *field = plain_tool->field;
     gdouble xoff, yoff;
     gdouble coords[2];
     gint i, nspec;
@@ -461,10 +437,10 @@ fill_locations(GwyToolSpectro *tool)
     g_return_if_fail(field);
 
     nspec = gwy_spectra_get_n_spectra(tool->spectra);
-    xoff = gwy_data_field_get_xoffset(plain_tool->data_field);
-    yoff = gwy_data_field_get_yoffset(plain_tool->data_field);
+    xoff = gwy_field_get_xoffset(plain_tool->field);
+    yoff = gwy_field_get_yoffset(plain_tool->field);
     for (i = 0; i < nspec; i++) {
-        gwy_spectra_itoxy(tool->spectra, i, &coords[0], &coords[1]);
+        gwy_spectra_getpos(tool->spectra, i, &coords[0], &coords[1]);
         coords[0] -= xoff;
         coords[1] -= yoff;
         gwy_selection_set_object(plain_tool->selection, i, coords);
@@ -573,7 +549,7 @@ show_curve(GwyToolSpectro *tool,
     }
 
     if (gcmodel) {
-        gwy_graph_curve_model_set_data_from_dataline(gcmodel, tool->line, 0, 0);
+        gwy_graph_curve_model_set_data_from_line(gcmodel, tool->line, 0, 0);
     }
     else {
         const GwyRGBA *rgba;
@@ -589,12 +565,12 @@ show_curve(GwyToolSpectro *tool,
                      "color", rgba,
                      NULL);
         g_free(desc);
-        gwy_graph_curve_model_set_data_from_dataline(gcmodel, tool->line, 0, 0);
+        gwy_graph_curve_model_set_data_from_line(gcmodel, tool->line, 0, 0);
         gwy_graph_model_add_curve(tool->gmodel, gcmodel);
         g_object_unref(gcmodel);
 
         if (n == 0)
-            gwy_graph_model_set_units_from_data_line(tool->gmodel, tool->line);
+            gwy_graph_model_set_units_from_line(tool->gmodel, tool->line);
     }
     tool->line = NULL;
 }
@@ -604,14 +580,14 @@ gather_curve(GwyToolSpectro *tool,
              gint id)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    GwyDataLine *spectrum = gwy_spectra_get_spectrum(tool->spectra, id);
+    GwyLine *spectrum = gwy_spectra_get_spectrum(tool->spectra, id);
 
     g_return_if_fail(plain_tool->selection);
     if (!tool->line) {
         /* Use the first selected line as the template. */
-        tool->line = gwy_data_line_duplicate(spectrum);
-        tool->weights = gwy_data_line_new_alike(spectrum, TRUE);
-        gwy_data_line_fill(tool->weights, 1.0);
+        tool->line = gwy_line_copy(spectrum);
+        tool->weights = gwy_line_new_alike(spectrum, TRUE);
+        gwy_line_fill(tool->weights, 1.0);
     }
     else {
         /* Accumulate the others. */
@@ -622,27 +598,27 @@ gather_curve(GwyToolSpectro *tool,
 }
 
 static void
-accumulate_lines(GwyDataLine *accum, GwyDataLine *dline, GwyDataLine *weights)
+accumulate_lines(GwyLine *accum, GwyLine *dline, GwyLine *weights)
 {
     gint i, n, n1;
     gdouble *adata, *data, *wdata;
     gdouble real, off, real1, off1;
 
-    n = gwy_data_line_get_res(accum);
-    n1 = gwy_data_line_get_res(dline);
-    data = gwy_data_line_get_data(dline);
-    adata = gwy_data_line_get_data(accum);
-    real = gwy_data_line_get_real(accum);
-    real1 = gwy_data_line_get_real(dline);
-    off = gwy_data_line_get_offset(accum);
-    off1 = gwy_data_line_get_offset(dline);
+    n = gwy_line_get_res(accum);
+    n1 = gwy_line_get_res(dline);
+    data = gwy_line_get_data(dline);
+    adata = gwy_line_get_data(accum);
+    real = gwy_line_get_real(accum);
+    real1 = gwy_line_get_real(dline);
+    off = gwy_line_get_offset(accum);
+    off1 = gwy_line_get_offset(dline);
 
     if (n1 == n
         && fabs(real1 - real) <= 1e-9*(fabs(real1) + fabs(real))
         && fabs(off1 - off) <= 1e-9*(fabs(off1) + fabs(off))) {
         for (i = 0; i < n1; i++)
             adata[i] += data[i];
-        gwy_data_line_add(weights, 1.0);
+        gwy_line_add(weights, 1.0);
         return;
     }
 
@@ -650,7 +626,7 @@ accumulate_lines(GwyDataLine *accum, GwyDataLine *dline, GwyDataLine *weights)
         return;
 
     /* This is not very good but better than a CRTICIAL message we used to do here... */
-    wdata = gwy_data_line_get_data(weights);
+    wdata = gwy_line_get_data(weights);
     for (i = 0; i < n; i++) {
         gdouble x = (i + 0.5)*real/n + off;
         gint j = floor((x - off1)/real1*n1);
@@ -676,18 +652,18 @@ show_averaged(GwyToolSpectro *tool)
                  "description", gwy_spectra_get_title(tool->spectra),
                  "color", rgba,
                  NULL);
-    n = gwy_data_line_get_res(tool->line);
-    adata = gwy_data_line_get_data(tool->line);
-    wdata = gwy_data_line_get_data(tool->weights);
+    n = gwy_line_get_res(tool->line);
+    adata = gwy_line_get_data(tool->line);
+    wdata = gwy_line_get_data(tool->weights);
     for (i = 0; i < n; i++)
         adata[i] /= wdata[i];
-    gwy_graph_curve_model_set_data_from_dataline(gcmodel, tool->line, 0, 0);
+    gwy_graph_curve_model_set_data_from_line(gcmodel, tool->line, 0, 0);
     gwy_graph_model_add_curve(tool->gmodel, gcmodel);
     g_object_unref(gcmodel);
 
-    gwy_graph_model_set_units_from_data_line(tool->gmodel, tool->line);
-    GWY_OBJECT_UNREF(tool->line);
-    GWY_OBJECT_UNREF(tool->weights);
+    gwy_graph_model_set_units_from_line(tool->gmodel, tool->line);
+    g_clear_object(&tool->line);
+    g_clear_object(&tool->weights);
 }
 
 static void
@@ -695,7 +671,7 @@ update_header(GwyToolSpectro *tool,
               guint col,
               GString *str,
               const gchar *title,
-              GwySIValueFormat *vf)
+              GwyValueFormat *vf)
 {
     GtkTreeViewColumn *column;
     GtkLabel *label;
@@ -719,7 +695,7 @@ render_cell(GtkCellLayout *layout,
             gpointer user_data)
 {
     GwyToolSpectro *tool = (GwyToolSpectro*)user_data;
-    const GwySIValueFormat *vf = GWY_PLAIN_TOOL(tool)->coord_format;
+    const GwyValueFormat *vf = GWY_PLAIN_TOOL(tool)->coord_format;
     guint id = GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(layout), "id"));
     gchar buf[48];
     gdouble val;
@@ -733,9 +709,9 @@ render_cell(GtkCellLayout *layout,
     }
 
     if (id == COLUMN_X)
-        gwy_spectra_itoxy(tool->spectra, idx, &val, NULL);
+        gwy_spectra_getpos(tool->spectra, idx, &val, NULL);
     else if (id == COLUMN_Y)
-        gwy_spectra_itoxy(tool->spectra, idx, NULL, &val);
+        gwy_spectra_getpos(tool->spectra, idx, NULL, &val);
     else {
         g_return_if_reached();
     }
@@ -750,7 +726,7 @@ render_cell(GtkCellLayout *layout,
 
 static void
 render_colour(G_GNUC_UNUSED GtkCellLayout *layout,
-              G_GNUC_UNUSED GtkCellRenderer *renderer,
+              GtkCellRenderer *renderer,
               GtkTreeModel *model,
               GtkTreeIter *iter,
               gpointer user_data)
@@ -760,24 +736,23 @@ render_colour(G_GNUC_UNUSED GtkCellLayout *layout,
     GwyGraphModel *gmodel;
     GwyGraphCurveModel *gcmodel;
     GwyRGBA *rgba;
-    guint idx, pixel, n, i;
+    guint idx, n, i;
 
-    pixel = 0;
-    if (gtk_tree_selection_iter_is_selected(sel, iter)) {
-        gmodel = tool->gmodel;
-        gtk_tree_model_get(model, iter, 0, &idx, -1);
-        n = gwy_graph_model_get_n_curves(gmodel);
-        for (i = 0; i < n; i++) {
-            gcmodel = gwy_graph_model_get_curve(tool->gmodel, i);
-            if (GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(gcmodel), "sid")) == idx) {
-                g_object_get(gcmodel, "color", &rgba, NULL);
-                pixel = 0xff | gwy_rgba_to_pixbuf_pixel(rgba);
-                gwy_rgba_free(rgba);
-                break;
-            }
+    if (!gtk_tree_selection_iter_is_selected(sel, iter))
+        return;
+
+    gmodel = tool->gmodel;
+    gtk_tree_model_get(model, iter, 0, &idx, -1);
+    n = gwy_graph_model_get_n_curves(gmodel);
+    for (i = 0; i < n; i++) {
+        gcmodel = gwy_graph_model_get_curve(tool->gmodel, i);
+        if (GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(gcmodel), "sid")) == idx) {
+            g_object_get(gcmodel, "color", &rgba, NULL);
+            g_object_set(renderer, "color", rgba, NULL);
+            gwy_rgba_free(rgba);
+            break;
         }
     }
-    gdk_pixbuf_fill(tool->colorpixbuf, pixel);
 }
 
 static void
@@ -819,9 +794,10 @@ gwy_tool_spectro_apply(GwyToolSpectro *tool)
     }
 
     if (average || !separate) {
-        gmodel = gwy_graph_model_duplicate(tool->gmodel);
+        gmodel = gwy_graph_model_copy(tool->gmodel);
         g_object_set(gmodel, "label-visible", TRUE, NULL);
-        gwy_app_data_browser_add_graph_model(gmodel, plain_tool->container, TRUE);
+        gint newid = gwy_file_add_graph(GWY_FILE(plain_tool->container), gmodel);
+        gwy_file_set_visible(GWY_FILE(plain_tool->container), GWY_FILE_GRAPH, newid, TRUE);
         g_object_unref(gmodel);
         return;
     }
@@ -830,13 +806,14 @@ gwy_tool_spectro_apply(GwyToolSpectro *tool)
         gmodel = gwy_graph_model_new_alike(tool->gmodel);
         g_object_set(gmodel, "label-visible", TRUE, NULL);
         gcmodel = gwy_graph_model_get_curve(tool->gmodel, i);
-        gcmodel = gwy_graph_curve_model_duplicate(gcmodel);
+        gcmodel = gwy_graph_curve_model_copy(gcmodel);
         gwy_graph_model_add_curve(gmodel, gcmodel);
         g_object_unref(gcmodel);
         g_object_get(gcmodel, "description", &s, NULL);
         g_object_set(gmodel, "title", s, NULL);
         g_free(s);
-        gwy_app_data_browser_add_graph_model(gmodel, plain_tool->container, TRUE);
+        gint newid = gwy_file_add_graph(GWY_FILE(plain_tool->container), gmodel);
+        gwy_file_set_visible(GWY_FILE(plain_tool->container), GWY_FILE_GRAPH, newid, TRUE);
         g_object_unref(gmodel);
     }
 }
