@@ -1,5 +1,5 @@
 /*
- *  $Id: stats.c 26745 2024-10-18 15:17:53Z yeti-dn $
+ *  $Id: stats.c 28978 2025-12-11 10:41:31Z yeti-dn $
  *  Copyright (C) 2003-2022 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,20 +16,12 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <errno.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyresults.h>
-#include <libgwymodule/gwymodule-tool.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libprocess/datafield.h>
-#include <libprocess/stats.h>
-#include <libprocess/stats_uncertainty.h>
-#include <libgwydgets/gwystock.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
+#include <gwy.h>
+#include "libgwyapp/sanity.h"
 
 enum {
     PARAM_MASKING,
@@ -79,13 +71,6 @@ struct _GwyToolStats {
     gdouble rsel[4];
 
     gboolean same_units;
-    gboolean has_calibration;
-    GwyDataField *xunc;
-    GwyDataField *yunc;
-    GwyDataField *zunc;
-
-    /* potential class data */
-    GType layer_type_rect;
 };
 
 struct _GwyToolStatsClass {
@@ -111,16 +96,6 @@ static void         param_changed                   (GwyToolStats *tool,
 static void         update_sensitivity              (GwyToolStats *tool);
 static void         update_labels                   (GwyToolStats *tool);
 static gboolean     calculate                       (GwyToolStats *tool);
-static void         calculate_uncertainties         (GwyToolStats *tool,
-                                                     StatsUncertanties *unc,
-                                                     GwyDataField *field,
-                                                     GwyDataField *mask,
-                                                     GwyMaskingType masking,
-                                                     guint nn,
-                                                     gint col,
-                                                     gint row,
-                                                     gint w,
-                                                     gint h);
 static void         update_units                    (GwyToolStats *tool);
 static GwyResults*  create_results                  (void);
 
@@ -174,7 +149,7 @@ gwy_tool_stats_class_init(GwyToolStatsClass *klass)
 
     gobject_class->finalize = gwy_tool_stats_finalize;
 
-    tool_class->stock_id = GWY_STOCK_STAT_QUANTITIES;
+    tool_class->icon_name = GWY_ICON_STAT_QUANTITIES;
     tool_class->title = _("Statistical Quantities");
     tool_class->tooltip = _("Statistical quantities");
     tool_class->prefix = "/module/stats";
@@ -192,8 +167,8 @@ gwy_tool_stats_finalize(GObject *object)
     GwyToolStats *tool = GWY_TOOL_STATS(object);
 
     gwy_params_save_to_settings(tool->params);
-    GWY_OBJECT_UNREF(tool->params);
-    GWY_OBJECT_UNREF(tool->results);
+    g_clear_object(&tool->params);
+    g_clear_object(&tool->results);
 
     G_OBJECT_CLASS(gwy_tool_stats_parent_class)->finalize(object);
 }
@@ -203,17 +178,13 @@ gwy_tool_stats_init(GwyToolStats *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
 
-    tool->layer_type_rect = gwy_plain_tool_check_layer_type(plain_tool, "GwyLayerRectangle");
-    if (!tool->layer_type_rect)
-        return;
-
     plain_tool->lazy_updates = TRUE;
-    plain_tool->unit_style = GWY_SI_UNIT_FORMAT_VFMARKUP;
+    plain_tool->unit_style = GWY_UNIT_FORMAT_VFMARKUP;
 
     tool->params = gwy_params_new_from_settings(define_module_params());
     tool->results = create_results();
 
-    gwy_plain_tool_connect_selection(plain_tool, tool->layer_type_rect, "rectangle");
+    gwy_plain_tool_connect_selection(plain_tool, GWY_TYPE_LAYER_RECTANGLE, "rectangle");
     memset(tool->isel_prev, 0xff, 4*sizeof(gint));
     gwy_plain_tool_enable_selection_holding(plain_tool);
 
@@ -225,21 +196,21 @@ gwy_tool_stats_rect_updated(GwyToolStats *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
 
-    gwy_rect_selection_labels_select(tool->rlabels, plain_tool->selection, plain_tool->data_field);
+    gwy_rect_selection_labels_select(tool->rlabels, plain_tool->selection, plain_tool->field);
 }
 
 static void
 gwy_tool_stats_init_dialog(GwyToolStats *tool)
 {
     GtkDialog *dialog = GTK_DIALOG(GWY_TOOL(tool)->dialog);
-    GtkWidget *hbox, *vbox, *image;
+    GtkWidget *hbox, *vbox;
     GwyParamTable *table;
 
-    hbox = gwy_hbox_new(6);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 6);
     gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(dialog)), hbox, FALSE, FALSE, 0);
 
     /* Selection info */
-    vbox = gwy_vbox_new(0);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
     gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);
 
     tool->rlabels = gwy_rect_selection_labels_new(TRUE, G_CALLBACK(gwy_tool_stats_rect_updated), tool);
@@ -281,9 +252,7 @@ gwy_tool_stats_init_dialog(GwyToolStats *tool)
     gwy_plain_tool_add_param_table(GWY_PLAIN_TOOL(tool), table);
     gtk_box_pack_start(GTK_BOX(hbox), gwy_param_table_widget(table), FALSE, FALSE, 0);
 
-    tool->update = gtk_dialog_add_button(dialog, _("_Update"), GWY_TOOL_RESPONSE_UPDATE);
-    image = gtk_image_new_from_stock(GTK_STOCK_EXECUTE, GTK_ICON_SIZE_BUTTON);
-    gtk_button_set_image(GTK_BUTTON(tool->update), image);
+    tool->update = gwy_add_update_button_to_dialog(GTK_DIALOG(dialog));
     gwy_plain_tool_add_clear_button(GWY_PLAIN_TOOL(tool));
     gwy_tool_add_hide_button(GWY_TOOL(tool), TRUE);
     gwy_help_add_to_tool_dialog(dialog, GWY_TOOL(tool), GWY_HELP_DEFAULT);
@@ -303,8 +272,6 @@ gwy_tool_stats_data_switched(GwyTool *gwytool,
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(gwytool);
     GwyToolStats *tool = GWY_TOOL_STATS(gwytool);
     gboolean ignore = (data_view == plain_tool->data_view);
-    gchar xukey[32], yukey[32], zukey[32];
-    GwyContainer *container;
 
     GWY_TOOL_CLASS(gwy_tool_stats_parent_class)->data_switched(gwytool, data_view);
 
@@ -313,20 +280,11 @@ gwy_tool_stats_data_switched(GwyTool *gwytool,
 
     gwy_param_table_set_sensitive(tool->table_results, PARAM_REPORT_STYLE, FALSE);
     if (data_view) {
-        container = plain_tool->container;
-        gwy_object_set_or_reset(plain_tool->layer, tool->layer_type_rect,
+        gwy_object_set_or_reset(plain_tool->layer, GWY_TYPE_LAYER_RECTANGLE,
                                 "editable", TRUE,
                                 "focus", -1,
                                 NULL);
         gwy_selection_set_max_objects(plain_tool->selection, 1);
-
-        g_snprintf(xukey, sizeof(xukey), "/%d/data/cal_xunc", plain_tool->id);
-        g_snprintf(yukey, sizeof(yukey), "/%d/data/cal_yunc", plain_tool->id);
-        g_snprintf(zukey, sizeof(zukey), "/%d/data/cal_zunc", plain_tool->id);
-
-        tool->has_calibration = (gwy_container_gis_object_by_name(container, xukey, &tool->xunc)
-                                 && gwy_container_gis_object_by_name(container, yukey, &tool->yunc)
-                                 && gwy_container_gis_object_by_name(container, zukey, &tool->zunc));
         gwy_plain_tool_hold_selection(plain_tool, gwy_params_get_flags(tool->params, PARAM_HOLD_SELECTION));
         update_units(tool);
     }
@@ -337,14 +295,14 @@ static void
 update_units(GwyToolStats *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    GwyDataField *field = plain_tool->data_field;
-    GwySIUnit *siunitxy = gwy_data_field_get_si_unit_xy(field);
-    GwySIUnit *siunitz = gwy_data_field_get_si_unit_z(field);
+    GwyField *field = plain_tool->field;
+    GwyUnit *siunitxy = gwy_field_get_unit_xy(field);
+    GwyUnit *siunitz = gwy_field_get_unit_z(field);
 
     gwy_results_set_unit(tool->results, "x", siunitxy);
     gwy_results_set_unit(tool->results, "y", siunitxy);
     gwy_results_set_unit(tool->results, "z", siunitz);
-    tool->same_units = gwy_si_unit_equal(siunitxy, siunitz);
+    tool->same_units = gwy_unit_equal(siunitxy, siunitz);
 }
 
 static void
@@ -352,7 +310,7 @@ update_selected_rectangle(GwyToolStats *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
     GwySelection *selection = plain_tool->selection;
-    GwyDataField *field = plain_tool->data_field;
+    GwyField *field = plain_tool->field;
     gint n = selection ? gwy_selection_get_data(selection, NULL) : 0;
 
     gwy_rect_selection_labels_fill(tool->rlabels, n == 1 ? selection : NULL, field, NULL, tool->isel);
@@ -367,7 +325,7 @@ param_changed(GwyToolStats *tool, gint id)
     if (id == PARAM_INSTANT_UPDATE)
         do_update = do_update || gwy_params_get_boolean(params, PARAM_INSTANT_UPDATE);
     if (id == PARAM_MASKING)
-        do_update = do_update || (GWY_PLAIN_TOOL(tool)->data_field && GWY_PLAIN_TOOL(tool)->mask_field);
+        do_update = do_update || (GWY_PLAIN_TOOL(tool)->field && GWY_PLAIN_TOOL(tool)->mask_field);
     if (id < 0 || id == PARAM_INSTANT_UPDATE)
         update_sensitivity(tool);
     if (do_update)
@@ -384,18 +342,6 @@ static void
 gwy_tool_stats_data_changed(GwyPlainTool *plain_tool)
 {
     GwyToolStats *tool = GWY_TOOL_STATS(plain_tool);
-    GwyContainer *container = plain_tool->container;
-    gchar xukey[24], yukey[24], zukey[24];
-
-    g_snprintf(xukey, sizeof(xukey), "/%d/data/cal_xunc", plain_tool->id);
-    g_snprintf(yukey, sizeof(yukey), "/%d/data/cal_yunc", plain_tool->id);
-    g_snprintf(zukey, sizeof(zukey), "/%d/data/cal_zunc", plain_tool->id);
-
-    tool->has_calibration = FALSE;
-    if (gwy_container_gis_object_by_name(container, xukey, &tool->xunc)
-        && gwy_container_gis_object_by_name(container, yukey, &tool->yunc)
-        && gwy_container_gis_object_by_name(container, zukey, &tool->zunc))
-        GWY_TOOL_STATS(plain_tool)->has_calibration = TRUE;
 
     update_selected_rectangle(tool);
     update_units(tool);
@@ -417,7 +363,7 @@ gwy_tool_stats_response(GwyTool *tool,
 {
     GWY_TOOL_CLASS(gwy_tool_stats_parent_class)->response(tool, response_id);
 
-    if (response_id == GWY_TOOL_RESPONSE_UPDATE)
+    if (response_id == GWY_RESPONSE_UPDATE)
         update_labels(GWY_TOOL_STATS(tool));
 }
 
@@ -446,7 +392,7 @@ update_labels(GwyToolStats *tool)
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
     guint i;
 
-    if (!plain_tool->data_field) {
+    if (!plain_tool->field) {
         for (i = 0; i < G_N_ELEMENTS(results_ids); i++)
             gwy_param_table_results_clear(tool->table_results, results_ids[i]);
         return;
@@ -463,11 +409,11 @@ update_labels(GwyToolStats *tool)
 }
 
 static gdouble
-gwy_data_field_scan_line_discrepancy(GwyDataField *field,
-                                     GwyDataField *mask,
-                                     GwyMaskingType masking,
-                                     gint col, gint row,
-                                     gint width, gint height)
+gwy_field_scan_line_discrepancy(GwyField *field,
+                                GwyField *mask,
+                                GwyMaskingType masking,
+                                gint col, gint row,
+                                gint width, gint height)
 {
     gint xres = field->xres, yres = field->yres, i, j, n;
     const gdouble *drow, *drowp, *drown, *mrow;
@@ -483,14 +429,16 @@ gwy_data_field_scan_line_discrepancy(GwyDataField *field,
         return 0.0;
 
     n = 0;
+    const gdouble *data = gwy_field_get_data_const(field);
+    const gdouble *mdata = mask ? gwy_field_get_data_const(mask) : NULL;
     for (i = 0; i < height; i++) {
         /* Take the two neighbour rows, except for the first and last one, where we simply calculate the difference
          * from the sole neigbour. */
-        drow = field->data + (row + i)*xres + col;
+        drow = data + (row + i)*xres + col;
         drowp = (row + i > 0) ? drow - xres : drow + xres;
         drown = (row + i < yres-1) ? drow + xres : drow - xres;
         if (masking == GWY_MASK_INCLUDE) {
-            mrow = mask->data + (row + i)*xres + col;
+            mrow = mdata + (row + i)*xres + col;
             for (j = 0; j < width; j++) {
                 if (mrow[j] > 0.0) {
                     v = drow[j] - 0.5*(drowp[j] + drown[j]);
@@ -500,7 +448,7 @@ gwy_data_field_scan_line_discrepancy(GwyDataField *field,
             }
         }
         else if (masking == GWY_MASK_EXCLUDE) {
-            mrow = mask->data + (row + i)*xres + col;
+            mrow = mdata + (row + i)*xres + col;
             for (j = 0; j < width; j++) {
                 if (mrow[j] <= 0.0) {
                     v = drow[j] - 0.5*(drowp[j] + drown[j]);
@@ -525,11 +473,10 @@ static gboolean
 calculate(GwyToolStats *tool)
 {
     GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
-    GwyDataField *field = plain_tool->data_field;
-    GwyDataField *mask = plain_tool->mask_field;
+    GwyField *field = plain_tool->field;
+    GwyField *mask = plain_tool->mask_field;
     GwyMaskingType masking = gwy_params_get_masking(tool->params, PARAM_MASKING, &mask);
     GwyResults *results = tool->results;
-    StatsUncertanties unc;
     gdouble xoff, yoff, q;
     gdouble min, max, avg, median, Sa, rms, rms_gw, skew, kurtosis;
     gdouble projarea, area = 0.0, Sdq, volume, var, phi = 0.0, theta = 0.0, linedis;
@@ -547,15 +494,15 @@ calculate(GwyToolStats *tool)
     if (!w || !h)
         return FALSE;
 
-    xoff = gwy_data_field_get_xoffset(field);
-    yoff = gwy_data_field_get_yoffset(field);
+    xoff = gwy_field_get_xoffset(field);
+    yoff = gwy_field_get_yoffset(field);
 
-    q = gwy_data_field_get_dx(field) * gwy_data_field_get_dy(field);
+    q = gwy_field_get_dx(field) * gwy_field_get_dy(field);
     if (mask) {
         if (masking == GWY_MASK_INCLUDE)
-            gwy_data_field_area_count_in_range(mask, NULL, col, row, w, h, 0.0, 0.0, &nn, NULL);
+            gwy_field_area_count_in_range(mask, NULL, GWY_MASK_IGNORE, col, row, w, h, 0.0, 0.0, &nn, NULL);
         else
-            gwy_data_field_area_count_in_range(mask, NULL, col, row, w, h, 1.0, 1.0, NULL, &nn);
+            gwy_field_area_count_in_range(mask, NULL, GWY_MASK_IGNORE, col, row, w, h, 1.0, 1.0, NULL, &nn);
         nn = w*h - nn;
     }
     else
@@ -563,27 +510,26 @@ calculate(GwyToolStats *tool)
     projarea = nn * q;
     /* TODO: do something more reasonable when nn == 0 */
 
-    gwy_data_field_area_get_min_max_mask(field, mask, masking, col, row, w, h, &min, &max);
-    gwy_data_field_area_get_stats_mask(field, mask, masking, col, row, w, h, &avg, &Sa, &rms, &skew, &kurtosis);
-    rms_gw = gwy_data_field_area_get_grainwise_rms(field, mask, masking, col, row, w, h);
-    median = gwy_data_field_area_get_median_mask(field, mask, masking, col, row, w, h);
-    var = gwy_data_field_area_get_variation(field, mask, masking, col, row, w, h);
-    Sdq = gwy_data_field_area_get_surface_slope_mask(field, mask, masking, col, row, w, h);
+    gwy_field_area_get_min_max(field, mask, masking, col, row, w, h, &min, &max);
+    gwy_field_area_get_stats(field, mask, masking, col, row, w, h, &avg, &Sa, &rms, &skew, &kurtosis);
+    /* TODO Nield: grainwise rms only makes sense in INCLUDE mode. The new function does not have modes. */
+    rms_gw = gwy_field_area_get_grainwise_rms(field, mask, masking, col, row, w, h);
+    median = gwy_field_area_get_median(field, mask, masking, col, row, w, h);
+    var = gwy_field_area_get_variation(field, mask, masking, col, row, w, h);
+    Sdq = gwy_field_area_get_surface_slope(field, mask, masking, col, row, w, h);
 
-    linedis = gwy_data_field_scan_line_discrepancy(field, mask, masking, col, row, w, h);
+    linedis = gwy_field_scan_line_discrepancy(field, mask, masking, col, row, w, h);
     if (linedis > 0.0)
-        linedis /= sqrt(gwy_data_field_area_get_mean_square(field, mask, masking, col, row, w, h));
+        linedis /= sqrt(gwy_field_area_get_mean_square(field, mask, masking, col, row, w, h));
 
     if (tool->same_units)
-        area = gwy_data_field_area_get_surface_area_mask(field, mask, masking, col, row, w, h);
-    volume = gwy_data_field_area_get_volume(field, NULL, mask, col, row, w, h);
-    if (masking == GWY_MASK_EXCLUDE)
-        volume = gwy_data_field_area_get_volume(field, NULL, NULL, col, row, w, h) - volume;
+        area = gwy_field_area_get_surface_area(field, mask, masking, col, row, w, h);
+    volume = gwy_field_area_get_volume(field, NULL, mask, masking, col, row, w, h);
 
     if (tool->same_units) {
         gdouble nx, ny, nz;
-        if ((have_inclination = gwy_data_field_area_get_normal_coeffs_mask(field, mask, masking, col, row, w, h,
-                                                                           &nx, &ny, &nz))) {
+        if ((have_inclination = gwy_field_area_get_normal_coeffs(field, mask, masking, col, row, w, h,
+                                                                 &nx, &ny, &nz))) {
             theta = atan2(hypot(nx, ny), nz);
             /* Inclination is upslope, not down. It is not the normal. */
             phi = atan2(ny, nx) + G_PI;
@@ -618,78 +564,30 @@ calculate(GwyToolStats *tool)
     if (area < 120.0*projarea)
         gwy_results_bind_formats(results, "area", "projarea", NULL);
 
-    if (tool->has_calibration) {
-        calculate_uncertainties(tool, &unc, field, mask, masking, nn, col, row, w, h);
-        gwy_results_fill_values_with_errors(results,
-                                            "avg", avg, unc.avg,
-                                            "Sa", Sa, unc.Sa,
-                                            "rms", rms, unc.rms,
-                                            "skew", skew, unc.skew,
-                                            "kurtosis", kurtosis, unc.kurtosis,
-                                            "projarea", projarea, unc.projarea,
-                                            "phi", phi, unc.phi,
-                                            "theta", theta, unc.theta,
-                                            NULL);
-    }
-    else {
-        gwy_results_fill_values(results,
-                                "avg", avg,
-                                "Sa", Sa,
-                                "rms", rms,
-                                "skew", skew,
-                                "kurtosis", kurtosis,
-                                "projarea", projarea,
-                                "phi", phi,
-                                "theta", theta,
-                                NULL);
-    }
+    gwy_results_fill_values(results,
+                            "avg", avg,
+                            "Sa", Sa,
+                            "rms", rms,
+                            "skew", skew,
+                            "kurtosis", kurtosis,
+                            "projarea", projarea,
+                            "phi", phi,
+                            "theta", theta,
+                            NULL);
+
     if (!have_inclination)
         gwy_results_set_na(results, "phi", "theta", NULL);
     if (!tool->same_units)
         gwy_results_set_na(tool->results, "area", "theta", "phi", NULL);
 
-    gwy_results_fill_filename(results, "file", plain_tool->container);
-    gwy_results_fill_channel(results, "image", plain_tool->container, plain_tool->id);
+    gwy_results_fill_filename(results, "file", GWY_FILE(plain_tool->container));
+    gwy_results_fill_data_name(results, "image", GWY_FILE(plain_tool->container), GWY_FILE_IMAGE, plain_tool->id);
 
     gwy_param_table_set_sensitive(tool->table_results, PARAM_REPORT_STYLE, TRUE);
 
     return TRUE;
 }
 
-static void
-calculate_uncertainties(GwyToolStats *tool, StatsUncertanties *unc,
-                        GwyDataField *field, GwyDataField *mask,
-                        GwyMaskingType masking, guint nn,
-                        gint col, gint row, gint w, gint h)
-{
-    gint xres, yres, oldx, oldy;
-
-    g_return_if_fail(tool->has_calibration);
-
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
-    oldx = gwy_data_field_get_xres(tool->xunc);
-    oldy = gwy_data_field_get_yres(tool->xunc);
-    // FIXME, functions should work with data of any size
-    gwy_data_field_resample(tool->xunc, xres, yres, GWY_INTERPOLATION_BILINEAR);
-    gwy_data_field_resample(tool->yunc, xres, yres, GWY_INTERPOLATION_BILINEAR);
-    gwy_data_field_resample(tool->zunc, xres, yres, GWY_INTERPOLATION_BILINEAR);
-
-    unc->projarea = gwy_data_field_area_get_projected_area_uncertainty(nn, tool->xunc, tool->yunc);
-
-    gwy_data_field_area_get_stats_uncertainties_mask(field, tool->zunc, mask, masking, col, row, w, h,
-                                                     &unc->avg, &unc->Sa, &unc->rms, &unc->skew, &unc->kurtosis);
-
-    if (tool->same_units && !mask) {
-        gwy_data_field_area_get_inclination_uncertainty(field, tool->zunc, tool->xunc, tool->yunc, col, row, w, h,
-                                                        &unc->theta, &unc->phi);
-    }
-
-    gwy_data_field_resample(tool->xunc, oldx, oldy, GWY_INTERPOLATION_BILINEAR);
-    gwy_data_field_resample(tool->yunc, oldx, oldy, GWY_INTERPOLATION_BILINEAR);
-    gwy_data_field_resample(tool->zunc, oldx, oldy, GWY_INTERPOLATION_BILINEAR);
-}
-
 static GwyResults*
 create_results(void)
 {
