@@ -1,6 +1,6 @@
 /* This is a GENERATED file. */
 #include <glib.h>
-#include <libgwymodule/gwymodule.h>
+#include <libgwyapp/gwyapp.h>
 
 static const GwyModuleRecord* register_bundle(void);
 
@@ -12,43 +12,40 @@ static GwyModuleInfo module_info = {
 
 GWY_MODULE_QUERY(module_info)
 
-GwyModuleInfo* _gwy_module_query__volume_1dfft(void);
-GwyModuleInfo* _gwy_module_query__volume_addframe(void);
-GwyModuleInfo* _gwy_module_query__volume_arithmetic(void);
-GwyModuleInfo* _gwy_module_query__volume_asciiexport(void);
-GwyModuleInfo* _gwy_module_query__volume_calibrate(void);
-GwyModuleInfo* _gwy_module_query__volume_cluster(void);
-GwyModuleInfo* _gwy_module_query__volume_crop(void);
-GwyModuleInfo* _gwy_module_query__volume_drift(void);
-GwyModuleInfo* _gwy_module_query__volume_equiplane(void);
-GwyModuleInfo* _gwy_module_query__volume_extract(void);
-GwyModuleInfo* _gwy_module_query__volume_facetlevel(void);
-GwyModuleInfo* _gwy_module_query__volume_fdfit(void);
-GwyModuleInfo* _gwy_module_query__volume_flattenbase(void);
-GwyModuleInfo* _gwy_module_query__volume_imagize(void);
-GwyModuleInfo* _gwy_module_query__volume_invert(void);
-GwyModuleInfo* _gwy_module_query__volume_lawnize(void);
-GwyModuleInfo* _gwy_module_query__volume_linelevel(void);
-GwyModuleInfo* _gwy_module_query__volume_linestat(void);
-GwyModuleInfo* _gwy_module_query__volume_localdetect(void);
+GwyModuleInfo* _gwy_module_query__volume_1dfft         (void);
+GwyModuleInfo* _gwy_module_query__volume_addframe      (void);
+GwyModuleInfo* _gwy_module_query__volume_arithmetic    (void);
+GwyModuleInfo* _gwy_module_query__volume_asciiexport   (void);
+GwyModuleInfo* _gwy_module_query__volume_calibrate     (void);
+GwyModuleInfo* _gwy_module_query__volume_cluster       (void);
+GwyModuleInfo* _gwy_module_query__volume_crop          (void);
+GwyModuleInfo* _gwy_module_query__volume_drift         (void);
+GwyModuleInfo* _gwy_module_query__volume_equiplane     (void);
+GwyModuleInfo* _gwy_module_query__volume_facetlevel    (void);
+GwyModuleInfo* _gwy_module_query__volume_fdfit         (void);
+GwyModuleInfo* _gwy_module_query__volume_flattenbase   (void);
+GwyModuleInfo* _gwy_module_query__volume_lawnize       (void);
+GwyModuleInfo* _gwy_module_query__volume_linelevel     (void);
+GwyModuleInfo* _gwy_module_query__volume_linestat      (void);
+GwyModuleInfo* _gwy_module_query__volume_localdetect   (void);
 GwyModuleInfo* _gwy_module_query__volume_meanplanelevel(void);
-GwyModuleInfo* _gwy_module_query__volume_mfmrecalc(void);
-GwyModuleInfo* _gwy_module_query__volume_outliers(void);
-GwyModuleInfo* _gwy_module_query__volume_planelevel(void);
-GwyModuleInfo* _gwy_module_query__volume_planestat(void);
-GwyModuleInfo* _gwy_module_query__volume_psf(void);
-GwyModuleInfo* _gwy_module_query__volume_removeframe(void);
-GwyModuleInfo* _gwy_module_query__volume_rephase(void);
-GwyModuleInfo* _gwy_module_query__volume_scars(void);
-GwyModuleInfo* _gwy_module_query__volume_slice(void);
-GwyModuleInfo* _gwy_module_query__volume_stepline(void);
-GwyModuleInfo* _gwy_module_query__volume_strayfield(void);
-GwyModuleInfo* _gwy_module_query__volume_swaxes(void);
-GwyModuleInfo* _gwy_module_query__volume_xyarithmetics(void);
-GwyModuleInfo* _gwy_module_query__volume_xystitch(void);
-GwyModuleInfo* _gwy_module_query__volume_zcal(void);
-GwyModuleInfo* _gwy_module_query__volume_zposlevel(void);
-GwyModuleInfo* _gwy_module_query__volumeops(void);
+GwyModuleInfo* _gwy_module_query__volume_mfmrecalc     (void);
+GwyModuleInfo* _gwy_module_query__volume_outliers      (void);
+GwyModuleInfo* _gwy_module_query__volume_planelevel    (void);
+GwyModuleInfo* _gwy_module_query__volume_planestat     (void);
+GwyModuleInfo* _gwy_module_query__volume_psf           (void);
+GwyModuleInfo* _gwy_module_query__volume_removeframe   (void);
+GwyModuleInfo* _gwy_module_query__volume_rephase       (void);
+GwyModuleInfo* _gwy_module_query__volume_scars         (void);
+GwyModuleInfo* _gwy_module_query__volume_slice         (void);
+GwyModuleInfo* _gwy_module_query__volume_stepline      (void);
+GwyModuleInfo* _gwy_module_query__volume_strayfield    (void);
+GwyModuleInfo* _gwy_module_query__volume_swaxes        (void);
+GwyModuleInfo* _gwy_module_query__volume_xyarithmetic  (void);
+GwyModuleInfo* _gwy_module_query__volume_xystitch      (void);
+GwyModuleInfo* _gwy_module_query__volume_zcal          (void);
+GwyModuleInfo* _gwy_module_query__volume_zposlevel     (void);
+GwyModuleInfo* _gwy_module_query__volumeops            (void);
 
 static const GwyModuleRecord modules[] = {
   { _gwy_module_query__volume_1dfft, "volume_1dfft", },
@@ -60,12 +57,9 @@ static const GwyModuleRecord modules[] = {
   { _gwy_module_query__volume_crop, "volume_crop", },
   { _gwy_module_query__volume_drift, "volume_drift", },
   { _gwy_module_query__volume_equiplane, "volume_equiplane", },
-  { _gwy_module_query__volume_extract, "volume_extract", },
   { _gwy_module_query__volume_facetlevel, "volume_facetlevel", },
   { _gwy_module_query__volume_fdfit, "volume_fdfit", },
   { _gwy_module_query__volume_flattenbase, "volume_flattenbase", },
-  { _gwy_module_query__volume_imagize, "volume_imagize", },
-  { _gwy_module_query__volume_invert, "volume_invert", },
   { _gwy_module_query__volume_lawnize, "volume_lawnize", },
   { _gwy_module_query__volume_linelevel, "volume_linelevel", },
   { _gwy_module_query__volume_linestat, "volume_linestat", },
@@ -83,7 +77,7 @@ static const GwyModuleRecord modules[] = {
   { _gwy_module_query__volume_stepline, "volume_stepline", },
   { _gwy_module_query__volume_strayfield, "volume_strayfield", },
   { _gwy_module_query__volume_swaxes, "volume_swaxes", },
-  { _gwy_module_query__volume_xyarithmetics, "volume_xyarithmetics", },
+  { _gwy_module_query__volume_xyarithmetic, "volume_xyarithmetic", },
   { _gwy_module_query__volume_xystitch, "volume_xystitch", },
   { _gwy_module_query__volume_zcal, "volume_zcal", },
   { _gwy_module_query__volume_zposlevel, "volume_zposlevel", },
