@@ -1,5 +1,5 @@
 /*
- *  $Id: volume_1dfft.c 26643 2024-09-19 16:50:43Z klapetek $
+ *  $Id: volume_1dfft.c 29023 2025-12-19 13:06:16Z yeti-dn $
  *  Copyright (C) 2015-2023 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,26 +16,11 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwythreads.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwynlfit.h>
-#include <libprocess/brick.h>
-#include <libprocess/stats.h>
-#include <libprocess/linestats.h>
-#include <libprocess/gwyprocess.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libprocess/correlation.h>
-#include <libgwydgets/gwydataview.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwydgets/gwydgetutils.h>
-#include <libgwymodule/gwymodule-volume.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
-#include "libgwyddion/gwyomp.h"
+#include <gwy.h>
+#include "libgwyddion/omp.h"
 
 #define RUN_MODES (GWY_RUN_INTERACTIVE)
 
@@ -66,48 +51,46 @@ enum {
     PARAM_DIRECTION,
     PARAM_INTERPOLATION,
     PARAM_MODULUS
-//    PARAM_UPDATE
 };
 
-
 typedef struct {
     GwyParams *params;
     GwyBrick *brick;
     GwyBrick *result;
-    GwyDataLine *modulus;
+    GwyLine *modulus;
 } ModuleArgs;
 
 typedef struct {
     ModuleArgs *args;
     GtkWidget *dialog;
     GwyParamTable *table_options;
-    GwyContainer *data;
     GwyGraphModel *gmodel;
+    GtkWidget *dataview;
+    GtkWidget *resultview;
     GwySelection *selection;
     gboolean computed;
 } ModuleGUI;
 
-
-static gboolean              module_register          (void);
-static GwyParamDef*          define_module_params     (void);
-static void                  volfftf_1d               (GwyContainer *data,
-                                                       GwyRunType runtype);
-static GwyDialogOutcome      run_gui                  (ModuleArgs *args,
-                                                       GwyContainer *data,
-                                                       gint id);
-static void                  param_changed            (ModuleGUI *gui,
-                                                       gint id);
-static void                  dialog_response          (GwyDialog *dialog,
-                                                       gint response,
-                                                       ModuleGUI *gui);
-static void                  preview                  (gpointer user_data);
-static void                  update_image             (ModuleGUI *gui,
-                                                       gint z);
-static void                  graph_selected           (ModuleGUI *gui);
-static void                  ensure_modulus           (ModuleArgs *args);
-static void                  plot_modulus             (ModuleGUI *gui);
-static GwyDataLine*          calculate_weights        (ModuleArgs *args,
-                                                       GwySelection *selection);
+static gboolean         module_register     (void);
+static GwyParamDef*     define_module_params(void);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
+static GwyDialogOutcome run_gui             (ModuleArgs *args,
+                                             GwyFile *data,
+                                             gint id);
+static void             param_changed       (ModuleGUI *gui,
+                                             gint id);
+static void             dialog_response     (GwyDialog *dialog,
+                                             gint response,
+                                             ModuleGUI *gui);
+static void             preview             (gpointer user_data);
+static void             update_image        (ModuleGUI *gui,
+                                             gint z);
+static void             graph_selected      (ModuleGUI *gui);
+static void             ensure_modulus      (ModuleArgs *args);
+static void             plot_modulus        (ModuleGUI *gui);
+static GwyLine*         calculate_weights   (ModuleArgs *args,
+                                             GwySelection *selection);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
@@ -125,7 +108,7 @@ static gboolean
 module_register(void)
 {
     gwy_volume_func_register("volume_1dfft",
-                             (GwyVolumeFunc)&volfftf_1d,
+                             module_main,
                              N_("/_Correct Data/1D _FFT Filtering..."),
                              NULL,
                              RUN_MODES,
@@ -147,8 +130,8 @@ define_module_params(void)
         { N_("Suppress"),  SUPPRESS_NEIGBOURHOOD, },
     };
     static const GwyEnum modulus[] = {
-        { N_("spectra|Individual"), MODULUS_INDIVIDUAL, },
-        { N_("spectra|Average"),    MODULUS_AVERAGE,    },
+        { gwy_NC("spectra", "Individual"), MODULUS_INDIVIDUAL, },
+        { gwy_NC("spectra", "Average"),    MODULUS_AVERAGE,    },
     };
 
     static GwyParamDef *paramdef = NULL;
@@ -168,32 +151,30 @@ define_module_params(void)
                            GWY_ORIENTATION_HORIZONTAL);
     gwy_param_def_add_enum(paramdef, PARAM_INTERPOLATION, "interpolation", NULL, GWY_TYPE_INTERPOLATION_TYPE,
                            GWY_INTERPOLATION_LINEAR);
-//    gwy_param_def_add_instant_updates(paramdef, PARAM_UPDATE, "update", NULL, TRUE);
     gwy_param_def_add_int(paramdef, PARAM_Z, "z", "Preview level", 0, G_MAXINT, 0);
     return paramdef;
 }
 
 static void
-volfftf_1d(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
     GwyBrick *brick = NULL;
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     gint oldid, newid;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    g_return_if_fail(g_type_from_name("GwyLayerPoint"));
+    g_return_if_fail(mode & RUN_MODES);
 
-    gwy_app_data_browser_get_current(GWY_APP_BRICK, &brick,
-                                     GWY_APP_BRICK_ID, &oldid,
-                                     0);
+    gwy_data_browser_get_current(GWY_APP_BRICK, &brick,
+                                 GWY_APP_BRICK_ID, &oldid,
+                                 0);
     g_return_if_fail(GWY_IS_BRICK(brick));
     args.result = NULL;
     args.modulus = NULL;
     args.brick = brick;
     args.params = gwy_params_new_from_settings(define_module_params());
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, oldid);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -202,25 +183,24 @@ volfftf_1d(GwyContainer *data, GwyRunType runtype)
     if (outcome != GWY_DIALOG_HAVE_RESULT)
         goto end;
 
-    newid = gwy_app_data_browser_add_brick(args.result, NULL, data, TRUE);
-
-    gwy_app_set_brick_title(data, newid, _("1D FFT Filtered"));
-    gwy_app_sync_volume_items(data, data, oldid, newid, FALSE,
-                              GWY_DATA_ITEM_GRADIENT,
-                              0);
-
-    gwy_app_volume_log_add_volume(data, -1, newid);
+    newid = gwy_file_add_volume(data, args.result, NULL);
+    gwy_file_set_visible(data, GWY_FILE_VOLUME, newid, TRUE);
+    gwy_file_set_title(data, GWY_FILE_VOLUME, newid, _("1D FFT Filtered"), TRUE);
+    gwy_file_sync_items(data, GWY_FILE_VOLUME, oldid,
+                        data, GWY_FILE_VOLUME, newid,
+                        GWY_FILE_ITEM_PALETTE, FALSE);
+    gwy_log_add(data, GWY_FILE_VOLUME, -1, newid);
 
 end:
-    GWY_OBJECT_UNREF(args.modulus);
+    g_clear_object(&args.modulus);
     g_object_unref(args.params);
     g_object_unref(args.result);
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
-    GtkWidget *hbox, *dataview, *align;
+    GtkWidget *hbox, *dataview, *widget;
     GwyParamTable *table;
     GwyDialog *dialog;
     GwyGraph *graph;
@@ -228,47 +208,32 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     ModuleGUI gui;
     GwyDialogOutcome outcome;
     GwyBrick *brick = args->brick;
-    GwyDataField *field = gwy_data_field_new(gwy_brick_get_xres(brick),
-                                             gwy_brick_get_yres(brick),
-                                             gwy_brick_get_xreal(brick),
-                                             gwy_brick_get_yreal(brick),
-                                             TRUE);
-    GwyDataField *result = gwy_data_field_new_alike(field, TRUE);
+    GwyField *field = gwy_brick_new_field_like_xy_plane(brick, TRUE);
+    GwyField *result = gwy_field_new_alike(field, TRUE);
 
-    const guchar *gradient;
-
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
-    gui.data = gwy_container_new();
     gui.computed = FALSE;
 
-    args->result = gwy_brick_duplicate(brick);
-
-    gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(0), field);
-    if (gwy_container_gis_string(data, gwy_app_get_brick_palette_key_for_id(id), &gradient))
-        gwy_container_set_const_string(gui.data, gwy_app_get_data_palette_key_for_id(0), gradient);
-
-    gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(1), result);
-    if (gwy_container_gis_string(data, gwy_app_get_brick_palette_key_for_id(id), &gradient))
-        gwy_container_set_const_string(gui.data, gwy_app_get_data_palette_key_for_id(1), gradient);
-
+    args->result = gwy_brick_copy(brick);
 
     gui.dialog = gwy_dialog_new(_("1D FFT Filter"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_CLEAR, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-
-    hbox = gwy_hbox_new(0);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
     gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
     gwy_dialog_add_content(dialog, hbox, FALSE, FALSE, 0);
 
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
+    dataview = gui.dataview = gwy_create_preview(field, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_VOLUME, id, GWY_FILE_ITEM_PALETTE);
     gtk_box_pack_start(GTK_BOX(hbox), dataview, FALSE, FALSE, 4);
 
-    dataview = gwy_create_preview(gui.data, 1, PREVIEW_SIZE, FALSE);
+    dataview = gui.resultview = gwy_create_preview(result, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_VOLUME, id, GWY_FILE_ITEM_PALETTE);
     gtk_box_pack_start(GTK_BOX(hbox), dataview, FALSE, FALSE, 4);
 
-    hbox = gwy_hbox_new(0);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
     gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
     gwy_dialog_add_content(dialog, hbox, FALSE, FALSE, 0);
 
@@ -293,50 +258,38 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_param_table_append_combo(table, PARAM_OUTPUT);
     gwy_param_table_append_combo(table, PARAM_INTERPOLATION);
     gwy_param_table_append_combo(table, PARAM_MODULUS);
-//    gwy_param_table_append_checkbox(table, PARAM_UPDATE);
 
-    align = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-    gtk_container_add(GTK_CONTAINER(align), gwy_param_table_widget(table));
-
-    gtk_box_pack_start(GTK_BOX(hbox), align, FALSE, FALSE, 0);
+    widget = gwy_param_table_widget(table);
+    gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 0);
     gwy_dialog_add_param_table(dialog, table);
 
-
     g_signal_connect_swapped(gui.table_options, "param-changed", G_CALLBACK(param_changed), &gui);
     g_signal_connect_after(dialog, "response", G_CALLBACK(dialog_response), &gui);
     gwy_dialog_set_preview_func(dialog, GWY_PREVIEW_IMMEDIATE, preview, &gui, NULL);
 
     outcome = gwy_dialog_run(dialog);
 
-    g_object_unref(gui.data);
     g_object_unref(gui.gmodel);
 
     return outcome;
 }
 
-
 static void
 param_changed(ModuleGUI *gui, gint id)
 {
-    GwyParamTable *table = gui->table_options;
-    GwyParams *params = gui->args->params;
     GwyModulusType mtype = gwy_params_get_enum(gui->args->params, PARAM_MODULUS);
 
-    if (id < 0 || id == PARAM_DIRECTION || id == PARAM_MODULUS
-        || (id == PARAM_Z && mtype == MODULUS_INDIVIDUAL)) {
-        GWY_OBJECT_UNREF(gui->args->modulus);
+    if (id < 0 || id == PARAM_DIRECTION || id == PARAM_MODULUS || (id == PARAM_Z && mtype == MODULUS_INDIVIDUAL)) {
+        g_clear_object(&gui->args->modulus);
         if (id < 0 || id == PARAM_DIRECTION)
             gwy_selection_clear(gui->selection);
         ensure_modulus(gui->args);
         plot_modulus(gui);
         gui->computed = FALSE;
     }
-    if (id < 0 || id == PARAM_OUTPUT || id == PARAM_INTERPOLATION
-        || id == PARAM_SUPPRESS) {
+    if (id < 0 || id == PARAM_OUTPUT || id == PARAM_INTERPOLATION || id == PARAM_SUPPRESS)
         gui->computed = FALSE;
-    }
 
-    //if (id != PARAM_UPDATE)
     gwy_dialog_invalidate(GWY_DIALOG(gui->dialog));
 }
 
@@ -347,7 +300,7 @@ dialog_response(G_GNUC_UNUSED GwyDialog *dialog, gint response, ModuleGUI *gui)
         gwy_selection_clear(gui->selection);
 
     if (response == GWY_RESPONSE_RESET)
-        gwy_brick_copy(gui->args->brick, gui->args->result, FALSE);
+        gwy_brick_copy_data(gui->args->brick, gui->args->result);
 
     preview(gui);
 }
@@ -355,18 +308,18 @@ dialog_response(G_GNUC_UNUSED GwyDialog *dialog, gint response, ModuleGUI *gui)
 static void
 update_image(ModuleGUI *gui, gint z)
 {
-    GwyDataField *dfield;
-    dfield = gwy_container_get_object(gui->data, gwy_app_get_data_key_for_id(0));
-    gwy_brick_extract_xy_plane(gui->args->brick, dfield, CLAMP(z, 0, gui->args->brick->zres-1));
-    gwy_data_field_data_changed(dfield);
+    ModuleArgs *args = gui->args;
+    gint zres = gwy_brick_get_zres(args->brick);
 
-    dfield = gwy_container_get_object(gui->data, gwy_app_get_data_key_for_id(1));
-    gwy_brick_extract_xy_plane(gui->args->result, dfield, CLAMP(z, 0, gui->args->result->zres-1));
+    GwyField *field = gwy_data_view_get_field(GWY_DATA_VIEW(gui->dataview));
+    gwy_brick_extract_xy_plane(args->brick, field, CLAMP(z, 0, zres-1));
+    gwy_field_data_changed(field);
 
-    gwy_data_field_data_changed(dfield);
+    GwyField *result = gwy_data_view_get_field(GWY_DATA_VIEW(gui->resultview));
+    gwy_brick_extract_xy_plane(args->result, result, CLAMP(z, 0, zres-1));
+    gwy_field_data_changed(result);
 }
 
-
 static void
 preview(gpointer user_data)
 {
@@ -375,25 +328,23 @@ preview(gpointer user_data)
     gint z = gwy_params_get_int(gui->args->params, PARAM_Z);
     GwyOrientation direction = gwy_params_get_enum(args->params, PARAM_DIRECTION);
     GwyInterpolationType interpolation = gwy_params_get_enum(args->params, PARAM_INTERPOLATION);
-    GwyDataLine *weights;
-    gint k, xres, yres, zres;
+    GwyLine *weights;
+    gint k, zres;
     GwyBrick *brick = args->brick;
     GwyBrick *result = args->result;
-    GwyDataField *dfield, *myfield;
+    GwyField *dfield, *myfield;
 
     if (!gui->computed) {
-       xres = gwy_brick_get_xres(brick);
-       yres = gwy_brick_get_yres(brick);
        zres = gwy_brick_get_zres(brick);
 
-       dfield = gwy_data_field_new(xres, yres, gwy_brick_get_xreal(brick), gwy_brick_get_yreal(brick), FALSE);
-       myfield = gwy_data_field_new_alike(dfield, FALSE);
+       dfield = gwy_brick_new_field_like_xy_plane(brick, FALSE);
+       myfield = gwy_field_new_alike(dfield, FALSE);
        ensure_modulus(args);
 
        weights = calculate_weights(args, gui->selection);
        for (k = 0; k < zres; k++) {
            gwy_brick_extract_xy_plane(brick, dfield, k);
-           gwy_data_field_fft_filter_1d(dfield, myfield, weights, direction, interpolation);
+           gwy_field_fft_filter_1d(dfield, myfield, weights, direction, interpolation);
            gwy_brick_set_xy_plane(result, myfield, k);
        }
        gui->computed = TRUE;
@@ -422,8 +373,8 @@ ensure_modulus(ModuleArgs *args)
     GwyBrick *brick = args->brick;
     gint z = gwy_params_get_int(args->params, PARAM_Z);
     GwyModulusType mtype = gwy_params_get_enum(args->params, PARAM_MODULUS);
-    GwyDataField *dfield;
-    GwyDataLine *modulus = NULL, *levmodulus;
+    GwyField *dfield;
+    GwyLine *modulus = NULL, *levmodulus;
     gint i, k, res, zres;
     gdouble m;
     gdouble *d;
@@ -431,32 +382,32 @@ ensure_modulus(ModuleArgs *args)
     if (args->modulus)
         return;
 
-    dfield = gwy_data_field_new(gwy_brick_get_xres(brick), gwy_brick_get_yres(brick),
-                                gwy_brick_get_xreal(brick), gwy_brick_get_yreal(brick), FALSE);
+    dfield = gwy_brick_new_field_like_xy_plane(brick, FALSE);
 
     if (mtype == MODULUS_INDIVIDUAL) {
-        modulus = args->modulus = gwy_data_line_new(1, 1.0, FALSE);
+        modulus = args->modulus = gwy_line_new(1, 1.0, FALSE);
         gwy_brick_extract_xy_plane(brick, dfield, z);
-        gwy_data_field_psdf(dfield, modulus, direction, GWY_INTERPOLATION_LINEAR, GWY_WINDOWING_RECT, -1);
-    } else {
-        levmodulus = gwy_data_line_new(1, 1.0, FALSE);
+        gwy_field_psdf(dfield, modulus, direction, GWY_WINDOWING_RECT);
+    }
+    else {
+        levmodulus = gwy_line_new(1, 1.0, FALSE);
         zres = gwy_brick_get_zres(brick);
         for (k = 0; k < zres; k++) {
             gwy_brick_extract_xy_plane(brick, dfield, k);
-            gwy_data_field_psdf(dfield, levmodulus, direction, GWY_INTERPOLATION_LINEAR, GWY_WINDOWING_RECT, -1);
+            gwy_field_psdf(dfield, levmodulus, direction, GWY_WINDOWING_RECT);
             if (k == 0)
-                modulus = args->modulus = gwy_data_line_new_alike(levmodulus, TRUE);
-            gwy_data_line_sum_lines(modulus, modulus, levmodulus);
+                modulus = args->modulus = gwy_line_new_alike(levmodulus, TRUE);
+            gwy_line_sum_lines(modulus, modulus, levmodulus);
         }
-        gwy_data_line_multiply(modulus, 1.0/(gdouble)zres);
+        gwy_line_multiply(modulus, 1.0/(gdouble)zres);
         g_object_unref(levmodulus);
     }
 
-    m = gwy_data_line_get_max(modulus);
+    m = gwy_line_max(modulus);
     if (!m)
         m = 1.0;
-    d = gwy_data_line_get_data(modulus);
-    res = gwy_data_line_get_res(modulus);
+    d = gwy_line_get_data(modulus);
+    res = gwy_line_get_res(modulus);
     for (i = 0; i < res; i++)
         d[i] = (d[i] > 0.0 ? sqrt(d[i]/m) : 0.0);
 
@@ -467,18 +418,18 @@ static void
 plot_modulus(ModuleGUI *gui)
 {
     GwyGraphCurveModel *cmodel;
-    GwyDataLine *modulus = gui->args->modulus;
+    GwyLine *modulus = gui->args->modulus;
 
     gwy_graph_model_remove_all_curves(gui->gmodel);
 
     cmodel = gwy_graph_curve_model_new();
-    gwy_graph_curve_model_set_data_from_dataline(cmodel, modulus, 0, 0);
+    gwy_graph_curve_model_set_data_from_line(cmodel, modulus, 0, 0);
     g_object_set(cmodel,
                  "mode", GWY_GRAPH_CURVE_LINE,
                  "description", _("FFT Modulus"),
                  NULL);
     g_object_set(gui->gmodel,
-                 "si-unit-x", gwy_data_line_get_si_unit_x(modulus),
+                 "unit-x", gwy_line_get_unit_x(modulus),
                  "axis-label-bottom", "k",
                  "axis-label-left", "",
                  NULL);
@@ -487,30 +438,32 @@ plot_modulus(ModuleGUI *gui)
     g_object_unref(cmodel);
 }
 
-static GwyDataLine*
+/* XXX: This function is identical among image 1D FFT filter, volume 1D FFT filter and XYZ FFT.
+ * Make a library function! */
+static GwyLine*
 calculate_weights(ModuleArgs *args, GwySelection *selection)
 {
     GwyFFTFilt1DSuppressType suppres = gwy_params_get_enum(args->params, PARAM_SUPPRESS);
     GwyFFTFilt1DOutputType output = gwy_params_get_enum(args->params, PARAM_OUTPUT);
-    GwyDataLine *weights, *modulus = args->modulus;
+    GwyLine *weights, *modulus = args->modulus;
     gint res, k, nsel;
     gdouble sel[2], *w;
     gint fill_from, fill_to;
 
-    res = gwy_data_line_get_res(modulus);
-    weights = gwy_data_line_new_alike(modulus, TRUE);
+    res = gwy_line_get_res(modulus);
+    weights = gwy_line_new_alike(modulus, TRUE);
 
     nsel = gwy_selection_get_data(selection, NULL);
     for (k = 0; k < nsel; k++) {
         gwy_selection_get_object(selection, k, sel);
         GWY_ORDER(gdouble, sel[0], sel[1]);
-        fill_from = MAX(0, gwy_data_line_rtoi(weights, sel[0]));
+        fill_from = MAX(0, gwy_line_rtoi(weights, sel[0]));
         fill_from = MIN(res, fill_from);
-        fill_to = MIN(res, gwy_data_line_rtoi(weights, sel[1]));
-        gwy_data_line_part_fill(weights, fill_from, fill_to, 1.0);
+        fill_to = MIN(res, gwy_line_rtoi(weights, sel[1]));
+        gwy_line_part_fill(weights, fill_from, fill_to - fill_from, 1.0);
     }
 
-    w = gwy_data_line_get_data(weights);
+    w = gwy_line_get_data(weights);
     if (output == OUTPUT_MARKED) {
         for (k = 0; k < res; k++)
             w[k] = 1.0 - w[k];
@@ -519,17 +472,17 @@ calculate_weights(ModuleArgs *args, GwySelection *selection)
     /* For Suppress, interpolate PSDF linearly between endpoints.  Since we pass weights to the filter, not PSDF
      * itself, we have to divide to get the weight. */
     if (suppres == SUPPRESS_NEIGBOURHOOD) {
-        GwyDataLine *buf = gwy_data_line_duplicate(modulus);
+        GwyLine *buf = gwy_line_copy(modulus);
         gdouble *b, *m;
 
         /*invert the weights as the laplace correction treats it as a mask*/
         for (k = 0; k < res; k++)
             w[k] = 1.0 - w[k];
 
-        gwy_data_line_correct_laplace(buf, weights);
-        b = gwy_data_line_get_data(buf);
-        m = gwy_data_line_get_data(modulus);
-        
+        gwy_line_correct_laplace(buf, weights);
+        b = gwy_line_get_data(buf);
+        m = gwy_line_get_data(modulus);
+
         for (k = 0; k < res; k++)
             w[k] = (m[k] > 0.0 ? fmin(b[k]/m[k], 1.0) : 0.0);
         g_object_unref(buf);
@@ -538,5 +491,4 @@ calculate_weights(ModuleArgs *args, GwySelection *selection)
     return weights;
 }
 
-
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
