@@ -1,6 +1,6 @@
 /*
- *  $Id: volume_arithmetic.c 26750 2024-10-18 15:41:48Z yeti-dn $
- *  Copyright (C) 2018-2023 David Necas (Yeti).
+ *  $Id: volume_arithmetic.c 28906 2025-11-24 16:45:11Z yeti-dn $
+ *  Copyright (C) 2018-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -18,24 +18,15 @@
 /* FIXME: Code is about 90% identical with volume_xyarithmetics.c. Merge! */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <glib/gstdio.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwyexpr.h>
-#include <libprocess/brick.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/correct.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwydgets/gwyradiobuttons.h>
-#include <libgwymodule/gwymodule-volume.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwymoduleutils-file.h>
+#include <gwy.h>
+#include "libgwyapp/sanity.h"
 #include "../process/preview.h"
 
-#define ARITH_RUN_MODES GWY_RUN_INTERACTIVE
+#define RUN_MODES GWY_RUN_INTERACTIVE
 
 enum {
     NARGS = 8,
@@ -80,7 +71,7 @@ typedef struct {
     GwyParams *params;
     GtkListStore *history;
     GwyBrick *result;
-    GwyDataField *preview;
+    GwyField *preview;
 } ModuleArgs;
 
 typedef struct {
@@ -102,13 +93,12 @@ typedef struct {
     GwyParamTable *table;
     GtkWidget *result;
     GSList *dataunits;
-    GwyContainer *data;
 } ModuleGUI;
 
 static gboolean         module_register   (void);
-static void             arithmetic        (GwyContainer *data,
-                                           GwyRunType run);
-static GwyDialogOutcome run_gui           (GwyContainer *data,
+static void             module_main       (GwyFile *data,
+                                           GwyRunModeFlags mode);
+static GwyDialogOutcome run_gui           (GwyFile *data,
                                            gint id,
                                            ModuleArgs *args,
                                            EvaluationData *evdata);
@@ -136,7 +126,7 @@ static GwyBrick*        make_x            (GwyBrick *brick);
 static GwyBrick*        make_y            (GwyBrick *brick);
 static GwyBrick*        make_z            (GwyBrick *brick);
 static GwyBrick*        make_zcal         (GwyBrick *brick,
-                                           GwyDataLine *zcal);
+                                           GwyLine *zcal);
 static void             sanitise_params   (ModuleArgs *args);
 static GtkListStore*    load_history      (void);
 static void             save_history      (GtkListStore *history);
@@ -157,10 +147,10 @@ static gboolean
 module_register(void)
 {
     gwy_volume_func_register("volume_arithmetic",
-                             (GwyVolumeFunc)&arithmetic,
+                             module_main,
                              N_("/_Basic Operations/_Arithmetic..."),
-                             GWY_STOCK_VOLUME_ARITHMETIC,
-                             ARITH_RUN_MODES,
+                             GWY_ICON_VOLUME_ARITHMETIC,
+                             RUN_MODES,
                              GWY_MENU_FLAG_VOLUME,
                              N_("Arithmetic operations on volume data"));
 
@@ -193,7 +183,7 @@ define_module_params(void)
 }
 
 static void
-arithmetic(GwyContainer *data, GwyRunType run)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
     EvaluationData evdata;
@@ -202,18 +192,18 @@ arithmetic(GwyContainer *data, GwyRunType run)
     gint newid;
     guint i;
 
-    g_return_if_fail(run & ARITH_RUN_MODES);
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_BRICK_ID, &dataid.id,
-                                     GWY_APP_CONTAINER_ID, &dataid.datano,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_BRICK_ID, &dataid.id,
+                                 GWY_APP_CONTAINER_ID, &dataid.datano,
+                                 0);
 
     g_return_if_fail(gwy_app_data_id_verify_volume(&dataid));
     args.params = gwy_params_new_from_settings(define_module_params());
     gwy_params_set_volume_id(args.params, PARAM_VOLUME, dataid);
     args.history = load_history();
     args.result = gwy_brick_new(PREVIEW_SIZE, PREVIEW_SIZE, 1, 1.0, 1.0, 1.0, TRUE);
-    args.preview = gwy_data_field_new(PREVIEW_SIZE, PREVIEW_SIZE, 1.0, 1.0, TRUE);
+    args.preview = gwy_field_new(PREVIEW_SIZE, PREVIEW_SIZE, 1.0, 1.0, TRUE);
     sanitise_params(&args);
 
     evdata.expr = gwy_expr_new();
@@ -236,16 +226,18 @@ arithmetic(GwyContainer *data, GwyRunType run)
     if (outcome != GWY_DIALOG_HAVE_RESULT)
         execute(&args, &evdata);
 
-    newid = gwy_app_data_browser_add_brick(args.result, args.preview, data, TRUE);
-    gwy_app_set_data_field_title(data, newid, _("Calculated"));
-    gwy_app_sync_volume_items(gwy_app_data_browser_get(evdata.first.datano), data, evdata.first.id, newid, FALSE,
-                              GWY_DATA_ITEM_GRADIENT,
-                              0);
-    gwy_app_volume_log_add_volume(data, -1, newid);
+    newid = gwy_file_add_volume(data, args.result, args.preview);
+
+    gwy_file_set_visible(data, GWY_FILE_VOLUME, newid, TRUE);
+    gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _("Calculated"), TRUE);
+    gwy_file_sync_items(gwy_data_browser_get_file(evdata.first.datano), GWY_FILE_VOLUME, evdata.first.id,
+                        data, GWY_FILE_VOLUME, newid,
+                        GWY_FILE_ITEM_PALETTE, FALSE);
+    gwy_log_add(data, GWY_FILE_VOLUME, -1, newid);
 
 end:
-    GWY_OBJECT_UNREF(args.result);
-    GWY_OBJECT_UNREF(args.preview);
+    g_clear_object(&args.result);
+    g_clear_object(&args.preview);
     g_object_unref(args.params);
     g_object_unref(args.history);
     gwy_expr_free(evdata.expr);
@@ -254,69 +246,60 @@ end:
 }
 
 static GwyDialogOutcome
-run_gui(GwyContainer *data, gint id,
+run_gui(GwyFile *data, gint id,
         ModuleArgs *args, EvaluationData *evdata)
 {
     gint dataunits = gwy_params_get_int(args->params, PARAM_DATAUNITS);
     const gchar *userunits = gwy_params_get_string(args->params, PARAM_USERUNITS);
-    GtkWidget *hbox, *hbox2, *table, *chooser, *entry, *label, *button;
+    GtkWidget *hbox, *hbox2, *grid, *chooser, *entry, *label, *button;
     ModuleGUI gui;
-    GwyDialogOutcome outcome;
     GwyDialog *dialog;
     guint i, row;
     gchar *s;
 
     gui.args = args;
     gui.evdata = evdata;
-    gui.data = gwy_container_new();
-    gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(0), args->preview);
-    gwy_app_sync_volume_items(data, gui.data, id, 0, FALSE,
-                              GWY_DATA_ITEM_GRADIENT,
-                              0);
 
     gui.dialog = gwy_dialog_new(_("Arithmetic"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_UPDATE, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    gui.view = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, TRUE);
+    gui.view = gwy_create_preview(args->preview, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(gui.view), data, GWY_FILE_VOLUME, id, GWY_FILE_ITEM_PALETTE);
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(gui.view), FALSE);
 
-    table = gtk_table_new(6 + NARGS, 3, FALSE);
-    gtk_table_set_row_spacings(GTK_TABLE(table), 2);
-    gtk_table_set_col_spacings(GTK_TABLE(table), 6);
-    gtk_container_set_border_width(GTK_CONTAINER(table), 4);
-    gtk_box_pack_start(GTK_BOX(hbox), table, TRUE, TRUE, 4);
+    grid = gtk_grid_new();
+    gtk_grid_set_row_spacing(GTK_GRID(grid), 2);
+    gtk_grid_set_column_spacing(GTK_GRID(grid), 6);
+    gtk_container_set_border_width(GTK_CONTAINER(grid), 4);
+    gtk_box_pack_start(GTK_BOX(hbox), grid, TRUE, TRUE, 4);
     row = 0;
 
     label = gtk_label_new_with_mnemonic(_("_Expression:"));
-    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
-    gtk_table_attach(GTK_TABLE(table), label, 0, 2, row, row+1, GTK_FILL, 0, 0, 0);
-    row++;
+    gtk_label_set_xalign(GTK_LABEL(label), 0.0);
+    gtk_grid_attach(GTK_GRID(grid), label, 0, row++, 2, 1);
 
     entry = gtk_combo_box_new_with_model_and_entry(GTK_TREE_MODEL(args->history));
     gtk_combo_box_set_entry_text_column(GTK_COMBO_BOX(entry), 0);
     gui.expression = entry;
     gtk_combo_box_set_active(GTK_COMBO_BOX(gui.expression), 0);
-    gtk_table_attach(GTK_TABLE(table), entry, 0, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
+    gtk_grid_attach(GTK_GRID(grid), entry, 0, row++, 3, 1);
     g_signal_connect_swapped(entry, "changed", G_CALLBACK(expr_changed), &gui);
     g_signal_connect_swapped(gtk_bin_get_child(GTK_BIN(entry)), "activate", G_CALLBACK(preview), &gui);
     gtk_label_set_mnemonic_widget(GTK_LABEL(label), entry);
-    row++;
 
     gui.result = label = gtk_label_new(NULL);
-    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
-    gtk_table_attach(GTK_TABLE(table), label, 0, 2, row, row+1, GTK_FILL, 0, 0, 0);
-    row++;
+    gtk_label_set_xalign(GTK_LABEL(label), 0.0);
+    gtk_grid_attach(GTK_GRID(grid), label, 0, row++, 3, 1);
 
-    gtk_table_set_row_spacing(GTK_TABLE(table), row-1, 8);
     label = gtk_label_new(_("Operands"));
-    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
-    gtk_table_attach(GTK_TABLE(table), label, 0, 2, row, row+1, GTK_FILL, 0, 0, 0);
+    gtk_label_set_xalign(GTK_LABEL(label), 0.0);
+    gwy_set_widget_padding(label, -1, -1, 8, -1);
+    gtk_grid_attach(GTK_GRID(grid), label, 0, row, 2, 1);
 
     label = gtk_label_new(_("Units"));
-    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
-    gtk_table_attach(GTK_TABLE(table), label, 2, 3, row, row+1, GTK_FILL, 0, 0, 0);
-    row++;
+    gtk_label_set_xalign(GTK_LABEL(label), 0.0);
+    gtk_grid_attach(GTK_GRID(grid), label, 2, row++, 1, 1);
 
     gui.dataunits = NULL;
     for (i = 0; i < NARGS; i++) {
@@ -326,15 +309,15 @@ run_gui(GwyContainer *data, gint id,
         s = g_strdup_printf("d_%d", i+1);
         label = gtk_label_new_with_mnemonic(evdata->name[i]);
         g_free(s);
-        gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
-        gtk_table_attach(GTK_TABLE(table), label, 0, 1, row, row+1, GTK_FILL, 0, 0, 0);
+        gtk_label_set_xalign(GTK_LABEL(label), 0.0);
+        gtk_grid_attach(GTK_GRID(grid), label, 0, row, 1, 1);
 
-        chooser = gwy_data_chooser_new_volumes();
+        chooser = gwy_data_chooser_new(GWY_FILE_VOLUME);
         dataid = gwy_params_get_data_id(args->params, PARAM_VOLUME + i);
         gwy_data_chooser_set_active_id(GWY_DATA_CHOOSER(chooser), &dataid);
         g_signal_connect_swapped(chooser, "changed", G_CALLBACK(data_chosen), &gui);
         g_object_set_data(G_OBJECT(chooser), "index", GUINT_TO_POINTER(i));
-        gtk_table_attach(GTK_TABLE(table), chooser, 1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
+        gtk_grid_attach(GTK_GRID(grid), chooser, 1, row, 1, 1);
         gtk_label_set_mnemonic_widget(GTK_LABEL(label), chooser);
 
         button = gtk_radio_button_new(gui.dataunits);
@@ -343,14 +326,12 @@ run_gui(GwyContainer *data, gint id,
         s = g_strdup_printf(_("Take result units from data d%d"), i+1);
         gtk_widget_set_tooltip_text(button, s);
         g_free(s);
-        gtk_table_attach(GTK_TABLE(table), button, 2, 3, row, row+1, 0, 0, 0, 0);
+        gtk_grid_attach(GTK_GRID(grid), button, 2, row++, 1, 1);
         g_signal_connect_swapped(button, "clicked", G_CALLBACK(dataunits_selected), &gui);
-
-        row++;
     }
 
-    hbox2 = gtk_hbox_new(FALSE, 6);
-    gtk_table_attach(GTK_TABLE(table), hbox2, 0, 2, row, row+1, GTK_FILL, 0, 0, 0);
+    hbox2 = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 6);
+    gtk_grid_attach(GTK_GRID(grid), hbox2, 0, row, 2, 1);
 
     label = gtk_label_new_with_mnemonic(_("Specify un_its:"));
     gtk_box_pack_start(GTK_BOX(hbox2), label, FALSE, FALSE, 0);
@@ -368,28 +349,23 @@ run_gui(GwyContainer *data, gint id,
     gui.dataunits = gtk_radio_button_get_group(GTK_RADIO_BUTTON(button));
     gwy_radio_button_set_value(button, USER_UNITS_ID);
     gtk_widget_set_tooltip_text(button, _("Specify result units explicitly"));
-    gtk_table_attach(GTK_TABLE(table), button, 2, 3, row, row+1, 0, 0, 0, 0);
+    gtk_grid_attach(GTK_GRID(grid), button, 2, row++, 1, 1);
     g_signal_connect_swapped(button, "clicked", G_CALLBACK(dataunits_selected), &gui);
-    row++;
 
     gui.table = gwy_param_table_new(args->params);
     gwy_param_table_append_entry(gui.table, PARAM_FILLER_VALUE);
     gwy_param_table_add_enabler(gui.table, PARAM_FIXED_FILLER, PARAM_FILLER_VALUE);
     gwy_param_table_entry_set_width(gui.table, PARAM_FILLER_VALUE, 12);
-    gtk_table_attach(GTK_TABLE(table), gwy_param_table_widget(gui.table), 0, 2, row, row+1, GTK_FILL, 0, 0, 0);
+    gtk_grid_attach(GTK_GRID(grid), gwy_param_table_widget(gui.table), 0, row++, 3, 1);
     gwy_dialog_add_param_table(dialog, gui.table);
-    row++;
 
     g_signal_connect_swapped(gui.table, "param-changed", G_CALLBACK(param_changed), &gui);
     gwy_dialog_set_preview_func(dialog, GWY_PREVIEW_UPON_REQUEST, preview, &gui, NULL);
     gtk_widget_grab_focus(gui.expression);
     gwy_radio_buttons_set_current(gui.dataunits, dataunits);
     expr_changed(&gui, gui.expression); /* resolves variables */
-    outcome = gwy_dialog_run(dialog);
 
-    g_object_unref(gui.data);
-
-    return outcome;
+    return gwy_dialog_run(dialog);
 }
 
 static void
@@ -437,7 +413,7 @@ expr_changed(ModuleGUI *gui, GtkWidget *entry)
         g_return_if_fail(nvars);
         if (nvars == 1) {
             gdouble v = gwy_expr_execute(evdata->expr, NULL);
-            message = s = g_strdup_printf("%g", v);
+            message = s = g_strdup_printf("%.8g", v);
             evdata->err = ARITHMETIC_NUMERIC;
         }
         else {
@@ -491,25 +467,16 @@ show_state(ModuleGUI *gui, const gchar *message)
     GtkDialog *dialog = GTK_DIALOG(gui->dialog);
     ModuleArgs *args = gui->args;
     EvaluationData *evdata = gui->evdata;
-    gboolean ok;
 
     if (!message && evdata->err != ARITHMETIC_NUMERIC) {
         message = check_bricks(args, evdata);
-        gtk_label_set_text(GTK_LABEL(gui->result), evdata->err ? message : NULL);
+        if (!evdata->err)
+            message = NULL;
     }
-    else {
-        if (message)
-            gtk_label_set_text(GTK_LABEL(gui->result), message);
-    }
-
-    ok = (evdata->err == ARITHMETIC_OK);
+    gboolean ok = (evdata->err == ARITHMETIC_OK || evdata->err == ARITHMETIC_NUMERIC);
     gtk_dialog_set_response_sensitive(dialog, GTK_RESPONSE_OK, ok);
     gtk_dialog_set_response_sensitive(dialog, GWY_RESPONSE_UPDATE, ok);
-
-    if (ok)
-        set_widget_as_ok_message(gui->result);
-    else
-        set_widget_as_error_message(gui->result);
+    gwy_set_message_label(GTK_LABEL(gui->result), message, ok ? GTK_MESSAGE_INFO : GTK_MESSAGE_ERROR, FALSE);
 }
 
 static const gchar*
@@ -517,7 +484,7 @@ check_bricks(ModuleArgs *args, EvaluationData *evdata)
 {
     guint first = 0, i;
     GwyBrick *bfirst, *brick;
-    GwyDataCompatibilityFlags diff;
+    GwyDataMismatchFlags diff;
     gboolean nd[NARGS];
 
     if (evdata->err & (ARITHMETIC_EXPR | ARITHMETIC_NUMERIC))
@@ -536,27 +503,27 @@ check_bricks(ModuleArgs *args, EvaluationData *evdata)
         return NULL;
     }
 
-    /* each window must match with first, this is transitive */
+    /* Each brick must match the first. This is transitive. */
     bfirst = gwy_params_get_volume(args->params, PARAM_VOLUME + first);
     for (i = first+1; i < NARGS; i++) {
         if (!nd[i])
             continue;
 
         brick = gwy_params_get_volume(args->params, PARAM_VOLUME + i);
-        diff = gwy_brick_check_compatibility(bfirst, brick,
-                                             GWY_DATA_COMPATIBILITY_RES
-                                             | GWY_DATA_COMPATIBILITY_REAL
-                                             | GWY_DATA_COMPATIBILITY_LATERAL
-                                             | GWY_DATA_COMPATIBILITY_AXISCAL);
+        diff = gwy_brick_is_incompatible(bfirst, brick,
+                                         GWY_DATA_MISMATCH_RES
+                                         | GWY_DATA_MISMATCH_REAL
+                                         | GWY_DATA_MISMATCH_LATERAL
+                                         | GWY_DATA_MISMATCH_AXISCAL);
         if (diff) {
             evdata->err |= ARITHMETIC_DATA;
-            if (diff & GWY_DATA_COMPATIBILITY_RES)
+            if (diff & GWY_DATA_MISMATCH_RES)
                 return _("Pixel dimensions differ");
-            if (diff & GWY_DATA_COMPATIBILITY_LATERAL)
+            if (diff & GWY_DATA_MISMATCH_LATERAL)
                 return _("Lateral dimensions are different physical quantities");
-            if (diff & GWY_DATA_COMPATIBILITY_REAL)
+            if (diff & GWY_DATA_MISMATCH_REAL)
                 return _("Physical dimensions differ");
-            if (diff & GWY_DATA_COMPATIBILITY_AXISCAL)
+            if (diff & GWY_DATA_MISMATCH_AXISCAL)
                 return _("Z-axis calibrations differ");
         }
     }
@@ -578,7 +545,7 @@ preview(gpointer user_data)
     execute(args, gui->evdata);
     g_return_if_fail(args->preview);
 
-    gwy_data_field_data_changed(args->preview);
+    gwy_field_data_changed(args->preview);
     gwy_set_data_preview_size(GWY_DATA_VIEW(gui->view), PREVIEW_SIZE);
     gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
 }
@@ -588,7 +555,7 @@ update_brick(GwyBrick *src, GwyBrick *target, gint *n, gdouble **tdata)
 {
     gint xres = gwy_brick_get_xres(src), yres = gwy_brick_get_yres(src), zres = gwy_brick_get_zres(src);
 
-    gwy_brick_resample(target, xres, yres, zres, GWY_INTERPOLATION_NONE);
+    gwy_brick_resize(target, xres, yres, zres);
     gwy_brick_copy_units(src, target);
     gwy_brick_set_xreal(target, gwy_brick_get_xreal(src));
     gwy_brick_set_yreal(target, gwy_brick_get_yreal(src));
@@ -604,7 +571,7 @@ static void
 execute(ModuleArgs *args, EvaluationData *evdata)
 {
     GwyBrick **data_bricks, *brick;
-    GwyDataLine *zcal = NULL;
+    GwyLine *zcal = NULL;
     const gdouble **d;
     gboolean nd[NARGS];
     gdouble *r = NULL;
@@ -672,8 +639,8 @@ execute(ModuleArgs *args, EvaluationData *evdata)
     /* Set units. */
     dataunits = gwy_params_get_int(args->params, PARAM_DATAUNITS);
     if (dataunits == USER_UNITS_ID) {
-        gwy_si_unit_set_from_string(gwy_brick_get_si_unit_w(args->result),
-                                    gwy_params_get_string(args->params, PARAM_USERUNITS));
+        gwy_unit_set_from_string(gwy_brick_get_unit_w(args->result),
+                                 gwy_params_get_string(args->params, PARAM_USERUNITS));
     }
     else
         gwy_brick_copy_units(gwy_params_get_volume(args->params, PARAM_VOLUME + dataunits), args->result);
@@ -718,10 +685,10 @@ fix_nans(ModuleArgs *args)
     gdouble filler_value = gwy_params_get_double(params, PARAM_FILLER_VALUE);
     GwyBrick *result = args->result;
     gint xres = gwy_brick_get_xres(result), yres = gwy_brick_get_yres(result), zres = gwy_brick_get_zres(result);
-    GwyDataField *plane = args->preview, *mask;
+    GwyField *plane = args->preview, *mask;
     gint i;
 
-    plane = gwy_data_field_new(xres, yres, 1.0, 1.0, FALSE);
+    plane = gwy_field_new(xres, yres, 1.0, 1.0, FALSE);
     for (i = 0; i < zres; i++) {
         gwy_brick_extract_xy_plane(result, plane, i);
         mask = gwy_app_channel_mask_of_nans(plane, FALSE);
@@ -729,9 +696,9 @@ fix_nans(ModuleArgs *args)
             continue;
 
         if (fixed_filler)
-            gwy_data_field_area_fill_mask(plane, mask, GWY_MASK_INCLUDE, 0, 0, xres, yres, filler_value);
+            gwy_field_area_fill(plane, mask, GWY_MASK_INCLUDE, 0, 0, xres, yres, filler_value);
         else
-            gwy_data_field_laplace_solve(plane, mask, -1, 0.25);
+            gwy_field_laplace_solve(plane, mask, -1, 0.25);
         gwy_brick_set_xy_plane(result, plane, i);
         g_object_unref(mask);
     }
@@ -797,15 +764,15 @@ make_z(GwyBrick *brick)
 }
 
 static GwyBrick*
-make_zcal(GwyBrick *brick, GwyDataLine *zcal)
+make_zcal(GwyBrick *brick, GwyLine *zcal)
 {
     gint xres = gwy_brick_get_xres(brick), yres = gwy_brick_get_yres(brick), zres = gwy_brick_get_zres(brick);
     GwyBrick *result = gwy_brick_new_alike(brick, FALSE);
     gdouble *data = gwy_brick_get_data(result);
-    const gdouble *zd = gwy_data_line_get_data_const(zcal);
+    const gdouble *zd = gwy_line_get_data_const(zcal);
     gint i, j;
 
-    g_return_val_if_fail(gwy_data_line_get_res(zcal) == xres, brick);
+    g_return_val_if_fail(gwy_line_get_res(zcal) == xres, brick);
     for (i = 0; i < zres; i++) {
         gdouble *rrow = data + i*xres*yres;
         gdouble z = zd[i];
