@@ -1,5 +1,5 @@
 /*
- *  $Id: volume_asciiexport.c 26507 2024-08-14 15:30:54Z yeti-dn $
+ *  $Id: volume_asciiexport.c 28786 2025-11-04 17:11:41Z yeti-dn $
  *  Copyright (C) 2018-2023 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,6 +16,7 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <locale.h>
 #include <string.h>
 #ifdef HAVE_UNISTD_H
@@ -23,14 +24,7 @@
 #endif
 #include <glib/gstdio.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwymodule/gwymodule-volume.h>
-#include <libprocess/brick.h>
-#include <libgwydgets/gwydgetutils.h>
-#include <libgwydgets/gwycombobox.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
+#include <gwy.h>
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
 
@@ -53,18 +47,18 @@ typedef struct {
     gboolean need_decimal_dot_option;
     GwyBrick *brick;
     /* Cached input data properties. */
-    const guchar *title;
+    const gchar *title;
 } ModuleArgs;
 
-static gboolean         module_register      (void);
-static GwyParamDef*     define_module_params (void);
-static void             volume_ascii_export  (GwyContainer *data,
-                                              GwyRunType mode);
-static GwyDialogOutcome run_gui              (ModuleArgs *args);
-static gchar*           export_brick         (gpointer user_data,
-                                              gssize *data_len);
-static void             destroy_brick        (gchar *data,
-                                              gpointer user_data);
+static gboolean         module_register     (void);
+static GwyParamDef*     define_module_params(void);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
+static GwyDialogOutcome run_gui             (ModuleArgs *args);
+static gchar*           export_brick        (gpointer user_data,
+                                             gssize *data_len);
+static void             destroy_brick       (gchar *data,
+                                             gpointer user_data);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
@@ -82,7 +76,7 @@ static gboolean
 module_register(void)
 {
     gwy_volume_func_register("volume_asciiexport",
-                             (GwyVolumeFunc)&volume_ascii_export,
+                             module_main,
                              N_("/_Basic Operations/Export _Text..."),
                              NULL,
                              RUN_MODES,
@@ -116,7 +110,7 @@ define_module_params(void)
 }
 
 static void
-volume_ascii_export(GwyContainer *data, GwyRunType mode)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
     GwyDialogOutcome outcome;
@@ -124,10 +118,10 @@ volume_ascii_export(GwyContainer *data, GwyRunType mode)
 
     g_return_if_fail(mode & RUN_MODES);
 
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_BRICK, &args.brick,
-                                     GWY_APP_BRICK_ID, &id,
-                                     0);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_BRICK, &args.brick,
+                                 GWY_APP_BRICK_ID, &id,
+                                 0);
     g_return_if_fail(GWY_IS_BRICK(args.brick));
 
     args.params = gwy_params_new_from_settings(define_module_params());
@@ -138,10 +132,10 @@ volume_ascii_export(GwyContainer *data, GwyRunType mode)
             goto finish;
     }
 
-    if (!gwy_container_gis_string(data, gwy_app_get_brick_title_key_for_id(id), &args.title))
+    if (!(args.title = gwy_file_get_title(data, GWY_FILE_VOLUME, id)))
         args.title = _("Volume data");
 
-    gwy_save_auxiliary_with_callback(_("Export to Text File"), NULL, export_brick, destroy_brick, &args);
+    gwy_save_auxiliary_with_callback(_("Export to Text File"), NULL, export_brick, destroy_brick, FALSE, &args);
 
 finish:
     g_object_unref(args.params);
