@@ -1,5 +1,5 @@
 /*
- *  $Id: volume_calibrate.c 26320 2024-05-06 11:27:10Z yeti-dn $
+ *  $Id: volume_calibrate.c 28808 2025-11-05 18:26:20Z yeti-dn $
  *  Copyright (C) 2013-2022 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,16 +16,12 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
 #include <string.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
+#include <gwy.h>
 #include <gtk/gtk.h>
-#include <libprocess/stats.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-volume.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
 
@@ -95,9 +91,9 @@ typedef struct {
     gint yres;
     gint zres;
     gint is_square;
-    GwySIUnit *xyunit;
-    GwySIUnit *zunit;
-    GwySIUnit *wunit;
+    GwyUnit *xyunit;
+    GwyUnit *zunit;
+    GwyUnit *wunit;
 } ModuleArgs;
 
 typedef struct {
@@ -107,18 +103,18 @@ typedef struct {
     GwyParamTable *table_z;
     GwyParamTable *table_offsets;
     GwyParamTable *table_value;
-    GwySIValueFormat *xyvf;
-    GwySIValueFormat *xycalvf;
-    GwySIValueFormat *zvf;
-    GwySIValueFormat *zcalvf;
-    GwySIValueFormat *wvf;
-    GwySIValueFormat *wcalvf;
+    GwyValueFormat *xyvf;
+    GwyValueFormat *xycalvf;
+    GwyValueFormat *zvf;
+    GwyValueFormat *zcalvf;
+    GwyValueFormat *wvf;
+    GwyValueFormat *wcalvf;
 } ModuleGUI;
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             volcal              (GwyContainer *data,
-                                             GwyRunType runtype);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static GwyDialogOutcome run_gui             (ModuleArgs *args);
 static GwyParamTable*   make_table_xy       (ModuleArgs *args);
 static GwyParamTable*   make_table_z        (ModuleArgs *args);
@@ -129,7 +125,7 @@ static void             param_changed       (ModuleGUI *gui,
 static void             dialog_response     (GwyDialog *dialog,
                                              gint response,
                                              ModuleGUI *gui);
-static gboolean         template_filter     (GwyContainer *data,
+static gboolean         template_filter     (GwyFile *data,
                                              gint id,
                                              gpointer user_data);
 static void             init_xyparams       (ModuleArgs *args);
@@ -152,9 +148,9 @@ static gboolean
 module_register(void)
 {
     gwy_volume_func_register("volcal",
-                             (GwyVolumeFunc)&volcal,
+                             module_main,
                              N_("/_Basic Operations/_Dimensions and Units..."),
-                             GWY_STOCK_VOLUME_DIMENSIONS,
+                             GWY_ICON_VOLUME_DIMENSIONS,
                              RUN_MODES,
                              GWY_MENU_FLAG_VOLUME,
                              N_("Change physical dimensions, units or value scale"));
@@ -235,12 +231,12 @@ calibrate_one_brick(GwyBrick *brick, GwyParams *params)
     if (gwy_params_get_enum(params, PARAM_XY_MODE) != MODE_KEEP) {
         gwy_brick_set_xreal(brick, gwy_params_get_double(params, PARAM_XREAL));
         gwy_brick_set_yreal(brick, gwy_params_get_double(params, PARAM_YREAL));
-        gwy_si_unit_assign(gwy_brick_get_si_unit_x(brick), gwy_params_get_unit(params, PARAM_XYUNIT, NULL));
-        gwy_si_unit_assign(gwy_brick_get_si_unit_y(brick), gwy_params_get_unit(params, PARAM_XYUNIT, NULL));
+        gwy_unit_assign(gwy_brick_get_unit_x(brick), gwy_params_get_unit(params, PARAM_XYUNIT, NULL));
+        gwy_unit_assign(gwy_brick_get_unit_y(brick), gwy_params_get_unit(params, PARAM_XYUNIT, NULL));
     }
     if (gwy_params_get_enum(params, PARAM_Z_MODE) != MODE_KEEP) {
         gwy_brick_set_zreal(brick, gwy_params_get_double(params, PARAM_ZREAL));
-        gwy_si_unit_assign(gwy_brick_get_si_unit_z(brick), gwy_params_get_unit(params, PARAM_ZUNIT, NULL));
+        gwy_unit_assign(gwy_brick_get_unit_z(brick), gwy_params_get_unit(params, PARAM_ZUNIT, NULL));
     }
     if (gwy_params_get_enum(params, PARAM_OFFSETS_MODE) != MODE_KEEP) {
         gwy_brick_set_xoffset(brick, gwy_params_get_double(params, PARAM_XOFFSET));
@@ -250,34 +246,34 @@ calibrate_one_brick(GwyBrick *brick, GwyParams *params)
     if (gwy_params_get_enum(params, PARAM_VALUE_MODE) != MODE_KEEP) {
         gwy_brick_multiply(brick, gwy_params_get_double(params, PARAM_WRATIO));
         gwy_brick_add(brick, gwy_params_get_double(params, PARAM_WSHIFT));
-        gwy_si_unit_assign(gwy_brick_get_si_unit_w(brick), gwy_params_get_unit(params, PARAM_WUNIT, NULL));
+        gwy_unit_assign(gwy_brick_get_unit_w(brick), gwy_params_get_unit(params, PARAM_WUNIT, NULL));
     }
 }
 
 static void
-calibrate_one_image(GwyDataField *field, GwyParams *params)
+calibrate_one_image(GwyField *field, GwyParams *params)
 {
     if (gwy_params_get_enum(params, PARAM_XY_MODE) != MODE_KEEP) {
-        gwy_data_field_set_xreal(field, gwy_params_get_double(params, PARAM_XREAL));
-        gwy_data_field_set_yreal(field, gwy_params_get_double(params, PARAM_YREAL));
-        gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(field), gwy_params_get_unit(params, PARAM_XYUNIT, NULL));
+        gwy_field_set_xreal(field, gwy_params_get_double(params, PARAM_XREAL));
+        gwy_field_set_yreal(field, gwy_params_get_double(params, PARAM_YREAL));
+        gwy_unit_assign(gwy_field_get_unit_xy(field), gwy_params_get_unit(params, PARAM_XYUNIT, NULL));
     }
     if (gwy_params_get_enum(params, PARAM_OFFSETS_MODE) != MODE_KEEP) {
-        gwy_data_field_set_xoffset(field, gwy_params_get_double(params, PARAM_XOFFSET));
-        gwy_data_field_set_yoffset(field, gwy_params_get_double(params, PARAM_YOFFSET));
+        gwy_field_set_xoffset(field, gwy_params_get_double(params, PARAM_XOFFSET));
+        gwy_field_set_yoffset(field, gwy_params_get_double(params, PARAM_YOFFSET));
     }
     if (gwy_params_get_enum(params, PARAM_VALUE_MODE) != MODE_KEEP) {
-        gwy_data_field_multiply(field, gwy_params_get_double(params, PARAM_WRATIO));
-        gwy_data_field_add(field, gwy_params_get_double(params, PARAM_WSHIFT));
-        gwy_si_unit_assign(gwy_data_field_get_si_unit_z(field), gwy_params_get_unit(params, PARAM_WUNIT, NULL));
+        gwy_field_multiply(field, gwy_params_get_double(params, PARAM_WRATIO));
+        gwy_field_add(field, gwy_params_get_double(params, PARAM_WSHIFT));
+        gwy_unit_assign(gwy_field_get_unit_z(field), gwy_params_get_unit(params, PARAM_WUNIT, NULL));
     }
 }
 
 static void
-volcal(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyBrick *brick;
-    GwyDataField *preview;
+    GwyField *preview;
     GQuark quark;
     gint oldid, newid;
     ModuleArgs args;
@@ -285,14 +281,14 @@ volcal(GwyContainer *data, GwyRunType runtype)
     GwyParams *params;
     gboolean new_channel;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    gwy_app_data_browser_get_current(GWY_APP_BRICK, &brick,
-                                     GWY_APP_BRICK_KEY, &quark,
-                                     GWY_APP_BRICK_ID, &oldid,
-                                     0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_data_browser_get_current(GWY_APP_BRICK, &brick,
+                                 GWY_APP_BRICK_KEY, &quark,
+                                 GWY_APP_BRICK_ID, &oldid,
+                                 0);
     g_return_if_fail(brick);
 
-    gwy_clear(&args, 1);
+    gwy_clear1(args);
     args.brick = brick;
     args.xres = gwy_brick_get_xres(brick);
     args.yres = gwy_brick_get_yres(brick);
@@ -305,16 +301,16 @@ volcal(GwyContainer *data, GwyRunType runtype)
     args.zoffset = gwy_brick_get_zoffset(brick);
     args.wmin = gwy_brick_get_min(brick);
     args.wmax = gwy_brick_get_max(brick);
-    args.xyunit = gwy_brick_get_si_unit_x(brick);
-    args.zunit = gwy_brick_get_si_unit_z(brick);
-    args.wunit = gwy_brick_get_si_unit_w(brick);
+    args.xyunit = gwy_brick_get_unit_x(brick);
+    args.zunit = gwy_brick_get_unit_z(brick);
+    args.wunit = gwy_brick_get_unit_w(brick);
     args.is_square = (fabs(log(args.yreal/args.yres * args.xres/args.xreal)) <= EPSILON);
     args.params = params = gwy_params_new_from_settings(define_module_params());
     init_xyparams(&args);
     init_zparams(&args);
     init_wparams(&args);
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args);
         gwy_params_save_to_settings(params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -323,28 +319,31 @@ volcal(GwyContainer *data, GwyRunType runtype)
 
     new_channel = gwy_params_get_boolean(params, PARAM_NEW_DATA);
     if (new_channel)
-        brick = gwy_brick_duplicate(brick);
+        brick = gwy_brick_copy(brick);
     else
-        gwy_app_undo_qcheckpointv(data, 1, &quark);
+        gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &quark);
 
     calibrate_one_brick(brick, params);
 
-    preview = gwy_container_get_object(data, gwy_app_get_brick_preview_key_for_id(oldid));
+    preview = gwy_container_get_object(GWY_CONTAINER(data), gwy_file_key_volume_picture(oldid));
     if (new_channel) {
-        preview = gwy_data_field_duplicate(preview);
+        preview = gwy_field_copy(preview);
         calibrate_one_image(preview, params);
-        newid = gwy_app_data_browser_add_brick(brick, preview, data, TRUE);
+        newid = gwy_file_add_volume(data, brick, preview);
+        gwy_file_set_visible(data, GWY_FILE_VOLUME, newid, TRUE);
         g_object_unref(brick);
         g_object_unref(preview);
-        gwy_app_sync_volume_items(data, data, oldid, newid, FALSE, GWY_DATA_ITEM_GRADIENT, 0);
-        gwy_app_set_brick_title(data, newid, _("Recalibrated Data"));
-        gwy_app_volume_log_add_volume(data, oldid, newid);
+        gwy_file_sync_items(data, GWY_FILE_VOLUME, oldid,
+                            data, GWY_FILE_VOLUME, newid,
+                            GWY_FILE_ITEM_PALETTE, FALSE);
+        gwy_file_set_title(data, GWY_FILE_VOLUME, newid, _("Recalibrated Data"), TRUE);
+        gwy_log_add(data, GWY_FILE_VOLUME, oldid, newid);
     }
     else {
         calibrate_one_image(preview, params);
         gwy_brick_data_changed(brick);
-        gwy_data_field_data_changed(preview);
-        gwy_app_volume_log_add_volume(data, oldid, oldid);
+        gwy_field_data_changed(preview);
+        gwy_log_add(data, GWY_FILE_VOLUME, oldid, oldid);
     }
 
 end:
@@ -359,17 +358,17 @@ run_gui(ModuleArgs *args)
     GwyParamTable *table;
     GtkWidget *hbox;
     ModuleGUI gui;
-    GwySIValueFormat *xyvf, *zvf, *wvf;
+    GwyValueFormat *xyvf, *zvf, *wvf;
     gchar *buf;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
 
     gui.dialog = gwy_dialog_new(_("Dimensions and Units"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    hbox = gwy_hbox_new(20);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 20);
     gwy_dialog_add_content(dialog, hbox, TRUE, TRUE, 0);
 
     table = gui.table_xy = make_table_xy(args);
@@ -391,9 +390,9 @@ run_gui(ModuleArgs *args)
     gtk_box_pack_start(GTK_BOX(hbox), gwy_param_table_widget(table), TRUE, TRUE, 0);
     gwy_dialog_add_param_table(dialog, table);
 
-    xyvf = gui.xyvf = gwy_brick_get_value_format_x(args->brick, GWY_SI_UNIT_FORMAT_VFMARKUP, NULL);
-    zvf = gui.zvf = gwy_brick_get_value_format_z(args->brick, GWY_SI_UNIT_FORMAT_VFMARKUP, NULL);
-    wvf = gui.wvf = gwy_brick_get_value_format_w(args->brick, GWY_SI_UNIT_FORMAT_VFMARKUP, NULL);
+    xyvf = gui.xyvf = gwy_brick_get_value_format_x(args->brick, GWY_UNIT_FORMAT_VFMARKUP, NULL);
+    zvf = gui.zvf = gwy_brick_get_value_format_z(args->brick, GWY_UNIT_FORMAT_VFMARKUP, NULL);
+    wvf = gui.wvf = gwy_brick_get_value_format_w(args->brick, GWY_UNIT_FORMAT_VFMARKUP, NULL);
 
     buf = g_strdup_printf("%.*f%s%s × %.*f%s%s",
                           xyvf->precision, args->xreal/xyvf->magnitude, *xyvf->units ? " " : "", xyvf->units,
@@ -426,12 +425,12 @@ run_gui(ModuleArgs *args)
     g_signal_connect_after(dialog, "response", G_CALLBACK(dialog_response), &gui);
     outcome = gwy_dialog_run(dialog);
 
-    GWY_SI_VALUE_FORMAT_FREE(gui.xyvf);
-    GWY_SI_VALUE_FORMAT_FREE(gui.xycalvf);
-    GWY_SI_VALUE_FORMAT_FREE(gui.zvf);
-    GWY_SI_VALUE_FORMAT_FREE(gui.zcalvf);
-    GWY_SI_VALUE_FORMAT_FREE(gui.wvf);
-    GWY_SI_VALUE_FORMAT_FREE(gui.wcalvf);
+    GWY_FREE_VALUE_FORMAT(gui.xyvf);
+    GWY_FREE_VALUE_FORMAT(gui.xycalvf);
+    GWY_FREE_VALUE_FORMAT(gui.zvf);
+    GWY_FREE_VALUE_FORMAT(gui.zcalvf);
+    GWY_FREE_VALUE_FORMAT(gui.wvf);
+    GWY_FREE_VALUE_FORMAT(gui.wcalvf);
 
     return outcome;
 }
@@ -574,8 +573,8 @@ param_changed_xy(ModuleGUI *gui, gint id)
     ModuleArgs *args = gui->args;
     GwyParams *params = args->params;
     GwyParamTable *table_xy = gui->table_xy, *table_offsets = gui->table_offsets;
-    GwySIUnit *unit, *unitcal;
-    GwySIValueFormat *vf;
+    GwyUnit *unit, *unitcal;
+    GwyValueFormat *vf;
     gdouble xreal, yreal, m;
     gboolean square, xreal_changed = FALSE, yreal_changed = FALSE, use_template = FALSE;
     CalibrateMode mode, offsets_mode;
@@ -617,9 +616,9 @@ param_changed_xy(ModuleGUI *gui, gint id)
                                    (xreal = gwy_brick_get_dx(template)*gwy_brick_get_xres(args->brick)));
         gwy_param_table_set_double(table_xy, PARAM_YREAL,
                                    (yreal = gwy_brick_get_dx(template)*gwy_brick_get_yres(args->brick)));
-        vf = gwy_brick_get_value_format_x(template, GWY_SI_UNIT_FORMAT_PLAIN, NULL);
+        vf = gwy_brick_get_value_format_x(template, GWY_UNIT_FORMAT_PLAIN, NULL);
         gwy_param_table_set_string(table_xy, PARAM_XYUNIT, vf->units);
-        gwy_si_unit_value_format_free(vf);
+        gwy_value_format_free(vf);
         xreal_changed = yreal_changed = TRUE;
     }
 
@@ -690,11 +689,11 @@ param_changed_xy(ModuleGUI *gui, gint id)
     /* Units are mostly just a presentational aspect.  When the user changes units we do not change any value. */
     if (id < 0 || id == PARAM_XYUNIT || xreal_changed || yreal_changed) {
         unit = gwy_params_get_unit(params, PARAM_XYUNIT, &power10);
-        unitcal = gwy_si_unit_divide(unit, args->xyunit, NULL);
-        gui->xyvf = gwy_si_unit_get_format_for_power10(unit, GWY_SI_UNIT_FORMAT_VFMARKUP, power10, gui->xyvf);
+        unitcal = gwy_unit_divide(unit, args->xyunit, NULL);
+        gui->xyvf = gwy_unit_get_format_for_power10(unit, GWY_UNIT_FORMAT_VFMARKUP, power10, gui->xyvf);
         gui->xyvf->precision = 4;
         m = 5.0*gwy_params_get_double(params, PARAM_XRATIO);
-        gui->xycalvf = gwy_si_unit_get_format_with_digits(unitcal, GWY_SI_UNIT_FORMAT_VFMARKUP, m, 6, gui->xycalvf);
+        gui->xycalvf = gwy_unit_get_format_with_digits(unitcal, GWY_UNIT_FORMAT_VFMARKUP, m, 6, gui->xycalvf);
         gwy_debug("XY %g (%d) [%s] for %g", gui->xycalvf->magnitude, gui->xycalvf->precision, gui->xycalvf->units, m);
         g_object_unref(unitcal);
         update_xy_formats(gui);
@@ -707,8 +706,8 @@ param_changed_z(ModuleGUI *gui, gint id)
     ModuleArgs *args = gui->args;
     GwyParams *params = args->params;
     GwyParamTable *table_z = gui->table_z, *table_offsets = gui->table_offsets;
-    GwySIUnit *unit, *unitcal;
-    GwySIValueFormat *vf;
+    GwyUnit *unit, *unitcal;
+    GwyValueFormat *vf;
     gdouble zreal, m;
     gboolean zreal_changed = FALSE, use_template = FALSE;
     CalibrateMode mode, offsets_mode;
@@ -744,9 +743,9 @@ param_changed_z(ModuleGUI *gui, gint id)
         GwyBrick *template = gwy_params_get_volume(params, PARAM_ZTEMPLATE);
         gwy_param_table_set_double(table_z, PARAM_ZREAL,
                                    (zreal = gwy_brick_get_dx(template)*gwy_brick_get_xres(args->brick)));
-        vf = gwy_brick_get_value_format_z(template, GWY_SI_UNIT_FORMAT_PLAIN, NULL);
+        vf = gwy_brick_get_value_format_z(template, GWY_UNIT_FORMAT_PLAIN, NULL);
         gwy_param_table_set_string(table_z, PARAM_ZUNIT, vf->units);
-        gwy_si_unit_value_format_free(vf);
+        gwy_value_format_free(vf);
         zreal_changed = TRUE;
     }
 
@@ -778,11 +777,11 @@ param_changed_z(ModuleGUI *gui, gint id)
     /* Units are mostly just a presentational aspect.  When the user changes units we do not change any value. */
     if (id < 0 || id == PARAM_ZUNIT || zreal_changed) {
         unit = gwy_params_get_unit(params, PARAM_ZUNIT, &power10);
-        unitcal = gwy_si_unit_divide(unit, args->zunit, NULL);
-        gui->zvf = gwy_si_unit_get_format_for_power10(unit, GWY_SI_UNIT_FORMAT_VFMARKUP, power10, gui->zvf);
+        unitcal = gwy_unit_divide(unit, args->zunit, NULL);
+        gui->zvf = gwy_unit_get_format_for_power10(unit, GWY_UNIT_FORMAT_VFMARKUP, power10, gui->zvf);
         gui->zvf->precision = 4;
         m = 5.0*gwy_params_get_double(params, PARAM_ZRATIO);
-        gui->zcalvf = gwy_si_unit_get_format_with_digits(unitcal, GWY_SI_UNIT_FORMAT_VFMARKUP, m, 6, gui->zcalvf);
+        gui->zcalvf = gwy_unit_get_format_with_digits(unitcal, GWY_UNIT_FORMAT_VFMARKUP, m, 6, gui->zcalvf);
         gwy_debug("Z %g (%d) [%s] for %g", gui->zcalvf->magnitude, gui->zcalvf->precision, gui->zcalvf->units, m);
         g_object_unref(unitcal);
         update_z_formats(gui);
@@ -795,7 +794,7 @@ param_changed_w(ModuleGUI *gui, gint id)
     ModuleArgs *args = gui->args;
     GwyParams *params = args->params;
     GwyParamTable *table_value = gui->table_value;
-    GwySIUnit *unit, *unitcal;
+    GwyUnit *unit, *unitcal;
     gdouble m, wmin, wratio, wshift, wrange;
     CalibrateMode mode;
     gint power10;
@@ -845,11 +844,11 @@ param_changed_w(ModuleGUI *gui, gint id)
     /* Units are mostly just a presentational aspect.  When the user changes units we do not change any value. */
     if (id < 0 || id == PARAM_WUNIT || id == PARAM_WRANGE || id == PARAM_WRATIO || id == PARAM_VALUE_MODE) {
         unit = gwy_params_get_unit(params, PARAM_WUNIT, &power10);
-        unitcal = gwy_si_unit_divide(unit, args->wunit, NULL);
-        gui->wvf = gwy_si_unit_get_format_for_power10(unit, GWY_SI_UNIT_FORMAT_VFMARKUP, power10, gui->wvf);
+        unitcal = gwy_unit_divide(unit, args->wunit, NULL);
+        gui->wvf = gwy_unit_get_format_for_power10(unit, GWY_UNIT_FORMAT_VFMARKUP, power10, gui->wvf);
         gui->wvf->precision = 4;
         m = 5.0*gwy_params_get_double(params, PARAM_WRATIO);
-        gui->wcalvf = gwy_si_unit_get_format_with_digits(unitcal, GWY_SI_UNIT_FORMAT_VFMARKUP, m, 6, gui->wcalvf);
+        gui->wcalvf = gwy_unit_get_format_with_digits(unitcal, GWY_UNIT_FORMAT_VFMARKUP, m, 6, gui->wcalvf);
         gwy_debug("W %g (%d) [%s] for %g", gui->wcalvf->magnitude, gui->wcalvf->precision, gui->wcalvf->units, m);
         g_object_unref(unitcal);
         update_w_formats(gui);
@@ -869,25 +868,25 @@ reset_formats(ModuleGUI *gui)
 {
     ModuleArgs *args = gui->args;
     GwyBrick *brick = args->brick;
-    GwySIUnit *unitcal = gwy_si_unit_new(NULL);
+    GwyUnit *unitcal = gwy_unit_new(NULL);
 
-    gui->xyvf = gwy_brick_get_value_format_x(brick, GWY_SI_UNIT_FORMAT_PLAIN, gui->xyvf);
+    gui->xyvf = gwy_brick_get_value_format_x(brick, GWY_UNIT_FORMAT_PLAIN, gui->xyvf);
     gwy_param_table_set_string(gui->table_xy, PARAM_XYUNIT, gui->xyvf->units);
-    gui->xyvf = gwy_brick_get_value_format_x(brick, GWY_SI_UNIT_FORMAT_VFMARKUP, gui->xyvf);
+    gui->xyvf = gwy_brick_get_value_format_x(brick, GWY_UNIT_FORMAT_VFMARKUP, gui->xyvf);
     gui->xyvf->precision += 2;
-    gui->xycalvf = gwy_si_unit_get_format_with_digits(unitcal, GWY_SI_UNIT_FORMAT_VFMARKUP, 10.0, 6, gui->xycalvf);
+    gui->xycalvf = gwy_unit_get_format_with_digits(unitcal, GWY_UNIT_FORMAT_VFMARKUP, 10.0, 6, gui->xycalvf);
 
-    gui->zvf = gwy_brick_get_value_format_z(brick, GWY_SI_UNIT_FORMAT_PLAIN, gui->zvf);
+    gui->zvf = gwy_brick_get_value_format_z(brick, GWY_UNIT_FORMAT_PLAIN, gui->zvf);
     gwy_param_table_set_string(gui->table_z, PARAM_ZUNIT, gui->zvf->units);
-    gui->zvf = gwy_brick_get_value_format_z(brick, GWY_SI_UNIT_FORMAT_VFMARKUP, gui->zvf);
+    gui->zvf = gwy_brick_get_value_format_z(brick, GWY_UNIT_FORMAT_VFMARKUP, gui->zvf);
     gui->zvf->precision += 2;
-    gui->zcalvf = gwy_si_unit_get_format_with_digits(unitcal, GWY_SI_UNIT_FORMAT_VFMARKUP, 10.0, 6, gui->zcalvf);
+    gui->zcalvf = gwy_unit_get_format_with_digits(unitcal, GWY_UNIT_FORMAT_VFMARKUP, 10.0, 6, gui->zcalvf);
 
-    gui->wvf = gwy_brick_get_value_format_z(brick, GWY_SI_UNIT_FORMAT_PLAIN, gui->wvf);
+    gui->wvf = gwy_brick_get_value_format_z(brick, GWY_UNIT_FORMAT_PLAIN, gui->wvf);
     gwy_param_table_set_string(gui->table_value, PARAM_WUNIT, gui->wvf->units);
-    gui->wvf = gwy_brick_get_value_format_z(brick, GWY_SI_UNIT_FORMAT_VFMARKUP, gui->wvf);
+    gui->wvf = gwy_brick_get_value_format_z(brick, GWY_UNIT_FORMAT_VFMARKUP, gui->wvf);
     gui->wvf->precision += 2;
-    gui->wcalvf = gwy_si_unit_get_format_with_digits(unitcal, GWY_SI_UNIT_FORMAT_VFMARKUP, 10.0, 6, gui->wcalvf);
+    gui->wcalvf = gwy_unit_get_format_with_digits(unitcal, GWY_UNIT_FORMAT_VFMARKUP, 10.0, 6, gui->wcalvf);
 
     g_object_unref(unitcal);
 
@@ -905,11 +904,11 @@ dialog_response(G_GNUC_UNUSED GwyDialog *dialog, gint response, ModuleGUI *gui)
 }
 
 static gboolean
-template_filter(GwyContainer *data, gint id, gpointer user_data)
+template_filter(GwyFile *data, gint id, gpointer user_data)
 {
     GwyBrick *otherbrick, *brick = (GwyBrick*)user_data;
 
-    if (!gwy_container_gis_object(data, gwy_app_get_brick_key_for_id(id), &otherbrick))
+    if (!(otherbrick = gwy_file_get_volume(data, id)))
         return FALSE;
     return otherbrick != brick;
 }
@@ -918,8 +917,8 @@ static void
 init_xyparams(ModuleArgs *args)
 {
     GwyParams *params = args->params;
-    GwySIValueFormat *vf = NULL;
-    GwySIUnit *xyunit;
+    GwyValueFormat *vf = NULL;
+    GwyUnit *xyunit;
     gdouble newxreal, newyreal, newxoffset, newyoffset;
     CalibrateMode mode;
 
@@ -931,7 +930,7 @@ init_xyparams(ModuleArgs *args)
         if (template) {
             newxreal = gwy_brick_get_dx(template)*gwy_brick_get_xres(args->brick);
             newyreal = gwy_brick_get_dy(template)*gwy_brick_get_yres(args->brick);
-            vf = gwy_brick_get_value_format_x(template, GWY_SI_UNIT_FORMAT_PLAIN, vf);
+            vf = gwy_brick_get_value_format_x(template, GWY_UNIT_FORMAT_PLAIN, vf);
             gwy_params_set_unit(params, PARAM_XYUNIT, vf->units);
         }
         else
@@ -940,17 +939,17 @@ init_xyparams(ModuleArgs *args)
     if (mode == MODE_KEEP) {
         newxreal = args->xreal;
         newyreal = args->yreal;
-        vf = gwy_brick_get_value_format_x(args->brick, GWY_SI_UNIT_FORMAT_PLAIN, vf);
+        vf = gwy_brick_get_value_format_x(args->brick, GWY_UNIT_FORMAT_PLAIN, vf);
     }
     else if (mode == MODE_SET_RANGE) {
         newxreal = gwy_params_get_double(params, PARAM_XREAL);
         newyreal = gwy_params_get_double(params, PARAM_YREAL);
-        vf = gwy_si_unit_get_format_with_digits(xyunit, GWY_SI_UNIT_FORMAT_PLAIN, newxreal, 6, vf);
+        vf = gwy_unit_get_format_with_digits(xyunit, GWY_UNIT_FORMAT_PLAIN, newxreal, 6, vf);
     }
     else if (mode == MODE_CALIBRATE) {
         newxreal = args->xreal*gwy_params_get_double(params, PARAM_XRATIO);
         newyreal = args->yreal*gwy_params_get_double(params, PARAM_YRATIO);
-        vf = gwy_si_unit_get_format_with_digits(xyunit, GWY_SI_UNIT_FORMAT_PLAIN, newxreal, 6, vf);
+        vf = gwy_unit_get_format_with_digits(xyunit, GWY_UNIT_FORMAT_PLAIN, newxreal, 6, vf);
     }
     else if (mode == MODE_MATCH) {
     }
@@ -986,15 +985,15 @@ init_xyparams(ModuleArgs *args)
     gwy_params_set_double(params, PARAM_XOFFSET, newxoffset);
     gwy_params_set_double(params, PARAM_YOFFSET, newyoffset);
 
-    gwy_si_unit_value_format_free(vf);
+    gwy_value_format_free(vf);
 }
 
 static void
 init_zparams(ModuleArgs *args)
 {
     GwyParams *params = args->params;
-    GwySIValueFormat *vf = NULL;
-    GwySIUnit *zunit;
+    GwyValueFormat *vf = NULL;
+    GwyUnit *zunit;
     gdouble newzreal, newzoffset;
     CalibrateMode mode;
 
@@ -1005,7 +1004,7 @@ init_zparams(ModuleArgs *args)
         GwyBrick *template = gwy_params_get_volume(params, PARAM_ZTEMPLATE);
         if (template) {
             newzreal = gwy_brick_get_dz(template)*gwy_brick_get_zres(args->brick);
-            vf = gwy_brick_get_value_format_z(template, GWY_SI_UNIT_FORMAT_PLAIN, vf);
+            vf = gwy_brick_get_value_format_z(template, GWY_UNIT_FORMAT_PLAIN, vf);
             gwy_params_set_unit(params, PARAM_ZUNIT, vf->units);
         }
         else
@@ -1013,15 +1012,15 @@ init_zparams(ModuleArgs *args)
     }
     if (mode == MODE_KEEP) {
         newzreal = args->zreal;
-        vf = gwy_brick_get_value_format_z(args->brick, GWY_SI_UNIT_FORMAT_PLAIN, vf);
+        vf = gwy_brick_get_value_format_z(args->brick, GWY_UNIT_FORMAT_PLAIN, vf);
     }
     else if (mode == MODE_SET_RANGE) {
         newzreal = gwy_params_get_double(params, PARAM_ZREAL);
-        vf = gwy_si_unit_get_format_with_digits(zunit, GWY_SI_UNIT_FORMAT_PLAIN, newzreal, 6, vf);
+        vf = gwy_unit_get_format_with_digits(zunit, GWY_UNIT_FORMAT_PLAIN, newzreal, 6, vf);
     }
     else if (mode == MODE_CALIBRATE) {
         newzreal = args->zreal*gwy_params_get_double(params, PARAM_ZRATIO);
-        vf = gwy_si_unit_get_format_with_digits(zunit, GWY_SI_UNIT_FORMAT_PLAIN, newzreal, 6, vf);
+        vf = gwy_unit_get_format_with_digits(zunit, GWY_UNIT_FORMAT_PLAIN, newzreal, 6, vf);
     }
     else if (mode == MODE_MATCH) {
     }
@@ -1047,15 +1046,15 @@ init_zparams(ModuleArgs *args)
     }
     gwy_params_set_double(params, PARAM_ZOFFSET, newzoffset);
 
-    gwy_si_unit_value_format_free(vf);
+    gwy_value_format_free(vf);
 }
 
 static void
 init_wparams(ModuleArgs *args)
 {
     GwyParams *params = args->params;
-    GwySIValueFormat *vf = NULL;
-    GwySIUnit *wunit;
+    GwyValueFormat *vf = NULL;
+    GwyUnit *wunit;
     gdouble newwrange, wshift, wcal, newwmin;
     CalibrateMode mode;
 
@@ -1067,21 +1066,21 @@ init_wparams(ModuleArgs *args)
         wshift = 0.0;
         wcal = 1.0;
         newwmin = args->wmin;
-        vf = gwy_brick_get_value_format_w(args->brick, GWY_SI_UNIT_FORMAT_PLAIN, vf);
+        vf = gwy_brick_get_value_format_w(args->brick, GWY_UNIT_FORMAT_PLAIN, vf);
     }
     else if (mode == MODE_SET_RANGE) {
         newwrange = gwy_params_get_double(params, PARAM_WRANGE);
         newwmin = gwy_params_get_double(params, PARAM_WMIN);
         wcal = (args->wmax - args->wmin > 0.0 ? newwrange/(args->wmax - args->wmin) : 0.0);
         wshift = newwmin - args->wmin;
-        vf = gwy_si_unit_get_format_with_digits(wunit, GWY_SI_UNIT_FORMAT_PLAIN, newwrange, 6, vf);
+        vf = gwy_unit_get_format_with_digits(wunit, GWY_UNIT_FORMAT_PLAIN, newwrange, 6, vf);
     }
     else if (mode == MODE_CALIBRATE) {
         wcal = gwy_params_get_double(params, PARAM_WRATIO);
         wshift = gwy_params_get_double(params, PARAM_WSHIFT);
         newwrange = (args->wmax - args->wmin)*wcal;
         newwmin = args->wmin - wshift;
-        vf = gwy_si_unit_get_format_with_digits(wunit, GWY_SI_UNIT_FORMAT_PLAIN, newwrange, 6, vf);
+        vf = gwy_unit_get_format_with_digits(wunit, GWY_UNIT_FORMAT_PLAIN, newwrange, 6, vf);
     }
     else {
         g_return_if_reached();
@@ -1092,7 +1091,7 @@ init_wparams(ModuleArgs *args)
     gwy_params_set_double(params, PARAM_WRATIO, wcal);
     gwy_params_set_double(params, PARAM_WSHIFT, wshift);
 
-    gwy_si_unit_value_format_free(vf);
+    gwy_value_format_free(vf);
 }
 
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
