@@ -1,5 +1,5 @@
 /*
- *  $Id: volume_drift.c 26354 2024-05-21 08:22:32Z yeti-dn $
+ *  $Id: volume_drift.c 29061 2026-01-02 15:01:53Z yeti-dn $
  *  Copyright (C) 2015-2023 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,26 +16,11 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwythreads.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwynlfit.h>
-#include <libprocess/brick.h>
-#include <libprocess/stats.h>
-#include <libprocess/linestats.h>
-#include <libprocess/gwyprocess.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libprocess/correlation.h>
-#include <libgwydgets/gwydataview.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwydgets/gwydgetutils.h>
-#include <libgwymodule/gwymodule-volume.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
-#include "libgwyddion/gwyomp.h"
+#include <gwy.h>
+#include "libgwyddion/omp.h"
 
 #define RUN_MODES (GWY_RUN_INTERACTIVE)
 
@@ -114,43 +99,42 @@ typedef struct {
 typedef struct {
     ModuleArgs *args;
     GtkWidget *dialog;
+    GtkWidget *dataview;
     GwyParamTable *table_quantity;
     GwyParamTable *table_options;
     GwyParamTable *table_x;
     GwyParamTable *table_y;
     GwyParamTable *table_z;
-    GwyContainer *data;
     GwyGraphModel *gmodel;
     GwySelection *graph_selection;
 } ModuleGUI;
 
-
-static gboolean                    module_register        (void);
-static GwyParamDef*                define_module_params   (void);
-static void                        drift                  (GwyContainer *data,
-                                                           GwyRunType runtype);
-static void                        execute                (ModuleArgs *args,
-                                                           GtkWindow *wait_window);
-static GwyDialogOutcome            run_gui                (ModuleArgs *args,
-                                                           GwyContainer *data,
-                                                           gint id);
-static void                        param_changed          (ModuleGUI *gui,
-                                                           gint id);
-static void                        dialog_response        (GwyDialog *dialog,
-                                                           gint response,
-                                                           ModuleGUI *gui);
-static void                        preview                (gpointer user_data);
-static void                        update_image           (ModuleGUI *gui,
-                                                           gint z);
-static void                        calculate_xydrift_data (ModuleArgs *args,
-                                                           GtkWindow *wait_window);
-static void                        calculate_zdrift_data  (ModuleArgs *args,
-                                                           GtkWindow *wait_window);
-static void                        graph_selection_changed(ModuleGUI *gui,
-                                                           gint id,
-                                                           GwySelection *selection);
-static void                        update_drift_curve     (ModuleGUI *gui);
-static void                        update_fit_curve       (ModuleGUI *gui);
+static gboolean         module_register        (void);
+static GwyParamDef*     define_module_params   (void);
+static void             module_main            (GwyFile *data,
+                                                GwyRunModeFlags mode);
+static void             execute                (ModuleArgs *args,
+                                                GtkWindow *wait_window);
+static GwyDialogOutcome run_gui                (ModuleArgs *args,
+                                                GwyFile *data,
+                                                gint id);
+static void             param_changed          (ModuleGUI *gui,
+                                                gint id);
+static void             dialog_response        (GwyDialog *dialog,
+                                                gint response,
+                                                ModuleGUI *gui);
+static void             preview                (gpointer user_data);
+static void             update_image           (ModuleGUI *gui,
+                                                gint z);
+static void             calculate_xydrift_data (ModuleArgs *args,
+                                                GtkWindow *wait_window);
+static void             calculate_zdrift_data  (ModuleArgs *args,
+                                                GtkWindow *wait_window);
+static void             graph_selection_changed(ModuleGUI *gui,
+                                                gint id,
+                                                GwySelection *selection);
+static void             update_drift_curve     (ModuleGUI *gui);
+static void             update_fit_curve       (ModuleGUI *gui);
 
 
 static GwyModuleInfo module_info = {
@@ -169,7 +153,7 @@ static gboolean
 module_register(void)
 {
     gwy_volume_func_register("volume_drift",
-                             (GwyVolumeFunc)&drift,
+                             module_main,
                              N_("/_Correct Data/Estimate _Drift..."),
                              NULL,
                              RUN_MODES,
@@ -197,7 +181,6 @@ define_module_params(void)
     if (paramdef)
         return paramdef;
 
-
     paramdef = gwy_param_def_new();
     gwy_param_def_set_function_name(paramdef, gwy_volume_func_current());
     gwy_param_def_add_gwyenum(paramdef, PARAM_DIR, "dir", _("Show direction"),
@@ -217,6 +200,8 @@ define_module_params(void)
     gwy_param_def_add_int(paramdef, PARAM_YORDER, "yorder", _("Polynomial degree"), 0, 10, 1);
     gwy_param_def_add_int(paramdef, PARAM_ZORDER, "zorder", _("Polynomial degree"), 0, 10, 1);
     gwy_param_def_add_int(paramdef, PARAM_NROT, "nrot", _("_Number of rotations"), 3, 100, 15);
+    /* FIXME: This is wrong. Raw parameters should be always in radians. Set a conversion factor with
+     * gwy_param_table_slider_set_scaling(). */
     gwy_param_def_add_double(paramdef, PARAM_ROTRANGE, "rotrange", _("_Rotation range"),
                              0.1, 10, 5);
 
@@ -224,7 +209,7 @@ define_module_params(void)
 }
 
 static void
-drift(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
     GwyBrick *brick = NULL;
@@ -232,12 +217,11 @@ drift(GwyContainer *data, GwyRunType runtype)
     gboolean output_drift, output_crop;
     gint oldid, newid, k, zres;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    g_return_if_fail(g_type_from_name("GwyLayerPoint"));
+    g_return_if_fail(mode & RUN_MODES);
 
-    gwy_app_data_browser_get_current(GWY_APP_BRICK, &brick,
-                                     GWY_APP_BRICK_ID, &oldid,
-                                     0);
+    gwy_data_browser_get_current(GWY_APP_BRICK, &brick,
+                                 GWY_APP_BRICK_ID, &oldid,
+                                 0);
     g_return_if_fail(GWY_IS_BRICK(brick));
     args.result = NULL;
     args.brick = brick;
@@ -268,7 +252,7 @@ drift(GwyContainer *data, GwyRunType runtype)
         args.zdrift_z[k] = k;
     }
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, oldid);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -276,7 +260,7 @@ drift(GwyContainer *data, GwyRunType runtype)
     }
 
     if (outcome != GWY_DIALOG_HAVE_RESULT)
-        execute(&args, gwy_app_find_window_for_volume(data, oldid));
+        execute(&args, gwy_data_browser_get_window_for_data(data, GWY_FILE_VOLUME, oldid));
 
     output_drift = gwy_params_get_boolean(args.params, PARAM_OUTPUT_DRIFT);
     output_crop = gwy_params_get_boolean(args.params, PARAM_OUTPUT_CROP);
@@ -286,9 +270,9 @@ drift(GwyContainer *data, GwyRunType runtype)
         GwyGraphCurveModel *gcmodel;
 
         if (!args.xydata_done)
-            calculate_xydrift_data(&args, gwy_app_find_window_for_volume(data, oldid));
+            calculate_xydrift_data(&args, gwy_data_browser_get_window_for_data(data, GWY_FILE_VOLUME, oldid));
         if (!args.zdata_done)
-            calculate_zdrift_data(&args, gwy_app_find_window_for_volume(data, oldid));
+            calculate_zdrift_data(&args, gwy_data_browser_get_window_for_data(data, GWY_FILE_VOLUME, oldid));
 
         gmodel = gwy_graph_model_new();
         g_object_set(gmodel,
@@ -307,7 +291,8 @@ drift(GwyContainer *data, GwyRunType runtype)
         gwy_graph_curve_model_set_data(gcmodel, args.ydrift_z, args.ydrift_drift, args.ydrift_n);
         gwy_graph_model_add_curve(gmodel, gcmodel);
         g_object_unref(gcmodel);
-        gwy_app_data_browser_add_graph_model(gmodel, data, TRUE);
+        newid = gwy_file_add_graph(data, gmodel);
+        gwy_file_set_visible(data, GWY_FILE_GRAPH, newid, TRUE);
         g_object_unref(gmodel);
 
         gmodel = gwy_graph_model_new();
@@ -322,18 +307,20 @@ drift(GwyContainer *data, GwyRunType runtype)
         gwy_graph_curve_model_set_data(gcmodel, args.zdrift_z, args.zdrift_drift, args.zdrift_n);
         gwy_graph_model_add_curve(gmodel, gcmodel);
         g_object_unref(gcmodel);
-        gwy_app_data_browser_add_graph_model(gmodel, data, TRUE);
+        newid = gwy_file_add_graph(data, gmodel);
+        gwy_file_set_visible(data, GWY_FILE_GRAPH, newid, TRUE);
         g_object_unref(gmodel);
     }
     if (output_crop) {
-        newid = gwy_app_data_browser_add_brick(args.result, NULL, data, TRUE);
+        newid = gwy_file_add_volume(data, args.result, NULL);
+        gwy_file_set_visible(data, GWY_FILE_VOLUME, newid, TRUE);
 
-        gwy_app_set_brick_title(data, newid, _("Drift corrected"));
-        gwy_app_sync_volume_items(data, data, oldid, newid, FALSE,
-                                  GWY_DATA_ITEM_GRADIENT,
-                                  0);
+        gwy_file_set_title(data, GWY_FILE_VOLUME, newid, _("Drift corrected"), TRUE);
+        gwy_file_sync_items(data, GWY_FILE_VOLUME, oldid,
+                            data, GWY_FILE_VOLUME, newid,
+                            GWY_FILE_ITEM_PALETTE, FALSE);
 
-        gwy_app_volume_log_add_volume(data, -1, newid);
+        gwy_log_add(data, GWY_FILE_VOLUME, -1, newid);
     }
 
 end:
@@ -355,7 +342,7 @@ end:
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     GtkWidget *hbox, *graph, *area, *dataview;
     GwyParamTable *table;
@@ -364,44 +351,27 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     GwyGraphCurveModel *gcmodel;
     GwyDialogOutcome outcome;
     GwyBrick *brick = args->brick;
-    GwyDataField *field = gwy_data_field_new(gwy_brick_get_xres(brick),
-                                             gwy_brick_get_yres(brick),
-                                             gwy_brick_get_xreal(brick),
-                                             gwy_brick_get_yreal(brick),
-                                             TRUE);
-    const guchar *gradient;
+    GwyField *field = gwy_brick_new_field_like_xy_plane(brick, TRUE);
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
-    gui.data = gwy_container_new();
 
     args->xydata_done = FALSE;
     args->zdata_done = FALSE;
 
-    args->result = gwy_brick_duplicate(brick);
-
-    gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(0), field);
-    if (gwy_container_gis_string(data, gwy_app_get_brick_palette_key_for_id(id), &gradient))
-        gwy_container_set_const_string(gui.data, gwy_app_get_data_palette_key_for_id(0), gradient);
+    args->result = gwy_brick_copy(brick);
 
     gui.dialog = gwy_dialog_new(_("Estimate Drift"));
     dialog = GWY_DIALOG(gui.dialog);
 
-    gtk_dialog_add_button(GTK_DIALOG(dialog),
-                          _("_Calculate drift"), RESPONSE_DRIFT);
-    gtk_dialog_add_button(GTK_DIALOG(dialog),
-                          _("_Apply correction"), RESPONSE_APPLY);
-    gtk_dialog_add_button(GTK_DIALOG(dialog),
-                          _("_Reset"), GWY_RESPONSE_RESET);
-    gtk_dialog_add_button(GTK_DIALOG(dialog),
-                          GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
-    gtk_dialog_add_button(GTK_DIALOG(dialog),
-                          GTK_STOCK_OK, GTK_RESPONSE_OK);
+    gtk_dialog_add_button(GTK_DIALOG(dialog), _("_Calculate drift"), RESPONSE_DRIFT);
+    gtk_dialog_add_button(GTK_DIALOG(dialog), _("_Apply correction"), RESPONSE_APPLY);
+    gtk_dialog_add_button(GTK_DIALOG(dialog), _("_Reset"), GWY_RESPONSE_RESET);
+    gwy_dialog_add_buttons(dialog, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
     gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);
 
-    //gwy_dialog_add_buttons(dialog, GWY_RESPONSE_UPDATE, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
-
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
+    dataview = gui.dataview = gwy_create_preview(field, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_VOLUME, id, GWY_FILE_ITEM_PALETTE);
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), FALSE);
 
     gui.gmodel = gwy_graph_model_new();
@@ -425,7 +395,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gui.graph_selection = gwy_graph_area_get_selection(GWY_GRAPH_AREA(area), GWY_GRAPH_STATUS_XLINES);
     gwy_selection_set_max_objects(gui.graph_selection, 1);
 
-    hbox = gwy_hbox_new(20);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 20);
     gwy_dialog_add_content(dialog, hbox, TRUE, TRUE, 4);
 
     table = gui.table_quantity = gwy_param_table_new(args->params);
@@ -487,7 +457,6 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     update_image(&gui, 0);
     outcome = gwy_dialog_run(dialog);
 
-    g_object_unref(gui.data);
     g_object_unref(gui.gmodel);
 
     return outcome;
@@ -541,7 +510,7 @@ dialog_response(G_GNUC_UNUSED GwyDialog *dialog, gint response, ModuleGUI *gui)
     if (response == GWY_RESPONSE_RESET) {
         gui->args->xydata_done = FALSE;
         gui->args->zdata_done = FALSE;
-        gwy_brick_copy(gui->args->brick, gui->args->result, FALSE);
+        gwy_brick_copy_data(gui->args->brick, gui->args->result);
         gwy_selection_clear(gui->graph_selection);
     } else if (response == RESPONSE_DRIFT) {
         update_drift_curve(gui);
@@ -557,16 +526,14 @@ calculate_zdrift_data(ModuleArgs *args, GtkWindow *wait_window)
 {
     gint k;
     GwyBrick *brick = args->brick;
-    gint xres = brick->xres;
-    gint yres = brick->yres;
-    gint zres = brick->zres;
-    GwyDataField *dfield;
+    gint zres = gwy_brick_get_zres(brick);
+    GwyField *dfield;
     gdouble zerodrift;
     gdouble *zdrift;
     gboolean cancelled = FALSE;
 
     gwy_app_wait_start(wait_window, _("Updating z drift data..."));
-    dfield = gwy_data_field_new(xres, yres, brick->xreal, brick->yreal, FALSE);
+    dfield = gwy_brick_new_field_like_xy_plane(brick, FALSE);
 
     args->zdata_done = TRUE;
     zdrift = g_new(gdouble, zres);
@@ -575,10 +542,10 @@ calculate_zdrift_data(ModuleArgs *args, GtkWindow *wait_window)
         gwy_brick_extract_xy_plane(brick, dfield, k);
         if (k == 0) {
             zdrift[k] = 0;
-            zerodrift = gwy_data_field_get_avg(dfield);
+            zerodrift = gwy_field_get_avg(dfield);
         }
         else
-            zdrift[k] = gwy_data_field_get_avg(dfield) - zerodrift;
+            zdrift[k] = gwy_field_get_avg(dfield) - zerodrift;
 
         if (!gwy_app_wait_set_fraction((gdouble)k/zres)) {
             clear_data(args);
@@ -622,23 +589,23 @@ calculate_xydrift_data(ModuleArgs *args, GtkWindow *wait_window)
     GwyParams *params = args->params;
     gint method = gwy_params_get_enum(params, PARAM_METHOD);
     gboolean rotate = gwy_params_get_boolean(params, PARAM_ROTATE);
-    gdouble rotrange = gwy_params_get_double(params, PARAM_ROTRANGE)*G_PI/180;
+    gdouble rotrange = gwy_deg2rad(gwy_params_get_double(params, PARAM_ROTRANGE));
     gint nrot = gwy_params_get_int(params, PARAM_NROT);
     gint xres = brick->xres;
     gint yres = brick->yres;
     gint zres = brick->zres;
     gdouble angle, rotscore, bestrotscore, bestrot = 0;
-    GwyDataField *dfield, *dfieldp, *score, *kernel, *rotdfield;
+    GwyField *dfield, *dfieldp, *score, *kernel, *rotdfield;
     gboolean cancelled = FALSE;
 
     gwy_app_wait_start(wait_window, _("Updating lateral drift data..."));
 
-    dfield = gwy_data_field_new(xres, yres, brick->xreal, brick->yreal, FALSE);
-    dfieldp = gwy_data_field_new(xres, yres, brick->xreal, brick->yreal, FALSE);
-    score = gwy_data_field_new_alike(dfieldp, FALSE);
+    dfield = gwy_brick_new_field_like_xy_plane(brick, FALSE);
+    dfieldp = gwy_brick_new_field_like_xy_plane(brick, FALSE);
+    score = gwy_field_new_alike(dfieldp, FALSE);
     xborder = xres/10;
     yborder = yres/10;
-    kernel = gwy_data_field_area_extract(dfield, xborder, yborder, xres - 2*xborder, yres - 2*yborder);
+    kernel = gwy_field_area_extract(dfield, xborder, yborder, xres - 2*xborder, yres - 2*yborder);
 
     xdrift = g_new(gdouble, zres);
     ydrift = g_new(gdouble, zres);
@@ -655,14 +622,15 @@ calculate_xydrift_data(ModuleArgs *args, GtkWindow *wait_window)
         xsum = 0;
         ysum = 0;
         rsum = 0;
-    } else {
+    }
+    else {
         kstart = 0;
         gwy_brick_sum_xy_plane(brick, dfieldp);
     }
 
     for (k = kstart; k < zres; k++) {
         if (method == METHOD_INDIVIDUAL)
-           gwy_brick_extract_xy_plane(brick, dfieldp, k-1);
+            gwy_brick_extract_xy_plane(brick, dfieldp, k-1);
 
         gwy_brick_extract_xy_plane(brick, dfield, k);
 
@@ -676,17 +644,17 @@ calculate_xydrift_data(ModuleArgs *args, GtkWindow *wait_window)
             for (m = 0; m < nrot; m++) {
                 angle = -rotrange/2 + m*rotrange/nrot;
 
-                rotdfield = gwy_data_field_new_rotated(dfield, NULL, angle,
-                                                       GWY_INTERPOLATION_BILINEAR,
-                                                       GWY_ROTATE_RESIZE_SAME_SIZE);
-                gwy_data_field_area_copy(rotdfield, kernel, xborder, yborder, xres - 2*xborder, yres - 2*yborder, 0, 0);
+                rotdfield = gwy_field_new_rotated(dfield, NULL, angle,
+                                                  GWY_INTERPOLATION_LINEAR,
+                                                  GWY_ROTATE_RESIZE_SAME_SIZE);
+                gwy_field_area_copy(rotdfield, kernel, xborder, yborder, xres - 2*xborder, yres - 2*yborder, 0, 0);
 
-                gwy_data_field_correlation_search(dfieldp, kernel, NULL, score, GWY_CORRELATION_POC,
-                                                  0.1, GWY_EXTERIOR_MIRROR_EXTEND, 0);
+                gwy_field_correlation_search(dfieldp, kernel, NULL, score, GWY_CORRELATION_POC,
+                                             0.1, GWY_EXTERIOR_MIRROR, 0);
 
-                rotscore = gwy_data_field_get_max(score);
+                rotscore = gwy_field_get_max(score);
                 rotdata[m] = rotscore;
-                //printf("%g %g\n", angle*180/G_PI, rotscore);
+                //printf("%g %g\n", gwy_rad2deg(angle), rotscore);
                 if (rotscore > bestrotscore) {
                     bestrotscore = rotscore;
                     bestrot = angle;
@@ -697,37 +665,37 @@ calculate_xydrift_data(ModuleArgs *args, GtkWindow *wait_window)
             }
             if (bestrotm > 0 && bestrotm < (nrot-1)) {
                 gwy_math_refine_maximum_1d(rotdata + bestrotm - 1, &xrot);
-                //printf("rot %g     %g -> %g\n", xrot, bestrot*180/G_PI,( bestrot+xrot*rotrange/nrot)*180/G_PI);
+                //printf("rot %g     %g -> %g\n", xrot, gwy_rad2deg(bestrot),( bestrot+xrot*rotrange/nrot)*180/G_PI);
                 bestrot += xrot*rotrange/nrot;
             }
 
             g_free(rotdata);
-            //printf("rotrange %g  bestangle %g\n", rotrange, bestrot*180/G_PI);
+            //printf("rotrange %g  bestangle %g\n", rotrange, gwy_rad2deg(bestrot));
         }
 
         if (rotate)
-            gwy_data_field_rotate(dfield, bestrot, GWY_INTERPOLATION_BILINEAR);
+            gwy_field_rotate(dfield, bestrot, GWY_INTERPOLATION_LINEAR);
 
-        gwy_data_field_area_copy(dfield, kernel, xborder, yborder, xres - 2*xborder, yres - 2*yborder, 0, 0);
+        gwy_field_area_copy(dfield, kernel, xborder, yborder, xres - 2*xborder, yres - 2*yborder, 0, 0);
 
-        gwy_data_field_correlation_search(dfieldp, kernel, NULL, score, GWY_CORRELATION_POC,
-                                          0.1, GWY_EXTERIOR_MIRROR_EXTEND, 0);
+        gwy_field_correlation_search(dfieldp, kernel, NULL, score, GWY_CORRELATION_POC,
+                                     0.1, GWY_EXTERIOR_MIRROR, 0);
 
       //remove all the far values
-        //gwy_data_field_area_clear(score, 0, 0, xres/2 - xborder + 1, yres);
-        //gwy_data_field_area_clear(score, 0, 0, xres, yres/2 - yborder + 1);
-        //gwy_data_field_area_clear(score, xres/2 + xborder - 1, 0, xres/2 - xborder + 1, yres);
-        //gwy_data_field_area_clear(score, 0, yres/2 + yborder - 1, xres, yres/2 - yborder + 1);
+        //gwy_field_area_clear(score, 0, 0, xres/2 - xborder + 1, yres);
+        //gwy_field_area_clear(score, 0, 0, xres, yres/2 - yborder + 1);
+        //gwy_field_area_clear(score, xres/2 + xborder - 1, 0, xres/2 - xborder + 1, yres);
+        //gwy_field_area_clear(score, 0, yres/2 + yborder - 1, xres, yres/2 - yborder + 1);
 
 
-        if (gwy_data_field_get_local_maxima_list(score, &xoff, &yoff, &maxscore, 1, 0, 0.0, FALSE)) {
+        if (gwy_field_get_local_maxima_list(score, &xoff, &yoff, &maxscore, 1, 0, 0.0, FALSE)) {
             //printf("%g %g  %d %d  x %g  y %g score %g\n", xoff, yoff, xres/2, yres/2,
                    //xoff-xres/2, yoff-yres/2, maxscore);
             if (xoff > 0 && xoff < (xres - 1) && yoff > 0 && yoff < (yres - 1)) {
                 n = 0;
                 for (j = -1; j <= 1; j++) {
                     for (i = -1; i <= 1; i++) {
-                        zvals[n++] = gwy_data_field_get_val(score, xoff+i, yoff+j);
+                        zvals[n++] = gwy_field_get_val(score, xoff+i, yoff+j);
                     }
                 }
                 gwy_math_refine_maximum_2d(zvals, &xoffset, &yoffset);
@@ -744,7 +712,7 @@ calculate_xydrift_data(ModuleArgs *args, GtkWindow *wait_window)
         if (method == METHOD_INDIVIDUAL) {
             xsum += xoffset;
             ysum += yoffset;
-            rsum += bestrot*180/G_PI;
+            rsum += gwy_rad2deg(bestrot);
             xdrift[k] = xsum;
             ydrift[k] = ysum;
             rdrift[k] = rsum;
@@ -752,7 +720,7 @@ calculate_xydrift_data(ModuleArgs *args, GtkWindow *wait_window)
         else {
             xdrift[k] = xoffset;
             ydrift[k] = yoffset;
-            rdrift[k] = bestrot*180/G_PI;
+            rdrift[k] = gwy_rad2deg(bestrot);
         }
 
         if (!gwy_app_wait_set_fraction((gdouble)k/zres)) {
@@ -772,7 +740,6 @@ calculate_xydrift_data(ModuleArgs *args, GtkWindow *wait_window)
         args->xydata_done = TRUE;
     }
 
-
     g_object_unref(dfield);
     g_object_unref(dfieldp);
     g_object_unref(score);
@@ -794,7 +761,7 @@ fit_polynom(gint degree, gboolean raw, gdouble *z, gdouble *drift, gdouble *fit,
            fit[k] = drift[k];
     }
     else {
-       coeffs = gwy_math_fit_polynom(n, z, drift, degree, NULL);
+       coeffs = gwy_math_fit_poly(z, drift, n, degree, NULL);
        for (k = 0; k < n; k++)
            fit[k] = polynom(degree, coeffs, k);
 
@@ -846,7 +813,8 @@ update_fit_curve(ModuleGUI *gui)
         else if (dir == DIR_Y) {
             gwy_graph_curve_model_set_data(gcmodel, args->ydrift_z, args->ydrift_drift, args->ydrift_n);
             gwy_graph_curve_model_set_data(fitmodel, args->ydrift_z, args->ydrift_fit, args->ydrift_n);
-        } else {
+        }
+        else {
             gwy_graph_curve_model_set_data(gcmodel, args->rdrift_z, args->rdrift_drift, args->rdrift_n);
             gwy_graph_curve_model_set_data(fitmodel, args->rdrift_z, args->rdrift_drift, args->rdrift_n);
         }
@@ -859,38 +827,42 @@ update_fit_curve(ModuleGUI *gui)
         gwy_graph_curve_model_set_data(fitmodel, args->zdrift_z, args->zdrift_fit, args->zdrift_n);
     }
 
-    if (dir == DIR_X)
+    if (dir == DIR_X) {
         g_object_set(gui->gmodel,
                      "axis-label-left", _("x drift [px]"),
                      "axis-label-bottom", "slice level",
                      NULL);
-    else if (dir == DIR_Y)
+    }
+    else if (dir == DIR_Y) {
         g_object_set(gui->gmodel,
                      "axis-label-left", _("y drift [px]"),
                      "axis-label-bottom", "slice level",
                      NULL);
-    else if (dir == DIR_Z)
+    }
+    else if (dir == DIR_Z) {
         g_object_set(gui->gmodel,
                      "axis-label-left", _("z drift"),
                      "axis-label-bottom", "slice level",
                      NULL);
-    else
+    }
+    else {
         g_object_set(gui->gmodel,
                      "axis-label-left", _("rotation [deg]"),
                      "axis-label-bottom", "slice level",
                      NULL);
+    }
 }
 
 static void
 update_image(ModuleGUI *gui, gint z)
 {
-    GwyDataField *dfield;
-    GwyBrick *brick = gui->args->result;
-    dfield = gwy_container_get_object(gui->data, gwy_app_get_data_key_for_id(0));
+    ModuleArgs *args = gui->args;
+    GwyBrick *brick = args->result;
+    gint zres = gwy_brick_get_zres(brick);
+    GwyField *field = gwy_data_view_get_field(GWY_DATA_VIEW(gui->dataview));
 
-    gwy_brick_extract_xy_plane(brick, dfield, CLAMP(z, 0, brick->zres-1));
-
-    gwy_data_field_data_changed(dfield);
+    gwy_brick_extract_xy_plane(brick, field, CLAMP(z, 0, zres-1));
+    gwy_field_data_changed(field);
 }
 
 static void
@@ -918,7 +890,7 @@ preview(gpointer user_data)
 }
 
 static void
-subpixel_area_copy(GwyDataField *dfield, GwyDataField *cropfield,
+subpixel_area_copy(GwyField *dfield, GwyField *cropfield,
                    gint xres, gint yres,
                    gdouble xs, gdouble ys)
 {
@@ -932,10 +904,8 @@ subpixel_area_copy(GwyDataField *dfield, GwyDataField *cropfield,
         }
     }
 
-    gwy_data_field_sample_distorted(dfield, cropfield,
-                                    coords, GWY_INTERPOLATION_SCHAUM,
-                                    GWY_EXTERIOR_FIXED_VALUE,
-                                    gwy_data_field_get_avg(dfield));
+    gwy_field_sample_distorted(dfield, cropfield, coords,
+                               GWY_INTERPOLATION_SCHAUM, GWY_EXTERIOR_FIXED_VALUE, gwy_field_get_avg(dfield));
 
     g_free(coords);
 }
@@ -954,7 +924,7 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
     gdouble xs, ys;
     GwyBrick *original = args->brick;
     GwyBrick *brick = args->result;
-    GwyDataField *dfield, *cropfield;
+    GwyField *dfield, *cropfield;
     gint xres = gwy_brick_get_xres(original);
     gint yres = gwy_brick_get_yres(original);
     gint zres = gwy_brick_get_zres(original);
@@ -965,15 +935,15 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
     yc = args->ydrift_fit[zres/2];
 
     gwy_app_wait_start(wait_window, _("Cropping data..."));
-    dfield = gwy_data_field_new(xres, yres, gwy_brick_get_xreal(original), gwy_brick_get_yreal(original), FALSE);
-    cropfield = gwy_data_field_new(nxres, nyres, gwy_brick_get_xreal(brick), gwy_brick_get_yreal(brick), FALSE);
+    dfield = gwy_field_new(xres, yres, gwy_brick_get_xreal(original), gwy_brick_get_yreal(original), FALSE);
+    cropfield = gwy_field_new(nxres, nyres, gwy_brick_get_xreal(brick), gwy_brick_get_yreal(brick), FALSE);
 
     for (k = 0; k < zres; k++) {
         if (xdo || ydo || zdo)
             gwy_brick_extract_xy_plane(original, dfield, k);
 
         if (xdo || ydo) {
-            gwy_data_field_fill(cropfield, gwy_data_field_get_avg(dfield));
+            gwy_field_fill(cropfield, gwy_field_get_avg(dfield));
         }
 
         if (xdo)
@@ -989,13 +959,13 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
             subpixel_area_copy(dfield, cropfield, xres, yres, xs, ys);
         }
         else if (zdo)
-            gwy_data_field_area_copy(dfield, cropfield, 0, 0, xres, yres, xs, ys);
+            gwy_field_area_copy(dfield, cropfield, 0, 0, xres, yres, xs, ys);
 
         if (rotate)
-            gwy_data_field_rotate(cropfield, args->rdrift_drift[k]*G_PI/180, GWY_INTERPOLATION_BILINEAR);
+            gwy_field_rotate(cropfield, args->rdrift_drift[k]*G_PI/180, GWY_INTERPOLATION_LINEAR);
 
         if (zdo)
-            gwy_data_field_add(cropfield, -args->zdrift_fit[k]);
+            gwy_field_add(cropfield, -args->zdrift_fit[k]);
 
         if (xdo || ydo || zdo)
             gwy_brick_set_xy_plane(brick, cropfield, k);
