@@ -1,5 +1,5 @@
 /*
- *  $Id: volume_equiplane.c 26744 2024-10-18 15:03:29Z yeti-dn $
+ *  $Id: volume_equiplane.c 28905 2025-11-24 15:51:18Z yeti-dn $
  *  Copyright (C) 2019-2023 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,17 +16,11 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwythreads.h>
-#include <libprocess/brick.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-volume.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
-#include "libgwyddion/gwyomp.h"
+#include <gwy.h>
+#include "libgwyddion/omp.h"
 
 #define RUN_MODES (GWY_RUN_INTERACTIVE)
 
@@ -60,15 +54,14 @@ typedef enum {
 typedef struct {
     GwyParams *params;
     GwyBrick *brick;
-    GwyDataField *result;
-    GwyDataLine *calibration;
+    GwyField *result;
+    GwyLine *calibration;
     gdouble value;
 } ModuleArgs;
 
 typedef struct {
     ModuleArgs *args;
-    GwyContainer *data;
-    GwyDataField *xyplane;
+    GwyField *xyplane;
     GtkWidget *dialog;
     GwyParamTable *table_value;
     GwyParamTable *table_output;
@@ -76,16 +69,16 @@ typedef struct {
     GtkWidget *dataview;
     GwySelection *image_selection;
     GwySelection *graph_selection;
-    GwySIValueFormat *vf;
+    GwyValueFormat *vf;
 } ModuleGUI;
 
 static gboolean         module_register        (void);
 static GwyParamDef*     define_module_params   (void);
-static void             equiplane              (GwyContainer *data,
-                                                GwyRunType run);
+static void             module_main            (GwyFile *data,
+                                                GwyRunModeFlags mode);
 static void             execute                (ModuleArgs *args);
 static GwyDialogOutcome run_gui                (ModuleArgs *args,
-                                                GwyContainer *data,
+                                                GwyFile *data,
                                                 gint id);
 static void             param_changed          (ModuleGUI *gui,
                                                 gint id);
@@ -123,7 +116,7 @@ static gboolean
 module_register(void)
 {
     gwy_volume_func_register("volume_equiplane",
-                             (GwyVolumeFunc)&equiplane,
+                             module_main,
                              N_("/SPM M_odes/_Isosurface Image..."),
                              NULL,
                              RUN_MODES,
@@ -158,52 +151,49 @@ define_module_params(void)
     gwy_param_def_add_gwyenum(paramdef, PARAM_SEARCH_DIR, "search_dir", _("_Search direction"),
                               searchdirs, G_N_ELEMENTS(searchdirs), SEARCH_BILATERAL);
     gwy_param_def_add_boolean(paramdef, PARAM_KEEP_SIGN, "keep_sign", _("Preserve _intersection sign"), TRUE);
-    gwy_param_def_add_gwyenum(paramdef, PARAM_SHOW_TYPE, "show_type", gwy_sgettext("verb|_Display"),
+    gwy_param_def_add_gwyenum(paramdef, PARAM_SHOW_TYPE, "show_type", C_("verb", "_Display"),
                               displays, G_N_ELEMENTS(displays), SHOW_DATA);
     gwy_param_def_add_instant_updates(paramdef, PARAM_UPDATE, "update", NULL, TRUE);
     return paramdef;
 }
 
-static GwySIValueFormat*
+static GwyValueFormat*
 make_zvf(GwyBrick *brick)
 {
-    return gwy_si_unit_get_format_with_digits(gwy_brick_get_si_unit_z(brick), GWY_SI_UNIT_FORMAT_VFMARKUP,
-                                              gwy_brick_get_zreal(brick), 5, NULL);
+    return gwy_unit_get_format_with_digits(gwy_brick_get_unit_z(brick), GWY_UNIT_FORMAT_VFMARKUP,
+                                           gwy_brick_get_zreal(brick), 5, NULL);
 }
 
 static void
-equiplane(GwyContainer *data, GwyRunType run)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
     GwyDialogOutcome outcome;
-    GwySIValueFormat *zvf;
+    GwyValueFormat *zvf;
     GwyBrick *brick;
     gint id, newid;
-    GwySIUnit *zunit;
+    GwyUnit *zunit;
     gdouble const_value;
     gchar *title;
 
-    g_return_if_fail(run & RUN_MODES);
-    g_return_if_fail(g_type_from_name("GwyLayerPoint"));
+    g_return_if_fail(mode & RUN_MODES);
 
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_BRICK, &args.brick,
-                                     GWY_APP_BRICK_ID, &id,
-                                     0);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_BRICK, &args.brick,
+                                 GWY_APP_BRICK_ID, &id,
+                                 0);
     g_return_if_fail(GWY_IS_BRICK(args.brick));
 
     brick = args.brick;
     args.calibration = gwy_brick_get_zcalibration(brick);
-    if (args.calibration && gwy_brick_get_zres(brick) != gwy_data_line_get_res(args.calibration))
+    if (args.calibration && gwy_brick_get_zres(brick) != gwy_line_get_res(args.calibration))
         args.calibration = NULL;
     args.params = gwy_params_new_from_settings(define_module_params());
     sanitise_params(&args);
-    args.result = gwy_data_field_new(1, 1, 1.0, 1.0, TRUE);
-    gwy_brick_extract_xy_plane(brick, args.result, 0);
-    gwy_data_field_clear(args.result);
-    /* Do this after gwy_brick_extract_xy_plane() because the units actually differ. */
-    zunit = (args.calibration ? gwy_data_line_get_si_unit_y(args.calibration) : gwy_brick_get_si_unit_z(args.brick));
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_z(args.result), zunit);
+    args.result = gwy_brick_new_field_like_xy_plane(args.brick, TRUE);
+    /* Do this after because the units actually differ. */
+    zunit = (args.calibration ? gwy_line_get_unit_y(args.calibration) : gwy_brick_get_unit_z(args.brick));
+    gwy_unit_assign(gwy_field_get_unit_z(args.result), zunit);
 
     outcome = run_gui(&args, data, id);
     gwy_params_save_to_settings(args.params);
@@ -214,45 +204,40 @@ equiplane(GwyContainer *data, GwyRunType run)
         execute(&args);
 
     zvf = make_zvf(brick);
-    newid = gwy_app_data_browser_add_data_field(args.result, data, TRUE);
+    newid = gwy_file_add_image(data, args.result);
+    gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
     const_value = get_constant_value(&args);
     title = g_strdup_printf(_("Isosurface z for %.*f %s"), zvf->precision, const_value/zvf->magnitude, zvf->units);
-    gwy_container_set_string(data, gwy_app_get_data_key_for_id(newid), title);
-    gwy_app_channel_log_add(data, -1, newid, "volume::volume_equiplane", NULL);
-    gwy_si_unit_value_format_free(zvf);
+    gwy_file_pass_title(data, GWY_FILE_IMAGE, newid, title);
+    gwy_log_add_full(data, GWY_FILE_IMAGE, -1, newid, "volume::volume_equiplane", NULL);
+    gwy_value_format_free(zvf);
 
 end:
-    GWY_OBJECT_UNREF(args.result);
+    g_clear_object(&args.result);
     g_object_unref(args.params);
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     GwyBrick *brick = args->brick;
-    GtkWidget *hbox, *align;
+    GtkWidget *hbox;
     GwyGraphArea *area;
     GwyGraph *graph;
     GwyParamTable *table;
     GwyDialog *dialog;
     ModuleGUI gui;
     GwyDialogOutcome outcome;
-    const guchar *gradient;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
 
-    gui.data = gwy_container_new();
-    gui.xyplane = gwy_data_field_new_alike(args->result, FALSE);
+    gui.xyplane = gwy_field_new_alike(args->result, FALSE);
     extract_xyplane(&gui);
-    gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(0), args->result);
 
-    if (gwy_container_gis_string(data, gwy_app_get_brick_palette_key_for_id(id), &gradient))
-        gwy_container_set_const_string(gui.data, gwy_app_get_data_palette_key_for_id(0), gradient);
-
-    gui.vf = gwy_si_unit_get_format_with_digits(gwy_brick_get_si_unit_w(brick), GWY_SI_UNIT_FORMAT_VFMARKUP,
-                                                gwy_brick_get_max(brick) - gwy_brick_get_min(brick), 5, /* 5 digits */
-                                                NULL);
+    gui.vf = gwy_unit_get_format_with_digits(gwy_brick_get_unit_w(brick), GWY_UNIT_FORMAT_VFMARKUP,
+                                             gwy_brick_get_max(brick) - gwy_brick_get_min(brick), 5, /* 5 digits */
+                                             NULL);
 
     gui.gmodel = gwy_graph_model_new();
     setup_gmodel(args, gui.gmodel);
@@ -261,15 +246,15 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_UPDATE, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    hbox = gwy_hbox_new(0);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
     gwy_dialog_add_content(dialog, hbox, FALSE, FALSE, 4);
 
-    align = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-    gtk_box_pack_start(GTK_BOX(hbox), align, FALSE, FALSE, 0);
-
-    gui.dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
-    gtk_container_add(GTK_CONTAINER(align), gui.dataview);
-    gui.image_selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(gui.dataview), 0, "Point", 1, TRUE);
+    gui.dataview = gwy_create_preview(args->result, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(gui.dataview), data, GWY_FILE_VOLUME, id, GWY_FILE_ITEM_PALETTE);
+    gtk_widget_set_halign(gui.dataview, GTK_ALIGN_START);
+    gtk_widget_set_valign(gui.dataview, GTK_ALIGN_START);
+    gtk_box_pack_start(GTK_BOX(hbox), gui.dataview, FALSE, FALSE, 0);
+    gui.image_selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(gui.dataview), GWY_TYPE_LAYER_POINT, 1, TRUE);
 
     graph = GWY_GRAPH(gwy_graph_new(gui.gmodel));
     gwy_graph_enable_user_input(graph, FALSE);
@@ -281,7 +266,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gui.graph_selection = gwy_graph_area_get_selection(area, GWY_GRAPH_STATUS_XLINES);
     gwy_selection_set_max_objects(gui.graph_selection, 1);
 
-    hbox = gwy_hbox_new(24);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 24);
     gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), hbox, TRUE, TRUE, 4);
 
     gui.table_value = table = gwy_param_table_new(args->params);
@@ -316,8 +301,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 
     g_object_unref(gui.gmodel);
     g_object_unref(gui.xyplane);
-    g_object_unref(gui.data);
-    gwy_si_unit_value_format_free(gui.vf);
+    gwy_value_format_free(gui.vf);
 
     return outcome;
 }
@@ -364,7 +348,7 @@ static void
 update_constant_value(ModuleGUI *gui)
 {
     ModuleArgs *args = gui->args;
-    GwySIValueFormat *vf = gui->vf;
+    GwyValueFormat *vf = gui->vf;
     gdouble const_value = get_constant_value(args);
     gchar *s;
 
@@ -388,17 +372,15 @@ param_changed(ModuleGUI *gui, gint id)
          * original data are show. */
         if (show_type == SHOW_DATA && !gwy_params_get_boolean(params, PARAM_UPDATE)) {
             extract_xyplane(gui);
-            gwy_data_field_data_changed(gui->xyplane);
+            gwy_field_data_changed(gui->xyplane);
         }
     }
     if (id < 0 || id == PARAM_XPOS || id == PARAM_YPOS)
         extract_graph_curve(args, gwy_graph_model_get_curve(gui->gmodel, 0));
     if (id < 0 || id == PARAM_ZPOS || id == PARAM_XPOS || id == PARAM_YPOS)
         update_constant_value(gui);
-    if (id < 0 || id == PARAM_SHOW_TYPE) {
-        gwy_container_set_object(gui->data, gwy_app_get_data_key_for_id(0),
-                                 show_type == SHOW_DATA ? gui->xyplane : args->result);
-    }
+    if (id < 0 || id == PARAM_SHOW_TYPE)
+        gwy_data_view_set_field(GWY_DATA_VIEW(gui->dataview), show_type == SHOW_DATA ? gui->xyplane : args->result);
     if (id != PARAM_UPDATE)
         gwy_dialog_invalidate(GWY_DIALOG(gui->dialog));
 }
@@ -421,13 +403,13 @@ preview(gpointer user_data)
 
     if (show_type == SHOW_RESULT) {
         execute(args);
-        gwy_data_field_data_changed(args->result);
+        gwy_field_data_changed(args->result);
         gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
     }
     /* Keep the result invalidated when showing the original data not running execute()! */
     if (show_type == SHOW_DATA) {
         extract_xyplane(gui);
-        gwy_data_field_data_changed(gui->xyplane);
+        gwy_field_data_changed(gui->xyplane);
     }
 }
 
@@ -465,14 +447,14 @@ static void
 isosurface_nearest(ModuleArgs *args)
 {
     GwyBrick *brick = args->brick;
-    GwyDataField *field = args->result;
+    GwyField *field = args->result;
     gint xres = gwy_brick_get_xres(brick), yres = gwy_brick_get_yres(brick), zres = gwy_brick_get_zres(brick);
     gint zpos = gwy_params_get_int(args->params, PARAM_ZPOS);
     gint scan_dir = (gint)gwy_params_get_enum(args->params, PARAM_SEARCH_DIR);
     gint k, dist, lev, dir;
     gdouble const_value = get_constant_value(args);
     const gdouble *lowerplane, *upperplane, *d = gwy_brick_get_data_const(brick);
-    gdouble *f = gwy_data_field_get_data(field);
+    gdouble *f = gwy_field_get_data(field);
     gint n = xres*yres, remaining = n;
     gint *intersection = g_new(gint, n);
     gboolean find_increasing, find_decreasing;
@@ -565,20 +547,20 @@ extract_graph_curve(ModuleArgs *args, GwyGraphCurveModel *gcmodel)
     gint zres = gwy_brick_get_zres(brick);
     gint xpos = gwy_params_get_int(args->params, PARAM_XPOS);
     gint ypos = gwy_params_get_int(args->params, PARAM_YPOS);
-    GwyDataLine *line, *calibration = args->calibration;
+    GwyLine *line, *calibration = args->calibration;
 
-    line = gwy_data_line_new(1, 1.0, FALSE);
+    line = gwy_line_new(1, 1.0, FALSE);
     gwy_brick_extract_line(brick, line, xpos, ypos, 0, xpos, ypos, zres, FALSE);
-    gwy_data_line_set_offset(line, gwy_brick_get_zoffset(brick));
+    gwy_line_set_offset(line, gwy_brick_get_zoffset(brick));
 
     if (calibration) {
         gwy_graph_curve_model_set_data(gcmodel,
-                                       gwy_data_line_get_data(calibration), gwy_data_line_get_data(line),
-                                       gwy_data_line_get_res(line));
+                                       gwy_line_get_data(calibration), gwy_line_get_data(line),
+                                       gwy_line_get_res(line));
         gwy_graph_curve_model_enforce_order(gcmodel);
     }
     else
-        gwy_graph_curve_model_set_data_from_dataline(gcmodel, line, 0, 0);
+        gwy_graph_curve_model_set_data_from_line(gcmodel, line, 0, 0);
 
     g_object_unref(line);
 }
@@ -597,14 +579,14 @@ static void
 setup_gmodel(ModuleArgs *args, GwyGraphModel *gmodel)
 {
     GwyBrick *brick = args->brick;
-    GwyDataLine *calibration = args->calibration;
-    GwySIUnit *xunit = (calibration ? gwy_data_line_get_si_unit_y(calibration) : gwy_brick_get_si_unit_z(brick));
+    GwyLine *calibration = args->calibration;
+    GwyUnit *xunit = (calibration ? gwy_line_get_unit_y(calibration) : gwy_brick_get_unit_z(brick));
     GwyGraphCurveModel *gcmodel;
 
     g_object_set(gmodel,
                  "label-visible", FALSE,
-                 "si-unit-x", xunit,
-                 "si-unit-y", gwy_brick_get_si_unit_w(brick),
+                 "unit-x", xunit,
+                 "unit-y", gwy_brick_get_unit_w(brick),
                  "axis-label-bottom", "z",
                  "axis-label-left", "w",
                  NULL);
