@@ -1,5 +1,5 @@
 /*
- *  $Id: volume_lawnize.c 25241 2023-02-23 16:00:42Z klapetek $
+ *  $Id: volume_lawnize.c 28808 2025-11-05 18:26:20Z yeti-dn $
  *  Copyright (C) 2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,19 +16,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/brick.h>
-#include <libprocess/lawn.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwydgets/gwydgetutils.h>
-#include <libgwymodule/gwymodule-volume.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
+#include <gwy.h>
 
 #define RUN_MODES (GWY_RUN_INTERACTIVE)
 
@@ -56,7 +47,7 @@ typedef struct {
     GwyBrick *brick;
     GwyLawn *result;
     /* Cached input brick info. */
-    GwyDataLine *calibration;
+    GwyLine *calibration;
 } ModuleArgs;
 
 typedef struct {
@@ -67,13 +58,13 @@ typedef struct {
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             lawnize             (GwyContainer *data,
-                                             GwyRunType runtype);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static void             execute             (ModuleArgs *args,
-                                             GwyContainer *data,
+                                             GwyFile *data,
                                              gint id);
 static GwyDialogOutcome run_gui             (ModuleArgs *args);
-static gboolean         other_brick_filter  (GwyContainer *data,
+static gboolean         other_brick_filter  (GwyFile *data,
                                              gint id,
                                              gpointer user_data);
 static void             param_changed       (ModuleGUI *gui,
@@ -95,7 +86,7 @@ static gboolean
 module_register(void)
 {
     gwy_volume_func_register("volume_lawnize",
-                             (GwyVolumeFunc)&lawnize,
+                             module_main,
                              N_("/_Basic Operations/Convert to _Curve Map..."),
                              NULL,
                              RUN_MODES,
@@ -130,26 +121,26 @@ define_module_params(void)
 }
 
 static void
-lawnize(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
     GwyBrick *brick = NULL;
-    GwyDataField *preview;
+    GwyField *preview;
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     gint oldid, newid;
 
-    g_return_if_fail(runtype & RUN_MODES);
+    g_return_if_fail(mode & RUN_MODES);
 
-    gwy_app_data_browser_get_current(GWY_APP_BRICK, &brick,
-                                     GWY_APP_BRICK_ID, &oldid,
-                                     0);
+    gwy_data_browser_get_current(GWY_APP_BRICK, &brick,
+                                 GWY_APP_BRICK_ID, &oldid,
+                                 0);
     g_return_if_fail(GWY_IS_BRICK(brick));
     args.result = NULL;
     args.brick = brick;
     args.calibration = gwy_brick_get_zcalibration(brick);
     args.params = gwy_params_new_from_settings(define_module_params());
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -157,14 +148,14 @@ lawnize(GwyContainer *data, GwyRunType runtype)
     }
     execute(&args, data, oldid);
 
-    preview = gwy_container_get_object(data, gwy_app_get_brick_preview_key_for_id(oldid));
-    preview = gwy_data_field_duplicate(preview);
-    newid = gwy_app_data_browser_add_lawn(args.result, preview, data, TRUE);
+    preview = gwy_field_copy(gwy_container_get_object(GWY_CONTAINER(data), gwy_file_key_volume_picture(oldid)));
+    newid = gwy_file_add_cmap(data, args.result, preview);
+    gwy_file_set_visible(data, GWY_FILE_CMAP, newid, TRUE);
     g_object_unref(preview);
-    gwy_app_curve_map_log_add(data, -1, newid, "volume::volume_lawnize", NULL);
+    gwy_log_add_full(data, GWY_FILE_CMAP, -1, newid, "volume::volume_lawnize", NULL);
 
 end:
-    GWY_OBJECT_UNREF(args.result);
+    g_clear_object(&args.result);
     g_object_unref(args.params);
 }
 
@@ -216,14 +207,14 @@ param_changed(ModuleGUI *gui, gint id)
 }
 
 typedef struct {
-    GwyContainer *container;
+    GwyFile *container;
     gint id;
     GwyBrick *brick;
     const gdouble *data;
 } OtherData;
 
 static GArray*
-gather_other_bricks(ModuleArgs *args, GwyContainer *data, gint id)
+gather_other_bricks(ModuleArgs *args, GwyFile *data, gint id)
 {
     GwyParams *params = args->params;
     GArray *otherdata = g_array_new(FALSE, FALSE, sizeof(OtherData));
@@ -242,7 +233,7 @@ gather_other_bricks(ModuleArgs *args, GwyContainer *data, gint id)
 
         item.brick = gwy_params_get_volume(params, PARAM_ADD1 + i);
         dataid = gwy_params_get_data_id(params, PARAM_ADD1 + i);
-        item.container = gwy_app_data_browser_get(dataid.datano);
+        item.container = gwy_data_browser_get_file(dataid.datano);
         item.id = dataid.id;
         g_array_append_val(otherdata, item);
     }
@@ -251,7 +242,7 @@ gather_other_bricks(ModuleArgs *args, GwyContainer *data, gint id)
 }
 
 static GArray*
-gather_bricks_in_file(ModuleArgs *args, GwyContainer *data)
+gather_bricks_in_file(ModuleArgs *args, GwyFile *data)
 {
     GArray *otherdata = g_array_new(FALSE, FALSE, sizeof(OtherData));
     GwyBrick *brick = args->brick;
@@ -259,12 +250,12 @@ gather_bricks_in_file(ModuleArgs *args, GwyContainer *data)
     gint *ids;
     gint i;
 
-    ids = gwy_app_data_browser_get_volume_ids(data);
+    ids = gwy_file_get_ids(data, GWY_FILE_VOLUME);
     for (i = 0; ids[i] >= 0; i++) {
         if (!other_brick_filter(data, ids[i], brick))
             continue;
 
-        item.brick = gwy_container_get_object(data, gwy_app_get_brick_key_for_id(ids[i]));
+        item.brick = gwy_file_get_volume(data, ids[i]);
         item.container = data;
         item.id = ids[i];
         g_array_append_val(otherdata, item);
@@ -275,11 +266,11 @@ gather_bricks_in_file(ModuleArgs *args, GwyContainer *data)
 }
 
 static void
-execute(ModuleArgs *args, GwyContainer *data, gint id)
+execute(ModuleArgs *args, GwyFile *data, gint id)
 {
     GwyParams *params = args->params;
     GwyBrick *brick = args->brick;
-    GwyDataLine *calibration = gwy_params_get_boolean(params, PARAM_ZCAL) ? args->calibration : NULL;
+    GwyLine *calibration = gwy_params_get_boolean(params, PARAM_ZCAL) ? args->calibration : NULL;
     gint xres = gwy_brick_get_xres(brick), yres = gwy_brick_get_yres(brick), zres = gwy_brick_get_zres(brick);
     const gdouble *caldata;
     gdouble *curvedata;
@@ -287,7 +278,6 @@ execute(ModuleArgs *args, GwyContainer *data, gint id)
     gint i, j, k, m, n, nbricks, ncurves;
     GArray *allbricks_array;
     OtherData *allbricks;
-    gchar *s;
 
     if (gwy_params_get_boolean(params, PARAM_ALL))
         allbricks_array = gather_bricks_in_file(args, data);
@@ -297,7 +287,7 @@ execute(ModuleArgs *args, GwyContainer *data, gint id)
 
     nbricks = allbricks_array->len;
     ncurves = nbricks + !!calibration;
-    caldata = calibration ? gwy_data_line_get_data(calibration) : NULL;
+    caldata = calibration ? gwy_line_get_data(calibration) : NULL;
     for (m = 0; m < nbricks; m++)
         allbricks[m].data = gwy_brick_get_data_const(allbricks[m].brick);
 
@@ -321,20 +311,21 @@ execute(ModuleArgs *args, GwyContainer *data, gint id)
         }
     }
 
-    gwy_si_unit_assign(gwy_lawn_get_si_unit_xy(lawn), gwy_brick_get_si_unit_x(brick));
+    gwy_unit_assign(gwy_lawn_get_unit_xy(lawn), gwy_brick_get_unit_x(brick));
     n = 0;
 
     if (calibration) {
-        gwy_si_unit_assign(gwy_lawn_get_si_unit_curve(lawn, n), gwy_data_line_get_si_unit_y(calibration));
+        gwy_unit_assign(gwy_lawn_get_unit_curve(lawn, n), gwy_line_get_unit_y(calibration));
         gwy_lawn_set_curve_label(lawn, n, _("Z calibration"));
         n++;
     }
 
     for (m = 0; m < nbricks; m++) {
-        gwy_si_unit_assign(gwy_lawn_get_si_unit_curve(lawn, n), gwy_brick_get_si_unit_w(allbricks[m].brick));
-        s = gwy_app_get_brick_title(allbricks[m].container, allbricks[m].id);
-        gwy_lawn_set_curve_label(lawn, n, s);
-        g_free(s);
+        gwy_unit_assign(gwy_lawn_get_unit_curve(lawn, n), gwy_brick_get_unit_w(allbricks[m].brick));
+
+        const gchar *s;
+        if ((s = gwy_file_get_title(allbricks[m].container, GWY_FILE_VOLUME, allbricks[m].id)))
+            gwy_lawn_set_curve_label(lawn, n, s);
         n++;
     }
 
@@ -343,16 +334,15 @@ execute(ModuleArgs *args, GwyContainer *data, gint id)
 }
 
 static gboolean
-other_brick_filter(GwyContainer *data, gint id, gpointer user_data)
+other_brick_filter(GwyFile *data, gint id, gpointer user_data)
 {
     GwyBrick *otherbrick, *brick = (GwyBrick*)user_data;
 
-    if (!gwy_container_gis_object(data, gwy_app_get_brick_key_for_id(id), &otherbrick))
+    if (!(otherbrick = gwy_file_get_volume(data, id)))
         return FALSE;
     /* Ignore zcal, we take it from the master brick. */
-    return !gwy_brick_check_compatibility(brick, otherbrick,
-                                          GWY_DATA_COMPATIBILITY_RES
-                                          | GWY_DATA_COMPATIBILITY_REAL
-                                          | GWY_DATA_COMPATIBILITY_LATERAL);
+    return !gwy_brick_is_incompatible(brick, otherbrick,
+                                      GWY_DATA_MISMATCH_RES | GWY_DATA_MISMATCH_REAL | GWY_DATA_MISMATCH_LATERAL);
 }
+
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
