@@ -1,5 +1,5 @@
 /*
- *  $Id: volume_linelevel.c 26554 2024-08-19 11:16:05Z yeti-dn $
+ *  $Id: volume_linelevel.c 29061 2026-01-02 15:01:53Z yeti-dn $
  *  Copyright (C) 2015-2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,33 +16,17 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwythreads.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwynlfit.h>
-#include <libprocess/brick.h>
-#include <libprocess/stats.h>
-#include <libprocess/linestats.h>
-#include <libprocess/gwyprocess.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libprocess/correlation.h>
-#include <libgwydgets/gwydataview.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwydgets/gwydgetutils.h>
-#include <libgwymodule/gwymodule-volume.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
-#include "libgwyddion/gwyomp.h"
+#include <gwy.h>
+#include "libgwyddion/omp.h"
 
 #define RUN_MODES (GWY_RUN_INTERACTIVE)
 
 /* Lower symmetric part indexing; i MUST be greater or equal than j */
 #define SLi(a, i, j) a[(i)*((i) + 1)/2 + (j)]
 
-
 enum {
     PREVIEW_SIZE = 360,
     MAX_DEGREE = 5,
@@ -79,57 +63,56 @@ typedef struct {
 typedef struct {
     ModuleArgs *args;
     GtkWidget *dialog;
+    GtkWidget *dataview;
     GwyParamTable *table_options;
-    GwyContainer *data;
 } ModuleGUI;
 
-
-static gboolean              module_register          (void);
-static GwyParamDef*          define_module_params     (void);
-static void                  line_level               (GwyContainer *data,
-                                                       GwyRunType runtype);
-static void                  execute                  (ModuleArgs *args);
-static GwyDialogOutcome      run_gui                  (ModuleArgs *args,
-                                                       GwyContainer *data,
-                                                       gint id);
-static void                  param_changed            (ModuleGUI *gui,
-                                                       gint id);
-static void                  dialog_response          (GwyDialog *dialog,
-                                                       gint response,
-                                                       ModuleGUI *gui);
-static void                  preview                  (gpointer user_data);
-static void                  update_image             (ModuleGUI *gui,
-                                                       gint z);
-static void                  linematch_do_trimmed_mean(GwyDataField *field,
-                                                       GwyDataField *mask,
-                                                       GwyDataLine *shifts,
-                                                       GwyMaskingType masking,
-                                                       gdouble trim_fraction);
-static void                  linematch_do_trimmed_diff(GwyDataField *field,
-                                                       GwyDataField *mask,
-                                                       GwyDataLine *shifts,
-                                                       GwyMaskingType masking,
-                                                       gdouble trim_fraction);
-static void                  linematch_do_modus       (GwyDataField *field,
-                                                       GwyDataField *mask,
-                                                       GwyDataLine *shifts,
-                                                       GwyMaskingType masking);
-static void                  linematch_do_match       (GwyDataField *field,
-                                                       GwyDataField *mask,
-                                                       GwyDataLine *shifts,
-                                                       GwyMaskingType masking);
-static void                  linematch_do_facet_tilt  (GwyDataField *field,
-                                                       GwyDataField *mask,
-                                                       GwyDataLine *shifts,
-                                                       GwyMaskingType masking);
-static void                  zero_level_row_shifts    (GwyDataLine *shifts);
+static gboolean         module_register          (void);
+static GwyParamDef*     define_module_params     (void);
+static void             module_main              (GwyFile *data,
+                                                  GwyRunModeFlags mode);
+static void             execute                  (ModuleArgs *args);
+static GwyDialogOutcome run_gui                  (ModuleArgs *args,
+                                                  GwyFile *data,
+                                                  gint id);
+static void             param_changed            (ModuleGUI *gui,
+                                                  gint id);
+static void             dialog_response          (GwyDialog *dialog,
+                                                  gint response,
+                                                  ModuleGUI *gui);
+static void             preview                  (gpointer user_data);
+static void             update_image             (ModuleGUI *gui,
+                                                  gint z);
+static void             linematch_do_trimmed_mean(GwyField *field,
+                                                  GwyField *mask,
+                                                  GwyLine *shifts,
+                                                  GwyMaskingType masking,
+                                                  gdouble trim_fraction);
+static void             linematch_do_trimmed_diff(GwyField *field,
+                                                  GwyField *mask,
+                                                  GwyLine *shifts,
+                                                  GwyMaskingType masking,
+                                                  gdouble trim_fraction);
+static void             linematch_do_modus       (GwyField *field,
+                                                  GwyField *mask,
+                                                  GwyLine *shifts,
+                                                  GwyMaskingType masking);
+static void             linematch_do_match       (GwyField *field,
+                                                  GwyField *mask,
+                                                  GwyLine *shifts,
+                                                  GwyMaskingType masking);
+static void             linematch_do_facet_tilt  (GwyField *field,
+                                                  GwyField *mask,
+                                                  GwyLine *shifts,
+                                                  GwyMaskingType masking);
+static void             zero_level_row_shifts    (GwyLine *shifts);
 
 static const GwyEnum methods[] = {
-    { N_("linematch|Polynomial"),        LINE_MATCH_POLY,         },
+    { gwy_NC("linematch", "Polynomial"),        LINE_MATCH_POLY,         },
     { N_("Median"),                      LINE_MATCH_MEDIAN,       },
     { N_("Median of differences"),       LINE_MATCH_MEDIAN_DIFF,  },
     { N_("Modus"),                       LINE_MATCH_MODUS,        },
-    { N_("linematch|Matching"),          LINE_MATCH_MATCH,        },
+    { gwy_NC("linematch", "Matching"),          LINE_MATCH_MATCH,        },
     { N_("Trimmed mean"),                LINE_MATCH_TRIMMED_MEAN, },
     { N_("Trimmed mean of differences"), LINE_MATCH_TMEAN_DIFF,   },
     { N_("Facet-level tilt"),            LINE_MATCH_FACET_TILT,   },
@@ -152,7 +135,7 @@ static gboolean
 module_register(void)
 {
     gwy_volume_func_register("volume_linelevel",
-                             (GwyVolumeFunc)&line_level,
+                             module_main,
                              N_("/_Correct Data/XY _Align Rows..."),
                              NULL,
                              RUN_MODES,
@@ -184,25 +167,24 @@ define_module_params(void)
 }
 
 static void
-line_level(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
     GwyBrick *brick = NULL;
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     gint oldid, newid;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    g_return_if_fail(g_type_from_name("GwyLayerPoint"));
+    g_return_if_fail(mode & RUN_MODES);
 
-    gwy_app_data_browser_get_current(GWY_APP_BRICK, &brick,
-                                     GWY_APP_BRICK_ID, &oldid,
-                                     0);
+    gwy_data_browser_get_current(GWY_APP_BRICK, &brick,
+                                 GWY_APP_BRICK_ID, &oldid,
+                                 0);
     g_return_if_fail(GWY_IS_BRICK(brick));
     args.result = NULL;
     args.brick = brick;
     args.params = gwy_params_new_from_settings(define_module_params());
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, oldid);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -211,14 +193,16 @@ line_level(GwyContainer *data, GwyRunType runtype)
     if (outcome != GWY_DIALOG_HAVE_RESULT)
         execute(&args);
 
-    newid = gwy_app_data_browser_add_brick(args.result, NULL, data, TRUE);
+    newid = gwy_file_add_volume(data, args.result, NULL);
 
-    gwy_app_set_brick_title(data, newid, _("Line leveled"));
-    gwy_app_sync_volume_items(data, data, oldid, newid, FALSE,
-                              GWY_DATA_ITEM_GRADIENT,
-                              0);
+    gwy_file_set_visible(data, GWY_FILE_VOLUME, newid, TRUE);
 
-    gwy_app_volume_log_add_volume(data, -1, newid);
+    gwy_file_set_title(data, GWY_FILE_VOLUME, newid, _("Line leveled"), TRUE);
+    gwy_file_sync_items(data, GWY_FILE_VOLUME, oldid,
+                        data, GWY_FILE_VOLUME, newid,
+                        GWY_FILE_ITEM_PALETTE, FALSE);
+
+    gwy_log_add(data, GWY_FILE_VOLUME, -1, newid);
 
 end:
     g_object_unref(args.params);
@@ -226,36 +210,26 @@ end:
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     GtkWidget *hbox, *dataview;
     GwyParamTable *table;
     GwyDialog *dialog;
     ModuleGUI gui;
-    GwyDialogOutcome outcome;
     GwyBrick *brick = args->brick;
-    GwyDataField *field = gwy_data_field_new(gwy_brick_get_xres(brick),
-                                             gwy_brick_get_yres(brick),
-                                             gwy_brick_get_xreal(brick),
-                                             gwy_brick_get_yreal(brick),
-                                             TRUE);
-    const guchar *gradient;
+    GwyField *field = gwy_brick_new_field_like_xy_plane(brick, TRUE);
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
-    gui.data = gwy_container_new();
 
-    args->result = gwy_brick_duplicate(brick);
-
-    gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(0), field);
-    if (gwy_container_gis_string(data, gwy_app_get_brick_palette_key_for_id(id), &gradient))
-        gwy_container_set_const_string(gui.data, gwy_app_get_data_palette_key_for_id(0), gradient);
+    args->result = gwy_brick_copy(brick);
 
     gui.dialog = gwy_dialog_new(_("Line level"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_UPDATE, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
+    dataview = gui.dataview = gwy_create_preview(field, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_VOLUME, id, GWY_FILE_ITEM_PALETTE);
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), FALSE);
 
     table = gui.table_options = gwy_param_table_new(args->params);
@@ -285,14 +259,9 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     g_signal_connect_after(dialog, "response", G_CALLBACK(dialog_response), &gui);
     gwy_dialog_set_preview_func(dialog, GWY_PREVIEW_IMMEDIATE, preview, &gui, NULL);
 
-    outcome = gwy_dialog_run(dialog);
-
-    g_object_unref(gui.data);
-
-    return outcome;
+    return gwy_dialog_run(dialog);
 }
 
-
 static void
 param_changed(ModuleGUI *gui, gint id)
 {
@@ -304,7 +273,7 @@ static void
 dialog_response(G_GNUC_UNUSED GwyDialog *dialog, gint response, ModuleGUI *gui)
 {
     if (response == GWY_RESPONSE_RESET)
-        gwy_brick_copy(gui->args->brick, gui->args->result, FALSE);
+        gwy_brick_copy_data(gui->args->brick, gui->args->result);
 
     preview(gui);
 }
@@ -312,16 +281,14 @@ dialog_response(G_GNUC_UNUSED GwyDialog *dialog, gint response, ModuleGUI *gui)
 static void
 update_image(ModuleGUI *gui, gint z)
 {
-    GwyDataField *dfield;
     GwyBrick *brick = gui->args->result;
-    dfield = gwy_container_get_object(gui->data, gwy_app_get_data_key_for_id(0));
+    gint zres = gwy_brick_get_zres(brick);
+    GwyField *field = gwy_data_view_get_field(GWY_DATA_VIEW(gui->dataview));
 
-    gwy_brick_extract_xy_plane(brick, dfield, CLAMP(z, 0, brick->zres-1));
-
-    gwy_data_field_data_changed(dfield);
+    gwy_brick_extract_xy_plane(brick, field, CLAMP(z, 0, zres-1));
+    gwy_field_data_changed(field);
 }
 
-
 static void
 preview(gpointer user_data)
 {
@@ -340,8 +307,8 @@ execute(ModuleArgs *args)
     gint k;
     GwyBrick *original = args->brick;
     GwyBrick *brick = args->result;
-    GwyDataField *dfield, *myfield;
-    GwyDataLine *shifts;
+    GwyField *dfield, *myfield;
+    GwyLine *shifts;
     LineMatchMethod method = gwy_params_get_enum(params, PARAM_METHOD);
     gdouble trim_fraction = gwy_params_get_double(params, PARAM_TRIM_FRACTION);
     gint max_degree = gwy_params_get_int(params, PARAM_MAX_DEGREE);
@@ -351,8 +318,8 @@ execute(ModuleArgs *args)
     gint yres = gwy_brick_get_yres(brick);
     gint zres = gwy_brick_get_zres(brick);
 
-    dfield = gwy_data_field_new(xres, yres, gwy_brick_get_xreal(brick), gwy_brick_get_yreal(brick), FALSE);
-    shifts = gwy_data_line_new(gwy_data_field_get_yres(dfield), gwy_data_field_get_yreal(dfield), FALSE);
+    dfield = gwy_field_new(xres, yres, gwy_brick_get_xreal(brick), gwy_brick_get_yreal(brick), FALSE);
+    shifts = gwy_line_new(gwy_field_get_yres(dfield), gwy_field_get_yreal(dfield), FALSE);
 
     for (k = 0; k < zres; k++) {
         gwy_brick_extract_xy_plane(original, dfield, k);
@@ -360,8 +327,8 @@ execute(ModuleArgs *args)
         /* Transpose the fields if necessary. */
         myfield = dfield;
         if (direction == GWY_ORIENTATION_VERTICAL) {
-            myfield = gwy_data_field_new_alike(dfield, FALSE);
-            gwy_data_field_flip_xy(dfield, myfield, FALSE);
+            myfield = gwy_field_new_alike(dfield, FALSE);
+            gwy_field_transpose(dfield, myfield, FALSE);
         }
 
         /* Perform the correction. */
@@ -369,7 +336,7 @@ execute(ModuleArgs *args)
             if (max_degree == 0)
                 linematch_do_trimmed_mean(myfield, NULL, shifts, GWY_MASK_IGNORE, 0.0);
             else
-                gwy_data_field_row_level_poly(myfield, NULL, GWY_MASK_IGNORE, max_degree, shifts);
+                gwy_field_row_level_poly(myfield, NULL, GWY_MASK_IGNORE, max_degree, shifts);
         }
         else if (method == LINE_MATCH_MEDIAN)
             linematch_do_trimmed_mean(myfield, NULL, shifts, GWY_MASK_IGNORE, 0.5);
@@ -391,7 +358,7 @@ execute(ModuleArgs *args)
 
         /* Transpose back if necessary. */
         if (direction == GWY_ORIENTATION_VERTICAL) {
-            gwy_data_field_flip_xy(myfield, dfield, FALSE);
+            gwy_field_transpose(myfield, dfield, FALSE);
             g_object_unref(myfield);
         }
 
@@ -403,49 +370,49 @@ execute(ModuleArgs *args)
 }
 
 static void
-linematch_do_trimmed_mean(GwyDataField *field, GwyDataField *mask, GwyDataLine *shifts,
+linematch_do_trimmed_mean(GwyField *field, GwyField *mask, GwyLine *shifts,
                           GwyMaskingType masking, gdouble trimfrac)
 {
-    GwyDataLine *myshifts;
+    GwyLine *myshifts;
 
-    myshifts = gwy_data_field_find_row_shifts_trimmed_mean(field, mask, masking, trimfrac, 0);
-    gwy_data_field_subtract_row_shifts(field, myshifts);
-    gwy_data_line_assign(shifts, myshifts);
+    myshifts = gwy_field_find_row_shifts_trimmed_mean(field, mask, masking, trimfrac, 0);
+    gwy_field_subtract_row_shifts(field, myshifts);
+    gwy_line_assign(shifts, myshifts);
     g_object_unref(myshifts);
 }
 static void
-linematch_do_trimmed_diff(GwyDataField *field, GwyDataField *mask, GwyDataLine *shifts,
+linematch_do_trimmed_diff(GwyField *field, GwyField *mask, GwyLine *shifts,
                           GwyMaskingType masking, gdouble trimfrac)
 {
-    GwyDataLine *myshifts;
+    GwyLine *myshifts;
 
-    myshifts = gwy_data_field_find_row_shifts_trimmed_diff(field, mask, masking, trimfrac, 0);
-    gwy_data_field_subtract_row_shifts(field, myshifts);
-    gwy_data_line_assign(shifts, myshifts);
+    myshifts = gwy_field_find_row_shifts_trimmed_diff(field, mask, masking, trimfrac, 0);
+    gwy_field_subtract_row_shifts(field, myshifts);
+    gwy_line_assign(shifts, myshifts);
     g_object_unref(myshifts);
 }
 
 static void
-linematch_do_modus(GwyDataField *field, GwyDataField *mask, GwyDataLine *modi,
+linematch_do_modus(GwyField *field, GwyField *mask, GwyLine *modi,
                    GwyMaskingType masking)
 {
     gint xres, yres;
     const gdouble *d, *m;
     gdouble total_median;
 
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
-    total_median = gwy_data_field_area_get_median_mask(field, mask, masking, 0, 0, xres, yres);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
+    total_median = gwy_field_area_get_median(field, mask, masking, 0, 0, xres, yres);
 
-    d = gwy_data_field_get_data(field);
-    m = mask ? gwy_data_field_get_data_const(mask) : NULL;
+    d = gwy_field_get_data(field);
+    m = mask ? gwy_field_get_data_const(mask) : NULL;
 #ifdef _OPENMP
 #pragma omp parallel if(gwy_threads_are_enabled()) default(none) \
             shared(d,m,modi,xres,yres,masking,total_median)
 #endif
     {
-        GwyDataLine *line = gwy_data_line_new(xres, 1.0, FALSE);
-        gdouble *buf = gwy_data_line_get_data(line);
+        GwyLine *line = gwy_line_new(xres, 1.0, FALSE);
+        gdouble *buf = gwy_line_get_data(line);
         gint ifrom = gwy_omp_chunk_start(yres), ito = gwy_omp_chunk_end(yres);
         gint i;
 
@@ -466,12 +433,12 @@ linematch_do_modus(GwyDataField *field, GwyDataField *mask, GwyDataLine *modi,
             if (!count)
                 modus = total_median;
             else if (count < 9)
-                modus = gwy_math_median(count, buf);
+                modus = gwy_math_median(buf, count);
             else {
                 gint seglen = GWY_ROUND(sqrt(count)), bestj = 0;
                 gdouble diff, bestdiff = G_MAXDOUBLE;
 
-                gwy_math_sort(count, buf);
+                gwy_math_sort(buf, count);
                 for (j = 0; j + seglen-1 < count; j++) {
                     diff = buf[j + seglen-1] - buf[j];
                     if (diff < bestdiff) {
@@ -486,28 +453,29 @@ linematch_do_modus(GwyDataField *field, GwyDataField *mask, GwyDataLine *modi,
                 modus /= count;
             }
 
-            gwy_data_line_set_val(modi, i, modus);
+            gwy_line_set_val(modi, i, modus);
         }
 
         g_object_unref(line);
     }
 
     zero_level_row_shifts(modi);
-    gwy_data_field_subtract_row_shifts(field, modi);
+    gwy_field_subtract_row_shifts(field, modi);
 }
+
 static void
-linematch_do_match(GwyDataField *field, GwyDataField *mask, GwyDataLine *shifts,
+linematch_do_match(GwyField *field, GwyField *mask, GwyLine *shifts,
                    GwyMaskingType masking)
 {
     gint xres, yres, k;
     const gdouble *d, *m;
     gdouble *s;
 
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
-    d = gwy_data_field_get_data(field);
-    m = mask ? gwy_data_field_get_data_const(mask) : NULL;
-    s = gwy_data_line_get_data(shifts);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
+    d = gwy_field_get_data(field);
+    m = mask ? gwy_field_get_data_const(mask) : NULL;
+    s = gwy_line_get_data(shifts);
 
 #ifdef _OPENMP
 #pragma omp parallel if(gwy_threads_are_enabled()) default(none) \
@@ -582,8 +550,9 @@ linematch_do_match(GwyDataField *field, GwyDataField *mask, GwyDataLine *shifts,
         s[k] += s[k-1];
 
     zero_level_row_shifts(shifts);
-    gwy_data_field_subtract_row_shifts(field, shifts);
+    gwy_field_subtract_row_shifts(field, shifts);
 }
+
 static gdouble
 row_fit_facet_tilt(const gdouble *drow,
                    const gdouble *mrow,
@@ -676,8 +645,9 @@ untilt_row(gdouble *drow, guint res, gdouble bx)
         drow[i] -= bx*x;
     }
 }
+
 static void
-linematch_do_facet_tilt(GwyDataField *field, GwyDataField *mask, GwyDataLine *shifts,
+linematch_do_facet_tilt(GwyField *field, GwyField *mask, GwyLine *shifts,
                         GwyMaskingType masking)
 {
     guint xres, yres, i, j, mincount;
@@ -685,13 +655,13 @@ linematch_do_facet_tilt(GwyDataField *field, GwyDataField *mask, GwyDataLine *sh
     gdouble *data, *drow;
     gdouble dx, tilt;
 
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
-    dx = gwy_data_field_get_dx(field);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
+    dx = gwy_field_get_dx(field);
     mincount = GWY_ROUND(log(xres) + 1);
 
-    data = gwy_data_field_get_data(field);
-    mdata = mask ? gwy_data_field_get_data_const(mask) : NULL;
+    data = gwy_field_get_data(field);
+    mdata = mask ? gwy_field_get_data_const(mask) : NULL;
     for (i = 0; i < yres; i++) {
         drow = data + i*xres;
         mrow = mdata ? mdata + i*xres : NULL;
@@ -705,15 +675,13 @@ linematch_do_facet_tilt(GwyDataField *field, GwyDataField *mask, GwyDataLine *sh
 
     /* FIXME: Should we put the tilts there to confuse the user?  We need to
      * make sure all functions set the units correctly in such case. */
-    gwy_data_line_clear(shifts);
+    gwy_line_clear(shifts);
 }
 
 static void
-zero_level_row_shifts(GwyDataLine *shifts)
+zero_level_row_shifts(GwyLine *shifts)
 {
-    gwy_data_line_add(shifts, -gwy_data_line_get_avg(shifts));
+    gwy_line_add(shifts, -gwy_line_avg(shifts));
 }
 
-
-
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
