@@ -1,5 +1,5 @@
 /*
- *  $Id: volume_linestat.c 25565 2023-07-25 14:56:22Z yeti-dn $
+ *  $Id: volume_linestat.c 29023 2025-12-19 13:06:16Z yeti-dn $
  *  Copyright (C) 2015-2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,21 +16,11 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwythreads.h>
-#include <libprocess/brick.h>
-#include <libprocess/linestats.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libgwydgets/gwydataview.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwydgets/gwydgetutils.h>
-#include <libgwymodule/gwymodule-volume.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
-#include "libgwyddion/gwyomp.h"
+#include <gwy.h>
+#include "libgwyddion/omp.h"
 
 #define RUN_MODES (GWY_RUN_INTERACTIVE)
 
@@ -60,7 +50,7 @@ typedef enum {
     NOUTPUTS
 } LineStatOutput;
 
-typedef gdouble (*LineStatFunc)(GwyDataLine *dataline);
+typedef gdouble (*LineStatFunc)(GwyLine *line);
 
 typedef struct {
     const gchar *name;
@@ -71,7 +61,7 @@ typedef struct {
 typedef struct {
     GwyBrick *brick;
     const gdouble *db;
-    GwyDataLine *dline;
+    GwyLine *dline;
     gdouble *buf;
     guint npts;
     guint npixels;
@@ -82,11 +72,11 @@ typedef struct {
 typedef struct {
     GwyParams *params;
     GwyBrick *brick;
-    GwyDataField *result;
+    GwyField *result;
     /* Cached input brick info. */
     gboolean units_equal;
-    GwyDataLine *calibration;
-    GwySIUnit *zunit;
+    GwyLine *calibration;
+    GwyUnit *zunit;
     gdouble zmin;
     gdouble zmax;
 } ModuleArgs;
@@ -96,20 +86,19 @@ typedef struct {
     GtkWidget *dialog;
     GwyParamTable *table_quantity;
     GwyParamTable *table_options;
-    GwyContainer *data;
     GwySelection *image_selection;
     GwyGraphModel *gmodel;
     GwySelection *graph_selection;
-    GwySIValueFormat *vf;
+    GwyValueFormat *vf;
 } ModuleGUI;
 
 static gboolean                    module_register        (void);
 static GwyParamDef*                define_module_params   (void);
-static void                        line_stat              (GwyContainer *data,
-                                                           GwyRunType runtype);
+static void                        module_main            (GwyFile *data,
+                                                           GwyRunModeFlags mode);
 static void                        execute                (ModuleArgs *args);
 static GwyDialogOutcome            run_gui                (ModuleArgs *args,
-                                                           GwyContainer *data,
+                                                           GwyFile *data,
                                                            gint id);
 static gboolean                    quantity_filter        (const GwyEnum *enumval,
                                                            gpointer user_data);
@@ -129,32 +118,34 @@ static void                        graph_selection_changed(ModuleGUI *gui,
                                                            GwySelection *selection);
 static void                        update_graph_curve     (ModuleGUI *gui);
 static void                        update_current_value   (ModuleGUI *gui);
-static gdouble                     get_data_line_range    (GwyDataLine *dataline);
-static gdouble                     get_data_line_slope    (GwyDataLine *dataline);
-static gdouble                     get_data_line_Rz       (GwyDataLine *dataline);
-static gdouble                     get_data_line_Rt       (GwyDataLine *dataline);
+static gdouble                     get_line_minpos_i      (GwyLine *line);
+static gdouble                     get_line_maxpos_i      (GwyLine *line);
+static gdouble                     get_line_range         (GwyLine *line);
+static gdouble                     get_line_slope         (GwyLine *line);
+static gdouble                     get_line_Rz            (GwyLine *line);
+static gdouble                     get_line_Rt            (GwyLine *line);
 static const LineStatQuantityInfo* find_quantity          (GwyLineStatQuantity quantity);
 static void                        sanitise_params        (ModuleArgs *args);
 
 /* XXX: This is more or less identical to tools/linestat.c. */
 static const LineStatQuantityInfo quantities[] =  {
-    { N_("Mean"),               GWY_LINE_STAT_MEAN,      gwy_data_line_get_avg,       },
-    { N_("Median"),             GWY_LINE_STAT_MEDIAN,    gwy_data_line_get_median,    },
-    { N_("Minimum"),            GWY_LINE_STAT_MINIMUM,   gwy_data_line_get_min,       },
-    { N_("Maximum"),            GWY_LINE_STAT_MAXIMUM,   gwy_data_line_get_max,       },
-    { N_("Min. position"),      GWY_LINE_STAT_MINPOS,    gwy_data_line_min_pos_i,     },
-    { N_("Max. position"),      GWY_LINE_STAT_MAXPOS,    gwy_data_line_max_pos_i,     },
-    { N_("Range"),              GWY_LINE_STAT_RANGE,     get_data_line_range,         },
-    { N_("Slope"),              GWY_LINE_STAT_SLOPE,     get_data_line_slope,         },
-    { N_("tan β<sub>0</sub>"),  GWY_LINE_STAT_TAN_BETA0, gwy_data_line_get_tan_beta0, },
-    { N_("Variation"),          GWY_LINE_STAT_VARIATION, gwy_data_line_get_variation, },
-    { N_("Developed length"),   GWY_LINE_STAT_LENGTH,    gwy_data_line_get_length,    },
-    { N_("Ra"),                 GWY_LINE_STAT_RA,        gwy_data_line_get_ra,        },
-    { N_("Rq (RMS)"),           GWY_LINE_STAT_RMS,       gwy_data_line_get_rms,       },
-    { N_("Rz"),                 GWY_LINE_STAT_RZ,        get_data_line_Rz,            },
-    { N_("Rt"),                 GWY_LINE_STAT_RT,        get_data_line_Rt,            },
-    { N_("Skew"),               GWY_LINE_STAT_SKEW,      gwy_data_line_get_skew,      },
-    { N_("Excess kurtosis"),    GWY_LINE_STAT_KURTOSIS,  gwy_data_line_get_kurtosis,  },
+    { N_("Mean"),              GWY_LINE_STAT_MEAN,      gwy_line_avg,       },
+    { N_("Median"),            GWY_LINE_STAT_MEDIAN,    gwy_line_median,    },
+    { N_("Minimum"),           GWY_LINE_STAT_MINIMUM,   gwy_line_min,       },
+    { N_("Maximum"),           GWY_LINE_STAT_MAXIMUM,   gwy_line_max,       },
+    { N_("Min. position"),     GWY_LINE_STAT_MINPOS,    get_line_minpos_i,  },
+    { N_("Max. position"),     GWY_LINE_STAT_MAXPOS,    get_line_maxpos_i,  },
+    { N_("Range"),             GWY_LINE_STAT_RANGE,     get_line_range,     },
+    { N_("Slope"),             GWY_LINE_STAT_SLOPE,     get_line_slope,     },
+    { N_("tan β<sub>0</sub>"), GWY_LINE_STAT_TAN_BETA0, gwy_line_tan_beta0, },
+    { N_("Variation"),         GWY_LINE_STAT_VARIATION, gwy_line_variation, },
+    { N_("Developed length"),  GWY_LINE_STAT_LENGTH,    gwy_line_length,    },
+    { N_("Ra"),                GWY_LINE_STAT_RA,        gwy_line_ra,        },
+    { N_("Rq (RMS)"),          GWY_LINE_STAT_RMS,       gwy_line_rms,       },
+    { N_("Rz"),                GWY_LINE_STAT_RZ,        get_line_Rz,        },
+    { N_("Rt"),                GWY_LINE_STAT_RT,        get_line_Rt,        },
+    { N_("Skew"),              GWY_LINE_STAT_SKEW,      gwy_line_skew,      },
+    { N_("Excess kurtosis"),   GWY_LINE_STAT_KURTOSIS,  gwy_line_kurtosis,  },
 };
 
 static GwyModuleInfo module_info = {
@@ -173,9 +164,9 @@ static gboolean
 module_register(void)
 {
     gwy_volume_func_register("volume_linestat",
-                             (GwyVolumeFunc)&line_stat,
+                             module_main,
                              N_("/_Statistics/Summarize _Profiles..."),
-                             GWY_STOCK_VOLUME_LINE_STATS,
+                             GWY_ICON_VOLUME_LINE_STATS,
                              RUN_MODES,
                              GWY_MENU_FLAG_VOLUME,
                              N_("Summarize profiles"));
@@ -217,7 +208,7 @@ define_module_params(void)
 }
 
 static void
-line_stat(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
     GwyBrick *brick = NULL;
@@ -227,20 +218,19 @@ line_stat(GwyContainer *data, GwyRunType runtype)
     const gchar *title;
     gint oldid, newid;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    g_return_if_fail(g_type_from_name("GwyLayerPoint"));
+    g_return_if_fail(mode & RUN_MODES);
 
-    gwy_app_data_browser_get_current(GWY_APP_BRICK, &brick,
-                                     GWY_APP_BRICK_ID, &oldid,
-                                     0);
+    gwy_data_browser_get_current(GWY_APP_BRICK, &brick,
+                                 GWY_APP_BRICK_ID, &oldid,
+                                 0);
     g_return_if_fail(GWY_IS_BRICK(brick));
     args.result = NULL;
     args.brick = brick;
     args.params = gwy_params_new_from_settings(define_module_params());
-    args.result = gwy_data_field_new(gwy_brick_get_xres(brick), gwy_brick_get_yres(brick), 1.0, 1.0, TRUE);
+    args.result = gwy_brick_new_field_like_xy_plane(args.brick, TRUE);
     sanitise_params(&args);
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, oldid);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -252,13 +242,16 @@ line_stat(GwyContainer *data, GwyRunType runtype)
     output_type = gwy_params_get_enum(args.params, PARAM_OUTPUT_TYPE);
     quantity = gwy_params_get_enum(args.params, PARAM_QUANTITY);
     if (output_type == OUTPUT_IMAGE) {
-        newid = gwy_app_data_browser_add_data_field(args.result, data, TRUE);
-        title = gwy_sgettext(find_quantity(quantity)->name);
-        gwy_app_set_data_field_title(data, newid, title);
-        gwy_app_channel_log_add(data, -1, newid, "volume::volume_linestat", NULL);
+        newid = gwy_file_add_image(data, args.result);
+        gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+        title = gwy_C(find_quantity(quantity)->name);
+        gwy_file_set_title(data, GWY_FILE_IMAGE, newid, title, TRUE);
+        gwy_log_add_full(data, GWY_FILE_IMAGE, -1, newid, "volume::volume_linestat", NULL);
+    }
+    else if (output_type == OUTPUT_PREVIEW) {
+        /* FIXME: add undo */
+        gwy_container_set_object(GWY_CONTAINER(data), gwy_file_key_volume_picture(oldid), args.result);
     }
-    else if (output_type == OUTPUT_PREVIEW)
-        gwy_container_set_object(data, gwy_app_get_brick_preview_key_for_id(oldid), args.result);
     else {
         g_assert_not_reached();
     }
@@ -269,46 +262,41 @@ end:
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
-    GtkWidget *hbox, *graph, *area, *dataview, *align;
+    GtkWidget *hbox, *graph, *area, *dataview;
     GwyParamTable *table;
     GwyDialog *dialog;
     ModuleGUI gui;
-    GwyDataField *field = args->result;
+    GwyField *field = args->result;
     GwyGraphCurveModel *gcmodel;
     GwyDialogOutcome outcome;
     GwyBrick *brick = args->brick;
-    const guchar *gradient;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
-    gui.vf = gwy_si_unit_get_format_with_digits(args->zunit, GWY_SI_UNIT_FORMAT_VFMARKUP,
-                                                fmax(fabs(args->zmax), fabs(args->zmin)), 5, NULL);
-    gui.data = gwy_container_new();
-    gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(0), field);
-    if (gwy_container_gis_string(data, gwy_app_get_brick_palette_key_for_id(id), &gradient))
-        gwy_container_set_const_string(gui.data, gwy_app_get_data_palette_key_for_id(0), gradient);
+    gui.vf = gwy_unit_get_format_with_digits(args->zunit, GWY_UNIT_FORMAT_VFMARKUP,
+                                             fmax(fabs(args->zmax), fabs(args->zmin)), 5, NULL);
 
     gui.dialog = gwy_dialog_new(_("Summarize Volume Profiles"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_UPDATE, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    hbox = gwy_hbox_new(0);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
     gwy_dialog_add_content(dialog, hbox, TRUE, TRUE, 0);
 
-    align = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-    gtk_box_pack_start(GTK_BOX(hbox), align, FALSE, FALSE, 0);
-
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
-    gtk_container_add(GTK_CONTAINER(align), dataview);
-    gui.image_selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(dataview), 0, "Point", 1, TRUE);
+    dataview = gwy_create_preview(field, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_VOLUME, id, GWY_FILE_ITEM_PALETTE);
+    gtk_widget_set_halign(dataview, GTK_ALIGN_START);
+    gtk_widget_set_valign(dataview, GTK_ALIGN_START);
+    gtk_box_pack_start(GTK_BOX(hbox), dataview, FALSE, FALSE, 0);
+    gui.image_selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(dataview), GWY_TYPE_LAYER_POINT, 1, TRUE);
 
     gui.gmodel = gwy_graph_model_new();
     g_object_set(gui.gmodel,
                  "label-visible", FALSE,
-                 "si-unit-x", args->zunit,
-                 "si-unit-y", gwy_brick_get_si_unit_w(brick),
+                 "unit-x", args->zunit,
+                 "unit-y", gwy_brick_get_unit_w(brick),
                  NULL);
 
     gcmodel = gwy_graph_curve_model_new();
@@ -326,7 +314,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gui.graph_selection = gwy_graph_area_get_selection(GWY_GRAPH_AREA(area), GWY_GRAPH_STATUS_XSEL);
     gwy_selection_set_max_objects(gui.graph_selection, 1);
 
-    hbox = gwy_hbox_new(20);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 20);
     gwy_dialog_add_content(dialog, hbox, TRUE, TRUE, 4);
 
     table = gui.table_quantity = gwy_param_table_new(args->params);
@@ -361,9 +349,8 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 
     outcome = gwy_dialog_run(dialog);
 
-    g_object_unref(gui.data);
     g_object_unref(gui.gmodel);
-    gwy_si_unit_value_format_free(gui.vf);
+    gwy_value_format_free(gui.vf);
 
     return outcome;
 }
@@ -464,7 +451,7 @@ update_graph_curve(ModuleGUI *gui)
 {
     ModuleArgs *args = gui->args;
     GwyBrick *brick = args->brick;
-    GwyDataLine *line = gwy_data_line_new(1, 1.0, FALSE);
+    GwyLine *line = gwy_line_new(1, 1.0, FALSE);
     gint col = gwy_params_get_int(args->params, PARAM_XPOS);
     gint row = gwy_params_get_int(args->params, PARAM_YPOS);
     GwyGraphCurveModel *gcmodel = gwy_graph_model_get_curve(gui->gmodel, 0);
@@ -472,12 +459,12 @@ update_graph_curve(ModuleGUI *gui)
 
     gwy_brick_extract_line(brick, line, col, row, 0, col, row, gwy_brick_get_zres(brick), TRUE);
     if (args->calibration) {
-        xdata = gwy_data_line_get_data(args->calibration);
-        ydata = gwy_data_line_get_data(line);
+        xdata = gwy_line_get_data(args->calibration);
+        ydata = gwy_line_get_data(line);
         gwy_graph_curve_model_set_data(gcmodel, xdata, ydata, gwy_brick_get_zres(brick));
     }
     else
-        gwy_graph_curve_model_set_data_from_dataline(gcmodel, line, 0, 0);
+        gwy_graph_curve_model_set_data_from_line(gcmodel, line, 0, 0);
     g_object_unref(line);
 
     update_current_value(gui);
@@ -489,14 +476,14 @@ update_current_value(ModuleGUI *gui)
     ModuleArgs *args = gui->args;
     gint col = gwy_params_get_int(args->params, PARAM_XPOS);
     gint row = gwy_params_get_int(args->params, PARAM_YPOS);
-    GwySIValueFormat *vf;
-    GwySIUnit *unit;
+    GwyValueFormat *vf;
+    GwyUnit *unit;
     gdouble v;
     gchar *s;
 
-    v = gwy_data_field_get_val(args->result, col, row);
-    unit = gwy_data_field_get_si_unit_z(args->result);
-    vf = gui->vf = gwy_si_unit_get_format_with_digits(unit, GWY_SI_UNIT_FORMAT_VFMARKUP, v, 3, gui->vf);
+    v = gwy_field_get_val(args->result, col, row);
+    unit = gwy_field_get_unit_z(args->result);
+    vf = gui->vf = gwy_unit_get_format_with_digits(unit, GWY_UNIT_FORMAT_VFMARKUP, v, 3, gui->vf);
     s = g_strdup_printf("%.*f%s%s", vf->precision, v/vf->magnitude, *vf->units ? " " : "", vf->units);
     gwy_param_table_info_set_valuestr(gui->table_quantity, LABEL_VALUE, s);
     g_free(s);
@@ -536,7 +523,7 @@ preview(gpointer user_data)
     ModuleGUI *gui = (ModuleGUI*)user_data;
 
     execute(gui->args);
-    gwy_data_field_data_changed(gui->args->result);
+    gwy_field_data_changed(gui->args->result);
     update_graph_curve(gui);
     gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
 }
@@ -553,7 +540,7 @@ line_stat_iter_init(LineStatIter *iter, GwyBrick *brick,
     iter->planesize = brick->xres*brick->yres;
     iter->db = gwy_brick_get_data_const(brick) + zfrom*iter->planesize + kfrom;
     iter->buf = g_new(gdouble, MIN(BLOCK_SIZE, iter->npixels) * iter->npts);
-    iter->dline = gwy_data_line_new(1, 1.0, FALSE);
+    iter->dline = gwy_line_new(1, 1.0, FALSE);
     iter->k = (guint)(-1);
     /* Sets up line properties. */
     gwy_brick_extract_line(brick, iter->dline, 0, 0, zfrom, 0, 0, zto, TRUE);
@@ -579,45 +566,57 @@ line_stat_iter_next(LineStatIter *iter)
         }
         kk = 0;
     }
-    memcpy(iter->dline->data, iter->buf + kk*npts, npts * sizeof(gdouble));
+    gwy_assign(gwy_line_get_data(iter->dline), iter->buf + kk*npts, npts);
 }
 
 static void
 line_stat_iter_free(LineStatIter *iter)
 {
     g_free(iter->buf);
-    gwy_object_unref(iter->dline);
+    g_clear_object(&iter->dline);
 }
 
 static gdouble
-get_data_line_range(GwyDataLine *dataline)
+get_line_range(GwyLine *line)
 {
     gdouble min, max;
 
-    gwy_data_line_get_min_max(dataline, &min, &max);
+    gwy_line_min_max(line, &min, &max);
     return max - min;
 }
 
 static gdouble
-get_data_line_Rt(GwyDataLine *dataline)
+get_line_minpos_i(GwyLine *line)
 {
-    gwy_data_line_add(dataline, -gwy_data_line_get_avg(dataline));
-    return gwy_data_line_get_xtm(dataline, 1, 1);
+    return gwy_line_min_pos_i(line);
 }
 
 static gdouble
-get_data_line_Rz(GwyDataLine *dataline)
+get_line_maxpos_i(GwyLine *line)
 {
-    gwy_data_line_add(dataline, -gwy_data_line_get_avg(dataline));
-    return gwy_data_line_get_xtm(dataline, 5, 1);
+    return gwy_line_max_pos_i(line);
 }
 
 static gdouble
-get_data_line_slope(GwyDataLine *dataline)
+get_line_Rt(GwyLine *line)
+{
+    gwy_line_add(line, -gwy_line_avg(line));
+    return gwy_line_xtm(line, 1, 1);
+}
+
+static gdouble
+get_line_Rz(GwyLine *line)
+{
+    gwy_line_add(line, -gwy_line_avg(line));
+    return gwy_line_xtm(line, 5, 1);
+}
+
+static gdouble
+get_line_slope(GwyLine *line)
 {
     gdouble b;
-    gwy_data_line_get_line_coeffs(dataline, NULL, &b);
-    return b*gwy_data_line_get_res(dataline)/gwy_data_line_get_real(dataline);
+    gwy_line_fit_line(line, NULL, &b);
+    return b*gwy_line_get_res(line)/gwy_line_get_real(line);
 }
 
 static void
@@ -628,16 +627,16 @@ execute(ModuleArgs *args)
     gint zfrom = gwy_params_get_int(params, PARAM_ZFROM);
     gint zto = gwy_params_get_int(params, PARAM_ZTO);
     GwyBrick *brick = args->brick;
-    GwyDataField *field = args->result;
-    GwySIUnit *imgunit, *wunit;
-    GwyDataLine *calibration = args->calibration;
+    GwyField *field = args->result;
+    GwyUnit *imgunit, *wunit;
+    GwyLine *calibration = args->calibration;
     gint xres = gwy_brick_get_xres(brick), yres = gwy_brick_get_yres(brick), zres = gwy_brick_get_zres(brick);
     LineStatFunc lsfunc = NULL;
     gint i, j;
     guint k;
     gdouble *data, val, zreal, zoffset;
 
-    /* Quantities we handle (somewhat inefficiently) by using DataLine statistics. */
+    /* Quantities we handle (somewhat inefficiently) by using Line statistics. */
     lsfunc = find_quantity(quantity)->func;
 
     if (zfrom == -1 && zto == -1) {
@@ -654,6 +653,7 @@ execute(ModuleArgs *args)
             shared(brick,xres,yres,zfrom,zto,field,lsfunc)
 #endif
     {
+        gdouble *d = gwy_field_get_data(field);
         LineStatIter iter;
         guint kfrom = gwy_omp_chunk_start(xres*yres);
         guint kto = gwy_omp_chunk_end(xres*yres);
@@ -661,43 +661,43 @@ execute(ModuleArgs *args)
         line_stat_iter_init(&iter, brick, kfrom, kto, zfrom, zto);
         for (k = kfrom; k < kto; k++) {
             line_stat_iter_next(&iter);
-            field->data[k] = lsfunc(iter.dline);
+            d[k] = lsfunc(iter.dline);
         }
         line_stat_iter_free(&iter);
     }
 
     if ((quantity == GWY_LINE_STAT_MINPOS) || (quantity == GWY_LINE_STAT_MAXPOS)) {
-        gwy_data_field_add(field, zfrom);
+        gwy_field_add(field, zfrom);
         if (calibration) {
-            data = gwy_data_field_get_data(field);
+            data = gwy_field_get_data(field);
             for (i = 0; i < xres*yres; i++) {
                 j = *(data);
-                val = gwy_data_line_get_val(calibration, j);
+                val = gwy_line_get_val(calibration, j);
                 *(data++) = val;
             }
-            gwy_data_field_data_changed(field);
+            gwy_field_data_changed(field);
         }
         else {
             zreal = gwy_brick_get_zreal(brick);
             zoffset = gwy_brick_get_zoffset(brick);
-            gwy_data_field_multiply(field, zreal/zres);
-            gwy_data_field_add(field, zoffset);
+            gwy_field_multiply(field, zreal/zres);
+            gwy_field_add(field, zoffset);
         }
     }
 
-    imgunit = gwy_data_field_get_si_unit_z(field);
-    wunit = gwy_brick_get_si_unit_w(brick);
+    imgunit = gwy_field_get_unit_z(field);
+    wunit = gwy_brick_get_unit_w(brick);
 
     if ((quantity == GWY_LINE_STAT_MINPOS) || (quantity == GWY_LINE_STAT_MAXPOS))
-        gwy_si_unit_assign(gwy_data_field_get_si_unit_z(field), args->zunit);
+        gwy_unit_assign(gwy_field_get_unit_z(field), args->zunit);
     else if ((quantity == GWY_LINE_STAT_TAN_BETA0) || (quantity == GWY_LINE_STAT_SLOPE))
-        gwy_si_unit_divide(wunit, args->zunit, imgunit);
+        gwy_unit_divide(wunit, args->zunit, imgunit);
     else if ((quantity == GWY_LINE_STAT_SKEW) || (quantity == GWY_LINE_STAT_KURTOSIS))
-        gwy_si_unit_set_from_string(imgunit, NULL);
+        gwy_unit_clear(imgunit);
     else if (quantity == GWY_LINE_STAT_VARIATION)
-        gwy_si_unit_multiply(wunit, args->zunit, imgunit);
+        gwy_unit_multiply(wunit, args->zunit, imgunit);
 
-    gwy_data_field_invalidate(field);
+    gwy_field_invalidate(field);
 }
 
 static const LineStatQuantityInfo*
@@ -732,23 +732,23 @@ sanitise_params(ModuleArgs *args)
 {
     GwyParams *params = args->params;
     GwyBrick *brick = args->brick;
-    GwyDataLine *calibration;
-    GwySIUnit *wunit;
+    GwyLine *calibration;
+    GwyUnit *wunit;
 
     calibration = gwy_brick_get_zcalibration(brick);
-    if (calibration && (gwy_brick_get_zres(brick) != gwy_data_line_get_res(calibration)))
+    if (calibration && (gwy_brick_get_zres(brick) != gwy_line_get_res(calibration)))
         calibration = NULL;
     args->calibration = calibration;
 
-    wunit = gwy_brick_get_si_unit_w(brick);
-    args->zunit = (calibration ? gwy_data_line_get_si_unit_y(calibration) : gwy_brick_get_si_unit_z(brick));
-    args->units_equal = gwy_si_unit_equal(wunit, args->zunit);
+    wunit = gwy_brick_get_unit_w(brick);
+    args->zunit = (calibration ? gwy_line_get_unit_y(calibration) : gwy_brick_get_unit_z(brick));
+    args->units_equal = gwy_unit_equal(wunit, args->zunit);
     if (!args->units_equal) {
         if (gwy_params_get_enum(params, PARAM_QUANTITY) == GWY_LINE_STAT_LENGTH)
             gwy_params_set_enum(params, PARAM_QUANTITY, GWY_LINE_STAT_TAN_BETA0);
     }
-    args->zmin = (calibration ? gwy_data_line_get_min(calibration) : gwy_brick_get_zoffset(brick));
-    args->zmax = (calibration ? gwy_data_line_get_max(calibration) : args->zmin + gwy_brick_get_zreal(brick));
+    args->zmin = (calibration ? gwy_line_min(calibration) : gwy_brick_get_zoffset(brick));
+    args->zmax = (calibration ? gwy_line_max(calibration) : args->zmin + gwy_brick_get_zreal(brick));
 
     sanitise_one_param(params, PARAM_XPOS, 0, gwy_brick_get_xres(brick)-1, gwy_brick_get_xres(brick)/2);
     sanitise_one_param(params, PARAM_YPOS, 0, gwy_brick_get_yres(brick)-1, gwy_brick_get_yres(brick)/2);
