@@ -1,6 +1,6 @@
 /*
- *  $Id: volume_planestat.c 26510 2024-08-15 17:12:23Z yeti-dn $
- *  Copyright (C) 2015-2022 David Necas (Yeti).
+ *  $Id: volume_planestat.c 29059 2026-01-02 14:27:52Z yeti-dn $
+ *  Copyright (C) 2015-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -16,21 +16,11 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwythreads.h>
-#include <libprocess/brick.h>
-#include <libprocess/stats.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libgwydgets/gwydataview.h>
-#include <libgwydgets/gwylayer-basic.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-volume.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
-#include "libgwyddion/gwyomp.h"
+#include <gwy.h>
+#include "libgwyddion/omp.h"
 
 #define ENTROPY_NORMAL 1.41893853320467274178l
 
@@ -84,7 +74,7 @@ typedef enum {
     GWY_PLANE_STAT_FLAGS_ALL     = (1 << 2) - 1,
 } PlaneStatFlags;
 
-typedef gdouble (*PlaneStatFunc)(GwyDataField *datafield);
+typedef gdouble (*PlaneStatFunc)(GwyField *field);
 
 typedef struct {
     PlaneStatQuantity quantity;
@@ -104,20 +94,20 @@ typedef struct {
     /* Cached input data properties. */
     gboolean lateral_equal;
     gboolean all_equal;
-    GwyDataLine *calibration;
+    GwyLine *calibration;
 } ModuleArgs;
 
 typedef struct {
     ModuleArgs *args;
-    GwyDataField *orig_preview;
+    GwyField *orig_preview;
     GtkWidget *dialog;
+    GtkWidget *dataview;
     GwyParamTable *table;
-    GwyContainer *data;
     GwySelection *image_selection;
     GwySelection *graph_selection;
-    GwySIValueFormat *xvf;
-    GwySIValueFormat *yvf;
-    GwySIValueFormat *vf;
+    GwyValueFormat *xvf;
+    GwyValueFormat *yvf;
+    GwyValueFormat *vf;
     GtkAdjustment *adj[4];
     GtkWidget *real_coord[4];
     SelectionSource rectangle_source;
@@ -126,7 +116,7 @@ typedef struct {
 typedef struct {
     GwyBrick *brick;
     const gdouble *db;
-    GwyDataLine *dline;
+    GwyLine *dline;
     gdouble *buf;
     guint npts;
     guint npixels;
@@ -135,10 +125,10 @@ typedef struct {
 
 static gboolean                  module_register          (void);
 static GwyParamDef*              define_module_params     (void);
-static void                      plane_stat               (GwyContainer *data,
-                                                           GwyRunType run);
+static void                      module_main              (GwyFile *data,
+                                                           GwyRunModeFlags mode);
 static GwyDialogOutcome          run_gui                  (ModuleArgs *args,
-                                                           GwyContainer *data,
+                                                           GwyFile *data,
                                                            gint id);
 static void                      execute                  (ModuleArgs *args);
 static GtkWidget*                construct_selection      (ModuleGUI *gui);
@@ -164,30 +154,30 @@ static void                      update_real_size         (ModuleGUI *gui,
 static void                      update_graph_ordinate    (ModuleArgs *args);
 static void                      update_image_selection   (ModuleGUI *gui);
 static void                      update_graph_selection   (ModuleGUI *gui);
-static gdouble                   get_plane_range          (GwyDataField *field);
-static gdouble                   get_plane_Sa             (GwyDataField *field);
-static gdouble                   get_plane_median         (GwyDataField *field);
-static gdouble                   get_plane_skew           (GwyDataField *field);
-static gdouble                   get_plane_kurtosis       (GwyDataField *field);
-static gdouble                   get_plane_entropy_deficit(GwyDataField *field);
+static gdouble                   get_plane_range          (GwyField *field);
+static gdouble                   get_plane_Sa             (GwyField *field);
+static gdouble                   get_plane_median         (GwyField *field);
+static gdouble                   get_plane_skew           (GwyField *field);
+static gdouble                   get_plane_kurtosis       (GwyField *field);
+static gdouble                   get_plane_entropy_deficit(GwyField *field);
 static const PlaneStatQuantInfo* get_quantity_info        (PlaneStatQuantity quantity);
 static void                      sanitise_params          (ModuleArgs *args);
 
 static const PlaneStatQuantInfo quantities[] = {
     {
-        GWY_PLANE_STAT_MEAN, 0, 0, 0, 1, gwy_data_field_get_avg,
+        GWY_PLANE_STAT_MEAN, 0, 0, 0, 1, gwy_field_get_avg,
         N_("Mean"), "μ",
     },
     {
-        GWY_PLANE_STAT_RMS, 0, 0, 0, 1, gwy_data_field_get_rms,
+        GWY_PLANE_STAT_RMS, 0, 0, 0, 1, gwy_field_get_rms,
         N_("RMS"), "σ",
     },
     {
-        GWY_PLANE_STAT_MIN, 0, 0, 0, 1, gwy_data_field_get_min,
+        GWY_PLANE_STAT_MIN, 0, 0, 0, 1, gwy_field_get_min,
         N_("Minimum"), "v<sub>min</sub>",
     },
     {
-        GWY_PLANE_STAT_MAX, 0, 0, 0, 1, gwy_data_field_get_max,
+        GWY_PLANE_STAT_MAX, 0, 0, 0, 1, gwy_field_get_max,
         N_("Maximum"), "v<sub>min</sub>",
     },
     {
@@ -211,11 +201,11 @@ static const PlaneStatQuantInfo quantities[] = {
         N_("Median"), "m",
     },
     {
-        GWY_PLANE_STAT_VARIATION, GWY_PLANE_STAT_LATERAL_EQUAL, 1, 0, 1, gwy_data_field_get_variation,
+        GWY_PLANE_STAT_VARIATION, GWY_PLANE_STAT_LATERAL_EQUAL, 1, 0, 1, gwy_field_get_variation,
         N_("Variation"), "var",
     },
     {
-        GWY_PLANE_STAT_ENTROPY, 0, 0, 0, 0, gwy_data_field_get_entropy,
+        GWY_PLANE_STAT_ENTROPY, 0, 0, 0, 0, gwy_field_get_entropy,
         N_("Entropy"), "H",
     },
     {
@@ -229,7 +219,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Summarizes volume data planes to a graph."),
     "Yeti <yeti@gwyddion.net>",
-    "2.1",
+    "2.2",
     "David Nečas (Yeti)",
     "2018",
 };
@@ -240,9 +230,9 @@ static gboolean
 module_register(void)
 {
     gwy_volume_func_register("volume_planestat",
-                             (GwyVolumeFunc)&plane_stat,
+                             module_main,
                              N_("/_Statistics/Summarize P_lanes..."),
-                             GWY_STOCK_VOLUME_PLANE_STATS,
+                             GWY_ICON_VOLUME_PLANE_STATS,
                              RUN_MODES,
                              GWY_MENU_FLAG_VOLUME,
                              N_("Summarize planes"));
@@ -279,38 +269,37 @@ define_module_params(void)
 }
 
 static void
-plane_stat(GwyContainer *data, GwyRunType run)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
-    GwySIUnit *xunit, *yunit, *zunit, *wunit;
+    GwyUnit *xunit, *yunit, *zunit, *wunit;
     GwyAppDataId target_graph_id;
     GwyDialogOutcome outcome;
     GwyGraphCurveModel *gcmodel;
     gint id;
 
-    g_return_if_fail(run & RUN_MODES);
-    g_return_if_fail(g_type_from_name("GwyLayerRectangle"));
+    g_return_if_fail(mode & RUN_MODES);
 
-    gwy_app_data_browser_get_current(GWY_APP_BRICK, &args.brick,
-                                     GWY_APP_BRICK_ID, &id,
-                                     0);
+    gwy_data_browser_get_current(GWY_APP_BRICK, &args.brick,
+                                 GWY_APP_BRICK_ID, &id,
+                                 0);
     g_return_if_fail(GWY_IS_BRICK(args.brick));
 
     args.params = gwy_params_new_from_settings(define_module_params());
     args.calibration = gwy_brick_get_zcalibration(args.brick);
-    if (args.calibration && (gwy_brick_get_zres(args.brick) != gwy_data_line_get_res(args.calibration)))
+    if (args.calibration && (gwy_brick_get_zres(args.brick) != gwy_line_get_res(args.calibration)))
         args.calibration = NULL;
 
-    xunit = gwy_brick_get_si_unit_x(args.brick);
-    yunit = gwy_brick_get_si_unit_y(args.brick);
-    wunit = gwy_brick_get_si_unit_w(args.brick);
-    args.lateral_equal = gwy_si_unit_equal(xunit, yunit);
-    args.all_equal = args.lateral_equal && gwy_si_unit_equal(wunit, xunit);
+    xunit = gwy_brick_get_unit_x(args.brick);
+    yunit = gwy_brick_get_unit_y(args.brick);
+    wunit = gwy_brick_get_unit_w(args.brick);
+    args.lateral_equal = gwy_unit_equal(xunit, yunit);
+    args.all_equal = args.lateral_equal && gwy_unit_equal(wunit, xunit);
     sanitise_params(&args);
 
     args.gmodel = gwy_graph_model_new();
-    zunit = (args.calibration ? gwy_data_line_get_si_unit_y(args.calibration) : gwy_brick_get_si_unit_z(args.brick));
-    g_object_set(args.gmodel, "si-unit-x", zunit, "axis-label-bottom", "z", NULL);
+    zunit = (args.calibration ? gwy_line_get_unit_y(args.calibration) : gwy_brick_get_unit_z(args.brick));
+    g_object_set(args.gmodel, "unit-x", zunit, "axis-label-bottom", "z", NULL);
     update_graph_ordinate(&args);
 
     gcmodel = gwy_graph_curve_model_new();
@@ -334,43 +323,32 @@ end:
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     ModuleGUI gui;
-    GtkWidget *hbox, *dataview, *graph, *area, *align;
+    GtkWidget *hbox, *dataview, *graph, *area;
     GwyDialogOutcome outcome;
     GwyParamTable *table;
-    GwyDataField *field;
+    GwyField *field;
     GwyDialog *dialog;
-    const guchar *gradient;
     gint i;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
-    gui.xvf = gwy_brick_get_value_format_x(args->brick, GWY_SI_UNIT_FORMAT_VFMARKUP, NULL);
-    gui.yvf = gwy_brick_get_value_format_y(args->brick, GWY_SI_UNIT_FORMAT_VFMARKUP, NULL);
-    gui.data = gwy_container_new();
-    gui.orig_preview = gwy_container_get_object(data, gwy_app_get_brick_preview_key_for_id(id));
-    field = gwy_data_field_new_alike(gui.orig_preview, TRUE);
-    gwy_container_pass_object(gui.data, gwy_app_get_data_key_for_id(0), field);
+    gui.xvf = gwy_brick_get_value_format_x(args->brick, GWY_UNIT_FORMAT_VFMARKUP, NULL);
+    gui.yvf = gwy_brick_get_value_format_y(args->brick, GWY_UNIT_FORMAT_VFMARKUP, NULL);
+    gui.orig_preview = gwy_container_get_object(GWY_CONTAINER(data), gwy_file_key_volume_picture(id));
 
     gui.dialog = gwy_dialog_new(_("Summarize Volume Planes"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_UPDATE, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    hbox = gwy_hbox_new(0);
-    gwy_dialog_add_content(dialog, hbox, FALSE, FALSE, 4);
-
-    if (gwy_container_gis_string(data, gwy_app_get_brick_palette_key_for_id(id), &gradient))
-        gwy_container_set_const_string(gui.data, gwy_app_get_data_palette_key_for_id(0), gradient);
-
-    align = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-    gtk_box_pack_start(GTK_BOX(hbox), align, FALSE, FALSE, 0);
-
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
-    gui.image_selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(dataview), 0, "Rectangle", 1, TRUE);
-    gtk_container_add(GTK_CONTAINER(align), dataview);
+    field = gwy_field_new_alike(gui.orig_preview, TRUE);
+    dataview = gui.dataview = gwy_create_preview(field, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_VOLUME, id, GWY_FILE_ITEM_PALETTE);
+    gui.image_selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(dataview), GWY_TYPE_LAYER_RECTANGLE, 1, TRUE);
     update_image_selection(&gui);
+    hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), FALSE);
 
     g_object_set(args->gmodel, "label-visible", FALSE, NULL);  /* Only here. */
     graph = gwy_graph_new(args->gmodel);
@@ -384,7 +362,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_selection_set_max_objects(gui.graph_selection, 1);
     update_graph_selection(&gui);
 
-    hbox = gwy_hbox_new(24);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 24);
     gwy_dialog_add_content(dialog, hbox, FALSE, FALSE, 4);
 
     gtk_box_pack_start(GTK_BOX(hbox), construct_selection(&gui), FALSE, FALSE, 0);
@@ -402,6 +380,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     else
         gwy_param_table_alt_set_brick_pixel_z(table, PARAM_LEVEL, args->brick);
     gwy_param_table_add_enabler(table, PARAM_SHOW_PLANE, PARAM_LEVEL);
+    gwy_param_table_append_separator(table);
     gwy_param_table_append_target_graph(table, PARAM_TARGET_GRAPH, args->gmodel);
     gwy_param_table_append_checkbox(table, PARAM_UPDATE);
     gtk_box_pack_start(GTK_BOX(hbox), gwy_param_table_widget(table), FALSE, FALSE, 0);
@@ -417,40 +396,39 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 
     outcome = gwy_dialog_run(dialog);
 
-    g_object_unref(gui.data);
-    gwy_si_unit_value_format_free(gui.xvf);
-    gwy_si_unit_value_format_free(gui.yvf);
-    GWY_SI_VALUE_FORMAT_FREE(gui.vf);
+    gwy_value_format_free(gui.xvf);
+    gwy_value_format_free(gui.yvf);
+    GWY_FREE_VALUE_FORMAT(gui.vf);
 
     return outcome;
 }
 
 static GtkAdjustment*
-attach_coord_row(GtkTable *table, gint row, gint value, gint max, const gchar *name, gint id,
+attach_coord_row(GtkGrid *grid, gint row, gint value, gint max, const gchar *name, gint id,
                  GtkWidget **real_label)
 {
     GtkWidget *label, *spin;
     GtkAdjustment *adj;
 
     label = gtk_label_new_with_mnemonic(name);
-    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
-    gtk_table_attach(table, label, 0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
+    gtk_label_set_xalign(GTK_LABEL(label), 0.0);
+    gtk_grid_attach(grid, label, 0, row, 1, 1);
 
     adj = GTK_ADJUSTMENT(gtk_adjustment_new(value, 0, max, 1, 10, 0));
     g_object_set_data(G_OBJECT(adj), "id", GINT_TO_POINTER(id));
     spin = gtk_spin_button_new(adj, 0, 0);
     gtk_entry_set_width_chars(GTK_ENTRY(spin), 4);
     gtk_label_set_mnemonic_widget(GTK_LABEL(label), spin);
-    gtk_table_attach(table, spin, 2, 3, row, row+1, GTK_FILL, 0, 0, 0);
+    gtk_grid_attach(grid, spin, 2, row, 1, 1);
 
     label = *real_label = gtk_label_new(NULL);
     gtk_label_set_width_chars(GTK_LABEL(label), 12);
-    gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
-    gtk_table_attach(table, label, 1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
+    gtk_label_set_xalign(GTK_LABEL(label), 1.0);
+    gtk_grid_attach(grid, label, 1, row, 1, 1);
 
     label = gtk_label_new(_("px"));
-    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
-    gtk_table_attach(table, label, 3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
+    gtk_label_set_xalign(GTK_LABEL(label), 0.0);
+    gtk_grid_attach(grid, label, 3, row, 1, 1);
 
     return adj;
 }
@@ -462,25 +440,24 @@ construct_selection(ModuleGUI *gui)
     GwyParams *params = args->params;
     GwyBrick *brick = args->brick;
     gint xres = gwy_brick_get_xres(brick), yres = gwy_brick_get_yres(brick);
-    GtkTable *table;
 
-    table = GTK_TABLE(gtk_table_new(6, 4, FALSE));
-    gtk_container_set_border_width(GTK_CONTAINER(table), 4);
-    gtk_table_set_col_spacings(table, 8);
-    gtk_table_set_row_spacings(table, 2);
+    GtkGrid *grid = GTK_GRID(gtk_grid_new());
+    gtk_container_set_border_width(GTK_CONTAINER(grid), 4);
+    gtk_grid_set_column_spacing(grid, 8);
+    gtk_grid_set_row_spacing(grid, 2);
 
-    gtk_table_attach(table, gwy_label_new_header(_("Origin")), 0, 2, 0, 1, GTK_FILL, 0, 0, 0);
-    gui->adj[PARAM_COL] = attach_coord_row(table, 1, gwy_params_get_int(params, PARAM_COL), xres, _("_X"),
+    gtk_grid_attach(grid, gwy_label_new_header(_("Origin")), 0, 0, 2, 1);
+    gui->adj[PARAM_COL] = attach_coord_row(grid, 1, gwy_params_get_int(params, PARAM_COL), xres, _("_X"),
                                            PARAM_COL, &gui->real_coord[PARAM_COL]);
-    gui->adj[PARAM_ROW] = attach_coord_row(table, 2, gwy_params_get_int(params, PARAM_ROW), yres, _("_Y"),
+    gui->adj[PARAM_ROW] = attach_coord_row(grid, 2, gwy_params_get_int(params, PARAM_ROW), yres, _("_Y"),
                                            PARAM_ROW, &gui->real_coord[PARAM_ROW]);
-    gtk_table_attach(table, gwy_label_new_header(_("Size")), 0, 2, 3, 4, GTK_FILL, 0, 0, 0);
-    gui->adj[PARAM_WIDTH] = attach_coord_row(table, 4, gwy_params_get_int(params, PARAM_WIDTH), xres, _("_Width"),
+    gtk_grid_attach(grid, gwy_label_new_header(_("Size")), 0, 3, 2, 1);
+    gui->adj[PARAM_WIDTH] = attach_coord_row(grid, 4, gwy_params_get_int(params, PARAM_WIDTH), xres, _("_Width"),
                                              PARAM_WIDTH, &gui->real_coord[PARAM_WIDTH]);
-    gui->adj[PARAM_HEIGHT] = attach_coord_row(table, 5, gwy_params_get_int(params, PARAM_HEIGHT), yres, _("_Height"),
+    gui->adj[PARAM_HEIGHT] = attach_coord_row(grid, 5, gwy_params_get_int(params, PARAM_HEIGHT), yres, _("_Height"),
                                               PARAM_HEIGHT, &gui->real_coord[PARAM_HEIGHT]);
 
-    return GTK_WIDGET(table);
+    return GTK_WIDGET(grid);
 }
 
 static void
@@ -600,14 +577,14 @@ static void
 update_graph_ordinate(ModuleArgs *args)
 {
     const PlaneStatQuantInfo *info = get_quantity_info(gwy_params_get_enum(args->params, PARAM_QUANTITY));
-    GwySIUnit *xunit = gwy_brick_get_si_unit_x(args->brick);
-    GwySIUnit *yunit = gwy_brick_get_si_unit_y(args->brick);
-    GwySIUnit *wunit = gwy_brick_get_si_unit_w(args->brick);
-    GwySIUnit *unit = gwy_si_unit_new(NULL);
+    GwyUnit *xunit = gwy_brick_get_unit_x(args->brick);
+    GwyUnit *yunit = gwy_brick_get_unit_y(args->brick);
+    GwyUnit *wunit = gwy_brick_get_unit_w(args->brick);
+    GwyUnit *unit = gwy_unit_new(NULL);
 
-    gwy_si_unit_power_multiply(xunit, info->powerx, yunit, info->powery, unit);
-    gwy_si_unit_power_multiply(unit, 1, wunit, info->powerw, unit);
-    g_object_set(args->gmodel, "axis-label-left", info->symbol, "si-unit-y", unit, NULL);
+    gwy_unit_power_multiply(xunit, info->powerx, yunit, info->powery, unit);
+    gwy_unit_power_multiply(unit, 1, wunit, info->powerw, unit);
+    g_object_set(args->gmodel, "axis-label-left", info->symbol, "unit-y", unit, NULL);
     g_object_unref(unit);
 }
 
@@ -616,7 +593,7 @@ update_real_size(ModuleGUI *gui, gint id)
 {
     GwyBrick *brick = gui->args->brick;
     gint i = gwy_params_get_int(gui->args->params, id);
-    GwySIValueFormat *vf;
+    GwyValueFormat *vf;
     gdouble v;
     gchar *s;
 
@@ -657,14 +634,14 @@ static void
 show_extracted_plane(ModuleGUI *gui)
 {
     ModuleArgs *args = gui->args;
-    GwyDataField *field = gwy_container_get_object(gui->data, gwy_app_get_data_key_for_id(0));
+    GwyField *field = gwy_data_view_get_field(GWY_DATA_VIEW(gui->dataview));
 
     if (gwy_params_get_boolean(args->params, PARAM_SHOW_PLANE))
         gwy_brick_extract_xy_plane(args->brick, field, gwy_params_get_int(args->params, PARAM_LEVEL));
     else
-        gwy_data_field_copy(gui->orig_preview, field, TRUE);
+        gwy_field_assign(field, gui->orig_preview);
 
-    gwy_data_field_data_changed(field);
+    gwy_field_data_changed(field);
 }
 
 static void
@@ -673,9 +650,9 @@ preview(gpointer user_data)
     ModuleGUI *gui = (ModuleGUI*)user_data;
     ModuleArgs *args = gui->args;
     GwyGraphCurveModel *gcmodel;
-    GwySIValueFormat *vf;
+    GwyValueFormat *vf;
     gint n, zlevel;
-    GwySIUnit *unit;
+    GwyUnit *unit;
     gdouble v;
     gchar *s;
 
@@ -691,8 +668,8 @@ preview(gpointer user_data)
         return;
 
     v = gwy_graph_curve_model_get_ydata(gcmodel)[zlevel];
-    g_object_get(args->gmodel, "si-unit-y", &unit, NULL);
-    gui->vf = gwy_si_unit_get_format_with_digits(unit, GWY_SI_UNIT_FORMAT_VFMARKUP, v, 3, gui->vf);
+    g_object_get(args->gmodel, "unit-y", &unit, NULL);
+    gui->vf = gwy_unit_get_format_with_digits(unit, GWY_UNIT_FORMAT_VFMARKUP, v, 3, gui->vf);
     g_object_unref(unit);
     vf = gui->vf;
     s = g_strdup_printf("%.*f%s%s", vf->precision, v/vf->magnitude, *vf->units ? " " : "", vf->units);
@@ -756,7 +733,7 @@ execute(ModuleArgs *args)
     gint k, n;
 
     if (args->calibration)
-        xdata = g_memdup(gwy_data_line_get_data(args->calibration), zres*sizeof(gdouble));
+        xdata = g_memdup(gwy_line_get_data(args->calibration), zres*sizeof(gdouble));
     else
         xdata = gwy_math_linspace(NULL, zres, 0.5*zreal/zres + zoff, zreal/zres);
     ydata = g_new(gdouble, zres);
@@ -774,7 +751,7 @@ execute(ModuleArgs *args)
             shared(brick,ydata,zres,w,h,col,row,func)
 #endif
     {
-        GwyDataField *field = gwy_data_field_new(w, h, 1.0*w, 1.0*h, FALSE);
+        GwyField *field = gwy_field_new(w, h, 1.0*w, 1.0*h, FALSE);
         gint kk, kfrom = gwy_omp_chunk_start(zres), kto = gwy_omp_chunk_end(zres);
 
         for (kk = kfrom; kk < kto; kk++) {
@@ -816,59 +793,59 @@ get_quantity_info(PlaneStatQuantity quantity)
 }
 
 static gdouble
-get_plane_range(GwyDataField *field)
+get_plane_range(GwyField *field)
 {
     gdouble min, max;
 
-    gwy_data_field_get_min_max(field, &min, &max);
+    gwy_field_get_min_max(field, &min, &max);
     return max - min;
 }
 
 static gdouble
-get_plane_Sa(GwyDataField *field)
+get_plane_Sa(GwyField *field)
 {
     gdouble Sa;
 
-    gwy_data_field_get_stats(field, NULL, &Sa, NULL, NULL, NULL);
+    gwy_field_get_stats(field, NULL, &Sa, NULL, NULL, NULL);
 
     return Sa;
 }
 
 static gdouble
-get_plane_median(GwyDataField *field)
+get_plane_median(GwyField *field)
 {
-    guint xres = gwy_data_field_get_xres(field);
-    guint yres = gwy_data_field_get_yres(field);
+    guint xres = gwy_field_get_xres(field);
+    guint yres = gwy_field_get_yres(field);
 
     /* Reshuffle the data because the field is just a scratch buffer anyway. */
-    return gwy_math_median(xres*yres, gwy_data_field_get_data(field));
+    return gwy_math_median(gwy_field_get_data(field), xres*yres);
 }
 
 static gdouble
-get_plane_skew(GwyDataField *field)
+get_plane_skew(GwyField *field)
 {
     gdouble rms, skew;
 
-    gwy_data_field_get_stats(field, NULL, NULL, &rms, &skew, NULL);
+    gwy_field_get_stats(field, NULL, NULL, &rms, &skew, NULL);
 
     return rms > 0.0 ? skew : 0.0;
 }
 
 static gdouble
-get_plane_kurtosis(GwyDataField *field)
+get_plane_kurtosis(GwyField *field)
 {
     gdouble rms, kurtosis;
 
-    gwy_data_field_get_stats(field, NULL, NULL, &rms, NULL, &kurtosis);
+    gwy_field_get_stats(field, NULL, NULL, &rms, NULL, &kurtosis);
 
     return rms > 0.0 ? kurtosis : 0.0;
 }
 
 static gdouble
-get_plane_entropy_deficit(GwyDataField *field)
+get_plane_entropy_deficit(GwyField *field)
 {
-    gdouble H = gwy_data_field_get_entropy(field);
-    gdouble rms = gwy_data_field_get_rms(field);
+    gdouble H = gwy_field_get_entropy(field);
+    gdouble rms = gwy_field_get_rms(field);
 
     if (rms > 10.0*G_MINDOUBLE && H < 0.1*G_MAXDOUBLE)
         return ENTROPY_NORMAL + log(rms) - H;
