@@ -1,5 +1,5 @@
 /*
- *  $Id: volume_rephase.c 25693 2023-09-22 12:17:47Z yeti-dn $
+ *  $Id: volume_rephase.c 28808 2025-11-05 18:26:20Z yeti-dn $
  *  Copyright (C) 2019-2023 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,17 +16,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/brick.h>
-#include <libprocess/gwyprocess.h>
-#include <libgwydgets/gwynullstore.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-volume.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
+#include <gwy.h>
 
 #define RUN_MODES (GWY_RUN_INTERACTIVE)
 
@@ -52,13 +45,12 @@ typedef struct {
 typedef struct {
     GwyParams *params;
     GwyBrick *brick;
-    GwyDataLine *zcalibration;
+    GwyLine *zcalibration;
 } ModuleArgs;
 
 typedef struct {
     ModuleArgs *args;
-    GwyContainer *data;
-    GwyDataField *image;
+    GwyField *image;
     GwyGraphModel *gmodel;
     GwySelection *iselection;
     GwySelection *gselection;
@@ -70,10 +62,10 @@ typedef struct {
 
 static gboolean         module_register        (void);
 static GwyParamDef*     define_module_params   (void);
-static void             rephase                (GwyContainer *data,
-                                                GwyRunType run);
+static void             module_main            (GwyFile *data,
+                                                GwyRunModeFlags mode);
 static GwyDialogOutcome run_gui                (ModuleArgs *args,
-                                                GwyContainer *data,
+                                                GwyFile *data,
                                                 gint id);
 static void             param_changed          (ModuleGUI *gui,
                                                 gint id);
@@ -81,7 +73,7 @@ static void             dialog_response_after  (GtkDialog *dialog,
                                                 gint response,
                                                 ModuleGUI *gui);
 static void             execute                (ModuleArgs *args,
-                                                GwyContainer *data,
+                                                GwyFile *data,
                                                 gint id);
 static void             preview                (gpointer user_data);
 static void             point_selection_changed(ModuleGUI *gui,
@@ -94,7 +86,7 @@ static void             update_position        (ModuleGUI *gui,
                                                 const RephasePos *pos,
                                                 gboolean assume_changed);
 static void             extract_image_plane    (ModuleArgs *args,
-                                                GwyDataField *dfield);
+                                                GwyField *dfield);
 static void             extract_graph_curve    (ModuleArgs *args,
                                                 GwyGraphCurveModel *gcmodel);
 static void             extract_gmodel         (ModuleArgs *args,
@@ -119,7 +111,7 @@ static gboolean
 module_register(void)
 {
     gwy_volume_func_register("volume_rephase",
-                             (GwyVolumeFunc)&rephase,
+                             module_main,
                              N_("/SPM M_odes/_Adjust Phase..."),
                              NULL,
                              RUN_MODES,
@@ -149,18 +141,17 @@ define_module_params(void)
 }
 
 static void
-rephase(GwyContainer *data, GwyRunType run)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
     GwyDialogOutcome outcome;
     gint id;
 
-    g_return_if_fail(run & RUN_MODES);
-    g_return_if_fail(g_type_from_name("GwyLayerPoint"));
+    g_return_if_fail(mode & RUN_MODES);
 
-    gwy_app_data_browser_get_current(GWY_APP_BRICK, &args.brick,
-                                     GWY_APP_BRICK_ID, &id,
-                                     0);
+    gwy_data_browser_get_current(GWY_APP_BRICK, &args.brick,
+                                 GWY_APP_BRICK_ID, &id,
+                                 0);
     g_return_if_fail(GWY_IS_BRICK(args.brick));
     args.params = gwy_params_new_from_settings(define_module_params());
     args.zcalibration = gwy_brick_get_zcalibration(args.brick);
@@ -188,25 +179,20 @@ append_position(GwyParamTable *table, gint id, gint res)
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     GwyBrick *brick = args->brick;
-    GtkWidget *hbox, *graph, *align;
+    GtkWidget *hbox, *graph;
     GwyDialog *dialog;
     GwyParamTable *table;
     GwyGraphArea *area;
     ModuleGUI gui;
     GwyDialogOutcome outcome;
     GwyGraphCurveModel *gcmodel;
-    const guchar *gradient;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
-    gui.data = gwy_container_new();
-    gui.image = gwy_data_field_new(PREVIEW_SIZE, PREVIEW_SIZE, 1.0, 1.0, TRUE);
-    gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(0), gui.image);
-    if (gwy_container_gis_string(data, gwy_app_get_brick_palette_key_for_id(id), &gradient))
-        gwy_container_set_const_string(gui.data, gwy_app_get_data_palette_key_for_id(0), gradient);
+    gui.image = gwy_field_new(PREVIEW_SIZE, PREVIEW_SIZE, 1.0, 1.0, TRUE);
 
     gui.gmodel = gwy_graph_model_new();
     g_object_set(gui.gmodel, "label-visible", FALSE, NULL);
@@ -218,15 +204,10 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    hbox = gwy_hbox_new(0);
-    gwy_dialog_add_content(dialog, hbox, FALSE, FALSE, 4);
-
-    align = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-    gtk_box_pack_start(GTK_BOX(hbox), align, FALSE, FALSE, 0);
-
-    gui.dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
-    gui.iselection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(gui.dataview), 0, "Point", 1, TRUE);
-    gtk_container_add(GTK_CONTAINER(align), gui.dataview);
+    gui.dataview = gwy_create_preview(gui.image, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(gui.dataview), data, GWY_FILE_VOLUME, id, GWY_FILE_ITEM_PALETTE);
+    gui.iselection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(gui.dataview), GWY_TYPE_LAYER_POINT, 1, TRUE);
+    hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(gui.dataview), FALSE);
 
     graph = gwy_graph_new(gui.gmodel);
     gwy_graph_enable_user_input(GWY_GRAPH(graph), FALSE);
@@ -237,7 +218,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_graph_area_set_status(area, GWY_GRAPH_STATUS_XLINES);
     gui.gselection = gwy_graph_area_get_selection(area, GWY_GRAPH_STATUS_XLINES);
 
-    hbox = gwy_hbox_new(24);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 24);
     gwy_dialog_add_content(dialog, hbox, TRUE, TRUE, 4);
 
     table = gui.table = gwy_param_table_new(args->params);
@@ -270,7 +251,6 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 
     g_object_unref(gui.gmodel);
     g_object_unref(gui.image);
-    g_object_unref(gui.data);
 
     return outcome;
 }
@@ -293,8 +273,8 @@ point_selection_changed(ModuleGUI *gui,
                         gint id,
                         GwySelection *selection)
 {
-    GwyDataField *field = gui->image;
-    gint xres = gwy_data_field_get_xres(field), yres = gwy_data_field_get_yres(field);
+    GwyField *field = gui->image;
+    gint xres = gwy_field_get_xres(field), yres = gwy_field_get_yres(field);
     gint i, j;
     RephasePos pos;
     gdouble xy[2];
@@ -306,8 +286,8 @@ point_selection_changed(ModuleGUI *gui,
         return;
 
     fill_pos_from_params(gui->args->params, &pos);
-    j = CLAMP(gwy_data_field_rtoj(field, xy[0]), 0, xres-1);
-    i = CLAMP(gwy_data_field_rtoi(field, xy[1]), 0, yres-1);
+    j = CLAMP(gwy_field_rtoj(field, xy[0]), 0, xres-1);
+    i = CLAMP(gwy_field_rtoi(field, xy[1]), 0, yres-1);
 
     pos.x = j;
     pos.y = i;
@@ -389,7 +369,7 @@ preview(gpointer user_data)
     extract_graph_curve(args, gcmodel);
 
     extract_image_plane(args, gui->image);
-    gwy_data_field_data_changed(gui->image);
+    gwy_field_data_changed(gui->image);
     gwy_set_data_preview_size(GWY_DATA_VIEW(gui->dataview), PREVIEW_SIZE);
 }
 
@@ -400,7 +380,7 @@ get_shifted_values(GwyBrick *b1, GwyBrick *b2,
                    gint shift, gint right, gint invert,
                    gdouble *val1, gdouble *val2)
 {
-    const gdouble *b1data = b1->data, *b2data = b2->data;
+    const gdouble *b1data = gwy_brick_get_data_const(b1), *b2data = gwy_brick_get_data_const(b2);
     gint xres = b1->xres, yres = b1->yres, zres = b1->zres;
     gint pos;
 
@@ -436,7 +416,7 @@ get_shifted_values(GwyBrick *b1, GwyBrick *b2,
 }
 
 static void
-execute(ModuleArgs *args, GwyContainer *data, gint id)
+execute(ModuleArgs *args, GwyFile *data, gint id)
 {
     GwyParams *params = args->params;
     GwyBrick *second_brick = gwy_params_get_volume(params, PARAM_OTHER_VOLUME);
@@ -475,28 +455,32 @@ execute(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_brick_data_changed(result1);
     gwy_brick_data_changed(result2);
 
-    newid = gwy_app_data_browser_add_brick(result1, NULL, data, TRUE);
+    newid = gwy_file_add_volume(data, result1, NULL);
 
-    gwy_app_set_brick_title(data, newid, _("Phase adjusted result A"));
-    gwy_app_sync_volume_items(data, data, id, newid, FALSE,
-                              GWY_DATA_ITEM_GRADIENT,
-                              0);
-    gwy_app_volume_log_add_volume(data, -1, newid);
+    gwy_file_set_visible(data, GWY_FILE_VOLUME, newid, TRUE);
 
-    newid = gwy_app_data_browser_add_brick(result2, NULL, data, TRUE);
+    gwy_file_set_title(data, GWY_FILE_VOLUME, newid, _("Phase adjusted result A"), TRUE);
+    gwy_file_sync_items(data, GWY_FILE_VOLUME, id,
+                        data, GWY_FILE_VOLUME, newid,
+                        GWY_FILE_ITEM_PALETTE, FALSE);
+    gwy_log_add(data, GWY_FILE_VOLUME, -1, newid);
 
-    gwy_app_set_brick_title(data, newid, _("Phase adjusted result B"));
-    gwy_app_sync_volume_items(data, data, id, newid, FALSE,
-                              GWY_DATA_ITEM_GRADIENT,
-                              0);
-    gwy_app_volume_log_add_volume(data, -1, newid);
+    newid = gwy_file_add_volume(data, result2, NULL);
+
+    gwy_file_set_visible(data, GWY_FILE_VOLUME, newid, TRUE);
+
+    gwy_file_set_title(data, GWY_FILE_VOLUME, newid, _("Phase adjusted result B"), TRUE);
+    gwy_file_sync_items(data, GWY_FILE_VOLUME, id,
+                        data, GWY_FILE_VOLUME, newid,
+                        GWY_FILE_ITEM_PALETTE, FALSE);
+    gwy_log_add(data, GWY_FILE_VOLUME, -1, newid);
 
     g_object_unref(result1);
     g_object_unref(result2);
 }
 
 static void
-extract_image_plane(ModuleArgs *args, GwyDataField *dfield)
+extract_image_plane(ModuleArgs *args, GwyField *dfield)
 {
     gint zres = gwy_brick_get_zres(args->brick);
     gint z = gwy_params_get_int(args->params, PARAM_ZPOS);
@@ -516,7 +500,7 @@ extract_image_plane(ModuleArgs *args, GwyDataField *dfield)
        }
     }
 
-    gwy_data_field_data_changed(dfield);
+    gwy_field_data_changed(dfield);
 }
 
 static void
@@ -527,9 +511,9 @@ extract_graph_curve(ModuleArgs *args, GwyGraphCurveModel *gcmodel)
     gboolean right = gwy_params_get_boolean(args->params, PARAM_RIGHT);
     gboolean invert = gwy_params_get_boolean(args->params, PARAM_INVERT);
     GwyBrick *brick = args->brick;
-    GwyDataLine *line = gwy_data_line_new(1, 1.0, FALSE);
-    GwyDataLine *second_line = gwy_data_line_new(1, 1.0, FALSE);
-    GwyDataLine *merged_line;
+    GwyLine *line = gwy_line_new(1, 1.0, FALSE);
+    GwyLine *second_line = gwy_line_new(1, 1.0, FALSE);
+    GwyLine *merged_line;
     gint zres = gwy_brick_get_zres(args->brick);
     gdouble zreal = gwy_brick_get_zreal(args->brick);
     RephasePos pos;
@@ -542,10 +526,10 @@ extract_graph_curve(ModuleArgs *args, GwyGraphCurveModel *gcmodel)
     gwy_brick_extract_line(brick, line, pos.x, pos.y, 0, pos.x, pos.y, zres, FALSE);
     gwy_brick_extract_line(second_brick, second_line, pos.x, pos.y, 0, pos.x, pos.y, zres, FALSE);
 
-    merged_line = gwy_data_line_new(2*zres, 2*zreal, FALSE);
-    data = gwy_data_line_get_data(merged_line);
-    data1 = gwy_data_line_get_data(line);
-    data2 = gwy_data_line_get_data(second_line);
+    merged_line = gwy_line_new(2*zres, 2*zreal, FALSE);
+    data = gwy_line_get_data(merged_line);
+    data1 = gwy_line_get_data(line);
+    data2 = gwy_line_get_data(second_line);
 
     for (i = 0; i < zres; i++) {
         if (right) {
@@ -565,7 +549,7 @@ extract_graph_curve(ModuleArgs *args, GwyGraphCurveModel *gcmodel)
                  NULL);
     g_free(desc);
 
-    gwy_graph_curve_model_set_data_from_dataline(gcmodel, merged_line, 0, 0);
+    gwy_graph_curve_model_set_data_from_line(gcmodel, merged_line, 0, 0);
 
     g_object_unref(line);
     g_object_unref(second_line);
@@ -576,14 +560,14 @@ static void
 extract_gmodel(ModuleArgs *args, GwyGraphModel *gmodel)
 {
     GwyBrick *brick = args->brick;
-    GwySIUnit *xunit, *yunit;
+    GwyUnit *xunit, *yunit;
 
-    xunit = (args->zcalibration ? gwy_data_line_get_si_unit_y(args->zcalibration) : gwy_brick_get_si_unit_z(brick));
-    yunit = gwy_brick_get_si_unit_w(brick);
+    xunit = (args->zcalibration ? gwy_line_get_unit_y(args->zcalibration) : gwy_brick_get_unit_z(brick));
+    yunit = gwy_brick_get_unit_w(brick);
     g_object_set(gmodel,
                  "title", _("Volume Z graphs"),
-                 "si-unit-x", xunit,
-                 "si-unit-y", yunit,
+                 "unit-x", xunit,
+                 "unit-y", yunit,
                  "axis-label-bottom", "z",
                  "axis-label-left", "w",
                  NULL);
