@@ -1,5 +1,5 @@
 /*
- *  $Id: volume_scars.c 26394 2024-06-19 15:19:21Z yeti-dn $
+ *  $Id: volume_scars.c 28905 2025-11-24 15:51:18Z yeti-dn $
  *  Copyright (C) 2015-2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,26 +16,11 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwythreads.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwynlfit.h>
-#include <libprocess/brick.h>
-#include <libprocess/stats.h>
-#include <libprocess/linestats.h>
-#include <libprocess/gwyprocess.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libprocess/correlation.h>
-#include <libgwydgets/gwydataview.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwydgets/gwydgetutils.h>
-#include <libgwymodule/gwymodule-volume.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
-#include "libgwyddion/gwyomp.h"
+#include <gwy.h>
+#include "libgwyddion/omp.h"
 
 #define RUN_MODES (GWY_RUN_INTERACTIVE)
 
@@ -44,12 +29,6 @@ enum {
     MAX_LENGTH = 1024
 };
 
-typedef enum {
-    FEATURES_POSITIVE = 1 << 0,
-    FEATURES_NEGATIVE = 1 << 2,
-    FEATURES_BOTH     = (FEATURES_POSITIVE | FEATURES_NEGATIVE),
-} FeatureType;
-
 enum {
     PARAM_TYPE,
     PARAM_THRESHOLD_HIGH,
@@ -70,29 +49,28 @@ typedef struct {
 typedef struct {
     ModuleArgs *args;
     GtkWidget *dialog;
+    GtkWidget *dataview;
     GwyParamTable *table_options;
-    GwyContainer *data;
 } ModuleGUI;
 
-
-static gboolean              module_register          (void);
-static GwyParamDef*          define_module_params     (void);
-static void                  scars                    (GwyContainer *data,
-                                                       GwyRunType runtype);
-static gboolean              execute                  (ModuleArgs *args,
-                                                       GtkWindow *wait_window);
-static GwyDialogOutcome      run_gui                  (ModuleArgs *args,
-                                                       GwyContainer *data,
-                                                       gint id);
-static void                  param_changed            (ModuleGUI *gui,
-                                                       gint id);
-static void                  dialog_response          (GwyDialog *dialog,
-                                                       gint response,
-                                                       ModuleGUI *gui);
-static void                  preview                  (gpointer user_data);
-static void                  update_image             (ModuleGUI *gui,
-                                                       gint z);
-static void                  sanitise_params          (ModuleArgs *args);
+static gboolean         module_register     (void);
+static GwyParamDef*     define_module_params(void);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
+static gboolean         execute             (ModuleArgs *args,
+                                             GtkWindow *wait_window);
+static GwyDialogOutcome run_gui             (ModuleArgs *args,
+                                             GwyFile *data,
+                                             gint id);
+static void             param_changed       (ModuleGUI *gui,
+                                             gint id);
+static void             dialog_response     (GwyDialog *dialog,
+                                             gint response,
+                                             ModuleGUI *gui);
+static void             preview             (gpointer user_data);
+static void             update_image        (ModuleGUI *gui,
+                                             gint z);
+static void             sanitise_params     (ModuleArgs *args);
 
 
 static GwyModuleInfo module_info = {
@@ -111,7 +89,7 @@ static gboolean
 module_register(void)
 {
     gwy_volume_func_register("volume_scars",
-                             (GwyVolumeFunc)&scars,
+                             module_main,
                              N_("/_Correct Data/_Scars..."),
                              NULL,
                              RUN_MODES,
@@ -125,9 +103,9 @@ static GwyParamDef*
 define_module_params(void)
 {
     static const GwyEnum feature_types[] = {
-        { N_("Positive"), FEATURES_POSITIVE, },
-        { N_("Negative"), FEATURES_NEGATIVE, },
-        { N_("Both"),     FEATURES_BOTH,     },
+        { N_("Positive"), GWY_SIGN_POSITIVE, },
+        { N_("Negative"), GWY_SIGN_NEGATIVE, },
+        { N_("Both"),     GWY_SIGN_BOTH,     },
     };
     static GwyParamDef *paramdef = NULL;
 
@@ -138,7 +116,7 @@ define_module_params(void)
     gwy_param_def_set_function_name(paramdef, "scars");
     gwy_param_def_add_int(paramdef, PARAM_Z, "z", "Preview level", 0, G_MAXINT, 0);
     gwy_param_def_add_gwyenum(paramdef, PARAM_TYPE, "type", _("Scars type"),
-                              feature_types, G_N_ELEMENTS(feature_types), FEATURES_BOTH);
+                              feature_types, G_N_ELEMENTS(feature_types), GWY_SIGN_BOTH);
     gwy_param_def_add_double(paramdef, PARAM_THRESHOLD_HIGH, "threshold_high", _("_Hard threshold"), 0.0, 2.0, 0.666);
     gwy_param_def_add_double(paramdef, PARAM_THRESHOLD_LOW, "threshold_low", _("_Soft threshold"), 0.0, 2.0, 0.25);
     gwy_param_def_add_int(paramdef, PARAM_MIN_LENGTH, "min_len", _("Minimum _length"), 1, MAX_LENGTH, 16);
@@ -149,41 +127,38 @@ define_module_params(void)
 }
 
 static void
-scars(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
     GwyBrick *brick = NULL;
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     gint oldid, newid;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    g_return_if_fail(g_type_from_name("GwyLayerPoint"));
+    g_return_if_fail(mode & RUN_MODES);
 
-    gwy_app_data_browser_get_current(GWY_APP_BRICK, &brick,
-                                     GWY_APP_BRICK_ID, &oldid,
-                                     0);
+    gwy_data_browser_get_current(GWY_APP_BRICK, &brick,
+                                 GWY_APP_BRICK_ID, &oldid,
+                                 0);
     g_return_if_fail(GWY_IS_BRICK(brick));
     args.result = NULL;
     args.brick = brick;
     args.params = gwy_params_new_from_settings(define_module_params());
     sanitise_params(&args);
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, oldid);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
             goto end;
     }
-    if (execute(&args, gwy_app_find_window_for_volume(data, oldid))) {
-
-        newid = gwy_app_data_browser_add_brick(args.result, NULL, data, TRUE);
-
-        gwy_app_set_brick_title(data, newid, _("Scars corrected"));
-        gwy_app_sync_volume_items(data, data, oldid, newid, FALSE,
-                                  GWY_DATA_ITEM_GRADIENT,
-                                  0);
-
-        gwy_app_volume_log_add_volume(data, -1, newid);
+    if (execute(&args, gwy_data_browser_get_window_for_data(data, GWY_FILE_VOLUME, oldid))) {
+        newid = gwy_file_add_volume(data, args.result, NULL);
+        gwy_file_set_visible(data, GWY_FILE_VOLUME, newid, TRUE);
+        gwy_file_set_title(data, GWY_FILE_VOLUME, newid, _("Scars corrected"), TRUE);
+        gwy_file_sync_items(data, GWY_FILE_VOLUME, oldid,
+                            data, GWY_FILE_VOLUME, newid,
+                            GWY_FILE_ITEM_PALETTE, FALSE);
+        gwy_log_add(data, GWY_FILE_VOLUME, -1, newid);
     }
 
 end:
@@ -192,7 +167,7 @@ end:
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     GtkWidget *hbox, *dataview;
     GwyParamTable *table;
@@ -200,30 +175,20 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     ModuleGUI gui;
     GwyDialogOutcome outcome;
     GwyBrick *brick = args->brick;
-    GwyDataField *field = gwy_data_field_new(gwy_brick_get_xres(brick),
-                                             gwy_brick_get_yres(brick),
-                                             gwy_brick_get_xreal(brick),
-                                             gwy_brick_get_yreal(brick),
-                                             TRUE);
-    GwyDataField *mask = gwy_data_field_new_alike(field, TRUE);
-    const guchar *gradient;
+    GwyField *field = gwy_brick_new_field_like_xy_plane(brick, TRUE);
+    GwyField *mask = gwy_field_new_alike(field, TRUE);
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
-    gui.data = gwy_container_new();
 
-    args->result = gwy_brick_duplicate(brick);
-
-    gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(0), field);
-    if (gwy_container_gis_string(data, gwy_app_get_brick_palette_key_for_id(id), &gradient))
-        gwy_container_set_const_string(gui.data, gwy_app_get_data_palette_key_for_id(0), gradient);
-    gwy_container_set_object(gui.data, gwy_app_get_mask_key_for_id(0), mask);
+    args->result = gwy_brick_copy(brick);
 
     gui.dialog = gwy_dialog_new(_("Scars"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_UPDATE, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
+    dataview = gui.dataview = gwy_create_preview(field, mask, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_VOLUME, id, GWY_FILE_ITEM_PALETTE);
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), FALSE);
 
     table = gui.table_options = gwy_param_table_new(args->params);
@@ -244,7 +209,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_param_table_append_radio(table, PARAM_TYPE);
 
     gwy_param_table_append_header(table, -1, _("Options"));
-    gwy_param_table_append_mask_color(table, PARAM_MASK_COLOR, gui.data, 0, data, id);
+    gwy_param_table_append_image_mask_color(table, PARAM_MASK_COLOR, GWY_DATA_VIEW(dataview), data, id);
     gwy_param_table_append_checkbox(table, PARAM_UPDATE);
 
     gwy_dialog_add_param_table(dialog, table);
@@ -256,12 +221,12 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 
     outcome = gwy_dialog_run(dialog);
 
-    g_object_unref(gui.data);
+    g_object_unref(mask);
+    g_object_unref(field);
 
     return outcome;
 }
 
-
 static void
 param_changed(ModuleGUI *gui, gint id)
 {
@@ -286,52 +251,50 @@ static void
 dialog_response(G_GNUC_UNUSED GwyDialog *dialog, gint response, ModuleGUI *gui)
 {
     if (response == GWY_RESPONSE_RESET)
-        gwy_brick_copy(gui->args->brick, gui->args->result, FALSE);
+        gwy_brick_copy_data(gui->args->brick, gui->args->result);
 
     preview(gui);
 }
 
 static void
-mark_scars(GwyDataField *dfield, GwyDataField *mask, gdouble threshold_high, gdouble threshold_low,
-           gint min_len, gint max_width, FeatureType type)
+mark_scars(GwyField *dfield, GwyField *mask, gdouble threshold_high, gdouble threshold_low,
+           gint min_len, gint max_width, GwySignFlags type)
 {
-    GwyDataField *tmp;
-
-    if (type == FEATURES_POSITIVE || type == FEATURES_NEGATIVE) {
-        gwy_data_field_mark_scars(dfield, mask, threshold_high, threshold_low,
-                                  min_len, max_width, type == FEATURES_NEGATIVE);
-    } else {
-        gwy_data_field_mark_scars(dfield, mask, threshold_high, threshold_low, min_len, max_width, FALSE);
-        tmp = gwy_data_field_new_alike(dfield, FALSE);
-        gwy_data_field_mark_scars(dfield, tmp, threshold_high, threshold_low, min_len, max_width, TRUE);
-        gwy_data_field_max_of_fields(mask, mask, tmp);
-        g_object_unref(tmp);
+    if (type == GWY_SIGN_POSITIVE || type == GWY_SIGN_NEGATIVE) {
+        gwy_field_mark_scars(dfield, mask, threshold_high, threshold_low,
+                             min_len, max_width, type == GWY_SIGN_NEGATIVE);
+        return;
     }
+
+    gwy_field_mark_scars(dfield, mask, threshold_high, threshold_low, min_len, max_width, FALSE);
+    GwyField *tmp = gwy_field_new_alike(dfield, FALSE);
+    gwy_field_mark_scars(dfield, tmp, threshold_high, threshold_low, min_len, max_width, TRUE);
+    gwy_field_max_of_fields(mask, mask, tmp);
+    g_object_unref(tmp);
 }
 
 static void
 update_image(ModuleGUI *gui, gint z)
 {
-    GwyDataField *dfield, *mask;
     GwyBrick *brick = gui->args->result;
     GwyParams *params = gui->args->params;
-    FeatureType type = gwy_params_get_enum(params, PARAM_TYPE);
+    GwySignFlags type = gwy_params_get_enum(params, PARAM_TYPE);
     gdouble threshold_high = gwy_params_get_double(params, PARAM_THRESHOLD_HIGH);
     gdouble threshold_low = gwy_params_get_double(params, PARAM_THRESHOLD_LOW);
     gint min_len = gwy_params_get_int(params, PARAM_MIN_LENGTH);
     gint max_width = gwy_params_get_int(params, PARAM_MAX_WIDTH);
+    gint zres = gwy_brick_get_zres(brick);
 
-    dfield = gwy_container_get_object(gui->data, gwy_app_get_data_key_for_id(0));
-    mask = gwy_container_get_object(gui->data, gwy_app_get_mask_key_for_id(0));
+    GwyField *dfield = gwy_data_view_get_field(GWY_DATA_VIEW(gui->dataview));
+    GwyField *mask = gwy_data_view_get_mask(GWY_DATA_VIEW(gui->dataview));
 
-    gwy_brick_extract_xy_plane(brick, dfield, CLAMP(z, 0, brick->zres-1));
-    gwy_data_field_data_changed(dfield);
+    gwy_brick_extract_xy_plane(brick, dfield, CLAMP(z, 0, zres-1));
+    gwy_field_data_changed(dfield);
 
     mark_scars(dfield, mask, threshold_high, threshold_low, min_len, max_width, type);
-    gwy_data_field_data_changed(mask);
+    gwy_field_data_changed(mask);
 }
 
-
 static void
 preview(gpointer user_data)
 {
@@ -348,8 +311,8 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
     gint k;
     GwyBrick *original = args->brick;
     GwyBrick *brick = args->result;
-    GwyDataField *dfield, *mask;
-    FeatureType type = gwy_params_get_enum(params, PARAM_TYPE);
+    GwyField *dfield, *mask;
+    GwySignFlags type = gwy_params_get_enum(params, PARAM_TYPE);
     gdouble threshold_high = gwy_params_get_double(params, PARAM_THRESHOLD_HIGH);
     gdouble threshold_low = gwy_params_get_double(params, PARAM_THRESHOLD_LOW);
     gint min_len = gwy_params_get_int(params, PARAM_MIN_LENGTH);
@@ -360,15 +323,15 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
     gint yres = gwy_brick_get_yres(brick);
     gint zres = gwy_brick_get_zres(brick);
 
-    dfield = gwy_data_field_new(xres, yres, gwy_brick_get_xreal(brick), gwy_brick_get_yreal(brick), FALSE);
-    mask = gwy_data_field_new_alike(dfield, FALSE);
+    dfield = gwy_field_new(xres, yres, gwy_brick_get_xreal(brick), gwy_brick_get_yreal(brick), FALSE);
+    mask = gwy_field_new_alike(dfield, FALSE);
 
     gwy_app_wait_start(wait_window, _("Removing scars..."));
 
     for (k = 0; k < zres; k++) {
         gwy_brick_extract_xy_plane(original, dfield, k);
         mark_scars(dfield, mask, threshold_high, threshold_low, min_len, max_width, type);
-        gwy_data_field_laplace_solve(dfield, mask, -1, 1.0);
+        gwy_field_laplace_solve(dfield, mask, -1, 1.0);
         gwy_brick_set_xy_plane(brick, dfield, k);
 
         if (!gwy_app_wait_set_fraction((gdouble)k/zres)) {
@@ -395,5 +358,4 @@ sanitise_params(ModuleArgs *args)
         gwy_params_set_double(params, PARAM_THRESHOLD_HIGH, threshold_low);
 }
 
-
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
