@@ -1,5 +1,5 @@
 /*
- *  $Id: volume_slice.c 26413 2024-06-26 15:38:30Z yeti-dn $
+ *  $Id: volume_slice.c 28848 2025-11-10 18:54:08Z yeti-dn $
  *  Copyright (C) 2015-2023 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,17 +16,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/brick.h>
-#include <libprocess/gwyprocess.h>
-#include <libgwydgets/gwynullstore.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-volume.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
+#include <gwy.h>
 
 #define RUN_MODES (GWY_RUN_NONINTERACTIVE | GWY_RUN_INTERACTIVE)
 
@@ -77,13 +70,12 @@ typedef struct {
     GwyParams *params;
     GwyBrick *brick;
     GArray *allpos;
-    GwyDataLine *zcalibration;
+    GwyLine *zcalibration;
 } ModuleArgs;
 
 typedef struct {
     ModuleArgs *args;
-    GwyContainer *data;
-    GwyDataField *image;
+    GwyField *image;
     GwyGraphModel *gmodel;
     GwySelection *iselection;
     GwySelection *gselection;
@@ -100,19 +92,19 @@ typedef struct {
 
 static gboolean         module_register            (void);
 static GwyParamDef*     define_module_params       (void);
-static void             slice                      (GwyContainer *data,
-                                                    GwyRunType run);
+static void             module_main                (GwyFile *data,
+                                                    GwyRunModeFlags mode);
 static GwyDialogOutcome run_gui                    (ModuleArgs *args,
-                                                    GwyContainer *data,
+                                                    GwyFile *data,
                                                     gint id);
 static void             param_changed              (ModuleGUI *gui,
                                                     gint id);
 static void             execute                    (ModuleArgs *args,
-                                                    GwyContainer *data,
+                                                    GwyFile *data,
                                                     gint id);
 static void             preview                    (gpointer user_data);
 static void             extract_one_image          (ModuleArgs *args,
-                                                    GwyContainer *data,
+                                                    GwyFile *data,
                                                     gint id,
                                                     gint idx);
 static GtkWidget*       create_coordlist           (ModuleGUI *gui);
@@ -136,7 +128,7 @@ static void             update_position            (ModuleGUI *gui,
                                                     gboolean assume_changed);
 static void             update_multiselection      (ModuleGUI *gui);
 static void             extract_image_plane        (ModuleArgs *args,
-                                                    GwyDataField *dfield,
+                                                    GwyField *dfield,
                                                     guint idx);
 static void             extract_graph_curve        (ModuleArgs *args,
                                                     GwyGraphCurveModel *gcmodel,
@@ -164,9 +156,9 @@ static gboolean
 module_register(void)
 {
     gwy_volume_func_register("volume_slice",
-                             (GwyVolumeFunc)&slice,
+                             module_main,
                              N_("/_Basic Operations/Cut and _Slice..."),
-                             GWY_STOCK_VOLUME_SLICE,
+                             GWY_ICON_VOLUME_SLICE,
                              RUN_MODES,
                              GWY_MENU_FLAG_VOLUME,
                              N_("Extract image planes and line graphs"));
@@ -209,29 +201,28 @@ define_module_params(void)
 }
 
 static void
-slice(GwyContainer *data, GwyRunType run)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
     GwyDialogOutcome outcome;
     gint id;
 
-    g_return_if_fail(run & RUN_MODES);
-    g_return_if_fail(g_type_from_name("GwyLayerPoint"));
+    g_return_if_fail(mode & RUN_MODES);
 
-    gwy_app_data_browser_get_current(GWY_APP_BRICK, &args.brick,
-                                     GWY_APP_BRICK_ID, &id,
-                                     0);
+    gwy_data_browser_get_current(GWY_APP_BRICK, &args.brick,
+                                 GWY_APP_BRICK_ID, &id,
+                                 0);
     g_return_if_fail(GWY_IS_BRICK(args.brick));
 
     args.zcalibration = gwy_brick_get_zcalibration(args.brick);
-    if (args.zcalibration && gwy_brick_get_zres(args.brick) != gwy_data_line_get_res(args.zcalibration))
+    if (args.zcalibration && gwy_brick_get_zres(args.brick) != gwy_line_get_res(args.zcalibration))
         args.zcalibration = NULL;
     args.allpos = g_array_new(FALSE, FALSE, sizeof(SlicePos));
     args.params = gwy_params_new_from_settings(define_module_params());
     sanitise_params(&args);
     append_current_pos(&args);
 
-    if (run == GWY_RUN_INTERACTIVE) {
+    if (mode == GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, id);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -255,26 +246,21 @@ append_position(GwyParamTable *table, gint id, gint res)
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     GwyBrick *brick = args->brick;
-    GtkWidget *hbox, *graph, *align;
+    GtkWidget *hbox, *graph;
     GwyDialog *dialog;
     GwyParamTable *table;
     GwyGraphArea *area;
     ModuleGUI gui;
     GwyDialogOutcome outcome;
     GwyGraphCurveModel *gcmodel;
-    const guchar *gradient;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
     gui.store = gwy_null_store_new(1);
-    gui.data = gwy_container_new();
-    gui.image = gwy_data_field_new(PREVIEW_SIZE, PREVIEW_SIZE, 1.0, 1.0, TRUE);
-    gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(0), gui.image);
-    if (gwy_container_gis_string(data, gwy_app_get_brick_palette_key_for_id(id), &gradient))
-        gwy_container_set_const_string(gui.data, gwy_app_get_data_palette_key_for_id(0), gradient);
+    gui.image = gwy_field_new(PREVIEW_SIZE, PREVIEW_SIZE, 1.0, 1.0, TRUE);
 
     gui.gmodel = gwy_graph_model_new();
     g_object_set(gui.gmodel, "label-visible", FALSE, NULL);
@@ -287,15 +273,15 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_CLEAR, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    hbox = gwy_hbox_new(0);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
     gwy_dialog_add_content(dialog, hbox, FALSE, FALSE, 4);
 
-    align = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-    gtk_box_pack_start(GTK_BOX(hbox), align, FALSE, FALSE, 0);
-
-    gui.dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
-    gui.iselection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(gui.dataview), 0, "Point", 1, TRUE);
-    gtk_container_add(GTK_CONTAINER(align), gui.dataview);
+    gui.dataview = gwy_create_preview(gui.image, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(gui.dataview), data, GWY_FILE_VOLUME, id, GWY_FILE_ITEM_PALETTE);
+    gtk_widget_set_halign(gui.dataview, GTK_ALIGN_START);
+    gtk_widget_set_valign(gui.dataview, GTK_ALIGN_START);
+    gui.iselection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(gui.dataview), GWY_TYPE_LAYER_POINT, 1, TRUE);
+    gtk_box_pack_start(GTK_BOX(hbox), gui.dataview, FALSE, FALSE, 0);
 
     graph = gwy_graph_new(gui.gmodel);
     gwy_graph_enable_user_input(GWY_GRAPH(graph), FALSE);
@@ -306,7 +292,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_graph_area_set_status(area, GWY_GRAPH_STATUS_XLINES);
     gui.gselection = gwy_graph_area_get_selection(area, GWY_GRAPH_STATUS_XLINES);
 
-    hbox = gwy_hbox_new(24);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 24);
     gwy_dialog_add_content(dialog, hbox, TRUE, TRUE, 4);
 
     table = gui.table_options = gwy_param_table_new(args->params);
@@ -351,7 +337,6 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     g_object_unref(gui.store);
     g_object_unref(gui.gmodel);
     g_object_unref(gui.image);
-    g_object_unref(gui.data);
 
     return outcome;
 }
@@ -516,8 +501,8 @@ point_selection_changed(ModuleGUI *gui,
     ModuleArgs *args = gui->args;
     SliceBasePlane base_plane = gwy_params_get_enum(args->params, PARAM_BASE_PLANE);
     SliceOutputType output_type = gwy_params_get_enum(args->params, PARAM_OUTPUT_TYPE);
-    GwyDataField *field = gui->image;
-    gint xres = gwy_data_field_get_xres(field), yres = gwy_data_field_get_yres(field);
+    GwyField *field = gui->image;
+    gint xres = gwy_field_get_xres(field), yres = gwy_field_get_yres(field);
     gint i, j;
     SlicePos pos;
     gdouble xy[2];
@@ -533,8 +518,8 @@ point_selection_changed(ModuleGUI *gui,
     if (output_type == OUTPUT_GRAPHS)
         gui->current_object = id;
 
-    j = CLAMP(gwy_data_field_rtoj(field, xy[0]), 0, xres-1);
-    i = CLAMP(gwy_data_field_rtoi(field, xy[1]), 0, yres-1);
+    j = CLAMP(gwy_field_rtoj(field, xy[0]), 0, xres-1);
+    i = CLAMP(gwy_field_rtoi(field, xy[1]), 0, yres-1);
 
     fill_pos_from_params(args->params, &pos);
     if (base_plane == PLANE_XY || base_plane == PLANE_XZ)
@@ -710,7 +695,7 @@ preview(gpointer user_data)
     extract_graph_curve(args, gcmodel, id);
 
     extract_image_plane(args, gui->image, id);
-    gwy_data_field_data_changed(gui->image);
+    gwy_field_data_changed(gui->image);
     gwy_set_data_preview_size(GWY_DATA_VIEW(gui->dataview), PREVIEW_SIZE);
 }
 
@@ -749,7 +734,7 @@ update_multiselection(ModuleGUI *gui)
 }
 
 static void
-execute(ModuleArgs *args, GwyContainer *data, gint id)
+execute(ModuleArgs *args, GwyFile *data, gint id)
 {
     gboolean multiselect = gwy_params_get_boolean(args->params, PARAM_MULTISELECT);
     SliceOutputType output_type = gwy_params_get_enum(args->params, PARAM_OUTPUT_TYPE);
@@ -784,60 +769,60 @@ execute(ModuleArgs *args, GwyContainer *data, gint id)
 }
 
 static void
-extract_one_image(ModuleArgs *args, GwyContainer *data, gint id, gint idx)
+extract_one_image(ModuleArgs *args, GwyFile *data, gint id, gint idx)
 {
     SliceBasePlane base_plane = gwy_params_get_enum(args->params, PARAM_BASE_PLANE);
     SlicePos *pos = &g_array_index(args->allpos, SlicePos, idx);
     GwyBrick *brick = args->brick;
-    GwyDataField *dfield;
-    GwySIValueFormat *vf;
+    GwyField *dfield;
+    GwyValueFormat *vf;
     gchar *title = NULL;
-    const guchar *gradient;
     gdouble r;
     gint i, newid;
 
-    dfield = gwy_data_field_new(1, 1, 1.0, 1.0, TRUE);
+    dfield = gwy_field_new(1, 1, 1.0, 1.0, TRUE);
     extract_image_plane(args, dfield, idx);
-    newid = gwy_app_data_browser_add_data_field(dfield, data, TRUE);
+    newid = gwy_file_add_image(data, dfield);
+    gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
     g_object_unref(dfield);
-    gwy_app_channel_log_add(data, -1, newid, "volume::volume_slice", NULL);
-
-    if (gwy_container_gis_string(data, gwy_app_get_brick_palette_key_for_id(id), &gradient))
-        gwy_container_set_const_string(data, gwy_app_get_data_palette_key_for_id(newid), gradient);
+    gwy_file_sync_items(data, GWY_FILE_VOLUME, id,
+                        data, GWY_FILE_IMAGE, newid,
+                        GWY_FILE_ITEM_PALETTE, FALSE);
+    gwy_log_add_full(data, GWY_FILE_IMAGE, -1, newid, "volume::volume_slice", NULL);
 
     if (base_plane == PLANE_XY || base_plane == PLANE_YX) {
         i = pos->z;
         r = gwy_brick_ktor_cal(brick, i);
         if (args->zcalibration)
-            vf = gwy_data_line_get_value_format_y(args->zcalibration, GWY_SI_UNIT_FORMAT_VFUNICODE, NULL);
+            vf = gwy_line_get_value_format_y(args->zcalibration, GWY_UNIT_FORMAT_VFUNICODE, NULL);
         else
-            vf = gwy_brick_get_value_format_z(brick, GWY_SI_UNIT_FORMAT_VFUNICODE, NULL);
+            vf = gwy_brick_get_value_format_z(brick, GWY_UNIT_FORMAT_VFUNICODE, NULL);
         title = g_strdup_printf(_("Z slice at %.*f%s%s (#%d)"),
                                 vf->precision, r/vf->magnitude, strlen(vf->units) ? " " : "", vf->units, i);
     }
     else if (base_plane == PLANE_XZ || base_plane == PLANE_ZX) {
         i = pos->y;
         r = gwy_brick_jtor(brick, i);
-        vf = gwy_brick_get_value_format_y(brick, GWY_SI_UNIT_FORMAT_VFUNICODE, NULL);
+        vf = gwy_brick_get_value_format_y(brick, GWY_UNIT_FORMAT_VFUNICODE, NULL);
         title = g_strdup_printf(_("Y slice at %.*f%s%s (#%d)"),
                                 vf->precision, r/vf->magnitude, strlen(vf->units) ? " " : "", vf->units, i);
     }
     else if (base_plane == PLANE_YZ || base_plane == PLANE_ZY) {
         i = pos->x;
         r = gwy_brick_itor(brick, i);
-        vf = gwy_brick_get_value_format_x(brick, GWY_SI_UNIT_FORMAT_VFUNICODE, NULL);
+        vf = gwy_brick_get_value_format_x(brick, GWY_UNIT_FORMAT_VFUNICODE, NULL);
         title = g_strdup_printf(_("X slice at %.*f%s%s (#%d)"),
                                 vf->precision, r/vf->magnitude, strlen(vf->units) ? " " : "", vf->units, i);
     }
     else {
         g_return_if_reached();
     }
-    gwy_si_unit_value_format_free(vf);
-    gwy_container_set_string(data, gwy_app_get_data_title_key_for_id(newid), title);
+    gwy_value_format_free(vf);
+    gwy_file_pass_title(data, GWY_FILE_IMAGE, newid, title);
 }
 
 static void
-extract_image_plane(ModuleArgs *args, GwyDataField *dfield,
+extract_image_plane(ModuleArgs *args, GwyField *dfield,
                     guint idx)
 {
     SliceBasePlane base_plane = gwy_params_get_enum(args->params, PARAM_BASE_PLANE);
@@ -859,8 +844,8 @@ extract_image_plane(ModuleArgs *args, GwyDataField *dfield,
     }
 
     if (do_flip) {
-        GwyDataField *tmp = gwy_data_field_duplicate(dfield);
-        gwy_data_field_flip_xy(tmp, dfield, FALSE);
+        GwyField *tmp = gwy_field_copy(dfield);
+        gwy_field_transpose(tmp, dfield, FALSE);
         g_object_unref(tmp);
     }
 }
@@ -873,26 +858,26 @@ extract_graph_curve(ModuleArgs *args,
     SliceBasePlane base_plane = gwy_params_get_enum(args->params, PARAM_BASE_PLANE);
     const SlicePos *pos = &g_array_index(args->allpos, SlicePos, idx);
     GwyBrick *brick = args->brick;
-    GwyDataLine *line, *calibration = NULL;
+    GwyLine *line, *calibration = NULL;
     gchar *desc;
 
     gwy_debug("%d (%u)", idx, (guint)args->allpos->len);
     gwy_debug("(%d, %d, %d)", pos->x, pos->y, pos->z);
-    line = gwy_data_line_new(1, 1.0, FALSE);
+    line = gwy_line_new(1, 1.0, FALSE);
     if (base_plane == PLANE_XY || base_plane == PLANE_YX) {
         gwy_brick_extract_line(brick, line, pos->x, pos->y, 0, pos->x, pos->y, brick->zres, FALSE);
-        gwy_data_line_set_offset(line, brick->zoff);
+        gwy_line_set_offset(line, brick->zoff);
         calibration = args->zcalibration;
         desc = g_strdup_printf(_("Z graph at x: %d y: %d"), pos->x, pos->y);
     }
     else if (base_plane == PLANE_YZ || base_plane == PLANE_ZY) {
         gwy_brick_extract_line(brick, line, 0, pos->y, pos->z, brick->xres-1, pos->y, pos->z, FALSE);
-        gwy_data_line_set_offset(line, brick->xoff);
+        gwy_line_set_offset(line, brick->xoff);
         desc = g_strdup_printf(_("X graph at y: %d z: %d"), pos->y, pos->z);
     }
     else if (base_plane == PLANE_XZ || base_plane == PLANE_ZX) {
         gwy_brick_extract_line(brick, line, pos->x, 0, pos->z, pos->x, brick->yres-1, pos->z, FALSE);
-        gwy_data_line_set_offset(line, brick->yoff);
+        gwy_line_set_offset(line, brick->yoff);
         desc = g_strdup_printf(_("Y graph at x: %d z: %d"), pos->x, pos->z);
     }
     else {
@@ -906,12 +891,12 @@ extract_graph_curve(ModuleArgs *args,
 
     if (calibration) {
         gwy_graph_curve_model_set_data(gcmodel,
-                                       gwy_data_line_get_data(calibration), gwy_data_line_get_data(line),
-                                       gwy_data_line_get_res(line));
+                                       gwy_line_get_data(calibration), gwy_line_get_data(line),
+                                       gwy_line_get_res(line));
         gwy_graph_curve_model_enforce_order(gcmodel);
     }
     else
-        gwy_graph_curve_model_set_data_from_dataline(gcmodel, line, 0, 0);
+        gwy_graph_curve_model_set_data_from_line(gcmodel, line, 0, 0);
 
     g_object_unref(line);
 }
@@ -922,7 +907,7 @@ extract_gmodel(ModuleArgs *args, GwyGraphModel *gmodel)
     SliceBasePlane base_plane = gwy_params_get_enum(args->params, PARAM_BASE_PLANE);
     GwyBrick *brick = args->brick;
     const gchar *xlabel, *ylabel, *gtitle;
-    GwySIUnit *xunit = NULL, *yunit;
+    GwyUnit *xunit = NULL, *yunit;
 
     if (base_plane == PLANE_XY || base_plane == PLANE_YX) {
         gtitle = _("Volume Z graphs");
@@ -945,20 +930,20 @@ extract_gmodel(ModuleArgs *args, GwyGraphModel *gmodel)
 
     if (base_plane == PLANE_XY || base_plane == PLANE_YX) {
         if (args->zcalibration)
-            xunit = gwy_data_line_get_si_unit_y(args->zcalibration);
+            xunit = gwy_line_get_unit_y(args->zcalibration);
         else
-            xunit = gwy_brick_get_si_unit_z(brick);
+            xunit = gwy_brick_get_unit_z(brick);
     }
     else if (base_plane == PLANE_YZ || base_plane == PLANE_ZY)
-        xunit = gwy_brick_get_si_unit_x(brick);
+        xunit = gwy_brick_get_unit_x(brick);
     else if (base_plane == PLANE_ZX || base_plane == PLANE_XZ)
-        xunit = gwy_brick_get_si_unit_y(brick);
-    yunit = gwy_brick_get_si_unit_w(brick);
+        xunit = gwy_brick_get_unit_y(brick);
+    yunit = gwy_brick_get_unit_w(brick);
 
     g_object_set(gmodel,
                  "title", gtitle,
-                 "si-unit-x", xunit,
-                 "si-unit-y", yunit,
+                 "unit-x", xunit,
+                 "unit-y", yunit,
                  "axis-label-bottom", xlabel,
                  "axis-label-left", ylabel,
                  NULL);
