@@ -1,5 +1,5 @@
 /*
- *  $Id: volume_strayfield.c 26744 2024-10-18 15:03:29Z yeti-dn $
+ *  $Id: volume_strayfield.c 28860 2025-11-14 12:59:38Z yeti-dn $
  *  Copyright (C) 2015-2023 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,19 +16,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/brick.h>
-#include <libprocess/stats.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libprocess/mfm.h>
-#include <libprocess/filters.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-volume.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
+#include <gwy.h>
 
 #define RUN_MODES (GWY_RUN_INTERACTIVE)
 
@@ -67,16 +58,15 @@ typedef struct {
     GwyParams *params;
     GwyBrick *brick;
     GwyBrick *strayfield;
-    GwyDataLine *rmsdiff;
+    GwyLine *rmsdiff;
     /* Cached input data properties. */
-    GwyDataLine *calibration;
+    GwyLine *calibration;
 } ModuleArgs;
 
 typedef struct {
     ModuleArgs *args;
     GwyParams *params;
-    GwyContainer *data;
-    GwyDataField *xyplane;
+    GwyField *xyplane;
     GtkWidget *dialog;
     GwyParamTable *table_z;
     GwyParamTable *table_display;
@@ -88,31 +78,31 @@ typedef struct {
     gint calculated_zfrom;
 } ModuleGUI;
 
-static gboolean         module_register         (void);
-static void             strayfield              (GwyContainer *data,
-                                                 GwyRunType run);
-static GwyDialogOutcome run_gui                 (ModuleArgs *args,
-                                                 GwyContainer *data,
-                                                 gint id);
-static void             param_changed           (ModuleGUI *gui,
-                                                 gint id);
-static gboolean         execute                 (ModuleArgs *args,
-                                                 GtkWindow *wait_window);
-static void             preview                 (gpointer user_data);
-static void             point_selection_changed (ModuleGUI *gui,
-                                                 gint id,
-                                                 GwySelection *selection);
-static void             graph_selection_changed (ModuleGUI *gui,
-                                                 gint id,
-                                                 GwySelection *selection);
-static void             extract_xyplane         (ModuleGUI *gui);
-static void             update_graph_curves     (ModuleGUI *gui);
-static void             setup_gmodel            (ModuleArgs *args,
-                                                 GwyGraphModel *gmodel);
-static void             dialog_response_after   (GtkDialog *dialog,
-                                                 gint response,
-                                                 ModuleGUI *gui);
-static void             sanitise_params         (ModuleArgs *args);
+static gboolean         module_register        (void);
+static void             module_main            (GwyFile *data,
+                                                GwyRunModeFlags mode);
+static GwyDialogOutcome run_gui                (ModuleArgs *args,
+                                                GwyFile *data,
+                                                gint id);
+static void             param_changed          (ModuleGUI *gui,
+                                                gint id);
+static gboolean         execute                (ModuleArgs *args,
+                                                GtkWindow *wait_window);
+static void             preview                (gpointer user_data);
+static void             point_selection_changed(ModuleGUI *gui,
+                                                gint id,
+                                                GwySelection *selection);
+static void             graph_selection_changed(ModuleGUI *gui,
+                                                gint id,
+                                                GwySelection *selection);
+static void             extract_xyplane        (ModuleGUI *gui);
+static void             update_graph_curves    (ModuleGUI *gui);
+static void             setup_gmodel           (ModuleArgs *args,
+                                                GwyGraphModel *gmodel);
+static void             dialog_response_after  (GtkDialog *dialog,
+                                                gint response,
+                                                ModuleGUI *gui);
+static void             sanitise_params        (ModuleArgs *args);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
@@ -130,7 +120,7 @@ static gboolean
 module_register(void)
 {
     gwy_volume_func_register("volume_strayfield",
-                             (GwyVolumeFunc)&strayfield,
+                             module_main,
                              N_("/SPM M_odes/_Stray Field Consistency..."),
                              NULL,
                              RUN_MODES,
@@ -161,9 +151,9 @@ define_module_params(void)
 
     paramdef = gwy_param_def_new();
     gwy_param_def_set_function_name(paramdef, gwy_volume_func_current());
-    gwy_param_def_add_gwyenum(paramdef, PARAM_QUANTITY, "quantity", gwy_sgettext("_Quantity"),
+    gwy_param_def_add_gwyenum(paramdef, PARAM_QUANTITY, "quantity", _("_Quantity"),
                               quantities, G_N_ELEMENTS(quantities), GWY_STRAYFIELD_SINGLE);
-    gwy_param_def_add_gwyenum(paramdef, PARAM_SHOW_TYPE, "show_type", gwy_sgettext("verb|_Display"),
+    gwy_param_def_add_gwyenum(paramdef, PARAM_SHOW_TYPE, "show_type", C_("verb", "_Display"),
                               displays, G_N_ELEMENTS(displays), SHOW_DATA);
     gwy_param_def_add_int(paramdef, PARAM_XPOS, "xpos", _("_X"), -1, G_MAXINT, -1);
     gwy_param_def_add_int(paramdef, PARAM_YPOS, "ypos", _("_Y"), -1, G_MAXINT, -1);
@@ -176,27 +166,26 @@ define_module_params(void)
 }
 
 static void
-strayfield(GwyContainer *data, GwyRunType run)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
     GwyDialogOutcome outcome;
     gint id, zres;
 
-    g_return_if_fail(run & RUN_MODES);
-    g_return_if_fail(g_type_from_name("GwyLayerPoint"));
+    g_return_if_fail(mode & RUN_MODES);
 
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_BRICK, &args.brick,
-                                     GWY_APP_BRICK_ID, &id,
-                                     0);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_BRICK, &args.brick,
+                                 GWY_APP_BRICK_ID, &id,
+                                 0);
     g_return_if_fail(GWY_IS_BRICK(args.brick));
 
     args.calibration = gwy_brick_get_zcalibration(args.brick);
     zres = gwy_brick_get_zres(args.brick);
-    if (args.calibration && zres != gwy_data_line_get_res(args.calibration))
+    if (args.calibration && zres != gwy_line_get_res(args.calibration))
         args.calibration = NULL;
     args.strayfield = gwy_brick_new_alike(args.brick, TRUE);
-    args.rmsdiff = gwy_data_line_new(zres, zres, TRUE);
+    args.rmsdiff = gwy_line_new(zres, zres, TRUE);
     args.params = gwy_params_new_from_settings(define_module_params());
     sanitise_params(&args);
 
@@ -214,29 +203,23 @@ end:
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     GwyBrick *brick = args->brick;
     gint zres = gwy_brick_get_zres(brick);
-    GtkWidget *hbox, *align;
+    GtkWidget *hbox;
     GwyGraphArea *area;
     GwyGraph *graph;
     GwyParamTable *table;
     GwyDialog *dialog;
     ModuleGUI gui;
     GwyDialogOutcome outcome;
-    const guchar *gradient;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
 
-    gui.data = gwy_container_new();
-    gui.xyplane = gwy_data_field_new(1, 1, 1.0, 1.0, FALSE);
+    gui.xyplane = gwy_field_new(1, 1, 1.0, 1.0, FALSE);
     extract_xyplane(&gui);
-    gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(0), gui.xyplane);
-
-    if (gwy_container_gis_string(data, gwy_app_get_brick_palette_key_for_id(id), &gradient))
-        gwy_container_set_const_string(gui.data, gwy_app_get_data_palette_key_for_id(0), gradient);
 
     gui.gmodel = gwy_graph_model_new();
     setup_gmodel(args, gui.gmodel);
@@ -245,15 +228,15 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_UPDATE, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, 0);
 
-    hbox = gwy_hbox_new(0);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
     gwy_dialog_add_content(dialog, hbox, FALSE, FALSE, 4);
 
-    align = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-    gtk_box_pack_start(GTK_BOX(hbox), align, FALSE, FALSE, 0);
-
-    gui.dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
-    gtk_container_add(GTK_CONTAINER(align), gui.dataview);
-    gui.image_selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(gui.dataview), 0, "Point", 1, TRUE);
+    gui.dataview = gwy_create_preview(gui.xyplane, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(gui.dataview), data, GWY_FILE_VOLUME, id, GWY_FILE_ITEM_PALETTE);
+    gtk_widget_set_halign(gui.dataview, GTK_ALIGN_START);
+    gtk_widget_set_valign(gui.dataview, GTK_ALIGN_START);
+    gtk_box_pack_start(GTK_BOX(hbox), gui.dataview, FALSE, FALSE, 0);
+    gui.image_selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(gui.dataview), GWY_TYPE_LAYER_POINT, 1, TRUE);
 
     gui.graph = gwy_graph_new(gui.gmodel);
     graph = GWY_GRAPH(gui.graph);
@@ -266,7 +249,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gui.graph_selection = gwy_graph_area_get_selection(area, GWY_GRAPH_STATUS_XLINES);
     gwy_selection_set_max_objects(gui.graph_selection, 1);
 
-    hbox = gwy_hbox_new(24);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 24);
     gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), hbox, TRUE, TRUE, 4);
 
     gui.table_z = table = gwy_param_table_new(args->params);
@@ -300,7 +283,6 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 
     g_object_unref(gui.gmodel);
     g_object_unref(gui.xyplane);
-    g_object_unref(gui.data);
 
     return outcome;
 }
@@ -310,14 +292,14 @@ setup_gmodel(ModuleArgs *args, GwyGraphModel *gmodel)
 {
     static const gchar *descriptions[NCURVES] = { N_("Extracted"), N_("Calculated"), };
     GwyBrick *brick = args->brick;
-    GwyDataLine *calibration = args->calibration;
-    GwySIUnit *xunit = (calibration ? gwy_data_line_get_si_unit_y(calibration) : gwy_brick_get_si_unit_z(brick));
+    GwyLine *calibration = args->calibration;
+    GwyUnit *xunit = (calibration ? gwy_line_get_unit_y(calibration) : gwy_brick_get_unit_z(brick));
     GwyGraphCurveModel *gcmodel;
     guint i;
 
     g_object_set(gmodel,
-                 "si-unit-x", xunit,
-                 "si-unit-y", gwy_brick_get_si_unit_w(brick),
+                 "unit-x", xunit,
+                 "unit-y", gwy_brick_get_unit_w(brick),
                  "axis-label-bottom", "z",
                  "axis-label-left", "w",
                  NULL);
@@ -347,7 +329,7 @@ param_changed(ModuleGUI *gui, gint id)
     }
     if (id < 0 || id == PARAM_ZFROM || id == PARAM_SHOW_TYPE) {
         extract_xyplane(gui);
-        gwy_data_field_data_changed(gui->xyplane);
+        gwy_field_data_changed(gui->xyplane);
     }
     if (id < 0 || id == PARAM_XPOS || id == PARAM_YPOS || id == PARAM_QUANTITY)
         update_graph_curves(gui);
@@ -374,7 +356,7 @@ preview(gpointer user_data)
         gui->calculated_zfrom = gwy_params_get_int(gui->args->params, PARAM_ZFROM);
     update_graph_curves(gui);
     extract_xyplane(gui);
-    gwy_data_field_data_changed(gui->xyplane);
+    gwy_field_data_changed(gui->xyplane);
 }
 
 static void
@@ -441,8 +423,8 @@ compute_strayfield_brick(GwyBrick *brick, GwyBrick *result,
                          gint zfrom, gint zto,
                          GtkWindow *wait_window)
 {
-    GwyDataField *shiftedfield;
-    GwyDataField *basefield;
+    GwyField *shiftedfield;
+    GwyField *basefield;
     gboolean cancelled = FALSE;
     gdouble z0, z;
     gint level;
@@ -453,15 +435,15 @@ compute_strayfield_brick(GwyBrick *brick, GwyBrick *result,
     if (wait_window)
         gwy_app_wait_start(wait_window, _("Building stray field dependence..."));
 
-    basefield = gwy_data_field_new(1, 1, 1.0, 1.0, FALSE);
+    basefield = gwy_field_new(1, 1, 1.0, 1.0, FALSE);
     gwy_brick_extract_xy_plane(brick, basefield, zfrom);
-    shiftedfield = gwy_data_field_new_alike(basefield, FALSE);
+    shiftedfield = gwy_field_new_alike(basefield, FALSE);
     gwy_brick_clear(result);
 
     z0 = gwy_brick_ktor_cal(brick, zfrom);
     for (level = zfrom; level < zto; level++) {
         z = gwy_brick_ktor_cal(brick, level);
-        gwy_data_field_mfm_shift_z(basefield, shiftedfield, z - z0);
+        gwy_field_mfm_shift_z(basefield, shiftedfield, z - z0);
         gwy_brick_set_xy_plane(result, shiftedfield, level);
         if (wait_window && !gwy_app_wait_set_fraction(((gdouble)(level-zfrom))/(zto-zfrom))) {
             cancelled = TRUE;
@@ -480,31 +462,31 @@ compute_strayfield_brick(GwyBrick *brick, GwyBrick *result,
     return !cancelled;
 }
 
-static GwyDataLine*
+static GwyLine*
 make_xdata(GwyBrick *brick, gint from, gint to)
 {
-    GwyDataLine *xdataline, *calibration;
+    GwyLine *xline, *calibration;
     gint res = to - from;
     gdouble *d;
     gdouble dz, zoff;
 
     calibration = gwy_brick_get_zcalibration(brick);
-    if (calibration && (gwy_brick_get_zres(brick) != gwy_data_line_get_res(calibration)))
+    if (calibration && (gwy_brick_get_zres(brick) != gwy_line_get_res(calibration)))
         calibration = NULL;
 
     if (calibration) {
-        xdataline = gwy_data_line_duplicate(calibration);
-        gwy_data_line_resize(xdataline, from, to);
+        xline = gwy_line_copy(calibration);
+        gwy_line_crop(xline, from, to-from);
     }
     else {
-        xdataline = gwy_data_line_new(to - from, to - from, FALSE);
-        d = gwy_data_line_get_data(xdataline);
+        xline = gwy_line_new(to - from, to - from, FALSE);
+        d = gwy_line_get_data(xline);
         dz = gwy_brick_get_dz(brick);
         zoff = gwy_brick_get_zoffset(brick);
         gwy_math_linspace(d, res, zoff + from*dz, dz);
     }
 
-    return xdataline;
+    return xline;
 }
 
 static gboolean
@@ -520,8 +502,8 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
     if (!ndata || !compute_strayfield_brick(brick, strayfield, zfrom, zto, wait_window))
         return FALSE;
 
-    gwy_data_line_resample(args->rmsdiff, ndata, GWY_INTERPOLATION_NONE);
-    d = gwy_data_line_get_data(args->rmsdiff);
+    gwy_line_resize(args->rmsdiff, ndata);
+    d = gwy_line_get_data(args->rmsdiff);
     d[0] = 0;
     for (k = 0; k < ndata; k++)
         d[k] = get_brick_self_plane_rms(strayfield, zfrom, zfrom+k);
@@ -535,16 +517,16 @@ extract_one_graph_curve(GwyBrick *brick,
                         gint zfrom, gint zto,
                         GwyGraphCurveModel *gcmodel)
 {
-    GwyDataLine *xdataline, *ydataline;
+    GwyLine *xline, *yline;
     gint ndata = zto - zfrom;
 
-    xdataline = make_xdata(brick, zfrom, zto);
-    ydataline = gwy_data_line_new(ndata, ndata, FALSE);
-    gwy_brick_extract_line(brick, ydataline, col, row, zfrom, col, row, zto, FALSE);
+    xline = make_xdata(brick, zfrom, zto);
+    yline = gwy_line_new(ndata, ndata, FALSE);
+    gwy_brick_extract_line(brick, yline, col, row, zfrom, col, row, zto, FALSE);
     gwy_graph_curve_model_set_data(gcmodel,
-                                   gwy_data_line_get_data(xdataline), gwy_data_line_get_data(ydataline), ndata);
-    g_object_unref(xdataline);
-    g_object_unref(ydataline);
+                                   gwy_line_get_data(xline), gwy_line_get_data(yline), ndata);
+    g_object_unref(xline);
+    g_object_unref(yline);
 }
 
 static void
@@ -571,13 +553,13 @@ static void
 plot_rmsdiff_curve(ModuleArgs *args, gint zfrom, GwyGraphCurveModel *gcmodel)
 {
     gint zto = gwy_brick_get_zres(args->brick);
-    GwyDataLine *xdataline, *rmsdiff = args->rmsdiff;
+    GwyLine *xline, *rmsdiff = args->rmsdiff;
     gint ndata = zto - zfrom;
 
-    xdataline = make_xdata(args->brick, zfrom, zto);
+    xline = make_xdata(args->brick, zfrom, zto);
     gwy_graph_curve_model_set_data(gcmodel,
-                                   gwy_data_line_get_data(xdataline), gwy_data_line_get_data(rmsdiff), ndata);
-    g_object_unref(xdataline);
+                                   gwy_line_get_data(xline), gwy_line_get_data(rmsdiff), ndata);
+    g_object_unref(xline);
 }
 
 static void
