@@ -1,5 +1,5 @@
 /*
- *  $Id: volume_swaxes.c 25827 2023-10-11 11:42:22Z klapetek $
+ *  $Id: volume_swaxes.c 28848 2025-11-10 18:54:08Z yeti-dn $
  *  Copyright (C) 2017-2023 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,15 +16,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/brick.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-volume.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
+#include <gwy.h>
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
 
@@ -63,8 +58,8 @@ typedef struct {
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
-static void             volume_swaxes       (GwyContainer *data,
-                                             GwyRunType mode);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static GwyDialogOutcome run_gui             (ModuleArgs *args);
 static void             param_changed       (ModuleGUI *gui,
                                              gint id);
@@ -72,7 +67,7 @@ static void             update_third_axis   (ModuleGUI *gui,
                                              gint changed_axis);
 static gboolean         axes_are_consistent (const AxisType *xyz);
 static void             execute             (ModuleArgs *args,
-                                             GwyContainer *data,
+                                             GwyFile *data,
                                              gint id);
 static void             sanitize_params     (ModuleArgs *args);
 
@@ -92,9 +87,9 @@ static gboolean
 module_register(void)
 {
     gwy_volume_func_register("volume_swaxes",
-                             (GwyVolumeFunc)&volume_swaxes,
+                             module_main,
                              N_("/_Basic Operations/S_wap Axes..."),
-                             GWY_STOCK_VOLUME_SWAP_AXES,
+                             GWY_ICON_VOLUME_SWAP_AXES,
                              RUN_MODES,
                              GWY_MENU_FLAG_VOLUME,
                              N_("Swap axes"));
@@ -131,7 +126,7 @@ define_module_params(void)
 }
 
 static void
-volume_swaxes(GwyContainer *data, GwyRunType mode)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
     GwyDialogOutcome outcome;
@@ -139,10 +134,10 @@ volume_swaxes(GwyContainer *data, GwyRunType mode)
 
     g_return_if_fail(mode & RUN_MODES);
 
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_BRICK, &args.brick,
-                                     GWY_APP_BRICK_ID, &id,
-                                     0);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_BRICK, &args.brick,
+                                 GWY_APP_BRICK_ID, &id,
+                                 0);
     g_return_if_fail(GWY_IS_BRICK(args.brick));
 
     args.has_zcal = !!gwy_brick_get_zcalibration(args.brick);
@@ -168,7 +163,7 @@ run_gui(ModuleArgs *args)
     GwyParamTable *table;
     ModuleGUI gui;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
     gui.last_changed = 1;
     gui.second_last_changed = 0;
@@ -266,7 +261,7 @@ axes_are_consistent(const AxisType *xyz)
 }
 
 static void
-execute(ModuleArgs *args, GwyContainer *data, gint id)
+execute(ModuleArgs *args, GwyFile *data, gint id)
 {
     GwyParams *params = args->params;
     AxisType ax = gwy_params_get_enum(params, PARAM_X);
@@ -277,7 +272,7 @@ execute(ModuleArgs *args, GwyContainer *data, gint id)
     GwyBrickTransposeType transtype;
     gboolean xinv = ax & 1, yinv = ay & 1, zinv = az & 1;
     GwyBrick *result, *brick = args->brick;
-    GwyDataField *tmp, *preview = NULL;
+    GwyField *tmp, *preview = NULL;
     gdouble xoff, yoff;
     gint xres, yres, newid;
     GQuark quarks[2];
@@ -302,60 +297,63 @@ execute(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_brick_transpose(brick, result, transtype, xinv, yinv, zinv);
 
     /* Reuse the old preview if the XY plane is preserved. */
-    if (gwy_container_gis_object(data, gwy_app_get_brick_preview_key_for_id(id), &preview)
+    if (gwy_container_gis_object(GWY_CONTAINER(data), gwy_file_key_volume_picture(id), &preview)
         && (bx == AXIS_XPOS || bx == AXIS_YPOS)
         && (by == AXIS_XPOS || by == AXIS_YPOS)) {
         if (ax == AXIS_YPOS && ay == AXIS_XNEG)
-            tmp = gwy_data_field_new_rotated_90(preview, TRUE);
+            tmp = gwy_field_new_rotated_90(preview, TRUE);
         else if (ax == AXIS_YNEG && ay == AXIS_XPOS)
-            tmp = gwy_data_field_new_rotated_90(preview, FALSE);
+            tmp = gwy_field_new_rotated_90(preview, FALSE);
         else {
-            tmp = gwy_data_field_duplicate(preview);
+            tmp = gwy_field_copy(preview);
             if (ax == AXIS_XPOS && ay == AXIS_YPOS) {
                 /* Do nothing. */
             }
             else if (ax == AXIS_XNEG && ay == AXIS_YNEG)
-                gwy_data_field_invert(tmp, TRUE, TRUE, FALSE);
+                gwy_field_flip(tmp, TRUE, TRUE);
             else if (ax == AXIS_XNEG && ay == AXIS_YPOS)
-                gwy_data_field_invert(tmp, FALSE, TRUE, FALSE);
+                gwy_field_flip(tmp, TRUE, FALSE);
             else if (ax == AXIS_XPOS && ay == AXIS_YNEG)
-                gwy_data_field_invert(tmp, TRUE, FALSE, FALSE);
+                gwy_field_flip(tmp, FALSE, TRUE);
             else if (ax == AXIS_YPOS && ay == AXIS_XPOS)
-                gwy_data_field_flip_xy(preview, tmp, FALSE);
+                gwy_field_transpose(preview, tmp, FALSE);
             else if (ax == AXIS_YNEG && ay == AXIS_XNEG)
-                gwy_data_field_flip_xy(preview, tmp, TRUE);
+                gwy_field_transpose(preview, tmp, TRUE);
             else {
                 g_assert_not_reached();
             }
         }
-        GWY_SWAP(GwyDataField*, tmp, preview);
+        GWY_SWAP(GwyField*, tmp, preview);
 
-        xoff = gwy_data_field_get_xoffset(preview);
-        yoff = gwy_data_field_get_yoffset(preview);
-        gwy_data_field_set_xoffset(preview, yoff);
-        gwy_data_field_set_yoffset(preview, xoff);
+        xoff = gwy_field_get_xoffset(preview);
+        yoff = gwy_field_get_yoffset(preview);
+        gwy_field_set_xoffset(preview, yoff);
+        gwy_field_set_yoffset(preview, xoff);
     }
     else {
         xres = gwy_brick_get_xres(result);
         yres = gwy_brick_get_yres(result);
-        preview = gwy_data_field_new(xres, yres, xres, yres, FALSE);
+        preview = gwy_field_new(xres, yres, xres, yres, FALSE);
         gwy_brick_mean_xy_plane(result, preview);
     }
 
     /* Create new channel or modify the current one. */
     if (new_channel) {
-        newid = gwy_app_data_browser_add_brick(result, preview, data, TRUE);
-        gwy_app_set_brick_title(data, newid, _("Rotated Data"));
-        gwy_app_volume_log_add_volume(data, id, newid);
-        gwy_app_sync_volume_items(data, data, id, newid, FALSE, GWY_DATA_ITEM_GRADIENT, 0);
+        newid = gwy_file_add_volume(data, result, preview);
+        gwy_file_set_visible(data, GWY_FILE_VOLUME, newid, TRUE);
+        gwy_file_set_title(data, GWY_FILE_VOLUME, newid, _("Rotated Data"), TRUE);
+        gwy_log_add(data, GWY_FILE_VOLUME, id, newid);
+        gwy_file_sync_items(data, GWY_FILE_VOLUME, id,
+                            data, GWY_FILE_VOLUME, newid,
+                            GWY_FILE_ITEM_PALETTE, FALSE);
     }
     else {
-        quarks[0] = gwy_app_get_brick_key_for_id(id);
-        quarks[1] = gwy_app_get_brick_preview_key_for_id(id);
-        gwy_app_undo_qcheckpointv(data, G_N_ELEMENTS(quarks), quarks);
-        gwy_container_set_object(data, quarks[0], result);
-        gwy_container_set_object(data, quarks[1], preview);
-        gwy_app_volume_log_add_volume(data, id, id);
+        quarks[0] = gwy_file_key_volume(id);
+        quarks[1] = gwy_file_key_volume_picture(id);
+        gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), G_N_ELEMENTS(quarks), quarks);
+        gwy_file_set_volume(data, id, result);
+        gwy_container_set_object(GWY_CONTAINER(data), quarks[1], preview);
+        gwy_log_add(data, GWY_FILE_VOLUME, id, id);
     }
 
     g_object_unref(result);
