@@ -1,5 +1,5 @@
 /*
- *  $Id: volume_xystitch.c 26738 2024-10-17 13:16:07Z klapetek $
+ *  $Id: volume_xystitch.c 28905 2025-11-24 15:51:18Z yeti-dn $
  *  Copyright (C) 2023-2024 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,26 +16,11 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwythreads.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwynlfit.h>
-#include <libprocess/brick.h>
-#include <libprocess/stats.h>
-#include <libprocess/linestats.h>
-#include <libprocess/gwyprocess.h>
-#include <libprocess/gwyprocesstypes.h>
-#include <libprocess/correlation.h>
-#include <libgwydgets/gwydataview.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwydgets/gwydgetutils.h>
-#include <libgwymodule/gwymodule-volume.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
-#include "libgwyddion/gwyomp.h"
+#include <gwy.h>
+#include "libgwyddion/omp.h"
 
 #define RUN_MODES (GWY_RUN_INTERACTIVE)
 
@@ -61,7 +46,6 @@ enum {
     PARAM_BACKGROUND
 };
 
-
 typedef struct {
     GwyParams *params;
     GwyBrick *brick;
@@ -69,7 +53,7 @@ typedef struct {
     gdouble *xshift;
     gdouble *yshift;
     gdouble *zshift;
-    GwyDataField *background;
+    GwyField *background;
 } ModuleArgs;
 
 typedef struct {
@@ -77,27 +61,25 @@ typedef struct {
     GtkWidget *dialog;
     GtkWidget *result;
     GwyParamTable *table_options;
-    GwyContainer *data;
     GtkWidget *message;
     GtkWidget *dataview;
 } ModuleGUI;
 
-
-static gboolean              module_register          (void);
-static GwyParamDef*          define_module_params     (void);
-static void                  xystitch                 (GwyContainer *data,
-                                                       GwyRunType runtype);
-static gboolean              execute                  (ModuleArgs *args,
-                                                       GtkWindow *wait_window);
-static GwyDialogOutcome      run_gui                  (ModuleArgs *args,
-                                                       GwyContainer *data,
-                                                       gint id);
-static void                  param_changed            (ModuleGUI *gui,
-                                                       gint id);
-static void                  dialog_response          (GwyDialog *dialog,
-                                                       gint response,
-                                                       ModuleGUI *gui);
-static void                  recalc                   (gpointer user_data);
+static gboolean         module_register     (void);
+static GwyParamDef*     define_module_params(void);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
+static gboolean         execute             (ModuleArgs *args,
+                                             GtkWindow *wait_window);
+static GwyDialogOutcome run_gui             (ModuleArgs *args,
+                                             GwyFile *data,
+                                             gint id);
+static void             param_changed       (ModuleGUI *gui,
+                                             gint id);
+static void             dialog_response     (GwyDialog *dialog,
+                                             gint response,
+                                             ModuleGUI *gui);
+static void             recalc              (gpointer user_data);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
@@ -115,7 +97,7 @@ static gboolean
 module_register(void)
 {
     gwy_volume_func_register("volume_xystitch",
-                             (GwyVolumeFunc)&xystitch,
+                             module_main,
                              N_("/SPM M_odes/_XY Stitch..."),
                              NULL,
                              RUN_MODES,
@@ -148,15 +130,12 @@ define_module_params(void)
     gwy_param_def_add_int(paramdef, PARAM_YORDER, "yorder", _("Y polynomial degree"), 1, 4, 1);
     gwy_param_def_add_boolean(paramdef, PARAM_GRAPHS, "graphs", _("Plot graphs"), FALSE);
     gwy_param_def_add_boolean(paramdef, PARAM_BACKGROUND, "background", _("E_xtract background"), FALSE);
-//    gwy_param_def_add_boolean(paramdef, PARAM_ROTATE, "rotate", _("A_djust rotation"), FALSE);
-//    gwy_param_def_add_double(paramdef, PARAM_MAXSHIFT, "max_shift", _("_Shift range"), 0.0, 50, 20);
-//    gwy_param_def_add_double(paramdef, PARAM_MAXROTATE, "max_rotate", _("_Rotation range"), 0.0, 10, 1);
 
     return paramdef;
 }
 
 static void
-xystitch(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
     GwyBrick *brick = NULL;
@@ -167,17 +146,15 @@ xystitch(GwyContainer *data, GwyRunType runtype)
     GwyGraphCurveModel *gcmodel;
     gdouble *ishift;
     gint i, nshift;
-    const guchar *gradient;
 
-    g_return_if_fail(runtype & RUN_MODES);
-    g_return_if_fail(g_type_from_name("GwyLayerPoint"));
-    gwy_clear(&args, 1);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_clear1(args);
 
-    gwy_app_data_browser_get_current(GWY_APP_BRICK, &brick,
-                                     GWY_APP_BRICK_ID, &oldid,
-                                     GWY_APP_CONTAINER_ID, &graph_id.datano,
-                                     GWY_APP_GRAPH_MODEL_ID, &graph_id.id,
-                                     0);
+    gwy_data_browser_get_current(GWY_APP_BRICK, &brick,
+                                 GWY_APP_BRICK_ID, &oldid,
+                                 GWY_APP_CONTAINER_ID, &graph_id.datano,
+                                 GWY_APP_GRAPH_MODEL_ID, &graph_id.id,
+                                 0);
     g_return_if_fail(GWY_IS_BRICK(brick));
 
     ishift = NULL;
@@ -186,34 +163,32 @@ xystitch(GwyContainer *data, GwyRunType runtype)
     gwy_params_set_graph_id(args.params, PARAM_XOFFSETS, graph_id);
     gwy_params_set_graph_id(args.params, PARAM_YOFFSETS, graph_id);
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, oldid);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
             goto end;
     }
     if (outcome != GWY_DIALOG_HAVE_RESULT) {
-        if (!execute(&args, gwy_app_find_window_for_channel(data, oldid)))
+        if (!execute(&args, gwy_data_browser_get_window_for_data(data, GWY_FILE_IMAGE, oldid)))
             goto end;
     }
 
     if (args.result) {
-        newid = gwy_app_data_browser_add_surface(args.result, data, TRUE);
-        if (gwy_container_gis_string(data, gwy_app_get_brick_palette_key_for_id(oldid), &gradient))
-           gwy_container_set_const_string(data, gwy_app_get_surface_palette_key_for_id(newid), gradient);
-        gwy_app_set_surface_title(data, newid, _("Stitched"));
+        newid = gwy_file_add_xyz(data, args.result);
+        gwy_file_set_visible(data, GWY_FILE_XYZ, newid, TRUE);
+        gwy_file_sync_items(data, GWY_FILE_VOLUME, oldid,
+                            data, GWY_FILE_IMAGE, newid,
+                            GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_REAL_SQUARE, FALSE);
+        gwy_file_set_title(data, GWY_FILE_XYZ, newid, _("Stitched"), TRUE);
 
         if (gwy_params_get_boolean(args.params, PARAM_BACKGROUND)) {
-            newid = gwy_app_data_browser_add_data_field(args.background, data, TRUE);
-            gwy_app_sync_data_items(data, data, oldid, newid, FALSE,
-                                    GWY_DATA_ITEM_GRADIENT,
-                                    GWY_DATA_ITEM_MASK_COLOR,
-                                    GWY_DATA_ITEM_REAL_SQUARE,
-                                    0);
-            if (gwy_container_gis_string(data, gwy_app_get_brick_palette_key_for_id(oldid), &gradient))
-                 gwy_container_set_const_string(data, gwy_app_get_data_palette_key_for_id(newid), gradient);
-
-            gwy_app_set_data_field_title(data, newid, _("Background"));
+            newid = gwy_file_add_image(data, args.background);
+            gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
+            gwy_file_sync_items(data, GWY_FILE_VOLUME, oldid,
+                                data, GWY_FILE_IMAGE, newid,
+                                GWY_FILE_ITEM_PALETTE | GWY_FILE_ITEM_REAL_SQUARE, FALSE);
+            gwy_file_set_title(data, GWY_FILE_IMAGE, newid, _("Background"), TRUE);
         }
 
         if (gwy_params_get_boolean(args.params, PARAM_GRAPHS)) {
@@ -224,79 +199,80 @@ xystitch(GwyContainer *data, GwyRunType runtype)
                 ishift[i] = i;
 
             if (gwy_params_get_boolean(args.params, PARAM_XSHIFT)) {
-               gmodel = gwy_graph_model_new();
-               g_object_set(gmodel,
-                            "title", _("X shift"),
-                            "axis-label-left", _("X shift"),
-                            "axis-label-bottom", _("Frame number"),
-                            "si-unit-y", gwy_brick_get_si_unit_x(brick),
-                            NULL);
+                gmodel = gwy_graph_model_new();
+                g_object_set(gmodel,
+                             "title", _("X shift"),
+                             "axis-label-left", _("X shift"),
+                             "axis-label-bottom", _("Frame number"),
+                             "unit-y", gwy_brick_get_unit_x(brick),
+                             NULL);
 
-               gcmodel = gwy_graph_curve_model_new();
-               gwy_graph_curve_model_set_data(gcmodel, ishift, args.xshift, nshift);
+                gcmodel = gwy_graph_curve_model_new();
+                gwy_graph_curve_model_set_data(gcmodel, ishift, args.xshift, nshift);
 
-               g_object_set(gcmodel, "description", _("x-axis shift"), NULL);
-               gwy_graph_model_add_curve(gmodel, gcmodel);
-               gwy_object_unref(gcmodel);
-               gwy_app_data_browser_add_graph_model(gmodel, data, TRUE);
-               gwy_object_unref(gmodel);
+                g_object_set(gcmodel, "description", _("x-axis shift"), NULL);
+                gwy_graph_model_add_curve(gmodel, gcmodel);
+                g_clear_object(&gcmodel);
+                newid = gwy_file_add_graph(data, gmodel);
+                gwy_file_set_visible(data, GWY_FILE_GRAPH, newid, TRUE);
+                g_clear_object(&gmodel);
             }
 
             if (gwy_params_get_boolean(args.params, PARAM_YSHIFT)) {
-               gmodel = gwy_graph_model_new();
-               g_object_set(gmodel,
-                            "title", _("Y shift"),
-                            "axis-label-left", _("y shift"),
-                            "axis-label-bottom", "frame number",
-                            "si-unit-y", gwy_brick_get_si_unit_y(brick),
-                            NULL);
+                gmodel = gwy_graph_model_new();
+                g_object_set(gmodel,
+                             "title", _("Y shift"),
+                             "axis-label-left", _("y shift"),
+                             "axis-label-bottom", "frame number",
+                             "unit-y", gwy_brick_get_unit_y(brick),
+                             NULL);
 
-               gcmodel = gwy_graph_curve_model_new();
-               gwy_graph_curve_model_set_data(gcmodel, ishift, args.yshift, nshift);
+                gcmodel = gwy_graph_curve_model_new();
+                gwy_graph_curve_model_set_data(gcmodel, ishift, args.yshift, nshift);
 
-               g_object_set(gcmodel, "description", _("y-axis shift"), NULL);
-               gwy_graph_model_add_curve(gmodel, gcmodel);
-               gwy_object_unref(gcmodel);
-               gwy_app_data_browser_add_graph_model(gmodel, data, TRUE);
-               gwy_object_unref(gmodel);
+                g_object_set(gcmodel, "description", _("y-axis shift"), NULL);
+                gwy_graph_model_add_curve(gmodel, gcmodel);
+                g_clear_object(&gcmodel);
+                newid = gwy_file_add_graph(data, gmodel);
+                gwy_file_set_visible(data, GWY_FILE_GRAPH, newid, TRUE);
+                g_clear_object(&gmodel);
             }
 
-
             if (gwy_params_get_boolean(args.params, PARAM_ZSHIFT)) {
-               gmodel = gwy_graph_model_new();
-               g_object_set(gmodel,
-                            "title", _("Z shift"),
-                            "axis-label-left", _("z shift"),
-                            "axis-label-bottom", "frame number",
-                            "si-unit-y", gwy_brick_get_si_unit_w(brick),
-                            NULL);
+                gmodel = gwy_graph_model_new();
+                g_object_set(gmodel,
+                             "title", _("Z shift"),
+                             "axis-label-left", _("z shift"),
+                             "axis-label-bottom", "frame number",
+                             "unit-y", gwy_brick_get_unit_w(brick),
+                             NULL);
 
-               gcmodel = gwy_graph_curve_model_new();
-               gwy_graph_curve_model_set_data(gcmodel, ishift, args.zshift, nshift);
+                gcmodel = gwy_graph_curve_model_new();
+                gwy_graph_curve_model_set_data(gcmodel, ishift, args.zshift, nshift);
 
-               g_object_set(gcmodel, "description", _("z-axis shift"), NULL);
-               gwy_graph_model_add_curve(gmodel, gcmodel);
-               gwy_object_unref(gcmodel);
-               gwy_app_data_browser_add_graph_model(gmodel, data, TRUE);
-               gwy_object_unref(gmodel);
+                g_object_set(gcmodel, "description", _("z-axis shift"), NULL);
+                gwy_graph_model_add_curve(gmodel, gcmodel);
+                g_clear_object(&gcmodel);
+                newid = gwy_file_add_graph(data, gmodel);
+                gwy_file_set_visible(data, GWY_FILE_GRAPH, newid, TRUE);
+                g_clear_object(&gmodel);
             }
 
-
             g_free(ishift);
         }
     }
 
 end:
     g_object_unref(args.params);
-    GWY_OBJECT_UNREF(args.result);
-    GWY_OBJECT_UNREF(args.background);
+    g_clear_object(&args.result);
+    g_clear_object(&args.background);
     g_free(args.xshift);
     g_free(args.yshift);
     g_free(args.zshift);
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     GtkWidget *hbox, *vbox, *label;
     GwyParamTable *table;
@@ -304,27 +280,20 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     ModuleGUI gui;
     GwyDialogOutcome outcome;
     GwyBrick *brick = args->brick;
-    GwyDataField *field = gwy_data_field_new(gwy_brick_get_xres(brick), gwy_brick_get_yres(brick),
-                                             gwy_brick_get_xreal(brick), gwy_brick_get_yreal(brick),
-                                             TRUE);
-    const guchar *gradient;
+    GwyField *field = gwy_brick_new_field_like_xy_plane(brick, TRUE);
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
-    gui.data = gwy_container_new();
-
-    gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(0), field);
-    if (gwy_container_gis_string(data, gwy_app_get_brick_palette_key_for_id(id), &gradient))
-        gwy_container_set_const_string(gui.data, gwy_app_get_data_palette_key_for_id(0), gradient);
 
     gui.dialog = gwy_dialog_new(_("XY Stitch"));
     dialog = GWY_DIALOG(gui.dialog);
-    gtk_dialog_add_button(GTK_DIALOG(dialog), gwy_sgettext("verb|_Stitch"), RESPONSE_RECALC);
+    gtk_dialog_add_button(GTK_DIALOG(dialog), C_("verb", "_Stitch"), RESPONSE_RECALC);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    gui.dataview = gwy_create_preview(gui.data, 0, PREVIEW_X_SIZE, FALSE);
+    gui.dataview = gwy_create_preview(field, NULL, PREVIEW_X_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(gui.dataview), data, GWY_FILE_VOLUME, id, GWY_FILE_ITEM_PALETTE);
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(gui.dataview), FALSE);
-    vbox = gwy_vbox_new(0);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 
     table = gui.table_options = gwy_param_table_new(args->params);
     gwy_param_table_append_graph_id(table, PARAM_XOFFSETS);
@@ -343,15 +312,12 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_param_table_append_separator(table);
     gwy_param_table_append_checkbox(table, PARAM_GRAPHS);
     gwy_param_table_append_checkbox(table, PARAM_BACKGROUND);
-    //gwy_param_table_append_checkbox(table, PARAM_ROTATE);
-    //gwy_param_table_append_slider(table, PARAM_MAXROTATE);
-    //gwy_param_table_set_unitstr(table, PARAM_MAXROTATE, "deg");
 
     gwy_dialog_add_param_table(dialog, table);
     gtk_box_pack_start(GTK_BOX(vbox), gwy_param_table_widget(table), FALSE, FALSE, 0);
 
     gui.result = label = gtk_label_new(NULL);
-    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+    gtk_label_set_xalign(GTK_LABEL(label), 0.0);
     gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);
 
@@ -362,8 +328,6 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 
     outcome = gwy_dialog_run(dialog);
 
-    g_object_unref(gui.data);
-
     return outcome;
 }
 
@@ -430,7 +394,7 @@ static void
 recalc(gpointer user_data)
 {
     ModuleGUI *gui = (ModuleGUI*)user_data;
-    GwyDataField *dfield = gwy_container_get_object(gui->data, gwy_app_get_data_key_for_id(0));
+    GwyField *dfield = gwy_data_view_get_field(GWY_DATA_VIEW(gui->dataview));
     gdouble xmin, xmax, ymin, ymax, xrange, yrange;
     gint newxres, newyres;
 
@@ -463,16 +427,16 @@ recalc(gpointer user_data)
 
     //printf("newres %d %d   rng %g %g\n", newxres, newyres, xrange, yrange);
 
-    gwy_data_field_resample(dfield, newxres, newyres, GWY_INTERPOLATION_NONE);
-    gwy_data_field_set_xoffset(dfield, xmin);
-    gwy_data_field_set_yoffset(dfield, ymin);
-    gwy_data_field_set_xreal(dfield, xrange);
-    gwy_data_field_set_yreal(dfield, yrange);
+    gwy_field_resize(dfield, newxres, newyres);
+    gwy_field_set_xoffset(dfield, xmin);
+    gwy_field_set_yoffset(dfield, ymin);
+    gwy_field_set_xreal(dfield, xrange);
+    gwy_field_set_yreal(dfield, yrange);
 
-    gwy_data_field_average_xyz(dfield, NULL,
-                               gwy_surface_get_data_const(gui->args->result),
-                               gwy_surface_get_npoints(gui->args->result));
-    gwy_data_field_data_changed(dfield);
+    gwy_field_average_xyz(dfield, NULL,
+                          gwy_surface_get_data_const(gui->args->result),
+                          gwy_surface_get_npoints(gui->args->result));
+    gwy_field_data_changed(dfield);
 
     gwy_set_data_preview_size(GWY_DATA_VIEW(gui->dataview), newxres);
     gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
@@ -649,25 +613,25 @@ end:
 }
 
 
-static GwyDataField *
-get_xy_shift(GwyDataField *mdetail, GwyDataField *ndetail, gdouble *xshift, gdouble *yshift)
+static GwyField *
+get_xy_shift(GwyField *mdetail, GwyField *ndetail, gdouble *xshift, gdouble *yshift)
 {
     gint i, j, n;
     gdouble xsh, ysh, xoff, yoff, maxscore, zvals[9];
-    GwyDataField *score = gwy_data_field_new_alike(mdetail, TRUE);
-    gint xres = gwy_data_field_get_xres(mdetail);
-    gint yres = gwy_data_field_get_yres(mdetail);
+    GwyField *score = gwy_field_new_alike(mdetail, TRUE);
+    gint xres = gwy_field_get_xres(mdetail);
+    gint yres = gwy_field_get_yres(mdetail);
 
-    gwy_data_field_correlation_search(mdetail, ndetail, NULL, score,
-                                      GWY_CORR_SEARCH_PHASE_ONLY_SCORE,
-                                      0.1, GWY_EXTERIOR_MIRROR_EXTEND, 0);
+    gwy_field_correlation_search(mdetail, ndetail, NULL, score,
+                                 GWY_CORR_SEARCH_PHASE_ONLY_SCORE,
+                                 0.1, GWY_EXTERIOR_MIRROR, 0);
 
-    if (gwy_data_field_get_local_maxima_list(score, &xoff, &yoff, &maxscore, 1, 0, 0.0, FALSE)) {
+    if (gwy_field_get_local_maxima_list(score, &xoff, &yoff, &maxscore, 1, 0, 0.0, FALSE)) {
         if (xoff > 0 && xoff < (xres - 1) && yoff > 0 && yoff < (yres - 1)) {
             n = 0;
             for (j = -1; j <= 1; j++) {
                 for (i = -1; i <= 1; i++) {
-                    zvals[n++] = gwy_data_field_get_val(score, xoff+i, yoff+j);
+                    zvals[n++] = gwy_field_get_val(score, xoff+i, yoff+j);
                 }
             }
             gwy_math_refine_maximum_2d(zvals, &xsh, &ysh);
@@ -682,8 +646,8 @@ get_xy_shift(GwyDataField *mdetail, GwyDataField *ndetail, gdouble *xshift, gdou
     else
         xsh = ysh = 0;
 
-    *xshift = gwy_data_field_itor(mdetail, xsh);
-    *yshift = gwy_data_field_jtor(mdetail, ysh);
+    *xshift = gwy_field_itor(mdetail, xsh);
+    *yshift = gwy_field_jtor(mdetail, ysh);
 
     //g_object_unref(score);
     return score;
@@ -697,7 +661,7 @@ get_shifts(GwyBrick *brick, gdouble *xoff, gdouble *yoff, gdouble *xshift, gdoub
     const gint xres = gwy_brick_get_xres(brick);
     const gint yres = gwy_brick_get_yres(brick);
     const gint zres = gwy_brick_get_zres(brick);
-    GwyDataField *mfield, *nfield, *mdetail, *ndetail;
+    GwyField *mfield, *nfield, *mdetail, *ndetail;
     gdouble **xs, **ys, **zs, mh, nh, xsh, ysh;
     gdouble polyterm[15], mdx, mdy;
     gint **rs;
@@ -760,8 +724,8 @@ get_shifts(GwyBrick *brick, gdouble *xoff, gdouble *yoff, gdouble *xshift, gdoub
     /* XXX: Otherwise res is uninitialised. It seems OK (in the uncommented code – the debugging code can definitely
      * use it uninitialised), but GCC complains about it. */
 
-    mfield = gwy_data_field_new(xres, yres, gwy_brick_get_xreal(brick), gwy_brick_get_yreal(brick), FALSE);
-    nfield = gwy_data_field_new(xres, yres, gwy_brick_get_xreal(brick), gwy_brick_get_yreal(brick), FALSE);
+    mfield = gwy_field_new(xres, yres, gwy_brick_get_xreal(brick), gwy_brick_get_yreal(brick), FALSE);
+    nfield = gwy_field_new(xres, yres, gwy_brick_get_xreal(brick), gwy_brick_get_yreal(brick), FALSE);
 
     //find z shift between mth and nth plane
     for (m = 0; m < zres; m++) {
@@ -787,8 +751,8 @@ get_shifts(GwyBrick *brick, gdouble *xoff, gdouble *yoff, gdouble *xshift, gdoub
                  if (m >= n)
                      continue;
 
-                 mdata = gwy_data_field_get_data(mfield);
-                 ndata = gwy_data_field_get_data(nfield);
+                 mdata = gwy_field_get_data(mfield);
+                 ndata = gwy_field_get_data(nfield);
 
              //  m x n overlap,
              // overlap posistion in image m: col1, row1, width, height
@@ -854,15 +818,15 @@ get_shifts(GwyBrick *brick, gdouble *xoff, gdouble *yoff, gdouble *xshift, gdoub
                 }
              }
              if (doz) {
-                mh = gwy_data_field_area_get_sum_mask(mfield, NULL, GWY_MASK_IGNORE, col1, row1, width, height);
-                nh = gwy_data_field_area_get_sum_mask(nfield, NULL, GWY_MASK_IGNORE, col2, row2, width, height);
+                mh = gwy_field_area_get_sum(mfield, NULL, GWY_MASK_IGNORE, col1, row1, width, height);
+                nh = gwy_field_area_get_sum(nfield, NULL, GWY_MASK_IGNORE, col2, row2, width, height);
                 zs[m][n] = mh - nh;
                 zs[n][m] = nh - mh;
               }
 
              if (dox || doy) {
-                mdetail = gwy_data_field_area_extract(mfield, col1, row1, width, height);
-                ndetail = gwy_data_field_area_extract(nfield, col2, row2, width, height);
+                mdetail = gwy_field_area_extract(mfield, col1, row1, width, height);
+                ndetail = gwy_field_area_extract(nfield, col2, row2, width, height);
 
                 if (get_xy_shift(mdetail, ndetail, &xsh, &ysh)) {
                     xs[m][n] = xsh*width*height;
@@ -1020,7 +984,7 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
     GwyBrick *brick = args->brick;
     GwySurface *frame;
     GwyXYZ *xyz, *sxyz;
-    GwyDataField *dfield;
+    GwyField *dfield;
     gdouble *xoffset, *yoffset, *xshift, *yshift, *zshift, poly[15], mx, my, *bdata = NULL;
     gboolean xflip = gwy_params_get_boolean(params, PARAM_XOFFSETS_FLIP);
     gboolean yflip = gwy_params_get_boolean(params, PARAM_YOFFSETS_FLIP);
@@ -1052,11 +1016,11 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
         g_object_unref(args->result);
 
     args->result = gwy_surface_new_sized(xres*yres*zres);
-    gwy_surface_set_si_unit_xy(args->result, gwy_brick_get_si_unit_x(brick));
-    gwy_surface_set_si_unit_z(args->result, gwy_brick_get_si_unit_w(brick));
+    gwy_unit_assign(gwy_surface_get_unit_xy(args->result), gwy_brick_get_unit_x(brick));
+    gwy_unit_assign(gwy_surface_get_unit_z(args->result), gwy_brick_get_unit_w(brick));
     xyz = gwy_surface_get_data(args->result);
 
-    dfield = gwy_data_field_new(xres, yres, gwy_brick_get_xreal(brick), gwy_brick_get_yreal(brick), FALSE);
+    dfield = gwy_field_new(xres, yres, gwy_brick_get_xreal(brick), gwy_brick_get_yreal(brick), FALSE);
 
     for (k = 0; k < zres; k++) {
         xshift[k] = 0;
@@ -1095,12 +1059,12 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
 
     if (dobackground) {
         if (!args->background) {
-            args->background = gwy_data_field_new(xres, yres,
-                                                  gwy_brick_get_xreal(brick), gwy_brick_get_yreal(brick), FALSE);
-            gwy_data_field_set_si_unit_xy(args->background, gwy_brick_get_si_unit_x(brick));
-            gwy_data_field_set_si_unit_z(args->background, gwy_brick_get_si_unit_w(brick));
+            args->background = gwy_field_new(xres, yres,
+                                             gwy_brick_get_xreal(brick), gwy_brick_get_yreal(brick), FALSE);
+            gwy_unit_assign(gwy_field_get_unit_xy(args->background), gwy_brick_get_unit_x(brick));
+            gwy_unit_assign(gwy_field_get_unit_z(args->background), gwy_brick_get_unit_w(brick));
         }
-        bdata = gwy_data_field_get_data(args->background);
+        bdata = gwy_field_get_data(args->background);
     }
 
     if (!gwy_app_wait_set_message("Merging data..."))
@@ -1111,7 +1075,7 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
         gwy_brick_extract_xy_plane(brick, dfield, k);
 
         frame = gwy_surface_new();
-        gwy_surface_set_from_data_field(frame, dfield);
+        gwy_surface_set_from_field(frame, dfield);
         sxyz = gwy_surface_get_data(frame);
         m = 0;
 
