@@ -1,5 +1,5 @@
 /*
- *  $Id: volume_zcal.c 26510 2024-08-15 17:12:23Z yeti-dn $
+ *  $Id: volume_zcal.c 29061 2026-01-02 15:01:53Z yeti-dn $
  *  Copyright (C) 2015-2023 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -16,16 +16,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/brick.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwydgets/gwygraph.h>
-#include <libgwymodule/gwymodule-volume.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
+#include <gwy.h>
 #include "../process/preview.h"
 
 #define RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
@@ -48,8 +42,8 @@ typedef enum {
 typedef struct {
     GwyParams *params;
     GwyBrick *brick;
-    GwyDataLine *calibration;
-    GwyDataLine *filecalibration;
+    GwyLine *calibration;
+    GwyLine *filecalibration;
 } ModuleArgs;
 
 typedef struct {
@@ -63,25 +57,25 @@ typedef struct {
 
 static gboolean         module_register             (void);
 static GwyParamDef*     define_module_params        (void);
-static void             zcal                        (GwyContainer *data,
-                                                     GwyRunType run);
+static void             module_main                 (GwyFile *data,
+                                                     GwyRunModeFlags mode);
 static void             execute                     (ModuleArgs *args,
-                                                     GwyContainer *data);
+                                                     GwyFile *data);
 static GwyDialogOutcome run_gui                     (ModuleArgs *args);
 static void             param_changed               (ModuleGUI *gui,
                                                      gint id);
 static void             preview                     (gpointer user_data);
 static GtkWidget*       create_filename_chooser     (gpointer user_data);
-static gboolean         filter_source_bricks        (GwyContainer *data,
+static gboolean         filter_source_bricks        (GwyFile *data,
                                                      gint id,
                                                      gpointer user_data);
 static void             file_selected               (GtkFileChooser *chooser,
                                                      ModuleGUI *gui);
 static void             setup_graph_from_calibration(ModuleGUI *gui);
 static void             extract_graph               (GwyGraphModel *gmodel,
-                                                     GwyDataLine *calibration,
+                                                     GwyLine *calibration,
                                                      const gchar *title);
-static GwyDataLine*     load_calibration_from_file  (const gchar *filename,
+static GwyLine*         load_calibration_from_file  (const gchar *filename,
                                                      GwyBrick *brick,
                                                      gchar **errmessage);
 
@@ -101,9 +95,9 @@ static gboolean
 module_register(void)
 {
     gwy_volume_func_register("volume_zcal",
-                             (GwyVolumeFunc)&zcal,
+                             module_main,
                              N_("/_Basic Operations/_Z-Calibration..."),
-                             GWY_STOCK_VOLUME_CALIBRATE,
+                             GWY_ICON_VOLUME_CALIBRATE,
                              RUN_MODES,
                              GWY_MENU_FLAG_VOLUME,
                              N_("Manage z-axis calibration"));
@@ -137,25 +131,25 @@ define_module_params(void)
 }
 
 static void
-zcal(GwyContainer *data, GwyRunType run)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     ModuleArgs args;
-    ZCalActionType mode;
+    ZCalActionType zcal_mode;
     gboolean mode_needs_zcal;
 
-    g_return_if_fail(run & RUN_MODES);
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_BRICK, &args.brick, 0);
+    g_return_if_fail(mode & RUN_MODES);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_BRICK, &args.brick, 0);
     g_return_if_fail(GWY_IS_BRICK(args.brick));
     args.calibration = gwy_brick_get_zcalibration(args.brick);
     if (args.calibration)
         g_object_ref(args.calibration);
     args.params = gwy_params_new_from_settings(define_module_params());
-    mode = gwy_params_get_enum(args.params, PARAM_MODE);
-    mode_needs_zcal = (mode == ZCAL_REMOVE || mode == ZCAL_EXTRACT);
+    zcal_mode = gwy_params_get_enum(args.params, PARAM_MODE);
+    mode_needs_zcal = (zcal_mode == ZCAL_REMOVE || zcal_mode == ZCAL_EXTRACT);
 
-    if (run == GWY_RUN_INTERACTIVE) {
+    if (mode == GWY_RUN_INTERACTIVE) {
         if (mode_needs_zcal && !args.calibration)
             gwy_params_set_enum(args.params, PARAM_MODE, ZCAL_ATTACH);
         outcome = run_gui(&args);
@@ -174,8 +168,8 @@ zcal(GwyContainer *data, GwyRunType run)
     execute(&args, data);
 
 fail:
-    GWY_OBJECT_UNREF(args.calibration);
-    GWY_OBJECT_UNREF(args.filecalibration);
+    g_clear_object(&args.calibration);
+    g_clear_object(&args.filecalibration);
     g_object_unref(args.params);
 }
 
@@ -189,7 +183,7 @@ run_gui(ModuleArgs *args)
     GwyDialogOutcome outcome;
     ModuleGUI gui;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
     gui.has_source = TRUE;
 
@@ -197,7 +191,7 @@ run_gui(ModuleArgs *args)
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    hbox = gwy_hbox_new(20);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 20);
     gwy_dialog_add_content(dialog, hbox, TRUE, TRUE, 0);
 
     table = gui.table = gwy_param_table_new(args->params);
@@ -268,7 +262,7 @@ param_changed(ModuleGUI *gui, gint id)
         gwy_param_table_set_sensitive(gui->table, PARAM_SOURCE, is_source);
     }
     if (id < 0 || id == PARAM_FILENAME) {
-        GWY_OBJECT_UNREF(args->filecalibration);
+        g_clear_object(&args->filecalibration);
         args->filecalibration = load_calibration_from_file(gwy_params_get_string(params, PARAM_FILENAME),
                                                            args->brick, &errmessage);
         gwy_param_table_set_label(gui->table, MESSAGE_ERROR, errmessage);
@@ -281,10 +275,10 @@ param_changed(ModuleGUI *gui, gint id)
 }
 
 static gboolean
-filter_source_bricks(GwyContainer *data, gint id, gpointer user_data)
+filter_source_bricks(GwyFile *data, gint id, gpointer user_data)
 {
     GwyBrick *brick = GWY_BRICK(user_data);
-    GwyBrick *source = gwy_container_get_object(data, gwy_app_get_brick_key_for_id(id));
+    GwyBrick *source = gwy_file_get_volume(data, id);
 
     if (source == brick)
         return FALSE;
@@ -311,7 +305,7 @@ preview(gpointer user_data)
     ZCalActionType mode = gwy_params_get_enum(args->params, PARAM_MODE);
     GwyBrick *source = gwy_params_get_volume(args->params, PARAM_SOURCE);
 
-    GWY_OBJECT_UNREF(args->calibration);
+    g_clear_object(&args->calibration);
 
     if (mode == ZCAL_ATTACH)
         args->calibration = args->filecalibration;
@@ -341,7 +335,7 @@ setup_graph_from_calibration(ModuleGUI *gui)
 
 static void
 extract_graph(GwyGraphModel *gmodel,
-              GwyDataLine *calibration,
+              GwyLine *calibration,
               const gchar *title)
 {
     GwyGraphCurveModel *gcmodel;
@@ -365,8 +359,8 @@ extract_graph(GwyGraphModel *gmodel,
 
     g_object_set(gcmodel, "description", title, NULL);
 
-    res = gwy_data_line_get_res(calibration);
-    ydata = gwy_data_line_get_data_const(calibration);
+    res = gwy_line_get_res(calibration);
+    ydata = gwy_line_get_data_const(calibration);
     xdata = gwy_math_linspace(NULL, res, 0, 1);
     gwy_graph_curve_model_set_data(gcmodel, xdata, ydata, res);
     g_free(xdata);
@@ -374,13 +368,13 @@ extract_graph(GwyGraphModel *gmodel,
     g_object_set(gmodel,
                  "axis-label-bottom", _("Index"),
                  "axis-label-left", _("Z axis value"),
-                 "si-unit-y", gwy_data_line_get_si_unit_y(calibration),
+                 "unit-y", gwy_line_get_unit_y(calibration),
                  "title", title,
                  NULL);
 }
 
 static void
-execute(ModuleArgs *args, GwyContainer *data)
+execute(ModuleArgs *args, GwyFile *data)
 {
     ZCalActionType mode = gwy_params_get_enum(args->params, PARAM_MODE);
     const gchar *filename = gwy_params_get_string(args->params, PARAM_FILENAME);
@@ -389,7 +383,7 @@ execute(ModuleArgs *args, GwyContainer *data)
     if (mode == ZCAL_ATTACH) {
         if (filename) {
             gchar *errmessage = NULL;
-            GwyDataLine *dline;
+            GwyLine *dline;
 
             dline = load_calibration_from_file(filename, args->brick, &errmessage);
             if (dline) {
@@ -406,18 +400,19 @@ execute(ModuleArgs *args, GwyContainer *data)
         gwy_brick_data_changed(args->brick);
     }
     else if (mode == ZCAL_EXTRACT) {
-        GwyDataLine *calibration = gwy_brick_get_zcalibration(args->brick);
+        GwyLine *calibration = gwy_brick_get_zcalibration(args->brick);
 
         if (calibration) {
             GwyGraphModel *gmodel = gwy_graph_model_new();
             extract_graph(gmodel, calibration, _("Z-calibration curve"));
-            gwy_app_data_browser_add_graph_model(gmodel, data, TRUE);
+            gint newid = gwy_file_add_graph(data, gmodel);
+            gwy_file_set_visible(data, GWY_FILE_GRAPH, newid, TRUE);
             g_object_unref(gmodel);
         }
     }
     else if (mode == ZCAL_ANOTHER) {
         if (source) {
-            GwyDataLine *calibration = gwy_data_line_duplicate(gwy_brick_get_zcalibration(source));
+            GwyLine *calibration = gwy_line_copy(gwy_brick_get_zcalibration(source));
             gwy_brick_set_zcalibration(args->brick, calibration);
             gwy_brick_data_changed(args->brick);
             g_object_unref(calibration);
@@ -428,10 +423,10 @@ execute(ModuleArgs *args, GwyContainer *data)
     }
 }
 
-static GwyDataLine*
+static GwyLine*
 load_calibration_from_file(const gchar *filename, GwyBrick *brick, gchar **errmessage)
 {
-    GwyDataLine *dline, *calibration;
+    GwyLine *dline, *calibration;
     GError *err = NULL;
     gdouble *data = NULL;
     gint nrows = -1, ncols = 1;
@@ -447,7 +442,7 @@ load_calibration_from_file(const gchar *filename, GwyBrick *brick, gchar **errme
         return NULL;
     }
 
-    /* We could parse directly to a dataline, but that would not lead to an error when there are too many values. */
+    /* We could parse directly to a line, but that would not lead to an error when there are too many values. */
     data = gwy_parse_doubles(buffer, NULL, 0, &nrows, &ncols, NULL, &err);
     gwy_file_abandon_contents(buffer, size, NULL);
     if (!data) {
@@ -463,16 +458,16 @@ load_calibration_from_file(const gchar *filename, GwyBrick *brick, gchar **errme
         return NULL;
     }
 
-    gwy_math_sort(nrows, data);
-    dline = gwy_data_line_new(brick->zres, brick->zreal, FALSE);
-    gwy_assign(dline->data, data, nrows);
+    gwy_math_sort(data, nrows);
+    dline = gwy_line_new(brick->zres, brick->zreal, FALSE);
+    gwy_assign(gwy_line_get_data(dline), data, nrows);
     g_free(data);
 
-    gwy_si_unit_assign(gwy_data_line_get_si_unit_x(dline), gwy_brick_get_si_unit_z(brick));
+    gwy_unit_assign(gwy_line_get_unit_x(dline), gwy_brick_get_unit_z(brick));
     /* Set the value unit either to the unit of existing calibration or again the same as the z-unit of the brick. */
     calibration = gwy_brick_get_zcalibration(brick);
-    gwy_si_unit_assign(gwy_data_line_get_si_unit_y(dline),
-                       (calibration ? gwy_data_line_get_si_unit_y(calibration) : gwy_brick_get_si_unit_z(brick)));
+    gwy_unit_assign(gwy_line_get_unit_y(dline),
+                       (calibration ? gwy_line_get_unit_y(calibration) : gwy_brick_get_unit_z(brick)));
 
     return dline;
 }
