@@ -1,5 +1,5 @@
 /*
- *  $Id: volume_zposlevel.c 26744 2024-10-18 15:03:29Z yeti-dn $
+ *  $Id: volume_zposlevel.c 29023 2025-12-19 13:06:16Z yeti-dn $
  *  Copyright (C) 2019-2023 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -16,17 +16,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/arithmetic.h>
-#include <libprocess/linestats.h>
-#include <libprocess/brick.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-volume.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
+#include <gwy.h>
 
 #define RUN_MODES (GWY_RUN_INTERACTIVE)
 
@@ -54,13 +47,12 @@ typedef struct {
     GwyParams *params;
     GwyBrick *brick;
     GwyBrick *result;
-    GwyDataLine *calibration;
+    GwyLine *calibration;
 } ModuleArgs;
 
 typedef struct {
     ModuleArgs *args;
-    GwyContainer *data;
-    GwyDataField *xyplane;
+    GwyField *xyplane;
     GtkWidget *dialog;
     GwyGraphModel *gmodel;
     GtkWidget *dataview;
@@ -68,16 +60,16 @@ typedef struct {
     GwyParamTable *table_output;
     GwySelection *image_selection;
     GwySelection *graph_selection;
-    GwySIValueFormat *vf;
+    GwyValueFormat *vf;
 } ModuleGUI;
 
 static gboolean         module_register        (void);
 static GwyParamDef*     define_module_params   (void);
-static void             zposlevel              (GwyContainer *data,
-                                                GwyRunType run);
+static void             module_main            (GwyFile *data,
+                                                GwyRunModeFlags mode);
 static void             execute                (ModuleArgs *args);
 static GwyDialogOutcome run_gui                (ModuleArgs *args,
-                                                GwyContainer *data,
+                                                GwyFile *data,
                                                 gint id);
 static void             param_changed          (ModuleGUI *gui,
                                                 gint id);
@@ -115,7 +107,7 @@ static gboolean
 module_register(void)
 {
     gwy_volume_func_register("volume_zposlevel",
-                             (GwyVolumeFunc)&zposlevel,
+                             module_main,
                              N_("/_Correct Data/S_hift Value To Zero..."),
                              NULL,
                              RUN_MODES,
@@ -143,34 +135,32 @@ define_module_params(void)
     gwy_param_def_add_int(paramdef, PARAM_YPOS, "ypos", _("_Y"), -1, G_MAXINT, -1);
     gwy_param_def_add_int(paramdef, PARAM_ZPOS, "zpos", _("_Z value"), -1, G_MAXINT, -1);
     gwy_param_def_add_int(paramdef, PARAM_RANGE, "range", _("_Z range"), 1, RANGE_MAX, 1);
-    gwy_param_def_add_gwyenum(paramdef, PARAM_SHOW_TYPE, "show_type", gwy_sgettext("verb|_Display"),
+    gwy_param_def_add_gwyenum(paramdef, PARAM_SHOW_TYPE, "show_type", C_("verb", "_Display"),
                               displays, G_N_ELEMENTS(displays), SHOW_DATA);
     return paramdef;
 }
 
 static void
-zposlevel(GwyContainer *data, GwyRunType run)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     ModuleArgs args;
     GwyDialogOutcome outcome;
-    gchar *title;
     gint id, newid;
 
-    g_return_if_fail(run & RUN_MODES);
-    g_return_if_fail(g_type_from_name("GwyLayerPoint"));
+    g_return_if_fail(mode & RUN_MODES);
 
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_BRICK, &args.brick,
-                                     GWY_APP_BRICK_ID, &id,
-                                     0);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_BRICK, &args.brick,
+                                 GWY_APP_BRICK_ID, &id,
+                                 0);
     g_return_if_fail(GWY_IS_BRICK(args.brick));
 
     args.calibration = gwy_brick_get_zcalibration(args.brick);
-    if (args.calibration && (gwy_brick_get_zres(args.brick) != gwy_data_line_get_res(args.calibration)))
+    if (args.calibration && (gwy_brick_get_zres(args.brick) != gwy_line_get_res(args.calibration)))
         args.calibration = NULL;
     args.params = gwy_params_new_from_settings(define_module_params());
     sanitise_params(&args);
-    args.result = gwy_brick_duplicate(args.brick);
+    args.result = gwy_brick_copy(args.brick);
 
     outcome = run_gui(&args, data, id);
     gwy_params_save_to_settings(args.params);
@@ -180,13 +170,16 @@ zposlevel(GwyContainer *data, GwyRunType run)
     if (outcome != GWY_DIALOG_HAVE_RESULT)
         execute(&args);
 
-    newid = gwy_app_data_browser_add_brick(args.result, NULL, data, TRUE);
-    title = g_strdup_printf(_("Shifted to zero for z level = %d"), gwy_params_get_int(args.params, PARAM_ZPOS));
-    gwy_container_set_string(data, gwy_app_get_brick_title_key_for_id(newid), title);
-    gwy_app_sync_volume_items(data, data, id, newid, FALSE,
-                              GWY_DATA_ITEM_GRADIENT,
-                              0);
-    gwy_app_volume_log_add_volume(data, id, newid);
+    newid = gwy_file_add_volume(data, args.result, NULL);
+
+    gwy_file_set_visible(data, GWY_FILE_VOLUME, newid, TRUE);
+    gwy_file_pass_title(data, GWY_FILE_VOLUME, newid,
+                        g_strdup_printf(_("Shifted to zero for z level = %d"),
+                                        gwy_params_get_int(args.params, PARAM_ZPOS)));
+    gwy_file_sync_items(data, GWY_FILE_VOLUME, id,
+                        data, GWY_FILE_VOLUME, newid,
+                        GWY_FILE_ITEM_PALETTE, FALSE);
+    gwy_log_add(data, GWY_FILE_VOLUME, id, newid);
 
 end:
     g_object_unref(args.result);
@@ -194,33 +187,27 @@ end:
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     GwyBrick *brick = args->brick;
     gint zres = gwy_brick_get_zres(brick);
-    GtkWidget *hbox, *align;
+    GtkWidget *hbox;
     GwyGraphArea *area;
     GwyGraph *graph;
     GwyParamTable *table;
     GwyDialog *dialog;
     ModuleGUI gui;
     GwyDialogOutcome outcome;
-    const guchar *gradient;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
 
-    gui.data = gwy_container_new();
-    gui.xyplane = gwy_data_field_new(1, 1, 1.0, 1.0, FALSE);
+    gui.xyplane = gwy_field_new(1, 1, 1.0, 1.0, FALSE);
     extract_xyplane(&gui);
-    gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(0), gui.xyplane);
 
-    if (gwy_container_gis_string(data, gwy_app_get_brick_palette_key_for_id(id), &gradient))
-        gwy_container_set_const_string(gui.data, gwy_app_get_data_palette_key_for_id(0), gradient);
-
-    gui.vf = gwy_si_unit_get_format_with_digits(gwy_brick_get_si_unit_w(brick), GWY_SI_UNIT_FORMAT_VFMARKUP,
-                                                gwy_brick_get_max(brick) - gwy_brick_get_min(brick), 5, /* 5 digits */
-                                                NULL);
+    gui.vf = gwy_unit_get_format_with_digits(gwy_brick_get_unit_w(brick), GWY_UNIT_FORMAT_VFMARKUP,
+                                             gwy_brick_get_max(brick) - gwy_brick_get_min(brick), 5, /* 5 digits */
+                                             NULL);
 
     gui.gmodel = gwy_graph_model_new();
     setup_gmodel(args, gui.gmodel);
@@ -229,15 +216,16 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_UPDATE, GWY_RESPONSE_RESET, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
-    hbox = gwy_hbox_new(0);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
     gwy_dialog_add_content(dialog, hbox, FALSE, FALSE, 4);
 
-    align = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-    gtk_box_pack_start(GTK_BOX(hbox), align, FALSE, FALSE, 0);
-
-    gui.dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
-    gtk_container_add(GTK_CONTAINER(align), gui.dataview);
-    gui.image_selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(gui.dataview), 0, "Point", 1, TRUE);
+    gui.dataview = gwy_create_preview(gui.xyplane, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(gui.dataview), data, GWY_FILE_VOLUME, id, GWY_FILE_ITEM_PALETTE);
+    gtk_widget_set_halign(gui.dataview, GTK_ALIGN_START);
+    gtk_widget_set_valign(gui.dataview, GTK_ALIGN_START);
+    gtk_box_pack_start(GTK_BOX(hbox), gui.dataview, FALSE, FALSE, 0);
+    gui.image_selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(gui.dataview),
+                                                          GWY_TYPE_LAYER_POINT, 1, TRUE);
 
     graph = GWY_GRAPH(gwy_graph_new(gui.gmodel));
     gwy_graph_enable_user_input(graph, FALSE);
@@ -249,7 +237,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gui.graph_selection = gwy_graph_area_get_selection(area, GWY_GRAPH_STATUS_XLINES);
     gwy_selection_set_max_objects(gui.graph_selection, 1);
 
-    hbox = gwy_hbox_new(24);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 24);
     gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), hbox, TRUE, TRUE, 4);
 
     gui.table_value = table = gwy_param_table_new(args->params);
@@ -284,8 +272,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 
     g_object_unref(gui.gmodel);
     g_object_unref(gui.xyplane);
-    g_object_unref(gui.data);
-    gwy_si_unit_value_format_free(gui.vf);
+    gwy_value_format_free(gui.vf);
 
     return outcome;
 }
@@ -332,7 +319,7 @@ static void
 update_constant_value(ModuleGUI *gui)
 {
     ModuleArgs *args = gui->args;
-    GwySIValueFormat *vf = gui->vf;
+    GwyValueFormat *vf = gui->vf;
     gdouble const_value = get_constant_value(args);
     gchar *s;
 
@@ -356,7 +343,7 @@ param_changed(ModuleGUI *gui, gint id)
     }
     if (id < 0 || id == PARAM_ZPOS || id == PARAM_SHOW_TYPE) {
         extract_xyplane(gui);
-        gwy_data_field_data_changed(gui->xyplane);
+        gwy_field_data_changed(gui->xyplane);
     }
     if (id < 0 || id == PARAM_XPOS || id == PARAM_YPOS)
         extract_graph_curve(args, gwy_graph_model_get_curve(gui->gmodel, 0));
@@ -388,34 +375,34 @@ preview(gpointer user_data)
         gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
     }
     extract_xyplane(gui);
-    gwy_data_field_data_changed(gui->xyplane);
+    gwy_field_data_changed(gui->xyplane);
     extract_graph_curve(args, gwy_graph_model_get_curve(gui->gmodel, 0));
 }
 
 static void
 execute(ModuleArgs *args)
 {
-    GwyDataField *shifts, *xyplane = NULL;
+    GwyField *shifts, *xyplane = NULL;
     GwyBrick *brick = args->result;
     gint zres = gwy_brick_get_zres(brick);
     gint zpos = gwy_params_get_int(args->params, PARAM_ZPOS);
     gint range = gwy_params_get_int(args->params, PARAM_RANGE);
     gint k, zfrom = MAX(zpos - (range - range/2), 0), zto = MIN(zpos + range/2, zres);
 
-    gwy_brick_copy(args->brick, args->result, FALSE);
-    shifts = gwy_data_field_new(1, 1, 1.0, 1.0, FALSE);
+    gwy_brick_copy_data(args->brick, args->result);
+    shifts = gwy_field_new(1, 1, 1.0, 1.0, FALSE);
     /* Unfortunately, gwy_brick_mean_plane() does not do what we need. It cannot limit the range along the axis it is
      * averaging. */
     gwy_brick_extract_xy_plane(brick, shifts, zfrom);
     for (k = zfrom+1; k < zto; k++) {
         if (!xyplane)
-            xyplane = gwy_data_field_new_alike(shifts, FALSE);
+            xyplane = gwy_field_new_alike(shifts, FALSE);
         gwy_brick_extract_xy_plane(brick, xyplane, zfrom);
-        gwy_data_field_sum_fields(shifts, shifts, xyplane);
+        gwy_field_sum_fields(shifts, shifts, xyplane);
     }
-    gwy_data_field_multiply(shifts, -1.0/(zto - zfrom));
+    gwy_field_multiply(shifts, -1.0/(zto - zfrom));
     gwy_brick_add_to_xy_planes(brick, shifts);
-    GWY_OBJECT_UNREF(xyplane);
+    g_clear_object(&xyplane);
     g_object_unref(shifts);
 }
 
@@ -427,20 +414,20 @@ extract_graph_curve(ModuleArgs *args, GwyGraphCurveModel *gcmodel)
     gint zres = gwy_brick_get_zres(brick);
     gint xpos = gwy_params_get_int(args->params, PARAM_XPOS);
     gint ypos = gwy_params_get_int(args->params, PARAM_YPOS);
-    GwyDataLine *line, *calibration = args->calibration;
+    GwyLine *line, *calibration = args->calibration;
 
-    line = gwy_data_line_new(1, 1.0, FALSE);
+    line = gwy_line_new(1, 1.0, FALSE);
     gwy_brick_extract_line(brick, line, xpos, ypos, 0, xpos, ypos, zres, FALSE);
-    gwy_data_line_set_offset(line, gwy_brick_get_zoffset(brick));
+    gwy_line_set_offset(line, gwy_brick_get_zoffset(brick));
 
     if (calibration) {
         gwy_graph_curve_model_set_data(gcmodel,
-                                       gwy_data_line_get_data(calibration), gwy_data_line_get_data(line),
-                                       gwy_data_line_get_res(line));
+                                       gwy_line_get_data(calibration), gwy_line_get_data(line),
+                                       gwy_line_get_res(line));
         gwy_graph_curve_model_enforce_order(gcmodel);
     }
     else
-        gwy_graph_curve_model_set_data_from_dataline(gcmodel, line, 0, 0);
+        gwy_graph_curve_model_set_data_from_line(gcmodel, line, 0, 0);
 
     g_object_unref(line);
 }
@@ -455,15 +442,15 @@ get_constant_value(ModuleArgs *args)
     gint zpos = gwy_params_get_int(args->params, PARAM_ZPOS);
     gint range = gwy_params_get_int(args->params, PARAM_RANGE);
     gint zfrom = MAX(zpos - (range - range/2), 0), zto = MIN(zpos + range/2, zres);
-    GwyDataLine *line;
+    GwyLine *line;
     gdouble avg;
 
     if (zfrom+1 >= zto)
         return gwy_brick_get_val(brick, xpos, ypos, zfrom);
 
-    line = gwy_data_line_new(1, 1.0, FALSE);
+    line = gwy_line_new(1, 1.0, FALSE);
     gwy_brick_extract_line(brick, line, xpos, ypos, zfrom, xpos, ypos, zto, FALSE);
-    avg = gwy_data_line_get_avg(line);
+    avg = gwy_line_avg(line);
     g_object_unref(line);
 
     return avg;
@@ -473,14 +460,14 @@ static void
 setup_gmodel(ModuleArgs *args, GwyGraphModel *gmodel)
 {
     GwyBrick *brick = args->brick;
-    GwyDataLine *calibration = args->calibration;
-    GwySIUnit *xunit = (calibration ? gwy_data_line_get_si_unit_y(calibration) : gwy_brick_get_si_unit_z(brick));
+    GwyLine *calibration = args->calibration;
+    GwyUnit *xunit = (calibration ? gwy_line_get_unit_y(calibration) : gwy_brick_get_unit_z(brick));
     GwyGraphCurveModel *gcmodel;
 
     g_object_set(gmodel,
                  "label-visible", FALSE,
-                 "si-unit-x", xunit,
-                 "si-unit-y", gwy_brick_get_si_unit_w(brick),
+                 "unit-x", xunit,
+                 "unit-y", gwy_brick_get_unit_w(brick),
                  "axis-label-bottom", "z",
                  "axis-label-left", "w",
                  NULL);
