@@ -1,5 +1,5 @@
 /*
- *  $Id: xyz_crop.c 28896 2025-11-20 12:14:27Z klapetek $
+ *  $Id: xyz_crop.c 28808 2025-11-05 18:26:20Z yeti-dn $
  *  Copyright (C) 2016-2023 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -15,15 +15,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/surface.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-xyz.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
+#include <gwy.h>
 
 #define RUN_MODES (GWY_RUN_INTERACTIVE)
 
@@ -44,7 +39,7 @@ typedef enum {
 typedef struct {
     GwyParams *params;
     GwySurface *surface;
-    GwyDataField *dfield;
+    GwyField *dfield;
     gdouble xy[4];
 } ModuleArgs;
 
@@ -52,32 +47,26 @@ typedef struct {
     ModuleArgs *args;
     GtkWidget *dialog;
     GwyParamTable *table_options;
-    GwyContainer *data;
     GwyVectorLayer *vlayer;
     GwySelection *selection;
     GwyRectSelectionLabels *rlabels;
     gdouble rsel[4];
     gint isel[4];
-
 } ModuleGUI;
 
-
-
-static gboolean         module_register           (void);
-static GwyParamDef*     define_module_params      (void);
-static GwySurface*      execute                   (ModuleArgs *args);
-static GwyDialogOutcome run_gui                   (ModuleArgs *args,
-                                                   GwyContainer *data,
-                                                   gint id);
-
-static void             param_changed             (ModuleGUI *gui,
-                                                   gint id);
-static void             xyzcrop                   (GwyContainer *data,
-                                                   GwyRunType runtype);
-static void             selection_changed         (ModuleGUI *gui);
-static void             rect_updated              (ModuleGUI *gui);
-static void             update_selected_rectangle (ModuleGUI *gui);
-
+static gboolean         module_register          (void);
+static GwyParamDef*     define_module_params     (void);
+static GwySurface*      execute                  (ModuleArgs *args);
+static GwyDialogOutcome run_gui                  (ModuleArgs *args,
+                                                  GwyFile *data,
+                                                  gint id);
+static void             param_changed            (ModuleGUI *gui,
+                                                  gint id);
+static void             module_main              (GwyFile *data,
+                                                  GwyRunModeFlags mode);
+static void             selection_changed        (ModuleGUI *gui);
+static void             rect_updated             (ModuleGUI *gui);
+static void             update_selected_rectangle(ModuleGUI *gui);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
@@ -95,7 +84,7 @@ static gboolean
 module_register(void)
 {
     gwy_xyz_func_register("xyz_crop",
-                          (GwyXYZFunc)&xyzcrop,
+                          module_main,
                           N_("/Crop..."),
                           NULL,
                           RUN_MODES,
@@ -121,29 +110,28 @@ define_module_params(void)
 }
 
 static void
-xyzcrop(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     ModuleArgs args;
     GwySurface *result = NULL;
     gint id, newid;
-    const guchar *gradient;
 
-    g_return_if_fail(runtype & RUN_MODES);
+    g_return_if_fail(mode & RUN_MODES);
 
-    gwy_app_data_browser_get_current(GWY_APP_SURFACE, &args.surface,
-                                     GWY_APP_SURFACE_ID, &id,
-                                     0);
+    gwy_data_browser_get_current(GWY_APP_SURFACE, &args.surface,
+                                 GWY_APP_SURFACE_ID, &id,
+                                 0);
     g_return_if_fail(GWY_IS_SURFACE(args.surface));
 
     args.params = gwy_params_new_from_settings(define_module_params());
-    args.dfield = gwy_data_field_new(10, 10, 10, 10, FALSE);
+    args.dfield = gwy_field_new(10, 10, 10, 10, FALSE);
 
-    gwy_preview_surface_to_datafield(args.surface, args.dfield,
-                                     PREVIEW_SIZE, PREVIEW_SIZE,
-                                     GWY_PREVIEW_SURFACE_FILL);
- 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    gwy_preview_surface_to_field(args.surface, args.dfield,
+                                 PREVIEW_SIZE, PREVIEW_SIZE,
+                                 GWY_PREVIEW_SURFACE_FILL);
+
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, id);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -154,13 +142,12 @@ xyzcrop(GwyContainer *data, GwyRunType runtype)
         result = execute(&args);
 
     if (result) {
-        newid = gwy_app_data_browser_add_surface(result, data, TRUE);
-        gwy_app_set_surface_title(data, newid, _("Cropped"));
-        if (gwy_container_gis_string(data, gwy_app_get_surface_palette_key_for_id(id), &gradient))
-            gwy_container_set_const_string(data, gwy_app_get_surface_palette_key_for_id(newid), gradient);
-
-        gwy_app_xyz_log_add(data, -1, newid, "xyz::xyz_crop", NULL);
-
+        newid = gwy_file_add_xyz(data, result);
+        gwy_file_set_visible(data, GWY_FILE_XYZ, newid, TRUE);
+        gwy_file_set_title(data, GWY_FILE_XYZ, newid, _("Cropped"), TRUE);
+        gwy_file_sync_items(data, GWY_FILE_XYZ, id,
+                            data, GWY_FILE_XYZ, newid,
+                            GWY_FILE_ITEM_PALETTE, FALSE);
         g_object_unref(result);
     }
 
@@ -169,39 +156,31 @@ end:
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     GwyDialog *dialog;
     ModuleGUI gui;
     GwyParamTable *table;
     GtkWidget *vbox, *hbox, *dataview;
     GwyDialogOutcome outcome;
-    const guchar *gradient;
 
     gui.dialog = gwy_dialog_new(_("Crop"));
     dialog = GWY_DIALOG(gui.dialog);
     gwy_dialog_add_buttons(dialog, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
     gui.args = args;
-    gui.data = gwy_container_new();
-    gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(0), args->dfield);
 
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
-    gui.selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(dataview),
-                                                    0, "Rectangle", 1, TRUE);
+    dataview = gwy_create_preview(args->dfield, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_XYZ, id, GWY_FILE_ITEM_PALETTE);
+    gui.selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(dataview), GWY_TYPE_LAYER_RECTANGLE, 1, TRUE);
     g_object_ref(gui.selection);
-    gui.vlayer = gwy_data_view_get_top_layer(GWY_DATA_VIEW(dataview));
+    gui.vlayer = gwy_data_view_get_interactive_layer(GWY_DATA_VIEW(dataview));
     g_object_ref(gui.vlayer);
-    g_signal_connect_swapped(gui.selection, "changed",
-                             G_CALLBACK(selection_changed), &gui);
-
-    if (gwy_container_gis_string(data, gwy_app_get_surface_palette_key_for_id(id), &gradient))
-       gwy_container_set_const_string(gui.data, gwy_app_get_data_palette_key_for_id(0), gradient);
-
+    g_signal_connect_swapped(gui.selection, "changed", G_CALLBACK(selection_changed), &gui);
 
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), FALSE);
 
-    vbox = gwy_vbox_new(0);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
     gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);
 
     gui.rlabels = gwy_rect_selection_labels_new(TRUE, G_CALLBACK(rect_updated), &gui);
@@ -217,7 +196,8 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     g_signal_connect_swapped(gui.table_options, "param-changed", G_CALLBACK(param_changed), &gui);
 
     outcome = gwy_dialog_run(dialog);
-    g_object_unref(gui.data);
+    g_object_unref(gui.vlayer);
+    g_object_unref(gui.selection);
 
     return outcome;
 }
@@ -230,7 +210,8 @@ selection_changed(ModuleGUI *gui)
 
     if (!gwy_selection_get_data(gui->selection, NULL)) {
         gtk_dialog_set_response_sensitive(GTK_DIALOG(gui->dialog), GTK_RESPONSE_OK, FALSE);
-    } else {
+    }
+    else {
         gtk_dialog_set_response_sensitive(GTK_DIALOG(gui->dialog), GTK_RESPONSE_OK, TRUE);
         gwy_selection_get_object(gui->selection, 0, xy);
         for (i = 0; i < 4; i++)
@@ -272,8 +253,8 @@ execute(ModuleArgs *args)
     if ((args->xy[2] - args->xy[0]) == 0 || (args->xy[3] - args->xy[1]) == 0)
         return NULL;
 
-    dfxoff = gwy_data_field_get_xoffset(args->dfield);
-    dfyoff = gwy_data_field_get_yoffset(args->dfield);
+    dfxoff = gwy_field_get_xoffset(args->dfield);
+    dfyoff = gwy_field_get_yoffset(args->dfield);
 
     xfrom = args->xy[0] + dfxoff;
     yfrom = args->xy[1] + dfyoff;
