@@ -1,5 +1,5 @@
 /*
- *  $Id: xyz_fft.c 28895 2025-11-20 12:13:29Z klapetek $
+ *  $Id: xyz_fft.c 28990 2025-12-12 17:49:45Z yeti-dn $
  *  Copyright (C) 2016-2023 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -15,16 +15,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/surface.h>
-#include <libprocess/gwyprocess.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-xyz.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
+#include <gwy.h>
 
 #define RUN_MODES (GWY_RUN_INTERACTIVE)
 
@@ -68,13 +62,12 @@ typedef enum {
     ZOOM_1   = 2,
 } FFTZoomType;
 
-
 typedef struct {
     GwyParams *params;
     GwySurface *surface;
     GwySurface *result;
     GwySelection *graph_selection;
-    GwyDataLine *fftinput;
+    GwyLine *fftinput;
 } ModuleArgs;
 
 typedef struct {
@@ -83,18 +76,19 @@ typedef struct {
     GwyParamTable *table_options;
     GwyParamTable *table_axis;
     GwyParamTable *table_view;
-    GwyContainer *data;
     GwyGraphModel *gmodel;
-    GwyDataField *dfield;
-    GwyDataLine *spectrum;
+    GwyField *dfield;
+    GwyLine *spectrum;
 } ModuleGUI;
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static gboolean         execute             (ModuleArgs *args,
                                              GtkWindow *wait_window);
 static GwyDialogOutcome run_gui             (ModuleArgs *args,
-                                             GwyContainer *data,
+                                             GwyFile *data,
                                              gint id);
 static void             preview             (gpointer user_data);
 static void             dialog_response     (GwyDialog *dialog,
@@ -104,10 +98,8 @@ static void             param_changed       (ModuleGUI *gui,
                                              gint id);
 static void             calculate_fft_curve (ModuleGUI *gui);
 static void             update_fft_curve    (ModuleGUI *gui);
-static void             xyz_fft             (GwyContainer *data,
-                                             GwyRunType runtype);
-static void             calculate_weights   (GwyDataLine *weights,
-                                             GwyDataLine *modulus,
+static void             calculate_weights   (GwyLine *weights,
+                                             GwyLine *modulus,
                                              ModuleArgs *args);
 
 static GwyModuleInfo module_info = {
@@ -126,7 +118,7 @@ static gboolean
 module_register(void)
 {
     gwy_xyz_func_register("xyz_fft",
-                          (GwyXYZFunc)&xyz_fft,
+                          &module_main,
                           N_("/_FFT Filtering..."),
                           NULL,
                           RUN_MODES,
@@ -140,8 +132,8 @@ static GwyParamDef*
 define_module_params(void)
 {
     static const GwyEnum outputs[] = {
-        { N_("Marked"),    OUTPUT_MARKED,    },
-        { N_("Unmarked"),  OUTPUT_UNMARKED,  },
+        { N_("Marked"),    OUTPUT_MARKED,   },
+        { N_("Unmarked"),  OUTPUT_UNMARKED, },
     };
     static const GwyEnum suppresses[] = {
         { N_("Null"),      SUPPRESS_NULL,         },
@@ -178,36 +170,33 @@ define_module_params(void)
 }
 
 static void
-xyz_fft(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     ModuleArgs args;
     gint id, newid;
-    const guchar *gradient;
 
-    g_return_if_fail(runtype & RUN_MODES);
+    g_return_if_fail(mode & RUN_MODES);
 
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_SURFACE, &args.surface,
-                                     GWY_APP_SURFACE_ID, &id,
-                                     0);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_SURFACE, &args.surface,
+                                 GWY_APP_SURFACE_ID, &id,
+                                 0);
     g_return_if_fail(GWY_IS_SURFACE(args.surface));
 
     args.params = gwy_params_new_from_settings(define_module_params());
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, id);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
             goto end;
     }
 
-    newid = gwy_app_data_browser_add_surface(args.result, data, TRUE);
-    gwy_app_set_surface_title(data, newid, _("FFT corrected"));
-    if (gwy_container_gis_string(data, gwy_app_get_surface_palette_key_for_id(id), &gradient))
-        gwy_container_set_const_string(data, gwy_app_get_surface_palette_key_for_id(newid), gradient);
+    newid = gwy_file_add_xyz(data, args.result);
 
-    gwy_app_xyz_log_add(data, -1, newid, "xyz::xyz_fft", NULL);
+    gwy_file_set_visible(data, GWY_FILE_XYZ, newid, TRUE);
+    gwy_file_set_title(data, GWY_FILE_XYZ, newid, _("FFT corrected"), TRUE);
 
     g_object_unref(args.result);
 
@@ -219,7 +208,7 @@ end:
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     GwyDialog *dialog;
     ModuleGUI gui;
@@ -227,7 +216,6 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gint n;
     GtkWidget *graph, *area, *hbox, *dataview;
     GwyGraphCurveModel *gcmodel;
-    const guchar *gradient;
     GwyDialogOutcome outcome;
 
     gui.dialog = gwy_dialog_new(_("FFT filtering"));
@@ -235,19 +223,14 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_RESET, GWY_RESPONSE_UPDATE, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
     gui.args = args;
-    gui.data = gwy_container_new();
-    gui.dfield = gwy_data_field_new(10, 10, 10, 10, FALSE);
+    gui.dfield = gwy_field_new(10, 10, 10, 10, FALSE);
     n = gwy_surface_get_npoints(args->surface);
-    args->fftinput = gwy_data_line_new(n, n, FALSE);
-    gui.spectrum = gwy_data_line_new(MAXN, n/2 + 1, FALSE);
+    args->fftinput = gwy_line_new(n, n, FALSE);
+    gui.spectrum = gwy_line_new(MAXN, n/2 + 1, FALSE);
 
-    gwy_preview_surface_to_datafield(args->surface, gui.dfield, PREVIEW_SIZE, PREVIEW_SIZE, GWY_PREVIEW_SURFACE_FILL);
-    gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(0), gui.dfield);
-
-    if (gwy_container_gis_string(data, gwy_app_get_surface_palette_key_for_id(id), &gradient))
-       gwy_container_set_const_string(gui.data, gwy_app_get_data_palette_key_for_id(0), gradient);
-
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
+    gwy_preview_surface_to_field(args->surface, gui.dfield, PREVIEW_SIZE, PREVIEW_SIZE, GWY_PREVIEW_SURFACE_FILL);
+    dataview = gwy_create_preview(gui.dfield, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_XYZ, id, GWY_FILE_ITEM_PALETTE);
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), FALSE);
 
     gui.gmodel = gwy_graph_model_new();
@@ -267,7 +250,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     args->graph_selection = gwy_graph_area_get_selection(GWY_GRAPH_AREA(area), GWY_GRAPH_STATUS_XSEL);
     gwy_selection_set_max_objects(args->graph_selection, 20);
 
-    hbox = gwy_hbox_new(10);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 10);
     gwy_dialog_add_content(dialog, hbox, FALSE, FALSE, 4);
 
     gui.table_axis = table = gwy_param_table_new(args->params);
@@ -301,7 +284,6 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     outcome = gwy_dialog_run(dialog);
 
     g_object_unref(gui.dfield);
-    g_object_unref(gui.data);
 
     return outcome;
 }
@@ -314,10 +296,10 @@ preview(gpointer user_data)
     if (!execute(gui->args, GTK_WINDOW(gui->dialog)))
         return;
 
-    gwy_preview_surface_to_datafield(gui->args->result, gui->dfield, PREVIEW_SIZE, PREVIEW_SIZE,
-                                     GWY_PREVIEW_SURFACE_FILL);
+    gwy_preview_surface_to_field(gui->args->result, gui->dfield, PREVIEW_SIZE, PREVIEW_SIZE,
+                                 GWY_PREVIEW_SURFACE_FILL);
 
-    gwy_data_field_data_changed(gui->dfield);
+    gwy_field_data_changed(gui->dfield);
     gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
     gtk_dialog_set_response_sensitive(GTK_DIALOG(gui->dialog), GTK_RESPONSE_OK, TRUE);
 }
@@ -327,10 +309,10 @@ dialog_response(GwyDialog *dialog, gint response, ModuleGUI *gui)
 {
     if (response == GWY_RESPONSE_RESET) {
         gwy_selection_clear(gui->args->graph_selection);
-        gwy_preview_surface_to_datafield(gui->args->surface, gui->dfield, PREVIEW_SIZE, PREVIEW_SIZE,
-                                         GWY_PREVIEW_SURFACE_FILL);
+        gwy_preview_surface_to_field(gui->args->surface, gui->dfield, PREVIEW_SIZE, PREVIEW_SIZE,
+                                     GWY_PREVIEW_SURFACE_FILL);
 
-        gwy_data_field_data_changed(gui->dfield);
+        gwy_field_data_changed(gui->dfield);
         gwy_dialog_invalidate(GWY_DIALOG(dialog));
         gtk_dialog_set_response_sensitive(GTK_DIALOG(dialog), GTK_RESPONSE_OK, FALSE);
     }
@@ -358,10 +340,10 @@ calculate_fft_curve(ModuleGUI *gui)
 {
     GwySurface *surface = gui->args->surface;
     const GwyXYZ *xyz;
-    GwyDataLine *rfft, *ifft;
+    GwyLine *rfft, *ifft;
     gdouble *linedata, *spectrumdata, *realdata, *imagdata, avg;
     gint i, k, m, n;
-    gint linen = gwy_data_line_get_res(gui->spectrum);
+    gint linen = gwy_line_get_res(gui->spectrum);
     gint avn;
     FFTDirType dir = gwy_params_get_enum(gui->args->params, PARAM_DIR);
 
@@ -370,9 +352,9 @@ calculate_fft_curve(ModuleGUI *gui)
     xyz = gwy_surface_get_data_const(surface);
     n = gwy_surface_get_npoints(surface);
 
-    rfft = gwy_data_line_new(n, n, FALSE);
-    ifft = gwy_data_line_new(n, n, FALSE);
-    linedata = gwy_data_line_get_data(gui->args->fftinput);
+    rfft = gwy_line_new(n, n, FALSE);
+    ifft = gwy_line_new(n, n, FALSE);
+    linedata = gwy_line_get_data(gui->args->fftinput);
 
     avg = 0;
     if (dir == DIR_X) {
@@ -394,13 +376,13 @@ calculate_fft_curve(ModuleGUI *gui)
        }
     }
     avg /= n;
-    gwy_data_line_add(gui->args->fftinput, -avg);
+    gwy_line_add(gui->args->fftinput, -avg);
 
-    gwy_data_line_fft_raw(gui->args->fftinput, NULL, rfft, ifft, GWY_TRANSFORM_DIRECTION_FORWARD);
-    realdata = gwy_data_line_get_data(rfft);
-    imagdata = gwy_data_line_get_data(ifft);
+    gwy_line_fft_raw(gui->args->fftinput, NULL, rfft, ifft, GWY_TRANSFORM_DIRECTION_FORWARD);
+    realdata = gwy_line_get_data(rfft);
+    imagdata = gwy_line_get_data(ifft);
 
-    spectrumdata = gwy_data_line_get_data(gui->spectrum);
+    spectrumdata = gwy_line_get_data(gui->spectrum);
     avn = n/linen/2;
 
     for (k = 0; k < linen; k++) {
@@ -411,7 +393,7 @@ calculate_fft_curve(ModuleGUI *gui)
         }
         spectrumdata[k] = avg/avn;
     }
-    gwy_data_line_data_changed(gui->spectrum);
+    gwy_line_data_changed(gui->spectrum);
 
     gwy_app_wait_finish();
     g_object_unref(rfft);
@@ -421,7 +403,7 @@ calculate_fft_curve(ModuleGUI *gui)
 static void
 update_fft_curve(ModuleGUI *gui)
 {
-    gint n = gwy_data_line_get_res(gui->spectrum);
+    gint n = gwy_line_get_res(gui->spectrum);
     FFTZoomType zoom = gwy_params_get_enum(gui->args->params, PARAM_ZOOM);
     GwyGraphCurveModel *gcmodel = gwy_graph_model_get_curve(gui->gmodel, 0);
 
@@ -430,14 +412,14 @@ update_fft_curve(ModuleGUI *gui)
     else if (zoom == ZOOM_1)
         n = n/100;
 
-    gwy_graph_curve_model_set_data_from_dataline(gcmodel, gui->spectrum, 0, n);
+    gwy_graph_curve_model_set_data_from_line(gcmodel, gui->spectrum, 0, n);
 }
 
 static gboolean
 execute(ModuleArgs *args, GtkWindow *wait_window)
 {
     GwySurface *surface = args->surface;
-    GwyDataLine *rfft = NULL, *ifft = NULL, *rresult = NULL, *iresult = NULL, *weights = NULL, *fullspectrum = NULL;
+    GwyLine *rfft = NULL, *ifft = NULL, *rresult = NULL, *iresult = NULL, *weights = NULL, *fullspectrum = NULL;
     GwySurface *result;
     const GwyXYZ *xyz;
     GwyXYZ *xyz_result;
@@ -446,7 +428,7 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
     FFTDirType dir = gwy_params_get_enum(args->params, PARAM_DIR);
     gboolean ok = FALSE;
 
-    GWY_OBJECT_UNREF(args->result);
+    g_clear_object(&args->result);
 
     xyz = gwy_surface_get_data_const(surface);
     n = gwy_surface_get_npoints(surface);
@@ -457,13 +439,13 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
 
     gwy_app_wait_start(wait_window, _("Filtering..."));
 
-    rfft = gwy_data_line_new(n, n, FALSE);
-    ifft = gwy_data_line_new(n, n, FALSE);
-    rresult = gwy_data_line_new(n, n, FALSE);
-    iresult = gwy_data_line_new(n, n, FALSE);
-    weights = gwy_data_line_new(n/2 + 1, n/2 + 1, TRUE);
-    fullspectrum = gwy_data_line_new(n/2 + 1, n/2 + 1, FALSE);
-    linedata = gwy_data_line_get_data(args->fftinput);
+    rfft = gwy_line_new(n, n, FALSE);
+    ifft = gwy_line_new(n, n, FALSE);
+    rresult = gwy_line_new(n, n, FALSE);
+    iresult = gwy_line_new(n, n, FALSE);
+    weights = gwy_line_new(n/2 + 1, n/2 + 1, TRUE);
+    fullspectrum = gwy_line_new(n/2 + 1, n/2 + 1, FALSE);
+    linedata = gwy_line_get_data(args->fftinput);
 
     if (dir == DIR_X) {
        for (k = 0; k < n; k++) {
@@ -481,20 +463,20 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
        }
     }
 
-    gwy_data_line_fft_raw(args->fftinput, NULL, rfft, ifft, GWY_TRANSFORM_DIRECTION_FORWARD);
+    gwy_line_fft_raw(args->fftinput, NULL, rfft, ifft, GWY_TRANSFORM_DIRECTION_FORWARD);
 
     if (!gwy_app_wait_set_fraction(0.4))
         goto end;
 
-    fsdata = gwy_data_line_get_data(fullspectrum);
-    realdata = gwy_data_line_get_data(rfft);
-    imagdata = gwy_data_line_get_data(ifft);
+    fsdata = gwy_line_get_data(fullspectrum);
+    realdata = gwy_line_get_data(rfft);
+    imagdata = gwy_line_get_data(ifft);
     for (k = 0; k < n/2; k++)
         fsdata[k] = sqrt(realdata[k]*realdata[k] + imagdata[k]*imagdata[k]);
 
     calculate_weights(weights, fullspectrum, args);
 
-    weightdata = gwy_data_line_get_data(weights);
+    weightdata = gwy_line_get_data(weights);
 
     if (!gwy_app_wait_set_fraction(0.6))
         goto end;
@@ -509,12 +491,12 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
     if (!gwy_app_wait_set_fraction(0.7))
         goto end;
 
-    gwy_data_line_fft_raw(rfft, ifft, rresult, iresult, GWY_TRANSFORM_DIRECTION_BACKWARD);
+    gwy_line_fft_raw(rfft, ifft, rresult, iresult, GWY_TRANSFORM_DIRECTION_BACKWARD);
 
     if (!gwy_app_wait_set_fraction(0.9))
         goto end;
 
-    rresultdata = gwy_data_line_get_data(rresult);
+    rresultdata = gwy_line_get_data(rresult);
 
     if (dir == DIR_X) {
        for (k = 0; k < n; k++) {
@@ -542,18 +524,20 @@ execute(ModuleArgs *args, GtkWindow *wait_window)
 
 end:
     gwy_app_wait_finish();
-    GWY_OBJECT_UNREF(rfft);
-    GWY_OBJECT_UNREF(ifft);
-    GWY_OBJECT_UNREF(rresult);
-    GWY_OBJECT_UNREF(iresult);
-    GWY_OBJECT_UNREF(weights);
-    GWY_OBJECT_UNREF(fullspectrum);
+    g_clear_object(&rfft);
+    g_clear_object(&ifft);
+    g_clear_object(&rresult);
+    g_clear_object(&iresult);
+    g_clear_object(&weights);
+    g_clear_object(&fullspectrum);
 
     return ok;
 }
 
+/* XXX: This function is identical among image 1D FFT filter, volume 1D FFT filter and XYZ FFT.
+ * Make a library function! */
 static void
-calculate_weights(GwyDataLine *weights, GwyDataLine *modulus, ModuleArgs *args)
+calculate_weights(GwyLine *weights, GwyLine *modulus, ModuleArgs *args)
 {
     GwyFFTFilt1DSuppressType suppres = gwy_params_get_enum(args->params, PARAM_SUPPRESS);
     GwyFFTFilt1DOutputType output = gwy_params_get_enum(args->params, PARAM_OUTPUT);
@@ -562,18 +546,18 @@ calculate_weights(GwyDataLine *weights, GwyDataLine *modulus, ModuleArgs *args)
     gdouble sel[2], *w;
     gint fill_from, fill_to;
 
-    res = gwy_data_line_get_res(weights);
+    res = gwy_line_get_res(weights);
     nsel = gwy_selection_get_data(selection, NULL);
     for (k = 0; k < nsel; k++) {
         gwy_selection_get_object(selection, k, sel);
         GWY_ORDER(gdouble, sel[0], sel[1]);
-        fill_from = MAX(0, gwy_data_line_rtoi(weights, sel[0]));
+        fill_from = MAX(0, gwy_line_rtoi(weights, sel[0]));
         fill_from = MIN(res, fill_from);
-        fill_to = MIN(res, gwy_data_line_rtoi(weights, sel[1]));
-        gwy_data_line_part_fill(weights, fill_from, fill_to, 1.0);
+        fill_to = MIN(res, gwy_line_rtoi(weights, sel[1]));
+        gwy_line_part_fill(weights, fill_from, fill_to - fill_from, 1.0);
     }
 
-    w = gwy_data_line_get_data(weights);
+    w = gwy_line_get_data(weights);
     if (output == OUTPUT_MARKED) {
         for (k = 0; k < res; k++)
             w[k] = 1.0 - w[k];
@@ -582,16 +566,16 @@ calculate_weights(GwyDataLine *weights, GwyDataLine *modulus, ModuleArgs *args)
     /* For Suppress, interpolate PSDF linearly between endpoints.  Since we pass weights to the filter, not PSDF
      * itself, we have to divide to get the weight. */
     if (suppres == SUPPRESS_NEIGBOURHOOD) {
-        GwyDataLine *buf = gwy_data_line_duplicate(modulus);
+        GwyLine *buf = gwy_line_copy(modulus);
         gdouble *b, *m;
 
         /*invert the weights as the laplace correction treats it as a mask*/
         for (k = 0; k < res; k++)
             w[k] = 1.0 - w[k];
 
-        gwy_data_line_correct_laplace(buf, weights);
-        b = gwy_data_line_get_data(buf);
-        m = gwy_data_line_get_data(modulus);
+        gwy_line_correct_laplace(buf, weights);
+        b = gwy_line_get_data(buf);
+        m = gwy_line_get_data(modulus);
 
         for (k = 0; k < res; k++)
             w[k] = (m[k] > 0.0 ? fmin(b[k]/m[k], 1.0) : 0.0);
