@@ -1,5 +1,5 @@
 /*
- *  $Id: xyz_level.c 28895 2025-11-20 12:13:29Z klapetek $
+ *  $Id: xyz_level.c 28808 2025-11-05 18:26:20Z yeti-dn $
  *  Copyright (C) 2016-2023 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -15,14 +15,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/surface.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-xyz.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 
 #define RUN_MODES (GWY_RUN_INTERACTIVE | GWY_RUN_IMMEDIATE)
 
@@ -52,17 +48,17 @@ typedef struct {
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
 static void             execute             (ModuleArgs *args,
-                                             GwyContainer *data,
+                                             GwyFile *data,
                                              gint id);
 static GwyDialogOutcome run_gui             (ModuleArgs *args);
 static void             param_changed       (ModuleGUI *gui,
                                              gint id);
-static void             xyzfixzero          (GwyContainer *data,
-                                             GwyRunType runtype);
-static void             xyzzeromean         (GwyContainer *data,
-                                             GwyRunType runtype);
-static void             xyzlevel            (GwyContainer *data,
-                                             GwyRunType runtype);
+static void             xyzfixzero          (GwyFile *data,
+                                             GwyRunModeFlags mode);
+static void             xyzzeromean         (GwyFile *data,
+                                             GwyRunModeFlags mode);
+static void             xyzlevel            (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static void             level_rotate_xyz    (GwySurface *surface,
                                              gdouble bx,
                                              gdouble by,
@@ -93,23 +89,23 @@ static gboolean
 module_register(void)
 {
     gwy_xyz_func_register("xyz_fix_zero",
-                          (GwyXYZFunc)&xyzfixzero,
+                          xyzfixzero,
                           N_("/Fix _Zero"),
-                          GWY_STOCK_FIX_ZERO,
+                          GWY_ICON_FIX_ZERO,
                           RUN_MODES,
                           GWY_MENU_FLAG_XYZ,
                           N_("Shift minimum data value to zero"));
     gwy_xyz_func_register("xyz_zero_mean",
-                          (GwyXYZFunc)&xyzzeromean,
+                          xyzzeromean,
                           N_("/Zero _Mean Value"),
-                          GWY_STOCK_ZERO_MEAN,
+                          GWY_ICON_ZERO_MEAN,
                           RUN_MODES,
                           GWY_MENU_FLAG_XYZ,
                           N_("Shift mean data value to zero"));
     gwy_xyz_func_register("xyz_level",
-                          (GwyXYZFunc)&xyzlevel,
+                          xyzlevel,
                           N_("/Plane _Level..."),
-                          GWY_STOCK_LEVEL,
+                          GWY_ICON_LEVEL,
                           RUN_MODES,
                           GWY_MENU_FLAG_XYZ,
                           N_("Level data by mean plane correction"));
@@ -140,7 +136,7 @@ define_module_params(void)
 }
 
 static void
-xyzfixzero(GwyContainer *data, G_GNUC_UNUSED GwyRunType runtype)
+xyzfixzero(GwyFile *data, G_GNUC_UNUSED GwyRunModeFlags mode)
 {
     GwySurface *surface = NULL;
     GQuark quark;
@@ -149,13 +145,13 @@ xyzfixzero(GwyContainer *data, G_GNUC_UNUSED GwyRunType runtype)
     guint k, n;
     gdouble zmin, zmax;
 
-    gwy_app_data_browser_get_current(GWY_APP_SURFACE, &surface,
-                                     GWY_APP_SURFACE_ID, &id,
-                                     0);
+    gwy_data_browser_get_current(GWY_APP_SURFACE, &surface,
+                                 GWY_APP_SURFACE_ID, &id,
+                                 0);
     g_return_if_fail(GWY_IS_SURFACE(surface));
 
-    quark = gwy_app_get_surface_key_for_id(id);
-    gwy_app_undo_qcheckpointv(data, 1, &quark);
+    quark = gwy_file_key_xyz(id);
+    gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &quark);
 
     gwy_surface_get_min_max(surface, &zmin, &zmax);
     xyz = gwy_surface_get_data(surface);
@@ -168,7 +164,7 @@ xyzfixzero(GwyContainer *data, G_GNUC_UNUSED GwyRunType runtype)
 
 /* FIXME: We should use mean weighted by area.  But that must wait until we can do such thing... */
 static void
-xyzzeromean(GwyContainer *data, G_GNUC_UNUSED GwyRunType runtype)
+xyzzeromean(GwyFile *data, G_GNUC_UNUSED GwyRunModeFlags mode)
 {
     GwySurface *surface = NULL;
     GQuark quark;
@@ -177,13 +173,13 @@ xyzzeromean(GwyContainer *data, G_GNUC_UNUSED GwyRunType runtype)
     guint k, n;
     gdouble zmean = 0.0;
 
-    gwy_app_data_browser_get_current(GWY_APP_SURFACE, &surface,
-                                     GWY_APP_SURFACE_ID, &id,
-                                     0);
+    gwy_data_browser_get_current(GWY_APP_SURFACE, &surface,
+                                 GWY_APP_SURFACE_ID, &id,
+                                 0);
     g_return_if_fail(GWY_IS_SURFACE(surface));
 
-    quark = gwy_app_get_surface_key_for_id(id);
-    gwy_app_undo_qcheckpointv(data, 1, &quark);
+    quark = gwy_file_key_xyz(id);
+    gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &quark);
 
     xyz = gwy_surface_get_data(surface);
     n = gwy_surface_get_npoints(surface);
@@ -197,28 +193,27 @@ xyzzeromean(GwyContainer *data, G_GNUC_UNUSED GwyRunType runtype)
 }
 
 static void
-xyzlevel(GwyContainer *data, GwyRunType runtype)
+xyzlevel(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     ModuleArgs args;
     gint id;
 
-    g_return_if_fail(runtype & RUN_MODES);
+    g_return_if_fail(mode & RUN_MODES);
 
-    gwy_app_data_browser_get_current(GWY_APP_SURFACE, &args.surface,
-                                     GWY_APP_SURFACE_ID, &id,
-                                     0);
+    gwy_data_browser_get_current(GWY_APP_SURFACE, &args.surface,
+                                 GWY_APP_SURFACE_ID, &id,
+                                 0);
     g_return_if_fail(GWY_IS_SURFACE(args.surface));
 
     args.params = gwy_params_new_from_settings(define_module_params());
-    args.same_units = gwy_si_unit_equal(gwy_surface_get_si_unit_xy(args.surface),
-                                        gwy_surface_get_si_unit_z(args.surface));
+    args.same_units = gwy_unit_equal(gwy_surface_get_unit_xy(args.surface), gwy_surface_get_unit_z(args.surface));
     /* sanitise_params(), but do not bother with a single param. */
     if (!args.same_units) {
         gwy_params_set_enum(args.params, PARAM_METHOD, XYZ_LEVEL_SUBTRACT);
         gwy_params_set_enum(args.params, PARAM_UPDATE_ALL, FALSE);
     }
-    if (runtype == GWY_RUN_INTERACTIVE && args.same_units) {
+    if (mode ==GWY_RUN_INTERACTIVE && args.same_units) {
         outcome = run_gui(&args);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -228,8 +223,6 @@ xyzlevel(GwyContainer *data, GwyRunType runtype)
     if (outcome == GWY_DIALOG_PROCEED)
         execute(&args, data, id);
 
-    gwy_app_xyz_log_add(data, -1, id, "xyz::xyz_level", NULL);
-
 end:
     g_object_unref(args.params);
 }
@@ -241,7 +234,7 @@ run_gui(ModuleArgs *args)
     GwyDialog *dialog;
     GwyParamTable *table;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
     gui.dialog = gwy_dialog_new(_("Level XYZ Data"));
     dialog = GWY_DIALOG(gui.dialog);
@@ -296,12 +289,12 @@ accelerate_convergence(const gdouble *seq, gdouble *x)
 
 static void
 execute(ModuleArgs *args,
-        GwyContainer *data,
+        GwyFile *data,
         gint id)
 {
     XYZLevelType method = gwy_params_get_enum(args->params, PARAM_METHOD);
     gboolean update_all = gwy_params_get_boolean(args->params, PARAM_UPDATE_ALL);
-    GQuark otherquark, quark = gwy_app_get_surface_key_for_id(id);
+    GQuark quark = gwy_file_key_xyz(id);
     GQuark *allquarks = NULL;
     GwySurface *surface = args->surface, *othersurface;
     GwyXYZ *xyz, c;
@@ -311,7 +304,7 @@ execute(ModuleArgs *args,
     gint *ids = NULL;
 
     if (method == XYZ_LEVEL_SUBTRACT) {
-        gwy_app_undo_qcheckpointv(data, 1, &quark);
+        gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &quark);
         find_plane_coeffs(surface, &a, &bx, &by, &c);
         gwy_debug("b %g %g", bx, by);
         xyz = gwy_surface_get_data(surface);
@@ -324,29 +317,23 @@ execute(ModuleArgs *args,
     }
 
     if (update_all) {
-        ids = gwy_app_data_browser_get_xyz_ids(data);
+        ids = gwy_file_get_ids(data, GWY_FILE_XYZ);
         for (kq = 0; ids[kq] > -1; kq++) {
-            if (ids[kq] == id) {
-                ids[nq++] = ids[kq];
-            }
-            else {
-                otherquark = gwy_app_get_surface_key_for_id(ids[kq]);
-                othersurface = gwy_container_get_object(data, otherquark);
-                if (gwy_surface_xy_is_compatible(surface, othersurface))
-                    ids[nq++] = ids[kq];
-            }
+            gint otherid = ids[kq];
+            if (otherid == id || gwy_surface_xy_is_compatible(surface, gwy_file_get_xyz(data, otherid)))
+                ids[nq++] = otherid;
         }
         ids[nq] = -1;
 
         g_assert(nq);
         allquarks = g_new(GQuark, nq);
         for (kq = 0; kq < nq; kq++)
-            allquarks[kq] = gwy_app_get_surface_key_for_id(ids[kq]);
-        gwy_app_undo_qcheckpointv(data, nq, allquarks);
+            allquarks[kq] = gwy_file_key_xyz(ids[kq]);
+        gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), nq, allquarks);
         g_free(allquarks);
     }
     else
-        gwy_app_undo_qcheckpointv(data, 1, &quark);
+        gwy_app_undo_qcheckpointv(GWY_CONTAINER(data), 1, &quark);
 
     /* find_plane_coeffs() calculates the mean plane in ordinary least-squares sense.  But this is not self-consistent
      * with rotation that should use total least squares.  Perform a few iterations. */
@@ -394,8 +381,7 @@ execute(ModuleArgs *args,
         if (ids[kq] == id)
             continue;
 
-        otherquark = gwy_app_get_surface_key_for_id(ids[kq]);
-        othersurface = gwy_container_get_object(data, otherquark);
+        othersurface = gwy_file_get_xyz(data, ids[kq]);
         xyz = gwy_surface_get_data(othersurface);
         for (k = 0; k < n; k++) {
             xyz[k].x = newxyz[k].x;
