@@ -1,5 +1,5 @@
 /*
- *  $Id: xyz_raster.c 26510 2024-08-15 17:12:23Z yeti-dn $
+ *  $Id: xyz_raster.c 28910 2025-11-24 18:11:09Z yeti-dn $
  *  Copyright (C) 2016-2023 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -15,24 +15,12 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libgwyddion/gwythreads.h>
-#include <libgwyddion/gwyutils.h>
-#include <libprocess/stats.h>
-#include <libprocess/filters.h>
-#include <libprocess/grains.h>
-#include <libprocess/triangulation.h>
-#include <libgwydgets/gwydataview.h>
-#include <libgwydgets/gwylayer-basic.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-xyz.h>
-#include <app/gwymoduleutils.h>
-#include <app/gwyapp.h>
-#include "libgwyddion/gwyomp.h"
+#include <gwy.h>
+#include "libgwyddion/omp.h"
 #include "../process/preview.h"
 
 #define RUN_MODES (GWY_RUN_INTERACTIVE | GWY_RUN_IMMEDIATE)
@@ -98,17 +86,16 @@ typedef struct {
 typedef struct {
     GwyParams *params;
     GwySurface *surface;
-    GwyDataField *regular;
-    GwyDataField *raster;
-    GwyDataField *nilmask;
+    GwyField *regular;
+    GwyField *raster;
+    GwyField *nilmask;
     XYZRasData rdata;
     XYZRasRange full_range; /* Real, not scaled. */
-    GwySIValueFormat *vf;
+    GwyValueFormat *vf;
 } ModuleArgs;
 
 typedef struct {
     ModuleArgs *args;
-    GwyContainer *data;
     GtkWidget *dialog;
     GtkWidget *directbox;
     GtkWidget *dataview;
@@ -127,19 +114,19 @@ typedef struct {
 } WorkQueue;
 
 static gboolean         module_register            (void);
-static void             xyzras                     (GwyContainer *data,
-                                                    GwyRunType run);
+static void             module_main                (GwyFile *data,
+                                                    GwyRunModeFlags mode);
 static GwyParamDef*     define_module_params       (void);
 static gboolean         execute                    (ModuleArgs *args,
                                                     GtkWindow *window,
                                                     gchar **error);
 static void             set_raster_field_properties(ModuleArgs *args);
-static void             add_field_to_data          (GwyDataField *field,
-                                                    GwyDataField *mask,
-                                                    GwyContainer *data,
+static void             add_field_to_data          (GwyField *field,
+                                                    GwyField *mask,
+                                                    GwyFile *data,
                                                     gint id);
 static GwyDialogOutcome run_gui                    (ModuleArgs *args,
-                                                    GwyContainer *data,
+                                                    GwyFile *data,
                                                     gint id);
 static void             param_changed              (ModuleGUI *gui,
                                                     gint id);
@@ -156,7 +143,7 @@ static void             triangulation_info         (ModuleGUI *gui);
 static void             render_regular_directly    (ModuleGUI *gui);
 static gboolean         interpolate_field          (guint npoints,
                                                     const GwyXYZ *points,
-                                                    GwyDataField *field,
+                                                    GwyField *field,
                                                     gdouble power,
                                                     GwySetFractionFunc set_fraction,
                                                     GwySetMessageFunc set_message);
@@ -164,11 +151,11 @@ static gboolean         extend_borders             (ModuleArgs *args,
                                                     gboolean check_for_changes,
                                                     gdouble epsrel);
 static void             initialise_range           (ModuleArgs *args);
-static void             set_range_from_field       (GwyDataField *field,
+static void             set_range_from_field       (GwyField *field,
                                                     XYZRasRange *range);
 static void             analyse_points             (ModuleArgs *args,
                                                     double epsrel);
-static GwyDataField*    check_regular_grid         (GwySurface *surface);
+static GwyField*        check_regular_grid         (GwySurface *surface);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
@@ -186,9 +173,9 @@ static gboolean
 module_register(void)
 {
     gwy_xyz_func_register("xyz_raster",
-                          (GwyXYZFunc)&xyzras,
+                          module_main,
                           N_("/_Rasterize..."),
-                          GWY_STOCK_RASTERIZE,
+                          GWY_ICON_RASTERIZE,
                           RUN_MODES,
                           GWY_MENU_FLAG_XYZ,
                           N_("Rasterize to image"));
@@ -207,9 +194,9 @@ define_module_params(void)
         { N_("Average"), GWY_INTERPOLATION_AVERAGE, },
     };
     static const GwyEnum exteriors[] = {
-        { N_("exterior|Border"),   GWY_EXTERIOR_BORDER_EXTEND, },
-        { N_("exterior|Mirror"),   GWY_EXTERIOR_MIRROR_EXTEND, },
-        { N_("exterior|Periodic"), GWY_EXTERIOR_PERIODIC,      },
+        { gwy_NC("exterior", "Border"),   GWY_EXTERIOR_BORDER, },
+        { gwy_NC("exterior", "Mirror"),   GWY_EXTERIOR_MIRROR, },
+        { gwy_NC("exterior", "Periodic"), GWY_EXTERIOR_PERIODIC,      },
     };
     static GwyParamDef *paramdef = NULL;
 
@@ -221,7 +208,7 @@ define_module_params(void)
     gwy_param_def_add_gwyenum(paramdef, PARAM_INTERPOLATION, "interpolation", _("_Interpolation type"),
                               interpolations, G_N_ELEMENTS(interpolations), GWY_INTERPOLATION_AVERAGE);
     gwy_param_def_add_gwyenum(paramdef, PARAM_EXTERIOR, "exterior", _("_Exterior type"),
-                              exteriors, G_N_ELEMENTS(exteriors), GWY_EXTERIOR_MIRROR_EXTEND);
+                              exteriors, G_N_ELEMENTS(exteriors), GWY_EXTERIOR_MIRROR);
     gwy_param_def_add_boolean(paramdef, PARAM_MASK_EMPTY, "mask_empty", _("_Mask empty regions"), TRUE);
     gwy_param_def_add_int(paramdef, PARAM_XRES, "xres", _("_Horizontal size"), 2, 16384, 512);
     gwy_param_def_add_int(paramdef, PARAM_YRES, "yres", _("_Vertical size"), 2, 16384, 512);
@@ -236,7 +223,7 @@ define_module_params(void)
 }
 
 static void
-xyzras(GwyContainer *data, GwyRunType run)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     ModuleArgs args;
@@ -245,17 +232,16 @@ xyzras(GwyContainer *data, GwyRunType run)
     gboolean ok = TRUE;
     gint id;
 
-    g_return_if_fail(run & RUN_MODES);
-    g_return_if_fail(g_type_from_name("GwyLayerRectangle"));
+    g_return_if_fail(mode & RUN_MODES);
 
-    gwy_clear(&args, 1);
-    gwy_app_data_browser_get_current(GWY_APP_SURFACE, &args.surface,
-                                     GWY_APP_SURFACE_ID, &id,
-                                     0);
+    gwy_clear1(args);
+    gwy_data_browser_get_current(GWY_APP_SURFACE, &args.surface,
+                                 GWY_APP_SURFACE_ID, &id,
+                                 0);
     g_return_if_fail(GWY_IS_SURFACE(args.surface));
 
     args.regular = check_regular_grid(args.surface);
-    if (args.regular && run == GWY_RUN_IMMEDIATE) {
+    if (args.regular && mode == GWY_RUN_IMMEDIATE) {
         add_field_to_data(args.regular, NULL, data, id);
         g_object_unref(args.regular);
         return;
@@ -265,20 +251,20 @@ xyzras(GwyContainer *data, GwyRunType run)
     args.rdata.points = g_array_new(FALSE, FALSE, sizeof(GwyXYZ));
     analyse_points(&args, EPSREL);
     initialise_range(&args);
-    args.vf = gwy_surface_get_value_format_xy(args.surface, GWY_SI_UNIT_FORMAT_VFMARKUP, NULL);
+    args.vf = gwy_surface_get_value_format_xy(args.surface, GWY_UNIT_FORMAT_VFMARKUP, NULL);
     if (args.regular)
-        args.raster = gwy_data_field_duplicate(args.regular);
+        args.raster = gwy_field_copy(args.regular);
     else {
-        args.raster = gwy_data_field_new(PREVIEW_SIZE, PREVIEW_SIZE,
-                                         args.full_range.max.x - args.full_range.min.x,
-                                         args.full_range.max.y - args.full_range.min.y,
-                                         TRUE);
-        gwy_data_field_set_xoffset(args.raster, args.full_range.min.x);
-        gwy_data_field_set_yoffset(args.raster, args.full_range.min.y);
-        gwy_surface_copy_units_to_data_field(args.surface, args.raster);
+        args.raster = gwy_field_new(PREVIEW_SIZE, PREVIEW_SIZE,
+                                    args.full_range.max.x - args.full_range.min.x,
+                                    args.full_range.max.y - args.full_range.min.y,
+                                    TRUE);
+        gwy_field_set_xoffset(args.raster, args.full_range.min.x);
+        gwy_field_set_yoffset(args.raster, args.full_range.min.y);
+        gwy_surface_copy_units_to_field(args.surface, args.raster);
     }
 
-    if (run == GWY_RUN_INTERACTIVE) {
+    if (mode == GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, id);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -286,12 +272,12 @@ xyzras(GwyContainer *data, GwyRunType run)
     }
 
     if (outcome != GWY_DIALOG_HAVE_RESULT)
-        ok = execute(&args, gwy_app_find_window_for_xyz(data, id), &error);
+        ok = execute(&args, gwy_data_browser_get_window_for_data(data, GWY_FILE_XYZ, id), &error);
 
     if (ok)
         add_field_to_data(args.raster, args.nilmask, data, id);
-    else if (run == GWY_RUN_INTERACTIVE) {
-        dialog = gtk_message_dialog_new(gwy_app_find_window_for_channel(data, id),
+    else if (mode == GWY_RUN_INTERACTIVE) {
+        dialog = gtk_message_dialog_new(gwy_data_browser_get_window_for_data(data, GWY_FILE_IMAGE, id),
                                         GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
                                         "%s", error);
         gtk_dialog_run(GTK_DIALOG(dialog));
@@ -300,46 +286,40 @@ xyzras(GwyContainer *data, GwyRunType run)
     g_free(error);
 
 end:
-    GWY_OBJECT_UNREF(args.rdata.triangulation);
-    gwy_si_unit_value_format_free(args.vf);
+    g_clear_object(&args.rdata.triangulation);
+    gwy_value_format_free(args.vf);
     g_array_free(args.rdata.points, TRUE);
     g_object_unref(args.params);
 }
 
 static void
-add_field_to_data(GwyDataField *field, GwyDataField *mask,
-                   GwyContainer *data, gint id)
+add_field_to_data(GwyField *field, GwyField *mask,
+                  GwyFile *data, gint id)
 {
-    const guchar *s;
-    gint newid;
-
-    newid = gwy_app_data_browser_add_data_field(field, data, TRUE);
+    gint newid = gwy_file_add_image(data, field);
+    gwy_file_set_visible(data, GWY_FILE_IMAGE, newid, TRUE);
     if (mask)
-        gwy_container_set_object(data,  gwy_app_get_mask_key_for_id(newid), mask);
-    gwy_app_channel_log_add(data, -1, newid, "xyz::xyz_raster", NULL);
+        gwy_file_pass_image_mask(data, newid, mask);
+    gwy_log_add_full(data, GWY_FILE_IMAGE, -1, newid, "xyz::xyz_raster", NULL);
 
-    if (gwy_container_gis_string(data, gwy_app_get_surface_palette_key_for_id(id), &s))
-        gwy_container_set_const_string(data, gwy_app_get_data_palette_key_for_id(newid), s);
-    if (gwy_container_gis_string(data, gwy_app_get_surface_title_key_for_id(id), &s))
-        gwy_container_set_const_string(data, gwy_app_get_data_title_key_for_id(newid), s);
+    gwy_file_sync_items(data, GWY_FILE_XYZ, id,
+                        data, GWY_FILE_IMAGE, newid,
+                        GWY_FILE_ITEM_PALETTE, FALSE);
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     GtkWidget *label, *hbox, *button;
     GwyDialogOutcome outcome;
     GwyDialog *dialog;
     GwyParamTable *table;
     ModuleGUI gui;
-    const guchar *gradient;
     GwySelection *selection;
 
-    gwy_clear(&gui, 1);
+    gwy_clear1(gui);
     gui.args = args;
     gui.zoom_stack = g_array_new(FALSE, FALSE, sizeof(XYZRasRange));
-    gui.data = gwy_container_new();
-    gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(0), args->raster);
     /* We actually measure the ranges in displayed units; they are GUI-only. Convert them through the selection-changed
      * chain. */
     gui.last_updated = LAST_UPDATED_X;
@@ -349,7 +329,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_dialog_add_buttons(dialog, GWY_RESPONSE_UPDATE, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, 0);
 
     if (args->regular) {
-        hbox = gui.directbox = gwy_hbox_new(8);
+        hbox = gui.directbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 8);
         gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
         gwy_dialog_add_content(dialog, hbox, FALSE, FALSE, 0);
 
@@ -361,10 +341,10 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
         gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
     }
 
-    gui.dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
-    selection = gui.selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(gui.dataview), 0, "Rectangle", 1, TRUE);
-    if (gwy_container_gis_string(data, gwy_app_get_surface_palette_key_for_id(id), &gradient))
-        gwy_container_set_const_string(gui.data, gwy_app_get_data_palette_key_for_id(0), gradient);
+    gui.dataview = gwy_create_preview(args->raster, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(gui.dataview), data, GWY_FILE_XYZ, id, GWY_FILE_ITEM_PALETTE);
+    selection = gui.selection = gwy_create_preview_vector_layer(GWY_DATA_VIEW(gui.dataview),
+                                                                GWY_TYPE_LAYER_RECTANGLE, 1, TRUE);
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(gui.dataview), TRUE);
 
     table = gui.table = gwy_param_table_new(args->params);
@@ -412,7 +392,6 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 
     outcome = gwy_dialog_run(dialog);
 
-    g_object_unref(gui.data);
     g_array_free(gui.zoom_stack, TRUE);
 
     return outcome;
@@ -463,8 +442,8 @@ dialog_response(ModuleGUI *gui, gint response)
         }
         set_range(gui, &range);
         set_raster_field_properties(args);
-        gwy_data_field_clear(args->raster);
-        gwy_data_field_data_changed(args->raster);
+        gwy_field_clear(args->raster);
+        gwy_field_data_changed(args->raster);
         gwy_selection_clear(gui->selection);
         gwy_param_table_set_sensitive(gui->table, BUTTON_UNZOOM_RANGES, n > 0);
     }
@@ -518,8 +497,8 @@ update_selection(ModuleGUI *gui)
     if (gui->changing_selection)
         return;
 
-    xoff = gwy_data_field_get_xoffset(args->raster);
-    yoff = gwy_data_field_get_yoffset(args->raster);
+    xoff = gwy_field_get_xoffset(args->raster);
+    yoff = gwy_field_get_yoffset(args->raster);
     xy[0] = q*gwy_params_get_double(params, PARAM_XMIN) - xoff;
     xy[1] = q*gwy_params_get_double(params, PARAM_YMIN) - yoff;
     xy[2] = q*gwy_params_get_double(params, PARAM_XMAX) - xoff;
@@ -541,8 +520,8 @@ selection_changed(ModuleGUI *gui,
         return;
 
     if (gwy_selection_get_data(selection, NULL) == 1) {
-        xoff = gwy_data_field_get_xoffset(args->raster);
-        yoff = gwy_data_field_get_yoffset(args->raster);
+        xoff = gwy_field_get_xoffset(args->raster);
+        yoff = gwy_field_get_yoffset(args->raster);
         gwy_selection_get_data(selection, xy);
         range.min.x = fmin(xy[0], xy[2]) + xoff;
         range.max.x = fmax(xy[0], xy[2]) + xoff;
@@ -564,7 +543,7 @@ preview(gpointer user_data)
     ModuleArgs *args = gui->args;
     gchar *error = NULL;
 
-    GWY_OBJECT_UNREF(args->nilmask);
+    g_clear_object(&args->nilmask);
     if (execute(args, GTK_WINDOW(gui->dialog), &error)) {
         GArray *zst = gui->zoom_stack;
         XYZRasRange range;
@@ -579,11 +558,11 @@ preview(gpointer user_data)
     else {
         gwy_param_table_set_label(gui->table, MESSAGE_ERROR, error);
         g_free(error);
-        gwy_data_field_clear(args->raster);
+        gwy_field_clear(args->raster);
     }
     triangulation_info(gui);
 
-    gwy_data_field_data_changed(args->raster);
+    gwy_field_data_changed(args->raster);
     gwy_set_data_preview_size(GWY_DATA_VIEW(gui->dataview), PREVIEW_SIZE);
 
     /* After doing preview the selection always covers the full data and thus is not useful. */
@@ -616,7 +595,7 @@ render_regular_directly(ModuleGUI *gui)
 {
     ModuleArgs *args = gui->args;
 
-    gwy_object_unref(args->raster);
+    g_clear_object(&args->raster);
     args->raster = g_object_ref(args->regular);
     gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
     gtk_dialog_response(GTK_DIALOG(gui->dialog), GTK_RESPONSE_OK);
@@ -639,18 +618,18 @@ execute(ModuleArgs *args, GtkWindow *window, gchar **error)
     gboolean mask_empty = gwy_params_get_boolean(params, PARAM_MASK_EMPTY);
     GwyTriangulation *triangulation = rdata->triangulation;
     GArray *points = rdata->points;
-    GwyDataField *field = args->raster, *mask;
+    GwyField *field = args->raster, *mask;
     GwySetMessageFunc set_message = (window ? gwy_app_wait_set_message : NULL);
     GwySetFractionFunc set_fraction = (window ? gwy_app_wait_set_fraction : NULL);
     gboolean ok = TRUE, extended;
 
-    GWY_OBJECT_UNREF(args->nilmask);
+    g_clear_object(&args->nilmask);
     gwy_debug("%g %g :: %g %g", xmin, xmax, ymin, ymax);
     if (!(xmax > xmin) || !(ymax > ymin)) {
         *error = g_strdup(_("Physical dimensions are invalid."));
         return FALSE;
     }
-    gwy_data_field_resample(field, xres, yres, GWY_INTERPOLATION_NONE);
+    gwy_field_resize(field, xres, yres);
     set_raster_field_properties(args);
 
     if ((gint)interpolation == GWY_INTERPOLATION_FIELD) {
@@ -665,13 +644,13 @@ execute(ModuleArgs *args, GtkWindow *window, gchar **error)
     else if ((gint)interpolation == GWY_INTERPOLATION_AVERAGE) {
         extend_borders(args, FALSE, EPSREL);
         if (mask_empty) {
-            args->nilmask = mask = gwy_data_field_new_alike(field, FALSE);
-            gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(mask), NULL);
-            gwy_data_field_average_xyz(field, mask, (const GwyXYZ*)points->data, points->len);
-            gwy_data_field_threshold(mask, G_MINDOUBLE, 1.0, 0.0);
+            args->nilmask = mask = gwy_field_new_alike(field, FALSE);
+            gwy_unit_clear(gwy_field_get_unit_z(mask));
+            gwy_field_average_xyz(field, mask, (const GwyXYZ*)points->data, points->len);
+            gwy_field_threshold(mask, G_MINDOUBLE, 1.0, 0.0);
         }
         else {
-            gwy_data_field_average_xyz(field, NULL, (const GwyXYZ*)points->data, points->len);
+            gwy_field_average_xyz(field, NULL, (const GwyXYZ*)points->data, points->len);
         }
         ok = TRUE;
     }
@@ -707,7 +686,7 @@ execute(ModuleArgs *args, GtkWindow *window, gchar **error)
     }
 
     if (!ok) {
-        GWY_OBJECT_UNREF(rdata->triangulation);
+        g_clear_object(&rdata->triangulation);
         *error = g_strdup(_("XYZ data regularization failed due to\nnumerical instability or was interrupted."));
         return FALSE;
     }
@@ -724,19 +703,19 @@ set_raster_field_properties(ModuleArgs *args)
     gdouble xmax = q*gwy_params_get_double(params, PARAM_XMAX);
     gdouble ymin = q*gwy_params_get_double(params, PARAM_YMIN);
     gdouble ymax = q*gwy_params_get_double(params, PARAM_YMAX);
-    GwyDataField *field = args->raster;
+    GwyField *field = args->raster;
 
-    gwy_data_field_set_xreal(field, xmax - xmin);
-    gwy_data_field_set_yreal(field, ymax - ymin);
-    gwy_data_field_set_xoffset(field, xmin);
-    gwy_data_field_set_yoffset(field, ymin);
-    gwy_surface_copy_units_to_data_field(args->surface, field);
+    gwy_field_set_xreal(field, xmax - xmin);
+    gwy_field_set_yreal(field, ymax - ymin);
+    gwy_field_set_xoffset(field, xmin);
+    gwy_field_set_yoffset(field, ymin);
+    gwy_surface_copy_units_to_field(args->surface, field);
 }
 
 static gboolean
 interpolate_field(guint npoints,
                   const GwyXYZ *points,
-                  GwyDataField *field,
+                  GwyField *field,
                   gdouble power,
                   GwySetFractionFunc set_fraction,
                   GwySetMessageFunc set_message)
@@ -747,13 +726,13 @@ interpolate_field(guint npoints,
     gint ipwr;
     gdouble *d;
 
-    xres = gwy_data_field_get_xres(field);
-    yres = gwy_data_field_get_yres(field);
-    xoff = gwy_data_field_get_xoffset(field);
-    yoff = gwy_data_field_get_yoffset(field);
-    qx = gwy_data_field_get_xreal(field)/xres;
-    qy = gwy_data_field_get_yreal(field)/yres;
-    d = gwy_data_field_get_data(field);
+    xres = gwy_field_get_xres(field);
+    yres = gwy_field_get_yres(field);
+    xoff = gwy_field_get_xoffset(field);
+    yoff = gwy_field_get_yoffset(field);
+    qx = gwy_field_get_xreal(field)/xres;
+    qy = gwy_field_get_yreal(field)/yres;
+    d = gwy_field_get_data(field);
 
     p = 0.5*power;
     if (fabs(power - GWY_ROUND(power)) < EPSREL)
@@ -850,7 +829,7 @@ extend_borders(ModuleArgs *args,
     }
     g_array_set_size(rdata->points, nbase);
 
-    if (exterior == GWY_EXTERIOR_BORDER_EXTEND) {
+    if (exterior == GWY_EXTERIOR_BORDER) {
         gwy_debug("exterior is BORDER, just reducing points to base");
         g_free(oldextpoints);
         return noldext > 0 || !check_for_changes;
@@ -876,7 +855,7 @@ extend_borders(ModuleArgs *args,
         gboolean txlok, txrok, tytok, tybok;
 
         pt2.z = pt.z;
-        if (exterior == GWY_EXTERIOR_MIRROR_EXTEND) {
+        if (exterior == GWY_EXTERIOR_MIRROR) {
             txl = 2.0*sxmin - pt.x;
             tyt = 2.0*symin - pt.y;
             txr = 2.0*sxmax - pt.x;
@@ -982,7 +961,7 @@ static void
 round_to_nice(gdouble *minval, gdouble *maxval)
 {
     gdouble range = *maxval - *minval;
-    gdouble base = pow10(floor(log10(range) - 2.0));
+    gdouble base = gwy_exp10(floor(log10(range) - 2.0));
 
     *minval = round_with_base(*minval, base);
     *maxval = round_with_base(*maxval, base);
@@ -1002,15 +981,15 @@ initialise_range(ModuleArgs *args)
 }
 
 static void
-set_range_from_field(GwyDataField *field, XYZRasRange *range)
+set_range_from_field(GwyField *field, XYZRasRange *range)
 {
-    gdouble xoff = gwy_data_field_get_xoffset(field);
-    gdouble yoff = gwy_data_field_get_yoffset(field);
+    gdouble xoff = gwy_field_get_xoffset(field);
+    gdouble yoff = gwy_field_get_yoffset(field);
 
     range->min.x = xoff;
-    range->max.x = gwy_data_field_get_xreal(field) + xoff;
+    range->max.x = gwy_field_get_xreal(field) + xoff;
     range->min.y = yoff;
-    range->max.y = gwy_data_field_get_yreal(field) + yoff;
+    range->max.y = gwy_field_get_yreal(field) + yoff;
 }
 
 static inline guint
@@ -1119,7 +1098,7 @@ analyse_points(ModuleArgs *args, double epsrel)
 
     /* Calculate data ranges */
     npoints = rdata->norigpoints = surface->n;
-    points = surface->data;
+    points = gwy_surface_get_data_const(surface);
     gwy_surface_get_xrange(surface, &xmin, &xmax);
     gwy_surface_get_yrange(surface, &ymin, &ymax);
 
@@ -1264,28 +1243,29 @@ analyse_points(ModuleArgs *args, double epsrel)
 }
 
 /* Create a data field directly if the XY positions form a complete regular grid.  */
-static GwyDataField*
+static GwyField*
 check_regular_grid(GwySurface *surface)
 {
     GwyXY xymin, dxy;
     guint n, xres, yres, k;
-    GwyDataField *field;
+    GwyField *field;
     gdouble *data;
     guint *map;
 
     n = surface->n;
-    if (!(map = gwy_check_regular_2d_grid((const gdouble*)surface->data, 3, n, -1.0, &xres, &yres, &xymin, &dxy)))
+    const GwyXYZ *xyz = gwy_surface_get_data_const(surface);
+    if (!(map = gwy_check_regular_2d_grid((const gdouble*)xyz, 3, n, -1.0, &xres, &yres, &xymin, &dxy)))
         return NULL;
 
-    field = gwy_data_field_new(xres, yres, xres*dxy.x, yres*dxy.y, FALSE);
-    data = gwy_data_field_get_data(field);
+    field = gwy_field_new(xres, yres, xres*dxy.x, yres*dxy.y, FALSE);
+    data = gwy_field_get_data(field);
     for (k = 0; k < n; k++)
-        data[k] = surface->data[map[k]].z;
+        data[k] = xyz[map[k]].z;
     g_free(map);
 
-    gwy_data_field_set_xoffset(field, xymin.x);
-    gwy_data_field_set_yoffset(field, xymin.y);
-    gwy_surface_copy_units_to_data_field(surface, field);
+    gwy_field_set_xoffset(field, xymin.x);
+    gwy_field_set_yoffset(field, xymin.y);
+    gwy_surface_copy_units_to_field(surface, field);
     return field;
 }
 
