@@ -1,5 +1,5 @@
 /*
- *  $Id: xyz_reduce.c 26355 2024-05-21 08:23:55Z yeti-dn $
+ *  $Id: xyz_reduce.c 28808 2025-11-05 18:26:20Z yeti-dn $
  *  Copyright (C) 2016-2023 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -15,14 +15,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/surface.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-xyz.h>
-#include <app/gwyapp.h>
+#include <gwy.h>
 
 #define RUN_MODES (GWY_RUN_INTERACTIVE | GWY_RUN_IMMEDIATE)
 
@@ -44,10 +40,10 @@ typedef struct {
 
 static gboolean         module_register     (void);
 static GwyParamDef*     define_module_params(void);
+static void             module_main         (GwyFile *data,
+                                             GwyRunModeFlags mode);
 static void             execute             (ModuleArgs *args);
 static GwyDialogOutcome run_gui             (ModuleArgs *args);
-static void             xyzreduce           (GwyContainer *data,
-                                             GwyRunType runtype);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
@@ -65,7 +61,7 @@ static gboolean
 module_register(void)
 {
     gwy_xyz_func_register("xyz_reduce",
-                          (GwyXYZFunc)&xyzreduce,
+                          module_main,
                           N_("/_Reduce Number of Points..."),
                           NULL,
                           RUN_MODES,
@@ -98,22 +94,21 @@ define_module_params(void)
 }
 
 static void
-xyzreduce(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     ModuleArgs args;
     gint id, newid;
-    const guchar *gradient;
 
-    g_return_if_fail(runtype & RUN_MODES);
+    g_return_if_fail(mode & RUN_MODES);
 
-    gwy_app_data_browser_get_current(GWY_APP_SURFACE, &args.surface,
-                                     GWY_APP_SURFACE_ID, &id,
-                                     0);
+    gwy_data_browser_get_current(GWY_APP_SURFACE, &args.surface,
+                                 GWY_APP_SURFACE_ID, &id,
+                                 0);
     g_return_if_fail(GWY_IS_SURFACE(args.surface));
 
     args.params = gwy_params_new_from_settings(define_module_params());
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -123,12 +118,13 @@ xyzreduce(GwyContainer *data, GwyRunType runtype)
     if (outcome == GWY_DIALOG_PROCEED)
         execute(&args);
 
-    newid = gwy_app_data_browser_add_surface(args.result, data, TRUE);
-    gwy_app_set_surface_title(data, newid, _("Reduced points"));
-    if (gwy_container_gis_string(data, gwy_app_get_surface_palette_key_for_id(id), &gradient))
-        gwy_container_set_const_string(data, gwy_app_get_surface_palette_key_for_id(newid), gradient);
+    newid = gwy_file_add_xyz(data, args.result);
 
-    gwy_app_xyz_log_add(data, -1, newid, "xyz::xyz_reduce", NULL);
+    gwy_file_set_visible(data, GWY_FILE_XYZ, newid, TRUE);
+    gwy_file_set_title(data, GWY_FILE_XYZ, newid, _("Reduced points"), TRUE);
+    gwy_file_sync_items(data, GWY_FILE_XYZ, id,
+                        data, GWY_FILE_XYZ, newid,
+                        GWY_FILE_ITEM_PALETTE, FALSE);
 
     g_object_unref(args.result);
 
