@@ -1,5 +1,5 @@
 /*
- *  $Id: xyz_zdrift.c 28893 2025-11-20 12:09:25Z klapetek $
+ *  $Id: xyz_zdrift.c 29061 2026-01-02 15:01:53Z yeti-dn $
  *  Copyright (C) 2016-2023 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -15,16 +15,10 @@
  */
 
 #include "config.h"
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgwyddion/gwymacros.h>
-#include <libgwyddion/gwymath.h>
-#include <libprocess/surface.h>
-#include <libprocess/gwyprocess.h>
-#include <libgwydgets/gwystock.h>
-#include <libgwymodule/gwymodule-xyz.h>
-#include <app/gwyapp.h>
-#include <app/gwymoduleutils.h>
+#include <gwy.h>
 
 #define RUN_MODES (GWY_RUN_INTERACTIVE)
 
@@ -64,33 +58,32 @@ typedef struct {
     ModuleArgs *args;
     GtkWidget *dialog;
     GwyParamTable *table_options;
-    GwyContainer *data;
     GwyGraphModel *gmodel;
     GwySelection *graph_selection;
-    GwyDataField *dfield;
+    GwyField *dfield;
     gboolean drift_done;
     gint ndiv;
  } ModuleGUI;
 
-static gboolean         module_register         (void);
-static GwyParamDef*     define_module_params    (void);
-static void             execute                 (ModuleArgs *args);
-static GwyDialogOutcome run_gui                 (ModuleArgs *args,
-                                                 GwyContainer *data,
-                                                 gint id);
-static void             graph_selection_changed (ModuleGUI *gui,
-                                                 gint id,
-                                                 GwySelection *selection);
-static void             dialog_response         (GwyDialog *dialog,
-                                                 gint response,
-                                                 ModuleGUI *gui);
-static void             param_changed           (ModuleGUI *gui,
-                                                 gint id);
-static void             update_drift_curve      (ModuleGUI *gui);
-static void             update_fit_curve        (ModuleGUI *gui);
-static void             xyzzdrift               (GwyContainer *data,
-                                                 GwyRunType runtype);
-static void             preview                 (gpointer user_data);
+static gboolean         module_register        (void);
+static GwyParamDef*     define_module_params   (void);
+static void             execute                (ModuleArgs *args);
+static GwyDialogOutcome run_gui                (ModuleArgs *args,
+                                                GwyFile *data,
+                                                gint id);
+static void             graph_selection_changed(ModuleGUI *gui,
+                                                gint id,
+                                                GwySelection *selection);
+static void             dialog_response        (GwyDialog *dialog,
+                                                gint response,
+                                                ModuleGUI *gui);
+static void             param_changed          (ModuleGUI *gui,
+                                                gint id);
+static void             update_drift_curve     (ModuleGUI *gui);
+static void             update_fit_curve       (ModuleGUI *gui);
+static void             module_main            (GwyFile *data,
+                                                GwyRunModeFlags mode);
+static void             preview                (gpointer user_data);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
@@ -108,7 +101,7 @@ static gboolean
 module_register(void)
 {
     gwy_xyz_func_register("xyz_zdrift",
-                          (GwyXYZFunc)&xyzzdrift,
+                          module_main,
                           N_("/Correct Z _Drift..."),
                           NULL,
                           RUN_MODES,
@@ -143,18 +136,17 @@ define_module_params(void)
 }
 
 static void
-xyzzdrift(GwyContainer *data, GwyRunType runtype)
+module_main(GwyFile *data, GwyRunModeFlags mode)
 {
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     ModuleArgs args;
     gint id, newid, k;
-    const guchar *gradient;
 
-    g_return_if_fail(runtype & RUN_MODES);
+    g_return_if_fail(mode & RUN_MODES);
 
-    gwy_app_data_browser_get_current(GWY_APP_SURFACE, &args.surface,
-                                     GWY_APP_SURFACE_ID, &id,
-                                     0);
+    gwy_data_browser_get_current(GWY_APP_SURFACE, &args.surface,
+                                 GWY_APP_SURFACE_ID, &id,
+                                 0);
     g_return_if_fail(GWY_IS_SURFACE(args.surface));
 
     args.params = gwy_params_new_from_settings(define_module_params());
@@ -166,7 +158,7 @@ xyzzdrift(GwyContainer *data, GwyRunType runtype)
     for (k = 0; k <= MAXORDER; k++)
         args.fitcoeffs[k] = 0;
 
-    if (runtype == GWY_RUN_INTERACTIVE) {
+    if (mode ==GWY_RUN_INTERACTIVE) {
         outcome = run_gui(&args, data, id);
         gwy_params_save_to_settings(args.params);
         if (outcome == GWY_DIALOG_CANCEL)
@@ -176,12 +168,13 @@ xyzzdrift(GwyContainer *data, GwyRunType runtype)
     if (outcome == GWY_DIALOG_PROCEED && !args.result)
         execute(&args);
 
-    newid = gwy_app_data_browser_add_surface(args.result, data, TRUE);
-    gwy_app_set_surface_title(data, newid, _("Z drift corrected"));
-    if (gwy_container_gis_string(data, gwy_app_get_surface_palette_key_for_id(id), &gradient))
-        gwy_container_set_const_string(data, gwy_app_get_surface_palette_key_for_id(newid), gradient);
+    newid = gwy_file_add_xyz(data, args.result);
 
-    gwy_app_xyz_log_add(data, -1, newid, "xyz::xyz_zdrift", NULL);
+    gwy_file_set_visible(data, GWY_FILE_XYZ, newid, TRUE);
+    gwy_file_set_title(data, GWY_FILE_XYZ, newid, _("Z drift corrected"), TRUE);
+    gwy_file_sync_items(data, GWY_FILE_XYZ, id,
+                        data, GWY_FILE_XYZ, newid,
+                        GWY_FILE_ITEM_PALETTE, FALSE);
 
     g_object_unref(args.result);
 
@@ -190,14 +183,13 @@ end:
 }
 
 static GwyDialogOutcome
-run_gui(ModuleArgs *args, GwyContainer *data, gint id)
+run_gui(ModuleArgs *args, GwyFile *data, gint id)
 {
     GwyDialog *dialog;
     ModuleGUI gui;
     GwyParamTable *table;
     GtkWidget *graph, *area, *hbox, *dataview;
     GwyGraphCurveModel *gcmodel;
-    const guchar *gradient;
     GwyDialogOutcome outcome;
 
     gui.dialog = gwy_dialog_new(_("Correct z drift"));
@@ -210,16 +202,12 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 
     gui.args = args;
     gui.drift_done = FALSE;
-    gui.data = gwy_container_new();
-    gui.dfield = gwy_data_field_new(10, 10, 10, 10, FALSE);
+    gui.dfield = gwy_field_new(10, 10, 10, 10, FALSE);
 
-    gwy_preview_surface_to_datafield(args->surface, gui.dfield, PREVIEW_SIZE, PREVIEW_SIZE, GWY_PREVIEW_SURFACE_FILL);
-    gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(0), gui.dfield);
+    gwy_preview_surface_to_field(args->surface, gui.dfield, PREVIEW_SIZE, PREVIEW_SIZE, GWY_PREVIEW_SURFACE_FILL);
 
-    if (gwy_container_gis_string(data, gwy_app_get_surface_palette_key_for_id(id), &gradient))
-       gwy_container_set_const_string(gui.data, gwy_app_get_data_palette_key_for_id(0), gradient);
-
-    dataview = gwy_create_preview(gui.data, 0, PREVIEW_SIZE, FALSE);
+    dataview = gwy_create_preview(gui.dfield, NULL, PREVIEW_SIZE);
+    gwy_setup_data_view(GWY_DATA_VIEW(dataview), data, GWY_FILE_XYZ, id, GWY_FILE_ITEM_PALETTE);
     hbox = gwy_create_dialog_preview_hbox(GTK_DIALOG(dialog), GWY_DATA_VIEW(dataview), FALSE);
 
     gui.gmodel = gwy_graph_model_new();
@@ -260,7 +248,6 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     outcome = gwy_dialog_run(dialog);
 
     g_object_unref(gui.dfield);
-    g_object_unref(gui.data);
 
     return outcome;
 }
@@ -270,9 +257,9 @@ dialog_response(G_GNUC_UNUSED GwyDialog *dialog, gint response, ModuleGUI *gui)
 {
     if (response == GWY_RESPONSE_RESET) {
         gwy_selection_clear(gui->graph_selection);
-        gwy_preview_surface_to_datafield(gui->args->surface, gui->dfield, PREVIEW_SIZE, PREVIEW_SIZE,
-                                         GWY_PREVIEW_SURFACE_FILL);
-        gwy_data_field_data_changed(gui->dfield);
+        gwy_preview_surface_to_field(gui->args->surface, gui->dfield, PREVIEW_SIZE, PREVIEW_SIZE,
+                                     GWY_PREVIEW_SURFACE_FILL);
+        gwy_field_data_changed(gui->dfield);
     }
     else if (response == RESPONSE_DRIFT) {
         update_drift_curve(gui);
@@ -280,10 +267,10 @@ dialog_response(G_GNUC_UNUSED GwyDialog *dialog, gint response, ModuleGUI *gui)
     }
     else if (response == RESPONSE_APPLY) {
         execute(gui->args);
-        gwy_preview_surface_to_datafield(gui->args->result, gui->dfield, PREVIEW_SIZE, PREVIEW_SIZE,
-                                         GWY_PREVIEW_SURFACE_FILL);
+        gwy_preview_surface_to_field(gui->args->result, gui->dfield, PREVIEW_SIZE, PREVIEW_SIZE,
+                                     GWY_PREVIEW_SURFACE_FILL);
 
-        gwy_data_field_data_changed(gui->dfield);
+        gwy_field_data_changed(gui->dfield);
         gwy_dialog_have_result(GWY_DIALOG(gui->dialog));
     }
 }
@@ -350,7 +337,7 @@ fit_polynom(gint degree, const gdouble *z, const gdouble *drift, gdouble *fit,
     gdouble *coeffs;
 
     //printf("from %d\n", from);
-    coeffs = gwy_math_fit_polynom(fitn, z + from, drift + from, degree, NULL);
+    coeffs = gwy_math_fit_poly(z + from, drift + from, fitn, degree, NULL);
     if (coeffs) {
         for (k = 0; k < n; k++)
             fit[k] = polynom(degree, coeffs, z[k]);
@@ -370,8 +357,8 @@ update_drift_curve(ModuleGUI *gui)
 {
     GwyGraphCurveModel *gcmodel = gwy_graph_model_get_curve(gui->gmodel, 0);
     GwySurface *surface = gui->args->surface;
-    GwyDataField *sfield, *mfield;
-    GwyDataLine *avg, *weights;
+    GwyField *sfield, *mfield;
+    GwyLine *avg, *weights;
     const GwyXYZ *xyz;
     gdouble *linedata, *xlinedata, *sdata, *mdata, *adata;
     gint k, n, m, l, linen, foundn;
@@ -414,13 +401,13 @@ update_drift_curve(ModuleGUI *gui)
     }
     else {
         printf("linen %d MAXN %d\n", linen, MAXN);
-        sfield = gwy_data_field_new(linen, MAXN, linen, MAXN, TRUE);
-        mfield = gwy_data_field_new_alike(sfield, TRUE);
-        avg = gwy_data_line_new(linen, linen, TRUE);
-        weights = gwy_data_line_new(linen, linen, TRUE);
+        sfield = gwy_field_new(linen, MAXN, linen, MAXN, TRUE);
+        mfield = gwy_field_new_alike(sfield, TRUE);
+        avg = gwy_line_new(linen, linen, TRUE);
+        weights = gwy_line_new(linen, linen, TRUE);
 
-        sdata = gwy_data_field_get_data(sfield);
-        mdata = gwy_data_field_get_data(mfield);
+        sdata = gwy_field_get_data(sfield);
+        mdata = gwy_field_get_data(mfield);
 
         foundn = 0;
         rng = g_rand_new();
@@ -454,12 +441,11 @@ update_drift_curve(ModuleGUI *gui)
         gwy_app_wait_finish();
 
         if (foundn > 0) {
-            gwy_data_field_get_line_stats_mask(sfield, mfield, GWY_MASK_INCLUDE,
-                                               avg, weights, 0, 0, linen, foundn,
-                                               GWY_LINE_STAT_MEAN,
-                                              GWY_ORIENTATION_VERTICAL);
-            adata = gwy_data_line_get_data(avg);
-            //wdata = gwy_data_line_get_data(weights);
+            gwy_field_area_get_line_stats(sfield, mfield, GWY_MASK_INCLUDE,
+                                          avg, weights, 0, 0, linen, foundn,
+                                          GWY_LINE_STAT_MEAN, GWY_ORIENTATION_VERTICAL);
+            adata = gwy_line_get_data(avg);
+            //wdata = gwy_line_get_data(weights);
 
             xlinedata[0] = 0;
             linedata[0] = adata[0];
