@@ -1,5 +1,5 @@
 /*
- *  $Id: dump-modules.c 21352 2018-08-27 20:05:16Z yeti-dn $
+ *  $Id: dump-modules.c 27960 2025-05-10 13:07:23Z yeti-dn $
  *  Copyright (C) 2003,2004 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -20,15 +20,13 @@
  */
 
 #include "config.h"
-#include <libgwyddion/gwymacros.h>
-#include <libgwymodule/gwymodule.h>
-#include <libgwyddion/gwyutils.h>
-#include <libgwyddion/gwyddion.h>
-#include <app/gwyapp.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <locale.h>
+#include <glib/gi18n.h>
+#include <libgwyddion/gwyddion.h>
+#include <libgwyapp/gwyapp.h>
 
 typedef struct {
     const gchar *name;
@@ -122,7 +120,7 @@ menu_path_print(const gchar *path, const gchar *tag)
     gchar *s;
     gint i;
 
-    path = gettext(path);
+    path = _(path);
     s = gwy_strkill(gwy_strreplace(path + (path[0] == '/'),
                                    "/", " → ", -1), "_");
     i = strlen(s);
@@ -200,22 +198,23 @@ main(int argc,
     };
     /* modules we know we don't want to see functions of (XXX: hack) */
     const gchar *no_func_modules[] = {
-        "plugin-proxy", "pygwy",
+        "pygwy",
     };
 
     gchar **module_dirs;
     GSList *m, *modules = NULL;
     gsize i;
 
-    gwy_type_init();
+    gwy_init();
     if (argc < 3) {
         g_printerr("Usage: dump-modules USER-GUIDE-MAP LANGUAGE.\n");
         return 1;
     }
+    gwy_init();
     g_setenv("LC_MESSAGES", argv[2], TRUE);
     g_setenv("LANGUAGE", argv[2], TRUE);
     setlocale(LC_MESSAGES, argv[2]);
-    gwy_app_init_i18n();
+    textdomain(GETTEXT_PACKAGE);
     read_user_guide(argv[1]);
 
     module_dirs = gwy_app_settings_get_module_dirs();
@@ -252,7 +251,7 @@ main(int argc,
         }
         else
             fprintf(stderr, "No user guide entry for %s\n", info->name);
-        tag_print("description", gettext(info->info->blurb));
+        tag_print("description", _(info->info->blurb));
         for (i = 0; i < G_N_ELEMENTS(no_func_modules); i++) {
             if (gwy_strequal(info->name, no_func_modules[i]))
                 break;
