@@ -1,4 +1,4 @@
-#!/usr/bin/python2
+#!/usr/bin/python
 # vim: fileencoding=utf-8 sw=4 ts=4 et :
 # Written by Yeti <yeti@gwyddion.net>. Public domain.
 import sys, os, re
@@ -113,8 +113,8 @@ except KeyError:
     sys.stderr.write('Unknown magic class %s\n' % sys.argv[1])
     sys.exit(1)
 except IndexError:
-    print 'Usage: %s CLASS [FILES...]' % sys.argv[0]
-    print 'Available classes: %s' % ' '.join(mclasses.keys())
+    sys.stderr.write('Usage: %s CLASS [FILES...]\n' % sys.argv[0])
+    sys.stderr.write('Available classes: %s\n' % ' '.join(mclasses.keys()))
     sys.exit(0)
 
 magic_block_re = re.compile(r'(?m)^/\*\*\n \* '
@@ -122,7 +122,7 @@ magic_block_re = re.compile(r'(?m)^/\*\*\n \* '
                             + r'(?P<body>(?: \* .*\n)+) \*\*/$')
 output = []
 
-if magic.has_key('header'):
+if 'header' in magic:
     output.append(magic['header'])
 
 # Keep the `GENERATED' string split to prevent match here
@@ -133,7 +133,7 @@ for filename in sys.argv[2:]:
     try:
         # output From module... comment only if something is found
         comment = magic['comment'] % ('From module ' + base)
-        for m in magic_block_re.finditer(file(filename).read()):
+        for m in magic_block_re.finditer(open(filename).read()):
             if comment:
                 output.append(comment)
                 comment = None
@@ -149,7 +149,7 @@ for filename in sys.argv[2:]:
         sys.stderr.write('Cannot read %s\n' % filename)
         sys.exit(1)
 
-if magic.has_key('footer'):
+if 'footer' in magic:
     output.append(magic['footer'])
 
-print '\n'.join(output)
+sys.stdout.write('\n'.join(output) + '\n')
