@@ -5,7 +5,7 @@ exec >'po/POTFILES.in'
 echo '# List of source files containing translatable strings.'
 echo '# This is a 'GENERATED' file, by utils/update-potfiles.sh.'
 
-for dir in libgwyddion libprocess libdraw libgwydgets libgwymodule app gwyddion modules; do
+for dir in libgwyddion libgwyui libgwyapp gwyddion modules; do
   echo
   echo "# $dir"
   find $dir -name '*.[ch]' -o -name '*.cc' \
